///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2008
// 
// Filename: XGS_PIC_NTSC_SNAKE_V010.c
//
// Original Author: Joshua Hintze
// 
// Last Modified: 9.23.08
//
// Description: A simple snake game that makes use of V-line, H-Line, multiple palettes, and gamepad use.
//				Also a little bit of sound for fun. Make sure to check out the death animations and the game
//              winning animation (if you are good).
//
// 				Gamepad controls are simple: D-Pad (Up, Down, Left, Right) to change direction of the snake.
//
//				Snake pieces and targets are 3 pixels x 4 pixels. The snake moves by one whole piece thus it
//              moves 3 pixels in the X direction and 4 pixels in the Y direction. So the board size is actually
//              25x25 different positions the snake and target may be in. 
//
//				A snake can collide against itself, wall, or a target. Each time a target is hit the snake moves 
//              a little bit faster, until the snake is 20 pieces long then the user wins.
//
//
// Required Files: XGS_PIC_SYSTEM_V010.c/h
//				   XGS_PIC_GFX_DRV_V010.c/h
//				   XGS_PIC_NTSC_160_192_2_V010.s/h
//				   XGS_PIC_GAMEPAD_DRV_V010.c/h
//				   XGS_PIC_SOUND_DRV_V010.c/h
//				   XGS_PIC_TECH_FONT_6x8_V010.h
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

// Include main system header
#include "XGS_PIC_SYSTEM_V010.h"			

// Include our graphics definition file
#include "XGS_PIC_NTSC_160_192_8_V010.h"
#include "XGS_PIC_GFX_DRV_V010.h"

// Gamepad and sound support drivers
#include "XGS_PIC_GAMEPAD_DRV_V010.h"
#include "XGS_PIC_SOUND_DRV_V010.h"

// Include our fonts and make a crude bitmap font engine
//#include "XGS_PIC_TECH_FONT_6x8_V010.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Directions of travel
#define DIR_LEFT 	0
#define DIR_RIGHT 	1
#define DIR_UP 		2
#define DIR_DOWN 	3

#define RAND_RANGE(x,y) ( ( x ) + (rand()%( ( y ) - ( x )+1)))

// Board dimensions
#define BOARD_LEFT		0
#define BOARD_RIGHT		160
#define BOARD_TOP		0
#define BOARD_BOTTOM 	192

// Game pause ability
//#define ALLOW_GAME_PAUSE

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// STRUCTS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
struct Point
{
	int x;
	int y;
};


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// The following are required for using the graphics driver in bitmap mode
// The VRAM buffer must be allocated into far memory space because it is so large it wont fit into near
unsigned char 	g_VRAMBuffer[VRAM_BUF_SIZE] __attribute__((far));	
unsigned short 	g_PaletteMap[PALETTE_MAP_SIZE];			// Table that contains which color table to assign 8x8 group
unsigned char	g_Palettes[MIN_PALETTE_SIZE*7];			// Our Color Table

//int g_HardImagePtr ;
//int	g_HardImagePage ;

// Game variables
int SnakeSize = 1;
struct Point SnakePiece[20], Target;
int ToneCounter = 0;
int Death = 0, Win = 0;
int DeathCounter = 0;
int WinCounter = 0;
int WinState = 0;
int Gamespeed = 8;
int Direction = DIR_DOWN;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS  
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Clears the screen and draws the game board
void DrawBlankGameBoard();

// Draws the snake with color
void DrawSnake(int Clear);

// Moves the snake by 1 piece
void MoveSnake(int x, int y);

// Draws our target
void DrawTarget(int Clear);

// Checks for collisions against wall, self, target
void CheckCollision();

// Draws our death animation and sounds
void DeathAnimation();

// Draws our win animation and sounds
void WinAnimation();

// Draws a single bitmap font character
void DrawBitmapFontChar(char ch, int x, int y);

// Draws a simple two digit score
void DrawScore();

// Our main start location
int main (void)
{
	int VsyncCounter = 0;
	
	// Always call ConfigureClock first
	SYS_ConfigureClock(FCY_NTSC);

	// Init the graphics system
	//GFX_InitBitmap(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, g_VRAMBuffer, g_Palettes, g_PaletteMap,g_HardImagePtr,g_HardImagePage);

	// Init our gamepad  - we will use controller 1 only
	Gamepad_Init();

	// Init our sound driver so we can use simple tones
	SND_Init();

	// Set our clipping regions
//	GFX_SetClipRegion(0, SCREEN_HEIGHT-1, 0, SCREEN_WIDTH-1);

	// Fill the palette map with 0, this way all 8x8 pixel sections use the first color table
//	memset(g_PaletteMap, 0, sizeof(g_PaletteMap));

	// Fill the entire palette with black and then we'll manually setup some colors
//	memset(g_Palettes, NTSC_BLACK, sizeof(g_Palettes));

   	//g_ImagePtr=0;
	//HardImagePtr=0;


/*
	// Palette 0
	g_Palettes[0]  = NTSC_BLACK;
    g_Palettes[1]  = NTSC_RED;
	g_Palettes[2]  = NTSC_YELLOW;
	g_Palettes[3]  = NTSC_GRAY7;

	// Palette 1
	g_Palettes[4]  = NTSC_BLACK;
    g_Palettes[5]  = NTSC_ORANGE;
	g_Palettes[6]  = NTSC_BLUE;
	g_Palettes[7]  = NTSC_GREEN;

	// Palette 2
	g_Palettes[8]  = NTSC_BLACK;
	g_Palettes[9]  = NTSC_WHITE;
	g_Palettes[10] = NTSC_GRAY7;
	g_Palettes[11] = NTSC_BLUE;

	// Palette 3
	g_Palettes[12] = NTSC_BLACK;
	g_Palettes[13] = NTSC_VIOLET;
	g_Palettes[14] = NTSC_PURPLE;
	g_Palettes[15] = NTSC_RED;

	// Palette 4
	g_Palettes[16] = NTSC_BLACK;
	g_Palettes[17] = NTSC_BLUE;
	g_Palettes[18] = NTSC_RED;
	g_Palettes[19] = NTSC_ORANGE;
*/


	// Palette 0
	g_Palettes[0]  = NTSC_BLACK;
    //g_Palettes[0]  = 0x90;
    g_Palettes[1]  = 0x90;
	g_Palettes[2]  = 0x91;
	g_Palettes[3]  = 0x92;

	// Palette 1
	g_Palettes[4]  = NTSC_BLACK;
    g_Palettes[5]  = 0x93;
	g_Palettes[6]  = 0x94;
	g_Palettes[7]  = 0x95;

	// Palette 2
	g_Palettes[8]  = NTSC_BLACK;
	g_Palettes[9]  = 0x96;
	g_Palettes[10] = 0x97;
	g_Palettes[11] = 0x98;

	// Palette 3
	g_Palettes[12] = NTSC_BLACK;
	g_Palettes[13] = 0x99;
	g_Palettes[14] = 0x9A;
	g_Palettes[15] = 0x9B;

	// Palette 4
	g_Palettes[16] = NTSC_BLACK;
	g_Palettes[17] = 0x9C;
	g_Palettes[18] = 0x9D;
	g_Palettes[19] = 0x9E;

	// Palette 5
	g_Palettes[20] = NTSC_BLACK;
	g_Palettes[21] = 0x9C;
	g_Palettes[22] = 0x9D;
	g_Palettes[23] = 0x9E;

	// Palette 6
	g_Palettes[24] = NTSC_BLACK;
	g_Palettes[25] = 0x9C;
	g_Palettes[26] = 0x9D;
	g_Palettes[27] = 0x9E;


	// Start the NTSC timer so we start drawing to the screen
	GFX_StartDrawing(SCREEN_TYPE_NTSC);

	while(1)
	{
		// Wait until we are in v-sync to do any processing
		WAIT_FOR_VSYNC_START();	

/*
		// Before doing any gameplay see if we are dead or if we won
		if(Death)
		{
			DeathAnimation();		
			continue;
		}

		if(Win)
		{
			WinAnimation();
			continue;
		}
*/

		// Get the direction on every frame
		unsigned char GamepadVal = Gamepad_Read(GAMEPAD_0);
		if(GamepadVal & GAMEPAD_RIGHT)
		{
			if(Direction != DIR_LEFT)
				Direction = DIR_RIGHT;
		}
		else if(GamepadVal & GAMEPAD_LEFT)
		{
			if(Direction != DIR_RIGHT)
				Direction = DIR_LEFT;
		}
		else if(GamepadVal & GAMEPAD_UP)
		{
			if(Direction != DIR_DOWN)
				Direction = DIR_UP;
		}
		else if(GamepadVal & GAMEPAD_DOWN)
		{
			if(Direction != DIR_UP)
				Direction = DIR_DOWN;
		}
#ifdef ALLOW_GAME_PAUSE
		else if(GamepadVal & GAMEPAD_ACTION1)
		{
			WAIT_FOR_VSYNC_END();
			continue;
		}
#endif

		
		// Wait here till we are done with v-sync
		WAIT_FOR_VSYNC_END();
	}
	return 0;
}
