///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2008
// 
// Filename: Centipede.c
//
// Original Author: Joshua Hintze
// 
// Last Modified: 3.28.08
//
// Description: Main source file for the centipede game.
//				
//				Controls:
//					Gamepad for directional movements. Action 2 for fire.
//
//
// Required Files: XGS_PIC_SYSTEM_V010.c/h
//				   XGS_PIC_GFX_DRV_V010.c/h
//				   XGS_PIC_NTSC_TILE1_V010.s/h
//				   XGS_PIC_GAMEPAD_DRV_V010.h
//
//
//
// Version 1:  Complete Working Game, but missing the scorpion and poisioned mushrooms
// Version 2:  Added the scorpion and poisioned mushrooms
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// Include main system header
#include "XGS_PIC_SYSTEM_V010.h"

// Include graphics drivers
#include "XGS_PIC_GFX_DRV_V010.h"
#include "XGS_PIC_NTSC_TILE1_V010.h"

// Include the gamepad driver
#include "XGS_PIC_GAMEPAD_DRV_V010.h"

// Include the sound driver
#include "XGS_PIC_SOUND_DRV_V010.h"

// Include sprite and tile data
#include "cent_tiles_v001.h"
#include "cent_sprites_v001.h"

// Include our hand created sounds
#include "cent_sounds_v001.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


// Defines for Game States
#define CENT_STATE_INIT_TITLE	0
#define CENT_STATE_TITLE		1
#define CENT_STATE_PLAY			2
#define CENT_STATE_NEXT_LEVEL	3
#define CENT_STATE_DYING		4
#define CENT_STATE_DEAD			5
#define CENT_STATE_GAME_OVER	6
#define CENT_STATE_ENTER_INT	7

// Generates a random number in range x->y inclusive
#define RAND_RANGE(x,y) ( ( x ) + (rand()%( ( y ) - ( x )+1)))

#define NUMBER_MUSHROOMS_LEVEL	30

// Tweaking factor for when a flea spawns
#define MIN_FLEA_PROB	250

// Spider movement states
#define SPIDER_STATE_UP				0
#define SPIDER_STATE_DOWN			1
#define SPIDER_STATE_LATERAL_UP		2
#define SPIDER_STATE_LATERAL_DOWN	3
#define SPIDER_STATE_CHASE			4

// Spider direction
#define SPIDER_DIR_LEFT				0
#define SPIDER_DIR_RIGHT			1

#define MAX_NUMBER_CENTIPEDES		10

// Centipede piece information
typedef struct
{
	int XTile;			// X tile position
	int YTile;			// Y tile position (negative value is off screen and not drawn)
	int XDir;			// X direction (-1 left, +1 right)
	int YDir;			// Y direction (-1 up, +1 down)
	int AnimationFrame;	// Animation frame from the offset normal
	int AnimationCtr;	// Counter for when to switch to different animation frame
	int IsHead;			// Flag for head piece
	int IsDead;			// Has already been started and shot in this round/life
	int IsPoisoned;		// In the posioned fallling state
} CentiPart;


// Sound 
typedef struct
{
	int PlayPosition;
	int CurrentNoteDuration;

	const unsigned int *Notes;
	const unsigned int *NotesDuration;
	int NumberNotes;
} Sound;

// Overrides from the standard sounds
#define SOUND_OVERRIDE_NONE			0
#define SOUND_OVERRIDE_BLASTER		1
#define SOUND_OVERRIDE_MUSHROOM		2
#define SOUND_OVERRIDE_EXPLOSION 	3

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// The TILE map buffer must be allocated into far memory space because it is so large it wont fit into near
unsigned char g_TileMap[SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT] __attribute__((far)); // 1 screens worth
unsigned char g_Palettes[MAX_PALETTE_SIZE]; 	// Different colors
Sprite g_Sprites[MAX_SPRITES];					// Max sprites

// Player sprite
Sprite *g_Player;								

// Missile sprite
Sprite *g_Missile;

// Flea sprite
Sprite *g_Flea;

// Spider sprite
Sprite *g_SpiderLeft;
Sprite *g_SpiderRight;

// Our hand created sounds
Sound g_BlasterSound = { 0, 0, BlasterNotes, BlasterNotesDur, sizeof(BlasterNotes)/sizeof(BlasterNotes[0]) };
Sound g_ExplosionSound = { 0,0, ExplosionNotes, ExplosionNotesDur, sizeof(ExplosionNotes)/sizeof(ExplosionNotes[0]) };
Sound g_MushroomSound = { 0,0, MushroomGrowNotes, MushroomGrowNotesDur, sizeof(MushroomGrowNotes)/sizeof(MushroomGrowNotes[0]) };
Sound g_SpiderSound = { 0,0, SpiderNotes, SpiderNotesDur, sizeof(SpiderNotes)/sizeof(SpiderNotes[0]) };
Sound g_GameSound = { 0,0, GameNotes, GameNotesDur, sizeof(GameNotes)/sizeof(GameNotes[0]) };
Sound g_FleaSound = { 0,0, FleaNotes, FleaNotesDur, sizeof(FleaNotes)/sizeof(FleaNotes[0]) };
Sound g_ScorpionSound = { 0,0, ScorpionNotes, ScorpionNotesDur, sizeof(ScorpionNotes)/sizeof(ScorpionNotes[0]) };

int g_SoundOverride = SOUND_OVERRIDE_NONE;
Sound *g_CurrentSound = 0;

// State variables
int g_GameState 		= CENT_STATE_INIT_TITLE;
int g_GameStateTimer	= 0;
int g_NumberLives		= 3;
unsigned int g_PlayerScore	= 0;
int g_MaxScores[4]		= {20000, 10000, 5000, 1000};
char g_MaxScoresInt[4][3] = { "XGS", "ISA", "WES", "OME" };
int g_GameLevel			= 0;
int g_MissileFired 		= 0;
int g_MushroomCount		= 0;
int g_RepairMushIndex	= 0;

// Spider State variables
int g_SpiderDir	= 0;	// Flag that switches to left (1) or right (0) for the direction of travel
int g_SpiderTopHeight = 0;
int g_SpiderBottomHeight = 0;
int g_SpiderState;
int g_SpiderSpeed = 1;
int g_SpiderStateCount;

// Global frame counter we can use as a random variable sometimes
int g_FrameCounter = 0;

// Used to know which index to enter intials too
int g_EnterInitialsIndex;
int g_EnterInitialsPtr;

// Variables used for state of centipede
CentiPart g_Centipede[MAX_NUMBER_CENTIPEDES];
int g_NumCentiStarted = 0;
int g_CentiSpeed = 5; // Controls the animation speed and hence the movement
int g_NumDeadCenti = 0;

// One and only bonus given at 12000
int g_IsBonusGiven = 0;

// Mushroom Color rotation
const unsigned char g_MushroomBorder[] = { 0x85,  0xB9, 0xA1, 0xC6, 0x6D, 0x6B } ;
const unsigned char g_MushroomInside[] = { 0x70,  0x3E, 0x34, 0x9D, 0xBC, 0xB1 } ;

// Centipede Color rotation
const unsigned char g_CentiInside[] = { 0x70,  0x45, 0x8A, 0xAF } ;

// Scorpion State machine
int g_ScorpionAlive = 0;
int g_ScorpionHeadPosX = 0;
int g_ScorpionHeadPosY = 0;
int g_ScorpionTailPosX = 0;
int g_ScorpionTailPosY = 0;
int g_ScorpionSpeed = 6;
int g_ScorpionDir = -1; // Negative for left, positive for right
int g_ScorpionFrameCtr;
int g_ScorpionAnimFrame;
int g_ScorpionHeadOverMush = 0;
int g_ScorpionTailOverMush = 0;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Displays the score(number) at tile position (x,y)
void DisplayNumber(int StartTileX, int StartTileY, unsigned int Number);

// Controls the flea
void Flea_Update();
void Flea_Start();

// Missile updating
void Missile_Update();

// Controls the spider
void Spider_Start();
void Spider_Update();

// Player collision
void ChkPlayerCollide();

// Controls the centipede
void Centi_Start();
void Centi_Update();

// Simple sound player
void PlaySounds();

// Scorpion control
void Scorpion_Start();
void Scorpion_Update();




// Our main start location
int main (void)
{
	int i;
	unsigned char LastGamePadValue = 0;

	// Always call ConfigureClock first
	SYS_ConfigureClock(FCY_NTSC);

	// Init the graphics system
	GFX_InitTile(g_TileData, g_TileMap, g_Palettes, g_Sprites);

	// Keyboard init
	Gamepad_Init();

	// Sound Init
	SND_Init();

	// Loop through and give everything their default color
	for(i = 0; i < MAX_PALETTE_SIZE; i++)
		g_Palettes[i] = i;

	// Fill up entire screen with black
	for(i = 0; i < SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT; i++)
		g_TileMap[i] = TILE_BLANK;

	// Setup sprite for the player
	g_Sprites[0].State = SPRITE_STATE_ON;
	g_Sprites[0].Attribute = 0;
	g_Sprites[0].X = 80;
	g_Sprites[0].Y = 176;
	g_Sprites[0].PixelData = &g_SpriteData[SPRITE_PLAYER*64];
	g_Player = &g_Sprites[0];

	// Setup sprite for missile
	g_Sprites[1].State = SPRITE_STATE_OFF;
	g_Sprites[1].Attribute = 0;
	g_Sprites[1].X = 80;
	g_Sprites[1].Y = 176;
	g_Sprites[1].PixelData = &g_SpriteData[SPRITE_MISSILE_NORMAL*64];
	g_Missile = &g_Sprites[1];


	// Setup sprite for missile
	g_Sprites[2].State = SPRITE_STATE_OFF;
	g_Sprites[2].Attribute = 0;
	g_Sprites[2].X = 80;
	g_Sprites[2].Y = 176;
	g_Sprites[2].PixelData = &g_SpriteData[SPRITE_FLEA_1*64];
	g_Flea = &g_Sprites[2];


	// Setup sprite for left side of spider
	g_Sprites[3].State = SPRITE_STATE_OFF;
	g_Sprites[3].Attribute = 0;
	g_Sprites[3].X = 80;
	g_Sprites[3].Y = 176;
	g_Sprites[3].PixelData = &g_SpriteData[SPRITE_SPIDER_LF_1*64];
	g_SpiderLeft = &g_Sprites[3];


	// Setup sprite for right side of spider
	g_Sprites[4].State = SPRITE_STATE_OFF;
	g_Sprites[4].Attribute = 0;
	g_Sprites[4].X = 80;
	g_Sprites[4].Y = 176;
	g_Sprites[4].PixelData = &g_SpriteData[SPRITE_SPIDER_RT_1*64];
	g_SpiderRight = &g_Sprites[4];



	// Start up our display interrupt
	GFX_StartDrawing(SCREEN_TYPE_NTSC);

	// Main Game Loop
	while(1)
	{
		// Wait until we are in v-sync to do any processing
		WAIT_FOR_VSYNC_START();

		// Increment our frame counter so we can use it as a random variable sometimes
		g_FrameCounter++;

		// Check the current game state
		switch(g_GameState)	
		{
		case CENT_STATE_INIT_TITLE:
		{
			// Fill up entire screen with black
			for(i = 0; i < SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT; i++)
				g_TileMap[i] = TILE_BLANK;

			// Initialize the game field with mushrooms
			g_MushroomCount = 0;
			int MushPlace;
			while(g_MushroomCount != NUMBER_MUSHROOMS_LEVEL)
			{
				MushPlace = RAND_RANGE(1*SCREEN_TILE_MAP_WIDTH, (SCREEN_TILE_MAP_WIDTH-1)*SCREEN_TILE_MAP_WIDTH);
				if(g_TileMap[MushPlace] != TILE_BLANK) continue;
				// We found an empty spot
				g_TileMap[MushPlace] = TILE_SOLID_GREEN_MUSH;
				g_MushroomCount++;
			}

			// Clear out space for high score and messages
			for(i = 0; i < 11; i++)
				memset(&g_TileMap[SCREEN_TILE_MAP_WIDTH*(2+i) + 3], TILE_BLANK, 13);

			// Display the high score
			DisplayNumber(18,0,g_MaxScores[0]);

			// Now replace middle section with high scores
			// H I G H   S C O R E		
			g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 4] = TILE_LETTER_START + 'H' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 5] = TILE_LETTER_START + 'I' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 6] = TILE_LETTER_START + 'G' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 7] = TILE_LETTER_START + 'H' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 9] = TILE_LETTER_START + 'S' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 10] = TILE_LETTER_START + 'C' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 11] = TILE_LETTER_START + 'O' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 12] = TILE_LETTER_START + 'R' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 13] = TILE_LETTER_START + 'E' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 14] = TILE_LETTER_START + 'S' - 'A';

			// SCORE INIT
			DisplayNumber(9, 4, g_MaxScores[0]);
			g_TileMap[SCREEN_TILE_MAP_WIDTH*4 + 11] = TILE_LETTER_START + g_MaxScoresInt[0][0] - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*4 + 12] = TILE_LETTER_START + g_MaxScoresInt[0][1] - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*4 + 13] = TILE_LETTER_START + g_MaxScoresInt[0][2] - 'A';
			DisplayNumber(9, 5, g_MaxScores[1]);
			g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 11] = TILE_LETTER_START + g_MaxScoresInt[1][0] - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 12] = TILE_LETTER_START + g_MaxScoresInt[1][1] - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 13] = TILE_LETTER_START + g_MaxScoresInt[1][2] - 'A';
			DisplayNumber(9, 6, g_MaxScores[2]);
			g_TileMap[SCREEN_TILE_MAP_WIDTH*6 + 11] = TILE_LETTER_START + g_MaxScoresInt[2][0] - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*6 + 12] = TILE_LETTER_START + g_MaxScoresInt[2][1] - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*6 + 13] = TILE_LETTER_START + g_MaxScoresInt[2][2] - 'A';
			DisplayNumber(9, 7, g_MaxScores[3]);
			g_TileMap[SCREEN_TILE_MAP_WIDTH*7 + 11] = TILE_LETTER_START + g_MaxScoresInt[3][0] - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*7 + 12] = TILE_LETTER_START + g_MaxScoresInt[3][1] - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*7 + 13] = TILE_LETTER_START + g_MaxScoresInt[3][2] - 'A';

			

			// P R E S S  S T A R T
			g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 4] = TILE_LETTER_START + 'P' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 5] = TILE_LETTER_START + 'R' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 6] = TILE_LETTER_START + 'E' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 7] = TILE_LETTER_START + 'S' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 8] = TILE_LETTER_START + 'S' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 10] = TILE_LETTER_START + 'S' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 11] = TILE_LETTER_START + 'T' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 12] = TILE_LETTER_START + 'A' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 13] = TILE_LETTER_START + 'R' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 14] = TILE_LETTER_START + 'T' - 'A';

			// B O N U S  1 2 0 0 0
			g_TileMap[SCREEN_TILE_MAP_WIDTH*11 + 4] = TILE_LETTER_START + 'B' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*11 + 5] = TILE_LETTER_START + 'O' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*11 + 6] = TILE_LETTER_START + 'N' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*11 + 7] = TILE_LETTER_START + 'U' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*11 + 8] = TILE_LETTER_START + 'S' - 'A';
			g_TileMap[SCREEN_TILE_MAP_WIDTH*11 + 10] = TILE_NUMBERS_START + 1;
			g_TileMap[SCREEN_TILE_MAP_WIDTH*11 + 11] = TILE_NUMBERS_START + 2;
			g_TileMap[SCREEN_TILE_MAP_WIDTH*11 + 12] = TILE_NUMBERS_START + 0;
			g_TileMap[SCREEN_TILE_MAP_WIDTH*11 + 13] = TILE_NUMBERS_START + 0;
			g_TileMap[SCREEN_TILE_MAP_WIDTH*11 + 14] = TILE_NUMBERS_START + 0;


			// Place bottom copyright fun
			g_TileMap[SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT-SCREEN_TILE_MAP_WIDTH/2 - 1] = TILE_XGS_LOGO_LEFT;
			g_TileMap[SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT-SCREEN_TILE_MAP_WIDTH/2] = TILE_XGS_LOGO_RIGHT;
			g_TileMap[SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT-SCREEN_TILE_MAP_WIDTH/2 + 1] = TILE_XGS_LOGO_RIGHT2;

			// Place number of guys at the top
			g_TileMap[7] = g_TileMap[8] = g_TileMap[9] = TILE_PLAYER;

			// Change the color back to normal if this is a start over
			// Change the colors of the mushroom tiles
			for(i=0; i < 4*64; i++)
			{
				if(g_TileData[TILE_SOLID_GREEN_MUSH*64+i] == g_MushroomBorder[g_GameLevel%6])
					g_TileData[TILE_SOLID_GREEN_MUSH*64+i] = g_MushroomBorder[0];
				else if(g_TileData[TILE_SOLID_GREEN_MUSH*64+i] == g_MushroomInside[g_GameLevel%6])
					g_TileData[TILE_SOLID_GREEN_MUSH*64+i] = g_MushroomInside[0];
			}

			// Change the colors of the centipede
			for(i=0; i < 4*64; i++)
			{
				if(g_TileData[TILE_CENTI_LEFT_1*64+i] == g_CentiInside[g_GameLevel%4])
					g_TileData[TILE_CENTI_LEFT_1*64+i] = g_CentiInside[0];
			}				

			// Change the color fo teh centipede head
			for(i=0; i < 2*64; i++)
			{
				if(g_TileData[TILE_HEAD_LEFT*64+i] == g_CentiInside[g_GameLevel%4])
					g_TileData[TILE_HEAD_LEFT*64+i] = g_CentiInside[0];
			}				

			
			// Init all game start variables
			g_GameStateTimer	= 0;
			g_NumberLives		= 3;
			g_PlayerScore		= 0;
			g_GameLevel			= 0;
			g_MissileFired 		= 0;
			g_IsBonusGiven 	    = 0;

			g_Player->State = SPRITE_STATE_ON;

			g_ScorpionAlive = 0; 

			// Move onto the title screen
			g_GameState = CENT_STATE_TITLE;
			break;
		}
		case CENT_STATE_TITLE:
		{
			// Read the gamepad
			unsigned char GamepadValue = Gamepad_Read(0);
			if(GamepadValue == LastGamePadValue) break;
			LastGamePadValue = GamepadValue;

			// Display the top scores, wait here until a key is pressed
			if(GamepadValue & GAMEPAD_START )
			{
				// Seed the random number generator with the frame counter
				srand(g_FrameCounter);

				// Recreate the board to remove the high scores

				// Fill up entire screen with black
				for(i = 0; i < SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT; i++)
					g_TileMap[i] = TILE_BLANK;

				// Display the high score
				DisplayNumber(18,0,g_MaxScores[0]);
	
				// Initialize the game field with mushrooms
				g_MushroomCount = 0;
				int MushPlace;
				while(g_MushroomCount != NUMBER_MUSHROOMS_LEVEL)
				{
					MushPlace = RAND_RANGE(2*SCREEN_TILE_MAP_WIDTH, (SCREEN_TILE_MAP_WIDTH-1)*SCREEN_TILE_MAP_WIDTH);
					if(g_TileMap[MushPlace] != TILE_BLANK) continue;
					// We found an empty spot
					g_TileMap[MushPlace] = TILE_SOLID_GREEN_MUSH;
					g_MushroomCount++;
				}

				// Clear out lives
				g_TileMap[7] = g_TileMap[8] = g_TileMap[9] = g_TileMap[10] = TILE_BLANK;								
				int i;
				for(i = 0; i < g_NumberLives; i++)
					g_TileMap[7+i] = TILE_PLAYER;
		
				g_GameState = CENT_STATE_PLAY;
			}
			break;
		}
		case CENT_STATE_PLAY:
		{

			// Read the gamepad
			unsigned char GamepadValue = Gamepad_Read(0);
			
			// Adjust the sprites positions with gamepad up/down/left/right
			if(GamepadValue & GAMEPAD_UP)
			{
				// Two rows available to move
				if(g_Player->Y > 192-TILE_HEIGHT*7)
					g_Player->Y-=2;
				
				// Check if we are now in a mushroom
				unsigned char TileType = g_TileMap[((g_Player->X - 5)>>3) + ((g_Player->Y - 4)>>3)*SCREEN_TILE_MAP_WIDTH];
				if(TileType >= TILE_SOLID_GREEN_MUSH && TileType <= TILE_HALF_GREEN_MUSH)
					g_Player->Y+=2;
			}
			else if(GamepadValue & GAMEPAD_DOWN)
			{
				if(g_Player->Y < 192-TILE_HEIGHT)
					g_Player->Y+=2;

				// Check if we are now in a mushroom
				unsigned char TileType = g_TileMap[((g_Player->X - 5)>>3) + ((g_Player->Y - 4)>>3)*SCREEN_TILE_MAP_WIDTH];
				if(TileType >= TILE_SOLID_GREEN_MUSH && TileType <= TILE_HALF_GREEN_MUSH)
					g_Player->Y-=2;
			}
	
			if(GamepadValue & GAMEPAD_RIGHT)
			{
				if(g_Player->X < 168-TILE_WIDTH)
					g_Player->X+=2;

				// Check if we are now in a mushroom
				unsigned char TileType = g_TileMap[((g_Player->X - 5)>>3) + ((g_Player->Y - 4)>>3)*SCREEN_TILE_MAP_WIDTH];
				if(TileType >= TILE_SOLID_GREEN_MUSH && TileType <= TILE_HALF_GREEN_MUSH)
					g_Player->X-=2;

			}
			else if(GamepadValue & GAMEPAD_LEFT)
			{
				if(g_Player->X > TILE_WIDTH)
					g_Player->X-=2;

				// Check if we are now in a mushroom
				unsigned char TileType = g_TileMap[((g_Player->X - 5)>>3) + ((g_Player->Y - 4)>>3)*SCREEN_TILE_MAP_WIDTH];
				if(TileType >= TILE_SOLID_GREEN_MUSH && TileType <= TILE_HALF_GREEN_MUSH)
					g_Player->X+=2;
			}

			if(GamepadValue & GAMEPAD_ACTION2)
			{
				// Test to see if a missile has previously been fired
				if(g_Missile->State == SPRITE_STATE_OFF)
				{
					// No so launch one at player position
					g_Missile->X = g_Player->X;
					g_Missile->Y = g_Player->Y - 7;
					g_Missile->State = SPRITE_STATE_ON;

					// Play sound blaster
					g_SoundOverride = SOUND_OVERRIDE_BLASTER;
				}
			}
			if(GamepadValue & GAMEPAD_SELECT)
			{
				g_GameState = CENT_STATE_NEXT_LEVEL;
				g_GameStateTimer = 60;
			}


			// Move the missile and check for collisions
			if(g_Missile->State == SPRITE_STATE_ON)
				Missile_Update();

			
			// Try to spawn the flea
			if(g_Flea->State == SPRITE_STATE_OFF)
			{
				// Spwan flea based on the number of mushrooms on the screen and
				// what level the player is on and a random variable
				if(g_MushroomCount < (int)(NUMBER_MUSHROOMS_LEVEL*.7 + g_GameLevel*2))
					Flea_Start();
			}
			else
				Flea_Update();

			// Try to spawn the spider
			if(g_SpiderLeft->State == SPRITE_STATE_OFF)
			{
				// Spawn spider occurance totally based on random and not level...
				// He appears quite often
				if(RAND_RANGE(0, 500) == 1)
					Spider_Start();
			}
			else
				Spider_Update();

			// Update the centipede
			if(g_NumCentiStarted != MAX_NUMBER_CENTIPEDES)
				Centi_Start();
			Centi_Update();			// Always update

			// Do scorpion update last since it overrides some tiles
			if(g_ScorpionAlive == 0)
			{
				if(g_GameLevel > 1)
				{
					// Do some probability for scorpion staring
					if(RAND_RANGE(0,1000 - g_GameLevel * 10) == 1)
						Scorpion_Start();
				}			
			}
			else
				Scorpion_Update();

			// Check for player collision
			ChkPlayerCollide();

			// Check to see if they've gotten a free guy at 12000
			if(g_IsBonusGiven == 0 && g_PlayerScore > 12000)
			{
				g_IsBonusGiven = 1;
				g_NumberLives++;
				// Clear out lives
				g_TileMap[7] = g_TileMap[8] = g_TileMap[9] = g_TileMap[10] = TILE_BLANK;								
				int i;
				for(i = 0; i < g_NumberLives; i++)
					g_TileMap[7+i] = TILE_PLAYER;
			}

			// Finally play some sounds
			PlaySounds();

			break;  
		}// end case CENT_STATE_PLAY

		case CENT_STATE_NEXT_LEVEL:

			// Stop sounds
			g_CurrentSound = 0;
			SND_PlayTone(0);

			if(--g_GameStateTimer == 0)
			{
				// Change the colors of the mushroom tiles
				for(i=0; i < 4*64; i++)
				{
					if(g_TileData[TILE_SOLID_GREEN_MUSH*64+i] == g_MushroomBorder[g_GameLevel%6])
						g_TileData[TILE_SOLID_GREEN_MUSH*64+i] = g_MushroomBorder[(g_GameLevel+1)%6];
					else if(g_TileData[TILE_SOLID_GREEN_MUSH*64+i] == g_MushroomInside[g_GameLevel%6])
						g_TileData[TILE_SOLID_GREEN_MUSH*64+i] = g_MushroomInside[(g_GameLevel+1)%6];
				}


				// Change the colors of the centipede
				for(i=0; i < 4*64; i++)
				{
					if(g_TileData[TILE_CENTI_LEFT_1*64+i] == g_CentiInside[g_GameLevel%4])
						g_TileData[TILE_CENTI_LEFT_1*64+i] = g_CentiInside[(g_GameLevel+1)%4];
				}				

				// Change the color fo teh centipede head
				for(i=0; i < 2*64; i++)
				{
					if(g_TileData[TILE_HEAD_LEFT*64+i] == g_CentiInside[g_GameLevel%4])
						g_TileData[TILE_HEAD_LEFT*64+i] = g_CentiInside[(g_GameLevel+1)%4];
				}				


				// Reset the centipede
				for(i = 0; i < MAX_NUMBER_CENTIPEDES && i < g_NumCentiStarted; i++)
				{
					if(g_Centipede[i].IsDead == 0)
						g_TileMap[g_Centipede[i].XTile + g_Centipede[i].YTile * SCREEN_TILE_MAP_WIDTH] = TILE_BLANK;
				}
				g_NumCentiStarted = 0;
				g_NumDeadCenti = 0;

				// Move onto the next level				
				g_GameLevel++;
				g_GameState = CENT_STATE_PLAY;

				// Increase the speed of the centipede
				if(g_GameLevel < 4)
				{
					if(g_CentiSpeed > 4) g_CentiSpeed--;
				}
				else if(g_GameLevel < 8)
				{
					if(g_CentiSpeed > 3) g_CentiSpeed--;
				}
				else
				{
					// Extreme speed!!!
					if(g_CentiSpeed > 2) g_CentiSpeed--;
				}
			}

			break;

		case CENT_STATE_DYING:

			// Keep the sounds playing
			PlaySounds();

			// Dying state we update the animation counter
			if(--g_Player->Attribute == 0)
			{
				g_Player->Attribute = 60; // ticks of animation
				g_Player->PixelData = &g_SpriteData[SPRITE_EXPLOSION_2*64];
				g_GameState = CENT_STATE_DEAD;
				g_GameStateTimer = 0;
			}
			break;

		case CENT_STATE_DEAD:

			// Keep the sounds playing
			PlaySounds();

			if(g_Player->Attribute && (--g_Player->Attribute == 0))
			{
				g_Player->X = 80;
				g_Player->Y = 176;
				g_Player->PixelData = &g_SpriteData[SPRITE_PLAYER*64];
				g_Player->State = SPRITE_STATE_OFF;

				// Update lives
				g_NumberLives -= 1;
				// Clear out lives
				g_TileMap[7] = g_TileMap[8] = g_TileMap[9] = g_TileMap[10] = TILE_BLANK;								
				int i;
				for(i = 0; i < g_NumberLives; i++)
					g_TileMap[7+i] = TILE_PLAYER;


				// Reset the centipede
				for(i = 0; i < MAX_NUMBER_CENTIPEDES && i < g_NumCentiStarted; i++)
				{
					if(g_Centipede[i].IsDead == 0)
						g_TileMap[g_Centipede[i].XTile + g_Centipede[i].YTile * SCREEN_TILE_MAP_WIDTH] = TILE_BLANK;
				}
				g_NumCentiStarted = 0;
				g_NumDeadCenti = 0;

				// Reset the centipedes speed
				g_CentiSpeed = 5;

				g_RepairMushIndex = SCREEN_TILE_MAP_WIDTH*SCREEN_TILE_MAP_HEIGHT-1;
				g_GameStateTimer = 10;
			}

			// Wait until a little time has elapsed for each repair mushroom
			if(g_GameStateTimer)
			{
				if(--g_GameStateTimer == 0)
				{
					// Now its time to count up the destroyed mushrooms and repair them
					for(i = g_RepairMushIndex; i >= 0; i--)
					{
						if((g_TileMap[i] >= TILE_RIGHT_GREEN_MUSH) && (g_TileMap[i] <= TILE_HALF_GREEN_MUSH))
						{
							// We found a beat up mushroom
							g_PlayerScore += 5;
							g_TileMap[i] = TILE_SOLID_GREEN_MUSH;
							g_RepairMushIndex = i;
							g_GameStateTimer = 10;
							g_SoundOverride = SOUND_OVERRIDE_MUSHROOM;
							break;
						}
						else if(g_TileMap[i] <= TILE_HALF_POIS_MUSH)
						{
							g_TileMap[i] = TILE_SOLID_GREEN_MUSH;
							g_RepairMushIndex = i;
							g_GameStateTimer = 10;
							g_SoundOverride = SOUND_OVERRIDE_MUSHROOM;
							break;
						}
					} // end search through tile map
					
					// Are we done repairing all mushrooms?
					if(i < 0)
					{
						if(g_NumberLives == 0) g_GameState = CENT_STATE_GAME_OVER;
						else
						{
							g_GameState = CENT_STATE_PLAY;
							g_Player->State = SPRITE_STATE_ON;
						}
					}
				}

			} // End timer for repairing mushrooms

			break;

		case CENT_STATE_GAME_OVER:
			// Check for high score and enter in initials

			for(i=0; i < 4; i++)
				if(g_PlayerScore > g_MaxScores[i])
					break;

			if(i < 4)
			{
				g_EnterInitialsIndex = i;
				g_EnterInitialsPtr = 0;

				// Setup text

				// P R E S S  S T A R T
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 2] = TILE_LETTER_START + 'E' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 3] = TILE_LETTER_START + 'N' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 4] = TILE_LETTER_START + 'T' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 5] = TILE_LETTER_START + 'E' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 6] = TILE_LETTER_START + 'R' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 8] = TILE_LETTER_START + 'I' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 9] = TILE_LETTER_START + 'N' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 10] = TILE_LETTER_START + 'I' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 11] = TILE_LETTER_START + 'T' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 12] = TILE_LETTER_START + 'I' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 13] = TILE_LETTER_START + 'A' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 14] = TILE_LETTER_START + 'L' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*3 + 15] = TILE_LETTER_START + 'S' - 'A';

				g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 8] = TILE_LETTER_START + 'A' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 9] = TILE_LETTER_START + 'A' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 10] = TILE_LETTER_START + 'A' - 'A';

				// Selector is the bug blaster
				g_TileMap[SCREEN_TILE_MAP_WIDTH*6 + 8] = TILE_PLAYER;

				// P R E S S  S T A R T
				g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 4] = TILE_LETTER_START + 'P' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 5] = TILE_LETTER_START + 'R' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 6] = TILE_LETTER_START + 'E' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 7] = TILE_LETTER_START + 'S' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 8] = TILE_LETTER_START + 'S' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 10] = TILE_LETTER_START + 'S' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 11] = TILE_LETTER_START + 'T' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 12] = TILE_LETTER_START + 'A' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 13] = TILE_LETTER_START + 'R' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*9 + 14] = TILE_LETTER_START + 'T' - 'A';

				// W H E N   D O N E
				g_TileMap[SCREEN_TILE_MAP_WIDTH*10 + 5] = TILE_LETTER_START + 'W' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*10 + 6] = TILE_LETTER_START + 'H' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*10 + 7] = TILE_LETTER_START + 'E' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*10 + 8] = TILE_LETTER_START + 'N' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*10 + 10] = TILE_LETTER_START + 'D' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*10 + 11] = TILE_LETTER_START + 'O' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*10 + 12] = TILE_LETTER_START + 'N' - 'A';
				g_TileMap[SCREEN_TILE_MAP_WIDTH*10 + 13] = TILE_LETTER_START + 'E' - 'A';

				
				g_MaxScores[i] = g_PlayerScore;

				g_GameState = CENT_STATE_ENTER_INT;
			}
			else
				g_GameState = CENT_STATE_INIT_TITLE;	// No high score

			break;

		// Enter high score initials
		case CENT_STATE_ENTER_INT:
		{		
			// Read the gamepad
			unsigned char GamepadValue = Gamepad_Read(0);

			// Don't grab repeated buttons
			if(GamepadValue == LastGamePadValue) break;
			LastGamePadValue = GamepadValue;
			
			// Adjust the sprites positions with gamepad up/down/left/right
			if((GamepadValue & GAMEPAD_RIGHT) && g_EnterInitialsPtr < 2)
			{
				g_TileMap[SCREEN_TILE_MAP_WIDTH*6 + 8 + g_EnterInitialsPtr] = TILE_BLANK;
				g_EnterInitialsPtr++;
				g_TileMap[SCREEN_TILE_MAP_WIDTH*6 + 8 + g_EnterInitialsPtr] = TILE_PLAYER;
			}
			if((GamepadValue & GAMEPAD_LEFT) && g_EnterInitialsPtr > 0)
			{
				g_TileMap[SCREEN_TILE_MAP_WIDTH*6 + 8 + g_EnterInitialsPtr] = TILE_BLANK;
				g_EnterInitialsPtr--;
				g_TileMap[SCREEN_TILE_MAP_WIDTH*6 + 8 + g_EnterInitialsPtr] = TILE_PLAYER;
			}
			if(GamepadValue & GAMEPAD_UP)
			{
				int CurrentLetter = g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 8 + g_EnterInitialsPtr] - TILE_LETTER_START;
				CurrentLetter = (CurrentLetter + 1) % 26; // 26 letters in alphabet
				g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 8 + g_EnterInitialsPtr] = CurrentLetter +  TILE_LETTER_START;
			}
			if(GamepadValue & GAMEPAD_DOWN)
			{
				int CurrentLetter = g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 8 + g_EnterInitialsPtr] - TILE_LETTER_START;
				CurrentLetter = (CurrentLetter - 1) % 26; // 26 letters in alphabet
				g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 8 + g_EnterInitialsPtr] = CurrentLetter +  TILE_LETTER_START;
			}
			if(GamepadValue & GAMEPAD_START)
			{
				int CurrentLetter = g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 8 + 0] - TILE_LETTER_START;
				g_MaxScoresInt[g_EnterInitialsIndex][0] = CurrentLetter + 'A';
				CurrentLetter = g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 8 + 1] - TILE_LETTER_START;
				g_MaxScoresInt[g_EnterInitialsIndex][1] = CurrentLetter + 'A';
				CurrentLetter = g_TileMap[SCREEN_TILE_MAP_WIDTH*5 + 8 + 2] - TILE_LETTER_START;
				g_MaxScoresInt[g_EnterInitialsIndex][2] = CurrentLetter + 'A';
				g_GameState = CENT_STATE_INIT_TITLE;
			}
			break;
		} // end CENT_STATE_ENTER_INT
		} // end switch case game state
		

		// Display the player score always
		DisplayNumber(6,0,g_PlayerScore);

		// Wait here till we are done with v-sync
		WAIT_FOR_VSYNC_END();

	}

	return 0;
}


// Displays the score(number) at tile position (x,y)
void DisplayNumber(int StartTileX, int StartTileY, unsigned int Number)
{
	int Index = StartTileY*SCREEN_TILE_MAP_WIDTH + StartTileX;

	do
	{
		g_TileMap[Index] = TILE_NUMBERS_START + Number%10;
		Number /= 10;
		Index--;
	}
	while(Number > 0);
}


// Spawns a flea at a random position at the top of the screen
// and initializes variables
void Flea_Start()
{
	// Find a random position on the screen to start the flea at
	int FleaStartPos = RAND_RANGE(1,20);	// 1 because 0 is off the screen when we times by 8
	g_Flea->X = FleaStartPos<<3;
	g_Flea->Y = 0;
	g_Flea->State = SPRITE_STATE_ON;
	g_Flea->Attribute = 8;					// Unused variable we will use as an animation counter
}


// Updates the fleas position
void Flea_Update()
{
	// Flea falls straight down
	g_Flea->Y += 2;
	if(--g_Flea->Attribute == 0)
	{
		if(g_Flea->PixelData == &g_SpriteData[SPRITE_FLEA_1*64])
			g_Flea->PixelData = &g_SpriteData[SPRITE_FLEA_2*64];
		else
			g_Flea->PixelData = &g_SpriteData[SPRITE_FLEA_1*64];
		g_Flea->Attribute = 8;

	}

	// See if there is a mushroom under us and make sure we are in play region
	if(g_Flea->Y > 32 && g_Flea->Y < 184)
	{
		// Check if the tile beneath is blank
		int TileIndex = ((g_Flea->Y-8)>>3)*SCREEN_TILE_MAP_WIDTH + ((g_Flea->X-8)>>3);
		if(g_TileMap[TileIndex] == TILE_BLANK)
		{
			if(RAND_RANGE(0,8) == 1)
			{
				g_TileMap[TileIndex] = TILE_SOLID_GREEN_MUSH;
				g_MushroomCount++;
			}
		}
	}

	// Check for off the screen
	if(g_Flea->Y > 192)
		g_Flea->State = SPRITE_STATE_OFF;
}


// Spider Starts alternates starting left or right and only travels one lateral direction
void Spider_Start()
{
	// Check last movement direction left (1), right (0)
	if(g_FrameCounter & 0x1)
	{
		g_SpiderDir = SPIDER_DIR_LEFT;	// left
		g_SpiderLeft->X = 160;
		g_SpiderRight->X = 168;
	}	
	else
	{
		g_SpiderDir = SPIDER_DIR_RIGHT;	// right
		g_SpiderLeft->X = 0;
		g_SpiderRight->X = 8;
	}

	g_SpiderLeft->Y = 150;
	g_SpiderLeft->Attribute = 8;	// Animation counter
	g_SpiderRight->Attribute = 0;	// Animation frame
	g_SpiderRight->Y = 150;
	g_SpiderLeft->State = SPRITE_STATE_ON;
	g_SpiderRight->State = SPRITE_STATE_ON;
	g_SpiderState = SPIDER_STATE_LATERAL_UP;
	g_SpiderStateCount = RAND_RANGE(30, 180);
	g_SpiderSpeed = 1;
}


void Spider_Update()
{
	int MoveToNextState = 0;

	switch(g_SpiderState)
	{
	case SPIDER_STATE_UP:

		// See if its time to transition to next state
		if(g_SpiderLeft->Y == 100){ MoveToNextState = 1; break; }

		// Always move Up
		g_SpiderLeft->Y -= g_SpiderSpeed;
		g_SpiderRight->Y -= g_SpiderSpeed;

		break;
	case SPIDER_STATE_DOWN:

		// See if its time to transition to next state
		if(g_SpiderLeft->Y == 192){ MoveToNextState = 1; break; }

		// Always move down
		g_SpiderLeft->Y += g_SpiderSpeed;
		g_SpiderRight->Y += g_SpiderSpeed;

		break;
	case SPIDER_STATE_LATERAL_UP:

		// See if its time to transition to next state
		if(g_SpiderLeft->Y == 100){ MoveToNextState = 1; break; }
		
		// Move position based on direction
		if(g_SpiderDir == SPIDER_DIR_LEFT) // left
		{
			g_SpiderLeft->X -= g_SpiderSpeed;
			g_SpiderRight->X -= g_SpiderSpeed;		
		}
		else
		{
			g_SpiderLeft->X += g_SpiderSpeed;
			g_SpiderRight->X += g_SpiderSpeed;
		}

		// Always move down
		g_SpiderLeft->Y -= g_SpiderSpeed;
		g_SpiderRight->Y -= g_SpiderSpeed;

		break;
	case SPIDER_STATE_LATERAL_DOWN:

		// See if its time to transition to next state
		if(g_SpiderLeft->Y == 192){ MoveToNextState = 1; break; }
		
		// Move position based on direction
		if(g_SpiderDir == SPIDER_DIR_LEFT) // left
		{
			g_SpiderLeft->X -= g_SpiderSpeed;
			g_SpiderRight->X -= g_SpiderSpeed;		
		}
		else
		{
			g_SpiderLeft->X += g_SpiderSpeed;
			g_SpiderRight->X += g_SpiderSpeed;
		}

		// Always move down
		g_SpiderLeft->Y += g_SpiderSpeed;
		g_SpiderRight->Y += g_SpiderSpeed;

		break;
	case SPIDER_STATE_CHASE:
		break;
	}

	// Check for spider off screen, do this by checking larger than 160 on the X,
	// since we are using unsigned ints this will cover going to far to the left as well
	// as to far to the right
	if(g_SpiderLeft->X > 160)
	{
		// Turn spider off
		g_SpiderLeft->State = SPRITE_STATE_OFF;
		g_SpiderRight->State = SPRITE_STATE_OFF;
		return;
	}

	// Check for next state transition
	if(MoveToNextState || (--g_SpiderStateCount == 0))
	{
		int RandVal = RAND_RANGE(0,10);
		if(RandVal < 3)
			g_SpiderState = SPIDER_STATE_UP;
		else if(RandVal < 6)
			g_SpiderState = SPIDER_STATE_DOWN;
		else if(RandVal < 8)
			g_SpiderState = SPIDER_STATE_LATERAL_UP;
		else
			g_SpiderState = SPIDER_STATE_LATERAL_DOWN;

		//g_SpiderState = RAND_RANGE(0,3);
		if(g_SpiderState == SPIDER_STATE_UP || g_SpiderState == SPIDER_STATE_DOWN)
			g_SpiderStateCount = RAND_RANGE(30, 180);
		else
			g_SpiderStateCount = RAND_RANGE(0, 60);
	}
	else
	{
		// Animate the spider
		if(--g_SpiderLeft->Attribute == 0)
		{
			// Reset the counter
			g_SpiderLeft->Attribute = 15;

			// Determine what frame we are showing
			if(g_SpiderRight->Attribute == 0)
			{
				g_SpiderRight->Attribute = 1;
				g_SpiderLeft->PixelData = &g_SpriteData[SPRITE_SPIDER_LF_2*64];
				g_SpiderRight->PixelData = &g_SpriteData[SPRITE_SPIDER_RT_2*64];
			}
			else if(g_SpiderRight->Attribute == 1)
			{
				g_SpiderRight->Attribute = 2;
				g_SpiderLeft->PixelData = &g_SpriteData[SPRITE_SPIDER_LF_3*64];
				g_SpiderRight->PixelData = &g_SpriteData[SPRITE_SPIDER_RT_3*64];
			}
			else
			{
				g_SpiderRight->Attribute = 0;
				g_SpiderLeft->PixelData = &g_SpriteData[SPRITE_SPIDER_LF_1*64];
				g_SpiderRight->PixelData = &g_SpriteData[SPRITE_SPIDER_RT_1*64];
			}
		}
	} // end animating spider


	// Now check to see if spider is going to eat a mushroom
	if((g_SpiderRight->Y & 0x3) == 0) // Check to see if we are on a divide by 8 Y boundry
	{
		// Check to see if the center X position is on a div		
		if(((g_SpiderRight->X-2) & 0x3) == 0)
		{
			// Now see if there is a mushroom under us
			int TileIndex = ((g_SpiderRight->Y-8)>>3)*SCREEN_TILE_MAP_WIDTH + ((g_SpiderRight->X-10)>>3);
			if(g_TileMap[TileIndex] >= TILE_SOLID_GREEN_MUSH && g_TileMap[TileIndex] <= TILE_HALF_GREEN_MUSH)
			{
				g_TileMap[TileIndex] = TILE_BLANK;
				g_PlayerScore += 100;
			}
		}
	}

} // end Spider_Update


// Updates the missile with collision and movement
void Missile_Update()
{
	int i;

	// Check for missile / mushroom collision and more
	int XIndex = (g_Missile->X - 5)/8;
	int YIndex = (g_Missile->Y - 8)/8;
	if(YIndex < 0) YIndex = 0;
	if(XIndex < 0) XIndex = 0;
	unsigned char TileType = g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex];

	// See what we collided with
	switch(TileType)
	{
	case TILE_SOLID_POIS_MUSH:
		g_Missile->State = SPRITE_STATE_OFF;	

		// Determine which side of the mushroom it was on
		if((g_Missile->X & 0x7) < 4)
			g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_LEFT_POIS_MUSH;
		else
			g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_RIGHT_POIS_MUSH;
		break;

	case TILE_RIGHT_POIS_MUSH:
		g_Missile->State = SPRITE_STATE_OFF;	

		// Determine which side of the mushroom it was on
		if((g_Missile->X & 0x7) < 4)
			g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_HALF_POIS_MUSH;
		else
		{
			g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_BLANK;
			g_PlayerScore++;
			g_MushroomCount--;
		}
		break;

	case TILE_LEFT_POIS_MUSH:
		g_Missile->State = SPRITE_STATE_OFF;	

		// Determine which side of the mushroom it was on
		if((g_Missile->X & 0x7) < 4)
		{
			g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_BLANK;
			g_PlayerScore++;
			g_MushroomCount--;
		}
		else
			g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_HALF_POIS_MUSH;
		break;

	case TILE_HALF_POIS_MUSH:
		g_Missile->State = SPRITE_STATE_OFF;	
		g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_BLANK;
		g_PlayerScore++;
		g_MushroomCount--;
		break;


	case TILE_SOLID_GREEN_MUSH:
		g_Missile->State = SPRITE_STATE_OFF;	

		// Determine which side of the mushroom it was on
		if((g_Missile->X & 0x7) < 4)
			g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_LEFT_GREEN_MUSH;
		else
			g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_RIGHT_GREEN_MUSH;
		break;

	case TILE_RIGHT_GREEN_MUSH:
		g_Missile->State = SPRITE_STATE_OFF;	

		// Determine which side of the mushroom it was on
		if((g_Missile->X & 0x7) < 4)
			g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_HALF_GREEN_MUSH;
		else
		{
			g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_BLANK;
			g_PlayerScore++;
			g_MushroomCount--;
		}
		break;

	case TILE_LEFT_GREEN_MUSH:
		g_Missile->State = SPRITE_STATE_OFF;	

		// Determine which side of the mushroom it was on
		if((g_Missile->X & 0x7) < 4)
		{
			g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_BLANK;
			g_PlayerScore++;
			g_MushroomCount--;
		}
		else
			g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_HALF_GREEN_MUSH;
		break;

	case TILE_HALF_GREEN_MUSH:
		g_Missile->State = SPRITE_STATE_OFF;	
		g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_BLANK;
		g_PlayerScore++;
		g_MushroomCount--;
		break;

	case TILE_CENTI_LEFT_1:
	case TILE_CENTI_LEFT_2:
	case TILE_CENTI_RIGHT_1:
	case TILE_CENTI_RIGHT_2:
	case TILE_HEAD_LEFT:
	case TILE_HEAD_RIGHT:
		// Find out which segment this belongs to
		for(i=0; i < g_NumCentiStarted && i < MAX_NUMBER_CENTIPEDES; i++)
		{
			if((g_Centipede[i].XTile == XIndex) && (g_Centipede[i].YTile == YIndex) && (g_Centipede[i].IsDead == 0))
				break;			
		}

		g_Missile->State = SPRITE_STATE_OFF;	
		g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex] = TILE_SOLID_GREEN_MUSH;
		g_MushroomCount++;
		g_Centipede[i].IsDead = 1;

		// Scoring head = 100, body = 10
		if(g_Centipede[i].IsHead) g_PlayerScore += 100;
		else g_PlayerScore += 10;

		g_SoundOverride = SOUND_OVERRIDE_MUSHROOM;

		// If the next segment is alive turn it into a head
		if(i < (g_NumCentiStarted - 1) && 
		   i < (MAX_NUMBER_CENTIPEDES - 1) &&
		   (g_Centipede[i+1].IsDead == 0))
			g_Centipede[i+1].IsHead = 1;

		// Finally increment number of dead centipedes so we can see if the round is over
		if(++g_NumDeadCenti == MAX_NUMBER_CENTIPEDES)
		{
			g_GameState = CENT_STATE_NEXT_LEVEL;
			g_Flea->State = SPRITE_STATE_OFF;
			g_SpiderLeft->State = SPRITE_STATE_OFF;	
			g_SpiderRight->State = SPRITE_STATE_OFF;
			g_GameStateTimer = 60;
		}

		break;

	case TILE_SCORPION_HEAD:
	case TILE_SCORPION_TAIL_1:
	case TILE_SCORPION_TAIL_2:

		g_ScorpionAlive = 0;

		// Replaces any tiles that were underneath the scorpion with good ones...we won't penalize them
		if(g_ScorpionHeadPosX >= 0 && g_ScorpionHeadPosX < SCREEN_TILE_MAP_WIDTH)
			g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH] = TILE_BLANK;
		if(g_ScorpionHeadOverMush) g_MushroomCount--;

		if(g_ScorpionTailPosX >= 0 && g_ScorpionTailPosX < SCREEN_TILE_MAP_WIDTH)
			g_TileMap[g_ScorpionTailPosX + g_ScorpionTailPosY * SCREEN_TILE_MAP_WIDTH] = TILE_BLANK;
		if(g_ScorpionTailOverMush) g_MushroomCount--;

		// Little bit of sound
		g_SoundOverride = SOUND_OVERRIDE_MUSHROOM;
		
		g_PlayerScore += 250;
		
		break;	
	
	}

	// Now check for Missile / Flea collision
	if(g_Flea->State == SPRITE_STATE_ON)
	{
		// We can do this because they are unsigned ints
		if((g_Missile->X - g_Flea->X < 8) &&
		   (g_Missile->Y - g_Flea->Y < 8))
			{
				g_Missile->State = SPRITE_STATE_OFF;	
				g_Flea->State = SPRITE_STATE_OFF;	
				g_PlayerScore += 200;			

				g_SoundOverride = SOUND_OVERRIDE_MUSHROOM;
				// Eventually display a score sprite?
			}
	}

	// Now check for Missile / Spider collision
	if(g_SpiderLeft->State == SPRITE_STATE_ON)
	{
		// We can do this because they are unsigned ints
		if((g_Missile->X - g_SpiderLeft->X < 14) &&
		   (g_Missile->Y - g_SpiderLeft->Y < 8))
			{
				g_Missile->State = SPRITE_STATE_OFF;	
				g_SpiderLeft->State = SPRITE_STATE_OFF;	
				g_SpiderRight->State = SPRITE_STATE_OFF;	

				// Score depends on Y position of the spider
				if(g_SpiderLeft->Y > 170)
					g_PlayerScore += 900;			
				else if(g_SpiderLeft->Y > 130)
					g_PlayerScore += 600;			
				else
					g_PlayerScore += 300;			

				g_SoundOverride = SOUND_OVERRIDE_MUSHROOM;

				// Eventually display a score sprite?
			}
	}

	// Move up the screen
	g_Missile->Y -= 4;

	// Check for off screen
	if(g_Missile->Y < TILE_HEIGHT)
		g_Missile->State = SPRITE_STATE_OFF;

}


void ChkPlayerCollide()
{
	int Collided = 0;

	// Spider - Player
	if(g_SpiderLeft->State == SPRITE_STATE_ON)
	{
		// We can do this because they are unsigned ints
		if((((g_Player->X - g_SpiderRight->X) < 7) || ((g_SpiderRight->X - g_Player->X) < 14 )) &&
		   (abs((int)g_Player->Y - (int)(g_SpiderLeft->Y+4)) < 5))
		{
			g_SpiderLeft->State = SPRITE_STATE_OFF;
			g_SpiderRight->State = SPRITE_STATE_OFF;
			Collided = 1;	
		}	
	}

	// Flea - Player
	if(g_Flea->State == SPRITE_STATE_ON)
	{
		if((abs((int)g_Player->X - (int)g_Flea->X) < 8) &&
		   (abs((int)g_Player->Y - (int)g_Flea->Y) < 8))
		{
			g_Flea->State = SPRITE_STATE_OFF;
			Collided = 1;	
		}	
	}

	// Centipede - Player
	int XIndex = (g_Player->X - 5)/8;
	int YIndex = (g_Player->Y - 8)/8;
	if(YIndex < 0) YIndex = 0;
	if(XIndex < 0) XIndex = 0;
	unsigned char TileType = g_TileMap[YIndex*SCREEN_TILE_MAP_WIDTH + XIndex];
	if((TileType >= TILE_CENTI_LEFT_1 && TileType <= TILE_CENTI_RIGHT_2) ||
	   (TileType == TILE_HEAD_LEFT) ||
	   (TileType == TILE_HEAD_RIGHT))
		Collided = 1;

	if(Collided)
	{
		g_SoundOverride = SOUND_OVERRIDE_EXPLOSION;
		g_GameState = CENT_STATE_DYING;
		g_Player->PixelData = &g_SpriteData[SPRITE_EXPLOSION_1*64];
		g_Player->Attribute = 15;	//  ticks of animation

		g_Missile->State = SPRITE_STATE_OFF;
		g_Flea->State = SPRITE_STATE_OFF;
		g_SpiderLeft->State = SPRITE_STATE_OFF;
		g_SpiderRight->State = SPRITE_STATE_OFF;
	}
}



// Starts the centipede pieces if there are any left to start
void Centi_Start()
{
	// Check for error
	if(g_NumCentiStarted == MAX_NUMBER_CENTIPEDES) return;

	// Look for a blank position
	if(g_TileMap[SCREEN_TILE_MAP_WIDTH + SCREEN_TILE_MAP_WIDTH/2] == TILE_BLANK)
	{
		// We have an empty slot
		g_Centipede[g_NumCentiStarted].XTile = SCREEN_TILE_MAP_WIDTH / 2;
		g_Centipede[g_NumCentiStarted].YTile = 1;
		g_Centipede[g_NumCentiStarted].XDir = -1;
		g_Centipede[g_NumCentiStarted].YDir = 1;
		g_Centipede[g_NumCentiStarted].AnimationFrame = 0;
		g_Centipede[g_NumCentiStarted].AnimationCtr = g_CentiSpeed;
		g_Centipede[g_NumCentiStarted].IsHead = (g_NumCentiStarted == 0 ? 1 : 0);
		g_Centipede[g_NumCentiStarted].IsDead = 0;
		g_Centipede[g_NumCentiStarted].IsPoisoned = 0;

		if(g_Centipede[g_NumCentiStarted].IsHead)
			g_TileMap[g_Centipede[g_NumCentiStarted].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[g_NumCentiStarted].YTile] = TILE_HEAD_LEFT;
		else
			g_TileMap[g_Centipede[g_NumCentiStarted].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[g_NumCentiStarted].YTile] = TILE_CENTI_LEFT_1;

		g_NumCentiStarted++;
	}
}


void Centi_Update()
{
	int i;

	// Loop through all the alive centipedes and move them based off their animation frame
	// and counter
	for(i=0; i < MAX_NUMBER_CENTIPEDES && i < g_NumCentiStarted; i++)
	{
		// Simple check to see if they are dead
		if(g_Centipede[i].IsDead) continue;

		if(--g_Centipede[i].AnimationCtr == 0)
		{
			// Update animation frame
			g_Centipede[i].AnimationFrame = (g_Centipede[i].AnimationFrame+1)%2;
			g_Centipede[i].AnimationCtr = g_CentiSpeed;

			// Check to see if we are back at zero and now need to move the centipede
			if(!g_Centipede[i].AnimationFrame)
			{
				// First erase current frame
				g_TileMap[g_Centipede[i].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[i].YTile] = TILE_BLANK;

				// Check to see if this guy is poisioned
				if(g_Centipede[i].IsPoisoned)
				{
					// Always going down and shuffling right and left
					g_Centipede[i].YTile += 1;
					if(g_Centipede[i].YTile & 0x1) // odd rows move to the right
						g_Centipede[i].XTile++;
					else
						g_Centipede[i].XTile--;

					// Now check for hitting the bottom of the screen
					if(g_Centipede[i].YTile > SCREEN_TILE_MAP_HEIGHT - 2)
					{
						g_Centipede[i].YDir = -1;
						g_Centipede[i].YTile -= 2;
						g_Centipede[i].IsPoisoned = 0; // remove poision at bottom
					}

					// Finally see if we are going to overwrite a mushroom and if so decrement the mushroom count
					unsigned char TileType = g_TileMap[g_Centipede[i].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[i].YTile];
					if((TileType <= TILE_HALF_POIS_MUSH) ||
					   (TileType >= TILE_SOLID_GREEN_MUSH && TileType <= TILE_HALF_GREEN_MUSH))
					{
						// Clear it out
						g_TileMap[g_Centipede[i].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[i].YTile] = TILE_BLANK;
						g_MushroomCount--;
					}

				}
				else
				{
					// Normal update state

					// Now update X position and Y position based on direction
					g_Centipede[i].XTile += g_Centipede[i].XDir;
	
					// Hit can be either off screen or hits a mushroom
					int Hit = 0;
					if(g_Centipede[i].XTile < 0) Hit = 1;				
					else if(g_Centipede[i].XTile >= SCREEN_TILE_MAP_WIDTH) Hit = 1;				
					else if(g_TileMap[g_Centipede[i].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[i].YTile] != TILE_BLANK)
					{
						Hit = 1;
						if(g_TileMap[g_Centipede[i].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[i].YTile] <= TILE_HALF_POIS_MUSH)
							g_Centipede[i].IsPoisoned = 1;
					}
	
	
					// Move centipede but first check for edge conditions
					if(Hit && g_Centipede[i].XDir < 0) // Hit while traveling left
					{
						g_Centipede[i].XDir = 1;
						g_Centipede[i].XTile += g_Centipede[i].XDir;
						g_Centipede[i].YTile += g_Centipede[i].YDir;
	
						// Check for Y direction and Y tile
						if(g_Centipede[i].YTile > SCREEN_TILE_MAP_HEIGHT - 2)
						{
							g_Centipede[i].YDir = -1;
							g_Centipede[i].YTile -= 2;
						}
						else if(g_Centipede[i].YDir < 0 && g_Centipede[i].YTile < SCREEN_TILE_MAP_HEIGHT - 8)
						{
							g_Centipede[i].YDir = 1;
							g_Centipede[i].YTile += 2;
						}
	
						// Now keep on traveling in the direction until we hit a blank tile hopefully we don't go off the other screen
						while(g_TileMap[g_Centipede[i].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[i].YTile] != TILE_BLANK)
							g_Centipede[i].XTile += g_Centipede[i].XDir;
					}
					else if(Hit && g_Centipede[i].XDir > 0) // Hit while traveling right
					{
						g_Centipede[i].XDir = -1;
						g_Centipede[i].XTile += g_Centipede[i].XDir;
						g_Centipede[i].YTile += g_Centipede[i].YDir;
	
						// Check for Y direction and Y tile
						if(g_Centipede[i].YTile > SCREEN_TILE_MAP_HEIGHT - 2)
						{
							g_Centipede[i].YDir = -1;
							g_Centipede[i].YTile -= 2;
						}
						else if(g_Centipede[i].YDir < 0 && g_Centipede[i].YTile < SCREEN_TILE_MAP_HEIGHT - 8)
						{
							g_Centipede[i].YDir = 1;
							g_Centipede[i].YTile += 2;
						}
	
						// Now keep on traveling in the direction until we hit a blank tile hopefully we don't go off the other screen
						while(g_TileMap[g_Centipede[i].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[i].YTile] != TILE_BLANK)
							g_Centipede[i].XTile += g_Centipede[i].XDir;
					}
	
					// Check for spawn if at 0,0 and a head
					if(g_Centipede[i].XTile == (SCREEN_TILE_MAP_WIDTH-1) && 
					   g_Centipede[i].YTile == (SCREEN_TILE_MAP_HEIGHT - 2) 
						&& g_Centipede[i].IsHead == 1)
					{
						// Spawn a dead centipede piece higher up
						int j;
						for(j=0; j < g_NumCentiStarted && j < MAX_NUMBER_CENTIPEDES; j++)
						{
							if(g_Centipede[j].IsDead)
							{
								g_Centipede[j].IsDead = 0; 
								g_Centipede[j].XTile = 0; 
								g_Centipede[j].YTile = SCREEN_TILE_MAP_HEIGHT - 8; 
								g_Centipede[j].XDir = 1;
								g_Centipede[j].YDir = 1;
								g_Centipede[j].IsHead = 1; 
								g_TileMap[g_Centipede[j].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[j].YTile] = TILE_HEAD_RIGHT;
								g_NumDeadCenti--;
								break;
							}	
						} // done for loop
					} // done check for spawn
				} // done not poisioned update
			} // end update position

			// Finally update with an appropriate tile
			if(g_Centipede[i].IsHead)
			{
				if(g_Centipede[i].XDir > 0)
					g_TileMap[g_Centipede[i].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[i].YTile] = TILE_HEAD_RIGHT;
				else
					g_TileMap[g_Centipede[i].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[i].YTile] = TILE_HEAD_LEFT;
			}
			else
			{
				if(g_Centipede[i].XDir > 0)
					g_TileMap[g_Centipede[i].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[i].YTile] = TILE_CENTI_RIGHT_1 + g_Centipede[i].AnimationFrame;
				else
					g_TileMap[g_Centipede[i].XTile + SCREEN_TILE_MAP_WIDTH * g_Centipede[i].YTile] = TILE_CENTI_LEFT_1 + g_Centipede[i].AnimationFrame;
			}

		}// end animation counter == 0
	} // end for loop over all centipedes
}


// Start the scorpion
void Scorpion_Start()
{
	g_ScorpionAlive = 1;
	g_ScorpionTailPosY = g_ScorpionHeadPosY = RAND_RANGE(4, 15);

	g_ScorpionHeadPosX = SCREEN_TILE_MAP_WIDTH-1;
	g_ScorpionTailPosX = SCREEN_TILE_MAP_WIDTH; // Off screen at first
	g_ScorpionDir = 1;

	// See if new head position is over a mushroom
//	if((g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH] >= TILE_SOLID_GREEN_MUSH &&
//       (g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH] <= TILE_HALF_GREEN_MUSH)) ||
// 	    g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH] < TILE_HALF_POIS_MUSH)
//		   g_ScorpionHeadOverMush = g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH];
//	else
//		g_ScorpionHeadOverMush = 0;
	g_ScorpionHeadOverMush = g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH];

	g_ScorpionTailOverMush = 0;

	// Now draw the head onto the screen 
	g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH] = TILE_SCORPION_HEAD;

	// Animation and movement frame counter
	g_ScorpionFrameCtr = g_ScorpionSpeed;
	g_ScorpionAnimFrame = 1;
}

// Update the scorpion position
void Scorpion_Update()
{
	if(--g_ScorpionFrameCtr == 0)
	{
		if(--g_ScorpionAnimFrame == 0)
		{
			// Clear out tail tile/drop poision mushroom, but only if its on screen
			if(g_ScorpionTailPosX < SCREEN_TILE_MAP_WIDTH)
			{
				//g_TileMap[g_ScorpionTailPosX + g_ScorpionTailPosY * SCREEN_TILE_MAP_WIDTH] = g_ScorpionTailOverMush ? g_ScorpionTailOverMush - TILE_SOLID_GREEN_MUSH : TILE_BLANK;
				
				// Now determine which tile to replace
				switch(g_ScorpionTailOverMush)
				{
				case TILE_SOLID_GREEN_MUSH: g_ScorpionTailOverMush = TILE_SOLID_POIS_MUSH; break;
				case TILE_RIGHT_GREEN_MUSH: g_ScorpionTailOverMush = TILE_RIGHT_POIS_MUSH; break;
				case TILE_LEFT_GREEN_MUSH: g_ScorpionTailOverMush = TILE_LEFT_POIS_MUSH; break;
				case TILE_HALF_GREEN_MUSH: g_ScorpionTailOverMush = TILE_HALF_POIS_MUSH; break;
				case TILE_CENTI_LEFT_1:	
				case TILE_CENTI_LEFT_2:	
				case TILE_CENTI_RIGHT_1:
				case TILE_CENTI_RIGHT_2:	
				case TILE_HEAD_LEFT:
				case TILE_HEAD_RIGHT:
					g_ScorpionTailOverMush = TILE_BLANK;
				}

				g_TileMap[g_ScorpionTailPosX + g_ScorpionTailPosY * SCREEN_TILE_MAP_WIDTH] = g_ScorpionTailOverMush;
			}

			// Time to shift the scorpion one whole tile
			g_ScorpionHeadPosX--;
			g_ScorpionTailPosX--;

			// Pass along the head mushroom information
			g_ScorpionTailOverMush = g_ScorpionHeadOverMush;
			

			// Draw head
			if(g_ScorpionHeadPosX >= 0)
			{
				// See if new head position is over a mushroom
				//if((g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH] >= TILE_SOLID_GREEN_MUSH &&
//			       (g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH] <= TILE_HALF_GREEN_MUSH)) ||
//			 	    g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH] < TILE_HALF_POIS_MUSH)
//					   g_ScorpionHeadOverMush = g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH];
//				else
//					g_ScorpionHeadOverMush = 0;
				g_ScorpionHeadOverMush = g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH];

				g_TileMap[g_ScorpionHeadPosX + g_ScorpionHeadPosY * SCREEN_TILE_MAP_WIDTH] = TILE_SCORPION_HEAD;
			}

			// Check for totally off the screen
			if(g_ScorpionTailPosX < 0)
			{
				g_ScorpionHeadOverMush = 0;
				g_ScorpionTailOverMush = 0;

				// Done
				g_ScorpionAlive = 0;
				return;
			}
			g_TileMap[g_ScorpionTailPosX + g_ScorpionTailPosY * SCREEN_TILE_MAP_WIDTH] = TILE_SCORPION_TAIL_1;	
			g_ScorpionAnimFrame = 2;
		}
		else
		{
			// Animate the tail if its onscreen
			if(g_ScorpionTailPosX < SCREEN_TILE_MAP_WIDTH)
				g_TileMap[g_ScorpionTailPosX + g_ScorpionTailPosY * SCREEN_TILE_MAP_WIDTH] = TILE_SCORPION_TAIL_2;	
		}

		// Set the timer again
		g_ScorpionFrameCtr = g_ScorpionSpeed;
	}
}


// Very rudimentary sound player
void PlaySounds()
{
	// First check to see if there are any sound overrides
	if(g_SoundOverride != SOUND_OVERRIDE_NONE)
	{
		switch(g_SoundOverride)
		{
		case SOUND_OVERRIDE_BLASTER:
			g_CurrentSound = &g_BlasterSound;
			break;
		case SOUND_OVERRIDE_EXPLOSION:
			g_CurrentSound = &g_ExplosionSound;
			break;
		case SOUND_OVERRIDE_MUSHROOM:
			g_CurrentSound = &g_MushroomSound;
			break;
		}

		// Common setup
		g_CurrentSound->PlayPosition = 0;
		g_CurrentSound->CurrentNoteDuration = 0;
		SND_PlayTone(g_CurrentSound->Notes[0]);

		// Shut off the override flag
		g_SoundOverride = SOUND_OVERRIDE_NONE;
	}

	if(g_CurrentSound == 0)
	{
		if(g_GameState == CENT_STATE_PLAY)
		{
			// Find a new note to play
			if(g_ScorpionAlive)
				g_CurrentSound = &g_ScorpionSound;
			else if(g_Flea->State == SPRITE_STATE_ON)
				g_CurrentSound = &g_FleaSound;
			else if(g_SpiderLeft->State == SPRITE_STATE_ON)
				g_CurrentSound = &g_SpiderSound;
			else
				g_CurrentSound = &g_GameSound;

			g_CurrentSound->PlayPosition = 0;
			g_CurrentSound->CurrentNoteDuration = 0;
			SND_PlayTone(g_CurrentSound->Notes[0]);
		}
		else
			return;
	}

	// Continue playing the sound
	if(++g_CurrentSound->CurrentNoteDuration == g_CurrentSound->NotesDuration[g_CurrentSound->PlayPosition])
	{
		// Move onto the next note, but first check to see if we are done with the sound
		if(++g_CurrentSound->PlayPosition == g_CurrentSound->NumberNotes)
		{
			// Done with this note
			g_CurrentSound = 0;
			SND_PlayTone(0);
			return;
		}

		// Reset the duration
		g_CurrentSound->CurrentNoteDuration = 0;
		SND_PlayTone(g_CurrentSound->Notes[g_CurrentSound->PlayPosition]);
	}
}

