;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/         
;/ Filename: JTC_NTSC_240_192_v001.s
;/
;/ Author: JT Cook - http://www.avalondreams.com
;/ 
;/ Description: 240x192 tile driver (30x24 tile map)
;/ Currently only supports tiles
;/ Does not use palette, only reads colors from tiles
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ INCLUDES 
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; Include our register and chip definitions file
.include "p24HJ256GP206.inc"


;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ DEFINES
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; width and height of screen 
.equ NTSC_SCREEN_WIDTH,     160
.equ NTSC_SCREEN_HEIGHT,    192

; controls the number of lines to shift the display vertically, later assign to global variable, so caller can control
.equ NTSC_VERTICAL_OFFSET,  40

; number of bits each pixel is encoded with
.equ NTSC_PIXELS_PER_BYTE,  4            

; Number of clocks per pixel (12 in the case of NTSC) Modifying this won't really change anything,
; you will need to change code delays to get the correct clocks per pixel change
.equ CLOCKS_PER_PIXEL, 8

; Number of lines we should repeat to reduce the vertical resolution
.equ LINE_REPT_COUNT,   1

; with the current D/A converter range 0 .. 15, 1 = 62mV
.equ NTSC_SYNC,           (0x0F)     ; 0V  
.equ NTSC_BLACK,          (0x4F)     ; 0.3V (about 30-40 IRE)

.equ NTSC_CBURST0,        (0x30) 
.equ NTSC_CBURST1,        (0x31) 
.equ NTSC_CBURST2,        (0x32) 

.equ NTSC_CHROMA_OFF, 	0x000F
.equ NTSC_OVERSCAN_CLR,	0x005F

.equ COLOR_LUMA,    (NTSC_CBURST1 + 0x40)

.equ NTSC_GREEN,    (COLOR_LUMA + 0)        
.equ NTSC_YELLOW,   (COLOR_LUMA + 2)        
.equ NTSC_ORANGE,   (COLOR_LUMA + 4)        
.equ NTSC_RED,      (COLOR_LUMA + 5)        
.equ NTSC_VIOLET,   (COLOR_LUMA + 9)        
.equ NTSC_PURPLE,   (COLOR_LUMA + 11)        
.equ NTSC_BLUE,     (COLOR_LUMA + 14)        

.equ NTSC_GRAY0,    (NTSC_BLACK + 0x10)        
.equ NTSC_GRAY1,    (NTSC_BLACK + 0x20)        
.equ NTSC_GRAY2,    (NTSC_BLACK + 0x30)        
.equ NTSC_GRAY3,    (NTSC_BLACK + 0x40)        
.equ NTSC_GRAY4,    (NTSC_BLACK + 0x50)        
.equ NTSC_GRAY5,    (NTSC_BLACK + 0x60)        
.equ NTSC_GRAY6,    (NTSC_BLACK + 0x70)        
.equ NTSC_GRAY7,    (NTSC_BLACK + 0x80)        
.equ NTSC_GRAY8,    (NTSC_BLACK + 0x90)        
.equ NTSC_GRAY9,    (NTSC_BLACK + 0xA0)        
.equ NTSC_WHITE,    (NTSC_BLACK + 0xB0) 



;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ EXTERNALS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

.extern _g_TileMapBasePtr       ; 16-bit pointer to tile map, allows indirection
.extern _g_SpriteList           ; 16-bit pointer to sprite table records
.extern _g_TileSet              ; 16-bit pointer to 8x8 tile bitmaps, allows indirection

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ SRAM SECTIONS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; The following section describes initialized (data) near section. The reason we want
; to place these variables into the near SRAM area is so that ASM codes, that don't use many
; bits to encode data, will be able to access the variables.
.section *,data,near
_g_CurrentLine:		.int 245		        ; Line count 0->261
_g_RasterLine:      .int 0                  ; Virtual Raster count 0->96
ReptLine:	        .int LINE_REPT_COUNT	; Repeated line counter 0->2
CurrVRAMPtr:	    .int 0			        ; Line count 0->525
CurrAttribPtr:	    .int 0			        ; Points to g_AttributeBuf that is defined in the C program as a global
_g_VsyncFlag:       .int 0			        ; Flag to indicate we are in v-sync
ReptAttrib:	        .int 8			        ; Repeat attribute table increment 8 pixel lines

; Un-initialized Data sections
.section *,bss,near
AttribScanLine: .space ((NTSC_SCREEN_WIDTH/8)*2)	; Contains pointers to the palette table for current scanline.


;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ GLOBAL EXPORTS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

.global __T1Interrupt   ; Declare Timer 1 ISR name global
.global _g_VsyncFlag    ; read only, Flag to indicate we are in v-sync
.global _g_CurrentLine  ; read only, current physical line 0...261
.global _g_RasterLine   ; Virtual Raster count 0->96


;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ ASM FUNCTIONS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; The following are code sections
.text           

__T1Interrupt:

		; Our instruction clock is running at 12*NTSC dot clock
		; so every 12 clock cycles is a normal 227 pixels
		disi #0x3FFF					; (1) Disable interupts for up to 10000 cycles

		; Save off variables we will use
		push.s						; (1) Push status registers
		push SR
		push W4						; (1) Temp variable
		push W5
		push RCOUNT					; (1) Make sure we are pushing Rcount since we are using repeat

		; Increment which line we are on
		inc _g_CurrentLine				; (1)
		mov _g_CurrentLine, W0				; (1) Set our line variable into W0

		; Test if we are done with this frame (262 lines)
		mov #261, W2					; (1)
		cp W0, W2					; (1) See if we are at line 262 = 0
		bra nz, DONE_LINE_TEST				; (1/2) if(_g_CurrentLine != 262) jump
		setm _g_CurrentLine				; (1) Reset Line counter set to 0xFFFF
DONE_LINE_TEST: 

		; Total = 10

        ; The following section of code determines what kind of line we should be 
        ; drawing to the screen. Is it a Vsync Line, Black Top/Bottom, or an
        ; Active Video line. All of this is determined by the _g_CurrentLine counter
        ; variable

        ; if(_g_CurrentLine < VSYNC_LINE_COUNT)
		mov #5, W2					; (1)
		cp W0, W2					; (1) Compare to 5
		bra gt, NOT_VSYNC_LINE				; (1/2) if(Line > 5) jump

		; Vsync line (18)
		repeat	#11					; (1)
		nop						; (1+n)
		goto DRAW_VSYNC_LINE				; (2) draw a black line to the screen

NOT_VSYNC_LINE:

		; Test to see if this is a top blanking line
		mov #NTSC_VERTICAL_OFFSET, W2			; (1)
		cp W0, W2					; (1) Compare to NTSC_VERTICAL_OFFSET
		bra ge, NOT_TOP_OVERSCAN_LINE			; (1/2) if(Line >= NTSC_VERTICAL_OFFSET) jump

		; Blank Line Top (18)
		nop						; (1)
        nop							; (1)
        clr _g_RasterLine                          		; (1) Clear out our raster line count
		mov _g_CurrentVRAMBuff, W1			; (1) Copy over CurrVRAMPtr = _g_CurrentVRAMBuff
		mov W1, CurrVRAMPtr				; (1) copy
		mov _g_AttribTable, W1				; (1) Copy over ATTRIBTABLE = g_AttributeTable
		mov W1, CurrAttribPtr				; (1) copy
		mov #LINE_REPT_COUNT, W1			; (1) Reset repeat line back to count
		mov W1, ReptLine							; (1)
		goto DRAW_OVERSCAN_LINE				; (2) draw a black line to the screen

NOT_TOP_OVERSCAN_LINE:

		; Test to see if this is a bottom blanking line
		mov #((NTSC_SCREEN_HEIGHT*LINE_REPT_COUNT)+NTSC_VERTICAL_OFFSET), W2	; (1)
		cp W0, W2					; (1) Compare to 245
		bra lt, NOT_BOTTOM_OVERSCAN_LINE		; (1/2) if(Line < 245) jump

		; Blank Line Bottom (18)
		repeat	#2					; (1)
		nop						; (1+n)
		setm _g_VsyncFlag				; (1) set the vsync flag 
		goto DRAW_OVERSCAN_LINE				; (2) draw a black line to the screen

NOT_BOTTOM_OVERSCAN_LINE:

		; This must be an active line (18)
		clr _g_VsyncFlag				; (1) g_VsyncFlag = 0
		nop						; (1) delay
		nop						; (1) delay
		nop						; (1) delay
		goto DRAW_ACTIVE_LINE				; (2) draw an active line to the screen



; ----------------- VSYNC LINE -------------------------
DRAW_VSYNC_LINE:

		; This draws a vsync line with h-sync pulse inverted for serration
		; H-sync serration (4.7us) 236
		mov #NTSC_BLACK,W0 	                        ; (1) turn hsync on
		mov W0, LATB					; (1)
		repeat #232					; (1)
		nop						; (1+n)
		mov #NTSC_SYNC, W0	                        ; (1)
		mov W0, LATB					; (1)

		; Done with this completely
		goto FINISHED_NTSC		; Dont need to count because interrupts will keep us locked



; ----------------- OVERSCAN LINE -------------------------
DRAW_OVERSCAN_LINE:

		; This draws a overscan line with h-sync (used for blanking lines)
		; H-sync (4.7uS) = 202
		mov #NTSC_SYNC, W0	                        ; (1) turn hsync on
		mov W0, LATB					; (1)
		repeat #198					; (1)
		nop						; (1+n)

		; Pre-burst (.6uS) = 26 clocks
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
		repeat #22					; (1)
		nop						; (1+n)


		; Color burst (10 NTSC clocks) = 120 clocks
		mov #NTSC_CBURST1, W0				; (1) turn hsync color burst on at phase 0
		mov W0, LATB					; (1)
		;repeat #116					; (1)
		repeat #(108-4)				; (1)
		nop						; (1+n)


		; Post-burst (.6uS) = 26 clocks
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
		repeat #22					; (1)
		nop						; (1+n)


		; Overscan color
		mov #NTSC_OVERSCAN_CLR, W0			; (1) turn hsync off black with a color signal
		mov W0, LATB					; (1)

		; Done with this completely
		goto FINISHED_NTSC				; Dont need to count because interrupts will keep us locked




; ----------------- ACTIVE LINE -------------------------
DRAW_ACTIVE_LINE:


		; ------- HSYNC ON ---------
		; 43 * X.XuS = clock cycles
		; H-sync (4.7uS) = 202
		mov #NTSC_SYNC, W0 	                        ; (1) turn hsync on
		mov W0, LATB					; (1)
		repeat #198					; (1)
		nop						; (1+n)

		; Pre-burst (.6uS) = 26 clocks
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
		repeat #22					; (1)
		nop						; (1+n)
        ;43
        ; 2.5uS - 108 clock cycles
		; Color burst (10 NTSC clocks) = 80 clocks (originally 120 clocks)
		mov #NTSC_CBURST1, W0 		        ; (1) turn hsync color burst on at phase 0
		mov W0, LATB				; (1)
		repeat #(108-4)				; (1)
		nop					; (1+n)
        ; Post-burst 1.6uS - 69
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB			; (1)
		repeat #(69-4)			; (1)
		nop						; (1+n)

		;Overscan 
		; FROM HERE ON it is imparative that we stay in sync with the color clock signal
		; (i.e. we don't change any pixels unless it is an integer multiple of 8)
		; If we don't do this then color changes will not be correct.
		; Post burst
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
		repeat #(16-4)					; (1)
		nop						; (1+n)


		; Left overscan color
		mov #(NTSC_OVERSCAN_CLR | NTSC_CHROMA_OFF), W0 ; (1) turn hsync off black with no color signal
		mov W0, LATB					; (1)
		
PIXEL_LOOP:     ;1920 clocks from overscan-to-overscan
;-------------------------------------------------------------------------------------------------
;256 = 7 clocks a pixel (=1792 clocks)
;240 = 8 clocks a pixel
;192 = 10 clocks a pixel
;160 = 12 clocks a pixel
; W0 - color to write to screen
; W1 - pointer to TileSet
; W2 - counts how many pixels we have left to draw in the scanline
; W3 - counter for how many pixels we have left to draw for tile
; W4 - horizontal offset for tile
; W5 - address for current tile map
; W6 - address for current tile map * 64 (tile offset)
; W7 - byte for tile from tilemap
;----- Set up scanline for tiles (15 cycles)
	    mov #240, W2; (1) pixel counter for number of pixels to draw
	    ;mov #192, W2; (1) pixel counter for number of pixels to draw
		mov _g_TileSet, W1			; (1) Copy over CurrVRAMPtr = _g_CurrentVRAMBuff
        mov #0, W3; (1)
        mov _g_RasterLine, W4 ; (1) grab current scanline
        and #0x07, W4         ;(1) mask it off so we can use it for tiles
        sl W4, #3, W4            ;(1) multiply by 8 for offset
        add W1, W4, W1            ;(1) set new offset for tile
        mov _g_RasterLine, W6     ;(1) get current scanline to calc tile map address
		asr W6, #3, W6            ;(1) divide by 8 to get the tile row
		mov #30, W5                ;(1) Value we are multiplying by
		mul.us W6, W5, W6        ;(1) multiply by 30 (number of horizontal tiles)
		mov W6, W5               ;(1) move back to original register
        mov _g_TileMapBasePtr, W6 ;(1) for adding
		add W5, W6, W5        ;(1) get address for tile map
		mov #0, W7 ;(1) clear upper 8 bits of W6 register for tile byte (make sure it is only an 8 bit value)
        ;;finish out overscan
		repeat #(120-15-4)        ; (1)
		nop						; (1+n)

;---- Place rendering engine here (1920 free clocks)
Draw_Pixel:
	    ;8 clocks a pixel (240 pixels)
		; unroll loop for a full tile (8 pixels)
        ;pixel 0---------------------
		mov.b [W5], W7 ; (1) get tile we are rendering from tile map
        sl W7, #6, W6;   (1) multiply by 64 (each tile is 64 bytes)
        add W6, W1, W6  ;(1) combine tile graphics offset to tile image
        mov.b [W6],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        add #1, W5     ;(1) advance to next tile
		nop ;(1)
        nop ;(1)
        ;pixel 1---------------------
        nop ;(1)
        nop ;(1)
	    nop ;(1)
        mov.b [W6+1],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
        mov W0, LATB ; (1) render pixel signal to screen
        nop ;(1)
		nop ;(1)
        nop ;(1)
        ;pixel 2---------------------
        nop ;(1)
        nop ;(1)
        nop ;(1)
        mov.b [W6+2],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
		mov W0, LATB ; (1) render pixel signal to screen
        nop ;(1)
		nop ;(1)
        nop ;(1)
        ;pixel 3---------------------
        nop ;(1) 
        nop ;(1) 
        nop ;(1)
        mov.b [W6+3],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
		mov W0, LATB ; (1) render pixel signal to screen
		nop ;(1)
        nop ;(1)
        nop ;(1)
        ;pixel 4---------------------
        nop ;(1)
        nop ;(1)
        nop ;(1)
        mov.b [W6+4],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
		mov W0, LATB ; (1) render pixel signal to screen
		nop ;(1)
        nop ;(1)
        nop ;(1)
        ;pixel 5---------------------
        nop ;(1) 
        nop ;(1) 
        nop ;(1)
        mov.b [W6+5],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
		mov W0, LATB ; (1) render pixel signal to screen
		nop ;(1)
        nop ;(1)
        nop ;(1)
        ;pixel 6---------------------
        nop ;(1) 
        nop ;(1) 
        nop ;(1)
        mov.b [W6+6],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
		mov W0, LATB ; (1) render pixel signal to screen
        nop ;(1)
		nop ;(1)
        nop ;(1)
        ;pixel 7---------------------
        nop ;(1)
        nop ;(1)
		nop ;(1)
        mov.b [W6+7],W0  ;(1) get pixel by taking pointer to tileset, adding pixel number for tile, adding tile from time map
		mov W0, LATB ; (1) render pixel signal to screen
        sub W2, #8, W2;  (1) - subtract 8 from amount of pixels left to draw
		bra nz, Draw_Pixel	; (1/2) check to see if we draw more pixels

;----
;-------------------------------------------------------------------------------------------------
		; Now set PORTB to overscan
		mov #(NTSC_OVERSCAN_CLR | NTSC_CHROMA_OFF), W0 ; (1) turn hsync off black with no color signal
		nop				; (1)
		nop				; (1)
		mov W0, LATB			; (1)

		; Now increment our framepointer if we are done with this line (every 2 lines)
		dec ReptLine			; (1)
		bra nz, FINISHED_NTSC		; (1/2)
		mov CurrVRAMPtr, W0		; (1)
		add #(NTSC_SCREEN_WIDTH/NTSC_PIXELS_PER_BYTE), W0	; (1) increment our pointer to the next line 
		mov W0, CurrVRAMPtr		; (1) save
		mov #LINE_REPT_COUNT, W0	; (1) 
		mov W0, ReptLine		; (1) Reset our line repeate counter
        inc _g_RasterLine                       ; (1) Increment our raster line count
		
		; Now test if we should increment our attribute table pointer. We only do this
		; everly 8 pixel lines so that our attribute table covers 8x8 tiles in a sense.
		dec ReptAttrib			; (1)
		bra nz, FINISHED_NTSC		; (1/2)
		mov #((NTSC_SCREEN_WIDTH/8)*2), W0; (1) increment pointer by sizeof(int)*20 tiles
		add CurrAttribPtr		; (1) increment it
		mov #8, W0			; (1)
		mov W0, ReptAttrib		; (1)



		; Done with this completely
FINISHED_NTSC:

        bclr IFS0, #T1IF           		; Clear the Timer1 Interrupt flag Status
                                   		; bit.

		pop RCOUNT			; (1) Make sure we are pushing Rcount since we are using repeat
		pop W5
		pop W4                   	; Retrieve context POP-ping from Stack
		pop SR
		pop.s				; (1) Push status registers

		disi #0x0			; Re-enable interrupts

        RETFIE                     		;Return from Interrupt Service routine




;--------End of All Code Sections ---------------------------------------------

.end                               ;End of program code in this file
        