;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ 
;/ Copyright Nurve Networks LLC 2008
;/ 
;/ Filename: XGS_PIC_NTSC_160_192_2_V010.S
;/
;/ Original Author: Joshua Hintze
;/ 
;/ Description: 160x192 resolution NTSC driver with 4-bits per pixel and multiple color palettes, thus
;/ every 4-bit pixel is used as an index into the color palette for that particular pixel's 8x8 group palette as shown 
;/ below: 
;/
;/                                Color Palette for 8x8 block
;/                                                    
;/ Pixel Value     Palette Index             Index      Color Encoding
;/   00       =         0    ------------->   0   - [ LUMA3..0 : CHROMA3..0 ] - 8-bit color (some values for sync etc.)
;/   01       =         1    ------------->   1   - [ LUMA3..0 : CHROMA3..0 ] - " "
;/   10       =         2    ------------->   2   - [ LUMA3..0 : CHROMA3..0 ] - " "
;/   11       =         3    ------------->   3   - [ LUMA3..0 : CHROMA3..0 ] - " "
;/
;/ Memory for bitmap modes is laid out from top left corner of the screen from left to right, top to bottom.
;/ Each byte represents 4 pixels where pixel 0,1,2,3 on screen are encoded frow high bit to low bit as shown below:
;/ 
;/ Pixel Encoding
;/
;/     P0  |   P1  |   P2  |   P3     <------ pixel #
;/ [ b7 b6 | b5 b4 | b3 b2 | b1 b0 ]  <------ bit encodings
;/ msb                            lsb        
;/ 
;/                Video RAM Layout
;/ (0,0).....................................(159,0)
;/ .P0 P1 ...                                 P159
;/ .
;/ .
;/ .
;/ .
;/ .
;/ .
;/ (0,191)...................................(159, 191)                          
;/ 
;/ For example, if you wanted to change the color of the pixel (0,0), then you would write the upper 4-bits
;/ of the very first byte of Video RAM.
;/
;/ The interface between the C caller and this driver is accomplished thru a pointer to VRAM from C,
;/ the palette base address exported back to C from this driver, along with a few status variables
;/ from the driver to interogate the current line, etc. See below:
;/
;/ There is a single pointer to VRAM, the driver expects the C caller to allocate memory and then 
;/ initialize a pointer with the following name to point to the VRAM:
;/ 
;/ .extern _g_CurrentVRAMBuff      // 16-bit pointer to video RAM. Note that C code strips the leading "_" underscore
;/                                 // so it is not needed in C code
;/
;/ .extern _g_ColorTable           // 16-bit pointer to the 16 entry color table array. C code must provide this
;/ .extern _g_AttribTable          // 16-bit pointer to a int array that contains the index of the 8x8 blocks that maps to the color tables
;/
;/
;/ This driver uses an Attribute Table simular to the Nintendo's PPU unit on the original NES. Basically this
;/ is just one more level of indirection allowing us to choose a seperate color table for each 8x8 pixel block
;/ by doing this we can have a lot more colors on the screen than just one 16 color table.
;/
;/ The way the attribute table works is that it is filled with an index to an array of color tables. If the index is 0
;/ then the color table that the pixel grouping will use is the first table. If the index is 1, then its the second table
;/ located 16 bytes after the first color table. Note: All color tables must be continous in memory.
;/
;/ In the case of this video driver being 160x192 we have 20x24 different attribute indices. As shown below
;/ ---------------------------------------------
;/ |   8x8  |   8x8  |   8x8  | ........|   8x8  |
;/ | Att. 0 | Att. 1 | Att. 2 |.........| Att. 19|
;/ | _ _ _ __ _ _ __ _ _ __ _ _ __ _ _ __ _ _ _ _|
;/ |   8x8  |   8x8  |   8x8  | ........|   8x8  |
;/ | Att. 20| Att. 21| Att. 22|.........| Att. 39|
;/ | _ _ _ __ _ _ __ _ _ __ _ _ __ _ _ __ _ _ _ _|
;/ |        |        |        | ........|        |
;/ |  ....  |  ....  |  ....  |.........|  ....  |
;/ | _ _ _ __ _ _ __ _ _ __ _ _ __ _ _ __ _ _ _ _|
;/ |   8x8  |   8x8  |   8x8  | ........|   8x8  |
;/ | Att.440| Att.441| Att.442|.........| Att.479|
;/ | _ _ _ __ _ _ __ _ _ __ _ _ __ _ _ __ _ _ _ _|
;/
;/ One last thing is that if you want to simplify this, just set the entire Attribute table to 0 and have only
;/ a single color table. In otherwords you can have more than one index pointing to the same table.
;/
;/ Next, the driver passses back the current absolute video line 0..261 via g_CurrentLine and the current
;/ virtual raster line via g_RasterLine. It also passes back a flag such that when we are NOT in Vsync it is 0,
;/ otherwise the VsyncFlag is set all bits high.
;/
;/ .global _g_CurrentLine  ; read only, current physical line 0...524
;/ .global _g_RasterLine   ; read only, current logical raster line
;/ .global _g_VsyncFlag    ; read only, Flag to indicate we are in v-sync

;/
;/ Modifications|Bugs|Additions:
;/
;/ Author Name: 
;/
;/
;/
;/
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ INCLUDES 
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; Include our register and chip definitions file
.include "p24HJ256GP206.inc"


;.include "RGW_PIC_DD_1_V011.s"

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ DEFINES
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; width and height of screen 
.equ NTSC_SCREEN_WIDTH,     160
.equ NTSC_SCREEN_HEIGHT,    192

; controls the number of lines to shift the display vertically, later assign to global variable, so caller can control
.equ NTSC_VERTICAL_OFFSET,  40

; number of bits each pixel is encoded with
.equ NTSC_PIXELS_PER_BYTE,  4            

; Number of clocks per pixel (6 in the case of NTSC) Modifying this won't really change anything,
; you will need to change code delays to get the correct clocks per pixel change
.equ CLOCKS_PER_PIXEL, 6

; Number of lines we should repeat to reduce the vertical resolution
.equ LINE_REPT_COUNT,   1

; with the current D/A converter range 0 .. 15, 1 = 62mV
.equ NTSC_SYNC,           (0x0F)     ; 0V  
.equ NTSC_BLACK,          (0x4F)     ; 0.3V (about 30-40 IRE)

.equ NTSC_CBURST0,        (0x30) 
.equ NTSC_CBURST1,        (0x31) 
.equ NTSC_CBURST2,        (0x32) 

.equ NTSC_CHROMA_OFF, 	0x000F
.equ NTSC_OVERSCAN_CLR,	0x005F

.equ COLOR_LUMA,    (NTSC_CBURST1 + 0x40)

.equ NTSC_GREEN,    (COLOR_LUMA + 0)        
.equ NTSC_YELLOW,   (COLOR_LUMA + 2)        
.equ NTSC_ORANGE,   (COLOR_LUMA + 4)        
.equ NTSC_RED,      (COLOR_LUMA + 5)        
.equ NTSC_VIOLET,   (COLOR_LUMA + 9)        
.equ NTSC_PURPLE,   (COLOR_LUMA + 11)        
.equ NTSC_BLUE,     (COLOR_LUMA + 14)        

.equ NTSC_GRAY0,    (NTSC_BLACK + 0x10)        
.equ NTSC_GRAY1,    (NTSC_BLACK + 0x20)        
.equ NTSC_GRAY2,    (NTSC_BLACK + 0x30)        
.equ NTSC_GRAY3,    (NTSC_BLACK + 0x40)        
.equ NTSC_GRAY4,    (NTSC_BLACK + 0x50)        
.equ NTSC_GRAY5,    (NTSC_BLACK + 0x60)        
.equ NTSC_GRAY6,    (NTSC_BLACK + 0x70)        
.equ NTSC_GRAY7,    (NTSC_BLACK + 0x80)        
.equ NTSC_GRAY8,    (NTSC_BLACK + 0x90)        
.equ NTSC_GRAY9,    (NTSC_BLACK + 0xA0)        
.equ NTSC_WHITE,    (NTSC_BLACK + 0xB0) 



;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ EXTERNALS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

.extern _g_CurrentVRAMBuff      ; 16-bit pointer to the vram that is meant to be drawn
.extern _g_ColorTable           ; 16-bit pointer to the array of color tables.
.extern _g_AttribTable          ; 16-bit pointer to a int array that contains the index of the 8x8 blocks that maps to the color tables
.extern _g_HardImagePtr 		;
.extern _g_HardImagePage		;

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ SRAM SECTIONS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; The following section describes initialized (data) near section. The reason we want
; to place these variables into the near SRAM area is so that ASM codes, that don't use many
; bits to encode data, will be able to access the variables.
.section *,data,near
_g_CurrentLine:		.int 245		        ; Line count 0->261
_g_RasterLine:      .int 0                  ; Virtual Raster count 0->96
ReptLine:	        .int LINE_REPT_COUNT	; Repeated line counter 0->2
CurrVRAMPtr:	    .int 0			        ; Line count 0->525
CurrAttribPtr:	    .int 0			        ; Points to g_AttributeBuf that is defined in the C program as a global
_g_VsyncFlag:       .int 0			        ; Flag to indicate we are in v-sync
ReptAttrib:	        .int 8			        ; Repeat attribute table increment 8 pixel lines

_g_HardImagePtr:  		.int 0					;
_g_HardImagePage: 		.int 0					;

;_g_ImagePtr: .int 1

cycleCount: .int 60*10;
cycleCurr:  .int 60*10;
cycleImage: .int 1;

; Un-initialized Data sections
.section *,bss,near
AttribScanLine: .space ((NTSC_SCREEN_WIDTH/8)*2)	; Contains pointers to the palette table for current scanline.

;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ GLOBAL EXPORTS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

.global __T1Interrupt   ; Declare Timer 1 ISR name global
.global _g_VsyncFlag    ; read only, Flag to indicate we are in v-sync
.global _g_CurrentLine  ; read only, current physical line 0...261
.global _g_RasterLine   ; Virtual Raster count 0->96

.global _g_ImagePtr;

.global HardImagePtr;
.global HardImagePage;
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/ ASM FUNCTIONS
;//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

; The following are code sections
.text           

__T1Interrupt:

		; Our instruction clock is running at 12*NTSC dot clock
		; so every 12 clock cycles is a normal 227 pixels
		disi #0x3FFF								; (1) Disable interupts for up to 10000 cycles

		; Save off variables we will use
		push.s										; (1) Push status registers
		push SR
		push W4										; (1) Temp variable
		push W5
		push RCOUNT									; (1) Make sure we are pushing Rcount since we are using repeat


		; Increment which line we are on
		inc _g_CurrentLine							; (1)
		mov _g_CurrentLine, W0						; (1) Set our line variable into W0

		; Test if we are done with this frame (262 lines)
		mov #261, W2								; (1)
		cp W0, W2									; (1) See if we are at line 262 = 0
		bra nz, DONE_LINE_TEST						; (1/2) if(_g_CurrentLine != 262) jump
		setm _g_CurrentLine							; (1) Reset Line counter set to 0xFFFF

/*
		mov #psvpage(ChipHead_160x192),w0		; Return the page bits from the image address
		mov w0, PSVPAG							; Program the Pgrogram Source View Page bits
		bset.b CORCONL, #PSV					; Turn on Program Source View
		mov #psvoffset(ChipHead_160x192),w0		; Finish with w0 holding correct offset to image display table
		mov w0, HardImagePtr
*/

/*

		mov #psvpage(RedCells),w0		; Return the page bits from the image address
		mov w0, PSVPAG							; Program the Pgrogram Source View Page bits
		bset.b CORCONL, #PSV					; Turn on Program Source View
		mov #psvoffset(RedCells),w0		; Finish with w0 holding correct offset to image display table
		mov w0, HardImagePtr
*/

/*

		mov #psvpage(Fish),w0		; Return the page bits from the image address
		mov w0, PSVPAG							; Program the Pgrogram Source View Page bits
		bset.b CORCONL, #PSV					; Turn on Program Source View
		mov #psvoffset(Fish),w0		; Finish with w0 holding correct offset to image display table
		mov w0, HardImagePtr
*/

		dec cycleCurr
		bra nz, SkipCycle
	    mov cycleCount, w1
		mov w1, cycleCurr
		dec cycleImage

SkipCycle:
		mov #1, w1
		mov cycleImage, w0
		cp w1, w0
		bra nz, Test2
;/*
Test1:
		mov #psvpage(Fish_320),w0					; (1) Return the page bits from the image address
		mov w0, PSVPAG								; (1) Program the Pgrogram Source View Page bits
		mov w0, _g_HardImagePage					; (1)
		bset.b CORCONL, #PSV						; (1) Turn on Program Source View
		mov #psvoffset(Fish_320),w0					; (1) Finish with w0 holding correct offset to image display table
		mov w0, _g_HardImagePtr						; (1)
		bra DONE_LINE_TEST

;*/

Test2:
;/*		

		mov #0, w1
		mov cycleImage, w0
		cp w1, w0
		bra nz, Test3
		;mov #1, w1
		;mov w1,cycleImage

		mov #psvpage(Dragon_320),w0 					; (1) Return the page bits from the image address
		mov w0, PSVPAG								; (1) Program the Pgrogram Source View Page bits
		mov w0, _g_HardImagePage					; (1)
		bset.b CORCONL, #PSV						; (1) Turn on Program Source View
		mov #psvoffset(Dragon_320),w0					; (1) Finish with w0 holding correct offset to image display table
		mov w0, _g_HardImagePtr						; (1)
		bra DONE_LINE_TEST 

;*/

Test3:
		mov #1, w1
		mov w1,cycleImage
		bra Test1




DONE_LINE_TEST: 

		; Total = 10

        ; The following section of code determines what kind of line we should be 
        ; drawing to the screen. Is it a Vsync Line, Black Top/Bottom, or an
        ; Active Video line. All of this is determined by the _g_CurrentLine counter
        ; variable

        ; if(_g_CurrentLine < VSYNC_LINE_COUNT)
		mov #5, W2									; (1)
		cp W0, W2									; (1) Compare to 5
		bra gt, NOT_VSYNC_LINE						; (1/2) if(Line > 5) jump

		; Vsync line (18)
		repeat	#11 									; (1)
		nop											; (1+n)
		goto DRAW_VSYNC_LINE						; (2) draw a black line to the screen

NOT_VSYNC_LINE:

		; Test to see if this is a top blanking line
		mov #NTSC_VERTICAL_OFFSET, W2				; (1)
		cp W0, W2									; (1) Compare to NTSC_VERTICAL_OFFSET
		bra ge, NOT_TOP_OVERSCAN_LINE				; (1/2) if(Line >= NTSC_VERTICAL_OFFSET) jump

		; Blank Line Top (18)
		nop											; (1)
        nop											; (1)
        clr _g_RasterLine                           ; (1) Clear out our raster line count
		mov _g_CurrentVRAMBuff, W1					; (1) Copy over CurrVRAMPtr = _g_CurrentVRAMBuff
		mov W1, CurrVRAMPtr							; (1) copy
		mov _g_AttribTable, W1					    ; (1) Copy over ATTRIBTABLE = g_AttributeTable
		mov W1, CurrAttribPtr						; (1) copy
		mov #LINE_REPT_COUNT, W1					; (1) Reset repeat line back to count
		mov W1, ReptLine							; (1)
		goto DRAW_OVERSCAN_LINE						; (2) draw a black line to the screen

NOT_TOP_OVERSCAN_LINE:

		; Test to see if this is a bottom blanking line
		mov #((NTSC_SCREEN_HEIGHT*LINE_REPT_COUNT)+NTSC_VERTICAL_OFFSET), W2	; (1)
		cp W0, W2									; (1) Compare to 245
		bra lt, NOT_BOTTOM_OVERSCAN_LINE			; (1/2) if(Line < 245) jump

		; Blank Line Bottom (18)
		repeat	#2									; (1)
		nop											; (1+n)
		setm _g_VsyncFlag							; (1) set the vsync flag 
		goto DRAW_OVERSCAN_LINE						; (2) draw a black line to the screen

NOT_BOTTOM_OVERSCAN_LINE:

		; This must be an active line (18)
		clr _g_VsyncFlag							; (1) g_VsyncFlag = 0
		nop											; (1) delay
		nop											; (1) delay
		nop											; (1) delay
		goto DRAW_ACTIVE_LINE						; (2) draw an active line to the screen



; ----------------- VSYNC LINE -------------------------
DRAW_VSYNC_LINE:

		; This draws a vsync line with h-sync pulse inverted for serration
		; H-sync serration (4.7us) 236
		mov #NTSC_BLACK,W0 	                        ; (1) turn hsync on
		mov W0, LATB								; (1)
		repeat #232									; (1)
		nop											; (1+n)
		mov #NTSC_SYNC, W0	                        ; (1)
		mov W0, LATB								; (1)

		; Done with this completely
		goto FINISHED_NTSC							; Dont need to count because interrupts will keep us locked



; ----------------- OVERSCAN LINE -------------------------
DRAW_OVERSCAN_LINE:

		; This draws a overscan line with h-sync (used for blanking lines)
		; H-sync (4.7uS) = 202
		mov #NTSC_SYNC, W0	                        ; (1) turn hsync on
		mov W0, LATB								; (1)
		repeat #198									; (1)
		nop											; (1+n)

		; Pre-burst (.6uS) = 26 clocks
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB								; (1)
		repeat #22									; (1)
		nop											; (1+n)


		; Color burst (10 NTSC clocks) = 120 clocks
		mov #NTSC_CBURST1, W0						; (1) turn hsync color burst on at phase 0
		mov W0, LATB								; (1)
		repeat #116									; (1)
		nop											; (1+n)


		; Post-burst (.6uS) = 26 clocks
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB								; (1)
		repeat #22									; (1)
		nop											; (1+n)


		; Overscan color
		mov #NTSC_OVERSCAN_CLR, W0					; (1) turn hsync off black with a color signal
		mov W0, LATB								; (1)

		; Done with this completely
		goto FINISHED_NTSC							; Dont need to count because interrupts will keep us locked




; ----------------- ACTIVE LINE -------------------------
DRAW_ACTIVE_LINE:


		; ------- HSYNC ON ---------
		; H-sync (4.7uS) = 202
		mov #NTSC_SYNC, W0 	                        ; (1) turn hsync on
		mov W0, LATB								; (1)

        ; Now we are going to pre-comnputer the pointers for each 8 pixel section of this scanline
        ; to its corresponding color table. We have plenty of time to do this in the hsync.
        ; Essentially the final resulting pointer is equal to:
        ;
        ;   FinalPointer[pixel] = g_ColorTable (base) + g_AttribTable[pixel] * 16 (size of an individual color table)

		; Copy over the pointers to the different palettes
		mov CurrAttribPtr, W0 						; (1) Get point to beginning of attribtute table for this line
		mov #4, W1									; (1) Move our multiplier to W1...this is for indexing into the attribute table
		mov _g_ColorTable, W2						; (1) Get starting pointer to Color Table
		mov #AttribScanLine, W3					    ; (1) Get start to this lines Attribute line

		; Time taken 20 * 2 = 40 clocks
.rept (NTSC_SCREEN_WIDTH/8)
		mul.uu W1, [W0++], W4						; (1) Figure out offset pointer into palette
		add W2, W4, [W3++]							; (1) Add ColorTable Start to Attribute*4
.endr

		repeat #154									; (1)
		nop											; (1+n)

		; Pre-burst (.6uS) = 26 clocks
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB								; (1)
		repeat #22									; (1)
		nop											; (1+n)


		; Color burst (10 NTSC clocks) = 120 clocks
		mov #NTSC_CBURST1, W0 					    ; (1) turn hsync color burst on at phase 0
		mov W0, LATB								; (1)
		repeat #116									; (1)
		nop											; (1+n)


		; FROM HERE ON it is imparative that we stay in sync with the color clock signal
		; (i.e. we don't change any pixels unless it is an integer multiple of 12)
		; If we don't do this then color changes will not be correct.
		; Post burst
		mov #NTSC_BLACK, W0	                        ; (1) turn hsync off black with no color signal
		mov W0, LATB								; (1)
		repeat #19									; (1)
		nop											; (1+n)


		; Left overscan color
		mov #(NTSC_OVERSCAN_CLR | NTSC_CHROMA_OFF), W0 ; (1) turn hsync off black with no color signal
		mov W0, LATB								; (1)
;		mov _g_Breazeway2, W0						; (1)
;		repeat #198									; (1)
;		repeat W0									; (1)
		repeat #168                                 ; (1)
		nop											; (1+n)


			; Now draw out some pixels (6 clocks per pixel)
			mov #(NTSC_SCREEN_WIDTH), W4			; (1) Rendering out 160*2=320 pixels
			mov _g_HardImagePage, w0					; (1)
			mov w0, PSVPAG							; (1)
			mov _g_HardImagePtr, w0					; (1)


; 320 pixels			
LINE_PIXEL_LOOP:
			mov 	[w0++], w1		  		; (2)
			mov		w1, LATB				; (1)	

			BTSTS.Z w0, #15					; (1)
			bra nz, NO_PAGE_CHANGE   		; (1/2)
			inc PSVPAG						; (1)
NO_PAGE_CHANGE:	
			
			nop								; (1)
			swap w1							; (1)
			;nop
			mov w1, LATB					; (1)
						

			; Branch until line is done (160 pixels)
			sub #1, W4								; (1) W4--
			bra nz, LINE_PIXEL_LOOP					; (1/2) if(W4 != 0) jmp

			mov w0, _g_HardImagePtr
			mov PSVPAG, w0
			mov w0, _g_HardImagePage
;*/


ALLDONE:
		; Now set PORTB to overscan
		mov #(NTSC_OVERSCAN_CLR | NTSC_CHROMA_OFF), W0 ; (1) turn hsync off black with no color signal
		nop											; (1)
		nop											; (1)
		mov W0, LATB								; (1)

		; Now increment our framepointer if we are done with this line (every 2 lines)
		dec ReptLine								; (1)
		bra nz, FINISHED_NTSC						; (1/2)
		mov CurrVRAMPtr, W0							; (1)
		add #(NTSC_SCREEN_WIDTH/NTSC_PIXELS_PER_BYTE), W0	; (1) increment our pointer to the next line 
		mov W0, CurrVRAMPtr							; (1) save
		mov #LINE_REPT_COUNT, W0					; (1) 
		mov W0, ReptLine							; (1) Reset our line repeate counter
        inc _g_RasterLine                           ; (1) Increment our raster line count
		
		; Now test if we should increment our attribute table pointer. We only do this
		; everly 8 pixel lines so that our attribute table covers 8x8 tiles in a sense.
		dec ReptAttrib								; (1)
		bra nz, FINISHED_NTSC						; (1/2)
		mov #((NTSC_SCREEN_WIDTH/8)*2), W0	    	; (1) increment pointer by sizeof(int)*20 tiles
		add CurrAttribPtr							; (1) increment it
		mov #8, W0									; (1)
		mov W0, ReptAttrib							; (1)



		; Done with this completely
FINISHED_NTSC:

        bclr IFS0, #T1IF           					; Clear the Timer1 Interrupt flag Status
                                   					; bit.

		pop RCOUNT									; (1) Make sure we are pushing Rcount since we are using repeat
		pop W5
		pop W4                   					; Retrieve context POP-ping from Stack
		pop SR
		pop.s										; (1) Push status registers

		disi #0x0									; Re-enable interrupts

        RETFIE                     					;Return from Interrupt Service routine


/* Divah 2
Fish_320:
.word 0x4444,0x4444,0x4444,0x4444,0x8373,0x8373,0x7383,0x4444,0x4444,0x4444,0x4444,0x4444,0x7343,0x6363,0x7373,0x4443,0x4444,0x4444,0x7363,0x5263,0x6363,0x6383,0x4444,0x6344,0x6363,0x8373,0x4444,0x4444,0x4444,0x4444,0x6343,0x7363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x6F6F,0x6F6F,0x6F6F,0x4141,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4442,0x6344,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x5243,0x8363,0xC4C3,0x4363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6343,0x8383,0xC4C3,0x7383,0x6363,0x4243,0x4242,0x4242
.word 0x4444,0x4444,0x4444,0x4444,0x6344,0x73C4,0x7363,0x8383,0x4473,0x4444,0x4444,0x4444,0x6344,0x6363,0x6363,0x7373,0x4463,0x6344,0x7373,0x6352,0x6363,0x6363,0x7383,0x7363,0x6363,0x7363,0x4473,0x4444,0x4444,0x4344,0x6363,0x7363,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x7F7F,0x6F6F,0x4141,0x4141,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4242,0x6344,0x63C4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x6342,0xC483,0xC4C3,0x6373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x8363,0xC3C4,0x7383,0x4363,0x4242,0x4442,0x4444,0x4444
.word 0x4444,0x4444,0x4444,0x4444,0x4444,0x6344,0x63C4,0x6352,0x7373,0x7383,0x4444,0x4444,0x7344,0x6363,0x6363,0x6363,0x5263,0x6363,0x5263,0x5263,0x6363,0x6363,0x6352,0x6363,0x6352,0x6352,0x7363,0x8383,0x7373,0x7363,0x6363,0x7363,0x4483,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5241,0xD462,0x52C4,0x4141,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x4242,0xD452,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x5242,0x8373,0xC4C3,0x7383,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x8363,0xC4B3,0x7383,0x4463,0x4442,0x4444,0x4444,0x4444,0x4444
.word 0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x8363,0x6373,0x7352,0x7373,0x7383,0x4444,0x6352,0x6352,0x5252,0x5263,0x6363,0x5263,0x5252,0x5252,0x5252,0x5252,0x6352,0x5263,0x5263,0x6352,0x6363,0x6363,0x6373,0x5252,0x7363,0x6363,0x6373,0x4444,0x4444,0x4444,0x4444,0x4242,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0x5F5F,0xC441,0xD4D4,0xD4E4,0x52C4,0x7F7F,0x6F6F,0x5241,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x4242,0x7343,0x73D4,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4142,0x6342,0x8383,0xC4C3,0x83C3,0x4352,0x4444,0x4444,0x4444,0x4444,0x4444,0x6344,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x7363,0xC483,0x7383,0x4463,0x4442,0x4244,0x4244,0x4444,0x4444,0x4444
.word 0x4344,0x6363,0x6363,0x4363,0x4444,0x4444,0x4444,0x7373,0x7363,0x6363,0x6373,0x7373,0x6363,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x6352,0x5252,0x5252,0x6363,0x6352,0x5252,0x6352,0x4383,0x4444,0x4444,0x4242,0x7F7F,0x4242,0x7F7F,0x4141,0x5241,0x4141,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5252,0xD4C4,0xD4D4,0x52C4,0x7F7F,0x6F6F,0x4141,0x4141,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4343,0xC452,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x6342,0x8373,0xC483,0xC4D4,0x83C4,0x4363,0x4444,0x4444,0x4444,0x5244,0x6352,0x5252,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5242,0x8373,0xC4C4,0xC4C4,0x83C4,0x7383,0x7352,0x4352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x7363,0xB383,0xC4C4,0x7383,0x4243,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7344,0x8383,0x8373,0x7383,0x8383,0x6383,0x4444,0x7373,0x5263,0x6363,0x5263,0x5252,0x6363,0x5252,0x5252,0x5252,0x5242,0x5242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x5242,0x5252,0x5252,0x5263,0x5252,0x5252,0x6352,0x6363,0x7373,0x4383,0x4244,0x4141,0x4241,0x5151,0x7F7F,0x4141,0x4242,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5252,0x5252,0x7F7F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x4141,0x7F7F,0x5243,0x83C4,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4142,0x6352,0x8373,0xC483,0xC4C3,0xC4D4,0xC4C4,0x8383,0x7373,0x7373,0x7373,0x5263,0x5263,0x5252,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x8363,0xC3C4,0x83C4,0x8383,0x8383,0xC383,0xC4C4,0x8383,0x7363,0x6352,0x5273,0x5263,0x5263,0x8383,0x8383,0xB3C3,0x83C4,0x7383,0x4263,0x4442,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4444
.word 0x4444,0x4444,0x8373,0x6373,0x6363,0x6352,0x7373,0x6363,0x6373,0x5252,0x5252,0x5252,0x4252,0x4242,0x5242,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x6352,0x5263,0x5252,0x5252,0x5252,0x4242,0x4252,0x4252,0x5252,0x6352,0x6363,0x5263,0x4242,0x4141,0x4141,0x4141,0x5252,0x4141,0x4141,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5252,0x4141,0x4141,0x4142,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x4141,0x5252,0x4242,0x7352,0x83C4,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6342,0x5252,0x6363,0x6363,0x8373,0xC483,0xC493,0xC4C3,0xC4C3,0xC3B3,0xC3C4,0xC4C3,0xC4D4,0xC4D4,0x83C4,0x7373,0x6352,0x5252,0x5252,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x8363,0xC4D4,0x8383,0x6373,0x6363,0x5252,0x6363,0x5252,0x8352,0x7383,0x7383,0x7373,0x8373,0x8383,0x8383,0x5252,0x5252,0x6363,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x4444,0x4444,0x6344,0x8383,0x7352,0x5273,0x7363,0x6363,0x6363,0x5252,0x5252,0x5252,0x5242,0x5252,0x5252,0x6363,0x6352,0x5273,0x7363,0x5273,0x6363,0x5263,0x6363,0x5252,0x5252,0x5263,0x5252,0x5273,0x5252,0x5252,0x4242,0x5242,0x5242,0x5252,0x4152,0x4141,0x4141,0x4141,0x4141,0x4152,0x4142,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5152,0x41C4,0x5252,0x4251,0x4141,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x5241,0x4141,0x5242,0xD473,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4241,0x4242,0x5252,0x5252,0x5263,0x8373,0x8383,0x8383,0xC483,0xC4C4,0x83C4,0xC4C4,0x93D4,0xC3D4,0xC3C3,0xC4C3,0x83C3,0x7383,0x7373,0x6363,0x5263,0x5252,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x8352,0x83C4,0x8383,0x6373,0x5252,0x4244,0x4242,0x4444,0x5252,0x6352,0x6363,0x6352,0x6363,0x6363,0x6352,0x6352,0x5252,0x4452,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x4444,0x4444,0x4444,0x8363,0x5263,0x6352,0x5252,0x5252,0x5252,0x5252,0x5242,0x5252,0x5252,0x6363,0x6363,0x6352,0x7373,0x5263,0x7363,0x7373,0x5252,0x7363,0x8383,0x5273,0x7363,0x8383,0x5252,0x5252,0x6363,0x7363,0x5252,0x5252,0x4252,0x4142,0x4141,0x4141,0x5252,0x4141,0x4242,0x5252,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0xC4C4,0x62D4,0xD4C4,0xD462,0x5252,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x6241,0x4141,0x4242,0x6352,0x5252,0x5263,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x5252,0x6352,0x7363,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0xC483,0x83C4,0xC4D4,0xD4D4,0xC3C4,0xC3C3,0x83C4,0x7373,0x6352,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x7383,0x7363,0x6373,0x6363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x4344,0xC4C4,0x8383,0x7383,0x6373,0x6352,0x5252,0x5252,0x5252,0x5252,0x6352,0x6373,0x5252,0x8373,0x5273,0x6352,0x6352,0x7383,0x6352,0x6363,0x7383,0x6352,0x6373,0x6373,0x7363,0x7373,0x5263,0x8363,0x8383,0x5263,0x7363,0x7373,0x5263,0x4141,0x5241,0x5242,0x4141,0x7F7F,0x5252,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6252,0xC4C4,0xC4D4,0x62E4,0x7F7F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x41C4,0x7F7F,0x5243,0x6363,0xC473,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x5242,0x6352,0x7373,0x7373,0x7373,0x7383,0x7383,0x8383,0x8373,0x8383,0x8383,0x8383,0x83C4,0xC4C3,0xC3C4,0xC4C4,0xC4C3,0x83C4,0x7373,0x6352,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7352,0x73C4,0x6373,0x7363,0x5263,0x8383,0x4363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244
.word 0x8383,0x7383,0x8383,0x6373,0x6363,0x5252,0x5252,0x4252,0x6352,0x6352,0x6363,0x5263,0x7363,0x7363,0x5273,0x6363,0x7363,0x8373,0x6383,0x7352,0xC483,0x6383,0x8363,0x83C3,0x6363,0x8373,0x7383,0x6352,0x8373,0x5273,0x7352,0x7373,0x5273,0x4141,0x42D4,0x5252,0x5252,0x5241,0x4152,0x4141,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5252,0xD452,0xF4D4,0x4152,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x4141,0x4343,0x5252,0x6363,0xC483,0x83D4,0x4463,0x4444,0x7373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x5241,0x6352,0x6352,0x6363,0x6363,0x6373,0x7373,0x7373,0x7383,0x8373,0x8383,0x8383,0x8383,0xC483,0xC493,0x9383,0xB3C4,0xD4C4,0x83C3,0x7373,0x4352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x7363,0x6373,0x6363,0x6352,0x8373,0x8373,0x8383,0x7373,0x7383,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x7383,0x7373,0x6373,0x5263,0x5252,0x4252,0x5252,0x6363,0x5252,0x5263,0x7373,0x6352,0x6363,0x7352,0x8383,0x6352,0x7373,0x8373,0x6383,0x6373,0x8383,0x8363,0x8383,0x5263,0x8383,0x8383,0x5273,0xC383,0x5283,0x5252,0x7363,0x5252,0xD441,0xD452,0x5252,0x4141,0xD452,0x4141,0x7F7F,0x6F6F,0x4141,0x7F7F,0x7F7F,0x4141,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4251,0xD4C4,0x62D4,0xC4C4,0x6F6F,0x6F6F,0x7F7F,0x4141,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x7363,0x8383,0xC4C4,0xC383,0x83C4,0x8363,0x8363,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x5242,0x5263,0x6363,0x6363,0x6363,0x6373,0x7363,0x7373,0x7373,0x7373,0x8373,0x8373,0x8383,0x8383,0x8383,0xC383,0x83C4,0xC4C4,0x83C3,0x7383,0x4363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x7383,0x6363,0x5273,0x8373,0x8383,0x7383,0x8383,0x7373,0x6363,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444
.word 0x8383,0x8363,0x6383,0x6363,0x5252,0x5242,0x6363,0x5263,0x6363,0x8373,0x8352,0x8383,0x8383,0x6373,0x7352,0x8383,0x83C4,0x7363,0xC483,0x6383,0x7352,0xC4C4,0x5283,0xC452,0x73C4,0x7363,0x8363,0x7383,0x8373,0x7383,0x7363,0xC463,0x4141,0xC4C4,0xD4C4,0x41E4,0x5241,0x4152,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x6241,0x62C4,0x41C4,0x8F8F,0x6F6F,0x4141,0x7F7F,0x5252,0x7F7F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x4242,0x4141,0x4241,0x5252,0x8363,0xC4B3,0x8383,0x7383,0x7373,0x4383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x6352,0x6363,0x6363,0x6363,0x6363,0x7352,0x6363,0x6363,0x7373,0x8383,0x7383,0x8373,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x83C3,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0x7373,0x7363,0x6352,0x8383,0x7373,0x8383,0x6373,0x4252,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7373,0x7373,0x6352,0x6363,0x5242,0x5252,0x7363,0x6373,0x6352,0x5252,0x7363,0x7373,0x6383,0x8373,0x6363,0x8373,0x6373,0x5283,0x8373,0xC483,0x6363,0x8383,0x6363,0x8363,0xC4C3,0x6352,0xC483,0x52C4,0x6363,0x5283,0x8373,0x5273,0xC4C4,0x6262,0x5262,0x4141,0x4242,0x5252,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4242,0x5252,0xC4C4,0x6F6F,0x7F7F,0x6F6F,0x5252,0x5252,0x4141,0xE441,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5252,0x7F7F,0x6363,0x5263,0x6342,0x5273,0x8363,0x7363,0x7373,0x83C4,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x6352,0x5263,0x6352,0x5263,0x6363,0x5252,0x7363,0x6373,0x7373,0x7373,0x7373,0x8373,0x8383,0x8383,0x8383,0x6373,0xC483,0x8383,0x7383,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0x6373,0x6383,0x7373,0x83C4,0x8373,0x7373,0x4463,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x7363,0x5252,0x5252,0x5252,0x6373,0x7363,0x7373,0x7363,0x8383,0x5263,0x6363,0x8363,0xC3C4,0xC4B3,0x6383,0x8373,0xC4B3,0x5283,0x8383,0x83C3,0x7363,0xC383,0x5273,0x8363,0x5273,0x8383,0x5283,0x8373,0x52C4,0xC473,0x4152,0x52D4,0x6262,0xC4C4,0x5252,0x4162,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x6F6F,0x4141,0x7F7F,0x7F7F,0x4141,0x6F6F,0x4141,0x6242,0xC4C4,0x52D4,0x6F6F,0x7F7F,0x4141,0x52C4,0xD4D4,0xD4D4,0x4141,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x4141,0xC483,0x6383,0x6342,0x6352,0x8373,0x83C4,0x8383,0x8383,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5242,0x5252,0x5252,0x5252,0x5263,0x5252,0x5263,0x6363,0x6363,0x7363,0x6373,0x6373,0x8383,0x8383,0x6363,0x7373,0x7363,0x7383,0x6363,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0x6363,0x6363,0x8373,0x7373,0xC483,0x7383,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444
.word 0x6373,0x6363,0x5252,0x5252,0x7352,0x7373,0x5273,0x6352,0x8363,0x8383,0xC4C4,0x6383,0x8373,0xC383,0x8383,0x6373,0x8363,0x7383,0x6363,0x8363,0x6373,0x6363,0x8383,0x6363,0xC383,0x63C4,0x6363,0x5273,0x8383,0x73C4,0xD452,0x6262,0x4141,0x5252,0x5252,0xD452,0x5252,0x5241,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x6F6F,0x4141,0x7F7F,0x4141,0x4141,0x6F6F,0x4141,0x5252,0x6262,0xD4D4,0x6F6F,0x7F7F,0x4141,0xF4C4,0x62D4,0x52D4,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x7F7F,0x9373,0xC4C4,0x6383,0x6342,0x7363,0xC483,0x8383,0x8383,0x63C3,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5242,0x6383,0x7363,0x6383,0x7352,0x5273,0x6352,0x5252,0x5252,0x5263,0x5263,0x6363,0x7363,0x6363,0x8363,0x6363,0x7383,0x6363,0x8363,0x7383,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0x7373,0x6352,0x6383,0x6363,0x8383,0x8383,0x7352,0x6373,0x5252,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444
.word 0x5263,0x5252,0x5242,0x5252,0x6363,0x8373,0x8383,0x7363,0x5273,0x7363,0x6383,0x8363,0x5263,0x7373,0x7373,0xC3C4,0x5283,0x8352,0xC3C4,0x5283,0xC483,0x83C3,0x7352,0x73C4,0x8363,0x6383,0x8373,0x83C3,0x7352,0x6363,0xC452,0x4162,0x4141,0x4141,0x5252,0x6262,0x4141,0x5252,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x5252,0x5241,0x6262,0x6F6F,0x5252,0x5252,0xD4D4,0x5262,0x41D4,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x8352,0x8383,0x8373,0x7383,0x6352,0x7363,0x8383,0x7373,0xC483,0x73C4,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x6363,0x6363,0x6352,0x5252,0x5263,0x5273,0x5252,0x5283,0x8363,0x5263,0x5263,0x5263,0x5263,0x6363,0x6373,0x7352,0xC483,0xD4D4,0xC4C3,0xC3C4,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6344,0x6363,0x6363,0x7373,0x7373,0x7363,0x6373,0x7363,0x6373,0x8373,0x7383,0x8383,0x8383,0x4473,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444
.word 0x5263,0x4252,0x6352,0x7352,0x5263,0x8373,0x7373,0x8363,0xC4C3,0x5273,0x5252,0x8383,0xC3C4,0x83C4,0x7352,0x8383,0x83C4,0x6363,0x7383,0x5273,0x8383,0x6373,0x7352,0x8383,0x5263,0x6373,0x7373,0x8383,0x8363,0x52C4,0x51C4,0x4141,0x4241,0x5241,0x5252,0x4152,0x6241,0x4142,0x7F7F,0x4141,0x4141,0x7F7F,0x6F6F,0x4141,0x4141,0x7F7F,0x5252,0x7F7F,0x7F7F,0x4152,0x4141,0xD452,0x6F6F,0x7F7F,0x6F6F,0x4252,0x5252,0x5252,0xC452,0x4141,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x7341,0x83C4,0x7383,0x8383,0x6373,0x6342,0x8373,0x8383,0x8373,0x8383,0xD483,0x7383,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x7373,0x5263,0x8363,0x5263,0x6363,0x5252,0x5252,0x6363,0x7373,0x5273,0x7363,0x5252,0x5252,0x6352,0x5252,0x6352,0x6363,0x8383,0x8383,0xC4D4,0x63C4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6373,0x5263,0x6352,0x7373,0x7363,0x7373,0x6373,0x6363,0x7383,0x6373,0x6363,0x6352,0x7373,0x4452,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444
.word 0x5252,0x6352,0x5263,0x8373,0x83C4,0x5263,0x6363,0x7352,0x8383,0x8383,0x6363,0x8363,0x7383,0xC463,0x73C3,0x6352,0x8363,0xC4B3,0x8363,0xC383,0x5263,0x8383,0x52C4,0x7363,0xC473,0xC4B3,0x6363,0x6373,0x8363,0x5263,0x41D4,0x4141,0x7F7F,0x62C4,0x4141,0x5252,0x5252,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4152,0x6F6F,0x5252,0x4141,0x4242,0xD441,0x6F6F,0x6F6F,0x4141,0x4141,0x5252,0x5252,0x5252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4141,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7373,0xC483,0x8383,0x8383,0xC383,0x6383,0x6342,0x8373,0x7373,0x8383,0x8383,0x8383,0x8383,0x8383,0x83C4,0x83C4,0x8383,0x7383,0x7373,0x7373,0x6373,0x5263,0x6363,0x6363,0x6373,0x6363,0x8373,0x6363,0x6373,0x5263,0x6352,0x7363,0x7352,0x5263,0x7352,0x4273,0x5252,0x5252,0x6363,0x8363,0x7383,0x7373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0xC383,0xC3C3,0x7383,0x5263,0x5252,0x5273,0x6363,0x6363,0x7373,0x6373,0x7363,0x7373,0x4363,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444
.word 0x5252,0x5252,0x7352,0x8373,0x6373,0x8383,0x8383,0x5283,0x7373,0x6373,0x8383,0x6383,0x5252,0x8383,0x83C4,0x7363,0x8352,0x8383,0x6383,0xC483,0x5273,0x8363,0x6383,0x8383,0x8352,0x8383,0x7363,0x83C3,0x6352,0xD452,0x4152,0x7F7F,0xC4C4,0x5241,0x5252,0x41C4,0x5252,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4142,0x6262,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4242,0x6F6F,0x4141,0x4141,0x4141,0x4141,0x4141,0x5252,0x5241,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x4141,0x4141,0x7F7F,0x7F7F,0x5252,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6363,0x8373,0x83C4,0x7383,0x7373,0xC483,0x5273,0x5252,0x7363,0x8373,0x7373,0x8383,0x8383,0x8383,0xC493,0xD4C4,0xC3C4,0xC4D4,0xC4D4,0x83D4,0xC483,0x8383,0x8383,0x8383,0x7373,0x7373,0x5263,0x7373,0x5273,0x7363,0x5283,0x6352,0x5263,0x6352,0x6373,0x5283,0x8352,0x5252,0x6342,0x6352,0x6363,0x5263,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x8373,0xD4C3,0xC4C4,0xC3B3,0x83C4,0x6373,0x5252,0x6363,0x8373,0x7383,0x8373,0x8373,0x7383,0x4243,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x5252,0x8363,0x5263,0x7363,0x6373,0x8383,0x7383,0x8373,0x8383,0x6363,0x8383,0x83C4,0x6363,0x8363,0x7373,0xB3C4,0x6363,0x7363,0x83C4,0x5263,0xC383,0x5283,0x5252,0x8383,0x5283,0x6352,0x7352,0xC483,0x6383,0xC462,0x4141,0x5252,0x6252,0x9F9F,0x4152,0x6241,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4142,0x4242,0x5251,0x4262,0x8F8F,0x4141,0x7F7F,0x4141,0x4141,0x5252,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x4141,0x5241,0x4242,0x6F6F,0x4152,0x6F6F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x8373,0xC483,0x8383,0x6373,0x8383,0x8383,0x5273,0x6352,0x6363,0x8373,0x7373,0x8383,0x8383,0x83C4,0x8383,0x8383,0x8383,0x83C4,0x8383,0x8383,0xC3C4,0xD4C4,0xC3C4,0xB3C4,0xC3C4,0xC4C3,0x8383,0x6373,0x6363,0x6363,0x7383,0x5252,0x5252,0x6352,0x5273,0x5273,0x5273,0x8383,0x4252,0x4242,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x8383,0xD483,0x8383,0x8383,0x8383,0x8383,0xB3C3,0x6383,0x6352,0x6352,0x7373,0x7373,0x6373,0x4352,0x4444,0x4442,0x4244,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444
.word 0x6352,0x8373,0x8373,0x5263,0x5252,0x7383,0x6383,0xC483,0xC4D4,0x5273,0x7373,0x6373,0xC383,0x5283,0x6352,0x8383,0x73C3,0x7352,0x8383,0x5263,0x8383,0x8383,0x73C4,0x8363,0x6373,0xC4C4,0x6363,0x8373,0x5283,0x52D4,0x4141,0x4141,0x5252,0x4141,0xC4D4,0x5252,0x4152,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4152,0x5241,0x4141,0x4152,0x7F7F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x4162,0x4141,0x7F7F,0x5252,0x5242,0x6F6F,0x4141,0x5241,0x4152,0x4141,0x5242,0x4152,0x4141,0x4141,0x7F7F,0x6F6F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8383,0x8373,0x8383,0x8373,0x7373,0x8383,0x8383,0x4263,0x6352,0x6363,0x8373,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x83C4,0x8383,0x8383,0x8383,0x8383,0x8383,0xC483,0xD4C4,0xC4C3,0xC4C4,0x7383,0x7363,0x5273,0x8363,0x6373,0x5252,0x5252,0x5252,0x7352,0x7352,0x7352,0x7373,0x6342,0x5242,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0x8383,0xD483,0x8383,0x8383,0x8383,0x8383,0x8383,0xC483,0x83C3,0x5263,0x6363,0x7373,0x8383,0x6363,0x5263,0x4352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6352,0x6373,0x8373,0x8383,0x6373,0x5263,0x5252,0x8383,0x7383,0xC483,0x5283,0x5263,0x8373,0x7383,0x6363,0x7352,0xC463,0x5283,0x6363,0x8383,0x6363,0x6363,0x8373,0x6363,0x6373,0x7363,0x7383,0x7363,0x5252,0x4142,0x5252,0x4242,0x5252,0xD441,0x5252,0xD4D4,0x4141,0x4141,0x4142,0x7F7F,0x4141,0x5252,0x4242,0x5262,0x6252,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x6252,0xC462,0x6262,0x5241,0xC4C4,0x4152,0x5252,0xC441,0x4152,0x52D4,0x62D4,0x52D4,0xD4D4,0xC442,0x5252,0x5252,0x5252,0x7F7F,0x4141,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8363,0x8383,0x8383,0x8383,0x8373,0x6373,0x7373,0x8383,0x5263,0x6363,0x6363,0x6363,0x7363,0x7363,0x6363,0x6373,0x8363,0x8373,0x8383,0x7373,0x8383,0x7383,0x8383,0x8373,0x8383,0x8383,0x83C4,0x83C4,0x8383,0x8383,0x7383,0x6363,0x6352,0x7373,0x6363,0x6352,0x5263,0x5252,0x5252,0x5252,0x5252,0x5273,0x7352,0x5273,0x8363,0x5252,0x8383,0xC483,0x83C4,0xC483,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0xC3C4,0x7383,0x8352,0x7383,0x6363,0x8373,0x83C3,0x8373,0x6373,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444
.word 0x5263,0x6363,0x7363,0x8373,0x8363,0xC3C4,0x6383,0x7363,0x6373,0x8383,0x83C3,0x7363,0x6352,0x6363,0xB3C4,0x73C4,0x8352,0x7383,0x6363,0x8373,0x6363,0x5283,0x6363,0x8363,0x5283,0x6363,0x5252,0x8363,0x6252,0x5252,0x4141,0x5252,0x4141,0xE452,0x41D4,0x4152,0x4141,0x4141,0x4141,0x4141,0x5241,0x5252,0x5252,0x41D4,0xC4C4,0x4141,0x6F6F,0x6F6F,0x4141,0x7F7F,0xD462,0xD4D4,0x6262,0x5252,0xC441,0xC441,0x5252,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF4F4,0xD4E4,0x52D4,0x52C4,0x4141,0x6262,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xC342,0x7383,0x8383,0x8383,0x7383,0x7383,0x5273,0x5252,0x6363,0x5263,0x5252,0x6363,0x6363,0x6352,0x5263,0x6363,0x8373,0x8373,0x7383,0x8383,0x8373,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x83C4,0xC473,0xC4C4,0x83C4,0x7383,0x7363,0x7383,0x4252,0x5273,0x6352,0x7373,0x5263,0x5252,0x5252,0x5252,0x5263,0x5273,0x5252,0x6373,0xC383,0x83C4,0x8383,0x8383,0x8383,0x8383,0x7383,0x8373,0x7373,0x7373,0x8383,0x73C4,0x6352,0x6373,0x8363,0x8383,0x83C3,0x8383,0x7383,0x5263,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442
.word 0xC483,0x7383,0x6352,0x6352,0x8363,0x8383,0x8363,0xB3C3,0x7383,0x7363,0x6373,0xC483,0xC4C4,0x5273,0x7363,0x8363,0x6383,0x7352,0x7373,0x6352,0x5273,0x6363,0x7363,0x6352,0x5273,0x7373,0x8363,0x5252,0x5252,0x5252,0x4141,0x4141,0x4141,0xC4E4,0xD441,0x4262,0x5241,0x4141,0x5241,0x4141,0x62D4,0x5252,0xD462,0xD441,0x41C4,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5252,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xE4D4,0xD4E4,0xF4F4,0xE4F4,0xE4F4,0xD4E4,0xE4D4,0xD4E4,0xC4C4,0x5252,0x4141,0x4152,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6363,0xC483,0x5273,0x7352,0xC483,0x83C4,0x6373,0x5273,0x5252,0x8373,0xC3C4,0x83C3,0x5263,0x7373,0x8383,0x8383,0x6363,0x7373,0x7373,0x8373,0x7383,0x8373,0x8373,0x7383,0x8383,0xD483,0x83C3,0x6373,0x6363,0x8373,0xC4C4,0x83C4,0x6383,0x6363,0x7383,0x6352,0x7373,0x5273,0x8363,0x83C4,0x6363,0x8383,0x5283,0x7373,0x6383,0x8352,0xC4C3,0xC3D4,0x83C4,0x83C4,0x8373,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x6383,0x7352,0x73C4,0x8373,0x8383,0x7383,0x73C4,0x4352,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442
.word 0x8383,0x6383,0xC373,0x7383,0x7352,0x7383,0x8363,0xC483,0x7383,0x7383,0x5252,0x8373,0x7383,0x8373,0x5263,0x8373,0x7373,0x5263,0x6363,0x6352,0x6373,0x5263,0x6363,0x5263,0x5252,0x6363,0x7363,0x4163,0x5252,0x4141,0x4141,0x4141,0x5252,0x6262,0x4152,0x4141,0x5252,0x4141,0x41C4,0x6241,0xD4D4,0x41D4,0xD4D4,0x5241,0x5262,0x4141,0x7F7F,0x7F7F,0x7F7F,0x5241,0xD452,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xF4F4,0xE4F4,0xE4D4,0xD4E4,0xD4E4,0x52D4,0x5241,0x4141,0x5252,0x5252,0x4141,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x8373,0x73C3,0x5252,0x8373,0x8383,0x7373,0x5273,0xC4C4,0xC3C4,0xD4C4,0xC4C3,0x6373,0x7373,0x7383,0x7363,0x6373,0x8373,0x8383,0x7373,0x8383,0x8373,0x7383,0x8373,0xC383,0x8383,0x6363,0xC3C3,0x4273,0x4242,0x4242,0x4242,0x4242,0x5252,0x5242,0x4252,0x4252,0x5242,0x6363,0x6363,0x5252,0x5252,0x6352,0x6363,0x5252,0xC483,0xC4D4,0xD4D4,0xC4C4,0x8383,0x8383,0x8373,0x8383,0x7373,0x8383,0x8383,0x7373,0x6363,0x8363,0x83C3,0x83C4,0x8383,0x83C4,0x4373,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7363,0x6373,0x8383,0x83C4,0xC473,0x83C4,0x6363,0x7373,0x6363,0xC383,0x52C4,0x5263,0x6352,0x7363,0x6373,0x6352,0x7363,0x7373,0x5252,0x5252,0x5263,0x6363,0x6352,0x5252,0x5263,0x5252,0x6352,0x5252,0x4141,0x4141,0x4141,0x4141,0xC441,0x4141,0x6262,0x5252,0x5262,0xD441,0x5252,0xE442,0xC462,0xD4D4,0x62E4,0xD4C4,0x41C4,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xC462,0xC4C4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xF4F4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0x6262,0x52C4,0x4152,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6341,0x5242,0x7363,0x6383,0x7342,0x7383,0x7363,0x6373,0x8373,0xC3C4,0xC3C4,0x83C4,0x6373,0x5263,0x5252,0x7352,0x5273,0x5252,0x7352,0x8383,0x7373,0x8383,0x8373,0x83C4,0x7383,0xC363,0x4263,0x7352,0xC483,0x7373,0xD483,0x8383,0x5273,0x8383,0x8383,0x8383,0x8383,0x8383,0x7373,0x7373,0x7373,0x7373,0x7373,0x8383,0x8383,0xC4C3,0xC3C3,0x93C4,0x83C4,0x8383,0x8373,0x8383,0x8383,0x8383,0x7373,0x7373,0x6383,0x5252,0x8352,0x8383,0x83C4,0x5283,0x4243,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4244
.word 0x6352,0x5263,0x8373,0x6383,0xC483,0x83C3,0x6363,0x5273,0x6352,0x8383,0x7383,0x8383,0x7383,0x6363,0x7363,0x63C4,0x6352,0x6363,0x5252,0x5252,0x5252,0x5252,0x4242,0x4242,0x4242,0x4242,0x5252,0x4141,0x4141,0x4141,0x4141,0x4141,0xE462,0x5252,0x4141,0x4141,0x5252,0x5252,0x4141,0xC4D4,0xF4D4,0xD462,0x41E4,0xC4C4,0x6262,0x6F6F,0x7F7F,0x6F6F,0x5252,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xF4F4,0xF3F4,0xF4F4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x5252,0x4141,0x4141,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x8352,0x6383,0x4252,0x6352,0x5252,0x8373,0x7383,0x6373,0x7352,0xC483,0x6373,0x6363,0x6363,0x6363,0x6363,0x6383,0x6363,0x8383,0x6373,0x7352,0x7373,0x8363,0xC473,0x8383,0x7352,0x4283,0x8363,0xC3C3,0xC3C4,0x7373,0xC3C4,0xC4C3,0x7383,0xC452,0xD4C4,0xD4C3,0xD4D4,0xC4C3,0xC4C4,0xC3C4,0xC4C4,0xD4C3,0xD4D4,0x8383,0xC483,0xC4D4,0xC4D4,0x83C3,0x8383,0x8373,0x7383,0x7383,0x8383,0x7383,0x8383,0x8373,0x6373,0x5252,0x7363,0x8363,0x5263,0x4252,0x4442,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xC383,0x73C4,0x5263,0x5252,0x7363,0x6373,0x8363,0xC383,0x5283,0x6352,0x5263,0x7373,0x6373,0x7373,0x6363,0x5252,0x5252,0x5252,0x5252,0x6363,0x6363,0x6363,0x5263,0x5263,0x5263,0x5252,0x4163,0x4141,0x4141,0x7F7F,0x4141,0x5241,0x4141,0x6241,0x4141,0x5252,0x5252,0xC462,0x5241,0x52D4,0xD462,0xC4D4,0x5251,0xE4D4,0x4141,0x7F7F,0x6F6F,0x7F7F,0x6241,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xC4D4,0x6252,0x4141,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x8383,0x7383,0x8373,0xC4C4,0x6383,0x6342,0x8383,0x8363,0x6383,0x8363,0x7363,0x8383,0x7363,0x8373,0x8383,0x6373,0x8363,0x6373,0x8352,0x8383,0x6373,0x6352,0x6383,0x5273,0x7342,0x8383,0x8383,0x8383,0xC4C4,0x6373,0xC483,0xC4C4,0x6383,0xC442,0xC393,0xC4C4,0xD4C3,0xC4C3,0xC4D3,0xD493,0xC3C4,0xC3D4,0x8383,0xC4C4,0xD4C4,0x93C4,0xC4C3,0x8383,0x8373,0x7383,0x8383,0x8383,0x8383,0x7373,0x7383,0x6373,0x6352,0x6352,0xC483,0x6383,0x4242,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444
.word 0xC483,0x5283,0xC483,0xC4C3,0x5252,0x5252,0x8373,0x8383,0x8363,0xC3C3,0x6383,0x5252,0x6363,0x5263,0x5252,0x5252,0x5252,0x5252,0x6373,0x5252,0x5252,0x5242,0x4263,0x4242,0x4242,0x4242,0x4142,0x4141,0x4141,0x7F7F,0x4141,0x6241,0x41E4,0x5241,0x4141,0x4141,0x4141,0x5252,0xD4D4,0x5252,0xC4C4,0xD462,0x6242,0x5262,0x4141,0x7F7F,0x6F6F,0x7F7F,0xC452,0xD4D4,0xD4C4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F3,0xF4F4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x5252,0x7F7F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x8383,0x7383,0x8373,0x8383,0x83C4,0x4263,0x8363,0x8373,0x6373,0x7383,0x83C4,0x8383,0x8383,0x5283,0xC473,0x83D4,0x5263,0x6373,0x8383,0x6383,0x7352,0x6383,0x6383,0x6363,0x6383,0x5252,0x6363,0x7373,0x8373,0x7373,0x8383,0x7373,0x4252,0xC473,0xD4C4,0xD4D3,0xC4C4,0xC4C4,0xC4C4,0xC4C3,0xD4C4,0xC3C4,0xC4C4,0xD4C3,0xC3B3,0xC4C4,0x93C4,0x7383,0x8373,0x8383,0x8383,0x8383,0x7383,0x8373,0x8373,0x5263,0x5252,0x8363,0x8383,0x7383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6363,0x5263,0x8373,0x8383,0xC373,0x83C4,0x5252,0x6363,0x8373,0x8383,0x7363,0x5283,0x6363,0x5252,0x5252,0x7352,0x6363,0x5252,0x5252,0x5252,0x4252,0x4242,0x4242,0x4242,0x4442,0x4242,0x4141,0x4141,0x4141,0x4141,0x4141,0x5252,0x5252,0x5252,0x4141,0x4141,0x5241,0x4141,0xD4D4,0x6252,0x5252,0x4152,0xE452,0x41C4,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x5252,0x4141,0x7F7F,0x6F6F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x7373,0x7373,0x7373,0x8363,0x8383,0x83C3,0x4263,0x8363,0x7352,0x8373,0x6363,0x8383,0x8373,0x6383,0x7363,0x7352,0xD483,0x83C3,0x6373,0x7352,0x7373,0x8363,0xC3C4,0x8383,0x8383,0x8383,0x5273,0x5252,0x5252,0x5252,0x5252,0x4252,0x8373,0x8383,0x8383,0x8383,0xC3C4,0xC3C3,0xD493,0x93D4,0x93D4,0xD4C4,0xC3C4,0xD4D4,0x6383,0x8383,0x5273,0x7373,0x8383,0x8383,0x7383,0x7373,0x7373,0x6373,0x5252,0x5252,0x7342,0x8383,0xC483,0x83C3,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x5273,0x7373,0x6363,0x7373,0x8352,0x8383,0xC483,0x83C4,0x7352,0x7373,0x7363,0x6363,0x5252,0x6352,0x5273,0x5252,0x5252,0x4252,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4141,0x4141,0x4141,0x4141,0x4141,0xD4C4,0x5252,0x4141,0x4141,0x4141,0x4141,0x4141,0x6241,0x6251,0x5252,0x5252,0xC462,0x4262,0x7F7F,0x7F7F,0x7F7F,0x5252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x6252,0x5252,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7373,0x8373,0x8363,0x6373,0x8383,0xC483,0x6383,0x5242,0x5252,0x8363,0x6363,0x6373,0x7373,0x6363,0x6363,0x6373,0x8352,0xD4C4,0xC3C4,0x6383,0x7363,0xD483,0x83C4,0xC483,0xC4C4,0xC3C4,0x83C4,0x7383,0x6363,0x6363,0x6363,0xC483,0xC3C3,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xC4C4,0xC4D4,0xC4C4,0xC3C3,0xC3D4,0xC3C4,0xC4D4,0x6363,0x7383,0x6352,0x8373,0x7383,0x8383,0x7373,0x5252,0x5252,0x5252,0x4252,0x7352,0x8383,0xC4C4,0x8383,0xC483,0x7383,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444
.word 0x83C4,0x5283,0x8383,0x5263,0x5263,0x5263,0x7363,0x6373,0x8383,0x5273,0x4252,0x5252,0x6352,0x5252,0x5252,0x4252,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4141,0x4141,0x4141,0x4141,0x5252,0x5252,0x5241,0x5252,0x4242,0x4141,0x4141,0x4141,0x5252,0x5241,0x5241,0x4141,0xC4E4,0x8F8F,0x4141,0x7F7F,0x6F6F,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xC4D4,0x6262,0x9F9F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7373,0x7383,0x7383,0x6363,0x6373,0x8383,0xC483,0x6383,0x5252,0x8373,0x7363,0x7373,0x6373,0x8363,0x6352,0x7373,0x6363,0x8352,0xC4C4,0xC4C3,0xC3C3,0x83C4,0x8383,0x83C3,0xC3C3,0xC4C3,0xB3C3,0xC3C4,0x83C4,0xC483,0xC483,0x93C4,0xC483,0x83C4,0xC483,0xC493,0xC4C3,0xC3C4,0x93C4,0xC3D4,0xD4C4,0xD4C3,0xB3C3,0x73C4,0x8352,0x5273,0x6363,0x8383,0x8383,0x8373,0x7363,0x5273,0x4242,0x5242,0x7363,0x8383,0x8383,0x7383,0xC483,0xC483,0x83C4,0x7383,0x4443,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x7363,0xC4C4,0x83C4,0x8352,0x8383,0x5263,0x5252,0x7383,0x5273,0x5252,0x6352,0x5263,0x5242,0x4452,0x4242,0x4242,0x4242,0x4442,0x4444,0x4244,0x4444,0x4442,0x4442,0x4444,0x4444,0x4141,0x4141,0x7F7F,0x7F7F,0x6241,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4152,0x5252,0x4141,0x5241,0xD462,0x4242,0x6F6F,0x4141,0x7F7F,0xD452,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x5252,0x4152,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0xC4C4,0x73C4,0x7363,0x7383,0x6363,0x6363,0x6363,0x7363,0x4263,0x8352,0x7373,0x5252,0x7363,0x6373,0x6352,0x6363,0x7373,0x6373,0x8383,0xC483,0x8383,0x8383,0x8383,0x83C4,0xC4C4,0xD4C3,0xC3C3,0xC3C4,0x83C4,0xD4C3,0xC3C3,0xD4B3,0x93D4,0xC4D4,0xC4D4,0xC3D4,0xD4C4,0xD4D4,0xC3D4,0xC3C4,0xC4C3,0xC3D4,0xC3C4,0x63C4,0x8363,0x7363,0x8352,0x7383,0x8383,0x6373,0x7363,0x7383,0x6363,0x7363,0x7363,0x6363,0x6363,0x7363,0x7383,0x7373,0x7383,0x7383,0x4363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6352,0x5273,0x8383,0x8383,0x8363,0x7373,0x8363,0x6383,0x6352,0x5252,0x6352,0x5252,0x5242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4141,0x4141,0x7F7F,0x4141,0x4152,0x4152,0x5241,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0xC441,0x5252,0x4141,0x4141,0x6F6F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x4262,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x8383,0x7383,0x7383,0x6383,0x7352,0x8383,0x8373,0x8363,0x6363,0x5252,0x7373,0x5252,0x6363,0x6363,0x5263,0x7373,0x7383,0x7373,0x6373,0x7373,0x7373,0x7373,0x7373,0x8383,0xD4C3,0xD4C3,0xC4C4,0xD4D4,0xD4C3,0xC4C4,0xC3C4,0xC3C4,0xD4C3,0xC4C3,0xC3C3,0xC4C3,0x93C3,0xD3C4,0xC4C4,0xD4C3,0xC4C3,0x83C3,0xD483,0x5273,0x7363,0x6363,0x8373,0x8373,0x6373,0x4242,0x8373,0x5273,0x8373,0xC3C4,0x7383,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xC3C4,0x8383,0x6363,0x6363,0x6363,0x6363,0x7352,0x6373,0x6373,0x5252,0x5263,0x5252,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x5252,0x4162,0x4152,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x5241,0x5252,0x7F7F,0x6F6F,0x6F6F,0x5252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0x5252,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6363,0x6363,0x6363,0x7363,0x5252,0x8373,0x7383,0x8373,0x7383,0x5252,0x6352,0x8383,0x7373,0x7373,0x8373,0x8383,0x8383,0x6383,0x7363,0x7363,0x8383,0x8383,0x8383,0x8383,0x8383,0xC4C4,0x93C3,0xC3D4,0xC4C3,0xD4C3,0xC4C3,0xC3C4,0xC3C4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD3D4,0xC3C4,0x83C4,0x8373,0x73C4,0x6352,0x8373,0x5273,0x8373,0x8383,0x5263,0x6352,0x7383,0x6363,0x8383,0xC493,0x73B3,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x7383,0x8373,0x8383,0x6373,0x8373,0x6383,0x6352,0x5252,0x5263,0x5252,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x4141,0x7F7F,0x7F7F,0x5252,0x5252,0x5252,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x52C4,0x5252,0x7F7F,0x6F6F,0x6F6F,0xC441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x4142,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7373,0x7373,0x4263,0x4242,0x5263,0x7352,0x73C4,0x7383,0x8383,0x7383,0x4252,0x5252,0x5252,0x6363,0x6373,0x6363,0x7373,0x6373,0x7363,0x7363,0x7363,0x6363,0x5263,0x8373,0xC3C4,0xC3C4,0xC4C4,0x83C4,0xC383,0xC3C4,0xC3D4,0xC3D4,0xC3D4,0x83C4,0x8383,0x8383,0x8383,0x8383,0x7383,0x7373,0xD483,0x83C4,0x5252,0x5263,0x8363,0x5283,0x8373,0x8383,0x6373,0x8373,0x6373,0x7352,0x8383,0x8383,0x83C3,0x4443,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x5263,0x5263,0x5263,0x5263,0x5263,0x5263,0x5263,0x5263,0x6363,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7373,0x5273,0xC483,0xC3C4,0x6383,0x8383,0x7383,0x5263,0x6352,0x4252,0x4452,0x4242,0x4442,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4141,0x4141,0x4141,0x4141,0x4141,0x4242,0x4141,0x4152,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x5262,0x4241,0x6F6F,0x6F6F,0x6F6F,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x5252,0x4141,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8342,0x7373,0xC383,0x83C4,0x7383,0x5252,0x8363,0x7373,0x7373,0x8373,0x8383,0x4263,0x5242,0x7363,0x8373,0xC4C4,0x8383,0x6363,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x6363,0x8383,0x93C4,0xC3D4,0x83C4,0xC483,0xC4C3,0xC4C4,0xC483,0xC4C3,0x8383,0x7373,0x7373,0x8373,0x8383,0xC4C4,0x73C4,0x5263,0x5252,0x5263,0x8373,0x5283,0x8363,0x8383,0x8383,0x6383,0x5273,0x8363,0xB3C4,0xC483,0x83C4,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5243,0x5263,0x8383,0x8383,0xC3C4,0xD4D4,0xC4C4,0xC4D4,0xB3C4,0xD4D4,0xB3C4,0xC4C4,0x8383,0x8383,0x5273,0x6363,0x4444,0x4444,0x4444,0x4444
.word 0xC4C3,0x83C4,0x8373,0x8383,0x6383,0x7373,0x5263,0x5252,0x5263,0x4252,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0xC462,0x5252,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x5252,0x5252,0x4141,0x6F6F,0x6F6F,0x6F6F,0xC462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x52D4,0x5252,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x7363,0x7373,0x8373,0x8383,0xC483,0x7383,0x5252,0xC473,0x7383,0x6373,0x7373,0x7383,0x6363,0x83C4,0x5273,0x7352,0x7373,0x8373,0x6373,0x6363,0x6373,0x6373,0xC483,0xC4C4,0xC4C4,0x73C4,0x6373,0x8363,0x8383,0x7373,0x7373,0x8383,0x8383,0x8383,0x8383,0xC483,0xD4C4,0xC4C4,0x8383,0x5263,0x6342,0x83C4,0x83C4,0x5263,0x7373,0x8383,0x5263,0x7373,0x7383,0x6363,0x6352,0xC483,0xC493,0x8383,0xC483,0x83C4,0x5273,0x4444,0x4444,0x5243,0x8373,0xC483,0xD4C4,0xC3C3,0xC3C4,0x83C4,0x8383,0x8383,0x6373,0x6373,0x7363,0x6363,0x6373,0x7363,0x8383,0x8383,0x8383,0x8383,0xC4C4,0x8383,0x5263,0x4463
.word 0x8383,0x8383,0x6363,0x6363,0x5263,0x6352,0x5252,0x6352,0x4252,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4442,0x4444,0x4242,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x6262,0x4242,0x5252,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x5241,0x4141,0x4141,0x6F6F,0x6F6F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x6262,0x4141,0x4141,0x7F7F,0x6F6F,0x6F6F,0x4141,0x7363,0x7373,0x8373,0x8383,0xC4C4,0x83C3,0x5273,0x5242,0xC473,0x7373,0x8373,0x6363,0x7352,0x7373,0x8373,0x6373,0x6363,0x5252,0x6373,0x6363,0x8363,0x8383,0x8383,0x8383,0x8383,0x8383,0xC4C4,0x83C3,0x8383,0x8383,0x8383,0x7383,0x6373,0x6373,0x8383,0xC483,0x8383,0x6373,0x4252,0x4241,0x6344,0xC4D4,0xC483,0x6373,0x7363,0x8373,0x8383,0x5263,0x5252,0x6352,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x7383,0x6363,0x7352,0x8383,0xD4C4,0x93D4,0x83C4,0x7383,0x6373,0x5252,0x5252,0x5252,0x4444,0x4242,0x4244,0x4242,0x4442,0x4242,0x5252,0x5252,0x6352,0x7373,0x7373,0x8383,0x8383,0x8383
.word 0x6352,0x5263,0x8363,0xC4C4,0x83C4,0x7363,0x6352,0x6363,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x5241,0x6262,0x4241,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x5241,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5252,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4241,0x8383,0xC4C4,0x83C3,0x7383,0xC483,0x8383,0x8383,0x7383,0x4252,0x7363,0x6363,0x5273,0x5252,0x7363,0x7373,0x5263,0x8373,0x8383,0x7363,0x7373,0x6373,0x7352,0x8383,0x8383,0x5273,0x4242,0x4242,0x6342,0x8383,0x8373,0x7383,0x8383,0x8383,0x8383,0x7383,0x4263,0x4242,0x4242,0x4242,0x4242,0x4444,0xC3C3,0x83C4,0x4452,0x5273,0x8363,0xC483,0xC4D4,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0xC483,0xC4C3,0xC4C3,0xC4C3,0x83C4,0x8383,0x6363,0x6363,0x5252,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4442,0x4442,0x4242,0x4244,0x4242,0x4242,0x4442,0x4242,0x4242,0x4442,0x5252,0x6363
.word 0xC4C3,0xC4B3,0x7352,0x8383,0x8383,0x5263,0x6352,0x5252,0x4452,0x4444,0x4442,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4242,0xC4C4,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5242,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0xC441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x52C4,0x4141,0x6F6F,0x6F6F,0x6F6F,0x4141,0x8383,0xE4D4,0xE4D4,0x8383,0x5283,0x6363,0x8363,0x8383,0x6383,0x6363,0x7363,0x8383,0x5273,0x6352,0x6383,0x7373,0x8373,0x7373,0x7352,0x6373,0x7383,0x5263,0x8363,0xD4D4,0x83B3,0x4263,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x6363,0x6363,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x5244,0xC4C4,0x5273,0x4244,0x4242,0x6352,0x8373,0xC3C4,0xD4C4,0xC4C4,0xC3D4,0xC3C4,0xD4C4,0xC4C3,0xC4D4,0x8383,0x7373,0x8373,0x8383,0x6363,0x5252,0x5252,0x4242,0x4242,0x4442,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4442,0x4444,0x4444,0x4442,0x4442,0x4242,0x4242,0x4242,0x4242
.word 0x8383,0x8383,0x5273,0x7373,0x7373,0x5252,0x5263,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x5252,0x5252,0x4141,0x4141,0x7F7F,0x4141,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x4152,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0xD452,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4142,0x4141,0x6F6F,0x6F6F,0x7F7F,0xE4E4,0xD4E4,0xD4D4,0x63E4,0x8363,0x6342,0x8383,0x8383,0xC483,0x8383,0x7373,0x7373,0x5273,0x5252,0x6373,0x7383,0x8373,0x7373,0x7373,0x7373,0x5263,0x8352,0xC3C4,0x93C3,0x5273,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x8373,0x8383,0x5273,0x4241,0x4444,0x4444,0x4444,0x4444,0x4444,0x8344,0x5273,0x4242,0x4444,0x4244,0x4242,0x5244,0x6373,0x6363,0x6363,0x6363,0x6363,0x6363,0x6363,0x6363,0x6363,0x5263,0x5252,0x5252,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444
.word 0x8383,0x8383,0x6363,0x6352,0x6373,0x5252,0x5252,0x4252,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4242,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5252,0x5252,0x4141,0x4141,0x7F7F,0x4141,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x5252,0x6F6F,0x6F6F,0x6F6F,0x4141,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5252,0x4141,0x6F6F,0x6F6F,0x7F7F,0xF4F4,0xD4D4,0xD452,0x42E4,0x5252,0x4263,0xC463,0x8373,0x7373,0x6373,0x8363,0x8383,0x5273,0x6352,0x6383,0x8373,0x7363,0x8383,0x8383,0x8383,0x5252,0xC383,0x93D4,0x6383,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6344,0xD4D4,0x83C3,0x4263,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x6344,0x4243,0x4244,0x4444,0x4442,0x4442,0x4242,0x4242,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x4252,0x4242,0x4242,0x4242,0x4244,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6363,0x7363,0x6383,0x8373,0x7383,0x6352,0x5252,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4141,0x4241,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4241,0x5252,0x4141,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x4141,0x6F6F,0x6F6F,0x6F6F,0x5252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4262,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xE4E4,0x5252,0xD452,0x83E4,0x8383,0x6383,0x6342,0x8383,0x7363,0x7373,0x7373,0x8383,0x6383,0x5252,0x6363,0x8363,0x6383,0x6363,0x8363,0x5273,0x6363,0x5263,0x5252,0x4352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x9352,0x6383,0x4452,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4452,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4442,0x4242,0x4244,0x4244,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4242,0x4444,0x4444
.word 0x8352,0x83C4,0x83C4,0x8373,0x7383,0x5252,0x5252,0x4442,0x4444,0x4244,0x4442,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4141,0x4242,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4241,0x5151,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x4141,0x6F6F,0x6F6F,0x6F6F,0xC441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x52C4,0x4141,0x7F7F,0x6F6F,0x7F7F,0x5252,0x5252,0xD462,0xC4E4,0x8383,0x83D4,0x4263,0x8363,0x7383,0x7373,0x7373,0x7373,0x8363,0x4263,0x6352,0x5273,0x6363,0x8383,0x8383,0x6363,0x8363,0x6383,0x7352,0x8383,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5252,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7352,0x8383,0x8383,0x6352,0x6363,0x5273,0x5252,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x4141,0x6F6F,0x6F6F,0x6F6F,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52C4,0x4141,0x6F6F,0x6F6F,0x7F7F,0x5252,0xD452,0xD4C4,0x83E4,0x8373,0x8383,0x7383,0x6342,0x8383,0x6383,0x6363,0x7373,0x7383,0x8373,0x4252,0x5252,0x5252,0x6363,0x6363,0x8383,0x8383,0x5273,0x6363,0x8363,0xB3C4,0x83C4,0x7373,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x8352,0x7373,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4244,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6383,0x8383,0x8383,0x8363,0x6383,0x5263,0x5252,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4244,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5241,0x4152,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x4152,0x6F6F,0x6F6F,0x4141,0x5252,0xD4D4,0xD462,0x73E4,0x7373,0x8373,0x8383,0x4273,0x6352,0xC483,0x83C4,0x8383,0x8363,0xC483,0x8383,0x5273,0x4242,0x6352,0x7383,0x8383,0x7373,0x7373,0x5273,0x7352,0x8363,0xC483,0xC4B3,0x7383,0x6373,0x4343,0x4343,0x4343,0x4343,0x7343,0x8383,0x83C4,0x8383,0x8383,0x7373,0x4443,0x4444,0x6363,0x7373,0x4473,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4443,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x6344,0x4352,0x4244,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x6363,0x6352,0x7363,0x5273,0x5252,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4442,0x4244,0x4444,0x7F7F,0x4242,0x7F7F,0x7F7F,0x4142,0x4241,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x5262,0x4152,0x5242,0xD4D4,0xC4C4,0x62C4,0x5252,0x6F6F,0x7F7F,0x7F7F,0xC462,0xD4C4,0xE4C4,0xC4C4,0x8383,0x6373,0x8383,0x8383,0x6383,0x5242,0x6363,0x8363,0x7383,0x6363,0xC483,0xC4C3,0x5283,0x6352,0x8373,0x7373,0x7373,0x6373,0x5252,0x7373,0x6373,0x6363,0x6363,0x6363,0x5263,0x7352,0x7373,0x7373,0x7373,0x7373,0x6373,0x7373,0x7373,0x6363,0x6363,0xC4C3,0x7383,0x4252,0x5252,0x5252,0x4444,0x4442,0x4444,0x4244,0x4444,0x6344,0x6383,0x4444,0x4442,0x4444,0x4444,0x4444,0x7363,0x8373,0xB3C4,0x6383,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4442,0x4244
.word 0x8383,0x6363,0x8383,0x7363,0x5273,0x5252,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4141,0x4242,0x7F7F,0x7F7F,0x4242,0x4241,0x4152,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x5252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4252,0x8F8F,0x6F6F,0x4040,0x4141,0xD462,0x62D4,0x5262,0x6F6F,0x7F7F,0x5252,0xD4D4,0xE462,0xD4D4,0x5283,0x8373,0xC4C3,0x8383,0xC4C3,0xC4C3,0x5283,0x8363,0xC4C4,0x7373,0x8373,0x8373,0x93C4,0x52C4,0x6352,0x7373,0x8363,0x7363,0x8383,0x7383,0x6352,0x7352,0x6363,0x7363,0x6383,0x8383,0x7373,0x7373,0x7383,0x7373,0x8373,0x7383,0x7363,0x7373,0x7373,0x6363,0x6363,0x8373,0x83B3,0x6373,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x8363,0xC4B3,0x4463,0x7363,0x8373,0xC4D4,0xB3D4,0xB3C4,0x8383,0x8383,0x5283,0x4442,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6373,0x8373,0x8373,0x7383,0x5273,0x5252,0x5252,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4141,0x4242,0x7F7F,0x7F7F,0x4142,0x4141,0x5252,0x4141,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x4141,0x6F6F,0x4141,0x5252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4152,0x4040,0x6F6F,0x4141,0xD4C5,0xE5D4,0xD4D4,0xD4D4,0xC4D4,0x6262,0x5252,0x6F6F,0x7F7F,0xC452,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x7383,0x4252,0x4242,0x6363,0x8383,0xC483,0xC4C4,0x7383,0x8373,0x8373,0xC483,0x8383,0x5283,0x5252,0x7383,0x7373,0x6363,0x7363,0x5273,0x5252,0x6363,0x8373,0x83C4,0x7383,0x8383,0xC383,0x8383,0x7373,0x7373,0x8383,0x8383,0x7383,0x7373,0x7383,0x7383,0x6373,0x6373,0x6363,0x7363,0xC383,0x73C4,0x7373,0x4443,0x4444,0x7343,0x8383,0x7383,0xC483,0x6363,0x6363,0x5263,0x5252,0x5252,0x5252,0x4243,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xC4D4,0x5273,0x7373,0x8373,0x5283,0x5252,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4242,0x7F7F,0x4242,0x7F7F,0x4242,0x4141,0x5252,0x4142,0x4141,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0xC4D4,0x6262,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4152,0x6F6F,0x7F7F,0x7F7F,0x6262,0xD4D4,0xE4E4,0xE4E5,0xE5E5,0xE5D4,0xD4D4,0x5252,0x5262,0x4141,0x4141,0xC462,0xD4C4,0xE462,0xE4D4,0x83C4,0xD483,0xD4C3,0x6383,0x5242,0xC473,0x8383,0x8383,0x7383,0x7373,0x8373,0x8383,0x8373,0x6383,0x5242,0x8363,0x7373,0x7363,0x5263,0x8363,0x7373,0x6383,0x7352,0x8373,0x7373,0x8383,0x8373,0x7363,0x8383,0x8383,0x7373,0x6373,0x8373,0x8383,0xC4C4,0xC4C3,0x6383,0x6363,0x7373,0x6373,0x6363,0x7363,0xC483,0xC4D4,0x83D4,0x7383,0xC473,0x73C4,0x5263,0x6352,0x6363,0x4452,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4442,0x4444,0x4444
.word 0x8383,0x6363,0x7363,0x8383,0x5283,0x5252,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4242,0x4142,0x4241,0x5151,0x7F7F,0x5252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6252,0x5252,0x62D4,0x4152,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6242,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4152,0x6F6F,0x6F6F,0x6F6F,0x4141,0xC442,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4152,0xC441,0xC4C4,0x4141,0x6241,0xD4D4,0xD4D4,0xE462,0xC3D4,0xC483,0x83C4,0x8363,0x8383,0x7383,0x4252,0x6363,0xC483,0xC4B3,0x8383,0x7373,0x8383,0x6383,0x8363,0x4263,0x7352,0x8383,0x6373,0x6373,0x8373,0x8363,0x7383,0x5263,0x6352,0x7363,0x83C4,0x8373,0x7373,0x8383,0x83C4,0x8373,0x7373,0x7373,0x8383,0x7373,0x7373,0x7383,0x7373,0x8383,0x7383,0x7363,0x6383,0x6363,0x7373,0x7383,0x6363,0x8373,0xC483,0xC4C4,0x83C3,0x7383,0x7383,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6373,0xC473,0x63C4,0x7363,0x5273,0x5263,0x5252,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4242,0x4242,0x4141,0x4141,0x7F7F,0x4152,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6242,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5252,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5241,0x5152,0x4152,0x7F7F,0xD4D4,0x62C4,0xE441,0xD452,0xD4D4,0xE4D4,0xD4D4,0x83D4,0xC483,0xC483,0x8383,0x7383,0x7363,0x8383,0x5273,0x4242,0x6363,0x8383,0x7373,0x7373,0x7363,0x83C4,0x6383,0x5242,0x8363,0x7363,0x7383,0x7373,0x7373,0x5263,0x8383,0x8373,0x7373,0x7373,0x7383,0x73C4,0x8373,0x7383,0x7373,0x6373,0x7373,0x7383,0x7363,0x8383,0x8383,0x8383,0x7383,0x8363,0x8363,0x7383,0x6373,0x7363,0x6373,0x8373,0x7373,0x6383,0x8373,0x83C4,0x8383,0x8383,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4444,0x4444
.word 0x5252,0x7383,0x7383,0x7363,0x5252,0x5263,0x5252,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4242,0x4144,0x4141,0x4141,0x7F7F,0x4152,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4141,0x4141,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x52C4,0x4141,0x4141,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5252,0xD4C4,0x62D4,0xD442,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0x63D4,0x6363,0x6363,0x8383,0xC483,0xC4C4,0x8383,0x8373,0x7373,0x7373,0x8373,0x7373,0x5263,0xC483,0x8383,0x8383,0x5273,0x5252,0x6363,0x7373,0x7373,0x6363,0x5263,0x5252,0x4242,0x4242,0x5242,0x5252,0x5252,0x6352,0x6363,0x7363,0x6373,0x6363,0x5263,0x5252,0x5252,0x5273,0x7373,0x8373,0x83C4,0x8383,0x6363,0x8363,0x8383,0x8383,0x7383,0x7373,0x7373,0x7363,0x8383,0x8383,0x8383,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8363,0x6352,0x6363,0xC483,0x5273,0x5263,0x6363,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4242,0x4444,0x4141,0x4141,0x7F7F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x7F7F,0x6F6F,0x4F4F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xD462,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x5252,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0xDDDD,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0xC441,0xD4D4,0x62D4,0xD451,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x42C4,0x4242,0x6363,0x6352,0x6363,0x6363,0x8383,0xD483,0xC3B3,0xD4C4,0xC3C3,0x83C4,0x8352,0xC4C3,0x7383,0x7373,0x8383,0x5263,0x5242,0x5252,0x5252,0x5252,0x4242,0x4242,0x4242,0x4242,0x4244,0x4142,0x5252,0x6352,0x5263,0x6352,0x5263,0x5252,0x6363,0x5252,0x6352,0x6363,0x5252,0x7363,0x8383,0x8383,0x8383,0x8383,0x6363,0x7363,0x8373,0x8373,0x7373,0x8373,0x83D4,0x6373,0x6363,0x4452,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444
.word 0x83C4,0x5283,0x7363,0x8373,0x6383,0x5273,0x5252,0x4452,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x7F7F,0x4242,0x7F7F,0x7F7F,0x4242,0x4144,0x4141,0x4141,0x7F7F,0x5252,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0xBFBF,0x6E6E,0x6E6E,0xCCCC,0xBFBF,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0xD441,0xD4D4,0xD4D4,0xD4D4,0x41C4,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5252,0x6E6E,0x7F7F,0x4E4E,0xDCDC,0x9F9F,0x6F6F,0x4F4F,0x4141,0xD441,0xD4D4,0x62D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x6352,0x5263,0x5252,0x5252,0x6363,0x6352,0xC483,0x7383,0x7383,0x8383,0x8383,0xC483,0x4283,0x8363,0x8383,0x6383,0x6383,0x4442,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x5242,0x5263,0x6363,0x7373,0x83C4,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x83C4,0x8373,0x7373,0x6383,0x4452,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4244,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444
.word 0x8383,0x7373,0x6383,0x7373,0x6383,0x6352,0x5252,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4141,0x4242,0x6F6F,0x7F7F,0x4242,0x4444,0x4141,0x4141,0x7F7F,0x5252,0x4141,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4141,0x5151,0x4E4E,0xFFFF,0x6E6E,0xEFEF,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6F6F,0x7F7F,0x6F6F,0xC4C4,0xDFDF,0x7E4E,0xDCDC,0xFC9D,0x4E4E,0xCFCF,0x7F7F,0x4F4F,0x6262,0xD4C4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0x52C4,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x5252,0x4252,0x6342,0x8373,0x8383,0x7383,0x7373,0x8373,0xC483,0x6383,0x4252,0x8363,0x8383,0x8383,0x4273,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4244,0x4444,0x4442,0x4444,0x4242,0x5252,0x5263,0x6363,0x8373,0x8383,0x8383,0x83C4,0xD4C4,0xC483,0x8383,0x8383,0x8373,0x6363,0x5263,0x4452,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7363,0x8383,0xC483,0x5283,0x7352,0x6352,0x5252,0x4452,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4442,0x4244,0x4444,0x4444,0x4444,0x4141,0x4242,0x4444,0x7F7F,0x4242,0x4144,0x4141,0x4141,0x7F7F,0x6252,0x7F7F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x5241,0xF3D4,0xF3F3,0xF4F4,0xD4E4,0xC463,0x7F7F,0x4A4A,0x6F6F,0xD441,0xE4D4,0xD4E4,0xD4D4,0x41D4,0x4141,0x8F8F,0x4242,0xD4C4,0xDFDF,0xFFFF,0x4E4E,0x4E4E,0xC4E4,0x4141,0x5F5F,0x5252,0xD4D4,0xE4D4,0xD4E4,0x62D4,0xE4D4,0xD4F4,0x62E4,0xD4D4,0x4352,0x4444,0x4444,0x4344,0x4444,0x4343,0x5252,0x7352,0x7363,0x6373,0x8373,0xC483,0xB3C4,0xC4C3,0x8383,0x7373,0x6373,0x5263,0x6342,0x6352,0xC383,0x8373,0x7373,0x4283,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6344,0x7363,0xC483,0x6383,0x5252,0x6363,0x8373,0x8383,0x83C4,0x8383,0x7383,0x7383,0x6373,0x7373,0x5263,0x5263,0x4444,0x4444,0x4442,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6383,0x7383,0xC483,0x7373,0x6383,0x6352,0x6352,0x5252,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4141,0x4242,0x4444,0x7F7F,0x4442,0x4144,0x7F7F,0x4142,0x7F7F,0x6252,0x7F7F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5252,0x4F4F,0x4F4F,0x5F5F,0x4141,0x4152,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0xD441,0xE4E4,0xE4E4,0xD4E4,0x41C4,0x4141,0x8F8F,0x4141,0x4242,0x6252,0xC4C4,0xD4D4,0x52C4,0x4141,0x5F5F,0x4141,0xC4C4,0xD4D4,0xE4D4,0xD4E4,0x62D4,0xE4D4,0xD4D4,0xD462,0x83D4,0x6373,0x6373,0x6352,0x6352,0x7352,0x6363,0x8383,0x8383,0x8383,0x6383,0x4242,0x4242,0x5252,0x5263,0x4242,0x4242,0x6342,0x6363,0x7373,0x6342,0x83C4,0x7373,0x8383,0x4263,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x6352,0xC483,0x8383,0x83C4,0x5263,0x5263,0x7352,0x6373,0x6373,0x7373,0x7363,0x6373,0x6363,0x5263,0x5263,0x5252,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x73C3,0x7363,0x7373,0x7352,0x8383,0x5273,0x5263,0x6352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4444,0x7F7F,0x4141,0x4242,0x4244,0x7F7F,0x4444,0x4444,0x4141,0x5242,0x7F7F,0x5252,0x4141,0x5252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x62D4,0x52C4,0x4152,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xD441,0xE4D4,0xE4F4,0xD4D4,0x52C4,0x7F7F,0x4A4A,0x7F7F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4141,0x5242,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0x62D4,0xD462,0xD4D4,0xD4E4,0x8383,0x8383,0x8383,0x7383,0x7373,0x7373,0x7373,0x7363,0x8373,0xC4C4,0x8383,0x83C4,0x7383,0x7352,0x7383,0x8383,0x8373,0x8383,0x8383,0x4283,0xC463,0x7383,0x8383,0x6383,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4344,0x7352,0x6363,0x8383,0x8383,0xC383,0x7383,0x5252,0x6352,0x6363,0x6363,0x6363,0x5252,0x5263,0x6363,0x5252,0x6363,0x7363,0x5273,0x4443,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4442,0x4442,0x4444,0x4444
.word 0xC3C4,0x7383,0x6352,0x7373,0x7373,0x5263,0x6373,0x6363,0x4443,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x4141,0x4242,0x4444,0x7F7F,0x4242,0x4144,0x7F7F,0x4151,0x4141,0x6262,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0x62D4,0xC4C4,0xC452,0xC4C4,0xC4C4,0x62C4,0x4141,0x6F6F,0x4141,0xD452,0xE4D4,0xE4F4,0xE4E4,0x41C4,0x7F7F,0x6F6F,0x5252,0x5242,0x5252,0x5252,0x5252,0xC452,0xD452,0xD4D4,0xD4D4,0xF8D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0x83D4,0x8383,0x8383,0x8383,0x8383,0x7383,0x8373,0x8373,0x8383,0x6383,0x6363,0x7373,0x6363,0x6352,0x8383,0x8383,0x8373,0x8383,0x8383,0x8383,0x4273,0x83C4,0x7383,0x8383,0x4263,0x4442,0x4444,0x4442,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7363,0x7363,0x7373,0x4252,0x8363,0xC483,0x7383,0x5273,0x5263,0x5252,0x6363,0x6363,0x5252,0x5252,0x6352,0x5252,0x7363,0x6373,0xC4C4,0x5252,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x5283,0x8383,0xC483,0x8373,0x8383,0x6352,0x5252,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4141,0x7F7F,0x4242,0x4444,0x4242,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x6262,0x4141,0x4141,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xC4D4,0xC462,0xD462,0xE4D4,0xF4F4,0xD4E4,0xD4D4,0x5262,0xC452,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xF8D4,0xD4E4,0xE4D4,0xD4E4,0xF8D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x62D4,0xD4C4,0xD4D4,0x4252,0x4242,0x4242,0x4142,0x5242,0x6352,0x6363,0x5263,0x5252,0x7373,0x7383,0x7373,0x8373,0x7383,0x8373,0x8383,0x8373,0x8383,0x73C4,0x73C4,0x7342,0x8383,0xC4C4,0x5273,0x4441,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4442,0x4444,0x8363,0x4263,0x4242,0x4242,0x4244,0x8373,0x83C3,0x6363,0x6373,0x5273,0x6363,0x5263,0x6363,0x5252,0x6373,0x8352,0x7373,0x5273,0xC383,0x8373,0x83C4,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x6373,0x8373,0x8383,0x7373,0x8383,0x6352,0x6352,0x4363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4141,0x7F7F,0x4242,0x4444,0x4444,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x6252,0x4141,0x4141,0x4141,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4C4,0xE4D4,0xF4F4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4F8,0xF8F8,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0x62C4,0xD4C4,0x52D4,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x7352,0x8383,0x83C3,0x83C3,0x83C4,0x8383,0x8383,0x7383,0x8373,0x7383,0x8383,0x8383,0xC483,0x4263,0x8352,0x83C4,0x5283,0x4242,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x6344,0x8383,0x7383,0x7383,0x7363,0x6373,0x8373,0x6363,0x8383,0x6363,0x8373,0x8363,0x73C4,0x6383,0x7352,0x7383,0x8363,0x8383,0x5252,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242
.word 0x5252,0x6373,0x7373,0x6373,0x6352,0x5263,0x5273,0x6363,0x6352,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4141,0x7F7F,0x4244,0x4444,0x4244,0x6F6F,0x7F7F,0x4141,0x4241,0x5252,0x5252,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xE4D4,0xF4F3,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4F4,0xE4E4,0xD4D4,0xE4D4,0xF4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0x62D4,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x6343,0x8383,0x8383,0xC4C4,0xC3C4,0xC4C4,0xC383,0x83C4,0xC4C4,0x8383,0x8383,0x8383,0x8383,0x7373,0x4263,0x4263,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x7352,0x6373,0x8373,0x8373,0x6363,0x6352,0xC463,0x7363,0xC483,0x8363,0xC473,0x7383,0x7373,0x8383,0x7363,0x4263,0x4242,0x4243,0x4242,0x4342,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x63C4,0xD4D4,0x7383,0x8352,0xC4C4,0x7383,0x6383,0x6363,0x5263,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4242,0x7F7F,0x4244,0x4444,0x4244,0x4242,0x7F7F,0x4141,0x4241,0x5252,0x6252,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xF4F8,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x52C4,0x4446,0x4444,0x4444,0x4444,0x4444,0x7343,0x8383,0x8383,0x7383,0x7363,0x8383,0x7383,0x8373,0x7373,0x7373,0x8373,0x7373,0x8383,0x7383,0x4263,0x7352,0x6383,0x4242,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x7363,0x5252,0x8383,0x8373,0x4242,0x5252,0x8373,0x7363,0x8363,0x7373,0x8373,0xC383,0x5273,0x8363,0x8383,0x5273,0x4443,0x4242,0x4242,0x4442,0x4442,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4244
.word 0x6383,0x8383,0x8383,0x8363,0x8383,0x6383,0x7373,0x7352,0x6352,0x5263,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4142,0x4141,0x4242,0x4444,0x4444,0x4142,0x7F7F,0x7F7F,0x5252,0x4141,0x5252,0x4242,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D3,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xE4D4,0xF4F4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x44C4,0x4444,0x4444,0x4444,0x4444,0xC463,0x7383,0x5252,0x4242,0x4242,0x4242,0x5242,0x6342,0x6363,0x6373,0x7363,0x6363,0x5263,0x4242,0x8363,0x8383,0x6383,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x5273,0x4142,0x8363,0x6383,0x4443,0x4342,0x8352,0xC473,0x5273,0x5252,0x8373,0x8373,0x5283,0x4241,0x8363,0x83C3,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6363,0x7373,0x6373,0x7363,0x7373,0x6363,0x7352,0x5252,0x6363,0x6363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x4144,0x7F7F,0x7F7F,0x4141,0x4241,0x4241,0x5252,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xF4F4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4462,0x4444,0x4444,0x4444,0x8343,0x6383,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x5252,0x7373,0x6363,0x7373,0x7373,0x8383,0x7383,0x4252,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4244,0x4444,0x4444,0x4444,0x6344,0x4452,0x4242,0x8342,0x5283,0x4444,0x4244,0x7342,0x83C4,0x5273,0x4242,0x7352,0x7383,0x73C3,0x4242,0x5242,0xC473,0x5273,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4244,0x4442,0x4444
.word 0xC363,0x7383,0x7363,0x6373,0x6363,0x8352,0xC4C4,0x6383,0x6352,0x5263,0x4463,0x4244,0x4244,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4242,0x4242,0x4244,0x4444,0x4444,0x4242,0x7F7F,0x7F7F,0x4141,0x4242,0x5241,0x9F9F,0x6F6F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xE4D4,0xF4F4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4652,0x4444,0x4444,0x5244,0x5273,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6343,0x8383,0xC3C4,0xC3C4,0xC4C3,0x8383,0x6373,0x4242,0x4242,0x4442,0x4442,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4342,0x4442,0x4244,0x7342,0x4273,0x4444,0x4244,0x5242,0x7383,0x4452,0x4244,0x4242,0x8363,0x7383,0x4444,0x4242,0x5242,0x7363,0x4452,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xC483,0xC483,0xC352,0xC3B3,0xC4C3,0xC473,0xD493,0x73C4,0x7352,0x5252,0x5263,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4444,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5252,0x4152,0x6F6F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xF4F4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0x62D4,0x4446,0x4444,0x4444,0x7363,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x8363,0x8383,0x7383,0x7373,0x4263,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x5242,0x4373,0x4444,0x4444,0x4242,0x6373,0x4452,0x4244,0x4242,0x6342,0x7383,0x4444,0x4244,0x4242,0x4241,0x4442,0x4444,0x4444,0x4442,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x7383,0x8363,0xC4C4,0x8383,0x8373,0x8383,0x83C4,0x5252,0x5283,0x6363,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5252,0x4142,0x7F7F,0x7F7F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xF4F4,0xE4F4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4444,0x4444,0x6344,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7352,0x7373,0x4263,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4452,0x4444,0x4444,0x4242,0x5252,0x4444,0x4444,0x4244,0x4142,0x8352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7373,0x5273,0xC483,0x8383,0x7373,0x6352,0x8383,0x6373,0x7373,0x6352,0x6352,0x4352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4242,0x4444,0x4444,0x4444,0x4141,0x7F7F,0x7F7F,0x7F7F,0x5252,0x4142,0x4142,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xF4D4,0xF3F4,0xE4F4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5262,0x4444,0x4444,0x4344,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x7373,0x4252,0x4441,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x5242,0x4444,0x4444,0x4444,0x4244,0x5241,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x83C4,0x6352,0x6363,0x6363,0x6363,0x8383,0x6373,0xC483,0x83C4,0x5263,0x5263,0x6352,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4244,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5241,0x7F7F,0x7F7F,0x7F7F,0x4241,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xD4C4,0xD4D4,0xF4D4,0xF5F5,0xE4F7,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4662,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4344,0x5252,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4342,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0xC452,0xC3B3,0x8383,0xC463,0x83C4,0x83C4,0x8383,0x6383,0x6363,0x7352,0x6352,0x6363,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4444,0x4444,0x4242,0x4244,0x4442,0x4444,0x4444,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5252,0x5241,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xF4F4,0xE4F4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4462,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x5243,0x4152,0x4442,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4242,0x4442,0x4242,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7373,0xC473,0x8383,0x8383,0x8363,0x8383,0x6383,0x5252,0x6352,0x8383,0x5273,0x7352,0x6352,0x4463,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x5252,0x4242,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD462,0xD4D4,0xD4D4,0xF4E4,0xF4F4,0xE4F4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x4263,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4244,0x4242,0x4442,0x4242,0x4242,0x4242,0x4442,0x4442,0x4442,0x4442,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6373,0x8383,0x8383,0x6373,0x6363,0x6363,0x6363,0xC3C4,0x5283,0x7352,0x6383,0x5252,0x6373,0x6363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4244,0x4242,0x4444,0x4444,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x5241,0x4152,0x4152,0x7F7F,0x7F7F,0x6F6F,0x4141,0x4141,0x4141,0x4141,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xF4D4,0xF3F3,0xE4F3,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD462,0x4452,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x5252,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4242,0x4442,0x4442,0x4442,0x4442,0x4244,0x4442,0x4442,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4442,0x4244,0x4244,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x5263,0x6363,0x6363,0x7363,0x83C4,0xC4D4,0x8363,0x83C3,0x7363,0x8383,0x7352,0x6383,0x7352,0x6363,0x4352,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4242,0x4444,0x4444,0x4444,0x4242,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0xC442,0x4141,0x7F7F,0x7F7F,0x6F6F,0x4141,0x4242,0x4141,0x4141,0x4141,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xF4E4,0xF4F4,0xE4F4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xD4C4,0x4444,0x4444,0x4444,0x4442,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4444,0x4442,0x4244,0x4242,0x4444,0x4442,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4242,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4442,0x4244,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x5273,0x8383,0x8383,0x6383,0xC4C3,0x8383,0x7383,0x7373,0x8352,0xC4C3,0x8363,0x83C4,0x5263,0x6373,0x7373,0x4463,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x5252,0x5252,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xE4E4,0xE4F4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x62C4,0x4444,0x4444,0x4242,0x4242,0x4444,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4244,0x4244,0x4242,0x4442,0x4442,0x4444,0x4242,0x4444,0x4242,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7352,0x8383,0x7383,0x5273,0x8383,0x6383,0x5252,0x8363,0x7383,0x7383,0x6363,0x8383,0x6373,0x5252,0x6373,0x6373,0x4463,0x4444,0x4444,0x4244,0x4444,0x4242,0x4244,0x4242,0x4444,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x5252,0x4242,0x4141,0x7F7F,0x7F7F,0x6F6F,0x4141,0x7F7F,0x4141,0x7F7F,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC4D4,0xC452,0xC4C4,0x6262,0xE4D4,0xE4E4,0xD4D4,0xC4D4,0x5262,0x5252,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4C4,0x44D4,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4442,0x4444,0x4444,0x4444,0x4442,0x4244,0x4444,0x4242,0x4244,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4244,0x4444,0x4444,0x4242,0x4242,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4242,0x4244,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4242,0x4442,0x4442,0x4444,0x4444,0x4442,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4442,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7352,0x7383,0x6363,0x7363,0x7373,0x5263,0xC383,0xC483,0x5283,0x6363,0x6363,0x5252,0x6352,0x5252,0x7352,0x7363,0x5263,0x4443,0x4244,0x4444,0x4444,0x4442,0x4244,0x4242,0x4244,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x5241,0xD452,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x5252,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0x41D4,0x5241,0x4141,0x6262,0x5262,0xD4D4,0xC4C4,0x41C4,0x5252,0x4141,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0x44D4,0x4444,0x4444,0x4244,0x4442,0x4442,0x4444,0x4444,0x4244,0x4242,0x4444,0x4244,0x4442,0x4242,0x4244,0x4444,0x4244,0x4444,0x4444,0x4242,0x4444,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4244,0x4444,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4442,0x4442,0x4244,0x4244,0x4242,0x4242,0x4244,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444
.word 0x6352,0x6363,0x5252,0x8363,0xC4C3,0x7383,0x7373,0x8383,0x7373,0xC483,0x83C4,0xC463,0xC4C4,0x8373,0x5273,0x6352,0x6373,0x7363,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x5252,0x6262,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4242,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x42D4,0x4141,0x4141,0x4141,0xD4C4,0xD4D4,0x6262,0x4141,0x4141,0xD452,0xF4F4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0x44D4,0x4244,0x4444,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4244,0x4442,0x4444,0x4444,0x4244,0x4244,0x4442,0x4444,0x4442,0x4444,0x4244,0x4442,0x4442,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4244,0x4244,0x4242,0x4442,0x4244,0x4244,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4244,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4244,0x4444,0x4444,0x4242,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442
.word 0x6352,0x7352,0x8383,0xC452,0x8383,0x83C3,0x6352,0x6373,0xC452,0xC3C4,0xC4C3,0x8373,0x93C4,0xC373,0x8383,0x5252,0x6373,0x6373,0x7363,0x6352,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0xD441,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xC4D4,0x4142,0x4141,0x4241,0x4241,0x5241,0x4142,0x4141,0x6252,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4C4,0xD4D4,0x4462,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4244,0x4442,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4242,0x4244,0x4444,0x4444,0x4242,0x4442,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4444,0x4244,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242
.word 0x6352,0x6363,0x7363,0x7352,0x8373,0x7383,0xD483,0xC4C4,0x63C3,0x8383,0x6383,0x5252,0x5252,0x8363,0x73C4,0x8373,0x5263,0x7352,0x6363,0x6373,0x5273,0x7373,0x6363,0x6352,0x4163,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4152,0x7F7F,0x4141,0x5252,0x41C4,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x5152,0x5252,0xD4C4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xF4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4442,0x4444,0x4242,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4244,0x4242,0x4244,0x4442,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4444,0x4442,0x4242,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4244,0x4242,0x4444,0x4244,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4442,0x4444,0x4444,0x4444
.word 0x6352,0x6352,0x7363,0x6352,0x6363,0x6352,0x8363,0x7383,0x6383,0x7373,0x5263,0xC473,0x83C3,0x6363,0x5283,0xC373,0x83C4,0x5263,0x8352,0x6363,0x6373,0x6373,0x7373,0x6363,0x4242,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4152,0x4141,0x4141,0x4141,0x5151,0x5252,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x4141,0xD441,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xF4D4,0xD4D4,0x6262,0xD4C4,0xE4D4,0xD4F4,0xE4D4,0xF4F4,0xF4D4,0xF4F4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4442,0x4444,0x4244,0x4442,0x4242,0x4242,0x4242,0x4442,0x4242,0x4242,0x4242,0x4244,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4244,0x4442,0x4242,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4242,0x4242,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444
.word 0x5242,0x5263,0x5252,0x7352,0xC4C4,0x83C4,0x7352,0x6373,0x8352,0xC4C4,0xC4C4,0xC463,0xC4C3,0x8383,0x83C4,0x7363,0x6373,0x5252,0x7373,0x5252,0x7363,0x6383,0x7373,0x7373,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x5252,0x4141,0x4141,0x4141,0xC441,0x4141,0x4141,0x6F6F,0x6F6F,0x7F7F,0x4141,0x4141,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xF3D4,0xF3F3,0xE4F4,0xE4D4,0xE4E4,0xE4D4,0xE4F4,0xD4E4,0xF4D4,0xD4E4,0xF4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x4444,0x4244,0x4442,0x4242,0x4442,0x4444,0x4442,0x4444,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4442,0x4242,0x4444,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242
.word 0x4242,0x5242,0x6373,0x7363,0x8383,0x8383,0x6363,0x5273,0xC473,0xC393,0xC3C4,0x8352,0x73C4,0x7363,0xC4C3,0x5283,0x8373,0x7383,0xD452,0x5273,0x6363,0x5252,0x5252,0x5252,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x5242,0x5241,0x4141,0x4141,0x5252,0x52C4,0x4141,0x6F6F,0x6F6F,0x6F6F,0x4141,0x4141,0xD441,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xC463,0xE463,0xE4F3,0xF2E4,0x63E4,0x4363,0x6343,0x7363,0xE4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4442,0x4444,0x4444,0x4242,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242
.word 0x4263,0x5252,0x5252,0x5252,0x6363,0x4263,0xC383,0xC3D4,0x63C4,0x83C4,0x8383,0x6352,0x6363,0x5283,0xC483,0x7373,0xD4C4,0xC4D4,0x7363,0x7363,0xD483,0x73C4,0x6352,0x5252,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x5241,0x5252,0x5252,0x4141,0x5151,0xC441,0x4242,0x7F7F,0x6F6F,0x6F6F,0x4141,0x4141,0x5252,0xD4D4,0xD4C4,0xD4D4,0xC4D4,0x5262,0x4242,0x6342,0x4243,0x5542,0x5563,0x6363,0x63C4,0x63C4,0x4343,0x6343,0x6343,0x4255,0x4343,0x6343,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4C4,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4242,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4442,0x4244,0x4244,0x4242,0x4442,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4242,0x4242,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442
.word 0x4283,0x5252,0x5263,0x5252,0x5252,0x5263,0xB383,0x83C4,0x6383,0x7373,0x7383,0xD483,0xD3D4,0x83C4,0x7363,0x6352,0x9383,0x83C4,0x6363,0x6383,0x7363,0x5263,0x8373,0x4242,0x4141,0x4141,0x4141,0x5252,0x7F7F,0x7F7F,0x6241,0x5252,0x5252,0x4141,0x4141,0x7F7F,0x52C4,0x7F7F,0x7F7F,0x6F6F,0x4141,0x4141,0x4141,0xC4C4,0xD4D4,0x62D4,0x4141,0x4242,0x4363,0x6355,0x73C4,0x6363,0x63C4,0x73C4,0x4355,0x6343,0x6355,0x6363,0xC5C4,0xC463,0x7363,0x4263,0x4242,0xD463,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4C4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x63C4,0x4242,0x6373,0x8352,0x8383,0x6383,0xC473,0x7383,0x5263,0x6352,0x7383,0xC463,0x8383,0x73C3,0x8352,0x83C4,0x6363,0x5263,0xD483,0x83C3,0xC383,0x7383,0x5252,0x7F7F,0x4141,0x4141,0x4141,0x4142,0x7F7F,0x7F7F,0x6241,0x5262,0x5252,0x4141,0x4141,0x5252,0xC4C4,0x4141,0x7F7F,0x6F6F,0x4141,0x4141,0x4141,0xC4C4,0xC4D4,0xD4D4,0xC4D4,0x4363,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x6F6F,0x6F6F,0x4242,0x6355,0x4343,0x4343,0x4243,0x6342,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x83D4,0x7373,0x7373,0x7373,0x7373,0x4363,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4244,0x4242,0x4442,0x4244,0x4244,0x4444,0x4444,0x4244,0x4442,0x4442,0x4242,0x4242,0x4242,0x4444,0x4442,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4442,0x4444,0x4442,0x4444,0x4242,0x4444,0x4242,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4244,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442
.word 0x83C3,0x5242,0x6352,0x6352,0x6373,0x6373,0x7373,0x6373,0xC452,0xC3D4,0xC4B3,0x8363,0x8373,0x6383,0xC4C4,0x83B3,0x8363,0x6383,0x8383,0x83C4,0xC483,0x83C3,0x4163,0x4141,0x4141,0x4141,0x7F7F,0x4152,0x7F7F,0x7F7F,0x4141,0x5252,0x5252,0x4141,0x4141,0x4141,0x4141,0x41C4,0x4141,0x6F6F,0x4141,0x4141,0x4141,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x63C5,0x5542,0x63C5,0xC4C5,0xC4C5,0x4243,0x4242,0x4242,0xC443,0xE5C5,0xD5F5,0xC5C4,0x5563,0xC573,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xC4C4,0x83D4,0x6373,0x7373,0x6363,0x6363,0x7373,0x7373,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4242,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4442,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4244,0x4444
.word 0xC4C4,0x4283,0x4252,0x5252,0x5252,0x6352,0x8383,0x8383,0x8352,0x8383,0x8383,0x6352,0x6363,0x8363,0xC3C3,0x8383,0xB363,0x83C4,0x6363,0x5273,0x8373,0x5283,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x5252,0x5252,0x7F7F,0x5252,0x6252,0x4162,0x4141,0x4141,0x5252,0x4141,0x4141,0x5241,0x4141,0x7F7F,0x4141,0x4141,0x6251,0xC4D4,0xC4D4,0xD4C4,0xD4D4,0x6362,0x73C5,0xC463,0xD5C5,0xC4C4,0xD5D5,0xF5D5,0xD5D5,0xE5D5,0xC5C5,0x6373,0x6363,0xC573,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xD4D4,0x7383,0x7363,0x7373,0x7363,0x7373,0x7363,0x6363,0x6373,0x7373,0x7373,0x4443,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4242,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4244,0x4444,0x4444,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0xC4C4,0x7383,0x4242,0x6363,0x8363,0x4283,0x7373,0x8373,0x7373,0x8383,0x7383,0xC473,0xC4B3,0x7373,0x8383,0x6383,0xC483,0xC493,0x6352,0x7383,0x6352,0x4252,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4162,0x4141,0x4141,0x4241,0x6262,0x6262,0x5252,0x4141,0x5252,0x4141,0x4141,0x4141,0x4152,0x7F7F,0x4141,0x4141,0x4141,0xD462,0xD4D4,0xD4D4,0xC4D4,0x52D4,0x7363,0x6355,0x6373,0xC4C4,0xD5D5,0xF5E5,0xD5C4,0xC4C4,0x63C4,0x6373,0x63C5,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x83D4,0xC383,0xC3C4,0xD4D4,0x83C4,0x8383,0x7373,0x6363,0x7363,0x7373,0x7373,0x6373,0x4463,0x4444,0x4244,0x4242,0x4242,0x4442,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4242,0x4242,0x4242,0x4444,0x4244,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244
.word 0xC3C4,0x83D4,0x4273,0x6352,0x6363,0x6363,0x6363,0x8373,0x6363,0x6363,0x7352,0x93C4,0x93C4,0x5283,0x6363,0x5263,0x7373,0x6373,0xC463,0xC4C4,0x5263,0x4141,0x4141,0x4141,0x4141,0x4141,0x5252,0x4152,0x5252,0x4141,0x4141,0xC4C4,0x4152,0x4152,0x4141,0xC441,0x4141,0x4141,0x4141,0x4141,0x5252,0x4141,0x4141,0x4141,0x6252,0xC4C4,0xC4C4,0xD4D4,0xD4D4,0x5252,0x73C4,0x6355,0x63C5,0xC573,0x6373,0x7363,0x6363,0xC463,0xC563,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0xD4D4,0x6383,0x5252,0x5252,0x5252,0x5252,0x5252,0x7373,0x8383,0x8373,0x8373,0x7363,0x7363,0x7373,0x7373,0x4463,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4244,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444
.word 0xC4C4,0xC4C3,0x6383,0x5242,0x5252,0x5263,0x6363,0x6373,0x5242,0x8363,0x6373,0x8383,0xD483,0x6383,0x5252,0x7373,0x7363,0x5252,0x7373,0x7383,0x4163,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x5252,0x4141,0x4141,0x4141,0x5262,0x4141,0x4152,0x4141,0x5252,0x4141,0x7F7F,0x4141,0x4141,0x5241,0x4141,0x7F7F,0x4141,0x5252,0xD462,0xD4C4,0xC4C4,0xD4D4,0x52D4,0x6342,0x63C5,0x5573,0x6363,0xC455,0x5555,0xC563,0x6363,0x62C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4C4,0xD4D4,0x83D4,0x6383,0x5252,0x6352,0x8373,0xC483,0x7383,0x5252,0x6352,0x8373,0xC3C4,0x7383,0x7373,0x8373,0x6373,0x7373,0x4463,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444
.word 0x83D4,0xC3C4,0x83C3,0x5273,0x5242,0x5252,0x4252,0x5242,0x8352,0x8383,0x52C3,0x7373,0x6383,0x7352,0x8383,0x8383,0x8352,0x7383,0x5252,0x5263,0x4141,0x4141,0x4141,0x4141,0x4141,0x5252,0x4141,0x5252,0x4141,0x7F7F,0x4141,0x5252,0x4152,0x4152,0x4141,0x4141,0x5241,0xD462,0xD4D4,0xD4D4,0x4152,0x7F7F,0x7F7F,0x4141,0x5241,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x42C4,0x5241,0x5142,0x4242,0x4251,0x4251,0x4251,0x5252,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x83D4,0x5273,0x6373,0x8373,0x83C4,0x8352,0x83C4,0x7383,0x8352,0x83C4,0x5263,0x5252,0x8373,0x83C4,0x7373,0x8373,0x7383,0x7373,0x7373,0x4443,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4244,0x4244,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4242,0x4242,0x4242,0x4244,0x4442,0x4244,0x4242,0x4242,0x4442,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242
.word 0xC4C3,0xC483,0xC4C4,0x7383,0x4252,0x6342,0x7363,0x8373,0x7352,0x7373,0x6383,0x6363,0x5263,0x8363,0x8383,0x6383,0x8373,0x7383,0x7363,0x4242,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4152,0x4242,0x4152,0x4141,0x4141,0x4141,0x6241,0x4141,0x4141,0x4141,0x4152,0xC452,0xC4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x4162,0x7F7F,0x4141,0x5252,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0x4141,0x4141,0x4141,0x4141,0x5241,0xD4C4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0x8383,0xC4C4,0x8383,0xC463,0xC4C3,0x5252,0x5273,0x6352,0x6352,0x5252,0x8363,0x6383,0x5252,0x7363,0x8383,0x7383,0x7383,0x7373,0x7373,0x6373,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x4442,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4244,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4242,0x4442,0x4242,0x4444,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xC3C4,0x83C4,0x8383,0xC4C3,0x6383,0x5242,0x6363,0x6352,0x6363,0x5252,0x5263,0x8352,0xC483,0x6383,0x6363,0x6352,0x6373,0x5263,0x4141,0x4141,0x4141,0x4241,0x4141,0x4141,0x4141,0x4141,0x4141,0x5151,0x4141,0x4141,0x4141,0x5252,0x4141,0x4141,0x4141,0xC452,0x6262,0x62C4,0xC462,0xC462,0xD4D4,0xD4C4,0xC4C4,0xD4C4,0x62D4,0x4242,0x6252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4C4,0xC4D4,0x6383,0x93C4,0x73C4,0x7352,0x5263,0x7373,0xC483,0xC4C4,0x7373,0xC383,0xC3C4,0x6383,0xC4C3,0x6383,0x5252,0x8363,0x8383,0x7373,0x7373,0x8373,0x7383,0x4463,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4442,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4244,0x4442,0x4242,0x4242,0x4242,0x4242,0x4442,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242
.word 0xC4D4,0xC493,0xC3C4,0xD4C4,0x83C3,0x5273,0x6342,0x5252,0x5252,0x5252,0x5252,0x6363,0x8373,0x5273,0x8373,0x7352,0x5263,0x4142,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x6F6F,0x5241,0x7F7F,0x4141,0x6241,0xC4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x5252,0xD462,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xF4F4,0xF4F4,0xF4F4,0xF4F4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8383,0x6383,0x8383,0x5283,0x5252,0x8373,0x6383,0x8383,0xC3C4,0x5283,0xC483,0xC4C3,0x6383,0xC483,0x7383,0x6352,0x5252,0x8363,0x83C4,0x8383,0x8373,0x8373,0x7373,0x4443,0x4444,0x4244,0x4242,0x4244,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x7383,0x4244,0xC3C4,0xC4C4,0xC4C4,0x83C4,0x5273,0x5242,0x6363,0x6373,0x5283,0x6352,0x5263,0x5283,0x7373,0x5273,0x4252,0x7F7F,0x4141,0x4152,0x5241,0x5252,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0xC452,0xD4D4,0xC4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x41D4,0xD4C4,0xD4D4,0xE4D4,0xE4D4,0xF4D4,0xE4E4,0xF4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7373,0x5263,0x7352,0x5252,0x7373,0xC473,0xC4C3,0x83C3,0x8363,0x8383,0x5263,0x7363,0x5263,0x5252,0x5263,0x7363,0x8383,0x8363,0x5273,0x6352,0x8383,0x8383,0x7373,0x6363,0x7373,0x4373,0x4444,0x4444,0x4444,0x4242,0x4442,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4442,0x4442,0x4244,0x4444,0x4444,0x4444,0x4242,0x4442,0x4442,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4244,0x4444,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4442,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4244,0x4442,0x4444,0x4444,0x4244,0x4442,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444
.word 0x4473,0x4442,0xC493,0x93C4,0x83C4,0xC3C4,0x83C4,0x4263,0x7352,0x6352,0x5273,0x6363,0x8352,0x6383,0x6363,0x5252,0x4141,0x4141,0x5252,0x5241,0x4142,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4F4,0xE4F4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7383,0xC3C4,0xC4B3,0x8373,0xC4C3,0xC4C3,0x8363,0xC483,0x83C4,0x6352,0x5252,0x6373,0xC383,0xC4C4,0x7383,0x8363,0xD4C4,0x83C4,0x7383,0x83C4,0x5263,0x7352,0x8383,0x8363,0x8383,0x7373,0x8373,0x4463,0x4244,0x4242,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4242,0x4242,0x4244,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4244,0x4442,0x4444,0x4444
.word 0x4442,0x8344,0xC4C4,0xC493,0x8383,0x8383,0xC3C4,0xC3C4,0x6383,0x5242,0x4252,0x5252,0x6363,0x5263,0x5263,0x4141,0x4141,0x4141,0x5241,0x5252,0x4141,0x4141,0x4141,0x4141,0x5241,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x5241,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC383,0x63C4,0x8383,0xD483,0xC483,0xC483,0x83C3,0x8363,0x8383,0x5273,0xC473,0xB3C3,0x83D4,0x8363,0xC3C3,0xC4C4,0x8363,0x8383,0x83C4,0x6383,0xC4C4,0x83C3,0x6373,0x6352,0x8373,0x83C3,0x7383,0x7363,0x7373,0x4463,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4444,0x4442,0x4242,0x4442,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4442,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x4444,0xC444,0xC4C3,0xC4C3,0xC3C4,0x8383,0xC4C4,0xC4C4,0xC3C3,0x6383,0x4242,0x5252,0x7373,0x5252,0x5252,0x4241,0x7F7F,0x4141,0x6242,0x4252,0x5252,0x4141,0x4141,0x4141,0x5241,0x4141,0x4141,0x4141,0x7F7F,0x6F6F,0x4141,0xD462,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC3D4,0x83C3,0x8383,0x8383,0x8373,0x8383,0x7383,0x6363,0x5252,0x7363,0xC463,0xC483,0x83C3,0x8363,0xC4C4,0x83C3,0x8363,0x8373,0x8383,0x5263,0x7373,0x6373,0x7352,0x5273,0x7352,0x8383,0x83C4,0x7383,0x7383,0x7373,0x4443,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4442,0x4244,0x4442,0x4442,0x4444,0x4442,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242
.word 0x4444,0xD444,0xC4C4,0x6383,0x4442,0xC463,0xC3C3,0xC3C4,0xC4C4,0xD483,0xC3C3,0x8383,0x5273,0x4142,0x5241,0x7F7F,0x5252,0x4141,0x52D4,0x5252,0x4141,0x4141,0x4141,0x4141,0x4142,0x4141,0x7F7F,0x4141,0x7F7F,0x5241,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x5252,0x5252,0x5252,0x5252,0x6262,0x6262,0x62C4,0xC4C4,0x8383,0x8383,0x8383,0x8383,0x7363,0x5252,0x5252,0x7373,0x6352,0xC483,0xC3C3,0xC3D4,0x8383,0x8383,0x6383,0x8363,0x8383,0x5273,0x6373,0x5252,0x8373,0x8363,0x8383,0x8383,0x8363,0x83C3,0x5273,0x7352,0xC483,0x83C4,0x7373,0x7373,0x7373,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4442,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444
.word 0x4444,0xD444,0x83C3,0x4263,0x4444,0xC444,0xC4D4,0xC4C4,0xC4C3,0xC483,0xC4C4,0xC4D4,0x5283,0x7F7F,0x4152,0x7F7F,0x4152,0x5241,0x5252,0x4152,0x4141,0x7F7F,0x7F7F,0x5252,0x4141,0x4141,0x4141,0x4141,0x5252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xC473,0x6383,0x7383,0x8383,0x5273,0x8363,0x73C4,0x8373,0xB3C3,0xC4C3,0x8373,0xC4C3,0xC4C3,0x7383,0x8383,0x5273,0x5263,0x5252,0x8383,0x8363,0xC3C4,0xC3D4,0x7373,0xC3C4,0xC4C4,0x8383,0xC3C4,0x83C4,0x5252,0x8363,0x83C4,0x8383,0x8383,0x7363,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444,0x4244,0x4244,0x4242,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4244,0x4442,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442
.word 0x4444,0xC444,0x63C3,0x4442,0x4444,0xC444,0xC4C3,0x8393,0x4442,0x8383,0xD3C4,0x5283,0x4141,0x5241,0x4141,0x4241,0x4141,0x5241,0x4141,0x5252,0x4141,0x7F7F,0x5252,0x4162,0x4141,0x4141,0x4141,0x5252,0xD4C4,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x7373,0x5252,0x5252,0x6352,0x6383,0xC3C4,0xC3D4,0x8363,0xC4C4,0x83C4,0x8363,0x8383,0x7383,0x7352,0xC483,0x83C3,0x8363,0xC4C4,0xC3C4,0x63C4,0xC483,0xC483,0x63C4,0x8383,0x8383,0x5283,0x7352,0x5263,0x7363,0x5252,0x8363,0x8383,0x7383,0x7363,0x7383,0x4463,0x4444,0x4242,0x4242,0x4442,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4242,0x4242,0x4244,0x4444,0x4244,0x4442,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244
.word 0x4444,0x8344,0x42C4,0x4444,0x4444,0xC444,0x83C3,0x4283,0x4444,0xC383,0x73C4,0x4141,0x4141,0x4242,0x7F7F,0x4141,0x4141,0x5252,0x5252,0x4141,0x4141,0x4141,0x4241,0x4141,0x4141,0x4141,0x4141,0xD452,0xC4D4,0xC4C4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x8363,0xD4C3,0xC4C3,0x83C4,0xC4C4,0x83C3,0x8373,0x8383,0x7383,0x7373,0x7373,0x5263,0xC473,0xC3B3,0xC3B3,0x7383,0x8383,0xC4C4,0x63C3,0x8383,0x8383,0x6383,0x8383,0x8383,0x7363,0xC483,0xC4C4,0x83C4,0x6383,0x6352,0xC483,0x8383,0x6373,0x7383,0x7373,0x4443,0x4442,0x4442,0x4242,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x4444,0x4444,0x4473,0x4444,0x4444,0xC344,0xC4C4,0x4442,0x4444,0x8344,0x5252,0x7F7F,0x4152,0x4141,0x5252,0x7F7F,0x4141,0x5252,0x4141,0x4141,0x7F7F,0x7F7F,0x5252,0x4141,0x4141,0x7F7F,0xD452,0xC4D4,0xD4C4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xC4D4,0xC462,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xC4D4,0x6262,0x6262,0x5252,0x7352,0xC483,0x7383,0xC483,0xC4C4,0x7373,0x7373,0x6373,0x5252,0x5252,0x7352,0x5252,0x8363,0xC3C4,0x9373,0xC4C4,0xC4C4,0x6383,0x7363,0x8373,0x5263,0x6363,0x5263,0x8363,0x5263,0x6352,0x6383,0xC483,0xB3C3,0xC3B3,0x83C4,0x5263,0x7352,0xC483,0x7383,0x8373,0x8373,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444
.word 0x4444,0x4444,0x4444,0x4444,0x4444,0x8344,0x8383,0x4442,0x4444,0x4144,0x7F7F,0x5252,0x4141,0x4141,0x4141,0x4141,0x4242,0x4141,0x5252,0x4141,0x4141,0x4141,0x4142,0x4141,0x4141,0x4141,0xC4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xC4D4,0x4141,0x4141,0x4141,0x4141,0x4141,0xD451,0xD4D4,0xD4D4,0x6262,0x6252,0x5252,0x7351,0xD4C4,0x6383,0x8383,0x7383,0x5252,0x5252,0x6352,0xC473,0xC4C3,0x6383,0xC3C4,0xC3D4,0xC4B3,0x8383,0x8383,0x8383,0x5273,0x5252,0x5252,0xC463,0xC3C3,0x83C4,0x8363,0xD4C3,0xD4D4,0x63C4,0xC483,0xD4C3,0xC4C3,0x8383,0x8363,0x5263,0x8363,0x83C4,0x7383,0x8373,0x8383,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4442,0x4344,0x4243,0x4442,0x4444,0x4244,0x4244,0x4242,0x4242,0x4442,0x4244,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4283,0x4444,0x4244,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0xD452,0xC4D4,0xC4D4,0xD4D4,0xD4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xC4C4,0x4141,0x4141,0xC4D4,0x4152,0x5252,0x4152,0x4141,0x4141,0xD4C4,0xD4D4,0x5252,0x5252,0xC473,0xC3C3,0x5273,0x5252,0x7352,0xC483,0xC4C3,0xC4C3,0x8373,0xC3C4,0x83C4,0x83C4,0xC483,0x83C4,0x7363,0x8383,0x6373,0x8352,0xD4C4,0xC4D4,0x8383,0xC4C4,0xD4C3,0x8383,0xC4C4,0xC3C3,0x83C4,0x8383,0xC3C4,0x8383,0x7373,0xC483,0x73C4,0x5252,0x8373,0x8383,0x7373,0x8383,0x4383,0x4244,0x4444,0x4242,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4442,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x8343,0x4443,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4473,0x4444,0x4242,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0xC4D4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0x5241,0x62C4,0x5252,0xD441,0x52C4,0x4141,0x62D4,0x4152,0x4141,0xC441,0xD4D4,0xD4D4,0x5252,0x8373,0x83C4,0x7352,0xD4C4,0xC4D4,0x8383,0x8383,0xC4C4,0xC463,0x8383,0x63C4,0x8383,0x8383,0x6383,0x7363,0x5252,0x7352,0xC463,0xC3C3,0xD4C3,0x63C3,0x83C4,0xC4C4,0x8363,0x8383,0xC483,0x8383,0x7373,0x8383,0x7383,0x5263,0x7363,0x7373,0x5252,0x7352,0x7383,0x7373,0x8383,0x7383,0x4444,0x4444,0x4242,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4244,0x4244,0x4444,0x4442,0x4244,0x4242,0x4242,0x4344,0xC473,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4242
.word 0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4242,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x5252,0xC4D4,0xD4C4,0xC4C4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0x4152,0x6262,0x4141,0x4141,0x4141,0x4141,0x4141,0xD4C4,0x52D4,0x4141,0x5252,0x5241,0xD4D4,0x6351,0x6363,0x6363,0x8383,0x8383,0x83C3,0x8363,0x8383,0x7383,0x7363,0x8383,0x6363,0x6363,0x5263,0x6352,0xC383,0xC3C4,0xC3D4,0x8363,0xC483,0xC4C4,0x6383,0x8383,0x7383,0x8373,0x7373,0x7383,0x6373,0x5263,0x6352,0x7373,0x8363,0x8383,0x8383,0x8383,0x8383,0xD4D4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x8343,0x63C3,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0xD452,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0x5252,0x5252,0x4141,0x5252,0x4252,0x4241,0x5252,0x4141,0x4141,0x4141,0x4141,0x4152,0x7F7F,0xD452,0x73D4,0x7352,0xC483,0x7383,0x8383,0x7383,0x6363,0x6363,0x5252,0x5252,0x5252,0xC383,0xD4D4,0xC4C3,0x6383,0x8383,0xC4C4,0x83C4,0x7363,0x8383,0x8383,0x6363,0x7373,0x5273,0x6352,0x6373,0x5263,0x7352,0x8383,0xC483,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x42C4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x8373,0x43C4,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242
.word 0x4244,0x4242,0x4244,0x4444,0x4444,0x4444,0x4242,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0xC4C4,0xD4D4,0xC4C4,0xC4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE441,0xD4D4,0x4141,0x5242,0x4152,0x5252,0xC4D4,0x5252,0x5252,0x5252,0x4142,0x4141,0x4141,0xC452,0x4141,0x4152,0xD4C4,0x83D4,0x8383,0x6352,0x5263,0x5252,0xC383,0xC4D4,0x6373,0xC4C3,0x83C4,0xC473,0xC483,0xD4C3,0x83C4,0x8383,0x8383,0x6383,0x5273,0x6352,0x5252,0xC483,0xC4C3,0x7383,0xC373,0xD4B3,0x83C3,0x8373,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4242,0x4242,0x4444,0x4442,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4244,0x4444,0x4444,0x4444,0x4242,0x4244,0x4442,0x4444,0x4444,0x4444,0x4444,0x7344,0xB383,0x4463,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444
.word 0x4444,0x4444,0x4244,0x4444,0x4444,0x4244,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0xC442,0xD4D4,0xC4C4,0xC4C4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xE4D4,0x41D4,0xE4D4,0x4141,0x4241,0xD4D4,0x5252,0x4141,0xC452,0x5262,0x4141,0x5252,0x4252,0x4141,0x4141,0x4141,0x4141,0x5241,0xD4D4,0xD4D4,0x6383,0x8363,0xC4C4,0x6383,0xC4C4,0xC383,0x5283,0x93C4,0x83C3,0x8363,0x8383,0x8383,0x6383,0x7373,0x6363,0x5252,0xD483,0xC3C3,0x7383,0xC483,0xD3D4,0x83C4,0x8363,0xC4C3,0x83C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4244,0x4244,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4244,0x4444,0x4444,0x4244,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4244,0x4444,0x4444,0x4442,0x4344,0x4244,0x4442,0x4242,0x4242,0x4442,0x4244,0x4442,0x4444,0x4444,0x8363,0x83C4,0x4444,0x4244,0x4442,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444
.word 0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x5241,0xD4C4,0xC4C4,0xD4C4,0xC4C4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD441,0xE4E4,0x5241,0x4142,0x6241,0x4141,0x7F7F,0x7F7F,0xC4D4,0x4152,0x5242,0x5151,0x4142,0x4252,0x5241,0x5252,0x5252,0x4141,0xD4D4,0x83D4,0x6363,0x7373,0x6383,0x8373,0x8383,0x8352,0xC483,0x7383,0x7363,0x7373,0x6373,0x8352,0xD4C4,0xC3C3,0x83C4,0xC473,0xC483,0x83C3,0xC463,0xC483,0xC4C3,0x8373,0x8383,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x8373,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x7343,0x83C4,0x43C4,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244
.word 0x4242,0x4442,0x4444,0x4444,0x4242,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0xD452,0xC4D4,0xC4C4,0xD4D4,0xC4C4,0xC4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0x41D4,0x41E4,0x5252,0x4141,0x4141,0x5252,0x5262,0x4141,0xD452,0x52C4,0x5151,0x5252,0x5252,0x4141,0x4141,0x4242,0x5252,0x4142,0xC4C4,0x83D4,0x5263,0x6363,0x5252,0x7363,0x5273,0x7363,0x7373,0x5273,0x5263,0x5252,0x8373,0x7373,0x93C4,0xC4C4,0xC4C3,0x8363,0xC483,0x83C4,0x8373,0x8383,0x83C4,0x8363,0x6273,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0x5262,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0xC473,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0xC373,0xC493,0x4483,0x4444,0x4444,0x4442,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242
.word 0x4444,0x4444,0x4442,0x4244,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x6241,0xC462,0xC4C4,0xD4D4,0xC4D4,0xC4C4,0xD4C4,0xD4D4,0xD4C4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0x5252,0x4141,0x5252,0x4141,0xC452,0x5262,0x5241,0xC4C4,0x4152,0x5241,0x6252,0x5252,0x5252,0x4151,0x5252,0x4252,0x5252,0x5241,0x6383,0x8383,0x7373,0x6383,0x7363,0x5252,0x5252,0x6363,0x8383,0xC4C4,0xD4C3,0x5283,0xC483,0x8383,0xC483,0x8373,0x8383,0x6383,0x7373,0x7373,0x5263,0x6252,0xD462,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x7342,0x73C3,0x4444,0x4442,0x4242,0x4442,0x4442,0x4444,0x7344,0xC4C4,0x8383,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x4444,0x4444,0x4444,0x4242,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0xD4C4,0xC4C4,0xC4D4,0xC4C4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62D4,0x4141,0x4141,0x4141,0xD4C4,0xD4D4,0x41C4,0x5252,0x4141,0x5241,0xC4C4,0x4152,0x5252,0x5252,0x4141,0x4141,0xC4C4,0x4141,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x4252,0x7352,0x7363,0x7383,0xC473,0x5283,0x8383,0xC483,0x83D4,0xC473,0x8383,0xC3C4,0x5283,0x8383,0x8383,0x8383,0x7363,0x7373,0x5252,0x5252,0x5252,0x7373,0x5252,0x62C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x42C4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4442,0x4344,0xC473,0x4463,0x4242,0x4242,0x4242,0x4242,0x4244,0xC443,0x8383,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444
.word 0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0xD4C4,0xC4D4,0xC4D4,0xD4C4,0xC4C4,0xC4C4,0xC4C4,0xC4D4,0xD4C4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD462,0x4141,0x4141,0xC441,0xD4D4,0xD4D4,0xD4D4,0x4141,0x7F7F,0x6262,0x62D4,0x4141,0x4141,0x4141,0x4141,0xD452,0xD4E4,0xD4E4,0xC4C4,0xD4D4,0x4141,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x6363,0x6363,0x6383,0x7383,0x8383,0x6383,0x8383,0x8383,0x8383,0x6363,0x7373,0x6373,0x5263,0x5252,0xC483,0x8363,0xC4C4,0x83C4,0x5252,0xC462,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4444,0x4442,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4442,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0x63B3,0x4444,0x4444,0x4444,0x4444,0x4444,0x8383,0x83C3,0x4483,0x4444,0x4444,0x4444,0x4244,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242
.word 0x4444,0x4444,0x4242,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0xD452,0xC4C4,0xC4C4,0xC4C4,0xC4C4,0xC4C4,0xC4D4,0xD4C4,0xD4D4,0xD4D4,0xD462,0xD4D4,0x6241,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x41D4,0x4141,0x4141,0x5252,0x4141,0x7F7F,0x4241,0x6262,0x5262,0x5241,0x41D4,0x5241,0x4141,0x4141,0xD441,0xD4D4,0x4141,0x4141,0x5252,0x5252,0x5252,0x4152,0x4241,0x5252,0x6352,0x6352,0x6363,0x6363,0x5263,0x7373,0x7373,0x6373,0x7363,0x5252,0x6352,0x8363,0xD4C4,0xC3D4,0x8363,0x8383,0x5273,0x6252,0xC462,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4244,0x4244,0x4444,0x4242,0x4442,0x4442,0x4242,0x4242,0x4444,0x4442,0x4342,0xC483,0x4443,0x4242,0x4442,0x4444,0x4444,0xC483,0x83C4,0x4473,0x4444,0x4444,0x4444,0x4442,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x4444,0x4244,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0xC4C4,0xC452,0xC4C4,0xD4C4,0xC4C4,0xC4C4,0xC4D4,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4C4,0x6251,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xC462,0x5252,0xD4D4,0x6252,0xD4D4,0x6262,0x4141,0x6262,0xD4D4,0xC4C4,0xD452,0x5252,0xE4D4,0xD4D4,0x52D4,0x6262,0x4141,0x7F7F,0x4141,0x4141,0x4252,0x5252,0x5252,0x4141,0x5241,0x5252,0x5252,0x5252,0x5252,0x6363,0xC483,0xC3D4,0x83C4,0xC452,0xC4C4,0xC4C3,0x8373,0x8383,0x83C4,0x8352,0x5273,0x6252,0x6262,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x42C4,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x7343,0x4443,0x4444,0x4444,0x4444,0x4244,0xB373,0x43C4,0x4244,0x4242,0x4242,0x4444,0x8383,0x8383,0x4473,0x4444,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444
.word 0x4444,0x4242,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0xD4D4,0xC4D4,0xD4D4,0xD4C4,0xD4C4,0xD4C4,0xC4C4,0xD4C4,0xD4C4,0xD4C4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0x4162,0xD452,0x41D4,0x6241,0x62D4,0x4141,0x5242,0x5252,0x4141,0x4142,0xC452,0xD4D4,0xD4D4,0xD4E4,0x41E4,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x5252,0x4141,0x5242,0x5252,0xC483,0x83C4,0x5283,0x8383,0x8383,0xC483,0x8373,0x83C4,0x8383,0x8363,0x8383,0x6383,0x6363,0x5252,0xC4C4,0xC462,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xC4C4,0x4442,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4442,0x4442,0x6342,0x4373,0x4444,0x4444,0x4442,0x4444,0x8343,0x73C3,0x4444,0x4242,0x4242,0x4442,0xC4C4,0x8383,0x4473,0x4444,0x4444,0x4444,0x4444,0x4344,0x4483,0x4444,0x4244,0x4444,0x4242,0x4242,0x4442,0x4444,0x4442,0x4444,0x4444,0x4444
.word 0x4244,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0xD452,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x62D4,0xD442,0xD4D4,0xD4D4,0xD4D4,0x4241,0x7F7F,0x4141,0xD462,0x62D4,0x4242,0x5252,0xC4C4,0xD4D4,0xD4E4,0xD4D4,0xC4C4,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x5242,0x4141,0x5252,0x4141,0x6352,0x5242,0x5252,0x6363,0x63C4,0x7373,0x7383,0x6373,0x8352,0x7383,0x6373,0x6363,0x6363,0x6352,0x5273,0x6252,0x6262,0xD4C4,0xD4C4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x7363,0x4443,0x4444,0x4444,0x4444,0x6342,0xC483,0x4463,0x4444,0x4444,0x6344,0xC493,0x8383,0x4483,0x4444,0x4444,0x4444,0x4444,0x8363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4442
.word 0x4242,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0xD4C4,0xD462,0xD4C4,0xD4C4,0xD4D4,0xC4D4,0xD4D4,0xD4C4,0xD4C4,0xD4C4,0xC4D4,0xD4D4,0xD462,0xD4D4,0xC4D4,0xD4D4,0xD441,0xD4D4,0xC4D4,0x6262,0x5252,0x6252,0x4141,0x4141,0x4141,0x5252,0x5252,0x6262,0xC4C4,0x41C4,0x5252,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x5141,0x4141,0x5252,0x4162,0x6262,0x5283,0x5252,0x5263,0x5252,0x5252,0x6363,0x5263,0x6363,0x6363,0x6352,0xC483,0xC4D4,0x5283,0x5252,0x6262,0x6262,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4244,0x8344,0x4473,0x4444,0x4244,0x4444,0x4444,0xC483,0x4483,0x4442,0x4242,0x7344,0x83D4,0x8383,0x4473,0x4444,0x4444,0x4244,0x6344,0x4383,0x4442,0x4444,0x4444,0x4444,0x4244,0x4442,0x4242,0x4442,0x4444,0x4244,0x4242,0x4244
.word 0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0xD4C4,0xC4D4,0xD4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xC4D4,0xD4D4,0xC4D4,0xC4C4,0x62D4,0xD4D4,0x52D4,0x5252,0x5241,0xD462,0x62D4,0x4141,0x5241,0x8F8F,0x4151,0xD452,0xE4E4,0xD4D4,0xD4D4,0x4141,0x4141,0x4141,0x4141,0x4141,0x5252,0x4242,0x4141,0xE441,0x52D4,0x4252,0x63C4,0x73C4,0x5242,0x5263,0x5252,0x7352,0x6363,0x83C4,0x8383,0x5283,0x8383,0x8383,0x4273,0x5252,0xC4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4442,0x4244,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x6342,0x73C4,0x4444,0x4442,0x4444,0x4244,0x8363,0x63C4,0x4444,0x4442,0x8344,0x83C3,0x7383,0x4383,0x4444,0x4442,0x4442,0x8343,0x4383,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4244,0x4242
.word 0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x5252,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xC4C4,0xD4C4,0xC4C4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD462,0xC4D4,0x41D4,0xC4C4,0xD4D4,0xD4D4,0xD462,0xD4D4,0x4141,0x4141,0xD4C4,0x41C4,0x5241,0xD462,0x52D4,0x4241,0xD462,0xD4D4,0xE4E4,0x41D4,0x4141,0x4141,0x4141,0x4141,0x4141,0x5141,0x4152,0x4141,0x5241,0xC4E4,0x4141,0x6252,0xC363,0x5283,0x6352,0x7363,0x8383,0x52C4,0x7373,0x7373,0x6373,0x7373,0x7373,0x5252,0x6262,0xC462,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4444,0x4444,0x4244,0x4244,0x4242,0x4442,0x4444,0x4242,0x4442,0x4442,0x4444,0x4244,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x6342,0x8383,0x4443,0x4444,0x4444,0x4244,0x8344,0xC483,0x4444,0x4444,0x8343,0x83C4,0xC483,0x4383,0x4444,0x4444,0x4444,0xC483,0x4473,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4242
.word 0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x5252,0xC4D4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xC4D4,0xC4D4,0xC4C4,0xD4C4,0xD4C4,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD452,0xD452,0xD4C4,0xD4D4,0xD4D4,0xD4C4,0x4141,0x4141,0x4141,0x4141,0xC452,0xD4D4,0xD4D4,0xC452,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0xD4D4,0x52D4,0x4141,0x6252,0xC483,0x5263,0x5263,0x7363,0x5263,0x6373,0x5252,0x7352,0xC483,0x52C4,0x5252,0xC4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xF4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4244,0xC473,0x4473,0x4444,0x4444,0x4444,0x8342,0x83C4,0x4463,0x4444,0xC473,0xC483,0x8383,0x4383,0x4444,0x4444,0x6344,0xC4C4,0x4473,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4244,0x4444,0x4244,0x4242,0x4242,0x4242
.word 0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x41D4,0x51D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4C4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4C4,0xC4D4,0xC4C4,0xD4D4,0xD4C4,0xD4D4,0x62D4,0xC4C4,0x5252,0xC4C4,0x4141,0x4141,0x4141,0x4152,0x4141,0xD4C4,0xE4D4,0xD4E4,0x41D4,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4241,0x4252,0x4141,0x4141,0x5241,0x62E4,0x5252,0xC4C4,0x8362,0x63C3,0x5252,0x5252,0x5252,0x5263,0x6352,0xC483,0x8363,0x5263,0x6262,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC4D4,0x4452,0x4244,0x4242,0x4242,0x4442,0x4244,0x4244,0x4442,0x4244,0x4444,0x4244,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x8363,0x6383,0x4444,0x4444,0x4444,0x8342,0x83C4,0x4473,0x4344,0xC483,0x8383,0x8383,0x4383,0x4444,0x4444,0x8344,0x83C4,0x4463,0x4444,0x4242,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4244,0x4244,0x4242
.word 0x7F7F,0x4141,0x4141,0x7F7F,0x5252,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x6F6F,0x4141,0x7F7F,0x7F7F,0x4242,0x4141,0xC4D4,0x62C4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4C4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0x52D4,0x5252,0x62D4,0x4141,0xC452,0x41D4,0x5252,0xD462,0x4162,0xC452,0x4141,0xC441,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x5252,0x4152,0x7F7F,0xD4D4,0x4152,0xC4D4,0x52C4,0xC363,0x4283,0x6352,0x8383,0x6383,0x8383,0x8373,0x5263,0x6252,0xC462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52C4,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x6344,0x4463,0x4444,0x4244,0x4444,0x4244,0x8343,0x7383,0x4244,0x4242,0x4244,0xC442,0x8383,0x4483,0x8363,0x73C4,0x83D4,0xC3C4,0x4383,0x4444,0x4444,0xC473,0x83C3,0x4463,0x4444,0x4444,0x4442,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4444,0x4444,0x4244
.word 0x4141,0x4141,0x4141,0x7F7F,0x5252,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4152,0x5252,0xD4D4,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xC4D4,0xD4D4,0xC4D4,0xD4D4,0xC4D4,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD462,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x5241,0x4262,0x5241,0xD4C4,0xD4D4,0x5252,0xD4D4,0x41D4,0x4141,0xD441,0xD4D4,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x5252,0x5252,0x4141,0x4141,0xE441,0x4152,0xE452,0xD451,0x6341,0x73C4,0x5242,0x7373,0x5283,0x6363,0x6363,0x5252,0x5252,0xC462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xF4F4,0xE4E4,0xD4E4,0xD4F4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4444,0x7383,0x4443,0x4444,0x4444,0x4444,0xC444,0x83C4,0x4443,0x4444,0x4244,0xC473,0x83C4,0x4373,0x8363,0x8383,0x8383,0x83C4,0x6373,0x4444,0x4444,0xC483,0x8383,0x4443,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4444,0x4244
.word 0x4141,0x4141,0x7F7F,0x4141,0x6262,0x6262,0x4141,0x6262,0x4141,0x4152,0x4141,0x5252,0x4141,0x4141,0x7F7F,0x7F7F,0x41D4,0xC4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD462,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0x5252,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0xE4D4,0xD4E4,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0xD4D4,0xD4D4,0xD441,0xD441,0x4141,0x8341,0x5263,0x5263,0x5252,0x5252,0x5252,0x5252,0xC462,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x7343,0x73C4,0x4444,0x4244,0x4444,0x8344,0x83C4,0x4243,0x4242,0x4244,0xC483,0x8383,0x8383,0x8383,0xC483,0x83C4,0x83C4,0x8373,0x4444,0x7344,0xC4C4,0x7383,0x4444,0x4444,0x4363,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444
.word 0x4141,0x5252,0x7F7F,0x5241,0x5252,0x5252,0x4152,0x6252,0x5242,0x4141,0x7F7F,0x6252,0x4141,0x4141,0x4141,0x7F7F,0x41D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x6262,0xD4D4,0xD4C4,0xD4C4,0xC4C4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x52D4,0x5252,0x5241,0xC462,0x4141,0x5252,0xC4C4,0x4141,0x52D4,0xE4D4,0xE4D4,0xC4C4,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4241,0x4141,0x4141,0x4141,0x41D4,0xD441,0x6242,0x5252,0xD441,0x4283,0x6352,0x5283,0x7383,0x4252,0x5252,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xD4F4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x6344,0x4444,0x4342,0x83C4,0x4463,0x4444,0x4444,0x8344,0x8383,0x4473,0x4444,0x8342,0xC4C4,0x7383,0xC473,0x8383,0x8383,0xC483,0xC483,0x7373,0x4463,0x8343,0xC483,0x6383,0x4444,0x6344,0x4483,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4442
.word 0x5252,0x4141,0x5252,0x6241,0xC452,0x5252,0x5252,0xD4D4,0xD452,0x52C4,0x4141,0xC4C4,0x6262,0x4141,0x4252,0x4141,0x52D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xC452,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xC4C4,0x4141,0xD452,0xD4D4,0x5241,0xD462,0xD4D4,0x7F7F,0x41D4,0xD4D4,0x4141,0xC4C4,0x4141,0xC4C4,0x41C4,0x4141,0x7F7F,0x4141,0x4141,0x4142,0x4141,0x4141,0x52E4,0x5241,0x4141,0x41D4,0x4142,0x63D4,0x6352,0x6363,0x5263,0x4242,0xC462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4F4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4452,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x6344,0x4373,0x4244,0xC373,0x4383,0x4244,0x4244,0x8363,0x8383,0x4473,0x4244,0xC473,0x83C4,0x8383,0x73C3,0x7383,0x83C4,0x8383,0xC483,0x7383,0x4473,0xC473,0x83C4,0x4383,0x6344,0x8383,0x4463,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4242
.word 0x4152,0x4141,0x5241,0x5252,0x6262,0x5262,0x5252,0xC4C4,0xD462,0x62D4,0x4141,0x62D4,0x6262,0x5252,0x5252,0x4152,0x52D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD462,0xD4C4,0xD4C4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0x41C4,0x6252,0x62D4,0x5252,0xC462,0x52D4,0x4141,0xD4D4,0x5252,0xD4C4,0xD4E4,0xC4C4,0x4141,0xC4C4,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x52E4,0x5241,0x4152,0x4141,0x4141,0xD452,0x5273,0x5252,0x4252,0x6252,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4442,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x7373,0x4444,0x8343,0x7383,0x4444,0x4444,0xC473,0x8383,0x4463,0x7342,0xC3C4,0x7383,0xC473,0x8383,0x8383,0xC483,0x8383,0x8383,0xC4C4,0x6373,0xC483,0x8383,0x4383,0x8363,0x7383,0x4444,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4244,0x4444,0x4444
.word 0x6262,0x4152,0x5252,0xC462,0xC4C4,0xD452,0xC462,0xC4D4,0x52D4,0xD4D4,0x5241,0xC4D4,0xD4D4,0x6262,0xD4C4,0x52D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x6241,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD462,0xD4D4,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x6252,0xD4C4,0x5241,0xD4C4,0xE4D4,0x41D4,0x4141,0x4141,0xD4D4,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x52E4,0x6262,0xD4D4,0xD4D4,0xD4D4,0x41D4,0xD4D4,0x5252,0x5242,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4F4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4244,0x8343,0x4473,0x8342,0x83C4,0x4444,0x7342,0x8383,0x8383,0x4243,0xC483,0x83C4,0x7373,0xC3C4,0x8373,0x8383,0x8383,0x73C4,0x83C4,0x8383,0x8373,0xC3C4,0xC483,0x6383,0xC4C4,0x4383,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242
.word 0x6262,0x6262,0x5262,0xC4C4,0xC4C4,0xD452,0xD4D4,0xD4D4,0xD462,0xD4D4,0x5242,0x62D4,0xD441,0x6262,0xD462,0x62D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC441,0xD4D4,0xD4C4,0xD4D4,0xC4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x4162,0x4152,0x6252,0xC4D4,0x5252,0xD462,0xD441,0x41C4,0xC4C4,0xD4D4,0xC4C4,0x41D4,0x4141,0xC4C4,0x62D4,0x4141,0x4141,0x4141,0x4141,0x4141,0x52E4,0xD441,0x41D4,0xE4D4,0x4141,0xD4D4,0x62D4,0x52D4,0x5252,0xC452,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4452,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4444,0x4444,0x8342,0x63C4,0x8342,0x73C3,0x4443,0x8343,0x83C4,0x83C4,0x7343,0xC383,0x8383,0x8383,0x8383,0x8373,0xC483,0x7383,0x7383,0x83C4,0x8383,0x8373,0x83C4,0x8383,0x8383,0x8383,0x4473,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242
.word 0x6262,0x5262,0xC462,0x62C4,0xC4D4,0xC4D4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4162,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0xD462,0xD4D4,0x5252,0xD4D4,0x41D4,0xC4C4,0x41D4,0x5241,0xD4D4,0xD4E4,0xC4C4,0x4141,0x4141,0xD4C4,0x4162,0x4141,0x4141,0x4241,0x41E4,0x5241,0x41D4,0xD462,0x4141,0x4141,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xF4D4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x44C4,0x4444,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4442,0x7342,0x83C4,0x8363,0x7383,0x4373,0x8373,0x8383,0x7383,0xC483,0x83B3,0x8383,0x8383,0x83C4,0x8383,0x8373,0x8383,0x8383,0xC483,0x83C3,0xC483,0x8383,0x8383,0x8383,0x7383,0x4444,0x4444,0x4444,0x4444,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244
.word 0x6262,0xC4C4,0xC4C4,0xD4D4,0xC4C4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0x62D4,0x42D4,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4D4,0xE4E4,0xE4F4,0xE4E4,0xF4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0x41C4,0xD462,0xD4D4,0xD4D4,0xC4D4,0xD4C4,0xC4C4,0xD4D4,0xD4D4,0x6262,0xC452,0xD4D4,0x5252,0xD4C4,0x51D4,0x6262,0x6262,0x41C4,0x4141,0xD452,0xC4D4,0x4141,0x7F7F,0x4141,0xD4C4,0x4162,0x7F7F,0xD442,0x41D4,0x6241,0x6262,0xD452,0x4141,0x4141,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4442,0x4444,0x4444,0x4244,0x4444,0x4244,0x4442,0x7342,0x83C4,0x8383,0x7383,0x7363,0xC483,0x8383,0x7383,0xC383,0x8383,0x8383,0x8383,0x83C4,0x8383,0x8383,0x8383,0x8383,0xC483,0x8383,0x8383,0x83C4,0x8373,0xC4C4,0x6383,0x4442,0x4444,0x4344,0x4443,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244
.word 0xC4C4,0xC4C4,0xC4C4,0xC4C4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4C4,0xD4D4,0xC4D4,0x4142,0xD462,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xC4D4,0x62D4,0x4141,0x4141,0x4141,0x4141,0x4152,0x4141,0x5252,0x5252,0x6262,0x6262,0x62C4,0xC4C4,0xC4C4,0xD4C4,0xD4D4,0xD4D4,0x4141,0xD4D4,0x41D4,0xE441,0x4152,0xE441,0x4141,0x4141,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4242,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x8342,0x8383,0x8383,0x7383,0x8363,0xC4C4,0x7383,0x8383,0x83C4,0x8383,0x8383,0x83C4,0x83C4,0x8373,0x83C4,0x8373,0x8383,0x8383,0x7383,0x8383,0x83C4,0x8383,0x8383,0x4483,0x6344,0x8373,0x4463,0x4442,0x4444,0x4442,0x4244,0x4242,0x4442,0x4244,0x4244,0x4242
.word 0xC4C4,0xC4C4,0xC4C4,0xD4C4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xE4E4,0xF4E4,0xE4F4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0x4162,0xD462,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x41C4,0xC452,0x4141,0x6252,0xD4D4,0x5241,0xC4C4,0x5252,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x6262,0x5242,0x4152,0xE441,0x4141,0x4141,0xC441,0x52D4,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x7344,0x4444,0x4444,0x8342,0x8383,0xC483,0x7383,0x8373,0x83C3,0x8383,0xC473,0x83C4,0xC473,0x83C4,0x8383,0x7383,0x8363,0x8383,0x8383,0x8383,0x83C4,0x7383,0x83C4,0x83C4,0x8383,0x7383,0x4463,0x7363,0x63C4,0x4444,0x4244,0x4444,0x4244,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444
.word 0xC4C4,0xC4C4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xF4E4,0xE4E4,0xF4F4,0xF4F4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0x4162,0xD462,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0x5252,0x5252,0xC4C4,0xC4D4,0x4141,0xD4C4,0xC4C4,0x4152,0x4141,0x5241,0x5252,0x6241,0x8F8F,0x7F7F,0x7F7F,0x4141,0xD452,0x5151,0xE441,0x4141,0xC4C4,0xD4D4,0x52C4,0x4141,0xD4D4,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4F4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4446,0x4444,0x4444,0x4244,0x4442,0x4242,0x7342,0x4483,0x4444,0xC463,0x73C3,0x8383,0x8373,0x8383,0x8383,0x8383,0xC483,0x8383,0x8383,0x8383,0x8383,0x7383,0x7373,0x8383,0x7383,0x8373,0xC383,0x8383,0x8383,0x8383,0x8383,0x7383,0x7363,0x73C4,0x4243,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4242
.word 0xC4C4,0xC4D4,0xD4D4,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xE4E4,0xE4E4,0xF4F4,0xF4E4,0xF4E4,0xF4F4,0xE4E4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xC4D4,0xD4D4,0xD4D4,0x62D4,0x6241,0xD4D4,0xD4D4,0xC4C4,0x52D4,0x4141,0x5242,0x51D4,0x4141,0x7F7F,0x4141,0x4241,0x4141,0xD452,0xD4D4,0xD462,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x6262,0xC4C4,0xC4D4,0x62E4,0x4141,0x4141,0x4141,0xD4D4,0x52D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4244,0x4242,0x4442,0x4444,0x4444,0x4242,0x4342,0x8383,0x4244,0x8373,0x83B3,0x8383,0x7373,0xC4C4,0x8383,0x7373,0x83C3,0x7373,0x8383,0x7383,0x8383,0x6383,0x6363,0x7373,0x8383,0x8373,0x8373,0xC473,0x8383,0x7383,0xC483,0x6383,0x8373,0x4383,0x4442,0x4442,0x4244,0x4444,0x4444,0x4444,0x4242,0x4244,0x4444,0x4444,0x4442
.word 0xC4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xE4E4,0xE4E4,0xF4F4,0xE4E4,0xE4E4,0xF4F4,0xF4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x4162,0xD462,0xD4D4,0x52D4,0x5252,0xC462,0x4141,0xC452,0x4141,0xD452,0x4162,0xD4C4,0x41D4,0x7F7F,0xD441,0x5252,0x4141,0x4141,0x4141,0x7F7F,0x6262,0x62E4,0x41E4,0x4141,0x4141,0x4141,0xD4D4,0xD4D4,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xE4D4,0xF4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4242,0x4442,0x4442,0x4473,0x4444,0x4244,0xC483,0x7383,0xC473,0x8383,0x83C4,0x8383,0xC4B3,0x83C4,0x8383,0xC4C4,0x8383,0x7383,0x7383,0x8383,0x7383,0x7363,0x8363,0x7373,0x8383,0x6383,0x8383,0x8383,0xC473,0x83C3,0x8363,0xC4C4,0x4263,0x4444,0x4444,0x4344,0x6373,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244,0x4242
.word 0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4F4,0xF4E4,0xF4E4,0xF4F4,0xF4E4,0xF4F4,0xF4E4,0xE4F4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4141,0xD462,0x5252,0xD4D4,0x5241,0xD4C4,0x7F7F,0xD452,0xD4D4,0xD452,0xD4D4,0x5241,0x4141,0x4141,0x4152,0x4141,0x7F7F,0x4141,0x4242,0x62E4,0x6262,0x4162,0x4141,0x4141,0xD4E4,0x52D4,0xC452,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4444,0x4442,0x7344,0x73B3,0x4244,0x8373,0x8383,0xC473,0xC483,0x73C4,0x8373,0x83C4,0x83C3,0x8383,0x8383,0x8383,0x8383,0x7383,0x83C4,0x7373,0x7363,0x6363,0x7383,0x7383,0x8373,0xC4C4,0x8383,0x8383,0x7383,0x8383,0x8383,0x4444,0x4444,0x6344,0x8373,0x4443,0x4244,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4F4,0xE4F4,0xD4F4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xD4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x5241,0x6252,0xD4D4,0x4141,0xC462,0x4152,0x5252,0xC4D4,0x6262,0xD4D4,0xD4D4,0x6262,0x4162,0x4141,0x4141,0x4141,0x6262,0x6262,0x62E4,0x4141,0xD441,0x42D4,0xE4D4,0xE4E4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4446,0x4244,0x4444,0x4444,0x4244,0xC443,0xC483,0x8342,0x83B3,0xC483,0xC473,0x83C3,0x8373,0x8383,0x8383,0xC483,0x83C4,0x8373,0x8373,0x7383,0x8383,0x7383,0x6373,0x7373,0x7363,0x8373,0x8383,0x7383,0x8373,0x8383,0x7373,0xC473,0x73C4,0x4442,0x6344,0x8383,0x4463,0x4244,0x4444,0x4244,0x4244,0x4442,0x4442,0x4444,0x4444
.word 0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x7F7F,0x4141,0x7F7F,0x5252,0x5252,0xC441,0xD4D4,0x5252,0x51C4,0x4141,0x4141,0x6262,0x7F7F,0x4141,0x4141,0x4141,0x4141,0xD4D4,0x4141,0xE441,0x41D4,0x4141,0x4141,0xD441,0x52D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4244,0x4442,0x4242,0x4442,0x4444,0x4342,0x83C4,0x73C4,0x83C4,0xC483,0x8373,0x7383,0x8383,0x8383,0x8383,0xB383,0x8383,0x8373,0x7383,0x73C3,0x8373,0x7373,0x6373,0x8363,0x8373,0x8383,0x8383,0x7373,0x8383,0x83C4,0x7383,0x8383,0x4383,0x4344,0x8373,0x6383,0x4444,0x4442,0x4244,0x4244,0x4242,0x4444,0x4444,0x4244,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4F4,0xE4E4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xC4C4,0xC4C4,0xD4D4,0xC4C4,0xD4D4,0x52D4,0xD441,0x41C4,0x4152,0x5252,0x6262,0xD4D4,0x4141,0x4242,0xD462,0x4162,0xD4D4,0x5241,0x41D4,0x4141,0x4141,0x4141,0x4141,0x6241,0xE4D4,0x41D4,0x4141,0x4141,0x4141,0x4141,0xD441,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xF4E4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4244,0x4244,0x4442,0x4444,0x7342,0x8383,0xC483,0xC473,0xC473,0x8383,0x8383,0x8383,0x8383,0xC483,0x8383,0x8383,0xC4C4,0x8383,0x7373,0xC473,0x7373,0x83C4,0x73C4,0x8383,0x7383,0x8363,0x8383,0x8383,0x7383,0xC3C4,0x4373,0x8373,0x7383,0x4243,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4242,0x4242,0x4242
.word 0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x5262,0x4141,0x5141,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0xC4C4,0xC4D4,0xC4C4,0xC4D4,0x5252,0xD4D4,0xD462,0x52D4,0x4141,0x5151,0xD4D4,0x5241,0xD462,0xC4C4,0x6262,0x4141,0x6262,0x6252,0x4141,0x41D4,0x4141,0x6262,0x41E4,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0xD462,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4444,0x4244,0x4442,0x4242,0x4342,0x7383,0xC473,0xC373,0x8383,0x8383,0x7383,0x8383,0x8383,0x8383,0x83C4,0x8383,0x83C4,0x8383,0x8383,0xC483,0x7373,0x7373,0x73C4,0x7383,0x7373,0x8383,0x8383,0x8373,0x8373,0x8383,0x7373,0xC483,0x4373,0x4442,0x4244,0x4442,0x4442,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4152,0x4241,0x4251,0x5242,0x4252,0x4241,0xC451,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0xC4C4,0x41D4,0xC4D4,0x4162,0x6241,0x5252,0x4141,0x4141,0x5252,0x4141,0x6262,0xC4C4,0x5241,0x62D4,0xD452,0x7F7F,0x6262,0x52C4,0x4141,0x4141,0x4141,0x4141,0x6262,0xD452,0x52D4,0x4141,0x5252,0x4141,0xD4C4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xE4F4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4362,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x8363,0x8383,0x8383,0x7383,0x8383,0x7383,0x8383,0x8383,0xC483,0x83B3,0x73C4,0x8383,0x8383,0xC4C4,0x73C4,0x8363,0x7373,0x6383,0x8373,0x8373,0x7383,0x8373,0x8383,0x8383,0x8383,0xC483,0x83C3,0x4263,0x4244,0x4242,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4442,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xC4D4,0xC4D4,0x5151,0x4252,0x5252,0x6262,0x5252,0x5252,0x4141,0x5141,0xD4C4,0xC4C4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5252,0xD4C4,0xC4C4,0x5252,0x5241,0xC4C4,0xC452,0x52D4,0x4141,0x6262,0x4141,0x4141,0x6262,0x4141,0xD452,0x7F7F,0xE462,0x41D4,0x4141,0x4141,0x5241,0x8F8F,0x5252,0xD4C4,0x5252,0xD441,0x4141,0x52C4,0xD4E4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4362,0x4444,0x4244,0x4444,0x4242,0x4242,0x4442,0x4244,0x8344,0x8383,0x8383,0x8383,0x8383,0x8373,0x8383,0x8383,0xC483,0x8383,0x8383,0x8383,0x7373,0x8373,0x6373,0x8373,0x83C3,0x7373,0x7373,0x7383,0x7373,0x8383,0x73C4,0xC4B3,0x7383,0x8383,0x6383,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4242,0x4242,0x4242,0x4242,0x4244
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xC4D4,0x5252,0x6252,0x6262,0x6262,0x6262,0x5262,0x5252,0x5252,0x4242,0x4141,0xD462,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0xD452,0xC4D4,0xD4D4,0xC4C4,0x5252,0x41C4,0x6262,0xD4D4,0x4141,0x6241,0x4141,0xE4D4,0x4162,0x6262,0xD462,0x5241,0x5252,0xE4D4,0xD4D4,0x5241,0xD462,0xD4E4,0x4162,0x5252,0x5252,0xD4E4,0x7F7F,0x4141,0x4141,0x6262,0xD4C4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xD4F4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x42C4,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x7342,0x73C3,0x8383,0x7383,0x8373,0x7383,0x8373,0x8383,0x8373,0x8383,0xC4C4,0x8383,0x7383,0x6373,0x7363,0x8383,0x7373,0x83C3,0x7373,0x7383,0x8383,0x83C4,0x8373,0xC483,0x7383,0xC483,0x4473,0x4444,0x6363,0x4463,0x4444,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x6262,0x6262,0xD462,0xC4C4,0x6262,0x5252,0x5252,0x5252,0x5252,0x5252,0x5142,0xD4C4,0xC4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x41D4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0x4141,0x7F7F,0x4141,0xD442,0xD4D4,0x41D4,0x7F7F,0x6262,0x6241,0x4162,0x6262,0x4162,0x4141,0xC441,0x62E4,0x4252,0x4251,0xD441,0xD4D4,0x41C4,0x4141,0x4141,0x4141,0x4141,0x62C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xF4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4462,0x4244,0x4442,0x4444,0x4444,0x4244,0x4442,0x4442,0x7342,0x73C4,0x8383,0x8383,0x8383,0x8383,0x8373,0xC483,0x7383,0xC4C4,0x83C4,0x8383,0x8383,0x6373,0x8363,0x8373,0x73C3,0x7383,0x7383,0x8373,0x8383,0x7383,0x8383,0x83B3,0x8373,0x8383,0x4263,0x7343,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0xC4C4,0xD462,0xC4D4,0xC4C4,0x6262,0x62C4,0x6262,0x5252,0x5252,0x5252,0x4141,0x4242,0x4142,0xD462,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x5252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4141,0x6252,0x52D4,0x6262,0xD4D4,0x5252,0xD452,0xD4D4,0x4141,0x4141,0x4141,0x5252,0xC462,0x62E4,0x4252,0x5252,0xD4D4,0x6262,0x7F7F,0x7F7F,0x4141,0x5252,0x6262,0xD441,0xD4D4,0x62C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xF4D4,0xE4E4,0xF4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xC4D4,0xD462,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4662,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4444,0x6342,0x8383,0x83C4,0x7383,0x7383,0x8383,0x7373,0x8383,0x7383,0x7373,0xC483,0x83C4,0x7373,0x6363,0x8373,0x83C4,0x7383,0x7383,0x7373,0x8383,0x8383,0x8383,0xC3C4,0x7383,0x8383,0x4383,0x7363,0x83C4,0x4243,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4442,0x4242,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xC4D4,0xC4C4,0xC4C4,0xC4C4,0xD4D4,0xC4C4,0xC4C4,0xC4C4,0x6262,0x5252,0x5252,0x5252,0x4252,0x4151,0x4242,0x4142,0xD4C4,0xD4C4,0xD4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4141,0xD452,0x4141,0x5241,0xD4D4,0x6242,0x52D4,0x4141,0xD452,0x5252,0xC4C4,0xE4D4,0x52D4,0x4141,0x5252,0x5262,0x6262,0x4141,0xE441,0xD4D4,0x4141,0xD4D4,0x4141,0xD4D4,0xC4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0x41D4,0xD462,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4462,0x4444,0x4442,0x4242,0x4444,0x4444,0x4244,0x4244,0x6342,0x8383,0x8373,0x8383,0x8383,0x8383,0x6383,0x8383,0x83C4,0x8383,0x8363,0x8383,0x6373,0x6363,0x8383,0x8373,0x7373,0x8383,0x8373,0x8373,0x8383,0xC373,0x8383,0x8383,0x8383,0x6373,0x8383,0x4383,0x4442,0x4444,0x4444,0x4244,0x4442,0x4444,0x4444,0x4444,0x4242,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4C4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xC462,0xC462,0x6262,0x5262,0x5252,0x5252,0x5252,0x4141,0x4251,0x4152,0x5241,0xD4C4,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0x52D4,0x5152,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4141,0x6262,0x5252,0x4152,0x6262,0xD4D4,0x4141,0xD441,0x5252,0xD452,0x6262,0xC4D4,0xD462,0xD4C4,0xC4D4,0x5252,0x4141,0x5252,0x6241,0xD4C4,0xD4D4,0x6262,0xD4D4,0x4241,0xD451,0xD452,0xC4D4,0xD462,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x44C4,0x4244,0x4444,0x4444,0x4444,0x4244,0x4442,0x4444,0x4244,0x8383,0x8383,0x73C4,0x7383,0x8373,0x83C4,0x7373,0x8383,0x7383,0x8363,0x8383,0x7383,0x6363,0x8373,0x7363,0x7373,0x73C4,0x8383,0x7383,0x8373,0xC4C3,0x8383,0xC4C3,0x7383,0xC483,0x73C4,0x4343,0x7363,0x4443,0x4444,0x4444,0x4444,0x4242,0x4244,0x4242,0x4444,0x4244
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xC4C4,0x62D4,0x5252,0x5252,0x5252,0x5252,0x5252,0x5242,0x5252,0x5252,0x5251,0xD4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x5252,0xD441,0xD4D4,0xD4D4,0xC4D4,0x6241,0x5262,0x4141,0x4162,0x5252,0x5241,0x5262,0x4141,0xD442,0x5252,0x4141,0x52C4,0x6252,0xD4D4,0x4141,0xD4C4,0xD4D4,0x5262,0x4142,0xD4C4,0xD4D4,0x4141,0xD4D4,0x7F7F,0x4141,0x52E4,0xD4D4,0xD4D4,0xC4D4,0xD4C4,0xD4E4,0xD4D4,0xE4D4,0xF4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x6241,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4462,0x4444,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4244,0xC463,0x8383,0x8383,0x8383,0x8373,0xC483,0x7383,0x8373,0xC483,0x8373,0x6373,0x6373,0x8373,0x8383,0x8373,0x7383,0x8383,0x7383,0x7373,0x8383,0x8383,0xB383,0x83C4,0x8383,0xC483,0x6383,0x8383,0x4373,0x4442,0x4244,0x4442,0x4444,0x4444,0x4444,0x4442,0x4242,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0xC4C4,0x62C4,0x5252,0x5252,0x5252,0x5241,0x4242,0x5252,0x5252,0x5252,0x5241,0xD4C4,0xC4D4,0xC4D4,0xD4D4,0x41D4,0x5252,0x5252,0xD4D4,0xD4D4,0x41D4,0x6262,0xC452,0x4152,0x5241,0xD4C4,0xD441,0xE4E4,0xD4D4,0x41D4,0xD4D4,0x4141,0x4141,0x5252,0x4141,0x4141,0x4141,0xD462,0x41D4,0x6262,0x4141,0x5252,0xD4D4,0xE462,0x4162,0xD452,0xD4D4,0xD4D4,0xD441,0xC441,0x6262,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xF4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD462,0xD4D4,0xF4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4662,0x4244,0x4242,0x4242,0x4442,0x4444,0x4242,0x4242,0x4444,0x7344,0xC483,0x8383,0x8383,0x7383,0x8383,0x83C3,0x7373,0x7373,0x8383,0x7383,0x6363,0x7373,0x8373,0x7383,0x6363,0x7383,0x7373,0x8373,0xC483,0xC473,0x83C4,0x8383,0xC483,0x7383,0xC483,0x6373,0x4444,0x4444,0x4444,0x4242,0x4442,0x4442,0x4444,0x4444,0x4244,0x4244
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC462,0xC4C4,0x6262,0x5252,0x5252,0x4152,0x5252,0x5252,0x5252,0x5252,0x4242,0x6251,0xD4D4,0xD4D4,0xD4C4,0xC4D4,0x5252,0x4152,0xD4C4,0xD4D4,0x62D4,0x5262,0x52E4,0x4141,0x4141,0x5241,0x52E4,0x4141,0xD452,0xE4D4,0xD4E4,0x5252,0x52C4,0x5252,0xC4C4,0xD462,0xD4D4,0xD4D4,0x6262,0x5252,0x5252,0x4141,0xD4D4,0xD462,0xE4D4,0xD4D4,0x4141,0xD462,0xD4D4,0x6262,0xD4D4,0xC4C4,0xD4E4,0xD4D4,0xE4E4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x41C4,0xD462,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4462,0x4444,0x4442,0x4442,0x4244,0x4242,0x4244,0x4444,0x4444,0x6342,0x83C4,0x8383,0x73C4,0x8383,0x7373,0xC483,0x6383,0x7363,0x7383,0x7373,0x7363,0x8383,0x6383,0xC483,0x6383,0x7383,0x7373,0x8383,0x8383,0xC3C4,0x83C4,0x8383,0x8383,0x8383,0x83C4,0x4263,0x4244,0x4242,0x4442,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244
.word 0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x6262,0x6262,0x5252,0x4242,0x5252,0x5252,0x5252,0x5252,0x5252,0x4142,0x4142,0xD4C4,0xD4D4,0xD4D4,0x5252,0x5252,0xC441,0xD4D4,0xD4D4,0x5252,0x5252,0x4141,0x4141,0x52D4,0x4141,0x4141,0x4141,0x5252,0xD4D4,0xE4D4,0xD4E4,0x52D4,0xD4D4,0xD4D4,0x41D4,0x4141,0x41D4,0xD4D4,0xD4D4,0x6262,0x7F7F,0xD4D4,0x62D4,0x6262,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xC452,0xC462,0xD4E4,0xE4D4,0xD4F4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0xD4D4,0xE4D4,0xF4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x44C4,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4442,0x4242,0x4242,0xC463,0xC483,0xC473,0x8373,0x7383,0x8363,0x8383,0x7383,0x7383,0x7383,0x6363,0x7383,0x8383,0x7383,0x8373,0x7383,0x8383,0x7383,0x8383,0x8383,0x8383,0x8383,0x8373,0x8383,0x6383,0x4442,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4242,0x4244,0x4442,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x6262,0xC4C4,0x6262,0x5252,0x5252,0x4242,0x5252,0x5252,0x5252,0x5252,0x5252,0x4141,0x6262,0xC4D4,0xD4D4,0xD4D4,0x5252,0x4152,0xD4C4,0x41D4,0x5262,0xC441,0x4152,0xD441,0x5252,0x4141,0x4141,0x4141,0x4141,0x4141,0xD452,0xD4D4,0x4141,0x6262,0xD4D4,0xE441,0xD4D4,0x4141,0xD441,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0x6262,0x7F7F,0x7F7F,0x4141,0x6242,0x4141,0x4141,0x62C4,0xD4C4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x6262,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4462,0x4244,0x4242,0x4444,0x4242,0x4242,0x4242,0x4244,0x4442,0x4244,0x8343,0x8383,0x73C4,0x73C4,0x8383,0x7373,0x7373,0x7383,0x8373,0x7373,0x6373,0xC483,0x8383,0x6363,0x8363,0x8373,0x7373,0x7383,0x8383,0x8383,0x83C4,0x8383,0x8383,0x83C4,0x4263,0x4244,0x4242,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4242,0x4244,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x6262,0x5252,0x5252,0x4252,0x5252,0x5252,0x5252,0x5252,0x5241,0x4141,0x4141,0x5141,0xD4C4,0xD4C4,0x4241,0x4251,0x5251,0xC4D4,0x5252,0x4141,0x52C4,0xD441,0x4152,0x4141,0x4141,0x4141,0x4241,0x5252,0xD4C4,0xD4D4,0x41D4,0xE462,0xD4D4,0xD4D4,0x6262,0x5241,0x6241,0x62E4,0x7F7F,0x4141,0x6241,0x7F7F,0xD4D4,0x6262,0x4141,0x4141,0x6262,0xD4D4,0xC4C4,0x7F7F,0xC462,0xE462,0xD4D4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C4,0x6242,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4444,0x4442,0x4244,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4444,0x4342,0xC483,0x8373,0x7363,0xC4C4,0x6383,0x8383,0x8383,0x6383,0x7363,0x8363,0x7363,0x7383,0x6383,0x8383,0x8373,0x8373,0x7383,0xC483,0x83C4,0x8383,0x7383,0x83C4,0x6383,0x4442,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4242
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0x6262,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5242,0x4141,0x4141,0x7F7F,0x6241,0xD4D4,0x41C4,0x4242,0x5152,0xD441,0x5252,0x5252,0x4141,0xE441,0x4252,0x5252,0x5252,0x5252,0x5252,0xD4C4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0x6262,0x5252,0x4141,0x5252,0xE4D4,0x41D4,0xD4D4,0xD4E4,0xD4D4,0x6241,0x41D4,0x4242,0xD4D4,0x62D4,0x41D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xC4D4,0xD462,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4462,0x4444,0x4444,0x4242,0x4442,0x4444,0x4442,0x4242,0x4244,0x4444,0x4244,0x8363,0x73C4,0x8383,0x8363,0x8383,0x83C4,0x8383,0x7383,0x6363,0x8383,0x73C4,0x7383,0x7363,0x7373,0x7383,0x7373,0xC483,0xC3C4,0x6383,0x8373,0x8383,0x8373,0x4373,0x4442,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4244,0x4244,0x4242,0x4244,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4C4,0x6262,0x5262,0x5252,0x5252,0x5242,0x5252,0x5252,0x5252,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x6262,0xD4D4,0x5252,0x4141,0x5142,0x4142,0x4241,0x4252,0x5242,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x5262,0x5252,0x4141,0x4141,0x4141,0x5241,0xD4D4,0x4141,0x4141,0x4141,0x4141,0x6262,0xD4D4,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD441,0xD4D4,0xC441,0x62C4,0xD4C4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0xD462,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x44C4,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4242,0x4242,0x4442,0x4342,0x8363,0x7383,0x7373,0xC483,0x8383,0x7373,0x6363,0x8363,0x83C4,0x6373,0x6363,0x8363,0x7373,0x8373,0x8383,0x83C4,0x8373,0xC4C4,0x8373,0x8383,0x6383,0x7342,0x7383,0x7373,0x4363,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4244
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x62C4,0x6262,0x6262,0x5262,0x5252,0x5252,0x5252,0x5252,0x5252,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x6252,0x52D4,0x4141,0x4141,0x4141,0x4142,0x4141,0x4152,0x52D4,0x6262,0xD462,0xD4D4,0xD4D4,0x62D4,0x6262,0x5152,0x5152,0x5252,0x4141,0x4141,0x4141,0x5241,0xE4E4,0x6252,0x6262,0x6262,0x6262,0x6262,0x5252,0x41C4,0xD4D4,0xD4D4,0x41D4,0xC4D4,0xD4D4,0x41C4,0xC4C4,0xC4D4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5262,0xD462,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4462,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4244,0x4444,0x4444,0x4244,0x4244,0x6344,0x8373,0x7373,0x7383,0x8383,0x6383,0x6363,0x7383,0x7373,0x8373,0x7383,0x8373,0x7373,0x8383,0x8383,0x8373,0x83C4,0x8383,0x8383,0x7383,0x7383,0x8383,0x4363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x6262,0x6262,0x6262,0x5252,0x5252,0x5252,0x5252,0x5252,0x4141,0x4141,0x4141,0x4141,0x6F6F,0x4242,0x4242,0x6252,0xC4D4,0x5241,0x4141,0x4141,0x5242,0x5252,0xD441,0x5252,0x4141,0x4142,0x4142,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x5241,0x52E4,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x5252,0x5241,0x5252,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC441,0x6252,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4444,0x4444,0x4244,0x4442,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4444,0x4344,0x4244,0x6343,0x7373,0x7383,0x6363,0x6373,0x6373,0x8383,0x8363,0x7383,0x6373,0x8383,0xC383,0xC4B3,0x8383,0x7383,0x8373,0x8383,0x8383,0x8373,0x8383,0x6363,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4242,0x4442,0x4242,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x62C4,0x6262,0x5262,0x5252,0x5252,0x5252,0x5252,0x5252,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4242,0x4442,0x4242,0xD452,0x52D4,0x5241,0xD4D4,0xD4D4,0xD4D4,0x4141,0x4142,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x5241,0x52E4,0x4141,0x4141,0x4141,0x4141,0xD462,0xE4E4,0x7F7F,0x7F7F,0xD4D4,0xC4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x41D4,0xD4D4,0xC4D4,0x6262,0xD4D4,0xD4D4,0xD4C4,0x6252,0xD4C4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4462,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4242,0x8344,0x4463,0x4244,0x6343,0x7383,0x7363,0x6363,0x6373,0x8373,0x6373,0x8373,0x8363,0x8373,0xC4C4,0xC4C4,0x8383,0x8373,0xC483,0x7383,0x7373,0x8383,0x5273,0x4243,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC462,0x6262,0x5252,0x5252,0x5252,0x5252,0x5151,0x4142,0x4141,0x4141,0x4141,0x6F6F,0x4242,0x4242,0x4242,0x4242,0xC452,0xD4D4,0x4141,0xC452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52C4,0xD4D4,0x5252,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0xE452,0x41D4,0x41D4,0x4141,0x7F7F,0x4141,0x4242,0xE4D4,0xD4D4,0xD4C4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xD4C4,0xD4D4,0xD4D4,0xC4C4,0xD4C4,0xD4D4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x44C4,0x4244,0x4442,0x4444,0x4444,0x4444,0x4242,0x4244,0x4242,0x4444,0x4444,0x8342,0x7383,0x4443,0x8363,0x7373,0x6363,0x6363,0x8373,0x7373,0xC383,0x6383,0x7373,0x6373,0x8363,0x8383,0xC483,0x83C4,0x7383,0x8383,0x8383,0x6363,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4244,0x4244,0x4244,0x4442,0x4444,0x4444,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0x6262,0x6262,0x5252,0x5252,0x5252,0x5252,0x4141,0x4141,0x4141,0x4141,0x6F6F,0x4242,0x4242,0x4242,0x4242,0x4242,0xC452,0xD4D4,0xD4D4,0xC4D4,0x52D4,0x5241,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0xE452,0x4152,0x6262,0x4141,0x4141,0x4141,0x5241,0xD4E4,0xE4E4,0xC4C4,0xC4C4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD451,0xD4D4,0x62C4,0x62C4,0xC4C4,0x62C4,0xC452,0xD4C4,0xD4D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x4462,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x8342,0x8383,0x8383,0x7383,0x6373,0x6352,0x8383,0x6363,0x7373,0x8373,0x6383,0x7383,0x7373,0x7383,0x8383,0x8383,0x8373,0x83C4,0xC483,0x5273,0x4243,0x4444,0x4444,0x4442,0x4442,0x4242,0x4242,0x4442,0x4242,0x4242,0x4244,0x4242,0x4442,0x4442,0x4444
.word 0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xC462,0x6262,0x5262,0x5252,0x5252,0x5252,0x5142,0x5241,0x4141,0x4141,0x7F7F,0x6F6F,0x4242,0x4244,0x4444,0x4242,0x4442,0x4242,0x4142,0xC452,0x4152,0x5241,0xD4D4,0x52D4,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0xC452,0xD4E4,0x4152,0x4141,0x4141,0x4141,0x4141,0xE452,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0x41E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x41C4,0xC441,0x6262,0xD4C4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4662,0x4444,0x4242,0x4242,0x4244,0x4242,0x4442,0x4444,0x4444,0x4443,0x4444,0x6342,0x83C4,0x8373,0x8383,0x6363,0x7373,0x8373,0x6373,0x7383,0x6363,0x8363,0x7383,0x8373,0x8383,0x8383,0x83C4,0xC4C4,0x8393,0x6373,0x4444,0x4444,0x4242,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0x5252,0x5252,0x5252,0x5252,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4242,0x4242,0x4242,0x4244,0x4242,0x4244,0x4442,0x5241,0xC4D4,0x6262,0xC442,0xD4D4,0x41C4,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x5252,0xD4D4,0x51C4,0x4141,0xD4D4,0x4141,0x4141,0x5252,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x52D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0x6252,0xD4C4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4452,0x4244,0x4444,0x4444,0x4242,0x4242,0x4242,0x4244,0x4444,0x4373,0x4444,0x4244,0xC483,0x7383,0x7373,0x5263,0x8373,0x7373,0x8373,0x7363,0x8383,0x7363,0x8383,0x8383,0x8383,0x8383,0x83C4,0x6383,0x6363,0x4243,0x4444,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4242,0x4442,0x4444,0x4444,0x4244,0x4442
.word 0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xD4C4,0x6262,0x6262,0x5252,0x5252,0x4252,0x4142,0x4141,0x4141,0x7F7F,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x5252,0x5252,0x7F7F,0xD4C4,0x62D4,0x4141,0xC441,0xC4D4,0x4141,0x4141,0x4141,0x4141,0x5252,0xE4D4,0x41E4,0x62D4,0xD4D4,0x62D4,0x4141,0x4141,0x4141,0xD442,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0x41E4,0xD4D4,0xD4E4,0xD4E4,0x52D4,0xD451,0xD4D4,0x41D4,0x4141,0x4141,0xC4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x44C4,0x4244,0x4442,0x4442,0x4444,0x4444,0x4444,0x4242,0x4244,0xC4C3,0x4443,0x4444,0x7342,0x83C4,0x6383,0x8373,0x7363,0x83C3,0x6363,0x8373,0x73C4,0x7373,0x6373,0x5263,0x6363,0x4463,0x4242,0x4242,0x4442,0x4244,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4242,0x4242,0x4242,0x4242,0x4444,0x4244,0x4442,0x4444
.word 0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4C4,0x5252,0x5252,0x5252,0x4142,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x4444,0x5244,0x5252,0x5252,0x4141,0x6241,0xD4C4,0xD4D4,0x4152,0xD452,0xD4D4,0xD4D4,0x5252,0xD452,0xE4E4,0x52D4,0x5252,0x4141,0x4141,0x4141,0x4141,0x4141,0x5241,0xE452,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xC4D4,0xE4D4,0xE4D4,0xD4E4,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4152,0x4141,0x4141,0xD441,0xD4D4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4462,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x83C4,0x4373,0x4444,0x4444,0xC483,0x5283,0x6363,0x7363,0x7383,0xC4C4,0x7373,0x8383,0x6383,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4244,0x4242,0x4442,0x4244,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4442
.word 0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x6262,0x5252,0x5252,0x5252,0x4141,0x4141,0x4141,0x7F7F,0x6F6F,0x4242,0x4242,0x4242,0x4442,0x4444,0x4242,0x5151,0x4141,0x4141,0x4141,0x6241,0xD4C4,0xD4D4,0xD4D4,0x52D4,0xD452,0xD4D4,0xD4D4,0x4152,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4241,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x41D4,0xD4E4,0xE441,0xD441,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x4141,0x4141,0x4141,0x7F7F,0xD452,0xE4E4,0xE4F4,0xE4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4244,0x4442,0x4244,0xC363,0x8383,0xC4C4,0x8383,0x8383,0x7383,0x7373,0x8383,0x8363,0x7363,0x8373,0x8383,0x6383,0x4244,0x4244,0x4442,0x4244,0x4442,0x4442,0x4242,0x4244,0x4442,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4242
.word 0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0x6262,0x5252,0x5252,0x5252,0x4152,0x4141,0x7F7F,0x7F7F,0x6F6F,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4141,0x4241,0x4141,0x4141,0x4141,0xC452,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4152,0x5252,0x52D4,0x4141,0x7F7F,0x4141,0x5242,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xC4D4,0x4141,0x4141,0x4141,0xD4D4,0xE4D4,0xD4D4,0x4141,0x4141,0x4141,0x4141,0x5262,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x4462,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4444,0xC442,0x8383,0xC383,0xC3C4,0xC4C3,0x7383,0x6363,0x6363,0x8383,0x7373,0x7363,0xC463,0x7373,0x4444,0x4444,0x4244,0x4242,0x4242,0x4244,0x4242,0x4242,0x4442,0x4444,0x4442,0x4444,0x4444,0x4442,0x4444,0x4442,0x4242,0x4442,0x4444,0x4244,0x4242,0x4244,0x4444,0x4244
.word 0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xC4C4,0x6262,0x6262,0x5252,0x5252,0x4141,0x4141,0x7F7F,0x7F7F,0x4242,0x4244,0x4244,0x4244,0x5244,0x5252,0x5241,0x4141,0x4242,0x4141,0x4141,0x4141,0xC451,0xC4D4,0x62D4,0x4152,0x5251,0xD4C4,0x41D4,0xD452,0x41C4,0xD4D4,0xD4D4,0x4152,0x5242,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0x41D4,0x5252,0x5252,0x62C4,0x4141,0x5241,0xD4D4,0xE441,0x5252,0x41D4,0xC4C4,0x41C4,0x5252,0x6262,0x4141,0x6262,0xE4D4,0xD4D4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4444,0x4242,0x4442,0x4242,0x4242,0x4444,0x4442,0x4444,0x4444,0xC4C4,0x8383,0xC383,0xC3C4,0x8363,0xC383,0x7373,0x6383,0x8373,0x7363,0x8383,0x7373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4244,0x4442,0x4244,0x4244,0x4244,0x4442,0x4444,0x4444,0x4242,0x4244,0x4242
.word 0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xC462,0x5262,0x5252,0x5252,0x4242,0x4142,0x4141,0x6F6F,0x7F7F,0x4242,0x4442,0x4442,0x4444,0x5143,0x4242,0x5242,0x5252,0x5252,0x5252,0x4141,0x4141,0x4141,0xD462,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x6262,0xC4D4,0x4141,0xD4D4,0x52D4,0x4141,0xD452,0xE4D4,0xD4D4,0xD4E4,0x52D4,0x5252,0x4141,0xD4D4,0x41D4,0x4141,0x4141,0xC4C4,0x5252,0x5252,0x4141,0xD4D4,0x4141,0x5252,0xD4D4,0x4141,0x4141,0xC4C4,0x5252,0x6262,0x5262,0xD452,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x4452,0x4244,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x4244,0x4444,0x6342,0xC4C4,0xC4C4,0xC3C4,0x6363,0x8373,0x8363,0x63C4,0x6373,0x6363,0x8363,0x7373,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444
.word 0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4F4,0xE4D4,0xE4E4,0xE4F4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0xC4C4,0x5252,0x5252,0x4252,0x4151,0x4141,0x7F7F,0x6F6F,0x4242,0x4242,0x4242,0x5244,0x5252,0x5252,0x5252,0x5252,0x4141,0x4141,0x4152,0x4142,0x4141,0x4141,0xD462,0xD4D4,0xD4D4,0x5241,0xD4D4,0x5252,0x4152,0x4141,0xE452,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD452,0x4141,0x6252,0x41C4,0xD462,0x5252,0x4141,0x5241,0x6262,0x4141,0xD452,0xD4D4,0x4141,0x4141,0x4141,0x4141,0xC4C4,0x5262,0x4141,0x6262,0xD441,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC462,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x73C4,0x4444,0x8373,0xC3C4,0x6363,0x6373,0x6363,0x7373,0x8373,0x8373,0x83C4,0x73C3,0x4383,0x4444,0x4444,0x4444,0x4442,0x4442,0x4242,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4442,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444
.word 0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x62C4,0x5262,0x6262,0x5252,0x4142,0x4141,0x7F7F,0x6F6F,0x4242,0x4242,0x4444,0x5252,0x5252,0x5252,0x5252,0x5252,0x4252,0x5241,0x4141,0x4141,0x4141,0x4141,0x5252,0xC4C4,0x4141,0xC452,0x6262,0x4141,0x5241,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0x41E4,0x5242,0x4242,0x4141,0x4141,0xD4D4,0xD4D4,0xD4E4,0x6262,0x4141,0x5241,0x6241,0x5241,0x62C4,0x4152,0xD4C4,0x7F7F,0x4141,0x4141,0x4141,0x5262,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x4452,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4242,0x4444,0x4444,0x83C4,0x4473,0x8363,0xC483,0x7363,0x6383,0x8383,0x8363,0x6383,0x7383,0x6363,0x8383,0x7373,0x4444,0x4242,0x4242,0x4442,0x4442,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4242,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4242,0x4244
.word 0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x6262,0x5252,0x5252,0x4152,0x4141,0x7F7F,0x6F6F,0x4242,0x4242,0x5242,0x5252,0x5252,0x5252,0x5252,0x6262,0x5252,0x5252,0x5252,0x4252,0x4142,0x4141,0x4141,0x4141,0x5252,0x5252,0x4152,0x5252,0xC4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4141,0x5251,0xC4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x4141,0xD4D4,0xD4D4,0x52D4,0x7F7F,0x4141,0x5252,0xD4D4,0x41C4,0xC452,0xD4D4,0x5241,0x5262,0x4141,0xD462,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x6262,0x4441,0x4444,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4244,0x4442,0xC444,0x8383,0x83C4,0xC483,0x6373,0x6363,0x6363,0x7363,0x7383,0x6373,0x6363,0x8373,0x8383,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4442,0x4244,0x4442,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62D4,0x6262,0x5262,0x5252,0x4152,0x4141,0x7F7F,0x7F7F,0x4242,0x4242,0x5244,0x6252,0x5252,0x6262,0x6252,0x5252,0x5252,0x6262,0x5252,0x5252,0x5252,0x4242,0x4141,0x4142,0x5241,0x5252,0x5252,0x4141,0x5241,0x5252,0xC462,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x41C4,0x4141,0x4141,0xD441,0x41D4,0x5241,0x5252,0x5252,0xC4C4,0x4141,0xE462,0xD4D4,0x62C4,0x4162,0x5241,0xD4C4,0x5241,0xD441,0x5252,0x41C4,0x4141,0x6262,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x4442,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4442,0x4444,0x7342,0xC483,0xC4B3,0xC3C4,0x5283,0x7352,0x73C3,0x8373,0xC463,0x7373,0xC483,0x8373,0x7383,0x7383,0x4443,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4F4F,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0x6262,0x5262,0x4141,0x5252,0x7F7F,0x7F7F,0x4242,0x4442,0x5252,0x5252,0x6262,0x6262,0x6262,0x5262,0x6262,0x6262,0x5262,0x5262,0x5252,0x4141,0x4141,0x4242,0x4141,0x5241,0x4141,0x4141,0x4141,0x4141,0x4141,0x5251,0x5251,0x4151,0x4141,0x5241,0x62C4,0x4141,0x4141,0x4152,0x41D4,0x4141,0x6241,0x5252,0xC4C4,0x4141,0xD4D4,0xD4D4,0x4141,0x4241,0xD4D4,0xD4D4,0x4141,0xD441,0xD4D4,0x5241,0x52D4,0xD452,0x6241,0x5252,0x6262,0xD4E4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4242,0x4442,0x4242,0xC463,0x83C4,0xC383,0x6383,0x6363,0x6373,0x8383,0x8373,0x6363,0x7363,0x8373,0x83C4,0x7383,0x7383,0x4443,0x4244,0x4442,0x4444,0x4444,0x4444,0x4242,0x4244,0x4242,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244
.word 0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x6262,0x5252,0x5252,0x4142,0x7F7F,0x6F6F,0x4242,0x5242,0x5252,0x6262,0x6262,0x6262,0x5262,0x6262,0x6262,0x6262,0x6262,0x5252,0x6262,0x5252,0x5242,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x5152,0x6252,0x6262,0xC4C4,0x5252,0x4141,0x4141,0x4242,0x4141,0x6252,0x4141,0x4141,0x7F7F,0x4141,0xC4C4,0xD4D4,0xD4D4,0x5252,0xC441,0xD4D4,0xD4D4,0x4141,0x4141,0x62D4,0x4141,0xD452,0xC4C4,0x5241,0x4141,0x6262,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC452,0x4452,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4442,0xC473,0xC4C4,0x83C4,0x7352,0x6373,0x6363,0x8373,0x8363,0x63C4,0x8383,0xC483,0xC473,0x8373,0x6383,0x4444,0x4444,0x4244,0x4442,0x4242,0x4442,0x4442,0x4242,0x4F4F,0x4242,0x4442,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x6262,0x6262,0x4242,0x5241,0x7F7F,0x6F6F,0x4242,0x6252,0x6262,0xC4C4,0xC4C4,0xC4C4,0xC462,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xC462,0xC4C4,0x6262,0x6262,0x5252,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x5252,0x5252,0x7F7F,0x4141,0x4141,0x4141,0xC4C4,0xD4D4,0x4152,0x4141,0xC441,0xC4D4,0x4141,0x4141,0x4141,0x5241,0x4141,0x5241,0xD441,0xD4C4,0x62D4,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x4452,0x4444,0x4242,0x4242,0x4242,0x4244,0x4444,0x4242,0x4442,0x4344,0x4363,0x4444,0x4244,0x7344,0x8383,0x5252,0x6373,0x6373,0x7363,0x6363,0x6363,0x6363,0x6363,0xC483,0xC473,0x8383,0x8383,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4F4F,0x4444,0x4444,0x4242,0x4242,0x4442,0x4442,0x4F4F,0x4442,0x4444,0x4242
.word 0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xF4E4,0xE4F4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0x6262,0x5252,0x4141,0x7F7F,0x7F7F,0x4242,0xC462,0xC462,0xC462,0xD4C4,0xD4C4,0xD4D4,0xD4C4,0xD4D4,0xC4D4,0xC4D4,0xC4C4,0xC4C4,0xC4C4,0x62C4,0x5262,0x4242,0x4152,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0xD4D4,0x52D4,0xC4C4,0x4141,0xD441,0x41D4,0x4141,0x4141,0x7F7F,0x4141,0x4141,0xD441,0xD4D4,0xD4C4,0x6262,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x6262,0x4452,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0xC473,0x4463,0x4444,0x7363,0xC483,0x6363,0x6363,0x7363,0x8373,0x8363,0x6383,0x6383,0x8383,0x9383,0x83C4,0x8383,0x8373,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4242,0x4444,0x4442,0x4444,0x4242,0x6F6F,0x4444,0x6F6F,0x4244,0x4244,0x4242
.word 0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4F4,0xE4E4,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0x6262,0x4152,0x7F7F,0x7F7F,0x5242,0xC4C4,0xC4D4,0xC4D4,0xC4C4,0xC4D4,0xC4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x6262,0x5252,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0xD4D4,0xD4D4,0xD4D4,0x4141,0x4141,0xD4D4,0x4141,0x4141,0x4141,0x4141,0x4141,0x5241,0xD4D4,0xD4D4,0xC4D4,0x62C4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x6262,0x4243,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4242,0x4444,0x4444,0xC442,0x8383,0x8383,0x83B3,0x8383,0x6383,0x6363,0x6373,0x8373,0x7363,0x7383,0x6373,0x6363,0xC463,0x6383,0x8383,0x8373,0x8373,0x63C4,0x4444,0x4244,0x4442,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4F4F,0x4242,0x4444
.word 0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xE4E4,0xF4E4,0xE4E4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x5262,0x5252,0x4141,0x7F7F,0xD441,0xC4C4,0x62C4,0xD4C4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xC4D4,0x6262,0x6262,0x5252,0x4152,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x5252,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0xE441,0xD4D4,0xD452,0x4141,0x4141,0xD4D4,0x5252,0x4141,0x4141,0x4141,0x4141,0xD441,0x5252,0xD4D4,0xD4D4,0xC4D4,0x62C4,0x5262,0xD4C4,0xC4D4,0xD462,0xC4C4,0x6262,0x4443,0x4244,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4444,0x4244,0x4442,0x8373,0xC483,0x83C4,0xC483,0x83C4,0x6363,0x6363,0x8373,0x6363,0x8363,0x7383,0x8383,0x8383,0x83C4,0x8383,0x8363,0x8383,0x8383,0x4373,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4242,0x4242,0x4242,0x4F4F,0x4242,0x4444,0x4242,0x4444,0x4242
.word 0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x6262,0x4252,0x4141,0x6F6F,0x62C4,0xC4C4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x6262,0x6262,0x4252,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0xD4D4,0xE4D4,0xD4D4,0x41C4,0x4141,0xC4D4,0xD4D4,0x4152,0x4141,0x4141,0x4141,0x6262,0x4162,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x5262,0x5252,0x6262,0xC462,0x41C4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4442,0xC473,0x8383,0x8383,0xC3C3,0x7383,0x6352,0x6352,0x7373,0x7363,0x6373,0x6363,0x8373,0x7373,0x8383,0x8363,0x73C4,0x7383,0x8383,0x4463,0x4444,0x4444,0x4444,0x4F4F,0x4442,0x4242,0x4444,0x4F4F,0x4444,0x4444,0x4F4F,0x4444,0x4244,0x4444,0x4242
.word 0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x62C4,0x5252,0x4141,0x5252,0xD4D4,0x62D4,0xD4C4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x62C4,0x62C4,0x5252,0x5242,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x6262,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0xD4D4,0x41D4,0x5241,0x4141,0x4141,0x4141,0x6262,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0x5252,0x5252,0x5252,0x5252,0x5262,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4244,0x6342,0xC4C4,0xC4C4,0xC4C4,0xC4C4,0x6383,0x5252,0x6363,0x8373,0x8373,0x63C3,0x8383,0x8363,0xC483,0x7373,0x6383,0x83C4,0x8373,0x63C4,0x4444,0x4444,0x4242,0x4F4F,0x4242,0x4F4F,0x4442,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444
.word 0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x5252,0x5252,0xC441,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0x6262,0x5252,0x4152,0x4141,0x4141,0x4141,0x4141,0x4142,0x4242,0x5252,0x6262,0x6262,0x5252,0x6262,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0xD4C4,0xD4D4,0xD4C4,0x41D4,0x4141,0xD4D4,0xC4C4,0x5252,0x4141,0x6241,0xD4D4,0xE4D4,0xD4D4,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x6262,0x6262,0x5262,0x5252,0x5252,0x5252,0x4443,0x4244,0x4242,0x4442,0x4442,0x4244,0x4442,0x4242,0x4442,0x4442,0x4244,0x4242,0x4342,0x8373,0xC483,0x6383,0x5252,0x6363,0x7363,0x6363,0x6363,0x8363,0x73C4,0xC373,0x73C4,0x8373,0x8383,0x8373,0x4383,0x4444,0x4444,0x4F4F,0x4F4F,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444
.word 0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x5252,0x4152,0xC462,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0x6262,0x4252,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x5241,0x5252,0x5252,0x5252,0xC452,0x6262,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0xD4C4,0xD462,0xD4C4,0xD4D4,0x4141,0xD4D4,0xD4D4,0x4141,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x5252,0x5252,0x6262,0x6262,0x5262,0x5252,0x5252,0x4444,0x4442,0x4242,0x4442,0x4444,0x4444,0x4242,0x4244,0x4442,0x4444,0x4444,0x7344,0x8383,0x8383,0x5273,0x5263,0x7352,0x7383,0xC483,0x7373,0x6383,0x6373,0x6383,0xC383,0x8383,0x73C4,0x8383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4442
.word 0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x6262,0x6262,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xC4C4,0x5252,0x4252,0x4142,0x4141,0x4141,0x4141,0x4141,0x4141,0x5252,0x5252,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4152,0x4162,0x7F7F,0x4152,0xD441,0xD4D4,0x41D4,0x6262,0x4162,0xD4D4,0xD4D4,0xD4D4,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0x5262,0x5252,0x6252,0x6262,0x6262,0x6262,0x5252,0x5252,0x4442,0x4444,0x4442,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0xC463,0xC483,0xC483,0xC4B3,0x6383,0x5252,0x6352,0x8373,0x8363,0x7373,0xC383,0x8363,0x6373,0x7373,0x8383,0x8383,0x4383,0x4244,0x4442,0x4244,0x4244,0x4444,0x4444,0x4444,0x4244,0x4442,0x4442,0x4444,0x4244,0x4242
.word 0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xF4F4,0xE4F4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xC4C4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0x6262,0x6262,0x5251,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0xC452,0xC4C4,0xD4C4,0xD4D4,0x62C4,0x6262,0xD4D4,0xD4D4,0x6241,0x4162,0x62E4,0x52D4,0x5252,0xD4D4,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC462,0x5262,0x5252,0x5252,0x5252,0x6262,0x6252,0x6262,0x6262,0x5252,0x4444,0x4444,0x4244,0x4242,0x4442,0x4242,0x4444,0x4444,0xC463,0x83C4,0x8383,0x8383,0xC383,0x83C3,0x5273,0x5263,0x6363,0xC483,0x8363,0x8383,0x8373,0x63C4,0x8383,0x7363,0x7383,0x8383,0x4444,0x4444,0x4244,0x4442,0x4242,0x4442,0x4444,0x4444,0x4244,0x4442,0x4442,0x4444,0x4444
*/


/*
Bird_2:
Fish_320:
.word 0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x5050,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x6060,0x6060,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x6060,0x9F9F,0x7070,0x9F9F,0x5050,0x4040,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xDFDF,0xDFDF,0xF4F4,0xCFCF,0xCFCF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x4040,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x8F8F,0x4040,0x4040,0x4040,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x5050,0x6060,0x6060,0x6060,0x9F9F,0x7070,0x7070,0x8080,0xBFBF,0xAFAF,0x6161,0x6060,0x5050,0x6060,0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x9F9F,0xAFAF,0x6161,0x6060,0x5050,0x6060,0x6060,0x6060,0x5050,0x8F8F,0x4040,0x5050,0x7070,0x7070,0xAFAF,0xCFCF,0xF0F0,0xDFDF,0xE3E3,0xCFCF,0xD3D3,0xE0E0,0xD0D0,0xD0D0,0x5151,0x4040,0x4040,0x4040,0x4040,0x4040,0x9F9F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x4040,0x4040,0x6060,0x6060,0x8F8F,0x4040,0x5050,0x6060,0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x6F7F
.word 0x5050,0x5050,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x6060,0x9F9F,0x6060,0x9F9F,0x7070,0x6060,0x7070,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x5050,0x9F9F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x5050,0x8F8F,0x5050,0x9F9F,0x5050,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x6060,0xBFBF,0xDFDF,0xF3F3,0xCFCF,0xE0E0,0xBFBF,0xBFBF,0xD0D0,0xBFBF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0xAFAF,0x9E9E,0xBFBF,0xAFAF,0xBFBF,0xCCCC,0xBFBF,0x9F9F,0x8F8F,0xBFBF,0x9E9E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x6F6F,0x7F7F,0x4040,0x8F8F,0x5050,0x4040,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x5050,0x9F9F,0x6060,0x8F8F,0x6060,0x8F8F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F40
.word 0x4040,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x4040,0x5050,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4050,0x8F8F,0x4040,0x5050,0x8F8F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x6060,0x9F9F,0x6060,0x9F9F,0x6060,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x6161,0x5050,0x5050,0x6060,0x9F9F,0x5050,0x8F8F,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x9F9F,0x6161,0x6060,0x4050,0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x6060,0xAFAF,0xAFAF,0x6060,0xAFAF,0xBFBF,0xDFDF,0xE0E0,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xD0D0,0xC0C0,0x6060,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0xBEBE,0xBFBF,0xAFAF,0xBCBC,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x5050,0x5050,0x5050,0x8F8F,0x6060,0x7070,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F
.word 0x8F8F,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x9F9F,0x4141,0x4040,0x7F7F,0x6F6F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x5050,0x6060,0x9F9F,0x6060,0x7070,0x6060,0x9F9F,0x7070,0x7070,0x9F9F,0x6060,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x4040,0x4040,0x7F7F,0x6F6F,0x4040,0x6F6F,0x4040,0x7F7F,0x9F9F,0xAFAF,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x5050,0x8F8F,0x9F9F,0xD0D0,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xF0F0,0xCFCF,0xE0F0,0xF3F3,0xF0F0,0xE0E0,0xBFBF,0xBFBF,0x9F9F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x6060,0x9F9F,0xAFAF,0x5050,0x4040,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F7F
.word 0x4040,0x4040,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x5050,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x6060,0x9F9F,0x6060,0x7070,0xAFAF,0x6060,0x6060,0x6060,0x9F9F,0x7070,0x5050,0x4040,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x6060,0x8F8F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x9F9F,0x6060,0x5050,0x4040,0x5050,0x5050,0x9F9F,0x9F9F,0x6060,0x6060,0xAFAF,0xCFCF,0xD0D0,0xAFAF,0xD0D0,0xBFBF,0xCFCF,0xF0F0,0xDFDF,0xE0E0,0xDFDF,0xF0F0,0xD0D0,0xC0C0,0x6060,0x4040,0x7F7F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x6060,0x6060,0x6060,0x9F9F,0x5050,0x6060,0x8F8F,0x4040,0x8F8F,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F6F
.word 0x7F7F,0x4040,0x4040,0x5050,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x9F9F,0x9F9F,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x5050,0x6060,0x7070,0x7070,0x7070,0x9F9F,0x6060,0x9F9F,0x6060,0x9F9F,0x9F9F,0x8F8F,0x5050,0x9F9F,0xAFAF,0xAFAF,0x7070,0xAFAF,0x9F9F,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x8F8F,0x7070,0x5050,0x4040,0x9F9F,0x9F9F,0x9F9F,0x7070,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xE0E0,0xCFCF,0x8080,0xBFBF,0xBFBF,0xE0E0,0xE0E0,0xF3F0,0xF3F3,0xF0F0,0xE0E0,0xBFBF,0xD0D0,0xBFBF,0x9F9F,0x4040,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x8F8F,0x9F9F,0x7070,0x9F9F,0x5050,0x9F9F,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x5050,0x8F8F,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F6F
.word 0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x9F9F,0x6161,0x9F9F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x7070,0x9F9F,0x6050,0x9F9F,0x5050,0x6060,0x5050,0x9F9F,0xAFAF,0xAFAF,0xC0C0,0xBFBF,0xC0C0,0x8080,0x9F9F,0x6060,0x8F8F,0x8F8F,0x7F7F,0x4040,0x9F9F,0xAFAF,0x6060,0x8F8F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xE0E0,0xE0E0,0xD0D0,0xCFCF,0xDFDF,0xE0E0,0xAFAF,0x7070,0xAFAF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xE0E0,0xCFCF,0xCFCF,0xE0E0,0xBFBF,0x6060,0x5050,0x9F9F,0xAFAF,0x9F9F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x5050,0x6060,0x9F9F,0x7070,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x6F6F,0x6F40
.word 0x4040,0x4040,0x4040,0x8F8F,0x5050,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x5151,0x8F8F,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x5050,0x5050,0x6060,0x6060,0x6060,0x7070,0x7070,0x5060,0x8F8F,0x5050,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xAFAF,0x8080,0xAFAF,0xBFBF,0x7070,0xAFAF,0x6060,0x5151,0x4040,0x8F8F,0x9F9F,0x7070,0x9F9F,0x8F8F,0xAFAF,0xE0E0,0xD0D0,0xCFCF,0xCFCF,0xDFDF,0xDEE0,0xE0E0,0xF0F0,0xBFBF,0x7070,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xE4E4,0xE0E0,0xCFCF,0xE0E0,0xD0D0,0xE0E0,0xBFBF,0x9F9F,0x8F8F,0xAFAF,0xBFBF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x8F8F,0xAFAF,0x9E9E,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x5050,0x6060,0x9F9F,0x9F9F,0x5050,0x8F8F,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x6F6F,0x7F6F
.word 0x4040,0x7F7F,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x6161,0x6161,0x9F9F,0x5050,0x4040,0x7F7F,0x8F8F,0x5050,0x5050,0x9F9F,0x5050,0x6060,0x9F9F,0x7070,0x6060,0x6060,0x9F9F,0x6060,0x9F9F,0x5050,0x8F8F,0x5050,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xD0D0,0x7070,0xC0C0,0xAFAF,0xBFBF,0x7070,0x9F9F,0x4141,0x8F8F,0x8F8F,0x7070,0x6060,0x5050,0x9F9F,0xBFBF,0xDFDF,0xE0E0,0xCFCF,0xF0F0,0xDFDF,0xF0F0,0xDFDF,0xF0F0,0xE0E0,0xBFBF,0xBFBF,0x7070,0x9F9F,0xBFBF,0xBFBF,0xCFCF,0xE3E3,0xBFBF,0xE3E3,0xBFBF,0xBFBF,0x6060,0x9F9F,0xAFAF,0xD0D0,0xAFAF,0x4040,0x7F7F,0x4040,0x7F7F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0x9E9E,0xBFBF,0xAFAF,0x8F8F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x4040,0x5050,0x8F8F,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x4040,0x4040,0x6F6F,0x4040,0x7F6F
.word 0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x4040,0x5050,0x8F8F,0x9F9F,0x5050,0x8F8F,0x7F7F,0x4040,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x7070,0x6060,0x6060,0x6060,0x7070,0x9F9F,0x5050,0x5050,0x6060,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0x7171,0x9F9F,0x9F9F,0xC0C0,0xAFAF,0xAFAF,0x6161,0x9F9F,0x8F8F,0x5050,0x6060,0xAFAF,0x9F9F,0x6060,0xBFBF,0xE0E0,0xE0E0,0xD0D0,0xBFBF,0xE0E0,0xCFCF,0xCFCF,0xE0E0,0xD0D0,0xD0D0,0xC0C0,0xAFAF,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xE0E0,0xD0D0,0xBFBF,0xE4E4,0xBFBF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0xBFBF,0xBFBF,0xBFBF,0x9E9E,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x5050,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F7F
.word 0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x8F8F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x6060,0x9F9F,0x6060,0x6060,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x8F8F,0x4040,0x8F8F,0x9F9F,0x6060,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x6060,0x7070,0xAFAF,0xBFBF,0xC0C0,0xAFAF,0x9F9F,0x6060,0x8F8F,0x9F9F,0x9F9F,0x7070,0x9F9F,0xBFBF,0xCFCF,0xBFBF,0xC0C0,0xAFAF,0xBFBF,0xC0C0,0xD0D0,0xBFBF,0xD0D0,0xBFBF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xD4D4,0xE0E0,0xCFCF,0xD0D0,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x6060,0x7F7F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0xBFBF,0xBFBF,0xBEBE,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x4040,0x8F8F,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x7F7F,0x5040,0x5050,0x8F8F,0x4040,0x4040,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F6F
.word 0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5060,0x6060,0x6060,0x9F9F,0x6060,0x9F9F,0x7070,0x6060,0x5050,0x8F8F,0x5050,0x9F9F,0xAFAF,0xAFAF,0x7070,0xAFAF,0xD4D4,0x6161,0x9F9F,0x9F9F,0xC0C0,0xC0C0,0xAFAF,0xC0C0,0x6161,0x9F9F,0x6060,0x6060,0x7070,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0x8080,0x9F9F,0x7070,0x7070,0xAFAF,0xBFBF,0xC0C0,0xAFAF,0x7070,0x6060,0x9F9F,0xBFBF,0xD3D3,0xAFAF,0xD0D0,0xE0E0,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0x8F8F,0x8F8F,0x5151,0x5050,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0x4040,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x4040,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x8F8F,0x8F8F,0x5050,0x8F8F,0x5050,0x4040,0x4040,0x8F8F,0x5050,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x407F
.word 0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x6060,0x5050,0x5050,0x9F9F,0x9F9F,0x6060,0x7070,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x5050,0x8F8F,0x6060,0xC0C0,0xC0C0,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xC0C0,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xC0C0,0xAFAF,0xC0C0,0x8080,0x6060,0x5050,0x9F9F,0xAFAF,0xD0D0,0xC0C0,0xC0C0,0x7070,0xAFAF,0x6060,0xAFAF,0xCFCF,0xCFCF,0xE0E0,0xD0D0,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0x4141,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x8F8F,0x5050,0x9F9F,0x5050,0x8F8F,0x5050,0x8F8F,0x4040,0x5050,0x5050,0x5050,0x5050,0x4040,0x8F8F,0x4040,0x5050,0x8F8F,0x4040,0x8F8F,0x5050,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F
.word 0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x5050,0x6060,0x5050,0x5050,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x5050,0x5050,0x8F8F,0x7070,0xBFBF,0x6161,0x6060,0x7070,0x6161,0xAFAF,0xAFAF,0x9F9F,0x7070,0xC0C0,0x7070,0x7070,0xAFAF,0xAFAF,0xAFAF,0x6060,0xC0C0,0xAFAF,0xC0C0,0xAFAF,0x7070,0x9F9F,0x5050,0x7070,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x7070,0x7070,0xAFAF,0xBFBF,0xE0E0,0xF0F0,0xE0E0,0xBFBF,0xBFBF,0xBFBF,0xDFDF,0xFAFA,0xCFCF,0xAFAF,0x7F7F,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4040,0x8F8F,0x5050,0x8F8F,0x4040,0x8F8F,0x5050,0x9F9F,0x6060,0x8F8F,0x5050,0x8F8F,0x8F8F,0x5050,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F40
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x5050,0x8F8F,0x5050,0x9F9F,0x6060,0x6060,0x9F9F,0x7070,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x5060,0x9F9F,0x5050,0x6060,0xD0D0,0xC0C0,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x6161,0x7070,0x9F9F,0xAFAF,0x7070,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x7070,0x9F9F,0xD0D0,0xAFAF,0x7070,0x7070,0x5050,0x8F8F,0x8F8F,0xC0C0,0xBFBF,0xBFBF,0x8080,0x6060,0xAFAF,0xC0C0,0xE0E0,0xCFCF,0xE0E0,0xE0E0,0xD0D0,0xBFBF,0xBFBF,0xCFCF,0xDFDF,0xEBEB,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x5050,0x5050,0x4040,0x4040,0x5050,0x6060,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F
.word 0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6060,0x6060,0x5060,0x5050,0x5050,0x6060,0x6060,0x6060,0x6060,0x7070,0x9F9F,0x6060,0x9F9F,0x5050,0x8F8F,0x6060,0xAFAF,0xC0C0,0xAFAF,0x6161,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x7070,0x7070,0xAFAF,0x6060,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x6060,0x8080,0x7070,0xAFAF,0x6060,0x5050,0x5050,0x9F9F,0xAFAF,0xC0C0,0xD0D0,0xBFBF,0xAFAF,0xC0C0,0xD0D0,0xD0D0,0xE0E0,0xBFBF,0xD0D0,0xBFBF,0xBFBF,0xBFBF,0xEBEB,0xCFCF,0xF6F6,0xCFCF,0xBFBF,0xCFCF,0xAFAF,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x9F9F,0xAFAF,0xBCBC,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x4040,0x6F6F,0x4040,0x6F6F,0x6F6F,0x4040,0x8F8F,0x5050,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x6060,0x5050,0x8F8F,0x8F8F,0x5050,0x9F9F,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x407F
.word 0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x5050,0x6060,0x6060,0x6060,0x5050,0x9F9F,0x6060,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x9F9F,0xC0C0,0xAFAF,0x6161,0xAFAF,0x5151,0x8F8F,0x4040,0x5050,0x5050,0x9F9F,0xAFAF,0x6060,0x9F9F,0x9F9F,0x9F9F,0x7070,0x9F9F,0x9F9F,0x6060,0x9F9F,0x6060,0x9F9F,0x5050,0x9F9F,0x8080,0xBFBF,0xAFAF,0xBFBF,0xE0E0,0xE0E0,0xAFAF,0xC0C0,0xBFBF,0xD0D0,0xD0D0,0xCFCF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xEBEB,0xDBDB,0xDBDB,0xEBEB,0xBFBF,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xBCBC,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x5050,0x9F9F,0x5050,0x5050,0x4040,0x4040,0x4040,0x6060,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x6060,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x8F7F
.word 0x7F7F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x6060,0x6060,0x6060,0x5050,0x5050,0x6050,0x6060,0x9F9F,0x7070,0x7070,0x9F9F,0x6060,0x6060,0x9F9F,0x7070,0x7070,0xC0C0,0xC0C0,0xC0C0,0xC0C0,0xAFAF,0x5050,0x4040,0x7F7F,0x8F8F,0x4040,0x6060,0x7070,0xAFAF,0x5050,0x5050,0x6060,0x9F9F,0x6060,0x6060,0x8F8F,0x5050,0x6060,0x5050,0x9F9F,0xD0D0,0xBFBF,0x8080,0x7070,0xE0E0,0xCFCF,0xE0E0,0xC0C0,0x7070,0xD0D0,0xE0E0,0xE0E0,0xD0D0,0xAFAF,0xAFAF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xCFCF,0xCFCF,0xF6F6,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x8F8F,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x7F7F,0x5050,0x9F9F,0x5050,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x7F40
.word 0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x5050,0x6060,0x6060,0x6060,0x6060,0x5050,0x9F9F,0x6060,0x7070,0x9F9F,0x7070,0x7070,0x7070,0xAFAF,0xD0C0,0xD0E0,0xD0D0,0xD0D0,0xD0E0,0xC0E3,0xB2C2,0x71C1,0x6161,0x5050,0x4040,0x4040,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6060,0xAFAF,0x9F9F,0x4040,0x4040,0x8F8F,0x8F8F,0xAFAF,0xBFBF,0xCFCF,0xD0D0,0x7070,0xBFBF,0xF0F0,0xF0F0,0xBFBF,0x9F9F,0x7070,0xBFBF,0xE0E0,0xD0D0,0x6161,0x9F9F,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xDBDB,0xEBEB,0xEAEA,0xCFCF,0xAFAF,0x8F8F,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0xAFAF,0xCCCC,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x4040,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x5050,0x9F9F,0x9F9F,0x5050,0x6060,0x8F8F,0x5050,0x4040,0x8F8F,0x4040,0x7F7F,0x5050,0x9F9F,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x5050,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F
.word 0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x4040,0x5050,0x5050,0x6060,0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x6060,0x7070,0x8080,0x8080,0xC0C0,0xE0E0,0xD0D0,0xE3D0,0xD3E3,0xD0D3,0xD3D3,0xD3D0,0xD3D0,0xC2C1,0xC2B1,0x81C1,0x7171,0x7181,0x9F9F,0x5050,0x7070,0x8080,0x7070,0xAFAF,0x6060,0x6060,0x9F9F,0x7070,0x6060,0x9F9F,0x7F7F,0x4040,0x8F8F,0xD0D0,0xE0E0,0xE0E0,0xBFBF,0x9F9F,0x7070,0xD0D0,0xBFBF,0x7070,0x6060,0x9F9F,0xC0C0,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x9F9F,0xBFBF,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x4040,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x6060,0x6060,0x9F9F,0x6060,0x5050,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x5050,0x6060,0x9F9F,0x5050,0x8F8F,0x5050,0x8F8F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x408F
.word 0x7F7F,0x6F6F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0x5050,0x9F9F,0x6060,0x6060,0x5050,0x6060,0x5050,0x5050,0x9F9F,0x6060,0x7070,0xD0C0,0xE0D0,0xD0D0,0xC0C0,0xD3D3,0xC1C1,0xC1D0,0xC3C1,0x8181,0xC0C3,0xC0C3,0xB1C3,0xB1B2,0xC1B1,0xC0C1,0xB1C3,0x71A1,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xC0C0,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x5050,0x4040,0x7F7F,0x4040,0xAFAF,0xCFCF,0xD0D0,0xC0C0,0x9F9F,0xAFAF,0xC0C0,0xAFAF,0x6060,0x9F9F,0x6060,0x8080,0x8080,0x7070,0xAFAF,0x6060,0x9F9F,0x5151,0x9F9F,0x9F9F,0xBFBF,0xDBDB,0xEAEA,0xCFCF,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x5050,0x6060,0x9F9F,0x9F9F,0x6060,0xAFAF,0x5050,0x9F9F,0x5050,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x8F50
.word 0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x5050,0x9F9F,0x6060,0x6060,0x6060,0x9F9F,0x5050,0x6060,0x5050,0x6060,0xC0C0,0xD0D0,0xD0D0,0xE3D0,0xC1D0,0xC1C3,0xC0D0,0xD0C2,0xD0D3,0xD1D0,0xD0D0,0xBFBF,0xD0C0,0xC3C0,0xD0C0,0xC0C3,0xC2B2,0xB181,0xC1B1,0x70C0,0x8180,0xC0B0,0xC0C0,0xBFBF,0xD0D0,0xC0C0,0xC0C0,0xAFAF,0x7070,0x9F9F,0x7F7F,0x4040,0x5050,0x9F9F,0xD0D0,0xE0E0,0xD0D0,0x7070,0xC0C0,0xAFAF,0x6060,0x9F9F,0x5050,0x7070,0xAFAF,0x7070,0x9F9F,0xAFAF,0xAFAF,0x6161,0xAFAF,0x5151,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCBCB,0xBFBF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x5151,0x5050,0x8F8F,0x6060,0x9F9F,0x5050,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x5050,0x8F8F,0x5050,0x5050,0x9F9F,0x5050,0x9F9F,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x8F8F,0x5050,0x8F8F,0x8F8F,0x5050,0x8F8F,0x4040,0x4040,0x8F8F
.word 0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x5050,0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x7070,0xC0C0,0xD0D0,0xC0C3,0xC0C3,0xC2C2,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xC1D2,0xD3D1,0xB1C0,0x81B2,0xC2C3,0xD0D0,0xD0D0,0x71C0,0xD0D0,0xB1C0,0xC1B2,0xB1C2,0xC2B1,0xC071,0xC0C0,0xD0D3,0xD0D0,0xD0D0,0x8080,0xAFAF,0x6060,0x4040,0x7F7F,0x7F7F,0x5050,0xBFBF,0xCFCF,0xC0C0,0xD0D0,0xBFBF,0xD0D0,0x7070,0x4040,0x8F8F,0x6060,0x7070,0x9F9F,0x6060,0x6060,0x6060,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x9F9F,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x4040,0x8F8F,0x5050,0x9F9F,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x4040,0x7F7F,0x5050,0x408F
.word 0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x6060,0xD0D0,0xC0C0,0xC3C0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xC2D0,0xC1C1,0xB1B1,0xB2B2,0xB1B2,0xB2B2,0xB2B1,0xA1B1,0xB2B2,0xB2B2,0xC1C3,0xC0C0,0xD0D0,0xB1C3,0xB1B1,0xA1A1,0xC2B1,0xB2C3,0xB1A1,0xC0C3,0xC0C0,0xD4C0,0x71C0,0xAFAF,0x4040,0x7F7F,0x4040,0x7F7F,0xC0C0,0xC0C0,0xBFBF,0xC0C0,0xD0D0,0xBFBF,0x5050,0x8F8F,0x4040,0x9F9F,0x7070,0x4040,0x9F9F,0x5151,0x9F9F,0xAFAF,0x5252,0x4141,0x8F8F,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xBFBF,0xCFCF,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0x5050,0x4040,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x4141,0x8F8F,0x4040,0x4040,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x5050,0x9F9F,0x6060,0x8F8F,0x5050,0x9F9F,0x6060,0x8F8F,0x6060,0x6060,0x6060,0x9F9F,0x9F9F,0x9F9F,0x5050,0x4040,0x8F8F,0x8F8F
.word 0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0xD0D0,0xC2D2,0xD0D3,0xCFCF,0xD3D0,0xC1C2,0xB1C1,0xA1B2,0xB2B2,0xB2A1,0x81B2,0x81B2,0xB2B2,0xB2B2,0xB2B2,0xA1B2,0xA1B2,0xB2A1,0xB1B2,0xD0C2,0xD0D0,0xC381,0xC2C0,0xA1A1,0xA1B1,0xA1B2,0xB1B1,0xC3B2,0x81C0,0xB1B1,0x7191,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x9F9F,0x8080,0xD0D0,0xE0E0,0xC0C0,0x9F9F,0x4040,0x7F7F,0x4040,0x6060,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x6262,0x9F9F,0x9F9F,0x8F8F,0xBFBF,0xCFCF,0xAFAF,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4040,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x4141,0x4141,0x7F7F,0x8F8F,0x8F8F,0x4141,0x7F7F,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x6060,0x5050,0x9F9F,0x6060,0x9F9F,0x6060,0x6060,0x9F9F,0x8F8F,0x8F8F,0x5050,0x8F50
.word 0x5050,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x5050,0x5050,0x6060,0x5050,0x5050,0x6060,0xD0D0,0xD3E0,0xD0C1,0xF0F0,0xD0E0,0xA1C0,0xB2B1,0xB2B2,0xB2A1,0xB2A1,0xA1B2,0xB2B2,0xB2B2,0x92B2,0x92B2,0xA1B2,0xB2B2,0xB2B2,0xB2B2,0xA1B2,0xB2B2,0xC3B2,0xD0D0,0xD0D0,0xB1C0,0xB1B1,0xA1A1,0xA1B2,0xB1B1,0xC3B1,0xB1B1,0xB1B1,0x7181,0x4060,0x4040,0x8F8F,0x5050,0x6060,0x7070,0x8080,0xBFBF,0xBFBF,0x4040,0x7F7F,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x4141,0x9F9F,0x9F9F,0x8F8F,0x4141,0x8F8F,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x9F9F,0xBFBF,0xCFCF,0xCFCF,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xAFAF,0x8F8F,0x4040,0x4040,0x4040,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x5050,0x8F8F,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x8F8F,0x8F8F,0x5050,0x8F8F,0x5050,0x9F9F,0x6060,0x9F9F,0x6060,0x9F9F,0x9F9F,0x6060,0x6060,0x5050,0x9F9F,0x8F9F
.word 0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x9F9F,0xE0E0,0xD1E3,0xF0E0,0xD0D0,0xB2B1,0xB2B2,0xB2B2,0xB2A1,0xB2B2,0xB2B2,0xB292,0x92B2,0xB2B2,0xB281,0xB281,0xB2B2,0xB292,0x9281,0xB2B2,0xB292,0xB2B2,0xB1B2,0xC381,0xD0C3,0xD3D3,0x71D0,0xB2B2,0xB2A1,0xB2A1,0xB1B1,0x81B1,0xA1B1,0xC1B1,0x71C1,0x7161,0x8181,0x6080,0x5050,0x7070,0xAFAF,0xBFBF,0xD0D0,0x9F9F,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x9F9F,0x9F9F,0x5252,0x9F9F,0x4141,0x4141,0x4141,0x5151,0x4141,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x9F9F,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xD0D0,0xBFBF,0xD0D0,0xD0D0,0xBFBF,0x6060,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x5050,0x5050,0x8F8F,0x9F9F,0x9F9F,0x4040,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x6060,0x5050,0x9F9F,0x6060,0x509F
.word 0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x5050,0x9F9F,0x6060,0xD070,0xD0D0,0xD1C1,0xDFDF,0xC1E0,0xA1B2,0xB2B2,0xB2A1,0x92B2,0xB2B2,0x8292,0xB2B2,0xB282,0x82B2,0x82B2,0x81B2,0x82B2,0x8281,0x81B2,0x81B2,0x81B2,0x81B2,0xB281,0xC0C2,0xC0C2,0xD0D3,0xBFBF,0x71C0,0xC1C1,0xB2B1,0xA1B2,0xB2B2,0x8181,0xA1B1,0xB1A1,0x81C1,0xB1C2,0xC2B1,0x8080,0x6060,0xC0C0,0xBFBF,0xBFBF,0x5050,0x8F8F,0x7F7F,0x7F7F,0x4040,0x5050,0x9F9F,0x9F9F,0x9F9F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x4141,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x8F8F,0xBFBF,0xDFDF,0xDFDF,0xDFDF,0xD0D0,0xD0D0,0xD0D0,0xCFCF,0xD0D0,0x8080,0xAFAF,0x6060,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x6060,0x6060,0x6060,0x9F9F,0x5050,0x9F9F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x9F9F,0x5050,0x8F8F,0x5050,0x6060,0x9F9F,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5050,0x5050,0x5050,0x9F50
.word 0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x5050,0x5050,0x6060,0xAFAF,0xD3D0,0xE0C1,0xD0D0,0xA1B1,0xB2B2,0xB2A1,0x92B2,0xB2B2,0xB2B2,0xB2B2,0xB282,0x82B2,0xB2B2,0x81B2,0xB282,0xB281,0x81B2,0x81B2,0xB282,0xB281,0xB281,0x8181,0x81C3,0xC0C3,0xD0C3,0xD0C1,0xBFBF,0x7171,0xD0D0,0xB1C2,0xB2B2,0xB2B2,0xB281,0xB1B1,0xB1B1,0xB2A1,0xB1B1,0xB1B1,0x7080,0xAFAF,0xD0D0,0xCFCF,0xBFBF,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x5151,0x5151,0x4141,0x8F8F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x9F9F,0xF5F5,0xF5F5,0xEFEF,0xF0F0,0xCFCF,0xC0C0,0xBFBF,0xD0D0,0xD0D0,0xBFBF,0x8080,0xAFAF,0x6060,0x8F8F,0x4040,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x4040,0x5050,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x5050,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x4040,0x6F6F,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x5050,0x9F9F,0x5060,0x9F9F,0x6060,0x7060,0x7070,0x6060,0x9F9F,0x6060,0x9F9F,0x509F
.word 0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x5050,0x9F9F,0x6060,0x6060,0xC0C0,0xC3D0,0xD0D0,0xC1C1,0xB2B1,0xB2A1,0x92B2,0xB2B2,0x92B2,0x92B2,0x82B2,0xB2B2,0xB282,0xB282,0xB282,0x81B3,0x81B3,0x8281,0xC381,0x8181,0x81C3,0x81C3,0x81C3,0xC081,0xC0C3,0xC0C3,0xC2C3,0xC3C0,0xC0C0,0xC071,0xD0D0,0x7171,0x8181,0x71C3,0xB281,0xA1A1,0xA1B1,0xB2B2,0xB1B2,0xC0C1,0xB0B0,0xBFBF,0xE0E0,0xBFBF,0x9F9F,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x4040,0x4141,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xF4F4,0xF5F5,0xF2F2,0xF0F0,0xF0F0,0xCFCF,0xBFBF,0xCFCF,0xE0E0,0xE0E0,0xCFCF,0xD0D0,0xAFAF,0x5050,0x9F9F,0x6060,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xBFBF,0xAFAF,0x6060,0x9F9F,0x9F9F,0x8F8F,0x4040,0x7F7F,0x4040,0x4141,0x4141,0x4141,0x5050,0x5050,0x5050,0x5050,0x6060,0x9F9F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x5050,0x9F9F,0x4040,0x8F8F,0x4040,0x7F7F,0x5050,0x8F8F,0x8F8F,0x5050,0x7F7F,0x5050,0x6060,0x9F9F,0x6050,0x6060,0x9F9F,0x7070,0x9F9F,0x5050,0x5050,0x6060,0x9F9F
.word 0x5151,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x6060,0x6060,0x6060,0x7070,0xC0D4,0xCFCF,0xB1D0,0xA1B2,0xB2A1,0xB2A1,0x92B2,0xB2B2,0xB2B2,0xB282,0x82B2,0xB2B2,0xB282,0xB282,0xB281,0x8182,0x81C3,0x81C3,0xC381,0x8181,0x8181,0x81C3,0x71C3,0x71C3,0xC0C3,0xC0C3,0xC0C3,0xC381,0x71C3,0xD0C3,0xC0C0,0x7171,0xC0C0,0xC0C0,0xB1C2,0xB1B1,0xB2B1,0xB2A1,0xB1A1,0xC0B1,0xE0D0,0xCFCF,0xD0D0,0x7070,0x8F8F,0x4040,0x7F7F,0x8F8F,0x4141,0x4141,0x4141,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xF0F0,0xEFEF,0xFFFF,0xDFDF,0xDFDF,0xF0F0,0xF0F0,0xDFDF,0xDFDF,0xE0DE,0xC0C0,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x7070,0x5050,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x5050,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x5050,0x9F9F,0x6060,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x8F8F,0x6060,0x5050,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x5050,0x9F9F,0x5050,0x8F60
.word 0x5050,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x7F7F,0x5050,0x6060,0x9F9F,0x5050,0xC0C0,0xD0C0,0xCFCF,0x91C0,0xB2B2,0xA1B1,0xA1B2,0xB2B2,0x92B2,0xB2B2,0xB2B2,0xB282,0xB2B2,0xB2B2,0x82B3,0x82B3,0xC382,0x8281,0x8181,0x81C3,0xC3C3,0x81C3,0xC0C3,0xC0C3,0xD071,0xC0C3,0xC0C3,0xC0C3,0x81C3,0xC381,0x8181,0x7171,0xC0C0,0x7171,0xC0C0,0xD0D0,0xC2C0,0xB1C1,0xB1B1,0xB2B2,0xB1A1,0xC1C1,0xCFCF,0xB1D0,0xD3D3,0x6060,0x8F8F,0x4040,0x4040,0x8F8F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xF0F0,0xF0F0,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF0F0,0xF0F0,0xEFEF,0xEFEF,0xEFEF,0xF0F0,0xE0E0,0xC0C0,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x4040,0x7F7F,0x7F7F,0x5050,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x5050,0x9F9F,0x6060,0x9F9F,0x5050,0x8F8F,0x8F8F,0x4040,0x6060,0x8F8F,0x8F8F,0x4040,0x5050,0x5050,0x6060,0x5050,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x9F9F,0x6060,0x5050,0x8F9F
.word 0x5151,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x5050,0x9F9F,0x5050,0x6050,0xAFAF,0x7171,0xD0D0,0xB2B1,0xB2A1,0xB2B2,0xB2B2,0xB292,0xB2B2,0xB292,0xB292,0xB3B2,0xB392,0xB282,0xB2B2,0xC382,0x8281,0x8182,0x81C3,0x8181,0x81C0,0xC0C3,0xC372,0xC381,0x71C3,0xD4C0,0xD071,0xD471,0xC3C0,0x8181,0x81C3,0x81C3,0x7171,0x71C0,0xC0C3,0xAFAF,0xAFAF,0xD0D0,0xD0D2,0xC3D0,0xB181,0xA1A1,0xC0C0,0xB1B1,0xC0A1,0xC0C0,0x6060,0x8F8F,0x4040,0x4141,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAEAE,0xBFBF,0xAFAF,0xCFCF,0xF0F0,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF0F0,0xF0F0,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xCFCF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x6060,0x4040,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x4040,0x4040,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x7F7F,0x4040,0x7F7F,0x4040,0x5050,0x5050,0x9F9F,0x6060,0x9F9F,0x4040,0x4040,0x7F7F,0x5050,0x5050,0x5050,0x8F8F,0x6060,0x9F9F,0x5050,0x5050,0x6060,0x9F9F,0x7070,0x5050,0x6060,0x5050,0x5050,0x9F9F,0x9F50
.word 0x6060,0x8F8F,0x8F8F,0x4040,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x6060,0xC0C0,0xE0D0,0xC0C0,0xA1B2,0xB2B2,0xB2A1,0xB292,0xB2B2,0xB292,0x82B2,0xB3B3,0xB292,0xB2B3,0x82B3,0x82B3,0x82B2,0xB3B3,0xB372,0xC371,0xC371,0xC3C3,0xC3C0,0xC3C0,0xC3C0,0xD3C0,0xC071,0xC3C3,0xD0D0,0xC071,0x81D3,0x7281,0xC381,0xB3B2,0x81B2,0xC381,0xD0C0,0xC0C0,0xAFAF,0xE3E0,0xD0E0,0x7171,0xB1B1,0xB1A1,0xC0C0,0xB1C1,0xC0B1,0xAFAF,0x5050,0x4040,0x4040,0x4141,0x8F8F,0x4040,0x6F6F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x8F8F,0x9F9F,0xBFBF,0xCCCC,0xBFBF,0xBFBF,0xCFCF,0xDFDF,0xF2F2,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF0F0,0xF0F0,0xF1F1,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xE0E0,0xDFDF,0xBFBF,0x9F9F,0x8F8F,0x5050,0x8F8F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4141,0x8F8F,0x8F8F,0x5050,0x7070,0x6060,0x5050,0x8F8F,0x8F8F,0x4040,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x6060,0x5050,0x5050,0x5050,0x6060,0x6060,0x5050,0x5050,0x5050,0x9F9F,0x6060,0x509F
.word 0xAFAF,0xAFAF,0x5050,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x5050,0x5050,0x8F8F,0x7171,0xD0D0,0xC2C2,0xB2A1,0xA1B2,0xB2B2,0xB2B2,0x92B2,0xB2B2,0xB2B2,0x92B2,0xB3B2,0x8282,0xB282,0x82B3,0x82B3,0x81B3,0x81B3,0x81B3,0xC371,0xC071,0xC0C3,0x71D3,0x71D3,0x71C3,0xC3D0,0xC0C3,0xC0C3,0xD4D4,0x7171,0x81C3,0x8172,0xB282,0xB282,0x81B2,0xC381,0xD0D0,0xC0C0,0xBFBF,0xE0E0,0xE0E0,0xC0C0,0xB1B1,0xC0C3,0xB171,0xB1A1,0x7081,0x4040,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xDCDC,0xCFCF,0xBFBF,0xAFAF,0xCFCF,0xF0F0,0xEFEF,0xF1F1,0xF0F1,0xF1F1,0xF0F0,0xF0F0,0xF0F0,0xF1F1,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF0F0,0xDFDF,0xCFCF,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x8F8F,0x8F8F,0x4040,0x5050,0x8F8F,0x9F9F,0x5050,0x9F9F,0x6060,0x4040,0x8F8F,0x7070,0x7070,0xAFAF,0x7070,0x6060,0xAFAF,0x5050,0x9F9F,0x5050,0x6060,0x6060,0x9F9F,0x5050,0x6060,0x6060,0x9F9F,0x9F50
.word 0xAFAF,0xC0C0,0xAFAF,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5050,0x5050,0x4040,0x4040,0x5050,0x5050,0x9F9F,0x5050,0x7070,0xD0D0,0xC3C3,0xB0D0,0xB2B2,0xB2A1,0x92B2,0x92B2,0xB2B2,0x82B2,0x82B2,0xB3B2,0xB282,0xB3B3,0xB3B3,0xB382,0xB382,0x8282,0x72B3,0x7281,0x71C3,0xC3D0,0xD4C0,0xD0C0,0xD3D0,0xD3C0,0xC0D4,0xC0C0,0xC0C0,0xC0C0,0xD0D4,0xC371,0x7271,0x8172,0x82B3,0xC382,0xC381,0xC3C0,0xC0C0,0xC0C0,0xE0E0,0xDFDF,0xD0D0,0xC1D0,0xC0C1,0x7171,0xB191,0x81B1,0x6161,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0xCCCC,0xCFCF,0xDBDB,0xBFBF,0xBFBF,0xAFAF,0xDFDF,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xEFEF,0xF1F1,0xF1F1,0xF1F1,0xF0F0,0xF0F0,0xDFDF,0xCFCF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0x4141,0x4141,0x7F7F,0x4141,0x4040,0x4040,0x8F8F,0x8F8F,0x5050,0x6060,0x8F8F,0x6060,0x6060,0xAFAF,0x9F9F,0xAFAF,0xD0D0,0x8080,0xAFAF,0x9F9F,0x7070,0x7070,0x6060,0x6060,0x9F9F,0x6060,0x5060,0x5050,0x5050,0x6060,0x6060,0x9F9F
.word 0xD0D0,0xBFBF,0x7070,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x9F9F,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x5050,0x8F8F,0x5050,0x5050,0x9F9F,0x5050,0x9F9F,0x6060,0xD4C0,0xC0C0,0xC0C0,0xB292,0xA1B2,0xB2B2,0xB2B2,0xB292,0xB2B2,0xB2B2,0xB392,0x82B3,0xB2B2,0xB382,0x82B3,0x82B3,0x72B3,0xC381,0x8172,0x81B3,0x71C3,0xC0C0,0xD4D4,0xC3C3,0xD0C0,0xC3C3,0xD4C0,0xC0C0,0xC3C3,0xC0C0,0x71D0,0x81D3,0x7271,0xB381,0x8282,0x81B2,0x8182,0xC0C3,0xC0D4,0xD0D0,0xCFCF,0xDFDF,0xD0D0,0xC1D3,0x71C1,0xB271,0xB1B1,0x6181,0x7151,0x6161,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0xBFBF,0xEBEB,0xCFCF,0xCFCF,0xBFBF,0xCFCF,0xDFDF,0xD0D0,0xBFBF,0xE0E0,0xF0F0,0xF0F0,0xE0E0,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xCFCF,0xE0E0,0xDFDF,0xCFCF,0xCFCF,0xE0E0,0xBFBF,0xCFCF,0xAEAE,0xBFBF,0xBFBF,0xBFBF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x6060,0x8F8F,0x5050,0x5050,0x9F9F,0xAFAF,0xAFAF,0x8080,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0x6060,0x9F9F,0xAFAF,0x7070,0x9F9F,0x5050,0x6060,0x8F8F,0x6060,0xAFC0
.word 0xAFAF,0xC0C0,0x6060,0x8F8F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x5050,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x6060,0x9F9F,0x5050,0x5050,0x9F9F,0x5050,0x8F8F,0x7171,0xC0C0,0x8181,0xA1B2,0xB2B2,0x92B2,0x82B2,0x82B2,0x92B2,0xB2B3,0x92B3,0xB3B2,0xB382,0x82B3,0xB382,0x72B3,0xB382,0x72B3,0xC3B3,0x7271,0xC371,0xD4C0,0xD0D0,0xC3D0,0xD0D0,0xC1C1,0xC0C0,0xC0D4,0x71D0,0xC0C0,0xAFAF,0xD4C0,0xC3C0,0x7172,0x82B3,0x82B3,0x81C3,0x81B3,0xC371,0xC0C0,0xD0D0,0xE0E0,0xD0D0,0xD0E0,0xD0D2,0x7161,0xB1C2,0xC2B1,0xC271,0x60C0,0x8F8F,0x4141,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xCCCC,0xEBEB,0xCBCB,0xBFBF,0xBFBF,0xCFCF,0xE0E0,0xCFCF,0xE0E0,0xE0E0,0xCFCF,0xE0E0,0xD0D0,0xD0D0,0xF0F0,0xD0D0,0xAFAF,0xBFBF,0xE0F0,0xD0D0,0xF0F0,0xDFDF,0xEFEF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x5050,0x9F9F,0x4141,0x7F7F,0x4141,0x4141,0x5050,0x9F9F,0x6060,0x9F9F,0x6060,0x9F9F,0x5050,0x7070,0xBFBF,0xD0D0,0xBFBF,0xC0C0,0x8080,0xC0C0,0x8080,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x5050,0x5050,0x5050,0x6060,0x7070,0xBFD0
.word 0xAFAF,0xBFBF,0x7070,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xD0D0,0xC0C0,0xD0D0,0xAFAF,0x7070,0x7070,0x9F9F,0x6060,0x5050,0x5050,0x8F8F,0x5050,0x6060,0xAFAF,0xC0C0,0xC0C0,0xB2B1,0x92B2,0xB2B2,0xB2B2,0x82B2,0xB3B2,0xB392,0xB382,0x82B3,0xB3B3,0xB382,0x82B3,0x7272,0x72C3,0x7282,0x7282,0x71B3,0x71C3,0xC3C3,0xC0C0,0xD0D0,0xD3D0,0xD3D3,0xD4D0,0xD0C0,0xD0D0,0x7171,0xC0D4,0x7070,0x71C3,0x7171,0x7272,0xB372,0x8272,0xC371,0xC071,0xC371,0xC0C0,0xBFBF,0xE0E0,0xD0D0,0xD0E0,0xC0D3,0xC2C2,0xA1B1,0x81C2,0x71B2,0x6161,0x4040,0x4141,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0xAFAF,0xCFCF,0xEBEB,0xCFCF,0xBFBF,0xAFAF,0xCFCF,0xCFCF,0xE0E0,0xDFDF,0xE0E0,0xD0D0,0xC0C0,0xAFAF,0xBFBF,0xD0D0,0x7070,0x8080,0xF0F0,0xF0F0,0xD0D0,0xF0F0,0xEFEF,0xEFEF,0xEFEF,0xDFDF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4141,0x4040,0x8F8F,0x4141,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5050,0x6060,0x9F9F,0x7070,0xD0D0,0xBFBF,0x7070,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0x6060,0x7070,0x7070,0x6060,0x5050,0x5050,0x5050,0x6060,0x8080,0xAFAF
.word 0xC0C0,0x8080,0xAFAF,0x7070,0x9F9F,0x5050,0x8F8F,0x8F8F,0x9F9F,0x7070,0xAFAF,0xC0C0,0xAFAF,0xD4D4,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x5050,0x71C0,0xD0D0,0xB0B0,0xB282,0x8191,0x91B2,0x82B2,0x82B2,0x82B2,0xB282,0xB2B3,0xB382,0x8282,0x7272,0x7272,0xC372,0xB372,0xB372,0xB372,0xC371,0x7171,0xC0D0,0xD0D4,0xD0D4,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD3D3,0xD0D0,0xC0C0,0xD0D4,0xC3C3,0xD471,0x7271,0x72C3,0xB3B3,0x7172,0xD472,0xC0C0,0xC0C0,0xD0C0,0xD0D0,0xE0E0,0xE3E0,0xD0E0,0xC0C0,0xC271,0xB1A1,0xB1B2,0x7171,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0xBFBF,0xCFCF,0xCFCF,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xE0E0,0xF0F0,0xCFCF,0xAFAF,0x6060,0xD0D0,0xCFCF,0xD0D0,0xAFAF,0x7070,0xF0F0,0xE0E0,0xD0D0,0xE0E0,0xEFEF,0xFFFF,0xEFEF,0xFEFE,0xCFCF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x5050,0x6060,0xAFAF,0x9F9F,0x6060,0x7070,0xBFBF,0xD0D0,0xC0C0,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x7070,0x9F9F,0x6060,0x9F9F,0x6060,0x5050,0x6060,0x6060,0xAFAF,0xAF70
.word 0xAFAF,0xBFBF,0xD0D0,0xBFBF,0xD0D0,0xAFAF,0x6060,0x6060,0xAFAF,0xAFAF,0x7171,0xAFAF,0x6262,0xAFAF,0x6262,0xAFAF,0xAFAF,0x9F9F,0x6161,0x6060,0x6060,0x4040,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0x7171,0x8080,0x8181,0xB2B2,0xB281,0xB2B2,0xB281,0xB282,0x82B3,0xB382,0x82B3,0x72B3,0x72B3,0x7271,0x7271,0x7272,0x7272,0x71B3,0x7272,0x72C3,0x7171,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD3D0,0xC0C0,0xD0D0,0x7171,0xC0C0,0x6171,0x7261,0x7272,0xB372,0xB3B3,0x6171,0xD471,0xAFAF,0xD0D0,0xD0D0,0xD0D0,0xE0E0,0xE3E0,0xE0E0,0x8181,0xB1B1,0xB2A1,0x81B1,0x5151,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x8F8F,0x4040,0x4040,0x7F7F,0x8F8F,0xAFAF,0xDFDF,0xCFCF,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xBFBF,0xF0F0,0xE0E0,0xC0C0,0x6060,0x6060,0xBFBF,0xF0F0,0xDFDF,0xD0D0,0xAFAF,0xD0D0,0xBFBF,0xAFAF,0xD0D0,0xDFDF,0xEFEF,0xFFFF,0xFFFF,0xEFEF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0xCCCC,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x5050,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xC0C0,0xC0C0,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x709F
.word 0xC0C0,0xD0D0,0xE0E0,0xE0E0,0xBFBF,0xD0D0,0x7070,0xAFAF,0x7070,0xAFAF,0xAFAF,0x6161,0xAFAF,0x6161,0xAFAF,0x6161,0x6161,0x6161,0x9F9F,0xAFAF,0x9F9F,0x4040,0x7F7F,0x4040,0x4040,0x7171,0xD0D0,0xCFCF,0x81C3,0x8181,0x81B2,0x81B2,0x82B2,0x8281,0xB382,0x82B3,0xB382,0x7172,0x7171,0x7171,0x6161,0x7271,0x7271,0x7272,0x72B3,0x7271,0x71C3,0xD0C3,0xD0E3,0xD3D3,0xD0D0,0xE4E4,0xD3D3,0xD0D0,0xD4D0,0xD0D0,0xD4D4,0xD4C0,0xD0D0,0x71D4,0x7272,0x72B3,0x7272,0xC371,0x7061,0xC0D3,0xD4D4,0xC0D0,0xD0D0,0xD0D0,0xE0E0,0xE0F0,0xBFBF,0xB1C0,0xB1B1,0xC2B2,0x5181,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0xAFAF,0xBFBF,0xBFBF,0x9F9F,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xCFCF,0xE0E0,0xBFBF,0x9F9F,0x8F8F,0xBFBF,0xF0F0,0xD0D0,0xBFBF,0x7070,0xBFBF,0xC0C0,0x7070,0xAFAF,0xCFCF,0xEFEF,0xFFFF,0xFFFF,0xFEFE,0xCDCD,0xCCCC,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x5050,0x7070,0xAFAF,0x7070,0xD0D0,0xC0C0,0xC0C0,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x7070,0x6060,0x9F9F,0x7070,0x6060,0x6060,0x5050,0x6060,0x6060,0x6060,0x9F9F
.word 0xAFAF,0xBFBF,0xD0D0,0xD0D0,0xD0D0,0xCFCF,0xBFBF,0x7070,0x9F9F,0x9F9F,0x6161,0xAFAF,0x6161,0xAFAF,0x6161,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x4040,0x7F7F,0x6060,0xAFAF,0x7171,0xC0C0,0x8181,0xB2B2,0xB281,0x8182,0x81B2,0xB3B2,0x82B2,0x82B2,0x71B3,0xC0C0,0xAFAF,0x9F9F,0x6161,0x6161,0x7262,0x7271,0xB371,0xC372,0xC371,0xD0D0,0xD0D0,0xD0E0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xC0C0,0xC0C0,0x7171,0x7161,0x7272,0x7272,0xC372,0x7172,0x7272,0xC0C0,0xC0D0,0xD4D4,0xC0C0,0xD0D0,0xE0E0,0xE0E0,0xD0D0,0xD3D3,0xA1B1,0xB2A1,0xC0B2,0x5151,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x5050,0x9F9F,0x5050,0x8F8F,0xAFAF,0xCFCF,0xCFCF,0xAFAF,0xCFCF,0xCFCF,0x8080,0x6060,0x9F9F,0xF0F0,0xE0E0,0xD0D0,0x7070,0xAFAF,0xD0D0,0x8080,0xAFAF,0x7070,0xD0D0,0xF0F0,0xF0F0,0xFFFF,0xFFFF,0xFFFF,0xDFDF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x6060,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x7F7F,0x5050,0xAFAF,0x7070,0x8080,0xBFBF,0xBFBF,0xC0C0,0x7070,0x7070,0x7070,0xAFAF,0x6060,0x9F9F,0x5050,0x7070,0x6060,0x6060,0x5050,0x7060,0x7070,0x6060,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x509F
.word 0xC0C0,0xD0D0,0xBFBF,0xD0D0,0xBFBF,0xD0D0,0xC0C0,0xAFAF,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6161,0x9F9F,0x4040,0x7F7F,0x7F7F,0x4040,0x7171,0xBFBF,0x8080,0xB281,0xB181,0x8181,0xC381,0x8181,0x8281,0xB3B2,0x8282,0x7171,0xAFAF,0x5050,0x4040,0x8F8F,0x6060,0x6161,0x7272,0x7272,0x8172,0x7172,0xD0C3,0xD0E4,0xBFBF,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD4D0,0xC0C0,0xC0C0,0xC3C3,0x6271,0x7271,0x7262,0x7262,0x7262,0x7172,0xC3C3,0xBFBF,0xC0C0,0xC0C0,0xC0C0,0xD0D0,0xE0E0,0xE0E0,0xBFBF,0xB2C0,0xB2B1,0xB2B2,0x7171,0x4040,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x8F8F,0x5050,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x6161,0xBFBF,0xBFBF,0x7070,0xAFAF,0xAFAF,0xE0E0,0xBFBF,0x6060,0x6060,0xC0C0,0xE0E0,0xE0E0,0x6060,0x8F8F,0xAFAF,0xD0D0,0xF3F3,0xF2F0,0xFFFF,0xFFFF,0xEFEF,0xCFCF,0xDEDE,0xCFCF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xC0C0,0x7070,0x9F9F,0x9F9F,0x8F8F,0x5050,0xAFAF,0xBFBF,0x9F9F,0x9F9F,0x7080,0x7070,0xC0C0,0xC0C0,0x8080,0xAFAF,0x7070,0x6060,0x7070,0x7070,0x9F9F,0x6060,0x5050,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x5050,0x5050,0x6060,0x9F9F
.word 0x7171,0x8080,0xC0C0,0xC0C0,0xC0C0,0xBFBF,0xBFBF,0x7070,0xAFAF,0x5151,0x9F9F,0x9F9F,0x9F9F,0x6161,0x9F9F,0x6161,0x5151,0x9F9F,0x9F9F,0xAFAF,0x5151,0x9F9F,0x4040,0x6F6F,0x5050,0xAFAF,0x7171,0xC0C0,0xC271,0x8181,0x81C3,0x81C3,0x81C3,0x81C3,0xB282,0xC382,0xD0D0,0x6060,0x6F6F,0x7F7F,0x4040,0x8F8F,0xAFAF,0x7161,0x7272,0xC372,0x8182,0xD471,0xD0D0,0xD0D0,0xD0D0,0xE0E0,0xD0D0,0xD0D0,0xD0D3,0xBFBF,0xD0D0,0xC0D4,0x8181,0x7171,0x7162,0x7262,0x6262,0x6272,0x6272,0x6161,0xC0C1,0xC0C0,0xD0D0,0xC0C0,0xD0D0,0xE0E0,0xE0E0,0xD0D0,0xC3C0,0xB1B1,0xB2B2,0x81B2,0x7171,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x7070,0xD0D0,0xD0D0,0x7070,0x9F9F,0x7070,0xCFCF,0xE0E0,0xF0F0,0xD0D0,0x5150,0x6151,0xAFAF,0xD0D0,0xE0E0,0xF3E3,0xF2F2,0xEFEF,0xCFCF,0xDFDF,0xBFBF,0x9F9F,0x6262,0x6161,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0x6060,0x9F9F,0xAFAF,0xF5F5,0xEFEF,0xDFDF,0xCFCF,0xAFAF,0x8080,0xD0D0,0xCFCF,0x7070,0x8080,0xAFAF,0x6060,0x7070,0x7070,0x6060,0x6060,0x5050,0x6060,0x6050,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x5050,0x8F8F,0x6060,0x9F50
.word 0x7171,0x7171,0xAFAF,0xAFAF,0x8080,0x8080,0xC0C0,0xBFBF,0x5151,0x8F8F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x6161,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x4040,0x4040,0x7F7F,0x8080,0xC0C0,0xBFBF,0x8181,0x81C3,0xC081,0x8181,0x81C0,0xB381,0x8282,0xC072,0xBFBF,0x7F7F,0x4040,0x6F6F,0x6F6F,0x4040,0x9F9F,0xAFAF,0x7271,0x7272,0xC372,0xC072,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD3D3,0xC0C0,0xBFBF,0xC0C3,0xC0C0,0xD0D0,0xD3D3,0x7171,0x6262,0x6262,0x6271,0x6262,0x6272,0x7171,0xD0D0,0xD0D4,0xD3D3,0xD0D0,0xD0D0,0xD0D0,0xE0E0,0xD0D0,0xD0D0,0xB2C0,0xB2B2,0xB2B2,0x71B1,0x4141,0x8F8F,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x6060,0x9F9F,0xAFAF,0xAFAF,0x6060,0xAFAF,0xE0E0,0xCFCF,0xC0C0,0x6060,0xAFAF,0xD0D0,0xF0F0,0xE0F0,0xCFCF,0x9F9F,0x9F9F,0x7171,0xAFAF,0x6161,0xAFAF,0xF0F0,0xF3F0,0xF0E0,0xDFDF,0xAFAF,0x6262,0xAFAF,0x9F9F,0x9F9F,0x6161,0x7070,0x9F9F,0x6060,0xAFAF,0x7070,0xF5F5,0xF5F5,0xF5F5,0xEFEF,0xD0D0,0x7070,0xC0C0,0xBFBF,0xD0D0,0xC0C0,0xAFAF,0x6070,0x6060,0x7070,0x5050,0x6050,0x9F9F,0x5050,0x6060,0x5050,0x6050,0x6060,0x6060,0x6050,0x6060,0x6060,0x5060,0x4040,0x5050,0x6060,0x509F
.word 0xC0C0,0x6161,0x7171,0x6161,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6161,0x8F8F,0x5151,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x6161,0x5151,0x8F8F,0x7F7F,0x7070,0xBFBF,0xC0C0,0xC0C0,0xB181,0x8181,0x82C2,0xC3C2,0xC381,0xB281,0xC381,0xD081,0x6060,0x4040,0x7F7F,0x6F6F,0x4040,0x7F7F,0x4040,0x6161,0x7162,0x8272,0x72C3,0xC381,0xE3E3,0xD0D0,0xD0D0,0xE4D0,0xD0D0,0xD0D0,0xD0D4,0xD0D0,0xD3D3,0xD3D0,0xC0C0,0x61D4,0x6271,0x6261,0x6262,0x7262,0x7162,0x6262,0xC3D0,0xC0C0,0xC0C0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xC3D0,0xB281,0xB2B2,0xC2A1,0x4071,0x4040,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x9F9F,0x7070,0x6060,0x6060,0x9F9F,0x8080,0xD0D0,0xC0C0,0xAFAF,0xC0C0,0xD0D0,0xE0E0,0xF3F3,0xE3E0,0x7171,0xAFAF,0x6161,0x6161,0x5151,0x9F9F,0xE0C0,0xF1F0,0xE0F2,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x7070,0xAFAF,0xF5E5,0xDFDF,0xF7F7,0xFFFF,0xDFDF,0xAFAF,0x8080,0xD0D0,0xBFBF,0xAFAF,0x7070,0x9F9F,0x6060,0x6060,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x6060,0x5050,0x6060,0x6060,0x6060,0x6060,0x6060,0x5050,0x4040,0x8F8F,0x4040,0x8F50
.word 0xD0D0,0x7070,0x9F9F,0xAFAF,0x6161,0x6161,0x6161,0x6161,0x9F9F,0x8F8F,0x4141,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x7070,0x9F9F,0x5050,0xD0D0,0xD3D3,0xC0C0,0xB281,0xB1B2,0x8181,0xC081,0x81C3,0xB381,0x82B2,0x8181,0xBFBF,0x7F7F,0x6F6F,0x4040,0x6F6F,0x4040,0x7F7F,0xAFAF,0x7171,0x7272,0xB382,0x71B3,0xD0D3,0xD0D0,0xD0E4,0xD0D0,0xD0D0,0xD0D4,0xD0D0,0xD0D4,0xD0D0,0xD0D0,0xD4C0,0x7171,0x6161,0x6262,0x6261,0x6162,0x6262,0x6262,0x7171,0xD4C0,0xC0C0,0xD3D3,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xC3D0,0x8171,0xB282,0xA1B2,0x61B1,0x8F8F,0x4040,0x8F8F,0x4040,0x5050,0x8F8F,0x5050,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6060,0x6060,0x9F9F,0x6060,0x9F9F,0x6060,0xAFAF,0x8080,0xAFAF,0x8080,0xD0D0,0xDDE0,0xCFCF,0xE0F0,0xBFBF,0x7171,0x6161,0x9F9F,0xAFAF,0x6161,0x7070,0xCFCF,0xF1F1,0xF0F0,0x8080,0xAFAF,0xAFAF,0xAFAF,0x5252,0x9F9F,0x6161,0xBFBF,0xD0D0,0xE0E0,0xE0E0,0xBFBF,0xBFBF,0xF5F5,0xF5F5,0xF7F7,0xF5F5,0xCFCF,0x7070,0xAFAF,0xC0C0,0x8080,0xC0C0,0xAFAF,0x7070,0x6060,0x7070,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x7070,0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x609F
.word 0xCFCF,0xD0D0,0x6161,0x5151,0x6161,0x9F9F,0x5151,0x9F9F,0x5151,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x6161,0xAFAF,0xAFAF,0x6161,0xC0C0,0xD0D0,0xD0D0,0xD0D0,0xAFAF,0xAFAF,0xD0D0,0xBFBF,0xC0C0,0x91B2,0xB281,0xC2C2,0xC381,0xC3C0,0xB281,0xB282,0xD081,0x5050,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4040,0x9F9F,0x6161,0x72C3,0x81B3,0xC3B3,0xC0C0,0xD0D3,0xD3D3,0xE4D0,0xD0D0,0xD0D0,0xD0D3,0xD4D4,0xD3D3,0xD0D0,0xC0C0,0xD0D0,0x7171,0x6261,0x6162,0x61C4,0x6262,0x6262,0xD471,0x7171,0xC0C0,0xD0D0,0xD3D3,0xE0E0,0xD0D0,0xD0E4,0xD0D0,0xD0D0,0x71D3,0x8272,0xB2B2,0x81B2,0x5151,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x9F9F,0x9F9F,0x9F9F,0x5050,0x6060,0x6060,0x7070,0xAFAF,0x8080,0xD0D0,0xCFCF,0xDFDF,0xEFEF,0xDFDF,0xC0C0,0x6161,0x8F8F,0x5151,0x6060,0x5050,0x9F9F,0xF0E0,0xE0F0,0x7070,0x9F9F,0x6161,0xD4D4,0xBFBF,0xAFAF,0xAFAF,0xE3E3,0xCFCF,0xF0F0,0xDFDF,0xF0F0,0xD0D0,0xAFAF,0xBFBF,0xE5E5,0xF6F5,0xCFCF,0xAFAF,0x9F9F,0x7070,0xC0C0,0xAFAF,0x7070,0x8080,0x6060,0x6060,0x6060,0x5050,0x5050,0x5050,0x4040,0x5050,0x5050,0x6060,0x5050,0x5050,0x6060,0x7070,0x5050,0x6060,0x6060,0x6060,0x9F9F,0x9F9F
.word 0xD0D0,0xD0D0,0xAFAF,0x5151,0x9F9F,0x6161,0x9F9F,0x5151,0x9F9F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xD0D0,0xD0D0,0xC0C0,0x7070,0xC3C3,0xD0D0,0xC0C0,0xB2B1,0x81B1,0xC381,0xC3C0,0x81C0,0xB281,0x82B2,0xC0C3,0xBFBF,0x4040,0x6F6F,0x6F6F,0x4040,0x4040,0x6F6F,0x5050,0x6161,0x7272,0xB382,0x7182,0xD0C3,0xD3D3,0xD0D0,0xD0D0,0xD0E4,0xD0D3,0xD3D3,0xD0D0,0xC0C0,0xC0D4,0xD3D3,0xC0D4,0x62D4,0x6161,0x6262,0x6261,0x6162,0x6162,0xAFAF,0xC0C0,0xC3C3,0xC0C0,0xC0C0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0x7261,0xB382,0xC2B2,0x4071,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x9F9F,0x7070,0x8080,0xBFBF,0xEFEF,0xEFEF,0xEFEF,0xE4E4,0x6161,0x5151,0x4141,0x8F8F,0x5050,0x8F8F,0x5151,0xE0E0,0xD0D0,0x6060,0x5151,0xAFAF,0xCFCF,0xD0D0,0xBFBF,0xE3E3,0xDFDF,0xF2F2,0xF0F0,0xE0E0,0xF0F0,0xAFAF,0x6161,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x8F8F,0x6060,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0x6060,0x5050,0x6060,0x9F9F,0x5050,0x5050,0x6060,0x8F8F,0x5050,0x5050,0x6060,0x5050,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x7070,0x5050
.word 0xBFBF,0xD0D0,0x5151,0x5151,0x6161,0x9F9F,0x6161,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xC0C0,0xD0D0,0xD0D0,0xBFBF,0xCFCF,0xD0D0,0xBFBF,0xD4D4,0xD0D0,0xD0D0,0xC0C0,0xB1B2,0xB1B1,0xC0C2,0xD0C3,0xC0C3,0xB2C3,0xB282,0xC082,0xCFCF,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x6F6F,0x8F8F,0x6161,0xB372,0x82B3,0x72B3,0xD0D4,0xD0D0,0xD0D4,0xD0D3,0xD0D0,0xD0D0,0xD0D3,0xD3D3,0xD0D4,0xD3D0,0xC0C0,0xC0C0,0x7171,0x6261,0x6162,0x6262,0x6271,0x6162,0x7161,0xC0D4,0xC3C3,0xC0C0,0xD3D3,0xD0D0,0xD0D0,0xD3D3,0xD4D0,0xD0D0,0xD3E0,0x6161,0x8272,0x82B3,0x51C2,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x4040,0x8F8F,0x5050,0x6060,0x8080,0xBFBF,0xBFBF,0xDFDF,0xFFFF,0xFFFF,0xDFDF,0xBFBF,0x5151,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x7070,0xBFBF,0x5050,0x8F8F,0x9F9F,0xF1E0,0xFFFF,0xF0F0,0xF0F0,0xF0F0,0xF2F2,0xF0F0,0xE0E0,0xE0E0,0xD0D0,0x7070,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x6060,0x9F9F,0x7070,0x6060,0x7070,0x7070,0x9F9F,0x6060,0x7070,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x6060,0x6060,0x6060,0x5050,0x8F9F
.word 0x5151,0x9F9F,0x6161,0x9F9F,0x8F8F,0x6161,0xAFAF,0x6161,0x9F9F,0x9F9F,0x5151,0x4141,0x8F8F,0x8F8F,0x5151,0x9F9F,0xC0C0,0xAFAF,0xAFAF,0xAFAF,0xD4D4,0xD0D0,0xBFBF,0xD0D0,0x7171,0xC0C0,0xBFBF,0xB0B0,0xA1B2,0xB1B2,0xC1C2,0xD0C1,0xC3D0,0xB281,0xB2B2,0xC382,0xD0D0,0x4040,0x6F6F,0x6F6F,0x4040,0x6F6F,0x4040,0x8F8F,0x6170,0x72C3,0x72B3,0x81B3,0xC0C0,0xC0D3,0xD0D0,0xD3D3,0xD3D3,0xD3D0,0xD0D0,0xD3D0,0xD0D0,0xD4D4,0xD4C0,0xD4D0,0xD4D0,0x6161,0xC462,0x6261,0x6262,0x6161,0x6262,0xD0D0,0xC0C3,0xD4D4,0xC0C0,0xD0D0,0xD0D4,0xD0D0,0xD0D0,0xC0C0,0xD0D0,0x7161,0x82C4,0xB2B3,0x71B2,0x8F8F,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x5050,0x5050,0x8F8F,0x6060,0x9F9F,0xBFBF,0xCFCF,0xDFDF,0xE0E0,0xE0E0,0xFFFF,0xFFFF,0xDFDF,0x5151,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x6060,0x4040,0x5050,0xE0E0,0xEFEF,0xFFFF,0xF2F2,0xF5F5,0xFFFF,0xFFFF,0xF0F0,0xEFEF,0xF0F0,0xCFCF,0xAFAF,0x7070,0x6161,0x5151,0x4141,0x4242,0x8F8F,0x6060,0xAFAF,0x7070,0x7070,0x6060,0xAFAF,0x8080,0xC0C0,0xAFAF,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5050,0x7070,0x9F9F,0x6060,0x5050,0x6060,0x5050
.word 0xAFAF,0x9F9F,0x9F9F,0x6060,0x6161,0x9F9F,0x5151,0x9F9F,0x9F9F,0x6161,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x6161,0xAFAF,0xC0C0,0xD4D4,0xAFAF,0xAFAF,0xD4D4,0xD4D4,0xD3D3,0x7171,0xD4D4,0xC0C0,0xC0C0,0xB2B2,0xA1B2,0xB1B2,0xD0D3,0xD0D3,0xB2C0,0xB282,0x8182,0xC0C0,0xAFAF,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x5050,0x6262,0x7272,0x82B3,0x7272,0xD0C3,0xC3C3,0xD3C3,0xD0D0,0xD0D3,0xD3D3,0xD3E3,0xD0D0,0xC0D4,0xD0D0,0xC0C0,0xC0C0,0xC0D4,0x6171,0x6261,0xC462,0x6271,0x6262,0x6161,0xC3C3,0xC3C3,0x7171,0xD4C0,0xD0D0,0xD0D0,0xD4D0,0xD0D0,0xC3C3,0xD3D3,0x61D4,0x7271,0x82B3,0xC3B3,0x6060,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xE0E0,0xDFDF,0xEFEF,0xFFFF,0xEFEF,0xEFEF,0xFFFF,0xFFFF,0xF0F0,0x4141,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x5050,0x6060,0x4040,0x8F8F,0xF0F0,0xFFFF,0xF1F1,0xF5F5,0xF3F3,0xF4F4,0xCFCF,0xF0F0,0xFFFF,0xF0F0,0xE0E0,0x8080,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x4141,0x5151,0x9F9F,0xAFAF,0x7070,0x9F9F,0x7070,0x6060,0x7070,0x6060,0x6060,0x5050,0x5050,0x4040,0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x7070,0x6060,0x6060,0x5050,0x6060,0x8F9F
.word 0xD0D0,0xC0C0,0xBFBF,0xBFBF,0xAFAF,0x6161,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x5151,0x5151,0x9F9F,0xC0C0,0xAFAF,0xAFAF,0xC0C0,0x7171,0xAFAF,0xAFAF,0xC0C0,0xD4D4,0xC0C0,0xD0D0,0x8181,0xA1A1,0xB2A1,0xC1B1,0xD0C2,0xE0E0,0xC1D3,0xB281,0x82B2,0xC0C3,0x5050,0x4040,0x6F6F,0x7F7F,0x4040,0x4040,0x6161,0x7161,0xB372,0xC372,0xC372,0xD371,0xD0D3,0xC3D0,0xD3D3,0xD3D3,0xD0D3,0xD0D0,0xD0D3,0xD3D3,0xD4D4,0xD4D0,0xD4D0,0xD0D0,0xD4D4,0x7162,0x6262,0x7262,0x6262,0x6262,0x7171,0xC0C0,0xC0C0,0xAFAF,0xC3C3,0x7171,0xC0C0,0xD0D4,0xD0D0,0xD0D0,0x6161,0x7272,0xB372,0x7172,0xBFBF,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x4040,0x5050,0x7070,0xAFAF,0xD0D0,0xDFDF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF0F0,0xCFCF,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x7F7F,0x5050,0x9F9F,0x8F8F,0xE0E0,0xEFEF,0xF2F2,0xF5F5,0xF4F4,0xBFBF,0xBFBF,0xC0C0,0xCFCF,0xF0F0,0xDFDF,0xBFBF,0x8080,0x7070,0x6060,0x5050,0x4040,0x8F8F,0x8F8F,0x6060,0x7060,0xAFAF,0x7070,0x7070,0x6060,0x5050,0x6060,0x6060,0x5050,0x4040,0x4040,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x6060,0x6060,0x9F9F,0x5060,0x5050,0x5050,0x509F
.word 0xD0D0,0xBFBF,0xC0C0,0xC0C0,0xC0C0,0xAFAF,0x5151,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0xC0C0,0xAFAF,0xD0D0,0xD0D0,0xD4D4,0x7171,0xD4D4,0xAFAF,0x7171,0xD0D0,0xD0D0,0xC0C0,0xA1B1,0xB2B2,0xC2A1,0xE3D0,0xD0E0,0xD0D0,0x82C2,0xB3B2,0x7181,0xAFAF,0x8F8F,0x6F6F,0x4040,0x6F6F,0x8F8F,0x6060,0x7261,0x7272,0x72B3,0x7172,0xD3D3,0xC3C0,0xC3C3,0xD3C0,0xD0D0,0xE4D0,0xD3C0,0xD0D0,0xD0D3,0xD0D0,0xD0E4,0xD0D0,0xD3D0,0x6171,0x6262,0x6262,0x6262,0x6162,0x6162,0xD3D3,0xC3C3,0xC3C3,0xC0D4,0xC3D0,0xD0D0,0xC3C3,0xD0D0,0xC3D0,0xE4E4,0x72D0,0xB372,0x72B3,0x7272,0xCFCF,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x9F9F,0xBFBF,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF0F0,0xFFFF,0xE0F2,0xE0E0,0xAFAF,0x6161,0x4141,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x8080,0xE0E0,0xF0F0,0xFFFF,0xF4F4,0xCFCF,0xBFBF,0xBFBF,0x6161,0x7070,0xD0D0,0xBFBF,0xD0D0,0xC0C0,0x8080,0xAFAF,0x9F9F,0x5050,0x9F9F,0x5050,0x6060,0x6060,0x9F9F,0xAFAF,0x7070,0xAFAF,0x6060,0x5050,0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x5050,0x5050,0x5050,0x6060,0x6060,0x6060,0x5050,0x6060,0x6060,0x9F60
.word 0xD0D0,0xC0C0,0xC0C0,0x6161,0xAFAF,0xC0C0,0x9F9F,0x4141,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x6161,0xAFAF,0x7171,0xAFAF,0xBFBF,0xAFAF,0x6262,0x6161,0x6161,0xAFAF,0x7171,0xD0D0,0xB0B0,0xB2B2,0xB1A1,0xB1B2,0xE0D3,0xE0E0,0xD0E0,0xB171,0xB282,0xC3B3,0xC0C0,0x5050,0x4040,0x7F7F,0x4040,0x5050,0x6151,0x7272,0x72C3,0x72B3,0x71C3,0xC0D4,0xC0D4,0xC0D4,0xC0D4,0xC3C3,0xC0C0,0xD0E4,0xC1C1,0xE4D0,0xD3D3,0xD0D0,0xD3D3,0xD0E4,0x7171,0x7162,0x71C4,0x62C4,0x6262,0x6161,0x7171,0x7181,0xC0C0,0xC0C0,0xD0D0,0xC3C3,0xC0C0,0x71D4,0xD0D0,0xD0D3,0x6262,0xC362,0x7272,0x6162,0xE0E0,0xBFBF,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x5050,0xE0E0,0xCFCF,0xEFEF,0xFFFF,0xF0F0,0xBFBF,0xCFCF,0xF3F3,0xBFBF,0x9F9F,0x5050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7070,0xCFCF,0xF0F0,0xFFFF,0xF2F2,0xCFCF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x8080,0x8080,0xAFAF,0x7070,0x9F9F,0xAFAF,0x6060,0x7070,0x7070,0x7070,0x7070,0x6060,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x609F
.word 0x7171,0x7171,0x7171,0x6161,0xAFAF,0x7070,0x4141,0x8F8F,0x4141,0x8F8F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x6161,0xAFAF,0xAFAF,0x6262,0xD4D4,0xC3C3,0xAFAF,0xAFAF,0xAFAF,0x6161,0xD0D0,0xBFBF,0xC0C0,0xB1A1,0xB2A1,0xC1B1,0xE0D0,0xF0F0,0xCFCF,0xC3D0,0xB381,0x82B2,0x7172,0xAFAF,0x9F9F,0x8F8F,0x5050,0x6151,0x6262,0x6272,0x7272,0x72B3,0x7172,0xD3D3,0xD4C0,0xD4C0,0xD4D0,0xD3D0,0xD0C3,0xD3D3,0xE4D0,0xD0D0,0xD0D3,0xD0E4,0xC0C0,0xD3D3,0x7171,0x6272,0x7272,0x7271,0x61C4,0x6262,0xD361,0xC3C3,0x7171,0xC3D4,0xD371,0xD0D0,0xD4D0,0xC0C0,0xD4D4,0xD4D0,0x62C0,0x7262,0x7283,0x6262,0xE0E0,0xCFCF,0x5050,0x7F7F,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0xBFBF,0xF0F0,0xF0F0,0xF0F0,0xCFCF,0xAFAF,0xBFBF,0xBFBF,0x9F9F,0x5050,0x4141,0x4141,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x5050,0x9F9F,0xF0F0,0xFFFF,0xFFFF,0xF0F0,0xBFBF,0x6161,0xAFAF,0x8080,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x7070,0xC0C0,0xBFBF,0x7070,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x7070,0x6060,0x5050,0x5050,0x6060,0x5050,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x5050,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F60
.word 0xD0D0,0x7171,0x7171,0xAFAF,0x7070,0xAFAF,0x9F9F,0x8F8F,0x4141,0x5050,0x8F8F,0x5151,0x9F9F,0x5151,0x6161,0x9F9F,0x6161,0x6161,0xAFAF,0xAFAF,0xAFAF,0x6262,0x6262,0x5252,0x6161,0xD0D0,0xDED0,0xC1C1,0xA1B1,0xB1B2,0xC2B1,0xE0E0,0xF0F0,0xE0F0,0xD0D0,0xB281,0xB382,0x7281,0x7071,0x5161,0x5160,0x6161,0x6161,0x7162,0x7262,0x72C3,0x7272,0x71C3,0xC0C3,0xC0D4,0xC0D4,0xC0C0,0xD0E4,0xD3D0,0xD0D0,0xD0E4,0xE4D0,0xD3D0,0xD0D0,0xD3D3,0xE4D0,0x6261,0x6262,0xC462,0x6262,0x6271,0x6162,0xD4D4,0x7171,0x71D3,0xD0D0,0x7171,0xC3C3,0xC0C0,0xC3C3,0x71D0,0xD0D0,0xD4D4,0x6261,0x72C4,0x5162,0xD0D0,0xD0D0,0xAFAF,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x5050,0xBFBF,0xCFCF,0xDFDF,0xCFCF,0xC0C0,0x7070,0xBFBF,0xAFAF,0x6060,0x9F9F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x5050,0x9F9F,0xF0F0,0xFFFF,0xFFFF,0xFFFF,0xF0F0,0x7070,0x9F9F,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x6060,0x9F9F,0x7070,0x6060,0x7070,0x9F9F,0x7070,0x9F9F,0xAFAF,0xBFBF,0x8080,0x8080,0x7070,0x6060,0x9F9F,0x5050,0x6060,0x6060,0x8080,0xAFAF,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x5050,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x609F
.word 0xBFBF,0xC3C3,0x7171,0xC0C0,0xC0C0,0x6161,0x5151,0x5151,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x5151,0x8F8F,0x9F9F,0x6161,0x9F9F,0x9F9F,0x6161,0x6161,0x6161,0xAFAF,0xAFAF,0x9F9F,0x5151,0xD0D0,0xD0D0,0xB1B1,0xA1A1,0xB2B1,0xD3B1,0xE0E0,0xF0F0,0xDFDF,0xF0F0,0x81D0,0x82B2,0xC3B3,0x7272,0x7162,0x6261,0x6261,0x6262,0x6262,0x7272,0x7272,0xC372,0x7172,0xD3D3,0xC371,0xD0C0,0xD0C3,0xD0D4,0xD3D3,0xD0D3,0xD0D4,0xE3E4,0xD0D0,0xD4D0,0xD0D0,0xC3C3,0x6161,0x7162,0x6262,0x6261,0x6262,0x6161,0x7171,0xC3D0,0x7171,0xC0D3,0xD0D4,0xD0D4,0xD471,0xC0C0,0xC3C3,0xC3D3,0x6161,0x6262,0x6272,0x6262,0x6161,0x7171,0xD0D0,0x8F8F,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x6F6F,0x7F7F,0x4040,0xBFBF,0xCFCF,0xBFBF,0xC0C0,0xBFBF,0x9F9F,0xBFBF,0x7070,0x9F9F,0x6060,0x4040,0x4040,0x8F8F,0x8F8F,0x4040,0x4141,0x6060,0xAFAF,0xF0F0,0xFFFF,0xFFFF,0xE0E0,0xBFBF,0x9F9F,0x6060,0xAFAF,0x7070,0x7070,0x6060,0x6060,0x7070,0x6060,0x6060,0x6060,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0x9F9F,0x7070,0x6060,0x9F9F,0xAFAF,0x6060,0x6060,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x5050,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x5060
.word 0xD0E0,0x61C0,0xAFAF,0xAFAF,0xBFBF,0xC0C0,0xAFAF,0xAFAF,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x6161,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x6161,0x5252,0x5151,0x6161,0xD3D3,0xCED0,0xB0B0,0xB2B1,0xA1B2,0xD0B2,0xE0E0,0xDFDF,0xDFDF,0xF0F0,0xC3C3,0xB381,0x7282,0x72B3,0x6271,0x7162,0x6162,0x6162,0xC472,0x7272,0x72C4,0x7272,0x7171,0xC3D0,0xD0D3,0xD3D4,0xD4D0,0xD4D0,0xD0D0,0xE4D0,0xD0D0,0xD0D3,0xD3D3,0xC0D3,0xD4D4,0x71D0,0x6261,0x6262,0x6272,0x6262,0x6162,0x5252,0xD0D0,0xD4D4,0x71C3,0xC3C3,0x7171,0xC0C0,0xD4D4,0x8181,0xC0D4,0xC0D4,0x6261,0x6262,0x7272,0x5162,0xD0D0,0xD0D0,0xD0D0,0x5050,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0xAFAF,0x7070,0x7070,0xAFAF,0x6060,0x6060,0x8080,0x5050,0x5050,0x9F9F,0x9F9F,0x5050,0x5050,0x4040,0x8F8F,0x5050,0x9F9F,0xF3D0,0xEFEF,0xFFFF,0xF0F0,0xCFCF,0x7070,0x5050,0x8F8F,0x7070,0x7070,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x7070,0x6060,0x6060,0x6060,0x9F9F,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0x8080,0x7070,0xAFAF,0xAFAF,0x7070,0x5050,0x8F8F,0x5050,0x9F9F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x6060,0x7070,0x7070,0x9F9F
.word 0xE4E4,0xC3C3,0x6161,0xBFBF,0xE0E0,0xBFBF,0xD0D0,0x7171,0x6161,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x6161,0x9F9F,0x6161,0x6161,0x6161,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0xC0C0,0xD0D0,0xC0C0,0xA1A1,0xB2B2,0xC3B2,0xE0E0,0xDFDF,0xEFEF,0xF0F0,0xE0E0,0x8281,0x82B3,0xC372,0x7272,0x6262,0x6262,0x6261,0x7162,0x71C4,0x7272,0x6272,0xC3D4,0xC3D0,0xC3C0,0xD0D0,0xD0D3,0xD0D0,0xD3E4,0xE0D0,0xD0E4,0xD0D0,0xC0D3,0xC0D4,0x71D0,0xD4E4,0x6161,0x7262,0x7262,0x71C4,0x6262,0x5162,0xC3C3,0xC071,0xD0D0,0xD471,0xD4C0,0xC0D4,0xD4C0,0x71C3,0x7171,0xD3D3,0x6161,0x6262,0xC462,0x6262,0x6161,0x71C3,0xD0D0,0xD0D0,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x5050,0x6060,0x8F8F,0x6060,0x8F8F,0xAFAF,0x7070,0x5050,0x8F8F,0x6060,0x5050,0x5050,0x8F8F,0x5050,0x4040,0x7070,0xE0E0,0xDFDF,0xFFFF,0xF0F0,0xBFBF,0x7070,0x9F9F,0x9F9F,0x5050,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x7070,0x6060,0x9F9F,0xAFAF,0x7070,0xAFAF,0x7070,0xC0C0,0xAFAF,0x7070,0xC0C0,0x9F9F,0x8080,0x7070,0xAFAF,0x9F9F,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x8F8F,0x5050,0x9F9F,0xAFAF,0x7070,0x6060
.word 0xD0D0,0xC0C0,0x7171,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xD4D4,0xAFAF,0xAFAF,0x6161,0xAFAF,0x6161,0xAFAF,0xAFAF,0x9F9F,0x5252,0x6161,0x9F9F,0xAFAF,0x5151,0x5151,0x4141,0x8F8F,0x6161,0x7171,0xDED0,0xB1B1,0xA1B1,0xB2A1,0xC1B1,0xF0F0,0xDFDF,0xDFDF,0xEFEF,0xE0E0,0x8171,0xC382,0x7272,0xC471,0x6271,0x6261,0x7262,0x7262,0x7262,0x7272,0x71C4,0xD071,0xC0D4,0xC3C3,0xD3C0,0xD4C0,0xD4D0,0xD3D0,0xD3D3,0xD3E0,0xD3D0,0xD3D3,0xD3D0,0xD3D3,0xD0D0,0x6261,0x6262,0x6162,0x6262,0x6272,0x6262,0x6161,0xD4D4,0xD471,0xC0C0,0x71D4,0xD4D4,0xC0C0,0xD3D3,0xC3C3,0xD3C3,0xC3C3,0xC461,0x6262,0x5262,0xC0C0,0xD4D4,0x7171,0xD0D0,0x6060,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x5050,0x5050,0x5050,0x7070,0x6060,0x8F8F,0x5050,0x6060,0x9F9F,0x5050,0x5050,0x8F8F,0x5050,0x9F9F,0xBFBF,0xF0F0,0xF3F3,0xBFBF,0xAFAF,0x9F9F,0x5050,0x5050,0x8F8F,0x7070,0x6060,0x9F9F,0x6060,0x7070,0x7070,0x6060,0x9F9F,0x6060,0x7060,0x7070,0xAFAF,0x8080,0xBFBF,0xE0E0,0xD0D0,0xD0D0,0xD0D0,0x7070,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7070,0x7070,0x6060,0x9F9F
.word 0xD0D0,0x7171,0x6161,0x7070,0xBFBF,0xBFBF,0xBFBF,0xC3C3,0x6262,0x6262,0xAFAF,0xAFAF,0xAFAF,0x7171,0x6262,0x6262,0xAFAF,0x9F9F,0xAFAF,0x6161,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x6060,0xC0C0,0xBFBF,0xC0C0,0xA1B2,0xB2B2,0xC1B2,0xE0E0,0xEFEF,0xEFEF,0xF0F0,0xDFDF,0xC3C0,0x72B3,0xC372,0x7172,0x6162,0x61C4,0x62C4,0x6272,0x6272,0x7272,0x6271,0xD4D0,0xC3D0,0xD0D3,0xD0D4,0xD0D3,0xD0D3,0xD0D3,0xD0D3,0xD0D4,0xD3C3,0xC3D0,0xD4D0,0xD3C0,0xC3D0,0x6261,0x7162,0x62C4,0x6262,0x6262,0x6161,0xD3D3,0x70C0,0xC3C3,0xC3C0,0xD0D0,0x71C3,0xD471,0xC0D3,0x7272,0xD0D0,0x5261,0x6262,0x6262,0x6262,0x6161,0x71D4,0xC0C0,0x71C3,0xAFAF,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x6F6F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x5050,0x5050,0x9F9F,0x6060,0x5050,0x5050,0x9F9F,0x6060,0x5050,0x8F8F,0x5050,0x4040,0xE0E0,0xF0F3,0xF3F3,0xCFCF,0x7070,0x7070,0x6060,0x8F8F,0x5050,0x5050,0x8F8F,0x6060,0x6060,0x6060,0xAFAF,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x7070,0x8080,0xAFAF,0xD0D0,0xF0F0,0xE0FE,0xE0E0,0xBFBF,0x7070,0x7070,0x5151,0x5151,0x4040,0x5050,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x6060,0xAFAF,0xAFAF,0x6060,0x6050
.word 0xCFCF,0x7171,0xAFAF,0x6161,0x7070,0xBFBF,0xD0D0,0xBFBF,0xD4D4,0xAFAF,0xD4D4,0x7171,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x6262,0x6161,0xAFAF,0xAFAF,0xD4D4,0xBFBF,0xAFAF,0xAFAF,0x8080,0xD0D0,0xD0D0,0xB0B0,0xB2A1,0xB2B2,0xC191,0xF0F0,0xDFDF,0xDFDF,0xDFDF,0xF0F0,0x7171,0xB381,0x72B3,0x7272,0x6272,0x7262,0x7162,0xC462,0x7262,0x72C4,0xD472,0xD0C3,0xC0C3,0xC0C3,0xD0D3,0xD0D4,0xD0D3,0xC0D4,0xD0D3,0xD4D0,0xD3C0,0xC0C3,0xC0D4,0xD0D4,0x61D4,0x6261,0xC472,0x6271,0x6272,0x6162,0x5252,0x7171,0xD4C3,0x7171,0xD4D4,0xD4C3,0xD3D3,0xC3C3,0x7171,0xC0D4,0xC3D4,0x6161,0x6262,0x7262,0x4162,0xD3D3,0xC0C0,0x7171,0xC0C0,0x7171,0x4040,0x7F7F,0x7F7F,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x9F9F,0x6060,0x7070,0x9F9F,0x6060,0x9F9F,0x6060,0x7070,0x6060,0x6060,0x9F9F,0xAFAF,0xF0F0,0xEFEF,0xCFCF,0x6161,0x9F9F,0x9F9F,0x9F9F,0x5050,0x8F8F,0x4040,0x5040,0x9F9F,0x6060,0x6060,0x6070,0x6060,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0xC0C0,0xF0E0,0xE0E0,0xDFDF,0x8080,0xAFAF,0x7070,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x4040,0x5050,0x6060,0x5050,0x5050,0x8F8F
.word 0xD0D0,0xC0C0,0xBFBF,0x6161,0x9F9F,0x7070,0xAFAF,0xD0D0,0xD4D4,0xD4D4,0xAFAF,0xBFBF,0xAFAF,0xD4D4,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xD4D4,0xD4D4,0xAFAF,0xE4E4,0xCFCF,0x7070,0xAFAF,0x8181,0xCECE,0xC0D0,0xA1B2,0xB2B2,0xD0B2,0xE0E0,0xDFDF,0xEFEF,0xDFDF,0xF0F0,0x71E0,0x72B3,0x7272,0xC472,0xC471,0x6271,0x6272,0x6272,0x7272,0x7272,0x7171,0xD3D3,0xC3C3,0xD4C0,0xC3C0,0xD4C0,0xD3D3,0xD4C0,0xD4D0,0xD3C0,0xD0D3,0xC0D4,0xC0D4,0xD0D4,0x61D4,0x6262,0x7162,0x6283,0x71C4,0x61C4,0x6161,0xD0D4,0xC0C0,0xC3C3,0x71C3,0xC0C0,0xD371,0xD471,0x7171,0x7171,0xD0C3,0x5162,0x6262,0x6262,0x6262,0x6161,0x71C3,0xC0D4,0xC3C3,0xC0C0,0x6161,0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x5050,0x9F9F,0x7070,0x6060,0x6060,0x6060,0x7070,0x6060,0x9F9F,0x7070,0xF0F0,0xFFFF,0xF0F0,0xCFCF,0x9F9F,0x5050,0x6060,0x6060,0x6060,0x5050,0x4040,0x8F8F,0x5050,0x5050,0x6060,0x9F9F,0x6060,0x6060,0xBFBF,0x8080,0x6060,0x9F9F,0x7070,0x7070,0x8080,0xD0D0,0xD0D0,0xD0D0,0xBFBF,0xAFAF,0x6060,0x4040,0x4141,0x4141,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x5050,0x5050
.word 0x7171,0xAFAF,0xD0D0,0xC0C0,0x5151,0x6161,0xAFAF,0xBFBF,0xD0D0,0xBFBF,0xD0D0,0xD4D4,0x7171,0xAFAF,0xD0D0,0x7171,0xAFAF,0xD4D4,0x6161,0xAFAF,0xAFAF,0xBFBF,0xD0D0,0xAFAF,0x6060,0xC0C0,0xD0D0,0xB2B2,0xB2A1,0x92B2,0xD0B2,0xCFCF,0xDFDF,0xEFEF,0xEFEF,0xF0F0,0xD0D0,0x7271,0x72C3,0x7272,0x7272,0xC462,0x6262,0x7262,0x7262,0x7172,0x71C4,0xD0D3,0xD081,0xC0C3,0xC3D4,0xC3C3,0xD0C0,0xC0D4,0xC0D4,0xD3D3,0xC3D0,0xE4D0,0xD4C0,0xD3C0,0x6161,0x6262,0x6262,0x6262,0x6262,0x6262,0x6161,0xC3C3,0xD471,0xC3D0,0xD4D0,0xC371,0xD4D4,0xD0D0,0x71D4,0x71D4,0xD4D4,0x6262,0xC462,0x6262,0x5262,0xC0C0,0xD4D4,0x7171,0xC0C0,0x7171,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x5050,0x9F9F,0x7070,0x7070,0x9F9F,0x6060,0x6060,0x6060,0xAFAF,0x7070,0xAFAF,0xF0F0,0xF0F0,0xEFEF,0xD0D0,0x5050,0x8F8F,0x9F9F,0x6060,0x8F8F,0x8F8F,0x5050,0x6060,0x7070,0x6060,0x9F9F,0x6060,0x5050,0x6060,0xF0F0,0xCFCF,0x6060,0x6060,0x6060,0x7070,0xAFAF,0xC0C0,0xBFBF,0xC0C0,0xD0D0,0xD0D0,0xAFAF,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x6F6F,0x4040,0x4040,0x5050,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4141,0x9F9F,0x8F8F,0x4040,0x8F8F
.word 0x6161,0x6161,0xAFAF,0xBFBF,0x6060,0x5050,0xAFAF,0xD0D0,0xBFBF,0xD0D0,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0xD4D4,0x6161,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x7171,0xC0C0,0xAEAE,0xA1C0,0xB2B2,0xB292,0xD3B2,0xE0E0,0xDFDF,0xDFDF,0xF0F0,0xF0F0,0xD4F0,0xB372,0x7272,0x72C4,0x6272,0x6262,0xC461,0xC471,0x7272,0x72C4,0xD361,0xC3D0,0xC3C3,0xD471,0xD0C3,0xC1D0,0xD4D4,0xC3C0,0xC3C3,0xC0C3,0xC0D4,0xC0D4,0xD0D0,0xD0D3,0x61C3,0x7262,0x6272,0x71C4,0x6262,0x6162,0x5252,0xD4D0,0xD4C0,0x6262,0xC0C0,0xD3D3,0xC072,0x7171,0xC3C3,0xC3C3,0x71E3,0x6262,0x6262,0x6262,0x4152,0xD4D4,0x71D4,0xC3C3,0xD4D4,0xC1C1,0x6071,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x5050,0x6060,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x7070,0xAFAF,0xC0C0,0xF0F0,0xF0F0,0xE0E0,0xAFAF,0x5050,0x6060,0x5050,0x5050,0x6060,0x5050,0x6060,0xAFAF,0xEFEF,0xAFAF,0x6060,0x9F9F,0x5050,0x6060,0xF0F0,0xC0C0,0x6060,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0xE0E0,0xBFBF,0x9F9F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x5151,0x4040,0x4040,0x407F
.word 0x6161,0xAFAF,0xD0D0,0xC0C0,0x5151,0x4141,0x7070,0xAFAF,0xC0C0,0xAFAF,0xAFAF,0xC0C0,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xD0D0,0xAFAF,0x6161,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xD4D0,0xBFBF,0x81C0,0xB2B2,0xB2B2,0xC0B2,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xF0F0,0xD0D0,0x7271,0x72B3,0x7272,0xC462,0x6271,0x6272,0x7262,0xC471,0x7271,0xD0D4,0x81D4,0xC0C3,0xD0D4,0xD4C0,0xD4C3,0xC3C0,0xD4D0,0xC3C0,0xD4C0,0xC3C3,0xD4C0,0xC3C3,0xE471,0x6161,0xC462,0xC362,0x6262,0x6272,0x6262,0x7171,0x7171,0x7171,0xC3C3,0x71D4,0xC3C3,0xD4D4,0x71D4,0xC3C3,0xD471,0xC3C3,0x6262,0x6262,0x6262,0x5151,0xC0C0,0x7171,0xC3C3,0xD0C0,0xC0C0,0xAFAF,0x5050,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x9F9F,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x7070,0x7070,0x7070,0x7070,0xD0D0,0xE0E0,0xCFCF,0xBFBF,0x6060,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0xE080,0xF0F0,0x6060,0x6060,0x6060,0x5050,0x6060,0xBFBF,0xD0D0,0xAFAF,0x6060,0xD0D0,0xBFBF,0x7070,0x7070,0x6060,0xD0D0,0xBFBF,0xAFAF,0x4141,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x7F7F
.word 0xBFBF,0x7070,0xAFAF,0xBFBF,0xAFAF,0x4141,0x4040,0x9F9F,0xAFAF,0xAFAF,0xC0C0,0xAFAF,0xD4D4,0x6161,0xAFAF,0xAFAF,0xD0D0,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0x7171,0xC0C0,0xBEBE,0xB1C0,0xB2B2,0xB292,0xD282,0xE0E0,0xDFDF,0xDFDF,0xF0F0,0xF0F0,0xD0D0,0x7271,0x72B3,0x72C3,0x6262,0x6272,0x7272,0x7262,0x7262,0x7172,0xD471,0xC3C0,0xC3C3,0xD471,0xC0C0,0xC0C3,0x71D4,0xC3D0,0xD4C0,0xD071,0xC0C3,0x71D4,0xD4D0,0xC3D0,0x6161,0x7162,0x6262,0x7272,0x71C4,0x6262,0x6262,0xC0D4,0xC0D4,0x70D4,0x7272,0xD0D0,0xC3C0,0xC3C3,0x7171,0xC0D4,0x61D4,0x6262,0x6262,0x6262,0x5162,0xE461,0x71D4,0xD4D0,0xD0D0,0xD3D3,0x7161,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x5050,0x7070,0xAFAF,0x7070,0x9F9F,0x6060,0x7070,0x6060,0x7070,0x7070,0x7070,0xAFAF,0xC0C0,0x8080,0x9F9F,0x5050,0x5050,0x5050,0x9F9F,0x6060,0x5060,0x6060,0xCFCF,0xE0E0,0xD0D0,0x5050,0x9F9F,0x6060,0x5050,0x7070,0xE0D0,0x7070,0xD0D0,0xD0D0,0xF0E0,0x6060,0x9F9F,0x8080,0xE4E4,0xE5E5,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F40
.word 0xD0D0,0xAFAF,0x7070,0xBFBF,0xC0C0,0x9F9F,0x8F8F,0x6060,0x7171,0x6161,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7171,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xC0D0,0xBEBE,0xB271,0xB2A1,0xB2B2,0xC0B2,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xF0F0,0xF0F0,0x72D3,0x72B3,0x7272,0x6272,0xC462,0x6262,0x7272,0x72C4,0x6262,0xD0D4,0xC3C3,0xD471,0xD4C0,0xD471,0xD4C0,0xD4C0,0x71D4,0xC0D4,0xD4C3,0xC3C3,0xD3C0,0x7171,0xC3C3,0x6162,0x6262,0x6262,0x6262,0x6262,0x6262,0x6161,0xAFAF,0x6262,0xC0C0,0xC3C3,0x72D4,0xD4D4,0xD4C0,0x7272,0x7171,0xD4D0,0x6251,0x6262,0x6262,0x5152,0xAFAF,0xC0C0,0xC3C3,0xD3C0,0xD0D0,0xAFAF,0x4061,0x7F7F,0x6F6F,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7070,0x7070,0x7070,0x9F9F,0x6060,0x6060,0x7070,0x6060,0x6060,0x7070,0x7070,0x8080,0x7070,0x7070,0x7070,0x6060,0x5050,0x5050,0x6060,0x9F9F,0x9F9F,0x7070,0xE0E0,0xD0D0,0xD0D0,0x6060,0x6060,0x5050,0x6060,0xC0C0,0xAFAF,0xE0E0,0xD0D0,0xC0C0,0xF0F0,0xAFAF,0x6060,0x7070,0xE4E4,0xD4D4,0x4141,0x4141,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F
.word 0xD0D0,0x7070,0x9F9F,0x7070,0xBFBF,0xAFAF,0x5151,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6161,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xD0D0,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xC3C3,0xC0C0,0xCFCF,0xB1C0,0x92B2,0x82B2,0xD0B3,0xE0E0,0xDFDF,0xDFDF,0xF0F0,0xF0F0,0xF3F0,0x71C0,0x7272,0xC472,0x7262,0x6272,0xC471,0x7262,0x7162,0x62C4,0xC3C0,0x71C3,0x71D3,0xC0D4,0xC3C3,0xC371,0x7171,0xC0C0,0xD4C0,0x7171,0xC371,0x7272,0xC3C0,0x6161,0x6161,0x7262,0x6262,0x6162,0x6262,0x5161,0x6262,0x71D0,0x71D4,0x71C4,0x7171,0x71D3,0x71D4,0xC0C0,0xD4D4,0xD471,0xD4D4,0x6251,0x6262,0x6262,0x5252,0xC061,0xC3D4,0xD0D0,0xD3D3,0xE3D0,0x5151,0x6161,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x5050,0x6060,0xAFAF,0x7070,0x5050,0x6060,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7060,0x9F9F,0x7070,0x8080,0x7070,0x9F9F,0x6060,0x9F9F,0x6060,0x6060,0x7070,0x7070,0x7171,0xC0C0,0xD0D0,0xAFAF,0x6060,0x5050,0x5050,0x6060,0xC0C0,0xD0D0,0x7070,0xD0D0,0xD0D0,0x6060,0x7070,0xAFAF,0xCFCF,0xD4D4,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x5050,0x4040,0x7F7F,0x7F7F,0x4040,0x4141,0x4040,0x4040,0x407F
.word 0xBFBF,0x8080,0x5050,0x9F9F,0xD0D0,0xAFAF,0xAFAF,0xAFAF,0x6161,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x6161,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xDEDE,0xCFCF,0xCFCF,0xD4D0,0xBFBF,0x8181,0xB2B2,0xB2B2,0xD3B2,0xCFCF,0xE0E0,0xDFDF,0xF0F0,0xF0F0,0xF0F0,0x72C3,0x7272,0x6272,0x6272,0x62C4,0x6272,0xC472,0x6262,0x6161,0xC3D3,0xC371,0xC3C0,0xD471,0xC371,0xC0C3,0xD4C3,0x71D4,0x70D4,0xC0E4,0xD4D4,0xD371,0xD471,0x7171,0x6252,0x6262,0x6262,0x6262,0x6262,0x5252,0x9F9F,0xD4D4,0x7171,0x7171,0xD4D4,0xD0D0,0x7171,0x7171,0xC0C3,0xD4D4,0x6180,0x5252,0x6262,0x6262,0x5252,0xD4D4,0x7171,0xD4C3,0xD0D0,0xD0D0,0x6161,0x6161,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x5050,0x9F9F,0x6060,0x6060,0x9F9F,0x5151,0x8F8F,0x7060,0x7070,0x7070,0x6060,0x6060,0x9F9F,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x7070,0x7070,0x7171,0xC0C0,0xD0D0,0x6060,0x6060,0x5050,0x8080,0xC0C0,0xD0D0,0x8080,0x7171,0x5050,0x5050,0xAFAF,0xD0F4,0xD4D4,0xAFAF,0x9F9F,0x4141,0x4040,0x4040,0x4040,0x4141,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x6F6F,0x8F8F,0x5050,0x8F8F,0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F
.word 0xBFBF,0x6060,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0x7171,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xCFCF,0xCFCF,0xCFCF,0xC0D0,0xD0D0,0x81C0,0xB2B2,0xB392,0xC082,0xCFCF,0xDFDF,0xDFDF,0xF0F0,0xE0F0,0xF0F2,0x71D3,0xC472,0x7272,0x7272,0x7262,0x6262,0x6262,0x7272,0x7262,0xC3D0,0x71C3,0xC0D4,0xC0D4,0xC0C0,0x7272,0x8080,0x7171,0xC3C3,0x7171,0xD071,0xD4D4,0xC0C0,0x61C4,0x6262,0x6262,0x6262,0x61C4,0x6262,0x5151,0x7162,0x7171,0xC0C0,0x7171,0xD471,0x7171,0x71D3,0x7272,0xD4D4,0x7171,0xD4D4,0x6252,0x6262,0x6262,0x4162,0xD0D0,0xC0D4,0xC0C0,0xD0D4,0xD4D4,0xD0D0,0x7162,0x4040,0x7F7F,0x5050,0x8F8F,0x8F8F,0x5050,0x5050,0x9F9F,0xC0C0,0x7070,0x6060,0x6060,0x5050,0x5050,0x9F9F,0x7070,0x6060,0x6060,0x6060,0x7070,0x6060,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x9F9F,0x7070,0xAFAF,0x8080,0x7070,0x7171,0xD0C0,0x5050,0x5050,0x5050,0x9F9F,0xC0C0,0xD0D0,0x7070,0x7070,0x9F9F,0x6060,0xD0D0,0xCFCF,0xAFAF,0x6161,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F
.word 0xD0D0,0xAFAF,0x4040,0x9F9F,0x7070,0xAFAF,0xC0C0,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xC0C0,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xD3D3,0xCEC0,0x8181,0xB2B2,0xB2B2,0xD0B3,0xE0E0,0xF0F0,0xF0F0,0xF0F0,0xF2F0,0xF4F0,0x72C0,0x7272,0xC472,0xC462,0x6272,0xC472,0x7262,0x6262,0xD461,0xC3C3,0xC381,0x72C3,0x7171,0x72D4,0xC3C3,0xD471,0xD4C0,0x7171,0x71D4,0xD4D4,0x8071,0x71D4,0x5151,0x6152,0x6262,0x6262,0x6261,0x6162,0x5152,0xAFAF,0x6171,0xC3C3,0x7171,0x7171,0x7171,0xAFAF,0x71D4,0x7171,0xC0C0,0x61D4,0x5252,0x6262,0x6262,0x5152,0x7171,0xD4D4,0xC3C3,0xD0D0,0xD0D0,0x6161,0x7151,0x9F9F,0x4040,0x9F9F,0x6060,0x5050,0x8F8F,0x6060,0x7070,0xC0C0,0xC0C0,0x7070,0x5050,0x8F8F,0x5050,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0x6060,0x6060,0x7070,0x9F9F,0xD070,0xD0D0,0x7070,0x8080,0xAFAF,0xD0D0,0x5050,0x5050,0x7050,0x8080,0xC0C0,0xD0D0,0x8080,0x5050,0x9F9F,0xAFAF,0xAFAF,0x7070,0x5151,0x4141,0x7F7F,0x4040,0x4040,0x4141,0x4040,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F6F
.word 0xF0F0,0xD0D0,0x6060,0xAFAF,0xD0D0,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xC0C0,0xC0C0,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0xBFBF,0xC3D0,0xC0C0,0xC0C0,0xB2B2,0xB292,0xC082,0xCFCF,0xF0E0,0xF0F0,0xF0F0,0xE0F0,0xE0F2,0x7172,0x72C3,0x7272,0x7272,0x6262,0x6262,0xC462,0xC462,0xD361,0xC3C0,0xD471,0xD471,0xC371,0x7171,0x7171,0x71D4,0x7171,0xC0C0,0x7171,0x71D4,0x7171,0x7171,0x6161,0x6262,0x6262,0x6262,0x6262,0x5262,0x5151,0xD4D4,0x6262,0x7272,0x7171,0xC3C3,0x62D4,0x7171,0x7171,0x7171,0xD4D4,0x7171,0x6152,0x6262,0x6262,0x5252,0x6262,0xC0D4,0xD0D0,0xD3D3,0xD3D3,0xD3D3,0x6262,0x7070,0x5050,0x5050,0x6060,0x8F8F,0x6060,0x6060,0x7070,0xC0C0,0x6060,0x9F9F,0x6060,0x4040,0x5050,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x6060,0x6060,0x7070,0xAFAF,0x6060,0x6060,0x7070,0xC0C0,0xAFAF,0x7070,0xBFBF,0xE0E0,0x7070,0xC0C0,0xC0C0,0xC0C0,0x5050,0x8F8F,0x5050,0xAFAF,0x8080,0xD0D0,0x6060,0x9F9F,0x6060,0x7171,0x6070,0x5050,0x8F8F,0x4141,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x7F40
.word 0xDFDF,0xD0D0,0xAFAF,0xAFAF,0xBFBF,0x5151,0x5050,0x5151,0x6060,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xC3C3,0xC0C0,0xB0B0,0xB2B2,0xB3B2,0xC382,0xCFCF,0xE0E0,0xF0F0,0xF0F0,0xF3E0,0xE3F0,0x72C3,0x7272,0x72C4,0x6262,0xC462,0x6262,0x6272,0x6172,0xC3D4,0xC372,0xD471,0x7171,0xD471,0xD471,0xD471,0x7070,0x7272,0x7171,0x71D4,0x7171,0x7272,0x71D4,0x5252,0x6252,0x6262,0x6262,0x6261,0x6161,0x5252,0xC0C0,0x7171,0x6262,0x7171,0x7171,0x6161,0xD4D4,0x7171,0xD4D4,0xD471,0x6161,0x5251,0x6252,0x6262,0x4141,0xC3C3,0x7171,0xC3C3,0xD4D0,0xD0D0,0x6161,0x6261,0xAFAF,0x9F9F,0x6060,0x5050,0x6060,0x6060,0x7070,0x8080,0x7070,0x7070,0x6060,0x8F8F,0x4040,0x8F8F,0x5050,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x7070,0x7070,0x6060,0xD0D0,0xDFDF,0x7070,0x7070,0xD070,0xD0D0,0xE0E0,0xE3F2,0xC0C0,0xC0C0,0x6060,0x5050,0x5050,0x7170,0xC0C0,0xD0D0,0x7070,0x5050,0x6060,0x6060,0x9F9F,0x5050,0x8F8F,0xE4E4,0xDFDF,0x4040,0x4141,0x6F6F,0x4040,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x6F6F,0x6F6F
.word 0xE0E0,0xDFDF,0xD0D0,0xAFAF,0x6060,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0x8080,0xBFBF,0x7171,0xC0C3,0xCFCF,0xB281,0xB282,0xC082,0xCFCF,0xE0E0,0xE0E0,0xF0F0,0xE0E3,0xE0F0,0xC371,0xB372,0x8372,0xC472,0x6262,0x6272,0x62C4,0x6272,0xD361,0x71C3,0x71D4,0x7272,0x7171,0x7171,0xC0C0,0x71D4,0xC3C3,0x7171,0x7171,0xC3C3,0xC0C0,0x6262,0x5151,0x6261,0x6261,0x6262,0x61C4,0x5262,0x6151,0x6262,0xC3C3,0x6161,0x6262,0xAFAF,0xC0C0,0x7161,0xAFAF,0x7171,0x7171,0xD4D4,0x5251,0x6262,0x6262,0x4152,0x9F9F,0x71C3,0xC3C3,0xD0D0,0xD3D3,0xD0D0,0x6251,0x9F9F,0x6060,0x5050,0x5050,0x6060,0x7070,0x7070,0xAFAF,0xD0D0,0xBFBF,0x6060,0x5050,0x6060,0x4040,0x6060,0x9F9F,0x6060,0x9F9F,0x6060,0x6060,0x5050,0x6060,0x6060,0x7070,0x6060,0x8080,0xCFCF,0xE4F3,0xAFAF,0x7070,0x7070,0x7171,0xF4C0,0xF0F0,0xC0E4,0xC0C0,0x5050,0x5050,0x8F8F,0x7070,0x6161,0xD0D0,0xD0D0,0x5050,0x5050,0x9F9F,0x5050,0x4040,0x6161,0xF0F0,0xEFEF,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x4040,0x7F6F
.word 0xAFAF,0xF0F0,0x8080,0x5050,0x8F8F,0x4040,0x4040,0x6060,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xD0D0,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xAEAE,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0xC071,0xD0D0,0xB281,0xB2B2,0xC0B3,0xE0E0,0xF0F0,0xF0E0,0xE0F3,0xF0E0,0xD3F0,0x72C3,0x83B3,0x6272,0x6262,0x6262,0x6262,0xC462,0x6162,0xC3D4,0xD471,0xD471,0xD471,0xD471,0xD471,0x7272,0xD461,0x7171,0xAFAF,0x7171,0x7171,0x6161,0x61D4,0x5252,0x6251,0x6262,0x6162,0x6261,0x5152,0x8F8F,0x7171,0x6161,0x7171,0x6161,0x7171,0x7171,0xBFBF,0x6171,0x7171,0x7171,0x5252,0x4141,0x6262,0x5252,0x4152,0x7171,0xD4D4,0xC3C3,0xD4D0,0xD0D0,0xD3D3,0x6151,0x5050,0x9F9F,0x5050,0x6060,0x9F9F,0x7070,0x7070,0xC0C0,0xD0D0,0xAFAF,0x9F9F,0x5050,0x9F9F,0x5050,0x8F8F,0x7070,0x7070,0x6060,0x9F9F,0x6060,0x5050,0x5050,0x6060,0x6060,0x7070,0x7070,0xF4E0,0xE4E4,0x7070,0xBFBF,0xE0E0,0x7171,0xD0D0,0xD0D3,0xD0D0,0xC0C0,0xC0C0,0x5050,0x5040,0x6060,0x7171,0x7171,0xD0D0,0x5050,0x5050,0x6060,0x8F8F,0x4040,0x9F9F,0xE0E0,0xBFBF,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x6F7F
.word 0xAFAF,0xBFBF,0xC0C0,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x9F9F,0x7070,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x6060,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xCECE,0xE0E0,0x7171,0xAFAF,0xC072,0xCFCF,0xB281,0xB3B2,0xB382,0xCFCF,0xE0E0,0xE0E0,0xE0E0,0xF3F0,0xD3F0,0x7271,0x72B3,0xC462,0x6272,0x6262,0x62C4,0x6272,0x6162,0x71D4,0x71D4,0x7171,0x7171,0x7171,0x7171,0x7272,0x7171,0x7171,0x7171,0x6161,0xD4D4,0xD471,0x6161,0x5251,0x6262,0x6262,0x5262,0x6262,0x4141,0x6161,0x7171,0x61D4,0x7171,0x6262,0xAFAF,0x6161,0x7171,0xAFAF,0x6262,0xAFAF,0x5171,0x5252,0x6262,0x5262,0x5151,0xD461,0x7171,0xD0D0,0xD0D4,0xD0D0,0xBFBF,0x6261,0x4141,0xAFAF,0x5050,0x6060,0x7070,0x7070,0xBFBF,0xCFCF,0xCFCF,0xAFAF,0x5151,0x5050,0x8F8F,0x6060,0x6060,0x7070,0x9F9F,0x9F9F,0x6060,0x6060,0x5050,0x6060,0x6060,0x6060,0x7070,0xE4E4,0xE5F4,0xE4E4,0x6161,0xF2F2,0xF0F0,0xD0D0,0x71D0,0x7171,0xC0D4,0xC0C0,0x6060,0x4040,0x8F8F,0x7171,0x7171,0xD0D0,0xD0D0,0x6060,0x6060,0x8F8F,0x5050,0x8F8F,0xE0E0,0xE0E0,0x4040,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x6F6F,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F40
.word 0x9F9F,0xC0C0,0x5050,0x4040,0x4040,0x7F7F,0x7F7F,0x9F9F,0x6060,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x6060,0x9F9F,0xAFAF,0xAFAF,0x9E9E,0xBEBE,0xCFCF,0xCECE,0xD0D0,0x7171,0xC371,0xBFBF,0xB2C0,0xB292,0xB2B3,0xD0D0,0xE0E0,0xE2E0,0xE3E0,0xF1F0,0xD3F2,0xB372,0x8372,0x6283,0x6262,0x62C4,0x6262,0x6262,0x6262,0xC3C3,0xD471,0xC3C3,0xD461,0xD471,0xD471,0xD4C0,0x7171,0x7171,0xD4D4,0x7171,0x7171,0x7171,0x5151,0x5141,0x6262,0x6261,0x6162,0x6152,0x4152,0x8F8F,0xAFAF,0x6161,0x7171,0x7171,0x6262,0x6161,0xAFAF,0xC3C3,0x6161,0x7171,0x6161,0x5241,0x6252,0x5262,0x4141,0x9F9F,0xD481,0xD4D0,0xD0D0,0xD3D3,0x6161,0x6152,0x8F8F,0xC0C0,0x6060,0x6060,0x7070,0x8080,0xE0E0,0xF0F0,0xBFBF,0xD4D4,0x9F9F,0x8F8F,0x4040,0x6060,0xAFAF,0xAFAF,0x6060,0x6060,0x5050,0x6060,0x5050,0x5050,0x6060,0x6060,0x7070,0xE4D0,0xE5E5,0xBFBF,0x5151,0x6161,0x7171,0xC0C0,0x7171,0xC0C0,0xD4C0,0xD0D0,0xC0C0,0x5050,0x4040,0x8F8F,0x8080,0xC0C0,0xC0C0,0xAFAF,0x9F9F,0x6060,0x6060,0xD0D0,0xE0E0,0xD0D0,0x9F9F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x6F6F
.word 0x9F9F,0x6060,0x8F8F,0x7F7F,0x4040,0x4040,0x7F7F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBEBE,0xF0F0,0xE1E0,0xDFDF,0x8080,0xC0C0,0x9191,0x82B2,0xB3B2,0xF0D0,0xE2F0,0xE0E1,0xF0F3,0xF1E1,0xC3E1,0xB372,0xC472,0x7262,0x6262,0x6262,0x6262,0x62C4,0x6262,0x71D4,0x71C3,0x7272,0x7171,0x7171,0x7171,0x6161,0x7171,0x7272,0x7171,0x7171,0x7171,0x7171,0x6262,0x4242,0x6152,0x6252,0x6261,0x6262,0x5151,0x6152,0x7171,0x6161,0x6161,0xAFAF,0x6161,0x6161,0x7171,0x6161,0x7171,0x7171,0x4141,0x5241,0x5251,0x6262,0x4242,0xE461,0xC0C0,0xD0D4,0xC3C3,0xD0D0,0x5151,0x6261,0x4141,0xBFBF,0x7070,0x7070,0x7070,0xD0D0,0xCFCF,0xEFEF,0xAFAF,0x9F9F,0x5151,0x4040,0x4040,0x9F9F,0x7070,0x8080,0x7070,0x6060,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x6060,0x7070,0xD4D4,0xBFBF,0xD4D4,0x9F9F,0x6060,0x7171,0xC0C0,0xC0D4,0xD4D4,0xC0C0,0xD0D3,0x7070,0x4040,0x8F8F,0xC060,0x7171,0xC0C0,0xC0C0,0xD0D0,0x6060,0x7070,0xD0D0,0xCFCF,0xE0E0,0xCFCF,0xAFAF,0x9F9F,0x8F8F,0x4040,0x6F6F,0x7F7F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F6F
.word 0x9F9F,0x5151,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0xC0C0,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6060,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xDEDE,0xF0FE,0xF1E1,0xE0E0,0x5151,0xD071,0x7171,0xB2B2,0xB2B2,0xF3C2,0xE0F0,0xE1E1,0xE1E1,0xF1E1,0x72E0,0xB372,0x7272,0x62C4,0x62C4,0x6283,0x62C4,0x6262,0x6262,0xC3C3,0x7172,0x71D4,0x72D4,0xC3C3,0x7272,0x7171,0x7171,0x7171,0x7171,0xAFAF,0x7171,0x7171,0x5151,0x5141,0x5251,0x6162,0x6252,0x5162,0x5152,0x9F9F,0x6161,0x7171,0x6161,0x6161,0xAFAF,0x6161,0x6161,0xAFAF,0x7171,0xAFAF,0x5252,0x5252,0x6262,0x5252,0x4141,0xC0C0,0xC3D4,0xD4D4,0xD0D3,0xD3D3,0xD0D0,0x6262,0x7070,0x8080,0x7070,0x7070,0x8080,0xBFBF,0xCFCF,0xF5F5,0xD4D4,0x9F9F,0xAFAF,0x8F8F,0x4040,0xC0C0,0xC0C0,0xAFAF,0x9F9F,0x9F9F,0x6060,0x5050,0x8F8F,0x5050,0x5050,0x6060,0x6060,0x6161,0x9F9F,0xAFAF,0x9F9F,0x4141,0x6161,0x7171,0x7171,0xC0C0,0xC0D4,0xD0D4,0xD0D0,0x5050,0x4040,0x9F9F,0xC0C0,0xC071,0xC0C0,0x8080,0x7070,0xAFAF,0xCFCF,0xF0F0,0xDFDF,0xE0E0,0xDFDF,0xCFCF,0xBFBF,0x8F8F,0x5151,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x6F6F,0x6F7F
.word 0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x9F9F,0xBFBF,0x8080,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5050,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCECE,0xF0F0,0xF1E1,0xF1F1,0x4141,0x5151,0xD0D0,0x92C2,0x92B2,0xF1C2,0xE2F0,0xE1E1,0xD1D1,0xE1E1,0xB3D3,0xB3B3,0x8383,0x6262,0x6262,0x6262,0x6262,0x5262,0xC361,0x71C3,0x7272,0x7171,0x7171,0x7272,0x7171,0x6161,0x71D4,0x7171,0x7171,0x7171,0x7171,0x6262,0x4161,0x5151,0x6252,0x5252,0x5261,0x5251,0x4141,0x9F9F,0x6161,0xAFAF,0x6161,0x9F9F,0x6161,0x6161,0x6161,0x6161,0xAFAF,0x7171,0x4141,0x5241,0x5262,0x5251,0x4141,0x9F9F,0x7171,0xC0D3,0xD4D4,0xD0D0,0xD4D4,0x6261,0x5151,0xBFBF,0x7070,0x7070,0x8080,0xD0D0,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x5050,0x4040,0x8F8F,0xC0C0,0xD0D0,0xAFAF,0x6060,0x6060,0x9F9F,0x5050,0x4040,0x5050,0x5050,0x6060,0x6060,0x9F9F,0x6161,0xAFAF,0x6161,0x9F9F,0x7171,0xC0C0,0xC0C0,0xC3C3,0xD371,0xF2E0,0xC0C0,0x8F8F,0x4040,0xC0C0,0x7171,0xC0C0,0xC0D4,0xD0D0,0xAFAF,0xE0E0,0xE0E0,0xDFDF,0xE0E0,0xF3F3,0xFFFF,0xFFFF,0xDFDF,0xBFBF,0xBFBF,0xDFDF,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F40
.word 0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xBFBF,0xBEBE,0xCFCF,0xE0E0,0xF1E1,0x7171,0x7F7F,0x4040,0x8F8F,0xB2E0,0xB2B2,0xD1B1,0xE1F1,0xD1D1,0xD1D2,0xE2E1,0x72C3,0x72B3,0x7283,0x6273,0x6273,0x6262,0x6273,0x6262,0xD4C4,0xC471,0x7272,0x71D4,0x61D4,0x7171,0x6262,0x7171,0x6161,0x6262,0x6161,0x6161,0xAFAF,0x7171,0x5151,0x5252,0x5151,0x6152,0x6152,0x5252,0x5151,0x7171,0xAFAF,0x6161,0xAFAF,0x6161,0x6161,0xAFAF,0xAFAF,0x6161,0x6161,0x7171,0x5151,0x5141,0x5252,0x6262,0x4141,0xC3C3,0xD0D4,0x7171,0xD0C3,0xD0D0,0x4141,0x6251,0x8F8F,0xD0D0,0x7070,0x7070,0xC0C0,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x6161,0x6060,0x8F8F,0xC0C0,0xE0E0,0xCFCF,0x7171,0x5151,0x8F8F,0x7070,0x8F8F,0x4040,0x4040,0x4040,0x5050,0x9F9F,0x5151,0xAFAF,0x5252,0x5151,0xD4D4,0xAFAF,0x6161,0x7171,0xD071,0xD0C3,0xEFEF,0xE0F1,0x6060,0x8F8F,0x8080,0xC0C0,0xC0D4,0xC0C0,0xC0C0,0xD0D0,0x8080,0xE0E0,0xE0E0,0xD0D0,0xDFDF,0xEFEF,0xEFEF,0xDFDF,0xCFCF,0xF0E0,0xBFBF,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x7F7F
.word 0x7F7F,0x4040,0x6F6F,0x7F7F,0x8F8F,0xAFAF,0xBFBF,0xD0D0,0xC0C0,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xE0F0,0xF1D1,0xF2F2,0x4141,0x6F6F,0x8F8F,0xB2E1,0xB2A1,0xE1B2,0xF1F1,0xD1D1,0xD1D1,0xD2E1,0xB372,0xB393,0x72C4,0x6273,0x7362,0x6262,0x6262,0x6262,0xC3C3,0x7171,0x7272,0x6161,0xD461,0xD461,0x6161,0x7171,0x7171,0x6161,0x7171,0x7171,0x7171,0x6161,0x4141,0x5151,0x5252,0x5261,0x6262,0x5152,0x5151,0xAFAF,0x6161,0x9F9F,0x6161,0x6161,0x9F9F,0x6161,0x6161,0xAFAF,0x6161,0x6161,0x4141,0x5252,0x5152,0x5252,0x4141,0xAFAF,0x7171,0xD4D4,0x7171,0xD4D4,0x5151,0x6262,0x5151,0xD0D0,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x4141,0x4141,0x5050,0x9F9F,0x5050,0x7070,0xE0E0,0xBFBF,0x5151,0x4040,0x8F8F,0x5050,0x4040,0x8F8F,0x4040,0x8F8F,0x6060,0x6060,0x9F9F,0x8F8F,0x9F9F,0x4141,0x9F9F,0x6161,0x6161,0xC0C0,0xC0D4,0xC3C3,0xD3D0,0xD3D0,0xE0E0,0xAFAF,0xD0D0,0x8080,0xC0C0,0xD0D0,0xC0C0,0xAFAF,0xBFBF,0xE0E0,0xCFCF,0xD0D0,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0x6060,0xBFBF,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x4040,0x7F7F,0x4040,0x608F
.word 0x4040,0x6F6F,0x7F7F,0x4040,0x9F9F,0xBFBF,0xD0D0,0xBFBF,0x7070,0x7070,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x6060,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0xAEAE,0xCFCF,0xDEDE,0xE1E1,0xF1D1,0x61F1,0x7F7F,0x4040,0x8F8F,0xB2E2,0xB2B2,0xD1B2,0xD1E1,0xD1D1,0xD2C2,0xD2E2,0xB3B3,0xB3B3,0x6272,0x62C4,0x62C4,0x6262,0x62C4,0x6152,0xD461,0x62D4,0x62D4,0x72D4,0x6161,0x6161,0x7171,0x6262,0x7171,0x6161,0x6161,0xAFAF,0x6161,0x6262,0x4242,0x5241,0x6161,0x5251,0x5261,0x5151,0x4141,0x6161,0x6161,0x6161,0x6161,0xAFAF,0x6161,0x6161,0x9F9F,0x6161,0x6161,0xAFAF,0x4141,0x6152,0x5152,0x5152,0x4141,0xD470,0x7171,0xC3C3,0xC0C0,0x7171,0xD0D0,0x6161,0x4162,0x7070,0x7070,0x8080,0xC0C0,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xE0F0,0x61D0,0x9F9F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x5050,0x8F8F,0x4040,0x5050,0x9F9F,0x4141,0x5151,0x4141,0x7F7F,0x4040,0x5151,0x8F8F,0x7171,0xC0C0,0xC0C3,0x71D3,0xD0C3,0xD0D0,0xD0D0,0xD0D0,0xC0C0,0x71D0,0xC0D4,0xD0D4,0xD0D0,0xBFBF,0xE0E0,0xE0E0,0xD0D0,0xBFBF,0xBFBF,0xCFCF,0xDFDF,0xD0D0,0xC0C0,0xE3E3,0xF1F1,0x9F9F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x4141,0x4141,0xAF9F
.word 0x7F7F,0x4040,0x6F6F,0x7F7F,0xBFBF,0xE0E0,0xCFCF,0xC0C0,0xAFAF,0x9F9F,0x9F9F,0x6060,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xD0F0,0xF1D1,0x51E2,0x7F7F,0x6F6F,0x4040,0xB2D2,0xB2B2,0xD1B2,0xD1D1,0xC2C2,0xD2C2,0xC3E2,0xB372,0x8393,0x8383,0x6262,0x6262,0x6273,0x6252,0x6262,0xD4D4,0x7161,0x6161,0x6161,0x7171,0x7162,0x6262,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x4141,0x5141,0x5252,0x6152,0x5251,0x4141,0x4141,0xAFAF,0x6161,0x6161,0xAFAF,0x6060,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0x5151,0x4141,0x5251,0x5252,0x4242,0x4141,0xAFAF,0x7171,0x7171,0x6161,0xC0C0,0xAFAF,0x6241,0x4162,0xAFAF,0x8080,0x7070,0x8080,0x5151,0x4141,0x4141,0x5050,0x5050,0x5151,0x5151,0xAFAF,0xE0E0,0xAFAF,0x8F8F,0x4040,0x4040,0x5050,0x5050,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x8F8F,0x4141,0x7F7F,0x4040,0x8F8F,0x5050,0x5151,0xC371,0xC0C0,0xD071,0xD3C0,0xE4D3,0xE3D0,0xD0D0,0xD0D0,0x7171,0xC0C0,0xD0D0,0xD0D0,0xE0E0,0xCFCF,0xCFCF,0xC0C0,0xAFAF,0xAFAF,0xCFCF,0xD0D0,0xBFBF,0xC0C0,0x9F9F,0xF0F0,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x8F8F,0x9F9F,0x9F61
.word 0x4040,0x7F7F,0x7F7F,0x8F8F,0xBFBF,0xF0F0,0xD0D0,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xE0E0,0xF1E1,0x41D3,0x4040,0x6F6F,0x7F7F,0xB3D3,0xB3B3,0xD1C2,0xC2D2,0xC2C2,0xD2C2,0xB3D2,0x9372,0xC483,0x7362,0x6262,0x6263,0x6262,0x5262,0x5252,0x71C4,0x6262,0x61C4,0x61D4,0x6262,0xAFAF,0x6161,0x7171,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x4141,0x5252,0x6151,0x5251,0x5152,0x5151,0x8F8F,0x6161,0x9F9F,0x9F9F,0x7070,0x9F9F,0xAFAF,0x6161,0x6161,0x7070,0xAFAF,0xAFAF,0x7F7F,0x5241,0x5261,0x4151,0x7F7F,0x7171,0xAFAF,0x7171,0xAFAF,0x7171,0x5151,0x6241,0x6262,0xD0D0,0x7070,0x7070,0xAFAF,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xC0C0,0xBFBF,0x7070,0x4040,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x5050,0x8F8F,0x8F8F,0x5151,0x9F9F,0x8F8F,0x4040,0x5050,0x5050,0x8F8F,0x6161,0xC0C0,0xC0D4,0xD4C3,0xD3C0,0xE0E0,0xD0D3,0xE3E3,0xD0D0,0xC0D3,0xD0D4,0xD0D0,0xD4D4,0xE0E0,0xE0E0,0xC0D0,0xAFAF,0x9F9F,0xAFAF,0xE0E0,0xBFBF,0xC0C0,0xAFAF,0x5140,0xC0C0,0xD0D0,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x5252,0x5252,0x8F9F
.word 0x8F8F,0x4040,0x4040,0x8F8F,0xAFAF,0xD0D0,0xAFAF,0xC0C0,0x7070,0x6060,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x6060,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xE0E0,0xE1D1,0x41D3,0x6F6F,0x6F6F,0x4040,0x72D3,0xB3B3,0xD2C2,0xC2D2,0xB2C2,0xC2B2,0x72C3,0x83C4,0x8383,0x6283,0x6273,0x5252,0x5252,0x5262,0x6152,0x6262,0x61C4,0x6161,0x6262,0x6262,0x6261,0x6161,0x6161,0x6161,0x6161,0xAFAF,0x6161,0x9F9F,0x5151,0x4141,0x5152,0x5252,0x6161,0x6161,0x4141,0x8F8F,0xAFAF,0x6161,0x6161,0x6161,0x5151,0x5151,0xAFAF,0x9F9F,0xAFAF,0x6161,0x6060,0x4141,0x4141,0x5252,0x5152,0x4141,0xAFAF,0x6161,0xAFAF,0x7171,0xD4D4,0x7171,0x6241,0x5162,0xD0D0,0xBFBF,0x6060,0x7070,0x5151,0x8F8F,0x8F8F,0x5151,0x5151,0x5151,0xC0C0,0xAFAF,0x7070,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x5050,0x4040,0x5050,0x9F9F,0x9F9F,0x7070,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x4040,0x5050,0x5151,0x6161,0xD4C0,0x71C0,0xD0D0,0xD0D3,0xD0D3,0xE0D0,0xD0F3,0xC0C0,0xD4D0,0xD0D0,0xD0C0,0xF0E3,0xDFDF,0xD4D4,0xAFAF,0x8F8F,0xD0D0,0xC0C0,0xC0C0,0xAFAF,0xAFAF,0x8F8F,0x7171,0x8080,0x8F8F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x8F8F,0x8F8F,0x7F7F
.word 0x5151,0x8F8F,0x4040,0x8F8F,0x9F9F,0xAFAF,0x7171,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xD0D0,0xE3D1,0x41D0,0x7F7F,0x4040,0x7F7F,0x83D4,0xB393,0xD3C3,0xB2B2,0xB3B2,0xC3C3,0x62C3,0x8383,0x6273,0x7373,0x6252,0x6252,0x5262,0x5252,0x5252,0x62D4,0x6161,0x6262,0x6262,0x6161,0x9F9F,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x4141,0x5151,0x6151,0x5252,0x4252,0x4141,0x5151,0x6061,0x9F9F,0x6161,0x9F9F,0x9F9F,0x6060,0x7070,0x7070,0x7070,0xAFAF,0x9F9F,0x4141,0x4141,0x5151,0x4251,0x4040,0x7171,0xAFAF,0x6161,0x6161,0x6161,0x4141,0x6262,0x5162,0x5151,0xC0C0,0x6060,0x5151,0x9F9F,0x5151,0x6060,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xC0C0,0x4040,0x8F8F,0x7F7F,0x4040,0x7F7F,0x6060,0x9F9F,0x5050,0x8F8F,0xAFAF,0xE0E0,0xE0E0,0xE0E0,0xD0E0,0xC0C0,0xAFAF,0x6060,0x8F8F,0x8F8F,0x4040,0xC3C3,0xC071,0xC0D4,0xC3C3,0xC3D0,0xD3D0,0xD3D0,0xF3E0,0xD3E0,0xC0C0,0xD4D4,0xD0D0,0xE3E3,0xD3E0,0xC0C0,0xAFAF,0xD0D0,0xD0D0,0xBFBF,0x7171,0xAFAF,0x9F9F,0x5050,0xAFAF,0xC0C0,0x9F9F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4141,0x4141,0x8F8F,0x7F41
.word 0xAFAF,0x6161,0xAFAF,0x6060,0x7070,0x7070,0xAFAF,0x7070,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xD1F0,0xE0D1,0x41D4,0x6F6F,0x6F6F,0x5151,0x7271,0xB383,0xC3C3,0xB3B3,0xB3B3,0xD4B3,0x6272,0x6283,0x7362,0x6262,0x5262,0x6352,0x5252,0x5251,0x5252,0x6262,0x6261,0x6161,0x6161,0x6161,0x6161,0x5151,0x9F9F,0x6161,0x9F9F,0x6161,0x9F9F,0x6161,0x4141,0x4141,0x5241,0x5252,0x6161,0x5151,0x7F7F,0x9F9F,0x9F9F,0x6161,0x6060,0x9F9F,0x6060,0x6060,0x9F9F,0x9F9F,0x9F9F,0x7070,0x4141,0x7F7F,0x4141,0x5252,0x4152,0x7F7F,0x6060,0x6161,0x6161,0x6161,0xAFAF,0x7171,0x6251,0x5162,0xD3D3,0xC0C0,0xAFAF,0x6060,0x4141,0x8F8F,0x9F9F,0x9F9F,0x7070,0x5151,0x7070,0xAFAF,0x9F9F,0x7F7F,0x4141,0x7F7F,0x4040,0x4040,0x5050,0x6060,0xAFAF,0xE0E0,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xE0E0,0xD0D0,0xBFBF,0x5050,0x4040,0x5050,0x7171,0xD4C0,0xC0D0,0xC0C3,0xD0D3,0xC0C3,0xD0D3,0xD0D3,0xF0E3,0xE3E0,0xD0D0,0xD3D0,0xE3D0,0xD0D0,0xC0C0,0xD0D0,0xE1E1,0xD0D0,0xC0C0,0xBFBF,0x6060,0x9F9F,0x8F8F,0xC0C0,0xAFAF,0x4070,0x7F7F,0x4040,0x6F6F,0x4040,0x6F6F,0x6F6F,0x4040,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F
.word 0xAFAF,0xAFAF,0xD4D4,0xC0C0,0xAFAF,0x7171,0x7070,0x7070,0x7070,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBEBE,0xCFCF,0xE0F0,0xE1D1,0x41D0,0x4040,0x4040,0xC3C3,0xC472,0xD472,0x72C3,0xB3B3,0xC472,0xC372,0x6262,0x6262,0x5263,0x5262,0x5252,0x5252,0x5252,0x4242,0x6262,0x6261,0x9F9F,0x5252,0x6161,0x6262,0x6161,0x6161,0x6161,0xAFAF,0x6161,0x9F9F,0x6161,0x6161,0x4141,0x4141,0x5142,0x5151,0x5251,0x6161,0x4141,0x7070,0x5151,0x9F9F,0x6060,0x6060,0x9F9F,0xAFAF,0x6060,0x6060,0x6060,0x9F9F,0x4040,0x4141,0x5241,0x5151,0x5252,0x4040,0x6161,0x9F9F,0x9F9F,0x9F9F,0x6161,0x4141,0x5252,0x6262,0x4141,0xC0C0,0x7070,0x6060,0x9F9F,0x9F9F,0x9F9F,0x5050,0x8F8F,0x9F9F,0x5151,0x9F9F,0x5151,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0x5050,0xF0F0,0xE0E0,0xCFCF,0xD0D0,0xC0C0,0xC0C0,0xD0D0,0xBFBF,0xD0D0,0xD0D0,0x7070,0x8F8F,0x8F8F,0x5151,0xC0C3,0xD4D0,0xD3C0,0xC3D0,0xC3D0,0xD3C0,0xD3D0,0xE0D0,0xE0E3,0xE3E3,0xD0D0,0xD0D0,0x71D3,0xAFAF,0xE0E0,0xE0E0,0x7171,0xD0D0,0xBFBF,0x9F9F,0x8F8F,0x4040,0x7171,0x7171,0xAFAF,0x6F6F,0x7F7F,0x4040,0x6F6F,0x4040,0x7F7F,0x6F6F,0x4040,0x4040,0x4141,0x4141,0x7F7F,0x417F
.word 0x6161,0xAFAF,0xE4E4,0xD3D3,0xC0C0,0x7070,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBEBE,0xBEBE,0xBEBE,0xCFCF,0xEEEE,0xD1F0,0xE2D1,0x51D3,0x7F7F,0x8F8F,0x7171,0x71D4,0xC372,0xC3C3,0xC372,0x7172,0x71D4,0x5252,0x5262,0x5262,0x5252,0x5252,0x5252,0x5152,0x4142,0x9F9F,0x6262,0x6162,0x6161,0x6161,0x6161,0x6161,0x9F9F,0xAFAF,0x6161,0x9F9F,0x6161,0x6161,0xAFAF,0x6161,0x4141,0x5151,0x5252,0x5152,0x4141,0x5151,0x9F9F,0x6161,0x6060,0x9F9F,0x5151,0x5151,0x6060,0x9F9F,0xAFAF,0x6060,0x7070,0x9F9F,0x4141,0x4141,0x5252,0x4151,0x7F7F,0x9F9F,0x6161,0x6161,0x6161,0x6161,0x5151,0x6161,0x5162,0x7171,0xD3D3,0x7070,0x9F9F,0x5050,0x5050,0x9F9F,0x5050,0x4040,0x4141,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0xBFBF,0xD0F3,0xD0D0,0xC0C0,0xD3D3,0xC0C0,0xC0C0,0xC0C0,0xD0D0,0xE0E0,0xCFCF,0x4150,0x8F8F,0xC060,0xC3C3,0xD0D0,0xD371,0xD4C0,0xD3C0,0xC0C3,0xD0C3,0xD0D3,0xE3D0,0xE0E0,0xD0E3,0xE0E4,0xD0D0,0x9F9F,0xC0C0,0xD0D3,0xAFAF,0x7171,0x7070,0x8F8F,0x4040,0x8F8F,0x7070,0x7070,0x7171,0x5050,0x4040,0x6F6F,0x7F7F,0x6F6F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F
.word 0x6161,0xAFAF,0xBFBF,0xE4E4,0xD3D3,0xAFAF,0x7060,0x7070,0x9F9F,0x6060,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xD0AE,0xCECE,0xDFDF,0xE1F0,0xE1D1,0xE2E2,0x5141,0x6161,0xC3C3,0xD471,0xC371,0x72C3,0x72C3,0x72D4,0xD472,0x5262,0x5262,0x5262,0x5262,0x5252,0x5252,0x5252,0x4141,0x6262,0x6261,0x9F9F,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x4141,0x4141,0x4242,0x5251,0x5251,0x4141,0x7F7F,0x6060,0x9F9F,0xAFAF,0x7070,0x9F9F,0x9F9F,0x6060,0x7070,0x7070,0x9F9F,0x9F9F,0x4040,0x4141,0x5151,0x5251,0x4141,0x6F6F,0x7070,0x9F9F,0x6060,0x9F9F,0x6161,0xAFAF,0x5252,0x5262,0x8F8F,0xD0D0,0xC0C0,0x6060,0x4040,0x8F8F,0x5050,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x5151,0xAFAF,0x7171,0x7171,0x8080,0xD4D4,0xD3D0,0xC3C3,0xC0C0,0xC0C0,0xD0E0,0xBFBF,0x5151,0xAFAF,0xC0C3,0xD3D3,0xC0D3,0xD0D3,0xC0D3,0xD3D0,0xC3D0,0xC3D0,0xD0D3,0xE0D3,0xE3E0,0xF2F0,0xF2F2,0x6161,0xE4E4,0xC0C0,0xC071,0xAFAF,0x9F9F,0x8F8F,0x4040,0x7F7F,0xC0C0,0xAFAF,0x7171,0xE3E3,0x8F8F,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4141,0x8F8F,0x4141,0x7F41
.word 0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0x71D4,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x5050,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xBFBF,0xCECE,0xCFCF,0xCFCF,0xDEDE,0xFEFE,0xE1E1,0xD1D1,0xD3E0,0x6161,0x6161,0x7171,0x71C3,0xD3C3,0x72D3,0x72C3,0x71C3,0xC3D4,0x6262,0x6252,0x6262,0x5252,0x5252,0x5252,0x5252,0x5252,0x6262,0x6262,0x6161,0x5252,0x6262,0x6161,0x6161,0x6161,0x9F9F,0x6161,0x6161,0x6161,0x9F9F,0x6161,0x9F9F,0x4141,0x5151,0x5152,0x4151,0x8F8F,0x4141,0x9F9F,0x6051,0x6161,0x9F9F,0x7070,0x6060,0x6060,0x9F9F,0x9F9F,0x9F9F,0x7070,0x5151,0x4141,0x5252,0x5152,0x4141,0x4040,0xAFAF,0x6161,0x9F9F,0x7070,0x9F9F,0x4060,0x5151,0x5152,0x6161,0xD0D0,0xD0D0,0xBFBF,0x4040,0x4141,0x8F8F,0x9F9F,0xBFBF,0xD0D0,0xAFAF,0xAFAF,0x9F9F,0x4141,0x7F7F,0x4141,0x4040,0x4040,0x4040,0x8F8F,0x6051,0x6060,0x9F9F,0x8080,0xD0D0,0xE4E4,0xD0D0,0xE0E0,0xE4E4,0xD0D0,0xC0C0,0x6060,0xC3C3,0xC071,0xC0D3,0x71D0,0xD0C3,0xD0D4,0xC3D3,0xD3D0,0xD3C0,0xD0C0,0xD0D3,0xE0E3,0xF0F0,0xD0D0,0xD0D0,0xC0E3,0x7171,0xC3C3,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6161,0x8080,0xD0D0,0xF0F0,0x9F9F,0x4040,0x6F6F,0x4141,0xF3F3,0xDFDF,0x7F7F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F
.word 0x6161,0xAFAF,0x7171,0x7171,0xAFAF,0xAFAF,0x6060,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x9E9E,0xBFBF,0xBFBF,0xBFBF,0xBEBE,0xCFCF,0xDFDF,0xE1F0,0xD1C1,0xD0E3,0x6161,0xC0C0,0xC0C3,0xC0D4,0xC0D4,0xC3C3,0xC371,0xC372,0xD471,0x5261,0xC462,0x6252,0x6252,0x6262,0x6252,0x5152,0x4242,0x7171,0x6162,0xAFAF,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x9F9F,0x6161,0x7070,0x6161,0x4151,0x4141,0x5152,0x5151,0x4141,0x4141,0x8F8F,0x7070,0x9F9F,0x5151,0x6060,0x9F9F,0x9F9F,0xAFAF,0x6060,0x6060,0x6060,0xAFAF,0x5050,0x4141,0x5151,0x5251,0x4141,0x6F6F,0x6060,0x9F9F,0x6060,0x7070,0x9F9F,0x9F9F,0x5241,0x5151,0x7F7F,0xD0D0,0xD0D0,0xF3F3,0x8F8F,0x4141,0x8F8F,0xE4E4,0xF3F3,0xFFFF,0xF1F1,0xF2F0,0xE0F0,0xCFCF,0x6161,0x8F8F,0x7F7F,0x8F8F,0x4040,0x4040,0x8F8F,0x9F9F,0xD061,0xF0F0,0xEFEF,0xF0F0,0xF3F0,0xF2F0,0xF0F0,0xF0F3,0xE0E0,0xD0D0,0x7070,0xD4C0,0xD3C0,0xC0C3,0xC0D3,0xD3D0,0xD0C0,0xC0D3,0xD0D3,0xD3C3,0xD3D0,0xD3D0,0xE3E0,0xE4D0,0xE4D0,0xC0C0,0x71D4,0x6060,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0xD0D0,0x7171,0x7171,0xE0E0,0x4040,0x6F6F,0x7F7F,0x4040,0xF3F3,0xDFDF,0x4040,0x7F7F,0x4141,0x4141,0x7F7F,0x7F41
.word 0xAFAF,0xAFAF,0x6161,0xAFAF,0x6161,0x6161,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x6060,0x8E8E,0xBEBE,0xBEBE,0xBFBF,0xCECE,0xFEFE,0xE0E0,0xD1D1,0xD4E0,0x7171,0xC0C0,0xD4C0,0xD471,0xD4D0,0xC3D0,0x71C3,0x72C3,0xC3C3,0x6262,0x6262,0x6262,0x6262,0x6252,0x5262,0x6262,0x5151,0x61C4,0x6262,0x6162,0x6262,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x9F9F,0x6161,0x9F9F,0x4141,0x5251,0x5251,0x5151,0x4141,0x5151,0x7070,0x9F9F,0x6060,0x9F9F,0x6060,0x6060,0x9F9F,0xAFAF,0x7070,0x7070,0x7070,0x8F8F,0x5141,0x5252,0x5151,0x4141,0x4040,0x9F9F,0x6060,0x9F9F,0x6060,0x7070,0x8F8F,0x5151,0x5152,0x7F7F,0xBFBF,0xE0E0,0xF0F0,0x6060,0x9F9F,0xE4E4,0xDFDF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF0F0,0x6161,0x4040,0x8F8F,0x7F7F,0x8F8F,0x5050,0xD0D0,0xF4F4,0xFFFF,0xF0F1,0xF3F0,0xE0E0,0xE0E3,0xE0E3,0xE3E0,0xF0F0,0xE3E3,0xC0C0,0xD371,0xC0C0,0x71D3,0xC3D0,0xD3D0,0xD0C3,0xD3D3,0xD3E0,0xC0D0,0xD0D3,0xD0D3,0xD0D3,0xD0D4,0xD0D0,0xD3D3,0xD4C0,0x5151,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x6161,0xC0C0,0x7171,0xD0D0,0x8F8F,0x7F7F,0x6F6F,0xE351,0xEFEF,0x4040,0x7F7F,0x4141,0x7F7F,0x8F8F,0x7F7F,0x7F7F
.word 0x9F9F,0xC3C3,0xAFAF,0x5151,0x8F8F,0x6060,0x5050,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xF0F0,0xE0F0,0xD381,0x7070,0x8171,0xD4C0,0xC0C0,0xC0D0,0xD0D0,0xD0D3,0xD3C3,0xD471,0xC371,0xC4C4,0x6262,0x6273,0x52C4,0x52C4,0x6262,0x5252,0x6252,0x7272,0x6262,0x6262,0x6262,0x6161,0x6161,0x6161,0x6161,0x6161,0x9F9F,0x6161,0x6161,0x6161,0x9F9F,0x4141,0x4141,0x5151,0x5151,0x4152,0x4141,0x6060,0x9F9F,0x7070,0x9F9F,0x7070,0x9F9F,0x6060,0x7070,0x7070,0x9F9F,0xAFAF,0xAFAF,0x4141,0x5151,0x5252,0x5251,0x4041,0x6F6F,0x9F9F,0x6060,0x6060,0xAFAF,0x9F9F,0x5050,0x5151,0x5152,0x4040,0xD0D0,0xCFCF,0xE0F0,0xAFAF,0xF3F3,0xF2F2,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0x4040,0x4040,0x4141,0xAFAF,0xF1F1,0xF5F5,0xF0F0,0xE0F3,0xD0D0,0xD4C0,0xD4C0,0xD4C0,0xD0D0,0xD0D3,0xE0E0,0xE4E0,0x7171,0xD471,0xD3C0,0xC072,0xD0C3,0xC3D0,0xF1E0,0xF0F0,0xD3D3,0xD3D0,0xD3D0,0xD0D0,0xD0D0,0xE3E3,0xC0E4,0xD0D4,0x7171,0x7F7F,0x8F8F,0x4040,0x4040,0x4040,0xD0D0,0x7171,0xC0C0,0xBFBF,0x4040,0x4040,0x4040,0xD0D0,0xC0C0,0x9F9F,0x9F9F,0x7F7F,0x4141,0x4141,0x4040,0x7F7F
.word 0x7070,0xAFAF,0xAFAF,0x6060,0x5050,0x5050,0x5050,0x9F9F,0x6060,0x9F9F,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xE0E0,0x6161,0x4040,0x7171,0xD081,0xD0D0,0xD3D3,0xD471,0xD481,0xD4D0,0xC0C3,0xC3C3,0xD4C0,0x6271,0x6262,0x62C4,0x6252,0x6262,0x6262,0x5252,0x5151,0x72C4,0x7171,0x6262,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0xAFAF,0x6161,0x7070,0x6161,0x4141,0x4242,0x5251,0x5251,0x4151,0x4141,0x9F9F,0x6060,0x6161,0x9F9F,0x6060,0xAFAF,0xAFAF,0x7070,0x9F9F,0x7070,0x7070,0x8080,0x4040,0x4241,0x5151,0x5252,0x4141,0x6F6F,0x6060,0x9F9F,0x9F9F,0x6060,0x6060,0x8F8F,0x6141,0x6161,0x4141,0xBFBF,0xE0E0,0xF0F0,0xE0E0,0xFFFF,0xFFFF,0xFFFF,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xFFFF,0xF3F3,0xE4E4,0x4141,0x8F8F,0x8F8F,0xF2F2,0xFFFF,0xFFFF,0xF3F3,0xD3D0,0xD3C1,0xD0D4,0xC0D0,0xC0D3,0xD4C0,0xD0D0,0xD0D3,0xE0E3,0xC0D3,0x71D3,0x71D0,0xD3C0,0xD0C3,0xD0C3,0xF2E3,0xE2F1,0xE0E0,0xD0E3,0xD0D3,0xD3D0,0xE4D3,0xD0D3,0xC0C0,0xD0D4,0xD4D4,0x4040,0x4040,0x5050,0x8F8F,0x8F8F,0xD0D0,0xD0D0,0xC0C0,0xD0D0,0x9F9F,0x5050,0x8F8F,0xD0D0,0xD0D0,0xF1E0,0xF0F0,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F
.word 0x9F9F,0x6060,0x6161,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x9F9F,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0xAFAF,0x9E9E,0xBFBF,0xBFBF,0x6060,0x8F8F,0x6060,0xB181,0xD0D0,0xC0E0,0xC281,0x71C0,0xC0C3,0xC0C0,0xD4C0,0xC3C0,0xC0D4,0x6262,0x6262,0x6262,0x6262,0x6262,0x6262,0x6262,0xD452,0x6171,0x6262,0x6262,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0xAFAF,0x6161,0x7070,0x4141,0x5141,0x5251,0x5151,0x4141,0x8F8F,0x7070,0x9F9F,0x6060,0x7070,0x9F9F,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x5050,0x4141,0x5151,0x5151,0x4141,0x5050,0xAFAF,0x6060,0x7070,0xAFAF,0x9F9F,0x5050,0x5151,0x6161,0x4040,0xE0E0,0xCFCF,0xF0F0,0xF1F1,0xFFFF,0xFFFF,0xF5F5,0xF5F5,0xBFBF,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xF3F3,0xBFBF,0xCFCF,0xAFAF,0xE0D4,0xE3E3,0xF3F3,0xF2F2,0xF1F2,0xD0F2,0xD0D3,0xE0E0,0xE0E0,0xD4D4,0xD4C0,0xD0D0,0xD0D4,0xD0D4,0xD3D0,0xD4E0,0xD071,0x71D4,0x71D4,0xD3D0,0xD3D0,0xD3D0,0xD3D0,0xE0D1,0xE3E0,0xD3E0,0xD0D4,0xD0D0,0xD4D3,0xC3D0,0xD4D0,0x6161,0x9F9F,0x9F9F,0xAFAF,0xC0C0,0xE0E0,0xE0F3,0xD0E0,0xE0D4,0xF0F2,0xF0F0,0xF0F0,0xF2F0,0xF0F0,0xF0F0,0xF2F2,0xF3F3,0xAFAF,0x4141,0x7F7F,0x7F7F,0x7F41
.word 0x4040,0x8F8F,0x5050,0x5050,0x6060,0x6060,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0x7070,0x8F8F,0x4040,0x8161,0xD3B1,0xDFDF,0xB3C0,0x82B1,0xC3B2,0xD381,0xE4D0,0xC0C3,0xC0D4,0xD4C3,0x62C0,0x6262,0x6262,0x62C4,0x6273,0x6262,0x5252,0x7171,0x71D4,0x61D4,0x7272,0x6161,0x7171,0x6161,0x6161,0x6161,0x6161,0x6161,0x9F9F,0x6161,0x6161,0x6161,0x8F8F,0x5141,0x5152,0x5252,0x4151,0x5151,0xAFAF,0x6161,0xAFAF,0xAFAF,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0xBFBF,0xC0C0,0x4040,0x4141,0x5151,0x4151,0x8F8F,0x7F7F,0x6060,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x4040,0x6261,0x5151,0x9F9F,0xD0D0,0xF0F0,0xFFFF,0xFFFF,0xFFFF,0xF5F5,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xF5F6,0xF5F5,0xF0F0,0xDFDF,0xD4D4,0xE3E3,0xDFDF,0xE0F0,0xF3E3,0xFFFF,0xE3E0,0xD3D0,0xD3D3,0xE3E3,0xD0D3,0xC3C0,0xC0C0,0xD0D4,0xD3C0,0xD3D0,0xD0D0,0xD0D0,0x71D3,0xC3C0,0xD071,0xC0C3,0xD0D3,0xC0C3,0xC0C3,0xC2C3,0xD3D0,0xD0D0,0xD3D0,0xD0D3,0xC3D0,0xD4D3,0xD0D0,0xC3C3,0xF3F4,0xD0D0,0xE0E0,0xF0F0,0xDFDF,0xF0F0,0xE0F3,0xF0E0,0xEFEF,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xF2F3,0xEFEF,0xAFAF,0x6161,0x4141,0x7F8F
.word 0x7F7F,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x8F8F,0x5050,0x5050,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x5050,0x8F8F,0x7F7F,0x6060,0xB281,0xD0D0,0xC0C0,0xA1B2,0xB2B2,0x81B2,0xD0C3,0xE0E0,0xC3D0,0xC3C0,0xD0D0,0x61C4,0x62C4,0x62C4,0x6262,0x6262,0x62C4,0x5162,0x6262,0x61D4,0x6161,0x6171,0x7171,0x7171,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0xAFAF,0x6161,0x4141,0x4141,0x5151,0x5152,0x4141,0x4141,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xC0C0,0xBFBF,0x4141,0x5151,0x5152,0x5151,0x4141,0x4040,0xAFAF,0x7070,0x7070,0x6060,0xAFAF,0x9F9F,0x4141,0x5151,0x5151,0x4040,0xE0E0,0xDFDF,0xF1F1,0xFFFF,0xFFFF,0xF5F5,0xD5D5,0xAFAF,0xAFAF,0x9F9F,0xF5F5,0xCFCF,0xEFEF,0xE0E0,0xBFBF,0xE4E4,0xF3F4,0xD3D3,0xE0E0,0xE0F3,0xD0E4,0xF0E4,0xF1F1,0xF3F0,0xD3D3,0xD3E4,0xD4C0,0xD4D0,0xC3C3,0xD0D0,0xC0D4,0xD3D4,0xD0D0,0x80D4,0x72C3,0xC3C0,0xD3D0,0xC3D0,0xC3D0,0xC1C0,0xC0C3,0xD0D3,0xD0E4,0xD4D0,0xC0D4,0xC0C3,0xC3D4,0xD061,0xF3F3,0xFFFF,0xDFDF,0xE0E0,0xE0E0,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xE0E0,0xE3E0,0xE0E0,0xE0E0,0xE0E0,0xE0E0,0xF0F0,0xF2F1,0xF0F0,0xF0F0,0xF3F3,0x9FF3
.word 0x4040,0x7F7F,0x7F7F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x8F8F,0x5050,0x9F9F,0x8F8F,0x5050,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x4040,0x4040,0x6F6F,0x7150,0xC0B3,0xCED0,0xB1D0,0xB2A1,0xA1B2,0xB2B2,0xD2B2,0xE0E0,0xD0D3,0xD3D3,0xD4D0,0x6271,0x7262,0x6262,0x6262,0x62C4,0x6262,0x5262,0xC3C3,0x7261,0x7171,0x6262,0x7171,0x6161,0x6161,0x6161,0x6161,0x7171,0x6161,0x7070,0x7070,0x6161,0x9F9F,0x4141,0x5241,0x5251,0x5251,0x4141,0x4141,0xAFAF,0x6161,0x6060,0x9F9F,0x7070,0x7070,0x8080,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x8F8F,0x5141,0x5151,0x5252,0x4141,0x8F8F,0x6060,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x7F7F,0x4141,0x8F8F,0x7F7F,0xCFCF,0xF0F0,0xF5F0,0xFFFF,0xFFFF,0xFFFF,0xF3F3,0xAFAF,0xC0C0,0x6161,0xAFAF,0xBFBF,0xCFCF,0xBFBF,0xCFCF,0xE4E4,0xF4F4,0xD0E3,0xD3D3,0xE4E4,0xD3D0,0xF1E0,0xFFFF,0xFFFF,0xD3D0,0xD0D0,0xD3D3,0xD4D4,0xC3C0,0xD3D0,0xD4D0,0xC0D0,0xD4D0,0xD4C0,0x7171,0xD0C3,0x71D3,0xC3C0,0xC3D0,0xD3C1,0xD2C1,0xF3F2,0xE0E0,0xD0E4,0xD4C0,0xD4C0,0x71D3,0xD4D4,0xF3F3,0xF0F0,0xE0E0,0xD0D0,0xF0E0,0xF0F0,0xF0F0,0xE0E0,0xD0E3,0xD0E4,0xD0D4,0xD3D0,0xD0D0,0xE0E4,0xE0F4,0xE0F0,0xD0E3,0xBFBF,0xD0D0,0xF0F3,0x9FDF
.word 0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x9F9F,0x8F8F,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4040,0x7F7F,0x7F7F,0x4040,0xB171,0xD0C0,0xBEBE,0xB2C0,0xB2B2,0xB2A1,0xB2B2,0xD1B1,0xF0F0,0xD3D0,0xD0D0,0xD0E4,0x6262,0xC472,0x8372,0x6262,0x6262,0x6262,0x5252,0x7172,0x61D4,0x7171,0x7171,0x6262,0x7171,0x6161,0x7171,0x6161,0xAFAF,0x7171,0x6161,0x9F9F,0x6161,0x4170,0x4141,0x5151,0x5152,0x5151,0x9F9F,0x8F8F,0x7070,0x7070,0x9F9F,0x6060,0x9F9F,0xAFAF,0x7070,0xAFAF,0xAFAF,0xBFBF,0xD0D0,0xAFAF,0x4140,0x5151,0x5151,0x4151,0x7F7F,0x5050,0x7070,0x6060,0x6060,0x9F9F,0x7070,0x4060,0x5140,0x5151,0x4151,0x7F7F,0xD0D0,0xF0F0,0xF3F3,0xFFFF,0xFFFF,0xFFFF,0xF4F4,0xE3E4,0xE3E3,0xAFAF,0x6262,0xAFAF,0xAFAF,0x9F9F,0xF3F3,0xF4F4,0xF5F5,0xEFEF,0xF4F4,0xE3D0,0xD3D3,0xF4D0,0xF0F0,0xF0F0,0xE4E4,0xD4D0,0xE3E3,0xC0D3,0xD4D4,0xC3C0,0xC0D0,0x71D3,0xC0D4,0xC0C0,0xD371,0xD4C0,0xD371,0xD0C3,0xC0C3,0xC0C3,0xF0D3,0xFFFF,0xF0F2,0xF0F0,0xD0F4,0xD0D4,0x7171,0xD0C3,0xF0F0,0xE0E0,0xD0D0,0xD0D0,0xF0F3,0xF0F0,0xE3F0,0xE4D0,0xE3D0,0xE3D0,0xD0D0,0xD0E4,0xD0E3,0xE3E3,0xF0F0,0xD0E3,0x7171,0x71C0,0xC0C0,0xD0D0,0x8F9F
.word 0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x9F9F,0x8F8F,0x5050,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x7040,0x8181,0xC3C3,0xC0C0,0xB281,0xB2A1,0xA1B2,0x92B2,0xD0B2,0xE0E0,0xD2D0,0xD2D2,0xD3E0,0x7271,0x7272,0x7283,0x62C4,0x62C4,0x62C4,0x6262,0xD4D4,0x6161,0x7172,0x6262,0x7171,0x6161,0x7171,0x6161,0xAFAF,0x7171,0x6161,0x6161,0x6161,0xAFAF,0x4141,0x4141,0x5152,0x5261,0x5252,0x4151,0x8F8F,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x8080,0x8080,0xC0C0,0xBFBF,0x7070,0x4141,0x6161,0x5151,0x4151,0x7F7F,0x9F9F,0x9F9F,0x7070,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x5050,0x4040,0x4040,0x6040,0x9F9F,0xF0F0,0xE0E0,0xF1F1,0xFFFF,0xFFFF,0xFFFF,0xF0F0,0xF0F0,0xE0E0,0x6161,0xD0D0,0x5151,0x8F8F,0xBFBF,0xEFEF,0xFFFF,0xFFFF,0xF3F3,0xD0E4,0xE3E3,0xF3F3,0xF3F3,0xE0E0,0xC3D0,0xD0D0,0xE0E0,0xCFCF,0xC3C0,0xC0C0,0xD0D4,0xD3D0,0xC3C0,0xD4C0,0x71C0,0xC071,0xC071,0xC0C3,0x71D3,0xC0C3,0xE2C2,0xE1F1,0xE0E2,0xE0E2,0xF0E2,0xF4F0,0xD3D3,0xF3D3,0xF0F0,0xD0D0,0xD0D0,0xF0D1,0xF0F0,0xE0F1,0xE0E0,0xE0F3,0xF0F0,0xF0F2,0xF2F2,0xF2F0,0xF2F0,0xF1F0,0xE0F1,0xD4D0,0xC3D0,0xC0C0,0x7171,0x5151,0x7F41
.word 0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x5050,0x8F8F,0x9F9F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x4040,0x7171,0xC081,0xBED0,0xB1D0,0xB2B1,0xB2A1,0xB2A1,0xA1B2,0xE0C2,0xF3F3,0xD1E0,0xE3D0,0xD3E0,0xC372,0xC472,0x8372,0x7272,0x7262,0x6262,0x6261,0x7171,0xD471,0x7171,0x7171,0x7171,0x7171,0x7171,0x6161,0x6171,0x6161,0x7171,0x6161,0x7070,0x7070,0x6161,0x4141,0x5252,0x5151,0x5162,0x4141,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xC0C0,0x5050,0x4141,0x5151,0x5251,0x4141,0x4040,0x9F9F,0x7070,0x7070,0x6060,0xAFAF,0x6060,0xC0C0,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x7070,0xC0C0,0xD0D0,0xF3F3,0xFFFF,0xEFEF,0xFFFF,0xF0F0,0xE0E0,0xCFCF,0xE0E0,0x7070,0xAFAF,0x8F8F,0x9F9F,0xEFEF,0xF5F5,0xFFFF,0xFFFF,0xF4F4,0xF0F0,0xD0E3,0xD0E4,0xD3D3,0xC0C0,0xD3D0,0xE3E0,0xE0F0,0xD0D0,0xD471,0xC3C0,0xC0D3,0xC0D4,0xC0D4,0xC0D4,0xD4C3,0xC371,0xC371,0xD0D3,0xC381,0xC3C0,0xD2D0,0xD2C1,0xD3D1,0xD3D0,0xF0F0,0xC3D0,0xE0E0,0xF0F0,0xE0E0,0xF1D3,0xF0F0,0xF0F2,0xE2F0,0xE1E2,0xE1E1,0xE1E2,0xF2F0,0xF2F0,0xE1F0,0xE0F2,0xE0E2,0xD0D3,0xD0C3,0x6161,0x4141,0x4141,0x7F7F,0x7F7F
.word 0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x4040,0x7F7F,0x6F6F,0x4040,0x4040,0x4040,0x8171,0xD0B2,0xCECE,0xB2C0,0xA1B2,0xA1B2,0xB2B2,0xB2A1,0xF0C1,0xE0E0,0xD1E0,0xF0E2,0xC3D1,0x72B3,0x72B3,0xC472,0x8362,0x8372,0xC472,0x6162,0x71D4,0x61D4,0x7272,0x7171,0x7171,0x7171,0x7171,0x7171,0xAFAF,0x7171,0x7070,0xAFAF,0x6161,0x6161,0x4141,0x4141,0x6152,0x6262,0x5151,0x4141,0x6060,0x6060,0x7070,0x6060,0x7070,0x7070,0xAFAF,0x8080,0x8080,0x8080,0xD0D0,0xBFBF,0x4040,0x5151,0x5151,0x5151,0x4041,0x7F7F,0x6060,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x6F6F,0x6F6F,0x7F7F,0x6060,0x7070,0xAFAF,0xD0D0,0xF1F1,0xFFFF,0xCFCF,0xFFFF,0xFFFF,0xEFEF,0xF0F0,0xF0F0,0xBFBF,0x6060,0x5151,0xAFAF,0xFFFF,0xF5F5,0xF5F5,0xFFFF,0xFFFF,0xE3E3,0xE3E3,0xD0D0,0xC3C3,0xE3D0,0xF0F0,0xF0F0,0xF0F0,0xE0F0,0xD0D4,0xD371,0xD3C0,0xD0C0,0x7171,0xD4C0,0x7171,0x71C0,0xC0D3,0xC3C0,0xC0C3,0xC0C3,0xD3C2,0xD1D1,0xD1D0,0xD2D0,0xE0E0,0xD3D3,0xF0F0,0xF0F0,0xE0F0,0xF0F0,0xF2F0,0xF1F0,0xE1E1,0xD1E1,0xC1D1,0xD0D2,0xE0E3,0xE0E0,0xE0E3,0xE3E0,0xC3D0,0xC3C0,0xC0C0,0x8F8F,0x7F7F,0x5151,0xBFBF,0x7F40
.word 0x4040,0x4040,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x5050,0x8F8F,0x5050,0x4050,0x7F7F,0x6F6F,0x4040,0x6F6F,0x4040,0x7070,0xB1B1,0xC1C1,0xC0C0,0xB2A1,0xB2A1,0xA1B2,0xA1B2,0xA1B2,0xF0C1,0xF0F0,0xE2E0,0xF2E0,0x81E2,0xB3B3,0xB393,0x7272,0xC472,0x8372,0x6262,0x6161,0xD471,0x6161,0x7171,0x7171,0x7171,0x7171,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0xAFAF,0x7070,0x4141,0x5141,0x5151,0x5162,0x5252,0x8F8F,0x9F9F,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0x7070,0x5050,0x5151,0x5151,0x5151,0x8F8F,0x6F6F,0xAFAF,0x7070,0xC0C0,0x6060,0xAFAF,0x6060,0xC0C0,0x4040,0x4040,0x4040,0x6F6F,0xAFAF,0xD0D0,0xD0D0,0xD0D0,0xF1F1,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x7070,0xAFAF,0x9F9F,0xBFBF,0xEFEF,0xEFEF,0xEFEF,0xF5F5,0xEFEF,0xEFEF,0xF3F3,0xD0F3,0xE0D0,0xF3F3,0xE0E0,0xE0E0,0xF0F4,0xF3F0,0xD0E0,0xC0D4,0xC0D3,0x71D4,0xC0D3,0x7171,0x71D0,0xC0D4,0x71C3,0xC0D3,0xC371,0xC381,0xC3C0,0xE3D0,0xD2D2,0xD0D2,0xE3D2,0xE3D0,0xF0F0,0xF0F0,0xF2F2,0xF0F0,0xF1F1,0xE1F1,0xD1D1,0xD0D1,0xD0D3,0xD0D3,0xD0D3,0xD3D3,0xD3C0,0xD3D0,0xC3C0,0xC3C0,0x4141,0x4141,0x4141,0xF2F2,0xF3F3,0x7F7F
.word 0x8F8F,0x4040,0x5050,0x4040,0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050,0x8F8F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7150,0xB181,0xCED0,0xB1D0,0xA1B1,0xA1B2,0xB2A1,0xB2A1,0xB1B2,0xF0C1,0xE0E0,0xE1D1,0xF0F0,0xB3E2,0xB382,0xB3B3,0xC4B3,0x7272,0x7283,0x62C4,0xC462,0x7171,0x7172,0x7171,0x6262,0x7171,0x7171,0x7171,0x7171,0x7171,0x7171,0x6161,0xAFAF,0x7070,0x7070,0x9F9F,0x5151,0x5152,0x6151,0x4151,0x4141,0xAFAF,0x7070,0x7070,0x8080,0xAFAF,0xBFBF,0xBFBF,0x7070,0x7070,0x8080,0xC0C0,0xAFAF,0x5050,0x5160,0x4141,0x5141,0x4141,0x4040,0x7070,0xAFAF,0xAFAF,0xBFBF,0x6060,0xAFAF,0x5050,0x7F7F,0x6F6F,0x6F6F,0x6060,0x7070,0xD0D0,0xBFBF,0xD0E3,0xF0E3,0xFFFF,0xFFFF,0xFFFF,0xF1F1,0xF0F0,0xE0E0,0xBFBF,0x7070,0x9F9F,0xCFCF,0xDFDF,0xBFBF,0xDFDF,0xDFDF,0xEFEF,0xFFFF,0xCFCF,0xBFBF,0xDFDF,0xE0E0,0xD0D0,0xD3D3,0xD3D0,0xD0D0,0xE3E0,0xD0D0,0xD0C3,0xD0D0,0xD471,0x7171,0x71D4,0xC371,0xC3C0,0xD3C0,0xC3C0,0xC3C0,0xC371,0xD3D0,0xC1D0,0xD2D3,0xD3D0,0xF0D0,0xF0F0,0xF0F0,0xF0F0,0xF2F2,0xF1F1,0xE1E1,0xD0D2,0xC1C3,0xC3C0,0xC3C0,0xC3C0,0xD3D0,0xC3C0,0xC3C0,0xC0D3,0xC0C3,0x6161,0x8F8F,0x6161,0xE0D3,0x6161,0x7F8F
.word 0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0xB161,0xC0B2,0xCFCF,0xB2D0,0xB2A1,0xB1A1,0xB2B2,0xB2A1,0xB2A1,0xF0C1,0xE0F0,0xE0E1,0xF3F0,0x82D0,0xB3B3,0xB393,0x9393,0x72B3,0x72C4,0x7272,0x7161,0xD4C3,0xD471,0xD471,0x7171,0x7272,0x6161,0x7171,0x7171,0x7171,0x6161,0xAFAF,0x7070,0x6161,0x6161,0x4141,0x4141,0x6151,0x5152,0x4151,0x6060,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xBEBE,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x5151,0x5151,0x5151,0x4141,0x7F7F,0x4040,0xAFAF,0x8080,0xC0C0,0x8080,0xAFAF,0x8080,0xAFAF,0x5140,0x4041,0x4041,0x4040,0xC0C0,0xC0C0,0xD3C0,0xE0E0,0xE3D0,0xF1F1,0xFFFF,0xF2F2,0xD0D0,0xBFBF,0xCFCF,0xE0E0,0xCFCF,0xAFAF,0xCFCF,0xBFBF,0xAFAF,0xBFBF,0xCFCF,0xDFDF,0xEFEF,0x9F9F,0xF0F0,0xE0E0,0xC0C0,0xD0D4,0xD071,0x7171,0xD0D3,0xD0D3,0xE0E3,0xD3D0,0xD471,0x80C3,0x71D4,0xD3C0,0x7171,0x71C3,0xC0D3,0x71C3,0xC0C3,0x71D3,0xD3D0,0xD3C0,0xD0D0,0xD0D3,0xF0E3,0xF0F0,0xF0F0,0xF1F1,0xF1F1,0xE0F1,0xD0E2,0xD3C1,0xC3C0,0xC0C3,0xC1C3,0xD0D3,0xC0D3,0x6161,0x7171,0xC3C0,0xD4C0,0xC0D0,0xC0C0,0xBFBF,0xD4D0,0xC0C0,0xBF71
.word 0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x5050,0x81B0,0xD081,0xCECE,0xB2B1,0xB1A1,0xB2A1,0xA1A1,0xA1B2,0xA1B2,0xF0C1,0xE0E0,0xF0D1,0xF0F0,0xB3C2,0xB3B3,0xB3B3,0xB3B3,0x8372,0x7272,0x62C4,0x6262,0x71D4,0x7171,0x7171,0x7171,0x7171,0x7171,0x6161,0x6161,0xAFAF,0x7171,0x6161,0x6161,0x7070,0xAFAF,0x8F8F,0x5141,0x6152,0x5151,0x5151,0x4040,0xBFBF,0x7070,0xAFAF,0x8080,0xAFAF,0xBFBF,0xBEBE,0xBFBF,0x7070,0x7070,0x6060,0xAFAF,0x5050,0x5151,0x4151,0x4151,0x4040,0x8F8F,0x8080,0xAFAF,0xAFAF,0xC0C0,0x7070,0xAFAF,0x5050,0x5151,0xC371,0x4172,0x7070,0xAFAF,0xC0C0,0xC3C3,0xE0E4,0xE3E3,0xE3C0,0xF3F0,0xEFEF,0xF4F3,0xF3F5,0xCFCF,0xDFDF,0xF0F0,0xAFAF,0xBFBF,0xAFAF,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0x5151,0xE0E0,0xD0D0,0xC0D3,0x7171,0xD4D4,0xC3D0,0xD4C0,0xC0C0,0xD0C3,0xD0E4,0xD0D3,0xC071,0x7171,0x71D4,0xD371,0xC371,0xD3D0,0xC371,0xC3C0,0x8171,0xD3D3,0xC3C0,0xD3C1,0xD3D0,0xE0E0,0xF0F0,0xF1F1,0xF0F0,0xF0F1,0xE0E1,0xC1D2,0xC1D3,0xC3C2,0xD3D0,0xE0D0,0xF3E0,0xE0E0,0x5151,0xC051,0xC0D4,0xC0C3,0x7171,0xD0D0,0xD1D1,0xC0C0,0xC0C0,0xF2E0
.word 0x5050,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7070,0xC281,0xC1C1,0xC0C0,0xA1A1,0xB2B2,0xB2B2,0xB2B2,0xB2A1,0xA1B2,0xE1C1,0xE0F0,0xF0E1,0xD1D1,0x82B2,0x82B3,0xA3B3,0x93B3,0xB3B3,0x7272,0x6272,0xD371,0xC381,0xD4D4,0xD471,0x7171,0xC3C3,0x7171,0x7171,0x7171,0x6171,0x6161,0x7171,0xAFAF,0x7070,0x6161,0x4140,0x5141,0x5151,0x5161,0x5151,0x8F8F,0x7070,0x8080,0x8080,0xAFAF,0x8080,0xAFAF,0xBEBE,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0x5050,0x5050,0x5151,0x5151,0x4141,0x6F6F,0x9F9F,0xAFAF,0xC0C0,0xC0C0,0xC0C0,0xAFAF,0xAFAF,0x5050,0x7F7F,0x7262,0x6262,0x4141,0xC0C0,0xD3D3,0xC0C0,0xD4C0,0xE2D0,0xD3D3,0xE3D3,0xF3F3,0xEFEF,0xFFFF,0xCFCF,0xDFDF,0xCFCF,0x9F9F,0xAFAF,0xAFAF,0x4141,0x5151,0xAFAF,0x9F9F,0xAFAF,0xE0E0,0xE0E0,0xC0C0,0x7171,0x4141,0x5050,0xD071,0xD0D3,0xD0D3,0xD4C0,0xC0C0,0xC3C0,0xD471,0xD061,0x7161,0x71C3,0x71C0,0xD0E3,0xC0C3,0xC072,0xC0C3,0xD0C3,0xC0C3,0xD0D3,0xD0D2,0xF0E3,0xF0F0,0xF1F1,0xF1F1,0xE1F0,0xD2D1,0xC1C1,0xC3C1,0xD0C0,0xD0E3,0xE3D3,0xF0E0,0xE0E0,0x8F8F,0xE4D3,0xD3C0,0xD4C0,0xD3D0,0xD0D0,0xD0C0,0xC3C3,0xC3C3,0xBFC3
.word 0x5050,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x6F6F,0x7140,0x8181,0xCED0,0xC0D0,0xA1B2,0xA1A1,0xA1A1,0xB2A1,0xA1B2,0xA1B2,0xE1C1,0xE1E0,0xE0E0,0xC1F0,0xB3B3,0xB3B2,0xB3B3,0x93B3,0x8393,0x72B3,0x6272,0xC3C3,0x7171,0xC371,0xC0C0,0x7171,0x7171,0x7171,0x7171,0x7171,0xAFAF,0x7171,0x8080,0x6161,0x6161,0x5050,0x4141,0x5151,0x5151,0x5152,0x4141,0x8F8F,0xAFAF,0x7070,0xAFAF,0xC0C0,0xAFAF,0x8080,0xBFBF,0xBFBF,0xAFAF,0x7070,0xAFAF,0x6060,0x4040,0x5151,0x5151,0x4141,0x5050,0xAFAF,0x8080,0xAFAF,0xAFAF,0xC0C0,0xC0C0,0xD0D0,0x7070,0x4140,0x6261,0x6161,0x7171,0x7171,0xD0D0,0xD4D4,0xD0D3,0xD0D3,0xF3E0,0xD0E0,0xE0E3,0xF3F3,0xFFFF,0xF3F3,0xBFBF,0x6060,0x5151,0x9F9F,0x6060,0x8F8F,0x9F9F,0xD0D0,0x4040,0x7070,0xE0E0,0xD0D0,0xC3C3,0x8F8F,0x7F7F,0x4040,0xAFAF,0xF0F0,0xF4F0,0xD0D0,0xC371,0xD071,0xC071,0x7162,0x70D4,0x7171,0x71D4,0xD3D0,0xC371,0xC371,0xC371,0xD3C0,0xC3C0,0xD3C0,0xD3D0,0xF2E0,0xF1F1,0xF1F1,0xE1F0,0xD2D1,0xD2D0,0xD2D2,0xD3D0,0xD0D3,0xD3D0,0xC3C0,0xF3D0,0xD0F3,0xD0D4,0xE0E0,0xD0D3,0x71C3,0xD4C0,0xE3D0,0xD3D3,0x71C0,0x71D0,0x60AF
.word 0x6060,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0xC051,0xC281,0xBEE0,0xB2C0,0xB1A1,0xB2B2,0xB2B2,0xB2B2,0xB2A1,0xA1B2,0xD1B1,0xE1E0,0xF0F0,0xC2E0,0x82B2,0xB3B3,0xA3B3,0xB3B3,0xB3B3,0xC472,0x7272,0x7272,0xC3D4,0x7171,0xD471,0xD471,0x7171,0x7171,0x7171,0x7171,0x7171,0xAFAF,0x6161,0xAFAF,0x7070,0x6161,0x4141,0x5252,0x5161,0x5161,0x4141,0x7070,0x8080,0xAFAF,0x8080,0xAFAF,0x8080,0x8080,0xAEAE,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x4140,0x6161,0x5151,0x4041,0x7F7F,0xAFAF,0xC0C0,0xC0C0,0xC0C0,0xBFBF,0xC0C0,0xD0D0,0x4141,0x5252,0x6261,0x4151,0x6161,0xC0C0,0xC3C3,0xD0D0,0xD0D0,0xE4E4,0xF0E3,0xF3F2,0xE3E0,0xF3F3,0xFFFF,0xF2F5,0xD0D0,0xAFAF,0x9F9F,0x5050,0x6060,0x5151,0x5050,0x9F9F,0x5050,0xAFAF,0xE0E0,0xC0D3,0x5050,0x4040,0x4040,0x7F7F,0xF0E3,0xF0F0,0xE0E0,0xE4E3,0xD0D0,0xD461,0xD471,0x7171,0x7161,0x71D4,0x7171,0xD0D3,0xC0C3,0xC0C3,0x71C3,0xD0C3,0xC0C3,0xD0C3,0xD1D3,0xF1F1,0xF1F1,0xE1F0,0xD1E2,0xD2D1,0xD1D2,0xE2D1,0xD1D2,0xD0D0,0x71D3,0x71D0,0xF2D3,0xF0F0,0xD0D0,0xE3E0,0xD4C0,0xD0D0,0x71D4,0xE2E2,0xD0D0,0xD4D4,0x7171,0x8F8F
.word 0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8180,0xD081,0xCFCF,0xB281,0xB2A1,0xA1A1,0xA1A1,0xA1A1,0xB2B2,0xB1A1,0xC1B1,0xE0D1,0xF0F0,0xB2D1,0xB382,0xB392,0xB3A3,0xB3B3,0xB393,0xB393,0x7272,0xC371,0x71D0,0xD471,0xD0D0,0xC0C0,0x7171,0x7171,0x7171,0xBFBF,0x7171,0x7171,0x7171,0x7070,0x6161,0x9F9F,0x5141,0x5151,0x5152,0x5151,0x4141,0x5050,0xAFAF,0x8080,0xBFBF,0x8080,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0x4040,0x4141,0x5151,0x5161,0x8F8F,0x7070,0xAFAF,0xC0C0,0xAFAF,0xAFAF,0xC0C0,0xC0C0,0xD0D0,0x9F9F,0x6151,0x6162,0x4151,0xAFAF,0x7171,0xC0C3,0xD3D3,0xE3D3,0xE0D3,0xE0E0,0xF1F2,0xF0F2,0xE0E0,0xBFBF,0xEFEF,0xF3F3,0xE0F0,0x4040,0x8F8F,0x5050,0x9F9F,0x4141,0x9F9F,0x4141,0xF0F0,0xE3F0,0xC0C0,0x4040,0x4040,0x7F7F,0x4040,0xF0F0,0xE0E0,0xC3C3,0xC0C0,0xC0D4,0xC0C3,0xD0D4,0xC3D0,0xD4C0,0x7171,0x71D4,0xD4D0,0xC3C0,0x71C0,0x71C3,0xD4C0,0xC3C0,0xC3C0,0xD1D0,0xF1F0,0xF1F1,0xD1E1,0xD1D1,0xD1D2,0xD1D1,0xE2D1,0xD3D0,0xD3E0,0xD071,0xC3C3,0xD071,0xE0E0,0xD0D0,0xE0E3,0xD0D3,0xC3D0,0x6161,0xBFBF,0x7171,0xC0C0,0x7171,0x7F9F
.word 0x5050,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x6040,0x8181,0xE0C2,0xD0CE,0xB2C0,0xB2A1,0xB2B2,0xB2B2,0xB2B2,0xA1B2,0xA1B2,0xC1B1,0xE0D1,0xF0F0,0x82D2,0xB3B2,0x92B3,0xB3B3,0xB3A3,0x93B3,0x72B3,0xC472,0xC0D4,0x71D4,0x7171,0x7171,0x7171,0xD0D0,0x7171,0x7171,0x7171,0x8080,0x7171,0x8080,0x7171,0x7070,0x4151,0x5141,0x5151,0x5161,0x5151,0x4141,0x9F9F,0x8080,0x8080,0x8080,0xBFBF,0xC0C0,0x8080,0xAFAF,0xBFBF,0xC0C0,0xC0C0,0x7070,0x4060,0x4151,0x5151,0x5151,0x8F8F,0x8F8F,0xC0C0,0xAFAF,0xC0C0,0xC0C0,0xC0C0,0xBFBF,0xD0D0,0x4170,0x5251,0x5151,0x5252,0xC061,0xC3C3,0xD0D0,0xD0D3,0xD0D0,0xE0E3,0xE3E4,0xF3E0,0xF0F0,0xE0F3,0xAFAF,0xBFBF,0xDFDF,0xFFFF,0xBFBF,0x4040,0x8F8F,0x6060,0x4141,0x6060,0x9F9F,0x9F9F,0xEFEF,0x8F8F,0x4040,0x4141,0x5050,0xE0E0,0xF0E0,0xE3E0,0xE0E0,0xE3D1,0xE3D0,0xF3E0,0xF0F0,0xF0F5,0xF0F3,0xD3E0,0x8080,0xD0D3,0xD4C0,0xD371,0xC071,0xC0D4,0xC3C3,0xD0C3,0xD1D3,0xF0E1,0xE1E1,0xD1D1,0xD1D2,0xF1F1,0xD2E1,0xD2D2,0xD0D3,0xC0E3,0x71D3,0x8181,0xD0D0,0xF0E2,0xF0E2,0xE3F0,0xD3D0,0xD0D0,0x7171,0x5151,0x7171,0x7171,0x7171,0x409F
.word 0x5050,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x7070,0xB181,0xC2C2,0xC0C0,0xA1B2,0xA1B2,0xA1A1,0xA1B2,0xA1B2,0xB2B2,0xB1A1,0xC1B1,0xF0E1,0xF0F0,0xB2B1,0x92B3,0xB3B3,0xB392,0xA3B3,0xB3B3,0xB3B3,0x7172,0x7171,0xD4C0,0xD071,0xD471,0xC0C0,0xD471,0x7171,0x7171,0x7171,0x7171,0x7171,0xAFAF,0x7171,0xAFAF,0x4141,0x5151,0x5161,0x5151,0x4151,0x4040,0x9F9F,0xAFAF,0xC0C0,0xAFAF,0x8080,0xAFAF,0xAFAF,0x7070,0x9E9E,0xAFAF,0xAFAF,0xAFAF,0x5050,0x5151,0x5151,0x5161,0x4141,0x9F9F,0xAFAF,0xC0C0,0xAFAF,0xC0C0,0xC0C0,0xC0C0,0xD0D0,0x9F9F,0x6151,0x5162,0x4141,0x7171,0xC0C0,0x7171,0xD3D3,0xD3D3,0xE3E0,0xE3D0,0xE3E0,0xF0F2,0xF2F2,0xF3F0,0xCFCF,0xAFAF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5151,0x6060,0x7171,0xBFBF,0xC0C0,0x71D0,0xE3D0,0xF3E0,0xE0E0,0xE0E3,0xE0E0,0xF0F0,0xF0F0,0xF0F0,0xE0F0,0xF3E0,0xF0F0,0xD0E2,0xE4D0,0xC0C0,0xC0D3,0x71C3,0xC3C0,0xC0C0,0xC1C3,0xD3D0,0xD2D0,0xD1E1,0xD1D2,0xE1D1,0xF0F0,0xE1F1,0xD2E0,0xD2D2,0xC0D3,0xC3C3,0xE3D0,0xF0E3,0xE3F0,0xE2E0,0xE0F1,0xD0D3,0xD0D4,0xE0E3,0xCFCF,0xC0C0,0xBFBF,0xC0C0,0x8F61
.word 0x7070,0x5050,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7171,0x8181,0xC0C0,0x8181,0xB2A1,0xB1A1,0xB2B2,0xB2A1,0xB2B2,0xB2A1,0xA1B1,0xC1B1,0xF0D1,0xE0E0,0x92B2,0xB3B3,0xB292,0xB3B3,0xB3B3,0xB3B3,0xB393,0x7272,0xC3D3,0x71D4,0xC3C3,0xC0C0,0x71D4,0xC0C0,0xC0C0,0x7171,0xC0C0,0xAFAF,0x7171,0x71C0,0x8080,0x5151,0x4141,0x5251,0x6161,0x6151,0x4151,0x4040,0x8080,0x7070,0xAFAF,0x7070,0xC0C0,0x8080,0x7070,0xAFAF,0xBFBF,0xBFBF,0x7070,0xC0C0,0x5151,0x5151,0x5151,0x4141,0x4040,0x7070,0xC0C0,0xBFBF,0xC0C0,0xC0C0,0xBFBF,0xBFBF,0xC0C0,0x4141,0x6161,0x5161,0x5151,0x7171,0xC3C3,0xC3C3,0xD0D0,0xD0D3,0xD0E4,0xE3E3,0xE3E0,0xE3E0,0xF0F0,0xF0F2,0xF0F1,0xE2E2,0xBFBF,0xBFBF,0xDFDF,0xF3F3,0xF4F5,0xD4E4,0xAFAF,0xD4D4,0x5151,0x9F9F,0xF3F3,0xD0D0,0x7171,0xAFAF,0xD0D0,0xD0D0,0xD3D3,0xC0C0,0xF2E3,0xF2F2,0xF3F0,0xD3E0,0xD3D0,0xD0D3,0xE3D0,0xF3E0,0xD0F3,0xD4D3,0xC3C0,0xC371,0xD072,0xC3C3,0xC3C0,0xC1C3,0xD2D3,0xD2D0,0xD1D1,0xD2D1,0xE1E2,0xF0F1,0xF1F1,0xF3F0,0xD3E0,0xE0C1,0xF0F3,0xE3F0,0xE0D1,0xF1E1,0xF3F0,0xE3E0,0xD3D0,0xF0D0,0xE0F0,0xE4E4,0xD3C0,0xC0C0,0xAFAF
.word 0x6060,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7140,0x81C0,0xBED0,0xB1D0,0xA1B2,0xB2B2,0xA1B2,0xA1B2,0xA1B2,0xB2B2,0xA1A1,0xC1B1,0xF0D1,0xE0E0,0xB2B2,0x92B3,0xB3B3,0x9292,0xB3B3,0xB3A3,0x72B3,0xC472,0xD4C0,0xC0C0,0xD4C0,0xD471,0x8080,0x71D4,0xC3C3,0x7171,0x7171,0x7171,0x7171,0xAFAF,0x7171,0x5151,0x5151,0x5151,0x5161,0x5151,0x4151,0x4040,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0xBFBF,0x8080,0xAFAF,0xAFAF,0x5141,0x5151,0x5151,0x5151,0x7F7F,0xAFAF,0xC0C0,0xC0C0,0xC0C0,0xAFAF,0xD0D0,0xD0C0,0xBFBF,0x6161,0x6151,0x6162,0x5151,0xAFAF,0xD071,0xC0C0,0xD4C0,0xE4E4,0xD0D0,0xD3D3,0xD3D0,0xE3D0,0xE3E0,0xF0E0,0xF1F1,0xFFFF,0xE0F2,0xE0E0,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xE4F5,0xE5E5,0xBFBF,0x8F8F,0xD0F3,0xD0D0,0x7171,0x7171,0xD3C0,0xC3D0,0x71D0,0xE371,0xF0F1,0xF0F0,0xD3E0,0xC3D0,0xC3C0,0xC0C0,0xD3C3,0xD3D0,0xE3D0,0xC0C0,0x71C3,0x71D3,0xD3C0,0xC0C3,0xC3C3,0xC0C0,0xC0C3,0xD2C2,0xD1D1,0xC1D3,0xD3C1,0xD2D1,0xF1E2,0xF1F1,0xF3F0,0xF3F0,0xE0F0,0xD1D1,0xE3E0,0xE1E2,0xE0E0,0xD0E2,0xD0D3,0xD3D3,0xF0F0,0xE0E0,0xD0D0,0xD0D4,0xE3D3
.word 0x5050,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0xC050,0xC282,0xD0D0,0x9191,0xB2B2,0xA1A1,0xB2A1,0xB2A1,0xB2A1,0xA1A1,0xB1B1,0xC1B1,0xF0D1,0xD1D1,0xB2B2,0xB2B3,0x92B3,0xB3B3,0xB392,0xB3B3,0xB393,0xC372,0xC3C3,0xC371,0xC3C3,0xD0D0,0xD4C0,0x8080,0xC0C0,0x7171,0xC0C0,0xC0C0,0x7171,0x7171,0x7171,0x6161,0x5151,0x5161,0x5151,0x5161,0x4141,0x7F7F,0x7070,0x7070,0xAFAF,0x8080,0x7070,0x7070,0x7070,0xAFAF,0xBFBF,0xAFAF,0xC0C0,0x5050,0x4140,0x6151,0x5151,0x5151,0x4141,0xD0D0,0xAFAF,0xAFAF,0xBFBF,0xD0D0,0xD0D0,0xBFBF,0xD0D0,0x4141,0x6251,0x5161,0x5151,0xC0D0,0xD4D4,0xD0D0,0xD0D0,0xD0D0,0xD3D3,0xD0D0,0xD3D3,0xE0E3,0xF0F3,0xE2F0,0xF1F0,0xF1F1,0xF1F1,0xEFEF,0xF0F1,0xCFCF,0xAFAF,0xAFAF,0xCFCF,0xE5E5,0xBFBF,0x6060,0x9F9F,0x4141,0x5151,0x9F9F,0xD3D3,0xC3C3,0x6161,0xE2E2,0xF0F0,0xD2F3,0xC1C2,0xD0D3,0xD0D3,0xC3D3,0xC0C0,0xC0C3,0xD0D3,0x71D4,0xC3C0,0xC071,0x71C3,0xC3C0,0x81C0,0xC3C3,0xC381,0xD2C1,0xD3D0,0xC1C1,0xD0D2,0xC1D2,0xE1D1,0xF1F2,0xF0F1,0xF0F0,0xE2E1,0xE0E3,0xD0D2,0xE3C1,0xE2F0,0xD2D1,0xC3D0,0xE1D0,0xF0F1,0xE0E0,0xE0D3,0xD0D0,0xE3BF
.word 0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x8150,0xB2B1,0xE0E0,0xB1D0,0xA1A1,0xB2B2,0xA1A1,0xA1B2,0xB2B2,0xA1B2,0xA1A1,0xC1B1,0xE1E1,0xC1F0,0x92B2,0x92B3,0xB2B2,0xB2B2,0x92B3,0xB3B3,0x72B3,0x71C3,0xD0D3,0x71C3,0xC3D0,0xD371,0xC0C0,0xC3C3,0xC3C3,0x7171,0x7171,0xC0C0,0x7171,0xC0C0,0xAFAF,0x4161,0x5151,0x6152,0x5161,0x5151,0x4141,0x4040,0xAFAF,0x7070,0x7070,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x6060,0xBFBF,0xBFBF,0xAFAF,0x4040,0x5151,0x6151,0x6161,0x4151,0x8F8F,0xC0C0,0xC0C0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xBFBF,0x6161,0x6161,0x6261,0x6161,0xD0D0,0xD0C0,0xD0D4,0xD3D3,0xD3D3,0xD0D0,0xE3E4,0xE0E0,0xF3E0,0xF0F0,0xF0F0,0xE1F1,0xF1E1,0xE1E1,0xF1E1,0xFFFF,0xF1F1,0xF0F0,0xBFBF,0xCFCF,0xCFCF,0x8080,0x9F9F,0x4040,0x7F7F,0x7F7F,0x7171,0xC0D0,0xC0C0,0x5151,0xF0F0,0xE0F0,0xD0D3,0xD0D2,0xC0D3,0xD0C3,0xD3C1,0xC3D3,0xC3C0,0xE3D0,0xC3C0,0xC071,0x71C3,0xD3C3,0xC371,0xC0C3,0xC371,0xC3C0,0xD2C1,0xC0D3,0xD3C2,0xD2D1,0xD3D0,0xE2D1,0xE1E1,0xD2D1,0xE2E2,0xF0E0,0xD0E1,0xC3D3,0xD2C0,0xE0F1,0xD3D2,0xC3D0,0xD0C3,0xD1D2,0xE0E0,0xE3D0,0xE4D0,0xE4D3
.word 0x5050,0x8F8F,0x4141,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x8F8F,0x4040,0x8F8F,0x7F7F,0xB260,0xC291,0xE0E0,0xA1C0,0xB2B1,0xA1A1,0xB2B2,0xB2A1,0xA1B2,0xA1B2,0xB1B1,0xD1C1,0xF0F0,0xB1D0,0xB3B2,0xB3B2,0xB3B3,0xB392,0xB3B2,0xB382,0xB3B3,0xD371,0xD4D0,0xD4C0,0xC371,0xC0C0,0x71D3,0xC0D4,0xC0D4,0xD0D0,0xD0D0,0x7171,0xC0C0,0x7171,0x7171,0x9F9F,0x6151,0x6161,0x5251,0x5161,0x4141,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0x7070,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0x6060,0x5141,0x5261,0x6161,0x4151,0x7070,0xBFBF,0xD0D0,0xD0D0,0xBFBF,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0x5151,0x6161,0x6161,0xD0D0,0xD3D3,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xE4D3,0xD0D0,0xE3E3,0xE0E3,0xF0F3,0xF0F2,0xF1F1,0xF1F1,0xF1F1,0xF1F1,0xF1F1,0xFFFF,0xF0F0,0xF0F0,0xEFEF,0xCFCF,0xAFAF,0x6060,0x4040,0x6F6F,0x5151,0xD0D0,0xC3C3,0x4040,0xF3F3,0xE0F0,0xD3E0,0xD2C1,0xD3D1,0xD3D0,0xD3D0,0xD0D3,0xD0D1,0xC0C3,0xD0D3,0xC0D4,0xC3C3,0xC371,0xC071,0xC0C3,0xC371,0x81C3,0xC082,0xC1C3,0xC3D0,0xC1C1,0xC1D3,0xC0D3,0xE0D3,0xE2D1,0xD2D1,0xD2D0,0xE3D1,0xE3E0,0xC0C1,0xC1C3,0xD2D1,0xD2D1,0xC3D0,0xC3C0,0xC3C0,0xD0C3,0xD0D3,0xD0D0,0xF4D0
.word 0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0xB161,0xD0B2,0xD0CE,0xB2B1,0xA1B2,0xB2B1,0xA1B1,0xB2A1,0xB2A1,0xB1A1,0xA1A1,0xD1B1,0xE0F0,0xB2C1,0x92B2,0x92B3,0x92B2,0x92B2,0x92B3,0xB3B3,0x72B3,0xC3C3,0x71D3,0x71D4,0xD3D3,0xD4C0,0xC3C3,0xD0D0,0xC0C0,0xD4C0,0xD4C0,0x7171,0xC0C0,0x7171,0xC0C0,0x5151,0x5261,0x6161,0x6151,0x5151,0x4141,0x9F9F,0xAFAF,0x8080,0x8080,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0xBFBF,0xC0C0,0xC0C0,0x8F8F,0x6151,0x6151,0x5161,0x5151,0x5050,0xD0D0,0xBFBF,0xD0D0,0xD0D0,0xD0D0,0xBFBF,0xD0D0,0xE0E0,0x6161,0x6261,0x5161,0xAFAF,0xD0D0,0xD0D0,0xD0D0,0xD3D0,0xD3D3,0xD0D0,0xE4D0,0xD0E3,0xE0E0,0xF0F3,0xF2F0,0xF1F1,0xF1F1,0xF1F1,0xF1F1,0xF1F1,0xFFFF,0xFFFF,0xFFFF,0xE0E0,0x6060,0x5050,0x9F9F,0x4040,0x6161,0xD0D0,0xD0D0,0x7070,0x4141,0xF0F0,0xE0F3,0xC1D3,0xD0D3,0xD0D3,0xC3D0,0xC3C0,0xD3C0,0xE3E0,0xD3D0,0xE3D0,0xC3C0,0x7171,0x7171,0x71C3,0x71C3,0x81C3,0xC371,0xC381,0xD0C3,0xC2C3,0xD0C2,0xD0D2,0xC3C1,0xD0E3,0xF0F2,0xD1E2,0xD1C2,0xD0D3,0xE3D1,0xC3D0,0xD0C3,0xE1D2,0xE1E1,0xC0D3,0x71D3,0xC0D4,0xD471,0xD0D0,0xD0D3,0xF3D3
.word 0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0xB170,0xE0B1,0xC0C0,0xA1B2,0xA1A1,0xA1A1,0xB2B2,0xA1B2,0xA1B2,0xA1B1,0xB1B1,0xE0C1,0xD1E0,0xB2B2,0xB292,0xB3B2,0xB392,0xB3B2,0xB3B2,0x82B3,0x81B3,0xD0D3,0xD0D4,0xD3C0,0x71D0,0xC0D4,0xC3C3,0xC0D4,0xD4D4,0xD3D3,0xD0D0,0xD0D0,0x7171,0xC0C0,0x6161,0x5151,0x6152,0x5261,0x5161,0x5151,0x4040,0x8080,0x8080,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0x4140,0x5151,0x6162,0x5261,0x8F8F,0xAFAF,0xC0C0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0x6161,0x7172,0xAFAF,0xD0D0,0xD0D0,0xD0D0,0xD0E4,0xE0E0,0xE0D0,0xE4E0,0xD0D0,0xD0D0,0xE4D0,0xE3E0,0xF3E0,0xF1F0,0xF1F1,0xF1F1,0xF1F1,0xF1F1,0xF1F1,0xFFFF,0xFFFF,0xEFEF,0x6060,0x5050,0x4040,0x8F8F,0x4040,0xD3D3,0x6161,0x8F8F,0x4141,0xF0F1,0xD2E0,0xD0C1,0xD4C0,0x71C0,0xC372,0xD371,0xD3D0,0xD3D0,0xD4C0,0xE3D0,0x71C3,0xC3C0,0xC371,0xC071,0xC371,0xC371,0x71C0,0xC0C3,0xD3C0,0xC3C0,0xD3C1,0xC3C1,0xD0C3,0xD2E0,0xF2F2,0xD0E1,0xC2D3,0xD3D0,0xE0D1,0xD0E3,0xD3C0,0xD1D3,0xD2E1,0xC3D0,0xD0C3,0xC3D0,0xD071,0xD0C3,0xD3D3,0xE0D0
.word 0x8F8F,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x8040,0xB2C2,0xC1C1,0xB1E0,0xB2B1,0xA1B1,0xA1B2,0xA1A1,0xB2B1,0xB1A1,0xA1A1,0xB1A1,0xE0C1,0xB1D3,0xB2A1,0xB3B2,0xB292,0x92B2,0x92B2,0x92B2,0xB2B3,0xB3B3,0xC0C3,0xC4D0,0xD471,0xD0D0,0xD0D4,0xC3C3,0xC0C0,0xC3C0,0xC0C0,0xD3D3,0xD4C0,0xC0C0,0x7171,0x7171,0x5151,0x6161,0x6161,0x6161,0x5151,0x4141,0xAFAF,0xAFAF,0x8080,0x8080,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0xBFBF,0xBFBF,0xC0C0,0x5151,0x6151,0x6262,0x5161,0x4141,0xD0D0,0xBFBF,0xD0D0,0xBFBF,0xBFBF,0xD0D0,0xD0D0,0xE0E0,0x7171,0x7171,0x7161,0x6161,0xBFBF,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xCFCF,0xD0D0,0xD0D0,0xD0D0,0xAFAF,0xAFAF,0xD0D0,0xD0D0,0xE0E0,0xD0D0,0xE0E0,0xF0F3,0xF1F1,0xF1F0,0xF0F1,0xE0E0,0x9F9F,0x5050,0x8F8F,0x4040,0x7F7F,0x9F9F,0x4040,0x9F9F,0xE1E1,0xE2F0,0xC1D2,0x71D3,0x71D3,0xD3D3,0xE0D0,0xE0F3,0xF0E3,0xE0E2,0x71D3,0xD071,0xC3D0,0x7171,0x71C3,0xC371,0xC071,0x7171,0x71C3,0xC381,0xC0C3,0xC0C3,0xC1C3,0xD0C3,0xF2D1,0xD1F2,0xC1D3,0xC1C3,0xC3D0,0xD0C2,0xD0D3,0xD3E3,0xC0C0,0xD0D3,0xD0D3,0xC0C3,0xD3C3,0xD3D0,0x71D4,0xD4D0,0xC3D0,0xE0D4
.word 0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x8181,0xB181,0xCEE0,0xB2D0,0xA1A1,0xA1A1,0xA1B1,0xB2B1,0xB2A1,0xB2A1,0xB1A1,0xB1A1,0xC1C1,0xB2B1,0xB2B2,0xB292,0x92B2,0xB2B2,0xB3B2,0xB3B2,0x82B2,0x81B2,0xD3C0,0xD0D4,0xD0D3,0xD4C0,0xC0C0,0xC0D0,0xD4D4,0xC0C0,0xC3C3,0xD4C0,0xD0D0,0xC0C0,0xC0C0,0x5151,0x6151,0x6152,0x6152,0x5162,0x4151,0x6060,0x8080,0x8080,0xAFAF,0xAFAF,0x8080,0x8080,0x7070,0x7070,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x5141,0x6261,0x6161,0x5161,0x4040,0xD0D0,0xD0D0,0xBFBF,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xE0E0,0xD0D0,0x6161,0x6171,0xD0D0,0xD0D0,0xBFBF,0xD0D0,0xD0D0,0xBFBF,0x8080,0x9F9F,0x8F8F,0x8F8F,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x6060,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xD0D0,0xBFBF,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0xE1E3,0xF0E1,0xD2F0,0xC3C0,0xD3C0,0xD3C0,0xD0D0,0xE3E3,0xE3E0,0xD3D0,0xD3D0,0xE3D0,0xD3D0,0x71D3,0x71C3,0xC371,0x7171,0x72C3,0xC381,0xC371,0xC371,0xC3C0,0xC3C2,0xC0C3,0xD2C1,0xF2E1,0xD2F0,0xC3D0,0xC3C0,0xC0C3,0xC3C2,0xD3D0,0xD0D0,0xD0D3,0xD0D3,0xD0D3,0xC3C3,0xD0D0,0xC0D4,0x71D0,0xD0D4,0xD0D0,0xE3C0
.word 0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x8170,0xB1B2,0xCFCF,0xA1B1,0xB1A1,0xB1B2,0xA1B2,0xA1B2,0xA1B2,0xA1B1,0xA1B1,0xB1B1,0xB1C1,0xB2A1,0x92B2,0xB2B2,0xB292,0xB2B2,0xB292,0xB292,0xB292,0xC3B2,0xD3C3,0xC3C0,0xC0C0,0xD0D4,0xD3D3,0xC3C3,0xC0D4,0xC0C0,0xC0C0,0xD0D0,0xD4D4,0xC0C0,0xC3C3,0x5161,0x6261,0x6161,0x6161,0x6151,0x5151,0x5050,0xAFAF,0xC0C0,0x8080,0x8080,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x6060,0x6161,0x6152,0x6161,0x4152,0x7F7F,0xBFBF,0xC0C0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xBFBF,0xD0D0,0xD0D0,0x6161,0x6161,0xD0D0,0xBFBF,0xD0D0,0xD0D0,0xAFAF,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x6060,0x6060,0x5151,0x9F9F,0x5050,0x5050,0x6060,0x6060,0x5050,0x5050,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4141,0xE0E0,0xF0F1,0xC1E3,0xD0C3,0xD3D3,0xD3D0,0xE4D0,0xD0D0,0xC0D3,0xC0C3,0x71D3,0x71D0,0xD0C3,0xC371,0x71C0,0x71C3,0xD371,0x7171,0x71C3,0x7171,0xC0C3,0xC0C3,0x81C3,0xC3C0,0xC1D3,0xC1C2,0xD1C3,0xD0D3,0xC2C3,0xC3D0,0xC0C3,0xD0D3,0xE3D3,0xE0E0,0xD0D3,0xD0D3,0xC0C3,0xD4C3,0xD0D0,0xC062,0xD0D4,0xC0C3,0xE3D4
.word 0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0xB170,0xC1B2,0xD0D0,0xB2C0,0xA1B2,0xB2A1,0xB2A1,0xB1A1,0xB1A1,0xA1A1,0xB1A1,0xB1B1,0xA1B2,0xB2B2,0xB292,0x92B2,0xB2B2,0xB292,0xB2B2,0xB2B2,0xB2B2,0xB2B2,0xC3C0,0xD3C0,0xD4C0,0xC0C0,0xC0C3,0xC0C0,0xD0D0,0xC3C3,0xD3D3,0xD0D0,0xC0C0,0xC0C0,0xC0C0,0x5161,0x6262,0x6162,0x6162,0x5161,0x4141,0x9F9F,0x8080,0x7070,0xBFBF,0xC0C0,0xC0C0,0x8080,0x8080,0x8080,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x6161,0x6161,0x6162,0x4141,0x8F8F,0xD0D0,0xD0D0,0xBFBF,0xD0D0,0xD0D0,0xCFCF,0xD0D0,0xD0D0,0x6161,0x6161,0xAFAF,0xD0D0,0xD0D0,0xBFBF,0x6060,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x8F8F,0x9F9F,0x5050,0x5050,0x5050,0x8F8F,0x9F9F,0x5050,0x8F8F,0x5050,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0x4040,0xE1E1,0xF1F2,0xC1D1,0xD3D0,0xD0D0,0xD0D3,0xD0D3,0x71C3,0xC3C0,0xD3D0,0xE0E3,0xC3D3,0xD471,0xC071,0xD472,0x71C0,0x71D4,0xC371,0x7171,0x71C3,0xC3C3,0xC3C0,0xD3C0,0xD1D3,0xD3D0,0xC1D1,0xC3C1,0xD3D0,0xD0C1,0xC1C3,0xD3D0,0xD3C1,0xD3C0,0xE3D0,0xE3E0,0xD3D0,0xC371,0xD0D0,0x71C3,0xD4D3,0xD0D0,0xC3C3,0xD0D0
.word 0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0xB281,0xD0B1,0xCECE,0xA191,0xB2A1,0xA1A1,0xA1B1,0xA1B2,0xA1B2,0xB1B2,0xA1A1,0xB1B2,0xB2A1,0xB2B2,0x92B2,0xB2B2,0xB292,0x92B2,0xB3B2,0xB292,0xB292,0xC3B2,0xD3C2,0xC0C3,0xC0D4,0xC0D4,0xD3D3,0xC0C0,0xD0C3,0xC0C0,0xD4D0,0xD0D0,0xC0C0,0xC3C3,0x7171,0x6161,0x6162,0x6261,0x6262,0x5161,0x5151,0x7070,0x8080,0xBFBF,0xC0C0,0xC0C0,0xAFAF,0xC0C0,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x6161,0x6161,0x6262,0x4141,0xD0D0,0xD0D0,0xBFBF,0xD0D0,0xBFBF,0xCFCF,0xD0D0,0xD0D0,0xBFBF,0x7070,0x6171,0xD0D0,0xD0D0,0xBFBF,0x7070,0x8F8F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x6060,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0xF2F2,0xF1F1,0xD3D2,0xC3D0,0xC0C3,0xC0D4,0xC371,0xD4C0,0xE4C0,0xF3F3,0xF1F0,0xE0F0,0x71D0,0xD3D4,0xF0E0,0xF0F3,0xD3D0,0x7171,0x71C3,0x7171,0xC371,0xE0D3,0xF0E3,0xF2F0,0xF0F1,0xD2E2,0xC2D0,0xC1C3,0xC3C3,0xC3C0,0xD371,0xC3D0,0xE3D0,0xF0E0,0xF0F0,0xE0F3,0xC0D3,0xD0D4,0xC0D4,0x7171,0xD0D3,0xD4D0,0xE3D3
.word 0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x8181,0xD0B2,0xC0C0,0xB2B2,0xA1B1,0xA1B2,0xB2B2,0xB2A1,0xA1A1,0xA1B1,0xB1A1,0xA1A1,0xA1B1,0x92B2,0xB2B2,0xB292,0x92B2,0xB2B2,0xB292,0xB2B2,0xB2B2,0xB1B2,0xC381,0xC3D0,0xD0D0,0xC3D0,0xC0C0,0xD4D0,0xD0D0,0xD4D0,0xAFAF,0xD3D0,0xD0D0,0xD0D0,0x6161,0x6262,0x6261,0x6261,0x6161,0x6161,0x7061,0x8080,0xC0C0,0x8080,0xC0C0,0xAFAF,0xC0C0,0xAFAF,0xC0C0,0x8080,0x7070,0xAFAF,0x7070,0xAFAF,0x6161,0x6261,0x4161,0x6060,0xBFBF,0xD0D0,0xD0D0,0xBFBF,0xD0D0,0xD0D0,0xCFCF,0xE0E0,0x8080,0xAFAF,0xBFBF,0xD0D0,0xBFBF,0x6060,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x8F8F,0x5050,0x8F8F,0x4140,0x8F8F,0x8F8F,0x5050,0x5050,0x5050,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x5050,0x6161,0xEFEF,0xF1F1,0xD0E0,0xC0D4,0xC3C0,0xC3C0,0xD0D4,0x7171,0xF0C3,0xFFFF,0xF2F2,0xF3F0,0xD0E4,0xE0D0,0xF0F2,0xF2F0,0xF3F0,0xD3E0,0xC3C0,0xC3C3,0xC0C3,0xE0D3,0xE0E2,0xF1E1,0xF1F1,0xE2E1,0xD0D2,0xC0C2,0xC0D3,0xC3C3,0xD0D3,0xD0C3,0xF0D3,0xF0F2,0xF0F3,0xF0E0,0xD0D3,0xC0D4,0xD471,0xD3D3,0xD4D0,0xD0D4,0xD0E0
.word 0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x5050,0xB171,0xD0B2,0xCFCF,0xA1B2,0xB2A1,0xB2A1,0xA1A1,0xB1B2,0xB1A1,0xB1A1,0xB1A1,0xB2A1,0xB2B2,0xB2B2,0xB2B2,0xB2B2,0xB292,0x92B2,0xB2B2,0xB292,0x92B2,0xB2B2,0xC1C2,0xD0C3,0xC3C3,0xC0D4,0xC0D4,0xC0D4,0xC3C3,0xD0D0,0xC0C0,0xD0D0,0xD0D0,0xC0C0,0x6171,0x7162,0x6162,0x6162,0x6161,0x6161,0x6161,0xC0C0,0x7070,0xC0C0,0xAFAF,0xC0C0,0xAFAF,0x8080,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x6161,0x5162,0x4151,0x8F8F,0xD0D0,0xD0D0,0xD0D0,0xBFBF,0xD0D0,0xD0D0,0xD0D0,0xBFBF,0xC0C0,0xBFBF,0xC0C0,0x5050,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5050,0xD0D0,0xAFAF,0x9F9F,0x9F9F,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x4040,0x5050,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x5151,0xE0E0,0xE0F3,0xF1F1,0xF0F0,0xC3D3,0xC0C3,0xD0D4,0x71E4,0x71C0,0x7171,0xF0D0,0xF0F0,0xF1F1,0xF1F1,0xF3F0,0xF3F3,0xE0E0,0xF0F3,0xF0F0,0xE0F3,0xE0E3,0xF0E0,0xE4E0,0xD3D0,0xE3D0,0xE2E0,0xF2F0,0xD1E0,0xE2D1,0xD3E1,0xC3C1,0xC0C3,0xC0D3,0xC0D3,0xE0E3,0xF1F1,0xE3E3,0xD3D0,0xD3D0,0x71D0,0xD0C3,0xD3D3,0xC0C0,0xE0D0,0xF4E3
.word 0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x81B0,0xD0B2,0xC0BE,0xB281,0xA1B2,0xB1B2,0xB2A1,0xB2A1,0xB2B2,0xA1B2,0xA1B2,0xA1B2,0xA1B2,0xA1B2,0x92B2,0x92B2,0xB2B2,0xB292,0xB2B2,0xB2A1,0xB2A1,0xB2A1,0xB2B2,0xC3C2,0xC0C0,0xD4D0,0xC0C0,0xD0D0,0xC0C0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0x7171,0x6261,0x6271,0x6261,0x6261,0x6161,0x6161,0xC0C0,0x6161,0x8080,0xC0C0,0x7070,0x8080,0x8080,0xAFAF,0x8080,0x8080,0x7070,0x7070,0x6060,0x7070,0x6161,0x6161,0x4141,0xD0D0,0xBFBF,0xBFBF,0xD0D0,0xD0D0,0xBFBF,0xBFBF,0xD0D0,0xC0C0,0xAFAF,0x6060,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0xE4E4,0xE0E0,0xE0F4,0xD3D3,0x6161,0x9F9F,0x8F8F,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x5151,0xE0E0,0xF0F0,0xF1F1,0xE1F1,0xE0D1,0xF3E0,0xD3D0,0xD4C0,0xD471,0xC371,0xE371,0xF0F0,0xF0F0,0xFFFF,0xF0F0,0xD0E0,0xE0E3,0xE0E0,0xF0F1,0xF0F0,0xE3F0,0xE0E0,0xF0F0,0xD0F0,0xC0D4,0xD0D3,0xE0E3,0xE2E2,0xF0F0,0xF0F1,0xC0D3,0xC381,0xD3D0,0xC3C0,0xD3D0,0xF1F1,0xE0E0,0xD0D3,0xD0D3,0xD471,0xE4C0,0xD0E3,0xD4D3,0xF4E3,0xF3E0
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x8161,0xD381,0xBFBF,0xA1B1,0xA1B2,0xB2B2,0xB2B2,0xA1B2,0xA1B2,0xA1B2,0xA1B2,0xA1B2,0xB2B2,0xB2B2,0xB2B2,0xB2B2,0xB2B2,0xB2B2,0xA1B2,0xA1B2,0xA1B2,0xB2B2,0xB1B2,0xC2B2,0xC3C3,0xD3C0,0xC0C3,0xD3D3,0xC3D0,0xC0C0,0xC0C0,0xD0D0,0xD4D4,0x7171,0x6271,0x6271,0x7172,0x61C4,0x7171,0x71C3,0x7171,0x7171,0x7171,0x6161,0x8080,0x7070,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0x6060,0x6161,0x6261,0x4151,0x8F8F,0xBFBF,0xD0D0,0xD0D0,0xBFBF,0xBFBF,0xD0D0,0xC0C0,0xAFAF,0x9F9F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xE0E0,0xDFDF,0xEFEF,0xF4F4,0xCFCF,0xF4F4,0xE4E4,0x5151,0x8F8F,0x8F8F,0x5050,0x5050,0x5050,0x4040,0x8F8F,0x4040,0x8F8F,0x5050,0xD0D0,0xE0F0,0xE0E0,0xE0E0,0xF1E0,0xF1F1,0xF1F1,0xF1F1,0xD4E0,0x71C0,0x71C0,0x7171,0xC3D0,0xF1E0,0xF0F0,0xF1F1,0xF2F2,0xC0E3,0xC0C3,0xE0D3,0xF0E3,0xF2F2,0xF0F0,0xF0F3,0xF0F0,0xF0F0,0xC3E0,0xD371,0xD3C1,0xF2E0,0xF0F1,0xF0F2,0xC3E0,0xD3D0,0xD3D0,0xC0D3,0xC0C3,0xE1D2,0xE0E1,0xD0D2,0xD0D3,0x71C3,0xD0E4,0xD0E4,0xE0C3,0xE0F3,0xE0F4
.word 0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7160,0xC081,0xCFCF,0xB271,0xB2B2,0xB2A1,0xB2A1,0xB2A1,0xB2A1,0xB2A1,0xB2A1,0xB2A1,0xA1B2,0xB292,0xB292,0xB292,0xB292,0xB292,0xB2A1,0xB2A1,0xB2A1,0xA1B2,0xB2B2,0xB2B1,0xC3C0,0xC0C3,0xD3D0,0xC3D0,0xD0D0,0xC0C0,0xC0C0,0x7171,0x6060,0x5050,0x8F8F,0x6161,0xC3C3,0xC381,0xC3C3,0xD3D0,0x7272,0x7171,0x7171,0x7171,0x8080,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x6060,0x6161,0x7171,0x5151,0x4141,0x6060,0xD0D0,0xD0D0,0xBFBF,0xD0D0,0xC0C0,0xAFAF,0xAFAF,0x5050,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x5151,0xE0E0,0xDFDF,0xEFEF,0xFFFF,0xDFDF,0xF4F4,0xF4F4,0xF3F3,0xF3F3,0x9F9F,0x4040,0x4040,0x8F8F,0x5050,0x8F8F,0x4040,0x4040,0xF0F0,0xE3E3,0xF2D0,0xE1E1,0xE0E0,0xD0D0,0xE0E0,0xF0F1,0xF1F1,0xF0F1,0xD0E3,0xD471,0xC371,0xD3C0,0xC0C3,0xE2D3,0xF1F0,0xFFFF,0xF1F1,0xC3E0,0xC3C0,0xD3D0,0xE2D1,0xF1E0,0xF0F0,0xF0F0,0xE0E0,0xF3E0,0xE0F3,0xC0D3,0xD0C3,0xF0E2,0xF3E0,0xF2E0,0xD3F0,0xD0C3,0xC0D3,0xC3C3,0xC3C0,0xD0D3,0xD0D2,0xD0E3,0xD0E3,0xD0C3,0xD3D3,0xC3C3,0xF3E3,0xF3F3,0xF2F3
.word 0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7070,0x7171,0xBED0,0xB1C0,0xA1B2,0xB2B2,0xA1B2,0xB2B2,0xB2B2,0xB2B2,0xA1B2,0xB2B2,0xB2A1,0xB2B2,0xB2B2,0xB2B2,0xB2B2,0xA1B2,0xB2A1,0xB2A1,0xB2A1,0xB2A1,0xB1B2,0xC2B2,0xC2C2,0xD3D0,0xD0D3,0xC0C3,0xD4D4,0xC3C0,0x6161,0x4141,0x7F7F,0x7F7F,0x4040,0x8161,0xA1B1,0xA1B1,0xB1A1,0x82B2,0xB3B3,0x72C3,0x72C3,0x7171,0xAFAF,0x7070,0xAFAF,0x6060,0xAFAF,0x9F9F,0x9F9F,0x6060,0x6161,0x6161,0x4151,0x9F9F,0xD0D0,0xBFBF,0xD0D0,0xC0C0,0xAFAF,0x9F9F,0x6060,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x5050,0xAFAF,0xF4F4,0xF2F2,0xEFEF,0xFFFF,0xF0F0,0xD0D0,0xBFBF,0xF4F4,0xF4F4,0xD0F4,0xC0C0,0x4040,0x5050,0x5050,0x4040,0x4040,0x7F7F,0xF1F1,0xF1F1,0xF1F1,0xF0F0,0xF0F1,0xE0E0,0xD0D0,0xE3E3,0xE1E2,0xF1F1,0xF1F1,0xD0D3,0xD0D3,0xD0D3,0x71D3,0x71C0,0xC071,0xF1F1,0xF1F1,0xF0F0,0xD0F2,0xD3D3,0xC3C0,0xE0D1,0xE2E2,0xF1E0,0xF1F0,0xE1F1,0xE0E0,0xF0F0,0x6262,0xC071,0xF2D3,0xE3E0,0xE2E0,0xE3E0,0xC3D0,0xD3D0,0x71D0,0xC3C3,0xC0D0,0xD2D3,0xD0D2,0xC0D3,0xC0D4,0xC0D4,0xD1D3,0xF0F3,0xF4F4,0xF0F0
.word 0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7040,0xC071,0xC0C0,0xC0C0,0xB282,0x92B2,0xB2B2,0xA1B2,0xA1B2,0xA1A1,0xB2A1,0xB2A1,0xB2B2,0x92B2,0xB2B2,0xA1B2,0xA1B2,0xB2A1,0xA1B2,0xA1B2,0xA1B2,0xA1B2,0xB2B2,0xC2B1,0xD0C2,0xC0D3,0xC3D0,0x7171,0x71C0,0x6171,0x6060,0x7F7F,0x4040,0x4040,0x6141,0xB1B1,0xB1B1,0xB1A1,0x92B2,0xB3B2,0xB382,0xB382,0x7272,0x7171,0xC0C0,0xBFBF,0x8080,0x7070,0x7070,0x7070,0x6060,0x6160,0x6161,0x6161,0x4141,0xAFAF,0xD0D0,0xC0C0,0xAFAF,0x9F9F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x8F8F,0xF4F4,0xCFCF,0xDFDF,0xF0F0,0xF0F0,0xFFFF,0x5050,0x9F9F,0xCFCF,0xBFBF,0xC0D0,0xD0D0,0xAFAF,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0xF0F0,0xD3D0,0xE0D0,0xF3E0,0xF0F0,0xE0E0,0xD0D0,0xD0D0,0xD3D0,0xF1E0,0xE1F1,0xD3E0,0xE3E0,0xE3E0,0xC3C0,0xD471,0x6261,0xF0E3,0xF0F0,0xF1F1,0xD3F0,0x7171,0x7162,0xD371,0xD1D0,0xE0D3,0xE2E0,0xF0F0,0xE0F1,0xF0F0,0x61E3,0x6262,0xE0C3,0xE0E3,0xD0E3,0xE0D2,0xD0C3,0xC0C3,0xC3C3,0xC3C0,0xC3C0,0xD0C1,0xD1D3,0xD4D0,0xD471,0xD4C0,0xF3F3,0xF3F3,0xE0F3,0xF0F3
.word 0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x6161,0x8281,0xD0B2,0xCFCF,0x81C0,0xB2B2,0x92B2,0xB2B2,0xB2A1,0xB2B2,0xB2A1,0xA1B2,0xA1B2,0xB2A1,0xA1A1,0xA1B2,0xB2B2,0xA1A1,0xB2B2,0xB2A1,0xA1B2,0xB2B2,0xB2B1,0xC0C2,0xD3C2,0xD3D0,0xC3C0,0xD471,0x7171,0x6161,0x4040,0x4040,0x6F6F,0x5140,0xB1C0,0xB1B1,0xA1A1,0xA1B1,0xB2B2,0xB2B2,0xB2B2,0x82B3,0xC3B2,0xC0C3,0xBFBF,0xC0C0,0x70C0,0x7070,0x7070,0x7070,0x6160,0x7161,0x6261,0x5161,0x9F9F,0x7070,0x5050,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x6F6F,0x4040,0xE4E4,0xF4F4,0xD0D0,0x7070,0xAFAF,0xAFAF,0xE0E0,0xC0C0,0x6060,0x7070,0x7070,0x8080,0x7070,0xD0D0,0x5050,0x5050,0x4040,0x7F7F,0x4040,0xD360,0xC3C0,0x71C3,0xD0C0,0xE0E4,0xE0E0,0xF3F3,0xD0E3,0xD0D4,0xE0D3,0xE1E1,0xE0E0,0xF0F2,0xF0F1,0x71E3,0x7171,0x7171,0xF0D4,0xF0F1,0xF0F0,0xF0F0,0x6171,0x6161,0x7171,0xC0D3,0xC3D2,0xD3D0,0xF2E0,0xF3F0,0xE0E0,0x7171,0x7171,0xF371,0xF0E0,0xD3E0,0xD3D0,0xC3C0,0xC3D3,0xC3C0,0xC3C0,0xC371,0xD0D3,0xD0D2,0xD0D3,0x71C0,0xD4D4,0xE3E3,0xE0E0,0xE3E3,0xF3F0
.word 0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0xB171,0x8181,0xD0D0,0xBFBF,0xB281,0xB2A1,0xA1A1,0xB2B2,0xA1A1,0xA1B2,0xA1B1,0xB2A1,0xA1B1,0xB2B2,0xB2A1,0xA1A1,0xB2B2,0xA1A1,0xB2B2,0xB2A1,0xB1A1,0xB1B2,0xD3C1,0xD0D0,0xD0D0,0x81D4,0x7181,0x6161,0x4040,0x6F6F,0x6F6F,0x6140,0xB181,0xB1A1,0xA1A1,0xB1A1,0xA1A1,0xB2A1,0xB2A1,0xB2B2,0xB2B2,0x72B2,0xC0C0,0x80C0,0x9080,0x8080,0x8080,0xAFAF,0x7170,0x7271,0x6271,0x6161,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0xBFBF,0xCFCF,0x8080,0x7070,0x8070,0x6060,0xAFAF,0x6060,0x8F8F,0x5050,0x5050,0xB0B0,0x6060,0x6060,0x7070,0x6060,0x4040,0x6F6F,0x4040,0xC070,0xD0D3,0xD3C0,0x7171,0xC380,0xE0D0,0xF0E0,0xE3F0,0xD3D0,0xD1D0,0xE3D1,0xF2E0,0xF1F1,0xF0F1,0xC3C3,0x7171,0x7171,0xE361,0xF2F0,0xF2F0,0xD3F0,0x6161,0x6262,0x6161,0xC361,0xC1C0,0xD0C3,0xE0D2,0xF1F1,0xF3F3,0xE0E0,0x7161,0xE061,0xF0F3,0xE0F3,0xC0D3,0xD3C3,0xC0C0,0x71C3,0xC0C3,0xC0C3,0xD3C1,0xD3D1,0xD4D0,0xD471,0xE3D0,0xF3F3,0xE3E3,0xF3F3,0xF0F0
.word 0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7150,0x8181,0xC0B2,0xE0E0,0xC0C0,0xA1B2,0xB2B2,0xA1B2,0xB2A1,0xB2A1,0xB2A1,0xB2B1,0xB2A1,0xA1B1,0xA1B2,0xB1B2,0xA1B2,0xB2A1,0xA1B2,0xB1B2,0xB2B2,0xC1B1,0xC0D3,0xD3C2,0xC371,0x7171,0x5161,0x4040,0x7F7F,0x4040,0x6161,0xB181,0xB1B1,0xA1B1,0xB1A1,0xB1B1,0xB2A1,0xB2B2,0xB2B2,0x92B2,0x81B2,0xC0C3,0xC0C0,0x71C0,0x71D0,0x80B0,0xAFAF,0x8080,0x7171,0x71C3,0x5171,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0xAFAF,0xE4E4,0xD0D0,0x7070,0x7070,0xAFAF,0x6060,0x7070,0x5060,0x5050,0x5050,0x6060,0x7070,0x6060,0x5050,0x7070,0x7070,0x9F9F,0x5050,0x8070,0x7080,0x8080,0xC0C3,0xC0C3,0xC3C3,0xD3C0,0xE0E0,0xF0E0,0xE0F3,0xD0E3,0xE0D3,0xF1F0,0xF1F1,0xF3E1,0xC0D0,0x6271,0x7171,0x7161,0xE0E0,0xE0F3,0xE0E0,0x6161,0x6161,0x6161,0x6161,0xC361,0xC3D0,0xD2C1,0xF1E2,0xF0F0,0xD0E3,0x71D4,0xE3E3,0xF0F0,0xE3E0,0xC3D0,0xC0C3,0xC3C3,0xC0C3,0x71D3,0x81C3,0xD2C3,0xD3D0,0x71D0,0xD0D4,0xD0D0,0xE0E0,0xF2F3,0xF0F1,0xF3F0
.word 0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x6060,0xC281,0xB281,0xD0C2,0xE0E0,0xC0C0,0xA1B2,0xB2A1,0xB2B1,0xA1A1,0xA1B2,0xA1B2,0xA1B2,0xA1B2,0xB2A1,0xB2A1,0xB2A1,0xB1B2,0xB2A1,0xC1B1,0xD0C1,0xB1C2,0xB2B1,0x7181,0x6181,0x6161,0x8F8F,0x7F7F,0x4040,0x6F6F,0xB161,0xB1B1,0xA1B1,0xB1B1,0xA1B1,0xA1A1,0xA1B2,0xA1A1,0xA1B2,0xB2B2,0xE0C3,0xE0E0,0xC0D0,0xC0C3,0xC071,0x7171,0x80C0,0x8180,0x81C3,0x7172,0x9F9F,0x7F7F,0x6F6F,0x4040,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x9F9F,0xE4D4,0xCFCF,0xBFBF,0x7070,0x9F9F,0x8080,0x7070,0x6060,0x6060,0x4040,0x4040,0x5050,0x8080,0x5050,0x4040,0x5060,0x5050,0xD070,0x70D0,0x7070,0x5050,0x7070,0x7070,0xC0C0,0x8171,0xC071,0xD071,0xE0E3,0xE0E0,0xE0F3,0xF3E0,0xE2E0,0xE1F0,0xE0E1,0x71E4,0x7171,0x6261,0x7171,0xE0E3,0xE3E0,0xD3E0,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0xE3D0,0xD3D0,0xF0F0,0xE0E0,0xD0D0,0xE0C3,0xE3E0,0xD0E0,0x71D3,0xD3C0,0xC3C0,0xC3C0,0xC371,0xC381,0xD3C0,0xD0D1,0xC0D4,0xD4D4,0xD0D4,0xE0E4,0xEFEF,0xF0F0,0xE0E3
.word 0x4040,0x4040,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7150,0xB281,0xB2B2,0xD1C1,0xE0E0,0xC3D0,0xB1B1,0xB1B2,0xB1B2,0xB2B2,0xB2B2,0xA1B2,0xB2B2,0xB2B1,0xB1B2,0xB1B2,0xC2B1,0xD3C0,0xD2E0,0xB2C2,0x82B1,0x7181,0x5151,0x5151,0x4040,0x7F7F,0x4040,0x7F7F,0x6040,0xB181,0xA1B1,0xB1B1,0xC1C1,0xA1B1,0xA1B1,0xB2B2,0xB2B2,0xB2B2,0xB192,0xD0D0,0xE0F0,0xC2C0,0xC0B0,0xC0C0,0xD0D0,0x71C0,0x82C2,0xB3B2,0x7171,0x4141,0x4040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5151,0xD4D4,0xCFCF,0xE5E5,0xC0C0,0x7070,0x8080,0x7070,0x9F9F,0x5050,0x6060,0x5050,0x4040,0x5050,0x7070,0x5050,0x5050,0x8F8F,0x5050,0xD0D0,0x6060,0x6060,0x5050,0x5060,0x4040,0x5151,0xD3C0,0xD4C0,0xD4C0,0xC0C0,0xE3D0,0xE3E0,0xF0E0,0xE0F3,0xE0F1,0xE0E2,0x7171,0x6161,0x7171,0x6171,0xE371,0xD0D0,0xE0D3,0x6161,0x6261,0x6161,0x6161,0x6161,0x5151,0xE361,0xD0C0,0xE0E0,0xD0E0,0xD0D3,0xE0E3,0xD0E3,0xD3D3,0xC3C0,0xC0C3,0x71C3,0xC0C3,0x71C3,0xC0C3,0xC0C3,0xD0D3,0xE4E4,0x7171,0xC0C0,0xD3D3,0xF2F2,0xE3E3,0xD3C0
.word 0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x8171,0xB291,0xB1A1,0xC2B1,0xD0D0,0xD0E3,0xD0D0,0xD0D0,0xC0C3,0xB1B1,0xB2C2,0x81C0,0xC1C3,0xD1D0,0xC1C1,0xC0D3,0xC1C2,0xA1B2,0xB2B1,0x6171,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0xC1C1,0xC1D1,0xC1C1,0xC1C1,0xB1C1,0xB2A1,0xB2B2,0xA1B2,0xA1B2,0xB2B2,0xD3B2,0xD0E0,0xC0D0,0xC081,0xC081,0xD0D0,0xC0C0,0xB281,0xB2B2,0x7182,0x5050,0x7F7F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x9F9F,0xAFAF,0x6262,0xBFBF,0xE5E5,0xCFCF,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x6060,0x5050,0x4040,0x4040,0x6060,0x8080,0x5060,0x4040,0x5050,0x4040,0xD0D0,0x7070,0x4040,0x4040,0x6060,0x4040,0x4141,0xC0C0,0xD0D3,0xD0E3,0xE0E3,0xE0E3,0xE0E0,0xF3E0,0xE2E0,0xE2E0,0xE3E0,0x71D0,0x6162,0x6161,0x6262,0xC061,0xD0D4,0xD0D0,0x6271,0x6161,0x5161,0x6161,0x5151,0x5161,0xC0C0,0xC3D4,0xC0C0,0xD0D0,0xD371,0xE3E0,0xD0D0,0xC0C3,0x71C3,0xC3C0,0xC0C3,0xC3C3,0xC3C0,0xC381,0xD3C3,0xD4D0,0xE0E0,0xE3E3,0xC0D4,0xD0D4,0xE0E3,0xD0E3,0xC1C3
.word 0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x8161,0xA1B2,0xB1B2,0xC2B1,0xD0C2,0xD0D3,0xD0E4,0xBFBF,0xBFBF,0xC0C0,0xC271,0xC1C1,0xC1D2,0xB2C2,0xB2A1,0x91B2,0x81B2,0x5171,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x6F6F,0x4040,0xBFBF,0xF1F1,0xE1F1,0xB1C1,0xA1A1,0xA1B1,0xB2B2,0xB2A1,0xB2A1,0xB2B2,0xB2B2,0xD0D0,0xC0D0,0xC3C0,0xB1C0,0xD0C0,0xBFBF,0x81C0,0xB2B2,0x82B2,0x6161,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xD0D0,0xAFAF,0x9F9F,0xBFBF,0xD0D0,0x7070,0x7070,0x8080,0x7070,0x9F9F,0x6060,0x6060,0x5050,0x4040,0x4040,0x7070,0x6060,0x4050,0x8F8F,0x4040,0x7070,0x5050,0x4040,0x4040,0x6060,0x4040,0x5050,0xE3E3,0xF0F0,0xE0E0,0xD0E0,0xD0E0,0xE0E0,0xE0E4,0xE0F0,0xE0E3,0xD0D3,0xD4D0,0x6161,0x5161,0x6161,0x7162,0xD4C0,0xD3C0,0x7171,0x6161,0x6161,0x5151,0x5161,0x5151,0x7251,0x80D0,0xC0D4,0xC0D4,0xD0C3,0xD0E0,0xD3D3,0xE3E0,0xE3D0,0xC0D3,0xC0C3,0x71C3,0x81C3,0xC3C0,0xD0D0,0xD3D3,0xE3E3,0xD0D0,0xD471,0xC0C0,0xD0D0,0xD0E3,0xD0D3
.word 0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x5050,0x6160,0x7171,0xB181,0xB2B1,0xB2B1,0xB2B1,0xC1B1,0xB1C1,0xB281,0x81B1,0xB2B1,0x82B1,0x8281,0x8181,0x6171,0x5151,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x9F9F,0xBFBF,0xF0E0,0xC1E1,0x8181,0xB2B1,0xB2A1,0xA1B2,0xB2B2,0xB2B2,0xB2A1,0xD0C1,0xD0D0,0xC0D0,0xC0C2,0xC0C0,0xCFCF,0xC0D0,0xB281,0x82B2,0x6172,0x8F8F,0x5050,0x8F8F,0x4040,0x6F6F,0x7F7F,0x6F6F,0x4040,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x9F9F,0xBFBF,0xD0D0,0xD3D3,0xF3F3,0xE0E0,0x7070,0x7070,0x6060,0x6060,0x9F9F,0x5050,0x6060,0x5050,0x4040,0x5050,0x6060,0x6070,0x6060,0x4040,0x8F8F,0x4040,0x5050,0x7F7F,0x4040,0x4040,0x5050,0xE0E0,0xF0F2,0xE0F0,0xD0D0,0xD0D3,0xD0D4,0xC0C3,0xD071,0xE0E3,0xE3D0,0xF4D0,0x7171,0x6161,0x6262,0x6161,0x6161,0xC0C3,0x7171,0x6171,0x6161,0x5161,0x5151,0x5151,0x5151,0xD361,0xD4C0,0x71C0,0xC0C3,0xD0C3,0xD0E4,0xE0D3,0xF0F3,0xF0F2,0xD3E0,0xC3C0,0xC371,0xC381,0xD0C3,0xE4D3,0xC3C3,0xF5E4,0xF5F5,0x7171,0xD4D0,0xD3C0,0xE3E0,0xD3D0
.word 0x4040,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6140,0x8171,0xB281,0xB181,0xA1A1,0xB2A1,0x7181,0x7161,0x8181,0x7171,0x6161,0x5151,0x7F7F,0x7F7F,0x6F6F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x4040,0x4040,0x4040,0x9F9F,0xDFDF,0x6161,0x6161,0xB281,0xA1B2,0xB2B2,0xA1B2,0xB2B2,0xD0C1,0xD0D0,0xC1D0,0xC0C1,0xC081,0xD0D0,0xC0C0,0xB271,0x82B2,0xC3B2,0x4040,0x4040,0x8F8F,0x4050,0x6F6F,0x4040,0x6F6F,0x4040,0x6F6F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7070,0xD0D0,0xF3F3,0xF0F0,0xDFDF,0x8080,0x7070,0x7070,0xAFAF,0x9F9F,0x7070,0x5050,0x6060,0x6060,0x4040,0x4040,0x6060,0x7070,0x5050,0x5050,0x4040,0x6040,0x4040,0x4040,0x4040,0x5050,0x7070,0xF0F0,0xF0F0,0xD0D0,0xD0D0,0xC0C0,0x71C0,0xD471,0xC371,0xD4C0,0xD0D0,0xD0D0,0xF0F0,0x6161,0x6151,0x6161,0x5161,0xD461,0x71C0,0xD4D4,0x6161,0x5151,0x6161,0x5151,0x5151,0x6161,0xC071,0xC0C3,0xC3C3,0xE3C0,0xE4D0,0xE0E0,0xE2E0,0xF3E0,0xF3E0,0x71D3,0xC3C0,0xC071,0xD3D3,0xD2E0,0xD0D0,0xC3C3,0xD4D4,0xD4D4,0x6161,0xD0D3,0xD0D0,0xD0C3
.word 0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x6150,0x7161,0x5161,0x5151,0x8F8F,0x9F9F,0x6161,0x8F8F,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x5050,0x7F7F,0x7F7F,0x8F8F,0xF0F0,0xBFBF,0x7F7F,0x8151,0xB2B2,0xB2B2,0xB2B2,0xB292,0xD0C1,0xE0E0,0xC0D0,0x81B1,0xC0B1,0xD0D0,0xCFCF,0xB1C0,0xB2B2,0xC3B2,0xD0D3,0xAFAF,0x7F7F,0x4040,0x8F8F,0x4040,0x6F6F,0x4040,0x6F6F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x6F6F,0x5151,0x8F8F,0x7F7F,0x4040,0x4040,0x9F9F,0x7070,0xF0F0,0xFFFF,0xD0D0,0xC0C0,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x6060,0x6060,0x5050,0x4040,0x5050,0x6060,0x6060,0x4050,0x4040,0x4040,0x7F7F,0x4060,0x4040,0x7F7F,0x4040,0x5050,0xF0F0,0xF0F0,0xE0E0,0xC0C3,0xC371,0xD471,0x71C0,0x71C0,0xC0C3,0xC0D4,0xD0D3,0xE3E0,0x6161,0x5151,0x6161,0x5151,0x6161,0xE3C3,0xE2E0,0xD3D3,0x71D3,0x6161,0x6161,0x6161,0xC071,0x7171,0xC371,0x7171,0xE3D0,0xD0D0,0xD3D0,0xE0E3,0xE0E2,0xE0F3,0xC3D0,0x71C3,0xC3C3,0xF3E0,0xE3E3,0xC3D3,0xD4D0,0xD4D4,0xE4E4,0xC0D4,0xC0D4,0xD3D3,0xD3D0
.word 0x4040,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6060,0x8F8F,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x5050,0x8F8F,0x8F8F,0x4040,0x4040,0xAFAF,0xCFCF,0x4040,0x6060,0xB171,0xB2B2,0xB292,0xB292,0xC0B2,0xE0E0,0xC0E0,0xB1B1,0xC0B1,0xD0D0,0xCFCF,0xB0C0,0xB291,0xB2B2,0xC3C3,0xCFCF,0x5151,0x4040,0x4040,0x7F7F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0x5050,0x4040,0x4040,0x7F7F,0x4040,0x6060,0xAFAF,0xF0F0,0xD0D0,0xC0C0,0x7070,0x7070,0x7070,0xAFAF,0x5050,0x6060,0x6060,0x5050,0x5050,0x5050,0x4040,0x7050,0x6060,0x8F8F,0x4040,0x4040,0x5050,0x6060,0x4040,0x4040,0x4040,0xF0F0,0xF0F0,0xF0F0,0x71E0,0x71C0,0xC0C3,0xC0C3,0xD0C3,0xD4D4,0xD3D3,0x71C0,0x7171,0xD3D3,0x61D0,0x6161,0x5151,0x5151,0x6151,0xF0F0,0xFFFF,0xD0D0,0xE0D0,0xE0E3,0xE0E3,0xD0D3,0xD0E4,0xC0D4,0x71D3,0xD470,0xE0E3,0xD3D3,0xD3C3,0xE3D0,0xE1E0,0xE1E1,0xC3C3,0xC071,0xE3E3,0xF5F5,0xD4D0,0x71C0,0xC0C0,0xD4D4,0xBFBF,0x7171,0xC071,0xE0D3,0xE0E3
.word 0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x4040,0x4040,0x6F6F,0xBFBF,0x4040,0x4040,0x7150,0xB2B2,0xA1B2,0xB2B2,0xB1B2,0xD0D0,0xE0E0,0xB1C0,0xC0B1,0xC0C0,0xE0E0,0xB0D0,0xB2A1,0x82B2,0xC3C3,0xCFCF,0xCFCF,0xAFAF,0x7F7F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x4040,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x9F9F,0x4040,0x7F7F,0x4040,0x8F8F,0xAFAF,0x7070,0xD0D0,0xD0D0,0x8080,0x8080,0x7070,0x8080,0x6060,0x8F8F,0xAFAF,0x6060,0x5050,0x6050,0x4040,0x4040,0x6060,0x6060,0x4050,0x5050,0x8F8F,0x4040,0x6060,0x4040,0x4040,0x4040,0xE0E0,0xF0F0,0xF0F0,0xD3D0,0xD3D0,0xD0D0,0xD0D0,0xCFCF,0xD0D0,0xC0E0,0xC372,0x7171,0xC0C0,0xE4E4,0x5151,0x5151,0x5151,0x5151,0x6161,0xE0F3,0xC3D0,0xD4D0,0xD0C0,0xD0D0,0xF3E0,0xF3E0,0xE3E0,0xE4D0,0xE4D0,0xF0F0,0x7171,0xD3C0,0xE2D0,0xF2E1,0xE2E0,0x71D0,0xF3D3,0xCFCF,0xBFBF,0x7171,0x7171,0xC3C3,0xD0D0,0x7171,0xD0C3,0xE3E4,0xE0E0,0xD3E0
.word 0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x8F8F,0x6060,0x5050,0x4040,0x8F8F,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7070,0x5050,0x8F8F,0x4040,0x7F7F,0x8F8F,0x4040,0x7F7F,0x5050,0xB181,0xA1B2,0xB2B2,0xA1B2,0xD0B1,0xF0F0,0xD0E0,0x81C0,0xD0C0,0xD0E0,0xC0C0,0xB281,0xB2B2,0x71C3,0xBFBF,0xBFBF,0xCFCF,0xAFAF,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xD0D0,0xE0E0,0xC0C0,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x5050,0x6060,0x5050,0x4040,0x5050,0x6060,0x4040,0x4040,0x4040,0x4040,0x6060,0x7F7F,0x4040,0x4040,0x5050,0xF0F0,0xF0F0,0xF3F3,0x7171,0xD0D0,0xBFBF,0xE0F0,0xD4D0,0xD4D0,0x7171,0x71D3,0x7171,0xD3D0,0x5151,0x4151,0x5151,0x6161,0x7161,0xC3C3,0xD071,0xC071,0x71D4,0xD0D3,0xD0D3,0xE0E0,0xF3E0,0xF0E0,0xE0F3,0xF0F2,0xF4F0,0xF0F3,0xF0F2,0xE1F1,0xE0E2,0xC0C3,0xE0E0,0xAFAF,0x7171,0x7171,0x71C3,0x7171,0xD0D3,0xF0E2,0xE0E3,0xF0E0,0xE3E3,0xD3D0
.word 0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x6060,0x7070,0x4040,0x4040,0x4040,0x4040,0x4040,0x9F9F,0xAFAF,0x4040,0x7F7F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x7150,0xB2B1,0xB2B2,0x92B2,0xC2B2,0xD0D0,0xE0E0,0x81C0,0xD0C0,0xD0E0,0xC0C0,0xB2B2,0xB2B2,0xC381,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x4040,0x6F6F,0x7F7F,0x4040,0x6F6F,0x4040,0x6F6F,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x4040,0x5050,0x4040,0x6F6F,0x6F6F,0x4040,0xD4D4,0xD0D0,0xE0E0,0xDEDE,0xE0E0,0x8080,0x7070,0x8070,0x7070,0xAFAF,0xAFAF,0xAFAF,0x5050,0x6060,0x4040,0x5050,0x6060,0x6060,0x5050,0x4040,0x4040,0x7F7F,0x6060,0x4040,0x7F7F,0x4040,0x6060,0xD0D0,0xE1E0,0xF0F0,0xC0C0,0xD0D0,0xD0D0,0xD4E0,0xD0D0,0x7171,0x7171,0xC371,0x7171,0xD071,0xD3D3,0x5151,0x7161,0x7171,0x7171,0x8181,0xC3C3,0xC3D0,0xD3D0,0xC3C0,0xD0D0,0xD0D3,0xD0D3,0xD0E3,0xE1E0,0xF2F0,0xF0F0,0xF0F0,0xF1F0,0xF2F0,0xD3E0,0x7171,0xD4D4,0x71D0,0xD4D4,0xD4D4,0xD371,0xC2D0,0xE3D0,0xE0E3,0xF3E0,0xE3E0,0xD0E0,0xC0D3
.word 0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x6060,0x7070,0x6060,0x9F9F,0xAFAF,0xAFAF,0x4040,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x4060,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0xB171,0x91B2,0x91B2,0xB1B2,0xE0C0,0xE0E0,0xC0D0,0xC1C0,0xC1D0,0x81D0,0x92B2,0xB2B2,0xD381,0xBFBF,0xCFCF,0xBFBF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x4040,0x6F6F,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x6F6F,0x5050,0x8F8F,0x7F7F,0x4040,0x8F8F,0xBFBF,0xE0E0,0xE0E0,0xFEFE,0xCFCF,0x8080,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0x6060,0x8F8F,0x5050,0x4040,0x5050,0x6060,0x5050,0x7F7F,0x4040,0x4040,0x6060,0x4040,0x4040,0x4040,0x7070,0xD0D0,0xD0D4,0xF3F3,0xC0C0,0x7171,0xD0C3,0xD0D4,0x6171,0x71D4,0x7171,0x7171,0x7171,0xD471,0xC0D0,0x7171,0x7171,0xD371,0xC371,0xC0C0,0xC071,0xD0D3,0xE3D0,0xD3D0,0xD4D0,0xD3C0,0xD3C0,0xD3D0,0xD0D3,0xD0D3,0xE0D3,0xF0E2,0xF0F2,0xE0E2,0xD0D3,0x7171,0xC0C0,0xD062,0xC0C0,0xC0C0,0xD0D3,0xD3D0,0xD3D0,0xD0D0,0xE0E3,0xD0E3,0xD3D0,0xD3C0
.word 0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x6060,0x6060,0x7070,0x7070,0x6060,0x7070,0x7070,0x4040,0x7F7F,0x4040,0x8F8F,0x5050,0x8F8F,0x9F9F,0x5050,0x4040,0x7F7F,0x7F7F,0x6060,0x81C0,0x81B1,0x81B2,0xD0C1,0xDFDF,0xD0D0,0xD0D0,0xC0C1,0x81B1,0xB2B2,0x81B2,0xC0C3,0xBFBF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4040,0xD0D0,0xD0D0,0x5050,0x7F7F,0x9F9F,0xD0D0,0xEFEF,0xDFDF,0xFEFE,0x80E0,0x7070,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0x6060,0x9F9F,0x5060,0x4040,0x4040,0x6060,0x6060,0x5050,0x4040,0x8F8F,0x4040,0x6060,0x5050,0x4040,0x4040,0xD0D0,0xF0F0,0xE0E0,0xE0E0,0xC3E4,0xD371,0xD4D0,0x61C0,0x71D4,0x7171,0x6162,0x6171,0x71D4,0x7171,0xD4C0,0xC0D3,0x7171,0xC0C3,0xD0D0,0x71C0,0xC3C3,0xE0E0,0xE3E3,0xE3F0,0xD3D0,0xD3D0,0xC0D3,0xC0C3,0xC0D3,0xD0C3,0xC0C3,0xD0D3,0xE0E3,0xD1E3,0xD3D0,0xD371,0xC3C0,0xD4D0,0xC3C0,0xC3C3,0xD4C0,0xD3C0,0xD3C0,0xD3C0,0xD3D0,0xD3D0,0xC0C3,0xC0D3
.word 0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x6060,0x4050,0x4040,0x4040,0x4040,0x9F9F,0x6060,0x6060,0x7070,0x6060,0x9F9F,0x6060,0x8F8F,0x7F7F,0x4040,0x4040,0x4040,0x6060,0x8F8F,0x6060,0x9F9F,0x9F9F,0x4040,0x4040,0x7040,0xB181,0xB281,0xD0B1,0xCFCF,0xE0E0,0xD0D0,0xB1C0,0xB2B1,0x81B1,0x81B2,0x7171,0xAFAF,0xCECE,0xCFCF,0xBFBF,0xAFAF,0x4040,0x8F8F,0x8F8F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x7F7F,0xD0D0,0xE0F0,0x6060,0x9F9F,0xE3E3,0xF0F0,0xEFEF,0xEFEF,0xFFFF,0xCECE,0xC0C0,0x7070,0x7070,0x7070,0x9D9D,0x7070,0x9F9F,0x7070,0x5050,0x4040,0x5050,0x5050,0x6060,0x5050,0x4040,0x4040,0x7F7F,0x6050,0x4040,0x7F7F,0x4040,0xF0F0,0xF0F0,0xE0E0,0xF0F0,0xE3F0,0xD0D0,0xC0C0,0x7171,0x7171,0x7171,0x7171,0x7171,0xC371,0x72C0,0xC0C3,0xD3D3,0x71D3,0xC3C0,0xC0D2,0xC0C0,0xD0C0,0xE4D0,0xF2D0,0xF2F0,0xE3E0,0xE0E0,0xE0E3,0xD0D3,0xD3D3,0xD3C0,0xC3D0,0xD3D0,0xE0D1,0xD3D1,0x71D0,0x7171,0xD472,0xD3C0,0xC0D0,0xC0C0,0xC0D4,0x71D4,0x71D0,0xC0D3,0xC0D4,0xC0D4,0xD0D3,0xD3D3
.word 0x4040,0x8F8F,0x5050,0x4040,0x4040,0x4040,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0xC0C0,0x9F9F,0x6060,0x6060,0x5050,0x8080,0xAFAF,0x6060,0x6060,0x6060,0x6060,0x6060,0x5050,0x4040,0x4040,0x9F9F,0x5050,0x9F9F,0x7070,0x9F9F,0x4040,0x4040,0x4040,0x7F7F,0x8181,0x8181,0xC0B1,0xE0D0,0xE0E0,0xC0D0,0xB1C0,0x81B1,0x81B1,0x8181,0x6171,0x9F9F,0xBEBE,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x5151,0x4141,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4040,0x4040,0xE0E0,0xF0F0,0xE0F0,0xBFBF,0xF0F0,0xDFDF,0xFEFE,0xFEFE,0xEFEF,0xD0DE,0xAFAF,0x8080,0x7070,0x7070,0x9D9D,0xAFAF,0x7070,0x709D,0x9F9F,0x5050,0x4040,0x5050,0x5050,0x5050,0x7F7F,0x4040,0x4040,0x6060,0x4050,0x4040,0x8F8F,0xF0F0,0xC0D0,0xC0C0,0xC0C0,0xE0E0,0xD0E4,0x61C3,0x71D4,0x7171,0x71D4,0x71C3,0xC3C3,0x71D0,0xC371,0x8171,0xD1C2,0xD0D0,0xD0D4,0xC0D0,0x8181,0xE4E4,0xE0E0,0xF0F3,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xE3E0,0xD0D0,0xC1D3,0xD0D3,0xD1D3,0xD3E0,0xD0D0,0xD3D3,0x6171,0x71C0,0xE0E3,0xF3F3,0xD4E0,0x71C0,0xC0C0,0xD0D4,0xC371,0xC3C0,0xD3C0,0xD3D0,0xC1D0
.word 0x8F8F,0x5050,0x9F9F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x8F8F,0x9F9F,0x6060,0x7070,0x7070,0x7070,0xAFAF,0x6060,0x7070,0x6060,0x6060,0x6060,0x7070,0x7070,0x9F9F,0x8F8F,0x6060,0x7070,0x7070,0xAFAF,0x6060,0x5050,0x8F8F,0x8F8F,0x4040,0xC080,0xC1C1,0xE0D0,0xD0E0,0xC0C0,0xA1A1,0x91B2,0x81B1,0x7181,0x7171,0x9F9F,0x9F9F,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x5050,0x9F9F,0x9F9F,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0xAFAF,0xE0E0,0xE0E0,0xF0F0,0xE0E0,0xDFDF,0xEFEF,0xEFEF,0xFEFE,0xDFDF,0xC0C0,0x7070,0x7070,0x7070,0xBFBF,0xAFAF,0x7070,0xAFAF,0x6060,0x5050,0x5050,0x6060,0x6060,0x5050,0x4040,0x4040,0x4040,0x5050,0x6060,0x7F7F,0x4040,0xD0D0,0x61C0,0xE0D4,0x71D3,0xE3D0,0xD0D0,0x7161,0x7171,0xC3C3,0x7171,0xD471,0xC0C0,0x7161,0x7171,0xD2C1,0xE1C2,0xE4F0,0xC0C0,0x6161,0xE0E0,0xE0E0,0xE3E0,0xE0E0,0xF3E0,0xF0F0,0xF0F0,0xF0F0,0xE0E0,0xE4D0,0xD0D0,0xD0D3,0xD0D3,0xD0D3,0xD3D3,0xC0C0,0xD461,0xE371,0xF0F0,0xFFFF,0xF0F0,0xD3D0,0xD4D4,0xC071,0xD0D4,0xD3C0,0xE3D0,0xD3D0,0xC3C0
.word 0x8F8F,0x9F9F,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x8F8F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x8F8F,0x6060,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xBFBF,0xC0C0,0x7070,0xAFAF,0x7070,0x5050,0x4040,0x8F8F,0xD1D0,0xE0E0,0xC2D0,0xB181,0xA1B1,0x81A1,0x71C0,0x7171,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9E9E,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x6060,0x6060,0x8F8F,0x4040,0x6F6F,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x4040,0x4040,0xC0C0,0xD0D0,0xE0E0,0xE0E0,0xBFBF,0xDFDF,0xFEFE,0xFEFE,0xEFEF,0xFEFE,0xD0D0,0x7070,0x7070,0xAFAF,0xADAD,0xAFAF,0xAFAF,0xAFAF,0x7070,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x4040,0x7F7F,0x7040,0x5050,0x4040,0x4040,0x5050,0x6161,0xD0D0,0xE0F3,0xD0D0,0x7171,0x7171,0x7171,0x7161,0x7171,0xC0C0,0x7171,0x7171,0xC371,0xD3C0,0xF1F0,0xD0F1,0x7171,0xD1D1,0xF0E1,0xE4E0,0xD0D0,0xD4C0,0xD4C0,0xD3D0,0xE4D0,0xD0D0,0xC0D4,0xD071,0xD371,0xC3C0,0xD3D0,0xD3D0,0x71C0,0x7161,0x7161,0xF3D0,0xFFFF,0xF1F0,0xF0F0,0x71E0,0xC0C0,0xD0D3,0xC0C3,0xD0D3,0xD0E3,0xD0D3,0xD0C3
.word 0x4040,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x5050,0x5050,0x6060,0x7070,0x8080,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x6060,0x8080,0x7070,0x7070,0x8080,0x7070,0x8080,0x7070,0x7070,0x6060,0x6060,0x9F9F,0xAFAF,0xE0E0,0xE0E0,0xB1D0,0xB1B1,0x81B1,0x7171,0x9F9F,0xAFAF,0xAFAF,0x7070,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x5050,0x5050,0x9F9F,0x4150,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x8F8F,0x8F8F,0xD0D0,0xD0D0,0xE0E0,0xDFDF,0xE0E0,0xCFCF,0xDFDF,0xDFDF,0xEFEF,0xDFDF,0xC0C0,0x8080,0xBFBF,0x8080,0xBDBD,0xC0C0,0x7070,0x9D9D,0x6060,0x5050,0x8F8F,0x6060,0x6060,0x6060,0x4040,0x4040,0x4040,0x6060,0x7070,0x7F7F,0x7F7F,0x4040,0x5050,0x9F9F,0xD0D0,0xD0D0,0x7181,0x71C3,0x7171,0x6161,0x7171,0x7171,0x8181,0x7171,0x7171,0xD0D3,0xF0E2,0xE1F1,0xD0D0,0xE0D0,0xDFDF,0xD0D0,0xC3C0,0xC071,0xC0D4,0xD3C0,0xD0D0,0xC0D4,0xC0C0,0xC3C3,0x71D3,0xC0D4,0xD0D3,0xC0D3,0x61D4,0x7171,0xD461,0xE0E0,0xF0F1,0xF0F0,0xF0F0,0xD3E3,0xC0C3,0xC0C3,0xD3D3,0xE3D0,0xD3D0,0xC3C0,0xC3D0
.word 0x7F7F,0x4040,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x8F8F,0x5050,0x5050,0x6060,0x7070,0x6060,0x5050,0xC0C0,0xC0C0,0x8080,0x7070,0xC0C0,0xC0C0,0xAFAF,0xE0F0,0xC0C0,0xD0D0,0xC0C0,0xC0C0,0xAFAF,0xC0C0,0x7070,0x9F9F,0x6060,0x9F9F,0xBFBF,0xD0E0,0xC1D0,0x71C0,0x6080,0x6060,0x8F8F,0x6060,0x9F9F,0x7070,0xAFAF,0x7070,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x5050,0x8F8F,0x6050,0x9F9F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4041,0x5050,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x4040,0x7070,0xD0D0,0xCFCF,0xFEFE,0xDFDF,0x80CE,0xAFAF,0xEEEE,0xDFDF,0xFFFF,0xDEFE,0xC0C0,0x8080,0xC0C0,0xAFAF,0xCFCF,0xBFBF,0xAFAF,0x7070,0xBFBF,0x9F9F,0x5050,0x5050,0x6060,0x5050,0x4040,0x4040,0x4040,0x4040,0x5060,0x4040,0x4040,0x4040,0x4040,0x4040,0xC0C0,0x71C0,0x7171,0xC071,0x71D4,0x6161,0x7171,0xC3C3,0x7171,0x7171,0x7171,0x7171,0xD0D0,0xF3F0,0xC3D0,0xE0E0,0xD0E0,0xD0D4,0xC371,0xD3D0,0xD3C0,0xD0D3,0xD0E4,0xE4D0,0xD3D0,0xC1D0,0xD0D0,0xD0D0,0xD0D3,0xD0C3,0x7171,0x7171,0xC061,0xF2F2,0xF0F0,0xF0F1,0xE3F0,0x71D0,0xC0C0,0x7171,0xD071,0xD0E3,0xC0D3,0xC1D3,0xD1D2
.word 0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x5050,0x9F9F,0x6070,0x9F9F,0x8080,0x7070,0xD0D0,0xE0E0,0x80D0,0xC0C0,0xBFBF,0xD0D0,0xE0F0,0xF0D0,0xE0E0,0xD0D0,0xC0C0,0xD0D0,0xC0C0,0xC0C0,0xD0D0,0xD0D0,0x7070,0x9F9F,0x5050,0xE0E0,0xD0D0,0x8080,0x5151,0x8F8F,0x8F8F,0x5050,0x6060,0x6060,0xAFAF,0x7070,0xAFAF,0x7070,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x5050,0x5050,0x9F9F,0xAFAF,0x5151,0x6F6F,0x4040,0x7070,0xAFAF,0xD0D0,0xC0C0,0xC0C0,0x5050,0x4040,0x4040,0x7F7F,0x9F9F,0xE0E0,0xE0E0,0xEFEF,0xEEEE,0xE0E0,0xAFAF,0xDFDF,0xFEFE,0xEFEF,0xFEFE,0xD0D0,0xC0C0,0x8080,0xAFAF,0xBDBD,0xBFBF,0x8080,0x5050,0xBFBF,0x6060,0x5050,0x5050,0x6060,0x6060,0x4040,0x7F7F,0x4040,0x6060,0x6060,0x4040,0x4040,0x7F7F,0x5050,0x7171,0xC0D4,0x7171,0x7181,0x71D4,0x7171,0x6161,0x6161,0x7171,0x7171,0x6161,0x7171,0x7171,0x7171,0xD0D0,0xD0D3,0xD0D0,0xD0D0,0xC0C0,0xC3C3,0xC0C0,0xD0D0,0xE3D0,0xE0E0,0xD0D0,0xC0C0,0xD0D0,0xF3E3,0xD3E3,0xD3D0,0xD4D0,0x7171,0x6171,0xE371,0xFFFF,0xF1F0,0xE3F0,0xD1E0,0xD0D0,0x71D3,0xC3C0,0xD3D0,0xD3D0,0xD3C0,0xD3C1,0xC1D0
.word 0x4040,0x4040,0x4040,0x8F8F,0x4040,0x5050,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0xAFAF,0xC0C0,0xD0D0,0xD0D0,0xE0E0,0xE0E0,0xBFBF,0xBFBF,0xE0C0,0xAFAF,0xCFCF,0xCFCF,0xF4F4,0xC0C0,0x8080,0x7070,0xC0C0,0xE0E0,0xEFEF,0xF0F0,0x7070,0x6060,0xBFBF,0xBFBF,0x4040,0x9F9F,0x6060,0x8F8F,0x7070,0x9F9F,0x9F9F,0x6060,0x6060,0x9F9F,0x7070,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7070,0x9F9F,0x5050,0x9F9F,0x7171,0x9F9F,0x4040,0x4040,0xE0E0,0xF0F0,0xF0F0,0xF0F0,0xF0F0,0xE0E0,0x7070,0x4040,0x4040,0x8F8F,0xE0E0,0xFEFE,0xEFEF,0xFFFF,0xE0E0,0xAFAF,0xDFDF,0xDFDF,0xEFEF,0xEFEF,0xC0C0,0xC0C0,0xBFBF,0xBFBF,0xCDCD,0xCFCF,0xBFBF,0x5050,0xBFBF,0x6060,0x6060,0x5050,0x6060,0x5050,0x4040,0x4040,0x4040,0x4040,0x6060,0x4040,0x4040,0x4040,0x5050,0xF0F0,0x61D0,0x7171,0x71D0,0x71C0,0x7171,0x6161,0xC0C3,0x71C3,0x6161,0x7171,0x7161,0xAFAF,0x7070,0xD061,0xD0D0,0xD0D0,0xC0C3,0xC0C0,0xC0C0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0x7171,0xD3D3,0xE0E0,0xE0E0,0xF1F2,0xD0E0,0x7171,0x6161,0xF0C3,0xF1F0,0xF0F1,0xE0F0,0xF0D2,0xE0F1,0xD0D0,0xC0D4,0xD0D3,0xC0D3,0xD3D3,0xD0D0,0xD2D3
.word 0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x9F9F,0xD0D0,0xF0F0,0xD0D0,0xC0C0,0xC0C0,0xC080,0xD0D0,0xE0E0,0xC080,0xC0C0,0xE0E0,0xCFCF,0x7070,0x7070,0x8080,0xC0C0,0xBFBF,0xFFFF,0xD0F0,0xAFAF,0xAFAF,0xCFCF,0x6060,0x5050,0x9F9F,0xD0D0,0x9F9F,0x7070,0x7070,0x6060,0x6060,0x9F9F,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x9F9F,0xAFAF,0xBFBF,0xAFAF,0x7070,0x9F9F,0x9F9F,0xBFBF,0x4141,0x7F7F,0x7F7F,0xFEE0,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xF0F0,0xF0F0,0x6161,0x8F8F,0x5050,0xBFBF,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xD0D0,0xFEFE,0xFEFE,0xFFFF,0xFEFE,0xCFCF,0xC0C0,0xC0C0,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0x6060,0xCDCD,0xCFCF,0x6060,0x6060,0x5050,0x5050,0x5050,0x4040,0x8F8F,0x4040,0x6060,0x7F7F,0x7F7F,0x4040,0x5050,0xF3F3,0x7171,0x6161,0xD4C0,0xD361,0x7171,0xC071,0xF0E3,0xD0F0,0x6161,0x6161,0x6161,0x6161,0x6060,0xD0D0,0xD3D3,0xC0C0,0x7171,0x7171,0xCFCF,0xC0C0,0x7171,0x7171,0x7171,0x7171,0x7171,0xF3C0,0xE3E0,0xE1E3,0xFFFF,0xD0E3,0xC0C3,0x7171,0xF3D0,0xF0F0,0xF1F1,0xE0E2,0xF1D2,0xF2F2,0xD0E0,0xD3D0,0xD3D0,0xD3D0,0xD1D0,0xD2E3,0xD2D0
.word 0x7F7F,0x6F6F,0x4040,0x4040,0x4040,0x8F8F,0x9F9F,0x6060,0x8F8F,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0xBFBF,0xD0E0,0xE0E0,0x80D0,0xBFBF,0xC0C0,0x7070,0xAFAF,0xD0D0,0xAFAF,0x7070,0xAFAF,0xC0C0,0x7070,0x7070,0x8080,0xC0C0,0xF0F0,0xFFFF,0xEFEF,0x80D0,0xAFAF,0x7070,0x5050,0x9F9F,0xBFBF,0xAFAF,0x7070,0x9F9F,0x9F9F,0x6060,0x9F9F,0x7070,0x9F9F,0x7070,0x8080,0xAFAF,0xAFAF,0x8080,0xBFBF,0xE0E0,0xE3E0,0xCFCF,0xE3E3,0xD0D0,0xD4D4,0xAFAF,0x4141,0x6060,0xDFDF,0xFEFE,0xDFDF,0xEFEF,0xEFEF,0xFFFF,0xF0F0,0xE0E0,0xC0C0,0x7070,0x6060,0xAFAF,0xDFDF,0xFFFF,0xFEFE,0xE0DE,0xDFDF,0xEFEF,0xFFFF,0xEFEF,0xC0C0,0xBFBF,0xC0C0,0xDDDD,0xBDBD,0xCFCF,0xCDCD,0x6060,0x7070,0xCFCF,0x5050,0x9F9F,0x6060,0x6050,0x4040,0x8F8F,0x5050,0x4040,0x5060,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x6060,0x4040,0xC071,0xD4D0,0x71C0,0xD3C0,0xF2F0,0xE0F3,0xC3C3,0x6161,0x6161,0x6161,0x6060,0x7171,0xD0E0,0xD4D4,0xC0C0,0x7070,0xD0D0,0x7171,0x7171,0x7171,0x7171,0x71D0,0xC0C0,0xE0E3,0xD0E3,0xE0E0,0xE0F0,0xD0D4,0x7171,0xC3C3,0xF0E3,0xF1F1,0xE0F1,0xD2E0,0xF0E1,0xF1F1,0xE3F3,0xE0E3,0xE0E2,0xF0E3,0xF1E2,0xF0F0,0xE1E2
.word 0x6F6F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x5050,0x5050,0x6060,0x5050,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0xF0F0,0xF0F0,0xE0E0,0xC0C0,0xC0C0,0xBFBF,0xC0D0,0x9F9F,0xAFAF,0x6060,0x6060,0x6060,0x7070,0x6060,0xC0C0,0x8080,0xD0D0,0xCFCF,0xF1F1,0xF0F0,0xE0E0,0x8080,0x6060,0x5050,0x7070,0xAFAF,0x6060,0x7070,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0xAFAF,0x8080,0x8080,0x7070,0xAFAF,0xC0C0,0xE0E0,0xDFDF,0xF5F5,0xE3E3,0xBFBF,0xD0D0,0x61D0,0x9F9F,0xF0F0,0xDFDF,0xE0E0,0xCFCF,0xE0E0,0xCFCF,0xEFEF,0xFFFF,0xFFFF,0xF0F0,0xE0F0,0xD0D0,0x6060,0xBFBF,0xFFFF,0xFFFF,0xDFDF,0xFEFE,0xEFEF,0xEFEF,0xEEEE,0xD0D0,0xBFBF,0xD0D0,0xCFCF,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0x9F9F,0xCFCF,0x7070,0x6060,0x6060,0x6060,0x6060,0x5040,0x4040,0x5050,0x7070,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x6060,0xC0C0,0xC0E3,0x7171,0xC071,0xE0D0,0xE2E0,0xC0E0,0x7061,0x5050,0x9F9F,0x5151,0xE0E0,0xE0E0,0xC0C0,0x7171,0xBFBF,0xAFAF,0x8080,0x4040,0x6060,0x7070,0x9F9F,0xD0D0,0xE0E3,0xE3E0,0xE3D0,0xD3D0,0xC0C0,0x7171,0xD061,0xF0E0,0xF0F1,0xE0E2,0xD2D3,0xF1E0,0xF1F0,0xE0E0,0xE0E0,0xE0E3,0xE0E2,0xE0E3,0xF2F0,0xF1F0
.word 0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x4040,0x8F8F,0x9F9F,0x6060,0x9F9F,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0xBEBE,0xEFEF,0xF0F0,0xE0F0,0xD0D0,0x8080,0xAFAF,0x6060,0x6060,0x5050,0x6060,0x6060,0x7070,0x9F9F,0x6060,0x9F9F,0x8080,0xBFBF,0xE0D0,0xEFEF,0xEFEF,0xD0D0,0xBFBF,0x7070,0xAFAF,0x9F9F,0x7070,0x9F9F,0x9F9F,0x7070,0xAFAF,0x9F9F,0x7070,0xBFBF,0x7070,0x8080,0xBFBF,0x8080,0x7070,0x7070,0x8080,0xE0E0,0xCFCF,0xEFEF,0xFFFF,0xE0F3,0xE0E0,0xE4E4,0xD0D0,0xFEFE,0xDFDF,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xBFBF,0xCFCF,0xF0F0,0xEFEF,0xFFFF,0xEFEF,0xE0E0,0xDFDF,0xFFFF,0xFFFF,0xFEFE,0xDFDF,0xEFEF,0xFEFE,0xFFFF,0xE0E0,0xD0D0,0xC0C0,0xCFCF,0xFEFE,0xCFCF,0xDDDD,0xDFDF,0x60C0,0x7070,0x7070,0x7070,0x6060,0x5050,0x5050,0x8F8F,0x5050,0x4040,0x6070,0x8F8F,0x4040,0x4040,0x4040,0x6140,0xC0D0,0x5050,0x7170,0x7071,0x7171,0xC0D0,0xD0D3,0xD0D0,0xD4D0,0x6161,0x6060,0x5040,0xD0D0,0xD0D0,0xC0C0,0x7171,0xBFBF,0x8080,0x8080,0x4040,0x7F7F,0x4040,0x4040,0x4140,0xE361,0xE0E0,0xE0E3,0xD0E3,0xD0D3,0x71D4,0x6171,0xD0D4,0xF1E3,0xF1F1,0xD3E0,0xD1D0,0xD2E0,0xD2D0,0xD3D0,0xD0D3,0xD0D2,0xD0D3,0xE0D2,0xF0F3,0xF1F1
.word 0x6F6F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0xF0F0,0xFFFF,0xFFFF,0xF4F0,0xD3D3,0x6060,0x7070,0x6060,0x6060,0x8F8F,0x5050,0x8F8F,0x6060,0x6060,0x5050,0x5050,0x6060,0x7070,0xCFCF,0xE0F0,0xCFCF,0xE0E0,0x7080,0xAFAF,0x6060,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x8080,0x7070,0xAFAF,0x8080,0x7070,0xBFBF,0x8080,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0xCFCF,0xF5F5,0xFFFF,0xF0F0,0xF0F2,0xE3E0,0xCFCF,0xCFCF,0xE0E0,0xCFCF,0xAFAF,0xCFCF,0xBFBF,0xC080,0x8080,0xAFAF,0xBFBF,0xFEFE,0xEFEF,0xFEFE,0xDFDF,0xEFEF,0xFFFF,0xEFEF,0xFEFE,0xFEFE,0xCFCF,0xFEFE,0xD0D0,0xCFCF,0xC0C0,0xD0D0,0xFEFE,0xDFDF,0xD0D0,0xEEEE,0xCFCF,0xC070,0xD0D0,0x6060,0xBFBF,0x6060,0x6060,0x4050,0x4040,0x5050,0x6060,0x4040,0x5050,0x8F8F,0x4040,0x7070,0xC0C0,0x7171,0xD0D0,0xC0C0,0x7070,0xC071,0xD0D0,0x7070,0xC0C0,0x7070,0x5151,0x8F8F,0xD0D0,0x8080,0x7171,0x7171,0xC0C0,0x7070,0x6060,0x8F8F,0x4040,0x4040,0x7F7F,0x8F8F,0xE0C0,0xF0F3,0xF0F0,0xE3F0,0xC3D0,0xD4C0,0x7171,0xC3C0,0xE0E0,0xF1F1,0xD2E0,0xE0D2,0xD0D3,0xD0D3,0xC0D3,0xC3C2,0xC3C1,0xD3C1,0xD0C3,0xE0E3,0xE1E2
.word 0x7F7F,0x6F6F,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x5050,0x8F8F,0x9F9F,0x5050,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0xDFDF,0xFFFF,0xFFFF,0xDFDF,0xD0D0,0x7070,0x7070,0x5050,0x4040,0x5050,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x6060,0x5050,0x9F9F,0xBFBF,0xF3F3,0xF3F3,0xF4F4,0xBFBF,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x6060,0xAFAF,0x7070,0xC0C0,0xBFBF,0x7070,0xC0C0,0x6060,0x7070,0xAFAF,0xCFCF,0xEFEF,0xFFFF,0xFFFF,0xF0F0,0xF3F3,0xE4E4,0xBFBF,0xC0C0,0x7070,0x8080,0xC0C0,0x7070,0xAFAF,0xBFBF,0xC0C0,0xD0D0,0xD0D0,0xCFCF,0xDFDF,0xDFDF,0xE0E0,0xFFFF,0xEFEF,0xDFDF,0xDFDF,0xBFBF,0xEFEF,0xEEEE,0xD0D0,0xBFBF,0xBFBF,0xCFCF,0xDFDF,0xFEFE,0xD070,0xE0FE,0xAFAF,0xC0C0,0x7070,0xC0C0,0x6060,0x5050,0x8F8F,0x5050,0x4040,0x7060,0x4040,0x4040,0x4040,0x4040,0x5050,0x7071,0x60D0,0x6060,0x7171,0x8071,0x7171,0xC0C0,0xC0C0,0x7060,0x5050,0x8F8F,0xD061,0xE0E0,0xC0C0,0xC0C0,0x7171,0x8080,0x7171,0x6060,0x4040,0x4040,0x7F7F,0x4040,0x4040,0xF3E4,0xF0F0,0xF3F0,0xE0F0,0xC0C0,0x8080,0x71D4,0xD071,0xE2E2,0xFFFF,0xD0E3,0xD3D0,0xC3D0,0xC3C0,0xD3C1,0xD0C1,0xD0D2,0xD0D2,0xC3D0,0xD3D0,0xE1D1
.word 0x7F7F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x4040,0x5050,0x6060,0x9F9F,0x5050,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x9F9F,0xFFFF,0xFFFF,0xF1F1,0xE0E0,0xAFAF,0xAFAF,0x7070,0x6060,0x5050,0x9F9F,0x4040,0x4040,0x4040,0x5151,0x8F8F,0x5050,0x8F8F,0x5050,0xAFAF,0xEFEF,0xFFFF,0xF4F4,0xD3D3,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0xC0C0,0xAFAF,0x5060,0x7070,0xAFAF,0xC0C0,0xC0C0,0xC0C0,0xAFAF,0x7070,0x7070,0xC0C0,0xCFCF,0xDFDF,0xDFDF,0xF0F0,0xF0F0,0xF3F3,0xDFDF,0xD0D0,0xAFAF,0x6060,0xAFAF,0x6070,0x9F9F,0xAFAF,0xC0C0,0xAFAF,0xBFBF,0xCFCF,0xC0C0,0xBFBF,0xC0C0,0xBFBF,0xD0D0,0xBFBF,0xC0C0,0xD0D0,0xD0D0,0xCFCF,0xEFEF,0xCFCF,0xC0C0,0xBFBF,0xAFAF,0xCFCF,0xDFDF,0xC0C0,0xDFDF,0xD0D0,0xD0D0,0xD0D0,0x7070,0xC0C0,0x5050,0xD0D0,0x4040,0x5050,0x6060,0x4040,0x8F8F,0x4040,0x4040,0x8080,0x8080,0xC071,0xC0C0,0xC0C0,0xD0D0,0x70C0,0xC0C0,0x5050,0x4040,0x4141,0x5151,0xE0E0,0xC0C0,0xC0C0,0x4141,0x71C0,0xC0C0,0x7171,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4141,0xF0F0,0xF0F0,0xF0F0,0xD0D0,0x71C3,0x71C3,0x7171,0x71E4,0xF1D3,0xE1F2,0xD0D2,0xD0D3,0xC0C3,0xC1D3,0xD0D2,0xC0D3,0xD1C3,0xD0E3,0xD3D3,0xD3D1,0xD1D0
.word 0x4040,0x6F6F,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x4040,0x8F8F,0x8F8F,0x5050,0x9F9F,0x5050,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x9F9F,0xCECE,0xEFEF,0xDFDF,0xE4F1,0xAFAF,0xC0C0,0xC0C0,0x6060,0x4040,0x8F8F,0x4050,0x4040,0x4040,0x7F7F,0x7F7F,0x5151,0x8F8F,0x4040,0x8F8F,0xAFAF,0xBFBF,0xDFDF,0xEFEF,0xF3F3,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x8080,0x7070,0x9F9F,0x6060,0x7070,0xAFAF,0xC0C0,0xAFAF,0x7070,0x7070,0xAFAF,0xD0D0,0xCFCF,0xCFCF,0xBFBF,0xDFDF,0xF0F0,0xF0F0,0xD0F0,0xBFBF,0x6060,0x6060,0x7070,0x9F9F,0x7070,0x8080,0xAFAF,0xBFBF,0xBFBF,0xD0D0,0x8080,0x7070,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0xC0C0,0xBFBF,0xEEEE,0xCFCF,0xBFBF,0xBFBF,0xC0C0,0xBDBD,0xE0E0,0xCFCF,0xBFBF,0xCFCF,0xDFDF,0xD0CD,0xC0C0,0x7070,0x9F9F,0xD0D0,0x6060,0x4040,0x6060,0x6060,0x5050,0x8F8F,0x4040,0x5050,0x8070,0xC0C0,0x7171,0xC071,0xD0D0,0xD0C0,0xD0D0,0x8080,0x4040,0x4040,0x8F8F,0xF0F0,0xD0D0,0x5050,0x4141,0x6161,0xC071,0x5050,0x4040,0x7F7F,0x4040,0x4040,0x5140,0xD1D1,0xF0F0,0xF0F0,0xF0F0,0xD4F0,0xD470,0x7171,0xC071,0x7171,0xD371,0xD3D0,0xD3D0,0xC3C3,0xC3C0,0xD2C1,0xC3D0,0xC3C0,0xE3C0,0xD3D0,0xC1D0,0xD3D0,0xD3C1
.word 0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x5050,0x9F9F,0x5050,0x5050,0x4040,0x8F8F,0x5050,0x8F8F,0x5050,0x5050,0x7070,0xAFAF,0xF0F0,0xBFBF,0xF4F4,0xC0C0,0xAFAF,0x8080,0x6060,0x4040,0x4040,0x5050,0x5050,0x8F8F,0x4040,0x4141,0x8F8F,0x5252,0x4141,0x5151,0x9F9F,0x9F9F,0xCFCF,0xEFEF,0xE0F0,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0xC0C0,0xAFAF,0x8080,0xAFAF,0xAFAF,0x7070,0xE0D0,0xCFCF,0xDFDF,0xF4F4,0xF3F3,0xCFCF,0xF0F0,0xE0E0,0xBFBF,0x6161,0x4040,0x8F8F,0x6060,0x6060,0x9F9F,0xAFAF,0xBFBF,0x7070,0x7070,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x7070,0xBFBF,0xD0D0,0xBFBF,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xC0C0,0xBFBF,0xC09D,0xD0C0,0xC0C0,0xC0C0,0xD0B0,0xE0E0,0xD0D0,0xC0C0,0xD0D0,0xC0C0,0x7070,0xF0F0,0xEFEF,0x5050,0x4040,0x6060,0x50D0,0x4040,0x5050,0x7070,0x7070,0xD0D0,0xE0E0,0x7070,0xD0D0,0xC0C0,0x70D0,0x6060,0x4141,0x8F8F,0x4141,0xEFEF,0x6161,0x8F8F,0x7F7F,0xC0C0,0xD3D4,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0xD0D0,0xF0E0,0xF0F0,0xF0F0,0xF0F0,0xC0D0,0xC071,0x7171,0x7171,0x7171,0xC0C3,0xD0D3,0xC0C3,0xC3C1,0xC1C2,0xD2C1,0xC1D2,0xC0C3,0xC0C3,0xD0D3,0xD3C3,0xC0C3,0xC1C3
.word 0x6F6F,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x6F6F,0x4040,0x4040,0x8F8F,0x6060,0x5050,0x8F8F,0x8F8F,0x5050,0x9F9F,0x5050,0x4040,0x9F9F,0x7070,0xBFBF,0xE4E4,0xBFBF,0xCFCF,0xD3D3,0x7070,0xC0C0,0x5050,0x8F8F,0x4040,0x8F8F,0x6060,0x4040,0x7F7F,0x4141,0x9F9F,0xAFAF,0x5151,0xAFAF,0x9F9F,0x9F9F,0xCFCF,0xDFDF,0xDFDF,0xD0D0,0x7070,0x9F9F,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0xAFAF,0xC0C0,0xAFAF,0x6060,0x6060,0xE0E0,0xDFDF,0xCFCF,0xF4F4,0xDFDF,0xEFEF,0xBFBF,0xCFCF,0xD0E0,0xC0C0,0xAFAF,0x4141,0x5050,0x6060,0x5050,0x6060,0x7070,0xBFBF,0xBFBF,0x6060,0x7070,0xC0C0,0x80AE,0x8080,0xC0C0,0xD0D0,0xD0D0,0xCFCF,0xE0E0,0xCFCF,0xBFBF,0xD0D0,0xD0D0,0xBFBF,0x7070,0xC0C0,0x8080,0x8080,0x70C0,0xD0C0,0xC0D0,0x8080,0x7070,0xD0D0,0xD0D0,0xD0D0,0xF0F0,0xE0FD,0x9F9F,0x4040,0x6060,0xF0F0,0x4040,0x4040,0x6050,0x7070,0xD080,0xD0E0,0xC070,0xD0D0,0x6070,0x5050,0x9F9F,0x4141,0x4041,0x5050,0xAFAF,0x5050,0x4040,0x6140,0xAFAF,0xF0D0,0xAFAF,0x4040,0x7F7F,0x5040,0xD1D1,0xF3F1,0xF0F0,0xF0E0,0xF0F3,0xD3E0,0x7171,0x6161,0x7171,0x6271,0x7171,0xE3C0,0xD3E0,0xD3D0,0xC0D3,0xC281,0xD1C2,0xC0D2,0xC3C2,0xC381,0xC3C0,0xC0C3,0xC381,0xC381
.word 0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x6F6F,0x4040,0x6F6F,0x7F7F,0x4040,0x8F8F,0x5050,0x5050,0x5050,0x8F8F,0x6060,0x4040,0x4040,0x6060,0x6060,0xBEBE,0xFFFF,0xE4E4,0xBFBF,0xC0C0,0xD0D0,0x5061,0x8F8F,0x4040,0x4141,0x5151,0x4040,0x8F8F,0x4040,0x5050,0x9F9F,0x6262,0x9F9F,0xAFAF,0x5151,0x9F9F,0xCFCF,0xBFBF,0xCFCF,0xCFCF,0x8080,0x7070,0x9F9F,0xAFAF,0x8080,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0x8080,0x8080,0x8080,0xC0C0,0xBFBF,0xF0F0,0xE4F0,0xCFCF,0xDFDF,0xDFDF,0xCFCF,0xBFBF,0x9F9F,0xF0F0,0xE3E3,0xAFAF,0x9F9F,0x4040,0x5050,0x5050,0x9F9F,0x6060,0xAFAF,0x8080,0x9F9F,0x6060,0x7070,0x6060,0x8080,0xBFBF,0xD0D0,0xBFBF,0xDFDF,0xFEFE,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0x7070,0x6070,0x8080,0x8090,0xC0C0,0x8080,0xD0D0,0xC0C0,0xB0B0,0xC0C0,0xE0D0,0xEEEE,0xCFCF,0xF0F0,0xF0F0,0x6060,0x6060,0x7070,0xE0D0,0x7070,0xAFAF,0x7070,0x6060,0x7070,0xD0D0,0x8080,0xE0E0,0x9F9F,0x6060,0x5051,0x5151,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0xD0D0,0x6161,0xAFAF,0x8F8F,0x7F7F,0x6040,0xE3E3,0xF0E0,0xF0F0,0xE0E0,0xE0E0,0xD0E0,0xD0D3,0x71D0,0x6161,0xD4D4,0x6161,0xD371,0xE3E3,0xD3E0,0xC3C0,0xC2D0,0xC2C3,0xD2C1,0xD3D1,0x81C2,0x71C3,0x71C3,0xC371,0x81C3,0x81C3
.word 0x4040,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x6F6F,0x7F7F,0x4040,0x6060,0x9F9F,0x6060,0x6060,0x9F9F,0x8F8F,0x5050,0x5050,0x9F9F,0xCFCF,0xDFDF,0xE4E4,0xBFBF,0xC0C0,0xE4E4,0xC0C0,0x4141,0x4141,0x7F7F,0x4141,0x4040,0x5050,0x8F8F,0x9F9F,0x7070,0x5151,0x6161,0xAFAF,0x7171,0xAFAF,0x9F9F,0xD4D4,0xAFAF,0xBFBF,0x9F9F,0x6060,0x7070,0x7070,0xBFBF,0xC0C0,0x7171,0x8080,0x8080,0x7070,0xAFAF,0xAFAF,0xC0C0,0xE0E0,0xE0E0,0xFFFF,0xEFEF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0xC0C0,0xE070,0xF0F0,0xBFBF,0x7070,0x4141,0x4040,0x4040,0x6060,0x6060,0xAFAF,0x8080,0xC0C0,0x7070,0x7070,0x7070,0xC0C0,0xBFBF,0xC0C0,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xDDDD,0xCFCF,0xCFCF,0x7070,0x6060,0x6060,0xC0C0,0x7070,0x8090,0xD080,0xC0D0,0x70C0,0x8080,0x7070,0xBDBD,0xCFCF,0xD080,0xF0F0,0xDFDF,0xDFDF,0x9F9F,0xD0D0,0x7070,0xD0E0,0xD0D0,0x7070,0x7070,0x7070,0xD0D0,0xC0C0,0xAFAF,0x6060,0x6161,0x9F9F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x6F6F,0xE0D3,0x7171,0x5050,0x4040,0x5140,0xE0E0,0xF0E0,0xF0F0,0xD0E0,0xD0D0,0xD0D3,0xD0C3,0xD0D0,0x7171,0x7161,0x7171,0xC371,0xE3D0,0xE0E0,0xC0D2,0xC0C3,0xC1C3,0xB281,0xC1C2,0xC1C1,0xC3C1,0x8181,0xC371,0x8171,0xC371,0xC371
.word 0x7F7F,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x5050,0x9F9F,0x6060,0x5050,0x9F9F,0x6060,0x6060,0x5050,0x9F9F,0x6060,0xBFBF,0xBFBF,0xD0F0,0xD0D0,0x7171,0xD0D0,0x4141,0x8F8F,0x4141,0x4141,0x8F8F,0x4040,0x8F8F,0x6060,0x6060,0x5050,0x5050,0x9F9F,0x7070,0xAFAF,0xE4E4,0xD4D4,0xD3D3,0xE4E4,0x7070,0x6060,0x6060,0xAFAF,0x7070,0xAFAF,0xBFBF,0xD0D0,0xBFBF,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0xCFCF,0xCFCF,0xDFDF,0xCFCF,0xAFAF,0x9F9F,0xAFAF,0x5050,0x9F9F,0xCFCF,0xD0F0,0x5070,0x8F8F,0x9F9F,0x8F8F,0x4041,0x8F8F,0x9F9F,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0xD0D0,0xAFAF,0xC0C0,0x8070,0x8080,0x8080,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0x7070,0xAFAF,0x5050,0x7070,0x7070,0xC080,0xD0D0,0xC0C0,0xD0D0,0xC080,0x8080,0xC0C0,0xD0D0,0xBFBF,0xBFBF,0xE0E0,0xC0C0,0xCFCF,0xC0C0,0xF0F0,0xC070,0xF0F0,0xC0C0,0xC0C0,0x7070,0x9F9F,0x6060,0x7070,0x5050,0x6161,0x6161,0x4051,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0xBFBF,0x5050,0x7F7F,0x5050,0xD0D0,0xE0E0,0xE0E0,0xD0D0,0xD0D0,0xC0C3,0xD3C0,0xD4D0,0xD4D0,0xC0C0,0x6161,0xD071,0xC0D4,0xE0D4,0xE0E3,0x71D3,0xC372,0xD3C0,0x81C2,0xC2C2,0xD3C2,0xC0C1,0x72C3,0x7171,0x71C3,0x71C3,0x8171
.word 0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x7F7F,0x5050,0x9F9F,0x6060,0x5050,0x9F9F,0x5050,0x6060,0x6060,0x7070,0xAFAF,0x8F8F,0xDFDF,0xE3E4,0x7171,0xAFAF,0x9F9F,0x4041,0x8F8F,0x8F8F,0x4141,0x4040,0x6060,0x7070,0x7070,0x6161,0x9F9F,0x6060,0x9F9F,0xAFAF,0xF2F2,0xE4E4,0xCFCF,0xF0F0,0xAFAF,0x6060,0x6060,0x7070,0xBFBF,0xC0C0,0x7070,0xBFBF,0xD0E4,0x7070,0xAFAF,0x6060,0x9F9F,0xAFAF,0xC0C0,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xD0D0,0xEFEF,0xEFEF,0x9F9F,0x5050,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x6060,0x7070,0xC0C0,0x9F9F,0xC0C0,0xAFAF,0xC0C0,0x8080,0xAFAF,0x8080,0x7070,0x8080,0x7070,0xD0D0,0xCFCF,0x7070,0xD0D0,0xC0C0,0xD0D0,0xAFAF,0x7070,0x8080,0xC0C0,0xC0C0,0x8080,0x8080,0xD070,0x8080,0x8080,0xD0D0,0xE0F0,0xD0D0,0xCFCF,0xCFCF,0xE0E0,0x9F9F,0xF0D0,0xD0F0,0xC0C0,0x8080,0x5060,0x5050,0x8F8F,0x5151,0x5151,0x5050,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0xE3E3,0xF1F1,0xD0D0,0xD0C0,0xC0C0,0xC0C3,0xD0C3,0xD0D4,0xC0C0,0x7171,0x6161,0x7171,0x7171,0xC3D0,0xE0D0,0xD0F3,0x7171,0x8171,0xD0C3,0xB2C3,0x81B2,0xC0C3,0xC3C2,0xC3C0,0x71C3,0x7171,0x7171,0x72C3
.word 0x4040,0x7F7F,0x8F8F,0x5050,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x8F8F,0x6060,0x6060,0x5050,0x6060,0x5050,0x9F9F,0x6060,0x6060,0x5050,0x9F9F,0xF3F3,0xD0D0,0xAFAF,0x7171,0x8080,0x5151,0x4040,0x4040,0x5151,0x8F8F,0x6060,0x9F9F,0xC0C0,0x7070,0x7070,0x8F8F,0x4040,0x9F9F,0xBFBF,0xCFCF,0xCFCF,0xF4F4,0xAFAF,0x8080,0xAFAF,0xBFBF,0xC0C0,0xC0C0,0xAFAF,0xBFBF,0xBFBF,0x6060,0x6060,0x6060,0x6060,0xC0C0,0xAFAF,0x9F9F,0x9F9F,0x5151,0x4141,0x8F8F,0xAFAF,0x4040,0x8F8F,0xEFEF,0xF0F0,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x9F9F,0x6060,0xAFAF,0xAFAF,0x6060,0x7070,0x7070,0xAFAF,0x6070,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0xCFCF,0xC0C0,0x9F9F,0xCFCF,0xAFAF,0xCFCF,0x7070,0x7070,0xC0C0,0xC0C0,0x80C0,0x8080,0x8080,0xD0D0,0x80C0,0x8080,0xBFBF,0xF0E0,0xE0F0,0xCFCF,0xC0EE,0xAFAF,0x7070,0x7070,0xF0F0,0xCFCF,0x9F9F,0x8F8F,0x7F7F,0x5151,0x8F8F,0x5050,0x5050,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0xBFBF,0xF3F3,0x71D0,0x71D3,0x71D4,0x7171,0xAFAF,0xC0C0,0x7171,0xD0D0,0x6161,0x6161,0xC071,0xD3D3,0xF3D0,0xE0E0,0x6171,0x7161,0xC1C3,0x81C3,0xB381,0xC381,0xC0C3,0xD0C3,0xC371,0x7171,0x71C3,0x7171
.word 0x4040,0x4040,0x5050,0x5050,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x5050,0x9F9F,0x6060,0x5050,0x8F8F,0x6060,0x6060,0xAFAF,0x7070,0xBFBF,0xCFCF,0xE2F0,0xE0D0,0xD0F3,0xAFAF,0xC0C0,0x5151,0x7F7F,0xC0C0,0x4141,0x8F8F,0x5050,0x6060,0xAFAF,0x6060,0x4141,0x8F8F,0x9F9F,0xAFAF,0xCFCF,0xCFCF,0xBFBF,0x9F9F,0xBFBF,0xC0C0,0xD0D0,0xBFBF,0x7070,0xC0C0,0x7070,0x6060,0x9F9F,0x6060,0x7070,0x9F9F,0x9F9F,0x7171,0x5151,0x5151,0x8F8F,0x8F8F,0x5050,0x4040,0x8F8F,0x7F7F,0xCFCF,0x5050,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x5050,0xAFAF,0x8080,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0x9F9F,0xAFAF,0x7070,0x7070,0xAFAF,0xE0E0,0xD0E0,0xD0D0,0xC0C0,0xE0DD,0x7070,0xC0C0,0x6060,0x7070,0xC0C0,0x7070,0x7070,0x7080,0x8080,0xC0C0,0x7080,0xD0D0,0xB0C0,0xE0D0,0xF0F0,0xCFCF,0xDFDF,0xD0D0,0xD0E0,0x7070,0x9F9F,0xBFBF,0x4040,0x4040,0x4040,0x4141,0x4141,0x5050,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0xF0F0,0xE0E0,0xC0C0,0xC0C0,0xAFAF,0x4040,0x4040,0x4040,0xD0D0,0x71C0,0x6060,0x7171,0xC3C3,0xD3C0,0xF3E0,0x71E0,0x6161,0x7162,0xD0C3,0x71C3,0x81B3,0x8181,0xC381,0xC3C3,0x71D0,0x71D4,0x7171,0x71C3
.word 0x5050,0x5050,0x6060,0x9F9F,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x6F6F,0x7F7F,0x5050,0x5050,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0xAFAF,0xD0D0,0xFEFE,0xF0F0,0xDFDF,0xE1E1,0xC0C0,0xD0D0,0x6060,0x4040,0x9F9F,0x6060,0x9F9F,0x5050,0x8F8F,0x5050,0x6060,0x8F8F,0x4141,0x6060,0xE5E5,0xCFCF,0xAFAF,0x9F9F,0x7070,0x7070,0xAFAF,0xBFBF,0xC0C0,0xAFAF,0xAFAF,0xAFAF,0x7070,0x6161,0x6060,0xAFAF,0x7070,0x7070,0xBFBF,0x9F9F,0x4141,0x4141,0x4040,0x8F8F,0x7F7F,0x4141,0x4040,0x9F9F,0x6060,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x9F9F,0xD0D0,0x7070,0x7070,0x6060,0x7070,0x5060,0x6060,0x9F9F,0x7070,0x7070,0xBFBF,0xC0C0,0x6060,0x8F8F,0xCFCF,0x6060,0xAFAF,0x9F9F,0x7070,0xAFAF,0x8080,0x8080,0x7070,0x7070,0xC0D0,0x7070,0x8080,0x8070,0xD0C0,0xF0F0,0xDFDF,0xDDDD,0xC0C0,0xCFCF,0x5050,0x4050,0x5050,0x8F8F,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0xAFAF,0x5050,0x6161,0x6060,0x4040,0x7F7F,0x8F8F,0x7171,0xC0C0,0x5151,0x7171,0x7171,0xE3D0,0xF0E0,0xE3E3,0x6161,0x6161,0xD371,0x71C0,0x7271,0xC372,0x7171,0xC071,0xC0C3,0xD3C3,0x7171,0x7271
.word 0x5050,0x6060,0x5050,0x6060,0x9F9F,0x5050,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x6050,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0xAFAF,0xCFCF,0xF0F0,0xF0F0,0xC0E0,0xC0C0,0x8080,0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x7F7F,0x9F9F,0xAFAF,0x5151,0x4141,0x7F7F,0x9F9F,0xD0D0,0x7070,0xD0D0,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0xC0C0,0xAFAF,0x6060,0x9F9F,0xAFAF,0x6161,0x5151,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x4040,0x6060,0x9F9F,0x5050,0x5050,0x6060,0x8F8F,0xC0C0,0x6060,0x9F9F,0x6060,0xD0D0,0x6060,0x6060,0x6060,0x7070,0x4050,0x5050,0x7070,0xAFAF,0x7070,0xBFBF,0x8080,0x7060,0xC0C0,0xBFBF,0x7070,0xC0C0,0x8080,0xC0C0,0xE0E0,0xDFDF,0xBFBF,0xD0D0,0x7070,0xC0C0,0x8F8F,0x4040,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x6161,0x4040,0x7F7F,0x4040,0xC040,0xB0B0,0x7171,0x5151,0xC0C0,0x7171,0xF1E3,0xF0F0,0x6161,0x6161,0x6161,0xD371,0xD4D4,0x7171,0x7171,0x7172,0x7172,0xC371,0xF0D0,0xD4D4,0x7171
.word 0x5050,0x5050,0x5050,0x5050,0x7070,0x6060,0x8F8F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x8F8F,0x9F9F,0x7070,0x6060,0x6060,0x6060,0xAFAF,0x6060,0x5050,0x7070,0xD0D0,0xBFBF,0xF0F0,0x7070,0x6060,0xC0C0,0x6060,0x9F9F,0x4040,0x8F8F,0x5050,0x5050,0x5050,0x4040,0x4040,0x5050,0x7070,0xAFAF,0x5151,0x8F8F,0x4040,0x8F8F,0xBFBF,0xD0D0,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x8080,0xAFAF,0x6060,0x8F8F,0x5050,0x7070,0x6060,0x8F8F,0x4040,0x4141,0x4141,0x4040,0x8F8F,0x4040,0x7F7F,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x6F6F,0x7F7F,0x9F9F,0x6060,0x5060,0x8F8F,0x5050,0x5050,0xBFBF,0x5050,0x7070,0x9F9F,0xC0C0,0x5050,0x6060,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x8080,0xAFAF,0xC0C0,0x7070,0x7070,0x7070,0x8080,0x7070,0xC0C0,0x7070,0xD0D0,0xE0E0,0xE0E0,0xC0C0,0xC0C0,0x5050,0x9F9F,0x4040,0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0xC061,0x6060,0x4040,0x7F7F,0x4141,0x7171,0xC0C0,0x6060,0x7171,0x6161,0xF0F0,0xDFDF,0x6161,0x6161,0x6261,0x6161,0x6171,0x7271,0x7271,0x7171,0x7171,0x7161,0xE4D4,0x71C0,0x7171
.word 0x5050,0x6060,0x5050,0x6060,0x9F9F,0x7070,0x5050,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x5050,0x6060,0x9F9F,0x6060,0x6060,0x9F9F,0x7070,0x7070,0x6060,0x9F9F,0xAFAF,0xD0D0,0xBFBF,0xBFBF,0x7070,0x8080,0xAFAF,0x7070,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x5151,0x8F8F,0x9F9F,0x5050,0x9F9F,0x4141,0x4141,0x8F8F,0xBFBF,0xCFCF,0x8080,0x7070,0x7070,0xAFAF,0xC0C0,0xAFAF,0x7171,0x7070,0x4040,0x8F8F,0x9F9F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x4040,0x7F7F,0x5050,0x7070,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x5050,0xBFBF,0x7070,0x5050,0x6060,0x5050,0x5050,0x5050,0x4040,0x4040,0x4040,0x5050,0x8F8F,0xAFAF,0xC070,0xD0D0,0x7070,0x7070,0xBFBF,0xC0C0,0x7070,0x7070,0xD0D0,0xBFBF,0xC0C0,0xE0E0,0xCFCF,0x7070,0x6060,0x5060,0x5050,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0xAFAF,0x8F8F,0x4040,0x4040,0x7070,0xC0C0,0x7171,0x7171,0x7171,0xC0C0,0xBFBF,0x6161,0x5161,0x6161,0x6161,0x7161,0x7161,0x7171,0x7171,0xD471,0x7161,0x7162,0x6171,0x7171,0xD471
.word 0x6060,0x5050,0x5050,0x6060,0x6060,0x7070,0x5050,0x9F9F,0x8F8F,0x5050,0x5050,0x5050,0x6060,0x9F9F,0x6060,0x7070,0x5060,0x5050,0x6060,0x6060,0xAFAF,0x7070,0x8F8F,0xE0E0,0xBFBF,0x7070,0x6060,0x7070,0xAFAF,0x7070,0x5050,0x9F9F,0x7070,0x9F9F,0x4040,0x4040,0x7F7F,0x9F9F,0x6060,0x7070,0x9F9F,0xAFAF,0x5252,0x7F7F,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5050,0x4040,0x4040,0x4141,0x4141,0x4141,0x4141,0x4141,0x8F8F,0x4040,0x8F8F,0xC0C0,0x5050,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x6F6F,0x7F7F,0x6F6F,0x4040,0x4040,0x4040,0x8F8F,0x5040,0x5050,0x9F9F,0x5050,0x5050,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x9F9F,0x9F9F,0xC0C0,0x7070,0xAFAF,0xC0C0,0x7070,0x7070,0xAFAF,0xBFBF,0xAFAF,0x7070,0x6060,0x7070,0x6060,0x5050,0x5050,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7171,0xD0D0,0x7171,0x7171,0x7171,0x61C0,0x6161,0x6161,0x6161,0x6161,0x6262,0x6261,0x7161,0x61D4,0x6171,0x7171,0x7171,0x7171,0xD471,0x7171
.word 0x6060,0x5050,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x5050,0x8F8F,0x6060,0x6060,0x6060,0x7070,0x6060,0x7070,0x6060,0x6060,0x6060,0xAFAF,0x7070,0x4040,0x9F9F,0x7070,0x6060,0x6060,0x6060,0x6050,0x4040,0x4040,0x6050,0xAFAF,0x4040,0x4040,0x8F8F,0x4040,0x5050,0x8F8F,0x8080,0x6262,0x9F9F,0x4141,0x4141,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x6060,0xAFAF,0xC0C0,0xD3D3,0x7171,0xAFAF,0x6161,0x6060,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x7F7F,0xD0D0,0x7070,0x4040,0x8F8F,0x4040,0x6F6F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x4040,0x8F8F,0x5050,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x5050,0x6060,0xBFBF,0x7070,0xBFBF,0x7070,0xAFAF,0x7070,0xAFAF,0x6060,0x8F8F,0x5050,0x6060,0x6060,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0xC0C0,0xC0C0,0xC0C0,0x7171,0xC0C0,0xAFAF,0x5151,0x6151,0x6161,0x6161,0x6161,0x6161,0x6171,0x6161,0x7171,0x7171,0x7171,0x7171,0x7171,0x7171
.word 0x5050,0x5050,0x5050,0x6060,0x6060,0x6060,0x6060,0x7070,0x9F9F,0x7070,0x7070,0x6060,0x6060,0x6060,0x8080,0xAFAF,0xC0C0,0x7070,0x6060,0x9F9F,0x7070,0x5050,0x8F8F,0x9F9F,0x7070,0xBFBF,0x7171,0x5050,0x8F8F,0x5050,0x4040,0x8F8F,0x6060,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x6060,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xD0D0,0xAFAF,0x7171,0x9F9F,0x9F9F,0xAFAF,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x6F6F,0x5050,0x5070,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x5050,0x7070,0x7070,0x6060,0x6060,0xC0C0,0x6060,0x5060,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7171,0xC0C0,0x6161,0x7171,0x6161,0x6161,0x6161,0x6161,0x6151,0x6161,0x6161,0x7161,0x6262,0x7171,0x6171,0x7272,0x7171,0x7271,0x7171,0x7171
.word 0x6060,0x5050,0x5050,0x6060,0x6060,0x6060,0x9F9F,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x6060,0x6060,0xAFAF,0x7070,0xAFAF,0x7070,0x9F9F,0x8080,0x7070,0x7070,0x9F9F,0xC0C0,0xAFAF,0x8080,0x7171,0x6060,0x5050,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x9F9F,0x6060,0x4040,0x4141,0x4040,0x4141,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x6060,0xAFAF,0xAFAF,0x8080,0xAFAF,0x5151,0x9F9F,0x6161,0x9F9F,0x8F8F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0xAFAF,0x6060,0x6060,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x4040,0x7F7F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x6060,0x9F9F,0x6060,0x6060,0x8F8F,0x5050,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x6040,0x8080,0x7171,0x7171,0x7171,0x6161,0x5161,0x5151,0x6161,0x6161,0x6161,0x6161,0x6161,0x7162,0x6161,0x7162,0x7171,0x7171,0x7171,0x7171,0x7171
.word 0x5050,0x5050,0x9F9F,0x5050,0x7070,0x6060,0x6060,0x6060,0xAFAF,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x8080,0x9F9F,0x6070,0x6060,0x6060,0x7070,0xAFAF,0xC0C0,0xAFAF,0x8080,0x9F9F,0x7070,0xAFAF,0x6060,0x5060,0x5050,0x4040,0x7F7F,0x4040,0x4141,0x7F7F,0x4040,0x4040,0x4040,0x6060,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xC0C0,0xAFAF,0x9F9F,0x9F9F,0x7070,0x7070,0x7070,0x9F9F,0x5050,0x8F8F,0x9F9F,0x9F9F,0x4141,0x4040,0x4040,0x6F6F,0x4040,0x6F6F,0x4040,0x4040,0x6F6F,0x4040,0x7070,0xD0D0,0xBFBF,0xE0E0,0x5050,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x5050,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x5050,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0xC0C0,0xC0C0,0x6161,0x6161,0x7070,0x5151,0x5161,0x5161,0x6161,0x6161,0x6161,0x6161,0x6161,0x7161,0x7171,0x7171,0x7271,0x7171,0x7171,0x71D4
.word 0x5050,0x5050,0x6060,0x6060,0x9F9F,0x6060,0x5050,0x6060,0x6060,0xAFAF,0x7070,0x6060,0x7070,0x7070,0xBFBF,0x7080,0x9F9F,0x5050,0x6060,0x7070,0x7070,0xAFAF,0x6060,0xAFAF,0x6070,0x9F9F,0x7070,0x9F9F,0x9F9F,0x6060,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x8F8F,0x5050,0x7F7F,0x4141,0x4141,0x4141,0x9F9F,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x7070,0xAFAF,0x5050,0x4040,0x4040,0x4141,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x4040,0x6F6F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x4040,0x9F9F,0xAFAF,0xF0F0,0xCFCF,0xD0D0,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x6060,0x7171,0x7171,0x7070,0x6161,0x5151,0x5161,0x5151,0x6161,0x5151,0x6161,0x6161,0x6162,0x6161,0x7161,0x6161,0x6271,0x7171,0x7171,0x7171,0x7161
.word 0x8F8F,0x6060,0x9F9F,0x5050,0x6060,0x5050,0x6060,0x8F8F,0x6060,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x6060,0x7070,0x7070,0x7070,0x6060,0x5050,0x6060,0x9F9F,0x6060,0x7070,0x7070,0x7070,0x5050,0x8F8F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4141,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0x8F8F,0x4040,0x8F8F,0xAFAF,0x6060,0x6060,0x9F9F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4040,0x4040,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x4040,0x5050,0x7070,0xD0D0,0xE0E0,0xE0E0,0x5050,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7070,0x7171,0x7171,0x6161,0x5151,0x5151,0x5151,0x5161,0x6161,0x6161,0x6161,0x6161,0x6262,0x6171,0x7162,0x7171,0x7171,0x6171,0x71D4,0x7171
.word 0x5050,0x7070,0x6060,0x6060,0x6060,0x5050,0x5050,0x5050,0x6060,0x6060,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x8080,0x6060,0xAFAF,0x8080,0xBFBF,0x7070,0x8F8F,0x4040,0x5050,0x9F9F,0x6060,0x9F9F,0x6060,0x9F9F,0x4040,0x4040,0x6F6F,0x7F7F,0x5151,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x4141,0x4141,0x4040,0x9F9F,0x5151,0x4141,0x8F8F,0x8F8F,0x6060,0x9F9F,0x6060,0x9F9F,0x8F8F,0x4040,0x4040,0x5050,0x5151,0x8F8F,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x6F6F,0x6F6F,0x4040,0x9F9F,0x7070,0xBFBF,0xD0D0,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x7F7F,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7070,0x7171,0x6171,0x6060,0x5151,0x5151,0x6161,0x5151,0x6161,0x5151,0x6161,0x6161,0x7161,0x6161,0x6171,0x7171,0x7171,0x7171,0x7161,0x71AF
.word 0x4040,0x5050,0xAFAF,0x5050,0x5050,0x5050,0x5050,0x5050,0x7070,0x9F9F,0x6060,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x8080,0x7070,0x9F9F,0x8080,0x7070,0x6060,0x5050,0x7F7F,0x7F7F,0x4040,0x8F8F,0x6060,0x5050,0x5050,0x4141,0x4040,0x4040,0x8F8F,0x5050,0x7F7F,0x6F6F,0x4040,0x8F8F,0x7F7F,0x4141,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x4141,0x4040,0x7F7F,0x9F9F,0x5050,0x9F9F,0x5050,0x5050,0x8F8F,0x8F8F,0xD0D0,0x5050,0x4040,0x4040,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x6060,0x7070,0xBFBF,0x8080,0x8F8F,0x4040,0x4040,0x6F6F,0x4040,0x6F6F,0x4040,0x6F6F,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x5050,0x7171,0xC0C0,0x6060,0x5151,0x5151,0x5151,0x5151,0x5151,0x6161,0x6161,0x6161,0x6161,0x6161,0x7171,0x6161,0x6171,0x7171,0x7171,0x7171,0x7171
.word 0x5050,0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x7070,0x6060,0x7070,0x7070,0x9F9F,0x6060,0x6070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x4040,0x4040,0x6F6F,0x6F6F,0x5050,0x8F8F,0x4040,0x5050,0x8F8F,0x4141,0x4141,0x5050,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4141,0x8F8F,0x4040,0x4040,0x5050,0x4141,0x8F8F,0x8F8F,0x4040,0x5050,0x8F8F,0x4050,0x8F8F,0x9F9F,0x5050,0x5050,0xAFAF,0x7070,0x8F8F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x6060,0x6060,0x8080,0x6060,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7070,0x6161,0x6161,0x4040,0x5151,0x5151,0x5151,0x5151,0x5151,0x6161,0x6161,0x6161,0x6161,0x6161,0x7171,0x7261,0x7171,0x7171,0xD461,0x7171
.word 0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0xAFAF,0x6060,0x5050,0x6060,0x7070,0x7070,0x9F9F,0x9F9F,0x7070,0x9F9F,0x7070,0x6060,0x9F9F,0x5050,0x4040,0x6F6F,0x4040,0x8F8F,0x5050,0x4040,0x8F8F,0x4050,0x8F8F,0x5050,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4040,0x7F7F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x4141,0x8F8F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x6060,0x8F8F,0x5050,0x8F8F,0x9F9F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x4040,0x6060,0xAFAF,0xE0E0,0x9F9F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x5050,0x4040,0x4040,0x5050,0x4040,0x6060,0x6060,0x5050,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x9F9F,0x7070,0x6161,0x4040,0x5151,0x5151,0x5151,0x5151,0x6151,0x5151,0x6161,0x6161,0x7161,0x7162,0x7171,0x7171,0x7171,0x7171,0x71D4,0x7171
*/


/*
bird_1:
Fish_320:
.word 0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x5151,0x9F9F,0x4141,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x5151,0x5151,0x5151,0x5151,0x4141,0x5151,0x5151,0x5151,0x5151,0x4141,0x5151,0x8F8F,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x4141,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x4141,0x4141,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xFDFD,0xFDFD,0xFDFD,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x8F8F,0x7F41
.word 0x8F8F,0x4141,0x4141,0x4141,0x4141,0x8F8F,0x4141,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x8F8F,0x5151,0x5151,0x8F8F,0x5151,0x5151,0x8F8F,0x5151,0x5151,0x5151,0x5151,0x5151,0x8F8F,0x5151,0x5151,0x4141,0x5151,0x5151,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x4141,0x5151,0x8F8F,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x6161,0x9F9F,0x6161,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x4141,0x4141,0x4141,0x5151,0x8F8F,0x4141,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xFDFD,0xDFDF,0xDFDF,0xEFEF,0xFDFD,0xEFEF,0xFDFD,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F
.word 0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x5151,0x5151,0x8F8F,0x5151,0x4141,0x4141,0x4141,0x5151,0x5151,0x5151,0x4141,0x4141,0x9F9F,0x9F9F,0x4141,0x9F9F,0x9F9F,0x8F8F,0x5151,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x6161,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x8F8F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x4141,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xFCFC,0xDFDF,0xEFEF,0xFDFD,0xFDFD,0xDFDF,0xFDFD,0xEFEF,0xFCFC,0xDFDF,0xDFDF,0xCFCF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x4141,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x4141,0x7F7F
.word 0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x8F8F,0x4141,0x4141,0x7F7F,0x8F8F,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x4141,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x5151,0x5151,0x8F8F,0x5151,0x8F8F,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x8F8F,0x4141,0x9F9F,0x9F9F,0x9F9F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xFDFD,0xEFEF,0xEFEF,0xFDFD,0xEFEF,0xDFDF,0xDFDF,0xFCFC,0xDFDF,0xECEC,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x5151,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F
.word 0x8F8F,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x4141,0x8F8F,0x5151,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x5151,0x5151,0x4141,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x8F8F,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x4141,0x5151,0x4141,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x5151,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x5151,0x9F9F,0x5151,0x5151,0x8F8F,0x8F8F,0x4141,0x4141,0x4141,0x8F8F,0x4141,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xECEC,0xDFDF,0xFDFD,0xFDFD,0xEFEF,0xFDFD,0xEFEF,0xFDFD,0xEFEF,0xEFEF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xECEC,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F
.word 0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x5151,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x5151,0x4141,0x4141,0x5151,0x4141,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x8F8F,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x4141,0x9F9F,0x8F8F,0x5151,0x8F8F,0x4141,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xFDFD,0xEFEF,0xFFFF,0xFFFF,0xFDFD,0xEFEF,0xFDFD,0xFDFD,0xFDFD,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F41
.word 0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x5151,0x8F8F,0x5151,0x5151,0x8F8F,0x4141,0x8F8F,0x8F8F,0x5151,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x8F8F,0x5151,0x4141,0x5151,0x9F9F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x5151,0x8F8F,0x5151,0x5151,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x5151,0x5151,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xFCFC,0xEFEF,0xFFFF,0xFDFD,0xFFFF,0xFFFF,0xFDFD,0xEFEF,0xEFEF,0xEFEF,0xFCFC,0xDFDF,0xFCFC,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F
.word 0x8F8F,0x5151,0x9F9F,0x5151,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x5151,0x8F8F,0x4141,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x4141,0x4141,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x4141,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x8F8F,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xFCFC,0xDFDF,0xFDFD,0xEFEF,0xFFFF,0xFDFD,0xFFFF,0xFFFF,0xFDFD,0xEFEF,0xFDFD,0xDFDF,0xFCFC,0xDFDF,0xCFCF,0xECEC,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x5151,0x9F9F,0x5151,0x8F8F,0x5151,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x4040,0x8F8F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5151,0x4141,0x4141,0x5151,0x8F8F,0x4141,0x9F9F,0x8F8F,0x8F8F,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x8F8F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x8F8F,0x9F9F,0x5151,0x8F8F,0x4141,0x4141,0x5151,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x5151,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xFDFD,0xEFEF,0xFDFD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xFDFD,0xEFEF,0xFDFD,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x7F41
.word 0x9F9F,0x5151,0x8F8F,0x5151,0x8F8F,0x5151,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x4040,0x4141,0x8F8F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x7F7F,0x8F8F,0x4141,0x4141,0x4141,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x5151,0x8F8F,0x5151,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x4141,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x8F8F,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x4141,0x4141,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xFDFD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xFFFF,0xFFFF,0xEFEF,0xEFEF,0xDFDF,0xECEC,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F
.word 0x5151,0x8F8F,0x5151,0x8F8F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4040,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x5151,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x5151,0x5151,0x4141,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x8F8F,0x4141,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xFCFC,0xFDFD,0xFDFD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xFFFF,0xFDFD,0xFDFD,0xFCFC,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F
.word 0x8F8F,0x5050,0x8F8F,0x5050,0x4141,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x4040,0x8F8F,0x4040,0x8F8F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4141,0x8F8F,0x7F7F,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x4141,0x8F8F,0x8F8F,0x5151,0x5151,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x4141,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x8F8F,0x5151,0x5151,0x5151,0x9F9F,0x4141,0x9F9F,0x8F8F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4141,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4141,0x4141,0x8F8F,0x4141,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xECEC,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xFDFD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xFFFF,0xEFEF,0xFBFB,0xDFDF,0xDFDF,0xECEC,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5151,0x9F9F,0x5050,0x5151,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x4141,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5151,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x5151,0x8F8F,0x4141,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4141,0x9F9F,0x8F8F,0x4141,0x9F9F,0x8F8F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x8F8F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xFBFB,0xFDFD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xEFEF,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F41
.word 0x5050,0x8F8F,0x9F9F,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x4141,0x5050,0x5050,0x4040,0x8F8F,0x4040,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x4040,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x8F8F,0x4141,0x4141,0x4141,0x8F8F,0x8F8F,0x5151,0x4141,0x8F8F,0x5151,0x9F9F,0x8F8F,0x5151,0x8F8F,0x4141,0x5151,0x8F8F,0x4141,0x5151,0x8F8F,0x4141,0x4141,0x9F9F,0x9F9F,0x9F9F,0x4141,0x9F9F,0x4141,0x9F9F,0x5151,0x8F8F,0x8F8F,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x5151,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xEFEF,0xFDFD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xEFEF,0xFCFC,0xFCFC,0xDFDF,0xDFDF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x4040,0x4141,0x8F8F,0x4141,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x4141,0x8F8F,0x4141,0x4141,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xCFCF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x5151,0x4141,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x5151,0x9F9F,0x4141,0x8F8F,0x5151,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xFBFB,0xFDFD,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xFFFF,0xFDFD,0xEFEF,0xDFDF,0xECEC,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x8F8F,0x5050,0x9F9F,0x5050,0x4141,0x8F8F,0x4141,0x5050,0x5050,0x4040,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x5151,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x4141,0x4141,0x4141,0x5151,0x9F9F,0x5151,0x9F9F,0x4141,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x4141,0x9F9F,0x5151,0x9F9F,0x5151,0x4141,0x9F9F,0x4141,0x5151,0x8F8F,0x4141,0x8F8F,0x5151,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x5151,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xFDFD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xEFEF,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x4141,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F
.word 0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x5050,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4141,0x4141,0x4141,0x8F8F,0x7F7F,0x4141,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x5151,0x8F8F,0x8F8F,0x4141,0x5151,0x5151,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x4141,0x9F9F,0x5151,0x4141,0x5151,0x9F9F,0x8F8F,0x4141,0x9F9F,0x4141,0x5151,0x9F9F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x5151,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xFBFB,0xEFEF,0xFDFD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xFDFD,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5151,0x8F8F,0x4141,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x4141,0x8F8F,0x5151,0x8F8F,0x4141,0x5151,0x4141,0x4141,0x4141,0x8F8F,0x5151,0x9F9F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x4141,0x4141,0x8F8F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x4141,0x9F9F,0x8F8F,0x4141,0x5151,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x5151,0x8F8F,0x4141,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xFBFB,0xFDFD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xFDFD,0xFBFB,0xFCFC,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x8F8F,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x7F7F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xEBEB,0xECEC,0xDFDF,0xEBEB,0xDFDF,0xEBEB,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x5151,0x4141,0x5151,0x4141,0x8F8F,0x5151,0x5151,0x8F8F,0x5151,0x8F8F,0x5151,0x5151,0x5151,0x4141,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCCCC,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xDFDF,0xEFEF,0xFDFD,0xFFFF,0xFDFD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F
.word 0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x4141,0x5050,0x8F8F,0x5050,0x5050,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x8F8F,0x4141,0x7F7F,0x4141,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xECEC,0xCFCF,0xDFDF,0xFCFC,0xCFCF,0xEBEB,0xCFCF,0xEBEB,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x5151,0x5151,0x5151,0x4141,0x8F8F,0x5151,0x5151,0x4141,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x4141,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x5151,0x9F9F,0x4141,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xCFCF,0xCFCF,0xFBFB,0xEFEF,0xFDFD,0xFFFF,0xFDFD,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xFDFD,0xDFDF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4040,0x7F7F,0x4141,0x4040,0x4141,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xCFCF,0xDFDF,0xDFDF,0xCFCF,0xDFDF,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x5151,0x5151,0x5151,0x5151,0x4141,0x5151,0x9F9F,0x5151,0x4141,0x8F8F,0x9F9F,0x5151,0x5151,0x4141,0x8F8F,0x8F8F,0x4141,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x4141,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x5151,0x8F8F,0x5151,0x9F9F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xEFEF,0xFDFD,0xEFEF,0xFDFD,0xFFFF,0xFDFD,0xFDFD,0xFDFD,0xEFEF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x6060,0x5050,0x8F8F,0x5151,0x5151,0x4040,0x5050,0x5050,0x5050,0x5040,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xFBFB,0xDFDF,0xDFDF,0xDFDF,0xFBFB,0xDFDF,0xDFDF,0xEBEB,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x5151,0x4141,0x8F8F,0x4141,0x8F8F,0x5151,0x4141,0x4141,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x8F8F,0x8F8F,0x9F9F,0x5151,0x4141,0x9F9F,0x5151,0x8F8F,0x8F8F,0x4141,0x9F9F,0x9F9F,0x5151,0x8F8F,0x5151,0x5151,0x9F9F,0x9F9F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x9F9F,0x5151,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xDFDF,0xFBFB,0xEFEF,0xFDFD,0xEFEF,0xFDFD,0xFFFF,0xFDFD,0xEFEF,0xEFEF,0xFCFC,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F
.word 0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x8F8F,0x7F7F,0x4141,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xEBEB,0xDFDF,0xEBEB,0xDFDF,0xDFDF,0xFBFB,0xFBFB,0xDFDF,0xDFDF,0xEBEB,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x5151,0x5151,0x5151,0x5151,0x5151,0x5151,0x5151,0x4141,0x5151,0x9F9F,0x8F8F,0x8F8F,0x4141,0x9F9F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x8F8F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xFDFD,0xFDFD,0xEFEF,0xFDFD,0xFDFD,0xFFFF,0xFDFD,0xDFDF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x6F6F,0x7F7F
.word 0x5050,0x5050,0x6060,0x5050,0x8F8F,0x5151,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x5050,0x5050,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xFBFB,0xFBFB,0xDFDF,0xFBFB,0xFBFB,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xEBEB,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x5151,0x4141,0x4141,0x8F8F,0x5151,0x4141,0x4141,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4141,0x5151,0x8F8F,0x4141,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x5151,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xCFCF,0xEBEB,0xFCFC,0xEFEF,0xEFEF,0xFDFD,0xFDFD,0xEFEF,0xFDFD,0xEFEF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x5151,0x4141,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F
.word 0x6060,0x6060,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x4050,0x5050,0x4040,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x4141,0x8F8F,0x4141,0x7F7F,0x4141,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xECEC,0xCFCF,0xDFDF,0xFBFB,0xFBFB,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xFBFB,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x5151,0x4141,0x5151,0x4141,0x5151,0x5151,0x5151,0x5151,0x8F8F,0x5151,0x8F8F,0x4141,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x8F8F,0x8F8F,0x4141,0x5151,0x8F8F,0x5151,0x4141,0x5151,0x8F8F,0x8F8F,0x5151,0x5151,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xDFDF,0xFBFB,0xFBFB,0xFDFD,0xEFEF,0xFDFD,0xFDFD,0xFCFC,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F
.word 0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x4040,0x4040,0x8F8F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x5151,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xEBEB,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xFBFB,0xFBFB,0xFBFB,0xDFDF,0xFBFB,0xFBFB,0xFBFB,0xDFDF,0xEBEB,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x5151,0x8F8F,0x4141,0x8F8F,0x4141,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x4141,0x5151,0x8F8F,0x5151,0x4141,0x5151,0x8F8F,0x5151,0x5151,0x8F8F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x4141,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x8F8F,0x9F9F,0x5151,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xFBFB,0xDFDF,0xFBFB,0xFBFB,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x6050,0x5050,0x6060,0x5050,0x5151,0x8F8F,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4141,0x4141,0x7F7F,0x4141,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xECEC,0xDFDF,0xDFDF,0xFBFB,0xDFDF,0xFBFB,0xFBFB,0xFBFB,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x9F9F,0x4141,0x8F8F,0x9F9F,0x9F9F,0x4141,0x5151,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x5151,0x5151,0x9F9F,0x5151,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xFDFD,0xDFDF,0xFBFB,0xDFDF,0xFCFC,0xFCFC,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4141,0x4141,0x4141,0x8F8F,0x8F8F,0x4141,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xFBFB,0xFBFB,0xDFDF,0xFBFB,0xDFDF,0xFBFB,0xDFDF,0xFAFA,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x6161,0x5151,0x8F8F,0x5151,0x8F8F,0x5151,0x5151,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x5151,0x5151,0x5151,0x5151,0x8F8F,0x4141,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x8F8F,0x9F9F,0x5151,0x8F8F,0x5151,0x4141,0x9F9F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xDFDF,0xFCFC,0xDFDF,0xFDFD,0xDFDF,0xFCFC,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F
.word 0x5050,0x6060,0x5050,0x5050,0x9F9F,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x4040,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xEFEF,0xEFEF,0xDFDF,0xDFDF,0xFBFB,0xDFDF,0xFAFA,0xDFDF,0xDFDF,0xDFDF,0xFBFB,0xFAFA,0xCFCF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x4141,0x8F8F,0x4141,0x9F9F,0x8F8F,0x5151,0x5151,0x5151,0x4141,0x5151,0x9F9F,0x9F9F,0x5151,0x8F8F,0x8F8F,0x5151,0x9F9F,0x5151,0x8F8F,0x9F9F,0x5151,0x8F8F,0x5151,0x9F9F,0x5151,0x4141,0x5151,0x8F8F,0x9F9F,0x5151,0x8F8F,0x9F9F,0x5151,0x8F8F,0x8F8F,0x5151,0x9F9F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x5151,0x8F8F,0x5151,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xECEC,0xCFCF,0xDFDF,0xFCFC,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x6060,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x6050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4141,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x6161,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xFAFA,0xDFDF,0xDFDF,0xF7F7,0xEFEF,0xFBFB,0xDFDF,0xEFEF,0xEFEF,0xEFEF,0xFBFB,0xDFDF,0xFAFA,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x5151,0x5151,0x4141,0x9F9F,0x4141,0x5151,0x9F9F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x9F9F,0x5151,0x9F9F,0x4141,0x9F9F,0x8F8F,0x4141,0x9F9F,0x9F9F,0x4141,0x4141,0x9F9F,0x9F9F,0x5151,0x5151,0x8F8F,0x8F8F,0x5151,0x4141,0x8F8F,0x8F8F,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xDFDF,0xECEC,0xDFDF,0xDFDF,0xFCFC,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F
.word 0x5050,0x6060,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x6060,0x5050,0x6060,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x4141,0x8F8F,0x5151,0x9F9F,0x6161,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xEFEF,0xEFEF,0xFBFB,0xDFDF,0xDFDF,0xDFDF,0xEFEF,0xDFDF,0xFAFA,0xEFEF,0xF7F7,0xEFEF,0xEFEF,0xDFDF,0xFAFA,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x8F8F,0x4141,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x8F8F,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x8F8F,0x5151,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x5151,0x8F8F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x5151,0x5151,0x8F8F,0x5151,0x8F8F,0x5151,0x4141,0x5151,0x8F8F,0x9F9F,0x4141,0x9F9F,0x9F9F,0x4141,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xECEC,0xDFDF,0xDFDF,0xECEC,0xDFDF,0xECEC,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x5151,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x6050,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5060,0x6060,0x5050,0x5050,0x5050,0x4040,0x4040,0x7F7F,0x4141,0x7F7F,0x4141,0x4040,0x4040,0x5050,0x4040,0x4040,0x4040,0x4040,0x4040,0x4141,0x4141,0x8F8F,0x4141,0x4141,0x8F8F,0x5151,0x9F9F,0x5151,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xEFEF,0xF7F7,0xEFEF,0xEFEF,0xEFEF,0xF7F7,0xEFEF,0xF7F7,0xEFEF,0xDFDF,0xFAFA,0xEFEF,0xFAFA,0xEFEF,0xFAFA,0xEFEF,0xDFDF,0xDFDF,0xCFCF,0xBFBF,0xAFAF,0x6161,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x8F8F,0x4141,0x8F8F,0x8F8F,0x5151,0x5151,0x8F8F,0x5151,0x8F8F,0x4141,0x8F8F,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xECEC,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x7F7F
.word 0x6060,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5060,0x6060,0x5050,0x6060,0x5050,0x5060,0x5050,0x5050,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x5050,0x5050,0x4050,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x6060,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xDFDF,0xCFCF,0xDFDF,0xDFDF,0xEFEF,0xEFEF,0xDFDF,0xEFEF,0xF7F7,0xEFEF,0xF7F7,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xFBFB,0xEFEF,0xF7F7,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x4141,0x9F9F,0x4141,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x8F8F,0x5151,0x8F8F,0x5151,0x5151,0x9F9F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x4141,0x4141,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x8F8F,0x5151,0x8F8F,0x5151,0x4141,0x9F9F,0x8F8F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xCFCF,0xFCFC,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F41
.word 0x5060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6050,0x5050,0x6060,0x5050,0x6060,0x6050,0x5050,0x5050,0x5050,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x4141,0x4141,0x4141,0x4141,0x8F8F,0x5151,0x5151,0x5050,0x5050,0x6060,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xF5F5,0xDFDF,0xDFDF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xF7F7,0xEFEF,0xFAFA,0xFBFB,0xEFEF,0xEFEF,0xFAFA,0xDFDF,0xF6F6,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x5151,0x4141,0x5151,0x9F9F,0x4141,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x4141,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x5151,0x8F8F,0x5151,0x5151,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x5151,0x9F9F,0x5151,0x8F8F,0x8F8F,0x5151,0x4141,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x4141,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xECEC,0xCFCF,0xCFCF,0xDFDF,0xCFCF,0xECEC,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x4141,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x6060,0x6050,0x5050,0x6060,0x5050,0x6060,0x5060,0x5050,0x4040,0x4040,0x4040,0x4141,0x7F7F,0x4141,0x8F8F,0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x5151,0x5151,0x5050,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xF7F7,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xF7F7,0xEFEF,0xF7F7,0xEFEF,0xEFEF,0xFBFB,0xFBFB,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x6161,0x9F9F,0x5151,0x9F9F,0x4141,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x4141,0x5151,0x8F8F,0x4141,0x4141,0x9F9F,0x8F8F,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x4141,0x9F9F,0x8F8F,0x4141,0x9F9F,0x9F9F,0x5151,0x8F8F,0x8F8F,0x9F9F,0x4141,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xECEC,0xECEC,0xCFCF,0xECEC,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x8F8F,0x7F7F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x7F7F,0x4141,0x7F7F
.word 0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5060,0x6060,0x5050,0x5060,0x5050,0x5060,0x6060,0x5050,0x6060,0x5050,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x4141,0x4141,0x8F8F,0x5151,0x4141,0x5151,0x5050,0x5050,0x7070,0xBFBF,0xD0D0,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xF5F5,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xFBFB,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xDFDF,0xDFDF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x5151,0x8F8F,0x4141,0x4141,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x4141,0x4141,0x8F8F,0x5151,0x8F8F,0x4141,0x8F8F,0x8F8F,0x9F9F,0x5151,0x8F8F,0x5151,0x5151,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x5151,0x8F8F,0x4141,0x4141,0x8F8F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F
.word 0x6060,0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5060,0x6060,0x5050,0x6060,0x5050,0x5060,0x5050,0x4040,0x4040,0x7F7F,0x7F7F,0x4141,0x4040,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x4141,0x8F8F,0x8F8F,0x5151,0x8F8F,0x4141,0x9F9F,0x5050,0x5050,0x6060,0xAFAF,0xBFBF,0xCFCF,0xDFDF,0xDFDF,0xEFEF,0xEFEF,0xDFDF,0xEFEF,0xEFEF,0xF7F7,0xEFEF,0xEFEF,0xEFEF,0xF7F7,0xEFEF,0xF7F7,0xEFEF,0xEFEF,0xEFEF,0xF7F7,0xFAFA,0xEFEF,0xF7F7,0xF7F7,0xDFDF,0xCFCF,0xCFCF,0xAFAF,0x6262,0x6161,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x8F8F,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x4141,0x9F9F,0x8F8F,0x5151,0x4141,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x8F8F,0x5151,0x8F8F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x4141,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x4141,0x8F8F,0x8F8F,0x7F7F,0x4141,0x7F7F,0x4141,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F
.word 0x5050,0x6060,0x6060,0x5050,0x5050,0x5050,0x6060,0x6050,0x5050,0x5050,0x6060,0x5060,0x6060,0x6060,0x5050,0x5050,0x4050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x5050,0x5050,0x5060,0x5050,0x6060,0x5050,0x5050,0x4141,0x8F8F,0x8F8F,0x5151,0x4141,0x5151,0x5050,0x6060,0xAFAF,0xD0D0,0xBFBF,0xCFCF,0xF4F4,0xF5F5,0xF5F5,0xEFEF,0xF5F5,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xFFFF,0xEFEF,0xFFFF,0xFFFF,0xF7F7,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xFBFB,0xEFEF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x5151,0x5151,0x9F9F,0x8F8F,0x4141,0x5151,0x9F9F,0x8F8F,0x5151,0x4141,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x4141,0x9F9F,0x4141,0x9F9F,0x5151,0x8F8F,0x8F8F,0x4141,0x9F9F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x8F8F,0x7F7F,0x7F7F
.word 0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5060,0x5050,0x5050,0x6060,0x6050,0x5050,0x6050,0x5050,0x5050,0x4040,0x4040,0x8F8F,0x4141,0x4040,0x4040,0x8F8F,0x5050,0x6050,0x5050,0x6050,0x5050,0x5050,0x8F8F,0x4141,0x8F8F,0x5151,0x4141,0x9F9F,0x5050,0x5050,0x6060,0x7070,0xD0D0,0xBFBF,0xE0E0,0xDFDF,0xDFDF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xF7F7,0xFFFF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xF7F7,0xEFEF,0xEFEF,0xFBFB,0xEFEF,0xEFEF,0xDFDF,0xE5E5,0xBFBF,0xAFAF,0xAFAF,0x5252,0x5252,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x9F9F,0x4141,0x9F9F,0x9F9F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x8F8F,0x8F8F,0x9F9F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x4141,0x4141,0x8F8F,0x4141,0x7F7F,0x4141,0x8F8F,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F
.word 0x6050,0x5050,0x6060,0x5050,0x5050,0x5050,0x6060,0x6060,0x6050,0x6060,0x5050,0x6060,0x6050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x8F8F,0x8F8F,0x4040,0x5040,0x5050,0x5060,0x5050,0x5050,0x5050,0x5050,0x4141,0x8F8F,0x5151,0x9F9F,0x5151,0x5150,0x5050,0x5050,0x6060,0xC0C0,0xAFAF,0xE0E0,0xCFCF,0xF4F4,0xF5F5,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xFFFF,0xFFFF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xF7F7,0xEFEF,0xEFEF,0xDFDF,0xDFDF,0xCFCF,0xBFBF,0x6262,0xAFAF,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x4141,0x5151,0x9F9F,0x5151,0x4141,0x5151,0x5151,0x9F9F,0x5151,0x4141,0x5252,0x6262,0x9F9F,0x5252,0x5252,0x9F9F,0x8F8F,0x4242,0x5151,0x4141,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x5151,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F41
.word 0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6050,0x5050,0x5050,0x5050,0x5050,0x6060,0x5060,0x6060,0x5050,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4141,0x4040,0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x4141,0x9F9F,0x5151,0x5050,0x6060,0x6060,0x7070,0xAFAF,0xD0D0,0xCFCF,0xF4F4,0xDFDF,0xDFDF,0xF5F5,0xF5F5,0xEFEF,0xF5F5,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xFFFF,0xF7F7,0xEFEF,0xEFEF,0xEFEF,0xF7F7,0xEFEF,0xF7F7,0xDFDF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x6161,0x6161,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x4141,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x7373,0xC4C4,0xC4C5,0x73C5,0xC5D4,0xC4C4,0xC4B4,0xC4C4,0xC4C4,0x63C5,0x6363,0x6363,0x6363,0x7373,0x5252,0x5252,0x9F9F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x6060,0x6060,0x6050,0x6050,0x6050,0x5050,0x6060,0x6060,0x5050,0x4050,0x4040,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050,0x5060,0x6050,0x5050,0x6050,0x5050,0x5151,0x4141,0x5151,0x8F8F,0x5151,0x9F9F,0x5050,0x5050,0x6060,0x7070,0x7171,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xF5F5,0xF5F5,0xEFEF,0xF5F5,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xFFFF,0xFFFF,0xEFEF,0xFBFB,0xEFEF,0xDFDF,0xDFDF,0xF6F6,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x6161,0x6161,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0x4141,0x5151,0x4141,0x5151,0x5151,0x5151,0x5151,0x8F8F,0x5252,0xC4C4,0xC5C4,0xC5C4,0xC5C4,0xC5C4,0xC4C5,0xC4C5,0xC4C5,0xC4C5,0xC4C5,0xC5C4,0xC5C4,0xC5C4,0xC5C4,0xC5C4,0xC5C4,0xC5C4,0xC573,0xC5C4,0x7362,0x6262,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F
.word 0x6050,0x5050,0x5050,0x5050,0x5050,0x6060,0x6060,0x5050,0x5050,0x5060,0x5050,0x6060,0x6060,0x6050,0x5050,0x5050,0x5050,0x8F8F,0x7F7F,0x4040,0x4040,0x5040,0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x5151,0x6060,0x5050,0x6060,0x6060,0x7070,0xAFAF,0xD0D0,0xE0E0,0xF4F4,0xF4F4,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xEFEF,0xF5F5,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xEFEF,0xF7F7,0xEFEF,0xEFEF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xD4D4,0xAFAF,0x6262,0x9F9F,0x6262,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x6161,0x9F9F,0x9F9F,0x9F9F,0x6161,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x6262,0xC5D4,0xC5C4,0xC5C5,0xC5C4,0xC5B4,0xC5B4,0xC5B4,0xC4C4,0xC5C4,0xC5C4,0xC4C4,0xC5C5,0xC5B4,0xC4C4,0xB4C5,0xB4C5,0x74C5,0x74B5,0xC4C5,0x63C5,0xC5C5,0xC4C5,0xC4C5,0x7373,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x5151,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F41
.word 0x5050,0x5050,0x6060,0x9F9F,0x6060,0x5050,0x5060,0x6060,0x6060,0x5050,0x6060,0x5060,0x5060,0x6060,0x6060,0x6060,0x4040,0x4040,0x8F8F,0x8F8F,0x4040,0x8F8F,0x5060,0x6060,0x5050,0x6060,0x5050,0x5050,0x9F9F,0x5151,0x5151,0x5151,0x5050,0x5050,0x5050,0x6060,0x7070,0xC0C0,0xC0C0,0xBFBF,0xCFCF,0xF4F4,0xDFDF,0xF5F5,0xDFDF,0xF5F5,0xDFDF,0xDFDF,0xDFDF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xFFFF,0xEFEF,0xEFEF,0xEFEF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5252,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x6161,0x9F9F,0x9F9F,0x6161,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x4141,0x5151,0x5151,0x4141,0x5151,0xC4C4,0xC5C4,0xC4C5,0xC4C5,0xB4C4,0xC4C5,0xB4C5,0xB4C5,0xB4C5,0xB5C5,0x74C5,0x74C5,0xC5C5,0xB4B4,0xB4B5,0xB4B5,0xB5B4,0xB5B4,0xC574,0xC574,0x7574,0xB5B4,0xB573,0xC4B4,0xC5C4,0xC5C4,0xB4C4,0xC4C4,0x6262,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x4141,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x6060,0x5060,0x5050,0x6060,0x5060,0x6060,0x6060,0x5060,0x6060,0x5050,0x5050,0x8F8F,0x4141,0x4040,0x5050,0x5050,0x5050,0x5050,0x5060,0x6060,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x5060,0x6060,0x6060,0x7070,0x6161,0xBFBF,0xE4E4,0xCFCF,0xF4F4,0xF4F4,0xDFDF,0xF5F5,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xF5F5,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xF7F7,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x6262,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5252,0x9F9F,0x9F9F,0x6161,0x6161,0x9F9F,0x6161,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5252,0xC5C5,0xC5C4,0xC4C5,0xC4C5,0xB5C5,0xC5C5,0xB5B4,0xB5B4,0xC4B5,0xC4B5,0xB474,0xC465,0xC475,0xC574,0xB4B5,0x63C5,0x75C5,0xB4C5,0xB4C5,0xB4B5,0x63B5,0x65C5,0x73C5,0x74C5,0xB5C5,0xC4C4,0xB4C5,0xB5B5,0xC565,0xC5C4,0x6262,0x6262,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x5151,0x8F8F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x5050,0x6060,0x5060,0x6060,0x5050,0x6060,0x6050,0x6060,0x5060,0x5060,0x6060,0x6050,0x5050,0x5050,0x4040,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x6050,0x5050,0x6050,0x5050,0x5050,0x5050,0x5151,0x4141,0x5151,0x9F9F,0x5050,0x5050,0x6050,0x7070,0x8080,0xAFAF,0x7171,0xD0D0,0xF4F4,0xCFCF,0xDFDF,0xF4F4,0xDFDF,0xDFDF,0xF4F4,0xF4F4,0xDFDF,0xDFDF,0xDFDF,0xF5F5,0xEFEF,0xEFEF,0xFFFF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xDFDF,0xDFDF,0xF6F6,0xCFCF,0xBFBF,0xE4E4,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x5252,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x4141,0x5252,0xC4C4,0xC5C4,0xC5C4,0xC5C4,0xC574,0xC5B4,0xC4B4,0xB4B5,0xC5C5,0xC565,0xB563,0xC5C4,0xB5C5,0x75C5,0xB5C4,0x73C5,0x63C5,0xB4B5,0xB4B5,0x74C5,0x65C5,0x57C4,0xC4B4,0xC4B5,0xB565,0xC5B4,0xB465,0xB565,0xC574,0xB463,0xB4C4,0xC565,0xB4B5,0xB4C5,0xC4C4,0x6262,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x5151,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x7F7F,0x8F8F,0x8F8F,0x4141,0x4141,0x8F8F,0x7F7F,0x4141,0x7F7F
.word 0x5050,0x5050,0x6060,0x6060,0x8F8F,0x6060,0x5050,0x5060,0x5060,0x6060,0x6060,0x6060,0x6060,0x5050,0x6060,0x6060,0x5050,0x8F8F,0x8F8F,0x4040,0x5050,0x6050,0x5050,0x6050,0x5050,0x6050,0x5050,0x5050,0x5151,0x8F8F,0x8F8F,0x5050,0x5050,0x5050,0x6060,0x7070,0x8080,0x8080,0xBFBF,0xBFBF,0xE4E4,0xF4F4,0xF4F4,0xCFCF,0xF4F4,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xF5F5,0xDFDF,0xF5F5,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xF5F5,0xEFEF,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xD4D4,0xBFBF,0x6262,0xAFAF,0x6161,0x6161,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5252,0x6161,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xD5D5,0xC5C4,0xC467,0xB4B5,0x65C5,0xC5C4,0x63C5,0xB5C5,0xC573,0x74C5,0x63C5,0xC4B5,0xB4B5,0xB465,0xC563,0xB563,0xB563,0xC565,0xC465,0xC4B5,0xB4B5,0x73B5,0xB4B5,0xC5C4,0xB573,0xC465,0x6565,0x65C4,0xC4B5,0x65C4,0xC5B4,0xB565,0xB4C5,0xC5B4,0x6565,0x5765,0xC565,0xC563,0x6262,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x5151,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x6060,0x5060,0x6060,0x6060,0x6060,0x6060,0x6060,0x5060,0x5060,0x6060,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x4141,0x8F8F,0x5050,0x6060,0x6060,0x5050,0x5060,0x5050,0x5050,0x5050,0x8F8F,0x5151,0x5151,0x5050,0x6060,0x6050,0x7070,0x7070,0xAFAF,0x7171,0xC0C0,0xE4E4,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xE3E3,0xF4F4,0xF4F4,0xDFDF,0xF5F5,0xEFEF,0xEFEF,0xEFEF,0xF5F5,0xEFEF,0xEFEF,0xF5F5,0xF5F5,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x5252,0x5151,0x5151,0x5151,0x6161,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6161,0x5151,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0xC452,0xC5C4,0xC4C5,0xB5C4,0x57C4,0xB5C4,0x7357,0xC5B5,0x57B4,0xB4C5,0xC5B4,0xC5B4,0xB5B4,0x57C4,0xB5C5,0x65C5,0x63C5,0x63C5,0x63B5,0x65C5,0x65C4,0xC563,0xB563,0xC563,0x6565,0x63C5,0x65B5,0xC465,0xB447,0xC565,0xD4B5,0xC4C5,0xC4C4,0xC4C4,0xB4B5,0xC4C5,0xB4C5,0x65B4,0x65B4,0x73C5,0xC4C4,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x4141,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x6060,0x5050,0x6060,0x5050,0x6050,0x5060,0x5060,0x6060,0x5050,0x6060,0x5050,0x6060,0x6060,0x5050,0x5050,0x8F8F,0x8F8F,0x4040,0x5050,0x5050,0x5050,0x6060,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x5050,0x6060,0x5050,0x6060,0x7070,0xAFAF,0x8080,0xC0C0,0xAFAF,0xD0D0,0xE3E3,0xCFCF,0xE4E4,0xE3E3,0xBFBF,0xD0D0,0xE3E3,0xF3F3,0xF3F3,0xDFDF,0xF3F3,0xEFEF,0xF5F5,0xEFEF,0xF5F5,0xEFEF,0xF5F5,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x6161,0x9F9F,0x9F9F,0x5252,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6161,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0xC4C4,0xC5C5,0xC4C5,0xB5C5,0x63C5,0xC5C5,0xC465,0xB5C5,0xC573,0xC465,0xC565,0xC457,0xC5C5,0xC4C5,0xC4B5,0x63B4,0xB4C5,0xC575,0xC565,0xB5B4,0xC5C4,0xC565,0x74C5,0x63C5,0x63B5,0x73C5,0x73B5,0xC465,0x7457,0x65B5,0x55B4,0xB563,0xB5C4,0xB5C4,0xB574,0xB573,0xB573,0xB574,0xC565,0xC465,0xB5C5,0xB5B4,0x73B4,0x6363,0x5252,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x5151,0x9F9F,0x5151,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F
.word 0x5050,0x5050,0x5050,0x6060,0x5050,0x5050,0x6060,0x5050,0x6060,0x5050,0x6060,0x6050,0x6060,0x5050,0x5060,0x5050,0x8F8F,0x4040,0x4040,0x5050,0x5050,0x5050,0x5060,0x5060,0x6060,0x6050,0x8F8F,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x6050,0x6060,0x7070,0x7070,0xC0C0,0xAFAF,0xD4D4,0xE4E4,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xD0D0,0xBFBF,0xE0E0,0xF3F3,0xF2F0,0xF2F2,0xEFEF,0xF2F2,0xFFFF,0xFFFF,0xFFFF,0xF5F5,0xDFDF,0xF4F4,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xE4E4,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6161,0x6161,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x6161,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x6262,0xC5C5,0xC5C4,0xC5C4,0xC463,0xC563,0x6365,0x57C5,0xC563,0x63C5,0xC5C5,0x73C5,0x63C5,0x63B5,0x65C5,0x65C5,0xB5C5,0x65B4,0x65C5,0xC5B4,0xC563,0x7375,0xB4C5,0xC563,0xC565,0xB563,0xC563,0xC563,0x65C5,0xB573,0xC563,0xB457,0x73B5,0xC4B5,0xC4B5,0xC4C5,0xB4B5,0x73B5,0x73B5,0x74B5,0xB565,0xC463,0xB457,0x5765,0xB4C5,0xAFAF,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F41
.word 0x5050,0x5050,0x5050,0x5050,0x6060,0x6060,0x6060,0x6050,0x6050,0x6060,0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x4040,0x8F8F,0x8F8F,0x5050,0x5050,0x6060,0x6050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x4141,0x5050,0x5050,0x6060,0x5050,0x6060,0x7070,0x8080,0xAFAF,0xC0C0,0xBFBF,0xD4D4,0xBFBF,0xE4E4,0xD0D0,0xD0D0,0xBFBF,0xE0E0,0xE3E3,0xF3F3,0xF2F2,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xCFCF,0xE4E4,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0xD5C4,0xC4C5,0xC4C5,0x5557,0x57C5,0xB5B5,0xC563,0x63B5,0xC5C4,0x6563,0x63C5,0x5763,0xC563,0xC563,0xC563,0xC463,0x7365,0xC547,0x6573,0x63C5,0xB5C5,0xB573,0x65C5,0x63B5,0x7365,0x6357,0x63C5,0x63B5,0x6355,0x47C5,0x65B4,0xB563,0xC465,0xC4C5,0xC4C5,0xC4C5,0xC5C5,0xC5C4,0x65C4,0x63B5,0x65C5,0x57C4,0x65B4,0xC465,0x6365,0x63C5,0x5151,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x5151,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x4141,0x8F8F,0x4141,0x7F7F,0x8F8F,0x7F7F,0x4141,0x8F8F,0x7F7F
.word 0x5050,0x9F9F,0x5050,0x5050,0x6060,0x5050,0x5060,0x6060,0x5050,0x5060,0x5060,0x5060,0x6060,0x6050,0x5050,0x5050,0x8F8F,0x4040,0x5050,0x5050,0x5050,0x5060,0x5050,0x5060,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x5060,0x7070,0x7070,0xAFAF,0x7171,0xC0C0,0xC0C0,0xAFAF,0xC0C0,0xBFBF,0xBFBF,0xD0D0,0xD3D3,0xE0E0,0xF0F4,0xF2F2,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF2F2,0xF3F3,0xF4F4,0xF4F4,0xCFCF,0xE4E4,0xBFBF,0xE4E4,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xD4D4,0xAFAF,0x6161,0xAFAF,0x9F9F,0x6161,0x5151,0x5151,0x5151,0x5151,0x5252,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0xC5C4,0xC4C5,0xC5C5,0x57C4,0xC573,0x6363,0x6363,0x55C5,0xC573,0x6357,0xC5C5,0xC563,0xC457,0x6357,0x6357,0x63C5,0x57C5,0x57C4,0x55C4,0xC557,0x5763,0x6363,0x63C5,0xC563,0x5763,0x57C4,0x65C4,0x5773,0x5773,0x57C4,0x7363,0x7357,0x63C5,0xC5C5,0xD5C4,0xD5D4,0xE5D5,0xD4D5,0xC4C5,0xC575,0xC563,0xB465,0xB457,0xC447,0x55C5,0xC563,0xC563,0x5252,0x9F9F,0x5151,0x8F8F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x8F8F,0x8F8F,0x4141,0x7F7F,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x6060,0x6060,0x6060,0x6050,0x5050,0x6060,0x5050,0x6060,0x6060,0x6060,0x6060,0x5050,0x4050,0x5050,0x8F8F,0x8F8F,0x9F9F,0x5060,0x6060,0x5060,0x6060,0x5050,0x5050,0x5050,0x4141,0x5050,0x5050,0x5050,0x5050,0x6060,0x6060,0x7070,0x8080,0x7171,0xBFBF,0xAFAF,0xAFAF,0x7171,0x7171,0xC0C0,0xAFAF,0xD3D3,0xF3F4,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xD4D4,0xAFAF,0xBFBF,0xBFBF,0x6161,0x6161,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0xD552,0xC4D4,0x55C5,0xC563,0xC563,0x6347,0x57C5,0x47C5,0xC563,0xC457,0x55C5,0x4763,0x6563,0x5773,0x57C4,0x47C4,0xC563,0x6363,0x6357,0xC457,0x4763,0xC563,0x6347,0xC557,0x6563,0x63C5,0x6347,0xC457,0x7347,0x63C5,0xC465,0xC457,0x5765,0xC574,0xC5C4,0xC5C5,0xE4D5,0xF5E5,0xD5E5,0xC5C4,0x65B4,0x63B5,0x55C5,0x65C4,0x55C4,0xC563,0x6563,0x74C4,0x63C5,0x6363,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x7F7F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F41
.word 0x5050,0x6060,0x6060,0x5050,0x5050,0x6060,0x5050,0x6060,0x6050,0x6060,0x6050,0x6050,0x6060,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5060,0x6060,0x5060,0x6060,0x5050,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x6060,0x7070,0x8080,0xAFAF,0xD4D4,0x7171,0x7171,0xAFAF,0xAFAF,0x6161,0xD0D0,0xE0E0,0xDFDF,0xF2F2,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF3F3,0xF3F3,0xF3F3,0xCFCF,0xE3E3,0xE0E0,0xE4E4,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0xD4D4,0x57C5,0x5773,0x63C5,0x5267,0x5763,0x4763,0xC452,0x4747,0x4763,0x5773,0x63C5,0x47C5,0x63C5,0x6347,0x6357,0x6357,0xC547,0x5773,0x4752,0xC447,0x6357,0xC563,0x6363,0x6357,0x5763,0xC563,0x5773,0x4763,0x6365,0x65C5,0x5563,0x73C5,0x6357,0xC5C5,0xC4D4,0xE5D5,0xF5F5,0xC4D5,0x63C5,0xC5B5,0xC5B4,0xC573,0xC447,0x7357,0x63B5,0x73C5,0x6347,0xC563,0x63C5,0x52C5,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x6060,0x6060,0x6060,0x5050,0x6060,0x5050,0x5060,0x5050,0x6060,0x6060,0x5050,0x6060,0x8F8F,0x5050,0x5050,0x8F8F,0x8F8F,0x5050,0x6060,0x5050,0x5060,0x5050,0x5050,0x5050,0x4040,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x6060,0x7070,0x7070,0x7171,0xAFAF,0x7171,0xAFAF,0xAFAF,0x7171,0x7171,0xAFAF,0xD3D3,0xF4F4,0xF2F3,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF3F3,0xF3F3,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xD4D4,0xAFAF,0xD4D4,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6161,0x6161,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x5252,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0xC5C5,0xC5C4,0xC573,0x5763,0x5752,0x4762,0x6257,0x6357,0x47C5,0x6347,0x63C5,0x63C5,0x5247,0x6347,0x5752,0x4752,0x6363,0x5247,0x52C5,0x5257,0x47C5,0x57C4,0x5763,0x63C5,0xC557,0xC563,0x6257,0x6357,0xC555,0xC563,0x5773,0x4555,0x57C4,0x4763,0x5763,0xC4C4,0xD5C5,0xD5D5,0xD5E5,0xC5C4,0xC574,0x63C5,0x74C5,0x65C5,0x57C4,0xC563,0xC563,0x5773,0x5563,0x63C5,0xC563,0x6363,0x5252,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x9F9F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x6060,0x6060,0x6060,0x5050,0x6060,0x6060,0x6050,0x6050,0x6050,0x6050,0x5050,0x5050,0x8F8F,0x8F8F,0x5050,0x5050,0x5050,0x5060,0x6060,0x6060,0x6060,0x5050,0x5050,0x8F8F,0x4040,0x5050,0x5050,0x5050,0x5060,0x5050,0x6060,0x7070,0xC0C0,0xC0C0,0xAFAF,0x7171,0x7171,0x6161,0x7070,0xD0D0,0xE3E3,0xF2F2,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF1F1,0xDFDF,0xCFCF,0xE0E0,0xE3E3,0xE0E0,0xE4E4,0xCFCF,0xD3D3,0xBFBF,0xAFAF,0xAFAF,0xD0D0,0xAFAF,0xD4D4,0xAFAF,0x6161,0x9F9F,0x6161,0x6161,0x5151,0x9F9F,0x5252,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x63C4,0xC5C5,0x5763,0x4752,0x4752,0x6347,0x5763,0x4763,0x5552,0x4763,0x5752,0x4752,0x5752,0x5752,0x5555,0x4545,0x4742,0x4763,0x4742,0x5752,0x6363,0x6257,0x63C5,0xC547,0x6363,0x5257,0x5752,0x5763,0x4763,0x4763,0xC563,0x5763,0x4752,0x6363,0x7357,0x7347,0xC4C5,0xD4D5,0xC4C5,0x74B5,0xB4C5,0xC574,0xC574,0xC563,0x6355,0x63B5,0x7347,0x7347,0xC565,0x6563,0x63C5,0x73C5,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F
.word 0x5050,0x5050,0x6060,0x5050,0x6060,0x6060,0x6050,0x6050,0x5050,0x6060,0x6060,0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x6060,0x5050,0x6050,0x6050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x6060,0x7070,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x6161,0xAFAF,0xE4E4,0xF3F3,0xF0F0,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF1F1,0xF3F3,0xE3E3,0xD0D0,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xE0E0,0xCFCF,0xBFBF,0xD0D0,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x5151,0xC4C4,0x57C5,0x4752,0x52C5,0x5247,0x5247,0x4747,0x4763,0x4752,0x5247,0x6357,0x6357,0x5247,0x5247,0x5257,0x4247,0x4247,0x4247,0x4747,0x4747,0x5255,0x6248,0x4757,0x4752,0x4252,0x5257,0x5757,0x5763,0x5752,0x4752,0x4752,0x4542,0x5763,0x5763,0x5763,0x5763,0x5763,0xC563,0xC5C4,0x65C4,0x73C5,0xC5B5,0x6563,0x6563,0x6365,0x55C5,0x5763,0x5773,0x5773,0x6363,0x73C5,0x63C4,0x65C5,0x52B4,0x9F9F,0x9F9F,0x6161,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x8F8F,0x4141,0x4141,0x4141,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x8F7F
.word 0x6060,0x5050,0x6060,0x6060,0x5060,0x6060,0x6060,0x6060,0x6050,0x6050,0x6050,0x6050,0x6050,0x6050,0x5050,0x5050,0x8F8F,0x5050,0x6060,0x5050,0x6060,0x6060,0x6060,0x6060,0x5050,0x5050,0x4040,0x5050,0x5050,0x6060,0x5050,0x6060,0x6060,0x7070,0x7070,0x7171,0x7171,0x7070,0x7171,0x9F9F,0x7070,0xC0C0,0xE0E0,0xF3F0,0xEFEF,0xF1F1,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF1F1,0xF3F3,0xCFCF,0xCFCF,0xE3E3,0xF3F3,0xF3F3,0xF4F4,0xCFCF,0xE4E4,0xD0D0,0xBFBF,0xD0D0,0xBFBF,0xD0D0,0xAFAF,0xC0C0,0x6161,0x6161,0x6161,0x5151,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5252,0x5252,0x73C5,0x5257,0x4752,0x4752,0x4742,0x5252,0x5247,0x5247,0x5752,0x4752,0x4752,0x4752,0x5542,0x4747,0x4242,0x5242,0x4747,0x4762,0x5552,0x4747,0x4852,0x4252,0x5247,0x5757,0xC5C5,0x5263,0x5257,0x5247,0x4247,0x4247,0x4545,0x5247,0x5247,0x5247,0x5247,0x6357,0x6357,0x6357,0xC565,0x5763,0x5563,0xC555,0xC547,0x5763,0xC455,0xC457,0xC465,0xC457,0xC547,0xC563,0xC563,0xC573,0xC565,0x7373,0x6161,0x9F9F,0x9F9F,0x9F9F,0x6161,0x9F9F,0x6161,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F
.word 0x5050,0x5060,0x6060,0x6060,0x6060,0x5050,0x5060,0x5050,0x6060,0x5050,0x6060,0x6060,0x5050,0x6060,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x6060,0x6060,0x6060,0x6050,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x6050,0x6050,0x6060,0x7070,0x8080,0xAFAF,0xAFAF,0x6161,0x7070,0xAFAF,0xBFBF,0xE3E3,0xF3F3,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF1F1,0xF3F3,0xCFCF,0xE0E0,0xBFBF,0xCFCF,0xF3F3,0xDFDF,0xF4F4,0xE3E3,0xCFCF,0xCFCF,0xE4E4,0xAFAF,0xD0D0,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0xC5C4,0x5257,0x4247,0x4747,0x4742,0x4342,0x4742,0x4242,0x4247,0x4747,0x4752,0x4752,0x4742,0x4742,0x4242,0x4747,0xC5C5,0xC5C5,0xC5C5,0xC5C5,0xC5C5,0x4242,0xC542,0xE5E5,0xD5D5,0xD5D5,0xD5D5,0x5757,0x4742,0x4242,0x4242,0x4542,0x4742,0x4242,0x4742,0x4752,0x5762,0x6363,0x6357,0x47C5,0x4763,0x47C4,0x63C5,0x6363,0xC447,0x63C5,0x73C5,0xC457,0x63C5,0x63C5,0x5763,0xC563,0x4773,0x6373,0x63C5,0xAFAF,0x9F9F,0x6161,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F41
.word 0x6060,0x5050,0x5050,0x5050,0x6060,0x5060,0x6060,0x6060,0x6050,0x6050,0x6050,0x6050,0x5060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5060,0x6060,0x6060,0x8F8F,0x5050,0x8F8F,0x5050,0x5050,0x5060,0x5050,0x5050,0x6060,0x7070,0xAFAF,0x6161,0x7070,0x7070,0x7070,0x7070,0xD071,0xE0E4,0xF3F3,0xF2F2,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF1F1,0xF1F1,0xF4F4,0xD0D0,0xBFBF,0xE3E3,0xE0E0,0xF3F3,0xF3F3,0xDFDF,0xF3F3,0xE3E3,0xCFCF,0xBFBF,0xE4E4,0xAFAF,0xAFAF,0xD0D0,0xAFAF,0x6161,0x6161,0x6161,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5752,0x4762,0x4752,0x4242,0x4242,0x4742,0x4242,0x4742,0x4242,0x4242,0x4242,0x4247,0x4242,0x4242,0x4742,0xC5C5,0xE5E5,0xC5C5,0xD4D4,0xD5D5,0xD5D5,0x5745,0xE5E5,0xF5F5,0xF6F6,0xDFDF,0xCFCF,0xD5D5,0xAFAF,0x5757,0x4247,0x4247,0x4242,0x4742,0x5247,0x5247,0x5247,0x4747,0x5763,0x6352,0x63C5,0x63C5,0x63C5,0xC5C5,0xC573,0xC563,0xC573,0xC5C4,0xC562,0xC563,0x7357,0x5257,0x6347,0x63C5,0xC563,0x6363,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x6161,0x9F9F,0x9F9F,0x5151,0x5151,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F
.word 0x6060,0x6060,0x6060,0x6060,0x6050,0x6060,0x5050,0x6060,0x6060,0x6060,0x5050,0x6060,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x6050,0x6060,0x6060,0x6050,0x5050,0x6060,0x5050,0x5050,0x5050,0x6060,0x6060,0x6050,0x6050,0x5050,0x6060,0x7070,0x7070,0x7070,0x6161,0x9F9F,0x9F9F,0xBFBF,0xF4F4,0xF3F3,0xEFEF,0xF1F1,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF1F1,0xF1F1,0xF3F3,0xE0E0,0xBFBF,0xE3E3,0xE0E0,0xDFDF,0xDFDF,0xF3F3,0xF3F3,0xF3F3,0xCFCF,0xE3E3,0xE3E3,0xBFBF,0xD0D0,0xD0D0,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x5151,0x5151,0x6161,0x5151,0x6161,0x9F9F,0x5151,0x5151,0x6363,0x6347,0x4247,0x4247,0x4747,0x4242,0x4242,0x4242,0x4244,0x4247,0x4747,0x4742,0x4747,0x4242,0xD5D5,0xD5D5,0xD5E5,0xD5D4,0xD5D5,0xE5E5,0x57D4,0xD4D4,0xE5E5,0xE5E5,0xCFCF,0xCFCF,0xBFBF,0xE5E5,0xF4E5,0xE5F5,0xBFBF,0x4747,0x4247,0x4747,0x4742,0x4742,0x4752,0x5763,0xC563,0xC457,0xC447,0xC457,0xC563,0xC563,0xC563,0xC573,0x63C4,0x63C5,0x63C5,0x52C5,0x63C5,0x6357,0x6347,0xC547,0x4763,0xC4C4,0x6262,0xAFAF,0xAFAF,0x6161,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x4141,0x8F8F,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x7F7F
.word 0x5050,0x5050,0x5050,0x6060,0x5050,0x5060,0x6060,0x6050,0x5050,0x6050,0x5060,0x5060,0x5060,0x6060,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x9F9F,0x6050,0x5050,0x6060,0x5060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5060,0x5050,0x5050,0x6060,0x6060,0x7070,0x9F9F,0x6060,0x9F9F,0x7060,0x7171,0xD0D0,0xE0E0,0xF2F2,0xF1F1,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF1F1,0xF1F1,0xF3F3,0xF0F0,0xCFCF,0xD0D0,0xCFCF,0xF4F4,0xF3F3,0xF3F3,0xEFEF,0xF3F3,0xDFDF,0xF3F3,0xCFCF,0xBFBF,0xD0D0,0xBFBF,0xBFBF,0xBFBF,0xC0C0,0xAFAF,0x6161,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5752,0x4752,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4742,0x4242,0x4545,0x4747,0xD5D5,0xD5D5,0xD5E4,0xAFAF,0xC5C5,0xD5D5,0xAFAF,0xC5C5,0xE5E5,0xE5E5,0xBFBF,0x9F9F,0x9F9F,0xBFBF,0xCFCF,0xF5F5,0xF5F7,0xD4E5,0xAFAF,0x6363,0x5247,0x5247,0x5257,0x63C5,0x62C5,0xC4C5,0x63C5,0x57C4,0xC4C5,0xC4C5,0x73C5,0x63C5,0xC547,0xC552,0xC563,0xC563,0xC5C4,0xC563,0x5763,0x6363,0xC457,0x6565,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x5151,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F8F
.word 0x5050,0x6050,0x9F9F,0x6060,0x6060,0x6060,0x5050,0x5060,0x6060,0x5050,0x6060,0x6060,0x5050,0x6060,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x6060,0x5060,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x6050,0x6050,0x5050,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0xD0D0,0xE3E3,0xF3F3,0xF1F1,0xFFFF,0xF1F1,0xF1F1,0xF1F1,0xF1F1,0xFFFF,0xF3F3,0xE3E3,0xCFCF,0xD0D0,0xBFBF,0xE0E0,0xF3F3,0xF3F3,0xF3F3,0xF0F0,0xF3F3,0xF3F3,0xDFDF,0xE3E3,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6161,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x6161,0x5151,0x9F9F,0x5151,0x6262,0x52C5,0x4247,0x4742,0x4646,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4747,0x4242,0xD5D5,0xD5D5,0xD5D4,0xAFAF,0xE5E5,0xC4C4,0xAFAF,0xD5C5,0x6363,0xD5D5,0xCFCF,0x9F9F,0x7F7F,0x8F8F,0xBFBF,0xCFCF,0xF5F5,0xE5E5,0xD5D5,0xD4C5,0x65C5,0xC5C5,0xC552,0xC5C5,0xC563,0xC563,0xC563,0xC552,0xC563,0x5762,0xC562,0xC552,0x63C5,0x63C5,0x63C5,0xC4C5,0xC5C5,0xC4C4,0x52C5,0xC547,0xC552,0xC447,0xC557,0xC563,0x6262,0xAFAF,0xAFAF,0xAFAF,0x6161,0x6161,0x9F9F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x4141,0x8F8F,0x7F7F
.word 0x5050,0x9F9F,0x6060,0x5050,0x5060,0x5060,0x6060,0x6060,0x5060,0x5060,0x5060,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x6060,0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0xD0D0,0xF4F4,0xF3F3,0xEFEF,0xF2F2,0xF1F1,0xFFFF,0xFFFF,0xF1F1,0xF3F2,0xDFDF,0xE0E0,0xD0D0,0xBFBF,0xE3E3,0xF4F4,0xF3F3,0xF0F0,0xF3F3,0xF3F3,0xF3F3,0xDFDF,0xD0D0,0xBFBF,0x7171,0xD0D0,0x7171,0xBFBF,0xD0D0,0xAFAF,0x6161,0xAFAF,0x6060,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x4242,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4247,0x4247,0x5555,0x4242,0xE5D5,0xCFCF,0xD5D5,0xE5E5,0x4747,0xD5D5,0xBFBF,0xE5D5,0xF5F5,0xCFCF,0xAFAF,0x7F7F,0x8F8F,0xD4D4,0xF6F6,0xF5F5,0xE5E5,0xC5C5,0x52C5,0xD457,0xC563,0xC5C5,0x6574,0x52C5,0x63C5,0x52C5,0xC5C5,0xC563,0xC563,0x63C5,0x73C5,0xC563,0xC5C4,0xC563,0xC563,0xC4C4,0x73C5,0x5765,0x4762,0x63C5,0x4763,0x6362,0x52C5,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x4141,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x8F41
.word 0x5050,0x6060,0x5050,0x6060,0x5050,0x5050,0x5060,0x6050,0x5050,0x5050,0x5050,0x6060,0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x5060,0x5060,0x5050,0x5050,0x5050,0x6060,0x6060,0x6060,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x5050,0x5050,0x5050,0x6060,0x7070,0xE0E0,0xF0F0,0xF2F2,0xF2F2,0xF2F2,0xF2F2,0xF2F2,0xF2F2,0xF3F2,0xF3F3,0xE0E0,0xD0D0,0xBFBF,0xE4E4,0xE0E0,0xF3F3,0xF3F3,0xF2F2,0xEFEF,0xF3F3,0xF3F3,0xE3E3,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x7171,0xAFAF,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x6161,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x5151,0x5252,0x4247,0x4242,0x4242,0x4646,0x4242,0x4747,0x4848,0x4242,0x5252,0x4747,0x4641,0x4242,0x5757,0xD5D5,0xE5D5,0xDFDF,0xD5D5,0x6363,0xE5D5,0xE5E5,0xD5D5,0xF5F5,0xE5E5,0xAFAF,0x9F9F,0xAFAF,0xBFBF,0xF5F5,0xF5F5,0xC5C5,0x57C4,0xD5D5,0xD5D5,0x52C5,0xC447,0xC4C5,0xC557,0xC563,0xC5C5,0xC562,0x62C5,0x62C5,0xC4C5,0xC557,0x47C4,0x63C4,0x63C5,0x63C5,0x73C5,0xC5C5,0xC5D4,0x62C5,0xC547,0x5763,0x6357,0x6557,0x62B5,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x7F7F,0x7F7F
.word 0x6050,0x6060,0x6060,0x6060,0x6060,0x6060,0x5050,0x6060,0x6050,0x5060,0x6060,0x6050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x6060,0x6060,0x5050,0x6060,0x7070,0x7070,0xAFAF,0xC0C0,0xC0C0,0x80C0,0xC0C0,0x8080,0x8080,0xBFBF,0x7070,0xAFAF,0x5060,0x6060,0xC0C0,0xCFCF,0xF3F3,0xF2F2,0xF2F2,0xFFFF,0xFFFF,0xF2F2,0xF3F3,0xDFDF,0xE0E0,0xD0D0,0xBFBF,0xD0D0,0xCFCF,0xF4F4,0xF3F3,0xF3F3,0xEFEF,0xF2F2,0xF2F2,0xF3F3,0xE0E0,0xBFBF,0xC0C0,0xC0C0,0xD0D0,0xAFAF,0xAFAF,0x7171,0xAFAF,0x6161,0x6161,0x6060,0x5151,0x6060,0x6051,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x4242,0xC5C5,0x4848,0x5252,0x4747,0xE4E4,0xD5D5,0x4747,0x4747,0x4242,0x4242,0x4542,0xC4C4,0xC5C5,0xAFAF,0xE5F5,0xC5C5,0xE5D5,0xBFBF,0xD5E5,0xE5D5,0xF6F6,0xE4E4,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xF5F5,0x6363,0xC5C5,0xD4D4,0xD5D4,0xC5E4,0xC5C5,0xC5C4,0x4763,0x63C4,0x47C5,0x6363,0xC5C5,0xC563,0xC563,0x6363,0x63C4,0x62C5,0xC5C5,0xC573,0xC563,0xC563,0xC463,0xC4C5,0xC5C4,0x57C4,0xC563,0x4762,0x52C4,0x63C5,0xC4C4,0x6161,0xAFAF,0x6161,0xAFAF,0x6161,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F8F
.word 0x5050,0x6060,0x6060,0x6060,0x5060,0x5050,0x6060,0x5050,0x6050,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x5060,0x5050,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x7070,0x7171,0x8080,0x7171,0x8080,0xC070,0xC070,0xC0C0,0xD0C0,0xD0D0,0xC0C0,0x7070,0xD0D0,0xF3F3,0xF2F2,0xEFEF,0xF2F2,0xF2F2,0xF3F2,0xF3F3,0xF3F3,0xE3E3,0xCFCF,0xD0D0,0xBFBF,0xE3E3,0xE0E0,0xF3F3,0xF3F3,0xFFFF,0xF2F2,0xF2F2,0xF3F3,0xCFCF,0xBFBF,0xC0C0,0xAFAF,0xAFAF,0xAFAF,0xC0C0,0xC0C0,0xAFAF,0x6161,0x9F9F,0x9F9F,0x9F9F,0x6060,0x5151,0x9F9F,0x5151,0x6060,0x5151,0x9F9F,0x5151,0x5151,0x5151,0x9F9F,0x5151,0x5151,0x9F9F,0xAFAF,0xF4E4,0xE4E5,0xE4E4,0xD5D5,0xD5D5,0xE5E5,0xD5D5,0x4242,0x4747,0x4247,0x4242,0x5252,0xC542,0xD5D5,0xD5D5,0xD5D5,0xE5D5,0xD5D5,0xCFCF,0xE5E5,0xCFCF,0xE5E5,0xE4E4,0xE5E5,0xF6F6,0xF6F6,0xD5D5,0xD5D5,0xC5D4,0xD5D5,0xD5D5,0xC4D5,0x4747,0xC563,0xC552,0xC447,0xC5C5,0x63C5,0x52C5,0xC5C5,0xC563,0x48C4,0xC5C4,0xC552,0xC563,0xC563,0xC563,0xC565,0xC573,0x63C5,0x63C5,0x57C4,0x5763,0x5752,0x5752,0x7373,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x7F7F
.word 0x6060,0x6060,0x6060,0x6050,0x6060,0x6050,0x5050,0x6050,0x5050,0x5060,0x5060,0x5060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5060,0x5050,0x9F9F,0x9F9F,0xAFAF,0x6060,0x5050,0x5050,0x5151,0x7070,0x7070,0x7070,0x7171,0x6161,0x7061,0x7070,0xAFAF,0xD0D0,0x8080,0xC0C0,0xE0E0,0xE3E3,0xF0F0,0xF2F2,0xF2F2,0xF3F3,0xF3F3,0xF4F4,0xCFCF,0xD0D0,0xD0D0,0xAFAF,0xD0D0,0xCFCF,0xE3E3,0xF3F3,0xEFEF,0xF2F2,0xF2F2,0xF3F3,0xDFDF,0xD0D0,0x7171,0xAFAF,0x7171,0x7171,0xAFAF,0xAFAF,0xAFAF,0x6161,0x6161,0x7070,0x6161,0x6161,0x9F9F,0x6060,0x6060,0x6060,0x5151,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0xBFBF,0xFFFF,0xF5F5,0xF5F5,0xE5E5,0xBFBF,0xE4E4,0xE4E4,0x4247,0x4242,0x4242,0x4646,0x4747,0xD5D5,0xE5E5,0xD5D5,0xD5D5,0xCFCF,0xD5D5,0xE5D5,0xE5E5,0xF6F6,0xF7F7,0xCFCF,0xCFCF,0xF6F6,0xD5D5,0xD5D5,0xC5D5,0xD5D5,0xD5D5,0xD5D4,0x57D4,0x6563,0x52C5,0x6367,0x47C4,0x65B4,0xC563,0xC5C4,0x6352,0x52C5,0x6247,0x6257,0x63C5,0x63C5,0x63C5,0x63C5,0x63C5,0x63C5,0xC563,0x5763,0x63C5,0x63C5,0x63C5,0xC5C5,0x6363,0x6262,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x5151,0x4141,0x8F8F,0x7F7F,0x4141,0x7F7F,0x8F7F
.word 0x5050,0x5060,0x5050,0x5050,0x6060,0x5050,0x6060,0x5050,0x6060,0x6060,0x5050,0x5050,0x5050,0x6060,0x8F8F,0x5050,0x5050,0x5050,0x6060,0x6060,0x6060,0xE0E0,0xF0F0,0xCFCF,0xBFBF,0x9F9F,0x5050,0x6060,0x6161,0x6170,0x7070,0x7070,0x6161,0x7070,0x7070,0xAFAF,0xAFAF,0x8080,0xC0C0,0xD0D0,0xF3F3,0xF2F2,0xF5F5,0xF3F3,0xF3F3,0xE0E0,0xD0D0,0xD0D0,0xAFAF,0xD0D0,0xE3E3,0xE0E0,0xF3F3,0xEFEF,0xF2F2,0xF2F2,0xF3F3,0xDFDF,0xE3E3,0xBFBF,0xAFAF,0x7171,0x6161,0xBFBF,0xD0D0,0xBFBF,0xD4D4,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6060,0x5151,0x5151,0x9F9F,0x5151,0x6060,0x6161,0x5151,0x6161,0x5151,0x5151,0x9F9F,0x9F9F,0x4141,0xF4F4,0xFFFF,0xEFEF,0xF5F5,0xF5F5,0xE5E5,0xAFAF,0xD5D5,0x9F9F,0x4242,0x4646,0x4242,0x4747,0xE5E5,0xE5E5,0xBFBF,0xD5D5,0xD5E5,0xE5E5,0xD5D5,0xE5E5,0xE5E5,0xF5F5,0xF6F5,0xF6F6,0xD5E5,0xD5D5,0xE5E5,0xD5D5,0xD5D5,0xD5D5,0xD5D5,0xC552,0xC447,0xC547,0xC573,0xC552,0xC447,0x52C5,0x5757,0x52C5,0x47C5,0x57C4,0x6762,0xC563,0xC552,0xC552,0xC552,0x5763,0xC563,0x6357,0x52C5,0x6347,0x6347,0x6247,0xC563,0xC563,0x9F9F,0x6262,0x6161,0xAFAF,0x6161,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x7F7F,0x7F7F
.word 0x5050,0x6060,0x5050,0x6060,0x6050,0x6060,0x5060,0x5060,0x6050,0x5050,0x6060,0x6050,0x5050,0x6060,0x6060,0x5050,0x5050,0x5050,0x6060,0x6060,0xBFBF,0xEFEF,0xFFFF,0xFFFF,0xF5F5,0xBFBF,0x9F9F,0x5050,0x6060,0x9F9F,0x7070,0x6161,0x7070,0x7170,0x8080,0x7070,0x7070,0x8080,0x8080,0xC0C0,0xE0E0,0xF3F0,0xF3F3,0xF3F3,0xE0E0,0xE3E3,0xBFBF,0xAFAF,0xC0C0,0xBFBF,0xE0E0,0xF3F3,0xF3F3,0xF2F2,0xFFFF,0xF2F2,0xF3F3,0xE0E0,0xBFBF,0x7171,0xC0C0,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xD0D0,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x6161,0xAFAF,0x6060,0x6060,0x5151,0x6060,0x9F9F,0x5151,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x5151,0x8F8F,0xDFDF,0xFFFF,0xEFEF,0xF5F5,0xEFEF,0xCFCF,0xAFAF,0xAFAF,0xD5D5,0x9F9F,0x5252,0x4247,0x6552,0xAFAF,0xD5D5,0xD5D5,0xE4E4,0xD5D5,0xE5E5,0xD5D5,0xD5D5,0xD5D5,0xD5D5,0xE5E5,0xD5D5,0xBFBF,0xD5D5,0xD5D5,0xE5E5,0xD5D5,0xE4E4,0xD5D5,0xC5C5,0x5762,0x5263,0x47C5,0x52C5,0x47C4,0xC4C4,0x73C5,0xC547,0xC452,0xC4C4,0xC4C5,0x63C5,0x63C5,0x47C5,0x52C5,0xC447,0x4557,0x57C4,0xC545,0x5763,0xC552,0xC552,0x5257,0x52C5,0xB4B4,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6161,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x7F41
.word 0x5050,0x5050,0x6050,0x5050,0x5060,0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x6060,0x6050,0x9F9F,0xDFDF,0xBFBF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xF4F4,0xAFAF,0x5050,0x6060,0x7070,0x7070,0x6161,0x6060,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x8080,0xBFBF,0xE0E0,0xE3E3,0xCFCF,0xBFBF,0xC0C0,0x7171,0xBFBF,0xE3E3,0xCFCF,0xF3F3,0xF2F2,0xFFFF,0xF2F2,0xF3F3,0xF3F3,0xE0E0,0xC0C0,0xAFAF,0xAFAF,0x7171,0xAFAF,0xD0D0,0xD3D3,0xBFBF,0xD0D0,0xC0C0,0xC0C0,0x7171,0x6161,0xAFAF,0x6161,0x6161,0x6060,0x5151,0x9F9F,0x6051,0x9F9F,0x6060,0x6060,0x6161,0x5151,0x5151,0x9F9F,0x5151,0xF3F3,0xF5F5,0xF5F5,0xFFFF,0xF5F5,0xE5E5,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xD5D5,0x9F9F,0x4248,0xC552,0xC5C5,0xAFAF,0xBFBF,0xD5D5,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xD5D5,0xD5D5,0xD5D5,0xE5E5,0xE4E4,0xE5E5,0xD5D5,0xE4E4,0xC4C5,0xC5C5,0xC563,0xC447,0xC552,0x6363,0x75C4,0x6363,0x52C5,0xC5C5,0xC5B5,0x5762,0xC552,0xC563,0xC563,0x4762,0x52C5,0x5247,0x6347,0x6363,0x6357,0x4763,0x4763,0xC552,0xB463,0xC5C5,0x7373,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x6161,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x8F8F,0x4141,0x7F7F,0x7F7F
.word 0x6050,0x6060,0x5050,0x6060,0x5050,0x6060,0x5060,0x6060,0x5060,0x5060,0x6050,0x5050,0x5050,0x6060,0x8F8F,0x5050,0x5050,0x5050,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xDFDF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x9F9F,0x5151,0x6060,0x6161,0x7070,0x7070,0x7070,0x6161,0x7070,0x7070,0x7070,0x8080,0x8080,0xBFBF,0xD0D0,0xCFCF,0xC0C0,0x7171,0xAFAF,0xBFBF,0xD0D0,0xE3E3,0xF3F3,0xF2F2,0xEFEF,0xF2F2,0xF2F2,0xF3F3,0xCFCF,0xBFBF,0xC0C0,0x7171,0x7070,0xAFAF,0xD0D0,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6161,0x7070,0x9F9F,0x6161,0x6060,0x5151,0x6060,0x5151,0x5151,0x6060,0x6060,0x9F9F,0x6060,0x5151,0xE4E4,0xFFFF,0xF2F2,0xEFEF,0xF5F5,0xF5F5,0xDFDF,0xAFAF,0x9F9F,0xAFAF,0x8F8F,0x8F8F,0xC4C4,0x4242,0x4248,0x4242,0xD5D4,0xD5D5,0xE5E5,0xE5E5,0xE5F5,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xD4D4,0xD5D5,0xD5D5,0xD5D5,0xF6E5,0xC5C5,0xC552,0xC5C5,0xD4D5,0x63C5,0x52C5,0x6363,0x63C5,0x57C5,0x63C5,0xC5C5,0xC5C4,0x5252,0x4763,0x52C5,0x63C5,0x52C5,0x63C5,0x63C5,0x5763,0x5752,0x4752,0x5747,0x5762,0x52C5,0x52C5,0x5257,0x6357,0x6357,0x6363,0xAFAF,0x6161,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5151,0x9F9F,0x4141,0x8F8F,0x8F8F,0x4141,0x8F7F
.word 0x5050,0x5050,0x5060,0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x5050,0x6060,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xCFCF,0xF5F5,0xF5F5,0xF5F4,0xFFFF,0xF4F4,0x6060,0x6060,0x6060,0x7070,0x6060,0x7070,0x7070,0x7070,0x7061,0x7070,0x7070,0x8080,0x8080,0xBFBF,0xD0D0,0xD0D0,0xAFAF,0x8080,0xC0C0,0xE3E3,0xE0E0,0xDFDF,0xF2F2,0xF2F2,0xF2F2,0xF3F3,0xE0E0,0xD0D0,0x7171,0xAFAF,0xAFAF,0xAFAF,0xD0D0,0xBFBF,0xCFCF,0xD0D0,0xD0D0,0xD0D0,0xAFAF,0x7171,0x7171,0x8080,0x7070,0xAFAF,0x6161,0x7070,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x9F9F,0x5150,0xAFAF,0xF2F2,0xF5F5,0xEFEF,0xF5F5,0xF5F5,0xE5E5,0xAFAF,0x9F9F,0xBFBF,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0xC5C5,0x5752,0xAFAF,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xF5F5,0xE5E5,0xD5D5,0xD5D5,0xC5C5,0xD5E4,0xD5D5,0xD5D5,0x4752,0x9F9F,0xE5D4,0xC5C5,0x5752,0xC552,0x6347,0x6357,0x6263,0xC457,0x5752,0x5752,0x57C5,0x7373,0xC457,0xC5C5,0x57C4,0x57C4,0xC5C4,0xC563,0x5752,0x5252,0x4763,0x5263,0x6347,0x6247,0x6257,0x5757,0x5773,0xC563,0x6363,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x6161,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F
.word 0x6050,0x5050,0x5050,0x6060,0x5050,0x5060,0x5060,0x6060,0x5060,0x6050,0x6050,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0xAFAF,0xFFFF,0xF5F5,0x9F9F,0xCFCF,0xFFFF,0x9F9F,0x5050,0x5150,0x6060,0x7061,0x6060,0x7060,0x7070,0x6060,0x7070,0x7070,0x8080,0x8080,0xAFAF,0xBFBF,0xBFBF,0x8080,0xAFAF,0xBFBF,0xE0E0,0xF4F4,0xF2F2,0xEFEF,0xF2F2,0xF3F3,0xF3F3,0xCFCF,0xD0D0,0xAFAF,0x6161,0x7070,0xC0C0,0xBFBF,0xCFCF,0xE4E4,0xBFBF,0xD0D0,0xBFBF,0x7171,0xAFAF,0xAFAF,0xAFAF,0x6161,0x6060,0x6161,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6161,0x6060,0x6060,0xF3F3,0xFFFF,0xF2F2,0xF5F5,0xF5F5,0xF5F5,0xD5D5,0x9F9F,0xE5E5,0xD5E4,0xBFBF,0x7F7F,0x7F7F,0x4141,0x8F8F,0xE5F5,0xD5D5,0xC5C5,0xF5F5,0xE5E5,0xD5D5,0xE5E5,0xE5E5,0xF6E5,0xF5F5,0xD5E5,0xD5D5,0xD4D4,0xC5C5,0x4752,0xE4E4,0xD5E5,0xD5D5,0xC5C4,0xC5C5,0x4763,0x5763,0xC552,0xC557,0x4752,0x52C5,0x63C5,0xC452,0xC565,0x6363,0x52B5,0x6257,0x6257,0x5257,0x52C5,0x62C5,0x4747,0xC552,0xC5C5,0xC552,0xC552,0x5762,0x4563,0xC447,0x5257,0x73C5,0xAFAF,0x6161,0xAFAF,0x6161,0x9F9F,0xAFAF,0x5151,0x9F9F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x7F7F
.word 0x5050,0x6050,0x6050,0x5050,0x6050,0x5050,0x5050,0x6050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x4141,0x8F8F,0x8F8F,0xAFAF,0xFFFF,0xF5F5,0x8F8F,0xF5F5,0xF5F5,0xBFBF,0xAFAF,0x9F9F,0x5050,0x6060,0x5051,0x5151,0x6050,0x7060,0x6060,0x7070,0x7070,0x8080,0x8080,0xC0C0,0xC0C0,0xBFBF,0xD0D0,0xE3E3,0xCFCF,0xF0F0,0xF2F2,0xF2F2,0xF2F2,0xF2F2,0xE0E0,0xD0D0,0x6161,0x7070,0xAFAF,0xAFAF,0xBFBF,0xE4E4,0xD0D0,0xCFCF,0xE4E4,0xBFBF,0xAFAF,0xBFBF,0x8080,0x6161,0x7070,0xAFAF,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x5050,0xF3F3,0xF3F3,0xF5F5,0xF2F2,0xEFEF,0xF5F5,0xE4F4,0xAFAF,0xF5E5,0xDFDF,0xE5E5,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0xBFBF,0xE4E4,0xC5C5,0xE5E5,0xDFDF,0xE5F5,0xD5D5,0xD5D5,0xF5F5,0xE5E5,0xE5E5,0xF5F5,0xF5F5,0xE5E5,0xD5D5,0xE5E5,0xD5D5,0x52C5,0x4747,0x63C5,0x6257,0x5257,0x47C5,0x6363,0x5763,0x5752,0x5752,0x4747,0x63C5,0xC555,0x5752,0x5762,0x5763,0xC563,0xC563,0x4763,0x52C5,0xC5C5,0x63C5,0x52C5,0x6357,0xC547,0x5563,0x4242,0xC552,0x6363,0xC4C4,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F8F
.word 0x5050,0x5050,0x5050,0x5060,0x5050,0x6050,0x5050,0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xEFEF,0xFFFF,0x8F8F,0xF4F4,0xEFEF,0xF4F5,0xE0E0,0x6060,0x5050,0x9F9F,0x9F9F,0x6060,0x5050,0x5050,0x6060,0x6061,0x7070,0x7070,0x7070,0xC0C0,0xAFAF,0xD0D0,0xCFCF,0xE0E0,0xF3F3,0xF2F2,0xEFEF,0xF2F2,0xF2F2,0xF3F3,0xCFCF,0xBFBF,0x7171,0xAFAF,0x8080,0xBFBF,0xD0D0,0xE3E3,0xCFCF,0xD0D0,0xCFCF,0xD0D0,0xC0C0,0x7171,0x8080,0x7070,0x6161,0x7070,0x6161,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0xF3F3,0xF5F5,0xF2F2,0xF5F5,0xF3F3,0xF5F5,0xF5F5,0xE4E4,0xF5F5,0xF5F5,0xDFDF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0xF5F5,0x6363,0x5757,0xE5D5,0xF5F5,0xE5E5,0xE5E5,0xE5E5,0xD5D5,0xF5E5,0xF6F5,0xE5F5,0xE5E5,0xE5E5,0xD5D5,0xD4D5,0x5757,0xC562,0x4763,0x5752,0xC552,0x6352,0x6357,0x52C5,0x52C5,0x63C5,0x73C5,0x5757,0x47C4,0x52C5,0x52C5,0x52C5,0x52C5,0x63C5,0x52C5,0xC563,0xC552,0xC573,0xC552,0xC563,0x5252,0x5757,0x4765,0x4242,0x6347,0x6363,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x7F7F
.word 0x5050,0x6060,0x6060,0x6050,0x6050,0x5050,0x5060,0x5050,0x5050,0x6060,0x5050,0x5050,0x5050,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xF7F7,0xEFEF,0x9F9F,0xAFAF,0xF5F5,0xDFDF,0xFFFF,0xDFDF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0x9F9F,0x8F8F,0x6050,0x7070,0x9F9F,0x7070,0x7070,0x8080,0xC0C0,0xBFBF,0xE0E0,0xCFCF,0xF3F3,0xF2F2,0xF2F2,0xF2F2,0xF3F3,0xCFCF,0xD0D0,0x7171,0x7070,0x7070,0xBFBF,0xE4E4,0xBFBF,0xE0E0,0xE4E4,0xBFBF,0xD0D0,0xBFBF,0xBFBF,0xAFAF,0x6161,0xAFAF,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x6060,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x5050,0xF4D0,0xF2F2,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xE4F4,0xE5E5,0xF5F5,0xF5F5,0xF4F5,0xBFBF,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0xBFBF,0xC4C4,0xD5D5,0xD5D5,0xF5F5,0xE5F5,0xE5E5,0xE5E5,0xE5E5,0xD5D5,0xD5D5,0xD5D5,0xD5D5,0xD5D5,0xE5E5,0xD5D5,0x5257,0x6565,0x6565,0x5257,0x6767,0x5763,0x6347,0xC557,0xC552,0x6752,0x5773,0x6363,0x5757,0x5763,0xC552,0xC547,0x5752,0xC563,0x6347,0x47C5,0x52C5,0x52C5,0x6357,0x6367,0x6563,0x4763,0x4747,0x4242,0x4747,0x7373,0xAFAF,0x6161,0xAFAF,0x6161,0x9F9F,0x5151,0x4141,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F41
.word 0x5050,0x5050,0x5050,0x5050,0x5050,0x6050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0xCFCF,0xE5E5,0xAFAF,0xCFCF,0xF5F5,0xE5E5,0xF5F5,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xCFCF,0xAFAF,0xBFBF,0x8F8F,0x5050,0x6060,0x7070,0x6060,0x7070,0x8080,0xAFAF,0xCFCF,0xF3F3,0xF3F3,0xF2F2,0xEFEF,0xF2F2,0xF3F3,0xE0E0,0xAFAF,0x7070,0x9F9F,0xAFAF,0xBFBF,0xE0E0,0xE3E3,0xCFCF,0xCFCF,0xE3E3,0xD0D0,0xC0C0,0x8080,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x7070,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x9F9F,0x6060,0x5150,0xF3F3,0xFFFF,0xEFEF,0xF3F3,0xF5F5,0xF5F5,0xF5F5,0xF5F4,0xF5F5,0xF5F5,0xF5F5,0xE4E4,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xD5D5,0xE5E5,0x6565,0xE5D5,0xF5F5,0xE5F5,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xE4E5,0xD5D5,0xC5C5,0x5763,0x5762,0x5563,0x63C5,0x6363,0x52C5,0x52C5,0x63C5,0x73C5,0x6565,0x52C5,0x63C5,0x4747,0x6352,0x6257,0x5247,0x6357,0x6347,0xC552,0xC563,0xC563,0xC552,0x6357,0x5257,0x4563,0x4247,0x4242,0x4747,0x6262,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F
.word 0x5050,0x5050,0x6050,0x5050,0x5060,0x6060,0x5050,0x5050,0x6050,0x5050,0x6060,0x5050,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xBFBF,0xDFDF,0xF5F5,0xF5F5,0xDFDF,0xEFEF,0xFFFF,0xF5F5,0xBFBF,0xF4F4,0x9F9F,0xAFAF,0xAFAF,0xCFCF,0x9F9F,0x4040,0x5050,0x6060,0x7060,0x7070,0xAFAF,0x8080,0xBFBF,0xDFDF,0xF0F0,0xF2F2,0xF2F2,0xF0F0,0xE0E0,0xBFBF,0x8080,0x7070,0xAFAF,0xD0D0,0xE3E3,0xCFCF,0xE0E0,0xE3E3,0xD0D0,0xBFBF,0xBFBF,0x6161,0xAFAF,0x6161,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x6060,0x6060,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0xF4F4,0xF2F2,0xF5F5,0xF3F3,0xF5F5,0xF3F3,0xF4F5,0xF5F4,0xF5F5,0xF4F4,0xF4F5,0xCFCF,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4242,0xAFAF,0xD5D5,0xD5D5,0xE5E5,0xD5E5,0xF5F5,0xF6F5,0xD5D5,0xD5D5,0xD5D5,0xE5E5,0xE5E5,0xE5E5,0xD5D5,0xD5D5,0xD5D5,0xD4D4,0x52C5,0x57C5,0x4763,0x5247,0xC5C5,0xC563,0x4763,0x63C5,0x52C5,0x57C5,0xC563,0x6565,0xC447,0x5763,0x6357,0x6357,0x63C5,0x6357,0x47C5,0x4747,0x47C4,0x4763,0x5762,0x5552,0x4747,0x6363,0x4545,0x5252,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x5151,0x9F9F,0x8F8F,0x4141,0x4141,0x7F7F,0x7F8F
.word 0x5050,0x5060,0x6060,0x6050,0x6060,0x6050,0x5050,0x5050,0x5050,0x6050,0x5050,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xF5F5,0xDFDF,0xDFDF,0xF5F5,0xFFFF,0xBFBF,0xBFBF,0xE5E5,0xDFDF,0xBFBF,0xBFBF,0xAFAF,0x8F8F,0xBFBF,0x9F9F,0x5050,0x5050,0x6060,0x6060,0x7070,0xAFAF,0xAFAF,0xCFCF,0xEFEF,0xF2F2,0xF3F3,0xDFDF,0xCFCF,0x7171,0xAFAF,0xAFAF,0xC0C0,0xBFBF,0xE3E3,0xF3F3,0xDFDF,0xE0E0,0xCFCF,0xD0D0,0xC0C0,0x7070,0x8080,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x6060,0x6060,0x6070,0x7070,0x6060,0x6060,0x6060,0x6060,0x5060,0xE3E3,0xF5F5,0xF3F3,0xF5F5,0xF3F3,0xF5F5,0xF5F3,0xF5F5,0xF3F3,0xF5F5,0xF5F5,0xCFCF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x9F9F,0xD4D4,0xD5D5,0xE5E5,0xCFCF,0xC5C5,0xD5D5,0xE5E5,0xE5E5,0xF4E5,0xF5E5,0xF5F5,0xD5E5,0xC5C5,0xC5C5,0x5252,0x7347,0xC547,0xC552,0xC563,0xC552,0x7347,0x52C5,0xC4C5,0xC563,0xC563,0xC463,0x6357,0x65C5,0x47C4,0xC457,0xC552,0x5763,0x5752,0x5773,0x6363,0x7365,0x5257,0x52C5,0x6357,0x5257,0x5247,0x5555,0x5247,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x6161,0x9F9F,0x4141,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x5060,0x5050,0x5050,0x5050,0x6060,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xDFDF,0xDFDF,0xCFCF,0xDFDF,0xF5F5,0xAFAF,0xBFBF,0xCFCF,0xBFBF,0xCFCF,0x9F9F,0xBFBF,0x9F9F,0xAFAF,0xCFCF,0xDFDF,0x6060,0x5050,0x6060,0x6060,0xAFAF,0xBFBF,0xBFBF,0xF0F0,0xEFEF,0xF0F0,0xF4F4,0xD0D0,0xAFAF,0x7070,0xAFAF,0xBFBF,0xE0E0,0xDFDF,0xDFDF,0xF3F3,0xE3E3,0xD0D0,0xBFBF,0x7171,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x6060,0x6060,0x6070,0x6060,0x6060,0x6070,0x6060,0x6060,0x6060,0x6060,0x5050,0xC0C0,0xF2F5,0xF5F5,0xF5F5,0xF3F5,0xF5F5,0xF4F5,0xF4F5,0xF5F5,0xF5F5,0xF4F4,0xF4F4,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xF5F4,0xE5E5,0xE5D5,0xE5E5,0xF5F5,0xDFDF,0xBFBF,0xD5D5,0xD5D5,0xE5E5,0xF5F5,0xE4E5,0xE4E4,0xD4D5,0xC5C5,0xC562,0x5247,0x6357,0x6357,0x63C5,0xC557,0x5752,0x5763,0x5763,0x6357,0x63C5,0x6357,0x7357,0x6347,0x6357,0x63C5,0x63C5,0x63C5,0x47C5,0x52C5,0x52C5,0xC447,0xC552,0x5752,0x4763,0x6363,0x4763,0x6363,0xC4C4,0xAFAF,0x6161,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x7F41
.word 0x5050,0x5050,0x6060,0x5050,0x6050,0x6050,0x5050,0x5050,0x5050,0x5050,0x6060,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0xCFCF,0xF5F5,0xF6F6,0xE5E5,0xCFCF,0xF6F6,0xE5E5,0xAFAF,0x9F9F,0xDFDF,0xBFBF,0xBFBF,0xCFCF,0x9F9F,0xEFEF,0xF3F3,0xCFCF,0x5050,0x5050,0x6060,0x7070,0xAFAF,0xADAD,0xBFBF,0xEFEF,0xE0E0,0xCFCF,0xBFBF,0x7070,0xD0D0,0xE0E0,0xCFCF,0xDFDF,0xF3F3,0xF3F3,0xE0E0,0xCFCF,0xD0D0,0x8080,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x5050,0xF3F3,0xFFFF,0xFFFF,0xF3F5,0xF5F5,0xF5F3,0xF4F4,0xF4F4,0xF4F4,0xF4F4,0xF4F4,0xE5E5,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xE5E5,0xDFDF,0xD5D5,0xF5F5,0xE5F5,0xE4D5,0xD5D5,0xC5C5,0xD5D5,0xF4F4,0xD5D5,0x6565,0xC5C5,0xC4C4,0x5257,0xC563,0xC563,0x4763,0x5763,0x63C4,0x6357,0x63C5,0x63C5,0x5763,0xC563,0xC563,0xC563,0xC563,0x5773,0x5763,0x5763,0x5752,0x6352,0x5763,0x5752,0x4752,0x5257,0x5257,0x5247,0x4742,0x5252,0x6347,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F
.word 0x6060,0x5050,0x5050,0x5060,0x5050,0x5050,0x5050,0x6050,0x6060,0x5050,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0xF5F5,0xEFEF,0xCFCF,0xCFCF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xF5F5,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xCFCF,0xF5F5,0xBFBF,0xBFBF,0xBFBF,0x5050,0x6060,0x6060,0xAFAF,0xBFBF,0xBDBD,0xDFDF,0xF4F4,0xBFBF,0x7070,0xAFAF,0xBFBF,0xCFCF,0xDFDF,0xF3F3,0xF0F0,0xDFDF,0xE3E3,0xD0D0,0x7070,0x6161,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x7060,0x7070,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6050,0xF4F4,0xFFFF,0xF5F5,0xF5F3,0xF5F5,0xF4F5,0xF4F4,0xF4F4,0xF4F4,0xF4F4,0xF5F5,0xF4F4,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0xF5F5,0xF5F5,0xE5E5,0xF6F6,0xE5E5,0xE5E5,0xE4E5,0xE5E5,0xE5E5,0xD5D5,0xC5C5,0x52C5,0x6547,0x4763,0x52C5,0x5257,0xC547,0xC552,0x4763,0x57C4,0x4763,0x6347,0x6267,0x6357,0x6357,0x5257,0x6347,0x5747,0xC552,0xC563,0xC563,0xC563,0x5257,0x62C5,0x5257,0xC552,0x4762,0x6363,0x4257,0x4242,0x6363,0x6247,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x5060,0x6050,0x5050,0x5050,0x5050,0x5050,0x5060,0x5050,0x5060,0x5050,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0xAFAF,0xF7F7,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xF5F5,0xBFBF,0xDFDF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xF5F5,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x5050,0x5050,0x6060,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xD0D0,0xAFAF,0xBFBF,0xCFCF,0xDFDF,0xDFDF,0xEFEF,0xDFDF,0xE0E0,0xCFCF,0xAFAF,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0x6060,0x6070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x5050,0xE0E0,0xF5F5,0xF5F3,0xF5F5,0xF5F3,0xF5F5,0xF4F4,0xF4F4,0xF4F4,0xE5E5,0xF4F4,0xCFCF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0xF7F5,0xE5F5,0xE5E5,0xE5E5,0xE5E5,0xF5F5,0xF6F5,0xE5E5,0xF5F5,0xE5F4,0xE4E4,0x52C5,0x6257,0x4747,0xC563,0x4763,0x6357,0x6357,0x5257,0x6357,0x5763,0xC563,0xC563,0x5763,0x5763,0x5763,0x6363,0x52C5,0x5257,0x6357,0x5257,0x5763,0x5752,0x5555,0x4257,0x4747,0x4763,0x5252,0x4242,0x5252,0x9F9F,0xAFAF,0xAFAF,0x6161,0x6161,0x9F9F,0x8F8F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x7F7F
.word 0x6060,0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0xFFFF,0xDFDF,0xBFBF,0xBFBF,0xAFAF,0xCFCF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xAFAF,0x9F9F,0xBFBF,0xAFAF,0x5050,0x6050,0x6060,0x6060,0xAFAF,0x8E8E,0xADAD,0xBFBF,0xAFAF,0xBFBF,0xCFCF,0xDFDF,0xDFDF,0xEFEF,0xDFDF,0xF3F3,0xCFCF,0xD0D0,0x7171,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x6070,0x6060,0x6060,0x6070,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x5050,0x6161,0xF2F2,0xF5F5,0xF3F3,0xF5F5,0xF5F5,0xF5F5,0xF4F4,0xE5E5,0xF4F4,0xF4F4,0xDFDF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0xAFAF,0xF5F5,0xF5F7,0xF6F5,0xE5E5,0xF6F6,0xF5F5,0xF6F6,0xE5E5,0xF6F6,0xD5D5,0x5757,0x5757,0x5762,0x5752,0x7373,0x63C5,0x63C5,0x47C5,0x6763,0xC563,0x5752,0x5763,0x5763,0x5763,0x5763,0x5763,0x5752,0xC563,0xC552,0x6752,0xC563,0x6352,0x6565,0x6363,0x5545,0x6363,0x5257,0x4747,0x9F9F,0x6262,0x6262,0xAFAF,0xAFAF,0x9F9F,0x5151,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x4141,0x7F7F
.word 0x6060,0x5050,0x6060,0x5050,0x5050,0x6050,0x5050,0x5050,0x6050,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0xFFFF,0xE5E5,0xBFBF,0xE5E5,0xBFBF,0xAFAF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xAFAF,0xBFBF,0x4141,0xE4E4,0xAFAF,0xBFBF,0x5050,0x5050,0x9F9F,0xAFAF,0xAFAF,0xADAD,0xCFCF,0xBFBF,0xCFCF,0xDFDF,0xEFEF,0xEFEF,0xEFEF,0xF0F0,0xCFCF,0xD0D0,0xAFAF,0xAFAF,0x7070,0x8080,0xAFAF,0xAFAF,0xAFAF,0x8080,0x7070,0x7070,0x6060,0x7070,0x9F9F,0x6060,0x6060,0x6060,0x7070,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6050,0xAFAF,0xF5F5,0xF3F5,0xF5F5,0xF5F5,0xF5F3,0xF4F4,0xF4F5,0xF4F4,0xF4F4,0xCFCF,0xE5E5,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0xBFBF,0xF5F5,0xF5F5,0xF6F6,0xF6F6,0xE5E5,0xE5E5,0xF6F6,0xE5E5,0xE5E5,0xC4D4,0x6365,0x6357,0x63C5,0x63B5,0x5247,0x6347,0x5247,0x63C5,0x6357,0x6357,0x52C5,0x63C5,0x6357,0x6257,0x6357,0x6357,0x5257,0x5257,0x6257,0x4257,0x57C5,0x5773,0x4752,0x4752,0x5763,0x5762,0x4545,0xC552,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F
.word 0x5050,0x5050,0x5050,0x5060,0x5050,0x5050,0x5050,0x6060,0x5050,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x9F9F,0xF5F5,0xBFBF,0xBFBF,0xAFAF,0xF5F5,0xE5E5,0x9F9F,0xE5E5,0xCFCF,0xCFCF,0xCFCF,0xF5F5,0x9F9F,0xBFBF,0x8F8F,0xBFBF,0xAFAF,0xBFBF,0x6060,0x5050,0x7070,0x9F9F,0xAFAF,0x9D9D,0xBDBD,0xCFCF,0xCFCF,0xDFDF,0xEFEF,0xEFEF,0xDFDF,0xDFDF,0xD0D0,0xAFAF,0x7070,0x7070,0xAFAF,0xAFAF,0xBFBF,0xC0C0,0x8080,0xAFAF,0x7070,0x6060,0x6060,0x7070,0x7060,0x7070,0x6070,0x6060,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x5050,0xD3D3,0xF3F3,0xF4F4,0xF4F4,0xF5F4,0xF5F5,0xF5F5,0xF5F5,0xF4F4,0xF4F4,0xF4F5,0xCFCF,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0xBFBF,0xF5F5,0xF5F5,0xF6F5,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xC5C5,0x5757,0x5752,0x57C4,0x5763,0x4863,0x5763,0x5752,0x5763,0x4752,0x5763,0x5763,0x5752,0x5752,0x5762,0x5763,0x5752,0x5763,0x5752,0x5763,0x5763,0x6363,0x6247,0x6357,0x5555,0x4343,0x6347,0x5252,0x4242,0x4545,0x5252,0x5252,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x6060,0x5060,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x9F9F,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0xAFAF,0xF7F7,0xDFDF,0xE5E5,0xCFCF,0x9F9F,0xF5F5,0xCFCF,0xAFAF,0xAFAF,0xE5E5,0xF7F7,0xEFEF,0x9F9F,0xBFBF,0xCFCF,0xAFAF,0xE5E5,0xDFDF,0x5070,0x5050,0x9F9F,0x9F9F,0x8E8E,0xBFBF,0xBDBD,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xF3F3,0xE0E0,0xCFCF,0xC0C0,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xC0C0,0xAFAF,0xAFAF,0x7070,0x7070,0x6060,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x5050,0x6060,0x5050,0x6060,0xF4F4,0xF4F5,0xF4F5,0xF4F4,0xF4F4,0xF5F5,0xF5F4,0xF5F5,0xE5E5,0xCFCF,0xCFCF,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xE5E5,0xF5F5,0xF6F6,0xF6F5,0xE5E5,0xD5D5,0xD5D5,0xC5C5,0x4763,0xC563,0x4752,0xC563,0x6247,0x6357,0x6357,0x6357,0x6357,0x6257,0x6257,0x6357,0x6357,0x6357,0x6357,0x6357,0x5257,0x6357,0x5257,0x5757,0x4742,0x4752,0x6363,0x5257,0x4247,0x4752,0x4747,0x4745,0x4545,0x5247,0x4747,0xAFAF,0x6161,0x5151,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x4141,0x7F7F,0x7F41
.word 0x5050,0x5050,0x5050,0x5060,0x5050,0x5060,0x6060,0x5050,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xF5F5,0xFFFF,0xEFEF,0x7F7F,0xF4F4,0xBFBF,0x8F8F,0xF4F4,0xF5F5,0xAFAF,0x9F9F,0xCFCF,0xFFFF,0xBFBF,0x9F9F,0xAFAF,0x9F9F,0xCFCF,0xCFCF,0xBFBF,0x5050,0x9F9F,0x9F9F,0xAFAF,0x9D9D,0xBDBD,0xF0F0,0xF3F3,0xF3F3,0xF3F3,0xDFDF,0xCFCF,0xD0D0,0x6060,0xAFAF,0xAFAF,0xAFAF,0xC0C0,0xC0C0,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x5060,0x8F8F,0xE3E3,0xF4F4,0xF4F4,0xF4F4,0xE5E5,0xF4F4,0xF5F5,0xE5F4,0xCFCF,0xF4E5,0xCFCF,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xE5E5,0xE5E5,0xF6F6,0xE5E5,0xD5D5,0xD5D5,0xBFBF,0xC565,0x4742,0x52C5,0x5257,0xC557,0x5752,0x5762,0x5752,0x5763,0x5763,0x5763,0x5763,0x5763,0x5763,0x5763,0x5763,0x5763,0x5763,0x5752,0x4752,0x5252,0x4242,0x4747,0x4763,0x4747,0x4242,0x5252,0x5245,0x5247,0x4545,0x4252,0x5252,0xAFAF,0x9F9F,0x5151,0x8F8F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F
.word 0x6060,0x5050,0x5050,0x5050,0x6050,0x5050,0x5050,0x6060,0xAFAF,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xBFBF,0xFFFF,0xFFFF,0x8F8F,0x8F8F,0xCFCF,0xBFBF,0x9F9F,0xAFAF,0xF5F5,0xCFCF,0xAFAF,0xF5F5,0xE5E5,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xF5F5,0xCFCF,0x8F8F,0x6060,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBDBD,0xDFDF,0xDFDF,0xF3F3,0xE0E0,0xD0D0,0x7070,0x9F9F,0x7070,0x7070,0x8080,0xAFAF,0x8080,0x7070,0xAFAF,0x7070,0x6060,0x6060,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x6060,0x6060,0x5050,0x6060,0x6060,0x6060,0x9F9F,0x7050,0xF4F4,0xF4F4,0xF5F4,0xF5F4,0xF4F4,0xF4F5,0xF5F4,0xF5F5,0xE5E5,0xCFCF,0xCFCF,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xE5E5,0xE5E5,0xE5E5,0xE5E5,0xD5D5,0xC5C5,0x5555,0x4747,0x5752,0xC552,0x5263,0x63C5,0x6357,0x62C5,0x6257,0x5257,0x52C5,0x5257,0x6257,0x5257,0x52C5,0x5257,0x5257,0x6257,0x6257,0x6357,0x6367,0x4757,0x4542,0x4242,0x4243,0x4752,0x6547,0x4747,0x4752,0x5555,0x4742,0x5242,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x8F8F,0x7F7F
.word 0x6060,0x5050,0x5050,0x5050,0x6060,0x5050,0x5050,0x6060,0xBFBF,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xBFBF,0xF5F5,0xE5F5,0xBFBF,0x7F7F,0x9F9F,0xCFCF,0xAFAF,0x9F9F,0xAFAF,0xF5F5,0xDFDF,0xAFAF,0xBFBF,0xCFCF,0xAFAF,0xAFAF,0x9F9F,0xDFDF,0xEFEF,0x5050,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9E9E,0xBFBF,0xF0F0,0xE0E0,0xE0E0,0xBFBF,0xAFAF,0x7070,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x6070,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x5060,0x6060,0x6060,0x5050,0xF4F4,0xF4F5,0xF4F4,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xF4F4,0xE5E5,0xE5E5,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0xE5E5,0xE5E5,0xE5E5,0xD5D5,0xC4C4,0x6347,0x6357,0x57C5,0x4763,0x6767,0x4752,0x5757,0x4863,0x4763,0x6357,0x5747,0x5762,0x5763,0x5763,0x5752,0x5763,0x5763,0x5763,0x6763,0x4763,0x65C4,0x7373,0x4747,0x4742,0x4747,0x4747,0x5252,0x5247,0x5248,0x4752,0x4752,0x4242,0x5252,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F8F
.word 0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x5050,0x9F9F,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0xBFBF,0xCFCF,0xE5E5,0xF5F5,0x9F9F,0x7F7F,0x8F8F,0xF4F4,0xCFCF,0xBFBF,0xAFAF,0xF5F5,0xDFDF,0xBFBF,0xBFBF,0xBFBF,0xD4D4,0xAFAF,0x9F9F,0xF5F5,0x5050,0x6060,0x9F9F,0xAFAF,0xAFAF,0x8E8E,0x9E9E,0xCFCF,0xCFCF,0xBFBF,0x8080,0x6060,0x9F9F,0x7070,0x7070,0x8080,0x8080,0xAFAF,0x7070,0x7070,0x6060,0x7070,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x5060,0x5050,0x5050,0xE3E3,0xF4F4,0xF4F4,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xDFDF,0xBFBF,0xCFCF,0xAFAF,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x9F9F,0xE5E5,0xD5D5,0xAFAF,0x63C5,0x52C5,0x6257,0x5247,0x5257,0x6365,0x6357,0x6355,0x6257,0x5257,0x5752,0x6352,0x6357,0x6257,0xC457,0x6357,0x52C5,0x5257,0x52C5,0x6257,0x5757,0x5555,0x47B5,0x4763,0x4252,0x4242,0x4242,0x4755,0x4752,0x4752,0x4545,0x5247,0x4247,0x9F9F,0x5151,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F
.word 0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0xF6F6,0xDFDF,0xF5F5,0xF5F5,0xAFAF,0x8F8F,0xAFAF,0xF4F4,0xF4F4,0xAFAF,0xAFAF,0xE5E5,0xBFBF,0xD5D5,0xBFBF,0xDFDF,0xF5F5,0x4141,0xF4F4,0xBFBF,0x5050,0x5050,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBDBD,0xD0D0,0x8080,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x6070,0xAFAF,0x7070,0x7070,0x6060,0x6060,0xAFAF,0x7070,0x6060,0x6060,0x9F9F,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x5060,0x6060,0xF4E4,0xF5F4,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xF4F4,0xBFBF,0xCFCF,0xAFAF,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xD5D5,0xBFBF,0x52C5,0xC5C5,0x5752,0x4863,0x5762,0x5752,0x5247,0x5757,0x6347,0x5257,0xC557,0x5752,0x5257,0x5257,0x4757,0x4752,0x6565,0x6347,0x5257,0x4747,0x5752,0x6563,0x6563,0x5762,0x6363,0x4748,0x4747,0x4247,0x4343,0x5252,0x4545,0x5555,0x4752,0x5555,0x5252,0x9F9F,0x8F8F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F41
.word 0x9F9F,0x6060,0x5050,0x5050,0x6050,0x5050,0x5050,0x7070,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x8F8F,0x9F9F,0x8F8F,0xDFDF,0xF7F7,0xEFEF,0xFFFF,0xF5F5,0xCFCF,0x9F9F,0x7F7F,0xCFCF,0xF5F5,0xBFBF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xF5F5,0xEFEF,0xCFCF,0x7F7F,0xCFCF,0x6060,0x5050,0x6060,0xAFAF,0xAFAF,0x8E8E,0xBFBF,0xBFBF,0x6060,0x6060,0x7070,0xAFAF,0x7070,0xC0C0,0x8080,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x7070,0x7070,0x9F9F,0x7070,0x6060,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x6060,0xBFBF,0xF4F4,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xF4F4,0xE5E5,0xE4E4,0xAFAF,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x8F8F,0xD5D5,0x5757,0x4763,0x52C5,0x6357,0x5248,0x63C5,0x4752,0x4763,0x4752,0x5757,0x5252,0x6357,0x5752,0x5752,0x6352,0x6357,0x6347,0x5757,0x5752,0x5252,0x5257,0x6357,0x5757,0x6357,0x4763,0x6363,0x5552,0x4242,0x4247,0x4247,0x4545,0x4752,0x4242,0x4545,0x5242,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F
.word 0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x6060,0xAFAF,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x6060,0x5050,0x8F8F,0x6F6F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0xBFBF,0xEFEF,0xEFEF,0xF5F5,0xFFFF,0xFFFF,0xF5F5,0xAFAF,0x6F6F,0xCFCF,0xFFFF,0xFFFF,0xF5F5,0xE4E4,0xBFBF,0x8F8F,0xF5F5,0xEFEF,0x4141,0xAFAF,0xAFAF,0x6060,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9D9D,0xBFBF,0x7070,0x6060,0x7070,0x7070,0xBFBF,0xAFAF,0xBFBF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x6060,0x6060,0x7070,0x7070,0x9F9F,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x6050,0xF4F4,0xF4F4,0xF4F4,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xE5E5,0xCFCF,0xAFAF,0xAFAF,0x7F7F,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x9F9F,0x6347,0xC5C5,0x5752,0xC552,0x6363,0x5247,0x5757,0x4752,0x5763,0x6365,0x4747,0x5247,0x5757,0x5752,0x4863,0x4752,0x4752,0x5555,0x5757,0x4852,0x4742,0x6363,0x6363,0x5247,0x5248,0x5247,0x4747,0x4745,0x4242,0x4242,0x4242,0x63C5,0x4747,0x4242,0x4747,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x6060,0x5050,0x5060,0x5050,0x6060,0x5050,0x5050,0x9F9F,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x5050,0x6060,0x7F7F,0x5F5F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0xEFEF,0xF7F7,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xF4F4,0x9F9F,0x7F7F,0x9F9F,0xE5E5,0xF5F5,0xFFFF,0xFFFF,0xF5F5,0x7F7F,0xF4F4,0xBFBF,0x5151,0x7070,0x6060,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xC0C0,0x7070,0xAFAF,0xAFAF,0x8080,0x8080,0x8080,0xBFBF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x7070,0x7070,0x7060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x6060,0xD4D4,0xE4E4,0xF5F4,0xF5F5,0xEFEF,0xF5F5,0xE5E5,0xDFDF,0xE5E5,0xAFAF,0x9F9F,0x4141,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4242,0xC5C5,0x6355,0x6347,0x5257,0x6557,0x4752,0x5252,0x5257,0x5757,0x4763,0x6363,0x5752,0x5252,0x5257,0x5257,0x5257,0x5257,0x5555,0x5255,0x5247,0x4343,0x4743,0x4755,0x6363,0x5752,0x4343,0x5252,0x4242,0x4247,0x4747,0xC447,0x6363,0x5257,0xC552,0x5252,0x5252,0x8F8F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F8F
.word 0x6060,0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x6060,0x5050,0x9F9F,0x7F7F,0x5F5F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xDFDF,0xEFEF,0xEFEF,0xF7F7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0xF5F5,0xFFFF,0xF4F4,0x8F8F,0x6161,0x6161,0x6061,0x6060,0x9F9F,0xAFAF,0x7070,0xAFAF,0x9D9D,0xAFAF,0xD0D0,0xBFBF,0x7070,0x8080,0x7070,0x7070,0x7070,0xC0C0,0xBFBF,0x8080,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x9F9F,0x7070,0x7070,0x9F9F,0x7070,0xAFAF,0x6060,0x6060,0x7070,0xAFAF,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x5050,0xE4E4,0xF4F4,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xDFDF,0x9F9F,0x8F8F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4747,0x4242,0x5757,0x5755,0x5763,0x4862,0x5252,0x4747,0x5763,0x4763,0x5752,0x5755,0x5247,0x6348,0x5757,0x4763,0x4752,0x5555,0x5247,0x5257,0x4747,0x4745,0x5252,0x5247,0x4747,0x5247,0x4742,0x4742,0x4242,0x4242,0x6363,0x5555,0x4747,0x5252,0x4747,0xC5C5,0x5757,0x5151,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F41
.word 0x9F9F,0x6060,0x5050,0x5050,0x5050,0x5050,0x6060,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x5050,0x5050,0x4040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xBFBF,0xF7F7,0xEFEF,0xFFFF,0xF7F7,0xFFFF,0xFFFF,0xFFFF,0xF4F4,0xCFCF,0x9F9F,0x6F6F,0x4141,0x8F8F,0xCFCF,0xE4E4,0x6161,0x5151,0x6161,0x6171,0x7070,0x6060,0x9F9F,0x6060,0xAFAF,0x9E9E,0xBFBF,0xDFDF,0xE0E0,0xD0D0,0x7070,0x7070,0x6060,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xC0C0,0x7070,0x9F9F,0x7070,0x7070,0x9F9F,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x6070,0x7070,0x6060,0x9F9F,0x6060,0x7070,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x5050,0x6161,0xF4F4,0xF5F5,0xEFEF,0xF5F5,0xF5F5,0xDFDF,0xF4F4,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5242,0x4747,0x6357,0x5763,0x5752,0xC552,0x5257,0x4747,0x5242,0x5257,0x6357,0x6357,0x6565,0x4763,0x5555,0x5563,0x4747,0x4247,0x4743,0x4752,0x4747,0x4555,0x5252,0x4347,0x4747,0x4242,0x4242,0x4242,0x4242,0x4742,0x4242,0xC5C5,0x6363,0x6363,0x4747,0x5252,0x5751,0x5262,0x4747,0x4141,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F
.word 0x6060,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x6060,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x5050,0x5050,0x9F9F,0x8F8F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xAFAF,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF3F3,0xBFBF,0xAFAF,0x9F9F,0x4141,0xE3E4,0xD4D4,0x6161,0x6161,0x7171,0x6061,0x6060,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xF0F0,0xEFEF,0xE0E0,0xBFBF,0xAFAF,0x7070,0x7070,0xAFAF,0xAFAF,0xD0D0,0xBFBF,0xBFBF,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x7070,0x7070,0x7070,0x9F9F,0x9F9F,0x6060,0x6060,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0xF4E4,0xF5F5,0xF5F5,0xEFEF,0xF5F5,0xF5F5,0xCFCF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4747,0xC5C5,0x6363,0x5248,0x5257,0x47C5,0x5252,0x4747,0x4852,0x5763,0xC552,0x6363,0x6357,0x5257,0x6347,0x5747,0x5552,0x4242,0x4747,0x4247,0x5255,0x5247,0x4247,0x4747,0x4242,0x4842,0x4242,0x4242,0x4747,0x4242,0x4242,0x6347,0x6363,0x4747,0x5252,0x4747,0x5252,0x5757,0x5252,0x4242,0x4141,0x4141,0x7F7F,0x4141,0x8F7F
.word 0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x7F7F,0x6F6F,0x5050,0x9F9F,0x8F8F,0x9F9F,0x7F7F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0xBFBF,0xF7F7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF5F5,0xFFFF,0xDFDF,0x8F8F,0x8F8F,0x6172,0xD471,0x6271,0x7271,0x7172,0x6060,0x7070,0x6060,0xAFAF,0x9D9D,0xBFBF,0xFEFE,0xFFFF,0xF0F0,0xE0E0,0xD0D0,0x8080,0xAFAF,0x8080,0xBFBF,0xBFBF,0xD0D0,0xC0C0,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x9F9F,0x6060,0x7070,0x9F9F,0x6070,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7050,0xE4E4,0xF4F4,0xDFDF,0xF5F5,0xF5F5,0xDFDF,0xDFDF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4242,0x5757,0x5555,0x6347,0x6363,0x52C5,0x5247,0x4757,0x5555,0x5257,0x5757,0x6357,0x6557,0x5763,0x5763,0x5752,0x5252,0x5247,0x4747,0x4242,0x4742,0x4747,0x4242,0x4742,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4742,0x4842,0x5252,0x4765,0x4762,0x4747,0x4263,0x4242,0x5245,0x5247,0x5248,0x8F8F,0x4141,0x8F8F,0x8F8F,0x7F41
.word 0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x6F6F,0x6F6F,0x8F8F,0x5050,0x5050,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0xAFAF,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0x4141,0x7171,0x7161,0xC3C3,0x8272,0x7172,0x7071,0x9F9F,0xAFAF,0x6060,0x7070,0x9E9E,0xEFEF,0xFFFF,0xFFFF,0xEFEF,0xCFCF,0xCFCF,0x8080,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xD0D0,0xAFAF,0x7070,0x6060,0xAFAF,0x7070,0x6060,0x7070,0xAFAF,0x7070,0x9F9F,0x6060,0x6060,0x7060,0x6060,0x6060,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x5050,0x9F9F,0xE4E4,0xF5F4,0xF5F5,0xDFDF,0xF5F5,0xDFDF,0xAFAF,0xAFAF,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4747,0x5252,0x6565,0x4863,0x4763,0x5752,0x4752,0x4343,0x6347,0x5757,0x6365,0x5247,0x52C5,0x5257,0x5257,0x5257,0x4247,0x4842,0x4752,0x4842,0x4242,0x5247,0x4247,0x4242,0x4242,0x4742,0x4242,0x4747,0x4747,0x4242,0x4242,0x6363,0x6357,0x6363,0x6247,0x6363,0x4747,0x4242,0x4242,0x4747,0x4752,0x4242,0x8F8F,0x4040,0x7F7F,0x8F7F
.word 0x9F9F,0x6060,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x7F7F,0x6F6F,0x6060,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0xAFAF,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF5F5,0xF7F7,0xF5F7,0xF5F5,0xF5F5,0xEFEF,0xFFFF,0xFFFF,0xBFBF,0x6241,0x6171,0x7261,0x8282,0x8282,0x80B3,0x8080,0x6060,0xAFAF,0xAFAF,0xAFAF,0xDFDF,0xFFFF,0xFFFF,0xF0F0,0xF0F0,0xD0D0,0xBFBF,0xAFAF,0x8080,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xC0C0,0xBFBF,0xAFAF,0x7070,0x6060,0x7070,0x6060,0x7070,0x6060,0x6070,0x7070,0x6060,0x9F9F,0x6060,0x7070,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6160,0xE4E4,0xE4E4,0xF5F5,0xF5F5,0xEFEF,0xF5F5,0xBFBF,0xAFAF,0x9F9F,0x5151,0x5151,0x5151,0x4141,0x4141,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x6F6F,0x6F6F,0x6F6F,0x4242,0x6357,0x6557,0x5555,0x6363,0x4747,0x4257,0x5247,0x5555,0x5773,0x5763,0x5763,0x5752,0x5757,0xC552,0x5752,0x4752,0x4242,0x4242,0x4242,0x4248,0x4747,0x4747,0x7F7F,0x4242,0x7F7F,0x4242,0x4242,0x5242,0x4747,0x4747,0x5555,0x6363,0x7447,0x5757,0x5763,0x6363,0x4747,0x4242,0x4242,0x4242,0x4742,0x5252,0x8F8F,0x4040,0x7F8F
.word 0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x6F6F,0x5050,0x9F9F,0x6060,0x4040,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0xAFAF,0xDFDF,0xF7F7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF7F7,0xF5F5,0xF5F5,0xEFEF,0xF5F5,0xEFEF,0xF5F5,0xFFFF,0xEFEF,0x6F6F,0xC361,0x7272,0x7272,0x8282,0x8282,0x7071,0x9F9F,0x6060,0x7070,0xAFAF,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xD0D0,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x7070,0x7070,0x9F9F,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x7070,0x6060,0x7060,0x6060,0x6060,0x6060,0x6060,0x5050,0x6161,0xE4E4,0xF4F4,0xF5F5,0xF5F5,0xDFDF,0xBFBF,0x9F9F,0x6161,0x6161,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4762,0x4242,0x4242,0x4242,0x7F7F,0x4242,0x6F6F,0x6F6F,0x4242,0x4747,0x5252,0x6363,0x4763,0x5755,0x5555,0x4752,0x4752,0xC547,0x6357,0x6257,0x5757,0x6357,0x6365,0x5257,0x4747,0x4747,0x4242,0x4247,0x4247,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4747,0x4747,0x6263,0x5757,0x5763,0x6363,0x7365,0x6363,0x4747,0x73C4,0x4747,0x4242,0x5247,0x4247,0x9F9F,0x5151,0x7F7F,0x8F7F
.word 0x6060,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x6F6F,0x5050,0x5050,0x5050,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x9F9F,0xEFEF,0xEFEF,0xF7F7,0xF7F7,0xF7F7,0xF5F5,0xF5F5,0xDFDF,0xE5E5,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0x4141,0x6141,0x8272,0x8282,0x8272,0x8282,0x7171,0x6060,0x9F9F,0x9F9F,0x8E8E,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF0F0,0xDFDF,0xCFCF,0xBFBF,0xC0C0,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0x7070,0x7070,0xAFAF,0x6060,0x7070,0x9F9F,0x6070,0x6070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0xE4E4,0xE4E4,0xF5F5,0xF5F5,0xDFDF,0xBFBF,0x9F9F,0x9F9F,0x6060,0x7070,0x6161,0x5151,0x9F9F,0x5252,0x5757,0x4763,0x4242,0x4242,0x4242,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4242,0x4747,0x4747,0x4747,0x5763,0x4763,0x4752,0x5555,0x6363,0x4852,0x4862,0x6563,0x5763,0x4747,0x4852,0x5252,0x5247,0x4747,0x7F7F,0x4242,0x4242,0x8F8F,0x4242,0x7F7F,0x6F6F,0x4242,0x4747,0x4242,0x5242,0x6247,0x5757,0x6263,0x6363,0x5757,0x6363,0x6257,0x6257,0x4747,0x6363,0x4747,0x4242,0x4242,0x5247,0x5252,0x4141,0x418F
.word 0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x8F8F,0x6F6F,0x6060,0x5050,0x9F9F,0x6060,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0xAFAF,0xFAFA,0xDFDF,0xEFEF,0xEFEF,0xEFEF,0xDFDF,0xF5F5,0xF5F5,0xDFDF,0xEFEF,0xFFFF,0xEFEF,0xEFEF,0xF5F5,0xF5F5,0xAFAF,0x6161,0x7271,0x8272,0x7282,0x8272,0x71B3,0x8080,0xAFAF,0xAFAF,0xAFAF,0xEEEE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xE0E0,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0xE4E4,0xF4F4,0xF5F5,0xEFEF,0xBFBF,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x6262,0x6565,0x4242,0x4242,0x4242,0x8F8F,0x4242,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x6363,0x5763,0x5752,0x4752,0x4752,0x4755,0x4863,0x4763,0x5752,0x5763,0x6363,0x4765,0x4242,0x4747,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x4747,0x4747,0x4747,0x6363,0x5757,0x5752,0xC463,0x6363,0x4863,0x5762,0x6363,0x52C5,0x6363,0x4242,0x4242,0x4747,0x6363,0x4242,0x8F42
.word 0x6060,0x6060,0x5050,0x8F8F,0x5050,0x8F8F,0x6050,0x8F8F,0x7F7F,0x5050,0x5050,0x5050,0x6060,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x9F9F,0xDFDF,0xF7F7,0xEFEF,0xEFEF,0xF5F5,0xF5F5,0xF7F7,0xF5F5,0xF7F7,0xFFFF,0xFFFF,0xFFFF,0xF5F5,0xF5F5,0xE5F5,0x9F9F,0x6F6F,0x7261,0x7272,0x8282,0x7282,0xB382,0x8072,0x8080,0xAFAF,0xAFAF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xE0E0,0xBFBF,0xC0C0,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xC0C0,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0xE4E4,0xE5E5,0xDFDF,0xF5F5,0xCFCF,0xAFAF,0x8F8F,0x6060,0x6161,0x6060,0x6060,0x5151,0x9F9F,0x62C5,0x4747,0x4747,0x4848,0x4242,0x7F7F,0x4242,0x4242,0x4747,0x7F7F,0x4242,0x6357,0x5257,0x4747,0x5257,0x5257,0x5257,0x6257,0x5757,0x4763,0x6363,0x5755,0x5252,0x4247,0x4242,0x4242,0x4747,0x4747,0x4646,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4747,0x4545,0x4545,0x4545,0x4752,0x6262,0xC5C5,0x4747,0x63D5,0x5252,0x4747,0x5247,0xC552,0x6565,0x4742,0x4242,0x4242,0x5252,0x4257,0x428F
.word 0x9F9F,0x5050,0x5050,0x5050,0x5060,0x5050,0x9F9F,0x8F8F,0x5050,0x5050,0x6060,0x5050,0x6060,0x6060,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x9F9F,0xDFDF,0xDFDF,0xF7F7,0xF7F7,0xDFDF,0xEFEF,0xEFEF,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xF7F7,0xF5F5,0xF5F5,0xDFDF,0x8F8F,0x6F6F,0x7152,0x7272,0x7282,0x8272,0x7272,0xB381,0xC0C0,0xAFAF,0x9F9F,0x9D9D,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xE0E0,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xC0C0,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6161,0xE4E4,0xE5E5,0xDFDF,0xCFCF,0xAFAF,0x8F8F,0x7070,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6363,0x6363,0x9F9F,0x4747,0x4242,0x4242,0x8F8F,0x4242,0x4248,0x4242,0x4747,0x4747,0x4852,0x5552,0x4752,0x4747,0x6363,0x4747,0x6357,0x5257,0x4755,0x5555,0x4747,0x4242,0x4242,0x4247,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x7F7F,0x4242,0x4747,0x4242,0x4747,0x5252,0x5247,0x6357,0x5757,0x5252,0xC552,0x6363,0x47C4,0x4343,0x4752,0x6363,0x6363,0x4747,0x4752,0x4242,0x4742,0x5252,0x428F
.word 0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x5050,0x5050,0x4040,0x8F8F,0x6060,0x9F9F,0x5050,0x9F9F,0x6060,0x5050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x9F9F,0xDFDF,0xCFCF,0xDFDF,0xDFDF,0xF5F5,0xFAFA,0xF7F7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF5F5,0xF5F5,0xF5F5,0xDFDF,0x7F7F,0x4141,0x6251,0x7271,0x8272,0x7282,0x8282,0x7282,0x7171,0xAFAF,0xAFAF,0xAFAF,0xFEFE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xCFCF,0xCFCF,0xD0D0,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x7F7F,0x6060,0x9F9F,0x6161,0x6161,0x6161,0x7373,0x4747,0x6363,0x5757,0x5252,0x4242,0x4747,0x8F8F,0x4747,0x5252,0x4242,0x5247,0x5247,0x5247,0x4747,0x5247,0x6347,0x6565,0x4763,0x4763,0x5752,0x6565,0x4763,0x4242,0x4646,0x4242,0x4242,0x4242,0x4747,0x4747,0x4242,0x4742,0x4242,0x4242,0x4646,0x4242,0x4747,0x5255,0x4747,0x4747,0x4747,0x5255,0x5757,0x5252,0xC5C5,0x5252,0x4763,0x4242,0x6347,0x52C5,0x4545,0x4747,0x4252,0x5252,0x4242,0x8F42
.word 0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x6060,0x6060,0x5050,0x6060,0x9F9F,0x6060,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0xAFAF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xEFEF,0xFFFF,0xFFFF,0xF7F7,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xCFCF,0x4141,0x4141,0x6251,0x7272,0x7272,0x8272,0x7272,0x81B3,0x7171,0x7070,0x8E8E,0x9F9F,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xEFEF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x9F9F,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7070,0x6060,0x9F9F,0x9F9F,0x9F9F,0x6363,0x5257,0x4747,0x4747,0x4747,0x4747,0x9F9F,0xAFAF,0x4247,0x4248,0x4747,0x4747,0x4747,0x5752,0x5752,0x4752,0x4247,0x4747,0x4747,0x6347,0x5257,0x5248,0x4747,0x4242,0x4747,0x4242,0x4247,0x4242,0x4242,0x4242,0x4242,0x4747,0x7F7F,0x4242,0x4747,0x4752,0x9F9F,0x4752,0x4752,0x4752,0x4747,0x4752,0x5742,0x6352,0xC5C5,0x5247,0x4747,0x5252,0x5763,0x5252,0x4763,0x4242,0x4747,0x4747,0x7F42
.word 0x6060,0x5050,0x5050,0x8F8F,0x8F8F,0x5050,0x5050,0x6060,0x6060,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x6060,0x9F9F,0x5050,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xAFAF,0xCFCF,0xDFDF,0xCFCF,0xF6F6,0xF6F6,0xF5F5,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xF5F5,0xF5F5,0xF5F5,0xE5E5,0x6F6F,0x4141,0x6161,0x7271,0x8272,0x7272,0x7282,0xC381,0x71C3,0xAFAF,0xAFAF,0x8E8E,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xF0F0,0xCFCF,0xD0D0,0xBFBF,0xBFBF,0xBFBF,0xC0C0,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x6060,0x6060,0x7060,0x6060,0x6060,0x7060,0x6161,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7070,0x6060,0x6060,0x6161,0xC5C5,0x5757,0x4862,0x4763,0x4242,0x4747,0x5555,0xC5C5,0x5252,0x4747,0x4242,0x5247,0x4242,0x4752,0x6363,0x4747,0x4242,0x4242,0x5552,0x4752,0x5752,0x5752,0x4752,0x4242,0x4242,0x4646,0x4642,0x4242,0x4242,0x4747,0x4747,0x4247,0x4242,0x4242,0x4242,0x4545,0x9F9F,0x4742,0x5252,0x6363,0x5252,0x6363,0x4747,0x4343,0x5248,0x5252,0x5252,0x4742,0x5752,0x6262,0x5547,0x5252,0x4747,0x4242,0x6552,0x429F
.word 0x8F8F,0x6060,0x5050,0x5050,0x5050,0x5050,0x6060,0x8F8F,0x5050,0x5050,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x9F9F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0xBFBF,0xBFBF,0xCFCF,0xFAFA,0xCFCF,0xCFCF,0xDFDF,0xF7F7,0xFFFF,0xF7F7,0xF7F7,0xF5F5,0xF5F5,0xF5F5,0xF5F5,0xBFBF,0x6F6F,0x5151,0x6261,0x7162,0x7272,0x8272,0x8272,0x8272,0x7171,0xAFAF,0x9D9D,0xBFBF,0xEEEE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xEFEF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x6060,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x6060,0x8F8F,0x7F7F,0x7F7F,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7352,0xC5C5,0x5252,0x4747,0x5752,0x5252,0xAFAF,0x4747,0x4242,0x4747,0x4242,0x4747,0x4747,0x4752,0x4247,0x4743,0x4747,0x4247,0x5248,0x5555,0x5257,0x4747,0x4747,0x4742,0x4242,0x4747,0x4742,0x4646,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x4242,0x4747,0x5742,0x4852,0x5752,0x5757,0x5752,0x4752,0x5252,0x4247,0x5248,0x4247,0x5252,0x5757,0x5252,0x4752,0x4242,0x4242,0x4747,0x5247
.word 0x5060,0x5050,0x9F9F,0x5050,0x9F9F,0x5050,0x8F8F,0x6060,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xF5F5,0xF5F5,0xEFEF,0xF7F7,0xEFEF,0xF5F5,0xF5F5,0xE5E5,0xF5F5,0xDFDF,0xAFAF,0x6F6F,0x4141,0x6161,0x7271,0x7272,0x7272,0x8282,0x7282,0xC072,0xAFAF,0xAFAF,0x9D9D,0xAFAF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFEFE,0xEFEF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x8080,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x6060,0x6060,0x7070,0x7070,0x6060,0x5050,0x8F8F,0x6F6F,0x7F7F,0x6060,0x6060,0x6161,0x6363,0xC5C5,0x5257,0x4747,0x4747,0xAFAF,0xAFAF,0x4242,0x4242,0x4747,0x5247,0x4747,0x4242,0x4242,0x4747,0x4242,0x4742,0x4752,0x4747,0x5252,0x4547,0x4752,0x4343,0x4242,0x4747,0x4742,0x4242,0x4242,0x4242,0x4245,0x4242,0x4747,0x4646,0x4242,0x8F8F,0x4545,0x4242,0x4242,0x4747,0x4242,0x5252,0x5555,0x6363,0x4747,0x4745,0x9F9F,0x5252,0x4242,0x4247,0x5247,0x5555,0x5252,0x4852,0x4747,0x4242,0x9F42
.word 0x9F9F,0x5050,0x6060,0x5050,0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x6060,0x5050,0x6060,0x9F9F,0x9F9F,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x9F9F,0x4141,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0xE5E5,0xDFDF,0xCFCF,0xDFDF,0xF5F5,0xDFDF,0xF5F5,0xDFDF,0xE5E5,0xCFCF,0xF5F5,0xCFCF,0x7F7F,0x7F7F,0x5151,0x6261,0x7162,0x7272,0x7272,0x8272,0x8272,0x6172,0xAFAF,0xAFAF,0xAFAF,0x9D9D,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xC0C0,0xAFAF,0x8080,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x6060,0x6060,0x6060,0x7070,0x6060,0x4040,0x6F6F,0x7F7F,0x9F9F,0x6262,0x5252,0x6257,0x4747,0x5252,0x5257,0x5757,0xAFAF,0x9F9F,0x4747,0x4752,0x4747,0x4242,0x4242,0x4747,0x4243,0x4242,0x4242,0x5247,0x4747,0x4355,0x4747,0x4545,0x4742,0x4242,0x4742,0x4242,0x4248,0x4242,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x8F8F,0x4242,0x4242,0x4747,0x5247,0x5247,0x5248,0x4752,0x4242,0x4242,0x4247,0x4247,0x4242,0x4242,0x4763,0x4852,0x5252,0x5242,0x4747,0x4242
.word 0x6050,0x6060,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x9F9F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0xAFAF,0xCFCF,0xF5F5,0xDFDF,0xDFDF,0xF6F6,0xE5E5,0xCFCF,0xF5F5,0xDFDF,0xAFAF,0x6F6F,0x7F7F,0x7F7F,0x6161,0x7261,0x7271,0x8282,0x7282,0x7282,0xD471,0xC0C0,0xAFAF,0x8E8E,0xAFAF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xFDFD,0xEFEF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8080,0x8080,0x7070,0xAFAF,0x7070,0xAFAF,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x7070,0x7070,0x6060,0x4040,0x7F7F,0x4141,0x6161,0xAFAF,0x9F9F,0x5247,0x5555,0x5757,0x6262,0xC4C4,0x4257,0x5757,0x5252,0x5555,0x4242,0x4242,0x4742,0x4747,0x4242,0x4747,0x4242,0x5552,0x4747,0x4242,0x4742,0x4242,0x4742,0x4242,0x4743,0x4242,0x4247,0x4747,0x4242,0x4646,0x4242,0x4242,0x4242,0x4545,0x9F9F,0x4242,0x5252,0x9F9F,0x4242,0x8F8F,0x4747,0x4747,0x4242,0x4545,0x5555,0x4242,0x4848,0x4742,0x4742,0x5252,0x5252,0x4747,0x4747,0x4752,0x429F
.word 0x9F9F,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0xCFCF,0xDFDF,0xE5E5,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5141,0x7161,0x8272,0x8272,0x9382,0x8282,0x72B3,0xC061,0xAFAF,0xBFBF,0x8E8E,0xAFAF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFEFE,0xFDFD,0xDFDF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x8080,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x7070,0x6070,0x7070,0x6060,0x7070,0x7070,0x6060,0x6060,0x5050,0x5151,0x6262,0x5252,0x5242,0x5252,0xC5C5,0xC4C4,0xC5C5,0xC5C5,0x5252,0x4747,0x4747,0x4242,0x4747,0x4747,0x4242,0x4742,0x4742,0x4242,0x4247,0x4247,0x4247,0x4242,0x4747,0x4242,0x4747,0x4242,0x4247,0x4747,0x4242,0x4242,0x7F7F,0x4242,0x4646,0x4242,0x7F7F,0x4242,0x4242,0x8F8F,0x4242,0x4747,0x4242,0x4242,0x4545,0x4252,0x4747,0x4747,0x5555,0x6565,0x4242,0x4242,0x4242,0x6363,0x5547,0x5252,0x4242,0x6363,0x4247
.word 0x5060,0x6060,0x5050,0x5050,0x5050,0x6060,0x6060,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x6060,0x5050,0x5050,0x5050,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0xAFAF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0x9F9F,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7261,0x7272,0x8282,0x8282,0x8282,0xB282,0xB3B3,0xC0C0,0xAFAF,0x9D9D,0xBFBF,0xFEFE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xC0C0,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x9F9F,0x6161,0x9F9F,0x8F8F,0xC5C5,0xC4C4,0xC5C5,0xC5C5,0x6363,0x4747,0x7373,0x4242,0x4242,0x4242,0x4247,0x4842,0x4242,0x4242,0x4247,0x4242,0x4747,0x4242,0x4646,0x4242,0x4242,0x4242,0x4747,0x4742,0x4242,0x4242,0x4646,0x4642,0x4242,0x4242,0x4242,0x4742,0x8F8F,0x8F8F,0x4242,0x4242,0x4242,0x4242,0x4747,0x4545,0x4747,0x5252,0x4242,0x5555,0x6363,0x4747,0x4752,0x4742,0x5742,0x5252,0x4247,0x4747,0x4545,0x529F
.word 0x9F9F,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x9F9F,0x6060,0x8F8F,0x6060,0x5050,0x9F9F,0x5050,0x9F9F,0x5050,0x9F9F,0x5050,0x5050,0x6060,0x6060,0x5050,0x9F9F,0x5050,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x7161,0x8272,0x8282,0x8282,0x92B2,0xB292,0x8292,0x8182,0xBFBF,0xAFAF,0x9D9D,0xDFDF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xFDFD,0xFDFD,0xEFEF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7060,0x6060,0x7070,0x6060,0x6060,0x6060,0x6161,0x6262,0x4242,0x4242,0x5252,0xC5C5,0xC5C5,0x6363,0xC5C5,0x4242,0x4747,0x5252,0x4242,0x4242,0x4242,0x4247,0x4247,0x4247,0x4242,0x4247,0x4242,0x4247,0x4242,0x4242,0x4247,0x4242,0x4742,0x4242,0x4248,0x4747,0x4242,0x4242,0x4242,0x4747,0x4242,0x7F7F,0x4242,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4242,0x5555,0x4747,0x4545,0x4545,0x4247,0x5247,0x5757,0x6363,0x4242,0x9F9F,0x6247,0x5757,0x5752,0x4242,0x4242,0x9F55
.word 0x6060,0x6060,0x5050,0x5050,0x6060,0x8F8F,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7161,0x8272,0x8282,0x8282,0xA392,0x8292,0x9292,0x81B2,0xB0B0,0xAFAF,0xAFAF,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xFCFC,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7060,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x6161,0xAFAF,0x4747,0x4747,0xC4C4,0xC4C4,0x6363,0x5757,0x5252,0x5248,0x5252,0x4248,0x7F7F,0x4247,0x4248,0x4242,0x8F8F,0x4242,0x4242,0x4747,0x4747,0x4747,0x4242,0x4242,0x4242,0x4742,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4242,0x8F8F,0x7F7F,0x4242,0x4242,0x4242,0x5252,0x4747,0x4742,0x4242,0x4242,0x4242,0x4752,0x6363,0x6565,0x6347,0x9F9F,0x4762,0x5252,0x4747,0x4747,0x4242
.word 0x9F9F,0x5050,0x6060,0x9F9F,0x9F9F,0x6060,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x6060,0x8F8F,0x5050,0x9F9F,0x6060,0x9F9F,0x5050,0x9F9F,0x5050,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5140,0x6261,0x8271,0x8282,0x92B2,0x9282,0x8292,0x8292,0xB382,0xC071,0x9E9E,0x8E8E,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xEFEF,0xFDFD,0xDFDF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7060,0x6060,0x6060,0x9F9F,0x6060,0x6161,0x5252,0x9F9F,0x4242,0x5252,0xC5C4,0x6565,0x6357,0x4747,0x4747,0x4747,0x5252,0x4242,0x4747,0x4242,0x4747,0x4242,0x4242,0x4747,0x4747,0x4545,0x4242,0x7F7F,0x4742,0x4242,0x4747,0x4242,0x4747,0x4247,0x4747,0x4242,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x9F9F,0x8F8F,0x4141,0x7F7F,0x8F8F,0x4242,0x4747,0x4545,0x5252,0x4742,0x4242,0x5248,0x4747,0x4752,0x4747,0x5252,0x5247,0x4747,0x5555,0x4247,0x5252,0x8F9F
.word 0x6060,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x6060,0x6060,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x5151,0x6161,0x8272,0x8282,0x8292,0x8292,0x9282,0x9282,0x8182,0xC3C3,0xAFAF,0xAFAF,0xFEFE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xFDFD,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x9F9F,0x9F9F,0x6161,0x4242,0x4242,0xC5C5,0x9F9F,0x6363,0x5252,0x5757,0x5252,0x6565,0x4242,0x4242,0x4242,0x4742,0x4242,0x8F8F,0x4747,0x4242,0x4242,0x4242,0x8F8F,0x4242,0x4242,0x4747,0x4242,0x4747,0x4242,0x7F7F,0x4242,0x7F7F,0x4242,0x4242,0x4242,0x7F7F,0x4242,0x4747,0x8F8F,0x4141,0x8F8F,0x4242,0x8F8F,0x4242,0x4545,0x4747,0x4747,0x4742,0x4242,0x6363,0x4747,0x5252,0x4747,0x5151,0x5252,0x5555,0x4242,0x4747,0x4347
.word 0x6060,0x9F9F,0x5050,0x5050,0x9F9F,0x5050,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x6060,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x6060,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x4040,0x4040,0x5151,0x6161,0x7271,0x8282,0x8292,0x9292,0x9292,0x8292,0x7182,0x7171,0xAFAF,0x8E8E,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x8080,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7070,0x6060,0x7070,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6070,0x6060,0x6060,0x6060,0x6060,0x7070,0x9F9F,0x9F9F,0x4242,0x4747,0x6373,0xC5C5,0x4747,0x5752,0x5757,0x5762,0x4242,0x4242,0x4747,0x4242,0x4747,0x4242,0x4742,0x4747,0x4747,0x4242,0x4646,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x8F8F,0x8F8F,0x4242,0x4242,0x4747,0x8F8F,0x5050,0x8F8F,0x4141,0x4242,0x4242,0x8F8F,0x4242,0x4242,0x4242,0x4242,0x4742,0x4242,0x4752,0x4747,0x5252,0x5247,0x4747,0x4545,0x4247,0x4752,0x529F
.word 0x5050,0x5060,0x6060,0x8F8F,0x5151,0x8F8F,0x5050,0x9F9F,0x9F9F,0x9F9F,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x9F9F,0x5050,0x9F9F,0x9F9F,0x5050,0x6060,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x5151,0x5151,0x6161,0x7271,0x8282,0x9282,0x8292,0x9292,0x9292,0x7282,0xC0C0,0x8E8E,0xAFAF,0xAFAF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0x9F9F,0x6060,0x6060,0x7070,0x7060,0x9F9F,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x6060,0x6060,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x6262,0x5757,0x5252,0x5252,0x4747,0x6363,0x4747,0x4242,0x4747,0x4242,0x4747,0x4242,0x4747,0x4242,0x4242,0x4242,0x4247,0x4242,0x4747,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x8F8F,0x8F8F,0x8F8F,0x4242,0x4242,0x7F7F,0x4242,0x5252,0x8F8F,0x4040,0x4141,0x8F8F,0x8F8F,0x4242,0x4242,0x8F8F,0x4242,0x4747,0x4242,0x4747,0x4242,0x5242,0x5252,0x4242,0x4752,0x4752,0x4852,0x5252,0x9F45
.word 0x5050,0x9F9F,0x8F8F,0x5050,0x8F8F,0x9F9F,0x8F8F,0x5151,0x5151,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5050,0x9F9F,0x5050,0x5050,0x8F8F,0x5050,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x4141,0x5050,0x5151,0x8F8F,0x7261,0x8282,0x9292,0x9282,0x9292,0x9292,0x8182,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDFD,0xFFFF,0xEFEF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x6060,0x7070,0x9F9F,0x7070,0x7070,0x6060,0x6070,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x6060,0xAFAF,0x6161,0x9F9F,0x4242,0x5763,0x4242,0xC5C5,0x4747,0x6363,0x4747,0x4242,0x4747,0x4242,0x4747,0x5252,0x4747,0x5555,0x4842,0x4742,0x4747,0x4242,0x4742,0x4242,0x4242,0x4247,0x4747,0x4242,0x4747,0x8F8F,0x4242,0x9F9F,0x8F8F,0x8F8F,0x4747,0x4242,0x4242,0x9F9F,0x8F8F,0x4040,0x8F8F,0x4141,0x4242,0x4242,0x4747,0x4747,0x4242,0x4242,0x4242,0x4247,0x4545,0x4747,0x4747,0x4747,0x4242,0x5252,0x4242,0x4545,0x429F
.word 0x9F9F,0x6060,0x6060,0x5050,0x5151,0x4141,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x5151,0x5151,0x5050,0x9F9F,0x5050,0x9F9F,0x9F9F,0x6060,0x9F9F,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4141,0x4040,0x4141,0x5151,0xC361,0xB282,0x8292,0x9292,0x9292,0x8292,0x8182,0x7171,0xBFBF,0x8E8E,0x9D9D,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0xAFAF,0x6070,0x6060,0x7070,0xAFAF,0x7070,0x6060,0x7070,0x7070,0x9F9F,0x7070,0x6070,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4242,0x6363,0x5247,0x4242,0x4242,0x5252,0x5247,0x4242,0x4747,0x4842,0x4242,0x4747,0x4747,0x4742,0x5252,0x4242,0x4747,0x4742,0x4242,0x4747,0x4742,0x4242,0x4742,0x4747,0x4242,0x8F8F,0x8F8F,0x4242,0x4242,0x8F8F,0x8F8F,0x4242,0x4242,0x8F8F,0x8F8F,0x4141,0x4141,0x4141,0x5151,0x4242,0x4747,0x4242,0x4842,0x4242,0x4242,0x7F7F,0x4747,0x4242,0x4242,0x4245,0x4742,0x4747,0x4742,0x4242,0x4742
.word 0x6060,0x6060,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x8F8F,0x7261,0x9282,0x9292,0x9292,0x9292,0x8292,0x7282,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x7070,0x7070,0x7070,0x6060,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x6060,0xAFAF,0x6060,0x5151,0x8F8F,0x4242,0x5757,0x47C5,0x5252,0x4242,0x4747,0x4242,0x4747,0x4242,0x4242,0x5247,0x5555,0x4752,0x4242,0x4247,0x5757,0x4752,0x4545,0x4747,0x4242,0x4747,0x4247,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x8F8F,0x8F8F,0x8F8F,0x4747,0x8F8F,0x8F8F,0x5050,0x4141,0x4141,0x4141,0x4242,0x4242,0x5247,0x4242,0x8F8F,0x7F7F,0x4141,0x4242,0x4747,0x4242,0x9F9F,0x4242,0x4242,0x4242,0x4242,0x8F8F
.word 0x6060,0x9F9F,0x5050,0x5050,0x5151,0x8F8F,0x5151,0x8F8F,0x5151,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5050,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x5151,0xB371,0x9282,0x9292,0x9292,0x9292,0x9292,0x8182,0x8080,0xAFAF,0x8E8E,0xAFAF,0xAFAF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xEFEF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x7070,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x7070,0x9F9F,0x9F9F,0x9F9F,0x4242,0x4242,0x6363,0x5252,0x4747,0x4242,0x4747,0x4242,0x4742,0x4242,0x4247,0x5757,0x4752,0x4242,0x4747,0x4242,0x4252,0x5247,0x4747,0x4242,0x4247,0x4545,0x4747,0x4747,0x4247,0x4747,0x8F8F,0x8F8F,0x8F8F,0x4747,0x4247,0x8F8F,0x4242,0x7F7F,0x4242,0x8F8F,0x8F8F,0x4141,0x5050,0x5151,0x4242,0x4242,0x4747,0x4742,0x4242,0x4141,0x4141,0x8F8F,0x4747,0x8F8F,0x4242,0x4242,0x4646,0x4242,0x4848,0x4242
.word 0x5050,0x5050,0x9F9F,0x9F9F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x8F8F,0x9F9F,0x5151,0x8F8F,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x5151,0x5050,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x4141,0x7171,0xB3C3,0xA382,0x9292,0x9292,0x9292,0x8282,0x8182,0xAFAF,0x9D9D,0xAFAF,0x8E8E,0xFEFE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x8080,0x7070,0x6060,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x6070,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x7070,0x7070,0x7070,0x6060,0x9F9F,0xAFAF,0x5151,0x4141,0x9F9F,0x4242,0x5752,0x5757,0x5252,0x4747,0x4242,0x4247,0x4747,0x4742,0x4242,0x5247,0x4747,0x4742,0x4242,0x4247,0x4747,0x4747,0x4742,0x4842,0x4242,0x4247,0x5247,0x4747,0x4242,0x4242,0x4747,0x4747,0x4242,0x4242,0x8F8F,0x7F7F,0x7F7F,0x4242,0x9F9F,0x8F8F,0x5050,0x5050,0x4141,0x8F8F,0x4242,0x4242,0x4747,0x4242,0x8F8F,0x8F8F,0x4141,0x4141,0x4142,0x4242,0x8F8F,0x8F8F,0x4545,0x4242,0x4242,0x8F47
.word 0x5050,0x6060,0x5050,0x5050,0x8F8F,0x8F8F,0x9F9F,0x4141,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x5151,0x8F8F,0x5050,0x9F9F,0x5050,0x9F9F,0x8F8F,0x8F8F,0x5151,0x4141,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x5151,0x5050,0x5151,0xC371,0x8272,0x9282,0x9282,0x9292,0x9292,0x8292,0x7181,0xD4D4,0xBFBF,0xBFBF,0xAFAF,0x9D9D,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x8080,0xAFAF,0x7070,0x9F9F,0x9F9F,0x7070,0x6060,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0x6060,0x6060,0x7070,0x6060,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0xAFAF,0x7070,0x6060,0x5252,0x9F9F,0x4747,0x6363,0x4242,0x4242,0x5248,0x4242,0x4545,0x4242,0x4742,0x5252,0x4242,0x4747,0x4243,0x4242,0x4545,0x4742,0x4242,0x4242,0x4742,0x4242,0x4242,0x4247,0x8F8F,0x8F8F,0x4242,0x4242,0x8F8F,0x4747,0x4242,0x8F8F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x5050,0x5050,0x5151,0x4150,0x4242,0x4842,0x4241,0x8F8F,0x4242,0x4051,0x4040,0x4141,0x9F9F,0x8F8F,0x4242,0x4742,0x4242,0x4747,0x4747,0x4242
.word 0x6060,0x9F9F,0x5050,0x5050,0x9F9F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x4040,0x7F7F,0x6060,0x9F9F,0x6161,0xD461,0x7271,0x8282,0x8282,0x9292,0x9292,0x9292,0x8292,0xC072,0xAFAF,0xAFAF,0x9E9E,0xAFAF,0xBFBF,0xDEDE,0xFFFF,0xFFFF,0xEFEF,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xC0C0,0xAFAF,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x9F9F,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x5252,0x9F9F,0x4242,0x5252,0x5555,0x4747,0x4242,0x4752,0x4742,0x4242,0x4247,0x4242,0x4247,0x4848,0x4242,0x4747,0x4742,0x4242,0x4747,0x4247,0x4247,0x4242,0x4247,0x4747,0x4242,0x4747,0x8F8F,0x8F8F,0x4242,0x4247,0x4242,0x8F8F,0x4242,0x4242,0x4742,0x9F9F,0x8F8F,0x8F8F,0x5050,0x5050,0x8F8F,0x4141,0x4242,0x8F8F,0x4242,0x9F9F,0x5050,0x8F8F,0x5151,0x5050,0x5050,0x4242,0x8F8F,0x4242,0x4242,0x4242,0x428F
.word 0x5050,0x5060,0x5050,0x9F9F,0x5151,0x9F9F,0x4141,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4141,0x5151,0x9F9F,0x8F8F,0x5151,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x5050,0x9F9F,0x6161,0xC361,0x8172,0xB3B3,0x8282,0x9282,0x9282,0x8292,0x9292,0x8182,0x71C3,0xAFAF,0xAFAF,0xAFAF,0x9D9D,0x8E8E,0xCFCF,0xFFFF,0xEFEF,0xDFDF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x6060,0x9F9F,0x7070,0x6060,0x7070,0x9F9F,0x7060,0x9F9F,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x6060,0x6060,0xAFAF,0x7070,0x7070,0x7070,0x6060,0x6060,0x7070,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0xAFAF,0x6060,0x9F9F,0x4242,0x9F9F,0x4242,0x4242,0x4545,0x4742,0x4242,0x4242,0x4547,0x4242,0x4747,0x4242,0x4542,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x4242,0x4247,0x4242,0x4242,0x4242,0x8F8F,0x8F8F,0x4742,0x4742,0x4242,0x7F7F,0x8F8F,0x5252,0x5050,0x5050,0x8F8F,0x4040,0x5040,0x8F8F,0x4242,0x4242,0x8F8F,0x8F8F,0x4141,0x5050,0x4141,0x5050,0x4040,0x4141,0x4242,0x4747,0x4242,0x4242,0x8F47
.word 0x5050,0x9F9F,0x5050,0x5050,0x5050,0x9F9F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x5151,0x5151,0x8F8F,0x9F9F,0x5151,0x8F8F,0x5151,0x9F9F,0x5151,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x6060,0x9F9F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x5050,0x8F8F,0x6161,0x6161,0x72C3,0xB382,0x8282,0x8282,0x82B2,0x9282,0x9282,0x9292,0xC392,0x7171,0xBFBF,0xAFAF,0x9E9E,0xAFAF,0xAFAF,0xADAD,0xDFDF,0xF5F5,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x9F9F,0x7070,0x6060,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x7070,0x9F9F,0x6060,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x4747,0x4747,0x4747,0x4242,0x4247,0x4747,0x4747,0x4545,0x4742,0x4242,0x4742,0x4242,0x4242,0x4242,0x4747,0x4747,0x4242,0x4646,0x4242,0x4242,0x4642,0x4747,0x8F8F,0x4242,0x8F8F,0x4747,0x4242,0x4646,0x4242,0x8F8F,0x4242,0x4242,0x4242,0x8F8F,0x8F8F,0x5050,0x5050,0x4040,0x4040,0x4141,0x4242,0x8F8F,0x4242,0x9F9F,0x5050,0x4141,0x5050,0x8F8F,0x4040,0x4141,0x4242,0x4242,0x4242,0x4242,0x428F
.word 0x5050,0x6060,0x5050,0x5050,0x9F9F,0x5050,0x8F8F,0x4141,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x7171,0x6161,0x5151,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x6060,0x6161,0x72D4,0x81C3,0x82B3,0xB382,0x8282,0x92B3,0x8292,0x9292,0xB292,0x8282,0xD0D0,0xBFBF,0xAFAF,0xAFAF,0x8E8E,0x9D9D,0xADAD,0xCFCF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x6060,0x6060,0x9F9F,0x7070,0x9F9F,0x5050,0x9F9F,0x9F9F,0x8F8F,0x6262,0x9F9F,0x4545,0x4745,0x4242,0x4247,0x4242,0x4747,0x4242,0x4742,0x4242,0x4242,0x4247,0x4747,0x4343,0x4242,0x4747,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4747,0x4742,0x4242,0x4747,0x4242,0x4747,0x4242,0x4646,0x4242,0x8F8F,0x4242,0x5050,0x5050,0x4040,0x4040,0x4040,0x4141,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x5050,0x4040,0x5050,0x4040,0x4141,0x7F7F,0x4242,0x4242,0x4747,0x8F47
.word 0x6060,0x6060,0x8F8F,0x5050,0x5050,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x4141,0x9F9F,0x5151,0x8F8F,0x5151,0x8F8F,0x8F8F,0x5151,0x7171,0x6171,0x6161,0x5151,0x5050,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x6060,0x6161,0x7171,0xC371,0xB381,0x8282,0x8282,0x8282,0xB392,0x9292,0x9292,0xB2B2,0xB3B2,0xC3C3,0xC0D0,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9E9E,0xCFCF,0xDFDF,0xBFBF,0xBFBF,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x6060,0x7070,0x6060,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x6060,0x7070,0x7070,0x9F9F,0x6060,0x7070,0x6060,0x6060,0x6060,0x7070,0x6060,0x7070,0x6060,0x6060,0x9F9F,0x7070,0x6060,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xC5C5,0x4242,0x4242,0x4242,0x4747,0x4242,0x4247,0x4247,0x4848,0x4747,0x4242,0x4747,0x4242,0x4242,0x4747,0x4242,0x4246,0x4242,0x4242,0x4742,0x4242,0x4646,0x4242,0x7F7F,0x4242,0x4242,0x4747,0x4242,0x4242,0x4747,0x4242,0x8F8F,0x4646,0x8F8F,0x8F8F,0x5050,0x5050,0x4040,0x4040,0x4141,0x4141,0x4242,0x8F8F,0x8F8F,0x5050,0x5050,0x4040,0x8F8F,0x5050,0x4040,0x4242,0x8F8F,0x4242,0x4242,0x4242
.word 0x5050,0x5050,0x6060,0x5050,0x9F9F,0x5151,0x9F9F,0x8F8F,0x4141,0x9F9F,0x9F9F,0x5151,0x8F8F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x72D3,0x72C3,0x7172,0x6171,0x6161,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4040,0x5050,0x8F8F,0x5050,0x6161,0x6261,0x7272,0x8272,0x82B3,0x8282,0x8282,0xB2B3,0xB292,0x92B3,0xB292,0x9292,0xB3B2,0xC381,0xB3B3,0xA081,0xAFAF,0x9D9D,0xAFAF,0x9D9D,0xBFBF,0xCFCF,0xD0D0,0xAFAF,0x7070,0x6060,0x7070,0x6060,0x6060,0x6060,0x7070,0x6060,0x7070,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0xAFAF,0x6060,0x6060,0x6060,0x9F9F,0x7070,0x7070,0x6060,0x7070,0x7070,0x9F9F,0x6060,0x6060,0xAFAF,0x9F9F,0x8E8E,0xAFAF,0x5252,0x9F9F,0x8F8F,0x4247,0x4242,0x4247,0x4742,0x5252,0x4242,0x4242,0x4747,0x4742,0x4642,0x4747,0x4747,0x4242,0x4242,0x4242,0x4747,0x4242,0x4742,0x4742,0x4242,0x4242,0x4747,0x4742,0x4242,0x4742,0x4747,0x4242,0x4747,0x4246,0x4747,0x5242,0x5050,0x5050,0x4040,0x5050,0x5050,0x4040,0x8F8F,0x4141,0x8F8F,0x9F9F,0x9F9F,0x5050,0x5050,0x5050,0x4040,0x4040,0x4141,0x4242,0x4242,0x4242,0x8F8F
.word 0x5050,0x6060,0x5050,0x6060,0x5050,0x9F9F,0x8F8F,0x5151,0x8F8F,0x5151,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0xC371,0x8281,0xB381,0xC372,0x5171,0x5161,0x5050,0x5050,0x8F8F,0x8F8F,0x9F9F,0x5151,0x6151,0x6171,0x71C3,0x7272,0xC381,0x72B3,0xB382,0xB382,0xB2B2,0xB2B2,0xB292,0xB2B3,0xB292,0x9282,0xB2B2,0x9282,0xC381,0xC080,0xAFAF,0x8E8E,0xBFBF,0xBFBF,0xD0D0,0xAFAF,0x7070,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x7070,0x9F9F,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x6060,0x7060,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x5252,0x9F9F,0x4242,0x4242,0x4242,0x4742,0x4242,0x4747,0x4752,0x4747,0x4242,0x4242,0x4747,0x4747,0x5242,0x4247,0x4747,0x4742,0x4242,0x4247,0x4242,0x4747,0x4242,0x4742,0x8F8F,0x4242,0x4242,0x4747,0x4747,0x4247,0x4242,0x4242,0x4242,0x4242,0x8F8F,0x8F8F,0x5050,0x4040,0x5040,0x4040,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x5050,0x4040,0x4040,0x4141,0x8F8F,0x4242,0x4242,0x428F
.word 0x5050,0x5050,0x6060,0x5050,0x5050,0x5050,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x4141,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x5151,0x9F9F,0x6161,0x81C3,0x8282,0x8182,0x7182,0x6172,0x6161,0x6161,0x9F9F,0x6060,0x6161,0x6161,0x6161,0x6161,0x72C3,0xC372,0xC372,0x81B3,0xC382,0x82B3,0xB2B2,0xB2B2,0x92B2,0x9292,0x92B2,0x92B3,0x92B3,0x8282,0x8282,0x8182,0xAFAF,0xAFAF,0xAFAF,0x9D9D,0x9E9E,0xBFBF,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x9F9F,0x7070,0x6060,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x9F9F,0x7070,0x6070,0x6060,0xAFAF,0x7070,0x9F9F,0x6060,0x7070,0xAFAF,0x7E7E,0xAFAF,0x9F9F,0x4747,0x9F9F,0x4242,0x5247,0x5757,0x4742,0x4742,0x4242,0x4255,0x4747,0x4247,0x4545,0x4242,0x4747,0x4242,0x4247,0x4242,0x4742,0x4242,0x4247,0x4742,0x4742,0x4747,0x4242,0x4247,0x4247,0x4742,0x4242,0x4242,0x4247,0x4747,0x4242,0x4242,0x4747,0x4242,0x5151,0x5050,0x4040,0x5050,0x4040,0x4040,0x8F8F,0x9F9F,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x4040,0x4040,0x4040,0x7F7F,0x4242,0x8F42
.word 0x5050,0x6060,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0xC371,0x8281,0x8282,0x72C3,0x6271,0x7172,0x7162,0x6161,0x6161,0x6161,0x6161,0x6161,0x7272,0xB3C3,0x7281,0x8182,0xB372,0x8281,0x82B3,0x92B3,0xB2B2,0x9292,0x8292,0xB392,0x8282,0x8282,0x8292,0x8282,0x7281,0x7171,0xAFAF,0x8E8E,0xAFAF,0x9D9D,0x7070,0xAFAF,0x6060,0x7070,0x9F9F,0x7070,0x6060,0xAFAF,0x7070,0x7070,0x6060,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x6060,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x5252,0x8F8F,0x8F8F,0x4242,0x4763,0x4242,0x4545,0x4247,0x4242,0x4545,0x4747,0x4242,0x4747,0x4742,0x4242,0x4742,0x4242,0x4247,0x4747,0x4842,0x4242,0x4747,0x4752,0x4242,0x4242,0x4747,0x4242,0x4747,0x4247,0x4242,0x4742,0x4646,0x4242,0x4747,0x4141,0x5050,0x5050,0x4040,0x5050,0x4040,0x4141,0x9F9F,0x5050,0x9F9F,0x5050,0x8F8F,0x5050,0x4040,0x4040,0x4040,0x4141,0x4141,0x7F7F,0x4242,0x428F
.word 0x5050,0x5050,0x6060,0x5050,0x5050,0x6060,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xC361,0xB3B3,0xC381,0x7281,0x7172,0x7272,0x7271,0xD461,0x6271,0x7261,0x7271,0x7272,0x82C3,0x8272,0x8272,0x8272,0x82B3,0x82B3,0x82B2,0xB3B2,0xB392,0x82B2,0x8292,0x8292,0x8282,0x8282,0x8282,0x8182,0xC072,0xAFAF,0xAFAF,0xAFAF,0x8E8E,0x9E9E,0xBFBF,0x6060,0x6060,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0xAFAF,0x7070,0xAFAF,0x6060,0x6060,0x6060,0x7070,0x9F9F,0x7070,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0x7070,0x9F9F,0x8E8E,0x9F9F,0x8E8E,0x6363,0x4242,0x8F8F,0x8F8F,0x4752,0x5252,0x5247,0x5247,0x4747,0x4247,0x4247,0x4545,0x4742,0x4742,0x4242,0x4848,0x4242,0x4742,0x4242,0x4742,0x4242,0x4742,0x4242,0x5247,0x4747,0x4747,0x4545,0x4247,0x4247,0x4545,0x4747,0x4242,0x4242,0x4242,0x5252,0x9F9F,0x7F7F,0x4050,0x4040,0x4040,0x4040,0x5050,0x5050,0x9F9F,0x5050,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4242,0x4242,0x8F8F
.word 0x5050,0x6060,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x5151,0x9F9F,0x9F9F,0x4141,0x8F8F,0x9F9F,0x5151,0x8F8F,0x4141,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4141,0x8F8F,0x9F9F,0x4141,0xC0C0,0x81C3,0x81C3,0x7272,0x7272,0x7172,0x7272,0x7172,0x7271,0x7272,0x7272,0x8272,0x7281,0x7282,0x8282,0xB382,0x8281,0x8282,0xB282,0xB2B3,0x92B3,0x8292,0xA382,0x8282,0x8292,0x9282,0x8282,0x7181,0x71D4,0xAFAF,0x9D9D,0x8E8E,0xAFAF,0xAFAF,0x9D9D,0x6060,0x6060,0x6060,0x6060,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0xAFAF,0x7070,0x9F9F,0x6070,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x9F9F,0x9E9E,0x9F9F,0xAFAF,0x9F9F,0x4247,0x8F8F,0x4242,0x4747,0x4862,0x5252,0x5252,0x4247,0x4242,0x4242,0x4247,0x4247,0x4242,0x4247,0x4247,0x4247,0x4646,0x4247,0x4247,0x4247,0x4248,0x4747,0x4747,0x5252,0x5247,0x4747,0x4742,0x4742,0x4742,0x4242,0x4747,0x4242,0x4242,0x4747,0x9F9F,0x7F7F,0x8F8F,0x4040,0x4040,0x4040,0x5050,0x9F9F,0x8F8F,0x5050,0x5050,0x4040,0x4040,0x5050,0x4040,0x4040,0x7F7F,0x4141,0x4242,0x8F8F,0x4242
.word 0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x5151,0x9F9F,0x8F8F,0x4141,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x7272,0xC3C3,0xC381,0x7172,0x7272,0x7272,0x7272,0x7272,0x7272,0x7281,0x8282,0x7282,0x8282,0xB372,0xB381,0x8272,0x8282,0x82B3,0x82C3,0x82B2,0x8292,0x82A3,0x9282,0x9292,0x9282,0x8292,0x7182,0xD471,0x7070,0xAFAF,0x8E8E,0xAFAF,0x8E8E,0x9E9E,0xBFBF,0x6060,0x6060,0x9F9F,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x9F9F,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x9F9F,0x9F9F,0x7070,0x7070,0xBFBF,0x7E7E,0xAFAF,0x4242,0x4242,0x4242,0x4242,0x6342,0x4747,0x5763,0x4747,0x5252,0x4747,0x4247,0x4247,0x4242,0x4247,0x4747,0x4242,0x4242,0x4742,0x4742,0x4242,0x4242,0x4242,0x4247,0x4752,0x5247,0x4747,0x5263,0x4247,0x4242,0x4747,0x4742,0x4242,0x4646,0x4242,0x4242,0x9F9F,0x8F8F,0x4040,0x8F8F,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4141,0x4242,0x9F8F
.word 0x5050,0x5050,0x5050,0x6060,0x5050,0x9F9F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4141,0x9F9F,0x8F8F,0xD451,0xC3C0,0x71C3,0x7272,0x7272,0x8282,0x8172,0x8282,0x8282,0x8282,0x8272,0x8272,0x8272,0x7282,0x72B3,0x8282,0x8282,0xB381,0x8282,0x82B3,0x82B3,0x8282,0x8282,0x9292,0x8292,0x8282,0xC372,0xAFAF,0xAFAF,0xAFAF,0x8E8E,0x9D9D,0xAFAF,0x9F9F,0x8E8E,0x5050,0x6060,0x6060,0x7070,0x7070,0x7070,0x6060,0xAFAF,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x9F9F,0x8E8E,0x9F9F,0xAFAF,0x8F8F,0x4242,0x4242,0x4242,0x4747,0x4752,0x5252,0x6363,0x4247,0x5252,0x4747,0x4745,0x4247,0x4747,0x5242,0x4747,0x4742,0x4242,0x4242,0x4747,0x4547,0x4747,0x4743,0x5252,0x4747,0x5252,0x5757,0x4347,0x4247,0x4742,0x4242,0x4742,0x4242,0x4242,0x4742,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5050,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x4141,0x8F8F,0x8F42
.word 0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x5050,0x5151,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7171,0xB1C3,0xB3C3,0x7272,0x8172,0x8272,0x8282,0x8282,0x8282,0x8272,0x8282,0x7282,0x7282,0x8272,0x8281,0x7282,0x8282,0x8282,0x8282,0x8282,0x8282,0x82A3,0x9282,0x92B3,0x9282,0xB382,0x7171,0x7070,0x9F9F,0x8E8E,0xAFAF,0x8E8E,0x9E9E,0x9F9F,0x9D70,0xBFBF,0x9F9F,0x9F9F,0x6060,0x9F9F,0x7070,0x9F9F,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0x6060,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x6060,0x7070,0x7070,0xAFAF,0x7070,0x9F9F,0xAFAF,0x9F9F,0x8E8E,0x4242,0x4747,0x4242,0x4242,0x4343,0x6363,0x5752,0x4752,0x4747,0x4845,0x6552,0x5252,0x4747,0x5252,0x4747,0x5252,0x4247,0x4747,0x4747,0x4542,0x4343,0x4542,0x4747,0x4742,0x5252,0x4747,0x4752,0x5252,0x4247,0x4247,0x4742,0x4242,0x4242,0x4242,0x5252,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x5050,0x9F9F,0x5050,0x4040,0x4040,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x4141,0x4141,0x4141,0x428F
.word 0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x4141,0x8F8F,0x9F9F,0x8F8F,0xC3C3,0xC3C3,0x7281,0x7271,0x7282,0x8282,0x8282,0x8282,0x8272,0x8282,0x7282,0x8272,0x8282,0x7282,0x72B3,0x8282,0x8282,0x8282,0x8282,0x8282,0x9282,0x9282,0x8292,0x9282,0x8292,0x81B2,0xC3C3,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x9F9F,0x6060,0x6060,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x6363,0x4242,0x4242,0x4747,0x4752,0x5555,0x5757,0x4552,0x5252,0x4747,0x4755,0x4247,0x4742,0x4545,0x4742,0x4242,0x4247,0x4242,0x4747,0x4247,0x4247,0x4247,0x4343,0x4747,0x5252,0x4747,0x4752,0x4742,0x4242,0x4247,0x4742,0x4747,0x4242,0x4747,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5050,0x5050,0x4040,0x4040,0x4040,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x4141,0x7F7F,0x8F41
.word 0x4040,0x4040,0x5050,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x7161,0x81C3,0x71C3,0x8272,0x8281,0x7282,0x8282,0x8282,0x8282,0x7282,0x8282,0x7282,0x7282,0xC382,0xB381,0x8282,0x8282,0x8282,0x8282,0x9282,0xB392,0x9292,0x8292,0x8292,0xB292,0xC2B2,0x81C3,0xAFAF,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x6060,0x6060,0x9F9F,0x7070,0x6060,0x9F9F,0x6060,0x6060,0xAFAF,0x7E7E,0xAFAF,0x4242,0x9F9F,0x4242,0x4242,0x5247,0x5252,0x5555,0x4752,0x4747,0x4755,0x5247,0x4545,0x4742,0x4747,0x4752,0x4247,0x4747,0x4242,0x4247,0x4742,0x4242,0x4747,0x4742,0x4742,0x5242,0x5247,0x4763,0x5247,0x4747,0x4742,0x4242,0x4747,0x4242,0x4242,0x4646,0x5252,0x9F9F,0x6E6E,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x5050,0x4050,0x4040,0x5050,0x8F8F,0x5050,0x4040,0x4040,0x4040,0x4141,0x4242,0x8F42
.word 0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x5151,0x9F9F,0x5151,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0xC3C3,0xC381,0x7272,0x7282,0x7272,0x8282,0x7282,0x7282,0x8282,0x8282,0x8272,0x8282,0x8282,0xC382,0xC382,0x82B2,0xB3B2,0xB3B2,0x82B2,0xB382,0xB2B2,0x9292,0x9292,0x9292,0x9292,0x82B2,0xC2C3,0xC0C0,0x7E7E,0x8F8F,0x9F9F,0xBFBF,0xBFBF,0x9D9D,0xAFAF,0x8E8E,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x9F9F,0x7070,0x7070,0x6060,0x7070,0x9F9F,0x7070,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x4242,0x4747,0x4242,0x4242,0x4747,0x4747,0x4852,0x5252,0x5247,0x5247,0x4747,0x4545,0x4247,0x5555,0x5247,0x4242,0x4247,0x4247,0x4842,0x4242,0x4747,0x4745,0x4242,0x4747,0x4747,0x4545,0x5247,0x4747,0x4742,0x4242,0x4247,0x4242,0x4747,0x4242,0x4343,0x9F9F,0x6E6E,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x4050,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x5050,0x4040,0x4040,0x4040,0x4242,0x418F
.word 0x4040,0x4040,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x6060,0x5050,0x8F8F,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xC371,0x82B2,0x7281,0x8272,0x8282,0x7281,0x8281,0x8282,0x7282,0x8282,0x8282,0x8282,0x8282,0x81C3,0x81B3,0x82C3,0x81C3,0xB2C3,0xB2B3,0xB2B2,0xB292,0x9292,0x9292,0x9292,0x9292,0xC3B2,0xC381,0xC071,0xAFAF,0x9F9F,0xAFAF,0x9D9D,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x4242,0x6363,0x4747,0x4242,0x5242,0x4747,0x5252,0x6347,0x4747,0x5252,0x5247,0x4247,0x4247,0x5247,0x4247,0x4747,0x4242,0x4842,0x4242,0x4747,0x5542,0x4242,0x5555,0x5242,0x5247,0x4547,0x4747,0x4245,0x4242,0x4247,0x4242,0x4747,0x4242,0x4242,0x4242,0x4747,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x4040,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x4141,0x4242,0x8F41
.word 0x4040,0x4040,0x4040,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x8F8F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x4141,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x4141,0xC371,0x8282,0x8272,0xC381,0x7281,0x8272,0x7282,0x8272,0x8272,0x8282,0x8282,0x8282,0xB382,0xB381,0x8282,0xB282,0xC3B2,0x82B2,0x92B3,0x9282,0x9292,0x9292,0x9282,0x9292,0x8292,0x81B2,0x71C3,0xC3C3,0xAFAF,0x9D9D,0x9E9E,0xAFAF,0xAFAF,0xAFAF,0x8E8E,0xAFAF,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x9F9F,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x6070,0x7070,0x9F9F,0x7070,0x6060,0x9F9F,0x6060,0x9F9F,0x9F9F,0xAFAF,0x8F8F,0x7E7E,0x7F7F,0x4747,0x4242,0x4242,0x4747,0x6565,0xC552,0x4747,0x4247,0x4752,0x4242,0x4742,0x4742,0x4742,0x4545,0x4343,0x4842,0x4242,0x4742,0x4247,0x4247,0x4247,0x4247,0x4747,0x4742,0x4242,0x4542,0x4747,0x4742,0x4242,0x4747,0x4247,0x4747,0x4747,0x4242,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5050,0x4040,0x5050,0x5050,0x9F9F,0x8F8F,0x8F8F,0x4040,0x4040,0x4141,0x4141,0x408F
.word 0x7F7F,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x8F8F,0x6060,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x6161,0xB2C3,0x8281,0x8282,0x8272,0x8272,0x7281,0x8272,0x7281,0x8282,0x8272,0x8272,0x8272,0x8282,0x8282,0x8282,0xB382,0x8282,0x82B3,0x82B2,0x9292,0x9292,0x9282,0x9292,0x9292,0x92B2,0xB292,0xC3C3,0x71C3,0xAFAF,0x9F9F,0xAFAF,0x9E9E,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x9F9F,0x6060,0x9F9F,0x6060,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x4242,0x4242,0x4242,0x4242,0x4742,0x4752,0xC5C5,0x4352,0x4742,0x4747,0x4742,0x4242,0x5247,0x4343,0x5555,0x4247,0x4247,0x4742,0x4242,0x4742,0x4242,0x4742,0x4242,0x4242,0x4242,0x4742,0x4247,0x4247,0x4242,0x4247,0x4444,0x4242,0x4242,0x4242,0x4242,0x4747,0x9F9F,0x9F9F,0x6E6E,0x7E7E,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x5050,0x4040,0x5050,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x4040,0x4141,0x4040,0x408F
.word 0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x5151,0x8F8F,0xD451,0xC381,0x8282,0x8281,0x8282,0x7282,0x8282,0x7272,0x8272,0x7272,0x8282,0x8282,0x7282,0x7282,0x8282,0x8282,0x8282,0x82B3,0x8282,0x9282,0x9282,0x9292,0x9292,0x9292,0x92B3,0x9282,0xB282,0x8181,0x81C3,0x7171,0x9F9F,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0x7E7E,0xAFAF,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x6060,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x9F9F,0x9F9F,0x6060,0xAFAF,0x7E7E,0x9F9F,0x4242,0x4742,0x4242,0x4747,0x5252,0x4848,0x5752,0x4747,0x4545,0x4242,0x4247,0x4247,0x4742,0x4742,0x4243,0x4742,0x4752,0x4242,0x4347,0x5247,0x4247,0x4247,0x4247,0x4247,0x4247,0x4147,0x4242,0x4742,0x4747,0x4742,0x4242,0x4747,0x4747,0x4242,0x7F7F,0x4242,0x9F9F,0x6E6E,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5050,0x4040,0x5050,0x6060,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x4040,0x8F40
.word 0x7F7F,0x4040,0x8F8F,0x4040,0x6060,0x8F8F,0x6060,0x5050,0x5050,0x5050,0x9F9F,0x5151,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x4141,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7171,0x82B2,0x8282,0x8282,0x8272,0x8281,0x7282,0x8282,0x7282,0x8282,0x8272,0x7272,0x7282,0x8282,0x7282,0x8282,0x8282,0x8282,0x8282,0x8282,0x9292,0x9282,0x9292,0xB2B2,0xB2B2,0x9282,0x8282,0xB3B2,0x8181,0xBFBF,0xAFAF,0x9F9F,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8E8E,0xAFAF,0x8E8E,0xAFAF,0xAFAF,0xBFBF,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x9F9F,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x9F9F,0x6060,0x6060,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x7F7F,0x4242,0x4242,0x4242,0x4747,0x5252,0x5757,0x4247,0x5555,0x4747,0x4242,0x4742,0x4747,0x4247,0x4842,0x4742,0x4242,0x4247,0x4242,0x4343,0x4747,0x4242,0x4242,0x4742,0x4242,0x4747,0x4742,0x4242,0x4247,0x4242,0x4247,0x4247,0x4242,0x4747,0x4246,0x4242,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7E7E,0x9F9F,0x9F9F,0x8F8F,0x4040,0x5050,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x5050,0x4040,0x4040
.word 0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x5151,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0xC361,0x82B2,0x8282,0x8282,0x8281,0x8272,0x8172,0x7282,0x8272,0x8282,0x8282,0x8282,0x8282,0x8272,0x8272,0x8282,0x8272,0x8282,0x8282,0x9292,0x8292,0xB292,0xB2B2,0xB2B2,0x9292,0x8292,0xB282,0x81B3,0x71C3,0xBFBF,0x8E8E,0x7E7E,0xAFAF,0xAFAF,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x8080,0x8080,0x7070,0xAFAF,0x7070,0x8080,0xAFAF,0x7070,0x9F9F,0x7070,0x6060,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x4646,0x4747,0x4242,0x4242,0x5252,0x4747,0x4747,0x4252,0x4747,0x4242,0x4545,0x5247,0x5252,0x5247,0x4248,0x4242,0x4742,0x4742,0x4747,0x4743,0x4342,0x4545,0x4747,0x4247,0x4747,0x4242,0x4247,0x4747,0x4242,0x4742,0x4742,0x4242,0x4545,0x4242,0x4242,0x4747,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x5050,0x5050,0x5050,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050,0x5050,0x8F40
.word 0x7F7F,0x4040,0x4040,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x9F9F,0x5151,0x9F9F,0x4141,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x4141,0x8F8F,0x5151,0x8F8F,0x8F8F,0xC361,0x8281,0x8282,0x8281,0x7282,0x8282,0x7282,0x8272,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x7282,0x8282,0x8282,0x9292,0x9292,0xB292,0xB2B2,0xB2B3,0x92B3,0x92B3,0x8282,0x82B2,0x8281,0x7171,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x8E8E,0xAFAF,0x7E7E,0x8E8E,0x7E7E,0xAFAF,0xAFAF,0x9D9D,0xBFBF,0xBFBF,0x7070,0x7070,0x7070,0x6060,0x9F9F,0x7060,0x9F9F,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x8080,0x7070,0x7070,0x7070,0x9F9F,0x6070,0x7070,0x6060,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x6060,0x9F9F,0x6060,0x5151,0x9F9F,0xAFAF,0x9F9F,0x7E7E,0x4242,0x4242,0x4242,0x4747,0x5742,0x5252,0x5247,0x4747,0x4242,0x4747,0x4765,0x4242,0x4347,0x4747,0x4242,0x4248,0x4247,0x4242,0x4247,0x4242,0x4247,0x4247,0x4242,0x4242,0x4247,0x4742,0x4242,0x4247,0x4247,0x4247,0x4242,0x4742,0x4242,0x4242,0x4242,0x4242,0xAFAF,0x7E7E,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4040,0x5050,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050,0x4040,0x408F
.word 0x4040,0x8F8F,0x4040,0x4040,0x5050,0x5050,0x5050,0x6060,0x5050,0x5050,0x5050,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x4141,0x6161,0xC3D3,0x82B2,0x8282,0x8282,0x8281,0x7282,0x8281,0x8282,0x8282,0x8282,0x8282,0x8282,0x8272,0x8282,0x8282,0x82B3,0x8282,0x9282,0x9292,0xB292,0x92B2,0xB2B2,0x92B2,0x9292,0x8292,0x8292,0x8282,0x8282,0xC3C3,0x8E8E,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9E9E,0x9D9D,0xBFBF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x9F9F,0x7070,0x9F9F,0x6060,0x6060,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x5752,0x4242,0x4848,0x4545,0x4242,0x4247,0x4742,0x4242,0x4747,0x4245,0x4247,0x4247,0x4742,0x4747,0x4742,0x4742,0x4742,0x4742,0x4742,0x4242,0x4247,0x4242,0x4247,0x4742,0x4742,0x4242,0x4747,0x4742,0x4242,0x5757,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050,0x4040,0x8F40
.word 0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5151,0x9F9F,0x4141,0x9F9F,0x9F9F,0x4141,0x8F8F,0x8F8F,0x9F9F,0x81C3,0xB2C3,0x8282,0x8281,0x8272,0x8281,0x8272,0x8272,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x82B3,0x8282,0x9282,0x9292,0x9282,0x9292,0x8292,0x8292,0xB392,0x8282,0x9282,0x8282,0x8282,0x7182,0xAFAF,0xAFAF,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0x7E7E,0xAFAF,0x8E8E,0xAFAF,0x7E7E,0xAFAF,0x7E7E,0x8E8E,0xAFAF,0xAFAF,0xADAD,0xBFBF,0xAFAF,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x8080,0xAFAF,0x8080,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x6060,0x9F9F,0x6060,0x9F9F,0x6060,0x5151,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4242,0x4747,0x4646,0x4242,0x4545,0x4742,0x6262,0x4747,0x4242,0x4747,0x4547,0x4742,0x4242,0x4747,0x4242,0x4747,0x4742,0x4747,0x4247,0x4542,0x4747,0x4247,0x4242,0x4247,0x4242,0x4247,0x4742,0x4742,0x4242,0x4247,0x4242,0x4742,0x4242,0x4242,0x4242,0x6262,0xAFAF,0x8F8F,0x6E6E,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x4050,0x4040,0x408F
.word 0x8F8F,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x5151,0x9F9F,0x5151,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0xC361,0xC3C1,0x81C2,0x82B2,0x8282,0x8282,0x8272,0x8282,0x8281,0x8272,0x8272,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x9292,0x8292,0x9292,0x9282,0x9282,0x8292,0x8292,0x9292,0x8292,0x8282,0x8282,0x8172,0xBFBF,0x8E8E,0xAFAF,0xAFAF,0x7E7E,0xAFAF,0xAFAF,0x7E7E,0xAFAF,0x8E8E,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xBFBF,0xBFBF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x8080,0xAFAF,0x8080,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x6060,0xAFAF,0x7070,0x7070,0x9F9F,0x6060,0x6060,0x9F9F,0x5050,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x7F7F,0x4247,0x4242,0x4242,0x4242,0x4242,0x5752,0x4747,0x4742,0x5252,0x4247,0x4242,0x4247,0x5247,0x5747,0x4242,0x4247,0x4245,0x4242,0x4742,0x4242,0x4545,0x4742,0x4242,0x4743,0x4242,0x4247,0x4747,0x4742,0x4242,0x4747,0x4247,0x4646,0x4242,0x4646,0x4742,0xAFAF,0x7E7E,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x4040,0x408F
.word 0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0xD3D3,0x81C3,0xB3C2,0x8281,0x8182,0x8182,0x8182,0xC3B3,0x82C3,0x8282,0x8282,0x8282,0x8272,0x7282,0x8282,0x8282,0x9282,0x9292,0x9292,0x8282,0x8292,0x8292,0x9282,0x9282,0xB392,0xB3B2,0x8282,0x8281,0x7172,0xAFAF,0xAFAF,0x9F9F,0x7E7E,0x9F9F,0x8E8E,0xAFAF,0x9F9F,0x8E8E,0xAFAF,0x8E8E,0x8E8E,0xAFAF,0x8E8E,0xAFAF,0x7E7E,0x9D9D,0x9D9D,0xBFBF,0x7070,0xAFAF,0x7070,0x7070,0x8080,0x7070,0xAFAF,0x8080,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0x6060,0x6060,0x7070,0x6060,0x7060,0x9F9F,0x6060,0x6060,0x9F9F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x8F8F,0x7E7E,0x4242,0x4242,0x8F8F,0x4242,0x4747,0x5555,0x6363,0x5252,0x4242,0x5247,0x4747,0x4743,0x4747,0x4747,0x5252,0x4247,0x4242,0x4247,0x4347,0x4747,0x4742,0x4742,0x4242,0x4747,0x4247,0x4742,0x4742,0x4242,0x4247,0x4247,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x5252,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x4050,0x4040,0x8F40
.word 0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x8F8F,0x5151,0x9F9F,0x8F8F,0x5151,0x8F8F,0x4141,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x5151,0x8F8F,0xD0D0,0xC3D3,0xB281,0x8282,0xB382,0xB381,0x7281,0x8181,0x81C3,0x81C3,0x7282,0x8282,0x7282,0x8282,0x8282,0x8282,0x9282,0x9292,0x9282,0x9292,0x9282,0x9282,0x9282,0x9282,0x92B2,0xB2B2,0x8282,0x7282,0x7172,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x7E7E,0x7E7E,0x9F9F,0x8E8E,0xAFAF,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0x9E9E,0x9E9E,0xBFBF,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x8080,0x8080,0xAFAF,0x8080,0x8080,0x7070,0x7070,0x7070,0xAFAF,0x6060,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x6060,0x5050,0x5050,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4242,0x4242,0x4646,0x4242,0x4242,0x4742,0x4852,0x4763,0x4242,0x4747,0x5252,0x5247,0x4252,0x5247,0x4747,0x4742,0x4742,0x4242,0x4242,0x4742,0x4242,0x4242,0x4248,0x4742,0x4242,0x4247,0x4242,0x4747,0x4242,0x4742,0x4742,0x4742,0x4242,0x4747,0x4242,0x4747,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x6E6E,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x4040,0x408F
.word 0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x8F8F,0x4141,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x4141,0x9F9F,0x8F8F,0x8F8F,0x4141,0x8F8F,0xBFBF,0xC2C3,0x82C3,0x8182,0x8172,0x81B3,0x8282,0x8172,0x8172,0x82C3,0x8282,0x7282,0x8282,0x8272,0x8282,0x9282,0x9292,0x9282,0x9292,0x9282,0x9292,0x9292,0x9292,0x9292,0xB2B3,0xB282,0x82B2,0x8282,0x6171,0xAFAF,0x8E8E,0x9F9F,0x7E7E,0x7E7E,0xAFAF,0x9F9F,0xAFAF,0x7E7E,0xAFAF,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8E8E,0xAFAF,0x9D9D,0x9D9D,0xAFAF,0x8080,0x7070,0x7070,0x8080,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x9F9F,0x9F9F,0x9F9F,0x5050,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x4242,0x4747,0x4242,0x4242,0x4242,0x5252,0x9F9F,0x4547,0x4343,0x4747,0x4242,0x4747,0x4242,0x4742,0x4343,0x4247,0x4542,0x4747,0x4247,0x4247,0x4248,0x4741,0x4242,0x4542,0x4242,0x4247,0x4742,0x4742,0x4242,0x4242,0x4242,0x4247,0x4242,0x4242,0x4242,0x4242,0x6E6E,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x4040,0x7F40
.word 0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x9F9F,0x8F8F,0x9F9F,0xD0D0,0x81C3,0x81C3,0x82C3,0x8282,0x8172,0x82B3,0xB381,0x8272,0x8172,0x7282,0x8282,0x8282,0xB282,0x9282,0x8292,0x8292,0x9282,0x8282,0x9292,0x92B3,0x92B3,0x8282,0xB282,0xB292,0xB2B3,0x82B3,0x7282,0x7172,0xAFAF,0xAFAF,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0x7E7E,0x8E8E,0x9F9F,0xAFAF,0x7E7E,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0x8E8E,0xBFBF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x8080,0xAFAF,0x8080,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x6060,0x5050,0x9F9F,0x5151,0x9F9F,0x6060,0x6060,0x9F9F,0x9F9F,0x8F8F,0x4242,0x4742,0x4242,0x4747,0x8F8F,0x4242,0x4247,0x4247,0x4242,0x4545,0x4747,0x4242,0x5247,0x4257,0x5247,0x4747,0x4242,0x4747,0x4742,0x4242,0x4242,0x4242,0x4747,0x4242,0x4247,0x4742,0x4242,0x4242,0x4747,0x4242,0x4247,0x4247,0x4242,0x4642,0x7F7F,0x4642,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7E7E,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x5050,0x4040,0x8F8F,0x8F40
.word 0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0xAFAF,0xBFBF,0xC3C3,0xC381,0xC381,0xB381,0x7281,0x7281,0x81C3,0x81B3,0x8282,0x8272,0x8282,0xB382,0x82B2,0x82B2,0x9292,0x9282,0x8292,0x9292,0xB392,0xB292,0x8292,0x92B2,0x9292,0xB2B3,0x82B2,0x82B2,0x8182,0x7172,0xAFAF,0x7E7E,0xAFAF,0x7E7E,0x7E7E,0x8E8E,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0xAFAF,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0x8E8E,0x9D9D,0xAFAF,0x8E8E,0x9D9D,0xAFAF,0x9F9F,0x8080,0x7070,0xAFAF,0x8080,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x6060,0x9F9F,0x6060,0x9F9F,0x8F8F,0x9F9F,0x5050,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x8F8F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4247,0x4742,0x5252,0x4745,0x4242,0x4747,0x4242,0x4247,0x4742,0x4242,0x4247,0x4742,0x4742,0x5555,0x4242,0x4747,0x4242,0x4247,0x4742,0x4742,0x4242,0x4747,0x4242,0x4242,0x4247,0x7F7F,0x4642,0x4242,0x8F8F,0x8F8F,0x9F9F,0x6E6E,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x9F9F,0x5050,0x4040,0x408F
.word 0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5151,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0x7171,0xC381,0xC381,0xC381,0x81C3,0xB3C3,0xC372,0xC3C3,0x82C3,0xC382,0x82B2,0x82C3,0x81C3,0x8282,0x9282,0x8282,0x8292,0x9292,0xB2B3,0xB2B3,0x92B3,0x92B3,0x92B3,0x92B3,0x82B2,0x82B3,0x8282,0x7282,0x7172,0xAFAF,0xAFAF,0xAFAF,0x8E8E,0x9F9F,0x9F9F,0x7E7E,0x8E8E,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0x8E8E,0x9D9D,0xAFAF,0x9E9E,0x9F9F,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x6060,0x7070,0x6060,0x6060,0x6060,0x6060,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8F8F,0x4242,0x4742,0x4242,0x4242,0x8F8F,0x4242,0x4247,0x4242,0x4242,0x4242,0x4247,0x4242,0x4747,0x5255,0x4747,0x4242,0x4242,0x4747,0x4242,0x4242,0x4247,0x4742,0x4747,0x4242,0x4247,0x4242,0x4242,0x4242,0x4242,0x4742,0x4742,0x4242,0x4242,0x4246,0x4242,0x4242,0x4747,0x9F9F,0x6E6E,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x8F8F,0x8F40
.word 0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x5151,0x5151,0x8F8F,0x8F8F,0x8F8F,0xBFBF,0xD4D4,0x81C3,0x81C3,0x81C3,0xC381,0xC3C3,0xC3D3,0xC3D0,0xC3C1,0xC2C2,0xC2C1,0xB2C2,0xB2B2,0x8282,0x8282,0x8292,0x9282,0x9282,0xB292,0xB2B2,0x8292,0x8292,0xB292,0x9292,0xB282,0xB2B2,0x8282,0x8282,0x7272,0x8080,0x7E7E,0x7E7E,0xAFAF,0xAFAF,0x8E8E,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0x9D9D,0x9E9E,0x8E8E,0xAFAF,0xAFAF,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x9F9F,0x6060,0x6060,0x9F9F,0x9F9F,0x8F8F,0x5151,0x9F9F,0x5151,0x9F9F,0x6060,0x6060,0x9F9F,0x7E7E,0x8F8F,0x7F7F,0x4242,0x4747,0x4242,0x4747,0x4242,0x8F8F,0x4242,0x4247,0x4347,0x4242,0x4247,0x5242,0x4747,0x4242,0x4247,0x4247,0x4242,0x4247,0x4247,0x4742,0x4242,0x4242,0x4242,0x4242,0x4742,0x4742,0x4742,0x4742,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4242,0x9F9F,0x8F8F,0x9F9F,0x6E6E,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x4040,0x8F7F
.word 0x5050,0x8F8F,0x5050,0x4040,0x4040,0x4040,0x5050,0x5050,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9D9D,0x71C0,0xC381,0xC381,0xC381,0xC3C3,0xD0C3,0xC1D3,0xC2C3,0xC2D3,0xD2C2,0xC2D2,0xC2C2,0xC3C2,0xB2B2,0x9282,0x8282,0xB392,0xB292,0xB2B3,0x8282,0xB382,0xB392,0xB3B2,0x8282,0x82B2,0x8282,0x8282,0x8182,0x7272,0x6161,0xAFAF,0xAFAF,0x9F9F,0x7E7E,0x7E7E,0x8E8E,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0x8E8E,0x8E8E,0x9F9F,0xAFAF,0x8E8E,0xBFBF,0x9D9D,0x8E8E,0xAFAF,0x6060,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x7070,0x7070,0x9F9F,0x6060,0x6060,0x6060,0x5151,0x8F8F,0x9F9F,0x9F9F,0x7070,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4742,0x4242,0x4747,0x4242,0x4742,0x4242,0x4747,0x4747,0x4242,0x4242,0x4247,0x4742,0x4842,0x4742,0x4742,0x4242,0x4242,0x4242,0x4242,0x4242,0x4742,0x4642,0x4242,0x4242,0x4242,0x4646,0x4242,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x8F8F
.word 0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x9F9F,0xAFAF,0xC371,0xC381,0xC381,0xC381,0xC381,0xD4C0,0xC3D3,0xC281,0xC2C2,0xC1D2,0xC1C2,0xC2C2,0xB2C2,0xB3B2,0xB2B2,0xB292,0x92B3,0x8292,0x9292,0xB282,0x9292,0xB2B3,0x8282,0x9282,0x8282,0x8282,0x8282,0x7282,0x7272,0x7171,0x7E7E,0x9F9F,0x7E7E,0x9F9F,0xAFAF,0x8E8E,0xAFAF,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0x8E8E,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0xAFAF,0x8E8E,0xBFBF,0x8E8E,0xAFAF,0x8F8F,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x9F9F,0x6060,0x9F9F,0x6060,0x8F8F,0x9F9F,0x9F9F,0x5151,0x9F9F,0x9F9F,0xAFAF,0x6060,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4742,0x4242,0x4743,0x4242,0x4742,0x4242,0x4742,0x4242,0x4242,0x4247,0x4247,0x4242,0x4747,0x4242,0x4242,0x4242,0x4742,0x4742,0x4646,0x4242,0x4747,0x4646,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7E7E,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x5050,0x4040,0x8F8F,0x8F40
.word 0x8F8F,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x4040,0x4040,0x4040,0x8F8F,0x4141,0x8F8F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x9D9D,0x7070,0xC371,0xC381,0xC372,0xC372,0xC371,0xC3C3,0x81C2,0xB2C3,0xC2C3,0xC2C2,0xC2C2,0xB2C2,0xB2C2,0xB2B2,0xB2C2,0xB2B2,0xB292,0x9282,0xB392,0xB3B2,0xB2B3,0xB3B2,0xB3B2,0x8282,0x8282,0x8282,0x8282,0x7282,0x7272,0x7171,0x9F9F,0xAFAF,0x9F9F,0x8E8E,0x8E8E,0xAFAF,0x7E7E,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0x8E8E,0x8E8E,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0x8E8E,0x8F8F,0x9F9F,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x9F9F,0x7070,0x6060,0x6060,0x9F9F,0x5050,0x5151,0x5151,0x9F9F,0x7070,0xAFAF,0x6060,0x9F9F,0x7070,0xAFAF,0x9F9F,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4747,0x4242,0x4747,0x4242,0x4747,0x4242,0x4242,0x4842,0x4742,0x4742,0x4747,0x4742,0x4247,0x4242,0x4242,0x4747,0x4747,0x4242,0x4742,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x8F8F,0x6E6E,0x8F8F,0x9F9F,0x9F9F,0x7E7E,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x8F8F
.word 0x5050,0x8F8F,0x5050,0x4040,0x4040,0x4040,0x5050,0x5050,0x5050,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x7171,0x81C3,0x81C3,0x81C3,0x71C3,0x8172,0x8281,0xB282,0xC2B2,0xC2C2,0xB2C2,0xB2B2,0xB2B2,0xB292,0xB2B2,0xB2B2,0xB2B3,0xB2B2,0xB2B2,0xB2B2,0xB2B2,0xB2B3,0x92B3,0x82B3,0x8282,0x8282,0x8282,0x8282,0x7272,0x7071,0xAFAF,0x7E7E,0x7E7E,0x9F9F,0xAFAF,0x7E7E,0x9F9F,0xAFAF,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0x9F9F,0x8E8E,0x8E8E,0x8E70,0xAFAF,0xAFAF,0x8E8E,0xAFAF,0x9D9D,0xAFAF,0x6060,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x9F9F,0x6060,0x9F9F,0x5050,0x9F9F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x6060,0x9F9F,0x7070,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x4747,0x4747,0x4242,0x4242,0x7F7F,0x4646,0x4242,0x4747,0x4247,0x5247,0x5252,0x4747,0x4242,0x4248,0x4444,0x4742,0x4742,0x4242,0x4247,0x4242,0x4242,0x4242,0x4242,0x4247,0x4747,0x4242,0x4242,0x4247,0x4242,0x4747,0x4242,0x4646,0x4242,0x4747,0x4242,0x8F8F,0x8F8F,0x8F8F,0x7E7E,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x4040,0x4040,0x8F7F
.word 0x8F8F,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x5050,0x4040,0x5050,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0x7180,0x7272,0xC371,0xC381,0xB381,0xB371,0xB381,0x8282,0xC382,0xC3C2,0xB2B2,0xB2B2,0xB2B2,0x9292,0x9292,0xB2B2,0xB2B2,0xB2B2,0xB3B2,0xC3B2,0xC3B2,0xB3B3,0xB382,0x8282,0x8282,0x82A3,0x8282,0x8282,0x7282,0x7272,0x7272,0xAFAF,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0x7E7E,0x7E7E,0x9F9F,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0x9F9F,0x8E8E,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0x9E9E,0x8F8F,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0x9F9F,0x7070,0x6060,0x9F9F,0x6060,0x5050,0x5151,0x5151,0x9F9F,0x6161,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x7070,0xAFAF,0x7E7E,0x8F8F,0x4242,0x4242,0x7F7F,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x9F9F,0x4747,0x4242,0x4842,0x4242,0x4242,0x4747,0x4242,0x4242,0x4747,0x4742,0x4742,0x4747,0x4742,0x4242,0x4242,0x4747,0x4747,0x4242,0x4246,0x4242,0x4242,0x4242,0x4247,0x4242,0x4246,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F
.word 0x5050,0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x4040,0x4040,0x8F8F,0x4040,0x4141,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0xAFAF,0x7161,0x7271,0x81C3,0x71C3,0x71C3,0x72B3,0x7272,0x8281,0xB2B2,0xB2C2,0xB2C2,0xB2B2,0x92B2,0x9292,0x9292,0xB292,0xB2B2,0xB3B2,0xB2B2,0xB2C3,0xB2B3,0xB3B2,0x82B2,0x82B3,0x9282,0x9282,0x92B3,0x8282,0x8282,0x7272,0x7171,0xAFAF,0x7E7E,0x7E7E,0xAFAF,0xAFAF,0x7E7E,0x8E8E,0xAFAF,0x9F9F,0x7E7E,0x8E8E,0x8E8E,0xAFAF,0x9F9F,0x8E8E,0x9F9F,0x8E8E,0x8E8E,0xAFAF,0x9D9D,0x8E8E,0xAFAF,0x4040,0x6060,0xAFAF,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x8F8F,0x9F9F,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6060,0x7070,0x6060,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x6F6F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x4247,0x4247,0x4742,0x4242,0x4743,0x5252,0x4747,0x4242,0x4242,0x4747,0x4247,0x4247,0x4242,0x4242,0x4242,0x4242,0x4242,0x4246,0x4242,0x4242,0x4242,0x4242,0x4747,0x4742,0x4242,0x4242,0x4242,0x7F7F,0x4242,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F
.word 0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x4040,0x5050,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x7161,0x8172,0x8172,0xC3B3,0xC381,0x8171,0x8282,0x8282,0xC382,0xC3B2,0xB3B2,0x82B2,0x82B2,0x9292,0x9292,0x92B2,0xB2B2,0xB2B2,0xB2C3,0xB282,0x82B3,0x82B3,0x82B3,0x8282,0x82B3,0x8292,0x8292,0x8282,0x8272,0x7272,0x7272,0xAFAF,0x9F9F,0x9F9F,0x7E7E,0x8E8E,0xAFAF,0xAFAF,0x7E7E,0xAFAF,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0xAFAF,0x8E8E,0xAFAF,0x8E8E,0xAFAF,0x8E8E,0x9E9E,0xAFAF,0x9E9D,0x9F9F,0x8F8F,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x9F9F,0x6060,0x5050,0x5050,0x8F8F,0x8F8F,0x5151,0x5151,0x9F9F,0x6060,0x9F9F,0x9F9F,0x7070,0x7070,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4247,0x4242,0x4247,0x4242,0x4247,0x4747,0x4242,0x4242,0x4242,0x4247,0x4247,0x4747,0x4742,0x4242,0x4646,0x4242,0x4747,0x4747,0x4742,0x4242,0x4242,0x4747,0x4742,0x4242,0x4242,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F
.word 0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x4040,0x5050,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x7171,0x7271,0x7272,0x7281,0x81B3,0x72B3,0x7272,0x8182,0xB282,0xB2C3,0xB2B2,0xB282,0x9282,0x8282,0x9292,0xB292,0x8292,0xB2B3,0x8282,0x82B3,0x82B2,0x8282,0x8282,0xB382,0x92B2,0x9282,0x8282,0x8282,0x8282,0x7272,0x7172,0xAFAF,0x7E7E,0x7E7E,0x9F9F,0xAFAF,0x7E7E,0x7E7E,0xAFAF,0x7E7E,0x7E7E,0x8E8E,0xAFAF,0xAFAF,0x8E8E,0xAFAF,0x8E8E,0xAFAF,0x8E8E,0x9D9D,0xAFAF,0x9D9D,0xAFAF,0xAFAF,0x4040,0x7070,0x9F9F,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x5050,0x5050,0x4141,0x5151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7070,0x9F9F,0xAFAF,0x7070,0xAFAF,0x8F8F,0x7F7F,0x4242,0x4242,0x5656,0x4242,0x4242,0x4646,0x4242,0x4747,0x4747,0x4242,0x4242,0x4747,0x4242,0x4247,0x4242,0x4242,0x4242,0x4247,0x4747,0x4242,0x4747,0x4242,0x4242,0x4747,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x4242,0x4646,0x7F7F,0x4242,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F7F
.word 0x5050,0x5050,0x4040,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x7161,0x7171,0x8172,0xC372,0xB381,0x7282,0x8281,0x8281,0xB282,0xB2C3,0x82B3,0x82B2,0xB392,0x9292,0xB2B2,0x92B2,0xB292,0x82B2,0xB282,0x8282,0x8282,0xB382,0x8282,0x9282,0xB392,0xA392,0x8292,0x8282,0x7282,0x7272,0x7172,0x8E8E,0xAFAF,0x9F9F,0x8E8E,0x7E7E,0xAFAF,0x8E8E,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0x9F9F,0x8E8E,0x9E9E,0xBFBF,0x8E8E,0x8E8E,0x7F7F,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x9F9F,0x6060,0x5050,0x8F8F,0x8F8F,0x4141,0x8F8F,0x9F9F,0x5151,0x6060,0x9F9F,0x6060,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x7F7F,0x4242,0x4242,0x4242,0x7F7F,0x5656,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4747,0x4742,0x4242,0x4646,0x4747,0x4242,0x4242,0x4242,0x4747,0x4242,0x4747,0x4742,0x4242,0x4242,0x4242,0x4747,0x4242,0x4742,0x4742,0x4242,0x4742,0x4242,0x4242,0x4642,0x7F7F,0x7F7F,0x6E6E,0x8F8F,0x6E6E,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F
.word 0x8F8F,0x5050,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x5050,0x5050,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x9F9F,0x7E7E,0x6161,0x7271,0x7281,0x8182,0x7282,0x8281,0x8272,0x8282,0x8282,0x8282,0x8282,0x8282,0x9282,0xB392,0x82B2,0x8292,0xB392,0x82B2,0x82B2,0xB282,0xB382,0x8282,0x8282,0x8292,0x9292,0x9282,0x8282,0x8282,0x8282,0x7282,0x6172,0xBFBF,0x7E7E,0x7E7E,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0x7E7E,0xAFAF,0x7E7E,0xAFAF,0x9F9F,0x608E,0x7E7E,0x8E8E,0x9F9F,0x8E8E,0x9E9E,0x9D9D,0x8E70,0xAFAF,0xAFAF,0x9F9F,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x9F9F,0x6060,0x5050,0x8F8F,0x5050,0x4141,0x9F9F,0x5151,0x5151,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x7070,0x7070,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x4742,0x4242,0x4242,0x4247,0x4242,0x4242,0x4242,0x4747,0x4646,0x4242,0x4242,0x4742,0x4242,0x4242,0x4742,0x4242,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F
.word 0x5050,0x5050,0x4040,0x4040,0x4040,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x4050,0x4040,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0x7160,0x7272,0x8172,0x7282,0x8282,0x8282,0x8282,0x8282,0xB282,0xB282,0x8282,0x9292,0x8282,0x8292,0x82B2,0x9292,0xB2B2,0xB2B2,0xB282,0x82B3,0x82B2,0x82B3,0x9292,0x9282,0x8282,0x8292,0x8282,0x8282,0x7282,0x7282,0x7172,0xAFAF,0x9F9F,0x9F9F,0x7E7E,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x7E7E,0x9F9F,0x8E8E,0x9F9F,0x8E8E,0x9F9F,0x9F9F,0x8E8E,0x9F9F,0x8E8E,0x9E9E,0x9E9E,0x8E8E,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x9F9F,0x6060,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x4141,0x8F8F,0x9F9F,0x6060,0x9F9F,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0x9F9F,0x8F8F,0x4242,0x5656,0x4242,0x4242,0x4242,0x4747,0x4747,0x4242,0x4247,0x4545,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x4646,0x4247,0x4742,0x4242,0x4242,0x4246,0x4646,0x4242,0x4247,0x4742,0x4242,0x4747,0x4247,0x4646,0x4242,0x7F7F,0x4242,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6E6E,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F
.word 0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8E8E,0x8F8F,0x6161,0x7271,0x7281,0x8182,0x8282,0x8282,0x8282,0xB282,0x82B2,0x82B3,0x82B2,0x8282,0x82B2,0x9282,0xB282,0x9282,0xB2B3,0xB3C3,0x82B2,0x82B2,0x82B3,0x8292,0x8282,0x8292,0x9282,0x92A3,0x9282,0xB3B3,0x8282,0x7272,0x7172,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7E7E,0x8E8E,0x7E7E,0x9F9F,0x9F9F,0x9F9F,0x7E7E,0x9F9F,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9D9D,0x9F9F,0x9F9F,0xAFAF,0x7070,0xAFAF,0x7070,0x6060,0x9F9F,0x6060,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x5151,0x8F8F,0x4141,0x6060,0x9F9F,0x9F9F,0x7070,0x9F9F,0x9F9F,0x6060,0x7070,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x4242,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4742,0x4646,0x4646,0x4242,0x4646,0x4646,0x4247,0x4242,0x4242,0x4242,0x4242,0x4242,0x4742,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F40
.word 0x5050,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8E8E,0x8F8F,0x6161,0x7171,0x7281,0x8281,0x8182,0x8282,0x8282,0xC381,0x82B3,0xB2B2,0xB282,0x9282,0x8282,0x8282,0xB2B2,0xB282,0xC3B2,0x82B2,0x82B2,0x82B3,0x82B2,0x8282,0x8292,0xA392,0xA392,0x8292,0xB392,0x8282,0x8282,0x7282,0x7172,0xBFBF,0x7E7E,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8E8E,0x9F9F,0x8E8E,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0x8E8E,0x9E9E,0xAFAF,0x8E8E,0xAFAF,0xAFAF,0x7070,0x7070,0x9F9F,0x6060,0x9F9F,0x5050,0x5050,0x5050,0x5050,0x5151,0x8F8F,0x5151,0x5151,0x9F9F,0x9F9F,0x6060,0x9F9F,0xAFAF,0x7070,0x9F9F,0xAFAF,0xAFAF,0x7070,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x4242,0x4242,0x4742,0x4242,0x4242,0x4242,0x4742,0x4747,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4247,0x4742,0x4242,0x4242,0x4247,0x4242,0x4242,0x4242,0x4747,0x4747,0x4242,0x4247,0x4242,0x4242,0x4747,0x4747,0x4242,0x4242,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F
.word 0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0xAFAF,0x9F9F,0x6161,0x7271,0x8272,0x8272,0x8282,0x8282,0x8282,0xC3B2,0xB281,0x8282,0x8282,0x8282,0x8282,0x9282,0x82B3,0xB2B2,0xB2B3,0xB2B3,0x82B3,0x8282,0x8282,0x9292,0x9282,0x9282,0x9282,0x8282,0x82B3,0x8282,0x7282,0x7272,0xD472,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0x8E8E,0x9F9F,0x7E7E,0x9F9F,0x7E7E,0x9F9F,0x8E8E,0xAFAF,0x8E8E,0x708E,0x8E8E,0x8E8E,0x9F9F,0x9D9D,0xBFBF,0x8F8F,0xAFAF,0x6060,0xAFAF,0x7070,0x6060,0x6060,0x6060,0x5050,0x9F9F,0x5050,0x5050,0x8F8F,0x5151,0x9F9F,0x9F9F,0x5151,0x9F9F,0xAFAF,0x7070,0x9F9F,0xAFAF,0x7070,0x6060,0x7070,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4242,0x4747,0x4242,0x7F7F,0x4242,0x4747,0x4242,0x4747,0x4242,0x4742,0x4242,0x4747,0x4242,0x4242,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4246,0x4242,0x4747,0x4742,0x4242,0x4242,0x4242,0x4742,0x4242,0x4242,0x4646,0x4242,0x4242,0x4747,0x4242,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x408F
.word 0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x9F9F,0x8E8E,0x9F9F,0x6161,0x8171,0x8172,0x8281,0x8281,0x8282,0x8282,0xB2B2,0x82B3,0x82B2,0x9282,0x8282,0x8292,0xB2B3,0xB2B2,0xB2B3,0x81C3,0x81C3,0xB3B2,0x82B2,0x9282,0xB282,0x92B2,0x9292,0x8292,0x9292,0x8292,0x8282,0x8282,0x7282,0x7171,0xBFBF,0x7E7E,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8E8E,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0xAFAF,0x8E8E,0x9F9F,0x8E8E,0x7E7E,0x9F9F,0x8E8E,0x9F9F,0x8E8E,0x9E9E,0x9F9F,0x9F9F,0x6060,0x7070,0x9F9F,0x9F9F,0x5050,0x5050,0x9F9F,0x5050,0x8F8F,0x5050,0x5151,0x9F9F,0x6060,0x9F9F,0x9F9F,0x6060,0x9F9F,0xAFAF,0x7070,0x9F9F,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0x8F8F,0x8F8F,0x6F6F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x4242,0x4742,0x4646,0x4242,0x4246,0x4242,0x4646,0x4242,0x4646,0x4246,0x4242,0x4747,0x4242,0x4242,0x4646,0x4246,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F
.word 0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x5050,0x5151,0x4141,0x8F8F,0x4141,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x7272,0x7271,0x8272,0x8282,0x8282,0x8282,0x81C2,0x82B2,0x8282,0x8282,0x9282,0xB282,0x8282,0x82B2,0x8282,0xC3B2,0xB382,0x8282,0x8282,0x8292,0xB2B2,0xB2B2,0x9292,0x9292,0x9282,0x92A3,0x8282,0x8282,0x7282,0x7172,0xD0D0,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x7E7E,0x8F8F,0x8F8F,0xAFAF,0x7E7E,0x9F9F,0x8F8F,0xAFAF,0x7E7E,0x9F9F,0x8E8E,0xAFAF,0xAFAF,0x8E8E,0xAFAF,0x7E7E,0x9D70,0x9D9D,0x9E9E,0x8F8F,0x7F7F,0x9F9F,0x7070,0x6060,0x6060,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x5151,0x9F9F,0x6060,0x9F9F,0x9F9F,0x7070,0x9F9F,0x7070,0x6060,0x7070,0x7070,0xAFAF,0x9F9F,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4747,0x4747,0x4242,0x4747,0x4242,0x4842,0x4242,0x4242,0x4747,0x4242,0x4242,0x4242,0x7F7F,0x4742,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4247,0x4242,0x4242,0x4242,0x4646,0x4247,0x4747,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7E7E,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F
.word 0x4040,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x5040,0x5050,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0x7E7E,0x9F9F,0x9F9F,0xC0C0,0x81C3,0x7171,0x8172,0x8182,0x8282,0xB281,0xC2B2,0x8282,0x8282,0x8282,0x82B2,0x82B2,0xB3B2,0xB3B2,0x82B2,0x82C3,0x82B2,0x9282,0x9292,0x9292,0xB292,0xB2B2,0x8292,0x8292,0x8292,0x8292,0x8292,0xB3B2,0x7282,0x7172,0xE0E0,0xCFCF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7E7E,0x8E8E,0x9F9F,0xAFAF,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0x8E8E,0x8E8E,0xBFBF,0x8F8F,0x7F7F,0x6060,0x6060,0x6060,0x9F9F,0x9F9F,0x5050,0x5050,0x8F8F,0x5050,0x5151,0x5050,0x5151,0x9F9F,0x9F9F,0x7070,0x9F9F,0x7070,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x7070,0x7070,0x8F8F,0x8F8F,0x7F7F,0x4242,0x8F8F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x4242,0x4747,0x4646,0x4642,0x4242,0x7F7F,0x4742,0x4242,0x4242,0x4747,0x4242,0x7F7F,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4646,0x4242,0x7F7F,0x8F8F,0x6E6E,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F40
.word 0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x4141,0x8F8F,0x8F8F,0x4040,0x7F7F,0x9F9F,0x8F8F,0xAFAF,0xD3D0,0x7171,0x7161,0x8271,0x8272,0x8282,0x8292,0xC3B2,0xB381,0xB282,0xB282,0x82B2,0x82B2,0x8282,0x8282,0x8282,0x8282,0x8282,0x92B2,0x9292,0x9292,0xB392,0xB3B2,0x9292,0x8292,0x8292,0x9282,0x92B3,0x82B3,0x7282,0xD471,0xE0E3,0xCFCF,0xAFAF,0x9F9F,0x8F8F,0x8E8E,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7E7E,0x9F9F,0xAFAF,0x7E7E,0x8E8E,0x8E8E,0x9F9F,0xAFAF,0x8E8E,0x9F9F,0x8E8E,0x9D9D,0x9E9E,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0x6060,0x5050,0x5050,0x9F9F,0x5050,0x5050,0x9F9F,0x9F9F,0x9F9F,0x5050,0x9F9F,0x7070,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x7070,0x7070,0x7070,0x6060,0x9F9F,0xAFAF,0x9F9F,0x7F7F,0x8F8F,0x7F7F,0x4242,0x7F7F,0x4242,0x5656,0x5656,0x5656,0x4747,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x4246,0x4242,0x4646,0x4242,0x4242,0x4747,0x4242,0x4242,0x4247,0x4747,0x4242,0x4646,0x7F7F,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F
.word 0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x5050,0x5050,0x4040,0x4141,0x4040,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0xAFAF,0xD0D0,0xBFBF,0x6161,0x8171,0x8182,0x8282,0xB282,0xC2B2,0xC3B2,0xC3B2,0xC3C2,0x82B2,0xB282,0x8282,0x8282,0x8282,0xB382,0xB282,0x9282,0x9292,0x9292,0x9292,0x92B2,0x82B2,0x8292,0x9282,0x9282,0xB382,0x8282,0x7282,0xD071,0xE4E4,0xF2F2,0xAFAF,0x8F8F,0x8F8F,0x9F9F,0x7E7E,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x8E8E,0x8E8E,0x9F9F,0xAFAF,0xAFAF,0x8E8E,0x8E8E,0xAFAF,0x8E8E,0xAFAF,0x9E9E,0x8E8E,0x8F8F,0x9F9F,0x7F7F,0x6060,0x6060,0x9F9F,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x9F9F,0x6060,0x9F9F,0x9F9F,0x7070,0xAFAF,0x7070,0xAFAF,0xAFAF,0x9F9F,0x6060,0x7070,0x7070,0x9F9F,0x7F7F,0x8F8F,0x7F7F,0x4242,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x4247,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4646,0x4242,0x4646,0x4646,0x4242,0x4242,0x4646,0x4646,0x4242,0x4242,0x4242,0x4646,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F
.word 0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x9F9F,0x8F8F,0xAFAF,0xD4D4,0x61D4,0x9F9F,0xC361,0x8282,0x9281,0xB282,0xC381,0xB381,0xC381,0x81C3,0xB2C3,0x8282,0x8282,0x8282,0xC382,0x8282,0x9292,0x8292,0x8292,0x9292,0x9282,0xB392,0x8292,0x8292,0xA392,0xB392,0xB292,0x8282,0xC372,0xC471,0xC0C0,0xCFCF,0x8E8E,0x8F8F,0x8F8F,0x7E7E,0xAFAF,0x7E7E,0x8F8F,0x7F7F,0x8F8F,0x7E7E,0x8E8E,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0x9F9F,0x9F9F,0x8E8E,0x9F9F,0x8E8E,0xAFAF,0x8E8E,0x9F9F,0x9F9F,0x7F7F,0x8F8F,0x6060,0x5050,0x5050,0x8F8F,0x8F8F,0x5050,0x5151,0x9F9F,0x9F9F,0x6060,0x6060,0x9F9F,0x6060,0x9F9F,0x6060,0xAFAF,0x9F9F,0x6060,0x7070,0x6060,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x6E6E,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F
.word 0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x5040,0x4040,0x4040,0x4040,0x8F8F,0x9F9F,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x7171,0x8281,0x8282,0x82A1,0xC2B2,0x81C3,0xC3B3,0xC381,0x82B2,0x8282,0x8282,0x8282,0x8282,0xB282,0x9292,0x9292,0x9292,0x8292,0x9292,0x9282,0x9292,0x8282,0x8292,0xB292,0xB382,0x8282,0x72C3,0x7172,0xAFAF,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0x9F9F,0x7E7E,0xAFAF,0x7E7E,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x7E7E,0x7E7E,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0x9F9F,0x8E8E,0x8E8E,0x9D9D,0x9D9D,0x9F9F,0x9F9F,0x7F7F,0x5050,0x9F9F,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x5151,0x6060,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x9F9F,0xAFAF,0x7070,0x7070,0x5050,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x4242,0x4242,0x4747,0x4242,0x4242,0x4747,0x4242,0x4242,0x4242,0x4747,0x4242,0x4242,0x4646,0x4242,0x4747,0x4242,0x4242,0x4646,0x4242,0x4646,0x4242,0x4747,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F
.word 0x4040,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x7E7E,0x9F9F,0x6060,0x9F9F,0x6161,0xC361,0x8281,0x8282,0xB2B2,0xB2B2,0xB382,0xC381,0xB2B2,0x8282,0x82B2,0x8282,0x8282,0x82B2,0x82B2,0x8292,0x9282,0x9282,0x9292,0x8282,0x8292,0x8282,0x9282,0x9282,0x8282,0x82B3,0xB2B3,0xC381,0x7171,0x6161,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x7E7E,0xAFAF,0x7E7E,0xAFAF,0x7E7E,0x7E7E,0x7E7E,0x8E8E,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0x8E8E,0x6060,0xAFAF,0x8E8E,0x9E9E,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x9F9F,0x5050,0x9F9F,0x6060,0x9F9F,0x7070,0x9F9F,0xAFAF,0x7070,0xAFAF,0x9F9F,0x7070,0x7070,0x7070,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x4242,0x4242,0x4242,0x4242,0x4747,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4646,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F
.word 0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7171,0x8272,0xB282,0x8282,0xB2C3,0x82C2,0x81B3,0x81B3,0x82B3,0x8282,0x8282,0x8282,0x8282,0x9282,0x9282,0x9282,0x8292,0x9292,0x8292,0x9292,0x8292,0x8292,0xB392,0xB2B2,0xB282,0xB382,0x81C3,0x7172,0xD4D4,0xAFAF,0x6E6E,0x7F7F,0x8F8F,0x8F8F,0x7E8E,0x9F9F,0x7E7E,0x7E7E,0x9F9F,0x9F9F,0x8E8E,0xAFAF,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0x8E8E,0x9F9F,0x7E7E,0x8E8E,0xAFAF,0x9D9E,0x8E8E,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x6060,0x9F9F,0x9F9F,0x9F9F,0x6060,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x9F9F,0x9F9F,0x7070,0x7070,0x7070,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x4242,0x4242,0x4242,0x4747,0x4242,0x4747,0x4747,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4646,0x4242,0x7F7F,0x4646,0x4646,0x4242,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F
.word 0x4040,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x7E7E,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x7171,0x8271,0xB281,0xB2B2,0xC381,0xC381,0xC382,0xB3B3,0x8281,0x8272,0x8281,0x8282,0x8282,0xB382,0x82B2,0x8292,0x9292,0x9282,0x9282,0xB392,0x8292,0xB392,0x9292,0xB2B3,0x82B3,0x8182,0xC3C3,0x61D4,0x6161,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0x8E8E,0x7E7E,0xAFAF,0x7E7E,0x7E7E,0x8E8E,0x7E7E,0x9F9F,0x7E7E,0xAFAF,0xAFAF,0xAFAF,0x8E8E,0xAFAF,0x9F9F,0x8E8E,0xAFAF,0x8E8E,0x9F9F,0x8F8F,0x7F7F,0x5050,0x5050,0x8F8F,0x9F9F,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0xAFAF,0x9F9F,0x7070,0xAFAF,0x9F9F,0x6060,0x7070,0x7070,0x9F9F,0x9F9F,0x6060,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4242,0x8F8F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4646,0x7F7F,0x4242,0x4646,0x4242,0x4646,0x7F7F,0x4242,0x4242,0x4246,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F
.word 0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8272,0xB282,0x82B2,0xB282,0xB2C3,0xB381,0x8181,0xC382,0x81B3,0xB382,0x8281,0x8282,0xB2B3,0x8292,0x8292,0x8292,0x9292,0xB2B3,0x92B3,0x9282,0x9282,0xB3B2,0x92B2,0x8282,0x7282,0xC0C3,0x7171,0xBFBF,0xAFAF,0x6E6E,0x8F8F,0x8F8F,0x7E7E,0x8E8E,0xAFAF,0x8E8E,0x7E7E,0x9F9F,0xAFAF,0x8E8E,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0x9E9E,0x9D9D,0x7E7E,0x8F8F,0x6F6F,0x8F8F,0x5050,0x5050,0x5050,0x9F9F,0x8F8F,0x5050,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x7070,0x9F9F,0x6060,0x6060,0x9F9F,0x9F9F,0x9F9F,0x7070,0x7070,0x6060,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4242,0x7F7F,0x6F6F,0x4242,0x4242,0x5656,0x4747,0x5642,0x4242,0x4242,0x4242,0x4242,0x4646,0x4242,0x4646,0x4242,0x4242,0x4242,0x4242,0x4646,0x4646,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F
.word 0x6F6F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x9F9F,0x7F7F,0x6E6E,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x6161,0xC371,0x81C3,0x82B3,0x81C3,0x81B3,0x81B3,0xC3C3,0x81C3,0x81C3,0x71B3,0x8282,0xC3B3,0x82B2,0x8282,0x8292,0x9282,0xB282,0x8292,0x9282,0x8282,0xB382,0xB2B3,0x8282,0x8282,0xB382,0xC371,0x7171,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x8E8E,0x9F9F,0x607E,0x7E7E,0x8E8E,0xAFAF,0x7E7E,0x7E7E,0x8F8F,0x7E7E,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0x9E9E,0x9E9E,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x5050,0x8F8F,0x5050,0x5050,0x6060,0x9F9F,0x6060,0x9F9F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x7070,0x7070,0x7070,0x9F9F,0x9F9F,0x9F9F,0x7070,0x4040,0x6F6F,0x7F7F,0x6F6F,0x4242,0x8F8F,0x4242,0x4242,0x4242,0x5656,0x4242,0x4242,0x7F7F,0x4242,0x4646,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4646,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x4242,0x4242,0x4242,0x7F7F,0x4242,0x4242,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F8F
.word 0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x9F9F,0x7F7F,0x7E7E,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x8181,0xB2C3,0x82B2,0xB3B2,0xB3B2,0xB381,0xB381,0x81C3,0x8282,0x8282,0x8282,0x8282,0x82B2,0x9292,0x9292,0x9292,0x8292,0x9292,0x8282,0x9292,0xB292,0xB3B2,0x8292,0x8282,0x8182,0x7272,0x71C3,0xAFAF,0x7E7E,0x6E6E,0x8F8F,0x7F7F,0x8F8F,0x7E7E,0x8E8E,0x8E8E,0x7E7E,0x9F9F,0x7E7E,0xAFAF,0x7E7E,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0x8E8E,0x9F9F,0x8E8E,0xAFAF,0x9D9D,0x9D9D,0x9E9E,0x9F9F,0x6F6F,0x8F8F,0x8F8F,0x5050,0x5050,0x9F9F,0x5050,0x9F9F,0x9F9F,0x6060,0x6060,0x9F9F,0x9F9F,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x9F9F,0x9F9F,0x9F9F,0x6060,0x7070,0x9F9F,0x9F9F,0x9F9F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4242,0x7F7F,0x7F7F,0x4242,0x4242,0x4242,0x5656,0x4256,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4242,0x4646,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4246,0x4242,0x4242,0x4242,0x4646,0x4242,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x4040,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x9F9F,0x7F7F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x6161,0xC372,0x8182,0x8182,0x81B3,0x81B3,0xB3C3,0x81B3,0xB381,0x7282,0x7282,0x8282,0x82B2,0x8282,0x8282,0x8282,0x8282,0x9282,0x8282,0x8292,0x8282,0x8282,0x82B3,0x8282,0x8282,0x8172,0x71C3,0x7172,0x6161,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x8E8E,0x9F9F,0x8E8E,0xAFAF,0x8E8E,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0x7E7E,0x7E60,0xAFAF,0x8E8E,0x9F9F,0xAFAF,0x9E9E,0x8E8E,0x8E8E,0xAFAF,0x6F6F,0x7F7F,0x5050,0x5050,0x8F8F,0x5050,0x9F9F,0x6060,0x6060,0x9F9F,0x9F9F,0x6060,0x6060,0x9F9F,0x9F9F,0x6060,0x9F9F,0x7070,0x7070,0x7070,0x6060,0x9F9F,0x9F9F,0x6060,0x7070,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x4242,0x7F7F,0x4242,0x4242,0x7F7F,0x4646,0x4242,0x4242,0x4242,0x4242,0x4242,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F
.word 0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x6F6F,0x4E4E,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7161,0x7161,0x7162,0x7161,0x6261,0x7161,0x6161,0x6162,0x6161,0x6162,0x6171,0x7162,0x7162,0x7262,0x7271,0x7271,0x7271,0x7271,0x7262,0x6272,0x6271,0x7271,0x6271,0x6271,0x6261,0x6161,0x5161,0x9F9F,0x9F9F,0x6E6E,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8F8F,0x6F6F,0x6F6F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x5050,0x5050,0x8F8F,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050,0x5050,0x8F8F,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4242,0x4242,0x4242,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x6F6F,0x6F6F,0x6F6F,0x4242,0x4242,0x4242,0x4242,0x6F6F,0x4242,0x4242,0x4242,0x4242,0x6F6F,0x4242,0x4242,0x6F6F,0x4242,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F
*/


;/* Fish 2
Fish_320:
.word 0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x5040,0x5050,0x5050,0x5050,0x5050,0x6060,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x5151,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x6A6A,0x9F9F,0x5050,0x5050,0x5050,0x6060,0x5050,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xD5E6,0xBFBF,0xBFBF,0xD5D5,0xBFBF,0xBFBF,0xAFAF,0x5252,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x5C4A,0x7C7C,0x8C7C,0x7B8B,0x6B8C,0x7B7C,0x7C7C,0x4A4A,0x5C5C,0x6C6C,0x7C7C,0x6C6C,0x6C6C,0x4A4A,0x5C5C,0x4A4A,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F7F
.word 0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x7070,0x7070,0xAFAF,0x8B8B,0x4848,0x4242,0x4141,0x8F8F,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x4949,0x9F9F,0x8F8F,0x8F8F,0x5050,0x6060,0x6060,0x5050,0x4040,0x4040,0x8F8F,0x9F9F,0x7070,0x9F9F,0x9E9E,0xCECE,0xBFBF,0xCBCB,0xCFCF,0xAFAF,0xBFBF,0xC0C0,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x4A4A,0x6F6F,0x6F6F,0x5C5C,0x4A4A,0x7C6E,0x7B8C,0x7B8C,0x8C7B,0x8C8B,0x7C8C,0x4E4E,0x4E4E,0x7C7C,0x7C7C,0x6C7E,0x5C5C,0x6E6E,0x6C6C,0x8F8F,0x4C4C,0x5C5C,0x4A4A,0x5C5C,0x4A4A,0x4A4A,0x4E4E,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x4C4C,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F6F
.word 0x8F8F,0x5050,0x5040,0x8F8F,0x4040,0x5050,0x8F8F,0x8F8F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x5050,0x6060,0x5050,0x5050,0x5060,0x6060,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x5050,0x5050,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0xAFAF,0x9F9F,0x4040,0x4040,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x7F7F,0x7E50,0x6060,0x9F9F,0x9F9F,0xAEAE,0xDCBE,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xC0C0,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7E6C,0x8D8C,0x7C7C,0x8C4A,0x7B7B,0x8B8B,0x6C6C,0x8F8F,0x8F8F,0x7D7E,0x4A7E,0x4A6E,0x6C6C,0x6C6C,0x4A4A,0x4E4E,0x4A4A,0x5C5C,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x4C4C,0x7F7F,0x4E4E,0x4C4C,0x4C4C,0x7F7F,0x4C4C,0x6F6F,0x4C4C,0x7F7F,0x4C4C,0x4C4C,0x7F7F,0x4E4E,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x5050,0x5050,0x4040,0x7F7F,0x5050,0x9F9F,0x9F9F,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F7F
.word 0x4040,0x5050,0x8F8F,0x5050,0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6E6E,0x8F8F,0x4040,0x6060,0x9F9F,0x6060,0x6060,0x7070,0x9F9F,0x6060,0x7070,0x6060,0x9F9F,0x4242,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x4040,0x8F8F,0x8E8E,0xAFAF,0x5050,0x7E7E,0xAE8E,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAEBC,0x9E9E,0xC0C0,0xAFAF,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4C4C,0x6C4A,0x8D7C,0x8D8D,0x7C7C,0x7C7C,0x8B8C,0x7B7B,0x4A8C,0x4C4C,0x6E4A,0x7C7C,0x7E7C,0x7C7C,0x4A7E,0x6C6C,0x7C7C,0x4A4A,0x4A4A,0x8F8F,0x8F8F,0x4C4C,0x4C4C,0x7F7F,0x8F8F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x5050,0x707E,0xAFAF,0x9F9F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F6F
.word 0x7F7F,0x4040,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x4A4A,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x5050,0x7060,0x6060,0x6060,0x6060,0x7070,0x7070,0x6060,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x5050,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4E4E,0x8F8F,0x8F8F,0x4141,0x4040,0x8F8F,0x4040,0x8F8F,0x9F9F,0x5050,0x8F8F,0x9F9F,0x9E9E,0xAFAF,0x8E8E,0xAFAF,0x8E8E,0xBEBE,0xBFBF,0xCFCF,0xDADA,0xBFBF,0xCFCF,0x8080,0x8080,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x8C8C,0x8C8C,0x7C7C,0x6B6B,0x8B8B,0x6B6B,0x4A4A,0x5C5C,0x8F8F,0x7E4A,0x4A4A,0x4A4A,0x4A4A,0x4A4A,0x4A4A,0x7F7F,0x7F7F,0x4A4A,0x4C4C,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x7F7F,0x4C4C,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x4040,0x8F8F,0x5050,0x6060,0xAFAF,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F
.word 0x4040,0x4040,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0x4E4E,0x4848,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6060,0x6060,0x6060,0x7070,0x7070,0x6060,0x6060,0x7070,0x6060,0x9F9F,0x5050,0x8F8F,0x7F7F,0x8F8F,0x7E7E,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x4040,0x8F8F,0x4040,0x8F8F,0x9F9F,0x9F9F,0x5050,0x8F8F,0x8E8E,0x9E9E,0x7070,0x708E,0x9E9E,0xAFAF,0xCFCF,0xCFCF,0xCFCF,0xE4E4,0xCFCF,0xCFCF,0xAFAF,0x7070,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4C4C,0x4C4C,0x6C6C,0x7C7C,0x4A4A,0x7C7C,0x8C6B,0x6B6B,0x6B6B,0x8F8F,0x4A4A,0x4E4E,0x8F8F,0x5C5C,0x6C6C,0x6C6C,0x5C5C,0x7F7F,0x5C5C,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4A4A,0x4C4C,0x6F6F,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x4C4C,0x4E4E,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0x5050,0x5050,0x6060,0x6060,0x8F8F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F7F
.word 0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x4A4A,0x9F9F,0x4A4A,0x4A4A,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5050,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6E6E,0x9F9F,0xAFAF,0x6060,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x9F9F,0x4040,0x7F7F,0x7E7E,0x9F9F,0x9F9F,0xAFAF,0x9E9E,0x9E9E,0x708E,0x8E8E,0xAEAE,0xBFBF,0x8E70,0xAFAF,0x9E9E,0xBFBF,0xE0E0,0xE0E0,0xBFBF,0xCFCF,0xD0D0,0xC0C0,0xBFBF,0xAFAF,0x8F8F,0x7F7F,0x8F8F,0x7C7C,0x4C4C,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x4A4A,0x6C6C,0x7C7C,0x7B4A,0x4A7B,0x8F8F,0x4C4C,0x7F7F,0x4A4A,0x5C5C,0x5C5C,0x6C4A,0x4A4A,0x8F8F,0x4C4C,0x6F6F,0x4C4C,0x4C4C,0x4C4C,0x4C4C,0x7F7F,0x6F6F,0x7F7F,0x4C4C,0x4C4C,0x7F7F,0x8F8F,0x8F8F,0x4C4C,0x4E4E,0x4E4E,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7070,0x7070,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F
.word 0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x4E4E,0x9F9F,0x4848,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x7E7E,0xAFAF,0xAFAF,0x8080,0x7070,0x8F8F,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x5050,0x8F8F,0x8E8E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0xAEAE,0xBFBF,0x9F9F,0x7070,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xC0C0,0x6060,0x8F8F,0x6E6E,0xAFAF,0x4A4A,0x7F7F,0x4C4C,0x7F7F,0x4A4A,0x5C5C,0x4A4A,0x8F8F,0x4A4A,0x7B8C,0x4A4A,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x6C6C,0x7C7C,0x6C6C,0x4C4C,0x6F6F,0x7F7F,0x4C4C,0x7F7F,0x4C4C,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x6060,0x7070,0x5050,0x5050,0x5050,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x8F8F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F7F
.word 0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x6A6A,0x9F9F,0x4242,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x8F8F,0x8F8F,0x8F8F,0x5050,0x7060,0x6060,0x7070,0x7060,0x7070,0x7070,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x6E6E,0x8E8E,0xAFAF,0xAFAF,0x7070,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x4141,0x7F7F,0x4E4E,0x9F9F,0x7070,0x9F9F,0x7E7E,0x8E8E,0x809E,0x709E,0x9E9E,0xBFBF,0xBEBE,0xBFBF,0xBEBE,0xBFBF,0x7070,0xAFAF,0x8E8E,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xD0D0,0xAFAF,0x9F9F,0x7F7F,0x9F9F,0xAFAF,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4C4C,0x8F8F,0x4A4A,0x6E6E,0x6A6A,0x4A4A,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x5C4C,0x7C4B,0x7C7C,0x4E4E,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4C4C,0x7F7F,0x4C4C,0x4C4C,0x4C4C,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F6F
.word 0x4040,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4A4A,0x5252,0x9F9F,0x8F8F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x6060,0x6060,0x7070,0x7070,0x6060,0x7070,0x6060,0x9F9F,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBCBC,0x7171,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x4242,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x5050,0x8F8F,0x9E9E,0xBEBE,0x9E9E,0x9E9E,0xBEBE,0xCFCF,0xCECE,0xCFCF,0xD0D0,0xBFBF,0xAFAF,0x8E70,0x9F9F,0x9F9F,0x6A6A,0x7878,0xBFBF,0xCFCF,0xD0D0,0xBFBF,0xBFBF,0xAFAF,0x8F8F,0x6C6C,0x4A4A,0x6868,0x6C6C,0x7F7F,0x7F7F,0x6F6F,0x4C4C,0x5C5C,0x4A4A,0x7F7F,0x5C5C,0x4A4B,0x4C4C,0x7F7F,0x4C4C,0x7F7F,0x4C4C,0x7C7C,0x8C7C,0x7D7E,0x6E6E,0x4C4C,0x7F7F,0x4C4C,0x4A4A,0x5C5C,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4C4C,0x4C4C,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x5040,0x6060,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x5050,0x9F9F,0x5050,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F7F
.word 0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x6060,0x6060,0x7070,0x7060,0x7070,0x7060,0x7070,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x6E6E,0x7C7C,0x9B9B,0xAFAF,0xAFAF,0x6060,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x4242,0x8F8F,0x8F8F,0x8F8F,0x6060,0x9F9F,0x9F9F,0x8E8E,0xE0E0,0xC0C0,0xAFAF,0xAFAF,0xBFBF,0xD0D0,0xBFBF,0xAED0,0x8080,0x7070,0x8E8E,0x9F9F,0x4A4A,0x5858,0xC6C6,0xBFBF,0xD0D0,0xBFBF,0xBFBF,0xAFAF,0x49BB,0x4A4A,0x4A4A,0x499C,0xAFAF,0x9F9F,0x4A4A,0x7F7F,0x4C4C,0x7F7F,0x5C5C,0x7F7F,0x7F7F,0x4A4A,0x6C4A,0x4A4A,0x4A4C,0x6F6F,0x6F6F,0x6C5C,0x8C7D,0x8C8C,0x8D8C,0x4D7D,0x4C4C,0x4C4C,0x6F6F,0x7F7F,0x4A4A,0x4C4C,0x4C4C,0x4C4C,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x4C4C,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x6F6F,0x7F7F,0x4C4C,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x6F6F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4242,0x4141,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x6060,0x6060,0x6060,0x6060,0x6070,0x6060,0x6060,0x6060,0x5050,0x4040,0x4040,0x8F8F,0x8F8F,0x7E7E,0x9F9F,0x4A4A,0xAFAF,0xC5C5,0x4242,0x9F9F,0x9F9F,0x7070,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7E60,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0x8E70,0x6070,0x6E6E,0x4A4A,0x5858,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x8B8B,0x7A7A,0x8F8F,0x7C7C,0x4A4A,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x5C5C,0x8F8F,0x4C4C,0x4C4C,0x4C4C,0x7C7C,0x5C5C,0x4A4A,0x4C4C,0x5C5C,0x6C6C,0x8B9C,0x7C8C,0x8E8E,0x7C8C,0x4A4E,0x5C5C,0x4C4C,0x4C4A,0x5C5C,0x4A4A,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x4C4C,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x4C4C,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F7F
.word 0x7F7F,0x4040,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5050,0x5050,0x5050,0x5050,0x6060,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x7070,0x4040,0x8F8F,0x8F8F,0x9F9F,0x7070,0xAFAF,0x9F9F,0x8B8B,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x707E,0xBFBF,0xAFAF,0xAFAF,0x5252,0x9F9F,0x8F8F,0x5050,0x7070,0x7070,0x8E8E,0xAFAF,0xAFAF,0x7070,0x6070,0x9F9F,0x9E70,0x7070,0x709E,0xAFAF,0x7070,0x7070,0x9F9F,0x7E7E,0x7E7E,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0x6A6A,0x8B8B,0x8B8B,0x4A4A,0x4A4A,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4A4A,0x4A4A,0x4A4A,0x4E4E,0x5C5C,0x4A4A,0x8C8C,0x8C8C,0x4A8C,0x8C8C,0x9F9F,0x4A4A,0x6C6C,0x4E4E,0x4E4E,0x4A4A,0x7F7F,0x4C4C,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4C4C,0x4E4E,0x4C4C,0x7F7F,0x6F6F,0x4C4C,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F6F
.word 0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x6060,0x6070,0x7070,0x6060,0x6060,0x7070,0x5060,0x8F8F,0x7F7F,0x8F8F,0x7070,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x4A4A,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x7070,0x8080,0x7070,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8E8E,0x8E70,0xAFAF,0x9F9F,0x5050,0x8F8F,0x8E50,0x9E70,0xAFAF,0x9E70,0x7070,0xAFAF,0xAFAF,0x8F8F,0x8E8E,0x9E9E,0xAEAE,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBCBC,0xBBAC,0x6A9B,0x8F8F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x4A4A,0x4C4C,0x5C5C,0x4A4A,0x7C4A,0x7E6B,0x4A4A,0x7F7F,0x4A5C,0x6C6C,0x6C6C,0x4A4A,0x4A4A,0x7F7F,0x6F6F,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F7F
.word 0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x6060,0x5050,0x6060,0x6060,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x4050,0x8F8F,0x6060,0x8080,0xC0C0,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x4949,0x9F9F,0x9F9F,0x9F9F,0x7070,0x6060,0x9F9F,0x9F9F,0x8C8C,0xAFAF,0x9F9F,0x9E70,0x7070,0x7070,0x7070,0x6060,0x5050,0x4040,0x7070,0x9E9E,0x9E9E,0x709E,0xAFAF,0x7070,0x7070,0xAFAF,0xAFAF,0xBEBE,0xCFCF,0xBFBF,0x8080,0xAFAF,0xBBBB,0xCBCB,0xDACB,0x9CCB,0x6B6B,0x7F7F,0x4A4A,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x8F8F,0x7F7F,0x7F7F,0x4B4B,0x6B6B,0x4A4A,0x6C6C,0x4C4C,0x8F8F,0x4A7C,0x7C4A,0x4A4A,0x5C5C,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x4C4C,0x4C4C,0x4E4E,0x4C4C,0x7F7F,0x4C4C,0x4C4C,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x5050,0x8F8F,0x9F9F,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x6060,0x5060,0x6060,0x6060,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x5050,0x8F8F,0x8F8F,0x8F8F,0x7070,0xAFAF,0xAFAF,0x4848,0x8F8F,0x9F9F,0x4848,0x9F9F,0x9F9F,0x7070,0x7070,0x7070,0x9F9F,0x9F9F,0x7B7B,0xAFAF,0x8F8F,0x7070,0xAFAF,0x7070,0x708E,0x6060,0x5050,0x8F8F,0x8F8F,0x708E,0x9E9E,0x7070,0x6060,0x7070,0xAFAF,0x7070,0xAFAF,0xD0D0,0xBFBF,0xC0C0,0xBFBF,0xAFAF,0x9B8B,0xCBCB,0xCBDA,0xBBDA,0x4A8C,0x5C5C,0x5C6C,0x4A4A,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4C4C,0x6F6F,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x4C4C,0x4A4A,0x7F7F,0x7F7F,0x6C4A,0x6B4A,0x7C6B,0x4A4A,0x4A4A,0x4A6C,0x4A7C,0x9F9F,0x4A4A,0x7F7F,0x7F7F,0x4C4C,0x4C4C,0x4C4C,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x4040,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4E4E,0x8F8F,0x6060,0x6060,0x6050,0x9F9F,0x5050,0x9F9F,0x6060,0x7070,0x7070,0x7070,0x6060,0x6060,0x9F9F,0x5050,0x4040,0x8F8F,0x7070,0xC0C0,0x9F9F,0x9F9F,0x8F8F,0x4A4A,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x7070,0x9F9F,0x8F8F,0x4A4A,0xAFAF,0xAFAF,0x5050,0x9F9F,0x7070,0x7070,0x7070,0x6060,0x6060,0x8F8F,0x9F9F,0x9E70,0x7070,0x708E,0x9E9E,0xAFAF,0x7070,0xAFAF,0xAFAF,0x8EC0,0xC0C0,0xC0C0,0xAFAF,0x8B8D,0x9B8B,0xBB9B,0xBBBB,0xBBBB,0x8C9B,0x8B7B,0x8B8B,0x4A4A,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4C4C,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x4C4C,0x5C5C,0x7C7C,0x8C7B,0x6C6B,0x4A4E,0x6E6E,0x7C7C,0x4A6C,0x4C4C,0x4C4C,0x4C4C,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x4C4C,0x4A4A,0x7F7F,0x4C4C,0x7F7F,0x4E4E,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4C4C,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F
.word 0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6060,0x6060,0x6060,0x6060,0x6050,0x6060,0x6060,0x9F9F,0x7070,0x7070,0x7070,0x6060,0x6060,0x5050,0x8F8F,0x8F8F,0x7060,0xAFAF,0x7070,0xAFAF,0x4848,0x4848,0x4141,0x7F7F,0x8F8F,0x8F8F,0x6060,0x7070,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5050,0x9F9F,0x6060,0x9F9F,0x6060,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0x7070,0x9E9E,0x9E80,0xBFBF,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x9E9E,0xAFAF,0x9B9B,0x9B9B,0xBC9B,0xCCCC,0xCBDA,0x9BDA,0xBBAB,0xDADA,0x7C9C,0x4C4C,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4A4A,0x6B6C,0x8C7B,0x9C9C,0x9C9C,0x7E7C,0x7E7C,0x7C7C,0x6C6C,0x4A4A,0x6C6C,0x4E4E,0x8F8F,0x4E4E,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x7F7F
.word 0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6060,0x7060,0x6060,0x5060,0x5050,0x5050,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x5050,0x6060,0x5050,0x7070,0x7070,0x8080,0xC080,0x8080,0x6262,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x7070,0x6060,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x5050,0x4040,0x8F8F,0x8F8F,0x9F9F,0x9E9E,0xAFAF,0x7070,0x7070,0xAFAF,0x9E9E,0xBFBF,0x709E,0xAFAF,0x7070,0xAFAF,0xBFBF,0xC0C0,0xAFAF,0x6A6A,0xBC9C,0xBCBC,0xDBDB,0xBBDB,0xABDA,0xBBCA,0xDACA,0x8B9B,0x4A4A,0x7F7F,0x4C4C,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4C4C,0x4E4E,0x9C7B,0x9C8B,0x8B8B,0x8B8B,0x8C8C,0x9E9E,0x8C8C,0x7C7C,0x7C7C,0x4E4A,0x6E6E,0x6E6E,0x8F8F,0x4C4C,0x7F7F,0x4C4C,0x4C4C,0x8F8F,0x5C5C,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x7F7F,0x8F8F,0x5050,0x5050,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x6060,0x6060,0x6060,0x5050,0x5050,0x6060,0x6060,0x7070,0x7070,0x6060,0x7070,0x6060,0x7070,0xD0B0,0xD0D0,0xC0D0,0xD0D0,0xD0D0,0xB1D0,0xC2C2,0xC3B3,0x5252,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6060,0x708E,0x9F9F,0x8F8F,0x4E4E,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x9E9E,0xAFAF,0x7070,0xAFAF,0x7070,0xBFBF,0xBFBF,0x7070,0x7070,0x7070,0x7070,0xAFAF,0xC0C0,0x9F9F,0x7B7B,0xAFAF,0xAFAF,0xAFAF,0xCBCB,0x9BBB,0xAB9B,0xBBBB,0x9BBB,0x4A8C,0x4A4C,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7B6C,0x8B8B,0x7B8B,0xAFAF,0x6A6A,0x9E9E,0x9E9E,0x8C8C,0x8B8B,0x4B4B,0x6C6C,0x6E7E,0x9F9F,0x8F8F,0x7F7F,0x4E4E,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x4E4E,0x4E4E,0x4C4C,0x4C4C,0x7F7F,0x7F7F,0x4E4E,0x8F8F,0x6E6E,0x8F8F,0x8F8F,0x5050,0x5050,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x5050,0x5050,0x4040,0x8F8F,0x8F8F,0x4040,0x8F8F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x4040,0x8F8F
.word 0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x6060,0x6060,0x6060,0x6060,0x5050,0x5050,0x6060,0x6060,0x7070,0x7070,0xC070,0xC0C0,0xC0C0,0xD0D0,0xC1C1,0xB1B1,0x91C0,0xB1C0,0xC1C1,0x92A1,0xB2A1,0x92A1,0x8171,0x7182,0x8F8F,0x8F8F,0x8F8F,0x7070,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8E8E,0x8080,0xBFBF,0x8E70,0x6070,0x8E8E,0x9E70,0x8080,0x7070,0x6060,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x4949,0x8B8B,0x9B9B,0xCA9B,0xDABB,0x699B,0x6B6B,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x7C5C,0x8B7B,0x8B8B,0x4A4A,0x4A4A,0x7C7C,0x8B8B,0xBFBF,0x9C8D,0x6B6B,0x4A4A,0x6C6C,0x8F8F,0x4E4E,0x4E4E,0x7F7F,0x7F7F,0x4C4C,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050,0x9F9F,0x8F8F,0x5050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x408F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x6060,0x6060,0x5060,0x5060,0x6060,0x5060,0x5050,0x6060,0x6060,0xD080,0xD0C0,0xC0C0,0xD0C0,0xB1C1,0xC292,0xB1C1,0x91B2,0x8182,0x9192,0x9291,0x9292,0x8282,0xB2B2,0x81C2,0xB182,0x81A1,0x8F8F,0x8F8F,0x7070,0x7070,0x8E8E,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x6060,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAEAE,0xBFBF,0x7070,0x6060,0x9E70,0x7070,0xAFAF,0x9F9F,0x5050,0x8F8F,0x7070,0x7070,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x484A,0x8F8F,0x4A4A,0x8B9C,0x9B9B,0xDABB,0xBBBB,0x7B8B,0x4E4A,0x5C5C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4C4C,0x7F7F,0x7B7C,0x8B8B,0x8B8B,0x6B6B,0x4A4A,0x7F7F,0x8C7B,0x9C9C,0x8B8B,0x4A7C,0x5C5C,0x8F8F,0x5C5C,0x8F8F,0x4E4E,0x4E4E,0x4E4E,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4C4C,0x4E4E,0x4E4E,0x4E4E,0x4E4E,0x4C4C,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x5050,0x9F9F,0x9F9F,0x6060,0x5050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x5050,0x8F8F,0x4040,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x8F8F,0x4040,0x5050,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F
.word 0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x5050,0x8F8F,0x5050,0x7070,0x5060,0x6060,0x6060,0x6060,0x5050,0x5050,0x7070,0xC0C0,0xD0D0,0xC1D0,0xC0C0,0x9181,0xA192,0xA192,0xC1B1,0xC1C1,0xC0C1,0x9E9E,0xC0B0,0xA1B1,0xB1A1,0x91A1,0x9292,0x8281,0xB192,0xA0A0,0xA080,0xB0B0,0x9F9F,0xAFAF,0xBFBF,0xC0C0,0x7070,0x7070,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7070,0xBFBF,0xBFBF,0xAFAF,0x9E70,0x7070,0x709E,0x6060,0x4040,0x8F8F,0x7070,0x7070,0x9F9F,0x5050,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x4949,0x8F8F,0x4A4A,0x598B,0x8B8B,0x8BCA,0x8C8B,0x4A4A,0x4A4A,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6C6C,0x9C4A,0x6A6A,0x4A4A,0x5C5C,0x8F8F,0x4A4A,0x8F8F,0x4A4A,0x8C8B,0x6E6E,0x7F7F,0x4A4A,0x4C4C,0x5C5C,0x7F7F,0x8F8F,0x4E4E,0x5C5C,0x4C4C,0x4C4C,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F8F
.word 0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x6050,0x6060,0x5050,0x5060,0x6060,0x6060,0x8080,0xC070,0xB0C0,0x91B1,0x9292,0x92B1,0xB1B1,0xBFBF,0x9E9E,0xC0BE,0xB1A1,0xC0C1,0xA1A1,0x9281,0xA1A1,0xC0C0,0xC0C0,0x71B0,0xC0C0,0x8281,0xB1B2,0xA1A1,0xB0B2,0xC0B0,0x9170,0xA1B0,0xAFAF,0x8080,0xAFAF,0x9F9F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x8E8E,0xAFAF,0xBFBF,0x7070,0x7070,0x9E70,0x7070,0x9F9F,0x4040,0x8F8F,0x5050,0x7070,0x6060,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x4847,0x8F8F,0x4A4A,0x8C7E,0x9C7B,0x7A8B,0x6A6A,0x6A6A,0x4A7E,0x5C5C,0x4A4A,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7C4E,0x7E7E,0xAFAF,0x9F9F,0x6C6C,0x4A4A,0x7F7F,0x4A4A,0x4A4A,0x4A4A,0x4A4A,0x5C5C,0x6C6C,0x5C5C,0x8F8F,0x6E6E,0x6E6E,0x5C5C,0x5C5C,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x4E4E,0x4E4E,0x4C4C,0x8F8F,0x4C4C,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x5050,0x6060,0x9F9F,0x8F8F,0x6060,0x9F9F,0x8F8F,0x4040,0x8F8F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x6060,0x6050,0x5050,0x5050,0x5050,0x5050,0xD080,0xC0D0,0xA1B0,0xC1A1,0xC0C0,0xD0C0,0x9E9E,0xA1A0,0xA1A1,0x9191,0x9282,0x9292,0x8192,0x9292,0x9282,0x8292,0xB282,0xA192,0xB1B1,0xC0AE,0x9281,0x82B1,0xB292,0xA1B2,0xB2C0,0xB292,0xA1A1,0xC0C0,0xA0B0,0x90A0,0x5050,0x8F8F,0x7F7F,0x7F7F,0x5050,0x7070,0x7070,0x709E,0x8E70,0xAFAF,0xAFAF,0x6060,0x8F8F,0x7F7F,0x8F8F,0x7070,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4848,0x4848,0x4747,0x6C6C,0x8C8C,0xBBBB,0x5959,0x6A6A,0x4A4A,0x4A4A,0x4A4A,0x8F8F,0x7F7F,0x7F7F,0x4C4C,0x4C4C,0x4C4C,0x7F7F,0x4E4C,0x8D8D,0x9E9E,0x8E9E,0x8E8E,0x7E7E,0x9F9F,0x8F8F,0x7F7F,0x4A4A,0x8F8F,0x8F8F,0x4A4A,0x6C6C,0x6C7C,0x6C6C,0x6E6E,0x7E7E,0x4A4A,0x5C5C,0x4C4C,0x7F7F,0x6F6F,0x4C4C,0x4E4E,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x4848,0x4747,0x4747,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4040,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x5050,0x9F9F,0x9F9F,0x6060,0x5050,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F40
.word 0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x5050,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x7060,0xC0C0,0xB1D0,0xC0B2,0x9EBE,0xB0B0,0xB1B2,0x91A1,0x9291,0x9292,0x9292,0x8281,0x8281,0x8282,0x9282,0x8282,0x9282,0x8292,0x9292,0x9192,0xC1A1,0xC0C0,0x9182,0x8291,0x82B2,0x92B2,0x9292,0xB292,0xB191,0x92B0,0xB2B2,0x80B1,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0x7070,0x7070,0x7070,0x8E8E,0xBFBF,0x5050,0x7F7F,0x7F7F,0x5040,0x5050,0x8F8F,0x8F8F,0x8F8F,0x494A,0x4949,0x4848,0x8F8F,0x8F8F,0xAFAF,0xD6D6,0x9F9F,0x4A4A,0x8F8F,0x4A5C,0x4A4A,0x7F7F,0x4C4C,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4C4A,0x8C8C,0xBCBC,0xCFCF,0xBFBF,0xBFBF,0xAEAE,0x8E8E,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x5C5C,0x7C7C,0x6E6E,0x4A4A,0x7C7C,0x9F9F,0x4E4E,0x6E6E,0x6E6E,0x4E4E,0x4C4C,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4747,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x5050,0x5050,0x9F9F,0x5050,0x6060,0x6060,0x9F9F,0x9F9F,0x6060,0x5050,0x8F8F,0x5050,0x8F8F
.word 0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x5050,0x5050,0x6060,0x8F8F,0x5050,0x6060,0xD0C0,0xB1D0,0xB1A1,0xAEAE,0xB0D0,0x92A0,0x9292,0x92B2,0x9292,0x8292,0x8282,0x8292,0x8282,0x8281,0x8282,0x9182,0x8282,0x9282,0x8282,0x8292,0x9281,0xB292,0xC091,0xC0C0,0x9291,0xB292,0xB2B2,0x8292,0xB2B2,0x8192,0xB292,0xB2B2,0x8091,0x4040,0x7F7F,0x4040,0x8F8F,0x6060,0x7070,0x7070,0x7070,0xBFBF,0x5050,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4A4A,0x8F8F,0x4848,0x8F8F,0x4848,0x7F7F,0x9F9F,0x4747,0x4A4A,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x6F6F,0x7F7F,0x6F6F,0x4A4A,0x7F7F,0x7F7F,0x9C9C,0xDBDB,0xBFBF,0x9E9E,0x9E9E,0x7070,0x9E70,0x4E4E,0x7F7F,0x7F7F,0x4A4A,0x8F8F,0x8F8F,0x4A4A,0x5C5C,0x6C7C,0x6E6E,0x7C7C,0x7C7C,0x7E7E,0x7E7E,0x4E4E,0x4C4C,0x7F7F,0x7F7F,0x8F8F,0x4E4E,0x7F7F,0x8F8F,0x8F8F,0x4747,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x5050,0x8F8F,0x8F8F,0x9F9F,0x5050,0x9F9F,0x7070,0x7070,0x708E,0x708E,0x9F9F,0x9F9F,0x6060,0x8F9F
.word 0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x6060,0xD0E0,0xC1C1,0xE0D2,0xB0BE,0x9281,0x9292,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8281,0x8282,0x8182,0x8282,0x9281,0xB291,0xB181,0xC0C0,0x7271,0x9282,0x9292,0x82B2,0xB2B2,0x8192,0xB281,0xB2B2,0xB0B2,0x8040,0xB091,0x5080,0x8F8F,0x7070,0xAFAF,0x9E9E,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x4848,0x4849,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x4242,0x8F8F,0x8F8F,0x4A4A,0x7F7F,0x7F7F,0x4A4A,0x6F6F,0x7F7F,0x4C4C,0x7F7F,0x7C7C,0x8C8C,0xBC9E,0xCBCB,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x4E4E,0x4A4A,0x6C6C,0x4A6C,0x6E4A,0x8E4A,0x8E8E,0xAFAF,0x6E6E,0x8F8F,0x4C4C,0x4C4C,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4A4A,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4E4E,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x5050,0x5050,0x6060,0x9F9F,0x6060,0x9F9F,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x509F
.word 0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x5050,0x8F8F,0xC060,0xC1C1,0xC2B1,0xCEDE,0x81C0,0x82A1,0x9292,0x8292,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8182,0x8182,0x8282,0x8281,0x8272,0x8281,0x8282,0x8182,0x8282,0x9292,0xB1A1,0xC0B1,0xD0D0,0x8160,0xA1B2,0x8292,0xB2A3,0x82B2,0x8172,0xB2B2,0xB2B2,0x72B2,0xB2B2,0xB0B2,0x6090,0x6060,0x7070,0xC09E,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4A4A,0x4949,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4A4A,0x4A4A,0x7F7F,0x4A4A,0x4C4C,0x4A4A,0x7F7F,0x7F7F,0x6F6F,0x4C4C,0xDA9B,0xE7E7,0xDBDB,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x7070,0x709E,0x709E,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x5C5C,0x4E4E,0x6C6C,0x4A4A,0x4E4E,0x7E7E,0xAFAF,0x6E6E,0x7F7F,0x4C4C,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x6060,0x6060,0x9F9F,0x7070,0x7070,0x708E,0xAFAF,0x6060,0x6060,0xAFAF,0x9F70
.word 0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x6060,0x6060,0xC0C0,0xC1C0,0xD1B1,0xB0CE,0x92A1,0x9292,0x8282,0x9282,0x9282,0x8282,0x8282,0x8282,0x8282,0x8282,0x7282,0x8272,0x7281,0x7282,0x7282,0x8172,0x8281,0x8272,0x8172,0x8282,0xA192,0xB1A1,0xC1B1,0x71C0,0xC0C0,0xB1B1,0x82A1,0xB282,0x8292,0x8271,0xB3B2,0x92B2,0x9292,0xB292,0xC0B1,0x60B0,0x708E,0x9E8E,0x9E9E,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4747,0x4747,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x4A4A,0x4C4C,0x4A4A,0x6F6F,0x7F7F,0x6C6C,0xE5DA,0xE7E7,0xECEC,0xCFCF,0xBFBF,0x7070,0xAFAF,0xAFAF,0x9E70,0x9E70,0x7070,0xAFAF,0x8F8F,0x7F7F,0x8F8F,0x4E4E,0x4A4A,0x4A4A,0x6C6C,0x4A4E,0x7C4A,0x9F9F,0xAFAF,0x8F8F,0x4E4E,0x4E4E,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x6060,0xAFAF,0x7070,0x7070,0x7070,0x8E70,0x7070,0x9F9F,0x6060,0x6060,0x9F9F
.word 0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x6060,0x9F9F,0x6060,0xC060,0x91C1,0xAEA1,0xA1AE,0x9282,0x8292,0x9292,0x8282,0x8282,0x8282,0x7282,0x8282,0x7282,0x7282,0x8282,0x7282,0x8172,0x8182,0x8182,0x8272,0x8172,0x8281,0x7281,0x8182,0xA192,0xA192,0xB1A1,0xA1B1,0x9191,0xC081,0xC0C0,0x7272,0x8272,0x7182,0x9282,0xB2B2,0x9292,0x8282,0xB282,0xB0B1,0x8080,0xAFAF,0xCECE,0x9E9E,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4848,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4A4A,0x7F7F,0x6F6F,0x7F7F,0x4A4A,0x4C4C,0x6C6C,0x9C9C,0xD6D6,0xE7E7,0xCFCF,0xDFDF,0x9E9E,0x9E9E,0xAFAF,0x9E9E,0xC09E,0x9E9E,0x9E9E,0x708E,0x5050,0x9F9F,0x5050,0x8F8F,0x7F7F,0x4A4A,0x6C4A,0x9F9F,0x6E6E,0x8E8E,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4C4C,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x6060,0x7060,0x7070,0x7070,0x7070,0x7070,0x8E70,0x7070,0x9F9F,0x7070,0x8F9F
.word 0x5151,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x9F9F,0x6060,0x6060,0x8080,0x81C2,0xAED0,0x91A0,0x9292,0x9292,0x8292,0x92A3,0x9282,0x8282,0x8282,0x7282,0x8282,0x8282,0x7282,0x7282,0x7282,0x7272,0x7272,0x8182,0x8282,0x8282,0x8282,0x8281,0x9282,0x9292,0xA192,0x9292,0x8291,0x7181,0xC091,0x7070,0x6161,0xB081,0x8171,0x92B2,0xB291,0x9291,0x8282,0xB282,0xB0B1,0x9E80,0xBFBF,0xC0C0,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4C4C,0x4A4C,0x6C6C,0x4A4A,0x7C5C,0x9C8C,0x9E9E,0xBFBF,0xF0F0,0xDFDF,0xCEDE,0xBFBF,0xAEAE,0xBEBE,0xD0D0,0xBFBF,0xAEAE,0x9E9E,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x5C5C,0x4E4E,0x8F8F,0x6E4A,0x8E8E,0x9F9F,0x8F8F,0x8F8F,0x4E4E,0x4E6E,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x4848,0x4848,0x4747,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x5050,0x5050,0x8F8F,0x6060,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x708E,0x7070,0x7070,0x9F9F,0x8F60
.word 0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x6060,0x5050,0x6050,0xB0B0,0xA191,0x9EBE,0x92A0,0x9292,0x9292,0x8292,0xA392,0x8282,0x8282,0xA382,0xA382,0xA382,0xA382,0x7282,0x7282,0x7282,0x7281,0x8281,0x7281,0x8182,0x7282,0x8281,0x8282,0x8281,0xA182,0x9292,0xA182,0x8282,0x8281,0x8281,0x8181,0x71B0,0xC071,0x8080,0xAFAF,0xB191,0xA1B1,0x9291,0x8281,0xB2B2,0xC0C0,0x9E9E,0xB1B0,0xB1B1,0x507E,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4747,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x4A4A,0x4A4A,0x7C4A,0x4A4A,0x6C4E,0x8C7C,0x7B7B,0x7B7B,0x9E9E,0xCFCF,0xFEFE,0xF0F0,0xEEFE,0xE0E0,0xDFDF,0xCFCF,0xCFCF,0xCECE,0xDEDE,0xBEBE,0xAEAE,0x9E9E,0xAFAF,0x8F8F,0x5C5C,0x4C4A,0x7F7F,0xAFAF,0x9F9F,0x8F8F,0x4E4E,0x6E6E,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4A4A,0x7F7F,0x8F8F,0x4A4A,0x7F7F,0x8F8F,0x4A4A,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x4040,0x5050,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x6060,0x6070,0x9F9F,0x7070,0x7070,0x6060,0x6060,0x7070,0x7070,0x6060,0x609F
.word 0x6161,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x5050,0x5050,0x8080,0xD081,0xB09E,0x9281,0x8292,0xA392,0x8292,0x9282,0xA382,0xA3A3,0x8282,0x82A3,0x93A3,0x8282,0xA382,0x7282,0x8272,0x7272,0x7272,0x8282,0x8282,0x8282,0x8282,0x9292,0x8182,0x9282,0x9192,0x9292,0x8292,0x7282,0x7272,0x7281,0x7171,0x8171,0xC0A1,0xC0C0,0x9F9F,0xC1C0,0xB1C1,0x81A1,0x9282,0xA1B2,0xA1A1,0x9191,0xA1B1,0x8E70,0x7070,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x5C5C,0x6B6B,0x7C7C,0x7C7C,0x8C7C,0x6B7B,0x8C8C,0xDE9E,0xCFCF,0xDFDF,0xEFEF,0xF0F0,0xFEFE,0xF0F0,0xD0D0,0xBFBF,0xCFCF,0xF0F0,0xF0F0,0xDFDF,0xCFCF,0x9E9E,0x7E8E,0x6C6C,0x8F8F,0x5C5C,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x5C5C,0x4C4C,0x7F7F,0x6F6F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4848,0x7F7F,0x4A4A,0x7F7F,0x7F7F,0x4A4A,0x4A4A,0x7F7F,0x7F7F,0x4040,0x5050,0x9F9F,0x5050,0x9F9F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x6060,0x6060,0x6060,0x8F9F
.word 0x5151,0x4E4E,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x4040,0x7F7F,0x8F8F,0x5050,0x5050,0x5050,0xB080,0xD0B0,0xB09E,0x9292,0x9292,0x9292,0x92A3,0x8282,0x8282,0x8282,0x8293,0x8293,0x8282,0x9393,0x8293,0x8293,0x7293,0x7272,0x8272,0x8281,0x8282,0x9292,0x8292,0x8282,0x9282,0x8192,0x9282,0x9192,0x9281,0x8281,0x7271,0x8272,0x82B3,0x8282,0x8281,0xC0C0,0x7070,0xAFAF,0xD0D0,0xC0D0,0x7171,0xA192,0xA1A1,0x9F9F,0xB1B2,0xA0A1,0x6060,0x4050,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4A4A,0x8C7C,0x8B8B,0x8C8C,0x8C4A,0xBEBE,0xDFDF,0xDFDF,0xEFEF,0xEFEF,0xF0F0,0xDFDF,0xCFCF,0xBFBF,0xF0F0,0xEFEF,0xFFFF,0xF0F0,0xDFDF,0xBFBF,0x9E9E,0x7C7E,0x6C6C,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4A4A,0x5C5C,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4646,0x7F7F,0x7F7F,0x6F6F,0x4747,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x9F9F,0x5050,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x6060,0x9F9F,0x9F9F,0xAFAF,0x8E70,0x7070,0x6060,0x6060,0x6060,0x7070,0x7070,0x7070,0x9F9F,0x6060,0x7070,0x6060,0x509F
.word 0x7070,0x7E7E,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x6060,0x7171,0xBEA1,0x819E,0x9292,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0xA382,0x9382,0x9372,0x8272,0x7293,0x7282,0x7282,0x7282,0x7172,0x8272,0x9292,0x92A1,0x9292,0x9192,0x9282,0x9292,0x8281,0xB281,0x8181,0x8282,0x7282,0x7271,0x8272,0x8282,0x8172,0x9181,0xD0C0,0x70C0,0xC0C0,0xE0E0,0xD0D0,0xA1A1,0xA1A1,0xA1A1,0xB270,0x91B2,0x50B0,0x8F8F,0x4141,0x7F7F,0x7F7F,0x4747,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7D7D,0xAC8C,0x8B9B,0x7B8B,0x8C8C,0xBB8B,0xCFCF,0xDFDF,0xF0F0,0xE0F0,0xF0F0,0xF0F0,0xBFBF,0xBFBF,0xF0E0,0xF0F0,0xF0F0,0xEFEF,0xDFDF,0xCFCF,0xCEBE,0x7C7C,0x5C5C,0x8F8F,0x9F9F,0x8F8F,0x4E4E,0x4E4E,0x4E4E,0x7C7C,0x4E4E,0x4C4C,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7070,0x5050,0x9F9F,0x6060,0x9F9F,0x8F8F,0x7070,0xAFAF,0x8E70,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x8E60,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0x8F60
.word 0xAFAF,0x708E,0x6060,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x7F7F,0x6E6E,0x8F8F,0x6E6E,0x8F8F,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x5050,0x8F8F,0x5050,0x7050,0xC1D0,0xB2B2,0xA09E,0x8292,0x9292,0x8282,0x9382,0x8282,0x8282,0x8282,0x8282,0x8293,0x7293,0x7293,0x9393,0x9372,0x7272,0x7272,0x7272,0x8272,0x9282,0xA192,0x92A1,0x9292,0x9192,0x9292,0xA192,0xC381,0xC281,0x8181,0x8282,0x7272,0x7272,0x7272,0x7282,0x8182,0xA181,0xB1C0,0x90B0,0xBFBF,0xBEBE,0xD0D0,0xB1B0,0xA1B1,0x7081,0xB2B2,0x81B2,0x5050,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4C4C,0x8C6C,0xABAB,0x9BAC,0x9C8B,0x8B8B,0x7B7B,0xBEBE,0xE0E0,0xD0D0,0xD0D0,0xDEDE,0xCFCF,0xBFBF,0xBFBF,0xE0E0,0xF0F0,0xF0F0,0xF0F0,0xDFDF,0xBFBF,0xBEBE,0x7EAE,0x8F8F,0x7E7E,0x8E8E,0x8E7E,0x4E7E,0x7C6C,0x7C7C,0x8C8C,0x6E4A,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4242,0x4747,0x4A4A,0x7F7F,0x4A4A,0x7F7F,0x4A4A,0x4A4A,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x6060,0x9F9F,0x7070,0x8F8F,0x9F9F,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0x6060,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x6060,0x9FAF
.word 0x9F9F,0xBFBF,0x6080,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4C4C,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x5050,0x5050,0x8F8F,0xB171,0x9EA1,0xA0A0,0x9292,0x9282,0x8292,0x9282,0x82A3,0xA382,0xA382,0x93A3,0x8272,0xA382,0x9382,0x9372,0x7272,0x7293,0x7293,0x7272,0x7272,0x8182,0x9292,0xB1B1,0xB2B1,0xB292,0xA192,0xC282,0xC081,0x7172,0x81D0,0x8281,0x7281,0x7271,0x7272,0x82A3,0x8272,0x7282,0xA191,0xA1B2,0x80A0,0xAEAE,0xD0AE,0xC0C0,0xB1C1,0x61B2,0xB261,0xB2B3,0x51B2,0x8161,0x5151,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x5C5C,0x9C6B,0xBBCB,0x9BAB,0x9B9B,0x8B9B,0x9E9C,0xCFCF,0xC0C0,0x9F9F,0x9EC0,0xBFBF,0xBFBF,0x8080,0xAFAF,0xCFCF,0xD0E0,0xD0E0,0xD0D0,0xCFCF,0xAFAF,0xAFAF,0xAEAE,0x8E9E,0x9E9E,0xCFCF,0x9E9E,0x8C8E,0x8C7E,0x8C8C,0x8C8C,0x6B6B,0x4C4C,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x4A4A,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050,0x9F9F,0x8E60,0xAFAF,0x9F9F,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0x708E,0x7070,0x7070,0x9E9E,0x7070,0x7070,0x6060,0x6060,0x6060,0x7070,0xAFAF
.word 0x9E9E,0x8080,0x7171,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x6060,0x9271,0x9EB0,0x819E,0x8282,0x8292,0x8282,0x8282,0x8282,0xA382,0xA382,0x8282,0x93A3,0x9372,0x9393,0x7293,0x8393,0x9372,0x7272,0x7272,0x7272,0x8282,0x81B2,0xC3B1,0xC3C0,0xC2C0,0xB1C2,0xB381,0xC3C0,0x71D0,0x72C3,0x81C0,0x9282,0x7172,0x7272,0x9372,0x7293,0x7272,0x8272,0x9181,0xC0B1,0x80C0,0x9EC0,0xD0D0,0xC1D0,0xA1C1,0x7161,0xB2B2,0x92B2,0x9271,0x9191,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4B4A,0xAB8C,0xCBBB,0x9BBB,0x8A9B,0x9C9C,0xAEAE,0x9E9E,0x9E9E,0xBFBF,0xC09E,0xBFBF,0x8080,0x708E,0x7070,0xBFBF,0x8080,0x6060,0x7070,0xBFBF,0xAFAF,0xBEBE,0xCFCF,0xCECE,0xBEBE,0x9EAE,0x8D8D,0x7C7C,0x7C8C,0x8C8C,0x8C7C,0x7C7C,0x6C6C,0x4E4E,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4747,0x7F7F,0x7F7F,0x8F8F,0x6E6E,0x9F9F,0x8F8F,0x5050,0x8F8F,0x5050,0xAFAF,0xBFBF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x8E70,0x7060,0x708E,0x9E70,0x7070,0x6060,0x6060,0x6060,0x6060,0x9F9F,0xBF70
.word 0x7070,0x8080,0x7070,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4E4E,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x4848,0x4848,0x4949,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x6060,0x8F8F,0x5050,0x8F8F,0x7F7F,0x5050,0x7171,0xC081,0xA08E,0x9292,0x8282,0x8282,0x8282,0x8272,0x8282,0x9382,0x9372,0x8293,0x9393,0x7272,0x7272,0x7283,0x9372,0x8372,0x7293,0x7172,0x7272,0xC0B2,0xC0C3,0xC0C3,0xC3C0,0xC2C0,0xC0C3,0xC381,0xD371,0xD480,0xC371,0x8281,0x8282,0x6272,0x7262,0x9372,0x7272,0x7272,0x8272,0x8191,0xC0B0,0xC0C0,0xC0C0,0xC0D0,0xC1D1,0x61B1,0x9292,0xB2B3,0x7192,0x8292,0x4141,0x7F7F,0x4242,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7E5C,0xAB9C,0xCABB,0x9B9B,0x7B7B,0x7B7B,0x9E9E,0xBFBF,0xAFAF,0xBFBF,0xCFCF,0xAFAF,0x7070,0x7070,0x9E9E,0x709E,0x6060,0x7070,0xBEBE,0xBFBF,0xAFAF,0xBFBF,0xEEEE,0xBEEE,0xAEAE,0x9E9E,0x6C7E,0x4A4A,0x8C7C,0x7C7C,0x4A4A,0x4A7C,0x4A7C,0x5C5C,0x5C5C,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4747,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x9F9F,0x5050,0x9F9F,0x6060,0x9F9F,0xBFBF,0x7070,0x7070,0x708E,0x7070,0x9E9E,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x7070,0x6060,0x7070,0x70AF
.word 0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x5050,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x4848,0x6262,0xC5C5,0x9F9F,0x4949,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x5050,0x8F8F,0x7F7F,0x4040,0x6060,0xC0C3,0x8E81,0xA08E,0x8282,0x8281,0x8282,0x8282,0x8282,0x7272,0x8293,0x93A3,0x9372,0x7293,0x7283,0x7283,0x8372,0x7283,0x7283,0x7272,0x7272,0x7272,0xB282,0xC0C1,0xD0C3,0xC0D4,0x71D0,0xC3D0,0xC0D0,0xC0C3,0x8080,0x8181,0x71C0,0x8282,0x6272,0x7262,0x7262,0x9372,0x7272,0x7271,0x9192,0xC081,0xC0B0,0xC0C0,0xBFBF,0xD0D0,0xB1D1,0x9E70,0x9281,0x92B2,0xB282,0x4172,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5C5C,0x8D8D,0xCCCC,0x8B9C,0x6A6A,0x7B6A,0x8C8C,0xAEAE,0xBFBF,0xAEAE,0xCFCF,0xBFBF,0x7070,0x7070,0x9E9E,0x8E8E,0xAFAF,0x7070,0xAFAF,0xD0AE,0xD0D0,0x7070,0xAFAF,0xCECE,0xDEDE,0xBECD,0xCECE,0x9E9E,0x6E6C,0x7C7C,0x4A4A,0x7C4B,0x8C8C,0x7C7E,0x8F8F,0x8F8F,0x4A4A,0x7F7F,0x7F7F,0x4A4A,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5050,0x8F8F,0x7070,0xC0C0,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x7060,0x7070,0x6070,0x708E,0x7070,0x6070,0x7070,0x7070,0x7070,0xAF70
.word 0xAFAF,0x8080,0xAFAF,0x9E9E,0x7070,0x6060,0x9F9F,0x9F9F,0x7E7E,0xAFAF,0x9F9F,0xAFAF,0x5757,0x4747,0x4848,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7171,0xC0B1,0x908E,0x7271,0x8181,0x8282,0x8282,0x8281,0x8282,0x7272,0xA372,0x7293,0x7272,0x6272,0x6262,0x7262,0x7262,0x8372,0x8372,0x7272,0x7272,0x8172,0xD0C3,0x71D0,0xD0D0,0xD0D0,0xD0C0,0xC0D4,0xD071,0xC0C3,0xD4C0,0xD371,0x72C0,0x7182,0x6262,0x8372,0x7272,0x7272,0x7172,0xB272,0xC3C0,0xC0C0,0xC0B0,0xD0C0,0xD0D0,0xE0D0,0xC0C1,0x817E,0xB282,0x9292,0xB3B2,0x4242,0x7F7F,0x4242,0x7F7F,0x4242,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8D8D,0xCFCF,0xBFBF,0x4A4A,0x6B7C,0xBC8D,0xAFAF,0xAFAF,0xCFCF,0xCFCF,0xBFBF,0x5050,0x8F8F,0xAFAF,0xBFBF,0x9E9E,0xAFAF,0x7070,0xAFAF,0xBFBF,0x7070,0x7070,0xCEAE,0xDFDF,0xFEFC,0xEEEE,0x9E9E,0x7B9C,0x7C7C,0x7C8C,0x6B6B,0x8B8C,0x4A8C,0x4E4E,0x5C5C,0x8F8F,0x4A4A,0x4A4A,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x5050,0x8E8E,0x6060,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x6060,0x6060,0x7070,0x7070,0x6060,0x7060,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x7060,0x7070,0xAF70
.word 0x8E80,0x9E9E,0x9E9E,0xAFAF,0x9E9E,0xAFAF,0x9F9F,0x9F9F,0x6060,0x9F9F,0x5757,0x5757,0x4747,0x9F9F,0x4747,0x9F9F,0x4848,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6060,0xAFAF,0xC081,0x808E,0x8181,0x8282,0x8281,0x8281,0x7282,0x8272,0x8282,0x7293,0x7293,0x6272,0x7262,0x6172,0x6262,0x6262,0x7262,0x7262,0x7283,0x7272,0x8272,0x71C0,0xD0D3,0x7171,0xC3C0,0xD0D0,0xC0C0,0xD0D0,0xC3D0,0xC0C0,0x7171,0xC271,0x6281,0x6272,0x7283,0x8383,0x8372,0x6272,0x6261,0xC071,0x7171,0xC0C0,0xC0C0,0xC0C0,0xD0D0,0xCFCF,0x7070,0x9291,0x92B2,0xB282,0x42B2,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9E9E,0xAFAF,0x8F8F,0x8B8C,0xCCCC,0xBBBB,0xAFAF,0xAEAE,0xD0D0,0x7070,0x6060,0x5050,0xAEAE,0xCFCF,0x9E80,0x7070,0x7070,0x8080,0x7070,0x7070,0x7070,0xBFBF,0xECEC,0xFDFD,0xFEFE,0xDCEE,0xCCCC,0x8C9C,0x4A8C,0x6B6B,0x9C9C,0x9C9C,0x4A4A,0x8F8F,0x6E6E,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7070,0x9E9E,0x708E,0xAFAF,0x9E9E,0x8080,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x7070,0x7060,0x7070,0x6060,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x509F
.word 0x8080,0x8080,0xC0C0,0xC0C0,0xC0C0,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4747,0x9F9F,0x5757,0x9F9F,0x4848,0x9F9F,0x4848,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7171,0x9E81,0xB08E,0x8272,0x8181,0x8182,0x7282,0x8182,0x7282,0x7293,0x8282,0x7272,0x8181,0x9F9F,0x8F8F,0x6161,0x6261,0x6262,0x6262,0x6272,0x7272,0x7272,0xD392,0xC0C0,0xD0D0,0xD0D0,0xD0D0,0xD4C0,0xD3C0,0xC0C0,0xC371,0x71C0,0x71D4,0x7281,0x6272,0x8362,0x8362,0x8362,0x7262,0x6262,0xD3C0,0xC081,0x7171,0xC0C0,0xC080,0xD0D0,0xD0D0,0xC0C0,0xC0C0,0x9292,0x92B2,0xB282,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7C7C,0xACAC,0xBFBF,0xAFAF,0xAFAF,0xBEBE,0x6060,0x9F9F,0x8E8E,0xBFBF,0xD0D0,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0xE0E0,0xDFDF,0xDFDF,0xDFDF,0xDBFB,0xACAC,0x8C8C,0xAC8D,0xBBBB,0x6A6A,0x9F9F,0xAFAF,0x4A4A,0x7E7E,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x4040,0xAFAF,0x7070,0x8070,0x9E9E,0x8070,0xC0C0,0x6060,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x8F8F,0x7070,0x7070,0x6060,0x6060,0x7070,0x7060,0x7070,0x7070,0x6060,0x7070,0x6060,0x6060,0x8F8F
.word 0x8080,0xC0C0,0x8080,0xC0C0,0xAFAF,0x809E,0x7070,0x6060,0x8F8F,0x4747,0x9F9F,0x9F9F,0x4848,0x9F9F,0x4848,0x9F9F,0x4848,0x8F8F,0x4A4A,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6060,0x7171,0x8181,0xA08E,0x8181,0x8182,0x7281,0x7281,0x7271,0x8281,0x9382,0x7272,0x7172,0x509E,0x5050,0x7F7F,0x7F7F,0x7070,0x6261,0x7262,0x7262,0x7272,0x7272,0xC182,0xD0D0,0xD0D0,0xC0C0,0xC0C0,0xC0C0,0xC0C0,0xC0D4,0x71D0,0xD3D3,0xD480,0xB371,0x6271,0x6262,0x6262,0x6262,0x7262,0x7262,0x7183,0x71D0,0xC0C3,0xC0C0,0xAFAF,0xC0C0,0xD0D0,0xBFBF,0xAFAF,0x82A1,0xA392,0x9292,0x7171,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x6E6E,0x6B6B,0x9F9F,0x4949,0x9F9F,0x9C9C,0xAFAF,0x9F9F,0x6060,0x9F9F,0xBFBF,0x7070,0x7070,0x9F9F,0xAFAF,0x7070,0xBFBF,0x7070,0x4040,0x6161,0xC3C3,0xE0D3,0xDFDF,0xFBFB,0xFBFA,0xDCFB,0x9E9C,0xBCBC,0x5858,0x4848,0x4949,0x4949,0xAFAF,0x8B8B,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0x4A9C,0x8F8F,0x8F8F,0x7070,0x7070,0x8080,0x7070,0xAFAF,0x8080,0x6070,0x7070,0x7070,0x7070,0x7070,0x6070,0x5050,0x6060,0x7070,0x6060,0x7060,0x7070,0x7070,0x6070,0x7070,0x8E70,0x6070,0x6060,0x6060,0x609F
.word 0x8071,0xC070,0xB0B0,0xAFAF,0x8080,0x9E80,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x4747,0x4848,0x9F9F,0x4848,0x4848,0x9F9F,0x8F8F,0x4848,0x4A4A,0x4849,0x4848,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6161,0xB081,0xA07E,0x8172,0x7272,0x8281,0x9281,0x7281,0x7272,0x8272,0x7282,0xAEC0,0x6060,0x6F6F,0x6F6F,0x6F6F,0x7E7E,0x7070,0x6261,0x6272,0x6272,0x7272,0x8172,0xC0D4,0xC0C0,0xC0C0,0xD3D3,0xC0C0,0x7171,0x71C0,0xC0E4,0xD071,0xD0D0,0x7282,0x6272,0x6262,0x6262,0x6262,0x6273,0x6262,0xD462,0xD3C0,0xD470,0xC0C0,0xC0C0,0x7070,0xD0C0,0xD0D0,0xC0C0,0xB180,0x92B1,0x8282,0x8292,0x4282,0x4141,0x4040,0x4040,0x4040,0x8F8F,0x4141,0x8F8F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x9F9F,0x4A4A,0x8E8E,0x9F9F,0x6060,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x6060,0x7060,0xAFAF,0xD0D0,0xCFCF,0xBFBF,0x8F8F,0x5252,0x6262,0x7171,0xD0D3,0xE4E4,0xF4F4,0xCFCF,0xBFBF,0xBEBE,0x4949,0x4848,0x4848,0x4848,0x4949,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x8F8F,0x9F9F,0xE9D9,0xEAF9,0xCBDA,0xBFBF,0x7070,0x8080,0xD0D0,0xBEBE,0x6070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7060,0x5050,0x5050,0x6060,0x6060,0x7070,0x7060,0x7070,0x6070,0x7060,0x7070,0x7070,0x9F9F,0x5050,0x6060,0x9F60
.word 0x7171,0x7070,0xAFAF,0x8080,0xAFAF,0x7070,0x8080,0xAFAF,0x5252,0x4A4A,0x8F8F,0x8F8F,0x4848,0x9F9F,0x9F9F,0x4848,0x4A4A,0x8F8F,0x4A4A,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x6060,0x7171,0x8EB0,0x719E,0x8181,0x8181,0x8172,0x8182,0x8192,0x8281,0x7282,0x8172,0x709E,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x9E9E,0x5261,0x7262,0x7272,0x7272,0x8282,0xD0C0,0xC0C0,0xC0C0,0xD071,0xC0C0,0x8181,0xD0D0,0x7171,0xC3D0,0xC071,0x72C3,0x7172,0x6262,0x6262,0x5262,0x6262,0x6262,0x6162,0xC0C4,0xB0C2,0xC3D3,0xC0C0,0xC0C0,0xC0C0,0xD0D0,0xC0C0,0xC0C0,0x82A1,0x9292,0x8282,0x71B2,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x4141,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8E70,0xBFBF,0x8080,0x7070,0x6060,0x9F9F,0x7070,0xE0D0,0xD0E0,0x70D0,0x8F8F,0x6363,0x6262,0x9F9F,0x8F8F,0x7161,0xE0E0,0xD3F3,0xD0D0,0xCCCC,0xAFAF,0x4848,0x4949,0x4848,0x4848,0x9F9F,0x6060,0x6060,0x9F9F,0x6060,0x6060,0xD9CB,0xE7D7,0xF7E7,0xF8F8,0x9EDE,0x7070,0xC070,0xBFBF,0x8EC0,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x6070,0x6060,0x7070,0x6050,0x6060,0x6060,0x7070,0x6060,0x6070,0x7060,0x7070,0x7060,0x5050,0x4040,0x5050,0x6060,0x409F
.word 0x7171,0x7171,0x7070,0xAFAF,0x6060,0x6060,0x9F9F,0xAFAF,0x4848,0x8F8F,0x4A4A,0x4A4A,0x9F9F,0x4848,0x4848,0x9F9F,0x9F9F,0x4A4A,0x4949,0x9F9F,0x4848,0x4141,0x8F8F,0x7F7F,0x7F7F,0xD0D0,0xA1A1,0xA09E,0x8281,0x8282,0x8181,0x8182,0x8182,0x7272,0x8272,0xB082,0x51AE,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7171,0x6262,0x9362,0x7272,0x8272,0xC3C3,0xC0D3,0xC0D3,0xD0D0,0xD0D4,0xD0C0,0xC0C3,0xC3C3,0x8181,0xC3D0,0x81C3,0x6272,0x5262,0x5262,0x6263,0x7362,0x6262,0x7262,0xB371,0xC381,0xC0C0,0xC0C3,0xC0C0,0xD0C0,0xC0C0,0xC0C0,0xD0D0,0x8192,0x8272,0x9282,0xB2B3,0x7272,0x4040,0x4040,0x8F8F,0x8F8F,0x4141,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x6060,0x708E,0x7070,0x7070,0xAFAF,0x7070,0x709E,0x9E9E,0xD0D0,0xD0D3,0xBFBF,0x5252,0x9F9F,0x5252,0x5252,0x5151,0xE0E0,0xF0F3,0xE0E0,0xC0C0,0xBFBF,0x4848,0x4949,0x4949,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x7070,0xAFAF,0xD7D7,0xEAEA,0xE8E7,0xE8F8,0xCFCF,0x7070,0x8080,0xC0C0,0x707E,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x6060,0x6060,0x5050,0x5050,0x6070,0x6060,0x6060,0x6060,0x6060,0x7060,0x6070,0x7070,0x6060,0x5050,0x5050,0x5050,0x4050
.word 0xD0D0,0x5171,0x9F9F,0x5151,0x9F9F,0x9F9F,0x5151,0x6161,0x5252,0x7F7F,0x7F7F,0x4A4A,0x4A4A,0x4848,0x9F9F,0x4848,0x4848,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0x7171,0x9F9F,0x8F8F,0x8080,0xC3D0,0x9191,0xA08E,0x8181,0x8181,0x8282,0x9281,0x82A1,0x8282,0x8282,0x7171,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x707E,0x6261,0x6272,0x9372,0x7272,0xD092,0xC0C0,0xD4C0,0xC0D0,0x72C0,0xD0D4,0xD472,0xD0C0,0xC0C3,0xC0C3,0xD071,0x7272,0x6262,0x6252,0x6252,0x5252,0x6252,0x6262,0xC372,0xC371,0xC081,0xD0D0,0xC0C0,0xBFBF,0xC0C0,0xC0C0,0xC0C0,0xB2D0,0x7272,0x9282,0xB282,0x6192,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x5050,0x9F9F,0x7070,0x8E70,0x7070,0x7070,0xAFAF,0x9E9E,0xBEBE,0xD0D0,0x62D0,0x6262,0x5252,0x5252,0x9F9F,0x5252,0x8F8F,0xF0D0,0xF0F0,0xC0E0,0x6060,0x9F9F,0x5958,0x5858,0x4949,0x4848,0x9F9F,0x9F9F,0xBFBF,0x9E9E,0xCFCF,0xAFAF,0xC6C6,0xD7DA,0xE7E9,0xE8E9,0xE9E8,0xBFBF,0x6060,0x7070,0xAFAF,0x8080,0x9E9E,0x7070,0x7070,0x6070,0x7070,0x6060,0x6050,0x5050,0x5050,0x6050,0x5060,0x5060,0x5050,0x7050,0x7070,0x6070,0x7070,0x6060,0x6060,0x9F9F,0x6060,0x609F
.word 0xD0D0,0xBFBF,0x6161,0x4242,0x5252,0x5252,0x4242,0x8F8F,0x8F8F,0x4A4A,0x4A4A,0x7F7F,0x8F8F,0x4A4A,0x4949,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xC0C0,0x7070,0xAFAF,0x6060,0xC0C0,0x9EB1,0xA09E,0x8292,0x8182,0x8281,0x9281,0x8192,0x8281,0x7282,0xC072,0x50AE,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7E7E,0x6261,0x9362,0x7272,0x8293,0xD382,0xD3C0,0x71D0,0xD4D3,0xD0D0,0xD471,0xC3C0,0xD372,0xC0C3,0xC0D4,0xC3D4,0x72C0,0x6272,0x6262,0x5252,0x6262,0x5252,0x5262,0xC372,0xB371,0xD472,0x71C0,0xC3C3,0xC0C0,0xBFBF,0xD0D0,0x80C0,0xC0C0,0x72B2,0x9362,0xA382,0x82B2,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6060,0x6060,0x9F9F,0x4040,0x8F8F,0x5050,0x6060,0x7070,0x7070,0x9E9E,0xAFAF,0xBEBE,0xCECE,0xCFCF,0xC4C4,0x6262,0x4242,0x8F8F,0x9F9F,0x9F9F,0x5151,0xD0D0,0xD0E0,0x7070,0x8F8F,0x4848,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0xD6D6,0xDBDB,0xCECE,0xCFCF,0xDFDF,0x7070,0x9F9F,0xD669,0xD7DA,0xE9D7,0x59DA,0xAFAF,0x8F8F,0x7070,0x8080,0x8070,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x5060,0x5050,0x6060,0x6060,0x6060,0x5050,0x6060,0x7060,0x7070,0x6060,0x6060,0x6060,0x6070,0x9F9F,0x9F70
.word 0xBFBF,0xBFBF,0x5261,0x4242,0x4242,0x5151,0x4747,0x4742,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x8F8F,0x8F8F,0x9F9F,0x4A4A,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xD0D0,0xC0C0,0x7171,0xC3C3,0x9EB0,0x819E,0x9281,0x8291,0xA192,0xA1A1,0x7281,0x7281,0x9282,0xA071,0x51AE,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8E8E,0x6161,0x7262,0x9393,0x7272,0xD093,0x8181,0xD0C3,0xC0C0,0xE4C3,0xC3D0,0xD0C3,0x71C2,0xC0C3,0x72C3,0xC3C0,0xC372,0x6272,0x6252,0x6262,0x6263,0x6262,0x6252,0x7161,0x71D3,0x7171,0xD0D4,0xC071,0xC0C0,0xC0C0,0xC0C3,0xD0D0,0xD071,0x82D0,0x6261,0x9393,0xB2A3,0x7272,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x5050,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xDEDE,0xCEDE,0xDBBE,0xBFBF,0x5251,0x4141,0x7F7F,0x8F8F,0x8F8F,0x4141,0x8080,0xD0D0,0x7070,0x4141,0x8F8F,0xAFAF,0xCFCF,0xCFCF,0xAFAF,0xBFBF,0xD7D7,0xF6F6,0xD0CE,0xD0D0,0xBFBF,0xAFAF,0x4949,0x4949,0x5959,0x59DA,0x4949,0x4A4A,0x6060,0x9F9F,0x7070,0x7070,0x9F9F,0x7070,0x6070,0x5050,0x7060,0x6060,0x6060,0x5050,0x6060,0x5050,0x5060,0x6060,0x5050,0x6050,0x7060,0x7070,0x7060,0x7070,0x7070,0x6060,0x7070,0x509F
.word 0xAFAF,0xC0C0,0x6262,0x4242,0x4242,0x4242,0x9F9F,0x4747,0x4949,0x8F8F,0x8F8F,0x4A4A,0x7F7F,0x4A4A,0x4A4A,0x4A4A,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xC0C0,0xBFBF,0xAFAF,0x7162,0x81D3,0x9EC0,0xA0AE,0x9192,0x9281,0x9291,0xB1A1,0x91A1,0x8282,0x8282,0x9182,0xAEAE,0x6F6F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5271,0x8372,0x9372,0x7272,0xD482,0xC2D0,0xC381,0xC3C3,0xC0C3,0xC0C3,0xC3C2,0xD3C3,0xC3C0,0xC0D0,0xC0D4,0x81C3,0x72B2,0x6252,0x5252,0x5262,0x6262,0x5252,0xC362,0x71C3,0x71D3,0x71C0,0xC3C3,0xC0C0,0xBFBF,0xD0D0,0xC3D4,0xD0D0,0xC1D3,0x6262,0x9372,0xA372,0x5182,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x7070,0xAFAF,0x7070,0xCE9E,0xDEDE,0xDEDE,0xEADB,0x5252,0x4242,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xC0C0,0x4060,0x8F8F,0x9F9F,0xCFCF,0xDFDF,0xDEDE,0xCFCF,0xFBFB,0xDFDF,0xDEDE,0xD0D0,0xAFAF,0x9E9E,0x7070,0x6060,0x4949,0x4949,0x9F9F,0x4949,0x8F8F,0x8F8F,0x6060,0x6060,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x9F9F,0x5050,0x5050,0x6060,0x5050,0x5050,0x6060,0x5060,0x6060,0x6060,0x7070,0x6060,0x7070,0x6060,0x6060,0x6060,0x5050
.word 0x5151,0x8F8F,0x5151,0x8F8F,0x8F8F,0x9F9F,0x4742,0x9F9F,0x9F9F,0x9F9F,0x4848,0x8F8F,0x8F8F,0x4A4A,0x8F8F,0x9F9F,0xBFBF,0xAFAF,0xAFAF,0xC5C5,0xAFAF,0xAFAF,0xC0C0,0xD4D4,0x6262,0x8181,0x9EB1,0xA09E,0x9291,0x9192,0xA1B2,0xA1B1,0xA1B1,0x8291,0x9382,0x8172,0xBFBF,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x5170,0x7283,0x7293,0x7272,0xC082,0xB1D4,0xC0C2,0x81C3,0xD0C3,0xD3C3,0xC0C3,0xC0C3,0x71C3,0xC3D3,0xC371,0xC3C0,0xA3C0,0x5262,0x6262,0x6252,0x5263,0x6262,0x6252,0xD371,0xB382,0xD0D0,0xC3C0,0x7171,0x81C3,0x71D0,0xC0D0,0xC371,0xD0D0,0x6271,0x7262,0xA3A3,0x8282,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x9F9F,0x9E9E,0x9E9E,0xAE9E,0x9E9E,0xBFBF,0xDFDF,0xDEEE,0xCFCF,0x4242,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x5050,0x8F8F,0x7F7F,0x9F9F,0xF0F0,0xF0F0,0xDFDF,0xF6F6,0xF5FA,0xF7F7,0xCFCF,0xDEDE,0xBFBF,0x709E,0x7070,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4848,0x4848,0x6060,0x8E8E,0x6060,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x5060,0x5050,0x5050,0x5050,0x6050,0x5050,0x5050,0x5050,0x6060,0x6060,0x7070,0x7070,0x6070,0x6060,0x6060,0x609F
.word 0x6161,0x8F8F,0x9F9F,0x9F9F,0x4242,0x4242,0x9F9F,0x4A42,0x9F9F,0x4A4A,0x8F8F,0x4A4A,0x7F7F,0x8F8F,0x4848,0x9F9F,0xAFAF,0x7171,0xAFAF,0xC5C5,0x5757,0xAFAF,0xC5C5,0x6161,0x62C4,0xC3C3,0x9EB1,0xA09E,0x8192,0x8292,0x9292,0xC1B1,0xB1B1,0x8291,0x8282,0x7182,0xAEB0,0x9F9F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x6261,0x8362,0x8393,0x7272,0xC393,0x82C0,0xC3B3,0xC1C2,0xC3C3,0xC3C0,0xC3D0,0xB372,0xC3C3,0xD071,0x71C3,0x71C3,0xB2C3,0x6262,0x6262,0x7362,0x6262,0x5252,0x6262,0x8181,0x72C3,0xC481,0xC071,0xD0D0,0xC0C0,0xC3D4,0x7171,0x71D0,0xD3C0,0x6271,0x8362,0xA393,0x82A3,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x9F9F,0x7070,0x9E9E,0xD0AE,0xCFCF,0xDFDF,0xCFCF,0xCFCF,0xEEEE,0xEEEE,0xCFCF,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4141,0x5050,0x7F7F,0x7F7F,0xBFBF,0xF0F0,0xEFEF,0xE5E5,0xE6E5,0xEAD7,0xBFBF,0xF0F0,0xFEFE,0xD0DE,0xAFAF,0x8E70,0x6070,0x5050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x5050,0x7070,0x6060,0x7070,0x6060,0x7070,0x7070,0x7070,0x6060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6050,0x6060,0x7070,0x7070,0x6070,0x5050,0x6060,0x9F70
.word 0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x4242,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x4848,0x4A4A,0x4949,0x4242,0x9F9F,0x7171,0xAFAF,0xC4C4,0xAFAF,0xC4C4,0x4763,0x6262,0xC4C4,0x7171,0xC0C3,0x9EB0,0x81AE,0x9292,0x9281,0xA191,0xC1B1,0xD0D0,0x81B1,0x8281,0x9382,0x80B0,0x5151,0x6F6F,0x6F6F,0x4141,0x6F6F,0x7F7F,0x6060,0x6251,0x7283,0x7283,0x7283,0xC382,0x92C3,0x7292,0xC3C3,0xC281,0x81C3,0xC0B3,0xC0D1,0xC0C3,0xC3C2,0xC3C0,0xB2C3,0xC2B2,0x6272,0x6252,0x6252,0x6262,0x5262,0x6252,0xB3B3,0xC371,0xC072,0x72D4,0xB3B3,0xC072,0xC0C0,0xC3C3,0xC3C3,0xD1D3,0x62C3,0x8362,0x8372,0x6272,0xBFBF,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5050,0x8F8F,0xAFAF,0x9E9E,0xBFBF,0xFEFE,0xEFEF,0xEFEF,0xDEDE,0xFEFE,0xFEF0,0xDFDF,0xBFBF,0x4242,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x8F8F,0x8F8F,0x7070,0xDFDF,0xF5F5,0xE6E6,0xC7D7,0xBFBF,0x9F9F,0x7070,0xBFBF,0xF0F0,0x9EBE,0xAFAF,0x7070,0x7070,0x8F8F,0x4040,0x7F7F,0x4242,0x8F8F,0x5050,0x6060,0x708E,0x7070,0x6060,0x6060,0x6060,0x6050,0x6060,0x6060,0x5050,0x5050,0x5040,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x7070,0x7060,0x6060,0x6060,0x6060,0x5060
.word 0xD0D0,0xC0C0,0x8080,0x7171,0xAFAF,0x5252,0x8F8F,0x7F7F,0x4A4A,0x4A4A,0x4A4A,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4848,0xAFAF,0xC4C4,0xAFAF,0xD4D4,0xAFAF,0x5757,0x6363,0xC4C4,0x7262,0xC0C0,0x9EA1,0xA09E,0x9292,0x8292,0xA192,0xC1C1,0xD0D0,0xB1D0,0x8282,0x8293,0x9072,0x8E8E,0x4141,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5151,0x7262,0x7262,0x7283,0x7272,0xC282,0xB392,0xB2C2,0xB182,0x81C3,0xD2C1,0xC382,0xC3C2,0xC3C1,0xD0D0,0xC0D3,0xC1D0,0xC2C2,0x6293,0x6252,0x6362,0x6362,0x6262,0x6252,0x7261,0x72B3,0x71C3,0x71D0,0xC3C0,0xC371,0x7182,0xC0D4,0x82C2,0xD0C2,0x6281,0x7283,0x8383,0x5162,0xBEBE,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7070,0x9E9E,0xAFAF,0xEEEE,0xEFEF,0xEFEF,0xDFDF,0xDFDF,0xF0F0,0xE0E0,0xBFBF,0x9F9F,0x4747,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x7070,0x7070,0xFEFE,0xEFEF,0xDFDF,0xD6C7,0x8B8B,0xAFAF,0x9F9F,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x5060,0x8F8F,0x4141,0x8F8F,0x8F8F,0x5050,0x7070,0x7070,0x7070,0x7070,0x6060,0x5050,0x6060,0x6050,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x6060,0x7060,0x7070,0x7060,0x6060,0x7070,0x8F70
.word 0xC0C0,0xB0B0,0x71B0,0xAFAF,0xAFAF,0xAFAF,0x4242,0x4848,0x8F8F,0x4A4A,0x8F8F,0x4A4A,0x9F9F,0x4242,0x4747,0x9F9F,0x9F9F,0x6161,0xC5C5,0xAFAF,0x63C4,0x5757,0x4848,0x4747,0x6262,0xD071,0x9EC2,0xA09E,0x9192,0x9182,0x9281,0xD0B1,0xE0E0,0xC0D0,0x8191,0x8282,0x72A3,0x8181,0x7E7E,0x7F7F,0x7F7F,0x7F7F,0x5050,0x6261,0x6262,0x8383,0x8372,0x6272,0xC3A3,0x9282,0x8292,0x82B3,0xB3C0,0xC1B3,0xC2C1,0x81B1,0xC3C1,0xB2C3,0xB2C1,0xC2C3,0xC3C1,0x6272,0x6262,0x6273,0x6262,0x5252,0x5262,0xE362,0x8282,0xC372,0xC472,0xD071,0xC0D4,0xC372,0x7172,0xC2C2,0xC381,0x52C3,0x8362,0x8383,0x5262,0xD0D0,0x9E9E,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7E7E,0xAFAF,0xBE9E,0xDFDF,0xF0F0,0xF0F0,0xBFBF,0x9E9E,0xCFCF,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x4242,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xBFBF,0xCECE,0xDFDF,0xEFEF,0x59D7,0xAFAF,0x9F9F,0xBFBF,0x6060,0x6060,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0x9F9F,0x5050,0x6060,0x7E50,0x8E60,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x5050,0x5050,0x5050,0x8F8F,0x4050,0x4040,0x8F8F,0x4040,0x4040,0x5050,0x5050,0x6060,0x6060,0x6070,0x6060,0x7070,0x7060,0x6070
.word 0x7161,0x7171,0x7162,0x6162,0x9F9F,0x7070,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0x4A4A,0x9F9F,0x4747,0x8F8F,0x8F8F,0x5252,0xAFAF,0x5757,0x9F9F,0xD4D4,0x5757,0x4848,0x4852,0x4747,0x7171,0xC0C0,0xAEC0,0xA09E,0x9292,0x9291,0xA1A1,0xD0C1,0xE0E0,0xCFCF,0x82B0,0x8272,0x7282,0x7171,0x6060,0x9F9F,0x8F8F,0x4141,0x5251,0x6261,0x6262,0x7283,0x7283,0x7283,0xC382,0xB2B2,0xB292,0xB3B2,0xB2C1,0xB282,0xC0C1,0xC1D3,0xB1B2,0x82C3,0xB2C2,0xB381,0xC2B2,0x6272,0x6262,0x7362,0x7373,0x6262,0x5252,0xC362,0x7293,0x72C3,0x72B3,0x72C3,0x7171,0xC3C0,0x72B3,0xC3C0,0xC3C0,0x6281,0x6262,0x6283,0x4262,0xD0D0,0xBFBF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x8E8E,0xBFBF,0xF0F0,0xCFCF,0x8080,0x9F9F,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5151,0xDEDE,0xF0F0,0xEFEF,0xDFDF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x5050,0x708E,0x7070,0x708E,0xAFAF,0x7070,0x6060,0x6060,0x6060,0x5050,0x6060,0x5050,0x5050,0x5050,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x6060,0x7070,0x7070,0x6060,0x7070,0x6070,0x6060
.word 0xD4D4,0x7172,0x61D4,0xAFAF,0x7070,0xAFAF,0x5252,0x8F8F,0x4848,0x4A4A,0x4A4A,0x4848,0x8F8F,0x4747,0x4343,0x9F9F,0x6262,0x5757,0x4848,0x6363,0x9F9F,0x5757,0x4848,0x4747,0x8042,0xC3C3,0x9EB0,0x91AE,0x9191,0x9292,0xA192,0xC1C1,0xCFCF,0xD0D0,0xB1B1,0x9381,0x9382,0x7272,0x6171,0x5161,0x5151,0x5251,0x6252,0x6262,0x8362,0x8362,0x8372,0x7272,0x82C3,0x9292,0xB2B2,0xB2C2,0xC1C2,0xB2C2,0xB3C2,0xD1C0,0xD0C1,0xC3C1,0xC2C1,0xB2C2,0xC3C2,0x5252,0x6262,0x6273,0x6262,0x6273,0x5252,0xB352,0x7293,0x7272,0xC3C0,0xB3B3,0x71C3,0xC0C4,0x72C3,0x71B3,0xC3C3,0x62B3,0x7352,0x8362,0x4163,0x7070,0xD080,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xAEAE,0xAEAE,0xCFCF,0x8080,0x8E8E,0x9F9F,0x9E9E,0x9F9F,0x8F8F,0x9F9F,0x7F7F,0x6F6F,0x4242,0x7F7F,0x7F7F,0x7F7F,0x4242,0x4141,0xAFAF,0xDFDF,0xFFFF,0xF0F0,0xCFCF,0x9F9F,0x9F9F,0x6060,0x6060,0x7070,0x7070,0x7070,0xAFAF,0x6070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x9E9E,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x7060,0x7070,0xAFAF,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x5050,0x5050,0x6060,0x6060,0x6060,0x7070,0x6060,0x5060
.word 0xD0D0,0x7172,0x7171,0xAFAF,0xAFAF,0x6161,0x5252,0x4747,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x4747,0x4848,0x9F9F,0x5555,0x4747,0x4747,0x4747,0x4242,0x8F8F,0xB1C0,0xAEC0,0x819E,0x9292,0x8292,0xB191,0xD0D0,0xCFCF,0xBEBE,0xCFCF,0x8191,0xA372,0x7272,0x6272,0x6262,0x6262,0x6262,0x6262,0x6252,0x6262,0x8383,0x8362,0x9372,0xB2B2,0x8282,0x8292,0xB2B2,0xB2B3,0xC2B1,0xB2C1,0xB2B3,0xC2C1,0xB2D0,0xC0D0,0xB2C1,0xB2D1,0x6262,0x5252,0x5262,0x5262,0x6262,0x5252,0xC052,0x83B3,0x72B3,0xC372,0xC371,0x7272,0x61B3,0xC3C3,0x83B3,0xC372,0x5272,0x6252,0x6273,0x4252,0x9F9F,0xC071,0xC0C0,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8E8E,0xAEAE,0xBFBF,0xAFAF,0x6060,0x9F9F,0x7070,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x4242,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x6060,0xCFCF,0xEFEF,0xF0F0,0xDFDF,0xBFBF,0x8F8F,0x5050,0x7070,0x708E,0x7070,0x7070,0x9F9F,0x7060,0x6060,0x7070,0x6060,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x9F9F,0x6060,0x6070,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x6060,0x6060,0x7070,0x7070,0x7070,0x6060
.word 0xC1C1,0x7282,0x6161,0xAFAF,0xD0D0,0xC0C0,0xAFAF,0x5252,0x4747,0x8F8F,0x4A4A,0x4848,0x4848,0x4848,0x4747,0x4747,0x9F9F,0x4848,0x4747,0x9F9F,0x4747,0x9F9F,0x4747,0x4242,0x7171,0x81D0,0xAEC1,0xA09E,0x9291,0x8182,0xB182,0xBEE0,0xBEBE,0xBEBE,0xBEBE,0x71D0,0x7282,0x7293,0x7283,0x6262,0x6262,0x5262,0x5252,0x7362,0x8362,0x8372,0x6262,0x8262,0xB2C2,0xB292,0xB2B1,0xB2B2,0xC3B1,0xB2B2,0xC1D0,0xC2B2,0xB2C2,0x82C3,0xA3B3,0x82C3,0x72C3,0x6262,0x7362,0x6262,0x5263,0x5262,0x5252,0x9F9F,0xB382,0x7293,0x82B3,0xD071,0xC3C3,0x8372,0x71B3,0xB3C0,0xB3B3,0x5262,0x6252,0x7373,0x4273,0xC341,0xD371,0xD0D0,0xAFAF,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x7070,0x7070,0x7070,0x9F9F,0x6060,0x7070,0x5050,0x8F8F,0x8F8F,0x4040,0x8F8F,0x4141,0x8F8F,0x4242,0x5252,0x5151,0xBFBF,0xEFEF,0xEFEF,0xF0F0,0xAFAF,0x7070,0x8F8F,0x8F8F,0x6060,0x7070,0x7070,0x7070,0x6060,0x6060,0x7070,0x7070,0x6060,0x6060,0x7070,0x6060,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x5050,0x9F9F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x5050,0x6060,0x7070,0x7070,0x609F
.word 0xC0C0,0x82C3,0x6172,0x9E9E,0xBFBF,0xAFAF,0xBFBF,0x52C4,0x4747,0x4747,0x4242,0x9F9F,0x9F9F,0x9F9F,0x4848,0x4242,0x4747,0x8F8F,0x4747,0x8F8F,0x4747,0x8F8F,0x8F8F,0x4A4A,0x6161,0xC2C2,0xAEC0,0xB09E,0x9292,0x9292,0x9182,0xD0D0,0xAEAE,0xAEAE,0xBEBE,0x90AE,0x7272,0x7293,0x8372,0x6262,0x5262,0x6252,0x6252,0x6262,0x6283,0x6283,0x6283,0xB372,0x8282,0x9282,0xC1B2,0xC2C2,0xB1C1,0xB3C1,0xD1C2,0xC2D0,0xC1B2,0xB2B3,0xA3B3,0x82B3,0x93B2,0x5252,0x6262,0x6273,0x7362,0x6262,0x5252,0xB362,0x71A3,0x82C3,0x72B3,0xB383,0xD371,0xB381,0x9383,0x72C3,0xC3B3,0x52B3,0x6362,0x6262,0x4252,0xC351,0x7271,0x71C0,0xC0D0,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x5050,0x8F8F,0x5050,0x5050,0x7070,0x6060,0x8F8F,0x8F8F,0x5050,0x9F9F,0x4040,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0xD0D0,0xDFDF,0xEFEF,0xDFDF,0xAFAF,0x7070,0x8F8F,0x8F8F,0x8F8F,0x7070,0x7070,0x7070,0x7060,0x7070,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x9F9F,0x8E70,0x7070,0x8E8E,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x5050,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x8F8F,0x7F7F,0x4040,0x5050,0x8F8F,0x6060,0x7070,0x7070,0x6070
.word 0xD0D0,0x72B1,0x6062,0x9F9F,0x9E9E,0xBFBF,0xD4D4,0xAFAF,0x5252,0x4545,0x4848,0x4848,0x9F9F,0x4949,0x4747,0x4848,0x4242,0x4747,0x9F9F,0x4747,0x8F8F,0x4747,0x4747,0x8F8F,0x8F8F,0x8171,0x9EB0,0x91BE,0x9292,0x9292,0xB181,0xCFCF,0xBEBE,0xAEAE,0xBEBE,0xCFCF,0x7271,0x8372,0x7272,0x6283,0x6262,0x5262,0x6262,0x6273,0x6273,0x8362,0x6272,0xC293,0x92C2,0x9292,0xB1B3,0xB282,0xB2B2,0xC1C2,0xC2B2,0xD0C2,0xC2C2,0xC2B2,0xB1C2,0xB3B2,0x93C2,0x6252,0x6252,0x6262,0x6262,0x7362,0x4262,0xB361,0x7293,0x81B3,0xB3B3,0x9393,0xC372,0xC372,0x93B3,0x6293,0xC3C3,0x4293,0x6262,0x7363,0x4362,0x7171,0xB3B3,0xC371,0xD1D1,0x6060,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4040,0x8F8F,0x6060,0x9F9F,0x6060,0x4040,0x7F7F,0x9F9F,0x5050,0x4040,0x4040,0x7F7F,0x4040,0x8080,0xBFBF,0xF5F5,0xCFCF,0x8E8E,0x7070,0x9F9F,0x4040,0x5040,0x5050,0x9F9F,0x6070,0x6060,0x6060,0x7070,0x7070,0x7070,0x6060,0x7070,0x6060,0x7070,0x7070,0x7070,0xAFAF,0x9E80,0x9E9E,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x4848,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x8070,0x7070,0x7070,0x6060
.word 0xC0C0,0x7272,0x6262,0x6060,0xAFAF,0xAFAF,0xAFAF,0xB4B4,0x5757,0x4747,0x5757,0x5252,0x4848,0xAFAF,0xAFAF,0x4352,0x4747,0x9F9F,0x9F9F,0x5757,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x7171,0xC0C0,0x9EB0,0x819E,0x9292,0x8292,0xA182,0xBED0,0xBEAE,0xBEBE,0xCFCF,0xAEAE,0x7281,0x7272,0x6272,0x6272,0x6262,0x6252,0x7362,0x6262,0x6262,0x6283,0x6262,0xB293,0x92B2,0xB2B2,0xB2B2,0xB2B2,0xB2B2,0xB2C3,0xC2C2,0xB2B2,0xB282,0xB2C3,0xB3B2,0xC2B2,0x72D2,0x6262,0x6262,0x5273,0x5263,0x5262,0x4352,0xD3D3,0x93B3,0x7283,0x82C3,0xB3C3,0x7283,0x7272,0xB3B3,0x8393,0xC3B3,0x5283,0x6352,0x5262,0x4252,0xC352,0x7172,0x71C3,0x7271,0xC0C0,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x7070,0x6060,0x4040,0x5050,0x8F8F,0x5050,0x6060,0x8F8F,0x8F8F,0x7F7F,0xBFBF,0xE5E5,0xCFCF,0xAFAF,0x6060,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4040,0x6060,0x6060,0x6060,0x7070,0x7070,0x6060,0x7070,0x6060,0x6060,0x7060,0x7070,0x7070,0x7070,0x8080,0xCFCF,0xAEAE,0x9E9E,0xAFAF,0x7070,0xAFAF,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x6060,0x7070,0x6060,0x7070,0x5060
.word 0xBFBF,0x7171,0x6262,0xAFAF,0x7070,0xAFAF,0xAFAF,0xC5C5,0x6363,0xC552,0x9F9F,0xC5C5,0xAFAF,0x6767,0xAFAF,0xAFAF,0x5555,0x4852,0x5757,0xAFAF,0x6767,0xD6D6,0xAFAF,0x9F9F,0x8080,0x9292,0x8EB1,0xA09E,0x9291,0x8282,0xA182,0xCED0,0xBEBE,0xBEBE,0xBEBE,0xAEAE,0x7180,0x7293,0x8383,0x6283,0x6262,0x7362,0x6262,0x7362,0x7362,0x7262,0x7283,0xC292,0x92B3,0xB2B3,0xB2B2,0xC2C2,0xB1C2,0x82B3,0x82B2,0xB3B2,0xC392,0x8282,0xB2B3,0xB382,0x62C3,0x6252,0x6273,0x6273,0x7362,0x5262,0x5252,0xD062,0xB382,0x8393,0x7293,0xD371,0x9382,0xB383,0x6193,0x72C3,0x7293,0x5262,0x6252,0x7373,0x4252,0xD361,0xC372,0xC372,0x72C0,0xB172,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6060,0x5050,0x9F9F,0x9F9F,0x9F9F,0x6050,0x7070,0x5050,0x5050,0x5050,0xAFAF,0xDFDF,0xDFDF,0xCFCF,0xAFAF,0x8F8F,0x6060,0x6060,0x5050,0x8F8F,0x7F7F,0x5050,0x6060,0x6060,0x6060,0x7070,0x7070,0x6060,0x6060,0x7070,0x7070,0x7060,0x7070,0x708E,0x7070,0xD0D0,0xC0C0,0x9E9E,0x7070,0x9F9F,0x7070,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x5040,0x9F9F,0x6050,0x6060,0x8F50
.word 0xC0C0,0x7171,0xAFAF,0x5172,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xC5C5,0xAFAF,0xC5C5,0x4747,0xAFAF,0xAFAF,0x4848,0xAFAF,0x6565,0xC5C5,0x5252,0x4747,0x6969,0xBBBC,0xBFBF,0xAFAF,0x6161,0x81D0,0x9EC0,0xA0A0,0x9292,0x8292,0xB182,0xAEAE,0xAEAE,0xBEBE,0xBEBE,0xBEBE,0x71D0,0x7272,0x7272,0x7362,0x7362,0x6262,0x6273,0x6273,0x6273,0x6283,0x7262,0xC2B2,0xB292,0x9292,0xB392,0xA382,0xB282,0xB392,0xB2C2,0xC2B2,0xB2C2,0xB2A3,0xB292,0xB392,0x62B3,0x6252,0x6262,0x7373,0x7362,0x5273,0x6352,0x93D4,0x8282,0x72B3,0x8393,0x93B3,0xC372,0x9393,0xC493,0xB372,0xC3B3,0x5273,0x6352,0x7352,0x4263,0xB3B3,0x72B3,0x6271,0xC0D4,0xB261,0x6161,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5050,0x9F9F,0x6060,0x7070,0x6060,0x5050,0x5050,0x7070,0x6060,0x6060,0x9F9F,0xFEFE,0xEFEF,0xDFDF,0xBFBF,0x4141,0x8F8F,0x9F9F,0x5050,0x8F8F,0x4040,0x4040,0x5040,0x6050,0x9F9F,0x6060,0x7070,0x6060,0x6060,0xAFAF,0x7070,0xAFAF,0x7070,0x6070,0x7070,0x7070,0xC0C0,0x8080,0xAFAF,0x709E,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x4242,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x5050,0x5050,0x5050
.word 0x7171,0x7171,0xBFBF,0x6161,0x4141,0x5151,0xAFAF,0xBFBF,0xAFAF,0x6767,0xAFAF,0xAFAF,0x5757,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6363,0x63C5,0x5757,0x5959,0xBFBF,0x9B9B,0x9F9F,0x8F8F,0x9171,0x9E9E,0x8180,0x8282,0x8282,0xA181,0xBFBF,0xAEAE,0xBEBE,0xBEBE,0xD0D0,0x8181,0x7272,0x8383,0x6283,0x6283,0x6273,0x6362,0x6262,0x8362,0x8372,0x8362,0xB3B2,0xB382,0xB3B2,0x92B2,0x82B2,0xC2B2,0x82B2,0x82B2,0xC2B2,0xB2C3,0xC2C2,0xB3B2,0xC2B2,0x52B2,0x6262,0x6252,0x5262,0x6262,0x6262,0x5252,0xB371,0x7283,0xB3C3,0x72C3,0x9393,0xC372,0xB3B3,0x7293,0x6283,0xB3C3,0x5252,0x7373,0x5262,0x4252,0xC352,0x7283,0x71C3,0xC361,0x61D3,0x8181,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x6060,0x9F9F,0x7070,0x6060,0x7070,0x6060,0x7060,0x6060,0x6060,0xD0D0,0xDFDF,0xEFEF,0xDEDE,0xAFAF,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x7F7F,0x6060,0xAFAF,0x6060,0x6060,0x6060,0x6060,0x7060,0xFEFE,0xAEAE,0x6070,0x6060,0x9F9F,0x7070,0x7070,0x7070,0x8E70,0x8070,0x9EC0,0xC0C0,0x9F9F,0x7F7F,0x7F7F,0x4242,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x7F7F,0x5050,0x7F40
.word 0x6161,0x9F9F,0x8080,0x6161,0x4141,0x5151,0x9F9F,0xAFAF,0xD4D4,0xAFAF,0xAFAF,0xC5C5,0x9F9F,0x4848,0xAFAF,0xAFAF,0x6A6A,0xAFAF,0x5252,0x4747,0xAFAF,0x8B6A,0xAFAF,0x9F9F,0xB0B0,0xC0C0,0x8EC0,0x81C0,0x9292,0x8282,0xB182,0x9E9E,0xBEBE,0xBEBE,0xBFBF,0xD0D0,0x81DE,0x7262,0x7283,0x8362,0x7362,0x5262,0x6273,0x7362,0x6283,0x6262,0x8272,0xB2C2,0x8282,0x82A3,0xB2A3,0xB2C2,0xB282,0xB292,0x9282,0x82B2,0xA392,0x81B2,0xB182,0xC2C3,0x5293,0x6262,0x7373,0x6263,0x6362,0x5252,0x6242,0xC3C3,0x9393,0x7183,0xB3C3,0x93C3,0x9383,0xD462,0x93B3,0x7283,0x93D4,0x5252,0x6252,0x6363,0x4252,0xC3C3,0x9393,0x72C3,0x71D3,0x71D0,0x8181,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6060,0x7070,0x9F9F,0x6060,0x5060,0x7070,0x6060,0x6060,0x6060,0x8080,0xCFCF,0xEEEE,0xCFCF,0xAFAF,0x7F7F,0x8F8F,0x8F8F,0x5050,0x9F9F,0x8F8F,0x5050,0x6060,0xDEFE,0x6060,0x6060,0x6060,0x5050,0x6050,0xE0E0,0x70D0,0x6060,0x9F9F,0x6070,0x6060,0x7070,0x7070,0x7070,0x6060,0xBFBF,0xAFAF,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x4040,0x7F7F,0x6F6F,0x6F6F,0x4040,0x8F8F,0x4242,0x8F8F,0x7F7F,0x4040,0x7F8F
.word 0x9F9F,0x6161,0xAFAF,0xC0C0,0x4141,0x7F7F,0x4141,0x7070,0xC0C0,0x9F9F,0xAFAF,0xAFAF,0xC5C5,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x6A6A,0xC5C5,0x9F9F,0x4848,0xAFAF,0x4A4A,0x4A4A,0x8F8F,0xA171,0x9E9E,0xB090,0x8292,0x8282,0xA182,0xAED0,0xAEAE,0xAEAE,0xBEBE,0xBFBF,0xC0DE,0x7272,0x6283,0x8383,0x6262,0x7362,0x6262,0x6273,0x8362,0x6283,0xB362,0x82B3,0xA382,0xB2A3,0x82B2,0xB392,0x82B3,0xB3B2,0x82A3,0xB3A3,0xA3B3,0xA382,0x82B3,0xC382,0x5262,0x8362,0x8362,0x7362,0x5273,0x5262,0x6252,0xC072,0xB382,0x8393,0x7272,0xB3C3,0x8372,0x7293,0xC462,0xC493,0x62B3,0x5263,0x6352,0x6262,0x4252,0xB352,0x9393,0xD472,0xD371,0xC3C3,0xB152,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x6060,0x6060,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x7060,0x7060,0x8080,0xBFBF,0xC0C0,0x7070,0x5050,0x4040,0x8F8F,0x5050,0x5050,0x5060,0x5050,0x6060,0xD0D0,0xE0E0,0x5050,0x6060,0x6060,0x5060,0x5050,0xB0B0,0xBFBF,0x7070,0x7060,0x9E9E,0x7070,0x7070,0x7070,0x7070,0x7070,0xD6D6,0x5858,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4242,0x6F6F,0x4242,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x407F
.word 0x7171,0xAFAF,0x9F9F,0xBFBF,0x7070,0x4242,0x4141,0x9F9F,0xAFAF,0x9F9F,0x4848,0xAFAF,0x9F9F,0x4848,0x4848,0x5858,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x4A4A,0x7C8C,0x8C8C,0x7E7E,0x8181,0x92D0,0x9E9E,0x82A0,0x8292,0x8292,0xB182,0x9E9E,0xAEAE,0xAEAE,0xBEBE,0xD0D0,0xC0DE,0x6262,0x8372,0x6283,0x7373,0x6262,0x6273,0x7362,0x6262,0x8362,0xC372,0xA392,0x82A3,0x82B3,0xB282,0x82B3,0x8282,0xB2B2,0x92B3,0xC382,0xB282,0x82A3,0x93A3,0xC3A3,0x5262,0x6252,0x6263,0x6262,0x6283,0x6262,0x7342,0x93C4,0x7193,0xA3C3,0x7393,0xC372,0x93C3,0x9383,0x7183,0x93B3,0x83C4,0x6252,0x5273,0x6352,0x4252,0xC362,0x7293,0x81B3,0x80D4,0xD0D0,0x6252,0x7171,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6060,0x7070,0x7070,0x9F9F,0x6060,0x6060,0x6060,0x6060,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x5050,0x8F8F,0x5050,0x5050,0x6060,0x5050,0x6060,0x6060,0xFEFE,0xC0C0,0xBFBF,0x6060,0x6060,0x6060,0x5060,0xC070,0x7070,0x709E,0xAFAF,0xD080,0xCEE0,0x7060,0x6060,0x8080,0xD5D5,0x9BD6,0x4848,0x6F6F,0x7F7F,0x4242,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F6F
.word 0xBFBF,0x6060,0x9F9F,0xAFAF,0xBFBF,0x4141,0x7F7F,0x4141,0x9F9F,0x4949,0x9F9F,0x4848,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x8B8B,0x8B8B,0x9F9F,0xC5C5,0x7E7E,0x8C8C,0x8D8D,0x9E8C,0xD4D4,0xA181,0x9E8E,0x8290,0x8292,0x8282,0xA182,0x9E9E,0xBEBE,0xAEAE,0xBEBE,0xD0D0,0xE0E0,0x7272,0x7283,0x7383,0x5262,0x6373,0x6362,0x8362,0x8373,0x6262,0xB3A3,0x82A3,0xA382,0xA382,0x82A3,0x92A3,0xA3B2,0xA3A3,0xA3A3,0x82A3,0xB2B3,0xC382,0x8282,0x93B3,0x5262,0x6362,0x5262,0x6362,0x6252,0x5273,0x6363,0x9371,0x6283,0x71B3,0x72B3,0x7393,0xC371,0x93B3,0xB483,0xC362,0x73B3,0x5242,0x7373,0x5252,0x4252,0xB361,0xC4A3,0xD471,0xD471,0xC3C3,0x5271,0x7171,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5050,0x7070,0x7070,0x5050,0x5050,0x9F9F,0x6060,0x7070,0x7060,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x5050,0x5050,0x5050,0x6060,0x6060,0x5050,0xE0D0,0xC0C0,0xC0C0,0x5050,0x6060,0x6060,0x6060,0x8080,0x8080,0xAFAF,0xC080,0x8080,0xF0F0,0x607E,0x5050,0x6060,0xAFAF,0xD6D6,0x4949,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4242,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4242,0x7F7F,0x4040,0x7F7F,0x7F7F
.word 0xC0C0,0x6080,0x9F9F,0xAFAF,0x7171,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x4A4A,0x9F9F,0x4848,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x7B7B,0x8D8D,0x9F9F,0xAFAF,0x9C7E,0x8D8D,0x9DAC,0x7171,0x91C0,0x9E9E,0x91A0,0x8282,0x82A3,0x9172,0x9EAE,0xBEBE,0xAEAE,0xBEBE,0xD0D0,0xE0E0,0x7282,0x8372,0x6262,0x7362,0x6262,0x7362,0x7362,0x7362,0x6262,0xB3B3,0x8282,0x9382,0xA3A3,0x9282,0x82B3,0xA3B3,0x8282,0xA382,0x9372,0x8293,0x81B3,0x82B3,0x62B3,0x5252,0x6262,0x5252,0x5263,0x5252,0x4252,0x9342,0x71E4,0x8372,0x6283,0xB3D4,0x83C3,0x8383,0xC361,0x8383,0x7283,0x73C3,0x5242,0x5262,0x6363,0x4242,0xC3C3,0x7293,0x71C3,0xC0C3,0xD0D0,0x41C3,0x6162,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6060,0x7070,0x6060,0x8F8F,0x8F8F,0x7050,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x6070,0x5050,0x5050,0x5050,0x6060,0x7070,0x9F9F,0x6060,0x6150,0x9090,0xC0C0,0x7070,0x6060,0x5060,0x6060,0x6060,0x8080,0xC0C0,0x8080,0xB190,0x5050,0x5060,0x6060,0x7070,0xD5D5,0xC6C6,0x4949,0x7F7F,0x4242,0x7F7F,0x4242,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x4040,0x8F8F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x7F7F
.word 0xAFAF,0xAFAF,0x5050,0x9F9F,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x4A4A,0xAFAF,0xAFAF,0x8C8C,0x7E7E,0x4A4A,0x8E8E,0xBCBC,0xAC9E,0xCCCC,0x81D4,0x9ED0,0x8190,0x8282,0x8282,0xA182,0xBFBF,0x9E9E,0xBEBE,0xCFCF,0xD0D0,0xE0E0,0x6282,0x6262,0x7383,0x6273,0x7373,0x6273,0x7373,0x6262,0x7352,0xB3C2,0x9382,0xB3A3,0x8293,0x82A3,0xA3A3,0xB382,0xA3A3,0x8282,0x82B3,0x8282,0x9372,0x9372,0x62A3,0x5252,0x7362,0x6273,0x5262,0x6362,0x4252,0x4242,0xC371,0x7272,0x6283,0x6272,0x72C3,0x83B3,0xC462,0xC361,0xC483,0x5272,0x5242,0x6352,0x6262,0x4252,0xB352,0x9372,0xC372,0xD4C0,0xD071,0x5252,0x7251,0x7F7F,0x6F6F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x6060,0x6060,0x9F9F,0x9F9F,0x4040,0x6060,0x6060,0x7070,0x6070,0x6070,0x7070,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x6060,0x6060,0x7060,0x7070,0x6060,0x7180,0x80C0,0xD0D0,0x6060,0x6060,0x5050,0x8050,0x80C0,0xC0C0,0x7070,0xB0A0,0x4040,0x5050,0x9F9F,0xD5D5,0xAFAF,0x5757,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4242,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5050,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F7F
.word 0x7070,0x6060,0x9F9F,0x7E7E,0x6060,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x7C7C,0xAFAF,0xAFAF,0x9F9F,0x8C8C,0xAFAF,0x7E7E,0x9F9F,0x7E7E,0x7E7E,0xAFAF,0xAFAF,0x8E8E,0x8D8D,0x719E,0xB2C0,0x9EB0,0x8181,0x8292,0xA382,0xA182,0x9EBE,0xBFBF,0xBFBF,0xD0BE,0xD0D0,0xE0E0,0x7292,0x8362,0x8362,0x7362,0x6262,0x6273,0x6263,0x7373,0x8362,0x82C3,0x8293,0x8293,0x82A3,0xB393,0x8282,0x7272,0x7282,0x7293,0xA3A3,0x7272,0xC371,0xA372,0x5283,0x5252,0x5252,0x6362,0x6273,0x5262,0x4252,0xB363,0x6172,0x71D4,0x72D4,0x8383,0x61C3,0x93D4,0x8362,0xD462,0x9372,0x6283,0x5252,0x5263,0x6352,0x4252,0xC342,0xB393,0xD362,0x7171,0xD0C3,0x61D3,0x7252,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x7070,0x7070,0x7070,0x7070,0x5050,0x5050,0x9F9F,0x6050,0x7060,0x7070,0x7070,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x6070,0x7070,0x6060,0x6060,0x6060,0x7070,0x7070,0x7070,0xC0C0,0xC071,0xD070,0x5050,0x5050,0x6060,0x7070,0xC0C0,0xC0C0,0x7070,0x9090,0x5040,0x7060,0xC0C0,0xCFCF,0x7171,0x5757,0x4848,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F6F
.word 0xBFBF,0x8080,0x4040,0x9F9F,0xAFAF,0x9F9F,0x8B8B,0x4848,0x4848,0x9F9F,0x9F9F,0x8C8C,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x7E7E,0xAFAF,0x7E7E,0x7E7E,0x9F9F,0x8E8E,0xAFAF,0xBC8D,0xB1B2,0x9EA1,0x8181,0x8292,0x8282,0x9172,0x9E9E,0xBEBE,0xAEAE,0xBFBF,0xD1E0,0xE1E1,0x6272,0x8372,0x6283,0x6273,0x7373,0x7362,0x7362,0x5262,0x8262,0x82B3,0xA393,0xA393,0xA393,0x9372,0x7293,0x9393,0x7272,0x7293,0x7282,0x7272,0x72C4,0x8282,0x5283,0x5252,0x5273,0x6252,0x5252,0x5252,0x4242,0x8361,0x6273,0xC361,0x7272,0x83B3,0x6273,0x7171,0x83C4,0x8362,0xD471,0x43B3,0x5242,0x6362,0x5252,0x4252,0xB3B3,0x7283,0x71D4,0xC3C3,0xD4C0,0x52D0,0x6251,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x5050,0x6060,0x7070,0x7070,0x7070,0x6060,0x5050,0x5050,0x5050,0x7070,0x7070,0x7070,0x7070,0x7070,0x6070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7060,0x6060,0x6060,0x7060,0x7070,0x7070,0xCFCF,0x7060,0x80C0,0x8080,0xC0C0,0x6050,0x5050,0x6050,0x70C0,0x9090,0xC0C0,0x6080,0x5050,0x5050,0xAFAF,0x7070,0x5050,0x8F8F,0x8F8F,0x4242,0x6F6F,0x7F7F,0x4242,0x7F7F,0x4242,0x4242,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F
.word 0xCFCF,0x7070,0x8F8F,0x8E7E,0xBFBF,0x4A4A,0x9F9F,0x4242,0x9F9F,0x8F8F,0x7E7E,0x8E8E,0xAFAF,0x8E8E,0x7E7E,0x7E7E,0xAFAF,0x7E7E,0x7E7E,0x7E7E,0x9F9F,0x7E7E,0x9F9F,0xAFAF,0x8D8D,0xA182,0xAE91,0x909E,0x8282,0xA382,0xA082,0x9E9E,0x8080,0xBFBF,0xD0E0,0xD1E0,0xD1E0,0x6281,0x8372,0x8362,0x7373,0x6273,0x6352,0x6263,0x6273,0xB362,0x82A3,0x9393,0x9393,0x8272,0x7283,0x7272,0x9393,0x8383,0x7272,0x6272,0x72B3,0x7272,0x9383,0x4262,0x6252,0x6262,0x5273,0x5252,0x5273,0x5242,0xD4D4,0x6262,0x52C4,0xC361,0x7272,0x7372,0x8383,0x7171,0x8383,0x7373,0x7371,0x5242,0x6263,0x5263,0x4242,0xC462,0xC493,0xC371,0x71D3,0xC3C3,0xD0D0,0x6152,0x9F9F,0x8F8F,0x5050,0x5050,0x5050,0x7060,0x7070,0x7070,0x7070,0x7070,0x6060,0x8F8F,0x8F8F,0x4040,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x6060,0x6060,0x7070,0x7070,0x7070,0x7060,0x6060,0x7070,0xAFAF,0x8080,0x7070,0xF070,0xEFEF,0x6070,0x7172,0x70C0,0xC0C0,0x6060,0x5050,0x6060,0x8080,0xB0B0,0xC0C0,0x8080,0x5050,0x6161,0x6161,0x6060,0x5050,0x7F7F,0x4242,0x7F7F,0x4242,0x4242,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F6F
.word 0xCFCF,0xCFCF,0x7E80,0x9E7E,0x7070,0x4242,0x8F8F,0x9F9F,0x9F9F,0x7C7C,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0x9F9F,0x7E7E,0x9F9F,0x7E7E,0x7E7E,0x8E8E,0x8E8E,0x6060,0xAFAF,0x9B8D,0xA1D4,0xD092,0xA09E,0x8292,0xA382,0x9272,0xAEAE,0xBFBF,0xD0D0,0xE0E0,0xD1D0,0xD2E0,0x7272,0x8383,0x6283,0x6262,0x6362,0x6273,0x6362,0x6273,0xB373,0x93A3,0x9372,0x8372,0x9393,0x9372,0x9372,0x7272,0x7272,0x7272,0x72D4,0x71D4,0x8372,0x8372,0x5252,0x5252,0x6352,0x6362,0x5262,0x5262,0x5242,0x7171,0xC471,0x6262,0xC4C4,0x7172,0x8361,0x6273,0xD461,0xC461,0xB373,0x7362,0x4242,0x5252,0x5273,0x4242,0xC352,0x7272,0xC3C3,0xD0D0,0xC0C3,0x6262,0x6252,0x9F9F,0x8F8F,0x5050,0x6060,0x5050,0x7070,0x7060,0x7070,0x7070,0x7070,0x6060,0x5050,0x5050,0x7F7F,0x4040,0x6060,0x7070,0x6070,0x6060,0x6060,0x6060,0x6060,0x7070,0x7070,0x6060,0x7070,0xAE70,0xE0E0,0x7070,0x7060,0xC070,0xC0C0,0xC0C0,0xE3F2,0xC071,0x9090,0x5050,0x5050,0x5050,0x8080,0x70C0,0xD0D0,0x6080,0x5050,0x5050,0x6060,0x4040,0x8F8F,0x4747,0xD542,0xFAFA,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4242,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F
.word 0xAEAE,0xAEAE,0x70D0,0x7E7E,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x7E7E,0x9E8E,0xAEAE,0xBFBF,0x7E7E,0x7E7E,0x9F9F,0x7E7E,0x9F9F,0x7E7E,0x8E8E,0x9E9E,0x9E9E,0x9F9F,0x9F9F,0x829B,0xB081,0x909E,0x8281,0x82A3,0x9193,0x9E9E,0xC0C0,0xD0D0,0xE0D0,0xD0D1,0xC1F0,0x7272,0x6283,0x7383,0x6373,0x5273,0x6373,0x7362,0x6262,0xB383,0x9393,0x9383,0x8362,0x9383,0x9383,0x7283,0x8393,0xC372,0x7162,0x7172,0xC471,0xC371,0x7372,0x4242,0x5252,0x5252,0x6262,0x5263,0x5252,0x4242,0xC462,0x7162,0x6262,0x6262,0xD4D4,0xC4C4,0xC461,0x6362,0x7171,0xC462,0x5272,0x5242,0x5252,0x6352,0x5252,0xC452,0xC372,0xD471,0xC381,0xC3C3,0x61D0,0x6242,0xAE41,0x5050,0x5050,0x6060,0x6060,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x8F8F,0x6060,0x5050,0x7F7F,0x4040,0x6060,0x6060,0x6060,0x6060,0x5050,0x6060,0x6050,0x7060,0x7070,0x7070,0x7070,0xCFCF,0xCFCF,0x9F9F,0x7070,0x6060,0xC071,0xF2B0,0xF2F2,0x80D0,0xB0C0,0x8080,0x5050,0x8F8F,0x7170,0x7171,0xD090,0x50D0,0x6060,0x5050,0x5050,0x8F8F,0x7F7F,0x7F7F,0xF1F1,0x9C9C,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F7F
.word 0x9E60,0xBE9E,0x80D0,0x4050,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x9F9F,0x6E6E,0x8E8E,0xAFAF,0x8E8E,0x9E9E,0xBFBF,0x7E7E,0x7E7E,0x5050,0x8E8E,0x8E8E,0x9E9E,0x9E9E,0x9E9E,0x8E8E,0x4A4A,0x7272,0xA082,0x9E9E,0x9282,0x8282,0xA082,0x9E9E,0xD0D0,0xE0D0,0xD0D0,0xD1D1,0xC2C2,0x7272,0x7383,0x7362,0x6262,0x6263,0x6263,0x7373,0x6373,0x9393,0x7283,0x8393,0x8393,0x7272,0x7262,0xB372,0x7293,0x6283,0xC472,0x6272,0x7272,0x9361,0x6272,0x4242,0x6252,0x5262,0x5252,0x6262,0x4252,0x4242,0x62D4,0xC362,0x7272,0x6373,0x6262,0x7271,0x6161,0x7362,0xC462,0x6262,0x42C4,0x5242,0x6373,0x5252,0x4242,0x7272,0xC483,0x71C3,0x71D3,0xC0C3,0x52E3,0x6241,0x4040,0x9F9F,0x8F8F,0x6060,0x7070,0x7070,0x7070,0x9E70,0xD0D0,0xAFAF,0x8F8F,0x4040,0x9F9F,0x8F8F,0x6060,0x7070,0x6060,0x6060,0x6060,0x5060,0x5050,0x5050,0x6060,0x6060,0x7060,0xC060,0xE6E6,0xC4C6,0x5050,0xFE80,0x70E0,0x7171,0x8171,0x8192,0xD0C1,0x8080,0x60C0,0x5050,0x4040,0x7171,0x7171,0xB080,0xCFCF,0x5050,0x6060,0x5050,0x4040,0x7F7F,0x4141,0xE0E0,0xAFAF,0x6F6F,0x6F6F,0x6F6F,0x4242,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4242,0x6F6F,0x6F6F
.word 0x8E8E,0xC0C0,0x8080,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x7E7E,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0x5050,0x8F8F,0x7E7E,0x9F9F,0x8E8E,0x9E8E,0x9E9E,0x9E9E,0x9E9E,0xC0C0,0xAFAF,0xA072,0x9EAE,0x8271,0xA3A3,0x7193,0x9EC0,0xD0C0,0xD0E0,0xD1D0,0xE0D0,0xB2F0,0x8362,0x8383,0x6273,0x6373,0x6362,0x6273,0x7363,0x5252,0x9383,0x7283,0x7283,0x9383,0x9393,0x9372,0x7272,0x7262,0x7262,0x6272,0x7272,0x72C4,0x62C4,0x5283,0x5242,0x5252,0x6352,0x5252,0x5252,0x4242,0x5242,0x6262,0x6273,0x6262,0x6262,0x6273,0x5252,0x6272,0x6262,0x6362,0xE461,0x4261,0x4242,0x5252,0x4263,0x4242,0xD452,0x7272,0xC371,0xE471,0xD371,0xAFAF,0x6252,0x6F6F,0xAFAF,0x5050,0x6060,0x7070,0x7070,0x9E70,0xBEBE,0xBFBF,0xAFAF,0x4040,0x7F7F,0x5050,0x6060,0x6060,0x7070,0x9F9F,0x5050,0x5050,0x6050,0x5050,0x6060,0x6060,0x7060,0x7060,0x8080,0xD6D5,0xC6C6,0x5051,0xCFCF,0xE0E0,0xC0E0,0xC0C3,0x8181,0xC0B2,0x80D0,0x50C0,0x5050,0x5050,0x7171,0x7171,0xC0C0,0xC0C0,0x6060,0x7070,0x5050,0x4040,0x5050,0xAFAF,0xBFBF,0x6060,0x7F7F,0x4242,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F7F
.word 0x9F9F,0x7080,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6E6E,0x7E7E,0x7E7E,0x8E8E,0xAFAF,0x9F9F,0xAFAF,0x6060,0x9F9F,0x9F9F,0x9F9F,0x7E8E,0x8E8E,0x8E8E,0x9E9E,0x9E9E,0x9E9E,0xB2C0,0x9F9F,0x8172,0x9E9E,0x92A0,0x82A3,0x8293,0xCED0,0xD0C0,0xD1E0,0xC1D0,0xF0D1,0xB2E1,0x8372,0x7383,0x6362,0x6263,0x7363,0x6362,0x5263,0x7352,0x93B3,0x9393,0x9372,0x7272,0x9383,0x9393,0x8383,0x7272,0xC362,0x7162,0x6272,0x6262,0x7271,0x5273,0x4242,0x5252,0x6252,0x6352,0x5252,0x4252,0x4242,0x62C0,0x5283,0x6252,0x6161,0x5252,0x6273,0xD4D4,0x7272,0x7361,0x6262,0x4261,0x4242,0x5242,0x5263,0x4242,0x7243,0x72C4,0xC3C3,0x71C3,0xD0D3,0x42C0,0x6252,0x7F7F,0x8080,0x6060,0x6060,0x7070,0x7070,0x9E9E,0xDFDF,0xAFAF,0x4848,0x8F8F,0x7F7F,0x7F7F,0x7070,0x7070,0x7070,0x6060,0x5050,0x5050,0x6060,0x5050,0x5050,0x6060,0x7070,0x7070,0xD5D5,0xD6D6,0x57C6,0x8F8F,0x7272,0xC080,0xC380,0x8181,0x81C3,0x9291,0xC0C1,0x8080,0x5050,0x5050,0xC050,0xC071,0xC071,0xD080,0x7070,0x6060,0x5050,0x6050,0xC0C0,0xCFCF,0xC0D0,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F
.word 0x8F8F,0x6060,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0x9F9F,0x7E7E,0x9F9F,0x9F9F,0x7E7E,0x8E8E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0xD1C0,0xCECE,0x7161,0x9EA0,0x8290,0xA382,0x8282,0xB1B1,0xD0E0,0xD1D1,0xE1D1,0xE1D1,0x72E2,0x7283,0x7383,0x7373,0x6362,0x6373,0x6352,0x7362,0x7352,0x9393,0x8383,0x8393,0x8393,0x8372,0x8362,0x7272,0xC483,0x6262,0xC472,0x7162,0x7272,0x72D4,0x4262,0x4242,0x5252,0x5252,0x5252,0x6252,0x4242,0x4242,0x7373,0x6271,0x7373,0x6161,0x6161,0x5263,0x6262,0x6261,0xAFAF,0xC462,0x4242,0x4242,0x4252,0x5252,0x4242,0xD462,0xB372,0x72C3,0x71D4,0xD4D4,0x41D3,0x6252,0x4141,0x6060,0x6060,0x7070,0x7070,0x7070,0xFEFE,0xDAF9,0x4848,0x4848,0x8F8F,0x7F7F,0x7F7F,0x7070,0x7070,0xAFAF,0x6060,0x5050,0x9F9F,0x5050,0x8F8F,0x5050,0x6060,0x6060,0x8070,0xC562,0x58C5,0x4848,0x4141,0x5151,0xC371,0xC071,0x8181,0x81B1,0xA192,0xD0A1,0x70C0,0x4040,0x5050,0xC0C0,0x6271,0xC071,0xC071,0x7EC0,0x5060,0x6060,0xAFAF,0xBEBE,0xBEBE,0xBFBF,0x7E7E,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4242,0x7F7F,0x6F6F,0x6F6F,0x7F6F
.word 0x4E4E,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8E8E,0xAFAF,0xAFAF,0x8E8E,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5050,0x9F9F,0x8E8E,0x9F9F,0xAFAF,0x9E9E,0x9E8E,0x9E9E,0xE0AE,0xE1D1,0xD3FE,0x6161,0xC081,0x719E,0xA3A3,0x93A3,0xA1A1,0xC1E0,0xD1D1,0xD2D1,0xE1D1,0x62C2,0x8372,0x7383,0x6362,0x6363,0x6362,0x7363,0x5273,0x8352,0x72B3,0x7283,0x8383,0x8383,0x9383,0x7283,0x7262,0x7162,0x71C4,0x6162,0x62C4,0x61C4,0x8361,0x4262,0x4242,0x5252,0x5252,0x5252,0x5252,0x4242,0x5252,0x6262,0xAFAF,0x5262,0x5252,0x6161,0x5262,0x6362,0x9F9F,0x6272,0x9F9F,0x4242,0x5242,0x5263,0x4242,0x4242,0xC3C3,0x9393,0xC372,0xD3C3,0xC071,0x52D4,0x6352,0x4040,0x7EC0,0x7070,0x7070,0x7070,0x7070,0xE9E9,0xD7EA,0x5858,0x8F8F,0x9F9F,0x7F7F,0x7F7F,0x8080,0x7070,0x7070,0x6060,0x9F9F,0x8F8F,0x9F9F,0x4040,0x5050,0x6060,0x7060,0x6060,0x5252,0x8F8F,0xC6C6,0x9F9F,0x4141,0xC0C0,0x7171,0xB271,0xB181,0xA182,0xC381,0x70C0,0x4040,0x4040,0x6060,0xC071,0x71C3,0x70D0,0x70C0,0x6060,0x9E70,0xBFBF,0xCFCF,0xBFBF,0xD0C0,0xBCBD,0xBDCC,0x4A8E,0x8F8F,0xC542,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F
.word 0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4E4E,0x7E7E,0xAFAF,0xAFAF,0x8E70,0x7070,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0x7E7E,0x9F9F,0x5050,0x8E8E,0x9E8E,0x8E8E,0x9E8E,0xAE9E,0xE0BE,0xF1D1,0x8282,0x6F6F,0x7150,0xC04E,0x92C3,0x82B3,0xD1B2,0xD1F0,0xD2D1,0xC1D2,0xE1C2,0x72C3,0x8383,0x7383,0x7363,0x6263,0x5263,0x5273,0x5263,0x9363,0x8393,0x8383,0x7283,0x7262,0x7283,0x8383,0x8362,0x7272,0x6161,0x7272,0x6162,0x6262,0x6283,0x4242,0x4142,0x5242,0x6252,0x5252,0x4252,0x4242,0x9F9F,0x5262,0x7162,0xAFAF,0x6352,0x5252,0x6161,0x4262,0x6262,0x9F9F,0x6161,0x6363,0x5242,0x5252,0x4252,0x4242,0xC4C4,0x83B3,0xC3B3,0x71C3,0xD4C3,0x4141,0x6252,0x4141,0xBFBF,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0x5959,0xBBBB,0xAFAF,0x9F9F,0x7F7F,0x4040,0xD080,0xD0D0,0x6060,0x8F8F,0x6060,0x6060,0x4040,0x7F7F,0x4040,0x5050,0x5050,0x8F8F,0x9F9F,0x4848,0x4949,0x4747,0x5252,0x6262,0xC071,0x81C3,0xB1B2,0xB282,0xF2D2,0xE0E0,0x4040,0x8F8F,0xC060,0x72C0,0xC071,0xC0C0,0x80D0,0x7070,0xAFAF,0xAEAE,0xBEBE,0xD0AE,0xECD0,0xEAEA,0xFCFB,0x9CAD,0x7C7C,0xD4D4,0x4A4A,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F7F
.word 0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0x8E8E,0x9F9F,0xAFAF,0xAFAF,0x8E8E,0x8E8E,0x7E7E,0x9F9F,0x8E8E,0x9E9E,0x9E9E,0x9E9E,0xC0BE,0xE1D1,0x72F1,0x6F6F,0x6F6F,0xBEBE,0x82E1,0xB3B2,0xD282,0xD2F0,0xD2C1,0xC2C2,0xE2D1,0x8393,0x8393,0x7373,0x6373,0x6362,0x7352,0x6263,0x5252,0xC473,0x6283,0x7383,0x8383,0x8383,0x6262,0x7262,0xC462,0x6362,0xC462,0x7252,0x6161,0x8383,0x6262,0x4242,0x4242,0x4252,0x5252,0x5252,0x5252,0x4242,0x9F9F,0x6262,0x5252,0x6262,0x5262,0x5252,0x9F9F,0x9F9F,0x5252,0x6152,0xC4C4,0x4251,0x4242,0x5242,0x4252,0x4242,0xD352,0xC472,0x7272,0xD472,0x7171,0x7171,0x6252,0x4242,0xD0D0,0x7070,0x7070,0x7070,0x7070,0x4A4A,0x4A4A,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x8080,0xD0D0,0xBFBF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x4040,0x4040,0x4040,0x5050,0x5050,0x4242,0x4848,0x9F9F,0x4747,0xC5C5,0x6262,0x7272,0x7171,0x81B2,0xB291,0xFFFF,0xD1E1,0xC0C0,0x4040,0xC0C0,0xC060,0x8182,0xC081,0xD080,0xAFAF,0x7070,0xBFBF,0xBFBF,0xC0C0,0xFEFE,0xEAEA,0xBCEC,0xBCBC,0x8D8D,0xCECE,0xAFAF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F
.word 0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7E7E,0x9F9F,0xAFAF,0xBFBF,0x7080,0xAFAF,0xAFAF,0x8E8E,0x8E8E,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0x9E8E,0xAFAF,0x9E9E,0x9E9E,0xAE9E,0xD0D0,0xF1C1,0x83F1,0x6F6F,0x6F6F,0xAEAE,0xA3D1,0xB3B2,0xD292,0xE1F1,0xD2C2,0xD2D2,0xD3E2,0x9372,0x8383,0x7383,0x6373,0x5263,0x5263,0x6363,0x5263,0xB373,0x8383,0x8383,0x8362,0x7383,0x8383,0x6273,0x6162,0x6262,0x6262,0x7373,0x6273,0x6161,0x52D4,0x4242,0x4242,0x5252,0x5251,0x5252,0x4252,0x4242,0x6262,0x9F9F,0x5252,0x9F9F,0x5151,0x5252,0x5252,0x6161,0x5151,0x6363,0x6161,0x4242,0x4242,0x4242,0x4352,0x4242,0x6262,0x72C4,0xD4C3,0x6161,0x71C3,0x41D3,0x6363,0x4141,0xC0C0,0x7070,0x7070,0x7070,0xAFAF,0x4848,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0xC0C0,0xD0D0,0xBFBF,0x4157,0x7F7F,0x5050,0x9F9F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x5050,0x4747,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x71C3,0xA1B2,0x82B2,0xB2C3,0xC1B1,0xE0D0,0x9F9F,0xC0C0,0x7070,0x81C3,0xD4C0,0xC0C0,0xC0C0,0x9F9F,0xAFAF,0xBFBF,0xC0C0,0xCBCB,0xCCCB,0xAEBE,0xBFBF,0xCFCF,0x8080,0x5151,0xE4E4,0x6F6F,0x7F7F,0x4242,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4242,0x9F41
.word 0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6E6E,0xBFBF,0xBFBF,0xC0C0,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x7070,0x8E8E,0x7E7E,0x9F9F,0x9F9F,0x5050,0x5050,0x9E8E,0x9E9E,0x9E9E,0x9E9E,0xC1C1,0xE1C1,0x62F2,0x6F6F,0x6F6F,0x6F6F,0xA3D2,0xB382,0xC2B3,0xD2E2,0xC2C2,0xC3B2,0xC3D2,0x9383,0x9383,0x7373,0x5263,0x6373,0x6363,0x6362,0x4252,0x93C4,0x7383,0x7383,0x8373,0x8362,0x7373,0x6273,0xC462,0x6262,0x6261,0x6261,0x5252,0x6262,0x5262,0x4242,0x4142,0x5252,0x4252,0x5251,0x4242,0x4242,0x9F9F,0x6161,0x6161,0x6152,0x9F9F,0x6161,0x6351,0x5151,0x9F9F,0x6161,0x5252,0x4242,0x5243,0x4252,0x4242,0x4242,0xD4D4,0xC461,0x7161,0x71D4,0xC3C3,0x5252,0x5241,0x5252,0xD0D0,0x7070,0x7070,0xC070,0x7070,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xE0E0,0x8080,0x8F8F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x5050,0x9F9F,0x8F8F,0x7F7F,0x424A,0x6F6F,0x7F7F,0x7F7F,0x4242,0x8181,0x9281,0x92A1,0x8282,0xA182,0xC0B1,0xD0D0,0x71C0,0x9090,0xC281,0xC071,0xC071,0xB0B0,0xAEAE,0xBFBF,0xC0C0,0xB0B0,0x7B8B,0x8C8C,0xBFBF,0xCFCF,0xAEAE,0xC0C0,0xE4E4,0xF7F7,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4242,0x6F6F,0x7F7F,0x4246,0x4248,0x709F
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9E8E,0xBEBE,0xBFBF,0xBFBF,0x6060,0x9F9F,0x9F9F,0x9F9F,0x7E60,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0x8E7E,0x7E7E,0x9F9F,0xAFAF,0x9E9E,0x9E9E,0xC1C1,0xE1C1,0x42D2,0x6F6F,0x6F6F,0x6F6F,0xA3E3,0xA3A3,0xC2B2,0xC2D3,0xC2C2,0xC2B3,0xB3E3,0x8383,0x8383,0x7373,0x6362,0x6363,0x6352,0x6352,0x5242,0x83B4,0x6283,0x8362,0x73A4,0x6273,0x6272,0xC461,0x6273,0x6252,0x6262,0x6161,0x6262,0x6262,0x4262,0x4242,0x4242,0x5142,0x5252,0x4252,0x4242,0x4242,0x5252,0x6152,0x9F9F,0x6262,0x5151,0xAFAF,0x5252,0x5151,0x6161,0x9F9F,0x5252,0x6F6F,0x6342,0x4252,0x4242,0x6F6F,0x6161,0x6262,0xD461,0x6161,0x6162,0x5151,0x5252,0x4173,0xC0C0,0x7070,0x7070,0x8080,0x4848,0x4848,0x7F7F,0x8F8F,0x5050,0x8F8F,0x4242,0x9F9F,0xD0D0,0xC0C0,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x7F7F,0x4747,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7171,0x8281,0x91B2,0x9182,0xB292,0xC1C2,0xC1C1,0xD0D0,0xC0D0,0x9281,0xC3C0,0xC0D0,0xB0B0,0xBFBF,0xBFBF,0xD0D0,0xC0C0,0x9F9F,0x7C7C,0xBFBF,0x9E9E,0x8080,0x8080,0xD4D4,0xE0F4,0xBFBF,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x4242,0x4848,0x4747,0x9F47
.word 0x7F7F,0x6F6F,0x6F6F,0x6E4E,0xAE8E,0xD0D0,0xBFBF,0xAFAF,0x7E7E,0x9F9F,0xAFAF,0x8E8E,0x7E7E,0x9F9F,0x9F9F,0x5050,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0x9E9E,0x9E9E,0xAE9E,0xC0C0,0xE1D1,0x42C2,0x6F6F,0x6F6F,0x6F6F,0x93D4,0xA393,0xD2C3,0xB2D3,0xC3B3,0xC3B2,0x83D3,0x8383,0x7373,0x6373,0x6363,0x5252,0x5263,0x5263,0x6352,0x7383,0x7373,0x7373,0x6362,0x6373,0x6352,0x6162,0x6262,0x6363,0x6363,0x5252,0x6363,0x5252,0x5262,0x4242,0x4242,0x4252,0x5252,0x4152,0x4242,0x7F7F,0x6161,0x5252,0x9F9F,0x9F9F,0x5252,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x4242,0x4242,0x4252,0x4242,0x4242,0xC3C3,0x6161,0xC4C4,0x6271,0xD4D4,0x4141,0x7342,0x4262,0xC0C0,0x7070,0x7070,0xAFAF,0x8F8F,0x8F8F,0x4848,0x4848,0x7F7F,0x4242,0x8F8F,0xC0C0,0x8080,0x5050,0x4040,0x6F6F,0x7F7F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x6F6F,0x8F8F,0x4848,0x7F7F,0x8F8F,0x4242,0x8F8F,0x5241,0x81C3,0xB291,0x9281,0xC2A1,0xD2D2,0xC1B1,0xD3C1,0xD0D0,0x81C1,0xC0B2,0xD371,0xC0C0,0xD0D0,0xBFBF,0xB0B0,0xAFAF,0x8F8F,0x9F9F,0xBFBF,0x7070,0xD0B0,0x7E7E,0x4040,0x7070,0xC0C0,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4747,0x7F7F,0x4747,0x8F8F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7E7E,0xBFBF,0x8080,0x7070,0x7070,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0x8E8E,0x9F9F,0x9F9F,0x5050,0x9F9F,0x708E,0x9E8E,0x9E9E,0xD0D0,0xE1C1,0x41B3,0x6F6F,0x6F6F,0x6F6F,0x83C3,0x9383,0xD3C3,0xB2C3,0xB3B3,0xC3B3,0x73C3,0x8373,0x7373,0x6363,0x6352,0x6343,0x6352,0x4252,0x6342,0x62B4,0x7373,0x6363,0x6363,0x6273,0x6362,0x7373,0x6261,0x6262,0x6262,0x6262,0x5252,0x6161,0x4252,0x4242,0x4142,0x5242,0x5252,0x4252,0x4242,0x7F7F,0xAFAF,0x5252,0x5151,0x8F8F,0x4141,0x5151,0x9F9F,0x5151,0x9F9F,0x5151,0x4141,0x6F6F,0x4242,0x5242,0x4242,0x4242,0x9F9F,0x6262,0x6261,0x7373,0x7162,0x4141,0x5242,0x4173,0xD0D0,0x7070,0x7070,0x6060,0x4747,0x4848,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x4848,0xBFBF,0x5050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8271,0xA192,0x9292,0xC2A1,0xB2B2,0xB1B1,0xD0B1,0xD0D1,0x8281,0xD3B1,0xC0C0,0xC0B3,0xE0E0,0xD0E0,0xC0C0,0x4A4A,0x7F7F,0xD0D0,0xC0C0,0x7070,0x8080,0x4E6E,0x7F7F,0x7071,0x7171,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x6F6F,0x4242,0x6F6F,0x7F7F,0x7F7F,0x4747,0x7F7F,0x7F7F
.word 0x9F9F,0x4242,0x7F7F,0x8F8F,0x8F8F,0x6161,0x7070,0x9F9F,0x9F9F,0x5050,0x9F9F,0xAFAF,0x7E7E,0x8E8E,0x8E8E,0x8E8E,0x8E8E,0x8E8E,0x8E8E,0x8E8E,0x9E9E,0x9E9E,0x9E9E,0xB1D0,0xD1B1,0x42B3,0x6F6F,0x6F6F,0x6F6F,0x73C4,0x9383,0xC3B3,0xA3B3,0xA3B3,0xB3A3,0x6293,0x7373,0x6363,0x6363,0x4343,0x4252,0x4352,0x4242,0x6342,0x6373,0x6273,0x7362,0x6262,0x6262,0x6262,0x5262,0x5252,0x9F9F,0x6161,0x6161,0x6161,0x6161,0x4252,0x4242,0x5242,0x5252,0x5142,0x4252,0x4242,0x8F8F,0x5252,0x6161,0x9F9F,0x5151,0x9F9F,0x4242,0x5151,0x9F9F,0x9F9F,0xAFAF,0x4242,0x4242,0x4242,0x5242,0x4442,0x6F6F,0x6161,0x6161,0x6262,0x6262,0x7373,0x6262,0x6342,0x4262,0xC0C0,0xC0C0,0x9F9F,0x9F9F,0x4747,0x7F7F,0x8F8F,0x8F8F,0x4242,0x5151,0xC0C0,0x7070,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6060,0x4040,0x8F8F,0x7F7F,0x7070,0xCFCF,0xCFCF,0xD0D0,0xE0E0,0xE4E4,0x9F9F,0x8F8F,0x4141,0x8F8F,0x4141,0x8261,0xB292,0x92A1,0xA192,0xB2A1,0xB1B1,0xB1B1,0xE1C1,0xC1E0,0x81B1,0xC3C0,0xC0C0,0xE3D0,0xD0D0,0x71C3,0x6C6C,0xAFAF,0xD0D0,0xAFAF,0xC071,0x9F9F,0x8F8F,0x8F8F,0xD0D0,0xB071,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4848,0x7F7F
.word 0x4747,0x5757,0x9F9F,0x4051,0x8F8F,0x7070,0x7070,0xAFAF,0x7E7E,0x7E7E,0x9F9F,0x7E7E,0x8E8E,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9E8E,0x9E9E,0x9E9E,0xAE9E,0xC1C1,0xE1C1,0x4282,0x6F6F,0x6F6F,0xC4C4,0x7383,0x8373,0xB393,0x9393,0x9393,0xC483,0x6373,0x6373,0x6363,0x6363,0x4252,0x5243,0x4243,0x4242,0x4742,0x6263,0x5263,0x5263,0x5262,0x5252,0x5252,0x5252,0x4752,0x5252,0x5247,0x4242,0x5242,0x6262,0x4242,0x4242,0x4241,0x5152,0x5252,0x4252,0x4242,0x9F9F,0x5252,0x9F9F,0x6161,0x9F9F,0x9F9F,0x9F9F,0x4141,0x9F9F,0x5050,0x9F9F,0xAFAF,0x6F6F,0x4242,0x4242,0x4252,0x6F6F,0x6262,0x6262,0x9F9F,0x5252,0x6262,0x4242,0x6242,0x4262,0xD3D3,0xD070,0x5070,0x4150,0x8F8F,0x4A4A,0x4848,0x5050,0x6060,0x9F9F,0x7070,0x9F9F,0x9F9F,0x7F7F,0x4242,0x6F6F,0x7F7F,0x4040,0x5050,0x4040,0xC0C0,0xCECE,0xBFBF,0xC0C0,0xC0C0,0xAFAF,0xC0C0,0xC0E0,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x8181,0x9292,0x92B1,0xB291,0xA1B2,0xB292,0xB1B1,0xB1B1,0xE0D1,0xD0E0,0xC0C3,0xB3C2,0xD0D0,0x81D3,0x9E91,0xD4D4,0xD0F3,0xAFAF,0x8080,0xD0D0,0x5C5C,0x4A4A,0x5C5C,0x8071,0x7171,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4242,0x7F7F,0x6F6F,0x7F42
.word 0x5757,0xD6D6,0xC4C4,0xC4C4,0x6161,0x7171,0x7070,0x6060,0x6060,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9E9E,0x9E9E,0xC0C0,0xD1C1,0xC3C3,0x6F6F,0x4242,0xB4B4,0x7362,0xB483,0xB493,0x8393,0x8383,0xB483,0x5273,0x6363,0x4352,0x5252,0x4342,0x4342,0x5242,0x4242,0x6245,0x5263,0x4343,0x4752,0x4752,0x6352,0x6363,0x5247,0x5252,0x6161,0x5252,0x5252,0x5252,0x5252,0x9F9F,0x4242,0x4242,0x5241,0x5252,0x4252,0x6F6F,0x9F9F,0x5252,0x4252,0x9F9F,0x5151,0x9F9F,0x5151,0x9F9F,0x4141,0x9F9F,0x9F9F,0x4160,0x4242,0x4242,0x4241,0x4252,0x6F6F,0x6262,0x6262,0x6161,0x6262,0x5252,0x4172,0x4242,0x4273,0xC3C3,0xD3D3,0x7070,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x5050,0xF0F0,0xF0F0,0xC0C0,0xC080,0x8080,0xC0C0,0x8080,0xAFAF,0xC0C0,0xD0D0,0x6060,0x7F7F,0x4141,0xB351,0xA182,0xA1B1,0xB292,0xB1B1,0xA1A1,0xB2A1,0xB1B1,0xC1B1,0xE0D1,0xD3E0,0xC181,0xC3C1,0xC371,0xAFAF,0xD0D0,0xD0E0,0x8080,0xD0C3,0x4E4E,0x7F7F,0x7F7F,0x7F7F,0xC0C0,0xC070,0x7171,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4A4A,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x6262,0x4949,0xD5D5,0xC4C4,0x7171,0x61B0,0x5050,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7E7E,0x8E8E,0x8E8E,0x8E8E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0xBE9E,0xC1C1,0xD1C1,0x52C2,0x7F7F,0x4242,0x9393,0x8383,0x9383,0x8393,0x8383,0x8383,0x9383,0x4263,0x4352,0x5263,0x4343,0x4242,0x4242,0x4242,0x4242,0x6363,0x6362,0x6262,0x6162,0x6161,0x6161,0x6262,0x6161,0x6161,0x6262,0x6161,0x6161,0x6161,0x6161,0x4252,0x4242,0x4242,0x5242,0x5142,0x4242,0x4242,0x9F9F,0x5151,0x5151,0x5252,0x4141,0x4141,0x9F9F,0x9F9F,0x9F9F,0x6060,0x5151,0x9F9F,0x4242,0x4242,0x5242,0x4142,0x5F5F,0x9F9F,0x6262,0x6161,0x5252,0x5151,0x4242,0x5252,0x4262,0xC0C0,0xD071,0x40C0,0x9F9F,0x5050,0x4040,0x9F9F,0x8F8F,0x7F7F,0x4848,0x7F7F,0x4A4A,0x8F8F,0x6F6F,0x7F7F,0x4242,0x7F7F,0x4040,0x8F8F,0xBFBF,0xC0E0,0xC0B0,0xC0D0,0xC1C0,0xC3B1,0xC0C0,0xC0C0,0x8080,0xD0D0,0xCFCF,0x4040,0x8F8F,0x8181,0x9282,0xB1B1,0x9292,0xB2B1,0xB2A1,0xA1A1,0xB1B2,0xB1B1,0xB1B1,0xE0D1,0xC3D0,0xD3D0,0x81B1,0x9F9F,0xD0D0,0xC0C3,0x7171,0x7171,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x6161,0x6161,0x8181,0x4242,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4646,0x4646,0x7F7F
.word 0x9F9F,0x9F9F,0x8B8B,0xD4D4,0xD4D4,0x7171,0x6060,0x7070,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0x7E7E,0xAFAF,0x8E8E,0xAFAF,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0x9E9E,0x9E9E,0xCEAE,0xD1E0,0xD1D2,0x62D2,0x4142,0x5151,0x7283,0x8383,0xB393,0x72B3,0x9393,0x8383,0x8383,0x5273,0x5263,0x4342,0x4242,0x4342,0x4352,0x4242,0x4242,0x6363,0x6262,0x5252,0x5252,0x7373,0x6262,0x6152,0x5252,0x5252,0x5252,0x5251,0x5252,0x6161,0x6161,0x4241,0x4242,0x5241,0x4152,0x4242,0x4242,0x7F7F,0x9F9F,0x6060,0x9F9F,0x6060,0x9F9F,0x8F8F,0x4141,0x5050,0x6060,0x9F9F,0x9F9F,0x5151,0x4242,0x4252,0x4251,0x4242,0x6F6F,0x6262,0x5151,0x9F9F,0x5151,0x6161,0xAFAF,0x5252,0x5252,0x6F6F,0xC0D3,0xC0C0,0x8F8F,0x8F8F,0x7F7F,0x4A4A,0x4848,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x4A4A,0x4242,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8171,0x71D0,0x7070,0x8070,0x7171,0xC3C3,0xC381,0x7171,0xB0B0,0xD0D0,0x4242,0x5151,0xC351,0xA182,0xB1B1,0x9292,0xB1B1,0xA1B2,0xB2B2,0xB2A1,0xB1B1,0xB1B1,0xC1B1,0xD0E0,0xD3D3,0xF2F2,0x6161,0xD0D0,0xD471,0xC272,0xAFAF,0x6E6E,0x4C4C,0x7F7F,0x7F7F,0xB071,0x6161,0x7271,0xDBE4,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4545,0x8F8F,0x7F7F,0x7F7F,0x4646,0x7F7F,0x7F7F,0x7F42
.word 0x5252,0x9F9F,0xAFAF,0xD4D4,0x6363,0x6161,0x7070,0xAFAF,0x8E8E,0xAFAF,0x9E9E,0x6060,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8E8E,0x9E8E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0xCEBE,0xD1D1,0xD2C2,0x82D1,0x5252,0xC361,0x7282,0x7293,0xB3A3,0x9393,0x8393,0x8393,0x9393,0x4263,0x5252,0x5263,0x4352,0x4252,0x4242,0x4242,0x4242,0x6363,0x6363,0x6363,0x4752,0x6352,0x6352,0x5252,0x6352,0x5252,0x6363,0x5252,0x5252,0x5252,0x5252,0x6363,0x4242,0x4242,0x4241,0x4142,0x4242,0x5151,0x5151,0x5252,0x5151,0x9F9F,0x6060,0x6060,0x9F9F,0x4141,0x8F8F,0x6060,0x6060,0x8F8F,0x4242,0x5242,0x5242,0x4242,0x6F6F,0x6060,0x9F9F,0x5252,0x9F9F,0x5050,0x4242,0x5252,0x4252,0x6F6F,0xD0D0,0xC0C0,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x4A4A,0xD5D5,0xBFBF,0xAFAF,0x9F9F,0x4242,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x7070,0x7070,0xD0C0,0xC0C0,0xC1C2,0xD2C1,0xD0D0,0x7171,0xD0D0,0x6161,0x8181,0xA182,0xB1B2,0x92A1,0xB2A1,0xB2B1,0xB2A1,0xB1B1,0xB2A1,0xB1A1,0xB1B1,0xD1B1,0xF2E2,0xF5F5,0xD361,0xC3D0,0x71C3,0x70B2,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6161,0x7171,0xB361,0xF0F0,0x6A6A,0x4242,0x6F6F,0x7F7F,0xF573,0x6B6B,0x6F6F,0x7F7F,0x7F7F,0x4747,0x7F7F,0x7F7F
.word 0x9F9F,0x4848,0xAFAF,0xAFAF,0x4848,0x62C5,0x5050,0x708E,0xAFAF,0x8E8E,0x6060,0x7E7E,0x7E7E,0x8E8E,0x8E8E,0x8E8E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0xBEBE,0xD0DE,0xD2B1,0xC2D1,0x6262,0xD072,0x8272,0x8282,0xB3B2,0x93B3,0x9393,0x8393,0x9383,0x5273,0x6363,0x5263,0x5243,0x5243,0x5263,0x4242,0x4742,0x7352,0x6262,0x6262,0x6162,0x6262,0x6262,0x6262,0x6262,0x6161,0x6161,0x6161,0x6161,0x6161,0x6161,0x4242,0x4242,0x4142,0x5252,0x4241,0x4242,0x7F7F,0x5151,0x5151,0x4141,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x6060,0x5050,0x8F8F,0x9F9F,0x5151,0x4242,0x4241,0x5251,0x4242,0x6F6F,0x9F9F,0x5151,0x8F8F,0x5151,0x9F9F,0x6060,0x4343,0x4252,0x6F6F,0xD0D0,0xD0D0,0x40F0,0x8F8F,0x7F7F,0x7F7F,0xD6D6,0xF6F6,0xF2F5,0xF3F2,0xE5F5,0xDFDF,0xCFCF,0x4242,0x4646,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5151,0x6161,0xDFDF,0xEFEF,0xE0E0,0xF0E0,0xE0F0,0xF0F0,0xE0F0,0xD0E0,0x7070,0xC360,0x9281,0xB1A1,0x92A1,0xB192,0xB1B1,0xB292,0xB1B2,0xB2B1,0xB1A1,0xB1B1,0xB1B1,0xD1D1,0xC2B2,0xD0D0,0x71D4,0x8272,0x8181,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4E4E,0xC0C0,0x8080,0xC371,0xBFBF,0x8F8F,0x6F6F,0x6F6F,0xB4B4,0xFFFF,0xCFCF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x9F9F,0xAFAF,0x4863,0x5252,0x9F9F,0x5151,0x5050,0x6060,0x9F9F,0x7070,0x9F9F,0x9F9F,0x8E8E,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0x8E9E,0x9E9E,0x9E9E,0x9E9E,0xCEBE,0xD1E0,0xC2B1,0xC2D1,0x8162,0xC3C3,0xB281,0xA392,0xB2B2,0xB3C2,0x9382,0x8372,0x9372,0x5273,0x6263,0x6363,0x6352,0x4352,0x4352,0x4252,0x4242,0x73B4,0x6373,0x5262,0x7373,0x6262,0x6262,0x6261,0x6161,0x6262,0x6262,0x5252,0x9F9F,0x6161,0x6262,0x4242,0x4242,0x4242,0x4142,0x4242,0x4242,0x7F7F,0x9F9F,0x9F9F,0x5151,0x5160,0x5050,0x5050,0x6060,0x9F9F,0x9F9F,0x7070,0x6060,0x4141,0x4242,0x4242,0x4152,0x4242,0x7F7F,0x8F8F,0x5151,0x9F9F,0x7070,0x9F9F,0x8F8F,0x4242,0x4252,0x6F6F,0x8080,0xBFBF,0xF0F0,0x4242,0x4848,0xC6C6,0xE7E7,0xF7F7,0xF7F7,0xF7F7,0xF7F7,0xEFEF,0xF3F3,0xF2F2,0xAFAF,0x4242,0x8F8F,0x7F7F,0x7F7F,0x4141,0xE4E4,0xF4D4,0xEFEF,0xF0F0,0xE0E0,0xD0D0,0xC1D0,0xC1D0,0xD0D0,0xF0E3,0xC0E0,0xC0C0,0x8282,0xA1A1,0x92A1,0xA192,0xB1B2,0xB2A1,0xB2B2,0xB1C1,0xB2B1,0xB1B1,0xB1B1,0xC1B1,0xC272,0xD3D3,0xC371,0xB282,0x7272,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7171,0x8080,0x8171,0xD0D0,0x8F8F,0x6F6F,0x6F6F,0xD142,0xCECE,0x9F9F,0x6F6F,0x4242,0x7F7F,0x4A4A,0x6F6F,0x6F6F
.word 0x9F9F,0xAFAF,0x6161,0x5151,0x4040,0x8F8F,0x6050,0x9F9F,0x7070,0xAFAF,0xAFAF,0x8E8E,0xAFAF,0xAFAF,0x9F9F,0x8E8E,0x8E8E,0x9E9E,0x9E9E,0x9E9E,0x9E8E,0x9E9E,0xAEAE,0xC1C1,0x9192,0x6060,0x8171,0x7181,0x81D0,0x8282,0xC392,0xB2B2,0x82A3,0x93A3,0x9393,0x5273,0x6363,0x6352,0x6363,0x5263,0x5263,0x4243,0x7342,0x6283,0x7362,0x7363,0x7363,0x5263,0x6352,0x5252,0x6252,0x5252,0x5252,0x5252,0x5252,0x6161,0x6161,0x4242,0x4141,0x5142,0x5252,0x4242,0x4242,0x8F8F,0x6151,0x6060,0x9F9F,0x9F9F,0x9F9F,0x5050,0x5050,0x9F9F,0x7070,0x7070,0x7070,0x6F6F,0x4142,0x4142,0x5252,0x4242,0x6F6F,0xAFAF,0x8F8F,0x6060,0x9F9F,0x6060,0x7F7F,0x4241,0x4242,0x6F6F,0xAFAF,0xC0C0,0xE2E2,0x8F8F,0xF4F4,0xF7F7,0xF7F7,0xF5F7,0xF7F7,0xF7F7,0xF7F7,0xF6F7,0xF5F7,0xEFEF,0xF4F4,0xAFAF,0x7F7F,0x6F6F,0x4141,0x8F8F,0xE5F3,0xF6F6,0xFFFF,0xD0D0,0x9281,0x8282,0x8282,0x8292,0xB2B2,0xC1B1,0xE0D1,0xD4D0,0x8282,0x9292,0x82A1,0x9282,0xB2B1,0xA1B2,0xF5D1,0xE1E1,0xB1B1,0xB1B1,0xB1C1,0xB1B1,0xC1B2,0xD3D3,0x82D4,0xC382,0x7272,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0xC0C0,0x71C0,0xC071,0x7070,0x7F7F,0x6F6F,0x6F6F,0x7171,0xC0C0,0x4242,0x8F8F,0x7F7F,0x4646,0x7F7F,0x4242,0x7F7F
.word 0x9F9F,0xAFAF,0x6161,0x5151,0x4040,0x5040,0x6050,0x9F9F,0x8E8E,0xAFAF,0xAFAF,0x8E8E,0x8E70,0xAFAF,0x8E8E,0x8E8E,0x9F9F,0x9F9F,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0xC0C0,0x7070,0x5151,0x7161,0xA181,0xBFBF,0x8281,0x8282,0xB282,0xB3B2,0xA392,0x8282,0xB393,0x5283,0x6252,0x7363,0x6352,0x6352,0x6363,0x4242,0x7342,0x6283,0x6283,0x6262,0x6262,0x7262,0x6262,0x6262,0x6262,0x6261,0x6262,0x6161,0x9F9F,0x6161,0x5151,0x4242,0x4242,0x5241,0x4242,0x4241,0x4242,0x9F9F,0x5151,0x6060,0x6060,0x9F9F,0x6060,0x9F9F,0x9F9F,0x6060,0x9F9F,0x6060,0x7070,0x7F7F,0x4242,0x5142,0x4152,0x4242,0x6F6F,0xAFAF,0x8F8F,0x6060,0x5050,0x8F8F,0x8F8F,0x6141,0x6162,0x7F7F,0xC0C0,0xBFBF,0xD0D0,0x9F9F,0xEFEF,0xF7F5,0xF5F7,0xE7F7,0xD7E7,0xF7E7,0xF9F7,0xD7D7,0xF7F7,0xEFEF,0xF6F6,0xD6D6,0x4242,0x7F7F,0x7F7F,0xF5F5,0xEFEF,0xF5F6,0xC1C1,0x9291,0xC2B2,0xB1C2,0xB1B1,0xA1B2,0xB192,0xB2A1,0xB1B1,0xE0D1,0x8281,0x8282,0x9292,0x9282,0xB1A1,0xB1A1,0xF2C2,0xD1F2,0xE1D1,0xC1D1,0xB1B1,0xB1B1,0xD3B1,0xC3C1,0xB3B2,0xC3B2,0xB3B3,0x6F6F,0x8F8F,0x8F8F,0x7F7F,0x4242,0xC0D0,0xC0C0,0xB0C3,0xBFBF,0x4040,0x4242,0x6262,0xC0D0,0xD0D0,0xF2D0,0x6A6A,0x4242,0x7F7F,0x4242,0x7F7F,0x6F7F
.word 0x8F8F,0x8F8F,0x5252,0x5050,0x5050,0x5050,0x5050,0x8E60,0x6060,0x6060,0x9F9F,0x9F9F,0x8E8E,0x7E7E,0x9F9F,0x9F9F,0x8E8E,0x8E8E,0x8E8E,0x8E8E,0x8E8E,0x8E8E,0x8080,0x7F7F,0x5050,0x9281,0xAEB1,0xB0B0,0xB292,0x8282,0xB272,0xB1C3,0x9282,0x82B3,0xB382,0x5283,0x6363,0x5273,0x6363,0x6352,0x5252,0x4263,0x8343,0x73B4,0x7373,0x8373,0x6262,0x6262,0x7162,0x7171,0x6262,0x6161,0x6161,0x6161,0x6161,0x6161,0x6261,0x7F7F,0x4242,0x5242,0x5241,0x4142,0x6F6F,0x6161,0x6161,0x5151,0x5151,0x6060,0x9F9F,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x5151,0x4242,0x4142,0x5142,0x4242,0x6F6F,0x508E,0x8F8F,0x9F9F,0x9F9F,0x9E9E,0x4040,0x5151,0x6162,0x5252,0x8E8E,0xAFAF,0xE0E0,0xF0F0,0xF5F5,0xF7F7,0xE7F7,0x59E9,0xC7DA,0xCBCB,0xE6DA,0xD7D7,0xD7E7,0xE5E5,0xAFAF,0xD6D6,0x9F9F,0xF463,0xE4E4,0xF6E6,0xF5F7,0xF3F3,0xB2E3,0xC2C2,0xD3D3,0xC3D3,0xB2B2,0xB1B2,0xA1B1,0xB1B2,0xB2B1,0xB1A1,0xB1E1,0x8272,0x8291,0x9281,0xB192,0xC1B2,0x92B1,0xB282,0xC1B1,0xD1D1,0xB1C1,0xB1B1,0xC2C1,0x82B3,0x92C3,0xB3B3,0x6293,0x8F8F,0x8F8F,0x5151,0xAFAF,0xE0E0,0xE0E0,0xD0D0,0xC172,0xF0F0,0xDFDF,0xF0F0,0xDFDF,0xDFDF,0xE0E0,0xF3E3,0xF3F3,0x8F8F,0x4242,0x7F7F,0x6F6F,0x427F
.word 0x8F8F,0x8F8F,0x8F8F,0x6060,0x5050,0x6060,0x6050,0x6050,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x7E7E,0x8E8E,0x9F9F,0x7E7E,0x9F9F,0x9E8E,0x9E9E,0x7070,0x8F8F,0x7F7F,0x8160,0xA182,0xAEBE,0x91A0,0x9282,0x8282,0xB181,0xC3D0,0x92B2,0x9292,0xB3B3,0x6293,0x6362,0x6352,0x6362,0x6373,0x6362,0x4252,0x8373,0x7262,0x8362,0x7362,0x6262,0x6273,0x6262,0x6262,0x6252,0x6262,0x5252,0x5252,0x5151,0x9F9F,0x6161,0x4242,0x4142,0x4242,0x4152,0x4242,0x4242,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x6060,0x6060,0x9F9F,0x7070,0x9F9F,0x7070,0x8080,0x9F9F,0x5252,0x5142,0x4242,0x4141,0x6F6F,0x5050,0x6060,0x6060,0x8F8F,0x8E8E,0x8E8E,0x4141,0x5252,0x4051,0x4040,0x9E9E,0xF0F0,0xF5F5,0xF7F7,0xF6F7,0xDBE7,0xD6D6,0x7B7B,0xAFAF,0xC7C7,0xE6EA,0xE6D6,0xCFCF,0xCFCF,0xAFAF,0xD5D4,0xDFDF,0xD0E3,0xE5E4,0xF6F6,0xD3D0,0x9282,0xC1C2,0xD3C2,0xB1B1,0x82B2,0xC3B3,0xB1C3,0xB1A1,0xB1B1,0xA1B1,0xD1B1,0x72A1,0x8292,0x8282,0xB192,0xC1B1,0xB292,0xA192,0xA192,0xB1A1,0xB1D1,0xB1C1,0xC2B1,0x82B3,0xB3B2,0x93B3,0xB372,0xE6E6,0xCFCF,0xF0D0,0xF0F0,0xF0F0,0xD0E0,0xD3D0,0xF0D0,0xDFDF,0xE0E0,0xE0E0,0xE0E0,0xCFCF,0xCFCF,0xE0E0,0xE4D3,0xDBDB,0x8F8F,0x4242,0x4642,0x7F7F
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x6050,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x7E7E,0x9F9F,0x9F9F,0x7E7E,0x8E8E,0x8E8E,0xAFAF,0x6060,0x7F7F,0x7F7F,0x6060,0x8182,0xAEA1,0xA0AE,0x9282,0x8292,0x8282,0xB182,0xD0D0,0xB2B2,0xB2B2,0xB2B2,0x5262,0x6273,0x6373,0x6363,0x7352,0x6263,0x4252,0x9352,0x6283,0x6283,0x6272,0x7272,0x6271,0x6262,0x6262,0x6262,0x6262,0x6262,0x6262,0x6262,0x6161,0x6262,0x6F6F,0x4242,0x4152,0x4252,0x4241,0x5151,0x6161,0x7061,0x6060,0xAFAF,0x9F9F,0xAFAF,0x6060,0x6060,0x6060,0x6060,0x7070,0xAFAF,0x4141,0x5242,0x4251,0x4251,0x4242,0x7F7F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9E9E,0x8E8E,0x4141,0x5252,0x5151,0x9E40,0x8080,0xBEBE,0xF3F3,0xF7F7,0xF7F5,0xF9F6,0xBBE6,0x4848,0x9F9F,0x4949,0xEAD6,0xF6E6,0xEBFB,0xCDCD,0xAFAF,0xD668,0xE4E4,0x7272,0xD3C3,0xE0E0,0xC1B3,0xE3C2,0xF5F5,0xE6F4,0xC3D4,0xC2C3,0x8291,0xB3B3,0xB1B2,0xB1B1,0xA1B1,0xA192,0x92D1,0x8271,0x8281,0xA192,0xB2B1,0xA192,0x92B1,0xB2A1,0x9292,0xB192,0xB1B1,0xB2B1,0xA382,0xB3B2,0x93B3,0x9383,0xE5E5,0xDFDF,0xBECE,0xBFBF,0xD0D0,0xE0D0,0xF0F0,0xF0F0,0xD0E0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xF1F0,0xF3F3,0xE0E0,0xDFDF,0xF3E4,0x7FF9
.word 0x6F6F,0x7F7F,0x7F7F,0x5050,0x5050,0x6060,0x5060,0x6050,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7E7E,0x7E7E,0x9F9F,0x7E7E,0x9F9F,0x4040,0x7F7F,0x6F6F,0x8140,0x8181,0xAEAE,0x91B0,0x9292,0x9282,0x8292,0xB182,0xE0E0,0xB2B1,0xB2B2,0xB2C2,0x6262,0x7362,0x6362,0x6262,0x6363,0x6362,0x5252,0x9373,0x6273,0x7383,0xC462,0x6262,0x6262,0x6161,0x7162,0x7171,0x6161,0x6161,0x6161,0x6161,0x6161,0x4242,0x4242,0x4242,0x5241,0x4251,0x4242,0x7F7F,0x5151,0x9F9F,0x6161,0x6060,0x7070,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x7F7F,0x4242,0x5251,0x4241,0x7F7F,0x8F8F,0x9F9F,0x5050,0x6060,0x9E9E,0x9E9E,0x6060,0x6F6F,0x4242,0x6F6F,0x7F7F,0x9E9E,0xBFBF,0xF6E3,0xF7F7,0xF7F7,0xF5F5,0xD5D5,0x9F9F,0xC3C3,0x7F7F,0x5858,0xBBBB,0xBBBB,0x8C8C,0xBFBF,0xD6D6,0xE5D6,0xD3D4,0xC371,0xD3D4,0xB2B2,0xB1B1,0xFDFD,0xEFEF,0xC3D4,0xC3C3,0xC3D4,0xB382,0x92B3,0xB1B2,0xB1B1,0x92A1,0xC1A1,0x7292,0x8272,0xB292,0x9292,0xB291,0x92B2,0xB1B1,0xC2A1,0xE2E2,0xD1D1,0xC2C1,0x82A3,0xA3A3,0x93C4,0x8372,0xEED4,0xCEE0,0xD0D0,0xC0C0,0xE0C0,0xF0F0,0xE0E0,0xD0D0,0xC0C0,0xB1C1,0xA1B1,0x81A1,0xC0B1,0xD3C0,0xD0D0,0xD0D0,0x81C1,0xD0D0,0xC0C0,0xD2D2,0x4AEF
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7E7E,0x9F9F,0x7E7E,0x9F9F,0x9F9F,0x7E7E,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8190,0xB181,0x9E9E,0x81A0,0x9292,0x9292,0x92A3,0xC182,0xD0D0,0xC1C1,0xC2B2,0xB2C2,0x6262,0x7373,0x6273,0x6373,0x6362,0x6362,0x4252,0x9373,0x7283,0x7262,0x6262,0x6262,0x6283,0x6262,0x6262,0x6162,0xD461,0x6161,0x6161,0x6161,0x6161,0x6161,0x4242,0x4241,0x5252,0x5242,0x4141,0x4242,0x7070,0x6060,0x6060,0x6060,0x9F9F,0x6060,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0x4242,0x5142,0x4251,0x4251,0x7F7F,0x8F8F,0x6060,0x9F9F,0x8E60,0x9F9F,0x7070,0x9F9F,0x5140,0x4251,0x4152,0x6F6F,0x8E8E,0xBFBF,0xD4C0,0xF7F7,0xF7F7,0xEFEF,0xF5F5,0xD3D3,0xE0E0,0x4861,0x9F9F,0xAFAF,0x9F9F,0x8B8B,0xDCDC,0xE6D6,0xF6F6,0xF7F5,0xC3E4,0xD3D3,0xC2C3,0xE3C2,0xF0F0,0xCFCF,0x71C0,0x71D4,0xE6E6,0x92D4,0x8282,0xB2B2,0xA1B1,0x92B1,0x9292,0x81C1,0x8172,0xA182,0x8292,0xA192,0x92B1,0xB192,0xF3B2,0xFFFF,0xF0F0,0xF0F0,0xB2E0,0xB3B2,0x93B3,0xC383,0xF0F0,0xC0C0,0xB0B0,0xD080,0xE0E0,0xE0E0,0xC0D0,0xB1C1,0xC1C1,0xB1C1,0xB1C2,0xC1C2,0xC2C1,0xC3C3,0xE1E1,0xB1D1,0x8182,0xB071,0x8181,0xC3C2,0x7F8F
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7E50,0x9F9F,0x9F9F,0x7E7E,0x7E7E,0x407E,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7070,0x7281,0x9E81,0x909E,0x9281,0x92B3,0xB2B3,0x82B3,0xB192,0xDEDE,0xB1B1,0xC1C1,0xB2D1,0x6262,0x6283,0x7373,0x7362,0x7362,0x6373,0x5252,0x8373,0x6283,0x7272,0x7262,0x6271,0xD462,0x6261,0x6262,0x7373,0x6161,0x6262,0xAFAF,0x5252,0x6161,0x4242,0x4242,0x5142,0x5152,0x5242,0x4242,0x7F7F,0xAFAF,0x6161,0x9F9F,0x9F9F,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x6060,0xAFAF,0x5050,0x4141,0x5241,0x5251,0x4241,0x6F6F,0x8F8F,0x6060,0x9F9F,0x6060,0x9E9E,0x7070,0x8080,0x5151,0x6F6F,0x4141,0x5040,0x6060,0xBEBE,0xC0C0,0xF7F7,0xF7F7,0xF7F7,0xFAEB,0xD0E4,0xD0D0,0xCFCF,0x5252,0xAFAF,0x9F9F,0x7F7F,0xDCBC,0xF7F6,0xF7F7,0xF7F7,0xD4EB,0xD4C3,0xE3D3,0xE3D3,0xD0D0,0xE0E0,0xC0C3,0xC3C0,0xC1C1,0x7171,0x82B2,0xA182,0xB1A1,0xA1A1,0x9192,0xC1B2,0x7272,0x9282,0x8282,0x9292,0x92B2,0x9282,0xD2B2,0xE3F3,0xD192,0xC1D1,0xE0E1,0xE0F0,0x72D4,0xE093,0xEEEE,0xD0D0,0xC0C0,0xF0D1,0xE0F0,0xD0D0,0xD1D1,0xE1D1,0xE1E1,0xF0F1,0xF0F0,0xF1E1,0xF0F1,0xF2F1,0xD0F2,0x92A1,0xB192,0xB2B2,0x71B1,0x9F9F,0x6F6F
.word 0x7F7F,0x4040,0x4040,0x5050,0x5050,0x8F8F,0x5050,0x5050,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7E7E,0x9F9F,0x4040,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7160,0x8181,0x9E9E,0x91B0,0x9292,0xB392,0x82B3,0x92B3,0xD092,0xE0CE,0xC1C1,0xD1B1,0xB2D2,0x8372,0x8383,0x7383,0x7373,0x6273,0x6273,0x5252,0x8383,0x7262,0x8383,0x6262,0xC462,0x6161,0x7271,0x6161,0x7162,0x7272,0x7272,0x6162,0x6161,0x6161,0x4252,0x4242,0x5241,0x5252,0x5251,0x4142,0x8F8F,0x6070,0x7070,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x9E70,0x7070,0x6060,0x4141,0x5152,0x4142,0x4142,0x6F6F,0x9F9F,0x9F9F,0x6060,0x6060,0xAFAF,0x608E,0xAFAF,0x6F6F,0x6F6F,0x6F6F,0x8080,0x7070,0xC070,0x7171,0xF7D2,0xE7F5,0xDEDE,0xEEEE,0xFBFB,0xD0D0,0xC0C0,0xE0E0,0x6060,0x9F9F,0x8F8F,0x7F7F,0xF7FA,0xE7EA,0xFAFA,0xFBFB,0xCFCF,0xC3C3,0xD4D0,0xD3C3,0xD072,0x9371,0xC0C3,0xC0C2,0xDFDF,0x92D0,0x8282,0xB1B2,0xB2B1,0x92A1,0xA182,0x72B1,0x8281,0x8281,0x9292,0x91B2,0x9282,0x92A1,0xB2B2,0xB1B1,0xB1B1,0xC2B1,0xE1E1,0x72D0,0xD1D1,0xCFCF,0xD0D0,0xE0B2,0xE0E0,0xE0E0,0xD1E1,0xD1D1,0xD1D1,0xD1C1,0xF0D1,0xE1E0,0xD1D1,0xD1D1,0xB1C1,0xA1B1,0xB192,0x7171,0x7F7F,0x7F7F,0x6F6F,0x6F7F
.word 0x4040,0x7F7F,0x4040,0x5040,0x4040,0x5050,0x5050,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7E50,0x7E7E,0x9F9F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8171,0x9181,0x9E9E,0xB381,0xB392,0xB2B3,0xB392,0x9292,0xD0B2,0xD0CE,0xD1C1,0xE1C1,0x82D1,0x8372,0x8383,0x7383,0x6273,0x7373,0x7373,0x6352,0x83C3,0x6272,0x6272,0x6172,0x62C4,0xC461,0x6262,0x6262,0x7171,0x6161,0x6161,0x6161,0x7171,0x9F9F,0x5151,0x4242,0x5252,0x5152,0x4252,0x4141,0x7070,0x9F9F,0x6060,0x6060,0x9F9F,0x7070,0x6060,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x8F8F,0x5252,0x4151,0x4152,0x5252,0x4040,0x708E,0x7070,0x9F9F,0x7070,0x9E9E,0x7070,0xAFAF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8080,0x7070,0xC0C0,0xF371,0xFBFB,0xCEBC,0xFDFD,0xFBFB,0xDFDF,0xDFDF,0xEFEF,0x7070,0x5252,0x8F8F,0xCBCB,0xEBFA,0xDBDB,0xF8E9,0xF8FA,0xE9E8,0xCFCF,0xC1C3,0x71C2,0x72C3,0xE0D3,0xE0E0,0xE0E0,0xE0E0,0xD0D0,0x8292,0xB292,0xA1B1,0x92A1,0x8281,0x92A1,0x8172,0x8272,0x9292,0x9292,0x8282,0xA192,0xB192,0xD1C1,0xC1C1,0xB1B1,0xE0C1,0x82C3,0xE0E0,0xE0E0,0xD0E0,0xE0E1,0xE0D0,0xE1E0,0xD1D1,0xC1C1,0xA1C1,0xC1B1,0xD1C1,0xD1C1,0xD1C1,0xC1C1,0xA1B1,0x9292,0xC0B2,0x7F7F,0x7F7F,0xC442,0xD5D5,0x7F42
.word 0x7F7F,0x4040,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7E7E,0x5050,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7070,0x8181,0x9E92,0xA0AE,0x92A1,0xB3B2,0xB392,0xB392,0x92B3,0xE0B1,0xC0DE,0xD1D1,0xE0D1,0x82D2,0x9383,0x8383,0x6283,0x7383,0x7373,0x6262,0x6252,0x7283,0x8362,0x8362,0xC462,0x7161,0x6262,0x6262,0xD462,0x6262,0x6262,0x6161,0xAFAF,0x6161,0x6161,0x6161,0x4242,0x5251,0x5251,0x4142,0x4242,0x9F9F,0x7070,0xAFAF,0x9F9F,0x7070,0xAFAF,0x6060,0x7070,0x7070,0x708E,0xAFAF,0x7070,0x5060,0x5150,0x4251,0x4251,0x4141,0x6F6F,0x9F9F,0x9F9F,0x7070,0xAFAF,0xAFAF,0x8E70,0x4040,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x8080,0xC0C0,0xD0D0,0xC3C3,0xFCF0,0xF3F3,0xEFEF,0xEFEF,0xFBFB,0xEFEF,0xE0E0,0xAFAF,0x6060,0x9F9F,0xECEC,0xEBEB,0xDBEA,0xEAEA,0xEBF9,0xEAEA,0xEAEB,0xF6F6,0xC3F3,0xC0C0,0xDFDF,0xE0E0,0xD0D0,0xE0E0,0xE0E0,0x92E0,0x9282,0xB2A1,0x92A1,0x7292,0x8282,0x7282,0x8182,0x9282,0x92A1,0x9192,0x9282,0x8292,0xC1C2,0xB1C1,0xC1C1,0xD1B1,0xD181,0xDFDF,0xE0E0,0xE0E0,0xE0E0,0xE1F0,0xD1D1,0xB1C1,0xB1C1,0xB1B1,0xB1B1,0xA1B1,0xA1B1,0xB1B1,0xB1C1,0x92A1,0x92A1,0x7171,0x7F7F,0x7F7F,0xA3A3,0x6A6A,0x6F6F
.word 0x7F7F,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x8F8F,0x7E7E,0x8F8F,0x7E7E,0x5050,0x5050,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x9040,0x8281,0xAEC1,0xA0A0,0xB3B2,0xB292,0xB2B3,0x92B3,0x8292,0xB1B1,0xD0DE,0xC1C1,0xF0D1,0x72C1,0x9393,0x8393,0x8383,0x7383,0x8362,0x7373,0x8352,0x83A3,0x7272,0x7272,0x6262,0xC462,0x7161,0x7171,0x6171,0x7272,0x6262,0x7171,0x6162,0x6161,0x7070,0x4242,0x4241,0x4251,0x5152,0x4252,0x6F6F,0x7070,0x7070,0x7070,0x7070,0x8E8E,0x9E8E,0x709E,0x7070,0x6060,0x9E70,0x9E70,0x6060,0x6060,0x5151,0x4141,0x4241,0x4141,0x6F6F,0xAFAF,0x7070,0x709E,0x7070,0xAFAF,0x7070,0xAFAF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7070,0xD080,0x7171,0xCFCF,0xF292,0xFFFF,0xF5F5,0xDFDF,0xF3F3,0xBEBE,0xDFDF,0xAFAF,0x9F9F,0x8F8F,0xCBBB,0xBCCC,0xACAC,0xCBCC,0xDCCC,0xFAFA,0xDCF7,0xEBEB,0xAFAF,0xF3F3,0xD0D0,0x81C0,0x81C3,0xC0C3,0xD0C1,0xE0E3,0x82C1,0xB292,0x92A1,0x8192,0x7172,0x8192,0x8272,0x8282,0xA1B2,0x8282,0x9292,0x8292,0xC1C2,0xB1B2,0xB1B1,0xC1C1,0xE0C2,0xE0E0,0xE0E0,0xF0E0,0xF0F0,0xF0F0,0xC1D1,0xB1C1,0xB2B1,0x92A1,0x9292,0xB292,0xC2B1,0x92B2,0xA192,0x9292,0xA192,0x6161,0x7F7F,0x4342,0xD0C1,0xC1C1,0x7F6F
.word 0x4040,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x7E7E,0x7E40,0x9F9F,0x8F8F,0x4040,0x4040,0x4040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8270,0x9181,0x9E9E,0x92A0,0xB292,0x92B3,0x92B3,0xB2B3,0x92B3,0xE0B1,0xC1C1,0xD1C1,0xE0E0,0x72B1,0x9393,0x9393,0x8383,0x8383,0x8373,0x6273,0x8362,0x72A3,0x7272,0x7272,0xC471,0x6162,0xC462,0x6262,0x7171,0x7171,0x7171,0x7171,0x7070,0x7061,0x7171,0x4242,0x4142,0x5252,0x5251,0x4241,0x7F7F,0xAFAF,0x6060,0x7070,0x7070,0x8E80,0x9E9E,0x9E9E,0x9E9E,0x9F9F,0x7070,0xAFAF,0xAFAF,0x4141,0x5261,0x5251,0x4241,0x4141,0x8E40,0x7070,0x7070,0x7070,0x7070,0x7070,0x708E,0x5050,0x4141,0x4141,0x7F7F,0x8040,0x8080,0x8080,0xC381,0xE0E0,0xB2D3,0xF0F2,0xFEFE,0xF7F7,0xF0F0,0x9F9F,0xBEBE,0xCFCF,0xAFAF,0x9F9F,0x9BCB,0x8B8B,0x4A4A,0x9E9E,0x9E9E,0xEBCC,0x9CEC,0x8F8F,0x6161,0xD0D0,0x7171,0xB282,0xB291,0xB281,0xB282,0xC1A1,0xD1D1,0x92B1,0x9192,0x8182,0x7272,0x7292,0x8272,0x9281,0xB2B2,0x8281,0x9281,0x8281,0xB1B2,0xB1A1,0xB1C1,0xB1B1,0xE0C1,0xE0E0,0xCFCF,0xF0F0,0xF0F0,0xD1E0,0xC1D1,0xB2B1,0x9292,0x9292,0xB2A1,0xC1B1,0xA1B1,0x8181,0x9272,0xA192,0xB292,0x71C2,0xD4D4,0xC1C1,0x81C2,0x8181,0xC4AF
.word 0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x8181,0xB081,0x9E9E,0xB291,0xB3B2,0xB392,0xB3B2,0xB392,0x8292,0xD0B1,0xD0DE,0xE0C1,0xE0EE,0x9392,0x9393,0x9383,0x8393,0x8383,0x6283,0x6273,0x8362,0x72A3,0x8372,0x8372,0x7172,0x7172,0x61D4,0x6262,0x6262,0x6262,0xAFAF,0x6261,0x9F9F,0x6161,0xAFAF,0x4141,0x5242,0x5152,0x4152,0x4152,0x6F6F,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x9E9E,0x9E9E,0x8E9E,0x7070,0x6060,0x7070,0xAFAF,0x4140,0x5151,0x4141,0x4142,0x6F6F,0x8F8F,0xAFAF,0x7070,0xAFAF,0x7070,0x8E70,0x7070,0xAFAF,0x5252,0xA383,0x4272,0x4040,0x8080,0xC0C0,0x72C3,0xC1D1,0xD0D0,0xC272,0xEAE0,0xEBEB,0xD5D5,0xE6E6,0xAFAF,0xFEFE,0xCECE,0x9C9C,0xABAB,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9C9C,0x8D8D,0x6161,0xEFEF,0xC0C0,0xB291,0x71C1,0xB1B1,0x92B2,0x91B2,0x9292,0xB192,0xD1D1,0x9292,0x7282,0x8271,0x7182,0x8172,0x8272,0xB1B1,0x9282,0x8282,0x8282,0xC1B2,0x9292,0xB1B1,0xB1C1,0xDEC1,0xE0D0,0xF0F0,0xF0F0,0xE1E0,0xC1D1,0xB2B1,0xB2B2,0x9292,0xC1B2,0xC1C1,0xE1E0,0xE0E0,0x7F7F,0x8282,0x92B2,0xA192,0xD092,0xC0C0,0xD0D0,0x8181,0xB3C3,0xF2D0
.word 0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6060,0x8181,0xAE92,0x90AE,0x8291,0x92B3,0x92B3,0x92B3,0x92B3,0x9282,0xD0A1,0xC0C0,0xE0C1,0xC1C1,0x9381,0x9393,0x9393,0x9383,0x8383,0x8362,0x6283,0x9362,0x93B3,0x7272,0x7282,0xC472,0xC471,0x7161,0x7271,0x7171,0x6262,0x6262,0x6262,0x6162,0x7171,0x6161,0x4242,0x5142,0x5241,0x5251,0x4251,0x7F7F,0x6060,0x7070,0x7070,0xAFAF,0x7070,0x9E70,0x9E9E,0x9E9E,0x709E,0x9F9F,0x9E9E,0x5050,0x4040,0x5151,0x5142,0x4141,0x6F6F,0x9F9F,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x9E9E,0x5050,0x6F6F,0xA352,0x4273,0x7F7F,0xC080,0x8181,0x71C3,0x8282,0xD1B2,0x61D3,0xD2B3,0xEAEA,0xF7F7,0xF7F7,0xD5D5,0xFEBE,0xCFCF,0x6E6E,0x9F9F,0x9F9F,0x7F7F,0x8F8F,0xAFAF,0x9F9F,0x9F9F,0x5151,0xDFDF,0x91C0,0x8181,0x7F7F,0x5151,0xB281,0xB1B2,0xB2C2,0x8292,0xC191,0x82B1,0x8182,0x7272,0x7282,0x7272,0x7282,0xB1C1,0x8282,0x8282,0x8182,0xB192,0x9292,0xB1B1,0xB1C1,0xE0C1,0xE0E0,0xF0F0,0xE0F0,0xD1D1,0xB1C1,0xB2B1,0x92A1,0xC192,0xC1D0,0xC1B1,0xD0E0,0xD1D1,0x4343,0xC1C2,0xB292,0x9292,0xA182,0x70C0,0xC3C0,0xC381,0x8171,0xC281
.word 0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8080,0x8281,0x9EA1,0xA0A0,0xB2B2,0xB392,0xB292,0x92B3,0xB3B2,0x9292,0xD1B2,0xC1D0,0xEED1,0xA1A1,0x9382,0xA393,0x9393,0x8393,0x8383,0x8383,0x6262,0xA373,0x7293,0x7293,0x8293,0x7272,0x7171,0x7272,0x7272,0x7172,0x7171,0x7171,0x7171,0x7070,0x7171,0x4141,0x4242,0x4251,0x5251,0x5251,0x4141,0x7F7F,0xAFAF,0x7060,0xAFAF,0x7070,0x7070,0x7070,0x9E9E,0x8E9E,0x8E70,0x9F9F,0x8E8E,0x9F9F,0x5151,0x5151,0x4251,0x4242,0x6F6F,0xAFAF,0x7070,0x9E70,0x7070,0x9E70,0x8080,0x60C0,0xAFAF,0x5252,0x6252,0x4141,0xC041,0xB0B0,0xC371,0xC2C3,0xC2C3,0xC2C2,0xD1D1,0xA3D2,0xD1C2,0xF7F7,0xF7F5,0xE9E9,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x4242,0x8F8F,0xCFCF,0x7F7F,0x7F7F,0xD0D0,0xB0B0,0x71B2,0x7F7F,0x7F7F,0x6F6F,0xD1D1,0xF0F0,0xE0E0,0x92C1,0x8182,0x8282,0x7282,0x7172,0x7172,0x7272,0x7281,0xB1C1,0x8182,0x8182,0x8282,0xB1B2,0x9292,0xB1B2,0xB1B1,0xF0D1,0xE0F0,0xE0F0,0xD1D1,0xB1C1,0xC2B1,0xC2B1,0xB2C2,0xD0B1,0xB1B2,0x9292,0xD1B2,0x81D1,0xD0C3,0xD1E1,0xB1B2,0x92A1,0x9292,0xE3D2,0xC282,0xC371,0x82C0,0x8F71
.word 0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8160,0xA081,0xAEAE,0x92A0,0x92B3,0x92B3,0xB3B3,0xB392,0x82B3,0x9292,0xC1A1,0xD1D0,0xEED0,0x82D0,0xA372,0x9393,0x8393,0x8393,0x8383,0x8383,0x6273,0xB372,0x72A3,0x8272,0xC371,0x7272,0x72C3,0x7271,0x7272,0x6161,0x7171,0x6161,0x7171,0x7171,0x6161,0x4242,0x4241,0x5251,0x5251,0x4251,0x4141,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x9E9E,0x9E9E,0x708E,0x9E9E,0x8E8E,0x4040,0x4141,0x5151,0x5152,0x4141,0x5050,0x708E,0xAFAF,0x7070,0x708E,0x8080,0x8080,0xBFBF,0x4060,0x5141,0x5252,0x4252,0xC051,0x7171,0x71C3,0xC381,0xD3C0,0xD3D0,0xD1C2,0xF2F2,0xB3D3,0xD0D0,0xF6F7,0xF6E6,0xD0D0,0x9F9F,0x5151,0x8F8F,0x5151,0x9F9F,0x7F7F,0x9F9F,0x7F7F,0xD0D0,0xC0C0,0xC1C0,0x7070,0x7F7F,0x7F7F,0x6F6F,0xD0D0,0xE0E0,0xD1D0,0xD1D1,0x82A1,0x7272,0x7281,0x7172,0x7262,0x7271,0x7272,0xA1C2,0x8282,0x8282,0x8281,0xB192,0x9292,0xB1B2,0xB1B1,0xF0E1,0xE1E0,0xD1D1,0xC1C1,0xB1B1,0xC2C1,0xD1D1,0xB2D1,0xB1D1,0x9292,0x8192,0xF592,0xD0D0,0xD0D0,0xC1D1,0xB1B2,0xA1B2,0x8292,0xD2D2,0x7272,0x71C0,0x7172,0x7F8F
.word 0x5050,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8170,0xA181,0x9E9E,0x9281,0x92B2,0xB3B2,0x92B2,0xB3B2,0x92B2,0x9292,0xB1A1,0xC1C1,0xEEE0,0x82B1,0xA3A3,0x9393,0x9393,0x9393,0x8393,0x8383,0x6283,0x8272,0x72A3,0x8282,0x8172,0x7272,0x7171,0x71D4,0x71C3,0x7171,0x71D4,0x7171,0x7171,0x6161,0x7171,0x4161,0x4142,0x5142,0x5152,0x5152,0x4141,0x8E50,0x7070,0x7070,0x7070,0x708E,0xAFAF,0x7070,0x7070,0x9E9E,0x8E8E,0x709E,0x9E9E,0x4150,0x4241,0x5142,0x4161,0x4242,0x8E8E,0x8E70,0x7070,0x7070,0x7070,0x8080,0xC0C0,0xC0C0,0x8080,0x5252,0x6252,0x4142,0xD4D4,0x7181,0xC0D4,0xB1D3,0xC2C2,0xD3C2,0xC2D3,0xF1D1,0xF2F0,0xBFBF,0xEAEA,0xF6F6,0xF9F9,0xE0F0,0x9F9F,0x7F7F,0x8F8F,0x5151,0x4242,0x8F8F,0x7F7F,0xAFAF,0xC1F0,0x50C2,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0xE0E0,0xD0D0,0x8181,0x9182,0x92B2,0x8281,0x9282,0x8292,0x7172,0x7162,0x7172,0xB1B1,0x9192,0x8292,0x8272,0xB291,0x9292,0xB2A1,0xD1B1,0xE1E1,0xE1E1,0xC1D1,0xB1C1,0xC2C2,0xC1C1,0xD1C2,0xC2B2,0x92E0,0x9292,0x82A1,0x7282,0xE0E0,0xB1C0,0xD1D1,0xB1B2,0xB2B1,0x4171,0xEBC4,0xC3C3,0x7272,0xC371,0x7F8F
.word 0x5050,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x5050,0x72B0,0xD081,0x80AE,0xB2B0,0xB3B3,0xB392,0xB392,0x92A3,0xA3B3,0x9292,0xC192,0xD1C1,0xDEDE,0x81A1,0x9393,0xA3A3,0x9393,0x8393,0x9383,0x8383,0x6283,0xB393,0x8282,0x8282,0xB372,0xC371,0xC372,0x7161,0x7272,0x7171,0x6161,0x7171,0x7171,0x7171,0x6170,0x4242,0x4241,0x5251,0x5241,0x4251,0x4141,0x7070,0x8E70,0x7070,0x7070,0x7070,0x7070,0x7070,0x9E70,0x8E9E,0x7070,0x7070,0x7070,0x4040,0x4241,0x5141,0x5152,0x4141,0x8F8F,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0x8080,0x8080,0x7070,0x5242,0x5152,0x4242,0x7161,0xC371,0xC371,0xC3C0,0xC1C2,0xC1C2,0xD1C1,0xD1D2,0xE2E2,0xC3F0,0x9F9F,0xBCBC,0xF9F9,0xFEFE,0xAFAF,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4242,0x8F8F,0x8F8F,0xD0D0,0x7F7F,0x8F8F,0x4141,0x4141,0xD151,0xE0E0,0xD1D0,0xD1D1,0xC1D1,0xB1C1,0xF1D2,0xF0F0,0xF0F0,0xE1E1,0x92D1,0x7172,0xB1B2,0x9292,0x9192,0x7282,0xA192,0x9292,0xB292,0xC1B1,0xD1C1,0xD1D1,0xC1C1,0xC1C2,0xE2E2,0xC2C1,0xC2C2,0xB2B2,0xB1C1,0xA182,0x8281,0xD181,0xE0E0,0xE1E1,0xC1E0,0xB1B2,0xB1B1,0xB2B2,0x7F7F,0x8282,0x71C0,0x71C3,0x7F8F
.word 0x6060,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x4040,0x4040,0x7070,0x8181,0x9E92,0xB09E,0x9292,0xB392,0xB3B2,0x92B3,0xB3B3,0x9292,0x9292,0xB192,0xD0C1,0xD0DE,0xA391,0xA393,0x93A3,0x9393,0x9393,0x9393,0x8393,0x7262,0xB393,0x7282,0x7282,0xB381,0xC371,0x7171,0x72D3,0xC381,0x7171,0x7171,0x7171,0x6161,0x7171,0x7171,0x4242,0x5241,0x5151,0x5152,0x4151,0x6F6F,0x8E8E,0x6070,0x7070,0x708E,0x708E,0xAFAF,0x7070,0x8E8E,0x9E9E,0x708E,0x708E,0xAFAF,0x4140,0x5151,0x5142,0x5251,0x4141,0x8F8F,0xAFAF,0x7070,0x7070,0x808E,0xAFAF,0x8080,0xC0C0,0x4141,0x5142,0x5252,0x4141,0x7272,0xC372,0xC371,0xC371,0xC2C3,0xC1D3,0xD2C2,0xD2D2,0xD1D1,0xF0E1,0xF3F3,0xBFBF,0x8C8C,0xBCBC,0xC5C5,0xAFAF,0x9F9F,0x6363,0x4747,0x8F8F,0x4747,0x8F8F,0xAFAF,0x7171,0xD0D0,0xC3C0,0xD3C0,0xD0D0,0xD1D0,0xD0E0,0xD0D1,0xD1D1,0xF0F0,0xF0F0,0xE0E0,0xD1D1,0xE0D1,0xF1E0,0xC2D1,0xC1B1,0x92B1,0x9291,0x7281,0xB292,0x9292,0xB2B2,0xB1B2,0xC1C1,0xC1C1,0xC2B1,0xD2C1,0xEFEF,0xD1E1,0xB2D1,0xC2C2,0x92B2,0x8192,0xC292,0xF0E0,0xD1D1,0xD1C1,0xD1E1,0xB1C1,0xB1B1,0xE0C2,0xCFCF,0x7272,0x71C3,0x8181,0x7F9F
.word 0x6070,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7171,0x8181,0xAEA1,0x819E,0xB392,0x92B2,0x92B3,0xB3B2,0x9292,0x82B3,0x9292,0xB1A1,0xE0C1,0xD0DE,0x8282,0x9393,0xA3A3,0x93A3,0x9393,0x9383,0x8383,0x8383,0xB382,0x82A3,0xC382,0xC371,0x7271,0xC372,0x7171,0x61C3,0x7271,0xD4C0,0x7171,0xD471,0x7171,0x4171,0x4142,0x5142,0x5252,0x5251,0x4251,0x4141,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x9E9E,0xAFAF,0x6060,0x7070,0x5151,0x5152,0x4251,0x5151,0x6F6F,0xAFAF,0x7070,0xAFAF,0xAFAF,0x8080,0x7070,0xC0C0,0xC0C0,0x4141,0x5241,0x5261,0x4242,0xD371,0xC072,0xC071,0xC3C3,0xC3C0,0xC2C3,0xC2C3,0xC2C2,0xD1D2,0xE1D2,0xF0E1,0xEFEF,0x71E3,0x9F9F,0x9E8C,0xF7DB,0xF6F6,0xD7E7,0xC6C6,0x4848,0xC6C6,0x4848,0x5252,0xE0F3,0xC0C0,0x7171,0x7161,0xD3C3,0xC1D3,0xC3B1,0x8181,0xF1B2,0xF0F0,0xD1E0,0xB1C1,0xA1B1,0xB1A1,0xC1B1,0xE0D1,0xC1E1,0xA1A1,0x8292,0x8182,0xB282,0x9292,0xA1B2,0xB292,0xB1B1,0xB1C2,0xC1C2,0xB1C1,0xC1C2,0xE1E1,0xE1F0,0xE1E1,0xB2C2,0xC292,0xF0F0,0xC1D1,0xC1C1,0xE1D1,0xD1F0,0xC1C1,0xB2B1,0xE0C1,0xE0E0,0xE0E0,0xD4C0,0xC3C0,0x6171
.word 0x6060,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8040,0x8171,0x9EC0,0x9190,0x9282,0xB382,0x8292,0x92B3,0xB3B3,0x9292,0x9292,0xB192,0xB1B1,0xC0DE,0x8292,0xA393,0xA393,0x93A3,0x9393,0x8393,0x9393,0x6262,0xB3A3,0x82A3,0x8182,0x71C3,0xC371,0x71C3,0xD471,0x7171,0x71D4,0x6161,0x7272,0x8080,0x7171,0xAFAF,0x4142,0x5152,0x5152,0x5151,0x5142,0x6F6F,0xAFAF,0x7070,0x7070,0x7070,0x708E,0x7070,0x8E8E,0x7070,0x8E9E,0x8E8E,0x8E70,0x8080,0x5141,0x5142,0x4152,0x5152,0x6F6F,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0x8080,0xAFAF,0xC0C0,0x4141,0x6252,0x4152,0x4142,0x71D4,0x71D0,0xD3D3,0xC071,0xC3C3,0xC3C0,0xC3C3,0xC3C2,0xC2B2,0xD1D1,0xD1C1,0xF0E0,0xEFEF,0xE0F3,0xBFBF,0xAFAF,0xBC8D,0xDACB,0xE8F9,0xD5E7,0xD7D7,0xC7C7,0x8F8F,0xC2C2,0x8291,0x8272,0x8182,0x9181,0x9281,0x8182,0xC271,0xF0F0,0xD1E0,0xB1C1,0xB2B2,0xA192,0xA192,0xA1A1,0xB1A1,0xC1D1,0x9292,0x8292,0x7282,0xB182,0x9282,0x9292,0x9292,0xB292,0xC2B2,0xC1C1,0xB2C2,0xB2B2,0xC2B1,0xD1C1,0xF2F2,0xE1F0,0xE0E1,0xC1D1,0xC1C1,0xC1D1,0xE1C1,0xD1F0,0xC1D1,0xB1C1,0xF0B2,0xF0F0,0xD0D0,0xC0C3,0xC0D4,0xE4C3
.word 0x5050,0x4040,0x7F7F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8140,0x8281,0x9EB1,0x8180,0xB392,0x9292,0xB2B3,0xB392,0x92B2,0x92B3,0x9292,0xB1A1,0xC1C1,0xC1DE,0x8292,0xA393,0xA3A3,0xA3A3,0x9393,0x9393,0x8383,0x9383,0xB3B3,0x82A3,0xB3A3,0xC372,0x71C3,0xC371,0x71D4,0xE471,0xD480,0x7171,0x7171,0xD471,0x7171,0x5252,0x5241,0x5251,0x5251,0x5251,0x4141,0x6F6F,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x708E,0x7070,0x8E8E,0x9E9E,0x9E70,0x7070,0x8080,0x4242,0x5151,0x5251,0x4251,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0x8080,0x8080,0xAFAF,0x4270,0x5251,0x5261,0x5241,0xC0C0,0xC3C3,0xC0C0,0xD4D0,0xD0C0,0xC0D3,0xC3C0,0xC3C1,0xD1C1,0xE0E0,0xD1D0,0xD1D1,0xE1D1,0xF1F0,0xF0F0,0xF0F0,0xBFBF,0x9F9F,0x7E4E,0xDB8D,0xDAD6,0xAFAF,0x8F8F,0x5151,0x8F8F,0x8F8F,0xD341,0x81C1,0xA1B2,0x7281,0xD1D1,0xE0E0,0xB1D1,0xB2B2,0xB1B1,0xB1C1,0x92B1,0x9292,0x9292,0xD1B1,0x9292,0x8282,0x8281,0x9282,0x9292,0x9292,0x9292,0x9292,0xC1B2,0xB2C1,0xB1B2,0xC2C1,0xC2C2,0xD2C2,0xF1F1,0xD1D1,0xE1E1,0xC1E1,0xD1C1,0xB1C1,0xC2B1,0xD1F0,0xC1C1,0xB2B1,0xE1B2,0xE0E0,0xD0D0,0xD0C1,0xC3D0,0xD582
.word 0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x8140,0x9281,0xAEC1,0xB2B0,0xB2B2,0xB2B3,0x92B3,0x92B3,0xB3B3,0x9292,0x9292,0xB192,0xDED1,0x9191,0x8282,0xA3A3,0xA3A3,0xA3A3,0x93A3,0x9393,0x7293,0xA372,0xB3B3,0x8282,0x8282,0x8282,0xC372,0x7171,0xD371,0xC071,0xC061,0x71D4,0xC3C3,0x7070,0x7171,0x4262,0x5241,0x5251,0x5151,0x5152,0x4142,0x6F6F,0xAFAF,0x6060,0x7070,0x708E,0x708E,0xAFAF,0xAFAF,0x7070,0x9E9E,0x9E9E,0x9E9E,0x4040,0x5151,0x5252,0x5151,0x4152,0x8F8F,0xAFAF,0x7070,0x7070,0xAFAF,0x8080,0xBFBF,0xC0C0,0xD080,0x4141,0x6152,0x6152,0x6262,0xD4C0,0xC080,0xC0C3,0xC0D3,0xC0D4,0xC3C3,0xC1C2,0xD0C1,0xE0D0,0xE0E0,0xE0E0,0xD1E1,0xE1D1,0xE1D1,0xF1E1,0xF0F1,0xF0F0,0xDFDF,0xAFAF,0xDEDE,0xBFBF,0xAFAF,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6161,0x8181,0xB1B1,0x6F6F,0xF0F0,0xD0D0,0xB2B2,0xC1C1,0x92B2,0xB292,0xB1B1,0xA1A1,0x9292,0xD1B2,0x8291,0x8282,0x8282,0x8282,0x8282,0x8182,0x8282,0x9281,0xC1B2,0xB2B1,0xB2B2,0xC2C1,0xB2B1,0xC2C1,0xC2D2,0xC1C1,0xD1D1,0xE1D1,0xB1C1,0x92B1,0xB292,0xC1E0,0xB2C1,0x92B1,0xC192,0xC1C2,0xD3D0,0xC3C2,0xD0D0,0xD5C3
.word 0x4040,0x8F8F,0x4141,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x9170,0x8192,0x9ED0,0x8291,0xB392,0xB392,0xB392,0xB3B2,0x9282,0x92B3,0x92B2,0xC1B1,0xDED0,0x92C0,0xA382,0x9393,0xA3A3,0xA3A3,0xA3A3,0x9393,0x8383,0xB293,0xA3B3,0xA3A3,0xB3A3,0xC382,0xB382,0xC3C3,0x71C0,0xC3C3,0x71D4,0xC370,0x7171,0x71C0,0x71C0,0x4241,0x5251,0x5152,0x5152,0x5152,0x4141,0x5050,0x7070,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x8E70,0x8E70,0x9E9E,0x8E70,0xAFAF,0x6060,0x5142,0x5152,0x5262,0x4151,0x6060,0x8080,0xAFAF,0xAFAF,0x7070,0xAFAF,0x8080,0x8080,0xD0D0,0x4251,0x6261,0x5261,0xC051,0xC0C0,0xD4D4,0xC3C0,0xD3C0,0xC3C0,0xC2C0,0xC2C2,0xD2C2,0xD2E3,0xE0E0,0xE0F0,0xE1E1,0xF1F1,0xE1F1,0xE1E1,0xF0F1,0xF0F0,0xF0F0,0xE0E0,0xFEFE,0xAEAE,0x5050,0x4040,0x7F7F,0x7F7F,0xC141,0xB0B0,0xB291,0x8F8F,0xF062,0xD0D0,0xB2C1,0xC2B1,0xC1C1,0xC1C1,0xB1C2,0xC1C2,0xB1D1,0x91B2,0xD192,0x9292,0x8281,0x8172,0x9272,0x8192,0x8282,0x8281,0x9282,0xB1B2,0xB2B2,0xC2B2,0xB1C2,0x92C2,0xC1C2,0xC1C2,0xC2C2,0xB1B1,0xC1C1,0xC1E1,0xB292,0xB2A1,0xC2D1,0xC2C2,0x92B2,0x9292,0x8282,0xD082,0xC0C2,0xD0C3,0xD5B3
.word 0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0xB270,0xB182,0xC08E,0xB291,0xB2B3,0xB292,0xB2B3,0x92B3,0xB2B3,0x9292,0x9292,0xC1B2,0xD0D0,0x9291,0x9382,0xA3A3,0xB3A3,0xA3A3,0x93A3,0x93A3,0x7293,0xB382,0x92B3,0x8282,0x9282,0xB3B3,0xC381,0xC372,0xC471,0x72C0,0xC081,0xC0D4,0xC371,0x7171,0x6161,0x5142,0x5252,0x5251,0x5251,0x4151,0x4242,0x7070,0x9F9F,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0x9E60,0x709E,0x9E9E,0x809E,0x4141,0x5251,0x5251,0x5152,0x5252,0x4040,0xAFAF,0x7070,0x7070,0x8080,0xC0C0,0x8080,0xBFBF,0xC0C0,0x5161,0x6262,0x4162,0xAFAF,0xC0C0,0xC0C0,0xC0D4,0xC0C0,0xC1C3,0xC3C3,0xD3C2,0xD3C2,0xD0C1,0xD1E3,0xE1E1,0xF1E1,0xF1E1,0xF1F1,0xF1F1,0xF1F1,0xFEFE,0xFEFE,0xFEFE,0xCECE,0x6060,0x5050,0x5050,0x7F7F,0x6F6F,0xD0D3,0x8181,0xB3B3,0x4242,0xE0E0,0xC1E0,0xC2B1,0xB1C1,0xA1B1,0x92A1,0x9292,0x9292,0xD1D1,0xB1C1,0xD1B1,0x8192,0x7282,0x7281,0x8272,0x8281,0x7282,0x7282,0x8282,0xB2B2,0x92B2,0xC292,0xB1B1,0x92B2,0xC1C1,0xF2E2,0xB1E1,0xB2B2,0xC1B1,0xD1C1,0x92C2,0x9292,0xD2C2,0xE2D2,0x92C2,0x8292,0x8282,0xB382,0xC3B1,0xC3C1,0xE3B2
.word 0x4141,0x7F7F,0x4040,0x4040,0x7F7F,0x8F8F,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x8170,0x9292,0xA0AE,0xB3B2,0x92B2,0xB2B3,0xB392,0xB392,0x9292,0x92B3,0x92B2,0xD0A1,0xB0B0,0x9292,0xA3A3,0xA3A3,0xA3A3,0xA3A3,0xA3A3,0x9393,0x7293,0xB292,0xA3B2,0x9282,0x9282,0x8282,0x82C3,0x71C3,0x71C3,0xC0D4,0x71C3,0x7171,0x7171,0xD471,0x52C0,0x4241,0x5251,0x5261,0x5251,0x4251,0x4141,0x9F9F,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x8E70,0x7070,0x9E9E,0x9E9E,0xAFAF,0x7070,0x4040,0x5152,0x6152,0x5251,0x4141,0x7070,0xBFBF,0x9E70,0x7070,0xC09E,0xAFAF,0x9EC0,0xC0C0,0xC0C0,0x6250,0x6161,0x5161,0xC0D0,0xC0C0,0xC0C3,0xC0C0,0xD4D0,0xD0D0,0xD0D0,0xC0C1,0xC0C3,0xD3D0,0xD1C1,0xD1D2,0xE1D1,0xE1E1,0xE1E1,0xF1F1,0xF1F1,0xF1F1,0xF0F0,0xF0F0,0xFEFE,0x6060,0x5050,0x8F8F,0x7F7F,0x6F6F,0xC2C2,0x7171,0x6F6F,0xD4D4,0xD0D0,0xC1C1,0xA1B1,0x92B2,0x8292,0x7282,0x9282,0xB2B2,0xC1B1,0x8281,0xD1C1,0x8292,0x8172,0x7272,0x8172,0x7282,0x7281,0x7281,0x8281,0xB2A1,0x92A1,0xB1B2,0xB2B2,0xB292,0xC1D1,0xF3E2,0xC2D1,0xB2B2,0xC2B1,0xC1B1,0xA1D1,0xB192,0xD2C2,0xC2D2,0x92B2,0xB292,0x92B2,0x8282,0xB292,0xC0C1,0xD192
.word 0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x9F9F,0x5050,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x8080,0x9281,0xAEA1,0x81C0,0x82B2,0xB3B2,0xB392,0xB3B2,0xB392,0x92B2,0x9292,0xA192,0xD0B1,0xA1B1,0xA392,0xA382,0xA3A3,0xA3A3,0xB3B3,0xA3A3,0xA393,0x8272,0xB2A3,0x92B2,0x8282,0xB3B2,0x82C3,0x81C3,0xC3C3,0x71C3,0x71C3,0xD0D4,0xC0C3,0xB0C3,0x71C0,0x5252,0x5241,0x5152,0x6152,0x5152,0x4152,0x4141,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0x8E8E,0xAFAF,0x9E9E,0x509E,0x4141,0x5251,0x5261,0x5152,0x4242,0xAFAF,0x9E70,0x80C0,0xBFBF,0x8080,0x8080,0x8080,0xAFAF,0xC0C0,0x6251,0x6262,0x6262,0xAFAF,0xC0C0,0xD0D0,0xC0C0,0xC0C0,0xC0C0,0xD0D0,0xC0C0,0x8080,0xAFAF,0xC0C0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xE0E0,0xF0F0,0xF0F0,0xF1F1,0xF0F0,0xF0F0,0x5050,0x5050,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x6F6F,0x6252,0xF061,0xD1E0,0xB2B1,0x92B2,0x9282,0x8282,0xC1B2,0xE1D1,0xE1E1,0xD1E1,0x72C2,0xC172,0x8292,0x7272,0x7172,0x8272,0x8172,0x8272,0x8272,0x9272,0xB292,0x9292,0xB292,0x92B2,0xF2D2,0xB1E1,0x92B2,0xB292,0x92B1,0xB1B2,0xC1C2,0xC1C1,0xB281,0xC2C2,0xB2B1,0x8292,0xC2B2,0xB2B1,0x8292,0xB282,0xB3C3,0xD192
.word 0x4141,0x4040,0x7F7F,0x4040,0x4040,0x8F8F,0x8F8F,0x9F9F,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x8F8F,0x9150,0x8281,0x9ED0,0x82B0,0xB2B2,0xB2B3,0x92B2,0xB2B3,0x92B3,0xB392,0xB292,0x9292,0xB1B1,0x9291,0x8282,0xA3A3,0xB382,0xB3B3,0xA3A3,0xA3A3,0x82A3,0x82A3,0xB2B3,0x92B2,0xB292,0xC381,0xC381,0xC381,0xC371,0xD371,0xD071,0x7171,0xC3C0,0x7171,0xC072,0x5271,0x5152,0x6152,0x5252,0x5251,0x4151,0x6060,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0x8E70,0x7070,0x8E9E,0x9E70,0xAFAF,0x5241,0x5261,0x6252,0x5251,0x4040,0xAFAF,0x8080,0x9E9E,0x7070,0xAFAF,0xAE80,0x8080,0xC0C0,0x7171,0x6151,0x6171,0x7070,0xC0C0,0xAFAF,0xC0C0,0xC0C0,0xC0C0,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0x5050,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0xE2F5,0xE0F0,0xC1D1,0xA1B2,0x9292,0xB2B2,0xC1B1,0xD1D1,0xC1D1,0xC1C1,0xB1B1,0xC1C1,0xB192,0x8292,0x8182,0x7272,0x8172,0x7272,0x7272,0x7272,0x9272,0x9292,0x9292,0xB292,0xC1B2,0xF3D2,0xC1E2,0x92B2,0xB292,0xB2B2,0xB292,0xB1B1,0xC1C2,0xB2D1,0xC2B1,0xB2B2,0x9292,0xB2B2,0xB2B2,0x8282,0xB282,0xB2B2,0xD282
.word 0x7F7F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x5050,0x9F9F,0x5050,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x7F7F,0x8170,0x9181,0xAEAE,0xB291,0xB3B2,0x9292,0xB3B3,0xB392,0xB292,0x92B3,0x9292,0xB1A1,0x91A1,0x8292,0xA382,0x8282,0xA3B3,0x82A3,0xA3B3,0xA3B3,0xA3A3,0x9282,0xB292,0x92B2,0xB282,0x81C2,0x81C3,0x72C3,0xC3C0,0xC3C0,0xC4C0,0xD3C0,0x7171,0xD4C0,0xD0D0,0x4162,0x5251,0x5262,0x6152,0x5152,0x4142,0x4040,0x7070,0x7070,0x7070,0xAFAF,0x8E70,0x7070,0x7070,0xAFAF,0x9E70,0x709E,0x8E8E,0x9F9F,0x5251,0x6152,0x6152,0x4152,0x6F6F,0xAFAF,0x7070,0x8080,0xBFBF,0x8080,0xC0C0,0xAFAF,0x8080,0x60D0,0x7161,0x6052,0xC0C0,0xAFAF,0xC0C0,0xC0C0,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x5050,0x9F9F,0x5151,0x4141,0x4050,0x4050,0x4050,0x4040,0x4040,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0xC1C1,0xE1E1,0xB2C1,0xB292,0xC2B2,0xC1B1,0xD1D1,0xB1C1,0x92A1,0x8282,0x8282,0x9282,0xB292,0x8292,0x7271,0x7271,0x7271,0x7272,0x7281,0x7281,0x9281,0x9292,0x8282,0xB2B2,0xB2B1,0xB292,0xC292,0xB2D1,0xB292,0xB292,0xB292,0xC2B2,0xC2B2,0xC1E0,0xB1C2,0xB1C2,0x8292,0xC2B2,0x92B2,0x8282,0xB2B3,0x82B2,0xC2B2
.word 0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x6060,0x4040,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8180,0xB292,0xAEAE,0xB2A0,0x92B3,0xB2B3,0xB292,0xB2B3,0xB3B2,0xB292,0x9292,0xA192,0x9292,0x8292,0x8282,0x82A3,0x92A3,0xB3B3,0xA382,0x8282,0x8292,0x9282,0xB292,0x9292,0x82B2,0xC281,0xC381,0xC3C0,0x71C3,0x71D4,0x80D4,0x7171,0xC3D0,0x7171,0x8181,0x5252,0x6252,0x5251,0x5261,0x5261,0x4251,0x7070,0x7070,0x7070,0x7070,0x7070,0x708E,0x7070,0xAFAF,0x7070,0x8E8E,0x709E,0x8E8E,0x9F9F,0x6160,0x5252,0x5261,0x4152,0x7F7F,0xC0C0,0xAFAF,0x9E70,0x9E80,0xAFAF,0x9E70,0x8080,0xBFBF,0x60C0,0x6262,0x5151,0xBFBF,0xC0C0,0xC0C0,0x7070,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x4141,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0xE262,0xD1F1,0xB2B1,0xC2B1,0xB1B1,0xC1B1,0x92B1,0x8292,0x8282,0xC292,0xC1D1,0x8192,0x8282,0x7182,0x7272,0x7172,0x7272,0x7271,0x7272,0x7272,0x9282,0x9292,0x9292,0xB2B2,0xC2B2,0xB2C2,0x92B2,0xC292,0xB2C2,0xB292,0xB2B2,0xB1B2,0xB2B2,0xC192,0xC1C1,0x92B1,0x9292,0xB2B2,0x82B2,0xB372,0xC2B3,0x8282,0xD3B2
.word 0x4040,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x8181,0xC182,0x909E,0xB281,0xB392,0xB3B2,0x92B3,0xB392,0x9292,0x9292,0x9292,0x9291,0x9292,0x8282,0x92A3,0xB3A3,0xA3B3,0xA382,0x82B3,0x82A3,0x8282,0x9282,0xB292,0x92B2,0xB1B2,0xC3B2,0xC3C0,0xC071,0xC0C3,0xD0C0,0xD371,0xD4C0,0xC0C0,0xC0D4,0x62C0,0x6151,0x5262,0x6162,0x6152,0x5152,0x5141,0xAFAF,0x7070,0x8E70,0x7070,0x7070,0x7070,0x8E70,0x8E70,0x7070,0x8E70,0xAFAF,0x8E70,0x6060,0x5161,0x5261,0x6152,0x4242,0x9F9F,0xAFAF,0x9E70,0x7070,0x7070,0xBFBF,0xBFBF,0xBFBF,0x7070,0x7070,0x5171,0xD0D0,0xC0C0,0xC0C0,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x5151,0x6060,0x4141,0x4141,0x4141,0x5050,0x5050,0x5050,0x4040,0x4040,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6363,0xFFFF,0xF2F2,0xB1C2,0xB2B1,0xA192,0x92B2,0x8282,0x9292,0xC192,0xF0F0,0xF0F0,0xC1C1,0x7172,0x9272,0xE1D1,0xE1E1,0x82C2,0x7162,0x7172,0x7172,0x8272,0xC1B2,0xE1E1,0xE1E1,0xF1E1,0xC1D1,0xB2B2,0x92B2,0xB2B2,0x8292,0xB282,0xB2B2,0xC1B2,0xE0E1,0xE0E0,0xC1F0,0x9282,0xB2B2,0x8282,0xC382,0xB2C3,0xB2B2,0xC2B2
.word 0x8F8F,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8281,0xB181,0xB09E,0xB392,0x92B3,0x92B3,0x92B2,0x92B3,0xB392,0xB2B2,0x9292,0x9192,0x9292,0x9292,0xA382,0x8282,0x8282,0x92B3,0x82A3,0x9282,0x9282,0x9282,0x9282,0xB2B2,0xB2B2,0x81B2,0xB1B2,0xD3C3,0xC371,0xC3C3,0x7171,0xC0C0,0xC071,0x7171,0x6171,0x6252,0x6152,0x5252,0x5262,0x5251,0x8051,0x7080,0x7070,0x7070,0x7070,0x8E70,0x7070,0x7070,0x7070,0x8E8E,0x7070,0x9E9E,0x608E,0x9F9F,0x5261,0x6152,0x4252,0x5050,0x8080,0x8080,0x709E,0x9E9E,0x9E70,0xAFAF,0xAFAF,0xAFAF,0x708E,0x7070,0xC0C0,0xBFBF,0xAFAF,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4141,0x8F8F,0x8F8F,0x5050,0x4040,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x5151,0x4242,0xF5F5,0xF3F3,0xB1B1,0x92A1,0x9282,0x8292,0x9292,0x8182,0xF172,0xF0F0,0xE1E0,0xE1E1,0x72D1,0xE1C2,0xE0E1,0xF0E0,0xE1F0,0x82C2,0x8282,0x7272,0x8272,0xD1B1,0xD1D1,0xE1D1,0xE1E1,0xD1C1,0xB1C1,0x92B2,0xB2B2,0x9292,0xB292,0x92B1,0xE0C1,0xE1E0,0xD0F0,0xF0E0,0xB2C1,0x92B2,0x8282,0xC3B3,0xC3C3,0xC282,0xB2E3
.word 0x8F8F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9171,0xC181,0x809E,0xB282,0xB392,0x92B2,0xB3B3,0xB3B2,0x92B2,0x9292,0x9292,0x9292,0x9282,0xA3B3,0x82B3,0x8282,0x82A3,0xA3B3,0x82A3,0x8292,0x9292,0x8292,0xB292,0xB2B2,0x9292,0xA1B2,0x82C3,0xC3C0,0xC3C0,0xC3C0,0xC0C0,0xC0C0,0xD0D0,0xC0C0,0x5272,0x5262,0x6262,0x6162,0x5162,0x6152,0x9090,0x8080,0x8080,0x7070,0x708E,0x7070,0x8E70,0x7070,0xAFAF,0x7070,0x7070,0x708E,0x6060,0x6060,0x5271,0x5261,0x4142,0x7F7F,0x9EC0,0x9E70,0x8080,0x7070,0x9E9E,0x8E70,0x8E8E,0xAFAF,0x7070,0xBFBF,0xAFAF,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x6262,0xAFAF,0x5252,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x5050,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xD0D0,0xD2F0,0xF2D2,0xD2D2,0x92B2,0x9292,0xC192,0x82B1,0x7282,0x7281,0xF081,0xF0F0,0xE0F0,0xF0F0,0xE1F0,0xD0E0,0xD1D1,0xE0D1,0xF0E0,0xE1E0,0xC1D1,0xE1D1,0xD2D1,0xB192,0xC1C1,0xD1C1,0xE1D1,0xC2C1,0xD1C1,0xB2D1,0x92B2,0x9282,0xB1C2,0x92B2,0xD1C1,0xE1E1,0xC1C1,0xB1B2,0xC2C1,0x8292,0xC372,0xC3C3,0x82C3,0xE3B2,0xE3D0
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x8290,0xB182,0x909E,0xB381,0x92B2,0xB2B3,0x9292,0x9282,0x8292,0xB392,0x9292,0x8292,0x8292,0x9292,0x8282,0x82B3,0xA382,0x8292,0x9292,0x92B3,0x9282,0x9282,0x9292,0xB292,0xB292,0xB2B2,0xB291,0x81C3,0xC0C3,0xD071,0x8181,0xC0C0,0xC0D4,0xD471,0x6251,0x6162,0x5262,0x5262,0x6251,0x7171,0x7171,0x80B0,0x8090,0xC0C0,0x7080,0x7070,0x7070,0x7070,0x7070,0x7070,0x8E8E,0xAFAF,0x6060,0x8080,0x6162,0x5252,0x4141,0x9F9F,0x8080,0xBFBF,0xAFAF,0x9E9E,0x7070,0xAFAF,0x7070,0x7070,0x8E8E,0x5050,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0xD4D4,0xFBFB,0xDFDF,0xBFBF,0x6262,0x4141,0x8F8F,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x7241,0xC0E0,0xE0E0,0xE1E1,0xD2F1,0xD2C2,0xE1E2,0xB2C1,0x82B1,0x7272,0x8182,0xC182,0xE0E0,0xF0F0,0xDFDF,0xF0F0,0xC1C1,0xC1C1,0xD1D0,0xE0E0,0xE0F0,0xD0E1,0xE0D0,0xF0F0,0xB2E1,0x9292,0xC1B2,0xD1C1,0xD1D1,0xE1E1,0xE1E1,0x82B2,0x8292,0xB2C2,0x92B2,0xB1B1,0xE2E2,0xC1C1,0xB1B1,0xB1B1,0x7282,0xD3B3,0xD3C3,0x8281,0xE3D3,0xE0D3
.word 0x6F6F,0x4040,0x7F7F,0x4040,0x5050,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7180,0xA181,0x8E8E,0x92B0,0x92B3,0xB392,0x92A3,0x82B3,0xB392,0x9292,0x8292,0x9292,0x92B3,0xB3B3,0x92B3,0x82A3,0xA392,0x8282,0x92B3,0xB392,0x9292,0x8292,0x8292,0x9292,0x92B2,0xB292,0xA1B2,0xC2B1,0xD381,0x71C0,0xD4D0,0x8080,0xC0C0,0x71C0,0x6262,0x6262,0x6262,0x6262,0x7162,0x7171,0x7271,0x7171,0x70C0,0x8070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x6060,0x7070,0x6170,0x5262,0x4151,0x6F6F,0xC0C0,0xC0C0,0xAFAF,0xAFAF,0x7070,0xAFAF,0x9E70,0xAFAF,0x6060,0x5050,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0xAFAF,0xEBEB,0xDFDF,0xE6E6,0xE6D5,0xD7E6,0xD5D5,0x4141,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4140,0x7F7F,0xE260,0xF3F3,0xD0D0,0xD0B0,0xD1C1,0xE1E1,0xF1F1,0xE1F1,0xB2C1,0x8292,0x7281,0x8282,0x9272,0xE0C1,0xE0E0,0xF0F0,0xF0F0,0x92C1,0x9291,0xC1C1,0xD1C1,0xF0E1,0xF1F0,0xD0E0,0xE0E0,0xF0F0,0x82C2,0x9282,0xC1B2,0xD1D1,0xE1D1,0xE1E0,0x82E1,0xB192,0xB1C2,0x9292,0x9292,0xD2C2,0xC1D1,0xC1C1,0xB1C1,0x8282,0xD3C3,0x71C3,0xD3B3,0xD0F4,0xD2D3
.word 0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7160,0xB081,0x9E9E,0x8272,0x8292,0x92B3,0xB392,0x9292,0x92B3,0x8292,0x9292,0xB392,0xB392,0x9282,0xA382,0xA392,0x92B3,0x92B3,0xB392,0x9292,0x8292,0x8292,0x9292,0x9292,0xB292,0xB2B2,0xB2B2,0xB2B1,0x81C0,0xC0C0,0xC0C0,0x71D0,0x6161,0x4141,0x4141,0x5151,0x7282,0x7282,0xB282,0x82B2,0x6272,0x6262,0x6262,0x70C0,0x8080,0x7070,0x7070,0x7070,0x9F9F,0x6060,0x708E,0x9F9F,0x6160,0x6262,0x5261,0x4242,0xC0C0,0xAFAF,0x8E70,0x9E70,0x7070,0xAFAF,0x7070,0x708E,0x5050,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xD4D4,0xFCFC,0xF5F5,0xFDFD,0xCBCB,0xBFBF,0xD6D6,0xF6E6,0xD5D5,0x8F8F,0x7F7F,0x4040,0x7F7F,0x5050,0x8F8F,0x7F7F,0xC3C3,0xEFEF,0xD3F1,0xE1C1,0x70E0,0xC0C0,0xC0C0,0xC1D0,0xD1D1,0xF1F1,0xE1F1,0x92C1,0x7282,0x8282,0xA192,0x9292,0xD1A1,0xF0F0,0xF0F0,0xF0F0,0x92D1,0xB292,0xC2A1,0xD1C1,0xD0D1,0xF0E0,0xE1F0,0xD0D0,0xE0D0,0xD1F0,0xB292,0xB1B2,0xE1D1,0xD1E0,0xD1D1,0xB2F0,0xC2B2,0xB1B2,0x9292,0x9282,0xB1B1,0xC1C1,0xC1C1,0xB1C1,0x8272,0xC3C3,0xB3C3,0xC3C3,0xD4E4,0xD1E3
.word 0x6F6F,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x4040,0x7F7F,0x6150,0x9071,0x9E9E,0x8290,0xB2A3,0x9282,0x82B3,0xB392,0x9292,0x8282,0xB392,0xB392,0x8292,0x8292,0x92A3,0x8282,0xB392,0x9292,0x92B3,0x9292,0x9282,0x9292,0x9292,0x9292,0xB292,0xB1B2,0xB1B2,0xB292,0xC3C0,0x81C0,0x7181,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x8251,0xB2B2,0xB2B2,0xB2B2,0xA3B3,0xB393,0x7272,0x7272,0x6262,0x7070,0x7070,0x6060,0x8E8E,0x7070,0x6060,0x9F9F,0x6050,0x6261,0x6172,0x4142,0x4141,0xC0C0,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x9F9F,0x4040,0x8F8F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0xD5D5,0xDFDF,0xDFDF,0xEFEF,0xFDFD,0xBFBF,0xEAEA,0xD6D6,0xD6E6,0xD0D5,0x7171,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x9F9F,0xF2F2,0xFEF0,0xE0E0,0xF0F0,0xE0E0,0xC0C0,0xC1D0,0xC1D1,0xE1D1,0xE1E1,0xB2C1,0xA192,0xB1B2,0x92A1,0x8282,0x7272,0xF0E0,0xF0F0,0xF0F0,0xB1E0,0xC1C1,0x9292,0xC1C1,0xC1D1,0xD0D2,0xE1E0,0xE3E0,0xD0D0,0xE0E0,0x7272,0x8272,0xE1C2,0xC1D1,0xC1C1,0xC2E1,0xB2B2,0xB2B1,0x8292,0x9292,0x92B1,0xB1B1,0xB1C1,0xB2C1,0xB282,0x72C3,0xC382,0xF4F4,0xD3E4,0xF0E4
.word 0x7F7F,0x4040,0x4040,0x8F8F,0x7F7F,0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7040,0x7171,0x8E91,0x909E,0x9281,0x92A3,0x9282,0x92A3,0x92A3,0x92B3,0x92B3,0x9292,0x9282,0x92A3,0x8292,0x9292,0x9292,0xB392,0x9292,0x92B3,0x9292,0x9282,0x9282,0x9292,0xB1B2,0xB1B1,0xA1A1,0x81B1,0x7172,0x6272,0x6161,0x6F6F,0x6F6F,0x7F7F,0x6161,0xB2B2,0xB2B2,0x92B2,0xA3B3,0xA3A3,0x7293,0x7293,0x7272,0xC062,0x9E8E,0x9E9E,0x7070,0x7060,0x9E9E,0x9E9E,0x6060,0x6160,0x6162,0x5252,0x4141,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD7D7,0xD6D6,0xBEDE,0xDEBE,0xFEFE,0xFEFE,0x6070,0x4040,0xBFBF,0xAFAF,0x90D3,0x8080,0x6161,0x7F7F,0x4040,0x4040,0x7F7F,0x6F6F,0xE0E0,0x92A1,0xC1A1,0xE0D0,0xE0E0,0xD0F0,0xB0C0,0xB2C0,0xB1C1,0xE1C1,0xD1E1,0xC1C1,0xE1D1,0xB1D1,0x8292,0x7281,0x6271,0xF0C2,0xE0E0,0xE0E0,0x82F0,0x6282,0x6262,0xB182,0xC1B1,0xC1C1,0xD1C1,0xE1D1,0xD3F0,0xF0D0,0x5252,0x6262,0xE172,0xC1C1,0xB1C1,0xD1C1,0x92B2,0xB2B2,0x9292,0x9292,0x9292,0xC2B2,0xC1B1,0xA1C1,0x9282,0x8282,0xD4D4,0xCFCF,0xD3D3,0xCFF3
.word 0x7F7F,0x7F7F,0x8F8F,0x4040,0x4040,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7070,0x8181,0xB081,0xAE9E,0x8290,0x8282,0x8282,0xB392,0xB3B2,0x9292,0xB392,0xB382,0xB392,0xB392,0xB392,0xB392,0x92B3,0x92B3,0x92B3,0x8292,0x9292,0x8292,0x9291,0xB292,0xB1B1,0xB1B1,0x92B1,0x7182,0x6272,0x4161,0x6F6F,0x6F6F,0x6F6F,0x5151,0xB182,0xB2B2,0x92B2,0xB3B2,0xB392,0xB3B3,0xB3B3,0xB3B3,0xB393,0x8172,0x70C0,0x7070,0x8070,0x7070,0x7070,0x7070,0x5160,0x6261,0x6261,0x5261,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0xD6D6,0xE6DA,0x7070,0x7070,0x7060,0x9E70,0xBEBE,0x5080,0x4050,0x9F9F,0x5050,0x90A0,0xD070,0x70E0,0x5050,0x4040,0x8F8F,0x7F7F,0x6F6F,0xB140,0x92A1,0x8192,0xA181,0xD0C0,0xE0D0,0xD0F0,0xB1D0,0xB292,0xC1C1,0xD1D1,0xE0C1,0xE1E1,0xF0E1,0x82B1,0x7172,0x6272,0xE172,0xE0E0,0xE1E0,0x72F0,0x6162,0x7162,0x7262,0xB1B2,0xB2B1,0xC1B1,0xD1D1,0xE0F0,0xC0D3,0x62E0,0x7162,0xD172,0xE1D1,0xC1D1,0xB1B1,0xB292,0x92B2,0x9282,0x9292,0x9282,0xB1B2,0xC1C1,0x92C1,0x82A3,0xC5A3,0xCFCF,0xD4E4,0xC2E4,0xE0E0
.word 0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x8171,0x9181,0x8EB0,0x808E,0x8271,0x92B2,0x92B3,0x9292,0xB2B3,0xB2B3,0xB292,0xB292,0xB392,0x9292,0x92B3,0xB392,0x92B2,0x8292,0x9292,0x8292,0x9292,0x9292,0xB1A1,0xC1B1,0xC0D0,0x7292,0x7272,0x4161,0x7F7F,0x7F7F,0x7F7F,0x5151,0xB281,0xB2B2,0x92B2,0xB2B2,0xB292,0xB2B3,0x92B3,0xA3B3,0xA3A3,0x72B3,0xB0B0,0x8080,0x8080,0x8080,0x8080,0x7070,0x7070,0x7161,0x6262,0x5152,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0xD6C6,0xD5D5,0x7070,0x7070,0x7070,0x7070,0x7070,0x508E,0x5050,0x5050,0x5040,0x7090,0x5060,0x5050,0x7070,0x5050,0x7F7F,0x6F6F,0x7070,0x9180,0xC1B1,0x81A1,0x8181,0xA181,0xD0C0,0xE0E0,0xD1E0,0x92C2,0xC1B1,0xC1C1,0xE1D0,0xE1F1,0xE0E1,0x71E0,0x6272,0x7171,0xC162,0xE1E1,0xE0D1,0x81E0,0x5252,0x5261,0x5152,0x9262,0xB1B1,0xA192,0xC1B1,0xF2D1,0xE3E3,0x81D0,0x6262,0xD162,0xE0E0,0xC1E1,0xB2B2,0xB282,0x92B2,0x8292,0x8292,0x9282,0xC2B2,0xC1B1,0x72B1,0xB382,0xD581,0xE0E0,0xBFBF,0xC1D4,0xE0CF
.word 0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8150,0x7182,0xA181,0xAEAE,0x81AE,0xB382,0x9292,0xB382,0x82B2,0x9292,0xB392,0xB392,0xB2B3,0xB392,0x92B2,0xB2B3,0x9282,0x92B3,0x9292,0x9282,0x8182,0xA191,0xB1B1,0xA1A1,0x8292,0x7272,0x5161,0x7F7F,0x6F6F,0x6F6F,0x5151,0xB292,0xB2B1,0xB2B2,0xB292,0x92B2,0xB3B3,0xB392,0x8292,0x82B3,0x82A3,0xB072,0x9090,0xB0B0,0xC0B0,0x8080,0x8080,0x7070,0x7280,0x7272,0x5172,0x7F7F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xC6C6,0xD6D6,0x70CC,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x5060,0x4050,0x4040,0x5040,0x7090,0x4050,0x5040,0x7070,0x8070,0x4050,0x5050,0x9080,0x7080,0x8180,0xA1A1,0x8182,0x8181,0xB2A1,0xD0D0,0xE0E0,0xD1E0,0xB1C1,0xC1B1,0xD2E0,0xE1E1,0xE0D2,0x72D0,0x6272,0x6262,0x8261,0xD1D1,0xD1D0,0xD0D1,0x6152,0x6252,0x5261,0x5261,0xB272,0xB2B1,0xC1B1,0xF2C1,0xFFFF,0xD1D0,0x6272,0xD162,0xE0E0,0xC1D1,0x92B1,0xB182,0x9292,0x9292,0x8292,0x8282,0xC292,0xC1C2,0x8282,0x72B3,0xC3C3,0xE4E4,0xE3D3,0xE0D1,0xE1E1
.word 0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6060,0x8281,0x8282,0xB091,0x9E9E,0x9090,0xB382,0x92B2,0xB282,0xB3B3,0xB2B3,0x92B3,0xA392,0xB3B2,0xB392,0x92B2,0x92B3,0x8292,0x8282,0xA192,0xB1B1,0xB2B1,0x9292,0x7282,0x7172,0x5151,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0xB160,0xB2B1,0x92B2,0xB292,0xB2B2,0xB2B3,0x92B2,0x92B3,0x92B3,0x8282,0xD092,0xCFCF,0xB1C0,0xA0A1,0x9090,0xB090,0x7070,0x8080,0x7282,0x7272,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0xD6C6,0xD6CA,0x7070,0x7070,0x7070,0x7070,0x7070,0x7060,0x6060,0x8F8F,0x5040,0x5050,0x6080,0x5050,0x4040,0x6060,0x6060,0xC060,0x80C0,0x6060,0x4050,0x4070,0x7150,0xA1B1,0x8182,0x8181,0xC282,0xD0D0,0xE0D0,0xD1E0,0xD1D1,0xD1D1,0xD1E1,0xE0C1,0x81C3,0x6172,0x6262,0x6262,0xD0C1,0xD0C1,0xD3C1,0x6261,0x5261,0x6251,0x6162,0x6252,0x7162,0xD182,0xB2A1,0xD0E0,0xD0D0,0x81D1,0xE172,0xD1D0,0xC1D1,0x82B1,0xB292,0x9292,0x9292,0x7282,0x8282,0xB292,0xB1C1,0xA392,0xD472,0xD471,0xD3C0,0xF5F5,0xE0E0,0xC1C1
.word 0x7F7F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x6F6F,0x7140,0x8181,0x9292,0xC1A1,0xBEBE,0x81B0,0x9282,0x92B3,0x9292,0x8282,0xB382,0xB392,0xB292,0x8282,0x9282,0x9292,0xA192,0xC0A1,0xC1D1,0xB2B1,0x9292,0x7182,0x5141,0x5151,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x5050,0xB292,0x9292,0xC2B2,0xC1D2,0x92B2,0xB392,0x92B3,0x92B3,0xB3B3,0xA392,0xCEB1,0xC0BE,0xB091,0xB0A0,0xA1A0,0x9EC0,0x8080,0x8181,0x72B3,0x6172,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x5151,0xC6C5,0xD6D6,0xBFBF,0x7070,0x7070,0x7070,0x7070,0x5050,0x7070,0x5060,0x5050,0x5050,0x5040,0x7080,0x4050,0x5050,0x5050,0x4040,0xE0E0,0x5090,0x4060,0x4040,0x5060,0x7F7F,0xB040,0x92B2,0x8181,0x8281,0xB282,0xC1C1,0xD0D0,0xE0D1,0xD1D1,0xD1D1,0xD0D1,0x62C0,0x6162,0x7161,0x6162,0xC182,0xB1C1,0xD0C1,0x5262,0x6261,0x6152,0x6252,0x5251,0x5251,0xD162,0x9292,0xD0D0,0xC3D0,0xD1B1,0xD1D1,0xC1C1,0x92B1,0x7292,0xB2B2,0x8292,0x8292,0x8282,0x8281,0xB292,0xB2C1,0xC4D3,0xC4B3,0x71D4,0xC3C3,0xF5F5,0xC2FA,0xB2A1
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6060,0x8171,0x9292,0x9192,0xB191,0xD0D0,0xBFBF,0xD0C0,0xC1C0,0x92A1,0x9282,0x9292,0x8282,0xA1A1,0xC1C1,0xB1B1,0xA1B1,0xB2B2,0xB2B2,0x92B3,0x6172,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xA141,0xC1C1,0xC1C1,0xC1C2,0xB2C2,0xB392,0x92B3,0xB392,0xB392,0x8292,0xB182,0xC0D0,0x81B0,0x91B0,0xA191,0xC0B0,0xA09E,0x9171,0x8282,0x7182,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0x4747,0x5959,0xDAD6,0xD7D7,0x7070,0x7070,0x9E9E,0x7070,0x5050,0x6060,0x6060,0x4050,0x4040,0x5050,0x7080,0x5060,0x4050,0x5050,0x4040,0xC040,0x4060,0x4040,0x4040,0x6060,0x4040,0x7F7F,0xA140,0xC1B2,0xC1D1,0xD1C1,0xD0D0,0xD1D1,0xE0D0,0xC1D1,0xD1D1,0xD2C1,0x62B2,0x6261,0x6152,0x6262,0xA162,0xB1A1,0xB1B1,0x6281,0x6152,0x5251,0x6152,0x6152,0x5152,0x9252,0x82A1,0xB1B1,0x81D0,0x9281,0xC1D1,0x92B1,0xB292,0x7272,0x92B2,0x9282,0x8292,0x8282,0x9282,0xB192,0x72D3,0xFDE3,0x72D4,0xD471,0xC072,0xD3C3,0x92C2,0xB292
.word 0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x5050,0x8171,0xB292,0xB292,0xB2B2,0xA1A1,0xC2B1,0xC0C0,0x9E9E,0xAEAE,0x7070,0x9181,0xB1B1,0xC1C1,0x82A1,0x9282,0xB2B2,0xA382,0x4171,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0xF3F3,0xF1F1,0xE1F2,0xB1D2,0xB2B2,0x8292,0xA3B3,0x8292,0x92B3,0x82B3,0x8282,0xB1B1,0xA0C0,0xB0A1,0xA0A1,0xB091,0x8EBE,0x8190,0x8292,0x8282,0x6262,0x4040,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xAFAF,0xD4D4,0x5252,0x8F8F,0x6A6A,0xBFBF,0x7070,0x7070,0x7070,0x7070,0x9E70,0x6060,0x6050,0x4050,0x4040,0x5050,0x7070,0x6060,0x4050,0x5050,0x4040,0x7040,0x5050,0x7F7F,0x4040,0x7040,0x4040,0x6F6F,0xE171,0xE0F0,0xD0E0,0xD0D0,0xD0C1,0xD0C1,0xD0C1,0xD1E0,0xC1D0,0xC1B1,0x61C1,0x5262,0x5261,0x6251,0x8261,0xB1A1,0xB1B1,0x5281,0x5261,0x5252,0x5261,0x5261,0x5252,0x9252,0x82A1,0xC082,0x9292,0xB282,0xC1D0,0xB2B2,0xD1D1,0xB1C1,0x8292,0x9292,0x8282,0x8282,0x9282,0xC2B1,0x93C3,0xE4E4,0x7272,0x72B3,0xC372,0x81C1,0xC1D1,0xB1B1
.word 0x4040,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6160,0x7171,0x9282,0xB292,0xB292,0x9282,0x91A1,0xA091,0xB291,0x92B2,0x9292,0x8282,0x8282,0x8292,0x6272,0x5151,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7E7E,0x7E7E,0xF0F0,0xB2E1,0x7181,0x9282,0x8292,0x9292,0x92B3,0x8282,0x9282,0xC092,0xC0C0,0xC0B0,0xA1A1,0xA191,0x9EC0,0x9090,0x92A0,0x8282,0x6172,0x6F6F,0x8F8F,0x4141,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x4141,0x6F6F,0x6F6F,0x7F7F,0x7E7E,0xAFAF,0xD0D0,0xD4C0,0xF6F6,0x8080,0x7070,0x7070,0x7070,0x6060,0x9F9F,0x6060,0x6060,0x5050,0x4040,0x5040,0x7070,0x7060,0x6060,0x5040,0x4040,0x7070,0x4060,0x4040,0x4040,0x6060,0x4040,0xD160,0xF0F0,0xD0D0,0xC1D0,0xB1C1,0xB2B1,0xA1A1,0x9292,0xC1E0,0xC1C1,0xD0C1,0x72E0,0x6161,0x5252,0x5261,0x6252,0x9292,0x8182,0x5282,0x5261,0x5261,0x5251,0x5252,0x5151,0xA152,0x9292,0x8282,0x9192,0xC182,0xB1D0,0xE1C1,0xE1E1,0xE0E1,0xB2D1,0x9292,0x8282,0x8282,0xB292,0xB3C2,0x71C3,0xD7C4,0xE9E9,0xB372,0x81C3,0xD281,0xC1D0,0xC1B1
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6040,0x8271,0x9282,0x9292,0xB2A1,0x92B2,0x8182,0x6161,0x7182,0x7171,0x6161,0x5151,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0xE0E0,0x4161,0x4141,0x8281,0x9292,0x92A3,0x8282,0x9292,0xC092,0xB0BE,0xC0B0,0xA1A1,0xA091,0x9EC0,0x809E,0x91A0,0x8282,0x8272,0x6161,0x6F6F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0xD070,0xE0E0,0xF0F0,0xCFCF,0x7070,0x7070,0x7070,0x7070,0x7060,0x6060,0x6060,0x5060,0x5060,0x4040,0x4040,0x7060,0x6070,0x4060,0x5050,0x4040,0x7070,0x6060,0x4040,0x7F7F,0x5050,0x5050,0xF0F0,0xD0D0,0xD0D0,0xC0C0,0xB281,0x8281,0x7282,0x7272,0xB192,0xC1B1,0xD0C1,0xE0E0,0x5252,0x5261,0x6152,0x5261,0x9271,0x7281,0x5271,0x5152,0x5252,0x5251,0x5251,0x5252,0xA162,0x8281,0x8292,0x8192,0xD192,0xB2C1,0xC1D1,0xD1D1,0xD1D1,0xE0D1,0x82B2,0x8281,0x8282,0xD292,0xD3C3,0x82C3,0xC6B4,0xDA68,0x7262,0xC372,0xC281,0xC2C2,0xB282
.word 0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x4040,0x7050,0x7171,0x6161,0x4141,0x7F7F,0x6160,0x4160,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x4040,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0xDFDF,0xAFAF,0x6F6F,0x8151,0x8292,0x8282,0xA392,0x9292,0xB082,0xAEAE,0xA1B0,0x91A1,0xA091,0xAEA0,0x9E9E,0x81A0,0x8292,0x8293,0x9292,0x4141,0x6F6F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x9F9F,0xD0D0,0xEFEF,0x7070,0x7070,0x7070,0x7070,0x6060,0x8E50,0x6060,0x9F9F,0x6060,0x5050,0x4040,0x4040,0x7060,0x6060,0x4050,0x4040,0x4040,0x6060,0x4060,0x4040,0x4040,0x4040,0xD040,0xEFEF,0xD0D0,0xC0D0,0x8281,0x8281,0x8281,0x8281,0x8181,0x9282,0xB192,0xC3C1,0xE0D0,0x6161,0x5152,0x5261,0x5152,0x7252,0xB282,0xD2E3,0xB2C1,0x8281,0x6162,0x5162,0x4152,0x8281,0x8181,0x9281,0x7281,0xD0B1,0xB1B1,0xB1C2,0xD1C1,0xD1D1,0xD1D1,0x82C1,0x7282,0x9282,0xE4D3,0xE3D3,0xB2C3,0xD482,0x6757,0xD4D6,0x8282,0xB292,0xB1B1,0xB1B2
.word 0x7F7F,0x7F7F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0xEEEE,0xBFBF,0x6F6F,0x4040,0x8281,0x8282,0x9282,0x8282,0xB082,0xAEAE,0xB0C0,0xB0B0,0xA081,0xAEB0,0xAEAE,0xA0A0,0x8291,0x8282,0xA3A3,0xAEAE,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x6060,0xBFBF,0xF0F0,0x7070,0x7070,0x7070,0x7070,0x6070,0x6060,0x7070,0x608E,0x5050,0x5060,0x4050,0x4040,0x7060,0x6060,0x4050,0x5050,0x4040,0x6060,0x4060,0x6F6F,0x4040,0x4040,0x6060,0xD0CD,0xD0D0,0x72C0,0x9282,0x82A1,0x8292,0x9292,0x61C0,0xB1F0,0x8272,0x8181,0xD082,0x41D4,0x6152,0x5252,0x5251,0x5142,0x8282,0xFFFF,0xC1E0,0xD1C1,0xD1D0,0xD1D1,0xB2C1,0xC1C2,0x72B2,0x7172,0x7262,0xD2E2,0x81C1,0xB292,0xD1B1,0xD1D1,0xD1D1,0x72C1,0x8272,0xE6C3,0xEBEB,0x82B3,0x7181,0xB282,0x71D0,0x5959,0x8272,0x9282,0xC1B2,0xC1E1
.word 0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6F6F,0xCECE,0x7F7F,0x6F6F,0x7140,0x8182,0x9282,0x8282,0x8282,0xA1A1,0x80AE,0x91C0,0x8191,0xD091,0x9EAE,0x9090,0x92B0,0x8282,0x7282,0xCBAC,0xBBBB,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x9F9F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8080,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x5050,0x9E9E,0x7070,0x5060,0x5060,0x4050,0x4040,0x7050,0x6070,0x5050,0x4040,0x4040,0x4040,0x5070,0x4040,0x6F6F,0x4040,0x5050,0xE0E0,0xD0D0,0xA1C0,0xB1B1,0xC0B1,0x6161,0xBFBF,0x9D9D,0x92D0,0x7182,0x7272,0x8271,0x62D0,0x5241,0x5251,0x4251,0x5251,0x5252,0xE3E5,0x9181,0xB1A1,0xC1B1,0xD0C0,0xE0D0,0xE0E0,0xD1D1,0xC2C1,0x8292,0xFFFF,0x41D3,0xC272,0xC1C1,0xD1D1,0xD1D1,0x72C2,0xE092,0xABCC,0xBBBB,0x7282,0x7272,0x8282,0xD1C3,0x47D0,0x8272,0xD2B2,0xE1D1,0xB1D1
.word 0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x4040,0x8F8F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6E6E,0x9F9F,0x6060,0x8F8F,0x4040,0x4040,0x8E8E,0x4040,0x7F7F,0x4040,0xB281,0x8282,0x9292,0x8292,0xC092,0x9EBE,0xC0C0,0x81A0,0xC0B0,0x8080,0xA0A0,0x9181,0x8282,0x7282,0x6C6C,0x9C8C,0x8C8C,0xAFAF,0x7F7F,0x4141,0x6F6F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x4141,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0x9F9F,0xC0C0,0x9E9E,0x7070,0x7070,0x7070,0x7070,0x7070,0x708E,0xAFAF,0x609E,0x5050,0x6060,0x5050,0x4040,0x6050,0x6070,0x4060,0x4040,0x4040,0x4040,0x6060,0x7F7F,0x4040,0x4040,0x7040,0xE0E0,0xE0D0,0x81E1,0xC0A0,0xAFAF,0xC0C0,0xD0D0,0xB181,0x82B2,0x8281,0x7172,0x7172,0xE0B2,0x5142,0x4142,0x5241,0x7161,0x6272,0x8171,0x9292,0x8281,0x9281,0xB2A1,0xC1B1,0xC1C1,0xD0D1,0xD1E1,0xE1E0,0xF0E1,0xE2E0,0xE1E1,0xD1E1,0xE1D1,0xD1C1,0x8192,0xE0E0,0x4261,0x7263,0x7272,0x7282,0x7281,0xC282,0xD1D1,0xE1D1,0xE0D1,0xC1D1,0xB2C1
.word 0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7E7E,0x5060,0x8F8F,0x7060,0x5050,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x9E60,0x5070,0x4040,0x7F7F,0x7F7F,0x9E9E,0x4040,0x7F7F,0x8181,0x8191,0x9292,0x8282,0x8182,0xAEB0,0xC0AE,0x90A0,0xB091,0xC0D0,0xB0B0,0x9181,0x7282,0xA382,0x8B8B,0x7C7B,0x7C7C,0x6C4A,0x4A4A,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0xD659,0xAFAF,0xBFBF,0xAEAE,0x708E,0x7070,0x7070,0x7070,0x9E70,0x7070,0x9E70,0xAFAF,0x6060,0x6050,0x4040,0x5050,0x6060,0x6060,0x5050,0x7F7F,0x4040,0x7F7F,0x5070,0x4040,0x4040,0x7F7F,0x5050,0xC080,0xD1D0,0xFFFF,0xB081,0xC0C0,0xD0C0,0xB1D0,0xB2B1,0x7282,0x7172,0x7282,0x7271,0xD271,0x42D0,0x6251,0x7162,0x7272,0x8272,0x8282,0xA182,0xA1A1,0xA192,0xA192,0xB1B2,0xB1B1,0xC1B1,0xC1C1,0xD1C1,0xD1D1,0xF0E0,0xF0F0,0xE1F0,0xD1E1,0xB1D1,0x7272,0x7171,0xC3C3,0x7272,0x7271,0x9272,0xB1B2,0xC1C1,0xD1C1,0xD1D1,0xD1D1,0xC1C1,0xB2A1
.word 0x4040,0x7F7F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x6060,0x7070,0x5050,0x8E8E,0x6070,0x6060,0x4040,0x6F6F,0x6F6F,0x8F8F,0x607E,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8190,0x8282,0x8281,0x8182,0xD0B0,0xC0AE,0xA0A0,0xB1B0,0xB0C0,0x90B1,0x8281,0x8282,0xB382,0x8C8C,0x9B9B,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x6F6F,0x4242,0x6F6F,0x6F6F,0x4141,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5050,0x8F8F,0x6F6F,0x6F6F,0x8F8F,0xAFAF,0xC0C0,0x9E9E,0xAE9E,0x7070,0x7070,0x7070,0x7070,0x7070,0x9E9E,0xAFAF,0xAFAF,0x5050,0x6060,0x4040,0x4040,0x7050,0x6070,0x5060,0x7F7F,0x4040,0x4040,0x6070,0x4040,0x6F6F,0x4040,0x7050,0xB261,0xA1A1,0xCFCF,0xB180,0x7282,0xB282,0xB1A1,0x7282,0x7172,0x6262,0x8172,0x7272,0x8172,0xC3C1,0x7172,0x7272,0x8272,0x8181,0x8181,0x9281,0xC1A1,0xC1D1,0xB1C2,0xB2B1,0xB2A1,0xA1A1,0xB1A1,0xB1B1,0xB1B1,0xC1B1,0xD0D1,0xD1E1,0xD1D1,0xB1C1,0x7272,0x7272,0x7171,0xB3D0,0xB292,0xB1B1,0xB1B1,0xB1B1,0xC1B1,0xD1C1,0xC1C1,0xA1B1,0xB192
.word 0x4040,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x8F8F,0x5050,0x6060,0x7070,0x7070,0x6060,0x9E60,0x509E,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0x5050,0x8F8F,0x9F9F,0x5050,0x6060,0x7F7F,0x7F7F,0x5050,0x8181,0x8281,0x8181,0xC081,0xAEDE,0x9080,0xC0C0,0xA1A1,0x91A1,0x8181,0x8282,0x8282,0x8D8D,0x9B9C,0x8B9B,0x4949,0x8F8F,0x7F7F,0x7F7F,0x4242,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0xC0C0,0x70C0,0x7F7F,0x7F7F,0x8F8F,0xBEBE,0xCEBE,0x9E9E,0xBEBE,0x709E,0x7070,0x7070,0x7070,0x7070,0x8E9E,0x7070,0x7070,0x6070,0x5050,0x5050,0x5050,0x6050,0x6060,0x4060,0x4040,0x4040,0x7F7F,0x6060,0x5050,0x4040,0x4040,0xF0C0,0xDFDF,0xE0E0,0xE0E0,0xB2C2,0xA181,0xB1B1,0x7292,0x7172,0x6262,0x6162,0x6162,0x8172,0x7271,0xC172,0x62C1,0x6261,0xA181,0x8192,0x9191,0x9272,0xD0C1,0xD3D0,0xC2F2,0xB1B1,0xB2B1,0xB2B2,0xB2B2,0xB292,0xB292,0x9292,0xB192,0xD1C1,0xC1C1,0xA1B1,0x7272,0xB2B1,0xB2B2,0x9292,0xB292,0xA192,0xA1A1,0xA1B2,0xB1A1,0xB1B1,0xA1C1,0x9292,0xB2B1
.word 0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x8F8F,0x6060,0x4040,0x4040,0x7F7F,0x7F7F,0x607E,0x6060,0x6060,0x7070,0x9F9F,0x6060,0x7070,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5050,0x8F8F,0x7070,0x7E7E,0x4040,0x7F7F,0x7F7F,0x8080,0x8181,0x8181,0xB182,0xBEBE,0x809E,0xD0D0,0x91B0,0x91A1,0x8281,0x8282,0x7281,0x7E7E,0x9D8E,0x8B9C,0x8B8B,0x4A4A,0x8F8F,0x4A4A,0x424A,0x6F6F,0x6F6F,0x4242,0x6F6F,0x4242,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0xE0E0,0xE0E0,0x8080,0x4848,0xCFCF,0xBEBE,0xCECE,0xCEBE,0xAEDE,0x709E,0x7070,0x7070,0x7070,0x7070,0x9E9E,0x8E70,0x9E70,0x7070,0x6060,0x4050,0x4040,0x5050,0x6060,0x4060,0x7F7F,0x4040,0x4040,0x6060,0x4050,0x6F6F,0x4040,0xD0D0,0xD0D0,0xD0D0,0xE0E0,0xD2F3,0xC1B2,0x82C2,0x7172,0x7262,0x7271,0x6271,0x7262,0x8272,0x7282,0x8281,0xD2C1,0x82B3,0xB282,0xB1D1,0x8181,0xD0B1,0xC3C0,0xE1C3,0xE1F0,0xC1D1,0xD1D1,0xC1D1,0xB1B1,0xB2B1,0xB2B1,0xB2B2,0xC1B2,0xC1C1,0xB1C1,0x92B2,0x7272,0x9272,0xB2B1,0x92A1,0xA192,0x9292,0x9292,0x92A1,0x92A1,0xA1A1,0xA192,0xC1B2,0xB2B1
.word 0x8F8F,0x9F9F,0x9F9F,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8E8E,0x8080,0x6060,0x7070,0x6060,0x5050,0x7070,0x6060,0x7070,0x7070,0x7060,0x6060,0x6070,0x4040,0x4040,0x7F7F,0x4050,0x5050,0x8E50,0x7070,0x9F9F,0x4040,0x8F8F,0x4040,0x7F7F,0x7150,0x8181,0xA181,0x9EB0,0xC0AE,0xC0B0,0x9090,0x9191,0x8181,0x8181,0x6271,0x6C6C,0x8E7D,0x8D8E,0x8C9C,0x7C4A,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x4242,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0xAFAF,0xBFBF,0xD0D0,0xE6E6,0xBEBE,0xCEBE,0xCECE,0xCECE,0xAECE,0x9E9E,0x7070,0x7070,0x7070,0x7070,0x9E9E,0x708E,0xAFAF,0x8E8E,0x6060,0x4040,0x5050,0x5050,0x6060,0x4050,0x4040,0x4040,0x7F7F,0x7050,0x6060,0x7F7F,0xE040,0xDEDE,0xB0B0,0x80A0,0x8080,0xC2D3,0x81D1,0x7281,0x7271,0x7271,0x7272,0x8272,0x9292,0x7281,0x8172,0x8282,0xD282,0xB1B1,0xA182,0xB0B1,0x81B0,0xD0D0,0xE0D1,0xE0E0,0xF0E0,0xF0F0,0xF0F0,0xE0F0,0xD1E0,0xB1C1,0xB1C1,0xC1B1,0xC1B1,0xC1C1,0xB1C1,0xB1B2,0x7171,0x8272,0xE1B1,0xE0E1,0x92D1,0x92A1,0x92A1,0x9292,0x9292,0x9292,0xB192,0xB1C1,0xB1B2
.word 0x8F8F,0x9F9F,0x6060,0x5050,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x8F8F,0x608E,0x6060,0x6060,0x7070,0x7060,0x9E9E,0x6070,0x7070,0x6060,0x9F9F,0x7070,0x7070,0x6060,0x4070,0x4040,0x8F8F,0x5050,0x6060,0x8E70,0x7070,0x4040,0x7F7F,0x8F8F,0x7F7F,0x9180,0xA1A1,0xC0C0,0xB0D0,0xA0A0,0x8181,0x9291,0x7281,0x7281,0x6171,0x4A4A,0x4A4A,0x7D7D,0x7E8E,0x7D7D,0x7C7C,0x7C7C,0x9F9F,0x8F8F,0x4242,0x4242,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0xAFAF,0x7070,0xBFBF,0xCFCF,0x7070,0xBEBE,0xCEBE,0xBEBE,0xBECE,0x9EAE,0x7070,0x7070,0x7070,0x9E70,0x9E9E,0x7070,0xAFAF,0x9E9E,0x5070,0x5050,0x4040,0x5050,0x6060,0x5050,0x4040,0x4040,0x4040,0x7050,0x4060,0x4040,0x7F7F,0xB0D0,0x81B0,0xE292,0x7182,0xD0D0,0x81D0,0x7172,0x7162,0x7272,0x7272,0x8281,0x8182,0x7272,0x8271,0x92B2,0xD2B2,0xC1F3,0xC081,0x7170,0xE0B2,0xD3E0,0xD0D0,0xC1C1,0xD0D0,0xE0E0,0xE0E0,0xE0E0,0xD0D0,0xD0D0,0xC1C1,0xC1C1,0xB1C1,0xC1C1,0xB1B1,0x72B1,0x7262,0xB272,0xF1F1,0xF0F0,0xE0E0,0x81B2,0x8292,0xA1A1,0xA192,0xA1A1,0xC1C1,0xB1C1,0x92A1
.word 0x7F7F,0x9F9F,0x608E,0x5050,0x8F8F,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x5050,0x6060,0x7070,0x7070,0x7070,0x7070,0x6070,0x6070,0x7070,0x6080,0x7070,0x7070,0x7070,0x7070,0x6060,0x8E60,0x7070,0x7070,0x708E,0x7070,0x7070,0x8F8F,0x7F7F,0x6060,0xD0D0,0xD0E0,0x91C0,0x91A0,0x9191,0x9191,0x8181,0x7081,0x9F9F,0x4A4A,0x8F8F,0x5C5C,0x7C7E,0x7E7E,0x6C6C,0x6C6C,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x4242,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7070,0x7070,0x8E8E,0xBFBF,0x9E70,0x9E9E,0xBEAE,0xBEBE,0xCEDE,0xAEAE,0x7070,0x7070,0x7070,0x9E70,0x9E9E,0xAFAF,0x8E70,0x9E70,0x6060,0x5050,0x5050,0x5050,0x6060,0x5050,0x4040,0x4040,0x7F7F,0x7040,0x5060,0x7F7F,0x4040,0x7070,0x7181,0xC2C2,0xD0E3,0xC0C0,0x8272,0x6261,0x7271,0x7171,0x8172,0x7172,0x7271,0x7162,0x7162,0xC182,0xF2F1,0xC1E1,0x7171,0xD171,0xE0E0,0xC0D0,0xB2C2,0xB1B1,0x92B2,0xC0B1,0xB1C0,0xB0C0,0x8181,0x8282,0x9282,0x8292,0xB1A1,0xB1B1,0x8292,0x7172,0x6162,0xF192,0xF0F0,0xF0F0,0xFEF0,0x82D0,0xA192,0x9292,0xB2A1,0xC1B2,0xC1C1,0xB2B1,0xB2B2
.word 0x7F7F,0x9F9F,0x5050,0x5050,0x5050,0x6060,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x8F8F,0x5060,0x5050,0x7070,0x7070,0x7070,0x7070,0x6080,0x7060,0x7070,0x7060,0x7060,0x8080,0x7070,0x7070,0x7070,0x7070,0x709E,0x7070,0x7070,0x7070,0x6060,0x9F9F,0x9F9F,0xBEBE,0xD0D0,0x91C0,0xA191,0x91A1,0x7181,0x7070,0x7E7E,0xAFAF,0x9F9F,0x9F9F,0x6E6E,0x8F8F,0x6C6C,0x6C6C,0x4A4A,0x4E4E,0x4A4A,0x8F8F,0x4040,0x5050,0x4242,0x6F6F,0x6F6F,0x6F6F,0x4242,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7060,0x8E8E,0x9E70,0x9E9E,0xAFAF,0x9E9E,0xAE9E,0xBE9E,0xCEDE,0x9EAE,0x9E70,0x7070,0x7070,0x7070,0x9E9E,0x8E8E,0x7070,0x9E9E,0x6060,0x5050,0x5050,0x5060,0x6060,0x5050,0x4040,0x4040,0x4040,0x7070,0x5070,0x7F7F,0x4040,0x4040,0x4040,0x9F9F,0xD0D0,0x81C0,0x6281,0x7272,0x7172,0x6262,0x7162,0x7272,0x7182,0x7172,0x8272,0xB192,0xE1C1,0xD1F2,0x71C1,0xE0D1,0xCFCF,0x81C0,0x8181,0x8282,0x9281,0x9291,0xA1B2,0xB2B2,0xB292,0x9281,0x9291,0x8281,0xC1B2,0xB2B1,0x6281,0x6271,0x7261,0xD1D1,0xF0F0,0xE0E0,0xE0E0,0x92D0,0x9292,0xA1A1,0xB2B1,0xC1C1,0xB1C1,0xB2A1,0xB1B2
.word 0x7F7F,0x4040,0x8F8F,0x9F9F,0x5050,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x506E,0x6050,0x5040,0x5050,0x7060,0x5060,0x6060,0xC070,0x7090,0x7070,0x8080,0xC0C0,0xB0D0,0x7070,0xC0F0,0xC0C0,0x7070,0x7070,0x7080,0x7070,0x7070,0x7070,0x7070,0x7070,0x9E9E,0x9E9E,0xB0C0,0xA1C0,0x8191,0x7090,0x4050,0x7F7F,0x8F8F,0x9F9F,0x6060,0xAFAF,0x9F9F,0x9F9F,0x4C4C,0x6C6C,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x4040,0x6060,0x4242,0x4242,0x6F6F,0x6F6F,0x4242,0x4142,0x4241,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0x7070,0xBFBF,0x9E70,0xAEAE,0x9E9E,0x709E,0xAFAF,0xAE9E,0x9E9E,0xCEDE,0xAEAE,0x708E,0x7070,0x8E70,0x7070,0x9E9E,0x709E,0x7070,0x9E70,0x8E9E,0x6060,0x5050,0x9F9F,0x6050,0x5050,0x4040,0x4040,0x4040,0x7040,0x4060,0x4040,0x7F7F,0x7F7F,0x5050,0x5151,0xB081,0x7181,0x7171,0x8172,0x7281,0x6162,0x7161,0xD072,0x7172,0x7162,0x7172,0xB082,0xB1B1,0xE2F1,0xC2C0,0xD0D0,0xD0D0,0x81C0,0x92B2,0x92A1,0xB1B1,0xC1C1,0xC1C1,0xC1C1,0xB1B1,0xB1C1,0xB2B1,0xB1C1,0xB1B1,0xA1B2,0x7272,0x7162,0x8262,0xE0E0,0xE0E0,0xE0E0,0xC1D0,0x92B1,0xB1A1,0x7281,0xB182,0xB1C1,0xA1B1,0xB1B2,0xB2B1
.word 0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x6060,0x9E8E,0x8080,0x6060,0x7070,0x7090,0xC0C0,0xAEAE,0x70D0,0xAFAF,0xBFBF,0x9EB0,0xC0DE,0xCECE,0xD0D0,0xC09E,0x7070,0x9E9E,0x7070,0x709E,0xD0C0,0x70C0,0x7070,0x6060,0x7050,0xAE9E,0xB0D0,0x8080,0x5050,0x6F6F,0x8F8F,0x8F8F,0x8E8E,0x7070,0x8E70,0x7070,0x6060,0x9F9F,0x8F8F,0x6E6E,0x5C5C,0x5C5C,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0xAFAF,0x8F8F,0x6F6F,0x7F7F,0x4141,0xB070,0xC090,0x80C0,0x62C0,0x4141,0x6F6F,0x6F6F,0x7F7F,0x9E70,0xC0C0,0xAEAE,0xDECE,0xAEBE,0x709E,0x9E60,0x9E9E,0xBEBE,0xCECE,0x9EBE,0x7070,0x7070,0x7070,0x9E70,0x9E9E,0x9E9E,0xAFAF,0x5050,0x709E,0x5050,0x6060,0x6060,0x6060,0x5050,0x4040,0x4040,0x4040,0x6040,0x5060,0x7F7F,0x4040,0x4040,0x5050,0x8141,0x81B1,0x7181,0x8272,0x8281,0x6272,0x5161,0x7261,0x7181,0x7172,0x6162,0x7162,0x7171,0x80C0,0xD1A1,0xC0B2,0xD0D0,0xC0C0,0xB282,0xB1A1,0xC181,0xD0C1,0xD1C1,0xC1D1,0xC1D0,0xC0C0,0xC1C0,0xE1C1,0xC1D1,0xB1B1,0x92B2,0x7162,0x6271,0xD162,0xF0F0,0xE0E0,0xD0E0,0xD1C1,0xA1C0,0xB1A1,0x8271,0xC1B1,0xB1C1,0xB2A1,0xC1B2,0xC2B1
.word 0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x7F7F,0x8F8F,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7070,0xAFAF,0x9EC0,0xC0C0,0xAEAE,0xC0C0,0x70B0,0x909E,0xD070,0xB0B0,0xC09E,0xBEBE,0xCFCF,0x5081,0x7070,0x7070,0x7070,0x9E70,0xFEDE,0xC0E0,0x7070,0x6060,0x9E9E,0x709E,0x8F8F,0x8E8E,0x7070,0x8F8F,0x5050,0x6060,0x6060,0x9F9F,0x6060,0x9F9F,0x6060,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4040,0x7F7F,0x8F8F,0x5252,0x4242,0x6F6F,0x6F6F,0xC0C0,0xCECE,0xCFCF,0xCFCF,0xF0F0,0xC0D0,0x4161,0x6F6F,0x7F7F,0x8F8F,0x9E9E,0xAE9E,0xDEDE,0xEEEE,0xBEBE,0x9E9E,0x9EAE,0xBE9E,0xDECE,0xAEBE,0x709E,0x7070,0x7070,0x9E9E,0x9E9E,0xAFAF,0x7070,0x5050,0x9E9E,0x7070,0x5050,0x5050,0x6060,0x6050,0x5050,0x4040,0x4040,0x6060,0x5070,0x4040,0x4040,0x7F7F,0x5050,0xF571,0x71B2,0x7172,0x81A1,0x7272,0x7171,0x6252,0x9272,0x7272,0x6271,0x6271,0x7271,0x7171,0x7070,0x8161,0xC1C1,0x71C0,0xC371,0x80B1,0xC072,0xC0C0,0xC0C0,0xC0C0,0xD0D0,0xC0C0,0x7171,0xC262,0xD1E1,0xC1C1,0xF1E2,0xC1C1,0x6272,0x6172,0xE172,0xE0E0,0xE1E0,0xC1D1,0xD1C1,0xC1E1,0xA1B1,0xB1A1,0xA1B1,0xA1A1,0xB2B2,0xC1C1,0xC2B1
.word 0x6F6F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x9F9F,0x4040,0x5050,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x708E,0x7070,0xD0AE,0x80C0,0x7070,0x7070,0x60B0,0x808E,0xE09E,0x8080,0x8E90,0x80D0,0x71AE,0x7070,0x7070,0x7070,0x7080,0xAFAF,0xDFDF,0xD0D0,0x6060,0x9E9E,0x709E,0x5050,0x8F8F,0x9E9E,0xD0D0,0x9F9F,0x9F9F,0x9F9F,0x7070,0x7070,0x6060,0x7070,0xAFAF,0x7070,0x6060,0x9F9F,0x9F9F,0x7E7E,0x8C8C,0x9F9F,0x6060,0x5151,0x9F9F,0xD6D6,0x8F8F,0x4242,0x7F7F,0xBEBE,0xDEDE,0xDEEE,0xDEDE,0xFEEE,0xFEFE,0xD0D0,0x6161,0x7F7F,0x4040,0x9E9E,0x9E70,0xFEEE,0xFEFE,0xAEDE,0x7070,0xAEBE,0xAEAE,0xDEDE,0xAEBE,0x7070,0x709E,0x7070,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x6060,0xBE9E,0x709E,0x6060,0x7070,0x5050,0x6060,0x4040,0x4040,0x4050,0x6060,0x5060,0x7F7F,0x7F7F,0x4040,0x5050,0xBFBF,0x7192,0x7061,0x81B0,0x7271,0x6272,0x9261,0xE1E1,0x81E1,0x6262,0x6162,0x6172,0x6161,0x7070,0xC151,0xC1D1,0xC0C3,0xA181,0x7070,0x9DD0,0x7070,0x7171,0x6272,0x6171,0x7161,0x6171,0xE0B2,0xC1D1,0xD1C1,0xFFFF,0xC1E0,0x81B1,0x6272,0xE0A1,0xE1E0,0xD1E1,0xC1D1,0xE1C1,0xF0F0,0xB1C1,0xB1A1,0xB1B1,0xC2B1,0xC1C1,0xC2C1,0xB1B2
.word 0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4040,0x9F9F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x7F7F,0xE08E,0xD0C0,0xD0D0,0x80D0,0xC0C0,0x7070,0x80C0,0x9E70,0xC0C0,0x9E60,0x60C0,0xAFAF,0x8080,0x7060,0x8080,0x8080,0x8080,0xDFDF,0xEFEF,0xDFDF,0x7080,0x9E9E,0x6060,0x5050,0x6060,0x9E9E,0x8E8E,0x9F9F,0x7060,0x7070,0x9F9F,0x9F9F,0x7070,0x6060,0x7070,0x708E,0xAFAF,0x9F9F,0x8E8E,0xAFAF,0xBFBF,0xE4E4,0xD6D6,0xD4D4,0x5959,0xAFAF,0x4247,0x7F7F,0x4040,0xAEAE,0xBEBE,0xAEAE,0xCECE,0xFEEE,0xEEEE,0xEEEE,0xC0D0,0x60C0,0x5060,0x8E70,0x9E9E,0xFEBE,0xFEFE,0xBEEE,0x9E9E,0xBEBE,0xCEBE,0xDEDE,0x9EBE,0x7070,0x9E9E,0x7070,0x9E9E,0x9E9E,0x709E,0x9E9E,0x6060,0x9E70,0x70AE,0x6060,0x7070,0x6060,0x5050,0x4050,0x4040,0x4050,0x5050,0x4060,0x4040,0x4040,0x7F7F,0x5050,0x7F7F,0x4060,0x6060,0xA1A1,0xB1C2,0x8181,0xC1A1,0xE1E1,0xE1F1,0x6171,0x7171,0x5170,0x7162,0x5151,0xD071,0xD0D0,0x71C0,0xA1A1,0xC061,0xAFAF,0xC180,0x7181,0x8281,0xB281,0xB2B1,0xB281,0xD1D1,0xC1C1,0xD1C1,0xE2E2,0x9281,0x81A1,0x8281,0xE0C1,0xE1D1,0xD1D1,0xC1C1,0xE0C1,0xF0F0,0xD1E0,0xD1D1,0xE1D1,0xD1D1,0xE1D1,0xE1E1,0xD1D1
.word 0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7E40,0xDEDE,0xD0E0,0xD0D0,0x8090,0x809E,0x9E9E,0x90D0,0x9E8E,0x6060,0x5060,0x9F9F,0x6060,0x9F9F,0x6070,0x7070,0x70C0,0x8080,0xBFBF,0xDFDF,0xF0FE,0x9EC0,0x7070,0x6070,0x5050,0x8E60,0x709E,0x6060,0x7060,0x8E60,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xF0F0,0xFCFC,0xF9F6,0xD5EA,0xAFAF,0xD4D4,0x61D0,0x5252,0xBFBF,0x9E9E,0x9E9E,0x709E,0x708E,0xAE9E,0xDEBE,0xEEEE,0xEEEE,0xDEDE,0xD0E0,0x70B0,0x6070,0xEE8E,0xFEFE,0xDEFE,0x9EBE,0xBEBE,0xAEAE,0xEEDE,0xAECE,0x7070,0x8E70,0x7070,0xCECE,0x9E9E,0x9E9E,0x9E9E,0x709E,0x7060,0x9E9E,0x6060,0x6060,0x6060,0x6060,0x4050,0x4040,0x5050,0x6060,0x5070,0x7F7F,0x5050,0x4040,0x4040,0x6F6F,0x4141,0x4040,0x8190,0xC0D0,0x8171,0xA181,0xC0C1,0xE0D0,0x81D1,0x7061,0x5161,0x9F9F,0x7171,0xD0D0,0xC0C0,0x8181,0x7081,0xC0C0,0x9090,0x8181,0x7F7F,0x8F8F,0x6060,0x6060,0xD151,0xC1D1,0xC1D1,0xC1C1,0x92C1,0xA192,0x6281,0xB162,0xD1D1,0xD1E1,0xC1D1,0xC1B1,0xE0D1,0xD1D1,0xD1D1,0xC1D1,0xD1C1,0xD1C1,0xC1D1,0xE0E1,0xF1E1
.word 0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0x9F9F,0x5050,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x8E8E,0xF0FE,0xF0F0,0xD0D0,0xB0B0,0x8080,0x708E,0x7070,0x5060,0x5050,0x6060,0x5050,0x9F9F,0x6070,0x6060,0x9F9F,0x7070,0x808E,0xC0C0,0xF0FE,0xAEAE,0xC0C0,0x8080,0x7070,0x6060,0x6060,0xAFAF,0x9F9F,0x9F9F,0x7070,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0xAFAF,0x8E70,0x7070,0x7070,0x7070,0xC08E,0xBE9E,0xF8EB,0xF9F9,0xD0E4,0xD0E0,0xD3D3,0xAFAF,0xAEDE,0xAEAE,0x7070,0x7070,0x9E70,0x7070,0xAFAF,0x9E9E,0xBEAE,0xEECE,0xEEFE,0xCEDE,0xC0D0,0xCECE,0xFEFE,0xFEFE,0x9EDE,0xCEAE,0x9EBE,0xDEDE,0xBEEE,0x709E,0x709E,0x7070,0x9E9E,0x9EBE,0x8E8E,0x9E9E,0x9EAE,0x7070,0x9E70,0x6070,0x6060,0x7070,0x6060,0x5060,0x4040,0x5050,0x5040,0x6080,0x4040,0x6060,0x7F7F,0x4040,0xA0A0,0xB1B1,0x5050,0x8060,0x6181,0x7180,0xB090,0xC0A1,0xA1B0,0xC1D0,0x7161,0x6161,0x4040,0xD071,0x8080,0xC0C0,0x7180,0xC0B1,0x9F9F,0xC080,0x8F8F,0x7F7F,0x4040,0x7F7F,0x8F8F,0xD161,0xC1C1,0xC1C1,0xC1D1,0xA1B1,0x8292,0x7162,0xB162,0xE1C1,0xD1E1,0xB1D1,0xC1B1,0xB1D1,0xC1C1,0xB1B1,0xB1B1,0xB1C1,0xB1C1,0xC1B1,0xE0E0,0xE1D1
.word 0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x5050,0x9F9F,0x6060,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x7F7F,0x6F6F,0xCE8E,0xEFEF,0xEFEF,0xE0E0,0x70C2,0x6060,0x7070,0x5060,0x4070,0x5050,0x8F8F,0x4040,0x6060,0x5050,0x5050,0x4040,0x6060,0x8E70,0xE0E0,0xD0D0,0xBFBF,0xD4D4,0x6080,0x7070,0xAFAF,0x9F9F,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x9E70,0x7070,0x7070,0x7070,0x7070,0x7070,0x708E,0x7070,0x7070,0x7070,0xBFBF,0xE9DB,0xFDFA,0xDFDF,0xE0E0,0xD4D4,0xC5D6,0x9E9E,0x809E,0x709E,0x9E70,0x709E,0x709E,0x7070,0x7080,0x7070,0x9E70,0xBE9E,0xDEDE,0xBECE,0xAEBE,0xEEBE,0xEEEE,0xBEDE,0xAE9E,0x9EBE,0xDE9E,0xCEEE,0x709E,0x8E8E,0x7070,0x7070,0x9E9E,0xBFBF,0x70BE,0xAEAE,0x709E,0x7070,0xC09E,0x6060,0x709E,0x6060,0x6060,0x4040,0x4050,0x4040,0x6080,0x4040,0x6060,0x4040,0x4040,0x8080,0xB0A0,0xB0B0,0xC150,0x7181,0x6170,0xB081,0xC0B0,0x70B0,0xB080,0x6190,0x4040,0x7272,0xE0E0,0x8080,0x81B0,0x7171,0x8181,0x7070,0xC0C0,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0xD182,0xE0E0,0xE0E0,0xC1E0,0xB1B1,0x8192,0x8172,0xA1A1,0xD1D1,0xF1F1,0xB1C1,0xC1C1,0xB1B1,0xB1A1,0x92B1,0x92A1,0xB2A1,0xB292,0xB2B2,0xC1D2,0xD1D1
.word 0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x5050,0x8F8F,0x9F9F,0x4040,0x8F8F,0x4040,0x7F7F,0x4040,0x6F6F,0x8F8F,0x4040,0x7F7F,0x4040,0x6E6E,0xCECE,0xEFEF,0xBEF1,0xD0D0,0x80D3,0x7070,0x7070,0x6060,0x4040,0x4040,0x4040,0x4040,0x4040,0x4040,0x8F8F,0x7070,0x8F8F,0x9E60,0xBEBE,0xE3F2,0xE6E5,0xE6E6,0xAFAF,0x7070,0x7070,0x7070,0x708E,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x6060,0xAFAF,0x7070,0xAFAF,0x7070,0x8E8E,0x7070,0x7070,0x9F9F,0x8080,0xCE9E,0xF9E7,0xFBFA,0xDFDF,0xE0E0,0xF4F4,0xC7C7,0xAFAF,0x709E,0x7070,0x7070,0x709E,0x7070,0x9E70,0x9E9E,0x9E70,0x9E9E,0xAFAF,0xAE9E,0xBEBE,0x9E9E,0xDEDE,0xCEEE,0xCECE,0x9E9E,0x9EAE,0x9E70,0xDEDE,0x9EAE,0x9E70,0xCFCF,0x9E9E,0xAEAE,0x9E9E,0xCECE,0x9E70,0xCECE,0x7070,0x9E70,0x6060,0x9E70,0x6070,0x5050,0x4040,0x5050,0x4040,0x7070,0x4040,0x5050,0x7F7F,0x4040,0x9040,0x9081,0x70B0,0xD050,0x6190,0x8071,0xA180,0xB0A0,0x6080,0x7060,0x5171,0x7F7F,0xD051,0x7070,0xC280,0x81B1,0x71C0,0x8081,0x7160,0x6060,0x4040,0x7F7F,0x7F7F,0x4040,0x4141,0xE0C1,0xE0E0,0xE0F0,0xD0F0,0x82A1,0x9292,0x7281,0x9282,0xE3E3,0xFFFF,0xB1C1,0xC1C1,0x92B1,0xB2A1,0xC2B1,0xB1B2,0xC2B1,0xB1C1,0xB1B1,0xB2B2,0xD2C2
.word 0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x5050,0xAFAF,0x5050,0x9F9F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x8E8E,0xF2FE,0xFFFF,0xE3E3,0x7E7E,0x7E71,0xAFAF,0x60B0,0x5050,0x5050,0x4050,0x7F7F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x4040,0x5040,0x4040,0x9D9D,0xFAFA,0xF9E5,0xE4E4,0xAFAF,0x7070,0x7070,0x8E70,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x6080,0x6060,0x7050,0x7070,0x9E9E,0x7070,0x8070,0x8E80,0x7070,0x7060,0x9F9F,0xBEBE,0xDCCC,0xBCEB,0xDEDE,0xCFCF,0xF4F4,0xBFBF,0xBFBF,0x7070,0x6070,0x7070,0x7070,0x7070,0x7070,0x7070,0x9E9E,0x9E9E,0x9E9E,0x7070,0x7070,0x7070,0x7070,0x709E,0x7070,0x7070,0x7070,0x7070,0xEE9E,0x9EDE,0x9E9E,0x709E,0x709E,0x9E9E,0x9E9E,0x8E9E,0x8080,0x9EBE,0x9E70,0x8080,0x8080,0x7070,0x6080,0x6060,0xD050,0x4060,0x4040,0x6060,0x4040,0x5040,0x4040,0x4040,0x8040,0x8080,0xA090,0xA0A0,0x71A1,0xC1C1,0x9071,0xA0A0,0x5080,0x4040,0x7F7F,0x4141,0xC1C1,0xC0C0,0x80C1,0x6161,0x7181,0xC081,0x7071,0x7F7F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x7272,0xE0F1,0xE0E0,0xF0E0,0xC1E0,0x7282,0x7181,0x7272,0x7292,0xE392,0xD1F3,0xB1B1,0xB1C1,0xA192,0xB1B2,0xB1C2,0xB1B1,0xD192,0xB1E0,0xB2C2,0xB1C1,0xC1C2
.word 0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x5050,0x5050,0x9F9F,0x5050,0x9F9F,0x9F9F,0x4040,0x7F7F,0x8F8F,0x4040,0x4040,0x8F8F,0x5050,0x9E8E,0xDEDE,0xEECC,0xB4E3,0x7E7E,0xC0C0,0x8080,0x50C0,0x7F7F,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x4242,0x8F8F,0x8F8F,0x8F8F,0x8E7E,0x8C6A,0xF5EC,0xCDCD,0xD4D4,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x6060,0x5050,0x7070,0x7070,0x7070,0xBFBF,0x8080,0x7060,0x7070,0x8080,0xBFBF,0x9C9C,0xAFAF,0xF0F0,0xE0E0,0xCFCF,0xC0E0,0x9F9F,0x8F8F,0x7070,0x7060,0x7070,0x7070,0x708E,0x9E9E,0x7070,0x7070,0x70BE,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x9E70,0x9EBE,0x8E8E,0x7070,0x9E9E,0x7070,0xAE9E,0x7070,0x9E9E,0x9E9E,0xBE9E,0x9E9E,0x9E9E,0x7080,0x709E,0x5050,0xFEFE,0x5050,0x4040,0x7050,0x5060,0x5050,0x4040,0x4040,0x7070,0x8070,0xB081,0x90A1,0xA090,0xB0C0,0xA0B1,0x90A1,0x9090,0x4141,0x4242,0xB3B3,0xF0F0,0xB0C0,0x8080,0x7F7F,0xB281,0x7281,0x8181,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD171,0xE0F0,0xE0F0,0xE0E0,0x81D0,0x8172,0x7272,0x8172,0x7192,0x8272,0x9292,0xB1B1,0x9281,0x9292,0xC2B1,0x92B1,0xA192,0xC192,0xC2C1,0xC2B1,0xC2C2,0xB1B2
.word 0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x5050,0x5050,0x9F9F,0x6060,0x8F8F,0x9F9F,0x8F8F,0x5050,0x8F8F,0x5050,0x5050,0x7070,0x9E9E,0xDFDF,0xBE4A,0xD5DC,0x6262,0x7061,0x8080,0x5050,0x4040,0x8F8F,0x4040,0x5050,0x4040,0x6F6F,0x4242,0x4747,0x4242,0x6F6F,0x8F8F,0x4E4E,0x9F9F,0xAE9C,0xDFDF,0xD0F0,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6060,0x6060,0x7070,0xAFAF,0xAFAF,0x7070,0x6060,0xAFAF,0xD0D0,0xDEBE,0xCECE,0xD5D5,0xD5D5,0xBEBE,0xD0D0,0x80B0,0x618E,0x5252,0x4040,0x7F7F,0x6070,0x5050,0xAFAF,0x9E70,0x9E70,0x709E,0x7070,0x7070,0x9E9E,0x9E9E,0x9E9E,0x7070,0x7070,0x7070,0x708E,0x709E,0x709E,0x708E,0x7070,0x9E70,0x7070,0x9E8E,0x9E9E,0x7070,0x8080,0x8090,0x8080,0xC0C0,0x7070,0x7070,0x8080,0x7070,0x6060,0xDEE0,0xCECE,0x4040,0x5050,0x6060,0x50C0,0x7F7F,0x6060,0x7060,0x7070,0xD0A0,0x80D0,0x9071,0xC0C0,0xC080,0x60D0,0x8080,0x4141,0x4141,0x7F7F,0xFFFF,0x60C0,0x8F8F,0x5151,0xA1C0,0xD2B2,0xAFAF,0x7F7F,0x6F6F,0x6F6F,0x6060,0xC281,0xE0E1,0xE0E0,0xE0E0,0xE0E0,0x82C1,0x8272,0x7182,0x7272,0x7182,0xB272,0xB1C1,0xC292,0x92B2,0xB292,0xC2B1,0x92C1,0x9292,0x9292,0xB2B2,0x92B2,0xB292,0xB292
.word 0x6F6F,0x7F7F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8E50,0x9F9F,0x5050,0x6060,0x5050,0x9F9F,0x8F8F,0x4040,0x5050,0x708E,0x7070,0xCE9E,0xC5E4,0xAFAF,0xEA8D,0xD3D3,0xC0C0,0x70B0,0x4040,0x8F8F,0x7F7F,0x8F8F,0x5070,0x7F7F,0x7F7F,0x4242,0x4A4A,0x6457,0x4842,0x4545,0x5252,0x8E8E,0xE4E4,0x9E9E,0xDEDE,0xBFBF,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x7070,0x6060,0x9F9F,0x7070,0x8080,0x8080,0x6060,0x9F9F,0xC070,0xCECE,0xBFBF,0xD7CB,0xFBDB,0xFAFA,0x7E7E,0x8E8E,0xD0D0,0x7272,0x6262,0x4242,0x6F6F,0x5060,0x6060,0x7070,0x7070,0x9E9E,0x7070,0x7070,0x7070,0x7070,0x9E9E,0x7070,0xAFAF,0x7070,0xAFAF,0x9EC0,0xAFAF,0x9E9E,0x9E70,0x9E70,0x7070,0xAFAF,0x7070,0x7070,0x8080,0x8080,0x8080,0x9090,0x80C0,0x7080,0x7070,0xC070,0x9E80,0x6060,0xEEF0,0x9EBE,0x5050,0x4040,0x7060,0x80DE,0x5050,0x5040,0x6050,0x7070,0xB090,0xB0F0,0x9080,0x80D0,0x6060,0x5060,0x8F8F,0x4241,0x4141,0x4141,0x7C7C,0x8F8F,0x7F7F,0xB0B0,0x6161,0xC2C2,0x9F9F,0x7F7F,0x4040,0x4040,0xD171,0xE0E0,0xE0E0,0xD0D0,0xE0D0,0xC1D0,0x8281,0x7281,0x7271,0x6162,0x7262,0xD192,0xC1D1,0xB2B2,0x92C2,0xB292,0xC2B2,0xA1D2,0x9292,0x8292,0x92A1,0x9292,0x9282,0x8282
.word 0x6F6F,0x4040,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x8F8F,0x6060,0x5050,0x9F9F,0x6060,0x6060,0x607E,0x8F8F,0x4040,0x7070,0x7070,0xFE9E,0xF5F5,0x49D5,0x7D7D,0x72D0,0xC0C0,0x4070,0x7F7F,0x4040,0x4141,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0x4242,0xAFAF,0x4242,0x8C49,0xDBDB,0xBFBF,0xBFBF,0x80AE,0x7070,0x7070,0x8E70,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0xB080,0x8080,0xFECE,0xCFCF,0xBBBB,0xDCDB,0x9DCC,0x8E8E,0xBFBF,0x9E9E,0xCED0,0xE0E0,0xAFAF,0x8F8F,0x7F7F,0x6060,0x5050,0x6060,0x6060,0x709E,0x7070,0x7060,0x7070,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0x9E9E,0xAE9E,0x9EAE,0x9E9E,0x9E9E,0x9E9E,0xCE9E,0x7070,0x6070,0x7070,0x7080,0x7070,0x8070,0xC090,0x8090,0x7070,0x7070,0xAFAF,0x9EAE,0x809E,0xEEF0,0x8ED0,0x5050,0x8F8F,0x8060,0x9090,0x7060,0x8070,0x4060,0x7060,0x8070,0xC0C0,0x9090,0x8080,0x5050,0x7070,0x6052,0x4152,0x7F7F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x9061,0x7171,0x4E70,0x7F7F,0x6F6F,0x5050,0xC171,0xE0E0,0xE0E0,0xD0D0,0xD0D0,0xC0D0,0xB1C2,0x81C2,0x7161,0x7272,0x6261,0xB272,0xD1D1,0xC1C1,0xB2A1,0x92B1,0x9282,0xC1B2,0xC1B1,0x9282,0x8282,0x8282,0x8282,0x8282,0x8282
.word 0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x6060,0x9F9F,0x9F9F,0x6060,0x5050,0x8F8F,0x6060,0x8E50,0xCE9E,0x9E9E,0xC4C4,0xAFAF,0x7070,0xC3C3,0x41A1,0x7F7F,0x7F7F,0x6F6F,0x4242,0x4040,0x8F8F,0x4040,0x8E8E,0x8080,0x7F7F,0x8F8F,0xAFAF,0x9F9F,0x6262,0x484A,0x6161,0x6E6E,0xAFAF,0x5050,0x608E,0x7070,0x8E70,0xAFAF,0xC0C0,0x7070,0x7070,0x8080,0x9F9F,0x7070,0x6060,0x808E,0xE0E0,0xBFBF,0xF0F0,0xD5F0,0x7B7B,0x7E7D,0x7E7E,0x8F8F,0x5151,0xD050,0xD0D0,0x9F9F,0x9F9F,0x4141,0x4242,0x6050,0x5050,0x6060,0x708E,0x7070,0x709E,0x7070,0x7060,0x7070,0x7070,0x7070,0x7070,0x9E9E,0x7070,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x709E,0x7070,0x6070,0x7060,0x7070,0x7070,0x7070,0x7070,0x80C0,0x8080,0x7070,0x7070,0xAE9E,0x709E,0x7070,0xD0D0,0x9E9E,0xAEAE,0x6050,0x6060,0xD070,0x80F0,0x9E9E,0x6080,0x7060,0x7050,0x8080,0xC0C0,0x8E50,0x6050,0x5261,0x6161,0x5151,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xD3E3,0x9081,0x8F8F,0x7F7F,0x6060,0xD181,0xE0E0,0xD0D0,0xC0D0,0xC0C0,0xB2C0,0xB2B2,0xC0B1,0x7182,0x7251,0x7271,0x8272,0xC292,0xD1D1,0xA1C1,0x9282,0xB2B2,0x8282,0xB1A1,0xB2C2,0x82B1,0x7272,0x7272,0x7272,0x7282,0x8282
.word 0x4040,0x8F8F,0x4040,0x8F8F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x8F8F,0x9F9F,0x7070,0x6060,0x7070,0x7070,0x6060,0x6060,0x5050,0x7060,0x8E8E,0xBE6E,0xC3E3,0x81B1,0x7070,0xD0D0,0x5151,0x7F7F,0x4242,0x4242,0x4242,0x7F7F,0x4040,0x5050,0x5060,0x4141,0x8F8F,0x5050,0x8F8F,0xAFAF,0xC4C4,0x4242,0xD4C7,0xD6D4,0xAFAF,0x6060,0x6060,0xAFAF,0x7070,0x8E8E,0x8080,0xAFAF,0x8080,0x7070,0x7070,0xAFAF,0x7070,0x9E70,0xD0D0,0xBEBE,0xBEBE,0xCCCC,0x4949,0x4242,0x8F8F,0x8F8F,0x9F9F,0xBFBF,0x80D0,0x40B0,0x4E4E,0x9F9F,0x7F7F,0x4141,0x4040,0x8F8F,0x7070,0x7070,0x9E9E,0x7070,0x7070,0x709E,0x7070,0x709E,0x7070,0x7070,0x7070,0x7070,0x9E9E,0x9E9E,0xAFAF,0xAFAF,0x7070,0x6060,0x7070,0x7070,0x7070,0x7070,0xC070,0x8080,0x7080,0x7070,0x7070,0x7080,0x9E9E,0x7070,0xC0C0,0x9E70,0x9E9E,0x7070,0x70CE,0x7050,0xE0FE,0x8090,0x7070,0x6080,0x6060,0x5050,0x7E7E,0x4050,0x6161,0x6161,0x5151,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x6F6F,0xBFBF,0xAFAF,0x6F6F,0x4040,0xC171,0xD0E0,0xC0C0,0xD0D0,0xC0C0,0xA181,0xB192,0xB1B1,0x72C0,0x8181,0x7162,0x7272,0xB2B1,0xD192,0xC1D1,0x72B1,0x8272,0xB1B2,0x8292,0xB292,0xB2B2,0xB2C2,0x7172,0x7172,0x7172,0x7272,0x7271
.word 0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x6060,0x9F9F,0x6060,0x7070,0x9E9E,0x6060,0x6060,0x6060,0x709E,0xAFAF,0x6E6E,0xE4E4,0xE4E4,0x7272,0x4242,0x6262,0x4242,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6060,0x9F9F,0x7180,0x5151,0x7070,0x9F9F,0x8F8F,0xBB4A,0xBBBB,0xD5D5,0xDBE6,0xF4FE,0xAFAF,0x6060,0x6050,0x7070,0xAFAF,0x7070,0xAFAF,0xD4D4,0xD5D5,0xAFAF,0x9F9F,0x6060,0x9F9F,0x8080,0x9F9F,0x7E7E,0x6A6A,0xAFAF,0x9F9F,0x8F8F,0xAFAF,0x7F7F,0xD0D0,0xFEFE,0xB0FE,0x8F8F,0x4140,0x6F6F,0x4141,0x7F7F,0x7F7F,0x6060,0x5060,0x8E70,0xAFAF,0x7070,0xAFAF,0x9E70,0x8E70,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x9EC0,0x7070,0x7070,0x7070,0xC080,0x8080,0x7080,0x7070,0x9E70,0x8080,0x8E70,0x9E9E,0xD0D0,0x8E90,0x9E70,0x609E,0x9E9E,0x7060,0x9090,0x8080,0x7070,0xC0C0,0x6060,0x7F7F,0x4040,0x5141,0x4141,0x6161,0x4141,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0xD241,0xF1F1,0xC0C0,0xB0C0,0xA1B0,0x9281,0xB1B2,0xC1B1,0x71D0,0x81C0,0x5181,0x7172,0x9282,0xA1B1,0xD0B2,0xC1E0,0x6281,0x8271,0xC2B2,0x8292,0x8282,0xB292,0xB292,0x82B2,0x7162,0x6272,0x7172,0x7272
.word 0x7F7F,0x8F8F,0x5050,0x4040,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x7E50,0x7070,0x7070,0x6060,0x6060,0x5050,0x6060,0x6060,0x9E70,0x7070,0x8E8E,0xD3DE,0xC3C3,0x4862,0x6161,0x8080,0x4271,0x7F7F,0x6F6F,0x6262,0x6F6F,0x6060,0x8F8F,0xC0C0,0x7070,0x6161,0x4141,0x7F7F,0x4A4A,0x9C6A,0xBFBF,0xBBBB,0xDBDB,0x8F8F,0x8080,0x7080,0xAFAF,0x7070,0xAFAF,0x9F9F,0xBFBF,0xAFAF,0x6060,0x6060,0x5060,0x5050,0x9F9F,0xC0C0,0x8F8F,0x9F9F,0x4A4A,0x7F7F,0x8D8D,0x9F9F,0x8F8F,0x8E8E,0xEEEE,0xDEDE,0x4040,0x7F7F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x9F9F,0x6050,0x708E,0x709E,0x6060,0x7070,0x7070,0x709E,0x7070,0x7070,0x708E,0x708E,0x7070,0x7070,0x9E9E,0xAFAF,0x7070,0x9E9E,0x9E70,0x9E9E,0x7070,0x7070,0x7070,0x8080,0x8080,0x7070,0x7070,0x8070,0x8080,0x7070,0xAFAF,0xE0E0,0x8080,0x9E9E,0x70AE,0xBE60,0x6080,0x8070,0xDEDE,0xCFCF,0x9F9F,0x8F8F,0x7F7F,0x4242,0x8F8F,0x5151,0x4040,0x4141,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0xD4D4,0xD0D0,0x8181,0xB281,0xA1A1,0xC0B1,0x8080,0xC0C0,0xC0C0,0xC371,0x7171,0x7171,0x8272,0x82A1,0xE0B1,0xB1D1,0x6162,0x6262,0xC292,0x7292,0x7282,0x8282,0x9292,0xB1B2,0x7292,0x7162,0x7172,0x7272
.word 0x4040,0x4040,0x9F9F,0x5050,0x4040,0x4040,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4050,0x9F9F,0x709E,0x6060,0x6060,0x6060,0x6060,0x7070,0x7070,0x7070,0xBFBF,0xDE7E,0xD1F1,0x7171,0xC2E0,0x9F9F,0xB0C0,0x6161,0x6F6F,0x9090,0x4161,0x8F8F,0x5050,0x8F8F,0x7070,0x9F9F,0x7F7F,0x4141,0x8F8F,0xD667,0xBEBC,0x45CB,0xAE6C,0x5252,0x9E9E,0x9E9E,0xAFAF,0x9E80,0x7070,0x7070,0x9F9F,0x5050,0x9F9F,0x6060,0x7070,0x5050,0x9F9F,0x8080,0x4A42,0x4A4A,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0xDE9E,0x7070,0x4040,0x4040,0x4040,0x4141,0x4141,0x6F6F,0x7F7F,0xAFAF,0x7070,0x7070,0x9F9F,0x7070,0x9E9E,0x7070,0x7070,0x9E70,0x7070,0x7070,0x7070,0xAEAE,0x8080,0x7070,0x8E8E,0xD0D0,0x6070,0x709E,0x7060,0x7070,0x709E,0x7070,0x7070,0x7070,0x7070,0x7070,0x7090,0x8E70,0x7070,0xC080,0xDFDF,0x9E9E,0xAEBE,0xAFAF,0x80AE,0x6070,0x6060,0x709D,0x4040,0x7F7F,0x4040,0x4141,0x4141,0x4141,0x4040,0x4040,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0xAFAF,0xE0E0,0x81A1,0x71C0,0x7171,0x7F7F,0x7F7F,0xC0C0,0x7170,0x81C0,0x5151,0x7172,0x8172,0xB2B2,0xE0C1,0x71D0,0x6252,0x6161,0xB282,0x7282,0x7271,0x7272,0x8282,0x9292,0x9292,0x6272,0x6271,0x7172
.word 0x5050,0x5050,0x5050,0x6060,0x5050,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x6060,0x6050,0xAFAF,0x7070,0x6060,0x6060,0x6070,0x7070,0xAFAF,0x9E70,0xC09E,0xFE5C,0xFFFF,0xDED0,0xD1F0,0x80A0,0x8080,0x61B0,0x4141,0x9F9F,0x9F9F,0x8F8F,0x5050,0x7F7F,0x7E7E,0x5050,0x4040,0x7F7F,0x9F9F,0xEABB,0x48D5,0x7C7C,0x7C7C,0x9F9F,0x7070,0x7070,0x7070,0xAFAF,0x8E70,0x7070,0x6070,0x6060,0x7070,0x5060,0x7070,0x6060,0xAFAF,0x4343,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x4848,0x7F7F,0x9E9E,0x4060,0x4040,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0xBFBF,0x7070,0x7070,0x7060,0x709E,0x6060,0x6060,0x9F9F,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x6060,0x9E9E,0x5060,0x9E9E,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x809E,0x7080,0x7070,0x8080,0x9080,0xD0E0,0x9E9E,0x9E9E,0x709E,0xBEBE,0x6070,0x5050,0x7E50,0x4040,0x4040,0x8F8F,0x4141,0x7F7F,0x4141,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x6F6F,0x4040,0x7F7F,0x508C,0x8F8F,0x7070,0x6161,0x6F6F,0x6F6F,0xC0C0,0x7171,0xC071,0x4151,0x7182,0x8271,0xC1C1,0xE0E1,0x61C1,0x6161,0x6252,0xC272,0x62B2,0x7172,0x7272,0x7271,0x8272,0x9282,0x92B2,0x7162,0x7272
.word 0x5050,0x5050,0x6060,0x5050,0x6060,0x5050,0x4040,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x5050,0x8F8F,0x6060,0x708E,0x5050,0x6060,0x6060,0x6060,0x8E70,0x5050,0x8E8E,0x8E70,0xAEAE,0xBFBF,0xE0F0,0x80D0,0x7070,0x8070,0x6060,0x5050,0x7F7F,0x5252,0x8F8F,0x4050,0x5040,0x4040,0x4242,0x7F7F,0x8F8F,0xAFAF,0x9F9F,0x4242,0x7F7F,0x8F8F,0xC0C0,0x8E70,0x8E8E,0x7070,0x7070,0x7070,0xAFAF,0x7070,0xB0B0,0x6060,0x9F9F,0x9E9E,0x7070,0x9F9F,0x4747,0x8F8F,0x7F7F,0x4242,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5050,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x8F8F,0x6070,0x7070,0x608E,0x5050,0x7070,0x7070,0x7060,0x707E,0xAFAF,0x6070,0x7070,0x7070,0x6070,0x5050,0x6060,0x6060,0x9E9E,0x708E,0x7070,0x7070,0x7070,0x9E70,0x8070,0x7070,0x7070,0x7070,0x9090,0xC0C0,0xAEAE,0x709E,0x7080,0x9E70,0x508E,0x4040,0x4040,0x4040,0x8F8F,0x4040,0x4141,0x4141,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6161,0x8080,0x6F6F,0x4040,0x7171,0x8181,0x81C3,0x5151,0xC371,0x7272,0xE1D1,0xF5F5,0x52D0,0x5261,0x6162,0xC262,0x6262,0x6271,0x7172,0x7262,0x7271,0x8272,0xD3D3,0x72C3,0x7271
.word 0x5050,0x6060,0x5050,0x6060,0x7070,0x6060,0x4040,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x5050,0x5050,0x8E70,0x7070,0x5050,0x6060,0x7070,0x8080,0x5050,0x4050,0x7070,0x7070,0x9E8E,0xEEEE,0xC0C0,0x6070,0x9090,0x7070,0x4060,0x7F7F,0x7F7F,0x5050,0x8F8F,0x8F8F,0x4040,0x7F7F,0x6F6F,0x9F9F,0x9F9F,0x4242,0x4747,0x7F7F,0x7F7F,0x9E9E,0x70C0,0xAFAF,0x6060,0x7070,0x6060,0x7070,0xAFAF,0x7070,0x6060,0x5050,0x8F8F,0x7070,0x9F9F,0x8F8F,0x7F7F,0x4646,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7E7E,0x9F9F,0x7040,0x5060,0x5050,0x5050,0x4040,0xCEC0,0x9F9F,0xAFAF,0x7050,0x7070,0x5050,0x6050,0x6060,0x6060,0x4040,0x5050,0x8E50,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x9E70,0x7070,0x7070,0x7070,0x7070,0x8080,0x8080,0xAEC0,0x7070,0x708E,0x7050,0x6070,0x7F7F,0x5050,0x8F8F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x7F7F,0xB272,0x6060,0x4040,0x6F6F,0x7171,0x81C0,0xC080,0x5250,0x7181,0x7271,0xF2F2,0xCDEC,0x5252,0x5261,0x6161,0x6162,0x7172,0x6262,0x6271,0x7172,0x6162,0x7162,0x8282,0x7172,0x7172
.word 0x5050,0x6060,0x6060,0x6060,0x7070,0x6060,0x5050,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x5050,0x5050,0x7070,0x6060,0x6060,0x9F9F,0x6060,0x8E70,0x7070,0x5060,0x9E7E,0x7070,0x7070,0x9E9E,0x7070,0x7070,0x7070,0x6070,0x5050,0x4040,0x5050,0x4141,0x4050,0x4040,0x8F8F,0x5151,0x4141,0x5050,0x8F8F,0x424A,0x7F7F,0x6F6F,0x6E6E,0xAEAE,0xBFBF,0x7070,0x8E70,0x7070,0x7070,0x7070,0x5050,0x9F9F,0x6060,0x8F8F,0x5050,0x7E7E,0x5050,0x7F7F,0x4242,0x8F8F,0x4646,0x4646,0x4242,0x6F6F,0x7F7F,0x7050,0x5050,0x4040,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x5050,0x4050,0x8F8F,0x6060,0x4040,0x8E8E,0x6070,0x6050,0x9E9E,0x6060,0x5050,0x5040,0x4040,0x4040,0x4040,0x5040,0x6060,0x9E9E,0x7070,0x709E,0x7070,0x7070,0x9E70,0x7070,0x7060,0x709E,0x9E70,0x9E9E,0x9080,0x9E80,0x709E,0x7070,0x5050,0x5050,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6161,0x8F8F,0x7F7F,0x7F7F,0x7140,0x7171,0x71C0,0x7161,0x7281,0xC071,0xBFBF,0x5151,0x6162,0x6152,0x6252,0x6261,0x6162,0x7162,0x7162,0x6262,0x6271,0x6271,0x7162,0x7172,0x6272
.word 0x5050,0x6060,0x5060,0x5050,0x9E60,0x5080,0x5050,0x5050,0x4040,0x4040,0x4040,0x5050,0x6060,0x6060,0x6060,0x6070,0x6060,0x6070,0x6060,0x6070,0x9E9E,0x6080,0x7E50,0x8E8E,0x709E,0x6060,0x9F9F,0x7070,0xAFAF,0x6060,0x9F9F,0x5050,0x9F9F,0x5050,0x4040,0x7F7F,0x4040,0x8F8F,0x5050,0x7060,0x4242,0x5858,0x4747,0x4242,0x4A4A,0x8C8C,0x7E4E,0xAFAF,0x6060,0x7E7E,0x7070,0x709E,0xAFAF,0x6262,0x9F9F,0x8F8F,0x7E7E,0x4040,0x7F7F,0x7F7F,0x4A4A,0x7F7F,0x7F7F,0x7F7F,0x4242,0x6F6F,0x8080,0x80D0,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x5050,0x4040,0x5050,0x5040,0x4050,0x507E,0x6040,0x5060,0x5050,0x5050,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x9E9E,0x7070,0x7070,0x7090,0x709E,0x709E,0x709E,0x7060,0x8E70,0x708E,0x8E70,0x7070,0x7070,0x9E70,0x6060,0x5050,0x4040,0x8F8F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x4040,0x4040,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7171,0x70C0,0xC0C0,0x7172,0x7271,0x7171,0x6161,0x5162,0x5251,0x6151,0x5261,0x5261,0x6261,0x6271,0x6262,0x6271,0x7162,0x7162,0x6272,0x7271,0x7271
.word 0x5050,0x5060,0x6060,0x7070,0x8E70,0x6060,0x6060,0x6060,0x5050,0x4050,0x8F8F,0x6060,0x6060,0x6060,0x7070,0x7070,0x5070,0x6060,0x6060,0x9E60,0x7070,0x6060,0x4040,0x9E8E,0x6070,0x6060,0x6050,0x5050,0x6060,0x4040,0x4040,0x5050,0x6060,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x5050,0xAFAF,0x4848,0x4848,0x4242,0x7F7F,0x4A4A,0x4848,0x6E6E,0x6060,0x7E7E,0x7070,0x8080,0xAFAF,0xC5C5,0xAFAF,0x4848,0x9F9F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x7F7F,0x7F7F,0x7F7F,0x6060,0x6060,0x5050,0x4040,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4040,0x4040,0x8F8F,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x5050,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x5050,0x7070,0x709E,0x9E60,0x7070,0x7070,0x9E9E,0x7070,0x9E9E,0x6070,0x5050,0x5050,0x7070,0x5050,0x5050,0x8F8F,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7171,0x7170,0xC371,0x8172,0x7171,0x5252,0x5261,0x6161,0x5261,0x5152,0x6162,0x6162,0x6162,0x6171,0x7162,0x6271,0x6271,0x7171,0x7162,0x7172
.word 0x5050,0x5050,0x5050,0x7070,0x7060,0x6060,0x6060,0x7070,0x6060,0x6060,0x7070,0x6060,0x6070,0x6060,0x8080,0x9E9E,0x9090,0x7070,0x6060,0x7070,0x7070,0x4060,0x4040,0x5050,0xAFAF,0x7171,0x8080,0x5050,0x9F9F,0x5050,0x7F7F,0x4040,0x6060,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x5050,0x8080,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0xAFAF,0x4949,0x6A6A,0x7E7E,0x9F9F,0x9E8E,0xAFAF,0x8080,0x7171,0x4747,0x4A4A,0x5252,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x706E,0x5070,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x8F8F,0x6040,0x7060,0x709E,0x7060,0x9E60,0x709E,0x6060,0xAFAF,0x5050,0x4040,0x4040,0x8F8F,0x8F8F,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x6161,0xB0C0,0xD4D4,0x71C0,0x7171,0xD061,0x6161,0x5261,0x5261,0x6152,0x6161,0x5261,0x6261,0x6162,0x6262,0x6261,0x7162,0x7172,0x7262,0x7271,0x7262
.word 0x5040,0x6050,0x6060,0x6060,0x6060,0x6070,0x7070,0x8E60,0x7070,0x7070,0x7070,0x6060,0x6070,0x6060,0x9E9E,0x9E80,0x708E,0xAFAF,0x9E70,0x7070,0x7070,0x6060,0x8E8E,0x6090,0x708E,0x8E8E,0x61C4,0x6060,0x5050,0x8F8F,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x59C5,0x5959,0x4A4A,0x9F9F,0x8F8F,0x707E,0x7070,0xAFAF,0x6060,0x8F8F,0x4A4A,0x4848,0x4141,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8090,0x6070,0x6060,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x5050,0x6060,0x6070,0x7070,0x6070,0x5050,0x5050,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x4040,0x6060,0x7171,0xB080,0x6171,0x6171,0xAFAF,0x5261,0x5161,0x6152,0x6251,0x6152,0x6152,0x6162,0x6261,0x6161,0x7172,0x7271,0x7162,0x7172,0x7172,0x7171
.word 0x8F8F,0x5050,0x5050,0x6060,0x7070,0x7070,0x7060,0x6060,0xAFAF,0x7070,0x7070,0x7070,0x8E70,0x7070,0x7080,0x7070,0x6070,0x6070,0x6060,0x7070,0x708E,0xAFAF,0x9E80,0x7070,0x7070,0x6060,0x7070,0x6070,0x5060,0x5050,0x4040,0x6F6F,0x5050,0x4141,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6060,0x4040,0x7F7F,0x6F6F,0x4242,0x7F7F,0xAFAF,0x7B7B,0x9F9F,0x5C5C,0x5060,0x9F9F,0xAFAF,0x8F8F,0x8F8F,0x4A4A,0x8F8F,0x4A4A,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x6060,0x9E9E,0xAFAF,0xD0AE,0x40B0,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x8F8F,0x5050,0x8F8F,0x4040,0x8F8F,0x8F8F,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x4040,0x4040,0x5050,0x4040,0x4040,0x4040,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x6180,0x7171,0x6171,0x7272,0x6161,0x5161,0x6152,0x5261,0x5161,0x5261,0x6161,0x6152,0x6162,0x6262,0x6261,0x6271,0x6271,0x6271,0x6271,0x6272
.word 0x5060,0x6060,0x6060,0x6060,0x8E70,0x5060,0x6060,0x6060,0x9E70,0x9E70,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0x6080,0x6060,0x708E,0x7070,0x7070,0x7070,0x7070,0x9E70,0x9E9E,0x5070,0x7070,0x6060,0x9F9F,0x6060,0x5050,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x4040,0x4040,0x7F7F,0x6F6F,0x4646,0x7F7F,0x8F8F,0x5959,0x7A7A,0x8F8F,0x7E7E,0xAFAF,0x7070,0x6060,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7070,0x7070,0x9E9E,0x9E9E,0x5050,0x7F7F,0x7F7F,0x4040,0x4040,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x4040,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4141,0x70D0,0xC080,0x61C3,0x7171,0x5152,0x6252,0x5251,0x6161,0x6152,0x6152,0x6252,0x6261,0x6261,0x7161,0x7162,0x6262,0x7271,0x6271,0x7271,0x7171
.word 0x8F8F,0x6060,0x6070,0x6060,0x6060,0x5060,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x9E70,0x7070,0x7070,0x7060,0x709E,0x7070,0x7070,0x608E,0x4040,0x6060,0x5060,0x9F9F,0x7070,0x6060,0x7070,0x6060,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4141,0x4141,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4242,0x7F7F,0x7F7F,0x4A4A,0x9F9F,0x4A4A,0x7F7F,0x4E4E,0x708E,0xAFAF,0x9F9F,0x6060,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x4242,0x7F7F,0x7F7F,0x7F7F,0x4242,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x8F8F,0x7070,0xBEBE,0x7070,0x5050,0x4040,0x4040,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x5151,0x7171,0x7171,0x6171,0x6161,0x5261,0x5161,0x6152,0x5251,0x6261,0x6151,0x5161,0x5162,0x6162,0x6262,0x6162,0x7162,0x7162,0x7162,0x7162,0x7262
.word 0x6040,0x7070,0x7070,0x6060,0x6060,0x5050,0x5050,0x6060,0x7070,0x6060,0x8E8E,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6070,0x9E9E,0x7070,0x709E,0x6070,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5070,0x8F8F,0x6060,0x5050,0x7F7F,0x7F7F,0x6F6F,0x4142,0x5151,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4242,0x4242,0x7F7F,0x8F8F,0x9F9F,0x4242,0x7F7F,0x4C4C,0x6060,0x9F9F,0x6070,0x8F8F,0x5050,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6050,0x9E9E,0x7070,0x9E9E,0x7070,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x4040,0x4040,0x6F6F,0x4040,0x4040,0x4040,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x6F6F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4040,0x4040,0x7180,0x8171,0x6171,0x5151,0x5151,0x5152,0x5261,0x6161,0x5152,0x5261,0x6152,0x6152,0x6162,0x7162,0x6271,0x7171,0x7172,0x6272,0x7271,0x7171
.word 0x4040,0x7070,0x7070,0x7E60,0x5060,0x5050,0x6060,0x7070,0x7070,0x7070,0x7070,0x709E,0x7070,0x7070,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x5050,0x6F6F,0x7F7F,0x4040,0x8F8F,0x5050,0x8F8F,0x4141,0x4141,0x4141,0x6F6F,0x8F8F,0x8F8F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4242,0x7F7F,0x4A4A,0x4A4A,0x7F7F,0x7F7F,0x7F7F,0x7E7E,0x8F8F,0x9F9F,0x5050,0x8F8F,0x7F7F,0x7070,0xC1C1,0x6161,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x7F7F,0x6060,0x7070,0x709E,0xAFAF,0x5050,0x7F7F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x4040,0x7F7F,0x7171,0xC371,0x5182,0x5141,0x5251,0x5251,0x5161,0x5152,0x6261,0x6151,0x5261,0x6161,0x6262,0x6261,0x7162,0x6262,0x6271,0x7171,0x7162,0x6262
.word 0x5050,0x7060,0xAFAF,0x5060,0x5050,0x6060,0x5050,0x6060,0x6060,0x6060,0x7070,0x708E,0x709E,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x9E9E,0x6060,0x5060,0x4040,0x7F7F,0x6F6F,0x7F7F,0x4050,0x8F8F,0x4040,0x8F8F,0x4141,0x4242,0x4242,0x4242,0x7F7F,0x4141,0x6F6F,0x6F6F,0x4242,0x6F6F,0x4646,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4848,0x7F7F,0x7F7F,0x4040,0x5050,0x5050,0x8F8F,0x6161,0x8F8F,0x7F7F,0xAE7E,0x7070,0x7F7F,0x7F7F,0x4242,0x6F6F,0x6F6F,0x4242,0x4242,0x6F6F,0x7F7F,0x7070,0x6060,0x8070,0x5060,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7170,0x7261,0x7171,0x8F8F,0x5151,0x5151,0x6152,0x5251,0x5151,0x5162,0x6152,0x6261,0x6161,0x7162,0x6271,0x7171,0x7162,0x6272,0x7271,0x7171
.word 0x6060,0x6060,0x7070,0x5060,0x5050,0x5050,0x5060,0x5050,0x7060,0xB08E,0x6050,0x6060,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x6070,0x9F9F,0x8F8F,0x4040,0x6F6F,0x6F6F,0x5050,0x4040,0x8F8F,0x4040,0x8F8F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4747,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7070,0x6060,0xAEAE,0x6060,0x4040,0x4040,0x4040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6060,0x4040,0x4040,0x4060,0x6050,0x6070,0x6060,0x4040,0x4040,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4040,0x4040,0x4040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x4040,0x4040,0x6F6F,0x4040,0x7F7F,0x7F7F,0x4040,0x7171,0x7171,0x6161,0x4141,0x5251,0x5251,0x5151,0x6152,0x6251,0x5251,0x6161,0x6262,0x6261,0x6171,0x6262,0x6271,0x7162,0x7162,0x7272,0x7262
;*/


/* Fish
Fish_2:
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5040,0x8F8F,0x8F8F,0x5050,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xD5E6,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x5C4A,0x7C7C,0x8C7C,0x7B8B,0x6B8C,0x7B7C,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x7C7C,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F7F
.word 0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7C6E,0x7B8C,0x7B8C,0x8C7B,0x8C8B,0x7C8C,0x8F8F,0x7F7F,0x7C7C,0x8F8F,0x6C7E,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F6F
.word 0x8F8F,0x8F8F,0x5040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5060,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7E50,0x9F9F,0x8F8F,0xAFAF,0xAFAF,0xDCBE,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7E6C,0x8D8C,0x9F9F,0x8C4A,0xAFAF,0x8B8B,0x9F9F,0x7F7F,0x8F8F,0x7D7E,0x4A7E,0x4A6E,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F7F
.word 0x4040,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x6060,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x8F8F,0x9F9F,0xAE8E,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xAEBC,0xBFBF,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6C4A,0x8D7C,0x8D8D,0x9F9F,0x7C7C,0x8B8C,0xAFAF,0x4A8C,0x8F8F,0x6E4A,0xAFAF,0x7E7C,0x9F9F,0x4A7E,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x707E,0xAFAF,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F6F
.word 0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7060,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8B8B,0xAFAF,0x8F8F,0x7F7F,0x8F8F,0x7E4A,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F
.word 0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x6060,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x708E,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xCFCF,0xBFBF,0xAFAF,0xBFBF,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x8F8F,0x8C6B,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F7F
.word 0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x7070,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x708E,0xAFAF,0xBFBF,0xAFAF,0x8E70,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xBFBF,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7B4A,0x4A7B,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6C4A,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F
.word 0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9E9E,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x4A4A,0x7B8C,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6C6C,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F7F
.word 0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x7060,0x9F9F,0x9F9F,0x7060,0xAFAF,0x7070,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0x9F9F,0x7F7F,0x8F8F,0xAFAF,0x809E,0x709E,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0x9F9F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x5C4C,0x7C4B,0x7C7C,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F6F
.word 0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0xCFCF,0xBFBF,0xAFAF,0xBFBF,0xCFCF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x8E70,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x4A4B,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8C7C,0x7D7E,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x5040,0x9F9F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F7F
.word 0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x7060,0x9F9F,0x7060,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAED0,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0x49BB,0x8F8F,0x9F9F,0x499C,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x6C4A,0x7F7F,0x4A4C,0x7F7F,0x6F6F,0x6C5C,0x8C7D,0xAFAF,0x8D8C,0x4D7D,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F
.word 0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6070,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x7E60,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8E70,0x6070,0x8F8F,0x9F9F,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x8B9C,0x7C8C,0xAFAF,0x7C8C,0x4A4E,0x7F7F,0x7F7F,0x4C4A,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F7F
.word 0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x9F9F,0x9F9F,0x6060,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x707E,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x6070,0x9F9F,0x9E70,0xAFAF,0x709E,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x8B8B,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x4A8C,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F6F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6070,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5060,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x8E70,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8E50,0x9E70,0xAFAF,0x9E70,0x7070,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xBBAC,0x6A9B,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4A4A,0x7F7F,0x7F7F,0x9F9F,0x7C4A,0x7E6B,0x9F9F,0x8F8F,0x4A5C,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F7F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6060,0x6060,0x9F9F,0x9F9F,0x8F8F,0x4050,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9E70,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x709E,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xCBCB,0xDACB,0x9CCB,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x4A7C,0x7C4A,0x9F9F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5060,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5050,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x708E,0x9F9F,0x9F9F,0x8F8F,0xAFAF,0x708E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9B8B,0xCBCB,0xCBDA,0xBBDA,0x4A8C,0x5C5C,0x5C6C,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0x7F7F,0x6F6F,0x8F8F,0x6C4A,0x6B4A,0x7C6B,0x8F8F,0x7F7F,0x4A6C,0x4A7C,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6060,0x9F9F,0x6050,0x9F9F,0x5050,0x9F9F,0x6060,0x9F9F,0x7070,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x7070,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0xAFAF,0x9E70,0xAFAF,0x708E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8EC0,0xBFBF,0xAFAF,0xAFAF,0x8B8D,0x9B8B,0xBB9B,0xBBBB,0xBBBB,0x8C9B,0x8B7B,0x8B8B,0x4A4A,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x7C7C,0x8C7B,0x6C6B,0x4A4E,0x8F8F,0x8F8F,0x4A6C,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F
.word 0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x6060,0x9F9F,0x6050,0x9F9F,0x6060,0x9F9F,0x7070,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7060,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9E80,0xAFAF,0xAFAF,0x7070,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xBC9B,0xBFBF,0xCBDA,0x9BDA,0xBBAB,0xDADA,0x7C9C,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6B6C,0x8C7B,0xAFAF,0x9F9F,0x7E7C,0x7E7C,0x7C7C,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x7060,0x9F9F,0x5060,0x9F9F,0x9F9F,0x9F9F,0x7070,0x9F9F,0xAFAF,0x7070,0x5050,0x9F9F,0x9F9F,0x7070,0xAFAF,0xBFBF,0xC080,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0x709E,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0x9F9F,0xAFAF,0xBC9C,0xBFBF,0xBFBF,0xBBDB,0xABDA,0xBBCA,0xDACA,0x8B9B,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x9C7B,0x9C8B,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x7C7C,0x9F9F,0x4E4A,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F
.word 0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x6060,0x6060,0x9F9F,0x5050,0x9F9F,0x6060,0x9F9F,0x9F9F,0x6060,0x9F9F,0xAFAF,0xAFAF,0xD0B0,0xD0D0,0xC0D0,0xD0D0,0xD0D0,0xB1D0,0xC2C2,0xC3B3,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x708E,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0x9BBB,0xAB9B,0xBBBB,0x9BBB,0x4A8C,0x4A4C,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7B6C,0x8B8B,0x7B8B,0xAFAF,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x6E7E,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F
.word 0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5050,0x9F9F,0x9F9F,0x7070,0xAFAF,0xC070,0xBFBF,0xC0C0,0xD0D0,0xC1C1,0xB1B1,0x91C0,0xB1C0,0xC1C1,0x92A1,0xB2A1,0x92A1,0x8171,0x7182,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0xBFBF,0xAFAF,0x8E70,0x6070,0xAFAF,0x9E70,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9B9B,0xCA9B,0xDABB,0x699B,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7C5C,0x8B7B,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x9C8D,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x408F
.word 0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x6060,0x5060,0x5060,0x8F8F,0x5060,0x9F9F,0x9F9F,0xAFAF,0xD080,0xD0C0,0xCFCF,0xD0C0,0xB1C1,0xC292,0xB1C1,0x91B2,0x8182,0x9192,0x9291,0x9292,0x8282,0xB2B2,0x81C2,0xB182,0x81A1,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9E70,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x484A,0x8F8F,0x9F9F,0x8B9C,0x9B9B,0xDABB,0xBBBB,0x7B8B,0x4E4A,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x7B7C,0xAFAF,0x9F9F,0x9F9F,0x7F7F,0x8F8F,0x8C7B,0x9F9F,0xAFAF,0x4A7C,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5060,0x9F9F,0x9F9F,0x6060,0x8F8F,0x9F9F,0xAFAF,0xC0C0,0xD0D0,0xC1D0,0xC0C0,0x9181,0xA192,0xA192,0xC1B1,0xC1C1,0xC0C1,0xAFAF,0xC0B0,0xA1B1,0xB1A1,0x91A1,0x9292,0x8281,0xB192,0x9F9F,0xA080,0xB0B0,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xBFBF,0xAFAF,0xAFAF,0x9E70,0xAFAF,0x709E,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x598B,0x8B8B,0x8BCA,0x8C8B,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x9C4A,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8C8B,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F8F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x6050,0x9F9F,0x5050,0x5060,0x9F9F,0x9F9F,0x9F9F,0xC070,0xB0C0,0x91B1,0x9292,0x92B1,0xBFBF,0xBFBF,0xBFBF,0xC0BE,0xB1A1,0xC0C1,0xA1A1,0x9281,0xA1A1,0xC0C0,0xC0C0,0x71B0,0xC0C0,0x8281,0xB1B2,0xA1A1,0xB0B2,0xC0B0,0x9170,0xA1B0,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x9E70,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4847,0x8F8F,0x8F8F,0x8C7E,0x9C7B,0x7A8B,0xAFAF,0x9F9F,0x4A7E,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7C4E,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x6050,0x9F9F,0x9F9F,0x5050,0x9F9F,0xD080,0xC0D0,0xA1B0,0xC1A1,0xBFBF,0xD0C0,0xBFBF,0xA1A0,0xA1A1,0x9191,0x9282,0x9292,0x8192,0x9292,0x9282,0x8292,0xB282,0xA192,0xAFAF,0xC0AE,0x9281,0x82B1,0xB292,0xA1B2,0xB2C0,0xB292,0xA1A1,0xAFAF,0xA0B0,0x90A0,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xAFAF,0xAFAF,0x709E,0x8E70,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x4A4A,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4E4C,0x9F9F,0x9E9E,0x8E9E,0x8E8E,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x4A4A,0x6C6C,0x6C7C,0x6C6C,0x9F9F,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F40
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x5050,0x5050,0x8F8F,0x7060,0xBFBF,0xB1D0,0xC0B2,0x9EBE,0xBFBF,0xB1B2,0x91A1,0x9291,0x9292,0x9292,0x8281,0x8281,0x8282,0x9282,0x8282,0x9282,0x8292,0x9292,0x9192,0xC1A1,0xC0C0,0x9182,0x8291,0x82B2,0x92B2,0x9292,0xB292,0xB191,0x92B0,0xB2B2,0x80B1,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x5040,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x494A,0x9F9F,0x9F9F,0x7F7F,0x9F9F,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x4A5C,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4C4A,0x8F8F,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4E4E,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0xD0C0,0xB1D0,0xB1A1,0xDFDF,0xB0D0,0x92A0,0x9292,0x92B2,0x9292,0x8292,0x8282,0x8292,0x8282,0x8281,0x8282,0x9182,0x8282,0x9282,0x8282,0x8292,0x9281,0xB292,0xC091,0xAFAF,0x9291,0xB292,0xB2B2,0x8292,0xB2B2,0x8192,0xB292,0xB2B2,0x8091,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x7070,0xAFAF,0xBFBF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0x9E70,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x4A4A,0x9F9F,0x6C7C,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x708E,0x708E,0x9F9F,0x9F9F,0x9F9F,0x8F9F
.word 0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x9F9F,0x9F9F,0xD0E0,0xC1C1,0xE0D2,0xB0BE,0x9281,0x9292,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8281,0x8282,0x8182,0x8282,0x9281,0xB291,0xB181,0xAFAF,0x7271,0x9282,0x9292,0x82B2,0xB2B2,0x8192,0xB281,0xB2B2,0xB0B2,0x8040,0xB091,0x5080,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x4849,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0xAFAF,0x8C8C,0xBC9E,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4A6C,0x6E4A,0x8E4A,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x6060,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x509F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xC060,0xBFBF,0xC2B1,0xCEDE,0x81C0,0x82A1,0x9292,0x8292,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0x8182,0x8182,0x8282,0x8281,0x8272,0x8281,0x8282,0x8182,0x8282,0x9292,0xB1A1,0xC0B1,0xAFAF,0x8160,0xA1B2,0x8292,0xB2A3,0x82B2,0x8172,0xB2B2,0xB2B2,0x72B2,0xB2B2,0xB0B2,0x6090,0x9F9F,0xAFAF,0xC09E,0xAFAF,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x9F9F,0xDA9B,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x709E,0x709E,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x708E,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F70
.word 0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x6060,0xAFAF,0xC1C0,0xD1B1,0xB0CE,0x92A1,0x9292,0x8282,0x9282,0x9282,0x8282,0x8282,0x8282,0x8282,0x8282,0x7282,0x8272,0x7281,0x7282,0x7282,0x8172,0x8281,0x8272,0x8172,0x8282,0xA192,0xB1A1,0xC1B1,0x71C0,0xAFAF,0xB1B1,0x82A1,0xB282,0x8292,0x8271,0xB3B2,0x92B2,0x9292,0xB292,0xC0B1,0x60B0,0x708E,0x9E8E,0xBFBF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xE5DA,0xE7E7,0xDFDF,0xCFCF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9E70,0x9E70,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x4A4E,0x7C4A,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x8E70,0x7070,0x9F9F,0x9F9F,0xAFAF,0x9F9F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xC060,0x91C1,0xAEA1,0xA1AE,0x9282,0x8292,0x9292,0x8282,0x8282,0x8282,0x7282,0x8282,0x7282,0x7282,0x8282,0x7282,0x8172,0x8182,0x8182,0x8272,0x8172,0x8281,0x7281,0x8182,0xA192,0xA192,0xB1A1,0xA1B1,0x9191,0xC081,0xBFBF,0xAFAF,0x8272,0x7182,0x9282,0xB2B2,0x9292,0x8282,0xB282,0xB0B1,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0x9E9E,0xAFAF,0xAFAF,0xC09E,0xAFAF,0xAFAF,0x708E,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x6C4A,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x7060,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x8E70,0x9F9F,0x9F9F,0x9F9F,0x8F9F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x81C2,0xAED0,0x91A0,0x9292,0x9292,0x8292,0x92A3,0x9282,0x8282,0x8282,0x7282,0x8282,0x8282,0x7282,0x7282,0x7282,0x7272,0x7272,0x8182,0x8282,0x8282,0x8282,0x8281,0x9282,0x9292,0xA192,0x9292,0x8291,0x7181,0xC091,0xAFAF,0x9F9F,0xB081,0x8171,0x92B2,0xB291,0x9291,0x8282,0xB282,0xB0B1,0x9E80,0xCFCF,0xBFBF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4A4C,0x7F7F,0x7F7F,0x7C5C,0x9C8C,0xBFBF,0xCFCF,0xCFCF,0xDFDF,0xCEDE,0xBFBF,0xAEAE,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x6E4A,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4E6E,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7070,0xAFAF,0xAFAF,0x7070,0x708E,0x9F9F,0x9F9F,0x9F9F,0x8F60
.word 0x9F9F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x6050,0xAFAF,0xA191,0x9EBE,0x92A0,0x9292,0x9292,0x8292,0xA392,0x8282,0x8282,0xA382,0xA382,0xA382,0xA382,0x7282,0x7282,0x7282,0x7281,0x8281,0x7281,0x8182,0x7282,0x8281,0x8282,0x8281,0xA182,0x9292,0xA182,0x8282,0x8281,0x8281,0x8181,0x71B0,0xC071,0xAFAF,0xAFAF,0xB191,0xA1B1,0x9291,0x8281,0xB2B2,0xC0C0,0x9E9E,0xB1B0,0x9F9F,0x507E,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7C4A,0x4A4A,0x6C4E,0x8C7C,0x9F9F,0x9F9F,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xEEFE,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0x9F9F,0x8F8F,0x8F8F,0x4C4A,0x8F8F,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0x6070,0xAFAF,0x7070,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x609F
.word 0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xD081,0xB09E,0x9281,0x8292,0xA392,0x8292,0x9282,0xA382,0xA3A3,0x8282,0x82A3,0x93A3,0x8282,0xA382,0x7282,0x8272,0x7272,0x7272,0x8282,0x8282,0x8282,0x8282,0x9292,0x8182,0x9282,0x9192,0x9292,0x8292,0x7282,0x7272,0x7281,0x7171,0x8171,0xC0A1,0x9F9F,0x9F9F,0xC1C0,0xB1C1,0x81A1,0x9282,0xA1B2,0xAFAF,0xAFAF,0xA1B1,0x8E70,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8C7C,0x6B7B,0x8C8C,0xDE9E,0xDFDF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xDFDF,0xCFCF,0xCFCF,0xDFDF,0xEFEF,0xDFDF,0xDFDF,0xBFBF,0xAFAF,0x7E8E,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7070,0xAFAF,0xAFAF,0x9F9F,0x6060,0xAFAF,0x9F9F,0x9F9F,0x8F9F
.word 0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xB080,0xD0B0,0xB09E,0x9292,0x9292,0x9292,0x92A3,0x8282,0x8282,0x8282,0x8293,0x8293,0x8282,0x9393,0x8293,0x8293,0x7293,0x7272,0x8272,0x8281,0x8282,0x9292,0x8292,0x8282,0x9282,0x8192,0x9282,0x9192,0x9281,0x8281,0x7271,0x8272,0x82B3,0x8282,0x8281,0xC0C0,0xBFBF,0xAFAF,0xD0D0,0xC0D0,0xAFAF,0xA192,0xA1A1,0x9F9F,0xB1B2,0xA0A1,0x9F9F,0x4050,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x4A4A,0x8C7C,0x8B8B,0x9F9F,0x8C4A,0xBFBF,0xCFCF,0xDFDF,0xEFEF,0xEFEF,0xEFEF,0xCFCF,0xBFBF,0xCFCF,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xCFCF,0xBFBF,0xBFBF,0x7C7E,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x8E70,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x7070,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x509F
.word 0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xBEA1,0x819E,0x9292,0x8282,0x8282,0x8282,0x8282,0x8282,0x8282,0xA382,0x9382,0x9372,0x8272,0x7293,0x7282,0x7282,0x7282,0x7172,0x8272,0x9292,0x92A1,0x9292,0x9192,0x9282,0x9292,0x8281,0xB281,0x8181,0x8282,0x7282,0x7271,0x8272,0x8282,0x8172,0x9181,0xD0C0,0x70C0,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0xA1A1,0xAFAF,0xB270,0x91B2,0x50B0,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0xAC8C,0x8B9B,0x7B8B,0x8C8C,0xBB8B,0xDFDF,0xDFDF,0xF0F0,0xE0F0,0xDFDF,0xCFCF,0xBFBF,0xCFCF,0xF0E0,0xFFFF,0xEFEF,0xEFEF,0xCFCF,0xCFCF,0xCEBE,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x8E70,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8E60,0xAFAF,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x8F60
.word 0xAFAF,0x708E,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7050,0xC1D0,0xCFCF,0xA09E,0x8292,0x9292,0x8282,0x9382,0x8282,0x8282,0x8282,0x8282,0x8293,0x7293,0x7293,0x9393,0x9372,0x7272,0x7272,0x7272,0x8272,0x9282,0xA192,0x92A1,0x9292,0x9192,0x9292,0xA192,0xC381,0xC281,0x8181,0x8282,0x7272,0x7272,0x7272,0x7282,0x8182,0xA181,0xB1C0,0x90B0,0xCFCF,0xCFCF,0x9F9F,0xB1B0,0xA1B1,0x7081,0xB2B2,0x81B2,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8C6C,0xABAB,0x9BAC,0x9C8B,0x9F9F,0xAFAF,0xCFCF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xCFCF,0xDFDF,0xF0F0,0xF0F0,0xF0F0,0xCFCF,0xBFBF,0xBFBF,0x7EAE,0x8F8F,0xAFAF,0xAFAF,0x8E7E,0x4E7E,0x7C6C,0x9F9F,0x8C8C,0x6E4A,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x7070,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9FAF
.word 0xAFAF,0xAFAF,0x6080,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xB171,0x9EA1,0xBFBF,0x9292,0x9282,0x8292,0x9282,0x82A3,0xA382,0xA382,0x93A3,0x8272,0xA382,0x9382,0x9372,0x7272,0x7293,0x7293,0x7272,0x7272,0x8182,0x9292,0xB1B1,0xB2B1,0xB292,0xA192,0xC282,0xC081,0x7172,0x81D0,0x8281,0x7281,0x7271,0x7272,0x82A3,0x8272,0x7282,0xA191,0xA1B2,0x80A0,0xBFBF,0xD0AE,0x9F9F,0xB1C1,0x61B2,0xB261,0xB2B3,0x51B2,0x8161,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x9C6B,0xBBCB,0x9BAB,0xAFAF,0x8B9B,0x9E9C,0xCFCF,0xBFBF,0xAFAF,0x9EC0,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xCFCF,0xD0E0,0xD0E0,0xCFCF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0x8E9E,0xBFBF,0xBFBF,0x9F9F,0x8C8E,0x8C7E,0x8C8C,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8E60,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x708E,0x9F9F,0x7070,0xAFAF,0xAFAF,0x9F9F,0x6060,0x9F9F,0x9F9F,0x7070,0xAFAF
.word 0xAFAF,0xBFBF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9271,0x9EB0,0x819E,0x8282,0x8292,0x8282,0x8282,0x8282,0xA382,0xA382,0x8282,0x93A3,0x9372,0x9393,0x7293,0x8393,0x9372,0x7272,0x7272,0x7272,0x8282,0x81B2,0xC3B1,0xC3C0,0xC2C0,0xB1C2,0xB381,0xC3C0,0x71D0,0x72C3,0x81C0,0x9282,0x7172,0x7272,0x9372,0x7293,0x7272,0x8272,0x9181,0xC0B1,0x80C0,0x9EC0,0xBFBF,0xC1D0,0xA1C1,0x7161,0xB2B2,0x92B2,0x9271,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4B4A,0xAB8C,0xCBBB,0x9BBB,0x8A9B,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xC09E,0xAFAF,0xBFBF,0x708E,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xCFCF,0xCFCF,0xBEBE,0x9EAE,0x9F9F,0xAFAF,0x7C8C,0x9F9F,0x8C7C,0x7C7C,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8E70,0x7060,0x708E,0x9E70,0x7070,0x9F9F,0x9F9F,0x9F9F,0x6060,0xAFAF,0xBF70
.word 0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xC081,0xA08E,0x9292,0x8282,0x8282,0x8282,0x8272,0x8282,0x9382,0x9372,0x8293,0x9393,0x7272,0x7272,0x7283,0x9372,0x8372,0x7293,0x7172,0x7272,0xC0B2,0xC0C3,0xC0C3,0xC3C0,0xC2C0,0xC0C3,0xC381,0xD371,0xD480,0xC371,0x8281,0x8282,0x6272,0x7262,0x9372,0x7272,0x7272,0x8272,0x8191,0xC0B0,0xAFAF,0xBFBF,0xC0D0,0xC1D1,0x61B1,0xAFAF,0xB2B3,0x7192,0x8292,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7E5C,0xAB9C,0xCABB,0x9B9B,0x9F9F,0xAFAF,0xCFCF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x709E,0xAFAF,0x9F9F,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xCFCF,0xBEEE,0xBFBF,0x9E9E,0x6C7E,0x9F9F,0x8C7C,0x7C7C,0x9F9F,0x4A7C,0x4A7C,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x708E,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7070,0x70AF
.word 0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0xC0C3,0x8E81,0xA08E,0x8282,0x8281,0x8282,0x8282,0x8282,0x7272,0x8293,0x93A3,0x9372,0x7293,0x7283,0x7283,0x8372,0x7283,0x7283,0x7272,0x7272,0x7272,0xB282,0xC0C1,0xD0C3,0xC0D4,0x71D0,0xC3D0,0xC0D0,0xC0C3,0xBFBF,0xAFAF,0x71C0,0x8282,0x6272,0x7262,0x7262,0x9372,0x7272,0x7271,0x9192,0xC081,0xC0B0,0xBFBF,0xBFBF,0xD0D0,0xB1D1,0x9E70,0x9281,0x92B2,0xB282,0x4172,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xCCCC,0x8B9C,0x9F9F,0x7B6A,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xAFAF,0x7070,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xD0AE,0xCFCF,0xAFAF,0xBFBF,0xDFDF,0xDEDE,0xBECD,0xCFCF,0xAFAF,0x6E6C,0x9F9F,0x9F9F,0x7C4B,0x9F9F,0x7C7E,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7060,0x9F9F,0x6070,0x708E,0x9F9F,0x6070,0xAFAF,0xAFAF,0xAFAF,0xAF70
.word 0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xC0B1,0x908E,0x7271,0x8181,0x8282,0x8282,0x8281,0x8282,0x7272,0xA372,0x7293,0x7272,0x6272,0x6262,0x7262,0x7262,0x8372,0x8372,0x7272,0x7272,0x8172,0xD0C3,0x71D0,0xBFBF,0xBFBF,0xD0C0,0xC0D4,0xD071,0xC0C3,0xD4C0,0xD371,0x72C0,0x7182,0x6262,0x8372,0x7272,0x7272,0x7172,0xB272,0xC3C0,0xC0C0,0xC0B0,0xD0C0,0xD0D0,0xE0D0,0xC0C1,0x817E,0xB282,0x9292,0xB3B2,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xBFBF,0xBFBF,0xAFAF,0x4A4A,0x6B7C,0xBC8D,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xCEAE,0xDFDF,0xFEFC,0xEEEE,0xCFCF,0x7B9C,0x9F9F,0x7C8C,0x8F8F,0x8B8C,0x4A8C,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8E8E,0xAFAF,0x9F9F,0x7070,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6060,0x7060,0xAFAF,0x9F9F,0x7070,0x9F9F,0x7070,0x7060,0xAFAF,0xAF70
.word 0x8E80,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xBFBF,0xC081,0x808E,0x8181,0x8282,0x8281,0x8281,0x7282,0x8272,0x8282,0x7293,0x7293,0x6272,0x7262,0x6172,0x6262,0x6262,0x7262,0x7262,0x7283,0x7272,0x8272,0x71C0,0xD0D3,0xBFBF,0xC3C0,0xAFAF,0xBFBF,0xAFAF,0xC3D0,0xAFAF,0xBFBF,0xC271,0x6281,0x6272,0x7283,0x8383,0x8372,0x6272,0x6261,0xC071,0xAFAF,0xC0C0,0xC0C0,0xAFAF,0xD0D0,0xCFCF,0x9F9F,0x9291,0x92B2,0xB282,0x42B2,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0xBFBF,0x9F9F,0x9F9F,0x8B8C,0xBFBF,0xBFBF,0xAFAF,0xCFCF,0xBFBF,0xBFBF,0x9F9F,0xAFAF,0xAFAF,0xCFCF,0x9E80,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0xDCEE,0xBFBF,0x8C9C,0x4A8C,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0x708E,0xAFAF,0xAFAF,0x8080,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0x6060,0x9F9F,0x9F9F,0x7060,0x7070,0x9F9F,0x9F9F,0x7070,0xAFAF,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x509F
.word 0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0x9F9F,0x9E81,0xB08E,0x8272,0x8181,0x8182,0x7282,0x8182,0x7282,0x7293,0x8282,0x7272,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x6261,0x6262,0x6262,0x6272,0x7272,0x7272,0xD392,0xAFAF,0xD0D0,0xBFBF,0xAFAF,0xD4C0,0xD3C0,0xAFAF,0xC371,0x71C0,0x71D4,0x7281,0x6272,0x8362,0x8362,0x8362,0x7262,0x6262,0xD3C0,0xC081,0xBFBF,0xAFAF,0xC080,0xD0D0,0xBFBF,0xBFBF,0xAFAF,0x9292,0x92B2,0xB282,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xBFBF,0xBFBF,0xDFDF,0xEFEF,0xDFDF,0xDBFB,0xCFCF,0xAFAF,0xAC8D,0xBBBB,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x7070,0x8070,0xAFAF,0x8070,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x7070,0x9F9F,0x6060,0x6060,0x7070,0x7060,0x7070,0xAFAF,0x6060,0x9F9F,0x9F9F,0x9F9F,0x8F8F
.word 0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x809E,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0xAFAF,0xAFAF,0xA08E,0x8181,0x8182,0x7281,0x7281,0x7271,0x8281,0x9382,0x7272,0x7172,0x509E,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x6261,0x7262,0x7262,0x7272,0x7272,0xC182,0xD0D0,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xC0D4,0x71D0,0xBFBF,0xD480,0xB371,0x6271,0x6262,0x6262,0x6262,0x7262,0x7262,0x7183,0x71D0,0xC0C3,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x82A1,0xA392,0x9292,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xE0D3,0xDFDF,0xEFEF,0xFBFA,0xDCFB,0x9E9C,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0x4A9C,0x8F8F,0x9F9F,0x7070,0x7070,0x8080,0xBFBF,0xAFAF,0xAFAF,0x6070,0x9F9F,0xAFAF,0x7070,0x9F9F,0x6070,0x5050,0x6060,0x7070,0x6060,0x7060,0x7070,0x9F9F,0x6070,0xAFAF,0x8E70,0x6070,0x9F9F,0x9F9F,0x609F
.word 0x8071,0xC070,0xAFAF,0xAFAF,0xAFAF,0x9E80,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x4849,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0xAFAF,0xB081,0xA07E,0x8172,0x7272,0x8281,0x9281,0x7281,0x7272,0x8272,0x7282,0xAEC0,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xAFAF,0x6261,0x6272,0x6272,0x7272,0x8172,0xC0D4,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0x71C0,0xC0E4,0xD071,0xAFAF,0x7282,0x6272,0x6262,0x6262,0x6262,0x6273,0x6262,0xD462,0xD3C0,0xD470,0xC0C0,0xC0C0,0xAFAF,0xD0C0,0xBFBF,0xAFAF,0xB180,0x92B1,0x8282,0x8292,0x4282,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0x9F9F,0x8F8F,0x7060,0xAFAF,0xBFBF,0xCFCF,0x9F9F,0x7F7F,0x5252,0x9F9F,0xAFAF,0xD0D3,0xBFBF,0xDFDF,0xCFCF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x6060,0x8F8F,0x8F8F,0xBFBF,0xE9D9,0xEAF9,0xCBDA,0xAFAF,0xAFAF,0x8080,0xBFBF,0xAFAF,0x6070,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0x7060,0xAFAF,0x8F8F,0x9F9F,0x6060,0x9F9F,0x7060,0x7070,0x6070,0x7060,0x7070,0x7070,0x8F8F,0x8F8F,0x9F9F,0x9F60
.word 0x7171,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x4848,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0xAFAF,0x8EB0,0x719E,0x8181,0x8181,0x8172,0x8182,0x8192,0x8281,0x7282,0x8172,0x709E,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x5261,0x7262,0x7272,0x7272,0x8282,0xD0C0,0xAFAF,0xBFBF,0xD071,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xC3D0,0xC071,0x72C3,0x7172,0x6262,0x6262,0x5262,0x6262,0x6262,0x6162,0xC0C4,0xB0C2,0xC3D3,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0x82A1,0x9292,0x8282,0x71B2,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8E70,0xBFBF,0xBFBF,0x7070,0x9F9F,0xAFAF,0xAFAF,0xE0D0,0xD0E0,0x70D0,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x7161,0xDFDF,0xD3F3,0xCFCF,0xCFCF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xD9CB,0xE7D7,0xF7E7,0xF8F8,0x9EDE,0xAFAF,0xC070,0xBFBF,0x8EC0,0xAFAF,0xAFAF,0x7070,0x9F9F,0x9F9F,0x6070,0x9F9F,0x9F9F,0x6050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6070,0x7060,0x9F9F,0x7060,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x409F
.word 0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x9F9F,0xAFAF,0xBFBF,0xA09E,0x8281,0x8282,0x8181,0x8182,0x8182,0x7272,0x8272,0xB082,0x51AE,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x9F9F,0xAFAF,0x6262,0x9362,0x7272,0x8272,0xBFBF,0xC0D3,0xC0D3,0xBFBF,0xD0D4,0xD0C0,0xC0C3,0xC3C3,0xAFAF,0xC3D0,0x81C3,0x6272,0x5262,0x5262,0x6263,0x7362,0x6262,0x7262,0xB371,0xC381,0xAFAF,0xC0C3,0xBFBF,0xD0C0,0xBFBF,0xBFBF,0xBFBF,0x8192,0x8272,0x9282,0xB2B3,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x6060,0x708E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x709E,0xBFBF,0xD0D0,0xD0D3,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0xAFAF,0xF0F3,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xD7D7,0xEAEA,0xE8E7,0xE8F8,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0x707E,0x7070,0xAFAF,0x7070,0x7070,0x7070,0x9F9F,0x6060,0x9F9F,0x9F9F,0x6070,0x6060,0x6060,0x9F9F,0x9F9F,0x7060,0x6070,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x4050
.word 0xAFAF,0x5171,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0xC3D0,0xBFBF,0xA08E,0x8181,0x8181,0x8282,0x9281,0x82A1,0x8282,0x8282,0xBFBF,0xCFCF,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x707E,0x6261,0x6272,0x9372,0x7272,0xD092,0xBFBF,0xD4C0,0xC0D0,0x72C0,0xD0D4,0xD472,0xD0C0,0xC0C3,0xC0C3,0xD071,0x7272,0x6262,0x6252,0x6252,0x5252,0x6252,0x6262,0xC372,0xC371,0xC081,0xAFAF,0xC0C0,0xBFBF,0xC0C0,0xC0C0,0xBFBF,0xB2D0,0x7272,0x9282,0xB282,0x6192,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x8E70,0xAFAF,0x7070,0xAFAF,0x9E9E,0xBFBF,0xCFCF,0x62D0,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0xF0D0,0xF0F0,0xC0E0,0x9F9F,0x9F9F,0x5958,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xAFAF,0xAFAF,0xD7DA,0xE7E9,0xE8E9,0xE9E8,0xAFAF,0x6060,0x7070,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x6070,0xAFAF,0x6060,0x6050,0x8F8F,0x5050,0x6050,0x5060,0x5060,0x8F8F,0x7050,0x7070,0x6070,0x9F9F,0x6060,0x9F9F,0x9F9F,0x8F8F,0x609F
.word 0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9EB1,0xA09E,0x8292,0x8182,0x8281,0x9281,0x8192,0x8281,0x7282,0xC072,0x50AE,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xAFAF,0x6261,0x9362,0x7272,0x8293,0xD382,0xD3C0,0x71D0,0xD4D3,0xD0D0,0xD471,0xC3C0,0xD372,0xC0C3,0xC0D4,0xC3D4,0x72C0,0x6272,0x6262,0x5252,0x6262,0x5252,0x5262,0xC372,0xB371,0xD472,0x71C0,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x80C0,0xBFBF,0x72B2,0x9362,0xA382,0x82B2,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x7070,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0xAFAF,0xDFDF,0xD0E0,0x9F9F,0x8F8F,0x9F9F,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xCFCF,0xCFCF,0xDFDF,0xCFCF,0xAFAF,0xAFAF,0xD669,0xD7DA,0xE9D7,0x59DA,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x8070,0x7070,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5060,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x6060,0x7060,0xAFAF,0x9F9F,0x6060,0x6060,0x6070,0x9F9F,0x9F70
.word 0xBFBF,0xBFBF,0x5261,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4742,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x9EB0,0x819E,0x9281,0x8291,0xA192,0xA1A1,0x7281,0x7281,0x9282,0xA071,0x51AE,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6161,0x7262,0x9393,0x7272,0xD093,0xBFBF,0xD0C3,0xC0C0,0xE4C3,0xC3D0,0xD0C3,0x71C2,0xC0C3,0x72C3,0xC3C0,0xC372,0x6272,0x6252,0x6262,0x6263,0x6262,0x6252,0x7161,0x71D3,0xAFAF,0xD0D4,0xC071,0xAFAF,0xC0C0,0xC0C3,0xCFCF,0xD071,0x82D0,0x6261,0x9393,0xB2A3,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xCEDE,0xDBBE,0x9F9F,0x5251,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xBFBF,0xBFBF,0x9F9F,0x8F8F,0xBFBF,0xCFCF,0xBFBF,0xAFAF,0xBFBF,0xCFCF,0xDFDF,0xD0CE,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x59DA,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x6070,0x9F9F,0x7060,0x6060,0x6060,0x9F9F,0x6060,0x5050,0x5060,0x6060,0x9F9F,0x6050,0x7060,0x9F9F,0x7060,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x509F
.word 0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0x7162,0x81D3,0x9EC0,0xA0AE,0x9192,0x9281,0x9291,0xB1A1,0x91A1,0x8282,0x8282,0x9182,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x9F9F,0x5271,0x8372,0x9372,0x7272,0xD482,0xC2D0,0xC381,0xBFBF,0xC0C3,0xC0C3,0xC3C2,0xD3C3,0xC3C0,0xC0D0,0xC0D4,0x81C3,0x72B2,0x6252,0x5252,0x5262,0x6262,0x5252,0xC362,0x71C3,0x71D3,0x71C0,0xAFAF,0xC0C0,0xAFAF,0xAFAF,0xC3D4,0xAFAF,0xC1D3,0x6262,0x9372,0xA372,0x5182,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xCE9E,0xDFDF,0xCFCF,0xEADB,0xAFAF,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0x4060,0x8F8F,0xAFAF,0xDFDF,0xEFEF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xDFDF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x7070,0xAFAF,0x7070,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5050,0x9F9F,0x9F9F,0x9F9F,0x6060,0x5060,0x9F9F,0x9F9F,0x7070,0x6060,0x7070,0x9F9F,0x9F9F,0x9F9F,0x5050
.word 0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x4742,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9EB1,0xA09E,0x9291,0x9192,0xA1B2,0xA1B1,0xA1B1,0x8291,0x9382,0x8172,0xBFBF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x9F9F,0x5170,0x7283,0x7293,0x7272,0xC082,0xB1D4,0xC0C2,0x81C3,0xD0C3,0xD3C3,0xC0C3,0xC0C3,0x71C3,0xC3D3,0xC371,0xC3C0,0xA3C0,0x5262,0x6262,0x6252,0x5263,0x6262,0x6252,0xD371,0xB382,0xAFAF,0xC3C0,0xAFAF,0x81C3,0x71D0,0xC0D0,0xC371,0xBFBF,0x6271,0x7262,0xA3A3,0x8282,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9E9E,0xAE9E,0xBFBF,0xCFCF,0xDFDF,0xDEEE,0xBFBF,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0xBFBF,0xEFEF,0xEFEF,0xDFDF,0xCFCF,0xF5FA,0xDFDF,0xCFCF,0xCFCF,0xBFBF,0x709E,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x7070,0xAFAF,0xAFAF,0xBFBF,0x9F9F,0x6060,0x5060,0x8F8F,0x8F8F,0x5050,0x6050,0x5050,0x5050,0x5050,0x6060,0x9F9F,0x7070,0x7070,0x6070,0x9F9F,0x9F9F,0x609F
.word 0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4A42,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x62C4,0xAFAF,0x9EB1,0xA09E,0x8192,0x8292,0x9292,0xC1B1,0xB1B1,0x8291,0x8282,0x7182,0xAEB0,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0x6261,0x8362,0x8393,0x7272,0xC393,0x82C0,0xC3B3,0xC1C2,0xC3C3,0xC3C0,0xC3D0,0xB372,0xC3C3,0xD071,0x71C3,0x71C3,0xB2C3,0x6262,0x6262,0x7362,0x6262,0x5252,0x6262,0xAFAF,0x72C3,0xC481,0xC071,0xAFAF,0xBFBF,0xC3D4,0xBFBF,0x71D0,0xD3C0,0x6271,0x8362,0xA393,0x82A3,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9E9E,0xD0AE,0xCFCF,0xDFDF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0xDFDF,0xEFEF,0xEFEF,0xDFDF,0xE6E5,0xEAD7,0xBFBF,0xBFBF,0xEFEF,0xD0DE,0xAFAF,0x8E70,0x6070,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x7070,0x7070,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5050,0x6050,0x6060,0xAFAF,0x9F9F,0x6070,0x9F9F,0x9F9F,0x9F70
.word 0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x4763,0x9F9F,0xAFAF,0xAFAF,0xC0C3,0x9EB0,0x81AE,0x9292,0x9281,0xA191,0xC1B1,0xD0D0,0x81B1,0x8281,0x9382,0x80B0,0xBFBF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0x6251,0x7283,0x7283,0x7283,0xC382,0x92C3,0x7292,0xC3C3,0xC281,0x81C3,0xC0B3,0xC0D1,0xC0C3,0xC3C2,0xC3C0,0xB2C3,0xC2B2,0x6272,0x6252,0x6252,0x6262,0x5262,0x6252,0xB3B3,0xC371,0xC072,0x72D4,0xB3B3,0xC072,0xAFAF,0xC3C3,0xAFAF,0xD1D3,0x62C3,0x8362,0x8372,0x6272,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0x9E9E,0xDFDF,0xDFDF,0xEFEF,0xEFEF,0xDFDF,0xDFDF,0xFEF0,0xCFCF,0xAFAF,0x9F9F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0x8F8F,0x8F8F,0xCFCF,0xEFEF,0xDFDF,0xCFCF,0xC7D7,0xAFAF,0xAFAF,0xAFAF,0xCFCF,0xDFDF,0x9EBE,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x708E,0xAFAF,0x9F9F,0x9F9F,0x6060,0x6050,0x9F9F,0x6060,0x8F8F,0x8F8F,0x5040,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x6060,0x7070,0x7060,0x6060,0x9F9F,0x6060,0x5060
.word 0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x7262,0xAFAF,0x9EA1,0xA09E,0x9292,0x8292,0xA192,0xC1C1,0xD0D0,0xB1D0,0x8282,0x8293,0x9072,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x7262,0x7262,0x7283,0x7272,0xC282,0xB392,0xB2C2,0xB182,0x81C3,0xD2C1,0xC382,0xC3C2,0xC3C1,0xD0D0,0xC0D3,0xC1D0,0xC2C2,0x6293,0x6252,0x6362,0x6362,0x6262,0x6252,0x7261,0x72B3,0x71C3,0x71D0,0xC3C0,0xC371,0x7182,0xC0D4,0x82C2,0xD0C2,0x6281,0x7283,0x8383,0x5162,0xBEBE,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9E9E,0xBFBF,0xEFEF,0xEFEF,0xEFEF,0xDFDF,0xDFDF,0xEFEF,0xDFDF,0xAFAF,0x9F9F,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xBFBF,0xDFDF,0xEFEF,0xBFBF,0xD6C7,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x5060,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x7070,0x6060,0x9F9F,0x8F8F,0x6050,0x8F8F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x5050,0x6060,0x7060,0x9F9F,0x7060,0x9F9F,0xAFAF,0x8F70
.word 0xC0C0,0xAFAF,0x71B0,0xAFAF,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x63C4,0xAFAF,0x9F9F,0x9F9F,0xBFBF,0xD071,0x9EC2,0xA09E,0x9192,0x9182,0x9281,0xD0B1,0xBFBF,0xC0D0,0x8191,0x8282,0x72A3,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x6261,0x6262,0x8383,0x8372,0x6272,0xC3A3,0x9282,0x8292,0x82B3,0xB3C0,0xC1B3,0xC2C1,0x81B1,0xC3C1,0xB2C3,0xB2C1,0xC2C3,0xC3C1,0x6272,0x6262,0x6273,0x6262,0x5252,0x5262,0xE362,0x8282,0xC372,0xC472,0xD071,0xC0D4,0xC372,0x7172,0xAFAF,0xC381,0x52C3,0x8362,0x8383,0x5262,0xBFBF,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0xBFBF,0xBE9E,0xEFEF,0xEFEF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0xCFCF,0xDFDF,0xDFDF,0xDFDF,0x59D7,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7E50,0x8E60,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x6060,0x9F9F,0x9F9F,0x5050,0x9F9F,0x8F8F,0x4050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x6070,0x9F9F,0x7070,0x7060,0x6070
.word 0x7161,0x7171,0x7162,0x6162,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x4852,0x9F9F,0x9F9F,0xBFBF,0xAEC0,0xA09E,0x9292,0x9291,0xA1A1,0xD0C1,0xBFBF,0xBFBF,0x82B0,0x8272,0x7282,0x7171,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x5251,0x6261,0x6262,0x7283,0x7283,0x7283,0xC382,0xB2B2,0xB292,0xB3B2,0xB2C1,0xB282,0xC0C1,0xC1D3,0xB1B2,0x82C3,0xB2C2,0xB381,0xC2B2,0x6272,0x6262,0x7362,0x7373,0x6262,0x5252,0xC362,0x7293,0x72C3,0x72B3,0x72C3,0xBFBF,0xC3C0,0x72B3,0xC3C0,0xC3C0,0x6281,0x6262,0x6283,0x4262,0xAFAF,0xBFBF,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0x9F9F,0xBFBF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xBFBF,0xCFCF,0xEFEF,0xDFDF,0xCFCF,0xAFAF,0x8F8F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x708E,0xAFAF,0x708E,0xAFAF,0xAFAF,0x9F9F,0x6060,0x9F9F,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x6060,0x9F9F,0x9F9F,0x6060,0x9F9F,0x6070,0x6060
.word 0xBFBF,0x7172,0x61D4,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8042,0xAFAF,0x9EB0,0x91AE,0x9191,0x9292,0xA192,0xCFCF,0xBFBF,0xCFCF,0xBFBF,0x9381,0x9382,0x7272,0x6171,0x5161,0x5151,0x5251,0x6252,0x6262,0x8362,0x8362,0x8372,0x7272,0x82C3,0x9292,0xB2B2,0xB2C2,0xC1C2,0xB2C2,0xB3C2,0xD1C0,0xD0C1,0xC3C1,0xC2C1,0xB2C2,0xC3C2,0x5252,0x6262,0x6273,0x6262,0x6273,0x5252,0xB352,0x7293,0x7272,0xC3C0,0xB3B3,0x71C3,0xC0C4,0x72C3,0x71B3,0xC3C3,0x62B3,0x7352,0x8362,0x4163,0xBFBF,0xD080,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xCFCF,0xEFEF,0xEFEF,0xEFEF,0xAFAF,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x7070,0xAFAF,0x7070,0xAFAF,0x6070,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9E9E,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7060,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x7070,0x9F9F,0x5060
.word 0xAFAF,0x7172,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0xAFAF,0xB1C0,0xAEC0,0x819E,0x9292,0x8292,0xB191,0xD0D0,0xBFBF,0xBEBE,0xBFBF,0x8191,0xA372,0x7272,0x6272,0x6262,0x6262,0x6262,0x6262,0x6252,0x6262,0x8383,0x8362,0x9372,0xB2B2,0x8282,0x8292,0xB2B2,0xB2B3,0xC2B1,0xB2C1,0xB2B3,0xC2C1,0xB2D0,0xC0D0,0xB2C1,0xB2D1,0x6262,0x5252,0x5262,0x5262,0x6262,0x5252,0xC052,0x83B3,0x72B3,0xC372,0xC371,0x7272,0x61B3,0xC3C3,0x83B3,0xC372,0x5272,0x6252,0x6273,0x4252,0xBFBF,0xC071,0xBFBF,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xEFEF,0xEFEF,0xEFEF,0xBFBF,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0x708E,0x9F9F,0x7070,0x9F9F,0x7060,0x9F9F,0x9F9F,0x6060,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x6070,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x6060
.word 0xCFCF,0x7282,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x81D0,0xAEC1,0xA09E,0x9291,0x8182,0xB182,0xBEE0,0xCFCF,0xBFBF,0xBFBF,0x71D0,0x7282,0x7293,0x7283,0x6262,0x6262,0x5262,0x5252,0x7362,0x8362,0x8372,0x6262,0x8262,0xB2C2,0xB292,0xB2B1,0xB2B2,0xC3B1,0xB2B2,0xC1D0,0xC2B2,0xB2C2,0x82C3,0xA3B3,0x82C3,0x72C3,0x6262,0x7362,0x6262,0x5263,0x5262,0x5252,0xBFBF,0xB382,0x7293,0x82B3,0xD071,0xC3C3,0x8372,0x71B3,0xB3C0,0xB3B3,0x5262,0x6252,0x7373,0x4273,0xC341,0xD371,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0xCFCF,0xEFEF,0xEFEF,0xCFCF,0xAFAF,0x9F9F,0x7F7F,0x8F8F,0xAFAF,0x7070,0xAFAF,0x9F9F,0x6060,0x9F9F,0x7070,0x7070,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x6060,0x9F9F,0x7070,0x609F
.word 0xBFBF,0x82C3,0x6172,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0x52C4,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xAEC0,0xB09E,0x9292,0x9292,0x9182,0xCFCF,0xAEAE,0xCFCF,0xBFBF,0x90AE,0x7272,0x7293,0x8372,0x6262,0x5262,0x6252,0x6252,0x6262,0x6283,0x6283,0x6283,0xB372,0x8282,0x9282,0xC1B2,0xC2C2,0xB1C1,0xB3C1,0xD1C2,0xC2D0,0xC1B2,0xB2B3,0xA3B3,0x82B3,0x93B2,0x5252,0x6262,0x6273,0x7362,0x6262,0x5252,0xB362,0x71A3,0x82C3,0x72B3,0xB383,0xD371,0xB381,0x9383,0x72C3,0xC3B3,0x52B3,0x6362,0x6262,0x4252,0xC351,0x7271,0x71C0,0xC0D0,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xDFDF,0xEFEF,0xCFCF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x7060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8E70,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0xAFAF,0x6070
.word 0xBFBF,0x72B1,0x6062,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x5252,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0x8171,0x9EB0,0x91BE,0x9292,0x9292,0xB181,0xBFBF,0xBFBF,0xAEAE,0xBFBF,0xBFBF,0x7271,0x8372,0x7272,0x6283,0x6262,0x5262,0x6262,0x6273,0x6273,0x8362,0x6272,0xC293,0x92C2,0x9292,0xB1B3,0xB282,0xB2B2,0xC1C2,0xC2B2,0xD0C2,0xC2C2,0xC2B2,0xB1C2,0xB3B2,0x93C2,0x6252,0x6252,0x6262,0x6262,0x7362,0x4262,0xB361,0x7293,0x81B3,0xB3B3,0x9393,0xC372,0xC372,0x93B3,0x6293,0xC3C3,0x4293,0x6262,0x7363,0x4362,0x9F9F,0xB3B3,0xC371,0xAFAF,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x9F9F,0xCFCF,0xCFCF,0xCFCF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x5040,0x8F8F,0xAFAF,0x6070,0x9F9F,0xAFAF,0x9F9F,0x7070,0x9F9F,0x6060,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9E80,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x7070,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8070,0xAFAF,0xAFAF,0x6060
.word 0xAFAF,0x7272,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x4352,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0xAFAF,0x9EB0,0x819E,0x9292,0x8292,0xA182,0xBED0,0xBEAE,0xBEBE,0xBFBF,0xCFCF,0x7281,0x7272,0x6272,0x6272,0x6262,0x6252,0x7362,0x6262,0x6262,0x6283,0x6262,0xB293,0x92B2,0xB2B2,0xB2B2,0xB2B2,0xB2B2,0xB2C3,0xC2C2,0xB2B2,0xB282,0xB2C3,0xB3B2,0xC2B2,0x72D2,0x6262,0x6262,0x5273,0x5263,0x5262,0x4352,0xAFAF,0x93B3,0x7283,0x82C3,0xB3C3,0x7283,0x7272,0xB3B3,0x8393,0xC3B3,0x5283,0x6352,0x5262,0x4252,0xC352,0x7172,0x71C3,0x7271,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0xCFCF,0xCFCF,0xCFCF,0xAFAF,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x6060,0x9F9F,0xAFAF,0x9F9F,0x7070,0xAFAF,0x9F9F,0x7060,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xCFCF,0xAFAF,0xBFBF,0xAFAF,0x7070,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7070,0x9F9F,0x9F9F,0x5060
.word 0xBFBF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xC552,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x4852,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0xAFAF,0x8EB1,0xA09E,0x9291,0x8282,0xA182,0xCED0,0xCFCF,0xBEBE,0xBFBF,0xBFBF,0x7180,0x7293,0x8383,0x6283,0x6262,0x7362,0x6262,0x7362,0x7362,0x7262,0x7283,0xC292,0x92B3,0xB2B3,0xB2B2,0xC2C2,0xB1C2,0x82B3,0x82B2,0xB3B2,0xC392,0x8282,0xB2B3,0xB382,0x62C3,0x6252,0x6273,0x6273,0x7362,0x5262,0x5252,0xD062,0xB382,0x8393,0x7293,0xD371,0x9382,0xB383,0x6193,0x72C3,0x7293,0x5262,0x6252,0x7373,0x4252,0xD361,0xC372,0xC372,0x72C0,0xB172,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x6050,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0xBFBF,0xDFDF,0xDFDF,0xBFBF,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x6060,0x7070,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x7060,0x9F9F,0x708E,0xAFAF,0xD0D0,0xC0C0,0xBFBF,0xAFAF,0xAFAF,0x7070,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5040,0x8F8F,0x6050,0x8F8F,0x8F50
.word 0xAFAF,0xAFAF,0xAFAF,0x5172,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBBBC,0xAFAF,0x9F9F,0x9F9F,0x81D0,0x9EC0,0xBFBF,0x9292,0x8292,0xB182,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0x71D0,0x7272,0x7272,0x7362,0x7362,0x6262,0x6273,0x6273,0x6273,0x6283,0x7262,0xC2B2,0xB292,0x9292,0xB392,0xA382,0xB282,0xB392,0xB2C2,0xC2B2,0xB2C2,0xB2A3,0xB292,0xB392,0x62B3,0x6252,0x6262,0x7373,0x7362,0x5273,0x6352,0x93D4,0x8282,0x72B3,0x8393,0x93B3,0xC372,0x9393,0xC493,0xB372,0xC3B3,0x5273,0x6352,0x7352,0x4263,0x9F9F,0x72B3,0x6271,0xC0D4,0xB261,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xCFCF,0xEFEF,0xDFDF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x5040,0x6050,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0xBFBF,0x7070,0x9F9F,0x9F9F,0x6070,0xAFAF,0xAFAF,0xBFBF,0x8080,0xAFAF,0x709E,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5050
.word 0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x63C5,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9171,0xAFAF,0x8180,0x8282,0x8282,0xA181,0xBFBF,0xCFCF,0xBEBE,0xBEBE,0xCFCF,0xBFBF,0x7272,0x8383,0x6283,0x6283,0x6273,0x6362,0x6262,0x8362,0x8372,0x8362,0xB3B2,0xB382,0xB3B2,0x92B2,0x82B2,0xC2B2,0x82B2,0x82B2,0xC2B2,0xB2C3,0xC2C2,0xB3B2,0xC2B2,0x52B2,0x6262,0x6252,0x5262,0x6262,0x6262,0x5252,0xB371,0x7283,0xB3C3,0x72C3,0x9393,0xC372,0xB3B3,0x7293,0x6283,0xB3C3,0x5252,0x7373,0x5262,0x4252,0xC352,0x7283,0x71C3,0xC361,0x61D3,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7060,0x6060,0x9F9F,0x9F9F,0xEFEF,0xDFDF,0xCFCF,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x5050,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x7060,0xEFEF,0x9F9F,0x6070,0x6060,0x9F9F,0x7070,0xAFAF,0xAFAF,0x8E70,0x8070,0x9EC0,0xBFBF,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F40
.word 0x6161,0x9F9F,0xAFAF,0xBFBF,0x9F9F,0x8F8F,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x8B6A,0x9F9F,0x8F8F,0x8F8F,0xBFBF,0x8EC0,0x81C0,0x9292,0x8282,0xB182,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0x81DE,0x7262,0x7283,0x8362,0x7362,0x5262,0x6273,0x7362,0x6283,0x6262,0x8272,0xB2C2,0x8282,0x82A3,0xB2A3,0xB2C2,0xB282,0xB292,0x9282,0x82B2,0xA392,0x81B2,0xB182,0xC2C3,0x5293,0x6262,0x7373,0x6263,0x6362,0x5252,0x6242,0xBFBF,0x9393,0x7183,0xB3C3,0x93C3,0x9383,0xD462,0x93B3,0x7283,0x93D4,0x5252,0x6252,0x6363,0x4252,0x9F9F,0x9393,0x72C3,0x71D3,0x71D0,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x5060,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xBFBF,0xDFDF,0xCFCF,0xBFBF,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xBFBF,0xDEFE,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6050,0xCFCF,0x70D0,0x9F9F,0x9F9F,0x6070,0x9F9F,0x7070,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F8F
.word 0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xA171,0xAFAF,0xB090,0x8292,0x8282,0xA182,0xAED0,0xBFBF,0xAEAE,0xCFCF,0xBFBF,0xC0DE,0x7272,0x6283,0x8383,0x6262,0x7362,0x6262,0x6273,0x8362,0x6283,0xB362,0x82B3,0xA382,0xB2A3,0x82B2,0xB392,0x82B3,0xB3B2,0x82A3,0xB3A3,0xA3B3,0xA382,0x82B3,0xC382,0x5262,0x8362,0x8362,0x7362,0x5273,0x5262,0x6252,0xC072,0xB382,0x8393,0x7272,0xB3C3,0x8372,0x7293,0xC462,0xC493,0x62B3,0x5263,0x6352,0x6262,0x4252,0xB352,0x9393,0xD472,0xD371,0xBFBF,0xB152,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x7060,0x7060,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x5060,0x8F8F,0x9F9F,0x9F9F,0xE0E0,0xBFBF,0x6060,0x9F9F,0x5060,0x9F9F,0xAFAF,0xBFBF,0x9F9F,0x7060,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xBFBF,0xBFBF,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x407F
.word 0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x7F7F,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x7C8C,0x9F9F,0x9F9F,0x9F9F,0x92D0,0x9E9E,0x82A0,0x8292,0x8292,0xB182,0xBFBF,0xAEAE,0xBFBF,0xBFBF,0xD0D0,0xC0DE,0x6262,0x8372,0x6283,0x7373,0x6262,0x6273,0x7362,0x6262,0x8362,0xC372,0xA392,0x82A3,0x82B3,0xB282,0x82B3,0x8282,0xB2B2,0x92B3,0xC382,0xB282,0x82A3,0x93A3,0xC3A3,0x5262,0x6252,0x6263,0x6262,0x6283,0x6262,0x7342,0x93C4,0x7193,0xA3C3,0x7393,0xC372,0x93C3,0x9383,0x7183,0x93B3,0x83C4,0x6252,0x5273,0x6352,0x4252,0xC362,0x7293,0x81B3,0x80D4,0xD0D0,0x6252,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x6060,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xCFCF,0xEFEF,0xAFAF,0x9F9F,0x6060,0x9F9F,0x5060,0xC070,0xBFBF,0x709E,0xBFBF,0xD080,0xCEE0,0x7060,0x9F9F,0x9F9F,0xBFBF,0x9BD6,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F6F
.word 0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8D8D,0x9E8C,0xBFBF,0xA181,0x9E8E,0x8290,0x8292,0x8282,0xA182,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xCFCF,0x7272,0x7283,0x7383,0x5262,0x6373,0x6362,0x8362,0x8373,0x6262,0xB3A3,0x82A3,0xA382,0xA382,0x82A3,0x92A3,0xA3B2,0xA3A3,0xA3A3,0x82A3,0xB2B3,0xC382,0x8282,0x93B3,0x5262,0x6362,0x5262,0x6362,0x6252,0x5273,0x9F9F,0x9371,0x6283,0x71B3,0x72B3,0x7393,0xC371,0x93B3,0xB483,0xC362,0x73B3,0x5242,0x7373,0x5252,0x4252,0xB361,0xC4A3,0xD471,0xD471,0xCFCF,0x5271,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0x7070,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x7060,0x7070,0x7070,0x7070,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0xE0D0,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xBFBF,0xAFAF,0xC080,0xAFAF,0xEFEF,0x607E,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0xBFBF,0x6080,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9C7E,0xAFAF,0x9DAC,0xBFBF,0x91C0,0xBFBF,0x91A0,0x8282,0x82A3,0x9172,0x9EAE,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xE0E0,0x7282,0x8372,0x6262,0x7362,0x6262,0x7362,0x7362,0x7362,0x6262,0xB3B3,0x8282,0x9382,0xA3A3,0x9282,0x82B3,0xA3B3,0x8282,0xA382,0x9372,0x8293,0x81B3,0x82B3,0x62B3,0x5252,0x6262,0x5252,0x5263,0x5252,0x4252,0x9342,0x71E4,0x8372,0x6283,0xB3D4,0x83C3,0x8383,0xC361,0x8383,0x7283,0x73C3,0x5242,0x5262,0x6363,0x4242,0x9F9F,0x7293,0x71C3,0xC0C3,0xAFAF,0x41C3,0x6162,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x7070,0x9F9F,0x8F8F,0x8F8F,0x7050,0x9F9F,0x7070,0x9F9F,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x6070,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6150,0x9090,0xAFAF,0x9F9F,0x9F9F,0x5060,0x6060,0xBFBF,0xAFAF,0xBFBF,0x9F9F,0xB190,0xCFCF,0x5060,0x9F9F,0xAFAF,0xD5D5,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x9F9F,0xAFAF,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBCBC,0xAC9E,0xBFBF,0x81D4,0x9ED0,0x8190,0x8282,0x8282,0xA182,0xBFBF,0xBFBF,0xBEBE,0xBFBF,0xD0D0,0xDFDF,0x6282,0x6262,0x7383,0x6273,0x7373,0x6273,0x7373,0x6262,0x7352,0xB3C2,0x9382,0xB3A3,0x8293,0x82A3,0xA3A3,0xB382,0xA3A3,0x8282,0x82B3,0x8282,0x9372,0x9372,0x62A3,0x5252,0x7362,0x6273,0x5262,0x6362,0x4252,0x9F9F,0xC371,0x7272,0x6283,0x6272,0x72C3,0x83B3,0xC462,0xC361,0xC483,0x5272,0x5242,0x6352,0x6262,0x4252,0xB352,0x9372,0xC372,0xD4C0,0xD071,0xBFBF,0x7251,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x7070,0x6070,0x6070,0x9F9F,0x6060,0x7070,0x7070,0x7070,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7060,0x9F9F,0x9F9F,0x7180,0x80C0,0x9F9F,0x6060,0x9F9F,0x9F9F,0x8050,0x80C0,0xAFAF,0xAFAF,0xB0A0,0x8F8F,0x8F8F,0xBFBF,0xCFCF,0x9F9F,0x9F9F,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F7F
.word 0xAFAF,0x9F9F,0x7F7F,0x8F8F,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x719E,0xB2C0,0x9EB0,0xAFAF,0x8292,0xA382,0xA182,0x9EBE,0xBFBF,0xCFCF,0xD0BE,0xCFCF,0xE0E0,0x7292,0x8362,0x8362,0x7362,0x6262,0x6273,0x6263,0x7373,0x8362,0x82C3,0x8293,0x8293,0x82A3,0xB393,0x8282,0x7272,0x7282,0x7293,0xA3A3,0x7272,0xC371,0xA372,0x5283,0x5252,0x5252,0x6362,0x6273,0x5262,0x4252,0xB363,0x6172,0x71D4,0x72D4,0x8383,0x61C3,0x93D4,0x8362,0xD462,0x9372,0x6283,0x5252,0x5263,0x6352,0x4252,0xC342,0xB393,0xD362,0xBFBF,0xD0C3,0x61D3,0x7252,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x6050,0x7060,0x9F9F,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x6070,0x9F9F,0x6060,0x9F9F,0x9F9F,0x9F9F,0x7070,0xAFAF,0x9F9F,0xC071,0xD070,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x5040,0x7060,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F6F
.word 0xBFBF,0x9F9F,0x8F8F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x7E7E,0x7E7E,0xAFAF,0xAFAF,0x9F9F,0xBC8D,0xB1B2,0x9EA1,0xBFBF,0x8292,0x8282,0x9172,0x9E9E,0xBFBF,0xCFCF,0xCFCF,0xD1E0,0xE1E1,0x6272,0x8372,0x6283,0x6273,0x7373,0x7362,0x7362,0x5262,0x8262,0x82B3,0xA393,0xA393,0xA393,0x9372,0x7293,0x9393,0x7272,0x7293,0x7282,0x7272,0x72C4,0x8282,0x5283,0x5252,0x5273,0x6252,0x5252,0x5252,0x4242,0x8361,0x6273,0xC361,0xAFAF,0x83B3,0x6273,0xAFAF,0x83C4,0x8362,0xD471,0x43B3,0x5242,0x6362,0x5252,0x4252,0x9F9F,0x7283,0x71D4,0xC3C3,0xD4C0,0x52D0,0x6251,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x7070,0x9F9F,0xAFAF,0x6070,0x7070,0x7070,0x9F9F,0x7070,0xAFAF,0x7060,0xAFAF,0x6060,0x7060,0x7070,0xBFBF,0xAFAF,0x7060,0x80C0,0xBFBF,0xAFAF,0x6050,0x9F9F,0x6050,0x70C0,0xAFAF,0x9F9F,0x6080,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F
.word 0xCFCF,0xCFCF,0x8F8F,0x8E7E,0xAFAF,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xA182,0xAE91,0x909E,0x8282,0xA382,0xA082,0xBFBF,0xBFBF,0xBFBF,0xD0E0,0xD1E0,0xD1E0,0x6281,0x8372,0x8362,0x7373,0x6273,0x6352,0x6263,0x6273,0xB362,0x82A3,0x9393,0x9393,0x8272,0x7283,0x7272,0x9393,0x8383,0x7272,0x6272,0x72B3,0x7272,0x9383,0x4262,0x6252,0x6262,0x5273,0x5252,0x5273,0x5242,0xAFAF,0x6262,0x52C4,0xC361,0xAFAF,0x7372,0x8383,0xAFAF,0x8383,0xBFBF,0x7371,0x5242,0x6263,0x5263,0x4242,0xC462,0xC493,0xC371,0x71D3,0xBFBF,0x9F9F,0x6152,0x9F9F,0x8F8F,0x9F9F,0x5050,0x9F9F,0x7060,0x9F9F,0xAFAF,0xAFAF,0x7070,0x9F9F,0x8F8F,0x7F7F,0x4040,0x9F9F,0x7070,0xAFAF,0xAFAF,0x6060,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x7060,0x9F9F,0x7070,0xAFAF,0xAFAF,0xAFAF,0xF070,0xCFCF,0x6070,0x7172,0x70C0,0xAFAF,0x9F9F,0x5050,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F6F
.word 0xCFCF,0xBFBF,0x7E80,0x9E7E,0xBFBF,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x7E7E,0x9F9F,0x8E8E,0xAFAF,0x9F9F,0xAFAF,0x9B8D,0xA1D4,0xD092,0xA09E,0x8292,0xA382,0x9272,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xD1D0,0xD2E0,0x7272,0x8383,0x6283,0x6262,0x6362,0x6273,0x6362,0x6273,0xB373,0x93A3,0x9372,0x8372,0x9393,0x9372,0x9372,0x7272,0x7272,0x7272,0x72D4,0x71D4,0x8372,0x8372,0x5252,0x5252,0x6352,0x6362,0x5262,0x5262,0x5242,0xAFAF,0xC471,0x6262,0x9F9F,0x7172,0x8361,0x6273,0xD461,0xC461,0xB373,0x7362,0x4242,0x5252,0x5273,0x4242,0xC352,0x7272,0xC3C3,0xAFAF,0xC0C3,0xBFBF,0x6252,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7060,0x7070,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7070,0x6070,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0xAFAF,0x9F9F,0x7070,0xAE70,0xBFBF,0xAFAF,0x7060,0xC070,0xC0C0,0xBFBF,0xE3F2,0xC071,0xAFAF,0x9F9F,0x9F9F,0x5050,0x8080,0x70C0,0xAFAF,0x6080,0x5050,0x9F9F,0x9F9F,0x4040,0x8F8F,0x8F8F,0xD542,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F
.word 0xBFBF,0xCFCF,0x70D0,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9E8E,0xCFCF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x7E7E,0x9F9F,0xAFAF,0xAFAF,0x9E9E,0xAFAF,0xAFAF,0xAFAF,0x829B,0xB081,0x909E,0x8281,0x82A3,0x9193,0x9E9E,0xBFBF,0xCFCF,0xE0D0,0xD0D1,0xC1F0,0x7272,0x6283,0x7383,0x6373,0x5273,0x6373,0x7362,0x6262,0xB383,0x9393,0x9383,0x8362,0x9383,0x9383,0x7283,0x8393,0xC372,0x7162,0x7172,0xC471,0xC371,0x7372,0x4242,0x5252,0x5252,0x6262,0x5263,0x5252,0x4242,0xC462,0x7162,0x6262,0x6262,0xAFAF,0x9F9F,0xC461,0x6362,0xAFAF,0xC462,0x5272,0x5242,0x5252,0x6352,0x6F6F,0xC452,0xC372,0xD471,0xC381,0xC3C3,0x61D0,0x6242,0xAE41,0x8F8F,0x5050,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0x9F9F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6050,0x7060,0x7070,0x9F9F,0x7070,0xCFCF,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0xC071,0xF2B0,0xEFEF,0x80D0,0xB0C0,0x8F8F,0x5050,0x8F8F,0x7170,0xAFAF,0xD090,0x50D0,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0xBFBF,0xFFFF,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F
.word 0x9E60,0xBE9E,0x80D0,0x4050,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8E8E,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8E8E,0x9F9F,0xAFAF,0x9E9E,0x9E9E,0xAFAF,0xAFAF,0xAFAF,0xA082,0xBFBF,0x9282,0x8282,0xA082,0xBFBF,0xBFBF,0xE0D0,0xD0D0,0xD1D1,0xCFCF,0x7272,0x7383,0x7362,0x6262,0x6263,0x6263,0x7373,0x6373,0x9393,0x7283,0x8393,0x8393,0x7272,0x7262,0xB372,0x7293,0x6283,0xC472,0x6272,0x7272,0x9361,0x6272,0x4242,0x6252,0x5262,0x5252,0x6262,0x4252,0x9F9F,0x62D4,0xC362,0xAFAF,0x6373,0x9F9F,0x7271,0xBFBF,0x7362,0xC462,0xAFAF,0x42C4,0x5242,0x6373,0x5252,0x4242,0x9F9F,0xC483,0x71C3,0x71D3,0xC0C3,0x52E3,0x6241,0xAFAF,0x8F8F,0x9F9F,0x6060,0x9F9F,0x7070,0x7070,0x9E70,0xBFBF,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5060,0x5050,0x9F9F,0x9F9F,0x9F9F,0x7060,0xC060,0xBFBF,0xC4C6,0x9F9F,0xFE80,0x70E0,0x7171,0x8171,0x8192,0xD0C1,0xAFAF,0x60C0,0x8F8F,0x8F8F,0x9F9F,0x7171,0xB080,0x9F9F,0x5050,0x8F8F,0x5050,0x7F7F,0x6F6F,0xBFBF,0xBFBF,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F
.word 0x9F9F,0xBFBF,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x7E7E,0x9F9F,0xAFAF,0x9E8E,0x9E9E,0x9E9E,0x9E9E,0x9F9F,0x9F9F,0xA072,0x9EAE,0x8271,0xA3A3,0x7193,0x9EC0,0xD0C0,0xD0E0,0xD1D0,0xE0D0,0xB2F0,0x8362,0x8383,0x6273,0x6373,0x6362,0x6273,0x7363,0x5252,0x9383,0x7283,0x7283,0x9383,0x9393,0x9372,0x7272,0x7262,0x7262,0x6272,0x7272,0x72C4,0x62C4,0x5283,0x5242,0x5252,0x6352,0x5252,0x5252,0x4242,0x5242,0xAFAF,0x6273,0xAFAF,0x6262,0x6273,0xAFAF,0x6272,0xAFAF,0x6362,0xE461,0x4261,0x4242,0x5252,0x4263,0x4242,0xD452,0x7272,0xC371,0xE471,0xD371,0x8F8F,0x6252,0xAFAF,0x8F8F,0x5050,0x9F9F,0x7070,0xAFAF,0x9E70,0xCFCF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x6050,0x8F8F,0x8F8F,0x9F9F,0x7060,0x7060,0xBFBF,0xD6D5,0x9F9F,0x5051,0xFFFF,0xBFBF,0xC0E0,0xC0C3,0x8181,0xC0B2,0x80D0,0x50C0,0x8F8F,0x8F8F,0x8F8F,0x7171,0xAFAF,0xBFBF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0xCFCF,0xBFBF,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F
.word 0xAFAF,0x7080,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7E8E,0x9F9F,0xAFAF,0x9E9E,0x9E9E,0xBFBF,0xB2C0,0x9F9F,0x8172,0xBFBF,0x92A0,0x82A3,0x8293,0xCED0,0xD0C0,0xD1E0,0xC1D0,0xF0D1,0xB2E1,0x8372,0x7383,0x6362,0x6263,0x7363,0x6362,0x5263,0x7352,0x93B3,0x9393,0x9372,0x7272,0x9383,0x9393,0x8383,0x7272,0xC362,0x7162,0x6272,0xAFAF,0x7271,0x5273,0x4242,0x5252,0x6252,0x6352,0x5252,0x4252,0x9F9F,0x62C0,0x5283,0x6252,0xAFAF,0x5252,0x6273,0x9F9F,0xAFAF,0x7361,0xAFAF,0x4261,0x4242,0x5242,0x5263,0x6F6F,0x7243,0x72C4,0xC3C3,0x71C3,0xD0D3,0x42C0,0x6252,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x7070,0xAFAF,0xCFCF,0xCFCF,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x5050,0x8F8F,0x6060,0x9F9F,0xAFAF,0xAFAF,0xD6D6,0x57C6,0x8F8F,0x9F9F,0xC080,0xC380,0x8181,0x81C3,0x9291,0xC0C1,0x9F9F,0x8F8F,0x8F8F,0xC050,0xC071,0xC071,0xD080,0xAFAF,0xAFAF,0x9F9F,0x6050,0x9F9F,0xCFCF,0xC0D0,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F
.word 0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9E9E,0x9E9E,0x9E9E,0xBFBF,0xD1C0,0xBFBF,0x7161,0x9EA0,0x8290,0xA382,0x8282,0xCFCF,0xD0E0,0xD1D1,0xE1D1,0xE1D1,0x72E2,0x7283,0x7383,0x7373,0x6362,0x6373,0x6352,0x7362,0x7352,0x9393,0x8383,0x8393,0x8393,0x8372,0x8362,0x7272,0xC483,0x6262,0xC472,0x7162,0x7272,0x72D4,0x4262,0x4242,0x5252,0x5252,0x5252,0x6252,0x4242,0x9F9F,0xAFAF,0x6271,0x9F9F,0xBFBF,0xAFAF,0x5263,0x9F9F,0x6261,0xAFAF,0xC462,0x7F7F,0x4242,0x4252,0x5252,0x6F6F,0xD462,0xB372,0x72C3,0x71D4,0xBFBF,0x41D3,0x6252,0x9F9F,0xBFBF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xBFBF,0xDAF9,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x5050,0x8F8F,0x5050,0x8F8F,0x9F9F,0x8070,0xC562,0x58C5,0xAFAF,0x8F8F,0x8F8F,0xC371,0xC071,0x8181,0x81B1,0xA192,0xD0A1,0x70C0,0x8F8F,0x8F8F,0x8F8F,0x6271,0xC071,0xC071,0x7EC0,0x5060,0x9F9F,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F6F
.word 0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8E8E,0x9F9F,0xAFAF,0xAFAF,0x9E8E,0x9E9E,0xE0AE,0xE1D1,0xD3FE,0x7F7F,0xC081,0x719E,0xA3A3,0x93A3,0xDFDF,0xC1E0,0xD1D1,0xD2D1,0xE1D1,0x62C2,0x8372,0x7383,0x6362,0x6363,0x6362,0x7363,0x5273,0x8352,0x72B3,0x7283,0x8383,0x8383,0x9383,0x7283,0x7262,0x7162,0x71C4,0x6162,0x62C4,0x61C4,0x8361,0x4262,0x4242,0x5252,0x5252,0x5252,0x5252,0x7F7F,0x9F9F,0x6262,0xAFAF,0x5262,0xAFAF,0xAFAF,0x5262,0x6362,0xAFAF,0x6272,0xAFAF,0x4242,0x5242,0x5263,0x4242,0x4242,0x9F9F,0x9393,0xC372,0xD3C3,0xC071,0x52D4,0x6352,0x8F8F,0x7EC0,0x7070,0xAFAF,0x7070,0xBFBF,0xCFCF,0xD7EA,0xBFBF,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x9F9F,0x7060,0xAFAF,0x8F8F,0x9F9F,0xBFBF,0x8F8F,0x8F8F,0x8F8F,0x7171,0xB271,0xB181,0xA182,0xC381,0x70C0,0x8F8F,0x8F8F,0xBFBF,0xC071,0x71C3,0x70D0,0x70C0,0x9F9F,0x9E70,0xBFBF,0xBFBF,0xBFBF,0xD0C0,0xBCBD,0xBDCC,0x4A8E,0x6F6F,0xC542,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F
.word 0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xAFAF,0x8E70,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9E8E,0xAFAF,0x9E8E,0xAE9E,0xE0BE,0xF1D1,0xFFFF,0x6F6F,0x7150,0xC04E,0x92C3,0x82B3,0xD1B2,0xD1F0,0xD2D1,0xC1D2,0xE1C2,0x72C3,0x8383,0x7383,0x7363,0x6263,0x5263,0x5273,0x5263,0x9363,0x8393,0x8383,0x7283,0x7262,0x7283,0x8383,0x8362,0x7272,0x6161,0xAFAF,0x6162,0xAFAF,0x6283,0x4242,0x4142,0x5242,0x6252,0x5252,0x4252,0x4242,0xAFAF,0x5262,0x7162,0xAFAF,0x6352,0xAFAF,0xAFAF,0x4262,0x6262,0xAFAF,0xAFAF,0x6F6F,0x5242,0x5252,0x4252,0x6F6F,0x8F8F,0x83B3,0xC3B3,0x71C3,0xD4C3,0xAFAF,0x6252,0x9F9F,0x9F9F,0xAFAF,0x7070,0x7070,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0x9F9F,0x8F8F,0x7F7F,0x9F9F,0xD080,0xBFBF,0xAFAF,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0xC071,0x81C3,0xB1B2,0xB282,0xF2D2,0xAFAF,0x8F8F,0x8F8F,0xC060,0x72C0,0xC071,0xAFAF,0x80D0,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xD0AE,0xECD0,0xEAEA,0xFCFB,0x9CAD,0x8F8F,0xFFFF,0xEFEF,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F
.word 0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8E8E,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9E9E,0xC0BE,0xE1D1,0x72F1,0x7F7F,0x6F6F,0x6F6F,0x82E1,0xB3B2,0xD282,0xD2F0,0xD2C1,0xC2C2,0xE2D1,0x8393,0x8393,0x7373,0x6373,0x6362,0x7352,0x6263,0x5252,0xC473,0x6283,0x7383,0x8383,0x8383,0x6262,0x7262,0xC462,0x6362,0xC462,0x7252,0xAFAF,0xAFAF,0x6262,0x7F7F,0x4242,0x4252,0x5252,0x5252,0x5252,0x4242,0xAFAF,0x6262,0x9F9F,0x9F9F,0x5262,0x9F9F,0xAFAF,0xAFAF,0x5252,0x6152,0xAFAF,0x4251,0x4242,0x5242,0x4252,0x6F6F,0xD352,0xC472,0x7272,0xD472,0xBFBF,0x7F7F,0x6252,0x7F7F,0xBFBF,0x7070,0x7070,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x9F9F,0xCFCF,0xAFAF,0x9F9F,0x7F7F,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0x8F8F,0x8F8F,0xC5C5,0xAFAF,0x9F9F,0x7171,0x81B2,0xB291,0xEFEF,0xD1E1,0x8F8F,0x4040,0xBFBF,0xC060,0x8182,0xC081,0xD080,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xC0C0,0xCFCF,0xEAEA,0xBCEC,0xBCBC,0xBFBF,0xCFCF,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xAFAF,0xBFBF,0x7080,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0x9E8E,0xAFAF,0xAFAF,0x9E9E,0xAE9E,0xCFCF,0xF1C1,0x83F1,0x6F6F,0x6F6F,0x7F7F,0xA3D1,0xB3B2,0xD292,0xE1F1,0xD2C2,0xD2D2,0xD3E2,0x9372,0x8383,0x7383,0x6373,0x5263,0x5263,0x6363,0x5263,0xB373,0x8383,0x8383,0x8362,0x7383,0x8383,0x6273,0x6162,0x6262,0xAFAF,0xAFAF,0x6273,0x6161,0x52D4,0x4242,0x4242,0x5252,0x5251,0x5252,0x4252,0x7F7F,0x9F9F,0xAFAF,0x5252,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x4242,0x6F6F,0x4242,0x4352,0x6F6F,0xAFAF,0x72C4,0xD4C3,0xAFAF,0x71C3,0x41D3,0x7F7F,0x7F7F,0xAFAF,0x7070,0xAFAF,0x7070,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0xD0D0,0xAFAF,0x4157,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x71C3,0xA1B2,0x82B2,0xB2C3,0xC1B1,0xE0D0,0x7F7F,0xC0C0,0xAFAF,0x81C3,0xD4C0,0xC0C0,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCCCB,0xAEBE,0xCFCF,0xBFBF,0xBFBF,0x8F8F,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F41
.word 0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9E8E,0x9E9E,0xAFAF,0x9E9E,0xCFCF,0xE1C1,0x62F2,0x7F7F,0x6F6F,0xBFBF,0xA3D2,0xB382,0xC2B3,0xD2E2,0xC2C2,0xC3B2,0xC3D2,0x9383,0x9383,0x7373,0x5263,0x6373,0x6363,0x6362,0x4252,0x93C4,0x7383,0x7383,0x8373,0x8362,0x7373,0x6273,0xC462,0xAFAF,0x6261,0x6261,0xAFAF,0x9F9F,0x5262,0x4242,0x4142,0x5252,0x4252,0x5251,0x4242,0x7F7F,0x9F9F,0xAFAF,0x9F9F,0x6152,0x9F9F,0x9F9F,0x6351,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x6F6F,0x5243,0x4252,0x4242,0x6F6F,0x9F9F,0xC461,0x7161,0x71D4,0xAFAF,0xBFBF,0x5241,0x6F6F,0x9F9F,0xAFAF,0x7070,0xC070,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xE0E0,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x424A,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0x9281,0x92A1,0x8282,0xA182,0xC0B1,0xD0D0,0x71C0,0xAFAF,0xC281,0xC071,0xC071,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x7B8B,0xAFAF,0xCFCF,0xCFCF,0xAFAF,0xC0C0,0x8F8F,0xFFFF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4246,0x4248,0x709F
.word 0x7F7F,0x6F6F,0x6F6F,0x9F9F,0x9E8E,0xBFBF,0xBFBF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x7E60,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8E7E,0x9F9F,0xAFAF,0xAFAF,0x9E9E,0xBFBF,0xCFCF,0xE1C1,0x42D2,0x6F6F,0x6F6F,0xAFAF,0xA3E3,0xA3A3,0xC2B2,0xC2D3,0xC2C2,0xC2B3,0xB3E3,0x8383,0x8383,0x7373,0x6362,0x6363,0x6352,0x6352,0x5242,0x83B4,0x6283,0x8362,0x73A4,0x6273,0x6272,0xC461,0x6273,0x6252,0x6262,0xAFAF,0xAFAF,0x9F9F,0x4262,0x6F6F,0x4242,0x5142,0x5252,0x4252,0x4242,0x9F9F,0x9F9F,0x6152,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x6F6F,0x6342,0x4252,0x4242,0x6F6F,0xAFAF,0x6262,0xD461,0xAFAF,0x6162,0xAFAF,0x6F6F,0x4173,0x9F9F,0xAFAF,0x7070,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xD0D0,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8281,0x91B2,0x9182,0xB292,0xC1C2,0xC1C1,0xD0D0,0xC0D0,0x9281,0xC3C0,0xC0D0,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0xCFCF,0xCFCF,0xAFAF,0x8F8F,0x7F7F,0xE0F4,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x9F47
.word 0x6F6F,0x6F6F,0x7F7F,0x6E4E,0xAE8E,0xCFCF,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0x9E9E,0xAFAF,0xAE9E,0xBFBF,0xE1D1,0x42C2,0x6F6F,0x7F7F,0x9F9F,0x93D4,0xA393,0xD2C3,0xB2D3,0xC3B3,0xC3B2,0x83D3,0x8383,0x7373,0x6373,0x6363,0x5252,0x5263,0x5263,0x6352,0x7383,0x7373,0x7373,0x6362,0x6373,0x6352,0x6162,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x5262,0x7F7F,0x4242,0x4252,0x5252,0x4152,0x6F6F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x7F7F,0x4242,0x4252,0x4242,0x6F6F,0xAFAF,0xAFAF,0x9F9F,0x6271,0xAFAF,0xAFAF,0x7342,0x4262,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5241,0x81C3,0xB291,0x9281,0xC2A1,0xD2D2,0xC1B1,0xD3C1,0xAFAF,0x81C1,0xC0B2,0xD371,0xC0C0,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0xBFBF,0xBFBF,0xAFAF,0xD0B0,0x8F8F,0x7F7F,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F
.word 0x7F7F,0x6F6F,0x6F6F,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x708E,0x9E8E,0xBFBF,0xBFBF,0xE1C1,0x41B3,0x6F6F,0x6F6F,0x8F8F,0x83C3,0x9383,0xD3C3,0xB2C3,0xB3B3,0xC3B3,0x73C3,0x8373,0x7373,0x6363,0x6352,0x6343,0x6352,0x4252,0x6342,0x62B4,0x7373,0x6363,0x6363,0x6273,0x6362,0x9F9F,0x6261,0x6262,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x4252,0x4242,0x4142,0x5242,0x5252,0x4252,0x4242,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x4141,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5151,0xAFAF,0x6F6F,0x4242,0x5242,0x4242,0x6F6F,0x9F9F,0x9F9F,0x6261,0x9F9F,0x7162,0xAFAF,0x5242,0x4173,0x9F9F,0xAFAF,0x7070,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8271,0xA192,0x9292,0xC2A1,0xB2B2,0xB1B1,0xD0B1,0xD0D1,0x8281,0xD3B1,0xC0C0,0xC0B3,0xE0E0,0xD0E0,0xC0C0,0x9F9F,0x8F8F,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0x4E6E,0x7F7F,0x7071,0xAFAF,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xB1D0,0xD1B1,0x42B3,0x6F6F,0x6F6F,0x9F9F,0x73C4,0x9383,0xC3B3,0xA3B3,0xA3B3,0xB3A3,0x6293,0x7373,0x6363,0x6363,0x4343,0x4252,0x4352,0x4242,0x6342,0x6373,0x6273,0x7362,0x6262,0x9F9F,0x9F9F,0x5262,0x9F9F,0xAFAF,0x9F9F,0x6161,0xAFAF,0x9F9F,0x4252,0x6F6F,0x5242,0x5252,0x5142,0x4252,0x6F6F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6F6F,0x6F6F,0x5242,0x4442,0x6F6F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x7F7F,0x6342,0x4262,0x8F8F,0xAFAF,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x8F8F,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8261,0xB292,0x92A1,0xA192,0xB2A1,0xB1B1,0xB1B1,0xE1C1,0xC1E0,0x81B1,0xC3C0,0xC0C0,0xE3D0,0xAFAF,0x71C3,0x9F9F,0xBFBF,0xAFAF,0xAFAF,0xC071,0x8F8F,0x8F8F,0x7F7F,0xAFAF,0xB071,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F
.word 0x9F9F,0x9F9F,0xAFAF,0x4051,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9E8E,0x9E9E,0x9E9E,0xAE9E,0xCFCF,0xE1C1,0x4282,0x7F7F,0x7F7F,0x7F7F,0x7383,0x8373,0xB393,0x9393,0x9393,0xC483,0x6373,0x6373,0x6363,0x6363,0x4252,0x5243,0x4243,0x4242,0x4742,0x6263,0x5263,0x5263,0x5262,0x9F9F,0x5252,0x9F9F,0x4752,0x5252,0x5247,0x9F9F,0x5242,0x9F9F,0x9F9F,0x6F6F,0x4241,0x5152,0x5252,0x4252,0x6F6F,0x9F9F,0x5252,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x7F7F,0x6F6F,0x4242,0x4242,0x4252,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6262,0xAFAF,0x6242,0x4262,0x7F7F,0xD070,0x5070,0x4150,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xCFCF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xC0E0,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9292,0x92B1,0xB291,0xA1B2,0xB292,0xB1B1,0xB1B1,0xE0D1,0xD0E0,0xC0C3,0xB3C2,0xBFBF,0x81D3,0x9E91,0x9F9F,0xD0F3,0xBFBF,0x8080,0x9F9F,0x7F7F,0x4A4A,0x7F7F,0x8071,0xAFAF,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F42
.word 0x9F9F,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9E9E,0xBFBF,0xBFBF,0xD1C1,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0x7362,0xB483,0xB493,0x8393,0x8383,0xB483,0x5273,0x6363,0x4352,0x5252,0x4342,0x4342,0x5242,0x4242,0x6245,0x5263,0x4343,0x4752,0x4752,0x6352,0x9F9F,0x5247,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x4242,0x4242,0x5241,0x5252,0x4252,0x7F7F,0x9F9F,0x8F8F,0x4252,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x4160,0x6F6F,0x4242,0x4241,0x4252,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x4172,0x6F6F,0x4273,0x7F7F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0xCFCF,0xBFBF,0xC080,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0x7F7F,0x8F8F,0x8F8F,0xB351,0xA182,0xA1B1,0xB292,0xB1B1,0xA1A1,0xB2A1,0xB1B1,0xC1B1,0xE0D1,0xD3E0,0xC181,0xC3C1,0xC371,0x9F9F,0x9F9F,0xD0E0,0xAFAF,0xD0C3,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0xAFAF,0xC070,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F
.word 0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0x61B0,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0xAFAF,0x8E8E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0xBE9E,0xCFCF,0xD1C1,0x52C2,0x7F7F,0x7F7F,0xAFAF,0x8383,0x9383,0x8393,0x8383,0x8383,0x9383,0x4263,0x4352,0x5263,0x4343,0x4242,0x4242,0x4242,0x4242,0x8F8F,0x6362,0x6262,0x6162,0x9F9F,0x6161,0xAFAF,0x9F9F,0x6161,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4252,0x6F6F,0x4242,0x5242,0x5142,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x5242,0x4142,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x7F7F,0x4262,0x6F6F,0xD071,0x40C0,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0xFFFF,0xC0E0,0xC0B0,0xC0D0,0xC1C0,0xC3B1,0xC0C0,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0x7F7F,0x8F8F,0x9F9F,0x9282,0xB1B1,0x9292,0xB2B1,0xB2A1,0xA1A1,0xB1B2,0xB1B1,0xB1B1,0xE0D1,0xC3D0,0xD3D0,0x81B1,0x8F8F,0xBFBF,0xC0C3,0x7171,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xAFAF,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F
.word 0x9F9F,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0x6060,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x8E8E,0x9E9E,0x9E9E,0xCEAE,0xD1E0,0xD1D2,0x62D2,0x4142,0x9F9F,0x7283,0x8383,0xB393,0x72B3,0x9393,0x8383,0x8383,0x5273,0x5263,0x4342,0x4242,0x4342,0x4352,0x4242,0x4242,0xAFAF,0x6262,0x5252,0x5252,0x9F9F,0x9F9F,0x6152,0x9F9F,0x9F9F,0x5252,0x5251,0x9F9F,0x9F9F,0x9F9F,0x4241,0x6F6F,0x5241,0x4152,0x4242,0x4242,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6F6F,0x6F6F,0x4252,0x4251,0x6F6F,0x7F7F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8F8F,0x6F6F,0x5252,0xAFAF,0xC0D3,0xAFAF,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xAFAF,0x8171,0x71D0,0x9F9F,0x8070,0xAFAF,0xC3C3,0xC381,0xAFAF,0xAFAF,0xD0D0,0xAFAF,0x8F8F,0xC351,0xA182,0xB1B1,0x9292,0xB1B1,0xA1B2,0xB2B2,0xB2A1,0xB1B1,0xB1B1,0xC1B1,0xD0E0,0xEFEF,0xBFBF,0x7F7F,0xD0D0,0xD471,0xC272,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0xB071,0xAFAF,0x7271,0xDBE4,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F42
.word 0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9E8E,0x9E9E,0xBFBF,0x9E9E,0x9E9E,0xCEBE,0xDFDF,0xD2C2,0x82D1,0x9F9F,0xC361,0x7282,0x7293,0xB3A3,0x9393,0x8393,0x8393,0x9393,0x4263,0x5252,0x5263,0x4352,0x4252,0x4242,0x4242,0x4242,0x6363,0x6363,0x6363,0x4752,0x6352,0x6352,0x5252,0x6352,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x6F6F,0x6F6F,0x4242,0x4241,0x4142,0x6F6F,0x7F7F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0xAFAF,0x7F7F,0x6F6F,0x5242,0x5242,0x6F6F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6F6F,0x4252,0x9F9F,0xAFAF,0xC0C0,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x9F9F,0x7070,0xD0C0,0xBFBF,0xC1C2,0xD2C1,0xBFBF,0xBFBF,0xBFBF,0x9F9F,0xAFAF,0xA182,0xB1B2,0x92A1,0xB2A1,0xB2B1,0xB2A1,0xB1B1,0xB2A1,0xB1A1,0xB1B1,0xD1B1,0xF2E2,0xBFBF,0xD361,0xC3D0,0x71C3,0x70B2,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0xBFBF,0xAFAF,0xB361,0xEFEF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xF573,0xFFFF,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F
.word 0xAFAF,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x62C5,0x9F9F,0x708E,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0x9E9E,0xCFCF,0xD0DE,0xD2B1,0xC2D1,0x6262,0xD072,0x8272,0x8282,0xB3B2,0x93B3,0x9393,0x8393,0x9383,0x5273,0x6363,0x5263,0x5243,0x5243,0x5263,0x4242,0x4742,0x7352,0x6262,0x6262,0x6162,0x6262,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x6161,0xAFAF,0x6161,0xAFAF,0x9F9F,0x8F8F,0x4242,0x4142,0x5252,0x4241,0x6F6F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x6F6F,0x4242,0x4241,0x5251,0x6F6F,0x6F6F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x5F5F,0x6F6F,0x4252,0x8F8F,0xAFAF,0xCFCF,0x40F0,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xEFEF,0xF2F5,0xF3F2,0xE5F5,0xCFCF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xBFBF,0xEFEF,0xDFDF,0xDFDF,0xF0E0,0xE0F0,0xF0F0,0xE0F0,0xD0E0,0x9F9F,0xC360,0x9281,0xB1A1,0x92A1,0xB192,0xB1B1,0xB292,0xB1B2,0xB2B1,0xB1A1,0xB1B1,0xB1B1,0xD1D1,0xC2B2,0xD0D0,0x71D4,0x8272,0x9F9F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x6F6F,0xAFAF,0xAFAF,0xC371,0xBFBF,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0xEFEF,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F
.word 0x9F9F,0xAFAF,0x4863,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8E9E,0x9E9E,0x9E9E,0x9E9E,0xCEBE,0xD1E0,0xC2B1,0xC2D1,0x8162,0xC3C3,0xB281,0xA392,0xB2B2,0xB3C2,0x9382,0x8372,0x9372,0x5273,0x6263,0x6363,0x6352,0x4352,0x4352,0x4252,0x8F8F,0x73B4,0x6373,0x5262,0xAFAF,0x6262,0x6262,0x6261,0xAFAF,0x6262,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x4242,0x4242,0x4142,0x4242,0x6F6F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5160,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x6F6F,0x4242,0x4242,0x4152,0x6F6F,0x7F7F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x6F6F,0x4242,0x4252,0x7F7F,0xAFAF,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0xAFAF,0xEFEF,0xF7F7,0xFFFF,0xF7F7,0xEFEF,0xEFEF,0xEFEF,0xCFCF,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x8F8F,0x9F9F,0xBFBF,0xF4D4,0xEFEF,0xF0F0,0xE0E0,0xD0D0,0xC1D0,0xC1D0,0xD0D0,0xF0E3,0xC0E0,0x9F9F,0x8282,0xA1A1,0x92A1,0xA192,0xB1B2,0xB2A1,0xB2B2,0xB1C1,0xB2B1,0xB1B1,0xB1B1,0xC1B1,0xC272,0xD3D3,0xC371,0xB282,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xBFBF,0xAFAF,0x8171,0xBFBF,0x6F6F,0x7F7F,0x6F6F,0xD142,0xEFEF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F
.word 0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x6050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8E8E,0x9E9E,0xAFAF,0xAFAF,0x9E8E,0x9E9E,0xCFCF,0xCFCF,0x9192,0xAFAF,0x8171,0x7181,0x81D0,0x8282,0xC392,0xB2B2,0x82A3,0x93A3,0x9393,0x5273,0x6363,0x6352,0x6363,0x5263,0x5263,0x4243,0x7342,0x6283,0x7362,0x7363,0x7363,0x5263,0x6352,0x9F9F,0x6252,0x9F9F,0x5252,0x5252,0x5252,0x9F9F,0x9F9F,0x4242,0x6F6F,0x5142,0x5252,0x4242,0x6F6F,0x9F9F,0x6151,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x6F6F,0x4142,0x4142,0x5252,0x6F6F,0x7F7F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x5F5F,0x4241,0x4242,0x7F7F,0xBFBF,0xBFBF,0xDFDF,0x8F8F,0xAFAF,0xEFEF,0xF7F7,0xF5F7,0xF7F7,0xF7F7,0xF7F7,0xF6F7,0xF5F7,0xFFFF,0xEFEF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xE5F3,0xEFEF,0xDFDF,0xD0D0,0x9281,0x8282,0x8282,0x8292,0xB2B2,0xC1B1,0xE0D1,0xD4D0,0x8282,0x9292,0x82A1,0x9282,0xB2B1,0xA1B2,0xF5D1,0xFFFF,0xB1B1,0xB1B1,0xB1C1,0xB1B1,0xC1B2,0xD3D3,0x82D4,0xC382,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0xAFAF,0x71C0,0xC071,0xAFAF,0x6F6F,0x6F6F,0x6F6F,0xBFBF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F
.word 0x9F9F,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x5040,0x6050,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x8E70,0xAFAF,0x8E8E,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9E9E,0xAFAF,0xAFAF,0xBFBF,0x7F7F,0x7161,0xA181,0xBFBF,0x8281,0x8282,0xB282,0xB3B2,0xA392,0x8282,0xB393,0x5283,0x6252,0x7363,0x6352,0x6352,0x6363,0x4242,0x7342,0x6283,0x6283,0x6262,0x6262,0x7262,0xAFAF,0x6262,0xAFAF,0x6261,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5F5F,0x4242,0x5241,0x4242,0x4241,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6060,0x9F9F,0xAFAF,0xAFAF,0x6F6F,0x6F6F,0x5142,0x4152,0x6F6F,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0xAFAF,0x6F6F,0x6141,0x6162,0x7F7F,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xEFEF,0xF7F5,0xF5F7,0xE7F7,0xD7E7,0xF7E7,0xF9F7,0xD7D7,0xF7F7,0xDFDF,0xCFCF,0x9F9F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xDFDF,0xF5F6,0xFFFF,0x9291,0xC2B2,0xB1C2,0xB1B1,0xA1B2,0xB192,0xB2A1,0xB1B1,0xE0D1,0x8281,0x8282,0x9292,0x9282,0xB1A1,0xB1A1,0xF2C2,0xD1F2,0xE1D1,0xC1D1,0xB1B1,0xB1B1,0xD3B1,0xC3C1,0xB3B2,0xC3B2,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0xC0D0,0xAFAF,0xB0C3,0xAFAF,0x7F7F,0x7F7F,0x8F8F,0xC0D0,0xAFAF,0xF2D0,0xFFFF,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F7F
.word 0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8E60,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x8E8E,0x8E8E,0x8E8E,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x6F6F,0x9281,0xAEB1,0xBFBF,0xB292,0x8282,0xB272,0xB1C3,0x9282,0x82B3,0xB382,0x5283,0x6363,0x5273,0x6363,0x6352,0x5252,0x4263,0x8343,0x73B4,0x7373,0x8373,0x6262,0xAFAF,0x7162,0x9F9F,0xAFAF,0xAFAF,0x6161,0xAFAF,0x6161,0x9F9F,0x6261,0x7F7F,0x4242,0x5242,0x5241,0x4142,0x6F6F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x6F6F,0x4242,0x4142,0x5142,0x6F6F,0x8F8F,0x508E,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x6162,0x6F6F,0xBFBF,0xBFBF,0xBFBF,0xFFFF,0xFFFF,0xEFEF,0xE7F7,0x59E9,0xC7DA,0xCFCF,0xE6DA,0xD7D7,0xD7E7,0xBFBF,0xAFAF,0xD6D6,0x9F9F,0xF463,0xBFBF,0xF6E6,0xF5F7,0xF3F3,0xB2E3,0xC2C2,0xD3D3,0xC3D3,0xB2B2,0xB1B2,0xA1B1,0xB1B2,0xB2B1,0xB1A1,0xB1E1,0x8272,0x8291,0x9281,0xB192,0xC1B2,0x92B1,0xB282,0xC1B1,0xD1D1,0xB1C1,0xB1B1,0xC2C1,0x82B3,0x92C3,0xB3B3,0x6293,0x8F8F,0x8F8F,0x9F9F,0xBFBF,0xCFCF,0xCFCF,0xD0D0,0xC172,0xCFCF,0xDFDF,0xDFDF,0xEFEF,0xDFDF,0xE0E0,0xF3E3,0xAFAF,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x427F
.word 0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x6050,0x6050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9E8E,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x8160,0xA182,0xAEBE,0x91A0,0x9282,0x8282,0xB181,0xC3D0,0x92B2,0x9292,0xB3B3,0x6293,0x6362,0x6352,0x6362,0x6373,0x6362,0x4252,0x8373,0x7262,0x8362,0x7362,0x6262,0x6273,0x6262,0x6262,0x6252,0x6262,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x4242,0x4142,0x4242,0x4152,0x4242,0x7F7F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x6F6F,0x6F6F,0x5142,0x4242,0x6F6F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0x8F8F,0x7F7F,0x5252,0x4051,0xBFBF,0xBFBF,0xBFBF,0xFFFF,0xFFFF,0xF6F7,0xDBE7,0xBFBF,0x9F9F,0xAFAF,0xAFAF,0xE6EA,0xE6D6,0xDFDF,0xBFBF,0xAFAF,0xD5D4,0xFFFF,0xD0E3,0xE5E4,0xDFDF,0xD3D0,0x9282,0xC1C2,0xD3C2,0xB1B1,0x82B2,0xC3B3,0xB1C3,0xB1A1,0xB1B1,0xA1B1,0xD1B1,0x72A1,0x8292,0x8282,0xB192,0xC1B1,0xB292,0xA192,0xA192,0xB1A1,0xB1D1,0xB1C1,0xC2B1,0x82B3,0xB3B2,0x93B3,0xB372,0xE6E6,0xBFBF,0xF0D0,0xDFDF,0xCFCF,0xD0E0,0xD3D0,0xF0D0,0xDFDF,0xCFCF,0xCFCF,0xCFCF,0xDFDF,0xDFDF,0xE0E0,0xE4D3,0xFFFF,0x9F9F,0x8F8F,0x4642,0x7F7F
.word 0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x6050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8E8E,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x8182,0xAEA1,0xA0AE,0x9282,0x8292,0x8282,0xB182,0xCFCF,0xB2B2,0xB2B2,0xB2B2,0x5262,0x6273,0x6373,0x6363,0x7352,0x6263,0x4252,0x9352,0x6283,0x6283,0x6272,0x7272,0x6271,0x6262,0xAFAF,0xAFAF,0xAFAF,0x6262,0x9F9F,0x9F9F,0x6161,0x8F8F,0x6F6F,0x4242,0x4152,0x4252,0x4241,0x7F7F,0x9F9F,0x7061,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x6F6F,0x5242,0x4251,0x4251,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0xAFAF,0xAFAF,0x7F7F,0x6F6F,0x5252,0x6F6F,0x9E40,0xBFBF,0xCFCF,0xFFFF,0xF7F7,0xF7F5,0xF9F6,0xBBE6,0x8F8F,0x9F9F,0x8F8F,0xEAD6,0xF6E6,0xEBFB,0xCFCF,0x9F9F,0xD668,0xBFBF,0x7272,0xD3C3,0xE0E0,0xC1B3,0xE3C2,0xDFDF,0xE6F4,0xC3D4,0xC2C3,0x8291,0xB3B3,0xB1B2,0xB1B1,0xA1B1,0xA192,0x92D1,0x8271,0x8281,0xA192,0xB2B1,0xA192,0x92B1,0xB2A1,0x9292,0xB192,0xB1B1,0xB2B1,0xA382,0xB3B2,0x93B3,0x9383,0xFFFF,0xDFDF,0xBECE,0xBFBF,0xD0D0,0xE0D0,0xDFDF,0xCFCF,0xD0E0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xD0D0,0xBFBF,0xF1F0,0xF3F3,0xCFCF,0xCFCF,0xF3E4,0x7FF9
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x5060,0x6050,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7E7E,0x9F9F,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x8140,0x8181,0xBFBF,0x91B0,0x9292,0x9282,0x8292,0xB182,0xCFCF,0xB2B1,0xB2B2,0xB2C2,0x6262,0x7362,0x6362,0x6262,0x6363,0x6362,0x5252,0x9373,0x6273,0x7383,0xC462,0x6262,0xAFAF,0xAFAF,0x7162,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x4242,0x4242,0x5241,0x4251,0x4242,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0x6F6F,0x4242,0x5251,0x4241,0x6F6F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x7F7F,0x6F6F,0x4242,0x7F7F,0xAFAF,0xBFBF,0xCFCF,0xF6E3,0xEFEF,0xEFEF,0xEFEF,0xCFCF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xD6D6,0xE5D6,0xD3D4,0xC371,0xD3D4,0xB2B2,0xEFEF,0xDFDF,0xCFCF,0xC3D4,0xC3C3,0xC3D4,0xB382,0x92B3,0xB1B2,0xB1B1,0x92A1,0xC1A1,0x7292,0x8272,0xB292,0x9292,0xB291,0x92B2,0xB1B1,0xC2A1,0xE2E2,0xD1D1,0xC2C1,0x82A3,0xA3A3,0x93C4,0x8372,0xEED4,0xCEE0,0xAFAF,0xC0C0,0xE0C0,0xDFDF,0xCFCF,0xD0D0,0xC0C0,0xB1C1,0xA1B1,0x81A1,0xC0B1,0xD3C0,0xD0D0,0xD0D0,0x81C1,0xAFAF,0xAFAF,0xEFEF,0x4AEF
.word 0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8190,0xB181,0x9E9E,0x81A0,0x9292,0x9292,0x92A3,0xC182,0xCFCF,0xC1C1,0xC2B2,0xB2C2,0x6262,0x7373,0x6273,0x6373,0x6362,0x6362,0x4252,0x9373,0x7283,0x7262,0x6262,0x6262,0x6283,0x6262,0xAFAF,0x6162,0xD461,0x6161,0x6161,0x9F9F,0xAFAF,0x7F7F,0x6F6F,0x4241,0x5252,0x5242,0x6F6F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x7F7F,0x5142,0x4251,0x4251,0x6F6F,0x9F9F,0x9F9F,0x9F9F,0x8E60,0xAFAF,0xAFAF,0x7F7F,0x5140,0x4251,0x4152,0x8F8F,0xAFAF,0xCFCF,0xD4C0,0xF7F7,0xEFEF,0xEFEF,0xBFBF,0xAFAF,0xCFCF,0x4861,0x9F9F,0xAFAF,0x8F8F,0x8F8F,0xCFCF,0xE6D6,0xCFCF,0xF7F5,0xC3E4,0xBFBF,0xC2C3,0xE3C2,0xDFDF,0xDFDF,0x71C0,0x71D4,0xBFBF,0x92D4,0x8282,0xB2B2,0xA1B1,0x92B1,0x9292,0x81C1,0x8172,0xA182,0x8292,0xA192,0x92B1,0xB192,0xF3B2,0xFFFF,0xF0F0,0xF0F0,0xB2E0,0xB3B2,0x93B3,0xC383,0xDFDF,0xBFBF,0xAFAF,0xD080,0xDFDF,0xE0E0,0xC0D0,0xB1C1,0xC1C1,0xB1C1,0xB1C2,0xC1C2,0xC2C1,0xEFEF,0xEFEF,0xB1D1,0x8182,0xB071,0x8181,0xC3C2,0x7F8F
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x7E50,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x407E,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7281,0x9E81,0x909E,0x9281,0x92B3,0xB2B3,0x82B3,0xB192,0xBFBF,0xB1B1,0xC1C1,0xB2D1,0x6262,0x6283,0x7373,0x7362,0x7362,0x6373,0x5252,0x8373,0x6283,0x7272,0x7262,0x6271,0xD462,0x6261,0x6262,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x6F6F,0x5142,0x5152,0x5242,0x6F6F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7F7F,0x5241,0x5251,0x4241,0x6F6F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x5040,0xAFAF,0xAFAF,0xBFBF,0xFFFF,0xFFFF,0xEFEF,0xFAEB,0xD0E4,0xCFCF,0x9F9F,0x9F9F,0xBFBF,0x8F8F,0x8F8F,0xDCBC,0xF7F6,0xF7F7,0xF7F7,0xD4EB,0xD4C3,0xE3D3,0xE3D3,0xBFBF,0xCFCF,0xC0C3,0xC3C0,0xCFCF,0xBFBF,0x82B2,0xA182,0xB1A1,0xA1A1,0x9192,0xC1B2,0x7272,0x9282,0x8282,0x9292,0x92B2,0x9282,0xD2B2,0xE3F3,0xD192,0xC1D1,0xE0E1,0xE0F0,0x72D4,0xE093,0xCFCF,0xD0D0,0xC0C0,0xF0D1,0xE0F0,0xD0D0,0xD1D1,0xE1D1,0xE1E1,0xF0F1,0xF0F0,0xF1E1,0xF0F1,0xF2F1,0xD0F2,0x92A1,0xB192,0xB2B2,0x71B1,0x6F6F,0x6F6F
.word 0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7160,0x8181,0xBFBF,0x91B0,0x9292,0xB392,0x82B3,0x92B3,0xD092,0xE0CE,0xC1C1,0xD1B1,0xB2D2,0x8372,0x8383,0x7383,0x7373,0x6273,0x6273,0x5252,0x8383,0x7262,0x8383,0x6262,0xC462,0x6161,0x7271,0xAFAF,0x7162,0xAFAF,0xAFAF,0x6162,0xAFAF,0x6161,0x4252,0x6F6F,0x5241,0x5252,0x5251,0x4142,0x9F9F,0x6070,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9E70,0xAFAF,0x8F8F,0x4141,0x5152,0x4142,0x4142,0x7F7F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x608E,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0xC070,0xCFCF,0xF7D2,0xE7F5,0xCFCF,0xDFDF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0x6060,0x8F8F,0x8F8F,0xAFAF,0xF7FA,0xE7EA,0xEFEF,0xEFEF,0xBFBF,0xC3C3,0xD4D0,0xD3C3,0xD072,0x9371,0xC0C3,0xC0C2,0xBFBF,0x92D0,0x8282,0xB1B2,0xB2B1,0x92A1,0xA182,0x72B1,0x8281,0x8281,0x9292,0x91B2,0x9282,0x92A1,0xB2B2,0xB1B1,0xB1B1,0xC2B1,0xCFCF,0x72D0,0xDFDF,0xCFCF,0xBFBF,0xE0B2,0xDFDF,0xE0E0,0xD1E1,0xD1D1,0xD1D1,0xD1C1,0xF0D1,0xE1E0,0xD1D1,0xD1D1,0xB1C1,0xA1B1,0xB192,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F7F
.word 0x7F7F,0x8F8F,0x8F8F,0x5040,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7E50,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8171,0x9181,0x9E9E,0xB381,0xB392,0xB2B3,0xB392,0x9292,0xD0B2,0xD0CE,0xD1C1,0xE1C1,0x82D1,0x8372,0x8383,0x7383,0x6273,0x7373,0x7373,0x6352,0x83C3,0x6272,0x6272,0x6172,0x62C4,0xC461,0xAFAF,0x6262,0xAFAF,0x6161,0x6161,0x6161,0x9F9F,0xAFAF,0x7F7F,0x4242,0x5252,0x5152,0x4252,0x6F6F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x4151,0x4152,0x6F6F,0x6F6F,0x708E,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x9F9F,0x9F9F,0xCFCF,0xBFBF,0xF371,0xBFBF,0xCEBC,0xDFDF,0xEFEF,0xDFDF,0xEFEF,0xCFCF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0xEBFA,0xDBDB,0xF8E9,0xF8FA,0xE9E8,0xBFBF,0xC1C3,0x71C2,0x72C3,0xE0D3,0xE0E0,0xDFDF,0xDFDF,0xCFCF,0x8292,0xB292,0xA1B1,0x92A1,0x8281,0x92A1,0x8172,0x8272,0x9292,0x9292,0x8282,0xA192,0xB192,0xD1C1,0xC1C1,0xB1B1,0xE0C1,0x82C3,0xDFDF,0xCFCF,0xD0E0,0xE0E1,0xE0D0,0xE1E0,0xD1D1,0xC1C1,0xA1C1,0xC1B1,0xD1C1,0xD1C1,0xD1C1,0xC1C1,0xA1B1,0x9292,0xC0B2,0x6F6F,0x6F6F,0xC442,0x7F7F,0x7F42
.word 0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4040,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8181,0x9E92,0xA0AE,0x92A1,0xB3B2,0xB392,0xB392,0x92B3,0xE0B1,0xC0DE,0xD1D1,0xE0D1,0x82D2,0x9383,0x8383,0x6283,0x7383,0x7373,0x6262,0x6252,0x7283,0x8362,0x8362,0xC462,0x7161,0xAFAF,0x6262,0xD462,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x7F7F,0x4242,0x5251,0x5251,0x4142,0x7F7F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x708E,0xAFAF,0xAFAF,0x5060,0x5150,0x4251,0x4251,0x6F6F,0x7F7F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x8E70,0xAFAF,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x9F9F,0xBFBF,0xD0D0,0xAFAF,0xFCF0,0xEFEF,0xFFFF,0xEFEF,0xDFDF,0xEFEF,0xFFFF,0x9F9F,0x9F9F,0x8F8F,0xAFAF,0xEBEB,0xDBEA,0xEAEA,0xEBF9,0xEAEA,0xEAEB,0xEFEF,0xC3F3,0xCFCF,0xCFCF,0xBFBF,0xD0D0,0xE0E0,0xE0E0,0x92E0,0x9282,0xB2A1,0x92A1,0x7292,0x8282,0x7282,0x8182,0x9282,0x92A1,0x9192,0x9282,0x8292,0xC1C2,0xB1C1,0xC1C1,0xD1B1,0xD181,0xCFCF,0xCFCF,0xCFCF,0xE0E0,0xE1F0,0xD1D1,0xB1C1,0xB1C1,0xB1B1,0xB1B1,0xA1B1,0xA1B1,0xB1B1,0xB1C1,0x92A1,0x92A1,0x7F7F,0x7F7F,0x6F6F,0xFFFF,0xEFEF,0x6F6F
.word 0x7F7F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9040,0x8281,0xAEC1,0xBFBF,0xB3B2,0xB292,0xB2B3,0x92B3,0x8292,0xCFCF,0xD0DE,0xC1C1,0xF0D1,0x72C1,0x9393,0x8393,0x8383,0x7383,0x8362,0x7373,0x8352,0x83A3,0x7272,0x7272,0x6262,0xC462,0x7161,0xAFAF,0x6171,0xAFAF,0x6262,0xAFAF,0x6162,0xAFAF,0xAFAF,0x9F9F,0x4241,0x4251,0x5152,0x4252,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9E8E,0x709E,0x9F9F,0xAFAF,0x9E70,0x9E70,0xAFAF,0x8F8F,0x5151,0x4141,0x4241,0x6F6F,0x8F8F,0x9F9F,0xAFAF,0x709E,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0xAFAF,0xD080,0xBFBF,0xBFBF,0xF292,0xEFEF,0xFFFF,0xEFEF,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0xCBBB,0xBCCC,0xAFAF,0xCBCC,0xDCCC,0xDFDF,0xDCF7,0xBFBF,0xAFAF,0xCFCF,0xD0D0,0x81C0,0x81C3,0xC0C3,0xD0C1,0xE0E3,0x82C1,0xB292,0x92A1,0x8192,0x7172,0x8192,0x8272,0x8282,0xA1B2,0x8282,0x9292,0x8292,0xC1C2,0xB1B2,0xB1B1,0xC1C1,0xE0C2,0xCFCF,0xCFCF,0xF0E0,0xDFDF,0xEFEF,0xC1D1,0xB1C1,0xB2B1,0x92A1,0x9292,0xB292,0xC2B1,0x92B2,0xA192,0x9292,0xA192,0x7F7F,0x7F7F,0x4342,0xD0C1,0x9F9F,0x7F6F
.word 0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7E40,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x8270,0x9181,0x9E9E,0x92A0,0xB292,0x92B3,0x92B3,0xB2B3,0x92B3,0xE0B1,0xCFCF,0xD1C1,0xDFDF,0x72B1,0x9393,0x9393,0x8383,0x8383,0x8373,0x6273,0x8362,0x72A3,0x7272,0x7272,0xC471,0x6162,0xC462,0x6262,0xAFAF,0x7171,0x7171,0xAFAF,0xAFAF,0x7061,0x9F9F,0x6F6F,0x4142,0x5252,0x5251,0x4241,0x8F8F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8E80,0x9E9E,0x9E9E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x5261,0x5251,0x4241,0x6F6F,0x8E40,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x708E,0xAFAF,0x4141,0x7F7F,0x7F7F,0x8040,0xAFAF,0xAFAF,0xC381,0xE0E0,0xB2D3,0xF0F2,0xDFDF,0xEFEF,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x9BCB,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xEBCC,0x9CEC,0x7F7F,0xCFCF,0xCFCF,0xBFBF,0xB282,0xB291,0xB281,0xB282,0xC1A1,0xD1D1,0x92B1,0x9192,0x8182,0x7272,0x7292,0x8272,0x9281,0xB2B2,0x8281,0x9281,0x8281,0xB1B2,0xB1A1,0xB1C1,0xB1B1,0xE0C1,0xCFCF,0xCFCF,0xDFDF,0xF0F0,0xD1E0,0xC1D1,0xB2B1,0x9292,0x9292,0xB2A1,0xC1B1,0xA1B1,0x8181,0x9272,0xA192,0xB292,0x71C2,0xAFAF,0xAFAF,0x81C2,0x8181,0xC4AF
.word 0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x8181,0xB081,0xBFBF,0xB291,0xB3B2,0xB392,0xB3B2,0xB392,0x8292,0xD0B1,0xD0DE,0xE0C1,0xE0EE,0x9392,0x9393,0x9383,0x8393,0x8383,0x6283,0x6273,0x8362,0x72A3,0x8372,0x8372,0x7172,0x7172,0x61D4,0xAFAF,0x6262,0x6262,0xAFAF,0x6261,0xAFAF,0x9F9F,0x9F9F,0x7F7F,0x5242,0x5152,0x4152,0x4152,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9E9E,0x8E9E,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x4140,0x5151,0x4141,0x4142,0x6F6F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8E70,0xAFAF,0x8F8F,0x6F6F,0xA383,0x4272,0x9F9F,0xAFAF,0xC0C0,0x72C3,0xC1D1,0xBFBF,0xC272,0xEAE0,0xCFCF,0xCFCF,0xDFDF,0x9F9F,0xFEFE,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0xCFCF,0xBFBF,0xB291,0x71C1,0xB1B1,0x92B2,0x91B2,0x9292,0xB192,0xD1D1,0x9292,0x7282,0x8271,0x7182,0x8172,0x8272,0xB1B1,0x9282,0x8282,0x8282,0xC1B2,0x9292,0xB1B1,0xB1C1,0xDEC1,0xE0D0,0xDFDF,0xF0F0,0xE1E0,0xC1D1,0xB2B1,0xB2B2,0x9292,0xC1B2,0xC1C1,0xE1E0,0xAFAF,0x7F7F,0x6F6F,0x92B2,0xA192,0xD092,0xBFBF,0xBFBF,0x8181,0xB3C3,0xF2D0
.word 0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8181,0xAE92,0x90AE,0x8291,0x92B3,0x92B3,0x92B3,0x92B3,0x9282,0xD0A1,0xCFCF,0xE0C1,0xDFDF,0x9381,0x9393,0x9393,0x9383,0x8383,0x8362,0x6283,0x9362,0x93B3,0x7272,0x7282,0xC472,0xC471,0x7161,0x7271,0xAFAF,0xAFAF,0x6262,0xAFAF,0x6162,0xAFAF,0x6161,0x6F6F,0x5142,0x5241,0x5251,0x4251,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9E70,0x9E9E,0xAFAF,0x709E,0x9F9F,0xAFAF,0xAFAF,0x4040,0x5151,0x5142,0x4141,0x6F6F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x7F7F,0xA352,0x4273,0x9F9F,0xC080,0xAFAF,0x71C3,0x8282,0xD1B2,0x61D3,0xD2B3,0xCFCF,0xDFDF,0xF7F7,0x9F9F,0xFEBE,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x7F7F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0xCFCF,0xBFBF,0x91C0,0x9F9F,0x7F7F,0x7F7F,0xB281,0xB1B2,0xB2C2,0x8292,0xC191,0x82B1,0x8182,0x7272,0x7282,0x7272,0x7282,0xB1C1,0x8282,0x8282,0x8182,0xB192,0x9292,0xB1B1,0xB1C1,0xE0C1,0xDFDF,0xF0F0,0xE0F0,0xD1D1,0xB1C1,0xB2B1,0x92A1,0xC192,0xC1D0,0xC1B1,0xD0E0,0xBFBF,0x7F7F,0xC1C2,0xB292,0x9292,0xA182,0x70C0,0xC3C0,0xC381,0x8171,0xC281
.word 0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8281,0x9EA1,0xAFAF,0xB2B2,0xB392,0xB292,0x92B3,0xB3B2,0x9292,0xD1B2,0xC1D0,0xEED1,0xDFDF,0x9382,0xA393,0x9393,0x8393,0x8383,0x8383,0x6262,0xA373,0x7293,0x7293,0x8293,0x7272,0x7171,0xAFAF,0xAFAF,0x7172,0x7171,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x7F7F,0x4251,0x5251,0x5251,0x6F6F,0x9F9F,0x9F9F,0x7060,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x9E9E,0x8E9E,0x8E70,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x5151,0x4251,0x6F6F,0x8F8F,0xAFAF,0xAFAF,0x9E70,0xAFAF,0x9E70,0xAFAF,0x60C0,0x8F8F,0x6F6F,0x6252,0x4141,0xC041,0xAFAF,0xC371,0xC2C3,0xC2C3,0xC2C2,0xEFEF,0xA3D2,0xD1C2,0xDFDF,0xF7F5,0xBFBF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0xBFBF,0x8F8F,0x7F7F,0x9F9F,0xCFCF,0xBFBF,0x71B2,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xF0F0,0xE0E0,0x92C1,0x8182,0x8282,0x7282,0x7172,0x7172,0x7272,0x7281,0xB1C1,0x8182,0x8182,0x8282,0xB1B2,0x9292,0xB1B2,0xB1B1,0xF0D1,0xE0F0,0xE0F0,0xD1D1,0xB1C1,0xC2B1,0xC2B1,0xB2C2,0xD0B1,0xB1B2,0x9292,0xD1B2,0x81D1,0xD0C3,0xD1E1,0xB1B2,0x92A1,0x9292,0xE3D2,0xC282,0xC371,0x82C0,0x8F71
.word 0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8160,0xA081,0xCFCF,0x92A0,0x92B3,0x92B3,0xB3B3,0xB392,0x82B3,0x9292,0xC1A1,0xD1D0,0xEED0,0x82D0,0xA372,0x9393,0x8393,0x8393,0x8383,0x8383,0x6273,0xB372,0x72A3,0x8272,0xC371,0x7272,0x72C3,0x7271,0x7272,0xAFAF,0xAFAF,0x6161,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x4241,0x5251,0x5251,0x4251,0x7F7F,0x8F8F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9E9E,0xAFAF,0x708E,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x5151,0x5152,0x7F7F,0x6F6F,0x708E,0xAFAF,0xAFAF,0x708E,0xAFAF,0xBFBF,0xBFBF,0x4060,0x5141,0x5252,0x4252,0xC051,0xAFAF,0x71C3,0xC381,0xD3C0,0xD3D0,0xD1C2,0xF2F2,0xB3D3,0xDFDF,0xF6F7,0xF6E6,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x7F7F,0x7F7F,0x9F9F,0xBFBF,0xC1C0,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xDFDF,0xCFCF,0xD1D0,0xD1D1,0x82A1,0x7272,0x7281,0x7172,0x7262,0x7271,0x7272,0xA1C2,0x8282,0x8282,0x8281,0xB192,0x9292,0xB1B2,0xB1B1,0xF0E1,0xE1E0,0xD1D1,0xC1C1,0xB1B1,0xC2C1,0xD1D1,0xB2D1,0xB1D1,0x9292,0x8192,0xF592,0xFFFF,0xBFBF,0xC1D1,0xB1B2,0xA1B2,0x8292,0xFFFF,0xCFCF,0x71C0,0x7172,0x7F8F
.word 0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8170,0xA181,0x9E9E,0x9281,0x92B2,0xB3B2,0x92B2,0xB3B2,0x92B2,0x9292,0xB1A1,0xC1C1,0xEEE0,0x82B1,0xA3A3,0x9393,0x9393,0x9393,0x8393,0x8383,0x6283,0x8272,0x72A3,0x8282,0x8172,0x7272,0x7171,0x71D4,0x71C3,0x7171,0x71D4,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x4161,0x4142,0x5142,0x5152,0x5152,0x7F7F,0x8E50,0xAFAF,0xAFAF,0xAFAF,0x708E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x709E,0xAFAF,0x4150,0x4241,0x5142,0x4161,0x6F6F,0x6F6F,0x8E70,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6F6F,0x5252,0x6252,0x4142,0x9F9F,0x7181,0xC0D4,0xB1D3,0xC2C2,0xD3C2,0xC2D3,0xF1D1,0xF2F0,0xAFAF,0xAFAF,0xDFDF,0xDFDF,0xE0F0,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0xDFDF,0xC1F0,0x50C2,0x7F7F,0x7F7F,0x6F6F,0xAFAF,0xE0E0,0xBFBF,0x8181,0x9182,0x92B2,0x8281,0x9282,0x8292,0x7172,0x7162,0x7172,0xB1B1,0x9192,0x8292,0x8272,0xB291,0x9292,0xB2A1,0xD1B1,0xE1E1,0xE1E1,0xC1D1,0xB1C1,0xC2C2,0xC1C1,0xD1C2,0xC2B2,0x92E0,0x9292,0x82A1,0x7282,0xBFBF,0xB1C0,0xD1D1,0xB1B2,0xB2B1,0x4171,0xEBC4,0x8F8F,0xAFAF,0xC371,0x7F8F
.word 0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x72B0,0xD081,0x80AE,0xB2B0,0xB3B3,0xB392,0xB392,0x92A3,0xA3B3,0x9292,0xC192,0xD1C1,0xCFCF,0x81A1,0x9393,0xA3A3,0x9393,0x8393,0x9383,0x8383,0x6283,0xB393,0x8282,0x8282,0xB372,0xC371,0xC372,0x7161,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7171,0xAFAF,0x6170,0x8F8F,0x4241,0x5251,0x5241,0x4251,0x6F6F,0x9F9F,0x8E70,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9E70,0x8E9E,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x4241,0x5141,0x5152,0x6F6F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0x7F7F,0x5242,0x5152,0x4242,0x7161,0xC371,0xC371,0xC3C0,0xC1C2,0xC1C2,0xD1C1,0xD1D2,0xDFDF,0xC3F0,0x8F8F,0xAFAF,0xCFCF,0xFFFF,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x7F7F,0xDFDF,0xFFFF,0x7F7F,0x7F7F,0x4141,0x4141,0xD151,0xCFCF,0xD1D0,0xD1D1,0xC1D1,0xB1C1,0xF1D2,0xF0F0,0xF0F0,0xE1E1,0x92D1,0x7172,0xB1B2,0x9292,0x9192,0x7282,0xA192,0x9292,0xB292,0xC1B1,0xD1C1,0xD1D1,0xC1C1,0xC1C2,0xE2E2,0xC2C1,0xC2C2,0xB2B2,0xB1C1,0xA182,0x8281,0xD181,0xE0E0,0xE1E1,0xC1E0,0xB1B2,0xB1B1,0x9F9F,0x6F6F,0x6F6F,0x71C0,0x71C3,0x7F8F
.word 0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8181,0x9E92,0xB09E,0x9292,0xB392,0xB3B2,0x92B3,0xB3B3,0x9292,0x9292,0xB192,0xD0C1,0xD0DE,0xA391,0xA393,0x93A3,0x9393,0x9393,0x9393,0x8393,0x7262,0xB393,0x7282,0x7282,0xB381,0xC371,0x7171,0x72D3,0xC381,0xAFAF,0x7171,0x7171,0xAFAF,0xAFAF,0xAFAF,0x7F7F,0x5241,0x5151,0x5152,0x4151,0x7F7F,0x9F9F,0x6070,0xAFAF,0x708E,0x708E,0xAFAF,0xAFAF,0x9F9F,0x9E9E,0x708E,0x708E,0x9F9F,0x4140,0x5151,0x5142,0x5251,0x5F5F,0xAFAF,0xAFAF,0x7070,0xAFAF,0x808E,0xAFAF,0xAFAF,0xBFBF,0x9F9F,0x5142,0x5252,0x4141,0xBFBF,0xC372,0xC371,0xC371,0xC2C3,0xC1D3,0xD2C2,0xD2D2,0xD1D1,0xF0E1,0xCFCF,0x9F9F,0x9F9F,0xCFCF,0xCFCF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x6F6F,0xAFAF,0xC3C0,0xD3C0,0xD0D0,0xD1D0,0xD0E0,0xD0D1,0xD1D1,0xEFEF,0xF0F0,0xE0E0,0xD1D1,0xE0D1,0xF1E0,0xC2D1,0xC1B1,0x92B1,0x9291,0x7281,0xB292,0x9292,0xB2B2,0xB1B2,0xC1C1,0xC1C1,0xC2B1,0xD2C1,0xEFEF,0xD1E1,0xB2D1,0xC2C2,0x92B2,0x8192,0xC292,0xF0E0,0xD1D1,0xD1C1,0xD1E1,0xB1C1,0xB1B1,0xE0C2,0xBFBF,0x8F8F,0x71C3,0xAFAF,0x7F9F
.word 0x6070,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8181,0xAEA1,0x819E,0xB392,0x92B2,0x92B3,0xB3B2,0x9292,0x82B3,0x9292,0xB1A1,0xE0C1,0xD0DE,0x8282,0x9393,0xA3A3,0x93A3,0x9393,0x9383,0x8383,0x8383,0xB382,0x82A3,0xC382,0xC371,0x7271,0xC372,0x7171,0x61C3,0x7271,0xD4C0,0xAFAF,0xD471,0x7171,0x4171,0x4142,0x5142,0x5252,0x5251,0x4251,0x7F7F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x7F7F,0x5152,0x4251,0x5151,0x7F7F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6F6F,0x5241,0x5261,0x8F8F,0xD371,0xC072,0xC071,0xC3C3,0xC3C0,0xC2C3,0xC2C3,0xC2C2,0xD1D2,0xE1D2,0xF0E1,0xDFDF,0x71E3,0x9F9F,0x9E8C,0xF7DB,0xF6F6,0xD7E7,0xBFBF,0x4848,0xBFBF,0x8F8F,0x7F7F,0xE0F3,0xBFBF,0xAFAF,0x7161,0xD3C3,0xC1D3,0xC3B1,0x8181,0xF1B2,0xF0F0,0xD1E0,0xB1C1,0xA1B1,0xB1A1,0xC1B1,0xE0D1,0xC1E1,0xA1A1,0x8292,0x8182,0xB282,0x9292,0xA1B2,0xB292,0xB1B1,0xB1C2,0xC1C2,0xB1C1,0xC1C2,0xE1E1,0xE1F0,0xE1E1,0xB2C2,0xC292,0xDFDF,0xC1D1,0xC1C1,0xE1D1,0xD1F0,0xC1C1,0xB2B1,0xE0C1,0xBFBF,0xCFCF,0xD4C0,0xC3C0,0x6171
.word 0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8040,0x8171,0x9EC0,0x9190,0x9282,0xB382,0x8292,0x92B3,0xB3B3,0x9292,0x9292,0xB192,0xCFCF,0xC0DE,0x8292,0xA393,0xA393,0x93A3,0x9393,0x8393,0x9393,0x6262,0xB3A3,0x82A3,0x8182,0x71C3,0xC371,0x71C3,0xD471,0x7171,0x71D4,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x4142,0x5152,0x5152,0x5151,0x5142,0x7F7F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x708E,0xAFAF,0x9F9F,0xAFAF,0x8E9E,0xAFAF,0x8E70,0x8F8F,0x5141,0x5142,0x4152,0x5152,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xC0C0,0x9F9F,0x6252,0x4152,0x4142,0x71D4,0x71D0,0xD3D3,0xC071,0xC3C3,0xC3C0,0xC3C3,0xC3C2,0xC2B2,0xD1D1,0xD1C1,0xF0E0,0xFFFF,0xE0F3,0xAFAF,0xAFAF,0xBC8D,0xDACB,0xE8F9,0xD5E7,0xD7D7,0x9F9F,0xBFBF,0xFFFF,0x8291,0x8272,0x8182,0x9181,0x9281,0x8182,0xC271,0xDFDF,0xD1E0,0xB1C1,0xB2B2,0xA192,0xA192,0xA1A1,0xB1A1,0xC1D1,0x9292,0x8292,0x7282,0xB182,0x9282,0x9292,0x9292,0xB292,0xC2B2,0xC1C1,0xB2C2,0xB2B2,0xC2B1,0xD1C1,0xEFEF,0xE1F0,0xE0E1,0xC1D1,0xC1C1,0xC1D1,0xE1C1,0xD1F0,0xC1D1,0xB1C1,0xF0B2,0xBFBF,0xBFBF,0xC0C3,0xC0D4,0xE4C3
.word 0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8140,0x8281,0x9EB1,0x8180,0xB392,0x9292,0xB2B3,0xB392,0x92B2,0x92B3,0x9292,0xB1A1,0xCFCF,0xC1DE,0x8292,0xA393,0xA3A3,0xA3A3,0x9393,0x9393,0x8383,0x9383,0xB3B3,0x82A3,0xB3A3,0xC372,0x71C3,0xC371,0x71D4,0xE471,0xD480,0x7171,0x7171,0xD471,0x7171,0xAFAF,0x5241,0x5251,0x5251,0x5251,0x4141,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x708E,0x9F9F,0xAFAF,0xAFAF,0x9E70,0xAFAF,0x8F8F,0x7F7F,0x5151,0x5251,0x4251,0x7F7F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x4270,0x5251,0x5261,0x5241,0xC0C0,0xC3C3,0xAFAF,0xD4D0,0xD0C0,0xC0D3,0xC3C0,0xC3C1,0xD1C1,0xE0E0,0xD1D0,0xD1D1,0xE1D1,0xF1F0,0xEFEF,0xF0F0,0x9F9F,0x8F8F,0x7E4E,0xDB8D,0xDAD6,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0xD341,0x81C1,0xA1B2,0x7281,0xEFEF,0xDFDF,0xB1D1,0xB2B2,0xB1B1,0xB1C1,0x92B1,0x9292,0x9292,0xD1B1,0x9292,0x8282,0x8281,0x9282,0x9292,0x9292,0x9292,0x9292,0xC1B2,0xB2C1,0xB1B2,0xC2C1,0xC2C2,0xD2C2,0xEFEF,0xD1D1,0xE1E1,0xC1E1,0xD1C1,0xB1C1,0xC2B1,0xD1F0,0xC1C1,0xB2B1,0xE1B2,0xE0E0,0xBFBF,0xD0C1,0xC3D0,0xD582
.word 0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8140,0x9281,0xAEC1,0xB2B0,0xB2B2,0xB2B3,0x92B3,0x92B3,0xB3B3,0x9292,0x9292,0xB192,0xDED1,0xCFCF,0x8282,0xA3A3,0xA3A3,0xA3A3,0x93A3,0x9393,0x7293,0xA372,0xB3B3,0x8282,0x8282,0x8282,0xC372,0x7171,0xD371,0xC071,0xC061,0x71D4,0xBFBF,0xAFAF,0xAFAF,0x4262,0x5241,0x5251,0x5151,0x5152,0x4142,0x8F8F,0xAFAF,0xAFAF,0x7070,0x708E,0x708E,0xAFAF,0x9F9F,0xAFAF,0x9E9E,0xAFAF,0xAFAF,0xAFAF,0x6F6F,0x5252,0x5151,0x4152,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xD080,0x8F8F,0x6152,0x6152,0xBFBF,0xD4C0,0xC080,0xC0C3,0xC0D3,0xC0D4,0xC3C3,0xC1C2,0xD0C1,0xE0D0,0xCFCF,0xE0E0,0xD1E1,0xE1D1,0xE1D1,0xF1E1,0xF0F1,0xDFDF,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0xCFCF,0xBFBF,0x8F8F,0xAFAF,0xCFCF,0xD0D0,0xB2B2,0xC1C1,0x92B2,0xB292,0xB1B1,0xA1A1,0x9292,0xD1B2,0x8291,0x8282,0x8282,0x8282,0x8282,0x8182,0x8282,0x9281,0xC1B2,0xB2B1,0xB2B2,0xC2C1,0xB2B1,0xC2C1,0xC2D2,0xC1C1,0xD1D1,0xE1D1,0xB1C1,0x92B1,0xB292,0xC1E0,0xB2C1,0x92B1,0xC192,0xC1C2,0xD3D0,0xC3C2,0xD0D0,0xD5C3
.word 0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9170,0x8192,0x9ED0,0x8291,0xB392,0xB392,0xB392,0xB3B2,0x9282,0x92B3,0x92B2,0xC1B1,0xDED0,0x92C0,0xA382,0x9393,0xA3A3,0xA3A3,0xA3A3,0x9393,0x8383,0xB293,0xA3B3,0xA3A3,0xB3A3,0xC382,0xB382,0xC3C3,0x71C0,0xAFAF,0x71D4,0xC370,0xAFAF,0x71C0,0x71C0,0x4241,0x5251,0x5152,0x5152,0x5152,0x7F7F,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x8E70,0x8E70,0xAFAF,0x8E70,0xAFAF,0x6F6F,0x5142,0x5152,0x5262,0x4151,0x7F7F,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0x4251,0x6261,0x5261,0xC051,0xAFAF,0xAFAF,0xC3C0,0xD3C0,0xC3C0,0xC2C0,0xC2C2,0xD2C2,0xD2E3,0xE0E0,0xE0F0,0xE1E1,0xF1F1,0xE1F1,0xE1E1,0xF0F1,0xDFDF,0xEFEF,0xCFCF,0xEFEF,0x9F9F,0x9F9F,0x8F8F,0x6F6F,0x6F6F,0xC141,0xCFCF,0xB291,0x7F7F,0xF062,0xCFCF,0xB2C1,0xC2B1,0xC1C1,0xC1C1,0xB1C2,0xC1C2,0xB1D1,0x91B2,0xD192,0x9292,0x8281,0x8172,0x9272,0x8192,0x8282,0x8281,0x9282,0xB1B2,0xB2B2,0xC2B2,0xB1C2,0x92C2,0xC1C2,0xC1C2,0xC2C2,0xB1B1,0xC1C1,0xC1E1,0xB292,0xB2A1,0xC2D1,0xC2C2,0x92B2,0x9292,0x8282,0xD082,0xC0C2,0xD0C3,0xD5B3
.word 0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0xB270,0xB182,0xC08E,0xB291,0xB2B3,0xB292,0xB2B3,0x92B3,0xB2B3,0x9292,0x9292,0xC1B2,0xCFCF,0x9291,0x9382,0xA3A3,0xB3A3,0xA3A3,0x93A3,0x93A3,0x7293,0xB382,0x92B3,0x8282,0x9282,0xB3B3,0xC381,0xC372,0xC471,0x72C0,0xC081,0xC0D4,0xC371,0x7171,0xBFBF,0x5142,0x5252,0x5251,0x5251,0x4151,0x7F7F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9E60,0x709E,0xAFAF,0x809E,0x8F8F,0x5251,0x5251,0x5152,0x6F6F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0x5161,0x6262,0x4162,0xBFBF,0xAFAF,0xC0C0,0xC0D4,0xBFBF,0xC1C3,0xC3C3,0xD3C2,0xD3C2,0xD0C1,0xD1E3,0xE1E1,0xF1E1,0xF1E1,0xF1F1,0xF1F1,0xEFEF,0xEFEF,0xDFDF,0xFEFE,0xBFBF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0xD0D3,0x8181,0x8F8F,0x7F7F,0xDFDF,0xC1E0,0xC2B1,0xB1C1,0xA1B1,0x92A1,0x9292,0x9292,0xD1D1,0xB1C1,0xD1B1,0x8192,0x7282,0x7281,0x8272,0x8281,0x7282,0x7282,0x8282,0xB2B2,0x92B2,0xC292,0xB1B1,0x92B2,0xC1C1,0xF2E2,0xB1E1,0xB2B2,0xC1B1,0xD1C1,0x92C2,0x9292,0xD2C2,0xE2D2,0x92C2,0x8292,0x8282,0xB382,0xC3B1,0xC3C1,0xE3B2
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8170,0xCFCF,0xA0AE,0xB3B2,0x92B2,0xB2B3,0xB392,0xB392,0x9292,0x92B3,0x92B2,0xD0A1,0xCFCF,0x9292,0xA3A3,0xA3A3,0xA3A3,0xA3A3,0xA3A3,0x9393,0x7293,0xB292,0xA3B2,0x9282,0x9282,0x8282,0x82C3,0x71C3,0x71C3,0xC0D4,0x71C3,0xAFAF,0xBFBF,0xD471,0x52C0,0x4241,0x5251,0x5261,0x5251,0x4251,0x7F7F,0xAFAF,0x7070,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x8E70,0xAFAF,0xAFAF,0x9E9E,0xAFAF,0xAFAF,0x7F7F,0x5152,0x6152,0x5251,0x6F6F,0xAFAF,0xAFAF,0x9E70,0xBFBF,0xC09E,0xAFAF,0x9EC0,0xC0C0,0xBFBF,0x6250,0x6161,0x5161,0xC0D0,0xAFAF,0xC0C3,0xBFBF,0xD4D0,0xD0D0,0xD0D0,0xC0C1,0xC0C3,0xD3D0,0xD1C1,0xD1D2,0xE1D1,0xE1E1,0xE1E1,0xF1F1,0xF1F1,0xF1F1,0xF0F0,0xF0F0,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0xBFBF,0xFFFF,0x9F9F,0x6F6F,0x6F6F,0xDFDF,0xC1C1,0xA1B1,0x92B2,0x8292,0x7282,0x9282,0xB2B2,0xC1B1,0x8281,0xD1C1,0x8292,0x8172,0x7272,0x8172,0x7282,0x7281,0x7281,0x8281,0xB2A1,0x92A1,0xB1B2,0xB2B2,0xB292,0xC1D1,0xF3E2,0xC2D1,0xB2B2,0xC2B1,0xC1B1,0xA1D1,0xB192,0xD2C2,0xC2D2,0x92B2,0xB292,0x92B2,0x8282,0xB292,0xC0C1,0xD192
.word 0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9281,0xAEA1,0x81C0,0x82B2,0xB3B2,0xB392,0xB3B2,0xB392,0x92B2,0x9292,0xA192,0xD0B1,0xA1B1,0xA392,0xA382,0xA3A3,0xA3A3,0xB3B3,0xA3A3,0xA393,0x8272,0xB2A3,0x92B2,0x8282,0xB3B2,0x82C3,0x81C3,0xC3C3,0x71C3,0x71C3,0xD0D4,0xC0C3,0xB0C3,0x71C0,0xAFAF,0x5241,0x5152,0x6152,0x5152,0x4152,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x509E,0x7F7F,0x5251,0x5261,0x5152,0x6F6F,0xBFBF,0x9E70,0x80C0,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0x9F9F,0x6251,0x6262,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xD0D0,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xD0D0,0xBFBF,0xBFBF,0xE0E0,0xDFDF,0xFFFF,0xF1F1,0xF0F0,0xCFCF,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x6252,0xF061,0xD1E0,0xB2B1,0x92B2,0x9282,0x8282,0xC1B2,0xE1D1,0xE1E1,0xD1E1,0x72C2,0xC172,0x8292,0x7272,0x7172,0x8272,0x8172,0x8272,0x8272,0x9272,0xB292,0x9292,0xB292,0x92B2,0xF2D2,0xB1E1,0x92B2,0xB292,0x92B1,0xB1B2,0xC1C2,0xC1C1,0xB281,0xC2C2,0xB2B1,0x8292,0xC2B2,0xB2B1,0x8292,0xB282,0xB3C3,0xD192
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x4040,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x9150,0x8281,0x9ED0,0x82B0,0xB2B2,0xB2B3,0x92B2,0xB2B3,0x92B3,0xB392,0xB292,0x9292,0xB1B1,0x9291,0x8282,0xA3A3,0xB382,0xB3B3,0xA3A3,0xA3A3,0x82A3,0x82A3,0xB2B3,0x92B2,0xB292,0xC381,0xC381,0xC381,0xC371,0xD371,0xD071,0xAFAF,0xC3C0,0x7171,0xC072,0x5271,0x5152,0x6152,0x5252,0x5251,0x4151,0x7F7F,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x8E70,0xAFAF,0x8E9E,0x9E70,0x9F9F,0x5241,0x5261,0x6252,0x5251,0x6F6F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAE80,0xBFBF,0xBFBF,0xBFBF,0x6151,0x6171,0xBFBF,0xC0C0,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xAFAF,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0xE2F5,0xE0F0,0xC1D1,0xA1B2,0x9292,0xB2B2,0xC1B1,0xD1D1,0xC1D1,0xC1C1,0xB1B1,0xC1C1,0xB192,0x8292,0x8182,0x7272,0x8172,0x7272,0x7272,0x7272,0x9272,0x9292,0x9292,0xB292,0xC1B2,0xF3D2,0xC1E2,0x92B2,0xB292,0xB2B2,0xB292,0xB1B1,0xC1C2,0xB2D1,0xC2B1,0xB2B2,0x9292,0xB2B2,0xB2B2,0x8282,0xB282,0xB2B2,0xD282
.word 0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8170,0x9181,0xCFCF,0xB291,0xB3B2,0x9292,0xB3B3,0xB392,0xB292,0x92B3,0x9292,0xB1A1,0x91A1,0x8292,0xA382,0x8282,0xA3B3,0x82A3,0xA3B3,0xA3B3,0xA3A3,0x9282,0xB292,0x92B2,0xB282,0x81C2,0x81C3,0x72C3,0xC3C0,0xC3C0,0xC4C0,0xD3C0,0xAFAF,0xD4C0,0xAFAF,0x4162,0x5251,0x5262,0x6152,0x5152,0x4142,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8E70,0xAFAF,0xAFAF,0x9F9F,0x9E70,0x709E,0xAFAF,0x8F8F,0x5251,0x6152,0x6152,0x4152,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0x60D0,0x7161,0x6052,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x4050,0x4050,0x4050,0x4040,0x4040,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xE1E1,0xB2C1,0xB292,0xC2B2,0xC1B1,0xD1D1,0xB1C1,0x92A1,0x8282,0x8282,0x9282,0xB292,0x8292,0x7271,0x7271,0x7271,0x7272,0x7281,0x7281,0x9281,0x9292,0x8282,0xB2B2,0xB2B1,0xB292,0xC292,0xB2D1,0xB292,0xB292,0xB292,0xC2B2,0xC2B2,0xC1E0,0xB1C2,0xB1C2,0x8292,0xC2B2,0x92B2,0x8282,0xB2B3,0x82B2,0xC2B2
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8180,0xB292,0xAFAF,0xB2A0,0x92B3,0xB2B3,0xB292,0xB2B3,0xB3B2,0xB292,0x9292,0xA192,0x9292,0x8292,0x8282,0x82A3,0x92A3,0xB3B3,0xA382,0x8282,0x8292,0x9282,0xB292,0x9292,0x82B2,0xC281,0xC381,0xC3C0,0x71C3,0x71D4,0x80D4,0xBFBF,0xC3D0,0x7171,0xAFAF,0x5252,0x6252,0x5251,0x5261,0x5261,0x4251,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0x7070,0x708E,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x709E,0xAFAF,0x9F9F,0x6160,0x5252,0x5261,0x4152,0xAFAF,0xAFAF,0xAFAF,0x9E70,0x9E80,0xBFBF,0x9E70,0xAFAF,0xAFAF,0x60C0,0x6262,0xBFBF,0xBFBF,0xBFBF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0xE262,0xD1F1,0xB2B1,0xC2B1,0xB1B1,0xC1B1,0x92B1,0x8292,0x8282,0xC292,0xC1D1,0x8192,0x8282,0x7182,0x7272,0x7172,0x7272,0x7271,0x7272,0x7272,0x9282,0x9292,0x9292,0xB2B2,0xC2B2,0xB2C2,0x92B2,0xC292,0xB2C2,0xB292,0xB2B2,0xB1B2,0xB2B2,0xC192,0xC1C1,0x92B1,0x9292,0xB2B2,0x82B2,0xB372,0xC2B3,0x8282,0xD3B2
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8181,0xC182,0x909E,0xB281,0xB392,0xB3B2,0x92B3,0xB392,0x9292,0x9292,0x9292,0x9291,0x9292,0x8282,0x92A3,0xB3A3,0xA3B3,0xA382,0x82B3,0x82A3,0x8282,0x9282,0xB292,0x92B2,0xB1B2,0xC3B2,0xC3C0,0xC071,0xC0C3,0xD0C0,0xD371,0xD4C0,0xAFAF,0xC0D4,0x62C0,0x6151,0x5262,0x6162,0x6152,0x5152,0x5141,0xAFAF,0x7070,0x8E70,0xAFAF,0xAFAF,0xAFAF,0x8E70,0x8E70,0x7070,0x8E70,0xAFAF,0x8E70,0x9F9F,0x5161,0x5261,0x6152,0x6F6F,0xAFAF,0xAFAF,0x9E70,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x5171,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x5050,0x5050,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xFFFF,0xFFFF,0xB1C2,0xB2B1,0xA192,0x92B2,0x8282,0x9292,0xC192,0xEFEF,0xEFEF,0xDFDF,0x7172,0x9272,0xE1D1,0xE1E1,0x82C2,0x7162,0x7172,0x7172,0x8272,0xC1B2,0xE1E1,0xE1E1,0xF1E1,0xC1D1,0xB2B2,0x92B2,0xB2B2,0x8292,0xB282,0xB2B2,0xC1B2,0xE0E1,0xE0E0,0xC1F0,0x9282,0xB2B2,0x8282,0xC382,0xB2C3,0xB2B2,0xC2B2
.word 0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8281,0xB181,0xB09E,0xB392,0x92B3,0x92B3,0x92B2,0x92B3,0xB392,0xB2B2,0x9292,0x9192,0x9292,0x9292,0xA382,0x8282,0x8282,0x92B3,0x82A3,0x9282,0x9282,0x9282,0x9282,0xB2B2,0xB2B2,0x81B2,0xB1B2,0xD3C3,0xC371,0xAFAF,0xBFBF,0xAFAF,0xC071,0xBFBF,0x6171,0x6252,0x6152,0x5252,0x5262,0x5251,0x8051,0x7080,0xBFBF,0xAFAF,0xAFAF,0x8E70,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x608E,0xAFAF,0x5261,0x6152,0x4252,0x6F6F,0xBFBF,0xBFBF,0x709E,0xAFAF,0x9E70,0xAFAF,0xAFAF,0xAFAF,0x708E,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xAFAF,0xFFFF,0xEFEF,0xB1B1,0x92A1,0x9282,0x8292,0x9292,0x8182,0xF172,0xEFEF,0xE1E0,0xE1E1,0x72D1,0xE1C2,0xE0E1,0xF0E0,0xE1F0,0x82C2,0x8282,0x7272,0x8272,0xD1B1,0xD1D1,0xE1D1,0xE1E1,0xD1C1,0xB1C1,0x92B2,0xB2B2,0x9292,0xB292,0x92B1,0xE0C1,0xE1E0,0xD0F0,0xF0E0,0xB2C1,0x92B2,0x8282,0xC3B3,0xC3C3,0xC282,0xB2E3
.word 0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9171,0xC181,0x809E,0xB282,0xB392,0x92B2,0xB3B3,0xB3B2,0x92B2,0x9292,0x9292,0x9292,0x9282,0xA3B3,0x82B3,0x8282,0x82A3,0xA3B3,0x82A3,0x8292,0x9292,0x8292,0xB292,0xB2B2,0x9292,0xA1B2,0x82C3,0xC3C0,0xC3C0,0xC3C0,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0x5272,0x5262,0x6262,0x6162,0x5162,0x6152,0xAFAF,0xAFAF,0xAFAF,0x7070,0x708E,0xAFAF,0x8E70,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x708E,0x9F9F,0x9F9F,0x5271,0x5261,0x4142,0xAFAF,0x9EC0,0x9E70,0xAFAF,0xAFAF,0xAFAF,0x8E70,0xAFAF,0xAFAF,0x7070,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0xDFDF,0xD2F0,0xF2D2,0xEFEF,0x92B2,0x9292,0xC192,0x82B1,0x7282,0x7281,0xF081,0xDFDF,0xE0F0,0xF0F0,0xE1F0,0xD0E0,0xD1D1,0xE0D1,0xF0E0,0xE1E0,0xC1D1,0xE1D1,0xD2D1,0xB192,0xC1C1,0xD1C1,0xE1D1,0xC2C1,0xD1C1,0xB2D1,0x92B2,0x9282,0xB1C2,0x92B2,0xD1C1,0xEFEF,0xEFEF,0xB1B2,0xC2C1,0x8292,0xC372,0xC3C3,0x82C3,0xE3B2,0xE3D0
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8290,0xB182,0x909E,0xB381,0x92B2,0xB2B3,0x9292,0x9282,0x8292,0xB392,0x9292,0x8292,0x8292,0x9292,0x8282,0x82B3,0xA382,0x8292,0x9292,0x92B3,0x9282,0x9282,0x9292,0xB292,0xB292,0xB2B2,0xB291,0x81C3,0xC0C3,0xD071,0xAFAF,0xBFBF,0xC0D4,0xD471,0x6251,0x6162,0x5262,0x5262,0x6251,0x7171,0x7171,0x80B0,0x8090,0xAFAF,0x7080,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x9F9F,0x6162,0x5252,0x6F6F,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xBFBF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7241,0xC0E0,0xE0E0,0xE1E1,0xD2F1,0xD2C2,0xE1E2,0xB2C1,0x82B1,0x7272,0x8182,0xC182,0xDFDF,0xDFDF,0xEFEF,0xEFEF,0xC1C1,0xC1C1,0xD1D0,0xE0E0,0xE0F0,0xD0E1,0xE0D0,0xDFDF,0xB2E1,0x9292,0xC1B2,0xD1C1,0xD1D1,0xE1E1,0xE1E1,0x82B2,0x8292,0xB2C2,0x92B2,0xB1B1,0xFFFF,0xFFFF,0xB1B1,0xB1B1,0x7282,0xD3B3,0xD3C3,0x8281,0xE3D3,0xE0D3
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7180,0xA181,0xAFAF,0x92B0,0x92B3,0xB392,0x92A3,0x82B3,0xB392,0x9292,0x8292,0x9292,0x92B3,0xB3B3,0x92B3,0x82A3,0xA392,0x8282,0x92B3,0xB392,0x9292,0x8292,0x8292,0x9292,0x92B2,0xB292,0xA1B2,0xC2B1,0xD381,0x71C0,0xD4D0,0xBFBF,0xBFBF,0x71C0,0x6262,0x6262,0x6262,0x6262,0x7162,0xBFBF,0x7271,0x7171,0x70C0,0x8070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x8F8F,0x6170,0x5262,0x4151,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9E70,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0xCFCF,0xDFDF,0xDFDF,0xE6E6,0xE6D5,0xD7E6,0xAFAF,0x9F9F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x4140,0x6F6F,0xE260,0xCFCF,0xBFBF,0xD0B0,0xD1C1,0xE1E1,0xF1F1,0xE1F1,0xB2C1,0x8292,0x7281,0x8282,0x9272,0xE0C1,0xDFDF,0xEFEF,0xF0F0,0x92C1,0x9291,0xC1C1,0xD1C1,0xF0E1,0xF1F0,0xD0E0,0xCFCF,0xDFDF,0x82C2,0x9282,0xC1B2,0xD1D1,0xE1D1,0xE1E0,0x82E1,0xB192,0xB1C2,0x9292,0x9292,0xD2C2,0xC1D1,0xC1C1,0xB1C1,0x8282,0xD3C3,0x71C3,0xD3B3,0xD0F4,0xD2D3
.word 0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7160,0xB081,0x9E9E,0x8272,0x8292,0x92B3,0xB392,0x9292,0x92B3,0x8292,0x9292,0xB392,0xB392,0x9282,0xA382,0xA392,0x92B3,0x92B3,0xB392,0x9292,0x8292,0x8292,0x9292,0x9292,0xB292,0xB2B2,0xB2B2,0xB2B1,0x81C0,0xAFAF,0xAFAF,0x71D0,0x8F8F,0x7F7F,0x4141,0x5151,0x7282,0x7282,0xB282,0x82B2,0x6272,0x6262,0x6262,0x70C0,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x708E,0x9F9F,0x6160,0x6262,0x5261,0x6F6F,0x9F9F,0xAFAF,0x8E70,0x9E70,0xAFAF,0xAFAF,0xAFAF,0x708E,0x5050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x9F9F,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0xF6E6,0xCFCF,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xFFFF,0xD3F1,0xE1C1,0x70E0,0xBFBF,0xC0C0,0xC1D0,0xD1D1,0xF1F1,0xE1F1,0x92C1,0x7282,0x8282,0xA192,0x9292,0xD1A1,0xF0F0,0xDFDF,0xDFDF,0x92D1,0xB292,0xC2A1,0xD1C1,0xD0D1,0xF0E0,0xE1F0,0xD0D0,0xE0D0,0xD1F0,0xB292,0xB1B2,0xE1D1,0xD1E0,0xD1D1,0xB2F0,0xC2B2,0xB1B2,0x9292,0x9282,0xB1B1,0xC1C1,0xC1C1,0xB1C1,0x8272,0xC3C3,0xB3C3,0xCFCF,0xD4E4,0xD1E3
.word 0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6150,0x9071,0xBFBF,0x8290,0xB2A3,0x9282,0x82B3,0xB392,0x9292,0x8282,0xB392,0xB392,0x8292,0x8292,0x92A3,0x8282,0xB392,0x9292,0x92B3,0x9292,0x9282,0x9292,0x9292,0x9292,0xB292,0xB1B2,0xB1B2,0xB292,0xC3C0,0x81C0,0x7181,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8251,0xB2B2,0xB2B2,0xB2B2,0xA3B3,0xB393,0x7272,0x7272,0x6262,0x7070,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x6050,0x6261,0x6172,0x4142,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0xAFAF,0xBFBF,0xEFEF,0xDFDF,0xFFFF,0xDFDF,0x9F9F,0xBFBF,0xD6D6,0xD6E6,0xD0D5,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0xFFFF,0xEFEF,0xFEF0,0xDFDF,0xDFDF,0xAFAF,0xC0C0,0xC1D0,0xC1D1,0xE1D1,0xE1E1,0xB2C1,0xA192,0xB1B2,0x92A1,0x8282,0x7272,0xF0E0,0xF0F0,0xF0F0,0xB1E0,0xC1C1,0x9292,0xC1C1,0xC1D1,0xD0D2,0xE1E0,0xE3E0,0xD0D0,0xDFDF,0x7272,0x8272,0xE1C2,0xC1D1,0xC1C1,0xC2E1,0xB2B2,0xB2B1,0x8292,0x9292,0x92B1,0xB1B1,0xB1C1,0xB2C1,0xB282,0x72C3,0xC382,0xF4F4,0xD3E4,0xF0E4
.word 0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7040,0x7171,0x8E91,0x909E,0x9281,0x92A3,0x9282,0x92A3,0x92A3,0x92B3,0x92B3,0x9292,0x9282,0x92A3,0x8292,0x9292,0x9292,0xB392,0x9292,0x92B3,0x9292,0x9282,0x9282,0x9292,0xB1B2,0xB1B1,0xA1A1,0x81B1,0x7172,0x6272,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xB2B2,0xB2B2,0x92B2,0xA3B3,0xA3A3,0x7293,0x7293,0x7272,0xC062,0x9E8E,0xAFAF,0x7070,0x7060,0xAFAF,0xAFAF,0x6060,0x6160,0x6162,0x5252,0x7F7F,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0xCFCF,0xCFCF,0xBEDE,0xDEBE,0xDFDF,0xCFCF,0x6070,0xBFBF,0xAFAF,0xBFBF,0x90D3,0xCFCF,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0xBFBF,0x92A1,0xC1A1,0xE0D0,0xE0E0,0xD0F0,0xB0C0,0xB2C0,0xB1C1,0xE1C1,0xD1E1,0xC1C1,0xE1D1,0xB1D1,0x8292,0x7281,0x6271,0xF0C2,0xE0E0,0xE0E0,0x82F0,0x6282,0x6262,0xB182,0xC1B1,0xC1C1,0xD1C1,0xE1D1,0xD3F0,0xF0D0,0xCFCF,0x6262,0xE172,0xC1C1,0xB1C1,0xD1C1,0x92B2,0xB2B2,0x9292,0x9292,0x9292,0xC2B2,0xC1B1,0xA1C1,0x9282,0x8282,0xDFDF,0xCFCF,0xCFCF,0xCFF3
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x8181,0xB081,0xAE9E,0x8290,0x8282,0x8282,0xB392,0xB3B2,0x9292,0xB392,0xB382,0xB392,0xB392,0xB392,0xB392,0x92B3,0x92B3,0x92B3,0x8292,0x9292,0x8292,0x9291,0xB292,0xB1B1,0xB1B1,0x92B1,0x7182,0x6272,0x4161,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0xB182,0xB2B2,0x92B2,0xB3B2,0xB392,0xB3B3,0xB3B3,0xB3B3,0xB393,0x8172,0x70C0,0x7070,0x8070,0x7070,0x7070,0x7070,0x5160,0x6261,0x6261,0x5261,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xE6DA,0xBFBF,0x7070,0x7060,0x9E70,0xAFAF,0x5080,0x4050,0x9F9F,0x8F8F,0x90A0,0xD070,0x70E0,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xB140,0x92A1,0x8192,0xA181,0xD0C0,0xE0D0,0xD0F0,0xB1D0,0xB292,0xC1C1,0xD1D1,0xE0C1,0xE1E1,0xF0E1,0x82B1,0x7172,0x6272,0xE172,0xE0E0,0xE1E0,0x72F0,0x6162,0x7162,0x7262,0xB1B2,0xB2B1,0xC1B1,0xD1D1,0xE0F0,0xC0D3,0x62E0,0x7162,0xD172,0xE1D1,0xC1D1,0xB1B1,0xB292,0x92B2,0x9282,0x9292,0x9282,0xB1B2,0xC1C1,0x92C1,0x82A3,0xC5A3,0xCFCF,0xD4E4,0xC2E4,0xE0E0
.word 0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8171,0x9181,0x8EB0,0x808E,0x8271,0x92B2,0x92B3,0x9292,0xB2B3,0xB2B3,0xB292,0xB292,0xB392,0x9292,0x92B3,0xB392,0x92B2,0x8292,0x9292,0x8292,0x9292,0x9292,0xB1A1,0xC1B1,0xC0D0,0x7292,0x7272,0x4161,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xB281,0xB2B2,0x92B2,0xB2B2,0xB292,0xB2B3,0x92B3,0xA3B3,0xA3A3,0x72B3,0xAFAF,0x8080,0x8080,0x8080,0x8080,0x9F9F,0x7070,0x7161,0x6262,0x5152,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xD6C6,0xBFBF,0xAFAF,0x9F9F,0x7070,0x7070,0x7070,0x508E,0x8F8F,0x5050,0x5040,0x7090,0x5060,0x9F9F,0x7070,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x9180,0xC1B1,0x81A1,0x8181,0xA181,0xD0C0,0xE0E0,0xD1E0,0x92C2,0xC1B1,0xC1C1,0xE1D0,0xE1F1,0xE0E1,0x71E0,0x6272,0x7171,0xC162,0xE1E1,0xE0D1,0x81E0,0x5252,0x5261,0x5152,0x9262,0xB1B1,0xA192,0xC1B1,0xF2D1,0xEFEF,0x81D0,0x6262,0xD162,0xE0E0,0xC1E1,0xB2B2,0xB282,0x92B2,0x8292,0x8292,0x9282,0xC2B2,0xC1B1,0x72B1,0xB382,0xD581,0xBFBF,0xBFBF,0xC1D4,0xE0CF
.word 0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8150,0x7182,0xA181,0xBFBF,0x81AE,0xB382,0x9292,0xB382,0x82B2,0x9292,0xB392,0xB392,0xB2B3,0xB392,0x92B2,0xB2B3,0x9282,0x92B3,0x9292,0x9282,0x8182,0xA191,0xB1B1,0xA1A1,0x8292,0x7272,0x5161,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0xB292,0xB2B1,0xB2B2,0xB292,0x92B2,0xB3B3,0xB392,0x8292,0x82B3,0x82A3,0xB072,0xAFAF,0xB0B0,0xC0B0,0x8080,0xAFAF,0xAFAF,0x7280,0x7272,0x5172,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0xBFBF,0x70CC,0x7070,0x9F9F,0xAFAF,0x9F9F,0x7070,0x5060,0x4050,0x7F7F,0x5040,0x7090,0x4050,0x5040,0x7070,0x8070,0x4050,0x7F7F,0x9080,0x7080,0x8180,0xA1A1,0x8182,0x8181,0xB2A1,0xD0D0,0xE0E0,0xD1E0,0xB1C1,0xC1B1,0xD2E0,0xE1E1,0xE0D2,0x72D0,0x6272,0x6262,0x8261,0xD1D1,0xD1D0,0xD0D1,0x6152,0x6252,0x5261,0x5261,0xB272,0xB2B1,0xC1B1,0xF2C1,0xDFDF,0xD1D0,0x6272,0xD162,0xE0E0,0xC1D1,0x92B1,0xB182,0x9292,0x9292,0x8292,0x8282,0xC292,0xC1C2,0x8282,0x72B3,0xC3C3,0xCFCF,0xE3D3,0xE0D1,0xE1E1
.word 0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8281,0x8282,0xB091,0xCFCF,0xAFAF,0xB382,0x92B2,0xB282,0xB3B3,0xB2B3,0x92B3,0xA392,0xB3B2,0xB392,0x92B2,0x92B3,0x8292,0x8282,0xA192,0xB1B1,0xB2B1,0x9292,0x7282,0x7172,0x5151,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0xB160,0xB2B1,0x92B2,0xB292,0xB2B2,0xB2B3,0x92B2,0x92B3,0x92B3,0x8282,0xD092,0xCFCF,0xB1C0,0xA0A1,0x9090,0xB090,0xAFAF,0x8080,0x7282,0x7272,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0xD6C6,0xD6CA,0xAFAF,0x7070,0x9F9F,0xAFAF,0x9F9F,0x7060,0x6060,0x8F8F,0x5040,0x8F8F,0x6080,0x8F8F,0x4040,0x6060,0x6060,0xC060,0x80C0,0x6060,0x4050,0x4070,0x7150,0xA1B1,0x8182,0x8181,0xC282,0xD0D0,0xE0D0,0xD1E0,0xD1D1,0xD1D1,0xD1E1,0xE0C1,0x81C3,0x6172,0x6262,0x6262,0xD0C1,0xD0C1,0xD3C1,0x6261,0x5261,0x6251,0x6162,0x6252,0x7162,0xD182,0xB2A1,0xD0E0,0xD0D0,0x81D1,0xE172,0xD1D0,0xC1D1,0x82B1,0xB292,0x9292,0x9292,0x7282,0x8282,0xB292,0xB1C1,0xA392,0xD472,0xD471,0xD3C0,0xFFFF,0xFFFF,0xC1C1
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7140,0x8181,0x9292,0xC1A1,0xCFCF,0x81B0,0x9282,0x92B3,0x9292,0x8282,0xB382,0xB392,0xB292,0x8282,0x9282,0x9292,0xA192,0xC0A1,0xC1D1,0xB2B1,0x9292,0x7182,0x5141,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0xB292,0x9292,0xC2B2,0xC1D2,0x92B2,0xB392,0x92B3,0x92B3,0xB3B3,0xA392,0xCEB1,0xC0BE,0xB091,0xB0A0,0xA1A0,0x9EC0,0x8080,0x8181,0x72B3,0x6172,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xC6C5,0xBFBF,0xBFBF,0x9F9F,0xAFAF,0x7070,0xAFAF,0xAFAF,0x9F9F,0x5060,0x8F8F,0x7F7F,0x5040,0x7080,0x4050,0x8F8F,0x5050,0x8F8F,0x8F8F,0x5090,0x4060,0x7F7F,0x5060,0x7F7F,0xB040,0x92B2,0x8181,0x8281,0xB282,0xC1C1,0xD0D0,0xE0D1,0xD1D1,0xD1D1,0xD0D1,0x62C0,0x6162,0x7161,0x6162,0xC182,0xB1C1,0xD0C1,0x5262,0x6261,0x6152,0x6252,0x5251,0x5251,0xD162,0x9292,0xBFBF,0xC3D0,0xD1B1,0xD1D1,0xC1C1,0x92B1,0x7292,0xB2B2,0x8292,0x8292,0x8282,0x8281,0xB292,0xB2C1,0xC4D3,0xC4B3,0x71D4,0xC3C3,0xFFFF,0xC2FA,0xB2A1
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8171,0x9292,0x9192,0xB191,0xD0D0,0xBFBF,0xD0C0,0xC1C0,0x92A1,0x9282,0x9292,0x8282,0xA1A1,0xC1C1,0xB1B1,0xA1B1,0xB2B2,0xB2B2,0x92B3,0x6172,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0xA141,0xC1C1,0xC1C1,0xC1C2,0xB2C2,0xB392,0x92B3,0xB392,0xB392,0x8292,0xB182,0xC0D0,0x81B0,0x91B0,0xA191,0xC0B0,0xA09E,0x9171,0x8282,0x7182,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xDAD6,0xBFBF,0x7070,0x7070,0xAFAF,0x7070,0x9F9F,0x8F8F,0x8F8F,0x4050,0x8F8F,0x5050,0x7080,0x5060,0x4050,0x5050,0x7F7F,0xC040,0x4060,0x4040,0x7F7F,0x6060,0x7F7F,0x6F6F,0xA140,0xC1B2,0xC1D1,0xD1C1,0xD0D0,0xD1D1,0xE0D0,0xC1D1,0xD1D1,0xD2C1,0x62B2,0x6261,0x6152,0x6262,0xA162,0xB1A1,0xB1B1,0x6281,0x6152,0x5251,0x6152,0x6152,0x5152,0x9252,0x82A1,0xBFBF,0x81D0,0x9281,0xC1D1,0x92B1,0xB292,0x7272,0x92B2,0x9282,0x8292,0x8282,0x9282,0xB192,0x72D3,0xFDE3,0x72D4,0xD471,0xC072,0xD3C3,0x92C2,0xB292
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8171,0xB292,0xB292,0xB2B2,0xA1A1,0xC2B1,0xBFBF,0x9E9E,0xAEAE,0xAFAF,0x9181,0xB1B1,0xC1C1,0x82A1,0x9282,0xB2B2,0xA382,0x4171,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0xBFBF,0xDFDF,0xE1F2,0xB1D2,0xB2B2,0x8292,0xA3B3,0x8292,0x92B3,0x82B3,0x8282,0xBFBF,0xA0C0,0xB0A1,0xA0A1,0xB091,0x8EBE,0x8190,0x8292,0x8282,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x9E70,0x9F9F,0x6050,0x4050,0x8F8F,0x8F8F,0x7070,0x6060,0x4050,0x8F8F,0x7F7F,0x7040,0x7F7F,0x7F7F,0x7F7F,0x7040,0x7F7F,0x7F7F,0xE171,0xE0F0,0xD0E0,0xD0D0,0xD0C1,0xD0C1,0xD0C1,0xD1E0,0xC1D0,0xC1B1,0x61C1,0x5262,0x5261,0x6251,0x8261,0xB1A1,0xB1B1,0x5281,0x5261,0x5252,0x5261,0x5261,0x5252,0x9252,0x82A1,0xC082,0x9292,0xB282,0xC1D0,0xB2B2,0xD1D1,0xB1C1,0x8292,0x9292,0x8282,0x8282,0x9282,0xC2B1,0x93C3,0xFFFF,0xDFDF,0x72B3,0xC372,0x81C1,0xC1D1,0xB1B1
.word 0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6160,0x7171,0x9282,0xB292,0xB292,0x9282,0x91A1,0xA091,0xB291,0x92B2,0x9292,0x8282,0x8282,0x8292,0x6272,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0xBFBF,0xBFBF,0xAFAF,0xB2E1,0x7181,0x9282,0x8292,0x9292,0x92B3,0x8282,0x9282,0xC092,0xC0C0,0xC0B0,0xA1A1,0xA191,0x9EC0,0xAFAF,0x92A0,0x8282,0x6172,0x6F6F,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0xBFBF,0xAFAF,0xBFBF,0xD4C0,0xBFBF,0xCFCF,0x7070,0x7070,0xAFAF,0x9F9F,0xAFAF,0x6060,0x9F9F,0x8F8F,0x7F7F,0x5040,0x7070,0x7060,0x7F7F,0x5040,0x8F8F,0x7F7F,0x4060,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xD160,0xDFDF,0xD0D0,0xC1D0,0xB1C1,0xB2B1,0xA1A1,0x9292,0xC1E0,0xC1C1,0xD0C1,0x72E0,0x6161,0x5252,0x5261,0x6252,0x9292,0x8182,0x5282,0x5261,0x5261,0x5251,0x5252,0x5151,0xA152,0x9292,0x8282,0x9192,0xC182,0xB1D0,0xE1C1,0xE1E1,0xE0E1,0xB2D1,0x9292,0x8282,0x8282,0xB292,0xB3C2,0x71C3,0xD7C4,0xBFBF,0xB372,0x81C3,0xD281,0xC1D0,0xC1B1
.word 0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6040,0x8271,0x9282,0x9292,0xB2A1,0x92B2,0x8182,0x6161,0x7182,0x7171,0x6161,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xBFBF,0xFFFF,0x4161,0x5F5F,0x8281,0x9292,0x92A3,0x8282,0x9292,0xC092,0xB0BE,0xC0B0,0xA1A1,0xA091,0x9EC0,0x809E,0x91A0,0x8282,0x8272,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0xD070,0xCFCF,0xDFDF,0xCFCF,0x7070,0xAFAF,0xAFAF,0x9F9F,0x7060,0xAFAF,0x9F9F,0x5060,0x5060,0x8F8F,0x8F8F,0x7060,0x6070,0x4060,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x8F8F,0xEFEF,0xCFCF,0xD0D0,0xC0C0,0xB281,0x8281,0x7282,0x7272,0xB192,0xC1B1,0xD0C1,0xBFBF,0x5252,0x5261,0x6152,0x5261,0x9271,0x7281,0x5271,0x5152,0x5252,0x5251,0x5251,0x5252,0xA162,0x8281,0x8292,0x8192,0xD192,0xB2C1,0xC1D1,0xD1D1,0xD1D1,0xE0D1,0x82B2,0x8281,0x8282,0xD292,0xD3C3,0x82C3,0xC6B4,0xDA68,0x7262,0xC372,0xC281,0xC2C2,0xB282
.word 0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7050,0x7171,0x6161,0x6F6F,0x8F8F,0x6160,0x4160,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x6F6F,0x6F6F,0x8F8F,0xEFEF,0x6F6F,0x6F6F,0x8151,0x8292,0x8282,0xA392,0x9292,0xB082,0xAEAE,0xA1B0,0x91A1,0xA091,0xAEA0,0x9E9E,0x81A0,0x8292,0x8293,0xBFBF,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0xBFBF,0xEFEF,0xDFDF,0xBFBF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x8E50,0xAFAF,0x9F9F,0x6060,0x8F8F,0x7F7F,0x8F8F,0x7060,0x6060,0x4050,0x7F7F,0x7F7F,0x7F7F,0x4060,0x6F6F,0x7F7F,0x7F7F,0xD040,0xCFCF,0xCFCF,0xC0D0,0x8281,0x8281,0x8281,0x8281,0x8181,0x9282,0xB192,0xC3C1,0xE0D0,0x6161,0x5152,0x5261,0x5152,0x7252,0xB282,0xD2E3,0xB2C1,0x8281,0x6162,0x5162,0x4152,0x8281,0x8181,0x9281,0x7281,0xD0B1,0xB1B1,0xB1C2,0xD1C1,0xD1D1,0xD1D1,0x82C1,0x7282,0x9282,0xE4D3,0xE3D3,0xB2C3,0xD482,0x6757,0xD4D6,0x8282,0xB292,0xB1B1,0xB1B2
.word 0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x8281,0x8282,0x9282,0x8282,0xB082,0xBFBF,0xB0C0,0xB0B0,0xA081,0xAEB0,0xBFBF,0xA0A0,0x8291,0x8282,0xBFBF,0xBFBF,0xAFAF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x6060,0xCFCF,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0x6070,0x9F9F,0x7070,0x608E,0x5050,0x5060,0x4050,0x8F8F,0x7060,0x6060,0x4050,0x7F7F,0x6F6F,0x7F7F,0x4060,0x7F7F,0x7F7F,0x6F6F,0xCFCF,0xD0CD,0xCFCF,0x72C0,0x9282,0x82A1,0x8292,0x9292,0x61C0,0xB1F0,0x8272,0x8181,0xD082,0x41D4,0x6152,0x5252,0x5251,0x5142,0xFFFF,0xFFFF,0xC1E0,0xD1C1,0xD1D0,0xD1D1,0xB2C1,0xC1C2,0x72B2,0x7172,0x7262,0xD2E2,0x81C1,0xB292,0xD1B1,0xD1D1,0xD1D1,0x72C1,0x8272,0xE6C3,0xBFBF,0x82B3,0x7181,0xB282,0x71D0,0xAFAF,0x8272,0x9282,0xC1B2,0xC1E1
.word 0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0x7F7F,0x6F6F,0x7140,0x8182,0x9282,0x8282,0x8282,0xCFCF,0x80AE,0x91C0,0x8191,0xD091,0x9EAE,0x9090,0x92B0,0x8282,0x7282,0xCBAC,0xBFBF,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x5060,0x5060,0x4050,0x4040,0x7050,0x6070,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x5070,0x7F7F,0x7F7F,0x7F7F,0xDFDF,0xCFCF,0xDFDF,0xA1C0,0xB1B1,0xC0B1,0xBFBF,0xCFCF,0xBFBF,0x92D0,0x7182,0x7272,0x8271,0x62D0,0x5241,0x5251,0x4251,0x5251,0xBFBF,0xE3E5,0x9181,0xB1A1,0xC1B1,0xD0C0,0xE0D0,0xE0E0,0xD1D1,0xC2C1,0x8292,0xFFFF,0x41D3,0xC272,0xC1C1,0xD1D1,0xD1D1,0x72C2,0xE092,0xABCC,0xAFAF,0x7282,0x7272,0x8282,0xD1C3,0x47D0,0x8272,0xD2B2,0xE1D1,0xB1D1
.word 0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0xAFAF,0x7F7F,0x7F7F,0xB281,0x8282,0x9292,0x8292,0xC092,0x9EBE,0xBFBF,0x81A0,0xC0B0,0xBFBF,0xA0A0,0x9181,0x8282,0x7282,0xAFAF,0x9C8C,0x8C8C,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0x9E9E,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0x708E,0xAFAF,0x609E,0x9F9F,0x6060,0x8F8F,0x7F7F,0x6050,0x6070,0x4060,0x7F7F,0x7F7F,0x4040,0x6060,0x7F7F,0x6F6F,0x4040,0x7040,0xE0E0,0xE0D0,0x81E1,0xC0A0,0xBFBF,0xDFDF,0xCFCF,0xB181,0x82B2,0x8281,0x7172,0x7172,0xE0B2,0x5142,0x4142,0x5241,0x7161,0x6272,0x8171,0x9292,0x8281,0x9281,0xB2A1,0xC1B1,0xC1C1,0xD0D1,0xD1E1,0xE1E0,0xF0E1,0xE2E0,0xE1E1,0xD1E1,0xE1D1,0xD1C1,0x8192,0xBFBF,0x4261,0x7263,0x7272,0x7282,0x7281,0xC282,0xD1D1,0xE1D1,0xE0D1,0xC1D1,0xB2C1
.word 0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5060,0x8F8F,0x7060,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x9E60,0x5070,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x8191,0x9292,0x8282,0x8182,0xAEB0,0xC0AE,0x90A0,0xB091,0xC0D0,0xB0B0,0x9181,0x7282,0xA382,0xBFBF,0x7C7B,0x7C7C,0x6C4A,0x4A4A,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x9F9F,0xD659,0xAFAF,0xAFAF,0xAFAF,0x708E,0xAFAF,0x7070,0x7070,0x9E70,0xAFAF,0x9E70,0xAFAF,0x9F9F,0x6050,0x8F8F,0x8F8F,0x8F8F,0x6060,0x5050,0x7F7F,0x7F7F,0x7F7F,0x5070,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0xC080,0xD1D0,0xEFEF,0xB081,0xC0C0,0xD0C0,0xB1D0,0xB2B1,0x7282,0x7172,0x7282,0x7271,0xD271,0x42D0,0x6251,0x7162,0x7272,0x8272,0x8282,0xA182,0xA1A1,0xA192,0xA192,0xB1B2,0xB1B1,0xC1B1,0xC1C1,0xD1C1,0xD1D1,0xF0E0,0xF0F0,0xE1F0,0xD1E1,0xB1D1,0x7272,0xAFAF,0xC3C3,0xAFAF,0x7271,0x9272,0xB1B2,0xC1C1,0xD1C1,0xD1D1,0xD1D1,0xC1C1,0xB2A1
.word 0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6060,0x9F9F,0x9F9F,0xAFAF,0x6070,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x607E,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8190,0x8282,0x8281,0x8182,0xD0B0,0xC0AE,0xA0A0,0xB1B0,0xB0C0,0x90B1,0x8281,0x8282,0xB382,0xBFBF,0x9B9B,0x9F9F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0xBFBF,0xCFCF,0xBFBF,0xAE9E,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x4040,0x8F8F,0x7050,0x6070,0x5060,0x7F7F,0x4040,0x7F7F,0x6070,0x4040,0x7F7F,0x7F7F,0x7050,0xB261,0xA1A1,0xBFBF,0xB180,0x7282,0xB282,0xB1A1,0x7282,0x7172,0x6262,0x8172,0x7272,0x8172,0xC3C1,0x7172,0x7272,0x8272,0x8181,0x8181,0x9281,0xC1A1,0xC1D1,0xB1C2,0xB2B1,0xB2A1,0xA1A1,0xB1A1,0xB1B1,0xB1B1,0xC1B1,0xD0D1,0xD1E1,0xD1D1,0xB1C1,0x7272,0x7272,0x7171,0xB3D0,0xB292,0xB1B1,0xB1B1,0xB1B1,0xC1B1,0xD1C1,0xC1C1,0xA1B1,0xB192
.word 0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0x7070,0x7070,0x6060,0x9E60,0x509E,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8181,0x8281,0x8181,0xC081,0xAEDE,0x9080,0xC0C0,0xA1A1,0x91A1,0x8181,0x8282,0x8282,0xAFAF,0x9B9C,0x8B9B,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0xAFAF,0x70C0,0x6F6F,0x7F7F,0xAFAF,0xAFAF,0xCEBE,0xCFCF,0xDFDF,0x709E,0xAFAF,0x7070,0xAFAF,0xAFAF,0x8E9E,0xAFAF,0xAFAF,0x6070,0x9F9F,0x8F8F,0x7F7F,0x6050,0x6060,0x4060,0x7F7F,0x7F7F,0x7F7F,0x6060,0x7F7F,0x7F7F,0x7F7F,0xF0C0,0xDFDF,0xCFCF,0xBFBF,0xB2C2,0xA181,0xB1B1,0x7292,0x7172,0x6262,0x6162,0x6162,0x8172,0x7271,0xC172,0x62C1,0x6261,0xA181,0x8192,0x9191,0x9272,0xD0C1,0xD3D0,0xC2F2,0xB1B1,0xB2B1,0xB2B2,0xB2B2,0xB292,0xB292,0x9292,0xB192,0xD1C1,0xC1C1,0xA1B1,0x7272,0xB2B1,0xB2B2,0x9292,0xB292,0xA192,0xA1A1,0xA1B2,0xB1A1,0xB1B1,0xA1C1,0x9292,0xB2B1
.word 0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x6060,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x607E,0x8F8F,0x6060,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x8181,0x8181,0xB182,0xCFCF,0x809E,0xD0D0,0x91B0,0x91A1,0x8281,0x8282,0x7281,0x9F9F,0x9D8E,0x8B9C,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x424A,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0xBFBF,0xCFCF,0x9F9F,0x8F8F,0xCFCF,0xBFBF,0xCECE,0xCEBE,0xAEDE,0x709E,0x7070,0xAFAF,0x7070,0xAFAF,0x9E9E,0x8E70,0x9E70,0xAFAF,0x8F8F,0x4050,0x8F8F,0x9F9F,0x6060,0x4060,0x7F7F,0x7F7F,0x6F6F,0x6060,0x4050,0x7F7F,0xBFBF,0xDFDF,0xBFBF,0xBFBF,0xCFCF,0xD2F3,0xC1B2,0x82C2,0x7172,0x7262,0x7271,0x6271,0x7262,0x8272,0x7282,0x8281,0xD2C1,0x82B3,0xB282,0xB1D1,0x8181,0xD0B1,0xC3C0,0xE1C3,0xE1F0,0xC1D1,0xD1D1,0xC1D1,0xB1B1,0xB2B1,0xB2B1,0xB2B2,0xC1B2,0xC1C1,0xB1C1,0x92B2,0x7272,0x9272,0xB2B1,0x92A1,0xA192,0x9292,0x9292,0x92A1,0x92A1,0xA1A1,0xA192,0xC1B2,0xB2B1
.word 0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x6060,0x9F9F,0x6060,0x5050,0xAFAF,0x9F9F,0x9F9F,0x7070,0x7060,0x6060,0x6070,0x4040,0x7F7F,0x8F8F,0x4050,0x8F8F,0x8E50,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7150,0x8181,0xA181,0x9EB0,0xC0AE,0xC0B0,0x9090,0x9191,0x8181,0x8181,0x6271,0x8F8F,0x8E7D,0x8D8E,0x8C9C,0x7C4A,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x9F9F,0xCFCF,0xCFCF,0xCFCF,0xAFAF,0xCFCF,0xCEBE,0xCECE,0xCECE,0xAECE,0x9E9E,0xAFAF,0x7070,0xAFAF,0x7070,0x9E9E,0x708E,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x5050,0x6060,0x4050,0x7F7F,0x4040,0x7F7F,0x7050,0x7F7F,0x6F6F,0xE040,0xBFBF,0xB0B0,0x80A0,0xAFAF,0xC2D3,0x81D1,0x7281,0x7271,0x7271,0x7272,0x8272,0x9292,0x7281,0x8172,0x8282,0xD282,0xFFFF,0xA182,0xB0B1,0x81B0,0xD0D0,0xE0D1,0xE0E0,0xF0E0,0xDFDF,0xDFDF,0xE0F0,0xD1E0,0xB1C1,0xB1C1,0xC1B1,0xC1B1,0xC1C1,0xB1C1,0xB1B2,0x7171,0x8272,0xE1B1,0xE0E1,0x92D1,0x92A1,0x92A1,0x9292,0x9292,0x9292,0xB192,0xB1C1,0xB1B2
.word 0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x608E,0x9F9F,0x6060,0xAFAF,0x7060,0xAFAF,0x6070,0x7070,0x9F9F,0x9F9F,0x7070,0xAFAF,0x9F9F,0x4070,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x8E70,0x9F9F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x9180,0xA1A1,0xBFBF,0xB0D0,0xA0A0,0x8181,0x9291,0x7281,0x7281,0x6171,0x8F8F,0x8F8F,0x7D7D,0x7E8E,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xCFCF,0xCEBE,0xBEBE,0xBECE,0x9EAE,0xAFAF,0xAFAF,0x7070,0x9E70,0x9E9E,0xAFAF,0xAFAF,0xAFAF,0x5070,0x8F8F,0x8F8F,0x5050,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7050,0x4060,0x6F6F,0x8F8F,0xB0D0,0x81B0,0xE292,0x7182,0xD0D0,0x81D0,0x7172,0x7162,0x7272,0x7272,0x8281,0x8182,0x7272,0x8271,0x92B2,0xD2B2,0xC1F3,0xC081,0x7170,0xE0B2,0xD3E0,0xD0D0,0xC1C1,0xD0D0,0xCFCF,0xCFCF,0xCFCF,0xD0D0,0xD0D0,0xC1C1,0xC1C1,0xB1C1,0xC1C1,0xB1B1,0x72B1,0x7262,0xB272,0xDFDF,0xDFDF,0xDFDF,0x81B2,0x8292,0xA1A1,0xA192,0xA1A1,0xC1C1,0xB1C1,0x92A1
.word 0x9F9F,0x9F9F,0x608E,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x7070,0x7070,0x7070,0x6070,0x6070,0xAFAF,0x6080,0x9F9F,0x7070,0xAFAF,0x8F8F,0x9F9F,0x8E60,0x7070,0xAFAF,0x708E,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0xD0D0,0xD0E0,0x91C0,0x91A0,0x9191,0x9191,0x8181,0x7081,0x9F9F,0x9F9F,0x8F8F,0x5C5C,0x7C7E,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0x9E70,0xCFCF,0xBEAE,0xBEBE,0xCEDE,0xBFBF,0x7070,0x7070,0xAFAF,0x9E70,0x9E9E,0xAFAF,0x8E70,0x9E70,0xAFAF,0x8F8F,0x5050,0x8F8F,0x8F8F,0x5050,0x8F8F,0x7F7F,0x7F7F,0x7040,0x5060,0x6F6F,0x4040,0x7F7F,0x7181,0xFFFF,0xD0E3,0xCFCF,0x8272,0x6261,0x7271,0x7171,0x8172,0x7172,0x7271,0x7162,0x7162,0xC182,0xF2F1,0xC1E1,0x7171,0xD171,0xCFCF,0xC0D0,0xB2C2,0xB1B1,0x92B2,0xC0B1,0xB1C0,0xB0C0,0x8181,0x8282,0x9282,0x8292,0xB1A1,0xB1B1,0x8292,0x7172,0x6162,0xF192,0xF0F0,0xF0F0,0xFEF0,0x82D0,0xA192,0x9292,0xB2A1,0xC1B2,0xC1C1,0xB2B1,0xB2B2
.word 0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5060,0x8F8F,0xAFAF,0xAFAF,0x7070,0xAFAF,0x6080,0x7060,0x7070,0x7060,0x7060,0x9F9F,0x7070,0xAFAF,0x7070,0x7070,0x709E,0x7070,0x7070,0x9F9F,0x9F9F,0x9F9F,0xBFBF,0xBEBE,0xD0D0,0x91C0,0xA191,0x91A1,0x7181,0x8F8F,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7060,0xAFAF,0x9E70,0x9E9E,0xAFAF,0x9E9E,0xAE9E,0xBE9E,0xCEDE,0x9EAE,0x9E70,0xAFAF,0x7070,0xAFAF,0x9E9E,0xAFAF,0xAFAF,0x9E9E,0xAFAF,0x8F8F,0x8F8F,0x5060,0x6060,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5070,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xAFAF,0xCFCF,0x81C0,0x6281,0x7272,0x7172,0x6262,0x7162,0x7272,0x7182,0x7172,0x8272,0xB192,0xE1C1,0xD1F2,0x71C1,0xE0D1,0xCFCF,0x81C0,0x8181,0x8282,0x9281,0x9291,0xA1B2,0xB2B2,0xB292,0x9281,0x9291,0x8281,0xC1B2,0xB2B1,0x6281,0x6271,0x7261,0xDFDF,0xDFDF,0xE0E0,0xE0E0,0x92D0,0x9292,0xA1A1,0xB2B1,0xC1C1,0xB1C1,0xB2A1,0xB1B2
.word 0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x506E,0x6050,0x5040,0x8F8F,0x7060,0x5060,0x9F9F,0xC070,0x7090,0x7070,0xAFAF,0x9F9F,0xB0D0,0xBFBF,0xC0F0,0xCFCF,0x7070,0xBFBF,0x7080,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xB0C0,0xA1C0,0x8191,0x7090,0x4050,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4142,0x4241,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x9F9F,0x9F9F,0xBFBF,0x9E70,0xBFBF,0xBFBF,0x709E,0xBFBF,0xAE9E,0xCFCF,0xCEDE,0xAEAE,0x708E,0x7070,0x8E70,0xAFAF,0x9E9E,0x709E,0xAFAF,0x9E70,0x8E9E,0x8F8F,0x5050,0x8F8F,0x6050,0x5050,0x7F7F,0x7F7F,0x7F7F,0x7040,0x4060,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xB081,0x7181,0x7171,0x8172,0x7281,0x6162,0x7161,0xD072,0x7172,0x7162,0x7172,0xB082,0xB1B1,0xE2F1,0xC2C0,0xD0D0,0xBFBF,0x81C0,0x92B2,0x92A1,0xB1B1,0xC1C1,0xC1C1,0xC1C1,0xB1B1,0xB1C1,0xB2B1,0xB1C1,0xB1B1,0xA1B2,0x7272,0x7162,0x8262,0xEFEF,0xDFDF,0xE0E0,0xC1D0,0x92B1,0xB1A1,0x7281,0xB182,0xB1C1,0xA1B1,0xB1B2,0xB2B1
.word 0x6F6F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9E8E,0x9F9F,0x6060,0xAFAF,0x7090,0x9F9F,0xCFCF,0x70D0,0xAFAF,0xBFBF,0x9EB0,0xC0DE,0xBFBF,0xEFEF,0xC09E,0xBFBF,0xAFAF,0xBFBF,0x709E,0xD0C0,0x70C0,0xAFAF,0x8F8F,0x7050,0xAE9E,0xB0D0,0x8080,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8E70,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x9F9F,0xB070,0xC090,0x80C0,0x62C0,0x4141,0x7F7F,0x7F7F,0x8F8F,0x9E70,0xBFBF,0xBFBF,0xDECE,0xAEBE,0x709E,0x9E60,0xBFBF,0xBFBF,0xCECE,0x9EBE,0x7070,0xAFAF,0xAFAF,0x9E70,0x9E9E,0xAFAF,0xAFAF,0xAFAF,0x709E,0x9F9F,0x8F8F,0x6060,0x8F8F,0x5050,0x8F8F,0x7F7F,0x7F7F,0x6040,0x5060,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8141,0x81B1,0x7181,0x8272,0x8281,0x6272,0x5161,0x7261,0x7181,0x7172,0x6162,0x7162,0x7171,0x80C0,0xD1A1,0xC0B2,0xD0D0,0xC0C0,0xB282,0xB1A1,0xC181,0xD0C1,0xD1C1,0xC1D1,0xC1D0,0xC0C0,0xC1C0,0xE1C1,0xC1D1,0xB1B1,0x92B2,0x7162,0x6271,0xD162,0xEFEF,0xE0E0,0xD0E0,0xD1C1,0xA1C0,0xB1A1,0x8271,0xC1B1,0xB1C1,0xB2A1,0xC1B2,0xC2B1
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7070,0xAFAF,0x9EC0,0xBFBF,0xAFAF,0xAFAF,0x70B0,0x909E,0xD070,0xAFAF,0xC09E,0xBFBF,0xCFCF,0x5081,0x7070,0x7070,0xAFAF,0x9E70,0xFEDE,0xC0E0,0x9F9F,0x9F9F,0x9E9E,0x709E,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x8F8F,0x6F6F,0x7F7F,0xC0C0,0xCFCF,0xDFDF,0xCFCF,0xDFDF,0xC0D0,0x4161,0x7F7F,0x7F7F,0xAFAF,0x9E9E,0xAE9E,0xDFDF,0xCFCF,0xBFBF,0x9F9F,0x9EAE,0xBE9E,0xDECE,0xAEBE,0x709E,0x7070,0xAFAF,0x9E9E,0x9E9E,0xAFAF,0xAFAF,0x9F9F,0x9E9E,0x9F9F,0x5050,0x9F9F,0x6060,0x6050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x5070,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xF571,0x71B2,0x7172,0x81A1,0x7272,0x7171,0x6252,0x9272,0x7272,0x6271,0x6271,0x7271,0x7171,0x9F9F,0x8161,0xC1C1,0x71C0,0xC371,0x80B1,0xC072,0xBFBF,0xBFBF,0xC0C0,0xBFBF,0xAFAF,0xAFAF,0xC262,0xD1E1,0xC1C1,0xF1E2,0xC1C1,0x6272,0x6172,0xE172,0xDFDF,0xE1E0,0xC1D1,0xD1C1,0xC1E1,0xA1B1,0xB1A1,0xA1B1,0xA1A1,0xB2B2,0xC1C1,0xC2B1
.word 0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x708E,0xAFAF,0xD0AE,0x80C0,0xAFAF,0x7070,0x60B0,0x808E,0xE09E,0x8080,0x8E90,0x80D0,0x71AE,0xAFAF,0x7070,0x7070,0x7080,0xCFCF,0xEFEF,0xCFCF,0x6060,0x9F9F,0x709E,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x7070,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0xAFAF,0x9F9F,0x7F7F,0x6F6F,0x8F8F,0xBEBE,0xDEDE,0xDEEE,0xDEDE,0xFEEE,0xCFCF,0xD0D0,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0x9E70,0xFEEE,0xFEFE,0xAEDE,0xAFAF,0xAEBE,0xCFCF,0xDEDE,0xAEBE,0xAFAF,0x709E,0xAFAF,0xAFAF,0x9E9E,0x9E9E,0xAFAF,0x6060,0xBE9E,0x709E,0x9F9F,0x9F9F,0x9F9F,0x6060,0x8F8F,0x7F7F,0x4050,0x7F7F,0x5060,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xFFFF,0x7192,0x7061,0x81B0,0x7271,0x6272,0x9261,0xFFFF,0x81E1,0x6262,0x6162,0x6172,0x6161,0x9F9F,0xC151,0xC1D1,0xC0C3,0xA181,0xAFAF,0x9DD0,0xAFAF,0x7171,0x6272,0x6171,0x7161,0x6171,0xE0B2,0xC1D1,0xD1C1,0xFFFF,0xC1E0,0x81B1,0x6272,0xE0A1,0xE1E0,0xD1E1,0xC1D1,0xE1C1,0xEFEF,0xB1C1,0xB1A1,0xB1B1,0xC2B1,0xC1C1,0xC2C1,0xB1B2
.word 0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xE08E,0xD0C0,0xD0D0,0x80D0,0xAFAF,0xBFBF,0x80C0,0x9E70,0xAFAF,0x9E60,0x60C0,0x9F9F,0xAFAF,0x7060,0x9F9F,0xAFAF,0xBFBF,0xEFEF,0xEFEF,0xCFCF,0x7080,0x9E9E,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x7060,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x708E,0x9F9F,0x9F9F,0x9F9F,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xCFCF,0xAFAF,0xAFAF,0x4247,0x8F8F,0xBFBF,0xBFBF,0xBEBE,0xAEAE,0xCECE,0xFEEE,0xDFDF,0xCFCF,0xC0D0,0x60C0,0x5060,0x8E70,0xAFAF,0xFEBE,0xEFEF,0xBEEE,0xAFAF,0xBEBE,0xCEBE,0xDEDE,0x9EBE,0xAFAF,0xAFAF,0x7070,0x9E9E,0x9E9E,0x709E,0x9E9E,0xAFAF,0x9E70,0x70AE,0x9F9F,0xAFAF,0x6060,0x5050,0x4050,0x8F8F,0x4050,0x7F7F,0x4060,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4060,0x7F7F,0xA1A1,0xB1C2,0x8181,0xC1A1,0xE1E1,0xE1F1,0x6171,0x7171,0x5170,0x7162,0x9F9F,0xD071,0xBFBF,0x71C0,0xAFAF,0xC061,0xAFAF,0xC180,0x7181,0x8281,0xB281,0xB2B1,0xB281,0xD1D1,0xC1C1,0xD1C1,0xCFCF,0x9281,0x81A1,0x8281,0xE0C1,0xE1D1,0xD1D1,0xC1C1,0xE0C1,0xDFDF,0xD1E0,0xD1D1,0xE1D1,0xD1D1,0xE1D1,0xE1E1,0xD1D1
.word 0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7E40,0xDFDF,0xD0E0,0xCFCF,0x8090,0x809E,0x9E9E,0x90D0,0x9E8E,0xAFAF,0x5060,0x8F8F,0x6060,0x9F9F,0x6070,0xAFAF,0x70C0,0xBFBF,0xCFCF,0xDFDF,0xF0FE,0x9EC0,0x7070,0x6070,0x9F9F,0x8E60,0x709E,0x6060,0x7060,0x8E60,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0xAFAF,0xBFBF,0xCFCF,0xF9F6,0xD5EA,0xBFBF,0xBFBF,0x61D0,0x8F8F,0xCFCF,0xAFAF,0xAFAF,0x709E,0x708E,0xAE9E,0xDEBE,0xEFEF,0xDFDF,0xCFCF,0xD0E0,0x70B0,0x6070,0xEE8E,0xDFDF,0xDEFE,0x9EBE,0xBEBE,0xCFCF,0xEEDE,0xAECE,0xAFAF,0x8E70,0xAFAF,0xBFBF,0xAFAF,0x9E9E,0x9E9E,0x709E,0x7060,0x9E9E,0x6060,0xAFAF,0x6060,0x9F9F,0x4050,0x7F7F,0x7F7F,0x7F7F,0x5070,0x7F7F,0x7F7F,0x6F6F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x8190,0xC0D0,0x8171,0xA181,0xC0C1,0xE0D0,0x81D1,0x7061,0x5161,0x8F8F,0x7F7F,0xBFBF,0xAFAF,0xAFAF,0x7081,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0xD151,0xC1D1,0xC1D1,0xC1C1,0x92C1,0xA192,0x6281,0xB162,0xD1D1,0xD1E1,0xC1D1,0xC1B1,0xE0D1,0xD1D1,0xD1D1,0xC1D1,0xD1C1,0xD1C1,0xC1D1,0xE0E1,0xF1E1
.word 0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xF0FE,0xFFFF,0xBFBF,0xAFAF,0x9F9F,0x708E,0xAFAF,0x5060,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x6070,0x8F8F,0xAFAF,0x9F9F,0x808E,0xBFBF,0xF0FE,0xCFCF,0xCFCF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x8E70,0x7070,0xAFAF,0xAFAF,0xC08E,0xBE9E,0xF8EB,0xCFCF,0xD0E4,0xD0E0,0xBFBF,0xAFAF,0xAEDE,0xAEAE,0xAFAF,0xAFAF,0x9E70,0xAFAF,0xAFAF,0x9E9E,0xBEAE,0xEECE,0xEEFE,0xCEDE,0xC0D0,0xBFBF,0xDFDF,0xFEFE,0x9EDE,0xCEAE,0x9EBE,0xCFCF,0xBEEE,0x709E,0x709E,0xAFAF,0x9E9E,0x9EBE,0x8E8E,0xAFAF,0x9EAE,0x7070,0x9E70,0x6070,0xAFAF,0x9F9F,0x6060,0x5060,0x8F8F,0x8F8F,0x5040,0x6080,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xB1B1,0x8F8F,0x8060,0x6181,0x7180,0xB090,0xC0A1,0xA1B0,0xC1D0,0x7161,0x8F8F,0x7F7F,0xD071,0xBFBF,0xAFAF,0x7180,0xC0B1,0x9F9F,0xC080,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD161,0xC1C1,0xC1C1,0xC1D1,0xA1B1,0x8292,0x7162,0xB162,0xE1C1,0xD1E1,0xB1D1,0xC1B1,0xB1D1,0xC1C1,0xB1B1,0xB1B1,0xB1C1,0xB1C1,0xC1B1,0xE0E0,0xE1D1
.word 0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0xCE8E,0xFFFF,0xEFEF,0xDFDF,0x70C2,0x6060,0x9F9F,0x5060,0x4070,0x5050,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0x9F9F,0x6060,0x8E70,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0x6080,0xAFAF,0x9F9F,0xAFAF,0x7070,0xAFAF,0x7070,0x9F9F,0xAFAF,0x9F9F,0x9E70,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7070,0x708E,0xAFAF,0x7070,0x7070,0xCFCF,0xE9DB,0xFDFA,0xCFCF,0xE0E0,0xBFBF,0xC5D6,0xAFAF,0x809E,0x709E,0x9E70,0x709E,0x709E,0x7070,0x7080,0x7070,0x9E70,0xBE9E,0xDEDE,0xBECE,0xAEBE,0xEEBE,0xDFDF,0xBEDE,0xAE9E,0x9EBE,0xDE9E,0xCEEE,0x709E,0xAFAF,0x7070,0x7070,0xCFCF,0xBFBF,0x70BE,0xAFAF,0x709E,0x7070,0xC09E,0x6060,0x709E,0x9F9F,0x6060,0x8F8F,0x4050,0x8F8F,0x6080,0x7F7F,0x7F7F,0x7F7F,0x4040,0x7F7F,0xB0A0,0x7F7F,0xC150,0x7181,0x6170,0xB081,0xC0B0,0x70B0,0xB080,0x6190,0x8F8F,0x7F7F,0xBFBF,0xAFAF,0x81B0,0x7171,0xAFAF,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD182,0xE0E0,0xE0E0,0xC1E0,0xB1B1,0x8192,0x8172,0xA1A1,0xFFFF,0xEFEF,0xB1C1,0xC1C1,0xB1B1,0xB1A1,0x92B1,0x92A1,0xB2A1,0xB292,0xB2B2,0xC1D2,0xD1D1
.word 0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0xEFEF,0xFFFF,0xBEF1,0xCFCF,0x80D3,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x4040,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9E60,0xAFAF,0xE3F2,0xE6E5,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x708E,0xAFAF,0xAFAF,0xAFAF,0x7070,0xAFAF,0x6060,0x9F9F,0x7070,0xBFBF,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0xAFAF,0xCE9E,0xF9E7,0xFBFA,0xDFDF,0xE0E0,0xBFBF,0xC7C7,0xAFAF,0x709E,0x7070,0xAFAF,0x709E,0x7070,0x9E70,0xAFAF,0x9E70,0xAFAF,0xAFAF,0xAE9E,0xBEBE,0x9E9E,0xBFBF,0xCEEE,0xCECE,0x9E9E,0x9EAE,0x9E70,0xDEDE,0x9EAE,0x9E70,0xAFAF,0x9E9E,0xAFAF,0x9E9E,0xAFAF,0x9E70,0xBFBF,0x7070,0x9E70,0xAFAF,0x9E70,0x6070,0x5050,0x8F8F,0x8F8F,0x8F8F,0x7070,0x4040,0x8F8F,0x7F7F,0x7F7F,0x9040,0x9081,0x70B0,0xD050,0x6190,0x8071,0xA180,0xB0A0,0x6080,0x7060,0x5171,0x7F7F,0xD051,0xBFBF,0xC280,0x81B1,0x71C0,0x8081,0x7160,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0xE0C1,0xE0E0,0xE0F0,0xD0F0,0x82A1,0x9292,0x7281,0x9282,0xFFFF,0xEFEF,0xB1C1,0xC1C1,0x92B1,0xB2A1,0xC2B1,0xB1B2,0xC2B1,0xB1C1,0xB1B1,0xB2B2,0xD2C2
.word 0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0xF2FE,0xEFEF,0xAFAF,0xBFBF,0x7E71,0xAFAF,0x60B0,0x9F9F,0x8F8F,0x4050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5040,0x8F8F,0x8F8F,0xEFEF,0xF9E5,0xDFDF,0xAFAF,0xAFAF,0xAFAF,0x8E70,0xAFAF,0x7070,0xAFAF,0xAFAF,0xBFBF,0x6080,0x9F9F,0x7050,0xAFAF,0xAFAF,0xAFAF,0x8070,0x8E80,0x9F9F,0x7060,0xAFAF,0xBFBF,0xDCCC,0xBCEB,0xDFDF,0xDFDF,0xDFDF,0xBFBF,0xBFBF,0x9F9F,0x6070,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0x9E9E,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0x709E,0xAFAF,0x7070,0xAFAF,0xAFAF,0xEE9E,0x9EDE,0xAFAF,0x709E,0x709E,0xAFAF,0xBFBF,0x8E9E,0x8080,0x9EBE,0x9E70,0x8080,0xAFAF,0x7070,0x6080,0x8F8F,0xD050,0x4060,0x8F8F,0x6060,0x7F7F,0x5040,0x7F7F,0x7F7F,0x8040,0x8080,0xA090,0x8F8F,0x71A1,0xC1C1,0x9071,0xA0A0,0x5080,0x7F7F,0x7F7F,0x7F7F,0xCFCF,0xAFAF,0x80C1,0x7F7F,0x7181,0xC081,0x7071,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0xE0F1,0xE0E0,0xF0E0,0xC1E0,0x7282,0x7181,0x7272,0x7292,0xE392,0xD1F3,0xB1B1,0xB1C1,0xA192,0xB1B2,0xB1C2,0xB1B1,0xD192,0xB1E0,0xB2C2,0xB1C1,0xC1C2
.word 0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9E8E,0xEFEF,0xEECC,0xB4E3,0x9F9F,0x9F9F,0x9F9F,0x50C0,0x8F8F,0x8F8F,0x8F8F,0x4040,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8E7E,0x8C6A,0xF5EC,0xDFDF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x7060,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xCFCF,0xCFCF,0xCFCF,0xC0E0,0xBFBF,0x7F7F,0x9F9F,0x7060,0x7070,0xAFAF,0x708E,0x9E9E,0xAFAF,0xAFAF,0x70BE,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0x9E70,0x9EBE,0xBFBF,0xAFAF,0xAFAF,0x7070,0xAE9E,0xBFBF,0x9E9E,0x9E9E,0xBE9E,0xBFBF,0x9E9E,0x7080,0x709E,0x5050,0xBFBF,0xBFBF,0x8F8F,0x7050,0x5060,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8070,0xB081,0x90A1,0xA090,0xB0C0,0xA0B1,0x90A1,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0xFFFF,0xB0C0,0x8F8F,0x7F7F,0xB281,0x7281,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xD171,0xE0F0,0xE0F0,0xDFDF,0x81D0,0x8172,0x7272,0x8172,0x7192,0x8272,0x9292,0xB1B1,0x9281,0x9292,0xC2B1,0x92B1,0xA192,0xC192,0xC2C1,0xC2B1,0xC2C2,0xB1B2
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBE4A,0xD5DC,0xAFAF,0x7061,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4242,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAE9C,0xEFEF,0xD0F0,0x9F9F,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7070,0x9F9F,0x6060,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x6060,0xAFAF,0xD0D0,0xDEBE,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xDFDF,0x80B0,0x618E,0x8F8F,0x6F6F,0x8F8F,0x6070,0x9F9F,0xAFAF,0x9E70,0x9E70,0x709E,0xAFAF,0xAFAF,0x9E9E,0x9E9E,0xAFAF,0x7070,0x7070,0xAFAF,0x708E,0x709E,0x709E,0x708E,0xAFAF,0x9E70,0xAFAF,0x9E8E,0x9E9E,0xAFAF,0x8080,0x8090,0x8080,0xAFAF,0x7070,0xAFAF,0x8080,0x7070,0x9F9F,0xDEE0,0xAFAF,0x4040,0x8F8F,0xBFBF,0x50C0,0x7F7F,0x7F7F,0x7060,0x7070,0xD0A0,0x80D0,0x9071,0xC0C0,0xC080,0x60D0,0x9F9F,0x7F7F,0x7F7F,0xBFBF,0xEFEF,0x60C0,0x6F6F,0x6F6F,0xA1C0,0xD2B2,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0xC281,0xE0E1,0xCFCF,0xCFCF,0xE0E0,0x82C1,0x8272,0x7182,0x7272,0x7182,0xB272,0xB1C1,0xC292,0x92B2,0xB292,0xC2B1,0x92C1,0x9292,0x9292,0xB2B2,0x92B2,0xB292,0xB292
.word 0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8E50,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x708E,0xAFAF,0xCE9E,0xC5E4,0xAFAF,0xEA8D,0xAFAF,0x9F9F,0x70B0,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0x5070,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0x6457,0x4842,0x9F9F,0x7F7F,0x7F7F,0xAFAF,0xDFDF,0xBFBF,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7070,0x9F9F,0xAFAF,0x6060,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xC070,0xBFBF,0xBFBF,0xD7CB,0xFBDB,0xCFCF,0xAFAF,0xBFBF,0xD0D0,0x9F9F,0xAFAF,0x7F7F,0x8F8F,0x5060,0x8F8F,0x9F9F,0x9F9F,0x9E9E,0xAFAF,0x9F9F,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0x9EC0,0xAFAF,0xAFAF,0x9E70,0x9E70,0xBFBF,0xAFAF,0x7070,0x7070,0x8080,0x8080,0x8080,0xBFBF,0x80C0,0x7080,0x7070,0xC070,0x9E80,0xAFAF,0xEEF0,0x9EBE,0x8F8F,0x8F8F,0x7060,0x80DE,0x6F6F,0x5040,0x6050,0x7070,0xB090,0xB0F0,0x9080,0x80D0,0x6060,0x5060,0x7F7F,0x4241,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0xFFFF,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0xD171,0xE0E0,0xCFCF,0xD0D0,0xE0D0,0xC1D0,0x8281,0x7281,0x7271,0x6162,0x7262,0xD192,0xC1D1,0xB2B2,0x92C2,0xB292,0xC2B2,0xA1D2,0x9292,0x8292,0x92A1,0x9292,0x9282,0x8282
.word 0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x607E,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xFE9E,0xEFEF,0x49D5,0xBFBF,0x72D0,0x9F9F,0x4070,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xBFBF,0xAFAF,0x8C49,0x9F9F,0xBFBF,0xBFBF,0x80AE,0xAFAF,0xAFAF,0x8E70,0x7070,0xAFAF,0xAFAF,0x7070,0x9F9F,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0xB080,0xAFAF,0xFECE,0xBFBF,0xBBBB,0xDCDB,0x9DCC,0xAFAF,0x8F8F,0x8F8F,0xCED0,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x709E,0xAFAF,0x7060,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0xAFAF,0x9E9E,0xAE9E,0x9EAE,0x9E9E,0x9E9E,0xAFAF,0xCE9E,0x7070,0x6070,0x7070,0x7080,0x7070,0x8070,0xC090,0x8090,0xAFAF,0xAFAF,0xBFBF,0x9EAE,0x809E,0xEEF0,0x8ED0,0xBFBF,0x8F8F,0x8060,0xDFDF,0x7060,0x8070,0x4060,0x7060,0x8070,0xC0C0,0x9090,0xCFCF,0x8F8F,0x8F8F,0x6052,0x4152,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x9061,0x9F9F,0x4E70,0x7F7F,0x6F6F,0x7F7F,0xC171,0xE0E0,0xCFCF,0xBFBF,0xD0D0,0xC0D0,0xB1C2,0x81C2,0x7161,0x7272,0x6261,0xB272,0xD1D1,0xC1C1,0xB2A1,0x92B1,0x9282,0xC1B2,0xC1B1,0x9282,0x8282,0x8282,0x8282,0x8282,0x8282
.word 0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8E50,0xCE9E,0xCFCF,0x9F9F,0x9F9F,0xBFBF,0xBFBF,0x41A1,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x8F8F,0x9F9F,0x484A,0xAFAF,0xBFBF,0x9F9F,0x9F9F,0x608E,0xAFAF,0x8E70,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x7070,0xAFAF,0x808E,0xCFCF,0xCFCF,0xDFDF,0xD5F0,0xAFAF,0x7E7D,0x8F8F,0xAFAF,0xAFAF,0xD050,0xCFCF,0x8F8F,0xAFAF,0x7F7F,0x7F7F,0x6050,0x9F9F,0x9F9F,0x708E,0xAFAF,0x709E,0xAFAF,0x7060,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x9E9E,0xAFAF,0xAFAF,0x9E9E,0x709E,0xAFAF,0x6070,0x7060,0xAFAF,0x7070,0x7070,0x7070,0x80C0,0x8080,0x7070,0xAFAF,0xAE9E,0x709E,0xAFAF,0xCFCF,0xBFBF,0xBFBF,0x6050,0xBFBF,0xD070,0x80F0,0xAFAF,0x6080,0x7060,0x7050,0x8080,0xAFAF,0x8E50,0x6050,0x5261,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xD3E3,0x9081,0x6F6F,0x7F7F,0x6F6F,0xD181,0xE0E0,0xCFCF,0xC0D0,0xC0C0,0xB2C0,0xB2B2,0xC0B1,0x7182,0x7251,0x7271,0x8272,0xC292,0xD1D1,0xA1C1,0x9282,0xB2B2,0x8282,0xB1A1,0xB2C2,0x82B1,0x7272,0x7272,0x7272,0x7282,0x8282
.word 0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x6060,0x9F9F,0x7060,0xBFBF,0xBE6E,0xC3E3,0x81B1,0xAFAF,0xBFBF,0x8F8F,0x8F8F,0x4242,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x5060,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xBFBF,0xBFBF,0xBFBF,0xD4C7,0xD6D4,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x7070,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9E70,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xEFEF,0x80D0,0x40B0,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x709E,0xAFAF,0x709E,0xAFAF,0x7070,0xAFAF,0xAFAF,0x9E9E,0x9E9E,0xAFAF,0xAFAF,0x7070,0x8F8F,0x7070,0x7070,0x7070,0x7070,0xC070,0x8080,0x7080,0x7070,0x7070,0x7080,0xAFAF,0x7070,0xCFCF,0x9E70,0x9E9E,0xAFAF,0x70CE,0x7050,0xE0FE,0x8090,0xAFAF,0x6080,0x6060,0x9F9F,0x9F9F,0x4050,0x8F8F,0x6161,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0xEFEF,0x7F7F,0x7F7F,0x6F6F,0xC171,0xD0E0,0xBFBF,0xAFAF,0xC0C0,0xA181,0xB192,0xB1B1,0x72C0,0x9F9F,0x7162,0x7272,0xB2B1,0xD192,0xC1D1,0x72B1,0x8272,0xB1B2,0x8292,0xB292,0xB2B2,0xB2C2,0x7172,0x7172,0x7172,0x7272,0x7271
.word 0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0x9F9F,0x6060,0x9F9F,0x9E9E,0x6060,0x9F9F,0x9F9F,0x709E,0x8F8F,0x7F7F,0xCFCF,0xBFBF,0x9F9F,0xBFBF,0x8F8F,0x6F6F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x7180,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0xBB4A,0xDFDF,0xAFAF,0xDBE6,0xF4FE,0x9F9F,0x8F8F,0x6050,0xAFAF,0xAFAF,0x7070,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x7F7F,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0xFEFE,0xB0FE,0x9F9F,0x4140,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x5060,0x8E70,0xAFAF,0xAFAF,0xAFAF,0x9E70,0x8E70,0x7070,0x7070,0x7070,0xAFAF,0xAFAF,0x7070,0x7070,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9EC0,0x7070,0x7070,0x7070,0xC080,0x8080,0x7080,0x7070,0x9E70,0x8080,0x8E70,0x9E9E,0xD0D0,0x8E90,0x9E70,0x609E,0xBFBF,0x7060,0xDFDF,0xCFCF,0x7070,0xAFAF,0x6060,0x8F8F,0x4040,0x5141,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0xD241,0xCFCF,0xC0C0,0xB0C0,0xA1B0,0x9281,0xB1B2,0xC1B1,0x71D0,0x81C0,0x5181,0x7172,0x9282,0xA1B1,0xD0B2,0xC1E0,0x6281,0x8271,0xC2B2,0x8292,0x8282,0xB292,0xB292,0x82B2,0x7162,0x6272,0x7172,0x7272
.word 0x7F7F,0x8F8F,0x5050,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x7E50,0x9F9F,0x9F9F,0x6060,0x6060,0x5050,0x9F9F,0x9F9F,0x9E70,0x8F8F,0x7F7F,0xD3DE,0xAFAF,0x4862,0xAFAF,0xAFAF,0x4271,0x8F8F,0x7F7F,0xAFAF,0x7F7F,0x9F9F,0x9F9F,0xC0C0,0xBFBF,0x9F9F,0x7F7F,0x7F7F,0xAFAF,0x9C6A,0xAFAF,0xCFCF,0xCFCF,0x8F8F,0xAFAF,0x7080,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xBFBF,0xAFAF,0x6060,0x9F9F,0x5060,0x8F8F,0xAFAF,0x9F9F,0x8F8F,0xAFAF,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0xDFDF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0x6050,0x708E,0x709E,0x9F9F,0xAFAF,0x7070,0x709E,0x7070,0xAFAF,0x708E,0x708E,0xAFAF,0xAFAF,0x9E9E,0xAFAF,0xAFAF,0x9E9E,0x9E70,0x9E9E,0xAFAF,0xAFAF,0x7070,0xAFAF,0x8080,0x7070,0x7070,0x8070,0x8080,0x7070,0xAFAF,0xE0E0,0xCFCF,0x9E9E,0x70AE,0xBE60,0x6080,0x8070,0xCFCF,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xFFFF,0xFFFF,0x8181,0xB281,0xA1A1,0xC0B1,0x9F9F,0x8F8F,0xAFAF,0xC371,0x8F8F,0x7171,0x8272,0x82A1,0xE0B1,0xB1D1,0x6162,0x6262,0xC292,0x7292,0x7282,0x8282,0x9292,0xB1B2,0x7292,0x7162,0x7172,0x7272
.word 0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4050,0xAFAF,0x709E,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xBFBF,0xDE7E,0xD1F1,0xCFCF,0xC2E0,0xAFAF,0xB0C0,0x6F6F,0x7F7F,0xAFAF,0x4161,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x7F7F,0x8F8F,0x9F9F,0xD667,0xBEBC,0x45CB,0xAE6C,0x8F8F,0xAFAF,0x9E9E,0xBFBF,0x9E80,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x8F8F,0x4A42,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0xDE9E,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0xAFAF,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0x7070,0x9F9F,0x9E70,0x7070,0xAFAF,0xAFAF,0xAFAF,0x8080,0x7070,0x9F9F,0xBFBF,0x6070,0x709E,0x7060,0xAFAF,0x709E,0x7070,0x7070,0x7070,0x7070,0xAFAF,0x7090,0x8E70,0x7070,0xC080,0xBFBF,0x9E9E,0xAEBE,0xAFAF,0x80AE,0x6070,0xAFAF,0x709D,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0xFFFF,0xFFFF,0x81A1,0x71C0,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x7170,0x81C0,0x9F9F,0x7172,0x8172,0xB2B2,0xE0C1,0x71D0,0x6252,0x6161,0xB282,0x7282,0x7271,0x7272,0x8282,0x9292,0x9292,0x6272,0x6271,0x7172
.word 0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x5050,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x6050,0xAFAF,0x9F9F,0x6060,0x9F9F,0x6070,0x9F9F,0xAFAF,0x9E70,0xC09E,0xFE5C,0xDFDF,0xDED0,0xD1F0,0x80A0,0xBFBF,0x61B0,0x7F7F,0x9F9F,0x7F7F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0xEABB,0x48D5,0x9F9F,0x7F7F,0x9F9F,0x7070,0xAFAF,0xBFBF,0xAFAF,0x8E70,0xAFAF,0x6070,0x9F9F,0x9F9F,0x5060,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x4060,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x7060,0x709E,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0x9E9E,0x5060,0x9E9E,0x9F9F,0x7070,0x7070,0x7070,0x7070,0x7070,0x7070,0x809E,0x7080,0x7070,0x8080,0x9080,0xD0E0,0xAFAF,0x9E9E,0x709E,0xBFBF,0x6070,0x8F8F,0x7E50,0x4040,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x508C,0x8F8F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xC071,0x4151,0x7182,0x8271,0xC1C1,0xE0E1,0x61C1,0x6161,0x6252,0xC272,0x62B2,0x7172,0x7272,0x7271,0x8272,0x9282,0x92B2,0x7162,0x7272
.word 0x5050,0x8F8F,0x9F9F,0x5050,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x6060,0x708E,0x9F9F,0x9F9F,0x9F9F,0x6060,0x8E70,0x9F9F,0x9F9F,0x8E70,0xBFBF,0xDFDF,0xE0F0,0x80D0,0xAFAF,0x8070,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x4050,0x5040,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xAFAF,0x8E70,0xBFBF,0xAFAF,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0xCFCF,0xAFAF,0x6070,0x9F9F,0x608E,0x8F8F,0x9F9F,0x9F9F,0x7060,0x707E,0xAFAF,0x6070,0x9F9F,0xAFAF,0x6070,0x5050,0x6060,0xAFAF,0xAFAF,0x708E,0x7070,0xAFAF,0x7070,0x9E70,0x8070,0x7070,0x7070,0x7070,0xBFBF,0xC0C0,0xAFAF,0x709E,0x7080,0x9E70,0x508E,0x7F7F,0x4040,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6161,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xBFBF,0x81C3,0x9F9F,0xC371,0x7272,0xE1D1,0xFFFF,0x52D0,0x5261,0x6162,0xC262,0xBFBF,0x6271,0x7172,0x7262,0x7271,0x8272,0xEFEF,0x72C3,0x7271
.word 0x8F8F,0x8F8F,0x5050,0x9F9F,0x9F9F,0x6060,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x9F9F,0x8E70,0x9F9F,0x5050,0x6060,0x9F9F,0xAFAF,0x8F8F,0x4050,0x9F9F,0xAFAF,0x9E8E,0xDFDF,0xAFAF,0x6070,0xAFAF,0x9F9F,0x4060,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0xCFCF,0x70C0,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7040,0x5060,0x9F9F,0x5050,0x8F8F,0xCEC0,0x8F8F,0xAFAF,0x7050,0xAFAF,0x8F8F,0x6050,0x6060,0x8F8F,0x8F8F,0x8F8F,0x8E50,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x9E70,0x7070,0x7070,0xAFAF,0x7070,0xAFAF,0xBFBF,0xAEC0,0x7070,0x708E,0x7050,0x6070,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0xB272,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x81C0,0xC080,0x5250,0x7181,0x7271,0xFFFF,0xCDEC,0x9F9F,0x5261,0x6161,0x6162,0x7172,0x6262,0x6271,0x7172,0x6162,0x7162,0xCFCF,0x7172,0x7172
.word 0x5050,0x6060,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x6060,0x9F9F,0xAFAF,0x8E70,0x7070,0x5060,0x9E7E,0x7070,0xAFAF,0x9E9E,0x7070,0xAFAF,0x7070,0x6070,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x4050,0x4040,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x7F7F,0x424A,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xAFAF,0x9F9F,0x8E70,0x9F9F,0xAFAF,0xBFBF,0x9F9F,0xAFAF,0x9F9F,0x7F7F,0x8F8F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7050,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4050,0x9F9F,0x9F9F,0x8F8F,0xBFBF,0x6070,0x6050,0x9F9F,0xAFAF,0x8F8F,0x5040,0x4040,0x8F8F,0x8F8F,0x5040,0x6060,0xAFAF,0x7070,0x709E,0xAFAF,0xAFAF,0x9E70,0x7070,0x7060,0x709E,0x9E70,0x9E9E,0x9080,0x9E80,0x709E,0x7070,0x5050,0x5050,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xCFCF,0x6F6F,0x6F6F,0x7F7F,0x7140,0xAFAF,0x71C0,0x7161,0x7281,0xC071,0xBFBF,0x9F9F,0x6162,0x6152,0x6252,0x6261,0x6162,0x7162,0x7162,0x6262,0x6271,0x6271,0x7162,0x7172,0x6272
.word 0x5050,0x9F9F,0x5060,0x9F9F,0x9E60,0x5080,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x5050,0x8F8F,0x6060,0xAFAF,0x6070,0x9F9F,0x6070,0x6060,0x6070,0x9E9E,0x6080,0x7E50,0xCFCF,0x709E,0x6060,0x9F9F,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x7060,0x9F9F,0xBFBF,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0x7E4E,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x709E,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x9F9F,0x80D0,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x8F8F,0x5040,0x4050,0x507E,0x6040,0x5060,0x5050,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x7070,0xAFAF,0x7090,0x709E,0x709E,0x709E,0x7060,0x8E70,0x708E,0x8E70,0xAFAF,0x7070,0x9E70,0x6060,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x70C0,0xBFBF,0x7172,0x7271,0xAFAF,0x6161,0x5162,0x5251,0x6151,0x5261,0x5261,0x6261,0x6271,0x6262,0x6271,0x7162,0x7162,0x6272,0x7271,0x7271
.word 0x9F9F,0x5060,0x9F9F,0x9F9F,0x8E70,0x6060,0x9F9F,0xAFAF,0x9F9F,0x4050,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x7070,0x7070,0x5070,0x6060,0xAFAF,0x9E60,0xAFAF,0x8F8F,0x4040,0x9E8E,0x6070,0x9F9F,0x6050,0x9F9F,0x6060,0x8F8F,0x4040,0x8F8F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0xAFAF,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0xAFAF,0x8F8F,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0xAFAF,0xBFBF,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4040,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4040,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7070,0x709E,0x9E60,0xAFAF,0x7070,0xAFAF,0x7070,0x9E9E,0x6070,0x8F8F,0x8F8F,0x7070,0x5050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xAFAF,0xAFAF,0x7170,0xC371,0x8172,0xAFAF,0xBFBF,0x5261,0x6161,0x5261,0x5152,0x6162,0x6162,0x6162,0x6171,0x7162,0x6271,0x6271,0x7171,0x7162,0x7172
.word 0x5050,0x8F8F,0xAFAF,0xAFAF,0x7060,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x9F9F,0x6070,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x7070,0xAFAF,0x7070,0x7070,0x4060,0x8F8F,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0x5050,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0xAFAF,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x9E8E,0xAFAF,0xBFBF,0x9F9F,0x8F8F,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x706E,0x5070,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6040,0x7060,0x709E,0x7060,0x9E60,0x709E,0x6060,0x8F8F,0x7F7F,0x4040,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4040,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xB0C0,0xAFAF,0x71C0,0x7171,0xD061,0x9F9F,0x5261,0x5261,0x6152,0x6161,0x5261,0x6261,0x6162,0x6262,0x6261,0x7162,0x7172,0x7262,0x7271,0x7262
.word 0x5040,0x6050,0x6060,0x9F9F,0x9F9F,0x6070,0x9F9F,0x8E60,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x6070,0x6060,0xAFAF,0x9E80,0x708E,0xAFAF,0x9E70,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x6090,0x708E,0xAFAF,0x61C4,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x8F8F,0x7F7F,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x9F9F,0x59C5,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0x707E,0x7070,0x9F9F,0xAFAF,0x8F8F,0x9F9F,0xAFAF,0x7F7F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0x8090,0x6070,0x6060,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x5050,0x6060,0x6070,0x7070,0x6070,0x5050,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xAFAF,0xB080,0x6171,0x6171,0x9F9F,0x5261,0x5161,0x6152,0x6251,0x6152,0x6152,0x6162,0x6261,0x6161,0x7172,0x7271,0x7162,0x7172,0x7172,0x7171
.word 0x9F9F,0x9F9F,0x9F9F,0x6060,0xAFAF,0x9F9F,0x7060,0x6060,0xAFAF,0xAFAF,0xAFAF,0x7070,0x8E70,0xAFAF,0x7080,0x7070,0x6070,0x6070,0xAFAF,0x7070,0x708E,0xAFAF,0x9E80,0x7070,0x9F9F,0x8F8F,0x9F9F,0x6070,0x5060,0x5050,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x9F9F,0x5060,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x9F9F,0xAFAF,0xAFAF,0xD0AE,0x40B0,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0x6180,0xAFAF,0x6171,0xAFAF,0x6161,0x5161,0x6152,0x5261,0x5161,0x5261,0x6161,0x6152,0x6162,0x6262,0x6261,0x6271,0x6271,0x6271,0x6271,0x6272
.word 0x5060,0x8F8F,0x9F9F,0x6060,0x8E70,0x5060,0x9F9F,0xAFAF,0x9E70,0x9E70,0x7070,0xAFAF,0x7070,0xAFAF,0xAFAF,0x6080,0x6060,0x708E,0x7070,0x9F9F,0xAFAF,0xAFAF,0x9E70,0xAFAF,0x5070,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xAFAF,0xBFBF,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0x7070,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x9F9F,0xAFAF,0xCFCF,0x9E9E,0xAFAF,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x70D0,0xC080,0x61C3,0x9F9F,0x5152,0x6252,0x5251,0x6161,0x6152,0x6152,0x6252,0x6261,0x6261,0x7161,0x7162,0x6262,0x7271,0x6271,0x7271,0x7171
.word 0x9F9F,0xAFAF,0x6070,0x9F9F,0x9F9F,0x5060,0x9F9F,0x9F9F,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x9E70,0xAFAF,0x7070,0x7060,0x709E,0xAFAF,0x7070,0x608E,0x9F9F,0x9F9F,0x5060,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x708E,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xAFAF,0x6171,0x9F9F,0x5261,0x5161,0x6152,0x5251,0x6261,0x6151,0x5161,0x5162,0x6162,0x6262,0x6162,0x7162,0x7162,0x7162,0x7162,0x7262
.word 0x6040,0x9F9F,0x9F9F,0x9F9F,0x6060,0x5050,0x5050,0x8F8F,0x9F9F,0x6060,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x6070,0x9F9F,0xAFAF,0x709E,0x6070,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x5070,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x6F6F,0x6F6F,0x4142,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x6070,0x8F8F,0x8F8F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6050,0xAFAF,0xAFAF,0xAFAF,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7180,0x8171,0x6171,0x9F9F,0x5151,0x5152,0x5261,0x6161,0x5152,0x5261,0x6152,0x6152,0x6162,0x7162,0x6271,0x7171,0x7172,0x6272,0x7271,0x7171
.word 0xAFAF,0x9F9F,0xAFAF,0x7E60,0x5060,0x9F9F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x7070,0x709E,0x7070,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0x7070,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5050,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0x8F8F,0xBFBF,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0x8F8F,0x7070,0x709E,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xC371,0x5182,0x5141,0x5251,0x5251,0x5161,0x5152,0x6261,0x6151,0x5261,0x6161,0x6262,0x6261,0x7162,0x6262,0x6271,0x7171,0x7162,0x6262
.word 0x9F9F,0x7060,0x9F9F,0x5060,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x708E,0x709E,0x9F9F,0xAFAF,0x7070,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0x5060,0x4040,0x6F6F,0x6F6F,0x7F7F,0x4050,0x8F8F,0x4040,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0xAE7E,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x7070,0x9F9F,0x8070,0x5060,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7170,0x7261,0x8F8F,0x8F8F,0x5151,0x5151,0x6152,0x5251,0x5151,0x5162,0x6152,0x6261,0x6161,0x7162,0x6271,0x7171,0x7162,0x6272,0x7271,0x7171
.word 0x9F9F,0x9F9F,0x9F9F,0x5060,0x5050,0x8F8F,0x5060,0x8F8F,0x7060,0xB08E,0x6050,0x6060,0x9F9F,0xAFAF,0xAFAF,0x7070,0xAFAF,0xAFAF,0xAFAF,0x7070,0x6070,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0x9F9F,0xAFAF,0xCFCF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6060,0x4040,0x7F7F,0x4060,0x6050,0x6070,0x6060,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xAFAF,0x7F7F,0x7F7F,0x5251,0x5251,0x5151,0x6152,0x6251,0x5251,0x6161,0x6262,0x6261,0x6171,0x6262,0x6271,0x7162,0x7162,0x7272,0x7262
*/


/* Pond
Fish_320:
.word 0x8061,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7080,0x7080,0x7080,0x7080,0x7080,0x7080,0x7080,0x7080,0x7080,0x7080,0x7080,0x7080,0x7080,0x7080,0x7080,0x7080,0x8070,0x7070,0x8070,0x8070,0x7070,0x8070,0x8060,0x8070,0x7070,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8050,0x8070,0x8070,0x8070,0x8070,0x7060,0x8070,0x7070,0x7060,0x8060,0x7060,0x7060,0x7060,0x7060,0x8070,0x8070,0xBFBF,0xBFBF,0x7070,0x7060,0x7060,0x7060,0x7060,0x7060,0x7060,0x7060,0x5060,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6141,0x7180,0x7190,0x7190,0x7180,0x7190,0x7190,0x9090,0x9090,0x9071,0x9090,0x9090,0x9090,0x8071,0x5051,0x7070,0x7080,0x9080,0x9090,0x8070,0x7090,0x7080,0x9080,0x8070,0x8070,0x8070,0x8070,0x9070,0x9070,0x8071,0x7090,0x7090,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x7080,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x6060,0x7060,0x7070,0x7080,0x9070,0x8070,0x7070,0x6070,0x6080,0x7060,0x7070,0x7070,0x7070,0x8070,0x6070,0x7070,0x7060,0x8060,0x6070,0x6070,0x7070,0x7070,0x5080,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x8060,0x6060,0x6070,0x6070,0x7070,0x7060,0x6060,0x7080,0x6070,0x6070,0x6070,0x6070,0x6050,0xAFAF,0xBFBF,0xBFBF,0x7080,0x7060,0xBFBF,0x6070,0x6070,0x6070,0x6070,0x5070,0x6060,0x6070,0x5070,0x5050,0x5040,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x7F7F,0x9F9F,0x5050,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x6F6F,0x7061,0x8070,0x8080,0x8080,0x9070,0x9070,0x8070,0x9070,0x7070,0x8070,0x7070,0x8080,0x9F9F,0x6F6F,0x7060,0x8060,0x7070,0x7180,0x7090,0x9070,0x9070,0x7070,0x7080,0x9070,0x8070,0x9070,0x7070,0x7080,0x7070,0x9070,0x8F8F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F4F
.word 0x5070,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0x6F6F,0x7F7F,0x6070,0x7060,0x7070,0x7080,0x6080,0x6080,0x8070,0x7060,0x7080,0x8070,0x7080,0x8080,0x6070,0x7060,0x7060,0x7080,0x7080,0x7070,0x7070,0x6060,0x6070,0x5060,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5050,0x6060,0x7060,0x7060,0x6060,0x7060,0x6080,0x8060,0x9F9F,0x8070,0x7060,0x6060,0x7060,0x7060,0x9F9F,0x8080,0xAFAF,0x6060,0xBFBF,0x6060,0x7060,0x6060,0x7060,0x6060,0x6050,0x6070,0x5060,0x7060,0x6060,0x8060,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5060,0x4040,0x4050,0x4040,0x4050,0x5050,0x5050,0x5040,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x7060,0x8070,0x8080,0x9090,0x9080,0x8090,0x9090,0x9070,0x8090,0x7090,0x80C0,0x7090,0x6F6F,0x6F6F,0x7070,0x7080,0x7080,0x9080,0x8080,0x8070,0x8070,0x9070,0xAFAF,0x7070,0x8080,0x7080,0x9080,0x9070,0x9070,0x6180,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x5040,0x5040,0x5040,0x7050,0x7060,0x7050,0x7060,0x7060,0x8F8F,0x7F7F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F5F
.word 0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0x7060,0x7060,0x7070,0x8070,0x7060,0x7070,0x7070,0x7070,0x8070,0x7080,0x7060,0x7070,0x7080,0x7080,0x6080,0x5050,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5060,0x6060,0x5060,0x6070,0x6070,0x7060,0x7070,0x7060,0x7080,0x7070,0x6070,0x6060,0x7060,0x6070,0x5070,0x8060,0x7080,0x7070,0x7060,0x6060,0x7060,0x5070,0x5070,0x6070,0x6070,0x7070,0x6060,0x7060,0x6070,0x7080,0x7080,0x5080,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x5050,0x5050,0x4050,0x6050,0x5040,0x5040,0x5060,0x5050,0x5050,0x4050,0x6F6F,0x5F5F,0x4F4F,0x6F6F,0x7F7F,0x9090,0x9071,0x8070,0x9070,0x9070,0xBFBF,0x8080,0x7090,0x9080,0x8070,0x4070,0x5F5F,0x6F6F,0x6070,0x8080,0xBFBF,0x8080,0x7080,0x7070,0x7080,0x7070,0x8080,0xBFBF,0x8070,0x7070,0x7090,0x9070,0x8080,0x5050,0x6F6F,0x6F6F,0x5040,0x6040,0x6060,0x5050,0x5060,0x4050,0x5050,0x5050,0x6070,0x6070,0xAFAF,0x8060,0x8060,0x8070,0x6060,0x5070,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F4F
.word 0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x8F8F,0x6070,0x7060,0x7080,0x6080,0x8060,0x8070,0x7090,0x7080,0x6060,0x6070,0x8070,0x7070,0xAFAF,0x7F7F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x6070,0x6070,0x6070,0x7060,0x7060,0x6060,0xAFAF,0x7070,0x7070,0x6080,0x7060,0x6070,0x6070,0x9F9F,0x5070,0x6070,0x8070,0x7070,0x7060,0x7060,0x6060,0x6060,0x7060,0x6060,0x7060,0x6060,0x7070,0x7070,0x8060,0x6060,0x6070,0x7080,0x4070,0x5F5F,0x5040,0x6F6F,0x4050,0x4050,0x5050,0x4050,0x5040,0x4040,0x4050,0x4050,0x5040,0x4050,0x5060,0x6050,0x5040,0x5050,0x5F5F,0x4F4F,0x5F5F,0x6140,0x8090,0x9090,0x8090,0x7080,0x9080,0x8080,0x9070,0x8080,0x6190,0x5F5F,0x5F5F,0x6050,0x7080,0x8070,0x7070,0x7090,0x7090,0x7090,0x7070,0x7080,0x7070,0x7070,0x7070,0x8070,0x8070,0x6190,0x8F8F,0x6F6F,0x5050,0x5040,0x5040,0x6050,0x5060,0x6050,0x5050,0x6050,0x5050,0x6060,0x5050,0x6060,0x6050,0x5060,0x5070,0x6070,0x7060,0x6060,0x6070,0x5050,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F5F
.word 0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x7F7F,0x8F8F,0x9F9F,0x6060,0x7060,0x7070,0x7060,0x6060,0x5060,0x6060,0x5050,0xAFAF,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6050,0x7060,0x6060,0x7070,0x6070,0x6080,0x9F9F,0x7080,0x9F9F,0x9F9F,0x8070,0x6070,0x7060,0x7060,0x6080,0x7060,0x8050,0x8070,0x8070,0xAFAF,0x9F9F,0xAFAF,0x7060,0x9F9F,0x7080,0x9F9F,0x6080,0x6070,0x8060,0x6070,0x7080,0x7080,0x8070,0x6070,0x5F5F,0x6F6F,0x4040,0x4050,0x4050,0x5050,0x5050,0x5050,0x4060,0x4050,0x5050,0x4040,0x4050,0x5050,0x5040,0x5060,0x5060,0x5050,0x6F6F,0x5F5F,0x7F7F,0x6050,0x7180,0x9070,0x8070,0x7070,0x8080,0x7090,0xBFBF,0x7F7F,0x5F5F,0x4F4F,0x6F6F,0x8070,0x8070,0x9080,0x7080,0x8080,0x8070,0x9070,0x7070,0x7080,0x7080,0x9080,0x8070,0xAFAF,0x8F8F,0x5040,0x5040,0x5040,0x5040,0x5060,0x5060,0x7060,0x6050,0x5060,0x5060,0x6060,0x9F9F,0x7070,0x9F9F,0x7060,0x6060,0x6050,0x7050,0x6050,0x6070,0x7060,0x6060,0x4070,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F5F
.word 0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5050,0x7050,0x6070,0x7060,0x6060,0x8070,0x9F9F,0x7080,0x6070,0x8060,0x8070,0x6070,0x6070,0x6080,0x6070,0x6060,0x9F9F,0x7060,0x8090,0xBFBF,0xBFBF,0x8070,0x7070,0x9F9F,0x8080,0x7060,0x8080,0x9F9F,0x8070,0x7070,0x8070,0x7060,0x7060,0x7080,0x6090,0x5F5F,0x6F6F,0x5050,0x5040,0x5040,0x6050,0x6050,0x5050,0x5040,0x5050,0x4040,0x5050,0x5040,0x4040,0x5050,0x5040,0x6040,0x5050,0x5060,0x5F5F,0x5F5F,0x7F7F,0x8F8F,0xAFAF,0x8070,0x8080,0x7090,0x90C0,0x5070,0x4F4F,0x4F4F,0x5F5F,0x7F7F,0x7090,0x8080,0x7080,0x7090,0x7090,0x7080,0x7080,0x9090,0x8080,0x7090,0xBFBF,0xAFAF,0x4040,0x5050,0x5040,0x5040,0x5050,0x6050,0x6050,0x6070,0x6060,0x5060,0x7060,0x7050,0x9F9F,0x7060,0x5060,0x5070,0x5050,0x6060,0x7050,0x6050,0x5070,0x6060,0x6070,0x7070,0x7060,0x5050,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F4F
.word 0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6070,0x6060,0x8070,0xAFAF,0x6070,0x8070,0x7060,0x9F9F,0x6070,0x6060,0x6060,0x6070,0x9F9F,0x6070,0x7070,0x7060,0x9080,0x80BE,0x8080,0xBFBF,0x7080,0x9F9F,0x8060,0x6060,0x8060,0x6060,0x7080,0x6060,0x9F9F,0x6060,0x7070,0x7070,0x7070,0x7090,0x5F5F,0x6F6F,0x4040,0x4050,0x4050,0x5060,0x5050,0x5050,0x5050,0x4050,0x5050,0x5050,0x5050,0x5040,0x4050,0x4050,0x5050,0x6050,0x5050,0x5050,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0xAFAF,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0xBFBF,0x7090,0x8070,0x9070,0x7070,0x7090,0x8080,0x8080,0x7190,0x7090,0x4050,0x6F6F,0x6F6F,0x4040,0x5050,0x5060,0x7060,0x6070,0x7060,0x9F9F,0x9F9F,0x6050,0x9F9F,0x5060,0x7060,0x6070,0x9F9F,0x6050,0x7060,0x9F9F,0x5070,0x7060,0x7050,0x7060,0x5050,0x5060,0x6050,0xAFAF,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5050,0x5040,0x5040,0x7050,0x7060,0x7080,0x7080,0x7080,0x8080,0x8060,0x8060,0x8070,0x7070,0x6070,0x9F9F,0xAFAF,0x8060,0x6080,0x6060,0x9F9F,0xAFAF,0x80A0,0x9080,0xBE80,0xBFBF,0x8070,0x9F9F,0x8070,0x7060,0x7060,0x7060,0x7060,0x8060,0x6080,0x9F9F,0x8070,0x8080,0xBFBF,0x6F6F,0x5050,0x4050,0x4050,0x4050,0x6050,0x6050,0x5050,0x5050,0x5050,0x5050,0x4060,0x5050,0x5050,0x5040,0x5040,0x5040,0x4040,0x5050,0x5060,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x7080,0x7070,0x7090,0x7090,0x9090,0x7070,0x7070,0xAFAF,0x5060,0x6F6F,0x4F4F,0x5F5F,0x5050,0x5050,0x7050,0x7060,0x6060,0x9F9F,0x5070,0x6070,0x6070,0x9F9F,0x6070,0xAFAF,0x6060,0x7060,0x5060,0x9F9F,0xAFAF,0xAFAF,0x6070,0xAFAF,0x5060,0x9F9F,0x7050,0xAFAF,0xAFAF,0x6060,0x5070,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F4F
.word 0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5040,0x5040,0x5040,0x5040,0x6040,0x6070,0x7080,0x9F9F,0xBFBF,0x8070,0xAFAF,0x7080,0xBFBF,0x7070,0x7070,0x9F9F,0xBFBF,0x7070,0x9F9F,0x7070,0x7070,0x7080,0xD070,0xC090,0xAFAF,0xBFBF,0x70C0,0x7070,0x6080,0x9F9F,0x6070,0x6060,0x7060,0x9F9F,0x6080,0x7060,0x7080,0x6060,0x7070,0x9070,0x6F6F,0x4040,0x4050,0x4050,0x5050,0x5040,0x6070,0x7060,0x6050,0x6050,0x6050,0x6050,0x5050,0x5050,0x5050,0x5040,0x5040,0x4050,0x5050,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5050,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4040,0x7050,0x9F9F,0x7060,0x9F9F,0x6060,0x7060,0x7060,0x5060,0x7060,0x7060,0x6060,0xAFAF,0xBFBF,0xAFAF,0x6080,0x6060,0x7060,0x8F8F,0x6060,0x7050,0x5050,0x5060,0x7060,0x6060,0x5070,0x6060,0x4080,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F5F
.word 0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5060,0x4040,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x7050,0x7060,0x9F9F,0x8080,0x7080,0x7080,0x8070,0x9F9F,0x7070,0x9F9F,0x7070,0x9F9F,0x7080,0x6070,0xBFBF,0x9F9F,0x6080,0x7070,0xBFBF,0xAFAF,0x80BE,0xAFAF,0xBFBF,0x8060,0x9F9F,0x7070,0xBFBF,0x8070,0x9F9F,0x7080,0x6060,0x6070,0x6060,0x6070,0x8070,0x5070,0x5F5F,0x6F6F,0x5040,0x5040,0x4040,0x5050,0x9F9F,0x9F9F,0x5070,0x5070,0x5060,0x5050,0x5060,0x5050,0x5050,0x5050,0x4050,0x5040,0x4040,0x4050,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6050,0x6050,0x6070,0x6070,0x7060,0x7060,0x8F8F,0x9F9F,0x7060,0x6070,0x5060,0x6070,0x7060,0x8070,0xBFBF,0xAFAF,0x9F9F,0x6070,0x7060,0x7050,0x9F9F,0x7050,0x6060,0x5050,0x6050,0x7060,0x7050,0x5060,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F
.word 0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4150,0x6F6F,0x4150,0x6142,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x6040,0x5050,0x5050,0x4050,0x5050,0x5050,0x5050,0x5040,0x8F8F,0x6070,0x8060,0x6060,0x6060,0xBFBF,0x7080,0x7080,0x8060,0x7080,0x7080,0x8070,0x9F9F,0x6080,0x7070,0x9F9F,0x7060,0x6050,0xBFBF,0xC080,0x9080,0xBFBF,0x9070,0x7070,0x7080,0x6070,0x6070,0x6070,0x7070,0x6060,0x7060,0x9F9F,0x7060,0x7060,0xBFBF,0x6F6F,0x5F5F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x6060,0x7060,0x7060,0x6050,0x6060,0x6050,0x6050,0x6050,0x5050,0x4050,0x4050,0x5050,0x5040,0x4050,0x5050,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x6050,0x7060,0x8060,0x9F9F,0x6070,0x7050,0x7060,0x6060,0x9F9F,0x6060,0xAFAF,0x6060,0xAFAF,0xBFBF,0x6080,0x7070,0x7060,0x6060,0xAFAF,0x9F9F,0x5060,0x6070,0x9F9F,0x9F9F,0x5060,0x5050,0x6060,0x7050,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F5F
.word 0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x5150,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x6F6F,0x4050,0x5050,0x4040,0x5040,0x5040,0x5060,0x4050,0x7F7F,0x7050,0x7060,0x6070,0x8070,0xBFBF,0x7070,0x7060,0xBFBF,0x8070,0x7060,0x6080,0x6080,0x7080,0x7080,0x9F9F,0x7070,0xAFAF,0x4160,0xAFAF,0xBFBF,0xBFBF,0x7080,0xBFBF,0xAFAF,0xBFBF,0x9F9F,0x8070,0x7060,0x6070,0x9F9F,0x6080,0x8070,0xAFAF,0x8080,0x7080,0x6F6F,0x5F5F,0x4050,0x4050,0x4050,0x4050,0x4050,0x5050,0x6070,0x6060,0x7070,0x7060,0x5060,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x4050,0x5040,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x7F7F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5050,0x7050,0x6070,0x9F9F,0x8070,0x7060,0x6060,0x6070,0x6060,0x6070,0x5070,0x6060,0x6070,0x7060,0x7070,0x6060,0x7060,0x6060,0x6070,0x6070,0x7060,0xAFAF,0x6060,0x7050,0x7060,0x7060,0x9F9F,0x6060,0x6050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F
.word 0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5241,0x6262,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4242,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5040,0x5050,0x6050,0x5050,0x5040,0x5050,0x6060,0x7070,0x7070,0x8070,0x8060,0xBFBF,0x7070,0xAFAF,0x6080,0x8070,0x8060,0x8070,0x8070,0x7070,0xBFBF,0x9F9F,0x6060,0x9F9F,0xC070,0x9F9F,0xBFBF,0x7070,0x7080,0x7070,0x8080,0x7060,0x7060,0x8060,0x8060,0x8060,0x9F9F,0x8070,0xBFBF,0x7090,0x5F5F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x5040,0x8F8F,0xAFAF,0x9F9F,0x6070,0x6070,0x5070,0x5060,0x5060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5040,0x4040,0x5050,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x6070,0x9F9F,0x6070,0x6060,0x6070,0x6070,0x7050,0x5060,0x7060,0x7060,0x9F9F,0x7050,0x9F9F,0x7060,0x8080,0x6060,0x6070,0x9F9F,0x7060,0x9F9F,0x6060,0x9F9F,0x5060,0x5070,0x9F9F,0x7060,0x5060,0x6070,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F4F
.word 0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6141,0xBFBF,0xBFBF,0xE4E4,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4141,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5040,0x4050,0x5050,0x5040,0x5040,0x6F6F,0x6F6F,0x5040,0x5040,0x6F6F,0x5040,0x6F6F,0x5050,0x5060,0x5060,0x4050,0x4050,0x6050,0x7070,0xBFBF,0xBFBF,0x9F9F,0x7070,0x7070,0x7080,0x7080,0xAFAF,0x7070,0x7080,0x8060,0x9F9F,0x7070,0x7070,0x7080,0x7F7F,0x5050,0x9F9F,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0x9F9F,0xAFAF,0xBFBF,0x7060,0x6070,0x8070,0x7080,0x8090,0x4060,0x5F5F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x5050,0x6050,0x5050,0x9F9F,0xAFAF,0x8F8F,0x6050,0x6050,0x5050,0x5060,0x5050,0x5050,0x5050,0x4040,0x4050,0x5060,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6050,0x7060,0x6060,0x7060,0x7060,0x6060,0x9F9F,0x9F9F,0x7060,0x7050,0x6060,0x6070,0x9F9F,0x5060,0x9F9F,0x6060,0x8080,0x7060,0x6070,0xAFAF,0xAFAF,0xAFAF,0x6060,0x7070,0x6060,0x6060,0x6070,0x6070,0x6070,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F5F
.word 0x6F6F,0x6F6F,0x7F7F,0x4050,0x4050,0x6050,0xBFBF,0x7F7F,0x4141,0x6F6F,0xAFAF,0x6152,0x5142,0x5051,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5050,0x4050,0x5040,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x5050,0x6F6F,0x5F5F,0x5050,0x5040,0x5050,0x6050,0x5050,0x5060,0x4050,0x9F9F,0x9F9F,0x6080,0x8070,0x8070,0x8070,0x8060,0x7070,0x7070,0x7080,0x9F9F,0x6080,0x6080,0x7060,0x7070,0x8080,0x6080,0x6F6F,0x6F6F,0x4040,0x6070,0x8070,0xBFBF,0x70BE,0x7070,0x6070,0x8060,0x7060,0x6080,0x8070,0x9F9F,0x7070,0xAFAF,0x7070,0x5F5F,0x6F6F,0x5050,0x5040,0x5040,0x5040,0x5050,0x5050,0x5060,0x6050,0x6050,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x5050,0x5050,0x5040,0x5050,0x5040,0x5050,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5060,0x6060,0x7060,0x7060,0x7060,0x7060,0x7060,0x6060,0x6070,0x6070,0x6060,0x6060,0x5060,0x4050,0x6050,0x7060,0x9F9F,0x6070,0x7060,0x7070,0x9F9F,0x7070,0x6060,0x6060,0x9F9F,0x7060,0x9F9F,0x9F9F,0x6050,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F4F
.word 0x4050,0x5050,0x7050,0x5060,0xAFAF,0xAFAF,0xBE90,0xCEC0,0xCFCF,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x6050,0x5050,0x5050,0x5060,0x5050,0x5050,0x5050,0x4050,0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x4040,0x4050,0x6040,0x5050,0x5060,0x6050,0x5050,0x5040,0x6050,0x9F9F,0x6070,0x7070,0xAFAF,0x7080,0x8070,0x7070,0x7070,0x7090,0x7060,0x7060,0x7080,0x7080,0x7070,0xAFAF,0x5F5F,0x4050,0x5050,0x5050,0x6070,0x7070,0x7080,0x7060,0x6070,0x6070,0x7060,0x8060,0x6070,0x8080,0x8060,0x9070,0x5070,0x5F5F,0x6F6F,0x5040,0x5050,0x5050,0x5050,0x5060,0x5060,0x6050,0x5050,0x5060,0x5050,0x7F7F,0x7F7F,0x4050,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x5040,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5050,0x5070,0x6070,0x6070,0x6070,0x7060,0x6060,0x6070,0x7060,0x7070,0x7060,0x6060,0x6060,0x5050,0x8F8F,0x8F8F,0x7060,0x9F9F,0x6080,0xAFAF,0x7070,0x9F9F,0x7060,0x7060,0x7060,0x5060,0x6070,0x7060,0x5070,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F5F
.word 0x5040,0x7050,0x5050,0x6060,0x6060,0x7060,0x6060,0x8060,0xCFCF,0xAFAF,0xBFBF,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6040,0x6060,0x5070,0x5070,0x5050,0x6050,0x5040,0x5060,0x5050,0x5040,0x5040,0x4040,0x6F6F,0x4050,0x6F6F,0x6040,0x5050,0x5060,0x6050,0x5050,0x5050,0x5050,0x5050,0x5060,0x7050,0x6060,0x8070,0x7060,0x6060,0x6080,0x7070,0x7080,0x6070,0x6070,0x7070,0x7070,0x8080,0x6F6F,0x6F6F,0x4050,0x5040,0x6050,0x7060,0x6080,0x6060,0x9F9F,0x7080,0x9F9F,0x8060,0x6060,0x6080,0x6060,0x7080,0x7080,0x6F6F,0x6F6F,0x4050,0x5050,0x6050,0x6050,0x5060,0x6050,0x6050,0x5050,0x4060,0x6050,0x5050,0x6F6F,0x6F6F,0x7F7F,0x4040,0x5040,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x7F7F,0x7050,0x7060,0x6060,0x7060,0x6060,0x6070,0x7060,0x9F9F,0x7060,0x6060,0x5070,0x5070,0x5070,0x5050,0x6040,0x5050,0x6070,0x7060,0x6060,0x6070,0x6060,0x9F9F,0x7060,0x6050,0x6060,0x6050,0x6050,0x5060,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F4F
.word 0x6050,0x5050,0x7050,0x5050,0x5060,0x6060,0x7060,0x6070,0x6060,0xBFBF,0xDEC0,0x70DE,0xAFAF,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x5040,0x5050,0x5060,0x7050,0x7050,0x7060,0x5060,0x4050,0x5050,0x5050,0x5040,0x5040,0x6F6F,0x6F6F,0x5F5F,0x4050,0x5050,0x6050,0x5050,0x5070,0x6050,0x6050,0x6050,0x4050,0x5040,0x5050,0x6060,0x6070,0x7080,0xBFBF,0x6070,0x7080,0x6060,0x7070,0x6060,0x7080,0x8080,0x6070,0x5F5F,0x5F5F,0x7F7F,0x5050,0x5050,0x6070,0xAFAF,0x7060,0x6060,0x6070,0x6080,0x6080,0x8060,0x9F9F,0x8060,0xBFBF,0x6080,0x5F5F,0x6F6F,0x5040,0x5050,0x5050,0x5060,0x6050,0x5050,0x5050,0x5060,0x5050,0x4050,0x4050,0x5050,0x6F6F,0x4040,0x5050,0x4050,0x4050,0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x6050,0x7060,0x7060,0x7060,0x7060,0x7060,0x8060,0x6070,0x9F9F,0x7060,0x6060,0x6050,0x4050,0x5050,0x8F8F,0x7050,0x6060,0x7060,0x6060,0x6070,0x6060,0x6060,0x7050,0x5050,0x6070,0x6070,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F5F
.word 0x5050,0x5070,0x5070,0x6050,0x6050,0x5060,0x5060,0x5060,0x5060,0x6070,0x6060,0xC070,0xEFEF,0xBFBF,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x5040,0x5040,0x7050,0x6050,0x5070,0x5050,0x5060,0x5050,0x5060,0x5050,0x5050,0x4050,0x5040,0x5040,0x6F6F,0x6F6F,0x5040,0x5040,0x5050,0x6050,0x5050,0x5060,0x5050,0x5050,0x5040,0x5050,0x5040,0x8F8F,0x8060,0x8060,0x7070,0x8060,0x7060,0x7060,0x6060,0x7060,0x8070,0x9F9F,0x5060,0x5F5F,0x5F5F,0x5040,0x8F8F,0x7050,0x9F9F,0x7070,0x6080,0x7060,0x9F9F,0x8060,0x7060,0x7060,0x7060,0x6080,0x7080,0x5090,0x5F5F,0x6F6F,0x4050,0x5050,0x6050,0x5050,0x5060,0x5060,0x6050,0x6040,0x5050,0x5040,0x5050,0x4050,0x7040,0x6050,0x5050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5050,0x5050,0x6060,0x6070,0x6070,0x5070,0x7F7F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6050,0x7060,0x6070,0x6070,0x6080,0x6070,0x6070,0x6060,0x6050,0x6070,0x9F9F,0x5070,0x5070,0x6F6F,0x7F7F,0x5060,0x5060,0x6050,0x6060,0x7050,0x5050,0x6060,0x9F9F,0x6060,0x7050,0x5050,0x7060,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F4F
.word 0x5060,0x5050,0x6050,0x5060,0x5060,0x5060,0x6050,0x6050,0x5050,0x5050,0x7060,0x5050,0xCFCF,0xBFBF,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x5050,0x5050,0x5060,0x7050,0x7050,0x6050,0x5070,0x6050,0x6050,0x6050,0x5050,0x4050,0x4050,0x5040,0x4050,0x6F6F,0x4050,0x5040,0x5040,0x5050,0x5050,0x5050,0x6050,0x5050,0x4040,0x7F7F,0x5040,0x6060,0x7080,0x8080,0x6080,0x7070,0x6070,0x6070,0x7070,0x7070,0x8090,0x6F6F,0x6F6F,0x5050,0x4050,0x6050,0x5070,0x6070,0x7060,0x9F9F,0x6070,0x6070,0x9F9F,0x6080,0x6080,0x6080,0xBFBF,0x6070,0x7F7F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x4040,0x6050,0x5040,0x6050,0x4050,0x5050,0x5040,0x5040,0xBFBF,0xCFCF,0xAFAF,0x9F9F,0x5070,0x5070,0x5050,0x5040,0x4050,0x6F6F,0x5040,0x6F6F,0x6F6F,0x4050,0x5050,0x6060,0x7060,0x7060,0x6060,0x6080,0x4070,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5060,0x7060,0x6060,0x9F9F,0x9F9F,0x5080,0x7060,0x7050,0x7060,0x7060,0x7050,0x6050,0x6F6F,0x5F5F,0x5040,0x5050,0xAFAF,0x5070,0x6070,0x6070,0x6060,0x7070,0x6060,0x5060,0x5070,0x7F7F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F5F
.word 0x6040,0x6050,0x5050,0x6060,0x5050,0x5050,0x5050,0x5060,0x5060,0x6050,0x5050,0x5060,0x4050,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x5050,0x6050,0x6070,0x6070,0x6060,0x6050,0x6070,0x7050,0x5050,0x5060,0x5060,0x5050,0x4050,0x5050,0x4040,0x5050,0x4050,0x6F6F,0x4050,0x4050,0x5040,0x5050,0x5060,0x5060,0x6050,0x5050,0x5050,0x4050,0x5050,0x8060,0x6070,0x8070,0x6060,0x7060,0x8060,0x8060,0x6060,0x5070,0x5F5F,0x5F5F,0x4040,0x6F6F,0x8F8F,0x7060,0x7060,0x9F9F,0x6070,0x5070,0x6060,0x5060,0x8F8F,0x8F8F,0x7050,0x5060,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x5050,0x5050,0x4050,0x4050,0x4050,0xC050,0xC0BE,0xAFAF,0x7070,0x9F9F,0x6070,0x5060,0x4050,0x5040,0x5050,0x6F6F,0x4050,0x4050,0x4050,0x6050,0x5050,0x5050,0x6060,0x6070,0x7060,0x7060,0x4050,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x6040,0x9F9F,0x7070,0x7060,0x6070,0x7060,0x6060,0x5070,0x6050,0x6060,0x9F9F,0x5070,0x6F6F,0x5F5F,0x5050,0x8F8F,0x5060,0x7060,0x6060,0x7050,0x5060,0x6050,0x9F9F,0x8F8F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F
.word 0x4050,0x5050,0x7050,0x9F9F,0x5060,0x5060,0x5060,0x4050,0x5050,0x5050,0x5060,0x6050,0x5050,0x4050,0x5F5F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x7050,0x5050,0x5050,0x7050,0x7060,0x5060,0x5070,0x5070,0x5070,0x5050,0x5050,0x5070,0x5050,0x5050,0x5050,0x4040,0x4050,0x5050,0x6F6F,0x5040,0x5050,0x5050,0x5050,0x6050,0x4050,0x5060,0x6050,0x4050,0x5050,0x8F8F,0x5060,0x5060,0x7060,0x6070,0x9F9F,0x9F9F,0x6F6F,0x6F6F,0x5F5F,0x5040,0x5F5F,0x5F5F,0x5050,0x7F7F,0x7F7F,0x5050,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x5040,0x4050,0x6F6F,0x7F7F,0x5040,0x6070,0x6070,0x6070,0x6070,0x9F9F,0x5060,0x5040,0x4050,0x4040,0x5050,0x5040,0x5040,0x5040,0x5050,0x5060,0x6050,0x6050,0x5050,0x5070,0x5050,0x7060,0x5050,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x7F7F,0x5050,0x6050,0x6070,0x7060,0x7060,0x7060,0x7050,0x7050,0x5060,0x6070,0x5070,0x5F5F,0x5F5F,0x7F7F,0x5050,0x5060,0x6060,0x7050,0x5070,0x7060,0x5060,0x5040,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F
.word 0x5050,0x5050,0x5050,0x6050,0x5050,0x7050,0x6050,0x6050,0x5050,0x6050,0x5050,0x5050,0x5040,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x4040,0x4050,0x4050,0x5050,0x5050,0x5060,0x5060,0x5070,0x5050,0x5060,0x5050,0x7050,0x6060,0x6050,0x7050,0x6050,0x5070,0x5060,0x5070,0x5040,0x4040,0x4050,0x5040,0x6F6F,0x4040,0x5050,0x5060,0x5060,0x6050,0x5050,0x5060,0x6050,0x5050,0x5040,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5050,0x5050,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x6050,0x7060,0xAFAF,0x6060,0x6060,0x7050,0x5070,0x4040,0x5050,0x4040,0x6F6F,0x5040,0x5050,0x6050,0x5040,0x5050,0x5060,0x6050,0x5050,0x5070,0x6070,0x6080,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x8F8F,0x6050,0x7060,0x9F9F,0x6070,0x5060,0x6070,0x7050,0x7050,0x5060,0x5F5F,0x5F5F,0x7F7F,0x6050,0x6050,0x5060,0x6060,0x5050,0x7F7F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F
.word 0x4050,0x6050,0x6050,0x5050,0x5060,0x5060,0x5050,0x5050,0x5050,0x5060,0x5050,0x4040,0x4050,0x6F6F,0x4050,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x5050,0x5070,0x5060,0x6050,0x5050,0x6050,0x5070,0x9F9F,0x9F9F,0x9F9F,0x6060,0x5060,0x5060,0x5070,0x5050,0x7060,0x5050,0x6060,0x6050,0x5040,0x6F6F,0x4040,0x5050,0x5040,0x5050,0x6050,0x5050,0x6050,0x5050,0x5060,0x5070,0x5060,0x5050,0x5050,0x5060,0x5050,0x6F6F,0x5050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5040,0x6F6F,0x6050,0x5050,0x5060,0x5050,0x6050,0x5050,0x5050,0x5050,0x5040,0x5040,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5040,0x4040,0x5050,0x7050,0x6050,0x5070,0x6050,0x6060,0x7050,0x6050,0x4050,0x5050,0x6F6F,0x4050,0x5060,0x5060,0x5050,0x5040,0x5050,0x5060,0x6050,0x7050,0x7060,0x8060,0x6F6F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x7F7F,0x8F8F,0x5050,0x6050,0x7050,0x5050,0x6070,0x6060,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x4050,0x4050,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F5F
.word 0x6040,0x5050,0x5060,0x6060,0x6050,0x6050,0x6050,0x6050,0x5050,0x5040,0x5040,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x6050,0x5050,0x6060,0x5050,0x9F9F,0x9F9F,0x5060,0xAFAF,0x7050,0x7060,0x7060,0x7050,0x7050,0x6060,0x7050,0x7060,0x5070,0x6070,0x6070,0x5070,0x5060,0x4050,0x5040,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4050,0x5050,0x5060,0x5060,0x5050,0x8F8F,0x5050,0x5050,0x5060,0x6070,0x6070,0x4060,0x5050,0x6070,0x5050,0x5040,0x5040,0x5060,0x6F6F,0x5050,0x5070,0x5060,0x6050,0x5050,0x5060,0x5050,0x4050,0x4050,0x4050,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x5050,0x6F6F,0x6F6F,0x4040,0x5050,0x6050,0x5050,0x5060,0x6050,0x5060,0x7060,0x5060,0x5060,0x5060,0x4040,0x4050,0x5050,0x5070,0x6060,0x5040,0x5050,0x5050,0x6050,0x5050,0x6050,0x6070,0x7060,0x7070,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5040,0x5040,0x7F7F,0x5040,0x4040,0x5050,0x5060,0x7050,0x5050,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F
.word 0x5050,0x5060,0x5050,0x5070,0x5060,0x5050,0x5050,0x5060,0x5050,0x4050,0x4050,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5040,0x5040,0x5060,0x6050,0x5050,0x5060,0x6070,0x7050,0x5050,0x7060,0x5050,0x5050,0x5050,0x5050,0x5070,0x7060,0x6070,0x6050,0x7060,0x6050,0x5070,0x7060,0x7060,0x7050,0x5050,0x6050,0x5050,0x5040,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5040,0x5050,0x6050,0x5050,0x4050,0x5040,0x6050,0x5050,0x4050,0x5050,0x5050,0x5060,0x5050,0x5050,0x5070,0x5F5F,0x5050,0x7050,0x6060,0x6060,0x5060,0x6050,0x5050,0x6050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x8F8F,0x5050,0x5040,0x5040,0x5040,0x4050,0x5040,0x8F8F,0xBEC0,0xAFAF,0x5050,0x8F8F,0x5060,0x6050,0x5060,0x6070,0x5050,0x6050,0x5060,0x5040,0x5050,0x6050,0x7050,0x6050,0x5060,0x6050,0x5050,0x5060,0x5060,0x6050,0x6050,0x5050,0x7070,0xAFAF,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x4050,0x4050,0x4050,0x5050,0x5040,0x6F6F,0x6F6F,0x7F7F,0x4050,0x4050,0x5050,0x4050,0x5060,0x5050,0x5050,0x5050,0x6060,0x5070,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F5F
.word 0x6060,0x5050,0x6070,0x6050,0x5060,0x6070,0x6050,0x5050,0x5050,0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x7F7F,0x5050,0x9F9F,0x9F9F,0x7050,0x9F9F,0x9F9F,0x6070,0x9F9F,0x6060,0x7050,0x6070,0x6070,0x7060,0x6050,0x5060,0x6060,0x6070,0x6070,0x7060,0x7060,0x7060,0x6060,0x6070,0x7050,0x5060,0x5050,0x4050,0x5050,0x5050,0x5050,0x6F6F,0x6F6F,0x5040,0x4050,0x7F7F,0x4050,0x5050,0x7F7F,0x5050,0x5050,0x7F7F,0x6050,0x7060,0x7050,0x7060,0x5060,0x5F5F,0x5F5F,0x5060,0x6060,0x5060,0x5070,0x5060,0x5060,0x5060,0x5040,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6040,0x5050,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x4050,0x6050,0x8F8F,0xAFAF,0xCFCF,0xBFBF,0x7070,0xBFBF,0x5060,0x5060,0x7050,0x5050,0x5070,0x5060,0x6050,0x5050,0x8F8F,0x9F9F,0x5080,0x6060,0x5050,0x5060,0x5060,0x5050,0x5050,0x5060,0x5060,0x6050,0x5050,0x6060,0x7F7F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5050,0x5040,0x5040,0x6050,0x5050,0x6F6F,0x5F5F,0x6F6F,0x5040,0x5050,0x4050,0x6050,0x5050,0x5050,0x5050,0x4050,0x5050,0x6050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F4F
.word 0x6070,0x6070,0x6050,0x6060,0x5060,0x5050,0x5060,0x5050,0x5050,0x5050,0x5050,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5050,0x6050,0x7050,0xAFAF,0x6060,0xAFAF,0x9F9F,0x6070,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x6070,0x9F9F,0x7060,0x7060,0x7060,0x7060,0x6060,0x6070,0x6070,0x6070,0x6060,0x7060,0x7050,0x6070,0x5050,0x6050,0x5050,0x5050,0x5040,0x5040,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x4050,0x5040,0x5050,0x5060,0x7060,0x7050,0x5060,0x5F5F,0x5F5F,0x5040,0x6060,0x6070,0x7050,0x5050,0x5050,0x5050,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5040,0x5040,0x5060,0x5060,0xAFAF,0x6060,0xAFAF,0xAFAF,0x5060,0x7070,0x6060,0xAFAF,0x7060,0xAFAF,0x7070,0xBFBF,0x9F9F,0x5050,0x6050,0x5050,0x6050,0x5050,0x5060,0x8F8F,0x8090,0x7060,0x7050,0x7060,0x5050,0x5050,0x5060,0x5060,0x5050,0x5050,0x5060,0x6050,0x6070,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4040,0x4050,0x4050,0x4050,0x5050,0x7F7F,0x6F6F,0x5040,0x4050,0x4050,0x5060,0x4050,0x5050,0x5050,0x5050,0x6050,0x4040,0x4050,0x5050,0x5050,0x5050,0x5F5F,0x5F5F,0x4F4F,0x5F5F
.word 0x7060,0x7060,0x6070,0x6070,0x5070,0x5070,0x6050,0x6050,0x5050,0x5050,0x5040,0x5040,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5040,0x8F8F,0x5060,0x8F8F,0x5050,0x5060,0x6060,0x6060,0x7050,0x6060,0x6070,0x9F9F,0x7070,0x9F9F,0x7070,0x9F9F,0x6070,0x9F9F,0x7070,0x7080,0x7080,0x7070,0x6060,0x7060,0x6060,0xAFAF,0x9F9F,0xBFBF,0xBFBF,0xAFAF,0x5060,0x5050,0x4050,0x5040,0x5050,0x4050,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5040,0x5040,0x5060,0x6050,0x5060,0x6070,0x4070,0x5F5F,0x5F5F,0x5F5F,0x5040,0x7050,0x6060,0x5070,0x5050,0x5F5F,0x5F5F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x4060,0x6050,0x8F8F,0x6070,0x6070,0x6060,0x6060,0x5060,0x9F9F,0x6070,0x7060,0x9F9F,0x7070,0xAFAF,0xBFBF,0x9F9F,0x8F8F,0x4060,0x6050,0x5050,0x5060,0x6050,0x6060,0x6060,0x6070,0x5050,0x5060,0x6060,0x5060,0x7050,0x6050,0x6050,0x6050,0x6050,0x6060,0x7050,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x5050,0x8F8F,0x5050,0x4050,0x7F7F,0x6F6F,0x4050,0x5040,0x5050,0x5050,0x6050,0x5050,0x5060,0x5050,0x5050,0x6050,0x5050,0x5050,0x6050,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F4F
.word 0x7060,0x6070,0x9F9F,0xAFAF,0x9F9F,0x6050,0x5060,0x5050,0x5060,0x5060,0x5050,0x5040,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0x5050,0x5050,0x5060,0x5060,0x7050,0x9F9F,0x9F9F,0x5060,0x7050,0x9F9F,0x6050,0x8F8F,0x7070,0x6050,0x7060,0x7060,0x7060,0x7060,0x8060,0xAFAF,0x9F9F,0x5050,0x6070,0x7060,0x6060,0x5070,0x6060,0x7050,0x9F9F,0xAFAF,0xAFAF,0x5050,0x4050,0x4040,0x5040,0x4050,0x4050,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5050,0x5060,0x7050,0x6050,0x5060,0x5F5F,0x5F5F,0x5F5F,0x5050,0x5050,0x4060,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5040,0x5040,0x5040,0x5040,0x5050,0x5050,0x5060,0x7050,0x5050,0x8F8F,0x5070,0x6050,0x6050,0x5060,0x6050,0x6070,0x7080,0xAFAF,0x7080,0xBFBF,0xBFBF,0x80BE,0x8F8F,0x7F7F,0x8F8F,0x6050,0x9F9F,0x9F9F,0x7080,0x7060,0x7060,0x6050,0x6050,0x5050,0x5070,0x5050,0x5060,0x5060,0x5050,0x6060,0x6080,0x4060,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5040,0x5040,0x5040,0x6050,0x5060,0x6060,0x8F8F,0x7F7F,0x5F5F,0x4050,0x5060,0x6050,0x6050,0x5050,0x6050,0x5050,0x6050,0x5050,0x5050,0x6050,0x5040,0x5050,0x7050,0x4060,0x5F5F,0x5F5F,0x5F5F
.word 0xBFBF,0xAFAF,0xAFAF,0x6060,0x5060,0x6050,0x8F8F,0x6050,0x6050,0x5050,0x4050,0x4050,0x6F6F,0x6F6F,0x5050,0x6F6F,0x5050,0x6040,0x6050,0x6050,0x5050,0x5060,0x5070,0x7050,0x9F9F,0x5070,0x6050,0x5060,0x7060,0x5050,0x5060,0x6060,0x8F8F,0x6060,0x9F9F,0x9F9F,0x8060,0x7060,0x7060,0x5060,0x6060,0x7050,0x6050,0x6050,0x5050,0x7050,0x5060,0x6060,0xAFAF,0x4050,0x5050,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x7060,0x5050,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x5050,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x5040,0x5050,0x5050,0x6050,0x5050,0x8F8F,0x5060,0x6050,0x6050,0x9F9F,0x6060,0x8F8F,0x6050,0x6050,0x7060,0x7060,0x8060,0x9F9F,0x7070,0xBFBF,0xBE70,0xBFBF,0x6F6F,0x5050,0x5050,0x9F9F,0x9F9F,0x7060,0x6080,0x5070,0x5070,0x5070,0x6060,0x9F9F,0x7070,0x9F9F,0x7060,0x6060,0x8050,0x7060,0x5060,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x4050,0x5050,0x5050,0x6050,0x5050,0x4060,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x6070,0x7050,0x6050,0x5050,0x5060,0x5060,0x6050,0x4050,0x5060,0x5050,0x5040,0x5050,0x6050,0x5F5F,0x5F5F,0x4F5F
.word 0x7070,0x6060,0x6070,0xAFAF,0x8F8F,0x8F8F,0x4050,0x8F8F,0x5040,0x6050,0x5050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x5050,0x6050,0xAFAF,0x9F9F,0x6060,0x5060,0x6060,0x6050,0x6050,0x9F9F,0x5050,0x5070,0xAFAF,0x9F9F,0x7060,0x5050,0x7060,0x6070,0x6070,0xAFAF,0x6060,0x9F9F,0x7050,0x5050,0xAFAF,0x5060,0x6050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5040,0x5040,0x6050,0x5050,0x5050,0x5060,0x5060,0x6050,0x6070,0x6050,0x5050,0x5070,0x5060,0x6050,0x5050,0x8F8F,0x5050,0x6050,0x6050,0x6070,0x6060,0x7070,0x8070,0xCFCF,0xBFBF,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x5060,0x6070,0x9F9F,0x6060,0x7050,0x9F9F,0x7050,0x7050,0x5050,0x7050,0x9F9F,0x6070,0x6060,0x6070,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0x5050,0x6070,0x6050,0x5040,0x6F6F,0x5040,0x5040,0x6040,0x8F8F,0x5060,0x6050,0x5050,0x6F6F,0x5F5F,0x6F6F,0x7050,0x6050,0x5060,0x6070,0x6050,0x5050,0x6050,0x5050,0x5060,0x5050,0x5050,0x5050,0x4050,0x5050,0x5F5F,0x4F4F,0x5F5F
.word 0xAFAF,0xAFAF,0x7060,0x6060,0xAFAF,0xAFAF,0x6F6F,0x6F6F,0x7F7F,0x5040,0x8F8F,0x5040,0x4050,0x6F6F,0x6F6F,0x5040,0x6F6F,0x5050,0x8F8F,0x5070,0x6060,0x6070,0x6060,0xAFAF,0x6050,0x9F9F,0x6070,0x5070,0x9F9F,0x7060,0x7060,0x5060,0x6070,0x6070,0xAFAF,0x7060,0x6050,0x7060,0x7050,0x6050,0x9F9F,0x6070,0x6060,0x7050,0x5050,0x5060,0x5060,0x5050,0x6050,0x5050,0x5040,0x5050,0x4060,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5040,0x5060,0x5060,0x6070,0x7070,0x5070,0x6050,0x6050,0x5050,0x6050,0x5050,0x5060,0x5050,0x5050,0x4050,0x5060,0x5060,0x7060,0x6060,0x5060,0x6060,0x6070,0x9F9F,0x7060,0x7070,0xCFCF,0xAFAF,0x6F6F,0x5040,0x5040,0x5050,0x7050,0x6070,0x6070,0x6060,0x5060,0x5060,0x5050,0x9F9F,0x5070,0x6070,0x7050,0x9F9F,0x8F8F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5050,0x4050,0x4050,0x6F6F,0x6F6F,0x7050,0x9080,0x8070,0x6060,0x4060,0x5F5F,0x5040,0x5040,0x5050,0x5060,0x6050,0x5050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5050,0x6060,0x7050,0x5060,0x7070,0x5050,0x5060,0x6050,0x6050,0x5050,0x5050,0x5040,0x4060,0x5F5F,0x4F4F,0x5F5F
.word 0x5050,0x6050,0x6050,0x7060,0x6060,0x6060,0x5050,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x5040,0x4040,0x5050,0x6F6F,0x7F7F,0x5040,0x7050,0x9F9F,0x6060,0x6050,0x6050,0x6050,0xAFAF,0x7060,0x5050,0x6050,0x6060,0x5070,0x5050,0x7060,0x9F9F,0x7060,0x6060,0x6070,0x6060,0x5070,0x5050,0x5070,0x7070,0x5050,0x5060,0x5060,0x6050,0x6050,0x5050,0x6050,0x5050,0x5050,0x4050,0x5050,0x5050,0x5050,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x6050,0x5050,0x7050,0x5050,0x9F9F,0x9F9F,0x5050,0x5060,0x5060,0x5060,0x9F9F,0x6050,0x5050,0x4040,0x5050,0x8F8F,0x6040,0x6060,0x6060,0x6060,0x6060,0x6050,0x6070,0x9F9F,0x9F9F,0x9070,0xE0BE,0xAFAF,0x6F6F,0x7F7F,0x5050,0x6060,0x7050,0x6050,0x6050,0x7050,0x6050,0x6050,0x5050,0x5050,0x5050,0x6070,0x6080,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6050,0x6070,0x5070,0x5050,0x6040,0x6050,0x7050,0x8070,0x7070,0x6080,0x5070,0x5050,0x5040,0x7F7F,0x4050,0x5060,0x7050,0x5050,0x6050,0x6F6F,0x4050,0x5050,0x6F6F,0x8F8F,0x7050,0x5050,0x5070,0x5060,0x5050,0x6050,0x5050,0x5060,0x5050,0x6050,0x4050,0x5060,0x6050,0x5F5F,0x5F5F,0x5F5F
.word 0x5060,0x6060,0x6050,0x5060,0x5050,0x5050,0x5050,0x5040,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x5060,0x7060,0x9F9F,0x8F8F,0x5060,0x5070,0x6050,0x6050,0x7050,0x6060,0x6050,0x6050,0x7060,0x6060,0x9F9F,0x6070,0x7060,0x6060,0x9F9F,0x7050,0x6050,0x7050,0x5050,0x6050,0x5050,0x9F9F,0x5070,0x5050,0x6050,0x5050,0x5050,0x5050,0x6050,0x5050,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5040,0x5050,0x5050,0x5050,0x5060,0x5070,0x6070,0x6060,0x6050,0x6050,0x6050,0x5050,0x6070,0x9F9F,0x6050,0x8F8F,0x4050,0x5050,0x8F8F,0xAFAF,0x5060,0x9F9F,0x9F9F,0x6050,0x6050,0x7060,0x6060,0x6070,0x9F9F,0x7060,0x4060,0x6F6F,0x8F8F,0x5050,0x5060,0x6050,0x5070,0x5070,0x5050,0x5060,0x6050,0x6050,0x6050,0x6050,0x6060,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x7050,0x7080,0x7080,0x6080,0x5070,0x6060,0x5050,0x6070,0x6070,0x8060,0x7060,0x7060,0x5060,0x6F6F,0x5040,0x5050,0x5050,0x5060,0x7050,0x5060,0x5040,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x9F9F,0x5050,0x5050,0x6050,0x5060,0x5070,0x6050,0x5050,0x5060,0x5060,0x6050,0x5050,0x5060,0x5F5F,0x5F5F,0x5F4F
.word 0x6040,0x5050,0x4050,0x8F8F,0x5050,0x5050,0x4040,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x5050,0x5060,0x5060,0x5070,0x6060,0x6050,0x6060,0x5070,0x9F9F,0x6060,0x5050,0x9F9F,0x5070,0x9F9F,0x6050,0x6060,0x9F9F,0xAFAF,0x6060,0x6050,0x5060,0x5050,0x6070,0x5060,0x6050,0x5050,0x7050,0x6050,0x5050,0x5060,0x5050,0x6050,0x5040,0x6050,0x5050,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x4050,0x4050,0x6050,0x6050,0x6050,0x5060,0x9F9F,0x5070,0x6050,0x5070,0x9F9F,0x5060,0x6050,0x6050,0x6070,0x8F8F,0x5050,0x5040,0x4040,0x7050,0x6060,0x5060,0x7050,0x7050,0x5060,0x5050,0x5060,0x6070,0x9F9F,0x7060,0x7060,0x6050,0x5F5F,0x6F6F,0x6050,0x6050,0x5060,0x5050,0x6060,0x6060,0x5050,0x5050,0x5060,0x5060,0x5050,0x5050,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5050,0x7080,0x7080,0x6080,0x7060,0x7060,0x7060,0x7050,0x6060,0x7060,0x7060,0x6060,0x5070,0x6050,0x5040,0x6F6F,0x5040,0x5060,0x6050,0x9F9F,0x5050,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5040,0x5070,0x7F7F,0x5060,0x6050,0x5050,0x5060,0x6050,0x6050,0x5040,0x5060,0x6050,0x5050,0x5F5F,0x5F5F,0x4F5F
.word 0x4050,0x6050,0x5040,0x4040,0x5050,0x5050,0x4050,0x7F7F,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5050,0x6050,0x6050,0x7050,0x9F9F,0x9F9F,0x6060,0x9F9F,0x5070,0x5050,0x7060,0x7060,0x7060,0x5050,0x7070,0x9F9F,0x6070,0x6060,0x5060,0x9F9F,0x7060,0x5050,0x8F8F,0x9F9F,0x5060,0x5060,0x5060,0x9F9F,0x5060,0x6050,0x6050,0x5040,0x5050,0x5050,0x4060,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5050,0x5050,0x5070,0x5070,0x5070,0xAFAF,0x5070,0x7050,0x5060,0x7050,0x7060,0x7050,0x5050,0x5050,0x6050,0x7060,0x5050,0x5040,0x4050,0x5050,0x5060,0x6060,0x8F8F,0x5050,0x6060,0x6050,0x5050,0x7050,0x5050,0x5060,0x5070,0xAFAF,0x6F6F,0x6F6F,0x5040,0x5060,0x6050,0x6050,0x8F8F,0x8F8F,0x5050,0x5060,0x5050,0x8F8F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x4050,0x7070,0x9090,0x8070,0x7070,0x7070,0x6070,0x5070,0x6070,0x7060,0x6050,0x6070,0x5070,0x6060,0x5050,0x4060,0x4050,0x4050,0x6040,0x5050,0x5060,0x5060,0x6F6F,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5050,0x5050,0x7F7F,0x8F8F,0x5060,0x6060,0x7050,0x5050,0x8F8F,0x5060,0x5050,0x4060,0x6F6F,0x6F6F,0x6F6F,0x5F5F
.word 0x5040,0x5050,0x4050,0x5050,0x4040,0x4040,0x4050,0x4050,0x4050,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x5050,0x6060,0x6060,0x5050,0x7060,0x6060,0x5060,0x6070,0x7050,0x5060,0x9F9F,0x5060,0x5050,0x5060,0x5060,0x7060,0x7060,0x7050,0x7050,0x6050,0x9F9F,0x7050,0x5050,0x5060,0x9F9F,0x5070,0x6060,0x7050,0x6050,0x5050,0x4050,0x6050,0x5050,0x5050,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6050,0x6050,0x7050,0x6050,0x9F9F,0x6050,0x7060,0x7050,0x5050,0x6060,0x5070,0x5050,0x9F9F,0x7050,0x7050,0x5050,0x5050,0x5050,0x5050,0x6060,0x6060,0x6060,0x6060,0x6060,0x6050,0x9F9F,0x8F8F,0x6060,0x5050,0x5060,0x5060,0x7050,0x6050,0x5060,0x6F6F,0x6050,0x5050,0x5060,0x5050,0x5060,0x6050,0x6050,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5060,0x7050,0x8060,0x9F9F,0x7080,0x7070,0x7070,0x7070,0x6060,0x7050,0x6060,0x7050,0x7060,0x6050,0x6070,0x5060,0x5050,0x4050,0x7F7F,0x8F8F,0x5060,0x6050,0x4050,0x6F6F,0x4040,0x6F6F,0x5F5F,0x5F5F,0x6040,0x5050,0x6F6F,0x6F6F,0x5050,0x8F8F,0x5070,0x6050,0x6050,0x5050,0x5050,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040
.word 0x5040,0x5040,0x5050,0x4050,0x5040,0x4050,0x4050,0x4050,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x8F8F,0x9F9F,0x6050,0x7050,0x5060,0x6050,0x7050,0x5050,0x5060,0x7050,0x7050,0x6060,0x6050,0x7060,0x9F9F,0x5060,0x5050,0x5070,0x5050,0x6050,0x6050,0x5070,0x6050,0x6050,0x7050,0x9F9F,0x5060,0x5060,0x5050,0x4050,0x5050,0x5050,0x6050,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x7050,0x5060,0x9F9F,0x7060,0x6070,0x5060,0x6070,0x9F9F,0x7060,0x9F9F,0x7070,0x5050,0x5070,0x7060,0x6060,0x6050,0x5050,0x6070,0x6070,0x7050,0x6050,0x5050,0x5060,0xAFAF,0x8F8F,0x6050,0x9F9F,0x6050,0x5060,0x5060,0x5060,0x6060,0x7050,0x5040,0x8F8F,0x6050,0x6050,0x6050,0x5050,0x5060,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7040,0x7060,0x6050,0x5060,0x7060,0x7060,0x6060,0x7060,0x7060,0x6070,0x7060,0x6070,0x7060,0x6070,0x6070,0x7060,0x5050,0x6060,0x6060,0x5050,0x4050,0x5040,0x5040,0x5060,0x5050,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5060,0x5040,0x6F6F,0x7F7F,0x6050,0x5050,0x5060,0x5060,0x5050,0x7F7F,0x5F5F,0x5040,0x4050,0x4050,0x4050
.word 0x4050,0x4050,0x4040,0x5050,0x4050,0x5040,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4050,0x5050,0x8F8F,0x6060,0x7050,0x7050,0x5060,0x5070,0x7060,0x5050,0x5070,0x5050,0x5060,0x6050,0x5070,0x6060,0x6050,0x5050,0x5070,0x5060,0x6050,0x5050,0x5060,0x5050,0x5050,0x6050,0x5050,0x6050,0x6050,0x5060,0x5050,0x5060,0x8F8F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5040,0x5040,0x5060,0x6050,0x7060,0x5060,0x6060,0x6060,0x9F9F,0x6060,0x6060,0x7050,0x5050,0x6060,0x7050,0x9F9F,0x9F9F,0x5070,0xAFAF,0x7070,0x5060,0x9F9F,0x6050,0x6070,0x7050,0x5060,0x6070,0x6050,0x7060,0x5060,0x5060,0x6050,0x5050,0x6060,0x6070,0x6F6F,0x6F6F,0x5050,0x5050,0x5060,0x6050,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x8060,0x6070,0x6080,0x6070,0x5070,0x7060,0x7060,0x9F9F,0x6070,0x7060,0x6060,0x7060,0x6060,0x6060,0x7060,0x5070,0x7070,0x7060,0x5060,0x5070,0x5050,0x4050,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x4060,0x6F6F,0x6F6F,0x4050,0x8F8F,0x5050,0x5050,0x7F7F,0x6F6F,0x5050,0x4050,0x5040,0x5040,0x5040
.word 0x5040,0x5040,0x5050,0x5040,0x5050,0x7F7F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5050,0x8F8F,0x6050,0x5060,0x5070,0x7050,0x5050,0x5070,0x7050,0x6050,0x7050,0x9F9F,0x5050,0x7050,0x6050,0xAFAF,0x6050,0x6050,0x5050,0x5070,0x6050,0x6050,0x6050,0x5050,0x5060,0x5050,0x8F8F,0x5050,0x4060,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x4040,0x4050,0x5060,0x6060,0x7050,0x5050,0x7070,0x6060,0xAFAF,0x6070,0x6070,0x6050,0x6050,0x5060,0x5060,0x5050,0x6060,0x5060,0x6050,0x7070,0x6070,0x6060,0x7070,0x6060,0x7050,0x5050,0x6060,0x6050,0x9F9F,0x5050,0x9F9F,0x6060,0x5060,0x5060,0x6050,0x7050,0x5F5F,0x5F5F,0x5040,0x8F8F,0x7F7F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7080,0x7080,0x6070,0x7060,0x7060,0x6060,0x6070,0x6080,0x7060,0x7060,0x7070,0x6060,0x7070,0x7060,0x6060,0x6060,0x8060,0x6060,0x6070,0x7050,0x6050,0x6050,0x5050,0x5050,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7050,0x5060,0x5F5F,0x5040,0x5040,0x4050,0x5060,0x7F7F,0x6F6F,0x5040,0x4040,0x5050,0x5050,0x5040,0x5040
.word 0x4050,0x4050,0x4040,0x5050,0x4040,0x5050,0x5040,0x5040,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x8F8F,0x6050,0x5050,0x6050,0x5070,0x6060,0x5060,0x8F8F,0x5050,0x5050,0x5070,0x7050,0x5050,0x5070,0x6050,0x6050,0xAFAF,0x9F9F,0x5050,0x9F9F,0x5060,0x5050,0xAFAF,0x6050,0x6050,0xBE50,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x6050,0x7050,0x5050,0x5070,0x5060,0x7050,0x6060,0x6070,0x6050,0x7060,0x6060,0x6060,0x6050,0xAFAF,0xAFAF,0x5060,0x9F9F,0x7070,0x9F9F,0x7060,0x5060,0x6060,0x6050,0xAFAF,0x6050,0x7050,0x7050,0x5070,0x6070,0x9F9F,0x6050,0x5050,0x5070,0x5050,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5050,0x7060,0x7060,0x7070,0x6070,0x7060,0x7060,0x7060,0x7060,0x7060,0x6060,0x8060,0x7060,0x7060,0x6070,0x6070,0x8080,0x7070,0x7060,0x7060,0x5070,0x7060,0x6050,0x6060,0x5050,0x5070,0x6F6F,0x5050,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5060,0x5F5F,0x5040,0x5040,0x7F7F,0x7F7F,0x5050,0x5040,0x5040,0x5040,0x5050,0x5050,0x4050,0x5070
.word 0x4040,0x4050,0x5050,0x5050,0x6050,0x4050,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5050,0x8F8F,0xAFAF,0x6050,0xAFAF,0x6050,0x6050,0x7070,0x7060,0x5050,0x5060,0x6050,0x5050,0x5060,0x6050,0x8F8F,0xAFAF,0x6050,0x5050,0x5050,0xCFCF,0x50DE,0x8F8F,0x7F7F,0xCFCF,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6040,0x5050,0x5070,0x6050,0x7050,0x7050,0x7050,0x6070,0x7060,0x7050,0x7050,0x5050,0x5060,0x5050,0x6050,0x9F9F,0x6060,0x9F9F,0x9F9F,0x6060,0x7060,0x9F9F,0x7060,0x6060,0x6050,0x5060,0x7050,0x9F9F,0x5050,0x5050,0x6050,0x6050,0x5060,0x6050,0x5050,0x5060,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6040,0x5050,0x6070,0x6060,0x6070,0x6060,0x6060,0x7060,0x6060,0x6070,0x6070,0x7060,0x9F9F,0x7060,0x8070,0x7060,0x7080,0x7080,0x6070,0x6070,0x5070,0x6060,0x5070,0x7060,0x7050,0x6060,0x5060,0x4050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x6F6F,0x5050,0x5050,0x5050,0x4050,0x4050,0x4050,0x5050,0x5060,0x6040,0x5040,0x5050,0x5040
.word 0x5050,0x5040,0x5040,0x5050,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x6050,0x8F8F,0x8F8F,0x5040,0x8F8F,0x5060,0x5050,0x5040,0x8F8F,0xCFCF,0xD0FE,0x5040,0x6050,0x8F8F,0xBFBF,0x8F8F,0x6F6F,0xBFBF,0xFFFF,0xAFAF,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5050,0x6050,0x5050,0x5070,0x5060,0x5050,0x5070,0x6060,0x6070,0x5060,0x5070,0x6060,0x5050,0x5060,0x8F8F,0x6070,0xAFAF,0x6060,0x7080,0x7060,0x6060,0x7060,0x6060,0x6050,0x6060,0x6050,0x5070,0x7060,0x5070,0x5070,0x5060,0x6050,0x5050,0x5060,0x6050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x5050,0x5070,0x6060,0x6070,0x7060,0x6070,0x6070,0x9F9F,0x6080,0x7060,0x5060,0x6070,0x7070,0xBFBF,0x7080,0x8080,0xAFAF,0x6060,0x6060,0x7060,0x6060,0x6070,0x6060,0x5070,0x6070,0x6070,0x5050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4040,0x5050,0x6050,0x6050,0x5060,0x5040,0x5040,0x6050,0x5040,0x5040,0x5050,0x4050,0x5050,0x4050
.word 0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4050,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xFEFE,0x6F6F,0x7F7F,0x9F9F,0xFEDE,0xAFAF,0x5F5F,0x7F7F,0xFFFF,0xFFFF,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6050,0x5050,0x5060,0x7F7F,0x6F6F,0x7F7F,0x6060,0x6060,0x7050,0x5070,0x6050,0x5060,0x5060,0x6050,0x5050,0x8F8F,0x9F9F,0x9F9F,0x7060,0x7060,0x7070,0x6060,0x7060,0x9F9F,0x6050,0x5050,0x6050,0x5050,0x9F9F,0x8F8F,0x6060,0x5050,0x5060,0x6050,0x5050,0x5060,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x6040,0x5040,0x5050,0x5040,0x4050,0x6F6F,0x6050,0x7060,0x6060,0x7060,0x6060,0x7060,0x7060,0x6060,0x6070,0x6070,0x6070,0x9F9F,0x6060,0xBFBF,0xBFBF,0x7070,0x6080,0x7070,0x7060,0x6060,0x5070,0x7060,0x5050,0x7060,0x5050,0x7050,0x5060,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5050,0x5050,0x5060,0x5050,0x5050,0x5060,0x5050,0x5050,0x4050,0x5050,0x5050,0x6050,0x5050
.word 0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x4050,0x5050,0x8F8F,0x7F7F,0x8F8F,0xCFCF,0xEFEF,0xAFAF,0xBFBF,0xBFBF,0xFFFF,0xFFFF,0x6F6F,0xBFBF,0xFFFF,0xFFFF,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5040,0xBFBF,0xCFCF,0x6F6F,0x6F6F,0x5050,0x6070,0x6060,0x5070,0x6050,0x5070,0x7050,0x5050,0x5060,0x5050,0x6050,0x8F8F,0x6050,0x6050,0x6070,0x6050,0x6050,0x6060,0x6070,0x6050,0x6060,0x5070,0x7050,0x6050,0x5060,0x5060,0x5060,0x6050,0x8F8F,0x5060,0x9F9F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x4050,0x5050,0x5050,0x4050,0x4050,0x4050,0x4050,0x6060,0x7060,0x7060,0x6070,0x6070,0x6070,0x6070,0x7060,0x7060,0x6060,0x7060,0x5050,0x8F8F,0x5050,0x6060,0x7080,0x8060,0x7060,0x6070,0x6070,0x6070,0x6070,0x7060,0x6060,0x7060,0x5050,0x5070,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x6050,0x5050,0x6040,0x6050,0x6060,0x6050,0x6050,0x6050,0x5050,0x5050,0x5050,0x5070,0x5050
.word 0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5050,0x5050,0x4040,0x4050,0x4050,0x4050,0x5050,0xBFBF,0x9F9F,0x9F9F,0xFEFE,0xDEFE,0xFEFE,0xD7FE,0xCFCF,0xFFFF,0x8F8F,0xBFBF,0xFFFF,0xFFFF,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0xAFAF,0x9F9F,0xCFCF,0xBEF0,0x5F5F,0x6F6F,0x6040,0x6050,0x6050,0x5050,0x7060,0x6050,0x5070,0x6050,0x5050,0x5060,0x5050,0x7F7F,0x5050,0x7060,0x9F9F,0x7060,0x6060,0x6050,0x6050,0xAFAF,0x6050,0x5050,0x5060,0x5050,0x6050,0x5050,0x5050,0x5060,0x6050,0x5050,0xAFAF,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5050,0x9F9F,0x5050,0x6050,0x9F9F,0x9F9F,0x5040,0x5040,0xAFAF,0x6070,0x6070,0x7060,0x7060,0x7060,0x6060,0x7070,0x6070,0x8F8F,0x6F6F,0x6F6F,0x5050,0x7050,0x7060,0x6060,0x6070,0x7060,0x7060,0x7060,0x6060,0x6070,0x6070,0x6060,0x5070,0x5060,0x6080,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5060,0x5050,0x5060,0x5060,0x5050,0x7050,0x6050,0x5050,0x5050,0x5060,0x5060,0x5050,0x6050,0x6050
.word 0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x4050,0x5050,0x5040,0x5050,0xAFAF,0xAFAF,0x5040,0x5040,0x5040,0x8F8F,0xBFBF,0x9F9F,0xCFCF,0xFFFF,0xFEFE,0xFFFF,0xFFFF,0xBFBF,0xFFFF,0xBFBF,0xBE79,0xFFFF,0xFFFF,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0xBFBF,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x7F7F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xAFAF,0x6F6F,0x6F6F,0x5050,0x5070,0x5070,0x5070,0x5060,0x5060,0x6050,0x5050,0x5060,0x6050,0x6F6F,0x5040,0x5040,0x6060,0x6060,0x5070,0x7050,0x6050,0x5050,0x6070,0x6050,0x5060,0x6050,0x6050,0x5050,0x5060,0x5050,0x5050,0x5060,0x5070,0x9F9F,0x6F6F,0x4050,0x5050,0x5040,0xBFBF,0xFFFF,0x5050,0xBFBF,0xEFEF,0xCFCF,0x5050,0x4050,0xFEDE,0xBFBF,0x7060,0x5060,0x6050,0x6070,0x9F9F,0x5060,0x6F6F,0x6F6F,0x6F6F,0x5050,0x7050,0x6060,0x6070,0x6080,0x7070,0x6070,0x6070,0x8060,0x7060,0x7060,0x7050,0x6050,0x6050,0x5060,0x7F7F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5060,0x6050,0x5050,0x5060,0x5060,0x6050,0x5070,0x5060,0x6050,0x5050,0x5050,0x6070,0x5070
.word 0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x8060,0x4070,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x6050,0x9F9F,0xBE8C,0x9F9F,0x4050,0x4050,0x5060,0xBFBF,0xD7EE,0xBFBF,0xFEFD,0xFFFF,0xFFFF,0xF8FD,0xF7CB,0xFFFF,0xAFAF,0xDDD7,0xFFFF,0xFFFF,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0xBFBF,0xD6FB,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0xAFAF,0xBFBF,0xBFBF,0x7060,0xAFAF,0xBFBF,0x7080,0x5F5F,0x6F6F,0x6F6F,0x5050,0x6050,0x8F8F,0x6050,0x5060,0x7050,0x6050,0x5070,0x5070,0x7F7F,0x6F6F,0x4050,0x5050,0x5070,0x6060,0x7060,0x9F9F,0x6060,0x5070,0x6050,0x6050,0x6050,0x5050,0x5060,0x4060,0x5050,0x5060,0x5060,0x5050,0x5070,0xAFAF,0x8F8F,0x6F6F,0x4040,0x4050,0xBFBF,0xFFFF,0x9F9F,0xAFAF,0xEFEF,0xEFEF,0x6050,0xBFBF,0xFFFF,0xBFBF,0x5050,0x7070,0x6070,0x9F9F,0xCFCF,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x6070,0x7070,0x7060,0x8060,0x7060,0x8060,0x7060,0x6060,0x6070,0x6070,0x7060,0x6060,0x6060,0x7F7F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x7050,0x6060,0x5050,0x5060,0x6050,0x6050,0x5060,0x6050,0x6050,0x5050,0x5060,0x7050,0x5060,0x6050
.word 0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x7090,0x6F6F,0x5F5F,0x6F6F,0x4040,0x4050,0x5050,0x5060,0xAFAF,0xAFAF,0xCFCF,0x8F8F,0x5040,0x9F9F,0xCFCF,0xBE7A,0xBFBF,0xFFFF,0xFAEE,0xFFFF,0xFFFF,0xEED7,0xFFFF,0x8AFE,0xCFCF,0xFFFF,0xFFFF,0x6F6F,0x4F4F,0xAFAF,0x4F4F,0x6F6F,0xF7BE,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0xBFBF,0x6060,0x6060,0xAFAF,0x7070,0x8070,0x9F9F,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x5050,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0x5050,0x8F8F,0x5F5F,0x4050,0x4050,0x5060,0x6050,0x5070,0x5050,0x6060,0x5050,0x6050,0x5050,0x5060,0x5050,0x6050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5060,0x4050,0x9F9F,0xAFAF,0x8F8F,0x5050,0x6050,0xBFBF,0xFFFF,0x60CE,0xAFAF,0xFEFE,0xBFBF,0x5050,0xFFFF,0xFFFF,0x50BE,0x5050,0x6050,0x7060,0xEFEF,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x7060,0x7050,0x7060,0x6070,0x6060,0x6070,0x6060,0x6080,0x6080,0x7060,0x7060,0x7070,0x5070,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5070,0x5060,0x6060,0x5050,0x5060,0x5050,0x6050,0x5050,0x5050,0x5060,0x6040,0x5050,0x5060,0x6060
.word 0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x7170,0x4070,0x6F6F,0x5040,0x4050,0x5050,0x5040,0x7040,0x9F9F,0x9F9F,0xAFAF,0xBC7A,0xBEDE,0x5050,0x5050,0xCFCF,0xCFCF,0xCFCF,0xFEFB,0xFCE7,0xFFFF,0xFFFF,0xE7FE,0xFFFF,0xCFCF,0xF7FD,0xFFFF,0xFFFF,0x5F5F,0xCFCF,0xFFFF,0x4F4F,0x8F8F,0xFFFF,0x5F5F,0x4F4F,0x6F6F,0x5F5F,0x6050,0x5060,0x5060,0x5060,0x6070,0xAFAF,0xAFAF,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5050,0x7F7F,0x5040,0x6F6F,0x6F6F,0x5040,0x5040,0x5050,0x7070,0x6050,0x6050,0x7050,0x5050,0x5060,0x6050,0x8F8F,0x8F8F,0x5060,0x8F8F,0x5050,0x5050,0x5050,0x8F8F,0x6050,0x6F6F,0x8F8F,0xAFAF,0xAFAF,0x5050,0x5050,0xBCBE,0xFFFF,0xBEFE,0xBFBF,0xFFFF,0xBEF0,0xCFCF,0xFEFD,0xFFFF,0xCFCF,0x7F7F,0x7070,0xBFBF,0xFFFF,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x5060,0x6060,0x6070,0x6060,0x7070,0x6060,0x7070,0x7060,0x7070,0x8080,0x8070,0x5080,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5050,0x5060,0x6050,0x6050,0x6050,0x5060,0x6050,0x6050,0x5050,0x5060,0x7050,0x6060,0x5060
.word 0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x9080,0x6F6F,0x4050,0x6F6F,0x5040,0x4040,0x4050,0x5050,0xBFBF,0x7060,0x8A9E,0xAFAF,0xDFDF,0x9F9F,0xAFAF,0x9CBE,0xBFBF,0xE9BE,0xE7EE,0xBFBF,0xFFFF,0xFFFF,0xEEFB,0xFFFF,0xD7F7,0xFFFF,0xFFFF,0xFFFF,0x7F7F,0xFFFF,0xAFAF,0xBFBF,0xFFFF,0xEFEF,0x5F5F,0x5F5F,0x6F6F,0x4050,0x4050,0x6050,0x5040,0x6050,0x6050,0x6070,0x6080,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x4050,0x4050,0x4050,0x5060,0x5050,0x5060,0x5060,0x6050,0x9F9F,0x5050,0x5060,0xAFAF,0xBFBF,0x9F9F,0x5060,0x5050,0x6050,0x6050,0x5050,0x7F7F,0x5F5F,0xAFAF,0x9F9F,0x8F8F,0x5050,0x7050,0xAFAF,0xFFFF,0xBFBF,0xEFEF,0xFFFF,0xEEFE,0xCFCF,0xFFFF,0xFFFF,0x50BE,0x5060,0xBFBF,0xFDDE,0xFFFF,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5040,0x7060,0x6070,0x7060,0x6060,0x6060,0x6070,0x6060,0x8070,0x8070,0x7070,0x9F9F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6060,0x6050,0x6050,0x6060,0x5050,0x5050,0x5060,0x6050,0x5050,0x5060,0x5050,0x5040,0x5070,0x6050,0x7060
.word 0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x7041,0x8071,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x5050,0x6050,0xAFAF,0xBFBF,0x9F9F,0xBFBF,0xBEDE,0x6070,0xCFCF,0xAFAF,0xEE8A,0xEEF9,0xDED7,0xFFFF,0xFFFF,0xE7FB,0xFFFF,0xEEFE,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFFFF,0x8F8F,0x9F9F,0xFFFF,0xBFBF,0x6F6F,0x6F6F,0x4050,0x4050,0x6F6F,0x5050,0x5050,0x5060,0x5070,0x6060,0x5060,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5040,0x6040,0x6050,0x5070,0x6050,0x6050,0x6060,0x6050,0x6050,0x8F8F,0xAFAF,0xBEAE,0x5050,0x5060,0x5050,0x5050,0x7F7F,0x5F5F,0x6F6F,0xBFBF,0x9F9F,0x9F9F,0x6050,0xAFAF,0xAADE,0xCFCF,0xFFFF,0xFFFF,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0x5050,0xAFAF,0xFFFF,0xFFFF,0x4F4F,0x6F6F,0x5F5F,0x4050,0x5060,0x6070,0x7060,0x6060,0x6060,0xAFAF,0x7050,0x7050,0x9F9F,0xAFAF,0x7080,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5040,0x5050,0x5F5F,0x6F6F,0x7050,0x6050,0x5050,0x5050,0x5050,0x5060,0x5060,0x6050,0x5050,0x5050,0x5040,0x5040,0x5050,0x7060,0x5070,0x5060
.word 0x5060,0x5040,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x9051,0x70B0,0x6F6F,0x4050,0x4050,0x8F8F,0x8F8F,0x8F8F,0x5050,0x5060,0x5050,0x9F9F,0xBFBF,0x9F9F,0xBFBF,0xAFAF,0xBFBF,0xAAD7,0xCFCF,0xCFCF,0xCFCF,0xFEFE,0xFFFF,0xF9FE,0xFFFF,0xE7FD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xAFAF,0xFBD7,0xFFFF,0x40E4,0x5040,0x6F6F,0x5040,0x5040,0x4050,0x6040,0x6050,0x5050,0x5050,0x5060,0x5060,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5040,0x5050,0x6050,0x6050,0x5050,0x5060,0x5050,0x5060,0x5050,0x5060,0x8F8F,0xAFAF,0x9F9F,0x5040,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0xAFAF,0xAFAF,0xBFBF,0x5050,0xCE50,0xBFBF,0xFEE9,0xEFEF,0xEFEF,0xFFFF,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0x8F8F,0xBFBF,0x6F6F,0x5050,0x7060,0x6060,0x6070,0x6070,0x6070,0x6060,0x6070,0xAFAF,0x8060,0x9070,0x5070,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5050,0x6F6F,0x5070,0x5050,0x5060,0x6050,0x6050,0x6050,0x6050,0x5050,0x5060,0x4050,0x7F7F,0x7F7F,0x6050,0x5050,0x6050,0x7050
.word 0x6060,0x6080,0x5070,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x7170,0x6071,0x6F6F,0x4040,0x4050,0x5050,0xBFBF,0x9F9F,0xAFAF,0x8F8F,0x5060,0x9F9F,0x8ACA,0xBFBF,0xCFCF,0xE5EB,0xF7BE,0xAFAF,0x8ACE,0xD7D7,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF7FE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFED7,0xFFFF,0x9F9F,0xCFCF,0xBFBF,0x5040,0x5050,0x5050,0x5060,0x5050,0x6050,0x6050,0x6050,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4050,0x5050,0x6050,0x5060,0x5060,0xAFAF,0x6050,0x6050,0x5050,0x5050,0x4040,0xAFAF,0xE96B,0xDEFE,0x6F6F,0x6F6F,0x9F9F,0x5F5F,0x5F5F,0x7F7F,0xBFBF,0x4B8A,0xBFBF,0x9F9F,0xDE70,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF7DC,0xFFFF,0xFFFF,0xFFFF,0xAFAF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xAFAF,0x6F6F,0x5050,0x5070,0x5050,0x5050,0x8F8F,0x8F8F,0x6060,0x9F9F,0x6060,0x6070,0x8070,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4050,0x5050,0x4060,0x6F6F,0x6050,0x6050,0x5050,0x5060,0x5060,0x5050,0x5050,0x6050,0x5050,0x4050,0x4050,0x7F7F,0x7F7F,0x7F7F,0x4050,0x5060
.word 0xAFAF,0x8060,0x8070,0x9F9F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x7F7F,0x7F7F,0x6F6F,0x5050,0x5040,0x5050,0xBFBF,0x9F9F,0x9F9F,0xAFAF,0x5050,0xAFAF,0xBFBF,0x7A8A,0xAFAF,0xDED7,0xCCE7,0xCFCF,0xCFCF,0xBFBF,0xBCD7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xFEDE,0xBFBF,0xFFFF,0xFFFF,0x8F8F,0x4050,0x6050,0x6050,0x5040,0x5060,0x5060,0x5060,0x5050,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x5050,0x5050,0x6050,0x5050,0x5050,0x5060,0x5050,0x5060,0x5050,0x9F9F,0xAFAF,0xFFFF,0x8F8F,0x4F4F,0xBFBF,0x6F6F,0x4F4F,0x5F5F,0x8AD7,0xBFBF,0xAFAF,0x509E,0xCFCF,0xBE7A,0xFEEB,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xAFAF,0xF8CE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5040,0x5040,0x5F5F,0x5050,0x5050,0x5060,0x5050,0x6050,0x6050,0x6050,0x5050,0x8F8F,0x5050,0x5040,0x6F6F,0x4050,0x4050,0x5040,0x5050
.word 0x6060,0x6060,0x9F9F,0x7070,0x6060,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x9F9F,0xAFAF,0x9F9F,0x6F6F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5040,0x6050,0x6050,0x5050,0x9F9F,0x8F8F,0xAFAF,0xAFAF,0x6060,0x9F9F,0xBFBF,0xAFAF,0xAFAF,0xD7EE,0xD7BE,0xBFBF,0xE6AC,0xD7BE,0xFFFF,0xFFFF,0xF7FB,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF9FE,0xCFCF,0xFFFF,0xFADE,0xFFFF,0x8F8F,0x4050,0x5050,0x5050,0x5070,0x5060,0x6050,0x5050,0x6050,0x6050,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0x6F6F,0x4040,0x4050,0x4040,0x5050,0x5060,0x5060,0x5060,0x5060,0x5050,0x5050,0x5040,0x4050,0x7F7F,0x9F9F,0xDE8A,0xFDFB,0x4F4F,0x8F8F,0xAFAF,0x4F4F,0x6F6F,0xBFBF,0x9F9F,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xF9DE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xAFAF,0xDEDC,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5050,0x4040,0x5F5F,0x6050,0x5070,0x6050,0x5070,0x5060,0x5050,0x5060,0x5060,0x6050,0x8F8F,0x4050,0x7F7F,0x6F6F,0x5050,0x5040,0x5060
.word 0x5060,0x5060,0x6060,0x6070,0x6070,0x8070,0x5060,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x7F7F,0xBFBF,0xF0BE,0xF0DE,0xEFEF,0xEFEF,0xAFAF,0x5F5F,0x5F5F,0x5F5F,0x5050,0x5060,0x5050,0x5070,0x5050,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xCBE9,0xBFBF,0xBFBF,0xEBDE,0xBFBF,0x79BE,0xBFBF,0xCFCF,0xFEEE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xAFAF,0x5040,0x5040,0x6050,0x5050,0x5060,0x5060,0x6050,0x8F8F,0x5050,0x5050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x4040,0x6F6F,0x5040,0x5050,0x5040,0x8F8F,0x8F8F,0x5050,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x5050,0x5F5F,0xAFAF,0xBFBF,0xFFFF,0xBFBF,0x5F5F,0xBFBF,0x7F7F,0x6F6F,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xCFCF,0xCD8A,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5050,0x5F5F,0x5060,0x5050,0x5060,0x5050,0x6050,0x6050,0x5050,0x5050,0x4050,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0x5040,0x6050,0x5050
.word 0x6050,0x6070,0x6070,0x7060,0x7060,0x8080,0x8070,0x4160,0x6F6F,0x6F6F,0x6050,0xC080,0xC090,0xD0C0,0xBEC0,0xAFAF,0xCFCF,0xF0D0,0x61DE,0x5F5F,0x5F5F,0x6F6F,0x6050,0x6050,0x6050,0x6050,0x6050,0xBFBF,0x9F9F,0xAFAF,0xE9D7,0x9F9F,0xCFCF,0xBEC9,0xAFAF,0xD7D7,0xAFAF,0xBFBF,0xAFAF,0xD7EE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFEF7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xFBF9,0xFEF9,0xFFFF,0x9F9F,0x4040,0x4050,0x6F6F,0x5050,0x7060,0x6070,0x5050,0x5060,0x5050,0x4050,0x5040,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x5050,0x4050,0x5050,0x7F7F,0x7F7F,0x4050,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6070,0x7080,0x5F5F,0x6F6F,0x9F9F,0xBFBF,0xFBF7,0x4F4F,0xCFCF,0xBFBF,0x6F6F,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEBD7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFCF9,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x5050,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5F5F,0x5050,0x6050,0x5050,0x5060,0x5060,0x5060,0x6050,0x4050,0x5050,0x9EDE,0x8F8F,0x7F7F,0x7F7F,0x6040,0x5050,0x6040
.word 0x7050,0x9F9F,0x6060,0x6070,0x8080,0x7060,0xBFBF,0x8090,0x6F6F,0x7050,0x9080,0x7080,0xBFBF,0xBFBF,0x8070,0x9070,0x8070,0xBFBF,0xBFBF,0x5090,0x6F6F,0x6F6F,0x5050,0x5050,0x5050,0x5070,0x6050,0x5050,0x9F9F,0xAFAF,0xDA7A,0xBFBF,0xC9BE,0xBFBF,0xBE9A,0xCFCF,0xC7F9,0xAFAF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xFEFD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xCFCF,0xFFFF,0xCFCF,0x8F8F,0x4060,0x4050,0x5050,0x5040,0x6050,0x5060,0x5060,0x5050,0x4050,0x5040,0x4050,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x4050,0x5050,0x5050,0x5040,0x7060,0x5050,0x5050,0x5050,0x5050,0x4050,0x5050,0x5050,0x5070,0x7060,0x7060,0x8060,0x9F9F,0x9F9F,0xAFAF,0xCFCF,0xF7FB,0xBFBF,0x6F6F,0xBFBF,0x6F6F,0xAFAF,0x9F9F,0xBFBF,0xBE9A,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFECC,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xE7E9,0xF7EE,0xFFFF,0xFFFF,0xFFFF,0x9F9F,0x5070,0x4050,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5040,0x6F6F,0x5050,0x6060,0x5060,0x8F8F,0x5050,0x5050,0x8F8F,0x5050,0xBFBF,0x9F9F,0x7F7F,0xF969,0x8F8F,0x6F6F,0x5040,0x4050
.word 0x6050,0x7070,0x7050,0x9F9F,0x6060,0xAFAF,0x8070,0x7070,0x6F6F,0x6F6F,0x8070,0x7080,0x9090,0x7070,0x7080,0x7080,0x7080,0x7060,0x7070,0x7080,0x5080,0x5F5F,0x6F6F,0x6050,0x5070,0x7050,0x5050,0x5070,0x9F9F,0xAFAF,0xBFBF,0xE9BC,0xBFBF,0xD7D7,0xAFAF,0xCFCF,0xBFBF,0x9F9F,0xBFBF,0xAFAF,0x5F5F,0x6F6F,0xFFFF,0xF7F7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFFFF,0xE7BD,0xFFFF,0x42BD,0x6050,0x5050,0x4050,0x5040,0x5040,0x5060,0x6070,0x6050,0x5050,0x5040,0x5050,0x7F7F,0x4050,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x6050,0x5050,0x6070,0x6070,0x6070,0x6070,0x7080,0x5060,0x5060,0x6050,0x5050,0x5040,0x5050,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0x8A7A,0xFFFF,0xF7FE,0xBFBF,0x8AD9,0x9F9F,0xCFCF,0xBFBF,0xAFAF,0xCFCF,0xCFCF,0xFEFB,0xFFFF,0xFFFF,0xFFFF,0xDDE7,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xDDD7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x60DD,0x4050,0x4050,0x4050,0x5040,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4050,0x6F6F,0x5050,0x6050,0x6050,0x6050,0x5060,0x8F8F,0x5060,0x4050,0xBFBF,0x6F6F,0x7F7F,0xFFFF,0xAFAF,0x6F6F,0x4050,0x5050
.word 0x5070,0x5050,0x6070,0x7060,0x7060,0x8070,0x8060,0x8060,0x5080,0x5F5F,0x7070,0x8070,0x7060,0x8080,0x8070,0x8070,0x7070,0x8060,0x6070,0x7060,0x7060,0x4050,0x4050,0x4050,0x6050,0x5050,0x5060,0x5050,0x5060,0x9F9F,0xBFBF,0x7A57,0xC9F9,0xBFBF,0xBFBF,0xAFAF,0xE6E9,0xBFBF,0xBFBF,0x5F5F,0x8F8F,0x7F7F,0xFFFF,0xE6FE,0xEFEF,0xEFEF,0xFFFF,0xAFAF,0xBFBF,0xCFCF,0xFFFF,0xBFBF,0x8F8F,0x5050,0x5050,0x6050,0x4050,0x4050,0x5050,0x7050,0x5050,0x4050,0x8F8F,0x4040,0x4050,0x5040,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5040,0x5050,0x5040,0x5050,0x5070,0x7060,0x8060,0x6060,0x7060,0x7060,0x5060,0x5050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x8F8F,0x496B,0x59BE,0x9F9F,0xBFBF,0xBFBF,0xEE7A,0xFFFF,0xBEFB,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xF9D7,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xFAEC,0xFFFF,0xFFFF,0xFFFF,0xD7EB,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x5050,0x6050,0xBFBF,0x5040,0x5040,0x5040,0x8F8F,0xBFBF,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5050,0x5050,0x5060,0x5050,0x5060,0x5050,0x5040,0x8F8F,0xAFAF,0xCFCF,0xD7FA,0x7F7F,0x6F6F,0x5040,0x5040
.word 0x6060,0x7060,0x9F9F,0x6070,0x6070,0x6060,0x6070,0x6070,0x6070,0x6F6F,0x6040,0x6070,0x7060,0x6060,0x6060,0x9F9F,0x6070,0x6080,0x6060,0x6070,0x6070,0x8060,0xAFAF,0x8F8F,0x5050,0x8F8F,0x5050,0x5050,0x6050,0x6050,0xBFBF,0xAFAF,0xCFCF,0xAFAF,0xAFAF,0xC749,0xAFAF,0xAFAF,0x42D4,0x7F7F,0xD7E6,0x5F5F,0x6F6F,0xBFBF,0xF4E6,0xFFFF,0xFFFF,0xBFBF,0xFAFB,0xCFCF,0xFFFF,0xBFBF,0x5050,0x5060,0x8F8F,0xCEDE,0x5060,0x4050,0x5050,0x5060,0x5070,0x4050,0x4050,0x5050,0x5040,0x4050,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5060,0x5050,0x5070,0x6050,0x7050,0x8060,0x8070,0x7060,0x5060,0x4050,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x9F9F,0xAFAF,0xEB59,0x9AF9,0xAFAF,0xBFBF,0xBFBF,0xFBF7,0xEFEF,0xD7E7,0xCE9A,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xDEBC,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF7E7,0xFFFF,0xFFFF,0xFFFF,0xEED7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x6050,0xCFCF,0xFFFF,0x4050,0x4050,0xBFBF,0xFFFF,0xFFFF,0x4F4F,0x4F4F,0x4F4F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4150,0x5F5F,0x5F5F,0x7F7F,0x8F8F,0x6040,0x5050,0x4050,0x5050,0x5040,0x8F8F,0x8F8F,0x8F8F,0xCFCF,0x6F6F,0x5F5F,0x6F6F,0x4050
.word 0x6070,0x9F9F,0x7050,0x5050,0x7050,0x7050,0x6060,0x8060,0xBFBF,0x6F6F,0x6F6F,0x8060,0x7060,0x8060,0x6070,0x6060,0x7060,0x7060,0x7060,0x6050,0x7050,0x8060,0x7070,0xDE80,0x9F9F,0x5050,0x5040,0x5040,0x5050,0x6050,0x5050,0xBFBF,0x594B,0xCFCF,0xBFBF,0x9F9F,0xBFBF,0x9F9F,0x5F5F,0x6F6F,0xBFBF,0x7F7F,0x6F6F,0xBFBF,0xE4BD,0xFFFF,0x7ADD,0xEB7A,0xBFBF,0xFDD7,0x9F9F,0x4040,0x5050,0x5050,0xBFBF,0xFFFF,0x50BE,0x5050,0x5050,0x5040,0x6060,0x5050,0x5040,0x7F7F,0x7F7F,0x5040,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x7050,0x7060,0x5070,0x5050,0x5060,0x5050,0x7070,0x6070,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xCFCF,0xAFAF,0xEEF9,0xBFBF,0xBE8A,0xAFAF,0xBFBF,0xFEF7,0xBFBF,0xAFAF,0xCFCF,0x7ABE,0xBFBF,0xF98A,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xF7FD,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xF7EE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x9F9F,0xFFFF,0xFFFF,0x50BD,0x5040,0x9F9F,0xFFFF,0xFFFF,0xBFBF,0x4F4F,0x5F5F,0x6F6F,0x8F8F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xBFBF,0x7F7F,0xCE42,0xD7EA,0x5F5F,0x5F5F,0x5050,0x5040
.word 0x7060,0x6060,0x6070,0x5070,0x6070,0x5070,0x7070,0x6060,0x7060,0x4070,0x6050,0x6070,0x6070,0x6070,0xAFAF,0x6080,0x6070,0x9F9F,0x6070,0x7070,0x5060,0x6070,0x6070,0x9060,0xC0DE,0x5040,0x5040,0x5040,0x6050,0x5050,0x6070,0x9F9F,0xBFBF,0xBFBF,0xAFAF,0x49E9,0x9F9F,0xAFAF,0x9F9F,0xC5CE,0xAFAF,0x4768,0x9F9F,0xEFEF,0xAFAF,0xCFCF,0xBFBF,0xFE59,0xBDF9,0xCFCF,0x9F9F,0x5050,0x5050,0x5050,0xBD50,0xEFEF,0xBFBF,0x6050,0x8F8F,0x8F8F,0x5050,0x5050,0x5050,0x8F8F,0xBFBF,0x7F7F,0x5F5F,0x6F6F,0x4050,0x5050,0x6070,0x6060,0x7060,0x5070,0x5050,0x8F8F,0xAFAF,0x6070,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x49D7,0xF769,0xD7FD,0x8A7A,0xBFBF,0xCFCF,0xFFFF,0xD9F9,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xF9EA,0xFEDE,0xFFFF,0xFFFF,0xCFCF,0xFEE9,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xDFDF,0xFFFF,0xCFCF,0x5040,0x9F9F,0xFFFF,0xFFFF,0xEFEF,0x5F5F,0x5F5F,0x4050,0xBFBF,0x6F6F,0x6F6F,0xAFAF,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0xAFAF,0x7F7F,0xCFCF,0x9F9F,0x6F6F,0x5F5F,0x6F6F,0x4050
.word 0x6070,0x7050,0x5050,0x7060,0x5050,0x6050,0x5060,0x6050,0x7060,0x5050,0x6F6F,0x7060,0x6060,0x6060,0x7080,0x6060,0x6070,0x9F9F,0x7060,0x6060,0x7060,0x6050,0x7050,0x6070,0x5080,0x6F6F,0x7F7F,0x5040,0x4050,0x5060,0x5050,0x6070,0xBFBF,0x5949,0x9F9F,0xCFCF,0x7F7F,0x9F9F,0xF9E6,0xBFBF,0x8F8F,0xCD57,0xFDF7,0xD8FB,0xBFBF,0xCFCF,0xBFBF,0xC84B,0xF7F7,0x42BD,0x6F6F,0x5050,0x5050,0x5050,0xBFBF,0xFFFF,0xFFFF,0x5050,0x8F8F,0x50BE,0x8F8F,0x5060,0x8F8F,0xFFFF,0x50F0,0x6F6F,0x6F6F,0x6F6F,0x6050,0x6080,0x8060,0x6070,0x6070,0x5050,0x5050,0x5050,0x5050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x5050,0x6F6F,0x8F8F,0x5050,0xBFBF,0xBFBF,0xFCF8,0xBFBF,0x8A7A,0x8AF9,0xFFFF,0xDEF7,0xCFCF,0xAFAF,0xBFBF,0x6BBE,0xBFBF,0xCFCF,0xCFCF,0xFFFF,0x5F5F,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFEEB,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0x9F9F,0x6F6F,0x6050,0xBFBF,0x70E0,0x7F7F,0x7F7F,0xC060,0x7080,0x6080,0x7F7F,0x4F4F,0x4F4F,0x7F7F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x8F8F,0x5141,0x8F8F,0x9F9F,0x5F5F,0x5F5F,0x6F6F,0x506F
.word 0x9F9F,0x6070,0x9F9F,0x6060,0xAFAF,0x7070,0x7050,0x6070,0x6070,0x5060,0x6F6F,0x6070,0x7050,0x9F9F,0x6070,0x7080,0x5060,0x6070,0x6080,0x6060,0x5070,0x7060,0x5070,0x7060,0x6060,0x6F6F,0x6F6F,0x7F7F,0x5040,0x6040,0x6050,0x5060,0x5170,0xAFAF,0x7A59,0xCFCF,0x7F7F,0x5952,0xCFCF,0xBFBF,0x4759,0xCFCF,0xFFFF,0xBFBF,0x9F9F,0xBFBF,0xF9E9,0xAFAF,0xF7EB,0x8F8F,0x5F5F,0x5050,0x5050,0xAFAF,0xAFAF,0xFFFF,0xEFEF,0xBFBF,0xFFFF,0x50FE,0x5050,0x8F8F,0x9F9F,0xEFEF,0x6F6F,0x5F5F,0x5040,0x7050,0x7060,0x6060,0x6080,0x7060,0x6060,0x6060,0x6F6F,0x5040,0x6F6F,0x6F6F,0x5050,0x6F6F,0x5040,0x5050,0x5040,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0xBFBF,0x8F8F,0xBFBF,0xBFBF,0xBFBF,0xD7FB,0xBFBF,0xBFBF,0xCFCF,0xE9FE,0xCFCF,0xCFCF,0x8ABE,0xBFBF,0xBFBF,0xD7FA,0xF7CE,0x42EE,0x6F6F,0xF9E6,0xFEE9,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFEFB,0xFFFF,0xFFFF,0xBFBF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0x6F6F,0x8040,0x9080,0x7090,0x8F8F,0x9F9F,0x5F5F,0xAFAF,0x8070,0xBFBF,0xCFCF,0x5060,0x6F6F,0x7F7F,0x7F7F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x8F8F,0xAFAF,0xAFAF,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x6070,0x5070,0x6070,0x9F9F,0x7060,0x6060,0x6070,0x9F9F,0x8060,0x8F8F,0x5F5F,0x7060,0x7060,0x8060,0x6060,0x6060,0x8070,0x7060,0x7060,0x7060,0x6050,0x6060,0x5060,0x5060,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5050,0xAFAF,0xAFAF,0xBFBF,0x9F9F,0x9F9F,0xF9F9,0x7AE7,0xAFAF,0xFBD9,0xBFBF,0xBFBF,0xAFAF,0xD77A,0xDDEB,0x7A69,0xFEE7,0xFFFF,0xAFAF,0x6050,0x6050,0x6050,0xD8BE,0xFFFF,0xFFFF,0xCFCF,0xFFFF,0xAFAF,0x9F9F,0xAFAF,0xBFBF,0xFFFF,0xBFBF,0xAFAF,0x5050,0x6050,0x6070,0x6070,0x6070,0x6070,0x6070,0x5050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0x8F8F,0xBCC7,0xAFAF,0xAFAF,0xC749,0xFAFA,0xCFCF,0xDE8A,0xBFBF,0xF7EB,0xD7BE,0xAFAF,0xCFCF,0xCFCF,0x9F9F,0xCFCF,0x7F7F,0x4150,0x6152,0xBFBF,0xF5DD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xFFFF,0xFFFF,0xF9FE,0xCFCF,0x6F6F,0x5040,0xBFBF,0xBFBF,0xBFBF,0x9F9F,0x8F8F,0x5F5F,0x5F5F,0x8090,0x9070,0x7070,0xC090,0xBFBF,0x8F8F,0xAFAF,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x8F8F,0x4041,0xCFCF,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x6060,0x5060,0x6060,0x7060,0x6060,0x6060,0x7050,0x6060,0x6070,0x5060,0x6F6F,0x5070,0x6070,0x6060,0x7070,0x8060,0x9F9F,0x6080,0x9F9F,0x6070,0x6070,0x6050,0x7060,0x7050,0x6070,0x9F9F,0xAFAF,0xCFCF,0xCFCF,0xAFAF,0xCFCF,0xE5BE,0xBCBE,0xCFCF,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xCFCF,0xCFCF,0x8F8F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xF78A,0xFEF5,0xCFCF,0xBFBF,0xFFFF,0x70FE,0x5040,0x5060,0xBFBF,0xCFCF,0xFFFF,0xFFFF,0xDDFE,0xFFFF,0xBFBF,0xFE40,0xBFBF,0xFEBE,0xEFEF,0xFFFF,0x50DE,0x5040,0x6070,0x7060,0x7060,0x7060,0x6060,0x7060,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x4050,0x6F6F,0x5050,0x5040,0x5050,0x5040,0x6F6F,0x4050,0x4050,0x4050,0x4040,0xBFBF,0x9AC9,0xBFBF,0x8F8F,0xF769,0xDEFB,0x7A59,0xF9F9,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xBFBF,0x9F9F,0xCFCF,0x5F5F,0x5F5F,0x9F9F,0x7F7F,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x7AFE,0xFFFF,0xDDF7,0xCCF9,0xCFCF,0x7F7F,0x6F6F,0x4050,0x7060,0x7080,0x9070,0x4060,0x4050,0x5050,0x7F7F,0x6060,0x7070,0x7080,0xAFAF,0x8F8F,0x9F9F,0xAFAF,0x7F7F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0xBFBF,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0xAFAF,0xAFAF,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x5070,0x6060,0x6060,0x6060,0x5060,0x6070,0x6070,0x7050,0x8060,0x6F6F,0x7040,0x5060,0x8060,0x8080,0x7060,0x6080,0x8070,0x6070,0x7070,0x6080,0x7060,0x6070,0x6070,0x6050,0x7050,0x6060,0x8F8F,0x9F9F,0xBFBF,0xE9DE,0xBFBF,0xCFCF,0xE6E7,0xF7FA,0x7F7F,0xAFAF,0xBE7A,0x9F9F,0xDA42,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0xAFAF,0xBFBF,0xBDD7,0xCFCF,0xAFAF,0xFE7A,0xFFFF,0xFFFF,0x6060,0x5050,0xBFBF,0xF7CB,0xFFFF,0xFFFF,0xF7EB,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xFEE6,0xFFFF,0xFFFF,0x9F9F,0x5050,0xBE50,0xBFBF,0x6060,0x8070,0x8090,0x6080,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5050,0x4040,0x6F6F,0x4040,0x4050,0x9F9F,0xBFBF,0x5040,0x5040,0x5050,0x4040,0xBFBF,0xAFAF,0xAFAF,0x8F8F,0xF8F8,0xBFBF,0xBFBF,0xCCE7,0xD7BE,0xAFAF,0xAFAF,0x9F9F,0xBFBF,0x7F7F,0x6F6F,0xBFBF,0xAFAF,0x6F6F,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xDED7,0xBCD7,0xCFCF,0xFAEB,0x6F6F,0x4050,0x8F8F,0x6060,0x9F9F,0x6080,0x5050,0x9F9F,0x8F8F,0x6F6F,0x7050,0x6060,0x6070,0x8A70,0x8F8F,0x7F7F,0xCFCF,0xE9D7,0x5F5F,0x5F5F,0x6F6F,0x8F8F,0xEAD7,0x5F5F,0x8F8F,0xD7C7,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5041,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F5F
.word 0x6050,0x6060,0x6060,0x5070,0x7060,0x9F9F,0x5060,0x6070,0x9F9F,0x6F6F,0x6050,0x7060,0x6080,0x7060,0xAFAF,0x8060,0x8060,0x7060,0x8070,0x9F9F,0x7070,0x9F9F,0x6080,0x6080,0x6070,0x6070,0x8080,0x7070,0xAFAF,0xCFCF,0xEBEE,0xE7DE,0xDEEB,0xF7EE,0xAFAF,0x8F8F,0x9F9F,0xF98A,0xBFBF,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0xD77A,0xAFAF,0x5F5F,0xBFBF,0xBFBF,0xFFFF,0xFFFF,0x50FE,0x6040,0x8F8F,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBEF7,0xDECB,0xFFFF,0xFFFF,0x6F6F,0xAFAF,0xEFEF,0x6080,0x7070,0x7060,0x6060,0x7F7F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x9F9F,0x8F8F,0x6F6F,0x5040,0x5040,0xAFAF,0xF9DB,0x8F8F,0x5040,0x5040,0x4050,0x5050,0x9F9F,0xD7BC,0x9F9F,0x496B,0xBFBF,0xD7DC,0xE78A,0xEEEB,0xD7D7,0xBFBF,0x4852,0x7F7F,0xE7D6,0x8F8F,0xAFAF,0xCFCF,0xE6D7,0x7F7F,0xBFBF,0x7ABD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xCFCF,0xCFCF,0xF7FE,0xE6FB,0x8F8F,0x8F8F,0x6040,0x9F9F,0x8080,0x7060,0x7F7F,0xBFBF,0x5040,0x5F5F,0x7F7F,0x7070,0x7060,0xAFAF,0x7F7F,0x7F7F,0xBFBF,0xF8D7,0x7F7F,0x4F4F,0x8F8F,0xBFBF,0xFFFF,0x7F7F,0xF9C7,0xFFFF,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F4F
.word 0x5070,0x9F9F,0x5070,0x7060,0x6050,0x6060,0x7070,0x7060,0x6080,0x6F6F,0x7050,0x7080,0x8070,0x8080,0x7080,0xAFAF,0x7080,0x7080,0x7070,0x8080,0x7070,0x8060,0x8060,0x7060,0x7060,0x9F9F,0x7060,0x7080,0x7080,0x5F5F,0x8F8F,0xBFBF,0xF7DE,0xDEF7,0xFFFF,0x7F7F,0x8F8F,0xAFAF,0x9F9F,0xCFCF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xBFBF,0xBFBF,0x8F8F,0x5F5F,0xBFBF,0xF7EB,0xFFFF,0xBFBF,0x5050,0xCFCF,0xFECB,0xFFFF,0xF8FD,0xFEFD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFFFF,0xBFBF,0x5F5F,0xBFBF,0xFFFF,0x6060,0x5050,0x6060,0x9F9F,0x5F5F,0x6F6F,0x6F6F,0x5050,0x6F6F,0x4050,0xAFAF,0xDEBE,0x5050,0x5040,0x7F7F,0xAFAF,0xFFFF,0xBFBF,0x5050,0x5040,0x5040,0x5050,0x4050,0xCFCF,0x8AE9,0xBFBF,0xBFBF,0xF9E9,0xBFBF,0xFAE9,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0xE9D7,0xC7BD,0x9F9F,0xBFBF,0x9F9F,0xF7E9,0xEFEF,0xFFFF,0xFEF7,0xFFFF,0xFFFF,0xCFCF,0xBFBF,0xFBDD,0xFFFF,0xFFFF,0x7F7F,0x5040,0x6050,0x8F8F,0xAFAF,0xBFBF,0xBFBF,0x7F7F,0x8F8F,0x9F9F,0x5F5F,0x6F6F,0x6070,0x5070,0xBFBF,0x7F7F,0x8F8F,0xBE49,0xBFBF,0x8F8F,0x4F4F,0x7F7F,0xFAF9,0xFFFF,0x59CE,0xFFFF,0xCFCF,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x7F7F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F5F
.word 0x6060,0x6060,0x6060,0x6060,0x5070,0x6060,0x5050,0xAFAF,0xBFBF,0x6F6F,0x7060,0x7080,0x8090,0x9070,0x8070,0x9070,0x8070,0x8060,0x7070,0x6070,0xBFBF,0x6070,0x6080,0x9F9F,0x7070,0x9F9F,0x6080,0x8080,0x7080,0x6F6F,0x5F5F,0x6F6F,0x8F8F,0xCFCF,0xFFFF,0xE6FA,0x7F7F,0x7F7F,0xAFAF,0xCFCF,0x8F8F,0x8F8F,0x5948,0x9F9F,0xBFBF,0xBFBF,0xE9D7,0x9F9F,0x59BD,0xCFCF,0xFFFF,0xFFFF,0x4070,0x9F9F,0xFEE8,0xFFFF,0xFEF7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xEEC9,0xFFFF,0x7F7F,0xAFAF,0xFFFF,0x9F9F,0x5070,0x6070,0x6070,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x8F8F,0xF9DE,0xEBD8,0x40EE,0x8F8F,0xAFAF,0xDEDE,0xEEDE,0xBEDE,0xBFBF,0xAFAF,0x6050,0x8F8F,0x5050,0x7F7F,0xBFBF,0x9F9F,0x4959,0xBFBF,0xD7D9,0xBFBF,0xD7F9,0x9F9F,0xBFBF,0x9F9F,0x6742,0x4251,0xBFBF,0xCFCF,0xBFBF,0xAFAF,0xBFBF,0x5949,0xCFCF,0xFFFF,0xFEFD,0xFFFF,0xF7FD,0xCFCF,0x9F9F,0xFBFA,0xFEF7,0xD7EB,0x8F8F,0x5040,0x5060,0x8F8F,0x5070,0xAFAF,0x7070,0xAFAF,0x6F6F,0x7F7F,0x7F7F,0x5F5F,0x5F5F,0x5060,0xAFAF,0x9F9F,0x7F7F,0x7F7F,0xAFAF,0xE7D7,0xCFCF,0x5F5F,0xBFBF,0xFFFF,0xFFFF,0x7AE7,0xFBE6,0xFBF7,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F
.word 0x6070,0x7050,0x6070,0xAFAF,0x6070,0x5070,0x5070,0x7060,0x6F6F,0x5F5F,0x9070,0x9090,0x9070,0x7090,0x7090,0x7080,0x7080,0x8070,0x9F9F,0x8070,0x8070,0x8060,0x9F9F,0x8070,0x8080,0x7060,0x8060,0x7070,0x9070,0x6090,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0xCFCF,0xE6FB,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0xBFBF,0x8A7A,0xBFBF,0xCFCF,0xBFBF,0xFE7A,0xFFFF,0xFFFF,0x50DD,0xAFAF,0xFEEB,0xFFFF,0xF7FE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xD7FA,0xE7E8,0xFFFF,0x6F6F,0xAFAF,0xFFFF,0x6F6F,0x7F7F,0x7060,0x5F5F,0x8F8F,0xAFAF,0x6F6F,0x4050,0x5040,0x5040,0xBFBF,0xFDF8,0xD7FE,0xAFAF,0x9F9F,0xBFBF,0xCFCF,0xEEE0,0xEFEF,0xFEFE,0xFEFE,0xCEDE,0xBFBF,0x9F9F,0xAFAF,0xBFBF,0xCFCF,0xBFBF,0xAFAF,0xDCD7,0xCFCF,0xDEE9,0xCFCF,0x9F9F,0x7F7F,0x7F7F,0xAFAF,0xEBE6,0xCAEA,0x7F7F,0xBFBF,0x598A,0x8F8F,0x7F7F,0xFFFF,0xF7F7,0xF9F7,0xCFCF,0xBFBF,0xFBF7,0xCFCF,0xBFBF,0x8F8F,0x9F9F,0x6060,0x7060,0x6060,0xAFAF,0x5050,0xBFBF,0x6080,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x7F7F,0x6050,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xBFBF,0xD7BE,0x7F7F,0xFEE9,0xFFFF,0xFFFF,0xBFBF,0xF7D7,0xFFFF,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F
.word 0x9F9F,0x6070,0x6060,0x7060,0x5060,0x6070,0x6060,0x5060,0x5F5F,0x6F6F,0x8F8F,0x9060,0xC070,0x9080,0x9090,0x7080,0x7080,0x8080,0x9070,0xAFAF,0x6080,0x6080,0x8070,0x6080,0x6070,0x6080,0x8080,0x8070,0x9080,0x4070,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x5142,0x5F5F,0x9F9F,0xBFBF,0xBFBF,0xEBE7,0xBCD7,0xBFBF,0xFFFF,0xFFFF,0x9F9F,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xD7EE,0xDECA,0xFFFF,0xBDE6,0xF9D7,0xFFFF,0x5F5F,0x5F5F,0x7F7F,0x5F5F,0x7F7F,0x9F9F,0x8F8F,0x6F6F,0x4050,0x4050,0xBFBF,0xF8EB,0xCFCF,0x9F9F,0xBFBF,0xBFBF,0xBFBF,0xDEF4,0xEEEE,0xFFFF,0xFEFE,0xFEFE,0xBEFB,0x9A7A,0xBFBF,0xFDEE,0xBFBF,0xC8BC,0xBFBF,0x9F9F,0xCFCF,0xCFCF,0xE7DA,0x597A,0x9F9F,0x4242,0xAFAF,0xFBFE,0xCFCF,0xAFAF,0xC98A,0xBFBF,0x4140,0x5F5F,0xEFEF,0xEBF7,0x7ACD,0x8F8F,0xFAF9,0xBFBF,0xAFAF,0x8F8F,0x5040,0x6050,0x5070,0x6050,0xAFAF,0x5060,0xAFAF,0x7060,0x5070,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6060,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xBFBF,0xAFAF,0xCFCF,0xFFFF,0xFFFF,0xBFBF,0xFDF9,0xBFBF,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F5F
.word 0x6070,0x6070,0x7060,0x6060,0x9F9F,0x7060,0x9F9F,0x5F5F,0x5F5F,0x4050,0x5050,0x9F9F,0x9F9F,0xAFAF,0x7080,0x8070,0x8070,0x7070,0x6080,0x8070,0x8060,0x8070,0x7080,0x8070,0x7070,0x9F9F,0x9F9F,0x7080,0x9070,0xBFBF,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0xCFCF,0x9ABD,0xE9BD,0xE7DD,0xCFCF,0xF7DD,0xFFFF,0xFFFF,0xBFBF,0xFDFA,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCBF8,0xF7E6,0xFFFF,0xF9FC,0xFFFF,0x8F8F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0xBFBF,0xAFAF,0x9F9F,0x5040,0x5040,0xAFAF,0xFAE7,0xF9F7,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xEEDE,0xFFFF,0xFFFF,0xFFFF,0x9F9F,0x9F9F,0xBFBF,0xFAFE,0xBFBF,0xD779,0x4969,0xBFBF,0xCFCF,0xE9EE,0x9F9F,0x9F9F,0x7F7F,0xF942,0xD7E9,0x7F7F,0x7F7F,0xBFBF,0x6057,0x6F6F,0x5F5F,0xBFBF,0xAFAF,0x7F7F,0xF949,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0x7060,0x7060,0x7060,0x6060,0x6050,0x9F9F,0x5070,0xBFBF,0x8F8F,0x6F6F,0x9F9F,0x594B,0x9F9F,0x5F5F,0x5050,0xAFAF,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0xAFAF,0xFDE6,0xFFFF,0xFFFF,0xFAF9,0xFFFF,0x7F7F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F
.word 0x7060,0x7060,0x5060,0x8070,0x7060,0x9F9F,0x6F6F,0x5F5F,0x5040,0x5040,0x6050,0x7080,0xBFBF,0x7060,0xAFAF,0x9F9F,0x9F9F,0x7070,0x8080,0x8060,0x6060,0x6070,0x7070,0x6070,0x8080,0x8070,0x8070,0x8070,0x9070,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x7F7F,0x6F6F,0x4F4F,0x9F9F,0xBFBF,0xCFCF,0xCFCF,0xEBE9,0xCFCF,0xFFFF,0xFFFF,0xEFEF,0xDCCD,0xFEF7,0xFFFF,0xFDFE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xD7F9,0xF7CC,0xFFFF,0xFFFF,0xF7DE,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x9F9F,0x9F9F,0x8F8F,0x4050,0x5060,0xBFBF,0xF7EB,0xF7ED,0xD7E7,0xFBDD,0xDEFE,0xBFBF,0xFFFF,0xFFFF,0xDFDF,0xEFEF,0xFFFF,0xFEFE,0xFFFF,0xBFBF,0x8F8F,0xCFCF,0xFAFA,0xCFCF,0xBED7,0xBFBF,0xCFCF,0xDCE9,0x4BD7,0x7F7F,0x8F8F,0xF9D7,0xBFBF,0x8F8F,0x7F7F,0x8F8F,0xAFAF,0x6F6F,0x5F5F,0x8F8F,0x8F8F,0xBFBF,0xBFBF,0x9F9F,0x8F8F,0x8F8F,0x6060,0x6070,0x6070,0x7060,0x7060,0x6050,0x7060,0xAFAF,0x6050,0x5080,0x6F6F,0x8F8F,0x9F9F,0x8F8F,0x6F6F,0x6F6F,0x4B50,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0xFEFA,0xFFFF,0xF7FD,0xFFFF,0xFFFF,0x5F5F,0x6F6F,0x4F4F,0x4F4F,0x5F5F,0xCFCF,0xF2F0,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x6070,0x6050,0x7060,0x7060,0x5080,0x5F5F,0x5F5F,0x6050,0x6070,0x7060,0x7060,0x7070,0x7070,0x6080,0x7F7F,0x4050,0x5060,0x7050,0x9F9F,0x8060,0x8060,0x6070,0x6080,0x8070,0x7070,0x7080,0x7080,0x7080,0x7080,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x8F8F,0x8A7A,0x5959,0x6F6F,0x6F6F,0xBFBF,0xCFCF,0xCFCF,0xFEFE,0xF9F9,0xCFCF,0xFFFF,0xBBFE,0xF7D7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xD7DC,0xFDE7,0xFFFF,0xD7DC,0xF9F9,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0xCFCF,0xAFAF,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xF7F9,0xFEF8,0xCFCF,0xFEF7,0xFFFF,0xCFCF,0xFFFF,0xFFFF,0xEFEF,0x70BE,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0x8F8F,0x4B49,0xBFBF,0xE9F9,0xCFCF,0xD7D7,0xCFCF,0xCFCF,0xCFCF,0x7F7F,0x7F7F,0xAFAF,0x5949,0x8F8F,0x9F9F,0x4242,0x6F6F,0x6F6F,0x5F5F,0x8F8F,0x8F8F,0xBFBF,0x5949,0x9F9F,0x6040,0x6060,0x7060,0x7060,0x9F9F,0x5070,0x5060,0x6070,0x6070,0x6060,0xAFAF,0x6F6F,0x6F6F,0x7F7F,0x4B59,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0xCE49,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x5F5F,0x8F8F,0x4F4F,0x6F6F,0xCFCF,0xFFFF,0x9F9F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F
.word 0x7060,0x7060,0x6070,0x7070,0x6F6F,0x5F5F,0x4050,0x8080,0x7060,0x6080,0x6070,0x6070,0x7070,0x8060,0x5060,0x6F6F,0x7F7F,0x5050,0x6070,0xAFAF,0x9F9F,0x8070,0x9F9F,0x9F9F,0x9F9F,0x7080,0x7080,0x7080,0x6090,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0xCFCF,0xBFBF,0xBFBF,0x7F7F,0x8F8F,0xACC7,0xBFBF,0xCFCF,0xDDF8,0xE9E7,0xFFFF,0xD7FE,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xE7E7,0xFFFF,0xFBF7,0xECD7,0x9F9F,0x4050,0x4060,0x7F7F,0x4F4F,0x5F5F,0x6F6F,0xBE50,0x9F9F,0xBFBF,0xAFAF,0x6070,0xCFCF,0xFDF7,0xFFFF,0xFAF7,0xFFFF,0xFBF7,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0x50EE,0x5050,0xCFCF,0xFEF7,0xEFEF,0xBFBF,0x7F7F,0x8F8F,0x9F9F,0xCFCF,0xBFBF,0xE6D7,0xE6BE,0xBFBF,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0xAFAF,0x8F8F,0x9F9F,0x8F8F,0x6060,0x7060,0x5070,0x5060,0x8060,0x6060,0x7070,0x5060,0x5060,0x5060,0x6060,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0xBFBF,0xCFCF,0xFFFF,0xFEF7,0xFFFF,0xBFBF,0x7F7F,0x6BF9,0x7F7F,0xAFAF,0xBFBF,0xFFFF,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F
.word 0x6060,0x6070,0x7070,0x7F7F,0x4050,0x7050,0x7060,0x6060,0x8080,0x8060,0x6060,0xBFBF,0x7060,0x6080,0x6080,0x5F5F,0x5F5F,0x7F7F,0x5050,0x7050,0x8070,0x7060,0x7070,0x6070,0x8070,0x9F9F,0x7080,0xBE80,0x7F7F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x9F9F,0x7F7F,0xBFBF,0xEEBE,0xCFCF,0xCFCF,0x6F6F,0xBFBF,0x7A7A,0x7A7A,0xBFBF,0xBFBF,0xCFCF,0x8CD7,0xBFBF,0xF8DE,0xCFCF,0xCFCF,0xCFCF,0xF7CB,0xFEF7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCCE7,0xFFFF,0xFFFF,0xCFCF,0x9F9F,0x5050,0x5050,0x4070,0x6F6F,0x6F6F,0x5040,0x9F9F,0xAFAF,0x9F9F,0xCFCF,0xAFAF,0xD79A,0xF7EB,0xF8F7,0xFDFE,0xFFFF,0xFFFF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xEE70,0xBFBF,0xBFBF,0xEFEF,0xFFFF,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0xCFCF,0xBFBF,0x7F7F,0x8F8F,0x7F7F,0x576B,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x5060,0x7070,0x5060,0x7060,0x6070,0x6060,0x6070,0x6070,0x7050,0x5070,0x7060,0x5050,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xAFAF,0xBFBF,0xFFFF,0xFFFF,0xCFCF,0xFFFF,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0xBFBF,0xEFEF,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F
.word 0x4060,0x6050,0x8F8F,0x4050,0x7050,0x6060,0x6070,0x7070,0x7070,0x6060,0x7070,0x7060,0x6070,0x6080,0x6080,0x5060,0x5F5F,0x6F6F,0x6050,0x6050,0xAFAF,0x7080,0x7070,0x7060,0x6060,0x7070,0xAFAF,0x5060,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0xCFCF,0x9F9F,0x5F5F,0xBFBF,0xAFAF,0xBFBF,0x5959,0xBFBF,0xCFCF,0xE68A,0xCFCF,0xFEE7,0xF9F7,0xCFCF,0xBDF7,0xFED7,0xFFFF,0xFEFB,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF7F9,0xFFFF,0xCCF9,0x59D7,0x5040,0x6050,0x6050,0x5050,0x4060,0x6F6F,0x4040,0xAFAF,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x69BE,0xFFFF,0xFBFE,0xFFFF,0xFFFF,0xFEF7,0xD7D7,0xFDDE,0xFFFF,0xFFFF,0xBFBF,0xEFEF,0x70F7,0x6050,0xAFAF,0xCFCF,0xFFFF,0xBFBF,0x6F6F,0x8F8F,0x8F8F,0xE979,0xAFAF,0xE6BE,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x8F8F,0x4040,0x6070,0x6070,0x5050,0x7070,0x5060,0x7050,0x7060,0x7050,0x9F9F,0x5060,0x7060,0x6050,0x5060,0x5F5F,0x6F6F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x7F7F,0x8F8F,0xFFFF,0xCFCF,0xFE7A,0xFBF7,0xAFAF,0xBFBF,0x7F7F,0x9F9F,0xAFAF,0xFFFF,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x7F7F,0x406F
.word 0x5040,0x6060,0x7060,0x7060,0x7070,0x8060,0x8060,0x8060,0x8060,0x7060,0x7060,0x7060,0x6080,0x8060,0x8060,0x6080,0x5F5F,0x6F6F,0x7F7F,0x8F8F,0x5060,0x7060,0x7070,0x7080,0x7070,0x8060,0x5060,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x5040,0x5050,0x4040,0xBFBF,0xD6E7,0x7F7F,0x6F6F,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xC88A,0xCFCF,0xBFBF,0xCFCF,0xFAFE,0xCFCF,0xD7FE,0xBFBF,0xFEF7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFEFD,0xFFFF,0xFEFD,0xD7E7,0xBFBF,0x5050,0x7F7F,0x8F8F,0x4050,0x4050,0x4050,0x5050,0x8F8F,0x9F9F,0x597A,0xAFAF,0xBFBF,0xCFCF,0xF9F8,0xF7EE,0xFBF8,0xFFFF,0xFFFF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFFFF,0xAFAF,0xCFCF,0xCFCF,0xFFFF,0xBEFE,0xCFCF,0xBFBF,0x6F6F,0x7F7F,0x8F8F,0xAFAF,0xCFCF,0xCFCF,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x5060,0x7050,0x7050,0x7060,0x6060,0x7060,0x5060,0x5070,0x6060,0x6070,0x7050,0x9F9F,0x9F9F,0x6060,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x4F4F,0x6F6F,0x5F5F,0x8F8F,0xAFAF,0xDEE6,0xBFBF,0xCFCF,0xBFBF,0xEE7A,0x7F7F,0x8F8F,0x9F9F,0xBFBF,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F50
.word 0x7060,0x6060,0x7070,0x8060,0x7060,0x7060,0x6060,0x7070,0x6080,0xBFBF,0x7070,0x8060,0x6060,0x6070,0x8070,0x7070,0x6F6F,0x5F5F,0x6F6F,0x8F8F,0x8F8F,0x9F9F,0x6070,0x8060,0x6060,0x5080,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5050,0x5040,0x6F6F,0x6F6F,0x5040,0x5040,0x4040,0x5040,0x6F6F,0x8F8F,0x69F9,0x9F9F,0xAFAF,0xD7BC,0xBFBF,0xAFAF,0xBFBF,0xBD8A,0xCFCF,0xDDDB,0xFFFF,0xD7FE,0xCFCF,0xE5CB,0xF7E6,0xF7F9,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFFFF,0xFFFF,0xBFBF,0x8F8F,0x5040,0xBFBF,0x7F7F,0x8F8F,0x7F7F,0x5040,0x6F6F,0x5F5F,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0xBFBF,0xCFCF,0xBFBF,0xFEE7,0xFEF7,0xFFFF,0xCFCF,0xFBF8,0xFFFF,0xDEF7,0xFFFF,0xFFFF,0xAFAF,0xCFCF,0xDEBE,0xFFFF,0xFFFF,0x5050,0xBFBF,0xD6E6,0xAFAF,0x7F7F,0x7F7F,0xBFBF,0xAFAF,0xBFBF,0x7F7F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6040,0x5050,0x5070,0x6060,0x6070,0x7060,0x7050,0x7060,0x6060,0x6070,0x5050,0x7070,0x7060,0x9F9F,0x8F8F,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0xAFAF,0xBFBF,0x8F8F,0xBFBF,0x494B,0xBFBF,0x7F7F,0x9F9F,0xBFBF,0x8F8F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F
.word 0x6070,0x6080,0x6080,0x6080,0x7070,0x7070,0x7070,0x8060,0x8060,0x7060,0x6070,0x6070,0x6070,0x9F9F,0x9F9F,0x8080,0x5070,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x6F6F,0x5F5F,0x5F5F,0x60F0,0x4050,0x5050,0x5040,0x6F6F,0x4040,0x4050,0x4050,0x5050,0x4050,0x5F5F,0x7F7F,0xCFCF,0x7AD7,0x9F9F,0xBFBF,0xCCC9,0xBFBF,0xBFBF,0xE7F9,0xCFCF,0xAFAF,0xCFCF,0xCFCF,0xF9D6,0x9F9F,0xEBF9,0xDDE6,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xF959,0xFFFF,0xF8FB,0x8F8F,0x5040,0xBFBF,0x49EE,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x4050,0x6F6F,0x7F7F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xCFCF,0xAFAF,0xFFFF,0xFEF7,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xBFBF,0x9F9F,0xEFEF,0xFFFF,0x7060,0x7060,0x5050,0x6050,0xAFAF,0xAFAF,0x6F6F,0x6F6F,0x9F9F,0xBFBF,0x8F8F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6050,0x6050,0x5060,0x6060,0x6070,0x6070,0x6070,0x7050,0x6060,0x6070,0x5060,0x5070,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0xBFBF,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0xBFBF,0x8F8F,0xAFAF,0xBFBF,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5050,0x5040,0x6F6F,0x5050
.word 0x6060,0x6070,0x7070,0x7070,0x7060,0x8060,0x7060,0x6060,0x6080,0x6070,0x8060,0x6060,0x9F9F,0x8060,0x8070,0xBFBF,0x8070,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x8F8F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x9F9F,0x6050,0x5050,0x4040,0x4050,0x5050,0x5040,0x5040,0x4040,0x6F6F,0x5F5F,0x6F6F,0xAFAF,0xD8BD,0x9F9F,0x7A59,0xBFBF,0x79CC,0xBFBF,0xBFBF,0xBDF9,0xCFCF,0xCFCF,0xE6F9,0xCFCF,0x57C7,0xBFBF,0xF5D7,0xFFFF,0xFFFF,0xFFFF,0x6BD7,0xAFAF,0xFFFF,0xD7E7,0x9F9F,0x4050,0xFABD,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5040,0x5F5F,0x8F8F,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xBFBF,0xBFBF,0xCFCF,0xF7FD,0xD7EE,0xBE7A,0xFFFF,0xFFFF,0xBFBF,0xF7FD,0x59F9,0xBFBF,0xAFAF,0xDEF7,0xBFBF,0x7060,0x6050,0x6070,0x5060,0x5060,0xAFAF,0x5052,0x5F5F,0x6F6F,0x8F8F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5050,0x5060,0x6070,0x6070,0x6050,0x7060,0x7060,0x7060,0x7060,0x7060,0x4050,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x6F6F,0x7F7F,0x5F5F,0x5F5F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x5050,0x4040
.word 0x8060,0x7060,0x7060,0x6060,0x7070,0x7060,0x7070,0x8070,0xAFAF,0x8060,0x6070,0x7060,0x6060,0x6070,0x6050,0x7060,0x7080,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5050,0x5060,0x4050,0x4050,0x6F6F,0x4040,0x4050,0x4050,0x5050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xBFBF,0xAAF9,0x8F8F,0x7A49,0xBFBF,0xBFBF,0x9F9F,0xE7E6,0xCFCF,0xBFBF,0xAFAF,0xD652,0xBFBF,0x8AC6,0xBFBF,0xEFEF,0xFFFF,0xCFCF,0x9F9F,0xFB8A,0xFFFF,0xAFAF,0x5041,0x9F9F,0x57F9,0x6F6F,0x4050,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x6F6F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x6949,0xBFBF,0x9F9F,0xCFCF,0xCFCF,0xBFBF,0xFFFF,0xFFFF,0xF959,0xFFFF,0xCFCF,0x8F8F,0xBFBF,0x8F8F,0x6050,0x5060,0x6060,0x5050,0x6050,0x7050,0x6050,0x5160,0x5040,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5050,0x7060,0x7060,0x7060,0x7060,0x6070,0x6060,0x6070,0x7070,0x6070,0x5050,0x6F6F,0x7F7F,0x8F8F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x9F9F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x9F9F,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5050,0x5040,0x6F6F,0x4050,0x4050,0x4040,0x5050
.word 0x7060,0x6060,0x6070,0x7070,0x6080,0x7080,0x7060,0x6060,0x6080,0x6080,0x6060,0x6070,0x7060,0x6060,0x7070,0x7060,0x8070,0x5070,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x4050,0x4040,0x6050,0x5050,0x5060,0x4050,0x4050,0x5050,0x7F7F,0x5040,0x8F8F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0x8AD7,0xAFAF,0x7A59,0xD7BC,0xBFBF,0xAFAF,0xD6D6,0xAFAF,0x52E6,0xBFBF,0x48C5,0xAFAF,0xAFAF,0xEFEF,0xF7F4,0x79F7,0xAFAF,0xFFFF,0xFFFF,0xBFBF,0xAFAF,0xF7FD,0x6F6F,0x4050,0x4050,0x4150,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x4050,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0x5952,0xAFAF,0xD7BE,0xE649,0xFFFF,0xBFBF,0xFFFF,0xBFBF,0x8F8F,0x9F9F,0xAFAF,0x7050,0x6060,0x5050,0x5060,0x7050,0x5060,0x5050,0x5050,0x5060,0x6060,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x6060,0x6070,0x6060,0x6070,0x6060,0x7070,0x6060,0x7070,0x6060,0x6070,0x7060,0x4060,0x5040,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x8F8F,0x7F7F,0x7F7F,0x9F9F,0x6F6F,0x8F8F,0x6F6F,0x5041,0x5F5F,0x5F5F,0x5F5F,0x5040,0x4040,0x6F6F,0x5F5F,0x5050,0x5040,0x4050,0x4050
.word 0x7F7F,0x5050,0x7050,0x8060,0x7060,0x6070,0x6060,0x6080,0x8060,0x7060,0x7060,0x7060,0x7060,0x7060,0x6060,0x6070,0x9F9F,0x5070,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x6F6F,0x5040,0x5050,0x4040,0x5050,0x5050,0x5040,0x5040,0x4040,0x5040,0x8F8F,0xEFEF,0xFFFF,0xFFFF,0xEFEF,0xF0FE,0xEFEF,0xFFFF,0xCFCF,0xBFBF,0xAFAF,0xCFCF,0x69BD,0xBFBF,0x9F9F,0xBFBF,0xBFBF,0x59C6,0xBFBF,0x5763,0x5752,0xE451,0xCFCF,0xBFBF,0xBFBF,0xFEF9,0xBFBF,0x7F7F,0xAFAF,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x7F7F,0x8F8F,0xAFAF,0x9F9F,0x8F8F,0xAFAF,0x9F9F,0xBFBF,0xBFBF,0x9F9F,0xBFBF,0xF7BE,0xBFBF,0xFFFF,0x6BD7,0x9F9F,0x4B49,0x8F8F,0x6060,0x6060,0x5070,0x6050,0x5050,0x5050,0x5060,0x5050,0x6050,0x6050,0x5050,0x5050,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x6070,0x7050,0x6070,0x7060,0x6070,0x6080,0x7080,0x6060,0x6070,0x6070,0x7080,0xAFAF,0x9070,0x9090,0x4070,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x6F6F,0x7F7F,0x8F8F,0x5F5F,0x5F5F,0x7F7F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5050,0x5040,0x4050
.word 0x5F5F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xBFBF,0x6070,0x7060,0x6060,0x6070,0x5070,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x4050,0x4050,0x5040,0x4050,0x5050,0x5040,0x5050,0x5040,0x5050,0x5050,0xCFCF,0xFEF0,0xEFEF,0xEFEF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF7FD,0xCFCF,0x9F9F,0x7A59,0xCFCF,0xBFBF,0x5969,0xBFBF,0xE663,0x8F8F,0xBFBF,0xBFBF,0xBFBF,0x5257,0xE7F4,0x7AC7,0x59D7,0xFFFF,0xBFBF,0xBFBF,0xD7F9,0x6F6F,0x4050,0x4050,0x7F7F,0x4050,0x5040,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0x49D7,0xCFCF,0x49E6,0xBFBF,0xD7D7,0xF77A,0xBFBF,0x8F8F,0x8F8F,0x9F9F,0x6050,0x7050,0x5050,0x6050,0x6060,0x5060,0x5060,0x6050,0x6050,0x5050,0x5050,0x5060,0x5060,0x6050,0x5040,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x6060,0x6060,0x7060,0x6060,0x7060,0x6060,0x6060,0x6070,0x7060,0x8060,0x8060,0x7060,0x9070,0x9070,0x7180,0x4260,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x8F8F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5050,0x5050,0x5050,0x5050
.word 0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x4050,0x5050,0x5050,0x5050,0x5040,0x5050,0x5050,0x5060,0x4040,0x8F8F,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xDDFB,0xBFBF,0x8F8F,0xC76B,0xCFCF,0xBFBF,0xAFAF,0xBFBF,0x7F7F,0x9F9F,0xBC49,0xD6D7,0x4761,0xBCE6,0xBFBF,0xBFBF,0xCFCF,0x8F8F,0xBFBF,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x7F7F,0xBFBF,0x9F9F,0x8F8F,0xBFBF,0x8AD7,0xBFBF,0xCFCF,0x59D6,0x6F6F,0x8F8F,0x5040,0x6070,0x6060,0x7060,0x5050,0x5060,0x5070,0x5050,0x4050,0x5050,0x5050,0x5060,0x5050,0x5050,0x5050,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x8F8F,0x5060,0x5060,0x5070,0x6070,0x7060,0x7060,0x7060,0x8060,0x7060,0x6060,0x6070,0x8060,0x9070,0x9090,0x4150,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5050,0x5050,0x5040
.word 0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4050,0x5040,0x5040,0x5040,0x4040,0x5050,0x5050,0x5040,0x5050,0x5050,0x6050,0x5050,0x5060,0x6F6F,0x6F6F,0xBFBF,0xFEEE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBDF9,0xCFCF,0x9F9F,0xD77A,0xBFBF,0xAFAF,0xBFBF,0x7F7F,0x596B,0xBFBF,0xBFBF,0xD7D7,0x7F7F,0x9F9F,0xF7BC,0x48DD,0xBFBF,0x7F7F,0x6F6F,0x4050,0x4050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7AD7,0x7F7F,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0x8F8F,0x6F6F,0x7F7F,0x6050,0x5050,0x5050,0x5060,0x7050,0x7050,0x5050,0x5060,0x6050,0x5050,0x5050,0x5050,0x5050,0x6050,0x6040,0x5050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x6050,0x6050,0x7050,0x6050,0x7070,0x8060,0x9F9F,0x6070,0x7070,0x9F9F,0x6060,0x9F9F,0x7080,0x9080,0x5060,0x5F5F,0x6F6F,0x4250,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5050,0x5050,0x5040,0x5050
.word 0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5040,0x4050,0x4050,0x4050,0x5050,0x4040,0x4050,0x5050,0x5040,0x5050,0x5060,0x6050,0x5050,0x5050,0x6F6F,0x6F6F,0x6040,0xCFCF,0xDFDF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBDFA,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0x7A7A,0x8F8F,0x7F7F,0xC77A,0xD7D7,0xBFBF,0x7F7F,0xBFBF,0xE6D7,0x9F9F,0x59E6,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0xAFAF,0x7F7F,0xBFBF,0x7F7F,0x7F7F,0xCFCF,0x7AC7,0xBFBF,0xBFBF,0x5F5F,0x6F6F,0x4050,0x5050,0x5070,0x5060,0x6050,0x5070,0x5050,0x7050,0x6050,0x5050,0x5050,0x5050,0x5050,0x5040,0x4040,0x5050,0x5050,0x4040,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x8F8F,0x6050,0x6050,0x6070,0x6060,0x6070,0x6070,0x8060,0x6060,0x6080,0x5070,0x6060,0x8070,0x8070,0x9F9F,0x5F5F,0x4040,0x5040,0x5050,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5040,0x5F5F,0x5F5F,0x5050,0x5050,0x5050,0x5050
.word 0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x5040,0x4050,0x4050,0x5050,0x6050,0x5050,0x6050,0x5050,0x5060,0x4050,0x6F6F,0x5050,0x5040,0x5050,0x5050,0xBFBF,0xDDF5,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x7F7F,0x596B,0x8F8F,0xCFCF,0x8F8F,0x7F7F,0x9F9F,0x42BD,0xBFBF,0x6F6F,0x6F6F,0x5050,0x5040,0x5040,0x5040,0x5040,0x5040,0x4050,0x5050,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0x4B59,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7A59,0xBFBF,0x9F9F,0x6F6F,0x6F6F,0x5050,0x5040,0x5060,0x7050,0x6050,0x5050,0x6050,0x6050,0x5050,0x5070,0x5060,0x5050,0x5050,0x5050,0x5050,0x5060,0x6050,0x7F7F,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x7F7F,0x5050,0x7050,0x6070,0x6060,0x8070,0x6060,0x6070,0x6070,0x7050,0x7050,0x7070,0x7080,0x5050,0x5F5F,0x5F5F,0x5040,0x6050,0x7060,0x5060,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5040,0x5040,0x4050,0x6F6F,0x5F5F,0x5040,0x4050,0x5050,0x5050
.word 0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x4050,0x4050,0x5050,0x5050,0x5050,0x5060,0x5050,0x5050,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4040,0x5040,0x4040,0x6F6F,0xAFAF,0xBFBF,0xFEFE,0xDFDF,0x6F6F,0x8F8F,0x59BD,0x9F9F,0x7F7F,0x8F8F,0xBFBF,0x8F8F,0x7F7F,0x7F7F,0xAFAF,0x8F8F,0x9F9F,0x5F5F,0x5060,0x5050,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x5040,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xBFBF,0x6F6F,0x5F5F,0x5050,0x4040,0x4050,0x6040,0x5050,0x5050,0x5060,0x5060,0x5050,0x5070,0x5050,0x5050,0x5060,0x6050,0x5050,0x5050,0x5040,0x7F7F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x8F8F,0xAFAF,0x7060,0x6060,0x8060,0x7060,0x6060,0x6070,0x6070,0x6070,0x5070,0x4150,0x5F5F,0x5F5F,0x4050,0x5050,0x6060,0x7070,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x5040,0x6F6F,0x4050,0x5050,0x5060,0x4040
.word 0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x5050,0x6050,0x5050,0x6050,0x4040,0x5050,0x4050,0x6F6F,0x5050,0x6F6F,0x4050,0x6F6F,0x5050,0x4050,0x5050,0x6F6F,0x5F5F,0x5040,0x6060,0xAFAF,0x5F5F,0x6F6F,0x8F8F,0xBFBF,0x9F9F,0x9F9F,0xBFBF,0x5959,0x7F7F,0x8F8F,0x9F9F,0x8F8F,0x6F6F,0x5040,0x5040,0x4050,0x4050,0x4050,0x4050,0x5040,0x5040,0x4050,0x4050,0x5040,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0xAFAF,0x6F6F,0x5F5F,0x6F6F,0x4040,0x5040,0x5050,0x5060,0x6050,0x6050,0x5050,0x6050,0x6050,0x6050,0x6050,0x6050,0x6050,0x5050,0x5060,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x4150,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x8F8F,0x8060,0xAFAF,0x6070,0xAFAF,0x7060,0x6060,0x7060,0x5060,0x5050,0x5F5F,0x4F4F,0x6F6F,0x6050,0x7050,0x6060,0x5050,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x7F7F,0x4060,0x6F6F,0x5F5F,0x5050,0x5050,0x4040,0x5050
.word 0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4050,0x4050,0x4050,0x4050,0x5050,0x5050,0x5050,0x5040,0x5040,0x5040,0x4040,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x4050,0x4040,0x6F6F,0x5040,0x6F6F,0x5F5F,0x6F6F,0x5050,0x5050,0x5060,0x5F5F,0x5F5F,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0xBFBF,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6050,0x5060,0x5050,0x5040,0x4050,0x4050,0x4050,0x4050,0x4050,0x5040,0x5040,0x6F6F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0xAFAF,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x6050,0x7060,0x5050,0x5060,0x5060,0x5050,0x5050,0x5060,0x5060,0x5050,0x6050,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5041,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5050,0x7070,0x6060,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5050,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x7F7F,0xAFAF,0x9F9F,0xAFAF,0x8F8F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x5050,0x6050,0x5050,0x5040
.word 0x6070,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x5050,0x5040,0x5050,0x5050,0x5040,0x5050,0x7F7F,0x4050,0x4050,0x5050,0x5050,0x5F5F,0x5050,0x5040,0x5040,0x5050,0x6F6F,0x6F6F,0x5040,0x6050,0x7050,0x5060,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0xAFAF,0xAFAF,0xAFAF,0x6F6F,0x6F6F,0x5050,0x5050,0x6050,0x5050,0x5050,0x5040,0x5050,0x5040,0x5050,0x5040,0x5040,0x5040,0x4040,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x6050,0x6050,0x5050,0x5060,0x9F9F,0x6060,0x6050,0x6050,0x6070,0x9F9F,0x6050,0x6050,0x8F8F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x7F7F,0x5050,0x4150,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5050,0x9F9F,0x5050,0x8F8F,0x9F9F,0x5040,0x6F6F,0x6F6F,0x5F5F,0x5040,0x4050,0x5040,0x4060
.word 0x7060,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x4050,0x8F8F,0x5050,0x4050,0x5050,0x4050,0x4050,0x4040,0x4050,0x4050,0x4050,0x5050,0x5060,0x5F5F,0x6F6F,0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6050,0x5050,0x5050,0x5060,0x5040,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6050,0x6050,0x6050,0x5050,0x5050,0x5060,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x5050,0x6F6F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x8F8F,0x5060,0x5060,0x7050,0x6060,0x9F9F,0x5050,0x6060,0x6050,0x6070,0x5050,0x8F8F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x9F9F,0x6F6F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x6040,0xAFAF,0x6060,0x9F9F,0x9F9F,0x5050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5050,0x4060,0x5050
.word 0x6060,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x4050,0x5050,0x6F6F,0x4040,0x4050,0x5050,0x6F6F,0x6F6F,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5050,0x5060,0x5060,0x5060,0x6050,0x6050,0x5050,0x6050,0x5050,0x5050,0x5050,0x5040,0x5040,0x5040,0x4F4F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x5050,0x8F8F,0x6060,0x9F9F,0x7060,0x6050,0x8F8F,0x9F9F,0x8F8F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x4050,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x8F8F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x4040,0x5060,0x5050,0x8F8F,0x5050,0x5040,0x5050,0x4050,0x6F6F,0x5F5F,0x4050,0x8F8F,0x6040,0x5050
.word 0x6050,0x7F7F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5050,0x6050,0x6040,0x5050,0x6050,0x5050,0x6060,0x4050,0x6F6F,0x6F6F,0x5050,0x6050,0x5050,0x6050,0x5050,0x5060,0x6050,0x5050,0x5050,0x4050,0x4050,0x4050,0x4050,0x5F5F,0x5F5F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x7F7F,0x4050,0x8F8F,0x8F8F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x6F6F,0x7F7F,0xBFBF,0x9F9F,0x7F7F,0x6F6F,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x5040,0x5050,0x5040,0x5040,0x4040,0x5050,0x5040,0x5050,0x5060,0x5060,0x8F8F,0x5060,0x4050,0x5F5F,0x6F6F,0x6F6F,0x4050,0x4050,0x5050
.word 0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0x9F9F,0xAFAF,0x9F9F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x5F5F,0x4050,0x5050,0x5050,0x4050,0x6050,0x5050,0x5050,0x5050,0x5050,0x5040,0x5F5F,0x5060,0x5060,0x5050,0x5060,0x5060,0x6050,0x5050,0x5050,0x4040,0x4050,0x5050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0xBFBF,0xFEBD,0xEFEF,0xEFEF,0xDDFE,0xBFBF,0x6060,0x9F9F,0x4060,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4040,0x4050,0x5050,0x5040,0x8F8F,0x5060,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F7F
.word 0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x7F7F,0x9F9F,0xEEDE,0xBFBF,0x8F8F,0x7F7F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x6F6F,0x5F5F,0x5040,0x5040,0x5040,0x6040,0x5050,0x5050,0x5050,0x5060,0x4060,0x4050,0x6F6F,0x5050,0x5050,0x5050,0x5050,0x5050,0x5060,0x5050,0x4050,0x4060,0x5050,0x5040,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xF0DE,0xBDFE,0xFED0,0xD0BD,0xCFCF,0x6080,0x4060,0x5F5F,0x5F5F,0x5040,0x6F6F,0x5F5F,0x5F5F,0x4050,0x5050,0x6F6F,0x4050,0x5040,0x5040,0x5040,0x5050,0x5040,0x4040,0x5050,0x5060,0x6050,0x6050,0x4050,0x6F6F,0x6F6F,0x4050,0x5050,0x5F5F,0x5F5F
.word 0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0xAFAF,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5050,0x4050,0x5050,0x5050,0x5050,0x6040,0x5040,0x5040,0x5040,0x6F6F,0x5050,0x6050,0x5050,0x6050,0x6050,0x4050,0x5060,0x5050,0x5040,0x5040,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5040,0x6F6F,0x5040,0x6040,0x6050,0x5050,0x5050,0x6060,0x7060,0x7060,0xAFAF,0xBD90,0xCFCF,0xDEEE,0xFEF0,0xEFEF,0xFFFF,0xDDFE,0xAFAF,0x4050,0x6F6F,0x5040,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4040,0x5040,0x5040,0x5050,0x5050,0x5060,0x5060,0x5040,0x5F5F,0x6F6F,0x4050,0x4050,0x5F5F,0x5F5F
.word 0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x8F8F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x5040,0x5040,0x5040,0x5050,0x5050,0x5050,0x5040,0x5040,0x6F6F,0x5040,0x4050,0x5050,0x5060,0x5050,0x5040,0x5050,0x5050,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x5050,0x5040,0x5040,0x5040,0x6050,0x6050,0x5050,0x8F8F,0x5060,0x6060,0x9F9F,0x9F9F,0xCFCF,0xCDC0,0xCFCF,0xF0BD,0xEFEF,0xEFEF,0xBFBF,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5040,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x7F7F,0x4050,0x4050,0x5040,0x5050,0x5040,0x8F8F,0x7F7F,0x5F5F,0x5050,0x8F8F,0x8F8F,0xAFAF,0x5F6F
.word 0x4050,0x5040,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5070,0x6050,0x5050,0x5050,0x4070,0x5040,0x5040,0x5040,0x5040,0x4050,0x5040,0x4050,0x6F6F,0x7F7F,0xAFAF,0x7F7F,0x6F6F,0x6F6F,0x4050,0x4050,0x5050,0x5050,0x5050,0x4050,0x4050,0x4050,0x6F6F,0x6F6F,0x6050,0x5050,0x5050,0x5050,0x4050,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x5040,0x5050,0x5050,0x4050,0x5060,0x5060,0x5050,0x5060,0x6050,0x7050,0x6050,0x6050,0x7060,0x5050,0x7050,0x9F9F,0x6070,0x9F9F,0xBFBF,0x9F9F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x6F6F,0x5040,0x5040,0x5040,0x7F7F,0x5050,0x4040,0x5040,0x5040,0x6050,0x5040,0x5F5F,0x6F6F,0xAFAF,0xBE60,0xCFCF,0x5F7F
.word 0x5040,0x6F6F,0x5040,0x5040,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xAFAF,0x6060,0x6070,0x7060,0x5060,0x6050,0x4050,0x4050,0x5050,0x5060,0xAFAF,0x4050,0x5040,0x5040,0x6070,0x5040,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0x8F8F,0x4050,0x5040,0x5050,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x5040,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x7F7F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x4050,0x4050,0x4050,0x5050,0x6050,0x5050,0x5040,0x5050,0x5050,0x5050,0x5060,0x5050,0x5060,0x5050,0x8F8F,0x5060,0xAFAF,0x7060,0x9F9F,0x5060,0x5050,0x5040,0x7F7F,0x4050,0x5F5F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x4050,0x4050,0x6F6F,0x4050,0x4050,0x5050,0x4040,0x5050,0x4050,0x4050,0x4050,0x5040,0x5F5F,0x8F8F,0x6050,0xCFCF,0xAFAF,0x5F9F
.word 0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5050,0x6070,0x6070,0x6060,0x5060,0x5070,0x5050,0x5060,0x5050,0x5040,0x5040,0x5050,0xBFBF,0x4040,0x5050,0x5050,0x5050,0x5050,0x6F6F,0x6F6F,0x6040,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x4050,0x4050,0x5050,0x6050,0x5050,0x5050,0x5050,0x5050,0x5060,0x5060,0x5050,0x5050,0x9F9F,0x5040,0x5060,0x5050,0x5050,0x5060,0x5050,0x5050,0x5060,0x6050,0x5050,0x5050,0x9F9F,0x5040,0x5040,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x4040,0x4050,0x4050,0x5040,0x8F8F,0x5040,0x5040,0x5040,0x7F7F,0x5F5F,0x7040,0x9F9F,0x9F9F,0x9F9F,0x5F40
.word 0x5040,0x5040,0x5050,0x4040,0x6F6F,0x4050,0x6F6F,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5040,0x4050,0x5050,0x5050,0x5050,0x7050,0x6060,0x5060,0x5060,0x6050,0x6060,0x5050,0x5060,0x5050,0x5050,0x5060,0x4050,0x5050,0x4040,0x5050,0x5060,0x4040,0x6F6F,0x5F5F,0x6050,0x5060,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x8F8F,0x8F8F,0x9F9F,0x8F8F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x5040,0x6F6F,0x5040,0x4050,0x5060,0x5060,0x6050,0x5050,0x5040,0x7F7F,0x5060,0x8F8F,0xDEE9,0xAFAF,0x5050,0x4050,0x6050,0x5040,0x5040,0x5060,0x5050,0x5060,0x8F8F,0x5060,0x5040,0x5050,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x4040,0x5050,0x5040,0x5040,0x4050,0x5050,0x8F8F,0x8F8F,0x4050,0x6F6F,0x5F5F,0x4050,0x5050,0x5040,0x5050,0x5F7F
.word 0x4050,0x4050,0x4040,0x6F6F,0x5040,0x6F6F,0x5040,0x6F6F,0x4050,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x4050,0x5050,0x4050,0x4050,0x7050,0x5060,0x5060,0x5060,0x6060,0x7050,0x6050,0x5050,0x5060,0x5060,0x5050,0x5050,0x5050,0x5050,0x5040,0x5040,0x4050,0x6050,0x5040,0x4050,0x6F6F,0x5F5F,0x5050,0x5040,0x6050,0x6050,0x5050,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x7F7F,0xBFBF,0xC0CD,0xBFBF,0xBDE0,0xFED0,0xDDF0,0x8F8F,0x5F5F,0x7F7F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x5050,0x5050,0x5040,0x4050,0x6F6F,0x5F5F,0x6F6F,0x5050,0x9F9F,0xFADD,0xBFBF,0x4050,0x8F8F,0x4050,0x4050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x5040,0x5040,0x4040,0x4050,0x5040,0x4050,0x5050,0x5040,0x6F6F,0x5F5F,0x6F6F,0x8F8F,0x8F8F,0x7F7F,0x5F6F
.word 0x4050,0x6F6F,0x5050,0x6F6F,0x4050,0x4050,0x6F6F,0x5050,0x6F6F,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x5050,0x6050,0x5050,0x6040,0x5050,0x5060,0x5070,0x5050,0x6050,0x5050,0x5050,0x6050,0x5060,0x5050,0x6050,0x5050,0x5060,0x4060,0x4050,0x4050,0x6040,0x4050,0x6050,0x5040,0x5041,0x5F5F,0x6F6F,0x5050,0x5050,0x5050,0x5060,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x4050,0x5040,0x8060,0x6070,0x6060,0x5060,0x6050,0xAFAF,0xBFBF,0xAFAF,0xAFAF,0xFFFF,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5050,0x5050,0x5040,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x9F9F,0xBFBF,0xF78A,0x50DD,0x8F8F,0x5040,0x5040,0x5040,0x4040,0x5060,0x8F8F,0x4060,0x5050,0x4040,0x5040,0x6050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x5050,0x5040,0x4050,0x5050,0x4050,0x4050,0x6F6F,0x5F5F,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F
.word 0x5040,0x4050,0x6F6F,0x5050,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x5050,0x6050,0x5050,0x5050,0x5050,0x6050,0x6050,0x5050,0x5060,0x5060,0x5070,0x5060,0x5060,0x5060,0x5060,0x5060,0x6040,0x6050,0x5050,0x4050,0x4050,0x5060,0x6040,0x5050,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5050,0x5050,0x5050,0x4050,0x4050,0x5060,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0x6F6F,0x4040,0x4050,0x4050,0x5050,0x8F8F,0x6060,0xAFAF,0xAFAF,0x7060,0x9F9F,0x6060,0x9F9F,0xBD80,0xFFFF,0x6F6F,0x4F4F,0x4F4F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5040,0x4050,0x4050,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0xAFAF,0xBB79,0xFBF7,0xCFCF,0x8F8F,0x4050,0x4050,0x7F7F,0x5050,0x4040,0x5050,0x8F8F,0x4050,0x6040,0x6050,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5040,0x5040,0x4040,0x5050,0x4040,0x5050,0x7F7F,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x4050,0x5040,0x4050,0x4050,0x5050,0x6F6F,0x5040,0x4050,0x6F6F,0x5040,0x5050,0x5F5F,0x5F5F,0xAFAF,0x5F5F,0x6F6F,0x5041,0x5040,0x4050,0x6050,0x5050,0x5060,0x6060,0x5060,0x6060,0x5050,0x5060,0x6050,0x5050,0x7050,0x5050,0x5060,0x5060,0x6060,0x6060,0x5050,0x5060,0x5050,0x5050,0x5040,0x5050,0x5050,0x5050,0x5060,0x5050,0x5050,0x6F6F,0x5F5F,0x5050,0x5040,0x4050,0x5050,0x5040,0x6050,0x5040,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x5050,0x6F6F,0x4050,0x5F5F,0x5F5F,0x6F6F,0x5050,0x6F6F,0x4050,0x5050,0x5040,0x6F6F,0x4040,0x5050,0x7060,0x6060,0x6060,0x5070,0x7050,0xAFAF,0x7060,0xAFAF,0xFFFF,0xCFCF,0x4F4F,0x8F8F,0x6F6F,0x4050,0x5050,0x6F6F,0xBFBF,0x5F5F,0x6F6F,0x5050,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x8A79,0xAFAF,0xE7F9,0xF7FA,0xF9CC,0x6F6F,0x4050,0x4050,0x4050,0x5040,0x5040,0x4040,0x7F7F,0x6F6F,0x5050,0x8F8F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5040,0x5040,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x5040,0x4050,0x5040,0x5040,0x5040,0x4050,0x4050,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x8F8F,0xAFAF,0x6F6F,0x5040,0x4050,0x6050,0x5050,0x6060,0x5050,0x6060,0xAFAF,0x60BE,0x5060,0x6050,0x5050,0x5060,0x5060,0x6060,0x6050,0x5060,0x6060,0x5050,0x5060,0x5050,0x5060,0x4060,0x5060,0x6040,0x5050,0x5050,0x5040,0x5060,0x4050,0x6F6F,0x6F6F,0x5040,0x5050,0x5050,0x5050,0x5060,0x5050,0x4050,0x5050,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x6F6F,0x5F5F,0x4050,0x5040,0x4050,0x4050,0x5040,0x5040,0x5050,0x5050,0x6040,0x9F9F,0x9F9F,0x5050,0xCFCF,0xBFBF,0x6060,0x6070,0xAFAF,0xFFFF,0xFFFF,0x5F5F,0x8F8F,0x6F6F,0x5040,0x4040,0x6F6F,0xF9F7,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x7F7F,0xCFCF,0xE7E9,0xF8EB,0xF7F9,0xD8F8,0x8F8F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x5F5F,0x4050,0x8F8F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x4050,0x4050,0x4050,0x4050,0x5040,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x6F6F,0x5040,0x4050,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x4050,0x4050,0x6F6F,0x6F6F,0x7F7F,0xBFBF,0xFEFD,0x5040,0x5040,0x8F8F,0x5050,0x6050,0x5050,0x6070,0x9F9F,0xBFBF,0xEFEF,0x6050,0x5050,0x6050,0x6060,0x5050,0x5050,0x5060,0x5050,0x6050,0x6050,0x5050,0x6050,0x5050,0x5050,0x5040,0x4050,0x5050,0x5050,0x6050,0x4050,0x4150,0x6F6F,0x6F6F,0x4050,0x5050,0x5040,0x5040,0x5040,0x5040,0x6050,0x4050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x5F5F,0x6F6F,0x5040,0x6F6F,0x5040,0x5040,0x4050,0x4050,0x4040,0x5040,0x5050,0xAFAF,0xFECE,0xAFAF,0xDD50,0xCDFE,0xBFBF,0x9F9F,0xBFBF,0xFFFF,0xFFFF,0x9F9F,0xFFFF,0x50BE,0x6F6F,0x5F5F,0xBFBF,0xF9FE,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0xBFBF,0xBCF9,0xEBC8,0x8AF8,0xCFCF,0xD7DC,0xFFFF,0xAFAF,0x4050,0x4050,0x4050,0x7F7F,0x5F5F,0x4050,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x4050,0x4050,0x6F6F,0x6F6F,0x7F7F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x5040,0x5040,0x4040,0x5050,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x6F6F,0x5040,0x7F7F,0xAFAF,0xBFBF,0xFFFF,0xCFCF,0x5040,0xEE50,0x50FE,0x5060,0x6050,0x6050,0xCE50,0xFFFF,0xBFBF,0x5050,0x5050,0x4050,0x5050,0x5060,0x7070,0x5050,0x5050,0x5060,0x5060,0x5050,0x5060,0x6050,0x6050,0x6050,0x6050,0x4050,0x5060,0x5060,0x5060,0x4050,0x6F6F,0x5F5F,0x4050,0x4050,0x5050,0x5040,0x5050,0x5040,0x5040,0x5040,0x4050,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5050,0x5040,0x5040,0x5040,0x5F5F,0x6F6F,0x5050,0x5040,0x5050,0x5040,0x4040,0x4050,0x7F7F,0x5050,0x5060,0x8F8F,0xFFFF,0xFEFE,0xCFCF,0xFFFF,0xFEFE,0x60DD,0xBFBF,0xEFEF,0xFFFF,0xFFFF,0xFBF7,0x8F8F,0x7F7F,0x6F6F,0x9F9F,0xFFFF,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0x8F8F,0xFFFF,0xD7F7,0xF959,0xBFBF,0xD77A,0xE7EB,0xEFEF,0x50DE,0x5050,0x5040,0x5040,0x5040,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x4050,0x6F6F,0x6F6F,0x4050,0x6F6F,0x4050,0x6F6F,0x4040,0x4050,0x4050,0x5040,0x4040,0x6F6F,0x4040,0x6F6F,0x5040,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x4050,0x4050,0x5050,0x4040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x6F6F,0xAFAF,0x9F9F,0xAFAF,0xFFFF,0xAFAF,0x5040,0x9F9F,0xBFBF,0x5050,0x5060,0x8F8F,0xF7CE,0xFFFF,0x70FE,0x6050,0x5050,0x5060,0x5050,0x7050,0x6050,0x6050,0x5050,0x5050,0x6060,0x5050,0x7050,0x5050,0x5050,0x5050,0x4050,0x5050,0x5050,0x5050,0x5050,0x5040,0x6F6F,0x5F5F,0x6F6F,0x5050,0x5040,0x4050,0x4050,0x4050,0x4050,0x4050,0x7F7F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x5F5F,0x6F6F,0x4040,0x4050,0x4040,0x4050,0x5050,0x7F7F,0x6F6F,0x6050,0x6050,0x6050,0xCFCF,0xFFFF,0xEFEF,0xFFFF,0xFFFF,0xFEFE,0x8F8F,0xFFFF,0xFFFF,0xEFEF,0xFFFF,0xFFFF,0x6F6F,0x7F7F,0xF7DE,0xFFFF,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xAFAF,0x6F6F,0x6F6F,0xAFAF,0x5959,0xEAF9,0xEBF7,0xBFBF,0x59BB,0xBFBF,0xFAF9,0xFAF7,0x8F8F,0x5060,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x4050,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x5040,0x5040,0x6F6F,0x6F6F,0x4050,0x4050,0x5040,0x5040,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0xBFBF,0xCE7A,0xFFFF,0xF0F7,0x5040,0xFE70,0xEFEF,0x6050,0x6050,0xBFBF,0xFEFA,0xFFFF,0xFFFF,0x6050,0x5070,0x6050,0x6050,0x5050,0x5060,0x5070,0x5050,0x7050,0x5050,0x5060,0x5060,0x5060,0x6050,0x6050,0x5050,0x7050,0x6050,0x5050,0x5040,0x5050,0x5040,0x5F5F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x5050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x5F5F,0x5F5F,0x5050,0x5050,0x4050,0x4050,0x5040,0x5050,0x5040,0x5050,0x5050,0x8F8F,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xF7FE,0xFFFF,0xAFAF,0xBFBF,0xFFFF,0xFFFF,0xF7FD,0xFFFF,0x6F6F,0xBFBF,0xFFFF,0xFFFF,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0x5F5F,0x7F7F,0x7F7F,0x9F9F,0xF9E7,0xF9F8,0x49C8,0xBFBF,0x797A,0xFDF7,0xF7F7,0x60F7,0x5040,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x5050,0x4060,0x5F5F,0x5F5F,0x5050,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5050,0x4040,0x6F6F,0x4050,0x6F6F,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F50
.word 0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0xBFBF,0xBFBF,0xAFAF,0xFFFF,0xFEFE,0xBFBF,0xFFFF,0xFFFF,0xCFCF,0x5050,0xAFAF,0xEBEE,0xFFFF,0x70FE,0x5050,0x6050,0x6070,0x5060,0x6050,0x5050,0x7050,0x5050,0x5050,0x5070,0x6050,0x5060,0x6050,0x5070,0x5060,0x7050,0x5050,0x4050,0x5060,0x5050,0x5050,0x6F6F,0x6F6F,0x5F5F,0x5050,0x5050,0x5040,0x5040,0x5040,0x5040,0x4040,0x4050,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x6050,0x5050,0x4050,0x6F6F,0x5F5F,0x5050,0x5050,0x5050,0x5040,0x4050,0x4040,0x5050,0x5060,0x5060,0x5050,0xBFBF,0xF7EB,0xFDFE,0xEFEF,0xEFEF,0xFFFF,0xFFFF,0xBFBF,0xFEBD,0xFFFF,0xFFFF,0xFFFF,0x5F5F,0x6F6F,0xFFFF,0xFFFF,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0xAFAF,0x6F6F,0x5F5F,0x8F8F,0x7A59,0xE7D7,0xD7BC,0xBFBF,0x797A,0xFE59,0xF7F7,0xFDF8,0x9F9F,0x4050,0x5040,0x5040,0x7F7F,0x4050,0x4050,0x4050,0x5050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5050,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F5F
.word 0x5040,0x5040,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x4050,0x5050,0x5F5F,0xAFAF,0xBFBF,0xBFBF,0xE9FA,0xFEFA,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x60FE,0x9F9F,0xCBCE,0xFFFF,0xFFFF,0x50EE,0x5050,0x6050,0x5050,0x6060,0x5060,0x5060,0x6070,0x5060,0x7050,0x5050,0x6060,0x5070,0x7070,0x5050,0x6060,0x5060,0x5060,0x6050,0x5050,0x5060,0x5060,0x6F6F,0x6F6F,0x7F7F,0x6040,0x5050,0x5060,0x5040,0x5040,0x5040,0x5040,0x5040,0x7F7F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5050,0x4050,0x4050,0x5050,0x5050,0x5070,0x5060,0x5F5F,0x5F5F,0x5050,0x5060,0x5060,0x5050,0x8F8F,0x5050,0x5050,0x6060,0x5050,0xAFAF,0x8A60,0xFEE7,0xFFFF,0xF7FE,0xFFFF,0xFFFF,0xF7FE,0xC7FE,0x9F9F,0xF7FE,0xFFFF,0xFDF7,0x7F7F,0x8F8F,0xFFFF,0xFFFF,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0xD7DE,0xAFAF,0x6F6F,0x6F6F,0x7F7F,0xBFBF,0xBFBF,0xD67A,0x79BC,0xBFBF,0xCFCF,0xFDF7,0xF7F9,0x9F9F,0x7F7F,0x7F7F,0x5040,0x5040,0x7F7F,0x4050,0x5050,0x4050,0x5050,0x6F6F,0x5F5F,0x6F6F,0x5050,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x6F6F,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F
.word 0x4050,0x4050,0x4050,0x6F6F,0x4050,0x5050,0x5040,0x6F6F,0x7F7F,0x8F8F,0x9F9F,0xBFBF,0xDE8A,0xFEF7,0xFFFF,0xDDD7,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xBFBF,0xCFCF,0xFFFF,0xFFFF,0x70BE,0x5050,0x5060,0x5060,0x8F8F,0x9F9F,0x5050,0x5050,0x5070,0x5060,0x5070,0x7050,0x7050,0x5060,0x7070,0x6060,0x7050,0x7050,0x5050,0x5060,0x5050,0x5040,0x4050,0x6F6F,0x6F6F,0x5050,0x5050,0x5050,0x4050,0x4050,0x4050,0x4050,0x4050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4040,0x5040,0x5040,0x6050,0x6050,0x5050,0x5050,0x5F5F,0x6F6F,0x5040,0x6050,0x8F8F,0x5060,0x5070,0x4050,0x5040,0x5070,0x7050,0xBDBD,0xCFCF,0xF78A,0xF8FB,0xFBF8,0xFDF7,0xFFFF,0xFFFF,0xFEF7,0xAFAF,0xF7F9,0xFEFE,0xFFFF,0xBFBF,0xAFAF,0xFFFF,0xFFFF,0x5F5F,0x5F5F,0x5F5F,0x9F9F,0xAFAF,0x5949,0x6F6F,0x6F6F,0x9F9F,0x6F6F,0x5959,0xBFBF,0xD7D7,0xD759,0xF9E9,0xF7F7,0xFFFF,0x8F8F,0xAFAF,0x8F8F,0x4050,0x4050,0x5050,0x5040,0x5040,0x5060,0x6050,0x5F5F,0x6F6F,0x6F6F,0x5070,0x5F5F,0x5F5F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x4050,0x4050,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F5F
.word 0x5040,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x8F8F,0xBFBF,0xBFBF,0xBFBF,0xD79C,0xF7EE,0xEEF7,0xCFCF,0xFEFA,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xFBD7,0xFFFF,0xEFEF,0xAFAF,0x6060,0x5050,0xAFAF,0xDEBE,0xDEFE,0x5070,0x6070,0x7060,0x6050,0x7060,0x6060,0x6060,0x6070,0x6060,0x6060,0x6050,0x5060,0x5060,0x6050,0x5050,0x5060,0x5050,0x6F6F,0x5F5F,0x6050,0x6050,0x5050,0x5050,0x5050,0x5040,0x5040,0x5040,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x5050,0x5040,0x5060,0x5060,0x5050,0x5060,0x5060,0x5F5F,0x5F5F,0x6050,0x8F8F,0x6070,0x6050,0x5050,0x4050,0x6060,0x7060,0x5060,0xBFBF,0xE8E9,0xCFCF,0xEBF7,0xF7E7,0xF7FB,0xF7FD,0xFFFF,0xFFFF,0x7ADE,0xCFCF,0xFFFF,0xFFFF,0xF9FB,0xAFAF,0xFFFF,0xFFFF,0x5F5F,0x6F6F,0xAFAF,0xFFFF,0x8F8F,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x497A,0x9F9F,0xBFBF,0xFAF8,0xEBF7,0xF7F7,0x9F9F,0xBD70,0x7F7F,0x4050,0x5050,0x5040,0x5040,0x5040,0x6050,0x5050,0x5F5F,0x5F5F,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F
.word 0x4050,0x4050,0x6F6F,0x5040,0x5050,0x5050,0x5040,0x5040,0x8F8F,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xFEF7,0xE9F7,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xF7FE,0xCBE9,0xFFFF,0xFFFF,0xFFFF,0x6080,0x5050,0x9F9F,0xCFCF,0xFFFF,0xAFAF,0x6050,0x5070,0x6070,0x7060,0x9F9F,0x7060,0x6070,0x6070,0x6070,0x6070,0x6070,0x6070,0x5070,0x6070,0x6050,0x5050,0x4060,0x6F6F,0x6F6F,0x5050,0x5050,0x5060,0x5060,0x5060,0x4050,0x4050,0x4050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4040,0x5050,0x6040,0x5050,0x5060,0x6050,0x5040,0x6F6F,0x5F5F,0x7F7F,0x7050,0x5050,0x5060,0x8F8F,0x5050,0x7050,0x6070,0x5060,0x9F9F,0xE7EC,0xD7EB,0xCFCF,0xEBFE,0xFAF7,0xFEF7,0xF7F7,0xFFFF,0xD7F7,0xBFBF,0xF7FD,0xFFFF,0xFFFF,0xCFCF,0xFBE7,0xFFFF,0x6F6F,0x5F5F,0xFFFF,0xFFFF,0x7F7F,0xFE42,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xAFAF,0xD78A,0xD779,0xFEF8,0xF7F8,0xFFFF,0x9F9F,0xCFCF,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x6050,0x4050,0x5050,0x5F5F,0x5F5F,0x7040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F
.word 0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x8F8F,0xCFCF,0xBFBF,0x69BE,0xBFBF,0xFBF9,0xFAF7,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEEDC,0xFFFF,0xFFFF,0xFFFF,0x5060,0x5070,0xBFBF,0xFFFF,0xFFFF,0xCFCF,0x7050,0x7060,0x6060,0x6070,0x6050,0x9F9F,0x7060,0x7060,0x7060,0x7060,0x6060,0x6050,0x5050,0x5050,0x4060,0x5050,0x5040,0x6F6F,0x5F5F,0x6F6F,0x6050,0x6050,0x5040,0x5040,0x5050,0x5040,0x5040,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5050,0x5040,0x4050,0x5050,0x5040,0x4050,0x4050,0x6F6F,0x5F5F,0x5040,0x5050,0x8F8F,0x8F8F,0x6070,0x8F8F,0x6070,0x7060,0x6050,0xCFCF,0xEBE7,0xCFCF,0xD7CC,0xD7DB,0xF8F9,0xF8F7,0xFBF7,0xFFFF,0xFFFF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFFFF,0xFFFF,0x5F5F,0xBFBF,0xFFFF,0xE9F7,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0xBFBF,0xCFCF,0xCFCF,0xF8F7,0xF7F7,0xF7FD,0xBDF5,0x8F8F,0x5040,0x5040,0x4050,0x5050,0x5050,0x4050,0x5060,0x5060,0x5F5F,0x5F5F,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5050,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F
.word 0x5040,0x4050,0x5050,0x5040,0x5040,0x4050,0x8F8F,0x4050,0x8F8F,0xAFAF,0xBFBF,0xCFCF,0x697A,0xEEDE,0xF9EE,0xDCF9,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF7F7,0xFFFF,0xFFFF,0xCFCF,0x7070,0x9F9F,0xFEFE,0xEFEF,0xFFFF,0x5070,0x5060,0x6060,0xAFAF,0x6060,0x6060,0x6070,0x6070,0x6060,0x6070,0x9F9F,0xAFAF,0xBFBF,0x9F9F,0x8F8F,0x8F8F,0x4050,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x4050,0x4050,0x4050,0x4050,0x5040,0x7F7F,0x5F5F,0x5F5F,0x7F7F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x4040,0x4050,0x5040,0x5050,0x6F6F,0x5F5F,0x4050,0x5040,0x5040,0x5060,0x5050,0x6050,0x7060,0x6060,0x6060,0xBFBF,0xE7AB,0xCBE9,0xCFCF,0xCFCF,0xCFCF,0xF9E9,0xFFFF,0xFDF7,0xFDF7,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xC7DE,0xFFFF,0xFFFF,0x6F6F,0xF942,0xFFFF,0xD7FB,0xCFCF,0xD8E9,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x5958,0xBFBF,0xCBD7,0xEEF9,0xFEF7,0xFFFF,0xF7DE,0x7F7F,0x6F6F,0x5050,0x5050,0x5040,0x6040,0x5050,0x6050,0x5050,0x5F5F,0x6F6F,0x5040,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0x4050,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F
.word 0x4050,0x4050,0x4040,0x4050,0x4050,0x5040,0xBFBF,0x8F8F,0x7F7F,0xD7CC,0xBFBF,0x8AE6,0xBFBF,0xE9C8,0xCFCF,0xE7EE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0x6060,0xCFCF,0xEFEF,0xFFFF,0xBFBF,0x8060,0x7060,0x6070,0x7060,0xAFAF,0x60C0,0x7080,0x8F8F,0x8F8F,0x6050,0xBFBF,0xBFBF,0xAFAF,0x7070,0x5060,0x8F8F,0x8F8F,0x6050,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x4050,0x5050,0x5040,0x5040,0x5040,0x5040,0x5040,0x7F7F,0x6F6F,0x5050,0x6F6F,0x4050,0x7F7F,0x5040,0x4050,0x4050,0x6F6F,0x5F5F,0x6F6F,0x5040,0x5050,0x8F8F,0x6050,0x5050,0x6050,0x6070,0x6070,0x9F9F,0xBFBF,0xCFCF,0xD7DA,0xF9CC,0xD7E9,0xD7BB,0xFBF7,0xFFFF,0xFFFF,0xD7F9,0xF7DD,0xFFFF,0xFFFF,0xCFCF,0xFEF9,0xFFFF,0x6F6F,0xF7CC,0xFFFF,0xD7FE,0xF9F7,0xF98A,0x59F9,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0xCFCF,0xCFCF,0xF8F9,0xFFFF,0xFFFF,0xFBD7,0x7F7F,0x4050,0x4050,0x5040,0x5040,0x5040,0x5050,0x4050,0x5050,0x5F5F,0x6F6F,0x5070,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5070,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F
.word 0x4050,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0xBFBF,0xBFBF,0x7F7F,0xBFBF,0xBFBF,0xBFBF,0x8AD7,0xBFBF,0xE7BC,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0xFAEB,0xFFFF,0xFFFF,0x60DE,0x5050,0x5050,0x5050,0x9E70,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0x5040,0x9F9F,0xBFBF,0x6060,0x6080,0x9F9F,0x5070,0x5050,0x5060,0x4050,0x4050,0x6F6F,0x6F6F,0x8F8F,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x9F9F,0x6F6F,0x5F5F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x5040,0x4040,0x4050,0x4050,0x6F6F,0x5040,0x5040,0x6F6F,0x5F5F,0x6F6F,0x5050,0x5040,0x4050,0x4050,0x5040,0x5040,0x5040,0x5F5F,0x6F6F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7050,0x8F8F,0x7060,0x5070,0xBFBF,0xCFCF,0xCFCF,0xE7D7,0xCFCF,0xD7D7,0xCFCF,0xFFFF,0xFEF7,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xD7ED,0xEFEF,0xFFFF,0x7F7F,0xFDF7,0xFFFF,0xFBE7,0xD7EE,0xAFAF,0x59E7,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0xBFBF,0xF9F9,0xFFFF,0xFFFF,0xCFCF,0xCFCF,0x6F6F,0x5040,0x5040,0x4050,0x4050,0x5050,0x5050,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5060,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x5040,0x5040,0x4040,0x9F9F,0x8F8F,0x4040,0x9F9F,0x8ABE,0xAFAF,0xBCC7,0xAFAF,0xD7C7,0xBFBF,0xDED7,0xCFCF,0xFEFB,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x8F8F,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0x5060,0x5050,0x8F8F,0x8F8F,0x6050,0x8F8F,0x5060,0x7050,0x6060,0x8F8F,0x6060,0x6070,0x7070,0x6060,0x7070,0x6060,0x6070,0x5050,0x5050,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5050,0x8F8F,0x6F6F,0x5F5F,0x5F5F,0x7F7F,0xAFAF,0x5059,0x6F6F,0x5F5F,0x4050,0x5040,0x4040,0x5050,0x5040,0x5040,0x5040,0x4050,0x4050,0x4050,0x5F5F,0x6F6F,0x5040,0x4040,0x7F7F,0x5040,0x5040,0x7F7F,0x5040,0x5F5F,0x5F5F,0x5040,0x5040,0x7F7F,0xAFAF,0xAFAF,0xAFAF,0x8F8F,0x7060,0x9F9F,0x7F7F,0xCFCF,0xD78A,0xCFCF,0xBCE7,0xF9FE,0xCBD7,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFEFE,0xFFFF,0xCFCF,0xFFFF,0xAFAF,0xF949,0xFFFF,0xFFFF,0xFDFE,0x7AE7,0x9F9F,0xBFBF,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0xBFBF,0xF9D7,0xD7FE,0xFFFF,0xCFCF,0x5959,0xBFBF,0x6F6F,0x5040,0x5040,0x4040,0x7F7F,0x7F7F,0x5040,0x5050,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x7F7F,0x5060,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F5F
.word 0x4050,0x4050,0x5050,0x8F8F,0xBFBF,0x8F8F,0x9F9F,0xCFCF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xE9BE,0xF9EE,0xF7F7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFEFE,0xFFFF,0xFFFF,0xAFAF,0xFFFF,0xFFFF,0xFFFF,0xAFAF,0x5040,0x4050,0x6050,0x5050,0x5060,0x5050,0x5050,0x4050,0x5050,0x6050,0x6050,0x7050,0x6050,0x6070,0x5060,0x5060,0x6050,0x5060,0x6050,0x5040,0x6F6F,0x6F6F,0x5040,0xBFBF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xAFAF,0xCFCF,0x6F6F,0x6F6F,0x6F6F,0x5040,0x4050,0x5050,0x4040,0x4050,0x4050,0x4050,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x6F6F,0x5050,0x5040,0x7F7F,0x4050,0x4050,0x4050,0x6F6F,0x5F5F,0x6F6F,0x4050,0x7F7F,0x8F8F,0xCFCF,0xBFBF,0xAFAF,0xAFAF,0x8F8F,0x8F8F,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xE7D7,0xE6DC,0xCFCF,0xF7FB,0xFFFF,0xF7F8,0xFEF9,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFEF9,0xFFFF,0xFFFF,0xF8F7,0xBFBF,0xAFAF,0xBFBF,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0xBFBF,0xBFBF,0xCFCF,0x8F8F,0x7A69,0x6F6F,0x6F6F,0x7F7F,0x5050,0x6F6F,0x6F6F,0x4050,0x5050,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F4F
.word 0x6F6F,0x4050,0x6F6F,0x4050,0xBFBF,0xCE7A,0x9F9F,0xBFBF,0xBFBF,0xBE8A,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xDEE9,0xF7FE,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xF7CE,0xFDFD,0xFFFF,0xFFFF,0xAFAF,0x5050,0x5050,0x5050,0x5070,0x6050,0x5050,0x5050,0x4060,0x5040,0x5050,0x5050,0x5070,0x5050,0x7050,0x6070,0x6060,0x6050,0x8F8F,0x8F8F,0x4050,0x6F6F,0x5F5F,0x5050,0x9F9F,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0xBFBF,0xF979,0x41FE,0x9F9F,0x5F5F,0x4050,0x4050,0x5040,0x4050,0x4050,0x4050,0x5050,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x5040,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x5050,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0xBFBF,0xBFBF,0xBFBF,0xE9BD,0xE7DD,0xE7FE,0xE7F9,0xFEFE,0xFFFF,0xE6FE,0xFFFF,0xFEFE,0xFFFF,0xFEFE,0xFFFF,0xD7F7,0xF5CC,0xFFFF,0xFFFF,0xD7E9,0xAFAF,0xBFBF,0x6959,0x6F6F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x5959,0xAFAF,0xBFBF,0xD759,0xAFAF,0x8F8F,0x5F5F,0x6F6F,0x5F5F,0x7F7F,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F5F
.word 0xBFBF,0x8F8F,0x5040,0x6F6F,0x8F8F,0xBFBF,0xBFBF,0xAFAF,0xE98A,0xAFAF,0xCFCF,0xBFBF,0x9F9F,0xBFBF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFEF7,0xFBF8,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xBFBF,0x5050,0x5050,0x5050,0x6050,0x5050,0x5070,0x6050,0x6050,0x5060,0x6050,0x5050,0x5060,0x6050,0x6060,0x5050,0x5050,0x9F9F,0x5060,0xBFBF,0xBDFE,0x5050,0x6F6F,0x5F5F,0x6F6F,0xBFBF,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0xAFAF,0xBFBF,0xAFAF,0xBD58,0x6F6F,0x5040,0x5040,0x4050,0x5040,0x5040,0x5040,0x4040,0x6F6F,0x4050,0x4050,0x4050,0x5050,0x4050,0x6F6F,0x5050,0x6F6F,0x5040,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x8F8F,0xAFAF,0xCBE6,0xAFAF,0x8AE9,0xBFBF,0xBFBF,0xBDC9,0xBFBF,0xCFCF,0xEBE7,0xCFCF,0xFEE6,0xFFFF,0xF7DD,0xFFFF,0xFFFF,0xF7FE,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFDE6,0xF7F7,0x59BB,0x8AEE,0x9F9F,0x69D6,0xBFBF,0x5959,0x7F7F,0x7F7F,0x6F6F,0x4259,0xBFBF,0xBFBF,0x69C6,0xCFCF,0x9F9F,0xAFAF,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F4F
.word 0xEFEF,0xFEFE,0xBEFE,0x9F9F,0x9F9F,0x8AD7,0xBFBF,0x9F9F,0xAFAF,0xCFCF,0xBFBF,0xCFCF,0xAFAF,0xD6AE,0xBFBF,0xCFCF,0xEFEF,0xFFFF,0xEFEF,0xEEF7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFDF7,0xCFCF,0x5050,0x5040,0x5070,0x5060,0x6050,0x5050,0x6060,0x5060,0x7050,0x4050,0x7050,0x5050,0x6050,0x5050,0x6050,0x6050,0x6050,0x6050,0x8F8F,0xF0DE,0xAFAF,0x7F7F,0x5F5F,0x7F7F,0xCFCF,0xBFBF,0x4050,0x6F6F,0x6F6F,0xBFBF,0xF98A,0x49FE,0xEFEF,0x6F6F,0x5050,0x5040,0x4050,0x5050,0x5040,0x5040,0x5040,0x5040,0x4040,0x4050,0x4050,0x4050,0x4040,0x4050,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5040,0x4050,0x5050,0x5040,0x8F8F,0xBFBF,0xCFCF,0xBFBF,0xBB69,0x7969,0xCFCF,0xCFCF,0xAFAF,0xE7CC,0xF9DD,0xCFCF,0xEFEF,0xF4F7,0xDFDF,0xCFCF,0xDEF5,0xFFFF,0xFFFF,0xFFFF,0xE7DE,0xFEFD,0xDBF7,0xBFBF,0x7F7F,0x9F9F,0xCFCF,0xAFAF,0xFED7,0x597A,0x7F7F,0x5959,0x7F7F,0x5757,0x7F7F,0xCFCF,0xCFCF,0x8F8F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F5F
.word 0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0x7ABE,0xBFBF,0xBFBF,0xD7BE,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xDEE6,0xF4F7,0xCFCF,0xCFCF,0xE7DE,0xFFFF,0xFFFF,0xF7FD,0xFFFF,0xFFFF,0xBFBF,0x5040,0x5050,0x6060,0x7F7F,0x5050,0x5060,0x6050,0x5050,0x5050,0x6060,0x5060,0x5060,0x6050,0x8F8F,0x6050,0x9F9F,0x5060,0x5050,0x5050,0xBFBF,0xEEFE,0xEFEF,0x6F6F,0x6F6F,0x7F7F,0xE9DE,0xBFBF,0x9F9F,0x4040,0x8F8F,0x8ABD,0xFFFF,0xBFBF,0xFFFF,0x5F5F,0xCFCF,0xBFBF,0x6050,0x4050,0x4050,0x4050,0x4050,0x4050,0x5050,0x5040,0x5040,0x5040,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x4050,0x4050,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5040,0x4040,0x5F5F,0x5F5F,0x8F8F,0xBFBF,0xBDE6,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBD99,0xFEE6,0xFEE7,0xE6F9,0xEFEF,0xEFEF,0xE6DD,0xD7D7,0xEFEF,0xEFEF,0xF4FE,0xFFFF,0xFEE7,0xF7FA,0xAFAF,0xD749,0x4F4F,0x6F6F,0xBFBF,0x599C,0xCFCF,0x9F9F,0x5949,0x7F7F,0xAFAF,0x7F7F,0x7F7F,0xF959,0x7AD7,0xAFAF,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F
.word 0xCFCF,0xFFFF,0xF7FE,0xCFCF,0xFFFF,0xCFCF,0xBFBF,0x9F9F,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0x61D6,0xE5BE,0xCFCF,0xF4F4,0xE6E4,0xEFEF,0xEFEF,0xFFFF,0xFFFF,0xF7FD,0xFFFF,0x40CE,0x4050,0x6050,0x5060,0x5F5F,0x6F6F,0x6060,0x5050,0x7060,0x6050,0x5050,0x6050,0x6050,0x5050,0x6050,0x5050,0x5060,0x5050,0x6050,0x6050,0xBFBF,0xFEDE,0xFEFE,0xDFDF,0x5F5F,0x9F9F,0xEBF7,0xBFBF,0x8F8F,0x5050,0xBFBF,0xBFBF,0xF9BD,0xCFCF,0xFFFF,0x9F9F,0xFFFF,0x50C0,0x5050,0xBFBF,0xBFBF,0x6050,0x6050,0x5050,0x5040,0x5050,0x5040,0x4040,0x5050,0x4050,0x6F6F,0x4050,0x6F6F,0x5040,0x5040,0x5F5F,0x5F5F,0x6F6F,0x5040,0x4050,0x4060,0x5F5F,0x6F6F,0x4050,0xAFAF,0xBFBF,0xCFCF,0xBFBF,0xD9BD,0xCFCF,0xCFCF,0xCFCF,0xF5EE,0xE6FE,0xFEE4,0xE6F5,0xAFAF,0x4979,0xEFEF,0xEFEF,0xEFEF,0xEFEF,0xF8F7,0xCFCF,0x6959,0x7F7F,0x5F5F,0x5F5F,0x8F8F,0x8AE6,0xBFBF,0x5969,0xAFAF,0x8F8F,0x597A,0x7A59,0x597A,0xBFBF,0xBFBF,0xAFAF,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F
.word 0x4040,0xBFBF,0xFFFF,0xCFCF,0xDE7A,0xFFFF,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0xBEC4,0xE4C4,0xBFBF,0xAFAF,0xCFCF,0xDEF4,0xFFFF,0xFFFF,0xFFFF,0xD7FB,0x9F9F,0x6050,0x5050,0x4050,0x5F5F,0x7F7F,0x5050,0x5060,0x5050,0x5050,0x6050,0x5050,0x5050,0x6050,0x5050,0x5050,0x5050,0x4050,0x4050,0x5050,0x5050,0xEFEF,0xFEFE,0xFFFF,0x9F9F,0x8F8F,0xE7FE,0x59BD,0xAFAF,0x4040,0xAFAF,0x7AD7,0xE9D8,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0x9F9F,0xAFAF,0xEFEF,0x50FE,0x6050,0x5050,0x5040,0x6050,0x4050,0x4050,0x5050,0x4040,0x5040,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x4050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x7F7F,0x8F8F,0x9F9F,0xBFBF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xCFCF,0xEFEF,0xE4F7,0xF462,0xBFBF,0x5959,0x52E6,0xF441,0xE0F2,0xFEF2,0xEEE6,0xDDF7,0xC7D7,0xBFBF,0x5F5F,0x5F5F,0x5F5F,0xBFBF,0xCFCF,0xBFBF,0xCFCF,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0xBFBF,0xE749,0x8F8F,0x576B,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F6F
.word 0x5040,0xAFAF,0xFFFF,0xFFFF,0x59EE,0xBFBF,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x5263,0xBFBF,0x61E4,0xC461,0x4861,0xE4E4,0xF7E4,0xEFEF,0xFFFF,0xEFEF,0xAFAF,0x5060,0x5050,0x7050,0x6F6F,0x5F5F,0x6F6F,0x8F8F,0x5050,0x5060,0x5060,0x5050,0x6060,0x6040,0x8F8F,0x6050,0x5050,0x4050,0x8F8F,0x5040,0x5050,0x4050,0xCFCF,0xCFCF,0xFEFE,0xFFFF,0xAFAF,0xF9F8,0xAFAF,0x9F9F,0x8F8F,0xBD50,0xBFBF,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x8F8F,0xFFFF,0xFFFF,0x60F0,0x6060,0x5060,0x5060,0x5060,0x6050,0x4050,0x4050,0x5040,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5940,0xBFBF,0xAFAF,0xBFBF,0xBCE6,0xE9D7,0xAFAF,0xF7FA,0xCFCF,0xC261,0x59E6,0xE6C5,0xAFAF,0x5257,0xD0D0,0xF4C1,0xCFCF,0xCFCF,0xCFCF,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0xBFBF,0x69D7,0xBFBF,0x8AE6,0xBFBF,0xAFAF,0x9F9F,0x6949,0xBFBF,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F
.word 0x4050,0x6F6F,0x8F8F,0xFEFE,0xEEF8,0xBFBF,0x59BE,0xD769,0xBFBF,0x8F8F,0x9F9F,0x4248,0x9F9F,0x8F8F,0xAFAF,0x5252,0xBFBF,0x71E4,0xBFBF,0xC371,0xAFAF,0xBFBF,0xFFFF,0xEFEF,0xBFBF,0x5050,0x5060,0x5060,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x6050,0x8F8F,0x6050,0x5050,0x5050,0x5060,0x8F8F,0x8F8F,0x5050,0xAFAF,0x8F8F,0x4050,0x5040,0x9F9F,0xCFCF,0xFEFD,0xFFFF,0xFFFF,0xFEF9,0xBFBF,0xBFBF,0xAFAF,0xCFCF,0xCFCF,0xE9F9,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0xFFFF,0xEFEF,0x9F9F,0x5050,0x5070,0x5050,0x6050,0x5050,0x5060,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x9F9F,0xBFBF,0xE6C7,0xD7BD,0xBFBF,0xBDF9,0xCFCF,0xE4E6,0x7171,0xE4E6,0xBFBF,0x9F9F,0xEEE6,0xE6F7,0xAFAF,0xF9FB,0xC7BB,0x59BD,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0xAFAF,0xBFBF,0xAFAF,0xCFCF,0xBFBF,0x4959,0x9F9F,0xBFBF,0x9F9F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5F6F
.word 0x5F5F,0x4F4F,0x6F6F,0xBFBF,0xF9F7,0x7AE7,0xBFBF,0x9F9F,0x7ABE,0xAFAF,0x8F8F,0x8F8F,0xBFBF,0x4242,0xAFAF,0xD452,0x6152,0xD461,0x5261,0xD262,0xE4D2,0xF4E4,0xFDFD,0xBFBF,0x5050,0x5060,0x6050,0x5060,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5050,0x5060,0x5060,0x4060,0x8F8F,0x5060,0x6050,0x5060,0x5060,0x4050,0xBD60,0xAFAF,0x8F8F,0x4050,0xCFCF,0xBFBF,0xBFBF,0xFEFE,0xFFFF,0xF7F7,0xD7CD,0xBFBF,0x9F9F,0xAFAF,0xFB79,0xFEE7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0x6060,0x5050,0x6060,0x7050,0x5060,0x4060,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xE6D7,0xDFDF,0xAFAF,0xD4C2,0xBFBF,0xC662,0xDDE6,0xE7FB,0x8F8F,0x7F7F,0xBFBF,0xAFAF,0x7F7F,0x7F7F,0xAFAF,0xCDE6,0x7F7F,0x6F6F,0x7F7F,0x59E6,0x8F8F,0x8F8F,0x9F9F,0xAFAF,0x8F8F,0x5959,0x7F7F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x5F5F,0x5F6F
.word 0x9F9F,0x8F8F,0x6F6F,0xAFAF,0xFEEE,0xCFCF,0x9F9F,0xCFCF,0xE7E9,0x497A,0x8F8F,0xAFAF,0x6142,0x7F7F,0x9F9F,0x9F9F,0x62D5,0x61E4,0x61C2,0x8071,0xBFBF,0xFFFF,0xCFCF,0xCFCF,0x6050,0x5050,0x5070,0x5060,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x4050,0x6040,0x5040,0x5050,0xBFBF,0xAFAF,0x9F9F,0x8F8F,0x9F9F,0xE9FE,0xBFBF,0xFEFE,0xFFFF,0xF7CD,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0x60CD,0x6070,0x7070,0x7060,0x5050,0x5050,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x4050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x9F9F,0xBD59,0xBFBF,0xAFAF,0xCFCF,0xAFAF,0xF4E6,0x62E4,0xAFAF,0xC5D4,0xE4E4,0xEAF7,0xCFCF,0x7F7F,0xAFAF,0xCB79,0x8F8F,0x7F7F,0xBFBF,0xF9F9,0xE9FA,0xAFAF,0xAFAF,0x8F8F,0xBFBF,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F6F
.word 0x9F9F,0x8AD7,0xBFBF,0x8F8F,0xCFCF,0xF9FD,0xCFCF,0xBFBF,0xCFCF,0xBFBF,0x8F8F,0x8F8F,0x5142,0x6152,0x7752,0x9F9F,0x5249,0xC470,0x61D2,0xBFBF,0x8F8F,0xBED7,0xBFBF,0x9F9F,0x5060,0x6050,0x5050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x8F8F,0x5060,0x5060,0x5050,0x5060,0x6050,0x5050,0x8F8F,0x5040,0x4050,0x4050,0xBFBF,0xBFBF,0x9F9F,0x4050,0xBFBF,0xBCE6,0xDEE6,0xFFFF,0xFEF7,0xBFBF,0xAFAF,0x9F9F,0xBFBF,0xD7BD,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFEF7,0x5070,0x7050,0x5060,0x5050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5060,0x5050,0x5040,0x4040,0x6F6F,0x5040,0x5040,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0xAFAF,0xD97A,0xCFCF,0xAB8A,0xBFBF,0xCFCF,0xC7D7,0xCFCF,0xCFCF,0x6162,0xE4C6,0xDD71,0xDCF9,0xBFBF,0x7F7F,0x9F9F,0xCFCF,0x7F7F,0x8F8F,0xF9EE,0xCFCF,0x8F8F,0xAFAF,0xBFBF,0xBFBF,0xFFFF,0xFFFF,0xEFEF,0xE6BD,0x9F9F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x4050,0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F6F
.word 0x6F6F,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xEE79,0xF9F9,0xBFBF,0x7AD9,0x7F7F,0x9F9F,0xAFAF,0x4260,0x8162,0x7162,0xAFAF,0xD6E4,0xBFBF,0xAFAF,0xCFCF,0xBFBF,0x59BC,0xCFCF,0x7050,0x7050,0x5050,0x5070,0x5060,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x4060,0x6050,0x4040,0x5050,0x5050,0x5050,0x4060,0x5040,0x5040,0xBFBF,0x7A7A,0xE9DD,0x8F8F,0xFE50,0xBFBF,0xCFCF,0xFEF9,0xE6FD,0xBFBF,0xBFBF,0xAFAF,0xAFAF,0xCFCF,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0x7060,0x5050,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x7050,0x5050,0x5050,0x4050,0x4050,0x5050,0x5040,0x4050,0x4050,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0xBFBF,0xBFBF,0xCFCF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x9F9F,0x6263,0x6152,0xE7E2,0xE6EB,0xAFAF,0x7F7F,0xF759,0xCFCF,0xFE61,0xEAF8,0xBFBF,0x8F8F,0x8F8F,0xAFAF,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x50FE,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x7F7F,0x6F6F
.word 0x5F5F,0x7F7F,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xCFCF,0xD7C9,0xBFBF,0x7F7F,0x9F9F,0xAFAF,0x8F8F,0xAFAF,0x7162,0xD4C2,0xE452,0xD6C4,0xBFBF,0x8F8F,0xBFBF,0xCFCF,0x8F8F,0x5040,0x5050,0x5060,0x6050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x6050,0x5050,0x5060,0x5050,0x4050,0x5050,0x5050,0x5040,0x5040,0x8F8F,0xBFBF,0xFE7A,0xCDF9,0xBFBF,0xCFCF,0xAFAF,0xFEAC,0xBFBF,0xAFAF,0x9F9F,0x499D,0xBFBF,0xBFBF,0xDFDF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xCFCF,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5050,0x5050,0x5060,0x4050,0x5040,0x5040,0x6F6F,0x4050,0x6F6F,0x5040,0x5F5F,0x5F5F,0x5F5F,0x4050,0x4050,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0xAFAF,0x7A7A,0xD7D7,0xBD8A,0xBDC5,0xAFAF,0x9F9F,0xBFBF,0x6259,0xCFCF,0xCFCF,0x9F9F,0x9F9F,0xE4E0,0xF5BD,0xD7EA,0xBFBF,0x8F8F,0x9F9F,0xBFBF,0xF9D7,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xEFEF,0xCFCF,0x8F8F,0x4242,0x7F7F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0xAFAF,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x4050,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x6F6F,0x5F5F,0x5F5F
.word 0x4F4F,0x5F5F,0x7F7F,0x9F9F,0xBFBF,0xCFCF,0x4979,0xAFAF,0xBFBF,0xAFAF,0x7F7F,0x5949,0xAFAF,0x5146,0x70D4,0x61E4,0x7162,0xCFCF,0xD7D7,0x7F7F,0xBFBF,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x8F8F,0x5060,0x5050,0x6050,0x5050,0x5060,0x5050,0x4050,0x4050,0x5050,0xBFBF,0x8A7A,0xF9EE,0x8AE7,0xAFAF,0xBFBF,0xCFCF,0xD6BD,0x8F8F,0x8F8F,0xBFBF,0x9F9F,0xE661,0xBFBF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x8F8F,0xBD80,0x5060,0x4050,0x5060,0x4040,0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4050,0x4050,0x4050,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x8F8F,0x9F9F,0xAFAF,0xAFAF,0xCFCF,0xE6BD,0x9F9F,0x5969,0xBFBF,0xE9BD,0xCFCF,0xAFAF,0xBFBF,0xDED4,0xBFBF,0x8F8F,0x8F8F,0xAFAF,0xBFBF,0xAFAF,0xCFCF,0xBDD7,0xCFCF,0xF9DD,0xEFEF,0xCFCF,0xAFAF,0x6F6F,0x6F6F,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6050,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x506F
.word 0x6F6F,0x6F6F,0x4040,0x7F7F,0x8F8F,0xBFBF,0xBFBF,0x9F9F,0xBFBF,0xBFBF,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0x8F8F,0x4262,0xE461,0xD7D7,0xCFCF,0x8F8F,0xBFBF,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6060,0x5050,0x5060,0x5050,0x5050,0x5050,0x6050,0x5050,0x4050,0x5050,0x8F8F,0xBFBF,0xDA7A,0xBFBF,0x7AD7,0xBFBF,0xCFCF,0x9F9F,0x9F9F,0x4242,0xBFBF,0x5142,0xD452,0x9F9F,0xFEF7,0xF7F7,0xF7FD,0xF9FA,0x7F7F,0x6F6F,0x6F6F,0x5050,0x4050,0x9F9F,0xBFBF,0x8F8F,0x7F7F,0x5050,0x5050,0x6F6F,0x6F6F,0x4050,0x8F8F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x4040,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xCFCF,0xCFCF,0xBDE6,0xBFBF,0xD4D6,0xAFAF,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x4050
.word 0x4050,0x4060,0x5050,0x6F6F,0x7F7F,0x8F8F,0xAFAF,0xBFBF,0x9F9F,0x9F9F,0xBFBF,0x8F8F,0x9F9F,0x9F9F,0xBFBF,0x4241,0xBFBF,0xBEE6,0x8F8F,0x9F9F,0x9F9F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x9F9F,0x9F9F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5050,0x5060,0x6050,0x6050,0x6050,0x6050,0x5050,0x4060,0x5040,0x5040,0x6F6F,0x8F8F,0xBFBF,0xDAD7,0xBFBF,0xBFBF,0xC7BD,0xCFCF,0x4242,0x9F9F,0x8F8F,0x9F9F,0xAFAF,0x61D4,0xFEF4,0xFEFE,0xD7E7,0xBFBF,0x6F6F,0x5040,0x5050,0x4050,0x5040,0x5040,0x4040,0x4050,0x6050,0x5050,0x5050,0x5040,0x4050,0x4050,0x4050,0x8F8F,0x9F9F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x6F6F,0x5050,0x5040,0x4040,0x8F8F,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0xAFAF,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xAFAF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x4050,0x4050,0x5040
.word 0x5040,0x5040,0x5040,0x4050,0x4050,0x4050,0x4040,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xBFBF,0xAFAF,0xBFBF,0x8F8F,0xAFAF,0xBFBF,0x8AD7,0xBFBF,0xAFAF,0x8F8F,0x5F5F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0x8F8F,0x6F6F,0x4F4F,0x5F5F,0x5050,0x6050,0x8F8F,0x5050,0x8F8F,0x5050,0x4060,0x8F8F,0x7F7F,0x5F5F,0x5F5F,0x4F4F,0xAFAF,0xF97A,0xBFBF,0xD78A,0xBFBF,0xAFAF,0x8F8F,0x8F8F,0xF9DE,0xBFBF,0xE451,0x71D4,0xCFCF,0x6F6F,0xBFBF,0x6F6F,0x5F5F,0x5050,0x5060,0x5050,0x4050,0x4050,0x4050,0x5050,0x5040,0x5050,0x5050,0x5050,0x5040,0x5040,0x5040,0x5040,0x4040,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x4050,0x7050,0x7F7F,0x9F9F,0x7F7F,0x6F6F,0x5040,0x4141,0x7F7F,0x6F6F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0xCFCF,0xD7BD,0xDEDD,0xF7FE,0xFFFF,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xBFBF,0x5F5F,0x5050,0x5040,0x4040,0x5050,0x5040,0x5040,0x5040
.word 0x5050,0x4050,0x5050,0x5040,0x5040,0x5040,0x5040,0x4040,0x4040,0x8F8F,0x7F7F,0xAFAF,0xD7BE,0xCFCF,0x6F6F,0x7F7F,0xBFBF,0xAFAF,0x8F8F,0xBFBF,0x5F5F,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x9F9F,0x9F9F,0xAFAF,0x9F9F,0xBFBF,0xAFAF,0xCFCF,0xCFCF,0x8F8F,0x7F7F,0x4050,0x5050,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x8F8F,0xCFCF,0xCFCF,0xD9D9,0xBFBF,0x4248,0xF742,0xF9F7,0x9F9F,0x7162,0xC1B0,0x42E6,0xAFAF,0x427A,0x5F5F,0x5F5F,0x5060,0x5050,0x4050,0x5050,0x5050,0x5050,0x5040,0x6050,0x5050,0x5050,0x4050,0x4050,0x4050,0x4050,0x4050,0x5050,0x4040,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0xBFBF,0x9F9F,0xBFBF,0xF5F4,0xBFBF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0xCFCF,0xFFFF,0xFFFF,0xFFFF,0x7F7F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0xAFAF,0x7070,0x8F8F,0x6F6F,0x6F6F,0x5050,0x4040,0x6F6F,0x6F6F,0x6F50
.word 0x5050,0x5050,0x4050,0x5040,0x5040,0x5040,0x5040,0x5040,0x7F7F,0x8F8F,0x6F6F,0x7F7F,0x9F9F,0xBFBF,0x427A,0x7F7F,0x9F9F,0xBFBF,0x4159,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xAFAF,0x4B4B,0xAFAF,0xBFBF,0x9E4B,0xBFBF,0xBDCE,0xFEEE,0xFFFF,0xDEFE,0x7F7F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0xBFBF,0xE9D9,0x9F9F,0xBFBF,0x8F8F,0xBFBF,0xEBF7,0xBFBF,0x7142,0xF4C2,0x5F5F,0x8F8F,0x6F6F,0x5F5F,0x6F6F,0x6040,0x5050,0x4050,0x5050,0x5050,0x5060,0x5040,0x5060,0x6050,0x4050,0x5040,0x5040,0x5040,0x5040,0x5040,0x4040,0x5050,0x5F5F,0x5F5F,0x6F6F,0x5060,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x9F9F,0xAFAF,0xC7BD,0xBFBF,0xAFAF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x8F8F,0xFFFF,0xFFFF,0xFFFF,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x6060,0xAFAF,0x5070,0x5050,0x5050,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F
.word 0x4040,0x5040,0x5040,0x4050,0x4050,0x4050,0x5050,0x7F7F,0x9F9F,0xDEE6,0xE6F6,0xBFBF,0xAFAF,0xAFAF,0xAFAF,0x6F6F,0xAFAF,0x9F9F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x8F8F,0x9F9F,0x9F9F,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xAFAF,0xCFCF,0xFEEE,0xFFFF,0xBFBF,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0xAFAF,0xBFBF,0x8F8F,0x4241,0xBFBF,0xCFCF,0x9F9F,0x5142,0xBFBF,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x6050,0x5040,0x6050,0x6050,0x5050,0x4060,0x6050,0x5050,0x5060,0x5040,0x5040,0x5040,0x5040,0x5040,0x5040,0x4040,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0xAFAF,0xCFCF,0xEBE7,0xBFBF,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0xBFBF,0xFFFF,0xFFFF,0xAFAF,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5040,0x5040,0x5050,0x5060,0x5050,0x7060,0x5060,0x6F6F,0x5050,0x5040,0x5050,0x5040
.word 0x5050,0x5050,0x5050,0x5040,0x5040,0x5040,0x8F8F,0xBFBF,0xCFCF,0xF5DE,0xFEFE,0xFFFF,0xEFEF,0xE4F4,0xBFBF,0xAFAF,0x7F7F,0x7F7F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x9F9F,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xCFCF,0xDECE,0xFEEB,0xEFEF,0xEFEF,0xFFFF,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x7F7F,0xAFAF,0x8F8F,0x9F9F,0xCFCF,0xAFAF,0xBFBF,0x9F9F,0xBFBF,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6050,0x5050,0x6050,0x5050,0x4060,0x5060,0x6050,0x5050,0x5060,0x5050,0x5050,0x5040,0x5040,0x5040,0x5040,0x7F7F,0x5050,0x5F5F,0x5F5F,0x4040,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x7F7F,0x8F8F,0x8F8F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0xEFEF,0xFFFF,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5050,0x4050,0x5040,0x4040,0x5050,0x5060,0x5050,0x5060,0x6060,0x6070,0x4060,0x5040,0x5050,0x5050,0x5050
.word 0x4040,0x4040,0x4040,0x4050,0x5050,0x5050,0xBFBF,0xBFBF,0xCFCF,0xDEF7,0xEFEF,0xFEFE,0xEFEF,0xDEDE,0xEFEF,0xBFBF,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x8F8F,0x9F9F,0x7F7F,0x9F9F,0xBFBF,0xCFCF,0xBFBF,0xAFAF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0x8F8F,0x5070,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x8F8F,0xAFAF,0x8F8F,0x9F9F,0xE9D8,0x9F9F,0xAFAF,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5050,0x5050,0x5060,0x6050,0x5050,0x6050,0x4050,0x5050,0x6050,0x6050,0x5050,0x4050,0x4050,0x4050,0x4050,0x5050,0x5040,0x5F5F,0x5F5F,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5040,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x8F8F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x4040,0x5040,0x4050,0x5050,0x5050,0x5050,0x5060,0x6050,0x8060,0x9F9F,0x5050,0x6F6F,0x6050,0x5050,0x5060
.word 0x5050,0x5050,0x5050,0x5040,0x5040,0x9F9F,0xBFBF,0xDEF9,0xDEE7,0xF5EE,0xEFEF,0xFFFF,0xFFFF,0xEFEF,0xAFAF,0x8F8F,0x6F6F,0x6F6F,0x6050,0x6050,0x4050,0x7F7F,0x4040,0x4040,0x8F8F,0x4040,0x4041,0x6F6F,0x6F6F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x6F6F,0x5050,0x6070,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x8F8F,0x9F9F,0x9F9F,0xAFAF,0xAFAF,0xBFBF,0xBFBF,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0xAFAF,0xAFAF,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5050,0x5050,0x5050,0x5060,0x5050,0x5060,0x6050,0x5050,0x5050,0x5050,0x6050,0x5050,0x5040,0x5040,0x7F7F,0x4040,0x4050,0x5F5F,0x5F5F,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x5040,0x5050,0x4040,0x4050,0x4040,0x6050,0x5050,0x5060,0x6070,0x6070,0x5070,0x4050,0x5050,0x6060,0x5050
.word 0x4040,0x4040,0x5040,0x5040,0x8F8F,0xBFBF,0xE7DE,0xDFDF,0xF7DE,0xFEF7,0xEFEF,0xCFCF,0x9F9F,0x6F6F,0x6F6F,0x5040,0x5040,0x6F6F,0x5040,0x5050,0x4060,0x5050,0x5050,0x5040,0x4040,0x5050,0x5040,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0x5060,0x6050,0x6050,0x5040,0x4F4F,0x4F4F,0x5F5F,0x9F9F,0xAFAF,0x8A7A,0x9F9F,0xAFAF,0xAFAF,0xAFAF,0xE6BD,0xBFBF,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x5F5F,0x6F6F,0x4050,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5060,0x5050,0x5050,0x5050,0x5060,0x5060,0x4060,0x5050,0x4050,0x5050,0x5050,0x5040,0x5050,0x5040,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x4050,0x5040,0x5040,0x5040,0x5040,0x5040,0x4050,0x4040,0x5050,0x5040,0x5050,0x5050,0x5060,0x6050,0x5050,0x7050,0x6050,0x5050,0x6F6F,0x5050,0x6050
.word 0x5050,0x5050,0x5040,0x5050,0xAFAF,0xCFCF,0xCFCF,0xCFCF,0xBFBF,0xAFAF,0x7F7F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6050,0x6050,0x5050,0x6050,0x4050,0x4050,0x5050,0x5040,0x5040,0x4050,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x4050,0x4050,0x4050,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x9F9F,0xCD59,0xCFCF,0xBDD7,0xDEE7,0xEFEF,0xE0F4,0xBFBF,0xAFAF,0x8F8F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x4050,0x4040,0x6F6F,0x5F5F,0x6F6F,0x4060,0x6050,0x5050,0x5060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5060,0x5050,0x5050,0x4060,0x4050,0x4050,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5050,0x5040,0x4040,0x5050,0x5040,0x5040,0x5040,0x5040,0x4040,0x5050,0x4050,0x5040,0x4040,0x5040,0x5050,0x6050,0x6050,0x6050,0x6050,0x7070,0x7060,0x8F8F,0x6040,0x5060
.word 0x6F6F,0x7F7F,0x4050,0x4040,0x4040,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4040,0x5060,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5F5F,0x5040,0x5050,0x5060,0x5060,0x5050,0x5040,0x5040,0x4050,0x4050,0x5050,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6050,0x4050,0x5040,0x5040,0x5050,0x6060,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x8F8F,0xBFBF,0xAFAF,0xF7FE,0xEEFE,0xDEFE,0x80DD,0xAFAF,0x4252,0x5F5F,0x5F5F,0x5F5F,0x5050,0x5040,0x5050,0x5F5F,0x5F5F,0x5040,0x5050,0x5050,0x6050,0x6050,0x6050,0x5060,0x5060,0x6050,0x5050,0x5050,0x5050,0x5050,0x5050,0x5040,0x5040,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0x6F6F,0x5F5F,0x6F6F,0x4050,0x5F5F,0x6F6F,0x4040,0x4050,0x5050,0x4040,0x4050,0x4050,0x4050,0x4050,0x5050,0x4040,0x5040,0x4050,0x5050,0x5050,0x5050,0x8F8F,0x5060,0x7050,0x7050,0x6060,0x7070,0x5060,0x7F7F,0x6050
.word 0x5040,0x4050,0x4050,0x5050,0x4050,0x5050,0x5050,0x5050,0x5050,0x5050,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x4050,0x5F5F,0x5050,0x6050,0x6060,0x5060,0x5060,0x5050,0x6050,0x5050,0x6040,0x6060,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x4040,0x5040,0x4050,0x5050,0x5050,0x5060,0x5050,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x5F5F,0x5040,0x4040,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6050,0x5050,0x5060,0x5050,0x5050,0x5050,0x5050,0x5050,0x8F8F,0x5040,0x5050,0x4040,0x5050,0x4050,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x5050,0x4050,0x4050,0x5040,0x4050,0x4050,0x4050,0x4050,0x4050,0x5040,0x5050,0x4040,0x4050,0x5050,0x5040,0x6050,0x8F8F,0x5060,0x5050,0x7050,0x6060,0x5060,0x6F6F,0x5050
.word 0x6F6F,0x5040,0x5040,0x4040,0x5040,0x5040,0x5050,0x5050,0x5050,0x5040,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5050,0x6F6F,0x5040,0x6F6F,0x4040,0x4050,0x5050,0x6070,0x5070,0x5060,0x6070,0x6050,0x6050,0x5060,0x7050,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4050,0x5060,0x5060,0x5040,0x5040,0x5040,0x5040,0x5060,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x7F7F,0x9F9F,0x4050,0x4050,0x6F6F,0x6F6F,0x5050,0x5050,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5050,0x4050,0x5040,0x5050,0x5050,0x8F8F,0x5050,0x5040,0x6050,0x5050,0x5040,0x6040,0x5050,0x8F8F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x4050,0x4050,0x5040,0x5040,0x5050,0x5040,0x5040,0x5040,0x5040,0x5040,0x4050,0x4040,0x5050,0x5040,0x4040,0x5050,0x5050,0x6050,0x5050,0x5070,0x5070,0x6050,0x5060,0x6F6F,0x505F
.word 0x5040,0x5040,0x7F7F,0x5050,0x5040,0x5040,0x4040,0x4050,0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x4050,0x4050,0x5F5F,0x5F5F,0x5040,0x5040,0x7050,0x8060,0x7060,0x7060,0x5050,0x6070,0x7060,0x6050,0x4070,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5050,0x6050,0x5040,0x5040,0x5040,0x5040,0x5050,0x6050,0x5050,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0xAFAF,0xCFCF,0xBDF0,0x9F9F,0x5040,0x6050,0x5040,0x5050,0x5050,0x4060,0x6F6F,0x7F7F,0x5F5F,0x5F5F,0x5050,0x5040,0x4050,0x5050,0x8F8F,0x6050,0x8F8F,0x4050,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x5050,0x6050,0x5050,0x5060,0x5050,0x5040,0x4040,0x5050,0x4050,0x5040,0x5040,0x5050,0x5050,0x6050,0x5050,0x5060,0x6050,0x6050,0x7050,0x7050,0x6F6F,0x6F5F
.word 0x6F6F,0x6F6F,0x5050,0x4040,0x6F6F,0x4050,0x5050,0x5040,0x5040,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x6050,0x6080,0x6070,0x6070,0x6060,0x5070,0x7060,0x6060,0x5070,0x6F6F,0x5F5F,0x6F6F,0x6050,0x5050,0x5050,0x5050,0x4050,0x5050,0x5050,0x5060,0x4050,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x9F9F,0x9F9F,0xFFFF,0xAFAF,0x5060,0x5050,0x6060,0x6F6F,0x6040,0x4050,0x5040,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5060,0x5070,0x5060,0x7050,0x5050,0x4050,0x4050,0x5050,0x5050,0x5040,0x5040,0x4050,0x5050,0x5050,0x4050,0x5050,0x8F8F,0x8F8F,0x7050,0x5050,0x5050,0x5060,0x5F5F
.word 0x5040,0x5040,0x6F6F,0x7F7F,0x5040,0x4040,0x4050,0x5040,0x5040,0x5040,0x6F6F,0x6F6F,0x4050,0x5050,0x5040,0x4040,0x4050,0x6F6F,0x5F5F,0x5050,0x5050,0x5040,0x6040,0x8050,0x7070,0x5060,0x6070,0x6070,0x6070,0x7060,0x7060,0x4050,0x6F6F,0x5F5F,0x5070,0x6050,0x6050,0x5060,0x6040,0x5050,0x5050,0x5050,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0xAFAF,0xC0FE,0xF0BD,0xEFEF,0xBFBF,0x9F9F,0x6050,0xAFAF,0x4050,0x5050,0x5060,0x4050,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x5050,0x5050,0x7050,0x7050,0x6050,0x5070,0x6050,0x6050,0x5050,0x5050,0x5050,0x5050,0x4050,0x5050,0x5050,0x5040,0x5050,0x5040,0x5060,0x6060,0x8F8F,0x7060,0x6070,0x5050,0x5F6F
.word 0x5050,0x5050,0x5F5F,0x6F6F,0x4050,0x5050,0x5040,0x4050,0x4050,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x5050,0x5040,0x6F6F,0x6F6F,0x6050,0x5050,0x5050,0x5040,0x5040,0x7070,0x6080,0x8060,0x7060,0x7070,0x6070,0x6070,0x5060,0x5F5F,0x6F6F,0x5050,0x5060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5060,0x5060,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x8F8F,0xCFCF,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0xAFAF,0x6060,0x5050,0x7F7F,0x6F6F,0x5050,0x4060,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x5040,0x6050,0x5050,0x6050,0x6070,0x7050,0x5050,0x5060,0x5050,0x5050,0x5050,0x5050,0x5050,0x5050,0x4040,0x4050,0x4050,0x5050,0x5050,0x5050,0x7050,0x6060,0x5050,0x5070,0x5F6F
.word 0x6050,0x6F6F,0x5F5F,0x4050,0x4050,0x5040,0x7F7F,0x5040,0x5040,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x5040,0x5050,0x6F6F,0x6F6F,0x5050,0x5060,0x4060,0x4050,0x8F8F,0x5040,0x7070,0x6060,0x6070,0x6050,0x7070,0x7060,0x5060,0x5F5F,0x6F6F,0x5060,0x6050,0x6050,0x5040,0x5060,0x5050,0x5060,0x6050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5040,0x5040,0x5050,0x6070,0xAFAF,0x7060,0xAFAF,0xAFAF,0x9F9F,0x5050,0x6F6F,0x6050,0x5050,0x4050,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x6F6F,0x5F5F,0x7F7F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6070,0x5070,0x5070,0x6060,0x5060,0x5070,0x6050,0x5050,0x5050,0x5050,0x4060,0x5050,0x5050,0x5050,0x5050,0x5040,0x5050,0x5040,0x5050,0x5060,0x7060,0x5060,0x6050,0x4F50
.word 0x7F7F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x5050,0x5060,0x5060,0x6F6F,0x6F6F,0x6040,0x5050,0x5050,0x5050,0x4050,0x8F8F,0x7050,0x8060,0x7070,0x8060,0x6070,0x5060,0x5070,0x6F6F,0x5F5F,0x7040,0x5050,0x5060,0x6050,0x5050,0x5050,0x5050,0x7050,0x5060,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x8F8F,0x8F8F,0xAFAF,0x9F9F,0x9F9F,0x6070,0x8F8F,0x4040,0x5050,0x5060,0x5050,0x5040,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5050,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x6050,0x5050,0x7050,0x7060,0x7050,0x7050,0x5050,0x6050,0x5050,0x6050,0x5050,0x5050,0x8F8F,0x6050,0x8F8F,0x5050,0x5040,0x4050,0x5040,0x5050,0x5060,0x5070,0x5070,0x5F6F
.word 0x5F5F,0x5F5F,0x5F5F,0x6F6F,0xAFAF,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4050,0x4050,0x5060,0x6050,0x6050,0x4050,0x4050,0x5F5F,0x5050,0x6050,0x5050,0x5040,0x5040,0x5040,0x8F8F,0x9F9F,0x8F8F,0x5060,0x6050,0x7070,0x5060,0x6F6F,0x5F5F,0x6F6F,0x6050,0x5050,0x5060,0x5050,0x5050,0x5050,0x5060,0x6F6F,0x6F6F,0x6F6F,0x4050,0x5050,0x5040,0x6040,0x6050,0xAFAF,0x6050,0xAFAF,0x7070,0xAFAF,0xAFAF,0x5050,0x6F6F,0x6060,0x6050,0x4050,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x5050,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x5040,0x6F6F,0x6F6F,0x4050,0x5060,0x7050,0x6070,0x6070,0x5070,0x5050,0x6050,0x5050,0x5040,0x4050,0x5050,0x4050,0x5050,0x5050,0x6050,0x4050,0x4050,0x8F8F,0x5050,0x8F8F,0x7050,0x7050,0x5050,0x5F50
.word 0x5F5F,0x5F5F,0x6F6F,0x5040,0xBFBF,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x5040,0x5040,0x5040,0x5060,0x5050,0x5060,0x6F6F,0x6F6F,0x6F6F,0x5060,0x5050,0x4060,0x4050,0x4050,0x5040,0x5050,0x5050,0x5040,0x6050,0x4040,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5050,0x5060,0x5050,0x5050,0x5050,0x5050,0x7F7F,0x4050,0x4050,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x6050,0x6050,0x5070,0xAFAF,0xAFAF,0xAFAF,0x6070,0xAFAF,0x6F6F,0x5050,0x5070,0x5050,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x5040,0x5040,0x4150,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x5050,0x6F6F,0x4050,0x6F6F,0x5040,0x6F6F,0x6F6F,0x5040,0x6F6F,0x5040,0x5060,0x5050,0x5050,0x7050,0x5070,0x5060,0x6050,0x5050,0x5040,0x4050,0x4050,0x5050,0x4040,0x4050,0x5040,0x5040,0x5050,0x5060,0x6060,0x5060,0x6060,0x5070,0x5F7F
.word 0x6F6F,0x5040,0x5040,0x5050,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4050,0x4050,0x6050,0x6050,0x6050,0x5040,0x5050,0x6F6F,0x6F6F,0x6050,0x4050,0x5040,0x4050,0x5050,0x4040,0x4050,0x5050,0x7050,0x5050,0x4060,0x7F7F,0x4050,0x5F5F,0x6F6F,0x4060,0x5050,0x5050,0x4050,0x5050,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0x4050,0x5050,0x8F8F,0x5060,0x8F8F,0x6050,0x9F9F,0x7060,0xAFAF,0x5050,0x6F6F,0x6050,0x6050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5050,0x5040,0x4050,0x6F6F,0x4050,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x7F7F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x6F6F,0x5040,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0x5050,0x6F6F,0x4050,0x6F6F,0x5050,0x7050,0x6050,0x6050,0x5050,0x6050,0x6050,0x5050,0x5060,0x5060,0x5060,0x5050,0x4050,0x4050,0x5050,0x5040,0x5050,0x5060,0x4050,0x4050,0x8F8F,0x8F8F,0x8F8F,0x6F60
.word 0x6F6F,0x4050,0x4050,0x6040,0x5050,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5050,0x5060,0x5050,0x5050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5050,0x6050,0x5050,0x5050,0x4040,0x5050,0x5040,0x5050,0x5060,0x5060,0x5050,0x4050,0x5F5F,0x6F6F,0x6F6F,0x5040,0x4040,0x5040,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x4050,0x4050,0x8F8F,0x5040,0x5060,0x8F8F,0x5050,0xAFAF,0x6070,0xAFAF,0x7060,0x8F8F,0x6F6F,0x7050,0x4050,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x5040,0x4F4F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x4040,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x4050,0x5F5F,0x5F5F,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x4040,0x4050,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x4050,0x6F6F,0x4040,0x5050,0x6F6F,0x5050,0x7F7F,0x6050,0x5070,0x5070,0x6050,0x5050,0x5050,0x5050,0x5040,0x5050,0x6050,0x8F8F,0x5050,0x5050,0x5040,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5060
.word 0x4040,0x4050,0x4050,0x4050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5050,0x6050,0x6050,0x5050,0x5050,0x5050,0x6F6F,0x5F5F,0x5060,0x5060,0x5060,0x5050,0x5050,0x4050,0x4050,0x6050,0x6050,0x6060,0x6050,0x5050,0x5040,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x5050,0x7F7F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x5050,0x4040,0x5040,0x7F7F,0x4050,0x8F8F,0x5050,0x5060,0x8F8F,0x6050,0xAFAF,0x7060,0x9F9F,0x5050,0x6F6F,0x8F8F,0x5050,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5040,0x5040,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x5050,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5060,0x5050,0x5060,0x5060,0x8F8F,0x4050,0x4050,0x4050,0x5040,0x5050,0x5040,0x4050,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x8F7F
.word 0x5050,0x5040,0x5040,0x5040,0x5040,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x4050,0x5050,0x6050,0x4050,0x4040,0x6F6F,0x6F6F,0x5050,0x5050,0x5050,0x5060,0x5050,0x5060,0x5040,0x4050,0x5050,0x5050,0x5060,0x5050,0x4050,0x5F5F,0x5F5F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x5050,0x6F6F,0x5050,0x6F6F,0x4050,0x5050,0x6F6F,0x4050,0x4050,0x4050,0x5060,0x5040,0x5060,0x6050,0x5050,0x8F8F,0x7060,0x5060,0x6F6F,0x6050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5050,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x5040,0x4050,0x4050,0x5040,0x6F6F,0x4050,0x6F6F,0x5F5F,0x6F6F,0x5050,0x5050,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x5050,0x6050,0x5050,0x5060,0x5050,0x8F8F,0x5060,0x5050,0x8F8F,0x4050,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x5040,0x5040,0x5040,0x5040,0x5050,0x5060,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5050,0x4050,0x5060,0x5050,0x6F6F,0x5F5F,0x6050,0x6050,0x6050,0x5050,0x5060,0x5050,0x5040,0x5060,0x5050,0x6050,0x6050,0x6050,0x5050,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x4050,0x6F6F,0x4050,0x6F6F,0x6F6F,0x5040,0x5040,0x7F7F,0x4050,0x8F8F,0x5060,0x8F8F,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x4050,0x6F6F,0x4050,0x5050,0x4040,0x4050,0x5050,0x5040,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5040,0x5040,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4050,0x5050,0x5040,0x8F8F,0x5050,0x6050,0x4040,0x4050,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F5F
.word 0x4050,0x4050,0x4050,0x4050,0x5040,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x5040,0x5050,0x6040,0x4050,0x6F6F,0x6F6F,0x5040,0x5070,0x5050,0x5060,0x6050,0x5050,0x6050,0x5050,0x5050,0x5050,0x5050,0x5060,0x4050,0x5050,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0x5040,0x4050,0x4050,0x5050,0x5040,0x5050,0x8F8F,0x4050,0x5060,0x5040,0x7F7F,0x6F6F,0x5F5F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x5050,0x5040,0x5040,0x5040,0x5040,0x4040,0x4150,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4050,0x5050,0x5050,0x4050,0x8F8F,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F4F
.word 0x4040,0x4050,0x4050,0x4050,0x5050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5050,0x5050,0x5050,0x6F6F,0x5F5F,0x7F7F,0x7050,0x6050,0x5050,0x5070,0x6050,0x5050,0x5060,0x5060,0x4050,0x5050,0x5040,0x6050,0x4050,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x5040,0x5040,0x5060,0x8F8F,0x6040,0x5050,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x6F6F,0x5F5F,0x6F6F,0x5040,0x5040,0x5040,0x4040,0x4050,0x4050,0x4050,0x4050,0x5050,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5040,0x4050,0x5050,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4050,0x5040,0x7F7F,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F
.word 0x6F6F,0x6F6F,0x5040,0x7F7F,0x4050,0x5050,0x5050,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x6050,0x5050,0x6F6F,0x5F5F,0x6F6F,0x5060,0x5070,0x6050,0x5050,0x5060,0x5050,0x5050,0x8F8F,0x6050,0x4040,0x4050,0x4040,0x5050,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x5050,0x5040,0x5040,0x4050,0x7F7F,0x7F7F,0x4050,0x5040,0x5050,0x5040,0x5050,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x4040,0x6F6F,0x5F5F,0x6F6F,0x5040,0x6F6F,0x5040,0x4040,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5050,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F
.word 0x5040,0x5040,0x5040,0x4050,0x4050,0x6050,0x6F6F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x4050,0x5050,0x6F6F,0x5F5F,0x6F6F,0x5070,0x6060,0x5050,0x5060,0x6050,0x8F8F,0x5050,0x5060,0x4050,0x8F8F,0x5050,0x4050,0x5050,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x5050,0x6050,0x5050,0x5060,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5040,0x4050,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x6F6F,0x5040,0x4050,0x6F6F,0x6F6F,0x6F6F,0x4050,0x5040,0x6F6F,0x4050,0x5040,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4040,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4040,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F
.word 0x4050,0x4050,0x4050,0x5040,0x5040,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5040,0x5050,0x6F6F,0x6F6F,0x5F5F,0x5050,0x6060,0x5060,0x6050,0x5050,0x5050,0x5060,0x6050,0x5050,0x5040,0x7F7F,0x7F7F,0x4040,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x4050,0x6F6F,0x5050,0x6F6F,0x5050,0x5040,0x5040,0x7F7F,0x4050,0x4050,0x8F8F,0x5040,0x6050,0x5050,0x4050,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5040,0x7F7F,0x5040,0x5F5F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x5040,0x5F5F,0x5F5F,0x5040,0x5040,0x6F6F,0x6F6F,0x5040,0x6F6F,0x5050,0x6F6F,0x5040,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F4F
.word 0x5040,0x5040,0x5040,0x4050,0x5050,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6050,0x8F8F,0x5050,0x5060,0x6050,0x6050,0x5050,0x4050,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x7F7F,0x4050,0x4050,0x4060,0x5050,0x5050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5040,0x5050,0x5040,0x5040,0x5050,0x5F5F,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x4050,0x6F6F,0x5F5F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5040,0x6F6F,0x4050,0x4050,0x4050,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F
.word 0x6F6F,0x4050,0x6F6F,0x5040,0x5040,0x7F7F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0x5050,0x5060,0x6050,0x5050,0x5050,0x8F8F,0x5050,0x7F7F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x4040,0x6F6F,0x4040,0x5050,0x5040,0x5040,0x5040,0x4040,0x5050,0x5050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4040,0x7F7F,0x8F8F,0x5050,0x5F5F,0x4050,0x7F7F,0x4050,0x5050,0x6050,0x5050,0x6F6F,0x5F5F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x5F5F,0x4040,0x4050,0x4050,0x6F6F,0x6F6F,0x4050,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F5F
.word 0x5040,0x5040,0x5050,0x6F6F,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5050,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0x7F7F,0x4050,0x4040,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x6050,0x5040,0x5050,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x4050,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x4050,0x6F6F,0x6F6F,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0x6F6F,0x5040,0x5040,0x6F6F,0x4050,0x7F7F,0x4040,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x4050,0x6F6F,0x5040,0x4040,0x6F6F,0x6F6F,0x6F6F,0x4050,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x5060,0x6050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5040,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x4050,0x5050,0x4050,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5050,0x5040,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5050,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5060,0x5050,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x4040,0x4050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x5040,0x4040,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5050,0x5050,0x4050,0x5040,0x4050,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5050,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x5050,0x4050,0x6F6F,0x5040,0x4050,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x4050,0x6F6F,0x4040,0x5050,0x5040,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4050,0x5050,0x5060,0x5050,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x7F7F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x4040,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x6F6F,0x6F6F,0x6F6F,0x4040,0x5050,0x4040,0x4050,0x6F6F,0x6F6F,0x5F5F,0x5040,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x4050,0x6F6F,0x5040,0x5040,0x5040,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5050,0x5050,0x5050,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x6F6F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x5050,0x4040,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F
.word 0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x5050,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x4040,0x4050,0x4050,0x4050,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5040,0x5050,0x5050,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x7F7F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4150,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0x6F6F,0x5040,0x6F6F,0x6F6F,0x4040,0x6F6F,0x6F6F,0x5050,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F
.word 0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x5050,0x5050,0x6F6F,0x4050,0x4050,0x4050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x4050,0x6F6F,0x5040,0x5040,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x4050,0x4040,0x4050,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x6F6F,0x4040,0x4050,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F
.word 0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x4050,0x5050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x4050,0x5050,0x5050,0x6050,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x7F7F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x6F6F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5060,0x6F6F,0x6F6F,0x6F6F,0x5040,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4040,0x6F6F,0x5040,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F6F
.word 0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5050,0x5040,0x7F7F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5040,0x6F6F,0x5040,0x6F6F,0x6F6F,0x4050,0x4050,0x5050,0x4050,0x5040,0x4050,0x5040,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x4050,0x5050,0x6050,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4040,0x7F7F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F
.word 0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x4050,0x4050,0x4040,0x4050,0x4050,0x4050,0x4050,0x5050,0x6050,0x5040,0x5040,0x4040,0x4050,0x4050,0x5050,0x5040,0x6F6F,0x5050,0x4050,0x5050,0x5050,0x5050,0x5040,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x4050,0x4050,0x5050,0x5040,0x5040,0x5040,0x5040,0x5040,0x5050,0x5050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F
.word 0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x5040,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5040,0x5050,0x5040,0x5040,0x5040,0x5040,0x4040,0x4050,0x4050,0x6F6F,0x5050,0x5040,0x5040,0x4040,0x6F6F,0x6F6F,0x6040,0x6050,0x5050,0x5040,0x5050,0x8F8F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4050,0x4050,0x4050,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x4050,0x5F5F,0x6F6F,0x6F6F,0x5040,0x4050,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F6F
.word 0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x6F6F,0x6F6F,0x4050,0x6F6F,0x5F5F,0x5F5F,0x5050,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x4040,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x4050,0x5040,0x5040,0x5040,0x5050,0x6F6F,0x6F6F,0x6050,0x5060,0x5050,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4050,0x4050,0x4050,0x4050,0x4050,0x6F6F,0x4050,0x4050,0x4050,0x6F6F,0x6F6F,0x4040,0x5050,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x4050,0x6050,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F
.word 0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x5040,0x5040,0x5040,0x5040,0x5040,0x4050,0x4050,0x6F6F,0x6F6F,0x5050,0x4040,0x7050,0x5050,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5040,0x5040,0x5040,0x4050,0x4050,0x6F6F,0x5040,0x5050,0x4050,0x5050,0x4040,0x4040,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x4040,0x5050,0x5050,0x5050,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F
.word 0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x4050,0x5040,0x4050,0x5050,0x5040,0x5040,0x5040,0x5040,0x6F6F,0x5040,0x5040,0x5040,0x4040,0x6F6F,0x4050,0x5050,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5050,0x5050,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5040,0x4050,0x4050,0x5040,0x5040,0x5040,0x5040,0x5040,0x5040,0x5040,0x4040,0x4040,0x4050,0x5040,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5050,0x5040,0x5050,0x4050,0x5040,0x5040,0x5050,0x4050,0x7F7F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F6F
*/



;/* Dragon Girl
Dragon_320:
.word 0x4444,0x4444,0x4444,0x4444,0x8373,0x8373,0x7383,0x4444,0x4444,0x4444,0x4444,0x4444,0x7343,0x6363,0x7373,0x4443,0x4444,0x4444,0x7363,0x5263,0x6363,0x6383,0x4444,0x6344,0x6363,0x8373,0x4444,0x4444,0x4444,0x4444,0x6343,0x7363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x6F6F,0x7F7F,0x4141,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4442,0x6344,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x5243,0x8363,0xC4C3,0x4363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6343,0x8383,0xC4C3,0x7383,0x6363,0x4243,0x4242,0x4242
.word 0x4444,0x4444,0x4444,0x4444,0x6344,0x73C4,0x7363,0x8383,0x4473,0x4444,0x4444,0x4444,0x6344,0x6363,0x6363,0x7373,0x4463,0x6344,0x7373,0x6352,0x6363,0x6363,0x7383,0x7363,0x6363,0x7363,0x4473,0x4444,0x4444,0x4344,0x6363,0x7363,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6344,0x63C4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x6342,0xC483,0xC4C3,0x6373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x8363,0xC3C4,0x7383,0x4363,0x4242,0x4442,0x4444,0x4444
.word 0x4444,0x4444,0x4444,0x4444,0x4444,0x6344,0x63C4,0x6352,0x7373,0x7383,0x4444,0x4444,0x7344,0x6363,0x6363,0x6363,0x5263,0x6363,0x5263,0x5263,0x6363,0x6363,0x6352,0x6363,0x6352,0x6352,0x7363,0x8383,0x7373,0x7363,0x6363,0x7363,0x4483,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x7F7F,0x5F5F,0x7F7F,0x5241,0xD462,0x52C4,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4242,0xD452,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x5242,0x8373,0xC4C3,0x7383,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x8363,0xC4B3,0x7383,0x4463,0x4442,0x4444,0x4444,0x4444,0x4444
.word 0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x8363,0x6373,0x7352,0x7373,0x7383,0x4444,0x6352,0x6352,0x5252,0x5263,0x6363,0x5263,0x5252,0x5252,0x5252,0x5252,0x6352,0x5263,0x5263,0x6352,0x6363,0x6363,0x6373,0x5252,0x7363,0x6363,0x6373,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xC441,0xD4D4,0xD4E4,0x52C4,0x6F6F,0x6F6F,0x5241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7343,0x73D4,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4142,0x6342,0x8383,0xC4C3,0x83C3,0x4352,0x4444,0x4444,0x4444,0x4444,0x4444,0x6344,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x7363,0xC483,0x7383,0x4463,0x4442,0x4244,0x4244,0x4444,0x4444,0x4444
.word 0x4344,0x6363,0x6363,0x4363,0x4444,0x4444,0x4444,0x7373,0x7363,0x6363,0x6373,0x7373,0x6363,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x6352,0x5252,0x5252,0x6363,0x6352,0x5252,0x6352,0x4383,0x4444,0x4444,0x7F7F,0x7F7F,0x4242,0x6F6F,0x7F7F,0x5241,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x6F6F,0xD4C4,0xD4D4,0x52C4,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x7F7F,0xC452,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x6342,0x8373,0xC483,0xC4D4,0x83C4,0x4363,0x4444,0x4444,0x4444,0x5244,0x6352,0x5252,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5242,0x8373,0xC4C4,0xC4C4,0x83C4,0x7383,0x7352,0x4352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x7363,0xB383,0xC4C4,0x7383,0x4243,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7344,0x8383,0x8373,0x7383,0x8383,0x6383,0x4444,0x7373,0x5263,0x6363,0x5263,0x5252,0x6363,0x5252,0x5252,0x5252,0x5242,0x5242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x5242,0x5252,0x5252,0x5263,0x5252,0x5252,0x6352,0x6363,0x7373,0x4383,0x4244,0x7F7F,0x4241,0x7F7F,0x7F7F,0x4141,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x5243,0x83C4,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4142,0x6352,0x8373,0xC483,0xC4C3,0xC4D4,0xC4C4,0x8383,0x7373,0x7373,0x7373,0x5263,0x5263,0x5252,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x8363,0xC3C4,0x83C4,0x8383,0x8383,0xC383,0xC4C4,0x8383,0x7363,0x6352,0x5273,0x5263,0x5263,0x8383,0x8383,0xB3C3,0x83C4,0x7383,0x4263,0x4442,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4444
.word 0x4444,0x4444,0x8373,0x6373,0x6363,0x6352,0x7373,0x6363,0x6373,0x5252,0x5252,0x5252,0x4252,0x4242,0x5242,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x6352,0x5263,0x5252,0x5252,0x5252,0x4242,0x4252,0x4252,0x5252,0x6352,0x6363,0x5263,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x5F5F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x4142,0x4141,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7352,0x83C4,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6342,0x5252,0x6363,0x6363,0x8373,0xC483,0xC493,0xC4C3,0xC4C3,0xC3B3,0xC3C4,0xC4C3,0xC4D4,0xC4D4,0x83C4,0x7373,0x6352,0x5252,0x5252,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x8363,0xC4D4,0x8383,0x6373,0x6363,0x5252,0x6363,0x5252,0x8352,0x7383,0x7383,0x7373,0x8373,0x8383,0x8383,0x5252,0x5252,0x6363,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x4444,0x4444,0x6344,0x8383,0x7352,0x5273,0x7363,0x6363,0x6363,0x5252,0x5252,0x5252,0x5242,0x5252,0x5252,0x6363,0x6352,0x5273,0x7363,0x5273,0x6363,0x5263,0x6363,0x5252,0x5252,0x5263,0x5252,0x5273,0x5252,0x5252,0x4242,0x5242,0x5242,0x5252,0x4152,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4152,0x4142,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x5152,0x41C4,0x9F9F,0x4251,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x5241,0x7F7F,0x5242,0xD473,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4241,0x4242,0x5252,0x5252,0x5263,0x8373,0x8383,0x8383,0xC483,0xC4C4,0x83C4,0xC4C4,0x93D4,0xC3D4,0xC3C3,0xC4C3,0x83C3,0x7383,0x7373,0x6363,0x5263,0x5252,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x8352,0x83C4,0x8383,0x6373,0x5252,0x4244,0x4242,0x4444,0x5252,0x6352,0x6363,0x6352,0x6363,0x6363,0x6352,0x6352,0x5252,0x4452,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x4444,0x4444,0x4444,0x8363,0x5263,0x6352,0x5252,0x5252,0x5252,0x5252,0x5242,0x5252,0x5252,0x6363,0x6363,0x6352,0x7373,0x5263,0x7363,0x7373,0x5252,0x7363,0x8383,0x5273,0x7363,0x8383,0x5252,0x5252,0x6363,0x7363,0x5252,0x5252,0x4252,0x4142,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x62D4,0xD4C4,0xD462,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6241,0x8F8F,0x7F7F,0x6352,0x5252,0x5263,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x5252,0x6352,0x7363,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0xC483,0x83C4,0xC4D4,0xD4D4,0xC3C4,0xC3C3,0x83C4,0x7373,0x6352,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x7383,0x7363,0x6373,0x6363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x4344,0xC4C4,0x8383,0x7383,0x6373,0x6352,0x5252,0x5252,0x5252,0x5252,0x6352,0x6373,0x5252,0x8373,0x5273,0x6352,0x6352,0x7383,0x6352,0x6363,0x7383,0x6352,0x6373,0x6373,0x7363,0x7373,0x5263,0x8363,0x8383,0x5263,0x7363,0x7373,0x5263,0x4141,0x5241,0x5242,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6252,0x9F9F,0xC4D4,0x62E4,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x41C4,0x7F7F,0x5243,0x6363,0xC473,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x5242,0x6352,0x7373,0x7373,0x7373,0x7383,0x7383,0x8383,0x8373,0x8383,0x8383,0x8383,0x83C4,0xC4C3,0xC3C4,0xC4C4,0xC4C3,0x83C4,0x7373,0x6352,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7352,0x73C4,0x6373,0x7363,0x5263,0x8383,0x4363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244
.word 0x8383,0x7383,0x8383,0x6373,0x6363,0x5252,0x5252,0x4252,0x6352,0x6352,0x6363,0x5263,0x7363,0x7363,0x5273,0x6363,0x7363,0x8373,0x6383,0x7352,0xC483,0x6383,0x8363,0x83C3,0x6363,0x8373,0x7383,0x6352,0x8373,0x5273,0x7352,0x7373,0x5273,0x7F7F,0x42D4,0x9F9F,0x7F7F,0x5241,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xD452,0xF4D4,0x4152,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x5252,0x6363,0xC483,0x83D4,0x4463,0x4444,0x7373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x5241,0x6352,0x6352,0x6363,0x6363,0x6373,0x7373,0x7373,0x7383,0x8373,0x8383,0x8383,0x8383,0xC483,0xC493,0x9383,0xB3C4,0xD4C4,0x83C3,0x7373,0x4352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x7363,0x6373,0x6363,0x6352,0x8373,0x8373,0x8383,0x7373,0x7383,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x7383,0x7373,0x6373,0x5263,0x5252,0x4252,0x5252,0x6363,0x5252,0x5263,0x7373,0x6352,0x6363,0x7352,0x8383,0x6352,0x7373,0x8373,0x6383,0x6373,0x8383,0x8363,0x8383,0x5263,0x8383,0x8383,0x5273,0xC383,0x5283,0x5252,0x7363,0x7F7F,0xD441,0xD452,0x5252,0x7F7F,0xD452,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4251,0xD4C4,0x62D4,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7363,0x8383,0xC4C4,0xC383,0x83C4,0x8363,0x8363,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x5242,0x5263,0x6363,0x6363,0x6363,0x6373,0x7363,0x7373,0x7373,0x7373,0x8373,0x8373,0x8383,0x8383,0x8383,0xC383,0x83C4,0xC4C4,0x83C3,0x7383,0x4363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x7383,0x6363,0x5273,0x8373,0x8383,0x7383,0x8383,0x7373,0x6363,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444
.word 0x8383,0x8363,0x6383,0x6363,0x5252,0x5242,0x6363,0x5263,0x6363,0x8373,0x8352,0x8383,0x8383,0x6373,0x7352,0x8383,0x83C4,0x7363,0xC483,0x6383,0x7352,0xC4C4,0x5283,0xC452,0x73C4,0x7363,0x8363,0x7383,0x8373,0x7383,0x7363,0xC463,0x4141,0xC4C4,0xD4C4,0x41E4,0x5241,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x6241,0x62C4,0x41C4,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x4241,0x5252,0x8363,0xC4B3,0x8383,0x7383,0x7373,0x4383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x6352,0x6363,0x6363,0x6363,0x6363,0x7352,0x6363,0x6363,0x7373,0x8383,0x7383,0x8373,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x83C3,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0x7373,0x7363,0x6352,0x8383,0x7373,0x8383,0x6373,0x4252,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7373,0x7373,0x6352,0x6363,0x5242,0x5252,0x7363,0x6373,0x6352,0x5252,0x7363,0x7373,0x6383,0x8373,0x6363,0x8373,0x6373,0x5283,0x8373,0xC483,0x6363,0x8383,0x6363,0x8363,0xC4C3,0x6352,0xC483,0x52C4,0x6363,0x5283,0x8373,0x5273,0x8F8F,0x9F9F,0x5262,0x4141,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4141,0xAFAF,0x9F9F,0xC4C4,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x5252,0x7F7F,0xE441,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x5263,0x6342,0x5273,0x8363,0x7363,0x7373,0x83C4,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x6352,0x5263,0x6352,0x5263,0x6363,0x5252,0x7363,0x6373,0x7373,0x7373,0x7373,0x8373,0x8383,0x8383,0x8383,0x6373,0xC483,0x8383,0x7383,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0x6373,0x6383,0x7373,0x83C4,0x8373,0x7373,0x4463,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x7363,0x5252,0x5252,0x5252,0x6373,0x7363,0x7373,0x7363,0x8383,0x5263,0x6363,0x8363,0xC3C4,0xC4B3,0x6383,0x8373,0xC4B3,0x5283,0x8383,0x83C3,0x7363,0xC383,0x5273,0x8363,0x5273,0x8383,0x5283,0x8373,0x52C4,0xC473,0x4152,0x52D4,0x8F8F,0x7F7F,0x8F8F,0x4162,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6242,0xC4C4,0x52D4,0x6F6F,0x7F7F,0x4141,0x52C4,0xD4D4,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x4141,0xC483,0x6383,0x6342,0x6352,0x8373,0x83C4,0x8383,0x8383,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5242,0x5252,0x5252,0x5252,0x5263,0x5252,0x5263,0x6363,0x6363,0x7363,0x6373,0x6373,0x8383,0x8383,0x6363,0x7373,0x7363,0x7383,0x6363,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0x6363,0x6363,0x8373,0x7373,0xC483,0x7383,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444
.word 0x6373,0x6363,0x5252,0x5252,0x7352,0x7373,0x5273,0x6352,0x8363,0x8383,0xC4C4,0x6383,0x8373,0xC383,0x8383,0x6373,0x8363,0x7383,0x6363,0x8363,0x6373,0x6363,0x8383,0x6363,0xC383,0x63C4,0x6363,0x5273,0x8383,0x73C4,0xD452,0x9F9F,0x8F8F,0x5252,0x8F8F,0xD452,0x7F7F,0x5241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5252,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x4141,0xF4C4,0x62D4,0x52D4,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x9373,0xC4C4,0x6383,0x6342,0x7363,0xC483,0x8383,0x8383,0x63C3,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5242,0x6383,0x7363,0x6383,0x7352,0x5273,0x6352,0x5252,0x5252,0x5263,0x5263,0x6363,0x7363,0x6363,0x8363,0x6363,0x7383,0x6363,0x8363,0x7383,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0x7373,0x6352,0x6383,0x6363,0x8383,0x8383,0x7352,0x6373,0x5252,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444
.word 0x5263,0x5252,0x5242,0x5252,0x6363,0x8373,0x8383,0x7363,0x5273,0x7363,0x6383,0x8363,0x5263,0x7373,0x7373,0xC3C4,0x5283,0x8352,0xC3C4,0x5283,0xC483,0x83C3,0x7352,0x73C4,0x8363,0x6383,0x8373,0x83C3,0x7352,0x6363,0xC452,0x4162,0x7F7F,0x7F7F,0x5252,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x4141,0x8F8F,0x5241,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x9F9F,0x5262,0x41D4,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x8352,0x8383,0x8373,0x7383,0x6352,0x7363,0x8383,0x7373,0xC483,0x73C4,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x6363,0x6363,0x6352,0x5252,0x5263,0x5273,0x5252,0x5283,0x8363,0x5263,0x5263,0x5263,0x5263,0x6363,0x6373,0x7352,0xC483,0xD4D4,0xC4C3,0xC3C4,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6344,0x6363,0x6363,0x7373,0x7373,0x7363,0x6373,0x7363,0x6373,0x8373,0x7383,0x8383,0x8383,0x4473,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444
.word 0x5263,0x4252,0x6352,0x7352,0x5263,0x8373,0x7373,0x8363,0xC4C3,0x5273,0x5252,0x8383,0xC3C4,0x83C4,0x7352,0x8383,0x83C4,0x6363,0x7383,0x5273,0x8383,0x6373,0x7352,0x8383,0x5263,0x6373,0x7373,0x8383,0x8363,0x52C4,0x51C4,0x8F8F,0x4241,0x5241,0x9F9F,0x4152,0x6241,0x4142,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4152,0x4141,0xD452,0x6F6F,0x6F6F,0x6F6F,0x4252,0x5252,0x9F9F,0xC452,0x4141,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7341,0x83C4,0x7383,0x8383,0x6373,0x6342,0x8373,0x8383,0x8373,0x8383,0xD483,0x7383,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x7373,0x5263,0x8363,0x5263,0x6363,0x5252,0x5252,0x6363,0x7373,0x5273,0x7363,0x5252,0x5252,0x6352,0x5252,0x6352,0x6363,0x8383,0x8383,0xC4D4,0x63C4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6373,0x5263,0x6352,0x7373,0x7363,0x7373,0x6373,0x6363,0x7383,0x6373,0x6363,0x6352,0x7373,0x4452,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444
.word 0x5252,0x6352,0x5263,0x8373,0x83C4,0x5263,0x6363,0x7352,0x8383,0x8383,0x6363,0x8363,0x7383,0xC463,0x73C3,0x6352,0x8363,0xC4B3,0x8363,0xC383,0x5263,0x8383,0x52C4,0x7363,0xC473,0xC4B3,0x6363,0x6373,0x8363,0x5263,0x41D4,0x4141,0x7F7F,0x62C4,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x4141,0x4152,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xD441,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x9F9F,0x5252,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xC483,0x8383,0x8383,0xC383,0x6383,0x6342,0x8373,0x7373,0x8383,0x8383,0x8383,0x8383,0x8383,0x83C4,0x83C4,0x8383,0x7383,0x7373,0x7373,0x6373,0x5263,0x6363,0x6363,0x6373,0x6363,0x8373,0x6363,0x6373,0x5263,0x6352,0x7363,0x7352,0x5263,0x7352,0x4273,0x5252,0x5252,0x6363,0x8363,0x7383,0x7373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0xC383,0xC3C3,0x7383,0x5263,0x5252,0x5273,0x6363,0x6363,0x7373,0x6373,0x7363,0x7373,0x4363,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444
.word 0x5252,0x5252,0x7352,0x8373,0x6373,0x8383,0x8383,0x5283,0x7373,0x6373,0x8383,0x6383,0x5252,0x8383,0x83C4,0x7363,0x8352,0x8383,0x6383,0xC483,0x5273,0x8363,0x6383,0x8383,0x8352,0x8383,0x7363,0x83C3,0x6352,0xD452,0x4152,0x7F7F,0x7F7F,0x5241,0x7F7F,0x41C4,0x9F9F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x4141,0x4141,0x4142,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x5241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x8373,0x83C4,0x7383,0x7373,0xC483,0x5273,0x5252,0x7363,0x8373,0x7373,0x8383,0x8383,0x8383,0xC493,0xD4C4,0xC3C4,0xC4D4,0xC4D4,0x83D4,0xC483,0x8383,0x8383,0x8383,0x7373,0x7373,0x5263,0x7373,0x5273,0x7363,0x5283,0x6352,0x5263,0x6352,0x6373,0x5283,0x8352,0x5252,0x6342,0x6352,0x6363,0x5263,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x8373,0xD4C3,0xC4C4,0xC3B3,0x83C4,0x6373,0x5252,0x6363,0x8373,0x7383,0x8373,0x8373,0x7383,0x4243,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x5252,0x8363,0x5263,0x7363,0x6373,0x8383,0x7383,0x8373,0x8383,0x6363,0x8383,0x83C4,0x6363,0x8363,0x7373,0xB3C4,0x6363,0x7363,0x83C4,0x5263,0xC383,0x5283,0x5252,0x8383,0x5283,0x6352,0x7352,0xC483,0x6383,0xC462,0x7F7F,0x7F7F,0x6252,0x7F7F,0x4152,0x6241,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x4142,0x8F8F,0x5251,0x4262,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x5241,0x7F7F,0x6F6F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8373,0xC483,0x8383,0x6373,0x8383,0x8383,0x5273,0x6352,0x6363,0x8373,0x7373,0x8383,0x8383,0x83C4,0x8383,0x8383,0x8383,0x83C4,0x8383,0x8383,0xC3C4,0xD4C4,0xC3C4,0xB3C4,0xC3C4,0xC4C3,0x8383,0x6373,0x6363,0x6363,0x7383,0x5252,0x5252,0x6352,0x5273,0x5273,0x5273,0x8383,0x4252,0x4242,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x8383,0xD483,0x8383,0x8383,0x8383,0x8383,0xB3C3,0x6383,0x6352,0x6352,0x7373,0x7373,0x6373,0x4352,0x4444,0x4442,0x4244,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444
.word 0x6352,0x8373,0x8373,0x5263,0x5252,0x7383,0x6383,0xC483,0xC4D4,0x5273,0x7373,0x6373,0xC383,0x5283,0x6352,0x8383,0x73C3,0x7352,0x8383,0x5263,0x8383,0x8383,0x73C4,0x8363,0x6373,0xC4C4,0x6363,0x8373,0x5283,0x52D4,0x7F7F,0x4141,0x8F8F,0x4141,0xC4D4,0x7F7F,0x4152,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4152,0x5241,0x8F8F,0x4152,0x6F6F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x4162,0x4141,0x6F6F,0x7F7F,0x5242,0x6F6F,0x7F7F,0x5241,0x4152,0x4141,0x5242,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x8383,0x8373,0x8383,0x8373,0x7373,0x8383,0x8383,0x4263,0x6352,0x6363,0x8373,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x83C4,0x8383,0x8383,0x8383,0x8383,0x8383,0xC483,0xD4C4,0xC4C3,0xC4C4,0x7383,0x7363,0x5273,0x8363,0x6373,0x5252,0x5252,0x5252,0x7352,0x7352,0x7352,0x7373,0x6342,0x5242,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0x8383,0xD483,0x8383,0x8383,0x8383,0x8383,0x8383,0xC483,0x83C3,0x5263,0x6363,0x7373,0x8383,0x6363,0x5263,0x4352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6352,0x6373,0x8373,0x8383,0x6373,0x5263,0x5252,0x8383,0x7383,0xC483,0x5283,0x5263,0x8373,0x7383,0x6363,0x7352,0xC463,0x5283,0x6363,0x8383,0x6363,0x6363,0x8373,0x6363,0x6373,0x7363,0x7383,0x7363,0x5252,0x4142,0x7F7F,0x7F7F,0x8F8F,0xD441,0x7F7F,0x9F9F,0x7F7F,0x4141,0x4142,0x7F7F,0x4141,0x5252,0x9F9F,0x5262,0x6252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6252,0xC462,0x8F8F,0x5241,0x7F7F,0x4152,0x5252,0xC441,0x4152,0x52D4,0x62D4,0x52D4,0xD4D4,0xC442,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x8363,0x8383,0x8383,0x8383,0x8373,0x6373,0x7373,0x8383,0x5263,0x6363,0x6363,0x6363,0x7363,0x7363,0x6363,0x6373,0x8363,0x8373,0x8383,0x7373,0x8383,0x7383,0x8383,0x8373,0x8383,0x8383,0x83C4,0x83C4,0x8383,0x8383,0x7383,0x6363,0x6352,0x7373,0x6363,0x6352,0x5263,0x5252,0x5252,0x5252,0x5252,0x5273,0x7352,0x5273,0x8363,0x5252,0x8383,0xC483,0x83C4,0xC483,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0xC3C4,0x7383,0x8352,0x7383,0x6363,0x8373,0x83C3,0x8373,0x6373,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444
.word 0x5263,0x6363,0x7363,0x8373,0x8363,0xC3C4,0x6383,0x7363,0x6373,0x8383,0x83C3,0x7363,0x6352,0x6363,0xB3C4,0x73C4,0x8352,0x7383,0x6363,0x8373,0x6363,0x5283,0x6363,0x8363,0x5283,0x6363,0x5252,0x8363,0x6252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xE452,0x41D4,0x4152,0x7F7F,0x4141,0x7F7F,0x4141,0x5241,0xAFAF,0x5252,0x41D4,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0xD462,0xAFAF,0x6262,0x9F9F,0xC441,0xC441,0x9F9F,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF4F4,0xD4E4,0x52D4,0x52C4,0x9F9F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xC342,0x7383,0x8383,0x8383,0x7383,0x7383,0x5273,0x5252,0x6363,0x5263,0x5252,0x6363,0x6363,0x6352,0x5263,0x6363,0x8373,0x8373,0x7383,0x8383,0x8373,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x83C4,0xC473,0xC4C4,0x83C4,0x7383,0x7363,0x7383,0x4252,0x5273,0x6352,0x7373,0x5263,0x5252,0x5252,0x5252,0x5263,0x5273,0x5252,0x6373,0xC383,0x83C4,0x8383,0x8383,0x8383,0x8383,0x7383,0x8373,0x7373,0x7373,0x8383,0x73C4,0x6352,0x6373,0x8363,0x8383,0x83C3,0x8383,0x7383,0x5263,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442
.word 0xC483,0x7383,0x6352,0x6352,0x8363,0x8383,0x8363,0xB3C3,0x7383,0x7363,0x6373,0xC483,0xC4C4,0x5273,0x7363,0x8363,0x6383,0x7352,0x7373,0x6352,0x5273,0x6363,0x7363,0x6352,0x5273,0x7373,0x8363,0x5252,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xC4E4,0xD441,0x4262,0x5241,0x4141,0x5241,0x4141,0x62D4,0xAFAF,0xD462,0xD441,0x41C4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xE4D4,0xD4E4,0xF4F4,0xE4F4,0xE4F4,0xD4E4,0xE4D4,0xD4E4,0xC4C4,0x7F7F,0x7F7F,0x4152,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0xC483,0x5273,0x7352,0xC483,0x83C4,0x6373,0x5273,0x5252,0x8373,0xC3C4,0x83C3,0x5263,0x7373,0x8383,0x8383,0x6363,0x7373,0x7373,0x8373,0x7383,0x8373,0x8373,0x7383,0x8383,0xD483,0x83C3,0x6373,0x6363,0x8373,0xC4C4,0x83C4,0x6383,0x6363,0x7383,0x6352,0x7373,0x5273,0x8363,0x83C4,0x6363,0x8383,0x5283,0x7373,0x6383,0x8352,0xC4C3,0xC3D4,0x83C4,0x83C4,0x8373,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x6383,0x7352,0x73C4,0x8373,0x8383,0x7383,0x73C4,0x4352,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442
.word 0x8383,0x6383,0xC373,0x7383,0x7352,0x7383,0x8363,0xC483,0x7383,0x7383,0x5252,0x8373,0x7383,0x8373,0x5263,0x8373,0x7373,0x5263,0x6363,0x6352,0x6373,0x5263,0x6363,0x5263,0x5252,0x6363,0x7363,0x4163,0x5252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x4152,0x4141,0x8F8F,0x4141,0x41C4,0x6241,0xD4D4,0x41D4,0xD4D4,0x5241,0x5262,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x5241,0xD452,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xF4F4,0xE4F4,0xE4D4,0xD4E4,0xD4E4,0x52D4,0x5241,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8373,0x73C3,0x5252,0x8373,0x8383,0x7373,0x5273,0xC4C4,0xC3C4,0xD4C4,0xC4C3,0x6373,0x7373,0x7383,0x7363,0x6373,0x8373,0x8383,0x7373,0x8383,0x8373,0x7383,0x8373,0xC383,0x8383,0x6363,0xC3C3,0x4273,0x4242,0x4242,0x4242,0x4242,0x5252,0x5242,0x4252,0x4252,0x5242,0x6363,0x6363,0x5252,0x5252,0x6352,0x6363,0x5252,0xC483,0xC4D4,0xD4D4,0xC4C4,0x8383,0x8383,0x8373,0x8383,0x7373,0x8383,0x8383,0x7373,0x6363,0x8363,0x83C3,0x83C4,0x8383,0x83C4,0x4373,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7363,0x6373,0x8383,0x83C4,0xC473,0x83C4,0x6363,0x7373,0x6363,0xC383,0x52C4,0x5263,0x6352,0x7363,0x6373,0x6352,0x7363,0x7373,0x5252,0x5252,0x5263,0x6363,0x6352,0x5252,0x5263,0x5252,0x6352,0x5252,0x4141,0x7F7F,0x7F7F,0x7F7F,0xC441,0x4141,0x9F9F,0x7F7F,0x5262,0xD441,0x8F8F,0xE442,0xC462,0xD4D4,0x62E4,0xD4C4,0x41C4,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xC462,0xC4C4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xF4F4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0x9F9F,0x52C4,0x4152,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6341,0x5242,0x7363,0x6383,0x7342,0x7383,0x7363,0x6373,0x8373,0xC3C4,0xC3C4,0x83C4,0x6373,0x5263,0x5252,0x7352,0x5273,0x5252,0x7352,0x8383,0x7373,0x8383,0x8373,0x83C4,0x7383,0xC363,0x4263,0x7352,0xC483,0x7373,0xD483,0x8383,0x5273,0x8383,0x8383,0x8383,0x8383,0x8383,0x7373,0x7373,0x7373,0x7373,0x7373,0x8383,0x8383,0xC4C3,0xC3C3,0x93C4,0x83C4,0x8383,0x8373,0x8383,0x8383,0x8383,0x7373,0x7373,0x6383,0x5252,0x8352,0x8383,0x83C4,0x5283,0x4243,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4244
.word 0x6352,0x5263,0x8373,0x6383,0xC483,0x83C3,0x6363,0x5273,0x6352,0x8383,0x7383,0x8383,0x7383,0x6363,0x7363,0x63C4,0x6352,0x6363,0x5252,0x5252,0x5252,0x5252,0x4242,0x4242,0x4242,0x4242,0x5252,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0xE462,0x7F7F,0x4141,0x8F8F,0x7F7F,0x9F9F,0x4141,0xC4D4,0xF4D4,0xD462,0x41E4,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xF4F4,0xF3F4,0xF4F4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8352,0x6383,0x4252,0x6352,0x5252,0x8373,0x7383,0x6373,0x7352,0xC483,0x6373,0x6363,0x6363,0x6363,0x6363,0x6383,0x6363,0x8383,0x6373,0x7352,0x7373,0x8363,0xC473,0x8383,0x7352,0x4283,0x8363,0xC3C3,0xC3C4,0x7373,0xC3C4,0xC4C3,0x7383,0xC452,0xD4C4,0xD4C3,0xD4D4,0xC4C3,0xC4C4,0xC3C4,0xC4C4,0xD4C3,0xD4D4,0x8383,0xC483,0xC4D4,0xC4D4,0x83C3,0x8383,0x8373,0x7383,0x7383,0x8383,0x7383,0x8383,0x8373,0x6373,0x5252,0x7363,0x8363,0x5263,0x4252,0x4442,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xC383,0x73C4,0x5263,0x5252,0x7363,0x6373,0x8363,0xC383,0x5283,0x6352,0x5263,0x7373,0x6373,0x7373,0x6363,0x5252,0x5252,0x5252,0x5252,0x6363,0x6363,0x6363,0x5263,0x5263,0x5263,0x5252,0x4163,0x4141,0x7F7F,0x7F7F,0x7F7F,0x5241,0x9F9F,0x6241,0x9F9F,0x7F7F,0x7F7F,0xC462,0x5241,0x52D4,0xD462,0xC4D4,0x5251,0xE4D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6241,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xC4D4,0x6252,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7383,0x8373,0xC4C4,0x6383,0x6342,0x8383,0x8363,0x6383,0x8363,0x7363,0x8383,0x7363,0x8373,0x8383,0x6373,0x8363,0x6373,0x8352,0x8383,0x6373,0x6352,0x6383,0x5273,0x7342,0x8383,0x8383,0x8383,0xC4C4,0x6373,0xC483,0xC4C4,0x6383,0xC442,0xC393,0xC4C4,0xD4C3,0xC4C3,0xC4D3,0xD493,0xC3C4,0xC3D4,0x8383,0xC4C4,0xD4C4,0x93C4,0xC4C3,0x8383,0x8373,0x7383,0x8383,0x8383,0x8383,0x7373,0x7383,0x6373,0x6352,0x6352,0xC483,0x6383,0x4242,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444
.word 0xC483,0x5283,0xC483,0xC4C3,0x5252,0x5252,0x8373,0x8383,0x8363,0xC3C3,0x6383,0x5252,0x6363,0x5263,0x5252,0x5252,0x5252,0x5252,0x6373,0x5252,0x5252,0x5242,0x4263,0x4242,0x4242,0x4242,0x4142,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6241,0x41E4,0x5241,0x4141,0x4141,0x9F9F,0x5252,0x7F7F,0xAFAF,0xC4C4,0xD462,0x6242,0x5262,0x4141,0x6F6F,0x6F6F,0x7F7F,0xC452,0xD4D4,0xD4C4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F3,0xF4F4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x5252,0x7F7F,0x6F6F,0x7F7F,0x4141,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7383,0x8373,0x8383,0x83C4,0x4263,0x8363,0x8373,0x6373,0x7383,0x83C4,0x8383,0x8383,0x5283,0xC473,0x83D4,0x5263,0x6373,0x8383,0x6383,0x7352,0x6383,0x6383,0x6363,0x6383,0x5252,0x6363,0x7373,0x8373,0x7373,0x8383,0x7373,0x4252,0xC473,0xD4C4,0xD4D3,0xC4C4,0xC4C4,0xC4C4,0xC4C3,0xD4C4,0xC3C4,0xC4C4,0xD4C3,0xC3B3,0xC4C4,0x93C4,0x7383,0x8373,0x8383,0x8383,0x8383,0x7383,0x8373,0x8373,0x5263,0x5252,0x8363,0x8383,0x7383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6363,0x5263,0x8373,0x8383,0xC373,0x83C4,0x5252,0x6363,0x8373,0x8383,0x7363,0x5283,0x6363,0x5252,0x5252,0x7352,0x6363,0x5252,0x5252,0x5252,0x4252,0x4242,0x4242,0x4242,0x4442,0x4242,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x5241,0x8F8F,0x7F7F,0x6252,0x8F8F,0x4152,0xE452,0x41C4,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7373,0x7373,0x8363,0x8383,0x83C3,0x4263,0x8363,0x7352,0x8373,0x6363,0x8383,0x8373,0x6383,0x7363,0x7352,0xD483,0x83C3,0x6373,0x7352,0x7373,0x8363,0xC3C4,0x8383,0x8383,0x8383,0x5273,0x5252,0x5252,0x5252,0x5252,0x4252,0x8373,0x8383,0x8383,0x8383,0xC3C4,0xC3C3,0xD493,0x93D4,0x93D4,0xD4C4,0xC3C4,0xD4D4,0x6383,0x8383,0x5273,0x7373,0x8383,0x8383,0x7383,0x7373,0x7373,0x6373,0x5252,0x5252,0x7342,0x8383,0xC483,0x83C3,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x5273,0x7373,0x6363,0x7373,0x8352,0x8383,0xC483,0x83C4,0x7352,0x7373,0x7363,0x6363,0x5252,0x6352,0x5273,0x5252,0x5252,0x4252,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0xD4C4,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6241,0x6251,0x9F9F,0x8F8F,0xC462,0x4262,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x6252,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8373,0x8363,0x6373,0x8383,0xC483,0x6383,0x5242,0x5252,0x8363,0x6363,0x6373,0x7373,0x6363,0x6363,0x6373,0x8352,0xD4C4,0xC3C4,0x6383,0x7363,0xD483,0x83C4,0xC483,0xC4C4,0xC3C4,0x83C4,0x7383,0x6363,0x6363,0x6363,0xC483,0xC3C3,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xC4C4,0xC4D4,0xC4C4,0xC3C3,0xC3D4,0xC3C4,0xC4D4,0x6363,0x7383,0x6352,0x8373,0x7383,0x8383,0x7373,0x5252,0x5252,0x5252,0x4252,0x7352,0x8383,0xC4C4,0x8383,0xC483,0x7383,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444
.word 0x83C4,0x5283,0x8383,0x5263,0x5263,0x5263,0x7363,0x6373,0x8383,0x5273,0x4252,0x5252,0x6352,0x5252,0x5252,0x4252,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x5252,0x5241,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x8F8F,0x5241,0x5241,0x8F8F,0xC4E4,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xC4D4,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7383,0x7383,0x6363,0x6373,0x8383,0xC483,0x6383,0x5252,0x8373,0x7363,0x7373,0x6373,0x8363,0x6352,0x7373,0x6363,0x8352,0xC4C4,0xC4C3,0xC3C3,0x83C4,0x8383,0x83C3,0xC3C3,0xC4C3,0xB3C3,0xC3C4,0x83C4,0xC483,0xC483,0x93C4,0xC483,0x83C4,0xC483,0xC493,0xC4C3,0xC3C4,0x93C4,0xC3D4,0xD4C4,0xD4C3,0xB3C3,0x73C4,0x8352,0x5273,0x6363,0x8383,0x8383,0x8373,0x7363,0x5273,0x4242,0x5242,0x7363,0x8383,0x8383,0x7383,0xC483,0xC483,0x83C4,0x7383,0x4443,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x7363,0xC4C4,0x83C4,0x8352,0x8383,0x5263,0x5252,0x7383,0x5273,0x5252,0x6352,0x5263,0x5242,0x4452,0x4242,0x4242,0x4242,0x4442,0x4444,0x4244,0x4444,0x4442,0x4442,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6241,0x9F9F,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x4152,0x8F8F,0x4141,0x5241,0xD462,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xD452,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x5252,0x4152,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x73C4,0x7363,0x7383,0x6363,0x6363,0x6363,0x7363,0x4263,0x8352,0x7373,0x5252,0x7363,0x6373,0x6352,0x6363,0x7373,0x6373,0x8383,0xC483,0x8383,0x8383,0x8383,0x83C4,0xC4C4,0xD4C3,0xC3C3,0xC3C4,0x83C4,0xD4C3,0xC3C3,0xD4B3,0x93D4,0xC4D4,0xC4D4,0xC3D4,0xD4C4,0xD4D4,0xC3D4,0xC3C4,0xC4C3,0xC3D4,0xC3C4,0x63C4,0x8363,0x7363,0x8352,0x7383,0x8383,0x6373,0x7363,0x7383,0x6363,0x7363,0x7363,0x6363,0x6363,0x7363,0x7383,0x7373,0x7383,0x7383,0x4363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6352,0x5273,0x8383,0x8383,0x8363,0x7373,0x8363,0x6383,0x6352,0x5252,0x6352,0x5252,0x5242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4152,0x4152,0x5241,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x4141,0x4141,0xC441,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x4262,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7383,0x7383,0x6383,0x7352,0x8383,0x8373,0x8363,0x6363,0x5252,0x7373,0x5252,0x6363,0x6363,0x5263,0x7373,0x7383,0x7373,0x6373,0x7373,0x7373,0x7373,0x7373,0x8383,0xD4C3,0xD4C3,0xC4C4,0xD4D4,0xD4C3,0xC4C4,0xC3C4,0xC3C4,0xD4C3,0xC4C3,0xC3C3,0xC4C3,0x93C3,0xD3C4,0xC4C4,0xD4C3,0xC4C3,0x83C3,0xD483,0x5273,0x7363,0x6363,0x8373,0x8373,0x6373,0x4242,0x8373,0x5273,0x8373,0xC3C4,0x7383,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xC3C4,0x8383,0x6363,0x6363,0x6363,0x6363,0x7352,0x6373,0x6373,0x5252,0x5263,0x5252,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x4162,0x4152,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x5241,0x5252,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6363,0x6363,0x7363,0x5252,0x8373,0x7383,0x8373,0x7383,0x5252,0x6352,0x8383,0x7373,0x7373,0x8373,0x8383,0x8383,0x6383,0x7363,0x7363,0x8383,0x8383,0x8383,0x8383,0x8383,0xC4C4,0x93C3,0xC3D4,0xC4C3,0xD4C3,0xC4C3,0xC3C4,0xC3C4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD3D4,0xC3C4,0x83C4,0x8373,0x73C4,0x6352,0x8373,0x5273,0x8373,0x8383,0x5263,0x6352,0x7383,0x6363,0x8383,0xC493,0x73B3,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x7383,0x8373,0x8383,0x6373,0x8373,0x6383,0x6352,0x5252,0x5263,0x5252,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5252,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x52C4,0x8F8F,0x6F6F,0x7F7F,0x6F6F,0xC441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x4142,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7373,0x4263,0x4242,0x5263,0x7352,0x73C4,0x7383,0x8383,0x7383,0x4252,0x5252,0x5252,0x6363,0x6373,0x6363,0x7373,0x6373,0x7363,0x7363,0x7363,0x6363,0x5263,0x8373,0xC3C4,0xC3C4,0xC4C4,0x83C4,0xC383,0xC3C4,0xC3D4,0xC3D4,0xC3D4,0x83C4,0x8383,0x8383,0x8383,0x8383,0x7383,0x7373,0xD483,0x83C4,0x5252,0x5263,0x8363,0x5283,0x8373,0x8383,0x6373,0x8373,0x6373,0x7352,0x8383,0x8383,0x83C3,0x4443,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x5263,0x5263,0x5263,0x5263,0x5263,0x5263,0x5263,0x5263,0x6363,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7373,0x5273,0xC483,0xC3C4,0x6383,0x8383,0x7383,0x5263,0x6352,0x4252,0x4452,0x4242,0x4442,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0x7F7F,0x4152,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5262,0x4241,0x6F6F,0x6F6F,0x7F7F,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x8342,0x7373,0xC383,0x83C4,0x7383,0x5252,0x8363,0x7373,0x7373,0x8373,0x8383,0x4263,0x5242,0x7363,0x8373,0xC4C4,0x8383,0x6363,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x6363,0x8383,0x93C4,0xC3D4,0x83C4,0xC483,0xC4C3,0xC4C4,0xC483,0xC4C3,0x8383,0x7373,0x7373,0x8373,0x8383,0xC4C4,0x73C4,0x5263,0x5252,0x5263,0x8373,0x5283,0x8363,0x8383,0x8383,0x6383,0x5273,0x8363,0xB3C4,0xC483,0x83C4,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5243,0x5263,0x8383,0x8383,0xC3C4,0xD4D4,0xC4C4,0xC4D4,0xB3C4,0xD4D4,0xB3C4,0xC4C4,0x8383,0x8383,0x5273,0x6363,0x4444,0x4444,0x4444,0x4444
.word 0xC4C3,0x83C4,0x8373,0x8383,0x6383,0x7373,0x5263,0x5252,0x5263,0x4252,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xC462,0x7F7F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0xC462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x52D4,0x7F7F,0x4141,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7363,0x7373,0x8373,0x8383,0xC483,0x7383,0x5252,0xC473,0x7383,0x6373,0x7373,0x7383,0x6363,0x83C4,0x5273,0x7352,0x7373,0x8373,0x6373,0x6363,0x6373,0x6373,0xC483,0xC4C4,0xC4C4,0x73C4,0x6373,0x8363,0x8383,0x7373,0x7373,0x8383,0x8383,0x8383,0x8383,0xC483,0xD4C4,0xC4C4,0x8383,0x5263,0x6342,0x83C4,0x83C4,0x5263,0x7373,0x8383,0x5263,0x7373,0x7383,0x6363,0x6352,0xC483,0xC493,0x8383,0xC483,0x83C4,0x5273,0x4444,0x4444,0x5243,0x8373,0xC483,0xD4C4,0xC3C3,0xC3C4,0x83C4,0x8383,0x8383,0x6373,0x6373,0x7363,0x6363,0x6373,0x7363,0x8383,0x8383,0x8383,0x8383,0xC4C4,0x8383,0x5263,0x4463
.word 0x8383,0x8383,0x6363,0x6363,0x5263,0x6352,0x5252,0x6352,0x4252,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4442,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5241,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x7F7F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7363,0x7373,0x8373,0x8383,0xC4C4,0x83C3,0x5273,0x5242,0xC473,0x7373,0x8373,0x6363,0x7352,0x7373,0x8373,0x6373,0x6363,0x5252,0x6373,0x6363,0x8363,0x8383,0x8383,0x8383,0x8383,0x8383,0xC4C4,0x83C3,0x8383,0x8383,0x8383,0x7383,0x6373,0x6373,0x8383,0xC483,0x8383,0x6373,0x4252,0x4241,0x6344,0xC4D4,0xC483,0x6373,0x7363,0x8373,0x8383,0x5263,0x5252,0x6352,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x7383,0x6363,0x7352,0x8383,0xD4C4,0x93D4,0x83C4,0x7383,0x6373,0x5252,0x5252,0x5252,0x4444,0x4242,0x4244,0x4242,0x4442,0x4242,0x5252,0x5252,0x6352,0x7373,0x7373,0x8383,0x8383,0x8383
.word 0x6352,0x5263,0x8363,0xC4C4,0x83C4,0x7363,0x6352,0x6363,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5241,0x7F7F,0x4241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5241,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4241,0x8383,0xC4C4,0x83C3,0x7383,0xC483,0x8383,0x8383,0x7383,0x4252,0x7363,0x6363,0x5273,0x5252,0x7363,0x7373,0x5263,0x8373,0x8383,0x7363,0x7373,0x6373,0x7352,0x8383,0x8383,0x5273,0x4242,0x4242,0x6342,0x8383,0x8373,0x7383,0x8383,0x8383,0x8383,0x7383,0x4263,0x4242,0x4242,0x4242,0x4242,0x4444,0xC3C3,0x83C4,0x4452,0x5273,0x8363,0xC483,0xC4D4,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0xC483,0xC4C3,0xC4C3,0xC4C3,0x83C4,0x8383,0x6363,0x6363,0x5252,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4442,0x4442,0x4242,0x4244,0x4242,0x4242,0x4442,0x4242,0x4242,0x4442,0x5252,0x6363
.word 0xC4C3,0xC4B3,0x7352,0x8383,0x8383,0x5263,0x6352,0x5252,0x4452,0x4444,0x4442,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5242,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0xC441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x52C4,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0xE4D4,0xE4D4,0x8383,0x5283,0x6363,0x8363,0x8383,0x6383,0x6363,0x7363,0x8383,0x5273,0x6352,0x6383,0x7373,0x8373,0x7373,0x7352,0x6373,0x7383,0x5263,0x8363,0xD4D4,0x83B3,0x4263,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x6363,0x6363,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x5244,0xC4C4,0x5273,0x4244,0x4242,0x6352,0x8373,0xC3C4,0xD4C4,0xC4C4,0xC3D4,0xC3C4,0xD4C4,0xC4C3,0xC4D4,0x8383,0x7373,0x8373,0x8383,0x6363,0x5252,0x5252,0x4242,0x4242,0x4442,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4442,0x4444,0x4444,0x4442,0x4442,0x4242,0x4242,0x4242,0x4242
.word 0x8383,0x8383,0x5273,0x7373,0x7373,0x5252,0x5263,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5252,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4152,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0xD452,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x9F9F,0x4142,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0xD4E4,0xD4D4,0x63E4,0x8363,0x6342,0x8383,0x8383,0xC483,0x8383,0x7373,0x7373,0x5273,0x5252,0x6373,0x7383,0x8373,0x7373,0x7373,0x7373,0x5263,0x8352,0xC3C4,0x93C3,0x5273,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x8373,0x8383,0x5273,0x4241,0x4444,0x4444,0x4444,0x4444,0x4444,0x8344,0x5273,0x4242,0x4444,0x4244,0x4242,0x5244,0x6373,0x6363,0x6363,0x6363,0x6363,0x6363,0x6363,0x6363,0x6363,0x5263,0x5252,0x5252,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444
.word 0x8383,0x8383,0x6363,0x6352,0x6373,0x5252,0x5252,0x4252,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0xD4D4,0xD452,0x42E4,0x5252,0x4263,0xC463,0x8373,0x7373,0x6373,0x8363,0x8383,0x5273,0x6352,0x6383,0x8373,0x7363,0x8383,0x8383,0x8383,0x5252,0xC383,0x93D4,0x6383,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6344,0xD4D4,0x83C3,0x4263,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x6344,0x4243,0x4244,0x4444,0x4442,0x4442,0x4242,0x4242,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x4252,0x4242,0x4242,0x4242,0x4244,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6363,0x7363,0x6383,0x8373,0x7383,0x6352,0x5252,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x4241,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4241,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4262,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5252,0xD452,0x83E4,0x8383,0x6383,0x6342,0x8383,0x7363,0x7373,0x7373,0x8383,0x6383,0x5252,0x6363,0x8363,0x6383,0x6363,0x8363,0x5273,0x6363,0x5263,0x5252,0x4352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x9352,0x6383,0x4452,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4452,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4442,0x4242,0x4244,0x4244,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4242,0x4444,0x4444
.word 0x8352,0x83C4,0x83C4,0x8373,0x7383,0x5252,0x5252,0x4442,0x4444,0x4244,0x4442,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4241,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0xC441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x52C4,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x9F9F,0xD462,0xC4E4,0x8383,0x83D4,0x4263,0x8363,0x7383,0x7373,0x7373,0x7373,0x8363,0x4263,0x6352,0x5273,0x6363,0x8383,0x8383,0x6363,0x8363,0x6383,0x7352,0x8383,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5252,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7352,0x8383,0x8383,0x6352,0x6363,0x5273,0x5252,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52C4,0x4141,0x7F7F,0x6F6F,0x7F7F,0x5252,0xD452,0xD4C4,0x83E4,0x8373,0x8383,0x7383,0x6342,0x8383,0x6383,0x6363,0x7373,0x7383,0x8373,0x4252,0x5252,0x5252,0x6363,0x6363,0x8383,0x8383,0x5273,0x6363,0x8363,0xB3C4,0x83C4,0x7373,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x8352,0x7373,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4244,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6383,0x8383,0x8383,0x8363,0x6383,0x5263,0x5252,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4244,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5241,0x4152,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x4152,0x7F7F,0x6F6F,0x4141,0x5252,0xD4D4,0xD462,0x73E4,0x7373,0x8373,0x8383,0x4273,0x6352,0xC483,0x83C4,0x8383,0x8363,0xC483,0x8383,0x5273,0x4242,0x6352,0x7383,0x8383,0x7373,0x7373,0x5273,0x7352,0x8363,0xC483,0xC4B3,0x7383,0x6373,0x4343,0x4343,0x4343,0x4343,0x7343,0x8383,0x83C4,0x8383,0x8383,0x7373,0x4443,0x4444,0x6363,0x7373,0x4473,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4443,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x6344,0x4352,0x4244,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x6363,0x6352,0x7363,0x5273,0x5252,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4442,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4142,0x4241,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x5262,0x4152,0x5242,0xD4D4,0xC4C4,0x62C4,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0xC462,0xD4C4,0xE4C4,0xC4C4,0x8383,0x6373,0x8383,0x8383,0x6383,0x5242,0x6363,0x8363,0x7383,0x6363,0xC483,0xC4C3,0x5283,0x6352,0x8373,0x7373,0x7373,0x6373,0x5252,0x7373,0x6373,0x6363,0x6363,0x6363,0x5263,0x7352,0x7373,0x7373,0x7373,0x7373,0x6373,0x7373,0x7373,0x6363,0x6363,0xC4C3,0x7383,0x4252,0x5252,0x5252,0x4444,0x4442,0x4444,0x4244,0x4444,0x6344,0x6383,0x4444,0x4442,0x4444,0x4444,0x4444,0x7363,0x8373,0xB3C4,0x6383,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4442,0x4244
.word 0x8383,0x6363,0x8383,0x7363,0x5273,0x5252,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4241,0x4152,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4252,0x6F6F,0x6F6F,0x7F7F,0x4141,0xD462,0x62D4,0x5262,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0xE462,0xD4D4,0x5283,0x8373,0xC4C3,0x8383,0xC4C3,0xC4C3,0x5283,0x8363,0xC4C4,0x7373,0x8373,0x8373,0x93C4,0x52C4,0x6352,0x7373,0x8363,0x7363,0x8383,0x7383,0x6352,0x7352,0x6363,0x7363,0x6383,0x8383,0x7373,0x7373,0x7383,0x7373,0x8373,0x7383,0x7363,0x7373,0x7373,0x6363,0x6363,0x8373,0x83B3,0x6373,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x8363,0xC4B3,0x4463,0x7363,0x8373,0xC4D4,0xB3D4,0xB3C4,0x8383,0x8383,0x5283,0x4442,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6373,0x8373,0x8373,0x7383,0x5273,0x5252,0x5252,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4142,0x8F8F,0x7F7F,0x7F7F,0x4152,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4152,0x6F6F,0x7F7F,0x6F6F,0xD4C5,0xE5D4,0xD4D4,0xD4D4,0xC4D4,0x6262,0x9F9F,0x7F7F,0x7F7F,0xC452,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x7383,0x4252,0x4242,0x6363,0x8383,0xC483,0xC4C4,0x7383,0x8373,0x8373,0xC483,0x8383,0x5283,0x5252,0x7383,0x7373,0x6363,0x7363,0x5273,0x5252,0x6363,0x8373,0x83C4,0x7383,0x8383,0xC383,0x8383,0x7373,0x7373,0x8383,0x8383,0x7383,0x7373,0x7383,0x7383,0x6373,0x6373,0x6363,0x7363,0xC383,0x73C4,0x7373,0x4443,0x4444,0x7343,0x8383,0x7383,0xC483,0x6363,0x6363,0x5263,0x5252,0x5252,0x5252,0x4243,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xC4D4,0x5273,0x7373,0x8373,0x5283,0x5252,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4142,0x7F7F,0x4152,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xC4D4,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4152,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xD4D4,0xE4E4,0xE4E5,0xE5E5,0xE5D4,0xD4D4,0x9F9F,0x5262,0x6F6F,0x4141,0xC462,0xD4C4,0xE462,0xE4D4,0x83C4,0xD483,0xD4C3,0x6383,0x5242,0xC473,0x8383,0x8383,0x7383,0x7373,0x8373,0x8383,0x8373,0x6383,0x5242,0x8363,0x7373,0x7363,0x5263,0x8363,0x7373,0x6383,0x7352,0x8373,0x7373,0x8383,0x8373,0x7363,0x8383,0x8383,0x7373,0x6373,0x8373,0x8383,0xC4C4,0xC4C3,0x6383,0x6363,0x7373,0x6373,0x6363,0x7363,0xC483,0xC4D4,0x83D4,0x7383,0xC473,0x73C4,0x5263,0x6352,0x6363,0x4452,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4442,0x4444,0x4444
.word 0x8383,0x6363,0x7363,0x8383,0x5283,0x5252,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4142,0x4241,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6252,0x8F8F,0x62D4,0x4152,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6242,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4152,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xC442,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4152,0xC441,0x9F9F,0x7F7F,0x6241,0xD4D4,0xD4D4,0xE462,0xC3D4,0xC483,0x83C4,0x8363,0x8383,0x7383,0x4252,0x6363,0xC483,0xC4B3,0x8383,0x7373,0x8383,0x6383,0x8363,0x4263,0x7352,0x8383,0x6373,0x6373,0x8373,0x8363,0x7383,0x5263,0x6352,0x7363,0x83C4,0x8373,0x7373,0x8383,0x83C4,0x8373,0x7373,0x7373,0x8383,0x7373,0x7373,0x7383,0x7373,0x8383,0x7383,0x7363,0x6383,0x6363,0x7373,0x7383,0x6363,0x8373,0xC483,0xC4C4,0x83C3,0x7383,0x7383,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6373,0xC473,0x63C4,0x7363,0x5273,0x5263,0x5252,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x4152,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6242,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x5241,0x5152,0x4152,0x6F6F,0x7F7F,0x62C4,0xE441,0xD452,0xD4D4,0xE4D4,0xD4D4,0x83D4,0xC483,0xC483,0x8383,0x7383,0x7363,0x8383,0x5273,0x4242,0x6363,0x8383,0x7373,0x7373,0x7363,0x83C4,0x6383,0x5242,0x8363,0x7363,0x7383,0x7373,0x7373,0x5263,0x8383,0x8373,0x7373,0x7373,0x7383,0x73C4,0x8373,0x7383,0x7373,0x6373,0x7373,0x7383,0x7363,0x8383,0x8383,0x8383,0x7383,0x8363,0x8363,0x7383,0x6373,0x7363,0x6373,0x8373,0x7373,0x6383,0x8373,0x83C4,0x8383,0x8383,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4444,0x4444
.word 0x5252,0x7383,0x7383,0x7363,0x5252,0x5263,0x5252,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4144,0x7F7F,0x4141,0x7F7F,0x4152,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x52C4,0x4141,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0xD4C4,0x62D4,0xD442,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0x63D4,0x6363,0x6363,0x8383,0xC483,0xC4C4,0x8383,0x8373,0x7373,0x7373,0x8373,0x7373,0x5263,0xC483,0x8383,0x8383,0x5273,0x5252,0x6363,0x7373,0x7373,0x6363,0x5263,0x5252,0x4242,0x4242,0x5242,0x5252,0x5252,0x6352,0x6363,0x7363,0x6373,0x6363,0x5263,0x5252,0x5252,0x5273,0x7373,0x8373,0x83C4,0x8383,0x6363,0x8363,0x8383,0x8383,0x7383,0x7373,0x7373,0x7363,0x8383,0x8383,0x8383,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8363,0x6352,0x6363,0xC483,0x5273,0x5263,0x6363,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x4152,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xD462,0xD4D4,0xD4D4,0xD4D4,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0xC441,0xD4D4,0x62D4,0xD451,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x42C4,0x4242,0x6363,0x6352,0x6363,0x6363,0x8383,0xD483,0xC3B3,0xD4C4,0xC3C3,0x83C4,0x8352,0xC4C3,0x7383,0x7373,0x8383,0x5263,0x5242,0x5252,0x5252,0x5252,0x4242,0x4242,0x4242,0x4242,0x4244,0x4142,0x5252,0x6352,0x5263,0x6352,0x5263,0x5252,0x6363,0x5252,0x6352,0x6363,0x5252,0x7363,0x8383,0x8383,0x8383,0x8383,0x6363,0x7363,0x8373,0x8373,0x7373,0x8373,0x83D4,0x6373,0x6363,0x4452,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444
.word 0x83C4,0x5283,0x7363,0x8373,0x6383,0x5273,0x5252,0x4452,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4242,0x4144,0x4141,0x7F7F,0x7F7F,0x5252,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x9F9F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0xD441,0xD4D4,0xD4D4,0xD4D4,0x41C4,0x6F6F,0x7F7F,0x5F5F,0x6F6F,0xCFCF,0x6E6E,0x5F5F,0x6F6F,0xBFBF,0x7F7F,0x4F4F,0x5F5F,0x6F6F,0xD441,0xD4D4,0x62D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x6352,0x5263,0x5252,0x5252,0x6363,0x6352,0xC483,0x7383,0x7383,0x8383,0x8383,0xC483,0x4283,0x8363,0x8383,0x6383,0x6383,0x4442,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x5242,0x5263,0x6363,0x7373,0x83C4,0x8383,0x8383,0x8383,0x8383,0x8383,0x8383,0x83C4,0x8373,0x7373,0x6383,0x4452,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4244,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444
.word 0x8383,0x7373,0x6383,0x7373,0x6383,0x6352,0x5252,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4242,0x7F7F,0x4141,0x4141,0x7F7F,0x9F9F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0xCFCF,0xCFCF,0xAFAF,0xFFFF,0xAFAF,0xAFAF,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0xD441,0xD4D4,0xD4D4,0xD4D4,0x7F7F,0x6F6F,0x7F7F,0x5F5F,0x7F7F,0xFFFF,0x7E4E,0xFFFF,0xFC9D,0xEFEF,0x9F9F,0x6F6F,0x4F4F,0x6F6F,0xD4C4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0x52C4,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x5252,0x4252,0x6342,0x8373,0x8383,0x7383,0x7373,0x8373,0xC483,0x6383,0x4252,0x8363,0x8383,0x8383,0x4273,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4244,0x4444,0x4442,0x4444,0x4242,0x5252,0x5263,0x6363,0x8373,0x8383,0x8383,0x83C4,0xD4C4,0xC483,0x8383,0x8383,0x8373,0x6363,0x5263,0x4452,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7363,0x8383,0xC483,0x5283,0x7352,0x6352,0x5252,0x4452,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4442,0x4244,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4242,0x4144,0x7F7F,0x4141,0x7F7F,0x6252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4F4F,0x7F7F,0x5241,0xF3D4,0xF3F3,0xF4F4,0xD4E4,0xC463,0x6F6F,0x8F8F,0x6F6F,0xD441,0xE4D4,0xD4E4,0xD4D4,0x41D4,0x7F7F,0x8F8F,0x7F7F,0xD4C4,0xDFDF,0xDFDF,0xAFAF,0xDFDF,0xC4E4,0x7F7F,0x4F4F,0x6F6F,0xD4D4,0xE4D4,0xD4E4,0x62D4,0xE4D4,0xD4F4,0x62E4,0xD4D4,0x4352,0x4444,0x4444,0x4344,0x4444,0x4343,0x5252,0x7352,0x7363,0x6373,0x8373,0xC483,0xB3C4,0xC4C3,0x8383,0x7373,0x6373,0x5263,0x6342,0x6352,0xC383,0x8373,0x7373,0x4283,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6344,0x7363,0xC483,0x6383,0x5252,0x6363,0x8373,0x8383,0x83C4,0x8383,0x7383,0x7383,0x6373,0x7373,0x5263,0x5263,0x4444,0x4444,0x4442,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6383,0x7383,0xC483,0x7373,0x6383,0x6352,0x6352,0x5252,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4442,0x4144,0x7F7F,0x4142,0x7F7F,0x6252,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4F4F,0x4F4F,0x6F6F,0x7F7F,0x4152,0x4141,0x7F7F,0x6F6F,0x8F8F,0x6F6F,0xD441,0xE4E4,0xE4E4,0xD4E4,0x41C4,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6252,0xC4C4,0xD4D4,0x52C4,0x7F7F,0x4F4F,0x4F4F,0xC4C4,0xD4D4,0xE4D4,0xD4E4,0x62D4,0xE4D4,0xD4D4,0xD462,0x83D4,0x6373,0x6373,0x6352,0x6352,0x7352,0x6363,0x8383,0x8383,0x8383,0x6383,0x4242,0x4242,0x5252,0x5263,0x4242,0x4242,0x6342,0x6363,0x7373,0x6342,0x83C4,0x7373,0x8383,0x4263,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x6352,0xC483,0x8383,0x83C4,0x5263,0x5263,0x7352,0x6373,0x6373,0x7373,0x7363,0x6373,0x6363,0x5263,0x5263,0x5252,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x73C3,0x7363,0x7373,0x7352,0x8383,0x5273,0x5263,0x6352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5242,0x7F7F,0xAFAF,0x4141,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x62D4,0x52C4,0x4152,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0xD441,0xE4D4,0xE4F4,0xD4D4,0x52C4,0x7F7F,0x7F7F,0x6F6F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x6F6F,0x6F6F,0x5242,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0x62D4,0xD462,0xD4D4,0xD4E4,0x8383,0x8383,0x8383,0x7383,0x7373,0x7373,0x7373,0x7363,0x8373,0xC4C4,0x8383,0x83C4,0x7383,0x7352,0x7383,0x8383,0x8373,0x8383,0x8383,0x4283,0xC463,0x7383,0x8383,0x6383,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4344,0x7352,0x6363,0x8383,0x8383,0xC383,0x7383,0x5252,0x6352,0x6363,0x6363,0x6363,0x5252,0x5263,0x6363,0x5252,0x6363,0x7363,0x5273,0x4443,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4442,0x4442,0x4444,0x4444
.word 0xC3C4,0x7383,0x6352,0x7373,0x7373,0x5263,0x6373,0x6363,0x4443,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x6F6F,0x4444,0x7F7F,0x7F7F,0x4144,0x7F7F,0x4151,0x7F7F,0x6262,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xD4D4,0xD4D4,0x62D4,0xC4C4,0xC452,0xC4C4,0xC4C4,0x62C4,0x7F7F,0x6F6F,0x7F7F,0xD452,0xE4D4,0xE4F4,0xE4E4,0x41C4,0x7F7F,0x6F6F,0x7F7F,0x5242,0x5252,0x5252,0x5252,0xC452,0xD452,0xD4D4,0xD4D4,0xF8D4,0xE4E4,0xE4D4,0xD4D4,0x9F9F,0xD4C4,0xD4D4,0x83D4,0x8383,0x8383,0x8383,0x8383,0x7383,0x8373,0x8373,0x8383,0x6383,0x6363,0x7373,0x6363,0x6352,0x8383,0x8383,0x8373,0x8383,0x8383,0x8383,0x4273,0x83C4,0x7383,0x8383,0x4263,0x4442,0x4444,0x4442,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7363,0x7363,0x7373,0x4252,0x8363,0xC483,0x7383,0x5273,0x5263,0x5252,0x6363,0x6363,0x5252,0x5252,0x6352,0x5252,0x7363,0x6373,0xC4C4,0x5252,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x5283,0x8383,0xC483,0x8373,0x8383,0x6352,0x5252,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x4242,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xC4D4,0xC462,0xD462,0xE4D4,0xF4F4,0xD4E4,0xD4D4,0x5262,0xC452,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xF8D4,0xD4E4,0xE4D4,0xD4E4,0xF8D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x62D4,0xD4C4,0xD4D4,0x4252,0x4242,0x4242,0x4142,0x5242,0x6352,0x6363,0x5263,0x5252,0x7373,0x7383,0x7373,0x8373,0x7383,0x8373,0x8383,0x8373,0x8383,0x73C4,0x73C4,0x7342,0x8383,0xC4C4,0x5273,0x4441,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4442,0x4444,0x8363,0x4263,0x4242,0x4242,0x4244,0x8373,0x83C3,0x6363,0x6373,0x5273,0x6363,0x5263,0x6363,0x5252,0x6373,0x8352,0x7373,0x5273,0xC383,0x8373,0x83C4,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x6373,0x8373,0x8383,0x7373,0x8383,0x6352,0x6352,0x4363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x4242,0x4444,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4C4,0xE4D4,0xF4F4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4F8,0xF8F8,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0x62C4,0xD4C4,0x52D4,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x7352,0x8383,0x83C3,0x83C3,0x83C4,0x8383,0x8383,0x7383,0x8373,0x7383,0x8383,0x8383,0xC483,0x4263,0x8352,0x83C4,0x5283,0x4242,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x6344,0x8383,0x7383,0x7383,0x7363,0x6373,0x8373,0x6363,0x8383,0x6363,0x8373,0x8363,0x73C4,0x6383,0x7352,0x7383,0x8363,0x8383,0x5252,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242
.word 0x5252,0x6373,0x7373,0x6373,0x6352,0x5263,0x5273,0x6363,0x6352,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6F6F,0x7F7F,0x4244,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x4241,0x7F7F,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xE4D4,0xF4F3,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4F4,0xE4E4,0xD4D4,0xE4D4,0xF4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0x62D4,0x5252,0x4444,0x4444,0x4444,0x4444,0x4444,0x6343,0x8383,0x8383,0xC4C4,0xC3C4,0xC4C4,0xC383,0x83C4,0xC4C4,0x8383,0x8383,0x8383,0x8383,0x7373,0x4263,0x4263,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x7352,0x6373,0x8373,0x8373,0x6363,0x6352,0xC463,0x7363,0xC483,0x8363,0xC473,0x7383,0x7373,0x8383,0x7363,0x4263,0x4242,0x4243,0x4242,0x4342,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x63C4,0xD4D4,0x7383,0x8352,0xC4C4,0x7383,0x6383,0x6363,0x5263,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x4244,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x4241,0x7F7F,0x6252,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xF4F8,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x52C4,0x4446,0x4444,0x4444,0x4444,0x4444,0x7343,0x8383,0x8383,0x7383,0x7363,0x8383,0x7383,0x8373,0x7373,0x7373,0x8373,0x7373,0x8383,0x7383,0x4263,0x7352,0x6383,0x4242,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x7363,0x5252,0x8383,0x8373,0x4242,0x5252,0x8373,0x7363,0x8363,0x7373,0x8373,0xC383,0x5273,0x8363,0x8383,0x5273,0x4443,0x4242,0x4242,0x4442,0x4442,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4244
.word 0x6383,0x8383,0x8383,0x8363,0x8383,0x6383,0x7373,0x7352,0x6352,0x5263,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4142,0x6F6F,0x4242,0x4444,0x4444,0x4142,0x7F7F,0x7F7F,0x7F7F,0x4141,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D3,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xE4D4,0xF4F4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x44C4,0x4444,0x4444,0x4444,0x4444,0xC463,0x7383,0x5252,0x4242,0x4242,0x4242,0x5242,0x6342,0x6363,0x6373,0x7363,0x6363,0x5263,0x4242,0x8363,0x8383,0x6383,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x5273,0x4142,0x8363,0x6383,0x4443,0x4342,0x8352,0xC473,0x5273,0x5252,0x8373,0x8373,0x5283,0x4241,0x8363,0x83C3,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6363,0x7373,0x6373,0x7363,0x7373,0x6363,0x7352,0x5252,0x6363,0x6363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x4442,0x4444,0x4444,0x4144,0x7F7F,0x7F7F,0x4141,0x4241,0x4241,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xF4F4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4462,0x4444,0x4444,0x4444,0x8343,0x6383,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x5252,0x7373,0x6363,0x7373,0x7373,0x8383,0x7383,0x4252,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4244,0x4444,0x4444,0x4444,0x6344,0x4452,0x4242,0x8342,0x5283,0x4444,0x4244,0x7342,0x83C4,0x5273,0x4242,0x7352,0x7383,0x73C3,0x4242,0x5242,0xC473,0x5273,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4244,0x4442,0x4444
.word 0xC363,0x7383,0x7363,0x6373,0x6363,0x8352,0xC4C4,0x6383,0x6352,0x5263,0x4463,0x4244,0x4244,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4242,0x4242,0x4244,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x5241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xE4D4,0xF4F4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4652,0x4444,0x4444,0x5244,0x5273,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6343,0x8383,0xC3C4,0xC3C4,0xC4C3,0x8383,0x6373,0x4242,0x4242,0x4442,0x4442,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4342,0x4442,0x4244,0x7342,0x4273,0x4444,0x4244,0x5242,0x7383,0x4452,0x4244,0x4242,0x8363,0x7383,0x4444,0x4242,0x5242,0x7363,0x4452,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xC483,0xC483,0xC352,0xC3B3,0xC4C3,0xC473,0xD493,0x73C4,0x7352,0x5252,0x5263,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x4152,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xF4F4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0x62D4,0x4446,0x4444,0x4444,0x7363,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4344,0x8363,0x8383,0x7383,0x7373,0x4263,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x5242,0x4373,0x4444,0x4444,0x4242,0x6373,0x4452,0x4244,0x4242,0x6342,0x7383,0x4444,0x4244,0x4242,0x4241,0x4442,0x4444,0x4444,0x4442,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0x7383,0x8363,0xC4C4,0x8383,0x8373,0x8383,0x83C4,0x5252,0x5283,0x6363,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4142,0x7F7F,0x7F7F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xF4F4,0xE4F4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4444,0x4444,0x6344,0x4252,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7352,0x7373,0x4263,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4452,0x4444,0x4444,0x4242,0x5252,0x4444,0x4444,0x4244,0x4142,0x8352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7373,0x5273,0xC483,0x8383,0x7373,0x6352,0x8383,0x6373,0x7373,0x6352,0x6352,0x4352,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4242,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4142,0x4142,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xF4D4,0xF3F4,0xE4F4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5262,0x4444,0x4444,0x4344,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x7373,0x4252,0x4441,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x5242,0x4444,0x4444,0x4444,0x4244,0x5241,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x83C4,0x6352,0x6363,0x6363,0x6363,0x8383,0x6373,0xC483,0x83C4,0x5263,0x5263,0x6352,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x5241,0x7F7F,0x7F7F,0x7F7F,0x4241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xD4C4,0xD4D4,0xF4D4,0xF5F5,0xE4F7,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4662,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4344,0x5252,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4342,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x8383,0xC452,0xC3B3,0x8383,0xC463,0x83C4,0x83C4,0x8383,0x6383,0x6363,0x7352,0x6352,0x6363,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4444,0x4444,0x4242,0x4244,0x4442,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5252,0x5241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xFFFF,0xE4F4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4462,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x5243,0x4152,0x4442,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4242,0x4442,0x4242,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7373,0xC473,0x8383,0x8383,0x8363,0x8383,0x6383,0x5252,0x6352,0x8383,0x5273,0x7352,0x6352,0x4463,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD462,0xD4D4,0xD4D4,0xF4E4,0xFFFF,0xE4F4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x5244,0x4263,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4244,0x4242,0x4442,0x4242,0x4242,0x4242,0x4442,0x4442,0x4442,0x4442,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x6373,0x8383,0x8383,0x6373,0x6363,0x6363,0x6363,0xC3C4,0x5283,0x7352,0x6383,0x5252,0x6373,0x6363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4244,0x4242,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5241,0x4152,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xF4D4,0xF3F3,0xE4F3,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD462,0x4452,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x5252,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4242,0x4442,0x4442,0x4442,0x4442,0x4244,0x4442,0x4442,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4442,0x4244,0x4244,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x5263,0x6363,0x6363,0x7363,0x83C4,0xC4D4,0x8363,0x83C3,0x7363,0x8383,0x7352,0x6383,0x7352,0x6363,0x4352,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4242,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xC442,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xF4E4,0xFFFF,0xE4F4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x9F9F,0xD4C4,0x4444,0x4444,0x4444,0x4442,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4444,0x4442,0x4244,0x4242,0x4444,0x4442,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4242,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4442,0x4244,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x5273,0x8383,0x8383,0x6383,0xC4C3,0x8383,0x7383,0x7373,0x8352,0xC4C3,0x8363,0x83C4,0x5263,0x6373,0x7373,0x4463,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5252,0x5252,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xE4E4,0xE4F4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x62C4,0x4444,0x4444,0x4242,0x4242,0x4444,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4244,0x4244,0x4242,0x4442,0x4442,0x4444,0x4242,0x4444,0x4242,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7352,0x8383,0x7383,0x5273,0x8383,0x6383,0x5252,0x8363,0x7383,0x7383,0x6363,0x8383,0x6373,0x5252,0x6373,0x6373,0x4463,0x4444,0x4444,0x4244,0x4444,0x4242,0x4244,0x4242,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x8F8F,0x9F9F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC4D4,0xC452,0xC4C4,0x6262,0xE4D4,0xE4E4,0xD4D4,0xC4D4,0x5262,0x5252,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4C4,0x44D4,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4442,0x4444,0x4444,0x4444,0x4442,0x4244,0x4444,0x4242,0x4244,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4244,0x4444,0x4444,0x4242,0x4242,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4242,0x4244,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4242,0x4442,0x4442,0x4444,0x4444,0x4442,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4442,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x7352,0x7383,0x6363,0x7363,0x7373,0x5263,0xC383,0xC483,0x5283,0x6363,0x6363,0x5252,0x6352,0x5252,0x7352,0x7363,0x5263,0x4443,0x4244,0x4444,0x4444,0x4442,0x4244,0x4242,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x5241,0xD452,0x4141,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0x41D4,0x5241,0x7F7F,0xAFAF,0x5262,0xD4D4,0xC4C4,0x41C4,0x8F8F,0x9F9F,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0x44D4,0x4444,0x4444,0x4244,0x4442,0x4442,0x4444,0x4444,0x4244,0x4242,0x4444,0x4244,0x4442,0x4242,0x4244,0x4444,0x4244,0x4444,0x4444,0x4242,0x4444,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4244,0x4444,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4442,0x4442,0x4244,0x4244,0x4242,0x4242,0x4244,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444
.word 0x6352,0x6363,0x5252,0x8363,0xC4C3,0x7383,0x7373,0x8383,0x7373,0xC483,0x83C4,0xC463,0xC4C4,0x8373,0x5273,0x6352,0x6373,0x7363,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x5252,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x42D4,0x4141,0x4141,0x4141,0xD4C4,0xD4D4,0x8F8F,0x4141,0x4141,0xD452,0xF4F4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0x44D4,0x4244,0x4444,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4244,0x4442,0x4444,0x4444,0x4244,0x4244,0x4442,0x4444,0x4442,0x4444,0x4244,0x4442,0x4442,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4244,0x4244,0x4242,0x4442,0x4244,0x4244,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4244,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4244,0x4444,0x4444,0x4242,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442
.word 0x6352,0x7352,0x8383,0xC452,0x8383,0x83C3,0x6352,0x6373,0xC452,0xC3C4,0xC4C3,0x8373,0x93C4,0xC373,0x8383,0x5252,0x6373,0x6373,0x7363,0x6352,0x4444,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x9F9F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD441,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xC4D4,0x4142,0x4141,0x4241,0x4241,0x5241,0x4142,0x4141,0x6252,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4C4,0xD4D4,0x4462,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4244,0x4442,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4242,0x4244,0x4444,0x4444,0x4242,0x4442,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4444,0x4244,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242
.word 0x6352,0x6363,0x7363,0x7352,0x8373,0x7383,0xD483,0xC4C4,0x63C3,0x8383,0x6383,0x5252,0x5252,0x8363,0x73C4,0x8373,0x5263,0x7352,0x6363,0x6373,0x5273,0x7373,0x6363,0x6352,0x4163,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x41C4,0x4141,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x9F9F,0x5152,0x9F9F,0xD4C4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xF4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4442,0x4444,0x4242,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4244,0x4242,0x4244,0x4442,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4444,0x4442,0x4242,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4244,0x4242,0x4444,0x4244,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4442,0x4444,0x4444,0x4444
.word 0x6352,0x6352,0x7363,0x6352,0x6363,0x6352,0x8363,0x7383,0x6383,0x7373,0x5263,0xC473,0x83C3,0x6363,0x5283,0xC373,0x83C4,0x5263,0x8352,0x6363,0x6373,0x6373,0x7373,0x6363,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD441,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xF4D4,0xD4D4,0x9F9F,0xD4C4,0xE4D4,0xD4F4,0xE4D4,0xF4F4,0xF4D4,0xF4F4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4442,0x4444,0x4244,0x4442,0x4242,0x4242,0x4242,0x4442,0x4242,0x4242,0x4242,0x4244,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4244,0x4442,0x4242,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4242,0x4242,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444
.word 0x5242,0x5263,0x5252,0x7352,0xC4C4,0x83C4,0x7352,0x6373,0x8352,0xC4C4,0xC4C4,0xC463,0xC4C3,0x8383,0x83C4,0x7363,0x6373,0x5252,0x7373,0x5252,0x7363,0x6383,0x7373,0x7373,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0xC441,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xF3D4,0xF3F3,0xE4F4,0xE4D4,0xE4E4,0xE4D4,0xE4F4,0xD4E4,0xF4D4,0xD4E4,0xF4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x4444,0x4244,0x4442,0x4242,0x4442,0x4444,0x4442,0x4444,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4442,0x4242,0x4444,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242
.word 0x4242,0x5242,0x6373,0x7363,0x8383,0x8383,0x6363,0x5273,0xC473,0xC393,0xC3C4,0x8352,0x73C4,0x7363,0xC4C3,0x5283,0x8373,0x7383,0xD452,0x5273,0x6363,0x5252,0x5252,0x5252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5242,0x5241,0x7F7F,0x7F7F,0x7F7F,0x52C4,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD441,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xC463,0xE463,0xE4F3,0xF2E4,0x63E4,0x4363,0x6343,0x7363,0xE4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4442,0x4444,0x4444,0x4242,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242
.word 0x4263,0x5252,0x5252,0x5252,0x6363,0x4263,0xC383,0xC3D4,0x63C4,0x83C4,0x8383,0x6352,0x6363,0x5283,0xC483,0x7373,0xD4C4,0xC4D4,0x7363,0x7363,0xD483,0x73C4,0x6352,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5241,0x5252,0x7F7F,0x7F7F,0x7F7F,0xC441,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0xD4C4,0xD4D4,0xC4D4,0x5262,0x4242,0x6342,0x4243,0x5542,0x5563,0x6363,0x63C4,0x63C4,0x4343,0x6343,0x6343,0x4255,0x4343,0x6343,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4C4,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4242,0x4244,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4442,0x4244,0x4244,0x4242,0x4442,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4242,0x4242,0x4444,0x4442,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442
.word 0x4283,0x5252,0x5263,0x5252,0x5252,0x5263,0xB383,0x83C4,0x6383,0x7373,0x7383,0xD483,0xD3D4,0x83C4,0x7363,0x6352,0x9383,0x83C4,0x6363,0x6383,0x7363,0x5263,0x8373,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6241,0xAFAF,0x7F7F,0x7F7F,0x4141,0x7F7F,0x52C4,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xC4C4,0xD4D4,0x62D4,0x7F7F,0x4242,0x4363,0x6355,0x73C4,0x6363,0x63C4,0x73C4,0x4355,0x6343,0x6355,0x6363,0xC5C4,0xC463,0x7363,0x4263,0x6F6F,0xD463,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4C4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x63C4,0x4242,0x6373,0x8352,0x8383,0x6383,0xC473,0x7383,0x5263,0x6352,0x7383,0xC463,0x8383,0x73C3,0x8352,0x83C4,0x6363,0x5263,0xD483,0x83C3,0xC383,0x7383,0x5252,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4142,0x7F7F,0x7F7F,0x6241,0x5262,0x5252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xC4C4,0xC4D4,0xD4D4,0xC4D4,0x4363,0x4242,0x4242,0x4242,0x4242,0x4242,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6355,0x4343,0x4343,0x4243,0x6342,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x83D4,0x7373,0x7373,0x7373,0x7373,0x4363,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4244,0x4242,0x4442,0x4244,0x4244,0x4444,0x4444,0x4244,0x4442,0x4442,0x4242,0x4242,0x4242,0x4444,0x4442,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4442,0x4444,0x4442,0x4444,0x4242,0x4444,0x4242,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4244,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442
.word 0x83C3,0x5242,0x6352,0x6352,0x6373,0x6373,0x7373,0x6373,0xC452,0xC3D4,0xC4B3,0x8363,0x8373,0x6383,0xC4C4,0x83B3,0x8363,0x6383,0x8383,0x83C4,0xC483,0x83C3,0x4163,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4152,0x7F7F,0x7F7F,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x4141,0x4141,0x7F7F,0x41C4,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4141,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x63C5,0x5542,0x63C5,0xC4C5,0xC4C5,0x4243,0x4242,0x6F6F,0xC443,0xE5C5,0xD5F5,0xC5C4,0x5563,0xC573,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xC4C4,0x83D4,0x6373,0x7373,0x6363,0x6363,0x7373,0x7373,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4242,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4442,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4244,0x4444
.word 0xC4C4,0x4283,0x4252,0x5252,0x5252,0x6352,0x8383,0x8383,0x8352,0x8383,0x8383,0x6352,0x6363,0x8363,0xC3C3,0x8383,0xB363,0x83C4,0x6363,0x5273,0x8373,0x5283,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x5252,0x7F7F,0x7F7F,0x7F7F,0x6252,0x4162,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x5241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6251,0xC4D4,0xC4D4,0xD4C4,0xD4D4,0x6362,0x73C5,0xC463,0xD5C5,0xEFEF,0xFFFF,0xF5D5,0xEFEF,0xE5D5,0xC5C5,0x6373,0x6363,0xC573,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xD4D4,0x7383,0x7363,0x7373,0x7363,0x7373,0x7363,0x6363,0x6373,0x7373,0x7373,0x4443,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4242,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4244,0x4444,0x4444,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0xC4C4,0x7383,0x4242,0x6363,0x8363,0x4283,0x7373,0x8373,0x7373,0x8383,0x7383,0xC473,0xC4B3,0x7373,0x8383,0x6383,0xC483,0xC493,0x6352,0x7383,0x6352,0x4252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4162,0x7F7F,0x7F7F,0x4241,0xAFAF,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xD462,0xD4D4,0xD4D4,0xC4D4,0x52D4,0x7363,0x6355,0x6373,0xC4C4,0xEFEF,0xF5E5,0xD5C4,0xC4C4,0x63C4,0x6373,0x63C5,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x83D4,0xC383,0xC3C4,0xD4D4,0x83C4,0x8383,0x7373,0x6363,0x7363,0x7373,0x7373,0x6373,0x4463,0x4444,0x4244,0x4242,0x4242,0x4442,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4242,0x4242,0x4242,0x4444,0x4244,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244
.word 0xC3C4,0x83D4,0x4273,0x6352,0x6363,0x6363,0x6363,0x8373,0x6363,0x6363,0x7352,0x93C4,0x93C4,0x5283,0x6363,0x5263,0x7373,0x6373,0xC463,0xC4C4,0x5263,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x4152,0x4152,0x7F7F,0xC441,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6252,0xC4C4,0xC4C4,0xD4D4,0xD4D4,0x9F9F,0x73C4,0x6355,0x63C5,0xC573,0x6373,0x7363,0x6363,0xC463,0xC563,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0xD4D4,0x6383,0x5252,0x5252,0x5252,0x5252,0x5252,0x7373,0x8383,0x8373,0x8373,0x7363,0x7363,0x7373,0x7373,0x4463,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4244,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444
.word 0xC4C4,0xC4C3,0x6383,0x5242,0x5252,0x5263,0x6363,0x6373,0x5242,0x8363,0x6373,0x8383,0xD483,0x6383,0x5252,0x7373,0x7363,0x5252,0x7373,0x7383,0x4163,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x5262,0x9F9F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5241,0x7F7F,0x7F7F,0x4141,0x9F9F,0xD462,0xD4C4,0xC4C4,0xD4D4,0x52D4,0x6342,0x63C5,0x5573,0x6363,0xC455,0x5555,0xC563,0x6363,0x62C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4C4,0xD4D4,0x83D4,0x6383,0x5252,0x6352,0x8373,0xC483,0x7383,0x5252,0x6352,0x8373,0xC3C4,0x7383,0x7373,0x8373,0x6373,0x7373,0x4463,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444
.word 0x83D4,0xC3C4,0x83C3,0x5273,0x5242,0x5252,0x4252,0x5242,0x8352,0x8383,0x52C3,0x7373,0x6383,0x7352,0x8383,0x8383,0x8352,0x7383,0x5252,0x5263,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x8F8F,0x7F7F,0x5252,0x7F7F,0x7F7F,0x4141,0x8F8F,0x4152,0x4152,0x4141,0x7F7F,0x5241,0xD462,0xD4D4,0xD4D4,0x4152,0x7F7F,0x7F7F,0x7F7F,0x5241,0x9F9F,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x42C4,0x5241,0x5142,0x4242,0x4251,0x4251,0x4251,0x8F8F,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x83D4,0x5273,0x6373,0x8373,0x83C4,0x8352,0x83C4,0x7383,0x8352,0x83C4,0x5263,0x5252,0x8373,0x83C4,0x7373,0x8373,0x7383,0x7373,0x7373,0x4443,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4244,0x4244,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4242,0x4242,0x4242,0x4244,0x4442,0x4244,0x4242,0x4242,0x4442,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242
.word 0xC4C3,0xC483,0xC4C4,0x7383,0x4252,0x6342,0x7363,0x8373,0x7352,0x7373,0x6383,0x6363,0x5263,0x8363,0x8383,0x6383,0x8373,0x7383,0x7363,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4152,0x7F7F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x6241,0x9F9F,0x8F8F,0x7F7F,0x4152,0xC452,0xC4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x4162,0x7F7F,0x8F8F,0x5252,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x9F9F,0x4141,0x4141,0x4141,0x4141,0x5241,0xD4C4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0x8383,0xC4C4,0x8383,0xC463,0xC4C3,0x5252,0x5273,0x6352,0x6352,0x5252,0x8363,0x6383,0x5252,0x7363,0x8383,0x7383,0x7383,0x7373,0x7373,0x6373,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x4442,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4244,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4242,0x4442,0x4242,0x4444,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xC3C4,0x83C4,0x8383,0xC4C3,0x6383,0x5242,0x6363,0x6352,0x6363,0x5252,0x5263,0x8352,0xC483,0x6383,0x6363,0x6352,0x6373,0x5263,0x7F7F,0x7F7F,0x4141,0x4241,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x8F8F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xC452,0x6262,0x62C4,0xC462,0xC462,0xD4D4,0xD4C4,0xC4C4,0xD4C4,0x62D4,0x8F8F,0x6252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4C4,0xC4D4,0x6383,0x93C4,0x73C4,0x7352,0x5263,0x7373,0xC483,0xC4C4,0x7373,0xC383,0xC3C4,0x6383,0xC4C3,0x6383,0x5252,0x8363,0x8383,0x7373,0x7373,0x8373,0x7383,0x4463,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4442,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4244,0x4442,0x4242,0x4242,0x4242,0x4242,0x4442,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242
.word 0xC4D4,0xC493,0xC3C4,0xD4C4,0x83C3,0x5273,0x6342,0x5252,0x5252,0x5252,0x5252,0x6363,0x8373,0x5273,0x8373,0x7352,0x5263,0x4142,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x5241,0x7F7F,0x4141,0x6241,0xAFAF,0xC4D4,0xAFAF,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x8F8F,0xD462,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xF4F4,0xF4F4,0xF4F4,0xF4F4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8383,0x6383,0x8383,0x5283,0x5252,0x8373,0x6383,0x8383,0xC3C4,0x5283,0xC483,0xC4C3,0x6383,0xC483,0x7383,0x6352,0x5252,0x8363,0x83C4,0x8383,0x8373,0x8373,0x7373,0x4443,0x4444,0x4244,0x4242,0x4244,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x7383,0x4244,0xC3C4,0xC4C4,0xC4C4,0x83C4,0x5273,0x5242,0x6363,0x6373,0x5283,0x6352,0x5263,0x5283,0x7373,0x5273,0x4252,0x7F7F,0x4141,0x4152,0x5241,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4141,0xC452,0xD4D4,0xC4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x41D4,0xD4C4,0xD4D4,0xE4D4,0xE4D4,0xF4D4,0xE4E4,0xF4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7373,0x5263,0x7352,0x5252,0x7373,0xC473,0xC4C3,0x83C3,0x8363,0x8383,0x5263,0x7363,0x5263,0x5252,0x5263,0x7363,0x8383,0x8363,0x5273,0x6352,0x8383,0x8383,0x7373,0x6363,0x7373,0x4373,0x4444,0x4444,0x4444,0x4242,0x4442,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4442,0x4442,0x4244,0x4444,0x4444,0x4444,0x4242,0x4442,0x4442,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4244,0x4444,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4442,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4244,0x4442,0x4444,0x4444,0x4244,0x4442,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444
.word 0x4473,0x4442,0xC493,0x93C4,0x83C4,0xC3C4,0x83C4,0x4263,0x7352,0x6352,0x5273,0x6363,0x8352,0x6383,0x6363,0x5252,0x4141,0x4141,0x7F7F,0x5241,0x4142,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x4141,0x8F8F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4F4,0xE4F4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7383,0xC3C4,0xC4B3,0x8373,0xC4C3,0xC4C3,0x8363,0xC483,0x83C4,0x6352,0x5252,0x6373,0xC383,0xC4C4,0x7383,0x8363,0xD4C4,0x83C4,0x7383,0x83C4,0x5263,0x7352,0x8383,0x8363,0x8383,0x7373,0x8373,0x4463,0x4244,0x4242,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4242,0x4242,0x4244,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4244,0x4442,0x4444,0x4444
.word 0x4442,0x8344,0xC4C4,0xC493,0x8383,0x8383,0xC3C4,0xC3C4,0x6383,0x5242,0x4252,0x5252,0x6363,0x5263,0x5263,0x7F7F,0x7F7F,0x7F7F,0x5241,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x5241,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5241,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC383,0x63C4,0x8383,0xD483,0xC483,0xC483,0x83C3,0x8363,0x8383,0x5273,0xC473,0xB3C3,0x83D4,0x8363,0xC3C3,0xC4C4,0x8363,0x8383,0x83C4,0x6383,0xC4C4,0x83C3,0x6373,0x6352,0x8373,0x83C3,0x7383,0x7363,0x7373,0x4463,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4444,0x4442,0x4242,0x4442,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4442,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x4444,0xC444,0xC4C3,0xC4C3,0xC3C4,0x8383,0xC4C4,0xC4C4,0xC3C3,0x6383,0x4242,0x5252,0x7373,0x5252,0x7F7F,0x4241,0x7F7F,0x4141,0x6242,0x4252,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD462,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC3D4,0x83C3,0x8383,0x8383,0x8373,0x8383,0x7383,0x6363,0x5252,0x7363,0xC463,0xC483,0x83C3,0x8363,0xC4C4,0x83C3,0x8363,0x8373,0x8383,0x5263,0x7373,0x6373,0x7352,0x5273,0x7352,0x8383,0x83C4,0x7383,0x7383,0x7373,0x4443,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4442,0x4244,0x4442,0x4442,0x4444,0x4442,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242
.word 0x4444,0xD444,0xC4C4,0x6383,0x4442,0xC463,0xC3C3,0xC3C4,0xC4C4,0xD483,0xC3C3,0x8383,0x5273,0x4142,0x5241,0x7F7F,0x7F7F,0x7F7F,0x52D4,0x9F9F,0x4141,0x7F7F,0x4141,0x7F7F,0x4142,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x5241,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x5252,0x5252,0x5252,0x5252,0x6262,0x6262,0x62C4,0xC4C4,0x8383,0x8383,0x8383,0x8383,0x7363,0x5252,0x5252,0x7373,0x6352,0xC483,0xC3C3,0xC3D4,0x8383,0x8383,0x6383,0x8363,0x8383,0x5273,0x6373,0x5252,0x8373,0x8363,0x8383,0x8383,0x8363,0x83C3,0x5273,0x7352,0xC483,0x83C4,0x7373,0x7373,0x7373,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4442,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444
.word 0x4444,0xD444,0x83C3,0x4263,0x4444,0xC444,0xC4D4,0xC4C4,0xC4C3,0xC483,0xC4C4,0xC4D4,0x5283,0x7F7F,0x4152,0x7F7F,0x4152,0x5241,0x5252,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xC473,0x6383,0x7383,0x8383,0x5273,0x8363,0x73C4,0x8373,0xB3C3,0xC4C3,0x8373,0xC4C3,0xC4C3,0x7383,0x8383,0x5273,0x5263,0x5252,0x8383,0x8363,0xC3C4,0xC3D4,0x7373,0xC3C4,0xC4C4,0x8383,0xC3C4,0x83C4,0x5252,0x8363,0x83C4,0x8383,0x8383,0x7363,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444,0x4244,0x4244,0x4242,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4244,0x4442,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442
.word 0x4444,0xC444,0x63C3,0x4442,0x4444,0xC444,0xC4C3,0x8393,0x4442,0x8383,0xD3C4,0x5283,0x7F7F,0x5241,0x7F7F,0x4241,0x8F8F,0x5241,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4162,0x4141,0x7F7F,0x7F7F,0x6F6F,0xD4C4,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x7373,0x5252,0x5252,0x6352,0x6383,0xC3C4,0xC3D4,0x8363,0xC4C4,0x83C4,0x8363,0x8383,0x7383,0x7352,0xC483,0x83C3,0x8363,0xC4C4,0xC3C4,0x63C4,0xC483,0xC483,0x63C4,0x8383,0x8383,0x5283,0x7352,0x5263,0x7363,0x5252,0x8363,0x8383,0x7383,0x7363,0x7383,0x4463,0x4444,0x4242,0x4242,0x4442,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4242,0x4242,0x4244,0x4444,0x4244,0x4442,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244
.word 0x4444,0x8344,0x42C4,0x4444,0x4444,0xC444,0x83C3,0x4283,0x4444,0xC383,0x73C4,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x5252,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4241,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0xD452,0xC4D4,0xC4C4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x9F9F,0x5252,0x5252,0x9F9F,0x9F9F,0x5252,0x8363,0xD4C3,0xC4C3,0x83C4,0xC4C4,0x83C3,0x8373,0x8383,0x7383,0x7373,0x7373,0x5263,0xC473,0xC3B3,0xC3B3,0x7383,0x8383,0xC4C4,0x63C3,0x8383,0x8383,0x6383,0x8383,0x8383,0x7363,0xC483,0xC4C4,0x83C4,0x6383,0x6352,0xC483,0x8383,0x6373,0x7383,0x7373,0x4443,0x4442,0x4442,0x4242,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x4444,0x4444,0x4473,0x4444,0x4444,0xC344,0xC4C4,0x4442,0x4444,0x8344,0x7F7F,0x7F7F,0x4152,0x4141,0x7F7F,0x7F7F,0x4141,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x5252,0x4141,0x7F7F,0x7F7F,0xD452,0xC4D4,0xD4C4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xC4D4,0xC462,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xC4D4,0xAFAF,0xAFAF,0x5252,0x7352,0xC483,0x7383,0xC483,0xC4C4,0x7373,0x7373,0x6373,0x5252,0x5252,0x7352,0x5252,0x8363,0xC3C4,0x9373,0xC4C4,0xC4C4,0x6383,0x7363,0x8373,0x5263,0x6363,0x5263,0x8363,0x5263,0x6352,0x6383,0xC483,0xB3C3,0xC3B3,0x83C4,0x5263,0x7352,0xC483,0x7383,0x8373,0x8373,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444
.word 0x4444,0x4444,0x4444,0x4444,0x4444,0x8344,0x8383,0x4442,0x4444,0x4144,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4142,0x7F7F,0x7F7F,0x7F7F,0xC4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xC4D4,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0xD451,0xD4D4,0xD4D4,0x6262,0x6252,0x9F9F,0x7351,0xD4C4,0x6383,0x8383,0x7383,0x5252,0x5252,0x6352,0xC473,0xC4C3,0x6383,0xC3C4,0xC3D4,0xC4B3,0x8383,0x8383,0x8383,0x5273,0x5252,0x5252,0xC463,0xC3C3,0x83C4,0x8363,0xD4C3,0xD4D4,0x63C4,0xC483,0xD4C3,0xC4C3,0x8383,0x8363,0x5263,0x8363,0x83C4,0x7383,0x8373,0x8383,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4442,0x4344,0x4243,0x4442,0x4444,0x4244,0x4244,0x4242,0x4242,0x4442,0x4244,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4283,0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0xD452,0xC4D4,0xC4D4,0xD4D4,0xD4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0x7F7F,0x4141,0x7F7F,0xC4D4,0x4152,0x8F8F,0x4152,0x7F7F,0x4141,0xD4C4,0xD4D4,0x9F9F,0x5252,0xC473,0xC3C3,0x5273,0x5252,0x7352,0xC483,0xC4C3,0xC4C3,0x8373,0xC3C4,0x83C4,0x83C4,0xC483,0x83C4,0x7363,0x8383,0x6373,0x8352,0xD4C4,0xC4D4,0x8383,0xC4C4,0xD4C3,0x8383,0xC4C4,0xC3C3,0x83C4,0x8383,0xC3C4,0x8383,0x7373,0xC483,0x73C4,0x5252,0x8373,0x8383,0x7373,0x8383,0x4383,0x4244,0x4444,0x4242,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4442,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x8343,0x4443,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4473,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0xC4D4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0x7F7F,0x5241,0x62C4,0x7F7F,0xD441,0x52C4,0x7F7F,0x62D4,0x4152,0x7F7F,0xC441,0xD4D4,0x9F9F,0x5252,0x8373,0x83C4,0x7352,0xD4C4,0xC4D4,0x8383,0x8383,0xC4C4,0xC463,0x8383,0x63C4,0x8383,0x8383,0x6383,0x7363,0x5252,0x7352,0xC463,0xC3C3,0xD4C3,0x63C3,0x83C4,0xC4C4,0x8363,0x8383,0xC483,0x8383,0x7373,0x8383,0x7383,0x5263,0x7363,0x7373,0x5252,0x7352,0x7383,0x7373,0x8383,0x7383,0x4444,0x4444,0x4242,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4442,0x4244,0x4244,0x4444,0x4442,0x4244,0x4242,0x4242,0x4344,0xC473,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4242
.word 0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x5252,0xC4D4,0xD4C4,0xC4C4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0x7F7F,0x4152,0x8F8F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0xD4C4,0x52D4,0x4141,0x7F7F,0x5241,0xD4D4,0x6351,0x6363,0x6363,0x8383,0x8383,0x83C3,0x8363,0x8383,0x7383,0x7363,0x8383,0x6363,0x6363,0x5263,0x6352,0xC383,0xC3C4,0xC3D4,0x8363,0xC483,0xC4C4,0x6383,0x8383,0x7383,0x8373,0x7373,0x7383,0x6373,0x5263,0x6352,0x7373,0x8363,0x8383,0x8383,0x8383,0x8383,0xD4D4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x8343,0x63C3,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD452,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x5252,0x4252,0x4241,0x5252,0x4141,0x4141,0x7F7F,0x7F7F,0x4152,0x8F8F,0xD452,0x73D4,0x7352,0xC483,0x7383,0x8383,0x7383,0x6363,0x6363,0x5252,0x5252,0x5252,0xC383,0xD4D4,0xC4C3,0x6383,0x8383,0xC4C4,0x83C4,0x7363,0x8383,0x8383,0x6363,0x7373,0x5273,0x6352,0x6373,0x5263,0x7352,0x8383,0xC483,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x42C4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x8373,0x43C4,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242
.word 0x4244,0x4242,0x4244,0x4444,0x4444,0x4444,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0xC4C4,0xC4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE441,0x7F7F,0x4141,0x5242,0x4152,0x7F7F,0xC4D4,0x9F9F,0x8F8F,0x5252,0x4142,0x7F7F,0x4141,0xC452,0x4141,0x4152,0xD4C4,0x83D4,0x8383,0x6352,0x5263,0x5252,0xC383,0xC4D4,0x6373,0xC4C3,0x83C4,0xC473,0xC483,0xD4C3,0x83C4,0x8383,0x8383,0x6383,0x5273,0x6352,0x5252,0xC483,0xC4C3,0x7383,0xC373,0xD4B3,0x83C3,0x8373,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4242,0x4242,0x4444,0x4442,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x4244,0x4444,0x4444,0x4444,0x4242,0x4244,0x4442,0x4444,0x4444,0x4444,0x4444,0x7344,0xB383,0x4463,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444
.word 0x4444,0x4444,0x4244,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xC442,0xD4D4,0xC4C4,0xC4C4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xE4D4,0x41D4,0xE4D4,0x7F7F,0x4241,0x9F9F,0x5252,0x4141,0xC452,0x5262,0x9F9F,0x9F9F,0x4252,0x4141,0x8F8F,0x4141,0x4141,0x5241,0x7F7F,0xD4D4,0x6383,0x8363,0xC4C4,0x6383,0xC4C4,0xC383,0x5283,0x93C4,0x83C3,0x8363,0x8383,0x8383,0x6383,0x7373,0x6363,0x5252,0xD483,0xC3C3,0x7383,0xC483,0xD3D4,0x83C4,0x8363,0xC4C3,0x83C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4244,0x4244,0x4242,0x4242,0x4444,0x4444,0x4244,0x4242,0x4242,0x4442,0x4244,0x4444,0x4444,0x4244,0x4442,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4244,0x4444,0x4444,0x4442,0x4344,0x4244,0x4442,0x4242,0x4242,0x4442,0x4244,0x4442,0x4444,0x4444,0x8363,0x83C4,0x4444,0x4244,0x4442,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444
.word 0x4444,0x4444,0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5241,0xD4C4,0xC4C4,0xD4C4,0xC4C4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD441,0x7F7F,0x5241,0x4142,0x6241,0x7F7F,0x7F7F,0x9F9F,0xC4D4,0x4152,0x5242,0x9F9F,0x4142,0x4252,0x5241,0x5252,0x9F9F,0x7F7F,0xD4D4,0x83D4,0x6363,0x7373,0x6383,0x8373,0x8383,0x8352,0xC483,0x7383,0x7363,0x7373,0x6373,0x8352,0xD4C4,0xC3C3,0x83C4,0xC473,0xC483,0x83C3,0xC463,0xC483,0xC4C3,0x8373,0x8383,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x8373,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x7343,0x83C4,0x43C4,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244
.word 0x4242,0x4442,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD452,0xC4D4,0xC4C4,0xD4D4,0xC4C4,0xC4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0x41D4,0x41E4,0x5252,0x9F9F,0x7F7F,0x8F8F,0x5262,0x4141,0xD452,0x52C4,0x8F8F,0x5252,0x9F9F,0x4141,0x8F8F,0x9F9F,0x5252,0x4142,0x7F7F,0x83D4,0x5263,0x6363,0x5252,0x7363,0x5273,0x7363,0x7373,0x5273,0x5263,0x5252,0x8373,0x7373,0x93C4,0xC4C4,0xC4C3,0x8363,0xC483,0x83C4,0x8373,0x8383,0x83C4,0x8363,0x6273,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0x5262,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0xC473,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0xC373,0xC493,0x4483,0x4444,0x4444,0x4442,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242
.word 0x4444,0x4444,0x4442,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6241,0xC462,0xC4C4,0xD4D4,0xC4D4,0xC4C4,0xD4C4,0xD4D4,0xD4C4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7F7F,0x8F8F,0x4141,0x5252,0x7F7F,0xC452,0x5262,0x5241,0xC4C4,0x4152,0x5241,0x6252,0x9F9F,0x5252,0x4151,0x9F9F,0x4252,0x5252,0x5241,0x6383,0x8383,0x7373,0x6383,0x7363,0x5252,0x5252,0x6363,0x8383,0xC4C4,0xD4C3,0x5283,0xC483,0x8383,0xC483,0x8373,0x8383,0x6383,0x7373,0x7373,0x5263,0x6252,0xD462,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x7342,0x73C3,0x4444,0x4442,0x4242,0x4442,0x4442,0x4444,0x7344,0xC4C4,0x8383,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xD4C4,0xC4C4,0xC4D4,0xC4C4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62D4,0x7F7F,0x7F7F,0x7F7F,0xD4C4,0xD4D4,0x41C4,0x7F7F,0x4141,0x5241,0x9F9F,0x4152,0x7F7F,0x9F9F,0x7F7F,0x4141,0xC4C4,0x4141,0x9F9F,0x9F9F,0x9F9F,0x5252,0x5252,0x9F9F,0x8F8F,0x4252,0x7352,0x7363,0x7383,0xC473,0x5283,0x8383,0xC483,0x83D4,0xC473,0x8383,0xC3C4,0x5283,0x8383,0x8383,0x8383,0x7363,0x7373,0x5252,0x5252,0x5252,0x7373,0xAFAF,0x62C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x42C4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4442,0x4344,0xC473,0x4463,0x4242,0x4242,0x4242,0x4242,0x4244,0xC443,0x8383,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444
.word 0x4444,0x4444,0x4244,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4C4,0xC4D4,0xC4D4,0xD4C4,0xC4C4,0xC4C4,0xC4C4,0xC4D4,0xD4C4,0xD4D4,0xD462,0xD4D4,0x9F9F,0xD462,0x7F7F,0x7F7F,0xC441,0xD4D4,0xD4D4,0xD4D4,0x4141,0x7F7F,0x8F8F,0x62D4,0x4141,0x4141,0x7F7F,0x4141,0xD452,0xD4E4,0xD4E4,0x7F7F,0xD4D4,0x7F7F,0x9F9F,0x5252,0x5252,0x9F9F,0x9F9F,0x7F7F,0x5252,0x6363,0x6363,0x6383,0x7383,0x8383,0x6383,0x8383,0x8383,0x8383,0x6363,0x7373,0x6373,0x5263,0x5252,0xC483,0x8363,0xC4C4,0x83C4,0x5252,0xC462,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4444,0x4442,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4442,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x7344,0x63B3,0x4444,0x4444,0x4444,0x4444,0x4444,0x8383,0x83C3,0x4483,0x4444,0x4444,0x4444,0x4244,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242
.word 0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD452,0xC4C4,0xC4C4,0xC4C4,0xC4C4,0xC4C4,0xC4D4,0xD4C4,0xD4D4,0xD4D4,0xD462,0xD4D4,0x6241,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x41D4,0x7F7F,0x7F7F,0x5252,0x4141,0x7F7F,0x4241,0x6262,0x5262,0x5241,0x41D4,0x5241,0x7F7F,0x7F7F,0xD441,0xD4D4,0x4141,0x7F7F,0x8F8F,0x9F9F,0x5252,0x4152,0x4241,0x5252,0x6352,0x6352,0x6363,0x6363,0x5263,0x7373,0x7373,0x6373,0x7363,0x5252,0x6352,0x8363,0xD4C4,0xC3D4,0x8363,0x8383,0x5273,0x6252,0xC462,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4244,0x4244,0x4444,0x4242,0x4442,0x4442,0x4242,0x4242,0x4444,0x4442,0x4342,0xC483,0x4443,0x4242,0x4442,0x4444,0x4444,0xC483,0x83C4,0x4473,0x4444,0x4444,0x4444,0x4442,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0x4444,0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xC452,0xC4C4,0xD4C4,0xC4C4,0xC4C4,0xC4D4,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4C4,0x6251,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xC462,0x7F7F,0x7F7F,0x6252,0xD4D4,0x7F7F,0x4141,0x7F7F,0xD4D4,0x9F9F,0xD452,0x7F7F,0xE4D4,0xD4D4,0x52D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4252,0x9F9F,0x9F9F,0x7F7F,0x5241,0x5252,0x5252,0x5252,0x5252,0x6363,0xC483,0xC3D4,0x83C4,0xC452,0xC4C4,0xC4C3,0x8373,0x8383,0x83C4,0x8352,0x5273,0x6252,0xAFAF,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x42C4,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x7343,0x4443,0x4444,0x4444,0x4444,0x4244,0xB373,0x43C4,0x4244,0x4242,0x4242,0x4444,0x8383,0x8383,0x4473,0x4444,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444
.word 0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0xC4D4,0xD4D4,0xD4C4,0xD4C4,0xD4C4,0xC4C4,0xD4C4,0xD4C4,0xD4C4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0x4162,0xD452,0x41D4,0x6241,0x62D4,0x7F7F,0x5242,0x8F8F,0x4141,0x4142,0xC452,0xD4D4,0xD4D4,0xD4E4,0x41E4,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x5242,0x5252,0xC483,0x83C4,0x5283,0x8383,0x8383,0xC483,0x8373,0x83C4,0x8383,0x8363,0x8383,0x6383,0x6363,0x5252,0x9F9F,0xC462,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xC4C4,0x4442,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4442,0x4442,0x6342,0x4373,0x4444,0x4444,0x4442,0x4444,0x8343,0x73C3,0x4444,0x4242,0x4242,0x4442,0xC4C4,0x8383,0x4473,0x4444,0x4444,0x4444,0x4444,0x4344,0x4483,0x4444,0x4244,0x4444,0x4242,0x4242,0x4442,0x4444,0x4442,0x4444,0x4444,0x4444
.word 0x4244,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD452,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x62D4,0xD442,0xD4D4,0x7F7F,0x7F7F,0x4241,0x7F7F,0x9F9F,0xD462,0x62D4,0x7F7F,0x8F8F,0xC4C4,0xD4D4,0xD4E4,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x8F8F,0x5242,0x8F8F,0x7F7F,0x4141,0x6352,0x5242,0x5252,0x6363,0x63C4,0x7373,0x7383,0x6373,0x8352,0x7383,0x6373,0x6363,0x6363,0x6352,0x5273,0x6252,0x6262,0xD4C4,0xD4C4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x7363,0x4443,0x4444,0x4444,0x4444,0x6342,0xC483,0x4463,0x4444,0x4444,0x6344,0xC493,0x8383,0x4483,0x4444,0x4444,0x4444,0x4444,0x8363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4442
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD4C4,0xD462,0xD4C4,0xD4C4,0xD4D4,0xC4D4,0xD4D4,0xD4C4,0xD4C4,0xD4C4,0xC4D4,0xD4D4,0xD462,0xD4D4,0xC4D4,0x9F9F,0xD441,0xD4D4,0xC4D4,0x7F7F,0x7F7F,0x6252,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6262,0xC4C4,0x41C4,0x5252,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x5141,0x4141,0x8F8F,0x4162,0x8F8F,0x5283,0x5252,0x5263,0x5252,0x5252,0x6363,0x5263,0x6363,0x6363,0x6352,0xC483,0xC4D4,0x5283,0x5252,0x6262,0xAFAF,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4244,0x8344,0x4473,0x4444,0x4244,0x4444,0x4444,0xC483,0x4483,0x4442,0x4242,0x7344,0x83D4,0x8383,0x4473,0x4444,0x4444,0x4244,0x6344,0x4383,0x4442,0x4444,0x4444,0x4444,0x4244,0x4442,0x4242,0x4442,0x4444,0x4244,0x4242,0x4244
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0xD4C4,0xC4D4,0xD4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xC4D4,0xD4D4,0xC4D4,0x7F7F,0x62D4,0xD4D4,0x52D4,0x8F8F,0x5241,0xD462,0x62D4,0x8F8F,0x5241,0x7F7F,0x4151,0xD452,0xE4E4,0xD4D4,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x8F8F,0x7F7F,0xE441,0x52D4,0x4252,0x63C4,0x73C4,0x5242,0x5263,0x5252,0x7352,0x6363,0x83C4,0x8383,0x5283,0x8383,0x8383,0x4273,0x9F9F,0xC4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4442,0x4244,0x4242,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x6342,0x73C4,0x4444,0x4442,0x4444,0x4244,0x8363,0x63C4,0x4444,0x4442,0x8344,0x83C3,0x7383,0x4383,0x4444,0x4442,0x4442,0x8343,0x4383,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4244,0x4242
.word 0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5252,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xC4C4,0xD4C4,0xC4C4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD462,0xC4D4,0x41D4,0xC4C4,0xD4D4,0xD4D4,0xD462,0xD4D4,0x7F7F,0x9F9F,0xD4C4,0x41C4,0x5241,0xD462,0x52D4,0x4241,0xD462,0xD4D4,0xE4E4,0x41D4,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x5141,0x4152,0x7F7F,0x5241,0xC4E4,0x9F9F,0x6252,0xC363,0x5283,0x6352,0x7363,0x8383,0x52C4,0x7373,0x7373,0x6373,0x7373,0x7373,0x5252,0x6262,0xC462,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4444,0x4444,0x4244,0x4244,0x4242,0x4442,0x4444,0x4242,0x4442,0x4442,0x4444,0x4244,0x4444,0x4244,0x4244,0x4242,0x4242,0x4242,0x6342,0x8383,0x4443,0x4444,0x4444,0x4244,0x8344,0xC483,0x4444,0x4444,0x8343,0x83C4,0xC483,0x4383,0x4444,0x4444,0x4444,0xC483,0x4473,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4242
.word 0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xC4D4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xC4D4,0xC4D4,0xC4C4,0xD4C4,0xD4C4,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD452,0xD452,0xD4C4,0xD4D4,0xD4D4,0xD4C4,0x4141,0x7F7F,0x7F7F,0x7F7F,0xC452,0xD4D4,0x7F7F,0xC452,0xD4D4,0xE4D4,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x8F8F,0x4141,0x7F7F,0x7F7F,0x52D4,0x9F9F,0x6252,0xC483,0x5263,0x5263,0x7363,0x5263,0x6373,0x5252,0x7352,0xC483,0x52C4,0x9F9F,0xC4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xF4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4244,0xC473,0x4473,0x4444,0x4444,0x4444,0x8342,0x83C4,0x4463,0x4444,0xC473,0xC483,0x8383,0x4383,0x4444,0x4444,0x6344,0xC4C4,0x4473,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4244,0x4444,0x4244,0x4242,0x4242,0x4242
.word 0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x41D4,0x51D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4C4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4C4,0xC4D4,0xC4C4,0xD4D4,0xD4C4,0xD4D4,0x62D4,0x7F7F,0x5252,0xC4C4,0x7F7F,0x7F7F,0x7F7F,0x4152,0x9F9F,0xD4C4,0xE4D4,0xD4E4,0x41D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4241,0x4252,0x4141,0x4141,0x5241,0x62E4,0x8F8F,0x9F9F,0x8362,0x63C3,0x5252,0x5252,0x5252,0x5263,0x6352,0xC483,0x8363,0x5263,0x9F9F,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC4D4,0x4452,0x4244,0x4242,0x4242,0x4442,0x4244,0x4244,0x4442,0x4244,0x4444,0x4244,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x8363,0x6383,0x4444,0x4444,0x4444,0x8342,0x83C4,0x4473,0x4344,0xC483,0x8383,0x8383,0x4383,0x4444,0x4444,0x8344,0x83C4,0x4463,0x4444,0x4242,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4244,0x4244,0x4242
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xC4D4,0x62C4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4C4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0x52D4,0xAFAF,0x62D4,0x7F7F,0xC452,0x41D4,0x7F7F,0xD462,0x4162,0xC452,0x7F7F,0xC441,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x9F9F,0x4152,0x7F7F,0xD4D4,0x4152,0xC4D4,0x52C4,0xC363,0x4283,0x6352,0x8383,0x6383,0x8383,0x8373,0x5263,0x6252,0xC462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52C4,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x6344,0x4463,0x4444,0x4244,0x4444,0x4244,0x8343,0x7383,0x4244,0x4242,0x4244,0xC442,0x8383,0x4483,0x8363,0x73C4,0x83D4,0xC3C4,0x4383,0x4444,0x4444,0xC473,0x83C3,0x4463,0x4444,0x4444,0x4442,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4444,0x4444,0x4244
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5252,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4152,0x5252,0xD4D4,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xC4D4,0xD4D4,0xC4D4,0xD4D4,0xC4D4,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD462,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x5241,0x4262,0x5241,0xD4C4,0x7F7F,0xAFAF,0xD4D4,0x41D4,0x7F7F,0xD441,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x9F9F,0x5252,0x8F8F,0x7F7F,0xE441,0x4152,0xE452,0xD451,0x6341,0x73C4,0x5242,0x7373,0x5283,0x6363,0x6363,0x5252,0x9F9F,0xC462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xF4F4,0xE4E4,0xD4E4,0xD4F4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4442,0x4444,0x4444,0x4444,0x7383,0x4443,0x4444,0x4444,0x4444,0xC444,0x83C4,0x4443,0x4444,0x4244,0xC473,0x83C4,0x4373,0x8363,0x8383,0x8383,0x83C4,0x6373,0x4444,0x4444,0xC483,0x8383,0x4443,0x4444,0x4444,0x4444,0x4244,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4444,0x4244
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4152,0x7F7F,0x5252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x41D4,0xC4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD462,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xE4D4,0xD4E4,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x8F8F,0xD441,0xD441,0x4141,0x8341,0x5263,0x5263,0x5252,0x5252,0x5252,0x5252,0xC462,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x7343,0x73C4,0x4444,0x4244,0x4444,0x8344,0x83C4,0x4243,0x4242,0x4244,0xC483,0x8383,0x8383,0x8383,0xC483,0x83C4,0x83C4,0x8373,0x4444,0x7344,0xC4C4,0x7383,0x4444,0x4444,0x4363,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444
.word 0x7F7F,0x7F7F,0x7F7F,0x5241,0x9F9F,0x5252,0x4152,0x6252,0x5242,0x9F9F,0x7F7F,0x6252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x41D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFAF,0xD4D4,0xD4C4,0xD4C4,0xC4C4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x52D4,0x7F7F,0x5241,0xC462,0x4141,0x5252,0xC4C4,0x4141,0x52D4,0xE4D4,0xE4D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x4241,0x7F7F,0x7F7F,0x4141,0x41D4,0xD441,0x6242,0x7F7F,0xD441,0x4283,0x6352,0x5283,0x7383,0x4252,0x9F9F,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xD4F4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x6344,0x4444,0x4342,0x83C4,0x4463,0x4444,0x4444,0x8344,0x8383,0x4473,0x4444,0x8342,0xC4C4,0x7383,0xC473,0x8383,0x8383,0xC483,0xC483,0x7373,0x4463,0x8343,0xC483,0x6383,0x4444,0x6344,0x4483,0x4444,0x4444,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4242,0x4442
.word 0x7F7F,0x7F7F,0x7F7F,0x6241,0xC452,0x9F9F,0x9F9F,0x9F9F,0xD452,0x52C4,0x7F7F,0xAFAF,0x7F7F,0x7F7F,0x4252,0x4141,0x52D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xC452,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xC4C4,0x9F9F,0xD452,0xD4D4,0x5241,0xD462,0xD4D4,0x7F7F,0x41D4,0xD4D4,0x7F7F,0xC4C4,0x7F7F,0x7F7F,0x41C4,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4142,0x4141,0x4141,0x52E4,0x5241,0x9F9F,0x41D4,0x4142,0x63D4,0x6352,0x6363,0x5263,0x9F9F,0xC462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4F4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4452,0x4444,0x4444,0x4242,0x4242,0x4242,0x4242,0x4442,0x4442,0x4444,0x6344,0x4373,0x4244,0xC373,0x4383,0x4244,0x4244,0x8363,0x8383,0x4473,0x4244,0xC473,0x83C4,0x8383,0x73C3,0x7383,0x83C4,0x8383,0xC483,0x7383,0x4473,0xC473,0x83C4,0x4383,0x6344,0x8383,0x4463,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4242
.word 0x4152,0x4141,0x5241,0xAFAF,0x6262,0x5262,0x9F9F,0xC4C4,0xD462,0x62D4,0x4141,0x62D4,0x7F7F,0x7F7F,0x9F9F,0x4152,0x52D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD462,0xD4C4,0xD4C4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0x41C4,0x6252,0x62D4,0x8F8F,0xC462,0x52D4,0x7F7F,0x7F7F,0x7F7F,0xD4C4,0xD4E4,0x7F7F,0x7F7F,0xC4C4,0x4141,0x7F7F,0x7F7F,0x8F8F,0x4141,0x4141,0x7F7F,0x52E4,0x5241,0x4152,0x4141,0x4141,0xD452,0x5273,0x5252,0x4252,0x6252,0xAFAF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4452,0x4442,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x7373,0x4444,0x8343,0x7383,0x4444,0x4444,0xC473,0x8383,0x4463,0x7342,0xC3C4,0x7383,0xC473,0x8383,0x8383,0xC483,0x8383,0x8383,0xC4C4,0x6373,0xC483,0x8383,0x4383,0x8363,0x7383,0x4444,0x4242,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4244,0x4444,0x4444
.word 0x8F8F,0x4152,0xAFAF,0xC462,0xAFAF,0xD452,0xC462,0xC4D4,0x52D4,0xD4D4,0x5241,0xC4D4,0x7F7F,0x7F7F,0xD4C4,0x52D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x6241,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD462,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6252,0xD4C4,0x5241,0xD4C4,0xE4D4,0x41D4,0x4141,0x7F7F,0xD4D4,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x52E4,0x8F8F,0xD4D4,0xD4D4,0xD4D4,0x41D4,0xD4D4,0x5252,0x5242,0x9F9F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4F4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4244,0x8343,0x4473,0x8342,0x83C4,0x4444,0x7342,0x8383,0x8383,0x4243,0xC483,0x83C4,0x7373,0xC3C4,0x8373,0x8383,0x8383,0x73C4,0x83C4,0x8383,0x8373,0xC3C4,0xC483,0x6383,0xC4C4,0x4383,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242
.word 0x6262,0x8F8F,0x5262,0xC4C4,0xC4C4,0xD452,0xD4D4,0xD4D4,0xD462,0xD4D4,0x5242,0x62D4,0xD441,0x7F7F,0xD462,0x62D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC441,0xD4D4,0xD4C4,0xD4D4,0xC4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x4162,0x4152,0x6252,0xC4D4,0x7F7F,0xD462,0xD441,0x41C4,0x9F9F,0xD4D4,0xC4C4,0x41D4,0x7F7F,0x7F7F,0x62D4,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x52E4,0xD441,0x41D4,0xE4D4,0x4141,0xD4D4,0x62D4,0x52D4,0x9F9F,0xC452,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4452,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4444,0x4444,0x8342,0x63C4,0x8342,0x73C3,0x4443,0x8343,0x83C4,0x83C4,0x7343,0xC383,0x8383,0x8383,0x8383,0x8373,0xC483,0x7383,0x7383,0x83C4,0x8383,0x8373,0x83C4,0x8383,0x8383,0x8383,0x4473,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242
.word 0xAFAF,0x5262,0xC462,0x62C4,0xC4D4,0xC4D4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4162,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0xD462,0xD4D4,0x7F7F,0xD4D4,0x41D4,0xC4C4,0x41D4,0x5241,0x7F7F,0xD4E4,0x7F7F,0x7F7F,0x7F7F,0xD4C4,0x4162,0x7F7F,0x7F7F,0x4241,0x41E4,0x5241,0x41D4,0xD462,0x4141,0x4141,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xF4D4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x44C4,0x4444,0x4244,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244,0x4442,0x7342,0x83C4,0x8363,0x7383,0x4373,0x8373,0x8383,0x7383,0xC483,0x83B3,0x8383,0x8383,0x83C4,0x8383,0x8373,0x8383,0x8383,0xC483,0x83C3,0xC483,0x8383,0x8383,0x8383,0x7383,0x4444,0x4444,0x4444,0x4444,0x4242,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244
.word 0x6262,0x9F9F,0xC4C4,0xD4D4,0xC4C4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0x62D4,0x42D4,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4D4,0xE4E4,0xE4F4,0xE4E4,0xF4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0x41C4,0xD462,0xD4D4,0xD4D4,0xC4D4,0xD4C4,0xC4C4,0xD4D4,0xD4D4,0x7F7F,0xC452,0xD4D4,0x7F7F,0xD4C4,0x51D4,0x7F7F,0x6262,0x41C4,0x7F7F,0xD452,0xC4D4,0x7F7F,0x7F7F,0x7F7F,0xD4C4,0x4162,0x7F7F,0xD442,0x41D4,0x6241,0x8F8F,0xD452,0x4141,0x4141,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4442,0x4444,0x4444,0x4244,0x4444,0x4244,0x4442,0x7342,0x83C4,0x8383,0x7383,0x7363,0xC483,0x8383,0x7383,0xC383,0x8383,0x8383,0x8383,0x83C4,0x8383,0x8383,0x8383,0x8383,0xC483,0x8383,0x8383,0x83C4,0x8373,0xC4C4,0x6383,0x4442,0x4444,0x4344,0x4443,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244
.word 0xAFAF,0xC4C4,0xC4C4,0xC4C4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4C4,0xD4D4,0xC4D4,0x4142,0xD462,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xC4D4,0x62D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4152,0x8F8F,0x7F7F,0x9F9F,0x6262,0x6262,0x62C4,0xC4C4,0xC4C4,0xD4C4,0xD4D4,0xD4D4,0x4141,0x7F7F,0x41D4,0xE441,0x4152,0xE441,0x4141,0x4141,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4242,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x8342,0x8383,0x8383,0x7383,0x8363,0xC4C4,0x7383,0x8383,0x83C4,0x8383,0x8383,0x83C4,0x83C4,0x8373,0x83C4,0x8373,0x8383,0x8383,0x7383,0x8383,0x83C4,0x8383,0x8383,0x4483,0x6344,0x8373,0x4463,0x4442,0x4444,0x4442,0x4244,0x4242,0x4442,0x4244,0x4244,0x4242
.word 0xC4C4,0xC4C4,0xC4C4,0xD4C4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xE4E4,0xF4E4,0xE4F4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0x4162,0xD462,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x41C4,0xC452,0x7F7F,0x6252,0xD4D4,0x5241,0xC4C4,0x5252,0x8F8F,0x8F8F,0x4141,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x5242,0x4152,0xE441,0x7F7F,0x4141,0xC441,0x52D4,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x7344,0x4444,0x4444,0x8342,0x8383,0xC483,0x7383,0x8373,0x83C3,0x8383,0xC473,0x83C4,0xC473,0x83C4,0x8383,0x7383,0x8363,0x8383,0x8383,0x8383,0x83C4,0x7383,0x83C4,0x83C4,0x8383,0x7383,0x4463,0x7363,0x63C4,0x4444,0x4244,0x4444,0x4244,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444
.word 0xC4C4,0xC4C4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xF4E4,0xE4E4,0xF4F4,0xF4F4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0x4162,0xD462,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0x5252,0x7F7F,0x9F9F,0xC4D4,0x9F9F,0xD4C4,0x9F9F,0x4152,0x7F7F,0x5241,0x7F7F,0x6241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD452,0x8F8F,0xE441,0x7F7F,0x7F7F,0x7F7F,0x52C4,0x8F8F,0xD4D4,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4F4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4446,0x4444,0x4444,0x4244,0x4442,0x4242,0x7342,0x4483,0x4444,0xC463,0x73C3,0x8383,0x8373,0x8383,0x8383,0x8383,0xC483,0x8383,0x8383,0x8383,0x8383,0x7383,0x7373,0x8383,0x7383,0x8373,0xC383,0x8383,0x8383,0x8383,0x8383,0x7383,0x7363,0x73C4,0x4243,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x4242
.word 0xC4C4,0xC4D4,0xD4D4,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xE4E4,0xE4E4,0xF4F4,0xF4E4,0xF4E4,0xF4F4,0xE4E4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xC4D4,0xD4D4,0xD4D4,0x62D4,0x6241,0xD4D4,0xD4D4,0xC4C4,0x52D4,0x7F7F,0x5242,0x51D4,0x4141,0x7F7F,0x7F7F,0x4241,0x7F7F,0xD452,0x7F7F,0xD462,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6262,0x7F7F,0xC4D4,0x62E4,0x4141,0x4141,0x4141,0xD4D4,0x52D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4244,0x4242,0x4442,0x4444,0x4444,0x4242,0x4342,0x8383,0x4244,0x8373,0x83B3,0x8383,0x7373,0xC4C4,0x8383,0x7373,0x83C3,0x7373,0x8383,0x7383,0x8383,0x6383,0x6363,0x7373,0x8383,0x8373,0x8373,0xC473,0x8383,0x7383,0xC483,0x6383,0x8373,0x4383,0x4442,0x4442,0x4244,0x4444,0x4444,0x4444,0x4242,0x4244,0x4444,0x4444,0x4442
.word 0xC4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xE4E4,0xE4E4,0xF4F4,0xE4E4,0xE4E4,0xF4F4,0xF4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x4162,0xD462,0xD4D4,0x52D4,0x8F8F,0xC462,0x7F7F,0xC452,0x4141,0xD452,0x4162,0xD4C4,0x41D4,0x7F7F,0xD441,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x6262,0x62E4,0x41E4,0x7F7F,0x4141,0x4141,0xD4D4,0x9F9F,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xE4D4,0xF4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4242,0x4442,0x4442,0x4473,0x4444,0x4244,0xC483,0x7383,0xC473,0x8383,0x83C4,0x8383,0xC4B3,0x83C4,0x8383,0xC4C4,0x8383,0x7383,0x7383,0x8383,0x7383,0x7363,0x8363,0x7373,0x8383,0x6383,0x8383,0x8383,0xC473,0x83C3,0x8363,0xC4C4,0x4263,0x4444,0x4444,0x4344,0x6373,0x4444,0x4244,0x4444,0x4444,0x4444,0x4244,0x4242
.word 0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4F4,0xF4E4,0xF4E4,0xF4F4,0xF4E4,0xF4F4,0xF4E4,0xE4F4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4141,0xD462,0x5252,0xD4D4,0x5241,0xD4C4,0x8F8F,0xD452,0x7F7F,0xD452,0x7F7F,0x5241,0x7F7F,0x7F7F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x62E4,0x7F7F,0x4162,0x4141,0x4141,0xD4E4,0x52D4,0xC452,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4444,0x4442,0x7344,0x73B3,0x4244,0x8373,0x8383,0xC473,0xC483,0x73C4,0x8373,0x83C4,0x83C3,0x8383,0x8383,0x8383,0x8383,0x7383,0x83C4,0x7373,0x7363,0x6363,0x7383,0x7383,0x8373,0xC4C4,0x8383,0x8383,0x7383,0x8383,0x8383,0x4444,0x4444,0x6344,0x8373,0x4443,0x4244,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4F4,0xE4F4,0xD4F4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xD4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x5241,0x6252,0xD4D4,0x4141,0xC462,0x4152,0x7F7F,0xC4D4,0x7F7F,0xD4D4,0x7F7F,0x8F8F,0x4162,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6262,0x62E4,0x8F8F,0xD441,0x42D4,0xE4D4,0xE4E4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4446,0x4244,0x4444,0x4444,0x4244,0xC443,0xC483,0x8342,0x83B3,0xC483,0xC473,0x83C3,0x8373,0x8383,0x8383,0xC483,0x83C4,0x8373,0x8373,0x7383,0x8383,0x7383,0x6373,0x7373,0x7363,0x8373,0x8383,0x7383,0x8373,0x8383,0x7373,0xC473,0x73C4,0x4442,0x6344,0x8383,0x4463,0x4244,0x4444,0x4244,0x4244,0x4442,0x4442,0x4444,0x4444
.word 0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0xC441,0xD4D4,0x7F7F,0x51C4,0x7F7F,0x7F7F,0x6262,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0x4141,0xE441,0x41D4,0x4141,0x4141,0xD441,0x52D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4244,0x4442,0x4242,0x4442,0x4444,0x4342,0x83C4,0x73C4,0x83C4,0xC483,0x8373,0x7383,0x8383,0x8383,0x8383,0xB383,0x8383,0x8373,0x7383,0x73C3,0x8373,0x7373,0x6373,0x8363,0x8373,0x8383,0x8383,0x7373,0x8383,0x83C4,0x7383,0x8383,0x4383,0x4344,0x8373,0x6383,0x4444,0x4442,0x4244,0x4244,0x4242,0x4444,0x4444,0x4244,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4F4,0xE4E4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xC4C4,0xC4C4,0xD4D4,0xC4C4,0xD4D4,0x52D4,0xD441,0x41C4,0x4152,0x7F7F,0x7F7F,0xD4D4,0x7F7F,0x7F7F,0xD462,0x4162,0xD4D4,0x5241,0x41D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6241,0xE4D4,0x41D4,0x4141,0x8F8F,0x4141,0x4141,0xD441,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xF4E4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4244,0x4244,0x4442,0x4444,0x7342,0x8383,0xC483,0xC473,0xC473,0x8383,0x8383,0x8383,0x8383,0xC483,0x8383,0x8383,0xC4C4,0x8383,0x7373,0xC473,0x7373,0x83C4,0x73C4,0x8383,0x7383,0x8363,0x8383,0x8383,0x7383,0xC3C4,0x4373,0x8373,0x7383,0x4243,0x4444,0x4444,0x4442,0x4442,0x4444,0x4444,0x4242,0x4242,0x4242
.word 0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x5262,0x4141,0x5141,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0xC4C4,0xC4D4,0xC4C4,0xC4D4,0x7F7F,0x7F7F,0xD462,0x52D4,0x7F7F,0x7F7F,0x7F7F,0x5241,0xD462,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6252,0x7F7F,0x41D4,0x7F7F,0x7F7F,0x41E4,0x7F7F,0x4141,0x4141,0x8F8F,0x4141,0x4141,0xD462,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4444,0x4444,0x4444,0x4244,0x4442,0x4242,0x4342,0x7383,0xC473,0xC373,0x8383,0x8383,0x7383,0x8383,0x8383,0x8383,0x83C4,0x8383,0x83C4,0x8383,0x8383,0xC483,0x7373,0x7373,0x73C4,0x7383,0x7373,0x8383,0x8383,0x8373,0x8373,0x8383,0x7373,0xC483,0x4373,0x4442,0x4244,0x4442,0x4442,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4152,0x4241,0x4251,0x5242,0x4252,0x4241,0xC451,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0xC4C4,0x41D4,0xC4D4,0x4162,0x6241,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5241,0x62D4,0xD452,0x7F7F,0x6262,0x52C4,0x4141,0x7F7F,0x4141,0x7F7F,0x6262,0xD452,0x52D4,0x7F7F,0x7F7F,0x8F8F,0xD4C4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xE4F4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4362,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4244,0x8363,0x8383,0x8383,0x7383,0x8383,0x7383,0x8383,0x8383,0xC483,0x83B3,0x73C4,0x8383,0x8383,0xC4C4,0x73C4,0x8363,0x7373,0x6383,0x8373,0x8373,0x7383,0x8373,0x8383,0x8383,0x8383,0xC483,0x83C3,0x4263,0x4244,0x4242,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4442,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xC4D4,0xC4D4,0x8F8F,0x4252,0x5252,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5141,0xD4C4,0xC4C4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5252,0xD4C4,0xC4C4,0x7F7F,0x5241,0x7F7F,0xC452,0x52D4,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0xD452,0x7F7F,0xE462,0x41D4,0x7F7F,0x7F7F,0x5241,0x7F7F,0x7F7F,0xD4C4,0x7F7F,0xD441,0x4141,0x52C4,0xD4E4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4362,0x4444,0x4244,0x4444,0x4242,0x4242,0x4442,0x4244,0x8344,0x8383,0x8383,0x8383,0x8383,0x8373,0x8383,0x8383,0xC483,0x8383,0x8383,0x8383,0x7373,0x8373,0x6373,0x8373,0x83C3,0x7373,0x7373,0x7383,0x7373,0x8383,0x73C4,0xC4B3,0x7383,0x8383,0x6383,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4242,0x4242,0x4242,0x4242,0x4244
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xC4D4,0x8F8F,0x6252,0x9F9F,0xAFAF,0x6262,0x5262,0x5252,0x5252,0x4242,0x4141,0xD462,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0xD452,0xC4D4,0xD4D4,0xC4C4,0x7F7F,0x41C4,0x8F8F,0xD4D4,0x7F7F,0x6241,0x7F7F,0xE4D4,0x4162,0x8F8F,0xD462,0x5241,0x7F7F,0xE4D4,0xD4D4,0x5241,0xD462,0xD4E4,0x4162,0x7F7F,0x5252,0xD4E4,0x7F7F,0x7F7F,0x4141,0x9F9F,0xD4C4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xD4F4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x42C4,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x7342,0x73C3,0x8383,0x7383,0x8373,0x7383,0x8373,0x8383,0x8373,0x8383,0xC4C4,0x8383,0x7383,0x6373,0x7363,0x8383,0x7373,0x83C3,0x7373,0x7383,0x8383,0x83C4,0x8373,0xC483,0x7383,0xC483,0x4473,0x4444,0x6363,0x4463,0x4444,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x9F9F,0xAFAF,0xD462,0xC4C4,0xAFAF,0x9F9F,0xAFAF,0x9F9F,0x8F8F,0x8F8F,0x5142,0xD4C4,0xC4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x41D4,0x9F9F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0x4141,0x7F7F,0x4141,0xD442,0xD4D4,0x41D4,0x7F7F,0x9F9F,0x6241,0x4162,0x7F7F,0x4162,0x4141,0xC441,0x62E4,0x4252,0x4251,0xD441,0x7F7F,0x41C4,0x7F7F,0x7F7F,0x4141,0x4141,0x62C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xF4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4462,0x4244,0x4442,0x4444,0x4444,0x4244,0x4442,0x4442,0x7342,0x73C4,0x8383,0x8383,0x8383,0x8383,0x8373,0xC483,0x7383,0xC4C4,0x83C4,0x8383,0x8383,0x6373,0x8363,0x8373,0x73C3,0x7383,0x7383,0x8373,0x8383,0x7383,0x8383,0x83B3,0x8373,0x8383,0x4263,0x7343,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0xC4C4,0xD462,0xC4D4,0xC4C4,0x6262,0x62C4,0x6262,0x5252,0x5252,0x5252,0x4141,0x8F8F,0x4142,0xD462,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x5252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x7F7F,0x6252,0x52D4,0x7F7F,0x7F7F,0x7F7F,0xD452,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0xC462,0x62E4,0x4252,0x7F7F,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6262,0xD441,0xD4D4,0x62C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xF4D4,0xE4E4,0xF4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xC4D4,0xD462,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4662,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4444,0x6342,0x8383,0x83C4,0x7383,0x7383,0x8383,0x7373,0x8383,0x7383,0x7373,0xC483,0x83C4,0x7373,0x6363,0x8373,0x83C4,0x7383,0x7383,0x7373,0x8383,0x8383,0x8383,0xC3C4,0x7383,0x8383,0x4383,0x7363,0x83C4,0x4243,0x4444,0x4244,0x4242,0x4444,0x4444,0x4444,0x4442,0x4242,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xC4D4,0xAFAF,0xC4C4,0xC4C4,0xD4D4,0xC4C4,0xC4C4,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0x4252,0x4151,0x8F8F,0x4142,0xD4C4,0xD4C4,0xD4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x9F9F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x7F7F,0xD452,0x7F7F,0x5241,0x7F7F,0x6242,0x52D4,0x7F7F,0xD452,0x7F7F,0x7F7F,0xE4D4,0x52D4,0x8F8F,0x7F7F,0x5262,0x7F7F,0x7F7F,0xE441,0x7F7F,0x7F7F,0xD4D4,0x4141,0xD4D4,0xC4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0x41D4,0xD462,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4462,0x4444,0x4442,0x4242,0x4444,0x4444,0x4244,0x4244,0x6342,0x8383,0x8373,0x8383,0x8383,0x8383,0x6383,0x8383,0x83C4,0x8383,0x8363,0x8383,0x6373,0x6363,0x8383,0x8373,0x7373,0x8383,0x8373,0x8373,0x8383,0xC373,0x8383,0x8383,0x8383,0x6373,0x8383,0x4383,0x4442,0x4444,0x4444,0x4244,0x4442,0x4444,0x4444,0x4444,0x4242,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4C4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xC462,0xC462,0x6262,0x5262,0x5252,0x5252,0x9F9F,0x4141,0x4251,0x4152,0x5241,0xD4C4,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0x52D4,0x5152,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4141,0x6262,0x7F7F,0x4152,0x8F8F,0xD4D4,0x7F7F,0xD441,0x8F8F,0xD452,0x7F7F,0xC4D4,0xD462,0xD4C4,0xC4D4,0x5252,0x8F8F,0x8F8F,0x6241,0xD4C4,0xD4D4,0x6262,0xD4D4,0x4241,0xD451,0xD452,0xC4D4,0xD462,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x44C4,0x4244,0x4444,0x4444,0x4444,0x4244,0x4442,0x4444,0x4244,0x8383,0x8383,0x73C4,0x7383,0x8373,0x83C4,0x7373,0x8383,0x7383,0x8363,0x8383,0x7383,0x6363,0x8373,0x7363,0x7373,0x73C4,0x8383,0x7383,0x8373,0xC4C3,0x8383,0xC4C3,0x7383,0xC483,0x73C4,0x4343,0x7363,0x4443,0x4444,0x4444,0x4444,0x4242,0x4244,0x4242,0x4444,0x4244
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xC4C4,0x62D4,0xAFAF,0x9F9F,0x9F9F,0x5252,0x8F8F,0x5242,0x8F8F,0x9F9F,0x5251,0xD4C4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x9F9F,0xD441,0xD4D4,0xD4D4,0xC4D4,0x6241,0x5262,0xAFAF,0x4162,0x7F7F,0x5241,0x5262,0x7F7F,0xD442,0x7F7F,0x7F7F,0x52C4,0x6252,0x7F7F,0x4141,0xD4C4,0x9F9F,0x5262,0x4142,0xD4C4,0xD4D4,0x7F7F,0xD4D4,0x7F7F,0x4141,0x52E4,0xD4D4,0xD4D4,0xC4D4,0xD4C4,0xD4E4,0xD4D4,0xE4D4,0xF4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x6241,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4462,0x4444,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4244,0xC463,0x8383,0x8383,0x8383,0x8373,0xC483,0x7383,0x8373,0xC483,0x8373,0x6373,0x6373,0x8373,0x8383,0x8373,0x7383,0x8383,0x7383,0x7373,0x8383,0x8383,0xB383,0x83C4,0x8383,0xC483,0x6383,0x8383,0x4373,0x4442,0x4244,0x4442,0x4444,0x4444,0x4444,0x4442,0x4242,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0xC4C4,0x62C4,0x5252,0x5252,0x9F9F,0x5241,0x8F8F,0x5252,0x5252,0x9F9F,0x5241,0xD4C4,0xC4D4,0xC4D4,0xD4D4,0x41D4,0x5252,0x9F9F,0xD4D4,0xD4D4,0x41D4,0x9F9F,0xC452,0x4152,0x5241,0xD4C4,0xD441,0xE4E4,0xD4D4,0x41D4,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x7F7F,0x7F7F,0xD462,0x41D4,0x7F7F,0x9F9F,0x5252,0x7F7F,0xE462,0x4162,0xD452,0xD4D4,0xD4D4,0xD441,0xC441,0x7F7F,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xF4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD462,0xD4D4,0xF4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4662,0x4244,0x4242,0x4242,0x4442,0x4444,0x4242,0x4242,0x4444,0x7344,0xC483,0x8383,0x8383,0x7383,0x8383,0x83C3,0x7373,0x7373,0x8383,0x7383,0x6363,0x7373,0x8373,0x7383,0x6363,0x7383,0x7373,0x8373,0xC483,0xC473,0x83C4,0x8383,0xC483,0x7383,0xC483,0x6373,0x4444,0x4444,0x4444,0x4242,0x4442,0x4442,0x4444,0x4444,0x4244,0x4244
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC462,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x4152,0x8F8F,0x9F9F,0x5252,0x5252,0x8F8F,0x6251,0xD4D4,0xD4D4,0xD4C4,0xC4D4,0x9F9F,0x4152,0xD4C4,0xD4D4,0x62D4,0x5262,0x52E4,0x8F8F,0x4141,0x5241,0x52E4,0x7F7F,0xD452,0xE4D4,0xD4E4,0x7F7F,0x52C4,0x5252,0x7F7F,0xD462,0xD4D4,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD462,0xE4D4,0xD4D4,0x7F7F,0xD462,0x7F7F,0x6262,0x7F7F,0xC4C4,0xD4E4,0xD4D4,0xE4E4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x41C4,0xD462,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4462,0x4444,0x4442,0x4442,0x4244,0x4242,0x4244,0x4444,0x4444,0x6342,0x83C4,0x8383,0x73C4,0x8383,0x7373,0xC483,0x6383,0x7363,0x7383,0x7373,0x7363,0x8383,0x6383,0xC483,0x6383,0x7383,0x7373,0x8383,0x8383,0xC3C4,0x83C4,0x8383,0x8383,0x8383,0x83C4,0x4263,0x4244,0x4242,0x4442,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x4244
.word 0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x6262,0x9F9F,0x5252,0x9F9F,0x5252,0x5252,0x9F9F,0x9F9F,0x5252,0x4142,0x4142,0xD4C4,0xD4D4,0xD4D4,0x5252,0x9F9F,0xC441,0xD4D4,0x9F9F,0x9F9F,0x9F9F,0x4141,0x4141,0x52D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xE4D4,0xD4E4,0x52D4,0x7F7F,0xD4D4,0x41D4,0x7F7F,0x41D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x62D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xC452,0xC462,0xD4E4,0xE4D4,0xD4F4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0xD4D4,0xE4D4,0xF4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x44C4,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4442,0x4242,0x4242,0xC463,0xC483,0xC473,0x8373,0x7383,0x8363,0x8383,0x7383,0x7383,0x7383,0x6363,0x7383,0x8383,0x7383,0x8373,0x7383,0x8383,0x7383,0x8383,0x8383,0x8383,0x8383,0x8373,0x8383,0x6383,0x4442,0x4444,0x4244,0x4444,0x4442,0x4444,0x4444,0x4242,0x4244,0x4442,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x5252,0x9F9F,0x9F9F,0x5252,0x5252,0x5252,0x8F8F,0x4141,0x8F8F,0xC4D4,0xD4D4,0x8F8F,0x5252,0x4152,0xD4C4,0x41D4,0x5262,0xC441,0x4152,0xD441,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0xD452,0xD4D4,0x7F7F,0x7F7F,0xD4D4,0xE441,0x7F7F,0x7F7F,0xD441,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6242,0x8F8F,0x9F9F,0x62C4,0xD4C4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x8F8F,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4462,0x4244,0x4242,0x4444,0x4242,0x4242,0x4242,0x4244,0x4442,0x4244,0x8343,0x8383,0x73C4,0x73C4,0x8383,0x7373,0x7373,0x7383,0x8373,0x7373,0x6373,0xC483,0x8383,0x6363,0x8363,0x8373,0x7373,0x7383,0x8383,0x8383,0x83C4,0x8383,0x8383,0x83C4,0x4263,0x4244,0x4242,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4242,0x4244,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x9F9F,0x5252,0x9F9F,0x4252,0x5252,0x9F9F,0x9F9F,0x8F8F,0x5241,0x8F8F,0x7F7F,0x5141,0xD4C4,0xD4C4,0x4241,0x4251,0x5251,0xC4D4,0x9F9F,0x9F9F,0x52C4,0xD441,0x4152,0x4141,0x4141,0x7F7F,0x4241,0x5252,0xD4C4,0xD4D4,0x41D4,0xE462,0xD4D4,0xD4D4,0x6262,0x5241,0x6241,0x62E4,0x7F7F,0x7F7F,0x6241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0xC462,0xE462,0xD4D4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C4,0x6242,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xAFAF,0x4442,0x4244,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4444,0x4342,0xC483,0x8373,0x7363,0xC4C4,0x6383,0x8383,0x8383,0x6383,0x7363,0x8363,0x7363,0x7383,0x6383,0x8383,0x8373,0x8373,0x7383,0xC483,0x83C4,0x8383,0x7383,0x83C4,0x6383,0x4442,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4442,0x4242,0x4444,0x4444,0x4242
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x9F9F,0x6262,0xAFAF,0x5252,0x8F8F,0x9F9F,0x5252,0x5252,0x5252,0x5242,0x4141,0x4141,0x7F7F,0x6241,0xD4D4,0x41C4,0x9F9F,0x5152,0xD441,0x5252,0x5252,0x8F8F,0xE441,0x4252,0x5252,0x5252,0x5252,0xAFAF,0xD4C4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0xE4D4,0x41D4,0xD4D4,0xD4E4,0xD4D4,0x6241,0x41D4,0x4242,0x7F7F,0x62D4,0x41D4,0x7F7F,0x7F7F,0xC4C4,0x6262,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xC4D4,0xD462,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4462,0x4444,0x4444,0x4242,0x4442,0x4444,0x4442,0x4242,0x4244,0x4444,0x4244,0x8363,0x73C4,0x8383,0x8363,0x8383,0x83C4,0x8383,0x7383,0x6363,0x8383,0x73C4,0x7383,0x7363,0x7373,0x7383,0x7373,0xC483,0xC3C4,0x6383,0x8373,0x8383,0x8373,0x4373,0x4442,0x4444,0x4444,0x4244,0x4242,0x4444,0x4444,0x4244,0x4244,0x4242,0x4244,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4C4,0xAFAF,0x5262,0x9F9F,0x5252,0x5242,0x9F9F,0x9F9F,0x9F9F,0x8F8F,0x4141,0x4141,0x4141,0x6F6F,0x7F7F,0xD4D4,0x5252,0x8F8F,0x5142,0x4142,0x4241,0x4252,0x5242,0x9F9F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x5262,0x5252,0x4141,0x4141,0x7F7F,0x5241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0x7F7F,0x7F7F,0xD441,0x7F7F,0xC441,0x62C4,0xD4C4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8F8F,0xD462,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x44C4,0x4444,0x4444,0x4444,0x4242,0x4442,0x4444,0x4444,0x4242,0x4242,0x4442,0x4342,0x8363,0x7383,0x7373,0xC483,0x8383,0x7373,0x6363,0x8363,0x83C4,0x6373,0x6363,0x8363,0x7373,0x8373,0x8383,0x83C4,0x8373,0xC4C4,0x8373,0x8383,0x6383,0x7342,0x7383,0x7373,0x4363,0x4444,0x4444,0x4242,0x4444,0x4444,0x4444,0x4444,0x4242,0x4244
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x62C4,0x6262,0xAFAF,0x5262,0x9F9F,0x9F9F,0x5252,0x5252,0x5252,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6252,0x52D4,0x4141,0x4141,0x8F8F,0x4142,0x8F8F,0x4152,0x52D4,0x6262,0xD462,0xD4D4,0xD4D4,0x62D4,0x6262,0x5152,0x5152,0x8F8F,0x4141,0x7F7F,0x7F7F,0x5241,0x9F9F,0x6252,0x6262,0x6262,0x6262,0x6262,0x7F7F,0x41C4,0xD4D4,0xD4D4,0x41D4,0xC4D4,0x7F7F,0x41C4,0x7F7F,0xC4D4,0x9F9F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5262,0xD462,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4462,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4244,0x4444,0x4444,0x4244,0x4244,0x6344,0x8373,0x7373,0x7383,0x8383,0x6383,0x6363,0x7383,0x7373,0x8373,0x7383,0x8373,0x7373,0x8383,0x8383,0x8373,0x83C4,0x8383,0x8383,0x7383,0x7383,0x8383,0x4363,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFAF,0xAFAF,0x9F9F,0x5252,0x5252,0x9F9F,0x9F9F,0x8F8F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4242,0x6252,0xC4D4,0x5241,0x4141,0x4141,0x5242,0x8F8F,0xD441,0x8F8F,0x4141,0x4142,0x4142,0x4141,0x8F8F,0x8F8F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x5241,0x52E4,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5241,0x5252,0xD4E4,0xD4E4,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0x7F7F,0xC441,0x6252,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0xAFAF,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFAF,0x4444,0x4244,0x4442,0x4444,0x4444,0x4244,0x4244,0x4242,0x4242,0x4444,0x4344,0x4244,0x6343,0x7373,0x7383,0x6363,0x6373,0x6373,0x8383,0x8363,0x7383,0x6373,0x8383,0xC383,0xC4B3,0x8383,0x7383,0x8373,0x8383,0x8383,0x8373,0x8383,0x6363,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4442,0x4242,0x4242,0x4442,0x4242,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x62C4,0x9F9F,0x5262,0x9F9F,0x9F9F,0x5252,0x5252,0x5252,0x4141,0x4141,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4442,0x4242,0xD452,0x52D4,0x5241,0x9F9F,0xD4D4,0xD4D4,0x4141,0x4142,0x7F7F,0x4141,0x8F8F,0x4141,0x4141,0x4141,0x8F8F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x5241,0x52E4,0x7F7F,0x4141,0x4141,0x7F7F,0xD462,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0xC4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x41D4,0xD4D4,0xC4D4,0x6262,0xD4D4,0xD4D4,0xD4C4,0x6252,0xD4C4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4462,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4242,0x8344,0x4463,0x4244,0x6343,0x7383,0x7363,0x6363,0x6373,0x8373,0x6373,0x8373,0x8363,0x8373,0xC4C4,0xC4C4,0x8383,0x8373,0xC483,0x7383,0x7373,0x8383,0x5273,0x4243,0x4444,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC462,0x9F9F,0x5252,0x5252,0x9F9F,0x9F9F,0x8F8F,0x4142,0x4141,0x4141,0x6F6F,0x7F7F,0x7F7F,0x4242,0x4242,0x4242,0xC452,0xD4D4,0x4141,0xC452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52C4,0xD4D4,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4141,0xE452,0x41D4,0x41D4,0x7F7F,0x7F7F,0x4141,0x7F7F,0xE4D4,0xD4D4,0xD4C4,0x7F7F,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xD4C4,0xD4D4,0xD4D4,0x8F8F,0xD4C4,0xD4D4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x44C4,0x4244,0x4442,0x4444,0x4444,0x4444,0x4242,0x4244,0x4242,0x4444,0x4444,0x8342,0x7383,0x4443,0x8363,0x7373,0x6363,0x6363,0x8373,0x7373,0xC383,0x6383,0x7373,0x6373,0x8363,0x8383,0xC483,0x83C4,0x7383,0x8383,0x8383,0x6363,0x4244,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4244,0x4244,0x4244,0x4442,0x4444,0x4444,0x4444
.word 0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x9F9F,0x6262,0x9F9F,0x9F9F,0x5252,0x5252,0x5252,0x8F8F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x4242,0x4242,0x4242,0x4242,0x4242,0xC452,0xD4D4,0xD4D4,0xC4D4,0x52D4,0x5241,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0xE452,0x4152,0x7F7F,0x4141,0x4141,0x7F7F,0x5241,0xD4E4,0xE4E4,0x7F7F,0x7F7F,0x7F7F,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x7F7F,0xD451,0xD4D4,0x62C4,0x62C4,0x7F7F,0x62C4,0xC452,0xD4C4,0xD4D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x9F9F,0x4462,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x8342,0x8383,0x8383,0x7383,0x6373,0x6352,0x8383,0x6363,0x7373,0x8373,0x6383,0x7383,0x7373,0x7383,0x8383,0x8383,0x8373,0x83C4,0xC483,0x5273,0x4243,0x4444,0x4444,0x4442,0x4442,0x4242,0x4242,0x4442,0x4242,0x4242,0x4244,0x4242,0x4442,0x4442,0x4444
.word 0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xC462,0xAFAF,0x5262,0x5252,0x9F9F,0x9F9F,0x5142,0x5241,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4242,0x4244,0x4444,0x4242,0x4442,0x4242,0x4142,0xC452,0x4152,0x5241,0xD4D4,0x52D4,0x8F8F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0xC452,0xD4E4,0x4152,0x7F7F,0x7F7F,0x7F7F,0x4141,0xE452,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0x41E4,0xD4D4,0xD4D4,0x7F7F,0xD4D4,0xC4D4,0x41C4,0xC441,0xAFAF,0xD4C4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4662,0x4444,0x4242,0x4242,0x4244,0x4242,0x4442,0x4444,0x4444,0x4443,0x4444,0x6342,0x83C4,0x8373,0x8383,0x6363,0x7373,0x8373,0x6373,0x7383,0x6363,0x8363,0x7383,0x8373,0x8383,0x8383,0x83C4,0xC4C4,0x8393,0x6373,0x4444,0x4444,0x4242,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4442
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0x9F9F,0x9F9F,0x5252,0x5252,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x6F6F,0x4242,0x4242,0x4242,0x4244,0x4242,0x4244,0x4442,0x5241,0xC4D4,0x8F8F,0xC442,0xD4D4,0x41C4,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x9F9F,0xD4D4,0x51C4,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x52D4,0xC4D4,0xD4D4,0xD4D4,0x7F7F,0x7F7F,0x6252,0xD4C4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x4452,0x4244,0x4444,0x4444,0x4242,0x4242,0x4242,0x4244,0x4444,0x4373,0x4444,0x4244,0xC483,0x7383,0x7373,0x5263,0x8373,0x7373,0x8373,0x7363,0x8383,0x7363,0x8383,0x8383,0x8383,0x8383,0x83C4,0x6383,0x6363,0x4243,0x4444,0x4242,0x4442,0x4442,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4242,0x4442,0x4444,0x4444,0x4244,0x4442
.word 0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xD4C4,0x6262,0x6262,0x9F9F,0x9F9F,0x4252,0x4142,0x4141,0x4141,0x6F6F,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4242,0x5252,0x7F7F,0x9F9F,0xD4C4,0x62D4,0x4141,0xC441,0xC4D4,0x7F7F,0x4141,0x7F7F,0x4141,0x5252,0xE4D4,0x41E4,0x62D4,0x7F7F,0x62D4,0x4141,0x7F7F,0x7F7F,0xD442,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0x41E4,0xD4D4,0xD4E4,0xD4E4,0x52D4,0xD451,0xD4D4,0x41D4,0x7F7F,0x7F7F,0xC4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0x44C4,0x4244,0x4442,0x4442,0x4444,0x4444,0x4444,0x4242,0x4244,0xC4C3,0x4443,0x4444,0x7342,0x83C4,0x6383,0x8373,0x7363,0x83C3,0x6363,0x8373,0x73C4,0x7373,0x6373,0x5263,0x6363,0x4463,0x4242,0x4242,0x4442,0x4244,0x4442,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4242,0x4242,0x4242,0x4242,0x4444,0x4244,0x4442,0x4444
.word 0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xAFAF,0x9F9F,0x5252,0x5252,0x4142,0x8F8F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x4242,0x4242,0x4242,0x4242,0x4444,0x5244,0x5252,0x9F9F,0x4141,0x6241,0xD4C4,0xD4D4,0x4152,0xD452,0xD4D4,0xD4D4,0x9F9F,0xD452,0xE4E4,0x52D4,0x5252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5241,0xE452,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xC4D4,0xE4D4,0xE4D4,0xD4E4,0xD441,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4152,0x7F7F,0x7F7F,0xD441,0xD4D4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4462,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4442,0x83C4,0x4373,0x4444,0x4444,0xC483,0x5283,0x6363,0x7363,0x7383,0xC4C4,0x7373,0x8383,0x6383,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4244,0x4244,0x4242,0x4442,0x4244,0x4442,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4442
.word 0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x6262,0x9F9F,0x9F9F,0x9F9F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4242,0x4242,0x4242,0x4442,0x4444,0x4242,0x8F8F,0x4141,0x4141,0x7F7F,0x6241,0xD4C4,0xD4D4,0xD4D4,0x52D4,0xD452,0xD4D4,0xD4D4,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4241,0x9F9F,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7F7F,0x41D4,0xD4E4,0xE441,0xD441,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x7F7F,0x7F7F,0x4141,0x7F7F,0xD452,0xE4E4,0xE4F4,0xE4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0xAFAF,0x4444,0x4242,0x4242,0x4242,0x4242,0x4244,0x4442,0x4244,0xC363,0x8383,0xC4C4,0x8383,0x8383,0x7383,0x7373,0x8383,0x8363,0x7363,0x8373,0x8383,0x6383,0x4244,0x4244,0x4442,0x4244,0x4442,0x4442,0x4242,0x4244,0x4442,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4442,0x4444,0x4444,0x4444,0x4244,0x4242
.word 0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFAF,0x6262,0x5252,0x5252,0x9F9F,0x4152,0x4141,0x7F7F,0x7F7F,0x6F6F,0x4242,0x4242,0x4242,0x4242,0x4442,0x9F9F,0x4141,0x4241,0x4141,0x7F7F,0x7F7F,0xC452,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4152,0x7F7F,0x52D4,0x7F7F,0x7F7F,0x7F7F,0x5242,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0x7F7F,0xC4D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xE4D4,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x5262,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x4462,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0x4244,0x4444,0xC442,0x8383,0xC383,0xC3C4,0xC4C3,0x7383,0x6363,0x6363,0x8383,0x7373,0x7363,0xC463,0x7373,0x4444,0x4444,0x4244,0x4242,0x4242,0x4244,0x4242,0x4242,0x4442,0x4444,0x4442,0x4444,0x4444,0x4442,0x4444,0x4442,0x4242,0x4442,0x4444,0x4244,0x4242,0x4244,0x4444,0x4244
.word 0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xAFAF,0xAFAF,0x9F9F,0x5252,0x5252,0x4141,0x4141,0x7F7F,0x7F7F,0x4242,0x4244,0x4244,0x4244,0x5244,0x5252,0x5241,0x8F8F,0x4242,0x4141,0x4141,0x7F7F,0xC451,0xC4D4,0x62D4,0x4152,0x5251,0xD4C4,0x41D4,0xD452,0x41C4,0x7F7F,0xD4D4,0x4152,0x5242,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0x41D4,0x7F7F,0x5252,0x62C4,0x7F7F,0x5241,0xD4D4,0xE441,0x5252,0x41D4,0x7F7F,0x41C4,0x7F7F,0x6262,0x9F9F,0x7F7F,0xE4D4,0xD4D4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0x4452,0x4444,0x4242,0x4442,0x4242,0x4242,0x4444,0x4442,0x4444,0x4444,0xC4C4,0x8383,0xC383,0xC3C4,0x8363,0xC383,0x7373,0x6383,0x8373,0x7363,0x8383,0x7373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4444,0x4244,0x4442,0x4244,0x4244,0x4244,0x4442,0x4444,0x4444,0x4242,0x4244,0x4242
.word 0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0xC462,0x5262,0x5252,0x9F9F,0x9F9F,0x4142,0x7F7F,0x6F6F,0x7F7F,0x4242,0x4442,0x4442,0x4444,0x5143,0x8F8F,0x5242,0x9F9F,0x5252,0x8F8F,0x8F8F,0x4141,0x8F8F,0xD462,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x7F7F,0xC4D4,0x7F7F,0xD4D4,0x52D4,0x8F8F,0xD452,0xE4D4,0xD4D4,0xD4E4,0x52D4,0x7F7F,0x7F7F,0xD4D4,0x41D4,0x7F7F,0x4141,0x9F9F,0x7F7F,0x9F9F,0x4141,0xD4D4,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xC4C4,0x7F7F,0x8F8F,0x5262,0xD452,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x4452,0x4244,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x4244,0x4444,0x6342,0xC4C4,0xC4C4,0xC3C4,0x6363,0x8373,0x8363,0x63C4,0x6373,0x6363,0x8363,0x7373,0x4443,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444
.word 0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4F4,0xE4D4,0xE4E4,0xE4F4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x9F9F,0x9F9F,0x9F9F,0x4252,0x4151,0x7F7F,0x6F6F,0x7F7F,0x4242,0x4242,0x4242,0x5244,0x5252,0x9F9F,0x9F9F,0x5252,0x8F8F,0x4141,0x4152,0x4142,0x7F7F,0x4141,0xD462,0xD4D4,0xD4D4,0x5241,0xD4D4,0x7F7F,0x4152,0x4141,0xE452,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD452,0x7F7F,0x6252,0x41C4,0xD462,0x7F7F,0x7F7F,0x5241,0x9F9F,0x4141,0xD452,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5262,0x9F9F,0x6262,0xD441,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC462,0x9F9F,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4444,0x73C4,0x4444,0x8373,0xC3C4,0x6363,0x6373,0x6363,0x7373,0x8373,0x8373,0x83C4,0x73C3,0x4383,0x4444,0x4444,0x4444,0x4442,0x4442,0x4242,0x4242,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4442,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4444,0x4442,0x4444
.word 0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x62C4,0x5262,0x9F9F,0x5252,0x4142,0x4141,0x6F6F,0x7F7F,0x4242,0x4242,0x4444,0x5252,0x9F9F,0x9F9F,0x5252,0x9F9F,0x4252,0x5241,0x8F8F,0x4141,0x4141,0x8F8F,0x7F7F,0xC4C4,0x4141,0xC452,0x9F9F,0x7F7F,0x5241,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x7F7F,0x41E4,0x5242,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0xD4D4,0xD4E4,0x7F7F,0x7F7F,0x5241,0x6241,0x5241,0x62C4,0x4152,0xD4C4,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x5262,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x4452,0x4444,0x4444,0x4244,0x4242,0x4244,0x4242,0x4242,0x4444,0x4444,0x83C4,0x4473,0x8363,0xC483,0x7363,0x6383,0x8383,0x8363,0x6383,0x7383,0x6363,0x8383,0x7373,0x4444,0x4242,0x4242,0x4442,0x4442,0x4242,0x4444,0x4442,0x4444,0x4444,0x4444,0x4442,0x4242,0x4244,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242,0x4244,0x4242,0x4244
.word 0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0xAFAF,0x5252,0x9F9F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x4242,0x4242,0x5242,0x9F9F,0x5252,0x5252,0x9F9F,0x9F9F,0x5252,0x9F9F,0x5252,0x4252,0x4142,0x4141,0x4141,0x7F7F,0x7F7F,0x9F9F,0x4152,0x7F7F,0x9F9F,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x7F7F,0x5251,0x7F7F,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x7F7F,0xD4D4,0xD4D4,0x52D4,0x8F8F,0x4141,0x7F7F,0xD4D4,0x41C4,0xC452,0x7F7F,0x5241,0x5262,0x8F8F,0xD462,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x9F9F,0x4441,0x4444,0x4242,0x4442,0x4444,0x4444,0x4444,0x4244,0x4244,0x4442,0xC444,0x8383,0x83C4,0xC483,0x6373,0x6363,0x6363,0x7363,0x7383,0x6373,0x6363,0x8373,0x8383,0x4373,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4442,0x4442,0x4244,0x4442,0x4244,0x4F4F,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444
.word 0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62D4,0x6262,0x5262,0x9F9F,0x4152,0x4141,0x6F6F,0x7F7F,0x4242,0x4242,0x5244,0x6252,0x9F9F,0xAFAF,0x6252,0x9F9F,0x9F9F,0x6262,0xAFAF,0x9F9F,0x9F9F,0x4242,0x8F8F,0x4142,0x5241,0x9F9F,0x7F7F,0x4141,0x5241,0x9F9F,0xC462,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x41C4,0x7F7F,0x7F7F,0xD441,0x41D4,0x5241,0x5252,0x7F7F,0xC4C4,0x7F7F,0xE462,0xD4D4,0x62C4,0x4162,0x5241,0xD4C4,0x5241,0xD441,0x7F7F,0x41C4,0x9F9F,0x9F9F,0x4141,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0x4442,0x4444,0x4444,0x4444,0x4444,0x4442,0x4242,0x4242,0x4442,0x4444,0x7342,0xC483,0xC4B3,0xC3C4,0x5283,0x7352,0x73C3,0x8373,0xC463,0x7373,0xC483,0x8373,0x7383,0x7383,0x4443,0x4444,0x4444,0x4444,0x4244,0x4244,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4F4F,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4242,0x4242
.word 0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFAF,0x9F9F,0x5262,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x4242,0x4442,0x5252,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5262,0xAFAF,0x9F9F,0x5262,0x5262,0x5252,0x8F8F,0x4141,0x8F8F,0x7F7F,0x5241,0x8F8F,0x4141,0x8F8F,0x4141,0x8F8F,0x5251,0x5251,0x4151,0x4141,0x5241,0x62C4,0x4141,0x4141,0x4152,0x41D4,0x7F7F,0x6241,0xAFAF,0x8F8F,0x4141,0xD4D4,0xD4D4,0x7F7F,0x4241,0xAFAF,0xD4D4,0x4141,0xD441,0x7F7F,0x5241,0x52D4,0xD452,0x6241,0x8F8F,0x6262,0xD4E4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0x9F9F,0x4242,0x4242,0x4242,0x4242,0x4242,0x4444,0x4444,0x4242,0x4442,0x4242,0xC463,0x83C4,0xC383,0x6383,0x6363,0x6373,0x8383,0x8373,0x6363,0x7363,0x8373,0x83C4,0x7383,0x7383,0x4443,0x4244,0x4442,0x4444,0x4444,0x4444,0x4242,0x4244,0x6F6F,0x4F4F,0x4242,0x4442,0x4F4F,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244
.word 0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0x6262,0x9F9F,0x5252,0x4142,0x7F7F,0x6F6F,0x4242,0x5242,0x5252,0x6262,0x6262,0xAFAF,0x5262,0xAFAF,0x9F9F,0x6262,0x9F9F,0x9F9F,0x9F9F,0x5252,0x5242,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x8F8F,0x5152,0x6252,0x6262,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6252,0x9F9F,0x4141,0x7F7F,0x4141,0x7F7F,0xD4D4,0x7F7F,0x7F7F,0xC441,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x62D4,0x9F9F,0xD452,0x7F7F,0x5241,0xAFAF,0x6262,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC452,0x4452,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4242,0x4444,0x4442,0xC473,0xC4C4,0x83C4,0x7352,0x6373,0x6363,0x8373,0x8363,0x63C4,0x8383,0xC483,0xC473,0x8373,0x6383,0x4444,0x4444,0x4244,0x4442,0x4242,0x4442,0x4442,0x4242,0x4F4F,0x4F4F,0x4442,0x4F4F,0x4F4F,0x4242,0x4444,0x4444,0x4F4F,0x4444,0x4444,0x4444
.word 0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFAF,0x6262,0x9F9F,0x5241,0x6F6F,0x7F7F,0x4242,0x6252,0xAFAF,0xAFAF,0xAFAF,0xC4C4,0xC462,0xD4C4,0xD4C4,0xD4D4,0xD4D4,0xC462,0x9F9F,0xAFAF,0x9F9F,0x5252,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0xD4D4,0x4152,0x7F7F,0xC441,0xC4D4,0x4141,0x7F7F,0x7F7F,0x5241,0x7F7F,0x5241,0xD441,0xD4C4,0x62D4,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x9F9F,0x4452,0x4444,0x4242,0x4242,0x4242,0x4244,0x4444,0x4242,0x4442,0x4344,0x4363,0x4444,0x4244,0x7344,0x8383,0x5252,0x6373,0x6373,0x7363,0x6363,0x6363,0x6363,0x6363,0xC483,0xC473,0x8383,0x8383,0x4463,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4F4F,0x4444,0x4F4F,0x4F4F,0x6F6F,0x4442,0x4442,0x5F5F,0x4442,0x4444,0x4242
.word 0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xF4E4,0xE4F4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0x9F9F,0x5252,0x4141,0x6F6F,0x7F7F,0x4242,0xC462,0xC462,0xC462,0xD4C4,0xD4C4,0xD4D4,0xD4C4,0xD4D4,0xC4D4,0xC4D4,0xC4C4,0xC4C4,0xC4C4,0x62C4,0x5262,0x9F9F,0x4152,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0x52D4,0x7F7F,0x7F7F,0xD441,0x41D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD441,0xD4D4,0xD4C4,0x9F9F,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0xAFAF,0x4452,0x4444,0x4444,0x4244,0x4242,0x4242,0x4242,0x4444,0x4444,0x4444,0xC473,0x4463,0x4444,0x7363,0xC483,0x6363,0x6363,0x7363,0x8373,0x8363,0x6383,0x6383,0x8383,0x9383,0x83C4,0x8383,0x8373,0x6383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4444,0x4F4F,0x4F4F,0x4442,0x4F4F,0x4242,0x5F5F,0x4444,0x4F4F,0x4244,0x4244,0x4242
.word 0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4F4,0xE4E4,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0x9F9F,0x4152,0x6F6F,0x6F6F,0x5242,0xC4C4,0xC4D4,0xC4D4,0xC4C4,0xC4D4,0xC4D4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xAFAF,0xAFAF,0x5252,0x9F9F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD4D4,0xD4D4,0x7F7F,0x7F7F,0x4141,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5241,0xD4D4,0xD4D4,0xC4D4,0x62C4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x9F9F,0x6262,0x4243,0x4242,0x4242,0x4242,0x4442,0x4444,0x4444,0x4242,0x4444,0x4444,0xC442,0x8383,0x8383,0x83B3,0x8383,0x6383,0x6363,0x6373,0x8373,0x7363,0x7383,0x6373,0x6363,0xC463,0x6383,0x8383,0x8373,0x8373,0x63C4,0x4444,0x4244,0x4442,0x4442,0x4242,0x4242,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x4F4F,0x4F4F,0x6F6F,0x4242,0x4444
.word 0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xE4E4,0xF4E4,0xE4E4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xAFAF,0x5262,0x9F9F,0x6F6F,0x6F6F,0xD441,0xC4C4,0x62C4,0xD4C4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xC4D4,0x6262,0x9F9F,0x5252,0x4152,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xE441,0xD4D4,0xD452,0x7F7F,0x4141,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD441,0x7F7F,0xD4D4,0xD4D4,0xC4D4,0x62C4,0x5262,0xD4C4,0xC4D4,0xD462,0xC4C4,0x9F9F,0x4443,0x4244,0x4242,0x4242,0x4442,0x4242,0x4444,0x4444,0x4444,0x4244,0x4442,0x8373,0xC483,0x83C4,0xC483,0x83C4,0x6363,0x6363,0x8373,0x6363,0x8363,0x7383,0x8383,0x8383,0x83C4,0x8383,0x8363,0x8383,0x8383,0x4373,0x4444,0x4444,0x4444,0x4444,0x6F6F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4242,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4242
.word 0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFAF,0x4252,0x4141,0x6F6F,0x62C4,0xC4C4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0xAFAF,0xAFAF,0x4252,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xE4D4,0xD4D4,0x41C4,0x7F7F,0xC4D4,0x7F7F,0x4152,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4162,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0x5262,0x9F9F,0x9F9F,0xC462,0x41C4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4442,0x4442,0x4444,0x4444,0x4442,0xC473,0x8383,0x8383,0xC3C3,0x7383,0x6352,0x6352,0x7373,0x7363,0x6373,0x6363,0x8373,0x7373,0x8383,0x8363,0x73C4,0x7383,0x8383,0x4463,0x4444,0x4444,0x4444,0x5F5F,0x4442,0x4F4F,0x4F4F,0x6F6F,0x6F6F,0x4F4F,0x4F4F,0x4F4F,0x4244,0x4F4F,0x4242
.word 0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x62C4,0x9F9F,0x4141,0x6F6F,0xD4D4,0x62D4,0xD4C4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4D4,0x62C4,0x62C4,0x9F9F,0x5242,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0xD4D4,0xD4D4,0x7F7F,0x4141,0xD4D4,0x41D4,0x5241,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0xAFAF,0x5252,0x5252,0x9F9F,0x5262,0x4442,0x4444,0x4444,0x4444,0x4444,0x4444,0x4242,0x4242,0x4444,0x4444,0x4244,0x6342,0xC4C4,0xC4C4,0xC4C4,0xC4C4,0x6383,0x5252,0x6363,0x8373,0x8373,0x63C3,0x8383,0x8363,0xC483,0x7373,0x6383,0x83C4,0x8373,0x63C4,0x4444,0x4444,0x4242,0x4F4F,0x4242,0x5F5F,0x4442,0x4F4F,0x5F5F,0x4F4F,0x4F4F,0x4242,0x4242,0x4442,0x4444
.word 0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62C4,0xAFAF,0x7F7F,0xC441,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0xAFAF,0x9F9F,0x4152,0x4141,0x7F7F,0x7F7F,0x4141,0x4142,0x4242,0x5252,0x9F9F,0x6262,0x5252,0x9F9F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD4C4,0xD4D4,0xD4C4,0x41D4,0x7F7F,0xD4D4,0xC4C4,0x7F7F,0x7F7F,0x6241,0x7F7F,0xE4D4,0xD4D4,0xD441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x9F9F,0x9F9F,0x5262,0x9F9F,0x5252,0x5252,0x4443,0x4244,0x4242,0x4442,0x4442,0x4244,0x4442,0x4242,0x4442,0x4442,0x4244,0x4242,0x4342,0x8373,0xC483,0x6383,0x5252,0x6363,0x7363,0x6363,0x6363,0x8363,0x73C4,0xC373,0x73C4,0x8373,0x8383,0x8373,0x4383,0x4444,0x4444,0x4F4F,0x4F4F,0x4444,0x4244,0x4244,0x4444,0x4444,0x4444,0x4242,0x4F4F,0x4444,0x4444
.word 0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x9F9F,0x4152,0xC462,0xC4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62C4,0x6262,0x4252,0x4141,0x4141,0x7F7F,0x8F8F,0x8F8F,0x5241,0x8F8F,0x8F8F,0x9F9F,0xC452,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD4C4,0xD462,0xD4C4,0x7F7F,0x7F7F,0xD4D4,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0xD4D4,0xD4D4,0xD4E4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6262,0x9F9F,0x5252,0xAFAF,0x9F9F,0x5262,0x5252,0x5252,0x4444,0x4442,0x4242,0x4442,0x4444,0x4444,0x4242,0x4244,0x4442,0x4444,0x4444,0x7344,0x8383,0x8383,0x5273,0x5263,0x7352,0x7383,0xC483,0x7373,0x6383,0x6373,0x6383,0xC383,0x8383,0x73C4,0x8383,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x4244,0x4242,0x4442,0x4444,0x4444,0x4444,0x4242,0x4442
.word 0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x6262,0x9F9F,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xC4C4,0xAFAF,0x4252,0x4142,0x4141,0x7F7F,0x4141,0x4141,0x8F8F,0x5252,0x5252,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4152,0x4162,0x7F7F,0x4152,0xD441,0xD4D4,0x41D4,0x7F7F,0x4162,0xD4D4,0xD4D4,0xD4D4,0x6262,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFAF,0x5262,0x9F9F,0x6252,0x9F9F,0x9F9F,0x9F9F,0x9F9F,0x5252,0x4442,0x4444,0x4442,0x4242,0x4242,0x4444,0x4444,0x4444,0x4444,0x4444,0xC463,0xC483,0xC483,0xC4B3,0x6383,0x5252,0x6352,0x8373,0x8363,0x7373,0xC383,0x8363,0x6373,0x7373,0x8383,0x8383,0x4383,0x4244,0x4442,0x4244,0x4244,0x4444,0x4444,0x4444,0x4244,0x4442,0x4442,0x4444,0x4244,0x4242
.word 0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xF4F4,0xE4F4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x9F9F,0xC4C4,0xD4D4,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0x6262,0xAFAF,0x5251,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xC452,0xC4C4,0xD4C4,0xD4D4,0x62C4,0x7F7F,0xD4D4,0xD4D4,0x6241,0x4162,0x62E4,0x52D4,0x7F7F,0xD4D4,0x7F7F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC462,0x5262,0x9F9F,0x9F9F,0x5252,0x9F9F,0x6252,0x6262,0xAFAF,0x5252,0x4444,0x4444,0x4244,0x4242,0x4442,0x4242,0x4444,0x4444,0xC463,0x83C4,0x8383,0x8383,0xC383,0x83C3,0x5273,0x5263,0x6363,0xC483,0x8363,0x8383,0x8373,0x63C4,0x8383,0x7363,0x7383,0x8383,0x4444,0x4444,0x4244,0x4442,0x4242,0x4442,0x4444,0x4444,0x4244,0x4442,0x4442,0x4444,0x4444
;*/


/* Dragon Girl
Fish_320:
.word 0x4646,0x4646,0x4646,0x4646,0x8575,0x8575,0x7575,0x4646,0x4646,0x4646,0x4646,0x4646,0x7544,0x5465,0x7575,0x4645,0x4646,0x4646,0x7565,0x5454,0x5465,0x6585,0x4646,0x6546,0x7554,0x8575,0x4646,0x4646,0x4646,0x4646,0x6544,0x7554,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6F6F,0x6F6F,0x6F6F,0x4141,0x4444,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4646,0x6546,0x4665,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5444,0x8565,0xD4D4,0x4575,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6544,0x8585,0x95D4,0x7585,0x5465,0x4645,0x4646,0x4646
.word 0x4646,0x4644,0x4646,0x4644,0x4546,0x7595,0x7575,0x8585,0x4675,0x4644,0x4646,0x4646,0x5446,0x5475,0x5475,0x6475,0x4645,0x4544,0x7575,0x7554,0x7554,0x5465,0x7585,0x7554,0x7554,0x7554,0x4675,0x4446,0x4646,0x4444,0x7554,0x7575,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x5F5F,0x4444,0x6F6F,0x4444,0x4444,0x8F8F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x4646,0x4544,0x54D4,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x7541,0x9585,0x95D4,0x5475,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4444,0x8554,0xB5D4,0x75B4,0x4575,0x4646,0x4644,0x4644,0x465F
.word 0x4646,0x4646,0x4646,0x4646,0x4646,0x5446,0x54C4,0x6554,0x7564,0x7575,0x4446,0x4446,0x7546,0x7554,0x7554,0x7554,0x5454,0x6554,0x5454,0x5454,0x5475,0x5475,0x5454,0x5475,0x7554,0x7554,0x6454,0x7575,0x7554,0x7554,0x5454,0x7554,0x4685,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x7F7F,0x7F7F,0x4141,0x5454,0x4141,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4444,0x5441,0xD464,0x5485,0x5454,0x6F6F,0x4141,0x4444,0x7F7F,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4444,0x4141,0x7F7F,0x4646,0xC554,0x4475,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5444,0x8575,0x95D4,0x5485,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x8554,0x95C5,0x75B4,0x4475,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x8565,0x7575,0x7554,0x6475,0x7575,0x4646,0x5444,0x7554,0x5454,0x5465,0x5475,0x5475,0x5454,0x5454,0x5454,0x5454,0x6554,0x5454,0x5475,0x5454,0x5475,0x5475,0x5475,0x5454,0x7575,0x5475,0x5475,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4144,0x4444,0x7F7F,0x4444,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x6F6F,0x5F5F,0x4444,0x6F6F,0x6441,0xD4D4,0xD4E4,0x6464,0x7F7F,0x6F6F,0x5444,0x4141,0x4141,0x6F6F,0x7F7F,0x4444,0x6F6F,0x7F7F,0x4444,0x6F6F,0x4141,0x4141,0x4444,0x7544,0x75D4,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4146,0x7544,0xB485,0xD4B5,0x8595,0x4554,0x4646,0x4646,0x4646,0x4646,0x4646,0x5446,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x7554,0x9585,0x75B4,0x4475,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646
.word 0x4446,0x4545,0x4545,0x4545,0x4446,0x4646,0x4646,0x7575,0x7554,0x7554,0x5475,0x5475,0x7554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5454,0x5454,0x4475,0x4446,0x4646,0x4646,0x4444,0x5454,0x7F7F,0x4444,0x5441,0x4444,0x6F6F,0x6F6F,0x4444,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4141,0x6F6F,0x5454,0xD564,0xD4E4,0x6464,0x4444,0x6F6F,0x4441,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x4141,0x4444,0x4141,0x6F6F,0x4545,0x9554,0x6575,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5444,0x8575,0x95C4,0x95D4,0x8585,0x4454,0x4446,0x4646,0x4646,0x5446,0x6554,0x5465,0x4641,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4444,0x8575,0x8585,0x8595,0x7595,0x7585,0x5454,0x4444,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x5454,0x8485,0xC495,0x7585,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7546,0x8585,0x8575,0x7575,0x8575,0x4485,0x4446,0x7575,0x5475,0x5475,0x5465,0x5454,0x5475,0x5454,0x5454,0x5454,0x5444,0x5454,0x5444,0x4454,0x4454,0x4454,0x5454,0x4454,0x5441,0x5444,0x5454,0x5454,0x5465,0x5454,0x5454,0x6554,0x5475,0x7575,0x4475,0x4646,0x4141,0x5441,0x9F9F,0x7F7F,0x4144,0x5454,0x7F7F,0x7F7F,0x4444,0x7F7F,0x4444,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5441,0x5454,0x7F7F,0x7F7F,0x4444,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x4144,0x7F7F,0x5445,0x85D4,0x4665,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4146,0x6554,0x8575,0xC485,0xD4B5,0x95B5,0x8585,0x7585,0x7575,0x7575,0x7575,0x5454,0x5465,0x4444,0x4454,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x8554,0x9595,0x95C4,0x7585,0xB485,0x9585,0x9595,0x85B4,0x5454,0x5454,0x6575,0x5454,0x5454,0x8575,0x8585,0x9495,0x85C4,0x75B4,0x4675,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646
.word 0x4646,0x4646,0x8575,0x7575,0x6554,0x5454,0x6575,0x7554,0x5475,0x5454,0x5454,0x5454,0x5454,0x5444,0x5444,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x7541,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4454,0x5454,0x7554,0x7554,0x5454,0x4444,0x4441,0x4444,0x4444,0x5454,0x4141,0x8F8F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x5454,0x5454,0x4141,0x4154,0x4444,0x7F7F,0x6F6F,0x6F6F,0x4141,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4444,0x6F6F,0x7F7F,0x7F7F,0x4444,0x4444,0x4141,0x4646,0x7554,0x6495,0x4654,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x6541,0x5454,0x7554,0x7565,0x8575,0x95B4,0xD4C4,0xB595,0x9595,0xB5B5,0xB5D4,0x95D4,0x95C5,0x95C5,0x8585,0x6475,0x5454,0x4454,0x4454,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4446,0x8554,0xC4C5,0xB485,0x6575,0x5475,0x5454,0x6575,0x5454,0x8554,0x7585,0x7585,0x7575,0x8575,0x8575,0x8585,0x5454,0x5454,0x7575,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4446,0x4646,0x6546,0x8585,0x7554,0x5475,0x7575,0x7554,0x7554,0x5454,0x5454,0x5454,0x5444,0x5454,0x5454,0x5454,0x5454,0x5475,0x7554,0x5475,0x7554,0x5454,0x7554,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5454,0x5444,0x5441,0x5454,0x5454,0x4454,0x4141,0x4141,0x4444,0x4141,0x5454,0x5454,0x7F7F,0x4444,0x4444,0x7F7F,0x4444,0x7F7F,0x7F7F,0x4444,0x6F6F,0x6F6F,0x5454,0x4444,0x6464,0x5454,0x8F8F,0x7F7F,0x7F7F,0x4444,0x6F6F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x4444,0x6F6F,0x4141,0x4141,0x5444,0x8F8F,0x4446,0xC575,0x6575,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4441,0x5454,0x5454,0x5454,0x5454,0x8575,0xC485,0xC485,0x95B4,0x95C4,0x85C4,0xC495,0xC4E4,0xD4D4,0xD4B5,0x9584,0x85B5,0x7585,0x7575,0x5454,0x4454,0x4444,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x8544,0x8595,0x85B4,0x7575,0x5454,0x4644,0x4646,0x4444,0x5454,0x6554,0x7565,0x5454,0x5475,0x5475,0x6554,0x6554,0x5454,0x4454,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0x4646,0x4646,0x4646,0x7565,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x5444,0x5454,0x5454,0x5454,0x7554,0x7554,0x7575,0x5454,0x7575,0x7575,0x5454,0x7575,0x7585,0x5475,0x7554,0x8585,0x5454,0x5454,0x5454,0x7554,0x5454,0x5454,0x4454,0x4154,0x4141,0x4444,0x5454,0x4141,0x5454,0x4444,0x7F7F,0x4444,0x7F7F,0x6F6F,0x4141,0x7F7F,0x6F6F,0x4444,0x6F6F,0x4141,0x8585,0x64D4,0xD564,0xC4C4,0x5454,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4444,0x6464,0x4141,0x4646,0x6554,0x5454,0x5454,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x5444,0x5454,0x7554,0x7565,0xC485,0x8585,0xB485,0xB495,0x8585,0x8585,0x95B4,0xC4C4,0xB495,0x95C5,0xC5C5,0xB5C4,0xB584,0x8595,0x5475,0x5454,0x4144,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x7585,0x7565,0x5475,0x5475,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646
.word 0x4446,0x9585,0x8585,0x7585,0x5475,0x7554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x8575,0x5475,0x5454,0x7554,0x7585,0x5454,0x5475,0x7585,0x5454,0x7575,0x5475,0x7575,0x7575,0x5465,0x8554,0x8585,0x5475,0x7554,0x7575,0x5454,0x4141,0x5454,0x5454,0x8F8F,0x4444,0x5454,0x8F8F,0x7F7F,0x7F7F,0x4444,0x4141,0x6F6F,0x4444,0x6F6F,0x6F6F,0x4141,0x7F7F,0x6454,0x6464,0x64E5,0x64E4,0x7F7F,0x6F6F,0x6F6F,0x4144,0x7F7F,0x4444,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x6F6F,0x6F6F,0x4444,0x6F6F,0x7F7F,0x4444,0x7F7F,0x4141,0xC4C4,0x4141,0x5445,0x7554,0x9575,0x6575,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4146,0x5444,0x7554,0x7575,0x7575,0x7575,0x7585,0x7585,0x85B4,0x8575,0x8585,0x85B4,0xB4C4,0xC495,0xC495,0xD495,0x95D4,0x95D4,0x8595,0x7575,0x5454,0x4654,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7544,0x7595,0x5475,0x7575,0x5454,0x7585,0x4454,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F
.word 0x8575,0x75B4,0xB485,0x6575,0x6564,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5475,0x7575,0x7575,0x5475,0x7565,0x7554,0x8575,0x7575,0x7554,0xC485,0x7585,0x8554,0x85D4,0x5465,0x8575,0x7585,0x6554,0x8575,0x5475,0x7554,0x7575,0x5475,0x4444,0x44D4,0x5454,0x5454,0x5441,0x4444,0x4141,0x4444,0x7F7F,0x4141,0x7F7F,0x4144,0x6F6F,0x4141,0x4141,0x6F6F,0x4141,0x5454,0xD454,0xF5D4,0x5454,0x6F6F,0x7F7F,0x4141,0x5F5F,0x4444,0x4141,0x6F6F,0x6F6F,0x4444,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4444,0x4444,0x4141,0x4545,0x5454,0x7565,0x8585,0x75B5,0x4654,0x4646,0x7584,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5444,0x5454,0x6554,0x6564,0x5465,0x7575,0x7575,0x7575,0x75B4,0x8575,0x8585,0x85B4,0xC485,0xC485,0xC495,0xC485,0xC595,0xC595,0x8595,0x7575,0x4465,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4446,0x7565,0x5475,0x5475,0x7554,0x8575,0x8575,0x7585,0x7575,0x7575,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0x8585,0x7585,0x7575,0x5475,0x5475,0x5454,0x5454,0x5454,0x7554,0x5454,0x5475,0x7575,0x5454,0x7554,0x7554,0x8585,0x7554,0x7575,0x8575,0x6585,0x7575,0x8585,0x8565,0x8585,0x5475,0xC485,0xC485,0x5475,0xD485,0x5485,0x5454,0x7575,0x5151,0xD441,0xC4C4,0x6464,0x9F9F,0xD554,0x4141,0x7F7F,0x6F6F,0x4141,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4141,0x5454,0xD4D5,0x6464,0xC4C4,0x7F7F,0x4444,0x4141,0x4141,0x5454,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4444,0x5F5F,0x6F6F,0x6F6F,0x4444,0x4141,0x6F6F,0x4141,0x7F7F,0x4141,0x4444,0x4444,0x7575,0x8585,0x95C4,0xD485,0x7595,0x8565,0x8565,0x4445,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x5444,0x5475,0x7554,0x7554,0x7554,0x5475,0x7575,0x7575,0x7575,0x7575,0x8575,0x8575,0x85B4,0xB485,0xB495,0xD495,0x85C4,0xB5C4,0x85D4,0x7585,0x4454,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x7585,0x5475,0x5475,0x8575,0xB485,0x7585,0x8585,0x7575,0x7554,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646
.word 0xB485,0x8554,0x6585,0x6564,0x5454,0x5454,0x6554,0x5454,0x5475,0x8575,0x8554,0x8585,0xC485,0x5475,0x7554,0xB485,0x7595,0x7575,0xC485,0x7585,0x7554,0x95C4,0x5485,0x9565,0x75C4,0x7565,0x8575,0x7585,0x8575,0x75B4,0x7575,0x6454,0x4444,0xD564,0xD464,0xE4E4,0x5444,0x5454,0x4444,0x4141,0x4444,0x4444,0x4141,0x4141,0x6F6F,0x4444,0x4444,0x4444,0x7F7F,0x4141,0x6464,0x6464,0x41D5,0x4141,0x4444,0x4141,0x7F7F,0x5454,0x7F7F,0x4444,0x4141,0x7F7F,0x6F6F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x5454,0x4141,0x4144,0x5454,0x8575,0x95C5,0x8585,0x7585,0x7575,0x4485,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x5454,0x5475,0x5475,0x5475,0x5475,0x7554,0x7554,0x7554,0x7575,0xB485,0x7585,0x8575,0xB495,0x8585,0xB495,0xB485,0xB495,0x8585,0x7595,0x4475,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4446,0x6446,0x7575,0x7554,0x7554,0x7585,0x7575,0x8585,0x6575,0x4654,0x4644,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7575,0x7575,0x5454,0x5475,0x5454,0x5454,0x7575,0x5475,0x7554,0x5454,0x7565,0x7575,0x6585,0x9575,0x7565,0x8575,0x7575,0x5475,0x8575,0xC485,0x7554,0xB485,0x7554,0x8554,0x95D4,0x7554,0xD485,0x5495,0x6575,0x5485,0x8575,0x5475,0x6464,0x5454,0x6464,0x4444,0x6464,0x9F9F,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x4444,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4144,0x5454,0x7575,0xC4C4,0x7F7F,0x7F7F,0x6F6F,0x5454,0x5464,0x4141,0xE441,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x7F7F,0x4141,0x7F7F,0x4444,0x7F7F,0x7F7F,0x4441,0x8F8F,0x6565,0x5475,0x7544,0x5475,0x8565,0x7565,0x7575,0x7595,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4446,0x5454,0x5475,0x5454,0x5475,0x5475,0x5454,0x7565,0x5475,0x7575,0x7575,0x7575,0x8575,0x85B4,0x85B4,0x8585,0x6575,0xD485,0x8585,0x7585,0x4645,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x5446,0x7575,0x7585,0x7575,0x85C4,0x8575,0x7575,0x4475,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644
.word 0x8585,0x7565,0x5454,0x5454,0x5454,0x5475,0x7554,0x7575,0x7575,0x8585,0x5475,0x7554,0x8554,0xB5D4,0x95C5,0x7575,0x8575,0x95C5,0x5485,0x85B4,0x75B5,0x7575,0xD485,0x5475,0x8575,0x5475,0xB485,0x5485,0x8575,0x54D4,0xC475,0x5454,0x54E4,0x6464,0xD5D5,0x5454,0x6464,0x4444,0x4444,0x4141,0x4444,0x4141,0x7F7F,0x6F6F,0x4444,0x7F7F,0x7F7F,0x4441,0x4141,0x4444,0x7575,0xC4C4,0x5464,0x6F6F,0x4141,0x4154,0xC4C4,0xD4D4,0xD4D4,0x4444,0x4141,0x4444,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4444,0x6F6F,0x4444,0x6F6F,0x6F6F,0x4444,0x4441,0xC485,0x5485,0x5454,0x7554,0x8575,0x8595,0x85B4,0x7585,0x4665,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5441,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x7554,0x7554,0x7554,0x7575,0x6575,0x8585,0xB485,0x7565,0x7575,0x7565,0x7585,0x5465,0x4454,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x7546,0x5454,0x5465,0x8575,0x7575,0xC485,0x7585,0x5454,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0x5475,0x5475,0x5454,0x5454,0x7554,0x7575,0x5475,0x5454,0x7575,0xB485,0xC495,0x6585,0x8575,0xD4B4,0x8585,0x5475,0x8565,0x7585,0x5465,0x8575,0x7575,0x7554,0x8585,0x5465,0xD485,0x6595,0x7554,0x5475,0x8585,0x7595,0xD454,0x5454,0x4444,0x5454,0x5441,0xC4C4,0x5454,0x4444,0x7F7F,0x4141,0x4444,0x4141,0x4444,0x6F6F,0x4141,0x4444,0x4141,0x7F7F,0x6F6F,0x4141,0x5454,0x7575,0xD4D4,0x6F6F,0x7F7F,0x4154,0xF564,0x54D4,0x54D4,0x7F7F,0x7F7F,0x4141,0x6F6F,0x4444,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4444,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5454,0x7F7F,0x9575,0x95C4,0x6585,0x5454,0x7575,0xC485,0x8585,0x95B4,0x6595,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5446,0x6585,0x7575,0x7585,0x7554,0x5475,0x5454,0x5454,0x5454,0x5475,0x5454,0x7554,0x7554,0x6465,0x8554,0x7554,0x7585,0x5465,0x8575,0x7585,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7546,0x7575,0x7554,0x7585,0x7554,0x8585,0x85B4,0x7554,0x5475,0x5465,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646
.word 0x5465,0x5454,0x5454,0x5454,0x7554,0x8575,0x8585,0x7575,0x5475,0x7565,0x7585,0x8554,0x5475,0x7575,0x7575,0xB5C4,0x5485,0x8554,0xB5C4,0x5475,0xC485,0x85D4,0x7554,0x7595,0x7575,0x7585,0x8575,0x85D4,0x7565,0x7565,0x5454,0x6464,0x4441,0x4444,0x5454,0x6464,0x4444,0x4444,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4441,0x4141,0x4141,0x4444,0x4141,0x4444,0x4141,0x5454,0x5454,0xAFAF,0x6F6F,0x5454,0x5454,0xD4D4,0x6464,0xD4D4,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x4444,0x7F7F,0x4444,0x6F6F,0x4141,0x6F6F,0x4141,0x6F6F,0x6F6F,0x7F7F,0x4141,0x4444,0x8554,0xC485,0x8575,0x7585,0x6554,0x7575,0xB485,0x7575,0x9585,0x6485,0x4445,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x6465,0x6554,0x6554,0x5454,0x5475,0x5475,0x5454,0x5485,0x8565,0x5475,0x5475,0x5465,0x5454,0x6575,0x5475,0x7554,0x9585,0xC5D4,0x95D4,0x9585,0x4654,0x4446,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5444,0x6465,0x7554,0x7575,0x7575,0x7575,0x5475,0x7575,0x5475,0x8575,0x7575,0x8575,0x7585,0x4664,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x5454,0x5454,0x5454,0x7554,0x5475,0x8575,0x7575,0x8554,0x95D4,0x5475,0x5454,0xC485,0xD495,0x8595,0x7554,0xC485,0x85C4,0x7565,0x7585,0x5475,0x8585,0x6575,0x7554,0x85C4,0x5465,0x5475,0x7575,0xC485,0x8575,0x5495,0x54D4,0x4444,0x5454,0x5454,0x5454,0x4454,0x6464,0x5454,0x4444,0x4444,0x4444,0x7F7F,0x4444,0x7F7F,0x4441,0x7F7F,0x5454,0x7F7F,0x7F7F,0x5454,0x5454,0xD554,0x6F6F,0x4141,0x6F6F,0x5454,0x5454,0x5454,0xD554,0x4444,0x6F6F,0x4444,0x7F7F,0x4444,0x5F5F,0x6F6F,0x4444,0x5F5F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x4141,0x7544,0x85C4,0x7585,0xB485,0x5475,0x5454,0x8575,0x8585,0xB475,0x8585,0x9585,0x7564,0x4575,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x5446,0x7575,0x5475,0x8554,0x5465,0x7554,0x5454,0x5454,0x6575,0x7575,0x5475,0x7554,0x5454,0x5454,0x5454,0x5454,0x6554,0x5475,0xB485,0xC485,0xC4C5,0x5495,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x6565,0x5464,0x5454,0x7575,0x7554,0x7575,0x7575,0x5454,0x7585,0x6575,0x6564,0x6554,0x7575,0x4644,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F
.word 0x5454,0x5454,0x5475,0x8575,0x85C4,0x5465,0x7565,0x7554,0x8585,0x85C4,0x6575,0x8575,0x75B4,0x9565,0x75D4,0x7554,0x8554,0x95C5,0x8565,0xD485,0x5465,0x8585,0x5495,0x7565,0x9575,0x95C5,0x6575,0x7575,0x8554,0x6565,0x41D4,0x4444,0x7F7F,0x6464,0x5454,0x5454,0x9F9F,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x4444,0x4141,0x4444,0x4154,0x5454,0x4444,0x4141,0x5454,0x5454,0xD4D4,0x6F6F,0x4444,0x4141,0x4441,0x5454,0x6464,0x5454,0x7F7F,0x4141,0x4141,0x4141,0x6F6F,0x7F7F,0x4141,0x5F5F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x4444,0x6F6F,0x6F6F,0x6F6F,0x4141,0x7575,0x9585,0x85B4,0x8585,0xD485,0x6585,0x5454,0xB475,0x7575,0xB485,0x8585,0x8585,0x8585,0x7585,0xB485,0x8595,0x85B4,0x6585,0x7575,0x7575,0x5475,0x5475,0x7554,0x7554,0x7575,0x7554,0x8575,0x7565,0x5475,0x5475,0x5454,0x7575,0x7554,0x5475,0x7554,0x5475,0x5454,0x5454,0x5475,0x8575,0x7585,0x7575,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7546,0x9585,0xD4D4,0x7585,0x5475,0x5454,0x5475,0x7554,0x7554,0x7575,0x5475,0x7575,0x7575,0x4554,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0x5454,0x5454,0x7554,0x8575,0x5475,0x8585,0x8585,0x5485,0x7575,0x6575,0x8585,0x6585,0x5454,0x8585,0x85C4,0x7565,0x8554,0xB485,0x6585,0x9585,0x5475,0x8565,0x75B4,0x8585,0x8554,0x85B4,0x7554,0x85B5,0x6554,0xC454,0x5454,0x8F8F,0x6441,0x5454,0x5454,0x6464,0x5475,0x4141,0x4444,0x5454,0x4444,0x4444,0x4141,0x4444,0x5454,0x6464,0x8F8F,0x4141,0x4141,0x4444,0x4444,0x5454,0x6F6F,0x4141,0x4444,0x6F6F,0x4141,0x4444,0x9F9F,0x5441,0x6F6F,0x6F6F,0x4141,0x6F6F,0x4444,0x6F6F,0x4141,0x4441,0x4141,0x4444,0x5454,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6565,0x8575,0xC495,0x75B4,0x7575,0x9585,0x5475,0x5454,0x7575,0x8575,0x7575,0x85B4,0x85B4,0x85B4,0x9595,0xB5C4,0x95A5,0x95C5,0x85C5,0x85C5,0x9585,0x8585,0x85C4,0x8585,0x7575,0x7575,0x5454,0x7575,0x5475,0x7554,0x5485,0x5454,0x5475,0x5454,0x7575,0x5485,0x8554,0x5454,0x5454,0x6554,0x5465,0x5475,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x8575,0xC5B5,0x95D4,0xD4B5,0x8595,0x7575,0x5454,0x5475,0x8575,0x7575,0x8575,0x8575,0x7585,0x4645,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x5454,0x8554,0x5465,0x7554,0x7575,0xB485,0x7585,0x8575,0x85C4,0x5475,0xB485,0x8595,0x5475,0x7575,0x7575,0xC595,0x6575,0x7575,0x8595,0x5465,0xC585,0x5485,0x5454,0xB485,0x5485,0x6554,0x7554,0x95C4,0x75C4,0x6464,0x9F9F,0x5454,0x5454,0x5454,0x5454,0x5454,0x4444,0x7F7F,0x4141,0x8F8F,0x4141,0x4141,0x4154,0x5454,0x5454,0x6464,0x4444,0x4441,0x7F7F,0x4141,0x4141,0x5454,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x4444,0x4141,0x5444,0x4444,0x6F6F,0x4154,0x6F6F,0x4444,0x7F7F,0x4141,0x4444,0x4141,0x4144,0x6F6F,0x7F7F,0x4444,0x4141,0x4444,0x4444,0x6F6F,0x4444,0x4141,0xC475,0x95B4,0x8585,0x7575,0xB485,0xC485,0x5475,0x5454,0x5475,0x8575,0x7575,0x8585,0xB485,0xB495,0xB485,0x85C4,0x85B4,0x85C4,0x95B4,0x85B4,0xB5C4,0xC5C4,0xD4C4,0xC595,0xD495,0xB5D4,0x8585,0x7575,0x7554,0x7554,0x7585,0x5454,0x5454,0x5454,0x5475,0x5475,0x5475,0xB485,0x5454,0x5454,0x4654,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4545,0x8575,0xB585,0x85C4,0xC485,0x8585,0x85C4,0xB5D4,0x5485,0x7554,0x6554,0x7575,0x7575,0x5475,0x4554,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646
.word 0x6554,0x8575,0x8575,0x5475,0x5454,0x7585,0x6585,0xC485,0xC4C5,0x5475,0x7575,0x6575,0xD485,0x5485,0x7554,0x8585,0x75D4,0x7554,0xB485,0x5454,0xC485,0xB485,0x7595,0x8565,0x6575,0x9595,0x6465,0xB475,0x5485,0xD4D4,0x4444,0x4444,0x5454,0x4444,0x64D4,0x5454,0x4444,0x4444,0x4141,0x4444,0x4444,0x4444,0x5454,0x5454,0x5454,0x5454,0x7F7F,0x4444,0x7F7F,0x4444,0x7F7F,0x4164,0x4144,0x7F7F,0x5454,0x5454,0x6F6F,0x4141,0x5454,0x4141,0x4441,0x5441,0x5454,0x4141,0x4141,0x4444,0x4141,0x4444,0x4444,0x8F8F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x8585,0x8575,0x95B4,0x8575,0x7575,0x8585,0x8585,0x5465,0x7554,0x7554,0x8575,0xB485,0xB495,0x8585,0x8585,0x8585,0xB485,0xB495,0xB495,0xB495,0xB485,0x8585,0x8585,0x85B4,0x95C4,0xD495,0xD4D4,0xC495,0x7585,0x7565,0x5475,0x8554,0x6575,0x5454,0x5454,0x5454,0x7554,0x7554,0x7554,0x7575,0x5441,0x4444,0x4646,0x4646,0x4646,0x4646,0x4644,0x9494,0x8585,0xB585,0xB485,0xB495,0xB485,0xB485,0xB485,0x9585,0x85D4,0x5465,0x7554,0x7575,0x8585,0x7565,0x5454,0x4454,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446
.word 0x5454,0x7575,0x8575,0x8585,0x6575,0x5454,0x5454,0x8585,0x75B4,0xD485,0x5485,0x5475,0x8575,0x7585,0x5465,0x7554,0x9554,0x5485,0x6465,0xC485,0x6575,0x5475,0x8575,0x7554,0x5475,0x7575,0x7585,0x7554,0x5454,0x4154,0x5454,0x5454,0x9F9F,0xD444,0x9F9F,0x64C4,0x4141,0x4141,0x5454,0x7F7F,0x4141,0x5454,0x5454,0x6464,0xD554,0x8F8F,0x4444,0x4141,0x4444,0x4141,0x6454,0xC4C4,0x6464,0x5444,0xC4C4,0x5151,0x5454,0xC4C4,0x5454,0x54C4,0xD4D4,0xC4C4,0xC4D4,0xC441,0x5454,0x5454,0x6464,0x7F7F,0x4141,0x7F7F,0x4444,0x4444,0x7F7F,0x5454,0x6F6F,0x7F7F,0x6F6F,0x4444,0x8575,0x85B4,0xB485,0xB485,0x8575,0x6575,0x7575,0x8585,0x5465,0x5475,0x5475,0x6575,0x7575,0x7565,0x7565,0x6575,0x8575,0x8575,0x8585,0x7575,0x8585,0x75B4,0x85B4,0x8575,0xB485,0xC485,0x8595,0x85C4,0x8585,0x8585,0x7575,0x6575,0x7554,0x7575,0x5475,0x6554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x6444,0x5454,0x6454,0x5444,0x8575,0x9585,0xB495,0xC485,0xB495,0x8585,0x8575,0x8585,0x8585,0x85B4,0xB5C4,0x7585,0x8554,0x7585,0x7554,0x8575,0x85C5,0x7575,0x5475,0x4454,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646
.word 0x5475,0x5475,0x7565,0x8575,0x8565,0xD495,0x7585,0x7565,0x7575,0x8585,0x85D4,0x7565,0x6554,0x5475,0xC595,0x7595,0xB454,0x7585,0x5475,0x8575,0x5475,0x5485,0x6575,0x7575,0x5485,0x5465,0x5454,0x8575,0x6454,0x4444,0x8F8F,0x4441,0x4444,0xE4E4,0x41D4,0x9F9F,0x4444,0x4444,0x4444,0x4144,0x5454,0x7575,0x5454,0xD5D5,0x6454,0x4141,0x6F6F,0x6F6F,0x4141,0x7F7F,0xD564,0x64D4,0x7575,0x5454,0xD4D4,0x6454,0x5454,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF4F4,0xE4E4,0x6464,0x8585,0x6464,0x6464,0x4144,0x8F8F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x9554,0x7585,0x8585,0x8585,0x7585,0x7585,0x5475,0x5454,0x5475,0x5475,0x5454,0x6465,0x7554,0x5454,0x5475,0x7554,0x7575,0x8575,0x75B4,0x8585,0x8575,0x8575,0x8585,0x85B4,0x8585,0x85B4,0x85B4,0xB495,0xC475,0xC495,0xC4B5,0x7585,0x7554,0x7585,0x5454,0x5475,0x7554,0x7575,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5475,0x9585,0xB495,0xB495,0xB485,0x8585,0x8575,0x75B4,0x7575,0x7575,0x7575,0xC485,0x7595,0x6554,0x7575,0x8554,0xC485,0xB4B5,0x8585,0x7585,0x5454,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x9575,0x7585,0x7554,0x6554,0xB464,0xB485,0x8575,0x94D4,0x7585,0x7565,0x7575,0x9585,0x95C4,0x5475,0x7575,0x8565,0x6585,0x7554,0x7575,0x7554,0x5475,0x6575,0x7554,0x7554,0x5475,0x7575,0x8575,0x5454,0x4444,0x5454,0x4444,0x7F7F,0x4141,0x85E4,0xC4C4,0x5464,0x5454,0x4444,0x6464,0x4444,0x64D4,0x6464,0x7575,0xD454,0x4444,0x8F8F,0x7F7F,0x4444,0x4444,0x6464,0xD454,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xF4E4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xD4D4,0x64C4,0x6464,0x4444,0x5454,0x4141,0x7F7F,0x4444,0x6F6F,0x6F6F,0x4444,0x4141,0x6F6F,0x4141,0x7575,0xC485,0x5475,0x7565,0x95C4,0x85C4,0x7575,0x5475,0x5454,0x8575,0xB5C4,0x85D4,0x5465,0x7575,0x8585,0x8585,0x7565,0x7575,0x7575,0x8575,0x75B4,0x8575,0x8575,0x7585,0x85B4,0xC585,0x85D4,0x7575,0x7554,0x8575,0x95C4,0x85C4,0x6585,0x7554,0x7585,0x7554,0x7575,0x5475,0x8575,0x85C4,0x7565,0x8575,0x5485,0x7575,0x6585,0x8554,0xD495,0xD4C5,0xB495,0x8595,0x8575,0x85B4,0x8585,0xB485,0x8585,0x8585,0x85B4,0x75B4,0x7554,0x7595,0x8575,0x85B4,0x7585,0x75D4,0x4554,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xB485,0x7585,0xD475,0x7585,0x7554,0x7585,0x8554,0xC485,0x7585,0x7585,0x5454,0x8575,0x7585,0x8575,0x5465,0x8575,0x7575,0x5475,0x7554,0x5454,0x6575,0x5454,0x5475,0x5475,0x5454,0x5475,0x7565,0x4154,0x4444,0x8F8F,0x4444,0x4141,0x4144,0x6464,0x5454,0x4444,0x4444,0x4141,0xC4C4,0x6441,0xD4D5,0xD4D4,0xD4C4,0x5454,0x6464,0x4444,0x4141,0x7F7F,0x4141,0x5454,0x6464,0xD4C4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xE4F4,0xE4F4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0x5444,0x7F7F,0x5454,0x4154,0x4444,0x6F6F,0x6F6F,0x4444,0x7F7F,0x6F6F,0x4444,0x6F6F,0x5454,0x8575,0x75D4,0x5454,0x8575,0xB485,0x7575,0x5475,0x95C4,0xB5C4,0xC5D4,0x95D4,0x7575,0x7575,0x7585,0x7565,0x5475,0x8575,0x8585,0x7575,0x8585,0xC475,0x7585,0x8575,0xD485,0x85B4,0x7575,0xD4D4,0x5475,0x5444,0x5454,0x4454,0x5454,0x5454,0x5454,0x4454,0x5454,0x5444,0x7554,0x7554,0x5454,0x5454,0x6554,0x6564,0x5454,0xC485,0xC4C5,0xC5E4,0x95C4,0x85C4,0x85B4,0x8575,0x85B4,0x7575,0x7585,0x85B4,0x7575,0x5454,0x8575,0x85D4,0x95C4,0xB485,0xC495,0x4575,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646
.word 0x7554,0x5475,0x8585,0x85D4,0x9575,0x85D4,0x6575,0x7575,0x6575,0xD485,0x5495,0x5465,0x6554,0x7554,0x6475,0x6554,0x7554,0x7575,0x5454,0x5454,0x5475,0x5475,0x7554,0x5454,0x5454,0x5454,0x7554,0x5454,0x4144,0x4144,0x4141,0x4444,0x6464,0x4444,0x7575,0x4141,0x6464,0xD444,0x4444,0xE4E4,0x6464,0xE4C4,0x64E4,0xD4D5,0x6464,0x7F7F,0x6F6F,0x4141,0x7F7F,0xD564,0xD564,0xD474,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4F4,0xF5F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xC4E4,0x6464,0x54D5,0x5454,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x7F7F,0x4444,0x6F6F,0x6F6F,0x5444,0x5454,0x7575,0x6585,0x7554,0x7585,0x7575,0x5475,0x8575,0xD4D4,0xD495,0xB495,0x5475,0x5475,0x5454,0x7554,0x5475,0x5454,0x7554,0xB485,0x7575,0x8585,0x8575,0x85C4,0x85B4,0xB565,0x5465,0x7554,0x9585,0x7575,0xC575,0x8585,0x5475,0x8585,0x8585,0x85C4,0x8585,0x8585,0x7575,0x7575,0x7575,0x7575,0x7575,0x8585,0x8585,0x95D4,0xB5D4,0x95C4,0x85C4,0x85B4,0x8575,0xB485,0x8585,0x7585,0x7575,0x7575,0x7585,0x5454,0x7554,0x8585,0xB495,0x5485,0x4645,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F44
.word 0x7554,0x5454,0x8575,0x6585,0xC485,0xC4B5,0x5475,0x5475,0x7554,0x8585,0x75B4,0x8585,0x7585,0x5475,0x7575,0x64C4,0x7554,0x7554,0x5454,0x5454,0x5454,0x5454,0x4454,0x5441,0x4454,0x4454,0x5454,0x4444,0x8F8F,0x4444,0x4444,0x4141,0xF664,0x5454,0x4444,0x4141,0x5454,0x6464,0x4444,0x64C4,0xF4D4,0xD464,0xE4E4,0x5454,0x6464,0x4444,0x7F7F,0x4444,0x5454,0x6464,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE5E4,0xF4F4,0xF5F4,0xF4F4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0x6464,0x6464,0x4444,0x4444,0x6F6F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x6F6F,0x6F6F,0x8554,0x6585,0x5454,0x5454,0x5454,0x8575,0x7585,0x7575,0x7554,0xC495,0x7575,0x7554,0x7554,0x6554,0x7554,0x5485,0x5475,0xB485,0x6575,0x7554,0x7575,0x8575,0xC475,0xB495,0x7554,0x5485,0x8554,0x94D4,0xD4D4,0x7575,0xD495,0xD4D4,0x7585,0xC454,0xD495,0xD4B5,0xD4C5,0x95D4,0x95C4,0xB5C4,0x95C4,0xC5D4,0xC5E4,0x85B4,0x95B4,0xC4C5,0xD4C5,0x85D4,0x85C4,0x8575,0x7585,0x7585,0xB485,0x7585,0x8585,0x8575,0x5475,0x5454,0x7575,0x8565,0x5475,0x4645,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0xD485,0x7595,0x5475,0x5454,0x7565,0x5475,0x8575,0xC585,0x5485,0x7554,0x5454,0x7575,0x7575,0x7575,0x7554,0x5454,0x5454,0x5454,0x5454,0x6554,0x5454,0x5475,0x5454,0x5475,0x5454,0x5454,0x4454,0x4141,0x4144,0x7F7F,0x4141,0x5441,0x4444,0x6464,0x6464,0x5454,0x5454,0xC4C4,0x6441,0xD4D4,0xD564,0x64D4,0x5454,0xE4E4,0x4141,0x7F7F,0x4141,0x7F7F,0x6441,0xD4D5,0xD464,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D5,0xF4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xD4C5,0xD5E4,0xE4E4,0xD4D4,0xE4E4,0xC4D4,0x7575,0x4141,0x4141,0x4444,0x6F6F,0x6F6F,0x6F6F,0x4141,0x6F6F,0x6F6F,0xB4B4,0x7585,0x8575,0xC495,0x7585,0x6554,0x8585,0x8554,0x6585,0x8575,0x7565,0x8585,0x7565,0x9575,0xB485,0x7575,0x8554,0x5475,0x8554,0x8585,0x5475,0x6554,0x6585,0x5475,0x7554,0x8585,0x85C4,0xB485,0x9595,0x5475,0x95C4,0xC495,0x6585,0x9554,0xD4C4,0x95D4,0xC5D4,0xD4D4,0xC4C5,0xE495,0xD495,0xD4C5,0x8585,0xC495,0xE495,0x95C4,0x95D4,0x85C4,0x8575,0x75B4,0xB485,0xB485,0x8585,0x7575,0x7585,0x6575,0x7554,0x6554,0x9585,0x6585,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4644
.word 0xC485,0x5485,0xD485,0x95C5,0x5454,0x5454,0x8575,0x85B4,0x8565,0xD4D4,0x7585,0x5454,0x5465,0x5475,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5454,0x5454,0x4475,0x4454,0x5454,0x5454,0x5454,0x4441,0x4444,0x4141,0x7F7F,0x4444,0x41E4,0x4444,0x4444,0x4444,0x6444,0x5454,0xD5D5,0x6454,0x64D5,0xC4C4,0x6464,0x6464,0x4444,0x4444,0x6F6F,0x4141,0x6464,0xD4C4,0xD4D4,0xD4D4,0xD4E4,0xC5E4,0xE4E4,0xE4E4,0xE4E5,0xF4F5,0xF4F4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xC5D4,0xD4D4,0xD4D4,0x6464,0x5454,0x8F8F,0x6F6F,0x4141,0x4144,0x7F7F,0x4444,0x6F6F,0x6F6F,0x8585,0x7585,0x8575,0x85B4,0x85C4,0x5454,0xB454,0x8575,0x5475,0x7585,0x8595,0x85B4,0x8585,0x6585,0x9575,0x85E4,0x5475,0x7575,0x8585,0x75B4,0x7554,0x7585,0x5485,0x5475,0x6585,0x5454,0x6575,0x7575,0x8575,0x7575,0xB485,0x7575,0x5454,0xC475,0xC595,0xC5E4,0x95C4,0x95C4,0x95C4,0x95D4,0xC5C4,0xD4C4,0xC495,0xC5D4,0xB5C5,0x95C4,0x95C4,0x75B4,0x8575,0x8585,0x85B4,0x8585,0x7575,0x8575,0x8575,0x5465,0x5454,0x8554,0xC475,0x7585,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646
.word 0x7565,0x5454,0x8575,0xB485,0xD475,0xC495,0x5454,0x5475,0x8575,0x7585,0x7575,0x5485,0x5475,0x5454,0x5454,0x7554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5444,0x4446,0x4446,0x4446,0x4144,0x8F8F,0x4141,0x4141,0x4444,0x4141,0x6454,0x5454,0x5454,0x5454,0x4441,0x8F8F,0x5454,0xD444,0x6464,0x5454,0x4444,0xF554,0xC4C4,0x4444,0x7F7F,0x4141,0x4444,0xD564,0xD464,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xC4D4,0x6464,0x4444,0x7F7F,0x6F6F,0x4141,0x4144,0x6F6F,0x6F6F,0x6F6F,0x7575,0x7575,0x7575,0x8554,0x8585,0x85D4,0x5465,0x8554,0x7554,0x8575,0x7565,0x8585,0xB475,0x5485,0x7575,0x7554,0xE485,0x85D4,0x7575,0x7554,0x7575,0x7575,0xC595,0x8585,0x8585,0x85C4,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x8575,0x8585,0x85C4,0x95B4,0xD4C4,0xD4B5,0xE495,0xC4C5,0x95E4,0xC5C4,0xB5C4,0xC5E4,0x65C4,0xC485,0x5475,0x7575,0x7585,0xB485,0x7585,0x7575,0x7575,0x7575,0x5454,0x5454,0x7554,0xB485,0xD485,0x85B5,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0x5475,0x7575,0x6575,0x7575,0x8554,0x85B4,0xC485,0x8595,0x7554,0x7575,0x7554,0x5475,0x5454,0x5454,0x5475,0x5454,0x5454,0x5454,0x4444,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4141,0x4444,0x4444,0x4141,0x4444,0xD4D5,0x5454,0x4444,0x4444,0x7F7F,0x4444,0x4444,0x6441,0x7575,0x5454,0x5454,0x6464,0x6464,0x7F7F,0x7F7F,0x4444,0x6464,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4E4,0xD4C5,0xD4E4,0xD4E4,0xD4D4,0x64D4,0x6464,0xAFAF,0x4444,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7575,0x8575,0x8575,0x7575,0x85B4,0xC485,0x5485,0x5454,0x5454,0x8565,0x5475,0x6575,0x7575,0x5475,0x5475,0x6575,0x8554,0xD495,0xD4D4,0x7585,0x7554,0xE485,0xC495,0xC485,0xC495,0xD4B5,0x8595,0x7585,0x5475,0x5475,0x5475,0xC485,0xD4B5,0xD4C5,0xE4C5,0xD4C5,0xD4C5,0xD495,0xC4C5,0xC495,0xD4B5,0xD4C5,0xD495,0xC4C5,0x5475,0x7585,0x7554,0x8575,0x7585,0x8585,0x7575,0x5454,0x5454,0x5454,0x5454,0x7554,0x8585,0x95C4,0xB485,0xD485,0x7585,0x4645,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0xC495,0x5485,0x8585,0x5465,0x5475,0x5475,0x7565,0x6575,0x8585,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4141,0x4444,0x4141,0x4444,0x5151,0x5454,0x5454,0x4141,0x5454,0x4141,0x4141,0x4141,0x5454,0x4444,0x4444,0x5454,0xC4E4,0x4444,0x4141,0x7F7F,0x6F6F,0xD541,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4D5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0x64D4,0x7575,0x5454,0x4141,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7575,0x7585,0x7585,0x5465,0x7575,0x8585,0x95C4,0x65C4,0x5454,0x8575,0x7565,0x7575,0x5475,0x8575,0x6554,0x7575,0x6575,0x8554,0x9595,0xD4D4,0xD4B5,0xB4B5,0xB485,0xC4B5,0xD4B5,0xB5D4,0xC5C5,0xD495,0xC495,0xC485,0x9585,0x95C4,0x95C4,0x95C4,0x95B4,0x95C4,0x95D4,0xD4C4,0xC495,0xB5E4,0xD4D4,0xC5D4,0xC5D4,0x7595,0x8554,0x5475,0x5475,0xB485,0x7585,0x8575,0x7565,0x5475,0x4454,0x5454,0x7554,0x7585,0x8585,0x7585,0x9585,0x95B4,0x85C4,0x7585,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x8585,0x7565,0x95C4,0xC495,0x8554,0x8585,0x5454,0x5454,0x7585,0x5475,0x5454,0x5454,0x5475,0x5454,0x4454,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4141,0x4444,0x4141,0x7F7F,0x7541,0x5454,0x4444,0x4444,0x8F8F,0x4444,0x4444,0x4444,0x4444,0x5454,0x4444,0x6464,0xD454,0x4141,0x6F6F,0x4444,0x4141,0xD454,0x7474,0x64D4,0xD4E4,0xD4C5,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xF4D5,0xE4E4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4C5,0xD4D4,0xD4D4,0x6464,0x6464,0x4444,0x6F6F,0x4444,0x7F7F,0x6F6F,0x4444,0x4444,0x9595,0x75C4,0x7565,0x7585,0x5475,0x7575,0x7565,0x7554,0x5475,0x8554,0x7575,0x5454,0x7575,0x5475,0x7554,0x7554,0x7575,0x6575,0xB485,0xC495,0xB485,0xC485,0xB495,0xB495,0xB5D4,0xD4D4,0xD4B5,0xD495,0x85D4,0xE4B5,0xD4D4,0xD4B4,0xC4C5,0xC4C5,0xC4C5,0xD4C5,0xE495,0xE4C5,0xD4C5,0xD495,0x95D4,0xD4C5,0xD495,0x6595,0x8575,0x7575,0x8554,0x7585,0xB485,0x6575,0x7564,0x7585,0x7554,0x7554,0x7575,0x7565,0x6565,0x7575,0x75B4,0x7575,0x7585,0x7585,0x4554,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646
.word 0x6554,0x5475,0xB485,0x8585,0x8565,0x7575,0x8575,0x6585,0x6554,0x5454,0x5454,0x5454,0x5454,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4141,0x8F8F,0x4444,0x4141,0x5454,0x5454,0x5454,0x4444,0x7F7F,0x4141,0x4141,0x4141,0x4444,0x4441,0x4141,0xD544,0x5454,0x8F8F,0x4444,0x6F6F,0x4444,0xD4D4,0x85D5,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xF4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD5E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x64D5,0x41D5,0x7F7F,0x4444,0x7F7F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x8585,0x7585,0x7585,0x5485,0x7554,0x7585,0x8575,0x8565,0x5454,0x5454,0x7575,0x5454,0x7554,0x7554,0x5454,0x7575,0x7585,0x7575,0x7575,0x7575,0x7575,0x7575,0x7575,0x8585,0xD4D4,0xE4B5,0x95C4,0xC5E4,0xC5D4,0x95C4,0xB5D4,0xB5C4,0xD4D4,0x95D4,0xD4D4,0x95D4,0xC4D4,0xD495,0xC495,0xC5D4,0x95D4,0x85D4,0xC5B4,0x5475,0x7554,0x5475,0xC475,0x8575,0x7575,0x5454,0x7575,0x5475,0x8575,0xD495,0x7585,0x4446,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD495,0x8585,0x5465,0x5475,0x5465,0x5475,0x7554,0x5475,0x6475,0x5454,0x5454,0x5454,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4141,0x4444,0x4441,0x7F7F,0x4444,0x5454,0x6464,0x5454,0x4141,0x4144,0x4444,0x7F7F,0x4444,0x4141,0x4444,0x4444,0x5441,0x5454,0x8F8F,0x6F6F,0x6F6F,0x5454,0xC4C4,0xD474,0xD4D4,0xD4D4,0xE4D4,0xC5E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xD5E4,0xE4E4,0xE4E4,0xE4D4,0xE4C5,0xE4D4,0xD4C5,0xD4D4,0x7474,0x5464,0x5454,0x6F6F,0x7F7F,0x4444,0x6F6F,0x7F7F,0x6F6F,0x6541,0x5475,0x5475,0x7575,0x5454,0x8575,0x7585,0x8575,0x7585,0x5454,0x5454,0x8585,0x7575,0x7575,0x8575,0x7585,0x8585,0x5485,0x7575,0x7554,0x8585,0x8585,0xC485,0xB485,0xB495,0xD495,0xC4B5,0xD4C5,0xB5D4,0xC5D4,0x95D4,0xD4D4,0xD4B5,0xC495,0xC5C5,0xC5E4,0xC5E4,0xC5E4,0xC5E4,0xD4D4,0xB4B5,0x8575,0x75D4,0x6554,0x8575,0x5475,0x8575,0xB485,0x5454,0x5454,0x7585,0x5475,0xB485,0xC495,0x75B4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446
.word 0x8585,0x75B4,0x8575,0x8585,0x7575,0x8575,0x6585,0x7554,0x5454,0x5454,0x5454,0x4654,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4141,0x4141,0x7F7F,0x4141,0x5441,0x6464,0x4444,0x4444,0x8F8F,0x7F7F,0x4141,0x7F7F,0x4141,0x4444,0x4141,0x4444,0x51D5,0x6464,0x7F7F,0x7F7F,0x7F7F,0x6441,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D5,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD5D5,0x4154,0x7F7F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x4444,0x7575,0x7575,0x5454,0x5444,0x5454,0x7554,0x75C4,0x7585,0xB485,0x7585,0x5454,0x5454,0x5454,0x7554,0x5475,0x6575,0x7575,0x6575,0x7554,0x7575,0x7565,0x7565,0x5475,0x8575,0xD495,0x95D4,0xD4C4,0xB495,0xD485,0xD4D4,0xD4C5,0xD4C5,0xB5D4,0x85C4,0x85C4,0x85B4,0x85B4,0x85B4,0x75B4,0x7575,0xE485,0xB495,0x5454,0x5475,0xC454,0x5485,0x8575,0x8575,0x7575,0x8575,0x7575,0x7554,0xC485,0xB495,0x85D4,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5444,0x5454,0x5454,0x4454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646
.word 0x7575,0x5475,0x9585,0xD4C4,0x5485,0xB485,0x7585,0x5454,0x6554,0x5454,0x4454,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4144,0x4444,0x7F7F,0x4444,0x4141,0x5454,0x4444,0x5454,0x4144,0x7F7F,0x4444,0x4444,0x4444,0x7F7F,0x4444,0x4141,0x6464,0x4444,0x6F6F,0x6F6F,0x4444,0xD541,0xD574,0xD474,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4C5,0xD5D4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x4164,0x9F9F,0x4144,0x4141,0x6F6F,0x4444,0x7F7F,0x4141,0x8544,0x7575,0xD485,0x8595,0x7585,0x5454,0x8575,0x7575,0x7575,0x8575,0x7585,0x4475,0x5454,0x7575,0x8575,0x95C4,0x8585,0x7554,0x8585,0x8575,0x8585,0x8585,0xB485,0xB495,0x7565,0x8585,0xC495,0xC5C5,0xB4B5,0xD495,0xB5D4,0x95C4,0x95C4,0x95D4,0x85B4,0x7575,0x7575,0x8575,0x8585,0xC495,0x7595,0x5465,0x5454,0x5454,0x8575,0x54B4,0x8565,0x85B4,0x8585,0x75C4,0x5475,0x8554,0x94C4,0x95B4,0xB495,0x6585,0x4646,0x4446,0x4646,0x4646,0x4644,0x4446,0x6544,0x5454,0x7575,0x8585,0x9595,0xD4C5,0x9595,0x95B5,0xB595,0xC5B5,0xB595,0x9595,0x7575,0x6475,0x5454,0x7454,0x4446,0x4646,0x4646,0x465F
.word 0x95D4,0x8595,0xB475,0xB495,0x6585,0x7575,0x5454,0x5454,0x5475,0x4454,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x7F7F,0x4141,0x4141,0x4141,0x5441,0xC4C4,0x5454,0x4444,0x7F7F,0x4444,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x4444,0x5454,0x4444,0x7F7F,0x6F6F,0x6F6F,0xD564,0xC4C4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xD474,0x54D5,0x6464,0x4141,0x7F7F,0x7F7F,0x6F6F,0x4444,0x7F7F,0x7575,0x7575,0x8575,0x85C4,0x95B4,0x75C4,0x5454,0x9575,0x7585,0x6575,0x7575,0x7585,0x7554,0x8585,0x5475,0x7554,0x7575,0x8575,0x6575,0x5475,0x6575,0x7575,0x9585,0x95C4,0xC495,0x7595,0x7575,0x8554,0x85B4,0x7575,0x7575,0xB485,0xB495,0xC485,0x8585,0x9585,0xC5C4,0x95C4,0x85B4,0x5465,0x5454,0xC495,0xC495,0x5475,0x7575,0x8585,0x5465,0x7575,0x7585,0x5465,0x6554,0xC485,0xC495,0xB495,0x9585,0x85C4,0x5475,0x4646,0x4646,0x5444,0x8565,0x8585,0xC595,0xB595,0xD495,0xB495,0xB485,0x8585,0x6575,0x6575,0x7575,0x6565,0x7575,0x7565,0x8585,0x8585,0x8585,0x8585,0x9595,0x7575,0x5454,0x4665
.word 0xB485,0xB485,0x7554,0x7554,0x5454,0x7554,0x5454,0x6554,0x4454,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4141,0x4444,0x7F7F,0x4444,0x4444,0x6464,0x5454,0x5454,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x4444,0x6441,0x4444,0x4141,0x6F6F,0x4444,0x4141,0x64C4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xD5D4,0xE4E4,0xD4E4,0xD4E4,0xE4C5,0xE4D4,0xD4D4,0xD4D4,0x6464,0x4444,0x4444,0x4444,0x7F7F,0x7F7F,0x4141,0x7565,0x7575,0x8575,0x9585,0x95C4,0x85D4,0x5475,0x5444,0x9575,0x7575,0x8575,0x5465,0x7554,0x7575,0x8575,0x5475,0x5475,0x5454,0x5475,0x5475,0x8575,0x7585,0x85B4,0xB485,0x8585,0x85B4,0x95C4,0x85D4,0x7585,0x8585,0xB485,0x7585,0x7575,0x7575,0xB485,0xC485,0xC485,0x7575,0x5454,0x4641,0x5444,0x95E4,0x95B4,0x5475,0x7575,0x8575,0x8585,0x5465,0x5454,0x7554,0x8585,0x8585,0x85B4,0x8585,0x85C4,0x7585,0x7585,0x5454,0x7554,0x8575,0xB595,0x95D4,0x85C4,0x75B4,0x7575,0x5454,0x5454,0x6554,0x4444,0x4446,0x4446,0x4446,0x4446,0x4646,0x5444,0x6554,0x7554,0x7575,0x7575,0x8585,0x8585,0x7585
.word 0x5454,0x5465,0x8575,0x95C4,0x95C4,0x7565,0x6554,0x6564,0x4154,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4141,0x4141,0x4444,0x7F7F,0x4141,0x7575,0x4444,0x8F8F,0x7F7F,0x4444,0x4141,0x4444,0x4141,0x4444,0x4141,0x4444,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5454,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4C5,0xE4D4,0xE4E4,0xD5E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4C5,0xE4D4,0xD4D4,0x64D4,0xC4D4,0x4444,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x4444,0x8595,0x9595,0x85D4,0x75B4,0xC485,0x8585,0x95B4,0x7585,0x5454,0x7554,0x5475,0x5475,0x5454,0x7575,0x7575,0x5465,0x8575,0xB485,0x7554,0x7575,0x5475,0x7554,0x7585,0x8585,0x5475,0x4646,0x4444,0x6554,0x8585,0xB475,0x7585,0x7585,0x8585,0x8585,0x7575,0x5475,0x4654,0x4644,0x4646,0x4646,0x4446,0xD4D4,0xB495,0x4454,0x5475,0x8554,0x95B4,0x95E4,0x85C4,0x7585,0xB485,0x8585,0xC485,0x95B4,0x85B4,0x95B4,0x95D4,0xC495,0x9595,0xB495,0xB485,0x7554,0x7565,0x5454,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x5454,0x7565
.word 0x95D4,0x95C5,0x7554,0x8585,0xB485,0x5464,0x5454,0x5454,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4141,0x4444,0x7F7F,0x7F7F,0x4141,0x6454,0xD5D5,0x4441,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x5444,0x7F7F,0x4444,0x6F6F,0x6F6F,0x6441,0x64D5,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD5D4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x64D4,0x51D5,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4444,0x7541,0xE4D4,0xF4D4,0xC485,0x5485,0x7575,0x8565,0xB485,0x6585,0x5475,0x7575,0x8585,0x5475,0x7554,0x5475,0x7575,0x8575,0x7575,0x7554,0x6475,0x7585,0x5475,0x8565,0xE4C5,0x85B5,0x4665,0x4646,0x4646,0x4446,0x4444,0x4454,0x4454,0x5454,0x6575,0x6575,0x5454,0x4646,0x4644,0x4646,0x4646,0x4646,0x5446,0x9595,0x5475,0x4644,0x4644,0x6554,0x8575,0xD4C4,0xD4B5,0xC495,0xD4C5,0xC5B5,0xC595,0xA5D4,0xC4C5,0x8585,0x7575,0x8575,0xB485,0x7565,0x5454,0x5454,0x4646,0x4446,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646
.word 0xC485,0xB485,0x5475,0x7575,0x7575,0x5454,0x5454,0x4454,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4141,0x4141,0x7F7F,0x4444,0x7F7F,0x5454,0x5454,0x4141,0x4141,0x4141,0x4444,0x6F6F,0x4444,0x7F7F,0x6F6F,0x4444,0x5454,0x7F7F,0x7F7F,0x7F7F,0x4444,0xD454,0xC4C4,0x64D4,0xE4D4,0xD4D4,0xE4C5,0xE4E4,0xD4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xD5D4,0xE4E4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x54C4,0x4154,0x4444,0x7F7F,0x7F7F,0x4141,0xE4E4,0xD4E4,0xD4D4,0x75E4,0x8565,0x5444,0x7585,0x8585,0x95C4,0x7585,0x7575,0x7575,0x5475,0x5454,0x7575,0x7585,0x8575,0x7575,0x7575,0x7575,0x5465,0x8554,0xD495,0x95D4,0x5475,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x8575,0x8585,0x5475,0x4641,0x4644,0x4646,0x4646,0x4646,0x4644,0x8546,0x5475,0x4646,0x4646,0x4646,0x4646,0x5444,0x7575,0x7554,0x6575,0x6575,0x6575,0x6575,0x6575,0x7575,0x7565,0x5465,0x5454,0x5454,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x465F
.word 0xB485,0x8585,0x7554,0x6554,0x7575,0x5454,0x5454,0x4154,0x4646,0x4646,0x4646,0x4644,0x4446,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4141,0x4141,0x7F7F,0x4141,0x4444,0x5454,0x4141,0x4444,0x4144,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x4441,0x9F9F,0x7F7F,0x4444,0x6F6F,0x4141,0xD464,0xD4C4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E5,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0x5454,0x4444,0x7F7F,0x6F6F,0x7F7F,0xF444,0xD4D4,0xD451,0x54F4,0x5454,0x5475,0xD454,0x8575,0x7575,0x7575,0x8554,0x8585,0x5475,0x5454,0x5485,0x8575,0x7554,0x8585,0x8585,0x8585,0x5454,0xD485,0x95D4,0x7585,0x4654,0x4644,0x4646,0x4646,0x4644,0x4646,0x4644,0x5444,0xE4C5,0x85D4,0x4475,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x5446,0x4645,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4654,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646
.word 0x5465,0x7575,0x5485,0x8575,0x7585,0x6554,0x5454,0x4654,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4141,0x4641,0x6F6F,0x4444,0x7F7F,0x4141,0x4141,0x5454,0x4444,0x8F8F,0x4444,0x4444,0x6F6F,0x4444,0x7F7F,0x7F7F,0x4141,0x4444,0x6F6F,0x6F6F,0x7F7F,0x5454,0xD4D5,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D5,0xD4E4,0xE4D5,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0x8585,0x64D4,0x6464,0x6F6F,0x6F6F,0x4444,0x6F6F,0xE4E4,0x6454,0xD4D4,0x85E4,0x8585,0x6585,0x7554,0xB485,0x7554,0x7575,0x7575,0x85B4,0x7575,0x5454,0x6575,0x8554,0x6585,0x6575,0x8565,0x5475,0x6554,0x5454,0x5454,0x4554,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x8544,0x6585,0x4454,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4654,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x8554,0xB495,0xB495,0x8575,0x75B4,0x5454,0x5454,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4141,0x7F7F,0x4444,0x7F7F,0x4141,0x4444,0x5454,0x5454,0x4141,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4444,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6444,0xC4C4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xE4D5,0xD5E4,0xE4E4,0xC5E4,0xD4E4,0xD4D4,0xD4D4,0xD5D4,0x5454,0x4141,0x4444,0x6F6F,0x7F7F,0x5441,0x6464,0xE464,0x95E4,0x85B4,0x85D4,0x5454,0x8554,0x7585,0x7575,0x7575,0x7575,0x7575,0x4475,0x7554,0x5475,0x6465,0x8575,0x7585,0x5475,0x8575,0x6585,0x7554,0x7585,0x4464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5454,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5465,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7554,0x8585,0x85B4,0x7554,0x7554,0x5475,0x5454,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4444,0x4444,0x4141,0x4444,0x7F7F,0x4141,0x7F7F,0x4444,0x4141,0x4444,0x6F6F,0x4141,0x8F8F,0x7F7F,0x4444,0x6F6F,0xD564,0xD474,0xD4D4,0xD474,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4C5,0xE4D4,0xE4D4,0xD4D4,0xE4D5,0xD4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x64D4,0x64D4,0xC4C4,0x5454,0x6F6F,0x6F6F,0x4141,0x5454,0xD5D5,0xE464,0x85E4,0x8575,0x85C4,0x75C4,0x6554,0x8585,0x7585,0x7554,0x7575,0x7585,0x8575,0x5454,0x5454,0x5454,0x5475,0x7565,0x8575,0xB485,0x5475,0x5475,0x8575,0x84C4,0x7585,0x5464,0x4645,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x7544,0x7575,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646
.word 0x6585,0x7585,0x8585,0x8565,0x5485,0x5475,0x5454,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4444,0x4141,0x4444,0x4141,0x5454,0x5454,0x8F8F,0x4441,0x7F7F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4444,0x4444,0x4444,0x7F7F,0x6F6F,0xD464,0xC4C4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4C5,0xD4D4,0xC5D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xC5D4,0xD4D4,0xD4D4,0xC4C4,0x8585,0x4444,0x7F7F,0x6F6F,0x4144,0x5454,0xD4C4,0xC4C4,0x75F4,0x7575,0x8575,0x8585,0x5475,0x6554,0xC485,0x8595,0x8575,0x8554,0xD485,0x7585,0x5475,0x4454,0x7554,0x7585,0xB485,0x7575,0x7575,0x5475,0x7554,0x8554,0x9585,0x95A5,0x7585,0x6575,0x4445,0x4444,0x4444,0x4444,0x6544,0x8575,0x7585,0x7575,0x75A4,0x7564,0x4644,0x4646,0x5454,0x7575,0x4664,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4454,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x5F46
.word 0x8585,0x6575,0x7554,0x7554,0x5475,0x5454,0x5454,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4146,0x7F7F,0x4444,0x4444,0x7F7F,0x4646,0x4441,0x5454,0x5454,0x4141,0x7F7F,0x4444,0x7F7F,0x4141,0x4444,0x6F6F,0x4141,0x5454,0x7F7F,0x7F7F,0x7F7F,0x5454,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4C5,0x5464,0x4444,0x5454,0xC4C4,0xD564,0xD564,0x5464,0x7F7F,0x4444,0x7F7F,0xD564,0xD464,0xE464,0x95C4,0x8585,0x7575,0xB485,0x8585,0x6585,0x5454,0x5465,0x8575,0x75B4,0x5475,0xD485,0xC4B5,0x5485,0x5454,0x8575,0x7575,0x7575,0x5475,0x5454,0x7575,0x7575,0x7565,0x7554,0x5465,0x5475,0x7554,0x7575,0x7575,0x7575,0x7575,0x6575,0x7575,0x7575,0x5464,0x5465,0x9595,0x7564,0x5454,0x5454,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x6565,0x5475,0x4646,0x4646,0x4646,0x4646,0x4446,0x6544,0x7575,0xA595,0x6585,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xB485,0x7554,0x8585,0x7575,0x5475,0x5454,0x5454,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4141,0x4646,0x7F7F,0x4141,0x4444,0x4444,0x5454,0x4444,0x4444,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x5454,0x7F7F,0x7F7F,0x7F7F,0x5441,0xC4C4,0x8585,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4E4,0x64D4,0x5454,0x8F8F,0x6F6F,0x4141,0x5441,0xC4C4,0x6464,0x5454,0x7F7F,0x4141,0x5454,0xD4D4,0xE4E4,0xE4D4,0x5485,0xB475,0xC4B5,0x9585,0xD4D4,0xC4B5,0x5485,0x8575,0x95C4,0x7575,0x8575,0x8575,0x95C4,0x54C4,0x7554,0x7575,0x8575,0x7554,0x8585,0x7585,0x6554,0x7554,0x7554,0x7554,0x6585,0x7585,0x7575,0x7575,0x7585,0x7575,0x8575,0x7585,0x7554,0x7575,0x7575,0x5475,0x5475,0x8575,0x85A5,0x5475,0x4454,0x4646,0x4646,0x4644,0x4646,0x4646,0x8565,0x9584,0x4645,0x7565,0x8554,0x74B5,0xA574,0xA595,0x8585,0x8585,0x5485,0x4446,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0x7575,0x8575,0x8575,0x75B4,0x5475,0x5454,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4141,0x4646,0x4141,0x7F7F,0x4646,0x4441,0x4444,0x8F8F,0x4154,0x4444,0x6F6F,0x4444,0x4141,0x4444,0x7F7F,0x7F7F,0x4444,0x7F7F,0x6F6F,0x4444,0x7F7F,0x6F6F,0x4141,0x5454,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4E4,0x54D4,0x4141,0x4141,0x6F6F,0x4444,0xD4C4,0xE5D4,0xD4D4,0xD4D4,0x64D4,0x6464,0x5454,0x7F7F,0x7F7F,0x6464,0xD4D4,0xE485,0xD4D4,0x95C4,0x75C4,0x5454,0x4454,0x6575,0x8585,0xC485,0xC495,0x7585,0x8575,0x8575,0x95B4,0xC485,0x5485,0x5454,0x7585,0x7575,0x5475,0x7575,0x5475,0x5454,0x7554,0x8575,0x85C4,0x7585,0x8585,0x9585,0x8585,0x7575,0x7575,0x8585,0x85B4,0x7585,0x7575,0x7585,0x7585,0x7575,0x5475,0x5475,0x7575,0x9585,0x6485,0x5475,0x4644,0x4644,0x7544,0x8575,0x75B4,0x9585,0x7554,0x5465,0x5475,0x5454,0x5454,0x5454,0x4645,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646
.word 0x95E4,0x5475,0x7575,0x8575,0x5485,0x5454,0x5454,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4444,0x7F7F,0x4444,0x4141,0x6F6F,0x4444,0x5454,0x4141,0x7F7F,0x5454,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4141,0x4141,0x6F6F,0x4444,0x6F6F,0x64C4,0x6464,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xD454,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0x5454,0x6F6F,0x4141,0x7F7F,0x6464,0xE4D4,0xE4E4,0xE4E5,0xE5E5,0xE5D4,0xD4D4,0x5454,0x54D5,0x4444,0x4441,0xC4C4,0xD464,0xD4D4,0xE4E4,0xC495,0xC585,0xC5D4,0x7575,0x5454,0xD475,0x8585,0x8585,0x7575,0x7575,0x8575,0xB485,0x8575,0x7575,0x5444,0x8575,0x7575,0x7554,0x5475,0x8554,0x7575,0x5485,0x7554,0x8575,0x7575,0xB485,0x8575,0x7565,0x8585,0x8585,0x7575,0x5475,0x8575,0x8585,0xD495,0xC495,0x5485,0x5475,0x7575,0x5475,0x6465,0x7565,0x9585,0x85B5,0x8595,0x7575,0x9575,0x7595,0x5465,0x5454,0x5475,0x4454,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0xB485,0x7554,0x7554,0x8585,0x54B4,0x5454,0x5454,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x6F6F,0x4141,0x4444,0x7F7F,0x4644,0x4646,0x4441,0x5454,0x7F7F,0x4154,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4141,0x7F7F,0x7F7F,0x4444,0x6F6F,0x6F6F,0x4141,0x5454,0x5454,0x64D4,0x5454,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6451,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0x4151,0x6F6F,0x6F6F,0x7F7F,0x4444,0x8554,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x8585,0x6464,0x5454,0x6464,0x4141,0x6464,0xD4D5,0xE4D4,0xE464,0xD4D4,0x9585,0x85C4,0x8565,0x7585,0x7585,0x5454,0x7565,0x9575,0x95C5,0x85C4,0x7575,0x8585,0x7585,0x8565,0x4475,0x7554,0x8585,0x7575,0x5475,0x8575,0x8565,0x7585,0x5475,0x7554,0x7565,0xC495,0x8575,0x7575,0xB485,0xB495,0x8575,0x7575,0x7575,0xB485,0x7575,0x7575,0x7585,0x7575,0x8585,0x7575,0x7575,0x5485,0x6565,0x7575,0x7585,0x5475,0x8575,0x95B4,0xC495,0x85B5,0x7585,0x7585,0x4654,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7575,0x9575,0x75C4,0x7554,0x5475,0x5465,0x5454,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4141,0x4444,0x7F7F,0x4141,0x4646,0x4444,0x4141,0x4444,0x7F7F,0x5454,0x4144,0x7F7F,0x4444,0x7F7F,0x7F7F,0x6F6F,0x4144,0x7F7F,0x4444,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x5441,0x5454,0x5454,0x7F7F,0xC444,0xD564,0xD441,0xD564,0xD474,0xE4D4,0xE4C4,0x85E4,0x95B4,0x95C4,0x85C4,0x75B4,0x7575,0x8585,0x5475,0x5454,0x7565,0x8585,0x7575,0x7575,0x7565,0xC495,0x5485,0x5454,0x8565,0x7575,0x7585,0x7575,0x7575,0x5465,0x8585,0x8575,0x7575,0x7575,0x7585,0x7595,0x8575,0x75C4,0x7575,0x7575,0x7575,0x7585,0x7575,0x8585,0xC485,0x8585,0x75B4,0x8575,0x8554,0x7585,0x6475,0x7554,0x5475,0x8575,0x7575,0x6585,0x8575,0x85C4,0x85B4,0x85C4,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x465F
.word 0x5454,0x7585,0x7585,0x7565,0x5454,0x5464,0x5454,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x6F6F,0x4141,0x4444,0x7F7F,0x4444,0x4646,0x4444,0x4441,0x7F7F,0x4154,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4141,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x6F6F,0xD4D4,0xD4D4,0xD4D4,0x74D4,0x6464,0x4444,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x5454,0xD4D4,0x6464,0xD454,0xD4D4,0xD4D4,0xE4E4,0xE464,0x75D4,0x6575,0x6575,0x8585,0x9585,0x95C4,0x85B4,0x8575,0x7575,0x7575,0x8575,0x7575,0x5475,0xC485,0x8585,0xB485,0x5475,0x5454,0x6554,0x7575,0x7575,0x7554,0x5454,0x5454,0x4454,0x5454,0x5454,0x5454,0x5454,0x7554,0x6465,0x7554,0x5475,0x5475,0x5475,0x5454,0x5454,0x5475,0x7575,0x8575,0xB495,0xB485,0x5475,0x7575,0x8585,0x85B4,0x7585,0x7575,0x7575,0x7575,0x8585,0xB485,0xB495,0x4654,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646
.word 0x8554,0x6554,0x7554,0x9585,0x5475,0x5465,0x7554,0x4654,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4444,0x7F7F,0x7F7F,0x4444,0x4646,0x4646,0x4441,0x4141,0x7F7F,0x5454,0x4444,0x7F7F,0x7F7F,0x6F6F,0x4444,0x6F6F,0x4444,0x7F7F,0x6F6F,0x4141,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x4F4F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0xD464,0xD4D4,0xD4D4,0xD4D4,0x6464,0x4444,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xFCFC,0x5F5F,0x5F5F,0x6F6F,0x5F5F,0x4F4F,0x5F5F,0xC441,0xC4D4,0xC4C4,0xD454,0x64D4,0xE4D4,0xD4E4,0xE4D4,0x4664,0x5441,0x7554,0x6554,0x5465,0x5475,0x8585,0xC5C4,0xD4C5,0xE495,0x84D4,0x85D4,0x8554,0x95D4,0x75B4,0x7575,0x8585,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x4644,0x4446,0x4646,0x4646,0x4146,0x5454,0x5454,0x5475,0x7554,0x5454,0x5454,0x7554,0x5454,0x6554,0x5475,0x5454,0x7575,0x8585,0x8585,0x8585,0xB485,0x7554,0x7554,0x8575,0x8575,0x7575,0x8575,0x85D4,0x7575,0x5465,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x85C4,0x54C4,0x7575,0x8575,0x75C4,0x5475,0x5454,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4141,0x4646,0x4141,0x7F7F,0x4444,0x4146,0x4141,0x4444,0x7F7F,0x5454,0x4141,0x7F7F,0x4444,0x6F6F,0x7F7F,0x6F6F,0x4141,0x4141,0x7F7F,0x4444,0x6F6F,0x7F7F,0x6E6E,0x406E,0x7E6E,0x9D9D,0xBFBF,0x5F5F,0x4141,0x7F7F,0x6F6F,0x6441,0xD4D4,0xD4D4,0xD4D4,0x6464,0x6F6F,0x4444,0x6F6F,0x5F5F,0x5454,0x7E7E,0x7F7F,0x6E4C,0xDCDC,0x9F9F,0x6F6F,0x4F4F,0x4444,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD464,0xD4D4,0xE4D4,0xD4D4,0x4646,0x4446,0x4646,0x4644,0x4146,0x5454,0x7554,0x5465,0x5454,0x5454,0x5475,0x6554,0xC485,0x7585,0x7585,0x8585,0x85C4,0xC485,0x5485,0x8554,0x8585,0x6585,0x5485,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x5441,0x5465,0x7565,0x7575,0x85C4,0x95B4,0x85B4,0x85C4,0x85B4,0xC485,0x8585,0xC495,0x8575,0x7575,0x7585,0x4654,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0xC485,0x7575,0x7585,0x7575,0x5485,0x6554,0x5454,0x4654,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4444,0x4444,0x4444,0x7F7F,0x4646,0x4646,0x4444,0x4441,0x4141,0x6464,0x4444,0x7F7F,0x4141,0x7F7F,0x4141,0x6F6F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4141,0x5151,0x8E6E,0xF0F0,0x7E7E,0xEFEF,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0xD444,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x6F6F,0x7F7F,0x5F5F,0x6444,0xDFDF,0x8E6E,0xBDF0,0xFC9D,0x6E6E,0xCFCF,0x7F7F,0x4F4F,0x6464,0xD464,0xD4D4,0xD4D4,0xD4C4,0xE4E4,0xD4D4,0xE4E4,0x5485,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x5454,0x5454,0x7544,0x8575,0x85C4,0x7585,0x7575,0x8575,0x95B4,0x75B4,0x5454,0x8565,0x75B4,0xC485,0x5475,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x5454,0x5454,0x7565,0x8575,0xB485,0xB495,0x8595,0xE495,0x95B4,0x95B4,0x85B4,0x8575,0x5465,0x5475,0x4654,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7575,0x8585,0xC485,0x5485,0x7554,0x5454,0x5454,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4141,0x4646,0x4646,0x7F7F,0x4646,0x4146,0x4444,0x8F8F,0x7F7F,0x6454,0x8F8F,0x4141,0x7F7F,0x4444,0x6F6F,0x4444,0x4444,0x7F7F,0x4444,0x4141,0x6F6F,0x5F5F,0x5441,0xF474,0xF4E4,0xF4E5,0xD4E4,0xB465,0x7F7F,0x4A4A,0x6F6F,0xD441,0xE4E4,0xE4E4,0xD4E4,0x6464,0x4144,0x8F8F,0x4444,0xD464,0xDFDF,0xFEF0,0x6E9E,0xE06E,0x6464,0x8F8F,0x5F5F,0x5454,0xD4D4,0xE4E4,0xE4E4,0x64D4,0xE4D4,0xD4F4,0x64E4,0xD4E4,0x4544,0x4644,0x4644,0x4444,0x4444,0x4444,0x4154,0x6544,0x7554,0x5475,0x8575,0x9585,0xB4C4,0xC4B5,0x8585,0x7575,0x7575,0x5454,0x5454,0x5454,0xD485,0x8575,0x7575,0x4485,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x6454,0x9575,0x75C4,0x5454,0x7554,0x8575,0x8585,0x85C4,0x85B4,0x7585,0x7585,0x6575,0x7575,0x5475,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7585,0x7585,0x9585,0x7575,0x7585,0x7554,0x5454,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4441,0x4646,0x4646,0x4141,0x4646,0x4646,0x4141,0x4454,0x4141,0x5454,0x4444,0x7F7F,0x4444,0x6F6F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4441,0x4141,0x4F4F,0x4F4F,0x5F5F,0x4141,0x5454,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0xD444,0xE4E4,0xE4E4,0xD4E4,0x41D4,0x4141,0x8F8F,0x4444,0x4444,0x5454,0x7464,0x7474,0x5464,0x8F8F,0x5F5F,0x5454,0x6464,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0x85E4,0xD4D4,0x75D4,0x5475,0x5475,0x5454,0x6554,0x7554,0x5465,0x8585,0x8585,0x8585,0x7585,0x4454,0x5454,0x5454,0x5465,0x4454,0x5454,0x5444,0x5475,0x7575,0x6554,0x8595,0x7575,0x8585,0x4475,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x4446,0x6554,0x9585,0x85B4,0x8595,0x5465,0x5475,0x7554,0x6575,0x7575,0x7575,0x7575,0x6575,0x5464,0x5475,0x5454,0x5454,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646
.word 0x75D4,0x7565,0x7575,0x7554,0x7585,0x5475,0x5454,0x5454,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4141,0x4141,0x6F6F,0x4446,0x6F6F,0x4444,0x4146,0x4444,0x5454,0x7F7F,0x7564,0x8F8F,0x4154,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x6F6F,0x4141,0x4141,0x64D5,0xC4C4,0x6464,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4A4A,0x6F6F,0x7F7F,0xD441,0xE4D4,0xE4F5,0xE4E4,0x6464,0x7F7F,0x4A4A,0x7F7F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x5454,0x5454,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0x64D4,0xD464,0xD4D4,0xD4E4,0x8575,0x85C4,0xB485,0x7585,0x7575,0x7575,0x7575,0x7565,0x8575,0x95C4,0x85B4,0x8595,0x7585,0x7554,0x7585,0x8585,0x8575,0x8585,0xB485,0x5485,0x9554,0x75B4,0x8585,0x75B4,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x7554,0x5465,0xB485,0x7585,0xD4B4,0x7595,0x5454,0x6554,0x5464,0x5475,0x5475,0x5454,0x5475,0x7565,0x5454,0x5475,0x7575,0x4464,0x4645,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xD495,0x7585,0x6554,0x7575,0x7575,0x5454,0x7575,0x6465,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4141,0x4141,0x4646,0x4646,0x4141,0x7F7F,0x4646,0x4141,0x4444,0x4444,0x6464,0x4444,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4444,0x6F6F,0x4444,0x4444,0xD4D4,0xD4D4,0x64C4,0x64D5,0x6464,0x64D5,0xC464,0x6464,0x8F8F,0x6F6F,0x4141,0xD454,0xE4E4,0xE4F4,0xE4E4,0x54C4,0x7F7F,0x7F7F,0x5441,0x5454,0x6464,0x6464,0x5454,0x6464,0xE554,0xD4D4,0xE4D4,0xF8D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD464,0xE4E4,0x85D4,0x8585,0x85B4,0x8585,0x85B4,0x7585,0x8575,0x8575,0x8585,0x7575,0x6575,0x7575,0x6575,0x7554,0x8575,0x85B4,0x7575,0xB485,0xB485,0x8585,0x5475,0x85C4,0x7585,0xB485,0x5454,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7565,0x7554,0x7575,0x5454,0x8554,0x9585,0x75B4,0x5475,0x5454,0x5454,0x6465,0x7565,0x5454,0x5454,0x5454,0x5454,0x7554,0x5475,0xC495,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0xC485,0x5485,0x8585,0x9585,0x8575,0x85C4,0x5454,0x5454,0x4665,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4141,0x4444,0x4646,0x4646,0x4444,0x4444,0x4141,0x4444,0x4141,0x4141,0x6464,0x4141,0x4444,0x4444,0x7F7F,0x4444,0x6F6F,0x4141,0x6F6F,0x7F7F,0x4141,0xE4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0x64D4,0xD564,0xD451,0xE4D4,0xF4F5,0xE4E4,0xC4E4,0x6464,0x6454,0xD4D4,0xD4D4,0xD4D5,0xC4D4,0xF9D5,0xD4E5,0xF4D4,0xD4E4,0xF9E4,0xE4E4,0xE4E4,0xD4C5,0xD4D4,0x6485,0xD4D5,0xD4D4,0x4654,0x4646,0x4644,0x4146,0x5444,0x6554,0x7554,0x5454,0x5454,0x7575,0x7585,0x7575,0x8575,0x7585,0x8575,0x8585,0x8575,0x8585,0x7595,0x75C4,0x7554,0x8585,0x95C4,0x5475,0x4644,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0xA454,0x5454,0x5454,0x4446,0x4446,0x8575,0x85D4,0x6465,0x7575,0x5475,0x6465,0x5465,0x5454,0x5454,0x5475,0x8554,0x7575,0x5475,0xD485,0x8575,0x8595,0x4454,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644
.word 0xB485,0x7575,0x8575,0x85B4,0x7575,0xB485,0x7554,0x5454,0x4554,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4141,0x7F7F,0x4646,0x4644,0x4646,0x7F7F,0x4141,0x4141,0x4444,0x4444,0xD554,0x7F7F,0x4141,0x7F7F,0x4444,0x6F6F,0x7F7F,0x4141,0x6F6F,0x4141,0x4444,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD5E4,0xD4E4,0x64D4,0xD4D5,0xE4D4,0xF4F5,0xD4E4,0xD4D4,0xD4D4,0xF4E5,0xE4F9,0xF9F9,0xD4D4,0xD4D4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xE4E4,0xD4D4,0x6464,0xD4D4,0x5464,0x4646,0x4644,0x4646,0x4646,0x4644,0x4446,0x7554,0xC485,0xC495,0x85C5,0x85C4,0x8585,0xB485,0x7585,0x8575,0x75B4,0xB485,0xB485,0xC485,0x4464,0x8554,0x85C4,0x5485,0x4654,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4644,0x4644,0x4646,0x5446,0x8585,0x75B4,0x7585,0x7554,0x5475,0x8575,0x7554,0x8585,0x7565,0x8575,0x8565,0x75C4,0x6585,0x7554,0x7585,0x8565,0x8585,0x4444,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F
.word 0x5454,0x5475,0x7575,0x7575,0x7554,0x5454,0x5475,0x7554,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4444,0x7F7F,0x4646,0x4646,0x4646,0x4444,0x7F7F,0x4444,0x5441,0x9F9F,0x6464,0x4444,0x4141,0x4444,0x4141,0x4141,0x6F6F,0x4444,0x6F6F,0x4444,0x4141,0xD4D4,0xC5E4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xE464,0xE5D4,0xE4F5,0xE4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xC5E4,0xF4F4,0xE4E4,0xE4D4,0xE4D4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x64C4,0x5445,0x4647,0x4644,0x4646,0x4646,0x4646,0x5444,0x8575,0x8585,0x95C4,0xB5C4,0x95C4,0xD485,0xB495,0xC495,0x8585,0xB485,0xB495,0x8585,0x7575,0x5475,0x5454,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7541,0x7575,0x8575,0x8575,0x6465,0x7554,0xC454,0x7575,0x9575,0x8565,0x9575,0x7585,0x7575,0x8585,0x7565,0x5475,0x4646,0x4444,0x5454,0x4444,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646
.word 0x6595,0xE4C5,0x7585,0x8554,0x95C4,0x75C4,0x7585,0x5465,0x5475,0x4654,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4444,0x7F7F,0x4646,0x4646,0x4646,0x4646,0x4141,0x7F7F,0x5441,0x5454,0x6454,0x8F8F,0x4444,0x4141,0x6F6F,0x4141,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4444,0xE4E4,0xE4E4,0xE4C5,0xC5E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xF4F7,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5D4,0xD4D4,0x4545,0x4646,0x4646,0x4646,0x4646,0x4646,0x7545,0xB485,0x8585,0x75B4,0x7575,0x8585,0x7585,0x8575,0x7575,0x7575,0x8575,0x7575,0x8585,0x75B4,0x5475,0x7554,0x5485,0x4654,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4146,0x7565,0x5454,0x8585,0x8575,0x5444,0x5454,0x8575,0x7554,0x8575,0x7575,0x8575,0xD4B4,0x5475,0x8565,0x8585,0x5475,0x4644,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4646,0x4644
.word 0x6585,0x8585,0x85B4,0x8565,0x7585,0x7585,0x7575,0x7554,0x5454,0x5454,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4444,0x4646,0x4644,0x4646,0x4444,0x7F7F,0x4444,0x7F7F,0x4444,0x5454,0x5454,0x4141,0x4444,0x4444,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD574,0xD485,0xF4D4,0xF4F5,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xC5E4,0xD4E4,0xD4D4,0xD4D4,0x4664,0x4446,0x4646,0x4646,0x4646,0x8554,0x7585,0x5454,0x4454,0x4644,0x5441,0x5444,0x6554,0x5475,0x7575,0x7554,0x6575,0x5475,0x4454,0x7564,0x8585,0x6585,0x4654,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x5475,0x4146,0x8565,0x54B4,0x4645,0x4446,0x8554,0x9575,0x5475,0x5454,0x8575,0x8575,0x5485,0x5441,0x8565,0x85C5,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7554,0x7575,0x5475,0x7575,0x7575,0x5465,0x7554,0x5454,0x5475,0x5475,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4146,0x7F7F,0x7F7F,0x4144,0x5454,0x5454,0x5454,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4444,0x4141,0x6F6F,0x4444,0x4444,0xD4D4,0xE4D4,0xE4D4,0xC5E4,0xC5E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xF4D4,0xF4F4,0xD5E4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD5D4,0xE5E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xD4D4,0xC5D4,0xD4D4,0x64D4,0x4646,0x4646,0x4646,0x4646,0x7544,0x7585,0x4454,0x4646,0x4644,0x4646,0x4646,0x4644,0x4446,0x4646,0x5454,0x7575,0x7554,0x7575,0x7575,0x8585,0x75B4,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x5446,0x4454,0x4646,0x8541,0x5485,0x4646,0x4646,0x7541,0x85C4,0x5475,0x4146,0x7554,0x7585,0x75D4,0x4646,0x5441,0xC475,0x4475,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F
.word 0xD475,0x7585,0x7575,0x5475,0x5475,0x9554,0x95C4,0x5485,0x7554,0x5454,0x4654,0x4444,0x4646,0x4646,0x4444,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4444,0x7F7F,0x4141,0x5454,0x4444,0x5454,0x7F7F,0x4444,0x4141,0x7F7F,0x7F7F,0x4141,0x6F6F,0x7F7F,0x4441,0xD464,0xD4D4,0xD4C5,0xD4E4,0xD4E4,0xE4E4,0xC5D4,0xC5D4,0x74D4,0x64D4,0xC4D4,0xD4D4,0xE5D4,0xF4F4,0xE4F4,0xD4D4,0xD4C5,0xE4C5,0xC5D4,0xD4E4,0xD4F4,0xE4E4,0xE4E5,0xE4E4,0xE4D5,0xE4F4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x4654,0x4646,0x4646,0x4444,0x5475,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4444,0x8585,0x95D4,0xB5D4,0x95D4,0x8585,0x5475,0x5454,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4541,0x4446,0x4646,0x7544,0x5475,0x4446,0x4646,0x5446,0x7585,0x4654,0x4646,0x4146,0x8565,0x7585,0x4646,0x4646,0x5441,0x7554,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x8585,0xC485,0xD454,0xD4B4,0xC494,0x9575,0xC5C4,0x75C4,0x7554,0x5454,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4141,0x7F7F,0x4141,0x7F7F,0x4144,0x5454,0x4444,0x4141,0x4444,0x4444,0x4141,0x7F7F,0x4444,0x4141,0x7F7F,0x4444,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD5D4,0xD4D4,0xD464,0xD4D4,0xF4D4,0xF4F4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D5,0xF4E4,0xF4E4,0xE4E4,0xE5E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x64D4,0x4646,0x4646,0x4646,0x7544,0x4454,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x8554,0x85B4,0x7585,0x7575,0x5454,0x4454,0x4654,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x5446,0x4475,0x4646,0x4646,0x4146,0x6575,0x4654,0x4646,0x4646,0x6541,0x7585,0x4446,0x4646,0x4646,0x5441,0x4641,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646
.word 0xB485,0x7595,0x8575,0xC495,0x9585,0x8575,0xC485,0x8595,0x5454,0x5485,0x5465,0x4654,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4141,0x4141,0x4444,0x7F7F,0x4444,0x5454,0x4154,0x7F7F,0x7F7F,0x5454,0x7F7F,0x4444,0x7F7F,0x6F6F,0x4444,0x4141,0xC4C4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xC5E4,0xD4D4,0xD4D4,0xD464,0xD485,0xD4D4,0xD4D4,0xE5D4,0xF5F4,0xE4F4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4C5,0xD4D4,0xD4D4,0x4446,0x4646,0x4444,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x7544,0x7575,0x5454,0x4644,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x4646,0x5454,0x4646,0x4646,0x4446,0x4146,0x8554,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7575,0x5475,0xC485,0x8585,0x7575,0x6554,0xB485,0x6575,0x7575,0x6554,0x7554,0x4454,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4444,0x4646,0x4646,0x4446,0x4646,0x4646,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5454,0x5454,0x7F7F,0x4141,0x7F7F,0x4444,0x4141,0x4141,0x4141,0x7F7F,0x4444,0xC4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0x64D4,0xD5D5,0xC4C4,0xD4D4,0xD4D4,0xF4E4,0xF5F5,0xE4F4,0xD4E4,0xD4C5,0xC5D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0x5464,0x4646,0x4646,0x4546,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x7575,0x5454,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4441,0x4446,0x4446,0x4646,0x4646,0x4441,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4644,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F
.word 0x8595,0x6554,0x7554,0x7565,0x7565,0x7585,0x5475,0x9585,0x85C4,0x5454,0x5475,0x5454,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4141,0x4141,0x4444,0x7F7F,0x4444,0x6464,0x5454,0x4444,0x7F7F,0x4141,0x5454,0x4141,0x7F7F,0x4141,0x6F6F,0x4141,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD464,0xD464,0xD4D4,0xF4D4,0xF5F5,0xE4F7,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D5,0xD5E4,0xE4E4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5454,0x4654,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646
.word 0x85C4,0x9554,0xB5C5,0x8585,0x9554,0x85C4,0xC495,0xB485,0x7585,0x6465,0x7544,0x7554,0x6554,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4141,0x4141,0x7F7F,0x4444,0x6F6F,0x5454,0x5444,0x8F8F,0x4444,0x7F7F,0x4141,0x4444,0x4444,0x6F6F,0x4444,0x4444,0xD4C4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x6464,0x64D5,0xD4D4,0xD4D4,0xE4E4,0xF5F5,0xE4F4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4C5,0xD4E4,0xD4D4,0xD4E4,0x64D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4444,0x4154,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7575,0xC475,0xC485,0xB485,0x8575,0xB485,0x6585,0x5454,0x5454,0x8585,0x5475,0x7554,0x5454,0x4465,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4444,0x4141,0x4444,0x4141,0x4141,0x4141,0x5441,0x5454,0x4141,0x7F7F,0x4444,0x6F6F,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x6464,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD4D4,0x64D4,0xD4D4,0xF4E4,0xF4F4,0xE4F4,0xD4E4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xC5D4,0x64D4,0x6464,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646
.word 0x7575,0x8585,0xB485,0x5475,0x6575,0x5475,0x5475,0xD495,0x6585,0x7554,0x7585,0x5454,0x7575,0x5454,0x4644,0x4446,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4441,0x7F7F,0x7F7F,0x4444,0x7F7F,0x5454,0x5454,0x5454,0x4444,0x7F7F,0x6F6F,0x4444,0x4444,0x4141,0x4444,0x4141,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF5F5,0xE4F5,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4E4,0xD5D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x64D4,0xD564,0x4644,0x4444,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x5454,0x4654,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x5454,0x6565,0x5475,0x7575,0x8595,0x95E4,0x8565,0x85D4,0x7575,0xB485,0x7554,0x6585,0x7554,0x6554,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4141,0x4141,0x4444,0x4141,0x7F7F,0x5454,0x6464,0x4454,0x7F7F,0x7F7F,0x4141,0x4141,0x5454,0x4444,0x7F7F,0x4444,0xD4C4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD4E4,0xD4D4,0xF4E4,0xF4F4,0xE4F4,0xD4E4,0xE4C5,0xD4D4,0xD485,0xD4E4,0xE4C5,0xD4D4,0xE4D4,0xD4E4,0xC5D4,0xD4D4,0xD4E4,0xD474,0x6464,0xC4C4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F
.word 0x5475,0x8585,0x8585,0x7585,0x95D4,0x85B4,0x7585,0x7575,0x8554,0x95D4,0x7575,0x8595,0x5454,0x7575,0x7575,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4141,0x4444,0x7F7F,0x7F7F,0x4444,0x4141,0x7F7F,0x5454,0x6464,0x4444,0x4444,0x6F6F,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x6464,0xD4C4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0x85D4,0xD4D5,0xD4D4,0xE4D4,0xF4E4,0xE4E4,0xE4F4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64D5,0xD5D5,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7554,0xB485,0x7585,0x5475,0xB485,0x7585,0x5454,0x8575,0x75B4,0x7585,0x7554,0xC475,0x7575,0x5454,0x5475,0x5475,0x4465,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x7F7F,0x4141,0x4141,0x4444,0x4141,0x7F7F,0x7F7F,0x5444,0x4444,0x4141,0x6F6F,0x7F7F,0x4444,0x4444,0x7F7F,0x4444,0x7F7F,0xD4C4,0xD474,0xC5D4,0xE4D4,0xC5D4,0xE4E4,0xE4D4,0xD4C5,0x64E4,0x6464,0x6464,0x6464,0xE4D4,0xE4E4,0xD4D4,0x6485,0x6464,0x6464,0xD4E4,0xE4E5,0xE4D4,0xC5D4,0xD4E4,0xE4C5,0xE4D4,0xD4D4,0x74D4,0xD4D4,0x6464,0x46C4,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4444,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7554,0x7585,0x7565,0x7554,0x7575,0x5454,0xD485,0xC485,0x5495,0x7565,0x7554,0x5454,0x5454,0x5454,0x7554,0x7575,0x5454,0x4645,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4444,0x4141,0x4444,0x4141,0x5454,0x4444,0x7F7F,0x4141,0xD454,0x4444,0x7F7F,0x4444,0x7F7F,0x7F7F,0x4141,0x4141,0x5454,0xC4C4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xC5E4,0xD4D4,0x5454,0x4444,0x6464,0x5464,0xC4D4,0x6464,0xC4C4,0x5454,0x5454,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD4C4,0x46D4,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4644,0x4646,0x4646
.word 0x6554,0x5475,0x5454,0x8575,0xB5D4,0x75C4,0x7575,0x8585,0x7575,0x9585,0x85C4,0xC475,0xC495,0x8575,0x5475,0x6554,0x7575,0x5454,0x4575,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4141,0x4444,0x4141,0x6F6F,0x4141,0x4141,0x7F7F,0x4444,0x6464,0x4444,0x4141,0x7F7F,0x4141,0x4141,0x6F6F,0x4144,0x5454,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4C5,0xD4E4,0x41D4,0x4444,0x4441,0x4444,0xD4D4,0xD4D4,0x5464,0x4141,0x4444,0xC4C4,0xF4F4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0x8585,0xD4D4,0x4664,0x4444,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4444,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446
.word 0x5454,0x7554,0x7585,0xC454,0xB485,0x85B5,0x6554,0x6575,0x9554,0xD4C4,0xC4B5,0x8575,0x95C4,0xD475,0x8585,0x5454,0x5475,0x7575,0x7554,0x4454,0x4644,0x4646,0x4644,0x4646,0x4646,0x4141,0x4141,0x7F7F,0x4444,0x4444,0x4444,0x7F7F,0x4441,0x5454,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4141,0xD4D4,0x74D4,0xE4D4,0xC5D4,0xD5D4,0xD4E4,0xE4E4,0xD4D4,0x64D4,0x4444,0x4141,0x4444,0x4144,0x5444,0x4444,0x4444,0x6464,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4C5,0xD4E4,0x74D4,0x64D4,0xD4D5,0xD4D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x464F
.word 0x7554,0x7554,0x7554,0x7554,0x8575,0x75B4,0xC585,0x95C4,0x75D4,0x8585,0x64B4,0x5454,0x5454,0x8575,0x75C4,0x8575,0x5465,0x7554,0x5475,0x6575,0x5475,0x7554,0x6544,0x5465,0x4165,0x4444,0x4444,0x4141,0x7F7F,0x4141,0x5441,0x4141,0x4141,0x5454,0x5454,0x4141,0x7F7F,0x4444,0x7F7F,0x4444,0x4141,0x4444,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x6464,0x5454,0x5454,0xE4C4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xF5E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xC4C4,0x74D4,0xD464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646
.word 0x5454,0x6554,0x7575,0x7554,0x7554,0x5454,0x8575,0x7585,0x6585,0x7575,0x5454,0x9575,0x85C5,0x5465,0x5485,0xD475,0x8595,0x5465,0x8554,0x5475,0x7575,0x5475,0x7575,0x5475,0x5454,0x4141,0x4141,0x4444,0x7F7F,0x4444,0x9F9F,0x4444,0x4444,0x4141,0x6464,0x5454,0x7F7F,0x4141,0x6F6F,0x7F7F,0x4444,0x7F7F,0xC441,0xC4C4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xF4D4,0xC4E4,0x5454,0x64D5,0xE4E4,0xE4F4,0xE4D4,0xE4F4,0xF4E4,0xF4F5,0xE4E4,0xD5E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD464,0xD5D4,0xD4D4,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x5454,0x5454,0x5454,0x7554,0xC495,0xC495,0x7554,0x6575,0x8554,0xC495,0xC495,0xC475,0xB5D4,0xC485,0xB495,0x7554,0x7575,0x5454,0x7575,0x5454,0x7554,0x7585,0x7575,0x7575,0x4141,0x4144,0x4444,0x7F7F,0x4444,0x4141,0x4154,0x4144,0x4141,0x7F7F,0x6444,0x5454,0x4444,0x7F7F,0x4444,0x6F6F,0x4141,0x4444,0xC4C4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xE4D4,0xF5F5,0xE4F4,0xE4D4,0xE4F4,0xF4D4,0xE4F4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xD5E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xC4C4,0x6485,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0x5444,0x5454,0x5475,0x7575,0xC485,0x8585,0x7565,0x5475,0xC475,0xD495,0xD495,0x8554,0x7595,0x7565,0x95D4,0x5485,0x8575,0x7585,0xC554,0x5475,0x5475,0x5454,0x5454,0x5454,0x4141,0x4444,0x7F7F,0x4141,0x7F7F,0x7F7F,0x5454,0x4444,0x4444,0x4444,0x5454,0x41D5,0x4141,0x4444,0x6F6F,0x6F6F,0x4444,0x4141,0x8585,0xC4C4,0xC4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xC4D4,0xB4C5,0xC565,0xE475,0xE4F5,0xF1E4,0x65E4,0x4565,0x6545,0x8465,0xD4D5,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xC4D4,0x64D4,0x44D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4446
.word 0x5454,0x5454,0x5454,0x5454,0x6575,0x4475,0xD485,0xD4C5,0x7595,0x95C4,0x85B4,0x7554,0x7565,0x5485,0xC485,0x7575,0xE495,0xC4C5,0x7575,0x7554,0xC585,0x75C4,0x5454,0x5454,0x4141,0x4444,0x4444,0x4141,0x4444,0x4141,0x4444,0x5464,0x5454,0x7F7F,0x4141,0xC441,0x4141,0x7F7F,0x6F6F,0x4444,0x4141,0x8F8F,0x5454,0x64D4,0x8585,0x74D4,0x64D4,0x5454,0x4444,0x6544,0x4445,0x6544,0x6565,0xC565,0xC575,0x6575,0x4545,0x6545,0x4545,0x4465,0x4545,0x7545,0xB4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD574,0x8585,0xC4C4,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4485,0x5454,0x5465,0x5454,0x5454,0x5454,0xC585,0x8595,0x7575,0x7575,0x7585,0xD485,0xD4D5,0x85D4,0x7575,0x7554,0xC485,0x8595,0x5465,0x7585,0x7565,0x5475,0x8575,0x5454,0x4444,0x4141,0x7F7F,0x5454,0x7F7F,0x7F7F,0x6444,0x5454,0x5454,0x4444,0x4444,0x7F7F,0x6464,0x4141,0x7F7F,0x6F6F,0x4444,0x4141,0x4441,0xC4C4,0xC4C4,0xC4C4,0x4141,0x4444,0x4564,0xC545,0xC565,0x6565,0x65C5,0x65C5,0x4565,0x6545,0xC565,0xC545,0xC5C5,0xC575,0xC565,0x4665,0x4444,0xD465,0xD464,0xD4D4,0x64D4,0xC4C4,0x64D4,0xD4D5,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46
.word 0x6595,0x5454,0x6564,0x8554,0x8585,0x54B4,0x9575,0x75B4,0x5475,0x5454,0x7585,0x9565,0x8585,0x75D4,0x8554,0x85C4,0x7565,0x5465,0xE485,0x85D4,0xD485,0x7585,0x5454,0x4141,0x4141,0x4444,0x4444,0x4141,0x7F7F,0x4444,0x6441,0xD5D5,0x5454,0x4444,0x4141,0x5454,0xC4C4,0x4444,0x4141,0x6F6F,0x4141,0x4444,0x7F7F,0xD564,0xD574,0xD574,0xD4C4,0x4565,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6F6F,0x4A4A,0x4444,0x5445,0x4544,0x4544,0x4444,0x6544,0xD4D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0x6464,0x85C4,0x6564,0x6465,0x6465,0x5454,0x4454,0x4446,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644
.word 0x85D4,0x5444,0x5454,0x7554,0x6575,0x7575,0x7575,0x5475,0xC454,0xD4C5,0x95C5,0x7575,0x8575,0x6585,0xC495,0x8594,0x8575,0x7585,0x8575,0x85D4,0xB5B4,0x85D4,0x4475,0x4141,0x4444,0x4141,0x4141,0x5454,0x4444,0x7F7F,0x4444,0x5454,0x4444,0x4141,0x4444,0x4141,0x4444,0x4164,0x4444,0x6F6F,0x4444,0x4141,0x4444,0x6464,0xC4C4,0xD4C4,0xC4C4,0xD4D4,0x45C5,0x4544,0x84C5,0xC5C5,0xC5C5,0x4445,0x4644,0x4444,0xC544,0xE5D6,0xD5F5,0xC5C5,0x4575,0xD5C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0x6464,0x64D4,0x85D4,0x5475,0x7575,0x7554,0x7554,0x7575,0x6575,0x5475,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646
.word 0x95D4,0x4485,0x5454,0x5454,0x5454,0x5454,0x8585,0x85C4,0x8554,0x85C4,0xB485,0x7554,0x7554,0x8565,0xB5D4,0x85C4,0xC565,0xC495,0x6575,0x5475,0x8575,0x54B4,0x4444,0x4141,0x4444,0x4441,0x7F7F,0x5464,0x5454,0x4141,0x5454,0x6464,0x6464,0x4444,0x4141,0x5454,0x4141,0x8F8F,0x5454,0x5454,0x7F7F,0x4444,0x4141,0x7575,0x64D4,0x64D4,0x64D4,0x85D5,0xC5C5,0xC575,0xD665,0xD5C4,0x7575,0xD5D5,0xF5D5,0xF5D5,0xE4D5,0xB4C5,0xC565,0x6554,0xC5C5,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xC4C4,0xD564,0xD4D4,0x7575,0x7575,0x7575,0x7575,0x7575,0x7554,0x7554,0x5475,0x5475,0x7554,0x4654,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F
.word 0xC495,0x7585,0x4454,0x7554,0x8554,0x5485,0x7575,0x8575,0x7575,0xB485,0x7585,0x9575,0x95C5,0x7575,0xC485,0x6585,0x9585,0x95C4,0x7554,0x7585,0x5454,0x4154,0x7F7F,0x4444,0x4141,0x8F8F,0x4141,0x6464,0x4141,0x7F7F,0x5441,0x7575,0x4444,0x4141,0x4444,0x5454,0x4444,0x4444,0x4141,0x4154,0x7F7F,0x4141,0x5454,0x5454,0xD4D4,0xD4D4,0xC4C4,0xC4C4,0x8585,0x65C5,0x6565,0x6575,0xC5C5,0xD4D4,0xF5E5,0xD5C5,0xC5B4,0xC575,0x7565,0x65C5,0x74C5,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD474,0xD464,0x8585,0x85D4,0xD485,0xD495,0xE4C5,0x8595,0x8585,0x7575,0x5475,0x7554,0x7575,0x7575,0x5475,0x4645,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646
.word 0xB5C4,0x85E4,0x5475,0x5454,0x6575,0x6554,0x5475,0x8575,0x7565,0x5475,0x7554,0xC495,0xC495,0x5485,0x6575,0x5475,0x7575,0x6575,0x9554,0x95C4,0x5465,0x4144,0x4141,0x8F8F,0x4144,0x4444,0x6464,0x4444,0x5454,0x4444,0x7F7F,0xD575,0x5454,0x4454,0x4141,0xC4C4,0x4141,0x7F7F,0x4444,0x4444,0x6464,0x4444,0x4141,0x5454,0x5454,0x6464,0x64D5,0x64D4,0xC4D4,0x5454,0x75D6,0x6545,0x65D5,0xC5C5,0x6575,0x6575,0xC575,0x7565,0x65C5,0xD4D4,0xE4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xC4C4,0x7575,0xD4D4,0x6575,0x5454,0x5454,0x5454,0x5454,0x5454,0x7575,0x8585,0x8575,0x8575,0x7575,0x7554,0x6475,0x6454,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446
.word 0xC495,0xB5D4,0x54C4,0x5454,0x5454,0x5475,0x7554,0x6575,0x5454,0x8554,0x7575,0xC485,0xD485,0x7585,0x5454,0x7575,0x7554,0x5454,0x7575,0x7585,0x4175,0x4444,0x4444,0x4141,0x4444,0x4444,0x4444,0x5454,0x8F8F,0x4141,0x4444,0x6464,0x6464,0x5454,0x4444,0x5441,0x4444,0x4141,0x4444,0x7F7F,0x6444,0x4141,0x7F7F,0x4444,0x5454,0xD5D5,0xC4C4,0xC4C4,0xC4C4,0xC4C4,0x4554,0x65D6,0x54C5,0x6565,0xC565,0x6545,0xC545,0xC554,0x7575,0xD4C4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x64D4,0x64D4,0xD4D5,0xC464,0x5485,0x5454,0x6554,0x8575,0x95C4,0x75B4,0x5454,0x7554,0x8575,0xD495,0x7585,0x7575,0x8575,0x5475,0x6475,0x4654,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x85E4,0xD495,0xB4B5,0x5475,0x5454,0x5454,0x5454,0x5454,0x8554,0xB485,0x54B4,0x7575,0x7585,0x7554,0x8585,0x8585,0x8554,0x75C4,0x5454,0x5465,0x7F7F,0x4141,0x4141,0x7F7F,0x4444,0x5441,0x8F8F,0x5454,0x4444,0x4141,0x4444,0x5454,0x4444,0x4444,0x4141,0x8F8F,0x4444,0xD564,0xD4D4,0x85D4,0x9F9F,0x7F7F,0x4444,0x4141,0x5454,0x6464,0xD4D4,0xD485,0x85D4,0xD464,0x5454,0x5454,0x5454,0x5454,0x4141,0x9F9F,0x5454,0x4141,0xD464,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x64D4,0xD4D4,0xC4C4,0x85C4,0x5475,0x7575,0x8575,0x8595,0x8554,0x85C4,0x7585,0x8554,0x85C4,0x5465,0x5454,0x8575,0x8595,0x7575,0x8575,0x7585,0x5475,0x5475,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46
.word 0x95D4,0x95B4,0xB5C4,0x7585,0x4454,0x6541,0x7554,0x8575,0x7554,0x7575,0x6585,0x7554,0x5454,0x8565,0x85B4,0x75B4,0x8575,0x7585,0x7575,0x5454,0x4141,0x4144,0x4444,0x4444,0x4141,0x5454,0x5454,0x4141,0x4141,0x4444,0x4141,0x4444,0x7575,0x4154,0x4444,0x5454,0x6454,0x64D5,0xD464,0xD4D4,0xD4C4,0x85D4,0x4444,0x7F7F,0x4441,0x5454,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xC4D4,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0xD4C4,0xE4E5,0xD4E4,0xE4D4,0xD4D4,0xD4C5,0xD4D4,0x64C4,0x64D5,0x8585,0x95C4,0x7585,0xD475,0xC4D4,0x5454,0x5475,0x6554,0x5454,0x5454,0x8575,0x7575,0x5454,0x7565,0x8585,0x7585,0x7575,0x7575,0x7575,0x5475,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xB5D4,0x85C4,0x85B4,0x95D4,0x5485,0x5454,0x5475,0x7554,0x7554,0x5454,0x5465,0x8554,0xC485,0x7585,0x7554,0x7554,0x6575,0x5475,0x4141,0x4141,0x4144,0x5454,0x4141,0x4141,0x4444,0x4441,0x4141,0x4444,0x4444,0x4141,0x7F7F,0x5444,0x8F8F,0x4444,0x8F8F,0xD554,0x6464,0x6464,0xC4C4,0x6464,0x64D4,0x64D4,0x64D5,0xC4D4,0xD4D4,0x5454,0x6454,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xD474,0x6585,0xC495,0x7595,0x7554,0x5475,0x7575,0xC485,0x9595,0x7575,0xD485,0xB5D4,0x54B4,0x95D4,0x6585,0x5454,0x8565,0x8585,0x7575,0x7575,0x8575,0x7575,0x4665,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0xC4C5,0xC495,0xD495,0xC5D4,0x85D4,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x7554,0x8575,0x5475,0x8575,0x7554,0x5454,0x4454,0x4141,0x4444,0x4444,0x4444,0x7F7F,0x5454,0x4444,0x7F7F,0x4444,0x4144,0x4444,0x4444,0x6F6F,0x5441,0x4444,0x4141,0x6444,0x6464,0x64D4,0xC4D5,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD464,0xD4D4,0x64D4,0x64D4,0x5454,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xF4F4,0xF4E4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xD4D4,0x74D4,0xD5D5,0xC4C4,0x85B4,0x75C4,0x8585,0x54B4,0x5454,0x8575,0x7585,0x8585,0xD4C4,0x5485,0x9585,0x95D4,0x7585,0xC485,0x7585,0x7554,0x5454,0x8565,0x8595,0x8585,0x8575,0x8575,0x7575,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x464F
.word 0xB485,0x4644,0xD495,0xC495,0xC495,0xC495,0x5475,0x5454,0x7554,0x5475,0x5485,0x6554,0x5454,0x5485,0x7575,0x5475,0x5454,0x4141,0x4144,0x5454,0x5441,0x4141,0x4444,0x4141,0x4141,0x4441,0x8F8F,0x5454,0x8F8F,0x7F7F,0x7F7F,0x4141,0x4444,0x6464,0xD4D5,0xC4D4,0xC4C4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x41D4,0xD464,0xD4D4,0xE4D4,0xD5D4,0xF4E4,0xE4E4,0xF4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x85D4,0x7575,0x5475,0x7554,0x5454,0x7575,0xD475,0xD4B5,0x85D4,0x8575,0x8585,0x5465,0x7565,0x5475,0x5454,0x5465,0x7575,0x8585,0x8565,0x5475,0x6554,0x8585,0x8575,0x7575,0x6554,0x7575,0x4454,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4444,0x4646,0x4646,0x4446,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4475,0x4646,0x9585,0x95C4,0x95C4,0xB5C4,0x85C4,0x5454,0x7554,0x7554,0x5475,0x5475,0x8554,0x7575,0x7554,0x5454,0x4141,0x4444,0x5454,0x5454,0x5454,0x4444,0x7F7F,0x4444,0x4444,0x5454,0x7F7F,0x4441,0x7F7F,0x4444,0x4141,0x4444,0x6441,0xD5D5,0x64D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0x6464,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4F4,0xE4F4,0xC5E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x7585,0xB5C4,0x95C5,0x8575,0xD4D4,0xC4B5,0x8564,0x95B4,0x85D4,0x6554,0x5454,0x7575,0xD485,0xC495,0x7585,0x8575,0xE495,0x85D4,0x7585,0xC495,0x5465,0x7554,0xB485,0x8554,0x8585,0x7575,0x8575,0x4654,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4646,0x8585,0x95D4,0x95C4,0x85B4,0xB485,0xD4B5,0xD4D4,0x6585,0x5454,0x4454,0x5454,0x6575,0x5475,0x5454,0x4444,0x4444,0x4141,0x5454,0x5454,0x5454,0x4444,0x4141,0x8F8F,0x5441,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x5454,0xD5D4,0xD464,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xE4C5,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0x51D4,0xD4D4,0xD5D4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0x74D4,0xD485,0x5495,0xC485,0xE485,0xC485,0xC495,0x85B5,0x8565,0xB485,0x5475,0x9575,0xC5D4,0x85C5,0x8565,0xB5D4,0x95D4,0x8565,0x85C4,0xB495,0x6585,0xC495,0x85B5,0x7575,0x7554,0x8575,0x85B5,0x7585,0x7554,0x5475,0x4664,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0x4646,0x9546,0x95D4,0x95D4,0xD4C4,0x8585,0x95C4,0x9595,0xB5D4,0x5475,0x5454,0x5454,0x6475,0x5454,0x5454,0x4141,0x8F8F,0x4444,0x6444,0x5454,0x5454,0x8F8F,0x4444,0x4144,0x5454,0x4444,0x5454,0x4141,0x4444,0x4141,0x4444,0xD464,0xC4C4,0x64D4,0xD4D4,0xD4D4,0xE4D4,0xD5D4,0xD4E4,0xE4E4,0xE4C5,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x64D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4E4,0x85D4,0xB485,0xB485,0x8585,0x85B4,0x75B4,0x5475,0x5454,0x7575,0x9565,0x95C4,0x85D4,0x8575,0xC495,0x85D4,0x8575,0xB475,0x8585,0x5465,0x7575,0x7575,0x7554,0x5475,0x7554,0xB485,0xB495,0x7585,0x7585,0x7575,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46
.word 0x4446,0xD446,0xC495,0x6585,0x4646,0x9565,0xD4D4,0xD4C4,0xC495,0xE485,0x95D4,0x8585,0x5475,0x4144,0x5444,0x7F7F,0x6464,0x4141,0xD4D4,0x5454,0x4444,0x4141,0x4141,0x4444,0x4141,0x4444,0x7F7F,0x4444,0x7F7F,0x5441,0xC4D4,0xC4C4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xC5D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x64D4,0x6464,0x6464,0x6464,0x6454,0x6464,0x6464,0x64D4,0xD464,0x8575,0x8585,0x8585,0xB485,0x7575,0x5454,0x5454,0x7575,0x5454,0xC485,0xC5B5,0xD4C5,0x8585,0x85B4,0x75C4,0x8565,0x85B4,0x5475,0x6575,0x5454,0x8575,0x8575,0x8585,0x7585,0x8575,0x85D4,0x5475,0x7554,0x9585,0x85C4,0x7575,0x7575,0x7575,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646
.word 0x4646,0xB546,0x85D4,0x4665,0x4446,0x8546,0xD4C5,0xD495,0xC4B5,0x9585,0xD4C4,0xD4C5,0x5485,0x4141,0x9F9F,0x4444,0x5454,0x5444,0x5454,0x4454,0x4141,0x7F7F,0x4444,0x5454,0x4444,0x8F8F,0x4144,0x4141,0x5454,0xD4D4,0xD4D4,0xD4C4,0x64D4,0xD4D4,0xD4C4,0xD4D4,0xD4E4,0xD4E4,0xC5E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xC5E4,0xE4E4,0xC5D4,0xD4E4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x6464,0x4444,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0xC475,0x7585,0x7585,0x95B4,0x5475,0x8554,0x7595,0x8575,0x94D4,0x95D4,0x8575,0xD4D4,0x95D4,0x75B4,0x8585,0x5475,0x5475,0x5454,0x8585,0x8575,0xB5C4,0xD4D4,0x7575,0xD4C4,0xC495,0x8585,0xB5C4,0x85C4,0x5454,0x8575,0x95C4,0x85B4,0x8585,0x7554,0x4575,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4444,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4646,0xC4C4,0x65B5,0x4646,0x4646,0xD446,0x95D4,0x85C4,0x4646,0x8585,0xE495,0x6585,0x4141,0x5444,0x8F8F,0x4144,0x4141,0x5454,0x5454,0x5454,0x8F8F,0x4444,0x5454,0x4444,0x4444,0x4141,0x4444,0x5454,0x64D4,0x64D5,0xD4D4,0xD464,0xD4D4,0xD4C4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D5,0xD5E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0x6464,0x5454,0x5454,0x4454,0x5454,0x4154,0x7575,0x5454,0x5454,0x6554,0x6585,0xD495,0xD4C5,0x8575,0xC495,0xC495,0x8575,0x8585,0x7585,0x7554,0xD485,0x8595,0x8565,0x95C4,0xD4C4,0x54D4,0xC485,0x9595,0x75C4,0x8585,0x85C4,0x54B4,0x7554,0x5475,0x7565,0x5454,0x8565,0x8585,0x75B4,0x7565,0x7585,0x4654,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x4646,0x8546,0x46D4,0x4646,0x4646,0x8544,0x85D4,0x4695,0x4646,0xD475,0x75D4,0x4444,0x5454,0x9F9F,0x4444,0x4141,0x4444,0x6464,0x5454,0x5454,0x4444,0x4141,0x5441,0x5454,0x4141,0x4141,0x4141,0x5454,0xC4C4,0xD464,0xD464,0xC4C4,0x64D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x5454,0x5454,0x5454,0x9F9F,0x5454,0x5454,0x8575,0xC5D4,0xB5D4,0x85C4,0x95C4,0x85D4,0xC475,0xB485,0x7595,0x7575,0x7575,0x5475,0x9575,0xD4C5,0xD4C5,0x7585,0xC485,0x9595,0x75D4,0x8585,0x85B4,0x5485,0xB485,0x8585,0x7565,0x9585,0x95C4,0x85C4,0x6585,0x5454,0x95B4,0x8585,0x5475,0x7585,0x6475,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F46
.word 0x4646,0x4646,0x4675,0x4646,0x4646,0x9546,0x95D4,0x4646,0x4644,0x8546,0x4154,0x7F7F,0x6464,0x4444,0x5454,0x4444,0x4141,0x5454,0x4444,0x4141,0x7F7F,0x4141,0x5454,0x4444,0x4444,0x7F7F,0xD454,0x64D4,0xC4D5,0xD4D4,0xD4D4,0xD485,0xD4D4,0x74D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xF4E4,0xE4E4,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xD4D4,0x64D4,0xD464,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0x64D4,0x7575,0xC4C4,0x6464,0x7554,0xC485,0x7585,0xC485,0xC495,0x7575,0x7575,0x7575,0x5454,0x5454,0x7554,0x5454,0x8565,0xB5C4,0xC475,0x9595,0xC495,0x6585,0x7575,0x8575,0x5465,0x6575,0x5475,0x8565,0x5465,0x6554,0x7585,0xC485,0xC5B5,0xD4B4,0x85D4,0x5475,0x7554,0x9575,0x7585,0x8575,0x7575,0x6585,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4446,0x4646,0x4644,0x4646,0x4646,0x8546,0xB485,0x4644,0x4646,0x4146,0x7F7F,0x5454,0x8F8F,0x4141,0x4141,0x4141,0x5454,0x4444,0x4444,0x8F8F,0x4444,0x4444,0x4444,0x4141,0x4141,0x4444,0x64D4,0xD4D4,0x8585,0x64C4,0x64D5,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD5D4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0x64E4,0x4141,0x7F7F,0x4141,0x4141,0x4141,0xD441,0xD4D4,0xD4D4,0x6464,0x7575,0x5454,0x7575,0xC5C4,0x75C4,0x8585,0x75C4,0x5454,0x5454,0x6554,0xD475,0xD4B5,0x6585,0xB5D4,0x95E4,0xD4C5,0x8585,0x85B4,0x85C4,0x5475,0x5454,0x5454,0xD475,0xD4B5,0x8595,0x8565,0xC5D4,0xC5E4,0x54C4,0xC485,0xC5D4,0x95D4,0x8585,0x8565,0x5465,0x7575,0x85D4,0x7575,0x8575,0x8575,0x4644,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4546,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4685,0x4646,0x4646,0x4141,0x4444,0x4141,0x4444,0x4444,0x4444,0x4141,0x4444,0x4141,0x4141,0x4444,0x7F7F,0x4141,0x4444,0x4444,0x4444,0xD454,0xC4C4,0xD564,0xD464,0xD464,0xC4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4C5,0xE4D4,0xE4E4,0xE4D5,0xE4F4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0x8585,0x4144,0x4444,0x64D4,0x6464,0x5444,0x6464,0x4444,0x4141,0xD564,0xD4D4,0x6454,0x5454,0xC475,0xD4C5,0x5475,0x5454,0x7554,0x9585,0xD4D4,0xC4B5,0x8575,0xB5C4,0x85D4,0xB495,0xD485,0x8595,0x7565,0x8585,0x7575,0x8554,0xC5C4,0x95D4,0x7585,0xD495,0xC5D4,0x8585,0xC495,0xD4B5,0xB495,0x8585,0xB5D4,0x85B4,0x7575,0xC485,0x7595,0x5454,0x8575,0x8585,0x7575,0x7585,0x4485,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x8544,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4444,0x464F
.word 0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4675,0x4644,0x5F5F,0x4141,0x4141,0x4444,0x4141,0x4444,0x4141,0x4444,0x7F7F,0x4144,0x4444,0x7F7F,0x4141,0x4444,0x4141,0x4141,0x4444,0x64D4,0x64D5,0xD5D5,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0x41D4,0x5454,0x54D4,0x4141,0xD5D5,0x6464,0x8F8F,0x64D4,0x4444,0x4444,0x6464,0xD4D4,0xD4D4,0x5454,0x8575,0x85C4,0x7554,0xD495,0xC4C5,0xB485,0xB495,0x95B5,0x9575,0x85B4,0x75C4,0x8585,0x95B4,0x54B4,0x7575,0x5454,0x7554,0x9565,0xD4D4,0xE4B5,0x75D4,0x85C4,0xC495,0x8565,0x95B4,0x95B4,0x85C4,0x7575,0xB485,0x7585,0x5465,0x7565,0x7575,0x5454,0x7554,0x75B4,0x7575,0xB485,0x7585,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x9575,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446
.word 0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4444,0x4444,0x4444,0x4141,0x4141,0x4444,0x8F8F,0x4141,0x4444,0x4141,0x4444,0x4141,0x4141,0x4444,0x4141,0x7F7F,0x4444,0x5454,0xC4C4,0x8585,0x6485,0x64D5,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D5,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0x41D4,0x5454,0x5454,0x4444,0x4444,0x4144,0x4444,0x4444,0xD4D4,0xC4C4,0x4141,0x5454,0x5454,0xD4D4,0x6565,0x7575,0x7554,0xB485,0xC485,0xC4B5,0x8575,0x8585,0x75B4,0x7554,0x8585,0x7565,0x7565,0x5454,0x7554,0xD485,0xB5D4,0xD4D4,0x7564,0x9585,0x95C4,0x54B4,0x8585,0x85B4,0x8575,0x7575,0x7585,0x6575,0x5475,0x5454,0x7575,0x8564,0x9585,0xC485,0xC485,0x8585,0xD4D4,0xD464,0xD4D4,0xD4D4,0x8574,0x8574,0x7464,0x4654,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x7544,0x6595,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4444,0x7F7F,0x4141,0x4444,0x7F7F,0x4141,0x4141,0x4144,0x4141,0x4444,0x8F8F,0x7F7F,0x4444,0x4141,0x4444,0x7F7F,0x4141,0xD454,0xD464,0xD564,0xD4D4,0xD485,0xD4C4,0xC4C4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xC5D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xD4D4,0x5454,0x4444,0x5454,0x5454,0x5454,0x5454,0x5454,0x4444,0x4141,0x4141,0x4444,0x6464,0x4141,0xD4D4,0x75D4,0x7554,0xC485,0x7585,0x8585,0x75B4,0x6554,0x5465,0x5454,0x5454,0x5454,0xD485,0xE4C5,0x95D4,0x6585,0x8585,0xC495,0x8595,0x7575,0x85B4,0x85C4,0x6465,0x7575,0x5475,0x6554,0x6575,0x5475,0x7554,0x8585,0xC485,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D5,0xD4D4,0x64D4,0x4464,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x9554,0x45C4,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x464F
.word 0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4444,0x4141,0x4444,0x4141,0x4441,0x7F7F,0x4444,0x4444,0x4444,0x7F7F,0x4441,0x4141,0x4141,0x4141,0x4141,0x4444,0x6464,0xD4D4,0xC4C4,0xC4C4,0xC485,0xC4C4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE441,0x41D4,0x4444,0x5454,0x5454,0x6444,0x64D5,0x5454,0x4444,0x5454,0x4454,0x4444,0x4141,0x6454,0x5454,0x5454,0xD485,0x75D4,0x8585,0x6554,0x5475,0x5454,0xB585,0x95E4,0x7575,0xD4D4,0xB495,0x9575,0x95C4,0xC5D4,0x85C4,0xB485,0xC485,0x6585,0x5475,0x6554,0x5454,0xC485,0xB5D4,0x7585,0xD475,0xE4B4,0x85D4,0x8575,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD4D4,0x4174,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5444,0xA585,0x4665,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4444,0x4141,0x4441,0x8F8F,0x4444,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x4441,0x7F7F,0x4444,0x7F7F,0x4144,0x7F7F,0x4141,0x5454,0xD4C4,0xD564,0xD464,0xD564,0xD464,0xD464,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D5,0xE4E4,0x41D4,0xE4D4,0x4141,0x4444,0xD564,0x5454,0x4141,0xD564,0x6464,0x4444,0x5454,0x5454,0x4141,0x4444,0x8F8F,0x4444,0x5454,0xD4D4,0xD4D4,0x7575,0x8554,0xC495,0x7585,0x95C4,0xD4B4,0x5485,0xC495,0x85D4,0x8575,0x85B4,0xB485,0x6595,0x7575,0x6575,0x5454,0xC585,0xB5D4,0x7585,0xD485,0xD4C5,0x85D4,0x8575,0x95D4,0xB495,0xD485,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4C5,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xD4D4,0xD464,0xD4D4,0xD4D4,0x4654,0x4444,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x8544,0xB495,0x4644,0x4644,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4646,0x4644,0x4644,0x4646,0x4646,0x4444,0x7F7F,0x4444,0x4141,0x4444,0x4141,0x4141,0x4444,0x4444,0x8F8F,0x4144,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x4444,0x5454,0xD475,0x6464,0xC4C4,0xD564,0x8585,0xD4D4,0xC4C4,0xD574,0xD4D4,0xD4D4,0xD4E4,0x64D4,0xD4D4,0xC5D4,0xD4E4,0xD4E4,0xE4D5,0xD441,0xE4E4,0x4144,0x5454,0x6441,0x4444,0x8F8F,0x6464,0x64D5,0x4444,0x5454,0x5454,0x5454,0x5454,0x4444,0x5454,0x5454,0x8F8F,0xD4D4,0x75D4,0x6575,0x7575,0x5485,0xC475,0x8585,0xB454,0x9585,0x75C4,0x7575,0x7575,0x7575,0x8554,0xE495,0xD495,0xB4B5,0xC475,0xD485,0x85D4,0x9575,0x95C4,0x95D4,0x8575,0xB485,0xD464,0xC4C4,0xD4D4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xE4D4,0xD4D5,0xE4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0x5474,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4444,0x4646,0x4644,0x4646,0x4446,0x7565,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6544,0xB495,0x4595,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4446,0x5F46
.word 0x4646,0x4646,0x4646,0x4646,0x4646,0x4141,0x4441,0x4141,0x4444,0x4141,0x4444,0x8F8F,0x4141,0x4141,0x4444,0x4141,0x4444,0x4144,0x7F7F,0x4444,0x4444,0x4141,0xD454,0x64D5,0x64D5,0xC4D4,0x8585,0x64D4,0xC4C4,0xD564,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0x6464,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0x41D4,0x41E4,0x5454,0x5454,0x4444,0x5454,0x5464,0x4141,0xD541,0x6464,0x5454,0x5454,0x5454,0x4444,0x4141,0x5454,0x5454,0x4154,0x6464,0x85D4,0x5454,0x5475,0x5454,0x7575,0x5475,0x7575,0x7575,0x5475,0x5454,0x5454,0x8575,0x7575,0x95C4,0x95C4,0x95D4,0x8575,0x95C4,0xB495,0x8575,0x85B4,0xB495,0x7575,0x6475,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4C5,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0xD4D5,0x4464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x9554,0x4665,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x9575,0xC495,0x4685,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x4141,0x7F7F,0x4141,0x4444,0x8F8F,0x4444,0x4141,0x4444,0x7F7F,0x4444,0x4141,0x4444,0x4141,0x4444,0x7F7F,0x7F7F,0x4141,0x6444,0x6464,0x6464,0xD4D4,0x64C4,0x64D4,0xD4D5,0xD464,0xD4D4,0xD485,0x61D4,0xD4D4,0x64D4,0xC4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x4444,0x4444,0x4444,0x5454,0x4444,0xD554,0x6464,0x4444,0xC4C4,0x5454,0x4444,0x6454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x6585,0xC485,0x7575,0x7585,0x7554,0x5454,0x5454,0x5475,0x8585,0x95C4,0xC5D4,0x5485,0xC485,0xB495,0xC485,0x8575,0xB485,0x7585,0x7575,0x7575,0x5475,0x6454,0xC4C4,0xD4D4,0xE4D4,0xD4E4,0xD4C5,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x4644,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x6446,0x6495,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x5446,0xC495,0x8585,0x4646,0x4444,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F46
.word 0x4646,0x4644,0x4646,0x4444,0x4141,0x4441,0x4141,0x4141,0x4444,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4444,0x4141,0x4444,0x4141,0x7F7F,0x4441,0x4141,0x5454,0xC4D5,0xC4C4,0x64C4,0xC4C4,0xC4C4,0xC4C4,0xC4C4,0xD4D4,0xD4D4,0xE564,0xD4D4,0xD4D4,0xC4C4,0x4141,0x4141,0x4141,0xE464,0xC5D4,0x6464,0x5454,0x4444,0x5454,0x6464,0x4444,0x8F8F,0x5464,0x4141,0x4444,0xD564,0x4441,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x7554,0x7575,0x7585,0x9575,0x5485,0x7585,0xC485,0xB4C5,0x9575,0x85C4,0xD4C4,0x6585,0x8585,0x85B4,0x8585,0x7565,0x7575,0x5454,0x5454,0x5454,0x7575,0x5454,0x64C4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xC5D4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xC5D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x4461,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x8575,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x9544,0x8585,0x75B4,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4646,0x4646,0x4646,0x4444,0x7F7F,0x4444,0x4141,0x4444,0x7F7F,0x7F7F,0x4444,0x4444,0x7F7F,0x4444,0x4141,0x4444,0x4141,0x7F7F,0x4444,0x4141,0x4444,0xD4C4,0x6464,0x64D4,0x64D5,0x64D5,0x64D5,0x64D4,0x64D5,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0x54D4,0xD464,0x4141,0x4444,0x6454,0xD4E5,0xE4D4,0xD4D4,0x4141,0x4141,0x6444,0xD5D5,0x4444,0x4441,0x8F8F,0x4444,0xD454,0xE4E4,0xE4E4,0xC4C4,0xD4D4,0x4444,0x5454,0x7575,0x5454,0x5454,0x5454,0x9F9F,0x5454,0x5475,0x6575,0x6585,0x7585,0x8585,0x6585,0xB485,0x8585,0xB495,0x5475,0x7575,0x7575,0x5465,0x5454,0x9585,0x8565,0xC495,0x8595,0x5454,0xC4C4,0xC4C4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E4,0xE4E4,0xE4D5,0xD4E4,0xD4D5,0xD4E4,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4C5,0xC5E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x4641,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7544,0x54A5,0x4646,0x4646,0x4646,0x4646,0x4646,0x8585,0x95D4,0x4685,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446
.word 0x4446,0x4646,0x4646,0x4444,0x4141,0x4444,0x7F7F,0x4141,0x4141,0x4144,0x4141,0x4141,0x4141,0x4141,0x4141,0x4444,0x7F7F,0x4444,0x7F7F,0x4141,0x4141,0x8585,0xC4C4,0xC4C4,0xC4C4,0xD464,0xC4C4,0x6464,0xD4D4,0x64D4,0xD4D4,0xD464,0xE4D4,0x4141,0x6464,0xD4D5,0xE4D4,0xE4D4,0xD4D4,0x41D4,0x4444,0x5454,0x5454,0x8F8F,0x4444,0x5454,0x6464,0x7575,0x4444,0x41D4,0x5454,0x7F7F,0x4444,0xE441,0xD4E4,0x4141,0x4444,0x5454,0x5454,0x6464,0x4154,0x5444,0x5454,0x5454,0x7554,0x7554,0x6565,0x5454,0x7575,0x7575,0x6575,0x7554,0x5454,0x5454,0x8575,0xC5C4,0xD4E4,0x8575,0x85C4,0x5475,0xD554,0xC464,0xD464,0xD4D4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4C5,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD464,0x5464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x9585,0x4645,0x4646,0x4646,0x4644,0x4646,0x9585,0xB4D4,0x4675,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646
.word 0x4646,0x4646,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4444,0x7F7F,0x4444,0x7F7F,0x4444,0x4444,0x4141,0x7F7F,0x4141,0x4141,0x4444,0xC4C4,0x5454,0x64C4,0x64D4,0x64D4,0x64D5,0x64D4,0x64D5,0x85D5,0xC4C4,0xC4C4,0xD464,0x6441,0x85D5,0x64D5,0xC4C4,0xD4D4,0xD4D4,0x6464,0x4444,0xD4D4,0x5454,0xC4C4,0x6464,0x4144,0x6464,0xD4D5,0x6464,0xC454,0x5454,0xE4D4,0xE4D4,0x54D4,0x6464,0x7F7F,0x4444,0x4444,0x4444,0x5454,0x5454,0x5454,0x8F8F,0x5444,0x5454,0x5454,0x5454,0x5454,0x5475,0x9585,0xD4E4,0x85B5,0x9554,0x95C4,0x95D4,0x8575,0xC485,0x8595,0x8554,0x5475,0x5454,0x6464,0xD464,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D5,0xE4D4,0xD4D4,0xD4C5,0xE4C5,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xC5E4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0x4464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x7545,0x4644,0x4646,0x4646,0x4646,0x4646,0xB575,0x4585,0x4446,0x4646,0x4646,0x4446,0xB485,0x8595,0x4675,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0x4646,0x4646,0x4141,0x4141,0x4444,0x7F7F,0x4444,0x4444,0x4444,0x4141,0x4141,0x4141,0x7F7F,0x4441,0x4141,0x4444,0x4141,0x7F7F,0x7F7F,0x4444,0xD441,0x64D5,0xD4D4,0xC4C4,0xC4C4,0xD564,0xC4C4,0xC4C4,0xC4C4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD464,0xD4D4,0xE4E4,0x6464,0xD454,0xD4D4,0x6441,0xD5D5,0x4444,0x5454,0x5454,0x4141,0x5454,0x5454,0xD4D4,0xE4E4,0xD4E4,0x41E4,0x7F7F,0x4444,0x4141,0x4141,0x4141,0x4444,0x4444,0x4444,0x5454,0x4441,0x5441,0x5454,0xC485,0x8595,0x5485,0x85B4,0xB485,0xC485,0xC475,0xC495,0xB485,0x8554,0x85B4,0x75B4,0x6565,0x5454,0xD564,0xD564,0xD4D4,0xD4D4,0xD4D4,0xC5E4,0xE4E4,0xD4E4,0xE4E4,0xD5E4,0xE4D4,0xD5D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0x6464,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5446,0x4475,0x4446,0x4646,0x4646,0x4644,0x8544,0x75D4,0x4646,0x4646,0x4646,0x4646,0x9585,0x85B4,0x4675,0x4646,0x4646,0x4646,0x4646,0x4444,0x4675,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4646,0x7F7F,0x4141,0x4444,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4444,0x7F7F,0x4444,0x4141,0x4444,0x7F7F,0x4444,0x4444,0x7F7F,0x4141,0xD454,0xD464,0x64D4,0x64D5,0x64D5,0x64D4,0x64D4,0x64D4,0xD4D4,0xC4C4,0xD464,0xC4D4,0xD464,0xD4D4,0x6464,0xC4C4,0x8585,0xD454,0xD4D4,0xD4D4,0x41D4,0x5454,0x4141,0x4444,0xD464,0x64D4,0x5454,0x4444,0xC464,0xE4E4,0xE4D4,0xC5D4,0x6464,0x4444,0x4141,0x4444,0x4444,0x4444,0x4441,0x5454,0x4444,0x4444,0x8F8F,0x7554,0x5444,0x5454,0x7565,0x75C4,0x7575,0x7585,0x7575,0x8554,0x75B4,0x7575,0x7554,0x6554,0x5454,0x5475,0x6464,0x6464,0xC4C4,0xC485,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xE4D4,0xD5D4,0xD4E4,0xD4E4,0xD4E4,0x51D4,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x7554,0x4644,0x4646,0x4646,0x4646,0x5446,0x9585,0x4654,0x4446,0x4646,0x6565,0xC495,0x8585,0x4475,0x4646,0x4646,0x4644,0x4646,0x8565,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x4646,0x4441,0x4141,0x7F7F,0x4141,0x4444,0x7F7F,0x4444,0x4141,0x4444,0x4141,0x7F7F,0x7F7F,0x4141,0x4141,0x4444,0x4141,0x7F7F,0x4444,0x4444,0xC4C4,0xD4D4,0xD564,0xD464,0xC4C4,0xC4C4,0xC4C4,0xD4D4,0xD564,0xD464,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xC4D4,0x6464,0xD454,0xD4D4,0x64E4,0x6464,0x4444,0x5454,0x4444,0x4141,0x4141,0x9F9F,0x5454,0x7575,0xC4C4,0x6464,0x5454,0x8F8F,0x4444,0x4444,0x4444,0x4141,0x4444,0x4444,0x5454,0x5454,0x4141,0x5454,0x4164,0x6464,0x54C4,0x5454,0x5454,0x5454,0x5454,0x7565,0x5454,0x5475,0x5475,0x5454,0xC485,0x95C5,0x5485,0x5454,0x7575,0x64D5,0x85D5,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD5E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xD4C5,0xE4E4,0xD4C5,0xE4E4,0xC5D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4C5,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x4641,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4444,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x8544,0x4675,0x4446,0x4646,0x4646,0x4644,0xC485,0x4685,0x4646,0x4646,0x7546,0x85C5,0x95B4,0x4475,0x4646,0x4644,0x4646,0x6546,0x4485,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646
.word 0x7F7F,0x4141,0x4141,0x4444,0x4444,0x7F7F,0x4444,0x6F6F,0x7F7F,0x4141,0x7F7F,0x4444,0x7F7F,0x4444,0x4444,0x7F7F,0x4444,0x7F7F,0x4141,0x4141,0xD4C4,0xD564,0xC4C4,0x8585,0xD564,0xD464,0xD564,0xD464,0x6464,0xD4D4,0xD4C4,0x64D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x6464,0xD4D4,0xD4D4,0xC4C4,0x5454,0x5454,0xD564,0x64D4,0x4444,0x5454,0x4444,0x4141,0xD4D4,0xE4E4,0xE4D4,0x41D4,0x4141,0x4141,0x4444,0x4444,0x4141,0x4441,0x4444,0x4444,0xE4E4,0x64D4,0x4444,0x7564,0x75C4,0x5454,0x5454,0x5454,0x7554,0x5475,0x8595,0x8585,0x5485,0x8585,0x85B4,0x5475,0x5454,0xC4C4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E5,0xC5E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xC5E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x54D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5446,0x7595,0x4646,0x4646,0x4646,0x4646,0x8574,0x6595,0x4646,0x4646,0x8546,0xB495,0x7595,0x4575,0x4646,0x4646,0x4646,0x8544,0x4585,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46
.word 0x4141,0x4444,0x4141,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4141,0x4141,0x4444,0x6F6F,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4444,0x5454,0x64D4,0x64D4,0xD4D4,0xD564,0x8585,0xC4C4,0xC4C4,0xC4C4,0xD564,0xD464,0xD464,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x41D4,0xD564,0xD464,0xD4D4,0xD454,0xD4D4,0x8F8F,0x5454,0xD4C4,0x6464,0x5454,0xD4D4,0x64D4,0x5454,0xE464,0xE4D4,0xE4E4,0xE4E4,0x4444,0x4444,0x4141,0x4144,0x4444,0x5454,0x4444,0x4141,0x5454,0xD5E4,0x5454,0x6464,0xD454,0x5485,0x7554,0x7554,0x8585,0x54C4,0x7575,0x7575,0x7575,0x7575,0x7575,0x5454,0x7575,0xD564,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xC5E4,0xD4E4,0xD4D4,0xC5E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xC5E4,0xE4D4,0xD4D4,0x64D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x6546,0x7585,0x4644,0x4646,0x4646,0x4646,0x8544,0x9585,0x4646,0x4446,0xB444,0x8595,0x95B4,0x4585,0x4646,0x4646,0x4646,0x9575,0x4675,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7F7F,0x4141,0x7F7F,0x4444,0x7F7F,0x4444,0x7F7F,0x4444,0x4444,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4444,0x4444,0x7F7F,0x4441,0x4141,0x5454,0x64C4,0xD4D5,0xC4C4,0xD4D4,0x74D4,0x64D5,0x64D5,0x64D4,0xD4C4,0xD4D4,0xD4D4,0xC4C4,0xD464,0xC4C4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xD464,0xD4D4,0xD464,0x4141,0x4444,0x4141,0x4444,0xD554,0xD4D4,0xD4D4,0xD554,0xD4D4,0xE4E4,0xD4E4,0x41E4,0x4141,0x4141,0x4444,0x4441,0x8F8F,0x4444,0x4444,0x4444,0x4444,0xD4D4,0x54D4,0x6464,0x6464,0x95C4,0x5465,0x5465,0x7565,0x5475,0x5475,0x5454,0x7554,0xC485,0x5495,0x6464,0x7575,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D5,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4E4,0xC5D4,0xE4D4,0xE4C5,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xC5E4,0xE4D4,0xD4D4,0xC5D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x5454,0x4646,0x4644,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x9575,0x4675,0x4646,0x4646,0x4646,0xB444,0xB495,0x4654,0x4646,0x9565,0x95B4,0xB485,0x4585,0x4446,0x4646,0x4444,0x9595,0x4675,0x4644,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0x4141,0x4444,0x4144,0x7F7F,0x4444,0x4441,0x8F8F,0x4141,0x7F7F,0x4141,0x4444,0x4444,0x4141,0x4141,0x4141,0x4444,0x7F7F,0x5454,0x41D4,0xD4D4,0xD464,0xD4C4,0x6485,0x64D5,0xD4D4,0xC4C4,0xD485,0xC4C4,0x64D4,0xD4D4,0xD464,0x64D5,0xD4D4,0xD4D4,0xC4C4,0x64D4,0x6464,0xD5D4,0xD4D4,0xD4D4,0x54D4,0x41D5,0x4444,0x64C4,0x8F8F,0x7F7F,0x4141,0x5454,0x4141,0xD485,0xE4E4,0xE4E4,0xD4D4,0x4444,0x4444,0x4444,0x8F8F,0x4144,0x5441,0x5454,0x4141,0x4444,0x5454,0x64E4,0x5454,0x54D5,0xC464,0x54D4,0x5454,0x5454,0x5454,0x5475,0x6554,0xC485,0x8575,0x6565,0x6454,0x64D5,0xD4D5,0x85D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4D5,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD5D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD5E4,0xE4E4,0xD4E4,0xC5E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x64D4,0x4654,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x8554,0x6585,0x4446,0x4646,0x4644,0x8546,0x85C4,0x4675,0x4444,0x9585,0xB485,0x8585,0x45B4,0x4646,0x4646,0x7546,0xB495,0x4665,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4446
.word 0x4141,0x4141,0x4141,0x4141,0x5454,0x4141,0x4141,0x4444,0x7F7F,0x4444,0x4141,0x7F7F,0x6F6F,0x4444,0x7F7F,0x4141,0x5454,0x5454,0xC4C4,0x8585,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD485,0xD4D4,0xC4C4,0xC4C4,0x8585,0xD4C4,0xD464,0x6464,0xD4C4,0xD485,0xC4C4,0xD4D4,0x64D4,0x64C4,0xD4D4,0xD4D4,0xD4D4,0x6454,0xD4D4,0x4144,0x6464,0xD4D4,0x5441,0xD464,0x4444,0x6451,0x4444,0x6441,0x8F8F,0x4141,0x4141,0x4141,0x4444,0x4444,0x4141,0x5454,0x4154,0x8F8F,0xD4D4,0x5454,0xD4D4,0x5464,0xD464,0x5485,0x5454,0x7585,0x7585,0x8585,0x8575,0x5454,0x6454,0xD5D5,0xD464,0xD474,0xD4D4,0xD4D4,0xE4E4,0xC5D4,0xE4D4,0xE4E4,0xE4C5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xC5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0x4464,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5446,0x4465,0x4646,0x4646,0x4646,0x4646,0x8554,0x75B4,0x4646,0x4646,0x4646,0x8546,0x8585,0x4675,0x7545,0x7595,0x95D4,0xB5C4,0x4585,0x4646,0x4646,0x9554,0x85D4,0x4665,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4444,0x4141,0x4444,0x7F7F,0x5454,0x5454,0x4444,0x4141,0x4141,0x4444,0x4141,0x4154,0x4444,0x7F7F,0x7F7F,0x7F7F,0x6464,0x5454,0xD4C4,0xD4D4,0xD4C4,0xD464,0xD4C4,0xD4D4,0xD464,0xC4C4,0x64C4,0x85D4,0x64D4,0xD4D5,0x6464,0xD4D4,0xD4C4,0x6464,0xD4D4,0xC4C4,0xD464,0xD4D4,0xD4D4,0xC4C4,0xD4D4,0x64D4,0x5454,0x5454,0x5454,0xD4D5,0x6464,0x5454,0xD4D5,0xD4D4,0x4444,0xD4D4,0xD4E4,0x4141,0x4444,0x4444,0x4444,0x4444,0x4141,0x5454,0x5454,0x4444,0x4141,0xE4E4,0x6464,0xE454,0xD554,0x6565,0x7595,0x5454,0x7575,0x5485,0x5465,0x5475,0x5454,0x7575,0x6464,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xC5D4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xD4E4,0xD5D4,0xE4E4,0xD4E4,0xE4E4,0xF4E4,0xE4E4,0xD4E4,0xE4F4,0xE4D4,0xD4E4,0xD4C5,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x5485,0x4644,0x4646,0x4646,0x4646,0x9544,0x8595,0x4645,0x4646,0x4646,0x9554,0x85C4,0x4575,0x8554,0x85B4,0x85C4,0x85C4,0x6575,0x4646,0x4646,0x9575,0x95B4,0x4645,0x4644,0x4646,0x4646,0x4444,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4141,0x4444,0x7F7F,0x4444,0x5464,0x4444,0x5454,0x6464,0x4444,0x4141,0x7F7F,0x5454,0x4141,0x4444,0x4444,0x4141,0x41C4,0x64D4,0xD4D4,0x64C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xD4D4,0xD4D4,0xD585,0xD464,0xD4C4,0xD4D4,0xD564,0x6464,0xC4D4,0x64D5,0xD4D4,0xD4D4,0x85D4,0xD464,0xD4D4,0xC4C4,0x5454,0x4444,0x7F7F,0x4444,0x4444,0x4141,0x4444,0x4141,0xE4D4,0xD4E4,0x4141,0x4141,0x4141,0x4141,0x4141,0x4144,0x4444,0x4444,0x4444,0x4141,0xC444,0x41D4,0xE454,0xD4D4,0x4141,0x8544,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x6464,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4C5,0xE4D4,0xD5D4,0xE4D4,0xD4D4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4C5,0xD4E4,0xD4D4,0xD4D4,0xC5E4,0xD4D4,0xD4D4,0x85D4,0x4651,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x7544,0x7595,0x4446,0x4646,0x4646,0xB444,0x7595,0x4645,0x4644,0x4446,0x9575,0x85B4,0x7585,0x8585,0x9585,0x85C4,0xC495,0x8575,0x4446,0x5446,0x9595,0x75B4,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4446
.word 0x4141,0x5454,0x7F7F,0x5441,0x5454,0x5454,0x4154,0x5454,0x5454,0x4464,0x4141,0x5454,0x5454,0x4141,0x4141,0x7F7F,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4C4,0xD464,0x6464,0x6464,0x64D4,0xC4C4,0xD4D4,0xC464,0xD464,0xD464,0xD4D4,0xE464,0xE4D4,0xD4D4,0x8585,0xC4C4,0x5454,0x5441,0xC464,0x4141,0x5454,0x6485,0x4141,0xC4C4,0xE4D4,0xE4E4,0x6464,0x4444,0x4444,0x4444,0x4444,0x8F8F,0x4444,0x5454,0x4141,0x4444,0x4141,0xE5E5,0xD444,0x6454,0x4444,0x8585,0x4485,0x7554,0x5485,0x7585,0x5454,0x6464,0x64D5,0xD4D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D5,0xE4E4,0xE4E4,0xD4E4,0xE5E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D5,0xD4D4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x6565,0x4644,0x4446,0x8595,0x4665,0x4646,0x4646,0xB444,0xC485,0x4675,0x4646,0x7546,0xC495,0x7585,0x9575,0x85B4,0x95B4,0x95B4,0x95B4,0x7575,0x4654,0x8544,0x95B4,0x6585,0x4446,0x4446,0x4685,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646
.word 0x5454,0x4141,0x5454,0x6464,0xC4C4,0x5454,0x5454,0x64D5,0xD554,0x6464,0x4444,0x64C4,0x6464,0x7F7F,0x5454,0x4441,0x54D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD574,0xD4D4,0xD4D4,0xC4D4,0x6464,0x85D5,0xC4C4,0xD4D4,0xD4D4,0xC4C4,0xD4D4,0xD4D4,0xD564,0xD4D4,0xC4D4,0xD464,0x5454,0xD554,0xD4D4,0x5454,0xD4D4,0xD5D4,0x8F8F,0x41D4,0xD4D4,0x4141,0x64C4,0x8F8F,0x6441,0x4444,0x4141,0x4444,0x4141,0x4141,0x4454,0x4141,0x4444,0x54E4,0x5454,0x5454,0x41D4,0x5454,0x54D4,0x5454,0x6575,0x5475,0x5454,0x7575,0xC4C4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xC5D4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xD4C5,0xD4C5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0x64E4,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6546,0x4454,0x4646,0x9575,0x4475,0x4446,0x4646,0x8574,0x8585,0x4675,0x4646,0x9554,0x85C4,0x7585,0x75D4,0x7585,0xB495,0x8585,0xC485,0x7585,0x4675,0x9575,0xB495,0x4585,0x4446,0x8585,0x4665,0x4644,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x464F
.word 0x6464,0x4144,0x4444,0x6475,0x6464,0x7575,0x5454,0xD564,0x8585,0x64D5,0x4141,0xD4D4,0x7575,0x5454,0x6464,0x5454,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD485,0x64D4,0xD4D4,0x64D4,0xD4D4,0xD464,0xD464,0x64D4,0x64D4,0x6464,0xD4D4,0xD4D4,0xC451,0xD4D4,0xD4D4,0x4444,0x6464,0xD4D4,0x5441,0xD564,0x5474,0x4444,0xD4D4,0x6464,0xE464,0xE4E4,0x6464,0x4444,0x6464,0x4444,0x4444,0x4141,0x4444,0x4141,0x4444,0x5454,0x54E4,0x5454,0x5454,0x4141,0x4444,0xD464,0x5475,0x5454,0x4454,0x6464,0x64D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xE4D4,0xE4D4,0xE4D4,0xE4C5,0xE4D4,0xC5D4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD5D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4C5,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xC5D4,0xE4E4,0xD4D4,0xD4D4,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4446,0x7575,0x4446,0x8544,0x7585,0x4646,0x4446,0x9575,0x95B4,0x4665,0x7575,0xD495,0x7585,0x9575,0x85C4,0xB485,0xD485,0xB485,0x9585,0x95C4,0x6575,0x9575,0x85C4,0x4585,0x8565,0x7585,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x6464,0x5454,0x6454,0xD5D5,0xD564,0xC454,0xD564,0x64D4,0xD4D4,0x64D4,0x5454,0x64D4,0xD4D4,0x4444,0xC4C4,0x54D4,0x64D4,0xD4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD5D4,0x8585,0x5454,0xD4D4,0x64D4,0xD4D5,0xD4C4,0x8585,0xC4C4,0xD4D4,0xD4D4,0xD464,0xD4D4,0x4141,0x4444,0x8F8F,0x4141,0x8F8F,0x5454,0xD585,0x5441,0xD4D5,0xE4D4,0x44E4,0x4141,0x4444,0x64D4,0x9F9F,0x4444,0x4444,0x7F7F,0x4444,0x4444,0x64E4,0x6464,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4C4,0x5454,0x5454,0x6464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4C5,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xC5E4,0xD5D4,0xE4D4,0xD4E4,0xD4E4,0xE4E5,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x5485,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x8544,0x4675,0x7546,0x75B5,0x4646,0x7575,0xB485,0xB485,0x4645,0x9575,0x95C4,0x7575,0xB5C4,0x8575,0x95B4,0xB485,0x7595,0xB495,0xB485,0x8575,0xD495,0xC485,0x6585,0xC495,0x4585,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4644
.word 0x6464,0xC4C4,0x6464,0x7575,0xC4C4,0x6464,0xC4C4,0xC4C4,0xC464,0xD4D4,0x4141,0x64C4,0xD4D4,0x6464,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xD5D4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x6464,0xD4D4,0xD4D4,0xD464,0xD564,0xD464,0xD4D4,0xD4D4,0xD4D4,0x6464,0x7575,0x5454,0x6454,0x64D5,0x5454,0xD564,0x6441,0xBFBF,0x6454,0xD4E4,0xD564,0x41D4,0x7F7F,0xC4C4,0x64D4,0x4444,0x4141,0x4444,0x4141,0x5454,0x54E4,0xD4D4,0xD4D4,0xE4E4,0x5454,0xD4C4,0x51D4,0x54D4,0x4141,0xD575,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xC5D4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4C5,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7546,0x5495,0x7546,0x75D4,0x4644,0x7545,0x8595,0x8595,0x6445,0xB585,0x85B4,0x85B4,0xB485,0x8575,0xD485,0x7585,0x7585,0x85C4,0x8585,0x8575,0x85C4,0x85B4,0xB485,0xB495,0x4675,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x464F
.word 0x6464,0x7575,0xC4C4,0x64D5,0x64D4,0x64D5,0xC4D4,0xD564,0xD4D4,0xD4C4,0xD454,0xD4D4,0xD4C4,0x6464,0xD4D5,0xD4D4,0xE4D4,0xE4D4,0xD5D4,0xE4D4,0xE4E4,0xE4F4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xD4D4,0x4141,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64C4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0xC4C4,0xD4D4,0x5454,0xD4D5,0x4444,0x64C4,0xD5D5,0x6441,0xD4D4,0xE4E4,0x8585,0x4141,0x4444,0xD464,0x6464,0x4444,0x4141,0x4144,0x44E4,0x5441,0x41D4,0xD464,0x4141,0x4141,0xD5D4,0xD4D4,0xC4C4,0xD464,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4C5,0xE4E4,0xD4E4,0xD4E4,0xC5E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4D4,0xD4E4,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4C5,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x7546,0x7595,0x8565,0x7595,0x4575,0x8575,0x85B4,0x75B4,0x9575,0xB4B5,0xB495,0x8585,0x8595,0xB485,0x8575,0x85B4,0x8585,0x95C4,0x85D4,0x95B4,0x8585,0xB485,0x8585,0x85B4,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xC4C4,0x6464,0x64C4,0xC4C4,0xC4C4,0xD4D4,0x6464,0x64D4,0x85D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x44D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4F4,0xE4E5,0xE5E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD485,0xD4C4,0x6464,0x6464,0xD564,0xD4D4,0xD4D4,0xC4C4,0xD564,0xD464,0xD4D4,0x6464,0x5454,0xD4D4,0x5454,0xD4D4,0x8585,0x6464,0x6464,0xD5D5,0x4141,0xD454,0x64E4,0x4444,0x7F7F,0x4141,0xD4D5,0x6464,0x4444,0xD554,0xE4E4,0x4444,0x6464,0xD454,0x4444,0x4441,0xD4D4,0x64D4,0xD4D4,0xD4D5,0xD4D4,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xD4E4,0xE4D4,0xE4C5,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4F4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x44D4,0x4644,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x5446,0x8595,0xB485,0x7585,0x7565,0x9585,0x85C4,0x7585,0xD485,0x8585,0x85B4,0xB485,0xB495,0x7585,0x8585,0xB485,0xB485,0xC485,0xB495,0xB485,0xB495,0x8575,0xC495,0x6585,0x4646,0x4646,0x4446,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46
.word 0x6464,0x64C4,0x64D5,0x64D5,0x64D4,0xC4D4,0xC4D4,0xD4D4,0xD4D4,0xD4C4,0xD464,0xD4D4,0xD4D4,0xD464,0xD464,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4F4,0xE5E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD4D4,0x64D4,0x5454,0xD464,0xD4D4,0xD4D4,0x6464,0xC4C4,0xC4C4,0x64C4,0x4444,0x4141,0x4141,0x5454,0x4444,0x4444,0x5454,0x5454,0x6464,0x6464,0xD575,0x6464,0x6485,0x74D5,0xD4D4,0xD4D4,0x8F8F,0x8585,0x41D4,0xD4D4,0x5454,0xE444,0x4444,0x4444,0x4141,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D5,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4C5,0xE4E4,0xD4E4,0xE4E4,0xC5E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0x51D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x8544,0x85B4,0x8585,0x75B4,0x8565,0xC495,0x7585,0xB485,0xB4B5,0xB495,0x8585,0xC495,0xB495,0x8575,0x85C4,0x8575,0x8585,0xB485,0x7585,0x8585,0x85C4,0xB485,0x8585,0x4685,0x4444,0x7575,0x4665,0x4646,0x4644,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646
.word 0xD564,0x6464,0xC4C4,0xC4C4,0xC4C4,0xC4C4,0xD4D4,0xD464,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xF4F4,0xE4E4,0xE5F4,0xE4F4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xC4C4,0xD4D4,0xD4C4,0x4444,0xD4D4,0xD464,0xD4D4,0xD464,0xD464,0x64D5,0x41D5,0x6454,0x4444,0x6454,0xC4D4,0x6464,0xC4C4,0x5454,0x4444,0x4444,0x4141,0x4444,0x4141,0x8F8F,0x4141,0x4141,0x6444,0x5454,0x6464,0xE454,0x4444,0x4141,0xC4C4,0x54D4,0x4444,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4C5,0xD4E4,0xD4E4,0xE4D4,0xC5D4,0xD4D4,0xD4E4,0xE4E4,0xC5D4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x54D4,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x5446,0x4646,0x4646,0xB4B4,0xB495,0x9585,0x7585,0x8575,0x85D4,0x85B4,0x9575,0x85C4,0x9575,0x85C4,0x85B4,0x75B4,0x8575,0xB485,0xB485,0xB485,0x85B5,0x75B4,0xB495,0xB495,0x8585,0x75B4,0x4665,0x7545,0x6595,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xC4C4,0x64C4,0x74D5,0xD564,0xD574,0xD464,0xD4C4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD5D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xF4F4,0xF4E5,0xE4E4,0xD4E4,0xD5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x8585,0xD4C4,0xD4C4,0xD485,0x4444,0xC4C4,0xD4D4,0x64D4,0xC4C4,0xC4C4,0x6464,0x5454,0xD5D5,0x64D4,0x5454,0xD4D5,0x6464,0x5454,0x4444,0x5441,0x4444,0x6441,0x4444,0x4444,0x4444,0x8F8F,0xD554,0x4141,0xE444,0x4141,0xC4C4,0xD4D4,0x6464,0x5454,0xD4D4,0x6464,0xD4C4,0xE464,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4D4,0xD5D4,0xE5E4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD474,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5446,0x4685,0x4644,0x9565,0x7595,0xB4C4,0x8575,0x8585,0x95B4,0x7585,0xC485,0xB495,0xC485,0xB485,0x8585,0x7595,0x7575,0x8585,0x7585,0x8575,0xB5B4,0x8585,0x85B4,0x8585,0x8575,0x7585,0x6465,0x7595,0x4645,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4644
.word 0xD464,0xC4C4,0xC4C4,0xC4C4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E5,0xF4D5,0xE4E4,0xE4E4,0xE4D5,0xE4F4,0xE4E4,0xE4E4,0xF4F4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4D5,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x8585,0xC4C4,0xC4C4,0xD4D4,0x51D4,0x5454,0xD4D4,0xC4D4,0xD464,0x6464,0x7F7F,0x5144,0xD5D5,0x5454,0x4141,0x4444,0x5454,0x8F8F,0xD554,0xD4D4,0xE664,0x4141,0x4141,0x4141,0x4444,0x4141,0x6464,0x6464,0xC4D4,0x64E4,0x4141,0x4444,0x4444,0xD4D4,0x41D4,0xD4D4,0x64D4,0xD4E4,0xD4C5,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xC5E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xD5E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x54D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4546,0x7585,0x4646,0xB454,0x85B5,0x8585,0x7575,0x95C4,0xB485,0x7575,0x95D4,0x7575,0xB485,0x7595,0xB485,0x7585,0x6554,0x7575,0xB485,0x9575,0x7575,0xC475,0xB495,0x7585,0x95C4,0x65B4,0x8575,0x45C4,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F
.word 0x64D4,0x64D5,0x64C4,0x74D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4C5,0xE4E4,0xC5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E5,0xF4E4,0xF4E4,0xF4F4,0xE4E5,0xE5E4,0xF4E4,0xF4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E5,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD464,0xD464,0xD4D4,0xD4D4,0xD4D4,0x5454,0xC4C4,0xC4D4,0x54D4,0x5454,0x6464,0x4444,0xD564,0x4141,0xD564,0x4164,0xD4D5,0xE4E4,0x4141,0xD444,0x5454,0x4444,0x4141,0x7F7F,0x7F7F,0x7575,0xE4E4,0x41E4,0x5454,0x4444,0x4141,0xE4D4,0x54D4,0xD4D4,0xD4D4,0xD464,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xC5E4,0xD4D4,0xD4C5,0xE4D4,0xC5D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xE4F4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4C5,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x85D4,0xD4D4,0x4646,0x4646,0x4646,0x4644,0x4454,0x4646,0x4446,0x9575,0x7575,0x9575,0xB485,0xB495,0x8585,0xC4B5,0x85B5,0x8585,0x95D4,0x85B4,0x7585,0x7585,0x9585,0x75B4,0x7575,0x8554,0x7575,0xB485,0x6585,0xB495,0x8585,0x9575,0x85D4,0x8575,0xC495,0x4665,0x4646,0x4646,0x4546,0x5475,0x4446,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646
.word 0xD564,0xD464,0xD4D4,0xD4D4,0xD4C4,0x64D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD5E4,0xF4E4,0xE4E4,0xE5E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xF4E4,0xF4E4,0xF4F4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4C5,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x6464,0xD5D5,0xD564,0xD464,0xD4D4,0xD4D4,0x64D4,0x8F8F,0xD464,0x5454,0xD4D4,0x4141,0xD464,0x5454,0xC4C4,0xD4D4,0x6451,0x41D4,0x5454,0x8F8F,0x4141,0x5454,0x4444,0x4444,0x4444,0x5454,0x64E4,0x6464,0x7575,0x4444,0x4144,0xD4E4,0xD4D4,0x6464,0xD4D4,0xD4D4,0xD4C4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD5E4,0xE5D4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x54D4,0x4646,0x4644,0x4646,0x4646,0x5446,0x5484,0x4646,0x8575,0x8585,0x9575,0x95B4,0x75C4,0x8575,0xB495,0x85D4,0x85B4,0x95B4,0x85B4,0x85B4,0x75B4,0xB495,0x7585,0x7554,0x5475,0x7585,0x7585,0x8575,0x95C4,0x85B4,0x85B4,0x75B4,0x7585,0x95C4,0x4644,0x4644,0x6565,0x7564,0x4645,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446
.word 0xD4D4,0xD4C4,0x64D4,0xD4D4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E5,0xE4F4,0xE4F4,0xE5E4,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4D4,0xE4F4,0xE4F4,0xE4E5,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xC4C4,0xD464,0xD464,0xD4D4,0xD4D4,0xC4D4,0x64D4,0x5454,0x6454,0xD4D5,0x5454,0x6464,0x5454,0x5454,0x64D5,0x6464,0xD4D5,0xC4C4,0x6464,0x6464,0x4141,0x4141,0x4444,0x6464,0x5454,0x64D4,0x4444,0xD4D4,0x41D4,0xE4D4,0xE4E4,0xC4C4,0x64D4,0x64D4,0xD4E4,0xD4C5,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xE4D4,0xE4E4,0xD4E4,0xD4D5,0xE4E4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0x4646,0x4646,0x4646,0x4646,0x4446,0x9544,0x8575,0x7546,0x85A5,0x95B4,0x9575,0x85D4,0x8575,0x85C4,0xB485,0x9585,0x85C4,0x8575,0x8575,0x7595,0x85B4,0x7575,0x7575,0x7575,0x7565,0x8575,0x8575,0x7585,0x8575,0x8585,0x7575,0xB575,0x75C4,0x4646,0x4446,0x85B4,0x4665,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4C4,0x6464,0xD4C4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE5E4,0xF4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E5,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xD474,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0x6464,0x8585,0x64D4,0xD4D4,0xD4D4,0xD4D4,0x8F8F,0x4141,0x7F7F,0x5454,0x5464,0x6464,0xD4D4,0x5454,0x6464,0x4141,0x4444,0x5464,0x4444,0x4141,0x4141,0x7F7F,0x4444,0xE4D4,0x4141,0xE444,0x41D4,0x4144,0x4444,0xD441,0xD4D4,0xD4D4,0xD4C4,0xE4D4,0xD4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xC5E4,0xD4E4,0xD4C5,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4C5,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xD474,0xD4D4,0x54D4,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x8595,0x7595,0x8595,0xC485,0xC475,0x7585,0xB485,0xB485,0x8585,0xB585,0xB485,0x8575,0x7575,0x75D4,0x8575,0x7575,0x5475,0x8575,0x8575,0x7585,0xC485,0x7575,0xB485,0xB4B5,0x7585,0x8585,0x4585,0x4446,0x8554,0x6585,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E5,0xE5E4,0xF4E4,0xE5E4,0xF4E4,0xE4E4,0xF4F4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4C5,0x74D4,0x6464,0xC4C4,0xC4C4,0x8585,0xD485,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD564,0xD564,0xD4D4,0xD564,0xC464,0xD564,0x54C4,0xD541,0x4164,0x5454,0x5454,0x6464,0x64D5,0x8F8F,0x5454,0xD564,0x4451,0xD4D5,0x5454,0xD4D4,0x4444,0x4444,0x4141,0x4444,0x6444,0xE4D4,0xD4D4,0x5454,0x4444,0x4444,0x5441,0xD4D4,0xD451,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4C5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x7546,0x8575,0xC4B4,0x9575,0x9575,0x85C4,0x7585,0x8585,0x85B4,0x95B4,0x8585,0xB485,0x9595,0x85C4,0x7575,0x9575,0x7575,0x7595,0x7595,0xB485,0x7585,0x7575,0xC485,0x8585,0x7585,0xD4D4,0x4575,0x8564,0x7585,0x4645,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x465F
.word 0x64D4,0xC4C4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD5D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x6464,0x4444,0x5444,0xD4D4,0xD564,0x64D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xD4D4,0x6464,0x64C4,0xC4C4,0x4141,0xD5D5,0xD564,0xD4D4,0x4444,0x5454,0xD4D4,0x4444,0xD454,0xAFAF,0x6464,0x4141,0x5441,0x6464,0x4444,0xD4D4,0x4141,0x6464,0x41E5,0x5454,0x4444,0x4141,0x4444,0x4141,0x4444,0xD464,0x6464,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4C5,0xD5D4,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xD4E4,0xE4C5,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xC5E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0x5464,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x7585,0x9575,0xD475,0xB485,0xB485,0x7585,0xB485,0x8585,0xC485,0xB495,0x8585,0x85C4,0x85B4,0x8585,0xC485,0x7575,0x7575,0x75C4,0x7585,0x7575,0x8585,0x85C4,0xB475,0x8575,0x8585,0x7575,0xC485,0x4575,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646
.word 0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xC5E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xD4E5,0xE4F4,0xE5E4,0xF4E4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0x64D4,0x4151,0x4141,0x4444,0x5454,0x5454,0x4141,0x5454,0x8585,0xC4C4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xC4C4,0xD4D4,0x6485,0x6464,0x4444,0x5454,0x4441,0x4141,0x5454,0x4444,0x6464,0x6464,0x4444,0x64C4,0xD4D4,0x4444,0x6464,0x6464,0x4444,0x7F7F,0x4444,0x4141,0x6464,0xD554,0x54D4,0x8F8F,0x5441,0x8F8F,0xD4C4,0x64D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4C5,0xD4E4,0xE4F4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xD5D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x4545,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x8554,0x8585,0x8585,0x7585,0x8585,0x7575,0x8585,0xB485,0x9585,0x85B5,0x75C4,0x8585,0xB485,0xC495,0x7595,0x8575,0x7575,0x7585,0x8575,0x8575,0x75B4,0x8575,0x8585,0xB485,0xB485,0x9585,0x85D4,0x4665,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5E4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E5,0xF4E4,0xE5E4,0xF4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x64D4,0xD464,0x4141,0x5454,0x6565,0x7575,0x5475,0x5454,0x5454,0x5454,0x64D5,0x64D5,0xC4C4,0x8585,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64D4,0x5454,0xD5D5,0xD5C4,0x5151,0x5441,0x6464,0xC454,0x41D4,0x4141,0x5454,0x4444,0x4441,0x4444,0x4444,0xD454,0x7F7F,0xF664,0x41D4,0x4444,0x4444,0x5454,0x4444,0x5151,0xD4C4,0x5454,0xD4D4,0x5454,0x6464,0xD4E4,0xD464,0xD4D4,0xE4D4,0xE4D4,0xD4C5,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xD4D4,0xD5D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xC4D4,0xD4D4,0x4464,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0xB444,0xB485,0xB485,0xB485,0xB485,0x8575,0x85B4,0x95C4,0xC4B4,0xB485,0x8585,0x85B4,0x7575,0x8575,0x6575,0x8575,0x85C5,0x7575,0x7575,0x7585,0x7575,0xB485,0x7595,0xD4B5,0x7585,0xB485,0x6595,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xD4E4,0xE4D5,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xF4E4,0xE5E4,0xE4E4,0xE4F4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0x6464,0x5454,0x5454,0x7575,0x7575,0x7575,0x5454,0x5454,0x5454,0x5454,0x4141,0xD464,0x64D4,0x64D5,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0x54D4,0xD4D4,0x64D4,0x64D4,0x64D5,0x9F9F,0xC4C4,0x5454,0xD4D4,0x7F7F,0x6444,0x8F8F,0xE4D4,0x6464,0x6464,0xD464,0x5454,0x5454,0xE4E4,0xD4E4,0x5441,0xD464,0xC4E4,0x6464,0x5151,0x7575,0xD4E4,0x8F8F,0x4444,0x4141,0x6464,0xD485,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xC5D4,0xD4E4,0xD4E4,0xE4E4,0xE4D5,0xE4E5,0xE4D4,0xD5D4,0xD4E4,0xE4D4,0xD4E4,0x64E4,0xD4E4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4564,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7546,0x75B5,0x8585,0x7585,0x8575,0x7585,0x8575,0x95B4,0x8575,0x8585,0x95C4,0xB485,0x7585,0x6575,0x7564,0x8585,0x7575,0x85D4,0x7575,0x7575,0x8585,0xC495,0x8575,0x9585,0x75B4,0x9585,0x4675,0x4644,0x5444,0x4654,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4F4,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD474,0x6464,0x7575,0xD564,0xD564,0xD564,0x6464,0x7575,0x5475,0x5454,0x4444,0x5454,0x4444,0xD464,0xC4C4,0xD464,0xD564,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x6454,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4444,0x4141,0x7F7F,0x5454,0xD554,0xD4D4,0xC4C4,0x8F8F,0x6464,0x4444,0x7575,0x6464,0x6464,0x4444,0xD541,0xE4E4,0x5454,0x5454,0xD444,0xD4D4,0x6464,0x4444,0x4141,0x4444,0x4444,0x6464,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4C5,0xE4D4,0xE4E4,0xE4E4,0xF4D4,0xE4E4,0xF4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D5,0xE4D4,0xD4C5,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xC4D4,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4446,0x4646,0x7575,0x75D4,0xB485,0x8585,0xB485,0x8585,0x8575,0xC485,0x7585,0x95C4,0x85C4,0x8585,0x85B4,0x5475,0x7575,0x8575,0x75C5,0x7585,0x7585,0x8575,0x85C4,0x75B4,0xB485,0x85B5,0x8575,0xB4C4,0x4665,0x5444,0x6585,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xD5D4,0xD4E4,0xE4C5,0xD4D4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4D5,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xC5E4,0xD4D4,0xD4D4,0x6464,0x64C4,0xC4C4,0xD5D5,0xC4C4,0x6464,0x64C4,0x7575,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0xD564,0xD4D4,0x64C4,0x74D4,0xD464,0x74D4,0xD4D4,0xD4D4,0x54D4,0x5454,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x4444,0x6454,0x54D4,0x6464,0x6464,0x5454,0xC454,0xD4D4,0x4444,0x4141,0x4444,0x6464,0x6464,0x64E5,0x5454,0x5454,0x64D5,0x4444,0x7F7F,0x4141,0x4141,0x5454,0x6464,0xD4D4,0xC4D4,0x6464,0xC5C4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD5E4,0xF4E4,0xE4E4,0xF4E4,0xE4D4,0xE4D4,0xE4D4,0xD4C5,0x64D4,0xC4C4,0xF4D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xC5D4,0xE4D4,0xD4D4,0x64D4,0x4664,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x5446,0x8585,0x85C4,0x75B4,0x7585,0xB485,0x7575,0xB485,0x7585,0x7575,0xC485,0xB495,0x7575,0x5475,0x8575,0x85C4,0x7585,0x7585,0x7575,0x8585,0x85B4,0x8585,0xB5D4,0x7585,0xC485,0x4585,0x6465,0x8595,0x4645,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD5D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x7474,0x64C4,0xD5D5,0x8585,0x74C4,0xD564,0x6464,0x7575,0x6464,0x7575,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0xC485,0xC4C4,0xD5D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x51D4,0x5454,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0x8F8F,0x5454,0x4141,0x5454,0xD4D4,0x5454,0x64D4,0x4444,0xD554,0x5454,0xC4C4,0xE4E4,0xD4D4,0x4444,0x5454,0x5454,0x6464,0x4141,0xE444,0xD4D4,0x4444,0xD4D4,0x4444,0xD4D4,0x64D4,0xD464,0xE4D4,0xC5D4,0xD4D4,0xC5D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4C5,0xD4E4,0xD4C5,0xD4E4,0x41D4,0xE464,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6546,0x9585,0x8575,0x8585,0x8575,0x8585,0x6585,0x8585,0xB495,0x8585,0x8565,0x8585,0x5475,0x5475,0x8585,0x8575,0x7575,0xB485,0x8575,0x8575,0xB485,0x9575,0xB4C4,0xB485,0xB495,0x7575,0x8585,0x45B4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4C5,0xD4D4,0x64D4,0x64D5,0x64D5,0xD4D4,0xD5D4,0xC4C4,0xC4C4,0x6464,0x7575,0x5475,0x5454,0x5454,0x5454,0x4444,0x4454,0x4444,0x5454,0xD4C4,0xD464,0xD464,0x6464,0xD4C4,0xD4D4,0xD4D4,0x5454,0xE464,0xD4D4,0xC5D4,0xD4D4,0x51D4,0x4144,0x6475,0x4444,0x5454,0x5454,0xD4C4,0x4141,0xC4C4,0x4141,0xC4C4,0x6464,0x64D4,0x64D5,0xD4D5,0x64D4,0x6464,0x5454,0x5454,0x6444,0xD4D5,0x64D4,0x6464,0xE4D4,0x4141,0xD454,0xD454,0x6464,0xD464,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D5,0xD4E4,0xD4D5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x6464,0xD4D4,0xE4E4,0xD5E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0xB475,0xB485,0x7595,0x7585,0xC475,0x8595,0x7575,0x8585,0x75B4,0x7575,0xB485,0x7585,0x7554,0x8575,0x7565,0x7575,0x7595,0x8585,0x75B4,0x8575,0xD4D4,0x8585,0x95D4,0x75B4,0x9585,0x75C4,0x4545,0x6454,0x4644,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4446,0x5F46
.word 0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xD5E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xC5E4,0xD4E4,0xD4D4,0xD4D4,0xD564,0xD4D4,0x8585,0xD4D4,0x64D4,0x85D4,0xD4D4,0xC4C4,0x64D5,0x7575,0x7575,0x7575,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0xD4D4,0x64D5,0x85D5,0xD4D4,0xD4D4,0x54D4,0x9F9F,0xD454,0xD4D4,0xD4D4,0x64D4,0x6464,0x5454,0xD5D5,0x6464,0x5454,0x5444,0x6464,0x4444,0xD541,0x6464,0x4444,0x6464,0x5454,0xC4C4,0x4141,0xE564,0xC4C4,0x5475,0x4444,0xD4D4,0x64D4,0x8F8F,0xD4D4,0x4141,0x4444,0x54E4,0xD4D4,0xD5D4,0x6464,0xD4D5,0xD4E4,0xD4D4,0xE4D4,0xF4C5,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0x64D4,0x7575,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0x64D4,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x9575,0x85B4,0x85B4,0x85B4,0x8575,0x95B4,0x75C4,0x8575,0x9585,0x8575,0x5475,0x7575,0x8575,0x8575,0x8575,0x7585,0x85C4,0x75B4,0x7575,0x8585,0x8585,0xA5B4,0xC495,0x8585,0xC485,0x6585,0x8585,0x4575,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4C5,0xE4C5,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4D4,0xD5D4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x6464,0xD564,0x6464,0x7575,0x7575,0x5454,0x4444,0x5441,0x5454,0x5454,0x5454,0x5454,0xC4C4,0xC4C4,0xC4C4,0xD464,0xD4D4,0x5454,0x5454,0xD4D4,0xD4D4,0xD4D4,0x6464,0xD575,0x5151,0x4444,0xD4C4,0xD4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0x4141,0x4444,0x6464,0x4444,0x7F7F,0x4141,0xD464,0xD4D4,0x6441,0x5454,0x5454,0xD444,0xE464,0x6464,0xE454,0xD4D4,0xD4D4,0xD441,0xC4C4,0x6464,0xD4D4,0xE4E4,0xD4D4,0xE4D4,0xE4E5,0xF4E4,0xE4E5,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0x64D4,0x64D5,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xC5D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x64D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7544,0x9585,0x7585,0x8585,0x75B4,0x8585,0x85D4,0x7575,0x7575,0xB485,0x7595,0x5475,0x7575,0xC475,0x7585,0x6575,0x7585,0x7575,0x8575,0x95B4,0x9575,0x85C4,0x85C4,0x95B4,0x75B4,0xD485,0x6575,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446
.word 0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD5D4,0xE4D4,0xE4E4,0xE4D4,0xD5E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4E4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4C4,0x6464,0x6475,0x7575,0x5454,0x5454,0x5454,0x4444,0x5454,0x5454,0x5454,0x5454,0x6441,0x64D5,0x64D4,0xD4D4,0x64C4,0x5454,0x5454,0xD4D4,0xD4D4,0x64D4,0x6565,0xE4E4,0x4444,0x4441,0x5454,0x54E4,0x4141,0xD454,0xD4E4,0xD4E4,0x5454,0x6464,0x5454,0x6464,0xD464,0xD4D4,0xD4D4,0x6464,0x5454,0x5454,0x4444,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0x7F7F,0xD454,0xD4D4,0x6464,0xD4D4,0xD464,0xC5D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0x74D4,0x5454,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x74D4,0xD4E5,0xD4D4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0x4664,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4544,0x85C4,0x95B4,0x75C4,0xB495,0x7575,0xC485,0x6585,0x7575,0x7585,0x7575,0x7554,0x8585,0x7585,0xC485,0x5485,0x7585,0x7575,0x8585,0xB485,0xD495,0xC495,0x8585,0xB485,0x8585,0x8595,0x4665,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646
.word 0xD4D4,0xC5D4,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xC5E4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64D5,0x7575,0x7575,0x7575,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4444,0x5454,0xD4D4,0xD464,0xD4D4,0x5454,0x5454,0x6454,0xD4D4,0x54D4,0x6464,0x6454,0x4444,0x4141,0xD4D4,0x4141,0x4444,0x8F8F,0x5441,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xC4C4,0xD4D4,0x6464,0x4141,0xD4D4,0xD4D4,0xE4E4,0x5441,0x8F8F,0xD444,0x6464,0x6444,0x7F7F,0x7F7F,0x7F7F,0x4444,0x5454,0x6464,0xC4C4,0xD4E4,0xE4E4,0xE4F4,0xE5E4,0xE4D4,0xE4E4,0xD4C5,0xD4D4,0x64D4,0x64D5,0xD4D5,0xE4D4,0xF4E5,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0x74D4,0xC4D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x9554,0x95B4,0x9575,0x8575,0x7585,0x8575,0x8585,0x7585,0x7575,0x7585,0x5475,0x7585,0x85B4,0x7585,0x8575,0x75B4,0x9585,0x75B4,0x8585,0x85B4,0x85B4,0x85B4,0x8575,0xB485,0x65C4,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4444,0x4646,0x4644
.word 0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xD5D4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xC4C4,0x6464,0x64C4,0x7575,0x5454,0x5454,0x5454,0x6464,0x5454,0x5454,0x5454,0x5454,0x5454,0x6441,0x64D5,0xD4D4,0x44D4,0x5454,0x4444,0xD4D4,0xD4D4,0x5464,0xD5D5,0x4141,0xD444,0x5454,0x4444,0x4444,0x4444,0x8F8F,0x4141,0xE454,0xD4D4,0x4441,0x6464,0xD4D4,0xE444,0xE4E4,0x4444,0xE444,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0xC441,0xAFAF,0x7F7F,0x4141,0x4441,0x6441,0x4141,0x4444,0x6475,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x5454,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x8544,0xB485,0x7595,0x75C4,0x8585,0x7575,0x7575,0x7585,0x8575,0x7575,0x7575,0xD475,0x8585,0x7565,0x8554,0x8575,0x7575,0x7585,0x85C4,0x8585,0x8595,0xB485,0x7585,0x8595,0x4675,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4E4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE5E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4C5,0xE4D4,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x74D4,0xD4D4,0xD4D4,0xD564,0x64D4,0x7575,0x6464,0x7575,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4444,0x8F8F,0x5454,0x64D5,0xD4D4,0x5441,0x5454,0x5454,0x64D4,0x6464,0x4444,0x6464,0xD4D4,0x6464,0x4141,0x4444,0x4141,0x5444,0x5454,0xC4C4,0xD4D4,0x41E4,0xE564,0xD4E4,0xC4D4,0x5464,0x5454,0x6464,0x64E4,0x4141,0x4444,0x6441,0x7F7F,0xD444,0x6464,0x4444,0x7F7F,0x6464,0xD544,0xC4C4,0x8F8F,0x6475,0xE464,0xE4E4,0xD5D4,0xD4E4,0xE4E4,0xE4D4,0x64D4,0xC4D4,0x6441,0xD4D4,0xE4E4,0xE4E5,0xE4E4,0xD4D5,0xD4E4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4C5,0xD4D4,0x6464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0xC485,0x8575,0x7575,0xC495,0x7585,0xB485,0x7585,0x75C4,0x7565,0x8554,0x7575,0x7585,0x6585,0xB485,0x8575,0x8575,0x75B4,0xC485,0xB495,0x85C4,0x75B4,0xB495,0x7585,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46
.word 0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xD4E4,0xE4F4,0xE4E4,0xE4E4,0xD5D4,0xE4D4,0xC5E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xC4C4,0xD5D5,0x7575,0x5475,0x5454,0x5454,0x5454,0x7575,0x5454,0x5454,0x5454,0x4444,0x4444,0x4444,0x4141,0xD4C4,0x4444,0x5454,0x5454,0xD4D4,0x5454,0x5454,0x4454,0xE541,0x5454,0x5454,0x5454,0x5454,0x6464,0xD4D5,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xC4D4,0x6464,0x9F9F,0x4444,0x6464,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0x6464,0x41D4,0x4154,0xC4C4,0x54D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64C4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD564,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD464,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0x4664,0x4644,0x4646,0x4444,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x8565,0x7595,0xB485,0x8575,0x85B4,0x85C4,0x8585,0x7585,0x6564,0x8585,0x75C4,0x7585,0x7575,0x7575,0x7585,0x7575,0x9585,0xB5C4,0x75B4,0x8575,0x8585,0x8575,0x4575,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4444,0x4646,0x4446,0x4646
.word 0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD5E4,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xE4D5,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D5,0xD4E4,0xD4C5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x6485,0x7575,0x6464,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4444,0x4444,0x4141,0x7F7F,0x6464,0xD4D4,0x5454,0x4444,0x5454,0x5454,0x4444,0x5454,0x5454,0x6464,0xD4E5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD485,0x64D4,0x6464,0x5454,0x5454,0x4444,0x4141,0x6444,0xC4C4,0x4444,0x4141,0x4141,0x4444,0x6464,0xD4D4,0x4444,0xD4D4,0xD4D4,0xD4D4,0xD441,0xC441,0x6441,0x6464,0x74D5,0xE4E4,0xC5D4,0xD4E4,0xC5D4,0x64D4,0x6464,0xD564,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD5D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4446,0x8565,0x64B4,0x7575,0x9585,0x85C4,0x7575,0x6575,0x8554,0x85D4,0x7575,0x7565,0x8554,0x7575,0x8575,0x8585,0x85C4,0x8575,0xC495,0x8575,0x95B4,0x75B4,0x7554,0x7575,0x6475,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46
.word 0xD4D4,0xC5D4,0xE4D4,0xC5D4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4C5,0xD4C5,0xD4E4,0xE4D4,0xD4D4,0x64D4,0xC4D4,0xC4C4,0xD564,0x5454,0x7575,0x6454,0x5454,0x5454,0x5454,0x5454,0x4444,0x4154,0x4444,0x4444,0x4141,0x7F7F,0x6444,0x54D5,0x5454,0x5454,0x5454,0x5454,0x4444,0x5454,0xD4D4,0x6475,0xD564,0xD4D4,0xD464,0x6464,0x6464,0x5454,0x4444,0x5454,0x4444,0x4141,0x4444,0x5441,0x54F5,0x6464,0x6464,0x6464,0x6464,0x6464,0x5454,0xC4C4,0xE4D4,0xD4E4,0x41E4,0x64D4,0xD4D4,0x6464,0xC4C4,0x64D5,0x6464,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x5454,0xD4D4,0xD4D4,0xE4E4,0xD5E4,0xE4E4,0xD4E4,0xD4E4,0xE4C5,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x6464,0x4685,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6544,0x8575,0x7575,0x7585,0xB485,0x6585,0x5475,0x7595,0x7575,0x8575,0x7585,0x8575,0x7575,0x7585,0x85B4,0x8575,0x85C4,0x85B4,0x9585,0x75B4,0x7585,0x8575,0x4565,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646
.word 0xC5D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4C5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0xC4C4,0xD4D4,0x6464,0x6464,0x7575,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4444,0x4141,0x7F7F,0x6F6F,0x4646,0x4646,0x6444,0x64D4,0x5454,0x5454,0x5454,0x5454,0x4444,0xC4C4,0x9F9F,0x4141,0x4141,0x5454,0x4444,0x4444,0x4444,0x5454,0x4444,0x4141,0x4444,0x4444,0x5454,0xE4E4,0x4444,0x8F8F,0x8F8F,0x8F8F,0x4444,0x5454,0x5454,0x5451,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xD4D4,0x4444,0x6454,0xE4D4,0xD4E4,0xD4D4,0xD464,0xD564,0xD464,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0x64D4,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x6544,0x7575,0x7585,0x7575,0x5475,0x7575,0xB485,0x8575,0x7585,0x7575,0x8575,0xD485,0xB5B5,0xB485,0x7585,0x8575,0x8585,0x85B4,0x8575,0x8585,0x6565,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xC5E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4F4,0xE4E5,0xE4E4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xC5D4,0xE4D4,0xD5D4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xC4C4,0xC4C4,0x7575,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4141,0x4444,0x7F7F,0x7F7F,0x4646,0x4646,0x4646,0xC4C4,0x54D4,0x5454,0xC4C4,0xD4C4,0xD4C4,0x4141,0x4454,0x4444,0x4444,0x4141,0x4444,0x5454,0x4141,0x4444,0x4444,0x4444,0x4141,0x4444,0x4141,0x54E4,0x4141,0x4444,0x4441,0x4444,0xC4C4,0xE4E4,0x4141,0x8F8F,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0x6464,0xD4D4,0xD4D4,0xC4C4,0x6464,0xC4C4,0xD4D4,0xD5E4,0xF4E4,0xD5E4,0xD4E4,0xD4E4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD474,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x7546,0x4665,0x4646,0x7544,0x7585,0x7554,0x7554,0x5475,0x9575,0x6575,0x8575,0x8565,0x8575,0x95D4,0xC4C4,0x8585,0x8575,0xC485,0x7585,0x7575,0xB485,0x5475,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xD4D4,0xD4D4,0xE4E4,0xE4C5,0xE4E4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4C5,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0x64C4,0xD564,0x5454,0x7575,0x6454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4444,0x4141,0x4444,0x4646,0x4646,0x4646,0x4646,0xD541,0xC4D4,0x4141,0xC4C4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0x6464,0xC4C4,0x4141,0x4141,0x4444,0x4444,0x4141,0x4141,0x4444,0x4444,0xE454,0x41D4,0x41D5,0x8F8F,0x8F8F,0x4444,0x4144,0xE4E4,0xD4D4,0xC4C4,0x8585,0xD4D4,0xE464,0xE4E4,0xD4E4,0xD4D4,0xD485,0xD4D4,0xD564,0xD4D4,0xD485,0xD4E4,0xD4C4,0xD541,0xD464,0xD4D4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4C5,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x46C4,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x6446,0x7585,0x4644,0x8545,0x7575,0x7565,0x7554,0x8575,0x7575,0xC585,0x7585,0x7575,0x7575,0x8565,0x8585,0x95B4,0x85C4,0x75B4,0x8585,0x85C4,0x6565,0x4644,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4444,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646
.word 0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE5E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4D4,0xC5E4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xC4C4,0xD5D5,0x6464,0x7575,0x6565,0x5454,0x5454,0x6464,0x5454,0x4444,0x4444,0x4141,0x4444,0x6F6F,0x4644,0x4646,0x4646,0x4646,0x4646,0xC454,0xC4D4,0x64D4,0x64D4,0xD4D4,0x5441,0x8F8F,0x4444,0x8F8F,0x4444,0x4444,0x4441,0x4444,0x4441,0x4444,0x4141,0x4141,0xE454,0x6464,0x6464,0x4144,0x4444,0x4141,0x6464,0xE4E4,0xE4E4,0x6464,0xC4C4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD441,0x74D4,0xD564,0x6464,0x6464,0x6464,0x6464,0xD4D5,0xE4D4,0xE4E4,0xE4E4,0xE4E5,0xD5E4,0xD4E4,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD474,0xD4D4,0xE4D4,0xD4D4,0x74D4,0x64D5,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x7544,0x8575,0x8585,0x7585,0x5475,0x5454,0x8585,0x7565,0x7575,0x8575,0x6585,0x7585,0x7575,0x7585,0xB485,0x8585,0x8575,0x8595,0x95B4,0x5475,0x4645,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4E4,0xC5E4,0xD4E4,0xE4E4,0xD5E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4C5,0xD4E4,0xD4C5,0xD4D4,0xD4E4,0xE4C5,0xE4E4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xC5D4,0xE4D4,0xC5D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD464,0xD564,0x6464,0x7575,0x7575,0x5454,0x5454,0x5454,0x4444,0x4444,0x8F8F,0x7F7F,0x7F7F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0xD541,0x4141,0x5454,0xD4D4,0x54D4,0x4444,0x4141,0x4444,0x4141,0x8F8F,0x4141,0x4444,0x4141,0x4444,0xD554,0xD4E4,0x4444,0x8F8F,0x4141,0x4444,0x4444,0xE454,0xE4E4,0xE4E4,0xE4E4,0x64D4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0x41E4,0xE4D4,0xC4C4,0xD4D4,0xD4D4,0xD564,0x6464,0x6454,0x64D5,0xD4D4,0xD4D4,0xD5E4,0xF4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0x64D4,0xD4E5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0x6464,0x4675,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4546,0xB495,0x8575,0x8575,0x7565,0x7575,0x8575,0x5475,0x7585,0x5475,0x8575,0x7575,0x8575,0x7585,0x8585,0x85D4,0x95C4,0x85C4,0x6575,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F
.word 0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D5,0xD5E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE5D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0x6464,0x6475,0x5454,0x5454,0x5454,0x5454,0x5454,0x4144,0x4444,0x4441,0x4141,0x7F7F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5441,0x64D4,0x6464,0x5454,0xD4D4,0x6464,0x4444,0x4444,0x4141,0x4444,0x4444,0x4444,0x4441,0x5454,0xE4D4,0x5464,0x4141,0xC4C4,0x4444,0x4141,0x4441,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xC5E4,0xE4E4,0xD585,0xE4D4,0x54D4,0x64D4,0xD4D4,0xD4D4,0x4444,0x8F8F,0x5454,0xD4C4,0xE474,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0x64D4,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4444,0x4646,0x4475,0x4646,0x4644,0x9575,0x7585,0x7575,0x5454,0x8575,0x7575,0x8575,0x7575,0x7585,0x7565,0x8585,0xB485,0xC485,0xB485,0x8595,0x75B4,0x6575,0x4645,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4444,0x4646,0x4446,0x4646
.word 0xC5D4,0xD4E4,0xE4E4,0xD4D5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D5,0xE4E5,0xE4E4,0xE4E4,0xE4D4,0xC5D4,0xE4D4,0xD4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xE4C5,0xE4E4,0xE4D5,0xE4E4,0xD5D4,0xD4E4,0xD4E4,0xE4E4,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0xC4C4,0xC4C4,0x6464,0x6464,0x6565,0x7575,0x5454,0x4444,0x4141,0x7F7F,0x7F7F,0x4444,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5144,0x5454,0x7F7F,0xC4D5,0x64D4,0x4141,0xC4C4,0x64D4,0x4444,0x4444,0x4141,0x4144,0x5454,0xE4D4,0xE4E4,0x64D4,0xD4D4,0x64D4,0x4444,0x4444,0x4444,0xD4D4,0xF4E4,0xE5E4,0xE4E4,0xD4E4,0x64D4,0xE4D4,0xE4E4,0xE4D4,0x41E4,0xE4D4,0xD4E4,0xE4E4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0x4444,0x4441,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0x64D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x9585,0x4644,0x4646,0x7544,0x85C4,0x75C4,0x8575,0x7565,0x85D4,0x6575,0x8575,0x75D4,0x7575,0x7575,0x5465,0x6575,0x4465,0x4646,0x4644,0x4446,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE5E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xC5E4,0xE4D4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4E4,0xE4C5,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xD5D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0x6464,0xC4C4,0x6565,0x5475,0x5454,0x4444,0x4444,0x4444,0x4441,0x7F7F,0x7F7F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4546,0x5454,0x5454,0x4144,0x4141,0xD4C4,0xC4D4,0x5454,0xD464,0xD4D4,0xD4D4,0x5454,0xD564,0xE4E4,0x54D4,0x5454,0x4141,0x4141,0x8F8F,0x4444,0x4141,0x5454,0xE464,0xE4F4,0xE4F4,0xD4E4,0xD4E4,0xD464,0xD4D4,0xE4E4,0xE4E4,0xD444,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x5151,0x4141,0x8F8F,0xD441,0xD4D4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0xB485,0x4475,0x4646,0x4446,0x9585,0x5485,0x6465,0x7554,0x7585,0x9595,0x7575,0x8585,0x6585,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xE4D4,0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E5,0xE4E4,0xD4E4,0xE4D5,0xD4D4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xF4D4,0xD4E4,0xD4F4,0xD4E4,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD485,0x64D5,0x6464,0x6475,0x5454,0x5454,0x5454,0x4444,0x5454,0x7F7F,0x7F7F,0x6F6F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4144,0x5454,0x4144,0x4444,0x4444,0x6464,0xD4D4,0x64D4,0xD4D5,0x51D4,0xD4D4,0xD4D4,0xD4D4,0x5151,0x4141,0x4444,0x4444,0x7F7F,0x4444,0x4141,0x5444,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xC4C4,0xD4D4,0x41D4,0xD4E4,0xE441,0xD441,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0x4444,0x4444,0x4441,0x4141,0xE454,0xF4E4,0xE4F4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xE4D4,0x74D4,0xD564,0x4664,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0xB565,0x8585,0x8585,0x8575,0xB485,0x7585,0x7575,0x8585,0x8565,0x7565,0x8575,0x85B4,0x65B4,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646
.word 0xD4E4,0xD4E4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xD4D4,0xE4D5,0xF4E4,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E5,0xE4F4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xC4C4,0xC4C4,0x7575,0x6565,0x5454,0x5454,0x4154,0x4444,0x4141,0x4444,0x6F6F,0x4444,0x4646,0x4646,0x4646,0x4646,0x5441,0x4444,0x5454,0x4141,0x4444,0x4141,0x6454,0xD4D4,0x8585,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5454,0x5454,0x51D4,0x8F8F,0x7F7F,0x4444,0x5454,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xE474,0xE4E4,0x64D4,0x8F8F,0x4141,0x4141,0xD441,0xE4D4,0x8585,0x4141,0x4444,0x7F7F,0x4444,0x6464,0xE4D4,0xE4E4,0xE4D5,0xD5D4,0xD4E4,0xC5E4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD5D4,0x5454,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x7446,0x85B4,0xB585,0x94D4,0x95D4,0x75B4,0x5475,0x7565,0x8585,0x7575,0x7554,0x9575,0x6475,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644
.word 0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D5,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xC5D4,0xD4E4,0xE4D5,0xE4E4,0xF4E4,0xF4E4,0xE5E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4C4,0x6464,0x64D5,0x6464,0x7575,0x5454,0x5454,0x4444,0x4441,0x7F7F,0x4141,0x4646,0x4646,0x4646,0x4646,0x4146,0x5454,0x4444,0x5454,0x5454,0x4444,0x4444,0x8F8F,0x6464,0xD4D4,0x6464,0x4141,0x6441,0xD4D4,0xD4D4,0xD454,0x6464,0xD4D4,0xD4D4,0x4154,0x5454,0xD4D4,0xE4E4,0xE4E4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0x5454,0x5464,0x6464,0x5454,0x5444,0xC4D4,0xE4E4,0x5451,0xD4D4,0x6444,0x4444,0x5454,0x6464,0x5444,0x6464,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0x64D4,0x64D4,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4446,0x95C4,0x85B4,0xD4B4,0xD495,0x8575,0xD485,0x7575,0x6585,0x8575,0x7575,0x8585,0x7575,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xE4E4,0xD4E4,0xD4C5,0xD4C5,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xF4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xD4D4,0xD564,0x5454,0x6565,0x7575,0x5454,0x4444,0x8F8F,0x6F6F,0x4141,0x4646,0x4446,0x4646,0x4446,0x4144,0x5454,0x5454,0x5454,0x5454,0x4444,0x4441,0x4441,0x4444,0xC4C4,0xD4D5,0xD564,0xD5D4,0x64D4,0x6464,0x64D5,0x4141,0xE4C4,0x54D4,0x5454,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0x54D4,0x4444,0x4141,0xD464,0xE4E4,0x4441,0x4141,0xD564,0x5454,0x5454,0x4141,0xD4D4,0x4141,0x5454,0xD4D4,0x4141,0x5454,0x6464,0x5454,0x6464,0x5454,0xD454,0xE4E4,0xD4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6546,0xC495,0x9595,0xB5D4,0x5465,0x8575,0x7575,0x6595,0x5475,0x5475,0x7575,0x7575,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4C5,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0x64D5,0x6464,0x7575,0x5454,0x5454,0x5454,0x4444,0x4444,0x7F7F,0x6F6F,0x4646,0x4646,0x4646,0x4446,0x5454,0x5454,0x5454,0x5454,0x5454,0x4154,0x5454,0x5454,0x4141,0x4444,0xD464,0xD4D4,0x74C4,0x5454,0xD4D4,0x9F9F,0x5454,0x4441,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xC5E4,0xE4D4,0xE4D4,0xD4D4,0xD454,0x4141,0x6454,0xD5D5,0xD564,0x9F9F,0x4141,0x5454,0x6464,0x4141,0xD454,0xD4D4,0x4144,0x4444,0x7F7F,0x4444,0x6444,0x6464,0x5454,0x6464,0xE444,0xE4E4,0xD4D4,0xD4E4,0xC5D4,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD574,0xD564,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4444,0x4646,0x4646,0x4646,0x7574,0x4446,0x8575,0xD4C4,0x5475,0x7575,0x7554,0x7575,0x8575,0x8575,0x85C4,0x75D4,0x4485,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4C5,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4D5,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD564,0x6464,0x6464,0x6464,0x5454,0x5454,0x4141,0x7F7F,0x4141,0x4446,0x4646,0x4646,0x5441,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4444,0x4444,0x4141,0x4444,0x4141,0x64C4,0x4444,0x6464,0x5454,0x4141,0x5444,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0x5454,0x5454,0x4141,0x5454,0xD4D4,0xD4D4,0xD4E4,0x5454,0x4444,0x4444,0x6444,0x5454,0xC4C4,0x5454,0xD564,0x7F7F,0x4444,0x4444,0x4444,0x6464,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0x64D4,0x6464,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x8585,0x4675,0x8565,0x9585,0x7565,0x5485,0x8585,0x8575,0x6585,0x7585,0x7575,0x8585,0x7575,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F46
.word 0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xD4E4,0xC5E4,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE5E4,0xF4E4,0xF4E4,0xF4E4,0xE4E5,0xE5F4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4C5,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x6464,0x7575,0x6565,0x5475,0x4444,0x4441,0x4444,0x6F6F,0x4646,0x4646,0x4446,0x5454,0x5454,0x5464,0x6464,0x6464,0x6465,0x5454,0x5454,0x5454,0x4154,0x4444,0x4141,0x8F8F,0x5454,0x6464,0x5454,0x5454,0x6464,0xD4D5,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD485,0x4141,0x5454,0xD5D5,0xD4C4,0xD4D4,0x85D4,0xD4D4,0x4141,0xE4D4,0xD4E4,0x54D4,0x8F8F,0x4444,0x5454,0xD4D4,0x6464,0xD454,0xC4C4,0x5441,0x6464,0x8F8F,0xC464,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0x6464,0x7575,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x9544,0x8585,0x85C4,0x95B4,0x7575,0x7554,0x7565,0x7554,0x7585,0x7575,0x5454,0x7575,0x7585,0x4475,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xD5E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xE4C5,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D5,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x64D5,0x7575,0x5454,0x5454,0x5454,0x4444,0x7F7F,0x4141,0x4646,0x4646,0x5146,0x6454,0x5464,0x7575,0x5454,0x6565,0x5454,0x6464,0x7575,0x5454,0x5454,0x5454,0x4444,0x4454,0x4141,0x5454,0x4444,0x4441,0x5454,0x6464,0x6464,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD485,0x4141,0x8F8F,0x8F8F,0xD441,0xD4D4,0x5441,0x9F9F,0x5454,0xD564,0x4444,0xE464,0xD4E4,0x6464,0x4444,0x5441,0xD4D4,0x5441,0xD4D4,0x5454,0x6464,0x7575,0x6454,0x4141,0xD4C4,0xE4D4,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x7546,0xC485,0x95B4,0xD4D4,0x5485,0x7554,0x75D4,0x8575,0x9565,0x7575,0x9585,0x8575,0x7585,0x7585,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x465F
.word 0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xD4F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E5,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD585,0x6464,0x6475,0x6464,0x5454,0x5454,0x4141,0x7F7F,0x4646,0x4646,0x5441,0x6464,0x7575,0x7575,0x7575,0x6464,0x7575,0x7575,0x6454,0xD5D5,0x5454,0x4444,0x4144,0x9F9F,0x4444,0x5454,0x8F8F,0x4444,0x4141,0x4444,0x4141,0x5454,0x5454,0x4154,0x4141,0x5441,0x6464,0x5454,0x4444,0x4154,0xD4D4,0x4141,0x4444,0x6475,0x6464,0x4141,0xD4D4,0xD4D4,0x4141,0x5454,0xD464,0xD4D4,0x4141,0xD554,0x7474,0x4141,0x54D5,0xD4D4,0x6464,0x5454,0x6464,0xD4E4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD474,0xC5D4,0xD4D4,0xD564,0xC4C4,0x4641,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0xC454,0xB495,0xD485,0x7585,0x7554,0x5475,0x8585,0x8575,0x6465,0x7565,0xC475,0xB495,0x7585,0x7575,0x4654,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D5,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD5D4,0xD5D4,0xE4E4,0xE4D5,0xE4F4,0xE4E4,0xE4F4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0x7575,0x7575,0x5454,0x4444,0x7F7F,0x6F6F,0x4644,0x4545,0x6454,0x7575,0x7575,0x7575,0x6464,0xD575,0x6464,0xD575,0x6464,0x5454,0x6464,0x5454,0x4444,0x4444,0x4444,0x7F7F,0x4444,0x4141,0x4444,0x8F8F,0x4444,0x4444,0x4154,0x6464,0x64D5,0x7575,0x4444,0x4444,0x4141,0x4444,0x4141,0x6454,0x4444,0x8F8F,0x4444,0x4444,0x8585,0xD4D4,0xD4D4,0x5454,0x6464,0xD4E5,0xD4D4,0x4141,0x5454,0x54D4,0x4141,0xD554,0x6464,0x4444,0x7575,0x6464,0xD4E4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xC4C4,0x6464,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0xC475,0xD495,0xB495,0x7554,0x7575,0x7565,0x8575,0x7575,0x7595,0x7585,0xC485,0x9575,0x8575,0x5475,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4644
.word 0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E5,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xD5D4,0xE4D4,0xD4C5,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xE4D4,0xD4C5,0xE4E4,0xE4E4,0xE4F4,0xF4E4,0xF4F4,0xF4E4,0xF4F4,0xE4E4,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xC5D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x6464,0x7575,0x5454,0x5454,0x7F7F,0x7F7F,0x4646,0x6444,0x7575,0xD564,0x6464,0xD5C4,0xC4C4,0xC4C4,0xC4C4,0xD464,0xD5C4,0xD564,0x6464,0x6464,0x6464,0x5464,0x4141,0x5454,0x4444,0x4441,0x8F8F,0x4144,0x4444,0x4444,0x4444,0x4444,0x4141,0x4444,0x4444,0x4141,0x4141,0x4444,0x4141,0x5454,0x4444,0x4141,0x7F7F,0x4444,0x8F8F,0x6464,0xD4D4,0x5454,0x4444,0x6464,0x64D4,0x4444,0x8F8F,0x4141,0x4444,0x4141,0x5454,0xD4D4,0xC4D4,0x7575,0x5454,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xC4C4,0x6464,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4564,0x4646,0x4646,0x7544,0x8585,0x5454,0x5475,0x5475,0x7575,0x7554,0x6554,0x6575,0x5475,0xC485,0x9575,0x7585,0x7575,0x4665,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4444,0x5F5F,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x465F
.word 0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD5E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4C5,0xE4D4,0xD4E4,0xE4E5,0xE4E4,0xF4E4,0xE4E4,0xE4F4,0xE5E4,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xD4E4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD564,0xC4C4,0x7575,0x5454,0x4444,0x7F7F,0x4444,0x4644,0x6464,0xD5D5,0x6464,0xD5C4,0xD464,0xD585,0xD464,0xD5D4,0xD4D4,0x8585,0x6464,0x64D5,0xD5D5,0x6464,0x7575,0x5454,0x5454,0x4441,0x7F7F,0x4444,0x4141,0x4444,0x4141,0x4141,0x4444,0x4444,0x8F8F,0x4141,0x4444,0x4444,0x4141,0x4444,0x4444,0x4141,0x4444,0x4444,0x4141,0x4444,0x4444,0xD4D4,0x54D4,0xD5D5,0x4141,0xE454,0x41D4,0x4444,0x4444,0x4444,0x4444,0x4444,0xD441,0xD4D4,0xD564,0x6464,0x6464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7575,0x7575,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x9575,0x4645,0x4644,0x7545,0xC485,0x5475,0x5475,0x7575,0x8575,0x8565,0x54C4,0x7585,0x8585,0x9585,0x85C4,0x85B4,0x8575,0x6585,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4F4F,0x4646,0x5F5F,0x4646,0x4646,0x4646
.word 0xD4E4,0xE4D5,0xE4E4,0xE5E4,0xE4E4,0xE4D5,0xE4F4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E5,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x54D5,0x5454,0x5454,0x4141,0x7F7F,0x5146,0x6464,0x6485,0x64D5,0x61D5,0xD5D5,0x64D4,0xD4D4,0x8585,0xD4C4,0x64D4,0xD4D4,0xD4D4,0x6464,0x6475,0x7575,0x5464,0x5454,0x4141,0x4444,0x4141,0x4444,0x4141,0x4444,0x4141,0x4141,0x7F7F,0x4444,0x4444,0x4141,0x4141,0x4444,0x8F8F,0x4441,0x7F7F,0x7F7F,0x4141,0x4141,0x4141,0x4141,0xD4E4,0xD4E4,0xD4D4,0x4444,0x4141,0xD4D4,0x4141,0x4444,0x4141,0x4141,0x7F7F,0x4444,0xD4D4,0xD4D4,0xD564,0x6464,0xD464,0xD4D4,0xD474,0xD4D4,0x6464,0xC4C4,0x7575,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7446,0x7585,0x8575,0x8584,0x95B4,0x7585,0x7554,0x5475,0x8575,0x7554,0x7585,0x5475,0x6575,0x9565,0x6585,0x8585,0x8575,0x8575,0x5485,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4C5,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D5,0xE4E4,0xE5E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xC4C4,0xC4C4,0x6464,0x6464,0x5454,0x8F8F,0x4141,0x6441,0x64D5,0xD5D5,0xD4C4,0x64D4,0x64D4,0xD4D4,0xD4C4,0x64C4,0x74D4,0xD4D5,0xD4D4,0xD464,0x64D5,0xD5D5,0x6464,0x6464,0x7575,0x5454,0x4141,0x8F8F,0x7F7F,0x7F7F,0x4141,0x4444,0x4444,0x4141,0x4141,0x4141,0x4444,0x4141,0x4444,0x4141,0x4444,0x4141,0x4444,0x4141,0x4444,0x4444,0x4444,0xE454,0xD4D4,0xE451,0x4141,0x4444,0xD4D4,0x5454,0x4141,0x4141,0x4444,0x4144,0xC4C4,0x5454,0xD4D4,0xD4D4,0xC4C4,0x64D5,0x5454,0xD4D4,0x64D4,0xD564,0x6464,0x7575,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x8575,0xC485,0xC495,0xC485,0xB495,0x7554,0x7554,0x8575,0x6575,0x8575,0x7585,0x8585,0xB485,0x8595,0x85B4,0x8565,0x85B4,0x8575,0x5475,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4444,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4444,0x4646,0x4646,0x4446
.word 0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD4E4,0xE4C5,0xE4D4,0xE4C5,0xE4E4,0xE4D4,0xC5D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE5D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D5,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xC4C4,0x54C4,0x5454,0x4441,0x6F6F,0xC4C4,0xC4C4,0xD485,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD464,0xD4D5,0x64D5,0xC4C4,0x6464,0x5454,0x5444,0x4441,0x4441,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4444,0x4444,0x4141,0x4444,0x7F7F,0x4444,0x4141,0x4444,0x4141,0x4444,0x4141,0x4141,0x4141,0xE4E4,0xE4D4,0xD4D4,0x6464,0x4141,0x85E4,0xD4D4,0x5454,0x4444,0x4141,0x8F8F,0x4444,0x6464,0xD4D4,0xD4D4,0xD4D4,0xD564,0x6464,0x51D5,0x6464,0x5454,0xC4C4,0x6464,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x9575,0x85C4,0xB485,0xC5B5,0x7585,0x5454,0x7554,0x7575,0x7554,0x6575,0x6575,0x8575,0x7575,0x8585,0x8565,0x7595,0x7585,0x8585,0x4465,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x5F46
.word 0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE5E4,0xE4E4,0xE4F4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E5,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E5,0xE4F4,0xF4E4,0xE4E4,0xE5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x6464,0x6464,0x4444,0x5454,0x64D4,0x64D4,0x64D5,0xD464,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7474,0x64C4,0x64D5,0x6464,0x6464,0x5454,0x8F8F,0x7F7F,0x4441,0x4444,0x4141,0x4444,0x4141,0x4141,0x4444,0x4141,0x4444,0x4141,0x6464,0x7F7F,0x4444,0x4141,0x4444,0x4444,0x4444,0xD444,0xD4D4,0xD4D4,0x44E4,0x4444,0xC4E4,0x5454,0x5454,0x4141,0x4444,0x4141,0x6441,0x41E5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x6565,0x7575,0x5454,0x5454,0x5464,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x6546,0xC495,0xC495,0xC495,0xC495,0x7585,0x5454,0x7554,0x8575,0x8575,0x64D4,0xC485,0x8575,0x95B4,0x7575,0x6585,0xB495,0x8575,0x6585,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646
.word 0xE4E4,0xD4E4,0xE4D5,0xD5E4,0xF4E4,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xC5D4,0xE4E4,0xC5D4,0xE4D4,0xD4E4,0xD4D5,0xE4E4,0xD4E5,0xE4F4,0xE5E4,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xD4E4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD574,0xD564,0x5454,0x4444,0x6464,0xD5D5,0x64D4,0xD4D4,0xD4D5,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0xC4C4,0xC4C4,0x7575,0x5454,0x4454,0x4441,0x8F8F,0x4141,0x4444,0x4444,0x5454,0x6454,0x5454,0x6475,0x5454,0x6464,0x4444,0x4141,0x7F7F,0x4141,0x4141,0x4141,0x4444,0xD464,0xD4D4,0xD464,0xD4D4,0x4141,0xC4E4,0xD564,0x5454,0x4444,0x6464,0xD4D4,0xD4E4,0xD4E4,0xD441,0xD4D4,0xD4D4,0xD464,0xD4C4,0x64D5,0x64D5,0x5454,0x7575,0x7575,0x5454,0x5454,0x5451,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x85B4,0xC485,0x6585,0x5454,0x7565,0x7554,0x7554,0x7565,0x8554,0x75C4,0xD475,0x7595,0x9575,0xB485,0x8575,0x4575,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xE4D4,0xE4C5,0xE4D4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4F4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xD4C5,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4C5,0xE4D4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xF4F4,0xE5E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4D5,0xD4E4,0xE4E4,0xE4D4,0xC5E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x6464,0x4154,0x6464,0x64C4,0xD4D4,0xD464,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD564,0xD585,0x6464,0x64C4,0x5454,0x4141,0x4444,0x7F7F,0x4444,0x4141,0x4141,0x5454,0x5454,0x5454,0xD554,0x5454,0x7F7F,0x7F7F,0x4141,0x4444,0x4444,0x4141,0x4444,0xD464,0xD4D4,0xD464,0xD4D4,0x4444,0xD4E4,0xBFBF,0x5454,0x4141,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xD464,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0xC4C4,0x6464,0x5454,0x5454,0x6475,0x5454,0x5454,0x5154,0x5454,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x7546,0x8585,0x8585,0x5475,0x5454,0x7554,0x7585,0x9585,0x7575,0x5485,0x7575,0x6585,0xD485,0x8585,0x75D4,0x8585,0x4446,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4446,0x4646,0x4644,0x4644,0x465F
.word 0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xE4F4,0xE4E5,0xE4F4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE5E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD474,0xC4C4,0x64C4,0xC4C4,0x64D5,0xD4D4,0xC4C4,0xD4D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8585,0x64D5,0xC4C4,0x7575,0x5454,0x4444,0x4441,0x4141,0x4444,0x4444,0x4444,0x5454,0x5454,0x8F8F,0x8F8F,0x4444,0x4141,0x4141,0x4141,0x4141,0x4444,0x7F7F,0x6464,0x6464,0x8F8F,0x5454,0xD441,0xC4E4,0x41D4,0x5454,0x6464,0xD4D4,0xD4D4,0xD4D4,0x6464,0xE4E5,0xE4D4,0xD4D4,0xD4D4,0xC4C4,0xD4D4,0xD564,0x7575,0x5454,0x7575,0x7575,0x7575,0x5454,0x6464,0x5454,0x5164,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0xC454,0xD485,0x95B4,0x95C5,0x75C4,0x5454,0x7554,0x8575,0x8565,0x7575,0xD485,0x8554,0x7575,0x7575,0x8585,0x8575,0x4475,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xD4E4,0xE5D4,0xE5D4,0xE4E4,0xE5E4,0xE4F4,0xF4E4,0xE4F4,0xE4E4,0xE4F4,0xE4D5,0xE4E5,0xD4F4,0xE5E4,0xE4E4,0xE4E4,0xF4E4,0xE5E4,0xE4E4,0xE4D5,0xE4E5,0xE4E4,0xD5E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x7575,0xC4C4,0xD4D4,0xD564,0xD485,0xD4D4,0xD4C4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD564,0x6464,0x7575,0x5454,0x5454,0x4444,0x8F8F,0x4141,0x7F7F,0x8F8F,0x4444,0x4141,0x4141,0x4141,0x4444,0x4444,0x7F7F,0x4444,0x6464,0x6485,0xD4D5,0xC4C4,0x6464,0x4444,0xE4D4,0x64D4,0x6464,0x4444,0x64E4,0x64D4,0x9F9F,0xE4D4,0x6464,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x64D4,0x64D5,0x6464,0x5454,0x5454,0x6565,0x6464,0x6465,0xD5D5,0x6464,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x9564,0x85C4,0xB495,0xB495,0xD485,0x85B5,0x5475,0x5465,0x7565,0xC485,0x8575,0x8585,0x8575,0x75C4,0x8575,0x7565,0x7585,0x7585,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F
;*/

/* Dragon Girl
Fish_320:
.word 0x4646,0x4646,0x4646,0x4646,0x8575,0x8575,0x7575,0x4646,0x4646,0x4646,0x4646,0x4646,0x7544,0x5465,0x7575,0x4645,0x4646,0x4646,0x7565,0x5454,0x5465,0x6585,0x4646,0x6546,0x7554,0x8575,0x4646,0x4646,0x4646,0x4646,0x6544,0x7554,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4141,0x4444,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4646,0x6546,0x4665,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5444,0x8565,0xD4D4,0x4575,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6544,0x8585,0x95D4,0x7585,0x5465,0x4645,0x4646,0x4646
.word 0x4646,0x4644,0x4646,0x4644,0x4546,0x7595,0x7575,0x8585,0x4675,0x4644,0x4646,0x4646,0x5446,0x5475,0x5475,0x6475,0x4645,0x4544,0x7575,0x7554,0x7554,0x5465,0x7585,0x7554,0x7554,0x7554,0x4675,0x4446,0x4646,0x4444,0x7554,0x7575,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x6F6F,0x7F7F,0x6F6F,0x4444,0x4444,0x8F8F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4544,0x54D4,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x7541,0x9585,0x95D4,0x5475,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4444,0x8554,0xB5D4,0x75B4,0x4575,0x4646,0x4644,0x4644,0x465F
.word 0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5446,0x54C4,0x6554,0x7564,0x7575,0x4446,0x4446,0x7546,0x7554,0x7554,0x7554,0x5454,0x6554,0x5454,0x5454,0x5475,0x5475,0x5454,0x5475,0x7554,0x7554,0x6454,0x7575,0x7554,0x7554,0x5454,0x7554,0x4685,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4F4F,0x7F7F,0x7F7F,0x4141,0x5454,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4444,0x5441,0xD464,0x5485,0x7F7F,0x6F6F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4646,0xC554,0x4475,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5444,0x8575,0x95D4,0x5485,0x4645,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4446,0x8554,0x95C5,0x75B4,0x4475,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x8565,0x7575,0x7554,0x6475,0x7575,0x4646,0x5444,0x7554,0x5454,0x5465,0x5475,0x5475,0x5454,0x5454,0x5454,0x5454,0x6554,0x5454,0x5475,0x5454,0x5475,0x5475,0x5475,0x5454,0x7575,0x5475,0x5475,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4144,0x7F7F,0x7F7F,0x4444,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6441,0xD4D4,0xD4E4,0x9F9F,0x7F7F,0x6F6F,0x5444,0x4141,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4444,0x7F7F,0x4141,0x7F7F,0x6F6F,0x7544,0x75D4,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4146,0x7544,0xB485,0xD4B5,0x8595,0x4554,0x4646,0x4646,0x4646,0x4646,0x4646,0x5446,0x4654,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x7554,0x9585,0x75B4,0x4475,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646
.word 0x4446,0x4545,0x4545,0x4545,0x4446,0x4646,0x4646,0x7575,0x7554,0x7554,0x5475,0x5475,0x7554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5454,0x5454,0x4475,0x4446,0x5F5F,0x4646,0x7F7F,0x5454,0x7F7F,0x4444,0x5441,0x8F8F,0x7F7F,0x6F6F,0x4444,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xD564,0xD4E4,0x6464,0x7F7F,0x6F6F,0x4441,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x4141,0x6F6F,0x7F7F,0x9554,0x6575,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5444,0x8575,0x95C4,0x95D4,0x8585,0x4454,0x4446,0x4646,0x5F5F,0x5446,0x6554,0x5465,0x4641,0x4644,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4444,0x8575,0x8585,0x8595,0x7595,0x7585,0x5454,0x4444,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x5454,0x8485,0xC495,0x7585,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7546,0x8585,0x8575,0x7575,0x8575,0x4485,0x4446,0x7575,0x5475,0x5475,0x5465,0x5454,0x5475,0x5454,0x5454,0x5454,0x5444,0x5454,0x5444,0x4454,0x4454,0x4454,0x5454,0x4454,0x5441,0x5444,0x5454,0x5454,0x5465,0x5454,0x5454,0x6554,0x5475,0x7575,0x4475,0x4646,0x6F6F,0x5441,0x9F9F,0x7F7F,0x4144,0x8F8F,0x7F7F,0x6F6F,0x4444,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5441,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4144,0x7F7F,0x5445,0x85D4,0x4665,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4146,0x6554,0x8575,0xC485,0xD4B5,0x95B5,0x8585,0x7585,0x7575,0x7575,0x7575,0x5454,0x5465,0x4444,0x4454,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x8554,0x9595,0x95C4,0x7585,0xB485,0x9585,0x9595,0x85B4,0x5454,0x5454,0x6575,0x5454,0x5454,0x8575,0x8585,0x9495,0x85C4,0x75B4,0x4675,0x4644,0x6F6F,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646
.word 0x4646,0x4646,0x8575,0x7575,0x6554,0x5454,0x6575,0x7554,0x5475,0x5454,0x5454,0x5454,0x5454,0x5444,0x5444,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x7541,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4454,0x5454,0x7554,0x7554,0x5454,0x4444,0x4441,0x8F8F,0x7F7F,0x5454,0x9F9F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x4444,0x6F6F,0x6F6F,0x7F7F,0x8F8F,0x4154,0x4444,0x6F6F,0x7F7F,0x6F6F,0x4141,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7554,0x6495,0x4654,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x6541,0x5454,0x7554,0x7565,0x8575,0x95B4,0xD4C4,0xB595,0x9595,0xB5B5,0xB5D4,0x95D4,0x95C5,0x95C5,0x8585,0x6475,0x5454,0x4454,0x4454,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4644,0x4446,0x8554,0xC4C5,0xB485,0x6575,0x5475,0x5454,0x6575,0x5454,0x8554,0x7585,0x7585,0x7575,0x8575,0x8575,0x8585,0x5454,0x5454,0x7575,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4446,0x4646,0x6546,0x8585,0x7554,0x5475,0x7575,0x7554,0x7554,0x5454,0x5454,0x5454,0x5444,0x5454,0x5454,0x5454,0x5454,0x5475,0x7554,0x5475,0x7554,0x5454,0x7554,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5454,0x5444,0x5441,0x5454,0x5454,0x4454,0x4141,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x5454,0x7F7F,0x7F7F,0x4444,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x5454,0xBFBF,0x6464,0x9F9F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x5444,0x7F7F,0x4446,0xC575,0x6575,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4441,0x5454,0x5454,0x5454,0x5454,0x8575,0xC485,0xC485,0x95B4,0x95C4,0x85C4,0xC495,0xC4E4,0xD4D4,0xD4B5,0x9584,0x85B5,0x7585,0x7575,0x5454,0x4454,0x4444,0x4644,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x8544,0x8595,0x85B4,0x7575,0x5454,0x4644,0x4646,0x4444,0x5454,0x6554,0x7565,0x5454,0x5475,0x5475,0x6554,0x6554,0x5454,0x4454,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x5F5F,0x4646,0x4646
.word 0x4646,0x5F5F,0x4646,0x7565,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x5444,0x5454,0x5454,0x5454,0x7554,0x7554,0x7575,0x5454,0x7575,0x7575,0x5454,0x7575,0x7585,0x5475,0x7554,0x8585,0x5454,0x5454,0x5454,0x7554,0x5454,0x5454,0x4454,0x4154,0x4141,0x4444,0x8F8F,0x4141,0x5454,0x4444,0x7F7F,0x4444,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x64D4,0xD564,0xC4C4,0x5454,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0x4141,0x4646,0x6554,0x5454,0x5454,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4446,0x5444,0x5454,0x7554,0x7565,0xC485,0x8585,0xB485,0xB495,0x8585,0x8585,0x95B4,0xC4C4,0xB495,0x95C5,0xC5C5,0xB5C4,0xB584,0x8595,0x5475,0x5454,0x4144,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x5F5F,0x4446,0x5F5F,0x4646,0x4644,0x5F5F,0x4446,0x7585,0x7565,0x5475,0x5475,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646
.word 0x4446,0x9585,0x8585,0x7585,0x5475,0x7554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x8575,0x5475,0x5454,0x7554,0x7585,0x5454,0x5475,0x7585,0x5454,0x7575,0x5475,0x7575,0x7575,0x5465,0x8554,0x8585,0x5475,0x7554,0x7575,0x5454,0x4141,0x9F9F,0x5454,0x7F7F,0x4444,0x5454,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x6F6F,0x6454,0xAFAF,0x64E5,0x64E4,0x7F7F,0x6F6F,0x6F6F,0x4144,0x7F7F,0x4444,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x6F6F,0x6F6F,0x4444,0x7F7F,0x6F6F,0x4444,0x7F7F,0x4141,0xC4C4,0x6F6F,0x5445,0x7554,0x9575,0x6575,0x4646,0x4646,0x4446,0x4446,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4146,0x5444,0x7554,0x7575,0x7575,0x7575,0x7585,0x7585,0x85B4,0x8575,0x8585,0x85B4,0xB4C4,0xC495,0xC495,0xD495,0x95D4,0x95D4,0x8595,0x7575,0x5454,0x4654,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x7544,0x7595,0x5475,0x7575,0x5454,0x7585,0x4454,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x5F5F,0x4646,0x4446,0x4646,0x465F
.word 0x8575,0x75B4,0xB485,0x6575,0x6564,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5475,0x7575,0x7575,0x5475,0x7565,0x7554,0x8575,0x7575,0x7554,0xC485,0x7585,0x8554,0x85D4,0x5465,0x8575,0x7585,0x6554,0x8575,0x5475,0x7554,0x7575,0x5475,0x8F8F,0x44D4,0x5454,0x5454,0x5441,0x4444,0x8F8F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4144,0x6F6F,0x4141,0x4141,0x6F6F,0x7F7F,0xAFAF,0xD454,0xF5D4,0x5454,0x6F6F,0x6F6F,0x7F7F,0x5F5F,0x4444,0x4141,0x6F6F,0x6F6F,0x4444,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4444,0x4444,0x4141,0x7F7F,0x5454,0x7565,0x8585,0x75B5,0x4654,0x5F5F,0x7584,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x5444,0x5454,0x6554,0x6564,0x5465,0x7575,0x7575,0x7575,0x75B4,0x8575,0x8585,0x85B4,0xC485,0xC485,0xC495,0xC485,0xC595,0xC595,0x8595,0x7575,0x4465,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4644,0x4646,0x4446,0x7565,0x5475,0x5475,0x7554,0x8575,0x8575,0x7585,0x7575,0x7575,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0x8585,0x7585,0x7575,0x5475,0x5475,0x5454,0x5454,0x5454,0x7554,0x5454,0x5475,0x7575,0x5454,0x7554,0x7554,0x8585,0x7554,0x7575,0x8575,0x6585,0x7575,0x8585,0x8565,0x8585,0x5475,0xC485,0xC485,0x5475,0xD485,0x5485,0x5454,0x7575,0x8F8F,0xD441,0xC4C4,0x6464,0x7F7F,0xD554,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x5454,0xD4D5,0xAFAF,0x6F6F,0x7F7F,0x4444,0x4141,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4444,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x8F8F,0x7F7F,0x7575,0x8585,0x95C4,0xD485,0x7595,0x8565,0x8565,0x4445,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x6F6F,0x5444,0x5475,0x7554,0x7554,0x7554,0x5475,0x7575,0x7575,0x7575,0x7575,0x8575,0x8575,0x85B4,0xB485,0xB495,0xD495,0x85C4,0xB5C4,0x85D4,0x7585,0x4454,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x7585,0x5475,0x5475,0x8575,0xB485,0x7585,0x8585,0x7575,0x7554,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646
.word 0xB485,0x8554,0x6585,0x6564,0x5454,0x5454,0x6554,0x5454,0x5475,0x8575,0x8554,0x8585,0xC485,0x5475,0x7554,0xB485,0x7595,0x7575,0xC485,0x7585,0x7554,0x95C4,0x5485,0x9565,0x75C4,0x7565,0x8575,0x7585,0x8575,0x75B4,0x7575,0x6454,0x4444,0xD564,0xD464,0x8F8F,0x5444,0x9F9F,0x4444,0x4141,0x4444,0x4444,0x4141,0x4141,0x7F7F,0x4444,0x4444,0x4444,0x7F7F,0x8F8F,0x8F8F,0xAFAF,0x41D5,0x6F6F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x6F6F,0x6F6F,0x6F6F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4141,0x4144,0x5454,0x8575,0x95C5,0x8585,0x7585,0x7575,0x4485,0x5F5F,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4446,0x5454,0x5475,0x5475,0x5475,0x5475,0x7554,0x7554,0x7554,0x7575,0xB485,0x7585,0x8575,0xB495,0x8585,0xB495,0xB485,0xB495,0x8585,0x7595,0x4475,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x6446,0x7575,0x7554,0x7554,0x7585,0x7575,0x8585,0x6575,0x4654,0x4644,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x4646,0x4646
.word 0x7575,0x7575,0x5454,0x5475,0x5454,0x5454,0x7575,0x5475,0x7554,0x5454,0x7565,0x7575,0x6585,0x9575,0x7565,0x8575,0x7575,0x5475,0x8575,0xC485,0x7554,0xB485,0x7554,0x8554,0x95D4,0x7554,0xD485,0x5495,0x6575,0x5485,0x8575,0x5475,0x8F8F,0x5454,0xAFAF,0x8F8F,0x6464,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4144,0xAFAF,0xAFAF,0xC4C4,0x7F7F,0x6F6F,0x7F7F,0x5454,0x5464,0x7F7F,0xE441,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4444,0x7F7F,0x7F7F,0x4441,0x7F7F,0x7F7F,0x5475,0x7544,0x5475,0x8565,0x7565,0x7575,0x7595,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4446,0x5454,0x5475,0x5454,0x5475,0x5475,0x5454,0x7565,0x5475,0x7575,0x7575,0x7575,0x8575,0x85B4,0x85B4,0x8585,0x6575,0xD485,0x8585,0x7585,0x4645,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x5446,0x7575,0x7585,0x7575,0x85C4,0x8575,0x7575,0x4475,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644
.word 0x8585,0x7565,0x5454,0x5454,0x5454,0x5475,0x7554,0x7575,0x7575,0x8585,0x5475,0x7554,0x8554,0xB5D4,0x95C5,0x7575,0x8575,0x95C5,0x5485,0x85B4,0x75B5,0x7575,0xD485,0x5475,0x8575,0x5475,0xB485,0x5485,0x8575,0x54D4,0xC475,0x9F9F,0x54E4,0x6464,0xD5D5,0x9F9F,0x6464,0x4444,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4441,0x6F6F,0x7F7F,0x9F9F,0xC4C4,0x5464,0x6F6F,0x7F7F,0x4154,0x9F9F,0xD4D4,0x6F6F,0x4444,0x4141,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x6F6F,0x4444,0x4441,0xC485,0x5485,0x5454,0x7554,0x8575,0x8595,0x85B4,0x7585,0x4665,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5441,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x7554,0x7554,0x7554,0x7575,0x6575,0x8585,0xB485,0x7565,0x7575,0x7565,0x7585,0x5465,0x4454,0x4446,0x4646,0x4F4F,0x4646,0x4644,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x5F5F,0x7546,0x5454,0x5465,0x8575,0x7575,0xC485,0x7585,0x5454,0x4644,0x5F5F,0x4646,0x4644,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646
.word 0x5475,0x5475,0x5454,0x5454,0x7554,0x7575,0x5475,0x5454,0x7575,0xB485,0xC495,0x6585,0x8575,0xD4B4,0x8585,0x5475,0x8565,0x7585,0x5465,0x8575,0x7575,0x7554,0x8585,0x5465,0xD485,0x6595,0x7554,0x5475,0x8585,0x7595,0xD454,0x5454,0x4444,0x5454,0x5441,0xC4C4,0x5454,0x4444,0x7F7F,0x4141,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x5454,0x7575,0x6F6F,0x7F7F,0x6F6F,0x4154,0xF564,0x54D4,0x54D4,0x8F8F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x9575,0x95C4,0x6585,0x5454,0x7575,0xC485,0x8585,0x95B4,0x6595,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x5446,0x6585,0x7575,0x7585,0x7554,0x5475,0x5454,0x5454,0x5454,0x5475,0x5454,0x7554,0x7554,0x6465,0x8554,0x7554,0x7585,0x5465,0x8575,0x7585,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x7546,0x7575,0x7554,0x7585,0x7554,0x8585,0x85B4,0x7554,0x5475,0x5465,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4646
.word 0x5465,0x5454,0x5454,0x5454,0x7554,0x8575,0x8585,0x7575,0x5475,0x7565,0x7585,0x8554,0x5475,0x7575,0x7575,0xB5C4,0x5485,0x8554,0xB5C4,0x5475,0xC485,0x85D4,0x7554,0x7595,0x7575,0x7585,0x8575,0x85D4,0x7565,0x7565,0x5454,0x8F8F,0x4441,0x4444,0x5454,0x6464,0x8F8F,0x9F9F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x4441,0x4141,0x7F7F,0x4444,0x6F6F,0x4444,0x8F8F,0x8F8F,0x5454,0xAFAF,0x6F6F,0x5454,0x7F7F,0xD4D4,0x6464,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x6F6F,0x6F6F,0x4444,0x6F6F,0x4141,0x6F6F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4444,0x8554,0xC485,0x8575,0x7585,0x6554,0x7575,0xB485,0x7575,0x9585,0x6485,0x4445,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x6465,0x6554,0x6554,0x5454,0x5475,0x5475,0x5454,0x5485,0x8565,0x5475,0x5475,0x5465,0x5454,0x6575,0x5475,0x7554,0x9585,0xC5D4,0x95D4,0x9585,0x4654,0x4446,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5444,0x6465,0x7554,0x7575,0x7575,0x7575,0x5475,0x7575,0x5475,0x8575,0x7575,0x8575,0x7585,0x4664,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646
.word 0x5454,0x5454,0x5454,0x7554,0x5475,0x8575,0x7575,0x8554,0x95D4,0x5475,0x5454,0xC485,0xD495,0x8595,0x7554,0xC485,0x85C4,0x7565,0x7585,0x5475,0x8585,0x6575,0x7554,0x85C4,0x5465,0x5475,0x7575,0xC485,0x8575,0x5495,0x54D4,0x4444,0x5454,0x5454,0x5454,0x4454,0x6464,0x5454,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4441,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0xD554,0x6F6F,0x4141,0x7F7F,0x9F9F,0x5454,0xAFAF,0xD554,0x4444,0x6F6F,0x6F6F,0x7F7F,0x4444,0x5F5F,0x7F7F,0x4444,0x5F5F,0x6F6F,0x7F7F,0x4444,0x6F6F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x7F7F,0x7544,0x85C4,0x7585,0xB485,0x5475,0x5454,0x8575,0x8585,0xB475,0x8585,0x9585,0x7564,0x4575,0x4446,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x5446,0x7575,0x5475,0x8554,0x5465,0x7554,0x5454,0x5454,0x6575,0x7575,0x5475,0x7554,0x5454,0x5454,0x5454,0x5454,0x6554,0x5475,0xB485,0xC485,0xC4C5,0x5495,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x5F5F,0x6565,0x5464,0x5454,0x7575,0x7554,0x7575,0x7575,0x5454,0x7585,0x6575,0x6564,0x6554,0x7575,0x4644,0x4644,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F
.word 0x5454,0x5454,0x5475,0x8575,0x85C4,0x5465,0x7565,0x7554,0x8585,0x85C4,0x6575,0x8575,0x75B4,0x9565,0x75D4,0x7554,0x8554,0x95C5,0x8565,0xD485,0x5465,0x8585,0x5495,0x7565,0x9575,0x95C5,0x6575,0x7575,0x8554,0x9F9F,0x41D4,0x8F8F,0x7F7F,0xAFAF,0x5454,0x5454,0x9F9F,0x8F8F,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x4154,0x5454,0x7F7F,0x4141,0x5454,0x8F8F,0xD4D4,0x6F6F,0x6F6F,0x4141,0x4441,0x5454,0x9F9F,0x5454,0x6F6F,0x6F6F,0x5F5F,0x4141,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x6F6F,0x7F7F,0x7F7F,0x4141,0x6F6F,0x9585,0x85B4,0x8585,0xD485,0x6585,0x5454,0xB475,0x7575,0xB485,0x8585,0x8585,0x8585,0x7585,0xB485,0x8595,0x85B4,0x6585,0x7575,0x7575,0x5475,0x5475,0x7554,0x7554,0x7575,0x7554,0x8575,0x7565,0x5475,0x5475,0x5454,0x7575,0x7554,0x5475,0x7554,0x5475,0x5454,0x5454,0x5475,0x8575,0x7585,0x7575,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7546,0x9585,0xD4D4,0x7585,0x5475,0x5454,0x5475,0x7554,0x7554,0x7575,0x5475,0x7575,0x7575,0x4554,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x4646,0x5F5F,0x4646
.word 0x5454,0x5454,0x7554,0x8575,0x5475,0x8585,0x8585,0x5485,0x7575,0x6575,0x8585,0x6585,0x5454,0x8585,0x85C4,0x7565,0x8554,0xB485,0x6585,0x9585,0x5475,0x8565,0x75B4,0x8585,0x8554,0x85B4,0x7554,0x85B5,0x6554,0xC454,0x5454,0x8F8F,0x6441,0x9F9F,0x8F8F,0x6464,0x5475,0x7F7F,0x7F7F,0x8F8F,0x4444,0x4444,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x4141,0x7F7F,0x8F8F,0x5454,0x6F6F,0x4141,0x4444,0x8F8F,0x4141,0x8F8F,0x9F9F,0x5441,0x6F6F,0x7F7F,0x4141,0x6F6F,0x4444,0x6F6F,0x4141,0x4441,0x7F7F,0x4444,0x5454,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6565,0x8575,0xC495,0x75B4,0x7575,0x9585,0x5475,0x5454,0x7575,0x8575,0x7575,0x85B4,0x85B4,0x85B4,0x9595,0xB5C4,0x95A5,0x95C5,0x85C5,0x85C5,0x9585,0x8585,0x85C4,0x8585,0x7575,0x7575,0x5454,0x7575,0x5475,0x7554,0x5485,0x5454,0x5475,0x5454,0x7575,0x5485,0x8554,0x5454,0x5454,0x6554,0x5465,0x5475,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x8575,0xC5B5,0x95D4,0xD4B5,0x8595,0x7575,0x5454,0x5475,0x8575,0x7575,0x8575,0x8575,0x7585,0x4645,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4646
.word 0x5454,0x8554,0x5465,0x7554,0x7575,0xB485,0x7585,0x8575,0x85C4,0x5475,0xB485,0x8595,0x5475,0x7575,0x7575,0xC595,0x6575,0x7575,0x8595,0x5465,0xC585,0x5485,0x5454,0xB485,0x5485,0x6554,0x7554,0x95C4,0x75C4,0x6464,0x7F7F,0x7F7F,0x5454,0x8F8F,0x5454,0xAFAF,0x8F8F,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4154,0x5454,0x9F9F,0x6464,0x4444,0x4441,0x7F7F,0x7F7F,0x4141,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x5444,0x4444,0x7F7F,0x4154,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x4144,0x7F7F,0x7F7F,0x6F6F,0x4141,0x4444,0x7F7F,0x7F7F,0x4444,0x4141,0xC475,0x95B4,0x8585,0x7575,0xB485,0xC485,0x5475,0x5454,0x5475,0x8575,0x7575,0x8585,0xB485,0xB495,0xB485,0x85C4,0x85B4,0x85C4,0x95B4,0x85B4,0xB5C4,0xC5C4,0xD4C4,0xC595,0xD495,0xB5D4,0x8585,0x7575,0x7554,0x7554,0x7585,0x5454,0x5454,0x5454,0x5475,0x5475,0x5475,0xB485,0x5454,0x5454,0x4654,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4545,0x8575,0xB585,0x85C4,0xC485,0x8585,0x85C4,0xB5D4,0x5485,0x7554,0x6554,0x7575,0x7575,0x5475,0x4554,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646
.word 0x6554,0x8575,0x8575,0x5475,0x5454,0x7585,0x6585,0xC485,0xC4C5,0x5475,0x7575,0x6575,0xD485,0x5485,0x7554,0x8585,0x75D4,0x7554,0xB485,0x5454,0xC485,0xB485,0x7595,0x8565,0x6575,0x9595,0x6465,0xB475,0x5485,0xD4D4,0x4444,0x4444,0x5454,0x4444,0x64D4,0x5454,0x9F9F,0x4444,0x4141,0x4444,0x7F7F,0x4444,0x9F9F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4164,0x4144,0x7F7F,0x7F7F,0x5454,0x6F6F,0x4141,0x5454,0x4141,0x4441,0x5441,0x5454,0x7F7F,0x4141,0x4444,0x6F6F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8585,0x8575,0x95B4,0x8575,0x7575,0x8585,0x8585,0x5465,0x7554,0x7554,0x8575,0xB485,0xB495,0x8585,0x8585,0x8585,0xB485,0xB495,0xB495,0xB495,0xB485,0x8585,0x8585,0x85B4,0x95C4,0xD495,0xD4D4,0xC495,0x7585,0x7565,0x5475,0x8554,0x6575,0x5454,0x5454,0x5454,0x7554,0x7554,0x7554,0x7575,0x5441,0x4444,0x4646,0x4646,0x4646,0x4646,0x4644,0x9494,0x8585,0xB585,0xB485,0xB495,0xB485,0xB485,0xB485,0x9585,0x85D4,0x5465,0x7554,0x7575,0x8585,0x7565,0x5454,0x4454,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446
.word 0x5454,0x7575,0x8575,0x8585,0x6575,0x5454,0x5454,0x8585,0x75B4,0xD485,0x5485,0x5475,0x8575,0x7585,0x5465,0x7554,0x9554,0x5485,0x6465,0xC485,0x6575,0x5475,0x8575,0x7554,0x5475,0x7575,0x7585,0x7554,0xAFAF,0x4154,0x5454,0x7F7F,0x8F8F,0xD444,0x8F8F,0x64C4,0x4141,0x7F7F,0x8F8F,0x7F7F,0x4141,0x5454,0x5454,0x6464,0xD554,0x8F8F,0x4444,0x7F7F,0x4444,0x7F7F,0x6454,0xAFAF,0x8F8F,0x5444,0x7F7F,0x5151,0x5454,0x9F9F,0x9F9F,0x54C4,0xD4D4,0x9F9F,0xC4D4,0xC441,0x7F7F,0x5454,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x5454,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x8575,0x85B4,0xB485,0xB485,0x8575,0x6575,0x7575,0x8585,0x5465,0x5475,0x5475,0x6575,0x7575,0x7565,0x7565,0x6575,0x8575,0x8575,0x8585,0x7575,0x8585,0x75B4,0x85B4,0x8575,0xB485,0xC485,0x8595,0x85C4,0x8585,0x8585,0x7575,0x6575,0x7554,0x7575,0x5475,0x6554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x6444,0x5454,0x6454,0x5444,0x8575,0x9585,0xB495,0xC485,0xB495,0x8585,0x8575,0x8585,0x8585,0x85B4,0xB5C4,0x7585,0x8554,0x7585,0x7554,0x8575,0x85C5,0x7575,0x5475,0x4454,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4F4F,0x4646
.word 0x5475,0x5475,0x7565,0x8575,0x8565,0xD495,0x7585,0x7565,0x7575,0x8585,0x85D4,0x7565,0x6554,0x5475,0xC595,0x7595,0xB454,0x7585,0x5475,0x8575,0x5475,0x5485,0x6575,0x7575,0x5485,0x5465,0x5454,0x8575,0x6454,0x9F9F,0x7F7F,0x4441,0x8F8F,0x9F9F,0x41D4,0x7F7F,0x4444,0x4444,0x4444,0x4144,0x7F7F,0x7575,0x9F9F,0xD5D5,0x6454,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0xD564,0x64D4,0x7575,0x5454,0xD4D4,0x6454,0xAFAF,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF4F4,0xE4E4,0xAFAF,0x8585,0x8F8F,0x7F7F,0x4144,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x9554,0x7585,0x8585,0x8585,0x7585,0x7585,0x5475,0x5454,0x5475,0x5475,0x5454,0x6465,0x7554,0x5454,0x5475,0x7554,0x7575,0x8575,0x75B4,0x8585,0x8575,0x8575,0x8585,0x85B4,0x8585,0x85B4,0x85B4,0xB495,0xC475,0xC495,0xC4B5,0x7585,0x7554,0x7585,0x5454,0x5475,0x7554,0x7575,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5475,0x9585,0xB495,0xB495,0xB485,0x8585,0x8575,0x75B4,0x7575,0x7575,0x7575,0xC485,0x7595,0x6554,0x7575,0x8554,0xC485,0xB4B5,0x8585,0x7585,0x5454,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4646
.word 0x9575,0x7585,0x7554,0x6554,0xB464,0xB485,0x8575,0x94D4,0x7585,0x7565,0x7575,0x9585,0x95C4,0x5475,0x7575,0x8565,0x6585,0x7554,0x7575,0x7554,0x5475,0x6575,0x7554,0x7554,0x5475,0x7575,0x8575,0x5454,0x4444,0x8F8F,0x4444,0x7F7F,0x4141,0x85E4,0xC4C4,0x5464,0x5454,0x4444,0x8F8F,0x8F8F,0x64D4,0x6464,0x7575,0xD454,0xAFAF,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0xD454,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xF4E4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xD4D4,0x64C4,0x8F8F,0x8F8F,0x5454,0x7F7F,0x7F7F,0x4444,0x6F6F,0x6F6F,0x4444,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0xC485,0x5475,0x7565,0x95C4,0x85C4,0x7575,0x5475,0x5454,0x8575,0xB5C4,0x85D4,0x5465,0x7575,0x8585,0x8585,0x7565,0x7575,0x7575,0x8575,0x75B4,0x8575,0x8575,0x7585,0x85B4,0xC585,0x85D4,0x7575,0x7554,0x8575,0x95C4,0x85C4,0x6585,0x7554,0x7585,0x7554,0x7575,0x5475,0x8575,0x85C4,0x7565,0x8575,0x5485,0x7575,0x6585,0x8554,0xD495,0xD4C5,0xB495,0x8595,0x8575,0x85B4,0x8585,0xB485,0x8585,0x8585,0x85B4,0x75B4,0x7554,0x7595,0x8575,0x85B4,0x7585,0x75D4,0x4554,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xB485,0x7585,0xD475,0x7585,0x7554,0x7585,0x8554,0xC485,0x7585,0x7585,0x5454,0x8575,0x7585,0x8575,0x5465,0x8575,0x7575,0x5475,0x7554,0x5454,0x6575,0x5454,0x5475,0x5475,0x5454,0x5475,0x7565,0x4154,0x4444,0x7F7F,0x8F8F,0x4141,0x4144,0x6464,0x5454,0x4444,0x9F9F,0x4141,0x8F8F,0x6441,0xD4D5,0xD4D4,0xD4C4,0x8F8F,0x6464,0x4444,0x4141,0x7F7F,0x7F7F,0x5454,0x6464,0xD4C4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xE4F4,0xE4F4,0xE4D4,0xE4E4,0xD4E4,0xAFAF,0x5444,0x8F8F,0x8F8F,0x4154,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4444,0x6F6F,0x5454,0x8575,0x75D4,0x5454,0x8575,0xB485,0x7575,0x5475,0x95C4,0xB5C4,0xC5D4,0x95D4,0x7575,0x7575,0x7585,0x7565,0x5475,0x8575,0x8585,0x7575,0x8585,0xC475,0x7585,0x8575,0xD485,0x85B4,0x7575,0xD4D4,0x5475,0x5444,0x5454,0x4454,0x5454,0x5454,0x5454,0x4454,0x5454,0x5444,0x7554,0x7554,0x5454,0x5454,0x6554,0x6564,0x5454,0xC485,0xC4C5,0xC5E4,0x95C4,0x85C4,0x85B4,0x8575,0x85B4,0x7575,0x7585,0x85B4,0x7575,0x5454,0x8575,0x85D4,0x95C4,0xB485,0xC495,0x4575,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646
.word 0x7554,0x5475,0x8585,0x85D4,0x9575,0x85D4,0x6575,0x7575,0x6575,0xD485,0x5495,0x5465,0x6554,0x7554,0x6475,0x6554,0x7554,0x7575,0x5454,0x5454,0x5475,0x5475,0x7554,0x5454,0x5454,0x5454,0x7554,0x9F9F,0x4144,0x4144,0x7F7F,0x4444,0x6464,0x4444,0x7575,0x9F9F,0x9F9F,0xD444,0x4444,0x9F9F,0xAFAF,0xE4C4,0x64E4,0xD4D5,0x6464,0x7F7F,0x6F6F,0x4141,0x7F7F,0xD564,0xD564,0xD474,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4F4,0xF5F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xC4E4,0x6464,0x54D5,0x5454,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x6F6F,0x4444,0x7F7F,0x6F6F,0x5444,0x5454,0x7575,0x6585,0x7554,0x7585,0x7575,0x5475,0x8575,0xD4D4,0xD495,0xB495,0x5475,0x5475,0x5454,0x7554,0x5475,0x5454,0x7554,0xB485,0x7575,0x8585,0x8575,0x85C4,0x85B4,0xB565,0x5465,0x7554,0x9585,0x7575,0xC575,0x8585,0x5475,0x8585,0x8585,0x85C4,0x8585,0x8585,0x7575,0x7575,0x7575,0x7575,0x7575,0x8585,0x8585,0x95D4,0xB5D4,0x95C4,0x85C4,0x85B4,0x8575,0xB485,0x8585,0x7585,0x7575,0x7575,0x7585,0x5454,0x7554,0x8585,0xB495,0x5485,0x4645,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F44
.word 0x7554,0x5454,0x8575,0x6585,0xC485,0xC4B5,0x5475,0x5475,0x7554,0x8585,0x75B4,0x8585,0x7585,0x5475,0x7575,0x64C4,0x7554,0x7554,0x5454,0x5454,0x5454,0x5454,0x4454,0x5441,0x4454,0x4454,0x5454,0x8F8F,0x8F8F,0x4444,0x7F7F,0x4141,0xF664,0x8F8F,0x9F9F,0x4141,0x8F8F,0x9F9F,0x4444,0x64C4,0xF4D4,0xD464,0xE4E4,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x5454,0x6464,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE5E4,0xF4F4,0xF5F4,0xF4F4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0x6464,0x6464,0x8F8F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4444,0x6F6F,0x6F6F,0x6F6F,0x8554,0x6585,0x5454,0x5454,0x5454,0x8575,0x7585,0x7575,0x7554,0xC495,0x7575,0x7554,0x7554,0x6554,0x7554,0x5485,0x5475,0xB485,0x6575,0x7554,0x7575,0x8575,0xC475,0xB495,0x7554,0x5485,0x8554,0x94D4,0xD4D4,0x7575,0xD495,0xD4D4,0x7585,0xC454,0xD495,0xD4B5,0xD4C5,0x95D4,0x95C4,0xB5C4,0x95C4,0xC5D4,0xC5E4,0x85B4,0x95B4,0xC4C5,0xD4C5,0x85D4,0x85C4,0x8575,0x7585,0x7585,0xB485,0x7585,0x8585,0x8575,0x5475,0x5454,0x7575,0x8565,0x5475,0x4645,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x4644,0x5F5F,0x4646,0x4646
.word 0xD485,0x7595,0x5475,0x5454,0x7565,0x5475,0x8575,0xC585,0x5485,0x7554,0x5454,0x7575,0x7575,0x7575,0x7554,0x5454,0x5454,0x5454,0x5454,0x6554,0x5454,0x5475,0x5454,0x5475,0x5454,0x5454,0x4454,0x4141,0x4144,0x7F7F,0x7F7F,0x5441,0xAFAF,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0xC4C4,0x6441,0x9F9F,0xD564,0x64D4,0x9F9F,0xE4E4,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6441,0xD4D5,0xD464,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D5,0xF4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xD4C5,0xD5E4,0xE4E4,0xD4D4,0xE4E4,0xC4D4,0x7575,0x8F8F,0x4141,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x6F6F,0xB4B4,0x7585,0x8575,0xC495,0x7585,0x6554,0x8585,0x8554,0x6585,0x8575,0x7565,0x8585,0x7565,0x9575,0xB485,0x7575,0x8554,0x5475,0x8554,0x8585,0x5475,0x6554,0x6585,0x5475,0x7554,0x8585,0x85C4,0xB485,0x9595,0x5475,0x95C4,0xC495,0x6585,0x9554,0xD4C4,0x95D4,0xC5D4,0xD4D4,0xC4C5,0xE495,0xD495,0xD4C5,0x8585,0xC495,0xE495,0x95C4,0x95D4,0x85C4,0x8575,0x75B4,0xB485,0xB485,0x8585,0x7575,0x7585,0x6575,0x7554,0x6554,0x9585,0x6585,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4644
.word 0xC485,0x5485,0xD485,0x95C5,0x5454,0x5454,0x8575,0x85B4,0x8565,0xD4D4,0x7585,0x5454,0x5465,0x5475,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5454,0x5454,0x4475,0x4454,0x5454,0x5454,0x8F8F,0x4441,0x4444,0x7F7F,0x7F7F,0x4444,0x41E4,0x9F9F,0x8F8F,0x8F8F,0x6444,0x5454,0xD5D5,0x6454,0x64D5,0xC4C4,0x9F9F,0xAFAF,0x4444,0x6F6F,0x6F6F,0x4141,0x6464,0xD4C4,0xD4D4,0xD4D4,0xD4E4,0xC5E4,0xE4E4,0xE4E4,0xE4E5,0xF4F5,0xF4F4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xC5D4,0xD4D4,0xD4D4,0x6464,0x5454,0x7F7F,0x6F6F,0x4141,0x4144,0x6F6F,0x4444,0x6F6F,0x6F6F,0x8585,0x7585,0x8575,0x85B4,0x85C4,0x5454,0xB454,0x8575,0x5475,0x7585,0x8595,0x85B4,0x8585,0x6585,0x9575,0x85E4,0x5475,0x7575,0x8585,0x75B4,0x7554,0x7585,0x5485,0x5475,0x6585,0x5454,0x6575,0x7575,0x8575,0x7575,0xB485,0x7575,0x5454,0xC475,0xC595,0xC5E4,0x95C4,0x95C4,0x95C4,0x95D4,0xC5C4,0xD4C4,0xC495,0xC5D4,0xB5C5,0x95C4,0x95C4,0x75B4,0x8575,0x8585,0x85B4,0x8585,0x7575,0x8575,0x8575,0x5465,0x5454,0x8554,0xC475,0x7585,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x4644,0x4646,0x4446,0x4F4F,0x4646
.word 0x7565,0x5454,0x8575,0xB485,0xD475,0xC495,0x5454,0x5475,0x8575,0x7585,0x7575,0x5485,0x5475,0x5454,0x5454,0x7554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5444,0x4446,0x4446,0x4446,0x4144,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x6454,0x5454,0x9F9F,0x5454,0x4441,0x9F9F,0x8F8F,0xD444,0x9F9F,0x9F9F,0x4444,0xF554,0xC4C4,0x4444,0x7F7F,0x4141,0x7F7F,0xD564,0xD464,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xC4D4,0x6464,0x4444,0x7F7F,0x6F6F,0x7F7F,0x4144,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7575,0x7575,0x8554,0x8585,0x85D4,0x5465,0x8554,0x7554,0x8575,0x7565,0x8585,0xB475,0x5485,0x7575,0x7554,0xE485,0x85D4,0x7575,0x7554,0x7575,0x7575,0xC595,0x8585,0x8585,0x85C4,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x8575,0x8585,0x85C4,0x95B4,0xD4C4,0xD4B5,0xE495,0xC4C5,0x95E4,0xC5C4,0xB5C4,0xC5E4,0x65C4,0xC485,0x5475,0x7575,0x7585,0xB485,0x7585,0x7575,0x7575,0x7575,0x5454,0x5454,0x7554,0xB485,0xD485,0x85B5,0x4654,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0x5475,0x7575,0x6575,0x7575,0x8554,0x85B4,0xC485,0x8595,0x7554,0x7575,0x7554,0x5475,0x5454,0x5454,0x5475,0x5454,0x5454,0x5454,0x4444,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0xD4D5,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4444,0x4444,0x6441,0x7575,0x9F9F,0x5454,0x6464,0x6464,0x7F7F,0x7F7F,0x7F7F,0x6464,0xAFAF,0xD4D4,0xD4D4,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4E4,0xD4C5,0xD4E4,0xD4E4,0xD4D4,0x64D4,0x6464,0x7F7F,0x7F7F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8575,0x8575,0x7575,0x85B4,0xC485,0x5485,0x5454,0x5454,0x8565,0x5475,0x6575,0x7575,0x5475,0x5475,0x6575,0x8554,0xD495,0xD4D4,0x7585,0x7554,0xE485,0xC495,0xC485,0xC495,0xD4B5,0x8595,0x7585,0x5475,0x5475,0x5475,0xC485,0xD4B5,0xD4C5,0xE4C5,0xD4C5,0xD4C5,0xD495,0xC4C5,0xC495,0xD4B5,0xD4C5,0xD495,0xC4C5,0x5475,0x7585,0x7554,0x8575,0x7585,0x8585,0x7575,0x5454,0x5454,0x5454,0x5454,0x7554,0x8585,0x95C4,0xB485,0xD485,0x7585,0x4645,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x5F5F,0x4646,0x4F4F,0x4646,0x4644
.word 0xC495,0x5485,0x8585,0x5465,0x5475,0x5475,0x7565,0x6575,0x8585,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4646,0x4446,0x6F6F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4141,0x4444,0x7F7F,0x7F7F,0x5151,0x5454,0x5454,0x4141,0x5454,0x4141,0x8F8F,0x4141,0x8F8F,0x9F9F,0x4444,0x8F8F,0xC4E4,0x4444,0x4141,0x7F7F,0x6F6F,0xD541,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4D5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0x64D4,0xAFAF,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7585,0x7585,0x5465,0x7575,0x8585,0x95C4,0x65C4,0x5454,0x8575,0x7565,0x7575,0x5475,0x8575,0x6554,0x7575,0x6575,0x8554,0x9595,0xD4D4,0xD4B5,0xB4B5,0xB485,0xC4B5,0xD4B5,0xB5D4,0xC5C5,0xD495,0xC495,0xC485,0x9585,0x95C4,0x95C4,0x95C4,0x95B4,0x95C4,0x95D4,0xD4C4,0xC495,0xB5E4,0xD4D4,0xC5D4,0xC5D4,0x7595,0x8554,0x5475,0x5475,0xB485,0x7585,0x8575,0x7565,0x5475,0x4454,0x5454,0x7554,0x7585,0x8585,0x7585,0x9585,0x95B4,0x85C4,0x7585,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x8585,0x7565,0x95C4,0xC495,0x8554,0x8585,0x5454,0x5454,0x7585,0x5475,0x5454,0x5454,0x5475,0x5454,0x4454,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x7F7F,0x7F7F,0x8F8F,0x4141,0x7F7F,0x7541,0x5454,0x8F8F,0x4444,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x4444,0x9F9F,0x4444,0x6464,0xD454,0x7F7F,0x6F6F,0x6F6F,0x4141,0xD454,0x7474,0x64D4,0xD4E4,0xD4C5,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xF4D5,0xE4E4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4C5,0xD4D4,0xD4D4,0x6464,0x8F8F,0x7F7F,0x6F6F,0x4444,0x7F7F,0x6F6F,0x4444,0x4444,0x6F6F,0x75C4,0x7565,0x7585,0x5475,0x7575,0x7565,0x7554,0x5475,0x8554,0x7575,0x5454,0x7575,0x5475,0x7554,0x7554,0x7575,0x6575,0xB485,0xC495,0xB485,0xC485,0xB495,0xB495,0xB5D4,0xD4D4,0xD4B5,0xD495,0x85D4,0xE4B5,0xD4D4,0xD4B4,0xC4C5,0xC4C5,0xC4C5,0xD4C5,0xE495,0xE4C5,0xD4C5,0xD495,0x95D4,0xD4C5,0xD495,0x6595,0x8575,0x7575,0x8554,0x7585,0xB485,0x6575,0x7564,0x7585,0x7554,0x7554,0x7575,0x7565,0x6565,0x7575,0x75B4,0x7575,0x7585,0x7585,0x4554,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646
.word 0x6554,0x5475,0xB485,0x8585,0x8565,0x7575,0x8575,0x6585,0x6554,0x5454,0x5454,0x5454,0x5454,0x4644,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x7F7F,0x8F8F,0x7F7F,0x4141,0x5454,0x8F8F,0x5454,0x8F8F,0x7F7F,0x7F7F,0x4141,0x4141,0x8F8F,0x4441,0x8F8F,0xD544,0x5454,0x8F8F,0x4444,0x6F6F,0x6F6F,0xD4D4,0x85D5,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xF4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD5E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x64D5,0x41D5,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7585,0x7585,0x5485,0x7554,0x7585,0x8575,0x8565,0x5454,0x5454,0x7575,0x5454,0x7554,0x7554,0x5454,0x7575,0x7585,0x7575,0x7575,0x7575,0x7575,0x7575,0x7575,0x8585,0xD4D4,0xE4B5,0x95C4,0xC5E4,0xC5D4,0x95C4,0xB5D4,0xB5C4,0xD4D4,0x95D4,0xD4D4,0x95D4,0xC4D4,0xD495,0xC495,0xC5D4,0x95D4,0x85D4,0xC5B4,0x5475,0x7554,0x5475,0xC475,0x8575,0x7575,0x5454,0x7575,0x5475,0x8575,0xD495,0x7585,0x4446,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4F4F,0x4F4F,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x5F5F,0x4646
.word 0xD495,0x8585,0x5465,0x5475,0x5465,0x5475,0x7554,0x5475,0x6475,0x5454,0x5454,0x5454,0x4644,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4141,0x4444,0x4441,0x7F7F,0x4444,0x9F9F,0x9F9F,0x5454,0x7F7F,0x4144,0x7F7F,0x7F7F,0x4444,0x4141,0x4444,0x4444,0x5441,0x9F9F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0xC4C4,0xD474,0xD4D4,0xD4D4,0xE4D4,0xC5E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xD5E4,0xE4E4,0xE4E4,0xE4D4,0xE4C5,0xE4D4,0xD4C5,0xD4D4,0xBFBF,0x5464,0x5454,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6541,0x5475,0x5475,0x7575,0x5454,0x8575,0x7585,0x8575,0x7585,0x5454,0x5454,0x8585,0x7575,0x7575,0x8575,0x7585,0x8585,0x5485,0x7575,0x7554,0x8585,0x8585,0xC485,0xB485,0xB495,0xD495,0xC4B5,0xD4C5,0xB5D4,0xC5D4,0x95D4,0xD4D4,0xD4B5,0xC495,0xC5C5,0xC5E4,0xC5E4,0xC5E4,0xC5E4,0xD4D4,0xB4B5,0x8575,0x75D4,0x6554,0x8575,0x5475,0x8575,0xB485,0x5454,0x5454,0x7585,0x5475,0xB485,0xC495,0x75B4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4446
.word 0x8585,0x75B4,0x8575,0x8585,0x7575,0x8575,0x6585,0x7554,0x5454,0x5454,0x5454,0x4654,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x5F5F,0x6F6F,0x4141,0x7F7F,0x4141,0x5441,0x6464,0x9F9F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4444,0x4141,0x8F8F,0x51D5,0x6464,0x7F7F,0x7F7F,0x6F6F,0x6441,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D5,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD5D5,0x4154,0x7F7F,0x7F7F,0x4141,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7575,0x5454,0x5444,0x5454,0x7554,0x75C4,0x7585,0xB485,0x7585,0x5454,0x5454,0x5454,0x7554,0x5475,0x6575,0x7575,0x6575,0x7554,0x7575,0x7565,0x7565,0x5475,0x8575,0xD495,0x95D4,0xD4C4,0xB495,0xD485,0xD4D4,0xD4C5,0xD4C5,0xB5D4,0x85C4,0x85C4,0x85B4,0x85B4,0x85B4,0x75B4,0x7575,0xE485,0xB495,0x5454,0x5475,0xC454,0x5485,0x8575,0x8575,0x7575,0x8575,0x7575,0x7554,0xC485,0xB495,0x85D4,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x5444,0x5454,0x5454,0x4454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4646
.word 0x7575,0x5475,0x9585,0xD4C4,0x5485,0xB485,0x7585,0x5454,0x6554,0x5454,0x4454,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4144,0x7F7F,0x7F7F,0x4444,0x7F7F,0x5454,0x7F7F,0x5454,0x4144,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x4141,0x6464,0x4444,0x7F7F,0x6F6F,0x6F6F,0xD541,0xD574,0xD474,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4C5,0xD5D4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x4164,0x7F7F,0x4144,0x6F6F,0x7F7F,0x4444,0x6F6F,0x4141,0x8544,0x7575,0xD485,0x8595,0x7585,0x5454,0x8575,0x7575,0x7575,0x8575,0x7585,0x4475,0x5454,0x7575,0x8575,0x95C4,0x8585,0x7554,0x8585,0x8575,0x8585,0x8585,0xB485,0xB495,0x7565,0x8585,0xC495,0xC5C5,0xB4B5,0xD495,0xB5D4,0x95C4,0x95C4,0x95D4,0x85B4,0x7575,0x7575,0x8575,0x8585,0xC495,0x7595,0x5465,0x5454,0x5454,0x8575,0x54B4,0x8565,0x85B4,0x8585,0x75C4,0x5475,0x8554,0x94C4,0x95B4,0xB495,0x6585,0x4646,0x4446,0x4646,0x4646,0x4644,0x4446,0x6544,0x5454,0x7575,0x8585,0x9595,0xD4C5,0x9595,0x95B5,0xB595,0xC5B5,0xB595,0x9595,0x7575,0x6475,0x5454,0x7454,0x4446,0x4646,0x4646,0x465F
.word 0x95D4,0x8595,0xB475,0xB495,0x6585,0x7575,0x5454,0x5454,0x5475,0x4454,0x6F6F,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4644,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x5441,0xC4C4,0x5454,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x4444,0x5454,0x4444,0x6F6F,0x6F6F,0x6F6F,0xD564,0xBFBF,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xD474,0x54D5,0x6464,0x4141,0x6F6F,0x7F7F,0x6F6F,0x4444,0x7F7F,0x7575,0x7575,0x8575,0x85C4,0x95B4,0x75C4,0x5454,0x9575,0x7585,0x6575,0x7575,0x7585,0x7554,0x8585,0x5475,0x7554,0x7575,0x8575,0x6575,0x5475,0x6575,0x7575,0x9585,0x95C4,0xC495,0x7595,0x7575,0x8554,0x85B4,0x7575,0x7575,0xB485,0xB495,0xC485,0x8585,0x9585,0xC5C4,0x95C4,0x85B4,0x5465,0x5454,0xC495,0xC495,0x5475,0x7575,0x8585,0x5465,0x7575,0x7585,0x5465,0x6554,0xC485,0xC495,0xB495,0x9585,0x85C4,0x5475,0x4646,0x4646,0x5444,0x8565,0x8585,0xC595,0xB595,0xD495,0xB495,0xB485,0x8585,0x6575,0x6575,0x7575,0x6565,0x7575,0x7565,0x8585,0x8585,0x8585,0x8585,0x9595,0x7575,0x5454,0x4665
.word 0xB485,0xB485,0x7554,0x7554,0x5454,0x7554,0x5454,0x6554,0x4454,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x6464,0x7F7F,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x6441,0x9F9F,0x8F8F,0x6F6F,0x4444,0x4141,0x64C4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xD5D4,0xE4E4,0xD4E4,0xD4E4,0xE4C5,0xE4D4,0xD4D4,0xD4D4,0x6464,0x4444,0x4444,0x4444,0x7F7F,0x6F6F,0x6F6F,0x7565,0x7575,0x8575,0x9585,0x95C4,0x85D4,0x5475,0x5444,0x9575,0x7575,0x8575,0x5465,0x7554,0x7575,0x8575,0x5475,0x5475,0x5454,0x5475,0x5475,0x8575,0x7585,0x85B4,0xB485,0x8585,0x85B4,0x95C4,0x85D4,0x7585,0x8585,0xB485,0x7585,0x7575,0x7575,0xB485,0xC485,0xC485,0x7575,0x5454,0x4641,0x5444,0x95E4,0x95B4,0x5475,0x7575,0x8575,0x8585,0x5465,0x5454,0x7554,0x8585,0x8585,0x85B4,0x8585,0x85C4,0x7585,0x7585,0x5454,0x7554,0x8575,0xB595,0x95D4,0x85C4,0x75B4,0x7575,0x5454,0x5454,0x6554,0x4444,0x4446,0x4446,0x4446,0x4446,0x4646,0x5444,0x6554,0x7554,0x7575,0x7575,0x8585,0x8585,0x7585
.word 0x5454,0x5465,0x8575,0x95C4,0x95C4,0x7565,0x6554,0x6564,0x4154,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x7F7F,0x7F7F,0x4141,0x4444,0x7F7F,0x9F9F,0x8F8F,0x4444,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4141,0x4444,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4C5,0xE4D4,0xE4E4,0xD5E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4C5,0xE4D4,0xD4D4,0x64D4,0xC4D4,0x4444,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x4141,0x7F7F,0x8595,0x9595,0x85D4,0x75B4,0xC485,0x8585,0x95B4,0x7585,0x5454,0x7554,0x5475,0x5475,0x5454,0x7575,0x7575,0x5465,0x8575,0xB485,0x7554,0x7575,0x5475,0x7554,0x7585,0x8585,0x5475,0x4646,0x4444,0x6554,0x8585,0xB475,0x7585,0x7585,0x8585,0x8585,0x7575,0x5475,0x4654,0x4644,0x4646,0x4646,0x4446,0xD4D4,0xB495,0x4454,0x5475,0x8554,0x95B4,0x95E4,0x85C4,0x7585,0xB485,0x8585,0xC485,0x95B4,0x85B4,0x95B4,0x95D4,0xC495,0x9595,0xB495,0xB485,0x7554,0x7565,0x5454,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x5454,0x7565
.word 0x95D4,0x95C5,0x7554,0x8585,0xB485,0x5464,0x5454,0x5454,0x4654,0x4646,0x5F5F,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6454,0xD5D5,0x4441,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x5444,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6441,0x64D5,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD5D4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x64D4,0x51D5,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x4444,0x7541,0xE4D4,0xF4D4,0xC485,0x5485,0x7575,0x8565,0xB485,0x6585,0x5475,0x7575,0x8585,0x5475,0x7554,0x5475,0x7575,0x8575,0x7575,0x7554,0x6475,0x7585,0x5475,0x8565,0xE4C5,0x85B5,0x4665,0x4646,0x4646,0x4446,0x4444,0x4454,0x4454,0x5454,0x6575,0x6575,0x5454,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x5446,0x9595,0x5475,0x4644,0x4644,0x6554,0x8575,0xD4C4,0xD4B5,0xC495,0xD4C5,0xC5B5,0xC595,0xA5D4,0xC4C5,0x8585,0x7575,0x8575,0xB485,0x7565,0x5454,0x5454,0x4646,0x4446,0x6F6F,0x4646,0x4444,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646
.word 0xC485,0xB485,0x5475,0x7575,0x7575,0x5454,0x5454,0x4454,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4141,0x7F7F,0x7F7F,0x4444,0x7F7F,0xAFAF,0x7F7F,0x8F8F,0x4141,0x4141,0x7F7F,0x6F6F,0x4444,0x7F7F,0x6F6F,0x4444,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0xD454,0xC4C4,0x64D4,0xE4D4,0xD4D4,0xE4C5,0xE4E4,0xD4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xD5D4,0xE4E4,0xD4E4,0xD4C5,0xD4D4,0xBFBF,0x54C4,0x4154,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0xE4E4,0xD4E4,0xD4D4,0x75E4,0x8565,0x5444,0x7585,0x8585,0x95C4,0x7585,0x7575,0x7575,0x5475,0x5454,0x7575,0x7585,0x8575,0x7575,0x7575,0x7575,0x5465,0x8554,0xD495,0x95D4,0x5475,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x8575,0x8585,0x5475,0x4641,0x4644,0x4646,0x4646,0x4646,0x4644,0x8546,0x5475,0x4646,0x4646,0x5F5F,0x4646,0x5444,0x7575,0x7554,0x6575,0x6575,0x6575,0x6575,0x6575,0x7575,0x7565,0x5465,0x5454,0x5454,0x4446,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x465F
.word 0xB485,0x8585,0x7554,0x6554,0x7575,0x5454,0x5454,0x4154,0x4646,0x4646,0x5F5F,0x4644,0x4446,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4644,0x4646,0x4646,0x4646,0x7F7F,0x4141,0x7F7F,0x4141,0x4444,0x5454,0x9F9F,0x4444,0x4144,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4444,0x6F6F,0x4441,0x9F9F,0x7F7F,0x4444,0x6F6F,0x4141,0xD464,0xD4C4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E5,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0x5454,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0xF444,0xD4D4,0xD451,0x54F4,0x5454,0x5475,0xD454,0x8575,0x7575,0x7575,0x8554,0x8585,0x5475,0x5454,0x5485,0x8575,0x7554,0x8585,0x8585,0x8585,0x5454,0xD485,0x95D4,0x7585,0x4654,0x4644,0x4646,0x4646,0x4644,0x5F5F,0x4644,0x5444,0xE4C5,0x85D4,0x4475,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x5446,0x4645,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4644,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4654,0x4644,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4644,0x4646,0x5F5F,0x4446,0x4646,0x4646
.word 0x5465,0x7575,0x5485,0x8575,0x7585,0x6554,0x5454,0x4654,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4141,0x4641,0x7F7F,0x7F7F,0x7F7F,0x4141,0x9F9F,0x5454,0x8F8F,0x8F8F,0x7F7F,0x4444,0x6F6F,0x4444,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x5454,0xD4D5,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D5,0xD4E4,0xE4D5,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xBFBF,0x64D4,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0xE4E4,0x6454,0xD4D4,0x85E4,0x8585,0x6585,0x7554,0xB485,0x7554,0x7575,0x7575,0x85B4,0x7575,0x5454,0x6575,0x8554,0x6585,0x6575,0x8565,0x5475,0x6554,0x5454,0x5454,0x4554,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x8544,0x6585,0x4454,0x4446,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4654,0x4646,0x4646,0x4646,0x5F5F,0x4644,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x6F6F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644
.word 0x8554,0xB495,0xB495,0x8575,0x75B4,0x5454,0x5454,0x4654,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4644,0x4141,0x7F7F,0x4444,0x7F7F,0x4141,0x4444,0x8F8F,0x5454,0x8F8F,0x4444,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6444,0xBFBF,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xE4D5,0xD5E4,0xE4E4,0xC5E4,0xD4E4,0xD4D4,0xD4D4,0xD5D4,0x5454,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x5441,0x6464,0xE464,0x95E4,0x85B4,0x85D4,0x5454,0x8554,0x7585,0x7575,0x7575,0x7575,0x7575,0x4475,0x7554,0x5475,0x6465,0x8575,0x7585,0x5475,0x8575,0x6585,0x7554,0x7585,0x4464,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5454,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5465,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6F6F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4644,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7554,0x8585,0x85B4,0x7554,0x7554,0x5475,0x5454,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x4141,0x7F7F,0x6F6F,0x4141,0x4444,0x6F6F,0x4141,0x8F8F,0x7F7F,0x4444,0x7F7F,0xD564,0xD474,0xD4D4,0xD474,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4C5,0xE4D4,0xE4D4,0xD4D4,0xE4D5,0xD4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x64D4,0x64D4,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x4141,0x5454,0xD5D5,0xE464,0x85E4,0x8575,0x85C4,0x75C4,0x6554,0x8585,0x7585,0x7554,0x7575,0x7585,0x8575,0x5454,0x5454,0x5454,0x5475,0x7565,0x8575,0xB485,0x5475,0x5475,0x8575,0x84C4,0x7585,0x5464,0x4645,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x7544,0x7575,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4446,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4644,0x5F5F,0x4646,0x4646,0x4646
.word 0x6585,0x7585,0x8585,0x8565,0x5485,0x5475,0x5454,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4644,0x4646,0x4646,0x5F5F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x5454,0x5454,0x7F7F,0x4441,0x7F7F,0x7F7F,0x4444,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4444,0x4444,0x6F6F,0x7F7F,0x7F7F,0xD464,0xC4C4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4C5,0xD4D4,0xC5D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xC5D4,0xD4D4,0xD4D4,0xC4C4,0x8585,0x4444,0x7F7F,0x6F6F,0x4144,0x5454,0xD4C4,0xC4C4,0x75F4,0x7575,0x8575,0x8585,0x5475,0x6554,0xC485,0x8595,0x8575,0x8554,0xD485,0x7585,0x5475,0x4454,0x7554,0x7585,0xB485,0x7575,0x7575,0x5475,0x7554,0x8554,0x9585,0x95A5,0x7585,0x6575,0x4445,0x4444,0x4444,0x4444,0x6544,0x8575,0x7585,0x7575,0x75A4,0x7564,0x4644,0x4646,0x5454,0x7575,0x4664,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4454,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x5F46
.word 0x8585,0x6575,0x7554,0x7554,0x5475,0x5454,0x5454,0x4446,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4146,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4441,0x5454,0x5454,0x4141,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x4444,0x6F6F,0x6F6F,0x5454,0x6F6F,0x7F7F,0x6F6F,0x5454,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4C5,0x5464,0x4444,0x9F9F,0xBFBF,0xD564,0xD564,0x5464,0x7F7F,0x7F7F,0x7F7F,0xD564,0xD464,0xE464,0x95C4,0x8585,0x7575,0xB485,0x8585,0x6585,0x5454,0x5465,0x8575,0x75B4,0x5475,0xD485,0xC4B5,0x5485,0x5454,0x8575,0x7575,0x7575,0x5475,0x5454,0x7575,0x7575,0x7565,0x7554,0x5465,0x5475,0x7554,0x7575,0x7575,0x7575,0x7575,0x6575,0x7575,0x7575,0x5464,0x5465,0x9595,0x7564,0x5454,0x5454,0x5454,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x5475,0x4646,0x4646,0x4646,0x4646,0x4446,0x6544,0x7575,0xA595,0x6585,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646
.word 0xB485,0x7554,0x8585,0x7575,0x5475,0x5454,0x5454,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4444,0x4444,0x5454,0x7F7F,0x8F8F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x5441,0xC4C4,0x8585,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4E4,0x64D4,0x5454,0x8F8F,0x6F6F,0x4141,0x5441,0xC4C4,0x6464,0xAFAF,0x7F7F,0x4141,0x5454,0xD4D4,0xE4E4,0xE4D4,0x5485,0xB475,0xC4B5,0x9585,0xD4D4,0xC4B5,0x5485,0x8575,0x95C4,0x7575,0x8575,0x8575,0x95C4,0x54C4,0x7554,0x7575,0x8575,0x7554,0x8585,0x7585,0x6554,0x7554,0x7554,0x7554,0x6585,0x7585,0x7575,0x7575,0x7585,0x7575,0x8575,0x7585,0x7554,0x7575,0x7575,0x5475,0x5475,0x8575,0x85A5,0x5475,0x4454,0x4646,0x4646,0x4644,0x4646,0x4646,0x8565,0x9584,0x4645,0x7565,0x8554,0x74B5,0xA574,0xA595,0x8585,0x8585,0x5485,0x4446,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0x7575,0x8575,0x8575,0x75B4,0x5475,0x5454,0x5454,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x7F7F,0x4646,0x7F7F,0x7F7F,0x4646,0x4441,0x9F9F,0x8F8F,0x4154,0x4444,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4444,0x6F6F,0x7F7F,0x4444,0x6F6F,0x6F6F,0x4141,0x9F9F,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4E4,0x54D4,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0xD4C4,0xE5D4,0xD4D4,0xD4D4,0x64D4,0x6464,0xAFAF,0x7F7F,0x7F7F,0x6464,0xD4D4,0xE485,0xD4D4,0x95C4,0x75C4,0x5454,0x4454,0x6575,0x8585,0xC485,0xC495,0x7585,0x8575,0x8575,0x95B4,0xC485,0x5485,0x5454,0x7585,0x7575,0x5475,0x7575,0x5475,0x5454,0x7554,0x8575,0x85C4,0x7585,0x8585,0x9585,0x8585,0x7575,0x7575,0x8585,0x85B4,0x7585,0x7575,0x7585,0x7585,0x7575,0x5475,0x5475,0x7575,0x9585,0x6485,0x5475,0x4644,0x4644,0x7544,0x8575,0x75B4,0x9585,0x7554,0x5465,0x5475,0x5454,0x5454,0x5454,0x4645,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4F4F,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646
.word 0x95E4,0x5475,0x7575,0x8575,0x5485,0x5454,0x5454,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F5F,0x4646,0x5F5F,0x7F7F,0x4444,0x4141,0x6F6F,0x4444,0x5454,0x4141,0x7F7F,0x9F9F,0x7F7F,0x4444,0x6F6F,0x7F7F,0x6F6F,0x4444,0x4141,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x64C4,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xD454,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xE4D4,0xE4E4,0xE4E5,0xE5E5,0xE5D4,0xD4D4,0xAFAF,0x54D5,0x7F7F,0x4441,0xC4C4,0xD464,0xD4D4,0xE4E4,0xC495,0xC585,0xC5D4,0x7575,0x5454,0xD475,0x8585,0x8585,0x7575,0x7575,0x8575,0xB485,0x8575,0x7575,0x5444,0x8575,0x7575,0x7554,0x5475,0x8554,0x7575,0x5485,0x7554,0x8575,0x7575,0xB485,0x8575,0x7565,0x8585,0x8585,0x7575,0x5475,0x8575,0x8585,0xD495,0xC495,0x5485,0x5475,0x7575,0x5475,0x6465,0x7565,0x9585,0x85B5,0x8595,0x7575,0x9575,0x7595,0x5465,0x5454,0x5475,0x4454,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4446,0x4646,0x4F4F,0x4646
.word 0xB485,0x7554,0x7554,0x8585,0x54B4,0x5454,0x5454,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4446,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x6F6F,0x4141,0x5F5F,0x7F7F,0x4644,0x7F7F,0x4441,0x8F8F,0x7F7F,0x4154,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x4444,0x6F6F,0x6F6F,0x6F6F,0x5454,0x5454,0x64D4,0x5454,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6451,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0x4151,0x6F6F,0x6F6F,0x7F7F,0x4444,0x8554,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0x8F8F,0x5454,0x6464,0x4141,0x6464,0xD4D5,0xE4D4,0xE464,0xD4D4,0x9585,0x85C4,0x8565,0x7585,0x7585,0x5454,0x7565,0x9575,0x95C5,0x85C4,0x7575,0x8585,0x7585,0x8565,0x4475,0x7554,0x8585,0x7575,0x5475,0x8575,0x8565,0x7585,0x5475,0x7554,0x7565,0xC495,0x8575,0x7575,0xB485,0xB495,0x8575,0x7575,0x7575,0xB485,0x7575,0x7575,0x7585,0x7575,0x8585,0x7575,0x7575,0x5485,0x6565,0x7575,0x7585,0x5475,0x8575,0x95B4,0xC495,0x85B5,0x7585,0x7585,0x4654,0x6F6F,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7575,0x9575,0x75C4,0x7554,0x5475,0x5465,0x5454,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4141,0x6F6F,0x7F7F,0x7F7F,0x4646,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0x4144,0x7F7F,0x4444,0x6F6F,0x7F7F,0x6F6F,0x4144,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4141,0x5441,0x9F9F,0x9F9F,0x7F7F,0xC444,0xD564,0xD441,0xD564,0xD474,0xE4D4,0xE4C4,0x85E4,0x95B4,0x95C4,0x85C4,0x75B4,0x7575,0x8585,0x5475,0x5454,0x7565,0x8585,0x7575,0x7575,0x7565,0xC495,0x5485,0x5454,0x8565,0x7575,0x7585,0x7575,0x7575,0x5465,0x8585,0x8575,0x7575,0x7575,0x7585,0x7595,0x8575,0x75C4,0x7575,0x7575,0x7575,0x7585,0x7575,0x8585,0xC485,0x8585,0x75B4,0x8575,0x8554,0x7585,0x6475,0x7554,0x5475,0x8575,0x7575,0x6585,0x8575,0x85C4,0x85B4,0x85C4,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4F4F,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x465F
.word 0x5454,0x7585,0x7585,0x7565,0x5454,0x5464,0x5454,0x4645,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4446,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4646,0x4444,0x4441,0x7F7F,0x4154,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x8F8F,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xAFAF,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0xAFAF,0xBFBF,0xD454,0xAFAF,0xD4D4,0xE4E4,0xE464,0x75D4,0x6575,0x6575,0x8585,0x9585,0x95C4,0x85B4,0x8575,0x7575,0x7575,0x8575,0x7575,0x5475,0xC485,0x8585,0xB485,0x5475,0x5454,0x6554,0x7575,0x7575,0x7554,0x5454,0x5454,0x4454,0x5454,0x5454,0x5454,0x5454,0x7554,0x6465,0x7554,0x5475,0x5475,0x5475,0x5454,0x5454,0x5475,0x7575,0x8575,0xB495,0xB485,0x5475,0x7575,0x8585,0x85B4,0x7585,0x7575,0x7575,0x7575,0x8585,0xB485,0xB495,0x4654,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4F4F,0x4446,0x4646,0x4646
.word 0x8554,0x6554,0x7554,0x9585,0x5475,0x5465,0x7554,0x4654,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4644,0x4F4F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4441,0x8F8F,0x7F7F,0x8F8F,0x4444,0x7F7F,0x7F7F,0x6F6F,0x4444,0x7F7F,0x4444,0x7F7F,0x6F6F,0x4141,0x5F5F,0x6F6F,0x7F7F,0x4F4F,0x4F4F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0xD464,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0x4444,0x6F6F,0x6F6F,0x5F5F,0x6F6F,0x5F5F,0xFCFC,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0xC441,0xC4D4,0xAFAF,0xD454,0x64D4,0xE4D4,0xD4E4,0xE4D4,0x4664,0x5441,0x7554,0x6554,0x5465,0x5475,0x8585,0xC5C4,0xD4C5,0xE495,0x84D4,0x85D4,0x8554,0x95D4,0x75B4,0x7575,0x8585,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x4644,0x4446,0x4646,0x4646,0x4146,0x5454,0x5454,0x5475,0x7554,0x5454,0x5454,0x7554,0x5454,0x6554,0x5475,0x5454,0x7575,0x8585,0x8585,0x8585,0xB485,0x7554,0x7554,0x8575,0x8575,0x7575,0x8575,0x85D4,0x7575,0x5465,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x85C4,0x54C4,0x7575,0x8575,0x75C4,0x5475,0x5454,0x4654,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x5F5F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x4F4F,0x4146,0x7F7F,0x4444,0x7F7F,0x5454,0x7F7F,0x7F7F,0x4444,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x9F9F,0x6E6E,0x406E,0x7E6E,0x9D9D,0xBFBF,0x5F5F,0x6F6F,0x6F6F,0x6F6F,0x6441,0xD4D4,0xD4D4,0xD4D4,0x9F9F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x5454,0x7E7E,0x4F4F,0x6E4C,0xDCDC,0x7F7F,0x6F6F,0x4F4F,0x4444,0xD4D4,0xD4D4,0x64D4,0xAFAF,0xD464,0xD4D4,0xE4D4,0xD4D4,0x4646,0x4446,0x4646,0x4644,0x4146,0x5454,0x7554,0x5465,0x5454,0x5454,0x5475,0x6554,0xC485,0x7585,0x7585,0x8585,0x85C4,0xC485,0x5485,0x8554,0x8585,0x6585,0x5485,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x5441,0x5465,0x7565,0x7575,0x85C4,0x95B4,0x85B4,0x85C4,0x85B4,0xC485,0x8585,0xC495,0x8575,0x7575,0x7585,0x4654,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x5F5F,0x4446,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646
.word 0xC485,0x7575,0x7585,0x7575,0x5485,0x6554,0x5454,0x4654,0x4644,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4444,0x7F7F,0x5F5F,0x6F6F,0x4646,0x4646,0x4444,0x4441,0x4141,0x6464,0x8F8F,0x7F7F,0x4141,0x7F7F,0x4141,0x6F6F,0x4141,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x4141,0x5151,0x8E6E,0xF0F0,0xAFAF,0xBFBF,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0xD444,0xD4D4,0xE4D4,0xD4D4,0x7F7F,0x6F6F,0x7F7F,0x5F5F,0x6444,0xFFFF,0x8E6E,0xBDF0,0xFC9D,0x6E6E,0xCFCF,0x6F6F,0x4F4F,0x6464,0xD464,0xD4D4,0xAFAF,0xD4C4,0xE4E4,0xD4D4,0xE4E4,0x5485,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x5454,0x5454,0x7544,0x8575,0x85C4,0x7585,0x7575,0x8575,0x95B4,0x75B4,0x5454,0x8565,0x75B4,0xC485,0x5475,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5454,0x5454,0x7565,0x8575,0xB485,0xB495,0x8595,0xE495,0x95B4,0x95B4,0x85B4,0x8575,0x5465,0x5475,0x4654,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7575,0x8585,0xC485,0x5485,0x7554,0x5454,0x5454,0x4654,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x6F6F,0x7F7F,0x4646,0x6F6F,0x7F7F,0x4646,0x4146,0x4444,0x8F8F,0x7F7F,0x6454,0x8F8F,0x4141,0x7F7F,0x4444,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x6F6F,0x7F7F,0x5441,0xF474,0xF4E4,0xF4E5,0xD4E4,0xB465,0x6F6F,0x4A4A,0x6F6F,0xD441,0xE4E4,0xE4E4,0xD4E4,0x6464,0x4144,0x8F8F,0x7F7F,0xD464,0xDFDF,0xFEF0,0x6E9E,0xE06E,0xCFCF,0x8F8F,0x4F4F,0x6F6F,0xD4D4,0xE4E4,0xE4E4,0x64D4,0xE4D4,0xD4F4,0x64E4,0xD4E4,0x4544,0x4644,0x4644,0x4444,0x4444,0x4444,0x4154,0x6544,0x7554,0x5475,0x8575,0x9585,0xB4C4,0xC4B5,0x8585,0x7575,0x7575,0x5454,0x5454,0x5454,0xD485,0x8575,0x7575,0x4485,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x6454,0x9575,0x75C4,0x5454,0x7554,0x8575,0x8585,0x85C4,0x85B4,0x7585,0x7585,0x6575,0x7575,0x5475,0x5454,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x4644,0x4646,0x5F5F,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7585,0x7585,0x9585,0x7575,0x7585,0x7554,0x5454,0x4454,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4441,0x7F7F,0x4646,0x6F6F,0x4646,0x7F7F,0x4141,0x4454,0x4141,0x5454,0x4444,0x7F7F,0x4444,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x4441,0x6F6F,0x4F4F,0x5F5F,0x5F5F,0x4141,0x8F8F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0xD444,0xE4E4,0xE4E4,0xD4E4,0x41D4,0x6F6F,0x7F7F,0x6F6F,0x8F8F,0xAFAF,0x7464,0x7474,0x5464,0x7F7F,0x5F5F,0x4F4F,0x6464,0xE4D4,0xE4E4,0xD4E4,0x9F9F,0xE4D4,0x85E4,0xD4D4,0x75D4,0x5475,0x5475,0x5454,0x6554,0x7554,0x5465,0x8585,0x8585,0x8585,0x7585,0x4454,0x5454,0x5454,0x5465,0x4454,0x5454,0x5444,0x5475,0x7575,0x6554,0x8595,0x7575,0x8585,0x4475,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x4446,0x6554,0x9585,0x85B4,0x8595,0x5465,0x5475,0x7554,0x6575,0x7575,0x7575,0x7575,0x6575,0x5464,0x5475,0x5454,0x5454,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646
.word 0x75D4,0x7565,0x7575,0x7554,0x7585,0x5475,0x5454,0x5454,0x4646,0x4644,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x6F6F,0x6F6F,0x6F6F,0x4446,0x6F6F,0x4444,0x4146,0x7F7F,0x5454,0x7F7F,0x7564,0x8F8F,0x4154,0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x64D5,0xC4C4,0x8F8F,0x4141,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0xD441,0xE4D4,0xE4F5,0xE4E4,0x9F9F,0x7F7F,0x7F7F,0x6F6F,0x5F5F,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x6F6F,0x5454,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0x64D4,0xD464,0xD4D4,0xD4E4,0x8575,0x85C4,0xB485,0x7585,0x7575,0x7575,0x7575,0x7565,0x8575,0x95C4,0x85B4,0x8595,0x7585,0x7554,0x7585,0x8585,0x8575,0x8585,0xB485,0x5485,0x9554,0x75B4,0x8585,0x75B4,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4644,0x4F4F,0x4646,0x5F5F,0x4646,0x4446,0x7554,0x5465,0xB485,0x7585,0xD4B4,0x7595,0x5454,0x6554,0x5464,0x5475,0x5475,0x5454,0x5475,0x7565,0x5454,0x5475,0x7575,0x4464,0x4645,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x465F
.word 0xD495,0x7585,0x6554,0x7575,0x7575,0x5454,0x7575,0x6465,0x4644,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4141,0x4141,0x5F5F,0x4646,0x6F6F,0x5F5F,0x7F7F,0x4141,0x8F8F,0x7F7F,0x6464,0x4444,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4444,0x4444,0xD4D4,0xD4D4,0x64C4,0x64D5,0x6464,0x64D5,0xC464,0x6464,0x8F8F,0x6F6F,0x7F7F,0xD454,0xE4E4,0xE4F4,0xE4E4,0x54C4,0x7F7F,0x6F6F,0x5441,0x5454,0x6464,0xAFAF,0x5454,0x6464,0xE554,0xD4D4,0xE4D4,0xF8D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD464,0xE4E4,0x85D4,0x8585,0x85B4,0x8585,0x85B4,0x7585,0x8575,0x8575,0x8585,0x7575,0x6575,0x7575,0x6575,0x7554,0x8575,0x85B4,0x7575,0xB485,0xB485,0x8585,0x5475,0x85C4,0x7585,0xB485,0x5454,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7565,0x7554,0x7575,0x5454,0x8554,0x9585,0x75B4,0x5475,0x5454,0x5454,0x6465,0x7565,0x5454,0x5454,0x5454,0x5454,0x7554,0x5475,0xC495,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0xC485,0x5485,0x8585,0x9585,0x8575,0x85C4,0x5454,0x5454,0x4665,0x4644,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x7F7F,0x7F7F,0x4646,0x4646,0x4444,0x7F7F,0x6F6F,0x4444,0x8F8F,0x7F7F,0x6464,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4141,0xE4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0x64D4,0xD564,0xD451,0xE4D4,0xF4F5,0xE4E4,0xC4E4,0x6464,0x6454,0xD4D4,0xD4D4,0xD4D5,0xC4D4,0xF9D5,0xD4E5,0xF4D4,0xD4E4,0xF9E4,0xE4E4,0xE4E4,0xD4C5,0xD4D4,0x6485,0xD4D5,0xD4D4,0x4654,0x4646,0x4644,0x4146,0x5444,0x6554,0x7554,0x5454,0x5454,0x7575,0x7585,0x7575,0x8575,0x7585,0x8575,0x8585,0x8575,0x8585,0x7595,0x75C4,0x7554,0x8585,0x95C4,0x5475,0x4644,0x4644,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0xA454,0x5454,0x5454,0x4446,0x4446,0x8575,0x85D4,0x6465,0x7575,0x5475,0x6465,0x5465,0x5454,0x5454,0x5475,0x8554,0x7575,0x5475,0xD485,0x8575,0x8595,0x4454,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644
.word 0xB485,0x7575,0x8575,0x85B4,0x7575,0xB485,0x7554,0x5454,0x4554,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4141,0x7F7F,0x4646,0x4644,0x4646,0x7F7F,0x4141,0x4141,0x4444,0x7F7F,0xD554,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x6F6F,0x4141,0x4444,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD5E4,0xD4E4,0x64D4,0xD4D5,0xE4D4,0xF4F5,0xD4E4,0xD4D4,0xD4D4,0xF4E5,0xE4F9,0xF9F9,0xD4D4,0xD4D4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xE4E4,0xD4D4,0x6464,0xD4D4,0x5464,0x4646,0x4644,0x4646,0x4646,0x4644,0x4446,0x7554,0xC485,0xC495,0x85C5,0x85C4,0x8585,0xB485,0x7585,0x8575,0x75B4,0xB485,0xB485,0xC485,0x4464,0x8554,0x85C4,0x5485,0x4654,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4644,0x4644,0x4646,0x5446,0x8585,0x75B4,0x7585,0x7554,0x5475,0x8575,0x7554,0x8585,0x7565,0x8575,0x8565,0x75C4,0x6585,0x7554,0x7585,0x8565,0x8585,0x4444,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x464F
.word 0x5454,0x5475,0x7575,0x7575,0x7554,0x5454,0x5475,0x7554,0x5454,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4444,0x7F7F,0x4646,0x4646,0x4646,0x5F5F,0x7F7F,0x4444,0x5441,0x9F9F,0x6464,0x4444,0x7F7F,0x7F7F,0x4141,0x4141,0x6F6F,0x4444,0x7F7F,0x4444,0x4141,0xD4D4,0xC5E4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xE464,0xE5D4,0xE4F5,0xE4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xC5E4,0xF4F4,0xE4E4,0xE4D4,0xE4D4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x64C4,0x5445,0x4647,0x4644,0x4646,0x4646,0x5F5F,0x5444,0x8575,0x8585,0x95C4,0xB5C4,0x95C4,0xD485,0xB495,0xC495,0x8585,0xB485,0xB495,0x8585,0x7575,0x5475,0x5454,0x4454,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x7541,0x7575,0x8575,0x8575,0x6465,0x7554,0xC454,0x7575,0x9575,0x8565,0x9575,0x7585,0x7575,0x8585,0x7565,0x5475,0x4646,0x4444,0x5454,0x4444,0x4446,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646
.word 0x6595,0xE4C5,0x7585,0x8554,0x95C4,0x75C4,0x7585,0x5465,0x5475,0x4654,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x5F5F,0x7F7F,0x4646,0x4646,0x4646,0x6F6F,0x7F7F,0x7F7F,0x5441,0x5454,0x6454,0x7F7F,0x8F8F,0x4141,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0xE4E4,0xE4E4,0xE4C5,0xC5E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xBFBF,0xE4D4,0xF4F7,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5D4,0xD4D4,0xBFBF,0x4646,0x4646,0x4646,0x4646,0x4646,0x7545,0xB485,0x8585,0x75B4,0x7575,0x8585,0x7585,0x8575,0x7575,0x7575,0x8575,0x7575,0x8585,0x75B4,0x5475,0x7554,0x5485,0x4654,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4146,0x7565,0x5454,0x8585,0x8575,0x5444,0x5454,0x8575,0x7554,0x8575,0x7575,0x8575,0xD4B4,0x5475,0x8565,0x8585,0x5475,0x4644,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4646,0x4644
.word 0x6585,0x8585,0x85B4,0x8565,0x7585,0x7585,0x7575,0x7554,0x5454,0x5454,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4444,0x6F6F,0x4644,0x4646,0x5F5F,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x5454,0x5454,0x4141,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD574,0xD485,0xF4D4,0xF4F5,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xC5E4,0xD4E4,0xD4D4,0xD4D4,0x4664,0x4446,0x4646,0x4646,0x4646,0x8554,0x7585,0x5454,0x4454,0x4644,0x5441,0x5444,0x6554,0x5475,0x7575,0x7554,0x6575,0x5475,0x4454,0x7564,0x8585,0x6585,0x4654,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x5F5F,0x4646,0x4446,0x5475,0x4146,0x8565,0x54B4,0x4645,0x4446,0x8554,0x9575,0x5475,0x5454,0x8575,0x8575,0x5485,0x5441,0x8565,0x85C5,0x4654,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4446,0x4446,0x4646,0x5F5F,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7554,0x7575,0x5475,0x7575,0x7575,0x5465,0x7554,0x5454,0x5475,0x5475,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x6F6F,0x4646,0x4646,0x4646,0x4146,0x7F7F,0x7F7F,0x4144,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4141,0x6F6F,0x4444,0x8F8F,0xD4D4,0xE4D4,0xE4D4,0xC5E4,0xC5E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xF4D4,0xF4F4,0xD5E4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD5D4,0xE5E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xD4D4,0xC5D4,0xD4D4,0x64D4,0x4646,0x4646,0x4646,0x4646,0x7544,0x7585,0x4454,0x4646,0x4644,0x4646,0x4646,0x4644,0x4446,0x4646,0x5454,0x7575,0x7554,0x7575,0x7575,0x8585,0x75B4,0x5454,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x5446,0x4454,0x4646,0x8541,0x5485,0x4646,0x4646,0x7541,0x85C4,0x5475,0x4146,0x7554,0x7585,0x75D4,0x4646,0x5441,0xC475,0x4475,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4644,0x465F
.word 0xD475,0x7585,0x7575,0x5475,0x5475,0x9554,0x95C4,0x5485,0x7554,0x5454,0x4654,0x4444,0x4646,0x5F5F,0x4F4F,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x5F5F,0x5F5F,0x4444,0x7F7F,0x4141,0x5454,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4441,0xD464,0xD4D4,0xD4C5,0xD4E4,0xD4E4,0xE4E4,0xC5D4,0xC5D4,0x74D4,0x64D4,0xC4D4,0xD4D4,0xE5D4,0xF4F4,0xE4F4,0xD4D4,0xD4C5,0xE4C5,0xC5D4,0xD4E4,0xD4F4,0xE4E4,0xE4E5,0xE4E4,0xE4D5,0xE4F4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x4654,0x4646,0x4646,0x4444,0x5475,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4444,0x8585,0x95D4,0xB5D4,0x95D4,0x8585,0x5475,0x5454,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4541,0x4446,0x5F5F,0x7544,0x5475,0x4446,0x4646,0x5446,0x7585,0x4654,0x4646,0x4146,0x8565,0x7585,0x4646,0x4646,0x5441,0x7554,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x8585,0xC485,0xD454,0xD4B4,0xC494,0x9575,0xC5C4,0x75C4,0x7554,0x5454,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4144,0x5454,0x4444,0x4141,0x4444,0x4444,0x4141,0x7F7F,0x4444,0x6F6F,0x7F7F,0x8F8F,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD5D4,0xBFBF,0xD464,0xD4D4,0xF4D4,0xF4F4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D5,0xF4E4,0xF4E4,0xE4E4,0xE5E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x64D4,0x4646,0x4646,0x4646,0x7544,0x4454,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x8554,0x85B4,0x7585,0x7575,0x5454,0x4454,0x4654,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x5446,0x4475,0x4646,0x4646,0x4146,0x6575,0x4654,0x5F5F,0x4646,0x6541,0x7585,0x4446,0x5F5F,0x4646,0x5441,0x4641,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x5F5F,0x4646
.word 0xB485,0x7595,0x8575,0xC495,0x9585,0x8575,0xC485,0x8595,0x5454,0x5485,0x5465,0x4654,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x6F6F,0x7F7F,0x6F6F,0x7F7F,0x8F8F,0x5454,0x4154,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4444,0x7F7F,0x6F6F,0x4444,0x8F8F,0xC4C4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xC5E4,0xD4D4,0xD4D4,0xD464,0xD485,0xD4D4,0xD4D4,0xE5D4,0xF5F4,0xE4F4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4C5,0xD4D4,0xD4D4,0x4446,0x4646,0x4444,0x4454,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x7544,0x7575,0x5454,0x4644,0x4646,0x4646,0x4644,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4644,0x4646,0x5F5F,0x4646,0x5454,0x4646,0x4646,0x4446,0x4146,0x8554,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7575,0x5475,0xC485,0x8585,0x7575,0x6554,0xB485,0x6575,0x7575,0x6554,0x7554,0x4454,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4444,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x4444,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x5454,0x5454,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4141,0x4141,0x4141,0x7F7F,0x8F8F,0xC4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0x64D4,0xD5D5,0xBFBF,0xD4D4,0xD4D4,0xF4E4,0xF5F5,0xE4F4,0xD4E4,0xD4C5,0xC5D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0x5464,0x4646,0x5F5F,0x4546,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x7575,0x5454,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4441,0x4446,0x4446,0x4646,0x4646,0x4441,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4644,0x4446,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F
.word 0x8595,0x6554,0x7554,0x7565,0x7565,0x7585,0x5475,0x9585,0x85C4,0x5454,0x5475,0x5454,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4444,0x8F8F,0x5454,0x4444,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD464,0xD464,0xD4D4,0xF4D4,0xF5F5,0xE4F7,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D5,0xD5E4,0xE4E4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0xAFAF,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5454,0x4654,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646
.word 0x85C4,0x9554,0xB5C5,0x8585,0x9554,0x85C4,0xC495,0xB485,0x7585,0x6465,0x7544,0x7554,0x6554,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x4646,0x6F6F,0x7F7F,0x4141,0x7F7F,0x4444,0x7F7F,0x9F9F,0x5444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4444,0x7F7F,0x7F7F,0x4444,0xD4C4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xBFBF,0x64D5,0xD4D4,0xD4D4,0xE4E4,0xF5F5,0xE4F4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4C5,0xD4E4,0xD4D4,0xD4E4,0x64D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4444,0x4154,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4446,0x5F5F,0x5F5F,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x5F5F,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7575,0xC475,0xC485,0xB485,0x8575,0xB485,0x6585,0x5454,0x5454,0x8585,0x5475,0x7554,0x5454,0x4465,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x6F6F,0x7F7F,0x4444,0x7F7F,0x4141,0x4141,0x5441,0x5454,0x7F7F,0x7F7F,0x4444,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0xBFBF,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xAFAF,0x64D4,0xD4D4,0xF4E4,0xFFFF,0xE4F4,0xD4E4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xC5D4,0x64D4,0x6464,0xAFAF,0x4646,0x4644,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646
.word 0x7575,0x8585,0xB485,0x5475,0x6575,0x5475,0x5475,0xD495,0x6585,0x7554,0x7585,0x5454,0x7575,0x5454,0x4644,0x4446,0x4646,0x4446,0x4646,0x4446,0x5F5F,0x4646,0x4F4F,0x4646,0x5F5F,0x4646,0x4441,0x7F7F,0x7F7F,0x4444,0x7F7F,0x5454,0x5454,0x8F8F,0x4444,0x7F7F,0x7F7F,0x4444,0x4444,0x6F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF5F5,0xE4F5,0xD4E4,0xE4E4,0xD4D4,0xBFBF,0xD4D4,0xD4D4,0xD4C5,0xE4E4,0xD5D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x64D4,0xD564,0x4644,0x4444,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x5454,0x4654,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x5F5F,0x4646,0x4646
.word 0x5454,0x6565,0x5475,0x7575,0x8595,0x95E4,0x8565,0x85D4,0x7575,0xB485,0x7554,0x6585,0x7554,0x6554,0x4454,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4141,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x9F9F,0x4454,0x7F7F,0x7F7F,0x4141,0x7F7F,0x5454,0x7F7F,0x7F7F,0x4444,0xD4C4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xAFAF,0xD464,0xD4E4,0xD4D4,0xF4E4,0xFFFF,0xE4F4,0xD4E4,0xE4C5,0xD4D4,0xD485,0xD4E4,0xE4C5,0xD4D4,0xE4D4,0xD4E4,0xC5D4,0xD4D4,0xD4E4,0xD474,0xAFAF,0xC4C4,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4644,0x4646,0x4646,0x4446,0x4F4F,0x4F4F,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F5F,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F
.word 0x5475,0x8585,0x8585,0x7585,0x95D4,0x85B4,0x7585,0x7575,0x8554,0x95D4,0x7575,0x8595,0x5454,0x7575,0x7575,0x4454,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4644,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x9F9F,0x6464,0x7F7F,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6464,0xD4C4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0x85D4,0xD4D5,0xD4D4,0xE4D4,0xF4E4,0xE4E4,0xE4F4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64D5,0xD5D5,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x4444,0x4646,0x5F5F,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7554,0xB485,0x7585,0x5475,0xB485,0x7585,0x5454,0x8575,0x75B4,0x7585,0x7554,0xC475,0x7575,0x5454,0x5475,0x5475,0x4465,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5444,0x4444,0x8F8F,0x6F6F,0x7F7F,0x4444,0x4444,0x7F7F,0x4444,0x8F8F,0xD4C4,0xD474,0xC5D4,0xE4D4,0xC5D4,0xE4E4,0xE4D4,0xD4C5,0x64E4,0x9F9F,0x6464,0x6464,0xE4D4,0xE4E4,0xD4D4,0x6485,0xAFAF,0x9F9F,0xD4E4,0xE4E5,0xE4D4,0xC5D4,0xD4E4,0xE4C5,0xE4D4,0xD4D4,0x74D4,0xD4D4,0x6464,0x46C4,0x4446,0x4646,0x4646,0x5F5F,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x5F5F,0x4446,0x5F5F,0x4646,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x4646,0x4644,0x4446,0x4F4F,0x4F4F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4444,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x5F5F,0x4F4F,0x5F5F,0x4644,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4644
.word 0x7554,0x7585,0x7565,0x7554,0x7575,0x5454,0xD485,0xC485,0x5495,0x7565,0x7554,0x5454,0x5454,0x5454,0x7554,0x7575,0x5454,0x4645,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4444,0x7F7F,0x4444,0x4141,0x7F7F,0x4444,0x7F7F,0x4141,0xD454,0x4444,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5454,0xC4C4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xC5E4,0xD4D4,0x5454,0x9F9F,0x6464,0x5464,0xC4D4,0x6464,0x8F8F,0x5454,0x8F8F,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD4C4,0x46D4,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x5F5F,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4646,0x4644,0x4644,0x4646,0x4646
.word 0x6554,0x5475,0x5454,0x8575,0xB5D4,0x75C4,0x7575,0x8585,0x7575,0x9585,0x85C4,0xC475,0xC495,0x8575,0x5475,0x6554,0x7575,0x5454,0x4575,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x6464,0xAFAF,0x4141,0x7F7F,0x6F6F,0x4141,0x7F7F,0x4144,0x5454,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4C5,0xD4E4,0x41D4,0x8F8F,0x4441,0x8F8F,0xD4D4,0xBFBF,0x5464,0x8F8F,0x4444,0xC4C4,0xF4F4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0x8585,0xD4D4,0x4664,0x4444,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F5F,0x4646,0x4F4F,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x4444,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4F4F,0x4646,0x5F5F,0x4446
.word 0x5454,0x7554,0x7585,0xC454,0xB485,0x85B5,0x6554,0x6575,0x9554,0xD4C4,0xC4B5,0x8575,0x95C4,0xD475,0x8585,0x5454,0x5475,0x7575,0x7554,0x4454,0x4644,0x4646,0x4644,0x4646,0x7F7F,0x4141,0x4141,0x7F7F,0x4444,0x6F6F,0x4444,0x7F7F,0x4441,0x8F8F,0x8F8F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x74D4,0xE4D4,0xC5D4,0xD5D4,0xD4E4,0xE4E4,0xD4D4,0x64D4,0x9F9F,0x4141,0x4444,0x4144,0x5444,0x4444,0x8F8F,0x9F9F,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4C5,0xD4E4,0x74D4,0x64D4,0xD4D5,0xD4D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x464F
.word 0x7554,0x7554,0x7554,0x7554,0x8575,0x75B4,0xC585,0x95C4,0x75D4,0x8585,0x64B4,0x5454,0x5454,0x8575,0x75C4,0x8575,0x5465,0x7554,0x5475,0x6575,0x5475,0x7554,0x6544,0x5465,0x4165,0x4444,0x7F7F,0x4141,0x7F7F,0x7F7F,0x5441,0x4141,0x7F7F,0x7F7F,0x5454,0x4141,0x7F7F,0x7F7F,0x6F6F,0x4444,0x4141,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x9F9F,0x9F9F,0x9F9F,0xE4C4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xF5E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xAFAF,0x74D4,0xD464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4446,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x5F5F,0x4F4F,0x5F5F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4446,0x4646,0x4644,0x4646,0x5F5F,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x5F5F,0x4F4F,0x4F4F,0x4646,0x5F5F,0x4646,0x5F5F,0x4446,0x5F5F,0x4646,0x4646,0x4646
.word 0x5454,0x6554,0x7575,0x7554,0x7554,0x5454,0x8575,0x7585,0x6585,0x7575,0x5454,0x9575,0x85C5,0x5465,0x5485,0xD475,0x8595,0x5465,0x8554,0x5475,0x7575,0x5475,0x7575,0x5475,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x4444,0x9F9F,0x4444,0x7F7F,0x8F8F,0x9F9F,0x5454,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0xC441,0xBFBF,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xF4D4,0xC4E4,0x5454,0x64D5,0xE4E4,0xE4F4,0xE4D4,0xE4F4,0xF4E4,0xF4F5,0xE4E4,0xD5E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD464,0xD5D4,0xBFBF,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4644,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646
.word 0x5454,0x5454,0x5454,0x7554,0xC495,0xC495,0x7554,0x6575,0x8554,0xC495,0xC495,0xC475,0xB5D4,0xC485,0xB495,0x7554,0x7575,0x5454,0x7575,0x5454,0x7554,0x7585,0x7575,0x7575,0x4141,0x4144,0x4444,0x7F7F,0x7F7F,0x4141,0x4154,0x4144,0x4141,0x8F8F,0x6444,0x9F9F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x4141,0x4444,0x8F8F,0x74D4,0xBFBF,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xE4D4,0xF5F5,0xE4F4,0xE4D4,0xE4F4,0xF4D4,0xE4F4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xD5E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xBFBF,0xBFBF,0x6485,0x4446,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4446,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4446,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x465F
.word 0x5444,0x5454,0x5475,0x7575,0xC485,0x8585,0x7565,0x5475,0xC475,0xD495,0xD495,0x8554,0x7595,0x7565,0x95D4,0x5485,0x8575,0x7585,0xC554,0x5475,0x5475,0x5454,0x5454,0x5454,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x5454,0x4444,0x4444,0x4444,0x5454,0x41D5,0x8F8F,0x4444,0x7F7F,0x6F6F,0x7F7F,0x7F7F,0x8F8F,0xC4C4,0xC4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xC4D4,0xB4C5,0xC565,0xE475,0xE4F5,0xF1E4,0x65E4,0x4565,0x6545,0x8465,0xD4D5,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xC4D4,0x64D4,0x44D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x5F5F,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4446
.word 0x5454,0x5454,0x5454,0x5454,0x6575,0x4475,0xD485,0xD4C5,0x7595,0x95C4,0x85B4,0x7554,0x7565,0x5485,0xC485,0x7575,0xE495,0xC4C5,0x7575,0x7554,0xC585,0x75C4,0x5454,0x5454,0x4141,0x4444,0x7F7F,0x4141,0x7F7F,0x4141,0x4444,0x5464,0x5454,0x7F7F,0x8F8F,0xC441,0x4141,0x7F7F,0x6F6F,0x4444,0x4141,0x8F8F,0x5454,0x64D4,0xBFBF,0x74D4,0x64D4,0xAFAF,0x4444,0x6544,0x4445,0x6544,0x6565,0xC565,0xC575,0x6575,0x4545,0x6545,0x4545,0x4465,0x4545,0x7545,0xB4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD574,0xBFBF,0xBFBF,0x9F9F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4446,0x4F4F,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x5F5F,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x5F5F,0x5F5F,0x4646
.word 0x4485,0x5454,0x5465,0x5454,0x5454,0x5454,0xC585,0x8595,0x7575,0x7575,0x7585,0xD485,0xD4D5,0x85D4,0x7575,0x7554,0xC485,0x8595,0x5465,0x7585,0x7565,0x5475,0x8575,0x5454,0x4444,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6444,0x5454,0x7F7F,0x8F8F,0x4444,0x8F8F,0x6464,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x4441,0xBFBF,0xC4C4,0xAFAF,0x7F7F,0x4444,0x4564,0xC545,0xC565,0x6565,0x65C5,0x65C5,0x4565,0x6545,0xC565,0xC545,0xC5C5,0xC575,0xC565,0x4665,0x4444,0xD465,0xD464,0xD4D4,0x64D4,0xBFBF,0x64D4,0xD4D5,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x5F5F,0x5F5F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F46
.word 0x6595,0x5454,0x6564,0x8554,0x8585,0x54B4,0x9575,0x75B4,0x5475,0x5454,0x7585,0x9565,0x8585,0x75D4,0x8554,0x85C4,0x7565,0x5465,0xE485,0x85D4,0xD485,0x7585,0x5454,0x7F7F,0x4141,0x4444,0x7F7F,0x4141,0x7F7F,0x4444,0x6441,0x9F9F,0x5454,0x4444,0x8F8F,0x5454,0xC4C4,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x4444,0x7F7F,0xD564,0xD574,0xD574,0xD4C4,0x4565,0x4444,0x4444,0x4444,0x4444,0x4444,0x4444,0x6F6F,0x6F6F,0x5F5F,0x5445,0x4544,0x4544,0x4444,0x6544,0xD4D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xBFBF,0x85C4,0x6564,0x6465,0x6465,0x5454,0x4454,0x4446,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644
.word 0x85D4,0x5444,0x5454,0x7554,0x6575,0x7575,0x7575,0x5475,0xC454,0xD4C5,0x95C5,0x7575,0x8575,0x6585,0xC495,0x8594,0x8575,0x7585,0x8575,0x85D4,0xB5B4,0x85D4,0x4475,0x4141,0x4444,0x4141,0x4141,0x8F8F,0x4444,0x7F7F,0x9F9F,0x5454,0x4444,0x7F7F,0x8F8F,0x4141,0x4444,0x4164,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x6464,0xC4C4,0xD4C4,0xC4C4,0xD4D4,0x45C5,0x4544,0x84C5,0xC5C5,0xC5C5,0x4445,0x4644,0x4444,0xC544,0xE5D6,0xD5F5,0xC5C5,0x4575,0xD5C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0x6464,0x64D4,0x85D4,0x5475,0x7575,0x7554,0x7554,0x7575,0x6575,0x5475,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x5F5F,0x4644,0x4646
.word 0x95D4,0x4485,0x5454,0x5454,0x5454,0x5454,0x8585,0x85C4,0x8554,0x85C4,0xB485,0x7554,0x7554,0x8565,0xB5D4,0x85C4,0xC565,0xC495,0x6575,0x5475,0x8575,0x54B4,0x4444,0x4141,0x4444,0x4441,0x7F7F,0x5464,0x5454,0x7F7F,0x7F7F,0x9F9F,0x6464,0x4444,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5454,0x7F7F,0x7F7F,0x4444,0x4141,0x9F9F,0x64D4,0x64D4,0x64D4,0x85D5,0xC5C5,0xC575,0xD665,0xD5C4,0x7575,0xD5D5,0xF5D5,0xF5D5,0xE4D5,0xB4C5,0xC565,0x6554,0xC5C5,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xBFBF,0xD564,0xD4D4,0x7575,0x7575,0x7575,0x7575,0x7575,0x7554,0x7554,0x5475,0x5475,0x7554,0x4654,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x4F4F,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4F4F,0x4F4F,0x5F5F,0x4446,0x4F4F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F
.word 0xC495,0x7585,0x4454,0x7554,0x8554,0x5485,0x7575,0x8575,0x7575,0xB485,0x7585,0x9575,0x95C5,0x7575,0xC485,0x6585,0x9585,0x95C4,0x7554,0x7585,0x5454,0x4154,0x7F7F,0x7F7F,0x4141,0x8F8F,0x4141,0x8F8F,0x4141,0x7F7F,0x5441,0xAFAF,0x4444,0x9F9F,0x7F7F,0x8F8F,0x4444,0x4444,0x8F8F,0x4154,0x7F7F,0x4141,0x5454,0x5454,0xAFAF,0xD4D4,0xC4C4,0xBFBF,0x8585,0x65C5,0x6565,0x6575,0xC5C5,0xD4D4,0xF5E5,0xD5C5,0xC5B4,0xC575,0x7565,0x65C5,0x74C5,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD474,0xD464,0x8585,0x85D4,0xD485,0xD495,0xE4C5,0x8595,0x8585,0x7575,0x5475,0x7554,0x7575,0x7575,0x5475,0x4645,0x4646,0x4446,0x4F4F,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646
.word 0xB5C4,0x85E4,0x5475,0x5454,0x6575,0x6554,0x5475,0x8575,0x7565,0x5475,0x7554,0xC495,0xC495,0x5485,0x6575,0x5475,0x7575,0x6575,0x9554,0x95C4,0x5465,0x4144,0x4141,0x7F7F,0x4144,0x8F8F,0x7F7F,0x9F9F,0x5454,0x7F7F,0x8F8F,0xD575,0x5454,0x4454,0x4141,0xC4C4,0x4141,0x7F7F,0x8F8F,0x8F8F,0x7F7F,0x4444,0x7F7F,0x5454,0x5454,0x6464,0x64D5,0x64D4,0xC4D4,0x9F9F,0x75D6,0x6545,0x65D5,0xC5C5,0x6575,0x6575,0xC575,0x7565,0x65C5,0xBFBF,0xE4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xC4C4,0xBFBF,0xD4D4,0x6575,0x5454,0x5454,0x5454,0x5454,0x5454,0x7575,0x8585,0x8575,0x8575,0x7575,0x7554,0x6475,0x6454,0x4645,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x4446,0x4F4F,0x4F4F,0x4446,0x4646,0x5F5F,0x5F5F,0x4644,0x5F5F,0x4F4F,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4644,0x4446,0x5F5F,0x5F5F,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4446
.word 0xC495,0xB5D4,0x54C4,0x5454,0x5454,0x5475,0x7554,0x6575,0x5454,0x8554,0x7575,0xC485,0xD485,0x7585,0x5454,0x7575,0x7554,0x5454,0x7575,0x7585,0x4175,0x4444,0x8F8F,0x7F7F,0x4444,0x8F8F,0x4444,0x5454,0x8F8F,0x4141,0x8F8F,0x9F9F,0x8F8F,0x9F9F,0x7F7F,0x5441,0x4444,0x4141,0x4444,0x7F7F,0x6444,0x4141,0x7F7F,0x8F8F,0x5454,0xAFAF,0xC4C4,0xC4C4,0xC4C4,0xAFAF,0x4554,0x65D6,0x54C5,0x6565,0xC565,0x6545,0xC545,0xC554,0xAFAF,0xD4C4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x64D4,0x64D4,0xD4D5,0xC464,0x5485,0x5454,0x6554,0x8575,0x95C4,0x75B4,0x5454,0x7554,0x8575,0xD495,0x7585,0x7575,0x8575,0x5475,0x6475,0x4654,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x5F5F,0x4644,0x4646,0x4446,0x4F4F,0x4F4F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x5F5F,0x4F4F,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x85E4,0xD495,0xB4B5,0x5475,0x5454,0x5454,0x5454,0x5454,0x8554,0xB485,0x54B4,0x7575,0x7585,0x7554,0x8585,0x8585,0x8554,0x75C4,0x5454,0x5465,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4444,0x5441,0x7F7F,0x5454,0x7F7F,0x7F7F,0x4444,0x5454,0x4444,0x9F9F,0x8F8F,0x7F7F,0x4444,0xD564,0xD4D4,0x85D4,0x9F9F,0x7F7F,0x7F7F,0x8F8F,0x5454,0x6464,0xD4D4,0xD485,0x85D4,0xD464,0xAFAF,0x5454,0x5454,0x9F9F,0x4141,0x9F9F,0x5454,0x4141,0xD464,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x64D4,0xBFBF,0xBFBF,0x85C4,0x5475,0x7575,0x8575,0x8595,0x8554,0x85C4,0x7585,0x8554,0x85C4,0x5465,0x5454,0x8575,0x8595,0x7575,0x8575,0x7585,0x5475,0x5475,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x5F46
.word 0x95D4,0x95B4,0xB5C4,0x7585,0x4454,0x6541,0x7554,0x8575,0x7554,0x7575,0x6585,0x7554,0x5454,0x8565,0x85B4,0x75B4,0x8575,0x7585,0x7575,0x7F7F,0x4141,0x4144,0x7F7F,0x4444,0x8F8F,0x5454,0x7F7F,0x4141,0x8F8F,0x7F7F,0x4141,0x4444,0x7575,0x4154,0x4444,0x5454,0x6454,0x64D5,0xD464,0xBFBF,0xD4C4,0x85D4,0x4444,0x7F7F,0x4441,0x9F9F,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xC4D4,0xAFAF,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x9F9F,0xD4C4,0xE4E5,0xD4E4,0xE4D4,0xD4D4,0xD4C5,0xBFBF,0x64C4,0x64D5,0x8585,0x95C4,0x7585,0xD475,0xC4D4,0x5454,0x5475,0x6554,0x5454,0x5454,0x8575,0x7575,0x5454,0x7565,0x8585,0x7585,0x7575,0x7575,0x7575,0x5475,0x4646,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4F4F,0x5F5F,0x4646,0x4646,0x5F5F,0x5F5F,0x4446,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xB5D4,0x85C4,0x85B4,0x95D4,0x5485,0x5454,0x5475,0x7554,0x7554,0x5454,0x5465,0x8554,0xC485,0x7585,0x7554,0x7554,0x6575,0x5475,0x7F7F,0x7F7F,0x4144,0x5454,0x8F8F,0x7F7F,0x8F8F,0x4441,0x4141,0x4444,0x4444,0x8F8F,0x8F8F,0x5444,0x8F8F,0x7F7F,0x8F8F,0xD554,0xAFAF,0xBFBF,0xAFAF,0x6464,0x64D4,0x64D4,0x64D5,0xC4D4,0xAFAF,0x5454,0x6454,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xD474,0x6585,0xC495,0x7595,0x7554,0x5475,0x7575,0xC485,0x9595,0x7575,0xD485,0xB5D4,0x54B4,0x95D4,0x6585,0x5454,0x8565,0x8585,0x7575,0x7575,0x8575,0x7575,0x4665,0x4644,0x4646,0x4646,0x4F4F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x5F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644
.word 0xC4C5,0xC495,0xD495,0xC5D4,0x85D4,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x7554,0x8575,0x5475,0x8575,0x7554,0x5454,0x4454,0x7F7F,0x4444,0x8F8F,0x8F8F,0x7F7F,0x5454,0x8F8F,0x7F7F,0x4444,0x4144,0x7F7F,0x8F8F,0x6F6F,0x5441,0x8F8F,0x8F8F,0x6444,0xAFAF,0x64D4,0xC4D5,0xBFBF,0xD4C4,0xD4D4,0xD4D4,0xD464,0xD4D4,0x64D4,0x64D4,0x8F8F,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xF4F4,0xF4E4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xD4D4,0x74D4,0xBFBF,0xBFBF,0x85B4,0x75C4,0x8585,0x54B4,0x5454,0x8575,0x7585,0x8585,0xD4C4,0x5485,0x9585,0x95D4,0x7585,0xC485,0x7585,0x7554,0x5454,0x8565,0x8595,0x8585,0x8575,0x8575,0x7575,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x464F
.word 0xB485,0x4644,0xD495,0xC495,0xC495,0xC495,0x5475,0x5454,0x7554,0x5475,0x5485,0x6554,0x5454,0x5485,0x7575,0x5475,0x5454,0x4141,0x4144,0x8F8F,0x5441,0x4141,0x4444,0x7F7F,0x8F8F,0x4441,0x8F8F,0x5454,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x6464,0xD4D5,0xC4D4,0xC4C4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x41D4,0xD464,0xD4D4,0xE4D4,0xD5D4,0xF4E4,0xE4E4,0xF4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x85D4,0x7575,0x5475,0x7554,0x5454,0x7575,0xD475,0xD4B5,0x85D4,0x8575,0x8585,0x5465,0x7565,0x5475,0x5454,0x5465,0x7575,0x8585,0x8565,0x5475,0x6554,0x8585,0x8575,0x7575,0x6554,0x7575,0x4454,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4444,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646
.word 0x4475,0x4646,0x9585,0x95C4,0x95C4,0xB5C4,0x85C4,0x5454,0x7554,0x7554,0x5475,0x5475,0x8554,0x7575,0x7554,0x5454,0x4141,0x4444,0x7F7F,0x8F8F,0x9F9F,0x4444,0x8F8F,0x4444,0x4444,0x7F7F,0x7F7F,0x4441,0x7F7F,0x6F6F,0x4141,0x4444,0x6441,0xD5D5,0x64D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xAFAF,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4F4,0xE4F4,0xC5E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x7585,0xB5C4,0x95C5,0x8575,0xD4D4,0xC4B5,0x8564,0x95B4,0x85D4,0x6554,0x5454,0x7575,0xD485,0xC495,0x7585,0x8575,0xE495,0x85D4,0x7585,0xC495,0x5465,0x7554,0xB485,0x8554,0x8585,0x7575,0x8575,0x4654,0x4444,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4446,0x4646,0x4F4F,0x5F5F,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x4F4F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646
.word 0x4646,0x6F6F,0x95D4,0x95C4,0x85B4,0xB485,0xD4B5,0xD4D4,0x6585,0x5454,0x4454,0x5454,0x6575,0x5475,0x5454,0x4444,0x7F7F,0x4141,0x9F9F,0x5454,0x5454,0x8F8F,0x7F7F,0x8F8F,0x5441,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0xD5D4,0xD464,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xE4C5,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0x51D4,0xD4D4,0xD5D4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0x74D4,0xD485,0x5495,0xC485,0xE485,0xC485,0xC495,0x85B5,0x8565,0xB485,0x5475,0x9575,0xC5D4,0x85C5,0x8565,0xB5D4,0x95D4,0x8565,0x85C4,0xB495,0x6585,0xC495,0x85B5,0x7575,0x7554,0x8575,0x85B5,0x7585,0x7554,0x5475,0x4664,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x5F5F,0x4646,0x4646,0x4F4F,0x4F4F,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x5F5F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x4F4F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0x4646,0x9546,0x95D4,0x95D4,0xD4C4,0x8585,0x95C4,0x9595,0xB5D4,0x5475,0x5454,0x5454,0x6475,0x5454,0x8F8F,0x4141,0x7F7F,0x4444,0x6444,0x5454,0x8F8F,0x7F7F,0x7F7F,0x4144,0x8F8F,0x4444,0x7F7F,0x4141,0x7F7F,0x4141,0x8F8F,0xD464,0xAFAF,0x64D4,0xD4D4,0xD4D4,0xE4D4,0xD5D4,0xD4E4,0xE4E4,0xE4C5,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x64D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4E4,0x85D4,0xB485,0xB485,0x8585,0x85B4,0x75B4,0x5475,0x5454,0x7575,0x9565,0x95C4,0x85D4,0x8575,0xC495,0x85D4,0x8575,0xB475,0x8585,0x5465,0x7575,0x7575,0x7554,0x5475,0x7554,0xB485,0xB495,0x7585,0x7585,0x7575,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4F4F,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4644,0x5F5F,0x4446,0x4646,0x4446,0x4F4F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46
.word 0x4446,0xD446,0xC495,0x6585,0x4646,0x9565,0xD4D4,0xD4C4,0xC495,0xE485,0x95D4,0x8585,0x5475,0x4144,0x5444,0x7F7F,0x6464,0x8F8F,0x9F9F,0x9F9F,0x4444,0x4141,0x4141,0x8F8F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x5441,0xC4D4,0xBFBF,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xC5D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x64D4,0x6464,0x9F9F,0x6464,0x6454,0x6464,0x6464,0x64D4,0xD464,0x8575,0x8585,0x8585,0xB485,0x7575,0x5454,0x5454,0x7575,0x5454,0xC485,0xC5B5,0xD4C5,0x8585,0x85B4,0x75C4,0x8565,0x85B4,0x5475,0x6575,0x5454,0x8575,0x8575,0x8585,0x7585,0x8575,0x85D4,0x5475,0x7554,0x9585,0x85C4,0x7575,0x7575,0x7575,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x5F5F,0x4F4F,0x4446,0x4646,0x4646,0x4644,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4444,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x5F5F,0x4F4F,0x4646,0x5F5F,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646
.word 0x4646,0xB546,0x85D4,0x4665,0x4446,0x8546,0xD4C5,0xD495,0xC4B5,0x9585,0xD4C4,0xD4C5,0x5485,0x4141,0x8F8F,0x4444,0x8F8F,0x5444,0x5454,0x4454,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x4144,0x4141,0x5454,0xD4D4,0xAFAF,0xD4C4,0x64D4,0xBFBF,0xD4C4,0xD4D4,0xD4E4,0xD4E4,0xC5E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xC5E4,0xE4E4,0xC5D4,0xD4E4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0xC475,0x7585,0x7585,0x95B4,0x5475,0x8554,0x7595,0x8575,0x94D4,0x95D4,0x8575,0xD4D4,0x95D4,0x75B4,0x8585,0x5475,0x5475,0x5454,0x8585,0x8575,0xB5C4,0xD4D4,0x7575,0xD4C4,0xC495,0x8585,0xB5C4,0x85C4,0x5454,0x8575,0x95C4,0x85B4,0x8585,0x7554,0x4575,0x4644,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4F4F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x5F5F,0x5F5F,0x4F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646
.word 0x4646,0x5F5F,0x65B5,0x4646,0x4646,0xD446,0x95D4,0x85C4,0x4646,0x8585,0xE495,0x6585,0x4141,0x5444,0x7F7F,0x4144,0x4141,0x5454,0x9F9F,0x5454,0x8F8F,0x4444,0x7F7F,0x4444,0x8F8F,0x4141,0x4444,0x5454,0x64D4,0x64D5,0xD4D4,0xD464,0xBFBF,0xD4C4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D5,0xD5E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0x8F8F,0x5454,0x5454,0x4454,0x5454,0x4154,0x7575,0x5454,0x5454,0x6554,0x6585,0xD495,0xD4C5,0x8575,0xC495,0xC495,0x8575,0x8585,0x7585,0x7554,0xD485,0x8595,0x8565,0x95C4,0xD4C4,0x54D4,0xC485,0x9595,0x75C4,0x8585,0x85C4,0x54B4,0x7554,0x5475,0x7565,0x5454,0x8565,0x8585,0x75B4,0x7565,0x7585,0x4654,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4F4F,0x4F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x5F5F,0x5F5F,0x4646,0x4F4F,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x4646,0x8546,0x46D4,0x6F6F,0x4646,0x8544,0x85D4,0x4695,0x4646,0xD475,0x75D4,0x8F8F,0x5454,0x7F7F,0x4444,0x8F8F,0x7F7F,0x6464,0x8F8F,0x8F8F,0x4444,0x7F7F,0x5441,0x5454,0x4141,0x4141,0x6F6F,0x5454,0xC4C4,0xD464,0xD464,0xC4C4,0x64D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x9F9F,0xAFAF,0x5454,0x9F9F,0x5454,0x5454,0x8575,0xC5D4,0xB5D4,0x85C4,0x95C4,0x85D4,0xC475,0xB485,0x7595,0x7575,0x7575,0x5475,0x9575,0xD4C5,0xD4C5,0x7585,0xC485,0x9595,0x75D4,0x8585,0x85B4,0x5485,0xB485,0x8585,0x7565,0x9585,0x95C4,0x85C4,0x6585,0x5454,0x95B4,0x8585,0x5475,0x7585,0x6475,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4646,0x5F5F,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F46
.word 0x4646,0x4646,0x4675,0x4646,0x4646,0x9546,0x95D4,0x4646,0x4644,0x8546,0x4154,0x7F7F,0x6464,0x4444,0x7F7F,0x4444,0x4141,0x9F9F,0x9F9F,0x4141,0x7F7F,0x7F7F,0x9F9F,0x8F8F,0x4444,0x7F7F,0xD454,0x64D4,0xC4D5,0xAFAF,0xD4D4,0xD485,0xD4D4,0x74D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xF4E4,0xE4E4,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xD4D4,0x64D4,0xD464,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0x64D4,0xAFAF,0xAFAF,0x6464,0x7554,0xC485,0x7585,0xC485,0xC495,0x7575,0x7575,0x7575,0x5454,0x5454,0x7554,0x5454,0x8565,0xB5C4,0xC475,0x9595,0xC495,0x6585,0x7575,0x8575,0x5465,0x6575,0x5475,0x8565,0x5465,0x6554,0x7585,0xC485,0xC5B5,0xD4B4,0x85D4,0x5475,0x7554,0x9575,0x7585,0x8575,0x7575,0x6585,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4446,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4F4F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4446,0x4646,0x4644,0x4646,0x4646,0x8546,0xB485,0x4644,0x4646,0x4146,0x7F7F,0x5454,0x7F7F,0x7F7F,0x4141,0x4141,0x5454,0x4444,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x4444,0x64D4,0xD4D4,0x8585,0x64C4,0x64D5,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD5D4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0x64E4,0x8F8F,0x7F7F,0x4141,0x4141,0x4141,0xD441,0xD4D4,0xD4D4,0x6464,0xAFAF,0x5454,0x9F9F,0xC5C4,0x75C4,0x8585,0x75C4,0x5454,0x5454,0x6554,0xD475,0xD4B5,0x6585,0xB5D4,0x95E4,0xD4C5,0x8585,0x85B4,0x85C4,0x5475,0x5454,0x5454,0xD475,0xD4B5,0x8595,0x8565,0xC5D4,0xC5E4,0x54C4,0xC485,0xC5D4,0x95D4,0x8585,0x8565,0x5465,0x7575,0x85D4,0x7575,0x8575,0x8575,0x4644,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F4F,0x4F4F,0x5F5F,0x4646,0x4646,0x4F4F,0x4F4F,0x5F5F,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4546,0x4645,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646
.word 0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4685,0x4646,0x4646,0x6F6F,0x4444,0x4141,0x4444,0x8F8F,0x7F7F,0x4141,0x8F8F,0x4141,0x4141,0x4444,0x7F7F,0x8F8F,0x4444,0x7F7F,0x8F8F,0xD454,0xC4C4,0xD564,0xD464,0xD464,0xBFBF,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4C5,0xE4D4,0xE4E4,0xE4D5,0xE4F4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0x8585,0x4144,0x8F8F,0x64D4,0x6464,0x5444,0x8F8F,0x7F7F,0x4141,0xD564,0xD4D4,0x6454,0xAFAF,0xC475,0xD4C5,0x5475,0x5454,0x7554,0x9585,0xD4D4,0xC4B5,0x8575,0xB5C4,0x85D4,0xB495,0xD485,0x8595,0x7565,0x8585,0x7575,0x8554,0xC5C4,0x95D4,0x7585,0xD495,0xC5D4,0x8585,0xC495,0xD4B5,0xB495,0x8585,0xB5D4,0x85B4,0x7575,0xC485,0x7595,0x5454,0x8575,0x8585,0x7575,0x7585,0x4485,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4F4F,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4F4F,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x5F5F,0x4644,0x4646,0x8544,0x4645,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x464F
.word 0x5F5F,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4675,0x4644,0x5F5F,0x4141,0x8F8F,0x7F7F,0x4141,0x4444,0x7F7F,0x8F8F,0x7F7F,0x4144,0x7F7F,0x7F7F,0x4141,0x4444,0x7F7F,0x4141,0x8F8F,0x64D4,0x64D5,0xD5D5,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0x41D4,0x8F8F,0x54D4,0x4141,0xD5D5,0x6464,0x8F8F,0x64D4,0x9F9F,0x4444,0x6464,0xD4D4,0xD4D4,0x5454,0x8575,0x85C4,0x7554,0xD495,0xC4C5,0xB485,0xB495,0x95B5,0x9575,0x85B4,0x75C4,0x8585,0x95B4,0x54B4,0x7575,0x5454,0x7554,0x9565,0xD4D4,0xE4B5,0x75D4,0x85C4,0xC495,0x8565,0x95B4,0x95B4,0x85C4,0x7575,0xB485,0x7585,0x5465,0x7565,0x7575,0x5454,0x7554,0x75B4,0x7575,0xB485,0x7585,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x5F5F,0x5F5F,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x9575,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446
.word 0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x7F7F,0x4444,0x7F7F,0x4141,0x4444,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4444,0x4141,0x4141,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x5454,0xAFAF,0x8585,0x6485,0x64D5,0xBFBF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D5,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0x41D4,0x8F8F,0x5454,0x8F8F,0x4444,0x4144,0x9F9F,0x8F8F,0xAFAF,0x9F9F,0x8F8F,0x7F7F,0x8F8F,0xD4D4,0x9F9F,0x7575,0x7554,0xB485,0xC485,0xC4B5,0x8575,0x8585,0x75B4,0x7554,0x8585,0x7565,0x7565,0x5454,0x7554,0xD485,0xB5D4,0xD4D4,0x7564,0x9585,0x95C4,0x54B4,0x8585,0x85B4,0x8575,0x7575,0x7585,0x6575,0x5475,0x5454,0x7575,0x8564,0x9585,0xC485,0xC485,0x8585,0xD4D4,0xD464,0xD4D4,0xD4D4,0x8574,0x8574,0x7464,0x4654,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4644,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x4F4F,0x5F5F,0x5F5F,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x7544,0x6595,0x4646,0x4646,0x4644,0x4446,0x4F4F,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x7F7F,0x7F7F,0x4141,0x4444,0x7F7F,0x4141,0x7F7F,0x4144,0x7F7F,0x4444,0x8F8F,0x7F7F,0x7F7F,0x4141,0x4444,0x7F7F,0x4141,0xD454,0xD464,0xD564,0xBFBF,0xD485,0xD4C4,0xBFBF,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xC5D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xD4D4,0x5454,0x4444,0x7F7F,0x5454,0x5454,0x8F8F,0x9F9F,0x9F9F,0x8F8F,0x8F8F,0x7F7F,0x6464,0x4141,0xAFAF,0x75D4,0x7554,0xC485,0x7585,0x8585,0x75B4,0x6554,0x5465,0x5454,0x5454,0x5454,0xD485,0xE4C5,0x95D4,0x6585,0x8585,0xC495,0x8595,0x7575,0x85B4,0x85C4,0x6465,0x7575,0x5475,0x6554,0x6575,0x5475,0x7554,0x8585,0xC485,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D5,0xD4D4,0x64D4,0x4464,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x9554,0x45C4,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x464F
.word 0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4444,0x4141,0x8F8F,0x4141,0x4441,0x7F7F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x4441,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4444,0x7F7F,0xD4D4,0xC4C4,0xC4C4,0xC485,0xBFBF,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE441,0x41D4,0x4444,0x9F9F,0x5454,0x6444,0x64D5,0x5454,0x9F9F,0x9F9F,0x4454,0x8F8F,0x4141,0x6454,0x8F8F,0x5454,0xD485,0x75D4,0x8585,0x6554,0x5475,0x5454,0xB585,0x95E4,0x7575,0xD4D4,0xB495,0x9575,0x95C4,0xC5D4,0x85C4,0xB485,0xC485,0x6585,0x5475,0x6554,0x5454,0xC485,0xB5D4,0x7585,0xD475,0xE4B4,0x85D4,0x8575,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD4D4,0x4174,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4444,0x4F4F,0x4646,0x4646,0x4646,0x5F5F,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5444,0xA585,0x4665,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4F4F,0x4646
.word 0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x7F7F,0x4141,0x4441,0x8F8F,0x4444,0x8F8F,0x4141,0x4141,0x8F8F,0x7F7F,0x4441,0x8F8F,0x4444,0x7F7F,0x4144,0x7F7F,0x4141,0x5454,0xD4C4,0xD564,0xD464,0xD564,0xD464,0xD464,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D5,0xE4E4,0x41D4,0xE4D4,0x8F8F,0x4444,0xD564,0x5454,0x8F8F,0xD564,0x6464,0x4444,0x5454,0x9F9F,0x4141,0x8F8F,0x9F9F,0x8F8F,0x5454,0xD4D4,0xD4D4,0x7575,0x8554,0xC495,0x7585,0x95C4,0xD4B4,0x5485,0xC495,0x85D4,0x8575,0x85B4,0xB485,0x6595,0x7575,0x6575,0x5454,0xC585,0xB5D4,0x7585,0xD485,0xD4C5,0x85D4,0x8575,0x95D4,0xB495,0xD485,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4C5,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xAFAF,0xD464,0xD4D4,0xD4D4,0x4654,0x4444,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x8544,0xB495,0x4644,0x4644,0x5F5F,0x4644,0x4646,0x4446,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646
.word 0x4646,0x4644,0x4644,0x4646,0x4646,0x6F6F,0x7F7F,0x4444,0x8F8F,0x4444,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4144,0x7F7F,0x8F8F,0x4141,0x7F7F,0x4141,0x7F7F,0x5454,0xD475,0xAFAF,0xC4C4,0xD564,0xBFBF,0xD4D4,0xC4C4,0xD574,0xD4D4,0xD4D4,0xD4E4,0x64D4,0xD4D4,0xC5D4,0xD4E4,0xD4E4,0xE4D5,0xD441,0xE4E4,0x4144,0x8F8F,0x6441,0x8F8F,0x8F8F,0x8F8F,0x64D5,0xAFAF,0x9F9F,0x5454,0x5454,0x5454,0x4444,0x5454,0x9F9F,0x8F8F,0xD4D4,0x75D4,0x6575,0x7575,0x5485,0xC475,0x8585,0xB454,0x9585,0x75C4,0x7575,0x7575,0x7575,0x8554,0xE495,0xD495,0xB4B5,0xC475,0xD485,0x85D4,0x9575,0x95C4,0x95D4,0x8575,0xB485,0xD464,0xC4C4,0xD4D4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xE4D4,0xD4D5,0xE4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0x5474,0x4645,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4646,0x4446,0x7565,0x4644,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x5F5F,0x6544,0xB495,0x4595,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4446,0x5F46
.word 0x5F5F,0x4646,0x4646,0x5F5F,0x6F6F,0x7F7F,0x4441,0x8F8F,0x4444,0x7F7F,0x7F7F,0x8F8F,0x4141,0x4141,0x7F7F,0x4141,0x4444,0x4144,0x7F7F,0x7F7F,0x4444,0x7F7F,0xD454,0x64D5,0x64D5,0xC4D4,0x8585,0x64D4,0xBFBF,0xD564,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xBFBF,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0x41D4,0x41E4,0xAFAF,0x5454,0x7F7F,0x9F9F,0x5464,0x4141,0xD541,0x6464,0x9F9F,0x5454,0x9F9F,0x9F9F,0x4141,0x5454,0x5454,0x4154,0x7F7F,0x85D4,0x5454,0x5475,0x5454,0x7575,0x5475,0x7575,0x7575,0x5475,0x5454,0x5454,0x8575,0x7575,0x95C4,0x95C4,0x95D4,0x8575,0x95C4,0xB495,0x8575,0x85B4,0xB495,0x7575,0x6475,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4C5,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFBF,0xD4D5,0x4464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x9554,0x4665,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x9575,0xC495,0x4685,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x4141,0x7F7F,0x4141,0x4444,0x7F7F,0x4444,0x4141,0x4444,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x6444,0xAFAF,0x6464,0xD4D4,0x64C4,0x64D4,0xD4D5,0xD464,0xD4D4,0xD485,0x61D4,0xD4D4,0x64D4,0xC4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x4444,0xAFAF,0x8F8F,0x5454,0x4444,0xD554,0x6464,0x9F9F,0xAFAF,0x8F8F,0x4444,0x6454,0x5454,0x5454,0x9F9F,0x5454,0x5454,0x9F9F,0x5454,0x6585,0xC485,0x7575,0x7585,0x7554,0x5454,0x5454,0x5475,0x8585,0x95C4,0xC5D4,0x5485,0xC485,0xB495,0xC485,0x8575,0xB485,0x7585,0x7575,0x7575,0x5475,0x6454,0xC4C4,0xBFBF,0xE4D4,0xD4E4,0xD4C5,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0x4644,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4444,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x6446,0x6495,0x4646,0x4646,0x5F5F,0x4644,0x4646,0x4446,0x5446,0xC495,0x8585,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x5F5F,0x4446,0x4646,0x4646,0x5F5F,0x5F46
.word 0x4646,0x4644,0x4646,0x6F6F,0x7F7F,0x4441,0x7F7F,0x7F7F,0x4444,0x7F7F,0x4141,0x7F7F,0x4141,0x4141,0x4444,0x4141,0x7F7F,0x4141,0x7F7F,0x4441,0x4141,0x5454,0xC4D5,0xBFBF,0x64C4,0xBFBF,0xBFBF,0xC4C4,0xBFBF,0xAFAF,0xD4D4,0xE564,0xD4D4,0xD4D4,0xAFAF,0x7F7F,0x7F7F,0x8F8F,0xE464,0xC5D4,0x6464,0x8F8F,0x4444,0x5454,0x6464,0x9F9F,0x8F8F,0x5464,0x4141,0x4444,0xD564,0x4441,0x9F9F,0xAFAF,0xAFAF,0x9F9F,0x5454,0x5454,0x5454,0x9F9F,0x7554,0x7575,0x7585,0x9575,0x5485,0x7585,0xC485,0xB4C5,0x9575,0x85C4,0xD4C4,0x6585,0x8585,0x85B4,0x8585,0x7565,0x7575,0x5454,0x5454,0x5454,0x7575,0x5454,0x64C4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xC5D4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xC5D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x4461,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4446,0x8575,0x4654,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x9544,0x8585,0x75B4,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646
.word 0x4646,0x4646,0x4646,0x6F6F,0x7F7F,0x8F8F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x8F8F,0x7F7F,0x7F7F,0x4444,0x4141,0x4444,0xD4C4,0xBFBF,0x64D4,0x64D5,0x64D5,0x64D5,0x64D4,0x64D5,0xC4D4,0xD4D4,0xAFAF,0xD4D4,0x54D4,0xD464,0x4141,0x7F7F,0x6454,0xD4E5,0xE4D4,0xD4D4,0x4141,0x7F7F,0x6444,0xAFAF,0x8F8F,0x4441,0x8F8F,0x8F8F,0xD454,0xE4E4,0xE4E4,0x8F8F,0xD4D4,0x4444,0x9F9F,0xAFAF,0x5454,0x5454,0x9F9F,0x9F9F,0x5454,0x5475,0x6575,0x6585,0x7585,0x8585,0x6585,0xB485,0x8585,0xB495,0x5475,0x7575,0x7575,0x5465,0x5454,0x9585,0x8565,0xC495,0x8595,0x5454,0xAFAF,0xBFBF,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E4,0xE4E4,0xE4D5,0xD4E4,0xD4D5,0xD4E4,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4C5,0xC5E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x4641,0x4646,0x5F5F,0x4446,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x5F5F,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7544,0x54A5,0x4646,0x4646,0x4646,0x4646,0x4646,0x8585,0x95D4,0x4685,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446
.word 0x4446,0x4646,0x4646,0x4444,0x4141,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4144,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x4141,0x9F9F,0xAFAF,0xAFAF,0xC4C4,0xD464,0xC4C4,0xBFBF,0xAFAF,0x64D4,0xD4D4,0xD464,0xE4D4,0x4141,0x6464,0xD4D5,0xE4D4,0xE4D4,0xD4D4,0x41D4,0x4444,0x5454,0x9F9F,0x8F8F,0x4444,0x8F8F,0x6464,0x9F9F,0x4444,0x41D4,0x8F8F,0x7F7F,0x7F7F,0xE441,0xD4E4,0x4141,0x7F7F,0x5454,0xAFAF,0x6464,0x4154,0x5444,0x5454,0x5454,0x7554,0x7554,0x6565,0x5454,0x7575,0x7575,0x6575,0x7554,0x5454,0x5454,0x8575,0xC5C4,0xD4E4,0x8575,0x85C4,0x5475,0xD554,0xC464,0xD464,0xD4D4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4C5,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD464,0x5464,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4644,0x4646,0x4646,0x4646,0x4446,0x9585,0x4645,0x4646,0x4646,0x4644,0x4646,0x9585,0xB4D4,0x4675,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646
.word 0x4646,0x4646,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4444,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0xC4C4,0xBFBF,0x64C4,0x64D4,0x64D4,0x64D5,0x64D4,0x64D5,0x85D5,0xC4C4,0xC4C4,0xD464,0x6441,0x85D5,0x64D5,0xC4C4,0xD4D4,0xD4D4,0x6464,0x4444,0x7F7F,0x5454,0xC4C4,0x8F8F,0x4144,0x6464,0xD4D5,0x9F9F,0xC454,0x5454,0xE4D4,0xE4D4,0x54D4,0x6464,0x7F7F,0x7F7F,0x4444,0x8F8F,0x9F9F,0x5454,0x5454,0x8F8F,0x5444,0x5454,0x5454,0x5454,0x5454,0x5475,0x9585,0xD4E4,0x85B5,0x9554,0x95C4,0x95D4,0x8575,0xC485,0x8595,0x8554,0x5475,0xAFAF,0xAFAF,0xD464,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D5,0xE4D4,0xD4D4,0xD4C5,0xE4C5,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xC5E4,0xD4E4,0xE4D4,0xD4E4,0xBFBF,0x4464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4644,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x7545,0x4644,0x4646,0x4646,0x4646,0x4646,0xB575,0x4585,0x4446,0x5F5F,0x4646,0x4446,0xB485,0x8595,0x4675,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0x4646,0x4646,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4444,0x4444,0x7F7F,0x4141,0x7F7F,0x4141,0x7F7F,0x4441,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0xD441,0x64D5,0xD4D4,0xBFBF,0xBFBF,0xD564,0xC4C4,0xBFBF,0xC4C4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0xD4C4,0xD464,0xD4D4,0xE4E4,0x6464,0xD454,0x8F8F,0x6441,0xD5D5,0x8F8F,0x5454,0x8F8F,0x8F8F,0x8F8F,0xBFBF,0xD4D4,0xE4E4,0xD4E4,0x41E4,0x7F7F,0x7F7F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4444,0x9F9F,0x7F7F,0x4441,0x5441,0x5454,0xC485,0x8595,0x5485,0x85B4,0xB485,0xC485,0xC475,0xC495,0xB485,0x8554,0x85B4,0x75B4,0x6565,0x5454,0xD564,0xD564,0xD4D4,0xD4D4,0xD4D4,0xC5E4,0xE4E4,0xD4E4,0xE4E4,0xD5E4,0xE4D4,0xD5D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0x6464,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x5F5F,0x5446,0x4475,0x4446,0x4646,0x5F5F,0x4644,0x8544,0x75D4,0x4646,0x4646,0x4646,0x4646,0x9585,0x85B4,0x4675,0x4646,0x4646,0x4646,0x4646,0x4444,0x4675,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646
.word 0x4646,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x4444,0x7F7F,0x4141,0xD454,0xD464,0x64D4,0x64D5,0x64D5,0x64D4,0x64D4,0x64D4,0xD4D4,0xBFBF,0xD464,0xC4D4,0xD464,0xD4D4,0x6464,0xC4C4,0xAFAF,0xD454,0xD4D4,0xD4D4,0x41D4,0x8F8F,0x7F7F,0x9F9F,0xD464,0x64D4,0x8F8F,0xAFAF,0xC464,0xE4E4,0xE4D4,0xC5D4,0x6464,0x7F7F,0x8F8F,0x4444,0x4444,0x4444,0x4441,0x9F9F,0x8F8F,0x9F9F,0x8F8F,0x7554,0x5444,0x5454,0x7565,0x75C4,0x7575,0x7585,0x7575,0x8554,0x75B4,0x7575,0x7554,0x6554,0x5454,0x5475,0x6464,0xAFAF,0xC4C4,0xC485,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xE4D4,0xD5D4,0xD4E4,0xD4E4,0xD4E4,0x51D4,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x4F4F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x7554,0x4644,0x4646,0x4646,0x4646,0x5446,0x9585,0x4654,0x4446,0x4646,0x6565,0xC495,0x8585,0x4475,0x4646,0x4646,0x4644,0x4646,0x8565,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4644
.word 0x7F7F,0x4441,0x6F6F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4141,0x7F7F,0x7F7F,0x4444,0xAFAF,0xAFAF,0xD564,0xD464,0xC4C4,0xC4C4,0xBFBF,0xAFAF,0xD564,0xD464,0xAFAF,0xD4C4,0xAFAF,0xD4D4,0xC4D4,0xAFAF,0xD454,0xD4D4,0x64E4,0x8F8F,0x9F9F,0xAFAF,0x4444,0x8F8F,0x4141,0x8F8F,0x7F7F,0x7575,0xC4C4,0x6464,0x5454,0x8F8F,0x7F7F,0x4444,0x4444,0x8F8F,0x4444,0x4444,0x8F8F,0x8F8F,0x4141,0x5454,0x4164,0x6464,0x54C4,0x5454,0x5454,0x5454,0x5454,0x7565,0x5454,0x5475,0x5475,0x5454,0xC485,0x95C5,0x5485,0x5454,0x7575,0x64D5,0x85D5,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD5E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xD4C5,0xE4E4,0xD4C5,0xE4E4,0xC5D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4C5,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x4641,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4F4F,0x4446,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x8544,0x4675,0x4446,0x5F5F,0x4646,0x4644,0xC485,0x4685,0x5F5F,0x5F5F,0x7546,0x85C5,0x95B4,0x4475,0x4646,0x4644,0x4646,0x6546,0x4485,0x4446,0x4646,0x4644,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4644,0x4446,0x5F5F,0x4646
.word 0x7F7F,0x7F7F,0x7F7F,0x4444,0x4444,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0xD4C4,0xD564,0xC4C4,0x8585,0xD564,0xD464,0xD564,0xD464,0xBFBF,0xBFBF,0xD4C4,0x64D4,0x64D4,0xD4D4,0xAFAF,0xD4D4,0x64D4,0x6464,0xD4D4,0xD4D4,0xC4C4,0x5454,0x5454,0xD564,0x64D4,0x4444,0x5454,0x8F8F,0x4141,0x9F9F,0xE4E4,0xE4D4,0x41D4,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4441,0x4444,0x4444,0x8F8F,0x64D4,0x9F9F,0x7564,0x75C4,0x5454,0x5454,0x5454,0x7554,0x5475,0x8595,0x8585,0x5485,0x8585,0x85B4,0x5475,0x5454,0xC4C4,0xD464,0xBFBF,0xD4D4,0xD4D4,0xE4D4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E5,0xC5E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xC5E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x54D4,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x5446,0x7595,0x4646,0x4646,0x4646,0x4646,0x8574,0x6595,0x4646,0x4646,0x8546,0xB495,0x7595,0x4575,0x4646,0x4646,0x4646,0x8544,0x4585,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x5F46
.word 0x4141,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x7F7F,0x6F6F,0x7F7F,0x4141,0x4141,0x4141,0x7F7F,0x7F7F,0x7F7F,0x9F9F,0x64D4,0x64D4,0xD4D4,0xD564,0xBFBF,0xC4C4,0xC4C4,0xBFBF,0xD564,0xD464,0xD464,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x41D4,0xD564,0xD464,0xD4D4,0xD454,0xD4D4,0x8F8F,0x5454,0xD4C4,0x6464,0x8F8F,0xD4D4,0x64D4,0x5454,0xE464,0xE4D4,0xE4E4,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x4144,0x8F8F,0x5454,0x4444,0x4141,0x5454,0xD5E4,0x9F9F,0x9F9F,0xD454,0x5485,0x7554,0x7554,0x8585,0x54C4,0x7575,0x7575,0x7575,0x7575,0x7575,0x5454,0xAFAF,0xD564,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xC5E4,0xD4E4,0xD4D4,0xC5E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xC5E4,0xE4D4,0xD4D4,0x64D4,0x4664,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x6546,0x7585,0x4644,0x4646,0x5F5F,0x4646,0x8544,0x9585,0x4646,0x4446,0xB444,0x8595,0x95B4,0x4585,0x4646,0x4646,0x4646,0x9575,0x4675,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4646
.word 0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x4444,0x4444,0x7F7F,0x4441,0x7F7F,0x8F8F,0x64C4,0xD4D5,0xC4C4,0xAFAF,0x74D4,0x64D5,0x64D5,0x64D4,0xD4C4,0xD4D4,0xD4D4,0xBFBF,0xD464,0xC4C4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xBFBF,0xD464,0xD4D4,0xD464,0x4141,0x4444,0x4141,0x9F9F,0xD554,0xD4D4,0xD4D4,0xD554,0xD4D4,0xE4E4,0xD4E4,0x41E4,0x7F7F,0x7F7F,0x8F8F,0x4441,0x7F7F,0x8F8F,0x8F8F,0x4444,0x4444,0xD4D4,0x54D4,0x9F9F,0x9F9F,0x95C4,0x5465,0x5465,0x7565,0x5475,0x5475,0x5454,0x7554,0xC485,0x5495,0x6464,0x7575,0xBFBF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D5,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4E4,0xC5D4,0xE4D4,0xE4C5,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xC5E4,0xE4D4,0xD4D4,0xC5D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xBFBF,0x4646,0x4644,0x4646,0x4444,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x9575,0x4675,0x4646,0x4646,0x4646,0xB444,0xB495,0x4654,0x4646,0x9565,0x95B4,0xB485,0x4585,0x4446,0x4646,0x4444,0x9595,0x4675,0x4644,0x4446,0x4646,0x4644,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x465F
.word 0x4141,0x4444,0x4144,0x7F7F,0x4444,0x4441,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x41D4,0x9F9F,0xD464,0xD4C4,0x6485,0x64D5,0xBFBF,0xBFBF,0xD485,0xC4C4,0x64D4,0xD4D4,0xD464,0x64D5,0xD4D4,0xD4D4,0xAFAF,0x64D4,0x6464,0xD5D4,0xBFBF,0xD4D4,0x54D4,0x41D5,0xAFAF,0x64C4,0x8F8F,0x7F7F,0x4141,0x8F8F,0x4141,0xD485,0xE4E4,0xE4E4,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x4144,0x5441,0x9F9F,0x4141,0x8F8F,0x8F8F,0x64E4,0x5454,0x54D5,0xC464,0x54D4,0x5454,0x5454,0x5454,0x5475,0x6554,0xC485,0x8575,0x6565,0x6454,0x64D5,0xD4D5,0x85D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4D5,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD5D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD5E4,0xE4E4,0xD4E4,0xC5E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x64D4,0x4654,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x8554,0x6585,0x4446,0x4646,0x4644,0x8546,0x85C4,0x4675,0x4444,0x9585,0xB485,0x8585,0x45B4,0x4646,0x4646,0x7546,0xB495,0x4665,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4644,0x4646,0x4446
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5454,0x4141,0x4141,0x4444,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x4444,0x7F7F,0x7F7F,0x7F7F,0x5454,0xAFAF,0x8585,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD485,0xAFAF,0xC4C4,0xBFBF,0xBFBF,0xD4C4,0xD464,0xBFBF,0xD4C4,0xD485,0xC4C4,0xD4D4,0x64D4,0x64C4,0xD4D4,0xD4D4,0x9F9F,0x6454,0xAFAF,0x4144,0x6464,0x8F8F,0x5441,0xD464,0xAFAF,0x6451,0x8F8F,0x6441,0x8F8F,0x7F7F,0x7F7F,0x4141,0x4444,0x8F8F,0x4141,0x5454,0x4154,0x7F7F,0xD4D4,0x5454,0xBFBF,0x5464,0xD464,0x5485,0x5454,0x7585,0x7585,0x8585,0x8575,0x5454,0x6454,0xAFAF,0xD464,0xD474,0xD4D4,0xD4D4,0xE4E4,0xC5D4,0xE4D4,0xE4E4,0xE4C5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xC5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0x4464,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5446,0x4465,0x4646,0x5F5F,0x4646,0x5F5F,0x8554,0x75B4,0x5F5F,0x4646,0x5F5F,0x8546,0x8585,0x4675,0x7545,0x7595,0x95D4,0xB5C4,0x4585,0x4646,0x5F5F,0x9554,0x85D4,0x4665,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4F4F,0x5F5F,0x4646,0x4646,0x4646
.word 0x7F7F,0x7F7F,0x4444,0x7F7F,0x5454,0x5454,0x4444,0x7F7F,0x7F7F,0x8F8F,0x4141,0x4154,0x4444,0x7F7F,0x7F7F,0x7F7F,0x6464,0x5454,0xD4C4,0x8F8F,0xD4C4,0xD464,0xD4C4,0xD4D4,0xD464,0xAFAF,0x64C4,0x85D4,0x64D4,0xD4D5,0xBFBF,0xBFBF,0xD4C4,0xBFBF,0xD4D4,0xBFBF,0xD464,0xD4D4,0xAFAF,0xBFBF,0xD4D4,0x64D4,0x5454,0x5454,0x8F8F,0xD4D5,0x6464,0x5454,0xD4D5,0xD4D4,0x4444,0x8F8F,0xD4E4,0x7F7F,0x4444,0x4444,0x4444,0x4444,0x4141,0x9F9F,0x5454,0x8F8F,0x4141,0x7F7F,0x6464,0xE454,0xD554,0x8F8F,0x7595,0x5454,0x7575,0x5485,0x5465,0x5475,0x5454,0xAFAF,0xBFBF,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xC5D4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xD4E4,0xD5D4,0xE4E4,0xD4E4,0xE4E4,0xF4E4,0xE4E4,0xD4E4,0xE4F4,0xE4D4,0xD4E4,0xD4C5,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xAFAF,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4446,0x5485,0x4644,0x4646,0x4646,0x4646,0x9544,0x8595,0x4645,0x4646,0x4646,0x9554,0x85C4,0x4575,0x8554,0x85B4,0x85C4,0x85C4,0x6575,0x4646,0x4646,0x9575,0x95B4,0x4645,0x4644,0x4646,0x5F5F,0x4F4F,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646
.word 0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x5464,0xAFAF,0x7F7F,0x6464,0x4444,0x4141,0x7F7F,0x5454,0x4141,0x4444,0x4444,0x7F7F,0x41C4,0x64D4,0xD4D4,0x64C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFBF,0xBFBF,0xBFBF,0xD585,0xD464,0xD4C4,0xD4D4,0xD564,0x6464,0xC4D4,0x64D5,0xAFAF,0xD4D4,0x85D4,0xD464,0xD4D4,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0x4141,0xE4D4,0xD4E4,0x4141,0x4141,0x4141,0x4141,0x7F7F,0x4144,0x4444,0x8F8F,0x8F8F,0x4141,0xC444,0x41D4,0xE454,0xD4D4,0x8F8F,0x8544,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0xBFBF,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4C5,0xE4D4,0xD5D4,0xE4D4,0xD4D4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4C5,0xD4E4,0xD4D4,0xD4D4,0xC5E4,0xD4D4,0xD4D4,0x85D4,0x4651,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4646,0x7544,0x7595,0x4446,0x4646,0x4646,0xB444,0x7595,0x4645,0x4644,0x4446,0x9575,0x85B4,0x7585,0x8585,0x9585,0x85C4,0xC495,0x8575,0x4446,0x5446,0x9595,0x75B4,0x4646,0x4646,0x4444,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x5F5F,0x4446,0x4646,0x4646,0x4446
.word 0x7F7F,0x8F8F,0x7F7F,0x5441,0xAFAF,0x5454,0x4154,0xAFAF,0x5454,0x4464,0x8F8F,0x5454,0x5454,0x4141,0x4141,0x7F7F,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4C4,0xD464,0xBFBF,0x6464,0x64D4,0xC4C4,0xD4D4,0xC464,0xD464,0xD464,0xD4D4,0xE464,0xE4D4,0xBFBF,0x8585,0xC4C4,0x5454,0x5441,0xC464,0x8F8F,0x9F9F,0x6485,0x8F8F,0xC4C4,0xE4D4,0xE4E4,0x6464,0x4444,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x5454,0x4141,0x7F7F,0x8F8F,0xE5E5,0xD444,0x6454,0x9F9F,0x8585,0x4485,0x7554,0x5485,0x7585,0x5454,0xAFAF,0x64D5,0xD4D5,0xD4D4,0xD4D4,0xBFBF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D5,0xE4E4,0xE4E4,0xD4E4,0xE5E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D5,0xD4D4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5464,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x6565,0x4644,0x4446,0x8595,0x4665,0x4646,0x4646,0xB444,0xC485,0x4675,0x5F5F,0x7546,0xC495,0x7585,0x9575,0x85B4,0x95B4,0x95B4,0x95B4,0x7575,0x4654,0x8544,0x95B4,0x6585,0x4446,0x4446,0x4685,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646
.word 0x7F7F,0x4141,0x5454,0x8F8F,0xAFAF,0x5454,0xAFAF,0x64D5,0xD554,0x6464,0x4444,0x64C4,0x6464,0x7F7F,0x5454,0x4441,0x54D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD574,0xD4D4,0xD4D4,0xC4D4,0x6464,0x85D5,0xC4C4,0xD4D4,0xD4D4,0xAFAF,0xD4D4,0xD4D4,0xD564,0xD4D4,0xC4D4,0xD464,0x5454,0xD554,0xD4D4,0x7F7F,0xD4D4,0xD5D4,0x7F7F,0x41D4,0xD4D4,0x4141,0x64C4,0x8F8F,0x6441,0x4444,0x4141,0x7F7F,0x7F7F,0x8F8F,0x4454,0x4141,0x4444,0x54E4,0x5454,0x5454,0x41D4,0x8F8F,0x54D4,0x5454,0x6575,0x5475,0x5454,0xBFBF,0xBFBF,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xC5D4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xD4C5,0xD4C5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0x64E4,0x4644,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x6546,0x4454,0x4646,0x9575,0x4475,0x4446,0x4646,0x8574,0x8585,0x4675,0x4646,0x9554,0x85C4,0x7585,0x75D4,0x7585,0xB495,0x8585,0xC485,0x7585,0x4675,0x9575,0xB495,0x4585,0x4446,0x8585,0x4665,0x4644,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x464F
.word 0x8F8F,0x4144,0x4444,0x6475,0x6464,0x7575,0xAFAF,0xD564,0xAFAF,0x64D5,0x4141,0xD4D4,0x7F7F,0x5454,0x9F9F,0x5454,0xAFAF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD485,0x64D4,0xD4D4,0x64D4,0xAFAF,0xD464,0xD464,0x64D4,0x64D4,0x6464,0xD4D4,0xD4D4,0xC451,0xD4D4,0xD4D4,0xBFBF,0x9F9F,0xAFAF,0x5441,0xD564,0x5474,0x4444,0x7F7F,0x8F8F,0xE464,0xE4E4,0x6464,0x7F7F,0x6464,0x8F8F,0x7F7F,0x7F7F,0x4444,0x4141,0x8F8F,0x5454,0x54E4,0x5454,0x9F9F,0x8F8F,0x4444,0xD464,0x5475,0x5454,0x4454,0x6464,0x64D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xE4D4,0xE4D4,0xE4D4,0xE4C5,0xE4D4,0xC5D4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD5D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4C5,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xC5D4,0xE4E4,0xD4D4,0xD4D4,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4446,0x7575,0x4446,0x8544,0x7585,0x4646,0x4446,0x9575,0x95B4,0x4665,0x7575,0xD495,0x7585,0x9575,0x85C4,0xB485,0xD485,0xB485,0x9585,0x95C4,0x6575,0x9575,0x85C4,0x4585,0x8565,0x7585,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4646,0x4646
.word 0x6464,0x9F9F,0x6454,0xAFAF,0xD564,0xC454,0xD564,0x64D4,0xD4D4,0x64D4,0x5454,0x64D4,0x8F8F,0x4444,0xC4C4,0x54D4,0x64D4,0xD4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xBFBF,0x64D4,0xD5D4,0x8585,0x5454,0xD4D4,0x64D4,0xD4D5,0xD4C4,0x8585,0xC4C4,0xD4D4,0xD4D4,0xD464,0xD4D4,0x4141,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0xAFAF,0xD585,0x5441,0xD4D5,0xE4D4,0x44E4,0x4141,0x8F8F,0x64D4,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x4444,0x4444,0x64E4,0x6464,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4C4,0x5454,0x5454,0x6464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4C5,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xC5E4,0xD5D4,0xE4D4,0xD4E4,0xD4E4,0xE4E5,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x5485,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x8544,0x4675,0x7546,0x75B5,0x4646,0x5F5F,0xB485,0xB485,0x4645,0x9575,0x95C4,0x7575,0xB5C4,0x8575,0x95B4,0xB485,0x7595,0xB495,0xB485,0x8575,0xD495,0xC485,0x6585,0xC495,0x4585,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4644
.word 0x6464,0x8F8F,0x6464,0x7575,0xC4C4,0x6464,0xC4C4,0xC4C4,0xC464,0xD4D4,0x4141,0x64C4,0xD4D4,0x7F7F,0xD464,0xAFAF,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xD5D4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x8F8F,0xD4D4,0xD4D4,0xD464,0xD564,0xD464,0xD4D4,0xD4D4,0xD4D4,0x6464,0x7575,0x5454,0x6454,0x64D5,0x8F8F,0xD564,0x6441,0x8F8F,0x6454,0xD4E4,0xD564,0x41D4,0x7F7F,0xC4C4,0x64D4,0x8F8F,0x8F8F,0x4444,0x7F7F,0x5454,0x54E4,0x9F9F,0xD4D4,0xE4E4,0x8F8F,0xD4C4,0x51D4,0x54D4,0xAFAF,0xD575,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xC5D4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4C5,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xAFAF,0x4644,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x7546,0x5495,0x7546,0x75D4,0x4644,0x7545,0x8595,0x8595,0x6445,0xB585,0x85B4,0x85B4,0xB485,0x8575,0xD485,0x7585,0x7585,0x85C4,0x8585,0x8575,0x85C4,0x85B4,0xB485,0xB495,0x4675,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x464F
.word 0x6464,0x7575,0xC4C4,0x64D5,0x64D4,0x64D5,0xC4D4,0xD564,0xD4D4,0xD4C4,0xD454,0xD4D4,0xD4C4,0x8F8F,0xD4D5,0xD4D4,0xE4D4,0xE4D4,0xD5D4,0xE4D4,0xE4E4,0xE4F4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xD4D4,0x4141,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64C4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0xAFAF,0xD4D4,0x7F7F,0xD4D5,0x4444,0x64C4,0x8F8F,0x6441,0x8F8F,0xE4E4,0x8585,0x7F7F,0x8F8F,0xD464,0x6464,0x4444,0x7F7F,0x4144,0x44E4,0x5441,0x41D4,0xD464,0x4141,0x4141,0xD5D4,0xD4D4,0xC4C4,0xD464,0xBFBF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4C5,0xE4E4,0xD4E4,0xD4E4,0xC5E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4D4,0xD4E4,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4C5,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x4664,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x7546,0x7595,0x8565,0x7595,0x4575,0x8575,0x85B4,0x75B4,0x9575,0xB4B5,0xB495,0x8585,0x8595,0xB485,0x8575,0x85B4,0x8585,0x95C4,0x85D4,0x95B4,0x8585,0xB485,0x8585,0x85B4,0x4644,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xC4C4,0x6464,0x64C4,0xC4C4,0xAFAF,0xD4D4,0xBFBF,0x64D4,0x85D4,0xBFBF,0xD4D4,0xD4D4,0xD4D4,0x44D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4F4,0xE4E5,0xE5E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD485,0xD4C4,0x6464,0x6464,0xD564,0xBFBF,0xD4D4,0xBFBF,0xD564,0xD464,0xD4D4,0x8F8F,0x5454,0xD4D4,0x5454,0xD4D4,0x9F9F,0x8F8F,0xAFAF,0x8F8F,0x4141,0xD454,0x64E4,0x7F7F,0x7F7F,0x8F8F,0xD4D5,0x6464,0x4444,0xD554,0xE4E4,0xAFAF,0x6464,0xD454,0x9F9F,0x4441,0xD4D4,0x64D4,0xD4D4,0xD4D5,0xD4D4,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xD4E4,0xE4D4,0xE4C5,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4F4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x44D4,0x4644,0x4646,0x4646,0x4644,0x4446,0x5F5F,0x4646,0x4F4F,0x4646,0x5446,0x8595,0xB485,0x7585,0x7565,0x9585,0x85C4,0x7585,0xD485,0x8585,0x85B4,0xB485,0xB495,0x7585,0x8585,0xB485,0xB485,0xC485,0xB495,0xB485,0xB495,0x8575,0xC495,0x6585,0x4646,0x5F5F,0x4446,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46
.word 0x6464,0x64C4,0x64D5,0x64D5,0x64D4,0xC4D4,0xC4D4,0xD4D4,0xBFBF,0xD4C4,0xD464,0xD4D4,0xD4D4,0xD464,0xD464,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4F4,0xE5E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD4D4,0x64D4,0x9F9F,0xD464,0xD4D4,0xD4D4,0x6464,0xBFBF,0xAFAF,0x64C4,0x8F8F,0x4141,0x4141,0x7F7F,0x4444,0x8F8F,0x5454,0x5454,0xAFAF,0xAFAF,0xD575,0x6464,0x6485,0x74D5,0xD4D4,0xD4D4,0x7F7F,0x8F8F,0x41D4,0xD4D4,0x9F9F,0xE444,0x4444,0x4444,0x4141,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D5,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4C5,0xE4E4,0xD4E4,0xE4E4,0xC5E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0x51D4,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4646,0x8544,0x85B4,0x8585,0x75B4,0x8565,0xC495,0x7585,0xB485,0xB4B5,0xB495,0x8585,0xC495,0xB495,0x8575,0x85C4,0x8575,0x8585,0xB485,0x7585,0x8585,0x85C4,0xB485,0x8585,0x4685,0x4444,0x7575,0x4665,0x4646,0x4644,0x4644,0x4646,0x5F5F,0x4646,0x4644,0x4F4F,0x4646
.word 0xD564,0xBFBF,0xC4C4,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xD464,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xF4F4,0xE4E4,0xE5F4,0xE4F4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xC4C4,0xD4D4,0xD4C4,0xAFAF,0xD4D4,0xD464,0xD4D4,0xD464,0xD464,0x64D5,0x41D5,0x6454,0x8F8F,0x6454,0xC4D4,0x8F8F,0xC4C4,0xAFAF,0x4444,0x4444,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4141,0x6444,0x5454,0x6464,0xE454,0x4444,0x4141,0xC4C4,0x54D4,0x9F9F,0xD4E4,0xD4D4,0xAFAF,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4C5,0xD4E4,0xD4E4,0xE4D4,0xC5D4,0xD4D4,0xD4E4,0xE4E4,0xC5D4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x54D4,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4446,0x5446,0x4646,0x4646,0xB4B4,0xB495,0x9585,0x7585,0x8575,0x85D4,0x85B4,0x9575,0x85C4,0x9575,0x85C4,0x85B4,0x75B4,0x8575,0xB485,0xB485,0xB485,0x85B5,0x75B4,0xB495,0xB495,0x8585,0x75B4,0x4665,0x7545,0x6595,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xC4C4,0x64C4,0x74D5,0xD564,0xD574,0xD464,0xD4C4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD5D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xF4F4,0xF4E5,0xE4E4,0xD4E4,0xD5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xAFAF,0xD4C4,0xD4C4,0xD485,0xAFAF,0xC4C4,0xD4D4,0x64D4,0xBFBF,0xC4C4,0x6464,0x5454,0xD5D5,0x64D4,0x5454,0xD4D5,0x6464,0x5454,0x4444,0x5441,0x9F9F,0x6441,0x4444,0x7F7F,0x7F7F,0x7F7F,0xD554,0x4141,0xE444,0x7F7F,0x8F8F,0xD4D4,0x6464,0x8F8F,0xD4D4,0x6464,0xD4C4,0xE464,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4D4,0xD5D4,0xE5E4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD474,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0x9F9F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x5446,0x4685,0x4644,0x9565,0x7595,0xB4C4,0x8575,0x8585,0x95B4,0x7585,0xC485,0xB495,0xC485,0xB485,0x8585,0x7595,0x7575,0x8585,0x7585,0x8575,0xB5B4,0x8585,0x85B4,0x8585,0x8575,0x7585,0x6465,0x7595,0x4645,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4644
.word 0xD464,0xC4C4,0xBFBF,0xBFBF,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E5,0xF4D5,0xE4E4,0xE4E4,0xE4D5,0xE4F4,0xE4E4,0xE4E4,0xF4F4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4D5,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x8585,0xC4C4,0xC4C4,0xD4D4,0x51D4,0x5454,0xD4D4,0xC4D4,0xD464,0x6464,0x7F7F,0x5144,0xD5D5,0x8F8F,0x4141,0x7F7F,0x5454,0x8F8F,0xD554,0xD4D4,0xE664,0x4141,0x4141,0x7F7F,0x8F8F,0x7F7F,0x6464,0x8F8F,0xC4D4,0x64E4,0x8F8F,0x4444,0x9F9F,0xD4D4,0x41D4,0xD4D4,0x64D4,0xD4E4,0xD4C5,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xC5E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xD5E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xBFBF,0x54D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4546,0x7585,0x4646,0xB454,0x85B5,0x8585,0x7575,0x95C4,0xB485,0x7575,0x95D4,0x7575,0xB485,0x7595,0xB485,0x7585,0x6554,0x7575,0xB485,0x9575,0x7575,0xC475,0xB495,0x7585,0x95C4,0x65B4,0x8575,0x45C4,0x4446,0x4646,0x5F5F,0x4646,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x4646,0x464F
.word 0x64D4,0x64D5,0x64C4,0x74D5,0xBFBF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4C5,0xE4E4,0xC5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E5,0xF4E4,0xF4E4,0xF4F4,0xE4E5,0xE5E4,0xF4E4,0xF4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E5,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD464,0xD464,0xD4D4,0xD4D4,0xD4D4,0x9F9F,0xC4C4,0xC4D4,0x54D4,0x8F8F,0x6464,0x8F8F,0xD564,0x8F8F,0xD564,0x4164,0xD4D5,0xE4E4,0x4141,0xD444,0x8F8F,0x4444,0x7F7F,0x7F7F,0x8F8F,0x7575,0xAFAF,0x41E4,0x7F7F,0x4444,0x4141,0xE4D4,0x54D4,0xD4D4,0xD4D4,0xD464,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xC5E4,0xD4D4,0xD4C5,0xE4D4,0xC5D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xE4F4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4C5,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x85D4,0xD4D4,0x4646,0x4646,0x4646,0x4644,0x4454,0x4646,0x4446,0x9575,0x7575,0x9575,0xB485,0xB495,0x8585,0xC4B5,0x85B5,0x8585,0x95D4,0x85B4,0x7585,0x7585,0x9585,0x75B4,0x7575,0x8554,0x7575,0xB485,0x6585,0xB495,0x8585,0x9575,0x85D4,0x8575,0xC495,0x4665,0x4646,0x4646,0x4546,0x5475,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646
.word 0xD564,0xD464,0xD4D4,0xD4D4,0xD4C4,0x64D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD5E4,0xF4E4,0xE4E4,0xE5E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xF4E4,0xF4E4,0xF4F4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4C5,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x6464,0xD5D5,0xD564,0xD464,0xD4D4,0xD4D4,0x64D4,0x8F8F,0xD464,0x9F9F,0xBFBF,0x4141,0xD464,0x5454,0xC4C4,0x7F7F,0x6451,0x41D4,0x7F7F,0x8F8F,0x8F8F,0x5454,0x4444,0x4444,0x4444,0x8F8F,0x64E4,0x6464,0x8F8F,0x4444,0x4144,0xD4E4,0xD4D4,0x6464,0xD4D4,0xD4D4,0xD4C4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD5E4,0xE5D4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x54D4,0x4646,0x4644,0x4646,0x4646,0x5446,0x5484,0x4646,0x8575,0x8585,0x9575,0x95B4,0x75C4,0x8575,0xB495,0x85D4,0x85B4,0x95B4,0x85B4,0x85B4,0x75B4,0xB495,0x7585,0x7554,0x5475,0x7585,0x7585,0x8575,0x95C4,0x85B4,0x85B4,0x75B4,0x7585,0x95C4,0x4644,0x4644,0x6565,0x7564,0x4645,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446
.word 0xBFBF,0xD4C4,0x64D4,0xD4D4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E5,0xE4F4,0xE4F4,0xE5E4,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4D4,0xE4F4,0xE4F4,0xE4E5,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFAF,0xC4C4,0xD464,0xD464,0xD4D4,0xD4D4,0xC4D4,0x64D4,0x5454,0x6454,0xD4D5,0x7F7F,0x6464,0x9F9F,0x8F8F,0x64D5,0x8F8F,0xD4D5,0xC4C4,0x6464,0x6464,0x7F7F,0x7F7F,0x7F7F,0x6464,0x5454,0x64D4,0x4444,0xD4D4,0x41D4,0xE4D4,0xE4E4,0xAFAF,0x64D4,0x64D4,0xD4E4,0xD4C5,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xE4D4,0xE4E4,0xD4E4,0xD4D5,0xE4E4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x9F9F,0x4646,0x4646,0x4646,0x4646,0x4446,0x9544,0x8575,0x7546,0x85A5,0x95B4,0x9575,0x85D4,0x8575,0x85C4,0xB485,0x9585,0x85C4,0x8575,0x8575,0x7595,0x85B4,0x7575,0x7575,0x7575,0x7565,0x8575,0x8575,0x7585,0x8575,0x8585,0x7575,0xB575,0x75C4,0x4646,0x4446,0x85B4,0x4665,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646
.word 0xD4C4,0xBFBF,0xD4C4,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE5E4,0xF4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E5,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xD474,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xBFBF,0xBFBF,0x64D4,0xD4D4,0xD4D4,0xD4D4,0x8F8F,0x4141,0x7F7F,0x8F8F,0x5464,0x6464,0xD4D4,0x8F8F,0x6464,0x4141,0x4444,0x5464,0x4444,0x7F7F,0x4141,0x8F8F,0x4444,0xE4D4,0x4141,0xE444,0x41D4,0x4144,0x9F9F,0xD441,0x9F9F,0xD4D4,0xD4C4,0xE4D4,0xD4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xC5E4,0xD4E4,0xD4C5,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4C5,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xD474,0xBFBF,0x54D4,0x4446,0x4646,0x5F5F,0x4644,0x4646,0x4446,0x8595,0x7595,0x8595,0xC485,0xC475,0x7585,0xB485,0xB485,0x8585,0xB585,0xB485,0x8575,0x7575,0x75D4,0x8575,0x7575,0x5475,0x8575,0x8575,0x7585,0xC485,0x7575,0xB485,0xB4B5,0x7585,0x8585,0x4585,0x4446,0x8554,0x6585,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E5,0xE5E4,0xF4E4,0xE5E4,0xF4E4,0xE4E4,0xF4F4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4C5,0x74D4,0xBFBF,0xBFBF,0xAFAF,0xBFBF,0xD485,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD564,0xD564,0xAFAF,0xD564,0xC464,0xD564,0x54C4,0xD541,0x4164,0x5454,0x8F8F,0x8F8F,0x64D5,0x8F8F,0x8F8F,0xD564,0x4451,0xD4D5,0x5454,0x8F8F,0x4444,0x4444,0x4141,0x7F7F,0x6444,0xE4D4,0xD4D4,0x8F8F,0x4444,0x4444,0x5441,0x8F8F,0xD451,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4C5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x7546,0x8575,0xC4B4,0x9575,0x9575,0x85C4,0x7585,0x8585,0x85B4,0x95B4,0x8585,0xB485,0x9595,0x85C4,0x7575,0x9575,0x7575,0x7595,0x7595,0xB485,0x7585,0x7575,0xC485,0x8585,0x7585,0xD4D4,0x4575,0x8564,0x7585,0x4645,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x465F
.word 0x64D4,0xC4C4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD5D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x9F9F,0x8F8F,0x5444,0xD4D4,0xD564,0x64D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xAFAF,0xBFBF,0x64C4,0xBFBF,0x4141,0x8F8F,0xD564,0x9F9F,0x4444,0x8F8F,0x7F7F,0x4444,0xD454,0x8F8F,0x6464,0x4141,0x5441,0x9F9F,0x8F8F,0x7F7F,0x4141,0x8F8F,0x41E5,0x5454,0x4444,0x4141,0x4444,0x8F8F,0x4444,0xD464,0x6464,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4C5,0xD5D4,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xD4E4,0xE4C5,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xC5E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0x5464,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646,0x4446,0x7585,0x9575,0xD475,0xB485,0xB485,0x7585,0xB485,0x8585,0xC485,0xB495,0x8585,0x85C4,0x85B4,0x8585,0xC485,0x7575,0x7575,0x75C4,0x7585,0x7575,0x8585,0x85C4,0xB475,0x8575,0x8585,0x7575,0xC485,0x4575,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646
.word 0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xC5E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xD4E5,0xE4F4,0xE5E4,0xF4E4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0x64D4,0x4151,0x4141,0x4444,0x5454,0x5454,0x4141,0xBFBF,0xBFBF,0xC4C4,0xD4C4,0xAFAF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xC4C4,0xD4D4,0x6485,0x8F8F,0x4444,0x5454,0x4441,0x4141,0x7F7F,0x8F8F,0x6464,0x6464,0x9F9F,0x64C4,0xD4D4,0x8F8F,0xAFAF,0x6464,0x4444,0x8F8F,0x8F8F,0x7F7F,0xAFAF,0xD554,0x54D4,0x8F8F,0x5441,0x8F8F,0xD4C4,0x64D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4C5,0xD4E4,0xE4F4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xD5D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFBF,0xD4C4,0x4545,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x8554,0x8585,0x8585,0x7585,0x8585,0x7575,0x8585,0xB485,0x9585,0x85B5,0x75C4,0x8585,0xB485,0xC495,0x7595,0x8575,0x7575,0x7585,0x8575,0x8575,0x75B4,0x8575,0x8585,0xB485,0xB485,0x9585,0x85D4,0x4665,0x4644,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x5F5F,0x4646,0x465F
.word 0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5E4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E5,0xF4E4,0xE5E4,0xF4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x64D4,0xD464,0x9F9F,0x5454,0x6565,0xAFAF,0x5475,0xAFAF,0x8F8F,0x5454,0x64D5,0x64D5,0xBFBF,0x8585,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64D4,0x5454,0xBFBF,0xD5C4,0x5151,0x5441,0x8F8F,0xC454,0x41D4,0x4141,0xAFAF,0x4444,0x4441,0xAFAF,0x8F8F,0xD454,0x8F8F,0xF664,0x41D4,0x4444,0x7F7F,0x5454,0x4444,0x7F7F,0xD4C4,0x8F8F,0x7F7F,0x7F7F,0x6464,0xD4E4,0xD464,0xD4D4,0xE4D4,0xE4D4,0xD4C5,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xD4D4,0xD5D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xC4D4,0xD4D4,0x4464,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0xB444,0xB485,0xB485,0xB485,0xB485,0x8575,0x85B4,0x95C4,0xC4B4,0xB485,0x8585,0x85B4,0x7575,0x8575,0x6575,0x8575,0x85C5,0x7575,0x7575,0x7585,0x7575,0xB485,0x7595,0xD4B5,0x7585,0xB485,0x6595,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646
.word 0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xD4E4,0xE4D5,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xF4E4,0xE5E4,0xE4E4,0xE4F4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xBFBF,0x9F9F,0x5454,0x7575,0x7575,0xAFAF,0xAFAF,0xAFAF,0x9F9F,0x9F9F,0x9F9F,0xD464,0x64D4,0x64D5,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0x54D4,0xAFAF,0x64D4,0x64D4,0x64D5,0x8F8F,0xC4C4,0x9F9F,0xD4D4,0x7F7F,0x6444,0x8F8F,0xE4D4,0x6464,0x8F8F,0xD464,0x5454,0x5454,0xE4E4,0xD4E4,0x5441,0xD464,0xC4E4,0x6464,0x5151,0x9F9F,0xD4E4,0x7F7F,0x4444,0x4141,0x6464,0xD485,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xC5D4,0xD4E4,0xD4E4,0xE4E4,0xE4D5,0xE4E5,0xE4D4,0xD5D4,0xD4E4,0xE4D4,0xD4E4,0x64E4,0xD4E4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xBFBF,0x4564,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7546,0x75B5,0x8585,0x7585,0x8575,0x7585,0x8575,0x95B4,0x8575,0x8585,0x95C4,0xB485,0x7585,0x6575,0x7564,0x8585,0x7575,0x85D4,0x7575,0x7575,0x8585,0xC495,0x8575,0x9585,0x75B4,0x9585,0x4675,0x4644,0x5444,0x4654,0x4644,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4446
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4F4,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD474,0xAFAF,0xAFAF,0xD564,0xD564,0xD564,0xAFAF,0xAFAF,0x5475,0x5454,0x9F9F,0x9F9F,0x4444,0xD464,0xC4C4,0xD464,0xD564,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x6454,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x8F8F,0x9F9F,0x7F7F,0x8F8F,0xD554,0xD4D4,0xC4C4,0x7F7F,0xAFAF,0x4444,0x9F9F,0x8F8F,0x8F8F,0x4444,0xD541,0xAFAF,0x9F9F,0x9F9F,0xD444,0x7F7F,0x6464,0x4444,0x4141,0x8F8F,0x4444,0x6464,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4C5,0xE4D4,0xE4E4,0xE4E4,0xF4D4,0xE4E4,0xF4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D5,0xE4D4,0xD4C5,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xC4D4,0xAFAF,0x4646,0x5F5F,0x4646,0x4644,0x4446,0x4446,0x4646,0x5F5F,0x75D4,0xB485,0x8585,0xB485,0x8585,0x8575,0xC485,0x7585,0x95C4,0x85C4,0x8585,0x85B4,0x5475,0x7575,0x8575,0x75C5,0x7585,0x7585,0x8575,0x85C4,0x75B4,0xB485,0x85B5,0x8575,0xB4C4,0x4665,0x5444,0x6585,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xD5D4,0xD4E4,0xE4C5,0xD4D4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4D5,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xC5E4,0xD4D4,0xD4D4,0xAFAF,0x64C4,0xC4C4,0xAFAF,0xBFBF,0x6464,0x64C4,0x7575,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0xD564,0xD4D4,0x64C4,0x74D4,0xD464,0x74D4,0xD4D4,0xD4D4,0x54D4,0x5454,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x8F8F,0x6454,0x54D4,0x8F8F,0x8F8F,0x5454,0xC454,0xD4D4,0x8F8F,0x4141,0x8F8F,0x7F7F,0x6464,0x64E5,0x5454,0x5454,0x64D5,0x4444,0x7F7F,0x7F7F,0x4141,0x8F8F,0x6464,0xD4D4,0xC4D4,0x6464,0xC5C4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD5E4,0xF4E4,0xE4E4,0xF4E4,0xE4D4,0xE4D4,0xE4D4,0xD4C5,0x64D4,0xAFAF,0xF4D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xC5D4,0xE4D4,0xD4D4,0x64D4,0x4664,0x4646,0x4646,0x4444,0x4646,0x4646,0x4646,0x4646,0x5446,0x8585,0x85C4,0x75B4,0x7585,0xB485,0x7575,0xB485,0x7585,0x7575,0xC485,0xB495,0x7575,0x5475,0x8575,0x85C4,0x7585,0x7585,0x7575,0x8585,0x85B4,0x8585,0xB5D4,0x7585,0xC485,0x4585,0x6465,0x8595,0x4645,0x4644,0x5F5F,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x5F5F,0x465F
.word 0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD5D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x7474,0x64C4,0xD5D5,0x8585,0x74C4,0xD564,0x6464,0xAFAF,0xAFAF,0x7575,0x5454,0x5454,0x5454,0x8F8F,0x9F9F,0x8F8F,0xC485,0xC4C4,0xD5D5,0xAFAF,0xD4D4,0xD4D4,0xD4D4,0x51D4,0x9F9F,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0x8F8F,0x5454,0x7F7F,0x8F8F,0x7F7F,0x5454,0x64D4,0x4444,0xD554,0x5454,0x8F8F,0xBFBF,0x9F9F,0x4444,0x5454,0xAFAF,0x6464,0x4141,0xE444,0x7F7F,0x8F8F,0xD4D4,0x4444,0xBFBF,0x64D4,0xD464,0xE4D4,0xC5D4,0xD4D4,0xC5D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4C5,0xD4E4,0xD4C5,0xD4E4,0x41D4,0xE464,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6546,0x9585,0x8575,0x8585,0x8575,0x8585,0x6585,0x8585,0xB495,0x8585,0x8565,0x8585,0x5475,0x5475,0x8585,0x8575,0x7575,0xB485,0x8575,0x8575,0xB485,0x9575,0xB4C4,0xB485,0xB495,0x7575,0x8585,0x45B4,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4C5,0xD4D4,0x64D4,0x64D5,0x64D5,0xD4D4,0xD5D4,0xBFBF,0xAFAF,0x6464,0xAFAF,0x5475,0x5454,0x9F9F,0x9F9F,0x4444,0x4454,0x9F9F,0x9F9F,0xD4C4,0xD464,0xD464,0x6464,0xD4C4,0xD4D4,0x9F9F,0x5454,0xE464,0xD4D4,0xC5D4,0xD4D4,0x51D4,0x4144,0x6475,0x9F9F,0x8F8F,0xAFAF,0xD4C4,0x4141,0x8F8F,0x4141,0xC4C4,0x6464,0x64D4,0x64D5,0xD4D5,0x64D4,0x6464,0x5454,0x8F8F,0x6444,0xD4D5,0x64D4,0xAFAF,0xE4D4,0x4141,0xD454,0xD454,0xBFBF,0xD464,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D5,0xD4E4,0xD4D5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xAFAF,0xD4D4,0xE4E4,0xD5E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0xB475,0xB485,0x7595,0x7585,0xC475,0x8595,0x7575,0x8585,0x75B4,0x7575,0xB485,0x7585,0x7554,0x8575,0x7565,0x7575,0x7595,0x8585,0x75B4,0x8575,0xD4D4,0x8585,0x95D4,0x75B4,0x9585,0x75C4,0x4545,0x6454,0x4644,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F5F,0x4446,0x5F46
.word 0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xD5E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xC5E4,0xD4E4,0xD4D4,0xD4D4,0xD564,0xD4D4,0x8585,0xD4D4,0x64D4,0x85D4,0xD4D4,0xAFAF,0x64D5,0x7575,0x7575,0x9F9F,0x9F9F,0x5454,0x9F9F,0x9F9F,0x5454,0x9F9F,0xBFBF,0x64D5,0x85D5,0xAFAF,0xD4D4,0x54D4,0x9F9F,0xD454,0xD4D4,0xD4D4,0x64D4,0x6464,0xAFAF,0x8F8F,0x8F8F,0x5454,0x5444,0x6464,0x7F7F,0xD541,0x6464,0x7F7F,0x9F9F,0x5454,0xC4C4,0x4141,0xE564,0xAFAF,0x5475,0x4444,0xD4D4,0x64D4,0x8F8F,0xD4D4,0x7F7F,0x8F8F,0x54E4,0xD4D4,0xD5D4,0x6464,0xD4D5,0xD4E4,0xD4D4,0xE4D4,0xF4C5,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0x64D4,0x9F9F,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0x64D4,0xAFAF,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x9575,0x85B4,0x85B4,0x85B4,0x8575,0x95B4,0x75C4,0x8575,0x9585,0x8575,0x5475,0x7575,0x8575,0x8575,0x8575,0x7585,0x85C4,0x75B4,0x7575,0x8585,0x8585,0xA5B4,0xC495,0x8585,0xC485,0x6585,0x8585,0x4575,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646
.word 0xD4D4,0xD4D4,0xD4C5,0xE4C5,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4D4,0xD5D4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xBFBF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFBF,0xD4C4,0x6464,0xD564,0xAFAF,0x9F9F,0x9F9F,0x5454,0x4444,0x5441,0x9F9F,0x9F9F,0x5454,0x5454,0xC4C4,0xC4C4,0xBFBF,0xD464,0xD4D4,0x5454,0x9F9F,0xD4D4,0xD4D4,0x8F8F,0x6464,0xD575,0x5151,0x4444,0xD4C4,0xD4D4,0xE4D4,0xE4E4,0x8F8F,0xD4D4,0x4141,0x9F9F,0x6464,0x4444,0x7F7F,0x7F7F,0xD464,0x8F8F,0x6441,0x5454,0xAFAF,0xD444,0xE464,0x6464,0xE454,0xD4D4,0xD4D4,0xD441,0x7F7F,0x7F7F,0xD4D4,0xE4E4,0xD4D4,0xE4D4,0xE4E5,0xF4E4,0xE4E5,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0x64D4,0x64D5,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xC5D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x64D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7544,0x9585,0x7585,0x8585,0x75B4,0x8585,0x85D4,0x7575,0x7575,0xB485,0x7595,0x5475,0x7575,0xC475,0x7585,0x6575,0x7585,0x7575,0x8575,0x95B4,0x9575,0x85C4,0x85C4,0x95B4,0x75B4,0xD485,0x6575,0x4646,0x4644,0x4646,0x5F5F,0x5F5F,0x4644,0x4646,0x4646,0x4646,0x4446
.word 0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD5D4,0xE4D4,0xE4E4,0xE4D4,0xD5E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4E4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4D4,0xD4C4,0x6464,0x6475,0xAFAF,0x5454,0x5454,0x9F9F,0x9F9F,0x5454,0x5454,0x5454,0x5454,0x6441,0x64D5,0x64D4,0xD4D4,0x64C4,0x5454,0x5454,0xD4D4,0xD4D4,0x64D4,0x6565,0x9F9F,0x8F8F,0x4441,0x5454,0x54E4,0x8F8F,0xD454,0xD4E4,0xD4E4,0x8F8F,0x6464,0x5454,0x8F8F,0xD464,0xD4D4,0xD4D4,0x8F8F,0x8F8F,0x8F8F,0x4444,0xD4D4,0xAFAF,0xE4D4,0xD4D4,0x8F8F,0xD454,0xD4D4,0xAFAF,0x8F8F,0xD464,0xC5D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0x74D4,0xAFAF,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x74D4,0xD4E5,0xD4D4,0xD4C5,0xE4D4,0xD4D4,0xAFAF,0x4664,0x4446,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4544,0x85C4,0x95B4,0x75C4,0xB495,0x7575,0xC485,0x6585,0x7575,0x7585,0x7575,0x7554,0x8585,0x7585,0xC485,0x5485,0x7585,0x7575,0x8585,0xB485,0xD495,0xC495,0x8585,0xB485,0x8585,0x8595,0x4665,0x4644,0x5F5F,0x5F5F,0x4646,0x4646,0x4F4F,0x5F5F,0x4646,0x4644,0x4646
.word 0xD4D4,0xC5D4,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xC5E4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64D5,0x7575,0xAFAF,0xAFAF,0x5454,0x9F9F,0x9F9F,0x9F9F,0x5454,0xAFAF,0x5454,0x9F9F,0x5454,0xAFAF,0xD464,0xD4D4,0x5454,0x9F9F,0x6454,0xD4D4,0x54D4,0x6464,0x6454,0x8F8F,0x8F8F,0xD4D4,0x8F8F,0x7F7F,0x7F7F,0x5441,0xE4E4,0xE4D4,0xD4E4,0x9F9F,0xC4C4,0xD4D4,0x6464,0x7F7F,0xD4D4,0x7F7F,0x7F7F,0x5441,0x8F8F,0xD444,0x6464,0x6444,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x6464,0xC4C4,0xD4E4,0xE4E4,0xE4F4,0xE5E4,0xE4D4,0xE4E4,0xD4C5,0xD4D4,0x64D4,0x64D5,0xD4D5,0xE4D4,0xF4E5,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0x74D4,0xC4D4,0xAFAF,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4444,0x4646,0x9554,0x95B4,0x9575,0x8575,0x7585,0x8575,0x8585,0x7585,0x7575,0x7585,0x5475,0x7585,0x85B4,0x7585,0x8575,0x75B4,0x9585,0x75B4,0x8585,0x85B4,0x85B4,0x85B4,0x8575,0xB485,0x65C4,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4444,0x4646,0x4644
.word 0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xD5D4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xBFBF,0xAFAF,0x64C4,0x7575,0xAFAF,0x5454,0x5454,0x9F9F,0x5454,0xAFAF,0x5454,0x9F9F,0x8F8F,0x6441,0x64D5,0xBFBF,0x44D4,0x9F9F,0x9F9F,0xD4D4,0xD4D4,0x5464,0xD5D5,0x4141,0xD444,0x5454,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0x7F7F,0xE454,0xD4D4,0x4441,0x6464,0xD4D4,0xE444,0x8F8F,0x4444,0xE444,0xD4D4,0x8F8F,0x7F7F,0x7F7F,0xC441,0xAFAF,0x7F7F,0x4141,0x4441,0x6441,0x4141,0x4444,0x6475,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x5454,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4664,0x5F5F,0x4646,0x4646,0x5F5F,0x5F5F,0x4644,0x4646,0x4646,0x4446,0x8544,0xB485,0x7595,0x75C4,0x8585,0x7575,0x7575,0x7585,0x8575,0x7575,0x7575,0xD475,0x8585,0x7565,0x8554,0x8575,0x7575,0x7585,0x85C4,0x8585,0x8595,0xB485,0x7585,0x8595,0x4675,0x4644,0x5F5F,0x4646,0x4646,0x4F4F,0x5F5F,0x4644,0x5F5F,0x4646,0x4F4F,0x4646
.word 0xD4D4,0xD4E4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE5E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4C5,0xE4D4,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x74D4,0xD4D4,0xD4D4,0xD564,0x64D4,0xAFAF,0xAFAF,0x9F9F,0x5454,0x9F9F,0x5454,0x5454,0x5454,0x9F9F,0x5454,0x9F9F,0x8F8F,0x5454,0x64D5,0xD4D4,0x5441,0x9F9F,0x5454,0x64D4,0x6464,0x9F9F,0x6464,0xD4D4,0x8F8F,0x8F8F,0x4444,0x4141,0x5444,0x5454,0xC4C4,0xD4D4,0x41E4,0xE564,0xD4E4,0xC4D4,0x5464,0x8F8F,0x8F8F,0x64E4,0x4141,0x4444,0x6441,0x7F7F,0xD444,0x6464,0x4444,0x8F8F,0xAFAF,0xD544,0xC4C4,0x8F8F,0x6475,0xE464,0xE4E4,0xD5D4,0xD4E4,0xE4E4,0xE4D4,0x64D4,0xC4D4,0x6441,0xD4D4,0xE4E4,0xE4E5,0xE4E4,0xD4D5,0xD4E4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4C5,0xD4D4,0x6464,0xAFAF,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4446,0xC485,0x8575,0x7575,0xC495,0x7585,0xB485,0x7585,0x75C4,0x7565,0x8554,0x7575,0x7585,0x6585,0xB485,0x8575,0x8575,0x75B4,0xC485,0xB495,0x85C4,0x75B4,0xB495,0x7585,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46
.word 0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xD4E4,0xE4F4,0xE4E4,0xE4E4,0xD5D4,0xE4D4,0xC5E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xC4C4,0xAFAF,0x7575,0x5475,0xAFAF,0x9F9F,0x5454,0x7575,0xAFAF,0x5454,0x9F9F,0x4444,0x4444,0x4444,0x4141,0xD4C4,0xAFAF,0x9F9F,0x9F9F,0xD4D4,0x9F9F,0x5454,0x4454,0xE541,0x9F9F,0x5454,0x9F9F,0x9F9F,0x6464,0xD4D5,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xC4D4,0x6464,0x9F9F,0x4444,0x6464,0xE4E4,0x7F7F,0xD4D4,0xD4E4,0xD4E4,0x6464,0x41D4,0x4154,0x8F8F,0x54D4,0x7F7F,0x7F7F,0x7F7F,0xAFAF,0x64C4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD564,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD464,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0x4664,0x4644,0x4646,0x5F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x8565,0x7595,0xB485,0x8575,0x85B4,0x85C4,0x8585,0x7585,0x6564,0x8585,0x75C4,0x7585,0x7575,0x7575,0x7585,0x7575,0x9585,0xB5C4,0x75B4,0x8575,0x8585,0x8575,0x4575,0x5F5F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4F4F,0x5F5F,0x4446,0x4646
.word 0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD5E4,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xE4D5,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D5,0xD4E4,0xD4C5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0x6485,0xAFAF,0xAFAF,0x5454,0x9F9F,0x5454,0x5454,0xAFAF,0x9F9F,0x8F8F,0x4444,0x8F8F,0x7F7F,0x6F6F,0x7F7F,0xD4D4,0x9F9F,0x4444,0x9F9F,0x5454,0x4444,0x9F9F,0x5454,0x6464,0xD4E5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD485,0x64D4,0x9F9F,0x5454,0x5454,0x4444,0x4141,0x6444,0x8F8F,0x8F8F,0x4141,0x4141,0x8F8F,0x6464,0xD4D4,0x8F8F,0xD4D4,0x8F8F,0xD4D4,0xD441,0xC441,0x6441,0x6464,0x74D5,0xE4E4,0xC5D4,0xD4E4,0xC5D4,0x64D4,0x9F9F,0xD564,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD5D4,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4446,0x4646,0x4644,0x4F4F,0x4646,0x4446,0x8565,0x64B4,0x7575,0x9585,0x85C4,0x7575,0x6575,0x8554,0x85D4,0x7575,0x7565,0x8554,0x7575,0x8575,0x8585,0x85C4,0x8575,0xC495,0x8575,0x95B4,0x75B4,0x7554,0x7575,0x6475,0x4454,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46
.word 0xD4D4,0xC5D4,0xE4D4,0xC5D4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4C5,0xD4C5,0xD4E4,0xE4D4,0xD4D4,0x64D4,0xC4D4,0xAFAF,0xD564,0xAFAF,0x7575,0x6454,0x9F9F,0x9F9F,0x5454,0x5454,0x4444,0x4154,0x8F8F,0x4444,0x7F7F,0x7F7F,0x6444,0x54D5,0x8F8F,0x8F8F,0x5454,0x5454,0x4444,0x5454,0xD4D4,0x6475,0xD564,0xD4D4,0xD464,0x6464,0x6464,0x5454,0x9F9F,0x5454,0x8F8F,0x4141,0x4444,0x5441,0x54F5,0x9F9F,0x6464,0x6464,0x6464,0xAFAF,0x5454,0xC4C4,0xE4D4,0xD4E4,0x41E4,0x64D4,0x7F7F,0x8F8F,0xC4C4,0x64D5,0x6464,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x5454,0xAFAF,0xD4D4,0xE4E4,0xD5E4,0xE4E4,0xD4E4,0xD4E4,0xE4C5,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x6464,0x4685,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6544,0x8575,0x7575,0x7585,0xB485,0x6585,0x5475,0x7595,0x7575,0x8575,0x7585,0x8575,0x7575,0x7585,0x85B4,0x8575,0x85C4,0x85B4,0x9585,0x75B4,0x7585,0x8575,0x4565,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x5F5F,0x4646
.word 0xC5D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4C5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0xC4C4,0xD4D4,0x6464,0x6464,0x7575,0xAFAF,0x9F9F,0x5454,0x5454,0x5454,0x5454,0x8F8F,0x8F8F,0x7F7F,0x6F6F,0x6F6F,0x4646,0x6444,0x64D4,0x8F8F,0x8F8F,0x5454,0x9F9F,0xAFAF,0x8F8F,0x9F9F,0x4141,0x8F8F,0x9F9F,0x9F9F,0x4444,0x8F8F,0x5454,0x8F8F,0x8F8F,0x4444,0x4444,0x8F8F,0x9F9F,0x4444,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x5451,0xE4E4,0xD4D4,0x8F8F,0xD4D4,0xD4D4,0xBFBF,0x8F8F,0x4444,0x6454,0xE4D4,0xD4E4,0xD4D4,0xD464,0xD564,0xD464,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0x64D4,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x5F5F,0x4646,0x4446,0x4446,0x6544,0x7575,0x7585,0x7575,0x5475,0x7575,0xB485,0x8575,0x7585,0x7575,0x8575,0xD485,0xB5B5,0xB485,0x7585,0x8575,0x8585,0x85B4,0x8575,0x8585,0x6565,0x4644,0x4446,0x5F5F,0x5F5F,0x4646,0x4446,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646
.word 0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xC5E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4F4,0xE4E5,0xE4E4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xC5D4,0xE4D4,0xD5D4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xC4C4,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x5454,0x5454,0xAFAF,0x5454,0x5454,0x8F8F,0x4444,0x7F7F,0x7F7F,0x6F6F,0x4646,0x4646,0xC4C4,0x54D4,0x5454,0xC4C4,0xD4C4,0xD4C4,0x8F8F,0x4454,0x4444,0x4444,0x4141,0x4444,0x5454,0x8F8F,0x8F8F,0x8F8F,0x4444,0x4141,0x4444,0xAFAF,0x54E4,0x8F8F,0x4444,0x4441,0x4444,0xAFAF,0x8F8F,0x4141,0x8F8F,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7F7F,0xD4C4,0xD4D4,0x6464,0xD4D4,0xD4D4,0xAFAF,0x6464,0xC4C4,0xD4D4,0xD5E4,0xF4E4,0xD5E4,0xD4E4,0xD4E4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD474,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0x4664,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x5F5F,0x7546,0x4665,0x4646,0x7544,0x7585,0x7554,0x7554,0x5475,0x9575,0x6575,0x8575,0x8565,0x8575,0x95D4,0xC4C4,0x8585,0x8575,0xC485,0x7585,0x7575,0xB485,0x5475,0x4645,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xD4D4,0xD4D4,0xE4E4,0xE4C5,0xE4E4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4C5,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0x64C4,0xD564,0x5454,0xAFAF,0x6454,0x5454,0x5454,0x5454,0x8F8F,0x5454,0x4444,0x7F7F,0x4444,0x4646,0x6F6F,0x4646,0x4646,0xD541,0xC4D4,0x4141,0x9F9F,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0xC4C4,0x4141,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4141,0x8F8F,0x8F8F,0xE454,0x41D4,0x41D5,0x7F7F,0x8F8F,0x4444,0x4144,0xE4E4,0xD4D4,0xC4C4,0x8585,0xD4D4,0xE464,0xE4E4,0xD4E4,0xD4D4,0xD485,0xD4D4,0xD564,0xAFAF,0xD485,0xD4E4,0xD4C4,0xD541,0xD464,0xD4D4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4C5,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x46C4,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4646,0x4646,0x6446,0x7585,0x4644,0x8545,0x7575,0x7565,0x7554,0x8575,0x7575,0xC585,0x7585,0x7575,0x7575,0x8565,0x8585,0x95B4,0x85C4,0x75B4,0x8585,0x85C4,0x6565,0x4644,0x4646,0x4644,0x4646,0x4446,0x5F5F,0x4646,0x4444,0x4646,0x5F5F,0x4446,0x4646,0x4644,0x4646
.word 0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE5E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4D4,0xC5E4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xC4C4,0xD5D5,0xAFAF,0xAFAF,0x6565,0x5454,0xAFAF,0x6464,0x5454,0x4444,0x8F8F,0x4141,0x7F7F,0x6F6F,0x4644,0x4646,0x4646,0x4646,0x4646,0xC454,0xC4D4,0x64D4,0x64D4,0xD4D4,0x5441,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x4444,0x4441,0x4444,0x4441,0x8F8F,0x4141,0x4141,0xE454,0x6464,0x7F7F,0x4144,0x8F8F,0x7F7F,0x6464,0xE4E4,0xE4E4,0x8F8F,0x8F8F,0x8F8F,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD441,0x74D4,0xD564,0xAFAF,0x6464,0xAFAF,0x6464,0xD4D5,0xE4D4,0xE4E4,0xE4E4,0xE4E5,0xD5E4,0xD4E4,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD474,0xD4D4,0xE4D4,0xD4D4,0x74D4,0x64D5,0xAFAF,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x7544,0x8575,0x8585,0x7585,0x5475,0x5454,0x8585,0x7565,0x7575,0x8575,0x6585,0x7585,0x7575,0x7585,0xB485,0x8585,0x8575,0x8595,0x95B4,0x5475,0x4645,0x4644,0x4646,0x5F5F,0x4446,0x4F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F4F,0x4646
.word 0xD4E4,0xC5E4,0xD4E4,0xE4E4,0xD5E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4C5,0xD4E4,0xD4C5,0xD4D4,0xD4E4,0xE4C5,0xE4E4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xC5D4,0xE4D4,0xC5D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD464,0xD564,0xAFAF,0x9F9F,0x7575,0x5454,0x5454,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x6F6F,0x8F8F,0xD541,0x4141,0x5454,0xD4D4,0x54D4,0x4444,0x7F7F,0x4444,0x7F7F,0x8F8F,0x4141,0x4444,0x4141,0x8F8F,0xD554,0xD4E4,0x9F9F,0x8F8F,0x4141,0x8F8F,0x4444,0xE454,0xE4E4,0xE4E4,0xE4E4,0x64D4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0x41E4,0xE4D4,0xC4C4,0x7F7F,0xD4D4,0xD564,0x6464,0x6454,0x64D5,0xD4D4,0xD4D4,0xD5E4,0xF4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0x64D4,0xD4E5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xBFBF,0x4675,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4546,0xB495,0x8575,0x8575,0x7565,0x7575,0x8575,0x5475,0x7585,0x5475,0x8575,0x7575,0x8575,0x7585,0x8585,0x85D4,0x95C4,0x85C4,0x6575,0x4644,0x4646,0x5F5F,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F
.word 0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D5,0xD5E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE5D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0x6464,0x6475,0x5454,0x5454,0x5454,0x9F9F,0x8F8F,0x4144,0x4444,0x4441,0x7F7F,0x7F7F,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5441,0x64D4,0x9F9F,0xAFAF,0xD4D4,0x9F9F,0x4444,0x8F8F,0x4141,0x8F8F,0x8F8F,0x8F8F,0x4441,0x5454,0xE4D4,0x5464,0x4141,0xC4C4,0x4444,0x8F8F,0x4441,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xC5E4,0xE4E4,0xD585,0xE4D4,0x54D4,0x64D4,0xD4D4,0xD4D4,0x4444,0x8F8F,0x5454,0xD4C4,0xE474,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0x64D4,0x9F9F,0x4446,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4F4F,0x4646,0x4475,0x4646,0x4644,0x9575,0x7585,0x7575,0x5454,0x8575,0x7575,0x8575,0x7575,0x7585,0x7565,0x8585,0xB485,0xC485,0xB485,0x8595,0x75B4,0x6575,0x4645,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x5F5F,0x4646,0x4446,0x4646
.word 0xC5D4,0xD4E4,0xE4E4,0xD4D5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D5,0xE4E5,0xE4E4,0xE4E4,0xE4D4,0xC5D4,0xE4D4,0xD4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xE4C5,0xE4E4,0xE4D5,0xE4E4,0xD5D4,0xD4E4,0xD4E4,0xE4E4,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFBF,0xC4C4,0xBFBF,0xAFAF,0x6464,0x6565,0x9F9F,0x5454,0x9F9F,0x4141,0x7F7F,0x6F6F,0x4444,0x4644,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x5144,0x5454,0x7F7F,0xC4D5,0x64D4,0x4141,0x9F9F,0x64D4,0x4444,0x8F8F,0x4141,0x4144,0xAFAF,0xE4D4,0x8F8F,0x64D4,0x8F8F,0x64D4,0x8F8F,0x4444,0x4444,0x8F8F,0xF4E4,0xE5E4,0xE4E4,0xD4E4,0x64D4,0xE4D4,0xE4E4,0xE4D4,0x41E4,0xE4D4,0xD4E4,0xE4E4,0xC4C4,0x9F9F,0xD4D4,0xD4D4,0x7F7F,0x4441,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0x64D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x9585,0x4644,0x4646,0x7544,0x85C4,0x75C4,0x8575,0x7565,0x85D4,0x6575,0x8575,0x75D4,0x7575,0x7575,0x5465,0x6575,0x4465,0x4646,0x4644,0x4446,0x4446,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646
.word 0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE5E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xC5E4,0xE4D4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4E4,0xE4C5,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xD5D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC4D4,0xC4C4,0x6464,0xAFAF,0x6565,0x5475,0x5454,0x4444,0x4444,0x8F8F,0x4441,0x7F7F,0x7F7F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4546,0x5454,0x9F9F,0x4144,0x4141,0xD4C4,0xC4D4,0x5454,0xD464,0xD4D4,0xD4D4,0x9F9F,0xD564,0xE4E4,0x54D4,0x5454,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x4141,0x5454,0xE464,0xE4F4,0xE4F4,0xD4E4,0xD4E4,0xD464,0xD4D4,0xE4E4,0xE4E4,0xD444,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x8F8F,0x4141,0x7F7F,0xD441,0xD4D4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFBF,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4444,0x4646,0x4646,0xB485,0x4475,0x4646,0x4446,0x9585,0x5485,0x6465,0x7554,0x7585,0x9595,0x7575,0x8585,0x6585,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4444,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xE4D4,0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E5,0xE4E4,0xD4E4,0xE4D5,0xD4D4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xF4D4,0xD4E4,0xD4F4,0xD4E4,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD485,0x64D5,0xAFAF,0x6475,0xAFAF,0x5454,0x9F9F,0x4444,0x8F8F,0x7F7F,0x7F7F,0x6F6F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4144,0x9F9F,0x4144,0x4444,0x8F8F,0x8F8F,0xAFAF,0x64D4,0xD4D5,0x51D4,0xAFAF,0xD4D4,0xD4D4,0x5151,0x4141,0x8F8F,0x4444,0x7F7F,0x4444,0x8F8F,0x5444,0xAFAF,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xBFBF,0xD4D4,0x41D4,0xD4E4,0xE441,0xD441,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0x4444,0x8F8F,0x4441,0x4141,0xE454,0xF4E4,0xE4F4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xE4D4,0x74D4,0xD564,0x4664,0x4446,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0xB565,0x8585,0x8585,0x8575,0xB485,0x7585,0x7575,0x8585,0x8565,0x7565,0x8575,0x85B4,0x65B4,0x4646,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646
.word 0xD4E4,0xD4E4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xD4D4,0xE4D5,0xF4E4,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E5,0xE4F4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xC4C4,0xAFAF,0x7575,0xAFAF,0x9F9F,0x5454,0x4154,0x4444,0x4141,0x7F7F,0x6F6F,0x5F5F,0x5F5F,0x4646,0x4646,0x4646,0x5441,0x9F9F,0x8F8F,0x4141,0x4444,0x7F7F,0x6454,0xD4D4,0x8585,0xBFBF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5454,0x5454,0x51D4,0x7F7F,0x7F7F,0x4444,0x9F9F,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xE474,0x7F7F,0x64D4,0x7F7F,0x7F7F,0x7F7F,0xD441,0xE4D4,0x8585,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x9F9F,0xE4D4,0xE4E4,0xE4D5,0xD5D4,0xD4E4,0xC5E4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD5D4,0xAFAF,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x7446,0x85B4,0xB585,0x94D4,0x95D4,0x75B4,0x5475,0x7565,0x8585,0x7575,0x7554,0x9575,0x6475,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x5F5F,0x4646,0x4646,0x4446,0x4646,0x4644
.word 0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D5,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xC5D4,0xD4E4,0xE4D5,0xE4E4,0xF4E4,0xF4E4,0xE5E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4C4,0x6464,0x64D5,0x6464,0x7575,0x5454,0x9F9F,0x9F9F,0x4441,0x7F7F,0x4141,0x4646,0x4646,0x5F5F,0x4646,0x4146,0x9F9F,0x9F9F,0x5454,0x5454,0x8F8F,0x4444,0x8F8F,0x9F9F,0xD4D4,0x6464,0x9F9F,0x6441,0xBFBF,0xD4D4,0xD454,0x8F8F,0xD4D4,0xD4D4,0x4154,0x5454,0xD4D4,0xE4E4,0xE4E4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0x8F8F,0x5464,0x6464,0x8F8F,0x5444,0xC4D4,0x8F8F,0x5451,0x7F7F,0x6444,0xAFAF,0x5454,0xAFAF,0x5444,0x6464,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xBFBF,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0x64D4,0x64D4,0x4654,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4446,0x4446,0x95C4,0x85B4,0xD4B4,0xD495,0x8575,0xD485,0x7575,0x6585,0x8575,0x7575,0x8585,0x7575,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x4646
.word 0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xE4E4,0xD4E4,0xD4C5,0xD4C5,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xF4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xC4C4,0xAFAF,0xD564,0x5454,0x6565,0x9F9F,0x5454,0x9F9F,0x8F8F,0x6F6F,0x4141,0x4646,0x4446,0x4646,0x4446,0x4144,0x8F8F,0x5454,0x9F9F,0x9F9F,0x4444,0x4441,0x4441,0x4444,0xC4C4,0xD4D5,0xD564,0xD5D4,0x64D4,0x6464,0x64D5,0x8F8F,0xE4C4,0x54D4,0x8F8F,0xAFAF,0xE4D4,0xD4E4,0xE4E4,0x54D4,0x9F9F,0x4141,0xD464,0xE4E4,0x4441,0x4141,0xD564,0x8F8F,0x5454,0x8F8F,0xD4D4,0x8F8F,0x8F8F,0x7F7F,0x4141,0x8F8F,0xBFBF,0x5454,0x6464,0x5454,0xD454,0xE4E4,0xD4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6546,0xC495,0x9595,0xB5D4,0x5465,0x8575,0x7575,0x6595,0x5475,0x5475,0x7575,0x7575,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4C5,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0x64D5,0xBFBF,0x7575,0x5454,0x5454,0x5454,0x9F9F,0x8F8F,0x7F7F,0x6F6F,0x4646,0x5F5F,0x5F5F,0x4446,0x9F9F,0x9F9F,0x9F9F,0x5454,0x8F8F,0x4154,0x9F9F,0x8F8F,0x8F8F,0x4444,0xD464,0xD4D4,0x74C4,0x8F8F,0xD4D4,0x8F8F,0x8F8F,0x4441,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xC5E4,0xE4D4,0xE4D4,0xD4D4,0xD454,0x8F8F,0x6454,0xD5D5,0xD564,0x9F9F,0x4141,0x5454,0xAFAF,0x7F7F,0xD454,0x8F8F,0x4144,0x4444,0x8F8F,0x8F8F,0x6444,0x6464,0x9F9F,0x6464,0xE444,0xE4E4,0xD4D4,0xD4E4,0xC5D4,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD574,0xD564,0x4646,0x4644,0x4446,0x4646,0x4644,0x5F5F,0x4444,0x5F5F,0x4646,0x4646,0x7574,0x4446,0x8575,0xD4C4,0x5475,0x7575,0x7554,0x7575,0x8575,0x8575,0x85C4,0x75D4,0x4485,0x4446,0x5F5F,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x4646
.word 0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4C5,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4D5,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD564,0x6464,0x6464,0x9F9F,0x9F9F,0x5454,0x7F7F,0x6F6F,0x4141,0x4446,0x4646,0x4646,0x5441,0x9F9F,0x5454,0x5454,0x9F9F,0x9F9F,0x8F8F,0x9F9F,0x4444,0x4141,0x4444,0x4141,0x64C4,0x4444,0x6464,0x5454,0x4141,0x5444,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0x7F7F,0x7F7F,0x5454,0x8F8F,0x4141,0x8F8F,0xD4D4,0xD4D4,0xD4E4,0x5454,0x4444,0x4444,0x6444,0x5454,0xAFAF,0x8F8F,0xD564,0x8F8F,0x8F8F,0x4444,0x8F8F,0x6464,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0x64D4,0xBFBF,0x4654,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x8585,0x4675,0x8565,0x9585,0x7565,0x5485,0x8585,0x8575,0x6585,0x7585,0x7575,0x8585,0x7575,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F46
.word 0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xD4E4,0xC5E4,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE5E4,0xF4E4,0xF4E4,0xF4E4,0xE4E5,0xE5F4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4C5,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xBFBF,0xC4C4,0x6464,0x7575,0x6565,0x5475,0x9F9F,0x4441,0x7F7F,0x7F7F,0x4646,0x4646,0x4446,0x5454,0xAFAF,0x5464,0xAFAF,0x6464,0x6465,0xAFAF,0x9F9F,0x5454,0x4154,0x4444,0x8F8F,0x8F8F,0x7F7F,0x6464,0x9F9F,0x5454,0xAFAF,0xD4D5,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD485,0x4141,0x5454,0x7F7F,0xD4C4,0xD4D4,0x85D4,0xD4D4,0x4141,0xE4D4,0xD4E4,0x54D4,0x8F8F,0x8F8F,0x5454,0xD4D4,0x6464,0xD454,0x8F8F,0x5441,0xAFAF,0x9F9F,0xC464,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xAFAF,0x7575,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x5F5F,0x4646,0x4646,0x9544,0x8585,0x85C4,0x95B4,0x7575,0x7554,0x7565,0x7554,0x7585,0x7575,0x5454,0x7575,0x7585,0x4475,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xD5E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xE4C5,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D5,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x64D5,0x7575,0xAFAF,0xAFAF,0x5454,0x8F8F,0x6F6F,0x7F7F,0x4646,0x4646,0x5146,0x6454,0x5464,0xAFAF,0xAFAF,0xAFAF,0x5454,0x6464,0xAFAF,0x5454,0x5454,0x9F9F,0x4444,0x4454,0x9F9F,0x5454,0x4444,0x4441,0x5454,0x9F9F,0x6464,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD485,0xAFAF,0x8F8F,0x8F8F,0xD441,0xD4D4,0x5441,0x9F9F,0x5454,0xD564,0x8F8F,0xE464,0xD4E4,0x6464,0xAFAF,0x5441,0xD4D4,0x5441,0xD4D4,0x7F7F,0x6464,0x8F8F,0x6454,0x8F8F,0xD4C4,0xE4D4,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x9F9F,0x5F5F,0x4446,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x7546,0xC485,0x95B4,0xD4D4,0x5485,0x7554,0x75D4,0x8575,0x9565,0x7575,0x9585,0x8575,0x7585,0x7585,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x5F5F,0x465F
.word 0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xD4F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E5,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD585,0xAFAF,0x6475,0x6464,0x5454,0x5454,0x4141,0x7F7F,0x4646,0x4646,0x5441,0xAFAF,0xAFAF,0xAFAF,0xAFAF,0x6464,0x7575,0x7575,0x6454,0x9F9F,0x9F9F,0x4444,0x4144,0x9F9F,0x7F7F,0x8F8F,0x8F8F,0x4444,0x4141,0x8F8F,0x4141,0x9F9F,0x5454,0x4154,0x4141,0x5441,0x6464,0x5454,0x4444,0x4154,0xD4D4,0x4141,0x4444,0x6475,0x6464,0x4141,0xD4D4,0xD4D4,0x8F8F,0x8F8F,0xD464,0xD4D4,0x4141,0xD554,0x7474,0x9F9F,0x54D5,0xD4D4,0x8F8F,0x5454,0x6464,0xD4E4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD474,0xC5D4,0xD4D4,0xD564,0xC4C4,0x4641,0x4446,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0xC454,0xB495,0xD485,0x7585,0x7554,0x5475,0x8585,0x8575,0x6465,0x7565,0xC475,0xB495,0x7585,0x7575,0x4654,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4F4F,0x4644,0x4646,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D5,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD5D4,0xD5D4,0xE4E4,0xE4D5,0xE4F4,0xE4E4,0xE4F4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xAFAF,0xAFAF,0x5454,0x4444,0x7F7F,0x6F6F,0x4644,0x5F5F,0x6454,0xAFAF,0xAFAF,0xAFAF,0x6464,0xD575,0x6464,0xD575,0x6464,0x5454,0xAFAF,0x5454,0x9F9F,0x8F8F,0x4444,0x7F7F,0x4444,0x4141,0x4444,0x8F8F,0x4444,0x9F9F,0x4154,0x6464,0x64D5,0x7575,0x4444,0x4444,0x4141,0x4444,0x8F8F,0x6454,0x4444,0x8F8F,0x4444,0x4444,0x8585,0xD4D4,0x7F7F,0x5454,0x8F8F,0xD4E5,0xD4D4,0x8F8F,0x7F7F,0x54D4,0x4141,0xD554,0x8F8F,0x4444,0x7575,0x6464,0xD4E4,0xBFBF,0xD4C4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xBFBF,0x6464,0x4654,0x5F5F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4446,0xC475,0xD495,0xB495,0x7554,0x7575,0x7565,0x8575,0x7575,0x7595,0x7585,0xC485,0x9575,0x8575,0x5475,0x4644,0x5F5F,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4644
.word 0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E5,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xD5D4,0xE4D4,0xD4C5,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xE4D4,0xD4C5,0xE4E4,0xE4E4,0xE4F4,0xF4E4,0xF4F4,0xF4E4,0xF4F4,0xE4E4,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xC5D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x6464,0x7575,0x5454,0x8F8F,0x7F7F,0x7F7F,0x4646,0x6444,0xAFAF,0xD564,0x6464,0xD5C4,0xAFAF,0xAFAF,0xAFAF,0xD464,0xD5C4,0xD564,0x6464,0xAFAF,0x6464,0x5464,0x8F8F,0x5454,0x7F7F,0x4441,0x8F8F,0x4144,0x8F8F,0x4444,0x8F8F,0x4444,0x8F8F,0x4444,0x8F8F,0x4141,0x4141,0x4444,0x8F8F,0x5454,0x4444,0x4141,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0xD4D4,0x5454,0x8F8F,0x6464,0x64D4,0x4444,0x7F7F,0x4141,0x9F9F,0x4141,0x5454,0xD4D4,0xC4D4,0xBFBF,0x5454,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xAFAF,0x6464,0x4654,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4564,0x4646,0x4646,0x7544,0x8585,0x5454,0x5475,0x5475,0x7575,0x7554,0x6554,0x6575,0x5475,0xC485,0x9575,0x7585,0x7575,0x4665,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F4F,0x4F4F,0x5F5F,0x4646,0x5F5F,0x4F4F,0x4646,0x4646,0x465F
.word 0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD5E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4C5,0xE4D4,0xD4E4,0xE4E5,0xE4E4,0xF4E4,0xE4E4,0xE4F4,0xE5E4,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xD4E4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD564,0xAFAF,0x7575,0x9F9F,0x9F9F,0x6F6F,0x4444,0x4644,0x6464,0xAFAF,0xBFBF,0xD5C4,0xD464,0xD585,0xD464,0xD5D4,0xAFAF,0xBFBF,0x6464,0x64D5,0xD5D5,0x6464,0x7575,0x5454,0x8F8F,0x4441,0x7F7F,0x7F7F,0x4141,0x4444,0x4141,0x8F8F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4444,0x7F7F,0x4141,0x8F8F,0x8F8F,0x7F7F,0x4444,0x4444,0x4141,0x4444,0x4444,0xD4D4,0x54D4,0x7F7F,0x8F8F,0xE454,0x41D4,0x7F7F,0x7F7F,0x8F8F,0x7F7F,0x8F8F,0xD441,0xD4D4,0xD564,0x6464,0x6464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0xAFAF,0x9F9F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x9575,0x4645,0x4644,0x7545,0xC485,0x5475,0x5475,0x7575,0x8575,0x8565,0x54C4,0x7585,0x8585,0x9585,0x85C4,0x85B4,0x8575,0x6585,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x5F5F,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x4646
.word 0xD4E4,0xE4D5,0xE4E4,0xE5E4,0xE4E4,0xE4D5,0xE4F4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E5,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x54D5,0xAFAF,0x8F8F,0x6F6F,0x7F7F,0x5146,0xBFBF,0x6485,0x64D5,0x61D5,0xD5D5,0x64D4,0xD4D4,0xBFBF,0xD4C4,0x64D4,0xD4D4,0xD4D4,0x6464,0x6475,0xAFAF,0x5464,0x5454,0x8F8F,0x8F8F,0x4141,0x7F7F,0x4141,0x4444,0x7F7F,0x4141,0x7F7F,0x4444,0x4444,0x4141,0x7F7F,0x4444,0x7F7F,0x4441,0x7F7F,0x7F7F,0x7F7F,0x4141,0x7F7F,0x8F8F,0xD4E4,0xD4E4,0x7F7F,0x7F7F,0x8F8F,0xD4D4,0x7F7F,0x4444,0x4141,0x7F7F,0x7F7F,0x9F9F,0xD4D4,0xD4D4,0xD564,0x6464,0xD464,0xD4D4,0xD474,0xD4D4,0x6464,0xAFAF,0xAFAF,0x4645,0x5F5F,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x7446,0x7585,0x8575,0x8584,0x95B4,0x7585,0x7554,0x5475,0x8575,0x7554,0x7585,0x5475,0x6575,0x9565,0x6585,0x8585,0x8575,0x8575,0x5485,0x4646,0x5F5F,0x4446,0x4F4F,0x4646,0x4F4F,0x4646,0x4F4F,0x4F4F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4F4F,0x5F5F,0x4646
.word 0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4C5,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D5,0xE4E4,0xE5E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xC4C4,0xC4C4,0xAFAF,0x6464,0x5454,0x6F6F,0x4141,0x6441,0x64D5,0xD5D5,0xD4C4,0x64D4,0x64D4,0xD4D4,0xD4C4,0x64C4,0x74D4,0xD4D5,0xD4D4,0xD464,0x64D5,0xD5D5,0x6464,0x6464,0x9F9F,0x5454,0x4141,0x8F8F,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4444,0x8F8F,0x7F7F,0x4141,0x4444,0x4141,0x8F8F,0x7F7F,0x9F9F,0x4141,0x7F7F,0x7F7F,0x8F8F,0x4444,0x4444,0xE454,0xD4D4,0xE451,0x8F8F,0x4444,0xD4D4,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4144,0x9F9F,0x5454,0xD4D4,0xBFBF,0xAFAF,0x64D5,0xAFAF,0xD4D4,0x64D4,0xD564,0x6464,0xAFAF,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x8575,0xC485,0xC495,0xC485,0xB495,0x7554,0x7554,0x8575,0x6575,0x8575,0x7585,0x8585,0xB485,0x8595,0x85B4,0x8565,0x85B4,0x8575,0x5475,0x4646,0x4646,0x4646,0x4644,0x4646,0x4F4F,0x4F4F,0x4F4F,0x5F5F,0x4646,0x5F5F,0x5F5F,0x4444,0x4646,0x4646,0x4446
.word 0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD4E4,0xE4C5,0xE4D4,0xE4C5,0xE4E4,0xE4D4,0xC5D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE5D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D5,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xC4C4,0x54C4,0x5454,0x4441,0x6F6F,0xAFAF,0xC4C4,0xD485,0xAFAF,0xAFAF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xBFBF,0xD4D4,0xD464,0xD4D5,0x64D5,0xC4C4,0x6464,0x5454,0x5444,0x4441,0x4441,0x7F7F,0x4141,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x8F8F,0x8F8F,0x8F8F,0x7F7F,0x4141,0x4444,0x7F7F,0x4444,0x7F7F,0x7F7F,0x4141,0xE4E4,0xE4D4,0xD4D4,0x8F8F,0x7F7F,0x85E4,0x7F7F,0x5454,0x7F7F,0x4141,0x7F7F,0x4444,0x7F7F,0xD4D4,0xD4D4,0xD4D4,0xD564,0x6464,0x51D5,0x9F9F,0x5454,0xC4C4,0x6464,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x5F5F,0x4646,0x4646,0x4446,0x9575,0x85C4,0xB485,0xC5B5,0x7585,0x5454,0x7554,0x7575,0x7554,0x6575,0x6575,0x8575,0x7575,0x8585,0x8565,0x7595,0x7585,0x8585,0x4465,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x5F5F,0x4F4F,0x5F46
.word 0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE5E4,0xE4E4,0xE4F4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E5,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E5,0xE4F4,0xF4E4,0xE4E4,0xE5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0x6464,0x6464,0x8F8F,0x5454,0x64D4,0x64D4,0x64D5,0xD464,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFAF,0x64C4,0x64D5,0x6464,0x9F9F,0x9F9F,0x8F8F,0x7F7F,0x4441,0x4444,0x4141,0x7F7F,0x4141,0x4141,0x7F7F,0x4141,0x7F7F,0x4141,0x6464,0x7F7F,0x7F7F,0x4141,0x7F7F,0x7F7F,0x4444,0xD444,0xD4D4,0xD4D4,0x44E4,0x4444,0xC4E4,0x5454,0x5454,0x8F8F,0x4444,0x4141,0x6441,0x41E5,0x7F7F,0xD4D4,0xD4D4,0xBFBF,0xAFAF,0x6464,0x6565,0x7575,0x9F9F,0xAFAF,0x5464,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4646,0x4644,0x6546,0xC495,0xC495,0xC495,0xC495,0x7585,0x5454,0x7554,0x8575,0x8575,0x64D4,0xC485,0x8575,0x95B4,0x7575,0x6585,0xB495,0x8575,0x6585,0x5F5F,0x4646,0x4646,0x4F4F,0x5F5F,0x4F4F,0x5F5F,0x4F4F,0x4646,0x4F4F,0x4F4F,0x5F5F,0x4646,0x5F5F,0x4646
.word 0xE4E4,0xD4E4,0xE4D5,0xD5E4,0xF4E4,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xC5D4,0xE4E4,0xC5D4,0xE4D4,0xD4E4,0xD4D5,0xE4E4,0xD4E5,0xE4F4,0xE5E4,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xD4E4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD574,0xD564,0x5454,0x9F9F,0x6464,0xAFAF,0x64D4,0xD4D4,0xD4D5,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xC4D4,0xBFBF,0xC4C4,0xAFAF,0xAFAF,0x9F9F,0x4454,0x4441,0x7F7F,0x7F7F,0x8F8F,0x4444,0x5454,0x6454,0xAFAF,0x6475,0x5454,0x6464,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x4444,0xD464,0xD4D4,0xD464,0xD4D4,0x7F7F,0xC4E4,0xD564,0x5454,0x4444,0x6464,0xD4D4,0xD4E4,0xD4E4,0xD441,0xD4D4,0xD4D4,0xD464,0xD4C4,0x64D5,0x64D5,0x5454,0x7575,0xAFAF,0xAFAF,0x5454,0x5451,0x4644,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4446,0x85B4,0xC485,0x6585,0x5454,0x7565,0x7554,0x7554,0x7565,0x8554,0x75C4,0xD475,0x7595,0x9575,0xB485,0x8575,0x4575,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646
.word 0xE4D4,0xE4C5,0xE4D4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4F4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xD4C5,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4C5,0xE4D4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xF4F4,0xE5E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4D5,0xD4E4,0xE4E4,0xE4D4,0xC5E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x6464,0x4154,0xBFBF,0x64C4,0xD4D4,0xD464,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD564,0xD585,0xAFAF,0x64C4,0x9F9F,0x4141,0x8F8F,0x7F7F,0x8F8F,0x8F8F,0x4141,0x5454,0x9F9F,0x5454,0xD554,0x5454,0x7F7F,0x7F7F,0x7F7F,0x4444,0x4444,0x4141,0x4444,0xD464,0xD4D4,0xD464,0xD4D4,0x4444,0xD4E4,0xBFBF,0x5454,0x7F7F,0x8F8F,0xD4E4,0xD4D4,0xE4E4,0xD464,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0xC4C4,0x6464,0x5454,0x5454,0x6475,0xAFAF,0x5454,0x5154,0x5454,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4446,0x4446,0x4646,0x7546,0x8585,0x8585,0x5475,0x5454,0x7554,0x7585,0x9585,0x7575,0x5485,0x7575,0x6585,0xD485,0x8585,0x75D4,0x8585,0x4446,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x5F5F,0x4446,0x4646,0x4644,0x4644,0x465F
.word 0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xE4F4,0xE4E5,0xE4F4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE5E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD474,0xAFAF,0x64C4,0xAFAF,0x64D5,0xD4D4,0xC4C4,0xD4D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8585,0x64D5,0xC4C4,0x7575,0x5454,0x4444,0x4441,0x7F7F,0x8F8F,0x4444,0x8F8F,0x5454,0x5454,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4141,0x4141,0x4444,0x8F8F,0x6464,0x8F8F,0x8F8F,0x5454,0xD441,0xC4E4,0x41D4,0x5454,0x6464,0xD4D4,0xD4D4,0xD4D4,0x6464,0xE4E5,0xE4D4,0xD4D4,0xD4D4,0xBFBF,0xD4D4,0xD564,0x7575,0xAFAF,0x9F9F,0x7575,0x7575,0xAFAF,0x6464,0x5454,0x5164,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F5F,0xC454,0xD485,0x95B4,0x95C5,0x75C4,0x5454,0x7554,0x8575,0x8565,0x7575,0xD485,0x8554,0x7575,0x7575,0x8585,0x8575,0x4475,0x4646,0x4646,0x4446,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xD4E4,0xE5D4,0xE5D4,0xE4E4,0xE5E4,0xE4F4,0xF4E4,0xE4F4,0xE4E4,0xE4F4,0xE4D5,0xE4E5,0xD4F4,0xE5E4,0xE4E4,0xE4E4,0xF4E4,0xE5E4,0xE4E4,0xE4D5,0xE4E5,0xE4E4,0xD5E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x7575,0xC4C4,0xAFAF,0xD564,0xD485,0xD4D4,0xD4C4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD564,0x6464,0xAFAF,0x5454,0x5454,0x8F8F,0x8F8F,0x7F7F,0x7F7F,0x8F8F,0x4444,0x7F7F,0x7F7F,0x4141,0x4444,0x4444,0x7F7F,0x7F7F,0xAFAF,0x6485,0xD4D5,0xC4C4,0x6464,0xAFAF,0xE4D4,0x64D4,0x8F8F,0xAFAF,0x64E4,0x64D4,0x9F9F,0xE4D4,0x8F8F,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x64D4,0x64D5,0xAFAF,0x9F9F,0xAFAF,0x6565,0xAFAF,0x6465,0xD5D5,0x6464,0x4454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x9564,0x85C4,0xB495,0xB495,0xD485,0x85B5,0x5475,0x5465,0x7565,0xC485,0x8575,0x8585,0x8575,0x75C4,0x8575,0x7565,0x7585,0x7585,0x4646,0x4646,0x5F5F,0x4646,0x5F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F5F,0x4646,0x465F
*/




/*   Dragon Girl
Fish_320:
.word 0x4646,0x4646,0x4646,0x4646,0x8575,0x8575,0x7575,0x4646,0x4646,0x4646,0x4646,0x4646,0x7544,0x5465,0x7575,0x4645,0x4646,0x4646,0x7565,0x5454,0x5465,0x6585,0x4646,0x6546,0x7554,0x8575,0x4646,0x4646,0x4646,0x4646,0x6544,0x7554,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6F4F,0x7F6F,0x6F6F,0x7F6F,0x4141,0x8F4F,0x6F7F,0x6F6F,0x6F6F,0x7F6F,0x7F6F,0x6F6F,0x5F6F,0x6F7F,0x5F7F,0x6F6F,0x6F6F,0x6F7F,0x6F7F,0x6F7F,0x6F6F,0x6F7F,0x6F7F,0x6F7F,0x7F6F,0x6F6F,0x6F6F,0x7F6F,0x7F6F,0x7F7F,0x7F7F,0x4646,0x6546,0x4665,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5444,0x8565,0xD4D4,0x4575,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6544,0x8585,0x95D4,0x7585,0x5465,0x4645,0x4646,0x4646
.word 0x4646,0x4644,0x4646,0x4644,0x4546,0x7595,0x7575,0x8585,0x4675,0x4644,0x4646,0x4646,0x5446,0x5475,0x5475,0x6475,0x4645,0x4544,0x7575,0x7554,0x7554,0x5465,0x7585,0x7554,0x7554,0x7554,0x4675,0x4446,0x4646,0x4444,0x7554,0x7575,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x6F5F,0x7F4F,0x6F6F,0x4F7F,0x4F8F,0x7F8F,0x6F6F,0x7F6F,0x6F6F,0x6F7F,0x6F7F,0x6F6F,0x6F6F,0x7F6F,0x6F6F,0x6F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F7F,0x6F7F,0x7F7F,0x6F6F,0x7F7F,0x7F4F,0x7F7F,0x4F7F,0x4544,0x54D4,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x7541,0x9585,0x95D4,0x5475,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4444,0x8554,0xB5D4,0x75B4,0x4575,0x4646,0x4644,0x4644,0x465F
.word 0x4646,0x5F4F,0x4646,0x4646,0x4646,0x5446,0x54C4,0x6554,0x7564,0x7575,0x4446,0x4446,0x7546,0x7554,0x7554,0x7554,0x5454,0x6554,0x5454,0x5454,0x5475,0x5475,0x5454,0x5475,0x7554,0x7554,0x6454,0x7575,0x7554,0x7554,0x5454,0x7554,0x4685,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4F4F,0x7F7F,0x7F7F,0x7F4F,0x8F5F,0x7F4F,0x6F6F,0x6F7F,0x6F7F,0x7F6F,0x7F6F,0x6F6F,0x6F6F,0x7F6F,0x6F6F,0x4F6F,0x5441,0xD464,0x5485,0x7F5F,0x6F6F,0x4F7F,0x4F7F,0x7F7F,0x7F4F,0x4F7F,0x6F7F,0x6F7F,0x6F7F,0x6F6F,0x4F7F,0x4F7F,0x7F7F,0x4646,0xC554,0x4475,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x5444,0x8575,0x95D4,0x5485,0x4645,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4446,0x8554,0x95C5,0x75B4,0x4475,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0x4446,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x8565,0x7575,0x7554,0x6475,0x7575,0x4646,0x5444,0x7554,0x5454,0x5465,0x5475,0x5475,0x5454,0x5454,0x5454,0x5454,0x6554,0x5454,0x5475,0x5454,0x5475,0x5475,0x5475,0x5454,0x7575,0x5475,0x5475,0x4646,0x4646,0x4646,0x4646,0x6F5F,0x4144,0x4F7F,0x7F7F,0x4F7F,0x7F4F,0x6F7F,0x7F6F,0x7F6F,0x7F6F,0x4F6F,0x7F6F,0x7F6F,0x7F5F,0x6F4F,0x6F6F,0x6441,0xD4D4,0xD4E4,0x9F6F,0x6F7F,0x6F6F,0x5444,0x7F4F,0x4F7F,0x7F6F,0x7F7F,0x4F7F,0x6F6F,0x7F7F,0x7F4F,0x7F6F,0x6F4F,0x4F7F,0x4F6F,0x7544,0x75D4,0x4644,0x4F5F,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4446,0x4146,0x7544,0xB485,0xD4B5,0x8595,0x4554,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x5446,0x4654,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4446,0x7554,0x9585,0x75B4,0x4475,0x4646,0x4644,0x5F4F,0x4646,0x4F5F,0x4646
.word 0x4446,0x4545,0x4545,0x4545,0x4446,0x4646,0x4646,0x7575,0x7554,0x7554,0x5475,0x5475,0x7554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5454,0x5454,0x4475,0x4446,0x5F4F,0x6F4F,0x7F4F,0x5454,0x6F7F,0x4F7F,0x5441,0x4F8F,0x7F6F,0x6F6F,0x4F7F,0x6F7F,0x6F7F,0x6F6F,0x6F6F,0x4F6F,0x6F6F,0x5F6F,0xD564,0xD4E4,0xAF6F,0x4F7F,0x6F6F,0x4441,0x7F8F,0x7F6F,0x7F7F,0x6F7F,0x6F7F,0x6F7F,0x6F4F,0x6F7F,0x6F6F,0x6F4F,0x4F7F,0x4F7F,0x6F6F,0x4F7F,0x9554,0x6575,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4446,0x5444,0x8575,0x95C4,0x95D4,0x8585,0x4454,0x4446,0x4646,0x5F4F,0x5446,0x6554,0x5465,0x4641,0x4644,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x4F5F,0x4446,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4446,0x5F4F,0x4646,0x4F5F,0x4646,0x4444,0x8575,0x8585,0x8595,0x7595,0x7585,0x5454,0x4444,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4446,0x5454,0x8485,0xC495,0x7585,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7546,0x8585,0x8575,0x7575,0x8575,0x4485,0x4446,0x7575,0x5475,0x5475,0x5465,0x5454,0x5475,0x5454,0x5454,0x5454,0x5444,0x5454,0x5444,0x4454,0x4454,0x4454,0x5454,0x4454,0x5441,0x5444,0x5454,0x5454,0x5465,0x5454,0x5454,0x6554,0x5475,0x7575,0x4475,0x4646,0x4F6F,0x5441,0x7F9F,0x7F7F,0x4144,0x8F5F,0x6F7F,0x6F7F,0x6F4F,0x7F7F,0x4F7F,0x6F6F,0x6F7F,0x6F6F,0x6F6F,0x6F6F,0x5441,0x5FAF,0x7F7F,0x7F7F,0x7F4F,0x7F6F,0x6F6F,0x6F7F,0x7F7F,0x6F6F,0x6F6F,0x4F6F,0x7F6F,0x6F7F,0x6F7F,0x7F6F,0x7F6F,0x7F7F,0x7F4F,0x4144,0x7F7F,0x5445,0x85D4,0x4665,0x4646,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4F5F,0x4146,0x6554,0x8575,0xC485,0xD4B5,0x95B5,0x8585,0x7585,0x7575,0x7575,0x7575,0x5454,0x5465,0x4444,0x4454,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x8554,0x9595,0x95C4,0x7585,0xB485,0x9585,0x9595,0x85B4,0x5454,0x5454,0x6575,0x5454,0x5454,0x8575,0x8585,0x9495,0x85C4,0x75B4,0x4675,0x4644,0x6F4F,0x4646,0x4F5F,0x4F5F,0x4446,0x4646,0x4646
.word 0x4646,0x4646,0x8575,0x7575,0x6554,0x5454,0x6575,0x7554,0x5475,0x5454,0x5454,0x5454,0x5454,0x5444,0x5444,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x7541,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4454,0x5454,0x7554,0x7554,0x5454,0x7F4F,0x4441,0x4F8F,0x4F7F,0x5F7F,0x9F4F,0x7F8F,0x7F6F,0x6F7F,0x7F7F,0x6F4F,0x7F7F,0x7F6F,0x5F6F,0x4F6F,0x7F6F,0x5F6F,0x5F7F,0x4F8F,0x4154,0x8F4F,0x6F7F,0x7F6F,0x7F6F,0x4F6F,0x7F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F,0x7F4F,0x6F6F,0x6F7F,0x6F7F,0x8F4F,0x4F7F,0x7F4F,0x4F7F,0x7554,0x6495,0x4654,0x4F5F,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x6541,0x5454,0x7554,0x7565,0x8575,0x95B4,0xD4C4,0xB595,0x9595,0xB5B5,0xB5D4,0x95D4,0x95C5,0x95C5,0x8585,0x6475,0x5454,0x4454,0x4454,0x4644,0x4646,0x4644,0x4646,0x5F4F,0x4646,0x4644,0x5F4F,0x4446,0x4646,0x4644,0x4646,0x4646,0x4F5F,0x4F4F,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4F5F,0x4644,0x4F5F,0x5F4F,0x4646,0x4644,0x4446,0x8554,0xC4C5,0xB485,0x6575,0x5475,0x5454,0x6575,0x5454,0x8554,0x7585,0x7585,0x7575,0x8575,0x8575,0x8585,0x5454,0x5454,0x7575,0x4454,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646
.word 0x4446,0x4646,0x6546,0x8585,0x7554,0x5475,0x7575,0x7554,0x7554,0x5454,0x5454,0x5454,0x5444,0x5454,0x5454,0x5454,0x5454,0x5475,0x7554,0x5475,0x7554,0x5454,0x7554,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5454,0x5444,0x5441,0x5454,0x5454,0x4454,0x4141,0x8F4F,0x8F4F,0x7F4F,0x8F5F,0x8F5F,0x6F7F,0x7F4F,0x4F6F,0x7F7F,0x4F6F,0x6F7F,0x6F7F,0x4F6F,0x7F6F,0x6F6F,0x5454,0x4FBF,0x9F6F,0x9F5F,0x6F8F,0x6F7F,0x6F7F,0x6F4F,0x6F6F,0x6F6F,0x6F6F,0x4F6F,0x6F6F,0x6F6F,0x6F7F,0x4F6F,0x6F7F,0x4F6F,0x6F6F,0x4F7F,0x7F4F,0x5444,0x7F8F,0x4446,0xC575,0x6575,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4441,0x5454,0x5454,0x5454,0x5454,0x8575,0xC485,0xC485,0x95B4,0x95C4,0x85C4,0xC495,0xC4E4,0xD4D4,0xD4B5,0x9584,0x85B5,0x7585,0x7575,0x5454,0x4454,0x4444,0x4644,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x8544,0x8595,0x85B4,0x7575,0x5454,0x4644,0x4646,0x4444,0x5454,0x6554,0x7565,0x5454,0x5475,0x5475,0x6554,0x6554,0x5454,0x4454,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4644,0x4F5F,0x4646,0x4646
.word 0x4646,0x5F4F,0x4646,0x7565,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x5444,0x5454,0x5454,0x5454,0x7554,0x7554,0x7575,0x5454,0x7575,0x7575,0x5454,0x7575,0x7585,0x5475,0x7554,0x8585,0x5454,0x5454,0x5454,0x7554,0x5454,0x5454,0x4454,0x4154,0x4F7F,0x8F4F,0x8F5F,0x7F4F,0x5F7F,0x4F9F,0x6F7F,0x6F4F,0x7F7F,0x7F6F,0x7F4F,0x6F7F,0x6F6F,0x4F6F,0x6F6F,0x6F4F,0x8F6F,0x64D4,0xD564,0xCFAF,0x7F5F,0x6F7F,0x6F7F,0x6F6F,0x7F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F,0x6F7F,0x6F6F,0x7F6F,0x4F7F,0x6F8F,0x4141,0x4F7F,0x6554,0x5454,0x5454,0x4644,0x4644,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4446,0x4646,0x4446,0x5444,0x5454,0x7554,0x7565,0xC485,0x8585,0xB485,0xB495,0x8585,0x8585,0x95B4,0xC4C4,0xB495,0x95C5,0xC5C5,0xB5C4,0xB584,0x8595,0x5475,0x5454,0x4144,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4446,0x4446,0x4646,0x4F5F,0x4446,0x5F4F,0x4646,0x4644,0x5F4F,0x4446,0x7585,0x7565,0x5475,0x5475,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646
.word 0x4446,0x9585,0x8585,0x7585,0x5475,0x7554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x8575,0x5475,0x5454,0x7554,0x7585,0x5454,0x5475,0x7585,0x5454,0x7575,0x5475,0x7575,0x7575,0x5465,0x8554,0x8585,0x5475,0x7554,0x7575,0x5454,0x7F4F,0x9F5F,0x5454,0x7F8F,0x7F4F,0x5454,0x7F8F,0x7F7F,0x7F7F,0x4F7F,0x4F7F,0x7F6F,0x4F7F,0x7F6F,0x7F6F,0x7F4F,0x6F7F,0x6454,0x6FAF,0x64E5,0x64E4,0x7F7F,0x7F6F,0x6F6F,0x4144,0x7F7F,0x4F6F,0x7F5F,0x6F6F,0x6F6F,0x6F6F,0x5F4F,0x6F6F,0x7F6F,0x4F6F,0x7F6F,0x6F7F,0x4F7F,0x7F7F,0x8F4F,0x8FCF,0x6F4F,0x5445,0x7554,0x9575,0x6575,0x4646,0x4646,0x4446,0x4446,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x5F4F,0x4646,0x4646,0x4146,0x5444,0x7554,0x7575,0x7575,0x7575,0x7585,0x7585,0x85B4,0x8575,0x8585,0x85B4,0xB4C4,0xC495,0xC495,0xD495,0x95D4,0x95D4,0x8595,0x7575,0x5454,0x4654,0x4F5F,0x4446,0x5F4F,0x4646,0x4F5F,0x4646,0x4644,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x7544,0x7595,0x5475,0x7575,0x5454,0x7585,0x4454,0x5F4F,0x4646,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4446,0x4646,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4F5F,0x5F4F,0x4646,0x4446,0x4646,0x465F
.word 0x8575,0x75B4,0xB485,0x6575,0x6564,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5475,0x7575,0x7575,0x5475,0x7565,0x7554,0x8575,0x7575,0x7554,0xC485,0x7585,0x8554,0x85D4,0x5465,0x8575,0x7585,0x6554,0x8575,0x5475,0x7554,0x7575,0x5475,0x4F8F,0x44D4,0x5F9F,0x7F5F,0x5441,0x4FAF,0x4F8F,0x4F6F,0x7F7F,0x7F4F,0x7F7F,0x4144,0x6F6F,0x4F7F,0x4F7F,0x7F6F,0x4F7F,0xAF5F,0xD454,0xF5D4,0x8F5F,0x6F6F,0x6F7F,0x4F7F,0x7F5F,0x7F4F,0x6F4F,0x6F6F,0x6F6F,0x6F4F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F,0x6F6F,0x7F6F,0x4F7F,0x4F8F,0x4F8F,0x4F7F,0x5454,0x7565,0x8585,0x75B5,0x4654,0x5F4F,0x7584,0x4646,0x4646,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4446,0x5444,0x5454,0x6554,0x6564,0x5465,0x7575,0x7575,0x7575,0x75B4,0x8575,0x8585,0x85B4,0xC485,0xC485,0xC495,0xC485,0xC595,0xC595,0x8595,0x7575,0x4465,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4644,0x4646,0x4446,0x7565,0x5475,0x5475,0x7554,0x8575,0x8575,0x7585,0x7575,0x7575,0x4644,0x4F5F,0x4646,0x5F4F,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0x8585,0x7585,0x7575,0x5475,0x5475,0x5454,0x5454,0x5454,0x7554,0x5454,0x5475,0x7575,0x5454,0x7554,0x7554,0x8585,0x7554,0x7575,0x8575,0x6585,0x7575,0x8585,0x8565,0x8585,0x5475,0xC485,0xC485,0x5475,0xD485,0x5485,0x5454,0x7575,0x8F5F,0xD441,0xCF9F,0x6F9F,0x7F9F,0xD554,0x8F4F,0x7F7F,0x7F6F,0x7F4F,0x7F7F,0x7F4F,0x6F7F,0x6F7F,0x7F7F,0x7F7F,0x4F7F,0x4F6F,0x5454,0xD4D5,0xAF6F,0x6FCF,0x7F7F,0x4F6F,0x4F6F,0x4F6F,0x5F6F,0x7F7F,0x6F7F,0x6F6F,0x6F6F,0x6F6F,0x4F6F,0x7F5F,0x6F6F,0x6F6F,0x4F6F,0x4F6F,0x7F6F,0x6F4F,0x7F7F,0x8F4F,0x8F4F,0x7F4F,0x7575,0x8585,0x95C4,0xD485,0x7595,0x8565,0x8565,0x4445,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4F6F,0x5444,0x5475,0x7554,0x7554,0x7554,0x5475,0x7575,0x7575,0x7575,0x7575,0x8575,0x8575,0x85B4,0xB485,0xB495,0xD495,0x85C4,0xB5C4,0x85D4,0x7585,0x4454,0x4646,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4446,0x7585,0x5475,0x5475,0x8575,0xB485,0x7585,0x8585,0x7575,0x7554,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F5F,0x4446,0x4446,0x4646,0x4F4F,0x5F4F,0x4646
.word 0xB485,0x8554,0x6585,0x6564,0x5454,0x5454,0x6554,0x5454,0x5475,0x8575,0x8554,0x8585,0xC485,0x5475,0x7554,0xB485,0x7595,0x7575,0xC485,0x7585,0x7554,0x95C4,0x5485,0x9565,0x75C4,0x7565,0x8575,0x7585,0x8575,0x75B4,0x7575,0x6454,0x4F9F,0xD564,0xD464,0x8FEF,0x5444,0x5F9F,0x4F7F,0x7F4F,0x7F4F,0x4F7F,0x4F7F,0x4F7F,0x7F6F,0x7F4F,0x4F7F,0x4F7F,0x6F7F,0x8F4F,0x6F8F,0x6FAF,0x41D5,0x6F4F,0x4F6F,0x4F7F,0x7F7F,0x7F5F,0x7F7F,0x7F4F,0x6F4F,0x6F7F,0x6F6F,0x6F4F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F6F,0x6F7F,0x7F7F,0x7F5F,0x4141,0x4144,0x5454,0x8575,0x95C5,0x8585,0x7585,0x7575,0x4485,0x5F4F,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4446,0x4446,0x5454,0x5475,0x5475,0x5475,0x5475,0x7554,0x7554,0x7554,0x7575,0xB485,0x7585,0x8575,0xB495,0x8585,0xB495,0xB485,0xB495,0x8585,0x7595,0x4475,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4446,0x6446,0x7575,0x7554,0x7554,0x7585,0x7575,0x8585,0x6575,0x4654,0x4644,0x4646,0x4446,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x4646,0x4646
.word 0x7575,0x7575,0x5454,0x5475,0x5454,0x5454,0x7575,0x5475,0x7554,0x5454,0x7565,0x7575,0x6585,0x9575,0x7565,0x8575,0x7575,0x5475,0x8575,0xC485,0x7554,0xB485,0x7554,0x8554,0x95D4,0x7554,0xD485,0x5495,0x6575,0x5485,0x8575,0x5475,0x6F8F,0x5FBF,0x6FAF,0x4F8F,0x6F9F,0x7F9F,0x7F4F,0x7F7F,0x4F7F,0x4F7F,0x7F7F,0x6F4F,0x7F7F,0x4F7F,0x7F7F,0x7F7F,0x6F7F,0x4144,0xAF5F,0xAF7F,0xAFCF,0x7F7F,0x6F7F,0x7F6F,0x5F7F,0x5464,0x4F7F,0xE441,0x6F7F,0x7F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4F6F,0x6F6F,0x6F7F,0x6F4F,0x6F7F,0x6F4F,0x6F7F,0x6F7F,0x4441,0x7F8F,0x6F7F,0x5475,0x7544,0x5475,0x8565,0x7565,0x7575,0x7595,0x4645,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4446,0x5454,0x5475,0x5454,0x5475,0x5475,0x5454,0x7565,0x5475,0x7575,0x7575,0x7575,0x8575,0x85B4,0x85B4,0x8585,0x6575,0xD485,0x8585,0x7585,0x4645,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x5446,0x7575,0x7585,0x7575,0x85C4,0x8575,0x7575,0x4475,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4644,0x4644,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4644
.word 0x8585,0x7565,0x5454,0x5454,0x5454,0x5475,0x7554,0x7575,0x7575,0x8585,0x5475,0x7554,0x8554,0xB5D4,0x95C5,0x7575,0x8575,0x95C5,0x5485,0x85B4,0x75B5,0x7575,0xD485,0x5475,0x8575,0x5475,0xB485,0x5485,0x8575,0x54D4,0xC475,0x5F9F,0x54E4,0x6F8F,0x7FDF,0x9F5F,0x8F6F,0x4F7F,0x4F8F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x7F6F,0x4F7F,0x7F7F,0x7F7F,0x4441,0x4F6F,0x4F7F,0x7F9F,0xCFAF,0x5464,0x6F6F,0x7F4F,0x4154,0x9FCF,0xD4D4,0x6FDF,0x4F7F,0x7F4F,0x6F4F,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F6F,0x6F6F,0x6F7F,0x6F7F,0x6F4F,0x6F6F,0x4F6F,0x7F6F,0x8F6F,0x8F4F,0x4441,0xC485,0x5485,0x5454,0x7554,0x8575,0x8595,0x85B4,0x7585,0x4665,0x4F5F,0x5F4F,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x5441,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x7554,0x7554,0x7554,0x7575,0x6575,0x8585,0xB485,0x7565,0x7575,0x7565,0x7585,0x5465,0x4454,0x4446,0x4646,0x4F4F,0x4646,0x4644,0x4F5F,0x4646,0x4F5F,0x4646,0x4646,0x4446,0x5F4F,0x7546,0x5454,0x5465,0x8575,0x7575,0xC485,0x7585,0x5454,0x4644,0x4F5F,0x4646,0x4644,0x4F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646
.word 0x5475,0x5475,0x5454,0x5454,0x7554,0x7575,0x5475,0x5454,0x7575,0xB485,0xC495,0x6585,0x8575,0xD4B4,0x8585,0x5475,0x8565,0x7585,0x5465,0x8575,0x7575,0x7554,0x8585,0x5465,0xD485,0x6595,0x7554,0x5475,0x8585,0x7595,0xD454,0xAF5F,0x8F4F,0x9F5F,0x5441,0xCF9F,0x7F5F,0x9F4F,0x7F7F,0x7F4F,0x7F4F,0x7F4F,0x7F4F,0x7F6F,0x7F4F,0x4F7F,0x7F4F,0x7F7F,0x7F6F,0x4F7F,0x9F5F,0xAF7F,0x6FDF,0x7F6F,0x6F7F,0x4154,0xF564,0x54D4,0x54D4,0x8F7F,0x7F7F,0x4F6F,0x6F6F,0x4F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4F6F,0x6F7F,0x6F6F,0x7F6F,0x7F6F,0x7F6F,0x6F6F,0x5F7F,0x7F7F,0x9575,0x95C4,0x6585,0x5454,0x7575,0xC485,0x8585,0x95B4,0x6595,0x4646,0x4646,0x4446,0x4646,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x5446,0x6585,0x7575,0x7585,0x7554,0x5475,0x5454,0x5454,0x5454,0x5475,0x5454,0x7554,0x7554,0x6465,0x8554,0x7554,0x7585,0x5465,0x8575,0x7585,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x7546,0x7575,0x7554,0x7585,0x7554,0x8585,0x85B4,0x7554,0x5475,0x5465,0x4454,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x5F4F,0x4646
.word 0x5465,0x5454,0x5454,0x5454,0x7554,0x8575,0x8585,0x7575,0x5475,0x7565,0x7585,0x8554,0x5475,0x7575,0x7575,0xB5C4,0x5485,0x8554,0xB5C4,0x5475,0xC485,0x85D4,0x7554,0x7595,0x7575,0x7585,0x8575,0x85D4,0x7565,0x7565,0xAF5F,0x8F6F,0x4441,0x4F8F,0x5F9F,0xAF6F,0x4F8F,0x4F9F,0x7F4F,0x4F7F,0x4F7F,0x7F7F,0x6F7F,0x4441,0x4F7F,0x4F7F,0x4F6F,0x6F4F,0x4F7F,0x8F4F,0x8F5F,0x5454,0x7FAF,0x6F6F,0x5F6F,0x5F7F,0xDF9F,0x9F6F,0x8FDF,0x6F4F,0x7F7F,0x7F7F,0x7F6F,0x6F7F,0x6F4F,0x6F4F,0x6F7F,0x4F7F,0x6F6F,0x4F7F,0x6F6F,0x6F4F,0x7F6F,0x7F6F,0x7F7F,0x4F6F,0x8F4F,0x8554,0xC485,0x8575,0x7585,0x6554,0x7575,0xB485,0x7575,0x9585,0x6485,0x4445,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F4F,0x4446,0x6465,0x6554,0x6554,0x5454,0x5475,0x5475,0x5454,0x5485,0x8565,0x5475,0x5475,0x5465,0x5454,0x6575,0x5475,0x7554,0x9585,0xC5D4,0x95D4,0x9585,0x4654,0x4446,0x5F4F,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5444,0x6465,0x7554,0x7575,0x7575,0x7575,0x5475,0x7575,0x5475,0x8575,0x7575,0x8575,0x7585,0x4664,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F4F,0x4446,0x4646,0x4644,0x4646,0x4F5F,0x5F4F,0x4F4F,0x4646,0x4646,0x4646
.word 0x5454,0x5454,0x5454,0x7554,0x5475,0x8575,0x7575,0x8554,0x95D4,0x5475,0x5454,0xC485,0xD495,0x8595,0x7554,0xC485,0x85C4,0x7565,0x7585,0x5475,0x8585,0x6575,0x7554,0x85C4,0x5465,0x5475,0x7575,0xC485,0x8575,0x5495,0x54D4,0x4F8F,0x5F8F,0x5F8F,0x9F5F,0x4454,0x6F8F,0x8F5F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x4F7F,0x7F7F,0x4441,0x7F7F,0x8F5F,0x6F7F,0x7F7F,0x8F5F,0x8F5F,0xD554,0x6F6F,0x6F4F,0x7F6F,0x9F5F,0x5454,0x5FAF,0xD554,0x4F7F,0x7F6F,0x4F6F,0x6F7F,0x4F6F,0x7F5F,0x7F6F,0x4F6F,0x7F5F,0x6F6F,0x7F6F,0x4F6F,0x7F6F,0x6F6F,0x6F6F,0x4F6F,0x7F6F,0x4F7F,0x7544,0x85C4,0x7585,0xB485,0x5475,0x5454,0x8575,0x8585,0xB475,0x8585,0x9585,0x7564,0x4575,0x4446,0x5F4F,0x4646,0x4F5F,0x4446,0x5F4F,0x4646,0x5446,0x7575,0x5475,0x8554,0x5465,0x7554,0x5454,0x5454,0x6575,0x7575,0x5475,0x7554,0x5454,0x5454,0x5454,0x5454,0x6554,0x5475,0xB485,0xC485,0xC4C5,0x5495,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4F5F,0x4446,0x4646,0x4F5F,0x6565,0x5464,0x5454,0x7575,0x7554,0x7575,0x7575,0x5454,0x7585,0x6575,0x6564,0x6554,0x7575,0x4644,0x4644,0x5F4F,0x4646,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F
.word 0x5454,0x5454,0x5475,0x8575,0x85C4,0x5465,0x7565,0x7554,0x8585,0x85C4,0x6575,0x8575,0x75B4,0x9565,0x75D4,0x7554,0x8554,0x95C5,0x8565,0xD485,0x5465,0x8585,0x5495,0x7565,0x9575,0x95C5,0x6575,0x7575,0x8554,0x9F6F,0x41D4,0x4F8F,0x8F7F,0xAF6F,0x8F5F,0x5F8F,0x7F9F,0x7F8F,0x7F4F,0x4F7F,0x7F7F,0x7F7F,0x7F4F,0x7F4F,0x8F4F,0x4154,0x8F5F,0x4F7F,0x7F4F,0x7F5F,0x5F8F,0xDF9F,0x7F6F,0x4F6F,0x6F4F,0x4441,0x5F9F,0x9F6F,0x5F7F,0x6F7F,0x6F4F,0x5F4F,0x6F4F,0x7F6F,0x6F7F,0x4F6F,0x7F5F,0x7F7F,0x6F6F,0x6F7F,0x7F4F,0x7F7F,0x6F4F,0x6F6F,0x7F6F,0x7F6F,0x7F4F,0x7F6F,0x9585,0x85B4,0x8585,0xD485,0x6585,0x5454,0xB475,0x7575,0xB485,0x8585,0x8585,0x8585,0x7585,0xB485,0x8595,0x85B4,0x6585,0x7575,0x7575,0x5475,0x5475,0x7554,0x7554,0x7575,0x7554,0x8575,0x7565,0x5475,0x5475,0x5454,0x7575,0x7554,0x5475,0x7554,0x5475,0x5454,0x5454,0x5475,0x8575,0x7585,0x7575,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x7546,0x9585,0xD4D4,0x7585,0x5475,0x5454,0x5475,0x7554,0x7554,0x7575,0x5475,0x7575,0x7575,0x4554,0x4646,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x4646,0x4F5F,0x4646
.word 0x5454,0x5454,0x7554,0x8575,0x5475,0x8585,0x8585,0x5485,0x7575,0x6575,0x8585,0x6585,0x5454,0x8585,0x85C4,0x7565,0x8554,0xB485,0x6585,0x9585,0x5475,0x8565,0x75B4,0x8585,0x8554,0x85B4,0x7554,0x85B5,0x6554,0xC454,0x5F9F,0x7F8F,0x6441,0x5F9F,0x5F8F,0x8F6F,0x5475,0x7F4F,0x7F4F,0x8F5F,0x7F4F,0x4F7F,0x4F7F,0x8F4F,0x8F5F,0x6F8F,0x7F8F,0x4F6F,0x4F7F,0x4F7F,0x4F8F,0x5F8F,0x6F6F,0x7F4F,0x4F7F,0x8F6F,0x7F4F,0x4F8F,0x7F9F,0x5441,0x7F6F,0x7F6F,0x4F6F,0x7F6F,0x7F4F,0x6F6F,0x7F4F,0x4441,0x4F7F,0x4F6F,0x7F5F,0x7F7F,0x6F6F,0x7F7F,0x7F6F,0x6F7F,0x7F7F,0x6F7F,0x8575,0xC495,0x75B4,0x7575,0x9585,0x5475,0x5454,0x7575,0x8575,0x7575,0x85B4,0x85B4,0x85B4,0x9595,0xB5C4,0x95A5,0x95C5,0x85C5,0x85C5,0x9585,0x8585,0x85C4,0x8585,0x7575,0x7575,0x5454,0x7575,0x5475,0x7554,0x5485,0x5454,0x5475,0x5454,0x7575,0x5485,0x8554,0x5454,0x5454,0x6554,0x5465,0x5475,0x4646,0x4446,0x4646,0x4644,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x8575,0xC5B5,0x95D4,0xD4B5,0x8595,0x7575,0x5454,0x5475,0x8575,0x7575,0x8575,0x8575,0x7585,0x4645,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4646
.word 0x5454,0x8554,0x5465,0x7554,0x7575,0xB485,0x7585,0x8575,0x85C4,0x5475,0xB485,0x8595,0x5475,0x7575,0x7575,0xC595,0x6575,0x7575,0x8595,0x5465,0xC585,0x5485,0x5454,0xB485,0x5485,0x6554,0x7554,0x95C4,0x75C4,0x6464,0x7F9F,0x5F7F,0xAF5F,0x8F5F,0x8F5F,0xAF5F,0x8F4F,0x7F7F,0x4F7F,0x7F8F,0x6F4F,0x7F4F,0x4154,0x5F8F,0x5F9F,0x9F6F,0x6F4F,0x4441,0x7F7F,0x4F7F,0x7F4F,0x6F5F,0x7F7F,0x7F6F,0x7F7F,0x6F7F,0x4F7F,0x4F7F,0x5444,0x4F9F,0x7F6F,0x4154,0x7F6F,0x7F4F,0x7F7F,0x7F4F,0x7F4F,0x4F7F,0x4144,0x7F6F,0x7F7F,0x6F4F,0x4F7F,0x4F6F,0x4F7F,0x7F6F,0x4F6F,0x4F7F,0xC475,0x95B4,0x8585,0x7575,0xB485,0xC485,0x5475,0x5454,0x5475,0x8575,0x7575,0x8585,0xB485,0xB495,0xB485,0x85C4,0x85B4,0x85C4,0x95B4,0x85B4,0xB5C4,0xC5C4,0xD4C4,0xC595,0xD495,0xB5D4,0x8585,0x7575,0x7554,0x7554,0x7585,0x5454,0x5454,0x5454,0x5475,0x5475,0x5475,0xB485,0x5454,0x5454,0x4654,0x4446,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x8575,0xB585,0x85C4,0xC485,0x8585,0x85C4,0xB5D4,0x5485,0x7554,0x6554,0x7575,0x7575,0x5475,0x4554,0x4646,0x4F5F,0x5F4F,0x4646,0x5F4F,0x4646,0x4F5F,0x4644,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4446,0x4646
.word 0x6554,0x8575,0x8575,0x5475,0x5454,0x7585,0x6585,0xC485,0xC4C5,0x5475,0x7575,0x6575,0xD485,0x5485,0x7554,0x8585,0x75D4,0x7554,0xB485,0x5454,0xC485,0xB485,0x7595,0x8565,0x6575,0x9595,0x6465,0xB475,0x5485,0x9FDF,0x7F4F,0x8F4F,0x5454,0x4F8F,0x64D4,0x5F8F,0x4F9F,0x4F8F,0x4F7F,0x4F7F,0x4F7F,0x4F8F,0x5F9F,0x5F8F,0x8F5F,0x8F5F,0x7F7F,0x8F4F,0x7F7F,0x7F4F,0x7F7F,0x4164,0x4144,0x7F7F,0x5F7F,0x5454,0x6F6F,0x7F4F,0x5F8F,0x4F9F,0x4441,0x5441,0x8F5F,0x7F4F,0x7F4F,0x7F4F,0x4F6F,0x4F7F,0x4F9F,0x7F8F,0x7F4F,0x7F7F,0x6F7F,0x6F6F,0x6F7F,0x4F7F,0x7F7F,0x6F7F,0x8585,0x8575,0x95B4,0x8575,0x7575,0x8585,0x8585,0x5465,0x7554,0x7554,0x8575,0xB485,0xB495,0x8585,0x8585,0x8585,0xB485,0xB495,0xB495,0xB495,0xB485,0x8585,0x8585,0x85B4,0x95C4,0xD495,0xD4D4,0xC495,0x7585,0x7565,0x5475,0x8554,0x6575,0x5454,0x5454,0x5454,0x7554,0x7554,0x7554,0x7575,0x5441,0x4444,0x4646,0x4646,0x4646,0x4646,0x4644,0x9F5F,0x8585,0xB585,0xB485,0xB495,0xB485,0xB485,0xB485,0x9585,0x85D4,0x5465,0x7554,0x7575,0x8585,0x7565,0x5454,0x4454,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4446,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446
.word 0x5454,0x7575,0x8575,0x8585,0x6575,0x5454,0x5454,0x8585,0x75B4,0xD485,0x5485,0x5475,0x8575,0x7585,0x5465,0x7554,0x9554,0x5485,0x6465,0xC485,0x6575,0x5475,0x8575,0x7554,0x5475,0x7575,0x7585,0x7554,0xAF5F,0x4154,0x5F8F,0x5F7F,0x8F9F,0xD444,0x8F9F,0x64C4,0x4F8F,0x4F7F,0x8F5F,0x7F7F,0x4141,0xAF5F,0x5454,0x9F6F,0xD554,0x7F8F,0x7F4F,0x7F4F,0x7F4F,0x7F4F,0x6454,0xCFAF,0x6F8F,0x5444,0x7FCF,0x8F5F,0x5454,0xCF9F,0x9F5F,0x54C4,0xAFDF,0x9FCF,0xC4D4,0xC441,0x5F7F,0x7F5F,0x7F6F,0x7F7F,0x7F4F,0x7F7F,0x6F4F,0x4F7F,0x7F7F,0x7F5F,0x7F6F,0x7F7F,0x7F6F,0x6F4F,0x8575,0x85B4,0xB485,0xB485,0x8575,0x6575,0x7575,0x8585,0x5465,0x5475,0x5475,0x6575,0x7575,0x7565,0x7565,0x6575,0x8575,0x8575,0x8585,0x7575,0x8585,0x75B4,0x85B4,0x8575,0xB485,0xC485,0x8595,0x85C4,0x8585,0x8585,0x7575,0x6575,0x7554,0x7575,0x5475,0x6554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x6444,0x5454,0x6454,0x5444,0x8575,0x9585,0xB495,0xC485,0xB495,0x8585,0x8575,0x8585,0x8585,0x85B4,0xB5C4,0x7585,0x8554,0x7585,0x7554,0x8575,0x85C5,0x7575,0x5475,0x4454,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x5F4F,0x4446,0x4446,0x4646,0x4F4F,0x4646
.word 0x5475,0x5475,0x7565,0x8575,0x8565,0xD495,0x7585,0x7565,0x7575,0x8585,0x85D4,0x7565,0x6554,0x5475,0xC595,0x7595,0xB454,0x7585,0x5475,0x8575,0x5475,0x5485,0x6575,0x7575,0x5485,0x5465,0x5454,0x8575,0x6454,0x4F9F,0x7F8F,0x4441,0x4F8F,0xEF9F,0x41D4,0x7F9F,0x8F4F,0x8F4F,0x7F4F,0x4144,0x5F7F,0x7FAF,0x5F9F,0x8FDF,0x6454,0x7F4F,0x7F6F,0x7F6F,0x8F4F,0x7F7F,0xD564,0x64D4,0x7FAF,0x5F9F,0xDF8F,0x6454,0x5FAF,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF4F4,0xE4E4,0xAF6F,0x9F8F,0x8F6F,0x6F7F,0x4144,0x7F8F,0x7F4F,0x7F7F,0x7F7F,0x7F7F,0x4F6F,0x7F6F,0x7F6F,0x7F6F,0x7F6F,0x9554,0x7585,0x8585,0x8585,0x7585,0x7585,0x5475,0x5454,0x5475,0x5475,0x5454,0x6465,0x7554,0x5454,0x5475,0x7554,0x7575,0x8575,0x75B4,0x8585,0x8575,0x8575,0x8585,0x85B4,0x8585,0x85B4,0x85B4,0xB495,0xC475,0xC495,0xC4B5,0x7585,0x7554,0x7585,0x5454,0x5475,0x7554,0x7575,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5475,0x9585,0xB495,0xB495,0xB485,0x8585,0x8575,0x75B4,0x7575,0x7575,0x7575,0xC485,0x7595,0x6554,0x7575,0x8554,0xC485,0xB4B5,0x8585,0x7585,0x5454,0x4646,0x4F5F,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x4446,0x4F4F,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x4646
.word 0x9575,0x7585,0x7554,0x6554,0xB464,0xB485,0x8575,0x94D4,0x7585,0x7565,0x7575,0x9585,0x95C4,0x5475,0x7575,0x8565,0x6585,0x7554,0x7575,0x7554,0x5475,0x6575,0x7554,0x7554,0x5475,0x7575,0x8575,0x5454,0x4FAF,0x8F5F,0x4F7F,0x8F7F,0x8F4F,0x85E4,0xCF8F,0x5464,0x5F8F,0x4F9F,0x6F8F,0x4F8F,0x64D4,0x9F6F,0xBF7F,0xD454,0x4FAF,0x7F8F,0x7F7F,0x7F4F,0x4F7F,0x6F7F,0xD454,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xDFAF,0xD4D4,0xE4E4,0xE4E4,0xF4E4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xD4D4,0x64C4,0x8F6F,0x4F8F,0x8F5F,0x7F4F,0x7F7F,0x4F7F,0x7F6F,0x6F6F,0x7F4F,0x4F6F,0x7F6F,0x4F6F,0x7F6F,0xC485,0x5475,0x7565,0x95C4,0x85C4,0x7575,0x5475,0x5454,0x8575,0xB5C4,0x85D4,0x5465,0x7575,0x8585,0x8585,0x7565,0x7575,0x7575,0x8575,0x75B4,0x8575,0x8575,0x7585,0x85B4,0xC585,0x85D4,0x7575,0x7554,0x8575,0x95C4,0x85C4,0x6585,0x7554,0x7585,0x7554,0x7575,0x5475,0x8575,0x85C4,0x7565,0x8575,0x5485,0x7575,0x6585,0x8554,0xD495,0xD4C5,0xB495,0x8595,0x8575,0x85B4,0x8585,0xB485,0x8585,0x8585,0x85B4,0x75B4,0x7554,0x7595,0x8575,0x85B4,0x7585,0x75D4,0x4554,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xB485,0x7585,0xD475,0x7585,0x7554,0x7585,0x8554,0xC485,0x7585,0x7585,0x5454,0x8575,0x7585,0x8575,0x5465,0x8575,0x7575,0x5475,0x7554,0x5454,0x6575,0x5454,0x5475,0x5475,0x5454,0x5475,0x7565,0x4154,0x9F4F,0x7F8F,0x8F4F,0x7F4F,0x4144,0x9F6F,0x8F5F,0x4F8F,0x4F9F,0x4141,0x8FCF,0x6441,0xD4D5,0x9FDF,0xD4C4,0x5F8F,0x9F6F,0x6F4F,0x7F4F,0x7F7F,0x7F4F,0x9F5F,0xBF6F,0xD4C4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xE4F4,0xE4F4,0xE4D4,0xE4E4,0xD4E4,0xAFEF,0x5444,0x8F7F,0x5F8F,0x4154,0x6F4F,0x6F6F,0x7F6F,0x6F4F,0x7F7F,0x7F6F,0x4F6F,0x6F6F,0x5F7F,0x8575,0x75D4,0x5454,0x8575,0xB485,0x7575,0x5475,0x95C4,0xB5C4,0xC5D4,0x95D4,0x7575,0x7575,0x7585,0x7565,0x5475,0x8575,0x8585,0x7575,0x8585,0xC475,0x7585,0x8575,0xD485,0x85B4,0x7575,0xD4D4,0x5475,0x5444,0x5454,0x4454,0x5454,0x5454,0x5454,0x4454,0x5454,0x5444,0x7554,0x7554,0x5454,0x5454,0x6554,0x6564,0x5454,0xC485,0xC4C5,0xC5E4,0x95C4,0x85C4,0x85B4,0x8575,0x85B4,0x7575,0x7585,0x85B4,0x7575,0x5454,0x8575,0x85D4,0x95C4,0xB485,0xC495,0x4575,0x4646,0x4644,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646
.word 0x7554,0x5475,0x8585,0x85D4,0x9575,0x85D4,0x6575,0x7575,0x6575,0xD485,0x5495,0x5465,0x6554,0x7554,0x6475,0x6554,0x7554,0x7575,0x5454,0x5454,0x5475,0x5475,0x7554,0x5454,0x5454,0x5454,0x7554,0x9F5F,0x4144,0x4144,0x4F7F,0x4F7F,0x6F9F,0x4F8F,0x7F9F,0x4F9F,0x9F6F,0xD444,0x4F9F,0xEF9F,0xAF6F,0xE4C4,0x64E4,0xD4D5,0x8F6F,0x7F7F,0x6F6F,0x7F4F,0x7F7F,0xD564,0xD564,0xD474,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4F4,0xF5F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xC4E4,0x6FAF,0x54D5,0x8F5F,0x7F4F,0x7F7F,0x7F4F,0x7F4F,0x6F7F,0x6F7F,0x6F4F,0x7F6F,0x6F6F,0x5444,0x5454,0x7575,0x6585,0x7554,0x7585,0x7575,0x5475,0x8575,0xD4D4,0xD495,0xB495,0x5475,0x5475,0x5454,0x7554,0x5475,0x5454,0x7554,0xB485,0x7575,0x8585,0x8575,0x85C4,0x85B4,0xB565,0x5465,0x7554,0x9585,0x7575,0xC575,0x8585,0x5475,0x8585,0x8585,0x85C4,0x8585,0x8585,0x7575,0x7575,0x7575,0x7575,0x7575,0x8585,0x8585,0x95D4,0xB5D4,0x95C4,0x85C4,0x85B4,0x8575,0xB485,0x8585,0x7585,0x7575,0x7575,0x7585,0x5454,0x7554,0x8585,0xB495,0x5485,0x4645,0x4446,0x4646,0x5F4F,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4F4F,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x5F44
.word 0x7554,0x5454,0x8575,0x6585,0xC485,0xC4B5,0x5475,0x5475,0x7554,0x8585,0x75B4,0x8585,0x7585,0x5475,0x7575,0x64C4,0x7554,0x7554,0x5454,0x5454,0x5454,0x5454,0x4454,0x5441,0x4454,0x4454,0x5454,0x4F8F,0x8F8F,0x4F8F,0x4F7F,0x4F7F,0xF664,0x5F8F,0x4F9F,0x8F4F,0x8F5F,0x9F6F,0x8F4F,0x64C4,0xF4D4,0xD464,0x8FEF,0xAF5F,0x7F6F,0x4F7F,0x6F7F,0x4F6F,0x5F7F,0xBF6F,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE5E4,0xF4F4,0xF5F4,0xF4F4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xBF6F,0x8F6F,0x8F4F,0x7F4F,0x7F6F,0x7F6F,0x7F6F,0x6F4F,0x7F6F,0x7F6F,0x6F6F,0x8554,0x6585,0x5454,0x5454,0x5454,0x8575,0x7585,0x7575,0x7554,0xC495,0x7575,0x7554,0x7554,0x6554,0x7554,0x5485,0x5475,0xB485,0x6575,0x7554,0x7575,0x8575,0xC475,0xB495,0x7554,0x5485,0x8554,0x94D4,0xD4D4,0x7575,0xD495,0xD4D4,0x7585,0xC454,0xD495,0xD4B5,0xD4C5,0x95D4,0x95C4,0xB5C4,0x95C4,0xC5D4,0xC5E4,0x85B4,0x95B4,0xC4C5,0xD4C5,0x85D4,0x85C4,0x8575,0x7585,0x7585,0xB485,0x7585,0x8585,0x8575,0x5475,0x5454,0x7575,0x8565,0x5475,0x4645,0x4446,0x4646,0x4F5F,0x4646,0x5F4F,0x4446,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4644,0x4F5F,0x4646,0x4646
.word 0xD485,0x7595,0x5475,0x5454,0x7565,0x5475,0x8575,0xC585,0x5485,0x7554,0x5454,0x7575,0x7575,0x7575,0x7554,0x5454,0x5454,0x5454,0x5454,0x6554,0x5454,0x5475,0x5454,0x5475,0x5454,0x5454,0x4454,0x8F4F,0x4144,0x7F7F,0x7F4F,0x5441,0x4FAF,0x6F8F,0x8F6F,0x5F7F,0x5F8F,0xCFAF,0x6441,0x9FDF,0xD564,0x64D4,0x5F9F,0xE4E4,0x7F4F,0x7F7F,0x4F7F,0x7F7F,0x6441,0xD4D5,0xD464,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D5,0xF4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xD4C5,0xD5E4,0xE4E4,0xD4D4,0xE4E4,0xC4D4,0x7F9F,0x4F8F,0x4F7F,0x4F7F,0x6F6F,0x6F6F,0x7F6F,0x7F4F,0x7F6F,0x6F6F,0xBF6F,0x7585,0x8575,0xC495,0x7585,0x6554,0x8585,0x8554,0x6585,0x8575,0x7565,0x8585,0x7565,0x9575,0xB485,0x7575,0x8554,0x5475,0x8554,0x8585,0x5475,0x6554,0x6585,0x5475,0x7554,0x8585,0x85C4,0xB485,0x9595,0x5475,0x95C4,0xC495,0x6585,0x9554,0xD4C4,0x95D4,0xC5D4,0xD4D4,0xC4C5,0xE495,0xD495,0xD4C5,0x8585,0xC495,0xE495,0x95C4,0x95D4,0x85C4,0x8575,0x75B4,0xB485,0xB485,0x8585,0x7575,0x7585,0x6575,0x7554,0x6554,0x9585,0x6585,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4644
.word 0xC485,0x5485,0xD485,0x95C5,0x5454,0x5454,0x8575,0x85B4,0x8565,0xD4D4,0x7585,0x5454,0x5465,0x5475,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5454,0x5454,0x4475,0x4454,0x5454,0x5454,0x8F5F,0x4441,0x4F8F,0x4F7F,0x7F7F,0xAF4F,0x41E4,0x9F4F,0x8F4F,0x8F4F,0x6444,0x9F5F,0xDF8F,0x6454,0x64D5,0xCFAF,0x6F9F,0xAF6F,0x8F4F,0x6F4F,0x6F6F,0x7F4F,0xAF6F,0xD4C4,0xDFBF,0xD4D4,0xD4E4,0xC5E4,0xE4E4,0xE4E4,0xE4E5,0xF4F5,0xF4F4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xC5D4,0xD4D4,0xD4D4,0xAF6F,0x5454,0x7F8F,0x6F6F,0x7F4F,0x4144,0x6F7F,0x4F6F,0x6F6F,0x6F6F,0x8F7F,0x7585,0x8575,0x85B4,0x85C4,0x5454,0xB454,0x8575,0x5475,0x7585,0x8595,0x85B4,0x8585,0x6585,0x9575,0x85E4,0x5475,0x7575,0x8585,0x75B4,0x7554,0x7585,0x5485,0x5475,0x6585,0x5454,0x6575,0x7575,0x8575,0x7575,0xB485,0x7575,0x5454,0xC475,0xC595,0xC5E4,0x95C4,0x95C4,0x95C4,0x95D4,0xC5C4,0xD4C4,0xC495,0xC5D4,0xB5C5,0x95C4,0x95C4,0x75B4,0x8575,0x8585,0x85B4,0x8585,0x7575,0x8575,0x8575,0x5465,0x5454,0x8554,0xC475,0x7585,0x4446,0x4646,0x4646,0x4646,0x5F4F,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4644,0x4646,0x4446,0x4F4F,0x4646
.word 0x7565,0x5454,0x8575,0xB485,0xD475,0xC495,0x5454,0x5475,0x8575,0x7585,0x7575,0x5485,0x5475,0x5454,0x5454,0x7554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5444,0x4446,0x4446,0x4446,0x4144,0x7F8F,0x8F4F,0x7F4F,0x4F7F,0x4F7F,0x6454,0x8F5F,0x9F5F,0x8F5F,0x4441,0x9F8F,0x8F5F,0xD444,0x6F9F,0x5F9F,0x4FAF,0xF554,0x8FCF,0x6F4F,0x6F7F,0x6F4F,0x4F7F,0xD564,0xD464,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xC4D4,0x6F8F,0x4F7F,0x7F7F,0x6F6F,0x7F4F,0x4144,0x6F6F,0x6F6F,0x6F6F,0x7F6F,0x7575,0x7575,0x8554,0x8585,0x85D4,0x5465,0x8554,0x7554,0x8575,0x7565,0x8585,0xB475,0x5485,0x7575,0x7554,0xE485,0x85D4,0x7575,0x7554,0x7575,0x7575,0xC595,0x8585,0x8585,0x85C4,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x8575,0x8585,0x85C4,0x95B4,0xD4C4,0xD4B5,0xE495,0xC4C5,0x95E4,0xC5C4,0xB5C4,0xC5E4,0x65C4,0xC485,0x5475,0x7575,0x7585,0xB485,0x7585,0x7575,0x7575,0x7575,0x5454,0x5454,0x7554,0xB485,0xD485,0x85B5,0x4654,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4644,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0x5475,0x7575,0x6575,0x7575,0x8554,0x85B4,0xC485,0x8595,0x7554,0x7575,0x7554,0x5475,0x5454,0x5454,0x5475,0x5454,0x5454,0x5454,0x4444,0x4646,0x4446,0x6F4F,0x4646,0x4F5F,0x4646,0x4646,0x7F4F,0x7F4F,0x7F4F,0x7F4F,0x4F7F,0xD4D5,0x5F8F,0x4F8F,0x4F8F,0x8F7F,0x8F4F,0x7F4F,0x6441,0x7F9F,0x5F9F,0x9F5F,0xAF6F,0x8F6F,0x7F7F,0x7F7F,0x4F7F,0x6F7F,0xDFAF,0xD4D4,0xD4D4,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4E4,0xD4C5,0xD4E4,0xD4E4,0xD4D4,0x64D4,0x6F9F,0x7FAF,0x7F4F,0x6F4F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F6F,0x8575,0x8575,0x7575,0x85B4,0xC485,0x5485,0x5454,0x5454,0x8565,0x5475,0x6575,0x7575,0x5475,0x5475,0x6575,0x8554,0xD495,0xD4D4,0x7585,0x7554,0xE485,0xC495,0xC485,0xC495,0xD4B5,0x8595,0x7585,0x5475,0x5475,0x5475,0xC485,0xD4B5,0xD4C5,0xE4C5,0xD4C5,0xD4C5,0xD495,0xC4C5,0xC495,0xD4B5,0xD4C5,0xD495,0xC4C5,0x5475,0x7585,0x7554,0x8575,0x7585,0x8585,0x7575,0x5454,0x5454,0x5454,0x5454,0x7554,0x8585,0x95C4,0xB485,0xD485,0x7585,0x4645,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4F5F,0x4F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4F5F,0x4646,0x4F4F,0x4646,0x4644
.word 0xC495,0x5485,0x8585,0x5465,0x5475,0x5475,0x7565,0x6575,0x8585,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4646,0x4446,0x6F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F5F,0x4F6F,0x4F8F,0x4F7F,0x4F7F,0x5F8F,0x5F9F,0x5F8F,0x4F9F,0x7F5F,0x7F4F,0x8F4F,0x7F4F,0x8F5F,0x9F4F,0x9F4F,0x8F5F,0xC4E4,0x4F8F,0x4F6F,0x6F7F,0x7F6F,0xD541,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4D5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0x64D4,0x7FAF,0x7F5F,0x7F4F,0x7F6F,0x6F7F,0x6F7F,0x7F7F,0x6F7F,0x6F6F,0x7F7F,0x7585,0x7585,0x5465,0x7575,0x8585,0x95C4,0x65C4,0x5454,0x8575,0x7565,0x7575,0x5475,0x8575,0x6554,0x7575,0x6575,0x8554,0x9595,0xD4D4,0xD4B5,0xB4B5,0xB485,0xC4B5,0xD4B5,0xB5D4,0xC5C5,0xD495,0xC495,0xC485,0x9585,0x95C4,0x95C4,0x95C4,0x95B4,0x95C4,0x95D4,0xD4C4,0xC495,0xB5E4,0xD4D4,0xC5D4,0xC5D4,0x7595,0x8554,0x5475,0x5475,0xB485,0x7585,0x8575,0x7565,0x5475,0x4454,0x5454,0x7554,0x7585,0x8585,0x7585,0x9585,0x95B4,0x85C4,0x7585,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646
.word 0x8585,0x7565,0x95C4,0xC495,0x8554,0x8585,0x5454,0x5454,0x7585,0x5475,0x5454,0x5454,0x5475,0x5454,0x4454,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4F5F,0x4646,0x7F4F,0x7F4F,0x8F4F,0x7F4F,0x7F7F,0x7541,0x5FAF,0x4F8F,0x4F8F,0x8F8F,0x7F4F,0x8F4F,0x4F7F,0x4F9F,0x5F9F,0x4F8F,0x6F8F,0xD454,0x7F4F,0x7F6F,0x6F4F,0x7F4F,0xD454,0xBF7F,0x64D4,0xD4E4,0xD4C5,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xF4D5,0xE4E4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4C5,0xD4D4,0xD4D4,0x9F6F,0x8F6F,0x4F7F,0x6F6F,0x4F7F,0x6F7F,0x6F6F,0x6F4F,0x4F6F,0x9F6F,0x75C4,0x7565,0x7585,0x5475,0x7575,0x7565,0x7554,0x5475,0x8554,0x7575,0x5454,0x7575,0x5475,0x7554,0x7554,0x7575,0x6575,0xB485,0xC495,0xB485,0xC485,0xB495,0xB495,0xB5D4,0xD4D4,0xD4B5,0xD495,0x85D4,0xE4B5,0xD4D4,0xD4B4,0xC4C5,0xC4C5,0xC4C5,0xD4C5,0xE495,0xE4C5,0xD4C5,0xD495,0x95D4,0xD4C5,0xD495,0x6595,0x8575,0x7575,0x8554,0x7585,0xB485,0x6575,0x7564,0x7585,0x7554,0x7554,0x7575,0x7565,0x6565,0x7575,0x75B4,0x7575,0x7585,0x7585,0x4554,0x4646,0x4646,0x5F4F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F4F,0x4446,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646
.word 0x6554,0x5475,0xB485,0x8585,0x8565,0x7575,0x8575,0x6585,0x6554,0x5454,0x5454,0x5454,0x5454,0x4644,0x6F4F,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F7F,0x7F8F,0x4F7F,0x7F4F,0x8F5F,0x8F5F,0x9F5F,0x8F4F,0x7F7F,0x4F7F,0x4F7F,0x4F7F,0x8F4F,0x4441,0x4F8F,0xD544,0x5FAF,0x7F8F,0x4F7F,0x7F6F,0x4F6F,0xD4D4,0x85D5,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xF4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD5E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x64D5,0x41D5,0x7F7F,0x4F7F,0x7F7F,0x7F7F,0x4F7F,0x7F6F,0x6F6F,0x8F6F,0x7585,0x7585,0x5485,0x7554,0x7585,0x8575,0x8565,0x5454,0x5454,0x7575,0x5454,0x7554,0x7554,0x5454,0x7575,0x7585,0x7575,0x7575,0x7575,0x7575,0x7575,0x7575,0x8585,0xD4D4,0xE4B5,0x95C4,0xC5E4,0xC5D4,0x95C4,0xB5D4,0xB5C4,0xD4D4,0x95D4,0xD4D4,0x95D4,0xC4D4,0xD495,0xC495,0xC5D4,0x95D4,0x85D4,0xC5B4,0x5475,0x7554,0x5475,0xC475,0x8575,0x7575,0x5454,0x7575,0x5475,0x8575,0xD495,0x7585,0x4446,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4644,0x4F5F,0x4F5F,0x4F4F,0x4F4F,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646
.word 0xD495,0x8585,0x5465,0x5475,0x5465,0x5475,0x7554,0x5475,0x6475,0x5454,0x5454,0x5454,0x4644,0x4644,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4644,0x4646,0x7F4F,0x4F7F,0x4441,0x7F7F,0x4F7F,0x5F9F,0x9F6F,0x8F5F,0x7F4F,0x4144,0x4F7F,0x7F7F,0x8F4F,0x8F4F,0x8F4F,0x8F4F,0x5441,0x5F9F,0x6F8F,0x6F6F,0x7F6F,0x5F7F,0xCFBF,0xD474,0xD4D4,0xD4D4,0xE4D4,0xC5E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xD5E4,0xE4E4,0xE4E4,0xE4D4,0xE4C5,0xE4D4,0xD4C5,0xD4D4,0xBF7F,0x5464,0x7F5F,0x7F6F,0x7F7F,0x6F4F,0x6F6F,0x6F7F,0x7F6F,0x6541,0x5475,0x5475,0x7575,0x5454,0x8575,0x7585,0x8575,0x7585,0x5454,0x5454,0x8585,0x7575,0x7575,0x8575,0x7585,0x8585,0x5485,0x7575,0x7554,0x8585,0x8585,0xC485,0xB485,0xB495,0xD495,0xC4B5,0xD4C5,0xB5D4,0xC5D4,0x95D4,0xD4D4,0xD4B5,0xC495,0xC5C5,0xC5E4,0xC5E4,0xC5E4,0xC5E4,0xD4D4,0xB4B5,0x8575,0x75D4,0x6554,0x8575,0x5475,0x8575,0xB485,0x5454,0x5454,0x7585,0x5475,0xB485,0xC495,0x75B4,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x5F4F,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x4446
.word 0x8585,0x75B4,0x8575,0x8585,0x7575,0x8575,0x6585,0x7554,0x5454,0x5454,0x5454,0x4654,0x4644,0x4F5F,0x5F4F,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4F5F,0x4F6F,0x4F7F,0x7F7F,0x7F4F,0x5441,0x6F9F,0x4F9F,0x4F8F,0x7F8F,0x7F7F,0x7F4F,0x7F7F,0x8F4F,0x8F4F,0x8F4F,0x4F8F,0x51D5,0x6F8F,0x6F7F,0x7F7F,0x6F7F,0x6441,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D5,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xAFDF,0x4154,0x7F7F,0x6F7F,0x4F6F,0x6F6F,0x7F6F,0x4F6F,0x7F6F,0x7575,0x5454,0x5444,0x5454,0x7554,0x75C4,0x7585,0xB485,0x7585,0x5454,0x5454,0x5454,0x7554,0x5475,0x6575,0x7575,0x6575,0x7554,0x7575,0x7565,0x7565,0x5475,0x8575,0xD495,0x95D4,0xD4C4,0xB495,0xD485,0xD4D4,0xD4C5,0xD4C5,0xB5D4,0x85C4,0x85C4,0x85B4,0x85B4,0x85B4,0x75B4,0x7575,0xE485,0xB495,0x5454,0x5475,0xC454,0x5485,0x8575,0x8575,0x7575,0x8575,0x7575,0x7554,0xC485,0xB495,0x85D4,0x4644,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4446,0x5444,0x5454,0x5454,0x4454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4644,0x4646,0x4646,0x5F4F,0x4646,0x5F4F,0x4446,0x4646
.word 0x7575,0x5475,0x9585,0xD4C4,0x5485,0xB485,0x7585,0x5454,0x6554,0x5454,0x4454,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x5F4F,0x4144,0x4F7F,0x7F7F,0x4F7F,0x4F7F,0xAF5F,0x7F4F,0x8F5F,0x4144,0x7F7F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x7F4F,0x4141,0xAF6F,0x9F4F,0x7F6F,0x6F6F,0x4F6F,0xD541,0xD574,0xD474,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4C5,0xD5D4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x4164,0x7F9F,0x4144,0x4F6F,0x7F6F,0x6F4F,0x6F7F,0x7F4F,0x8544,0x7575,0xD485,0x8595,0x7585,0x5454,0x8575,0x7575,0x7575,0x8575,0x7585,0x4475,0x5454,0x7575,0x8575,0x95C4,0x8585,0x7554,0x8585,0x8575,0x8585,0x8585,0xB485,0xB495,0x7565,0x8585,0xC495,0xC5C5,0xB4B5,0xD495,0xB5D4,0x95C4,0x95C4,0x95D4,0x85B4,0x7575,0x7575,0x8575,0x8585,0xC495,0x7595,0x5465,0x5454,0x5454,0x8575,0x54B4,0x8565,0x85B4,0x8585,0x75C4,0x5475,0x8554,0x94C4,0x95B4,0xB495,0x6585,0x4646,0x4446,0x5F4F,0x4646,0x4644,0x4446,0x6544,0x5454,0x7575,0x8585,0x9595,0xD4C5,0x9595,0x95B5,0xB595,0xC5B5,0xB595,0x9595,0x7575,0x6475,0x5454,0x7454,0x4446,0x4646,0x4646,0x465F
.word 0x95D4,0x8595,0xB475,0xB495,0x6585,0x7575,0x5454,0x5454,0x5475,0x4454,0x6F4F,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x5F4F,0x4646,0x4F5F,0x5F4F,0x4646,0x4F5F,0x4646,0x4644,0x7F4F,0x7F7F,0x7F4F,0x7F4F,0x7F4F,0x5441,0xCFAF,0x5F7F,0x4F9F,0x7F7F,0x4F7F,0x7F7F,0x4F7F,0x4F8F,0x4F7F,0x7F7F,0x9F4F,0x5454,0x4F8F,0x6F7F,0x6F6F,0x7F6F,0xD564,0xCFBF,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xD474,0x54D5,0x7F6F,0x7F4F,0x6F7F,0x7F7F,0x7F6F,0x4F6F,0x7F7F,0x7575,0x7575,0x8575,0x85C4,0x95B4,0x75C4,0x5454,0x9575,0x7585,0x6575,0x7575,0x7585,0x7554,0x8585,0x5475,0x7554,0x7575,0x8575,0x6575,0x5475,0x6575,0x7575,0x9585,0x95C4,0xC495,0x7595,0x7575,0x8554,0x85B4,0x7575,0x7575,0xB485,0xB495,0xC485,0x8585,0x9585,0xC5C4,0x95C4,0x85B4,0x5465,0x5454,0xC495,0xC495,0x5475,0x7575,0x8585,0x5465,0x7575,0x7585,0x5465,0x6554,0xC485,0xC495,0xB495,0x9585,0x85C4,0x5475,0x4646,0x4646,0x5444,0x8565,0x8585,0xC595,0xB595,0xD495,0xB495,0xB485,0x8585,0x6575,0x6575,0x7575,0x6565,0x7575,0x7565,0x8585,0x8585,0x8585,0x8585,0x9595,0x7575,0x5454,0x4665
.word 0xB485,0xB485,0x7554,0x7554,0x5454,0x7554,0x5454,0x6554,0x4454,0x4454,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4F4F,0x7F4F,0x7F4F,0x7F7F,0x4F7F,0x4F7F,0xAF6F,0x5F7F,0x8F5F,0x7F4F,0x7F7F,0x6F7F,0x7F7F,0x7F4F,0x7F7F,0x8F4F,0x6441,0x4F9F,0x4F8F,0x6F6F,0x4F7F,0x4F6F,0x64C4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xD5D4,0xE4E4,0xD4E4,0xD4E4,0xE4C5,0xE4D4,0xD4D4,0xAFDF,0x8F6F,0x8F4F,0x7F4F,0x6F4F,0x6F7F,0x6F7F,0x6F4F,0x7565,0x7575,0x8575,0x9585,0x95C4,0x85D4,0x5475,0x5444,0x9575,0x7575,0x8575,0x5465,0x7554,0x7575,0x8575,0x5475,0x5475,0x5454,0x5475,0x5475,0x8575,0x7585,0x85B4,0xB485,0x8585,0x85B4,0x95C4,0x85D4,0x7585,0x8585,0xB485,0x7585,0x7575,0x7575,0xB485,0xC485,0xC485,0x7575,0x5454,0x4641,0x5444,0x95E4,0x95B4,0x5475,0x7575,0x8575,0x8585,0x5465,0x5454,0x7554,0x8585,0x8585,0x85B4,0x8585,0x85C4,0x7585,0x7585,0x5454,0x7554,0x8575,0xB595,0x95D4,0x85C4,0x75B4,0x7575,0x5454,0x5454,0x6554,0x4444,0x4446,0x4446,0x4446,0x4446,0x4646,0x5444,0x6554,0x7554,0x7575,0x7575,0x8585,0x8585,0x7585
.word 0x5454,0x5465,0x8575,0x95C4,0x95C4,0x7565,0x6554,0x6564,0x4154,0x4646,0x4646,0x4644,0x4646,0x4646,0x4F5F,0x5F4F,0x4446,0x4F5F,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x7F4F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x9F4F,0x8F7F,0x9F4F,0x7F8F,0x7F7F,0x7F4F,0x7F4F,0x7F4F,0x7F4F,0x7F4F,0x4F7F,0x9F4F,0x8F7F,0x7F7F,0x7F6F,0x6F6F,0x5F7F,0xBFDF,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4C5,0xE4D4,0xE4E4,0xD5E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4C5,0xE4D4,0xD4D4,0x64D4,0xC4D4,0x4FAF,0x7F8F,0x7F7F,0x6F6F,0x6F7F,0x4F7F,0x4F7F,0x8595,0x9595,0x85D4,0x75B4,0xC485,0x8585,0x95B4,0x7585,0x5454,0x7554,0x5475,0x5475,0x5454,0x7575,0x7575,0x5465,0x8575,0xB485,0x7554,0x7575,0x5475,0x7554,0x7585,0x8585,0x5475,0x4646,0x4444,0x6554,0x8585,0xB475,0x7585,0x7585,0x8585,0x8585,0x7575,0x5475,0x4654,0x4644,0x4646,0x6F4F,0x4446,0xD4D4,0xB495,0x4454,0x5475,0x8554,0x95B4,0x95E4,0x85C4,0x7585,0xB485,0x8585,0xC485,0x95B4,0x85B4,0x95B4,0x95D4,0xC495,0x9595,0xB495,0xB485,0x7554,0x7565,0x5454,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x4F5F,0x5F4F,0x4446,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x5454,0x7565
.word 0x95D4,0x95C5,0x7554,0x8585,0xB485,0x5464,0x5454,0x5454,0x4654,0x4646,0x4F5F,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4644,0x4446,0x4646,0x4F6F,0x4F7F,0x7F7F,0x7F7F,0x7F4F,0x6454,0x7FDF,0x4441,0x4F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4F7F,0x4F7F,0x5444,0x7F7F,0x6F4F,0x6F6F,0x7F6F,0x6441,0x64D5,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD5D4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x64D4,0x51D5,0x7F7F,0x6F6F,0x6F7F,0x6F6F,0x7F4F,0x7541,0xE4D4,0xF4D4,0xC485,0x5485,0x7575,0x8565,0xB485,0x6585,0x5475,0x7575,0x8585,0x5475,0x7554,0x5475,0x7575,0x8575,0x7575,0x7554,0x6475,0x7585,0x5475,0x8565,0xE4C5,0x85B5,0x4665,0x4F6F,0x4646,0x4446,0x4444,0x4454,0x4454,0x5454,0x6575,0x6575,0x5454,0x4646,0x4644,0x4646,0x4F5F,0x4646,0x5446,0x9595,0x5475,0x4644,0x4644,0x6554,0x8575,0xD4C4,0xD4B5,0xC495,0xD4C5,0xC5B5,0xC595,0xA5D4,0xC4C5,0x8585,0x7575,0x8575,0xB485,0x7565,0x5454,0x5454,0x4646,0x4446,0x6F4F,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4F5F,0x4F5F,0x4446,0x4646
.word 0xC485,0xB485,0x5475,0x7575,0x7575,0x5454,0x5454,0x4454,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x6F4F,0x7F4F,0x4F7F,0x7F7F,0x6F4F,0x7F7F,0xAF5F,0x7F5F,0x4F8F,0x8F4F,0x4F7F,0x7F4F,0x6F6F,0x7F4F,0x7F7F,0x7F6F,0x8F4F,0x8F5F,0x7F7F,0x6F7F,0x6F7F,0x4F6F,0xD454,0xCFBF,0x64D4,0xE4D4,0xD4D4,0xE4C5,0xE4E4,0xD4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xD5D4,0xE4E4,0xD4E4,0xD4C5,0xD4D4,0xBFDF,0x54C4,0x4154,0x4F7F,0x6F7F,0x6F7F,0x6F4F,0xEF7F,0xD4E4,0xD4D4,0x75E4,0x8565,0x5444,0x7585,0x8585,0x95C4,0x7585,0x7575,0x7575,0x5475,0x5454,0x7575,0x7585,0x8575,0x7575,0x7575,0x7575,0x5465,0x8554,0xD495,0x95D4,0x5475,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4444,0x8575,0x8585,0x5475,0x4641,0x4644,0x5F4F,0x4646,0x4646,0x4644,0x8546,0x5475,0x4646,0x4646,0x5F4F,0x4646,0x5444,0x7575,0x7554,0x6575,0x6575,0x6575,0x6575,0x6575,0x7575,0x7565,0x5465,0x5454,0x5454,0x4446,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x465F
.word 0xB485,0x8585,0x7554,0x6554,0x7575,0x5454,0x5454,0x4154,0x4646,0x4646,0x4F5F,0x4644,0x4446,0x4446,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4F5F,0x4644,0x4646,0x5F4F,0x6F4F,0x7F4F,0x7F4F,0x7F7F,0x4F7F,0x7F4F,0x5454,0x4F9F,0x8F4F,0x4144,0x7F7F,0x7F7F,0x6F7F,0x7F7F,0x6F4F,0x6F7F,0x4441,0x7F9F,0x7F7F,0x4F6F,0x6F6F,0x4F7F,0xD464,0xD4C4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E5,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0x74D4,0xD4D4,0xBFDF,0x8F5F,0x6F4F,0x7F7F,0x7F6F,0x6F7F,0xF444,0xD4D4,0xD451,0x54F4,0x5454,0x5475,0xD454,0x8575,0x7575,0x7575,0x8554,0x8585,0x5475,0x5454,0x5485,0x8575,0x7554,0x8585,0x8585,0x8585,0x5454,0xD485,0x95D4,0x7585,0x4654,0x4644,0x4646,0x4646,0x4644,0x4F5F,0x4644,0x5444,0xE4C5,0x85D4,0x4475,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x5F4F,0x5446,0x4645,0x5F4F,0x4446,0x4646,0x4646,0x4646,0x4644,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4654,0x4644,0x4446,0x4646,0x4646,0x4F6F,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x4646,0x4F5F,0x4446,0x4646,0x4644,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4644,0x4646,0x4F5F,0x4446,0x4646,0x4646
.word 0x5465,0x7575,0x5485,0x8575,0x7585,0x6554,0x5454,0x4654,0x4446,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4F6F,0x4641,0x7F6F,0x7F4F,0x7F7F,0x7F4F,0x9F4F,0x7F5F,0x4F8F,0x7F8F,0x7F4F,0x4F7F,0x7F6F,0x7F4F,0x6F7F,0x7F7F,0x8F4F,0x7F4F,0x7F6F,0x7F6F,0x6F7F,0x5F7F,0xD4D5,0xDFBF,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D5,0xD4E4,0xE4D5,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0x8FBF,0x64D4,0x9F6F,0x7F6F,0x7F6F,0x4F6F,0x7F6F,0xEF7F,0x6454,0xDF9F,0x85E4,0x8585,0x6585,0x7554,0xB485,0x7554,0x7575,0x7575,0x85B4,0x7575,0x5454,0x6575,0x8554,0x6585,0x6575,0x8565,0x5475,0x6554,0x5454,0x5454,0x4554,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x8544,0x6585,0x4454,0x4446,0x5F4F,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4646,0x4654,0x4646,0x4646,0x4646,0x4F5F,0x4644,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x6F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4644,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4644
.word 0x8554,0xB495,0xB495,0x8575,0x75B4,0x5454,0x5454,0x4654,0x4646,0x4F5F,0x4F5F,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4F4F,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4644,0x4F7F,0x6F7F,0x7F4F,0x7F7F,0x4F7F,0x4F7F,0x5F8F,0x7F5F,0x8F4F,0x7F4F,0x7F7F,0x6F7F,0x6F7F,0x7F7F,0x6F7F,0x4F7F,0x4F7F,0x7F4F,0x6F6F,0x6F6F,0x7F6F,0x6444,0xCFBF,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xE4D5,0xD5E4,0xE4E4,0xC5E4,0xD4E4,0xD4D4,0xD4D4,0xD5D4,0x5FAF,0x7F4F,0x7F4F,0x6F6F,0x7F7F,0x5441,0x6F9F,0xE464,0x95E4,0x85B4,0x85D4,0x5454,0x8554,0x7585,0x7575,0x7575,0x7575,0x7575,0x4475,0x7554,0x5475,0x6465,0x8575,0x7585,0x5475,0x8575,0x6585,0x7554,0x7585,0x4464,0x4646,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x5454,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x5465,0x4644,0x4644,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x6F4F,0x4646,0x4F6F,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4644,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646
.word 0x7554,0x8585,0x85B4,0x7554,0x7554,0x5475,0x5454,0x4644,0x4F6F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4F6F,0x7F7F,0x7F7F,0x7F4F,0x7F7F,0x8F4F,0x8F4F,0x7F4F,0x8F4F,0x7F7F,0x7F4F,0x6F7F,0x6F4F,0x6F4F,0x6F4F,0x6F6F,0x4141,0x7F8F,0x6F7F,0x4F6F,0x7F6F,0xD564,0xD474,0xD4D4,0xD474,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4C5,0xE4D4,0xE4D4,0xD4D4,0xE4D5,0xD4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x64D4,0x64D4,0x9FCF,0x8F5F,0x7F6F,0x7F6F,0x6F4F,0x5454,0xDFAF,0xE464,0x85E4,0x8575,0x85C4,0x75C4,0x6554,0x8585,0x7585,0x7554,0x7575,0x7585,0x8575,0x5454,0x5454,0x5454,0x5475,0x7565,0x8575,0xB485,0x5475,0x5475,0x8575,0x84C4,0x7585,0x5464,0x4645,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4446,0x5F4F,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4446,0x7544,0x7575,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4F5F,0x4446,0x4F4F,0x4646,0x5F4F,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x4644,0x4F5F,0x4646,0x4646,0x4646
.word 0x6585,0x7585,0x8585,0x8565,0x5485,0x5475,0x5454,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x4644,0x5F4F,0x4646,0x4F5F,0x4644,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x5F4F,0x7F4F,0x7F4F,0x7F4F,0x4F7F,0x4F6F,0x5F7F,0x5F9F,0x7F8F,0x4441,0x7F7F,0x7F7F,0x6F4F,0x6F7F,0x7F7F,0x7F6F,0x7F7F,0x8F4F,0x6F4F,0x6F4F,0x6F7F,0x7F6F,0xD464,0xCFAF,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4C5,0xD4D4,0xC5D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xC5D4,0xD4D4,0xD4D4,0xCFBF,0x8FBF,0x4F9F,0x7F7F,0x6F6F,0x4144,0x9F5F,0xD4C4,0xCFAF,0x75F4,0x7575,0x8575,0x8585,0x5475,0x6554,0xC485,0x8595,0x8575,0x8554,0xD485,0x7585,0x5475,0x4454,0x7554,0x7585,0xB485,0x7575,0x7575,0x5475,0x7554,0x8554,0x9585,0x95A5,0x7585,0x6575,0x4445,0x4444,0x4444,0x4444,0x6544,0x8575,0x7585,0x7575,0x75A4,0x7564,0x4644,0x4646,0x5454,0x7575,0x4664,0x4F5F,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4644,0x4644,0x4446,0x5F4F,0x4646,0x4646,0x4446,0x4646,0x4446,0x4454,0x5F4F,0x4446,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x5F46
.word 0x8585,0x6575,0x7554,0x7554,0x5475,0x5454,0x5454,0x4446,0x4646,0x4644,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4146,0x7F7F,0x6F4F,0x4F7F,0x7F7F,0x7F4F,0x4441,0x8F5F,0x7F5F,0x4F8F,0x7F7F,0x6F4F,0x6F7F,0x6F4F,0x4F7F,0x7F6F,0x4F6F,0x5F7F,0x6F7F,0x6F7F,0x6F7F,0x5F7F,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4C5,0x5464,0x4F9F,0x5F9F,0xCFBF,0xD564,0xD564,0x5464,0x7F7F,0x7F4F,0x7F7F,0xD564,0xD464,0xE464,0x95C4,0x8585,0x7575,0xB485,0x8585,0x6585,0x5454,0x5465,0x8575,0x75B4,0x5475,0xD485,0xC4B5,0x5485,0x5454,0x8575,0x7575,0x7575,0x5475,0x5454,0x7575,0x7575,0x7565,0x7554,0x5465,0x5475,0x7554,0x7575,0x7575,0x7575,0x7575,0x6575,0x7575,0x7575,0x5464,0x5465,0x9595,0x7564,0x5454,0x5454,0x5454,0x4646,0x4F5F,0x4646,0x4F4F,0x4646,0x6F5F,0x5475,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x6544,0x7575,0xA595,0x6585,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4646
.word 0xB485,0x7554,0x8585,0x7575,0x5475,0x5454,0x5454,0x4646,0x4F5F,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F4F,0x4446,0x4646,0x6F4F,0x7F4F,0x7F4F,0x7F7F,0x6F4F,0x7F4F,0x9F4F,0x8F5F,0x7F4F,0x8F4F,0x7F7F,0x7F4F,0x6F7F,0x7F7F,0x7F7F,0x4F6F,0x4F6F,0x5F7F,0x6F7F,0x6F7F,0x6F7F,0x5441,0xCFBF,0x8FBF,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4E4,0x64D4,0x9F5F,0x6F8F,0x6F6F,0x7F4F,0x5441,0xCFAF,0x6FBF,0x5FAF,0x7F7F,0x4F7F,0x5F7F,0xD4D4,0xEFAF,0xE4D4,0x5485,0xB475,0xC4B5,0x9585,0xD4D4,0xC4B5,0x5485,0x8575,0x95C4,0x7575,0x8575,0x8575,0x95C4,0x54C4,0x7554,0x7575,0x8575,0x7554,0x8585,0x7585,0x6554,0x7554,0x7554,0x7554,0x6585,0x7585,0x7575,0x7575,0x7585,0x7575,0x8575,0x7585,0x7554,0x7575,0x7575,0x5475,0x5475,0x8575,0x85A5,0x5475,0x4454,0x4646,0x4646,0x4644,0x4646,0x4646,0x8565,0x9584,0x4645,0x7565,0x8554,0x74B5,0xA574,0xA595,0x8585,0x8585,0x5485,0x4446,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x465F
.word 0x7575,0x8575,0x8575,0x75B4,0x5475,0x5454,0x5454,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x6F4F,0x7F4F,0x6F4F,0x7F4F,0x7F7F,0x7F4F,0x4441,0x4F9F,0x7F8F,0x4154,0x7F4F,0x7F6F,0x6F4F,0x6F4F,0x6F4F,0x7F7F,0x7F7F,0x7F4F,0x6F7F,0x7F6F,0x4F6F,0x6F7F,0x6F6F,0x4F7F,0x5F9F,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4E4,0x54D4,0x4F9F,0x6F4F,0x7F6F,0x4F6F,0xD4C4,0xE5D4,0xD4D4,0xD4D4,0x64D4,0x6464,0x5FAF,0x7F7F,0x7F7F,0x6464,0xDFBF,0xE485,0xD4D4,0x95C4,0x75C4,0x5454,0x4454,0x6575,0x8585,0xC485,0xC495,0x7585,0x8575,0x8575,0x95B4,0xC485,0x5485,0x5454,0x7585,0x7575,0x5475,0x7575,0x5475,0x5454,0x7554,0x8575,0x85C4,0x7585,0x8585,0x9585,0x8585,0x7575,0x7575,0x8585,0x85B4,0x7585,0x7575,0x7585,0x7585,0x7575,0x5475,0x5475,0x7575,0x9585,0x6485,0x5475,0x4644,0x4644,0x7544,0x8575,0x75B4,0x9585,0x7554,0x5465,0x5475,0x5454,0x5454,0x5454,0x4645,0x4446,0x4646,0x5F4F,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x4F5F,0x5F4F,0x4646,0x4446,0x4F4F,0x4644,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646
.word 0x95E4,0x5475,0x7575,0x8575,0x5485,0x5454,0x5454,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F4F,0x4646,0x4F5F,0x7F7F,0x6F4F,0x4F7F,0x6F6F,0x4F5F,0x5F8F,0x4F9F,0x7F7F,0x9F5F,0x7F7F,0x7F4F,0x6F7F,0x6F7F,0x6F7F,0x4F6F,0x4F6F,0x4F7F,0x6F6F,0x4F7F,0x7F6F,0x64C4,0x7F6F,0x6F7F,0x6F6F,0x6F7F,0x8F7F,0xD454,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0x8F5F,0x7F6F,0x7F4F,0x7F7F,0x6F7F,0xE4D4,0xE4E4,0xE4E5,0xE5E5,0xE5D4,0xD4D4,0x5FAF,0x54D5,0x4F7F,0x4441,0xCFAF,0xD464,0xDFAF,0xE4E4,0xC495,0xC585,0xC5D4,0x7575,0x5454,0xD475,0x8585,0x8585,0x7575,0x7575,0x8575,0xB485,0x8575,0x7575,0x5444,0x8575,0x7575,0x7554,0x5475,0x8554,0x7575,0x5485,0x7554,0x8575,0x7575,0xB485,0x8575,0x7565,0x8585,0x8585,0x7575,0x5475,0x8575,0x8585,0xD495,0xC495,0x5485,0x5475,0x7575,0x5475,0x6465,0x7565,0x9585,0x85B5,0x8595,0x7575,0x9575,0x7595,0x5465,0x5454,0x5475,0x4454,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x5F4F,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x5F4F,0x4446,0x4646,0x4F4F,0x4646
.word 0xB485,0x7554,0x7554,0x8585,0x54B4,0x5454,0x5454,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4446,0x5F4F,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x7F6F,0x7F4F,0x4F5F,0x7F7F,0x4644,0x7F4F,0x4441,0x8F5F,0x7F7F,0x4154,0x7F7F,0x7F7F,0x6F7F,0x6F4F,0x6F4F,0x6F7F,0x7F7F,0x7F4F,0x7F6F,0x7F6F,0x4F6F,0xAF5F,0x5454,0x64D4,0x8F5F,0x6F6F,0x6F7F,0x7F7F,0x6F7F,0x6451,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0x4151,0x6F6F,0x6F6F,0x6F7F,0x4F6F,0x8554,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xAF8F,0x8F6F,0xAF5F,0x9F6F,0x4F7F,0x6F8F,0xD4D5,0xE4D4,0xE464,0xD4D4,0x9585,0x85C4,0x8565,0x7585,0x7585,0x5454,0x7565,0x9575,0x95C5,0x85C4,0x7575,0x8585,0x7585,0x8565,0x4475,0x7554,0x8585,0x7575,0x5475,0x8575,0x8565,0x7585,0x5475,0x7554,0x7565,0xC495,0x8575,0x7575,0xB485,0xB495,0x8575,0x7575,0x7575,0xB485,0x7575,0x7575,0x7585,0x7575,0x8585,0x7575,0x7575,0x5485,0x6565,0x7575,0x7585,0x5475,0x8575,0x95B4,0xC495,0x85B5,0x7585,0x7585,0x4654,0x4F6F,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4644
.word 0x7575,0x9575,0x75C4,0x7554,0x5475,0x5465,0x5454,0x5F4F,0x4646,0x4644,0x5F4F,0x4646,0x4644,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x5F4F,0x4446,0x4F6F,0x4F6F,0x7F7F,0x7F4F,0x4F6F,0x7F4F,0x8F4F,0x8F4F,0x7F7F,0x9F5F,0x4144,0x7F7F,0x6F4F,0x6F7F,0x7F7F,0x7F6F,0x4144,0x7F7F,0x4F6F,0x6F6F,0x7F6F,0x6F5F,0x5F5F,0x6F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8F4F,0x7F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4F6F,0x5441,0x5F9F,0x5F9F,0x7F7F,0xC444,0xD564,0xD441,0xD564,0xD474,0xE4D4,0xE4C4,0x85E4,0x95B4,0x95C4,0x85C4,0x75B4,0x7575,0x8585,0x5475,0x5454,0x7565,0x8585,0x7575,0x7575,0x7565,0xC495,0x5485,0x5454,0x8565,0x7575,0x7585,0x7575,0x7575,0x5465,0x8585,0x8575,0x7575,0x7575,0x7585,0x7595,0x8575,0x75C4,0x7575,0x7575,0x7575,0x7585,0x7575,0x8585,0xC485,0x8585,0x75B4,0x8575,0x8554,0x7585,0x6475,0x7554,0x5475,0x8575,0x7575,0x6585,0x8575,0x85C4,0x85B4,0x85C4,0x4654,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4F4F,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4F5F,0x4446,0x4446,0x4646,0x4646,0x4646,0x465F
.word 0x5454,0x7585,0x7585,0x7565,0x5454,0x5464,0x5454,0x4645,0x4646,0x4F4F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4446,0x7F6F,0x7F4F,0x4F7F,0x7F7F,0x4F7F,0x7F4F,0x8F4F,0x4441,0x7F7F,0x4154,0x7F7F,0x7F4F,0x6F7F,0x7F7F,0x6F7F,0x6F4F,0x4F6F,0x7F7F,0x7F6F,0x6F6F,0x5F7F,0x6F5F,0x5F5F,0x6F6F,0x6F6F,0x6F5F,0x6F6F,0x6F6F,0x6F6F,0x4F6F,0x8F6F,0xD4D4,0xD4D4,0xD4D4,0x74D4,0x6FAF,0x4F8F,0x7F7F,0x6F6F,0x6F6F,0x5F6F,0x5F6F,0x5F6F,0x5F5F,0x5F6F,0x5F5F,0x5F6F,0x6F6F,0x6F6F,0x5F6F,0xDFAF,0x6FBF,0xD454,0xDFAF,0xD4D4,0xE4E4,0xE464,0x75D4,0x6575,0x6575,0x8585,0x9585,0x95C4,0x85B4,0x8575,0x7575,0x7575,0x8575,0x7575,0x5475,0xC485,0x8585,0xB485,0x5475,0x5454,0x6554,0x7575,0x7575,0x7554,0x5454,0x5454,0x4454,0x5454,0x5454,0x5454,0x5454,0x7554,0x6465,0x7554,0x5475,0x5475,0x5475,0x5454,0x5454,0x5475,0x7575,0x8575,0xB495,0xB485,0x5475,0x7575,0x8585,0x85B4,0x7585,0x7575,0x7575,0x7575,0x8585,0xB485,0xB495,0x4654,0x4446,0x4646,0x4646,0x4644,0x4646,0x4F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4F4F,0x4446,0x4646,0x4646
.word 0x8554,0x6554,0x7554,0x9585,0x5475,0x5465,0x7554,0x4654,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F4F,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4644,0x4F4F,0x7F4F,0x7F7F,0x7F7F,0x4F7F,0x4F6F,0x7F4F,0x4441,0x4F8F,0x7F7F,0x8F5F,0x7F4F,0x7F7F,0x7F7F,0x6F6F,0x7F4F,0x7F6F,0x4F7F,0x7F7F,0x6F6F,0x6F4F,0x5F6F,0x6F6F,0x5F7F,0x4F5F,0x4F4F,0x6F6F,0x6F5F,0x6F6F,0x6F6F,0x6F7F,0x7F6F,0xD464,0xD4D4,0xD4D4,0xD4D4,0xAF6F,0x8F4F,0x6F6F,0x6F6F,0x5F6F,0x5F6F,0x5F6F,0xFF5F,0x5F5F,0x5F5F,0x5F6F,0x5F5F,0x5F4F,0x7F5F,0xC441,0xC4D4,0xAFCF,0xD454,0x64D4,0xE4D4,0xD4E4,0xE4D4,0x4664,0x5441,0x7554,0x6554,0x5465,0x5475,0x8585,0xC5C4,0xD4C5,0xE495,0x84D4,0x85D4,0x8554,0x95D4,0x75B4,0x7575,0x8585,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x4644,0x4446,0x4646,0x4646,0x4146,0x5454,0x5454,0x5475,0x7554,0x5454,0x5454,0x7554,0x5454,0x6554,0x5475,0x5454,0x7575,0x8585,0x8585,0x8585,0xB485,0x7554,0x7554,0x8575,0x8575,0x7575,0x8575,0x85D4,0x7575,0x5465,0x4654,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F5F,0x4446,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4644
.word 0x85C4,0x54C4,0x7575,0x8575,0x75C4,0x5475,0x5454,0x4654,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x5F4F,0x7F4F,0x4F7F,0x4F7F,0x4F6F,0x7F7F,0x4F4F,0x4146,0x4F7F,0x8F4F,0x7F7F,0x5454,0x4F7F,0x7F7F,0x4F7F,0x7F6F,0x7F7F,0x7F6F,0x7F4F,0x6F4F,0x6F7F,0x4F7F,0x6F6F,0x9F7F,0x6FBF,0x406E,0x7E6E,0x7F9F,0x7FBF,0x7F5F,0x6F4F,0x6F7F,0x7F6F,0x6441,0xD4D4,0xD4D4,0xD4D4,0x9F6F,0x6F6F,0x4F6F,0x6F6F,0x6F5F,0xCF5F,0x7E7E,0x4F7F,0x6E4C,0xBFDF,0x7F9F,0x4F6F,0x6F4F,0x4F6F,0xDF8F,0xD4D4,0x64D4,0xAFDF,0xD464,0xD4D4,0xE4D4,0xD4D4,0x4646,0x4446,0x4646,0x4644,0x4146,0x5454,0x7554,0x5465,0x5454,0x5454,0x5475,0x6554,0xC485,0x7585,0x7585,0x8585,0x85C4,0xC485,0x5485,0x8554,0x8585,0x6585,0x5485,0x4446,0x4646,0x4F5F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x5441,0x5465,0x7565,0x7575,0x85C4,0x95B4,0x85B4,0x85C4,0x85B4,0xC485,0x8585,0xC495,0x8575,0x7575,0x7585,0x4654,0x4646,0x4446,0x5F4F,0x4646,0x4F5F,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x5F4F,0x4446,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646
.word 0xC485,0x7575,0x7585,0x7575,0x5485,0x6554,0x5454,0x4654,0x4644,0x5F4F,0x4446,0x4646,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x7F4F,0x4F7F,0x4F7F,0x4F5F,0x6F7F,0x4646,0x7F4F,0x8F4F,0x4441,0x4F7F,0x6F9F,0x4F8F,0x7F7F,0x6F4F,0x7F7F,0x4F6F,0x6F6F,0x7F4F,0x6F7F,0x6F6F,0x7F7F,0x6F6F,0xCF4F,0x5FCF,0x8E6E,0xF0F0,0xAF7F,0xBFEF,0x6F7F,0x7F7F,0x7F7F,0x6F6F,0xD444,0xD4D4,0xE4D4,0xD4D4,0x7FDF,0x7F6F,0x6F7F,0x5F5F,0x6444,0xFFDF,0x8E6E,0xBDF0,0xFC9D,0xEF6F,0x9FCF,0x6F7F,0x4F4F,0x6F7F,0xD464,0xD4D4,0xAFDF,0xD4C4,0xE4E4,0xD4D4,0xE4E4,0x5485,0x4446,0x4646,0x4F5F,0x4646,0x4F6F,0x4646,0x4446,0x4446,0x5454,0x5454,0x7544,0x8575,0x85C4,0x7585,0x7575,0x8575,0x95B4,0x75B4,0x5454,0x8565,0x75B4,0xC485,0x5475,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x4446,0x5F4F,0x5F4F,0x4646,0x4F5F,0x4F5F,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x5454,0x5454,0x7565,0x8575,0xB485,0xB495,0x8595,0xE495,0x95B4,0x95B4,0x85B4,0x8575,0x5465,0x5475,0x4654,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F5F,0x4646
.word 0x7575,0x8585,0xC485,0x5485,0x7554,0x5454,0x5454,0x4654,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F4F,0x5F4F,0x4646,0x5F4F,0x4646,0x4644,0x6F4F,0x7F4F,0x4F7F,0x6F4F,0x7F7F,0x5F4F,0x4146,0x4F8F,0x8F8F,0x7F7F,0x6454,0x7F8F,0x7F4F,0x7F7F,0x4F6F,0x7F6F,0x6F4F,0x4F7F,0x6F7F,0x6F4F,0x4F6F,0x4F6F,0x7F5F,0x5441,0xF474,0xF4E4,0xF4E5,0xD4E4,0xB465,0x6F7F,0x4F8F,0x7F6F,0xD441,0xE4E4,0xE4E4,0xD4E4,0x8F6F,0x4144,0x8F8F,0x4F7F,0xD464,0xDFDF,0xFEF0,0x6E9E,0xE06E,0x6FCF,0x7F8F,0x4F5F,0x5F6F,0xD4D4,0xE4E4,0xE4E4,0x64D4,0xE4D4,0xD4F4,0x64E4,0xD4E4,0x4544,0x4644,0x4644,0x4444,0x4444,0x4444,0x4154,0x6544,0x7554,0x5475,0x8575,0x9585,0xB4C4,0xC4B5,0x8585,0x7575,0x7575,0x5454,0x5454,0x5454,0xD485,0x8575,0x7575,0x4485,0x4644,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x6454,0x9575,0x75C4,0x5454,0x7554,0x8575,0x8585,0x85C4,0x85B4,0x7585,0x7585,0x6575,0x7575,0x5475,0x5454,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4F4F,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4644,0x4646,0x4F5F,0x4446,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4644
.word 0x7585,0x7585,0x9585,0x7575,0x7585,0x7554,0x5454,0x4454,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x7F4F,0x4441,0x4F7F,0x6F4F,0x6F4F,0x4646,0x7F4F,0x7F4F,0x4454,0x4F7F,0xAF5F,0x4F7F,0x7F7F,0x7F4F,0x7F6F,0x6F6F,0x6F7F,0x7F4F,0x6F7F,0x7F7F,0x4441,0x6F4F,0x4F4F,0x5F4F,0x6F5F,0x4F8F,0x8F5F,0x8F4F,0x4F7F,0x6F7F,0x8F7F,0x7F7F,0xD444,0xE4E4,0xE4E4,0xD4E4,0x41D4,0x4F6F,0x7F8F,0x4F6F,0x8F4F,0xAF5F,0x7464,0x7474,0x5464,0x7F8F,0x4F5F,0x5F4F,0xBF6F,0xE4D4,0xE4E4,0xD4E4,0x9FDF,0xE4D4,0x85E4,0xDFAF,0x75D4,0x5475,0x5475,0x5454,0x6554,0x7554,0x5465,0x8585,0x8585,0x8585,0x7585,0x4454,0x5454,0x5454,0x5465,0x4454,0x5454,0x5444,0x5475,0x7575,0x6554,0x8595,0x7575,0x8585,0x4475,0x4646,0x4646,0x4F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x4446,0x6554,0x9585,0x85B4,0x8595,0x5465,0x5475,0x7554,0x6575,0x7575,0x7575,0x7575,0x6575,0x5464,0x5475,0x5454,0x5454,0x4646,0x4F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646
.word 0x75D4,0x7565,0x7575,0x7554,0x7585,0x5475,0x5454,0x5454,0x4646,0x4644,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4F5F,0x5F4F,0x5F4F,0x4646,0x4F6F,0x4F6F,0x6F6F,0x4446,0x7F6F,0x4F6F,0x4146,0x4F7F,0x5F9F,0x7F7F,0x7564,0x7F8F,0x4154,0x4F6F,0x6F7F,0x6F4F,0x7F4F,0x4F7F,0x7F6F,0x6F4F,0x4F7F,0x64D5,0x9FCF,0x8F6F,0x6F4F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4F6F,0x6F6F,0x7F7F,0xD441,0xE4D4,0xE4F5,0xE4E4,0x9F6F,0x7F7F,0x4F7F,0x6F7F,0x4F5F,0x5F4F,0x4F4F,0x4F4F,0x7F5F,0x5F6F,0x5454,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0x64D4,0xD464,0xD4D4,0xD4E4,0x8575,0x85C4,0xB485,0x7585,0x7575,0x7575,0x7575,0x7565,0x8575,0x95C4,0x85B4,0x8595,0x7585,0x7554,0x7585,0x8585,0x8575,0x8585,0xB485,0x5485,0x9554,0x75B4,0x8585,0x75B4,0x4644,0x4644,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4644,0x4F4F,0x4646,0x5F4F,0x4646,0x4446,0x7554,0x5465,0xB485,0x7585,0xD4B4,0x7595,0x5454,0x6554,0x5464,0x5475,0x5475,0x5454,0x5475,0x7565,0x5454,0x5475,0x7575,0x4464,0x4645,0x5F4F,0x4646,0x5F4F,0x4446,0x4646,0x4644,0x4646,0x4F5F,0x4446,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4F5F,0x5F4F,0x4646,0x4F5F,0x4646,0x4F4F,0x4646,0x465F
.word 0xD495,0x7585,0x6554,0x7575,0x7575,0x5454,0x7575,0x6465,0x4644,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F6F,0x4F7F,0x4F5F,0x4646,0x4F6F,0x5F7F,0x7F4F,0x4F7F,0x8F4F,0x7F4F,0xAF6F,0x7F4F,0x7F8F,0x6F4F,0x7F7F,0x6F7F,0x7F7F,0x7F4F,0x6F6F,0x7F4F,0x4F7F,0xD4D4,0xD4D4,0x64C4,0x64D5,0x6464,0x64D5,0xC464,0x6464,0x7F8F,0x7F6F,0x7F4F,0xD454,0xE4E4,0xE4F4,0xE4E4,0x54C4,0x7F7F,0x6F7F,0x5441,0x5F9F,0x9F6F,0xAF6F,0xAF5F,0x6464,0xE554,0xD4D4,0xE4D4,0xF8D4,0xE4E4,0xD4E4,0xD4E4,0xAFDF,0xD464,0xE4E4,0x85D4,0x8585,0x85B4,0x8585,0x85B4,0x7585,0x8575,0x8575,0x8585,0x7575,0x6575,0x7575,0x6575,0x7554,0x8575,0x85B4,0x7575,0xB485,0xB485,0x8585,0x5475,0x85C4,0x7585,0xB485,0x5454,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x7565,0x7554,0x7575,0x5454,0x8554,0x9585,0x75B4,0x5475,0x5454,0x5454,0x6465,0x7565,0x5454,0x5454,0x5454,0x5454,0x7554,0x5475,0xC495,0x4454,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0xC485,0x5485,0x8585,0x9585,0x8575,0x85C4,0x5454,0x5454,0x4665,0x4644,0x4F5F,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x5F4F,0x4446,0x4646,0x4644,0x4646,0x4F7F,0x4F7F,0x5F4F,0x4646,0x4F5F,0x4F7F,0x4F6F,0x7F4F,0x8F4F,0x7F4F,0x6464,0x4F7F,0x7F4F,0x4F7F,0x7F7F,0x4F6F,0x7F6F,0x4F7F,0x7F6F,0x7F7F,0x7F4F,0xE4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0x64D4,0xD564,0xD451,0xE4D4,0xF4F5,0xE4E4,0xC4E4,0x6464,0x6454,0xD4D4,0xD4D4,0xD4D5,0xC4D4,0xF9D5,0xD4E5,0xF4D4,0xD4E4,0xF9E4,0xE4E4,0xE4E4,0xD4C5,0xD4D4,0x6485,0xD4D5,0xD4D4,0x4654,0x4646,0x4644,0x4146,0x5444,0x6554,0x7554,0x5454,0x5454,0x7575,0x7585,0x7575,0x8575,0x7585,0x8575,0x8585,0x8575,0x8585,0x7595,0x75C4,0x7554,0x8585,0x95C4,0x5475,0x4644,0x4644,0x4F5F,0x4446,0x4646,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4446,0x5F4F,0x4646,0xA454,0x5454,0x5454,0x4446,0x4446,0x8575,0x85D4,0x6465,0x7575,0x5475,0x6465,0x5465,0x5454,0x5454,0x5475,0x8554,0x7575,0x5475,0xD485,0x8575,0x8595,0x4454,0x4446,0x5F4F,0x4646,0x4446,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x5F4F,0x4646,0x4644
.word 0xB485,0x7575,0x8575,0x85B4,0x7575,0xB485,0x7554,0x5454,0x4554,0x4646,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4F6F,0x7F7F,0x4646,0x4644,0x5F4F,0x7F7F,0x7F4F,0x7F4F,0x8F4F,0x7F4F,0xD554,0x8F7F,0x4F7F,0x7F7F,0x4F7F,0x6F6F,0x7F7F,0x7F4F,0x7F6F,0x7F4F,0x8F4F,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD5E4,0xD4E4,0x64D4,0xD4D5,0xE4D4,0xF4F5,0xD4E4,0xD4D4,0xD4D4,0xF4E5,0xE4F9,0xF9F9,0xD4D4,0xD4D4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xE4E4,0xD4D4,0x6FBF,0xDFAF,0x5464,0x4646,0x4644,0x4F5F,0x4646,0x4644,0x4446,0x7554,0xC485,0xC495,0x85C5,0x85C4,0x8585,0xB485,0x7585,0x8575,0x75B4,0xB485,0xB485,0xC485,0x4464,0x8554,0x85C4,0x5485,0x4654,0x4644,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4644,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4644,0x4644,0x4646,0x5446,0x8585,0x75B4,0x7585,0x7554,0x5475,0x8575,0x7554,0x8585,0x7565,0x8575,0x8565,0x75C4,0x6585,0x7554,0x7585,0x8565,0x8585,0x4444,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4F4F,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x464F
.word 0x5454,0x5475,0x7575,0x7575,0x7554,0x5454,0x5475,0x7554,0x5454,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4F5F,0x7F7F,0x5F4F,0x4646,0x4646,0x4F5F,0x7F7F,0x4F7F,0x5441,0x7F9F,0xAF6F,0x7F4F,0x7F4F,0x7F4F,0x6F4F,0x4F7F,0x7F6F,0x4F7F,0x7F6F,0x6F4F,0x4141,0xD4D4,0xC5E4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xE464,0xE5D4,0xE4F5,0xE4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xC5E4,0xF4F4,0xE4E4,0xE4D4,0xE4D4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x64C4,0x5445,0x4647,0x4644,0x4646,0x4646,0x4F5F,0x5444,0x8575,0x8585,0x95C4,0xB5C4,0x95C4,0xD485,0xB495,0xC495,0x8585,0xB485,0xB495,0x8585,0x7575,0x5475,0x5454,0x4454,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4F5F,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x7541,0x7575,0x8575,0x8575,0x6465,0x7554,0xC454,0x7575,0x9575,0x8565,0x9575,0x7585,0x7575,0x8585,0x7565,0x5475,0x4646,0x4444,0x5454,0x4444,0x4446,0x4646,0x4F5F,0x4446,0x4F4F,0x4646,0x4F4F,0x4646,0x4644,0x4F5F,0x4F5F,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646
.word 0x6595,0xE4C5,0x7585,0x8554,0x95C4,0x75C4,0x7585,0x5465,0x5475,0x4654,0x4646,0x4646,0x4644,0x4446,0x5F4F,0x4646,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x4F5F,0x7F7F,0x4646,0x4646,0x4F5F,0x6F4F,0x4F7F,0x8F7F,0x5441,0x8F5F,0x6454,0x7F8F,0x8F4F,0x4F7F,0x7F6F,0x6F4F,0x4F7F,0x7F7F,0x7F7F,0x7F7F,0x8F4F,0xE4E4,0xE4E4,0xE4C5,0xC5E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xDFBF,0xE4D4,0xF4F7,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5D4,0xD4D4,0x4FBF,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x7545,0xB485,0x8585,0x75B4,0x7575,0x8585,0x7585,0x8575,0x7575,0x7575,0x8575,0x7575,0x8585,0x75B4,0x5475,0x7554,0x5485,0x4654,0x4644,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4146,0x7565,0x5454,0x8585,0x8575,0x5444,0x5454,0x8575,0x7554,0x8575,0x7575,0x8575,0xD4B4,0x5475,0x8565,0x8585,0x5475,0x4644,0x4446,0x4646,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4446,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x5F4F,0x4646,0x4644
.word 0x6585,0x8585,0x85B4,0x8565,0x7585,0x7585,0x7575,0x7554,0x5454,0x5454,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4446,0x4646,0x7F4F,0x4F6F,0x4F6F,0x4644,0x4646,0x4F5F,0x7F7F,0x4F7F,0x9F7F,0x8F4F,0x9F5F,0x7F5F,0x4F7F,0x7F4F,0x4F7F,0x7F7F,0x7F7F,0x7F4F,0x4F6F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD574,0xD485,0xF4D4,0xF4F5,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xC5E4,0xD4E4,0xD4D4,0xD4D4,0x4664,0x4446,0x5F4F,0x4646,0x4646,0x8554,0x7585,0x5454,0x4454,0x4644,0x5441,0x5444,0x6554,0x5475,0x7575,0x7554,0x6575,0x5475,0x4454,0x7564,0x8585,0x6585,0x4654,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4446,0x4646,0x5F4F,0x4646,0x4446,0x5475,0x4146,0x8565,0x54B4,0x4645,0x4446,0x8554,0x9575,0x5475,0x5454,0x8575,0x8575,0x5485,0x5441,0x8565,0x85C5,0x4654,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4F5F,0x4446,0x4446,0x4646,0x5F4F,0x4646,0x4644,0x4446,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7554,0x7575,0x5475,0x7575,0x7575,0x5465,0x7554,0x5454,0x5475,0x5475,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F5F,0x4F5F,0x6F4F,0x4646,0x4646,0x4646,0x4146,0x7F7F,0x7F7F,0x4144,0x5F8F,0x5F8F,0x7F5F,0x7F7F,0x4F7F,0x7F7F,0x7F7F,0x6F4F,0x4F7F,0x7F6F,0x7F4F,0x8F4F,0xD4D4,0xE4D4,0xE4D4,0xC5E4,0xC5E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFDF,0xD4D4,0xF4D4,0xF4F4,0xD5E4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD5D4,0xE5E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xD4D4,0xC5D4,0xD4D4,0x64D4,0x4FAF,0x4646,0x4646,0x4646,0x7544,0x7585,0x4454,0x4646,0x4644,0x4F5F,0x4646,0x4644,0x4446,0x4646,0x5454,0x7575,0x7554,0x7575,0x7575,0x8585,0x75B4,0x5454,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x4F4F,0x4F4F,0x4646,0x4646,0x4446,0x5446,0x4454,0x4646,0x8541,0x5485,0x4646,0x4646,0x7541,0x85C4,0x5475,0x4146,0x7554,0x7585,0x75D4,0x4646,0x5441,0xC475,0x4475,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x5F4F,0x4646,0x4F5F,0x4644,0x465F
.word 0xD475,0x7585,0x7575,0x5475,0x5475,0x9554,0x95C4,0x5485,0x7554,0x5454,0x4654,0x4F4F,0x4646,0x4F5F,0x4F4F,0x4446,0x4646,0x4646,0x4F4F,0x4F5F,0x4646,0x5F4F,0x5F4F,0x5F4F,0x4646,0x4F5F,0x4F5F,0x4F7F,0x7F7F,0x4F7F,0x9F5F,0x9F4F,0x7F5F,0x7F7F,0x7F4F,0x7F4F,0x7F7F,0x7F7F,0x7F4F,0x7F6F,0x7F7F,0x4441,0xD464,0xD4D4,0xD4C5,0xD4E4,0xD4E4,0xE4E4,0xC5D4,0xC5D4,0x74D4,0x64D4,0xC4D4,0xD4D4,0xE5D4,0xF4F4,0xE4F4,0xD4D4,0xD4C5,0xE4C5,0xC5D4,0xD4E4,0xD4F4,0xE4E4,0xE4E5,0xE4E4,0xE4D5,0xE4F4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x4654,0x5F4F,0x4646,0x4444,0x5475,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4444,0x8585,0x95D4,0xB5D4,0x95D4,0x8585,0x5475,0x5454,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4F5F,0x4446,0x4446,0x4646,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4541,0x4446,0x5F4F,0x7544,0x5475,0x4446,0x5F4F,0x5446,0x7585,0x4654,0x4646,0x4146,0x8565,0x7585,0x4646,0x4646,0x5441,0x7554,0x4644,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4F5F,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x8585,0xC485,0xD454,0xD4B4,0xC494,0x9575,0xC5C4,0x75C4,0x7554,0x5454,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F7F,0x7F7F,0x4F7F,0x7F7F,0x4144,0x5F8F,0x4FAF,0x4F6F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x4F7F,0x4F6F,0x7F7F,0x8F4F,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD5D4,0xDFBF,0xD464,0xDFBF,0xF4D4,0xF4F4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D5,0xF4E4,0xF4E4,0xE4E4,0xE5E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x64D4,0x4646,0x4646,0x4646,0x7544,0x4454,0x4446,0x4646,0x4F5F,0x5F4F,0x4646,0x5F4F,0x4446,0x8554,0x85B4,0x7585,0x7575,0x5454,0x4454,0x4654,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4644,0x4646,0x4646,0x5446,0x4475,0x4646,0x4646,0x4146,0x6575,0x4654,0x5F4F,0x4646,0x6541,0x7585,0x4446,0x5F4F,0x4646,0x5441,0x4641,0x4646,0x4644,0x4F4F,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x5F4F,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4F5F,0x4646
.word 0xB485,0x7595,0x8575,0xC495,0x9585,0x8575,0xC485,0x8595,0x5454,0x5485,0x5465,0x4654,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4F6F,0x4F7F,0x4F6F,0x7F7F,0x4F8F,0x8F5F,0x4154,0x7F7F,0x7F7F,0x8F5F,0x7F7F,0x4F7F,0x7F7F,0x7F6F,0x7F4F,0x8F4F,0xBFCF,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xC5E4,0xD4D4,0xD4D4,0xD464,0xD485,0xD4D4,0xD4D4,0xE5D4,0xF5F4,0xE4F4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4C5,0xD4D4,0xAFDF,0x4446,0x4646,0x4444,0x4454,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4446,0x4646,0x7544,0x7575,0x5454,0x4644,0x4646,0x4646,0x4644,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4644,0x4F5F,0x4644,0x4F5F,0x5F4F,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4F5F,0x4644,0x4644,0x4646,0x4F5F,0x4646,0x5454,0x4646,0x4646,0x4446,0x4146,0x8554,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7575,0x5475,0xC485,0x8585,0x7575,0x6554,0xB485,0x6575,0x7575,0x6554,0x7554,0x4454,0x4446,0x5F4F,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4F5F,0x4F4F,0x4646,0x4F4F,0x4446,0x4646,0x4F5F,0x4F7F,0x7F7F,0x7F7F,0x7F7F,0x9F4F,0x8F5F,0x8F5F,0x7F7F,0x4F7F,0x7F7F,0x7F4F,0x6F4F,0x4F7F,0x4F6F,0x7F7F,0x8F4F,0xC4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0x64D4,0xBFDF,0xBFCF,0xD4D4,0xD4D4,0xF4E4,0xF5F5,0xE4F4,0xD4E4,0xD4C5,0xC5D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0x5464,0x4646,0x5F4F,0x4546,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x7575,0x5454,0x4644,0x4446,0x6F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x5F4F,0x4446,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4441,0x4446,0x4446,0x4646,0x4646,0x4441,0x4646,0x4644,0x4646,0x4644,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x5F4F,0x4646,0x4644,0x4644,0x4446,0x4646,0x4646,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F
.word 0x8595,0x6554,0x7554,0x7565,0x7565,0x7585,0x5475,0x9585,0x85C4,0x5454,0x5475,0x5454,0x4644,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4F4F,0x4646,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x4F7F,0x6F8F,0x5F8F,0x4F7F,0x7F7F,0x4F7F,0x5F7F,0x4F7F,0x7F7F,0x4F6F,0x7F6F,0x4141,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD464,0xD464,0xD4D4,0xF4D4,0xF5F5,0xE4F7,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D5,0xD5E4,0xE4E4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0x4FAF,0x5F4F,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x5454,0x4654,0x4646,0x4446,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4644,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4646,0x4446,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4F5F,0x4646
.word 0x85C4,0x9554,0xB5C5,0x8585,0x9554,0x85C4,0xC495,0xB485,0x7585,0x6465,0x7544,0x7554,0x6554,0x4446,0x4646,0x4646,0x4F5F,0x4646,0x4644,0x4F5F,0x5F4F,0x4F4F,0x4646,0x4646,0x4646,0x6F4F,0x7F4F,0x7F4F,0x7F7F,0x4F7F,0x7F6F,0x9F5F,0x5444,0x7F8F,0x4F7F,0x7F7F,0x7F4F,0x7F4F,0x4F7F,0x7F6F,0x7F4F,0x8F4F,0xD4C4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x6FBF,0x64D5,0xD4D4,0xD4D4,0xE4E4,0xFFFF,0xE4F4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4C5,0xD4E4,0xD4D4,0xD4E4,0x64D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4444,0x4154,0x4446,0x4446,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F4F,0x4446,0x5F4F,0x5F4F,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4F5F,0x4644,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7575,0xC475,0xC485,0xB485,0x8575,0xB485,0x6585,0x5454,0x5454,0x8585,0x5475,0x7554,0x5454,0x4465,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x6F4F,0x7F4F,0x4F7F,0x4F7F,0x4F6F,0x4F7F,0x5441,0x5FAF,0x7F4F,0x7F7F,0x4F7F,0x7F6F,0x7F4F,0x7F7F,0x7F7F,0x4F7F,0x4F8F,0xBF6F,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xDFAF,0x64D4,0xD4D4,0xF4E4,0xFFFF,0xE4F4,0xD4E4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xC5D4,0x64D4,0x6FAF,0x4FAF,0x4646,0x4644,0x4644,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4F5F,0x5F4F,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4F5F,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4F5F,0x5F4F,0x4646,0x4F5F,0x4446,0x5F4F,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x4446,0x4646
.word 0x7575,0x8585,0xB485,0x5475,0x6575,0x5475,0x5475,0xD495,0x6585,0x7554,0x7585,0x5454,0x7575,0x5454,0x4644,0x4446,0x4646,0x4446,0x4646,0x4446,0x5F4F,0x4646,0x4F4F,0x4646,0x4F5F,0x6F4F,0x4441,0x7F7F,0x7F7F,0x4F7F,0x7F7F,0x5F7F,0x9F5F,0x8F5F,0x4F7F,0x7F7F,0x7F6F,0x7F4F,0x7F4F,0x4F6F,0x4F7F,0x8F4F,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF5F5,0xE4F5,0xD4E4,0xE4E4,0xD4D4,0xDFBF,0xD4D4,0xD4D4,0xD4C5,0xE4E4,0xD5D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x64D4,0xD564,0x4644,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4446,0x4646,0x4446,0x5454,0x4654,0x4644,0x4F5F,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x5F4F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x4F4F,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4F5F,0x4F4F,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F4F,0x4646,0x4F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x4646,0x4F5F,0x4644,0x4F5F,0x4646,0x5F4F,0x4646,0x4646
.word 0x5454,0x6565,0x5475,0x7575,0x8595,0x95E4,0x8565,0x85D4,0x7575,0xB485,0x7554,0x6585,0x7554,0x6554,0x4454,0x4646,0x4646,0x4646,0x4F4F,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4F7F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x5F7F,0x6F9F,0x4454,0x7F7F,0x7F7F,0x4F7F,0x4F7F,0x5F7F,0x4F7F,0x7F7F,0x7F4F,0xD4C4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xAFDF,0xD464,0xD4E4,0xD4D4,0xF4E4,0xFFFF,0xE4F4,0xD4E4,0xE4C5,0xD4D4,0xD485,0xD4E4,0xE4C5,0xD4D4,0xE4D4,0xD4E4,0xC5D4,0xD4D4,0xD4E4,0xD474,0x6FAF,0xCFAF,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4644,0x4646,0x4646,0x4446,0x4F4F,0x4F4F,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x4F4F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4F4F,0x4646,0x4646,0x4F5F,0x4646,0x5F4F,0x4446,0x4F4F,0x4F5F,0x4646,0x4F5F,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F4F,0x4446,0x4646,0x4644,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F
.word 0x5475,0x8585,0x8585,0x7585,0x95D4,0x85B4,0x7585,0x7575,0x8554,0x95D4,0x7575,0x8595,0x5454,0x7575,0x7575,0x4454,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4644,0x4F6F,0x4F7F,0x7F7F,0x7F7F,0x4F7F,0x4F7F,0x7F7F,0x9F5F,0x9F6F,0x7F4F,0x4F7F,0x6F6F,0x7F7F,0x7F4F,0x7F7F,0x4141,0x4F7F,0xBF6F,0xD4C4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0x85D4,0xD4D5,0xD4D4,0xE4D4,0xF4E4,0xE4E4,0xE4F4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64D5,0xAFDF,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F4F,0x4F4F,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F4F,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4646,0x4F5F,0x4F5F,0x4F4F,0x5F4F,0x5F4F,0x4446,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x4F5F,0x4F5F,0x4F5F,0x4646,0x4644,0x4F4F,0x4F5F,0x5F4F,0x4446,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4446,0x4646,0x5F4F,0x4646,0x4F5F,0x4646
.word 0x7554,0xB485,0x7585,0x5475,0xB485,0x7585,0x5454,0x8575,0x75B4,0x7585,0x7554,0xC475,0x7575,0x5454,0x5475,0x5475,0x4465,0x4446,0x4646,0x5F4F,0x4646,0x4446,0x5F4F,0x5F4F,0x4646,0x7F7F,0x4F7F,0x7F4F,0x7F4F,0x4F7F,0x7F7F,0x7F7F,0x5444,0x4FAF,0x4F8F,0x7F6F,0x7F7F,0x4F6F,0x4F7F,0x7F7F,0x4F7F,0x8F7F,0xD4C4,0xD474,0xC5D4,0xE4D4,0xC5D4,0xE4E4,0xE4D4,0xD4C5,0x64E4,0x6F9F,0x6FAF,0x6464,0xE4D4,0xE4E4,0xD4D4,0x6485,0x6FAF,0x6F9F,0xD4E4,0xE4E5,0xE4D4,0xC5D4,0xD4E4,0xE4C5,0xE4D4,0xD4D4,0x74D4,0xAFDF,0xBF6F,0x46C4,0x4446,0x4F5F,0x4646,0x4F5F,0x4F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4446,0x4F4F,0x5F4F,0x4446,0x4F4F,0x4646,0x4646,0x4F5F,0x4446,0x5F4F,0x4646,0x4646,0x4F4F,0x4446,0x4646,0x5F4F,0x4646,0x4644,0x4446,0x4F4F,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F5F,0x5F4F,0x4646,0x5F4F,0x5F4F,0x4646,0x4646,0x4446,0x4F5F,0x4646,0x4644,0x4646,0x5F4F,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4F5F,0x4F5F,0x5F4F,0x4646,0x4F5F,0x4F4F,0x4F5F,0x4644,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4644
.word 0x7554,0x7585,0x7565,0x7554,0x7575,0x5454,0xD485,0xC485,0x5495,0x7565,0x7554,0x5454,0x5454,0x5454,0x7554,0x7575,0x5454,0x4645,0x5F4F,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x5F4F,0x7F4F,0x7F4F,0x7F4F,0x4F7F,0x5F7F,0x4F7F,0x6F7F,0x9F4F,0xD454,0x8F4F,0x7F7F,0x6F4F,0x7F7F,0x7F7F,0x7F4F,0x4F7F,0x5F7F,0xCF9F,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xC5E4,0x8FDF,0x9F5F,0x9F4F,0xAF6F,0x5464,0xC4D4,0xBF6F,0x8FCF,0x5454,0x5F8F,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD4C4,0x46D4,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x5F4F,0x4646,0x4446,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x4F5F,0x4F4F,0x4446,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x4F4F,0x4646,0x4644,0x4646,0x4644,0x4644,0x4646,0x4646
.word 0x6554,0x5475,0x5454,0x8575,0xB5D4,0x75C4,0x7575,0x8585,0x7575,0x9585,0x85C4,0xC475,0xC495,0x8575,0x5475,0x6554,0x7575,0x5454,0x4575,0x4446,0x4646,0x4646,0x4646,0x4644,0x6F4F,0x7F4F,0x4F7F,0x4F7F,0x7F6F,0x4F7F,0x4F7F,0x7F7F,0x4F7F,0x6F9F,0x4FAF,0x6F4F,0x6F7F,0x6F4F,0x4F7F,0x7F6F,0x4144,0x5F7F,0xDF8F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4C5,0xD4E4,0x41D4,0x8F4F,0x4441,0x4F8F,0xDFBF,0xBFDF,0x5464,0x4F8F,0x4F8F,0xCFAF,0xF4F4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0x8FBF,0xDFBF,0x4664,0x5F4F,0x4646,0x4F5F,0x4F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x5F4F,0x4446,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4F5F,0x5F4F,0x5F4F,0x4446,0x4F5F,0x4646,0x5F4F,0x4646,0x4F5F,0x4644,0x4F5F,0x4F5F,0x4646,0x4446,0x4F4F,0x4646,0x5F4F,0x4646,0x4F5F,0x4644,0x4646,0x4446,0x5F4F,0x4646,0x4F4F,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4F5F,0x4644,0x4F4F,0x4F4F,0x4F5F,0x4F4F,0x5F4F,0x4446,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x4F4F,0x5F4F,0x4446
.word 0x5454,0x7554,0x7585,0xC454,0xB485,0x85B5,0x6554,0x6575,0x9554,0xD4C4,0xC4B5,0x8575,0x95C4,0xD475,0x8585,0x5454,0x5475,0x7575,0x7554,0x4454,0x4644,0x4646,0x4644,0x4F5F,0x7F4F,0x4F7F,0x4F7F,0x7F7F,0x4F7F,0x4F6F,0x4F7F,0x7F7F,0x4441,0x5F8F,0x8F4F,0x7F4F,0x6F7F,0x6F7F,0x7F7F,0x7F7F,0x4F7F,0x4F7F,0xDF7F,0x74D4,0xE4D4,0xC5D4,0xD5D4,0xD4E4,0xE4E4,0xD4D4,0x64D4,0x4F9F,0x4F8F,0x9F4F,0x4144,0x5444,0x4F9F,0x4F8F,0x6F9F,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4C5,0xD4E4,0x74D4,0x64D4,0xD4D5,0xD4D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4446,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4F5F,0x4644,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x464F
.word 0x7554,0x7554,0x7554,0x7554,0x8575,0x75B4,0xC585,0x95C4,0x75D4,0x8585,0x64B4,0x5454,0x5454,0x8575,0x75C4,0x8575,0x5465,0x7554,0x5475,0x6575,0x5475,0x7554,0x6544,0x5465,0x4165,0x4F7F,0x4F7F,0x7F4F,0x7F7F,0x4F7F,0x5441,0x4F7F,0x4F7F,0x5F7F,0x5FAF,0x4F8F,0x7F7F,0x7F4F,0x6F7F,0x7F4F,0x4141,0x4F7F,0xDF7F,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x9F6F,0x9F5F,0x9F5F,0xE4C4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xF5E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xCFAF,0x74D4,0xD464,0x4646,0x4F5F,0x4646,0x4F5F,0x4F5F,0x4646,0x4646,0x4646,0x4F4F,0x4446,0x4446,0x5F4F,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4F5F,0x5F4F,0x4F4F,0x5F4F,0x4646,0x4F5F,0x4F5F,0x4446,0x5F4F,0x5F4F,0x4F4F,0x5F4F,0x5F4F,0x5F4F,0x4646,0x4F4F,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x5F4F,0x4446,0x4646,0x4644,0x4F4F,0x5F4F,0x5F4F,0x5F4F,0x4446,0x4F4F,0x4F5F,0x4F4F,0x4F5F,0x4F4F,0x4646,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4F5F,0x4F5F,0x4F4F,0x4F5F,0x4F5F,0x5F4F,0x5F4F,0x4F4F,0x4F4F,0x4646,0x5F4F,0x4646,0x5F4F,0x4446,0x5F4F,0x4646,0x4646,0x4646
.word 0x5454,0x6554,0x7575,0x7554,0x7554,0x5454,0x8575,0x7585,0x6585,0x7575,0x5454,0x9575,0x85C5,0x5465,0x5485,0xD475,0x8595,0x5465,0x8554,0x5475,0x7575,0x5475,0x7575,0x5475,0x7F5F,0x7F4F,0x4F7F,0x4F8F,0x7F7F,0x4F7F,0x8F9F,0x4F7F,0x4F7F,0x4F8F,0x9F6F,0x7F5F,0x6F7F,0x4F6F,0x7F6F,0x7F7F,0x4F7F,0x8F7F,0xC441,0xCFBF,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xF4D4,0xC4E4,0x5FAF,0x64D5,0xE4E4,0xE4F4,0xE4D4,0xE4F4,0xF4E4,0xF4F5,0xE4E4,0xD5E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD464,0xD5D4,0xBFDF,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x5F4F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4446,0x4646,0x4F5F,0x4644,0x5F4F,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x5F4F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4F4F,0x4F5F,0x4F4F,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F5F,0x4F5F,0x4644,0x5F4F,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4F5F,0x4F4F,0x4646,0x5F4F,0x4646,0x4646
.word 0x5454,0x5454,0x5454,0x7554,0xC495,0xC495,0x7554,0x6575,0x8554,0xC495,0xC495,0xC475,0xB5D4,0xC485,0xB495,0x7554,0x7575,0x5454,0x7575,0x5454,0x7554,0x7585,0x7575,0x7575,0x4F7F,0x4144,0x4F7F,0x7F7F,0x4F7F,0x4F6F,0x4154,0x4144,0x4F7F,0x8F7F,0x6444,0x5F9F,0x7F4F,0x7F7F,0x4F7F,0x7F6F,0x4141,0x4F7F,0xCF8F,0x74D4,0xBFDF,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xE4D4,0xF5F5,0xE4F4,0xE4D4,0xE4F4,0xF4D4,0xE4F4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xD5E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xBFDF,0xBFCF,0x6485,0x4446,0x5F4F,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4F4F,0x4646,0x4F5F,0x4644,0x4F4F,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4644,0x4646,0x4446,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4F4F,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4F5F,0x4F5F,0x4F5F,0x4F5F,0x5F4F,0x4446,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x4644,0x4646,0x5F4F,0x4446,0x4F4F,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x465F
.word 0x5444,0x5454,0x5475,0x7575,0xC485,0x8585,0x7565,0x5475,0xC475,0xD495,0xD495,0x8554,0x7595,0x7565,0x95D4,0x5485,0x8575,0x7585,0xC554,0x5475,0x5475,0x5454,0x5454,0x5454,0x4F7F,0x4F7F,0x8F7F,0x7F4F,0x7F7F,0x7F7F,0x9F5F,0x9F4F,0x4F7F,0x4F8F,0x5F8F,0x41D5,0x4F8F,0x4F6F,0x7F6F,0x7F6F,0x4F7F,0x4F7F,0x8F8F,0xCFBF,0xC4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xC4D4,0xB4C5,0xC565,0xE475,0xE4F5,0xF1E4,0x65E4,0x4565,0x6545,0x8465,0xD4D5,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xC4D4,0x64D4,0x44D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4644,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4446,0x5F4F,0x4646,0x4F4F,0x4446,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4F5F,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4646,0x4F4F,0x4F4F,0x4F4F,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4F5F,0x4F5F,0x5F4F,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4446
.word 0x5454,0x5454,0x5454,0x5454,0x6575,0x4475,0xD485,0xD4C5,0x7595,0x95C4,0x85B4,0x7554,0x7565,0x5485,0xC485,0x7575,0xE495,0xC4C5,0x7575,0x7554,0xC585,0x75C4,0x5454,0x7F5F,0x7F4F,0x8F4F,0x4F7F,0x7F4F,0x7F4F,0x4F7F,0x9F4F,0x5464,0x5F8F,0x7F7F,0x8F4F,0xC441,0x7F4F,0x7F7F,0x7F6F,0x4F7F,0x4F6F,0x7F8F,0x5F7F,0x64D4,0x8FBF,0x74D4,0x64D4,0x5FAF,0x4444,0x6544,0x4445,0x6544,0x6565,0xC565,0xC575,0x6575,0x4545,0x6545,0x4545,0x4465,0x4545,0x7545,0xB4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD574,0x8FBF,0xCFBF,0x4F9F,0x4646,0x4F5F,0x4446,0x4646,0x4F5F,0x4646,0x4644,0x4646,0x4446,0x4646,0x4F5F,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4F5F,0x4F5F,0x4644,0x4646,0x4446,0x4F4F,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4644,0x4F5F,0x5F4F,0x4646,0x4F4F,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4F4F,0x4646,0x4F5F,0x4446,0x4646,0x5F4F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x5F4F,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x5F4F,0x4F4F,0x4F4F,0x5F4F,0x4F4F,0x5F4F,0x5F4F,0x5F4F,0x4646
.word 0x4485,0x5454,0x5465,0x5454,0x5454,0x5454,0xC585,0x8595,0x7575,0x7575,0x7585,0xD485,0xD4D5,0x85D4,0x7575,0x7554,0xC485,0x8595,0x5465,0x7585,0x7565,0x5475,0x8575,0x7F5F,0x4F7F,0x4F8F,0x7F7F,0x7F5F,0x7F7F,0x7F7F,0x6444,0x5FAF,0x5F7F,0x4F8F,0x4F8F,0x8F7F,0x6464,0x4F7F,0x6F7F,0x7F6F,0x4F7F,0x7F4F,0x4441,0xCFBF,0xBFCF,0xAFCF,0x7F4F,0x4444,0x4564,0xC545,0xC565,0x6565,0x65C5,0x65C5,0x4565,0x6545,0xC565,0xC545,0xC5C5,0xC575,0xC565,0x4665,0x4F5F,0xD465,0xD464,0xD4D4,0x64D4,0xCFBF,0x64D4,0xD4D5,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x5F4F,0x4646,0x4F4F,0x4F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4F4F,0x5F4F,0x4646,0x4F5F,0x4646,0x5F4F,0x4F4F,0x4F4F,0x5F4F,0x4446,0x4F4F,0x4F5F,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4F5F,0x4F5F,0x4F5F,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x4F5F,0x4F5F,0x4644,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4F5F,0x4644,0x4F5F,0x4F5F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x5F46
.word 0x6595,0x5454,0x6564,0x8554,0x8585,0x54B4,0x9575,0x75B4,0x5475,0x5454,0x7585,0x9565,0x8585,0x75D4,0x8554,0x85C4,0x7565,0x5465,0xE485,0x85D4,0xD485,0x7585,0x5454,0x4F7F,0x4F7F,0x8F4F,0x4F7F,0x4141,0x7F7F,0x7F4F,0x6441,0x9FDF,0x9F5F,0x7F4F,0x8F4F,0x7F5F,0xCF8F,0x4F8F,0x4F7F,0x7F6F,0x4F6F,0x4F7F,0x8F7F,0xD564,0xD574,0xD574,0xD4C4,0x4565,0x4444,0x4F6F,0x7F4F,0x4444,0x7F4F,0x5F4F,0x5F6F,0x4F6F,0x4F5F,0x5445,0x4544,0x4544,0x4444,0x6544,0xD4D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xBF6F,0x85C4,0x6564,0x6465,0x6465,0x5454,0x4454,0x4446,0x5F4F,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x4F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4F4F,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4F4F,0x4646,0x4F5F,0x4F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F4F,0x4F4F,0x4F5F,0x4F5F,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F4F,0x4646,0x5F4F,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4644,0x5F4F,0x4646,0x4F5F,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x4F5F,0x4646,0x4644,0x4646,0x4646,0x4644
.word 0x85D4,0x5444,0x5454,0x7554,0x6575,0x7575,0x7575,0x5475,0xC454,0xD4C5,0x95C5,0x7575,0x8575,0x6585,0xC495,0x8594,0x8575,0x7585,0x8575,0x85D4,0xB5B4,0x85D4,0x4475,0x4F7F,0x4F7F,0x4F8F,0x4F7F,0x8F5F,0x6F4F,0x7F7F,0x9F4F,0x5454,0x4F9F,0x7F4F,0x8F4F,0x8F4F,0x8F4F,0x4164,0x7F4F,0x7F6F,0x4F7F,0x4F8F,0x4F7F,0xBF6F,0xBFCF,0xD4C4,0xCFBF,0xAFDF,0x45C5,0x4544,0x84C5,0xC5C5,0xC5C5,0x4445,0x4644,0x4F5F,0xC544,0xE5D6,0xD5F5,0xC5C5,0x4575,0xD5C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0x6FBF,0x64D4,0x85D4,0x5475,0x7575,0x7554,0x7554,0x7575,0x6575,0x5475,0x4446,0x4646,0x4F5F,0x4646,0x4F5F,0x4644,0x5F4F,0x4646,0x4646,0x5F4F,0x5F4F,0x4446,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F4F,0x4446,0x5F4F,0x4446,0x4F4F,0x4F5F,0x4F5F,0x4F5F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x5F4F,0x4646,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4446,0x4646,0x4F5F,0x4446,0x4F4F,0x4646,0x4F5F,0x4644,0x4646
.word 0x95D4,0x4485,0x5454,0x5454,0x5454,0x5454,0x8585,0x85C4,0x8554,0x85C4,0xB485,0x7554,0x7554,0x8565,0xB5D4,0x85C4,0xC565,0xC495,0x6575,0x5475,0x8575,0x54B4,0x8F4F,0x7F4F,0x8F4F,0x4441,0x7F7F,0x5464,0x5F7F,0x4F7F,0x5F7F,0x6F9F,0x8F6F,0x8F4F,0x4F7F,0x5F7F,0x4F8F,0x8F8F,0x5F8F,0x7F5F,0x7F7F,0x7F4F,0x4F7F,0x7F9F,0x64D4,0x64D4,0x64D4,0x85D5,0xCFAF,0xC575,0xD665,0xD5C4,0x7FEF,0xDFFF,0xF5D5,0xF5D5,0xE4D5,0xB4C5,0xC565,0x6554,0xC5C5,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xCFBF,0xD564,0xDFBF,0x7575,0x7575,0x7575,0x7575,0x7575,0x7554,0x7554,0x5475,0x5475,0x7554,0x4654,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4F5F,0x4F4F,0x4F4F,0x4646,0x4446,0x4F5F,0x4644,0x4F4F,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4F4F,0x4F4F,0x5F4F,0x4646,0x4644,0x4644,0x5F4F,0x4646,0x4646,0x4F5F,0x4644,0x4646,0x4646,0x4F4F,0x4F4F,0x4F5F,0x4446,0x4F4F,0x4F5F,0x4F5F,0x4F5F,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4644,0x4F4F,0x4F4F,0x4646,0x4F5F,0x4F4F,0x4F4F,0x4446,0x4F4F,0x4646,0x4F5F,0x4646,0x4F4F,0x464F
.word 0xC495,0x7585,0x4454,0x7554,0x8554,0x5485,0x7575,0x8575,0x7575,0xB485,0x7585,0x9575,0x95C5,0x7575,0xC485,0x6585,0x9585,0x95C4,0x7554,0x7585,0x5454,0x4154,0x7F7F,0x7F4F,0x4F7F,0x7F8F,0x7F4F,0x8F6F,0x4F7F,0x7F7F,0x5441,0x7FAF,0x4FAF,0x4F9F,0x4F7F,0x5F8F,0x4F8F,0x4F7F,0x8F4F,0x4154,0x7F7F,0x7F4F,0x5F7F,0x5F8F,0xDFAF,0xD4D4,0xCFBF,0xCFBF,0x9F8F,0x65C5,0x6565,0x6575,0xC5C5,0xEFDF,0xF5E5,0xD5C5,0xC5B4,0xC575,0x7565,0x65C5,0x74C5,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD474,0xD464,0x8FBF,0x85D4,0xD485,0xD495,0xE4C5,0x8595,0x8585,0x7575,0x5475,0x7554,0x7575,0x7575,0x5475,0x4645,0x4646,0x4446,0x4F4F,0x5F4F,0x5F4F,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4F5F,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F5F,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4F5F,0x4646,0x4F4F,0x4646,0x4F5F,0x4646,0x4446,0x5F4F,0x4646,0x4F4F,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646
.word 0xB5C4,0x85E4,0x5475,0x5454,0x6575,0x6554,0x5475,0x8575,0x7565,0x5475,0x7554,0xC495,0xC495,0x5485,0x6575,0x5475,0x7575,0x6575,0x9554,0x95C4,0x5465,0x4144,0x4F7F,0x7F8F,0x4144,0x8F4F,0x6F7F,0x4F9F,0x7F5F,0x4F7F,0x8F7F,0xD575,0x9F5F,0x4454,0x4F8F,0xCF7F,0x4F8F,0x8F7F,0x8F4F,0x4F8F,0x7F6F,0x4F7F,0x4F7F,0x5F7F,0xAF5F,0x6464,0x64D5,0x64D4,0xC4D4,0x5F9F,0x75D6,0x6545,0x65D5,0xC5C5,0x6575,0x6575,0xC575,0x7565,0x65C5,0xDFBF,0xE4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xCFAF,0x7FBF,0xD4D4,0x6575,0x5454,0x5454,0x5454,0x5454,0x5454,0x7575,0x8585,0x8575,0x8575,0x7575,0x7554,0x6475,0x6454,0x4645,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x4446,0x4F4F,0x4F4F,0x4446,0x4F4F,0x4F5F,0x4F5F,0x4644,0x4F5F,0x4F4F,0x4646,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x5F4F,0x5F4F,0x4646,0x5F4F,0x4646,0x4F4F,0x4F5F,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4F4F,0x4F5F,0x4646,0x4F5F,0x5F4F,0x4646,0x4446,0x4646,0x4644,0x4F5F,0x4F5F,0x4F5F,0x4F5F,0x4F5F,0x4F5F,0x4F5F,0x4646,0x5F4F,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4644,0x4446,0x5F4F,0x5F4F,0x4646,0x4F5F,0x4644,0x4F4F,0x4646,0x4446
.word 0xC495,0xB5D4,0x54C4,0x5454,0x5454,0x5475,0x7554,0x6575,0x5454,0x8554,0x7575,0xC485,0xD485,0x7585,0x5454,0x7575,0x7554,0x5454,0x7575,0x7585,0x4175,0x4F7F,0x4F8F,0x4F7F,0x4F8F,0x4F8F,0x4F8F,0x9F5F,0x7F8F,0x4F7F,0x8F4F,0x9F6F,0x8F6F,0x9F5F,0x7F4F,0x5441,0x4F8F,0x4F7F,0x4F7F,0x8F7F,0x6444,0x4F7F,0x7F7F,0x8F4F,0x5454,0xAFDF,0xBFCF,0xBFCF,0xBFCF,0xAFCF,0x4554,0x65D6,0x54C5,0x6565,0xC565,0x6545,0xC545,0xC554,0xAF7F,0xD4C4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x64D4,0x64D4,0xD4D5,0xC464,0x5485,0x5454,0x6554,0x8575,0x95C4,0x75B4,0x5454,0x7554,0x8575,0xD495,0x7585,0x7575,0x8575,0x5475,0x6475,0x4654,0x4644,0x4646,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4446,0x5F4F,0x4F4F,0x4646,0x4646,0x4646,0x4F5F,0x4644,0x4F5F,0x4446,0x4F4F,0x4F4F,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4F4F,0x4446,0x4646,0x4F5F,0x4F4F,0x4F5F,0x4F4F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F5F,0x4F5F,0x4646,0x4F5F,0x4446,0x4F4F,0x4F5F,0x4F5F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4F5F,0x4F4F,0x4F4F,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4F5F,0x4646
.word 0x85E4,0xD495,0xB4B5,0x5475,0x5454,0x5454,0x5454,0x5454,0x8554,0xB485,0x54B4,0x7575,0x7585,0x7554,0x8585,0x8585,0x8554,0x75C4,0x5454,0x5465,0x7F7F,0x8F4F,0x4F7F,0x7F7F,0x8F4F,0x5441,0x7F8F,0x5454,0x4F7F,0x4F7F,0x4F8F,0x5F9F,0x4FAF,0x4F9F,0x4F8F,0x7F8F,0x9F4F,0xD564,0xD4D4,0x85D4,0x8F9F,0x7F7F,0x7F4F,0x8F4F,0x9F5F,0x6464,0xD4D4,0xD485,0x85D4,0xD464,0x5FAF,0x5F8F,0x5F9F,0x5F9F,0x4F9F,0x9F9F,0x9F5F,0x9F4F,0xD464,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x64D4,0xBFDF,0xBFCF,0x85C4,0x5475,0x7575,0x8575,0x8595,0x8554,0x85C4,0x7585,0x8554,0x85C4,0x5465,0x5454,0x8575,0x8595,0x7575,0x8575,0x7585,0x5475,0x5475,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4F5F,0x4F5F,0x4F5F,0x4446,0x4646,0x4F5F,0x4646,0x4F4F,0x4646,0x4646,0x5F4F,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x5F4F,0x4F4F,0x5F4F,0x5F4F,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4F4F,0x4F5F,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x5F4F,0x5F46
.word 0x95D4,0x95B4,0xB5C4,0x7585,0x4454,0x6541,0x7554,0x8575,0x7554,0x7575,0x6585,0x7554,0x5454,0x8565,0x85B4,0x75B4,0x8575,0x7585,0x7575,0x7F5F,0x7F4F,0x4144,0x7F4F,0x7F4F,0x8F4F,0x8F5F,0x5F7F,0x4F9F,0x4F8F,0x4F7F,0x4F8F,0xAF4F,0x8F7F,0x4154,0x4F8F,0x8F5F,0x6454,0x64D5,0xD464,0xBFDF,0xD4C4,0x85D4,0x4FAF,0x7F7F,0x4441,0x5F9F,0xDFAF,0x64D4,0xD4D4,0xD4D4,0xC4D4,0x4FAF,0x4F8F,0x4F8F,0x4F8F,0x8F4F,0x9F4F,0xD4C4,0xE4E5,0xD4E4,0xE4D4,0xD4D4,0xD4C5,0xBFDF,0x64C4,0x64D5,0x8585,0x95C4,0x7585,0xD475,0xC4D4,0x5454,0x5475,0x6554,0x5454,0x5454,0x8575,0x7575,0x5454,0x7565,0x8585,0x7585,0x7575,0x7575,0x7575,0x5475,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4F5F,0x4646,0x4446,0x4646,0x4646,0x4F5F,0x4446,0x4F4F,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F4F,0x5F4F,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F4F,0x5F4F,0x4646,0x4F4F,0x4F5F,0x4F4F,0x4646,0x4F5F,0x5F4F,0x4446,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xB5D4,0x85C4,0x85B4,0x95D4,0x5485,0x5454,0x5475,0x7554,0x7554,0x5454,0x5465,0x8554,0xC485,0x7585,0x7554,0x7554,0x6575,0x5475,0x7F4F,0x4F7F,0x4144,0x5F8F,0x4F8F,0x4F7F,0x8F4F,0x4441,0x4F8F,0x9F4F,0x7F4F,0x8F4F,0x8F7F,0x5444,0x7F8F,0x4F7F,0x8F8F,0xD554,0x6FAF,0x6FBF,0xCFAF,0x6464,0x64D4,0x64D4,0x64D5,0xC4D4,0xAFDF,0x9F5F,0x6454,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xD474,0x6585,0xC495,0x7595,0x7554,0x5475,0x7575,0xC485,0x9595,0x7575,0xD485,0xB5D4,0x54B4,0x95D4,0x6585,0x5454,0x8565,0x8585,0x7575,0x7575,0x8575,0x7575,0x4665,0x4644,0x4F5F,0x4F4F,0x4F4F,0x5F4F,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4F5F,0x4F5F,0x4F4F,0x4646,0x4F5F,0x5F4F,0x4646,0x4F4F,0x5F4F,0x5F4F,0x5F4F,0x4446,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4F5F,0x4F4F,0x4F4F,0x4F4F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4446,0x5F4F,0x4646,0x4F4F,0x4F4F,0x4F4F,0x5F4F,0x5F4F,0x4F4F,0x4646,0x4446,0x4646,0x5F4F,0x4446,0x5F4F,0x5F4F,0x4F4F,0x4646,0x4644,0x4F4F,0x4646,0x4F4F,0x4F5F,0x4F5F,0x4F5F,0x4644
.word 0xC4C5,0xC495,0xD495,0xC5D4,0x85D4,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x7554,0x8575,0x5475,0x8575,0x7554,0x5454,0x4454,0x4F7F,0x4F7F,0x4F8F,0x4F8F,0x8F7F,0x5F7F,0x4F8F,0x7F7F,0x8F4F,0x4144,0x4F7F,0x4F8F,0x7F6F,0x5441,0x4F8F,0x8F4F,0x6444,0x6FAF,0x64D4,0xC4D5,0xBFDF,0xD4C4,0xAFDF,0xD4D4,0xD464,0xD4D4,0x64D4,0x64D4,0x5F8F,0xDFAF,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xF4F4,0xF4E4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xD4D4,0x74D4,0xBFDF,0xBFCF,0x85B4,0x75C4,0x8585,0x54B4,0x5454,0x8575,0x7585,0x8585,0xD4C4,0x5485,0x9585,0x95D4,0x7585,0xC485,0x7585,0x7554,0x5454,0x8565,0x8595,0x8585,0x8575,0x8575,0x7575,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4446,0x4646,0x4F5F,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x4446,0x4646,0x4F5F,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x464F
.word 0xB485,0x4644,0xD495,0xC495,0xC495,0xC495,0x5475,0x5454,0x7554,0x5475,0x5485,0x6554,0x5454,0x5485,0x7575,0x5475,0x5454,0x4F7F,0x4144,0x8F5F,0x5441,0x4FAF,0x4F7F,0x7F4F,0x8F4F,0x4441,0x7F8F,0x8F5F,0x7F8F,0x6F7F,0x7F7F,0x8F4F,0x8F4F,0x6464,0xD4D5,0xC4D4,0xCFBF,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x41D4,0xD464,0xD4D4,0xE4D4,0xD5D4,0xF4E4,0xE4E4,0xF4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x85D4,0x7575,0x5475,0x7554,0x5454,0x7575,0xD475,0xD4B5,0x85D4,0x8575,0x8585,0x5465,0x7565,0x5475,0x5454,0x5465,0x7575,0x8585,0x8565,0x5475,0x6554,0x8585,0x8575,0x7575,0x6554,0x7575,0x4454,0x4646,0x4646,0x4644,0x4F5F,0x4F5F,0x5F4F,0x4F4F,0x4F4F,0x4646,0x4446,0x4446,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4644,0x4F4F,0x4644,0x4F5F,0x4F4F,0x4F4F,0x4646,0x4446,0x5F4F,0x4646,0x4F4F,0x4646,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x5F4F,0x4446,0x4F4F,0x4F5F,0x4F4F,0x4646,0x4644,0x4F5F,0x4446,0x4F4F,0x4646,0x4646,0x5F4F,0x4F4F,0x4446,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4F5F,0x4F5F,0x4F5F,0x4646,0x4646,0x4F5F,0x4646
.word 0x4475,0x4646,0x9585,0x95C4,0x95C4,0xB5C4,0x85C4,0x5454,0x7554,0x7554,0x5475,0x5475,0x8554,0x7575,0x7554,0x5454,0x4141,0x8F4F,0x5F7F,0x5F8F,0x5F9F,0x4F7F,0x8F7F,0x8F4F,0x8F4F,0x5F7F,0x7F7F,0x4441,0x7F7F,0x6F4F,0x7F4F,0x8F4F,0x6441,0xBFDF,0x64D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0x6FAF,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4F4,0xE4F4,0xC5E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x7585,0xB5C4,0x95C5,0x8575,0xD4D4,0xC4B5,0x8564,0x95B4,0x85D4,0x6554,0x5454,0x7575,0xD485,0xC495,0x7585,0x8575,0xE495,0x85D4,0x7585,0xC495,0x5465,0x7554,0xB485,0x8554,0x8585,0x7575,0x8575,0x4654,0x4F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4644,0x4646,0x4646,0x4F4F,0x4646,0x4F5F,0x4F5F,0x4446,0x4646,0x4F4F,0x5F4F,0x4646,0x4F4F,0x4F5F,0x4F5F,0x4644,0x4F4F,0x5F4F,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4F5F,0x4F4F,0x4446,0x4F4F,0x4646,0x4446,0x4646,0x4646,0x4446,0x4F4F,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x5F4F,0x5F4F,0x4646,0x4646,0x4646
.word 0x4646,0x8F6F,0x95D4,0x95C4,0x85B4,0xB485,0xD4B5,0xD4D4,0x6585,0x5454,0x4454,0x5454,0x6575,0x5475,0x5454,0x7F4F,0x7F4F,0x4F7F,0x9F5F,0x9F5F,0x8F5F,0x8F4F,0x4F7F,0x7F8F,0x5441,0x8F4F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x4F7F,0x5F7F,0xD5D4,0xD464,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xE4C5,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0x51D4,0xD4D4,0xD5D4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0x74D4,0xD485,0x5495,0xC485,0xE485,0xC485,0xC495,0x85B5,0x8565,0xB485,0x5475,0x9575,0xC5D4,0x85C5,0x8565,0xB5D4,0x95D4,0x8565,0x85C4,0xB495,0x6585,0xC495,0x85B5,0x7575,0x7554,0x8575,0x85B5,0x7585,0x7554,0x5475,0x4664,0x4646,0x4F5F,0x4646,0x4446,0x4F4F,0x4F5F,0x4F4F,0x4F5F,0x4F4F,0x4F4F,0x5F4F,0x5F4F,0x5F4F,0x4F4F,0x4646,0x4646,0x4F5F,0x5F4F,0x5F4F,0x4646,0x4646,0x4644,0x4F5F,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x4646,0x4F5F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x5F4F,0x4646,0x4F4F,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4F5F,0x465F
.word 0x4646,0x9546,0x95D4,0x95D4,0xD4C4,0x8585,0x95C4,0x9595,0xB5D4,0x5475,0x5454,0x5454,0x6475,0x5454,0x8F5F,0x4141,0x7F8F,0x9F4F,0x6444,0x5F9F,0x8F5F,0x7F8F,0x7F4F,0x4144,0x5F8F,0x4F8F,0x5F7F,0x7F4F,0x7F4F,0x4F6F,0x8F4F,0xD464,0xCFAF,0x64D4,0xDFBF,0xD4D4,0xE4D4,0xD5D4,0xD4E4,0xE4E4,0xE4C5,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x64D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4E4,0x85D4,0xB485,0xB485,0x8585,0x85B4,0x75B4,0x5475,0x5454,0x7575,0x9565,0x95C4,0x85D4,0x8575,0xC495,0x85D4,0x8575,0xB475,0x8585,0x5465,0x7575,0x7575,0x7554,0x5475,0x7554,0xB485,0xB495,0x7585,0x7585,0x7575,0x4644,0x4646,0x4F5F,0x4F5F,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4446,0x4646,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4F4F,0x4646,0x4F5F,0x4446,0x4646,0x4646,0x4644,0x4F5F,0x4446,0x4F4F,0x4446,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x5F4F,0x5F4F,0x4646,0x4646,0x4644,0x4446,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4646,0x4F5F,0x4646,0x4446,0x5F46
.word 0x4446,0xD446,0xC495,0x6585,0x4646,0x9565,0xD4D4,0xD4C4,0xC495,0xE485,0x95D4,0x8585,0x5475,0x4144,0x5444,0x7F7F,0x6F7F,0x4F8F,0x9FDF,0x9F5F,0x4F8F,0x4F7F,0x4F7F,0x4F8F,0x8F4F,0x7F4F,0x7F7F,0x4F7F,0x6F7F,0x5441,0xC4D4,0xCFBF,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xC5D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x64D4,0x6FAF,0x9F6F,0x6464,0x6454,0x6464,0x6464,0x64D4,0xD464,0x8575,0x8585,0x8585,0xB485,0x7575,0x5454,0x5454,0x7575,0x5454,0xC485,0xC5B5,0xD4C5,0x8585,0x85B4,0x75C4,0x8565,0x85B4,0x5475,0x6575,0x5454,0x8575,0x8575,0x8585,0x7585,0x8575,0x85D4,0x5475,0x7554,0x9585,0x85C4,0x7575,0x7575,0x7575,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4644,0x4F5F,0x4F4F,0x4446,0x4646,0x4646,0x4644,0x4F5F,0x4F5F,0x4646,0x4646,0x4F4F,0x4646,0x4F4F,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4F5F,0x4446,0x4F4F,0x5F4F,0x4646,0x4F4F,0x5F4F,0x4F4F,0x4646,0x4F5F,0x5F4F,0x5F4F,0x4646,0x5F4F,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4F5F,0x4F5F,0x4F4F,0x4646,0x4F5F,0x4646,0x4F4F,0x4F4F,0x4F4F,0x4644,0x4646,0x4644,0x4646,0x4646
.word 0x4646,0xB546,0x85D4,0x4665,0x4446,0x8546,0xD4C5,0xD495,0xC4B5,0x9585,0xD4C4,0xD4C5,0x5485,0x4F7F,0x8F9F,0x4F7F,0x8F5F,0x5444,0x5FAF,0x4454,0x4F8F,0x7F7F,0x7F4F,0x8F5F,0x8F4F,0x7F8F,0x4144,0x7F4F,0x5F7F,0xD4D4,0xDFAF,0xD4C4,0x64D4,0xBFDF,0xD4C4,0xD4D4,0xD4E4,0xD4E4,0xC5E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xC5E4,0xE4E4,0xC5D4,0xD4E4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x8F6F,0x4F7F,0x8F8F,0x7F8F,0x8F8F,0x7F8F,0x8F8F,0xC475,0x7585,0x7585,0x95B4,0x5475,0x8554,0x7595,0x8575,0x94D4,0x95D4,0x8575,0xD4D4,0x95D4,0x75B4,0x8585,0x5475,0x5475,0x5454,0x8585,0x8575,0xB5C4,0xD4D4,0x7575,0xD4C4,0xC495,0x8585,0xB5C4,0x85C4,0x5454,0x8575,0x95C4,0x85B4,0x8585,0x7554,0x4575,0x4644,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x5F4F,0x4F4F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4646,0x4646,0x4446,0x4F4F,0x5F4F,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x4F5F,0x4F5F,0x4F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x5F4F,0x4646,0x4F4F,0x4F5F,0x4F5F,0x4646
.word 0x4646,0xCF5F,0x65B5,0x4646,0x4646,0xD446,0x95D4,0x85C4,0x4646,0x8585,0xE495,0x6585,0x8F4F,0x5444,0x7F8F,0x4144,0x4F8F,0x5F8F,0x9F5F,0x7F5F,0x7F8F,0x7F4F,0x5F7F,0x4FAF,0x4F8F,0x7F4F,0x4F7F,0x5F6F,0x64D4,0x64D5,0xD4D4,0xD464,0xBFDF,0xD4C4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D5,0xD5E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0x8F6F,0x5454,0x5454,0x4454,0x5454,0x4154,0x7575,0x5454,0x5454,0x6554,0x6585,0xD495,0xD4C5,0x8575,0xC495,0xC495,0x8575,0x8585,0x7585,0x7554,0xD485,0x8595,0x8565,0x95C4,0xD4C4,0x54D4,0xC485,0x9595,0x75C4,0x8585,0x85C4,0x54B4,0x7554,0x5475,0x7565,0x5454,0x8565,0x8585,0x75B4,0x7565,0x7585,0x4654,0x4646,0x4F4F,0x4646,0x4F5F,0x4644,0x5F4F,0x4646,0x4F5F,0x4F4F,0x4F4F,0x4F5F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F5F,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F5F,0x4F5F,0x4F5F,0x4F5F,0x4F5F,0x4F4F,0x5F4F,0x5F4F,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4644,0x4F4F,0x4446,0x4F4F,0x4646,0x4646,0x4F5F,0x4644,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x4646,0x8546,0x46D4,0x6F4F,0x4646,0x8544,0x85D4,0x4695,0x4646,0xD475,0x75D4,0x8F4F,0x5F7F,0x7F9F,0x7F4F,0x8F4F,0x4F7F,0x9F6F,0x8F5F,0x8F5F,0x7F4F,0x4F7F,0x5441,0x8F5F,0x7F4F,0x7F4F,0x6F4F,0xBF5F,0xBFCF,0xD464,0xD464,0xCFBF,0x64D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x5F9F,0x5FAF,0x5FAF,0x9F9F,0x5454,0x5454,0x8575,0xC5D4,0xB5D4,0x85C4,0x95C4,0x85D4,0xC475,0xB485,0x7595,0x7575,0x7575,0x5475,0x9575,0xD4C5,0xD4C5,0x7585,0xC485,0x9595,0x75D4,0x8585,0x85B4,0x5485,0xB485,0x8585,0x7565,0x9585,0x95C4,0x85C4,0x6585,0x5454,0x95B4,0x8585,0x5475,0x7585,0x6475,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F4F,0x4F4F,0x5F4F,0x4646,0x4644,0x5F4F,0x4646,0x4446,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4F5F,0x4F5F,0x4644,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x5F4F,0x5F4F,0x5F4F,0x4446,0x4F4F,0x5F46
.word 0x4646,0x4646,0x4675,0x4646,0x4646,0x9546,0x95D4,0x4646,0x4644,0x8546,0x4154,0x7F7F,0x8F6F,0x7F4F,0x5F7F,0x4F7F,0x8F4F,0x9F5F,0x9F4F,0x4F7F,0x7F7F,0x7F4F,0x9F5F,0x4F8F,0x4F8F,0x7F7F,0xD454,0x64D4,0xC4D5,0xAFDF,0xD4D4,0xD485,0xD4D4,0x74D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xF4E4,0xE4E4,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xD4D4,0x64D4,0xD464,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0x64D4,0x7FAF,0xCFAF,0x9F6F,0x7554,0xC485,0x7585,0xC485,0xC495,0x7575,0x7575,0x7575,0x5454,0x5454,0x7554,0x5454,0x8565,0xB5C4,0xC475,0x9595,0xC495,0x6585,0x7575,0x8575,0x5465,0x6575,0x5475,0x8565,0x5465,0x6554,0x7585,0xC485,0xC5B5,0xD4B4,0x85D4,0x5475,0x7554,0x9575,0x7585,0x8575,0x7575,0x6585,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x4646,0x4446,0x4F4F,0x4646,0x4646,0x4F5F,0x4F5F,0x4446,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4F5F,0x4F5F,0x4F5F,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4446,0x4646,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4F4F,0x4F5F,0x5F4F,0x4646,0x4644,0x4F4F,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4446,0x4646,0x4644,0x4646,0x4F5F,0x8546,0xB485,0x4644,0x4646,0x4146,0x7F7F,0x5F7F,0x7F8F,0x7F4F,0x8F4F,0x4F7F,0x5F7F,0x4F8F,0x4F9F,0x7F8F,0x4F7F,0x4F7F,0x4F9F,0x4F7F,0x4F7F,0x4F7F,0x64D4,0xBFDF,0xBF8F,0x64C4,0x64D5,0xBFDF,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD5D4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0x64E4,0x4F8F,0x8F7F,0x4F8F,0x4F8F,0x4F7F,0xD441,0xD4D4,0xD4D4,0xAF6F,0xAF7F,0x5454,0x7F9F,0xC5C4,0x75C4,0x8585,0x75C4,0x5454,0x5454,0x6554,0xD475,0xD4B5,0x6585,0xB5D4,0x95E4,0xD4C5,0x8585,0x85B4,0x85C4,0x5475,0x5454,0x5454,0xD475,0xD4B5,0x8595,0x8565,0xC5D4,0xC5E4,0x54C4,0xC485,0xC5D4,0x95D4,0x8585,0x8565,0x5465,0x7575,0x85D4,0x7575,0x8575,0x8575,0x4644,0x4644,0x4644,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x5F4F,0x4F4F,0x4646,0x4646,0x4446,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F4F,0x4F4F,0x5F4F,0x4646,0x5F4F,0x4F4F,0x4F4F,0x5F4F,0x5F4F,0x4446,0x4646,0x4646,0x4646,0x4F5F,0x4546,0x4645,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x4646
.word 0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4685,0x4646,0x4F5F,0x4F6F,0x4F7F,0x8F4F,0x7F4F,0x8F4F,0x4F7F,0x7F4F,0x8F4F,0x4F8F,0x4F7F,0x4F7F,0x7F7F,0x8F4F,0x7F4F,0x4F7F,0x4F8F,0xD454,0xCFBF,0xD564,0xD464,0xD464,0xCFBF,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4C5,0xE4D4,0xE4E4,0xE4D5,0xE4F4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0x8F8F,0x4144,0x4F8F,0x64D4,0x8F6F,0x5444,0x8F6F,0x7F4F,0x4F7F,0xD564,0xD4D4,0x6454,0x5FAF,0xC475,0xD4C5,0x5475,0x5454,0x7554,0x9585,0xD4D4,0xC4B5,0x8575,0xB5C4,0x85D4,0xB495,0xD485,0x8595,0x7565,0x8585,0x7575,0x8554,0xC5C4,0x95D4,0x7585,0xD495,0xC5D4,0x8585,0xC495,0xD4B5,0xB495,0x8585,0xB5D4,0x85B4,0x7575,0xC485,0x7595,0x5454,0x8575,0x8585,0x7575,0x7585,0x4485,0x5F4F,0x4646,0x4F5F,0x4F5F,0x4646,0x4F4F,0x5F4F,0x5F4F,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x4F5F,0x4646,0x4644,0x4646,0x4646,0x4F5F,0x4F5F,0x4F5F,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4F4F,0x4F5F,0x4F5F,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x4F4F,0x4F5F,0x4644,0x4646,0x8544,0x4645,0x4646,0x4F5F,0x4F5F,0x4F5F,0x4646,0x4646,0x4646,0x4F4F,0x5F4F,0x5F4F,0x5F4F,0x4646,0x4F4F,0x464F
.word 0x5F4F,0x5F4F,0x4646,0x4446,0x4646,0x4646,0x4675,0x4644,0x6F5F,0x7F4F,0x8F4F,0x7F4F,0x4F7F,0x4F7F,0x4F7F,0x4F8F,0x7F7F,0x4144,0x4F7F,0x7F7F,0x7F4F,0x8F4F,0x7F4F,0x8F4F,0x8F4F,0x64D4,0x64D5,0xBFDF,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0x41D4,0x5F8F,0x54D4,0x4F9F,0xDF8F,0x6464,0x8F8F,0x64D4,0x4F9F,0x4F8F,0x6F9F,0xD4D4,0x9FDF,0x5454,0x8575,0x85C4,0x7554,0xD495,0xC4C5,0xB485,0xB495,0x95B5,0x9575,0x85B4,0x75C4,0x8585,0x95B4,0x54B4,0x7575,0x5454,0x7554,0x9565,0xD4D4,0xE4B5,0x75D4,0x85C4,0xC495,0x8565,0x95B4,0x95B4,0x85C4,0x7575,0xB485,0x7585,0x5465,0x7565,0x7575,0x5454,0x7554,0x75B4,0x7575,0xB485,0x7585,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4F5F,0x5F4F,0x4446,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x5F4F,0x4446,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4446,0x4646,0x4446,0x5F4F,0x4646,0x5F4F,0x4446,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4644,0x4F4F,0x4446,0x9575,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446
.word 0x4646,0x4646,0x4446,0x4F4F,0x5F4F,0x4646,0x4646,0x4F5F,0x4F7F,0x4F7F,0x7F4F,0x4F7F,0x4F7F,0x7F8F,0x7F4F,0x7F4F,0x8F4F,0x7F4F,0x7F4F,0x7F4F,0x4F7F,0x8F4F,0x7F7F,0x8F4F,0x9F5F,0xAFCF,0xBF8F,0x6485,0x64D5,0xCFBF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D5,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0x41D4,0x8F5F,0xAF5F,0x8F4F,0x7F4F,0x4144,0x4F9F,0x4F8F,0xDFAF,0x9FCF,0x8F4F,0x5F7F,0x5F8F,0xD4D4,0x6F9F,0x7575,0x7554,0xB485,0xC485,0xC4B5,0x8575,0x8585,0x75B4,0x7554,0x8585,0x7565,0x7565,0x5454,0x7554,0xD485,0xB5D4,0xD4D4,0x7564,0x9585,0x95C4,0x54B4,0x8585,0x85B4,0x8575,0x7575,0x7585,0x6575,0x5475,0x5454,0x7575,0x8564,0x9585,0xC485,0xC485,0x8585,0xD4D4,0xD464,0xD4D4,0xD4D4,0x8574,0x8574,0x7464,0x4654,0x4646,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4644,0x4F4F,0x4646,0x5F4F,0x4646,0x4F5F,0x4F5F,0x4446,0x4646,0x4646,0x5F4F,0x4646,0x4F4F,0x4F5F,0x4F5F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x7544,0x6595,0x4646,0x4646,0x4644,0x4446,0x4F4F,0x4F4F,0x4646,0x4644,0x5F4F,0x4646,0x4F5F,0x4644,0x4646,0x4F5F,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x7F4F,0x7F7F,0x4141,0x4F7F,0x7F7F,0x7F4F,0x7F4F,0x4144,0x4F7F,0x4F8F,0x7F8F,0x7F7F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x7F4F,0xD454,0xD464,0xD564,0xBFDF,0xD485,0xD4C4,0xCFBF,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xC5D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xD4D4,0x5F8F,0x4F9F,0x5F7F,0x5F9F,0x5F9F,0x5F8F,0x5F9F,0x4F9F,0x8F4F,0x8F4F,0x7F4F,0x8F6F,0x4F8F,0xDFAF,0x75D4,0x7554,0xC485,0x7585,0x8585,0x75B4,0x6554,0x5465,0x5454,0x5454,0x5454,0xD485,0xE4C5,0x95D4,0x6585,0x8585,0xC495,0x8595,0x7575,0x85B4,0x85C4,0x6465,0x7575,0x5475,0x6554,0x6575,0x5475,0x7554,0x8585,0xC485,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D5,0xD4D4,0x64D4,0x4464,0x4646,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x5F4F,0x5F4F,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x4446,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x9554,0x45C4,0x4644,0x4F5F,0x5F4F,0x4646,0x4F4F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x464F
.word 0x5F4F,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4F4F,0x4F7F,0x4F7F,0x8F4F,0x7F4F,0x4441,0x7F7F,0x7F4F,0x4F8F,0x4F7F,0x7F7F,0x4441,0x7F4F,0x7F4F,0x7F4F,0x4F7F,0x4F7F,0x6F7F,0xAFDF,0xBFCF,0xAFCF,0xC485,0xCFBF,0xDFBF,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE441,0x41D4,0x4F7F,0x9F5F,0x8F5F,0x6444,0x64D5,0x5FAF,0x9F4F,0x9F5F,0x4454,0x4F8F,0x4F8F,0x6454,0x8F5F,0x8F5F,0xD485,0x75D4,0x8585,0x6554,0x5475,0x5454,0xB585,0x95E4,0x7575,0xD4D4,0xB495,0x9575,0x95C4,0xC5D4,0x85C4,0xB485,0xC485,0x6585,0x5475,0x6554,0x5454,0xC485,0xB5D4,0x7585,0xD475,0xE4B4,0x85D4,0x8575,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD4D4,0x4174,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x4644,0x5F4F,0x4446,0x4646,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4F5F,0x4646,0x4F5F,0x4644,0x5F4F,0x5F4F,0x5F4F,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F5F,0x4644,0x4F5F,0x4646,0x4F5F,0x4646,0x4F4F,0x4646,0x4646,0x5444,0xA585,0x4665,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4646,0x5F4F,0x5F4F,0x5F4F,0x4646,0x4F5F,0x4F4F,0x4646
.word 0x4646,0x4646,0x4646,0x4446,0x5F4F,0x5F4F,0x7F4F,0x7F4F,0x4441,0x7F8F,0x4F7F,0x8F7F,0x7F4F,0x7F4F,0x8F4F,0x7F7F,0x4441,0x8F7F,0x7F4F,0x7F7F,0x4144,0x7F7F,0x4F7F,0xBF5F,0xD4C4,0xD564,0xD464,0xD564,0xD464,0xD464,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D5,0xE4E4,0x41D4,0xE4D4,0x8F4F,0x9F4F,0xD564,0x9F5F,0x8F4F,0xD564,0xAF6F,0x9F4F,0x5454,0x5F9F,0x4F8F,0x8F4F,0x9F8F,0x8F4F,0x5454,0xDF8F,0xD4D4,0x7575,0x8554,0xC495,0x7585,0x95C4,0xD4B4,0x5485,0xC495,0x85D4,0x8575,0x85B4,0xB485,0x6595,0x7575,0x6575,0x5454,0xC585,0xB5D4,0x7585,0xD485,0xD4C5,0x85D4,0x8575,0x95D4,0xB495,0xD485,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4C5,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xCFBF,0xDFAF,0xD464,0xD4D4,0xD4D4,0x4654,0x4F4F,0x4F4F,0x5F4F,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x4F5F,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x5F4F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4F4F,0x4446,0x4F4F,0x4646,0x4644,0x4646,0x4646,0x4644,0x8544,0xB495,0x4644,0x4644,0x4F5F,0x4644,0x4646,0x4446,0x5F4F,0x4446,0x4F4F,0x4F5F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646
.word 0x4646,0x4644,0x4644,0x4646,0x4646,0x4F6F,0x7F7F,0x7F4F,0x8F4F,0x7F4F,0x7F4F,0x7F4F,0x7F4F,0x4F7F,0x7F8F,0x4144,0x7F7F,0x8F4F,0x4F7F,0x7F7F,0x7F4F,0x7F4F,0x9F5F,0xD475,0xAF6F,0xBFCF,0xD564,0x8FBF,0xD4D4,0xCFBF,0xD574,0xD4D4,0xD4D4,0xD4E4,0x64D4,0xD4D4,0xC5D4,0xD4E4,0xD4E4,0xE4D5,0xD441,0x8FEF,0x4144,0x8F5F,0x6441,0x4F8F,0x7F8F,0x6F8F,0x64D5,0x4FAF,0x5F9F,0x5F9F,0x8F5F,0x5454,0x9F4F,0x5454,0x5F9F,0x7F8F,0xD4D4,0x75D4,0x6575,0x7575,0x5485,0xC475,0x8585,0xB454,0x9585,0x75C4,0x7575,0x7575,0x7575,0x8554,0xE495,0xD495,0xB4B5,0xC475,0xD485,0x85D4,0x9575,0x95C4,0x95D4,0x8575,0xB485,0xD464,0xCFBF,0xD4D4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xE4D4,0xD4D5,0xE4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xBFDF,0xD4D4,0xD4D4,0x5474,0x4645,0x4646,0x4646,0x5F4F,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x4F5F,0x4446,0x4F4F,0x4646,0x5F4F,0x4646,0x4646,0x5F4F,0x4446,0x5F4F,0x4646,0x4F4F,0x4646,0x4644,0x4646,0x4446,0x7565,0x4644,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x4F5F,0x4F5F,0x6544,0xB495,0x4595,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F5F,0x4F5F,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4644,0x4446,0x5F46
.word 0x5F4F,0x4646,0x4F5F,0x5F4F,0x6F4F,0x7F4F,0x4441,0x4F8F,0x7F4F,0x7F4F,0x4F7F,0x7F8F,0x7F4F,0x7F4F,0x7F4F,0x8F4F,0x7F4F,0x4144,0x7F7F,0x7F4F,0x7F4F,0x7F4F,0xD454,0x64D5,0x64D5,0xC4D4,0x8FAF,0x64D4,0xCFBF,0xD564,0xDFBF,0xD4D4,0xE4D4,0xD4D4,0xBF6F,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0x41D4,0x41E4,0xAF5F,0x9F5F,0x4F7F,0x5F9F,0x5464,0x8F4F,0xD541,0x6464,0x5F9F,0xAF5F,0x9F5F,0x9F4F,0x8F4F,0x9F5F,0x9F5F,0x4154,0x6F7F,0x85D4,0x5454,0x5475,0x5454,0x7575,0x5475,0x7575,0x7575,0x5475,0x5454,0x5454,0x8575,0x7575,0x95C4,0x95C4,0x95D4,0x8575,0x95C4,0xB495,0x8575,0x85B4,0xB495,0x7575,0x6475,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4C5,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBF6F,0xD4D5,0x4464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F5F,0x4F4F,0x4646,0x4F4F,0x4F4F,0x4646,0x4646,0x9554,0x4665,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x9575,0xC495,0x4685,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F4F,0x4F5F,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x4F6F,0x7F7F,0x7F4F,0x7F4F,0x7F8F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x7F4F,0x7F4F,0x4F7F,0x4F7F,0x4F8F,0x7F7F,0x7F7F,0x7F4F,0x6444,0x6FAF,0x6FBF,0xBFDF,0x64C4,0x64D4,0xD4D5,0xD464,0xD4D4,0xD485,0x61D4,0xD4D4,0x64D4,0xC4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x4F8F,0x4FAF,0x4F8F,0x9F5F,0x8F4F,0xD554,0x9F6F,0x9F4F,0xAFCF,0x8F5F,0x9F4F,0x6454,0x5F9F,0x9F5F,0x9F5F,0x5454,0x5F9F,0x9F5F,0x5454,0x6585,0xC485,0x7575,0x7585,0x7554,0x5454,0x5454,0x5475,0x8585,0x95C4,0xC5D4,0x5485,0xC485,0xB495,0xC485,0x8575,0xB485,0x7585,0x7575,0x7575,0x5475,0x6454,0xCFAF,0xDFBF,0xE4D4,0xD4E4,0xD4C5,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xCFAF,0x4644,0x4644,0x4646,0x4644,0x4F5F,0x4646,0x5F4F,0x5F4F,0x4446,0x4F4F,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x4F4F,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x4446,0x6446,0x6495,0x4646,0x4F4F,0x4F5F,0x4644,0x4646,0x4446,0x5446,0xC495,0x8585,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x5F4F,0x5F4F,0x5F4F,0x4446,0x4F4F,0x4646,0x5F4F,0x5F46
.word 0x4646,0x4644,0x4F5F,0x6F4F,0x7F4F,0x4441,0x4F7F,0x4F7F,0x7F4F,0x7F4F,0x8F4F,0x7F7F,0x7F4F,0x7F4F,0x7F4F,0x7F4F,0x4F7F,0x4F8F,0x7F7F,0x4441,0x4F7F,0x5F8F,0xC4D5,0xBFCF,0x64C4,0xCFBF,0xCFBF,0xCFAF,0xCFBF,0xAFDF,0xD4D4,0xE564,0xD4D4,0xBFDF,0xAFCF,0x4F7F,0x4F7F,0x4F8F,0xE464,0xC5D4,0x9F6F,0x8F5F,0x8F4F,0xAF5F,0x6464,0x4F9F,0x9F8F,0x5464,0x4F8F,0x4F8F,0xD564,0x4441,0x5F9F,0x5FAF,0x5FAF,0x9F5F,0x9F5F,0x9F5F,0x9F5F,0x9F5F,0x7554,0x7575,0x7585,0x9575,0x5485,0x7585,0xC485,0xB4C5,0x9575,0x85C4,0xD4C4,0x6585,0x8585,0x85B4,0x8585,0x7565,0x7575,0x5454,0x5454,0x5454,0x7575,0xAF5F,0x64C4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xC5D4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xC5D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x4461,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4644,0x4F5F,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4644,0x4646,0x5F4F,0x5F4F,0x4646,0x4644,0x4446,0x8575,0x4654,0x4646,0x5F4F,0x4F4F,0x5F4F,0x5F4F,0x9544,0x8585,0x75B4,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x5F4F,0x4646,0x4F4F,0x4646,0x4644,0x4646,0x4F4F,0x4646,0x4F5F,0x4646,0x4646
.word 0x4646,0x4646,0x5F4F,0x4F6F,0x7F7F,0x8F4F,0x7F4F,0x7F4F,0x7F7F,0x7F7F,0x7F4F,0x4F7F,0x7F7F,0x7F4F,0x7F4F,0x8F4F,0x4F7F,0x7F7F,0x7F4F,0x4F7F,0x7F4F,0xD4C4,0x6FBF,0x64D4,0x64D5,0x64D5,0x64D5,0x64D4,0x64D5,0xC4D4,0xD4D4,0xDFAF,0xD4D4,0x54D4,0xD464,0x8F4F,0x7F4F,0x6454,0xD4E5,0xE4D4,0xD4D4,0x8F4F,0x4F7F,0x6444,0xAFDF,0x8F4F,0x4441,0x8F8F,0x8F4F,0xD454,0xE4E4,0xE4E4,0x8FCF,0xD4D4,0x8F4F,0x9F5F,0xAF7F,0x5454,0x5F9F,0x5F9F,0x8F9F,0x5454,0x5475,0x6575,0x6585,0x7585,0x8585,0x6585,0xB485,0x8585,0xB495,0x5475,0x7575,0x7575,0x5465,0x5454,0x9585,0x8565,0xC495,0x8595,0x5454,0xCFAF,0xCFBF,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E4,0xE4E4,0xE4D5,0xD4E4,0xD4D5,0xD4E4,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4C5,0xC5E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x4641,0x4646,0x4F5F,0x4446,0x5F4F,0x4446,0x4646,0x4646,0x4F4F,0x4646,0x4F4F,0x5F4F,0x4646,0x4646,0x5F4F,0x5F4F,0x5F4F,0x4F4F,0x4646,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x7544,0x54A5,0x4646,0x4646,0x4646,0x4646,0x4646,0x8585,0x95D4,0x4685,0x4646,0x4646,0x4646,0x5F4F,0x5F4F,0x5F4F,0x4646,0x4F5F,0x4446,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4446,0x5F4F,0x4446
.word 0x4446,0x4646,0x7F4F,0x7F4F,0x4141,0x4F7F,0x7F7F,0x7F4F,0x7F4F,0x4144,0x7F4F,0x4F7F,0x4F7F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x7F4F,0x7F7F,0x4F7F,0x4F8F,0x8F9F,0xAFCF,0xAFCF,0xBFCF,0xD464,0xCFAF,0x6FBF,0xAFDF,0x64D4,0xD4D4,0xD464,0xE4D4,0xAF4F,0x6F7F,0xD4D5,0xE4D4,0xE4D4,0xD4D4,0x41D4,0x4F8F,0x5F7F,0x9F5F,0x8F8F,0x4F7F,0x5F8F,0xAF6F,0x9F7F,0x9F4F,0x41D4,0x5F8F,0x8F7F,0x4F7F,0xE441,0xD4E4,0x8F4F,0x4F7F,0x5F9F,0x5FAF,0x9F6F,0x4154,0x5444,0x5454,0x5454,0x7554,0x7554,0x6565,0x5454,0x7575,0x7575,0x6575,0x7554,0x5454,0x5454,0x8575,0xC5C4,0xD4E4,0x8575,0x85C4,0x5475,0xD554,0xC464,0xD464,0xD4D4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4C5,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD464,0x5464,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4F5F,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4F5F,0x4644,0x4F5F,0x5F4F,0x4646,0x4446,0x9585,0x4645,0x4F5F,0x4646,0x4644,0x4646,0x9585,0xB4D4,0x4675,0x4644,0x4F5F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x4F4F,0x4F5F,0x4646
.word 0x4646,0x4F5F,0x4F5F,0x7F7F,0x7F4F,0x7F4F,0x7F4F,0x7F7F,0x7F7F,0x4F7F,0x7F7F,0x7F4F,0x7F7F,0x7F4F,0x4F7F,0x4F7F,0x7F7F,0x4F7F,0x4F7F,0x4F7F,0xCF7F,0xBF5F,0x64C4,0x64D4,0x64D4,0x64D5,0x64D4,0x64D5,0x85D5,0xCFBF,0xCFBF,0xD464,0x6441,0x85D5,0x64D5,0xCFBF,0xD4D4,0xD4D4,0x6464,0x4F8F,0x7FDF,0xAF5F,0xC4C4,0x8F6F,0x4144,0x6F8F,0xD4D5,0x9F6F,0xC454,0x5F7F,0xE4D4,0xE4D4,0x54D4,0x6F8F,0x8F7F,0x7F4F,0x8F4F,0x8F4F,0x9F5F,0x5454,0x5F9F,0x8F8F,0x5444,0x5454,0x5454,0x5454,0x5454,0x5475,0x9585,0xD4E4,0x85B5,0x9554,0x95C4,0x95D4,0x8575,0xC485,0x8595,0x8554,0x5475,0xAF5F,0xAF6F,0xD464,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D5,0xE4D4,0xD4D4,0xD4C5,0xE4C5,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xC5E4,0xD4E4,0xE4D4,0xD4E4,0xBFDF,0x4464,0x4646,0x4646,0x4F5F,0x4F5F,0x4F4F,0x4F4F,0x4644,0x4646,0x4446,0x4646,0x4646,0x4446,0x4F4F,0x4F5F,0x4644,0x4F5F,0x4646,0x4446,0x4646,0x4646,0x7545,0x4644,0x4646,0x4646,0x4646,0x4646,0xB575,0x4585,0x4446,0x5F4F,0x5F4F,0x4446,0xB485,0x8595,0x4675,0x4646,0x4646,0x4F4F,0x4446,0x4646,0x4644,0x4646,0x4644,0x4F5F,0x4646,0x4F4F,0x4F5F,0x4F4F,0x4646,0x4646,0x4646,0x465F
.word 0x4646,0x6F4F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x7F4F,0x7F4F,0x4F7F,0x8F4F,0x7F4F,0x4F7F,0x7F7F,0x4441,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x7F7F,0x7F4F,0xD441,0x64D5,0xBFDF,0xBFCF,0xBFCF,0xD564,0xCFAF,0xCFBF,0xCFBF,0xD464,0xD4D4,0xAFDF,0xD4D4,0xAFDF,0xD4C4,0xD464,0xD4D4,0xE4E4,0x9F6F,0xD454,0x8FDF,0x6441,0x9FDF,0x4F8F,0x5F9F,0x8F5F,0x4F8F,0x8F5F,0xBF5F,0xD4D4,0xE4E4,0xD4E4,0x41E4,0x8F7F,0x7F4F,0x7F4F,0x8F4F,0x8F4F,0x8F4F,0x9F4F,0x9F4F,0x7F5F,0x4441,0x5441,0x5454,0xC485,0x8595,0x5485,0x85B4,0xB485,0xC485,0xC475,0xC495,0xB485,0x8554,0x85B4,0x75B4,0x6565,0x5454,0xD564,0xD564,0xBFDF,0xD4D4,0xD4D4,0xC5E4,0xE4E4,0xD4E4,0xE4E4,0xD5E4,0xE4D4,0xD5D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0x6464,0x4654,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4F4F,0x5F4F,0x4F4F,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4F5F,0x4646,0x4F5F,0x5446,0x4475,0x4446,0x4646,0x4F5F,0x4644,0x8544,0x75D4,0x4646,0x4646,0x4646,0x4646,0x9585,0x85B4,0x4675,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4675,0x4F5F,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646
.word 0x5F4F,0x7F7F,0x7F4F,0x4F7F,0x7F7F,0x4F7F,0x7F7F,0x7F7F,0x7F4F,0x7F7F,0x4F7F,0x7F7F,0x7F4F,0x4F7F,0x4F7F,0x8F7F,0x4F7F,0x4F7F,0x7F7F,0x8F4F,0xD454,0xD464,0x64D4,0x64D5,0x64D5,0x64D4,0x64D4,0x64D4,0xD4D4,0xCFBF,0xD464,0xC4D4,0xD464,0xD4D4,0xBF6F,0xAFCF,0xAF8F,0xD454,0xD4D4,0x7FDF,0x41D4,0x5F8F,0x4F7F,0x9F4F,0xD464,0x64D4,0x8F5F,0xAF4F,0xC464,0xE4E4,0xE4D4,0xC5D4,0x8F6F,0x7F4F,0x8F4F,0x8F4F,0x7F4F,0x8F4F,0x4441,0x5F9F,0x4F8F,0x4F9F,0x8F8F,0x7554,0x5444,0x5454,0x7565,0x75C4,0x7575,0x7585,0x7575,0x8554,0x75B4,0x7575,0x7554,0x6554,0x5454,0x5475,0xAF6F,0xAF6F,0xAFCF,0xC485,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xE4D4,0xD5D4,0xD4E4,0xD4E4,0xD4E4,0x51D4,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4446,0x4446,0x7554,0x4644,0x4646,0x4646,0x4646,0x5446,0x9585,0x4654,0x4446,0x4646,0x6F5F,0xC495,0x8585,0x4475,0x4646,0x4646,0x4644,0x4F5F,0x8565,0x4644,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4446,0x4646,0x4646,0x4F5F,0x4F5F,0x4644
.word 0x7F4F,0x4441,0x4F6F,0x7F7F,0x4141,0x4F7F,0x7F7F,0x4F7F,0x4F6F,0x7F4F,0x7F4F,0x7F7F,0x7F7F,0x7F4F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x4F7F,0x4F7F,0xCFAF,0xDFAF,0xD564,0xD464,0xCFBF,0xCFBF,0xCFBF,0xDFAF,0xD564,0xD464,0xAFDF,0xD4C4,0xDFAF,0xD4D4,0xC4D4,0xAF6F,0xD454,0xD4D4,0x64E4,0x6F8F,0x4F9F,0xAF5F,0x8F4F,0x8F4F,0x4F7F,0x8F9F,0x5F7F,0x7FAF,0xCFAF,0x8F6F,0x5454,0x8F8F,0x7F4F,0x4F8F,0x4F8F,0x4F8F,0x4F8F,0x4F8F,0x5F8F,0x5F8F,0x4F8F,0x5F8F,0x4164,0x6F9F,0x54C4,0x5454,0x5454,0x5454,0x5454,0x7565,0x5454,0x5475,0x5475,0x5454,0xC485,0x95C5,0x5485,0x5454,0x7FAF,0x64D5,0x85D5,0xDFBF,0xD4D4,0xE4E4,0xD4E4,0xD5E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xD4C5,0xE4E4,0xD4C5,0xE4E4,0xC5D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4C5,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x4641,0x4F5F,0x4F4F,0x4446,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x5F4F,0x4646,0x4F4F,0x4446,0x5F4F,0x4446,0x4646,0x4646,0x4646,0x5F4F,0x8544,0x4675,0x4446,0x5F4F,0x4646,0x4644,0xC485,0x4685,0x4F5F,0x4F5F,0x7546,0x85C5,0x95B4,0x4475,0x4646,0x4644,0x4F5F,0x6546,0x4485,0x4446,0x4646,0x4644,0x4446,0x4F4F,0x4646,0x5F4F,0x4646,0x4644,0x4446,0x5F4F,0x4646
.word 0x7F7F,0x7F4F,0x7F4F,0x7F4F,0x7F4F,0x7F7F,0x4F7F,0x7F6F,0x7F7F,0x4F7F,0x7F7F,0x4F7F,0x7F7F,0x4F7F,0x4F7F,0x7F7F,0x4F7F,0x7F7F,0x4F7F,0x4F7F,0xD4C4,0xD564,0xCFBF,0x8FBF,0xD564,0xD464,0xD564,0xD464,0x6FBF,0xBFDF,0xD4C4,0x64D4,0x64D4,0xD4D4,0xAFDF,0xD4D4,0x64D4,0x6F8F,0xAFDF,0xD4D4,0x9FCF,0x5454,0x5F8F,0xD564,0x64D4,0x7F4F,0x5454,0x4F8F,0x4F9F,0xDF9F,0xE4E4,0xE4D4,0x41D4,0x4F7F,0x4F8F,0x8F4F,0x8F4F,0x8F4F,0x4441,0x4F9F,0x4F8F,0xEF8F,0x64D4,0x9F4F,0x7564,0x75C4,0x5454,0x5454,0x5454,0x7554,0x5475,0x8595,0x8585,0x5485,0x8585,0x85B4,0x5475,0xAF5F,0xAFCF,0xD464,0xDFBF,0xD4D4,0xD4D4,0xE4D4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E5,0xC5E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xC5E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x54D4,0x4646,0x4646,0x4F4F,0x5F4F,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F4F,0x4646,0x4F4F,0x5F4F,0x5F4F,0x5F4F,0x4646,0x5446,0x7595,0x4646,0x4646,0x4646,0x5F4F,0x8574,0x6595,0x4646,0x4646,0x8546,0xB495,0x7595,0x4575,0x4646,0x4F5F,0x4646,0x8544,0x4585,0x4646,0x4F4F,0x5F4F,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4F4F,0x4646,0x4646,0x5F46
.word 0x4F7F,0x4F7F,0x4F7F,0x4F7F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x7F4F,0x7F4F,0x4F7F,0x7F6F,0x7F4F,0x7F4F,0x4F7F,0x4F8F,0x7F7F,0x7F7F,0x7F4F,0x9F5F,0x64D4,0x64D4,0xD4D4,0xD564,0x8FBF,0xCFBF,0xCFAF,0xCFBF,0xD564,0xD464,0xD464,0xD4D4,0xD4D4,0xDF9F,0x64D4,0x41D4,0xD564,0xD464,0xD4D4,0xD454,0xD4D4,0x7F8F,0xAF5F,0xD4C4,0x9F6F,0x5F8F,0xDFAF,0x64D4,0x5F8F,0xE464,0xE4D4,0xE4E4,0x7FEF,0x7F4F,0x7F4F,0x8F4F,0x4144,0x4F8F,0x5F8F,0x4F9F,0x4F8F,0x5F7F,0xD5E4,0x5F9F,0x6F9F,0xD454,0x5485,0x7554,0x7554,0x8585,0x54C4,0x7575,0x7575,0x7575,0x7575,0x7575,0x5454,0x7FAF,0xD564,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xC5E4,0xD4E4,0xD4D4,0xC5E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xC5E4,0xE4D4,0xD4D4,0x64D4,0x4664,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x6546,0x7585,0x4644,0x4F5F,0x5F4F,0x4646,0x8544,0x9585,0x4646,0x4446,0xB444,0x8595,0x95B4,0x4585,0x4646,0x4646,0x4646,0x9575,0x4675,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x5F4F,0x4646,0x4646
.word 0x7F7F,0x4F7F,0x7F7F,0x8F4F,0x7F7F,0x4F7F,0x7F7F,0x7F4F,0x4F7F,0x7F6F,0x7F7F,0x7F7F,0x7F6F,0x4F7F,0x4F7F,0x7F7F,0x4441,0x4F7F,0x8F5F,0x64C4,0xD4D5,0xCFBF,0xDFAF,0x74D4,0x64D5,0x64D5,0x64D4,0xD4C4,0xDFAF,0xAFDF,0xBFCF,0xD464,0xCFBF,0x64D4,0xD4D4,0xDF9F,0xDF9F,0xCFBF,0xD464,0xBFDF,0xD464,0x8F4F,0x4F7F,0x4F7F,0x4F9F,0xD554,0xD4D4,0x7FDF,0xD554,0xD4D4,0xE4E4,0xD4E4,0x41E4,0x4F7F,0x4F7F,0x8F4F,0x4441,0x7F8F,0x8F4F,0x8F4F,0x8F4F,0x7F4F,0xDF8F,0x54D4,0x6F9F,0x6F9F,0x95C4,0x5465,0x5465,0x7565,0x5475,0x5475,0x5454,0x7554,0xC485,0x5495,0xAF6F,0xBF7F,0xBFCF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D5,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4E4,0xC5D4,0xE4D4,0xE4C5,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xC5E4,0xE4D4,0xD4D4,0xC5D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x5FBF,0x4646,0x4644,0x4F5F,0x4F4F,0x5F4F,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4F5F,0x4F5F,0x4F5F,0x4F5F,0x4646,0x4646,0x9575,0x4675,0x4646,0x4646,0x4646,0xB444,0xB495,0x4654,0x4646,0x9565,0x95B4,0xB485,0x4585,0x4446,0x4646,0x4444,0x9595,0x4675,0x4644,0x4446,0x4646,0x4644,0x4F5F,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x465F
.word 0x7F4F,0x7F4F,0x4144,0x7F7F,0x7F4F,0x4441,0x7F8F,0x4F7F,0x7F7F,0x7F4F,0x7F4F,0x4F7F,0x4F7F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x5F7F,0x41D4,0x9FDF,0xD464,0xD4C4,0x6485,0x64D5,0xBFDF,0xBFCF,0xD485,0xCFAF,0x64D4,0xD4D4,0xD464,0x64D5,0xD4D4,0xD4D4,0xCFAF,0x64D4,0x6464,0xD5D4,0xDFBF,0xD4D4,0x54D4,0x41D5,0xAF4F,0x64C4,0x8F8F,0x7F7F,0x8F4F,0x8F5F,0x9F4F,0xD485,0xE4E4,0xE4E4,0x7FDF,0x7F4F,0x7F4F,0x4F8F,0x7F8F,0x4144,0x5441,0x5F9F,0x4141,0x4F8F,0x5F8F,0x64E4,0x5F8F,0x54D5,0xC464,0x54D4,0x5454,0x5454,0x5454,0x5475,0x6554,0xC485,0x8575,0x9F6F,0x6454,0x64D5,0xD4D5,0x85D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4D5,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD5D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD5E4,0xE4E4,0xD4E4,0xC5E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x64D4,0x4654,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4F4F,0x4646,0x4F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x8554,0x6585,0x4446,0x4646,0x4644,0x8546,0x85C4,0x4675,0x4444,0x9585,0xB485,0x8585,0x45B4,0x4646,0x4646,0x7546,0xB495,0x4665,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x4446,0x4646,0x4644,0x4646,0x4446
.word 0x4F7F,0x4F7F,0x4F7F,0x4F7F,0x5F7F,0x8F4F,0x4F7F,0x4F7F,0x7F7F,0x4F7F,0x4F7F,0x7F7F,0x7F6F,0x4F7F,0x7F7F,0x7F4F,0x7F5F,0x8F5F,0xAFCF,0xAF8F,0xBFCF,0xD4D4,0xDFBF,0xBFDF,0xD4C4,0xD485,0xDFAF,0xBFCF,0xBFCF,0xBF8F,0xD4C4,0xD464,0xBF6F,0xD4C4,0xD485,0xCFBF,0xAFDF,0x64D4,0x64C4,0xD4D4,0xD4D4,0x9FDF,0x6454,0xAFDF,0x4144,0x6464,0x8FDF,0x5441,0xD464,0x4FAF,0x6451,0x4F8F,0x6441,0x8F8F,0x4F7F,0x4F7F,0x4F7F,0x4F8F,0x4F8F,0x4F8F,0xAF5F,0x4154,0x7F8F,0xD4D4,0x8F5F,0xBFDF,0x5464,0xD464,0x5485,0x5454,0x7585,0x7585,0x8585,0x8575,0x5454,0x6454,0xDFAF,0xD464,0xD474,0xD4D4,0xD4D4,0xE4E4,0xC5D4,0xE4D4,0xE4E4,0xE4C5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xC5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0x4464,0x4646,0x4646,0x5F4F,0x4446,0x4F4F,0x4646,0x5F4F,0x4646,0x4644,0x5F4F,0x4646,0x5F4F,0x5446,0x4465,0x4646,0x5F4F,0x4646,0x5F4F,0x8554,0x75B4,0x5F4F,0x5F4F,0x5F4F,0x8546,0x8585,0x4675,0x7545,0x7595,0x95D4,0xB5C4,0x4585,0x4646,0x4F5F,0x9554,0x85D4,0x4665,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4F4F,0x5F4F,0x4646,0x5F4F,0x4646
.word 0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x9F5F,0x8F5F,0x7F4F,0x4F7F,0x4F7F,0x8F4F,0x7F4F,0x4154,0x4F7F,0x7F7F,0x7F7F,0x7F7F,0x8F6F,0x9F5F,0xD4C4,0xDF8F,0xD4C4,0xD464,0xD4C4,0xD4D4,0xD464,0xCFAF,0x64C4,0x85D4,0x64D4,0xD4D5,0xBF6F,0xBFDF,0xD4C4,0xBF6F,0xD4D4,0xBFCF,0xD464,0xD4D4,0xAFDF,0xBFCF,0xD4D4,0x64D4,0x5F9F,0x5FAF,0x5F8F,0xD4D5,0x8F6F,0xAF5F,0xD4D5,0x8FDF,0x4F8F,0xDF8F,0xD4E4,0x7F4F,0x7F4F,0x7F4F,0x7F4F,0x8F4F,0x8F4F,0x9F5F,0x5454,0x4F8F,0x4F7F,0xEF7F,0x8F6F,0xE454,0xD554,0x6F8F,0x7595,0x5454,0x7575,0x5485,0x5465,0x5475,0x5454,0x7FAF,0xBF6F,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xC5D4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xD4E4,0xD5D4,0xE4E4,0xD4E4,0xE4E4,0xF4E4,0xE4E4,0xD4E4,0xE4F4,0xE4D4,0xD4E4,0xD4C5,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xAFDF,0x4646,0x4F5F,0x4644,0x4F5F,0x4646,0x4F4F,0x4644,0x4646,0x4F4F,0x4646,0x4446,0x4646,0x4446,0x5485,0x4644,0x4646,0x4646,0x4646,0x9544,0x8595,0x4645,0x4646,0x4646,0x9554,0x85C4,0x4575,0x8554,0x85B4,0x85C4,0x85C4,0x6575,0x4646,0x4646,0x9575,0x95B4,0x4645,0x4644,0x4646,0x4F5F,0x4F4F,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646
.word 0x4F7F,0x4F7F,0x7F7F,0x4F7F,0x5464,0x4FAF,0x5F7F,0x6F7F,0x4F7F,0x4F9F,0x7F7F,0x5454,0x4F7F,0x4F7F,0x4F7F,0x4F7F,0x41C4,0x64D4,0xD4D4,0x64C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xCFBF,0xDFBF,0xBFDF,0xD585,0xD464,0xD4C4,0xD4D4,0xD564,0xBF6F,0xC4D4,0x64D5,0xDFAF,0xD4D4,0x85D4,0xD464,0xD4D4,0x8FCF,0x8F5F,0x8F4F,0x8F7F,0x4F8F,0x4F7F,0x4F8F,0x4F7F,0x4F7F,0xE4D4,0xD4E4,0x8F4F,0x7F4F,0x7F4F,0x8F4F,0x7F4F,0x4144,0x9F4F,0x8F4F,0x8F4F,0x7F4F,0xC444,0x41D4,0xE454,0xDF8F,0x8F4F,0x8544,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0xBF6F,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4C5,0xE4D4,0xD5D4,0xE4D4,0xD4D4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4C5,0xD4E4,0xD4D4,0xD4D4,0xC5E4,0xD4D4,0xD4D4,0x85D4,0x4651,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F4F,0x4646,0x4646,0x7544,0x7595,0x4446,0x5F4F,0x4646,0xB444,0x7595,0x4645,0x4644,0x4446,0x9575,0x85B4,0x7585,0x8585,0x9585,0x85C4,0xC495,0x8575,0x4446,0x5446,0x9595,0x75B4,0x4646,0x4646,0x4444,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x5F4F,0x4446,0x4646,0x4F5F,0x4446
.word 0x7F4F,0x8F5F,0x7F7F,0x5441,0x5FAF,0x9F5F,0x4154,0xAF5F,0x9F5F,0x4464,0x4F8F,0xBF5F,0x5F7F,0x4F7F,0x7F4F,0x7F7F,0x8FDF,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xDFBF,0xD4D4,0xD4C4,0xD464,0xBF6F,0xAF6F,0x64D4,0xCFBF,0xDFBF,0xC464,0xD464,0xD464,0xD4D4,0xE464,0xE4D4,0xBFDF,0xBF8F,0x9FCF,0x8F5F,0x5441,0xC464,0x4F8F,0x9F5F,0x6485,0x4F8F,0x9FCF,0xE4D4,0xE4E4,0x8F6F,0x7F4F,0x7F4F,0x7F4F,0x7F4F,0x8F8F,0x4F8F,0x5F8F,0x4F8F,0x4F7F,0x4F8F,0x8FEF,0xD444,0x6454,0x4F9F,0x8F8F,0x4485,0x7554,0x5485,0x7585,0x5454,0x6FAF,0x64D5,0xD4D5,0xD4D4,0xD4D4,0xDFBF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D5,0xE4E4,0xE4E4,0xD4E4,0xE5E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D5,0xD4D4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5464,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x5F4F,0x4446,0x4F4F,0x4646,0x4646,0x6F5F,0x4644,0x4446,0x8595,0x4665,0x4646,0x4646,0xB444,0xC485,0x4675,0x5F4F,0x7546,0xC495,0x7585,0x9575,0x85B4,0x95B4,0x95B4,0x95B4,0x7575,0x4654,0x8544,0x95B4,0x6585,0x4446,0x4446,0x4685,0x4646,0x4646,0x4646,0x4F5F,0x4F5F,0x4646,0x4646,0x4F5F,0x4446,0x4646
.word 0x5F7F,0x4F8F,0x5F7F,0x6F8F,0xCFAF,0x5F9F,0x5FAF,0x64D5,0xD554,0xAF6F,0x7F4F,0x64C4,0x6F7F,0x7F7F,0x5454,0x4441,0x54D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD574,0xD4D4,0xD4D4,0xC4D4,0x6F9F,0x85D5,0xBFCF,0xD4D4,0xDFAF,0xCFAF,0xD4D4,0xD4D4,0xD564,0xD4D4,0xC4D4,0xD464,0x8F5F,0xD554,0xD4D4,0x5F7F,0xDFAF,0xD5D4,0x7F8F,0x41D4,0xD4D4,0x8F4F,0x64C4,0x7F8F,0x6441,0x4FAF,0x4F7F,0x4F7F,0x4F7F,0x8F4F,0x4454,0x4F9F,0x8F4F,0x54E4,0x5F9F,0x5F9F,0x41D4,0x8F5F,0x54D4,0x5454,0x6575,0x5475,0x9F5F,0xBF7F,0xBFCF,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xC5D4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xD4C5,0xD4C5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0x64E4,0x4644,0x4646,0x4446,0x5F4F,0x5F4F,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x6546,0x4454,0x4646,0x9575,0x4475,0x4446,0x5F4F,0x8574,0x8585,0x4675,0x4646,0x9554,0x85C4,0x7585,0x75D4,0x7585,0xB495,0x8585,0xC485,0x7585,0x4675,0x9575,0xB495,0x4585,0x4446,0x8585,0x4665,0x4644,0x4644,0x4646,0x4646,0x5F4F,0x4446,0x4646,0x4646,0x4646,0x464F
.word 0x8F6F,0x4144,0x9F4F,0x6475,0xAF6F,0xAF7F,0xAF5F,0xD564,0x8FAF,0x64D5,0x4F8F,0xAFDF,0x7F7F,0x8F5F,0x9F6F,0x8F5F,0xAFDF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xBFDF,0xD485,0x64D4,0xD4D4,0x64D4,0xDFAF,0xD464,0xD464,0x64D4,0x64D4,0x6FBF,0xD4D4,0xD4D4,0xC451,0xD4D4,0xD4D4,0x4FBF,0x6F9F,0xAFDF,0x5441,0xD564,0x5474,0x4F8F,0x7FDF,0x8F6F,0xE464,0xE4E4,0x8F6F,0x4F7F,0xBF6F,0x8F4F,0x7F4F,0x7F4F,0x8F4F,0x4F8F,0x8F4F,0x5F8F,0x54E4,0x5F8F,0x5F9F,0x4F8F,0x4F9F,0xD464,0x5475,0x5454,0x4454,0x6F9F,0x64D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xE4D4,0xE4D4,0xE4D4,0xE4C5,0xE4D4,0xC5D4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD5D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4C5,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xC5D4,0xE4E4,0xD4D4,0xD4D4,0x4FAF,0x4644,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x4F5F,0x4646,0x4446,0x4446,0x7575,0x4446,0x8544,0x7585,0x4646,0x4446,0x9575,0x95B4,0x4665,0x7F5F,0xD495,0x7585,0x9575,0x85C4,0xB485,0xD485,0xB485,0x9585,0x95C4,0x6575,0x9575,0x85C4,0x4585,0x8565,0x7585,0x4644,0x4F4F,0x4646,0x4446,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4F5F,0x4646
.word 0x8F6F,0x9F5F,0x6454,0xDFAF,0xD564,0xC454,0xD564,0x64D4,0x9FDF,0x64D4,0x5F8F,0x64D4,0xDF8F,0x4FAF,0xCFAF,0x54D4,0x64D4,0xD4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xDFBF,0x64D4,0xD5D4,0xAF8F,0xAF5F,0xD4D4,0x64D4,0xD4D5,0xD4C4,0x8FBF,0xCFBF,0xD4D4,0xAFDF,0xD464,0xD4D4,0x8F4F,0x4F7F,0x8F8F,0x8F4F,0x8F8F,0xAF5F,0xD585,0x5441,0xD4D5,0xE4D4,0x44E4,0x8F4F,0x4F8F,0x64D4,0x7F9F,0x8F4F,0x4F7F,0x8F7F,0x8F4F,0x8F4F,0x64E4,0x6F8F,0xD4D4,0xD4D4,0xE4D4,0x8FDF,0xD4C4,0x5454,0x5454,0xAF6F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4C5,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xC5E4,0xD5D4,0xE4D4,0xD4E4,0xD4E4,0xE4E5,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x5485,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x8544,0x4675,0x7546,0x75B5,0x4646,0x7F5F,0xB485,0xB485,0x4645,0x9575,0x95C4,0x7575,0xB5C4,0x8575,0x95B4,0xB485,0x7595,0xB495,0xB485,0x8575,0xD495,0xC485,0x6585,0xC495,0x4585,0x4446,0x4646,0x4646,0x4F5F,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4644
.word 0xAF6F,0x8FCF,0xAF6F,0xAF7F,0xAFCF,0xBF6F,0xAFCF,0xBFCF,0xC464,0xD4D4,0xAF4F,0x64C4,0xDF8F,0x7F6F,0xD464,0xAFDF,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xD5D4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4C4,0xD4D4,0xDFBF,0xD4D4,0x64D4,0x6F8F,0xD4D4,0xDFAF,0xD464,0xD564,0xD464,0xDFBF,0xD4D4,0xD4D4,0xAF6F,0x8F7F,0x8F5F,0x6454,0x64D5,0x5F8F,0xD564,0x6441,0x8FBF,0x6454,0xD4E4,0xD564,0x41D4,0x8F7F,0xCF7F,0x64D4,0x4F8F,0x4F8F,0x4F7F,0x4F7F,0x5F7F,0x54E4,0xDF9F,0x9FDF,0xE4E4,0x8F5F,0xD4C4,0x51D4,0x54D4,0xAF4F,0xD575,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xC5D4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4C5,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xAFDF,0x4644,0x4446,0x4646,0x4646,0x5F4F,0x5F4F,0x5F4F,0x4F4F,0x4646,0x4646,0x7546,0x5495,0x7546,0x75D4,0x4644,0x7545,0x8595,0x8595,0x6445,0xB585,0x85B4,0x85B4,0xB485,0x8575,0xD485,0x7585,0x7585,0x85C4,0x8585,0x8575,0x85C4,0x85B4,0xB485,0xB495,0x4675,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4F4F,0x4F5F,0x464F
.word 0x6464,0x7FAF,0xCFAF,0x64D5,0x64D4,0x64D5,0xC4D4,0xD564,0xBFDF,0xD4C4,0xD454,0xD4D4,0xD4C4,0x8F6F,0xD4D5,0xD4D4,0xE4D4,0xE4D4,0xD5D4,0xE4D4,0xE4E4,0xE4F4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xD4D4,0x4F9F,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64C4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0xCFAF,0xD4D4,0x5F7F,0xD4D5,0x4FBF,0x64C4,0x8FDF,0x6441,0xDF8F,0xE4E4,0x8F8F,0x7F4F,0x4F8F,0xD464,0x8F6F,0x7F4F,0x7F4F,0x4144,0x44E4,0x5441,0x41D4,0xD464,0x4141,0x4F8F,0xD5D4,0xD4D4,0xAFCF,0xD464,0xCFBF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4C5,0xE4E4,0xD4E4,0xD4E4,0xC5E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4D4,0xD4E4,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4C5,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x4664,0x4646,0x4F5F,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x7546,0x7595,0x8565,0x7595,0x4575,0x8575,0x85B4,0x75B4,0x9575,0xB4B5,0xB495,0x8585,0x8595,0xB485,0x8575,0x85B4,0x8585,0x95C4,0x85D4,0x95B4,0x8585,0xB485,0x8585,0x85B4,0x4644,0x4644,0x4646,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646
.word 0xCFAF,0xAF6F,0x64C4,0xCFBF,0xCFAF,0xDFBF,0xBF6F,0x64D4,0x85D4,0xDFBF,0xD4D4,0xD4D4,0xAFDF,0x44D4,0xDFAF,0xD4D4,0xD4D4,0xD4C5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4F4,0xE4E5,0xE5E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xBFDF,0xD485,0xD4C4,0x8F6F,0xBF6F,0xD564,0xDFBF,0xAFDF,0xBFCF,0xD564,0xD464,0xD4D4,0x8F6F,0xAF5F,0xD4D4,0x5F8F,0xDFAF,0x9F8F,0x6F8F,0x6FAF,0x8FDF,0x4F7F,0xD454,0x64E4,0x4F7F,0x8F7F,0x4F8F,0xD4D5,0x8F6F,0x8F4F,0xD554,0x8FEF,0xAF4F,0x8F6F,0xD454,0x4F9F,0x4441,0xDF8F,0x64D4,0xD4D4,0xD4D5,0xD4D4,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xD4E4,0xE4D4,0xE4C5,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4F4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x44D4,0x4644,0x4646,0x4646,0x4644,0x4446,0x5F4F,0x4646,0x4F4F,0x4646,0x5446,0x8595,0xB485,0x7585,0x7565,0x9585,0x85C4,0x7585,0xD485,0x8585,0x85B4,0xB485,0xB495,0x7585,0x8585,0xB485,0xB485,0xC485,0xB495,0xB485,0xB495,0x8575,0xC495,0x6585,0x4646,0x5F4F,0x4446,0x4644,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46
.word 0xAF6F,0x64C4,0x64D5,0x64D5,0x64D4,0xC4D4,0xC4D4,0xD4D4,0xDFBF,0xD4C4,0xD464,0xD4D4,0xD4D4,0xD464,0xD464,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4F4,0xE5E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFDF,0xD564,0xD4D4,0x64D4,0x5F9F,0xD464,0xD4D4,0xD4D4,0xBF6F,0xBFCF,0xAFCF,0x64C4,0x4F8F,0x4F7F,0x4F7F,0x5F7F,0x4F9F,0x4F8F,0x8F5F,0x5454,0xAF6F,0xAF6F,0xD575,0xBF6F,0x6485,0x74D5,0xD4D4,0xD4D4,0x7F8F,0x8F8F,0x41D4,0xDF9F,0x9F5F,0xE444,0x9F4F,0x9F4F,0x4141,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D5,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4C5,0xE4E4,0xD4E4,0xE4E4,0xC5E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0x51D4,0x4646,0x4646,0x4F5F,0x5F4F,0x4F4F,0x4646,0x4446,0x4646,0x4646,0x8544,0x85B4,0x8585,0x75B4,0x8565,0xC495,0x7585,0xB485,0xB4B5,0xB495,0x8585,0xC495,0xB495,0x8575,0x85C4,0x8575,0x8585,0xB485,0x7585,0x8585,0x85C4,0xB485,0x8585,0x4685,0x4444,0x7575,0x4665,0x4646,0x4644,0x4644,0x5F4F,0x5F4F,0x4646,0x4644,0x4F4F,0x4646
.word 0xD564,0xBF6F,0xAFCF,0xBFCF,0xBFCF,0xAFCF,0xBFDF,0xD464,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xF4F4,0xE4E4,0xE5F4,0xE4F4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xCFBF,0xD4D4,0xD4C4,0x4FAF,0xDF9F,0xD464,0xD4D4,0xD464,0xD464,0x64D5,0x41D5,0x6454,0x4F8F,0x6454,0xC4D4,0x6F8F,0xCFBF,0x5FAF,0x4F8F,0x4F8F,0x4F8F,0x4F8F,0x4F8F,0x8F8F,0x8F4F,0x8F4F,0x6444,0x9F5F,0x8F6F,0xE454,0x4F9F,0x4F8F,0xCF9F,0x54D4,0x4F9F,0xD4E4,0xD4D4,0xAFDF,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4C5,0xD4E4,0xD4E4,0xE4D4,0xC5D4,0xD4D4,0xD4E4,0xE4E4,0xC5D4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x54D4,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x4446,0x5446,0x4646,0x4646,0xBF5F,0xB495,0x9585,0x7585,0x8575,0x85D4,0x85B4,0x9575,0x85C4,0x9575,0x85C4,0x85B4,0x75B4,0x8575,0xB485,0xB485,0xB485,0x85B5,0x75B4,0xB495,0xB495,0x8585,0x75B4,0x4665,0x7545,0x6595,0x4646,0x4F4F,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xAFCF,0x64C4,0x74D5,0xD564,0xD574,0xD464,0xD4C4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD5D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xF4F4,0xF4E5,0xE4E4,0xD4E4,0xD5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFCF,0xAF8F,0xD4C4,0xD4C4,0xD485,0x4FAF,0xCFAF,0xD4D4,0x64D4,0xCFBF,0xCFBF,0x6F9F,0x5F8F,0xDFAF,0x64D4,0x5F8F,0xD4D5,0x9F6F,0x8F5F,0x8F4F,0x5441,0x4F9F,0x6441,0x8F4F,0x7F4F,0x7F4F,0x7F8F,0xD554,0x8F4F,0xE444,0x4F7F,0x8FCF,0xDF7F,0x9F6F,0x8F5F,0xD4D4,0xAF6F,0xD4C4,0xE464,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4D4,0xD5D4,0xE5E4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD474,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0x9FDF,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x5446,0x4685,0x4644,0x9565,0x7595,0xB4C4,0x8575,0x8585,0x95B4,0x7585,0xC485,0xB495,0xC485,0xB485,0x8585,0x7595,0x7575,0x8585,0x7585,0x8575,0xB5B4,0x8585,0x85B4,0x8585,0x8575,0x7585,0x6465,0x7595,0x4645,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4446,0x4F4F,0x4646,0x4F5F,0x4644
.word 0xD464,0xCFBF,0xCFBF,0xCFBF,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E5,0xF4D5,0xE4E4,0xE4E4,0xE4D5,0xE4F4,0xE4E4,0xE4E4,0xF4F4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4D5,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x8FAF,0xCFBF,0xCFBF,0xD4D4,0x51D4,0x5F9F,0xD4D4,0xC4D4,0xD464,0x9F6F,0x8F7F,0x5144,0x9FDF,0x8F5F,0x4F7F,0x4F7F,0x5F8F,0x7F8F,0xD554,0x7FDF,0xE664,0x4F8F,0x4F7F,0x4F7F,0x8F4F,0x4F7F,0x6464,0x6F8F,0xC4D4,0x64E4,0x4F8F,0x4F9F,0x4F9F,0xD4D4,0x41D4,0xD4D4,0x64D4,0xD4E4,0xD4C5,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xC5E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xD5E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xDFBF,0x54D4,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x5F4F,0x4546,0x7585,0x4646,0xB454,0x85B5,0x8585,0x7575,0x95C4,0xB485,0x7575,0x95D4,0x7575,0xB485,0x7595,0xB485,0x7585,0x6554,0x7575,0xB485,0x9575,0x7575,0xC475,0xB495,0x7585,0x95C4,0x65B4,0x8575,0x45C4,0x4446,0x4646,0x5F4F,0x4646,0x4646,0x5F4F,0x4F4F,0x4646,0x4646,0x4646,0x464F
.word 0x64D4,0x64D5,0x64C4,0x74D5,0xDFBF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4C5,0xE4E4,0xC5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E5,0xF4E4,0xF4E4,0xF4F4,0xE4E5,0xE5E4,0xF4E4,0xF4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E5,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD464,0xD464,0xD4D4,0xD4D4,0xD4D4,0x5F9F,0xCFAF,0xC4D4,0x54D4,0x5F8F,0x6464,0x4F8F,0xD564,0x8F4F,0xD564,0x4164,0xD4D5,0x7FEF,0x4F8F,0xD444,0x8F5F,0x7F4F,0x4F7F,0x8F7F,0x8F7F,0xAF7F,0xAFEF,0x41E4,0x5F7F,0x4F8F,0x9F4F,0xE4D4,0x54D4,0xDF9F,0xD4D4,0xD464,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xC5E4,0xD4D4,0xD4C5,0xE4D4,0xC5D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xE4F4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4C5,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x85D4,0x9FDF,0x4646,0x4646,0x4F5F,0x4644,0x4454,0x4646,0x4446,0x9575,0x7575,0x9575,0xB485,0xB495,0x8585,0xC4B5,0x85B5,0x8585,0x95D4,0x85B4,0x7585,0x7585,0x9585,0x75B4,0x7575,0x8554,0x7575,0xB485,0x6585,0xB495,0x8585,0x9575,0x85D4,0x8575,0xC495,0x4665,0x4646,0x4646,0x4546,0x5475,0x4446,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646
.word 0xD564,0xD464,0xDFBF,0xD4D4,0xD4C4,0x64D4,0xBFDF,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD5E4,0xF4E4,0xE4E4,0xE5E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xF4E4,0xF4E4,0xF4F4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4C5,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x6FBF,0xAFDF,0xD564,0xD464,0xD4D4,0xD4D4,0x64D4,0x8F8F,0xD464,0x5F9F,0xDFBF,0xAF4F,0xD464,0x5F7F,0xCF9F,0x7FDF,0x6451,0x41D4,0x5F7F,0x7F8F,0x8F4F,0x8F5F,0x7F4F,0x7F4F,0x7F4F,0x5F8F,0x64E4,0x8F6F,0x8F7F,0x9F4F,0x4144,0xD4E4,0xAFDF,0x6464,0xD4D4,0xBFDF,0xD4C4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD5E4,0xE5D4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x54D4,0x4646,0x4644,0x4646,0x4F5F,0x5446,0x5484,0x4646,0x8575,0x8585,0x9575,0x95B4,0x75C4,0x8575,0xB495,0x85D4,0x85B4,0x95B4,0x85B4,0x85B4,0x75B4,0xB495,0x7585,0x7554,0x5475,0x7585,0x7585,0x8575,0x95C4,0x85B4,0x85B4,0x75B4,0x7585,0x95C4,0x4644,0x4644,0x6F5F,0x7564,0x4645,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x5F4F,0x4446

.word 0xBFDF,0xD4C4,0x64D4,0xD4D4,0xCFBF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E5,0xE4F4,0xE4F4,0xE5E4,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4D4,0xE4F4,0xE4F4,0xE4E5,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFDF,0xBFCF,0xD464,0xD464,0xD4D4,0xD4D4,0xC4D4,0x64D4,0x5F8F,0x6454,0xD4D5,0x5F7F,0xAF6F,0x9F5F,0x5F8F,0x64D5,0x6F8F,0xD4D5,0x7FCF,0x8F6F,0x8F6F,0x7F4F,0x7F4F,0x7F4F,0x8F6F,0xAF5F,0x64D4,0x4F8F,0xDF9F,0x41D4,0xE4D4,0xE4E4,0xCFAF,0x64D4,0x64D4,0xD4E4,0xD4C5,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xE4D4,0xE4E4,0xD4E4,0xD4D5,0xE4E4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x9FDF,0x4646,0x4F5F,0x4646,0x4646,0x4446,0x9544,0x8575,0x7546,0x85A5,0x95B4,0x9575,0x85D4,0x8575,0x85C4,0xB485,0x9585,0x85C4,0x8575,0x8575,0x7595,0x85B4,0x7575,0x7575,0x7575,0x7565,0x8575,0x8575,0x7585,0x8575,0x8585,0x7575,0xB575,0x75C4,0x4646,0x4446,0x85B4,0x4665,0x4446,0x4646,0x5F4F,0x5F4F,0x4646,0x4F5F,0x4646,0x4646
.word 0xD4C4,0xBF6F,0xD4C4,0xCFBF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE5E4,0xF4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E5,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xD474,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0x6FBF,0x8FBF,0x64D4,0xD4D4,0xDFBF,0x9FDF,0x8F8F,0x4F7F,0x8F7F,0x5F8F,0x5464,0x6F8F,0xD4D4,0x5F8F,0x9F6F,0x8F4F,0x8F4F,0x5464,0x4F8F,0x4F7F,0x4F7F,0x8F7F,0x8F4F,0xE4D4,0x8F4F,0xE444,0x41D4,0x4144,0x4F9F,0xD441,0x9FDF,0xD4D4,0xD4C4,0xE4D4,0xD4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xC5E4,0xD4E4,0xD4C5,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4C5,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xD474,0xBFDF,0x54D4,0x4446,0x4646,0x4F5F,0x4644,0x4646,0x4446,0x8595,0x7595,0x8595,0xC485,0xC475,0x7585,0xB485,0xB485,0x8585,0xB585,0xB485,0x8575,0x7575,0x75D4,0x8575,0x7575,0x5475,0x8575,0x8575,0x7585,0xC485,0x7575,0xB485,0xB4B5,0x7585,0x8585,0x4585,0x4446,0x8554,0x6585,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E5,0xE5E4,0xF4E4,0xE5E4,0xF4E4,0xE4E4,0xF4F4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4C5,0x74D4,0xBF6F,0xBFCF,0xAFCF,0xBF8F,0xD485,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD564,0xD564,0xDFAF,0xD564,0xC464,0xD564,0x54C4,0xD541,0x4164,0x8F5F,0x5F8F,0x6F8F,0x64D5,0x8F8F,0x5F8F,0xD564,0x4451,0xD4D5,0x5F7F,0x8FDF,0x7F4F,0x7F4F,0x8F4F,0x7F4F,0x6444,0xE4D4,0x8FDF,0x8F5F,0x8F4F,0x8F4F,0x5441,0xDF8F,0xD451,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4C5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xBFDF,0xD4D4,0xD4D4,0xD4D4,0x9FCF,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4644,0x7546,0x8575,0xC4B4,0x9575,0x9575,0x85C4,0x7585,0x8585,0x85B4,0x95B4,0x8585,0xB485,0x9595,0x85C4,0x7575,0x9575,0x7575,0x7595,0x7595,0xB485,0x7585,0x7575,0xC485,0x8585,0x7585,0xD4D4,0x4575,0x8564,0x7585,0x4645,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4F5F,0x4646,0x465F
.word 0x64D4,0xCFBF,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD5D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x9F6F,0x8F4F,0x5444,0xD4D4,0xD564,0x64D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xDFAF,0xBF6F,0x64C4,0xCFBF,0x4F9F,0x8FDF,0xD564,0x9FDF,0x8F4F,0x5F8F,0x7FDF,0xAF4F,0xD454,0x8FAF,0x8F6F,0x8F4F,0x5441,0x6F9F,0x4F8F,0x7FDF,0x4F7F,0x6F8F,0x41E5,0x5F7F,0x4F8F,0x4F8F,0x4F8F,0x4F8F,0x8F4F,0xD464,0xBF6F,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4C5,0xD5D4,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xD4E4,0xE4C5,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xC5E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0x5464,0x4646,0x4F5F,0x4446,0x4646,0x4F5F,0x5F4F,0x4446,0x7585,0x9575,0xD475,0xB485,0xB485,0x7585,0xB485,0x8585,0xC485,0xB495,0x8585,0x85C4,0x85B4,0x8585,0xC485,0x7575,0x7575,0x75C4,0x7585,0x7575,0x8585,0x85C4,0xB475,0x8575,0x8585,0x7575,0xC485,0x4575,0x4646,0x4644,0x4F5F,0x4644,0x4646,0x5F4F,0x4646,0x4646,0x4644,0x4646
.word 0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xC5E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xD4E5,0xE4F4,0xE5E4,0xF4E4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0x64D4,0x4151,0x9F4F,0x9F4F,0x9F5F,0x9F5F,0x9F4F,0xBF5F,0xBF8F,0xBFCF,0xD4C4,0xDFAF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xCFBF,0x8FDF,0x6485,0x8F6F,0xAF4F,0x8F5F,0x4441,0x8F4F,0x5F7F,0x4F8F,0x6F8F,0x8F6F,0x9F4F,0x64C4,0xDF9F,0x4F8F,0x6FAF,0x6464,0x4F8F,0x8F7F,0x8F4F,0x7F4F,0xAF6F,0xD554,0x54D4,0x8F8F,0x5441,0x8F8F,0xD4C4,0x64D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4C5,0xD4E4,0xE4F4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xD5D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFDF,0xD4C4,0x4FAF,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x8554,0x8585,0x8585,0x7585,0x8585,0x7575,0x8585,0xB485,0x9585,0x85B5,0x75C4,0x8585,0xB485,0xC495,0x7595,0x8575,0x7575,0x7585,0x8575,0x8575,0x75B4,0x8575,0x8585,0xB485,0xB485,0x9585,0x85D4,0x4665,0x4644,0x4F4F,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4F5F,0x4646,0x465F
.word 0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5E4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E5,0xF4E4,0xE5E4,0xF4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x64D4,0xD464,0x9F4F,0x9F5F,0xAF6F,0xAF7F,0x5475,0x5FAF,0x8F5F,0x5454,0x64D5,0x64D5,0xCFBF,0xBF8F,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64D4,0x5F9F,0xBFDF,0xD5C4,0x8F5F,0x5441,0x8F6F,0xC454,0x41D4,0x4F8F,0xAF5F,0x7F4F,0x4441,0x4FAF,0x4F8F,0xD454,0x8F7F,0xF664,0x41D4,0x7F4F,0x4F7F,0x5F8F,0x4F9F,0x5F7F,0xD4C4,0x8F5F,0xDF7F,0x5F7F,0x6FAF,0xD4E4,0xD464,0xD4D4,0xE4D4,0xE4D4,0xD4C5,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xD4D4,0xD5D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xC4D4,0xD4D4,0x4464,0x4646,0x4644,0x4F5F,0x4F5F,0x4F5F,0x4F5F,0x5F4F,0xB444,0xB485,0xB485,0xB485,0xB485,0x8575,0x85B4,0x95C4,0xC4B4,0xB485,0x8585,0x85B4,0x7575,0x8575,0x6575,0x8575,0x85C5,0x7575,0x7575,0x7585,0x7575,0xB485,0x7595,0xD4B5,0x7585,0xB485,0x6595,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4F5F,0x4646,0x4F4F,0x4646
.word 0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xD4E4,0xE4D5,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xF4E4,0xE5E4,0xE4E4,0xE4F4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xBF6F,0x9F5F,0x5454,0x7FAF,0x7FAF,0x7FAF,0x5FAF,0xAF5F,0x9F5F,0x9F5F,0x9F4F,0xD464,0x64D4,0x64D5,0xAFDF,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0x54D4,0xDFAF,0x64D4,0x64D4,0x64D5,0x8F9F,0x8FCF,0x5F9F,0xD4D4,0x8F7F,0x6444,0x7F8F,0xE4D4,0x8F6F,0x8F6F,0xD464,0x5F7F,0x5F8F,0xE4E4,0xD4E4,0x5441,0xD464,0xC4E4,0x8F6F,0x5F7F,0x7F9F,0xD4E4,0x7F8F,0x8F4F,0x8F4F,0xAF6F,0xD485,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xC5D4,0xD4E4,0xD4E4,0xE4E4,0xE4D5,0xE4E5,0xE4D4,0xD5D4,0xD4E4,0xE4D4,0xD4E4,0x64E4,0xD4E4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xBFDF,0x4564,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7546,0x75B5,0x8585,0x7585,0x8575,0x7585,0x8575,0x95B4,0x8575,0x8585,0x95C4,0xB485,0x7585,0x6575,0x7564,0x8585,0x7575,0x85D4,0x7575,0x7575,0x8585,0xC495,0x8575,0x9585,0x75B4,0x9585,0x4675,0x4644,0x5444,0x4654,0x4644,0x4646,0x4F5F,0x4646,0x4646,0x4644,0x4646,0x4446
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4F4,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD474,0xAF6F,0xAF7F,0xD564,0xD564,0xD564,0xAF6F,0xAF7F,0x5475,0x5FAF,0x4F9F,0x9F5F,0x9F4F,0xD464,0xCFBF,0xD464,0xD564,0xDFBF,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0x8FDF,0x6454,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x4F8F,0x9F4F,0x7F7F,0x8F5F,0xD554,0xD4D4,0x7FCF,0x7F8F,0xAF6F,0xAF4F,0x9F7F,0x8F6F,0x8F6F,0x8F4F,0xD541,0xAFEF,0x9F5F,0x9F5F,0xD444,0x7FDF,0x8F6F,0x7F4F,0x7F4F,0x8F4F,0x9F4F,0xAF6F,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4C5,0xE4D4,0xE4E4,0xE4E4,0xF4D4,0xE4E4,0xF4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D5,0xE4D4,0xD4C5,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xC4D4,0x4FAF,0x4646,0x4F5F,0x4646,0x4644,0x4446,0x4446,0x4646,0x7F5F,0x75D4,0xB485,0x8585,0xB485,0x8585,0x8575,0xC485,0x7585,0x95C4,0x85C4,0x8585,0x85B4,0x5475,0x7575,0x8575,0x75C5,0x7585,0x7585,0x8575,0x85C4,0x75B4,0xB485,0x85B5,0x8575,0xB4C4,0x4665,0x5444,0x6585,0x4646,0x4646,0x4646,0x4646,0x4644,0x4F5F,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xD5D4,0xD4E4,0xE4C5,0xD4D4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4D5,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xC5E4,0xD4D4,0xD4D4,0xAF6F,0x64C4,0xCFAF,0xAFDF,0xBFCF,0xAF6F,0x64C4,0x7FAF,0x5FAF,0x9F5F,0x9F5F,0x8F5F,0x9F5F,0x8F5F,0xD564,0xBFDF,0x64C4,0x74D4,0xD464,0x74D4,0xD4D4,0xD4D4,0x54D4,0x5FAF,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x4F8F,0x6454,0x54D4,0x6F8F,0x8F6F,0x8F5F,0xC454,0x7FDF,0x8F4F,0x8F4F,0x8F4F,0x6F7F,0x6464,0x64E5,0x5F9F,0x5F8F,0x64D5,0x4FAF,0x7F7F,0x4F7F,0x4F7F,0x8F5F,0x6464,0xDF8F,0xC4D4,0xAF6F,0xC5C4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD5E4,0xF4E4,0xE4E4,0xF4E4,0xE4D4,0xE4D4,0xE4D4,0xD4C5,0x64D4,0xCFAF,0xF4D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xC5D4,0xE4D4,0xD4D4,0x64D4,0x4664,0x4646,0x4646,0x4F4F,0x5F4F,0x4F4F,0x4646,0x4646,0x5446,0x8585,0x85C4,0x75B4,0x7585,0xB485,0x7575,0xB485,0x7585,0x7575,0xC485,0xB495,0x7575,0x5475,0x8575,0x85C4,0x7585,0x7585,0x7575,0x8585,0x85B4,0x8585,0xB5D4,0x7585,0xC485,0x4585,0x6465,0x8595,0x4645,0x4644,0x4F5F,0x4F5F,0x4644,0x4F5F,0x4646,0x4F5F,0x4F5F,0x465F
.word 0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD5D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xAF7F,0x64C4,0xDFAF,0xAF8F,0x74C4,0xD564,0x6464,0x7FAF,0xAF6F,0xAF7F,0x5454,0x5FAF,0x5F9F,0x8F5F,0x9F5F,0x8F5F,0xC485,0xCFBF,0xDFBF,0xAFDF,0xD4D4,0xD4D4,0xD4D4,0x51D4,0x9F5F,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xAFDF,0x8F8F,0xBF5F,0x4F7F,0x5F8F,0x7FDF,0xAF5F,0x64D4,0x4F8F,0xD554,0x5F7F,0xCF8F,0xEFBF,0x9FDF,0x8F4F,0x5F7F,0x5FAF,0x6F8F,0x7F4F,0xE444,0xDF7F,0x4F8F,0xD4D4,0x4F8F,0xBFDF,0x64D4,0xD464,0xE4D4,0xC5D4,0xD4D4,0xC5D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4C5,0xD4E4,0xD4C5,0xD4E4,0x41D4,0xE464,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x4FAF,0x4644,0x4F5F,0x4646,0x4646,0x4646,0x5F4F,0x5F4F,0x6546,0x9585,0x8575,0x8585,0x8575,0x8585,0x6585,0x8585,0xB495,0x8585,0x8565,0x8585,0x5475,0x5475,0x8585,0x8575,0x7575,0xB485,0x8575,0x8575,0xB485,0x9575,0xB4C4,0xB485,0xB495,0x7575,0x8585,0x45B4,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4C5,0xD4D4,0x64D4,0x64D5,0x64D5,0xD4D4,0xD5D4,0xCFBF,0xCFAF,0xBF6F,0xAF7F,0x5475,0x5FAF,0x9F5F,0x9F5F,0x9F4F,0x4454,0x4F9F,0x5F9F,0xD4C4,0xD464,0xD464,0xBF6F,0xD4C4,0xD4D4,0x9FDF,0x5454,0xE464,0xD4D4,0xC5D4,0xD4D4,0x51D4,0x4144,0x6475,0x4F9F,0x8F5F,0xAF5F,0xD4C4,0x4F7F,0xCF8F,0x4F9F,0xCF9F,0x8F6F,0x64D4,0x64D5,0xD4D5,0x64D4,0x9F6F,0x8F5F,0x8F5F,0x6444,0xD4D5,0x64D4,0xAF6F,0xE4D4,0x4F8F,0xD454,0xD454,0x6FBF,0xD464,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D5,0xD4E4,0xD4D5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xAF6F,0xD4D4,0xE4E4,0xD5E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0xB475,0xB485,0x7595,0x7585,0xC475,0x8595,0x7575,0x8585,0x75B4,0x7575,0xB485,0x7585,0x7554,0x8575,0x7565,0x7575,0x7595,0x8585,0x75B4,0x8575,0xD4D4,0x8585,0x95D4,0x75B4,0x9585,0x75C4,0x4545,0x6454,0x4644,0x4646,0x4644,0x4646,0x5F4F,0x4446,0x5F4F,0x4446,0x5F46
.word 0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xD5E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xC5E4,0xD4E4,0xD4D4,0xD4D4,0xD564,0xBFDF,0xBF8F,0xD4D4,0x64D4,0x85D4,0xD4D4,0xCFAF,0x64D5,0x7FAF,0x7FAF,0x7F9F,0x5F9F,0x5F9F,0x5F9F,0x5F9F,0x5F9F,0x5F9F,0xDFBF,0x64D5,0x85D5,0xAFDF,0xD4D4,0x54D4,0x9F9F,0xD454,0xD4D4,0xD4D4,0x64D4,0x6F8F,0x5FAF,0xDF8F,0x8F6F,0x8F5F,0x5444,0x6FAF,0x4F7F,0xD541,0x7F6F,0x7F4F,0x9F6F,0xAF5F,0x8FCF,0x4141,0xE564,0xAFCF,0x5475,0x4F9F,0xDFBF,0x64D4,0x8F8F,0xD4D4,0x4F7F,0x8F4F,0x54E4,0xD4D4,0xD5D4,0x6FBF,0xD4D5,0xD4E4,0xD4D4,0xE4D4,0xF4C5,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0x64D4,0x7F9F,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0x64D4,0x4FAF,0x5F4F,0x4646,0x4644,0x4F5F,0x4F5F,0x4646,0x4446,0x5F4F,0x9575,0x85B4,0x85B4,0x85B4,0x8575,0x95B4,0x75C4,0x8575,0x9585,0x8575,0x5475,0x7575,0x8575,0x8575,0x8575,0x7585,0x85C4,0x75B4,0x7575,0x8585,0x8585,0xA5B4,0xC495,0x8585,0xC485,0x6585,0x8585,0x4575,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646
.word 0xD4D4,0xD4D4,0xD4C5,0xE4C5,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4D4,0xD5D4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xDFBF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFDF,0xD4C4,0xAF6F,0xD564,0xAF6F,0x9F7F,0x9F7F,0x5454,0x4F9F,0x5441,0x5F9F,0x9F5F,0x9F5F,0x5454,0xCFBF,0xCFBF,0xCFBF,0xD464,0x8FDF,0x5454,0x5F9F,0xD4D4,0xD4D4,0x8FDF,0xAF6F,0xD575,0x8F5F,0x9F4F,0xD4C4,0xDF8F,0xE4D4,0xE4E4,0x8FDF,0xD4D4,0x4F7F,0x4F9F,0x6F8F,0x4F9F,0x8F7F,0x7F4F,0xD464,0x8FDF,0x6441,0x8F5F,0xAF5F,0xD444,0xE464,0x7F6F,0xE454,0xD4D4,0xD4D4,0xD441,0xCF7F,0x6F7F,0xD4D4,0xE4E4,0xD4D4,0xE4D4,0xE4E5,0xF4E4,0xE4E5,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0x64D4,0x64D5,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xC5D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x64D4,0x4664,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4F5F,0x4646,0x7544,0x9585,0x7585,0x8585,0x75B4,0x8585,0x85D4,0x7575,0x7575,0xB485,0x7595,0x5475,0x7575,0xC475,0x7585,0x6575,0x7585,0x7575,0x8575,0x95B4,0x9575,0x85C4,0x85C4,0x95B4,0x75B4,0xD485,0x6575,0x4646,0x4644,0x4646,0x4F5F,0x4F5F,0x4644,0x4646,0x4F5F,0x4646,0x4446
.word 0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD5D4,0xE4D4,0xE4E4,0xE4D4,0xD5E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4E4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xBFDF,0xD4C4,0xBF6F,0x6475,0x7FAF,0x5F9F,0x5F9F,0x5F9F,0x4F9F,0x9F5F,0xAF5F,0x5454,0x5F9F,0x6441,0x64D5,0x64D4,0xBFDF,0x64C4,0x5FAF,0x5F9F,0xDFBF,0xD4D4,0x64D4,0x6FAF,0x9FEF,0x8F4F,0x4441,0x5F8F,0x54E4,0x4F8F,0xD454,0xD4E4,0xD4E4,0x8F5F,0x9F6F,0x5454,0x8F6F,0xD464,0xD4D4,0xD4D4,0x8F6F,0x8F5F,0x8F5F,0x4F8F,0xDF7F,0xDFAF,0xE4D4,0xD4D4,0x8F7F,0xD454,0x7FDF,0xAF6F,0x8FDF,0xD464,0xC5D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0x74D4,0x5FAF,0xDFAF,0xD4E4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x74D4,0xD4E5,0xD4D4,0xD4C5,0xE4D4,0xD4D4,0xAFDF,0x4664,0x4446,0x4646,0x5F4F,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4544,0x85C4,0x95B4,0x75C4,0xB495,0x7575,0xC485,0x6585,0x7575,0x7585,0x7575,0x7554,0x8585,0x7585,0xC485,0x5485,0x7585,0x7575,0x8585,0xB485,0xD495,0xC495,0x8585,0xB485,0x8585,0x8595,0x4665,0x4644,0x4F5F,0x4F5F,0x4646,0x4646,0x4F4F,0x4F5F,0x4646,0x4644,0x4646
.word 0xD4D4,0xC5D4,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xC5E4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64D5,0x7FAF,0x7FAF,0x7FAF,0x5F9F,0x9F5F,0x9F5F,0x9F5F,0x5454,0x5FAF,0x5F9F,0x4F9F,0x5F9F,0xDFAF,0xD464,0xD4D4,0x9F5F,0x9F5F,0x6454,0xD4D4,0x54D4,0xAF6F,0x6454,0x4F8F,0x4F8F,0x9FDF,0x8F4F,0x7F4F,0x7F8F,0x5441,0x8FEF,0xE4D4,0xD4E4,0x9FEF,0xCF8F,0xD4D4,0x9F6F,0x4F7F,0x9FDF,0xDF7F,0x7FEF,0x5441,0x7F8F,0xD444,0xAF6F,0x6444,0x7F7F,0x7F7F,0x7F7F,0x4F7F,0x5F7F,0x6F9F,0xCFAF,0xD4E4,0xE4E4,0xE4F4,0xE5E4,0xE4D4,0xE4E4,0xD4C5,0xD4D4,0x64D4,0x64D5,0xD4D5,0xE4D4,0xF4E5,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0x74D4,0xC4D4,0x4FAF,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x4F4F,0x4646,0x9554,0x95B4,0x9575,0x8575,0x7585,0x8575,0x8585,0x7585,0x7575,0x7585,0x5475,0x7585,0x85B4,0x7585,0x8575,0x75B4,0x9585,0x75B4,0x8585,0x85B4,0x85B4,0x85B4,0x8575,0xB485,0x65C4,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4644
.word 0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xD5D4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xCFBF,0xAF6F,0x64C4,0x7FAF,0x5FAF,0x5F9F,0x5F9F,0x6F9F,0x5FAF,0xAF5F,0x5454,0x5F9F,0x5F8F,0x6441,0x64D5,0xBFDF,0x44D4,0x5F9F,0x4F9F,0xDFBF,0x8FDF,0x5464,0xDF8F,0x4141,0xD444,0x8F5F,0x7F4F,0x4F8F,0x4F7F,0x8F8F,0x7F4F,0xE454,0xD4D4,0x4441,0x6F7F,0xBFDF,0xE444,0x8FEF,0x7F4F,0xE444,0x7FDF,0x8F7F,0x7F7F,0x7F7F,0xC441,0x7FAF,0x7F7F,0x7F4F,0x4441,0x6441,0x4F8F,0xAF4F,0x6475,0xEFAF,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xBFDF,0xAF5F,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFDF,0x4664,0x5F4F,0x4646,0x4646,0x4F5F,0x4F5F,0x4644,0x4646,0x4646,0x4446,0x8544,0xB485,0x7595,0x75C4,0x8585,0x7575,0x7575,0x7585,0x8575,0x7575,0x7575,0xD475,0x8585,0x7565,0x8554,0x8575,0x7575,0x7585,0x85C4,0x8585,0x8595,0xB485,0x7585,0x8595,0x4675,0x4644,0x4F5F,0x4646,0x4646,0x4F4F,0x4F5F,0x4644,0x5F4F,0x4646,0x4F4F,0x4646
.word 0xD4D4,0xD4E4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE5E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4C5,0xE4D4,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x74D4,0xD4D4,0xD4D4,0xD564,0x64D4,0x7FAF,0xAF6F,0x9F7F,0x5454,0x5F9F,0x9F5F,0x9F5F,0x5454,0x5F9F,0x5F8F,0x4F9F,0x8F8F,0x5F8F,0x64D5,0xD4D4,0x5441,0x5F9F,0x5454,0x64D4,0x6F9F,0x4F9F,0xAF6F,0xDF8F,0x8F6F,0x8F4F,0x8F4F,0x4141,0x5444,0x5454,0xCFBF,0xD4D4,0x41E4,0xE564,0xD4E4,0xC4D4,0x5464,0x5F8F,0x6F8F,0x64E4,0x4F7F,0x4F7F,0x6441,0x7F7F,0xD444,0x7F6F,0x4F7F,0x8F7F,0xAF6F,0xD544,0xCF7F,0x8F8F,0x6475,0xE464,0xE4E4,0xD5D4,0xD4E4,0xE4E4,0xE4D4,0x64D4,0xC4D4,0x6441,0xD4D4,0xE4E4,0xE4E5,0xE4E4,0xD4D5,0xD4E4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4C5,0xD4D4,0x6464,0x4FAF,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4F4F,0x5F4F,0x4646,0x4646,0x4446,0xC485,0x8575,0x7575,0xC495,0x7585,0xB485,0x7585,0x75C4,0x7565,0x8554,0x7575,0x7585,0x6585,0xB485,0x8575,0x8575,0x75B4,0xC485,0xB495,0x85C4,0x75B4,0xB495,0x7585,0x4646,0x4646,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x5F46
.word 0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xD4E4,0xE4F4,0xE4E4,0xE4E4,0xD5D4,0xE4D4,0xC5E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xBFDF,0xD4D4,0x64D4,0xCFBF,0xAFDF,0xAF7F,0x5475,0x5FAF,0x9F5F,0x5454,0x7F9F,0x5FAF,0x5F9F,0x9F5F,0x8F4F,0x8F4F,0x7F4F,0xAF4F,0xD4C4,0x4FAF,0x9F5F,0x9F5F,0xDF8F,0x9F5F,0x9F5F,0x4454,0xE541,0x9F5F,0x5454,0x5F9F,0x9F5F,0xAF6F,0xD4D5,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xC4D4,0x6FAF,0x8F9F,0x8F4F,0x7F6F,0xE4E4,0x7FDF,0xD4D4,0xD4E4,0xD4E4,0x6F7F,0x41D4,0x4154,0xCF8F,0x54D4,0x7FDF,0x7FDF,0x7FDF,0xAFDF,0x64C4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xAFDF,0xD564,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD464,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0x4664,0x4644,0x4646,0x5F4F,0x4646,0x4644,0x4646,0x4646,0x5F4F,0x4646,0x4644,0x8565,0x7595,0xB485,0x8575,0x85B4,0x85C4,0x8585,0x7585,0x6564,0x8585,0x75C4,0x7585,0x7575,0x7575,0x7585,0x7575,0x9585,0xB5C4,0x75B4,0x8575,0x8585,0x8575,0x4575,0x5F4F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4F4F,0x5F4F,0x4446,0x4646
.word 0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD5E4,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xE4D5,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D5,0xD4E4,0xD4C5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAF6F,0x6485,0x7FAF,0x6FAF,0xAF5F,0x9F5F,0x9F5F,0x5454,0x5FAF,0x9F5F,0x8F5F,0x9F4F,0x8F4F,0x7F4F,0x6F7F,0x6F7F,0xD4D4,0x5F9F,0x4F9F,0x9F5F,0x9F5F,0x9F4F,0x9F5F,0x9F5F,0x6464,0xD4E5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD485,0x64D4,0x9F6F,0x9F5F,0x8F5F,0x8F4F,0x7F4F,0x6444,0xCF8F,0x4F8F,0x4F7F,0x4F7F,0x4F8F,0x6F8F,0x7FDF,0x4F8F,0xD4D4,0xDF8F,0xDF8F,0xD441,0xC441,0x6441,0x6FAF,0x74D5,0xE4E4,0xC5D4,0xD4E4,0xC5D4,0x64D4,0x9F6F,0xD564,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD5D4,0x4FAF,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x4446,0x4646,0x4644,0x4F4F,0x4646,0x4446,0x8565,0x64B4,0x7575,0x9585,0x85C4,0x7575,0x6575,0x8554,0x85D4,0x7575,0x7565,0x8554,0x7575,0x8575,0x8585,0x85C4,0x8575,0xC495,0x8575,0x95B4,0x75B4,0x7554,0x7575,0x6475,0x4454,0x4646,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46
.word 0xD4D4,0xC5D4,0xE4D4,0xC5D4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4C5,0xD4C5,0xD4E4,0xE4D4,0xD4D4,0x64D4,0xC4D4,0xCFAF,0xD564,0xAF5F,0x9F7F,0x6454,0x5F9F,0x5F9F,0xAF5F,0x9F5F,0x8F4F,0x4154,0x4F8F,0x4F8F,0x7F4F,0x7F7F,0x6444,0x54D5,0x5F8F,0x5F8F,0x5F8F,0x8F5F,0x8F4F,0x8F5F,0x9FDF,0x6475,0xD564,0xD4D4,0xD464,0xAF6F,0x6464,0x5F9F,0x4F9F,0x5F9F,0x4F8F,0x4F8F,0x4F8F,0x5441,0x54F5,0x6F9F,0x6464,0x6464,0x6464,0xAF6F,0x8F5F,0x7FCF,0xE4D4,0xD4E4,0x41E4,0x64D4,0x7FDF,0x8F6F,0xCF8F,0x64D5,0x6F9F,0xDF8F,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x5FAF,0xDFAF,0xD4D4,0xE4E4,0xD5E4,0xE4E4,0xD4E4,0xD4E4,0xE4C5,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xAF6F,0x4685,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x5F4F,0x4646,0x6544,0x8575,0x7575,0x7585,0xB485,0x6585,0x5475,0x7595,0x7575,0x8575,0x7585,0x8575,0x7575,0x7585,0x85B4,0x8575,0x85C4,0x85B4,0x9585,0x75B4,0x7585,0x8575,0x4565,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4446,0x5F4F,0x4646
.word 0xC5D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4C5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0xCFBF,0xAFDF,0xAF6F,0x6464,0x7FAF,0x5FAF,0x9F5F,0xAF5F,0x5454,0x5F9F,0x5F8F,0x4F8F,0x4F8F,0x7F7F,0x7F6F,0x4F6F,0x4646,0x6444,0x64D4,0x5F8F,0x5F8F,0x5F8F,0x5F9F,0x4FAF,0xCF8F,0x9F9F,0x4F8F,0x8F4F,0x9F5F,0x9F4F,0x9F4F,0x8F4F,0x8F5F,0x8F4F,0x8F4F,0x7F4F,0x8F4F,0x5F8F,0x9FEF,0x8F4F,0x8F8F,0x8F8F,0x8F8F,0x8F4F,0x8F5F,0x5F7F,0x5451,0xE4E4,0xD4D4,0xDF8F,0xD4D4,0xD4D4,0xBFCF,0x8FDF,0xAF4F,0x6454,0xE4D4,0xD4E4,0xD4D4,0xD464,0xD564,0xD464,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0x64D4,0x4FAF,0x4644,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4F5F,0x4646,0x4446,0x4446,0x6544,0x7575,0x7585,0x7575,0x5475,0x7575,0xB485,0x8575,0x7585,0x7575,0x8575,0xD485,0xB5B5,0xB485,0x7585,0x8575,0x8585,0x85B4,0x8575,0x8585,0x6565,0x4644,0x4446,0x5F4F,0x5F4F,0x5F4F,0x4446,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x4646
.word 0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xC5E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4F4,0xE4E5,0xE4E4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xC5D4,0xE4D4,0xD5D4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xAFCF,0xC4C4,0xCFAF,0x7FAF,0x5FAF,0xAF5F,0x9F5F,0x5454,0x5FAF,0x5F9F,0x5F8F,0x4F8F,0x4F8F,0x7F7F,0x7F7F,0x4F6F,0x4F6F,0x4646,0xCF9F,0x54D4,0x5F8F,0xCFAF,0xD4C4,0xD4C4,0x4F8F,0x4454,0x4F8F,0x4F8F,0x4F8F,0x4F8F,0x5F8F,0x4F8F,0x4F8F,0x4F8F,0x4F8F,0x4F8F,0x4F8F,0xAF4F,0x54E4,0x8F4F,0x4444,0x4441,0x4F7F,0xCFAF,0xEF8F,0x8F4F,0x7F8F,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7FDF,0xD4C4,0xAFDF,0x6464,0xD4D4,0xD4D4,0xCFAF,0x6F9F,0xCFAF,0xD4D4,0xD5E4,0xF4E4,0xD5E4,0xD4E4,0xD4E4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD474,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0x4664,0x5F4F,0x4646,0x5F4F,0x5F4F,0x4446,0x5F4F,0x4646,0x4646,0x4446,0x5F4F,0x7546,0x4665,0x4646,0x7544,0x7585,0x7554,0x7554,0x5475,0x9575,0x6575,0x8575,0x8565,0x8575,0x95D4,0xC4C4,0x8585,0x8575,0xC485,0x7585,0x7575,0xB485,0x5475,0x4645,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xD4D4,0xD4D4,0xE4E4,0xE4C5,0xE4E4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4C5,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0x64C4,0xD564,0xAF5F,0xAF7F,0x6454,0x5F9F,0x5F9F,0x9F5F,0x8F5F,0x8F5F,0x8F4F,0x7F4F,0x4F6F,0x4F7F,0x6F4F,0x6F4F,0x4646,0xD541,0xC4D4,0x4F8F,0xCF9F,0xCFBF,0xD4D4,0xD4D4,0xD4D4,0xAF6F,0xC4C4,0x8F4F,0x8F4F,0x8F4F,0x8F4F,0x8F4F,0x8F4F,0x8F4F,0x8F4F,0xE454,0x41D4,0x41D5,0x7F8F,0x8F8F,0x8F4F,0x4144,0xE4E4,0xD4D4,0xC4C4,0x8F8F,0xDFBF,0xE464,0xE4E4,0xD4E4,0xD4D4,0xD485,0xD4D4,0xD564,0xAFDF,0xD485,0xD4E4,0xD4C4,0xD541,0xD464,0xD4D4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4C5,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xAFDF,0x46C4,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4446,0x4646,0x4646,0x6446,0x7585,0x4644,0x8545,0x7575,0x7565,0x7554,0x8575,0x7575,0xC585,0x7585,0x7575,0x7575,0x8565,0x8585,0x95B4,0x85C4,0x75B4,0x8585,0x85C4,0x6565,0x4644,0x4646,0x4644,0x4646,0x4446,0x5F4F,0x4646,0x4F4F,0x4646,0x5F4F,0x4446,0x4646,0x4644,0x4646
.word 0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE5E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4D4,0xC5E4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xCFBF,0xDFBF,0x6FAF,0x7FAF,0x6FAF,0x5FAF,0x5FAF,0x9F6F,0x5454,0x4F9F,0x4F8F,0x8F4F,0x7F4F,0x6F6F,0x4644,0x4F5F,0x4646,0x4F5F,0x6F4F,0xC454,0xC4D4,0x64D4,0x64D4,0x9FDF,0x5441,0x8F8F,0x8F4F,0x8F8F,0x8F4F,0x8F4F,0x4441,0x4F8F,0x4441,0x4F8F,0x8F4F,0x8F4F,0xE454,0x8F6F,0x7F6F,0x4144,0x8F4F,0x4F7F,0x6F8F,0xE4E4,0xE4E4,0x6F8F,0xCF8F,0x8FDF,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xDF8F,0xD441,0x74D4,0xD564,0x6FAF,0x6F7F,0x6FAF,0x6F9F,0xD4D5,0xE4D4,0xE4E4,0xE4E4,0xE4E5,0xD5E4,0xD4E4,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD474,0xD4D4,0xE4D4,0xD4D4,0x74D4,0x64D5,0x4FAF,0x4F5F,0x4F5F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4F4F,0x4F4F,0x4646,0x7544,0x8575,0x8585,0x7585,0x5475,0x5454,0x8585,0x7565,0x7575,0x8575,0x6585,0x7585,0x7575,0x7585,0xB485,0x8585,0x8575,0x8595,0x95B4,0x5475,0x4645,0x4644,0x4646,0x4F5F,0x4446,0x4F4F,0x4646,0x4644,0x4646,0x5F4F,0x4646,0x4F4F,0x4646,0x4F4F,0x4646
.word 0xD4E4,0xC5E4,0xD4E4,0xE4E4,0xD5E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4C5,0xD4E4,0xD4C5,0xD4D4,0xD4E4,0xE4C5,0xE4E4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xC5D4,0xE4D4,0xC5D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD464,0xD564,0xAF6F,0x9F7F,0x9F7F,0x9F5F,0x9F5F,0x9F5F,0x9F4F,0x8F4F,0x7F8F,0x7F7F,0x6F7F,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x6F4F,0x8F4F,0xD541,0x4F9F,0x5F8F,0xD4D4,0x54D4,0x4F8F,0x4F7F,0x4F8F,0x4F7F,0x8F8F,0x8F4F,0x4F8F,0x4F8F,0x4F8F,0xD554,0xD4E4,0x4F9F,0x8F8F,0x8F4F,0x8F4F,0x8F4F,0xE454,0xE4E4,0xE4E4,0xE4E4,0x64D4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0x41E4,0xE4D4,0xBFCF,0xDF7F,0xD4D4,0xD564,0x8F6F,0x6454,0x64D5,0xDFAF,0xD4D4,0xD5E4,0xF4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0x64D4,0xD4E5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xBF6F,0x4675,0x4646,0x4646,0x4F4F,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4546,0xB495,0x8575,0x8575,0x7565,0x7575,0x8575,0x5475,0x7585,0x5475,0x8575,0x7575,0x8575,0x7585,0x8585,0x85D4,0x95C4,0x85C4,0x6575,0x4644,0x4646,0x4F5F,0x4F5F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F
.word 0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D5,0xD5E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE5D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xBF6F,0x6475,0x5FAF,0x5454,0x5FAF,0x9F5F,0x8F5F,0x4144,0x4F8F,0x4441,0x7F4F,0x6F7F,0x5F4F,0x5F4F,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x5441,0x64D4,0x9F6F,0xAF5F,0xD4D4,0x9F6F,0x8F4F,0x8F4F,0x7F4F,0x8F4F,0x8F4F,0x8F4F,0x4441,0xAF5F,0xE4D4,0x5464,0x4F8F,0x8FCF,0x8F4F,0x8F4F,0x4441,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xC5E4,0xE4E4,0xD585,0xE4D4,0x54D4,0x64D4,0xD4D4,0xD4D4,0x4FBF,0x8F8F,0xAF5F,0xD4C4,0xE474,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0x64D4,0x4F9F,0x4446,0x4646,0x4646,0x4644,0x4646,0x4F5F,0x4F4F,0x4646,0x4475,0x4646,0x4644,0x9575,0x7585,0x7575,0x5454,0x8575,0x7575,0x8575,0x7575,0x7585,0x7565,0x8585,0xB485,0xC485,0xB485,0x8595,0x75B4,0x6575,0x4645,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F5F,0x4646,0x4644,0x4F5F,0x5F4F,0x4646,0x4446,0x4646
.word 0xC5D4,0xD4E4,0xE4E4,0xD4D5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D5,0xE4E5,0xE4E4,0xE4E4,0xE4D4,0xC5D4,0xE4D4,0xD4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xE4C5,0xE4E4,0xE4D5,0xE4E4,0xD5D4,0xD4E4,0xD4E4,0xE4E4,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBF6F,0xC4C4,0xCFBF,0xAF6F,0xAF6F,0xAF6F,0x9F7F,0x5454,0x4F9F,0x8F4F,0x8F7F,0x6F7F,0x7F4F,0x4644,0x4646,0x4646,0x4F5F,0x4F5F,0x5F4F,0x5144,0x8F5F,0x9F7F,0xC4D5,0x64D4,0x4F8F,0xCF9F,0x64D4,0x4F8F,0x4F8F,0x4F8F,0x4144,0xAF5F,0xE4D4,0x8FEF,0x64D4,0xDF8F,0x64D4,0x4F8F,0x4F8F,0x4F8F,0xDF8F,0xF4E4,0xE5E4,0xE4E4,0xD4E4,0x64D4,0xE4D4,0xE4E4,0xE4D4,0x41E4,0xE4D4,0xD4E4,0xE4E4,0x9FCF,0xDF9F,0xD4D4,0x8FDF,0x7F4F,0x4441,0xAFDF,0xE4D4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0x64D4,0x4664,0x5F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4446,0x9585,0x4644,0x4646,0x7544,0x85C4,0x75C4,0x8575,0x7565,0x85D4,0x6575,0x8575,0x75D4,0x7575,0x7575,0x5465,0x6575,0x4465,0x4646,0x4644,0x4446,0x4446,0x4646,0x4F5F,0x4646,0x4644,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x5F4F,0x4646,0x5F4F,0x4646,0x4646
.word 0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE5E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xC5E4,0xE4D4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4E4,0xE4C5,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xD5D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC4D4,0xCFBF,0xAF6F,0xAFCF,0xAF6F,0x5475,0x5F9F,0x4F9F,0x4F9F,0x4F8F,0x4441,0x6F7F,0x7F7F,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4546,0x9F5F,0x9F5F,0x4144,0x9F4F,0xD4C4,0xC4D4,0x8F5F,0xD464,0xD4D4,0xD4D4,0x9F5F,0xD564,0xE4E4,0x54D4,0x5F9F,0x8F4F,0x4F7F,0x8F8F,0x8F4F,0x4F7F,0x5F7F,0xE464,0xE4F4,0xE4F4,0xD4E4,0xD4E4,0xD464,0xD4D4,0xE4E4,0xE4E4,0xD444,0xDF7F,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x8F5F,0x7F4F,0x7F8F,0xD441,0xD4D4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6FBF,0x4FAF,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4F4F,0x4646,0x4F5F,0xB485,0x4475,0x4646,0x4446,0x9585,0x5485,0x6465,0x7554,0x7585,0x9595,0x7575,0x8585,0x6585,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4F5F,0x4646,0x4446,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4F5F,0x465F
.word 0xE4D4,0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E5,0xE4E4,0xD4E4,0xE4D5,0xD4D4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xF4D4,0xD4E4,0xD4F4,0xD4E4,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD485,0x64D5,0xAF6F,0x6475,0x5FAF,0x5FAF,0x9F5F,0x8F4F,0x8F5F,0x7F7F,0x7F7F,0x6F6F,0x4646,0x4646,0x4F5F,0x4F5F,0x4446,0x4144,0x9F5F,0x4144,0x4F8F,0x4F8F,0x6F8F,0xDFAF,0x64D4,0xD4D5,0x51D4,0xDFAF,0xD4D4,0xD4D4,0x8F5F,0x8F4F,0x8F4F,0x4F7F,0x7F7F,0x7F4F,0x8F4F,0x5444,0xDFAF,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xCFBF,0x7FDF,0x41D4,0xD4E4,0xE441,0xD441,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0x4F7F,0x4F8F,0x4441,0x4F7F,0xE454,0xF4E4,0xE4F4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xE4D4,0x74D4,0xD564,0x4664,0x4446,0x5F4F,0x5F4F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0xB565,0x8585,0x8585,0x8575,0xB485,0x7585,0x7575,0x8585,0x8565,0x7565,0x8575,0x85B4,0x65B4,0x4646,0x4F5F,0x4646,0x4644,0x4F5F,0x4F5F,0x4F5F,0x4F5F,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x5F4F,0x4646,0x4646,0x4646
.word 0xD4E4,0xD4E4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xD4D4,0xE4D5,0xF4E4,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E5,0xE4F4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xBFDF,0x64D4,0xCFBF,0xCFAF,0x7FAF,0x6FAF,0x9F5F,0x9F5F,0x4154,0x4F8F,0x4F8F,0x4F7F,0x7F6F,0x5F4F,0x5F4F,0x4646,0x5F4F,0x4646,0x5441,0x4F9F,0x5F8F,0x4F8F,0x8F4F,0x4F7F,0x6454,0xD4D4,0x8FAF,0xCFBF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8F5F,0x5F7F,0x51D4,0x7F8F,0x7F7F,0x7F4F,0x9F5F,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xE474,0x7FEF,0x64D4,0x7F8F,0x7F4F,0x7F4F,0xD441,0xE4D4,0x8F8F,0x8F4F,0x8F4F,0x8F7F,0x7F4F,0x9F6F,0xE4D4,0xE4E4,0xE4D5,0xD5D4,0xD4E4,0xC5E4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD5D4,0x5FAF,0x4FAF,0x4F5F,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F4F,0x4646,0x7446,0x85B4,0xB585,0x94D4,0x95D4,0x75B4,0x5475,0x7565,0x8585,0x7575,0x7554,0x9575,0x6475,0x4646,0x4646,0x4F5F,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4446,0x4F4F,0x4646,0x4644,0x4F5F,0x4F4F,0x4646,0x4F5F,0x4F4F,0x4646,0x4446,0x4646,0x4644
.word 0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D5,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xC5D4,0xD4E4,0xE4D5,0xE4E4,0xF4E4,0xF4E4,0xE5E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4C4,0x6464,0x64D5,0x6FAF,0x7FAF,0x5F9F,0x5F9F,0x4F9F,0x4441,0x7F7F,0x6F4F,0x4646,0x4646,0x5F4F,0x4646,0x4146,0x9F5F,0x9F4F,0x9F5F,0x9F5F,0x8F4F,0x8F4F,0x8F8F,0x6F9F,0xD4D4,0x6464,0x4F9F,0x6441,0xDFBF,0x8FDF,0xD454,0x8F6F,0xDF7F,0xD4D4,0x4154,0x9F5F,0xD4D4,0xE4E4,0xE4E4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xE4D4,0x7FEF,0x5F8F,0x5464,0x6FAF,0x8F5F,0x5444,0xC4D4,0xEF8F,0x5451,0x7FDF,0x6444,0x4FAF,0x5F7F,0xAF6F,0x5444,0x8F6F,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xBFDF,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0x64D4,0x64D4,0x4654,0x4646,0x4F5F,0x4F5F,0x4F5F,0x5F4F,0x4646,0x4644,0x4446,0x4446,0x95C4,0x85B4,0xD4B4,0xD495,0x8575,0xD485,0x7575,0x6585,0x8575,0x7575,0x8585,0x7575,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4646,0x4446,0x4446,0x4F4F,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4F4F,0x5F4F,0x4646
.word 0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xE4E4,0xD4E4,0xD4C5,0xD4C5,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xF4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xCFBF,0xAFDF,0xD564,0xAF5F,0xAF6F,0x9F7F,0x5454,0x4F9F,0x8F8F,0x6F6F,0x4F7F,0x5F4F,0x4446,0x4646,0x4446,0x4144,0x8F5F,0x5454,0x5F9F,0x9F5F,0x9F4F,0x4441,0x4441,0x4F8F,0xCFAF,0xD4D5,0xD564,0xD5D4,0x64D4,0x6F7F,0x64D5,0x4F8F,0xE4C4,0x54D4,0x5F8F,0xDFAF,0xE4D4,0xD4E4,0xE4E4,0x54D4,0x4F9F,0x4F7F,0xD464,0x7FEF,0x4441,0x8F4F,0xD564,0x8F5F,0x9F5F,0x8F4F,0xD4D4,0x8F4F,0x8F5F,0xDF7F,0x4F7F,0x5F8F,0xBF6F,0x8F5F,0x6F8F,0x5FAF,0xD454,0xE4E4,0xD4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFDF,0xAF6F,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x6546,0xC495,0x9595,0xB5D4,0x5465,0x8575,0x7575,0x6595,0x5475,0x5475,0x7575,0x7575,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4F4F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4C5,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0x64D5,0x6FBF,0x7FAF,0x5FAF,0x5454,0x5F9F,0x4F9F,0x4F8F,0x7F7F,0x7F6F,0x4646,0x4F5F,0x4F5F,0x4446,0x9F5F,0x9F5F,0x9F5F,0x9F5F,0x8F5F,0x4154,0x9F5F,0x8F5F,0x8F4F,0x8F4F,0xD464,0xBFDF,0x74C4,0x5F8F,0xD4D4,0x8F9F,0x8F5F,0x4441,0xDFAF,0xD4E4,0xE4D4,0xD4E4,0xC5E4,0xE4D4,0xE4D4,0xD4D4,0xD454,0x4F8F,0x6454,0x8FDF,0xD564,0x8F9F,0x4F7F,0x5F8F,0xAF6F,0x4F7F,0xD454,0x8FDF,0x4144,0x4F8F,0x8F7F,0x8F4F,0x6444,0x9F6F,0x9F5F,0xAF6F,0xE444,0xE4E4,0xD4D4,0xD4E4,0xC5D4,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD574,0xD564,0x4F9F,0x4644,0x4446,0x4646,0x4644,0x4F5F,0x4F4F,0x5F4F,0x4646,0x4646,0x7574,0x4446,0x8575,0xD4C4,0x5475,0x7575,0x7554,0x7575,0x8575,0x8575,0x85C4,0x75D4,0x4485,0x4446,0x5F4F,0x4646,0x4F4F,0x4F5F,0x4F5F,0x4644,0x4F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F5F,0x4F4F,0x4646,0x4F4F,0x4646,0x5F4F,0x5F4F,0x4646,0x4F5F,0x4646
.word 0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4C5,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4D5,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD564,0x6464,0xAF6F,0x9F6F,0x9F5F,0x8F5F,0x7F4F,0x6F7F,0x4F7F,0x4446,0x4646,0x4646,0x5441,0x5F9F,0x9F5F,0x5454,0x5F9F,0x5F9F,0x5F8F,0x4F9F,0x4F8F,0x8F4F,0x8F4F,0x4F7F,0x64C4,0x4F9F,0x6F9F,0x5FAF,0x8F4F,0x5444,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xEF7F,0x7FEF,0x9F5F,0x5F8F,0x4F7F,0x5F8F,0xD4D4,0xD4D4,0xD4E4,0x8F5F,0x8F4F,0x9F4F,0x6444,0x5F8F,0xAFCF,0x8F5F,0xD564,0x8F7F,0x4F8F,0x4F8F,0x4F8F,0x6464,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0x64D4,0x6FBF,0x4654,0x4646,0x4F5F,0x4F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x8585,0x4675,0x8565,0x9585,0x7565,0x5485,0x8585,0x8575,0x6585,0x7585,0x7575,0x8585,0x7575,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F4F,0x4646,0x4646,0x5F4F,0x4646,0x4F4F,0x4644,0x4646,0x4646,0x4446,0x5F46
.word 0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xD4E4,0xC5E4,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE5E4,0xF4E4,0xF4E4,0xF4E4,0xE4E5,0xE5F4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4C5,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xBFDF,0xBFCF,0xAF6F,0xAF7F,0xAF6F,0x5475,0x4F9F,0x4441,0x4F7F,0x7F6F,0x5F4F,0x5F4F,0x4446,0x9F5F,0xAF5F,0x5464,0x6FAF,0xAF6F,0x6465,0x5FAF,0x9F5F,0x9F5F,0x4154,0x4F8F,0x8F4F,0x8F8F,0x5F7F,0x9F6F,0x9F5F,0x5F7F,0x6FAF,0xD4D5,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD485,0x4F7F,0x5454,0x7FDF,0xD4C4,0xD4D4,0x85D4,0xD4D4,0x4F7F,0xE4D4,0xD4E4,0x54D4,0x8F8F,0x8F4F,0x5F8F,0xD4D4,0x8F6F,0xD454,0x8FCF,0x5441,0x6FAF,0x9F8F,0xC464,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xAF6F,0xAF7F,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x5F4F,0x4646,0x4F4F,0x9544,0x8585,0x85C4,0x95B4,0x7575,0x7554,0x7565,0x7554,0x7585,0x7575,0x5454,0x7575,0x7585,0x4475,0x4646,0x4646,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4F5F,0x4F5F,0x4F5F,0x5F4F,0x4646,0x4F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646
.word 0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xD5E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xE4C5,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D5,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x64D5,0x7FAF,0x5FAF,0x5FAF,0x9F5F,0x8F4F,0x6F7F,0x7F4F,0x4646,0x4646,0x5146,0x6454,0x5464,0x7FAF,0x5FAF,0x6FAF,0x5FAF,0xAF6F,0xAF7F,0x5454,0x5F9F,0x9F5F,0x8F4F,0x4454,0x9F4F,0x5454,0x4F9F,0x4441,0x5F9F,0x6F9F,0x6FAF,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD485,0x4FAF,0x8F8F,0x8F8F,0xD441,0x8FDF,0x5441,0x9F9F,0x8F5F,0xD564,0x4F8F,0xE464,0xD4E4,0x6464,0x4FAF,0x5441,0xDFAF,0x5441,0xDF9F,0x5F7F,0x8F6F,0x8F7F,0x6454,0x4F8F,0xD4C4,0xE4D4,0xD4D4,0xD4D4,0xD464,0xDFBF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x4F9F,0x4F5F,0x4446,0x5F4F,0x4446,0x4F4F,0x4646,0x4646,0x4644,0x4646,0x7546,0xC485,0x95B4,0xD4D4,0x5485,0x7554,0x75D4,0x8575,0x9565,0x7575,0x9585,0x8575,0x7585,0x7585,0x4644,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x5F4F,0x5F4F,0x4446,0x4646,0x4F5F,0x465F


.word 0xBFDF,0xBFDF,0xBF4F,0xBFDF,0xBFD4,0xE4E4,0xE4E4,0xE4D4, 0xD3E4,0xD3E4,0xD3E4,0xD3E4,0xD3E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4C3,0xE4D4,0xD4E4,0xE4E3,0xE4E4,0xF4E4,0xE4E4,0xE4F4,0xE3E4,0xF4E4,0xE3E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xD4E4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD562,0xC4AF,0x73AF,0x529F,0x429F,0x6F7F,0x6F42,0x4642,0x6262,0xAFD5,0xBF62,0xD5C4,0xD462,0xD583,0xD462,0xD5D4,0xAFD4,0xBF83,0xAF62,0x62D5,0xD5D5,0xAF62,0xAF73,0x9F52,0x8F52,0x4241,0x8F7F,0x7F42,0x8F41,0x8F42,0x8F41,0x8F41,0x8F42,0x428F,0x8F8F,0x7F41,0x8F42,0x7F42,0x418F,0x8F42,0x8F42,0x417F,0x427F,0x427F,0x417F,0x8F42,0x8F42,0xD4D4,0x52D4,0x7FD5,0x8F41,0xE452,0x41D4,0x7F42,0x7F42,0x428F,0x427F,0x428F,0xD441,0xD4D4,0xD562,0xAF62,0xBF62,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x73AF,0x73AF,0x469F,0x4646,0x4646,0x5F46,0x4246,0x4646,0x464F,0x4646,0x4642,0x4646,0x9373,0x4643,0x4642,0x7343,0xC483,0x5273,0x5273,0x7373,0x8373,0x8363,0x52C4,0x7383,0x8383,0x9383,0x83C4,0x83B4,0x8373,0x6383,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4246,0x4F46,0x464F,0x4642,0x464F,0x5F46,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646
.word 0xBFDF,0xBFDF,0xBF4F,0xBFDF,0xBFD4,0xE4D3,0xE4F4,0xE4D3, 0xD3E4,0xD3E4,0xD3E4,0xD3E4,0xD3E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E3,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x52D5,0x52AF,0x8F52,0x6F41,0x6F7F,0x5146,0xBF62,0x6283,0x62D5,0x61D5,0xAFD5,0x62D4,0xBFD4,0xBF83,0xD4C4,0x62D4,0xD4D4,0xD4D4,0xBF62,0x6273,0x73AF,0x5262,0x529F,0x418F,0x428F,0x417F,0x427F,0x417F,0x427F,0x417F,0x417F,0x7F7F,0x7F42,0x7F42,0x8F41,0x7F41,0x7F42,0x7F8F,0x4241,0x7F7F,0x7F7F,0x7F41,0x7F41,0x417F,0x418F,0xD4E4,0xD4E4,0xD47F,0x427F,0x418F,0xD4D4,0x417F,0x428F,0x417F,0x417F,0x8F7F,0x9F42,0xD4D4,0xD4D4,0xD562,0x6262,0xD462,0xD4D4,0xD472,0xD4D4,0x62AF,0xAFC4,0xAF73,0x4645,0x465F,0x465F,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x7246,0x7383,0x8373,0x8382,0x93B4,0x7383,0x7352,0x5273,0x8373,0x7352,0x7383,0x5273,0x6373,0x9363,0x6383,0x8383,0x8373,0x8373,0x5283,0x5F46,0x5F46,0x4246,0x464F,0x4646,0x4F46,0x464F,0x464F,0x4F4F,0x464F,0x464F,0x5F46,0x464F,0x4F46,0x4F4F,0x5F46,0x4646
.word 0xBFDF,0xBFDF,0xDFDF,0xBFDF,0xBFD4,0xE4E4,0xE4E4,0xE4E4, 0xD3E4,0xD3E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4C3,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D3,0xE4E4,0xE3E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xC4BF,0xC4BF,0x62AF,0xAF62,0x5252,0x6F8F,0x6F41,0x6241,0x62D5,0xAFD5,0xD4C4,0x62D4,0x62D4,0xBFD4,0xD4C4,0x62C4,0x72D4,0xD4D5,0xD4D4,0xD462,0x62D5,0xD5D5,0xAF62,0xAF62,0x9F73,0x5252,0x418F,0x7F8F,0x7F7F,0x7F7F,0x417F,0x427F,0x427F,0x418F,0x417F,0x417F,0x427F,0x417F,0x428F,0x7F41,0x9F42,0x7F41,0x7F42,0x7F41,0x8F42,0x7F42,0x7F42,0xE452,0xD4D4,0xE451,0x8F41,0x8F42,0xD4D4,0x527F,0x7F41,0x7F41,0x7F42,0x4142,0xC49F,0x8F52,0xD4D4,0xBFD4,0xAFC4,0x62D5,0x52AF,0xD4AF,0x62D4,0xD562,0x6262,0x73AF,0x4642,0x4646,0x4246,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x445F,0x8373,0xC483,0xC493,0xC483,0xB493,0x7352,0x7352,0x8373,0x6373,0x8373,0x7383,0x8383,0xB483,0x8393,0x83B4,0x8363,0x83B4,0x8373,0x5473,0x4646,0x4646,0x4646,0x4642,0x464F,0x464F,0x424F,0x464F,0x5F4F,0x4646,0x5F4F,0x465F,0x4F42,0x4F46,0x465F,0x4246
.word 0xBFDF,0xBFDF,0xDFDF,0xBFDF,0xBFD4,0xE4E4,0xE4E3,0xE4E4, 0xD3E4,0xD3E4,0xE4C3,0xE4D4,0xE4C3,0xE4E4,0xE4D4,0xC3D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE3D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D3,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D3,0xE4D3,0xE4E4,0xE4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xAFC4,0x52C4,0x52AF,0x4241,0x6F6F,0xAFC4,0xBFC4,0xD483,0xD4AF,0xD4AF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4C4,0xBFD4,0xD4D4,0xD462,0xD4D5,0x62D5,0xC4AF,0x62AF,0x5252,0x5242,0x4241,0x4241,0x7F7F,0x7F41,0x7F7F,0x7F7F,0x7F42,0x427F,0x418F,0x428F,0x8F7F,0x7F42,0x7F41,0x7F42,0x7F41,0x7F42,0x417F,0x417F,0x418F,0xE47F,0xE4D4,0xD4D4,0x8F62,0x7F41,0x83E4,0x7FD4,0x8F52,0x7F42,0x7F41,0x7F8F,0xAF42,0x7F62,0xD4D4,0xD4D4,0xD4D4,0xD562,0x6262,0x51D5,0x629F,0x52AF,0xC4BF,0x9F62,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x464F,0x4642,0x465F,0x4646,0x4646,0x4446,0x9373,0x83C4,0xB483,0xC3B3,0x7383,0x5252,0x7352,0x7373,0x7352,0x6373,0x6373,0x8373,0x7373,0x8383,0x8363,0x7393,0x7383,0x8383,0x4463,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x464F,0x464F,0x464F,0x4F46,0x4F46,0x4F4F,0x4F46,0x5F46,0x464F,0x5F46


.word 0xbFbf,0xbFb4,0xDFDf,0xBFef,0xBFD4,0xE4E4,0xE4E4,0xE4D4,0xD3E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4C3,0xE4D4,0xD4E4,0xE4E3,0xE4E4,0xF4E4,0xE4E4,0xE4F4,0xE3E4,0xF4E4,0xE3E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xD4E4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD562,0xC4AF,0x73AF,0x529F,0x429F,0x6F7F,0x6F42,0x4642,0x6262,0xAFD5,0xBF62,0xD5C4,0xD462,0xD583,0xD462,0xD5D4,0xAFD4,0xBF83,0xAF62,0x62D5,0xD5D5,0xAF62,0xAF73,0x9F52,0x8F52,0x4241,0x8F7F,0x7F42,0x8F41,0x8F42,0x8F41,0x8F41,0x8F42,0x428F,0x8F8F,0x7F41,0x8F42,0x7F42,0x418F,0x8F42,0x8F42,0x417F,0x427F,0x427F,0x417F,0x8F42,0x8F42,0xD4D4,0x52D4,0x7FD5,0x8F41,0xE452,0x41D4,0x7F42,0x7F42,0x428F,0x427F,0x428F,0xD441,0xD4D4,0xD562,0xAF62,0xBF62,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x73AF,0x73AF,0x469F,0x4646,0x4646,0x5F46,0x4246,0x4646,0x464F,0x4646,0x4642,0x4646,0x9373,0x4643,0x4642,0x7343,0xC483,0x5273,0x5273,0x7373,0x8373,0x8363,0x52C4,0x7383,0x8383,0x9383,0x83C4,0x83B4,0x8373,0x6383,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4246,0x4F46,0x464F,0x4642,0x464F,0x5F46,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646
.word 0xbFbf,0xbFb4,0xDFDf,0xBFef,0xBFD4,0xE4D3,0xE4F4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E3,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x52D5,0x52AF,0x8F52,0x6F41,0x6F7F,0x5146,0xBF62,0x6283,0x62D5,0x61D5,0xAFD5,0x62D4,0xBFD4,0xBF83,0xD4C4,0x62D4,0xD4D4,0xD4D4,0xBF62,0x6273,0x73AF,0x5262,0x529F,0x418F,0x428F,0x417F,0x427F,0x417F,0x427F,0x417F,0x417F,0x7F7F,0x7F42,0x7F42,0x8F41,0x7F41,0x7F42,0x7F8F,0x4241,0x7F7F,0x7F7F,0x7F41,0x7F41,0x417F,0x418F,0xD4E4,0xD4E4,0xD47F,0x427F,0x418F,0xD4D4,0x417F,0x428F,0x417F,0x417F,0x8F7F,0x9F42,0xD4D4,0xD4D4,0xD562,0x6262,0xD462,0xD4D4,0xD472,0xD4D4,0x62AF,0xAFC4,0xAF73,0x4645,0x465F,0x465F,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x7246,0x7383,0x8373,0x8382,0x93B4,0x7383,0x7352,0x5273,0x8373,0x7352,0x7383,0x5273,0x6373,0x9363,0x6383,0x8383,0x8373,0x8373,0x5283,0x5F46,0x5F46,0x4246,0x464F,0x4646,0x4F46,0x464F,0x464F,0x4F4F,0x464F,0x464F,0x5F46,0x464F,0x4F46,0x4F4F,0x5F46,0x4646
.word 0xbFbf,0xbFb4,0xDfDf,0xBFef,0xBFD4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4C3,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D3,0xE4E4,0xE3E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xC4BF,0xC4BF,0x62AF,0xAF62,0x5252,0x6F8F,0x6F41,0x6241,0x62D5,0xAFD5,0xD4C4,0x62D4,0x62D4,0xBFD4,0xD4C4,0x62C4,0x72D4,0xD4D5,0xD4D4,0xD462,0x62D5,0xD5D5,0xAF62,0xAF62,0x9F73,0x5252,0x418F,0x7F8F,0x7F7F,0x7F7F,0x417F,0x427F,0x427F,0x418F,0x417F,0x417F,0x427F,0x417F,0x428F,0x7F41,0x9F42,0x7F41,0x7F42,0x7F41,0x8F42,0x7F42,0x7F42,0xE452,0xD4D4,0xE451,0x8F41,0x8F42,0xD4D4,0x527F,0x7F41,0x7F41,0x7F42,0x4142,0xC49F,0x8F52,0xD4D4,0xBFD4,0xAFC4,0x62D5,0x52AF,0xD4AF,0x62D4,0xD562,0x6262,0x73AF,0x4642,0x4646,0x4246,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x445F,0x8373,0xC483,0xC493,0xC483,0xB493,0x7352,0x7352,0x8373,0x6373,0x8373,0x7383,0x8383,0xB483,0x8393,0x83B4,0x8363,0x83B4,0x8373,0x5473,0x4646,0x4646,0x4646,0x4642,0x464F,0x464F,0x424F,0x464F,0x5F4F,0x4646,0x5F4F,0x465F,0x4F42,0x4F46,0x465F,0x4246
.word 0xbFbf,0xbFb4,0xDfDf,0xBFef,0xBFD4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xD4E4,0xE4C3,0xE4D4,0xE4C3,0xE4E4,0xE4D4,0xC3D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE3D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D3,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D3,0xE4D3,0xE4E4,0xE4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xAFC4,0x52C4,0x52AF,0x4241,0x6F6F,0xAFC4,0xBFC4,0xD483,0xD4AF,0xD4AF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4C4,0xBFD4,0xD4D4,0xD462,0xD4D5,0x62D5,0xC4AF,0x62AF,0x5252,0x5242,0x4241,0x4241,0x7F7F,0x7F41,0x7F7F,0x7F7F,0x7F42,0x427F,0x418F,0x428F,0x8F7F,0x7F42,0x7F41,0x7F42,0x7F41,0x7F42,0x417F,0x417F,0x418F,0xE47F,0xE4D4,0xD4D4,0x8F62,0x7F41,0x83E4,0x7FD4,0x8F52,0x7F42,0x7F41,0x7F8F,0xAF42,0x7F62,0xD4D4,0xD4D4,0xD4D4,0xD562,0x6262,0x51D5,0x629F,0x52AF,0xC4BF,0x9F62,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x464F,0x4642,0x465F,0x4646,0x4646,0x4446,0x9373,0x83C4,0xB483,0xC3B3,0x7383,0x5252,0x7352,0x7373,0x7352,0x6373,0x6373,0x8373,0x7373,0x8383,0x8363,0x7393,0x7383,0x8383,0x4463,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x464F,0x464F,0x464F,0x4F46,0x4F46,0x4F4F,0x4F46,0x5F46,0x464F,0x5F46



.word 0xE4E4,0xD4E4,0xE4D5,0xD5E4,0xF4E4,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xC5D4,0xE4E4,0xC5D4,0xE4D4,0xD4E4,0xD4D5,0xE4E4,0xD4E5,0xE4F4,0xE5E4,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xD4E4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD574,0xD564,0x5F9F,0x4F9F,0x6F8F,0xAFDF,0x64D4,0xDFBF,0xD4D5,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xC4D4,0xCFBF,0xCFBF,0xCFAF,0x7FAF,0x9F5F,0x4454,0x4441,0x7F8F,0x7F4F,0x4F8F,0x4F9F,0x5454,0x6454,0xAF5F,0x6475,0x5FAF,0xAF6F,0x4F7F,0x4F7F,0x7F7F,0x4F7F,0x4F7F,0x4F7F,0x4F8F,0xD464,0xD4D4,0xD464,0x7FDF,0x7F4F,0xC4E4,0xD564,0x5F8F,0x4F8F,0x6F7F,0xDF7F,0xD4E4,0xD4E4,0xD441,0xD4D4,0xD4D4,0xD464,0xD4C4,0x64D5,0x64D5,0xAF5F,0xAF7F,0xAF7F,0xAF5F,0x5454,0x5451,0x4644,0x4F4F,0x5F4F,0x4646,0x4646,0x5F4F,0x4646,0x4F5F,0x5F4F,0x4646,0x4646,0x4646,0x4446,0x85B4,0xC485,0x6585,0x5454,0x7565,0x7554,0x7554,0x7565,0x8554,0x75C4,0xD475,0x7595,0x9575,0xB485,0x8575,0x4575,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4646,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4646
.word 0xE4D4,0xE4C5,0xE4D4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4F4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xD4C5,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4C5,0xE4D4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xF4F4,0xE5E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4D5,0xD4E4,0xE4E4,0xE4D4,0xC5E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6FAF,0x9F6F,0x4154,0xBF6F,0x64C4,0xD4D4,0xD464,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xDFBF,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD564,0xD585,0xAF6F,0x64C4,0x5F9F,0x4F8F,0x8F4F,0x7F7F,0x8F4F,0x8F4F,0x9F4F,0x9F5F,0x9F5F,0x9F5F,0xD554,0x8F5F,0x7F7F,0x7F7F,0x7F4F,0x7F4F,0x7F4F,0x7F4F,0x8F4F,0xD464,0xDF9F,0xD464,0x8FDF,0x7F4F,0xD4E4,0x7FBF,0x8F5F,0x7F4F,0xDF8F,0xD4E4,0xD4D4,0xE4E4,0xD464,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xAFDF,0xAFCF,0x6464,0x5F9F,0xAF5F,0x6475,0x5FAF,0x5FAF,0x5154,0x5454,0x4646,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x5F4F,0x4446,0x4446,0x4646,0x7546,0x8585,0x8585,0x5475,0x5454,0x7554,0x7585,0x9585,0x7575,0x5485,0x7575,0x6585,0xD485,0x8585,0x75D4,0x8585,0x4446,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4F5F,0x4446,0x4646,0x4644,0x4644,0x465F
.word 0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xE4F4,0xE4E5,0xE4F4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE5E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD474,0xAFCF,0x64C4,0xCFAF,0x64D5,0xD4D4,0xCFBF,0xD4D5,0xDFBF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8FBF,0x64D5,0xCFAF,0x7FAF,0x5F9F,0x4F9F,0x4441,0x4F7F,0x8F4F,0x8F4F,0x8F4F,0x5454,0x5F9F,0x7F8F,0x7F8F,0x7F4F,0x7F4F,0x4F7F,0x4F7F,0x4F7F,0x4F7F,0x8F7F,0x8F6F,0x8F6F,0x7F8F,0x8F5F,0xD441,0xC4E4,0x41D4,0x8F5F,0x7F6F,0xD4D4,0xD4D4,0xD4D4,0x6464,0xE4E5,0xE4D4,0xD4D4,0xD4D4,0xCFBF,0xD4D4,0xD564,0x7FAF,0x5FAF,0x7F9F,0x7F9F,0x7FAF,0xAF5F,0xAF6F,0x5454,0x5164,0x4644,0x4646,0x4646,0x4F5F,0x4F5F,0x4644,0x4646,0x4646,0x4646,0x4F5F,0xC454,0xD485,0x95B4,0x95C5,0x75C4,0x5454,0x7554,0x8575,0x8565,0x7575,0xD485,0x8554,0x7575,0x7575,0x8585,0x8575,0x4475,0x4646,0x4F5F,0x4446,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4F5F,0x4646,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xD4E4,0xE5D4,0xE5D4,0xE4E4,0xE5E4,0xE4F4,0xF4E4,0xE4F4,0xE4E4,0xE4F4,0xE4D5,0xE4E5,0xD4F4,0xE5E4,0xE4E4,0xE4E4,0xF4E4,0xE5E4,0xE4E4,0xE4D5,0xE4E5,0xE4E4,0xD5E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x7FBF,0xCFAF,0xDFAF,0xD564,0xD485,0xDFBF,0xD4C4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD564,0xAF6F,0xAF7F,0x5454,0x5F8F,0x4F8F,0x7F8F,0x4F7F,0x8F7F,0x8F8F,0x7F4F,0x7F4F,0x7F4F,0x4F7F,0x4F7F,0x4F7F,0x7F7F,0x7F4F,0xAF6F,0x6485,0xD4D5,0xC4C4,0x6464,0x4FAF,0xE4D4,0x64D4,0x6F8F,0x4FAF,0x64E4,0x64D4,0x7F9F,0xE4D4,0x6F8F,0xDF7F,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x64D4,0x64D5,0xAF6F,0x9F5F,0xAF5F,0xAF6F,0xAF6F,0x6465,0xDFAF,0x6F9F,0x4454,0x4646,0x4F5F,0x4646,0x4646,0x4F4F,0x4644,0x4646,0x4F5F,0x9564,0x85C4,0xB495,0xB495,0xD485,0x85B5,0x5475,0x5465,0x7565,0xC485,0x8575,0x8585,0x8575,0x75C4,0x8575,0x7565,0x7585,0x7585,0x4646,0x4646,0x5F4F,0x4646,0x5F4F,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4F5F,0x4646,0x465F
*/



/*   Dragon Girl
Fish_320:
.word 0x4646,0x4646,0x4646,0x4646,0x8373,0x8373,0x7373,0x4646,0x4646,0x4646,0x4646,0x4646,0x7342,0x5263,0x7373,0x4643,0x4646,0x4646,0x7363,0x5252,0x5263,0x6383,0x4646,0x6346,0x7352,0x8373,0x4646,0x4646,0x4646,0x4646,0x6342,0x7352,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6F46,0x7F6F,0x6F6F,0x7F6F,0x4141,0x8F42,0x6F7F,0x6F6F,0x6F6F,0x7F6F,0x7F6F,0x6F6F,0x5F6F,0x6F7F,0x5F7F,0x6F6F,0x6F6F,0x6F7F,0x6F7F,0x6F7F,0x6F6F,0x6F7F,0x6F7F,0x6F7F,0x7F6F,0x6F6F,0x6F6F,0x7F6F,0x7F6F,0x7F7F,0x7F7F,0x4646,0x6346,0x4663,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5242,0x8363,0xD4D4,0x4373,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6342,0x8383,0x93D4,0x7383,0x5263,0x4643,0x4646,0x4646
.word 0x4646,0x4642,0x4646,0x4642,0x4346,0x7393,0x7373,0x8383,0x4673,0x4642,0x4646,0x4646,0x5246,0x5273,0x5273,0x6273,0x4643,0x4342,0x7373,0x7352,0x7352,0x5263,0x7383,0x7352,0x7352,0x7352,0x4673,0x4246,0x4646,0x4442,0x7352,0x7373,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x6F5F,0x7F42,0x6F6F,0x427F,0x428F,0x7F8F,0x6F6F,0x7F6F,0x6F6F,0x6F7F,0x6F7F,0x6F6F,0x6F6F,0x7F6F,0x6F6F,0x6F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F7F,0x6F7F,0x7F7F,0x6F6F,0x7F7F,0x7F41,0x7F7F,0x467F,0x4342,0x52D4,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x7341,0x9383,0x93D4,0x5273,0x4646,0x4642,0x4246,0x4646,0x4642,0x4646,0x4642,0x4246,0x4646,0x4246,0x4646,0x4642,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4646,0x4642,0x4646,0x4642,0x4246,0x4646,0x4642,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4246,0x4646,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4442,0x8352,0xB3D4,0x73B4,0x4373,0x4646,0x4642,0x4642,0x465F
.word 0x4646,0x5F46,0x4646,0x4646,0x4646,0x5246,0x52C4,0x6352,0x7362,0x7373,0x4246,0x4246,0x7346,0x7352,0x7352,0x7352,0x5252,0x6352,0x5252,0x5252,0x5273,0x5273,0x5252,0x5273,0x7352,0x7352,0x6252,0x7373,0x7352,0x7352,0x5252,0x7352,0x4683,0x4646,0x465F,0x4646,0x465F,0x4646,0x424F,0x7F7F,0x7F7F,0x7F41,0x8F52,0x7F41,0x6F6F,0x6F7F,0x6F7F,0x7F6F,0x7F6F,0x6F6F,0x6F6F,0x7F6F,0x6F6F,0x426F,0x5241,0xD462,0x5283,0x7F52,0x6F6F,0x417F,0x427F,0x7F7F,0x7F42,0x427F,0x6F7F,0x6F7F,0x6F7F,0x6F6F,0x427F,0x417F,0x7F7F,0x4646,0xC352,0x4273,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5242,0x8373,0x93D4,0x5283,0x4643,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x465F,0x4246,0x8352,0x93C3,0x73B4,0x4473,0x4646,0x4642,0x4646,0x4646,0x4646
.word 0x4246,0x4646,0x4246,0x5F46,0x4646,0x4646,0x8363,0x7373,0x7352,0x6273,0x7373,0x4646,0x5242,0x7352,0x5252,0x5263,0x5273,0x5273,0x5252,0x5252,0x5252,0x5252,0x6352,0x5252,0x5273,0x5252,0x5273,0x5273,0x5273,0x5252,0x7373,0x5273,0x5273,0x4646,0x4646,0x4646,0x4646,0x6F5F,0x4142,0x427F,0x7F7F,0x427F,0x7F41,0x6F7F,0x7F6F,0x7F6F,0x7F6F,0x426F,0x7F6F,0x7F6F,0x7F5F,0x6F42,0x6F6F,0x6241,0xD4D4,0xD4E4,0x9F62,0x6F7F,0x6F6F,0x5242,0x7F41,0x417F,0x7F6F,0x7F7F,0x427F,0x6F6F,0x7F7F,0x7F42,0x7F6F,0x6F41,0x417F,0x426F,0x7342,0x73D4,0x4642,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4246,0x4146,0x7342,0xB483,0xD4B3,0x8393,0x4352,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5246,0x4652,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4246,0x7352,0x9383,0x73B4,0x4273,0x4646,0x4642,0x5F46,0x4646,0x465F,0x4646
.word 0x4246,0x4343,0x4343,0x4343,0x4246,0x4646,0x4646,0x7373,0x7352,0x7352,0x5273,0x5273,0x7352,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5273,0x5252,0x5252,0x5252,0x4273,0x4246,0x5F46,0x6F46,0x7F42,0x5252,0x6F7F,0x427F,0x5241,0x428F,0x7F6F,0x6F6F,0x427F,0x6F7F,0x6F7F,0x6F6F,0x6F6F,0x416F,0x6F6F,0x526F,0xD562,0xD4E4,0xAF62,0x427F,0x6F6F,0x4241,0x7F8F,0x7F6F,0x7F7F,0x6F7F,0x6F7F,0x6F7F,0x6F41,0x6F7F,0x6F6F,0x6F41,0x427F,0x417F,0x6F6F,0x437F,0x9352,0x6373,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x5242,0x8373,0x93C4,0x93D4,0x8383,0x4252,0x4246,0x4646,0x5F46,0x5246,0x6352,0x5263,0x4641,0x4642,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4246,0x4646,0x4646,0x4642,0x465F,0x4246,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4246,0x5F46,0x4646,0x465F,0x4646,0x4242,0x8373,0x8383,0x8393,0x7393,0x7383,0x5252,0x4242,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4246,0x5252,0x8283,0xC493,0x7383,0x4643,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642
.word 0x7346,0x8383,0x8373,0x7373,0x8373,0x4283,0x4246,0x7373,0x5273,0x5273,0x5263,0x5252,0x5273,0x5252,0x5252,0x5252,0x5242,0x5252,0x5242,0x4252,0x4252,0x4252,0x5252,0x4252,0x5241,0x5242,0x5252,0x5252,0x5263,0x5252,0x5252,0x6352,0x5273,0x7373,0x4273,0x4646,0x416F,0x5241,0x7F9F,0x7F7F,0x4142,0x8F52,0x6F7F,0x6F7F,0x6F42,0x7F7F,0x427F,0x6F6F,0x6F7F,0x6F6F,0x6F6F,0x6F6F,0x5241,0x52AF,0x7F7F,0x7F7F,0x7F42,0x7F6F,0x6F6F,0x6F7F,0x7F7F,0x6F6F,0x6F6F,0x426F,0x7F6F,0x6F7F,0x6F7F,0x7F6F,0x7F6F,0x7F7F,0x7F41,0x4142,0x7F7F,0x5243,0x83D4,0x4663,0x4646,0x4646,0x465F,0x4446,0x4646,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4146,0x6352,0x8373,0xC483,0xD4B3,0x93B3,0x8383,0x7383,0x7373,0x7373,0x7373,0x5252,0x5263,0x4242,0x4252,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x8352,0x9393,0x93C4,0x7383,0xB483,0x9383,0x9393,0x83B4,0x5252,0x5252,0x6373,0x5252,0x5252,0x8373,0x8383,0x9293,0x83C4,0x73B4,0x4673,0x4642,0x6F46,0x4646,0x465F,0x465F,0x4246,0x4646,0x4646
.word 0x4646,0x4646,0x8373,0x7373,0x6352,0x5252,0x6373,0x7352,0x5273,0x5252,0x5252,0x5252,0x5252,0x5242,0x5242,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x7341,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x4252,0x5252,0x7352,0x7352,0x5252,0x7F42,0x4241,0x428F,0x427F,0x527F,0x9F41,0x7F8F,0x7F6F,0x6F7F,0x7F7F,0x6F41,0x7F7F,0x7F6F,0x5F6F,0x426F,0x7F6F,0x526F,0x527F,0x418F,0x4152,0x8F42,0x6F7F,0x7F6F,0x7F6F,0x416F,0x7F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F,0x7F42,0x6F6F,0x6F7F,0x6F7F,0x8F42,0x427F,0x7F41,0x467F,0x7352,0x6293,0x4652,0x465F,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x6341,0x5252,0x7352,0x7363,0x8373,0x93B4,0xD4C4,0xB393,0x9393,0xB3B3,0xB3D4,0x93D4,0x93C3,0x93C3,0x8383,0x6273,0x5252,0x4252,0x4252,0x4642,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4642,0x5F46,0x4246,0x4646,0x4642,0x4646,0x4646,0x465F,0x4F46,0x4646,0x5F46,0x4646,0x465F,0x4646,0x465F,0x4644,0x465F,0x5F46,0x4646,0x4642,0x4246,0x8352,0xC4C3,0xB483,0x6373,0x5273,0x5252,0x6373,0x5252,0x8352,0x7383,0x7383,0x7373,0x8373,0x8373,0x8383,0x5252,0x5252,0x7373,0x4452,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646
.word 0x4246,0x4646,0x6346,0x8383,0x7352,0x5273,0x7373,0x7352,0x7352,0x5252,0x5252,0x5252,0x5242,0x5252,0x5252,0x5252,0x5252,0x5273,0x7352,0x5273,0x7352,0x5252,0x7352,0x5252,0x5252,0x5252,0x5252,0x5273,0x5252,0x5252,0x5242,0x5241,0x5252,0x5252,0x4252,0x4141,0x8F41,0x8F42,0x7F41,0x8F52,0x8F52,0x6F7F,0x7F42,0x426F,0x7F7F,0x426F,0x6F7F,0x6F7F,0x426F,0x7F6F,0x6F6F,0x5252,0x42BF,0x9F62,0x9F52,0x6F8F,0x6F7F,0x6F7F,0x6F42,0x6F6F,0x6F6F,0x6F6F,0x426F,0x6F6F,0x6F6F,0x6F7F,0x416F,0x6F7F,0x426F,0x6F6F,0x417F,0x7F41,0x5242,0x7F8F,0x4246,0xC373,0x6373,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4241,0x5252,0x5252,0x5252,0x5252,0x8373,0xC483,0xC483,0x93B4,0x93C4,0x83C4,0xC493,0xC4E4,0xD4D4,0xD4B3,0x9382,0x83B3,0x7383,0x7373,0x5252,0x4252,0x4242,0x4642,0x4646,0x4646,0x4F46,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x8342,0x8393,0x83B4,0x7373,0x5252,0x4644,0x4646,0x4442,0x5252,0x6352,0x7363,0x5252,0x5273,0x5273,0x6352,0x6352,0x5252,0x4452,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4642,0x465F,0x4646,0x4646
.word 0x4646,0x5F46,0x4646,0x7363,0x5273,0x5252,0x5252,0x5252,0x5252,0x5252,0x5242,0x5252,0x5252,0x5252,0x7352,0x7352,0x7373,0x5252,0x7373,0x7373,0x5252,0x7373,0x7383,0x5273,0x7352,0x8383,0x5252,0x5252,0x5252,0x7352,0x5252,0x5252,0x4252,0x4152,0x417F,0x8F42,0x8F52,0x7F41,0x527F,0x429F,0x6F7F,0x6F42,0x7F7F,0x7F6F,0x7F41,0x6F7F,0x6F6F,0x426F,0x6F6F,0x6F41,0x836F,0x62D4,0xD562,0xC4AF,0x7F52,0x6F7F,0x6F7F,0x6F6F,0x7F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F,0x6F7F,0x6F6F,0x7F6F,0x427F,0x628F,0x4141,0x467F,0x6352,0x5252,0x5252,0x4642,0x4642,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4246,0x4646,0x4246,0x5242,0x5252,0x7352,0x7363,0xC483,0x8383,0xB483,0xB493,0x8383,0x8383,0x93B4,0xC4C4,0xB493,0x93C3,0xC3C3,0xB3C4,0xB382,0x8393,0x5273,0x5252,0x4142,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4446,0x4246,0x4646,0x465F,0x4246,0x5F46,0x4646,0x4642,0x5F46,0x4246,0x7383,0x7363,0x5273,0x5273,0x4644,0x4246,0x4646,0x4646,0x4646,0x4246,0x4646,0x4246,0x4646,0x4642,0x4246,0x4646,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646
.word 0x4246,0x9383,0x8383,0x7383,0x5273,0x7352,0x5252,0x5252,0x5252,0x5252,0x5252,0x5273,0x5252,0x8373,0x5273,0x5252,0x7352,0x7383,0x5252,0x5273,0x7383,0x5252,0x7373,0x5273,0x7373,0x7373,0x5263,0x8352,0x8383,0x5273,0x7352,0x7373,0x5252,0x7F41,0x9F52,0x5252,0x7F8F,0x7F42,0x5252,0x7F8F,0x7F7F,0x7F7F,0x427F,0x417F,0x7F6F,0x427F,0x7F6F,0x7F6F,0x7F41,0x6F7F,0x6252,0x62AF,0x62E5,0x62E4,0x7F7F,0x7F6F,0x6F6F,0x4142,0x7F7F,0x426F,0x7F5F,0x6F6F,0x6F6F,0x6F6F,0x5F41,0x6F6F,0x7F6F,0x426F,0x7F6F,0x6F7F,0x427F,0x7F7F,0x8F41,0x8FC4,0x6F41,0x5243,0x7352,0x9373,0x6373,0x4646,0x4646,0x4446,0x4246,0x5F46,0x4646,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4646,0x4146,0x5242,0x7352,0x7373,0x7373,0x7373,0x7383,0x7383,0x83B4,0x8373,0x8383,0x83B4,0xB4C4,0xC493,0xC493,0xD493,0x93D4,0x93D4,0x8393,0x7373,0x5252,0x4652,0x465F,0x4246,0x5F46,0x4646,0x465F,0x4646,0x4642,0x4246,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x7342,0x7393,0x5273,0x7373,0x5252,0x7383,0x4452,0x5F46,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x5F46,0x4646,0x4246,0x4646,0x465F
.word 0x8373,0x73B4,0xB483,0x6373,0x6362,0x5252,0x5252,0x5252,0x5252,0x5252,0x5273,0x5273,0x7373,0x7373,0x5273,0x7363,0x7352,0x8373,0x7373,0x7352,0xC483,0x7383,0x8352,0x83D4,0x5263,0x8373,0x7383,0x6352,0x8373,0x5273,0x7352,0x7373,0x5273,0x428F,0x42D4,0x529F,0x7F52,0x5241,0x42AF,0x418F,0x426F,0x7F7F,0x7F41,0x7F7F,0x4142,0x6F6F,0x417F,0x417F,0x7F6F,0x417F,0xAF52,0xD452,0xF5D4,0x8F52,0x6F6F,0x6F7F,0x417F,0x7F5F,0x7F42,0x6F41,0x6F6F,0x6F6F,0x6F42,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F,0x6F6F,0x7F6F,0x427F,0x428F,0x418F,0x437F,0x5252,0x7363,0x8383,0x73B3,0x4652,0x5F46,0x7384,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4246,0x5242,0x5252,0x6352,0x6362,0x5263,0x7373,0x7373,0x7373,0x73B4,0x8373,0x8383,0x83B4,0xC483,0xC483,0xC493,0xC483,0xC393,0xC393,0x8393,0x7373,0x4263,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4642,0x4646,0x4246,0x7363,0x5273,0x5273,0x7352,0x8373,0x8373,0x7383,0x7373,0x7373,0x4642,0x465F,0x4646,0x5F46,0x4246,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646
.word 0x8383,0x7383,0x7373,0x5273,0x5273,0x5252,0x5252,0x5252,0x7352,0x5252,0x5273,0x7373,0x5252,0x7352,0x7352,0x8383,0x7352,0x7373,0x8373,0x6383,0x7373,0x8383,0x8363,0x8383,0x5273,0xC483,0xC483,0x5273,0xD483,0x5283,0x5252,0x7373,0x8F51,0xD441,0xC49F,0x629F,0x7F9F,0xD552,0x8F41,0x7F7F,0x7F6F,0x7F41,0x7F7F,0x7F42,0x6F7F,0x6F7F,0x7F7F,0x7F7F,0x427F,0x416F,0x5252,0xD4D5,0xAF62,0x6FC4,0x7F7F,0x426F,0x416F,0x416F,0x526F,0x7F7F,0x6F7F,0x6F6F,0x6F6F,0x6F6F,0x426F,0x7F5F,0x6F6F,0x6F6F,0x426F,0x416F,0x7F6F,0x6F41,0x7F7F,0x8F41,0x8F42,0x7F42,0x7373,0x8383,0x93C4,0xD483,0x7393,0x8363,0x8363,0x4443,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4642,0x4646,0x4642,0x4646,0x466F,0x5242,0x5273,0x7352,0x7352,0x7352,0x5273,0x7373,0x7373,0x7373,0x7373,0x8373,0x8373,0x83B4,0xB483,0xB493,0xD493,0x83C4,0xB3C4,0x83D4,0x7383,0x4252,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4246,0x7383,0x5273,0x5273,0x8373,0xB483,0x7383,0x8383,0x7373,0x7352,0x5252,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4246,0x4646,0x4646,0x465F,0x4446,0x4246,0x4646,0x464F,0x5F46,0x4646
.word 0xB483,0x8352,0x6383,0x6362,0x5252,0x5252,0x6352,0x5252,0x5273,0x8373,0x8352,0x8383,0xC483,0x5273,0x7352,0xB483,0x7393,0x7373,0xC483,0x7383,0x7352,0x93C4,0x5283,0x9363,0x73C4,0x7363,0x8373,0x7383,0x8373,0x73B4,0x7373,0x6252,0x429F,0xD562,0xD462,0x8FE4,0x5242,0x529F,0x427F,0x7F41,0x7F42,0x427F,0x417F,0x417F,0x7F6F,0x7F42,0x427F,0x427F,0x6F7F,0x8F41,0x628F,0x62AF,0x41D5,0x6F41,0x426F,0x417F,0x7F7F,0x7F52,0x7F7F,0x7F42,0x6F41,0x6F7F,0x6F6F,0x6F41,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F6F,0x6F7F,0x7F7F,0x7F52,0x4141,0x4142,0x5252,0x8373,0x93C3,0x8383,0x7383,0x7373,0x4283,0x5F46,0x4646,0x4646,0x4642,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4246,0x4246,0x5252,0x5273,0x5273,0x5273,0x5273,0x7352,0x7352,0x7352,0x7373,0xB483,0x7383,0x8373,0xB493,0x8383,0xB493,0xB483,0xB493,0x8383,0x7393,0x4273,0x4646,0x465F,0x4646,0x4246,0x4646,0x4246,0x4646,0x4646,0x5F46,0x4646,0x5F44,0x4646,0x4246,0x6246,0x7373,0x7352,0x7352,0x7383,0x7373,0x8383,0x6373,0x4652,0x4642,0x4646,0x4246,0x4246,0x4646,0x465F,0x4646,0x4646,0x4644,0x4F46,0x4646,0x4642,0x4646,0x4646,0x4642,0x4646,0x4642,0x4646,0x464F,0x4646,0x4646,0x4646
.word 0x7373,0x7373,0x5252,0x5273,0x5252,0x5252,0x7373,0x5273,0x7352,0x5252,0x7363,0x7373,0x6383,0x9373,0x7363,0x8373,0x7373,0x5273,0x8373,0xC483,0x7352,0xB483,0x7352,0x8352,0x93D4,0x7352,0xD483,0x5293,0x6373,0x5283,0x8373,0x5273,0x628F,0x52BF,0x62AF,0x428F,0x629F,0x7F9F,0x7F41,0x7F7F,0x417F,0x417F,0x7F7F,0x6F42,0x7F7F,0x417F,0x7F7F,0x7F7F,0x6F7F,0x4142,0xAF52,0xAF73,0xAFC4,0x7F7F,0x6F7F,0x7F6F,0x527F,0x5262,0x417F,0xE441,0x6F7F,0x7F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x426F,0x6F6F,0x6F7F,0x6F41,0x6F7F,0x6F42,0x6F7F,0x6F7F,0x4241,0x7F8F,0x637F,0x5273,0x7342,0x5273,0x8363,0x7363,0x7373,0x7393,0x4643,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x4246,0x4246,0x5252,0x5273,0x5252,0x5273,0x5273,0x5252,0x7363,0x5273,0x7373,0x7373,0x7373,0x8373,0x83B4,0x83B4,0x8383,0x6373,0xD483,0x8383,0x7383,0x4643,0x4646,0x4246,0x4646,0x464F,0x4646,0x4646,0x4642,0x4646,0x4246,0x4646,0x4646,0x4646,0x5246,0x7373,0x7383,0x7373,0x83C4,0x8373,0x7373,0x4473,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4642,0x4642,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4246,0x4646,0x4642
.word 0x8383,0x7363,0x5252,0x5252,0x5252,0x5273,0x7352,0x7373,0x7373,0x8383,0x5273,0x7352,0x8352,0xB3D4,0x93C3,0x7373,0x8373,0x93C3,0x5283,0x83B4,0x73B3,0x7373,0xD483,0x5273,0x8373,0x5273,0xB483,0x5283,0x8373,0x52D4,0xC473,0x529F,0x52E4,0x628F,0x7FD5,0x9F52,0x8F62,0x427F,0x428F,0x417F,0x427F,0x417F,0x7F7F,0x7F6F,0x427F,0x7F7F,0x7F7F,0x4241,0x416F,0x427F,0x739F,0xC4AF,0x5262,0x6F6F,0x7F41,0x4152,0x9FC4,0xD4D4,0x6FD4,0x427F,0x7F41,0x6F42,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F6F,0x6F6F,0x6F7F,0x6F7F,0x6F42,0x6F6F,0x426F,0x7F6F,0x8F6F,0x8F42,0x4241,0xC483,0x5283,0x5252,0x7352,0x8373,0x8393,0x83B4,0x7383,0x4663,0x465F,0x5F46,0x4646,0x4646,0x4246,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x5241,0x5252,0x5252,0x5252,0x5252,0x5252,0x5273,0x7352,0x7352,0x7352,0x7373,0x6373,0x8383,0xB483,0x7363,0x7373,0x7363,0x7383,0x5263,0x4452,0x4246,0x4646,0x464F,0x4646,0x4642,0x465F,0x4646,0x465F,0x4646,0x4646,0x4246,0x5F46,0x7346,0x5252,0x5263,0x8373,0x7373,0xC483,0x7383,0x5252,0x4642,0x465F,0x4646,0x4642,0x465F,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4F46,0x4646,0x4646
.word 0x5273,0x5273,0x5252,0x5252,0x7352,0x7373,0x5273,0x5252,0x7373,0xB483,0xC493,0x6383,0x8373,0xD4B4,0x8383,0x5273,0x8363,0x7383,0x5263,0x8373,0x7373,0x7352,0x8383,0x5263,0xD483,0x6393,0x7352,0x5273,0x8383,0x7393,0xD452,0xAF52,0x8F42,0x9F52,0x5241,0xC49F,0x7F52,0x9F42,0x7F7F,0x7F41,0x7F42,0x7F41,0x7F42,0x7F6F,0x7F41,0x427F,0x7F41,0x7F7F,0x7F6F,0x417F,0x9F52,0xAF73,0x6FD4,0x7F6F,0x6F7F,0x4152,0xF562,0x52D4,0x52D4,0x8F7F,0x7F7F,0x416F,0x6F6F,0x426F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x426F,0x6F7F,0x6F6F,0x7F6F,0x7F6F,0x7F6F,0x6F6F,0x527F,0x7F7F,0x9373,0x93C4,0x6383,0x5252,0x7373,0xC483,0x8383,0x93B4,0x6393,0x4646,0x4646,0x4246,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x5246,0x6383,0x7373,0x7383,0x7352,0x5273,0x5252,0x5252,0x5252,0x5273,0x5252,0x7352,0x7352,0x6263,0x8352,0x7352,0x7383,0x5263,0x8373,0x7383,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x7346,0x7373,0x7352,0x7383,0x7352,0x8383,0x83B4,0x7352,0x5273,0x5263,0x4252,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646
.word 0x5263,0x5252,0x5252,0x5252,0x7352,0x8373,0x8383,0x7373,0x5273,0x7363,0x7383,0x8352,0x5273,0x7373,0x7373,0xB3C4,0x5283,0x8352,0xB3C4,0x5273,0xC483,0x83D4,0x7352,0x7393,0x7373,0x7383,0x8373,0x83D4,0x7363,0x7363,0xAF52,0x8F62,0x4241,0x428F,0x529F,0xAF62,0x428F,0x429F,0x7F41,0x417F,0x417F,0x7F7F,0x6F7F,0x4241,0x417F,0x417F,0x426F,0x6F41,0x427F,0x8F41,0x8F52,0x5252,0x7FAF,0x6F6F,0x526F,0x527F,0xD49F,0x9F62,0x8FD4,0x6F41,0x7F7F,0x7F7F,0x7F6F,0x6F7F,0x6F41,0x6F42,0x6F7F,0x427F,0x6F6F,0x417F,0x6F6F,0x6F41,0x7F6F,0x7F6F,0x7F7F,0x416F,0x8F42,0x8352,0xC483,0x8373,0x7383,0x6352,0x7373,0xB483,0x7373,0x9383,0x6283,0x4443,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4246,0x6263,0x6352,0x6352,0x5252,0x5273,0x5273,0x5252,0x5283,0x8363,0x5273,0x5273,0x5263,0x5252,0x6373,0x5273,0x7352,0x9383,0xC3D4,0x93D4,0x9383,0x4652,0x4246,0x5F46,0x4646,0x4246,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x5242,0x6263,0x7352,0x7373,0x7373,0x7373,0x5273,0x7373,0x5273,0x8373,0x7373,0x8373,0x7383,0x4662,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x5F46,0x4446,0x4646,0x4642,0x4646,0x465F,0x5F46,0x4F46,0x4646,0x4646,0x4646
.word 0x5252,0x5252,0x5252,0x7352,0x5273,0x8373,0x7373,0x8352,0x93D4,0x5273,0x5252,0xC483,0xD493,0x8393,0x7352,0xC483,0x83C4,0x7363,0x7383,0x5273,0x8383,0x6373,0x7352,0x83C4,0x5263,0x5273,0x7373,0xC483,0x8373,0x5293,0x52D4,0x428F,0x528F,0x528F,0x9F52,0x4252,0x628F,0x8F52,0x427F,0x427F,0x427F,0x7F7F,0x427F,0x7F7F,0x4241,0x7F7F,0x8F52,0x6F7F,0x7F7F,0x8F52,0x8F52,0xD552,0x6F6F,0x6F41,0x7F6F,0x9F52,0x5252,0x52AF,0xD552,0x427F,0x7F6F,0x426F,0x6F7F,0x426F,0x7F5F,0x7F6F,0x426F,0x7F5F,0x6F6F,0x7F6F,0x426F,0x7F6F,0x6F6F,0x6F6F,0x426F,0x7F6F,0x417F,0x7342,0x83C4,0x7383,0xB483,0x5273,0x5252,0x8373,0x8383,0xB473,0x8383,0x9383,0x7362,0x4373,0x4246,0x5F46,0x4646,0x465F,0x4246,0x5F46,0x4646,0x5246,0x7373,0x5273,0x8352,0x5263,0x7352,0x5252,0x5252,0x6373,0x7373,0x5273,0x7352,0x5252,0x5252,0x5252,0x5252,0x6352,0x5273,0xB483,0xC483,0xC4C3,0x5293,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x465F,0x4246,0x4646,0x465F,0x6363,0x5262,0x5252,0x7373,0x7352,0x7373,0x7373,0x5252,0x7383,0x6373,0x6362,0x6352,0x7373,0x4642,0x4642,0x5F46,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x464F,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F
.word 0x5252,0x5252,0x5273,0x8373,0x83C4,0x5263,0x7363,0x7352,0x8383,0x83C4,0x6373,0x8373,0x73B4,0x9363,0x73D4,0x7352,0x8352,0x93C3,0x8363,0xD483,0x5263,0x8383,0x5293,0x7363,0x9373,0x93C3,0x6373,0x7373,0x8352,0x9F63,0x41D4,0x428F,0x8F7F,0xAF62,0x8F52,0x528F,0x7F9F,0x7F8F,0x7F41,0x417F,0x7F7F,0x7F7F,0x7F42,0x7F41,0x8F42,0x4152,0x8F52,0x427F,0x7F41,0x7F52,0x528F,0xD49F,0x7F6F,0x426F,0x6F41,0x4241,0x529F,0x9F62,0x527F,0x6F7F,0x6F41,0x5F41,0x6F41,0x7F6F,0x6F7F,0x416F,0x7F5F,0x7F7F,0x6F6F,0x6F7F,0x7F41,0x7F7F,0x6F42,0x6F6F,0x7F6F,0x7F6F,0x7F41,0x736F,0x9383,0x83B4,0x8383,0xD483,0x6383,0x5252,0xB473,0x7373,0xB483,0x8383,0x8383,0x8383,0x7383,0xB483,0x8393,0x83B4,0x6383,0x7373,0x7373,0x5273,0x5273,0x7352,0x7352,0x7373,0x7352,0x8373,0x7363,0x5273,0x5273,0x5252,0x7373,0x7352,0x5273,0x7352,0x5273,0x5252,0x5252,0x5273,0x8373,0x7383,0x7373,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x7346,0x9383,0xD4D4,0x7383,0x5273,0x5252,0x5273,0x7352,0x7352,0x7373,0x5273,0x7373,0x7373,0x4352,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4642,0x4646,0x4246,0x4646,0x4246,0x4646,0x465F,0x4646
.word 0x5252,0x5252,0x7352,0x8373,0x5273,0x8383,0x8383,0x5283,0x7373,0x6373,0x8383,0x6383,0x5252,0x8383,0x83C4,0x7363,0x8352,0xB483,0x6383,0x9383,0x5273,0x8363,0x73B4,0x8383,0x8352,0x83B4,0x7352,0x83B3,0x6352,0xC452,0x529F,0x7F8F,0x6241,0x529F,0x528F,0x8F62,0x5273,0x7F41,0x7F42,0x8F52,0x7F42,0x427F,0x417F,0x8F42,0x8F52,0x628F,0x7F8F,0x416F,0x417F,0x427F,0x428F,0x528F,0x6F6F,0x7F41,0x427F,0x8F6F,0x7F41,0x428F,0x7F9F,0x5241,0x7F6F,0x7F6F,0x416F,0x7F6F,0x7F42,0x6F6F,0x7F41,0x4241,0x417F,0x426F,0x7F52,0x7F7F,0x6F6F,0x7F7F,0x7F6F,0x6F7F,0x7F7F,0x637F,0x8373,0xC493,0x73B4,0x7373,0x9383,0x5273,0x5252,0x7373,0x8373,0x7373,0x83B4,0x83B4,0x83B4,0x9393,0xB3C4,0x93A3,0x93C3,0x83C3,0x83C3,0x9383,0x8383,0x83C4,0x8383,0x7373,0x7373,0x5252,0x7373,0x5273,0x7352,0x5283,0x5252,0x5273,0x5252,0x7373,0x5283,0x8352,0x5252,0x5252,0x6352,0x5263,0x5273,0x4646,0x4246,0x4646,0x4642,0x4646,0x465F,0x4246,0x4646,0x4646,0x8373,0xC3B3,0x93D4,0xD4B3,0x8393,0x7373,0x5252,0x5273,0x8373,0x7373,0x8373,0x8373,0x7383,0x4643,0x4642,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4F46,0x4446,0x4646,0x4646,0x4246,0x4646,0x464F,0x4646,0x464F,0x4646,0x4646
.word 0x5252,0x8352,0x5263,0x7352,0x7373,0xB483,0x7383,0x8373,0x83C4,0x5273,0xB483,0x8393,0x5273,0x7373,0x7373,0xC393,0x6373,0x7373,0x8393,0x5263,0xC383,0x5283,0x5252,0xB483,0x5283,0x6352,0x7352,0x93C4,0x73C4,0x6262,0x7F9F,0x527F,0xAF52,0x8F52,0x8F52,0xAF52,0x8F42,0x7F7F,0x417F,0x7F8F,0x6F41,0x7F41,0x4152,0x528F,0x529F,0x9F62,0x6F42,0x4241,0x7F7F,0x417F,0x7F41,0x6F52,0x7F7F,0x7F6F,0x7F7F,0x6F7F,0x427F,0x417F,0x5242,0x429F,0x7F6F,0x4152,0x7F6F,0x7F42,0x7F7F,0x7F41,0x7F42,0x417F,0x4142,0x7F6F,0x7F7F,0x6F42,0x417F,0x426F,0x427F,0x7F6F,0x426F,0x417F,0xC473,0x93B4,0x8383,0x7373,0xB483,0xC483,0x5273,0x5252,0x5273,0x8373,0x7373,0x8383,0xB483,0xB493,0xB483,0x83C4,0x83B4,0x83C4,0x93B4,0x83B4,0xB3C4,0xC3C4,0xD4C4,0xC393,0xD493,0xB3D4,0x8383,0x7373,0x7352,0x7352,0x7383,0x5252,0x5252,0x5252,0x5273,0x5273,0x5273,0xB483,0x5252,0x5252,0x4652,0x4246,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x435F,0x8373,0xB383,0x83C4,0xC483,0x8383,0x83C4,0xB3D4,0x5283,0x7352,0x6352,0x7373,0x7373,0x5273,0x4352,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x4646,0x465F,0x4642,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646
.word 0x6352,0x8373,0x8373,0x5273,0x5252,0x7383,0x6383,0xC483,0xC4C3,0x5273,0x7373,0x6373,0xD483,0x5283,0x7352,0x8383,0x73D4,0x7352,0xB483,0x5252,0xC483,0xB483,0x7393,0x8363,0x6373,0x9393,0x6263,0xB473,0x5283,0x9FD4,0x7F42,0x8F42,0x5252,0x428F,0x62D4,0x528F,0x429F,0x428F,0x417F,0x427F,0x427F,0x428F,0x529F,0x528F,0x8F52,0x8F52,0x7F7F,0x8F42,0x7F7F,0x7F42,0x7F7F,0x4162,0x4142,0x7F7F,0x527F,0x5252,0x6F6F,0x7F41,0x528F,0x419F,0x4241,0x5241,0x8F52,0x7F41,0x7F41,0x7F42,0x416F,0x427F,0x429F,0x7F8F,0x7F41,0x7F7F,0x6F7F,0x6F6F,0x6F7F,0x427F,0x7F7F,0x6F7F,0x8383,0x8373,0x93B4,0x8373,0x7373,0x8383,0x8383,0x5263,0x7352,0x7352,0x8373,0xB483,0xB493,0x8383,0x8383,0x8383,0xB483,0xB493,0xB493,0xB493,0xB483,0x8383,0x8383,0x83B4,0x93C4,0xD493,0xD4D4,0xC493,0x7383,0x7363,0x5273,0x8352,0x6373,0x5252,0x5252,0x5252,0x7352,0x7352,0x7352,0x7373,0x5241,0x4242,0x4646,0x4646,0x4646,0x4646,0x4644,0x945F,0x8383,0xB383,0xB483,0xB493,0xB483,0xB483,0xB483,0x9383,0x83D4,0x5263,0x7352,0x7373,0x8383,0x7363,0x5252,0x4252,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4642,0x4246,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x4246
.word 0x5252,0x7373,0x8373,0x8383,0x6373,0x5252,0x5252,0x8383,0x73B4,0xD483,0x5283,0x5273,0x8373,0x7383,0x5263,0x7352,0x9352,0x5283,0x6263,0xC483,0x6373,0x5273,0x8373,0x7352,0x5273,0x7373,0x7383,0x7352,0xAF52,0x4152,0x528F,0x527F,0x8F9F,0xD442,0x8F9F,0x62C4,0x418F,0x417F,0x8F52,0x7F7F,0x4141,0xAF52,0x5252,0x9F62,0xD552,0x7F8F,0x7F42,0x7F41,0x7F42,0x7F41,0x6252,0xC4AF,0x628F,0x5242,0x7FC4,0x8F51,0x5252,0xC49F,0x9F52,0x52C4,0xAFD4,0x9FC4,0xC4D4,0xC441,0x527F,0x7F52,0x7F62,0x7F7F,0x7F41,0x7F7F,0x6F42,0x427F,0x7F7F,0x7F52,0x7F6F,0x7F7F,0x7F6F,0x6F42,0x8373,0x83B4,0xB483,0xB483,0x8373,0x6373,0x7373,0x8383,0x5263,0x5273,0x5273,0x6373,0x7373,0x7363,0x7363,0x6373,0x8373,0x8373,0x8383,0x7373,0x8383,0x73B4,0x83B4,0x8373,0xB483,0xC483,0x8393,0x83C4,0x8383,0x8383,0x7373,0x6373,0x7352,0x7373,0x5273,0x6352,0x5252,0x5252,0x5252,0x5252,0x5252,0x5273,0x6242,0x5252,0x6252,0x5242,0x8373,0x9383,0xB493,0xC483,0xB493,0x8383,0x8373,0x8383,0x8383,0x83B4,0xB3C4,0x7383,0x8352,0x7383,0x7352,0x8373,0x83C3,0x7373,0x5273,0x4252,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x5F46,0x4446,0x4246,0x4646,0x4F46,0x4646
.word 0x5273,0x5273,0x7363,0x8373,0x8363,0xD493,0x7383,0x7363,0x7373,0x8383,0x83D4,0x7363,0x6352,0x5273,0xC393,0x7393,0xB452,0x7383,0x5273,0x8373,0x5273,0x5283,0x6373,0x7373,0x5283,0x5263,0x5252,0x8373,0x6252,0x429F,0x7F8F,0x4241,0x428F,0xE49F,0x41D4,0x7F9F,0x8F42,0x8F42,0x7F42,0x4142,0x527F,0x73AF,0x529F,0x8FD5,0x6252,0x7F41,0x7F6F,0x7F6F,0x8F41,0x7F7F,0xD562,0x62D4,0x73AF,0x529F,0xD48F,0x6252,0x52AF,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF4F4,0xE4E4,0xAF62,0x9F83,0x8F62,0x627F,0x4142,0x7F8F,0x7F42,0x7F7F,0x7F7F,0x7F7F,0x416F,0x7F6F,0x7F6F,0x7F6F,0x7F6F,0x9352,0x7383,0x8383,0x8383,0x7383,0x7383,0x5273,0x5252,0x5273,0x5273,0x5252,0x6263,0x7352,0x5252,0x5273,0x7352,0x7373,0x8373,0x73B4,0x8383,0x8373,0x8373,0x8383,0x83B4,0x8383,0x83B4,0x83B4,0xB493,0xC473,0xC493,0xC4B3,0x7383,0x7352,0x7383,0x5252,0x5273,0x7352,0x7373,0x5252,0x5252,0x5252,0x5252,0x5252,0x5273,0x5252,0x5273,0x9383,0xB493,0xB493,0xB483,0x8383,0x8373,0x73B4,0x7373,0x7373,0x7373,0xC483,0x7393,0x6352,0x7373,0x8352,0xC483,0xB4B3,0x8383,0x7383,0x5252,0x4646,0x465F,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4F46,0x4246,0x464F,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646
.word 0x9373,0x7383,0x7352,0x6352,0xB462,0xB483,0x8373,0x92D4,0x7383,0x7363,0x7373,0x9383,0x93C4,0x5273,0x7373,0x8363,0x6383,0x7352,0x7373,0x7352,0x5273,0x6373,0x7352,0x7352,0x5273,0x7373,0x8373,0x5252,0x42AF,0x8F52,0x427F,0x8F7F,0x8F41,0x83E4,0xC48F,0x5262,0x528F,0x429F,0x628F,0x428F,0x62D4,0x9F62,0xBF73,0xD452,0x42AF,0x7F8F,0x7F7F,0x7F42,0x427F,0x627F,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4AF,0xD4D4,0xE4E4,0xE4E4,0xF4E4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xD4D4,0x62C4,0x8F62,0x428F,0x8F52,0x7F41,0x7F7F,0x427F,0x7F6F,0x6F6F,0x7F42,0x416F,0x7F6F,0x416F,0x736F,0xC483,0x5273,0x7363,0x93C4,0x83C4,0x7373,0x5273,0x5252,0x8373,0xB3C4,0x83D4,0x5263,0x7373,0x8383,0x8383,0x7363,0x7373,0x7373,0x8373,0x73B4,0x8373,0x8373,0x7383,0x83B4,0xC383,0x83D4,0x7373,0x7352,0x8373,0x93C4,0x83C4,0x6383,0x7352,0x7383,0x7352,0x7373,0x5273,0x8373,0x83C4,0x7363,0x8373,0x5283,0x7373,0x6383,0x8352,0xD493,0xD4C3,0xB493,0x8393,0x8373,0x83B4,0x8383,0xB483,0x8383,0x8383,0x83B4,0x73B4,0x7352,0x7393,0x8373,0x83B4,0x7383,0x73D4,0x4352,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xB483,0x7383,0xD473,0x7383,0x7352,0x7383,0x8352,0xC483,0x7383,0x7383,0x5252,0x8373,0x7383,0x8373,0x5263,0x8373,0x7373,0x5273,0x7352,0x5252,0x6373,0x5252,0x5273,0x5273,0x5252,0x5273,0x7363,0x4152,0x9F42,0x7F8F,0x8F42,0x7F41,0x4142,0x9F62,0x8F52,0x428F,0x429F,0x4141,0x8FC4,0x6241,0xD4D5,0x9FD4,0xD4C4,0x528F,0x9F62,0x6F42,0x7F41,0x7F7F,0x7F41,0x9F52,0xBF62,0xD4C4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xE4F4,0xE4F4,0xE4D4,0xE4E4,0xD4E4,0xAFE4,0x5242,0x8F7F,0x528F,0x4152,0x6F42,0x6F6F,0x7F6F,0x6F42,0x7F7F,0x7F6F,0x426F,0x6F6F,0x527F,0x8373,0x73D4,0x5252,0x8373,0xB483,0x7373,0x5273,0x93C4,0xB3C4,0xC3D4,0x93D4,0x7373,0x7373,0x7383,0x7363,0x5273,0x8373,0x8383,0x7373,0x8383,0xC473,0x7383,0x8373,0xD483,0x83B4,0x7373,0xD4D4,0x5273,0x5242,0x5252,0x4252,0x5252,0x5252,0x5252,0x4252,0x5252,0x5242,0x7352,0x7352,0x5252,0x5252,0x6352,0x6362,0x5252,0xC483,0xC4C3,0xC3E4,0x93C4,0x83C4,0x83B4,0x8373,0x83B4,0x7373,0x7383,0x83B4,0x7373,0x5252,0x8373,0x83D4,0x93C4,0xB483,0xC493,0x4373,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4642,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646
.word 0x7352,0x5273,0x8383,0x83D4,0x9373,0x83D4,0x6373,0x7373,0x6373,0xD483,0x5293,0x5263,0x6352,0x7352,0x6273,0x6352,0x7352,0x7373,0x5252,0x5252,0x5273,0x5273,0x7352,0x5252,0x5252,0x5252,0x7352,0x9F52,0x4142,0x4142,0x417F,0x427F,0x629F,0x428F,0x739F,0x419F,0x9F62,0xD442,0x429F,0xE49F,0xAF62,0xE4C4,0x62E4,0xD4D5,0x8F62,0x7F7F,0x6F6F,0x7F41,0x7F7F,0xD562,0xD562,0xD472,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4F4,0xF3F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xC4E4,0x62AF,0x52D5,0x8F52,0x7F41,0x7F7F,0x7F41,0x7F41,0x6F7F,0x6F7F,0x6F42,0x7F6F,0x6F6F,0x5242,0x5252,0x7373,0x6383,0x7352,0x7383,0x7373,0x5273,0x8373,0xD4D4,0xD493,0xB493,0x5273,0x5273,0x5252,0x7352,0x5273,0x5252,0x7352,0xB483,0x7373,0x8383,0x8373,0x83C4,0x83B4,0xB363,0x5263,0x7352,0x9383,0x7373,0xC373,0x8383,0x5273,0x8383,0x8383,0x83C4,0x8383,0x8383,0x7373,0x7373,0x7373,0x7373,0x7373,0x8383,0x8383,0x93D4,0xB3D4,0x93C4,0x83C4,0x83B4,0x8373,0xB483,0x8383,0x7383,0x7373,0x7373,0x7383,0x5252,0x7352,0x8383,0xB493,0x5283,0x4643,0x4246,0x4646,0x5F46,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4F46,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x5F44
.word 0x7352,0x5252,0x8373,0x6383,0xC483,0xC4B3,0x5273,0x5273,0x7352,0x8383,0x73B4,0x8383,0x7383,0x5273,0x7373,0x62C4,0x7352,0x7352,0x5252,0x5252,0x5252,0x5252,0x4252,0x5241,0x4252,0x4252,0x5252,0x428F,0x8F8F,0x428F,0x427F,0x417F,0xF662,0x528F,0x429F,0x8F41,0x8F52,0x9F62,0x8F42,0x62C4,0xF4D4,0xD462,0x8FE4,0xAF52,0x7F62,0x427F,0x6F7F,0x426F,0x527F,0xBF62,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE3E4,0xF4F4,0xF3F4,0xF4F4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xBF62,0x8F62,0x8F42,0x7F42,0x7F6F,0x7F6F,0x7F6F,0x6F42,0x7F6F,0x7F6F,0x6F6F,0x8352,0x6383,0x5252,0x5252,0x5252,0x8373,0x7383,0x7373,0x7352,0xC493,0x7373,0x7352,0x7352,0x6352,0x7352,0x5283,0x5273,0xB483,0x6373,0x7352,0x7373,0x8373,0xC473,0xB493,0x7352,0x5283,0x8352,0x92D4,0xD4D4,0x7373,0xD493,0xD4D4,0x7383,0xC452,0xD493,0xD4B3,0xD4C3,0x93D4,0x93C4,0xB3C4,0x93C4,0xC3D4,0xC3E4,0x83B4,0x93B4,0xC4C3,0xD4C3,0x83D4,0x83C4,0x8373,0x7383,0x7383,0xB483,0x7383,0x8383,0x8373,0x5273,0x5252,0x7373,0x8363,0x5273,0x4643,0x4246,0x4646,0x465F,0x4646,0x5F46,0x4446,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4642,0x465F,0x4646,0x4646
.word 0xD483,0x7393,0x5273,0x5252,0x7363,0x5273,0x8373,0xC383,0x5283,0x7352,0x5252,0x7373,0x7373,0x7373,0x7352,0x5252,0x5252,0x5252,0x5252,0x6352,0x5252,0x5273,0x5252,0x5273,0x5252,0x5252,0x4252,0x8F41,0x4142,0x7F7F,0x7F41,0x5241,0x42AF,0x628F,0x8F62,0x527F,0x528F,0xC4AF,0x6241,0x9FD4,0xD562,0x62D4,0x529F,0xE4E4,0x7F41,0x7F7F,0x417F,0x7F7F,0x6241,0xD4D5,0xD462,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D3,0xF4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xD4C3,0xD3E4,0xE4E4,0xD4D4,0xE4E4,0xC4D4,0x739F,0x418F,0x417F,0x427F,0x6F6F,0x6F6F,0x7F6F,0x7F41,0x7F6F,0x6F6F,0xB46F,0x7383,0x8373,0xC493,0x7383,0x6352,0x8383,0x8352,0x6383,0x8373,0x7363,0x8383,0x7363,0x9373,0xB483,0x7373,0x8352,0x5273,0x8352,0x8383,0x5273,0x6352,0x6383,0x5273,0x7352,0x8383,0x83C4,0xB483,0x9393,0x5273,0x93C4,0xC493,0x6383,0x9352,0xD4C4,0x93D4,0xC3D4,0xD4D4,0xC4C3,0xE493,0xD493,0xD4C3,0x8383,0xC493,0xE493,0x93C4,0x93D4,0x83C4,0x8373,0x73B4,0xB483,0xB483,0x8383,0x7373,0x7383,0x6373,0x7352,0x6352,0x9383,0x6383,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4644,0x4642
.word 0xC483,0x5283,0xD483,0x93C3,0x5252,0x5252,0x8373,0x83B4,0x8363,0xD4D4,0x7383,0x5252,0x5263,0x5273,0x5252,0x5252,0x5252,0x5252,0x5273,0x5252,0x5252,0x5252,0x4273,0x4252,0x5252,0x5252,0x8F52,0x4241,0x428F,0x417F,0x7F7F,0xAF42,0x41E4,0x9F42,0x8F42,0x8F42,0x6242,0x9F52,0xD58F,0x6252,0x62D5,0xC4AF,0x629F,0xAF62,0x8F42,0x6F42,0x6F6F,0x7F41,0xAF62,0xD4C4,0xD4BF,0xD4D4,0xD4E4,0xC3E4,0xE4E4,0xE4E4,0xE4E3,0xF4F3,0xF4F4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xC3D4,0xD4D4,0xD4D4,0xAF62,0x5252,0x7F8F,0x6F6F,0x7F41,0x4142,0x6F7F,0x426F,0x6F6F,0x6F6F,0x837F,0x7383,0x8373,0x83B4,0x83C4,0x5252,0xB452,0x8373,0x5273,0x7383,0x8393,0x83B4,0x8383,0x6383,0x9373,0x83E4,0x5273,0x7373,0x8383,0x73B4,0x7352,0x7383,0x5283,0x5273,0x6383,0x5252,0x6373,0x7373,0x8373,0x7373,0xB483,0x7373,0x5252,0xC473,0xC393,0xC3E4,0x93C4,0x93C4,0x93C4,0x93D4,0xC3C4,0xD4C4,0xC493,0xC3D4,0xB3C3,0x93C4,0x93C4,0x73B4,0x8373,0x8383,0x83B4,0x8383,0x7373,0x8373,0x8373,0x5263,0x5252,0x8352,0xC473,0x7383,0x4246,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4646,0x465F,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4642,0x4646,0x4246,0x4F46,0x4646
.word 0x7363,0x5252,0x8373,0xB483,0xD473,0xC493,0x5252,0x5273,0x8373,0x7383,0x7373,0x5283,0x5273,0x5252,0x5252,0x7352,0x5252,0x5252,0x5252,0x5252,0x5252,0x5242,0x4246,0x4246,0x4246,0x4142,0x7F8F,0x8F41,0x7F41,0x427F,0x417F,0x6252,0x8F52,0x9F52,0x8F52,0x4241,0x9F8F,0x8F52,0xD442,0x629F,0x529F,0x42AF,0xF552,0x8FC4,0x6F42,0x6F7F,0x6F41,0x427F,0xD562,0xD462,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E3,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xC4D4,0x628F,0x427F,0x7F7F,0x6F6F,0x7F41,0x4142,0x6F6F,0x6F6F,0x6F6F,0x736F,0x7373,0x7373,0x8352,0x8383,0x83D4,0x5263,0x8352,0x7352,0x8373,0x7363,0x8383,0xB473,0x5283,0x7373,0x7352,0xE483,0x83D4,0x7373,0x7352,0x7373,0x7373,0xC393,0x8383,0x8383,0x83C4,0x5273,0x5252,0x5252,0x5252,0x5252,0x5252,0x8373,0x8383,0x83C4,0x93B4,0xD4C4,0xD4B3,0xE493,0xC4C3,0x93E4,0xC3C4,0xB3C4,0xC3E4,0x63C4,0xC483,0x5273,0x7373,0x7383,0xB483,0x7383,0x7373,0x7373,0x7373,0x5252,0x5252,0x7352,0xB483,0xD483,0x83B3,0x4652,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x4642,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646
.word 0x5273,0x7373,0x6373,0x7373,0x8352,0x83B4,0xC483,0x8393,0x7352,0x7373,0x7352,0x5273,0x5252,0x5252,0x5273,0x5252,0x5252,0x5252,0x4442,0x4646,0x4246,0x6F46,0x4646,0x465F,0x4646,0x4646,0x7F41,0x7F42,0x7F42,0x7F41,0x427F,0xD4D5,0x528F,0x428F,0x428F,0x8F7F,0x8F42,0x7F42,0x6241,0x739F,0x529F,0x9F52,0xAF62,0x8F62,0x7F7F,0x7F7F,0x427F,0x627F,0xD4AF,0xD4D4,0xD4D4,0xE4D4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D3,0xE4E4,0xD4C3,0xD4E4,0xD4E4,0xD4D4,0x62D4,0x629F,0x7FAF,0x7F42,0x6F41,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x736F,0x8373,0x8373,0x7373,0x83B4,0xC483,0x5283,0x5252,0x5252,0x8363,0x5273,0x6373,0x7373,0x5273,0x5273,0x6373,0x8352,0xD493,0xD4D4,0x7383,0x7352,0xE483,0xC493,0xC483,0xC493,0xD4B3,0x8393,0x7383,0x5273,0x5273,0x5273,0xC483,0xD4B3,0xD4C3,0xE4C3,0xD4C3,0xD4C3,0xD493,0xC4C3,0xC493,0xD4B3,0xD4C3,0xD493,0xC4C3,0x5273,0x7383,0x7352,0x8373,0x7383,0x8383,0x7373,0x5252,0x5252,0x5252,0x5252,0x7352,0x8383,0x93C4,0xB483,0xD483,0x7383,0x4643,0x4246,0x4646,0x4646,0x4642,0x4646,0x4246,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x465F,0x4646,0x464F,0x4646,0x4642
.word 0xC493,0x5283,0x8383,0x5263,0x5273,0x5273,0x7363,0x6373,0x8383,0x5273,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x4646,0x4246,0x6F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x416F,0x428F,0x417F,0x427F,0x518F,0x529F,0x528F,0x419F,0x7F52,0x7F41,0x8F41,0x7F41,0x8F52,0x9F42,0x9F42,0x8F52,0xC4E4,0x428F,0x416F,0x6F7F,0x7F6F,0xD541,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4D3,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0x62D4,0x73AF,0x7F52,0x7F41,0x7F6F,0x6F7F,0x6F7F,0x7F7F,0x6F7F,0x6F6F,0x737F,0x7383,0x7383,0x5263,0x7373,0x8383,0x93C4,0x63C4,0x5252,0x8373,0x7363,0x7373,0x5273,0x8373,0x6352,0x7373,0x6373,0x8352,0x9393,0xD4D4,0xD4B3,0xB4B3,0xB483,0xC4B3,0xD4B3,0xB3D4,0xC3C3,0xD493,0xC493,0xC483,0x9383,0x93C4,0x93C4,0x93C4,0x93B4,0x93C4,0x93D4,0xD4C4,0xC493,0xB3E4,0xD4D4,0xC3D4,0xC3D4,0x7393,0x8352,0x5273,0x5273,0xB483,0x7383,0x8373,0x7363,0x5273,0x4252,0x5252,0x7352,0x7383,0x8383,0x7383,0x9383,0x93B4,0x83C4,0x7383,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4246,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646
.word 0x8383,0x7363,0x93C4,0xC493,0x8352,0x8383,0x5252,0x5252,0x7383,0x5273,0x5252,0x5252,0x5273,0x5252,0x4452,0x4246,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4646,0x7F46,0x7F41,0x8F42,0x7F41,0x7F7F,0x7341,0x52AF,0x428F,0x428F,0x8F8F,0x7F42,0x8F42,0x427F,0x429F,0x529F,0x428F,0x628F,0xD452,0x7F41,0x7F6F,0x6F42,0x7F41,0xD452,0xBF72,0x62D4,0xD4E4,0xD4C3,0xE4E4,0xE4E4,0xE4C3,0xE4E4,0xE4E4,0xF4D3,0xE4E4,0xE4E4,0xE4D3,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4C3,0xD4D4,0xD4D4,0x9F62,0x8F62,0x427F,0x6F6F,0x427F,0x6F7F,0x6F6F,0x6F42,0x426F,0x936F,0x73C4,0x7363,0x7383,0x5273,0x7373,0x7363,0x7352,0x5273,0x8352,0x7373,0x5252,0x7373,0x5273,0x7352,0x7352,0x7373,0x6373,0xB483,0xC493,0xB483,0xC483,0xB493,0xB493,0xB3D4,0xD4D4,0xD4B3,0xD493,0x83D4,0xE4B3,0xD4D4,0xD4B2,0xC4C3,0xC4C3,0xC4C3,0xD4C3,0xE493,0xE4C3,0xD4C3,0xD493,0x93D4,0xD4C3,0xD493,0x6393,0x8373,0x7373,0x8352,0x7383,0xB483,0x6373,0x7362,0x7383,0x7352,0x7352,0x7373,0x7363,0x6363,0x7373,0x73B4,0x7373,0x7383,0x7383,0x4352,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x5F46,0x4246,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646
.word 0x6352,0x5273,0xB483,0x8383,0x8363,0x7373,0x8373,0x6383,0x6352,0x5252,0x5252,0x5252,0x5252,0x4642,0x6F46,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x425F,0x417F,0x7F8F,0x427F,0x7F41,0x8F52,0x8F52,0x9F52,0x8F42,0x7F7F,0x417F,0x417F,0x417F,0x8F42,0x4241,0x418F,0xD542,0x52AF,0x7F8F,0x427F,0x7F6F,0x426F,0xD4D4,0x83D5,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xF4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xD3E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x62D5,0x41D5,0x7F7F,0x427F,0x7F7F,0x7F7F,0x417F,0x7F6F,0x6F6F,0x836F,0x7383,0x7383,0x5283,0x7352,0x7383,0x8373,0x8363,0x5252,0x5252,0x7373,0x5252,0x7352,0x7352,0x5252,0x7373,0x7383,0x7373,0x7373,0x7373,0x7373,0x7373,0x7373,0x8383,0xD4D4,0xE4B3,0x93C4,0xC3E4,0xC3D4,0x93C4,0xB3D4,0xB3C4,0xD4D4,0x93D4,0xD4D4,0x93D4,0xC4D4,0xD493,0xC493,0xC3D4,0x93D4,0x83D4,0xC3B4,0x5273,0x7352,0x5273,0xC473,0x8373,0x7373,0x5252,0x7373,0x5273,0x8373,0xD493,0x7383,0x4246,0x4246,0x4646,0x4642,0x4246,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4642,0x465F,0x465F,0x464F,0x4F46,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x5F46,0x4646
.word 0xD493,0x8383,0x5263,0x5273,0x5263,0x5273,0x7352,0x5273,0x6273,0x5252,0x5252,0x5252,0x4642,0x4642,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4642,0x4646,0x7F41,0x427F,0x4241,0x7F7F,0x427F,0x529F,0x9F62,0x8F52,0x7F41,0x4142,0x427F,0x7F7F,0x8F42,0x8F41,0x8F42,0x8F42,0x5241,0x529F,0x6F8F,0x6F6F,0x7F6F,0x527F,0xC4BF,0xD472,0xD4D4,0xD4D4,0xE4D4,0xC3E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xD3E4,0xE4E4,0xE4E4,0xE4D4,0xE4C3,0xE4D4,0xD4C3,0xD4D4,0xBF72,0x5262,0x7F52,0x7F6F,0x7F7F,0x6F42,0x6F6F,0x6F7F,0x7F6F,0x6341,0x5273,0x5273,0x7373,0x5252,0x8373,0x7383,0x8373,0x7383,0x5252,0x5252,0x8383,0x7373,0x7373,0x8373,0x7383,0x8383,0x5283,0x7373,0x7352,0x8383,0x8383,0xC483,0xB483,0xB493,0xD493,0xC4B3,0xD4C3,0xB3D4,0xC3D4,0x93D4,0xD4D4,0xD4B3,0xC493,0xC3C3,0xC3E4,0xC3E4,0xC3E4,0xC3E4,0xD4D4,0xB4B3,0x8373,0x73D4,0x6352,0x8373,0x5273,0x8373,0xB483,0x5252,0x5252,0x7383,0x5273,0xB483,0xC493,0x73B2,0x4646,0x4646,0x465F,0x4646,0x465F,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4246
.word 0x8383,0x73B4,0x8373,0x8383,0x7373,0x8373,0x6383,0x7352,0x5252,0x5252,0x5252,0x4652,0x4642,0x465F,0x5F46,0x4646,0x4646,0x4642,0x465F,0x4646,0x4646,0x4446,0x5F46,0x4646,0x465F,0x416F,0x417F,0x7F7F,0x7F41,0x5241,0x629F,0x429F,0x428F,0x7F8F,0x7F7F,0x7F41,0x7F7F,0x8F41,0x8F42,0x8F41,0x428F,0x51D5,0x628F,0x6F7F,0x7F7F,0x6F7F,0x6241,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE3E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D3,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xAFD5,0x4152,0x7F7F,0x6F7F,0x416F,0x6F6F,0x7F6F,0x426F,0x736F,0x7373,0x5252,0x5242,0x5252,0x7352,0x73C4,0x7383,0xB483,0x7383,0x5252,0x5252,0x5252,0x7352,0x5273,0x6373,0x7373,0x6373,0x7352,0x7373,0x7363,0x7363,0x5273,0x8373,0xD493,0x93D4,0xD4C4,0xB493,0xD483,0xD4D4,0xD4C3,0xD4C3,0xB3D4,0x83C4,0x83C4,0x83B4,0x83B4,0x83B4,0x73B4,0x7373,0xE483,0xB493,0x5252,0x5273,0xC452,0x5283,0x8373,0x8373,0x7373,0x8373,0x7373,0x7352,0xC483,0xB493,0x83D4,0x4642,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4446,0x5242,0x5252,0x5252,0x4252,0x5252,0x5252,0x5252,0x5252,0x5252,0x4642,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4446,0x4646
.word 0x7373,0x5273,0x9383,0xD4C4,0x5283,0xB483,0x7383,0x5252,0x6352,0x5252,0x4452,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x4646,0x4646,0x5F46,0x4142,0x427F,0x7F7F,0x427F,0x417F,0xAF52,0x7F42,0x8F52,0x4142,0x7F7F,0x427F,0x427F,0x427F,0x7F7F,0x7F42,0x4141,0xAF62,0x9F42,0x7F6F,0x6F6F,0x426F,0xD541,0xD572,0xD472,0xD4D4,0xD4D4,0xD4C3,0xD4E4,0xE4E4,0xE4C3,0xD3D4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x4162,0x7F9F,0x4142,0x416F,0x7F6F,0x6F42,0x6F7F,0x7F41,0x8342,0x7373,0xD483,0x8393,0x7383,0x5252,0x8373,0x7373,0x7373,0x8373,0x7383,0x4273,0x5252,0x7373,0x8373,0x93C4,0x8383,0x7352,0x8383,0x8373,0x8383,0x8383,0xB483,0xB493,0x7363,0x8383,0xC493,0xC3C3,0xB4B3,0xD493,0xB3D4,0x93C4,0x93C4,0x93D4,0x83B4,0x7373,0x7373,0x8373,0x8383,0xC493,0x7393,0x5263,0x5252,0x5252,0x8373,0x52B4,0x8363,0x83B4,0x8383,0x73C4,0x5273,0x8352,0x92C4,0x93B4,0xB493,0x6383,0x4646,0x4246,0x5F46,0x4646,0x4642,0x4246,0x6344,0x5252,0x7373,0x8383,0x9393,0xD4C3,0x9393,0x93B3,0xB393,0xC3B3,0xB393,0x9393,0x7373,0x6273,0x5252,0x7452,0x4446,0x4646,0x4646,0x465F
.word 0x93D4,0x8393,0xB473,0xB493,0x6383,0x7373,0x5252,0x5252,0x5273,0x4252,0x6F46,0x4646,0x4646,0x4246,0x5F46,0x4646,0x5F46,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4642,0x7F46,0x7F7F,0x7F41,0x7F41,0x7F41,0x5241,0xC4AF,0x527F,0x429F,0x7F7F,0x427F,0x7F7F,0x417F,0x418F,0x417F,0x7F7F,0x9F42,0x5252,0x428F,0x6F7F,0x6F6F,0x7F6F,0xD562,0xC4BF,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xD472,0x52D5,0x7F62,0x7F41,0x6F7F,0x7F7F,0x7F6F,0x426F,0x7F7F,0x7373,0x7373,0x8373,0x83C4,0x93B4,0x73C4,0x5252,0x9373,0x7383,0x6373,0x7373,0x7383,0x7352,0x8383,0x5273,0x7352,0x7373,0x8373,0x6373,0x5273,0x6373,0x7373,0x9383,0x93C4,0xC493,0x7393,0x7373,0x8352,0x83B4,0x7373,0x7373,0xB483,0xB493,0xC483,0x8383,0x9383,0xC3C4,0x93C4,0x83B4,0x5263,0x5252,0xC493,0xC493,0x5273,0x7373,0x8383,0x5263,0x7373,0x7383,0x5263,0x6352,0xC483,0xC493,0xB493,0x9383,0x83C4,0x5273,0x4646,0x4646,0x5242,0x8363,0x8383,0xC393,0xB393,0xD493,0xB493,0xB483,0x8383,0x6373,0x6373,0x7373,0x6363,0x7373,0x7363,0x8383,0x8383,0x8383,0x8383,0x9393,0x7373,0x5252,0x4663
.word 0xB483,0xB483,0x7352,0x7352,0x5252,0x7352,0x5252,0x6352,0x4252,0x4252,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x424F,0x7F41,0x7F42,0x7F7F,0x427F,0x427F,0xAF62,0x527F,0x8F52,0x7F41,0x7F7F,0x6F7F,0x7F7F,0x7F42,0x7F7F,0x8F42,0x6241,0x429F,0x418F,0x6F6F,0x427F,0x416F,0x62C4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xD3D4,0xE4E4,0xD4E4,0xD4E4,0xE4C3,0xE4D4,0xD4D4,0xAFD4,0x8F62,0x8F42,0x7F42,0x6F42,0x6F7F,0x6F7F,0x6F41,0x7363,0x7373,0x8373,0x9383,0x93C4,0x83D4,0x5273,0x5242,0x9373,0x7373,0x8373,0x5263,0x7352,0x7373,0x8373,0x5273,0x5273,0x5252,0x5273,0x5273,0x8373,0x7383,0x83B4,0xB483,0x8383,0x83B4,0x93C4,0x83D4,0x7383,0x8383,0xB483,0x7383,0x7373,0x7373,0xB483,0xC483,0xC483,0x7373,0x5252,0x4641,0x5244,0x93E4,0x93B4,0x5273,0x7373,0x8373,0x8383,0x5263,0x5252,0x7352,0x8383,0x8383,0x83B4,0x8383,0x83C4,0x7383,0x7383,0x5252,0x7352,0x8373,0xB393,0x93D4,0x83C4,0x73B4,0x7373,0x5252,0x5252,0x6352,0x4444,0x4246,0x4246,0x4246,0x4246,0x4646,0x5242,0x6352,0x7352,0x7373,0x7373,0x8383,0x8383,0x7383
.word 0x5252,0x5263,0x8373,0x93C4,0x93C4,0x7363,0x6352,0x6362,0x4152,0x4646,0x4646,0x4642,0x4646,0x4646,0x465F,0x5F46,0x4446,0x465F,0x4246,0x4646,0x465F,0x4646,0x4646,0x5F46,0x7F42,0x417F,0x417F,0x427F,0x7F7F,0x9F41,0x8F73,0x9F42,0x7F8F,0x7F7F,0x7F42,0x7F41,0x7F42,0x7F41,0x7F42,0x417F,0x9F42,0x8F7F,0x7F7F,0x7F6F,0x6F6F,0x527F,0xBFD4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4C3,0xE4D4,0xE4E4,0xD3E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4C3,0xE4D4,0xD4D4,0x62D4,0xC4D4,0x42AF,0x7F8F,0x7F7F,0x6F6F,0x6F7F,0x417F,0x427F,0x8393,0x9393,0x83D4,0x73B4,0xC483,0x8383,0x93B4,0x7383,0x5252,0x7352,0x5273,0x5273,0x5252,0x7373,0x7373,0x5263,0x8373,0xB483,0x7352,0x7373,0x5273,0x7352,0x7383,0x8383,0x5273,0x4646,0x4242,0x6352,0x8383,0xB473,0x7383,0x7383,0x8383,0x8383,0x7373,0x5273,0x4652,0x4642,0x4646,0x6F46,0x4446,0xD4D4,0xB493,0x4452,0x5273,0x8352,0x93B4,0x93E4,0x83C4,0x7383,0xB483,0x8383,0xC483,0x93B4,0x83B4,0x93B4,0x93D4,0xC493,0x9393,0xB493,0xB483,0x7352,0x7363,0x5252,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4646,0x465F,0x5F46,0x4246,0x4646,0x4642,0x4646,0x4246,0x4646,0x4446,0x5252,0x7363
.word 0x93D4,0x93C3,0x7352,0x8383,0xB483,0x5262,0x5252,0x5252,0x4652,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4642,0x4246,0x4646,0x416F,0x427F,0x7F7F,0x7F7F,0x7F41,0x6252,0x7FD5,0x4241,0x427F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x417F,0x417F,0x5242,0x7F7F,0x6F42,0x6F6F,0x7F6F,0x6241,0x62D5,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD3D4,0xE4D4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x62D4,0x51D5,0x7F7F,0x6F6F,0x6F7F,0x6F6F,0x7F42,0x7341,0xE4D4,0xF4D4,0xC483,0x5283,0x7373,0x8363,0xB483,0x6383,0x5273,0x7373,0x8383,0x5273,0x7352,0x5273,0x7373,0x8373,0x7373,0x7352,0x6273,0x7383,0x5273,0x8363,0xE4C3,0x83B3,0x4663,0x466F,0x4646,0x4246,0x4242,0x4252,0x4252,0x5252,0x6373,0x6373,0x5252,0x4646,0x4642,0x4646,0x465F,0x4646,0x5246,0x9393,0x5273,0x4644,0x4642,0x6352,0x8373,0xD4C4,0xD4B3,0xC493,0xD4C3,0xC3B3,0xC393,0xA3D4,0xC4C3,0x8383,0x7373,0x8373,0xB483,0x7363,0x5252,0x5252,0x4646,0x4246,0x6F46,0x4646,0x5F42,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x465F,0x465F,0x4246,0x4646
.word 0xC483,0xB483,0x5273,0x7373,0x7373,0x5252,0x5252,0x4252,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F46,0x4646,0x6F46,0x7F41,0x417F,0x7F7F,0x6F42,0x7F7F,0xAF52,0x7F52,0x418F,0x8F41,0x417F,0x7F42,0x6F6F,0x7F42,0x7F7F,0x7F6F,0x8F42,0x8F52,0x7F7F,0x6F7F,0x6F7F,0x426F,0xD452,0xC4BF,0x62D4,0xE4D4,0xD4D4,0xE4C3,0xE4E4,0xD4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xD3D4,0xE4E4,0xD4E4,0xD4C3,0xD4D4,0xBFD4,0x52C4,0x4152,0x427F,0x6F7F,0x6F7F,0x6F41,0xE47F,0xD4E4,0xD4D4,0x73E4,0x8363,0x5242,0x7383,0x8383,0x93C4,0x7383,0x7373,0x7373,0x5273,0x5252,0x7373,0x7383,0x8373,0x7373,0x7373,0x7373,0x5263,0x8352,0xD493,0x93D4,0x5273,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4242,0x8373,0x8383,0x5273,0x4641,0x4642,0x5F46,0x4646,0x4646,0x4642,0x8346,0x5273,0x4646,0x4646,0x5F46,0x4646,0x5242,0x7373,0x7352,0x6373,0x6373,0x6373,0x6373,0x6373,0x7373,0x7363,0x5263,0x5252,0x5252,0x4246,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x465F
.word 0xB483,0x8383,0x7352,0x6352,0x7373,0x5252,0x5252,0x4152,0x4646,0x4646,0x465F,0x4642,0x4246,0x4246,0x4646,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4642,0x4646,0x5F46,0x6F46,0x7F41,0x7F41,0x7F7F,0x417F,0x7F42,0x5252,0x419F,0x8F42,0x4142,0x7F7F,0x7F7F,0x6F7F,0x7F7F,0x6F42,0x6F7F,0x4241,0x7F9F,0x7F7F,0x426F,0x6F6F,0x417F,0xD462,0xD4C4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E3,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0x72D4,0xD4D4,0xBFD4,0x8F52,0x6F42,0x7F7F,0x7F6F,0x6F7F,0xF442,0xD4D4,0xD451,0x52F4,0x5252,0x5273,0xD452,0x8373,0x7373,0x7373,0x8352,0x8383,0x5273,0x5252,0x5283,0x8373,0x7352,0x8383,0x8383,0x8383,0x5252,0xD483,0x93D4,0x7383,0x4652,0x4642,0x4646,0x4646,0x4642,0x465F,0x4642,0x5242,0xE4C3,0x83D4,0x4273,0x4646,0x5F46,0x4646,0x4646,0x4642,0x5F46,0x5246,0x4643,0x5F46,0x4246,0x4646,0x4646,0x4646,0x4642,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x4652,0x4642,0x4246,0x4646,0x4646,0x466F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4246,0x4646,0x465F,0x4246,0x4646,0x4642,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4642,0x4646,0x465F,0x4246,0x4646,0x4646
.word 0x5263,0x7373,0x5283,0x8373,0x7383,0x6352,0x5252,0x4652,0x4246,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x416F,0x4641,0x7F6F,0x7F42,0x7F7F,0x7F41,0x9F41,0x7F52,0x428F,0x7F8F,0x7F42,0x427F,0x7F6F,0x7F42,0x6F7F,0x7F7F,0x8F41,0x7F42,0x7F6F,0x7F6F,0x6F7F,0x527F,0xD4D5,0xD4BF,0xD4D4,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xD4D3,0xD4E4,0xE4D3,0xE4D4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0x83BF,0x62D4,0x9F62,0x7F6F,0x7F6F,0x426F,0x7F6F,0xE47F,0x6252,0xD49F,0x83E4,0x8383,0x6383,0x7352,0xB483,0x7352,0x7373,0x7373,0x83B4,0x7373,0x5252,0x6373,0x8352,0x6383,0x6373,0x8363,0x5273,0x6352,0x5252,0x5252,0x4352,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x8342,0x6383,0x4452,0x4246,0x5F46,0x4646,0x4646,0x4642,0x4F46,0x4646,0x4646,0x4652,0x4646,0x4646,0x4646,0x465F,0x4642,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x6F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4644,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4642
.word 0x8352,0xB493,0xB493,0x8373,0x73B4,0x5252,0x5252,0x4652,0x4646,0x465F,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4646,0x464F,0x4646,0x465F,0x4246,0x4646,0x4646,0x4642,0x417F,0x6F7F,0x7F42,0x7F7F,0x417F,0x427F,0x528F,0x7F52,0x8F41,0x7F42,0x7F7F,0x6F7F,0x6F7F,0x7F7F,0x6F7F,0x427F,0x427F,0x7F41,0x6F6F,0x6F6F,0x7F6F,0x6242,0xC4BF,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xD4E4,0xE4D3,0xD3E4,0xE4E4,0xC3E4,0xD4E4,0xD4D4,0xD4D4,0xD5D4,0x52AF,0x7F41,0x7F42,0x6F6F,0x7F7F,0x5241,0x629F,0xE462,0x93E4,0x83B4,0x83D4,0x5252,0x8352,0x7383,0x7373,0x7373,0x7373,0x7373,0x4273,0x7352,0x5273,0x6263,0x8373,0x7383,0x5273,0x8373,0x6383,0x7352,0x7383,0x4262,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x5252,0x4643,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4246,0x5263,0x4644,0x4642,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x6F46,0x4646,0x466F,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4642,0x4246,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646
.word 0x7352,0x8383,0x83B4,0x7352,0x7352,0x5273,0x5252,0x4642,0x466F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x465F,0x4646,0x416F,0x7F7F,0x7F7F,0x7F41,0x7F7F,0x8F42,0x8F42,0x7F41,0x8F42,0x7F7F,0x7F41,0x6F7F,0x6F42,0x6F41,0x6F42,0x6F6F,0x4141,0x7F8F,0x6F7F,0x426F,0x7F6F,0xD562,0xD472,0xD4D4,0xD472,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4C3,0xE4D4,0xE4D4,0xD4D4,0xE4D3,0xD4E4,0xE4E4,0xE4C3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x62D4,0x62D4,0x9FC4,0x8F52,0x7F6F,0x7F6F,0x6F41,0x5252,0xD5AF,0xE462,0x83E4,0x8373,0x83C4,0x73C4,0x6352,0x8383,0x7383,0x7352,0x7373,0x7383,0x8373,0x5252,0x5252,0x5252,0x5273,0x7363,0x8373,0xB483,0x5273,0x5273,0x8373,0x82C4,0x7383,0x5262,0x4643,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4246,0x5F46,0x4646,0x4642,0x465F,0x4646,0x4646,0x4246,0x7342,0x7373,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F,0x4246,0x4F46,0x4646,0x5F46,0x4246,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4642,0x465F,0x4646,0x4646,0x4646
.word 0x6383,0x7383,0x8383,0x8363,0x5283,0x5273,0x5252,0x4646,0x4646,0x4646,0x4644,0x465F,0x4642,0x5F46,0x4646,0x465F,0x4642,0x4646,0x5F46,0x4646,0x465F,0x4646,0x5F46,0x7F42,0x7F42,0x7F41,0x427F,0x416F,0x527F,0x529F,0x7F8F,0x4241,0x7F7F,0x7F7F,0x6F42,0x6F7F,0x7F7F,0x7F6F,0x7F7F,0x8F42,0x6F42,0x6F42,0x6F7F,0x7F6F,0xD462,0xC4AF,0xD4D4,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xD4C3,0xD4D4,0xC3D4,0xE4D4,0xC3D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D3,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xC3D4,0xD4D4,0xD4D4,0xC4BF,0x83BF,0x429F,0x7F7F,0x6F6F,0x4142,0x9F52,0xD4C4,0xC4AF,0x73F4,0x7373,0x8373,0x8383,0x5273,0x6352,0xC483,0x8393,0x8373,0x8352,0xD483,0x7383,0x5273,0x4252,0x7352,0x7383,0xB483,0x7373,0x7373,0x5273,0x7352,0x8352,0x9383,0x93A3,0x7383,0x6373,0x4243,0x4242,0x4242,0x4442,0x6342,0x8373,0x7383,0x7373,0x73A4,0x7362,0x4642,0x4646,0x5252,0x7373,0x4662,0x465F,0x4646,0x465F,0x4246,0x4646,0x4646,0x4644,0x4642,0x4246,0x5F46,0x4646,0x4646,0x4246,0x4646,0x4246,0x4252,0x5F46,0x4446,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x4646,0x4646,0x4642,0x4246,0x5F46
.word 0x8383,0x6373,0x7352,0x7352,0x5273,0x5252,0x5252,0x4246,0x4646,0x4642,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4146,0x7F7F,0x6F42,0x427F,0x7F7F,0x7F46,0x4241,0x8F52,0x7F52,0x418F,0x7F7F,0x6F42,0x6F7F,0x6F41,0x427F,0x7F6F,0x416F,0x527F,0x6F7F,0x6F7F,0x6F7F,0x527F,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xD4C3,0x5262,0x429F,0x529F,0xC4BF,0xD562,0xD562,0x5262,0x7F7F,0x7F42,0x7F7F,0xD562,0xD462,0xE462,0x93C4,0x8383,0x7373,0xB483,0x8383,0x6383,0x5252,0x5263,0x8373,0x73B4,0x5273,0xD483,0xC4B3,0x5283,0x5252,0x8373,0x7373,0x7373,0x5273,0x5252,0x7373,0x7373,0x7363,0x7352,0x5263,0x5273,0x7352,0x7373,0x7373,0x7373,0x7373,0x6373,0x7373,0x7373,0x5262,0x5263,0x9393,0x7362,0x5252,0x5252,0x5252,0x4646,0x465F,0x4646,0x4F46,0x4646,0x635F,0x5273,0x4646,0x464F,0x4646,0x4646,0x4246,0x6342,0x7373,0xA393,0x6383,0x4646,0x4646,0x465F,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646
.word 0xB483,0x7352,0x8383,0x7373,0x5273,0x5252,0x5252,0x4646,0x465F,0x4646,0x4646,0x4246,0x4646,0x4246,0x4646,0x4642,0x4646,0x4646,0x4446,0x5F46,0x4446,0x4646,0x6F46,0x7F41,0x7F46,0x7F7F,0x6F41,0x7F42,0x9F42,0x8F52,0x7F42,0x8F42,0x7F7F,0x7F41,0x6F7F,0x7F7F,0x7F7F,0x416F,0x416F,0x527F,0x6F7F,0x6F7F,0x6F7F,0x5241,0xC4BF,0x83BF,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xD4E4,0x62D4,0x9F52,0x6F8F,0x6F6F,0x7F41,0x5241,0xC4AF,0x62BF,0x52AF,0x7F7F,0x417F,0x527F,0xD4D4,0xE4AF,0xE4D4,0x5283,0xB473,0xC4B3,0x9383,0xD4D4,0xC4B3,0x5283,0x8373,0x93C4,0x7373,0x8373,0x8373,0x93C4,0x52C4,0x7352,0x7373,0x8373,0x7352,0x8383,0x7383,0x6352,0x7352,0x7352,0x7352,0x6383,0x7383,0x7373,0x7373,0x7383,0x7373,0x8373,0x7383,0x7352,0x7373,0x7373,0x5273,0x5273,0x8373,0x83A3,0x5273,0x4252,0x4646,0x4646,0x4644,0x4646,0x4646,0x8363,0x9382,0x4643,0x7363,0x8352,0x72B3,0xA372,0xA393,0x8383,0x8383,0x5283,0x4246,0x4646,0x4646,0x4246,0x4246,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4246,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x465F
.word 0x7373,0x8373,0x8373,0x73B4,0x5273,0x5252,0x5252,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4642,0x6F46,0x7F41,0x6F46,0x7F41,0x7F7F,0x7F46,0x4241,0x429F,0x7F8F,0x4152,0x7F42,0x7F6F,0x6F42,0x6F41,0x6F42,0x7F7F,0x7F7F,0x7F42,0x6F7F,0x7F6F,0x426F,0x6F7F,0x6F6F,0x417F,0x529F,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4E4,0x52D4,0x419F,0x6F41,0x7F6F,0x426F,0xD4C4,0xE5D4,0xD4D4,0xD4D4,0x62D4,0x6262,0x52AF,0x7F7F,0x7F7F,0x6262,0xD4BF,0xE483,0xD4D4,0x93C4,0x73C4,0x5252,0x4252,0x6373,0x8383,0xC483,0xC493,0x7383,0x8373,0x8373,0x93B4,0xC483,0x5283,0x5252,0x7383,0x7373,0x5273,0x7373,0x5273,0x5252,0x7352,0x8373,0x83C4,0x7383,0x8383,0x9383,0x8383,0x7373,0x7373,0x8383,0x83B4,0x7383,0x7373,0x7383,0x7383,0x7373,0x5273,0x5273,0x7373,0x9383,0x6283,0x5273,0x4642,0x4642,0x7342,0x8373,0x73B4,0x9383,0x7352,0x5263,0x5273,0x5252,0x5252,0x5252,0x4643,0x4246,0x4646,0x5F46,0x4246,0x4646,0x4646,0x464F,0x4646,0x465F,0x5F46,0x4646,0x4246,0x464F,0x4644,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646
.word 0x93E4,0x5273,0x7373,0x8373,0x5283,0x5252,0x5252,0x4246,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F46,0x4646,0x425F,0x7F7F,0x6F42,0x417F,0x6F6F,0x425F,0x528F,0x419F,0x7F7F,0x9F52,0x7F7F,0x7F42,0x6F7F,0x6F7F,0x6F7F,0x426F,0x416F,0x417F,0x6F6F,0x427F,0x7F6F,0x62C4,0x7F62,0x6F7F,0x6F6F,0x6F7F,0x8F7F,0xD452,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0x8F52,0x7F6F,0x7F41,0x7F7F,0x627F,0xE4D4,0xE4E4,0xE4E5,0xE5E5,0xE5D4,0xD4D4,0x52AF,0x52D5,0x427F,0x4241,0xC4AF,0xD462,0xD4AF,0xE4E4,0xC493,0xC383,0xC3D4,0x7373,0x5252,0xD473,0x8383,0x8383,0x7373,0x7373,0x8373,0xB483,0x8373,0x7373,0x5242,0x8373,0x7373,0x7352,0x5273,0x8352,0x7373,0x5283,0x7352,0x8373,0x7373,0xB483,0x8373,0x7363,0x8383,0x8383,0x7373,0x5273,0x8373,0x8383,0xD493,0xC493,0x5283,0x5273,0x7373,0x5273,0x6263,0x7363,0x9383,0x83B3,0x8393,0x7373,0x9373,0x7393,0x5263,0x5252,0x5273,0x4452,0x4646,0x4646,0x4646,0x4642,0x465F,0x5F46,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4642,0x465F,0x5F46,0x4246,0x4646,0x464F,0x4646
.word 0xB483,0x7352,0x7352,0x8383,0x52B4,0x5252,0x5252,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4642,0x4246,0x5F46,0x4646,0x4246,0x4646,0x465F,0x4646,0x4646,0x7F6F,0x7F41,0x425F,0x7F7F,0x4642,0x7F46,0x4241,0x8F52,0x7F7F,0x4152,0x7F7F,0x7F7F,0x6F7F,0x6F42,0x6F41,0x6F7F,0x7F7F,0x7F42,0x7F6F,0x7F6F,0x416F,0xAF52,0x5252,0x62D4,0x8F52,0x6F6F,0x6F7F,0x7F7F,0x6F7F,0x6251,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0x4151,0x6F6F,0x6F6F,0x6F7F,0x426F,0x8352,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xAF83,0x8F62,0xAF52,0x9F62,0x417F,0x628F,0xD4D5,0xE4D4,0xE462,0xD4D4,0x9383,0x83C4,0x8363,0x7383,0x7383,0x5252,0x7363,0x9373,0x93C3,0x83C4,0x7373,0x8383,0x7383,0x8363,0x4273,0x7352,0x8383,0x7373,0x5273,0x8373,0x8363,0x7383,0x5273,0x7352,0x7363,0xC493,0x8373,0x7373,0xB483,0xB493,0x8373,0x7373,0x7373,0xB483,0x7373,0x7373,0x7383,0x7373,0x8383,0x7373,0x7373,0x5283,0x6363,0x7373,0x7383,0x5273,0x8373,0x93B4,0xC493,0x83B3,0x7383,0x7383,0x4652,0x466F,0x4642,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4644
.word 0x7373,0x9373,0x73C4,0x7352,0x5273,0x5263,0x5252,0x5F46,0x4646,0x4642,0x5F46,0x4646,0x4644,0x4F46,0x4646,0x4646,0x4246,0x4646,0x465F,0x4646,0x5F46,0x4446,0x416F,0x426F,0x7F7F,0x7F41,0x466F,0x7F42,0x8F41,0x8F42,0x7F7F,0x9F52,0x4142,0x7F7F,0x6F42,0x6F7F,0x7F7F,0x7F6F,0x4142,0x7F7F,0x426F,0x6F6F,0x7F6F,0x6F5F,0x5F5F,0x6F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6241,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8F41,0x7F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x416F,0x5241,0x529F,0x529F,0x7F7F,0xC442,0xD562,0xD441,0xD562,0xD472,0xE4D4,0xE4C4,0x83E4,0x93B4,0x93C4,0x83C4,0x73B4,0x7373,0x8383,0x5273,0x5252,0x7363,0x8383,0x7373,0x7373,0x7363,0xC493,0x5283,0x5252,0x8363,0x7373,0x7383,0x7373,0x7373,0x5263,0x8383,0x8373,0x7373,0x7373,0x7383,0x7393,0x8373,0x73C4,0x7373,0x7373,0x7373,0x7383,0x7373,0x8383,0xC483,0x8383,0x73B4,0x8373,0x8352,0x7383,0x6273,0x7352,0x5273,0x8373,0x7373,0x6383,0x8373,0x83C4,0x83B4,0x83C4,0x4652,0x4646,0x4646,0x465F,0x4646,0x465F,0x4246,0x4646,0x4646,0x4642,0x4F46,0x4646,0x464F,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4642,0x4F46,0x4646,0x465F,0x4446,0x4246,0x4646,0x4646,0x4646,0x465F
.word 0x5252,0x7383,0x7383,0x7363,0x5252,0x5262,0x5252,0x4643,0x4646,0x4F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4446,0x4646,0x4246,0x7F6F,0x7F41,0x427F,0x7F7F,0x427F,0x7F46,0x8F42,0x4241,0x7F7F,0x4152,0x7F7F,0x7F42,0x6F7F,0x7F7F,0x6F7F,0x6F42,0x416F,0x7F7F,0x7F6F,0x6F6F,0x5F7F,0x6F5F,0x5F5F,0x6F6F,0x6F6F,0x6F5F,0x6F6F,0x6F6F,0x6F6F,0x416F,0x8F6F,0xD4D4,0xD4D4,0xD4D4,0x72D4,0x62AF,0x428F,0x7F7F,0x6F6F,0x6F6F,0x5F6F,0x5F6F,0x5F6F,0x5F5F,0x5F6F,0x5F5F,0x5F6F,0x6F6F,0x6F6F,0x526F,0xD4AF,0x62BF,0xD452,0xD4AF,0xD4D4,0xE4E4,0xE462,0x73D4,0x6373,0x6373,0x8383,0x9383,0x93C4,0x83B4,0x8373,0x7373,0x7373,0x8373,0x7373,0x5273,0xC483,0x8383,0xB483,0x5273,0x5252,0x6352,0x7373,0x7373,0x7352,0x5252,0x5252,0x4252,0x5252,0x5252,0x5252,0x5252,0x7352,0x6263,0x7352,0x5273,0x5273,0x5273,0x5252,0x5252,0x5273,0x7373,0x8373,0xB493,0xB483,0x5273,0x7373,0x8383,0x83B4,0x7383,0x7373,0x7373,0x7373,0x8383,0xB483,0xB493,0x4652,0x4246,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x464F,0x4246,0x4646,0x4646
.word 0x8352,0x6352,0x7352,0x9383,0x5273,0x5263,0x7352,0x4652,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4246,0x5F46,0x4246,0x464F,0x4646,0x4646,0x4646,0x4642,0x4F46,0x7F42,0x7F7F,0x7F7F,0x427F,0x466F,0x7F46,0x4241,0x418F,0x7F7F,0x8F52,0x7F42,0x7F7F,0x7F7F,0x6F6F,0x7F42,0x7F6F,0x427F,0x7F7F,0x6F6F,0x6F41,0x5F6F,0x6F6F,0x5F7F,0x4F5F,0x4F4F,0x6F6F,0x6F5F,0x6F6F,0x6F6F,0x6F7F,0x7F6F,0xD462,0xD4D4,0xD4D4,0xD4D4,0xAF62,0x8F42,0x6F6F,0x6F6F,0x5F6F,0x5F6F,0x5F6F,0xFC5F,0x5F5F,0x5F5F,0x5F6F,0x5F5F,0x5F4F,0x7F5F,0xC441,0xC4D4,0xAFC4,0xD452,0x62D4,0xE4D4,0xD4E4,0xE4D4,0x4662,0x5241,0x7352,0x6352,0x5263,0x5273,0x8383,0xC3C4,0xD4C3,0xE493,0x82D4,0x83D4,0x8352,0x93D4,0x73B4,0x7373,0x8383,0x5273,0x5252,0x5252,0x5252,0x5252,0x5252,0x4642,0x4246,0x4646,0x4646,0x4146,0x5252,0x5252,0x5273,0x7352,0x5252,0x5252,0x7352,0x5252,0x6352,0x5273,0x5252,0x7373,0x8383,0x8383,0x8383,0xB483,0x7352,0x7352,0x8373,0x8373,0x7373,0x8373,0x83D4,0x7373,0x5263,0x4652,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x4246,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4642
.word 0x83C4,0x52C4,0x7373,0x8373,0x73C4,0x5273,0x5252,0x4652,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F42,0x4646,0x5F46,0x7F46,0x417F,0x467F,0x416F,0x7F7F,0x4F42,0x4146,0x417F,0x8F42,0x7F7F,0x5252,0x417F,0x7F7F,0x427F,0x7F6F,0x7F7F,0x7F6F,0x7F41,0x6F41,0x6F7F,0x427F,0x6F6F,0x9F7F,0x6EBF,0x406E,0x7E6E,0x7F9D,0x7FBF,0x7F5F,0x6F41,0x6F7F,0x7F6F,0x6241,0xD4D4,0xD4D4,0xD4D4,0x9F62,0x6F6F,0x426F,0x6F6F,0x6F5F,0xCF52,0x7E7E,0x4F7F,0x6E4C,0xBFDC,0x7F9F,0x4F6F,0x6F4F,0x426F,0xD48F,0xD4D4,0x62D4,0xAFD4,0xD462,0xD4D4,0xE4D4,0xD4D4,0x4646,0x4246,0x4646,0x4642,0x4146,0x5252,0x7352,0x5263,0x5252,0x5252,0x5273,0x6352,0xC483,0x7383,0x7383,0x8383,0x83C4,0xC483,0x5283,0x8352,0x8383,0x6383,0x5283,0x4246,0x4646,0x465F,0x4642,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x5241,0x5263,0x7363,0x7373,0x83C4,0x93B4,0x83B4,0x83C4,0x83B4,0xC483,0x8383,0xC493,0x8373,0x7373,0x7383,0x4652,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4642,0x5F46,0x4246,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x4246,0x464F,0x4646,0x4646
.word 0xC483,0x7373,0x7383,0x7373,0x5283,0x6352,0x5252,0x4652,0x4642,0x5F46,0x4246,0x4646,0x4646,0x465F,0x4246,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x7F46,0x427F,0x427F,0x425F,0x6F7F,0x4646,0x7F46,0x8F42,0x4241,0x417F,0x629F,0x428F,0x7F7F,0x6F41,0x7F7F,0x416F,0x6F6F,0x7F41,0x6F7F,0x6F6F,0x7F7F,0x6F6F,0xCF41,0x51CF,0x8E6E,0xF0F0,0xAF7E,0xBFEF,0x6F7F,0x7F7F,0x7F7F,0x6F6F,0xD442,0xD4D4,0xE4D4,0xD4D4,0x7FD4,0x7F6F,0x6F7F,0x5F5F,0x6242,0xFFDF,0x8E6E,0xBDF0,0xFC9D,0xEF6E,0x9FCF,0x6F7F,0x4F4F,0x627F,0xD462,0xD4D4,0xAFD4,0xD4C4,0xE4E4,0xD4D4,0xE4E4,0x5283,0x4246,0x4646,0x465F,0x4646,0x466F,0x4646,0x4246,0x4246,0x5252,0x5252,0x7342,0x8373,0x83C4,0x7383,0x7373,0x8373,0x93B4,0x73B4,0x5252,0x8363,0x73B4,0xC483,0x5273,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4246,0x5F46,0x5F46,0x4646,0x465F,0x465F,0x5F42,0x4646,0x465F,0x4646,0x4646,0x5252,0x5252,0x7363,0x8373,0xB483,0xB493,0x8393,0xE493,0x93B4,0x93B4,0x83B4,0x8373,0x5263,0x5273,0x4652,0x4642,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x465F,0x4646
.word 0x7373,0x8383,0xC483,0x5283,0x7352,0x5252,0x5252,0x4652,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x464F,0x5F46,0x4646,0x5F46,0x4646,0x4642,0x6F46,0x7F41,0x467F,0x6F46,0x7F7F,0x5F46,0x4146,0x428F,0x8F8F,0x7F7F,0x6252,0x7F8F,0x7F41,0x7F7F,0x426F,0x7F6F,0x6F42,0x427F,0x6F7F,0x6F42,0x416F,0x4F6F,0x7F5F,0x5241,0xF272,0xF2E2,0xF4E3,0xD4E4,0xB463,0x6F7F,0x4A8F,0x7F6F,0xD441,0xE4E4,0xE4E4,0xD4E4,0x8F62,0x4142,0x8F8F,0x427F,0xD462,0xDFDF,0xFEF0,0x6E9E,0xE06E,0x62CF,0x7F8F,0x4F5F,0x526F,0xD4D4,0xE4E4,0xE4E4,0x62D4,0xE4D4,0xD4F4,0x62E4,0xD4E4,0x4342,0x4642,0x4642,0x4244,0x4244,0x4244,0x4152,0x6342,0x7352,0x5273,0x8373,0x9383,0xB2C4,0xC4B3,0x8383,0x7373,0x7373,0x5252,0x5252,0x5252,0xD483,0x8373,0x7373,0x4283,0x4642,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4242,0x6252,0x9373,0x73C4,0x5252,0x7352,0x8373,0x8383,0x83C4,0x83B4,0x7383,0x7383,0x6373,0x7373,0x5273,0x5252,0x4646,0x4646,0x465F,0x4646,0x465F,0x4F46,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4642,0x4646,0x465F,0x4246,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4642
.word 0x7383,0x7383,0x9383,0x7373,0x7383,0x7352,0x5252,0x4252,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x7F46,0x4241,0x467F,0x6F46,0x6F41,0x4646,0x7F46,0x7F41,0x4252,0x417F,0xAF52,0x427F,0x7F7F,0x7F42,0x7F6F,0x6F6F,0x6F7F,0x7F41,0x6F7F,0x7F7F,0x4241,0x6F41,0x4F4F,0x5F4F,0x6F5F,0x418F,0x8F52,0x8F41,0x417F,0x6F7F,0x8F7F,0x7F7F,0xD442,0xE4E4,0xE4E4,0xD4E4,0x41D4,0x416F,0x7F8F,0x426F,0x8F42,0xAF52,0x7262,0x7272,0x5262,0x7F8F,0x4F5F,0x524F,0xBF62,0xE4D4,0xE4E4,0xD4E4,0x9FD4,0xE4D4,0x83E4,0xD4AF,0x73D4,0x5273,0x5273,0x5252,0x6352,0x7352,0x5263,0x8383,0x8383,0x8383,0x7383,0x4252,0x5252,0x5252,0x5263,0x4252,0x5252,0x5242,0x5273,0x7373,0x6352,0x8393,0x7373,0x8383,0x4273,0x4646,0x4646,0x465F,0x4646,0x4642,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x4646,0x4642,0x4246,0x6352,0x9383,0x83B4,0x8393,0x5263,0x5273,0x7352,0x6373,0x7373,0x7373,0x7373,0x6373,0x5262,0x5273,0x5252,0x5252,0x4646,0x465F,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4F46,0x4446,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4642,0x4646
.word 0x73D4,0x7363,0x7373,0x7352,0x7383,0x5273,0x5252,0x5252,0x4646,0x4642,0x4646,0x4246,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x5F46,0x5F46,0x4646,0x416F,0x416F,0x6F6F,0x4246,0x7F6F,0x446F,0x4146,0x427F,0x529F,0x7F7F,0x7362,0x7F8F,0x4152,0x416F,0x6F7F,0x6F41,0x7F41,0x417F,0x7F6F,0x6F41,0x417F,0x62D5,0x9FC4,0x8F62,0x6F41,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4A6F,0x6F6F,0x7F7F,0xD441,0xE4D4,0xE4F3,0xE4E4,0x9F62,0x7F7F,0x4A7F,0x6F7F,0x4F5F,0x5F4F,0x4F4F,0x4F4F,0x7F5F,0x526F,0x5252,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0x62D4,0xD462,0xD4D4,0xD4E4,0x8373,0x83C4,0xB483,0x7383,0x7373,0x7373,0x7373,0x7363,0x8373,0x93C4,0x83B4,0x8393,0x7383,0x7352,0x7383,0x8383,0x8373,0x8383,0xB483,0x5283,0x9352,0x73B4,0x8383,0x73B4,0x4642,0x4642,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4642,0x4F46,0x4646,0x5F46,0x4646,0x4246,0x7352,0x5263,0xB483,0x7383,0xD4B4,0x7393,0x5252,0x6352,0x5262,0x5273,0x5273,0x5252,0x5273,0x7363,0x5252,0x5273,0x7373,0x4262,0x4643,0x5F46,0x4646,0x5F46,0x4446,0x4646,0x4642,0x4646,0x465F,0x4446,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x464F,0x4646,0x465F
.word 0xD493,0x7383,0x6352,0x7373,0x7373,0x5252,0x7373,0x6263,0x4642,0x4646,0x4246,0x4646,0x465F,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x416F,0x417F,0x465F,0x4646,0x416F,0x5F7F,0x7F46,0x417F,0x8F42,0x7F42,0xAF62,0x7F42,0x7F8F,0x6F41,0x7F7F,0x6F7F,0x7F7F,0x7F42,0x6F6F,0x7F42,0x427F,0xD4D4,0xD4D4,0x62C4,0x62D5,0x6262,0x62D5,0xC462,0x6262,0x7F8F,0x7F6F,0x7F41,0xD452,0xE4E4,0xE4F4,0xE4E4,0x52C4,0x7F7F,0x6F7F,0x5241,0x529F,0x9F62,0xAF62,0xAF52,0x6262,0xE552,0xD4D4,0xE4D4,0xF8D4,0xE4E4,0xD4E4,0xD4E4,0xAFD4,0xD462,0xE4E4,0x83D4,0x8383,0x83B4,0x8383,0x83B4,0x7383,0x8373,0x8373,0x8383,0x7373,0x6373,0x7373,0x6373,0x7352,0x8373,0x83B4,0x7373,0xB483,0xB483,0x8383,0x5273,0x83C4,0x7383,0xB483,0x5252,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x7363,0x7352,0x7373,0x5252,0x8352,0x9383,0x73B4,0x5273,0x5252,0x5252,0x6263,0x7363,0x5252,0x5252,0x5252,0x5252,0x7352,0x5273,0xC493,0x4252,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646
.word 0xC483,0x5283,0x8383,0x9383,0x8373,0x83C4,0x5252,0x5252,0x4663,0x4642,0x465F,0x4646,0x4646,0x4446,0x4646,0x464F,0x5F46,0x4246,0x4646,0x4644,0x4646,0x417F,0x427F,0x5F46,0x4646,0x425F,0x427F,0x416F,0x7F42,0x8F41,0x7F41,0x6262,0x417F,0x7F42,0x427F,0x7F7F,0x426F,0x7F6F,0x417F,0x7F6F,0x7F7F,0x7F41,0xE4C3,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0x62D4,0xD562,0xD451,0xE4D4,0xF4F3,0xE4E4,0xC4E4,0x6262,0x6252,0xD4D4,0xD4D4,0xD4D5,0xC4D4,0xF9D5,0xD4E5,0xF4D4,0xD4E4,0xF9E4,0xE4E4,0xE4E4,0xD4C3,0xD4D4,0x6283,0xD4D5,0xD4D4,0x4652,0x4646,0x4642,0x4146,0x5242,0x6352,0x7352,0x5252,0x5252,0x7373,0x7383,0x7373,0x8373,0x7383,0x8373,0x8383,0x8373,0x8383,0x7393,0x73C4,0x7352,0x8383,0x93C4,0x5273,0x4642,0x4642,0x465F,0x4246,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4246,0x5F46,0x4646,0xA452,0x5252,0x5252,0x4246,0x4246,0x8373,0x83D4,0x6263,0x7373,0x5273,0x6263,0x5263,0x5252,0x5252,0x5273,0x8352,0x7373,0x5273,0xD483,0x8373,0x8393,0x4252,0x4246,0x5F46,0x4646,0x4246,0x5F46,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4246,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4642
.word 0xB483,0x7373,0x8373,0x83B4,0x7373,0xB483,0x7352,0x5252,0x4352,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x416F,0x7F7F,0x4646,0x4642,0x5F46,0x7F7F,0x7F41,0x7F41,0x8F42,0x7F42,0xD552,0x8F7F,0x417F,0x7F7F,0x427F,0x6F6F,0x7F7F,0x7F41,0x7F6F,0x7F41,0x8F42,0xC3D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD3E4,0xD4E4,0x62D4,0xD4D5,0xE4D4,0xF4F3,0xD4E4,0xD4D4,0xD4D4,0xF4E5,0xE4F9,0xF9F9,0xD4D4,0xD4D4,0xE4C3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xD4E4,0xE4E4,0xD4D4,0x62BF,0xD4AF,0x5262,0x4646,0x4642,0x465F,0x4646,0x4642,0x4446,0x7352,0xC483,0xC493,0x83C3,0x83C4,0x8383,0xB483,0x7383,0x8373,0x73B4,0xB483,0xB483,0xC483,0x4262,0x8352,0x83C4,0x5283,0x4652,0x4642,0x4646,0x4646,0x4646,0x465F,0x4646,0x4642,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x4644,0x4644,0x4642,0x4646,0x5246,0x8383,0x73B4,0x7383,0x7352,0x5273,0x8373,0x7352,0x8383,0x7363,0x8373,0x8363,0x73C4,0x6383,0x7352,0x7383,0x8363,0x8383,0x4242,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4644,0x4646,0x4646,0x464F,0x4646,0x4642,0x4F46,0x4646,0x464F,0x4646,0x464F,0x4646,0x4646,0x464F
.word 0x5252,0x5273,0x7373,0x7373,0x7352,0x5252,0x5273,0x7352,0x5252,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4246,0x4646,0x4646,0x4646,0x4446,0x425F,0x7F7F,0x5F46,0x4646,0x4646,0x425F,0x7F7F,0x427F,0x5241,0x7F9F,0xAF62,0x7F42,0x7F41,0x7F42,0x6F41,0x417F,0x7F6F,0x427F,0x7F6F,0x6F42,0x4141,0xD4D4,0xC3E4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xE462,0xE3D4,0xE4F3,0xE4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xC3E4,0xF4F4,0xE4E4,0xE4D4,0xE4D4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x62C4,0x5243,0x4647,0x4642,0x4646,0x4646,0x465F,0x5242,0x8373,0x8383,0x93C4,0xB3C4,0x93C4,0xD483,0xB493,0xC493,0x8383,0xB483,0xB493,0x8383,0x7373,0x5273,0x5252,0x4252,0x4646,0x4646,0x4646,0x4646,0x465F,0x4246,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x7341,0x7373,0x8373,0x8373,0x6263,0x7352,0xC452,0x7373,0x9373,0x8363,0x9373,0x7383,0x7373,0x8383,0x7363,0x5273,0x4646,0x4242,0x5252,0x4242,0x4246,0x4646,0x465F,0x4246,0x4F46,0x4646,0x464F,0x4646,0x4642,0x465F,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646
.word 0x6393,0xE4C3,0x7383,0x8352,0x93C4,0x73C4,0x7383,0x5263,0x5273,0x4652,0x4646,0x4646,0x4644,0x4446,0x5F46,0x4646,0x4646,0x464F,0x4246,0x4646,0x4646,0x425F,0x7F7F,0x4646,0x4646,0x465F,0x6F46,0x417F,0x8F7F,0x5241,0x8F52,0x6252,0x7F8F,0x8F42,0x417F,0x7F6F,0x6F41,0x427F,0x7F7F,0x7F7F,0x7F7F,0x8F42,0xE4E4,0xE4E4,0xE4C3,0xC3E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4BF,0xE4D4,0xF4F7,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3D4,0xD4D4,0x43BF,0x4646,0x4646,0x4646,0x465F,0x4646,0x7343,0xB483,0x8383,0x73B4,0x7373,0x8383,0x7383,0x8373,0x7373,0x7373,0x8373,0x7373,0x8383,0x73B4,0x5273,0x7352,0x5283,0x4652,0x4642,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x4246,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4146,0x7363,0x5252,0x8383,0x8373,0x5242,0x5252,0x8373,0x7352,0x8373,0x7373,0x8373,0xD4B4,0x5273,0x8363,0x8383,0x5273,0x4642,0x4246,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x4642,0x4246,0x4646,0x4642,0x5F46,0x4646,0x4642
.word 0x6383,0x8383,0x83B4,0x8363,0x7383,0x7383,0x7373,0x7352,0x5252,0x5252,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4246,0x4646,0x7F46,0x426F,0x466F,0x4642,0x4646,0x425F,0x7F7F,0x427F,0x9F7F,0x8F42,0x9F52,0x7F52,0x417F,0x7F42,0x427F,0x7F7F,0x7F7F,0x7F41,0x416F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD572,0xD483,0xF4D4,0xF4F3,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xC3E4,0xD4E4,0xD4D4,0xD4D4,0x4662,0x4246,0x5F46,0x4646,0x4646,0x8352,0x7383,0x5252,0x4252,0x4644,0x5241,0x5242,0x6352,0x5273,0x7373,0x7352,0x6373,0x5273,0x4252,0x7362,0x8383,0x6383,0x4652,0x4642,0x4646,0x465F,0x4646,0x4646,0x4642,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4642,0x4246,0x4646,0x5F46,0x4646,0x4246,0x5273,0x4146,0x8363,0x52B4,0x4643,0x4246,0x8352,0x9373,0x5273,0x5252,0x8373,0x8373,0x5283,0x5241,0x8363,0x83C3,0x4652,0x4646,0x465F,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x4446,0x4246,0x4646,0x5F46,0x4646,0x4642,0x4446,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7352,0x7373,0x5273,0x7373,0x7373,0x5263,0x7352,0x5252,0x5273,0x5273,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x465F,0x425F,0x6F46,0x4646,0x4646,0x4646,0x4146,0x7F7F,0x7F7F,0x4142,0x528F,0x528F,0x7F52,0x7F7F,0x417F,0x7F7F,0x7F7F,0x6F42,0x417F,0x7F6F,0x7F42,0x8F42,0xD4D4,0xE4D4,0xE4D4,0xC3E4,0xC3E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFD4,0xD4D4,0xF4D4,0xF4F4,0xD3E4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD3D4,0xE3E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xD4D4,0xC3D4,0xD4D4,0x62D4,0x46AF,0x4646,0x4646,0x4646,0x7342,0x7383,0x4252,0x4646,0x4642,0x465F,0x4646,0x4642,0x4246,0x4646,0x5252,0x7373,0x7352,0x7373,0x7373,0x8383,0x73B4,0x5252,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4246,0x4646,0x4646,0x4642,0x464F,0x4F46,0x4646,0x4646,0x4246,0x5246,0x4452,0x4646,0x8341,0x5283,0x4646,0x4646,0x7341,0x83C4,0x5273,0x4146,0x7352,0x7383,0x73D4,0x4646,0x5241,0xC473,0x4273,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4246,0x4646,0x4646,0x4646,0x4646,0x464F,0x5F46,0x4646,0x465F,0x4642,0x465F
.word 0xD473,0x7383,0x7373,0x5273,0x5273,0x9352,0x93C4,0x5283,0x7352,0x5252,0x4652,0x4F42,0x4646,0x465F,0x4F42,0x4446,0x4646,0x4646,0x464F,0x465F,0x4646,0x5F46,0x5F46,0x5F46,0x4646,0x465F,0x425F,0x427F,0x7F7F,0x417F,0x9F52,0x9F42,0x7F52,0x7F7F,0x7F42,0x7F41,0x7F7F,0x7F7F,0x7F41,0x7F6F,0x7F7F,0x4241,0xD462,0xD4D4,0xD4C3,0xD4E4,0xD4E4,0xE4E4,0xC3D4,0xC3D4,0x72D4,0x62D4,0xC4D4,0xD4D4,0xE3D4,0xF4F4,0xE4F4,0xD4D4,0xD4C3,0xE4C3,0xC3D4,0xD4E4,0xD4F4,0xE4E4,0xE4E3,0xE4E4,0xE4D3,0xE4F4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x4652,0x5F46,0x4646,0x4242,0x5273,0x4642,0x4246,0x4646,0x4646,0x4646,0x4642,0x4646,0x4242,0x8383,0x93D4,0xB3D4,0x93D4,0x8383,0x5273,0x5252,0x4646,0x4642,0x465F,0x4646,0x4646,0x4642,0x465F,0x4646,0x465F,0x4246,0x4246,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4341,0x4246,0x5F46,0x7342,0x5273,0x4246,0x5F46,0x5246,0x7383,0x4652,0x4646,0x4146,0x8363,0x7383,0x4646,0x4646,0x5241,0x7352,0x4642,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x8383,0xC483,0xD452,0xD4B2,0xC492,0x9373,0xC3C4,0x73C4,0x7352,0x5252,0x5252,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x417F,0x7F7F,0x417F,0x7F7F,0x4142,0x528F,0x42AF,0x416F,0x427F,0x427F,0x417F,0x7F7F,0x427F,0x416F,0x7F7F,0x8F42,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD5D4,0xD4BF,0xD462,0xD4BF,0xF4D4,0xF4F4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D3,0xF4E4,0xF4E4,0xE4E4,0xE3E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x62D4,0x4646,0x4646,0x4646,0x7342,0x4452,0x4246,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x4246,0x8352,0x83B4,0x7383,0x7373,0x5252,0x4252,0x4652,0x4246,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4644,0x4646,0x4646,0x5246,0x4273,0x4646,0x4646,0x4146,0x6373,0x4652,0x5F46,0x4646,0x6341,0x7383,0x4246,0x5F46,0x4646,0x5241,0x4641,0x4646,0x4644,0x464F,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x4246,0x4F46,0x4646,0x5F46,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4246,0x4646,0x4642,0x4246,0x4646,0x465F,0x4646
.word 0xB483,0x7393,0x8373,0xC493,0x9383,0x8373,0xC483,0x8393,0x5252,0x5283,0x5263,0x4652,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4246,0x4646,0x4446,0x4646,0x416F,0x417F,0x426F,0x7F7F,0x428F,0x8F52,0x4152,0x7F7F,0x7F7F,0x8F52,0x7F7F,0x427F,0x7F7F,0x7F6F,0x7F42,0x8F41,0xBFC4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xC3E4,0xD4D4,0xD4D4,0xD462,0xD483,0xD4D4,0xD4D4,0xE3D4,0xF3F4,0xE4F4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4C3,0xD4D4,0xAFD4,0x4246,0x4646,0x4242,0x4452,0x4646,0x4646,0x465F,0x4646,0x4646,0x4246,0x4646,0x7342,0x7373,0x5252,0x4642,0x4646,0x4646,0x4642,0x4246,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4642,0x465F,0x4642,0x465F,0x5F44,0x4646,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4642,0x4642,0x4646,0x465F,0x4646,0x5252,0x4646,0x4646,0x4246,0x4146,0x8352,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4246,0x4646,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7373,0x5273,0xC483,0x8383,0x7373,0x6352,0xB483,0x6373,0x7373,0x6352,0x7352,0x4252,0x4246,0x5F46,0x4646,0x5F46,0x4646,0x4642,0x4646,0x465F,0x4F42,0x4646,0x464F,0x4246,0x4646,0x465F,0x427F,0x7F7F,0x7F7F,0x7F7F,0x9F41,0x8F52,0x8F52,0x7F7F,0x417F,0x7F7F,0x7F42,0x6F41,0x417F,0x416F,0x7F7F,0x8F42,0xC4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0x62D4,0xBFD5,0xBFC4,0xD4D4,0xD4D4,0xF4E4,0xF3F3,0xE4F4,0xD4E4,0xD4C3,0xC3D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0x5262,0x4646,0x5F46,0x4346,0x4646,0x4246,0x4646,0x4646,0x4246,0x4646,0x4646,0x4246,0x7373,0x5252,0x4642,0x4246,0x6F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4642,0x4646,0x5F46,0x4246,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4642,0x4241,0x4246,0x4246,0x4646,0x4646,0x4241,0x4646,0x4642,0x4646,0x4642,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4644,0x4644,0x4246,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F
.word 0x8393,0x6352,0x7352,0x7363,0x7363,0x7383,0x5273,0x9383,0x83C4,0x5252,0x5273,0x5252,0x4642,0x4646,0x4246,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x464F,0x464F,0x4646,0x417F,0x417F,0x427F,0x7F7F,0x427F,0x628F,0x528F,0x427F,0x7F7F,0x417F,0x527F,0x417F,0x7F7F,0x416F,0x7F6F,0x4141,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD462,0xD462,0xD4D4,0xF4D4,0xF3F3,0xE4F7,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D3,0xD3E4,0xE4E4,0xE4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0x62D4,0x46AF,0x5F46,0x4646,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x465F,0x4246,0x5252,0x4652,0x4646,0x4446,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4642,0x4642,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4642,0x4646,0x4644,0x4246,0x4646,0x4646,0x4246,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4646
.word 0x83C4,0x9352,0xB3C3,0x8383,0x9352,0x83C4,0xC493,0xB483,0x7383,0x6263,0x7342,0x7352,0x6352,0x4246,0x4646,0x4646,0x465F,0x4646,0x4642,0x465F,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x6F46,0x7F41,0x7F41,0x7F7F,0x427F,0x7F6F,0x9F52,0x5242,0x7F8F,0x427F,0x7F7F,0x7F41,0x7F42,0x427F,0x7F6F,0x7F42,0x8F42,0xD4C4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x62BF,0x62D5,0xD4D4,0xD4D4,0xE4E4,0xF5FF,0xE4F4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4C3,0xD4E4,0xD4D4,0xD4E4,0x62D4,0x4662,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4246,0x4646,0x4646,0x4242,0x4152,0x4246,0x4246,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4246,0x5F46,0x5F46,0x4646,0x465F,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4F46,0x4646,0x464F,0x4646,0x4F42,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4644,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7373,0xC473,0xC483,0xB483,0x8373,0xB483,0x6383,0x5252,0x5252,0x8383,0x5273,0x7352,0x5252,0x4463,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x6F42,0x7F41,0x427F,0x417F,0x416F,0x417F,0x5241,0x52AF,0x7F41,0x7F7F,0x427F,0x7F6F,0x7F41,0x7F7F,0x7F7F,0x417F,0x418F,0xBF62,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD562,0xD4AF,0x62D4,0xD4D4,0xF4E4,0xFFF4,0xE4F4,0xD4E4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xC3D4,0x62D4,0x62AF,0x46AF,0x4646,0x4644,0x4642,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4246,0x5252,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4246,0x4646,0x4642,0x465F,0x5F46,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4642,0x4646,0x4642,0x464F,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x5F46,0x4646,0x4646,0x465F,0x465F,0x4246,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4246,0x5F46,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4246,0x4646
.word 0x7373,0x8383,0xB483,0x5273,0x6373,0x5273,0x5273,0xD493,0x6383,0x7352,0x7383,0x5252,0x7373,0x5252,0x4644,0x4246,0x4646,0x4246,0x4646,0x4246,0x5F46,0x4646,0x4F42,0x4646,0x465F,0x6F46,0x4241,0x7F7F,0x7F7F,0x427F,0x7F7F,0x527F,0x9F52,0x8F52,0x427F,0x7F7F,0x7F6F,0x7F42,0x7F42,0x416F,0x427F,0x8F41,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF3F3,0xE4F3,0xD4E4,0xE4E4,0xD4D4,0xD4BF,0xD4D4,0xD4D4,0xD4C3,0xE4E4,0xD3D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x62D4,0xD562,0x4642,0x5F42,0x4646,0x5F46,0x4646,0x4646,0x4246,0x4646,0x4246,0x5252,0x4652,0x4642,0x465F,0x4646,0x465F,0x4646,0x4246,0x4646,0x5F46,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4642,0x4F46,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x465F,0x4F46,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4246,0x4F46,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x464F,0x4646,0x465F,0x464F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4646,0x465F,0x4644,0x465F,0x4646,0x5F46,0x4646,0x4646
.word 0x5252,0x6363,0x5273,0x7373,0x8393,0x93E4,0x8363,0x83D4,0x7373,0xB483,0x7352,0x6383,0x7352,0x6352,0x4252,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x425F,0x417F,0x417F,0x427F,0x417F,0x7F7F,0x527F,0x629F,0x4252,0x7F7F,0x7F7F,0x417F,0x417F,0x527F,0x427F,0x7F7F,0x7F42,0xD4C4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xAFD4,0xD462,0xD4E4,0xD4D4,0xF4E4,0xF4FF,0xE4F4,0xD4E4,0xE4C3,0xD4D4,0xD483,0xD4E4,0xE4C3,0xD4D4,0xE4D4,0xD4E4,0xC3D4,0xD4D4,0xD4E4,0xD472,0x62AF,0xC4AF,0x4646,0x4646,0x4646,0x465F,0x4646,0x4642,0x4646,0x4646,0x4446,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x5F42,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4642,0x4646,0x4646,0x4246,0x4F46,0x4F46,0x4646,0x4646,0x4646,0x465F,0x4246,0x464F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4246,0x4F46,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4246,0x464F,0x465F,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4642,0x4646,0x4646,0x4646,0x4642,0x4646,0x4246,0x5F46,0x4246,0x4646,0x4642,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F
.word 0x5273,0x8383,0x8383,0x7383,0x93D4,0x83B4,0x7383,0x7373,0x8352,0x93D4,0x7373,0x8393,0x5252,0x7373,0x7373,0x4252,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4642,0x416F,0x427F,0x7F7F,0x7F7F,0x427F,0x417F,0x7F7F,0x9F52,0x9F62,0x7F42,0x427F,0x6F6F,0x7F7F,0x7F41,0x7F7F,0x4141,0x417F,0xBF62,0xD4C4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0x83D4,0xD4D5,0xD4D4,0xE4D4,0xF4E4,0xE4E4,0xE4F4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x62D5,0xAFD5,0x4646,0x4646,0x465F,0x4646,0x4646,0x464F,0x4F46,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F44,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F46,0x4246,0x4646,0x4F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x4646,0x465F,0x465F,0x4F42,0x5F46,0x5F46,0x4446,0x4246,0x4F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x465F,0x465F,0x465F,0x4646,0x4642,0x464F,0x465F,0x5F46,0x4246,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x464F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4446,0x4646,0x5F46,0x4646,0x465F,0x4646
.word 0x7352,0xB483,0x7383,0x5273,0xB483,0x7383,0x5252,0x8373,0x73B4,0x7383,0x7352,0xC473,0x7373,0x5252,0x5273,0x5273,0x4463,0x4246,0x4646,0x5F46,0x4646,0x4246,0x5F46,0x5F46,0x4646,0x7F7F,0x417F,0x7F41,0x7F42,0x417F,0x7F7F,0x7F7F,0x5242,0x42AF,0x418F,0x7F6F,0x7F7F,0x426F,0x427F,0x7F7F,0x427F,0x8F7F,0xD4C4,0xD472,0xC3D4,0xE4D4,0xC3D4,0xE4E4,0xE4D4,0xD4C3,0x62E4,0x629F,0x62AF,0x6262,0xE4D4,0xE4E4,0xD4D4,0x6283,0x62AF,0x629F,0xD4E4,0xE4E3,0xE4D4,0xC3D4,0xD4E4,0xE4C3,0xE4D4,0xD4D4,0x72D4,0xAFD4,0xBF62,0x46C4,0x4246,0x465F,0x4646,0x465F,0x465F,0x4246,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4246,0x464F,0x5F46,0x4246,0x464F,0x4646,0x4646,0x465F,0x4446,0x5F46,0x4646,0x4646,0x4F46,0x4246,0x4646,0x5F46,0x4646,0x4642,0x4246,0x464F,0x4F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F42,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4446,0x465F,0x4646,0x4642,0x4646,0x5F44,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4446,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x465F,0x465F,0x5F46,0x4646,0x465F,0x464F,0x465F,0x4644,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4644
.word 0x7352,0x7383,0x7363,0x7352,0x7373,0x5252,0xD483,0xC483,0x5293,0x7363,0x7352,0x5252,0x5252,0x5252,0x7352,0x7373,0x5252,0x4643,0x5F46,0x4646,0x4446,0x464F,0x4646,0x4646,0x5F46,0x7F42,0x7F41,0x7F42,0x417F,0x527F,0x427F,0x6F7F,0x9F41,0xD452,0x8F42,0x7F7F,0x6F42,0x7F7F,0x7F7F,0x7F41,0x417F,0x527F,0xC49F,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xC3E4,0x8FD4,0x9F52,0x9F42,0xAF62,0x5262,0xC4D4,0xBF62,0x8FC4,0x5252,0x528F,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD562,0xD4C4,0x46D4,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x465F,0x4F46,0x5F46,0x4646,0x4246,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4642,0x4646,0x4646,0x465F,0x4F46,0x4246,0x4646,0x4F46,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4F46,0x4646,0x4642,0x4646,0x4642,0x4642,0x4646,0x4646
.word 0x6352,0x5273,0x5252,0x8373,0xB3D4,0x73C4,0x7373,0x8383,0x7373,0x9383,0x83C4,0xC473,0xC493,0x8373,0x5273,0x6352,0x7373,0x5252,0x4373,0x4446,0x4646,0x4646,0x4646,0x4644,0x6F46,0x7F41,0x427F,0x417F,0x7F6F,0x417F,0x417F,0x7F7F,0x427F,0x629F,0x42AF,0x6F41,0x6F7F,0x6F41,0x417F,0x7F6F,0x4142,0x527F,0xD48F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4C3,0xD4E4,0x41D4,0x8F42,0x4241,0x428F,0xD4BF,0xBFD4,0x5262,0x418F,0x428F,0xC4AF,0xF4F4,0xE4E4,0xE4D3,0xD4E4,0xD4E4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0x83BF,0xD4BF,0x4662,0x5F42,0x4646,0x465F,0x465F,0x4246,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x4646,0x465F,0x5F46,0x5F46,0x4446,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4642,0x465F,0x465F,0x4646,0x4246,0x4F46,0x4646,0x5F46,0x4646,0x465F,0x4642,0x4646,0x4246,0x5F46,0x4646,0x464F,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x465F,0x4642,0x464F,0x464F,0x465F,0x4F42,0x5F46,0x4246,0x4646,0x465F,0x4646,0x4246,0x4646,0x4246,0x464F,0x4646,0x4646,0x4F42,0x4F46,0x4F46,0x4F46,0x4F46,0x5F46,0x4246
.word 0x5252,0x7352,0x7383,0xC452,0xB483,0x83B3,0x6352,0x6373,0x9352,0xD4C4,0xC4B3,0x8373,0x93C4,0xD473,0x8383,0x5252,0x5273,0x7373,0x7352,0x4252,0x4642,0x4646,0x4642,0x465F,0x7F46,0x417F,0x417F,0x7F7F,0x427F,0x426F,0x427F,0x7F7F,0x4241,0x528F,0x8F42,0x7F42,0x6F7F,0x6F7F,0x7F7F,0x7F7F,0x427F,0x417F,0xD47F,0x72D4,0xE4D4,0xC3D4,0xD3D4,0xD4E4,0xE4E4,0xD4D4,0x62D4,0x429F,0x418F,0x9F42,0x4142,0x5242,0x429F,0x428F,0x629F,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4C3,0xD4E4,0x72D4,0x62D4,0xD4D5,0xD4D4,0x4662,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x4246,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4246,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x465F,0x4644,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4642,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F44,0x4646,0x465F,0x4646,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x464F
.word 0x7352,0x7352,0x7352,0x7352,0x8373,0x73B4,0xC383,0x93C4,0x73D4,0x8383,0x62B4,0x5252,0x5252,0x8373,0x73C4,0x8373,0x5263,0x7352,0x5273,0x6373,0x5273,0x7352,0x6342,0x5263,0x4163,0x427F,0x427F,0x7F41,0x7F7F,0x417F,0x5241,0x417F,0x417F,0x527F,0x52AF,0x418F,0x7F7F,0x7F42,0x6F7F,0x7F42,0x4141,0x427F,0xD47F,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x9F62,0x9F52,0x9F52,0xE4C4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xF3E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xC4AF,0x72D4,0xD462,0x4646,0x465F,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x464F,0x4446,0x4246,0x5F46,0x4246,0x4F46,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x465F,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x465F,0x5F46,0x4F46,0x5F46,0x4646,0x465F,0x465F,0x4246,0x5F46,0x5F46,0x4F46,0x5F46,0x5F46,0x5F46,0x4646,0x464F,0x4646,0x4646,0x5F46,0x4646,0x4642,0x5F46,0x4246,0x4646,0x4642,0x464F,0x5F46,0x5F46,0x5F46,0x4246,0x464F,0x465F,0x464F,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x465F,0x465F,0x464F,0x465F,0x465F,0x5F46,0x5F46,0x4F46,0x4F46,0x4646,0x5F46,0x4646,0x5F46,0x4246,0x5F46,0x4646,0x4646,0x4646
.word 0x5252,0x6352,0x7373,0x7352,0x7352,0x5252,0x8373,0x7383,0x6383,0x7373,0x5252,0x9373,0x83C3,0x5263,0x5283,0xD473,0x8393,0x5263,0x8352,0x5273,0x7373,0x5273,0x7373,0x5273,0x7F52,0x7F41,0x417F,0x428F,0x7F7F,0x427F,0x8F9F,0x427F,0x427F,0x418F,0x9F62,0x7F52,0x6F7F,0x416F,0x7F6F,0x7F7F,0x427F,0x8F7F,0xC441,0xC4BF,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xF4D4,0xC4E4,0x52AF,0x62D5,0xE4E4,0xE4F4,0xE4D4,0xE4F4,0xF4E4,0xF4F3,0xE4E4,0xD3E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD462,0xD5D4,0xBFD4,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4F42,0x4646,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x4644,0x4246,0x4646,0x465F,0x4642,0x5F46,0x4646,0x464F,0x4646,0x4646,0x4642,0x4646,0x4646,0x4446,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x4646,0x464F,0x465F,0x4F46,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x465F,0x4642,0x5F46,0x4646,0x4246,0x4646,0x465F,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4F46,0x4646,0x5F46,0x4646,0x4646
.word 0x5252,0x5252,0x5252,0x7352,0xC493,0xC493,0x7352,0x6373,0x8352,0xC493,0xC493,0xC473,0xB3D4,0xC483,0xB493,0x7352,0x7373,0x5252,0x7373,0x5252,0x7352,0x7383,0x7373,0x7373,0x417F,0x4142,0x427F,0x7F7F,0x427F,0x416F,0x4152,0x4142,0x417F,0x8F7F,0x6242,0x529F,0x7F42,0x7F7F,0x427F,0x7F6F,0x4141,0x427F,0xC48F,0x72D4,0xBFD4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4E4,0xE2D4,0xF3F3,0xE4F4,0xE4D4,0xE4F4,0xF4D4,0xE4F4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xD3E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xBFD4,0xBFC4,0x6283,0x4246,0x5F46,0x4646,0x465F,0x4246,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x464F,0x4646,0x465F,0x4642,0x464F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4642,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4642,0x4246,0x4646,0x465F,0x4646,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x465F,0x465F,0x465F,0x5F46,0x4246,0x5F46,0x4646,0x4246,0x4646,0x4646,0x464F,0x4646,0x4246,0x4646,0x465F,0x4646,0x4642,0x4646,0x5F46,0x4246,0x4F46,0x4246,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F
.word 0x5242,0x5252,0x5273,0x7373,0xC483,0x8383,0x7363,0x5273,0xC473,0xD493,0xD493,0x8352,0x7393,0x7363,0x93D4,0x5283,0x8373,0x7383,0xC352,0x5273,0x5273,0x5252,0x5252,0x5252,0x417F,0x427F,0x8F7F,0x7F41,0x7F7F,0x7F7F,0x9F52,0x9F42,0x427F,0x428F,0x528F,0x41D5,0x418F,0x426F,0x7F6F,0x7F6F,0x427F,0x417F,0x838F,0xC4BF,0xC4D4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xC4D4,0xB4C5,0xC563,0xE473,0xE4F3,0xF1E4,0x63E4,0x4363,0x6343,0x8463,0xD4D5,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4D4,0x62D4,0x42D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x464F,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4642,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x464F,0x464F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4642,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x4642,0x4246,0x5F46,0x4646,0x464F,0x4246,0x4646,0x4646,0x4642,0x5F46,0x4646,0x465F,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x464F,0x464F,0x464F,0x4646,0x4646,0x4F46,0x4F46,0x4646,0x465F,0x465F,0x5F46,0x4646,0x4646,0x4642,0x4246,0x4646,0x4644,0x4646,0x4642,0x4646,0x4246
.word 0x5252,0x5252,0x5252,0x5252,0x6373,0x4273,0xD483,0xD4C3,0x7393,0x93C4,0x83B4,0x7352,0x7363,0x5283,0xC483,0x7373,0xE493,0xC4C3,0x7373,0x7352,0xC383,0x73C4,0x5252,0x7F52,0x7F41,0x8F42,0x427F,0x7F41,0x7F42,0x417F,0x9F42,0x5262,0x528F,0x7F7F,0x8F41,0xC441,0x7F41,0x7F7F,0x7F6F,0x427F,0x416F,0x7F8F,0x527F,0x62D4,0x83BF,0x72D4,0x62D4,0x52AF,0x4242,0x6342,0x4243,0x6342,0x6363,0xC563,0xC573,0x6373,0x4343,0x6343,0x4343,0x4263,0x4343,0x7343,0xB4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD572,0x83BF,0xC4BF,0x469F,0x4646,0x465F,0x4246,0x4646,0x465F,0x4646,0x4642,0x4646,0x4246,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4644,0x465F,0x465F,0x4642,0x4646,0x4246,0x464F,0x4246,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x465F,0x4646,0x4642,0x465F,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4F46,0x4646,0x4646,0x4246,0x4F46,0x4646,0x464F,0x4646,0x465F,0x4446,0x4646,0x5F46,0x4646,0x464F,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x5F46,0x4F46,0x4F46,0x5F46,0x4F46,0x5F46,0x5F46,0x5F46,0x4646
.word 0x4283,0x5252,0x5263,0x5252,0x5252,0x5252,0xC383,0x8393,0x7373,0x7373,0x7383,0xD483,0xD4D3,0x83D4,0x7373,0x7352,0xC483,0x8393,0x5263,0x7383,0x7363,0x5273,0x8373,0x7F52,0x427F,0x418F,0x7F7F,0x7F52,0x7F7F,0x7F7F,0x6242,0x52AF,0x527F,0x428F,0x428F,0x8F7F,0x6262,0x417F,0x6F7F,0x7F6F,0x427F,0x7F41,0x4241,0xC4BF,0xBFC4,0xAFC4,0x7F41,0x4242,0x4364,0xC543,0xC563,0x6363,0x63C5,0x63C5,0x4363,0x6343,0xC563,0xC543,0xC5C5,0xC573,0xC563,0x4663,0x425F,0xD463,0xD462,0xD4D4,0x62D4,0xC4BF,0x62D4,0xD4D5,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x4642,0x464F,0x5F46,0x4646,0x465F,0x4646,0x5F46,0x4F46,0x4F46,0x5F46,0x4246,0x464F,0x465F,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x465F,0x465F,0x465F,0x4646,0x464F,0x4646,0x4646,0x4642,0x465F,0x465F,0x4644,0x4646,0x4646,0x4F46,0x4646,0x4642,0x465F,0x4642,0x465F,0x465F,0x4642,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x5F46
.word 0x6393,0x5252,0x6362,0x8352,0x8383,0x52B4,0x9373,0x73B4,0x5273,0x5252,0x7383,0x9363,0x8383,0x73D4,0x8352,0x83C4,0x7363,0x5263,0xE483,0x83D4,0xD483,0x7383,0x5252,0x417F,0x417F,0x8F42,0x427F,0x4141,0x7F7F,0x7F42,0x6241,0x9FD5,0x9F52,0x7F42,0x8F41,0x7F52,0xC48F,0x428F,0x417F,0x7F6F,0x416F,0x427F,0x8F7F,0xD562,0xD572,0xD572,0xD4C4,0x4563,0x4242,0x426F,0x7F42,0x4242,0x7F42,0x5F42,0x5F6F,0x4A6F,0x425F,0x5243,0x4342,0x4342,0x4242,0x6342,0xD4D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD562,0xBF62,0x83C4,0x6362,0x6263,0x6263,0x5252,0x4252,0x4246,0x5F46,0x4646,0x4246,0x4646,0x4446,0x4646,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x464F,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4F46,0x4646,0x465F,0x465F,0x464F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x464F,0x464F,0x465F,0x465F,0x4646,0x5F44,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4642,0x5F46,0x4646,0x465F,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4644,0x4646,0x4646,0x4642
.word 0x83D4,0x5242,0x5252,0x7352,0x6373,0x7373,0x7373,0x5273,0xC452,0xD4C3,0x93C3,0x7373,0x8373,0x6383,0xC493,0x8392,0x8373,0x7383,0x8373,0x83D4,0xB3B4,0x83D4,0x4273,0x417F,0x427F,0x418F,0x417F,0x8F52,0x6F42,0x7F7F,0x9F42,0x5252,0x429F,0x7F41,0x8F42,0x8F41,0x8F42,0x4162,0x7F42,0x7F6F,0x427F,0x418F,0x427F,0xBF62,0xBFC4,0xD4C4,0xC4BF,0xAFD4,0x43C5,0x4344,0x84C5,0xC5C5,0xC5C5,0x4445,0x4642,0x445F,0xC544,0xE5D6,0xD5F5,0xC5C5,0x4373,0xD5C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0x62BF,0x62D4,0x83D4,0x5273,0x7373,0x7352,0x7352,0x7373,0x6373,0x5473,0x4446,0x4646,0x465F,0x4646,0x465F,0x4644,0x5F46,0x4646,0x4646,0x5F46,0x5F46,0x4446,0x4646,0x465F,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4246,0x5F46,0x4246,0x464F,0x465F,0x465F,0x465F,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4644,0x4646,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4246,0x4F46,0x4646,0x4246,0x4646,0x465F,0x4246,0x4F46,0x4646,0x465F,0x4642,0x4646
.word 0x93D4,0x4283,0x5252,0x5252,0x5252,0x5252,0x8383,0x83C4,0x8352,0x83C4,0xB483,0x7352,0x7352,0x8363,0xB3D4,0x83C4,0xC363,0xC493,0x6373,0x5273,0x8373,0x52B4,0x8F42,0x7F41,0x8F42,0x4241,0x7F7F,0x5262,0x527F,0x417F,0x527F,0x629F,0x8F62,0x8F42,0x417F,0x527F,0x418F,0x8F8F,0x528F,0x7F52,0x7F7F,0x7F42,0x417F,0x739F,0x62D4,0x62D4,0x62D4,0x83D5,0xC5AF,0xC573,0xD663,0xD5C4,0x73EF,0xD5FF,0xF5D5,0xF3D5,0xE4D5,0xB4C5,0xC563,0x6352,0xC5C5,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xC4BF,0xD562,0xD4BF,0x7373,0x7373,0x7373,0x7373,0x7373,0x7352,0x7352,0x5273,0x5273,0x7352,0x4654,0x4642,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x465F,0x4246,0x4646,0x4646,0x4642,0x4646,0x4446,0x4646,0x4646,0x465F,0x4F42,0x4F46,0x4646,0x4446,0x465F,0x4642,0x464F,0x464F,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4F46,0x4F46,0x5F46,0x4646,0x4642,0x4642,0x5F46,0x4646,0x4646,0x465F,0x4642,0x4646,0x4646,0x464F,0x464F,0x465F,0x4246,0x464F,0x465F,0x465F,0x465F,0x4646,0x4646,0x4246,0x4646,0x464F,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4642,0x4F46,0x4F46,0x4646,0x465F,0x464F,0x464F,0x4246,0x4F46,0x4646,0x465F,0x4646,0x464F,0x464F
.word 0xC493,0x7383,0x4252,0x7352,0x8352,0x5283,0x7373,0x8373,0x7373,0xB483,0x7383,0x9373,0x93C3,0x7373,0xC483,0x6383,0x9383,0x93C4,0x7352,0x7383,0x5252,0x4152,0x7F7F,0x7F42,0x417F,0x7F8F,0x7F41,0x8F62,0x417F,0x7F7F,0x5241,0x73AF,0x42AF,0x419F,0x427F,0x528F,0x428F,0x427F,0x8F41,0x4152,0x7F7F,0x7F41,0x527F,0x528F,0xD4AF,0xD4D4,0xC4BF,0xC4BF,0x9F83,0x63C5,0x6363,0x6373,0xC5C5,0xEFD4,0xF5E5,0xD5C5,0xC5B4,0xC573,0x7363,0x63C5,0x72C5,0xD4D4,0xD4E4,0xC3D4,0xD4D4,0xD472,0xD462,0x83BF,0x83D4,0xD483,0xD493,0xE4C3,0x8393,0x8383,0x7373,0x5273,0x7352,0x7373,0x7373,0x5273,0x4643,0x4646,0x4246,0x4F46,0x5F46,0x5F46,0x4246,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4246,0x464F,0x4646,0x464F,0x4646,0x4646,0x4642,0x4646,0x4646,0x465F,0x4246,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x464F,0x4646,0x465F,0x4646,0x4446,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646
.word 0xB3C4,0x83E4,0x5273,0x5252,0x6373,0x6352,0x5273,0x8373,0x7363,0x5273,0x7352,0xC493,0xC493,0x5283,0x6373,0x5273,0x7373,0x6373,0x9352,0x93C4,0x5263,0x4142,0x417F,0x7F8F,0x4142,0x8F42,0x627F,0x429F,0x7F52,0x427F,0x8F7F,0xD573,0x9F52,0x4252,0x418F,0xC47F,0x418F,0x8F7F,0x8F42,0x428F,0x7F62,0x427F,0x417F,0x527F,0xAF52,0x6262,0x62D5,0x62D4,0xC4D4,0x529F,0x73D6,0x6343,0x63D5,0xC5C5,0x6373,0x6373,0xC573,0x7363,0x63C5,0xD4BF,0xE4D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xC4AF,0x73BF,0xD4D4,0x6373,0x5252,0x5252,0x5252,0x5252,0x5252,0x7373,0x8383,0x8373,0x8373,0x7373,0x7352,0x6273,0x6252,0x4643,0x4646,0x4646,0x4646,0x465F,0x464F,0x4246,0x464F,0x4F46,0x4246,0x464F,0x465F,0x465F,0x4642,0x465F,0x464F,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x4F46,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4642,0x4246,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x465F,0x5F46,0x4646,0x4246,0x4646,0x4642,0x465F,0x465F,0x465F,0x465F,0x465F,0x465F,0x465F,0x4646,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4246,0x4646,0x4642,0x4246,0x5F46,0x5F46,0x4646,0x465F,0x4642,0x464F,0x4646,0x4246
.word 0xC493,0xB3D4,0x52C4,0x5252,0x5252,0x5273,0x7352,0x6373,0x5252,0x8352,0x7373,0xC483,0xD483,0x7383,0x5252,0x7373,0x7352,0x5252,0x7373,0x7383,0x4173,0x427F,0x428F,0x417F,0x428F,0x428F,0x428F,0x9F52,0x7F8F,0x417F,0x8F42,0x9F62,0x8F62,0x9F52,0x7F42,0x5241,0x428F,0x417F,0x427F,0x8F7F,0x6242,0x417F,0x7F7F,0x8F42,0x5252,0xAFD5,0xBFC4,0xBFC4,0xBFC4,0xAFC4,0x4352,0x63D6,0x52C5,0x6363,0xC563,0x6343,0xC543,0xC552,0xAF73,0xD4C4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x62D4,0x62D4,0xD4D5,0xC462,0x5283,0x5252,0x6352,0x8373,0x93C4,0x73B4,0x5252,0x7352,0x8373,0xD493,0x7383,0x7373,0x8373,0x5273,0x6273,0x4652,0x4644,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4246,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x465F,0x4642,0x465F,0x4246,0x4F46,0x4F46,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4246,0x464F,0x4246,0x4646,0x465F,0x464F,0x465F,0x464F,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x465F,0x465F,0x4646,0x465F,0x4246,0x464F,0x465F,0x465F,0x4646,0x4646,0x4646,0x4644,0x4646,0x465F,0x4F46,0x4F46,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x465F,0x4646
.word 0x83E4,0xD493,0xB4B3,0x5273,0x5252,0x5252,0x5252,0x5252,0x8352,0xB483,0x52B2,0x7373,0x7383,0x7352,0x8383,0x8383,0x8352,0x73C4,0x5252,0x5263,0x7F7F,0x8F41,0x417F,0x7F7F,0x8F42,0x5241,0x7F8F,0x5252,0x427F,0x417F,0x428F,0x529F,0x42AF,0x429F,0x418F,0x7F8F,0x9F42,0xD562,0xD4D4,0x83D4,0x8F9F,0x7F7F,0x7F42,0x8F41,0x9F52,0x6262,0xD4D4,0xD483,0x83D4,0xD462,0x52AF,0x528F,0x529F,0x529F,0x419F,0x9F9F,0x9F52,0x9F41,0xD462,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x62D4,0xBFD4,0xBFC4,0x83C4,0x5273,0x7373,0x8373,0x8393,0x8352,0x83C4,0x7383,0x8352,0x83C4,0x5263,0x5252,0x8373,0x8393,0x7373,0x8373,0x7383,0x5273,0x5273,0x4642,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4246,0x4646,0x4646,0x465F,0x465F,0x465F,0x4246,0x4646,0x465F,0x4646,0x4F46,0x4646,0x4646,0x5F46,0x4246,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4F46,0x5F46,0x5F46,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x464F,0x464F,0x465F,0x465F,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x5F46
.word 0x93D4,0x93B4,0xB3C4,0x7383,0x4252,0x6341,0x7352,0x8373,0x7352,0x7373,0x6383,0x7352,0x5252,0x8363,0x83B4,0x73B4,0x8373,0x7383,0x7373,0x7F52,0x7F41,0x4142,0x7F42,0x7F42,0x8F41,0x8F52,0x527F,0x419F,0x418F,0x427F,0x418F,0xAF42,0x8F73,0x4152,0x428F,0x8F52,0x6252,0x62D5,0xD462,0xBFD4,0xD4C4,0x83D4,0x42AF,0x7F7F,0x4241,0x529F,0xD4AF,0x62D4,0xD4D4,0xD4D4,0xC4D4,0x42AF,0x428F,0x428F,0x428F,0x8F42,0x9F42,0xD4C4,0xE4E5,0xD4E4,0xE4D4,0xD4D4,0xD4C3,0xBFD4,0x62C4,0x62D5,0x8383,0x93C4,0x7383,0xD473,0xC4D4,0x5252,0x5273,0x6352,0x5252,0x5252,0x8373,0x7373,0x5252,0x7363,0x8383,0x7383,0x7373,0x7373,0x7373,0x5273,0x4646,0x465F,0x4646,0x5F44,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x4246,0x4646,0x4646,0x465F,0x4246,0x464F,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x464F,0x5F46,0x4646,0x4246,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x464F,0x465F,0x4F46,0x4646,0x465F,0x5F46,0x4246,0x4646,0x4642,0x4646,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x4646,0x4446,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xB3D4,0x83C4,0x83B4,0x93D4,0x5283,0x5252,0x5273,0x7352,0x7352,0x5252,0x5263,0x8352,0xC483,0x7383,0x7352,0x7352,0x6373,0x5273,0x7F41,0x417F,0x4142,0x528F,0x418F,0x417F,0x8F42,0x4241,0x418F,0x9F42,0x7F42,0x8F41,0x8F7F,0x5242,0x7F8F,0x427F,0x8F8F,0xD552,0x62AF,0x62BF,0xC4AF,0x6262,0x62D4,0x62D4,0x62D5,0xC4D4,0xAFD4,0x9F52,0x6252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD562,0xD472,0x6383,0xC493,0x7393,0x7352,0x5273,0x7373,0xC483,0x9393,0x7373,0xD483,0xB3D4,0x52B4,0x93D4,0x6383,0x5252,0x8363,0x8383,0x7373,0x7373,0x8373,0x7373,0x4663,0x4642,0x465F,0x4F46,0x4F46,0x5F46,0x5F46,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x464F,0x465F,0x465F,0x4F46,0x4646,0x465F,0x5F46,0x4646,0x464F,0x5F46,0x5F46,0x5F46,0x4246,0x4646,0x4646,0x465F,0x4646,0x465F,0x465F,0x4F46,0x4F46,0x4F44,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4246,0x5F46,0x4646,0x4F46,0x4F46,0x4F46,0x5F46,0x5F46,0x4F46,0x4646,0x4246,0x4646,0x5F46,0x4246,0x5F46,0x5F46,0x4F46,0x4646,0x4642,0x4F46,0x4646,0x464F,0x465F,0x465F,0x465F,0x4642
.word 0xC4C3,0xC493,0xD493,0xC3D4,0x83D4,0x5273,0x5252,0x5252,0x5252,0x5252,0x5252,0x7352,0x8373,0x5273,0x8373,0x7352,0x5252,0x4252,0x417F,0x427F,0x428F,0x428F,0x8F7F,0x527F,0x428F,0x7F7F,0x8F42,0x4142,0x427F,0x428F,0x7F6F,0x5241,0x428F,0x8F41,0x6242,0x62AF,0x62D4,0xC4D5,0xBFD4,0xD4C4,0xAFD4,0xD4D4,0xD462,0xD4D4,0x62D4,0x62D4,0x528F,0xD4AF,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xF4F4,0xF4E4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xD3D4,0xD4D4,0x72D4,0xBFD5,0xBFC4,0x83B4,0x73C4,0x8383,0x52B4,0x5252,0x8373,0x7383,0x8383,0xD4C4,0x5283,0x9383,0x93D4,0x7383,0xC483,0x7383,0x7352,0x5252,0x8363,0x8393,0x8383,0x8373,0x8373,0x7373,0x4642,0x4646,0x4646,0x4642,0x4646,0x4646,0x4246,0x4646,0x4646,0x4642,0x4646,0x464F,0x4646,0x4646,0x4246,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4246,0x4646,0x465F,0x4646,0x4646,0x4F42,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4246,0x4646,0x465F,0x464F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x464F
.word 0xB483,0x4644,0xD493,0xC493,0xC493,0xC493,0x5273,0x5252,0x7352,0x5273,0x5283,0x6352,0x5252,0x5283,0x7373,0x5273,0x5252,0x417F,0x4142,0x8F52,0x5241,0x41AF,0x427F,0x7F41,0x8F41,0x4241,0x7F8F,0x8F52,0x7F8F,0x6F7F,0x7F7F,0x8F41,0x8F42,0x6262,0xD4D5,0xC4D4,0xC4BF,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x41D4,0xD462,0xD4D4,0xE4D4,0xD3D4,0xF4E4,0xE4E4,0xF4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x83D4,0x7373,0x5273,0x7352,0x5252,0x7373,0xD473,0xD4B3,0x83D4,0x8373,0x8383,0x5263,0x7363,0x5273,0x5252,0x5263,0x7373,0x8383,0x8363,0x5273,0x6352,0x8383,0x8373,0x7373,0x6352,0x7373,0x4252,0x4646,0x4646,0x4642,0x465F,0x465F,0x5F46,0x4F46,0x4F46,0x4646,0x4246,0x4246,0x4F46,0x4646,0x464F,0x4646,0x4646,0x4246,0x4646,0x4642,0x464F,0x4642,0x465F,0x4F44,0x4F46,0x4646,0x4246,0x5F46,0x4646,0x4F42,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4246,0x464F,0x465F,0x4F46,0x4646,0x4642,0x465F,0x4246,0x4F46,0x4646,0x4646,0x5F46,0x4F46,0x4246,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x465F,0x4646
.word 0x4473,0x4646,0x9383,0x93C4,0x93C4,0xB3C4,0x83C4,0x5252,0x7352,0x7352,0x5273,0x5273,0x8352,0x7373,0x7352,0x5252,0x4141,0x8F42,0x527F,0x528F,0x529F,0x427F,0x8F7F,0x8F42,0x8F42,0x527F,0x7F7F,0x4241,0x7F7F,0x6F42,0x7F41,0x8F42,0x6241,0xBFD5,0x62D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0x62AF,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4F4,0xE4F4,0xC3E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x7383,0xB3C4,0x93C3,0x8373,0xD4D4,0xC4B3,0x8362,0x93B4,0x83D4,0x6352,0x5252,0x7373,0xD483,0xC493,0x7383,0x8373,0xE493,0x83D4,0x7383,0xC493,0x5263,0x7352,0xB483,0x8352,0x8383,0x7373,0x8373,0x4652,0x4F42,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4644,0x4646,0x4646,0x4F46,0x4646,0x465F,0x465F,0x4246,0x4646,0x4F46,0x5F46,0x4646,0x464F,0x465F,0x465F,0x4642,0x4F46,0x5F46,0x4646,0x465F,0x4646,0x5F46,0x4646,0x465F,0x464F,0x4246,0x4F46,0x4646,0x4246,0x4646,0x4646,0x4246,0x4F46,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4642,0x4646,0x4642,0x5F46,0x5F46,0x4646,0x4646,0x4646
.word 0x4646,0x836F,0x93D4,0x93C4,0x83B4,0xB483,0xD4B3,0xD4D4,0x6383,0x5252,0x4252,0x5252,0x6373,0x5273,0x5252,0x7F42,0x7F42,0x417F,0x9F52,0x9F52,0x8F52,0x8F42,0x417F,0x7F8F,0x5241,0x8F41,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x427F,0x527F,0xD5D4,0xD462,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xE4C3,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0x51D4,0xD4D4,0xD3D4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0x72D4,0xD483,0x5293,0xC483,0xE483,0xC483,0xC493,0x83B3,0x8363,0xB483,0x5273,0x9373,0xC3D4,0x83C3,0x8363,0xB3D4,0x93D4,0x8363,0x83C4,0xB493,0x6383,0xC493,0x83B3,0x7373,0x7352,0x8373,0x83B3,0x7383,0x7352,0x5273,0x4664,0x4646,0x465F,0x4646,0x4246,0x464F,0x465F,0x464F,0x465F,0x464F,0x464F,0x5F46,0x5F46,0x5F46,0x4F46,0x4646,0x4646,0x465F,0x5F46,0x5F46,0x4646,0x4646,0x4644,0x465F,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x4646,0x465F,0x4642,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x465F,0x465F,0x465F
.word 0x4646,0x9346,0x93D4,0x93D4,0xD4C4,0x8383,0x93C4,0x9393,0xB3D4,0x5273,0x5252,0x5252,0x6273,0x5252,0x8F52,0x4141,0x7F8F,0x9F42,0x6242,0x529F,0x8F52,0x7F8F,0x7F42,0x4142,0x528F,0x428F,0x527F,0x7F41,0x7F42,0x416F,0x8F42,0xD462,0xC4AF,0x62D4,0xD4BF,0xD4D4,0xE4D4,0xD3D4,0xD4E4,0xE4E4,0xE4C3,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x62D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4E4,0x83D4,0xB483,0xB483,0x8383,0x83B4,0x73B4,0x5273,0x5252,0x7373,0x9363,0x93C4,0x83D4,0x8373,0xC493,0x83D4,0x8373,0xB473,0x8383,0x5263,0x7373,0x7373,0x7352,0x5273,0x7352,0xB483,0xB493,0x7383,0x7383,0x7373,0x4642,0x4646,0x465F,0x465F,0x4646,0x4246,0x4646,0x4646,0x4246,0x4646,0x4642,0x4646,0x4642,0x4246,0x4F46,0x4646,0x4646,0x4642,0x4646,0x4246,0x4646,0x4646,0x4246,0x4646,0x465F,0x4246,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x464F,0x4646,0x465F,0x4246,0x4646,0x4646,0x4642,0x465F,0x4246,0x4F46,0x4246,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4642,0x4246,0x4646,0x465F,0x4646,0x4246,0x4646,0x4646,0x465F,0x4646,0x4246,0x5F46
.word 0x4246,0xD446,0xC493,0x6383,0x4646,0x9363,0xD4D4,0xD4C4,0xC493,0xE483,0x93D4,0x8383,0x5273,0x4142,0x5242,0x7F7F,0x627F,0x418F,0x9FD4,0x9F52,0x428F,0x417F,0x417F,0x428F,0x8F41,0x7F42,0x7F7F,0x427F,0x6F7F,0x5241,0xC4D4,0xC4BF,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xC3D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x62D4,0x62AF,0x9F62,0x6262,0x6252,0x6262,0x6262,0x62D4,0xD462,0x8373,0x8383,0x8383,0xB483,0x7373,0x5252,0x5252,0x7373,0x5252,0xC483,0xC3B3,0xD4C3,0x8383,0x83B4,0x73C4,0x8363,0x83B4,0x5273,0x6373,0x5252,0x8373,0x8373,0x8383,0x7383,0x8373,0x83D4,0x5273,0x7352,0x9383,0x83C4,0x7373,0x7373,0x7373,0x4642,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4F46,0x4646,0x4642,0x465F,0x464F,0x4246,0x4646,0x4646,0x4642,0x465F,0x465F,0x4646,0x4646,0x4F46,0x4646,0x4F42,0x5F46,0x4646,0x4246,0x4646,0x4646,0x465F,0x4246,0x4F46,0x5F46,0x4646,0x464F,0x5F46,0x4F46,0x4646,0x465F,0x5F46,0x5F46,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x465F,0x465F,0x464F,0x4646,0x465F,0x4646,0x464F,0x464F,0x464F,0x4642,0x4646,0x4644,0x4646,0x4646
.word 0x4646,0xB346,0x83D4,0x4663,0x4246,0x8346,0xD4C3,0xD493,0xC4B3,0x9383,0xD4C4,0xD4C3,0x5283,0x417F,0x8F9F,0x427F,0x8F52,0x5242,0x52AF,0x4252,0x418F,0x7F7F,0x7F42,0x8F52,0x8F42,0x7F8F,0x4142,0x7F41,0x527F,0xD4D4,0xD4AF,0xD4C4,0x62D4,0xBFD4,0xD4C4,0xD4D4,0xD4E4,0xD4E4,0xC3E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xC3E4,0xE4E4,0xC3D4,0xD4E4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x8F62,0x427F,0x8F8F,0x7F8F,0x8F8F,0x7F8F,0x8F8F,0xC473,0x7383,0x7383,0x93B4,0x5273,0x8352,0x7393,0x8373,0x92D4,0x93D4,0x8373,0xD4D4,0x93D4,0x73B4,0x8383,0x5273,0x5273,0x5252,0x8383,0x8373,0xB3C4,0xD4D4,0x7373,0xD4C4,0xC493,0x8383,0xB3C4,0x83C4,0x5252,0x8373,0x93C4,0x83B4,0x8383,0x7352,0x4373,0x4644,0x4646,0x4642,0x465F,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F42,0x4646,0x4642,0x4646,0x4646,0x4246,0x4F46,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4F42,0x4646,0x4642,0x465F,0x465F,0x4F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x5F46,0x4646,0x464F,0x465F,0x465F,0x4646
.word 0x4646,0xC45F,0x63B3,0x4646,0x4646,0xD446,0x93D4,0x83C4,0x4646,0x8383,0xE493,0x6383,0x8F41,0x5242,0x7F8F,0x4142,0x418F,0x528F,0x9F52,0x7F52,0x7F8F,0x7F42,0x527F,0x42AF,0x428F,0x7F41,0x427F,0x526F,0x62D4,0x62D5,0xD4D4,0xD462,0xBFD4,0xD4C4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D3,0xD3E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0x8F62,0x5252,0x5252,0x4252,0x5252,0x4152,0x7373,0x5252,0x5252,0x6352,0x6383,0xD493,0xD4C3,0x8373,0xC493,0xC493,0x8373,0x8383,0x7383,0x7352,0xD483,0x8393,0x8363,0x93C4,0xD4C4,0x52D4,0xC483,0x9393,0x73C4,0x8383,0x83C4,0x52B4,0x7352,0x5273,0x7363,0x5252,0x8363,0x8383,0x73B4,0x7363,0x7383,0x4652,0x4646,0x464F,0x4646,0x465F,0x4642,0x5F46,0x4646,0x465F,0x464F,0x464F,0x465F,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4246,0x4F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x465F,0x465F,0x465F,0x465F,0x464F,0x5F46,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4642,0x464F,0x4246,0x464F,0x4646,0x4646,0x465F,0x4644,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642
.word 0x4646,0x8346,0x46D4,0x6F46,0x4646,0x8342,0x83D4,0x4693,0x4646,0xD473,0x73D4,0x8F42,0x527F,0x7F9F,0x7F42,0x8F41,0x427F,0x9F62,0x8F52,0x8F52,0x7F42,0x417F,0x5241,0x8F52,0x7F41,0x7F41,0x6F41,0xBF52,0xBFC4,0xD462,0xD462,0xC4BF,0x62D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4C3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x529F,0x52AF,0x52AF,0x9F9F,0x5252,0x5252,0x8373,0xC3D4,0xB3D4,0x83C4,0x93C4,0x83D4,0xC473,0xB483,0x7393,0x7373,0x7373,0x5273,0x9373,0xD4C3,0xD4C3,0x7383,0xC483,0x9393,0x73D4,0x8383,0x83B4,0x5283,0xB483,0x8383,0x7363,0x9383,0x93C4,0x83C4,0x6383,0x5252,0x93B4,0x8383,0x5273,0x7383,0x6273,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4246,0x5F46,0x4F46,0x5F46,0x4646,0x4642,0x5F46,0x4646,0x4246,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4246,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x465F,0x4646,0x4646,0x4642,0x4646,0x4246,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4F46,0x4646,0x465F,0x465F,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4F46,0x4646,0x4246,0x5F46,0x5F46,0x5F46,0x4246,0x4F46,0x5F46
.word 0x4646,0x4646,0x4673,0x4646,0x4646,0x9346,0x93D4,0x4646,0x4642,0x8346,0x4152,0x7F7F,0x8F62,0x7F42,0x527F,0x427F,0x8F41,0x9F52,0x9F42,0x417F,0x7F7F,0x7F41,0x9F52,0x428F,0x428F,0x7F7F,0xD452,0x62D4,0xC4D5,0xAFD4,0xD4D4,0xD483,0xD4D4,0x72D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xF4E4,0xE4E4,0xE4E3,0xD3E4,0xE4E4,0xE4E4,0xD4D4,0x62D4,0xD462,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0x62D4,0x73AF,0xC4AF,0x9F62,0x7352,0xC483,0x7383,0xC483,0xC493,0x7373,0x7373,0x7373,0x5252,0x5252,0x7352,0x5252,0x8363,0xB3C4,0xC473,0x9393,0xC493,0x6383,0x7373,0x8373,0x5263,0x6373,0x5273,0x8363,0x5263,0x6352,0x7383,0xC483,0xC3B3,0xD4B2,0x83D4,0x5273,0x7352,0x9373,0x7383,0x8373,0x7373,0x6383,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x4646,0x4246,0x4F46,0x4646,0x4646,0x465F,0x465F,0x4246,0x4646,0x464F,0x4646,0x4446,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x4642,0x465F,0x4646,0x4246,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x5F46,0x4646,0x4646,0x465F,0x464F,0x465F,0x5F44,0x4646,0x4642,0x464F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4246,0x4646,0x4642,0x4646,0x465F,0x8346,0xB483,0x4642,0x4646,0x4146,0x7F7F,0x527F,0x7F8F,0x7F41,0x8F41,0x417F,0x527F,0x428F,0x429F,0x7F8F,0x427F,0x427F,0x429F,0x417F,0x417F,0x427F,0x62D4,0xBFD4,0xBF83,0x62C4,0x62D5,0xBFD4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD3D4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0x62E4,0x418F,0x8F7F,0x418F,0x418F,0x417F,0xD441,0xD4D4,0xD4D4,0xAF62,0xAF73,0x5252,0x739F,0xC3C4,0x73C4,0x8383,0x73C4,0x5252,0x5252,0x6352,0xD473,0xD4B3,0x6383,0xB3D4,0x93E4,0xD4C3,0x8383,0x83B4,0x83C4,0x5273,0x5252,0x5252,0xD473,0xD4B3,0x8393,0x8363,0xC3D4,0xC3E4,0x52C4,0xC483,0xC3D4,0x93D4,0x8383,0x8363,0x5263,0x7373,0x83D4,0x7373,0x8373,0x8373,0x4642,0x4644,0x4642,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4246,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x464F,0x4F46,0x5F46,0x4646,0x5F46,0x4F46,0x4F46,0x5F46,0x5F46,0x4446,0x4646,0x4646,0x4646,0x465F,0x4346,0x4643,0x465F,0x4646,0x4646,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4646,0x4646
.word 0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4683,0x4646,0x465F,0x416F,0x427F,0x8F41,0x7F42,0x8F42,0x427F,0x7F41,0x8F42,0x418F,0x417F,0x427F,0x7F7F,0x8F41,0x7F42,0x427F,0x428F,0xD452,0xC4BF,0xD562,0xD462,0xD462,0xC4BF,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4C3,0xE4D4,0xE4E4,0xE4D3,0xE4F4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0x8F83,0x4142,0x428F,0x62D4,0x8F62,0x5242,0x8F62,0x7F42,0x417F,0xD562,0xD4D4,0x6252,0x52AF,0xC473,0xD4C3,0x5273,0x5252,0x7352,0x9383,0xD4D4,0xC4B3,0x8373,0xB3C4,0x83D4,0xB493,0xD483,0x8393,0x7363,0x8383,0x7373,0x8352,0xC3C4,0x93D4,0x7383,0xD493,0xC3D4,0x8383,0xC493,0xD4B3,0xB493,0x8383,0xB3D4,0x83B4,0x7373,0xC483,0x7393,0x5252,0x8373,0x8383,0x7373,0x7383,0x4283,0x5F46,0x4646,0x465F,0x465F,0x4646,0x4F42,0x5F46,0x5F46,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x465F,0x464F,0x465F,0x4646,0x4644,0x4646,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x4246,0x4646,0x4642,0x4646,0x464F,0x465F,0x465F,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x464F,0x465F,0x4642,0x4646,0x8342,0x4643,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x4646,0x4F42,0x5F46,0x5F46,0x5F46,0x4646,0x444F,0x464F
.word 0x5F46,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4673,0x4642,0x6F5F,0x7F41,0x8F41,0x7F42,0x417F,0x427F,0x417F,0x428F,0x7F7F,0x4142,0x427F,0x7F7F,0x7F41,0x8F42,0x7F41,0x8F41,0x8F42,0x62D4,0x62D5,0xBFD5,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0x41D4,0x528F,0x52D4,0x419F,0xD58F,0x6262,0x8F8F,0x62D4,0x429F,0x428F,0x629F,0xD4D4,0x9FD4,0x5252,0x8373,0x83C4,0x7352,0xD493,0xC4C3,0xB483,0xB493,0x93B3,0x9373,0x83B4,0x73C4,0x8383,0x93B4,0x52B4,0x7373,0x5252,0x7352,0x9363,0xD4D4,0xE4B3,0x73D4,0x83C4,0xC493,0x8363,0x93B4,0x93B4,0x83C4,0x7373,0xB483,0x7383,0x5263,0x7363,0x7373,0x5252,0x7352,0x73B4,0x7373,0xB483,0x7383,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4246,0x4646,0x465F,0x465F,0x5F46,0x4446,0x4646,0x4646,0x4246,0x4646,0x465F,0x5F46,0x4446,0x4646,0x4646,0x4246,0x4F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4246,0x4646,0x4246,0x5F46,0x4646,0x5F46,0x4246,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4642,0x464F,0x4246,0x9373,0x4646,0x4644,0x4646,0x4646,0x4646,0x4642,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4246
.word 0x4646,0x4646,0x4246,0x4F46,0x5F46,0x4646,0x4646,0x425F,0x427F,0x427F,0x7F41,0x417F,0x427F,0x7F8F,0x7F41,0x7F42,0x8F41,0x7F42,0x7F41,0x7F41,0x427F,0x8F41,0x7F7F,0x8F42,0x9F52,0xAFC4,0xBF83,0x6283,0x62D5,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D3,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0x41D4,0x8F52,0xAF52,0x8F42,0x7F42,0x4142,0x429F,0x428F,0xD4AF,0x9FC4,0x8F41,0x527F,0x528F,0xD4D4,0x639F,0x7373,0x7352,0xB483,0xC483,0xC4B3,0x8373,0x8383,0x73B4,0x7352,0x8383,0x7363,0x7363,0x5252,0x7352,0xD483,0xB3D4,0xD4D4,0x7362,0x9383,0x93C4,0x52B4,0x8383,0x83B4,0x8373,0x7373,0x7383,0x6373,0x5273,0x5252,0x7373,0x8362,0x9383,0xC483,0xC483,0x8383,0xD4D4,0xD462,0xD4D4,0xD4D4,0x8372,0x8372,0x7262,0x4652,0x4646,0x4646,0x4642,0x4646,0x464F,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4644,0x4F46,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4246,0x4646,0x4646,0x5F46,0x4646,0x464F,0x465F,0x465F,0x4646,0x5F44,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x7342,0x6393,0x4646,0x4646,0x4642,0x4246,0x4F46,0x4F46,0x4646,0x4642,0x5F46,0x4646,0x465F,0x4642,0x4646,0x465F,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x7F42,0x7F7F,0x4141,0x427F,0x7F7F,0x7F41,0x7F41,0x4142,0x417F,0x428F,0x7F8F,0x7F7F,0x427F,0x417F,0x427F,0x7F7F,0x7F41,0xD452,0xD462,0xD562,0xBFD4,0xD483,0xD4C4,0xC4BF,0xD4D4,0xD4D4,0xD4C3,0xE4D4,0xC3D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xD3E4,0xD4E4,0xD4D4,0x528F,0x429F,0x527F,0x529F,0x529F,0x528F,0x529F,0x429F,0x8F41,0x8F41,0x7F42,0x8F62,0x418F,0xD4AF,0x73D4,0x7352,0xC483,0x7383,0x8383,0x73B4,0x6352,0x5263,0x5252,0x5252,0x5252,0xD483,0xE4C3,0x93D4,0x6383,0x8383,0xC493,0x8393,0x7373,0x83B4,0x83C4,0x6263,0x7373,0x5273,0x6352,0x6373,0x5273,0x7352,0x8383,0xC483,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xE4D4,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xD4D5,0xD4D4,0x62D4,0x4262,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x4646,0x4246,0x4F46,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4642,0x4646,0x4246,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x4F46,0x4646,0x4646,0x9352,0x43C4,0x4644,0x465F,0x5F46,0x4646,0x464F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4246,0x464F
.word 0x5F46,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x424F,0x427F,0x417F,0x8F42,0x7F41,0x4241,0x7F7F,0x7F42,0x428F,0x427F,0x7F7F,0x4241,0x7F41,0x7F41,0x7F41,0x417F,0x427F,0x627F,0xAFD4,0xBFC4,0xAFC4,0xC483,0xC4BF,0xD4BF,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE441,0x41D4,0x427F,0x9F52,0x8F52,0x6242,0x62D5,0x52AF,0x9F42,0x9F52,0x4252,0x428F,0x418F,0x6252,0x8F52,0x8F52,0xD483,0x73D4,0x8383,0x6352,0x5273,0x5252,0xB383,0x93E4,0x7373,0xD4D4,0xB493,0x9373,0x93C4,0xC3D4,0x83C4,0xB483,0xC483,0x6383,0x5273,0x6352,0x5252,0xC483,0xB3D4,0x7383,0xD473,0xE4B2,0x83D4,0x8373,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD4D4,0xD4D4,0xD4D4,0x4172,0x4646,0x4646,0x4246,0x4646,0x464F,0x4646,0x4646,0x4642,0x5F46,0x4246,0x4646,0x4646,0x4646,0x4F42,0x4F46,0x4646,0x465F,0x4646,0x465F,0x4642,0x5F46,0x5F46,0x5F46,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x4642,0x465F,0x4646,0x465F,0x4646,0x464F,0x4646,0x4646,0x5242,0xA383,0x4663,0x4646,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x4646,0x5F46,0x5F46,0x5F46,0x4646,0x465F,0x464F,0x4646
.word 0x4646,0x4646,0x4646,0x4246,0x5F46,0x5F46,0x7F42,0x7F41,0x4241,0x7F8F,0x427F,0x8F7F,0x7F41,0x7F41,0x8F41,0x7F7F,0x4241,0x8F7F,0x7F42,0x7F7F,0x4142,0x7F7F,0x417F,0xBF52,0xD4C4,0xD562,0xD462,0xD562,0xD462,0xD462,0xD4D4,0xD472,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D3,0xE4E4,0x41D4,0xE4D4,0x8F41,0x9F42,0xD562,0x9F52,0x8F41,0xD562,0xAF62,0x9F42,0x5252,0x529F,0x418F,0x8F42,0x9F8F,0x8F42,0x5252,0xD48F,0xD4D4,0x7373,0x8352,0xC493,0x7383,0x93C4,0xD4B4,0x5283,0xC493,0x83D4,0x8373,0x83B4,0xB483,0x6393,0x7373,0x6373,0x5252,0xC383,0xB3D4,0x7383,0xD483,0xD4C3,0x83D4,0x8373,0x93D4,0xB493,0xD483,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4C3,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4BF,0xD4AF,0xD462,0xD4D4,0xD4D4,0x4652,0x4F42,0x4F46,0x5F46,0x4646,0x4642,0x4646,0x4F46,0x4646,0x465F,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x464F,0x4246,0x464F,0x4646,0x4642,0x4646,0x4646,0x4642,0x8342,0xB493,0x4644,0x4642,0x465F,0x4642,0x4646,0x4246,0x5F46,0x4246,0x464F,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646
.word 0x4646,0x4644,0x4644,0x4646,0x4646,0x426F,0x7F7F,0x7F42,0x8F41,0x7F42,0x7F41,0x7F41,0x7F42,0x427F,0x7F8F,0x4142,0x7F7F,0x8F41,0x417F,0x7F7F,0x7F41,0x7F42,0x9F52,0xD473,0xAF62,0xBFC4,0xD562,0x83BF,0xD4D4,0xC4BF,0xD572,0xD4D4,0xD4D4,0xD4E4,0x62D4,0xD4D4,0xC3D4,0xD4E4,0xD4E4,0xE4D3,0xD441,0x8FE4,0x4142,0x8F52,0x6241,0x428F,0x7F8F,0x628F,0x62D5,0x42AF,0x529F,0x529F,0x8F52,0x5252,0x9F42,0x5252,0x529F,0x7F8F,0xD4D4,0x73D4,0x6373,0x7373,0x5283,0xC473,0x8383,0xB452,0x9383,0x73C4,0x7373,0x7373,0x7373,0x8352,0xE493,0xD493,0xB4B3,0xC473,0xD483,0x83D4,0x9373,0x93C4,0x93D4,0x8373,0xB483,0xD462,0xC4BF,0xD4D4,0xE4E4,0xD4E4,0xE4D3,0xE4D4,0xE4D4,0xD4D3,0xE4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xBFD4,0xD4D4,0xD4D4,0x5272,0x4643,0x4646,0x4646,0x5F46,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x465F,0x4246,0x464F,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4246,0x5F46,0x4646,0x4F42,0x4646,0x4642,0x4646,0x4246,0x7363,0x4644,0x4646,0x4646,0x4646,0x465F,0x464F,0x465F,0x465F,0x6342,0xB493,0x4393,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x4646,0x4642,0x465F,0x4646,0x4644,0x4246,0x5F46
.word 0x5F46,0x4646,0x465F,0x5F46,0x6F46,0x7F41,0x4241,0x418F,0x7F42,0x7F41,0x427F,0x7F8F,0x7F41,0x7F41,0x7F42,0x8F41,0x7F42,0x4142,0x7F7F,0x7F42,0x7F42,0x7F41,0xD452,0x62D5,0x62D5,0xC4D4,0x83AF,0x62D4,0xC4BF,0xD562,0xD4BF,0xD4D4,0xE4D4,0xD4D4,0xBF62,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0x41D4,0x41E4,0xAF52,0x9F52,0x427F,0x529F,0x5262,0x8F41,0xD541,0x6262,0x529F,0xAF52,0x9F52,0x9F42,0x8F41,0x9F52,0x9F52,0x4152,0x627F,0x83D4,0x5252,0x5273,0x5252,0x7373,0x5273,0x7373,0x7373,0x5273,0x5252,0x5252,0x8373,0x7373,0x93C4,0x93C4,0x93D4,0x8373,0x93C4,0xB493,0x8373,0x83B4,0xB493,0x7373,0x6273,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4C3,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4C3,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBF62,0xD4D5,0x4262,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x464F,0x4646,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4F46,0x4646,0x4F42,0x4F46,0x4646,0x4646,0x9352,0x4663,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x9373,0xC493,0x4683,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x464F,0x465F,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x416F,0x7F7F,0x7F41,0x7F42,0x7F8F,0x427F,0x417F,0x427F,0x7F7F,0x7F42,0x7F41,0x427F,0x417F,0x428F,0x7F7F,0x7F7F,0x7F41,0x6242,0x62AF,0x62BF,0xBFD4,0x62C4,0x62D4,0xD4D5,0xD462,0xD4D4,0xD483,0x61D4,0xD4D4,0x62D4,0xC4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x428F,0x42AF,0x428F,0x9F52,0x8F42,0xD552,0x9F62,0x9F42,0xAFC4,0x8F52,0x9F42,0x6252,0x529F,0x9F52,0x9F52,0x5252,0x529F,0x9F52,0x5252,0x6383,0xC483,0x7373,0x7383,0x7352,0x5252,0x5252,0x5273,0x8383,0x93C4,0xC3D4,0x5283,0xC483,0xB493,0xC483,0x8373,0xB483,0x7383,0x7373,0x7373,0x5273,0x6252,0xC4AF,0xD4BF,0xE4D4,0xD4E4,0xD4C3,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4C3,0xC3D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4AF,0x4642,0x4642,0x4646,0x4642,0x465F,0x4646,0x5F46,0x5F46,0x4246,0x464F,0x4646,0x4646,0x4646,0x465F,0x4246,0x464F,0x4F42,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4446,0x6246,0x6293,0x4646,0x464F,0x465F,0x4642,0x4646,0x4446,0x5246,0xC493,0x8383,0x4646,0x5F42,0x4646,0x4646,0x4646,0x4246,0x4646,0x4246,0x4646,0x5F46,0x5F46,0x5F46,0x4246,0x464F,0x4646,0x5F46,0x5F46
.word 0x4646,0x4642,0x465F,0x6F42,0x7F41,0x4241,0x417F,0x417F,0x7F42,0x7F41,0x8F41,0x7F7F,0x7F41,0x7F41,0x7F42,0x7F41,0x427F,0x418F,0x7F7F,0x4241,0x417F,0x528F,0xC4D5,0xBFC4,0x62C4,0xC4BF,0xC4BF,0xC4AF,0xC4BF,0xAFD4,0xD4D4,0xE562,0xD4D4,0xBFD4,0xAFC4,0x417F,0x417F,0x418F,0xE462,0xC3D4,0x9F62,0x8F52,0x8F42,0xAF52,0x6262,0x429F,0x9F8F,0x5262,0x418F,0x428F,0xD562,0x4241,0x529F,0x52AF,0x52AF,0x9F52,0x9F52,0x9F52,0x9F52,0x9F52,0x7352,0x7373,0x7383,0x9373,0x5283,0x7383,0xC483,0xB4C3,0x9373,0x83C4,0xD4C4,0x6383,0x8383,0x83B4,0x8383,0x7363,0x7373,0x5252,0x5252,0x5252,0x7373,0xAF52,0x62C4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xC3D4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xC3D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x4261,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4642,0x465F,0x4644,0x4646,0x465F,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4642,0x4646,0x5F46,0x5F46,0x4646,0x4642,0x4246,0x8373,0x4652,0x4646,0x5F46,0x4F46,0x5F46,0x5F46,0x9344,0x8383,0x73B4,0x4646,0x4646,0x4646,0x4642,0x4446,0x4646,0x5F46,0x4646,0x464F,0x4646,0x4642,0x4646,0x464F,0x4646,0x465F,0x4646,0x4646
.word 0x4646,0x4646,0x5F46,0x426F,0x7F7F,0x8F42,0x7F41,0x7F42,0x7F7F,0x7F7F,0x7F42,0x427F,0x7F7F,0x7F42,0x7F41,0x8F42,0x417F,0x7F7F,0x7F42,0x417F,0x7F42,0xD4C4,0x62BF,0x62D4,0x62D5,0x62D5,0x62D5,0x62D4,0x62D5,0xC4D4,0xD4D4,0xD4AF,0xD4D4,0x52D4,0xD462,0x8F41,0x7F42,0x6252,0xD4E5,0xE4D4,0xD4D4,0x8F41,0x417F,0x6242,0xAFD5,0x8F42,0x4241,0x8F8F,0x8F42,0xD452,0xE4E4,0xE4E4,0x8FC4,0xD4D4,0x8F42,0x9F52,0xAF73,0x5252,0x529F,0x529F,0x8F9F,0x5252,0x5273,0x6373,0x6383,0x7383,0x8383,0x6383,0xB483,0x8383,0xB493,0x5273,0x7373,0x7373,0x5263,0x5252,0x9383,0x8363,0xC493,0x8393,0x5252,0xC4AF,0xC4BF,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D3,0xE4E4,0xE4E4,0xE4D3,0xD4E4,0xD4D3,0xD4E4,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4C3,0xC3E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x4641,0x4646,0x465F,0x4246,0x5F46,0x4446,0x4646,0x4646,0x4F46,0x4646,0x4F46,0x5F46,0x4646,0x4646,0x5F46,0x5F46,0x5F46,0x4F46,0x4646,0x4F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x7342,0x52A3,0x4646,0x4646,0x4646,0x4646,0x4646,0x8383,0x93D4,0x4683,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x5F46,0x4646,0x465F,0x4246,0x464F,0x4646,0x465F,0x4646,0x4646,0x4246,0x5F46,0x4246
.word 0x4246,0x4646,0x7F46,0x7F42,0x4141,0x427F,0x7F7F,0x7F41,0x7F41,0x4142,0x7F41,0x417F,0x417F,0x417F,0x417F,0x427F,0x7F7F,0x7F42,0x7F7F,0x417F,0x418F,0x839F,0xAFC4,0xAFC4,0xBFC4,0xD462,0xC4AF,0x62BF,0xAFD4,0x62D4,0xD4D4,0xD462,0xE4D4,0xAF41,0x627F,0xD4D5,0xE4D4,0xE4D4,0xD4D4,0x41D4,0x428F,0x527F,0x9F52,0x8F8F,0x427F,0x528F,0xAF62,0x9F73,0x9F42,0x41D4,0x528F,0x8F7F,0x427F,0xE441,0xD4E4,0x8F41,0x427F,0x529F,0x52AF,0x9F62,0x4152,0x5242,0x5252,0x5252,0x7352,0x7352,0x6363,0x5252,0x7373,0x7373,0x6373,0x7352,0x5252,0x5252,0x8373,0xC3C4,0xD4E4,0x8373,0x83C4,0x5273,0xD552,0xC462,0xD462,0xD4D4,0xE4D4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4C3,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD462,0x5262,0x4646,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x465F,0x4642,0x465F,0x5F46,0x4646,0x4246,0x9383,0x4643,0x465F,0x4646,0x4644,0x4646,0x9383,0xB4D4,0x4673,0x4642,0x465F,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x464F,0x465F,0x4646
.word 0x4646,0x465F,0x415F,0x7F7F,0x7F41,0x7F41,0x7F41,0x7F7F,0x7F7F,0x427F,0x7F7F,0x7F42,0x7F7F,0x7F42,0x427F,0x417F,0x7F7F,0x417F,0x417F,0x427F,0xC47F,0xBF52,0x62C4,0x62D4,0x62D4,0x62D5,0x62D4,0x62D5,0x83D5,0xC4BF,0xC4BF,0xD462,0x6241,0x83D5,0x62D5,0xC4BF,0xD4D4,0xD4D4,0x6262,0x428F,0x7FD4,0xAF52,0xC4C4,0x8F62,0x4142,0x628F,0xD4D5,0x9F62,0xC452,0x527F,0xE4D4,0xE4D4,0x52D4,0x628F,0x8F7F,0x7F42,0x8F42,0x8F42,0x9F52,0x5252,0x529F,0x8F8F,0x5242,0x5252,0x5252,0x5252,0x5252,0x5273,0x9383,0xD4E4,0x83B3,0x9352,0x93C4,0x93D4,0x8373,0xC483,0x8393,0x8352,0x5273,0xAF52,0xAF62,0xD462,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D3,0xE4D4,0xD4D4,0xD4C3,0xE4C3,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xC3E4,0xD4E4,0xE4D4,0xD4E4,0xBFD4,0x4262,0x4646,0x4646,0x465F,0x465F,0x464F,0x464F,0x4642,0x4646,0x4246,0x4646,0x4646,0x4246,0x464F,0x465F,0x4642,0x465F,0x4646,0x4246,0x4646,0x4646,0x7343,0x4644,0x4646,0x4646,0x4646,0x4646,0xB373,0x4383,0x4246,0x5F46,0x5F46,0x4446,0xB483,0x8393,0x4673,0x4646,0x4646,0x464F,0x4246,0x4646,0x4642,0x4646,0x4644,0x465F,0x4646,0x464F,0x465F,0x464F,0x4646,0x4646,0x4646,0x465F
.word 0x4646,0x6F46,0x417F,0x417F,0x427F,0x7F7F,0x7F42,0x7F42,0x427F,0x8F41,0x7F41,0x417F,0x7F7F,0x4241,0x417F,0x427F,0x417F,0x7F7F,0x7F7F,0x7F42,0xD441,0x62D5,0xBFD4,0xBFC4,0xBFC4,0xD562,0xC4AF,0xC4BF,0xC4BF,0xD462,0xD4D4,0xAFD4,0xD4D4,0xAFD4,0xD4C4,0xD462,0xD4D4,0xE4E4,0x9F62,0xD452,0x8FD4,0x6241,0x9FD5,0x428F,0x529F,0x8F52,0x418F,0x8F52,0xBF52,0xD4D4,0xE4E4,0xD4E4,0x41E4,0x8F7F,0x7F42,0x7F41,0x8F41,0x8F41,0x8F42,0x9F42,0x9F42,0x7F52,0x4241,0x5241,0x5252,0xC483,0x8393,0x5283,0x83B4,0xB483,0xC483,0xC473,0xC493,0xB483,0x8352,0x83B4,0x73B4,0x6363,0x5252,0xD562,0xD562,0xBFD4,0xD4D4,0xD4D4,0xC3E4,0xE4E4,0xD4E4,0xE4E4,0xD3E4,0xE4D4,0xD3D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xC3D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0x6262,0x4652,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4F46,0x5F46,0x4F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4646,0x465F,0x5246,0x4273,0x4246,0x4646,0x465F,0x4642,0x8342,0x73D4,0x4646,0x4646,0x4646,0x4646,0x9383,0x83B4,0x4673,0x5F46,0x4646,0x4646,0x4646,0x445F,0x4673,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646
.word 0x5F46,0x7F7F,0x7F41,0x427F,0x7F7F,0x417F,0x7F7F,0x7F7F,0x7F41,0x7F7F,0x427F,0x7F7F,0x7F42,0x417F,0x427F,0x8F7F,0x427F,0x427F,0x7F7F,0x8F41,0xD452,0xD462,0x62D4,0x62D5,0x62D5,0x62D4,0x62D4,0x62D4,0xD4D4,0xC4BF,0xD462,0xC4D4,0xD462,0xD4D4,0xBF62,0xAFC4,0xAF83,0xD452,0xD4D4,0x7FD4,0x41D4,0x528F,0x417F,0x9F42,0xD462,0x62D4,0x8F52,0xAF42,0xC462,0xE4E4,0xE4D4,0xC3D4,0x8F62,0x7F42,0x8F41,0x8F42,0x7F42,0x8F42,0x4241,0x529F,0x428F,0x429F,0x8F8F,0x7352,0x5242,0x5252,0x7363,0x73C4,0x7373,0x7383,0x7373,0x8352,0x73B4,0x7373,0x7352,0x6352,0x5252,0x5273,0xAF62,0xAF62,0xAFC4,0xC483,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4C3,0xE4E4,0xE4D4,0xD3D4,0xD4E4,0xD4E4,0xD4E4,0x51D4,0x4246,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4246,0x4246,0x7352,0x4642,0x4646,0x4646,0x4646,0x5246,0x9383,0x4652,0x4246,0x4646,0x635F,0xC493,0x8383,0x4273,0x4646,0x4646,0x4644,0x465F,0x8363,0x4644,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4446,0x4646,0x4646,0x465F,0x465F,0x4642
.word 0x7F46,0x4241,0x416F,0x7F7F,0x4141,0x427F,0x7F7F,0x427F,0x416F,0x7F42,0x7F41,0x7F7F,0x7F7F,0x7F41,0x417F,0x427F,0x417F,0x7F7F,0x427F,0x427F,0xC4AF,0xD4AF,0xD562,0xD462,0xC4BF,0xC4BF,0xC4BF,0xD4AF,0xD562,0xD462,0xAFD4,0xD4C4,0xD4AF,0xD4D4,0xC4D4,0xAF62,0xD452,0xD4D4,0x62E4,0x628F,0x429F,0xAF52,0x8F42,0x8F41,0x417F,0x8F9F,0x527F,0x73AF,0xC4AF,0x8F62,0x5252,0x8F8F,0x7F42,0x428F,0x428F,0x418F,0x428F,0x428F,0x528F,0x528F,0x418F,0x528F,0x4162,0x629F,0x52C4,0x5252,0x5252,0x5252,0x5252,0x7363,0x5252,0x5273,0x5273,0x5252,0xC483,0x93C3,0x5283,0x5252,0x73AF,0x62D5,0x83D5,0xD4BF,0xD4D4,0xE4E4,0xD4E4,0xD3E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD3D4,0xE4E4,0xD4C3,0xE4E4,0xD4C3,0xE4E4,0xC3D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4C3,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x4641,0x465F,0x4F46,0x4246,0x4646,0x4646,0x4644,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4F42,0x4246,0x5F46,0x4246,0x4646,0x4646,0x4646,0x5F46,0x8344,0x4673,0x4246,0x5F46,0x4646,0x4642,0xC483,0x4683,0x465F,0x465F,0x7346,0x83C3,0x93B4,0x4473,0x4646,0x4642,0x465F,0x6346,0x4483,0x4246,0x4646,0x4642,0x4246,0x4F46,0x4646,0x5F46,0x4646,0x4642,0x4246,0x5F46,0x4646
.word 0x7F7F,0x7F41,0x7F41,0x7F42,0x7F42,0x7F7F,0x427F,0x7F6F,0x7F7F,0x417F,0x7F7F,0x427F,0x7F7F,0x427F,0x427F,0x7F7F,0x427F,0x7F7F,0x417F,0x417F,0xD4C4,0xD562,0xC4BF,0x83BF,0xD562,0xD462,0xD562,0xD462,0x62BF,0xBFD4,0xD4C4,0x62D4,0x62D4,0xD4D4,0xAFD4,0xD4D4,0x62D4,0x628F,0xAFD4,0xD4D4,0x9FC4,0x5252,0x528F,0xD562,0x62D4,0x7F42,0x5252,0x428F,0x419F,0xD49F,0xE4E4,0xE4D4,0x41D4,0x417F,0x418F,0x8F42,0x8F42,0x8F41,0x4241,0x429F,0x428F,0xE48F,0x62D4,0x9F42,0x7362,0x73C4,0x5252,0x5252,0x5252,0x7352,0x5273,0x8393,0x8383,0x5283,0x8383,0x83B4,0x5273,0xAF52,0xAFC4,0xD462,0xD4BF,0xD4D4,0xD4D4,0xE4D4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E3,0xC3E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xC3E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x52D4,0x4646,0x4646,0x4F46,0x5F46,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4F46,0x4646,0x4F46,0x5F46,0x5F46,0x5F46,0x4646,0x5246,0x7393,0x4646,0x4646,0x4646,0x5F46,0x8374,0x6393,0x4646,0x4646,0x8346,0xB493,0x7393,0x4373,0x4646,0x465F,0x4646,0x8342,0x4383,0x4646,0x4F46,0x5F46,0x4646,0x4646,0x4246,0x4646,0x465F,0x464F,0x4646,0x4646,0x5F46
.word 0x417F,0x427F,0x417F,0x417F,0x417F,0x417F,0x417F,0x7F7F,0x7F41,0x7F41,0x427F,0x7F6F,0x7F41,0x7F41,0x417F,0x418F,0x7F7F,0x7F7F,0x7F42,0x9F52,0x62D4,0x62D4,0xD4D4,0xD562,0x83BF,0xC4BF,0xC4AF,0xC4BF,0xD562,0xD462,0xD462,0xD4D4,0xD4D4,0xD49F,0x62D4,0x41D4,0xD562,0xD462,0xD4D4,0xD452,0xD4D4,0x7F8F,0xAF52,0xD4C4,0x9F62,0x528F,0xD4AF,0x62D4,0x528F,0xE462,0xE4D4,0xE4E4,0x7FE4,0x7F42,0x7F42,0x8F41,0x4142,0x428F,0x528F,0x429F,0x418F,0x527F,0xD5E4,0x529F,0x629F,0xD452,0x5283,0x7352,0x7352,0x8383,0x52C4,0x7373,0x7373,0x7373,0x7373,0x7373,0x5252,0x73AF,0xD562,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xC3E4,0xD4E4,0xD4D4,0xC3E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xC3E4,0xE4D4,0xD4D4,0x62D4,0x4662,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x5F46,0x6346,0x7383,0x4642,0x465F,0x5F46,0x4646,0x8344,0x9383,0x4646,0x4246,0xB442,0x8393,0x93B4,0x4383,0x4646,0x4646,0x4646,0x9373,0x4673,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x4646
.word 0x7F7F,0x417F,0x7F7F,0x8F42,0x7F7F,0x427F,0x7F7F,0x7F42,0x427F,0x7F6F,0x7F7F,0x7F7F,0x7F6F,0x427F,0x427F,0x7F7F,0x4241,0x417F,0x8F52,0x62C4,0xD4D5,0xC4BF,0xD4AF,0x72D4,0x62D5,0x62D5,0x62D4,0xD4C4,0xD4AF,0xAFD4,0xBFC4,0xD462,0xC4BF,0x62D4,0xD4D4,0xD49F,0xD49F,0xC4BF,0xD462,0xBFD4,0xD462,0x8F41,0x427F,0x417F,0x429F,0xD552,0xD4D4,0x7FD4,0xD552,0xD4D4,0xE4E4,0xD4E4,0x41E4,0x417F,0x417F,0x8F42,0x4241,0x7F8F,0x8F42,0x8F42,0x8F42,0x7F42,0xD48F,0x52D4,0x629F,0x629F,0x93C4,0x5263,0x5263,0x7363,0x5273,0x5273,0x5252,0x7352,0xC483,0x5293,0xAF62,0xBF73,0xBFC4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D3,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4E4,0xC3D4,0xE4D4,0xE4C3,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xC3E4,0xE4D4,0xD4D4,0xC3D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x52BF,0x4646,0x4644,0x465F,0x4F42,0x5F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x4646,0x4642,0x4646,0x465F,0x465F,0x465F,0x465F,0x4646,0x4646,0x9373,0x4673,0x4646,0x4646,0x4646,0xB442,0xB493,0x4652,0x4646,0x9363,0x93B4,0xB483,0x4383,0x4246,0x4646,0x4242,0x9393,0x4673,0x4642,0x4246,0x4646,0x4642,0x465F,0x465F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x465F
.word 0x7F41,0x7F42,0x4142,0x7F7F,0x7F42,0x4241,0x7F8F,0x417F,0x7F7F,0x7F41,0x7F42,0x427F,0x417F,0x417F,0x417F,0x427F,0x7F7F,0x527F,0x41D4,0x9FD4,0xD462,0xD4C4,0x6283,0x62D5,0xBFD4,0xBFC4,0xD483,0xC4AF,0x62D4,0xD4D4,0xD462,0x62D5,0xD4D4,0xD4D4,0xC4AF,0x62D4,0x6262,0xD5D4,0xD4BF,0xD4D4,0x52D4,0x41D5,0xAF42,0x62C4,0x8F8F,0x7F7F,0x8F41,0x8F52,0x9F41,0xD483,0xE4E4,0xE4E4,0x7FD4,0x7F42,0x7F42,0x428F,0x7F8F,0x4142,0x5241,0x529F,0x4141,0x428F,0x528F,0x62E4,0x528F,0x52D5,0xC462,0x52D4,0x5252,0x5252,0x5252,0x5273,0x6352,0xC483,0x8373,0x9F63,0x6252,0x62D5,0xD4D5,0x83D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xD4D3,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD3D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD3E4,0xE4E4,0xD4E4,0xC3E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x62D4,0x4652,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4F46,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4642,0x8352,0x6383,0x4446,0x4646,0x4644,0x8346,0x83C4,0x4673,0x4242,0x9383,0xB483,0x8383,0x43B4,0x4646,0x4646,0x7346,0xB493,0x4663,0x5F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4642,0x4646,0x4246
.word 0x417F,0x417F,0x417F,0x417F,0x527F,0x8F41,0x417F,0x427F,0x7F7F,0x427F,0x417F,0x7F7F,0x7F6F,0x427F,0x7F7F,0x7F41,0x7F52,0x8F52,0xAFC4,0xAF83,0xBFC4,0xD4D4,0xD4BF,0xBFD4,0xD4C4,0xD483,0xD4AF,0xBFC4,0xBFC4,0xBF83,0xD4C4,0xD462,0xBF62,0xD4C4,0xD483,0xC4BF,0xAFD4,0x62D4,0x62C4,0xD4D4,0xD4D4,0x9FD4,0x6252,0xAFD4,0x4142,0x6262,0x8FD4,0x5241,0xD462,0x42AF,0x6251,0x428F,0x6241,0x8F8F,0x417F,0x417F,0x417F,0x428F,0x428F,0x418F,0xAF52,0x4152,0x7F8F,0xD4D4,0x8F52,0xBFD4,0x5262,0xD462,0x5283,0x5252,0x7383,0x7383,0x8383,0x8373,0x5252,0x6252,0xD5AF,0xD462,0xD472,0xD4D4,0xD4D4,0xE4E4,0xC3D4,0xE4D4,0xE4E4,0xE4C3,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xC3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0x4262,0x4646,0x4646,0x5F46,0x4246,0x4F46,0x4646,0x5F46,0x4646,0x4642,0x5F46,0x4646,0x5F46,0x5246,0x4463,0x4646,0x5F46,0x4646,0x5F46,0x8354,0x73B4,0x5F46,0x5F46,0x5F46,0x8346,0x8383,0x4673,0x7343,0x7393,0x93D4,0xB3C4,0x4383,0x4646,0x465F,0x9352,0x83D4,0x4663,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4F46,0x5F46,0x4646,0x5F46,0x4646
.word 0x427F,0x417F,0x427F,0x7F7F,0x9F52,0x8F52,0x7F42,0x417F,0x417F,0x8F42,0x7F41,0x4152,0x427F,0x7F7F,0x7F7F,0x7F7F,0x8F62,0x9F52,0xD4C4,0xD48F,0xD4C4,0xD462,0xD4C4,0xD4D4,0xD462,0xC4AF,0x62C4,0x83D4,0x62D4,0xD4D5,0xBF62,0xBFD4,0xD4C4,0xBF62,0xD4D4,0xBFC4,0xD462,0xD4D4,0xAFD4,0xBFC4,0xD4D4,0x62D4,0x529F,0x52AF,0x528F,0xD4D5,0x8F62,0xAF52,0xD4D5,0x8FD4,0x428F,0xD48F,0xD4E4,0x7F41,0x7F42,0x7F42,0x7F42,0x8F42,0x8F41,0x9F52,0x5252,0x428F,0x417F,0xE47F,0x8F62,0xE452,0xD552,0x638F,0x7393,0x5252,0x7373,0x5283,0x5263,0x5273,0x5252,0x73AF,0xBF62,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xC3D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xC3D4,0xE4E4,0xE4E4,0xE4E4,0xD3D4,0xD4E4,0xD3D4,0xE4E4,0xD4E4,0xE4E4,0xF4E4,0xE4E4,0xD4E4,0xE4F4,0xE4D4,0xD4E4,0xD4C3,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xAFD4,0x4646,0x465F,0x4642,0x465F,0x4646,0x464F,0x4642,0x4646,0x464F,0x4646,0x4246,0x4646,0x4446,0x5283,0x4642,0x4646,0x4646,0x4646,0x9344,0x8393,0x4643,0x4646,0x4646,0x9352,0x83C4,0x4373,0x8352,0x83B4,0x83C4,0x83C4,0x6373,0x4646,0x4646,0x9373,0x93B4,0x4643,0x4642,0x4646,0x465F,0x4F42,0x4246,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646
.word 0x417F,0x427F,0x7F7F,0x427F,0x5262,0x42AF,0x527F,0x627F,0x427F,0x419F,0x7F7F,0x5252,0x417F,0x427F,0x427F,0x417F,0x41C4,0x62D4,0xD4D4,0x62C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4BF,0xD4BF,0xBFD4,0xD583,0xD462,0xD4C4,0xD4D4,0xD562,0xBF62,0xC4D4,0x62D5,0xD4AF,0xD4D4,0x83D4,0xD462,0xD4D4,0x8FC4,0x8F52,0x8F42,0x8F7F,0x428F,0x427F,0x418F,0x427F,0x417F,0xE4D4,0xD4E4,0x8F41,0x7F41,0x7F41,0x8F41,0x7F41,0x4142,0x9F42,0x8F42,0x8F42,0x7F41,0xC442,0x41D4,0xE452,0xD48F,0x8F41,0x8342,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0xBF62,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4C3,0xE4D4,0xD3D4,0xE4D4,0xD4D4,0xE4C3,0xE4E4,0xE4E4,0xE4E4,0xE4C3,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4C3,0xD4E4,0xD4D4,0xD4D4,0xC3E4,0xD4D4,0xD4D4,0x83D4,0x4651,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4642,0x464F,0x4646,0x4646,0x7342,0x7393,0x4246,0x5F46,0x4646,0xB442,0x7393,0x4643,0x4642,0x4246,0x9373,0x83B4,0x7383,0x8383,0x9383,0x83C4,0xC493,0x8373,0x4246,0x5246,0x9393,0x73B4,0x4646,0x4646,0x4242,0x4646,0x4646,0x4F46,0x4646,0x4446,0x5F46,0x4246,0x4646,0x465F,0x4246
.word 0x7F41,0x8F52,0x7F7F,0x5241,0x52AF,0x9F52,0x4152,0xAF52,0x9F52,0x4262,0x418F,0xBF52,0x527F,0x417F,0x7F41,0x7F7F,0x8FD4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4BF,0xD4D4,0xD4C4,0xD462,0xBF62,0xAF62,0x62D4,0xC4BF,0xD4BF,0xC462,0xD462,0xD462,0xD4D4,0xE462,0xE4D4,0xBFD4,0xBF83,0x9FC4,0x8F52,0x5241,0xC462,0x418F,0x9F52,0x6283,0x418F,0x9FC4,0xE4D4,0xE4E4,0x8F62,0x7F42,0x7F42,0x7F42,0x7F42,0x8F8F,0x428F,0x528F,0x418F,0x427F,0x418F,0x8FE5,0xD442,0x6252,0x429F,0x838F,0x4283,0x7352,0x5283,0x7383,0x5252,0x62AF,0x62D5,0xD4D5,0xD4D4,0xD4D4,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D3,0xE4E4,0xE4E4,0xD4E4,0xE3E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D3,0xD4D4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xC3D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5262,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4F46,0x4646,0x4646,0x635F,0x4644,0x4246,0x8393,0x4663,0x4646,0x4646,0xB442,0xC483,0x4673,0x5F46,0x7346,0xC493,0x7383,0x9373,0x83B4,0x93B4,0x93B4,0x93B4,0x7373,0x4652,0x8342,0x93B4,0x6383,0x4246,0x4246,0x4683,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x465F,0x4246,0x4646
.word 0x527F,0x418F,0x527F,0x628F,0xC4AF,0x529F,0x52AF,0x62D5,0xD552,0xAF62,0x7F42,0x62C4,0x627F,0x7F7F,0x5252,0x4241,0x52D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD572,0xD4D4,0xD4D4,0xC4D4,0x629F,0x83D5,0xBFC4,0xD4D4,0xD4AF,0xC4AF,0xD4D4,0xD4D4,0xD562,0xD4D4,0xC4D4,0xD462,0x8F52,0xD552,0xD4D4,0x527F,0xD4AF,0xD5D4,0x7F8F,0x41D4,0xD4D4,0x8F41,0x62C4,0x7F8F,0x6241,0x42AF,0x417F,0x427F,0x417F,0x8F41,0x4252,0x419F,0x8F42,0x52E4,0x529F,0x529F,0x41D4,0x8F52,0x52D4,0x5252,0x6373,0x5273,0x9F52,0xBF73,0xBFC4,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xC3D4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xD4C3,0xD4C3,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0x62E4,0x4642,0x4646,0x4246,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x5F46,0x4646,0x6346,0x4252,0x4646,0x9373,0x4273,0x4246,0x5F46,0x8374,0x8383,0x4673,0x4646,0x9352,0x83C4,0x7383,0x73D4,0x7383,0xB493,0x8383,0xC483,0x7383,0x4673,0x9373,0xB493,0x4383,0x4246,0x8383,0x4663,0x4642,0x4642,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x4646,0x464F
.word 0x8F62,0x4142,0x9F42,0x6273,0xAF62,0xAF73,0xAF52,0xD562,0x83AF,0x62D5,0x418F,0xAFD4,0x737F,0x8F52,0x9F62,0x8F52,0xAFD4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xBFD4,0xD483,0x62D4,0xD4D4,0x62D4,0xD4AF,0xD462,0xD462,0x62D4,0x62D4,0x62BF,0xD4D4,0xD4D4,0xC451,0xD4D4,0xD4D4,0x42BF,0x629F,0xAFD4,0x5241,0xD562,0x5272,0x428F,0x7FD4,0x8F62,0xE462,0xE4E4,0x8F62,0x427F,0xBF62,0x8F42,0x7F42,0x7F41,0x8F42,0x418F,0x8F42,0x528F,0x52E4,0x528F,0x529F,0x418F,0x429F,0xD462,0x5273,0x5252,0x4252,0x629F,0x62D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xE4D4,0xE4D4,0xE4D4,0xE4C3,0xE4D4,0xC3D4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD3D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4C3,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xC3D4,0xE4E4,0xD4D4,0xD4D4,0x46AF,0x4642,0x4F46,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4446,0x4246,0x7373,0x4246,0x8344,0x7383,0x4646,0x4446,0x9373,0x93B4,0x4663,0x735F,0xD493,0x7383,0x9373,0x83C4,0xB483,0xD483,0xB483,0x9383,0x93C4,0x6373,0x9373,0x83C4,0x4383,0x8363,0x7383,0x4644,0x4F46,0x4646,0x4246,0x4646,0x4646,0x4F46,0x4F46,0x4646,0x465F,0x4646
.word 0x8F62,0x9F52,0x6252,0xD5AF,0xD562,0xC452,0xD562,0x62D4,0x9FD4,0x62D4,0x528F,0x62D4,0xD48F,0x42AF,0xC4AF,0x52D4,0x62D4,0xD4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xD4BF,0x62D4,0xD5D4,0xAF83,0xAF52,0xD4D4,0x62D4,0xD4D5,0xD4C4,0x83BF,0xC4BF,0xD4D4,0xAFD4,0xD462,0xD4D4,0x8F41,0x427F,0x8F8F,0x8F41,0x8F8F,0xAF52,0xD583,0x5241,0xD4D5,0xE4D4,0x42E4,0x8F41,0x428F,0x62D4,0x7F9F,0x8F42,0x427F,0x8F7F,0x8F42,0x8F42,0x62E4,0x628F,0xD4D4,0xD4D4,0xE4D4,0x8FD4,0xD4C4,0x5252,0x5252,0xAF62,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4C3,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xC3E4,0xD3D4,0xE4D4,0xD4E4,0xD4E4,0xE4E3,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x5283,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x8342,0x4673,0x7346,0x73B3,0x4646,0x735F,0xB483,0xB483,0x4643,0x9373,0x93C4,0x7373,0xB3C4,0x8373,0x93B4,0xB483,0x7393,0xB493,0xB483,0x8373,0xD493,0xC483,0x6383,0xC493,0x4383,0x4246,0x4646,0x4646,0x465F,0x4646,0x4642,0x4646,0x4646,0x4642,0x4646,0x4642
.word 0xAF62,0x8FC4,0xAF62,0xAF73,0xAFC4,0xBF62,0xAFC4,0xBFC4,0xC462,0xD4D4,0xAF41,0x62C4,0xD48F,0x7F62,0xD462,0xAFD4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xD3D4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4C4,0xD4D4,0xD4BF,0xD4D4,0x62D4,0x628F,0xD4D4,0xD4AF,0xD462,0xD562,0xD462,0xD4BF,0xD4D4,0xD4D4,0xAF62,0x8F73,0x8F52,0x6252,0x62D5,0x528F,0xD562,0x6241,0x8FBF,0x6252,0xD4E4,0xD562,0x41D4,0x8F7F,0xC47F,0x62D4,0x428F,0x418F,0x427F,0x417F,0x527F,0x52E4,0xD49F,0x9FD4,0xE4E4,0x8F52,0xD4C4,0x51D4,0x52D4,0xAF41,0xD573,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xC3D4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D3,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4C3,0xE4D4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xAFD4,0x4642,0x4246,0x4646,0x4646,0x5F46,0x5F46,0x5F46,0x4F46,0x4646,0x4646,0x7346,0x5293,0x7346,0x73D4,0x4642,0x7343,0x8393,0x8393,0x6243,0xB383,0x83B4,0x83B4,0xB483,0x8373,0xD483,0x7383,0x7383,0x83C4,0x8383,0x8373,0x83C4,0x83B4,0xB483,0xB493,0x4673,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4246,0x4646,0x464F,0x465F,0x464F
.word 0x6262,0x73AF,0xC4AF,0x62D5,0x62D4,0x62D5,0xC4D4,0xD562,0xBFD4,0xD4C4,0xD452,0xD4D4,0xD4C4,0x8F62,0xD4D5,0xD4D4,0xE4D4,0xE4D4,0xD3D4,0xE4D4,0xE4E4,0xE4F4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD562,0xD4D4,0x419F,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x62C4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0xC4AF,0xD4D4,0x527F,0xD4D5,0x42BF,0x62C4,0x8FD5,0x6241,0xD48F,0xE4E4,0x8F83,0x7F41,0x428F,0xD462,0x8F62,0x7F42,0x7F41,0x4142,0x42E4,0x5241,0x41D4,0xD462,0x4141,0x418F,0xD5D4,0xD4D4,0xAFC4,0xD462,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4C3,0xE4E4,0xD4E4,0xD4E4,0xC3E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xD4D3,0xE4D4,0xD4E4,0xC3D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4C3,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x4662,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x7346,0x7393,0x8363,0x7393,0x4373,0x8373,0x83B4,0x73B4,0x9373,0xB4B3,0xB493,0x8383,0x8393,0xB483,0x8373,0x83B4,0x8383,0x93C4,0x83D4,0x93B4,0x8383,0xB483,0x8383,0x83B4,0x4644,0x4642,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646
.word 0xC4AF,0xAF62,0x62C4,0xC4BF,0xC4AF,0xD4BF,0xBF62,0x62D4,0x83D4,0xD4BF,0xD4D4,0xD4D4,0xAFD4,0x42D4,0xD4AF,0xD4D4,0xD4D4,0xD4C3,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4F4,0xE4E3,0xE3E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xBFD4,0xD483,0xD4C4,0x8F62,0xBF62,0xD562,0xD4BF,0xAFD4,0xBFC4,0xD562,0xD462,0xD4D4,0x8F62,0xAF52,0xD4D4,0x528F,0xD4AF,0x9F83,0x628F,0x62AF,0x8FD5,0x417F,0xD452,0x62E4,0x427F,0x8F7F,0x418F,0xD4D5,0x8F62,0x8F42,0xD552,0x8FE4,0xAF42,0x8F62,0xD452,0x429F,0x4241,0xD48F,0x62D4,0xD4D4,0xD4D5,0xD4D4,0xD4D4,0xD4E4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xD4E4,0xD4E4,0xE4D4,0xE4C3,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD3D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4F4,0xD4E4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4C3,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x42D4,0x4642,0x4646,0x4646,0x4642,0x4446,0x5F46,0x4646,0x4F46,0x4646,0x5246,0x8393,0xB483,0x7383,0x7363,0x9383,0x83C4,0x7383,0xD483,0x8383,0x83B4,0xB483,0xB493,0x7383,0x8383,0xB483,0xB483,0xC483,0xB493,0xB483,0xB493,0x8373,0xC493,0x6383,0x4646,0x5F46,0x4246,0x4642,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x5F46
.word 0xAF62,0x62C4,0x62D5,0x62D5,0x62D4,0xC4D4,0xC4D4,0xD4D4,0xD4BF,0xD4C4,0xD462,0xD4D4,0xD4D4,0xD462,0xD462,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4F4,0xE3E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFD4,0xD562,0xD4D4,0x62D4,0x529F,0xD462,0xD4D4,0xD4D4,0xBF62,0xBFC4,0xAFC4,0x62C4,0x428F,0x417F,0x417F,0x527F,0x429F,0x428F,0x8F52,0x5252,0xAF62,0xAF62,0xD573,0xBF62,0x6283,0x72D5,0xD4D4,0xD4D4,0x7F8F,0x838F,0x41D4,0xD49F,0x9F52,0xE442,0x9F42,0x9F42,0x4141,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D3,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4C3,0xE4E4,0xD4E4,0xE4E4,0xC3E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD472,0xD4D4,0xC3D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0x51D4,0x4646,0x4646,0x465F,0x5F46,0x4F46,0x4646,0x4246,0x4646,0x4646,0x8342,0x83B4,0x8383,0x73B4,0x8363,0xC493,0x7383,0xB483,0xB4B3,0xB493,0x8383,0xC493,0xB493,0x8373,0x83C4,0x8373,0x8383,0xB483,0x7383,0x8383,0x83C4,0xB483,0x8383,0x4683,0x4242,0x7373,0x4663,0x4646,0x4644,0x4642,0x5F46,0x5F46,0x4646,0x4642,0x4F46,0x4646
.word 0xD562,0xBF62,0xAFC4,0xBFC4,0xBFC4,0xAFC4,0xBFD4,0xD462,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E3,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xF4F4,0xE4E4,0xE3F4,0xE4F4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4BF,0xD4D4,0xD4C4,0x42AF,0xD49F,0xD462,0xD4D4,0xD462,0xD462,0x62D5,0x41D5,0x6252,0x428F,0x6252,0xC4D4,0x628F,0xC4BF,0x52AF,0x428F,0x428F,0x418F,0x428F,0x418F,0x8F8F,0x8F41,0x8F41,0x6242,0x9F52,0x8F62,0xE452,0x429F,0x418F,0xC49F,0x52D4,0x429F,0xD4E4,0xD4D4,0xAFD4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4C3,0xD4E4,0xD4E4,0xE4D4,0xC3D4,0xD4D4,0xD4E4,0xE4E4,0xC3D4,0xE4D4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4446,0x5246,0x4646,0x4646,0xB45F,0xB493,0x9383,0x7383,0x8373,0x83D4,0x83B4,0x9373,0x83C4,0x9373,0x83C4,0x83B4,0x73B4,0x8373,0xB483,0xB483,0xB483,0x83B3,0x73B4,0xB493,0xB493,0x8383,0x73B4,0x4663,0x7343,0x6393,0x4646,0x4F42,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xAFC4,0x62C4,0x72D5,0xD562,0xD572,0xD462,0xD4C4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD3D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE3E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xF4F4,0xF4E3,0xE4E4,0xD4E4,0xD3D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFC4,0xAF83,0xD4C4,0xD4C4,0xD483,0x42AF,0xC4AF,0xD4D4,0x62D4,0xC4BF,0xC4BF,0x629F,0x528F,0xD5AF,0x62D4,0x528F,0xD4D5,0x9F62,0x8F52,0x8F42,0x5241,0x429F,0x6241,0x8F42,0x7F42,0x7F42,0x7F8F,0xD552,0x8F41,0xE442,0x417F,0x8FC4,0xD47F,0x9F62,0x8F52,0xD4D4,0xAF62,0xD4C4,0xE462,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4D4,0xD3D4,0xE3E4,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD472,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x72D4,0x9FD4,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x5246,0x4683,0x4642,0x9363,0x7393,0xB4C4,0x8373,0x8383,0x93B4,0x7383,0xC483,0xB493,0xC483,0xB483,0x8383,0x7393,0x7373,0x8383,0x7383,0x8373,0xB3B4,0x8383,0x83B4,0x8383,0x8373,0x7383,0x6263,0x7393,0x4643,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4246,0x4F46,0x4646,0x465F,0x4642
.word 0xD462,0xC4BF,0xC4BF,0xC4BF,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E3,0xF4D3,0xE4E4,0xE4E4,0xE4D3,0xE4F4,0xE4E4,0xE4E4,0xF4F4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4D3,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x83AF,0xC4BF,0xC4BF,0xD4D4,0x51D4,0x529F,0xD4D4,0xC4D4,0xD462,0x9F62,0x8F7F,0x5142,0x9FD5,0x8F52,0x417F,0x427F,0x528F,0x7F8F,0xD552,0x7FD4,0xE662,0x418F,0x417F,0x417F,0x8F42,0x417F,0x6262,0x628F,0xC4D4,0x62E4,0x418F,0x429F,0x429F,0xD4D4,0x41D4,0xD4D4,0x62D4,0xD4E4,0xD4C3,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xC3E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xD3E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4BF,0x52D4,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4346,0x7383,0x4646,0xB452,0x83B3,0x8383,0x7373,0x93C4,0xB483,0x7373,0x93D4,0x7373,0xB483,0x7393,0xB483,0x7383,0x6352,0x7373,0xB483,0x9373,0x7373,0xC473,0xB493,0x7383,0x93C4,0x63B4,0x8373,0x43C4,0x4246,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x464F
.word 0x62D4,0x62D5,0x62C4,0x72D5,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4C3,0xE4E4,0xC3D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E3,0xF4E4,0xF4E4,0xF4F4,0xE4E3,0xE3E4,0xF4E4,0xF4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E3,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD562,0xD462,0xD462,0xD4D4,0xD4D4,0xD4D4,0x529F,0xC4AF,0xC4D4,0x52D4,0x528F,0x6262,0x428F,0xD562,0x8F41,0xD562,0x4162,0xD4D5,0x7FE4,0x418F,0xD442,0x8F52,0x7F42,0x417F,0x8F7F,0x8F7F,0xAF73,0xAFE4,0x41E4,0x527F,0x428F,0x9F41,0xE4D4,0x52D4,0xD49F,0xD4D4,0xD462,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xC3E4,0xD4D4,0xD4C3,0xE4D4,0xC3D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xE4F4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4C3,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x83D4,0x9FD4,0x4646,0x4646,0x465F,0x4642,0x4452,0x4646,0x4246,0x9373,0x7373,0x9373,0xB483,0xB493,0x8383,0xC4B3,0x83B3,0x8383,0x93D4,0x83B4,0x7383,0x7383,0x9383,0x73B4,0x7373,0x8352,0x7373,0xB483,0x6383,0xB493,0x8383,0x9373,0x83D4,0x8373,0xC493,0x4663,0x4646,0x4646,0x4346,0x5273,0x4246,0x4646,0x4646,0x4646,0x5F44,0x4646,0x4646
.word 0xD562,0xD462,0xD4BF,0xD4D4,0xD4C4,0x62D4,0xBFD4,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD3E4,0xF4E4,0xE4E4,0xE3E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xF4E4,0xF4E4,0xF4F4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4C3,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x62BF,0xAFD5,0xD562,0xD462,0xD4D4,0xD4D4,0x62D4,0x8F8F,0xD462,0x529F,0xD4BF,0xAF41,0xD462,0x527F,0xC49F,0x7FD4,0x6251,0x41D4,0x527F,0x7F8F,0x8F41,0x8F52,0x7F42,0x7F42,0x7F42,0x528F,0x62E4,0x8F62,0x8F73,0x9F42,0x4142,0xD4E4,0xAFD4,0x6262,0xD4D4,0xBFD4,0xD4C4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD3E4,0xE3D4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4646,0x4644,0x4646,0x465F,0x5246,0x5282,0x4646,0x8373,0x8383,0x9373,0x93B4,0x73C4,0x8373,0xB493,0x83D4,0x83B4,0x93B4,0x83B4,0x83B4,0x73B4,0xB493,0x7383,0x7352,0x5273,0x7383,0x7383,0x8373,0x93C4,0x83B4,0x83B4,0x73B4,0x7383,0x93C4,0x4644,0x4642,0x635F,0x7362,0x4643,0x4F46,0x4646,0x4646,0x4642,0x4646,0x5F46,0x4246
.word 0xBFD4,0xD4C4,0x62D4,0xD4D4,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4E3,0xE4F4,0xE4F4,0xE3E4,0xF4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4D4,0xE4F4,0xE4F4,0xE4E3,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFD4,0xBFC4,0xD462,0xD462,0xD4D4,0xD4D4,0xC4D4,0x62D4,0x528F,0x6252,0xD4D5,0x527F,0xAF62,0x9F52,0x528F,0x62D5,0x628F,0xD4D5,0x7FC4,0x8F62,0x8F62,0x7F41,0x7F41,0x7F42,0x8F62,0xAF52,0x62D4,0x428F,0xD49F,0x41D4,0xE4D4,0xE4E4,0xC4AF,0x62D4,0x62D4,0xD4E4,0xD4C3,0xD4D4,0xD4C3,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD3E4,0xE4D4,0xE4E4,0xD4E4,0xD4D3,0xE4E4,0xD4C3,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xC3D4,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x9FD4,0x4646,0x465F,0x4646,0x4646,0x4246,0x9344,0x8373,0x7346,0x83A3,0x93B4,0x9373,0x83D4,0x8373,0x83C4,0xB483,0x9383,0x83C4,0x8373,0x8373,0x7393,0x83B4,0x7373,0x7373,0x7373,0x7363,0x8373,0x8373,0x7383,0x8373,0x8383,0x7373,0xB373,0x73C4,0x4646,0x4246,0x83B4,0x4663,0x4246,0x4646,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x4646
.word 0xD4C4,0xBF62,0xD4C4,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4C3,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE3E4,0xF4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E3,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD472,0xD472,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD562,0x62BF,0x83BF,0x62D4,0xD4D4,0xD4BF,0x9FD4,0x8F8F,0x417F,0x8F7F,0x528F,0x5262,0x628F,0xD4D4,0x528F,0x9F62,0x8F41,0x8F42,0x5262,0x428F,0x417F,0x417F,0x8F7F,0x8F42,0xE4D4,0x8F41,0xE442,0x41D4,0x4142,0x429F,0xD441,0x9FD4,0xD4D4,0xD4C4,0xE4D4,0xD4D4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xC3E4,0xD4E4,0xD4C3,0xD4E4,0xE4D3,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4C3,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD472,0xD472,0xBFD4,0x52D4,0x4246,0x4646,0x465F,0x4642,0x4646,0x4246,0x8393,0x7393,0x8393,0xC483,0xC473,0x7383,0xB483,0xB483,0x8383,0xB383,0xB483,0x8373,0x7373,0x73D4,0x8373,0x7373,0x5273,0x8373,0x8373,0x7383,0xC483,0x7373,0xB483,0xB4B3,0x7383,0x8383,0x4383,0x4246,0x8352,0x6383,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F44,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E3,0xE3E4,0xF4E4,0xE3E4,0xF4E4,0xE4E4,0xF4F4,0xE4E3,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4C3,0x72D4,0xBF62,0xBFC4,0xAFC4,0xBF83,0xD483,0xD4D4,0xD4D4,0xD472,0xD4D4,0xD4D4,0xD562,0xD562,0xD4AF,0xD562,0xC462,0xD562,0x52C4,0xD541,0x4162,0x8F52,0x528F,0x628F,0x62D5,0x8F8F,0x528F,0xD562,0x4251,0xD4D5,0x527F,0x8FD4,0x7F42,0x7F42,0x8F41,0x7F42,0x6242,0xE4D4,0x8FD4,0x8F52,0x8F42,0x8F42,0x5241,0xD48F,0xD451,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xD4C3,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4D4,0xD4D4,0xD4D4,0x9FC4,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4642,0x7346,0x8373,0xC4B4,0x9373,0x9373,0x83C4,0x7383,0x8383,0x83B4,0x93B4,0x8383,0xB483,0x9393,0x83C4,0x7373,0x9373,0x7373,0x7393,0x7393,0xB483,0x7383,0x7373,0xC483,0x8383,0x7383,0xD4D4,0x4373,0x8362,0x7383,0x4643,0x4646,0x4646,0x4646,0x4246,0x4646,0x4642,0x465F,0x4646,0x465F
.word 0x62D4,0xC4BF,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD3D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x9F62,0x8F42,0x5242,0xD4D4,0xD562,0x62D4,0x72D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD562,0xD4AF,0xBF62,0x62C4,0xC4BF,0x419F,0x8FD5,0xD562,0x9FD4,0x8F42,0x528F,0x7FD4,0xAF42,0xD452,0x8FAF,0x8F62,0x8F41,0x5241,0x629F,0x428F,0x7FD4,0x417F,0x628F,0x41E5,0x527F,0x428F,0x418F,0x428F,0x418F,0x8F42,0xD462,0xBF62,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4C3,0xD3D4,0xE4D4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xE4C3,0xE4E4,0xD4E4,0xE4C3,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xC3E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD472,0x5262,0x4646,0x465F,0x4246,0x4646,0x465F,0x5F46,0x4246,0x7383,0x9373,0xD473,0xB483,0xB483,0x7383,0xB483,0x8383,0xC483,0xB493,0x8383,0x83C4,0x83B4,0x8383,0xC483,0x7373,0x7373,0x73C4,0x7383,0x7373,0x8383,0x83C4,0xB473,0x8373,0x8383,0x7373,0xC483,0x4373,0x4646,0x4642,0x465F,0x4642,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646
.word 0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xC3E4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E3,0xF4E4,0xE4E4,0xD4E3,0xE4F4,0xE3E4,0xF4E4,0xE4E4,0xD4E4,0xE4D3,0xE4D4,0xD4E4,0xC3D4,0xD4D4,0xD4D4,0x62D4,0x4151,0x9F41,0x9F42,0x9F52,0x9F52,0x9F41,0xBF52,0xBF83,0xBFC4,0xD4C4,0xD4AF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD562,0xC4BF,0x8FD4,0x6283,0x8F62,0xAF42,0x8F52,0x4241,0x8F41,0x527F,0x428F,0x628F,0x8F62,0x9F42,0x62C4,0xD49F,0x428F,0x62AF,0x6262,0x428F,0x8F7F,0x8F42,0x7F41,0xAF62,0xD552,0x52D4,0x8F8F,0x5241,0x8F8F,0xD4C4,0x62D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4C3,0xD4E4,0xE4F4,0xE4E4,0xE4D3,0xE4E4,0xE4D4,0xD3D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4C4,0x43AF,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x8352,0x8383,0x8383,0x7383,0x8383,0x7373,0x8383,0xB483,0x9383,0x83B3,0x73C4,0x8383,0xB483,0xC493,0x7393,0x8373,0x7373,0x7383,0x8373,0x8373,0x73B4,0x8373,0x8383,0xB483,0xB483,0x9383,0x83D4,0x4663,0x4642,0x464F,0x4646,0x4646,0x464F,0x4646,0x4646,0x465F,0x4646,0x465F
.word 0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC3E4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4D3,0xE4E3,0xF4E4,0xE3E4,0xF4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4C3,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x62D4,0xD462,0x9F41,0x9F52,0xAF63,0xAF73,0x5273,0x52AF,0x8F52,0x5252,0x62D5,0x62D5,0xC4BF,0xBF83,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x62D4,0x529F,0xBFD5,0xD5C4,0x8F51,0x5241,0x8F62,0xC452,0x41D4,0x418F,0xAF52,0x7F42,0x4241,0x42AF,0x428F,0xD452,0x8F7F,0xF662,0x41D4,0x7F42,0x427F,0x528F,0x429F,0x517F,0xD4C4,0x8F52,0xD47F,0x527F,0x62AF,0xD4E4,0xD462,0xD4D4,0xE4D4,0xE4D4,0xD4C3,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xC3D4,0xE4D4,0xE4D4,0xD4D4,0xD3D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4C3,0xD4D4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xC4D4,0xD4D4,0x4262,0x4646,0x4642,0x465F,0x465F,0x465F,0x465F,0x5F46,0xB444,0xB483,0xB483,0xB483,0xB483,0x8373,0x83B4,0x93C4,0xC4B4,0xB483,0x8383,0x83B4,0x7373,0x8373,0x6373,0x8373,0x83C3,0x7373,0x7373,0x7383,0x7373,0xB483,0x7393,0xD4B3,0x7383,0xB483,0x6393,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x465F,0x4646,0x464F,0x4646
.word 0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xE4D3,0xD4E4,0xE4D3,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xF4E4,0xE3E4,0xE4E4,0xE4F4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xBF62,0x9F52,0x5252,0x73AF,0x73AF,0x73AF,0x52AF,0xAF52,0x9F52,0x9F52,0x9F41,0xD462,0x62D4,0x62D5,0xAFD4,0x72D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0x52D4,0xD4AF,0x62D4,0x62D4,0x62D5,0x8F9F,0x8FC4,0x529F,0xD4D4,0x8F7F,0x6242,0x7F8F,0xE4D4,0x8F62,0x8F62,0xD462,0x527F,0x528F,0xE4E4,0xD4E4,0x5241,0xD462,0xC4E4,0x8F62,0x517F,0x739F,0xD4E4,0x7F8F,0x8F42,0x8F41,0xAF62,0xD483,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xC3D4,0xD4E4,0xD4E4,0xE4E4,0xE4D3,0xE4E3,0xE4D4,0xD3D4,0xD4E4,0xE4D4,0xD4E4,0x62E4,0xD4E4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4C3,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xBFD4,0x4362,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7346,0x73B3,0x8383,0x7383,0x8373,0x7383,0x8373,0x93B4,0x8373,0x8383,0x93C4,0xB483,0x7383,0x6373,0x7362,0x8383,0x7373,0x83D4,0x7373,0x7373,0x8383,0xC493,0x8373,0x9383,0x73B4,0x9383,0x4673,0x4644,0x5242,0x4652,0x4642,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4246
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4F4,0xE4E4,0xE4E3,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD472,0xAF62,0xAF73,0xD562,0xD562,0xD562,0xAF62,0xAF73,0x5273,0x52AF,0x429F,0x9F52,0x9F42,0xD462,0xC4BF,0xD462,0xD562,0xD4BF,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0x8FD4,0x6252,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x428F,0x9F41,0x7F7F,0x8F52,0xD552,0xD4D4,0x7FC4,0x7F8F,0xAF62,0xAF42,0x9F73,0x8F62,0x8F62,0x8F42,0xD541,0xAFE4,0x9F52,0x9F52,0xD442,0x7FD4,0x8F62,0x7F42,0x7F41,0x8F42,0x9F42,0xAF62,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4C3,0xE4D4,0xE4E4,0xE4E4,0xF4D4,0xE4E4,0xF4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D3,0xE4D4,0xD4C3,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD472,0xC4D4,0x46AF,0x4646,0x465F,0x4646,0x4644,0x4246,0x4246,0x4646,0x735F,0x73D4,0xB483,0x8383,0xB483,0x8383,0x8373,0xC483,0x7383,0x93C4,0x83C4,0x8383,0x83B4,0x5273,0x7373,0x8373,0x73C3,0x7383,0x7383,0x8373,0x83C4,0x73B4,0xB483,0x83B3,0x8373,0xB4C4,0x4663,0x5242,0x6383,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xD3D4,0xD4E4,0xE4C3,0xD4D4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4D3,0xE4E4,0xE4E4,0xE4E4,0xE4C3,0xE4E4,0xE4E4,0xC3E4,0xD4D4,0xD4D4,0xAF62,0x62C4,0xC4AF,0xAFD5,0xBFC4,0xAF62,0x62C4,0x73AF,0x52AF,0x9F52,0x9F52,0x8F52,0x9F52,0x8F52,0xD562,0xBFD4,0x62C4,0x72D4,0xD462,0x72D4,0xD4D4,0xD4D4,0x52D4,0x52AF,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x428F,0x6252,0x52D4,0x628F,0x8F62,0x8F52,0xC452,0x7FD4,0x8F42,0x8F41,0x8F42,0x627F,0x6262,0x62E5,0x529F,0x528F,0x62D5,0x42AF,0x7F7F,0x417F,0x417F,0x8F52,0x6262,0xD48F,0xC4D4,0xAF62,0xC3C4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD3E4,0xF4E4,0xE4E4,0xF4E4,0xE4D4,0xE4D4,0xE4D4,0xD4C3,0x62D4,0xC4AF,0xF4D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4E4,0xC3D4,0xE4D4,0xD4D4,0x62D4,0x4662,0x4646,0x4646,0x4F42,0x5F46,0x4F46,0x4646,0x4646,0x5246,0x8383,0x83C4,0x73B4,0x7383,0xB483,0x7373,0xB483,0x7383,0x7373,0xC483,0xB493,0x7373,0x5273,0x8373,0x83C4,0x7383,0x7383,0x7373,0x8383,0x83B4,0x8383,0xB3D4,0x7383,0xC483,0x4383,0x6263,0x8393,0x4643,0x4642,0x465F,0x465F,0x4642,0x465F,0x4646,0x465F,0x465F,0x465F
.word 0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD3D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D3,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xAF72,0x62C4,0xD5AF,0xAF83,0x72C4,0xD562,0x6262,0x73AF,0xAF62,0xAF73,0x5252,0x52AF,0x529F,0x8F52,0x9F52,0x8F52,0xC483,0xC4BF,0xD5BF,0xAFD4,0xD4D4,0xD4D4,0xD4D4,0x51D4,0x9F52,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xAFD4,0x8F8F,0xBF52,0x417F,0x528F,0x7FD4,0xAF52,0x62D4,0x428F,0xD552,0x527F,0xC48F,0xE4BF,0x9FD4,0x8F42,0x527F,0x52AF,0x628F,0x7F41,0xE442,0xD47F,0x428F,0xD4D4,0x428F,0xBFD4,0x62D4,0xD462,0xE4D4,0xC3D4,0xD4D4,0xC3D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4C3,0xD4E4,0xD4C3,0xD4E4,0x41D4,0xE462,0xE4E4,0xE4E4,0xD3E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x46AF,0x4642,0x465F,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x6346,0x9383,0x8373,0x8383,0x8373,0x8383,0x6383,0x8383,0xB493,0x8383,0x8363,0x8383,0x5273,0x5273,0x8383,0x8373,0x7373,0xB483,0x8373,0x8373,0xB483,0x9373,0xB4C4,0xB483,0xB493,0x7373,0x8383,0x43B4,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4C3,0xD4D4,0x62D4,0x62D5,0x62D5,0xD4D4,0xD5D4,0xC4BF,0xC4AF,0xBF62,0xAF73,0x5273,0x52AF,0x9F52,0x9F52,0x9F42,0x4252,0x429F,0x529F,0xD4C4,0xD462,0xD462,0xBF62,0xD4C4,0xD4D4,0x9FD4,0x5252,0xE462,0xD4D4,0xC3D4,0xD4D4,0x51D4,0x4142,0x6273,0x429F,0x8F52,0xAF52,0xD4C4,0x417F,0xC48F,0x419F,0xC49F,0x8F62,0x62D4,0x62D5,0xD4D5,0x62D4,0x9F62,0x8F52,0x8F52,0x6242,0xD4D5,0x62D4,0xAF62,0xE4D4,0x418F,0xD452,0xD452,0x62BF,0xD462,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D3,0xD4E4,0xD4D3,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xAF62,0xD4D4,0xE4E4,0xD3E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4662,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4446,0xB473,0xB483,0x7393,0x7383,0xC473,0x8393,0x7373,0x8383,0x73B4,0x7373,0xB483,0x7383,0x7352,0x8373,0x7363,0x7373,0x7393,0x8383,0x73B4,0x8373,0xD4D4,0x8383,0x93D4,0x73B4,0x9383,0x73C4,0x4343,0x6252,0x4642,0x4646,0x4642,0x4646,0x5F46,0x4246,0x5F46,0x4446,0x5F46
.word 0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xD3E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xC3E4,0xD4E4,0xD4D4,0xD4D4,0xD562,0xBFD4,0xBF83,0xD4D4,0x62D4,0x83D4,0xD4D4,0xC4AF,0x62D5,0x73AF,0x73AF,0x739F,0x529F,0x529F,0x529F,0x529F,0x529F,0x529F,0xD4BF,0x62D5,0x83D5,0xAFD4,0xD4D4,0x52D4,0x9F9F,0xD452,0xD4D4,0xD4D4,0x62D4,0x628F,0x52AF,0xD58F,0x8F62,0x8F52,0x5242,0x62AF,0x427F,0xD541,0x7F62,0x7F42,0x9F62,0xAF52,0x8FC4,0x4141,0xE562,0xAFC4,0x5273,0x429F,0xD4BF,0x62D4,0x8F8F,0xD4D4,0x417F,0x8F42,0x52E4,0xD4D4,0xD5D4,0x62BF,0xD4D5,0xD4E4,0xD4D4,0xE4D4,0xF4C3,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0x62D4,0x739F,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0x62D4,0x46AF,0x5F46,0x4646,0x4642,0x465F,0x465F,0x4646,0x4446,0x5F46,0x9373,0x83B4,0x83B4,0x83B4,0x8373,0x93B4,0x73C4,0x8373,0x9383,0x8373,0x5273,0x7373,0x8373,0x8373,0x8373,0x7383,0x83C4,0x73B4,0x7373,0x8383,0x8383,0xA3B4,0xC493,0x8383,0xC483,0x6383,0x8383,0x4373,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646
.word 0xD4D4,0xD4D4,0xD4C3,0xE4C3,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D3,0xE4D4,0xD3D4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4C4,0xAF62,0xD562,0xAF62,0x9F73,0x9F73,0x5252,0x429F,0x5241,0x529F,0x9F52,0x9F52,0x5252,0xC4BF,0xC4BF,0xC4BF,0xD462,0x8FD4,0x5252,0x529F,0xD4D4,0xD4D4,0x8FD4,0xAF62,0xD573,0x8F51,0x9F42,0xD4C4,0xD48F,0xE4D4,0xE4E4,0x8FD4,0xD4D4,0x417F,0x429F,0x628F,0x429F,0x8F7F,0x7F41,0xD462,0x8FD4,0x6241,0x8F52,0xAF52,0xD442,0xE462,0x7F62,0xE452,0xD4D4,0xD4D4,0xD441,0xC47F,0x627F,0xD4D4,0xE4E4,0xD4D4,0xE4D4,0xE4E3,0xF4E4,0xE4E3,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0x62D4,0x62D5,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xC3D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD472,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x62D4,0x4662,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x7344,0x9383,0x7383,0x8383,0x73B4,0x8383,0x83D4,0x7373,0x7373,0xB483,0x7393,0x5273,0x7373,0xC473,0x7383,0x6373,0x7383,0x7373,0x8373,0x93B4,0x9373,0x83C4,0x83C4,0x93B4,0x73B4,0xD483,0x6373,0x4646,0x4642,0x4646,0x465F,0x465F,0x4642,0x4646,0x465F,0x4646,0x4246
.word 0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD3D4,0xE4D4,0xE4E4,0xE4D4,0xD3E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E3,0xD3E4,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xD4E4,0xD4C3,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xBFD4,0xD4C4,0xBF62,0x6273,0x73AF,0x529F,0x529F,0x529F,0x429F,0x9F52,0xAF52,0x5252,0x529F,0x6241,0x62D5,0x62D4,0xBFD4,0x62C4,0x52AF,0x529F,0xD4BF,0xD4D4,0x62D4,0x63AF,0x9FE4,0x8F42,0x4241,0x528F,0x52E4,0x418F,0xD452,0xD4E4,0xD4E4,0x8F52,0x9F62,0x5252,0x8F62,0xD462,0xD4D4,0xD4D4,0x8F62,0x8F52,0x8F52,0x428F,0xD47F,0xD4AF,0xE4D4,0xD4D4,0x8F7F,0xD452,0x7FD4,0xAF62,0x8FD4,0xD462,0xC3D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0x72D4,0x52AF,0xD4AF,0xD4E4,0xE4E4,0xE4E4,0xD3E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x72D4,0xD4E5,0xD4D4,0xD4C3,0xE4D4,0xD4D4,0xAFD4,0x4662,0x4246,0x4646,0x5F46,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4342,0x83C4,0x93B4,0x73C4,0xB493,0x7373,0xC483,0x6383,0x7373,0x7383,0x7373,0x7352,0x8383,0x7383,0xC483,0x5283,0x7383,0x7373,0x8383,0xB483,0xD493,0xC493,0x8383,0xB483,0x8383,0x8393,0x4663,0x4642,0x465F,0x465F,0x4646,0x4646,0x464F,0x465F,0x4646,0x4644,0x4646
.word 0xD4D4,0xC3D4,0xE4D4,0xE4D4,0xC3D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xD4E4,0xC3E4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x62D5,0x73AF,0x73AF,0x73AF,0x529F,0x9F52,0x9F52,0x9F52,0x5252,0x52AF,0x529F,0x429F,0x529F,0xD4AF,0xD462,0xD4D4,0x9F52,0x9F52,0x6252,0xD4D4,0x52D4,0xAF62,0x6252,0x428F,0x418F,0x9FD4,0x8F41,0x7F42,0x7F8F,0x5241,0x8FE4,0xE4D4,0xD4E4,0x9FE4,0xC48F,0xD4D4,0x9F62,0x417F,0x9FD4,0xD47F,0x7FE4,0x5241,0x7F8F,0xD442,0xAF62,0x6242,0x7F7F,0x7F7F,0x7F7F,0x427F,0x527F,0x629F,0xC4AF,0xD4E4,0xE4E4,0xE4F4,0xE3E4,0xE4D4,0xE4E4,0xD4C3,0xD4D4,0x62D4,0x62D5,0xD4D5,0xE4D4,0xF4E3,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0x72D4,0xC4D4,0x46AF,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4F42,0x4646,0x9352,0x93B4,0x9373,0x8373,0x7383,0x8373,0x8383,0x7383,0x7373,0x7383,0x5273,0x7383,0x83B4,0x7383,0x8373,0x73B4,0x9383,0x73B4,0x8383,0x83B4,0x83B4,0x83B4,0x8373,0xB483,0x63C4,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F44,0x4646,0x4642
.word 0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xD3D4,0xE4E4,0xE4C3,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xC4BF,0xAF62,0x62C4,0x73AF,0x52AF,0x529F,0x529F,0x629F,0x52AF,0xAF52,0x5252,0x529F,0x528F,0x6241,0x62D5,0xBFD4,0x42D4,0x529F,0x429F,0xD4BF,0x8FD4,0x5262,0xD58F,0x4141,0xD442,0x8F52,0x7F42,0x428F,0x427F,0x8F8F,0x7F41,0xE452,0xD4D4,0x4241,0x627F,0xBFD4,0xE442,0x8FE4,0x7F42,0xE442,0x7FD4,0x8F7F,0x7F7F,0x7F7F,0xC441,0x7FAF,0x7F7F,0x7F41,0x4241,0x6241,0x418F,0xAF42,0x6273,0xE4AF,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xBFD4,0xAF52,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFD4,0x4662,0x5F46,0x4646,0x4646,0x465F,0x465F,0x4642,0x4646,0x4646,0x4246,0x8344,0xB483,0x7393,0x73C4,0x8383,0x7373,0x7373,0x7383,0x8373,0x7373,0x7373,0xD473,0x8383,0x7363,0x8352,0x8373,0x7373,0x7383,0x83C4,0x8383,0x8393,0xB483,0x7383,0x8393,0x4673,0x4642,0x465F,0x4646,0x4646,0x464F,0x465F,0x4644,0x5F46,0x4646,0x4F46,0x4646
.word 0xD4D4,0xD4E4,0xD4D4,0xD4C3,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE3E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4C3,0xE4D4,0xC3D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x72D4,0xD4D4,0xD4D4,0xD562,0x62D4,0x73AF,0xAF62,0x9F73,0x5252,0x529F,0x9F52,0x9F52,0x5252,0x529F,0x528F,0x429F,0x8F8F,0x528F,0x62D5,0xD4D4,0x5241,0x529F,0x5252,0x62D4,0x629F,0x429F,0xAF62,0xD48F,0x8F62,0x8F41,0x8F42,0x4141,0x5242,0x5252,0xC4BF,0xD4D4,0x41E4,0xE562,0xD4E4,0xC4D4,0x5262,0x528F,0x628F,0x62E4,0x417F,0x427F,0x6241,0x7F7F,0xD442,0x7F62,0x427F,0x8F7F,0xAF62,0xD542,0xC47F,0x8F8F,0x6273,0xE462,0xE4E4,0xD3D4,0xD4E4,0xE4E4,0xE4D4,0x62D4,0xC4D4,0x6241,0xD4D4,0xE4E4,0xE4E3,0xE4E4,0xD4D3,0xD4E4,0xD4C3,0xC3D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4C3,0xD4D4,0x6262,0x46AF,0x4646,0x5F46,0x4646,0x4646,0x4646,0x464F,0x5F46,0x4646,0x4646,0x4246,0xC483,0x8373,0x7373,0xC493,0x7383,0xB483,0x7383,0x73C4,0x7363,0x8352,0x7373,0x7383,0x6383,0xB483,0x8373,0x8373,0x73B4,0xC483,0xB493,0x83C4,0x73B4,0xB493,0x7383,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46
.word 0xD4D4,0xD4D4,0xD4C3,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xD4E4,0xE4F4,0xE4E4,0xE4E4,0xD3D4,0xE4D4,0xC3E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4D4,0x62D4,0xC4BF,0xAFD5,0xAF73,0x5273,0x52AF,0x9F52,0x5252,0x739F,0x52AF,0x529F,0x9F52,0x8F42,0x8F42,0x7F42,0xAF41,0xD4C4,0x42AF,0x9F52,0x9F52,0xD48F,0x9F52,0x9F52,0x4252,0xE541,0x9F52,0x5252,0x529F,0x9F52,0xAF62,0xD4D5,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xC4D4,0x62AF,0x8F9F,0x8F42,0x7F62,0xE4E4,0x7FD4,0xD4D4,0xD4E4,0xD4E4,0x627F,0x41D4,0x4152,0xC48F,0x52D4,0x7FD4,0x7FD4,0x7FD4,0xAFD4,0x62C4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xAFD4,0xD562,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD462,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0x62D4,0x4662,0x4642,0x4646,0x5F44,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4642,0x8363,0x7393,0xB483,0x8373,0x83B4,0x83C4,0x8383,0x7383,0x6362,0x8383,0x73C4,0x7383,0x7373,0x7373,0x7383,0x7373,0x9383,0xB3C4,0x73B4,0x8373,0x8383,0x8373,0x4373,0x5F46,0x4646,0x4646,0x4646,0x4642,0x4646,0x4644,0x4646,0x4F42,0x5F46,0x4246,0x4646
.word 0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD3E4,0xD4E4,0xE4D3,0xD4E4,0xE4E4,0xE4D3,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D3,0xD4E4,0xD4C3,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAF62,0x6283,0x73AF,0x62AF,0xAF52,0x9F52,0x9F52,0x5252,0x52AF,0x9F52,0x8F52,0x9F42,0x8F42,0x7F41,0x6F7F,0x627F,0xD4D4,0x529F,0x429F,0x9F52,0x9F52,0x9F42,0x9F52,0x9F52,0x6262,0xD4E5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD483,0x62D4,0x9F62,0x9F52,0x8F52,0x8F42,0x7F41,0x6242,0xC48F,0x428F,0x417F,0x417F,0x428F,0x628F,0x7FD4,0x428F,0xD4D4,0xD48F,0xD48F,0xD441,0xC441,0x6241,0x62AF,0x72D5,0xE4E4,0xC3D4,0xD4E4,0xC3D4,0x62D4,0x9F62,0xD562,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD3D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD5D4,0x46AF,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4642,0x4F46,0x4646,0x4246,0x8363,0x62B4,0x7373,0x9383,0x83C4,0x7373,0x6373,0x8352,0x83D4,0x7373,0x7363,0x8352,0x7373,0x8373,0x8383,0x83C4,0x8373,0xC493,0x8373,0x93B4,0x73B4,0x7352,0x7373,0x6273,0x4252,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46
.word 0xD4D4,0xC3D4,0xE4D4,0xC3D4,0xE4D4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E3,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4C3,0xD4C3,0xD4E4,0xE4D4,0xD4D4,0x62D4,0xC4D4,0xC4AF,0xD562,0xAF52,0x9F73,0x6252,0x529F,0x529F,0xAF52,0x9F52,0x8F42,0x4152,0x428F,0x428F,0x7F41,0x7F7F,0x6242,0x52D5,0x528F,0x528F,0x528F,0x8F52,0x8F42,0x8F52,0x9FD4,0x6273,0xD562,0xD4D4,0xD462,0xAF62,0x6262,0x529F,0x429F,0x529F,0x428F,0x418F,0x428F,0x5241,0x52F5,0x629F,0x6262,0x6262,0x6262,0xAF62,0x8F52,0x7FC4,0xE4D4,0xD4E4,0x41E4,0x62D4,0x7FD4,0x8F62,0xC48F,0x62D5,0x629F,0xD48F,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x52AF,0xD4AF,0xD4D4,0xE4E4,0xD3E4,0xE4E4,0xD4E4,0xD4E4,0xE4C3,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xAF62,0x4683,0x4646,0x4646,0x4246,0x4646,0x4F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x6344,0x8373,0x7373,0x7383,0xB483,0x6383,0x5273,0x7393,0x7373,0x8373,0x7383,0x8373,0x7373,0x7383,0x83B4,0x8373,0x83C4,0x83B4,0x9383,0x73B4,0x7383,0x8373,0x4363,0x4646,0x4646,0x4644,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4446,0x5F46,0x4646
.word 0xC3D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4C3,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x72D4,0xD4D4,0xC4BF,0xAFD4,0xAF62,0x6262,0x73AF,0x52AF,0x9F52,0xAF52,0x5252,0x529F,0x528F,0x428F,0x418F,0x7F7F,0x7F6F,0x466F,0x4646,0x6242,0x62D4,0x528F,0x528F,0x528F,0x529F,0x42AF,0xC48F,0x9F9F,0x418F,0x8F41,0x9F52,0x9F42,0x9F42,0x8F42,0x8F52,0x8F42,0x8F41,0x7F42,0x8F42,0x528F,0x9FE4,0x8F42,0x8F8F,0x8F8F,0x8F8F,0x8F42,0x8F52,0x527F,0x5251,0xE4E4,0xD4D4,0xD48F,0xD4D4,0xD4D4,0xBFC4,0x8FD4,0xAF42,0x6252,0xE4D4,0xD4E4,0xD4D4,0xD462,0xD562,0xD462,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0x62D4,0x46AF,0x4642,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x4246,0x4246,0x6344,0x7373,0x7383,0x7373,0x5273,0x7373,0xB483,0x8373,0x7383,0x7373,0x8373,0xD483,0xB3B3,0xB483,0x7383,0x8373,0x8383,0x83B4,0x8373,0x8383,0x6363,0x4642,0x4246,0x5F46,0x5F46,0x5F46,0x4246,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646
.word 0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xC3E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4F4,0xE4E3,0xE4E4,0xD4E4,0xE4E4,0xD3D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xC3D4,0xE4D4,0xD3D4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xAFC4,0xC4C4,0xC4AF,0x73AF,0x52AF,0xAF52,0x9F52,0x5252,0x52AF,0x529F,0x528F,0x418F,0x428F,0x7F7F,0x7F7F,0x466F,0x466F,0x4646,0xC49F,0x52D4,0x528F,0xC4AF,0xD4C4,0xD4C4,0x418F,0x4252,0x428F,0x428F,0x418F,0x428F,0x528F,0x418F,0x428F,0x428F,0x428F,0x418F,0x428F,0xAF41,0x52E4,0x8F41,0x4242,0x4241,0x427F,0xC4AF,0xE48F,0x8F41,0x7F8F,0xD4E4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7FD4,0xD4C4,0xAFD4,0x6262,0xD4D4,0xD4D4,0xC4AF,0x629F,0xC4AF,0xD4D4,0xD3E4,0xF4E4,0xD3E4,0xD4E4,0xD4E4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD472,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0x62D4,0x4662,0x5F46,0x4646,0x5F46,0x5F46,0x4246,0x5F46,0x4646,0x4646,0x4246,0x5F46,0x7346,0x4663,0x4646,0x7342,0x7383,0x7352,0x7352,0x5273,0x9373,0x6373,0x8373,0x8363,0x8373,0x93D4,0xC4C4,0x8383,0x8373,0xC483,0x7383,0x7373,0xB483,0x5273,0x4643,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xD4D4,0xD4D4,0xE4E4,0xE4C3,0xE4E4,0xE4D4,0xE4E4,0xE4D3,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4C3,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD472,0x62C4,0xD562,0xAF52,0xAF73,0x6252,0x529F,0x529F,0x9F52,0x8F52,0x8F52,0x8F42,0x7F41,0x426F,0x467F,0x6F46,0x6F46,0x4646,0xD541,0xC4D4,0x418F,0xC49F,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xAF62,0xC4C4,0x8F41,0x8F41,0x8F42,0x8F42,0x8F41,0x8F41,0x8F42,0x8F42,0xE452,0x41D4,0x41D5,0x7F8F,0x8F8F,0x8F42,0x4142,0xE4E4,0xD4D4,0xC4C4,0x838F,0xD4BF,0xE462,0xE4E4,0xD4E4,0xD4D4,0xD483,0xD4D4,0xD562,0xAFD4,0xD483,0xD4E4,0xD4C4,0xD541,0xD462,0xD4D4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4C3,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xAFD4,0x46C4,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x6246,0x7383,0x4642,0x8343,0x7373,0x7363,0x7352,0x8373,0x7373,0xC383,0x7383,0x7373,0x7373,0x8363,0x8383,0x93B4,0x83C4,0x73B4,0x8383,0x83C4,0x6363,0x4644,0x4646,0x4642,0x4646,0x4246,0x5F46,0x4646,0x4F42,0x4646,0x5F46,0x4246,0x4646,0x4644,0x4646
.word 0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE3E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4D4,0xC3E4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4C3,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xC4BF,0xD5BF,0x62AF,0x73AF,0x63AF,0x52AF,0x52AF,0x9F62,0x5252,0x429F,0x428F,0x8F41,0x7F42,0x6F6F,0x4642,0x465F,0x4646,0x465F,0x6F46,0xC452,0xC4D4,0x62D4,0x62D4,0x9FD4,0x5241,0x8F8F,0x8F42,0x8F8F,0x8F42,0x8F42,0x4241,0x428F,0x4241,0x428F,0x8F41,0x8F41,0xE452,0x8F62,0x7F62,0x4142,0x8F42,0x417F,0x628F,0xE4E4,0xE4E4,0x628F,0xC48F,0x8FD4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD48F,0xD441,0x72D4,0xD562,0x62AF,0x627F,0x62AF,0x629F,0xD4D5,0xE4D4,0xE4E4,0xE4E4,0xE4E3,0xD3E4,0xD4E4,0xD4D4,0xD4E4,0xC3D4,0xD4D4,0xD472,0xD4D4,0xE4D4,0xD4D4,0x72D4,0x62D5,0x46AF,0x465F,0x465F,0x4646,0x4246,0x4646,0x4646,0x4644,0x4F46,0x4F46,0x4646,0x7342,0x8373,0x8383,0x7383,0x5273,0x5252,0x8383,0x7363,0x7373,0x8373,0x6383,0x7383,0x7373,0x7383,0xB483,0x8383,0x8373,0x8393,0x93B4,0x5273,0x4643,0x4642,0x4646,0x465F,0x4246,0x464F,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4F46,0x4646,0x464F,0x4646
.word 0xD4E4,0xC3E4,0xD4E4,0xE4E4,0xD3E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4C3,0xD4E4,0xD4C3,0xD4D4,0xD4E4,0xE4C3,0xE4E4,0xD4E4,0xE4E4,0xD3D4,0xE4E4,0xC3D4,0xE4D4,0xC3D4,0xD4D4,0xD472,0xD4D4,0xD4D4,0xD462,0xD562,0xAF62,0x9F73,0x9F73,0x9F52,0x9F52,0x9F52,0x9F42,0x8F42,0x7F8F,0x7F7F,0x6F7F,0x4646,0x4646,0x4642,0x5F46,0x4646,0x6F46,0x8F46,0xD541,0x419F,0x528F,0xD4D4,0x52D4,0x428F,0x417F,0x428F,0x417F,0x8F8F,0x8F41,0x428F,0x418F,0x428F,0xD552,0xD4E4,0x429F,0x8F8F,0x8F41,0x8F42,0x8F42,0xE452,0xE4E4,0xE4E4,0xE4E4,0x62D4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0x41E4,0xE4D4,0xBFC4,0xD47F,0xD4D4,0xD562,0x8F62,0x6252,0x62D5,0xD4AF,0xD4D4,0xD3E4,0xF4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0x62D4,0xD4E5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xBF62,0x4673,0x4646,0x4646,0x464F,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4644,0x4646,0x4346,0xB493,0x8373,0x8373,0x7363,0x7373,0x8373,0x5273,0x7383,0x5273,0x8373,0x7373,0x8373,0x7383,0x8383,0x83D4,0x93C4,0x83C4,0x6373,0x4644,0x4646,0x465F,0x465F,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F
.word 0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D3,0xD3E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE3D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D3,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xBF62,0x6273,0x52AF,0x5252,0x52AF,0x9F52,0x8F52,0x4142,0x428F,0x4241,0x7F41,0x6F7F,0x5F46,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x5241,0x62D4,0x9F62,0xAF52,0xD4D4,0x9F62,0x8F42,0x8F42,0x7F41,0x8F42,0x8F42,0x8F42,0x4241,0xAF52,0xE4D4,0x5262,0x418F,0x8FC4,0x8F42,0x8F41,0x4241,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xC3E4,0xE4E4,0xD583,0xE4D4,0x52D4,0x62D4,0xD4D4,0xD4D4,0x42BF,0x8F8F,0xAF52,0xD4C4,0xE472,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0x62D4,0x469F,0x4246,0x4646,0x4646,0x4642,0x4646,0x465F,0x4F42,0x4646,0x4273,0x4646,0x4642,0x9373,0x7383,0x7373,0x5252,0x8373,0x7373,0x8373,0x7373,0x7383,0x7363,0x8383,0xB483,0xC483,0xB483,0x8393,0x73B4,0x6373,0x4643,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4642,0x465F,0x5F44,0x4646,0x4246,0x4646
.word 0xC3D4,0xD4E4,0xE4E4,0xD4D3,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4D3,0xE4E3,0xE4E4,0xE4E4,0xE4D4,0xC3D4,0xE4D4,0xD4E4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xE4C3,0xE4E4,0xE4D3,0xE4E4,0xD3D4,0xD4E4,0xD4E4,0xE4E4,0xC3D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBF62,0xC4C4,0xC4BF,0xAF62,0xAF62,0xAF63,0x9F73,0x5252,0x429F,0x8F41,0x8F7F,0x6F7F,0x7F42,0x4644,0x4646,0x4646,0x465F,0x465F,0x5F46,0x5142,0x8F52,0x9F7F,0xC4D5,0x62D4,0x418F,0xC49F,0x62D4,0x428F,0x428F,0x418F,0x4142,0xAF52,0xE4D4,0x8FE4,0x62D4,0xD48F,0x62D4,0x428F,0x428F,0x428F,0xD48F,0xF4E4,0xE3E4,0xE4E4,0xD4E4,0x62D4,0xE4D4,0xE4E4,0xE4D4,0x41E4,0xE4D4,0xD4E4,0xE4E4,0x9FC4,0xD49F,0xD4D4,0x8FD4,0x7F42,0x4241,0xAFD4,0xE4D4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xD4E4,0x62D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0x62D4,0x4662,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4246,0x9383,0x4642,0x4646,0x7342,0x83C4,0x73C4,0x8373,0x7363,0x83D4,0x6373,0x8373,0x73D4,0x7373,0x7373,0x5263,0x6373,0x4463,0x4646,0x4642,0x4246,0x4246,0x4646,0x465F,0x4646,0x4642,0x465F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646
.word 0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE3E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xC3E4,0xE4D4,0xD4E4,0xD4E4,0xC3D4,0xD4D4,0xD4E4,0xE4C3,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4C3,0xD3D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC4D4,0xC4BF,0xAF62,0xAFC4,0xAF63,0x5273,0x529F,0x429F,0x429F,0x428F,0x4241,0x6F7F,0x7F7F,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4346,0x9F52,0x9F52,0x4142,0x9F41,0xD4C4,0xC4D4,0x8F52,0xD462,0xD4D4,0xD4D4,0x9F52,0xD562,0xE4E4,0x52D4,0x529F,0x8F41,0x417F,0x8F8F,0x8F42,0x417F,0x527F,0xE462,0xE4F4,0xE4F4,0xD4E4,0xD4E4,0xD462,0xD4D4,0xE4E4,0xE4E4,0xD442,0xD47F,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x8F51,0x7F41,0x7F8F,0xD441,0xD4D4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62BF,0x46AF,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4F42,0x4646,0x465F,0xB483,0x4273,0x4646,0x4446,0x9383,0x5283,0x6263,0x7352,0x7383,0x9393,0x7373,0x8383,0x6383,0x4246,0x4646,0x4642,0x4646,0x4642,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x5F44,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x465F
.word 0xE4D4,0xE4E4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E3,0xE4E4,0xD4E4,0xE4D3,0xD4D4,0xE4D4,0xC3D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xF4D4,0xD4E4,0xD4F4,0xD4E4,0xE4D3,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD483,0x62D5,0xAF62,0x6273,0x52AF,0x52AF,0x9F52,0x8F42,0x8F52,0x7F7F,0x7F7F,0x6F6F,0x4646,0x4646,0x465F,0x465F,0x4246,0x4142,0x9F52,0x4142,0x428F,0x428F,0x628F,0xD4AF,0x62D4,0xD4D5,0x51D4,0xD4AF,0xD4D4,0xD4D4,0x8F51,0x8F41,0x8F42,0x427F,0x7F7F,0x7F42,0x8F41,0x5242,0xD4AF,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xC4BF,0x7FD4,0x41D4,0xD4E4,0xE441,0xD441,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0x427F,0x428F,0x4241,0x417F,0xE452,0xF4E4,0xE4F4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xE4D4,0x72D4,0xD562,0x4662,0x4246,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0xB363,0x8383,0x8383,0x8373,0xB483,0x7383,0x7373,0x8383,0x8363,0x7363,0x8373,0x83B4,0x63B4,0x4646,0x465F,0x4646,0x4642,0x465F,0x465F,0x465F,0x465F,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4646
.word 0xD4E4,0xD4E4,0xE4C3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xD4D4,0xE4D3,0xF4E4,0xD4E4,0xE4D3,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E3,0xE4F4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4C3,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0x62D4,0xC4BF,0xC4AF,0x73AF,0x63AF,0x9F52,0x9F52,0x4152,0x428F,0x418F,0x427F,0x7F6F,0x5F44,0x5F46,0x4646,0x5F46,0x4646,0x5241,0x429F,0x528F,0x418F,0x8F42,0x417F,0x6252,0xD4D4,0x83AF,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8F52,0x527F,0x51D4,0x7F8F,0x7F7F,0x7F42,0x9F52,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0x62D4,0xD4D4,0xD4D4,0xE472,0x7FE4,0x62D4,0x7F8F,0x7F41,0x7F41,0xD441,0xE4D4,0x8F83,0x8F41,0x8F42,0x8F7F,0x7F42,0x9F62,0xE4D4,0xE4E4,0xE4D3,0xD3D4,0xD4E4,0xC3E4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD5D4,0x52AF,0x46AF,0x465F,0x4646,0x4646,0x4246,0x4646,0x4646,0x5F46,0x4646,0x7246,0x83B4,0xB383,0x92D4,0x93D4,0x73B4,0x5273,0x7363,0x8383,0x7373,0x7352,0x9373,0x6273,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4246,0x4F46,0x4646,0x4642,0x465F,0x4F46,0x4646,0x465F,0x4F46,0x4646,0x4246,0x4646,0x4642
.word 0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D3,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xC3D4,0xD4E4,0xE4D3,0xE4E4,0xF4E4,0xF4E4,0xE3E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4C4,0x6262,0x62D5,0x62AF,0x73AF,0x529F,0x529F,0x429F,0x4241,0x7F7F,0x6F41,0x4646,0x4646,0x5F46,0x4646,0x4146,0x9F52,0x9F42,0x9F52,0x9F52,0x8F42,0x8F42,0x8F8F,0x629F,0xD4D4,0x6262,0x419F,0x6241,0xD4BF,0x8FD4,0xD452,0x8F62,0xD47F,0xD4D4,0x4152,0x9F52,0xD4D4,0xE4E4,0xE4E4,0xD4C3,0xC3D4,0xD4D4,0xD4D4,0xE4D4,0x7FE4,0x528F,0x5262,0x62AF,0x8F52,0x5242,0xC4D4,0xE48F,0x5251,0x7FD4,0x6242,0x42AF,0x527F,0xAF62,0x5242,0x8F62,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xBFD4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0x62D4,0x62D4,0x4652,0x4646,0x465F,0x465F,0x465F,0x5F46,0x4646,0x4642,0x4446,0x4246,0x93C4,0x83B4,0xD4B4,0xD493,0x8373,0xD483,0x7373,0x6383,0x8373,0x7373,0x8383,0x7373,0x4246,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4646,0x4446,0x4246,0x4F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4246,0x4F46,0x5F46,0x4646
.word 0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xE4E4,0xD4E4,0xD4C3,0xD4C3,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xF4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xC4BF,0xAFD4,0xD562,0xAF52,0xAF63,0x9F73,0x5252,0x429F,0x8F8F,0x6F6F,0x417F,0x5F46,0x4246,0x4646,0x4246,0x4142,0x8F52,0x5252,0x529F,0x9F52,0x9F42,0x4241,0x4241,0x428F,0xC4AF,0xD4D5,0xD562,0xD5D4,0x62D4,0x627F,0x62D5,0x418F,0xE4C4,0x52D4,0x528F,0xD4AF,0xE4D4,0xD4E4,0xE4E4,0x52D4,0x429F,0x417F,0xD462,0x7FE4,0x4241,0x8F41,0xD562,0x8F52,0x9F52,0x8F41,0xD4D4,0x8F41,0x8F52,0xD47F,0x417F,0x528F,0xBF62,0x8F52,0x628F,0x52AF,0xD452,0xE4E4,0xD4E4,0xE4D3,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xAF62,0x4652,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x6346,0xC493,0x9393,0xB3D4,0x5263,0x8373,0x7373,0x6393,0x5273,0x5273,0x7373,0x7373,0x4642,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x4F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x4246,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D3,0xE4E4,0xE4E4,0xE4E3,0xF4E4,0xE3E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4C3,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x72D4,0x62D5,0x62BF,0x73AF,0x52AF,0x5252,0x529F,0x429F,0x428F,0x7F7F,0x7F6F,0x4646,0x465F,0x465F,0x4246,0x9F52,0x9F52,0x9F52,0x9F52,0x8F52,0x4152,0x9F52,0x8F52,0x8F41,0x8F42,0xD462,0xBFD4,0x72C4,0x528F,0xD4D4,0x8F9F,0x8F52,0x4241,0xD4AF,0xD4E4,0xE4D4,0xD4E4,0xC3E4,0xE4D4,0xE4D4,0xD4D4,0xD452,0x418F,0x6252,0x8FD5,0xD562,0x8F9F,0x417F,0x528F,0xAF62,0x417F,0xD452,0x8FD4,0x4142,0x428F,0x8F7F,0x8F42,0x6242,0x9F62,0x9F52,0xAF62,0xE442,0xE4E4,0xD4D4,0xD4E4,0xC3D4,0xD4E4,0x62D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD572,0xD562,0x469F,0x4642,0x4246,0x4646,0x4642,0x465F,0x4F42,0x5F46,0x4646,0x4646,0x7372,0x4446,0x8373,0xD4C4,0x5273,0x7373,0x7352,0x7373,0x8373,0x8373,0x83C4,0x73D4,0x4283,0x4246,0x5F46,0x4646,0x464F,0x465F,0x465F,0x4642,0x4F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4F46,0x4646,0x464F,0x4646,0x5F46,0x5F46,0x4646,0x465F,0x4646
.word 0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4C3,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xD4D3,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD562,0x6262,0xAF62,0x9F62,0x9F52,0x8F52,0x7F41,0x6F7F,0x417F,0x4246,0x4646,0x4646,0x5241,0x529F,0x9F52,0x5252,0x529F,0x529F,0x528F,0x429F,0x428F,0x8F41,0x8F42,0x417F,0x62C4,0x429F,0x629F,0x52AF,0x8F41,0x5242,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xE47F,0x7FE4,0x9F52,0x528F,0x417F,0x528F,0xD4D4,0xD4D4,0xD4E4,0x8F52,0x8F42,0x9F42,0x6242,0x528F,0xAFC4,0x8F52,0xD562,0x8F7F,0x428F,0x428F,0x428F,0x6262,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0x62D4,0x62BF,0x4652,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x8383,0x4673,0x8363,0x9383,0x7363,0x5283,0x8383,0x8373,0x6383,0x7383,0x7373,0x8383,0x7373,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x464F,0x4646,0x4646,0x5F46,0x4646,0x464F,0x4642,0x4646,0x4646,0x4246,0x5F46
.word 0xE4E4,0xE4C3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xD4E4,0xC3E4,0xE4D4,0xE4D4,0xC3D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE3E4,0xF4E4,0xF4E4,0xF4E4,0xE4E3,0xE3F4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4C3,0xC3D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xBFC4,0xAF62,0xAF73,0xAF63,0x5273,0x429F,0x4241,0x427F,0x7F6F,0x5F46,0x5F46,0x4246,0x9F52,0xAF52,0x5262,0x62AF,0xAF62,0x6263,0x52AF,0x9F52,0x9F52,0x4152,0x428F,0x8F41,0x8F8F,0x527F,0x9F62,0x9F52,0x527F,0x62AF,0xD4D5,0xD4D4,0xD4D4,0xD4C3,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD483,0x417F,0x5252,0x7FD5,0xD4C4,0xD4D4,0x83D4,0xD4D4,0x417F,0xE4D4,0xD4E4,0x52D4,0x8F8F,0x8F42,0x528F,0xD4D4,0x8F62,0xD452,0x8FC4,0x5241,0x62AF,0x9F8F,0xC462,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xAF62,0xAF73,0x4652,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x464F,0x9344,0x8383,0x83C4,0x93B4,0x7373,0x7352,0x7363,0x7352,0x7383,0x7373,0x5252,0x7373,0x7383,0x4273,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x465F,0x465F,0x465F,0x5F46,0x4646,0x4F44,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646

;.word 0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xD3E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xE4E4,0xE4D3,0xE4E4,0xE4D4,0xD4E4,0xE4C3,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D3,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62D5,0x73AF,0x52AF,0x52AF,0x9F52,0x8F42,0x6F7F,0x7F41,0x4646,0x4646,0x5146,0x6252,0x5262,0x73AF,0x52AF,0x63AF,0x52AF,0xAF62,0xAF73,0x5252,0x529F,0x9F52,0x8F42,0x4252,0x9F41,0x5252,0x429F,0x4241,0x529F,0x629F,0x62AF,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD483,0x41AF,0x8F8F,0x8F8F,0xD441,0x8FD4,0x5241,0x9F9F,0x8F52,0xD562,0x428F,0xE462,0xD4E4,0x6262,0x42AF,0x5241,0xD4AF,0x5241,0xD49F,0x527F,0x8F62,0x8F73,0x6252,0x418F,0xD4C4,0xE4D4,0xD4D4,0xD4D4,0xD462,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x469F,0x465F,0x4246,0x5F46,0x4246,0x464F,0x4646,0x4646,0x4642,0x4646,0x7346,0xC483,0x93B2,0xD4D4,0x5283,0x7352,0x73D4,0x8373,0x9363,0x7373,0x9383,0x8373,0x7383,0x7383,0x4642,0x465F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x5F46,0x5F46,0x4246,0x4646,0x465F,0x465F
;.word 0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xD4F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E3,0xE4E3,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0x62D4,0xBFD4,0xD583,0xAF62,0x6273,0x9F62,0x9F52,0x8F52,0x7F41,0x6F7F,0x5F46,0x4646,0x5241,0x62AF,0x73AF,0x73AF,0x73AF,0xAF62,0xAF73,0xAF73,0x6252,0x9FD5,0x9F52,0x9F42,0x4142,0x8F9F,0x7F42,0x528F,0x8F8F,0x428F,0x418F,0x428F,0x9F41,0x9F52,0x9F52,0x4152,0x4141,0x5241,0x62BF,0x8F52,0x4242,0x4152,0x7FD4,0x7F41,0xAF42,0x6273,0x628F,0x419F,0xD4D4,0xD4D4,0x8F41,0x528F,0xD462,0xD4D4,0x8F41,0xD552,0x7F72,0x9F41,0x52D5,0xD49F,0x628F,0x528F,0x62AF,0xD4E4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD472,0xC3D4,0xD4D4,0xD562,0xC4AF,0x4641,0x4246,0x4F46,0x4646,0x4646,0x4246,0x5F46,0x4646,0x464F,0x4246,0x4646,0xC452,0xB493,0xD483,0x7383,0x7352,0x5273,0x8383,0x8373,0x6263,0x7363,0xC473,0xB493,0x7383,0x7373,0x4654,0x4646,0x4642,0x4646,0x465F,0x4646,0x4642,0x4246,0x4F46,0x464F,0x4642,0x465F,0x4F46,0x4644,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
;.word 0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D3,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD3D4,0xD3D4,0xE4E4,0xE4D3,0xE4F4,0xE4E4,0xE4F4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D3,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD562,0x73AF,0x73AF,0x529F,0x429F,0x7F7F,0x6F6F,0x4644,0x435F,0x6252,0x73AF,0x73AF,0x73AF,0xAF62,0xD573,0xAF62,0xD573,0x629F,0x52AF,0xAF62,0x5252,0x429F,0x428F,0x428F,0x7F7F,0x427F,0x418F,0x427F,0x8F8F,0x8F42,0x9F42,0x4152,0x629F,0x62D5,0x73AF,0x429F,0x429F,0x417F,0x429F,0x418F,0x6252,0x429F,0x8F8F,0x8F42,0x8F42,0x837F,0xD4D4,0x7FD4,0x8F52,0x628F,0xD4E5,0x7FD4,0x418F,0x527F,0x52D4,0x419F,0xD552,0x8F62,0xAF42,0x8F73,0x6262,0xD4E4,0xBFD4,0xD4C4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xC4BF,0xAF62,0x4652,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4246,0xC473,0xD493,0xB493,0x7352,0x7373,0x7363,0x8373,0x7373,0x7393,0x7383,0xC483,0x9373,0x8373,0x5273,0x4644,0x5F46,0x4646,0x4646,0x4642,0x464F,0x4646,0x465F,0x4F4F,0x464F,0x4646,0x4F5F,0x464F,0x4646,0x4642,0x4646,0x464F,0x4642,0x4646,0x4644
;.word 0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E3,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xD3D4,0xE4D4,0xD4C3,0xE4D4,0xC3D4,0xE4D4,0xE4D4,0xE4D4,0xD4C3,0xE4E4,0xE4E4,0xE4F4,0xF4E4,0xF4F4,0xF4E4,0xF4F4,0xE4E4,0xF4E4,0xE3E4,0xE4E4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xC3D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFD4,0x62D4,0xAF62,0xAF73,0x5252,0x528F,0x7F7F,0x7F7F,0x4646,0x6242,0x73AF,0xD562,0xAF62,0xD5C4,0xC4AF,0xC4AF,0xC4AF,0xD462,0xD5C4,0xD562,0x6262,0x62AF,0x9F62,0x5262,0x418F,0x8F52,0x7F42,0x4241,0x8F8F,0x4142,0x428F,0x428F,0x428F,0x428F,0x418F,0x8F42,0x8F42,0x7F41,0x8F41,0x7F42,0x8F41,0x528F,0x42AF,0x417F,0x8F7F,0x427F,0x8F8F,0x627F,0xD4D4,0x9F52,0x8F42,0x628F,0x62D4,0x8F42,0x7F8F,0x418F,0x9F42,0x418F,0x528F,0xD48F,0xC4D4,0x73BF,0x52AF,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x72D4,0xAFC4,0x6262,0x4652,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4246,0x464F,0x4646,0x4246,0x4364,0x4646,0x4646,0x7342,0x8383,0x5252,0x5273,0x5273,0x7373,0x7352,0x6352,0x6373,0x5273,0xC483,0x9373,0x7383,0x7373,0x4663,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x464F,0x424F,0x4F5F,0x4646,0x465F,0x4F4F,0x4646,0x465F,0x465F

.word 0xBFDF,0xBFDF,0xBF4F,0xBFDF,0xBFD4,0xE4E4,0xE4E4,0xE4D4,0xD3E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4C3,0xE4D4,0xD4E4,0xE4E3,0xE4E4,0xF4E4,0xE4E4,0xE4F4,0xE3E4,0xF4E4,0xE3E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xD4E4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD562,0xC4AF,0x73AF,0x529F,0x429F,0x6F7F,0x6F42,0x4642,0x6262,0xAFD5,0xBF62,0xD5C4,0xD462,0xD583,0xD462,0xD5D4,0xAFD4,0xBF83,0xAF62,0x62D5,0xD5D5,0xAF62,0xAF73,0x9F52,0x8F52,0x4241,0x8F7F,0x7F42,0x8F41,0x8F42,0x8F41,0x8F41,0x8F42,0x428F,0x8F8F,0x7F41,0x8F42,0x7F42,0x418F,0x8F42,0x8F42,0x417F,0x427F,0x427F,0x417F,0x8F42,0x8F42,0xD4D4,0x52D4,0x7FD5,0x8F41,0xE452,0x41D4,0x7F42,0x7F42,0x428F,0x427F,0x428F,0xD441,0xD4D4,0xD562,0xAF62,0xBF62,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x73AF,0x73AF,0x469F,0x4646,0x4646,0x5F46,0x4246,0x4646,0x464F,0x4646,0x4642,0x4646,0x9373,0x4643,0x4642,0x7343,0xC483,0x5273,0x5273,0x7373,0x8373,0x8363,0x52C4,0x7383,0x8383,0x9383,0x83C4,0x83B4,0x8373,0x6383,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4246,0x4F46,0x464F,0x4642,0x464F,0x5F46,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646
.word 0xBFDF,0xBFDF,0xBF4F,0xBFDF,0xBFD4,0xE4D3,0xE4F4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E3,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x52D5,0x52AF,0x8F52,0x6F41,0x6F7F,0x5146,0xBF62,0x6283,0x62D5,0x61D5,0xAFD5,0x62D4,0xBFD4,0xBF83,0xD4C4,0x62D4,0xD4D4,0xD4D4,0xBF62,0x6273,0x73AF,0x5262,0x529F,0x418F,0x428F,0x417F,0x427F,0x417F,0x427F,0x417F,0x417F,0x7F7F,0x7F42,0x7F42,0x8F41,0x7F41,0x7F42,0x7F8F,0x4241,0x7F7F,0x7F7F,0x7F41,0x7F41,0x417F,0x418F,0xD4E4,0xD4E4,0xD47F,0x427F,0x418F,0xD4D4,0x417F,0x428F,0x417F,0x417F,0x8F7F,0x9F42,0xD4D4,0xD4D4,0xD562,0x6262,0xD462,0xD4D4,0xD472,0xD4D4,0x62AF,0xAFC4,0xAF73,0x4645,0x465F,0x465F,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x7246,0x7383,0x8373,0x8382,0x93B4,0x7383,0x7352,0x5273,0x8373,0x7352,0x7383,0x5273,0x6373,0x9363,0x6383,0x8383,0x8373,0x8373,0x5283,0x5F46,0x5F46,0x4246,0x464F,0x4646,0x4F46,0x464F,0x464F,0x4F4F,0x464F,0x464F,0x5F46,0x464F,0x4F46,0x4F4F,0x5F46,0x4646
.word 0xBFDF,0xBFDF,0xDFDF,0xBFDF,0xBFD4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4C3,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D3,0xE4E4,0xE3E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xC4BF,0xC4BF,0x62AF,0xAF62,0x5252,0x6F8F,0x6F41,0x6241,0x62D5,0xAFD5,0xD4C4,0x62D4,0x62D4,0xBFD4,0xD4C4,0x62C4,0x72D4,0xD4D5,0xD4D4,0xD462,0x62D5,0xD5D5,0xAF62,0xAF62,0x9F73,0x5252,0x418F,0x7F8F,0x7F7F,0x7F7F,0x417F,0x427F,0x427F,0x418F,0x417F,0x417F,0x427F,0x417F,0x428F,0x7F41,0x9F42,0x7F41,0x7F42,0x7F41,0x8F42,0x7F42,0x7F42,0xE452,0xD4D4,0xE451,0x8F41,0x8F42,0xD4D4,0x527F,0x7F41,0x7F41,0x7F42,0x4142,0xC49F,0x8F52,0xD4D4,0xBFD4,0xAFC4,0x62D5,0x52AF,0xD4AF,0x62D4,0xD562,0x6262,0x73AF,0x4642,0x4646,0x4246,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x445F,0x8373,0xC483,0xC493,0xC483,0xB493,0x7352,0x7352,0x8373,0x6373,0x8373,0x7383,0x8383,0xB483,0x8393,0x83B4,0x8363,0x83B4,0x8373,0x5473,0x4646,0x4646,0x4646,0x4642,0x464F,0x464F,0x424F,0x464F,0x5F4F,0x4646,0x5F4F,0x465F,0x4F42,0x4F46,0x465F,0x4246
.word 0xBFDF,0xBFDF,0xDFDF,0xBFDF,0xBFD4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xD4E4,0xE4C3,0xE4D4,0xE4C3,0xE4E4,0xE4D4,0xC3D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE3D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D3,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D3,0xE4D3,0xE4E4,0xE4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xAFC4,0x52C4,0x52AF,0x4241,0x6F6F,0xAFC4,0xBFC4,0xD483,0xD4AF,0xD4AF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4C4,0xBFD4,0xD4D4,0xD462,0xD4D5,0x62D5,0xC4AF,0x62AF,0x5252,0x5242,0x4241,0x4241,0x7F7F,0x7F41,0x7F7F,0x7F7F,0x7F42,0x427F,0x418F,0x428F,0x8F7F,0x7F42,0x7F41,0x7F42,0x7F41,0x7F42,0x417F,0x417F,0x418F,0xE47F,0xE4D4,0xD4D4,0x8F62,0x7F41,0x83E4,0x7FD4,0x8F52,0x7F42,0x7F41,0x7F8F,0xAF42,0x7F62,0xD4D4,0xD4D4,0xD4D4,0xD562,0x6262,0x51D5,0x629F,0x52AF,0xC4BF,0x9F62,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x464F,0x4642,0x465F,0x4646,0x4646,0x4446,0x9373,0x83C4,0xB483,0xC3B3,0x7383,0x5252,0x7352,0x7373,0x7352,0x6373,0x6373,0x8373,0x7373,0x8383,0x8363,0x7393,0x7383,0x8383,0x4463,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x464F,0x464F,0x464F,0x4F46,0x4F46,0x4F4F,0x4F46,0x5F46,0x464F,0x5F46


.word 0xBFD4,0xBFD4,0xBFD4,0xBFD4,0xBFD4,0xE4E4,0xE4E4,0xE4D4,0xD3E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4C3,0xE4D4,0xD4E4,0xE4E3,0xE4E4,0xF4E4,0xE4E4,0xE4F4,0xE3E4,0xF4E4,0xE3E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xD4E4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD562,0xC4AF,0x73AF,0x529F,0x429F,0x6F7F,0x6F42,0x4642,0x6262,0xAFD5,0xBF62,0xD5C4,0xD462,0xD583,0xD462,0xD5D4,0xAFD4,0xBF83,0xAF62,0x62D5,0xD5D5,0xAF62,0xAF73,0x9F52,0x8F52,0x4241,0x8F7F,0x7F42,0x8F41,0x8F42,0x8F41,0x8F41,0x8F42,0x428F,0x8F8F,0x7F41,0x8F42,0x7F42,0x418F,0x8F42,0x8F42,0x417F,0x427F,0x427F,0x417F,0x8F42,0x8F42,0xD4D4,0x52D4,0x7FD5,0x8F41,0xE452,0x41D4,0x7F42,0x7F42,0x428F,0x427F,0x428F,0xD441,0xD4D4,0xD562,0xAF62,0xBF62,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x73AF,0x73AF,0x469F,0x4646,0x4646,0x5F46,0x4246,0x4646,0x464F,0x4646,0x4642,0x4646,0x9373,0x4643,0x4642,0x7343,0xC483,0x5273,0x5273,0x7373,0x8373,0x8363,0x52C4,0x7383,0x8383,0x9383,0x83C4,0x83B4,0x8373,0x6383,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4246,0x4F46,0x464F,0x4642,0x464F,0x5F46,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646
.word 0xBFD4,0xBFD4,0xBFD4,0xBFD4,0xBFD4,0xE4D3,0xE4F4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E3,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x52D5,0x52AF,0x8F52,0x6F41,0x6F7F,0x5146,0xBF62,0x6283,0x62D5,0x61D5,0xAFD5,0x62D4,0xBFD4,0xBF83,0xD4C4,0x62D4,0xD4D4,0xD4D4,0xBF62,0x6273,0x73AF,0x5262,0x529F,0x418F,0x428F,0x417F,0x427F,0x417F,0x427F,0x417F,0x417F,0x7F7F,0x7F42,0x7F42,0x8F41,0x7F41,0x7F42,0x7F8F,0x4241,0x7F7F,0x7F7F,0x7F41,0x7F41,0x417F,0x418F,0xD4E4,0xD4E4,0xD47F,0x427F,0x418F,0xD4D4,0x417F,0x428F,0x417F,0x417F,0x8F7F,0x9F42,0xD4D4,0xD4D4,0xD562,0x6262,0xD462,0xD4D4,0xD472,0xD4D4,0x62AF,0xAFC4,0xAF73,0x4645,0x465F,0x465F,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x7246,0x7383,0x8373,0x8382,0x93B4,0x7383,0x7352,0x5273,0x8373,0x7352,0x7383,0x5273,0x6373,0x9363,0x6383,0x8383,0x8373,0x8373,0x5283,0x5F46,0x5F46,0x4246,0x464F,0x4646,0x4F46,0x464F,0x464F,0x4F4F,0x464F,0x464F,0x5F46,0x464F,0x4F46,0x4F4F,0x5F46,0x4646
.word 0xBeD4,0xBeD4,0xBeD4,0xBFD4,0xBFD4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4C3,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D3,0xE4E4,0xE3E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xC4BF,0xC4BF,0x62AF,0xAF62,0x5252,0x6F8F,0x6F41,0x6241,0x62D5,0xAFD5,0xD4C4,0x62D4,0x62D4,0xBFD4,0xD4C4,0x62C4,0x72D4,0xD4D5,0xD4D4,0xD462,0x62D5,0xD5D5,0xAF62,0xAF62,0x9F73,0x5252,0x418F,0x7F8F,0x7F7F,0x7F7F,0x417F,0x427F,0x427F,0x418F,0x417F,0x417F,0x427F,0x417F,0x428F,0x7F41,0x9F42,0x7F41,0x7F42,0x7F41,0x8F42,0x7F42,0x7F42,0xE452,0xD4D4,0xE451,0x8F41,0x8F42,0xD4D4,0x527F,0x7F41,0x7F41,0x7F42,0x4142,0xC49F,0x8F52,0xD4D4,0xBFD4,0xAFC4,0x62D5,0x52AF,0xD4AF,0x62D4,0xD562,0x6262,0x73AF,0x4642,0x4646,0x4246,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x445F,0x8373,0xC483,0xC493,0xC483,0xB493,0x7352,0x7352,0x8373,0x6373,0x8373,0x7383,0x8383,0xB483,0x8393,0x83B4,0x8363,0x83B4,0x8373,0x5473,0x4646,0x4646,0x4646,0x4642,0x464F,0x464F,0x424F,0x464F,0x5F4F,0x4646,0x5F4F,0x465F,0x4F42,0x4F46,0x465F,0x4246
.word 0xBeD4,0xBeD4,0xBeD4,0xBFD4,0xBFD4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xD4E4,0xE4C3,0xE4D4,0xE4C3,0xE4E4,0xE4D4,0xC3D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE3D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D3,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D3,0xE4D3,0xE4E4,0xE4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xAFC4,0x52C4,0x52AF,0x4241,0x6F6F,0xAFC4,0xBFC4,0xD483,0xD4AF,0xD4AF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4C4,0xBFD4,0xD4D4,0xD462,0xD4D5,0x62D5,0xC4AF,0x62AF,0x5252,0x5242,0x4241,0x4241,0x7F7F,0x7F41,0x7F7F,0x7F7F,0x7F42,0x427F,0x418F,0x428F,0x8F7F,0x7F42,0x7F41,0x7F42,0x7F41,0x7F42,0x417F,0x417F,0x418F,0xE47F,0xE4D4,0xD4D4,0x8F62,0x7F41,0x83E4,0x7FD4,0x8F52,0x7F42,0x7F41,0x7F8F,0xAF42,0x7F62,0xD4D4,0xD4D4,0xD4D4,0xD562,0x6262,0x51D5,0x629F,0x52AF,0xC4BF,0x9F62,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x464F,0x4642,0x465F,0x4646,0x4646,0x4446,0x9373,0x83C4,0xB483,0xC3B3,0x7383,0x5252,0x7352,0x7373,0x7352,0x6373,0x6373,0x8373,0x7373,0x8383,0x8363,0x7393,0x7383,0x8383,0x4463,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x464F,0x464F,0x464F,0x4F46,0x4F46,0x4F4F,0x4F46,0x5F46,0x464F,0x5F46

.word 0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE3E4,0xE4E4,0xE4F4,0xE4E4,0xE4D3,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E3,0xE4E4,0xE4E3,0xE3E4,0xE4E4,0xE4E3,0xE4F4,0xF4E4,0xE4E4,0xE3D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xAFD4,0xD4C4,0x62AF,0x9F62,0x8F42,0x527F,0x62D4,0x62D4,0x62D5,0xD462,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAF72,0x62C4,0x62D5,0xAF62,0x9F62,0x529F,0x8F8F,0x7F7F,0x4241,0x428F,0x4141,0x7F42,0x7F41,0x417F,0x427F,0x417F,0x427F,0x418F,0x7F62,0x7F7F,0x7F42,0x7F41,0x7F42,0x7F42,0x8F42,0xD442,0xD4D4,0xD4D4,0x42E4,0x428F,0xC4E4,0x52AF,0x528F,0x418F,0x427F,0x7F41,0x6241,0x41E5,0xD47F,0xD4D4,0xD4D4,0xBFD4,0xAFD4,0x6262,0x63AF,0x73AF,0x529F,0x52AF,0x5262,0x4642,0x4646,0x4246,0x4646,0x4646,0x4646,0x464F,0x4646,0x4642,0x4646,0x4642,0x6346,0xC493,0xC493,0xC493,0xC493,0x7383,0x5252,0x7352,0x8373,0x8373,0x62D4,0xC483,0x8373,0x93B4,0x7373,0x6383,0xB493,0x8373,0x6383,0x5F46,0x4646,0x464F,0x4F4F,0x5F46,0x4F4F,0x465F,0x464F,0x4F46,0x464F,0x4F5F,0x5F46,0x4646,0x465F,0x4646
.word 0xE4E4,0xD4E4,0xE4D3,0xD3E4,0xF4E4,0xE4E4,0xE4E3,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4C3,0xE4D4,0xC3D4,0xE4E4,0xC3D4,0xE4D4,0xD4E4,0xD4D3,0xE4E4,0xD4E3,0xE4F4,0xE3E4,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4C3,0xD4E4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xD572,0xD562,0x529F,0x429F,0x628F,0xAFD5,0x62D4,0xD4BF,0xD4D5,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD4D4,0xD4D4,0xC4D4,0xC4BF,0xC4BF,0xC4AF,0x73AF,0x9F52,0x4252,0x4241,0x7F8F,0x7F41,0x428F,0x429F,0x5252,0x6252,0xAF52,0x6273,0x52AF,0xAF62,0x427F,0x417F,0x7F7F,0x417F,0x417F,0x417F,0x428F,0xD462,0xD4D4,0xD462,0x7FD4,0x7F41,0xC4E4,0xD562,0x528F,0x428F,0x627F,0xD47F,0xD4E4,0xD4E4,0xD441,0xD4D4,0xD4D4,0xD462,0xD4C4,0x62D5,0x62D5,0xAF52,0xAF73,0xAF73,0xAF52,0x5252,0x5251,0x4642,0x4F46,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4246,0x83B4,0xC483,0x6383,0x5252,0x7363,0x7352,0x7352,0x7363,0x8352,0x73C4,0xD473,0x7393,0x9373,0xB483,0x8373,0x4373,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646
.word 0xAF62,0xAF62,0xAF62,0xAF62,0xAF62,0xE4E3,0xE4E4,0xE4F4,0xE4E4,0xD4E4,0xE4D3,0xE4D4,0xD4C3,0xD4E4,0xD4E4,0xC3D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4C3,0xE4D4,0xE4E3,0xF4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xF4F4,0xE3E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4D3,0xD4E4,0xE4E4,0xE4D4,0xC3E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62AF,0x9F62,0x4152,0xBF62,0x62C4,0xD4D4,0xD462,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4BF,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD562,0xD583,0xAF62,0x62C4,0x529F,0x418F,0x8F42,0x7F7F,0x8F42,0x8F41,0x9F41,0x9F52,0x9F52,0x9F52,0xD552,0x8F52,0x7F7F,0x7F7F,0x7F41,0x7F42,0x7F42,0x7F41,0x8F42,0xD462,0xD49F,0xD462,0x8FD4,0x7F42,0xD4E4,0x7FBF,0x8F52,0x7F41,0xD48F,0xD4E4,0xD4D4,0xE4E4,0xD462,0xD4D4,0xD4D4,0xD4D4,0x72D4,0xAFD4,0xAFC4,0x6262,0x529F,0xAF52,0x6273,0x52AF,0x52AF,0x5152,0x5252,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4246,0x4646,0x7346,0x8383,0x8383,0x5273,0x5252,0x7352,0x7383,0x9383,0x7373,0x5283,0x7373,0x6383,0xD483,0x8383,0x73D4,0x8383,0x4446,0x4246,0x4646,0x4646,0x4646,0x4246,0x4646,0x4642,0x465F,0x4446,0x4646,0x4642,0x4642,0x465F
.word 0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xE4F4,0xE4E3,0xE4F4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE3E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD472,0xAFC4,0x62C4,0xC4AF,0x62D5,0xD4D4,0xC4BF,0xD4D5,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x83BF,0x62D5,0xC4AF,0x73AF,0x529F,0x429F,0x4241,0x417F,0x8F42,0x8F42,0x8F42,0x5252,0x529F,0x7F8F,0x7F8F,0x7F42,0x7F41,0x417F,0x417F,0x417F,0x427F,0x8F7F,0x8F62,0x8F62,0x7F8F,0x8F52,0xD441,0xC4E4,0x41D4,0x8F52,0x7F62,0xD4D4,0xD4D4,0xD4D4,0x6262,0xE4E5,0xE4D4,0xD4D4,0xD4D4,0xC4BF,0xD4D4,0xD562,0x73AF,0x52AF,0x739F,0x739F,0x73AF,0xAF52,0xAF62,0x5252,0x5162,0x4642,0x4646,0x4646,0x465F,0x465F,0x4642,0x4646,0x4646,0x4646,0x465F,0xC452,0xD483,0x93B4,0x93C3,0x73C4,0x5252,0x7352,0x8373,0x8363,0x7373,0xD483,0x8352,0x7373,0x7373,0x8383,0x8373,0x4273,0x4646,0x465F,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4C3,0xD4E4,0xD4E4,0xE3D4,0xE3D4,0xE4E4,0xE3E4,0xE4F4,0xF4E4,0xE4F4,0xE4E4,0xE4F4,0xE4D3,0xE4E3,0xD4F4,0xE3E4,0xE4E4,0xE4E4,0xF4E4,0xE3E4,0xE4E4,0xE4D3,0xE4E3,0xE4E4,0xD3E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x73BF,0xC4AF,0xD4AF,0xD562,0xD483,0xD4BF,0xD4C4,0x72D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD562,0xD562,0xAF62,0xAF73,0x5252,0x528F,0x428F,0x7F8F,0x417F,0x8F7F,0x8F8F,0x7F42,0x7F41,0x7F41,0x417F,0x427F,0x427F,0x7F7F,0x7F42,0xAF62,0x6283,0xD4D5,0xC4C4,0x6262,0x42AF,0xE4D4,0x62D4,0x628F,0x42AF,0x62E4,0x62D4,0x7F9F,0xE4D4,0x628F,0xD47F,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x62D4,0x62D5,0xAF62,0x9F52,0xAF52,0xAF63,0xAF62,0x6263,0xD5AF,0x629F,0x4252,0x4646,0x465F,0x4646,0x4646,0x464F,0x4642,0x4646,0x465F,0x9364,0x83C4,0xB493,0xB493,0xD483,0x83B3,0x5273,0x5263,0x7363,0xC483,0x8373,0x8383,0x8373,0x73C4,0x8373,0x7363,0x7383,0x7383,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F
*/



/*   Dragon Girl
Fish_320:
.word 0x4646,0x4646,0x4646,0x4646,0x8575,0x8575,0x7575,0x4646,0x4646,0x4646,0x4646,0x4646,0x7544,0x5465,0x7575,0x4645,0x4646,0x4646,0x7565,0x5454,0x5465,0x6585,0x4646,0x6546,0x7554,0x8575,0x4646,0x4646,0x4646,0x4646,0x6544,0x7554,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6F46,0x7F6F,0x6F6F,0x7F6F,0x4141,0x8F44,0x6F7F,0x6F6F,0x6F6F,0x7F6F,0x7F6F,0x6F6F,0x5F6F,0x6F7F,0x5F7F,0x6F6F,0x6F6F,0x6F7F,0x6F7F,0x6F7F,0x6F6F,0x6F7F,0x6F7F,0x6F7F,0x7F6F,0x6F6F,0x6F6F,0x7F6F,0x7F6F,0x7F7F,0x7F7F,0x4646,0x6546,0x4665,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5444,0x8565,0xD4D4,0x4575,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6544,0x8585,0x95D4,0x7585,0x5465,0x4645,0x4646,0x4646
.word 0x4646,0x4644,0x4646,0x4644,0x4546,0x7595,0x7575,0x8585,0x4675,0x4644,0x4646,0x4646,0x5446,0x5475,0x5475,0x6475,0x4645,0x4544,0x7575,0x7554,0x7554,0x5465,0x7585,0x7554,0x7554,0x7554,0x4675,0x4446,0x4646,0x4444,0x7554,0x7575,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x6F5F,0x7F44,0x6F6F,0x447F,0x448F,0x7F8F,0x6F6F,0x7F6F,0x6F6F,0x6F7F,0x6F7F,0x6F6F,0x6F6F,0x7F6F,0x6F6F,0x6F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F7F,0x6F7F,0x7F7F,0x6F6F,0x7F7F,0x7F41,0x7F7F,0x467F,0x4544,0x54D4,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x7541,0x9585,0x95D4,0x5475,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4644,0x4446,0x4646,0x4444,0x8554,0xB5D4,0x75B4,0x4575,0x4646,0x4644,0x4644,0x465F
.word 0x4646,0x5F46,0x4646,0x4646,0x4646,0x5446,0x54C4,0x6554,0x7564,0x7575,0x4446,0x4446,0x7546,0x7554,0x7554,0x7554,0x5454,0x6554,0x5454,0x5454,0x5475,0x5475,0x5454,0x5475,0x7554,0x7554,0x6454,0x7575,0x7554,0x7554,0x5454,0x7554,0x4685,0x4646,0x465F,0x4646,0x465F,0x4646,0x444F,0x7F7F,0x7F7F,0x7F41,0x8F54,0x7F41,0x6F6F,0x6F7F,0x6F7F,0x7F6F,0x7F6F,0x6F6F,0x6F6F,0x7F6F,0x6F6F,0x446F,0x5441,0xD464,0x5485,0x7F54,0x6F6F,0x417F,0x447F,0x7F7F,0x7F44,0x447F,0x6F7F,0x6F7F,0x6F7F,0x6F6F,0x447F,0x417F,0x7F7F,0x4646,0xC554,0x4475,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5444,0x8575,0x95D4,0x5485,0x4645,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x465F,0x4446,0x8554,0x95C5,0x75B4,0x4475,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0x4446,0x4646,0x4446,0x5F46,0x4646,0x4646,0x8565,0x7575,0x7554,0x6475,0x7575,0x4646,0x5444,0x7554,0x5454,0x5465,0x5475,0x5475,0x5454,0x5454,0x5454,0x5454,0x6554,0x5454,0x5475,0x5454,0x5475,0x5475,0x5475,0x5454,0x7575,0x5475,0x5475,0x4646,0x4646,0x4646,0x4646,0x6F5F,0x4144,0x447F,0x7F7F,0x447F,0x7F41,0x6F7F,0x7F6F,0x7F6F,0x7F6F,0x446F,0x7F6F,0x7F6F,0x7F5F,0x6F44,0x6F6F,0x6441,0xD4D4,0xD4E4,0x9F64,0x6F7F,0x6F6F,0x5444,0x7F41,0x417F,0x7F6F,0x7F7F,0x447F,0x6F6F,0x7F7F,0x7F44,0x7F6F,0x6F41,0x417F,0x446F,0x7544,0x75D4,0x4644,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4446,0x4146,0x7544,0xB485,0xD4B5,0x8595,0x4554,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5446,0x4654,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4446,0x7554,0x9585,0x75B4,0x4475,0x4646,0x4644,0x5F46,0x4646,0x465F,0x4646
.word 0x4446,0x4545,0x4545,0x4545,0x4446,0x4646,0x4646,0x7575,0x7554,0x7554,0x5475,0x5475,0x7554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5454,0x5454,0x4475,0x4446,0x5F46,0x6F46,0x7F44,0x5454,0x6F7F,0x447F,0x5441,0x448F,0x7F6F,0x6F6F,0x447F,0x6F7F,0x6F7F,0x6F6F,0x6F6F,0x416F,0x6F6F,0x546F,0xD564,0xD4E4,0xAF64,0x447F,0x6F6F,0x4441,0x7F8F,0x7F6F,0x7F7F,0x6F7F,0x6F7F,0x6F7F,0x6F41,0x6F7F,0x6F6F,0x6F41,0x447F,0x417F,0x6F6F,0x457F,0x9554,0x6575,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4446,0x5444,0x8575,0x95C4,0x95D4,0x8585,0x4454,0x4446,0x4646,0x5F46,0x5446,0x6554,0x5465,0x4641,0x4644,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4446,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x465F,0x4446,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4446,0x5F46,0x4646,0x465F,0x4646,0x4444,0x8575,0x8585,0x8595,0x7595,0x7585,0x5454,0x4444,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4446,0x5454,0x8485,0xC495,0x7585,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7546,0x8585,0x8575,0x7575,0x8575,0x4485,0x4446,0x7575,0x5475,0x5475,0x5465,0x5454,0x5475,0x5454,0x5454,0x5454,0x5444,0x5454,0x5444,0x4454,0x4454,0x4454,0x5454,0x4454,0x5441,0x5444,0x5454,0x5454,0x5465,0x5454,0x5454,0x6554,0x5475,0x7575,0x4475,0x4646,0x416F,0x5441,0x7F9F,0x7F7F,0x4144,0x8F54,0x6F7F,0x6F7F,0x6F44,0x7F7F,0x447F,0x6F6F,0x6F7F,0x6F6F,0x6F6F,0x6F6F,0x5441,0x54AF,0x7F7F,0x7F7F,0x7F44,0x7F6F,0x6F6F,0x6F7F,0x7F7F,0x6F6F,0x6F6F,0x446F,0x7F6F,0x6F7F,0x6F7F,0x7F6F,0x7F6F,0x7F7F,0x7F41,0x4144,0x7F7F,0x5445,0x85D4,0x4665,0x4646,0x4646,0x465F,0x4446,0x4646,0x4646,0x4446,0x5F46,0x4646,0x465F,0x4146,0x6554,0x8575,0xC485,0xD4B5,0x95B5,0x8585,0x7585,0x7575,0x7575,0x7575,0x5454,0x5465,0x4444,0x4454,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x8554,0x9595,0x95C4,0x7585,0xB485,0x9585,0x9595,0x85B4,0x5454,0x5454,0x6575,0x5454,0x5454,0x8575,0x8585,0x9495,0x85C4,0x75B4,0x4675,0x4644,0x6F46,0x4646,0x465F,0x465F,0x4446,0x4646,0x4646
.word 0x4646,0x4646,0x8575,0x7575,0x6554,0x5454,0x6575,0x7554,0x5475,0x5454,0x5454,0x5454,0x5454,0x5444,0x5444,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x7541,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4454,0x5454,0x7554,0x7554,0x5454,0x7F44,0x4441,0x448F,0x447F,0x547F,0x9F41,0x7F8F,0x7F6F,0x6F7F,0x7F7F,0x6F41,0x7F7F,0x7F6F,0x5F6F,0x446F,0x7F6F,0x546F,0x547F,0x418F,0x4154,0x8F44,0x6F7F,0x7F6F,0x7F6F,0x416F,0x7F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F,0x7F44,0x6F6F,0x6F7F,0x6F7F,0x8F44,0x447F,0x7F41,0x467F,0x7554,0x6495,0x4654,0x465F,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x6541,0x5454,0x7554,0x7565,0x8575,0x95B4,0xD4C4,0xB595,0x9595,0xB5B5,0xB5D4,0x95D4,0x95C5,0x95C5,0x8585,0x6475,0x5454,0x4454,0x4454,0x4644,0x4646,0x4644,0x4646,0x5F46,0x4646,0x4644,0x5F46,0x4446,0x4646,0x4644,0x4646,0x4646,0x465F,0x4F46,0x4646,0x5F46,0x4646,0x465F,0x4646,0x465F,0x4644,0x465F,0x5F46,0x4646,0x4644,0x4446,0x8554,0xC4C5,0xB485,0x6575,0x5475,0x5454,0x6575,0x5454,0x8554,0x7585,0x7585,0x7575,0x8575,0x8575,0x8585,0x5454,0x5454,0x7575,0x4454,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646
.word 0x4446,0x4646,0x6546,0x8585,0x7554,0x5475,0x7575,0x7554,0x7554,0x5454,0x5454,0x5454,0x5444,0x5454,0x5454,0x5454,0x5454,0x5475,0x7554,0x5475,0x7554,0x5454,0x7554,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5454,0x5444,0x5441,0x5454,0x5454,0x4454,0x4141,0x8F41,0x8F44,0x7F41,0x8F54,0x8F54,0x6F7F,0x7F44,0x446F,0x7F7F,0x446F,0x6F7F,0x6F7F,0x446F,0x7F6F,0x6F6F,0x5454,0x44BF,0x9F64,0x9F54,0x6F8F,0x6F7F,0x6F7F,0x6F44,0x6F6F,0x6F6F,0x6F6F,0x446F,0x6F6F,0x6F6F,0x6F7F,0x416F,0x6F7F,0x446F,0x6F6F,0x417F,0x7F41,0x5444,0x7F8F,0x4446,0xC575,0x6575,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4441,0x5454,0x5454,0x5454,0x5454,0x8575,0xC485,0xC485,0x95B4,0x95C4,0x85C4,0xC495,0xC4E4,0xD4D4,0xD4B5,0x9584,0x85B5,0x7585,0x7575,0x5454,0x4454,0x4444,0x4644,0x4646,0x4646,0x4F46,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x8544,0x8595,0x85B4,0x7575,0x5454,0x4644,0x4646,0x4444,0x5454,0x6554,0x7565,0x5454,0x5475,0x5475,0x6554,0x6554,0x5454,0x4454,0x4646,0x4644,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4644,0x465F,0x4646,0x4646
.word 0x4646,0x5F46,0x4646,0x7565,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x5444,0x5454,0x5454,0x5454,0x7554,0x7554,0x7575,0x5454,0x7575,0x7575,0x5454,0x7575,0x7585,0x5475,0x7554,0x8585,0x5454,0x5454,0x5454,0x7554,0x5454,0x5454,0x4454,0x4154,0x417F,0x8F44,0x8F54,0x7F41,0x547F,0x449F,0x6F7F,0x6F44,0x7F7F,0x7F6F,0x7F41,0x6F7F,0x6F6F,0x446F,0x6F6F,0x6F41,0x856F,0x64D4,0xD564,0xC4AF,0x7F54,0x6F7F,0x6F7F,0x6F6F,0x7F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F,0x6F7F,0x6F6F,0x7F6F,0x447F,0x648F,0x4141,0x467F,0x6554,0x5454,0x5454,0x4644,0x4644,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4446,0x4646,0x4446,0x5444,0x5454,0x7554,0x7565,0xC485,0x8585,0xB485,0xB495,0x8585,0x8585,0x95B4,0xC4C4,0xB495,0x95C5,0xC5C5,0xB5C4,0xB584,0x8595,0x5475,0x5454,0x4144,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4446,0x4446,0x4646,0x465F,0x4446,0x5F46,0x4646,0x4644,0x5F46,0x4446,0x7585,0x7565,0x5475,0x5475,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646
.word 0x4446,0x9585,0x8585,0x7585,0x5475,0x7554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x8575,0x5475,0x5454,0x7554,0x7585,0x5454,0x5475,0x7585,0x5454,0x7575,0x5475,0x7575,0x7575,0x5465,0x8554,0x8585,0x5475,0x7554,0x7575,0x5454,0x7F41,0x9F54,0x5454,0x7F8F,0x7F44,0x5454,0x7F8F,0x7F7F,0x7F7F,0x447F,0x417F,0x7F6F,0x447F,0x7F6F,0x7F6F,0x7F41,0x6F7F,0x6454,0x64AF,0x64E5,0x64E4,0x7F7F,0x7F6F,0x6F6F,0x4144,0x7F7F,0x446F,0x7F5F,0x6F6F,0x6F6F,0x6F6F,0x5F41,0x6F6F,0x7F6F,0x446F,0x7F6F,0x6F7F,0x447F,0x7F7F,0x8F41,0x8FC4,0x6F41,0x5445,0x7554,0x9575,0x6575,0x4646,0x4646,0x4446,0x4446,0x5F46,0x4646,0x4646,0x4644,0x4646,0x5F46,0x4646,0x4646,0x4146,0x5444,0x7554,0x7575,0x7575,0x7575,0x7585,0x7585,0x85B4,0x8575,0x8585,0x85B4,0xB4C4,0xC495,0xC495,0xD495,0x95D4,0x95D4,0x8595,0x7575,0x5454,0x4654,0x465F,0x4446,0x5F46,0x4646,0x465F,0x4646,0x4644,0x4446,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x7544,0x7595,0x5475,0x7575,0x5454,0x7585,0x4454,0x5F46,0x4646,0x4644,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4446,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x465F,0x5F46,0x4646,0x4446,0x4646,0x465F
.word 0x8575,0x75B4,0xB485,0x6575,0x6564,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5475,0x7575,0x7575,0x5475,0x7565,0x7554,0x8575,0x7575,0x7554,0xC485,0x7585,0x8554,0x85D4,0x5465,0x8575,0x7585,0x6554,0x8575,0x5475,0x7554,0x7575,0x5475,0x448F,0x44D4,0x549F,0x7F54,0x5441,0x44AF,0x418F,0x446F,0x7F7F,0x7F41,0x7F7F,0x4144,0x6F6F,0x417F,0x417F,0x7F6F,0x417F,0xAF54,0xD454,0xF5D4,0x8F54,0x6F6F,0x6F7F,0x417F,0x7F5F,0x7F44,0x6F41,0x6F6F,0x6F6F,0x6F44,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F,0x6F6F,0x7F6F,0x447F,0x448F,0x418F,0x457F,0x5454,0x7565,0x8585,0x75B5,0x4654,0x5F46,0x7584,0x4646,0x4646,0x4644,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4446,0x5444,0x5454,0x6554,0x6564,0x5465,0x7575,0x7575,0x7575,0x75B4,0x8575,0x8585,0x85B4,0xC485,0xC485,0xC495,0xC485,0xC595,0xC595,0x8595,0x7575,0x4465,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4644,0x5F46,0x4646,0x4644,0x4646,0x4446,0x7565,0x5475,0x5475,0x7554,0x8575,0x8575,0x7585,0x7575,0x7575,0x4644,0x465F,0x4646,0x5F46,0x4446,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0x8585,0x7585,0x7575,0x5475,0x5475,0x5454,0x5454,0x5454,0x7554,0x5454,0x5475,0x7575,0x5454,0x7554,0x7554,0x8585,0x7554,0x7575,0x8575,0x6585,0x7575,0x8585,0x8565,0x8585,0x5475,0xC485,0xC485,0x5475,0xD485,0x5485,0x5454,0x7575,0x8F51,0xD441,0xC49F,0x649F,0x7F9F,0xD554,0x8F41,0x7F7F,0x7F6F,0x7F41,0x7F7F,0x7F44,0x6F7F,0x6F7F,0x7F7F,0x7F7F,0x447F,0x416F,0x5454,0xD4D5,0xAF64,0x6FC4,0x7F7F,0x446F,0x416F,0x416F,0x546F,0x7F7F,0x6F7F,0x6F6F,0x6F6F,0x6F6F,0x446F,0x7F5F,0x6F6F,0x6F6F,0x446F,0x416F,0x7F6F,0x6F41,0x7F7F,0x8F41,0x8F44,0x7F44,0x7575,0x8585,0x95C4,0xD485,0x7595,0x8565,0x8565,0x4445,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x466F,0x5444,0x5475,0x7554,0x7554,0x7554,0x5475,0x7575,0x7575,0x7575,0x7575,0x8575,0x8575,0x85B4,0xB485,0xB495,0xD495,0x85C4,0xB5C4,0x85D4,0x7585,0x4454,0x4646,0x4644,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4446,0x7585,0x5475,0x5475,0x8575,0xB485,0x7585,0x8585,0x7575,0x7554,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x465F,0x4446,0x4446,0x4646,0x464F,0x5F46,0x4646
.word 0xB485,0x8554,0x6585,0x6564,0x5454,0x5454,0x6554,0x5454,0x5475,0x8575,0x8554,0x8585,0xC485,0x5475,0x7554,0xB485,0x7595,0x7575,0xC485,0x7585,0x7554,0x95C4,0x5485,0x9565,0x75C4,0x7565,0x8575,0x7585,0x8575,0x75B4,0x7575,0x6454,0x449F,0xD564,0xD464,0x8FE4,0x5444,0x549F,0x447F,0x7F41,0x7F44,0x447F,0x417F,0x417F,0x7F6F,0x7F44,0x447F,0x447F,0x6F7F,0x8F41,0x648F,0x64AF,0x41D5,0x6F41,0x446F,0x417F,0x7F7F,0x7F54,0x7F7F,0x7F44,0x6F41,0x6F7F,0x6F6F,0x6F41,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F6F,0x6F7F,0x7F7F,0x7F54,0x4141,0x4144,0x5454,0x8575,0x95C5,0x8585,0x7585,0x7575,0x4485,0x5F46,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4446,0x4446,0x5454,0x5475,0x5475,0x5475,0x5475,0x7554,0x7554,0x7554,0x7575,0xB485,0x7585,0x8575,0xB495,0x8585,0xB495,0xB485,0xB495,0x8585,0x7595,0x4475,0x4646,0x465F,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x5F46,0x4646,0x5F44,0x4646,0x4446,0x6446,0x7575,0x7554,0x7554,0x7585,0x7575,0x8585,0x6575,0x4654,0x4644,0x4646,0x4446,0x4446,0x4646,0x465F,0x4646,0x4646,0x4644,0x4F46,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x464F,0x4646,0x4646,0x4646
.word 0x7575,0x7575,0x5454,0x5475,0x5454,0x5454,0x7575,0x5475,0x7554,0x5454,0x7565,0x7575,0x6585,0x9575,0x7565,0x8575,0x7575,0x5475,0x8575,0xC485,0x7554,0xB485,0x7554,0x8554,0x95D4,0x7554,0xD485,0x5495,0x6575,0x5485,0x8575,0x5475,0x648F,0x54BF,0x64AF,0x448F,0x649F,0x7F9F,0x7F41,0x7F7F,0x417F,0x417F,0x7F7F,0x6F44,0x7F7F,0x417F,0x7F7F,0x7F7F,0x6F7F,0x4144,0xAF54,0xAF75,0xAFC4,0x7F7F,0x6F7F,0x7F6F,0x547F,0x5464,0x417F,0xE441,0x6F7F,0x7F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x446F,0x6F6F,0x6F7F,0x6F41,0x6F7F,0x6F44,0x6F7F,0x6F7F,0x4441,0x7F8F,0x657F,0x5475,0x7544,0x5475,0x8565,0x7565,0x7575,0x7595,0x4645,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4446,0x5454,0x5475,0x5454,0x5475,0x5475,0x5454,0x7565,0x5475,0x7575,0x7575,0x7575,0x8575,0x85B4,0x85B4,0x8585,0x6575,0xD485,0x8585,0x7585,0x4645,0x4646,0x4446,0x4646,0x464F,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x5446,0x7575,0x7585,0x7575,0x85C4,0x8575,0x7575,0x4475,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4644,0x4644,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4446,0x4646,0x4644
.word 0x8585,0x7565,0x5454,0x5454,0x5454,0x5475,0x7554,0x7575,0x7575,0x8585,0x5475,0x7554,0x8554,0xB5D4,0x95C5,0x7575,0x8575,0x95C5,0x5485,0x85B4,0x75B5,0x7575,0xD485,0x5475,0x8575,0x5475,0xB485,0x5485,0x8575,0x54D4,0xC475,0x549F,0x54E4,0x648F,0x7FD5,0x9F54,0x8F64,0x447F,0x448F,0x417F,0x447F,0x417F,0x7F7F,0x7F6F,0x447F,0x7F7F,0x7F7F,0x4441,0x416F,0x447F,0x759F,0xC4AF,0x5464,0x6F6F,0x7F41,0x4154,0x9FC4,0xD4D4,0x6FD4,0x447F,0x7F41,0x6F44,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F6F,0x6F6F,0x6F7F,0x6F7F,0x6F44,0x6F6F,0x446F,0x7F6F,0x8F6F,0x8F44,0x4441,0xC485,0x5485,0x5454,0x7554,0x8575,0x8595,0x85B4,0x7585,0x4665,0x465F,0x5F46,0x4646,0x4646,0x4446,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x5441,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x7554,0x7554,0x7554,0x7575,0x6575,0x8585,0xB485,0x7565,0x7575,0x7565,0x7585,0x5465,0x4454,0x4446,0x4646,0x464F,0x4646,0x4644,0x465F,0x4646,0x465F,0x4646,0x4646,0x4446,0x5F46,0x7546,0x5454,0x5465,0x8575,0x7575,0xC485,0x7585,0x5454,0x4644,0x465F,0x4646,0x4644,0x465F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4F46,0x4646,0x4646
.word 0x5475,0x5475,0x5454,0x5454,0x7554,0x7575,0x5475,0x5454,0x7575,0xB485,0xC495,0x6585,0x8575,0xD4B4,0x8585,0x5475,0x8565,0x7585,0x5465,0x8575,0x7575,0x7554,0x8585,0x5465,0xD485,0x6595,0x7554,0x5475,0x8585,0x7595,0xD454,0xAF54,0x8F44,0x9F54,0x5441,0xC49F,0x7F54,0x9F44,0x7F7F,0x7F41,0x7F44,0x7F41,0x7F44,0x7F6F,0x7F41,0x447F,0x7F41,0x7F7F,0x7F6F,0x417F,0x9F54,0xAF75,0x6FD4,0x7F6F,0x6F7F,0x4154,0xF564,0x54D4,0x54D4,0x8F7F,0x7F7F,0x416F,0x6F6F,0x446F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x446F,0x6F7F,0x6F6F,0x7F6F,0x7F6F,0x7F6F,0x6F6F,0x547F,0x7F7F,0x9575,0x95C4,0x6585,0x5454,0x7575,0xC485,0x8585,0x95B4,0x6595,0x4646,0x4646,0x4446,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x5446,0x6585,0x7575,0x7585,0x7554,0x5475,0x5454,0x5454,0x5454,0x5475,0x5454,0x7554,0x7554,0x6465,0x8554,0x7554,0x7585,0x5465,0x8575,0x7585,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x7546,0x7575,0x7554,0x7585,0x7554,0x8585,0x85B4,0x7554,0x5475,0x5465,0x4454,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x5F46,0x4646
.word 0x5465,0x5454,0x5454,0x5454,0x7554,0x8575,0x8585,0x7575,0x5475,0x7565,0x7585,0x8554,0x5475,0x7575,0x7575,0xB5C4,0x5485,0x8554,0xB5C4,0x5475,0xC485,0x85D4,0x7554,0x7595,0x7575,0x7585,0x8575,0x85D4,0x7565,0x7565,0xAF54,0x8F64,0x4441,0x448F,0x549F,0xAF64,0x448F,0x449F,0x7F41,0x417F,0x417F,0x7F7F,0x6F7F,0x4441,0x417F,0x417F,0x446F,0x6F41,0x447F,0x8F41,0x8F54,0x5454,0x7FAF,0x6F6F,0x546F,0x547F,0xD49F,0x9F64,0x8FD4,0x6F41,0x7F7F,0x7F7F,0x7F6F,0x6F7F,0x6F41,0x6F44,0x6F7F,0x447F,0x6F6F,0x417F,0x6F6F,0x6F41,0x7F6F,0x7F6F,0x7F7F,0x416F,0x8F44,0x8554,0xC485,0x8575,0x7585,0x6554,0x7575,0xB485,0x7575,0x9585,0x6485,0x4445,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4446,0x6465,0x6554,0x6554,0x5454,0x5475,0x5475,0x5454,0x5485,0x8565,0x5475,0x5475,0x5465,0x5454,0x6575,0x5475,0x7554,0x9585,0xC5D4,0x95D4,0x9585,0x4654,0x4446,0x5F46,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5444,0x6465,0x7554,0x7575,0x7575,0x7575,0x5475,0x7575,0x5475,0x8575,0x7575,0x8575,0x7585,0x4664,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F46,0x4446,0x4646,0x4644,0x4646,0x465F,0x5F46,0x4F46,0x4646,0x4646,0x4646
.word 0x5454,0x5454,0x5454,0x7554,0x5475,0x8575,0x7575,0x8554,0x95D4,0x5475,0x5454,0xC485,0xD495,0x8595,0x7554,0xC485,0x85C4,0x7565,0x7585,0x5475,0x8585,0x6575,0x7554,0x85C4,0x5465,0x5475,0x7575,0xC485,0x8575,0x5495,0x54D4,0x448F,0x548F,0x548F,0x9F54,0x4454,0x648F,0x8F54,0x447F,0x447F,0x447F,0x7F7F,0x447F,0x7F7F,0x4441,0x7F7F,0x8F54,0x6F7F,0x7F7F,0x8F54,0x8F54,0xD554,0x6F6F,0x6F41,0x7F6F,0x9F54,0x5454,0x54AF,0xD554,0x447F,0x7F6F,0x446F,0x6F7F,0x446F,0x7F5F,0x7F6F,0x446F,0x7F5F,0x6F6F,0x7F6F,0x446F,0x7F6F,0x6F6F,0x6F6F,0x446F,0x7F6F,0x417F,0x7544,0x85C4,0x7585,0xB485,0x5475,0x5454,0x8575,0x8585,0xB475,0x8585,0x9585,0x7564,0x4575,0x4446,0x5F46,0x4646,0x465F,0x4446,0x5F46,0x4646,0x5446,0x7575,0x5475,0x8554,0x5465,0x7554,0x5454,0x5454,0x6575,0x7575,0x5475,0x7554,0x5454,0x5454,0x5454,0x5454,0x6554,0x5475,0xB485,0xC485,0xC4C5,0x5495,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x465F,0x4446,0x4646,0x465F,0x6565,0x5464,0x5454,0x7575,0x7554,0x7575,0x7575,0x5454,0x7585,0x6575,0x6564,0x6554,0x7575,0x4644,0x4644,0x5F46,0x4646,0x4644,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x464F,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F
.word 0x5454,0x5454,0x5475,0x8575,0x85C4,0x5465,0x7565,0x7554,0x8585,0x85C4,0x6575,0x8575,0x75B4,0x9565,0x75D4,0x7554,0x8554,0x95C5,0x8565,0xD485,0x5465,0x8585,0x5495,0x7565,0x9575,0x95C5,0x6575,0x7575,0x8554,0x9F65,0x41D4,0x448F,0x8F7F,0xAF64,0x8F54,0x548F,0x7F9F,0x7F8F,0x7F41,0x417F,0x7F7F,0x7F7F,0x7F44,0x7F41,0x8F44,0x4154,0x8F54,0x447F,0x7F41,0x7F54,0x548F,0xD49F,0x7F6F,0x446F,0x6F41,0x4441,0x549F,0x9F64,0x547F,0x6F7F,0x6F41,0x5F41,0x6F41,0x7F6F,0x6F7F,0x416F,0x7F5F,0x7F7F,0x6F6F,0x6F7F,0x7F41,0x7F7F,0x6F44,0x6F6F,0x7F6F,0x7F6F,0x7F41,0x756F,0x9585,0x85B4,0x8585,0xD485,0x6585,0x5454,0xB475,0x7575,0xB485,0x8585,0x8585,0x8585,0x7585,0xB485,0x8595,0x85B4,0x6585,0x7575,0x7575,0x5475,0x5475,0x7554,0x7554,0x7575,0x7554,0x8575,0x7565,0x5475,0x5475,0x5454,0x7575,0x7554,0x5475,0x7554,0x5475,0x5454,0x5454,0x5475,0x8575,0x7585,0x7575,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x7546,0x9585,0xD4D4,0x7585,0x5475,0x5454,0x5475,0x7554,0x7554,0x7575,0x5475,0x7575,0x7575,0x4554,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x4646,0x465F,0x4646
.word 0x5454,0x5454,0x7554,0x8575,0x5475,0x8585,0x8585,0x5485,0x7575,0x6575,0x8585,0x6585,0x5454,0x8585,0x85C4,0x7565,0x8554,0xB485,0x6585,0x9585,0x5475,0x8565,0x75B4,0x8585,0x8554,0x85B4,0x7554,0x85B5,0x6554,0xC454,0x549F,0x7F8F,0x6441,0x549F,0x548F,0x8F64,0x5475,0x7F41,0x7F44,0x8F54,0x7F44,0x447F,0x417F,0x8F44,0x8F54,0x648F,0x7F8F,0x416F,0x417F,0x447F,0x448F,0x548F,0x6F6F,0x7F41,0x447F,0x8F6F,0x7F41,0x448F,0x7F9F,0x5441,0x7F6F,0x7F6F,0x416F,0x7F6F,0x7F44,0x6F6F,0x7F41,0x4441,0x417F,0x446F,0x7F54,0x7F7F,0x6F6F,0x7F7F,0x7F6F,0x6F7F,0x7F7F,0x657F,0x8575,0xC495,0x75B4,0x7575,0x9585,0x5475,0x5454,0x7575,0x8575,0x7575,0x85B4,0x85B4,0x85B4,0x9595,0xB5C4,0x95A5,0x95C5,0x85C5,0x85C5,0x9585,0x8585,0x85C4,0x8585,0x7575,0x7575,0x5454,0x7575,0x5475,0x7554,0x5485,0x5454,0x5475,0x5454,0x7575,0x5485,0x8554,0x5454,0x5454,0x6554,0x5465,0x5475,0x4646,0x4446,0x4646,0x4644,0x4646,0x465F,0x4446,0x4646,0x4646,0x8575,0xC5B5,0x95D4,0xD4B5,0x8595,0x7575,0x5454,0x5475,0x8575,0x7575,0x8575,0x8575,0x7585,0x4645,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F46,0x4446,0x4646,0x4646,0x4446,0x4646,0x464F,0x4646,0x464F,0x4646,0x4646
.word 0x5454,0x8554,0x5465,0x7554,0x7575,0xB485,0x7585,0x8575,0x85C4,0x5475,0xB485,0x8595,0x5475,0x7575,0x7575,0xC595,0x6575,0x7575,0x8595,0x5465,0xC585,0x5485,0x5454,0xB485,0x5485,0x6554,0x7554,0x95C4,0x75C4,0x6464,0x7F9F,0x547F,0xAF54,0x8F54,0x8F54,0xAF54,0x8F44,0x7F7F,0x417F,0x7F8F,0x6F41,0x7F41,0x4154,0x548F,0x549F,0x9F64,0x6F44,0x4441,0x7F7F,0x417F,0x7F41,0x6F54,0x7F7F,0x7F6F,0x7F7F,0x6F7F,0x447F,0x417F,0x5444,0x449F,0x7F6F,0x4154,0x7F6F,0x7F44,0x7F7F,0x7F41,0x7F44,0x417F,0x4144,0x7F6F,0x7F7F,0x6F44,0x417F,0x446F,0x447F,0x7F6F,0x446F,0x417F,0xC475,0x95B4,0x8585,0x7575,0xB485,0xC485,0x5475,0x5454,0x5475,0x8575,0x7575,0x8585,0xB485,0xB495,0xB485,0x85C4,0x85B4,0x85C4,0x95B4,0x85B4,0xB5C4,0xC5C4,0xD4C4,0xC595,0xD495,0xB5D4,0x8585,0x7575,0x7554,0x7554,0x7585,0x5454,0x5454,0x5454,0x5475,0x5475,0x5475,0xB485,0x5454,0x5454,0x4654,0x4446,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x455F,0x8575,0xB585,0x85C4,0xC485,0x8585,0x85C4,0xB5D4,0x5485,0x7554,0x6554,0x7575,0x7575,0x5475,0x4554,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x4646,0x465F,0x4644,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646
.word 0x6554,0x8575,0x8575,0x5475,0x5454,0x7585,0x6585,0xC485,0xC4C5,0x5475,0x7575,0x6575,0xD485,0x5485,0x7554,0x8585,0x75D4,0x7554,0xB485,0x5454,0xC485,0xB485,0x7595,0x8565,0x6575,0x9595,0x6465,0xB475,0x5485,0x9FD4,0x7F44,0x8F44,0x5454,0x448F,0x64D4,0x548F,0x449F,0x448F,0x417F,0x447F,0x447F,0x448F,0x549F,0x548F,0x8F54,0x8F54,0x7F7F,0x8F44,0x7F7F,0x7F44,0x7F7F,0x4164,0x4144,0x7F7F,0x547F,0x5454,0x6F6F,0x7F41,0x548F,0x419F,0x4441,0x5441,0x8F54,0x7F41,0x7F41,0x7F44,0x416F,0x447F,0x449F,0x7F8F,0x7F41,0x7F7F,0x6F7F,0x6F6F,0x6F7F,0x447F,0x7F7F,0x6F7F,0x8585,0x8575,0x95B4,0x8575,0x7575,0x8585,0x8585,0x5465,0x7554,0x7554,0x8575,0xB485,0xB495,0x8585,0x8585,0x8585,0xB485,0xB495,0xB495,0xB495,0xB485,0x8585,0x8585,0x85B4,0x95C4,0xD495,0xD4D4,0xC495,0x7585,0x7565,0x5475,0x8554,0x6575,0x5454,0x5454,0x5454,0x7554,0x7554,0x7554,0x7575,0x5441,0x4444,0x4646,0x4646,0x4646,0x4646,0x4644,0x945F,0x8585,0xB585,0xB485,0xB495,0xB485,0xB485,0xB485,0x9585,0x85D4,0x5465,0x7554,0x7575,0x8585,0x7565,0x5454,0x4454,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4644,0x4446,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446
.word 0x5454,0x7575,0x8575,0x8585,0x6575,0x5454,0x5454,0x8585,0x75B4,0xD485,0x5485,0x5475,0x8575,0x7585,0x5465,0x7554,0x9554,0x5485,0x6465,0xC485,0x6575,0x5475,0x8575,0x7554,0x5475,0x7575,0x7585,0x7554,0xAF54,0x4154,0x548F,0x547F,0x8F9F,0xD444,0x8F9F,0x64C4,0x418F,0x417F,0x8F54,0x7F7F,0x4141,0xAF54,0x5454,0x9F64,0xD554,0x7F8F,0x7F44,0x7F41,0x7F44,0x7F41,0x6454,0xC4AF,0x648F,0x5444,0x7FC4,0x8F51,0x5454,0xC49F,0x9F54,0x54C4,0xAFD4,0x9FC4,0xC4D4,0xC441,0x547F,0x7F54,0x7F64,0x7F7F,0x7F41,0x7F7F,0x6F44,0x447F,0x7F7F,0x7F54,0x7F6F,0x7F7F,0x7F6F,0x6F44,0x8575,0x85B4,0xB485,0xB485,0x8575,0x6575,0x7575,0x8585,0x5465,0x5475,0x5475,0x6575,0x7575,0x7565,0x7565,0x6575,0x8575,0x8575,0x8585,0x7575,0x8585,0x75B4,0x85B4,0x8575,0xB485,0xC485,0x8595,0x85C4,0x8585,0x8585,0x7575,0x6575,0x7554,0x7575,0x5475,0x6554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x6444,0x5454,0x6454,0x5444,0x8575,0x9585,0xB495,0xC485,0xB495,0x8585,0x8575,0x8585,0x8585,0x85B4,0xB5C4,0x7585,0x8554,0x7585,0x7554,0x8575,0x85C5,0x7575,0x5475,0x4454,0x4646,0x4646,0x4644,0x5F46,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x5F46,0x4446,0x4446,0x4646,0x4F46,0x4646
.word 0x5475,0x5475,0x7565,0x8575,0x8565,0xD495,0x7585,0x7565,0x7575,0x8585,0x85D4,0x7565,0x6554,0x5475,0xC595,0x7595,0xB454,0x7585,0x5475,0x8575,0x5475,0x5485,0x6575,0x7575,0x5485,0x5465,0x5454,0x8575,0x6454,0x449F,0x7F8F,0x4441,0x448F,0xE49F,0x41D4,0x7F9F,0x8F44,0x8F44,0x7F44,0x4144,0x547F,0x75AF,0x549F,0x8FD5,0x6454,0x7F41,0x7F6F,0x7F6F,0x8F41,0x7F7F,0xD564,0x64D4,0x75AF,0x549F,0xD48F,0x6454,0x54AF,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF4F4,0xE4E4,0xAF64,0x9F85,0x8F64,0x647F,0x4144,0x7F8F,0x7F44,0x7F7F,0x7F7F,0x7F7F,0x416F,0x7F6F,0x7F6F,0x7F6F,0x7F6F,0x9554,0x7585,0x8585,0x8585,0x7585,0x7585,0x5475,0x5454,0x5475,0x5475,0x5454,0x6465,0x7554,0x5454,0x5475,0x7554,0x7575,0x8575,0x75B4,0x8585,0x8575,0x8575,0x8585,0x85B4,0x8585,0x85B4,0x85B4,0xB495,0xC475,0xC495,0xC4B5,0x7585,0x7554,0x7585,0x5454,0x5475,0x7554,0x7575,0x5454,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5475,0x9585,0xB495,0xB495,0xB485,0x8585,0x8575,0x75B4,0x7575,0x7575,0x7575,0xC485,0x7595,0x6554,0x7575,0x8554,0xC485,0xB4B5,0x8585,0x7585,0x5454,0x4646,0x465F,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4F46,0x4446,0x464F,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646
.word 0x9575,0x7585,0x7554,0x6554,0xB464,0xB485,0x8575,0x94D4,0x7585,0x7565,0x7575,0x9585,0x95C4,0x5475,0x7575,0x8565,0x6585,0x7554,0x7575,0x7554,0x5475,0x6575,0x7554,0x7554,0x5475,0x7575,0x8575,0x5454,0x44AF,0x8F54,0x447F,0x8F7F,0x8F41,0x85E4,0xC48F,0x5464,0x548F,0x449F,0x648F,0x448F,0x64D4,0x9F64,0xBF75,0xD454,0x44AF,0x7F8F,0x7F7F,0x7F44,0x447F,0x647F,0xD454,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4AF,0xD4D4,0xE4E4,0xE4E4,0xF4E4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xD4D4,0x64C4,0x8F64,0x448F,0x8F54,0x7F41,0x7F7F,0x447F,0x7F6F,0x6F6F,0x7F44,0x416F,0x7F6F,0x416F,0x756F,0xC485,0x5475,0x7565,0x95C4,0x85C4,0x7575,0x5475,0x5454,0x8575,0xB5C4,0x85D4,0x5465,0x7575,0x8585,0x8585,0x7565,0x7575,0x7575,0x8575,0x75B4,0x8575,0x8575,0x7585,0x85B4,0xC585,0x85D4,0x7575,0x7554,0x8575,0x95C4,0x85C4,0x6585,0x7554,0x7585,0x7554,0x7575,0x5475,0x8575,0x85C4,0x7565,0x8575,0x5485,0x7575,0x6585,0x8554,0xD495,0xD4C5,0xB495,0x8595,0x8575,0x85B4,0x8585,0xB485,0x8585,0x8585,0x85B4,0x75B4,0x7554,0x7595,0x8575,0x85B4,0x7585,0x75D4,0x4554,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xB485,0x7585,0xD475,0x7585,0x7554,0x7585,0x8554,0xC485,0x7585,0x7585,0x5454,0x8575,0x7585,0x8575,0x5465,0x8575,0x7575,0x5475,0x7554,0x5454,0x6575,0x5454,0x5475,0x5475,0x5454,0x5475,0x7565,0x4154,0x9F44,0x7F8F,0x8F44,0x7F41,0x4144,0x9F64,0x8F54,0x448F,0x449F,0x4141,0x8FC4,0x6441,0xD4D5,0x9FD4,0xD4C4,0x548F,0x9F64,0x6F44,0x7F41,0x7F7F,0x7F41,0x9F54,0xBF64,0xD4C4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xE4F4,0xE4F4,0xE4D4,0xE4E4,0xD4E4,0xAFE4,0x5444,0x8F7F,0x548F,0x4154,0x6F44,0x6F6F,0x7F6F,0x6F44,0x7F7F,0x7F6F,0x446F,0x6F6F,0x547F,0x8575,0x75D4,0x5454,0x8575,0xB485,0x7575,0x5475,0x95C4,0xB5C4,0xC5D4,0x95D4,0x7575,0x7575,0x7585,0x7565,0x5475,0x8575,0x8585,0x7575,0x8585,0xC475,0x7585,0x8575,0xD485,0x85B4,0x7575,0xD4D4,0x5475,0x5444,0x5454,0x4454,0x5454,0x5454,0x5454,0x4454,0x5454,0x5444,0x7554,0x7554,0x5454,0x5454,0x6554,0x6564,0x5454,0xC485,0xC4C5,0xC5E4,0x95C4,0x85C4,0x85B4,0x8575,0x85B4,0x7575,0x7585,0x85B4,0x7575,0x5454,0x8575,0x85D4,0x95C4,0xB485,0xC495,0x4575,0x4646,0x4644,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4644,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4646
.word 0x7554,0x5475,0x8585,0x85D4,0x9575,0x85D4,0x6575,0x7575,0x6575,0xD485,0x5495,0x5465,0x6554,0x7554,0x6475,0x6554,0x7554,0x7575,0x5454,0x5454,0x5475,0x5475,0x7554,0x5454,0x5454,0x5454,0x7554,0x9F54,0x4144,0x4144,0x417F,0x447F,0x649F,0x448F,0x759F,0x419F,0x9F64,0xD444,0x449F,0xE49F,0xAF64,0xE4C4,0x64E4,0xD4D5,0x8F64,0x7F7F,0x6F6F,0x7F41,0x7F7F,0xD564,0xD564,0xD474,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4F4,0xF5F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xC4E4,0x64AF,0x54D5,0x8F54,0x7F41,0x7F7F,0x7F41,0x7F41,0x6F7F,0x6F7F,0x6F44,0x7F6F,0x6F6F,0x5444,0x5454,0x7575,0x6585,0x7554,0x7585,0x7575,0x5475,0x8575,0xD4D4,0xD495,0xB495,0x5475,0x5475,0x5454,0x7554,0x5475,0x5454,0x7554,0xB485,0x7575,0x8585,0x8575,0x85C4,0x85B4,0xB565,0x5465,0x7554,0x9585,0x7575,0xC575,0x8585,0x5475,0x8585,0x8585,0x85C4,0x8585,0x8585,0x7575,0x7575,0x7575,0x7575,0x7575,0x8585,0x8585,0x95D4,0xB5D4,0x95C4,0x85C4,0x85B4,0x8575,0xB485,0x8585,0x7585,0x7575,0x7575,0x7585,0x5454,0x7554,0x8585,0xB495,0x5485,0x4645,0x4446,0x4646,0x5F46,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4F46,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x5F44
.word 0x7554,0x5454,0x8575,0x6585,0xC485,0xC4B5,0x5475,0x5475,0x7554,0x8585,0x75B4,0x8585,0x7585,0x5475,0x7575,0x64C4,0x7554,0x7554,0x5454,0x5454,0x5454,0x5454,0x4454,0x5441,0x4454,0x4454,0x5454,0x448F,0x8F8F,0x448F,0x447F,0x417F,0xF664,0x548F,0x449F,0x8F41,0x8F54,0x9F64,0x8F44,0x64C4,0xF4D4,0xD464,0x8FE4,0xAF54,0x7F64,0x447F,0x6F7F,0x446F,0x547F,0xBF64,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE5E4,0xF4F4,0xF5F4,0xF4F4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xBF64,0x8F64,0x8F44,0x7F44,0x7F6F,0x7F6F,0x7F6F,0x6F44,0x7F6F,0x7F6F,0x6F6F,0x8554,0x6585,0x5454,0x5454,0x5454,0x8575,0x7585,0x7575,0x7554,0xC495,0x7575,0x7554,0x7554,0x6554,0x7554,0x5485,0x5475,0xB485,0x6575,0x7554,0x7575,0x8575,0xC475,0xB495,0x7554,0x5485,0x8554,0x94D4,0xD4D4,0x7575,0xD495,0xD4D4,0x7585,0xC454,0xD495,0xD4B5,0xD4C5,0x95D4,0x95C4,0xB5C4,0x95C4,0xC5D4,0xC5E4,0x85B4,0x95B4,0xC4C5,0xD4C5,0x85D4,0x85C4,0x8575,0x7585,0x7585,0xB485,0x7585,0x8585,0x8575,0x5475,0x5454,0x7575,0x8565,0x5475,0x4645,0x4446,0x4646,0x465F,0x4646,0x5F46,0x4446,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4644,0x465F,0x4646,0x4646
.word 0xD485,0x7595,0x5475,0x5454,0x7565,0x5475,0x8575,0xC585,0x5485,0x7554,0x5454,0x7575,0x7575,0x7575,0x7554,0x5454,0x5454,0x5454,0x5454,0x6554,0x5454,0x5475,0x5454,0x5475,0x5454,0x5454,0x4454,0x8F41,0x4144,0x7F7F,0x7F41,0x5441,0x44AF,0x648F,0x8F64,0x547F,0x548F,0xC4AF,0x6441,0x9FD4,0xD564,0x64D4,0x549F,0xE4E4,0x7F41,0x7F7F,0x417F,0x7F7F,0x6441,0xD4D5,0xD464,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D5,0xF4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xD4C5,0xD5E4,0xE4E4,0xD4D4,0xE4E4,0xC4D4,0x759F,0x418F,0x417F,0x447F,0x6F6F,0x6F6F,0x7F6F,0x7F41,0x7F6F,0x6F6F,0xB46F,0x7585,0x8575,0xC495,0x7585,0x6554,0x8585,0x8554,0x6585,0x8575,0x7565,0x8585,0x7565,0x9575,0xB485,0x7575,0x8554,0x5475,0x8554,0x8585,0x5475,0x6554,0x6585,0x5475,0x7554,0x8585,0x85C4,0xB485,0x9595,0x5475,0x95C4,0xC495,0x6585,0x9554,0xD4C4,0x95D4,0xC5D4,0xD4D4,0xC4C5,0xE495,0xD495,0xD4C5,0x8585,0xC495,0xE495,0x95C4,0x95D4,0x85C4,0x8575,0x75B4,0xB485,0xB485,0x8585,0x7575,0x7585,0x6575,0x7554,0x6554,0x9585,0x6585,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4644,0x4644
.word 0xC485,0x5485,0xD485,0x95C5,0x5454,0x5454,0x8575,0x85B4,0x8565,0xD4D4,0x7585,0x5454,0x5465,0x5475,0x5454,0x5454,0x5454,0x5454,0x5475,0x5454,0x5454,0x5454,0x4475,0x4454,0x5454,0x5454,0x8F54,0x4441,0x448F,0x417F,0x7F7F,0xAF44,0x41E4,0x9F44,0x8F44,0x8F44,0x6444,0x9F54,0xD58F,0x6454,0x64D5,0xC4AF,0x649F,0xAF64,0x8F44,0x6F44,0x6F6F,0x7F41,0xAF64,0xD4C4,0xD4BF,0xD4D4,0xD4E4,0xC5E4,0xE4E4,0xE4E4,0xE4E5,0xF4F5,0xF4F4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xC5D4,0xD4D4,0xD4D4,0xAF64,0x5454,0x7F8F,0x6F6F,0x7F41,0x4144,0x6F7F,0x446F,0x6F6F,0x6F6F,0x857F,0x7585,0x8575,0x85B4,0x85C4,0x5454,0xB454,0x8575,0x5475,0x7585,0x8595,0x85B4,0x8585,0x6585,0x9575,0x85E4,0x5475,0x7575,0x8585,0x75B4,0x7554,0x7585,0x5485,0x5475,0x6585,0x5454,0x6575,0x7575,0x8575,0x7575,0xB485,0x7575,0x5454,0xC475,0xC595,0xC5E4,0x95C4,0x95C4,0x95C4,0x95D4,0xC5C4,0xD4C4,0xC495,0xC5D4,0xB5C5,0x95C4,0x95C4,0x75B4,0x8575,0x8585,0x85B4,0x8585,0x7575,0x8575,0x8575,0x5465,0x5454,0x8554,0xC475,0x7585,0x4446,0x4646,0x4646,0x4646,0x5F46,0x4446,0x4646,0x465F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4644,0x4646,0x4446,0x4F46,0x4646
.word 0x7565,0x5454,0x8575,0xB485,0xD475,0xC495,0x5454,0x5475,0x8575,0x7585,0x7575,0x5485,0x5475,0x5454,0x5454,0x7554,0x5454,0x5454,0x5454,0x5454,0x5454,0x5444,0x4446,0x4446,0x4446,0x4144,0x7F8F,0x8F41,0x7F41,0x447F,0x417F,0x6454,0x8F54,0x9F54,0x8F54,0x4441,0x9F8F,0x8F54,0xD444,0x649F,0x549F,0x44AF,0xF554,0x8FC4,0x6F44,0x6F7F,0x6F41,0x447F,0xD564,0xD464,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xC4D4,0x648F,0x447F,0x7F7F,0x6F6F,0x7F41,0x4144,0x6F6F,0x6F6F,0x6F6F,0x756F,0x7575,0x7575,0x8554,0x8585,0x85D4,0x5465,0x8554,0x7554,0x8575,0x7565,0x8585,0xB475,0x5485,0x7575,0x7554,0xE485,0x85D4,0x7575,0x7554,0x7575,0x7575,0xC595,0x8585,0x8585,0x85C4,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x8575,0x8585,0x85C4,0x95B4,0xD4C4,0xD4B5,0xE495,0xC4C5,0x95E4,0xC5C4,0xB5C4,0xC5E4,0x65C4,0xC485,0x5475,0x7575,0x7585,0xB485,0x7585,0x7575,0x7575,0x7575,0x5454,0x5454,0x7554,0xB485,0xD485,0x85B5,0x4654,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x4644,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646
.word 0x5475,0x7575,0x6575,0x7575,0x8554,0x85B4,0xC485,0x8595,0x7554,0x7575,0x7554,0x5475,0x5454,0x5454,0x5475,0x5454,0x5454,0x5454,0x4444,0x4646,0x4446,0x6F46,0x4646,0x465F,0x4646,0x4646,0x7F41,0x7F44,0x7F44,0x7F41,0x447F,0xD4D5,0x548F,0x448F,0x448F,0x8F7F,0x8F44,0x7F44,0x6441,0x759F,0x549F,0x9F54,0xAF64,0x8F64,0x7F7F,0x7F7F,0x447F,0x647F,0xD4AF,0xD4D4,0xD4D4,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4E4,0xD4C5,0xD4E4,0xD4E4,0xD4D4,0x64D4,0x649F,0x7FAF,0x7F44,0x6F41,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x756F,0x8575,0x8575,0x7575,0x85B4,0xC485,0x5485,0x5454,0x5454,0x8565,0x5475,0x6575,0x7575,0x5475,0x5475,0x6575,0x8554,0xD495,0xD4D4,0x7585,0x7554,0xE485,0xC495,0xC485,0xC495,0xD4B5,0x8595,0x7585,0x5475,0x5475,0x5475,0xC485,0xD4B5,0xD4C5,0xE4C5,0xD4C5,0xD4C5,0xD495,0xC4C5,0xC495,0xD4B5,0xD4C5,0xD495,0xC4C5,0x5475,0x7585,0x7554,0x8575,0x7585,0x8585,0x7575,0x5454,0x5454,0x5454,0x5454,0x7554,0x8585,0x95C4,0xB485,0xD485,0x7585,0x4645,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4644,0x465F,0x4646,0x464F,0x4646,0x4644
.word 0xC495,0x5485,0x8585,0x5465,0x5475,0x5475,0x7565,0x6575,0x8585,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4646,0x4446,0x6F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x416F,0x448F,0x417F,0x447F,0x518F,0x549F,0x548F,0x419F,0x7F54,0x7F41,0x8F41,0x7F41,0x8F54,0x9F44,0x9F44,0x8F54,0xC4E4,0x448F,0x416F,0x6F7F,0x7F6F,0xD541,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4D5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0x64D4,0x75AF,0x7F54,0x7F41,0x7F6F,0x6F7F,0x6F7F,0x7F7F,0x6F7F,0x6F6F,0x757F,0x7585,0x7585,0x5465,0x7575,0x8585,0x95C4,0x65C4,0x5454,0x8575,0x7565,0x7575,0x5475,0x8575,0x6554,0x7575,0x6575,0x8554,0x9595,0xD4D4,0xD4B5,0xB4B5,0xB485,0xC4B5,0xD4B5,0xB5D4,0xC5C5,0xD495,0xC495,0xC485,0x9585,0x95C4,0x95C4,0x95C4,0x95B4,0x95C4,0x95D4,0xD4C4,0xC495,0xB5E4,0xD4D4,0xC5D4,0xC5D4,0x7595,0x8554,0x5475,0x5475,0xB485,0x7585,0x8575,0x7565,0x5475,0x4454,0x5454,0x7554,0x7585,0x8585,0x7585,0x9585,0x95B4,0x85C4,0x7585,0x4644,0x4646,0x465F,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646
.word 0x8585,0x7565,0x95C4,0xC495,0x8554,0x8585,0x5454,0x5454,0x7585,0x5475,0x5454,0x5454,0x5475,0x5454,0x4454,0x4446,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4646,0x7F46,0x7F41,0x8F44,0x7F41,0x7F7F,0x7541,0x54AF,0x448F,0x448F,0x8F8F,0x7F44,0x8F44,0x447F,0x449F,0x549F,0x448F,0x648F,0xD454,0x7F41,0x7F6F,0x6F44,0x7F41,0xD454,0xBF74,0x64D4,0xD4E4,0xD4C5,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xF4D5,0xE4E4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4C5,0xD4D4,0xD4D4,0x9F64,0x8F64,0x447F,0x6F6F,0x447F,0x6F7F,0x6F6F,0x6F44,0x446F,0x956F,0x75C4,0x7565,0x7585,0x5475,0x7575,0x7565,0x7554,0x5475,0x8554,0x7575,0x5454,0x7575,0x5475,0x7554,0x7554,0x7575,0x6575,0xB485,0xC495,0xB485,0xC485,0xB495,0xB495,0xB5D4,0xD4D4,0xD4B5,0xD495,0x85D4,0xE4B5,0xD4D4,0xD4B4,0xC4C5,0xC4C5,0xC4C5,0xD4C5,0xE495,0xE4C5,0xD4C5,0xD495,0x95D4,0xD4C5,0xD495,0x6595,0x8575,0x7575,0x8554,0x7585,0xB485,0x6575,0x7564,0x7585,0x7554,0x7554,0x7575,0x7565,0x6565,0x7575,0x75B4,0x7575,0x7585,0x7585,0x4554,0x4646,0x4646,0x5F46,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F46,0x4446,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4646
.word 0x6554,0x5475,0xB485,0x8585,0x8565,0x7575,0x8575,0x6585,0x6554,0x5454,0x5454,0x5454,0x5454,0x4644,0x6F46,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x445F,0x417F,0x7F8F,0x447F,0x7F41,0x8F54,0x8F54,0x9F54,0x8F44,0x7F7F,0x417F,0x417F,0x417F,0x8F44,0x4441,0x418F,0xD544,0x54AF,0x7F8F,0x447F,0x7F6F,0x446F,0xD4D4,0x85D5,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xF4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD5E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x64D5,0x41D5,0x7F7F,0x447F,0x7F7F,0x7F7F,0x417F,0x7F6F,0x6F6F,0x856F,0x7585,0x7585,0x5485,0x7554,0x7585,0x8575,0x8565,0x5454,0x5454,0x7575,0x5454,0x7554,0x7554,0x5454,0x7575,0x7585,0x7575,0x7575,0x7575,0x7575,0x7575,0x7575,0x8585,0xD4D4,0xE4B5,0x95C4,0xC5E4,0xC5D4,0x95C4,0xB5D4,0xB5C4,0xD4D4,0x95D4,0xD4D4,0x95D4,0xC4D4,0xD495,0xC495,0xC5D4,0x95D4,0x85D4,0xC5B4,0x5475,0x7554,0x5475,0xC475,0x8575,0x7575,0x5454,0x7575,0x5475,0x8575,0xD495,0x7585,0x4446,0x4446,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4644,0x465F,0x465F,0x464F,0x4F46,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x5F46,0x4646
.word 0xD495,0x8585,0x5465,0x5475,0x5465,0x5475,0x7554,0x5475,0x6475,0x5454,0x5454,0x5454,0x4644,0x4644,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4644,0x4646,0x7F41,0x447F,0x4441,0x7F7F,0x447F,0x549F,0x9F64,0x8F54,0x7F41,0x4144,0x447F,0x7F7F,0x8F44,0x8F41,0x8F44,0x8F44,0x5441,0x549F,0x6F8F,0x6F6F,0x7F6F,0x547F,0xC4BF,0xD474,0xD4D4,0xD4D4,0xE4D4,0xC5E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xD5E4,0xE4E4,0xE4E4,0xE4D4,0xE4C5,0xE4D4,0xD4C5,0xD4D4,0xBF74,0x5464,0x7F54,0x7F6F,0x7F7F,0x6F44,0x6F6F,0x6F7F,0x7F6F,0x6541,0x5475,0x5475,0x7575,0x5454,0x8575,0x7585,0x8575,0x7585,0x5454,0x5454,0x8585,0x7575,0x7575,0x8575,0x7585,0x8585,0x5485,0x7575,0x7554,0x8585,0x8585,0xC485,0xB485,0xB495,0xD495,0xC4B5,0xD4C5,0xB5D4,0xC5D4,0x95D4,0xD4D4,0xD4B5,0xC495,0xC5C5,0xC5E4,0xC5E4,0xC5E4,0xC5E4,0xD4D4,0xB4B5,0x8575,0x75D4,0x6554,0x8575,0x5475,0x8575,0xB485,0x5454,0x5454,0x7585,0x5475,0xB485,0xC495,0x75B4,0x4646,0x4646,0x465F,0x4646,0x465F,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x5F46,0x4646,0x4646,0x4446
.word 0x8585,0x75B4,0x8575,0x8585,0x7575,0x8575,0x6585,0x7554,0x5454,0x5454,0x5454,0x4654,0x4644,0x465F,0x5F46,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4446,0x5F46,0x4646,0x465F,0x416F,0x417F,0x7F7F,0x7F41,0x5441,0x649F,0x449F,0x448F,0x7F8F,0x7F7F,0x7F41,0x7F7F,0x8F41,0x8F44,0x8F41,0x448F,0x51D5,0x648F,0x6F7F,0x7F7F,0x6F7F,0x6441,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D5,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xAFD5,0x4154,0x7F7F,0x6F7F,0x416F,0x6F6F,0x7F6F,0x446F,0x756F,0x7575,0x5454,0x5444,0x5454,0x7554,0x75C4,0x7585,0xB485,0x7585,0x5454,0x5454,0x5454,0x7554,0x5475,0x6575,0x7575,0x6575,0x7554,0x7575,0x7565,0x7565,0x5475,0x8575,0xD495,0x95D4,0xD4C4,0xB495,0xD485,0xD4D4,0xD4C5,0xD4C5,0xB5D4,0x85C4,0x85C4,0x85B4,0x85B4,0x85B4,0x75B4,0x7575,0xE485,0xB495,0x5454,0x5475,0xC454,0x5485,0x8575,0x8575,0x7575,0x8575,0x7575,0x7554,0xC485,0xB495,0x85D4,0x4644,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4446,0x5444,0x5454,0x5454,0x4454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4644,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4446,0x4646
.word 0x7575,0x5475,0x9585,0xD4C4,0x5485,0xB485,0x7585,0x5454,0x6554,0x5454,0x4454,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x5F46,0x4144,0x447F,0x7F7F,0x447F,0x417F,0xAF54,0x7F44,0x8F54,0x4144,0x7F7F,0x447F,0x447F,0x447F,0x7F7F,0x7F44,0x4141,0xAF64,0x9F44,0x7F6F,0x6F6F,0x446F,0xD541,0xD574,0xD474,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4C5,0xD5D4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x4164,0x7F9F,0x4144,0x416F,0x7F6F,0x6F44,0x6F7F,0x7F41,0x8544,0x7575,0xD485,0x8595,0x7585,0x5454,0x8575,0x7575,0x7575,0x8575,0x7585,0x4475,0x5454,0x7575,0x8575,0x95C4,0x8585,0x7554,0x8585,0x8575,0x8585,0x8585,0xB485,0xB495,0x7565,0x8585,0xC495,0xC5C5,0xB4B5,0xD495,0xB5D4,0x95C4,0x95C4,0x95D4,0x85B4,0x7575,0x7575,0x8575,0x8585,0xC495,0x7595,0x5465,0x5454,0x5454,0x8575,0x54B4,0x8565,0x85B4,0x8585,0x75C4,0x5475,0x8554,0x94C4,0x95B4,0xB495,0x6585,0x4646,0x4446,0x5F46,0x4646,0x4644,0x4446,0x6544,0x5454,0x7575,0x8585,0x9595,0xD4C5,0x9595,0x95B5,0xB595,0xC5B5,0xB595,0x9595,0x7575,0x6475,0x5454,0x7454,0x4446,0x4646,0x4646,0x465F
.word 0x95D4,0x8595,0xB475,0xB495,0x6585,0x7575,0x5454,0x5454,0x5475,0x4454,0x6F46,0x4646,0x4646,0x4446,0x5F46,0x4646,0x5F46,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4644,0x7F46,0x7F7F,0x7F41,0x7F41,0x7F41,0x5441,0xC4AF,0x547F,0x449F,0x7F7F,0x447F,0x7F7F,0x417F,0x418F,0x417F,0x7F7F,0x9F44,0x5454,0x448F,0x6F7F,0x6F6F,0x7F6F,0xD564,0xC4BF,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xD474,0x54D5,0x7F64,0x7F41,0x6F7F,0x7F7F,0x7F6F,0x446F,0x7F7F,0x7575,0x7575,0x8575,0x85C4,0x95B4,0x75C4,0x5454,0x9575,0x7585,0x6575,0x7575,0x7585,0x7554,0x8585,0x5475,0x7554,0x7575,0x8575,0x6575,0x5475,0x6575,0x7575,0x9585,0x95C4,0xC495,0x7595,0x7575,0x8554,0x85B4,0x7575,0x7575,0xB485,0xB495,0xC485,0x8585,0x9585,0xC5C4,0x95C4,0x85B4,0x5465,0x5454,0xC495,0xC495,0x5475,0x7575,0x8585,0x5465,0x7575,0x7585,0x5465,0x6554,0xC485,0xC495,0xB495,0x9585,0x85C4,0x5475,0x4646,0x4646,0x5444,0x8565,0x8585,0xC595,0xB595,0xD495,0xB495,0xB485,0x8585,0x6575,0x6575,0x7575,0x6565,0x7575,0x7565,0x8585,0x8585,0x8585,0x8585,0x9595,0x7575,0x5454,0x4665
.word 0xB485,0xB485,0x7554,0x7554,0x5454,0x7554,0x5454,0x6554,0x4454,0x4454,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x444F,0x7F41,0x7F44,0x7F7F,0x447F,0x447F,0xAF64,0x547F,0x8F54,0x7F41,0x7F7F,0x6F7F,0x7F7F,0x7F44,0x7F7F,0x8F44,0x6441,0x449F,0x418F,0x6F6F,0x447F,0x416F,0x64C4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xD5D4,0xE4E4,0xD4E4,0xD4E4,0xE4C5,0xE4D4,0xD4D4,0xAFD4,0x8F64,0x8F44,0x7F44,0x6F44,0x6F7F,0x6F7F,0x6F41,0x7565,0x7575,0x8575,0x9585,0x95C4,0x85D4,0x5475,0x5444,0x9575,0x7575,0x8575,0x5465,0x7554,0x7575,0x8575,0x5475,0x5475,0x5454,0x5475,0x5475,0x8575,0x7585,0x85B4,0xB485,0x8585,0x85B4,0x95C4,0x85D4,0x7585,0x8585,0xB485,0x7585,0x7575,0x7575,0xB485,0xC485,0xC485,0x7575,0x5454,0x4641,0x5444,0x95E4,0x95B4,0x5475,0x7575,0x8575,0x8585,0x5465,0x5454,0x7554,0x8585,0x8585,0x85B4,0x8585,0x85C4,0x7585,0x7585,0x5454,0x7554,0x8575,0xB595,0x95D4,0x85C4,0x75B4,0x7575,0x5454,0x5454,0x6554,0x4444,0x4446,0x4446,0x4446,0x4446,0x4646,0x5444,0x6554,0x7554,0x7575,0x7575,0x8585,0x8585,0x7585
.word 0x5454,0x5465,0x8575,0x95C4,0x95C4,0x7565,0x6554,0x6564,0x4154,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F,0x5F46,0x4446,0x465F,0x4446,0x4646,0x465F,0x4646,0x4646,0x5F46,0x7F44,0x417F,0x417F,0x447F,0x7F7F,0x9F41,0x8F75,0x9F44,0x7F8F,0x7F7F,0x7F44,0x7F41,0x7F44,0x7F41,0x7F44,0x417F,0x9F44,0x8F7F,0x7F7F,0x7F6F,0x6F6F,0x547F,0xBFD4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4C5,0xE4D4,0xE4E4,0xD5E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4C5,0xE4D4,0xD4D4,0x64D4,0xC4D4,0x44AF,0x7F8F,0x7F7F,0x6F6F,0x6F7F,0x417F,0x447F,0x8595,0x9595,0x85D4,0x75B4,0xC485,0x8585,0x95B4,0x7585,0x5454,0x7554,0x5475,0x5475,0x5454,0x7575,0x7575,0x5465,0x8575,0xB485,0x7554,0x7575,0x5475,0x7554,0x7585,0x8585,0x5475,0x4646,0x4444,0x6554,0x8585,0xB475,0x7585,0x7585,0x8585,0x8585,0x7575,0x5475,0x4654,0x4644,0x4646,0x6F46,0x4446,0xD4D4,0xB495,0x4454,0x5475,0x8554,0x95B4,0x95E4,0x85C4,0x7585,0xB485,0x8585,0xC485,0x95B4,0x85B4,0x95B4,0x95D4,0xC495,0x9595,0xB495,0xB485,0x7554,0x7565,0x5454,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4646,0x465F,0x5F46,0x4446,0x4646,0x4644,0x4646,0x4446,0x4646,0x4446,0x5454,0x7565
.word 0x95D4,0x95C5,0x7554,0x8585,0xB485,0x5464,0x5454,0x5454,0x4654,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4644,0x4446,0x4646,0x416F,0x447F,0x7F7F,0x7F7F,0x7F41,0x6454,0x7FD5,0x4441,0x447F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x417F,0x417F,0x5444,0x7F7F,0x6F44,0x6F6F,0x7F6F,0x6441,0x64D5,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD5D4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x64D4,0x51D5,0x7F7F,0x6F6F,0x6F7F,0x6F6F,0x7F44,0x7541,0xE4D4,0xF4D4,0xC485,0x5485,0x7575,0x8565,0xB485,0x6585,0x5475,0x7575,0x8585,0x5475,0x7554,0x5475,0x7575,0x8575,0x7575,0x7554,0x6475,0x7585,0x5475,0x8565,0xE4C5,0x85B5,0x4665,0x466F,0x4646,0x4446,0x4444,0x4454,0x4454,0x5454,0x6575,0x6575,0x5454,0x4646,0x4644,0x4646,0x465F,0x4646,0x5446,0x9595,0x5475,0x4644,0x4644,0x6554,0x8575,0xD4C4,0xD4B5,0xC495,0xD4C5,0xC5B5,0xC595,0xA5D4,0xC4C5,0x8585,0x7575,0x8575,0xB485,0x7565,0x5454,0x5454,0x4646,0x4446,0x6F46,0x4646,0x5F44,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x465F,0x465F,0x4446,0x4646
.word 0xC485,0xB485,0x5475,0x7575,0x7575,0x5454,0x5454,0x4454,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F46,0x4646,0x6F46,0x7F41,0x417F,0x7F7F,0x6F44,0x7F7F,0xAF54,0x7F54,0x418F,0x8F41,0x417F,0x7F44,0x6F6F,0x7F44,0x7F7F,0x7F6F,0x8F44,0x8F54,0x7F7F,0x6F7F,0x6F7F,0x446F,0xD454,0xC4BF,0x64D4,0xE4D4,0xD4D4,0xE4C5,0xE4E4,0xD4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xD5D4,0xE4E4,0xD4E4,0xD4C5,0xD4D4,0xBFD4,0x54C4,0x4154,0x447F,0x6F7F,0x6F7F,0x6F41,0xE47F,0xD4E4,0xD4D4,0x75E4,0x8565,0x5444,0x7585,0x8585,0x95C4,0x7585,0x7575,0x7575,0x5475,0x5454,0x7575,0x7585,0x8575,0x7575,0x7575,0x7575,0x5465,0x8554,0xD495,0x95D4,0x5475,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4444,0x8575,0x8585,0x5475,0x4641,0x4644,0x5F46,0x4646,0x4646,0x4644,0x8546,0x5475,0x4646,0x4646,0x5F46,0x4646,0x5444,0x7575,0x7554,0x6575,0x6575,0x6575,0x6575,0x6575,0x7575,0x7565,0x5465,0x5454,0x5454,0x4446,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x465F
.word 0xB485,0x8585,0x7554,0x6554,0x7575,0x5454,0x5454,0x4154,0x4646,0x4646,0x465F,0x4644,0x4446,0x4446,0x4646,0x4646,0x4446,0x5F46,0x4646,0x465F,0x4644,0x4646,0x5F46,0x6F46,0x7F41,0x7F41,0x7F7F,0x417F,0x7F44,0x5454,0x419F,0x8F44,0x4144,0x7F7F,0x7F7F,0x6F7F,0x7F7F,0x6F44,0x6F7F,0x4441,0x7F9F,0x7F7F,0x446F,0x6F6F,0x417F,0xD464,0xD4C4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E5,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0x74D4,0xD4D4,0xBFD4,0x8F54,0x6F44,0x7F7F,0x7F6F,0x6F7F,0xF444,0xD4D4,0xD451,0x54F4,0x5454,0x5475,0xD454,0x8575,0x7575,0x7575,0x8554,0x8585,0x5475,0x5454,0x5485,0x8575,0x7554,0x8585,0x8585,0x8585,0x5454,0xD485,0x95D4,0x7585,0x4654,0x4644,0x4646,0x4646,0x4644,0x465F,0x4644,0x5444,0xE4C5,0x85D4,0x4475,0x4646,0x5F46,0x4646,0x4646,0x4644,0x5F46,0x5446,0x4645,0x5F46,0x4446,0x4646,0x4646,0x4646,0x4644,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0x4654,0x4644,0x4446,0x4646,0x4646,0x466F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4446,0x4646,0x465F,0x4446,0x4646,0x4644,0x4646,0x4646,0x4644,0x5F46,0x4646,0x4644,0x4646,0x465F,0x4446,0x4646,0x4646
.word 0x5465,0x7575,0x5485,0x8575,0x7585,0x6554,0x5454,0x4654,0x4446,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x416F,0x4641,0x7F6F,0x7F44,0x7F7F,0x7F41,0x9F41,0x7F54,0x448F,0x7F8F,0x7F44,0x447F,0x7F6F,0x7F44,0x6F7F,0x7F7F,0x8F41,0x7F44,0x7F6F,0x7F6F,0x6F7F,0x547F,0xD4D5,0xD4BF,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D5,0xD4E4,0xE4D5,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0x85BF,0x64D4,0x9F64,0x7F6F,0x7F6F,0x446F,0x7F6F,0xE47F,0x6454,0xD49F,0x85E4,0x8585,0x6585,0x7554,0xB485,0x7554,0x7575,0x7575,0x85B4,0x7575,0x5454,0x6575,0x8554,0x6585,0x6575,0x8565,0x5475,0x6554,0x5454,0x5454,0x4554,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4646,0x4646,0x8544,0x6585,0x4454,0x4446,0x5F46,0x4646,0x4646,0x4644,0x4F46,0x4646,0x4646,0x4654,0x4646,0x4646,0x4646,0x465F,0x4644,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x6F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4644,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4644
.word 0x8554,0xB495,0xB495,0x8575,0x75B4,0x5454,0x5454,0x4654,0x4646,0x465F,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4646,0x464F,0x4646,0x465F,0x4446,0x4646,0x4646,0x4644,0x417F,0x6F7F,0x7F44,0x7F7F,0x417F,0x447F,0x548F,0x7F54,0x8F41,0x7F44,0x7F7F,0x6F7F,0x6F7F,0x7F7F,0x6F7F,0x447F,0x447F,0x7F41,0x6F6F,0x6F6F,0x7F6F,0x6444,0xC4BF,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xE4D5,0xD5E4,0xE4E4,0xC5E4,0xD4E4,0xD4D4,0xD4D4,0xD5D4,0x54AF,0x7F41,0x7F44,0x6F6F,0x7F7F,0x5441,0x649F,0xE464,0x95E4,0x85B4,0x85D4,0x5454,0x8554,0x7585,0x7575,0x7575,0x7575,0x7575,0x4475,0x7554,0x5475,0x6465,0x8575,0x7585,0x5475,0x8575,0x6585,0x7554,0x7585,0x4464,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x5454,0x4645,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4446,0x5465,0x4644,0x4644,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x6F46,0x4646,0x466F,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4644,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646
.word 0x7554,0x8585,0x85B4,0x7554,0x7554,0x5475,0x5454,0x4644,0x466F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x465F,0x4646,0x416F,0x7F7F,0x7F7F,0x7F41,0x7F7F,0x8F44,0x8F44,0x7F41,0x8F44,0x7F7F,0x7F41,0x6F7F,0x6F44,0x6F41,0x6F44,0x6F6F,0x4141,0x7F8F,0x6F7F,0x446F,0x7F6F,0xD564,0xD474,0xD4D4,0xD474,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4C5,0xE4D4,0xE4D4,0xD4D4,0xE4D5,0xD4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x64D4,0x64D4,0x9FC4,0x8F54,0x7F6F,0x7F6F,0x6F41,0x5454,0xD5AF,0xE464,0x85E4,0x8575,0x85C4,0x75C4,0x6554,0x8585,0x7585,0x7554,0x7575,0x7585,0x8575,0x5454,0x5454,0x5454,0x5475,0x7565,0x8575,0xB485,0x5475,0x5475,0x8575,0x84C4,0x7585,0x5464,0x4645,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4446,0x5F46,0x4646,0x4644,0x465F,0x4646,0x4646,0x4446,0x7544,0x7575,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F,0x4446,0x4F46,0x4646,0x5F46,0x4446,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4644,0x465F,0x4646,0x4646,0x4646
.word 0x6585,0x7585,0x8585,0x8565,0x5485,0x5475,0x5454,0x4646,0x4646,0x4646,0x4644,0x465F,0x4644,0x5F46,0x4646,0x465F,0x4644,0x4646,0x5F46,0x4646,0x465F,0x4646,0x5F46,0x7F44,0x7F44,0x7F41,0x447F,0x416F,0x547F,0x549F,0x7F8F,0x4441,0x7F7F,0x7F7F,0x6F44,0x6F7F,0x7F7F,0x7F6F,0x7F7F,0x8F44,0x6F44,0x6F44,0x6F7F,0x7F6F,0xD464,0xC4AF,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4C5,0xD4D4,0xC5D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xC5D4,0xD4D4,0xD4D4,0xC4BF,0x85BF,0x449F,0x7F7F,0x6F6F,0x4144,0x9F54,0xD4C4,0xC4AF,0x75F4,0x7575,0x8575,0x8585,0x5475,0x6554,0xC485,0x8595,0x8575,0x8554,0xD485,0x7585,0x5475,0x4454,0x7554,0x7585,0xB485,0x7575,0x7575,0x5475,0x7554,0x8554,0x9585,0x95A5,0x7585,0x6575,0x4445,0x4444,0x4444,0x4444,0x6544,0x8575,0x7585,0x7575,0x75A4,0x7564,0x4644,0x4646,0x5454,0x7575,0x4664,0x465F,0x4646,0x465F,0x4446,0x4646,0x4646,0x4644,0x4644,0x4446,0x5F46,0x4646,0x4646,0x4446,0x4646,0x4446,0x4454,0x5F46,0x4446,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4446,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x5F46
.word 0x8585,0x6575,0x7554,0x7554,0x5475,0x5454,0x5454,0x4446,0x4646,0x4644,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4146,0x7F7F,0x6F44,0x447F,0x7F7F,0x7F46,0x4441,0x8F54,0x7F54,0x418F,0x7F7F,0x6F44,0x6F7F,0x6F41,0x447F,0x7F6F,0x416F,0x547F,0x6F7F,0x6F7F,0x6F7F,0x547F,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4C5,0x5464,0x449F,0x549F,0xC4BF,0xD564,0xD564,0x5464,0x7F7F,0x7F44,0x7F7F,0xD564,0xD464,0xE464,0x95C4,0x8585,0x7575,0xB485,0x8585,0x6585,0x5454,0x5465,0x8575,0x75B4,0x5475,0xD485,0xC4B5,0x5485,0x5454,0x8575,0x7575,0x7575,0x5475,0x5454,0x7575,0x7575,0x7565,0x7554,0x5465,0x5475,0x7554,0x7575,0x7575,0x7575,0x7575,0x6575,0x7575,0x7575,0x5464,0x5465,0x9595,0x7564,0x5454,0x5454,0x5454,0x4646,0x465F,0x4646,0x4F46,0x4646,0x655F,0x5475,0x4646,0x464F,0x4646,0x4646,0x4446,0x6544,0x7575,0xA595,0x6585,0x4646,0x4646,0x465F,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646
.word 0xB485,0x7554,0x8585,0x7575,0x5475,0x5454,0x5454,0x4646,0x465F,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F46,0x4446,0x4646,0x6F46,0x7F41,0x7F46,0x7F7F,0x6F41,0x7F44,0x9F44,0x8F54,0x7F44,0x8F44,0x7F7F,0x7F41,0x6F7F,0x7F7F,0x7F7F,0x416F,0x416F,0x547F,0x6F7F,0x6F7F,0x6F7F,0x5441,0xC4BF,0x85BF,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4E4,0x64D4,0x9F54,0x6F8F,0x6F6F,0x7F41,0x5441,0xC4AF,0x64BF,0x54AF,0x7F7F,0x417F,0x547F,0xD4D4,0xE4AF,0xE4D4,0x5485,0xB475,0xC4B5,0x9585,0xD4D4,0xC4B5,0x5485,0x8575,0x95C4,0x7575,0x8575,0x8575,0x95C4,0x54C4,0x7554,0x7575,0x8575,0x7554,0x8585,0x7585,0x6554,0x7554,0x7554,0x7554,0x6585,0x7585,0x7575,0x7575,0x7585,0x7575,0x8575,0x7585,0x7554,0x7575,0x7575,0x5475,0x5475,0x8575,0x85A5,0x5475,0x4454,0x4646,0x4646,0x4644,0x4646,0x4646,0x8565,0x9584,0x4645,0x7565,0x8554,0x74B5,0xA574,0xA595,0x8585,0x8585,0x5485,0x4446,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4644,0x4646,0x465F,0x4646,0x4646,0x465F
.word 0x7575,0x8575,0x8575,0x75B4,0x5475,0x5454,0x5454,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4644,0x6F46,0x7F41,0x6F46,0x7F41,0x7F7F,0x7F46,0x4441,0x449F,0x7F8F,0x4154,0x7F44,0x7F6F,0x6F44,0x6F41,0x6F44,0x7F7F,0x7F7F,0x7F44,0x6F7F,0x7F6F,0x446F,0x6F7F,0x6F6F,0x417F,0x549F,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4E4,0x54D4,0x419F,0x6F41,0x7F6F,0x446F,0xD4C4,0xE5D4,0xD4D4,0xD4D4,0x64D4,0x6464,0x54AF,0x7F7F,0x7F7F,0x6464,0xD4BF,0xE485,0xD4D4,0x95C4,0x75C4,0x5454,0x4454,0x6575,0x8585,0xC485,0xC495,0x7585,0x8575,0x8575,0x95B4,0xC485,0x5485,0x5454,0x7585,0x7575,0x5475,0x7575,0x5475,0x5454,0x7554,0x8575,0x85C4,0x7585,0x8585,0x9585,0x8585,0x7575,0x7575,0x8585,0x85B4,0x7585,0x7575,0x7585,0x7585,0x7575,0x5475,0x5475,0x7575,0x9585,0x6485,0x5475,0x4644,0x4644,0x7544,0x8575,0x75B4,0x9585,0x7554,0x5465,0x5475,0x5454,0x5454,0x5454,0x4645,0x4446,0x4646,0x5F46,0x4446,0x4646,0x4646,0x464F,0x4646,0x465F,0x5F46,0x4646,0x4446,0x464F,0x4644,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646
.word 0x95E4,0x5475,0x7575,0x8575,0x5485,0x5454,0x5454,0x4446,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F46,0x4646,0x445F,0x7F7F,0x6F44,0x417F,0x6F6F,0x445F,0x548F,0x419F,0x7F7F,0x9F54,0x7F7F,0x7F44,0x6F7F,0x6F7F,0x6F7F,0x446F,0x416F,0x417F,0x6F6F,0x447F,0x7F6F,0x64C4,0x7F64,0x6F7F,0x6F6F,0x6F7F,0x8F7F,0xD454,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0x8F54,0x7F6F,0x7F41,0x7F7F,0x647F,0xE4D4,0xE4E4,0xE4E5,0xE5E5,0xE5D4,0xD4D4,0x54AF,0x54D5,0x447F,0x4441,0xC4AF,0xD464,0xD4AF,0xE4E4,0xC495,0xC585,0xC5D4,0x7575,0x5454,0xD475,0x8585,0x8585,0x7575,0x7575,0x8575,0xB485,0x8575,0x7575,0x5444,0x8575,0x7575,0x7554,0x5475,0x8554,0x7575,0x5485,0x7554,0x8575,0x7575,0xB485,0x8575,0x7565,0x8585,0x8585,0x7575,0x5475,0x8575,0x8585,0xD495,0xC495,0x5485,0x5475,0x7575,0x5475,0x6465,0x7565,0x9585,0x85B5,0x8595,0x7575,0x9575,0x7595,0x5465,0x5454,0x5475,0x4454,0x4646,0x4646,0x4646,0x4644,0x465F,0x5F46,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4644,0x465F,0x5F46,0x4446,0x4646,0x464F,0x4646
.word 0xB485,0x7554,0x7554,0x8585,0x54B4,0x5454,0x5454,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4446,0x5F46,0x4646,0x4446,0x4646,0x465F,0x4646,0x4646,0x7F6F,0x7F41,0x445F,0x7F7F,0x4644,0x7F46,0x4441,0x8F54,0x7F7F,0x4154,0x7F7F,0x7F7F,0x6F7F,0x6F44,0x6F41,0x6F7F,0x7F7F,0x7F44,0x7F6F,0x7F6F,0x416F,0xAF54,0x5454,0x64D4,0x8F54,0x6F6F,0x6F7F,0x7F7F,0x6F7F,0x6451,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0x4151,0x6F6F,0x6F6F,0x6F7F,0x446F,0x8554,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xAF85,0x8F64,0xAF54,0x9F64,0x417F,0x648F,0xD4D5,0xE4D4,0xE464,0xD4D4,0x9585,0x85C4,0x8565,0x7585,0x7585,0x5454,0x7565,0x9575,0x95C5,0x85C4,0x7575,0x8585,0x7585,0x8565,0x4475,0x7554,0x8585,0x7575,0x5475,0x8575,0x8565,0x7585,0x5475,0x7554,0x7565,0xC495,0x8575,0x7575,0xB485,0xB495,0x8575,0x7575,0x7575,0xB485,0x7575,0x7575,0x7585,0x7575,0x8585,0x7575,0x7575,0x5485,0x6565,0x7575,0x7585,0x5475,0x8575,0x95B4,0xC495,0x85B5,0x7585,0x7585,0x4654,0x466F,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4644
.word 0x7575,0x9575,0x75C4,0x7554,0x5475,0x5465,0x5454,0x5F46,0x4646,0x4644,0x5F46,0x4646,0x4644,0x4F46,0x4646,0x4646,0x4446,0x4646,0x465F,0x4646,0x5F46,0x4446,0x416F,0x446F,0x7F7F,0x7F41,0x466F,0x7F44,0x8F41,0x8F44,0x7F7F,0x9F54,0x4144,0x7F7F,0x6F44,0x6F7F,0x7F7F,0x7F6F,0x4144,0x7F7F,0x446F,0x6F6F,0x7F6F,0x6F5F,0x5F5F,0x6F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6441,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8F41,0x7F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x416F,0x5441,0x549F,0x549F,0x7F7F,0xC444,0xD564,0xD441,0xD564,0xD474,0xE4D4,0xE4C4,0x85E4,0x95B4,0x95C4,0x85C4,0x75B4,0x7575,0x8585,0x5475,0x5454,0x7565,0x8585,0x7575,0x7575,0x7565,0xC495,0x5485,0x5454,0x8565,0x7575,0x7585,0x7575,0x7575,0x5465,0x8585,0x8575,0x7575,0x7575,0x7585,0x7595,0x8575,0x75C4,0x7575,0x7575,0x7575,0x7585,0x7575,0x8585,0xC485,0x8585,0x75B4,0x8575,0x8554,0x7585,0x6475,0x7554,0x5475,0x8575,0x7575,0x6585,0x8575,0x85C4,0x85B4,0x85C4,0x4654,0x4646,0x4646,0x465F,0x4646,0x465F,0x4446,0x4646,0x4646,0x4644,0x4F46,0x4646,0x464F,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4644,0x4F46,0x4646,0x465F,0x4446,0x4446,0x4646,0x4646,0x4646,0x465F
.word 0x5454,0x7585,0x7585,0x7565,0x5454,0x5464,0x5454,0x4645,0x4646,0x4F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4446,0x4646,0x4446,0x7F6F,0x7F41,0x447F,0x7F7F,0x447F,0x7F46,0x8F44,0x4441,0x7F7F,0x4154,0x7F7F,0x7F44,0x6F7F,0x7F7F,0x6F7F,0x6F44,0x416F,0x7F7F,0x7F6F,0x6F6F,0x5F7F,0x6F5F,0x5F5F,0x6F6F,0x6F6F,0x6F5F,0x6F6F,0x6F6F,0x6F6F,0x416F,0x8F6F,0xD4D4,0xD4D4,0xD4D4,0x74D4,0x64AF,0x448F,0x7F7F,0x6F6F,0x6F6F,0x5F6F,0x5F6F,0x5F6F,0x5F5F,0x5F6F,0x5F5F,0x5F6F,0x6F6F,0x6F6F,0x546F,0xD4AF,0x64BF,0xD454,0xD4AF,0xD4D4,0xE4E4,0xE464,0x75D4,0x6575,0x6575,0x8585,0x9585,0x95C4,0x85B4,0x8575,0x7575,0x7575,0x8575,0x7575,0x5475,0xC485,0x8585,0xB485,0x5475,0x5454,0x6554,0x7575,0x7575,0x7554,0x5454,0x5454,0x4454,0x5454,0x5454,0x5454,0x5454,0x7554,0x6465,0x7554,0x5475,0x5475,0x5475,0x5454,0x5454,0x5475,0x7575,0x8575,0xB495,0xB485,0x5475,0x7575,0x8585,0x85B4,0x7585,0x7575,0x7575,0x7575,0x8585,0xB485,0xB495,0x4654,0x4446,0x4646,0x4646,0x4644,0x4646,0x465F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x464F,0x4446,0x4646,0x4646
.word 0x8554,0x6554,0x7554,0x9585,0x5475,0x5465,0x7554,0x4654,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F46,0x4446,0x464F,0x4646,0x4646,0x4646,0x4644,0x4F46,0x7F44,0x7F7F,0x7F7F,0x447F,0x466F,0x7F46,0x4441,0x418F,0x7F7F,0x8F54,0x7F44,0x7F7F,0x7F7F,0x6F6F,0x7F44,0x7F6F,0x447F,0x7F7F,0x6F6F,0x6F41,0x5F6F,0x6F6F,0x5F7F,0x4F5F,0x4F4F,0x6F6F,0x6F5F,0x6F6F,0x6F6F,0x6F7F,0x7F6F,0xD464,0xD4D4,0xD4D4,0xD4D4,0xAF64,0x8F44,0x6F6F,0x6F6F,0x5F6F,0x5F6F,0x5F6F,0xFC5F,0x5F5F,0x5F5F,0x5F6F,0x5F5F,0x5F4F,0x7F5F,0xC441,0xC4D4,0xAFC4,0xD454,0x64D4,0xE4D4,0xD4E4,0xE4D4,0x4664,0x5441,0x7554,0x6554,0x5465,0x5475,0x8585,0xC5C4,0xD4C5,0xE495,0x84D4,0x85D4,0x8554,0x95D4,0x75B4,0x7575,0x8585,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x4644,0x4446,0x4646,0x4646,0x4146,0x5454,0x5454,0x5475,0x7554,0x5454,0x5454,0x7554,0x5454,0x6554,0x5475,0x5454,0x7575,0x8585,0x8585,0x8585,0xB485,0x7554,0x7554,0x8575,0x8575,0x7575,0x8575,0x85D4,0x7575,0x5465,0x4654,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x4446,0x4646,0x4644,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4644
.word 0x85C4,0x54C4,0x7575,0x8575,0x75C4,0x5475,0x5454,0x4654,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F44,0x4646,0x5F46,0x7F46,0x417F,0x467F,0x416F,0x7F7F,0x4F44,0x4146,0x417F,0x8F44,0x7F7F,0x5454,0x417F,0x7F7F,0x447F,0x7F6F,0x7F7F,0x7F6F,0x7F41,0x6F41,0x6F7F,0x447F,0x6F6F,0x9F7F,0x6EBF,0x406E,0x7E6E,0x7F9D,0x7FBF,0x7F5F,0x6F41,0x6F7F,0x7F6F,0x6441,0xD4D4,0xD4D4,0xD4D4,0x9F64,0x6F6F,0x446F,0x6F6F,0x6F5F,0xCF54,0x7E7E,0x4F7F,0x6E4C,0xBFDC,0x7F9F,0x4F6F,0x6F4F,0x446F,0xD48F,0xD4D4,0x64D4,0xAFD4,0xD464,0xD4D4,0xE4D4,0xD4D4,0x4646,0x4446,0x4646,0x4644,0x4146,0x5454,0x7554,0x5465,0x5454,0x5454,0x5475,0x6554,0xC485,0x7585,0x7585,0x8585,0x85C4,0xC485,0x5485,0x8554,0x8585,0x6585,0x5485,0x4446,0x4646,0x465F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x5441,0x5465,0x7565,0x7575,0x85C4,0x95B4,0x85B4,0x85C4,0x85B4,0xC485,0x8585,0xC495,0x8575,0x7575,0x7585,0x4654,0x4646,0x4446,0x5F46,0x4646,0x465F,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4644,0x5F46,0x4446,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x4446,0x464F,0x4646,0x4646
.word 0xC485,0x7575,0x7585,0x7575,0x5485,0x6554,0x5454,0x4654,0x4644,0x5F46,0x4446,0x4646,0x4646,0x465F,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x7F46,0x447F,0x447F,0x445F,0x6F7F,0x4646,0x7F46,0x8F44,0x4441,0x417F,0x649F,0x448F,0x7F7F,0x6F41,0x7F7F,0x416F,0x6F6F,0x7F41,0x6F7F,0x6F6F,0x7F7F,0x6F6F,0xCF41,0x51CF,0x8E6E,0xF0F0,0xAF7E,0xBFEF,0x6F7F,0x7F7F,0x7F7F,0x6F6F,0xD444,0xD4D4,0xE4D4,0xD4D4,0x7FD4,0x7F6F,0x6F7F,0x5F5F,0x6444,0xFFDF,0x8E6E,0xBDF0,0xFC9D,0xEF6E,0x9FCF,0x6F7F,0x4F4F,0x647F,0xD464,0xD4D4,0xAFD4,0xD4C4,0xE4E4,0xD4D4,0xE4E4,0x5485,0x4446,0x4646,0x465F,0x4646,0x466F,0x4646,0x4446,0x4446,0x5454,0x5454,0x7544,0x8575,0x85C4,0x7585,0x7575,0x8575,0x95B4,0x75B4,0x5454,0x8565,0x75B4,0xC485,0x5475,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4446,0x5F46,0x5F46,0x4646,0x465F,0x465F,0x5F44,0x4646,0x465F,0x4646,0x4646,0x5454,0x5454,0x7565,0x8575,0xB485,0xB495,0x8595,0xE495,0x95B4,0x95B4,0x85B4,0x8575,0x5465,0x5475,0x4654,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x465F,0x4646
.word 0x7575,0x8585,0xC485,0x5485,0x7554,0x5454,0x5454,0x4654,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x464F,0x5F46,0x4646,0x5F46,0x4646,0x4644,0x6F46,0x7F41,0x467F,0x6F46,0x7F7F,0x5F46,0x4146,0x448F,0x8F8F,0x7F7F,0x6454,0x7F8F,0x7F41,0x7F7F,0x446F,0x7F6F,0x6F44,0x447F,0x6F7F,0x6F44,0x416F,0x4F6F,0x7F5F,0x5441,0xF474,0xF4E4,0xF4E5,0xD4E4,0xB465,0x6F7F,0x4A8F,0x7F6F,0xD441,0xE4E4,0xE4E4,0xD4E4,0x8F64,0x4144,0x8F8F,0x447F,0xD464,0xDFDF,0xFEF0,0x6E9E,0xE06E,0x64CF,0x7F8F,0x4F5F,0x546F,0xD4D4,0xE4E4,0xE4E4,0x64D4,0xE4D4,0xD4F4,0x64E4,0xD4E4,0x4544,0x4644,0x4644,0x4444,0x4444,0x4444,0x4154,0x6544,0x7554,0x5475,0x8575,0x9585,0xB4C4,0xC4B5,0x8585,0x7575,0x7575,0x5454,0x5454,0x5454,0xD485,0x8575,0x7575,0x4485,0x4644,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4444,0x6454,0x9575,0x75C4,0x5454,0x7554,0x8575,0x8585,0x85C4,0x85B4,0x7585,0x7585,0x6575,0x7575,0x5475,0x5454,0x4646,0x4646,0x465F,0x4646,0x465F,0x4F46,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4644,0x4646,0x465F,0x4446,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4644
.word 0x7585,0x7585,0x9585,0x7575,0x7585,0x7554,0x5454,0x4454,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x7F46,0x4441,0x467F,0x6F46,0x6F41,0x4646,0x7F46,0x7F41,0x4454,0x417F,0xAF54,0x447F,0x7F7F,0x7F44,0x7F6F,0x6F6F,0x6F7F,0x7F41,0x6F7F,0x7F7F,0x4441,0x6F41,0x4F4F,0x5F4F,0x6F5F,0x418F,0x8F54,0x8F41,0x417F,0x6F7F,0x8F7F,0x7F7F,0xD444,0xE4E4,0xE4E4,0xD4E4,0x41D4,0x416F,0x7F8F,0x446F,0x8F44,0xAF54,0x7464,0x7474,0x5464,0x7F8F,0x4F5F,0x544F,0xBF64,0xE4D4,0xE4E4,0xD4E4,0x9FD4,0xE4D4,0x85E4,0xD4AF,0x75D4,0x5475,0x5475,0x5454,0x6554,0x7554,0x5465,0x8585,0x8585,0x8585,0x7585,0x4454,0x5454,0x5454,0x5465,0x4454,0x5454,0x5444,0x5475,0x7575,0x6554,0x8595,0x7575,0x8585,0x4475,0x4646,0x4646,0x465F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x4446,0x6554,0x9585,0x85B4,0x8595,0x5465,0x5475,0x7554,0x6575,0x7575,0x7575,0x7575,0x6575,0x5464,0x5475,0x5454,0x5454,0x4646,0x465F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F46,0x4446,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646
.word 0x75D4,0x7565,0x7575,0x7554,0x7585,0x5475,0x5454,0x5454,0x4646,0x4644,0x4646,0x4446,0x4646,0x465F,0x4646,0x4446,0x4646,0x465F,0x5F46,0x5F46,0x4646,0x416F,0x416F,0x6F6F,0x4446,0x7F6F,0x446F,0x4146,0x447F,0x549F,0x7F7F,0x7564,0x7F8F,0x4154,0x416F,0x6F7F,0x6F41,0x7F41,0x417F,0x7F6F,0x6F41,0x417F,0x64D5,0x9FC4,0x8F64,0x6F41,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4A6F,0x6F6F,0x7F7F,0xD441,0xE4D4,0xE4F5,0xE4E4,0x9F64,0x7F7F,0x4A7F,0x6F7F,0x4F5F,0x5F4F,0x4F4F,0x4F4F,0x7F5F,0x546F,0x5454,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0x64D4,0xD464,0xD4D4,0xD4E4,0x8575,0x85C4,0xB485,0x7585,0x7575,0x7575,0x7575,0x7565,0x8575,0x95C4,0x85B4,0x8595,0x7585,0x7554,0x7585,0x8585,0x8575,0x8585,0xB485,0x5485,0x9554,0x75B4,0x8585,0x75B4,0x4644,0x4644,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4644,0x4F46,0x4646,0x5F46,0x4646,0x4446,0x7554,0x5465,0xB485,0x7585,0xD4B4,0x7595,0x5454,0x6554,0x5464,0x5475,0x5475,0x5454,0x5475,0x7565,0x5454,0x5475,0x7575,0x4464,0x4645,0x5F46,0x4646,0x5F46,0x4446,0x4646,0x4644,0x4646,0x465F,0x4446,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4446,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x464F,0x4646,0x465F
.word 0xD495,0x7585,0x6554,0x7575,0x7575,0x5454,0x7575,0x6465,0x4644,0x4646,0x4446,0x4646,0x465F,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x416F,0x417F,0x465F,0x4646,0x416F,0x5F7F,0x7F46,0x417F,0x8F44,0x7F44,0xAF64,0x7F44,0x7F8F,0x6F41,0x7F7F,0x6F7F,0x7F7F,0x7F44,0x6F6F,0x7F44,0x447F,0xD4D4,0xD4D4,0x64C4,0x64D5,0x6464,0x64D5,0xC464,0x6464,0x7F8F,0x7F6F,0x7F41,0xD454,0xE4E4,0xE4F4,0xE4E4,0x54C4,0x7F7F,0x6F7F,0x5441,0x549F,0x9F64,0xAF64,0xAF54,0x6464,0xE554,0xD4D4,0xE4D4,0xF8D4,0xE4E4,0xD4E4,0xD4E4,0xAFD4,0xD464,0xE4E4,0x85D4,0x8585,0x85B4,0x8585,0x85B4,0x7585,0x8575,0x8575,0x8585,0x7575,0x6575,0x7575,0x6575,0x7554,0x8575,0x85B4,0x7575,0xB485,0xB485,0x8585,0x5475,0x85C4,0x7585,0xB485,0x5454,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x7565,0x7554,0x7575,0x5454,0x8554,0x9585,0x75B4,0x5475,0x5454,0x5454,0x6465,0x7565,0x5454,0x5454,0x5454,0x5454,0x7554,0x5475,0xC495,0x4454,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4644,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646
.word 0xC485,0x5485,0x8585,0x9585,0x8575,0x85C4,0x5454,0x5454,0x4665,0x4644,0x465F,0x4646,0x4646,0x4446,0x4646,0x464F,0x5F46,0x4446,0x4646,0x4644,0x4646,0x417F,0x447F,0x5F46,0x4646,0x445F,0x447F,0x416F,0x7F44,0x8F41,0x7F41,0x6464,0x417F,0x7F44,0x447F,0x7F7F,0x446F,0x7F6F,0x417F,0x7F6F,0x7F7F,0x7F41,0xE4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0x64D4,0xD564,0xD451,0xE4D4,0xF4F5,0xE4E4,0xC4E4,0x6464,0x6454,0xD4D4,0xD4D4,0xD4D5,0xC4D4,0xF9D5,0xD4E5,0xF4D4,0xD4E4,0xF9E4,0xE4E4,0xE4E4,0xD4C5,0xD4D4,0x6485,0xD4D5,0xD4D4,0x4654,0x4646,0x4644,0x4146,0x5444,0x6554,0x7554,0x5454,0x5454,0x7575,0x7585,0x7575,0x8575,0x7585,0x8575,0x8585,0x8575,0x8585,0x7595,0x75C4,0x7554,0x8585,0x95C4,0x5475,0x4644,0x4644,0x465F,0x4446,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4646,0x4446,0x5F46,0x4646,0xA454,0x5454,0x5454,0x4446,0x4446,0x8575,0x85D4,0x6465,0x7575,0x5475,0x6465,0x5465,0x5454,0x5454,0x5475,0x8554,0x7575,0x5475,0xD485,0x8575,0x8595,0x4454,0x4446,0x5F46,0x4646,0x4446,0x5F46,0x4646,0x4644,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x5F46,0x4646,0x4644
.word 0xB485,0x7575,0x8575,0x85B4,0x7575,0xB485,0x7554,0x5454,0x4554,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x416F,0x7F7F,0x4646,0x4644,0x5F46,0x7F7F,0x7F41,0x7F41,0x8F44,0x7F44,0xD554,0x8F7F,0x417F,0x7F7F,0x447F,0x6F6F,0x7F7F,0x7F41,0x7F6F,0x7F41,0x8F44,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD5E4,0xD4E4,0x64D4,0xD4D5,0xE4D4,0xF4F5,0xD4E4,0xD4D4,0xD4D4,0xF4E5,0xE4F9,0xF9F9,0xD4D4,0xD4D4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xE4E4,0xD4D4,0x64BF,0xD4AF,0x5464,0x4646,0x4644,0x465F,0x4646,0x4644,0x4446,0x7554,0xC485,0xC495,0x85C5,0x85C4,0x8585,0xB485,0x7585,0x8575,0x75B4,0xB485,0xB485,0xC485,0x4464,0x8554,0x85C4,0x5485,0x4654,0x4644,0x4646,0x4646,0x4646,0x465F,0x4646,0x4644,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4644,0x4644,0x4646,0x5446,0x8585,0x75B4,0x7585,0x7554,0x5475,0x8575,0x7554,0x8585,0x7565,0x8575,0x8565,0x75C4,0x6585,0x7554,0x7585,0x8565,0x8585,0x4444,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x464F,0x4646,0x4644,0x4F46,0x4646,0x464F,0x4646,0x464F,0x4646,0x4646,0x464F
.word 0x5454,0x5475,0x7575,0x7575,0x7554,0x5454,0x5475,0x7554,0x5454,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x445F,0x7F7F,0x5F46,0x4646,0x4646,0x445F,0x7F7F,0x447F,0x5441,0x7F9F,0xAF64,0x7F44,0x7F41,0x7F44,0x6F41,0x417F,0x7F6F,0x447F,0x7F6F,0x6F44,0x4141,0xD4D4,0xC5E4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xE464,0xE5D4,0xE4F5,0xE4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xC5E4,0xF4F4,0xE4E4,0xE4D4,0xE4D4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x64C4,0x5445,0x4647,0x4644,0x4646,0x4646,0x465F,0x5444,0x8575,0x8585,0x95C4,0xB5C4,0x95C4,0xD485,0xB495,0xC495,0x8585,0xB485,0xB495,0x8585,0x7575,0x5475,0x5454,0x4454,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x7541,0x7575,0x8575,0x8575,0x6465,0x7554,0xC454,0x7575,0x9575,0x8565,0x9575,0x7585,0x7575,0x8585,0x7565,0x5475,0x4646,0x4444,0x5454,0x4444,0x4446,0x4646,0x465F,0x4446,0x4F46,0x4646,0x464F,0x4646,0x4644,0x465F,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646
.word 0x6595,0xE4C5,0x7585,0x8554,0x95C4,0x75C4,0x7585,0x5465,0x5475,0x4654,0x4646,0x4646,0x4644,0x4446,0x5F46,0x4646,0x4646,0x464F,0x4446,0x4646,0x4646,0x445F,0x7F7F,0x4646,0x4646,0x465F,0x6F46,0x417F,0x8F7F,0x5441,0x8F54,0x6454,0x7F8F,0x8F44,0x417F,0x7F6F,0x6F41,0x447F,0x7F7F,0x7F7F,0x7F7F,0x8F44,0xE4E4,0xE4E4,0xE4C5,0xC5E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4BF,0xE4D4,0xF4F7,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5D4,0xD4D4,0x45BF,0x4646,0x4646,0x4646,0x465F,0x4646,0x7545,0xB485,0x8585,0x75B4,0x7575,0x8585,0x7585,0x8575,0x7575,0x7575,0x8575,0x7575,0x8585,0x75B4,0x5475,0x7554,0x5485,0x4654,0x4644,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4146,0x7565,0x5454,0x8585,0x8575,0x5444,0x5454,0x8575,0x7554,0x8575,0x7575,0x8575,0xD4B4,0x5475,0x8565,0x8585,0x5475,0x4644,0x4446,0x4646,0x4644,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4446,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x5F46,0x4646,0x4644
.word 0x6585,0x8585,0x85B4,0x8565,0x7585,0x7585,0x7575,0x7554,0x5454,0x5454,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4446,0x4646,0x7F46,0x446F,0x466F,0x4644,0x4646,0x445F,0x7F7F,0x447F,0x9F7F,0x8F44,0x9F54,0x7F54,0x417F,0x7F44,0x447F,0x7F7F,0x7F7F,0x7F41,0x416F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD574,0xD485,0xF4D4,0xF4F5,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xC5E4,0xD4E4,0xD4D4,0xD4D4,0x4664,0x4446,0x5F46,0x4646,0x4646,0x8554,0x7585,0x5454,0x4454,0x4644,0x5441,0x5444,0x6554,0x5475,0x7575,0x7554,0x6575,0x5475,0x4454,0x7564,0x8585,0x6585,0x4654,0x4644,0x4646,0x465F,0x4646,0x4646,0x4644,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4644,0x4446,0x4646,0x5F46,0x4646,0x4446,0x5475,0x4146,0x8565,0x54B4,0x4645,0x4446,0x8554,0x9575,0x5475,0x5454,0x8575,0x8575,0x5485,0x5441,0x8565,0x85C5,0x4654,0x4646,0x465F,0x4646,0x465F,0x4646,0x4446,0x4646,0x465F,0x4446,0x4446,0x4646,0x5F46,0x4646,0x4644,0x4446,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7554,0x7575,0x5475,0x7575,0x7575,0x5465,0x7554,0x5454,0x5475,0x5475,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x465F,0x445F,0x6F46,0x4646,0x4646,0x4646,0x4146,0x7F7F,0x7F7F,0x4144,0x548F,0x548F,0x7F54,0x7F7F,0x417F,0x7F7F,0x7F7F,0x6F44,0x417F,0x7F6F,0x7F44,0x8F44,0xD4D4,0xE4D4,0xE4D4,0xC5E4,0xC5E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFD4,0xD4D4,0xF4D4,0xF4F4,0xD5E4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD5D4,0xE5E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xD4D4,0xC5D4,0xD4D4,0x64D4,0x46AF,0x4646,0x4646,0x4646,0x7544,0x7585,0x4454,0x4646,0x4644,0x465F,0x4646,0x4644,0x4446,0x4646,0x5454,0x7575,0x7554,0x7575,0x7575,0x8585,0x75B4,0x5454,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4446,0x4646,0x4646,0x4644,0x464F,0x4F46,0x4646,0x4646,0x4446,0x5446,0x4454,0x4646,0x8541,0x5485,0x4646,0x4646,0x7541,0x85C4,0x5475,0x4146,0x7554,0x7585,0x75D4,0x4646,0x5441,0xC475,0x4475,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4646,0x4646,0x4646,0x4646,0x464F,0x5F46,0x4646,0x465F,0x4644,0x465F
.word 0xD475,0x7585,0x7575,0x5475,0x5475,0x9554,0x95C4,0x5485,0x7554,0x5454,0x4654,0x4F44,0x4646,0x465F,0x4F44,0x4446,0x4646,0x4646,0x464F,0x465F,0x4646,0x5F46,0x5F46,0x5F46,0x4646,0x465F,0x445F,0x447F,0x7F7F,0x417F,0x9F54,0x9F44,0x7F54,0x7F7F,0x7F44,0x7F41,0x7F7F,0x7F7F,0x7F41,0x7F6F,0x7F7F,0x4441,0xD464,0xD4D4,0xD4C5,0xD4E4,0xD4E4,0xE4E4,0xC5D4,0xC5D4,0x74D4,0x64D4,0xC4D4,0xD4D4,0xE5D4,0xF4F4,0xE4F4,0xD4D4,0xD4C5,0xE4C5,0xC5D4,0xD4E4,0xD4F4,0xE4E4,0xE4E5,0xE4E4,0xE4D5,0xE4F4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x4654,0x5F46,0x4646,0x4444,0x5475,0x4644,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4444,0x8585,0x95D4,0xB5D4,0x95D4,0x8585,0x5475,0x5454,0x4646,0x4644,0x465F,0x4646,0x4646,0x4644,0x465F,0x4646,0x465F,0x4446,0x4446,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4541,0x4446,0x5F46,0x7544,0x5475,0x4446,0x5F46,0x5446,0x7585,0x4654,0x4646,0x4146,0x8565,0x7585,0x4646,0x4646,0x5441,0x7554,0x4644,0x5F46,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x465F,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x8585,0xC485,0xD454,0xD4B4,0xC494,0x9575,0xC5C4,0x75C4,0x7554,0x5454,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x417F,0x7F7F,0x417F,0x7F7F,0x4144,0x548F,0x44AF,0x416F,0x447F,0x447F,0x417F,0x7F7F,0x447F,0x416F,0x7F7F,0x8F44,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD5D4,0xD4BF,0xD464,0xD4BF,0xF4D4,0xF4F4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D5,0xF4E4,0xF4E4,0xE4E4,0xE5E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x64D4,0x4646,0x4646,0x4646,0x7544,0x4454,0x4446,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x4446,0x8554,0x85B4,0x7585,0x7575,0x5454,0x4454,0x4654,0x4446,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4644,0x4646,0x4646,0x5446,0x4475,0x4646,0x4646,0x4146,0x6575,0x4654,0x5F46,0x4646,0x6541,0x7585,0x4446,0x5F46,0x4646,0x5441,0x4641,0x4646,0x4644,0x464F,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x4446,0x4F46,0x4646,0x5F46,0x4646,0x4644,0x5F46,0x4646,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x465F,0x4646
.word 0xB485,0x7595,0x8575,0xC495,0x9585,0x8575,0xC485,0x8595,0x5454,0x5485,0x5465,0x4654,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x416F,0x417F,0x446F,0x7F7F,0x448F,0x8F54,0x4154,0x7F7F,0x7F7F,0x8F54,0x7F7F,0x447F,0x7F7F,0x7F6F,0x7F44,0x8F41,0xBFC4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xC5E4,0xD4D4,0xD4D4,0xD464,0xD485,0xD4D4,0xD4D4,0xE5D4,0xF5F4,0xE4F4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4C5,0xD4D4,0xAFD4,0x4446,0x4646,0x4444,0x4454,0x4646,0x4646,0x465F,0x4646,0x4646,0x4446,0x4646,0x7544,0x7575,0x5454,0x4644,0x4646,0x4646,0x4644,0x4446,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4644,0x465F,0x4644,0x465F,0x5F44,0x4646,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4644,0x4644,0x4646,0x465F,0x4646,0x5454,0x4646,0x4646,0x4446,0x4146,0x8554,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7575,0x5475,0xC485,0x8585,0x7575,0x6554,0xB485,0x6575,0x7575,0x6554,0x7554,0x4454,0x4446,0x5F46,0x4646,0x5F46,0x4646,0x4644,0x4646,0x465F,0x4F44,0x4646,0x464F,0x4446,0x4646,0x465F,0x447F,0x7F7F,0x7F7F,0x7F7F,0x9F41,0x8F54,0x8F54,0x7F7F,0x417F,0x7F7F,0x7F44,0x6F41,0x417F,0x416F,0x7F7F,0x8F44,0xC4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0x64D4,0xBFD5,0xBFC4,0xD4D4,0xD4D4,0xF4E4,0xF5F5,0xE4F4,0xD4E4,0xD4C5,0xC5D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0x5464,0x4646,0x5F46,0x4546,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x7575,0x5454,0x4644,0x4446,0x6F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x5F46,0x4446,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4441,0x4446,0x4446,0x4646,0x4646,0x4441,0x4646,0x4644,0x4646,0x4644,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x5F46,0x4646,0x4644,0x4644,0x4446,0x4646,0x4646,0x4644,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F
.word 0x8595,0x6554,0x7554,0x7565,0x7565,0x7585,0x5475,0x9585,0x85C4,0x5454,0x5475,0x5454,0x4644,0x4646,0x4446,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x464F,0x464F,0x4646,0x417F,0x417F,0x447F,0x7F7F,0x447F,0x648F,0x548F,0x447F,0x7F7F,0x417F,0x547F,0x417F,0x7F7F,0x416F,0x7F6F,0x4141,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD464,0xD464,0xD4D4,0xF4D4,0xF5F5,0xE4F7,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D5,0xD5E4,0xE4E4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0x46AF,0x5F46,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646,0x465F,0x4446,0x5454,0x4654,0x4646,0x4446,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4644,0x4644,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4446,0x4646,0x4646,0x4446,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4446,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x465F,0x4646
.word 0x85C4,0x9554,0xB5C5,0x8585,0x9554,0x85C4,0xC495,0xB485,0x7585,0x6465,0x7544,0x7554,0x6554,0x4446,0x4646,0x4646,0x465F,0x4646,0x4644,0x465F,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x6F46,0x7F41,0x7F41,0x7F7F,0x447F,0x7F6F,0x9F54,0x5444,0x7F8F,0x447F,0x7F7F,0x7F41,0x7F44,0x447F,0x7F6F,0x7F44,0x8F44,0xD4C4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x64BF,0x64D5,0xD4D4,0xD4D4,0xE4E4,0xF5FF,0xE4F4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4C5,0xD4E4,0xD4D4,0xD4E4,0x64D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4646,0x4646,0x4444,0x4154,0x4446,0x4446,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4446,0x5F46,0x5F46,0x4646,0x465F,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4F46,0x4646,0x464F,0x4646,0x4F44,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x465F,0x4644,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7575,0xC475,0xC485,0xB485,0x8575,0xB485,0x6585,0x5454,0x5454,0x8585,0x5475,0x7554,0x5454,0x4465,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x6F44,0x7F41,0x447F,0x417F,0x416F,0x417F,0x5441,0x54AF,0x7F41,0x7F7F,0x447F,0x7F6F,0x7F41,0x7F7F,0x7F7F,0x417F,0x418F,0xBF64,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD4AF,0x64D4,0xD4D4,0xF4E4,0xFFF4,0xE4F4,0xD4E4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xC5D4,0x64D4,0x64AF,0x46AF,0x4646,0x4644,0x4644,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4446,0x5454,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x4644,0x465F,0x5F46,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x464F,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x5F46,0x4646,0x4646,0x465F,0x465F,0x4446,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4446,0x5F46,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4446,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4446,0x4646
.word 0x7575,0x8585,0xB485,0x5475,0x6575,0x5475,0x5475,0xD495,0x6585,0x7554,0x7585,0x5454,0x7575,0x5454,0x4644,0x4446,0x4646,0x4446,0x4646,0x4446,0x5F46,0x4646,0x4F44,0x4646,0x465F,0x6F46,0x4441,0x7F7F,0x7F7F,0x447F,0x7F7F,0x547F,0x9F54,0x8F54,0x447F,0x7F7F,0x7F6F,0x7F44,0x7F44,0x416F,0x447F,0x8F41,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF5F5,0xE4F5,0xD4E4,0xE4E4,0xD4D4,0xD4BF,0xD4D4,0xD4D4,0xD4C5,0xE4E4,0xD5D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x64D4,0xD564,0x4644,0x5F44,0x4646,0x5F46,0x4646,0x4646,0x4446,0x4646,0x4446,0x5454,0x4654,0x4644,0x465F,0x4646,0x465F,0x4646,0x4446,0x4646,0x5F46,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4644,0x4F46,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x465F,0x4F46,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4446,0x4F46,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x464F,0x4646,0x465F,0x464F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4646,0x465F,0x4644,0x465F,0x4646,0x5F46,0x4646,0x4646
.word 0x5454,0x6565,0x5475,0x7575,0x8595,0x95E4,0x8565,0x85D4,0x7575,0xB485,0x7554,0x6585,0x7554,0x6554,0x4454,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x445F,0x417F,0x417F,0x447F,0x417F,0x7F7F,0x547F,0x649F,0x4454,0x7F7F,0x7F7F,0x417F,0x417F,0x547F,0x447F,0x7F7F,0x7F44,0xD4C4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xAFD4,0xD464,0xD4E4,0xD4D4,0xF4E4,0xF4FF,0xE4F4,0xD4E4,0xE4C5,0xD4D4,0xD485,0xD4E4,0xE4C5,0xD4D4,0xE4D4,0xD4E4,0xC5D4,0xD4D4,0xD4E4,0xD474,0x64AF,0xC4AF,0x4646,0x4646,0x4646,0x465F,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F44,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4644,0x4646,0x4646,0x4446,0x4F46,0x4F46,0x4646,0x4646,0x4646,0x465F,0x4446,0x464F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4F46,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4446,0x464F,0x465F,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F46,0x4446,0x4646,0x4644,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F
.word 0x5475,0x8585,0x8585,0x7585,0x95D4,0x85B4,0x7585,0x7575,0x8554,0x95D4,0x7575,0x8595,0x5454,0x7575,0x7575,0x4454,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4644,0x416F,0x447F,0x7F7F,0x7F7F,0x447F,0x417F,0x7F7F,0x9F54,0x9F64,0x7F44,0x447F,0x6F6F,0x7F7F,0x7F41,0x7F7F,0x4141,0x417F,0xBF64,0xD4C4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0x85D4,0xD4D5,0xD4D4,0xE4D4,0xF4E4,0xE4E4,0xE4F4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64D5,0xAFD5,0x4646,0x4646,0x465F,0x4646,0x4646,0x464F,0x4F46,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F44,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F46,0x4446,0x4646,0x4F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x4646,0x465F,0x465F,0x4F44,0x5F46,0x5F46,0x4446,0x4446,0x4F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x465F,0x465F,0x465F,0x4646,0x4644,0x464F,0x465F,0x5F46,0x4446,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x464F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4446,0x4646,0x5F46,0x4646,0x465F,0x4646
.word 0x7554,0xB485,0x7585,0x5475,0xB485,0x7585,0x5454,0x8575,0x75B4,0x7585,0x7554,0xC475,0x7575,0x5454,0x5475,0x5475,0x4465,0x4446,0x4646,0x5F46,0x4646,0x4446,0x5F46,0x5F46,0x4646,0x7F7F,0x417F,0x7F41,0x7F44,0x417F,0x7F7F,0x7F7F,0x5444,0x44AF,0x418F,0x7F6F,0x7F7F,0x446F,0x447F,0x7F7F,0x447F,0x8F7F,0xD4C4,0xD474,0xC5D4,0xE4D4,0xC5D4,0xE4E4,0xE4D4,0xD4C5,0x64E4,0x649F,0x64AF,0x6464,0xE4D4,0xE4E4,0xD4D4,0x6485,0x64AF,0x649F,0xD4E4,0xE4E5,0xE4D4,0xC5D4,0xD4E4,0xE4C5,0xE4D4,0xD4D4,0x74D4,0xAFD4,0xBF64,0x46C4,0x4446,0x465F,0x4646,0x465F,0x465F,0x4446,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4446,0x464F,0x5F46,0x4446,0x464F,0x4646,0x4646,0x465F,0x4446,0x5F46,0x4646,0x4646,0x4F46,0x4446,0x4646,0x5F46,0x4646,0x4644,0x4446,0x464F,0x4F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F44,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4446,0x465F,0x4646,0x4644,0x4646,0x5F44,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4446,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x465F,0x465F,0x5F46,0x4646,0x465F,0x464F,0x465F,0x4644,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4644
.word 0x7554,0x7585,0x7565,0x7554,0x7575,0x5454,0xD485,0xC485,0x5495,0x7565,0x7554,0x5454,0x5454,0x5454,0x7554,0x7575,0x5454,0x4645,0x5F46,0x4646,0x4446,0x464F,0x4646,0x4646,0x5F46,0x7F44,0x7F41,0x7F44,0x417F,0x547F,0x447F,0x6F7F,0x9F41,0xD454,0x8F44,0x7F7F,0x6F44,0x7F7F,0x7F7F,0x7F41,0x417F,0x547F,0xC49F,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xC5E4,0x8FD4,0x9F54,0x9F44,0xAF64,0x5464,0xC4D4,0xBF64,0x8FC4,0x5454,0x548F,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD4C4,0x46D4,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x465F,0x4F46,0x5F46,0x4646,0x4446,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x465F,0x4F46,0x4446,0x4646,0x4F46,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4F46,0x4646,0x4644,0x4646,0x4644,0x4644,0x4646,0x4646
.word 0x6554,0x5475,0x5454,0x8575,0xB5D4,0x75C4,0x7575,0x8585,0x7575,0x9585,0x85C4,0xC475,0xC495,0x8575,0x5475,0x6554,0x7575,0x5454,0x4575,0x4446,0x4646,0x4646,0x4646,0x4644,0x6F46,0x7F41,0x447F,0x417F,0x7F6F,0x417F,0x417F,0x7F7F,0x447F,0x649F,0x44AF,0x6F41,0x6F7F,0x6F41,0x417F,0x7F6F,0x4144,0x547F,0xD48F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4C5,0xD4E4,0x41D4,0x8F44,0x4441,0x448F,0xD4BF,0xBFD4,0x5464,0x418F,0x448F,0xC4AF,0xF4F4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0x85BF,0xD4BF,0x4664,0x5F44,0x4646,0x465F,0x465F,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4446,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x4646,0x465F,0x5F46,0x5F46,0x4446,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4644,0x465F,0x465F,0x4646,0x4446,0x4F46,0x4646,0x5F46,0x4646,0x465F,0x4644,0x4646,0x4446,0x5F46,0x4646,0x464F,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x465F,0x4644,0x464F,0x464F,0x465F,0x4F44,0x5F46,0x4446,0x4646,0x465F,0x4646,0x4446,0x4646,0x4446,0x464F,0x4646,0x4646,0x4F44,0x4F46,0x4F46,0x4F46,0x4F46,0x5F46,0x4446
.word 0x5454,0x7554,0x7585,0xC454,0xB485,0x85B5,0x6554,0x6575,0x9554,0xD4C4,0xC4B5,0x8575,0x95C4,0xD475,0x8585,0x5454,0x5475,0x7575,0x7554,0x4454,0x4644,0x4646,0x4644,0x465F,0x7F46,0x417F,0x417F,0x7F7F,0x447F,0x446F,0x447F,0x7F7F,0x4441,0x548F,0x8F44,0x7F44,0x6F7F,0x6F7F,0x7F7F,0x7F7F,0x447F,0x417F,0xD47F,0x74D4,0xE4D4,0xC5D4,0xD5D4,0xD4E4,0xE4E4,0xD4D4,0x64D4,0x449F,0x418F,0x9F44,0x4144,0x5444,0x449F,0x448F,0x649F,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4C5,0xD4E4,0x74D4,0x64D4,0xD4D5,0xD4D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x4446,0x4646,0x465F,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4446,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x465F,0x4644,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F44,0x4646,0x465F,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x464F
.word 0x7554,0x7554,0x7554,0x7554,0x8575,0x75B4,0xC585,0x95C4,0x75D4,0x8585,0x64B4,0x5454,0x5454,0x8575,0x75C4,0x8575,0x5465,0x7554,0x5475,0x6575,0x5475,0x7554,0x6544,0x5465,0x4165,0x447F,0x447F,0x7F41,0x7F7F,0x417F,0x5441,0x417F,0x417F,0x547F,0x54AF,0x418F,0x7F7F,0x7F44,0x6F7F,0x7F44,0x4141,0x447F,0xD47F,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x9F64,0x9F54,0x9F54,0xE4C4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xF5E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xC4AF,0x74D4,0xD464,0x4646,0x465F,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x464F,0x4446,0x4446,0x5F46,0x4446,0x4F46,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x465F,0x5F46,0x4F46,0x5F46,0x4646,0x465F,0x465F,0x4446,0x5F46,0x5F46,0x4F46,0x5F46,0x5F46,0x5F46,0x4646,0x464F,0x4646,0x4646,0x5F46,0x4646,0x4644,0x5F46,0x4446,0x4646,0x4644,0x464F,0x5F46,0x5F46,0x5F46,0x4446,0x464F,0x465F,0x464F,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4646,0x465F,0x465F,0x464F,0x465F,0x465F,0x5F46,0x5F46,0x4F46,0x4F46,0x4646,0x5F46,0x4646,0x5F46,0x4446,0x5F46,0x4646,0x4646,0x4646
.word 0x5454,0x6554,0x7575,0x7554,0x7554,0x5454,0x8575,0x7585,0x6585,0x7575,0x5454,0x9575,0x85C5,0x5465,0x5485,0xD475,0x8595,0x5465,0x8554,0x5475,0x7575,0x5475,0x7575,0x5475,0x7F54,0x7F41,0x417F,0x448F,0x7F7F,0x447F,0x8F9F,0x447F,0x447F,0x418F,0x9F64,0x7F54,0x6F7F,0x416F,0x7F6F,0x7F7F,0x447F,0x8F7F,0xC441,0xC4BF,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xF4D4,0xC4E4,0x54AF,0x64D5,0xE4E4,0xE4F4,0xE4D4,0xE4F4,0xF4E4,0xF4F5,0xE4E4,0xD5E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD464,0xD5D4,0xBFD4,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F44,0x4646,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x4644,0x4446,0x4646,0x465F,0x4644,0x5F46,0x4646,0x464F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4646,0x464F,0x465F,0x4F46,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x465F,0x4644,0x5F46,0x4646,0x4446,0x4646,0x465F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4F46,0x4646,0x5F46,0x4646,0x4646
.word 0x5454,0x5454,0x5454,0x7554,0xC495,0xC495,0x7554,0x6575,0x8554,0xC495,0xC495,0xC475,0xB5D4,0xC485,0xB495,0x7554,0x7575,0x5454,0x7575,0x5454,0x7554,0x7585,0x7575,0x7575,0x417F,0x4144,0x447F,0x7F7F,0x447F,0x416F,0x4154,0x4144,0x417F,0x8F7F,0x6444,0x549F,0x7F44,0x7F7F,0x447F,0x7F6F,0x4141,0x447F,0xC48F,0x74D4,0xBFD4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xE4D4,0xF5F5,0xE4F4,0xE4D4,0xE4F4,0xF4D4,0xE4F4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xD5E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xBFD4,0xBFC4,0x6485,0x4446,0x5F46,0x4646,0x465F,0x4446,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x464F,0x4646,0x465F,0x4644,0x464F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4644,0x4646,0x4446,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x4446,0x4646,0x465F,0x4646,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x465F,0x465F,0x465F,0x5F46,0x4446,0x5F46,0x4646,0x4446,0x4646,0x4646,0x464F,0x4646,0x4446,0x4646,0x465F,0x4646,0x4644,0x4646,0x5F46,0x4446,0x4F46,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F
.word 0x5444,0x5454,0x5475,0x7575,0xC485,0x8585,0x7565,0x5475,0xC475,0xD495,0xD495,0x8554,0x7595,0x7565,0x95D4,0x5485,0x8575,0x7585,0xC554,0x5475,0x5475,0x5454,0x5454,0x5454,0x417F,0x447F,0x8F7F,0x7F41,0x7F7F,0x7F7F,0x9F54,0x9F44,0x447F,0x448F,0x548F,0x41D5,0x418F,0x446F,0x7F6F,0x7F6F,0x447F,0x417F,0x858F,0xC4BF,0xC4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xC4D4,0xB4C5,0xC565,0xE475,0xE4F5,0xF1E4,0x65E4,0x4565,0x6545,0x8465,0xD4D5,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xC4D4,0x64D4,0x44D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x464F,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4644,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x464F,0x464F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x465F,0x4646,0x4646,0x4644,0x4446,0x5F46,0x4646,0x464F,0x4446,0x4646,0x4646,0x4644,0x5F46,0x4646,0x465F,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646,0x464F,0x464F,0x464F,0x4646,0x4646,0x4F46,0x4F46,0x4646,0x465F,0x465F,0x5F46,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4446
.word 0x5454,0x5454,0x5454,0x5454,0x6575,0x4475,0xD485,0xD4C5,0x7595,0x95C4,0x85B4,0x7554,0x7565,0x5485,0xC485,0x7575,0xE495,0xC4C5,0x7575,0x7554,0xC585,0x75C4,0x5454,0x7F54,0x7F41,0x8F44,0x447F,0x7F41,0x7F44,0x417F,0x9F44,0x5464,0x548F,0x7F7F,0x8F41,0xC441,0x7F41,0x7F7F,0x7F6F,0x447F,0x416F,0x7F8F,0x547F,0x64D4,0x85BF,0x74D4,0x64D4,0x54AF,0x4444,0x6544,0x4445,0x6544,0x6565,0xC565,0xC575,0x6575,0x4545,0x6545,0x4545,0x4465,0x4545,0x7545,0xB4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD574,0x85BF,0xC4BF,0x469F,0x4646,0x465F,0x4446,0x4646,0x465F,0x4646,0x4644,0x4646,0x4446,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x465F,0x465F,0x4644,0x4646,0x4446,0x464F,0x4446,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x465F,0x4646,0x4644,0x465F,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4F46,0x4646,0x4646,0x4446,0x4F46,0x4646,0x464F,0x4646,0x465F,0x4446,0x4646,0x5F46,0x4646,0x464F,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x5F46,0x4F46,0x4F46,0x5F46,0x4F46,0x5F46,0x5F46,0x5F46,0x4646
.word 0x4485,0x5454,0x5465,0x5454,0x5454,0x5454,0xC585,0x8595,0x7575,0x7575,0x7585,0xD485,0xD4D5,0x85D4,0x7575,0x7554,0xC485,0x8595,0x5465,0x7585,0x7565,0x5475,0x8575,0x7F54,0x447F,0x418F,0x7F7F,0x7F54,0x7F7F,0x7F7F,0x6444,0x54AF,0x547F,0x448F,0x448F,0x8F7F,0x6464,0x417F,0x6F7F,0x7F6F,0x447F,0x7F41,0x4441,0xC4BF,0xBFC4,0xAFC4,0x7F41,0x4444,0x4564,0xC545,0xC565,0x6565,0x65C5,0x65C5,0x4565,0x6545,0xC565,0xC545,0xC5C5,0xC575,0xC565,0x4665,0x445F,0xD465,0xD464,0xD4D4,0x64D4,0xC4BF,0x64D4,0xD4D5,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x464F,0x5F46,0x4646,0x465F,0x4646,0x5F46,0x4F46,0x4F46,0x5F46,0x4446,0x464F,0x465F,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x465F,0x465F,0x465F,0x4646,0x464F,0x4646,0x4646,0x4644,0x465F,0x465F,0x4644,0x4646,0x4646,0x4F46,0x4646,0x4644,0x465F,0x4644,0x465F,0x465F,0x4644,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x5F46
.word 0x6595,0x5454,0x6564,0x8554,0x8585,0x54B4,0x9575,0x75B4,0x5475,0x5454,0x7585,0x9565,0x8585,0x75D4,0x8554,0x85C4,0x7565,0x5465,0xE485,0x85D4,0xD485,0x7585,0x5454,0x417F,0x417F,0x8F44,0x447F,0x4141,0x7F7F,0x7F44,0x6441,0x9FD5,0x9F54,0x7F44,0x8F41,0x7F54,0xC48F,0x448F,0x417F,0x7F6F,0x416F,0x447F,0x8F7F,0xD564,0xD574,0xD574,0xD4C4,0x4565,0x4444,0x446F,0x7F44,0x4444,0x7F44,0x5F44,0x5F6F,0x4A6F,0x445F,0x5445,0x4544,0x4544,0x4444,0x6544,0xD4D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xBF64,0x85C4,0x6564,0x6465,0x6465,0x5454,0x4454,0x4446,0x5F46,0x4646,0x4446,0x4646,0x4446,0x4646,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x464F,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4F46,0x4646,0x465F,0x465F,0x464F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x464F,0x464F,0x465F,0x465F,0x4646,0x5F44,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4644,0x5F46,0x4646,0x465F,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4644,0x4646,0x4646,0x4644
.word 0x85D4,0x5444,0x5454,0x7554,0x6575,0x7575,0x7575,0x5475,0xC454,0xD4C5,0x95C5,0x7575,0x8575,0x6585,0xC495,0x8594,0x8575,0x7585,0x8575,0x85D4,0xB5B4,0x85D4,0x4475,0x417F,0x447F,0x418F,0x417F,0x8F54,0x6F44,0x7F7F,0x9F44,0x5454,0x449F,0x7F41,0x8F44,0x8F41,0x8F44,0x4164,0x7F44,0x7F6F,0x447F,0x418F,0x447F,0xBF64,0xBFC4,0xD4C4,0xC4BF,0xAFD4,0x45C5,0x4544,0x84C5,0xC5C5,0xC5C5,0x4445,0x4644,0x445F,0xC544,0xE5D6,0xD5F5,0xC5C5,0x4575,0xD5C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0x64BF,0x64D4,0x85D4,0x5475,0x7575,0x7554,0x7554,0x7575,0x6575,0x5475,0x4446,0x4646,0x465F,0x4646,0x465F,0x4644,0x5F46,0x4646,0x4646,0x5F46,0x5F46,0x4446,0x4646,0x465F,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x4644,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4446,0x5F46,0x4446,0x464F,0x465F,0x465F,0x465F,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x4646,0x4644,0x5F46,0x4646,0x4644,0x4646,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F46,0x4646,0x4446,0x4646,0x465F,0x4446,0x4F46,0x4646,0x465F,0x4644,0x4646
.word 0x95D4,0x4485,0x5454,0x5454,0x5454,0x5454,0x8585,0x85C4,0x8554,0x85C4,0xB485,0x7554,0x7554,0x8565,0xB5D4,0x85C4,0xC565,0xC495,0x6575,0x5475,0x8575,0x54B4,0x8F44,0x7F41,0x8F44,0x4441,0x7F7F,0x5464,0x547F,0x417F,0x547F,0x649F,0x8F64,0x8F44,0x417F,0x547F,0x418F,0x8F8F,0x548F,0x7F54,0x7F7F,0x7F44,0x417F,0x759F,0x64D4,0x64D4,0x64D4,0x85D5,0xC5AF,0xC575,0xD665,0xD5C4,0x75EF,0xD5FF,0xF5D5,0xF5D5,0xE4D5,0xB4C5,0xC565,0x6554,0xC5C5,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xC4BF,0xD564,0xD4BF,0x7575,0x7575,0x7575,0x7575,0x7575,0x7554,0x7554,0x5475,0x5475,0x7554,0x4654,0x4644,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x465F,0x4446,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x465F,0x4F44,0x4F46,0x4646,0x4446,0x465F,0x4644,0x464F,0x464F,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4F46,0x4F46,0x5F46,0x4646,0x4644,0x4644,0x5F46,0x4646,0x4646,0x465F,0x4644,0x4646,0x4646,0x464F,0x464F,0x465F,0x4446,0x464F,0x465F,0x465F,0x465F,0x4646,0x4646,0x4446,0x4646,0x464F,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4644,0x4F46,0x4F46,0x4646,0x465F,0x464F,0x464F,0x4446,0x4F46,0x4646,0x465F,0x4646,0x464F,0x464F
.word 0xC495,0x7585,0x4454,0x7554,0x8554,0x5485,0x7575,0x8575,0x7575,0xB485,0x7585,0x9575,0x95C5,0x7575,0xC485,0x6585,0x9585,0x95C4,0x7554,0x7585,0x5454,0x4154,0x7F7F,0x7F44,0x417F,0x7F8F,0x7F41,0x8F64,0x417F,0x7F7F,0x5441,0x75AF,0x44AF,0x419F,0x447F,0x548F,0x448F,0x447F,0x8F41,0x4154,0x7F7F,0x7F41,0x547F,0x548F,0xD4AF,0xD4D4,0xC4BF,0xC4BF,0x9F85,0x65C5,0x6565,0x6575,0xC5C5,0xEFD4,0xF5E5,0xD5C5,0xC5B4,0xC575,0x7565,0x65C5,0x74C5,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD474,0xD464,0x85BF,0x85D4,0xD485,0xD495,0xE4C5,0x8595,0x8585,0x7575,0x5475,0x7554,0x7575,0x7575,0x5475,0x4645,0x4646,0x4446,0x4F46,0x5F46,0x5F46,0x4446,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4644,0x5F46,0x4646,0x4646,0x4446,0x464F,0x4646,0x464F,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F,0x4446,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x464F,0x4646,0x465F,0x4646,0x4446,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646
.word 0xB5C4,0x85E4,0x5475,0x5454,0x6575,0x6554,0x5475,0x8575,0x7565,0x5475,0x7554,0xC495,0xC495,0x5485,0x6575,0x5475,0x7575,0x6575,0x9554,0x95C4,0x5465,0x4144,0x417F,0x7F8F,0x4144,0x8F44,0x647F,0x449F,0x7F54,0x447F,0x8F7F,0xD575,0x9F54,0x4454,0x418F,0xC47F,0x418F,0x8F7F,0x8F44,0x448F,0x7F64,0x447F,0x417F,0x547F,0xAF54,0x6464,0x64D5,0x64D4,0xC4D4,0x549F,0x75D6,0x6545,0x65D5,0xC5C5,0x6575,0x6575,0xC575,0x7565,0x65C5,0xD4BF,0xE4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xC4AF,0x75BF,0xD4D4,0x6575,0x5454,0x5454,0x5454,0x5454,0x5454,0x7575,0x8585,0x8575,0x8575,0x7575,0x7554,0x6475,0x6454,0x4645,0x4646,0x4646,0x4646,0x465F,0x464F,0x4446,0x464F,0x4F46,0x4446,0x464F,0x465F,0x465F,0x4644,0x465F,0x464F,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4446,0x4F46,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4644,0x4446,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x465F,0x5F46,0x4646,0x4446,0x4646,0x4644,0x465F,0x465F,0x465F,0x465F,0x465F,0x465F,0x465F,0x4646,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4644,0x4446,0x5F46,0x5F46,0x4646,0x465F,0x4644,0x464F,0x4646,0x4446
.word 0xC495,0xB5D4,0x54C4,0x5454,0x5454,0x5475,0x7554,0x6575,0x5454,0x8554,0x7575,0xC485,0xD485,0x7585,0x5454,0x7575,0x7554,0x5454,0x7575,0x7585,0x4175,0x447F,0x448F,0x417F,0x448F,0x448F,0x448F,0x9F54,0x7F8F,0x417F,0x8F44,0x9F64,0x8F64,0x9F54,0x7F44,0x5441,0x448F,0x417F,0x447F,0x8F7F,0x6444,0x417F,0x7F7F,0x8F44,0x5454,0xAFD5,0xBFC4,0xBFC4,0xBFC4,0xAFC4,0x4554,0x65D6,0x54C5,0x6565,0xC565,0x6545,0xC545,0xC554,0xAF75,0xD4C4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x64D4,0x64D4,0xD4D5,0xC464,0x5485,0x5454,0x6554,0x8575,0x95C4,0x75B4,0x5454,0x7554,0x8575,0xD495,0x7585,0x7575,0x8575,0x5475,0x6475,0x4654,0x4644,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4446,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x465F,0x4644,0x465F,0x4446,0x4F46,0x4F46,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x464F,0x4446,0x4646,0x465F,0x464F,0x465F,0x464F,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x465F,0x465F,0x4646,0x465F,0x4446,0x464F,0x465F,0x465F,0x4646,0x4646,0x4646,0x4644,0x4646,0x465F,0x4F46,0x4F46,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x465F,0x4646
.word 0x85E4,0xD495,0xB4B5,0x5475,0x5454,0x5454,0x5454,0x5454,0x8554,0xB485,0x54B4,0x7575,0x7585,0x7554,0x8585,0x8585,0x8554,0x75C4,0x5454,0x5465,0x7F7F,0x8F41,0x417F,0x7F7F,0x8F44,0x5441,0x7F8F,0x5454,0x447F,0x417F,0x448F,0x549F,0x44AF,0x449F,0x418F,0x7F8F,0x9F44,0xD564,0xD4D4,0x85D4,0x8F9F,0x7F7F,0x7F44,0x8F41,0x9F54,0x6464,0xD4D4,0xD485,0x85D4,0xD464,0x54AF,0x548F,0x549F,0x549F,0x419F,0x9F9F,0x9F54,0x9F41,0xD464,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x64D4,0xBFD4,0xBFC4,0x85C4,0x5475,0x7575,0x8575,0x8595,0x8554,0x85C4,0x7585,0x8554,0x85C4,0x5465,0x5454,0x8575,0x8595,0x7575,0x8575,0x7585,0x5475,0x5475,0x4644,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4446,0x4646,0x4646,0x465F,0x465F,0x465F,0x4446,0x4646,0x465F,0x4646,0x4F46,0x4646,0x4646,0x5F46,0x4446,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4F46,0x5F46,0x5F46,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x464F,0x464F,0x465F,0x465F,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x5F46
.word 0x95D4,0x95B4,0xB5C4,0x7585,0x4454,0x6541,0x7554,0x8575,0x7554,0x7575,0x6585,0x7554,0x5454,0x8565,0x85B4,0x75B4,0x8575,0x7585,0x7575,0x7F54,0x7F41,0x4144,0x7F44,0x7F44,0x8F41,0x8F54,0x547F,0x419F,0x418F,0x447F,0x418F,0xAF44,0x8F75,0x4154,0x448F,0x8F54,0x6454,0x64D5,0xD464,0xBFD4,0xD4C4,0x85D4,0x44AF,0x7F7F,0x4441,0x549F,0xD4AF,0x64D4,0xD4D4,0xD4D4,0xC4D4,0x44AF,0x448F,0x448F,0x448F,0x8F44,0x9F44,0xD4C4,0xE4E5,0xD4E4,0xE4D4,0xD4D4,0xD4C5,0xBFD4,0x64C4,0x64D5,0x8585,0x95C4,0x7585,0xD475,0xC4D4,0x5454,0x5475,0x6554,0x5454,0x5454,0x8575,0x7575,0x5454,0x7565,0x8585,0x7585,0x7575,0x7575,0x7575,0x5475,0x4646,0x465F,0x4646,0x5F44,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x4446,0x4646,0x4646,0x465F,0x4446,0x464F,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x464F,0x5F46,0x4646,0x4446,0x4646,0x465F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x464F,0x465F,0x4F46,0x4646,0x465F,0x5F46,0x4446,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x4646,0x4446,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xB5D4,0x85C4,0x85B4,0x95D4,0x5485,0x5454,0x5475,0x7554,0x7554,0x5454,0x5465,0x8554,0xC485,0x7585,0x7554,0x7554,0x6575,0x5475,0x7F41,0x417F,0x4144,0x548F,0x418F,0x417F,0x8F44,0x4441,0x418F,0x9F44,0x7F44,0x8F41,0x8F7F,0x5444,0x7F8F,0x447F,0x8F8F,0xD554,0x64AF,0x64BF,0xC4AF,0x6464,0x64D4,0x64D4,0x64D5,0xC4D4,0xAFD4,0x9F54,0x6454,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xD474,0x6585,0xC495,0x7595,0x7554,0x5475,0x7575,0xC485,0x9595,0x7575,0xD485,0xB5D4,0x54B4,0x95D4,0x6585,0x5454,0x8565,0x8585,0x7575,0x7575,0x8575,0x7575,0x4665,0x4644,0x465F,0x4F46,0x4F46,0x5F46,0x5F46,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4446,0x464F,0x465F,0x465F,0x4F46,0x4646,0x465F,0x5F46,0x4646,0x464F,0x5F46,0x5F46,0x5F46,0x4446,0x4646,0x4646,0x465F,0x4646,0x465F,0x465F,0x4F46,0x4F46,0x4F44,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4446,0x5F46,0x4646,0x4F46,0x4F46,0x4F46,0x5F46,0x5F46,0x4F46,0x4646,0x4446,0x4646,0x5F46,0x4446,0x5F46,0x5F46,0x4F46,0x4646,0x4644,0x4F46,0x4646,0x464F,0x465F,0x465F,0x465F,0x4644
.word 0xC4C5,0xC495,0xD495,0xC5D4,0x85D4,0x5475,0x5454,0x5454,0x5454,0x5454,0x5454,0x7554,0x8575,0x5475,0x8575,0x7554,0x5454,0x4454,0x417F,0x447F,0x448F,0x448F,0x8F7F,0x547F,0x448F,0x7F7F,0x8F44,0x4144,0x447F,0x448F,0x7F6F,0x5441,0x448F,0x8F41,0x6444,0x64AF,0x64D4,0xC4D5,0xBFD4,0xD4C4,0xAFD4,0xD4D4,0xD464,0xD4D4,0x64D4,0x64D4,0x548F,0xD4AF,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xF4F4,0xF4E4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xD4D4,0x74D4,0xBFD5,0xBFC4,0x85B4,0x75C4,0x8585,0x54B4,0x5454,0x8575,0x7585,0x8585,0xD4C4,0x5485,0x9585,0x95D4,0x7585,0xC485,0x7585,0x7554,0x5454,0x8565,0x8595,0x8585,0x8575,0x8575,0x7575,0x4644,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x464F,0x4646,0x4646,0x4446,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4446,0x4646,0x465F,0x4646,0x4646,0x4F44,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4446,0x4646,0x465F,0x464F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x464F
.word 0xB485,0x4644,0xD495,0xC495,0xC495,0xC495,0x5475,0x5454,0x7554,0x5475,0x5485,0x6554,0x5454,0x5485,0x7575,0x5475,0x5454,0x417F,0x4144,0x8F54,0x5441,0x41AF,0x447F,0x7F41,0x8F41,0x4441,0x7F8F,0x8F54,0x7F8F,0x6F7F,0x7F7F,0x8F41,0x8F44,0x6464,0xD4D5,0xC4D4,0xC4BF,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x41D4,0xD464,0xD4D4,0xE4D4,0xD5D4,0xF4E4,0xE4E4,0xF4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x85D4,0x7575,0x5475,0x7554,0x5454,0x7575,0xD475,0xD4B5,0x85D4,0x8575,0x8585,0x5465,0x7565,0x5475,0x5454,0x5465,0x7575,0x8585,0x8565,0x5475,0x6554,0x8585,0x8575,0x7575,0x6554,0x7575,0x4454,0x4646,0x4646,0x4644,0x465F,0x465F,0x5F46,0x4F46,0x4F46,0x4646,0x4446,0x4446,0x4F46,0x4646,0x464F,0x4646,0x4646,0x4446,0x4646,0x4644,0x464F,0x4644,0x465F,0x4F44,0x4F46,0x4646,0x4446,0x5F46,0x4646,0x4F44,0x4646,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4446,0x464F,0x465F,0x4F46,0x4646,0x4644,0x465F,0x4446,0x4F46,0x4646,0x4646,0x5F46,0x4F46,0x4446,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x465F,0x4646
.word 0x4475,0x4646,0x9585,0x95C4,0x95C4,0xB5C4,0x85C4,0x5454,0x7554,0x7554,0x5475,0x5475,0x8554,0x7575,0x7554,0x5454,0x4141,0x8F44,0x547F,0x548F,0x549F,0x447F,0x8F7F,0x8F44,0x8F44,0x547F,0x7F7F,0x4441,0x7F7F,0x6F44,0x7F41,0x8F44,0x6441,0xBFD5,0x64D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0x64AF,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4F4,0xE4F4,0xC5E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x7585,0xB5C4,0x95C5,0x8575,0xD4D4,0xC4B5,0x8564,0x95B4,0x85D4,0x6554,0x5454,0x7575,0xD485,0xC495,0x7585,0x8575,0xE495,0x85D4,0x7585,0xC495,0x5465,0x7554,0xB485,0x8554,0x8585,0x7575,0x8575,0x4654,0x4F44,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4644,0x4646,0x4646,0x4F46,0x4646,0x465F,0x465F,0x4446,0x4646,0x4F46,0x5F46,0x4646,0x464F,0x465F,0x465F,0x4644,0x4F46,0x5F46,0x4646,0x465F,0x4646,0x5F46,0x4646,0x465F,0x464F,0x4446,0x4F46,0x4646,0x4446,0x4646,0x4646,0x4446,0x4F46,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x5F46,0x5F46,0x4646,0x4646,0x4646
.word 0x4646,0x856F,0x95D4,0x95C4,0x85B4,0xB485,0xD4B5,0xD4D4,0x6585,0x5454,0x4454,0x5454,0x6575,0x5475,0x5454,0x7F44,0x7F44,0x417F,0x9F54,0x9F54,0x8F54,0x8F44,0x417F,0x7F8F,0x5441,0x8F41,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x447F,0x547F,0xD5D4,0xD464,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xE4C5,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0x51D4,0xD4D4,0xD5D4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0x74D4,0xD485,0x5495,0xC485,0xE485,0xC485,0xC495,0x85B5,0x8565,0xB485,0x5475,0x9575,0xC5D4,0x85C5,0x8565,0xB5D4,0x95D4,0x8565,0x85C4,0xB495,0x6585,0xC495,0x85B5,0x7575,0x7554,0x8575,0x85B5,0x7585,0x7554,0x5475,0x4664,0x4646,0x465F,0x4646,0x4446,0x464F,0x465F,0x464F,0x465F,0x464F,0x464F,0x5F46,0x5F46,0x5F46,0x4F46,0x4646,0x4646,0x465F,0x5F46,0x5F46,0x4646,0x4646,0x4644,0x465F,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x4646,0x465F,0x4644,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x465F,0x465F,0x465F
.word 0x4646,0x9546,0x95D4,0x95D4,0xD4C4,0x8585,0x95C4,0x9595,0xB5D4,0x5475,0x5454,0x5454,0x6475,0x5454,0x8F54,0x4141,0x7F8F,0x9F44,0x6444,0x549F,0x8F54,0x7F8F,0x7F44,0x4144,0x548F,0x448F,0x547F,0x7F41,0x7F44,0x416F,0x8F44,0xD464,0xC4AF,0x64D4,0xD4BF,0xD4D4,0xE4D4,0xD5D4,0xD4E4,0xE4E4,0xE4C5,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x64D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4E4,0x85D4,0xB485,0xB485,0x8585,0x85B4,0x75B4,0x5475,0x5454,0x7575,0x9565,0x95C4,0x85D4,0x8575,0xC495,0x85D4,0x8575,0xB475,0x8585,0x5465,0x7575,0x7575,0x7554,0x5475,0x7554,0xB485,0xB495,0x7585,0x7585,0x7575,0x4644,0x4646,0x465F,0x465F,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x4644,0x4446,0x4F46,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x4646,0x4446,0x4646,0x465F,0x4446,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x464F,0x4646,0x465F,0x4446,0x4646,0x4646,0x4644,0x465F,0x4446,0x4F46,0x4446,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4644,0x4446,0x4646,0x465F,0x4646,0x4446,0x4646,0x4646,0x465F,0x4646,0x4446,0x5F46
.word 0x4446,0xD446,0xC495,0x6585,0x4646,0x9565,0xD4D4,0xD4C4,0xC495,0xE485,0x95D4,0x8585,0x5475,0x4144,0x5444,0x7F7F,0x647F,0x418F,0x9FD4,0x9F54,0x448F,0x417F,0x417F,0x448F,0x8F41,0x7F44,0x7F7F,0x447F,0x6F7F,0x5441,0xC4D4,0xC4BF,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xC5D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x64D4,0x64AF,0x9F64,0x6464,0x6454,0x6464,0x6464,0x64D4,0xD464,0x8575,0x8585,0x8585,0xB485,0x7575,0x5454,0x5454,0x7575,0x5454,0xC485,0xC5B5,0xD4C5,0x8585,0x85B4,0x75C4,0x8565,0x85B4,0x5475,0x6575,0x5454,0x8575,0x8575,0x8585,0x7585,0x8575,0x85D4,0x5475,0x7554,0x9585,0x85C4,0x7575,0x7575,0x7575,0x4644,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4F46,0x4646,0x4644,0x465F,0x464F,0x4446,0x4646,0x4646,0x4644,0x465F,0x465F,0x4646,0x4646,0x4F46,0x4646,0x4F44,0x5F46,0x4646,0x4446,0x4646,0x4646,0x465F,0x4446,0x4F46,0x5F46,0x4646,0x464F,0x5F46,0x4F46,0x4646,0x465F,0x5F46,0x5F46,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F,0x465F,0x464F,0x4646,0x465F,0x4646,0x464F,0x464F,0x464F,0x4644,0x4646,0x4644,0x4646,0x4646
.word 0x4646,0xB546,0x85D4,0x4665,0x4446,0x8546,0xD4C5,0xD495,0xC4B5,0x9585,0xD4C4,0xD4C5,0x5485,0x417F,0x8F9F,0x447F,0x8F54,0x5444,0x54AF,0x4454,0x418F,0x7F7F,0x7F44,0x8F54,0x8F44,0x7F8F,0x4144,0x7F41,0x547F,0xD4D4,0xD4AF,0xD4C4,0x64D4,0xBFD4,0xD4C4,0xD4D4,0xD4E4,0xD4E4,0xC5E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xC5E4,0xE4E4,0xC5D4,0xD4E4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x8F64,0x447F,0x8F8F,0x7F8F,0x8F8F,0x7F8F,0x8F8F,0xC475,0x7585,0x7585,0x95B4,0x5475,0x8554,0x7595,0x8575,0x94D4,0x95D4,0x8575,0xD4D4,0x95D4,0x75B4,0x8585,0x5475,0x5475,0x5454,0x8585,0x8575,0xB5C4,0xD4D4,0x7575,0xD4C4,0xC495,0x8585,0xB5C4,0x85C4,0x5454,0x8575,0x95C4,0x85B4,0x8585,0x7554,0x4575,0x4644,0x4646,0x4644,0x465F,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F44,0x4646,0x4644,0x4646,0x4646,0x4446,0x4F46,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4F44,0x4646,0x4644,0x465F,0x465F,0x4F46,0x4646,0x4644,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x5F46,0x4646,0x464F,0x465F,0x465F,0x4646
.word 0x4646,0xC45F,0x65B5,0x4646,0x4646,0xD446,0x95D4,0x85C4,0x4646,0x8585,0xE495,0x6585,0x8F41,0x5444,0x7F8F,0x4144,0x418F,0x548F,0x9F54,0x7F54,0x7F8F,0x7F44,0x547F,0x44AF,0x448F,0x7F41,0x447F,0x546F,0x64D4,0x64D5,0xD4D4,0xD464,0xBFD4,0xD4C4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D5,0xD5E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0x8F64,0x5454,0x5454,0x4454,0x5454,0x4154,0x7575,0x5454,0x5454,0x6554,0x6585,0xD495,0xD4C5,0x8575,0xC495,0xC495,0x8575,0x8585,0x7585,0x7554,0xD485,0x8595,0x8565,0x95C4,0xD4C4,0x54D4,0xC485,0x9595,0x75C4,0x8585,0x85C4,0x54B4,0x7554,0x5475,0x7565,0x5454,0x8565,0x8585,0x75B4,0x7565,0x7585,0x4654,0x4646,0x464F,0x4646,0x465F,0x4644,0x5F46,0x4646,0x465F,0x464F,0x464F,0x465F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4446,0x4F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x465F,0x465F,0x465F,0x465F,0x464F,0x5F46,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4644,0x464F,0x4446,0x464F,0x4646,0x4646,0x465F,0x4644,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x4646,0x8546,0x46D4,0x6F46,0x4646,0x8544,0x85D4,0x4695,0x4646,0xD475,0x75D4,0x8F44,0x547F,0x7F9F,0x7F44,0x8F41,0x447F,0x9F64,0x8F54,0x8F54,0x7F44,0x417F,0x5441,0x8F54,0x7F41,0x7F41,0x6F41,0xBF54,0xBFC4,0xD464,0xD464,0xC4BF,0x64D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x549F,0x54AF,0x54AF,0x9F9F,0x5454,0x5454,0x8575,0xC5D4,0xB5D4,0x85C4,0x95C4,0x85D4,0xC475,0xB485,0x7595,0x7575,0x7575,0x5475,0x9575,0xD4C5,0xD4C5,0x7585,0xC485,0x9595,0x75D4,0x8585,0x85B4,0x5485,0xB485,0x8585,0x7565,0x9585,0x95C4,0x85C4,0x6585,0x5454,0x95B4,0x8585,0x5475,0x7585,0x6475,0x4644,0x4646,0x465F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F46,0x4F46,0x5F46,0x4646,0x4644,0x5F46,0x4646,0x4446,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4446,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x465F,0x4646,0x4646,0x4644,0x4646,0x4446,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4F46,0x4646,0x465F,0x465F,0x4644,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x4F46,0x4646,0x4446,0x5F46,0x5F46,0x5F46,0x4446,0x4F46,0x5F46
.word 0x4646,0x4646,0x4675,0x4646,0x4646,0x9546,0x95D4,0x4646,0x4644,0x8546,0x4154,0x7F7F,0x8F64,0x7F44,0x547F,0x447F,0x8F41,0x9F54,0x9F44,0x417F,0x7F7F,0x7F41,0x9F54,0x448F,0x448F,0x7F7F,0xD454,0x64D4,0xC4D5,0xAFD4,0xD4D4,0xD485,0xD4D4,0x74D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xF4E4,0xE4E4,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xD4D4,0x64D4,0xD464,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0x64D4,0x75AF,0xC4AF,0x9F64,0x7554,0xC485,0x7585,0xC485,0xC495,0x7575,0x7575,0x7575,0x5454,0x5454,0x7554,0x5454,0x8565,0xB5C4,0xC475,0x9595,0xC495,0x6585,0x7575,0x8575,0x5465,0x6575,0x5475,0x8565,0x5465,0x6554,0x7585,0xC485,0xC5B5,0xD4B4,0x85D4,0x5475,0x7554,0x9575,0x7585,0x8575,0x7575,0x6585,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x4646,0x4446,0x4F46,0x4646,0x4646,0x465F,0x465F,0x4446,0x4646,0x464F,0x4646,0x4446,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x4644,0x465F,0x4646,0x4446,0x4646,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x5F46,0x4646,0x4646,0x465F,0x464F,0x465F,0x5F44,0x4646,0x4644,0x464F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4446,0x4646,0x4644,0x4646,0x465F,0x8546,0xB485,0x4644,0x4646,0x4146,0x7F7F,0x547F,0x7F8F,0x7F41,0x8F41,0x417F,0x547F,0x448F,0x449F,0x7F8F,0x447F,0x447F,0x449F,0x417F,0x417F,0x447F,0x64D4,0xBFD4,0xBF85,0x64C4,0x64D5,0xBFD4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD5D4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0x64E4,0x418F,0x8F7F,0x418F,0x418F,0x417F,0xD441,0xD4D4,0xD4D4,0xAF64,0xAF75,0x5454,0x759F,0xC5C4,0x75C4,0x8585,0x75C4,0x5454,0x5454,0x6554,0xD475,0xD4B5,0x6585,0xB5D4,0x95E4,0xD4C5,0x8585,0x85B4,0x85C4,0x5475,0x5454,0x5454,0xD475,0xD4B5,0x8595,0x8565,0xC5D4,0xC5E4,0x54C4,0xC485,0xC5D4,0x95D4,0x8585,0x8565,0x5465,0x7575,0x85D4,0x7575,0x8575,0x8575,0x4644,0x4644,0x4644,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4446,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x464F,0x4F46,0x5F46,0x4646,0x5F46,0x4F46,0x4F46,0x5F46,0x5F46,0x4446,0x4646,0x4646,0x4646,0x465F,0x4546,0x4645,0x465F,0x4646,0x4646,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4646,0x4646
.word 0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4685,0x4646,0x465F,0x416F,0x447F,0x8F41,0x7F44,0x8F44,0x447F,0x7F41,0x8F44,0x418F,0x417F,0x447F,0x7F7F,0x8F41,0x7F44,0x447F,0x448F,0xD454,0xC4BF,0xD564,0xD464,0xD464,0xC4BF,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4C5,0xE4D4,0xE4E4,0xE4D5,0xE4F4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0x8F85,0x4144,0x448F,0x64D4,0x8F64,0x5444,0x8F64,0x7F44,0x417F,0xD564,0xD4D4,0x6454,0x54AF,0xC475,0xD4C5,0x5475,0x5454,0x7554,0x9585,0xD4D4,0xC4B5,0x8575,0xB5C4,0x85D4,0xB495,0xD485,0x8595,0x7565,0x8585,0x7575,0x8554,0xC5C4,0x95D4,0x7585,0xD495,0xC5D4,0x8585,0xC495,0xD4B5,0xB495,0x8585,0xB5D4,0x85B4,0x7575,0xC485,0x7595,0x5454,0x8575,0x8585,0x7575,0x7585,0x4485,0x5F46,0x4646,0x465F,0x465F,0x4646,0x4F44,0x5F46,0x5F46,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x465F,0x464F,0x465F,0x4646,0x4644,0x4646,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x4446,0x4646,0x4644,0x4646,0x464F,0x465F,0x465F,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x464F,0x465F,0x4644,0x4646,0x8544,0x4645,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x4646,0x4F44,0x5F46,0x5F46,0x5F46,0x4646,0x444F,0x464F
.word 0x5F46,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4675,0x4644,0x6F5F,0x7F41,0x8F41,0x7F44,0x417F,0x447F,0x417F,0x448F,0x7F7F,0x4144,0x447F,0x7F7F,0x7F41,0x8F44,0x7F41,0x8F41,0x8F44,0x64D4,0x64D5,0xBFD5,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0x41D4,0x548F,0x54D4,0x419F,0xD58F,0x6464,0x8F8F,0x64D4,0x449F,0x448F,0x649F,0xD4D4,0x9FD4,0x5454,0x8575,0x85C4,0x7554,0xD495,0xC4C5,0xB485,0xB495,0x95B5,0x9575,0x85B4,0x75C4,0x8585,0x95B4,0x54B4,0x7575,0x5454,0x7554,0x9565,0xD4D4,0xE4B5,0x75D4,0x85C4,0xC495,0x8565,0x95B4,0x95B4,0x85C4,0x7575,0xB485,0x7585,0x5465,0x7565,0x7575,0x5454,0x7554,0x75B4,0x7575,0xB485,0x7585,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4446,0x4646,0x465F,0x465F,0x5F46,0x4446,0x4646,0x4646,0x4446,0x4646,0x465F,0x5F46,0x4446,0x4646,0x4646,0x4446,0x4F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4446,0x4646,0x4446,0x5F46,0x4646,0x5F46,0x4446,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4644,0x464F,0x4446,0x9575,0x4646,0x4644,0x4646,0x4646,0x4646,0x4644,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446
.word 0x4646,0x4646,0x4446,0x4F46,0x5F46,0x4646,0x4646,0x445F,0x447F,0x447F,0x7F41,0x417F,0x447F,0x7F8F,0x7F41,0x7F44,0x8F41,0x7F44,0x7F41,0x7F41,0x447F,0x8F41,0x7F7F,0x8F44,0x9F54,0xAFC4,0xBF85,0x6485,0x64D5,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D5,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0x41D4,0x8F54,0xAF54,0x8F44,0x7F44,0x4144,0x449F,0x448F,0xD4AF,0x9FC4,0x8F41,0x547F,0x548F,0xD4D4,0x659F,0x7575,0x7554,0xB485,0xC485,0xC4B5,0x8575,0x8585,0x75B4,0x7554,0x8585,0x7565,0x7565,0x5454,0x7554,0xD485,0xB5D4,0xD4D4,0x7564,0x9585,0x95C4,0x54B4,0x8585,0x85B4,0x8575,0x7575,0x7585,0x6575,0x5475,0x5454,0x7575,0x8564,0x9585,0xC485,0xC485,0x8585,0xD4D4,0xD464,0xD4D4,0xD4D4,0x8574,0x8574,0x7464,0x4654,0x4646,0x4646,0x4644,0x4646,0x464F,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4644,0x4F46,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4446,0x4646,0x4646,0x5F46,0x4646,0x464F,0x465F,0x465F,0x4646,0x5F44,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x7544,0x6595,0x4646,0x4646,0x4644,0x4446,0x4F46,0x4F46,0x4646,0x4644,0x5F46,0x4646,0x465F,0x4644,0x4646,0x465F,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46,0x7F44,0x7F7F,0x4141,0x447F,0x7F7F,0x7F41,0x7F41,0x4144,0x417F,0x448F,0x7F8F,0x7F7F,0x447F,0x417F,0x447F,0x7F7F,0x7F41,0xD454,0xD464,0xD564,0xBFD4,0xD485,0xD4C4,0xC4BF,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xC5D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xD4D4,0x548F,0x449F,0x547F,0x549F,0x549F,0x548F,0x549F,0x449F,0x8F41,0x8F41,0x7F44,0x8F64,0x418F,0xD4AF,0x75D4,0x7554,0xC485,0x7585,0x8585,0x75B4,0x6554,0x5465,0x5454,0x5454,0x5454,0xD485,0xE4C5,0x95D4,0x6585,0x8585,0xC495,0x8595,0x7575,0x85B4,0x85C4,0x6465,0x7575,0x5475,0x6554,0x6575,0x5475,0x7554,0x8585,0xC485,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D5,0xD4D4,0x64D4,0x4464,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x4446,0x4646,0x465F,0x4646,0x4446,0x4F46,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F46,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x4F46,0x4646,0x4646,0x9554,0x45C4,0x4644,0x465F,0x5F46,0x4646,0x464F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4446,0x464F
.word 0x5F46,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x444F,0x447F,0x417F,0x8F44,0x7F41,0x4441,0x7F7F,0x7F44,0x448F,0x447F,0x7F7F,0x4441,0x7F41,0x7F41,0x7F41,0x417F,0x447F,0x647F,0xAFD4,0xBFC4,0xAFC4,0xC485,0xC4BF,0xD4BF,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE441,0x41D4,0x447F,0x9F54,0x8F54,0x6444,0x64D5,0x54AF,0x9F44,0x9F54,0x4454,0x448F,0x418F,0x6454,0x8F54,0x8F54,0xD485,0x75D4,0x8585,0x6554,0x5475,0x5454,0xB585,0x95E4,0x7575,0xD4D4,0xB495,0x9575,0x95C4,0xC5D4,0x85C4,0xB485,0xC485,0x6585,0x5475,0x6554,0x5454,0xC485,0xB5D4,0x7585,0xD475,0xE4B4,0x85D4,0x8575,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD4D4,0x4174,0x4646,0x4646,0x4446,0x4646,0x464F,0x4646,0x4646,0x4644,0x5F46,0x4446,0x4646,0x4646,0x4646,0x4F44,0x4F46,0x4646,0x465F,0x4646,0x465F,0x4644,0x5F46,0x5F46,0x5F46,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x4644,0x465F,0x4646,0x465F,0x4646,0x464F,0x4646,0x4646,0x5444,0xA585,0x4665,0x4646,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x4646,0x5F46,0x5F46,0x5F46,0x4646,0x465F,0x464F,0x4646
.word 0x4646,0x4646,0x4646,0x4446,0x5F46,0x5F46,0x7F44,0x7F41,0x4441,0x7F8F,0x447F,0x8F7F,0x7F41,0x7F41,0x8F41,0x7F7F,0x4441,0x8F7F,0x7F44,0x7F7F,0x4144,0x7F7F,0x417F,0xBF54,0xD4C4,0xD564,0xD464,0xD564,0xD464,0xD464,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D5,0xE4E4,0x41D4,0xE4D4,0x8F41,0x9F44,0xD564,0x9F54,0x8F41,0xD564,0xAF64,0x9F44,0x5454,0x549F,0x418F,0x8F44,0x9F8F,0x8F44,0x5454,0xD48F,0xD4D4,0x7575,0x8554,0xC495,0x7585,0x95C4,0xD4B4,0x5485,0xC495,0x85D4,0x8575,0x85B4,0xB485,0x6595,0x7575,0x6575,0x5454,0xC585,0xB5D4,0x7585,0xD485,0xD4C5,0x85D4,0x8575,0x95D4,0xB495,0xD485,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4C5,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4BF,0xD4AF,0xD464,0xD4D4,0xD4D4,0x4654,0x4F44,0x4F46,0x5F46,0x4646,0x4644,0x4646,0x4F46,0x4646,0x465F,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x4646,0x4446,0x4646,0x464F,0x4446,0x464F,0x4646,0x4644,0x4646,0x4646,0x4644,0x8544,0xB495,0x4644,0x4644,0x465F,0x4644,0x4646,0x4446,0x5F46,0x4446,0x464F,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646
.word 0x4646,0x4644,0x4644,0x4646,0x4646,0x446F,0x7F7F,0x7F44,0x8F41,0x7F44,0x7F41,0x7F41,0x7F44,0x447F,0x7F8F,0x4144,0x7F7F,0x8F41,0x417F,0x7F7F,0x7F41,0x7F44,0x9F54,0xD475,0xAF64,0xBFC4,0xD564,0x85BF,0xD4D4,0xC4BF,0xD574,0xD4D4,0xD4D4,0xD4E4,0x64D4,0xD4D4,0xC5D4,0xD4E4,0xD4E4,0xE4D5,0xD441,0x8FE4,0x4144,0x8F54,0x6441,0x448F,0x7F8F,0x648F,0x64D5,0x44AF,0x549F,0x549F,0x8F54,0x5454,0x9F44,0x5454,0x549F,0x7F8F,0xD4D4,0x75D4,0x6575,0x7575,0x5485,0xC475,0x8585,0xB454,0x9585,0x75C4,0x7575,0x7575,0x7575,0x8554,0xE495,0xD495,0xB4B5,0xC475,0xD485,0x85D4,0x9575,0x95C4,0x95D4,0x8575,0xB485,0xD464,0xC4BF,0xD4D4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xE4D4,0xD4D5,0xE4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xBFD4,0xD4D4,0xD4D4,0x5474,0x4645,0x4646,0x4646,0x5F46,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4644,0x465F,0x4446,0x464F,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4446,0x5F46,0x4646,0x4F44,0x4646,0x4644,0x4646,0x4446,0x7565,0x4644,0x4646,0x4646,0x4646,0x465F,0x464F,0x465F,0x465F,0x6544,0xB495,0x4595,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x4646,0x4644,0x465F,0x4646,0x4644,0x4446,0x5F46
.word 0x5F46,0x4646,0x465F,0x5F46,0x6F46,0x7F41,0x4441,0x418F,0x7F44,0x7F41,0x447F,0x7F8F,0x7F41,0x7F41,0x7F44,0x8F41,0x7F44,0x4144,0x7F7F,0x7F44,0x7F44,0x7F41,0xD454,0x64D5,0x64D5,0xC4D4,0x85AF,0x64D4,0xC4BF,0xD564,0xD4BF,0xD4D4,0xE4D4,0xD4D4,0xBF64,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0x41D4,0x41E4,0xAF54,0x9F54,0x447F,0x549F,0x5464,0x8F41,0xD541,0x6464,0x549F,0xAF54,0x9F54,0x9F44,0x8F41,0x9F54,0x9F54,0x4154,0x647F,0x85D4,0x5454,0x5475,0x5454,0x7575,0x5475,0x7575,0x7575,0x5475,0x5454,0x5454,0x8575,0x7575,0x95C4,0x95C4,0x95D4,0x8575,0x95C4,0xB495,0x8575,0x85B4,0xB495,0x7575,0x6475,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4C5,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBF64,0xD4D5,0x4464,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646,0x464F,0x4646,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4F46,0x4646,0x4F44,0x4F46,0x4646,0x4646,0x9554,0x4665,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x9575,0xC495,0x4685,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x464F,0x465F,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x416F,0x7F7F,0x7F41,0x7F44,0x7F8F,0x447F,0x417F,0x447F,0x7F7F,0x7F44,0x7F41,0x447F,0x417F,0x448F,0x7F7F,0x7F7F,0x7F41,0x6444,0x64AF,0x64BF,0xBFD4,0x64C4,0x64D4,0xD4D5,0xD464,0xD4D4,0xD485,0x61D4,0xD4D4,0x64D4,0xC4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x448F,0x44AF,0x448F,0x9F54,0x8F44,0xD554,0x9F64,0x9F44,0xAFC4,0x8F54,0x9F44,0x6454,0x549F,0x9F54,0x9F54,0x5454,0x549F,0x9F54,0x5454,0x6585,0xC485,0x7575,0x7585,0x7554,0x5454,0x5454,0x5475,0x8585,0x95C4,0xC5D4,0x5485,0xC485,0xB495,0xC485,0x8575,0xB485,0x7585,0x7575,0x7575,0x5475,0x6454,0xC4AF,0xD4BF,0xE4D4,0xD4E4,0xD4C5,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4AF,0x4644,0x4644,0x4646,0x4644,0x465F,0x4646,0x5F46,0x5F46,0x4446,0x464F,0x4646,0x4646,0x4646,0x465F,0x4446,0x464F,0x4F44,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4446,0x6446,0x6495,0x4646,0x464F,0x465F,0x4644,0x4646,0x4446,0x5446,0xC495,0x8585,0x4646,0x5F44,0x4646,0x4646,0x4646,0x4446,0x4646,0x4446,0x4646,0x5F46,0x5F46,0x5F46,0x4446,0x464F,0x4646,0x5F46,0x5F46
.word 0x4646,0x4644,0x465F,0x6F44,0x7F41,0x4441,0x417F,0x417F,0x7F44,0x7F41,0x8F41,0x7F7F,0x7F41,0x7F41,0x7F44,0x7F41,0x447F,0x418F,0x7F7F,0x4441,0x417F,0x548F,0xC4D5,0xBFC4,0x64C4,0xC4BF,0xC4BF,0xC4AF,0xC4BF,0xAFD4,0xD4D4,0xE564,0xD4D4,0xBFD4,0xAFC4,0x417F,0x417F,0x418F,0xE464,0xC5D4,0x9F64,0x8F54,0x8F44,0xAF54,0x6464,0x449F,0x9F8F,0x5464,0x418F,0x448F,0xD564,0x4441,0x549F,0x54AF,0x54AF,0x9F54,0x9F54,0x9F54,0x9F54,0x9F54,0x7554,0x7575,0x7585,0x9575,0x5485,0x7585,0xC485,0xB4C5,0x9575,0x85C4,0xD4C4,0x6585,0x8585,0x85B4,0x8585,0x7565,0x7575,0x5454,0x5454,0x5454,0x7575,0xAF54,0x64C4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xC5D4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xC5D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x4461,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4644,0x465F,0x4644,0x4646,0x465F,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4646,0x4644,0x4646,0x5F46,0x5F46,0x4646,0x4644,0x4446,0x8575,0x4654,0x4646,0x5F46,0x4F46,0x5F46,0x5F46,0x9544,0x8585,0x75B4,0x4646,0x4646,0x4646,0x4644,0x4446,0x4646,0x5F46,0x4646,0x464F,0x4646,0x4644,0x4646,0x464F,0x4646,0x465F,0x4646,0x4646
.word 0x4646,0x4646,0x5F46,0x446F,0x7F7F,0x8F44,0x7F41,0x7F44,0x7F7F,0x7F7F,0x7F44,0x447F,0x7F7F,0x7F44,0x7F41,0x8F44,0x417F,0x7F7F,0x7F44,0x417F,0x7F44,0xD4C4,0x64BF,0x64D4,0x64D5,0x64D5,0x64D5,0x64D4,0x64D5,0xC4D4,0xD4D4,0xD4AF,0xD4D4,0x54D4,0xD464,0x8F41,0x7F44,0x6454,0xD4E5,0xE4D4,0xD4D4,0x8F41,0x417F,0x6444,0xAFD5,0x8F44,0x4441,0x8F8F,0x8F44,0xD454,0xE4E4,0xE4E4,0x8FC4,0xD4D4,0x8F44,0x9F54,0xAF75,0x5454,0x549F,0x549F,0x8F9F,0x5454,0x5475,0x6575,0x6585,0x7585,0x8585,0x6585,0xB485,0x8585,0xB495,0x5475,0x7575,0x7575,0x5465,0x5454,0x9585,0x8565,0xC495,0x8595,0x5454,0xC4AF,0xC4BF,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E4,0xE4E4,0xE4D5,0xD4E4,0xD4D5,0xD4E4,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4C5,0xC5E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x4641,0x4646,0x465F,0x4446,0x5F46,0x4446,0x4646,0x4646,0x4F46,0x4646,0x4F46,0x5F46,0x4646,0x4646,0x5F46,0x5F46,0x5F46,0x4F46,0x4646,0x4F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x7544,0x54A5,0x4646,0x4646,0x4646,0x4646,0x4646,0x8585,0x95D4,0x4685,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x5F46,0x4646,0x465F,0x4446,0x464F,0x4646,0x465F,0x4646,0x4646,0x4446,0x5F46,0x4446
.word 0x4446,0x4646,0x7F46,0x7F44,0x4141,0x447F,0x7F7F,0x7F41,0x7F41,0x4144,0x7F41,0x417F,0x417F,0x417F,0x417F,0x447F,0x7F7F,0x7F44,0x7F7F,0x417F,0x418F,0x859F,0xAFC4,0xAFC4,0xBFC4,0xD464,0xC4AF,0x64BF,0xAFD4,0x64D4,0xD4D4,0xD464,0xE4D4,0xAF41,0x647F,0xD4D5,0xE4D4,0xE4D4,0xD4D4,0x41D4,0x448F,0x547F,0x9F54,0x8F8F,0x447F,0x548F,0xAF64,0x9F75,0x9F44,0x41D4,0x548F,0x8F7F,0x447F,0xE441,0xD4E4,0x8F41,0x447F,0x549F,0x54AF,0x9F64,0x4154,0x5444,0x5454,0x5454,0x7554,0x7554,0x6565,0x5454,0x7575,0x7575,0x6575,0x7554,0x5454,0x5454,0x8575,0xC5C4,0xD4E4,0x8575,0x85C4,0x5475,0xD554,0xC464,0xD464,0xD4D4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4C5,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD464,0x5464,0x4646,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x465F,0x4644,0x465F,0x5F46,0x4646,0x4446,0x9585,0x4645,0x465F,0x4646,0x4644,0x4646,0x9585,0xB4D4,0x4675,0x4644,0x465F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x464F,0x465F,0x4646
.word 0x4646,0x465F,0x415F,0x7F7F,0x7F41,0x7F41,0x7F41,0x7F7F,0x7F7F,0x447F,0x7F7F,0x7F44,0x7F7F,0x7F44,0x447F,0x417F,0x7F7F,0x417F,0x417F,0x447F,0xC47F,0xBF54,0x64C4,0x64D4,0x64D4,0x64D5,0x64D4,0x64D5,0x85D5,0xC4BF,0xC4BF,0xD464,0x6441,0x85D5,0x64D5,0xC4BF,0xD4D4,0xD4D4,0x6464,0x448F,0x7FD4,0xAF54,0xC4C4,0x8F64,0x4144,0x648F,0xD4D5,0x9F64,0xC454,0x547F,0xE4D4,0xE4D4,0x54D4,0x648F,0x8F7F,0x7F44,0x8F44,0x8F44,0x9F54,0x5454,0x549F,0x8F8F,0x5444,0x5454,0x5454,0x5454,0x5454,0x5475,0x9585,0xD4E4,0x85B5,0x9554,0x95C4,0x95D4,0x8575,0xC485,0x8595,0x8554,0x5475,0xAF54,0xAF64,0xD464,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D5,0xE4D4,0xD4D4,0xD4C5,0xE4C5,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xC5E4,0xD4E4,0xE4D4,0xD4E4,0xBFD4,0x4464,0x4646,0x4646,0x465F,0x465F,0x464F,0x464F,0x4644,0x4646,0x4446,0x4646,0x4646,0x4446,0x464F,0x465F,0x4644,0x465F,0x4646,0x4446,0x4646,0x4646,0x7545,0x4644,0x4646,0x4646,0x4646,0x4646,0xB575,0x4585,0x4446,0x5F46,0x5F46,0x4446,0xB485,0x8595,0x4675,0x4646,0x4646,0x464F,0x4446,0x4646,0x4644,0x4646,0x4644,0x465F,0x4646,0x464F,0x465F,0x464F,0x4646,0x4646,0x4646,0x465F
.word 0x4646,0x6F46,0x417F,0x417F,0x447F,0x7F7F,0x7F44,0x7F44,0x447F,0x8F41,0x7F41,0x417F,0x7F7F,0x4441,0x417F,0x447F,0x417F,0x7F7F,0x7F7F,0x7F44,0xD441,0x64D5,0xBFD4,0xBFC4,0xBFC4,0xD564,0xC4AF,0xC4BF,0xC4BF,0xD464,0xD4D4,0xAFD4,0xD4D4,0xAFD4,0xD4C4,0xD464,0xD4D4,0xE4E4,0x9F64,0xD454,0x8FD4,0x6441,0x9FD5,0x448F,0x549F,0x8F54,0x418F,0x8F54,0xBF54,0xD4D4,0xE4E4,0xD4E4,0x41E4,0x8F7F,0x7F44,0x7F41,0x8F41,0x8F41,0x8F44,0x9F44,0x9F44,0x7F54,0x4441,0x5441,0x5454,0xC485,0x8595,0x5485,0x85B4,0xB485,0xC485,0xC475,0xC495,0xB485,0x8554,0x85B4,0x75B4,0x6565,0x5454,0xD564,0xD564,0xBFD4,0xD4D4,0xD4D4,0xC5E4,0xE4E4,0xD4E4,0xE4E4,0xD5E4,0xE4D4,0xD5D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0x6464,0x4654,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4F46,0x5F46,0x4F46,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4646,0x465F,0x5446,0x4475,0x4446,0x4646,0x465F,0x4644,0x8544,0x75D4,0x4646,0x4646,0x4646,0x4646,0x9585,0x85B4,0x4675,0x5F46,0x4646,0x4646,0x4646,0x445F,0x4675,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646
.word 0x5F46,0x7F7F,0x7F41,0x447F,0x7F7F,0x417F,0x7F7F,0x7F7F,0x7F41,0x7F7F,0x447F,0x7F7F,0x7F44,0x417F,0x447F,0x8F7F,0x447F,0x447F,0x7F7F,0x8F41,0xD454,0xD464,0x64D4,0x64D5,0x64D5,0x64D4,0x64D4,0x64D4,0xD4D4,0xC4BF,0xD464,0xC4D4,0xD464,0xD4D4,0xBF64,0xAFC4,0xAF85,0xD454,0xD4D4,0x7FD4,0x41D4,0x548F,0x417F,0x9F44,0xD464,0x64D4,0x8F54,0xAF44,0xC464,0xE4E4,0xE4D4,0xC5D4,0x8F64,0x7F44,0x8F41,0x8F44,0x7F44,0x8F44,0x4441,0x549F,0x448F,0x449F,0x8F8F,0x7554,0x5444,0x5454,0x7565,0x75C4,0x7575,0x7585,0x7575,0x8554,0x75B4,0x7575,0x7554,0x6554,0x5454,0x5475,0xAF64,0xAF64,0xAFC4,0xC485,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xE4D4,0xD5D4,0xD4E4,0xD4E4,0xD4E4,0x51D4,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4446,0x7554,0x4644,0x4646,0x4646,0x4646,0x5446,0x9585,0x4654,0x4446,0x4646,0x655F,0xC495,0x8585,0x4475,0x4646,0x4646,0x4644,0x465F,0x8565,0x4644,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4446,0x4646,0x4646,0x465F,0x465F,0x4644
.word 0x7F46,0x4441,0x416F,0x7F7F,0x4141,0x447F,0x7F7F,0x447F,0x416F,0x7F44,0x7F41,0x7F7F,0x7F7F,0x7F41,0x417F,0x447F,0x417F,0x7F7F,0x447F,0x447F,0xC4AF,0xD4AF,0xD564,0xD464,0xC4BF,0xC4BF,0xC4BF,0xD4AF,0xD564,0xD464,0xAFD4,0xD4C4,0xD4AF,0xD4D4,0xC4D4,0xAF64,0xD454,0xD4D4,0x64E4,0x648F,0x449F,0xAF54,0x8F44,0x8F41,0x417F,0x8F9F,0x547F,0x75AF,0xC4AF,0x8F64,0x5454,0x8F8F,0x7F44,0x448F,0x448F,0x418F,0x448F,0x448F,0x548F,0x548F,0x418F,0x548F,0x4164,0x649F,0x54C4,0x5454,0x5454,0x5454,0x5454,0x7565,0x5454,0x5475,0x5475,0x5454,0xC485,0x95C5,0x5485,0x5454,0x75AF,0x64D5,0x85D5,0xD4BF,0xD4D4,0xE4E4,0xD4E4,0xD5E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xD4C5,0xE4E4,0xD4C5,0xE4E4,0xC5D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4C5,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x4641,0x465F,0x4F46,0x4446,0x4646,0x4646,0x4644,0x4644,0x4646,0x4646,0x5F46,0x4646,0x4F44,0x4446,0x5F46,0x4446,0x4646,0x4646,0x4646,0x5F46,0x8544,0x4675,0x4446,0x5F46,0x4646,0x4644,0xC485,0x4685,0x465F,0x465F,0x7546,0x85C5,0x95B4,0x4475,0x4646,0x4644,0x465F,0x6546,0x4485,0x4446,0x4646,0x4644,0x4446,0x4F46,0x4646,0x5F46,0x4646,0x4644,0x4446,0x5F46,0x4646
.word 0x7F7F,0x7F41,0x7F41,0x7F44,0x7F44,0x7F7F,0x447F,0x7F6F,0x7F7F,0x417F,0x7F7F,0x447F,0x7F7F,0x447F,0x447F,0x7F7F,0x447F,0x7F7F,0x417F,0x417F,0xD4C4,0xD564,0xC4BF,0x85BF,0xD564,0xD464,0xD564,0xD464,0x64BF,0xBFD4,0xD4C4,0x64D4,0x64D4,0xD4D4,0xAFD4,0xD4D4,0x64D4,0x648F,0xAFD4,0xD4D4,0x9FC4,0x5454,0x548F,0xD564,0x64D4,0x7F44,0x5454,0x448F,0x419F,0xD49F,0xE4E4,0xE4D4,0x41D4,0x417F,0x418F,0x8F44,0x8F44,0x8F41,0x4441,0x449F,0x448F,0xE48F,0x64D4,0x9F44,0x7564,0x75C4,0x5454,0x5454,0x5454,0x7554,0x5475,0x8595,0x8585,0x5485,0x8585,0x85B4,0x5475,0xAF54,0xAFC4,0xD464,0xD4BF,0xD4D4,0xD4D4,0xE4D4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E5,0xC5E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xC5E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x54D4,0x4646,0x4646,0x4F46,0x5F46,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4F46,0x4646,0x4F46,0x5F46,0x5F46,0x5F46,0x4646,0x5446,0x7595,0x4646,0x4646,0x4646,0x5F46,0x8574,0x6595,0x4646,0x4646,0x8546,0xB495,0x7595,0x4575,0x4646,0x465F,0x4646,0x8544,0x4585,0x4646,0x4F46,0x5F46,0x4646,0x4646,0x4446,0x4646,0x465F,0x464F,0x4646,0x4646,0x5F46
.word 0x417F,0x447F,0x417F,0x417F,0x417F,0x417F,0x417F,0x7F7F,0x7F41,0x7F41,0x447F,0x7F6F,0x7F41,0x7F41,0x417F,0x418F,0x7F7F,0x7F7F,0x7F44,0x9F54,0x64D4,0x64D4,0xD4D4,0xD564,0x85BF,0xC4BF,0xC4AF,0xC4BF,0xD564,0xD464,0xD464,0xD4D4,0xD4D4,0xD49F,0x64D4,0x41D4,0xD564,0xD464,0xD4D4,0xD454,0xD4D4,0x7F8F,0xAF54,0xD4C4,0x9F64,0x548F,0xD4AF,0x64D4,0x548F,0xE464,0xE4D4,0xE4E4,0x7FE4,0x7F44,0x7F44,0x8F41,0x4144,0x448F,0x548F,0x449F,0x418F,0x547F,0xD5E4,0x549F,0x649F,0xD454,0x5485,0x7554,0x7554,0x8585,0x54C4,0x7575,0x7575,0x7575,0x7575,0x7575,0x5454,0x75AF,0xD564,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xC5E4,0xD4E4,0xD4D4,0xC5E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xC5E4,0xE4D4,0xD4D4,0x64D4,0x4664,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F46,0x6546,0x7585,0x4644,0x465F,0x5F46,0x4646,0x8544,0x9585,0x4646,0x4446,0xB444,0x8595,0x95B4,0x4585,0x4646,0x4646,0x4646,0x9575,0x4675,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x4646
.word 0x7F7F,0x417F,0x7F7F,0x8F44,0x7F7F,0x447F,0x7F7F,0x7F44,0x447F,0x7F6F,0x7F7F,0x7F7F,0x7F6F,0x447F,0x447F,0x7F7F,0x4441,0x417F,0x8F54,0x64C4,0xD4D5,0xC4BF,0xD4AF,0x74D4,0x64D5,0x64D5,0x64D4,0xD4C4,0xD4AF,0xAFD4,0xBFC4,0xD464,0xC4BF,0x64D4,0xD4D4,0xD49F,0xD49F,0xC4BF,0xD464,0xBFD4,0xD464,0x8F41,0x447F,0x417F,0x449F,0xD554,0xD4D4,0x7FD4,0xD554,0xD4D4,0xE4E4,0xD4E4,0x41E4,0x417F,0x417F,0x8F44,0x4441,0x7F8F,0x8F44,0x8F44,0x8F44,0x7F44,0xD48F,0x54D4,0x649F,0x649F,0x95C4,0x5465,0x5465,0x7565,0x5475,0x5475,0x5454,0x7554,0xC485,0x5495,0xAF64,0xBF75,0xBFC4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D5,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4E4,0xC5D4,0xE4D4,0xE4C5,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xC5E4,0xE4D4,0xD4D4,0xC5D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x54BF,0x4646,0x4644,0x465F,0x4F44,0x5F46,0x4646,0x465F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4646,0x465F,0x465F,0x465F,0x465F,0x4646,0x4646,0x9575,0x4675,0x4646,0x4646,0x4646,0xB444,0xB495,0x4654,0x4646,0x9565,0x95B4,0xB485,0x4585,0x4446,0x4646,0x4444,0x9595,0x4675,0x4644,0x4446,0x4646,0x4644,0x465F,0x465F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x465F
.word 0x7F41,0x7F44,0x4144,0x7F7F,0x7F44,0x4441,0x7F8F,0x417F,0x7F7F,0x7F41,0x7F44,0x447F,0x417F,0x417F,0x417F,0x447F,0x7F7F,0x547F,0x41D4,0x9FD4,0xD464,0xD4C4,0x6485,0x64D5,0xBFD4,0xBFC4,0xD485,0xC4AF,0x64D4,0xD4D4,0xD464,0x64D5,0xD4D4,0xD4D4,0xC4AF,0x64D4,0x6464,0xD5D4,0xD4BF,0xD4D4,0x54D4,0x41D5,0xAF44,0x64C4,0x8F8F,0x7F7F,0x8F41,0x8F54,0x9F41,0xD485,0xE4E4,0xE4E4,0x7FD4,0x7F44,0x7F44,0x448F,0x7F8F,0x4144,0x5441,0x549F,0x4141,0x448F,0x548F,0x64E4,0x548F,0x54D5,0xC464,0x54D4,0x5454,0x5454,0x5454,0x5475,0x6554,0xC485,0x8575,0x9F65,0x6454,0x64D5,0xD4D5,0x85D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4D5,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD5D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD5E4,0xE4E4,0xD4E4,0xC5E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x64D4,0x4654,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4F46,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x8554,0x6585,0x4446,0x4646,0x4644,0x8546,0x85C4,0x4675,0x4444,0x9585,0xB485,0x8585,0x45B4,0x4646,0x4646,0x7546,0xB495,0x4665,0x5F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4446,0x4646,0x4644,0x4646,0x4446
.word 0x417F,0x417F,0x417F,0x417F,0x547F,0x8F41,0x417F,0x447F,0x7F7F,0x447F,0x417F,0x7F7F,0x7F6F,0x447F,0x7F7F,0x7F41,0x7F54,0x8F54,0xAFC4,0xAF85,0xBFC4,0xD4D4,0xD4BF,0xBFD4,0xD4C4,0xD485,0xD4AF,0xBFC4,0xBFC4,0xBF85,0xD4C4,0xD464,0xBF64,0xD4C4,0xD485,0xC4BF,0xAFD4,0x64D4,0x64C4,0xD4D4,0xD4D4,0x9FD4,0x6454,0xAFD4,0x4144,0x6464,0x8FD4,0x5441,0xD464,0x44AF,0x6451,0x448F,0x6441,0x8F8F,0x417F,0x417F,0x417F,0x448F,0x448F,0x418F,0xAF54,0x4154,0x7F8F,0xD4D4,0x8F54,0xBFD4,0x5464,0xD464,0x5485,0x5454,0x7585,0x7585,0x8585,0x8575,0x5454,0x6454,0xD5AF,0xD464,0xD474,0xD4D4,0xD4D4,0xE4E4,0xC5D4,0xE4D4,0xE4E4,0xE4C5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xC5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0x4464,0x4646,0x4646,0x5F46,0x4446,0x4F46,0x4646,0x5F46,0x4646,0x4644,0x5F46,0x4646,0x5F46,0x5446,0x4465,0x4646,0x5F46,0x4646,0x5F46,0x8554,0x75B4,0x5F46,0x5F46,0x5F46,0x8546,0x8585,0x4675,0x7545,0x7595,0x95D4,0xB5C4,0x4585,0x4646,0x465F,0x9554,0x85D4,0x4665,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4F46,0x5F46,0x4646,0x5F46,0x4646
.word 0x447F,0x417F,0x447F,0x7F7F,0x9F54,0x8F54,0x7F44,0x417F,0x417F,0x8F44,0x7F41,0x4154,0x447F,0x7F7F,0x7F7F,0x7F7F,0x8F64,0x9F54,0xD4C4,0xD48F,0xD4C4,0xD464,0xD4C4,0xD4D4,0xD464,0xC4AF,0x64C4,0x85D4,0x64D4,0xD4D5,0xBF64,0xBFD4,0xD4C4,0xBF64,0xD4D4,0xBFC4,0xD464,0xD4D4,0xAFD4,0xBFC4,0xD4D4,0x64D4,0x549F,0x54AF,0x548F,0xD4D5,0x8F64,0xAF54,0xD4D5,0x8FD4,0x448F,0xD48F,0xD4E4,0x7F41,0x7F44,0x7F44,0x7F44,0x8F44,0x8F41,0x9F54,0x5454,0x448F,0x417F,0xE47F,0x8F64,0xE454,0xD554,0x658F,0x7595,0x5454,0x7575,0x5485,0x5465,0x5475,0x5454,0x75AF,0xBF64,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xC5D4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xD4E4,0xD5D4,0xE4E4,0xD4E4,0xE4E4,0xF4E4,0xE4E4,0xD4E4,0xE4F4,0xE4D4,0xD4E4,0xD4C5,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xAFD4,0x4646,0x465F,0x4644,0x465F,0x4646,0x464F,0x4644,0x4646,0x464F,0x4646,0x4446,0x4646,0x4446,0x5485,0x4644,0x4646,0x4646,0x4646,0x9544,0x8595,0x4645,0x4646,0x4646,0x9554,0x85C4,0x4575,0x8554,0x85B4,0x85C4,0x85C4,0x6575,0x4646,0x4646,0x9575,0x95B4,0x4645,0x4644,0x4646,0x465F,0x4F44,0x4446,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646
.word 0x417F,0x447F,0x7F7F,0x447F,0x5464,0x44AF,0x547F,0x647F,0x447F,0x419F,0x7F7F,0x5454,0x417F,0x447F,0x447F,0x417F,0x41C4,0x64D4,0xD4D4,0x64C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4BF,0xD4BF,0xBFD4,0xD585,0xD464,0xD4C4,0xD4D4,0xD564,0xBF64,0xC4D4,0x64D5,0xD4AF,0xD4D4,0x85D4,0xD464,0xD4D4,0x8FC4,0x8F54,0x8F44,0x8F7F,0x448F,0x447F,0x418F,0x447F,0x417F,0xE4D4,0xD4E4,0x8F41,0x7F41,0x7F41,0x8F41,0x7F41,0x4144,0x9F44,0x8F44,0x8F44,0x7F41,0xC444,0x41D4,0xE454,0xD48F,0x8F41,0x8544,0x5454,0x5454,0x5454,0x5454,0x5454,0x5454,0xBF64,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4C5,0xE4D4,0xD5D4,0xE4D4,0xD4D4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4C5,0xD4E4,0xD4D4,0xD4D4,0xC5E4,0xD4D4,0xD4D4,0x85D4,0x4651,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4644,0x464F,0x4646,0x4646,0x7544,0x7595,0x4446,0x5F46,0x4646,0xB444,0x7595,0x4645,0x4644,0x4446,0x9575,0x85B4,0x7585,0x8585,0x9585,0x85C4,0xC495,0x8575,0x4446,0x5446,0x9595,0x75B4,0x4646,0x4646,0x4444,0x4646,0x4646,0x4F46,0x4646,0x4446,0x5F46,0x4446,0x4646,0x465F,0x4446
.word 0x7F41,0x8F54,0x7F7F,0x5441,0x54AF,0x9F54,0x4154,0xAF54,0x9F54,0x4464,0x418F,0xBF54,0x547F,0x417F,0x7F41,0x7F7F,0x8FD4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4BF,0xD4D4,0xD4C4,0xD464,0xBF64,0xAF64,0x64D4,0xC4BF,0xD4BF,0xC464,0xD464,0xD464,0xD4D4,0xE464,0xE4D4,0xBFD4,0xBF85,0x9FC4,0x8F54,0x5441,0xC464,0x418F,0x9F54,0x6485,0x418F,0x9FC4,0xE4D4,0xE4E4,0x8F64,0x7F44,0x7F44,0x7F44,0x7F44,0x8F8F,0x448F,0x548F,0x418F,0x447F,0x418F,0x8FE5,0xD444,0x6454,0x449F,0x858F,0x4485,0x7554,0x5485,0x7585,0x5454,0x64AF,0x64D5,0xD4D5,0xD4D4,0xD4D4,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D5,0xE4E4,0xE4E4,0xD4E4,0xE5E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D5,0xD4D4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5464,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4446,0x4F46,0x4646,0x4646,0x655F,0x4644,0x4446,0x8595,0x4665,0x4646,0x4646,0xB444,0xC485,0x4675,0x5F46,0x7546,0xC495,0x7585,0x9575,0x85B4,0x95B4,0x95B4,0x95B4,0x7575,0x4654,0x8544,0x95B4,0x6585,0x4446,0x4446,0x4685,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x465F,0x4446,0x4646
.word 0x547F,0x418F,0x547F,0x648F,0xC4AF,0x549F,0x54AF,0x64D5,0xD554,0xAF64,0x7F44,0x64C4,0x647F,0x7F7F,0x5454,0x4441,0x54D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD574,0xD4D4,0xD4D4,0xC4D4,0x649F,0x85D5,0xBFC4,0xD4D4,0xD4AF,0xC4AF,0xD4D4,0xD4D4,0xD564,0xD4D4,0xC4D4,0xD464,0x8F54,0xD554,0xD4D4,0x547F,0xD4AF,0xD5D4,0x7F8F,0x41D4,0xD4D4,0x8F41,0x64C4,0x7F8F,0x6441,0x44AF,0x417F,0x447F,0x417F,0x8F41,0x4454,0x419F,0x8F44,0x54E4,0x549F,0x549F,0x41D4,0x8F54,0x54D4,0x5454,0x6575,0x5475,0x9F54,0xBF75,0xBFC4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xC5D4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xD4C5,0xD4C5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0x64E4,0x4644,0x4646,0x4446,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x5F46,0x4646,0x6546,0x4454,0x4646,0x9575,0x4475,0x4446,0x5F46,0x8574,0x8585,0x4675,0x4646,0x9554,0x85C4,0x7585,0x75D4,0x7585,0xB495,0x8585,0xC485,0x7585,0x4675,0x9575,0xB495,0x4585,0x4446,0x8585,0x4665,0x4644,0x4644,0x4646,0x4646,0x5F46,0x4446,0x4646,0x4646,0x4646,0x464F
.word 0x8F64,0x4144,0x9F44,0x6475,0xAF64,0xAF75,0xAF54,0xD564,0x85AF,0x64D5,0x418F,0xAFD4,0x757F,0x8F54,0x9F64,0x8F54,0xAFD4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xBFD4,0xD485,0x64D4,0xD4D4,0x64D4,0xD4AF,0xD464,0xD464,0x64D4,0x64D4,0x64BF,0xD4D4,0xD4D4,0xC451,0xD4D4,0xD4D4,0x44BF,0x649F,0xAFD4,0x5441,0xD564,0x5474,0x448F,0x7FD4,0x8F64,0xE464,0xE4E4,0x8F64,0x447F,0xBF64,0x8F44,0x7F44,0x7F41,0x8F44,0x418F,0x8F44,0x548F,0x54E4,0x548F,0x549F,0x418F,0x449F,0xD464,0x5475,0x5454,0x4454,0x649F,0x64D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xE4D4,0xE4D4,0xE4D4,0xE4C5,0xE4D4,0xC5D4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD5D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4C5,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xC5D4,0xE4E4,0xD4D4,0xD4D4,0x46AF,0x4644,0x4F46,0x4646,0x4646,0x4644,0x4646,0x465F,0x4646,0x4446,0x4446,0x7575,0x4446,0x8544,0x7585,0x4646,0x4446,0x9575,0x95B4,0x4665,0x755F,0xD495,0x7585,0x9575,0x85C4,0xB485,0xD485,0xB485,0x9585,0x95C4,0x6575,0x9575,0x85C4,0x4585,0x8565,0x7585,0x4644,0x4F46,0x4646,0x4446,0x4646,0x4646,0x4F46,0x4F46,0x4646,0x465F,0x4646
.word 0x8F64,0x9F54,0x6454,0xD5AF,0xD564,0xC454,0xD564,0x64D4,0x9FD4,0x64D4,0x548F,0x64D4,0xD48F,0x44AF,0xC4AF,0x54D4,0x64D4,0xD4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4BF,0x64D4,0xD5D4,0xAF85,0xAF54,0xD4D4,0x64D4,0xD4D5,0xD4C4,0x85BF,0xC4BF,0xD4D4,0xAFD4,0xD464,0xD4D4,0x8F41,0x447F,0x8F8F,0x8F41,0x8F8F,0xAF54,0xD585,0x5441,0xD4D5,0xE4D4,0x44E4,0x8F41,0x448F,0x64D4,0x7F9F,0x8F44,0x447F,0x8F7F,0x8F44,0x8F44,0x64E4,0x648F,0xD4D4,0xD4D4,0xE4D4,0x8FD4,0xD4C4,0x5454,0x5454,0xAF64,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4C5,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xC5E4,0xD5D4,0xE4D4,0xD4E4,0xD4E4,0xE4E5,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x5485,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4646,0x8544,0x4675,0x7546,0x75B5,0x4646,0x755F,0xB485,0xB485,0x4645,0x9575,0x95C4,0x7575,0xB5C4,0x8575,0x95B4,0xB485,0x7595,0xB495,0xB485,0x8575,0xD495,0xC485,0x6585,0xC495,0x4585,0x4446,0x4646,0x4646,0x465F,0x4646,0x4644,0x4646,0x4646,0x4644,0x4646,0x4644
.word 0xAF64,0x8FC4,0xAF64,0xAF75,0xAFC4,0xBF64,0xAFC4,0xBFC4,0xC464,0xD4D4,0xAF41,0x64C4,0xD48F,0x7F64,0xD464,0xAFD4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xD5D4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4C4,0xD4D4,0xD4BF,0xD4D4,0x64D4,0x648F,0xD4D4,0xD4AF,0xD464,0xD564,0xD464,0xD4BF,0xD4D4,0xD4D4,0xAF64,0x8F75,0x8F54,0x6454,0x64D5,0x548F,0xD564,0x6441,0x8FBF,0x6454,0xD4E4,0xD564,0x41D4,0x8F7F,0xC47F,0x64D4,0x448F,0x418F,0x447F,0x417F,0x547F,0x54E4,0xD49F,0x9FD4,0xE4E4,0x8F54,0xD4C4,0x51D4,0x54D4,0xAF41,0xD575,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xC5D4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4C5,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xAFD4,0x4644,0x4446,0x4646,0x4646,0x5F46,0x5F46,0x5F46,0x4F46,0x4646,0x4646,0x7546,0x5495,0x7546,0x75D4,0x4644,0x7545,0x8595,0x8595,0x6445,0xB585,0x85B4,0x85B4,0xB485,0x8575,0xD485,0x7585,0x7585,0x85C4,0x8585,0x8575,0x85C4,0x85B4,0xB485,0xB495,0x4675,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4446,0x4646,0x464F,0x465F,0x464F
.word 0x6464,0x75AF,0xC4AF,0x64D5,0x64D4,0x64D5,0xC4D4,0xD564,0xBFD4,0xD4C4,0xD454,0xD4D4,0xD4C4,0x8F64,0xD4D5,0xD4D4,0xE4D4,0xE4D4,0xD5D4,0xE4D4,0xE4E4,0xE4F4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xD4D4,0x419F,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64C4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0xC4AF,0xD4D4,0x547F,0xD4D5,0x44BF,0x64C4,0x8FD5,0x6441,0xD48F,0xE4E4,0x8F85,0x7F41,0x448F,0xD464,0x8F64,0x7F44,0x7F41,0x4144,0x44E4,0x5441,0x41D4,0xD464,0x4141,0x418F,0xD5D4,0xD4D4,0xAFC4,0xD464,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4C5,0xE4E4,0xD4E4,0xD4E4,0xC5E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4D4,0xD4E4,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4C5,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x4664,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46,0x7546,0x7595,0x8565,0x7595,0x4575,0x8575,0x85B4,0x75B4,0x9575,0xB4B5,0xB495,0x8585,0x8595,0xB485,0x8575,0x85B4,0x8585,0x95C4,0x85D4,0x95B4,0x8585,0xB485,0x8585,0x85B4,0x4644,0x4644,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646
.word 0xC4AF,0xAF64,0x64C4,0xC4BF,0xC4AF,0xD4BF,0xBF64,0x64D4,0x85D4,0xD4BF,0xD4D4,0xD4D4,0xAFD4,0x44D4,0xD4AF,0xD4D4,0xD4D4,0xD4C5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4F4,0xE4E5,0xE5E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xBFD4,0xD485,0xD4C4,0x8F64,0xBF64,0xD564,0xD4BF,0xAFD4,0xBFC4,0xD564,0xD464,0xD4D4,0x8F64,0xAF54,0xD4D4,0x548F,0xD4AF,0x9F85,0x648F,0x64AF,0x8FD5,0x417F,0xD454,0x64E4,0x447F,0x8F7F,0x418F,0xD4D5,0x8F64,0x8F44,0xD554,0x8FE4,0xAF44,0x8F64,0xD454,0x449F,0x4441,0xD48F,0x64D4,0xD4D4,0xD4D5,0xD4D4,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xD4E4,0xE4D4,0xE4C5,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4F4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x44D4,0x4644,0x4646,0x4646,0x4644,0x4446,0x5F46,0x4646,0x4F46,0x4646,0x5446,0x8595,0xB485,0x7585,0x7565,0x9585,0x85C4,0x7585,0xD485,0x8585,0x85B4,0xB485,0xB495,0x7585,0x8585,0xB485,0xB485,0xC485,0xB495,0xB485,0xB495,0x8575,0xC495,0x6585,0x4646,0x5F46,0x4446,0x4644,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46
.word 0xAF64,0x64C4,0x64D5,0x64D5,0x64D4,0xC4D4,0xC4D4,0xD4D4,0xD4BF,0xD4C4,0xD464,0xD4D4,0xD4D4,0xD464,0xD464,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4F4,0xE5E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFD4,0xD564,0xD4D4,0x64D4,0x549F,0xD464,0xD4D4,0xD4D4,0xBF64,0xBFC4,0xAFC4,0x64C4,0x448F,0x417F,0x417F,0x547F,0x449F,0x448F,0x8F54,0x5454,0xAF64,0xAF64,0xD575,0xBF64,0x6485,0x74D5,0xD4D4,0xD4D4,0x7F8F,0x858F,0x41D4,0xD49F,0x9F54,0xE444,0x9F44,0x9F44,0x4141,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D5,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4C5,0xE4E4,0xD4E4,0xE4E4,0xC5E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0x51D4,0x4646,0x4646,0x465F,0x5F46,0x4F46,0x4646,0x4446,0x4646,0x4646,0x8544,0x85B4,0x8585,0x75B4,0x8565,0xC495,0x7585,0xB485,0xB4B5,0xB495,0x8585,0xC495,0xB495,0x8575,0x85C4,0x8575,0x8585,0xB485,0x7585,0x8585,0x85C4,0xB485,0x8585,0x4685,0x4444,0x7575,0x4665,0x4646,0x4644,0x4644,0x5F46,0x5F46,0x4646,0x4644,0x4F46,0x4646
.word 0xD564,0xBF64,0xAFC4,0xBFC4,0xBFC4,0xAFC4,0xBFD4,0xD464,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xF4F4,0xE4E4,0xE5F4,0xE4F4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xC4BF,0xD4D4,0xD4C4,0x44AF,0xD49F,0xD464,0xD4D4,0xD464,0xD464,0x64D5,0x41D5,0x6454,0x448F,0x6454,0xC4D4,0x648F,0xC4BF,0x54AF,0x448F,0x448F,0x418F,0x448F,0x418F,0x8F8F,0x8F41,0x8F41,0x6444,0x9F54,0x8F64,0xE454,0x449F,0x418F,0xC49F,0x54D4,0x449F,0xD4E4,0xD4D4,0xAFD4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4C5,0xD4E4,0xD4E4,0xE4D4,0xC5D4,0xD4D4,0xD4E4,0xE4E4,0xC5D4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x54D4,0x4646,0x4644,0x5F46,0x4646,0x4646,0x4446,0x5446,0x4646,0x4646,0xB45F,0xB495,0x9585,0x7585,0x8575,0x85D4,0x85B4,0x9575,0x85C4,0x9575,0x85C4,0x85B4,0x75B4,0x8575,0xB485,0xB485,0xB485,0x85B5,0x75B4,0xB495,0xB495,0x8585,0x75B4,0x4665,0x7545,0x6595,0x4646,0x4F44,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xAFC4,0x64C4,0x74D5,0xD564,0xD574,0xD464,0xD4C4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD5D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xF4F4,0xF4E5,0xE4E4,0xD4E4,0xD5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFC4,0xAF85,0xD4C4,0xD4C4,0xD485,0x44AF,0xC4AF,0xD4D4,0x64D4,0xC4BF,0xC4BF,0x649F,0x548F,0xD5AF,0x64D4,0x548F,0xD4D5,0x9F64,0x8F54,0x8F44,0x5441,0x449F,0x6441,0x8F44,0x7F44,0x7F44,0x7F8F,0xD554,0x8F41,0xE444,0x417F,0x8FC4,0xD47F,0x9F64,0x8F54,0xD4D4,0xAF64,0xD4C4,0xE464,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4D4,0xD5D4,0xE5E4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD474,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0x9FD4,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x5446,0x4685,0x4644,0x9565,0x7595,0xB4C4,0x8575,0x8585,0x95B4,0x7585,0xC485,0xB495,0xC485,0xB485,0x8585,0x7595,0x7575,0x8585,0x7585,0x8575,0xB5B4,0x8585,0x85B4,0x8585,0x8575,0x7585,0x6465,0x7595,0x4645,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4446,0x4F46,0x4646,0x465F,0x4644
.word 0xD464,0xC4BF,0xC4BF,0xC4BF,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E5,0xF4D5,0xE4E4,0xE4E4,0xE4D5,0xE4F4,0xE4E4,0xE4E4,0xF4F4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4D5,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x85AF,0xC4BF,0xC4BF,0xD4D4,0x51D4,0x549F,0xD4D4,0xC4D4,0xD464,0x9F64,0x8F7F,0x5144,0x9FD5,0x8F54,0x417F,0x447F,0x548F,0x7F8F,0xD554,0x7FD4,0xE664,0x418F,0x417F,0x417F,0x8F44,0x417F,0x6464,0x648F,0xC4D4,0x64E4,0x418F,0x449F,0x449F,0xD4D4,0x41D4,0xD4D4,0x64D4,0xD4E4,0xD4C5,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xC5E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xD5E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4BF,0x54D4,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4546,0x7585,0x4646,0xB454,0x85B5,0x8585,0x7575,0x95C4,0xB485,0x7575,0x95D4,0x7575,0xB485,0x7595,0xB485,0x7585,0x6554,0x7575,0xB485,0x9575,0x7575,0xC475,0xB495,0x7585,0x95C4,0x65B4,0x8575,0x45C4,0x4446,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x464F
.word 0x64D4,0x64D5,0x64C4,0x74D5,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4C5,0xE4E4,0xC5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E5,0xF4E4,0xF4E4,0xF4F4,0xE4E5,0xE5E4,0xF4E4,0xF4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E5,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD464,0xD464,0xD4D4,0xD4D4,0xD4D4,0x549F,0xC4AF,0xC4D4,0x54D4,0x548F,0x6464,0x448F,0xD564,0x8F41,0xD564,0x4164,0xD4D5,0x7FE4,0x418F,0xD444,0x8F54,0x7F44,0x417F,0x8F7F,0x8F7F,0xAF75,0xAFE4,0x41E4,0x547F,0x448F,0x9F41,0xE4D4,0x54D4,0xD49F,0xD4D4,0xD464,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xC5E4,0xD4D4,0xD4C5,0xE4D4,0xC5D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xE4F4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4C5,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x85D4,0x9FD4,0x4646,0x4646,0x465F,0x4644,0x4454,0x4646,0x4446,0x9575,0x7575,0x9575,0xB485,0xB495,0x8585,0xC4B5,0x85B5,0x8585,0x95D4,0x85B4,0x7585,0x7585,0x9585,0x75B4,0x7575,0x8554,0x7575,0xB485,0x6585,0xB495,0x8585,0x9575,0x85D4,0x8575,0xC495,0x4665,0x4646,0x4646,0x4546,0x5475,0x4446,0x4646,0x4646,0x4646,0x5F44,0x4646,0x4646
.word 0xD564,0xD464,0xD4BF,0xD4D4,0xD4C4,0x64D4,0xBFD4,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD5E4,0xF4E4,0xE4E4,0xE5E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xF4E4,0xF4E4,0xF4F4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4C5,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x64BF,0xAFD5,0xD564,0xD464,0xD4D4,0xD4D4,0x64D4,0x8F8F,0xD464,0x549F,0xD4BF,0xAF41,0xD464,0x547F,0xC49F,0x7FD4,0x6451,0x41D4,0x547F,0x7F8F,0x8F41,0x8F54,0x7F44,0x7F44,0x7F44,0x548F,0x64E4,0x8F64,0x8F75,0x9F44,0x4144,0xD4E4,0xAFD4,0x6464,0xD4D4,0xBFD4,0xD4C4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD5E4,0xE5D4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x54D4,0x4646,0x4644,0x4646,0x465F,0x5446,0x5484,0x4646,0x8575,0x8585,0x9575,0x95B4,0x75C4,0x8575,0xB495,0x85D4,0x85B4,0x95B4,0x85B4,0x85B4,0x75B4,0xB495,0x7585,0x7554,0x5475,0x7585,0x7585,0x8575,0x95C4,0x85B4,0x85B4,0x75B4,0x7585,0x95C4,0x4644,0x4644,0x655F,0x7564,0x4645,0x4F46,0x4646,0x4646,0x4644,0x4646,0x5F46,0x4446
.word 0xBFD4,0xD4C4,0x64D4,0xD4D4,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E5,0xE4F4,0xE4F4,0xE5E4,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4D4,0xE4F4,0xE4F4,0xE4E5,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFD4,0xBFC4,0xD464,0xD464,0xD4D4,0xD4D4,0xC4D4,0x64D4,0x548F,0x6454,0xD4D5,0x547F,0xAF64,0x9F54,0x548F,0x64D5,0x648F,0xD4D5,0x7FC4,0x8F64,0x8F64,0x7F41,0x7F41,0x7F44,0x8F64,0xAF54,0x64D4,0x448F,0xD49F,0x41D4,0xE4D4,0xE4E4,0xC4AF,0x64D4,0x64D4,0xD4E4,0xD4C5,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xE4D4,0xE4E4,0xD4E4,0xD4D5,0xE4E4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x9FD4,0x4646,0x465F,0x4646,0x4646,0x4446,0x9544,0x8575,0x7546,0x85A5,0x95B4,0x9575,0x85D4,0x8575,0x85C4,0xB485,0x9585,0x85C4,0x8575,0x8575,0x7595,0x85B4,0x7575,0x7575,0x7575,0x7565,0x8575,0x8575,0x7585,0x8575,0x8585,0x7575,0xB575,0x75C4,0x4646,0x4446,0x85B4,0x4665,0x4446,0x4646,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x4646
.word 0xD4C4,0xBF64,0xD4C4,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE5E4,0xF4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E5,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xD474,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0x64BF,0x85BF,0x64D4,0xD4D4,0xD4BF,0x9FD4,0x8F8F,0x417F,0x8F7F,0x548F,0x5464,0x648F,0xD4D4,0x548F,0x9F64,0x8F41,0x8F44,0x5464,0x448F,0x417F,0x417F,0x8F7F,0x8F44,0xE4D4,0x8F41,0xE444,0x41D4,0x4144,0x449F,0xD441,0x9FD4,0xD4D4,0xD4C4,0xE4D4,0xD4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xC5E4,0xD4E4,0xD4C5,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4C5,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xD474,0xBFD4,0x54D4,0x4446,0x4646,0x465F,0x4644,0x4646,0x4446,0x8595,0x7595,0x8595,0xC485,0xC475,0x7585,0xB485,0xB485,0x8585,0xB585,0xB485,0x8575,0x7575,0x75D4,0x8575,0x7575,0x5475,0x8575,0x8575,0x7585,0xC485,0x7575,0xB485,0xB4B5,0x7585,0x8585,0x4585,0x4446,0x8554,0x6585,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F44,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E5,0xE5E4,0xF4E4,0xE5E4,0xF4E4,0xE4E4,0xF4F4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4C5,0x74D4,0xBF64,0xBFC4,0xAFC4,0xBF85,0xD485,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD564,0xD564,0xD4AF,0xD564,0xC464,0xD564,0x54C4,0xD541,0x4164,0x8F54,0x548F,0x648F,0x64D5,0x8F8F,0x548F,0xD564,0x4451,0xD4D5,0x547F,0x8FD4,0x7F44,0x7F44,0x8F41,0x7F44,0x6444,0xE4D4,0x8FD4,0x8F54,0x8F44,0x8F44,0x5441,0xD48F,0xD451,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xE4D4,0xC5D4,0xD4D4,0xD4C5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4D4,0xD4D4,0xD4D4,0x9FC4,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4644,0x7546,0x8575,0xC4B4,0x9575,0x9575,0x85C4,0x7585,0x8585,0x85B4,0x95B4,0x8585,0xB485,0x9595,0x85C4,0x7575,0x9575,0x7575,0x7595,0x7595,0xB485,0x7585,0x7575,0xC485,0x8585,0x7585,0xD4D4,0x4575,0x8564,0x7585,0x4645,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x465F,0x4646,0x465F
.word 0x64D4,0xC4BF,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD5D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x9F64,0x8F44,0x5444,0xD4D4,0xD564,0x64D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xD4AF,0xBF64,0x64C4,0xC4BF,0x419F,0x8FD5,0xD564,0x9FD4,0x8F44,0x548F,0x7FD4,0xAF44,0xD454,0x8FAF,0x8F64,0x8F41,0x5441,0x649F,0x448F,0x7FD4,0x417F,0x648F,0x41E5,0x547F,0x448F,0x418F,0x448F,0x418F,0x8F44,0xD464,0xBF64,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4C5,0xD5D4,0xE4D4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xD4E4,0xE4C5,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xC5E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0x5464,0x4646,0x465F,0x4446,0x4646,0x465F,0x5F46,0x4446,0x7585,0x9575,0xD475,0xB485,0xB485,0x7585,0xB485,0x8585,0xC485,0xB495,0x8585,0x85C4,0x85B4,0x8585,0xC485,0x7575,0x7575,0x75C4,0x7585,0x7575,0x8585,0x85C4,0xB475,0x8575,0x8585,0x7575,0xC485,0x4575,0x4646,0x4644,0x465F,0x4644,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4646
.word 0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xC5E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xD4E5,0xE4F4,0xE5E4,0xF4E4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0x64D4,0x4151,0x9F41,0x9F44,0x9F54,0x9F54,0x9F41,0xBF54,0xBF85,0xBFC4,0xD4C4,0xD4AF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD564,0xC4BF,0x8FD4,0x6485,0x8F64,0xAF44,0x8F54,0x4441,0x8F41,0x547F,0x448F,0x648F,0x8F64,0x9F44,0x64C4,0xD49F,0x448F,0x64AF,0x6464,0x448F,0x8F7F,0x8F44,0x7F41,0xAF64,0xD554,0x54D4,0x8F8F,0x5441,0x8F8F,0xD4C4,0x64D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4C5,0xD4E4,0xE4F4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xD5D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4C4,0x45AF,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x8554,0x8585,0x8585,0x7585,0x8585,0x7575,0x8585,0xB485,0x9585,0x85B5,0x75C4,0x8585,0xB485,0xC495,0x7595,0x8575,0x7575,0x7585,0x8575,0x8575,0x75B4,0x8575,0x8585,0xB485,0xB485,0x9585,0x85D4,0x4665,0x4644,0x464F,0x4646,0x4646,0x464F,0x4646,0x4646,0x465F,0x4646,0x465F
.word 0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5E4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E5,0xF4E4,0xE5E4,0xF4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x64D4,0xD464,0x9F41,0x9F54,0xAF65,0xAF75,0x5475,0x54AF,0x8F54,0x5454,0x64D5,0x64D5,0xC4BF,0xBF85,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64D4,0x549F,0xBFD5,0xD5C4,0x8F51,0x5441,0x8F64,0xC454,0x41D4,0x418F,0xAF54,0x7F44,0x4441,0x44AF,0x448F,0xD454,0x8F7F,0xF664,0x41D4,0x7F44,0x447F,0x548F,0x449F,0x517F,0xD4C4,0x8F54,0xD47F,0x547F,0x64AF,0xD4E4,0xD464,0xD4D4,0xE4D4,0xE4D4,0xD4C5,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xD4D4,0xD5D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xC4D4,0xD4D4,0x4464,0x4646,0x4644,0x465F,0x465F,0x465F,0x465F,0x5F46,0xB444,0xB485,0xB485,0xB485,0xB485,0x8575,0x85B4,0x95C4,0xC4B4,0xB485,0x8585,0x85B4,0x7575,0x8575,0x6575,0x8575,0x85C5,0x7575,0x7575,0x7585,0x7575,0xB485,0x7595,0xD4B5,0x7585,0xB485,0x6595,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x465F,0x4646,0x464F,0x4646
.word 0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xE4D5,0xD4E4,0xE4D5,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xF4E4,0xE5E4,0xE4E4,0xE4F4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xBF64,0x9F54,0x5454,0x75AF,0x75AF,0x75AF,0x54AF,0xAF54,0x9F54,0x9F54,0x9F41,0xD464,0x64D4,0x64D5,0xAFD4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0x54D4,0xD4AF,0x64D4,0x64D4,0x64D5,0x8F9F,0x8FC4,0x549F,0xD4D4,0x8F7F,0x6444,0x7F8F,0xE4D4,0x8F64,0x8F64,0xD464,0x547F,0x548F,0xE4E4,0xD4E4,0x5441,0xD464,0xC4E4,0x8F64,0x517F,0x759F,0xD4E4,0x7F8F,0x8F44,0x8F41,0xAF64,0xD485,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xC5D4,0xD4E4,0xD4E4,0xE4E4,0xE4D5,0xE4E5,0xE4D4,0xD5D4,0xD4E4,0xE4D4,0xD4E4,0x64E4,0xD4E4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xBFD4,0x4564,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7546,0x75B5,0x8585,0x7585,0x8575,0x7585,0x8575,0x95B4,0x8575,0x8585,0x95C4,0xB485,0x7585,0x6575,0x7564,0x8585,0x7575,0x85D4,0x7575,0x7575,0x8585,0xC495,0x8575,0x9585,0x75B4,0x9585,0x4675,0x4644,0x5444,0x4654,0x4644,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4446
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4F4,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD474,0xAF64,0xAF75,0xD564,0xD564,0xD564,0xAF64,0xAF75,0x5475,0x54AF,0x449F,0x9F54,0x9F44,0xD464,0xC4BF,0xD464,0xD564,0xD4BF,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0x8FD4,0x6454,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x448F,0x9F41,0x7F7F,0x8F54,0xD554,0xD4D4,0x7FC4,0x7F8F,0xAF64,0xAF44,0x9F75,0x8F64,0x8F64,0x8F44,0xD541,0xAFE4,0x9F54,0x9F54,0xD444,0x7FD4,0x8F64,0x7F44,0x7F41,0x8F44,0x9F44,0xAF64,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4C5,0xE4D4,0xE4E4,0xE4E4,0xF4D4,0xE4E4,0xF4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D5,0xE4D4,0xD4C5,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0xC4D4,0x46AF,0x4646,0x465F,0x4646,0x4644,0x4446,0x4446,0x4646,0x755F,0x75D4,0xB485,0x8585,0xB485,0x8585,0x8575,0xC485,0x7585,0x95C4,0x85C4,0x8585,0x85B4,0x5475,0x7575,0x8575,0x75C5,0x7585,0x7585,0x8575,0x85C4,0x75B4,0xB485,0x85B5,0x8575,0xB4C4,0x4665,0x5444,0x6585,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xD5D4,0xD4E4,0xE4C5,0xD4D4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4D5,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xC5E4,0xD4D4,0xD4D4,0xAF64,0x64C4,0xC4AF,0xAFD5,0xBFC4,0xAF64,0x64C4,0x75AF,0x54AF,0x9F54,0x9F54,0x8F54,0x9F54,0x8F54,0xD564,0xBFD4,0x64C4,0x74D4,0xD464,0x74D4,0xD4D4,0xD4D4,0x54D4,0x54AF,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x448F,0x6454,0x54D4,0x648F,0x8F64,0x8F54,0xC454,0x7FD4,0x8F44,0x8F41,0x8F44,0x647F,0x6464,0x64E5,0x549F,0x548F,0x64D5,0x44AF,0x7F7F,0x417F,0x417F,0x8F54,0x6464,0xD48F,0xC4D4,0xAF64,0xC5C4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD5E4,0xF4E4,0xE4E4,0xF4E4,0xE4D4,0xE4D4,0xE4D4,0xD4C5,0x64D4,0xC4AF,0xF4D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4E4,0xC5D4,0xE4D4,0xD4D4,0x64D4,0x4664,0x4646,0x4646,0x4F44,0x5F46,0x4F46,0x4646,0x4646,0x5446,0x8585,0x85C4,0x75B4,0x7585,0xB485,0x7575,0xB485,0x7585,0x7575,0xC485,0xB495,0x7575,0x5475,0x8575,0x85C4,0x7585,0x7585,0x7575,0x8585,0x85B4,0x8585,0xB5D4,0x7585,0xC485,0x4585,0x6465,0x8595,0x4645,0x4644,0x465F,0x465F,0x4644,0x465F,0x4646,0x465F,0x465F,0x465F
.word 0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD5D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xAF74,0x64C4,0xD5AF,0xAF85,0x74C4,0xD564,0x6464,0x75AF,0xAF64,0xAF75,0x5454,0x54AF,0x549F,0x8F54,0x9F54,0x8F54,0xC485,0xC4BF,0xD5BF,0xAFD4,0xD4D4,0xD4D4,0xD4D4,0x51D4,0x9F54,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xAFD4,0x8F8F,0xBF54,0x417F,0x548F,0x7FD4,0xAF54,0x64D4,0x448F,0xD554,0x547F,0xC48F,0xE4BF,0x9FD4,0x8F44,0x547F,0x54AF,0x648F,0x7F41,0xE444,0xD47F,0x448F,0xD4D4,0x448F,0xBFD4,0x64D4,0xD464,0xE4D4,0xC5D4,0xD4D4,0xC5D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4C5,0xD4E4,0xD4C5,0xD4E4,0x41D4,0xE464,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x46AF,0x4644,0x465F,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x6546,0x9585,0x8575,0x8585,0x8575,0x8585,0x6585,0x8585,0xB495,0x8585,0x8565,0x8585,0x5475,0x5475,0x8585,0x8575,0x7575,0xB485,0x8575,0x8575,0xB485,0x9575,0xB4C4,0xB485,0xB495,0x7575,0x8585,0x45B4,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4C5,0xD4D4,0x64D4,0x64D5,0x64D5,0xD4D4,0xD5D4,0xC4BF,0xC4AF,0xBF64,0xAF75,0x5475,0x54AF,0x9F54,0x9F54,0x9F44,0x4454,0x449F,0x549F,0xD4C4,0xD464,0xD464,0xBF64,0xD4C4,0xD4D4,0x9FD4,0x5454,0xE464,0xD4D4,0xC5D4,0xD4D4,0x51D4,0x4144,0x6475,0x449F,0x8F54,0xAF54,0xD4C4,0x417F,0xC48F,0x419F,0xC49F,0x8F64,0x64D4,0x64D5,0xD4D5,0x64D4,0x9F64,0x8F54,0x8F54,0x6444,0xD4D5,0x64D4,0xAF64,0xE4D4,0x418F,0xD454,0xD454,0x64BF,0xD464,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D5,0xD4E4,0xD4D5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xAF64,0xD4D4,0xE4E4,0xD5E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4664,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0xB475,0xB485,0x7595,0x7585,0xC475,0x8595,0x7575,0x8585,0x75B4,0x7575,0xB485,0x7585,0x7554,0x8575,0x7565,0x7575,0x7595,0x8585,0x75B4,0x8575,0xD4D4,0x8585,0x95D4,0x75B4,0x9585,0x75C4,0x4545,0x6454,0x4644,0x4646,0x4644,0x4646,0x5F46,0x4446,0x5F46,0x4446,0x5F46
.word 0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xD5E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xC5E4,0xD4E4,0xD4D4,0xD4D4,0xD564,0xBFD4,0xBF85,0xD4D4,0x64D4,0x85D4,0xD4D4,0xC4AF,0x64D5,0x75AF,0x75AF,0x759F,0x549F,0x549F,0x549F,0x549F,0x549F,0x549F,0xD4BF,0x64D5,0x85D5,0xAFD4,0xD4D4,0x54D4,0x9F9F,0xD454,0xD4D4,0xD4D4,0x64D4,0x648F,0x54AF,0xD58F,0x8F64,0x8F54,0x5444,0x64AF,0x447F,0xD541,0x7F64,0x7F44,0x9F64,0xAF54,0x8FC4,0x4141,0xE564,0xAFC4,0x5475,0x449F,0xD4BF,0x64D4,0x8F8F,0xD4D4,0x417F,0x8F44,0x54E4,0xD4D4,0xD5D4,0x64BF,0xD4D5,0xD4E4,0xD4D4,0xE4D4,0xF4C5,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0x64D4,0x759F,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0x64D4,0x46AF,0x5F46,0x4646,0x4644,0x465F,0x465F,0x4646,0x4446,0x5F46,0x9575,0x85B4,0x85B4,0x85B4,0x8575,0x95B4,0x75C4,0x8575,0x9585,0x8575,0x5475,0x7575,0x8575,0x8575,0x8575,0x7585,0x85C4,0x75B4,0x7575,0x8585,0x8585,0xA5B4,0xC495,0x8585,0xC485,0x6585,0x8585,0x4575,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646
.word 0xD4D4,0xD4D4,0xD4C5,0xE4C5,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4D4,0xD5D4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4C4,0xAF64,0xD564,0xAF64,0x9F75,0x9F75,0x5454,0x449F,0x5441,0x549F,0x9F54,0x9F54,0x5454,0xC4BF,0xC4BF,0xC4BF,0xD464,0x8FD4,0x5454,0x549F,0xD4D4,0xD4D4,0x8FD4,0xAF64,0xD575,0x8F51,0x9F44,0xD4C4,0xD48F,0xE4D4,0xE4E4,0x8FD4,0xD4D4,0x417F,0x449F,0x648F,0x449F,0x8F7F,0x7F41,0xD464,0x8FD4,0x6441,0x8F54,0xAF54,0xD444,0xE464,0x7F64,0xE454,0xD4D4,0xD4D4,0xD441,0xC47F,0x647F,0xD4D4,0xE4E4,0xD4D4,0xE4D4,0xE4E5,0xF4E4,0xE4E5,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0x64D4,0x64D5,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xC5D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD474,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x64D4,0x4664,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x7544,0x9585,0x7585,0x8585,0x75B4,0x8585,0x85D4,0x7575,0x7575,0xB485,0x7595,0x5475,0x7575,0xC475,0x7585,0x6575,0x7585,0x7575,0x8575,0x95B4,0x9575,0x85C4,0x85C4,0x95B4,0x75B4,0xD485,0x6575,0x4646,0x4644,0x4646,0x465F,0x465F,0x4644,0x4646,0x465F,0x4646,0x4446
.word 0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD5D4,0xE4D4,0xE4E4,0xE4D4,0xD5E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4E4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xBFD4,0xD4C4,0xBF64,0x6475,0x75AF,0x549F,0x549F,0x549F,0x449F,0x9F54,0xAF54,0x5454,0x549F,0x6441,0x64D5,0x64D4,0xBFD4,0x64C4,0x54AF,0x549F,0xD4BF,0xD4D4,0x64D4,0x65AF,0x9FE4,0x8F44,0x4441,0x548F,0x54E4,0x418F,0xD454,0xD4E4,0xD4E4,0x8F54,0x9F64,0x5454,0x8F64,0xD464,0xD4D4,0xD4D4,0x8F64,0x8F54,0x8F54,0x448F,0xD47F,0xD4AF,0xE4D4,0xD4D4,0x8F7F,0xD454,0x7FD4,0xAF64,0x8FD4,0xD464,0xC5D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0x74D4,0x54AF,0xD4AF,0xD4E4,0xE4E4,0xE4E4,0xD5E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x74D4,0xD4E5,0xD4D4,0xD4C5,0xE4D4,0xD4D4,0xAFD4,0x4664,0x4446,0x4646,0x5F46,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4544,0x85C4,0x95B4,0x75C4,0xB495,0x7575,0xC485,0x6585,0x7575,0x7585,0x7575,0x7554,0x8585,0x7585,0xC485,0x5485,0x7585,0x7575,0x8585,0xB485,0xD495,0xC495,0x8585,0xB485,0x8585,0x8595,0x4665,0x4644,0x465F,0x465F,0x4646,0x4646,0x464F,0x465F,0x4646,0x4644,0x4646
.word 0xD4D4,0xC5D4,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xC5E4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x64D5,0x75AF,0x75AF,0x75AF,0x549F,0x9F54,0x9F54,0x9F54,0x5454,0x54AF,0x549F,0x449F,0x549F,0xD4AF,0xD464,0xD4D4,0x9F54,0x9F54,0x6454,0xD4D4,0x54D4,0xAF64,0x6454,0x448F,0x418F,0x9FD4,0x8F41,0x7F44,0x7F8F,0x5441,0x8FE4,0xE4D4,0xD4E4,0x9FE4,0xC48F,0xD4D4,0x9F64,0x417F,0x9FD4,0xD47F,0x7FE4,0x5441,0x7F8F,0xD444,0xAF64,0x6444,0x7F7F,0x7F7F,0x7F7F,0x447F,0x547F,0x649F,0xC4AF,0xD4E4,0xE4E4,0xE4F4,0xE5E4,0xE4D4,0xE4E4,0xD4C5,0xD4D4,0x64D4,0x64D5,0xD4D5,0xE4D4,0xF4E5,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0x74D4,0xC4D4,0x46AF,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4F44,0x4646,0x9554,0x95B4,0x9575,0x8575,0x7585,0x8575,0x8585,0x7585,0x7575,0x7585,0x5475,0x7585,0x85B4,0x7585,0x8575,0x75B4,0x9585,0x75B4,0x8585,0x85B4,0x85B4,0x85B4,0x8575,0xB485,0x65C4,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F44,0x4646,0x4644
.word 0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xD5D4,0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xC4BF,0xAF64,0x64C4,0x75AF,0x54AF,0x549F,0x549F,0x649F,0x54AF,0xAF54,0x5454,0x549F,0x548F,0x6441,0x64D5,0xBFD4,0x44D4,0x549F,0x449F,0xD4BF,0x8FD4,0x5464,0xD58F,0x4141,0xD444,0x8F54,0x7F44,0x448F,0x447F,0x8F8F,0x7F41,0xE454,0xD4D4,0x4441,0x647F,0xBFD4,0xE444,0x8FE4,0x7F44,0xE444,0x7FD4,0x8F7F,0x7F7F,0x7F7F,0xC441,0x7FAF,0x7F7F,0x7F41,0x4441,0x6441,0x418F,0xAF44,0x6475,0xE4AF,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xBFD4,0xAF54,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFD4,0x4664,0x5F46,0x4646,0x4646,0x465F,0x465F,0x4644,0x4646,0x4646,0x4446,0x8544,0xB485,0x7595,0x75C4,0x8585,0x7575,0x7575,0x7585,0x8575,0x7575,0x7575,0xD475,0x8585,0x7565,0x8554,0x8575,0x7575,0x7585,0x85C4,0x8585,0x8595,0xB485,0x7585,0x8595,0x4675,0x4644,0x465F,0x4646,0x4646,0x464F,0x465F,0x4644,0x5F46,0x4646,0x4F46,0x4646
.word 0xD4D4,0xD4E4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE5E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4C5,0xE4D4,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x74D4,0xD4D4,0xD4D4,0xD564,0x64D4,0x75AF,0xAF64,0x9F75,0x5454,0x549F,0x9F54,0x9F54,0x5454,0x549F,0x548F,0x449F,0x8F8F,0x548F,0x64D5,0xD4D4,0x5441,0x549F,0x5454,0x64D4,0x649F,0x449F,0xAF64,0xD48F,0x8F64,0x8F41,0x8F44,0x4141,0x5444,0x5454,0xC4BF,0xD4D4,0x41E4,0xE564,0xD4E4,0xC4D4,0x5464,0x548F,0x648F,0x64E4,0x417F,0x447F,0x6441,0x7F7F,0xD444,0x7F64,0x447F,0x8F7F,0xAF64,0xD544,0xC47F,0x8F8F,0x6475,0xE464,0xE4E4,0xD5D4,0xD4E4,0xE4E4,0xE4D4,0x64D4,0xC4D4,0x6441,0xD4D4,0xE4E4,0xE4E5,0xE4E4,0xD4D5,0xD4E4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4C5,0xD4D4,0x6464,0x46AF,0x4646,0x5F46,0x4646,0x4646,0x4646,0x464F,0x5F46,0x4646,0x4646,0x4446,0xC485,0x8575,0x7575,0xC495,0x7585,0xB485,0x7585,0x75C4,0x7565,0x8554,0x7575,0x7585,0x6585,0xB485,0x8575,0x8575,0x75B4,0xC485,0xB495,0x85C4,0x75B4,0xB495,0x7585,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46
.word 0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xD4E4,0xE4F4,0xE4E4,0xE4E4,0xD5D4,0xE4D4,0xC5E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4D4,0x64D4,0xC4BF,0xAFD5,0xAF75,0x5475,0x54AF,0x9F54,0x5454,0x759F,0x54AF,0x549F,0x9F54,0x8F44,0x8F44,0x7F44,0xAF41,0xD4C4,0x44AF,0x9F54,0x9F54,0xD48F,0x9F54,0x9F54,0x4454,0xE541,0x9F54,0x5454,0x549F,0x9F54,0xAF64,0xD4D5,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xC4D4,0x64AF,0x8F9F,0x8F44,0x7F64,0xE4E4,0x7FD4,0xD4D4,0xD4E4,0xD4E4,0x647F,0x41D4,0x4154,0xC48F,0x54D4,0x7FD4,0x7FD4,0x7FD4,0xAFD4,0x64C4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xAFD4,0xD564,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD464,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0x4664,0x4644,0x4646,0x5F44,0x4646,0x4644,0x4646,0x4646,0x5F46,0x4646,0x4644,0x8565,0x7595,0xB485,0x8575,0x85B4,0x85C4,0x8585,0x7585,0x6564,0x8585,0x75C4,0x7585,0x7575,0x7575,0x7585,0x7575,0x9585,0xB5C4,0x75B4,0x8575,0x8585,0x8575,0x4575,0x5F46,0x4646,0x4646,0x4646,0x4644,0x4646,0x4644,0x4646,0x4F44,0x5F46,0x4446,0x4646
.word 0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD5E4,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xE4D5,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D5,0xD4E4,0xD4C5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAF64,0x6485,0x75AF,0x64AF,0xAF54,0x9F54,0x9F54,0x5454,0x54AF,0x9F54,0x8F54,0x9F44,0x8F44,0x7F41,0x6F7F,0x647F,0xD4D4,0x549F,0x449F,0x9F54,0x9F54,0x9F44,0x9F54,0x9F54,0x6464,0xD4E5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD485,0x64D4,0x9F64,0x9F54,0x8F54,0x8F44,0x7F41,0x6444,0xC48F,0x448F,0x417F,0x417F,0x448F,0x648F,0x7FD4,0x448F,0xD4D4,0xD48F,0xD48F,0xD441,0xC441,0x6441,0x64AF,0x74D5,0xE4E4,0xC5D4,0xD4E4,0xC5D4,0x64D4,0x9F64,0xD564,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD5D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD5D4,0x46AF,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4446,0x4646,0x4644,0x4F46,0x4646,0x4446,0x8565,0x64B4,0x7575,0x9585,0x85C4,0x7575,0x6575,0x8554,0x85D4,0x7575,0x7565,0x8554,0x7575,0x8575,0x8585,0x85C4,0x8575,0xC495,0x8575,0x95B4,0x75B4,0x7554,0x7575,0x6475,0x4454,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46
.word 0xD4D4,0xC5D4,0xE4D4,0xC5D4,0xE4D4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4C5,0xD4C5,0xD4E4,0xE4D4,0xD4D4,0x64D4,0xC4D4,0xC4AF,0xD564,0xAF54,0x9F75,0x6454,0x549F,0x549F,0xAF54,0x9F54,0x8F44,0x4154,0x448F,0x448F,0x7F41,0x7F7F,0x6444,0x54D5,0x548F,0x548F,0x548F,0x8F54,0x8F44,0x8F54,0x9FD4,0x6475,0xD564,0xD4D4,0xD464,0xAF64,0x6464,0x549F,0x449F,0x549F,0x448F,0x418F,0x448F,0x5441,0x54F5,0x649F,0x6464,0x6464,0x6464,0xAF64,0x8F54,0x7FC4,0xE4D4,0xD4E4,0x41E4,0x64D4,0x7FD4,0x8F64,0xC48F,0x64D5,0x649F,0xD48F,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x54AF,0xD4AF,0xD4D4,0xE4E4,0xD5E4,0xE4E4,0xD4E4,0xD4E4,0xE4C5,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xAF64,0x4685,0x4646,0x4646,0x4446,0x4646,0x4F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x6544,0x8575,0x7575,0x7585,0xB485,0x6585,0x5475,0x7595,0x7575,0x8575,0x7585,0x8575,0x7575,0x7585,0x85B4,0x8575,0x85C4,0x85B4,0x9585,0x75B4,0x7585,0x8575,0x4565,0x4646,0x4646,0x4644,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4446,0x5F46,0x4646
.word 0xC5D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4C5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0xC4BF,0xAFD4,0xAF64,0x6464,0x75AF,0x54AF,0x9F54,0xAF54,0x5454,0x549F,0x548F,0x448F,0x418F,0x7F7F,0x7F6F,0x466F,0x4646,0x6444,0x64D4,0x548F,0x548F,0x548F,0x549F,0x44AF,0xC48F,0x9F9F,0x418F,0x8F41,0x9F54,0x9F44,0x9F44,0x8F44,0x8F54,0x8F44,0x8F41,0x7F44,0x8F44,0x548F,0x9FE4,0x8F44,0x8F8F,0x8F8F,0x8F8F,0x8F44,0x8F54,0x547F,0x5451,0xE4E4,0xD4D4,0xD48F,0xD4D4,0xD4D4,0xBFC4,0x8FD4,0xAF44,0x6454,0xE4D4,0xD4E4,0xD4D4,0xD464,0xD564,0xD464,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C5,0xD4D4,0x64D4,0x46AF,0x4644,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x4446,0x4446,0x6544,0x7575,0x7585,0x7575,0x5475,0x7575,0xB485,0x8575,0x7585,0x7575,0x8575,0xD485,0xB5B5,0xB485,0x7585,0x8575,0x8585,0x85B4,0x8575,0x8585,0x6565,0x4644,0x4446,0x5F46,0x5F46,0x5F46,0x4446,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646
.word 0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xC5E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4F4,0xE4E5,0xE4E4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xC5D4,0xE4D4,0xD5D4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xAFC4,0xC4C4,0xC4AF,0x75AF,0x54AF,0xAF54,0x9F54,0x5454,0x54AF,0x549F,0x548F,0x418F,0x448F,0x7F7F,0x7F7F,0x466F,0x466F,0x4646,0xC49F,0x54D4,0x548F,0xC4AF,0xD4C4,0xD4C4,0x418F,0x4454,0x448F,0x448F,0x418F,0x448F,0x548F,0x418F,0x448F,0x448F,0x448F,0x418F,0x448F,0xAF41,0x54E4,0x8F41,0x4444,0x4441,0x447F,0xC4AF,0xE48F,0x8F41,0x7F8F,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7FD4,0xD4C4,0xAFD4,0x6464,0xD4D4,0xD4D4,0xC4AF,0x649F,0xC4AF,0xD4D4,0xD5E4,0xF4E4,0xD5E4,0xD4E4,0xD4E4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD474,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0x4664,0x5F46,0x4646,0x5F46,0x5F46,0x4446,0x5F46,0x4646,0x4646,0x4446,0x5F46,0x7546,0x4665,0x4646,0x7544,0x7585,0x7554,0x7554,0x5475,0x9575,0x6575,0x8575,0x8565,0x8575,0x95D4,0xC4C4,0x8585,0x8575,0xC485,0x7585,0x7575,0xB485,0x5475,0x4645,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xD4D4,0xD4D4,0xE4E4,0xE4C5,0xE4E4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4C5,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD474,0x64C4,0xD564,0xAF54,0xAF75,0x6454,0x549F,0x549F,0x9F54,0x8F54,0x8F54,0x8F44,0x7F41,0x446F,0x467F,0x6F46,0x6F46,0x4646,0xD541,0xC4D4,0x418F,0xC49F,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xAF64,0xC4C4,0x8F41,0x8F41,0x8F44,0x8F44,0x8F41,0x8F41,0x8F44,0x8F44,0xE454,0x41D4,0x41D5,0x7F8F,0x8F8F,0x8F44,0x4144,0xE4E4,0xD4D4,0xC4C4,0x858F,0xD4BF,0xE464,0xE4E4,0xD4E4,0xD4D4,0xD485,0xD4D4,0xD564,0xAFD4,0xD485,0xD4E4,0xD4C4,0xD541,0xD464,0xD4D4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4C5,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xAFD4,0x46C4,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4446,0x4646,0x4646,0x6446,0x7585,0x4644,0x8545,0x7575,0x7565,0x7554,0x8575,0x7575,0xC585,0x7585,0x7575,0x7575,0x8565,0x8585,0x95B4,0x85C4,0x75B4,0x8585,0x85C4,0x6565,0x4644,0x4646,0x4644,0x4646,0x4446,0x5F46,0x4646,0x4F44,0x4646,0x5F46,0x4446,0x4646,0x4644,0x4646
.word 0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE5E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xE4D4,0xC5E4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4C5,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xC4BF,0xD5BF,0x64AF,0x75AF,0x65AF,0x54AF,0x54AF,0x9F64,0x5454,0x449F,0x448F,0x8F41,0x7F44,0x6F6F,0x4644,0x465F,0x4646,0x465F,0x6F46,0xC454,0xC4D4,0x64D4,0x64D4,0x9FD4,0x5441,0x8F8F,0x8F44,0x8F8F,0x8F44,0x8F44,0x4441,0x448F,0x4441,0x448F,0x8F41,0x8F41,0xE454,0x8F64,0x7F64,0x4144,0x8F44,0x417F,0x648F,0xE4E4,0xE4E4,0x648F,0xC48F,0x8FD4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD48F,0xD441,0x74D4,0xD564,0x64AF,0x647F,0x64AF,0x649F,0xD4D5,0xE4D4,0xE4E4,0xE4E4,0xE4E5,0xD5E4,0xD4E4,0xD4D4,0xD4E4,0xC5D4,0xD4D4,0xD474,0xD4D4,0xE4D4,0xD4D4,0x74D4,0x64D5,0x46AF,0x465F,0x465F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4F46,0x4F46,0x4646,0x7544,0x8575,0x8585,0x7585,0x5475,0x5454,0x8585,0x7565,0x7575,0x8575,0x6585,0x7585,0x7575,0x7585,0xB485,0x8585,0x8575,0x8595,0x95B4,0x5475,0x4645,0x4644,0x4646,0x465F,0x4446,0x464F,0x4646,0x4644,0x4646,0x5F46,0x4646,0x4F46,0x4646,0x464F,0x4646
.word 0xD4E4,0xC5E4,0xD4E4,0xE4E4,0xD5E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4C5,0xD4E4,0xD4C5,0xD4D4,0xD4E4,0xE4C5,0xE4E4,0xD4E4,0xE4E4,0xD5D4,0xE4E4,0xC5D4,0xE4D4,0xC5D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xD464,0xD564,0xAF64,0x9F75,0x9F75,0x9F54,0x9F54,0x9F54,0x9F44,0x8F44,0x7F8F,0x7F7F,0x6F7F,0x4646,0x4646,0x4644,0x5F46,0x4646,0x6F46,0x8F46,0xD541,0x419F,0x548F,0xD4D4,0x54D4,0x448F,0x417F,0x448F,0x417F,0x8F8F,0x8F41,0x448F,0x418F,0x448F,0xD554,0xD4E4,0x449F,0x8F8F,0x8F41,0x8F44,0x8F44,0xE454,0xE4E4,0xE4E4,0xE4E4,0x64D4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0x41E4,0xE4D4,0xBFC4,0xD47F,0xD4D4,0xD564,0x8F64,0x6454,0x64D5,0xD4AF,0xD4D4,0xD5E4,0xF4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0x64D4,0xD4E5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xBF64,0x4675,0x4646,0x4646,0x464F,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4644,0x4646,0x4546,0xB495,0x8575,0x8575,0x7565,0x7575,0x8575,0x5475,0x7585,0x5475,0x8575,0x7575,0x8575,0x7585,0x8585,0x85D4,0x95C4,0x85C4,0x6575,0x4644,0x4646,0x465F,0x465F,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F
.word 0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D5,0xD5E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE5D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xBF64,0x6475,0x54AF,0x5454,0x54AF,0x9F54,0x8F54,0x4144,0x448F,0x4441,0x7F41,0x6F7F,0x5F46,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x5441,0x64D4,0x9F64,0xAF54,0xD4D4,0x9F64,0x8F44,0x8F44,0x7F41,0x8F44,0x8F44,0x8F44,0x4441,0xAF54,0xE4D4,0x5464,0x418F,0x8FC4,0x8F44,0x8F41,0x4441,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xC5E4,0xE4E4,0xD585,0xE4D4,0x54D4,0x64D4,0xD4D4,0xD4D4,0x44BF,0x8F8F,0xAF54,0xD4C4,0xE474,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0x64D4,0x469F,0x4446,0x4646,0x4646,0x4644,0x4646,0x465F,0x4F44,0x4646,0x4475,0x4646,0x4644,0x9575,0x7585,0x7575,0x5454,0x8575,0x7575,0x8575,0x7575,0x7585,0x7565,0x8585,0xB485,0xC485,0xB485,0x8595,0x75B4,0x6575,0x4645,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4644,0x465F,0x5F44,0x4646,0x4446,0x4646
.word 0xC5D4,0xD4E4,0xE4E4,0xD4D5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D5,0xE4E5,0xE4E4,0xE4E4,0xE4D4,0xC5D4,0xE4D4,0xD4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xE4C5,0xE4E4,0xE4D5,0xE4E4,0xD5D4,0xD4E4,0xD4E4,0xE4E4,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBF64,0xC4C4,0xC4BF,0xAF64,0xAF64,0xAF65,0x9F75,0x5454,0x449F,0x8F41,0x8F7F,0x6F7F,0x7F44,0x4644,0x4646,0x4646,0x465F,0x465F,0x5F46,0x5144,0x8F54,0x9F7F,0xC4D5,0x64D4,0x418F,0xC49F,0x64D4,0x448F,0x448F,0x418F,0x4144,0xAF54,0xE4D4,0x8FE4,0x64D4,0xD48F,0x64D4,0x448F,0x448F,0x448F,0xD48F,0xF4E4,0xE5E4,0xE4E4,0xD4E4,0x64D4,0xE4D4,0xE4E4,0xE4D4,0x41E4,0xE4D4,0xD4E4,0xE4E4,0x9FC4,0xD49F,0xD4D4,0x8FD4,0x7F44,0x4441,0xAFD4,0xE4D4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0x64D4,0x4664,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4446,0x9585,0x4644,0x4646,0x7544,0x85C4,0x75C4,0x8575,0x7565,0x85D4,0x6575,0x8575,0x75D4,0x7575,0x7575,0x5465,0x6575,0x4465,0x4646,0x4644,0x4446,0x4446,0x4646,0x465F,0x4646,0x4644,0x465F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646
.word 0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE5E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xC5E4,0xE4D4,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4E4,0xE4C5,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xD5D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC4D4,0xC4BF,0xAF64,0xAFC4,0xAF65,0x5475,0x549F,0x449F,0x449F,0x448F,0x4441,0x6F7F,0x7F7F,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4546,0x9F54,0x9F54,0x4144,0x9F41,0xD4C4,0xC4D4,0x8F54,0xD464,0xD4D4,0xD4D4,0x9F54,0xD564,0xE4E4,0x54D4,0x549F,0x8F41,0x417F,0x8F8F,0x8F44,0x417F,0x547F,0xE464,0xE4F4,0xE4F4,0xD4E4,0xD4E4,0xD464,0xD4D4,0xE4E4,0xE4E4,0xD444,0xD47F,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x8F51,0x7F41,0x7F8F,0xD441,0xD4D4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64BF,0x46AF,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4F44,0x4646,0x465F,0xB485,0x4475,0x4646,0x4446,0x9585,0x5485,0x6465,0x7554,0x7585,0x9595,0x7575,0x8585,0x6585,0x4446,0x4646,0x4644,0x4646,0x4644,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4446,0x4646,0x465F,0x5F44,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x465F
.word 0xE4D4,0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E5,0xE4E4,0xD4E4,0xE4D5,0xD4D4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xF4D4,0xD4E4,0xD4F4,0xD4E4,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xC5D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD485,0x64D5,0xAF64,0x6475,0x54AF,0x54AF,0x9F54,0x8F44,0x8F54,0x7F7F,0x7F7F,0x6F6F,0x4646,0x4646,0x465F,0x465F,0x4446,0x4144,0x9F54,0x4144,0x448F,0x448F,0x648F,0xD4AF,0x64D4,0xD4D5,0x51D4,0xD4AF,0xD4D4,0xD4D4,0x8F51,0x8F41,0x8F44,0x447F,0x7F7F,0x7F44,0x8F41,0x5444,0xD4AF,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xC4BF,0x7FD4,0x41D4,0xD4E4,0xE441,0xD441,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0x447F,0x448F,0x4441,0x417F,0xE454,0xF4E4,0xE4F4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC5D4,0xE4D4,0x74D4,0xD564,0x4664,0x4446,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0xB565,0x8585,0x8585,0x8575,0xB485,0x7585,0x7575,0x8585,0x8565,0x7565,0x8575,0x85B4,0x65B4,0x4646,0x465F,0x4646,0x4644,0x465F,0x465F,0x465F,0x465F,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4644,0x5F46,0x4646,0x4646,0x4646
.word 0xD4E4,0xD4E4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xD4D4,0xE4D5,0xF4E4,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E5,0xE4F4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4C5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0x64D4,0xC4BF,0xC4AF,0x75AF,0x65AF,0x9F54,0x9F54,0x4154,0x448F,0x418F,0x447F,0x7F6F,0x5F44,0x5F46,0x4646,0x5F46,0x4646,0x5441,0x449F,0x548F,0x418F,0x8F44,0x417F,0x6454,0xD4D4,0x85AF,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8F54,0x547F,0x51D4,0x7F8F,0x7F7F,0x7F44,0x9F54,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xE474,0x7FE4,0x64D4,0x7F8F,0x7F41,0x7F41,0xD441,0xE4D4,0x8F85,0x8F41,0x8F44,0x8F7F,0x7F44,0x9F64,0xE4D4,0xE4E4,0xE4D5,0xD5D4,0xD4E4,0xC5E4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD5D4,0x54AF,0x46AF,0x465F,0x4646,0x4646,0x4446,0x4646,0x4646,0x5F46,0x4646,0x7446,0x85B4,0xB585,0x94D4,0x95D4,0x75B4,0x5475,0x7565,0x8585,0x7575,0x7554,0x9575,0x6475,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4446,0x4F46,0x4646,0x4644,0x465F,0x4F46,0x4646,0x465F,0x4F46,0x4646,0x4446,0x4646,0x4644
.word 0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D5,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xC5D4,0xD4E4,0xE4D5,0xE4E4,0xF4E4,0xF4E4,0xE5E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4C4,0x6464,0x64D5,0x64AF,0x75AF,0x549F,0x549F,0x449F,0x4441,0x7F7F,0x6F41,0x4646,0x4646,0x5F46,0x4646,0x4146,0x9F54,0x9F44,0x9F54,0x9F54,0x8F44,0x8F44,0x8F8F,0x649F,0xD4D4,0x6464,0x419F,0x6441,0xD4BF,0x8FD4,0xD454,0x8F64,0xD47F,0xD4D4,0x4154,0x9F54,0xD4D4,0xE4E4,0xE4E4,0xD4C5,0xC5D4,0xD4D4,0xD4D4,0xE4D4,0x7FE4,0x548F,0x5464,0x64AF,0x8F54,0x5444,0xC4D4,0xE48F,0x5451,0x7FD4,0x6444,0x44AF,0x547F,0xAF64,0x5444,0x8F64,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xBFD4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0x64D4,0x64D4,0x4654,0x4646,0x465F,0x465F,0x465F,0x5F46,0x4646,0x4644,0x4446,0x4446,0x95C4,0x85B4,0xD4B4,0xD495,0x8575,0xD485,0x7575,0x6585,0x8575,0x7575,0x8585,0x7575,0x4446,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4646,0x4446,0x4446,0x4F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4446,0x4F46,0x5F46,0x4646
.word 0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xE4E4,0xD4E4,0xD4C5,0xD4C5,0xE4D4,0xC5D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xF4E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xC4BF,0xAFD4,0xD564,0xAF54,0xAF65,0x9F75,0x5454,0x449F,0x8F8F,0x6F6F,0x417F,0x5F46,0x4446,0x4646,0x4446,0x4144,0x8F54,0x5454,0x549F,0x9F54,0x9F44,0x4441,0x4441,0x448F,0xC4AF,0xD4D5,0xD564,0xD5D4,0x64D4,0x647F,0x64D5,0x418F,0xE4C4,0x54D4,0x548F,0xD4AF,0xE4D4,0xD4E4,0xE4E4,0x54D4,0x449F,0x417F,0xD464,0x7FE4,0x4441,0x8F41,0xD564,0x8F54,0x9F54,0x8F41,0xD4D4,0x8F41,0x8F54,0xD47F,0x417F,0x548F,0xBF64,0x8F54,0x648F,0x54AF,0xD454,0xE4E4,0xD4E4,0xE4D5,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xAF64,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x6546,0xC495,0x9595,0xB5D4,0x5465,0x8575,0x7575,0x6595,0x5475,0x5475,0x7575,0x7575,0x4644,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x4F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4446,0x4446,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D5,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4C5,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0x64D5,0x64BF,0x75AF,0x54AF,0x5454,0x549F,0x449F,0x448F,0x7F7F,0x7F6F,0x4646,0x465F,0x465F,0x4446,0x9F54,0x9F54,0x9F54,0x9F54,0x8F54,0x4154,0x9F54,0x8F54,0x8F41,0x8F44,0xD464,0xBFD4,0x74C4,0x548F,0xD4D4,0x8F9F,0x8F54,0x4441,0xD4AF,0xD4E4,0xE4D4,0xD4E4,0xC5E4,0xE4D4,0xE4D4,0xD4D4,0xD454,0x418F,0x6454,0x8FD5,0xD564,0x8F9F,0x417F,0x548F,0xAF64,0x417F,0xD454,0x8FD4,0x4144,0x448F,0x8F7F,0x8F44,0x6444,0x9F64,0x9F54,0xAF64,0xE444,0xE4E4,0xD4D4,0xD4E4,0xC5D4,0xD4E4,0x64D4,0xD4D4,0xD4D4,0xC5D4,0xD4D4,0xD574,0xD564,0x469F,0x4644,0x4446,0x4646,0x4644,0x465F,0x4F44,0x5F46,0x4646,0x4646,0x7574,0x4446,0x8575,0xD4C4,0x5475,0x7575,0x7554,0x7575,0x8575,0x8575,0x85C4,0x75D4,0x4485,0x4446,0x5F46,0x4646,0x464F,0x465F,0x465F,0x4644,0x4F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4F46,0x4646,0x464F,0x4646,0x5F46,0x5F46,0x4646,0x465F,0x4646
.word 0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4C5,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xD4D5,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD564,0x6464,0xAF64,0x9F64,0x9F54,0x8F54,0x7F41,0x6F7F,0x417F,0x4446,0x4646,0x4646,0x5441,0x549F,0x9F54,0x5454,0x549F,0x549F,0x548F,0x449F,0x448F,0x8F41,0x8F44,0x417F,0x64C4,0x449F,0x649F,0x54AF,0x8F41,0x5444,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xE47F,0x7FE4,0x9F54,0x548F,0x417F,0x548F,0xD4D4,0xD4D4,0xD4E4,0x8F54,0x8F44,0x9F44,0x6444,0x548F,0xAFC4,0x8F54,0xD564,0x8F7F,0x448F,0x448F,0x448F,0x6464,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0x64D4,0x64BF,0x4654,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x8585,0x4675,0x8565,0x9585,0x7565,0x5485,0x8585,0x8575,0x6585,0x7585,0x7575,0x8585,0x7575,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x464F,0x4646,0x4646,0x5F46,0x4646,0x464F,0x4644,0x4646,0x4646,0x4446,0x5F46
.word 0xE4E4,0xE4C5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xD5E4,0xE4E4,0xE4E4,0xD4E4,0xC5E4,0xE4D4,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE5E4,0xF4E4,0xF4E4,0xF4E4,0xE4E5,0xE5F4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4C5,0xC5D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xBFC4,0xAF64,0xAF75,0xAF65,0x5475,0x449F,0x4441,0x447F,0x7F6F,0x5F46,0x5F46,0x4446,0x9F54,0xAF54,0x5464,0x64AF,0xAF64,0x6465,0x54AF,0x9F54,0x9F54,0x4154,0x448F,0x8F41,0x8F8F,0x547F,0x9F64,0x9F54,0x547F,0x64AF,0xD4D5,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD485,0x417F,0x5454,0x7FD5,0xD4C4,0xD4D4,0x85D4,0xD4D4,0x417F,0xE4D4,0xD4E4,0x54D4,0x8F8F,0x8F44,0x548F,0xD4D4,0x8F64,0xD454,0x8FC4,0x5441,0x64AF,0x9F8F,0xC464,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xAF64,0xAF75,0x4654,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x464F,0x9544,0x8585,0x85C4,0x95B4,0x7575,0x7554,0x7565,0x7554,0x7585,0x7575,0x5454,0x7575,0x7585,0x4475,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x465F,0x465F,0x465F,0x5F46,0x4646,0x4F44,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646
.word 0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xD5E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xE4C5,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D5,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x64D5,0x75AF,0x54AF,0x54AF,0x9F54,0x8F44,0x6F7F,0x7F41,0x4646,0x4646,0x5146,0x6454,0x5464,0x75AF,0x54AF,0x65AF,0x54AF,0xAF64,0xAF75,0x5454,0x549F,0x9F54,0x8F44,0x4454,0x9F41,0x5454,0x449F,0x4441,0x549F,0x649F,0x64AF,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD485,0x41AF,0x8F8F,0x8F8F,0xD441,0x8FD4,0x5441,0x9F9F,0x8F54,0xD564,0x448F,0xE464,0xD4E4,0x6464,0x44AF,0x5441,0xD4AF,0x5441,0xD49F,0x547F,0x8F64,0x8F75,0x6454,0x418F,0xD4C4,0xE4D4,0xD4D4,0xD4D4,0xD464,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x469F,0x465F,0x4446,0x5F46,0x4446,0x464F,0x4646,0x4646,0x4644,0x4646,0x7546,0xC485,0x95B4,0xD4D4,0x5485,0x7554,0x75D4,0x8575,0x9565,0x7575,0x9585,0x8575,0x7585,0x7585,0x4644,0x465F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x5F46,0x5F46,0x4446,0x4646,0x465F,0x465F
.word 0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD5D4,0xE4E4,0xE4E4,0xE4E4,0xD4F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E5,0xE4E5,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0x64D4,0xBFD4,0xD585,0xAF64,0x6475,0x9F64,0x9F54,0x8F54,0x7F41,0x6F7F,0x5F46,0x4646,0x5441,0x64AF,0x75AF,0x75AF,0x75AF,0xAF64,0xAF75,0xAF75,0x6454,0x9FD5,0x9F54,0x9F44,0x4144,0x8F9F,0x7F44,0x548F,0x8F8F,0x448F,0x418F,0x448F,0x9F41,0x9F54,0x9F54,0x4154,0x4141,0x5441,0x64BF,0x8F54,0x4444,0x4154,0x7FD4,0x7F41,0xAF44,0x6475,0x648F,0x419F,0xD4D4,0xD4D4,0x8F41,0x548F,0xD464,0xD4D4,0x8F41,0xD554,0x7F74,0x9F41,0x54D5,0xD49F,0x648F,0x548F,0x64AF,0xD4E4,0xD4D4,0x64D4,0xD4D4,0xD4D4,0xD474,0xC5D4,0xD4D4,0xD564,0xC4AF,0x4641,0x4446,0x4F46,0x4646,0x4646,0x4446,0x5F46,0x4646,0x464F,0x4446,0x4646,0xC454,0xB495,0xD485,0x7585,0x7554,0x5475,0x8585,0x8575,0x6465,0x7565,0xC475,0xB495,0x7585,0x7575,0x4654,0x4646,0x4644,0x4646,0x465F,0x4646,0x4644,0x4446,0x4F46,0x464F,0x4644,0x465F,0x4F46,0x4644,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D5,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD5D4,0xD5D4,0xE4E4,0xE4D5,0xE4F4,0xE4E4,0xE4F4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D5,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0x75AF,0x75AF,0x549F,0x449F,0x7F7F,0x6F6F,0x4644,0x455F,0x6454,0x75AF,0x75AF,0x75AF,0xAF64,0xD575,0xAF64,0xD575,0x649F,0x54AF,0xAF64,0x5454,0x449F,0x448F,0x448F,0x7F7F,0x447F,0x418F,0x447F,0x8F8F,0x8F44,0x9F44,0x4154,0x649F,0x64D5,0x75AF,0x449F,0x449F,0x417F,0x449F,0x418F,0x6454,0x449F,0x8F8F,0x8F44,0x8F44,0x857F,0xD4D4,0x7FD4,0x8F54,0x648F,0xD4E5,0x7FD4,0x418F,0x547F,0x54D4,0x419F,0xD554,0x8F64,0xAF44,0x8F75,0x6464,0xD4E4,0xBFD4,0xD4C4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xC4BF,0xAF64,0x4654,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4446,0xC475,0xD495,0xB495,0x7554,0x7575,0x7565,0x8575,0x7575,0x7595,0x7585,0xC485,0x9575,0x8575,0x5475,0x4644,0x5F46,0x4646,0x4646,0x4644,0x464F,0x4646,0x465F,0x4F4F,0x464F,0x4646,0x4F5F,0x464F,0x4646,0x4644,0x4646,0x464F,0x4644,0x4646,0x4644
.word 0xE4E4,0xD4E4,0xE4D5,0xE4E4,0xE4E5,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xD5E4,0xE4E4,0xD5D4,0xE4D4,0xD4C5,0xE4D4,0xC5D4,0xE4D4,0xE4D4,0xE4D4,0xD4C5,0xE4E4,0xE4E4,0xE4F4,0xF4E4,0xF4F4,0xF4E4,0xF4F4,0xE4E4,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xC5D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFD4,0x64D4,0xAF64,0xAF75,0x5454,0x548F,0x7F7F,0x7F7F,0x4646,0x6444,0x75AF,0xD564,0xAF64,0xD5C4,0xC4AF,0xC4AF,0xC4AF,0xD464,0xD5C4,0xD564,0x6464,0x64AF,0x9F64,0x5464,0x418F,0x8F54,0x7F44,0x4441,0x8F8F,0x4144,0x448F,0x448F,0x448F,0x448F,0x418F,0x8F44,0x8F44,0x7F41,0x8F41,0x7F44,0x8F41,0x548F,0x44AF,0x417F,0x8F7F,0x447F,0x8F8F,0x647F,0xD4D4,0x9F54,0x8F44,0x648F,0x64D4,0x8F44,0x7F8F,0x418F,0x9F44,0x418F,0x548F,0xD48F,0xC4D4,0x75BF,0x54AF,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xAFC4,0x6464,0x4654,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4446,0x464F,0x4646,0x4446,0x4564,0x4646,0x4646,0x7544,0x8585,0x5454,0x5475,0x5475,0x7575,0x7554,0x6554,0x6575,0x5475,0xC485,0x9575,0x7585,0x7575,0x4665,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x464F,0x444F,0x4F5F,0x4646,0x465F,0x4F4F,0x4646,0x465F,0x465F
.word 0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD5E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4C5,0xE4D4,0xD4E4,0xE4E5,0xE4E4,0xF4E4,0xE4E4,0xE4F4,0xE5E4,0xF4E4,0xE5E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xD4E4,0xD4D4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD564,0xC4AF,0x75AF,0x549F,0x449F,0x6F7F,0x6F44,0x4644,0x6464,0xAFD5,0xBF64,0xD5C4,0xD464,0xD585,0xD464,0xD5D4,0xAFD4,0xBF85,0xAF64,0x64D5,0xD5D5,0xAF64,0xAF75,0x9F54,0x8F54,0x4441,0x8F7F,0x7F44,0x8F41,0x8F44,0x8F41,0x8F41,0x8F44,0x448F,0x8F8F,0x7F41,0x8F44,0x7F44,0x418F,0x8F44,0x8F44,0x417F,0x447F,0x447F,0x417F,0x8F44,0x8F44,0xD4D4,0x54D4,0x7FD5,0x8F41,0xE454,0x41D4,0x7F44,0x7F44,0x448F,0x447F,0x448F,0xD441,0xD4D4,0xD564,0xAF64,0xBF64,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x75AF,0x75AF,0x469F,0x4646,0x4646,0x5F46,0x4446,0x4646,0x464F,0x4646,0x4644,0x4646,0x9575,0x4645,0x4644,0x7545,0xC485,0x5475,0x5475,0x7575,0x8575,0x8565,0x54C4,0x7585,0x8585,0x9585,0x85C4,0x85B4,0x8575,0x6585,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4446,0x4F46,0x464F,0x4644,0x464F,0x5F46,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646
.word 0xD4E4,0xE4D5,0xE4E4,0xE5E4,0xE4E4,0xE4D5,0xE4F4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E5,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC5E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x6464,0x54D5,0x54AF,0x8F54,0x6F41,0x6F7F,0x5146,0xBF64,0x6485,0x64D5,0x61D5,0xAFD5,0x64D4,0xBFD4,0xBF85,0xD4C4,0x64D4,0xD4D4,0xD4D4,0xBF64,0x6475,0x75AF,0x5464,0x549F,0x418F,0x448F,0x417F,0x447F,0x417F,0x447F,0x417F,0x417F,0x7F7F,0x7F44,0x7F44,0x8F41,0x7F41,0x7F44,0x7F8F,0x4441,0x7F7F,0x7F7F,0x7F41,0x7F41,0x417F,0x418F,0xD4E4,0xD4E4,0xD47F,0x447F,0x418F,0xD4D4,0x417F,0x448F,0x417F,0x417F,0x8F7F,0x9F44,0xD4D4,0xD4D4,0xD564,0x6464,0xD464,0xD4D4,0xD474,0xD4D4,0x64AF,0xAFC4,0xAF75,0x4645,0x465F,0x465F,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x7446,0x7585,0x8575,0x8584,0x95B4,0x7585,0x7554,0x5475,0x8575,0x7554,0x7585,0x5475,0x6575,0x9565,0x6585,0x8585,0x8575,0x8575,0x5485,0x5F46,0x5F46,0x4446,0x464F,0x4646,0x4F46,0x464F,0x464F,0x4F4F,0x464F,0x464F,0x5F46,0x464F,0x4F46,0x4F4F,0x5F46,0x4646
.word 0xE4E4,0xE4E4,0xD5D4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4C5,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D5,0xE4E4,0xE5E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xC4BF,0xC4BF,0x64AF,0xAF64,0x5454,0x6F8F,0x6F41,0x6441,0x64D5,0xAFD5,0xD4C4,0x64D4,0x64D4,0xBFD4,0xD4C4,0x64C4,0x74D4,0xD4D5,0xD4D4,0xD464,0x64D5,0xD5D5,0xAF64,0xAF64,0x9F75,0x5454,0x418F,0x7F8F,0x7F7F,0x7F7F,0x417F,0x447F,0x447F,0x418F,0x417F,0x417F,0x447F,0x417F,0x448F,0x7F41,0x9F44,0x7F41,0x7F44,0x7F41,0x8F44,0x7F44,0x7F44,0xE454,0xD4D4,0xE451,0x8F41,0x8F44,0xD4D4,0x547F,0x7F41,0x7F41,0x7F44,0x4144,0xC49F,0x8F54,0xD4D4,0xBFD4,0xAFC4,0x64D5,0x54AF,0xD4AF,0x64D4,0xD564,0x6464,0x75AF,0x4644,0x4646,0x4446,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x445F,0x8575,0xC485,0xC495,0xC485,0xB495,0x7554,0x7554,0x8575,0x6575,0x8575,0x7585,0x8585,0xB485,0x8595,0x85B4,0x8565,0x85B4,0x8575,0x5475,0x4646,0x4646,0x4646,0x4644,0x464F,0x464F,0x444F,0x464F,0x5F4F,0x4646,0x5F4F,0x465F,0x4F44,0x4F46,0x465F,0x4446
.word 0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4E4,0xD4E4,0xE4C5,0xE4D4,0xE4C5,0xE4E4,0xE4D4,0xC5D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE5D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D5,0xE4E5,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D5,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xAFC4,0x54C4,0x54AF,0x4441,0x6F6F,0xAFC4,0xBFC4,0xD485,0xD4AF,0xD4AF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4C4,0xBFD4,0xD4D4,0xD464,0xD4D5,0x64D5,0xC4AF,0x64AF,0x5454,0x5444,0x4441,0x4441,0x7F7F,0x7F41,0x7F7F,0x7F7F,0x7F44,0x447F,0x418F,0x448F,0x8F7F,0x7F44,0x7F41,0x7F44,0x7F41,0x7F44,0x417F,0x417F,0x418F,0xE47F,0xE4D4,0xD4D4,0x8F64,0x7F41,0x85E4,0x7FD4,0x8F54,0x7F44,0x7F41,0x7F8F,0xAF44,0x7F64,0xD4D4,0xD4D4,0xD4D4,0xD564,0x6464,0x51D5,0x649F,0x54AF,0xC4BF,0x9F64,0x4646,0x4644,0x4646,0x4646,0x4646,0x4446,0x464F,0x4644,0x465F,0x4646,0x4646,0x4446,0x9575,0x85C4,0xB485,0xC5B5,0x7585,0x5454,0x7554,0x7575,0x7554,0x6575,0x6575,0x8575,0x7575,0x8585,0x8565,0x7595,0x7585,0x8585,0x4465,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x464F,0x464F,0x464F,0x4F46,0x4F46,0x4F4F,0x4F46,0x5F46,0x464F,0x5F46
.word 0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE5E4,0xE4E4,0xE4F4,0xE4E4,0xE4D5,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E5,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E5,0xE4F4,0xF4E4,0xE4E4,0xE5D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xAFD4,0xD4C4,0x64AF,0x9F64,0x8F44,0x547F,0x64D4,0x64D4,0x64D5,0xD464,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAF74,0x64C4,0x64D5,0xAF64,0x9F64,0x549F,0x8F8F,0x7F7F,0x4441,0x448F,0x4141,0x7F44,0x7F41,0x417F,0x447F,0x417F,0x447F,0x418F,0x7F64,0x7F7F,0x7F44,0x7F41,0x7F44,0x7F44,0x8F44,0xD444,0xD4D4,0xD4D4,0x44E4,0x448F,0xC4E4,0x54AF,0x548F,0x418F,0x447F,0x7F41,0x6441,0x41E5,0xD47F,0xD4D4,0xD4D4,0xBFD4,0xAFD4,0x6464,0x65AF,0x75AF,0x549F,0x54AF,0x5464,0x4644,0x4646,0x4446,0x4646,0x4646,0x4646,0x464F,0x4646,0x4644,0x4646,0x4644,0x6546,0xC495,0xC495,0xC495,0xC495,0x7585,0x5454,0x7554,0x8575,0x8575,0x64D4,0xC485,0x8575,0x95B4,0x7575,0x6585,0xB495,0x8575,0x6585,0x5F46,0x4646,0x464F,0x4F4F,0x5F46,0x4F4F,0x465F,0x464F,0x4F46,0x464F,0x4F5F,0x5F46,0x4646,0x465F,0x4646
.word 0xE4E4,0xD4E4,0xE4D5,0xD5E4,0xF4E4,0xE4E4,0xE4E5,0xE5E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4C5,0xE4D4,0xC5D4,0xE4E4,0xC5D4,0xE4D4,0xD4E4,0xD4D5,0xE4E4,0xD4E5,0xE4F4,0xE5E4,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4C5,0xD4E4,0xD4D4,0xD4E4,0xD4C5,0xD4D4,0xD4D4,0xD574,0xD564,0x549F,0x449F,0x648F,0xAFD5,0x64D4,0xD4BF,0xD4D5,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD464,0xD4D4,0xD4D4,0xD4D4,0xD474,0xD4D4,0xD4D4,0xC4D4,0xC4BF,0xC4BF,0xC4AF,0x75AF,0x9F54,0x4454,0x4441,0x7F8F,0x7F41,0x448F,0x449F,0x5454,0x6454,0xAF54,0x6475,0x54AF,0xAF64,0x447F,0x417F,0x7F7F,0x417F,0x417F,0x417F,0x448F,0xD464,0xD4D4,0xD464,0x7FD4,0x7F41,0xC4E4,0xD564,0x548F,0x448F,0x647F,0xD47F,0xD4E4,0xD4E4,0xD441,0xD4D4,0xD4D4,0xD464,0xD4C4,0x64D5,0x64D5,0xAF54,0xAF75,0xAF75,0xAF54,0x5454,0x5451,0x4644,0x4F46,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4446,0x85B4,0xC485,0x6585,0x5454,0x7565,0x7554,0x7554,0x7565,0x8554,0x75C4,0xD475,0x7595,0x9575,0xB485,0x8575,0x4575,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646
.word 0xE4D4,0xE4C5,0xE4D4,0xE4E4,0xE4E4,0xE4E5,0xE4E4,0xE4F4,0xE4E4,0xD4E4,0xE4D5,0xE4D4,0xD4C5,0xD4E4,0xD4E4,0xC5D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4C5,0xE4D4,0xE4E5,0xF4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xF4F4,0xE5E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4D5,0xD4E4,0xE4E4,0xE4D4,0xC5E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64AF,0x9F64,0x4154,0xBF64,0x64C4,0xD4D4,0xD464,0x64D4,0xD4D4,0xD4D4,0xD4D4,0xD4BF,0xD4D4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0xD4D4,0xD564,0xD585,0xAF64,0x64C4,0x549F,0x418F,0x8F44,0x7F7F,0x8F44,0x8F41,0x9F41,0x9F54,0x9F54,0x9F54,0xD554,0x8F54,0x7F7F,0x7F7F,0x7F41,0x7F44,0x7F44,0x7F41,0x8F44,0xD464,0xD49F,0xD464,0x8FD4,0x7F44,0xD4E4,0x7FBF,0x8F54,0x7F41,0xD48F,0xD4E4,0xD4D4,0xE4E4,0xD464,0xD4D4,0xD4D4,0xD4D4,0x74D4,0xAFD4,0xAFC4,0x6464,0x549F,0xAF54,0x6475,0x54AF,0x54AF,0x5154,0x5454,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4446,0x4446,0x4646,0x7546,0x8585,0x8585,0x5475,0x5454,0x7554,0x7585,0x9585,0x7575,0x5485,0x7575,0x6585,0xD485,0x8585,0x75D4,0x8585,0x4446,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x4644,0x465F,0x4446,0x4646,0x4644,0x4644,0x465F
.word 0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xE4F4,0xE4E5,0xE4F4,0xE4E4,0xE4E4,0xE5E4,0xE4E4,0xE5E4,0xD5E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD474,0xAFC4,0x64C4,0xC4AF,0x64D5,0xD4D4,0xC4BF,0xD4D5,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x85BF,0x64D5,0xC4AF,0x75AF,0x549F,0x449F,0x4441,0x417F,0x8F44,0x8F44,0x8F44,0x5454,0x549F,0x7F8F,0x7F8F,0x7F44,0x7F41,0x417F,0x417F,0x417F,0x447F,0x8F7F,0x8F64,0x8F64,0x7F8F,0x8F54,0xD441,0xC4E4,0x41D4,0x8F54,0x7F64,0xD4D4,0xD4D4,0xD4D4,0x6464,0xE4E5,0xE4D4,0xD4D4,0xD4D4,0xC4BF,0xD4D4,0xD564,0x75AF,0x54AF,0x759F,0x759F,0x75AF,0xAF54,0xAF64,0x5454,0x5164,0x4644,0x4646,0x4646,0x465F,0x465F,0x4644,0x4646,0x4646,0x4646,0x465F,0xC454,0xD485,0x95B4,0x95C5,0x75C4,0x5454,0x7554,0x8575,0x8565,0x7575,0xD485,0x8554,0x7575,0x7575,0x8585,0x8575,0x4475,0x4646,0x465F,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xE4E4,0xE4D5,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4C5,0xD4E4,0xD4E4,0xE5D4,0xE5D4,0xE4E4,0xE5E4,0xE4F4,0xF4E4,0xE4F4,0xE4E4,0xE4F4,0xE4D5,0xE4E5,0xD4F4,0xE5E4,0xE4E4,0xE4E4,0xF4E4,0xE5E4,0xE4E4,0xE4D5,0xE4E5,0xE4E4,0xD5E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x64D4,0x75BF,0xC4AF,0xD4AF,0xD564,0xD485,0xD4BF,0xD4C4,0x74D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD564,0xD564,0xAF64,0xAF75,0x5454,0x548F,0x448F,0x7F8F,0x417F,0x8F7F,0x8F8F,0x7F44,0x7F41,0x7F41,0x417F,0x447F,0x447F,0x7F7F,0x7F44,0xAF64,0x6485,0xD4D5,0xC4C4,0x6464,0x44AF,0xE4D4,0x64D4,0x648F,0x44AF,0x64E4,0x64D4,0x7F9F,0xE4D4,0x648F,0xD47F,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x64D4,0x64D5,0xAF64,0x9F54,0xAF54,0xAF65,0xAF64,0x6465,0xD5AF,0x649F,0x4454,0x4646,0x465F,0x4646,0x4646,0x464F,0x4644,0x4646,0x465F,0x9564,0x85C4,0xB495,0xB495,0xD485,0x85B5,0x5475,0x5465,0x7565,0xC485,0x8575,0x8585,0x8575,0x75C4,0x8575,0x7565,0x7585,0x7585,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F
*/



/*   Dragon Girl
Fish_320:
.word 0x4646,0x4646,0x4646,0x4646,0x8373,0x8373,0x7373,0x4646,0x4646,0x4646,0x4646,0x4646,0x7342,0x5263,0x7373,0x4643,0x4646,0x4646,0x7363,0x5252,0x5263,0x6383,0x4646,0x6346,0x7352,0x8373,0x4646,0x4646,0x4646,0x4646,0x6342,0x7352,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6F46,0x7F6F,0x6F6F,0x7F6F,0x4141,0x8F42,0x6F7F,0x6F6F,0x6F6F,0x7F6F,0x7F6F,0x6F6F,0x5F6F,0x6F7F,0x5F7F,0x6F6F,0x6F6F,0x6F7F,0x6F7F,0x6F7F,0x6F6F,0x6F7F,0x6F7F,0x6F7F,0x7F6F,0x6F6F,0x6F6F,0x7F6F,0x7F6F,0x7F7F,0x7F7F,0x4646,0x6346,0x4663,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5242,0x8363,0xD4D4,0x4373,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x6342,0x8383,0x93D4,0x7383,0x5263,0x4643,0x4646,0x4646
.word 0x4646,0x4642,0x4646,0x4642,0x4346,0x7393,0x7373,0x8383,0x4673,0x4642,0x4646,0x4646,0x5246,0x5273,0x5273,0x6273,0x4643,0x4342,0x7373,0x7352,0x7352,0x5263,0x7383,0x7352,0x7352,0x7352,0x4673,0x4246,0x4646,0x4442,0x7352,0x7373,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x6F5F,0x7F42,0x6F6F,0x427F,0x428F,0x7F8F,0x6F6F,0x7F6F,0x6F6F,0x6F7F,0x6F7F,0x6F6F,0x6F6F,0x7F6F,0x6F6F,0x6F7F,0x7F7F,0x7F7F,0x7F7F,0x6F6F,0x6F6F,0x7F7F,0x7F7F,0x7F7F,0x6F7F,0x6F7F,0x7F7F,0x6F6F,0x7F7F,0x7F41,0x7F7F,0x467F,0x4342,0x52D4,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x7341,0x9383,0x93D4,0x5273,0x4646,0x4642,0x4246,0x4646,0x4642,0x4646,0x4642,0x4246,0x4646,0x4246,0x4646,0x4642,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4646,0x4642,0x4646,0x4642,0x4246,0x4646,0x4642,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4246,0x4646,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4642,0x4246,0x4646,0x4442,0x8352,0xB3D4,0x73B4,0x4373,0x4646,0x4642,0x4642,0x465F
.word 0x4646,0x5F46,0x4646,0x4646,0x4646,0x5246,0x52C4,0x6352,0x7362,0x7373,0x4246,0x4246,0x7346,0x7352,0x7352,0x7352,0x5252,0x6352,0x5252,0x5252,0x5273,0x5273,0x5252,0x5273,0x7352,0x7352,0x6252,0x7373,0x7352,0x7352,0x5252,0x7352,0x4683,0x4646,0x465F,0x4646,0x465F,0x4646,0x424F,0x7F7F,0x7F7F,0x7F41,0x8F52,0x7F41,0x6F6F,0x6F7F,0x6F7F,0x7F6F,0x7F6F,0x6F6F,0x6F6F,0x7F6F,0x6F6F,0x426F,0x5241,0xD462,0x5283,0x7F52,0x6F6F,0x417F,0x427F,0x7F7F,0x7F42,0x427F,0x6F7F,0x6F7F,0x6F7F,0x6F6F,0x427F,0x417F,0x7F7F,0x4646,0xC352,0x4273,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5242,0x8373,0x93D4,0x5283,0x4643,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x465F,0x4246,0x8352,0x93C3,0x73B4,0x4473,0x4646,0x4642,0x4646,0x4646,0x4646
.word 0x4246,0x4646,0x4246,0x5F46,0x4646,0x4646,0x8363,0x7373,0x7352,0x6273,0x7373,0x4646,0x5242,0x7352,0x5252,0x5263,0x5273,0x5273,0x5252,0x5252,0x5252,0x5252,0x6352,0x5252,0x5273,0x5252,0x5273,0x5273,0x5273,0x5252,0x7373,0x5273,0x5273,0x4646,0x4646,0x4646,0x4646,0x6F5F,0x4142,0x427F,0x7F7F,0x427F,0x7F41,0x6F7F,0x7F6F,0x7F6F,0x7F6F,0x426F,0x7F6F,0x7F6F,0x7F5F,0x6F42,0x6F6F,0x6241,0xD4D4,0xD4E4,0x9F62,0x6F7F,0x6F6F,0x5242,0x7F41,0x417F,0x7F6F,0x7F7F,0x427F,0x6F6F,0x7F7F,0x7F42,0x7F6F,0x6F41,0x417F,0x426F,0x7342,0x73D4,0x4642,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4246,0x4146,0x7342,0xB483,0xD4B3,0x8393,0x4352,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5246,0x4652,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4246,0x7352,0x9383,0x73B4,0x4273,0x4646,0x4642,0x5F46,0x4646,0x465F,0x4646
.word 0x4246,0x4343,0x4343,0x4343,0x4246,0x4646,0x4646,0x7373,0x7352,0x7352,0x5273,0x5273,0x7352,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5273,0x5252,0x5252,0x5252,0x4273,0x4246,0x5F46,0x6F46,0x7F42,0x5252,0x6F7F,0x427F,0x5241,0x428F,0x7F6F,0x6F6F,0x427F,0x6F7F,0x6F7F,0x6F6F,0x6F6F,0x416F,0x6F6F,0x526F,0xD562,0xD4E4,0xAF62,0x427F,0x6F6F,0x4241,0x7F8F,0x7F6F,0x7F7F,0x6F7F,0x6F7F,0x6F7F,0x6F41,0x6F7F,0x6F6F,0x6F41,0x427F,0x417F,0x6F6F,0x437F,0x9352,0x6373,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x5242,0x8373,0x93C4,0x93D4,0x8383,0x4252,0x4246,0x4646,0x5F46,0x5246,0x6352,0x5263,0x4641,0x4642,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4246,0x4646,0x4646,0x4642,0x465F,0x4246,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4246,0x5F46,0x4646,0x465F,0x4646,0x4242,0x8373,0x8383,0x8393,0x7393,0x7383,0x5252,0x4242,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4246,0x5252,0x8283,0xC493,0x7383,0x4643,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642
.word 0x7346,0x8383,0x8373,0x7373,0x8373,0x4283,0x4246,0x7373,0x5273,0x5273,0x5263,0x5252,0x5273,0x5252,0x5252,0x5252,0x5242,0x5252,0x5242,0x4252,0x4252,0x4252,0x5252,0x4252,0x5241,0x5242,0x5252,0x5252,0x5263,0x5252,0x5252,0x6352,0x5273,0x7373,0x4273,0x4646,0x416F,0x5241,0x7F9F,0x7F7F,0x4142,0x8F52,0x6F7F,0x6F7F,0x6F42,0x7F7F,0x427F,0x6F6F,0x6F7F,0x6F6F,0x6F6F,0x6F6F,0x5241,0x52AF,0x7F7F,0x7F7F,0x7F42,0x7F6F,0x6F6F,0x6F7F,0x7F7F,0x6F6F,0x6F6F,0x426F,0x7F6F,0x6F7F,0x6F7F,0x7F6F,0x7F6F,0x7F7F,0x7F41,0x4142,0x7F7F,0x5243,0x83D4,0x4663,0x4646,0x4646,0x465F,0x4446,0x4646,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4146,0x6352,0x8373,0xC483,0xD4B3,0x93B3,0x8383,0x7383,0x7373,0x7373,0x7373,0x5252,0x5263,0x4242,0x4252,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x8352,0x9393,0x93C4,0x7383,0xB483,0x9383,0x9393,0x83B4,0x5252,0x5252,0x6373,0x5252,0x5252,0x8373,0x8383,0x9293,0x83C4,0x73B4,0x4673,0x4642,0x6F46,0x4646,0x465F,0x465F,0x4246,0x4646,0x4646
.word 0x4646,0x4646,0x8373,0x7373,0x6352,0x5252,0x6373,0x7352,0x5273,0x5252,0x5252,0x5252,0x5252,0x5242,0x5242,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x7341,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x4252,0x5252,0x7352,0x7352,0x5252,0x7F42,0x4241,0x428F,0x427F,0x527F,0x9F41,0x7F8F,0x7F6F,0x6F7F,0x7F7F,0x6F41,0x7F7F,0x7F6F,0x5F6F,0x426F,0x7F6F,0x526F,0x527F,0x418F,0x4152,0x8F42,0x6F7F,0x7F6F,0x7F6F,0x416F,0x7F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F,0x7F42,0x6F6F,0x6F7F,0x6F7F,0x8F42,0x427F,0x7F41,0x467F,0x7352,0x6293,0x4652,0x465F,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x6341,0x5252,0x7352,0x7363,0x8373,0x93B4,0xD4C4,0xB393,0x9393,0xB3B3,0xB3D4,0x93D4,0x93C3,0x93C3,0x8383,0x6273,0x5252,0x4252,0x4252,0x4642,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4642,0x5F46,0x4246,0x4646,0x4642,0x4646,0x4646,0x465F,0x4F46,0x4646,0x5F46,0x4646,0x465F,0x4646,0x465F,0x4644,0x465F,0x5F46,0x4646,0x4642,0x4246,0x8352,0xC4C3,0xB483,0x6373,0x5273,0x5252,0x6373,0x5252,0x8352,0x7383,0x7383,0x7373,0x8373,0x8373,0x8383,0x5252,0x5252,0x7373,0x4452,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646
.word 0x4246,0x4646,0x6346,0x8383,0x7352,0x5273,0x7373,0x7352,0x7352,0x5252,0x5252,0x5252,0x5242,0x5252,0x5252,0x5252,0x5252,0x5273,0x7352,0x5273,0x7352,0x5252,0x7352,0x5252,0x5252,0x5252,0x5252,0x5273,0x5252,0x5252,0x5242,0x5241,0x5252,0x5252,0x4252,0x4141,0x8F41,0x8F42,0x7F41,0x8F52,0x8F52,0x6F7F,0x7F42,0x426F,0x7F7F,0x426F,0x6F7F,0x6F7F,0x426F,0x7F6F,0x6F6F,0x5252,0x42BF,0x9F62,0x9F52,0x6F8F,0x6F7F,0x6F7F,0x6F42,0x6F6F,0x6F6F,0x6F6F,0x426F,0x6F6F,0x6F6F,0x6F7F,0x416F,0x6F7F,0x426F,0x6F6F,0x417F,0x7F41,0x5242,0x7F8F,0x4246,0xC373,0x6373,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4241,0x5252,0x5252,0x5252,0x5252,0x8373,0xC483,0xC483,0x93B4,0x93C4,0x83C4,0xC493,0xC4E4,0xD4D4,0xD4B3,0x9382,0x83B3,0x7383,0x7373,0x5252,0x4252,0x4242,0x4642,0x4646,0x4646,0x4F46,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x8342,0x8393,0x83B4,0x7373,0x5252,0x4644,0x4646,0x4442,0x5252,0x6352,0x7363,0x5252,0x5273,0x5273,0x6352,0x6352,0x5252,0x4452,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4642,0x465F,0x4646,0x4646
.word 0x4646,0x5F46,0x4646,0x7363,0x5273,0x5252,0x5252,0x5252,0x5252,0x5252,0x5242,0x5252,0x5252,0x5252,0x7352,0x7352,0x7373,0x5252,0x7373,0x7373,0x5252,0x7373,0x7383,0x5273,0x7352,0x8383,0x5252,0x5252,0x5252,0x7352,0x5252,0x5252,0x4252,0x4152,0x417F,0x8F42,0x8F52,0x7F41,0x527F,0x429F,0x6F7F,0x6F42,0x7F7F,0x7F6F,0x7F41,0x6F7F,0x6F6F,0x426F,0x6F6F,0x6F41,0x836F,0x62D4,0xD562,0xC4AF,0x7F52,0x6F7F,0x6F7F,0x6F6F,0x7F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F,0x6F7F,0x6F6F,0x7F6F,0x427F,0x628F,0x4141,0x467F,0x6352,0x5252,0x5252,0x4642,0x4642,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4246,0x4646,0x4246,0x5242,0x5252,0x7352,0x7363,0xC483,0x8383,0xB483,0xB493,0x8383,0x8383,0x93B4,0xC4C4,0xB493,0x93C3,0xC3C3,0xB3C4,0xB382,0x8393,0x5273,0x5252,0x4142,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4446,0x4246,0x4646,0x465F,0x4246,0x5F46,0x4646,0x4642,0x5F46,0x4246,0x7383,0x7363,0x5273,0x5273,0x4644,0x4246,0x4646,0x4646,0x4646,0x4246,0x4646,0x4246,0x4646,0x4642,0x4246,0x4646,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646
.word 0x4246,0x9383,0x8383,0x7383,0x5273,0x7352,0x5252,0x5252,0x5252,0x5252,0x5252,0x5273,0x5252,0x8373,0x5273,0x5252,0x7352,0x7383,0x5252,0x5273,0x7383,0x5252,0x7373,0x5273,0x7373,0x7373,0x5263,0x8352,0x8383,0x5273,0x7352,0x7373,0x5252,0x7F41,0x9F52,0x5252,0x7F8F,0x7F42,0x5252,0x7F8F,0x7F7F,0x7F7F,0x427F,0x417F,0x7F6F,0x427F,0x7F6F,0x7F6F,0x7F41,0x6F7F,0x6252,0x62AF,0x62E5,0x62E4,0x7F7F,0x7F6F,0x6F6F,0x4142,0x7F7F,0x426F,0x7F5F,0x6F6F,0x6F6F,0x6F6F,0x5F41,0x6F6F,0x7F6F,0x426F,0x7F6F,0x6F7F,0x427F,0x7F7F,0x8F41,0x8FC4,0x6F41,0x5243,0x7352,0x9373,0x6373,0x4646,0x4646,0x4446,0x4246,0x5F46,0x4646,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4646,0x4146,0x5242,0x7352,0x7373,0x7373,0x7373,0x7383,0x7383,0x83B4,0x8373,0x8383,0x83B4,0xB4C4,0xC493,0xC493,0xD493,0x93D4,0x93D4,0x8393,0x7373,0x5252,0x4652,0x465F,0x4246,0x5F46,0x4646,0x465F,0x4646,0x4642,0x4246,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x7342,0x7393,0x5273,0x7373,0x5252,0x7383,0x4452,0x5F46,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x5F46,0x4646,0x4246,0x4646,0x465F
.word 0x8373,0x73B4,0xB483,0x6373,0x6362,0x5252,0x5252,0x5252,0x5252,0x5252,0x5273,0x5273,0x7373,0x7373,0x5273,0x7363,0x7352,0x8373,0x7373,0x7352,0xC483,0x7383,0x8352,0x83D4,0x5263,0x8373,0x7383,0x6352,0x8373,0x5273,0x7352,0x7373,0x5273,0x428F,0x42D4,0x529F,0x7F52,0x5241,0x42AF,0x418F,0x426F,0x7F7F,0x7F41,0x7F7F,0x4142,0x6F6F,0x417F,0x417F,0x7F6F,0x417F,0xAF52,0xD452,0xF5D4,0x8F52,0x6F6F,0x6F7F,0x417F,0x7F5F,0x7F42,0x6F41,0x6F6F,0x6F6F,0x6F42,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F7F,0x6F6F,0x7F6F,0x427F,0x428F,0x418F,0x437F,0x5252,0x7363,0x8383,0x73B3,0x4652,0x5F46,0x7384,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4246,0x5242,0x5252,0x6352,0x6362,0x5263,0x7373,0x7373,0x7373,0x73B4,0x8373,0x8383,0x83B4,0xC483,0xC483,0xC493,0xC483,0xC393,0xC393,0x8393,0x7373,0x4263,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4642,0x4646,0x4246,0x7363,0x5273,0x5273,0x7352,0x8373,0x8373,0x7383,0x7373,0x7373,0x4642,0x465F,0x4646,0x5F46,0x4246,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646
.word 0x8383,0x7383,0x7373,0x5273,0x5273,0x5252,0x5252,0x5252,0x7352,0x5252,0x5273,0x7373,0x5252,0x7352,0x7352,0x8383,0x7352,0x7373,0x8373,0x6383,0x7373,0x8383,0x8363,0x8383,0x5273,0xC483,0xC483,0x5273,0xD483,0x5283,0x5252,0x7373,0x8F51,0xD441,0xC49F,0x629F,0x7F9F,0xD552,0x8F41,0x7F7F,0x7F6F,0x7F41,0x7F7F,0x7F42,0x6F7F,0x6F7F,0x7F7F,0x7F7F,0x427F,0x416F,0x5252,0xD4D5,0xAF62,0x6FC4,0x7F7F,0x426F,0x416F,0x416F,0x526F,0x7F7F,0x6F7F,0x6F6F,0x6F6F,0x6F6F,0x426F,0x7F5F,0x6F6F,0x6F6F,0x426F,0x416F,0x7F6F,0x6F41,0x7F7F,0x8F41,0x8F42,0x7F42,0x7373,0x8383,0x93C4,0xD483,0x7393,0x8363,0x8363,0x4443,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4642,0x4646,0x4642,0x4646,0x466F,0x5242,0x5273,0x7352,0x7352,0x7352,0x5273,0x7373,0x7373,0x7373,0x7373,0x8373,0x8373,0x83B4,0xB483,0xB493,0xD493,0x83C4,0xB3C4,0x83D4,0x7383,0x4252,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4246,0x7383,0x5273,0x5273,0x8373,0xB483,0x7383,0x8383,0x7373,0x7352,0x5252,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4246,0x4646,0x4646,0x465F,0x4446,0x4246,0x4646,0x464F,0x5F46,0x4646
.word 0xB483,0x8352,0x6383,0x6362,0x5252,0x5252,0x6352,0x5252,0x5273,0x8373,0x8352,0x8383,0xC483,0x5273,0x7352,0xB483,0x7393,0x7373,0xC483,0x7383,0x7352,0x93C4,0x5283,0x9363,0x73C4,0x7363,0x8373,0x7383,0x8373,0x73B4,0x7373,0x6252,0x429F,0xD562,0xD462,0x8FE4,0x5242,0x529F,0x427F,0x7F41,0x7F42,0x427F,0x417F,0x417F,0x7F6F,0x7F42,0x427F,0x427F,0x6F7F,0x8F41,0x628F,0x62AF,0x41D5,0x6F41,0x426F,0x417F,0x7F7F,0x7F52,0x7F7F,0x7F42,0x6F41,0x6F7F,0x6F6F,0x6F41,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x7F7F,0x6F6F,0x7F6F,0x6F7F,0x7F7F,0x7F52,0x4141,0x4142,0x5252,0x8373,0x93C3,0x8383,0x7383,0x7373,0x4283,0x5F46,0x4646,0x4646,0x4642,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4246,0x4246,0x5252,0x5273,0x5273,0x5273,0x5273,0x7352,0x7352,0x7352,0x7373,0xB483,0x7383,0x8373,0xB493,0x8383,0xB493,0xB483,0xB493,0x8383,0x7393,0x4273,0x4646,0x465F,0x4646,0x4246,0x4646,0x4246,0x4646,0x4646,0x5F46,0x4646,0x5F44,0x4646,0x4246,0x6246,0x7373,0x7352,0x7352,0x7383,0x7373,0x8383,0x6373,0x4652,0x4642,0x4646,0x4246,0x4246,0x4646,0x465F,0x4646,0x4646,0x4644,0x4F46,0x4646,0x4642,0x4646,0x4646,0x4642,0x4646,0x4642,0x4646,0x464F,0x4646,0x4646,0x4646
.word 0x7373,0x7373,0x5252,0x5273,0x5252,0x5252,0x7373,0x5273,0x7352,0x5252,0x7363,0x7373,0x6383,0x9373,0x7363,0x8373,0x7373,0x5273,0x8373,0xC483,0x7352,0xB483,0x7352,0x8352,0x93D4,0x7352,0xD483,0x5293,0x6373,0x5283,0x8373,0x5273,0x628F,0x52BF,0x62AF,0x428F,0x629F,0x7F9F,0x7F41,0x7F7F,0x417F,0x417F,0x7F7F,0x6F42,0x7F7F,0x417F,0x7F7F,0x7F7F,0x6F7F,0x4142,0xAF52,0xAF73,0xAFC4,0x7F7F,0x6F7F,0x7F6F,0x527F,0x5262,0x417F,0xE441,0x6F7F,0x7F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x426F,0x6F6F,0x6F7F,0x6F41,0x6F7F,0x6F42,0x6F7F,0x6F7F,0x4241,0x7F8F,0x637F,0x5273,0x7342,0x5273,0x8363,0x7363,0x7373,0x7393,0x4643,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x4246,0x4246,0x5252,0x5273,0x5252,0x5273,0x5273,0x5252,0x7363,0x5273,0x7373,0x7373,0x7373,0x8373,0x83B4,0x83B4,0x8383,0x6373,0xD483,0x8383,0x7383,0x4643,0x4646,0x4246,0x4646,0x464F,0x4646,0x4646,0x4642,0x4646,0x4246,0x4646,0x4646,0x4646,0x5246,0x7373,0x7383,0x7373,0x83C4,0x8373,0x7373,0x4473,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4642,0x4642,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4246,0x4646,0x4642
.word 0x8383,0x7363,0x5252,0x5252,0x5252,0x5273,0x7352,0x7373,0x7373,0x8383,0x5273,0x7352,0x8352,0xB3D4,0x93C3,0x7373,0x8373,0x93C3,0x5283,0x83B4,0x73B3,0x7373,0xD483,0x5273,0x8373,0x5273,0xB483,0x5283,0x8373,0x52D4,0xC473,0x529F,0x52E4,0x628F,0x7FD5,0x9F52,0x8F62,0x427F,0x428F,0x417F,0x427F,0x417F,0x7F7F,0x7F6F,0x427F,0x7F7F,0x7F7F,0x4241,0x416F,0x427F,0x739F,0xC4AF,0x5262,0x6F6F,0x7F41,0x4152,0x9FC4,0xD4D4,0x6FD4,0x427F,0x7F41,0x6F42,0x6F6F,0x7F7F,0x6F6F,0x6F6F,0x5F6F,0x6F6F,0x6F7F,0x6F7F,0x6F42,0x6F6F,0x426F,0x7F6F,0x8F6F,0x8F42,0x4241,0xC483,0x5283,0x5252,0x7352,0x8373,0x8393,0x83B4,0x7383,0x4663,0x465F,0x5F46,0x4646,0x4646,0x4246,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x5241,0x5252,0x5252,0x5252,0x5252,0x5252,0x5273,0x7352,0x7352,0x7352,0x7373,0x6373,0x8383,0xB483,0x7363,0x7373,0x7363,0x7383,0x5263,0x4452,0x4246,0x4646,0x464F,0x4646,0x4642,0x465F,0x4646,0x465F,0x4646,0x4646,0x4246,0x5F46,0x7346,0x5252,0x5263,0x8373,0x7373,0xC483,0x7383,0x5252,0x4642,0x465F,0x4646,0x4642,0x465F,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4F46,0x4646,0x4646
.word 0x5273,0x5273,0x5252,0x5252,0x7352,0x7373,0x5273,0x5252,0x7373,0xB483,0xC493,0x6383,0x8373,0xD4B4,0x8383,0x5273,0x8363,0x7383,0x5263,0x8373,0x7373,0x7352,0x8383,0x5263,0xD483,0x6393,0x7352,0x5273,0x8383,0x7393,0xD452,0xAF52,0x8F42,0x9F52,0x5241,0xC49F,0x7F52,0x9F42,0x7F7F,0x7F41,0x7F42,0x7F41,0x7F42,0x7F6F,0x7F41,0x427F,0x7F41,0x7F7F,0x7F6F,0x417F,0x9F52,0xAF73,0x6FD4,0x7F6F,0x6F7F,0x4152,0xF562,0x52D4,0x52D4,0x8F7F,0x7F7F,0x416F,0x6F6F,0x426F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x426F,0x6F7F,0x6F6F,0x7F6F,0x7F6F,0x7F6F,0x6F6F,0x527F,0x7F7F,0x9373,0x93C4,0x6383,0x5252,0x7373,0xC483,0x8383,0x93B4,0x6393,0x4646,0x4646,0x4246,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x5246,0x6383,0x7373,0x7383,0x7352,0x5273,0x5252,0x5252,0x5252,0x5273,0x5252,0x7352,0x7352,0x6263,0x8352,0x7352,0x7383,0x5263,0x8373,0x7383,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x7346,0x7373,0x7352,0x7383,0x7352,0x8383,0x83B4,0x7352,0x5273,0x5263,0x4252,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646
.word 0x5263,0x5252,0x5252,0x5252,0x7352,0x8373,0x8383,0x7373,0x5273,0x7363,0x7383,0x8352,0x5273,0x7373,0x7373,0xB3C4,0x5283,0x8352,0xB3C4,0x5273,0xC483,0x83D4,0x7352,0x7393,0x7373,0x7383,0x8373,0x83D4,0x7363,0x7363,0xAF52,0x8F62,0x4241,0x428F,0x529F,0xAF62,0x428F,0x429F,0x7F41,0x417F,0x417F,0x7F7F,0x6F7F,0x4241,0x417F,0x417F,0x426F,0x6F41,0x427F,0x8F41,0x8F52,0x5252,0x7FAF,0x6F6F,0x526F,0x527F,0xD49F,0x9F62,0x8FD4,0x6F41,0x7F7F,0x7F7F,0x7F6F,0x6F7F,0x6F41,0x6F42,0x6F7F,0x427F,0x6F6F,0x417F,0x6F6F,0x6F41,0x7F6F,0x7F6F,0x7F7F,0x416F,0x8F42,0x8352,0xC483,0x8373,0x7383,0x6352,0x7373,0xB483,0x7373,0x9383,0x6283,0x4443,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x5F46,0x4246,0x6263,0x6352,0x6352,0x5252,0x5273,0x5273,0x5252,0x5283,0x8363,0x5273,0x5273,0x5263,0x5252,0x6373,0x5273,0x7352,0x9383,0xC3D4,0x93D4,0x9383,0x4652,0x4246,0x5F46,0x4646,0x4246,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x5242,0x6263,0x7352,0x7373,0x7373,0x7373,0x5273,0x7373,0x5273,0x8373,0x7373,0x8373,0x7383,0x4662,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x5F46,0x4446,0x4646,0x4642,0x4646,0x465F,0x5F46,0x4F46,0x4646,0x4646,0x4646
.word 0x5252,0x5252,0x5252,0x7352,0x5273,0x8373,0x7373,0x8352,0x93D4,0x5273,0x5252,0xC483,0xD493,0x8393,0x7352,0xC483,0x83C4,0x7363,0x7383,0x5273,0x8383,0x6373,0x7352,0x83C4,0x5263,0x5273,0x7373,0xC483,0x8373,0x5293,0x52D4,0x428F,0x528F,0x528F,0x9F52,0x4252,0x628F,0x8F52,0x427F,0x427F,0x427F,0x7F7F,0x427F,0x7F7F,0x4241,0x7F7F,0x8F52,0x6F7F,0x7F7F,0x8F52,0x8F52,0xD552,0x6F6F,0x6F41,0x7F6F,0x9F52,0x5252,0x52AF,0xD552,0x427F,0x7F6F,0x426F,0x6F7F,0x426F,0x7F5F,0x7F6F,0x426F,0x7F5F,0x6F6F,0x7F6F,0x426F,0x7F6F,0x6F6F,0x6F6F,0x426F,0x7F6F,0x417F,0x7342,0x83C4,0x7383,0xB483,0x5273,0x5252,0x8373,0x8383,0xB473,0x8383,0x9383,0x7362,0x4373,0x4246,0x5F46,0x4646,0x465F,0x4246,0x5F46,0x4646,0x5246,0x7373,0x5273,0x8352,0x5263,0x7352,0x5252,0x5252,0x6373,0x7373,0x5273,0x7352,0x5252,0x5252,0x5252,0x5252,0x6352,0x5273,0xB483,0xC483,0xC4C3,0x5293,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x465F,0x4246,0x4646,0x465F,0x6363,0x5262,0x5252,0x7373,0x7352,0x7373,0x7373,0x5252,0x7383,0x6373,0x6362,0x6352,0x7373,0x4642,0x4642,0x5F46,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x464F,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F
.word 0x5252,0x5252,0x5273,0x8373,0x83C4,0x5263,0x7363,0x7352,0x8383,0x83C4,0x6373,0x8373,0x73B4,0x9363,0x73D4,0x7352,0x8352,0x93C3,0x8363,0xD483,0x5263,0x8383,0x5293,0x7363,0x9373,0x93C3,0x6373,0x7373,0x8352,0x9F63,0x41D4,0x428F,0x8F7F,0xAF62,0x8F52,0x528F,0x7F9F,0x7F8F,0x7F41,0x417F,0x7F7F,0x7F7F,0x7F42,0x7F41,0x8F42,0x4152,0x8F52,0x427F,0x7F41,0x7F52,0x528F,0xD49F,0x7F6F,0x426F,0x6F41,0x4241,0x529F,0x9F62,0x527F,0x6F7F,0x6F41,0x5F41,0x6F41,0x7F6F,0x6F7F,0x416F,0x7F5F,0x7F7F,0x6F6F,0x6F7F,0x7F41,0x7F7F,0x6F42,0x6F6F,0x7F6F,0x7F6F,0x7F41,0x736F,0x9383,0x83B4,0x8383,0xD483,0x6383,0x5252,0xB473,0x7373,0xB483,0x8383,0x8383,0x8383,0x7383,0xB483,0x8393,0x83B4,0x6383,0x7373,0x7373,0x5273,0x5273,0x7352,0x7352,0x7373,0x7352,0x8373,0x7363,0x5273,0x5273,0x5252,0x7373,0x7352,0x5273,0x7352,0x5273,0x5252,0x5252,0x5273,0x8373,0x7383,0x7373,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x7346,0x9383,0xD4D4,0x7383,0x5273,0x5252,0x5273,0x7352,0x7352,0x7373,0x5273,0x7373,0x7373,0x4352,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4642,0x4646,0x4246,0x4646,0x4246,0x4646,0x465F,0x4646
.word 0x5252,0x5252,0x7352,0x8373,0x5273,0x8383,0x8383,0x5283,0x7373,0x6373,0x8383,0x6383,0x5252,0x8383,0x83C4,0x7363,0x8352,0xB483,0x6383,0x9383,0x5273,0x8363,0x73B4,0x8383,0x8352,0x83B4,0x7352,0x83B3,0x6352,0xC452,0x529F,0x7F8F,0x6241,0x529F,0x528F,0x8F62,0x5273,0x7F41,0x7F42,0x8F52,0x7F42,0x427F,0x417F,0x8F42,0x8F52,0x628F,0x7F8F,0x416F,0x417F,0x427F,0x428F,0x528F,0x6F6F,0x7F41,0x427F,0x8F6F,0x7F41,0x428F,0x7F9F,0x5241,0x7F6F,0x7F6F,0x416F,0x7F6F,0x7F42,0x6F6F,0x7F41,0x4241,0x417F,0x426F,0x7F52,0x7F7F,0x6F6F,0x7F7F,0x7F6F,0x6F7F,0x7F7F,0x637F,0x8373,0xC493,0x73B4,0x7373,0x9383,0x5273,0x5252,0x7373,0x8373,0x7373,0x83B4,0x83B4,0x83B4,0x9393,0xB3C4,0x93A3,0x93C3,0x83C3,0x83C3,0x9383,0x8383,0x83C4,0x8383,0x7373,0x7373,0x5252,0x7373,0x5273,0x7352,0x5283,0x5252,0x5273,0x5252,0x7373,0x5283,0x8352,0x5252,0x5252,0x6352,0x5263,0x5273,0x4646,0x4246,0x4646,0x4642,0x4646,0x465F,0x4246,0x4646,0x4646,0x8373,0xC3B3,0x93D4,0xD4B3,0x8393,0x7373,0x5252,0x5273,0x8373,0x7373,0x8373,0x8373,0x7383,0x4643,0x4642,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4F46,0x4446,0x4646,0x4646,0x4246,0x4646,0x464F,0x4646,0x464F,0x4646,0x4646
.word 0x5252,0x8352,0x5263,0x7352,0x7373,0xB483,0x7383,0x8373,0x83C4,0x5273,0xB483,0x8393,0x5273,0x7373,0x7373,0xC393,0x6373,0x7373,0x8393,0x5263,0xC383,0x5283,0x5252,0xB483,0x5283,0x6352,0x7352,0x93C4,0x73C4,0x6262,0x7F9F,0x527F,0xAF52,0x8F52,0x8F52,0xAF52,0x8F42,0x7F7F,0x417F,0x7F8F,0x6F41,0x7F41,0x4152,0x528F,0x529F,0x9F62,0x6F42,0x4241,0x7F7F,0x417F,0x7F41,0x6F52,0x7F7F,0x7F6F,0x7F7F,0x6F7F,0x427F,0x417F,0x5242,0x429F,0x7F6F,0x4152,0x7F6F,0x7F42,0x7F7F,0x7F41,0x7F42,0x417F,0x4142,0x7F6F,0x7F7F,0x6F42,0x417F,0x426F,0x427F,0x7F6F,0x426F,0x417F,0xC473,0x93B4,0x8383,0x7373,0xB483,0xC483,0x5273,0x5252,0x5273,0x8373,0x7373,0x8383,0xB483,0xB493,0xB483,0x83C4,0x83B4,0x83C4,0x93B4,0x83B4,0xB3C4,0xC3C4,0xD4C4,0xC393,0xD493,0xB3D4,0x8383,0x7373,0x7352,0x7352,0x7383,0x5252,0x5252,0x5252,0x5273,0x5273,0x5273,0xB483,0x5252,0x5252,0x4652,0x4246,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x435F,0x8373,0xB383,0x83C4,0xC483,0x8383,0x83C4,0xB3D4,0x5283,0x7352,0x6352,0x7373,0x7373,0x5273,0x4352,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x4646,0x465F,0x4642,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646
.word 0x6352,0x8373,0x8373,0x5273,0x5252,0x7383,0x6383,0xC483,0xC4C3,0x5273,0x7373,0x6373,0xD483,0x5283,0x7352,0x8383,0x73D4,0x7352,0xB483,0x5252,0xC483,0xB483,0x7393,0x8363,0x6373,0x9393,0x6263,0xB473,0x5283,0x9FD4,0x7F42,0x8F42,0x5252,0x428F,0x62D4,0x528F,0x429F,0x428F,0x417F,0x427F,0x427F,0x428F,0x529F,0x528F,0x8F52,0x8F52,0x7F7F,0x8F42,0x7F7F,0x7F42,0x7F7F,0x4162,0x4142,0x7F7F,0x527F,0x5252,0x6F6F,0x7F41,0x528F,0x419F,0x4241,0x5241,0x8F52,0x7F41,0x7F41,0x7F42,0x416F,0x427F,0x429F,0x7F8F,0x7F41,0x7F7F,0x6F7F,0x6F6F,0x6F7F,0x427F,0x7F7F,0x6F7F,0x8383,0x8373,0x93B4,0x8373,0x7373,0x8383,0x8383,0x5263,0x7352,0x7352,0x8373,0xB483,0xB493,0x8383,0x8383,0x8383,0xB483,0xB493,0xB493,0xB493,0xB483,0x8383,0x8383,0x83B4,0x93C4,0xD493,0xD4D4,0xC493,0x7383,0x7363,0x5273,0x8352,0x6373,0x5252,0x5252,0x5252,0x7352,0x7352,0x7352,0x7373,0x5241,0x4242,0x4646,0x4646,0x4646,0x4646,0x4644,0x945F,0x8383,0xB383,0xB483,0xB493,0xB483,0xB483,0xB483,0x9383,0x83D4,0x5263,0x7352,0x7373,0x8383,0x7363,0x5252,0x4252,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4642,0x4246,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x4246
.word 0x5252,0x7373,0x8373,0x8383,0x6373,0x5252,0x5252,0x8383,0x73B4,0xD483,0x5283,0x5273,0x8373,0x7383,0x5263,0x7352,0x9352,0x5283,0x6263,0xC483,0x6373,0x5273,0x8373,0x7352,0x5273,0x7373,0x7383,0x7352,0xAF52,0x4152,0x528F,0x527F,0x8F9F,0xD442,0x8F9F,0x62C4,0x418F,0x417F,0x8F52,0x7F7F,0x4141,0xAF52,0x5252,0x9F62,0xD552,0x7F8F,0x7F42,0x7F41,0x7F42,0x7F41,0x6252,0xC4AF,0x628F,0x5242,0x7FC4,0x8F51,0x5252,0xC49F,0x9F52,0x52C4,0xAFD4,0x9FC4,0xC4D4,0xC441,0x527F,0x7F52,0x7F62,0x7F7F,0x7F41,0x7F7F,0x6F42,0x427F,0x7F7F,0x7F52,0x7F6F,0x7F7F,0x7F6F,0x6F42,0x8373,0x83B4,0xB483,0xB483,0x8373,0x6373,0x7373,0x8383,0x5263,0x5273,0x5273,0x6373,0x7373,0x7363,0x7363,0x6373,0x8373,0x8373,0x8383,0x7373,0x8383,0x73B4,0x83B4,0x8373,0xB483,0xC483,0x8393,0x83C4,0x8383,0x8383,0x7373,0x6373,0x7352,0x7373,0x5273,0x6352,0x5252,0x5252,0x5252,0x5252,0x5252,0x5273,0x6242,0x5252,0x6252,0x5242,0x8373,0x9383,0xB493,0xC483,0xB493,0x8383,0x8373,0x8383,0x8383,0x83B4,0xB3C4,0x7383,0x8352,0x7383,0x7352,0x8373,0x83C3,0x7373,0x5273,0x4252,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x5F46,0x4446,0x4246,0x4646,0x4F46,0x4646
.word 0x5273,0x5273,0x7363,0x8373,0x8363,0xD493,0x7383,0x7363,0x7373,0x8383,0x83D4,0x7363,0x6352,0x5273,0xC393,0x7393,0xB452,0x7383,0x5273,0x8373,0x5273,0x5283,0x6373,0x7373,0x5283,0x5263,0x5252,0x8373,0x6252,0x429F,0x7F8F,0x4241,0x428F,0xE49F,0x41D4,0x7F9F,0x8F42,0x8F42,0x7F42,0x4142,0x527F,0x73AF,0x529F,0x8FD5,0x6252,0x7F41,0x7F6F,0x7F6F,0x8F41,0x7F7F,0xD562,0x62D4,0x73AF,0x529F,0xD48F,0x6252,0x52AF,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF4F4,0xE4E4,0xAF62,0x9F83,0x8F62,0x627F,0x4142,0x7F8F,0x7F42,0x7F7F,0x7F7F,0x7F7F,0x416F,0x7F6F,0x7F6F,0x7F6F,0x7F6F,0x9352,0x7383,0x8383,0x8383,0x7383,0x7383,0x5273,0x5252,0x5273,0x5273,0x5252,0x6263,0x7352,0x5252,0x5273,0x7352,0x7373,0x8373,0x73B4,0x8383,0x8373,0x8373,0x8383,0x83B4,0x8383,0x83B4,0x83B4,0xB493,0xC473,0xC493,0xC4B3,0x7383,0x7352,0x7383,0x5252,0x5273,0x7352,0x7373,0x5252,0x5252,0x5252,0x5252,0x5252,0x5273,0x5252,0x5273,0x9383,0xB493,0xB493,0xB483,0x8383,0x8373,0x73B4,0x7373,0x7373,0x7373,0xC483,0x7393,0x6352,0x7373,0x8352,0xC483,0xB4B3,0x8383,0x7383,0x5252,0x4646,0x465F,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4F46,0x4246,0x464F,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646
.word 0x9373,0x7383,0x7352,0x6352,0xB462,0xB483,0x8373,0x92D4,0x7383,0x7363,0x7373,0x9383,0x93C4,0x5273,0x7373,0x8363,0x6383,0x7352,0x7373,0x7352,0x5273,0x6373,0x7352,0x7352,0x5273,0x7373,0x8373,0x5252,0x42AF,0x8F52,0x427F,0x8F7F,0x8F41,0x83E4,0xC48F,0x5262,0x528F,0x429F,0x628F,0x428F,0x62D4,0x9F62,0xBF73,0xD452,0x42AF,0x7F8F,0x7F7F,0x7F42,0x427F,0x627F,0xD452,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4AF,0xD4D4,0xE4E4,0xE4E4,0xF4E4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xD4D4,0x62C4,0x8F62,0x428F,0x8F52,0x7F41,0x7F7F,0x427F,0x7F6F,0x6F6F,0x7F42,0x416F,0x7F6F,0x416F,0x736F,0xC483,0x5273,0x7363,0x93C4,0x83C4,0x7373,0x5273,0x5252,0x8373,0xB3C4,0x83D4,0x5263,0x7373,0x8383,0x8383,0x7363,0x7373,0x7373,0x8373,0x73B4,0x8373,0x8373,0x7383,0x83B4,0xC383,0x83D4,0x7373,0x7352,0x8373,0x93C4,0x83C4,0x6383,0x7352,0x7383,0x7352,0x7373,0x5273,0x8373,0x83C4,0x7363,0x8373,0x5283,0x7373,0x6383,0x8352,0xD493,0xD4C3,0xB493,0x8393,0x8373,0x83B4,0x8383,0xB483,0x8383,0x8383,0x83B4,0x73B4,0x7352,0x7393,0x8373,0x83B4,0x7383,0x73D4,0x4352,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xB483,0x7383,0xD473,0x7383,0x7352,0x7383,0x8352,0xC483,0x7383,0x7383,0x5252,0x8373,0x7383,0x8373,0x5263,0x8373,0x7373,0x5273,0x7352,0x5252,0x6373,0x5252,0x5273,0x5273,0x5252,0x5273,0x7363,0x4152,0x9F42,0x7F8F,0x8F42,0x7F41,0x4142,0x9F62,0x8F52,0x428F,0x429F,0x4141,0x8FC4,0x6241,0xD4D5,0x9FD4,0xD4C4,0x528F,0x9F62,0x6F42,0x7F41,0x7F7F,0x7F41,0x9F52,0xBF62,0xD4C4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xF4F4,0xE4F4,0xE4F4,0xE4D4,0xE4E4,0xD4E4,0xAFE4,0x5242,0x8F7F,0x528F,0x4152,0x6F42,0x6F6F,0x7F6F,0x6F42,0x7F7F,0x7F6F,0x426F,0x6F6F,0x527F,0x8373,0x73D4,0x5252,0x8373,0xB483,0x7373,0x5273,0x93C4,0xB3C4,0xC3D4,0x93D4,0x7373,0x7373,0x7383,0x7363,0x5273,0x8373,0x8383,0x7373,0x8383,0xC473,0x7383,0x8373,0xD483,0x83B4,0x7373,0xD4D4,0x5273,0x5242,0x5252,0x4252,0x5252,0x5252,0x5252,0x4252,0x5252,0x5242,0x7352,0x7352,0x5252,0x5252,0x6352,0x6362,0x5252,0xC483,0xC4C3,0xC3E4,0x93C4,0x83C4,0x83B4,0x8373,0x83B4,0x7373,0x7383,0x83B4,0x7373,0x5252,0x8373,0x83D4,0x93C4,0xB483,0xC493,0x4373,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4644,0x4646,0x4642,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646
.word 0x7352,0x5273,0x8383,0x83D4,0x9373,0x83D4,0x6373,0x7373,0x6373,0xD483,0x5293,0x5263,0x6352,0x7352,0x6273,0x6352,0x7352,0x7373,0x5252,0x5252,0x5273,0x5273,0x7352,0x5252,0x5252,0x5252,0x7352,0x9F52,0x4142,0x4142,0x417F,0x427F,0x629F,0x428F,0x739F,0x419F,0x9F62,0xD442,0x429F,0xE49F,0xAF62,0xE4C4,0x62E4,0xD4D5,0x8F62,0x7F7F,0x6F6F,0x7F41,0x7F7F,0xD562,0xD562,0xD472,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4F4,0xF3F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xC4E4,0x62AF,0x52D5,0x8F52,0x7F41,0x7F7F,0x7F41,0x7F41,0x6F7F,0x6F7F,0x6F42,0x7F6F,0x6F6F,0x5242,0x5252,0x7373,0x6383,0x7352,0x7383,0x7373,0x5273,0x8373,0xD4D4,0xD493,0xB493,0x5273,0x5273,0x5252,0x7352,0x5273,0x5252,0x7352,0xB483,0x7373,0x8383,0x8373,0x83C4,0x83B4,0xB363,0x5263,0x7352,0x9383,0x7373,0xC373,0x8383,0x5273,0x8383,0x8383,0x83C4,0x8383,0x8383,0x7373,0x7373,0x7373,0x7373,0x7373,0x8383,0x8383,0x93D4,0xB3D4,0x93C4,0x83C4,0x83B4,0x8373,0xB483,0x8383,0x7383,0x7373,0x7373,0x7383,0x5252,0x7352,0x8383,0xB493,0x5283,0x4643,0x4246,0x4646,0x5F46,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4F46,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x5F44
.word 0x7352,0x5252,0x8373,0x6383,0xC483,0xC4B3,0x5273,0x5273,0x7352,0x8383,0x73B4,0x8383,0x7383,0x5273,0x7373,0x62C4,0x7352,0x7352,0x5252,0x5252,0x5252,0x5252,0x4252,0x5241,0x4252,0x4252,0x5252,0x428F,0x8F8F,0x428F,0x427F,0x417F,0xF662,0x528F,0x429F,0x8F41,0x8F52,0x9F62,0x8F42,0x62C4,0xF4D4,0xD462,0x8FE4,0xAF52,0x7F62,0x427F,0x6F7F,0x426F,0x527F,0xBF62,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE3E4,0xF4F4,0xF3F4,0xF4F4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xBF62,0x8F62,0x8F42,0x7F42,0x7F6F,0x7F6F,0x7F6F,0x6F42,0x7F6F,0x7F6F,0x6F6F,0x8352,0x6383,0x5252,0x5252,0x5252,0x8373,0x7383,0x7373,0x7352,0xC493,0x7373,0x7352,0x7352,0x6352,0x7352,0x5283,0x5273,0xB483,0x6373,0x7352,0x7373,0x8373,0xC473,0xB493,0x7352,0x5283,0x8352,0x92D4,0xD4D4,0x7373,0xD493,0xD4D4,0x7383,0xC452,0xD493,0xD4B3,0xD4C3,0x93D4,0x93C4,0xB3C4,0x93C4,0xC3D4,0xC3E4,0x83B4,0x93B4,0xC4C3,0xD4C3,0x83D4,0x83C4,0x8373,0x7383,0x7383,0xB483,0x7383,0x8383,0x8373,0x5273,0x5252,0x7373,0x8363,0x5273,0x4643,0x4246,0x4646,0x465F,0x4646,0x5F46,0x4446,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4642,0x465F,0x4646,0x4646
.word 0xD483,0x7393,0x5273,0x5252,0x7363,0x5273,0x8373,0xC383,0x5283,0x7352,0x5252,0x7373,0x7373,0x7373,0x7352,0x5252,0x5252,0x5252,0x5252,0x6352,0x5252,0x5273,0x5252,0x5273,0x5252,0x5252,0x4252,0x8F41,0x4142,0x7F7F,0x7F41,0x5241,0x42AF,0x628F,0x8F62,0x527F,0x528F,0xC4AF,0x6241,0x9FD4,0xD562,0x62D4,0x529F,0xE4E4,0x7F41,0x7F7F,0x417F,0x7F7F,0x6241,0xD4D5,0xD462,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D3,0xF4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xD4C3,0xD3E4,0xE4E4,0xD4D4,0xE4E4,0xC4D4,0x739F,0x418F,0x417F,0x427F,0x6F6F,0x6F6F,0x7F6F,0x7F41,0x7F6F,0x6F6F,0xB46F,0x7383,0x8373,0xC493,0x7383,0x6352,0x8383,0x8352,0x6383,0x8373,0x7363,0x8383,0x7363,0x9373,0xB483,0x7373,0x8352,0x5273,0x8352,0x8383,0x5273,0x6352,0x6383,0x5273,0x7352,0x8383,0x83C4,0xB483,0x9393,0x5273,0x93C4,0xC493,0x6383,0x9352,0xD4C4,0x93D4,0xC3D4,0xD4D4,0xC4C3,0xE493,0xD493,0xD4C3,0x8383,0xC493,0xE493,0x93C4,0x93D4,0x83C4,0x8373,0x73B4,0xB483,0xB483,0x8383,0x7373,0x7383,0x6373,0x7352,0x6352,0x9383,0x6383,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4644,0x4642
.word 0xC483,0x5283,0xD483,0x93C3,0x5252,0x5252,0x8373,0x83B4,0x8363,0xD4D4,0x7383,0x5252,0x5263,0x5273,0x5252,0x5252,0x5252,0x5252,0x5273,0x5252,0x5252,0x5252,0x4273,0x4252,0x5252,0x5252,0x8F52,0x4241,0x428F,0x417F,0x7F7F,0xAF42,0x41E4,0x9F42,0x8F42,0x8F42,0x6242,0x9F52,0xD58F,0x6252,0x62D5,0xC4AF,0x629F,0xAF62,0x8F42,0x6F42,0x6F6F,0x7F41,0xAF62,0xD4C4,0xD4BF,0xD4D4,0xD4E4,0xC3E4,0xE4E4,0xE4E4,0xE4E3,0xF4F3,0xF4F4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xC3D4,0xD4D4,0xD4D4,0xAF62,0x5252,0x7F8F,0x6F6F,0x7F41,0x4142,0x6F7F,0x426F,0x6F6F,0x6F6F,0x837F,0x7383,0x8373,0x83B4,0x83C4,0x5252,0xB452,0x8373,0x5273,0x7383,0x8393,0x83B4,0x8383,0x6383,0x9373,0x83E4,0x5273,0x7373,0x8383,0x73B4,0x7352,0x7383,0x5283,0x5273,0x6383,0x5252,0x6373,0x7373,0x8373,0x7373,0xB483,0x7373,0x5252,0xC473,0xC393,0xC3E4,0x93C4,0x93C4,0x93C4,0x93D4,0xC3C4,0xD4C4,0xC493,0xC3D4,0xB3C3,0x93C4,0x93C4,0x73B4,0x8373,0x8383,0x83B4,0x8383,0x7373,0x8373,0x8373,0x5263,0x5252,0x8352,0xC473,0x7383,0x4246,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4646,0x465F,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4642,0x4646,0x4246,0x4F46,0x4646
.word 0x7363,0x5252,0x8373,0xB483,0xD473,0xC493,0x5252,0x5273,0x8373,0x7383,0x7373,0x5283,0x5273,0x5252,0x5252,0x7352,0x5252,0x5252,0x5252,0x5252,0x5252,0x5242,0x4246,0x4246,0x4246,0x4142,0x7F8F,0x8F41,0x7F41,0x427F,0x417F,0x6252,0x8F52,0x9F52,0x8F52,0x4241,0x9F8F,0x8F52,0xD442,0x629F,0x529F,0x42AF,0xF552,0x8FC4,0x6F42,0x6F7F,0x6F41,0x427F,0xD562,0xD462,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4E3,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xC4D4,0x628F,0x427F,0x7F7F,0x6F6F,0x7F41,0x4142,0x6F6F,0x6F6F,0x6F6F,0x736F,0x7373,0x7373,0x8352,0x8383,0x83D4,0x5263,0x8352,0x7352,0x8373,0x7363,0x8383,0xB473,0x5283,0x7373,0x7352,0xE483,0x83D4,0x7373,0x7352,0x7373,0x7373,0xC393,0x8383,0x8383,0x83C4,0x5273,0x5252,0x5252,0x5252,0x5252,0x5252,0x8373,0x8383,0x83C4,0x93B4,0xD4C4,0xD4B3,0xE493,0xC4C3,0x93E4,0xC3C4,0xB3C4,0xC3E4,0x63C4,0xC483,0x5273,0x7373,0x7383,0xB483,0x7383,0x7373,0x7373,0x7373,0x5252,0x5252,0x7352,0xB483,0xD483,0x83B3,0x4652,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x4642,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646
.word 0x5273,0x7373,0x6373,0x7373,0x8352,0x83B4,0xC483,0x8393,0x7352,0x7373,0x7352,0x5273,0x5252,0x5252,0x5273,0x5252,0x5252,0x5252,0x4442,0x4646,0x4246,0x6F46,0x4646,0x465F,0x4646,0x4646,0x7F41,0x7F42,0x7F42,0x7F41,0x427F,0xD4D5,0x528F,0x428F,0x428F,0x8F7F,0x8F42,0x7F42,0x6241,0x739F,0x529F,0x9F52,0xAF62,0x8F62,0x7F7F,0x7F7F,0x427F,0x627F,0xD4AF,0xD4D4,0xD4D4,0xE4D4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D3,0xE4E4,0xD4C3,0xD4E4,0xD4E4,0xD4D4,0x62D4,0x629F,0x7FAF,0x7F42,0x6F41,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x736F,0x8373,0x8373,0x7373,0x83B4,0xC483,0x5283,0x5252,0x5252,0x8363,0x5273,0x6373,0x7373,0x5273,0x5273,0x6373,0x8352,0xD493,0xD4D4,0x7383,0x7352,0xE483,0xC493,0xC483,0xC493,0xD4B3,0x8393,0x7383,0x5273,0x5273,0x5273,0xC483,0xD4B3,0xD4C3,0xE4C3,0xD4C3,0xD4C3,0xD493,0xC4C3,0xC493,0xD4B3,0xD4C3,0xD493,0xC4C3,0x5273,0x7383,0x7352,0x8373,0x7383,0x8383,0x7373,0x5252,0x5252,0x5252,0x5252,0x7352,0x8383,0x93C4,0xB483,0xD483,0x7383,0x4643,0x4246,0x4646,0x4646,0x4642,0x4646,0x4246,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x465F,0x4646,0x464F,0x4646,0x4642
.word 0xC493,0x5283,0x8383,0x5263,0x5273,0x5273,0x7363,0x6373,0x8383,0x5273,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x4646,0x4246,0x6F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x416F,0x428F,0x417F,0x427F,0x518F,0x529F,0x528F,0x419F,0x7F52,0x7F41,0x8F41,0x7F41,0x8F52,0x9F42,0x9F42,0x8F52,0xC4E4,0x428F,0x416F,0x6F7F,0x7F6F,0xD541,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4D3,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0x62D4,0x73AF,0x7F52,0x7F41,0x7F6F,0x6F7F,0x6F7F,0x7F7F,0x6F7F,0x6F6F,0x737F,0x7383,0x7383,0x5263,0x7373,0x8383,0x93C4,0x63C4,0x5252,0x8373,0x7363,0x7373,0x5273,0x8373,0x6352,0x7373,0x6373,0x8352,0x9393,0xD4D4,0xD4B3,0xB4B3,0xB483,0xC4B3,0xD4B3,0xB3D4,0xC3C3,0xD493,0xC493,0xC483,0x9383,0x93C4,0x93C4,0x93C4,0x93B4,0x93C4,0x93D4,0xD4C4,0xC493,0xB3E4,0xD4D4,0xC3D4,0xC3D4,0x7393,0x8352,0x5273,0x5273,0xB483,0x7383,0x8373,0x7363,0x5273,0x4252,0x5252,0x7352,0x7383,0x8383,0x7383,0x9383,0x93B4,0x83C4,0x7383,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4246,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646
.word 0x8383,0x7363,0x93C4,0xC493,0x8352,0x8383,0x5252,0x5252,0x7383,0x5273,0x5252,0x5252,0x5273,0x5252,0x4452,0x4246,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4646,0x7F46,0x7F41,0x8F42,0x7F41,0x7F7F,0x7341,0x52AF,0x428F,0x428F,0x8F8F,0x7F42,0x8F42,0x427F,0x429F,0x529F,0x428F,0x628F,0xD452,0x7F41,0x7F6F,0x6F42,0x7F41,0xD452,0xBF72,0x62D4,0xD4E4,0xD4C3,0xE4E4,0xE4E4,0xE4C3,0xE4E4,0xE4E4,0xF4D3,0xE4E4,0xE4E4,0xE4D3,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4C3,0xD4D4,0xD4D4,0x9F62,0x8F62,0x427F,0x6F6F,0x427F,0x6F7F,0x6F6F,0x6F42,0x426F,0x936F,0x73C4,0x7363,0x7383,0x5273,0x7373,0x7363,0x7352,0x5273,0x8352,0x7373,0x5252,0x7373,0x5273,0x7352,0x7352,0x7373,0x6373,0xB483,0xC493,0xB483,0xC483,0xB493,0xB493,0xB3D4,0xD4D4,0xD4B3,0xD493,0x83D4,0xE4B3,0xD4D4,0xD4B2,0xC4C3,0xC4C3,0xC4C3,0xD4C3,0xE493,0xE4C3,0xD4C3,0xD493,0x93D4,0xD4C3,0xD493,0x6393,0x8373,0x7373,0x8352,0x7383,0xB483,0x6373,0x7362,0x7383,0x7352,0x7352,0x7373,0x7363,0x6363,0x7373,0x73B4,0x7373,0x7383,0x7383,0x4352,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x5F46,0x4246,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646
.word 0x6352,0x5273,0xB483,0x8383,0x8363,0x7373,0x8373,0x6383,0x6352,0x5252,0x5252,0x5252,0x5252,0x4642,0x6F46,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x425F,0x417F,0x7F8F,0x427F,0x7F41,0x8F52,0x8F52,0x9F52,0x8F42,0x7F7F,0x417F,0x417F,0x417F,0x8F42,0x4241,0x418F,0xD542,0x52AF,0x7F8F,0x427F,0x7F6F,0x426F,0xD4D4,0x83D5,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xF4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xD3E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x62D5,0x41D5,0x7F7F,0x427F,0x7F7F,0x7F7F,0x417F,0x7F6F,0x6F6F,0x836F,0x7383,0x7383,0x5283,0x7352,0x7383,0x8373,0x8363,0x5252,0x5252,0x7373,0x5252,0x7352,0x7352,0x5252,0x7373,0x7383,0x7373,0x7373,0x7373,0x7373,0x7373,0x7373,0x8383,0xD4D4,0xE4B3,0x93C4,0xC3E4,0xC3D4,0x93C4,0xB3D4,0xB3C4,0xD4D4,0x93D4,0xD4D4,0x93D4,0xC4D4,0xD493,0xC493,0xC3D4,0x93D4,0x83D4,0xC3B4,0x5273,0x7352,0x5273,0xC473,0x8373,0x7373,0x5252,0x7373,0x5273,0x8373,0xD493,0x7383,0x4246,0x4246,0x4646,0x4642,0x4246,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4642,0x465F,0x465F,0x464F,0x4F46,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x5F46,0x4646
.word 0xD493,0x8383,0x5263,0x5273,0x5263,0x5273,0x7352,0x5273,0x6273,0x5252,0x5252,0x5252,0x4642,0x4642,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4642,0x4646,0x7F41,0x427F,0x4241,0x7F7F,0x427F,0x529F,0x9F62,0x8F52,0x7F41,0x4142,0x427F,0x7F7F,0x8F42,0x8F41,0x8F42,0x8F42,0x5241,0x529F,0x6F8F,0x6F6F,0x7F6F,0x527F,0xC4BF,0xD472,0xD4D4,0xD4D4,0xE4D4,0xC3E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xD3E4,0xE4E4,0xE4E4,0xE4D4,0xE4C3,0xE4D4,0xD4C3,0xD4D4,0xBF72,0x5262,0x7F52,0x7F6F,0x7F7F,0x6F42,0x6F6F,0x6F7F,0x7F6F,0x6341,0x5273,0x5273,0x7373,0x5252,0x8373,0x7383,0x8373,0x7383,0x5252,0x5252,0x8383,0x7373,0x7373,0x8373,0x7383,0x8383,0x5283,0x7373,0x7352,0x8383,0x8383,0xC483,0xB483,0xB493,0xD493,0xC4B3,0xD4C3,0xB3D4,0xC3D4,0x93D4,0xD4D4,0xD4B3,0xC493,0xC3C3,0xC3E4,0xC3E4,0xC3E4,0xC3E4,0xD4D4,0xB4B3,0x8373,0x73D4,0x6352,0x8373,0x5273,0x8373,0xB483,0x5252,0x5252,0x7383,0x5273,0xB483,0xC493,0x73B2,0x4646,0x4646,0x465F,0x4646,0x465F,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4246
.word 0x8383,0x73B4,0x8373,0x8383,0x7373,0x8373,0x6383,0x7352,0x5252,0x5252,0x5252,0x4652,0x4642,0x465F,0x5F46,0x4646,0x4646,0x4642,0x465F,0x4646,0x4646,0x4446,0x5F46,0x4646,0x465F,0x416F,0x417F,0x7F7F,0x7F41,0x5241,0x629F,0x429F,0x428F,0x7F8F,0x7F7F,0x7F41,0x7F7F,0x8F41,0x8F42,0x8F41,0x428F,0x51D5,0x628F,0x6F7F,0x7F7F,0x6F7F,0x6241,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE3E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D3,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xAFD5,0x4152,0x7F7F,0x6F7F,0x416F,0x6F6F,0x7F6F,0x426F,0x736F,0x7373,0x5252,0x5242,0x5252,0x7352,0x73C4,0x7383,0xB483,0x7383,0x5252,0x5252,0x5252,0x7352,0x5273,0x6373,0x7373,0x6373,0x7352,0x7373,0x7363,0x7363,0x5273,0x8373,0xD493,0x93D4,0xD4C4,0xB493,0xD483,0xD4D4,0xD4C3,0xD4C3,0xB3D4,0x83C4,0x83C4,0x83B4,0x83B4,0x83B4,0x73B4,0x7373,0xE483,0xB493,0x5252,0x5273,0xC452,0x5283,0x8373,0x8373,0x7373,0x8373,0x7373,0x7352,0xC483,0xB493,0x83D4,0x4642,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4446,0x5242,0x5252,0x5252,0x4252,0x5252,0x5252,0x5252,0x5252,0x5252,0x4642,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4446,0x4646
.word 0x7373,0x5273,0x9383,0xD4C4,0x5283,0xB483,0x7383,0x5252,0x6352,0x5252,0x4452,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x4646,0x4646,0x5F46,0x4142,0x427F,0x7F7F,0x427F,0x417F,0xAF52,0x7F42,0x8F52,0x4142,0x7F7F,0x427F,0x427F,0x427F,0x7F7F,0x7F42,0x4141,0xAF62,0x9F42,0x7F6F,0x6F6F,0x426F,0xD541,0xD572,0xD472,0xD4D4,0xD4D4,0xD4C3,0xD4E4,0xE4E4,0xE4C3,0xD3D4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x4162,0x7F9F,0x4142,0x416F,0x7F6F,0x6F42,0x6F7F,0x7F41,0x8342,0x7373,0xD483,0x8393,0x7383,0x5252,0x8373,0x7373,0x7373,0x8373,0x7383,0x4273,0x5252,0x7373,0x8373,0x93C4,0x8383,0x7352,0x8383,0x8373,0x8383,0x8383,0xB483,0xB493,0x7363,0x8383,0xC493,0xC3C3,0xB4B3,0xD493,0xB3D4,0x93C4,0x93C4,0x93D4,0x83B4,0x7373,0x7373,0x8373,0x8383,0xC493,0x7393,0x5263,0x5252,0x5252,0x8373,0x52B4,0x8363,0x83B4,0x8383,0x73C4,0x5273,0x8352,0x92C4,0x93B4,0xB493,0x6383,0x4646,0x4246,0x5F46,0x4646,0x4642,0x4246,0x6344,0x5252,0x7373,0x8383,0x9393,0xD4C3,0x9393,0x93B3,0xB393,0xC3B3,0xB393,0x9393,0x7373,0x6273,0x5252,0x7452,0x4446,0x4646,0x4646,0x465F
.word 0x93D4,0x8393,0xB473,0xB493,0x6383,0x7373,0x5252,0x5252,0x5273,0x4252,0x6F46,0x4646,0x4646,0x4246,0x5F46,0x4646,0x5F46,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4642,0x7F46,0x7F7F,0x7F41,0x7F41,0x7F41,0x5241,0xC4AF,0x527F,0x429F,0x7F7F,0x427F,0x7F7F,0x417F,0x418F,0x417F,0x7F7F,0x9F42,0x5252,0x428F,0x6F7F,0x6F6F,0x7F6F,0xD562,0xC4BF,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xD472,0x52D5,0x7F62,0x7F41,0x6F7F,0x7F7F,0x7F6F,0x426F,0x7F7F,0x7373,0x7373,0x8373,0x83C4,0x93B4,0x73C4,0x5252,0x9373,0x7383,0x6373,0x7373,0x7383,0x7352,0x8383,0x5273,0x7352,0x7373,0x8373,0x6373,0x5273,0x6373,0x7373,0x9383,0x93C4,0xC493,0x7393,0x7373,0x8352,0x83B4,0x7373,0x7373,0xB483,0xB493,0xC483,0x8383,0x9383,0xC3C4,0x93C4,0x83B4,0x5263,0x5252,0xC493,0xC493,0x5273,0x7373,0x8383,0x5263,0x7373,0x7383,0x5263,0x6352,0xC483,0xC493,0xB493,0x9383,0x83C4,0x5273,0x4646,0x4646,0x5242,0x8363,0x8383,0xC393,0xB393,0xD493,0xB493,0xB483,0x8383,0x6373,0x6373,0x7373,0x6363,0x7373,0x7363,0x8383,0x8383,0x8383,0x8383,0x9393,0x7373,0x5252,0x4663
.word 0xB483,0xB483,0x7352,0x7352,0x5252,0x7352,0x5252,0x6352,0x4252,0x4252,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x424F,0x7F41,0x7F42,0x7F7F,0x427F,0x427F,0xAF62,0x527F,0x8F52,0x7F41,0x7F7F,0x6F7F,0x7F7F,0x7F42,0x7F7F,0x8F42,0x6241,0x429F,0x418F,0x6F6F,0x427F,0x416F,0x62C4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xD3D4,0xE4E4,0xD4E4,0xD4E4,0xE4C3,0xE4D4,0xD4D4,0xAFD4,0x8F62,0x8F42,0x7F42,0x6F42,0x6F7F,0x6F7F,0x6F41,0x7363,0x7373,0x8373,0x9383,0x93C4,0x83D4,0x5273,0x5242,0x9373,0x7373,0x8373,0x5263,0x7352,0x7373,0x8373,0x5273,0x5273,0x5252,0x5273,0x5273,0x8373,0x7383,0x83B4,0xB483,0x8383,0x83B4,0x93C4,0x83D4,0x7383,0x8383,0xB483,0x7383,0x7373,0x7373,0xB483,0xC483,0xC483,0x7373,0x5252,0x4641,0x5244,0x93E4,0x93B4,0x5273,0x7373,0x8373,0x8383,0x5263,0x5252,0x7352,0x8383,0x8383,0x83B4,0x8383,0x83C4,0x7383,0x7383,0x5252,0x7352,0x8373,0xB393,0x93D4,0x83C4,0x73B4,0x7373,0x5252,0x5252,0x6352,0x4444,0x4246,0x4246,0x4246,0x4246,0x4646,0x5242,0x6352,0x7352,0x7373,0x7373,0x8383,0x8383,0x7383
.word 0x5252,0x5263,0x8373,0x93C4,0x93C4,0x7363,0x6352,0x6362,0x4152,0x4646,0x4646,0x4642,0x4646,0x4646,0x465F,0x5F46,0x4446,0x465F,0x4246,0x4646,0x465F,0x4646,0x4646,0x5F46,0x7F42,0x417F,0x417F,0x427F,0x7F7F,0x9F41,0x8F73,0x9F42,0x7F8F,0x7F7F,0x7F42,0x7F41,0x7F42,0x7F41,0x7F42,0x417F,0x9F42,0x8F7F,0x7F7F,0x7F6F,0x6F6F,0x527F,0xBFD4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xE4C3,0xE4D4,0xE4E4,0xD3E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4C3,0xE4D4,0xD4D4,0x62D4,0xC4D4,0x42AF,0x7F8F,0x7F7F,0x6F6F,0x6F7F,0x417F,0x427F,0x8393,0x9393,0x83D4,0x73B4,0xC483,0x8383,0x93B4,0x7383,0x5252,0x7352,0x5273,0x5273,0x5252,0x7373,0x7373,0x5263,0x8373,0xB483,0x7352,0x7373,0x5273,0x7352,0x7383,0x8383,0x5273,0x4646,0x4242,0x6352,0x8383,0xB473,0x7383,0x7383,0x8383,0x8383,0x7373,0x5273,0x4652,0x4642,0x4646,0x6F46,0x4446,0xD4D4,0xB493,0x4452,0x5273,0x8352,0x93B4,0x93E4,0x83C4,0x7383,0xB483,0x8383,0xC483,0x93B4,0x83B4,0x93B4,0x93D4,0xC493,0x9393,0xB493,0xB483,0x7352,0x7363,0x5252,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4646,0x465F,0x5F46,0x4246,0x4646,0x4642,0x4646,0x4246,0x4646,0x4446,0x5252,0x7363
.word 0x93D4,0x93C3,0x7352,0x8383,0xB483,0x5262,0x5252,0x5252,0x4652,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4642,0x4246,0x4646,0x416F,0x427F,0x7F7F,0x7F7F,0x7F41,0x6252,0x7FD5,0x4241,0x427F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x417F,0x417F,0x5242,0x7F7F,0x6F42,0x6F6F,0x7F6F,0x6241,0x62D5,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD3D4,0xE4D4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x62D4,0x51D5,0x7F7F,0x6F6F,0x6F7F,0x6F6F,0x7F42,0x7341,0xE4D4,0xF4D4,0xC483,0x5283,0x7373,0x8363,0xB483,0x6383,0x5273,0x7373,0x8383,0x5273,0x7352,0x5273,0x7373,0x8373,0x7373,0x7352,0x6273,0x7383,0x5273,0x8363,0xE4C3,0x83B3,0x4663,0x466F,0x4646,0x4246,0x4242,0x4252,0x4252,0x5252,0x6373,0x6373,0x5252,0x4646,0x4642,0x4646,0x465F,0x4646,0x5246,0x9393,0x5273,0x4644,0x4642,0x6352,0x8373,0xD4C4,0xD4B3,0xC493,0xD4C3,0xC3B3,0xC393,0xA3D4,0xC4C3,0x8383,0x7373,0x8373,0xB483,0x7363,0x5252,0x5252,0x4646,0x4246,0x6F46,0x4646,0x5F42,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x465F,0x465F,0x4246,0x4646
.word 0xC483,0xB483,0x5273,0x7373,0x7373,0x5252,0x5252,0x4252,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x5F46,0x4646,0x6F46,0x7F41,0x417F,0x7F7F,0x6F42,0x7F7F,0xAF52,0x7F52,0x418F,0x8F41,0x417F,0x7F42,0x6F6F,0x7F42,0x7F7F,0x7F6F,0x8F42,0x8F52,0x7F7F,0x6F7F,0x6F7F,0x426F,0xD452,0xC4BF,0x62D4,0xE4D4,0xD4D4,0xE4C3,0xE4E4,0xD4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xD3D4,0xE4E4,0xD4E4,0xD4C3,0xD4D4,0xBFD4,0x52C4,0x4152,0x427F,0x6F7F,0x6F7F,0x6F41,0xE47F,0xD4E4,0xD4D4,0x73E4,0x8363,0x5242,0x7383,0x8383,0x93C4,0x7383,0x7373,0x7373,0x5273,0x5252,0x7373,0x7383,0x8373,0x7373,0x7373,0x7373,0x5263,0x8352,0xD493,0x93D4,0x5273,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4242,0x8373,0x8383,0x5273,0x4641,0x4642,0x5F46,0x4646,0x4646,0x4642,0x8346,0x5273,0x4646,0x4646,0x5F46,0x4646,0x5242,0x7373,0x7352,0x6373,0x6373,0x6373,0x6373,0x6373,0x7373,0x7363,0x5263,0x5252,0x5252,0x4246,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x465F
.word 0xB483,0x8383,0x7352,0x6352,0x7373,0x5252,0x5252,0x4152,0x4646,0x4646,0x465F,0x4642,0x4246,0x4246,0x4646,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4642,0x4646,0x5F46,0x6F46,0x7F41,0x7F41,0x7F7F,0x417F,0x7F42,0x5252,0x419F,0x8F42,0x4142,0x7F7F,0x7F7F,0x6F7F,0x7F7F,0x6F42,0x6F7F,0x4241,0x7F9F,0x7F7F,0x426F,0x6F6F,0x417F,0xD462,0xD4C4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E3,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0x72D4,0xD4D4,0xBFD4,0x8F52,0x6F42,0x7F7F,0x7F6F,0x6F7F,0xF442,0xD4D4,0xD451,0x52F4,0x5252,0x5273,0xD452,0x8373,0x7373,0x7373,0x8352,0x8383,0x5273,0x5252,0x5283,0x8373,0x7352,0x8383,0x8383,0x8383,0x5252,0xD483,0x93D4,0x7383,0x4652,0x4642,0x4646,0x4646,0x4642,0x465F,0x4642,0x5242,0xE4C3,0x83D4,0x4273,0x4646,0x5F46,0x4646,0x4646,0x4642,0x5F46,0x5246,0x4643,0x5F46,0x4246,0x4646,0x4646,0x4646,0x4642,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0x4652,0x4642,0x4246,0x4646,0x4646,0x466F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4246,0x4646,0x465F,0x4246,0x4646,0x4642,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4642,0x4646,0x465F,0x4246,0x4646,0x4646
.word 0x5263,0x7373,0x5283,0x8373,0x7383,0x6352,0x5252,0x4652,0x4246,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x416F,0x4641,0x7F6F,0x7F42,0x7F7F,0x7F41,0x9F41,0x7F52,0x428F,0x7F8F,0x7F42,0x427F,0x7F6F,0x7F42,0x6F7F,0x7F7F,0x8F41,0x7F42,0x7F6F,0x7F6F,0x6F7F,0x527F,0xD4D5,0xD4BF,0xD4D4,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xD4D3,0xD4E4,0xE4D3,0xE4D4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0x83BF,0x62D4,0x9F62,0x7F6F,0x7F6F,0x426F,0x7F6F,0xE47F,0x6252,0xD49F,0x83E4,0x8383,0x6383,0x7352,0xB483,0x7352,0x7373,0x7373,0x83B4,0x7373,0x5252,0x6373,0x8352,0x6383,0x6373,0x8363,0x5273,0x6352,0x5252,0x5252,0x4352,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x8342,0x6383,0x4452,0x4246,0x5F46,0x4646,0x4646,0x4642,0x4F46,0x4646,0x4646,0x4652,0x4646,0x4646,0x4646,0x465F,0x4642,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x6F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4644,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4642
.word 0x8352,0xB493,0xB493,0x8373,0x73B4,0x5252,0x5252,0x4652,0x4646,0x465F,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4646,0x464F,0x4646,0x465F,0x4246,0x4646,0x4646,0x4642,0x417F,0x6F7F,0x7F42,0x7F7F,0x417F,0x427F,0x528F,0x7F52,0x8F41,0x7F42,0x7F7F,0x6F7F,0x6F7F,0x7F7F,0x6F7F,0x427F,0x427F,0x7F41,0x6F6F,0x6F6F,0x7F6F,0x6242,0xC4BF,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xD4E4,0xE4D3,0xD3E4,0xE4E4,0xC3E4,0xD4E4,0xD4D4,0xD4D4,0xD5D4,0x52AF,0x7F41,0x7F42,0x6F6F,0x7F7F,0x5241,0x629F,0xE462,0x93E4,0x83B4,0x83D4,0x5252,0x8352,0x7383,0x7373,0x7373,0x7373,0x7373,0x4273,0x7352,0x5273,0x6263,0x8373,0x7383,0x5273,0x8373,0x6383,0x7352,0x7383,0x4262,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x5252,0x4643,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4246,0x5263,0x4644,0x4642,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x6F46,0x4646,0x466F,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4642,0x4246,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646
.word 0x7352,0x8383,0x83B4,0x7352,0x7352,0x5273,0x5252,0x4642,0x466F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x465F,0x4646,0x416F,0x7F7F,0x7F7F,0x7F41,0x7F7F,0x8F42,0x8F42,0x7F41,0x8F42,0x7F7F,0x7F41,0x6F7F,0x6F42,0x6F41,0x6F42,0x6F6F,0x4141,0x7F8F,0x6F7F,0x426F,0x7F6F,0xD562,0xD472,0xD4D4,0xD472,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4C3,0xE4D4,0xE4D4,0xD4D4,0xE4D3,0xD4E4,0xE4E4,0xE4C3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x62D4,0x62D4,0x9FC4,0x8F52,0x7F6F,0x7F6F,0x6F41,0x5252,0xD5AF,0xE462,0x83E4,0x8373,0x83C4,0x73C4,0x6352,0x8383,0x7383,0x7352,0x7373,0x7383,0x8373,0x5252,0x5252,0x5252,0x5273,0x7363,0x8373,0xB483,0x5273,0x5273,0x8373,0x82C4,0x7383,0x5262,0x4643,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4246,0x5F46,0x4646,0x4642,0x465F,0x4646,0x4646,0x4246,0x7342,0x7373,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F,0x4246,0x4F46,0x4646,0x5F46,0x4246,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4642,0x465F,0x4646,0x4646,0x4646
.word 0x6383,0x7383,0x8383,0x8363,0x5283,0x5273,0x5252,0x4646,0x4646,0x4646,0x4644,0x465F,0x4642,0x5F46,0x4646,0x465F,0x4642,0x4646,0x5F46,0x4646,0x465F,0x4646,0x5F46,0x7F42,0x7F42,0x7F41,0x427F,0x416F,0x527F,0x529F,0x7F8F,0x4241,0x7F7F,0x7F7F,0x6F42,0x6F7F,0x7F7F,0x7F6F,0x7F7F,0x8F42,0x6F42,0x6F42,0x6F7F,0x7F6F,0xD462,0xC4AF,0xD4D4,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xD4C3,0xD4D4,0xC3D4,0xE4D4,0xC3D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D3,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xC3D4,0xD4D4,0xD4D4,0xC4BF,0x83BF,0x429F,0x7F7F,0x6F6F,0x4142,0x9F52,0xD4C4,0xC4AF,0x73F4,0x7373,0x8373,0x8383,0x5273,0x6352,0xC483,0x8393,0x8373,0x8352,0xD483,0x7383,0x5273,0x4252,0x7352,0x7383,0xB483,0x7373,0x7373,0x5273,0x7352,0x8352,0x9383,0x93A3,0x7383,0x6373,0x4243,0x4242,0x4242,0x4442,0x6342,0x8373,0x7383,0x7373,0x73A4,0x7362,0x4642,0x4646,0x5252,0x7373,0x4662,0x465F,0x4646,0x465F,0x4246,0x4646,0x4646,0x4644,0x4642,0x4246,0x5F46,0x4646,0x4646,0x4246,0x4646,0x4246,0x4252,0x5F46,0x4446,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x4646,0x4646,0x4642,0x4246,0x5F46
.word 0x8383,0x6373,0x7352,0x7352,0x5273,0x5252,0x5252,0x4246,0x4646,0x4642,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4146,0x7F7F,0x6F42,0x427F,0x7F7F,0x7F46,0x4241,0x8F52,0x7F52,0x418F,0x7F7F,0x6F42,0x6F7F,0x6F41,0x427F,0x7F6F,0x416F,0x527F,0x6F7F,0x6F7F,0x6F7F,0x527F,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xD4C3,0x5262,0x429F,0x529F,0xC4BF,0xD562,0xD562,0x5262,0x7F7F,0x7F42,0x7F7F,0xD562,0xD462,0xE462,0x93C4,0x8383,0x7373,0xB483,0x8383,0x6383,0x5252,0x5263,0x8373,0x73B4,0x5273,0xD483,0xC4B3,0x5283,0x5252,0x8373,0x7373,0x7373,0x5273,0x5252,0x7373,0x7373,0x7363,0x7352,0x5263,0x5273,0x7352,0x7373,0x7373,0x7373,0x7373,0x6373,0x7373,0x7373,0x5262,0x5263,0x9393,0x7362,0x5252,0x5252,0x5252,0x4646,0x465F,0x4646,0x4F46,0x4646,0x635F,0x5273,0x4646,0x464F,0x4646,0x4646,0x4246,0x6342,0x7373,0xA393,0x6383,0x4646,0x4646,0x465F,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646
.word 0xB483,0x7352,0x8383,0x7373,0x5273,0x5252,0x5252,0x4646,0x465F,0x4646,0x4646,0x4246,0x4646,0x4246,0x4646,0x4642,0x4646,0x4646,0x4446,0x5F46,0x4446,0x4646,0x6F46,0x7F41,0x7F46,0x7F7F,0x6F41,0x7F42,0x9F42,0x8F52,0x7F42,0x8F42,0x7F7F,0x7F41,0x6F7F,0x7F7F,0x7F7F,0x416F,0x416F,0x527F,0x6F7F,0x6F7F,0x6F7F,0x5241,0xC4BF,0x83BF,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xD4E4,0x62D4,0x9F52,0x6F8F,0x6F6F,0x7F41,0x5241,0xC4AF,0x62BF,0x52AF,0x7F7F,0x417F,0x527F,0xD4D4,0xE4AF,0xE4D4,0x5283,0xB473,0xC4B3,0x9383,0xD4D4,0xC4B3,0x5283,0x8373,0x93C4,0x7373,0x8373,0x8373,0x93C4,0x52C4,0x7352,0x7373,0x8373,0x7352,0x8383,0x7383,0x6352,0x7352,0x7352,0x7352,0x6383,0x7383,0x7373,0x7373,0x7383,0x7373,0x8373,0x7383,0x7352,0x7373,0x7373,0x5273,0x5273,0x8373,0x83A3,0x5273,0x4252,0x4646,0x4646,0x4644,0x4646,0x4646,0x8363,0x9382,0x4643,0x7363,0x8352,0x72B3,0xA372,0xA393,0x8383,0x8383,0x5283,0x4246,0x4646,0x4646,0x4246,0x4246,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4246,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x465F
.word 0x7373,0x8373,0x8373,0x73B4,0x5273,0x5252,0x5252,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4642,0x6F46,0x7F41,0x6F46,0x7F41,0x7F7F,0x7F46,0x4241,0x429F,0x7F8F,0x4152,0x7F42,0x7F6F,0x6F42,0x6F41,0x6F42,0x7F7F,0x7F7F,0x7F42,0x6F7F,0x7F6F,0x426F,0x6F7F,0x6F6F,0x417F,0x529F,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4E4,0x52D4,0x419F,0x6F41,0x7F6F,0x426F,0xD4C4,0xE5D4,0xD4D4,0xD4D4,0x62D4,0x6262,0x52AF,0x7F7F,0x7F7F,0x6262,0xD4BF,0xE483,0xD4D4,0x93C4,0x73C4,0x5252,0x4252,0x6373,0x8383,0xC483,0xC493,0x7383,0x8373,0x8373,0x93B4,0xC483,0x5283,0x5252,0x7383,0x7373,0x5273,0x7373,0x5273,0x5252,0x7352,0x8373,0x83C4,0x7383,0x8383,0x9383,0x8383,0x7373,0x7373,0x8383,0x83B4,0x7383,0x7373,0x7383,0x7383,0x7373,0x5273,0x5273,0x7373,0x9383,0x6283,0x5273,0x4642,0x4642,0x7342,0x8373,0x73B4,0x9383,0x7352,0x5263,0x5273,0x5252,0x5252,0x5252,0x4643,0x4246,0x4646,0x5F46,0x4246,0x4646,0x4646,0x464F,0x4646,0x465F,0x5F46,0x4646,0x4246,0x464F,0x4644,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646
.word 0x93E4,0x5273,0x7373,0x8373,0x5283,0x5252,0x5252,0x4246,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4446,0x5F46,0x4646,0x425F,0x7F7F,0x6F42,0x417F,0x6F6F,0x425F,0x528F,0x419F,0x7F7F,0x9F52,0x7F7F,0x7F42,0x6F7F,0x6F7F,0x6F7F,0x426F,0x416F,0x417F,0x6F6F,0x427F,0x7F6F,0x62C4,0x7F62,0x6F7F,0x6F6F,0x6F7F,0x8F7F,0xD452,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0x8F52,0x7F6F,0x7F41,0x7F7F,0x627F,0xE4D4,0xE4E4,0xE4E5,0xE5E5,0xE5D4,0xD4D4,0x52AF,0x52D5,0x427F,0x4241,0xC4AF,0xD462,0xD4AF,0xE4E4,0xC493,0xC383,0xC3D4,0x7373,0x5252,0xD473,0x8383,0x8383,0x7373,0x7373,0x8373,0xB483,0x8373,0x7373,0x5242,0x8373,0x7373,0x7352,0x5273,0x8352,0x7373,0x5283,0x7352,0x8373,0x7373,0xB483,0x8373,0x7363,0x8383,0x8383,0x7373,0x5273,0x8373,0x8383,0xD493,0xC493,0x5283,0x5273,0x7373,0x5273,0x6263,0x7363,0x9383,0x83B3,0x8393,0x7373,0x9373,0x7393,0x5263,0x5252,0x5273,0x4452,0x4646,0x4646,0x4646,0x4642,0x465F,0x5F46,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4642,0x465F,0x5F46,0x4246,0x4646,0x464F,0x4646
.word 0xB483,0x7352,0x7352,0x8383,0x52B4,0x5252,0x5252,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4642,0x4246,0x5F46,0x4646,0x4246,0x4646,0x465F,0x4646,0x4646,0x7F6F,0x7F41,0x425F,0x7F7F,0x4642,0x7F46,0x4241,0x8F52,0x7F7F,0x4152,0x7F7F,0x7F7F,0x6F7F,0x6F42,0x6F41,0x6F7F,0x7F7F,0x7F42,0x7F6F,0x7F6F,0x416F,0xAF52,0x5252,0x62D4,0x8F52,0x6F6F,0x6F7F,0x7F7F,0x6F7F,0x6251,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0x4151,0x6F6F,0x6F6F,0x6F7F,0x426F,0x8352,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xAF83,0x8F62,0xAF52,0x9F62,0x417F,0x628F,0xD4D5,0xE4D4,0xE462,0xD4D4,0x9383,0x83C4,0x8363,0x7383,0x7383,0x5252,0x7363,0x9373,0x93C3,0x83C4,0x7373,0x8383,0x7383,0x8363,0x4273,0x7352,0x8383,0x7373,0x5273,0x8373,0x8363,0x7383,0x5273,0x7352,0x7363,0xC493,0x8373,0x7373,0xB483,0xB493,0x8373,0x7373,0x7373,0xB483,0x7373,0x7373,0x7383,0x7373,0x8383,0x7373,0x7373,0x5283,0x6363,0x7373,0x7383,0x5273,0x8373,0x93B4,0xC493,0x83B3,0x7383,0x7383,0x4652,0x466F,0x4642,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4644
.word 0x7373,0x9373,0x73C4,0x7352,0x5273,0x5263,0x5252,0x5F46,0x4646,0x4642,0x5F46,0x4646,0x4644,0x4F46,0x4646,0x4646,0x4246,0x4646,0x465F,0x4646,0x5F46,0x4446,0x416F,0x426F,0x7F7F,0x7F41,0x466F,0x7F42,0x8F41,0x8F42,0x7F7F,0x9F52,0x4142,0x7F7F,0x6F42,0x6F7F,0x7F7F,0x7F6F,0x4142,0x7F7F,0x426F,0x6F6F,0x7F6F,0x6F5F,0x5F5F,0x6F5F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6241,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8F41,0x7F6F,0x7F7F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x416F,0x5241,0x529F,0x529F,0x7F7F,0xC442,0xD562,0xD441,0xD562,0xD472,0xE4D4,0xE4C4,0x83E4,0x93B4,0x93C4,0x83C4,0x73B4,0x7373,0x8383,0x5273,0x5252,0x7363,0x8383,0x7373,0x7373,0x7363,0xC493,0x5283,0x5252,0x8363,0x7373,0x7383,0x7373,0x7373,0x5263,0x8383,0x8373,0x7373,0x7373,0x7383,0x7393,0x8373,0x73C4,0x7373,0x7373,0x7373,0x7383,0x7373,0x8383,0xC483,0x8383,0x73B4,0x8373,0x8352,0x7383,0x6273,0x7352,0x5273,0x8373,0x7373,0x6383,0x8373,0x83C4,0x83B4,0x83C4,0x4652,0x4646,0x4646,0x465F,0x4646,0x465F,0x4246,0x4646,0x4646,0x4642,0x4F46,0x4646,0x464F,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4642,0x4F46,0x4646,0x465F,0x4446,0x4246,0x4646,0x4646,0x4646,0x465F
.word 0x5252,0x7383,0x7383,0x7363,0x5252,0x5262,0x5252,0x4643,0x4646,0x4F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4446,0x4646,0x4246,0x7F6F,0x7F41,0x427F,0x7F7F,0x427F,0x7F46,0x8F42,0x4241,0x7F7F,0x4152,0x7F7F,0x7F42,0x6F7F,0x7F7F,0x6F7F,0x6F42,0x416F,0x7F7F,0x7F6F,0x6F6F,0x5F7F,0x6F5F,0x5F5F,0x6F6F,0x6F6F,0x6F5F,0x6F6F,0x6F6F,0x6F6F,0x416F,0x8F6F,0xD4D4,0xD4D4,0xD4D4,0x72D4,0x62AF,0x428F,0x7F7F,0x6F6F,0x6F6F,0x5F6F,0x5F6F,0x5F6F,0x5F5F,0x5F6F,0x5F5F,0x5F6F,0x6F6F,0x6F6F,0x526F,0xD4AF,0x62BF,0xD452,0xD4AF,0xD4D4,0xE4E4,0xE462,0x73D4,0x6373,0x6373,0x8383,0x9383,0x93C4,0x83B4,0x8373,0x7373,0x7373,0x8373,0x7373,0x5273,0xC483,0x8383,0xB483,0x5273,0x5252,0x6352,0x7373,0x7373,0x7352,0x5252,0x5252,0x4252,0x5252,0x5252,0x5252,0x5252,0x7352,0x6263,0x7352,0x5273,0x5273,0x5273,0x5252,0x5252,0x5273,0x7373,0x8373,0xB493,0xB483,0x5273,0x7373,0x8383,0x83B4,0x7383,0x7373,0x7373,0x7373,0x8383,0xB483,0xB493,0x4652,0x4246,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x464F,0x4246,0x4646,0x4646
.word 0x8352,0x6352,0x7352,0x9383,0x5273,0x5263,0x7352,0x4652,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4246,0x5F46,0x4246,0x464F,0x4646,0x4646,0x4646,0x4642,0x4F46,0x7F42,0x7F7F,0x7F7F,0x427F,0x466F,0x7F46,0x4241,0x418F,0x7F7F,0x8F52,0x7F42,0x7F7F,0x7F7F,0x6F6F,0x7F42,0x7F6F,0x427F,0x7F7F,0x6F6F,0x6F41,0x5F6F,0x6F6F,0x5F7F,0x4F5F,0x4F4F,0x6F6F,0x6F5F,0x6F6F,0x6F6F,0x6F7F,0x7F6F,0xD462,0xD4D4,0xD4D4,0xD4D4,0xAF62,0x8F42,0x6F6F,0x6F6F,0x5F6F,0x5F6F,0x5F6F,0xFC5F,0x5F5F,0x5F5F,0x5F6F,0x5F5F,0x5F4F,0x7F5F,0xC441,0xC4D4,0xAFC4,0xD452,0x62D4,0xE4D4,0xD4E4,0xE4D4,0x4662,0x5241,0x7352,0x6352,0x5263,0x5273,0x8383,0xC3C4,0xD4C3,0xE493,0x82D4,0x83D4,0x8352,0x93D4,0x73B4,0x7373,0x8383,0x5273,0x5252,0x5252,0x5252,0x5252,0x5252,0x4642,0x4246,0x4646,0x4646,0x4146,0x5252,0x5252,0x5273,0x7352,0x5252,0x5252,0x7352,0x5252,0x6352,0x5273,0x5252,0x7373,0x8383,0x8383,0x8383,0xB483,0x7352,0x7352,0x8373,0x8373,0x7373,0x8373,0x83D4,0x7373,0x5263,0x4652,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x4246,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4642
.word 0x83C4,0x52C4,0x7373,0x8373,0x73C4,0x5273,0x5252,0x4652,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F42,0x4646,0x5F46,0x7F46,0x417F,0x467F,0x416F,0x7F7F,0x4F42,0x4146,0x417F,0x8F42,0x7F7F,0x5252,0x417F,0x7F7F,0x427F,0x7F6F,0x7F7F,0x7F6F,0x7F41,0x6F41,0x6F7F,0x427F,0x6F6F,0x9F7F,0x6EBF,0x406E,0x7E6E,0x7F9D,0x7FBF,0x7F5F,0x6F41,0x6F7F,0x7F6F,0x6241,0xD4D4,0xD4D4,0xD4D4,0x9F62,0x6F6F,0x426F,0x6F6F,0x6F5F,0xCF52,0x7E7E,0x4F7F,0x6E4C,0xBFDC,0x7F9F,0x4F6F,0x6F4F,0x426F,0xD48F,0xD4D4,0x62D4,0xAFD4,0xD462,0xD4D4,0xE4D4,0xD4D4,0x4646,0x4246,0x4646,0x4642,0x4146,0x5252,0x7352,0x5263,0x5252,0x5252,0x5273,0x6352,0xC483,0x7383,0x7383,0x8383,0x83C4,0xC483,0x5283,0x8352,0x8383,0x6383,0x5283,0x4246,0x4646,0x465F,0x4642,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x5241,0x5263,0x7363,0x7373,0x83C4,0x93B4,0x83B4,0x83C4,0x83B4,0xC483,0x8383,0xC493,0x8373,0x7373,0x7383,0x4652,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4642,0x5F46,0x4246,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x4246,0x464F,0x4646,0x4646
.word 0xC483,0x7373,0x7383,0x7373,0x5283,0x6352,0x5252,0x4652,0x4642,0x5F46,0x4246,0x4646,0x4646,0x465F,0x4246,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x7F46,0x427F,0x427F,0x425F,0x6F7F,0x4646,0x7F46,0x8F42,0x4241,0x417F,0x629F,0x428F,0x7F7F,0x6F41,0x7F7F,0x416F,0x6F6F,0x7F41,0x6F7F,0x6F6F,0x7F7F,0x6F6F,0xCF41,0x51CF,0x8E6E,0xF0F0,0xAF7E,0xBFEF,0x6F7F,0x7F7F,0x7F7F,0x6F6F,0xD442,0xD4D4,0xE4D4,0xD4D4,0x7FD4,0x7F6F,0x6F7F,0x5F5F,0x6242,0xFFDF,0x8E6E,0xBDF0,0xFC9D,0xEF6E,0x9FCF,0x6F7F,0x4F4F,0x627F,0xD462,0xD4D4,0xAFD4,0xD4C4,0xE4E4,0xD4D4,0xE4E4,0x5283,0x4246,0x4646,0x465F,0x4646,0x466F,0x4646,0x4246,0x4246,0x5252,0x5252,0x7342,0x8373,0x83C4,0x7383,0x7373,0x8373,0x93B4,0x73B4,0x5252,0x8363,0x73B4,0xC483,0x5273,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4246,0x5F46,0x5F46,0x4646,0x465F,0x465F,0x5F42,0x4646,0x465F,0x4646,0x4646,0x5252,0x5252,0x7363,0x8373,0xB483,0xB493,0x8393,0xE493,0x93B4,0x93B4,0x83B4,0x8373,0x5263,0x5273,0x4652,0x4642,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x465F,0x4646
.word 0x7373,0x8383,0xC483,0x5283,0x7352,0x5252,0x5252,0x4652,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x464F,0x5F46,0x4646,0x5F46,0x4646,0x4642,0x6F46,0x7F41,0x467F,0x6F46,0x7F7F,0x5F46,0x4146,0x428F,0x8F8F,0x7F7F,0x6252,0x7F8F,0x7F41,0x7F7F,0x426F,0x7F6F,0x6F42,0x427F,0x6F7F,0x6F42,0x416F,0x4F6F,0x7F5F,0x5241,0xF272,0xF2E2,0xF4E3,0xD4E4,0xB463,0x6F7F,0x4A8F,0x7F6F,0xD441,0xE4E4,0xE4E4,0xD4E4,0x8F62,0x4142,0x8F8F,0x427F,0xD462,0xDFDF,0xFEF0,0x6E9E,0xE06E,0x62CF,0x7F8F,0x4F5F,0x526F,0xD4D4,0xE4E4,0xE4E4,0x62D4,0xE4D4,0xD4F4,0x62E4,0xD4E4,0x4342,0x4642,0x4642,0x4244,0x4244,0x4244,0x4152,0x6342,0x7352,0x5273,0x8373,0x9383,0xB2C4,0xC4B3,0x8383,0x7373,0x7373,0x5252,0x5252,0x5252,0xD483,0x8373,0x7373,0x4283,0x4642,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4242,0x6252,0x9373,0x73C4,0x5252,0x7352,0x8373,0x8383,0x83C4,0x83B4,0x7383,0x7383,0x6373,0x7373,0x5273,0x5252,0x4646,0x4646,0x465F,0x4646,0x465F,0x4F46,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4642,0x4646,0x465F,0x4246,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4642
.word 0x7383,0x7383,0x9383,0x7373,0x7383,0x7352,0x5252,0x4252,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x7F46,0x4241,0x467F,0x6F46,0x6F41,0x4646,0x7F46,0x7F41,0x4252,0x417F,0xAF52,0x427F,0x7F7F,0x7F42,0x7F6F,0x6F6F,0x6F7F,0x7F41,0x6F7F,0x7F7F,0x4241,0x6F41,0x4F4F,0x5F4F,0x6F5F,0x418F,0x8F52,0x8F41,0x417F,0x6F7F,0x8F7F,0x7F7F,0xD442,0xE4E4,0xE4E4,0xD4E4,0x41D4,0x416F,0x7F8F,0x426F,0x8F42,0xAF52,0x7262,0x7272,0x5262,0x7F8F,0x4F5F,0x524F,0xBF62,0xE4D4,0xE4E4,0xD4E4,0x9FD4,0xE4D4,0x83E4,0xD4AF,0x73D4,0x5273,0x5273,0x5252,0x6352,0x7352,0x5263,0x8383,0x8383,0x8383,0x7383,0x4252,0x5252,0x5252,0x5263,0x4252,0x5252,0x5242,0x5273,0x7373,0x6352,0x8393,0x7373,0x8383,0x4273,0x4646,0x4646,0x465F,0x4646,0x4642,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x4646,0x4642,0x4246,0x6352,0x9383,0x83B4,0x8393,0x5263,0x5273,0x7352,0x6373,0x7373,0x7373,0x7373,0x6373,0x5262,0x5273,0x5252,0x5252,0x4646,0x465F,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4F46,0x4446,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4642,0x4646
.word 0x73D4,0x7363,0x7373,0x7352,0x7383,0x5273,0x5252,0x5252,0x4646,0x4642,0x4646,0x4246,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x5F46,0x5F46,0x4646,0x416F,0x416F,0x6F6F,0x4246,0x7F6F,0x446F,0x4146,0x427F,0x529F,0x7F7F,0x7362,0x7F8F,0x4152,0x416F,0x6F7F,0x6F41,0x7F41,0x417F,0x7F6F,0x6F41,0x417F,0x62D5,0x9FC4,0x8F62,0x6F41,0x6F6F,0x6F6F,0x6F6F,0x6F6F,0x4A6F,0x6F6F,0x7F7F,0xD441,0xE4D4,0xE4F3,0xE4E4,0x9F62,0x7F7F,0x4A7F,0x6F7F,0x4F5F,0x5F4F,0x4F4F,0x4F4F,0x7F5F,0x526F,0x5252,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0x62D4,0xD462,0xD4D4,0xD4E4,0x8373,0x83C4,0xB483,0x7383,0x7373,0x7373,0x7373,0x7363,0x8373,0x93C4,0x83B4,0x8393,0x7383,0x7352,0x7383,0x8383,0x8373,0x8383,0xB483,0x5283,0x9352,0x73B4,0x8383,0x73B4,0x4642,0x4642,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4642,0x4F46,0x4646,0x5F46,0x4646,0x4246,0x7352,0x5263,0xB483,0x7383,0xD4B4,0x7393,0x5252,0x6352,0x5262,0x5273,0x5273,0x5252,0x5273,0x7363,0x5252,0x5273,0x7373,0x4262,0x4643,0x5F46,0x4646,0x5F46,0x4446,0x4646,0x4642,0x4646,0x465F,0x4446,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x464F,0x4646,0x465F
.word 0xD493,0x7383,0x6352,0x7373,0x7373,0x5252,0x7373,0x6263,0x4642,0x4646,0x4246,0x4646,0x465F,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x416F,0x417F,0x465F,0x4646,0x416F,0x5F7F,0x7F46,0x417F,0x8F42,0x7F42,0xAF62,0x7F42,0x7F8F,0x6F41,0x7F7F,0x6F7F,0x7F7F,0x7F42,0x6F6F,0x7F42,0x427F,0xD4D4,0xD4D4,0x62C4,0x62D5,0x6262,0x62D5,0xC462,0x6262,0x7F8F,0x7F6F,0x7F41,0xD452,0xE4E4,0xE4F4,0xE4E4,0x52C4,0x7F7F,0x6F7F,0x5241,0x529F,0x9F62,0xAF62,0xAF52,0x6262,0xE552,0xD4D4,0xE4D4,0xF8D4,0xE4E4,0xD4E4,0xD4E4,0xAFD4,0xD462,0xE4E4,0x83D4,0x8383,0x83B4,0x8383,0x83B4,0x7383,0x8373,0x8373,0x8383,0x7373,0x6373,0x7373,0x6373,0x7352,0x8373,0x83B4,0x7373,0xB483,0xB483,0x8383,0x5273,0x83C4,0x7383,0xB483,0x5252,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x7363,0x7352,0x7373,0x5252,0x8352,0x9383,0x73B4,0x5273,0x5252,0x5252,0x6263,0x7363,0x5252,0x5252,0x5252,0x5252,0x7352,0x5273,0xC493,0x4252,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646
.word 0xC483,0x5283,0x8383,0x9383,0x8373,0x83C4,0x5252,0x5252,0x4663,0x4642,0x465F,0x4646,0x4646,0x4446,0x4646,0x464F,0x5F46,0x4246,0x4646,0x4644,0x4646,0x417F,0x427F,0x5F46,0x4646,0x425F,0x427F,0x416F,0x7F42,0x8F41,0x7F41,0x6262,0x417F,0x7F42,0x427F,0x7F7F,0x426F,0x7F6F,0x417F,0x7F6F,0x7F7F,0x7F41,0xE4C3,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4D4,0x62D4,0xD562,0xD451,0xE4D4,0xF4F3,0xE4E4,0xC4E4,0x6262,0x6252,0xD4D4,0xD4D4,0xD4D5,0xC4D4,0xF9D5,0xD4E5,0xF4D4,0xD4E4,0xF9E4,0xE4E4,0xE4E4,0xD4C3,0xD4D4,0x6283,0xD4D5,0xD4D4,0x4652,0x4646,0x4642,0x4146,0x5242,0x6352,0x7352,0x5252,0x5252,0x7373,0x7383,0x7373,0x8373,0x7383,0x8373,0x8383,0x8373,0x8383,0x7393,0x73C4,0x7352,0x8383,0x93C4,0x5273,0x4642,0x4642,0x465F,0x4246,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4246,0x5F46,0x4646,0xA452,0x5252,0x5252,0x4246,0x4246,0x8373,0x83D4,0x6263,0x7373,0x5273,0x6263,0x5263,0x5252,0x5252,0x5273,0x8352,0x7373,0x5273,0xD483,0x8373,0x8393,0x4252,0x4246,0x5F46,0x4646,0x4246,0x5F46,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4246,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4642
.word 0xB483,0x7373,0x8373,0x83B4,0x7373,0xB483,0x7352,0x5252,0x4352,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x416F,0x7F7F,0x4646,0x4642,0x5F46,0x7F7F,0x7F41,0x7F41,0x8F42,0x7F42,0xD552,0x8F7F,0x417F,0x7F7F,0x427F,0x6F6F,0x7F7F,0x7F41,0x7F6F,0x7F41,0x8F42,0xC3D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD3E4,0xD4E4,0x62D4,0xD4D5,0xE4D4,0xF4F3,0xD4E4,0xD4D4,0xD4D4,0xF4E5,0xE4F9,0xF9F9,0xD4D4,0xD4D4,0xE4C3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xD4E4,0xE4E4,0xD4D4,0x62BF,0xD4AF,0x5262,0x4646,0x4642,0x465F,0x4646,0x4642,0x4446,0x7352,0xC483,0xC493,0x83C3,0x83C4,0x8383,0xB483,0x7383,0x8373,0x73B4,0xB483,0xB483,0xC483,0x4262,0x8352,0x83C4,0x5283,0x4652,0x4642,0x4646,0x4646,0x4646,0x465F,0x4646,0x4642,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x4644,0x4644,0x4642,0x4646,0x5246,0x8383,0x73B4,0x7383,0x7352,0x5273,0x8373,0x7352,0x8383,0x7363,0x8373,0x8363,0x73C4,0x6383,0x7352,0x7383,0x8363,0x8383,0x4242,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4644,0x4646,0x4646,0x464F,0x4646,0x4642,0x4F46,0x4646,0x464F,0x4646,0x464F,0x4646,0x4646,0x464F
.word 0x5252,0x5273,0x7373,0x7373,0x7352,0x5252,0x5273,0x7352,0x5252,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4246,0x4646,0x4646,0x4646,0x4446,0x425F,0x7F7F,0x5F46,0x4646,0x4646,0x425F,0x7F7F,0x427F,0x5241,0x7F9F,0xAF62,0x7F42,0x7F41,0x7F42,0x6F41,0x417F,0x7F6F,0x427F,0x7F6F,0x6F42,0x4141,0xD4D4,0xC3E4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xE462,0xE3D4,0xE4F3,0xE4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xC3E4,0xF4F4,0xE4E4,0xE4D4,0xE4D4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x62C4,0x5243,0x4647,0x4642,0x4646,0x4646,0x465F,0x5242,0x8373,0x8383,0x93C4,0xB3C4,0x93C4,0xD483,0xB493,0xC493,0x8383,0xB483,0xB493,0x8383,0x7373,0x5273,0x5252,0x4252,0x4646,0x4646,0x4646,0x4646,0x465F,0x4246,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x7341,0x7373,0x8373,0x8373,0x6263,0x7352,0xC452,0x7373,0x9373,0x8363,0x9373,0x7383,0x7373,0x8383,0x7363,0x5273,0x4646,0x4242,0x5252,0x4242,0x4246,0x4646,0x465F,0x4246,0x4F46,0x4646,0x464F,0x4646,0x4642,0x465F,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646
.word 0x6393,0xE4C3,0x7383,0x8352,0x93C4,0x73C4,0x7383,0x5263,0x5273,0x4652,0x4646,0x4646,0x4644,0x4446,0x5F46,0x4646,0x4646,0x464F,0x4246,0x4646,0x4646,0x425F,0x7F7F,0x4646,0x4646,0x465F,0x6F46,0x417F,0x8F7F,0x5241,0x8F52,0x6252,0x7F8F,0x8F42,0x417F,0x7F6F,0x6F41,0x427F,0x7F7F,0x7F7F,0x7F7F,0x8F42,0xE4E4,0xE4E4,0xE4C3,0xC3E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4BF,0xE4D4,0xF4F7,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3D4,0xD4D4,0x43BF,0x4646,0x4646,0x4646,0x465F,0x4646,0x7343,0xB483,0x8383,0x73B4,0x7373,0x8383,0x7383,0x8373,0x7373,0x7373,0x8373,0x7373,0x8383,0x73B4,0x5273,0x7352,0x5283,0x4652,0x4642,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x4246,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4146,0x7363,0x5252,0x8383,0x8373,0x5242,0x5252,0x8373,0x7352,0x8373,0x7373,0x8373,0xD4B4,0x5273,0x8363,0x8383,0x5273,0x4642,0x4246,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x4642,0x4246,0x4646,0x4642,0x5F46,0x4646,0x4642
.word 0x6383,0x8383,0x83B4,0x8363,0x7383,0x7383,0x7373,0x7352,0x5252,0x5252,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4246,0x4646,0x7F46,0x426F,0x466F,0x4642,0x4646,0x425F,0x7F7F,0x427F,0x9F7F,0x8F42,0x9F52,0x7F52,0x417F,0x7F42,0x427F,0x7F7F,0x7F7F,0x7F41,0x416F,0x7F7F,0x4141,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD572,0xD483,0xF4D4,0xF4F3,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xC3E4,0xD4E4,0xD4D4,0xD4D4,0x4662,0x4246,0x5F46,0x4646,0x4646,0x8352,0x7383,0x5252,0x4252,0x4644,0x5241,0x5242,0x6352,0x5273,0x7373,0x7352,0x6373,0x5273,0x4252,0x7362,0x8383,0x6383,0x4652,0x4642,0x4646,0x465F,0x4646,0x4646,0x4642,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4642,0x4246,0x4646,0x5F46,0x4646,0x4246,0x5273,0x4146,0x8363,0x52B4,0x4643,0x4246,0x8352,0x9373,0x5273,0x5252,0x8373,0x8373,0x5283,0x5241,0x8363,0x83C3,0x4652,0x4646,0x465F,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x4446,0x4246,0x4646,0x5F46,0x4646,0x4642,0x4446,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7352,0x7373,0x5273,0x7373,0x7373,0x5263,0x7352,0x5252,0x5273,0x5273,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x465F,0x425F,0x6F46,0x4646,0x4646,0x4646,0x4146,0x7F7F,0x7F7F,0x4142,0x528F,0x528F,0x7F52,0x7F7F,0x417F,0x7F7F,0x7F7F,0x6F42,0x417F,0x7F6F,0x7F42,0x8F42,0xD4D4,0xE4D4,0xE4D4,0xC3E4,0xC3E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFD4,0xD4D4,0xF4D4,0xF4F4,0xD3E4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD3D4,0xE3E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xD4D4,0xC3D4,0xD4D4,0x62D4,0x46AF,0x4646,0x4646,0x4646,0x7342,0x7383,0x4252,0x4646,0x4642,0x465F,0x4646,0x4642,0x4246,0x4646,0x5252,0x7373,0x7352,0x7373,0x7373,0x8383,0x73B4,0x5252,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4246,0x4646,0x4646,0x4642,0x464F,0x4F46,0x4646,0x4646,0x4246,0x5246,0x4452,0x4646,0x8341,0x5283,0x4646,0x4646,0x7341,0x83C4,0x5273,0x4146,0x7352,0x7383,0x73D4,0x4646,0x5241,0xC473,0x4273,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4246,0x4646,0x4646,0x4646,0x4646,0x464F,0x5F46,0x4646,0x465F,0x4642,0x465F
.word 0xD473,0x7383,0x7373,0x5273,0x5273,0x9352,0x93C4,0x5283,0x7352,0x5252,0x4652,0x4F42,0x4646,0x465F,0x4F42,0x4446,0x4646,0x4646,0x464F,0x465F,0x4646,0x5F46,0x5F46,0x5F46,0x4646,0x465F,0x425F,0x427F,0x7F7F,0x417F,0x9F52,0x9F42,0x7F52,0x7F7F,0x7F42,0x7F41,0x7F7F,0x7F7F,0x7F41,0x7F6F,0x7F7F,0x4241,0xD462,0xD4D4,0xD4C3,0xD4E4,0xD4E4,0xE4E4,0xC3D4,0xC3D4,0x72D4,0x62D4,0xC4D4,0xD4D4,0xE3D4,0xF4F4,0xE4F4,0xD4D4,0xD4C3,0xE4C3,0xC3D4,0xD4E4,0xD4F4,0xE4E4,0xE4E3,0xE4E4,0xE4D3,0xE4F4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x4652,0x5F46,0x4646,0x4242,0x5273,0x4642,0x4246,0x4646,0x4646,0x4646,0x4642,0x4646,0x4242,0x8383,0x93D4,0xB3D4,0x93D4,0x8383,0x5273,0x5252,0x4646,0x4642,0x465F,0x4646,0x4646,0x4642,0x465F,0x4646,0x465F,0x4246,0x4246,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4341,0x4246,0x5F46,0x7342,0x5273,0x4246,0x5F46,0x5246,0x7383,0x4652,0x4646,0x4146,0x8363,0x7383,0x4646,0x4646,0x5241,0x7352,0x4642,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x8383,0xC483,0xD452,0xD4B2,0xC492,0x9373,0xC3C4,0x73C4,0x7352,0x5252,0x5252,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x417F,0x7F7F,0x417F,0x7F7F,0x4142,0x528F,0x42AF,0x416F,0x427F,0x427F,0x417F,0x7F7F,0x427F,0x416F,0x7F7F,0x8F42,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD5D4,0xD4BF,0xD462,0xD4BF,0xF4D4,0xF4F4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D3,0xF4E4,0xF4E4,0xE4E4,0xE3E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x62D4,0x4646,0x4646,0x4646,0x7342,0x4452,0x4246,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x4246,0x8352,0x83B4,0x7383,0x7373,0x5252,0x4252,0x4652,0x4246,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4644,0x4646,0x4646,0x5246,0x4273,0x4646,0x4646,0x4146,0x6373,0x4652,0x5F46,0x4646,0x6341,0x7383,0x4246,0x5F46,0x4646,0x5241,0x4641,0x4646,0x4644,0x464F,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x4246,0x4F46,0x4646,0x5F46,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4246,0x4646,0x4642,0x4246,0x4646,0x465F,0x4646
.word 0xB483,0x7393,0x8373,0xC493,0x9383,0x8373,0xC483,0x8393,0x5252,0x5283,0x5263,0x4652,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4246,0x4646,0x4446,0x4646,0x416F,0x417F,0x426F,0x7F7F,0x428F,0x8F52,0x4152,0x7F7F,0x7F7F,0x8F52,0x7F7F,0x427F,0x7F7F,0x7F6F,0x7F42,0x8F41,0xBFC4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xC3E4,0xD4D4,0xD4D4,0xD462,0xD483,0xD4D4,0xD4D4,0xE3D4,0xF3F4,0xE4F4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4C3,0xD4D4,0xAFD4,0x4246,0x4646,0x4242,0x4452,0x4646,0x4646,0x465F,0x4646,0x4646,0x4246,0x4646,0x7342,0x7373,0x5252,0x4642,0x4646,0x4646,0x4642,0x4246,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4642,0x465F,0x4642,0x465F,0x5F44,0x4646,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4642,0x4642,0x4646,0x465F,0x4646,0x5252,0x4646,0x4646,0x4246,0x4146,0x8352,0x4646,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4246,0x4646,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644
.word 0x7373,0x5273,0xC483,0x8383,0x7373,0x6352,0xB483,0x6373,0x7373,0x6352,0x7352,0x4252,0x4246,0x5F46,0x4646,0x5F46,0x4646,0x4642,0x4646,0x465F,0x4F42,0x4646,0x464F,0x4246,0x4646,0x465F,0x427F,0x7F7F,0x7F7F,0x7F7F,0x9F41,0x8F52,0x8F52,0x7F7F,0x417F,0x7F7F,0x7F42,0x6F41,0x417F,0x416F,0x7F7F,0x8F42,0xC4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0x62D4,0xBFD5,0xBFC4,0xD4D4,0xD4D4,0xF4E4,0xF3F3,0xE4F4,0xD4E4,0xD4C3,0xC3D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0x5262,0x4646,0x5F46,0x4346,0x4646,0x4246,0x4646,0x4646,0x4246,0x4646,0x4646,0x4246,0x7373,0x5252,0x4642,0x4246,0x6F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4642,0x4646,0x5F46,0x4246,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4642,0x4241,0x4246,0x4246,0x4646,0x4646,0x4241,0x4646,0x4642,0x4646,0x4642,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4644,0x4644,0x4246,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F
.word 0x8393,0x6352,0x7352,0x7363,0x7363,0x7383,0x5273,0x9383,0x83C4,0x5252,0x5273,0x5252,0x4642,0x4646,0x4246,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x464F,0x464F,0x4646,0x417F,0x417F,0x427F,0x7F7F,0x427F,0x628F,0x528F,0x427F,0x7F7F,0x417F,0x527F,0x417F,0x7F7F,0x416F,0x7F6F,0x4141,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD462,0xD462,0xD4D4,0xF4D4,0xF3F3,0xE4F7,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D3,0xD3E4,0xE4E4,0xE4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0x62D4,0x46AF,0x5F46,0x4646,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x465F,0x4246,0x5252,0x4652,0x4646,0x4446,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4642,0x4642,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x4642,0x4646,0x4644,0x4246,0x4646,0x4646,0x4246,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4646
.word 0x83C4,0x9352,0xB3C3,0x8383,0x9352,0x83C4,0xC493,0xB483,0x7383,0x6263,0x7342,0x7352,0x6352,0x4246,0x4646,0x4646,0x465F,0x4646,0x4642,0x465F,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x6F46,0x7F41,0x7F41,0x7F7F,0x427F,0x7F6F,0x9F52,0x5242,0x7F8F,0x427F,0x7F7F,0x7F41,0x7F42,0x427F,0x7F6F,0x7F42,0x8F42,0xD4C4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x62BF,0x62D5,0xD4D4,0xD4D4,0xE4E4,0xF5FF,0xE4F4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4C3,0xD4E4,0xD4D4,0xD4E4,0x62D4,0x4662,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4246,0x4646,0x4646,0x4242,0x4152,0x4246,0x4246,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4246,0x5F46,0x5F46,0x4646,0x465F,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4F46,0x4646,0x464F,0x4646,0x4F42,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4644,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x7373,0xC473,0xC483,0xB483,0x8373,0xB483,0x6383,0x5252,0x5252,0x8383,0x5273,0x7352,0x5252,0x4463,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x6F42,0x7F41,0x427F,0x417F,0x416F,0x417F,0x5241,0x52AF,0x7F41,0x7F7F,0x427F,0x7F6F,0x7F41,0x7F7F,0x7F7F,0x417F,0x418F,0xBF62,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD562,0xD4AF,0x62D4,0xD4D4,0xF4E4,0xFFF4,0xE4F4,0xD4E4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xC3D4,0x62D4,0x62AF,0x46AF,0x4646,0x4644,0x4642,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4246,0x5252,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4246,0x4646,0x4642,0x465F,0x5F46,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4642,0x4646,0x4642,0x464F,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x5F46,0x4646,0x4646,0x465F,0x465F,0x4246,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4246,0x5F46,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4246,0x4646
.word 0x7373,0x8383,0xB483,0x5273,0x6373,0x5273,0x5273,0xD493,0x6383,0x7352,0x7383,0x5252,0x7373,0x5252,0x4644,0x4246,0x4646,0x4246,0x4646,0x4246,0x5F46,0x4646,0x4F42,0x4646,0x465F,0x6F46,0x4241,0x7F7F,0x7F7F,0x427F,0x7F7F,0x527F,0x9F52,0x8F52,0x427F,0x7F7F,0x7F6F,0x7F42,0x7F42,0x416F,0x427F,0x8F41,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xF4E4,0xF3F3,0xE4F3,0xD4E4,0xE4E4,0xD4D4,0xD4BF,0xD4D4,0xD4D4,0xD4C3,0xE4E4,0xD3D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x62D4,0xD562,0x4642,0x5F42,0x4646,0x5F46,0x4646,0x4646,0x4246,0x4646,0x4246,0x5252,0x4652,0x4642,0x465F,0x4646,0x465F,0x4646,0x4246,0x4646,0x5F46,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4642,0x4F46,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x465F,0x4F46,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4246,0x4F46,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x464F,0x4646,0x465F,0x464F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4446,0x4646,0x465F,0x4644,0x465F,0x4646,0x5F46,0x4646,0x4646
.word 0x5252,0x6363,0x5273,0x7373,0x8393,0x93E4,0x8363,0x83D4,0x7373,0xB483,0x7352,0x6383,0x7352,0x6352,0x4252,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x425F,0x417F,0x417F,0x427F,0x417F,0x7F7F,0x527F,0x629F,0x4252,0x7F7F,0x7F7F,0x417F,0x417F,0x527F,0x427F,0x7F7F,0x7F42,0xD4C4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xAFD4,0xD462,0xD4E4,0xD4D4,0xF4E4,0xF4FF,0xE4F4,0xD4E4,0xE4C3,0xD4D4,0xD483,0xD4E4,0xE4C3,0xD4D4,0xE4D4,0xD4E4,0xC3D4,0xD4D4,0xD4E4,0xD472,0x62AF,0xC4AF,0x4646,0x4646,0x4646,0x465F,0x4646,0x4642,0x4646,0x4646,0x4446,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x5F42,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4642,0x4646,0x4646,0x4246,0x4F46,0x4F46,0x4646,0x4646,0x4646,0x465F,0x4246,0x464F,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4246,0x4F46,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4246,0x464F,0x465F,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4642,0x4646,0x4646,0x4646,0x4642,0x4646,0x4246,0x5F46,0x4246,0x4646,0x4642,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4644,0x4646,0x4646,0x465F
.word 0x5273,0x8383,0x8383,0x7383,0x93D4,0x83B4,0x7383,0x7373,0x8352,0x93D4,0x7373,0x8393,0x5252,0x7373,0x7373,0x4252,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4642,0x416F,0x427F,0x7F7F,0x7F7F,0x427F,0x417F,0x7F7F,0x9F52,0x9F62,0x7F42,0x427F,0x6F6F,0x7F7F,0x7F41,0x7F7F,0x4141,0x417F,0xBF62,0xD4C4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0x83D4,0xD4D5,0xD4D4,0xE4D4,0xF4E4,0xE4E4,0xE4F4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x62D5,0xAFD5,0x4646,0x4646,0x465F,0x4646,0x4646,0x464F,0x4F46,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F44,0x4646,0x4644,0x4646,0x4646,0x4646,0x4F46,0x4246,0x4646,0x4F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x4646,0x465F,0x465F,0x4F42,0x5F46,0x5F46,0x4446,0x4246,0x4F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x465F,0x465F,0x465F,0x4646,0x4642,0x464F,0x465F,0x5F46,0x4246,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x464F,0x4646,0x4446,0x4646,0x4646,0x4644,0x4446,0x4646,0x5F46,0x4646,0x465F,0x4646
.word 0x7352,0xB483,0x7383,0x5273,0xB483,0x7383,0x5252,0x8373,0x73B4,0x7383,0x7352,0xC473,0x7373,0x5252,0x5273,0x5273,0x4463,0x4246,0x4646,0x5F46,0x4646,0x4246,0x5F46,0x5F46,0x4646,0x7F7F,0x417F,0x7F41,0x7F42,0x417F,0x7F7F,0x7F7F,0x5242,0x42AF,0x418F,0x7F6F,0x7F7F,0x426F,0x427F,0x7F7F,0x427F,0x8F7F,0xD4C4,0xD472,0xC3D4,0xE4D4,0xC3D4,0xE4E4,0xE4D4,0xD4C3,0x62E4,0x629F,0x62AF,0x6262,0xE4D4,0xE4E4,0xD4D4,0x6283,0x62AF,0x629F,0xD4E4,0xE4E3,0xE4D4,0xC3D4,0xD4E4,0xE4C3,0xE4D4,0xD4D4,0x72D4,0xAFD4,0xBF62,0x46C4,0x4246,0x465F,0x4646,0x465F,0x465F,0x4246,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4246,0x464F,0x5F46,0x4246,0x464F,0x4646,0x4646,0x465F,0x4446,0x5F46,0x4646,0x4646,0x4F46,0x4246,0x4646,0x5F46,0x4646,0x4642,0x4246,0x464F,0x4F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F42,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4446,0x465F,0x4646,0x4642,0x4646,0x5F44,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4446,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x465F,0x465F,0x5F46,0x4646,0x465F,0x464F,0x465F,0x4644,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4644
.word 0x7352,0x7383,0x7363,0x7352,0x7373,0x5252,0xD483,0xC483,0x5293,0x7363,0x7352,0x5252,0x5252,0x5252,0x7352,0x7373,0x5252,0x4643,0x5F46,0x4646,0x4446,0x464F,0x4646,0x4646,0x5F46,0x7F42,0x7F41,0x7F42,0x417F,0x527F,0x427F,0x6F7F,0x9F41,0xD452,0x8F42,0x7F7F,0x6F42,0x7F7F,0x7F7F,0x7F41,0x417F,0x527F,0xC49F,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xC3E4,0x8FD4,0x9F52,0x9F42,0xAF62,0x5262,0xC4D4,0xBF62,0x8FC4,0x5252,0x528F,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD562,0xD4C4,0x46D4,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x465F,0x4F46,0x5F46,0x4646,0x4246,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4644,0x4642,0x4646,0x4646,0x465F,0x4F46,0x4246,0x4646,0x4F46,0x4646,0x4446,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4F46,0x4646,0x4642,0x4646,0x4642,0x4642,0x4646,0x4646
.word 0x6352,0x5273,0x5252,0x8373,0xB3D4,0x73C4,0x7373,0x8383,0x7373,0x9383,0x83C4,0xC473,0xC493,0x8373,0x5273,0x6352,0x7373,0x5252,0x4373,0x4446,0x4646,0x4646,0x4646,0x4644,0x6F46,0x7F41,0x427F,0x417F,0x7F6F,0x417F,0x417F,0x7F7F,0x427F,0x629F,0x42AF,0x6F41,0x6F7F,0x6F41,0x417F,0x7F6F,0x4142,0x527F,0xD48F,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4C3,0xD4E4,0x41D4,0x8F42,0x4241,0x428F,0xD4BF,0xBFD4,0x5262,0x418F,0x428F,0xC4AF,0xF4F4,0xE4E4,0xE4D3,0xD4E4,0xD4E4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0x83BF,0xD4BF,0x4662,0x5F42,0x4646,0x465F,0x465F,0x4246,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x4646,0x465F,0x5F46,0x5F46,0x4446,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4642,0x465F,0x465F,0x4646,0x4246,0x4F46,0x4646,0x5F46,0x4646,0x465F,0x4642,0x4646,0x4246,0x5F46,0x4646,0x464F,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x465F,0x4642,0x464F,0x464F,0x465F,0x4F42,0x5F46,0x4246,0x4646,0x465F,0x4646,0x4246,0x4646,0x4246,0x464F,0x4646,0x4646,0x4F42,0x4F46,0x4F46,0x4F46,0x4F46,0x5F46,0x4246
.word 0x5252,0x7352,0x7383,0xC452,0xB483,0x83B3,0x6352,0x6373,0x9352,0xD4C4,0xC4B3,0x8373,0x93C4,0xD473,0x8383,0x5252,0x5273,0x7373,0x7352,0x4252,0x4642,0x4646,0x4642,0x465F,0x7F46,0x417F,0x417F,0x7F7F,0x427F,0x426F,0x427F,0x7F7F,0x4241,0x528F,0x8F42,0x7F42,0x6F7F,0x6F7F,0x7F7F,0x7F7F,0x427F,0x417F,0xD47F,0x72D4,0xE4D4,0xC3D4,0xD3D4,0xD4E4,0xE4E4,0xD4D4,0x62D4,0x429F,0x418F,0x9F42,0x4142,0x5242,0x429F,0x428F,0x629F,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4C3,0xD4E4,0x72D4,0x62D4,0xD4D5,0xD4D4,0x4662,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x4246,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4246,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x465F,0x4644,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4642,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F44,0x4646,0x465F,0x4646,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x464F
.word 0x7352,0x7352,0x7352,0x7352,0x8373,0x73B4,0xC383,0x93C4,0x73D4,0x8383,0x62B4,0x5252,0x5252,0x8373,0x73C4,0x8373,0x5263,0x7352,0x5273,0x6373,0x5273,0x7352,0x6342,0x5263,0x4163,0x427F,0x427F,0x7F41,0x7F7F,0x417F,0x5241,0x417F,0x417F,0x527F,0x52AF,0x418F,0x7F7F,0x7F42,0x6F7F,0x7F42,0x4141,0x427F,0xD47F,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x9F62,0x9F52,0x9F52,0xE4C4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xF3E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xC4AF,0x72D4,0xD462,0x4646,0x465F,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x464F,0x4446,0x4246,0x5F46,0x4246,0x4F46,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x465F,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x465F,0x5F46,0x4F46,0x5F46,0x4646,0x465F,0x465F,0x4246,0x5F46,0x5F46,0x4F46,0x5F46,0x5F46,0x5F46,0x4646,0x464F,0x4646,0x4646,0x5F46,0x4646,0x4642,0x5F46,0x4246,0x4646,0x4642,0x464F,0x5F46,0x5F46,0x5F46,0x4246,0x464F,0x465F,0x464F,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x465F,0x465F,0x464F,0x465F,0x465F,0x5F46,0x5F46,0x4F46,0x4F46,0x4646,0x5F46,0x4646,0x5F46,0x4246,0x5F46,0x4646,0x4646,0x4646
.word 0x5252,0x6352,0x7373,0x7352,0x7352,0x5252,0x8373,0x7383,0x6383,0x7373,0x5252,0x9373,0x83C3,0x5263,0x5283,0xD473,0x8393,0x5263,0x8352,0x5273,0x7373,0x5273,0x7373,0x5273,0x7F52,0x7F41,0x417F,0x428F,0x7F7F,0x427F,0x8F9F,0x427F,0x427F,0x418F,0x9F62,0x7F52,0x6F7F,0x416F,0x7F6F,0x7F7F,0x427F,0x8F7F,0xC441,0xC4BF,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xF4D4,0xC4E4,0x52AF,0x62D5,0xE4E4,0xE4F4,0xE4D4,0xE4F4,0xF4E4,0xF4F3,0xE4E4,0xD3E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD462,0xD5D4,0xBFD4,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4F42,0x4646,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x4644,0x4246,0x4646,0x465F,0x4642,0x5F46,0x4646,0x464F,0x4646,0x4646,0x4642,0x4646,0x4646,0x4446,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x4646,0x464F,0x465F,0x4F46,0x5F46,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x465F,0x4642,0x5F46,0x4646,0x4246,0x4646,0x465F,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4F46,0x4646,0x5F46,0x4646,0x4646
.word 0x5252,0x5252,0x5252,0x7352,0xC493,0xC493,0x7352,0x6373,0x8352,0xC493,0xC493,0xC473,0xB3D4,0xC483,0xB493,0x7352,0x7373,0x5252,0x7373,0x5252,0x7352,0x7383,0x7373,0x7373,0x417F,0x4142,0x427F,0x7F7F,0x427F,0x416F,0x4152,0x4142,0x417F,0x8F7F,0x6242,0x529F,0x7F42,0x7F7F,0x427F,0x7F6F,0x4141,0x427F,0xC48F,0x72D4,0xBFD4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4E4,0xE2D4,0xF3F3,0xE4F4,0xE4D4,0xE4F4,0xF4D4,0xE4F4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xD3E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xBFD4,0xBFC4,0x6283,0x4246,0x5F46,0x4646,0x465F,0x4246,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x464F,0x4646,0x465F,0x4642,0x464F,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4642,0x4646,0x4246,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4642,0x4246,0x4646,0x465F,0x4646,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x465F,0x465F,0x465F,0x5F46,0x4246,0x5F46,0x4646,0x4246,0x4646,0x4646,0x464F,0x4646,0x4246,0x4646,0x465F,0x4646,0x4642,0x4646,0x5F46,0x4246,0x4F46,0x4246,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F
.word 0x5242,0x5252,0x5273,0x7373,0xC483,0x8383,0x7363,0x5273,0xC473,0xD493,0xD493,0x8352,0x7393,0x7363,0x93D4,0x5283,0x8373,0x7383,0xC352,0x5273,0x5273,0x5252,0x5252,0x5252,0x417F,0x427F,0x8F7F,0x7F41,0x7F7F,0x7F7F,0x9F52,0x9F42,0x427F,0x428F,0x528F,0x41D5,0x418F,0x426F,0x7F6F,0x7F6F,0x427F,0x417F,0x838F,0xC4BF,0xC4D4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xC4D4,0xB4C5,0xC563,0xE473,0xE4F3,0xF1E4,0x63E4,0x4363,0x6343,0x8463,0xD4D5,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4D4,0x62D4,0x42D4,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x464F,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4642,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x464F,0x464F,0x4646,0x4646,0x4646,0x4646,0x4644,0x4646,0x4642,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4646,0x4642,0x4246,0x5F46,0x4646,0x464F,0x4246,0x4646,0x4646,0x4642,0x5F46,0x4646,0x465F,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x464F,0x464F,0x464F,0x4646,0x4646,0x4F46,0x4F46,0x4646,0x465F,0x465F,0x5F46,0x4646,0x4646,0x4642,0x4246,0x4646,0x4644,0x4646,0x4642,0x4646,0x4246
.word 0x5252,0x5252,0x5252,0x5252,0x6373,0x4273,0xD483,0xD4C3,0x7393,0x93C4,0x83B4,0x7352,0x7363,0x5283,0xC483,0x7373,0xE493,0xC4C3,0x7373,0x7352,0xC383,0x73C4,0x5252,0x7F52,0x7F41,0x8F42,0x427F,0x7F41,0x7F42,0x417F,0x9F42,0x5262,0x528F,0x7F7F,0x8F41,0xC441,0x7F41,0x7F7F,0x7F6F,0x427F,0x416F,0x7F8F,0x527F,0x62D4,0x83BF,0x72D4,0x62D4,0x52AF,0x4242,0x6342,0x4243,0x6342,0x6363,0xC563,0xC573,0x6373,0x4343,0x6343,0x4343,0x4263,0x4343,0x7343,0xB4C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD572,0x83BF,0xC4BF,0x469F,0x4646,0x465F,0x4246,0x4646,0x465F,0x4646,0x4642,0x4646,0x4246,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4644,0x465F,0x465F,0x4642,0x4646,0x4246,0x464F,0x4246,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x465F,0x4646,0x4642,0x465F,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4F46,0x4646,0x4646,0x4246,0x4F46,0x4646,0x464F,0x4646,0x465F,0x4446,0x4646,0x5F46,0x4646,0x464F,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x5F46,0x4F46,0x4F46,0x5F46,0x4F46,0x5F46,0x5F46,0x5F46,0x4646
.word 0x4283,0x5252,0x5263,0x5252,0x5252,0x5252,0xC383,0x8393,0x7373,0x7373,0x7383,0xD483,0xD4D3,0x83D4,0x7373,0x7352,0xC483,0x8393,0x5263,0x7383,0x7363,0x5273,0x8373,0x7F52,0x427F,0x418F,0x7F7F,0x7F52,0x7F7F,0x7F7F,0x6242,0x52AF,0x527F,0x428F,0x428F,0x8F7F,0x6262,0x417F,0x6F7F,0x7F6F,0x427F,0x7F41,0x4241,0xC4BF,0xBFC4,0xAFC4,0x7F41,0x4242,0x4364,0xC543,0xC563,0x6363,0x63C5,0x63C5,0x4363,0x6343,0xC563,0xC543,0xC5C5,0xC573,0xC563,0x4663,0x425F,0xD463,0xD462,0xD4D4,0x62D4,0xC4BF,0x62D4,0xD4D5,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x4642,0x464F,0x5F46,0x4646,0x465F,0x4646,0x5F46,0x4F46,0x4F46,0x5F46,0x4246,0x464F,0x465F,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x465F,0x465F,0x465F,0x4646,0x464F,0x4646,0x4646,0x4642,0x465F,0x465F,0x4644,0x4646,0x4646,0x4F46,0x4646,0x4642,0x465F,0x4642,0x465F,0x465F,0x4642,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x5F46
.word 0x6393,0x5252,0x6362,0x8352,0x8383,0x52B4,0x9373,0x73B4,0x5273,0x5252,0x7383,0x9363,0x8383,0x73D4,0x8352,0x83C4,0x7363,0x5263,0xE483,0x83D4,0xD483,0x7383,0x5252,0x417F,0x417F,0x8F42,0x427F,0x4141,0x7F7F,0x7F42,0x6241,0x9FD5,0x9F52,0x7F42,0x8F41,0x7F52,0xC48F,0x428F,0x417F,0x7F6F,0x416F,0x427F,0x8F7F,0xD562,0xD572,0xD572,0xD4C4,0x4563,0x4242,0x426F,0x7F42,0x4242,0x7F42,0x5F42,0x5F6F,0x4A6F,0x425F,0x5243,0x4342,0x4342,0x4242,0x6342,0xD4D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD562,0xBF62,0x83C4,0x6362,0x6263,0x6263,0x5252,0x4252,0x4246,0x5F46,0x4646,0x4246,0x4646,0x4446,0x4646,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x464F,0x5F46,0x4646,0x465F,0x4646,0x4646,0x465F,0x4F46,0x4646,0x465F,0x465F,0x464F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x464F,0x464F,0x465F,0x465F,0x4646,0x5F44,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4642,0x5F46,0x4646,0x465F,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4644,0x4646,0x4646,0x4642
.word 0x83D4,0x5242,0x5252,0x7352,0x6373,0x7373,0x7373,0x5273,0xC452,0xD4C3,0x93C3,0x7373,0x8373,0x6383,0xC493,0x8392,0x8373,0x7383,0x8373,0x83D4,0xB3B4,0x83D4,0x4273,0x417F,0x427F,0x418F,0x417F,0x8F52,0x6F42,0x7F7F,0x9F42,0x5252,0x429F,0x7F41,0x8F42,0x8F41,0x8F42,0x4162,0x7F42,0x7F6F,0x427F,0x418F,0x427F,0xBF62,0xBFC4,0xD4C4,0xC4BF,0xAFD4,0x43C5,0x4344,0x84C5,0xC5C5,0xC5C5,0x4445,0x4642,0x445F,0xC544,0xE5D6,0xD5F5,0xC5C5,0x4373,0xD5C5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0x62BF,0x62D4,0x83D4,0x5273,0x7373,0x7352,0x7352,0x7373,0x6373,0x5473,0x4446,0x4646,0x465F,0x4646,0x465F,0x4644,0x5F46,0x4646,0x4646,0x5F46,0x5F46,0x4446,0x4646,0x465F,0x4646,0x465F,0x4646,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4246,0x5F46,0x4246,0x464F,0x465F,0x465F,0x465F,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4644,0x4646,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4246,0x4F46,0x4646,0x4246,0x4646,0x465F,0x4246,0x4F46,0x4646,0x465F,0x4642,0x4646
.word 0x93D4,0x4283,0x5252,0x5252,0x5252,0x5252,0x8383,0x83C4,0x8352,0x83C4,0xB483,0x7352,0x7352,0x8363,0xB3D4,0x83C4,0xC363,0xC493,0x6373,0x5273,0x8373,0x52B4,0x8F42,0x7F41,0x8F42,0x4241,0x7F7F,0x5262,0x527F,0x417F,0x527F,0x629F,0x8F62,0x8F42,0x417F,0x527F,0x418F,0x8F8F,0x528F,0x7F52,0x7F7F,0x7F42,0x417F,0x739F,0x62D4,0x62D4,0x62D4,0x83D5,0xC5AF,0xC573,0xD663,0xD5C4,0x73EF,0xD5FF,0xF5D5,0xF3D5,0xE4D5,0xB4C5,0xC563,0x6352,0xC5C5,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xC4BF,0xD562,0xD4BF,0x7373,0x7373,0x7373,0x7373,0x7373,0x7352,0x7352,0x5273,0x5273,0x7352,0x4654,0x4642,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4646,0x465F,0x4246,0x4646,0x4646,0x4642,0x4646,0x4446,0x4646,0x4646,0x465F,0x4F42,0x4F46,0x4646,0x4446,0x465F,0x4642,0x464F,0x464F,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4F46,0x4F46,0x5F46,0x4646,0x4642,0x4642,0x5F46,0x4646,0x4646,0x465F,0x4642,0x4646,0x4646,0x464F,0x464F,0x465F,0x4246,0x464F,0x465F,0x465F,0x465F,0x4646,0x4646,0x4246,0x4646,0x464F,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4642,0x4F46,0x4F46,0x4646,0x465F,0x464F,0x464F,0x4246,0x4F46,0x4646,0x465F,0x4646,0x464F,0x464F
.word 0xC493,0x7383,0x4252,0x7352,0x8352,0x5283,0x7373,0x8373,0x7373,0xB483,0x7383,0x9373,0x93C3,0x7373,0xC483,0x6383,0x9383,0x93C4,0x7352,0x7383,0x5252,0x4152,0x7F7F,0x7F42,0x417F,0x7F8F,0x7F41,0x8F62,0x417F,0x7F7F,0x5241,0x73AF,0x42AF,0x419F,0x427F,0x528F,0x428F,0x427F,0x8F41,0x4152,0x7F7F,0x7F41,0x527F,0x528F,0xD4AF,0xD4D4,0xC4BF,0xC4BF,0x9F83,0x63C5,0x6363,0x6373,0xC5C5,0xEFD4,0xF5E5,0xD5C5,0xC5B4,0xC573,0x7363,0x63C5,0x72C5,0xD4D4,0xD4E4,0xC3D4,0xD4D4,0xD472,0xD462,0x83BF,0x83D4,0xD483,0xD493,0xE4C3,0x8393,0x8383,0x7373,0x5273,0x7352,0x7373,0x7373,0x5273,0x4643,0x4646,0x4246,0x4F46,0x5F46,0x5F46,0x4246,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4246,0x464F,0x4646,0x464F,0x4646,0x4646,0x4642,0x4646,0x4646,0x465F,0x4246,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x464F,0x4646,0x465F,0x4646,0x4446,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646
.word 0xB3C4,0x83E4,0x5273,0x5252,0x6373,0x6352,0x5273,0x8373,0x7363,0x5273,0x7352,0xC493,0xC493,0x5283,0x6373,0x5273,0x7373,0x6373,0x9352,0x93C4,0x5263,0x4142,0x417F,0x7F8F,0x4142,0x8F42,0x627F,0x429F,0x7F52,0x427F,0x8F7F,0xD573,0x9F52,0x4252,0x418F,0xC47F,0x418F,0x8F7F,0x8F42,0x428F,0x7F62,0x427F,0x417F,0x527F,0xAF52,0x6262,0x62D5,0x62D4,0xC4D4,0x529F,0x73D6,0x6343,0x63D5,0xC5C5,0x6373,0x6373,0xC573,0x7363,0x63C5,0xD4BF,0xE4D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xC4AF,0x73BF,0xD4D4,0x6373,0x5252,0x5252,0x5252,0x5252,0x5252,0x7373,0x8383,0x8373,0x8373,0x7373,0x7352,0x6273,0x6252,0x4643,0x4646,0x4646,0x4646,0x465F,0x464F,0x4246,0x464F,0x4F46,0x4246,0x464F,0x465F,0x465F,0x4642,0x465F,0x464F,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x4F46,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4642,0x4246,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x465F,0x5F46,0x4646,0x4246,0x4646,0x4642,0x465F,0x465F,0x465F,0x465F,0x465F,0x465F,0x465F,0x4646,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4246,0x4646,0x4642,0x4246,0x5F46,0x5F46,0x4646,0x465F,0x4642,0x464F,0x4646,0x4246
.word 0xC493,0xB3D4,0x52C4,0x5252,0x5252,0x5273,0x7352,0x6373,0x5252,0x8352,0x7373,0xC483,0xD483,0x7383,0x5252,0x7373,0x7352,0x5252,0x7373,0x7383,0x4173,0x427F,0x428F,0x417F,0x428F,0x428F,0x428F,0x9F52,0x7F8F,0x417F,0x8F42,0x9F62,0x8F62,0x9F52,0x7F42,0x5241,0x428F,0x417F,0x427F,0x8F7F,0x6242,0x417F,0x7F7F,0x8F42,0x5252,0xAFD5,0xBFC4,0xBFC4,0xBFC4,0xAFC4,0x4352,0x63D6,0x52C5,0x6363,0xC563,0x6343,0xC543,0xC552,0xAF73,0xD4C4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0x62D4,0x62D4,0xD4D5,0xC462,0x5283,0x5252,0x6352,0x8373,0x93C4,0x73B4,0x5252,0x7352,0x8373,0xD493,0x7383,0x7373,0x8373,0x5273,0x6273,0x4652,0x4644,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4246,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x465F,0x4642,0x465F,0x4246,0x4F46,0x4F46,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4246,0x464F,0x4246,0x4646,0x465F,0x464F,0x465F,0x464F,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x465F,0x465F,0x4646,0x465F,0x4246,0x464F,0x465F,0x465F,0x4646,0x4646,0x4646,0x4644,0x4646,0x465F,0x4F46,0x4F46,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x465F,0x4646
.word 0x83E4,0xD493,0xB4B3,0x5273,0x5252,0x5252,0x5252,0x5252,0x8352,0xB483,0x52B2,0x7373,0x7383,0x7352,0x8383,0x8383,0x8352,0x73C4,0x5252,0x5263,0x7F7F,0x8F41,0x417F,0x7F7F,0x8F42,0x5241,0x7F8F,0x5252,0x427F,0x417F,0x428F,0x529F,0x42AF,0x429F,0x418F,0x7F8F,0x9F42,0xD562,0xD4D4,0x83D4,0x8F9F,0x7F7F,0x7F42,0x8F41,0x9F52,0x6262,0xD4D4,0xD483,0x83D4,0xD462,0x52AF,0x528F,0x529F,0x529F,0x419F,0x9F9F,0x9F52,0x9F41,0xD462,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x62D4,0xBFD4,0xBFC4,0x83C4,0x5273,0x7373,0x8373,0x8393,0x8352,0x83C4,0x7383,0x8352,0x83C4,0x5263,0x5252,0x8373,0x8393,0x7373,0x8373,0x7383,0x5273,0x5273,0x4642,0x4446,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4246,0x4646,0x4646,0x465F,0x465F,0x465F,0x4246,0x4646,0x465F,0x4646,0x4F46,0x4646,0x4646,0x5F46,0x4246,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4F46,0x5F46,0x5F46,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4246,0x4646,0x4644,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x464F,0x464F,0x465F,0x465F,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x5F46
.word 0x93D4,0x93B4,0xB3C4,0x7383,0x4252,0x6341,0x7352,0x8373,0x7352,0x7373,0x6383,0x7352,0x5252,0x8363,0x83B4,0x73B4,0x8373,0x7383,0x7373,0x7F52,0x7F41,0x4142,0x7F42,0x7F42,0x8F41,0x8F52,0x527F,0x419F,0x418F,0x427F,0x418F,0xAF42,0x8F73,0x4152,0x428F,0x8F52,0x6252,0x62D5,0xD462,0xBFD4,0xD4C4,0x83D4,0x42AF,0x7F7F,0x4241,0x529F,0xD4AF,0x62D4,0xD4D4,0xD4D4,0xC4D4,0x42AF,0x428F,0x428F,0x428F,0x8F42,0x9F42,0xD4C4,0xE4E5,0xD4E4,0xE4D4,0xD4D4,0xD4C3,0xBFD4,0x62C4,0x62D5,0x8383,0x93C4,0x7383,0xD473,0xC4D4,0x5252,0x5273,0x6352,0x5252,0x5252,0x8373,0x7373,0x5252,0x7363,0x8383,0x7383,0x7373,0x7373,0x7373,0x5273,0x4646,0x465F,0x4646,0x5F44,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x4246,0x4646,0x4646,0x465F,0x4246,0x464F,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x464F,0x5F46,0x4646,0x4246,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x464F,0x465F,0x4F46,0x4646,0x465F,0x5F46,0x4246,0x4646,0x4642,0x4646,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x4646,0x4446,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xB3D4,0x83C4,0x83B4,0x93D4,0x5283,0x5252,0x5273,0x7352,0x7352,0x5252,0x5263,0x8352,0xC483,0x7383,0x7352,0x7352,0x6373,0x5273,0x7F41,0x417F,0x4142,0x528F,0x418F,0x417F,0x8F42,0x4241,0x418F,0x9F42,0x7F42,0x8F41,0x8F7F,0x5242,0x7F8F,0x427F,0x8F8F,0xD552,0x62AF,0x62BF,0xC4AF,0x6262,0x62D4,0x62D4,0x62D5,0xC4D4,0xAFD4,0x9F52,0x6252,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD562,0xD472,0x6383,0xC493,0x7393,0x7352,0x5273,0x7373,0xC483,0x9393,0x7373,0xD483,0xB3D4,0x52B4,0x93D4,0x6383,0x5252,0x8363,0x8383,0x7373,0x7373,0x8373,0x7373,0x4663,0x4642,0x465F,0x4F46,0x4F46,0x5F46,0x5F46,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x464F,0x465F,0x465F,0x4F46,0x4646,0x465F,0x5F46,0x4646,0x464F,0x5F46,0x5F46,0x5F46,0x4246,0x4646,0x4646,0x465F,0x4646,0x465F,0x465F,0x4F46,0x4F46,0x4F44,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x4246,0x5F46,0x4646,0x4F46,0x4F46,0x4F46,0x5F46,0x5F46,0x4F46,0x4646,0x4246,0x4646,0x5F46,0x4246,0x5F46,0x5F46,0x4F46,0x4646,0x4642,0x4F46,0x4646,0x464F,0x465F,0x465F,0x465F,0x4642
.word 0xC4C3,0xC493,0xD493,0xC3D4,0x83D4,0x5273,0x5252,0x5252,0x5252,0x5252,0x5252,0x7352,0x8373,0x5273,0x8373,0x7352,0x5252,0x4252,0x417F,0x427F,0x428F,0x428F,0x8F7F,0x527F,0x428F,0x7F7F,0x8F42,0x4142,0x427F,0x428F,0x7F6F,0x5241,0x428F,0x8F41,0x6242,0x62AF,0x62D4,0xC4D5,0xBFD4,0xD4C4,0xAFD4,0xD4D4,0xD462,0xD4D4,0x62D4,0x62D4,0x528F,0xD4AF,0xD4D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4F4,0xF4F4,0xF4E4,0xF4F4,0xE4E4,0xE4E4,0xE4E4,0xD3D4,0xD4D4,0x72D4,0xBFD5,0xBFC4,0x83B4,0x73C4,0x8383,0x52B4,0x5252,0x8373,0x7383,0x8383,0xD4C4,0x5283,0x9383,0x93D4,0x7383,0xC483,0x7383,0x7352,0x5252,0x8363,0x8393,0x8383,0x8373,0x8373,0x7373,0x4642,0x4646,0x4646,0x4642,0x4646,0x4646,0x4246,0x4646,0x4646,0x4642,0x4646,0x464F,0x4646,0x4646,0x4246,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4246,0x4646,0x465F,0x4646,0x4646,0x4F42,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4246,0x4646,0x465F,0x464F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x464F
.word 0xB483,0x4644,0xD493,0xC493,0xC493,0xC493,0x5273,0x5252,0x7352,0x5273,0x5283,0x6352,0x5252,0x5283,0x7373,0x5273,0x5252,0x417F,0x4142,0x8F52,0x5241,0x41AF,0x427F,0x7F41,0x8F41,0x4241,0x7F8F,0x8F52,0x7F8F,0x6F7F,0x7F7F,0x8F41,0x8F42,0x6262,0xD4D5,0xC4D4,0xC4BF,0xD4D4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x41D4,0xD462,0xD4D4,0xE4D4,0xD3D4,0xF4E4,0xE4E4,0xF4E4,0xE4E4,0xF4F4,0xE4F4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x83D4,0x7373,0x5273,0x7352,0x5252,0x7373,0xD473,0xD4B3,0x83D4,0x8373,0x8383,0x5263,0x7363,0x5273,0x5252,0x5263,0x7373,0x8383,0x8363,0x5273,0x6352,0x8383,0x8373,0x7373,0x6352,0x7373,0x4252,0x4646,0x4646,0x4642,0x465F,0x465F,0x5F46,0x4F46,0x4F46,0x4646,0x4246,0x4246,0x4F46,0x4646,0x464F,0x4646,0x4646,0x4246,0x4646,0x4642,0x464F,0x4642,0x465F,0x4F44,0x4F46,0x4646,0x4246,0x5F46,0x4646,0x4F42,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4246,0x464F,0x465F,0x4F46,0x4646,0x4642,0x465F,0x4246,0x4F46,0x4646,0x4646,0x5F46,0x4F46,0x4246,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x465F,0x4646
.word 0x4473,0x4646,0x9383,0x93C4,0x93C4,0xB3C4,0x83C4,0x5252,0x7352,0x7352,0x5273,0x5273,0x8352,0x7373,0x7352,0x5252,0x4141,0x8F42,0x527F,0x528F,0x529F,0x427F,0x8F7F,0x8F42,0x8F42,0x527F,0x7F7F,0x4241,0x7F7F,0x6F42,0x7F41,0x8F42,0x6241,0xBFD5,0x62D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0x62AF,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4F4,0xE4F4,0xC3E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x7383,0xB3C4,0x93C3,0x8373,0xD4D4,0xC4B3,0x8362,0x93B4,0x83D4,0x6352,0x5252,0x7373,0xD483,0xC493,0x7383,0x8373,0xE493,0x83D4,0x7383,0xC493,0x5263,0x7352,0xB483,0x8352,0x8383,0x7373,0x8373,0x4652,0x4F42,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4644,0x4646,0x4646,0x4F46,0x4646,0x465F,0x465F,0x4246,0x4646,0x4F46,0x5F46,0x4646,0x464F,0x465F,0x465F,0x4642,0x4F46,0x5F46,0x4646,0x465F,0x4646,0x5F46,0x4646,0x465F,0x464F,0x4246,0x4F46,0x4646,0x4246,0x4646,0x4646,0x4246,0x4F46,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4642,0x4646,0x4642,0x5F46,0x5F46,0x4646,0x4646,0x4646
.word 0x4646,0x836F,0x93D4,0x93C4,0x83B4,0xB483,0xD4B3,0xD4D4,0x6383,0x5252,0x4252,0x5252,0x6373,0x5273,0x5252,0x7F42,0x7F42,0x417F,0x9F52,0x9F52,0x8F52,0x8F42,0x417F,0x7F8F,0x5241,0x8F41,0x7F7F,0x7F7F,0x7F7F,0x7F7F,0x427F,0x527F,0xD5D4,0xD462,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0xE4C3,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0x51D4,0xD4D4,0xD3D4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0x72D4,0xD483,0x5293,0xC483,0xE483,0xC483,0xC493,0x83B3,0x8363,0xB483,0x5273,0x9373,0xC3D4,0x83C3,0x8363,0xB3D4,0x93D4,0x8363,0x83C4,0xB493,0x6383,0xC493,0x83B3,0x7373,0x7352,0x8373,0x83B3,0x7383,0x7352,0x5273,0x4664,0x4646,0x465F,0x4646,0x4246,0x464F,0x465F,0x464F,0x465F,0x464F,0x464F,0x5F46,0x5F46,0x5F46,0x4F46,0x4646,0x4646,0x465F,0x5F46,0x5F46,0x4646,0x4646,0x4644,0x465F,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x4646,0x465F,0x4642,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x5F46,0x4646,0x464F,0x465F,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x465F,0x465F,0x465F
.word 0x4646,0x9346,0x93D4,0x93D4,0xD4C4,0x8383,0x93C4,0x9393,0xB3D4,0x5273,0x5252,0x5252,0x6273,0x5252,0x8F52,0x4141,0x7F8F,0x9F42,0x6242,0x529F,0x8F52,0x7F8F,0x7F42,0x4142,0x528F,0x428F,0x527F,0x7F41,0x7F42,0x416F,0x8F42,0xD462,0xC4AF,0x62D4,0xD4BF,0xD4D4,0xE4D4,0xD3D4,0xD4E4,0xE4E4,0xE4C3,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x62D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD4E4,0x83D4,0xB483,0xB483,0x8383,0x83B4,0x73B4,0x5273,0x5252,0x7373,0x9363,0x93C4,0x83D4,0x8373,0xC493,0x83D4,0x8373,0xB473,0x8383,0x5263,0x7373,0x7373,0x7352,0x5273,0x7352,0xB483,0xB493,0x7383,0x7383,0x7373,0x4642,0x4646,0x465F,0x465F,0x4646,0x4246,0x4646,0x4646,0x4246,0x4646,0x4642,0x4646,0x4642,0x4246,0x4F46,0x4646,0x4646,0x4642,0x4646,0x4246,0x4646,0x4646,0x4246,0x4646,0x465F,0x4246,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x464F,0x4646,0x465F,0x4246,0x4646,0x4646,0x4642,0x465F,0x4246,0x4F46,0x4246,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4642,0x4246,0x4646,0x465F,0x4646,0x4246,0x4646,0x4646,0x465F,0x4646,0x4246,0x5F46
.word 0x4246,0xD446,0xC493,0x6383,0x4646,0x9363,0xD4D4,0xD4C4,0xC493,0xE483,0x93D4,0x8383,0x5273,0x4142,0x5242,0x7F7F,0x627F,0x418F,0x9FD4,0x9F52,0x428F,0x417F,0x417F,0x428F,0x8F41,0x7F42,0x7F7F,0x427F,0x6F7F,0x5241,0xC4D4,0xC4BF,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xC3D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x62D4,0x62AF,0x9F62,0x6262,0x6252,0x6262,0x6262,0x62D4,0xD462,0x8373,0x8383,0x8383,0xB483,0x7373,0x5252,0x5252,0x7373,0x5252,0xC483,0xC3B3,0xD4C3,0x8383,0x83B4,0x73C4,0x8363,0x83B4,0x5273,0x6373,0x5252,0x8373,0x8373,0x8383,0x7383,0x8373,0x83D4,0x5273,0x7352,0x9383,0x83C4,0x7373,0x7373,0x7373,0x4642,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4F46,0x4646,0x4642,0x465F,0x464F,0x4246,0x4646,0x4646,0x4642,0x465F,0x465F,0x4646,0x4646,0x4F46,0x4646,0x4F42,0x5F46,0x4646,0x4246,0x4646,0x4646,0x465F,0x4246,0x4F46,0x5F46,0x4646,0x464F,0x5F46,0x4F46,0x4646,0x465F,0x5F46,0x5F46,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x465F,0x465F,0x464F,0x4646,0x465F,0x4646,0x464F,0x464F,0x464F,0x4642,0x4646,0x4644,0x4646,0x4646
.word 0x4646,0xB346,0x83D4,0x4663,0x4246,0x8346,0xD4C3,0xD493,0xC4B3,0x9383,0xD4C4,0xD4C3,0x5283,0x417F,0x8F9F,0x427F,0x8F52,0x5242,0x52AF,0x4252,0x418F,0x7F7F,0x7F42,0x8F52,0x8F42,0x7F8F,0x4142,0x7F41,0x527F,0xD4D4,0xD4AF,0xD4C4,0x62D4,0xBFD4,0xD4C4,0xD4D4,0xD4E4,0xD4E4,0xC3E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xC3E4,0xE4E4,0xC3D4,0xD4E4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x8F62,0x427F,0x8F8F,0x7F8F,0x8F8F,0x7F8F,0x8F8F,0xC473,0x7383,0x7383,0x93B4,0x5273,0x8352,0x7393,0x8373,0x92D4,0x93D4,0x8373,0xD4D4,0x93D4,0x73B4,0x8383,0x5273,0x5273,0x5252,0x8383,0x8373,0xB3C4,0xD4D4,0x7373,0xD4C4,0xC493,0x8383,0xB3C4,0x83C4,0x5252,0x8373,0x93C4,0x83B4,0x8383,0x7352,0x4373,0x4644,0x4646,0x4642,0x465F,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F42,0x4646,0x4642,0x4646,0x4646,0x4246,0x4F46,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4F42,0x4646,0x4642,0x465F,0x465F,0x4F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x5F46,0x4646,0x464F,0x465F,0x465F,0x4646
.word 0x4646,0xC45F,0x63B3,0x4646,0x4646,0xD446,0x93D4,0x83C4,0x4646,0x8383,0xE493,0x6383,0x8F41,0x5242,0x7F8F,0x4142,0x418F,0x528F,0x9F52,0x7F52,0x7F8F,0x7F42,0x527F,0x42AF,0x428F,0x7F41,0x427F,0x526F,0x62D4,0x62D5,0xD4D4,0xD462,0xBFD4,0xD4C4,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D3,0xD3E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0x8F62,0x5252,0x5252,0x4252,0x5252,0x4152,0x7373,0x5252,0x5252,0x6352,0x6383,0xD493,0xD4C3,0x8373,0xC493,0xC493,0x8373,0x8383,0x7383,0x7352,0xD483,0x8393,0x8363,0x93C4,0xD4C4,0x52D4,0xC483,0x9393,0x73C4,0x8383,0x83C4,0x52B4,0x7352,0x5273,0x7363,0x5252,0x8363,0x8383,0x73B4,0x7363,0x7383,0x4652,0x4646,0x464F,0x4646,0x465F,0x4642,0x5F46,0x4646,0x465F,0x464F,0x464F,0x465F,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4246,0x4F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x465F,0x465F,0x465F,0x465F,0x464F,0x5F46,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4642,0x464F,0x4246,0x464F,0x4646,0x4646,0x465F,0x4644,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642
.word 0x4646,0x8346,0x46D4,0x6F46,0x4646,0x8342,0x83D4,0x4693,0x4646,0xD473,0x73D4,0x8F42,0x527F,0x7F9F,0x7F42,0x8F41,0x427F,0x9F62,0x8F52,0x8F52,0x7F42,0x417F,0x5241,0x8F52,0x7F41,0x7F41,0x6F41,0xBF52,0xBFC4,0xD462,0xD462,0xC4BF,0x62D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4C3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x529F,0x52AF,0x52AF,0x9F9F,0x5252,0x5252,0x8373,0xC3D4,0xB3D4,0x83C4,0x93C4,0x83D4,0xC473,0xB483,0x7393,0x7373,0x7373,0x5273,0x9373,0xD4C3,0xD4C3,0x7383,0xC483,0x9393,0x73D4,0x8383,0x83B4,0x5283,0xB483,0x8383,0x7363,0x9383,0x93C4,0x83C4,0x6383,0x5252,0x93B4,0x8383,0x5273,0x7383,0x6273,0x4642,0x4646,0x465F,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4246,0x5F46,0x4F46,0x5F46,0x4646,0x4642,0x5F46,0x4646,0x4246,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4246,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x465F,0x4646,0x4646,0x4642,0x4646,0x4246,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4F46,0x4646,0x465F,0x465F,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646,0x4646,0x4F46,0x4646,0x4246,0x5F46,0x5F46,0x5F46,0x4246,0x4F46,0x5F46
.word 0x4646,0x4646,0x4673,0x4646,0x4646,0x9346,0x93D4,0x4646,0x4642,0x8346,0x4152,0x7F7F,0x8F62,0x7F42,0x527F,0x427F,0x8F41,0x9F52,0x9F42,0x417F,0x7F7F,0x7F41,0x9F52,0x428F,0x428F,0x7F7F,0xD452,0x62D4,0xC4D5,0xAFD4,0xD4D4,0xD483,0xD4D4,0x72D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xF4E4,0xE4E4,0xE4E3,0xD3E4,0xE4E4,0xE4E4,0xD4D4,0x62D4,0xD462,0xD4D4,0xE4E4,0xE4D4,0xD4D4,0x62D4,0x73AF,0xC4AF,0x9F62,0x7352,0xC483,0x7383,0xC483,0xC493,0x7373,0x7373,0x7373,0x5252,0x5252,0x7352,0x5252,0x8363,0xB3C4,0xC473,0x9393,0xC493,0x6383,0x7373,0x8373,0x5263,0x6373,0x5273,0x8363,0x5263,0x6352,0x7383,0xC483,0xC3B3,0xD4B2,0x83D4,0x5273,0x7352,0x9373,0x7383,0x8373,0x7373,0x6383,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x4646,0x4246,0x4F46,0x4646,0x4646,0x465F,0x465F,0x4246,0x4646,0x464F,0x4646,0x4446,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x4642,0x465F,0x4646,0x4246,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x5F46,0x4646,0x4646,0x465F,0x464F,0x465F,0x5F44,0x4646,0x4642,0x464F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0x4246,0x4646,0x4642,0x4646,0x465F,0x8346,0xB483,0x4642,0x4646,0x4146,0x7F7F,0x527F,0x7F8F,0x7F41,0x8F41,0x417F,0x527F,0x428F,0x429F,0x7F8F,0x427F,0x427F,0x429F,0x417F,0x417F,0x427F,0x62D4,0xBFD4,0xBF83,0x62C4,0x62D5,0xBFD4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD3D4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0x62E4,0x418F,0x8F7F,0x418F,0x418F,0x417F,0xD441,0xD4D4,0xD4D4,0xAF62,0xAF73,0x5252,0x739F,0xC3C4,0x73C4,0x8383,0x73C4,0x5252,0x5252,0x6352,0xD473,0xD4B3,0x6383,0xB3D4,0x93E4,0xD4C3,0x8383,0x83B4,0x83C4,0x5273,0x5252,0x5252,0xD473,0xD4B3,0x8393,0x8363,0xC3D4,0xC3E4,0x52C4,0xC483,0xC3D4,0x93D4,0x8383,0x8363,0x5263,0x7373,0x83D4,0x7373,0x8373,0x8373,0x4642,0x4644,0x4642,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4246,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x464F,0x4F46,0x5F46,0x4646,0x5F46,0x4F46,0x4F46,0x5F46,0x5F46,0x4446,0x4646,0x4646,0x4646,0x465F,0x4346,0x4643,0x465F,0x4646,0x4646,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4446,0x4646,0x465F,0x4646,0x4646
.word 0x4646,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4683,0x4646,0x465F,0x416F,0x427F,0x8F41,0x7F42,0x8F42,0x427F,0x7F41,0x8F42,0x418F,0x417F,0x427F,0x7F7F,0x8F41,0x7F42,0x427F,0x428F,0xD452,0xC4BF,0xD562,0xD462,0xD462,0xC4BF,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4C3,0xE4D4,0xE4E4,0xE4D3,0xE4F4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0x8F83,0x4142,0x428F,0x62D4,0x8F62,0x5242,0x8F62,0x7F42,0x417F,0xD562,0xD4D4,0x6252,0x52AF,0xC473,0xD4C3,0x5273,0x5252,0x7352,0x9383,0xD4D4,0xC4B3,0x8373,0xB3C4,0x83D4,0xB493,0xD483,0x8393,0x7363,0x8383,0x7373,0x8352,0xC3C4,0x93D4,0x7383,0xD493,0xC3D4,0x8383,0xC493,0xD4B3,0xB493,0x8383,0xB3D4,0x83B4,0x7373,0xC483,0x7393,0x5252,0x8373,0x8383,0x7373,0x7383,0x4283,0x5F46,0x4646,0x465F,0x465F,0x4646,0x4F42,0x5F46,0x5F46,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4646,0x465F,0x464F,0x465F,0x4646,0x4644,0x4646,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x4246,0x4646,0x4642,0x4646,0x464F,0x465F,0x465F,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x464F,0x465F,0x4642,0x4646,0x8342,0x4643,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x4646,0x4F42,0x5F46,0x5F46,0x5F46,0x4646,0x444F,0x464F
.word 0x5F46,0x5F46,0x4646,0x4246,0x4646,0x4646,0x4673,0x4642,0x6F5F,0x7F41,0x8F41,0x7F42,0x417F,0x427F,0x417F,0x428F,0x7F7F,0x4142,0x427F,0x7F7F,0x7F41,0x8F42,0x7F41,0x8F41,0x8F42,0x62D4,0x62D5,0xBFD5,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0x41D4,0x528F,0x52D4,0x419F,0xD58F,0x6262,0x8F8F,0x62D4,0x429F,0x428F,0x629F,0xD4D4,0x9FD4,0x5252,0x8373,0x83C4,0x7352,0xD493,0xC4C3,0xB483,0xB493,0x93B3,0x9373,0x83B4,0x73C4,0x8383,0x93B4,0x52B4,0x7373,0x5252,0x7352,0x9363,0xD4D4,0xE4B3,0x73D4,0x83C4,0xC493,0x8363,0x93B4,0x93B4,0x83C4,0x7373,0xB483,0x7383,0x5263,0x7363,0x7373,0x5252,0x7352,0x73B4,0x7373,0xB483,0x7383,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4646,0x4646,0x4246,0x4646,0x465F,0x465F,0x5F46,0x4446,0x4646,0x4646,0x4246,0x4646,0x465F,0x5F46,0x4446,0x4646,0x4646,0x4246,0x4F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4246,0x4646,0x4246,0x5F46,0x4646,0x5F46,0x4246,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4642,0x464F,0x4246,0x9373,0x4646,0x4644,0x4646,0x4646,0x4646,0x4642,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4246
.word 0x4646,0x4646,0x4246,0x4F46,0x5F46,0x4646,0x4646,0x425F,0x427F,0x427F,0x7F41,0x417F,0x427F,0x7F8F,0x7F41,0x7F42,0x8F41,0x7F42,0x7F41,0x7F41,0x427F,0x8F41,0x7F7F,0x8F42,0x9F52,0xAFC4,0xBF83,0x6283,0x62D5,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D3,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0x41D4,0x8F52,0xAF52,0x8F42,0x7F42,0x4142,0x429F,0x428F,0xD4AF,0x9FC4,0x8F41,0x527F,0x528F,0xD4D4,0x639F,0x7373,0x7352,0xB483,0xC483,0xC4B3,0x8373,0x8383,0x73B4,0x7352,0x8383,0x7363,0x7363,0x5252,0x7352,0xD483,0xB3D4,0xD4D4,0x7362,0x9383,0x93C4,0x52B4,0x8383,0x83B4,0x8373,0x7373,0x7383,0x6373,0x5273,0x5252,0x7373,0x8362,0x9383,0xC483,0xC483,0x8383,0xD4D4,0xD462,0xD4D4,0xD4D4,0x8372,0x8372,0x7262,0x4652,0x4646,0x4646,0x4642,0x4646,0x464F,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4644,0x4F46,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4246,0x4646,0x4646,0x5F46,0x4646,0x464F,0x465F,0x465F,0x4646,0x5F44,0x4646,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x7342,0x6393,0x4646,0x4646,0x4642,0x4246,0x4F46,0x4F46,0x4646,0x4642,0x5F46,0x4646,0x465F,0x4642,0x4646,0x465F,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x7F42,0x7F7F,0x4141,0x427F,0x7F7F,0x7F41,0x7F41,0x4142,0x417F,0x428F,0x7F8F,0x7F7F,0x427F,0x417F,0x427F,0x7F7F,0x7F41,0xD452,0xD462,0xD562,0xBFD4,0xD483,0xD4C4,0xC4BF,0xD4D4,0xD4D4,0xD4C3,0xE4D4,0xC3D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xD3E4,0xD4E4,0xD4D4,0x528F,0x429F,0x527F,0x529F,0x529F,0x528F,0x529F,0x429F,0x8F41,0x8F41,0x7F42,0x8F62,0x418F,0xD4AF,0x73D4,0x7352,0xC483,0x7383,0x8383,0x73B4,0x6352,0x5263,0x5252,0x5252,0x5252,0xD483,0xE4C3,0x93D4,0x6383,0x8383,0xC493,0x8393,0x7373,0x83B4,0x83C4,0x6263,0x7373,0x5273,0x6352,0x6373,0x5273,0x7352,0x8383,0xC483,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xE4D4,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xD4D5,0xD4D4,0x62D4,0x4262,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x4646,0x4246,0x4F46,0x4646,0x465F,0x4646,0x4646,0x4644,0x4646,0x4646,0x4642,0x4646,0x4246,0x5F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4246,0x4646,0x4F46,0x4646,0x4646,0x9352,0x43C4,0x4644,0x465F,0x5F46,0x4646,0x464F,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4246,0x464F
.word 0x5F46,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x424F,0x427F,0x417F,0x8F42,0x7F41,0x4241,0x7F7F,0x7F42,0x428F,0x427F,0x7F7F,0x4241,0x7F41,0x7F41,0x7F41,0x417F,0x427F,0x627F,0xAFD4,0xBFC4,0xAFC4,0xC483,0xC4BF,0xD4BF,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE441,0x41D4,0x427F,0x9F52,0x8F52,0x6242,0x62D5,0x52AF,0x9F42,0x9F52,0x4252,0x428F,0x418F,0x6252,0x8F52,0x8F52,0xD483,0x73D4,0x8383,0x6352,0x5273,0x5252,0xB383,0x93E4,0x7373,0xD4D4,0xB493,0x9373,0x93C4,0xC3D4,0x83C4,0xB483,0xC483,0x6383,0x5273,0x6352,0x5252,0xC483,0xB3D4,0x7383,0xD473,0xE4B2,0x83D4,0x8373,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD4D4,0xD4D4,0xD4D4,0x4172,0x4646,0x4646,0x4246,0x4646,0x464F,0x4646,0x4646,0x4642,0x5F46,0x4246,0x4646,0x4646,0x4646,0x4F42,0x4F46,0x4646,0x465F,0x4646,0x465F,0x4642,0x5F46,0x5F46,0x5F46,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x4642,0x465F,0x4646,0x465F,0x4646,0x464F,0x4646,0x4646,0x5242,0xA383,0x4663,0x4646,0x4646,0x4646,0x465F,0x4646,0x4446,0x4646,0x4646,0x5F46,0x5F46,0x5F46,0x4646,0x465F,0x464F,0x4646
.word 0x4646,0x4646,0x4646,0x4246,0x5F46,0x5F46,0x7F42,0x7F41,0x4241,0x7F8F,0x427F,0x8F7F,0x7F41,0x7F41,0x8F41,0x7F7F,0x4241,0x8F7F,0x7F42,0x7F7F,0x4142,0x7F7F,0x417F,0xBF52,0xD4C4,0xD562,0xD462,0xD562,0xD462,0xD462,0xD4D4,0xD472,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xE4D3,0xE4E4,0x41D4,0xE4D4,0x8F41,0x9F42,0xD562,0x9F52,0x8F41,0xD562,0xAF62,0x9F42,0x5252,0x529F,0x418F,0x8F42,0x9F8F,0x8F42,0x5252,0xD48F,0xD4D4,0x7373,0x8352,0xC493,0x7383,0x93C4,0xD4B4,0x5283,0xC493,0x83D4,0x8373,0x83B4,0xB483,0x6393,0x7373,0x6373,0x5252,0xC383,0xB3D4,0x7383,0xD483,0xD4C3,0x83D4,0x8373,0x93D4,0xB493,0xD483,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4C3,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xC4BF,0xD4AF,0xD462,0xD4D4,0xD4D4,0x4652,0x4F42,0x4F46,0x5F46,0x4646,0x4642,0x4646,0x4F46,0x4646,0x465F,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x464F,0x4246,0x464F,0x4646,0x4642,0x4646,0x4646,0x4642,0x8342,0xB493,0x4644,0x4642,0x465F,0x4642,0x4646,0x4246,0x5F46,0x4246,0x464F,0x465F,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646
.word 0x4646,0x4644,0x4644,0x4646,0x4646,0x426F,0x7F7F,0x7F42,0x8F41,0x7F42,0x7F41,0x7F41,0x7F42,0x427F,0x7F8F,0x4142,0x7F7F,0x8F41,0x417F,0x7F7F,0x7F41,0x7F42,0x9F52,0xD473,0xAF62,0xBFC4,0xD562,0x83BF,0xD4D4,0xC4BF,0xD572,0xD4D4,0xD4D4,0xD4E4,0x62D4,0xD4D4,0xC3D4,0xD4E4,0xD4E4,0xE4D3,0xD441,0x8FE4,0x4142,0x8F52,0x6241,0x428F,0x7F8F,0x628F,0x62D5,0x42AF,0x529F,0x529F,0x8F52,0x5252,0x9F42,0x5252,0x529F,0x7F8F,0xD4D4,0x73D4,0x6373,0x7373,0x5283,0xC473,0x8383,0xB452,0x9383,0x73C4,0x7373,0x7373,0x7373,0x8352,0xE493,0xD493,0xB4B3,0xC473,0xD483,0x83D4,0x9373,0x93C4,0x93D4,0x8373,0xB483,0xD462,0xC4BF,0xD4D4,0xE4E4,0xD4E4,0xE4D3,0xE4D4,0xE4D4,0xD4D3,0xE4E4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xBFD4,0xD4D4,0xD4D4,0x5272,0x4643,0x4646,0x4646,0x5F46,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4642,0x465F,0x4246,0x464F,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4246,0x5F46,0x4646,0x4F42,0x4646,0x4642,0x4646,0x4246,0x7363,0x4644,0x4646,0x4646,0x4646,0x465F,0x464F,0x465F,0x465F,0x6342,0xB493,0x4393,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x465F,0x4646,0x4646,0x4646,0x4642,0x465F,0x4646,0x4644,0x4246,0x5F46
.word 0x5F46,0x4646,0x465F,0x5F46,0x6F46,0x7F41,0x4241,0x418F,0x7F42,0x7F41,0x427F,0x7F8F,0x7F41,0x7F41,0x7F42,0x8F41,0x7F42,0x4142,0x7F7F,0x7F42,0x7F42,0x7F41,0xD452,0x62D5,0x62D5,0xC4D4,0x83AF,0x62D4,0xC4BF,0xD562,0xD4BF,0xD4D4,0xE4D4,0xD4D4,0xBF62,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0x41D4,0x41E4,0xAF52,0x9F52,0x427F,0x529F,0x5262,0x8F41,0xD541,0x6262,0x529F,0xAF52,0x9F52,0x9F42,0x8F41,0x9F52,0x9F52,0x4152,0x627F,0x83D4,0x5252,0x5273,0x5252,0x7373,0x5273,0x7373,0x7373,0x5273,0x5252,0x5252,0x8373,0x7373,0x93C4,0x93C4,0x93D4,0x8373,0x93C4,0xB493,0x8373,0x83B4,0xB493,0x7373,0x6273,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4C3,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4C3,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBF62,0xD4D5,0x4262,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x4646,0x4646,0x4646,0x464F,0x4646,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4F46,0x4646,0x4F42,0x4F46,0x4646,0x4646,0x9352,0x4663,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x9373,0xC493,0x4683,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4644,0x464F,0x465F,0x4646
.word 0x4646,0x4646,0x4646,0x4646,0x416F,0x7F7F,0x7F41,0x7F42,0x7F8F,0x427F,0x417F,0x427F,0x7F7F,0x7F42,0x7F41,0x427F,0x417F,0x428F,0x7F7F,0x7F7F,0x7F41,0x6242,0x62AF,0x62BF,0xBFD4,0x62C4,0x62D4,0xD4D5,0xD462,0xD4D4,0xD483,0x61D4,0xD4D4,0x62D4,0xC4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0x428F,0x42AF,0x428F,0x9F52,0x8F42,0xD552,0x9F62,0x9F42,0xAFC4,0x8F52,0x9F42,0x6252,0x529F,0x9F52,0x9F52,0x5252,0x529F,0x9F52,0x5252,0x6383,0xC483,0x7373,0x7383,0x7352,0x5252,0x5252,0x5273,0x8383,0x93C4,0xC3D4,0x5283,0xC483,0xB493,0xC483,0x8373,0xB483,0x7383,0x7373,0x7373,0x5273,0x6252,0xC4AF,0xD4BF,0xE4D4,0xD4E4,0xD4C3,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4C3,0xC3D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4AF,0x4642,0x4642,0x4646,0x4642,0x465F,0x4646,0x5F46,0x5F46,0x4246,0x464F,0x4646,0x4646,0x4646,0x465F,0x4246,0x464F,0x4F42,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4446,0x6246,0x6293,0x4646,0x464F,0x465F,0x4642,0x4646,0x4446,0x5246,0xC493,0x8383,0x4646,0x5F42,0x4646,0x4646,0x4646,0x4246,0x4646,0x4246,0x4646,0x5F46,0x5F46,0x5F46,0x4246,0x464F,0x4646,0x5F46,0x5F46
.word 0x4646,0x4642,0x465F,0x6F42,0x7F41,0x4241,0x417F,0x417F,0x7F42,0x7F41,0x8F41,0x7F7F,0x7F41,0x7F41,0x7F42,0x7F41,0x427F,0x418F,0x7F7F,0x4241,0x417F,0x528F,0xC4D5,0xBFC4,0x62C4,0xC4BF,0xC4BF,0xC4AF,0xC4BF,0xAFD4,0xD4D4,0xE562,0xD4D4,0xBFD4,0xAFC4,0x417F,0x417F,0x418F,0xE462,0xC3D4,0x9F62,0x8F52,0x8F42,0xAF52,0x6262,0x429F,0x9F8F,0x5262,0x418F,0x428F,0xD562,0x4241,0x529F,0x52AF,0x52AF,0x9F52,0x9F52,0x9F52,0x9F52,0x9F52,0x7352,0x7373,0x7383,0x9373,0x5283,0x7383,0xC483,0xB4C3,0x9373,0x83C4,0xD4C4,0x6383,0x8383,0x83B4,0x8383,0x7363,0x7373,0x5252,0x5252,0x5252,0x7373,0xAF52,0x62C4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xC3D4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xC3D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x4261,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4642,0x465F,0x4644,0x4646,0x465F,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4642,0x4646,0x5F46,0x5F46,0x4646,0x4642,0x4246,0x8373,0x4652,0x4646,0x5F46,0x4F46,0x5F46,0x5F46,0x9344,0x8383,0x73B4,0x4646,0x4646,0x4646,0x4642,0x4446,0x4646,0x5F46,0x4646,0x464F,0x4646,0x4642,0x4646,0x464F,0x4646,0x465F,0x4646,0x4646
.word 0x4646,0x4646,0x5F46,0x426F,0x7F7F,0x8F42,0x7F41,0x7F42,0x7F7F,0x7F7F,0x7F42,0x427F,0x7F7F,0x7F42,0x7F41,0x8F42,0x417F,0x7F7F,0x7F42,0x417F,0x7F42,0xD4C4,0x62BF,0x62D4,0x62D5,0x62D5,0x62D5,0x62D4,0x62D5,0xC4D4,0xD4D4,0xD4AF,0xD4D4,0x52D4,0xD462,0x8F41,0x7F42,0x6252,0xD4E5,0xE4D4,0xD4D4,0x8F41,0x417F,0x6242,0xAFD5,0x8F42,0x4241,0x8F8F,0x8F42,0xD452,0xE4E4,0xE4E4,0x8FC4,0xD4D4,0x8F42,0x9F52,0xAF73,0x5252,0x529F,0x529F,0x8F9F,0x5252,0x5273,0x6373,0x6383,0x7383,0x8383,0x6383,0xB483,0x8383,0xB493,0x5273,0x7373,0x7373,0x5263,0x5252,0x9383,0x8363,0xC493,0x8393,0x5252,0xC4AF,0xC4BF,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D3,0xE4E4,0xE4E4,0xE4D3,0xD4E4,0xD4D3,0xD4E4,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4D4,0xD4C3,0xC3E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x4641,0x4646,0x465F,0x4246,0x5F46,0x4446,0x4646,0x4646,0x4F46,0x4646,0x4F46,0x5F46,0x4646,0x4646,0x5F46,0x5F46,0x5F46,0x4F46,0x4646,0x4F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x7342,0x52A3,0x4646,0x4646,0x4646,0x4646,0x4646,0x8383,0x93D4,0x4683,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x5F46,0x4646,0x465F,0x4246,0x464F,0x4646,0x465F,0x4646,0x4646,0x4246,0x5F46,0x4246
.word 0x4246,0x4646,0x7F46,0x7F42,0x4141,0x427F,0x7F7F,0x7F41,0x7F41,0x4142,0x7F41,0x417F,0x417F,0x417F,0x417F,0x427F,0x7F7F,0x7F42,0x7F7F,0x417F,0x418F,0x839F,0xAFC4,0xAFC4,0xBFC4,0xD462,0xC4AF,0x62BF,0xAFD4,0x62D4,0xD4D4,0xD462,0xE4D4,0xAF41,0x627F,0xD4D5,0xE4D4,0xE4D4,0xD4D4,0x41D4,0x428F,0x527F,0x9F52,0x8F8F,0x427F,0x528F,0xAF62,0x9F73,0x9F42,0x41D4,0x528F,0x8F7F,0x427F,0xE441,0xD4E4,0x8F41,0x427F,0x529F,0x52AF,0x9F62,0x4152,0x5242,0x5252,0x5252,0x7352,0x7352,0x6363,0x5252,0x7373,0x7373,0x6373,0x7352,0x5252,0x5252,0x8373,0xC3C4,0xD4E4,0x8373,0x83C4,0x5273,0xD552,0xC462,0xD462,0xD4D4,0xE4D4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4C3,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD462,0x5262,0x4646,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x465F,0x4642,0x465F,0x5F46,0x4646,0x4246,0x9383,0x4643,0x465F,0x4646,0x4644,0x4646,0x9383,0xB4D4,0x4673,0x4642,0x465F,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x465F,0x464F,0x465F,0x4646
.word 0x4646,0x465F,0x415F,0x7F7F,0x7F41,0x7F41,0x7F41,0x7F7F,0x7F7F,0x427F,0x7F7F,0x7F42,0x7F7F,0x7F42,0x427F,0x417F,0x7F7F,0x417F,0x417F,0x427F,0xC47F,0xBF52,0x62C4,0x62D4,0x62D4,0x62D5,0x62D4,0x62D5,0x83D5,0xC4BF,0xC4BF,0xD462,0x6241,0x83D5,0x62D5,0xC4BF,0xD4D4,0xD4D4,0x6262,0x428F,0x7FD4,0xAF52,0xC4C4,0x8F62,0x4142,0x628F,0xD4D5,0x9F62,0xC452,0x527F,0xE4D4,0xE4D4,0x52D4,0x628F,0x8F7F,0x7F42,0x8F42,0x8F42,0x9F52,0x5252,0x529F,0x8F8F,0x5242,0x5252,0x5252,0x5252,0x5252,0x5273,0x9383,0xD4E4,0x83B3,0x9352,0x93C4,0x93D4,0x8373,0xC483,0x8393,0x8352,0x5273,0xAF52,0xAF62,0xD462,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xD4D3,0xE4D4,0xD4D4,0xD4C3,0xE4C3,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xC3E4,0xD4E4,0xE4D4,0xD4E4,0xBFD4,0x4262,0x4646,0x4646,0x465F,0x465F,0x464F,0x464F,0x4642,0x4646,0x4246,0x4646,0x4646,0x4246,0x464F,0x465F,0x4642,0x465F,0x4646,0x4246,0x4646,0x4646,0x7343,0x4644,0x4646,0x4646,0x4646,0x4646,0xB373,0x4383,0x4246,0x5F46,0x5F46,0x4446,0xB483,0x8393,0x4673,0x4646,0x4646,0x464F,0x4246,0x4646,0x4642,0x4646,0x4644,0x465F,0x4646,0x464F,0x465F,0x464F,0x4646,0x4646,0x4646,0x465F
.word 0x4646,0x6F46,0x417F,0x417F,0x427F,0x7F7F,0x7F42,0x7F42,0x427F,0x8F41,0x7F41,0x417F,0x7F7F,0x4241,0x417F,0x427F,0x417F,0x7F7F,0x7F7F,0x7F42,0xD441,0x62D5,0xBFD4,0xBFC4,0xBFC4,0xD562,0xC4AF,0xC4BF,0xC4BF,0xD462,0xD4D4,0xAFD4,0xD4D4,0xAFD4,0xD4C4,0xD462,0xD4D4,0xE4E4,0x9F62,0xD452,0x8FD4,0x6241,0x9FD5,0x428F,0x529F,0x8F52,0x418F,0x8F52,0xBF52,0xD4D4,0xE4E4,0xD4E4,0x41E4,0x8F7F,0x7F42,0x7F41,0x8F41,0x8F41,0x8F42,0x9F42,0x9F42,0x7F52,0x4241,0x5241,0x5252,0xC483,0x8393,0x5283,0x83B4,0xB483,0xC483,0xC473,0xC493,0xB483,0x8352,0x83B4,0x73B4,0x6363,0x5252,0xD562,0xD562,0xBFD4,0xD4D4,0xD4D4,0xC3E4,0xE4E4,0xD4E4,0xE4E4,0xD3E4,0xE4D4,0xD3D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xC3D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D4,0xD4D4,0x6262,0x4652,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4F46,0x5F46,0x4F46,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4646,0x465F,0x5246,0x4273,0x4246,0x4646,0x465F,0x4642,0x8342,0x73D4,0x4646,0x4646,0x4646,0x4646,0x9383,0x83B4,0x4673,0x5F46,0x4646,0x4646,0x4646,0x445F,0x4673,0x465F,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646
.word 0x5F46,0x7F7F,0x7F41,0x427F,0x7F7F,0x417F,0x7F7F,0x7F7F,0x7F41,0x7F7F,0x427F,0x7F7F,0x7F42,0x417F,0x427F,0x8F7F,0x427F,0x427F,0x7F7F,0x8F41,0xD452,0xD462,0x62D4,0x62D5,0x62D5,0x62D4,0x62D4,0x62D4,0xD4D4,0xC4BF,0xD462,0xC4D4,0xD462,0xD4D4,0xBF62,0xAFC4,0xAF83,0xD452,0xD4D4,0x7FD4,0x41D4,0x528F,0x417F,0x9F42,0xD462,0x62D4,0x8F52,0xAF42,0xC462,0xE4E4,0xE4D4,0xC3D4,0x8F62,0x7F42,0x8F41,0x8F42,0x7F42,0x8F42,0x4241,0x529F,0x428F,0x429F,0x8F8F,0x7352,0x5242,0x5252,0x7363,0x73C4,0x7373,0x7383,0x7373,0x8352,0x73B4,0x7373,0x7352,0x6352,0x5252,0x5273,0xAF62,0xAF62,0xAFC4,0xC483,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4C3,0xE4E4,0xE4D4,0xD3D4,0xD4E4,0xD4E4,0xD4E4,0x51D4,0x4246,0x4646,0x4642,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x464F,0x464F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4246,0x4246,0x7352,0x4642,0x4646,0x4646,0x4646,0x5246,0x9383,0x4652,0x4246,0x4646,0x635F,0xC493,0x8383,0x4273,0x4646,0x4646,0x4644,0x465F,0x8363,0x4644,0x4646,0x4646,0x4646,0x4446,0x5F46,0x4446,0x4646,0x4646,0x465F,0x465F,0x4642
.word 0x7F46,0x4241,0x416F,0x7F7F,0x4141,0x427F,0x7F7F,0x427F,0x416F,0x7F42,0x7F41,0x7F7F,0x7F7F,0x7F41,0x417F,0x427F,0x417F,0x7F7F,0x427F,0x427F,0xC4AF,0xD4AF,0xD562,0xD462,0xC4BF,0xC4BF,0xC4BF,0xD4AF,0xD562,0xD462,0xAFD4,0xD4C4,0xD4AF,0xD4D4,0xC4D4,0xAF62,0xD452,0xD4D4,0x62E4,0x628F,0x429F,0xAF52,0x8F42,0x8F41,0x417F,0x8F9F,0x527F,0x73AF,0xC4AF,0x8F62,0x5252,0x8F8F,0x7F42,0x428F,0x428F,0x418F,0x428F,0x428F,0x528F,0x528F,0x418F,0x528F,0x4162,0x629F,0x52C4,0x5252,0x5252,0x5252,0x5252,0x7363,0x5252,0x5273,0x5273,0x5252,0xC483,0x93C3,0x5283,0x5252,0x73AF,0x62D5,0x83D5,0xD4BF,0xD4D4,0xE4E4,0xD4E4,0xD3E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD3D4,0xE4E4,0xD4C3,0xE4E4,0xD4C3,0xE4E4,0xC3D4,0xE4E4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4C3,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0x4641,0x465F,0x4F46,0x4246,0x4646,0x4646,0x4644,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4F42,0x4246,0x5F46,0x4246,0x4646,0x4646,0x4646,0x5F46,0x8344,0x4673,0x4246,0x5F46,0x4646,0x4642,0xC483,0x4683,0x465F,0x465F,0x7346,0x83C3,0x93B4,0x4473,0x4646,0x4642,0x465F,0x6346,0x4483,0x4246,0x4646,0x4642,0x4246,0x4F46,0x4646,0x5F46,0x4646,0x4642,0x4246,0x5F46,0x4646
.word 0x7F7F,0x7F41,0x7F41,0x7F42,0x7F42,0x7F7F,0x427F,0x7F6F,0x7F7F,0x417F,0x7F7F,0x427F,0x7F7F,0x427F,0x427F,0x7F7F,0x427F,0x7F7F,0x417F,0x417F,0xD4C4,0xD562,0xC4BF,0x83BF,0xD562,0xD462,0xD562,0xD462,0x62BF,0xBFD4,0xD4C4,0x62D4,0x62D4,0xD4D4,0xAFD4,0xD4D4,0x62D4,0x628F,0xAFD4,0xD4D4,0x9FC4,0x5252,0x528F,0xD562,0x62D4,0x7F42,0x5252,0x428F,0x419F,0xD49F,0xE4E4,0xE4D4,0x41D4,0x417F,0x418F,0x8F42,0x8F42,0x8F41,0x4241,0x429F,0x428F,0xE48F,0x62D4,0x9F42,0x7362,0x73C4,0x5252,0x5252,0x5252,0x7352,0x5273,0x8393,0x8383,0x5283,0x8383,0x83B4,0x5273,0xAF52,0xAFC4,0xD462,0xD4BF,0xD4D4,0xD4D4,0xE4D4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E3,0xC3E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xC3E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0x52D4,0x4646,0x4646,0x4F46,0x5F46,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4F46,0x4646,0x4F46,0x5F46,0x5F46,0x5F46,0x4646,0x5246,0x7393,0x4646,0x4646,0x4646,0x5F46,0x8374,0x6393,0x4646,0x4646,0x8346,0xB493,0x7393,0x4373,0x4646,0x465F,0x4646,0x8342,0x4383,0x4646,0x4F46,0x5F46,0x4646,0x4646,0x4246,0x4646,0x465F,0x464F,0x4646,0x4646,0x5F46
.word 0x417F,0x427F,0x417F,0x417F,0x417F,0x417F,0x417F,0x7F7F,0x7F41,0x7F41,0x427F,0x7F6F,0x7F41,0x7F41,0x417F,0x418F,0x7F7F,0x7F7F,0x7F42,0x9F52,0x62D4,0x62D4,0xD4D4,0xD562,0x83BF,0xC4BF,0xC4AF,0xC4BF,0xD562,0xD462,0xD462,0xD4D4,0xD4D4,0xD49F,0x62D4,0x41D4,0xD562,0xD462,0xD4D4,0xD452,0xD4D4,0x7F8F,0xAF52,0xD4C4,0x9F62,0x528F,0xD4AF,0x62D4,0x528F,0xE462,0xE4D4,0xE4E4,0x7FE4,0x7F42,0x7F42,0x8F41,0x4142,0x428F,0x528F,0x429F,0x418F,0x527F,0xD5E4,0x529F,0x629F,0xD452,0x5283,0x7352,0x7352,0x8383,0x52C4,0x7373,0x7373,0x7373,0x7373,0x7373,0x5252,0x73AF,0xD562,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xC3E4,0xD4E4,0xD4D4,0xC3E4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xC3E4,0xE4D4,0xD4D4,0x62D4,0x4662,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x5F46,0x6346,0x7383,0x4642,0x465F,0x5F46,0x4646,0x8344,0x9383,0x4646,0x4246,0xB442,0x8393,0x93B4,0x4383,0x4646,0x4646,0x4646,0x9373,0x4673,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x4646
.word 0x7F7F,0x417F,0x7F7F,0x8F42,0x7F7F,0x427F,0x7F7F,0x7F42,0x427F,0x7F6F,0x7F7F,0x7F7F,0x7F6F,0x427F,0x427F,0x7F7F,0x4241,0x417F,0x8F52,0x62C4,0xD4D5,0xC4BF,0xD4AF,0x72D4,0x62D5,0x62D5,0x62D4,0xD4C4,0xD4AF,0xAFD4,0xBFC4,0xD462,0xC4BF,0x62D4,0xD4D4,0xD49F,0xD49F,0xC4BF,0xD462,0xBFD4,0xD462,0x8F41,0x427F,0x417F,0x429F,0xD552,0xD4D4,0x7FD4,0xD552,0xD4D4,0xE4E4,0xD4E4,0x41E4,0x417F,0x417F,0x8F42,0x4241,0x7F8F,0x8F42,0x8F42,0x8F42,0x7F42,0xD48F,0x52D4,0x629F,0x629F,0x93C4,0x5263,0x5263,0x7363,0x5273,0x5273,0x5252,0x7352,0xC483,0x5293,0xAF62,0xBF73,0xBFC4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D3,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xE4E4,0xC3D4,0xE4D4,0xE4C3,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xC3E4,0xE4D4,0xD4D4,0xC3D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x52BF,0x4646,0x4644,0x465F,0x4F42,0x5F46,0x4646,0x465F,0x4646,0x4246,0x4646,0x4646,0x4642,0x4646,0x465F,0x465F,0x465F,0x465F,0x4646,0x4646,0x9373,0x4673,0x4646,0x4646,0x4646,0xB442,0xB493,0x4652,0x4646,0x9363,0x93B4,0xB483,0x4383,0x4246,0x4646,0x4242,0x9393,0x4673,0x4642,0x4246,0x4646,0x4642,0x465F,0x465F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x465F
.word 0x7F41,0x7F42,0x4142,0x7F7F,0x7F42,0x4241,0x7F8F,0x417F,0x7F7F,0x7F41,0x7F42,0x427F,0x417F,0x417F,0x417F,0x427F,0x7F7F,0x527F,0x41D4,0x9FD4,0xD462,0xD4C4,0x6283,0x62D5,0xBFD4,0xBFC4,0xD483,0xC4AF,0x62D4,0xD4D4,0xD462,0x62D5,0xD4D4,0xD4D4,0xC4AF,0x62D4,0x6262,0xD5D4,0xD4BF,0xD4D4,0x52D4,0x41D5,0xAF42,0x62C4,0x8F8F,0x7F7F,0x8F41,0x8F52,0x9F41,0xD483,0xE4E4,0xE4E4,0x7FD4,0x7F42,0x7F42,0x428F,0x7F8F,0x4142,0x5241,0x529F,0x4141,0x428F,0x528F,0x62E4,0x528F,0x52D5,0xC462,0x52D4,0x5252,0x5252,0x5252,0x5273,0x6352,0xC483,0x8373,0x9F63,0x6252,0x62D5,0xD4D5,0x83D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xD4D3,0xE4D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD3D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD3E4,0xE4E4,0xD4E4,0xC3E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0x62D4,0x4652,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4F46,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4446,0x4646,0x4642,0x8352,0x6383,0x4446,0x4646,0x4644,0x8346,0x83C4,0x4673,0x4242,0x9383,0xB483,0x8383,0x43B4,0x4646,0x4646,0x7346,0xB493,0x4663,0x5F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4642,0x4646,0x4246
.word 0x417F,0x417F,0x417F,0x417F,0x527F,0x8F41,0x417F,0x427F,0x7F7F,0x427F,0x417F,0x7F7F,0x7F6F,0x427F,0x7F7F,0x7F41,0x7F52,0x8F52,0xAFC4,0xAF83,0xBFC4,0xD4D4,0xD4BF,0xBFD4,0xD4C4,0xD483,0xD4AF,0xBFC4,0xBFC4,0xBF83,0xD4C4,0xD462,0xBF62,0xD4C4,0xD483,0xC4BF,0xAFD4,0x62D4,0x62C4,0xD4D4,0xD4D4,0x9FD4,0x6252,0xAFD4,0x4142,0x6262,0x8FD4,0x5241,0xD462,0x42AF,0x6251,0x428F,0x6241,0x8F8F,0x417F,0x417F,0x417F,0x428F,0x428F,0x418F,0xAF52,0x4152,0x7F8F,0xD4D4,0x8F52,0xBFD4,0x5262,0xD462,0x5283,0x5252,0x7383,0x7383,0x8383,0x8373,0x5252,0x6252,0xD5AF,0xD462,0xD472,0xD4D4,0xD4D4,0xE4E4,0xC3D4,0xE4D4,0xE4E4,0xE4C3,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xC3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0x4262,0x4646,0x4646,0x5F46,0x4246,0x4F46,0x4646,0x5F46,0x4646,0x4642,0x5F46,0x4646,0x5F46,0x5246,0x4463,0x4646,0x5F46,0x4646,0x5F46,0x8354,0x73B4,0x5F46,0x5F46,0x5F46,0x8346,0x8383,0x4673,0x7343,0x7393,0x93D4,0xB3C4,0x4383,0x4646,0x465F,0x9352,0x83D4,0x4663,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4F46,0x5F46,0x4646,0x5F46,0x4646
.word 0x427F,0x417F,0x427F,0x7F7F,0x9F52,0x8F52,0x7F42,0x417F,0x417F,0x8F42,0x7F41,0x4152,0x427F,0x7F7F,0x7F7F,0x7F7F,0x8F62,0x9F52,0xD4C4,0xD48F,0xD4C4,0xD462,0xD4C4,0xD4D4,0xD462,0xC4AF,0x62C4,0x83D4,0x62D4,0xD4D5,0xBF62,0xBFD4,0xD4C4,0xBF62,0xD4D4,0xBFC4,0xD462,0xD4D4,0xAFD4,0xBFC4,0xD4D4,0x62D4,0x529F,0x52AF,0x528F,0xD4D5,0x8F62,0xAF52,0xD4D5,0x8FD4,0x428F,0xD48F,0xD4E4,0x7F41,0x7F42,0x7F42,0x7F42,0x8F42,0x8F41,0x9F52,0x5252,0x428F,0x417F,0xE47F,0x8F62,0xE452,0xD552,0x638F,0x7393,0x5252,0x7373,0x5283,0x5263,0x5273,0x5252,0x73AF,0xBF62,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xC3D4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xC3D4,0xE4E4,0xE4E4,0xE4E4,0xD3D4,0xD4E4,0xD3D4,0xE4E4,0xD4E4,0xE4E4,0xF4E4,0xE4E4,0xD4E4,0xE4F4,0xE4D4,0xD4E4,0xD4C3,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xAFD4,0x4646,0x465F,0x4642,0x465F,0x4646,0x464F,0x4642,0x4646,0x464F,0x4646,0x4246,0x4646,0x4446,0x5283,0x4642,0x4646,0x4646,0x4646,0x9344,0x8393,0x4643,0x4646,0x4646,0x9352,0x83C4,0x4373,0x8352,0x83B4,0x83C4,0x83C4,0x6373,0x4646,0x4646,0x9373,0x93B4,0x4643,0x4642,0x4646,0x465F,0x4F42,0x4246,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646
.word 0x417F,0x427F,0x7F7F,0x427F,0x5262,0x42AF,0x527F,0x627F,0x427F,0x419F,0x7F7F,0x5252,0x417F,0x427F,0x427F,0x417F,0x41C4,0x62D4,0xD4D4,0x62C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4BF,0xD4BF,0xBFD4,0xD583,0xD462,0xD4C4,0xD4D4,0xD562,0xBF62,0xC4D4,0x62D5,0xD4AF,0xD4D4,0x83D4,0xD462,0xD4D4,0x8FC4,0x8F52,0x8F42,0x8F7F,0x428F,0x427F,0x418F,0x427F,0x417F,0xE4D4,0xD4E4,0x8F41,0x7F41,0x7F41,0x8F41,0x7F41,0x4142,0x9F42,0x8F42,0x8F42,0x7F41,0xC442,0x41D4,0xE452,0xD48F,0x8F41,0x8342,0x5252,0x5252,0x5252,0x5252,0x5252,0x5252,0xBF62,0xC4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0xD4C3,0xE4D4,0xD3D4,0xE4D4,0xD4D4,0xE4C3,0xE4E4,0xE4E4,0xE4E4,0xE4C3,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4C3,0xD4E4,0xD4D4,0xD4D4,0xC3E4,0xD4D4,0xD4D4,0x83D4,0x4651,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4642,0x464F,0x4646,0x4646,0x7342,0x7393,0x4246,0x5F46,0x4646,0xB442,0x7393,0x4643,0x4642,0x4246,0x9373,0x83B4,0x7383,0x8383,0x9383,0x83C4,0xC493,0x8373,0x4246,0x5246,0x9393,0x73B4,0x4646,0x4646,0x4242,0x4646,0x4646,0x4F46,0x4646,0x4446,0x5F46,0x4246,0x4646,0x465F,0x4246
.word 0x7F41,0x8F52,0x7F7F,0x5241,0x52AF,0x9F52,0x4152,0xAF52,0x9F52,0x4262,0x418F,0xBF52,0x527F,0x417F,0x7F41,0x7F7F,0x8FD4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4BF,0xD4D4,0xD4C4,0xD462,0xBF62,0xAF62,0x62D4,0xC4BF,0xD4BF,0xC462,0xD462,0xD462,0xD4D4,0xE462,0xE4D4,0xBFD4,0xBF83,0x9FC4,0x8F52,0x5241,0xC462,0x418F,0x9F52,0x6283,0x418F,0x9FC4,0xE4D4,0xE4E4,0x8F62,0x7F42,0x7F42,0x7F42,0x7F42,0x8F8F,0x428F,0x528F,0x418F,0x427F,0x418F,0x8FE5,0xD442,0x6252,0x429F,0x838F,0x4283,0x7352,0x5283,0x7383,0x5252,0x62AF,0x62D5,0xD4D5,0xD4D4,0xD4D4,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D3,0xE4E4,0xE4E4,0xD4E4,0xE3E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D3,0xD4D4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xC3D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x5262,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4F46,0x4646,0x4646,0x635F,0x4644,0x4246,0x8393,0x4663,0x4646,0x4646,0xB442,0xC483,0x4673,0x5F46,0x7346,0xC493,0x7383,0x9373,0x83B4,0x93B4,0x93B4,0x93B4,0x7373,0x4652,0x8342,0x93B4,0x6383,0x4246,0x4246,0x4683,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x465F,0x4246,0x4646
.word 0x527F,0x418F,0x527F,0x628F,0xC4AF,0x529F,0x52AF,0x62D5,0xD552,0xAF62,0x7F42,0x62C4,0x627F,0x7F7F,0x5252,0x4241,0x52D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xD4D4,0xD572,0xD4D4,0xD4D4,0xC4D4,0x629F,0x83D5,0xBFC4,0xD4D4,0xD4AF,0xC4AF,0xD4D4,0xD4D4,0xD562,0xD4D4,0xC4D4,0xD462,0x8F52,0xD552,0xD4D4,0x527F,0xD4AF,0xD5D4,0x7F8F,0x41D4,0xD4D4,0x8F41,0x62C4,0x7F8F,0x6241,0x42AF,0x417F,0x427F,0x417F,0x8F41,0x4252,0x419F,0x8F42,0x52E4,0x529F,0x529F,0x41D4,0x8F52,0x52D4,0x5252,0x6373,0x5273,0x9F52,0xBF73,0xBFC4,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xC3D4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xD4C3,0xD4C3,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0x62E4,0x4642,0x4646,0x4246,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x5F46,0x4646,0x6346,0x4252,0x4646,0x9373,0x4273,0x4246,0x5F46,0x8374,0x8383,0x4673,0x4646,0x9352,0x83C4,0x7383,0x73D4,0x7383,0xB493,0x8383,0xC483,0x7383,0x4673,0x9373,0xB493,0x4383,0x4246,0x8383,0x4663,0x4642,0x4642,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x4646,0x464F
.word 0x8F62,0x4142,0x9F42,0x6273,0xAF62,0xAF73,0xAF52,0xD562,0x83AF,0x62D5,0x418F,0xAFD4,0x737F,0x8F52,0x9F62,0x8F52,0xAFD4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xBFD4,0xD483,0x62D4,0xD4D4,0x62D4,0xD4AF,0xD462,0xD462,0x62D4,0x62D4,0x62BF,0xD4D4,0xD4D4,0xC451,0xD4D4,0xD4D4,0x42BF,0x629F,0xAFD4,0x5241,0xD562,0x5272,0x428F,0x7FD4,0x8F62,0xE462,0xE4E4,0x8F62,0x427F,0xBF62,0x8F42,0x7F42,0x7F41,0x8F42,0x418F,0x8F42,0x528F,0x52E4,0x528F,0x529F,0x418F,0x429F,0xD462,0x5273,0x5252,0x4252,0x629F,0x62D5,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xE4D4,0xE4D4,0xE4D4,0xE4C3,0xE4D4,0xC3D4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD3D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4C3,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xC3D4,0xE4E4,0xD4D4,0xD4D4,0x46AF,0x4642,0x4F46,0x4646,0x4646,0x4642,0x4646,0x465F,0x4646,0x4446,0x4246,0x7373,0x4246,0x8344,0x7383,0x4646,0x4446,0x9373,0x93B4,0x4663,0x735F,0xD493,0x7383,0x9373,0x83C4,0xB483,0xD483,0xB483,0x9383,0x93C4,0x6373,0x9373,0x83C4,0x4383,0x8363,0x7383,0x4644,0x4F46,0x4646,0x4246,0x4646,0x4646,0x4F46,0x4F46,0x4646,0x465F,0x4646
.word 0x8F62,0x9F52,0x6252,0xD5AF,0xD562,0xC452,0xD562,0x62D4,0x9FD4,0x62D4,0x528F,0x62D4,0xD48F,0x42AF,0xC4AF,0x52D4,0x62D4,0xD4E4,0xE4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xD4BF,0x62D4,0xD5D4,0xAF83,0xAF52,0xD4D4,0x62D4,0xD4D5,0xD4C4,0x83BF,0xC4BF,0xD4D4,0xAFD4,0xD462,0xD4D4,0x8F41,0x427F,0x8F8F,0x8F41,0x8F8F,0xAF52,0xD583,0x5241,0xD4D5,0xE4D4,0x42E4,0x8F41,0x428F,0x62D4,0x7F9F,0x8F42,0x427F,0x8F7F,0x8F42,0x8F42,0x62E4,0x628F,0xD4D4,0xD4D4,0xE4D4,0x8FD4,0xD4C4,0x5252,0x5252,0xAF62,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4C3,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xC3E4,0xD3D4,0xE4D4,0xD4E4,0xD4E4,0xE4E3,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x5283,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4642,0x5F46,0x4646,0x8342,0x4673,0x7346,0x73B3,0x4646,0x735F,0xB483,0xB483,0x4643,0x9373,0x93C4,0x7373,0xB3C4,0x8373,0x93B4,0xB483,0x7393,0xB493,0xB483,0x8373,0xD493,0xC483,0x6383,0xC493,0x4383,0x4246,0x4646,0x4646,0x465F,0x4646,0x4642,0x4646,0x4646,0x4642,0x4646,0x4642
.word 0xAF62,0x8FC4,0xAF62,0xAF73,0xAFC4,0xBF62,0xAFC4,0xBFC4,0xC462,0xD4D4,0xAF41,0x62C4,0xD48F,0x7F62,0xD462,0xAFD4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xD3D4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4C4,0xD4D4,0xD4BF,0xD4D4,0x62D4,0x628F,0xD4D4,0xD4AF,0xD462,0xD562,0xD462,0xD4BF,0xD4D4,0xD4D4,0xAF62,0x8F73,0x8F52,0x6252,0x62D5,0x528F,0xD562,0x6241,0x8FBF,0x6252,0xD4E4,0xD562,0x41D4,0x8F7F,0xC47F,0x62D4,0x428F,0x418F,0x427F,0x417F,0x527F,0x52E4,0xD49F,0x9FD4,0xE4E4,0x8F52,0xD4C4,0x51D4,0x52D4,0xAF41,0xD573,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xC3D4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D3,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4C3,0xE4D4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xAFD4,0x4642,0x4246,0x4646,0x4646,0x5F46,0x5F46,0x5F46,0x4F46,0x4646,0x4646,0x7346,0x5293,0x7346,0x73D4,0x4642,0x7343,0x8393,0x8393,0x6243,0xB383,0x83B4,0x83B4,0xB483,0x8373,0xD483,0x7383,0x7383,0x83C4,0x8383,0x8373,0x83C4,0x83B4,0xB483,0xB493,0x4673,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4246,0x4646,0x464F,0x465F,0x464F
.word 0x6262,0x73AF,0xC4AF,0x62D5,0x62D4,0x62D5,0xC4D4,0xD562,0xBFD4,0xD4C4,0xD452,0xD4D4,0xD4C4,0x8F62,0xD4D5,0xD4D4,0xE4D4,0xE4D4,0xD3D4,0xE4D4,0xE4E4,0xE4F4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD562,0xD4D4,0x419F,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x62C4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0x4141,0xC4AF,0xD4D4,0x527F,0xD4D5,0x42BF,0x62C4,0x8FD5,0x6241,0xD48F,0xE4E4,0x8F83,0x7F41,0x428F,0xD462,0x8F62,0x7F42,0x7F41,0x4142,0x42E4,0x5241,0x41D4,0xD462,0x4141,0x418F,0xD5D4,0xD4D4,0xAFC4,0xD462,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4C3,0xE4E4,0xD4E4,0xD4E4,0xC3E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xD4D3,0xE4D4,0xD4E4,0xC3D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4C3,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0x4662,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x4246,0x5F46,0x7346,0x7393,0x8363,0x7393,0x4373,0x8373,0x83B4,0x73B4,0x9373,0xB4B3,0xB493,0x8383,0x8393,0xB483,0x8373,0x83B4,0x8383,0x93C4,0x83D4,0x93B4,0x8383,0xB483,0x8383,0x83B4,0x4644,0x4642,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646
.word 0xC4AF,0xAF62,0x62C4,0xC4BF,0xC4AF,0xD4BF,0xBF62,0x62D4,0x83D4,0xD4BF,0xD4D4,0xD4D4,0xAFD4,0x42D4,0xD4AF,0xD4D4,0xD4D4,0xD4C3,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4F4,0xE4E3,0xE3E4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xBFD4,0xD483,0xD4C4,0x8F62,0xBF62,0xD562,0xD4BF,0xAFD4,0xBFC4,0xD562,0xD462,0xD4D4,0x8F62,0xAF52,0xD4D4,0x528F,0xD4AF,0x9F83,0x628F,0x62AF,0x8FD5,0x417F,0xD452,0x62E4,0x427F,0x8F7F,0x418F,0xD4D5,0x8F62,0x8F42,0xD552,0x8FE4,0xAF42,0x8F62,0xD452,0x429F,0x4241,0xD48F,0x62D4,0xD4D4,0xD4D5,0xD4D4,0xD4D4,0xD4E4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xD4E4,0xD4E4,0xE4D4,0xE4C3,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD3D4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4F4,0xD4E4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4C3,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x42D4,0x4642,0x4646,0x4646,0x4642,0x4446,0x5F46,0x4646,0x4F46,0x4646,0x5246,0x8393,0xB483,0x7383,0x7363,0x9383,0x83C4,0x7383,0xD483,0x8383,0x83B4,0xB483,0xB493,0x7383,0x8383,0xB483,0xB483,0xC483,0xB493,0xB483,0xB493,0x8373,0xC493,0x6383,0x4646,0x5F46,0x4246,0x4642,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4642,0x5F46
.word 0xAF62,0x62C4,0x62D5,0x62D5,0x62D4,0xC4D4,0xC4D4,0xD4D4,0xD4BF,0xD4C4,0xD462,0xD4D4,0xD4D4,0xD462,0xD462,0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4F4,0xE3E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFD4,0xD562,0xD4D4,0x62D4,0x529F,0xD462,0xD4D4,0xD4D4,0xBF62,0xBFC4,0xAFC4,0x62C4,0x428F,0x417F,0x417F,0x527F,0x429F,0x428F,0x8F52,0x5252,0xAF62,0xAF62,0xD573,0xBF62,0x6283,0x72D5,0xD4D4,0xD4D4,0x7F8F,0x838F,0x41D4,0xD49F,0x9F52,0xE442,0x9F42,0x9F42,0x4141,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D3,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4C3,0xE4E4,0xD4E4,0xE4E4,0xC3E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD472,0xD4D4,0xC3D4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0x51D4,0x4646,0x4646,0x465F,0x5F46,0x4F46,0x4646,0x4246,0x4646,0x4646,0x8342,0x83B4,0x8383,0x73B4,0x8363,0xC493,0x7383,0xB483,0xB4B3,0xB493,0x8383,0xC493,0xB493,0x8373,0x83C4,0x8373,0x8383,0xB483,0x7383,0x8383,0x83C4,0xB483,0x8383,0x4683,0x4242,0x7373,0x4663,0x4646,0x4644,0x4642,0x5F46,0x5F46,0x4646,0x4642,0x4F46,0x4646
.word 0xD562,0xBF62,0xAFC4,0xBFC4,0xBFC4,0xAFC4,0xBFD4,0xD462,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E3,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xF4F4,0xE4E4,0xE3F4,0xE4F4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xC4BF,0xD4D4,0xD4C4,0x42AF,0xD49F,0xD462,0xD4D4,0xD462,0xD462,0x62D5,0x41D5,0x6252,0x428F,0x6252,0xC4D4,0x628F,0xC4BF,0x52AF,0x428F,0x428F,0x418F,0x428F,0x418F,0x8F8F,0x8F41,0x8F41,0x6242,0x9F52,0x8F62,0xE452,0x429F,0x418F,0xC49F,0x52D4,0x429F,0xD4E4,0xD4D4,0xAFD4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4C3,0xD4E4,0xD4E4,0xE4D4,0xC3D4,0xD4D4,0xD4E4,0xE4E4,0xC3D4,0xE4D4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4446,0x5246,0x4646,0x4646,0xB45F,0xB493,0x9383,0x7383,0x8373,0x83D4,0x83B4,0x9373,0x83C4,0x9373,0x83C4,0x83B4,0x73B4,0x8373,0xB483,0xB483,0xB483,0x83B3,0x73B4,0xB493,0xB493,0x8383,0x73B4,0x4663,0x7343,0x6393,0x4646,0x4F42,0x4646,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xAFC4,0x62C4,0x72D5,0xD562,0xD572,0xD462,0xD4C4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD3D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE3E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xF4F4,0xF4E3,0xE4E4,0xD4E4,0xD3D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFC4,0xAF83,0xD4C4,0xD4C4,0xD483,0x42AF,0xC4AF,0xD4D4,0x62D4,0xC4BF,0xC4BF,0x629F,0x528F,0xD5AF,0x62D4,0x528F,0xD4D5,0x9F62,0x8F52,0x8F42,0x5241,0x429F,0x6241,0x8F42,0x7F42,0x7F42,0x7F8F,0xD552,0x8F41,0xE442,0x417F,0x8FC4,0xD47F,0x9F62,0x8F52,0xD4D4,0xAF62,0xD4C4,0xE462,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4D4,0xD3D4,0xE3E4,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD472,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x72D4,0x9FD4,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x5246,0x4683,0x4642,0x9363,0x7393,0xB4C4,0x8373,0x8383,0x93B4,0x7383,0xC483,0xB493,0xC483,0xB483,0x8383,0x7393,0x7373,0x8383,0x7383,0x8373,0xB3B4,0x8383,0x83B4,0x8383,0x8373,0x7383,0x6263,0x7393,0x4643,0x5F46,0x4646,0x4646,0x4646,0x4446,0x4246,0x4F46,0x4646,0x465F,0x4642
.word 0xD462,0xC4BF,0xC4BF,0xC4BF,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E3,0xF4D3,0xE4E4,0xE4E4,0xE4D3,0xE4F4,0xE4E4,0xE4E4,0xF4F4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4D3,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x83AF,0xC4BF,0xC4BF,0xD4D4,0x51D4,0x529F,0xD4D4,0xC4D4,0xD462,0x9F62,0x8F7F,0x5142,0x9FD5,0x8F52,0x417F,0x427F,0x528F,0x7F8F,0xD552,0x7FD4,0xE662,0x418F,0x417F,0x417F,0x8F42,0x417F,0x6262,0x628F,0xC4D4,0x62E4,0x418F,0x429F,0x429F,0xD4D4,0x41D4,0xD4D4,0x62D4,0xD4E4,0xD4C3,0xD4E4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xC3E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xD3E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4BF,0x52D4,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x5F46,0x4346,0x7383,0x4646,0xB452,0x83B3,0x8383,0x7373,0x93C4,0xB483,0x7373,0x93D4,0x7373,0xB483,0x7393,0xB483,0x7383,0x6352,0x7373,0xB483,0x9373,0x7373,0xC473,0xB493,0x7383,0x93C4,0x63B4,0x8373,0x43C4,0x4246,0x4646,0x5F46,0x4646,0x4646,0x5F46,0x4F46,0x4646,0x4646,0x4646,0x464F
.word 0x62D4,0x62D5,0x62C4,0x72D5,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4C3,0xE4E4,0xC3D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E3,0xF4E4,0xF4E4,0xF4F4,0xE4E3,0xE3E4,0xF4E4,0xF4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E3,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD562,0xD462,0xD462,0xD4D4,0xD4D4,0xD4D4,0x529F,0xC4AF,0xC4D4,0x52D4,0x528F,0x6262,0x428F,0xD562,0x8F41,0xD562,0x4162,0xD4D5,0x7FE4,0x418F,0xD442,0x8F52,0x7F42,0x417F,0x8F7F,0x8F7F,0xAF73,0xAFE4,0x41E4,0x527F,0x428F,0x9F41,0xE4D4,0x52D4,0xD49F,0xD4D4,0xD462,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xC3E4,0xD4D4,0xD4C3,0xE4D4,0xC3D4,0xE4E4,0xE4E4,0xD4E4,0xE4D4,0xE4F4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4C3,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x83D4,0x9FD4,0x4646,0x4646,0x465F,0x4642,0x4452,0x4646,0x4246,0x9373,0x7373,0x9373,0xB483,0xB493,0x8383,0xC4B3,0x83B3,0x8383,0x93D4,0x83B4,0x7383,0x7383,0x9383,0x73B4,0x7373,0x8352,0x7373,0xB483,0x6383,0xB493,0x8383,0x9373,0x83D4,0x8373,0xC493,0x4663,0x4646,0x4646,0x4346,0x5273,0x4246,0x4646,0x4646,0x4646,0x5F44,0x4646,0x4646
.word 0xD562,0xD462,0xD4BF,0xD4D4,0xD4C4,0x62D4,0xBFD4,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD3E4,0xF4E4,0xE4E4,0xE3E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xF4E4,0xF4E4,0xF4F4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4C3,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x62BF,0xAFD5,0xD562,0xD462,0xD4D4,0xD4D4,0x62D4,0x8F8F,0xD462,0x529F,0xD4BF,0xAF41,0xD462,0x527F,0xC49F,0x7FD4,0x6251,0x41D4,0x527F,0x7F8F,0x8F41,0x8F52,0x7F42,0x7F42,0x7F42,0x528F,0x62E4,0x8F62,0x8F73,0x9F42,0x4142,0xD4E4,0xAFD4,0x6262,0xD4D4,0xBFD4,0xD4C4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD3E4,0xE3D4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x52D4,0x4646,0x4644,0x4646,0x465F,0x5246,0x5282,0x4646,0x8373,0x8383,0x9373,0x93B4,0x73C4,0x8373,0xB493,0x83D4,0x83B4,0x93B4,0x83B4,0x83B4,0x73B4,0xB493,0x7383,0x7352,0x5273,0x7383,0x7383,0x8373,0x93C4,0x83B4,0x83B4,0x73B4,0x7383,0x93C4,0x4644,0x4642,0x635F,0x7362,0x4643,0x4F46,0x4646,0x4646,0x4642,0x4646,0x5F46,0x4246
.word 0xBFD4,0xD4C4,0x62D4,0xD4D4,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4E3,0xE4F4,0xE4F4,0xE3E4,0xF4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4D4,0xE4F4,0xE4F4,0xE4E3,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xAFD4,0xBFC4,0xD462,0xD462,0xD4D4,0xD4D4,0xC4D4,0x62D4,0x528F,0x6252,0xD4D5,0x527F,0xAF62,0x9F52,0x528F,0x62D5,0x628F,0xD4D5,0x7FC4,0x8F62,0x8F62,0x7F41,0x7F41,0x7F42,0x8F62,0xAF52,0x62D4,0x428F,0xD49F,0x41D4,0xE4D4,0xE4E4,0xC4AF,0x62D4,0x62D4,0xD4E4,0xD4C3,0xD4D4,0xD4C3,0xD4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xD3E4,0xE4D4,0xE4E4,0xD4E4,0xD4D3,0xE4E4,0xD4C3,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xC3D4,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x9FD4,0x4646,0x465F,0x4646,0x4646,0x4246,0x9344,0x8373,0x7346,0x83A3,0x93B4,0x9373,0x83D4,0x8373,0x83C4,0xB483,0x9383,0x83C4,0x8373,0x8373,0x7393,0x83B4,0x7373,0x7373,0x7373,0x7363,0x8373,0x8373,0x7383,0x8373,0x8383,0x7373,0xB373,0x73C4,0x4646,0x4246,0x83B4,0x4663,0x4246,0x4646,0x5F46,0x5F46,0x4646,0x465F,0x4646,0x4646
.word 0xD4C4,0xBF62,0xD4C4,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4E4,0xD4E4,0xE4E4,0xE4C3,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE3E4,0xF4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E3,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD472,0xD472,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD562,0x62BF,0x83BF,0x62D4,0xD4D4,0xD4BF,0x9FD4,0x8F8F,0x417F,0x8F7F,0x528F,0x5262,0x628F,0xD4D4,0x528F,0x9F62,0x8F41,0x8F42,0x5262,0x428F,0x417F,0x417F,0x8F7F,0x8F42,0xE4D4,0x8F41,0xE442,0x41D4,0x4142,0x429F,0xD441,0x9FD4,0xD4D4,0xD4C4,0xE4D4,0xD4D4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xC3E4,0xD4E4,0xD4C3,0xD4E4,0xE4D3,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4C3,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD472,0xD472,0xBFD4,0x52D4,0x4246,0x4646,0x465F,0x4642,0x4646,0x4246,0x8393,0x7393,0x8393,0xC483,0xC473,0x7383,0xB483,0xB483,0x8383,0xB383,0xB483,0x8373,0x7373,0x73D4,0x8373,0x7373,0x5273,0x8373,0x8373,0x7383,0xC483,0x7373,0xB483,0xB4B3,0x7383,0x8383,0x4383,0x4246,0x8352,0x6383,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F44,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E3,0xE3E4,0xF4E4,0xE3E4,0xF4E4,0xE4E4,0xF4F4,0xE4E3,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4C3,0x72D4,0xBF62,0xBFC4,0xAFC4,0xBF83,0xD483,0xD4D4,0xD4D4,0xD472,0xD4D4,0xD4D4,0xD562,0xD562,0xD4AF,0xD562,0xC462,0xD562,0x52C4,0xD541,0x4162,0x8F52,0x528F,0x628F,0x62D5,0x8F8F,0x528F,0xD562,0x4251,0xD4D5,0x527F,0x8FD4,0x7F42,0x7F42,0x8F41,0x7F42,0x6242,0xE4D4,0x8FD4,0x8F52,0x8F42,0x8F42,0x5241,0xD48F,0xD451,0xD4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xE4D4,0xC3D4,0xD4D4,0xD4C3,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4D4,0xD4D4,0xD4D4,0x9FC4,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4642,0x7346,0x8373,0xC4B4,0x9373,0x9373,0x83C4,0x7383,0x8383,0x83B4,0x93B4,0x8383,0xB483,0x9393,0x83C4,0x7373,0x9373,0x7373,0x7393,0x7393,0xB483,0x7383,0x7373,0xC483,0x8383,0x7383,0xD4D4,0x4373,0x8362,0x7383,0x4643,0x4646,0x4646,0x4646,0x4246,0x4646,0x4642,0x465F,0x4646,0x465F
.word 0x62D4,0xC4BF,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD3D4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0x9F62,0x8F42,0x5242,0xD4D4,0xD562,0x62D4,0x72D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD562,0xD4AF,0xBF62,0x62C4,0xC4BF,0x419F,0x8FD5,0xD562,0x9FD4,0x8F42,0x528F,0x7FD4,0xAF42,0xD452,0x8FAF,0x8F62,0x8F41,0x5241,0x629F,0x428F,0x7FD4,0x417F,0x628F,0x41E5,0x527F,0x428F,0x418F,0x428F,0x418F,0x8F42,0xD462,0xBF62,0xD4C4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4C3,0xD3D4,0xE4D4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xE4C3,0xE4E4,0xD4E4,0xE4C3,0xD4E4,0xD4E4,0xE4D4,0xE4E4,0xD4D4,0xC3E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD472,0x5262,0x4646,0x465F,0x4246,0x4646,0x465F,0x5F46,0x4246,0x7383,0x9373,0xD473,0xB483,0xB483,0x7383,0xB483,0x8383,0xC483,0xB493,0x8383,0x83C4,0x83B4,0x8383,0xC483,0x7373,0x7373,0x73C4,0x7383,0x7373,0x8383,0x83C4,0xB473,0x8373,0x8383,0x7373,0xC483,0x4373,0x4646,0x4642,0x465F,0x4642,0x4646,0x5F46,0x4646,0x4646,0x4642,0x4646
.word 0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xC3E4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E3,0xF4E4,0xE4E4,0xD4E3,0xE4F4,0xE3E4,0xF4E4,0xE4E4,0xD4E4,0xE4D3,0xE4D4,0xD4E4,0xC3D4,0xD4D4,0xD4D4,0x62D4,0x4151,0x9F41,0x9F42,0x9F52,0x9F52,0x9F41,0xBF52,0xBF83,0xBFC4,0xD4C4,0xD4AF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD562,0xC4BF,0x8FD4,0x6283,0x8F62,0xAF42,0x8F52,0x4241,0x8F41,0x527F,0x428F,0x628F,0x8F62,0x9F42,0x62C4,0xD49F,0x428F,0x62AF,0x6262,0x428F,0x8F7F,0x8F42,0x7F41,0xAF62,0xD552,0x52D4,0x8F8F,0x5241,0x8F8F,0xD4C4,0x62D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xD4C3,0xD4E4,0xE4F4,0xE4E4,0xE4D3,0xE4E4,0xE4D4,0xD3D4,0xD4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4C4,0x43AF,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x8352,0x8383,0x8383,0x7383,0x8383,0x7373,0x8383,0xB483,0x9383,0x83B3,0x73C4,0x8383,0xB483,0xC493,0x7393,0x8373,0x7373,0x7383,0x8373,0x8373,0x73B4,0x8373,0x8383,0xB483,0xB483,0x9383,0x83D4,0x4663,0x4642,0x464F,0x4646,0x4646,0x464F,0x4646,0x4646,0x465F,0x4646,0x465F
.word 0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC3E4,0xD4E4,0xD4E4,0xD4D4,0xE4E4,0xE4E4,0xE4D4,0xE4D3,0xE4E3,0xF4E4,0xE3E4,0xF4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4C3,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0x62D4,0xD462,0x9F41,0x9F52,0xAF63,0xAF73,0x5273,0x52AF,0x8F52,0x5252,0x62D5,0x62D5,0xC4BF,0xBF83,0xD4C4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x62D4,0x529F,0xBFD5,0xD5C4,0x8F51,0x5241,0x8F62,0xC452,0x41D4,0x418F,0xAF52,0x7F42,0x4241,0x42AF,0x428F,0xD452,0x8F7F,0xF662,0x41D4,0x7F42,0x427F,0x528F,0x429F,0x517F,0xD4C4,0x8F52,0xD47F,0x527F,0x62AF,0xD4E4,0xD462,0xD4D4,0xE4D4,0xE4D4,0xD4C3,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xC3D4,0xE4D4,0xE4D4,0xD4D4,0xD3D4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4C3,0xD4D4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xC4D4,0xD4D4,0x4262,0x4646,0x4642,0x465F,0x465F,0x465F,0x465F,0x5F46,0xB444,0xB483,0xB483,0xB483,0xB483,0x8373,0x83B4,0x93C4,0xC4B4,0xB483,0x8383,0x83B4,0x7373,0x8373,0x6373,0x8373,0x83C3,0x7373,0x7373,0x7383,0x7373,0xB483,0x7393,0xD4B3,0x7383,0xB483,0x6393,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x465F,0x4646,0x464F,0x4646
.word 0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xE4D3,0xD4E4,0xE4D3,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xF4E4,0xE3E4,0xE4E4,0xE4F4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xBF62,0x9F52,0x5252,0x73AF,0x73AF,0x73AF,0x52AF,0xAF52,0x9F52,0x9F52,0x9F41,0xD462,0x62D4,0x62D5,0xAFD4,0x72D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0x52D4,0xD4AF,0x62D4,0x62D4,0x62D5,0x8F9F,0x8FC4,0x529F,0xD4D4,0x8F7F,0x6242,0x7F8F,0xE4D4,0x8F62,0x8F62,0xD462,0x527F,0x528F,0xE4E4,0xD4E4,0x5241,0xD462,0xC4E4,0x8F62,0x517F,0x739F,0xD4E4,0x7F8F,0x8F42,0x8F41,0xAF62,0xD483,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xC3D4,0xD4E4,0xD4E4,0xE4E4,0xE4D3,0xE4E3,0xE4D4,0xD3D4,0xD4E4,0xE4D4,0xD4E4,0x62E4,0xD4E4,0xE4D4,0xE4E4,0xD4E4,0xD4D4,0xD4C3,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xBFD4,0x4362,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x7346,0x73B3,0x8383,0x7383,0x8373,0x7383,0x8373,0x93B4,0x8373,0x8383,0x93C4,0xB483,0x7383,0x6373,0x7362,0x8383,0x7373,0x83D4,0x7373,0x7373,0x8383,0xC493,0x8373,0x9383,0x73B4,0x9383,0x4673,0x4644,0x5242,0x4652,0x4642,0x4646,0x465F,0x4646,0x4646,0x4642,0x4646,0x4246
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4D4,0xE4F4,0xE4E4,0xE4E3,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD472,0xAF62,0xAF73,0xD562,0xD562,0xD562,0xAF62,0xAF73,0x5273,0x52AF,0x429F,0x9F52,0x9F42,0xD462,0xC4BF,0xD462,0xD562,0xD4BF,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0x8FD4,0x6252,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x428F,0x9F41,0x7F7F,0x8F52,0xD552,0xD4D4,0x7FC4,0x7F8F,0xAF62,0xAF42,0x9F73,0x8F62,0x8F62,0x8F42,0xD541,0xAFE4,0x9F52,0x9F52,0xD442,0x7FD4,0x8F62,0x7F42,0x7F41,0x8F42,0x9F42,0xAF62,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xE4D4,0xD4C3,0xE4D4,0xE4E4,0xE4E4,0xF4D4,0xE4E4,0xF4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xE4E4,0xE4D3,0xE4D4,0xD4C3,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD472,0xC4D4,0x46AF,0x4646,0x465F,0x4646,0x4644,0x4246,0x4246,0x4646,0x735F,0x73D4,0xB483,0x8383,0xB483,0x8383,0x8373,0xC483,0x7383,0x93C4,0x83C4,0x8383,0x83B4,0x5273,0x7373,0x8373,0x73C3,0x7383,0x7383,0x8373,0x83C4,0x73B4,0xB483,0x83B3,0x8373,0xB4C4,0x4663,0x5242,0x6383,0x4646,0x4646,0x4646,0x4646,0x4644,0x465F,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xE4E4,0xD3D4,0xD4E4,0xE4C3,0xD4D4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4D3,0xE4E4,0xE4E4,0xE4E4,0xE4C3,0xE4E4,0xE4E4,0xC3E4,0xD4D4,0xD4D4,0xAF62,0x62C4,0xC4AF,0xAFD5,0xBFC4,0xAF62,0x62C4,0x73AF,0x52AF,0x9F52,0x9F52,0x8F52,0x9F52,0x8F52,0xD562,0xBFD4,0x62C4,0x72D4,0xD462,0x72D4,0xD4D4,0xD4D4,0x52D4,0x52AF,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x428F,0x6252,0x52D4,0x628F,0x8F62,0x8F52,0xC452,0x7FD4,0x8F42,0x8F41,0x8F42,0x627F,0x6262,0x62E5,0x529F,0x528F,0x62D5,0x42AF,0x7F7F,0x417F,0x417F,0x8F52,0x6262,0xD48F,0xC4D4,0xAF62,0xC3C4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD3E4,0xF4E4,0xE4E4,0xF4E4,0xE4D4,0xE4D4,0xE4D4,0xD4C3,0x62D4,0xC4AF,0xF4D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4E4,0xC3D4,0xE4D4,0xD4D4,0x62D4,0x4662,0x4646,0x4646,0x4F42,0x5F46,0x4F46,0x4646,0x4646,0x5246,0x8383,0x83C4,0x73B4,0x7383,0xB483,0x7373,0xB483,0x7383,0x7373,0xC483,0xB493,0x7373,0x5273,0x8373,0x83C4,0x7383,0x7383,0x7373,0x8383,0x83B4,0x8383,0xB3D4,0x7383,0xC483,0x4383,0x6263,0x8393,0x4643,0x4642,0x465F,0x465F,0x4642,0x465F,0x4646,0x465F,0x465F,0x465F
.word 0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD3D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D3,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xAF72,0x62C4,0xD5AF,0xAF83,0x72C4,0xD562,0x6262,0x73AF,0xAF62,0xAF73,0x5252,0x52AF,0x529F,0x8F52,0x9F52,0x8F52,0xC483,0xC4BF,0xD5BF,0xAFD4,0xD4D4,0xD4D4,0xD4D4,0x51D4,0x9F52,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xAFD4,0x8F8F,0xBF52,0x417F,0x528F,0x7FD4,0xAF52,0x62D4,0x428F,0xD552,0x527F,0xC48F,0xE4BF,0x9FD4,0x8F42,0x527F,0x52AF,0x628F,0x7F41,0xE442,0xD47F,0x428F,0xD4D4,0x428F,0xBFD4,0x62D4,0xD462,0xE4D4,0xC3D4,0xD4D4,0xC3D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4C3,0xD4E4,0xD4C3,0xD4E4,0x41D4,0xE462,0xE4E4,0xE4E4,0xD3E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x46AF,0x4642,0x465F,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x6346,0x9383,0x8373,0x8383,0x8373,0x8383,0x6383,0x8383,0xB493,0x8383,0x8363,0x8383,0x5273,0x5273,0x8383,0x8373,0x7373,0xB483,0x8373,0x8373,0xB483,0x9373,0xB4C4,0xB483,0xB493,0x7373,0x8383,0x43B4,0x4646,0x4646,0x4646,0x4646,0x464F,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD4C3,0xD4D4,0x62D4,0x62D5,0x62D5,0xD4D4,0xD5D4,0xC4BF,0xC4AF,0xBF62,0xAF73,0x5273,0x52AF,0x9F52,0x9F52,0x9F42,0x4252,0x429F,0x529F,0xD4C4,0xD462,0xD462,0xBF62,0xD4C4,0xD4D4,0x9FD4,0x5252,0xE462,0xD4D4,0xC3D4,0xD4D4,0x51D4,0x4142,0x6273,0x429F,0x8F52,0xAF52,0xD4C4,0x417F,0xC48F,0x419F,0xC49F,0x8F62,0x62D4,0x62D5,0xD4D5,0x62D4,0x9F62,0x8F52,0x8F52,0x6242,0xD4D5,0x62D4,0xAF62,0xE4D4,0x418F,0xD452,0xD452,0x62BF,0xD462,0xE4D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D3,0xD4E4,0xD4D3,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xAF62,0xD4D4,0xE4E4,0xD3E4,0xE4E4,0xD4E4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x4662,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4446,0xB473,0xB483,0x7393,0x7383,0xC473,0x8393,0x7373,0x8383,0x73B4,0x7373,0xB483,0x7383,0x7352,0x8373,0x7363,0x7373,0x7393,0x8383,0x73B4,0x8373,0xD4D4,0x8383,0x93D4,0x73B4,0x9383,0x73C4,0x4343,0x6252,0x4642,0x4646,0x4642,0x4646,0x5F46,0x4246,0x5F46,0x4446,0x5F46
.word 0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xE4E4,0xD3E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xC3E4,0xD4E4,0xD4D4,0xD4D4,0xD562,0xBFD4,0xBF83,0xD4D4,0x62D4,0x83D4,0xD4D4,0xC4AF,0x62D5,0x73AF,0x73AF,0x739F,0x529F,0x529F,0x529F,0x529F,0x529F,0x529F,0xD4BF,0x62D5,0x83D5,0xAFD4,0xD4D4,0x52D4,0x9F9F,0xD452,0xD4D4,0xD4D4,0x62D4,0x628F,0x52AF,0xD58F,0x8F62,0x8F52,0x5242,0x62AF,0x427F,0xD541,0x7F62,0x7F42,0x9F62,0xAF52,0x8FC4,0x4141,0xE562,0xAFC4,0x5273,0x429F,0xD4BF,0x62D4,0x8F8F,0xD4D4,0x417F,0x8F42,0x52E4,0xD4D4,0xD5D4,0x62BF,0xD4D5,0xD4E4,0xD4D4,0xE4D4,0xF4C3,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0x62D4,0x739F,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0x62D4,0x46AF,0x5F46,0x4646,0x4642,0x465F,0x465F,0x4646,0x4446,0x5F46,0x9373,0x83B4,0x83B4,0x83B4,0x8373,0x93B4,0x73C4,0x8373,0x9383,0x8373,0x5273,0x7373,0x8373,0x8373,0x8373,0x7383,0x83C4,0x73B4,0x7373,0x8383,0x8383,0xA3B4,0xC493,0x8383,0xC483,0x6383,0x8383,0x4373,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646
.word 0xD4D4,0xD4D4,0xD4C3,0xE4C3,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4D3,0xE4D4,0xD3D4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4C4,0xAF62,0xD562,0xAF62,0x9F73,0x9F73,0x5252,0x429F,0x5241,0x529F,0x9F52,0x9F52,0x5252,0xC4BF,0xC4BF,0xC4BF,0xD462,0x8FD4,0x5252,0x529F,0xD4D4,0xD4D4,0x8FD4,0xAF62,0xD573,0x8F51,0x9F42,0xD4C4,0xD48F,0xE4D4,0xE4E4,0x8FD4,0xD4D4,0x417F,0x429F,0x628F,0x429F,0x8F7F,0x7F41,0xD462,0x8FD4,0x6241,0x8F52,0xAF52,0xD442,0xE462,0x7F62,0xE452,0xD4D4,0xD4D4,0xD441,0xC47F,0x627F,0xD4D4,0xE4E4,0xD4D4,0xE4D4,0xE4E3,0xF4E4,0xE4E3,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0x62D4,0x62D5,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xC3D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD472,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x62D4,0x4662,0x4646,0x5F46,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x7344,0x9383,0x7383,0x8383,0x73B4,0x8383,0x83D4,0x7373,0x7373,0xB483,0x7393,0x5273,0x7373,0xC473,0x7383,0x6373,0x7383,0x7373,0x8373,0x93B4,0x9373,0x83C4,0x83C4,0x93B4,0x73B4,0xD483,0x6373,0x4646,0x4642,0x4646,0x465F,0x465F,0x4642,0x4646,0x465F,0x4646,0x4246
.word 0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D4,0xD3D4,0xE4D4,0xE4E4,0xE4D4,0xD3E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E3,0xD3E4,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xD4E4,0xD4C3,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xBFD4,0xD4C4,0xBF62,0x6273,0x73AF,0x529F,0x529F,0x529F,0x429F,0x9F52,0xAF52,0x5252,0x529F,0x6241,0x62D5,0x62D4,0xBFD4,0x62C4,0x52AF,0x529F,0xD4BF,0xD4D4,0x62D4,0x63AF,0x9FE4,0x8F42,0x4241,0x528F,0x52E4,0x418F,0xD452,0xD4E4,0xD4E4,0x8F52,0x9F62,0x5252,0x8F62,0xD462,0xD4D4,0xD4D4,0x8F62,0x8F52,0x8F52,0x428F,0xD47F,0xD4AF,0xE4D4,0xD4D4,0x8F7F,0xD452,0x7FD4,0xAF62,0x8FD4,0xD462,0xC3D4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0x72D4,0x52AF,0xD4AF,0xD4E4,0xE4E4,0xE4E4,0xD3E4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0x72D4,0xD4E5,0xD4D4,0xD4C3,0xE4D4,0xD4D4,0xAFD4,0x4662,0x4246,0x4646,0x5F46,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4342,0x83C4,0x93B4,0x73C4,0xB493,0x7373,0xC483,0x6383,0x7373,0x7383,0x7373,0x7352,0x8383,0x7383,0xC483,0x5283,0x7383,0x7373,0x8383,0xB483,0xD493,0xC493,0x8383,0xB483,0x8383,0x8393,0x4663,0x4642,0x465F,0x465F,0x4646,0x4646,0x464F,0x465F,0x4646,0x4644,0x4646
.word 0xD4D4,0xC3D4,0xE4D4,0xE4D4,0xC3D4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xD4E4,0xC3E4,0xD4D4,0xE4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x62D5,0x73AF,0x73AF,0x73AF,0x529F,0x9F52,0x9F52,0x9F52,0x5252,0x52AF,0x529F,0x429F,0x529F,0xD4AF,0xD462,0xD4D4,0x9F52,0x9F52,0x6252,0xD4D4,0x52D4,0xAF62,0x6252,0x428F,0x418F,0x9FD4,0x8F41,0x7F42,0x7F8F,0x5241,0x8FE4,0xE4D4,0xD4E4,0x9FE4,0xC48F,0xD4D4,0x9F62,0x417F,0x9FD4,0xD47F,0x7FE4,0x5241,0x7F8F,0xD442,0xAF62,0x6242,0x7F7F,0x7F7F,0x7F7F,0x427F,0x527F,0x629F,0xC4AF,0xD4E4,0xE4E4,0xE4F4,0xE3E4,0xE4D4,0xE4E4,0xD4C3,0xD4D4,0x62D4,0x62D5,0xD4D5,0xE4D4,0xF4E3,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0x72D4,0xC4D4,0x46AF,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4F42,0x4646,0x9352,0x93B4,0x9373,0x8373,0x7383,0x8373,0x8383,0x7383,0x7373,0x7383,0x5273,0x7383,0x83B4,0x7383,0x8373,0x73B4,0x9383,0x73B4,0x8383,0x83B4,0x83B4,0x83B4,0x8373,0xB483,0x63C4,0x4646,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F44,0x4646,0x4642
.word 0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4E4,0xD3D4,0xE4E4,0xE4C3,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xC4BF,0xAF62,0x62C4,0x73AF,0x52AF,0x529F,0x529F,0x629F,0x52AF,0xAF52,0x5252,0x529F,0x528F,0x6241,0x62D5,0xBFD4,0x42D4,0x529F,0x429F,0xD4BF,0x8FD4,0x5262,0xD58F,0x4141,0xD442,0x8F52,0x7F42,0x428F,0x427F,0x8F8F,0x7F41,0xE452,0xD4D4,0x4241,0x627F,0xBFD4,0xE442,0x8FE4,0x7F42,0xE442,0x7FD4,0x8F7F,0x7F7F,0x7F7F,0xC441,0x7FAF,0x7F7F,0x7F41,0x4241,0x6241,0x418F,0xAF42,0x6273,0xE4AF,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xBFD4,0xAF52,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAFD4,0x4662,0x5F46,0x4646,0x4646,0x465F,0x465F,0x4642,0x4646,0x4646,0x4246,0x8344,0xB483,0x7393,0x73C4,0x8383,0x7373,0x7373,0x7383,0x8373,0x7373,0x7373,0xD473,0x8383,0x7363,0x8352,0x8373,0x7373,0x7383,0x83C4,0x8383,0x8393,0xB483,0x7383,0x8393,0x4673,0x4642,0x465F,0x4646,0x4646,0x464F,0x465F,0x4644,0x5F46,0x4646,0x4F46,0x4646
.word 0xD4D4,0xD4E4,0xD4D4,0xD4C3,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xE3E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4C3,0xE4D4,0xC3D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0x72D4,0xD4D4,0xD4D4,0xD562,0x62D4,0x73AF,0xAF62,0x9F73,0x5252,0x529F,0x9F52,0x9F52,0x5252,0x529F,0x528F,0x429F,0x8F8F,0x528F,0x62D5,0xD4D4,0x5241,0x529F,0x5252,0x62D4,0x629F,0x429F,0xAF62,0xD48F,0x8F62,0x8F41,0x8F42,0x4141,0x5242,0x5252,0xC4BF,0xD4D4,0x41E4,0xE562,0xD4E4,0xC4D4,0x5262,0x528F,0x628F,0x62E4,0x417F,0x427F,0x6241,0x7F7F,0xD442,0x7F62,0x427F,0x8F7F,0xAF62,0xD542,0xC47F,0x8F8F,0x6273,0xE462,0xE4E4,0xD3D4,0xD4E4,0xE4E4,0xE4D4,0x62D4,0xC4D4,0x6241,0xD4D4,0xE4E4,0xE4E3,0xE4E4,0xD4D3,0xD4E4,0xD4C3,0xC3D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4C3,0xD4D4,0x6262,0x46AF,0x4646,0x5F46,0x4646,0x4646,0x4646,0x464F,0x5F46,0x4646,0x4646,0x4246,0xC483,0x8373,0x7373,0xC493,0x7383,0xB483,0x7383,0x73C4,0x7363,0x8352,0x7373,0x7383,0x6383,0xB483,0x8373,0x8373,0x73B4,0xC483,0xB493,0x83C4,0x73B4,0xB493,0x7383,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46
.word 0xD4D4,0xD4D4,0xD4C3,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xD4E4,0xE4F4,0xE4E4,0xE4E4,0xD3D4,0xE4D4,0xC3E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4D4,0x62D4,0xC4BF,0xAFD5,0xAF73,0x5273,0x52AF,0x9F52,0x5252,0x739F,0x52AF,0x529F,0x9F52,0x8F42,0x8F42,0x7F42,0xAF41,0xD4C4,0x42AF,0x9F52,0x9F52,0xD48F,0x9F52,0x9F52,0x4252,0xE541,0x9F52,0x5252,0x529F,0x9F52,0xAF62,0xD4D5,0xD4D4,0xE4E4,0xE4D4,0xD4E4,0xC4D4,0x62AF,0x8F9F,0x8F42,0x7F62,0xE4E4,0x7FD4,0xD4D4,0xD4E4,0xD4E4,0x627F,0x41D4,0x4152,0xC48F,0x52D4,0x7FD4,0x7FD4,0x7FD4,0xAFD4,0x62C4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xAFD4,0xD562,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD462,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0x62D4,0x4662,0x4642,0x4646,0x5F44,0x4646,0x4642,0x4646,0x4646,0x5F46,0x4646,0x4642,0x8363,0x7393,0xB483,0x8373,0x83B4,0x83C4,0x8383,0x7383,0x6362,0x8383,0x73C4,0x7383,0x7373,0x7373,0x7383,0x7373,0x9383,0xB3C4,0x73B4,0x8373,0x8383,0x8373,0x4373,0x5F46,0x4646,0x4646,0x4646,0x4642,0x4646,0x4644,0x4646,0x4F42,0x5F46,0x4246,0x4646
.word 0xE4D4,0xD4D4,0xD4E4,0xD4E4,0xD3E4,0xD4E4,0xE4D3,0xD4E4,0xE4E4,0xE4D3,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D3,0xD4E4,0xD4C3,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAF62,0x6283,0x73AF,0x62AF,0xAF52,0x9F52,0x9F52,0x5252,0x52AF,0x9F52,0x8F52,0x9F42,0x8F42,0x7F41,0x6F7F,0x627F,0xD4D4,0x529F,0x429F,0x9F52,0x9F52,0x9F42,0x9F52,0x9F52,0x6262,0xD4E5,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD483,0x62D4,0x9F62,0x9F52,0x8F52,0x8F42,0x7F41,0x6242,0xC48F,0x428F,0x417F,0x417F,0x428F,0x628F,0x7FD4,0x428F,0xD4D4,0xD48F,0xD48F,0xD441,0xC441,0x6241,0x62AF,0x72D5,0xE4E4,0xC3D4,0xD4E4,0xC3D4,0x62D4,0x9F62,0xD562,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD3D4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD5D4,0x46AF,0x4646,0x465F,0x4646,0x5F46,0x4646,0x4246,0x4646,0x4642,0x4F46,0x4646,0x4246,0x8363,0x62B4,0x7373,0x9383,0x83C4,0x7373,0x6373,0x8352,0x83D4,0x7373,0x7363,0x8352,0x7373,0x8373,0x8383,0x83C4,0x8373,0xC493,0x8373,0x93B4,0x73B4,0x7352,0x7373,0x6273,0x4252,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46
.word 0xD4D4,0xC3D4,0xE4D4,0xC3D4,0xE4D4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4E3,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xE4E4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4E4,0xD4E4,0xD4C3,0xD4C3,0xD4E4,0xE4D4,0xD4D4,0x62D4,0xC4D4,0xC4AF,0xD562,0xAF52,0x9F73,0x6252,0x529F,0x529F,0xAF52,0x9F52,0x8F42,0x4152,0x428F,0x428F,0x7F41,0x7F7F,0x6242,0x52D5,0x528F,0x528F,0x528F,0x8F52,0x8F42,0x8F52,0x9FD4,0x6273,0xD562,0xD4D4,0xD462,0xAF62,0x6262,0x529F,0x429F,0x529F,0x428F,0x418F,0x428F,0x5241,0x52F5,0x629F,0x6262,0x6262,0x6262,0xAF62,0x8F52,0x7FC4,0xE4D4,0xD4E4,0x41E4,0x62D4,0x7FD4,0x8F62,0xC48F,0x62D5,0x629F,0xD48F,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x52AF,0xD4AF,0xD4D4,0xE4E4,0xD3E4,0xE4E4,0xD4E4,0xD4E4,0xE4C3,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xAF62,0x4683,0x4646,0x4646,0x4246,0x4646,0x4F46,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x6344,0x8373,0x7373,0x7383,0xB483,0x6383,0x5273,0x7393,0x7373,0x8373,0x7383,0x8373,0x7373,0x7383,0x83B4,0x8373,0x83C4,0x83B4,0x9383,0x73B4,0x7383,0x8373,0x4363,0x4646,0x4646,0x4644,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4446,0x5F46,0x4646
.word 0xC3D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4C3,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x72D4,0xD4D4,0xC4BF,0xAFD4,0xAF62,0x6262,0x73AF,0x52AF,0x9F52,0xAF52,0x5252,0x529F,0x528F,0x428F,0x418F,0x7F7F,0x7F6F,0x466F,0x4646,0x6242,0x62D4,0x528F,0x528F,0x528F,0x529F,0x42AF,0xC48F,0x9F9F,0x418F,0x8F41,0x9F52,0x9F42,0x9F42,0x8F42,0x8F52,0x8F42,0x8F41,0x7F42,0x8F42,0x528F,0x9FE4,0x8F42,0x8F8F,0x8F8F,0x8F8F,0x8F42,0x8F52,0x527F,0x5251,0xE4E4,0xD4D4,0xD48F,0xD4D4,0xD4D4,0xBFC4,0x8FD4,0xAF42,0x6252,0xE4D4,0xD4E4,0xD4D4,0xD462,0xD562,0xD462,0xD4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4C3,0xD4D4,0x62D4,0x46AF,0x4642,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x464F,0x465F,0x4646,0x4246,0x4246,0x6344,0x7373,0x7383,0x7373,0x5273,0x7373,0xB483,0x8373,0x7383,0x7373,0x8373,0xD483,0xB3B3,0xB483,0x7383,0x8373,0x8383,0x83B4,0x8373,0x8383,0x6363,0x4642,0x4246,0x5F46,0x5F46,0x5F46,0x4246,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4646
.word 0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xC3E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4F4,0xE4E3,0xE4E4,0xD4E4,0xE4E4,0xD3D4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xC3D4,0xE4D4,0xD3D4,0xD4E4,0xE4D4,0xE4E4,0xE4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xAFC4,0xC4C4,0xC4AF,0x73AF,0x52AF,0xAF52,0x9F52,0x5252,0x52AF,0x529F,0x528F,0x418F,0x428F,0x7F7F,0x7F7F,0x466F,0x466F,0x4646,0xC49F,0x52D4,0x528F,0xC4AF,0xD4C4,0xD4C4,0x418F,0x4252,0x428F,0x428F,0x418F,0x428F,0x528F,0x418F,0x428F,0x428F,0x428F,0x418F,0x428F,0xAF41,0x52E4,0x8F41,0x4242,0x4241,0x427F,0xC4AF,0xE48F,0x8F41,0x7F8F,0xD4E4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x7FD4,0xD4C4,0xAFD4,0x6262,0xD4D4,0xD4D4,0xC4AF,0x629F,0xC4AF,0xD4D4,0xD3E4,0xF4E4,0xD3E4,0xD4E4,0xD4E4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD472,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0x62D4,0x4662,0x5F46,0x4646,0x5F46,0x5F46,0x4246,0x5F46,0x4646,0x4646,0x4246,0x5F46,0x7346,0x4663,0x4646,0x7342,0x7383,0x7352,0x7352,0x5273,0x9373,0x6373,0x8373,0x8363,0x8373,0x93D4,0xC4C4,0x8383,0x8373,0xC483,0x7383,0x7373,0xB483,0x5273,0x4643,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F
.word 0xD4D4,0xD4D4,0xE4E4,0xE4C3,0xE4E4,0xE4D4,0xE4E4,0xE4D3,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xD4C3,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD472,0x62C4,0xD562,0xAF52,0xAF73,0x6252,0x529F,0x529F,0x9F52,0x8F52,0x8F52,0x8F42,0x7F41,0x426F,0x467F,0x6F46,0x6F46,0x4646,0xD541,0xC4D4,0x418F,0xC49F,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xAF62,0xC4C4,0x8F41,0x8F41,0x8F42,0x8F42,0x8F41,0x8F41,0x8F42,0x8F42,0xE452,0x41D4,0x41D5,0x7F8F,0x8F8F,0x8F42,0x4142,0xE4E4,0xD4D4,0xC4C4,0x838F,0xD4BF,0xE462,0xE4E4,0xD4E4,0xD4D4,0xD483,0xD4D4,0xD562,0xAFD4,0xD483,0xD4E4,0xD4C4,0xD541,0xD462,0xD4D4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4C3,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xAFD4,0x46C4,0x4646,0x4246,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4646,0x4646,0x6246,0x7383,0x4642,0x8343,0x7373,0x7363,0x7352,0x8373,0x7373,0xC383,0x7383,0x7373,0x7373,0x8363,0x8383,0x93B4,0x83C4,0x73B4,0x8383,0x83C4,0x6363,0x4644,0x4646,0x4642,0x4646,0x4246,0x5F46,0x4646,0x4F42,0x4646,0x5F46,0x4246,0x4646,0x4644,0x4646
.word 0xD4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE3E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xE4D4,0xC3E4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4C3,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4D4,0xC4BF,0xD5BF,0x62AF,0x73AF,0x63AF,0x52AF,0x52AF,0x9F62,0x5252,0x429F,0x428F,0x8F41,0x7F42,0x6F6F,0x4642,0x465F,0x4646,0x465F,0x6F46,0xC452,0xC4D4,0x62D4,0x62D4,0x9FD4,0x5241,0x8F8F,0x8F42,0x8F8F,0x8F42,0x8F42,0x4241,0x428F,0x4241,0x428F,0x8F41,0x8F41,0xE452,0x8F62,0x7F62,0x4142,0x8F42,0x417F,0x628F,0xE4E4,0xE4E4,0x628F,0xC48F,0x8FD4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD48F,0xD441,0x72D4,0xD562,0x62AF,0x627F,0x62AF,0x629F,0xD4D5,0xE4D4,0xE4E4,0xE4E4,0xE4E3,0xD3E4,0xD4E4,0xD4D4,0xD4E4,0xC3D4,0xD4D4,0xD472,0xD4D4,0xE4D4,0xD4D4,0x72D4,0x62D5,0x46AF,0x465F,0x465F,0x4646,0x4246,0x4646,0x4646,0x4644,0x4F46,0x4F46,0x4646,0x7342,0x8373,0x8383,0x7383,0x5273,0x5252,0x8383,0x7363,0x7373,0x8373,0x6383,0x7383,0x7373,0x7383,0xB483,0x8383,0x8373,0x8393,0x93B4,0x5273,0x4643,0x4642,0x4646,0x465F,0x4246,0x464F,0x4646,0x4642,0x4646,0x5F46,0x4646,0x4F46,0x4646,0x464F,0x4646
.word 0xD4E4,0xC3E4,0xD4E4,0xE4E4,0xD3E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xD4D4,0xE4E4,0xD4D4,0xD4C3,0xD4E4,0xD4C3,0xD4D4,0xD4E4,0xE4C3,0xE4E4,0xD4E4,0xE4E4,0xD3D4,0xE4E4,0xC3D4,0xE4D4,0xC3D4,0xD4D4,0xD472,0xD4D4,0xD4D4,0xD462,0xD562,0xAF62,0x9F73,0x9F73,0x9F52,0x9F52,0x9F52,0x9F42,0x8F42,0x7F8F,0x7F7F,0x6F7F,0x4646,0x4646,0x4642,0x5F46,0x4646,0x6F46,0x8F46,0xD541,0x419F,0x528F,0xD4D4,0x52D4,0x428F,0x417F,0x428F,0x417F,0x8F8F,0x8F41,0x428F,0x418F,0x428F,0xD552,0xD4E4,0x429F,0x8F8F,0x8F41,0x8F42,0x8F42,0xE452,0xE4E4,0xE4E4,0xE4E4,0x62D4,0xD4D4,0xE4E4,0xE4E4,0xD4E4,0x41E4,0xE4D4,0xBFC4,0xD47F,0xD4D4,0xD562,0x8F62,0x6252,0x62D5,0xD4AF,0xD4D4,0xD3E4,0xF4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xD4D4,0x62D4,0xD4E5,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xBF62,0x4673,0x4646,0x4646,0x464F,0x4F46,0x5F46,0x4646,0x4646,0x4646,0x4644,0x4646,0x4346,0xB493,0x8373,0x8373,0x7363,0x7373,0x8373,0x5273,0x7383,0x5273,0x8373,0x7373,0x8373,0x7383,0x8383,0x83D4,0x93C4,0x83C4,0x6373,0x4644,0x4646,0x465F,0x465F,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F
.word 0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xD4E4,0xE4D3,0xD3E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE3D4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xE4D4,0xE4E4,0xE4D3,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xBF62,0x6273,0x52AF,0x5252,0x52AF,0x9F52,0x8F52,0x4142,0x428F,0x4241,0x7F41,0x6F7F,0x5F46,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x5241,0x62D4,0x9F62,0xAF52,0xD4D4,0x9F62,0x8F42,0x8F42,0x7F41,0x8F42,0x8F42,0x8F42,0x4241,0xAF52,0xE4D4,0x5262,0x418F,0x8FC4,0x8F42,0x8F41,0x4241,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xE4D4,0xE4E4,0xC3E4,0xE4E4,0xD583,0xE4D4,0x52D4,0x62D4,0xD4D4,0xD4D4,0x42BF,0x8F8F,0xAF52,0xD4C4,0xE472,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0x62D4,0x469F,0x4246,0x4646,0x4646,0x4642,0x4646,0x465F,0x4F42,0x4646,0x4273,0x4646,0x4642,0x9373,0x7383,0x7373,0x5252,0x8373,0x7373,0x8373,0x7373,0x7383,0x7363,0x8383,0xB483,0xC483,0xB483,0x8393,0x73B4,0x6373,0x4643,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4642,0x465F,0x5F44,0x4646,0x4246,0x4646
.word 0xC3D4,0xD4E4,0xE4E4,0xD4D3,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4D3,0xE4E3,0xE4E4,0xE4E4,0xE4D4,0xC3D4,0xE4D4,0xD4E4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xE4C3,0xE4E4,0xE4D3,0xE4E4,0xD3D4,0xD4E4,0xD4E4,0xE4E4,0xC3D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBF62,0xC4C4,0xC4BF,0xAF62,0xAF62,0xAF63,0x9F73,0x5252,0x429F,0x8F41,0x8F7F,0x6F7F,0x7F42,0x4644,0x4646,0x4646,0x465F,0x465F,0x5F46,0x5142,0x8F52,0x9F7F,0xC4D5,0x62D4,0x418F,0xC49F,0x62D4,0x428F,0x428F,0x418F,0x4142,0xAF52,0xE4D4,0x8FE4,0x62D4,0xD48F,0x62D4,0x428F,0x428F,0x428F,0xD48F,0xF4E4,0xE3E4,0xE4E4,0xD4E4,0x62D4,0xE4D4,0xE4E4,0xE4D4,0x41E4,0xE4D4,0xD4E4,0xE4E4,0x9FC4,0xD49F,0xD4D4,0x8FD4,0x7F42,0x4241,0xAFD4,0xE4D4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xD4E4,0x62D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4D4,0x62D4,0x4662,0x5F46,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4246,0x9383,0x4642,0x4646,0x7342,0x83C4,0x73C4,0x8373,0x7363,0x83D4,0x6373,0x8373,0x73D4,0x7373,0x7373,0x5263,0x6373,0x4463,0x4646,0x4642,0x4246,0x4246,0x4646,0x465F,0x4646,0x4642,0x465F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x5F46,0x4646,0x5F46,0x4646,0x4646
.word 0xD4D4,0xE4D4,0xE4D4,0xE4E4,0xE3E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xC3E4,0xE4D4,0xD4E4,0xD4E4,0xC3D4,0xD4D4,0xD4E4,0xE4C3,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4C3,0xD3D4,0xD4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xC4D4,0xC4BF,0xAF62,0xAFC4,0xAF63,0x5273,0x529F,0x429F,0x429F,0x428F,0x4241,0x6F7F,0x7F7F,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4346,0x9F52,0x9F52,0x4142,0x9F41,0xD4C4,0xC4D4,0x8F52,0xD462,0xD4D4,0xD4D4,0x9F52,0xD562,0xE4E4,0x52D4,0x529F,0x8F41,0x417F,0x8F8F,0x8F42,0x417F,0x527F,0xE462,0xE4F4,0xE4F4,0xD4E4,0xD4E4,0xD462,0xD4D4,0xE4E4,0xE4E4,0xD442,0xD47F,0xE4E4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0x8F51,0x7F41,0x7F8F,0xD441,0xD4D4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62BF,0x46AF,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4F42,0x4646,0x465F,0xB483,0x4273,0x4646,0x4446,0x9383,0x5283,0x6263,0x7352,0x7383,0x9393,0x7373,0x8383,0x6383,0x4246,0x4646,0x4642,0x4646,0x4642,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F,0x4646,0x4246,0x4646,0x465F,0x5F44,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x465F,0x465F
.word 0xE4D4,0xE4E4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E3,0xE4E4,0xD4E4,0xE4D3,0xD4D4,0xE4D4,0xC3D4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xE4E4,0xF4D4,0xD4E4,0xD4F4,0xD4E4,0xE4D3,0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xC3D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD483,0x62D5,0xAF62,0x6273,0x52AF,0x52AF,0x9F52,0x8F42,0x8F52,0x7F7F,0x7F7F,0x6F6F,0x4646,0x4646,0x465F,0x465F,0x4246,0x4142,0x9F52,0x4142,0x428F,0x428F,0x628F,0xD4AF,0x62D4,0xD4D5,0x51D4,0xD4AF,0xD4D4,0xD4D4,0x8F51,0x8F41,0x8F42,0x427F,0x7F7F,0x7F42,0x8F41,0x5242,0xD4AF,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xC4BF,0x7FD4,0x41D4,0xD4E4,0xE441,0xD441,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0x427F,0x428F,0x4241,0x417F,0xE452,0xF4E4,0xE4F4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC3D4,0xE4D4,0x72D4,0xD562,0x4662,0x4246,0x5F46,0x5F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0xB363,0x8383,0x8383,0x8373,0xB483,0x7383,0x7373,0x8383,0x8363,0x7363,0x8373,0x83B4,0x63B4,0x4646,0x465F,0x4646,0x4642,0x465F,0x465F,0x465F,0x465F,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4642,0x5F46,0x4646,0x4646,0x4646
.word 0xD4E4,0xD4E4,0xE4C3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xD4D4,0xE4D3,0xF4E4,0xD4E4,0xE4D3,0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xE4E3,0xE4F4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xD4C3,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0x62D4,0xC4BF,0xC4AF,0x73AF,0x63AF,0x9F52,0x9F52,0x4152,0x428F,0x418F,0x427F,0x7F6F,0x5F44,0x5F46,0x4646,0x5F46,0x4646,0x5241,0x429F,0x528F,0x418F,0x8F42,0x417F,0x6252,0xD4D4,0x83AF,0xC4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x8F52,0x527F,0x51D4,0x7F8F,0x7F7F,0x7F42,0x9F52,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4E4,0x62D4,0xD4D4,0xD4D4,0xE472,0x7FE4,0x62D4,0x7F8F,0x7F41,0x7F41,0xD441,0xE4D4,0x8F83,0x8F41,0x8F42,0x8F7F,0x7F42,0x9F62,0xE4D4,0xE4E4,0xE4D3,0xD3D4,0xD4E4,0xC3E4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD5D4,0x52AF,0x46AF,0x465F,0x4646,0x4646,0x4246,0x4646,0x4646,0x5F46,0x4646,0x7246,0x83B4,0xB383,0x92D4,0x93D4,0x73B4,0x5273,0x7363,0x8383,0x7373,0x7352,0x9373,0x6273,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4246,0x4F46,0x4646,0x4642,0x465F,0x4F46,0x4646,0x465F,0x4F46,0x4646,0x4246,0x4646,0x4642
.word 0xE4E4,0xE4D4,0xD4E4,0xE4E4,0xE4D4,0xE4D3,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xC3D4,0xD4E4,0xE4D3,0xE4E4,0xF4E4,0xF4E4,0xE3E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xD4E4,0xE4D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4C4,0x6262,0x62D5,0x62AF,0x73AF,0x529F,0x529F,0x429F,0x4241,0x7F7F,0x6F41,0x4646,0x4646,0x5F46,0x4646,0x4146,0x9F52,0x9F42,0x9F52,0x9F52,0x8F42,0x8F42,0x8F8F,0x629F,0xD4D4,0x6262,0x419F,0x6241,0xD4BF,0x8FD4,0xD452,0x8F62,0xD47F,0xD4D4,0x4152,0x9F52,0xD4D4,0xE4E4,0xE4E4,0xD4C3,0xC3D4,0xD4D4,0xD4D4,0xE4D4,0x7FE4,0x528F,0x5262,0x62AF,0x8F52,0x5242,0xC4D4,0xE48F,0x5251,0x7FD4,0x6242,0x42AF,0x527F,0xAF62,0x5242,0x8F62,0xE4D4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xE4E4,0xD4D4,0xBFD4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0x62D4,0x62D4,0x4652,0x4646,0x465F,0x465F,0x465F,0x5F46,0x4646,0x4642,0x4446,0x4246,0x93C4,0x83B4,0xD4B4,0xD493,0x8373,0xD483,0x7373,0x6383,0x8373,0x7373,0x8383,0x7373,0x4246,0x4646,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4646,0x4446,0x4246,0x4F46,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x4646,0x4246,0x4F46,0x5F46,0x4646
.word 0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xE4E4,0xD4E4,0xD4C3,0xD4C3,0xE4D4,0xC3D4,0xE4D4,0xD4D4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xF4E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xC4BF,0xAFD4,0xD562,0xAF52,0xAF63,0x9F73,0x5252,0x429F,0x8F8F,0x6F6F,0x417F,0x5F46,0x4246,0x4646,0x4246,0x4142,0x8F52,0x5252,0x529F,0x9F52,0x9F42,0x4241,0x4241,0x428F,0xC4AF,0xD4D5,0xD562,0xD5D4,0x62D4,0x627F,0x62D5,0x418F,0xE4C4,0x52D4,0x528F,0xD4AF,0xE4D4,0xD4E4,0xE4E4,0x52D4,0x429F,0x417F,0xD462,0x7FE4,0x4241,0x8F41,0xD562,0x8F52,0x9F52,0x8F41,0xD4D4,0x8F41,0x8F52,0xD47F,0x417F,0x528F,0xBF62,0x8F52,0x628F,0x52AF,0xD452,0xE4E4,0xD4E4,0xE4D3,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xAF62,0x4652,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4646,0x6346,0xC493,0x9393,0xB3D4,0x5263,0x8373,0x7373,0x6393,0x5273,0x5273,0x7373,0x7373,0x4642,0x4646,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x4F46,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4246,0x4246,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4D3,0xE4E4,0xE4E4,0xE4E3,0xF4E4,0xE3E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xD4D4,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4C3,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x72D4,0x62D5,0x62BF,0x73AF,0x52AF,0x5252,0x529F,0x429F,0x428F,0x7F7F,0x7F6F,0x4646,0x465F,0x465F,0x4246,0x9F52,0x9F52,0x9F52,0x9F52,0x8F52,0x4152,0x9F52,0x8F52,0x8F41,0x8F42,0xD462,0xBFD4,0x72C4,0x528F,0xD4D4,0x8F9F,0x8F52,0x4241,0xD4AF,0xD4E4,0xE4D4,0xD4E4,0xC3E4,0xE4D4,0xE4D4,0xD4D4,0xD452,0x418F,0x6252,0x8FD5,0xD562,0x8F9F,0x417F,0x528F,0xAF62,0x417F,0xD452,0x8FD4,0x4142,0x428F,0x8F7F,0x8F42,0x6242,0x9F62,0x9F52,0xAF62,0xE442,0xE4E4,0xD4D4,0xD4E4,0xC3D4,0xD4E4,0x62D4,0xD4D4,0xD4D4,0xC3D4,0xD4D4,0xD572,0xD562,0x469F,0x4642,0x4246,0x4646,0x4642,0x465F,0x4F42,0x5F46,0x4646,0x4646,0x7372,0x4446,0x8373,0xD4C4,0x5273,0x7373,0x7352,0x7373,0x8373,0x8373,0x83C4,0x73D4,0x4283,0x4246,0x5F46,0x4646,0x464F,0x465F,0x465F,0x4642,0x4F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x465F,0x4F46,0x4646,0x464F,0x4646,0x5F46,0x5F46,0x4646,0x465F,0x4646
.word 0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4D4,0xD4E4,0xE4D4,0xD4C3,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xD4D3,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0xD562,0x6262,0xAF62,0x9F62,0x9F52,0x8F52,0x7F41,0x6F7F,0x417F,0x4246,0x4646,0x4646,0x5241,0x529F,0x9F52,0x5252,0x529F,0x529F,0x528F,0x429F,0x428F,0x8F41,0x8F42,0x417F,0x62C4,0x429F,0x629F,0x52AF,0x8F41,0x5242,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D4,0xD4D4,0xE47F,0x7FE4,0x9F52,0x528F,0x417F,0x528F,0xD4D4,0xD4D4,0xD4E4,0x8F52,0x8F42,0x9F42,0x6242,0x528F,0xAFC4,0x8F52,0xD562,0x8F7F,0x428F,0x428F,0x428F,0x6262,0xE4D4,0xD4E4,0xE4E4,0xD4E4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0x62D4,0x62BF,0x4652,0x4646,0x465F,0x4F46,0x4646,0x4646,0x4646,0x4646,0x4646,0x4642,0x8383,0x4673,0x8363,0x9383,0x7363,0x5283,0x8383,0x8373,0x6383,0x7383,0x7373,0x8383,0x7373,0x4646,0x4646,0x4642,0x4646,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4246,0x4646,0x464F,0x4646,0x4646,0x5F46,0x4646,0x464F,0x4642,0x4646,0x4646,0x4246,0x5F46
.word 0xE4E4,0xE4C3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xD3E4,0xE4E4,0xE4E4,0xD4E4,0xC3E4,0xE4D4,0xE4D4,0xC3D4,0xE4D4,0xE4D4,0xD4D4,0xE4E4,0xE3E4,0xF4E4,0xF4E4,0xF4E4,0xE4E3,0xE3F4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4C3,0xC3D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xBFC4,0xAF62,0xAF73,0xAF63,0x5273,0x429F,0x4241,0x427F,0x7F6F,0x5F46,0x5F46,0x4246,0x9F52,0xAF52,0x5262,0x62AF,0xAF62,0x6263,0x52AF,0x9F52,0x9F52,0x4152,0x428F,0x8F41,0x8F8F,0x527F,0x9F62,0x9F52,0x527F,0x62AF,0xD4D5,0xD4D4,0xD4D4,0xD4C3,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xD483,0x417F,0x5252,0x7FD5,0xD4C4,0xD4D4,0x83D4,0xD4D4,0x417F,0xE4D4,0xD4E4,0x52D4,0x8F8F,0x8F42,0x528F,0xD4D4,0x8F62,0xD452,0x8FC4,0x5241,0x62AF,0x9F8F,0xC462,0xE4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xAF62,0xAF73,0x4652,0x4646,0x4646,0x4646,0x4646,0x4646,0x5F46,0x5F46,0x4646,0x464F,0x9344,0x8383,0x83C4,0x93B4,0x7373,0x7352,0x7363,0x7352,0x7383,0x7373,0x5252,0x7373,0x7383,0x4273,0x4646,0x4646,0x4646,0x4246,0x5F46,0x4646,0x4646,0x465F,0x465F,0x465F,0x5F46,0x4646,0x4F44,0x5F4F,0x4646,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646
.word 0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xD3E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xF4E4,0xE4E4,0xE4D3,0xE4E4,0xE4D4,0xD4E4,0xE4C3,0xE4D4,0xD4D4,0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xD4D3,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xC4D4,0x62D5,0x73AF,0x52AF,0x52AF,0x9F52,0x8F42,0x6F7F,0x7F41,0x4646,0x4646,0x5146,0x6252,0x5262,0x73AF,0x52AF,0x63AF,0x52AF,0xAF62,0xAF73,0x5252,0x529F,0x9F52,0x8F42,0x4252,0x9F41,0x5252,0x429F,0x4241,0x529F,0x629F,0x62AF,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD483,0x41AF,0x8F8F,0x8F8F,0xD441,0x8FD4,0x5241,0x9F9F,0x8F52,0xD562,0x428F,0xE462,0xD4E4,0x6262,0x42AF,0x5241,0xD4AF,0x5241,0xD49F,0x527F,0x8F62,0x8F73,0x6252,0x418F,0xD4C4,0xE4D4,0xD4D4,0xD4D4,0xD462,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x469F,0x465F,0x4246,0x5F46,0x4246,0x464F,0x4646,0x4646,0x4642,0x4646,0x7346,0xC483,0x93B2,0xD4D4,0x5283,0x7352,0x73D4,0x8373,0x9363,0x7373,0x9383,0x8373,0x7383,0x7383,0x4642,0x465F,0x5F46,0x4646,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x4446,0x5F46,0x5F46,0x4246,0x4646,0x465F,0x465F
.word 0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD4E4,0xD4E4,0xD3D4,0xE4E4,0xE4E4,0xE4E4,0xD4F4,0xF4F4,0xF4F4,0xE4E4,0xE4E4,0xE4E3,0xE4E3,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0x62D4,0xBFD4,0xD583,0xAF62,0x6273,0x9F62,0x9F52,0x8F52,0x7F41,0x6F7F,0x5F46,0x4646,0x5241,0x62AF,0x73AF,0x73AF,0x73AF,0xAF62,0xAF73,0xAF73,0x6252,0x9FD5,0x9F52,0x9F42,0x4142,0x8F9F,0x7F42,0x528F,0x8F8F,0x428F,0x418F,0x428F,0x9F41,0x9F52,0x9F52,0x4152,0x4141,0x5241,0x62BF,0x8F52,0x4242,0x4152,0x7FD4,0x7F41,0xAF42,0x6273,0x628F,0x419F,0xD4D4,0xD4D4,0x8F41,0x528F,0xD462,0xD4D4,0x8F41,0xD552,0x7F72,0x9F41,0x52D5,0xD49F,0x628F,0x528F,0x62AF,0xD4E4,0xD4D4,0x62D4,0xD4D4,0xD4D4,0xD472,0xC3D4,0xD4D4,0xD562,0xC4AF,0x4641,0x4246,0x4F46,0x4646,0x4646,0x4246,0x5F46,0x4646,0x464F,0x4246,0x4646,0xC452,0xB493,0xD483,0x7383,0x7352,0x5273,0x8383,0x8373,0x6263,0x7363,0xC473,0xB493,0x7383,0x7373,0x4654,0x4646,0x4642,0x4646,0x465F,0x4646,0x4642,0x4246,0x4F46,0x464F,0x4642,0x465F,0x4F46,0x4644,0x465F,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646
.word 0xD4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4D3,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xE4D4,0xD3D4,0xD3D4,0xE4E4,0xE4D3,0xE4F4,0xE4E4,0xE4F4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D3,0xD4E4,0xE4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD562,0x73AF,0x73AF,0x529F,0x429F,0x7F7F,0x6F6F,0x4644,0x435F,0x6252,0x73AF,0x73AF,0x73AF,0xAF62,0xD573,0xAF62,0xD573,0x629F,0x52AF,0xAF62,0x5252,0x429F,0x428F,0x428F,0x7F7F,0x427F,0x418F,0x427F,0x8F8F,0x8F42,0x9F42,0x4152,0x629F,0x62D5,0x73AF,0x429F,0x429F,0x417F,0x429F,0x418F,0x6252,0x429F,0x8F8F,0x8F42,0x8F42,0x837F,0xD4D4,0x7FD4,0x8F52,0x628F,0xD4E5,0x7FD4,0x418F,0x527F,0x52D4,0x419F,0xD552,0x8F62,0xAF42,0x8F73,0x6262,0xD4E4,0xBFD4,0xD4C4,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xC4BF,0xAF62,0x4652,0x5F46,0x4646,0x4646,0x465F,0x4646,0x4646,0x4646,0x5F46,0x4646,0x465F,0x4246,0xC473,0xD493,0xB493,0x7352,0x7373,0x7363,0x8373,0x7373,0x7393,0x7383,0xC483,0x9373,0x8373,0x5273,0x4644,0x5F46,0x4646,0x4646,0x4642,0x464F,0x4646,0x465F,0x4F4F,0x464F,0x4646,0x4F5F,0x464F,0x4646,0x4642,0x4646,0x464F,0x4642,0x4646,0x4644
.word 0xE4E4,0xD4E4,0xE4D3,0xE4E4,0xE4E3,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xD3E4,0xE4E4,0xD3D4,0xE4D4,0xD4C3,0xE4D4,0xC3D4,0xE4D4,0xE4D4,0xE4D4,0xD4C3,0xE4E4,0xE4E4,0xE4F4,0xF4E4,0xF4F4,0xF4E4,0xF4F4,0xE4E4,0xF4E4,0xE3E4,0xE4E4,0xE4E4,0xD3D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xC3D4,0xD4E4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xAFD4,0x62D4,0xAF62,0xAF73,0x5252,0x528F,0x7F7F,0x7F7F,0x4646,0x6242,0x73AF,0xD562,0xAF62,0xD5C4,0xC4AF,0xC4AF,0xC4AF,0xD462,0xD5C4,0xD562,0x6262,0x62AF,0x9F62,0x5262,0x418F,0x8F52,0x7F42,0x4241,0x8F8F,0x4142,0x428F,0x428F,0x428F,0x428F,0x418F,0x8F42,0x8F42,0x7F41,0x8F41,0x7F42,0x8F41,0x528F,0x42AF,0x417F,0x8F7F,0x427F,0x8F8F,0x627F,0xD4D4,0x9F52,0x8F42,0x628F,0x62D4,0x8F42,0x7F8F,0x418F,0x9F42,0x418F,0x528F,0xD48F,0xC4D4,0x73BF,0x52AF,0xC4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x72D4,0xAFC4,0x6262,0x4652,0x4646,0x5F46,0x4646,0x4646,0x465F,0x4246,0x464F,0x4646,0x4246,0x4364,0x4646,0x4646,0x7342,0x8383,0x5252,0x5273,0x5273,0x7373,0x7352,0x6352,0x6373,0x5273,0xC483,0x9373,0x7383,0x7373,0x4663,0x4246,0x4646,0x4646,0x4646,0x4646,0x4646,0x4646,0x4F4F,0x4646,0x464F,0x424F,0x4F5F,0x4646,0x465F,0x4F4F,0x4646,0x465F,0x465F
.word 0xE4D4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD3E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xE4D4,0xD4E4,0xD4E4,0xE4C3,0xE4D4,0xD4E4,0xE4E3,0xE4E4,0xF4E4,0xE4E4,0xE4F4,0xE3E4,0xF4E4,0xE3E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xD4E4,0xD4D4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD562,0xC4AF,0x73AF,0x529F,0x429F,0x6F7F,0x6F42,0x4642,0x6262,0xAFD5,0xBF62,0xD5C4,0xD462,0xD583,0xD462,0xD5D4,0xAFD4,0xBF83,0xAF62,0x62D5,0xD5D5,0xAF62,0xAF73,0x9F52,0x8F52,0x4241,0x8F7F,0x7F42,0x8F41,0x8F42,0x8F41,0x8F41,0x8F42,0x428F,0x8F8F,0x7F41,0x8F42,0x7F42,0x418F,0x8F42,0x8F42,0x417F,0x427F,0x427F,0x417F,0x8F42,0x8F42,0xD4D4,0x52D4,0x7FD5,0x8F41,0xE452,0x41D4,0x7F42,0x7F42,0x428F,0x427F,0x428F,0xD441,0xD4D4,0xD562,0xAF62,0xBF62,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x73AF,0x73AF,0x469F,0x4646,0x4646,0x5F46,0x4246,0x4646,0x464F,0x4646,0x4642,0x4646,0x9373,0x4643,0x4642,0x7343,0xC483,0x5273,0x5273,0x7373,0x8373,0x8363,0x52C4,0x7383,0x8383,0x9383,0x83C4,0x83B4,0x8373,0x6383,0x4646,0x4646,0x4646,0x4646,0x4644,0x5F46,0x4246,0x4F46,0x464F,0x4642,0x464F,0x5F46,0x4F4F,0x4646,0x4F5F,0x4646,0x4646,0x4646
.word 0xD4E4,0xE4D3,0xE4E4,0xE3E4,0xE4E4,0xE4D3,0xE4F4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E3,0xE4E4,0xE4F4,0xE4F4,0xE4F4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xC3E4,0xE4D4,0xE4D4,0xE4D4,0xD4E4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0x6262,0x52D5,0x52AF,0x8F52,0x6F41,0x6F7F,0x5146,0xBF62,0x6283,0x62D5,0x61D5,0xAFD5,0x62D4,0xBFD4,0xBF83,0xD4C4,0x62D4,0xD4D4,0xD4D4,0xBF62,0x6273,0x73AF,0x5262,0x529F,0x418F,0x428F,0x417F,0x427F,0x417F,0x427F,0x417F,0x417F,0x7F7F,0x7F42,0x7F42,0x8F41,0x7F41,0x7F42,0x7F8F,0x4241,0x7F7F,0x7F7F,0x7F41,0x7F41,0x417F,0x418F,0xD4E4,0xD4E4,0xD47F,0x427F,0x418F,0xD4D4,0x417F,0x428F,0x417F,0x417F,0x8F7F,0x9F42,0xD4D4,0xD4D4,0xD562,0x6262,0xD462,0xD4D4,0xD472,0xD4D4,0x62AF,0xAFC4,0xAF73,0x4645,0x465F,0x465F,0x4646,0x465F,0x4646,0x4646,0x5F46,0x4646,0x465F,0x7246,0x7383,0x8373,0x8382,0x93B4,0x7383,0x7352,0x5273,0x8373,0x7352,0x7383,0x5273,0x6373,0x9363,0x6383,0x8383,0x8373,0x8373,0x5283,0x5F46,0x5F46,0x4246,0x464F,0x4646,0x4F46,0x464F,0x464F,0x4F4F,0x464F,0x464F,0x5F46,0x464F,0x4F46,0x4F4F,0x5F46,0x4646
.word 0xE4E4,0xE4E4,0xD3D4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4D4,0xD4C3,0xD4E4,0xD4D4,0xE4D4,0xE4D4,0xE4D4,0xE4D3,0xE4E4,0xE3E4,0xF4E4,0xF4E4,0xE4F4,0xF4F4,0xE4E4,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4D4,0xE4E4,0xD4D4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xC4BF,0xC4BF,0x62AF,0xAF62,0x5252,0x6F8F,0x6F41,0x6241,0x62D5,0xAFD5,0xD4C4,0x62D4,0x62D4,0xBFD4,0xD4C4,0x62C4,0x72D4,0xD4D5,0xD4D4,0xD462,0x62D5,0xD5D5,0xAF62,0xAF62,0x9F73,0x5252,0x418F,0x7F8F,0x7F7F,0x7F7F,0x417F,0x427F,0x427F,0x418F,0x417F,0x417F,0x427F,0x417F,0x428F,0x7F41,0x9F42,0x7F41,0x7F42,0x7F41,0x8F42,0x7F42,0x7F42,0xE452,0xD4D4,0xE451,0x8F41,0x8F42,0xD4D4,0x527F,0x7F41,0x7F41,0x7F42,0x4142,0xC49F,0x8F52,0xD4D4,0xBFD4,0xAFC4,0x62D5,0x52AF,0xD4AF,0x62D4,0xD562,0x6262,0x73AF,0x4642,0x4646,0x4246,0x464F,0x4646,0x465F,0x4646,0x4646,0x4646,0x4646,0x445F,0x8373,0xC483,0xC493,0xC483,0xB493,0x7352,0x7352,0x8373,0x6373,0x8373,0x7383,0x8383,0xB483,0x8393,0x83B4,0x8363,0x83B4,0x8373,0x5473,0x4646,0x4646,0x4646,0x4642,0x464F,0x464F,0x424F,0x464F,0x5F4F,0x4646,0x5F4F,0x465F,0x4F42,0x4F46,0x465F,0x4246
.word 0xE4E4,0xE4E4,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4E4,0xD4E4,0xE4C3,0xE4D4,0xE4C3,0xE4E4,0xE4D4,0xC3D4,0xD4E4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE3D4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4D3,0xE4E3,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D3,0xE4D3,0xE4E4,0xE4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xD4D4,0xD4C4,0xAFC4,0x52C4,0x52AF,0x4241,0x6F6F,0xAFC4,0xBFC4,0xD483,0xD4AF,0xD4AF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xBFD4,0xD4C4,0xBFD4,0xD4D4,0xD462,0xD4D5,0x62D5,0xC4AF,0x62AF,0x5252,0x5242,0x4241,0x4241,0x7F7F,0x7F41,0x7F7F,0x7F7F,0x7F42,0x427F,0x418F,0x428F,0x8F7F,0x7F42,0x7F41,0x7F42,0x7F41,0x7F42,0x417F,0x417F,0x418F,0xE47F,0xE4D4,0xD4D4,0x8F62,0x7F41,0x83E4,0x7FD4,0x8F52,0x7F42,0x7F41,0x7F8F,0xAF42,0x7F62,0xD4D4,0xD4D4,0xD4D4,0xD562,0x6262,0x51D5,0x629F,0x52AF,0xC4BF,0x9F62,0x4646,0x4642,0x4646,0x4646,0x4646,0x4246,0x464F,0x4642,0x465F,0x4646,0x4646,0x4446,0x9373,0x83C4,0xB483,0xC3B3,0x7383,0x5252,0x7352,0x7373,0x7352,0x6373,0x6373,0x8373,0x7373,0x8383,0x8363,0x7393,0x7383,0x8383,0x4463,0x4646,0x4646,0x4646,0x5F4F,0x4646,0x464F,0x464F,0x464F,0x4F46,0x4F46,0x4F4F,0x4F46,0x5F46,0x464F,0x5F46
.word 0xD4E4,0xE4E4,0xD4E4,0xE4E4,0xE3E4,0xE4E4,0xE4F4,0xE4E4,0xE4D3,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xE4E4,0xD4D4,0xD4E4,0xD4D4,0xD4E4,0xE4D4,0xD4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E3,0xE4E4,0xE4E3,0xE3E4,0xE4E4,0xE4E3,0xE4F4,0xF4E4,0xE4E4,0xE3D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4E4,0xD4E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xAFD4,0xD4C4,0x62AF,0x9F62,0x8F42,0x527F,0x62D4,0x62D4,0x62D5,0xD462,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xAF72,0x62C4,0x62D5,0xAF62,0x9F62,0x529F,0x8F8F,0x7F7F,0x4241,0x428F,0x4141,0x7F42,0x7F41,0x417F,0x427F,0x417F,0x427F,0x418F,0x7F62,0x7F7F,0x7F42,0x7F41,0x7F42,0x7F42,0x8F42,0xD442,0xD4D4,0xD4D4,0x42E4,0x428F,0xC4E4,0x52AF,0x528F,0x418F,0x427F,0x7F41,0x6241,0x41E5,0xD47F,0xD4D4,0xD4D4,0xBFD4,0xAFD4,0x6262,0x63AF,0x73AF,0x529F,0x52AF,0x5262,0x4642,0x4646,0x4246,0x4646,0x4646,0x4646,0x464F,0x4646,0x4642,0x4646,0x4642,0x6346,0xC493,0xC493,0xC493,0xC493,0x7383,0x5252,0x7352,0x8373,0x8373,0x62D4,0xC483,0x8373,0x93B4,0x7373,0x6383,0xB493,0x8373,0x6383,0x5F46,0x4646,0x464F,0x4F4F,0x5F46,0x4F4F,0x465F,0x464F,0x4F46,0x464F,0x4F5F,0x5F46,0x4646,0x465F,0x4646
.word 0xE4E4,0xD4E4,0xE4D3,0xD3E4,0xF4E4,0xE4E4,0xE4E3,0xE3E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xD4D4,0xD4D4,0xD4C3,0xE4D4,0xC3D4,0xE4E4,0xC3D4,0xE4D4,0xD4E4,0xD4D3,0xE4E4,0xD4E3,0xE4F4,0xE3E4,0xF4E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xE4E4,0xE4E3,0xF4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4C3,0xD4E4,0xD4D4,0xD4E4,0xD4C3,0xD4D4,0xD4D4,0xD572,0xD562,0x529F,0x429F,0x628F,0xAFD5,0x62D4,0xD4BF,0xD4D5,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD462,0xD4D4,0xD4D4,0xD4D4,0xD472,0xD4D4,0xD4D4,0xC4D4,0xC4BF,0xC4BF,0xC4AF,0x73AF,0x9F52,0x4252,0x4241,0x7F8F,0x7F41,0x428F,0x429F,0x5252,0x6252,0xAF52,0x6273,0x52AF,0xAF62,0x427F,0x417F,0x7F7F,0x417F,0x417F,0x417F,0x428F,0xD462,0xD4D4,0xD462,0x7FD4,0x7F41,0xC4E4,0xD562,0x528F,0x428F,0x627F,0xD47F,0xD4E4,0xD4E4,0xD441,0xD4D4,0xD4D4,0xD462,0xD4C4,0x62D5,0x62D5,0xAF52,0xAF73,0xAF73,0xAF52,0x5252,0x5251,0x4642,0x4F46,0x5F46,0x4646,0x4646,0x5F46,0x4646,0x465F,0x5F46,0x4646,0x4646,0x4646,0x4246,0x83B4,0xC483,0x6383,0x5252,0x7363,0x7352,0x7352,0x7363,0x8352,0x73C4,0xD473,0x7393,0x9373,0xB483,0x8373,0x4373,0x4646,0x4646,0x4F4F,0x4F4F,0x4646,0x4646,0x5F46,0x4646,0x4646,0x4646,0x4646,0x4F46,0x4646,0x4646
.word 0xE4D4,0xE4C3,0xE4D4,0xE4E4,0xE4E4,0xE4E3,0xE4E4,0xE4F4,0xE4E4,0xD4E4,0xE4D3,0xE4D4,0xD4C3,0xD4E4,0xD4E4,0xC3D4,0xD4D4,0xD4D4,0xD4E4,0xD4E4,0xD4C3,0xE4D4,0xE4E3,0xF4E4,0xE4E4,0xE4E4,0xF4E4,0xF4E4,0xF4E4,0xF4F4,0xE3E4,0xE4E4,0xE4F4,0xE4E4,0xE4E4,0xD4E4,0xD4D3,0xD4E4,0xE4E4,0xE4D4,0xC3E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62AF,0x9F62,0x4152,0xBF62,0x62C4,0xD4D4,0xD462,0x62D4,0xD4D4,0xD4D4,0xD4D4,0xD4BF,0xD4D4,0x72D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0xD4D4,0xD562,0xD583,0xAF62,0x62C4,0x529F,0x418F,0x8F42,0x7F7F,0x8F42,0x8F41,0x9F41,0x9F52,0x9F52,0x9F52,0xD552,0x8F52,0x7F7F,0x7F7F,0x7F41,0x7F42,0x7F42,0x7F41,0x8F42,0xD462,0xD49F,0xD462,0x8FD4,0x7F42,0xD4E4,0x7FBF,0x8F52,0x7F41,0xD48F,0xD4E4,0xD4D4,0xE4E4,0xD462,0xD4D4,0xD4D4,0xD4D4,0x72D4,0xAFD4,0xAFC4,0x6262,0x529F,0xAF52,0x6273,0x52AF,0x52AF,0x5152,0x5252,0x4646,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x5F46,0x4246,0x4246,0x4646,0x7346,0x8383,0x8383,0x5273,0x5252,0x7352,0x7383,0x9383,0x7373,0x5283,0x7373,0x6383,0xD483,0x8383,0x73D4,0x8383,0x4446,0x4246,0x4646,0x4646,0x4646,0x4246,0x4646,0x4642,0x465F,0x4446,0x4646,0x4642,0x4642,0x465F
.word 0xE4E4,0xE4D4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xD4E4,0xE4E4,0xE4E4,0xD4E4,0xE4E4,0xE4D4,0xD4E4,0xE4D4,0xE4D4,0xE4E4,0xE4D4,0xE4D4,0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xF4E4,0xF4F4,0xE4F4,0xE4E3,0xE4F4,0xE4E4,0xE4E4,0xE3E4,0xE4E4,0xE3E4,0xD3E4,0xE4E4,0xE4E4,0xE4E4,0xE4D4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4E4,0xD4D4,0xD472,0xAFC4,0x62C4,0xC4AF,0x62D5,0xD4D4,0xC4BF,0xD4D5,0xD4BF,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0x83BF,0x62D5,0xC4AF,0x73AF,0x529F,0x429F,0x4241,0x417F,0x8F42,0x8F42,0x8F42,0x5252,0x529F,0x7F8F,0x7F8F,0x7F42,0x7F41,0x417F,0x417F,0x417F,0x427F,0x8F7F,0x8F62,0x8F62,0x7F8F,0x8F52,0xD441,0xC4E4,0x41D4,0x8F52,0x7F62,0xD4D4,0xD4D4,0xD4D4,0x6262,0xE4E5,0xE4D4,0xD4D4,0xD4D4,0xC4BF,0xD4D4,0xD562,0x73AF,0x52AF,0x739F,0x739F,0x73AF,0xAF52,0xAF62,0x5252,0x5162,0x4642,0x4646,0x4646,0x465F,0x465F,0x4642,0x4646,0x4646,0x4646,0x465F,0xC452,0xD483,0x93B4,0x93C3,0x73C4,0x5252,0x7352,0x8373,0x8363,0x7373,0xD483,0x8352,0x7373,0x7373,0x8383,0x8373,0x4273,0x4646,0x465F,0x4246,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x465F,0x4646,0x4646,0x4646
.word 0xE4E4,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xE4E4,0xE4D3,0xE4E4,0xE4E4,0xD4D4,0xD4E4,0xD4E4,0xE4D4,0xD4D4,0xD4D4,0xD4C3,0xD4E4,0xD4E4,0xE3D4,0xE3D4,0xE4E4,0xE3E4,0xE4F4,0xF4E4,0xE4F4,0xE4E4,0xE4F4,0xE4D3,0xE4E3,0xD4F4,0xE3E4,0xE4E4,0xE4E4,0xF4E4,0xE3E4,0xE4E4,0xE4D3,0xE4E3,0xE4E4,0xD3E4,0xE4E4,0xE4D4,0xD4D4,0xD4D4,0xD4D4,0x62D4,0x73BF,0xC4AF,0xD4AF,0xD562,0xD483,0xD4BF,0xD4C4,0x72D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xE4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD4D4,0xD562,0xD562,0xAF62,0xAF73,0x5252,0x528F,0x428F,0x7F8F,0x417F,0x8F7F,0x8F8F,0x7F42,0x7F41,0x7F41,0x417F,0x427F,0x427F,0x7F7F,0x7F42,0xAF62,0x6283,0xD4D5,0xC4C4,0x6262,0x42AF,0xE4D4,0x62D4,0x628F,0x42AF,0x62E4,0x62D4,0x7F9F,0xE4D4,0x628F,0xD47F,0xD4D4,0xD4D4,0xD4E4,0xD4D4,0xD4D4,0x62D4,0x62D5,0xAF62,0x9F52,0xAF52,0xAF63,0xAF62,0x6263,0xD5AF,0x629F,0x4252,0x4646,0x465F,0x4646,0x4646,0x464F,0x4642,0x4646,0x465F,0x9364,0x83C4,0xB493,0xB493,0xD483,0x83B3,0x5273,0x5263,0x7363,0xC483,0x8373,0x8383,0x8373,0x73C4,0x8373,0x7363,0x7383,0x7383,0x4646,0x4646,0x5F46,0x4646,0x5F46,0x5F46,0x4646,0x4646,0x4646,0x4646,0x465F,0x4646,0x465F
*/




;--------End of All Code Sections ---------------------------------------------

.end                               ;End of program code in this file
        